// Copyright (C) 2024 The Qt Company Ltd.
// Copyright © 2004-2023 Unicode, Inc.
// SPDX-License-Identifier: Unicode-3.0

#ifndef QTIMEZONELOCALE_DATA_P_H
#define QTIMEZONELOCALE_DATA_P_H

//
//  W A R N I N G
//  -------------
//
// This file is not a normal header file.
// It defines data tables for internal use by QTimeZone's L10n code.
// It may change incompatibly between versions, or even be removed.
//
// We mean it.
//

// Only qtimezonelocale.cpp should #include this (after other things it needs),
// and even that only when feature icu is disabled.

QT_REQUIRE_CONFIG(timezone_locale);
#if QT_CONFIG(icu)
#  error "This file should only be needed (or seen) when ICU is not in use"
#endif

QT_BEGIN_NAMESPACE

#ifdef QT_CLDR_ZONE_DEBUG
#  define LOCALE_TAGS(lang, script, land) lang, script, land,
#else
#  define LOCALE_TAGS(lang, script, land)
#endif

namespace QtTimeZoneLocale {

/*
    Locale-specific data for timezone naming.
    https://www.unicode.org/reports/tr35/tr35-68/tr35-dates.html#Time_Zone_Names

    Available data, per locale, from LocaleScanner.timeZoneNames():

        formats: {
            'hour': ('+HH:mm', '-HH:mm'),
            'GMT': 'GMT%0',
            'region': ("%0 Time", std, dst),
            'fallback': "%1 (%0)" }
        zones: { ianaid: {
                'exemplarCity': name,
                'long': (gen, std, dst), 'short': (gen, std, dst) } }
        metazones: { meta: {'long': (gen, std, dst), 'short': (gen, std, dst) } }

    Mapped to C++ data-structures below (defined in QTZL_p.h) and used by
    QTZL.cpp, in conjunction with QTZP_data_p.h's locale-independent data
    conditioned on timezone_locale.
*/

// GENERATED PART STARTS HERE

/*
    This part of the file was generated on 2024-10-04 from the
    Common Locale Data Repository v45

    http://www.unicode.org/cldr/

    Do not edit this section: instead regenerate it using
    cldr2qlocalexml.py and qlocalexml2cpp.py on updated (or
    edited) CLDR data; see qtbase/util/locale_database/.
*/

// Sorted by locale index, then iana name
static inline constexpr LocaleZoneExemplar localeZoneExemplarTable[] = {
    // locInd, ianaInd, xcty{ind, sz}
    {    0,   14,       0,  6, }, // Africa/Asmara C/AnyScript/AnyTerritory
    {    0, 4337,       6,  8, }, // Africa/Sao_Tome
    {    0, 4154,      14,  8, }, // America/Asuncion
    {    0,  237,      22,  8, }, // America/Atikokan
    {    0, 7586,      30, 17, }, // America/Bahia_Banderas
    {    0, 3770,      47,  6, }, // America/Cancun
    {    0, 7737,      53, 13, }, // America/Ciudad_Juarez
    {    0, 5640,      66,  7, }, // America/Curacao
    {    0,  481,      73, 13, }, // America/Indiana/Knox
    {    0, 7924,      86, 16, }, // America/Indiana/Marengo
    {    0, 7948,     102, 19, }, // America/Indiana/Petersburg
    {    0, 7975,     121, 18, }, // America/Indiana/Tell_City
    {    0, 8001,     139, 14, }, // America/Indiana/Vevay
    {    0, 8023,     153, 18, }, // America/Indiana/Vincennes
    {    0, 8049,     171, 16, }, // America/Indiana/Winamac
    {    0, 8088,     187, 20, }, // America/Kentucky/Monticello
    {    0, 5849,     207, 22, }, // America/Lower_Princes
    {    0, 8183,     229,  6, }, // America/Merida
    {    0, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {    0, 8292,     255, 20, }, // America/North_Dakota/Center
    {    0, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {    0,  393,     298,  4, }, // America/Nuuk
    {    0, 6862,     302, 16, }, // America/Scoresbysund
    {    0, 8471,     318, 14, }, // America/St_Barthelemy
    {    0, 2045,     332, 10, }, // America/St_Johns
    {    0, 5780,     342,  9, }, // America/St_Kitts
    {    0, 5797,     351,  9, }, // America/St_Lucia
    {    0,  855,     360, 10, }, // America/St_Thomas
    {    0, 5830,     370, 11, }, // America/St_Vincent
    {    0, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {    0, 1226,     397, 16, }, // Asia/Ho_Chi_Minh
    {    0, 1152,     413,  9, }, // Asia/Kathmandu
    {    0,  992,     422,  7, }, // Asia/Kolkata
    {    0, 1178,     429,  5, }, // Asia/Macau
    {    0, 8671,     434,  8, }, // Asia/Qostanay
    {    0, 1202,     442,  6, }, // Asia/Yangon
    {    0, 1379,     448,  5, }, // Atlantic/Faroe
    {    0, 6784,     453, 10, }, // Atlantic/St_Helena
    {    0, 2419,     463,  4, }, // Europe/Kyiv
    {    0, 7174,     467,  7, }, // Indian/Reunion
    {    0, 3022,     474,  5, }, // Pacific/Chuuk
    {    0, 2961,     479,  7, }, // Pacific/Pohnpei
    {    1, 6751,     486,  6, }, // Africa/Abidjan Abkhazian/Cyrillic/Georgia
    {    1, 6723,     492,  5, }, // Africa/Accra
    {    1, 4961,     497, 11, }, // Africa/Addis_Ababa
    {    1, 7383,     508,  5, }, // Africa/Algiers
    {    1,   14,     513,  6, }, // Africa/Asmara
    {    1,   44,     519,  6, }, // Africa/Bamako
    {    1, 5159,     525,  5, }, // Africa/Bangui
    {    1, 6709,     530,  6, }, // Africa/Banjul
    {    1, 7398,     536,  5, }, // Africa/Bissau
    {    1, 4859,     541,  8, }, // Africa/Blantyre
    {    1, 5189,     549,  9, }, // Africa/Brazzaville
    {    1, 4824,     558,  9, }, // Africa/Bujumbura
    {    1, 2242,     567,  4, }, // Africa/Cairo
    {    1, 4019,     571, 10, }, // Africa/Casablanca
    {    1, 7412,     581,  5, }, // Africa/Ceuta
    {    1, 6736,     586,  7, }, // Africa/Conakry
    {    1, 6803,     593,  5, }, // Africa/Dakar
    {    1, 5032,     598, 12, }, // Africa/Dar_es_Salaam
    {    1, 4945,     610,  6, }, // Africa/Djibouti
    {    1, 5145,     616,  5, }, // Africa/Douala
    {    1, 5068,     621,  7, }, // Africa/El_Aaiun
    {    1, 6816,     628,  7, }, // Africa/Freetown
    {    1, 4808,     635,  8, }, // Africa/Gaborone
    {    1, 4917,     643,  6, }, // Africa/Harare
    {    1, 4381,     649, 12, }, // Africa/Johannesburg
    {    1, 4401,     661,  4, }, // Africa/Juba
    {    1, 5053,     665,  7, }, // Africa/Kampala
    {    1, 4426,     672,  6, }, // Africa/Khartoum
    {    1, 4875,     678,  6, }, // Africa/Kigali
    {    1, 5208,     684,  7, }, // Africa/Kinshasa
    {    1, 4646,     691,  5, }, // Africa/Lagos
    {    1, 5238,     696,  8, }, // Africa/Libreville
    {    1, 6832,     704,  4, }, // Africa/Lome
    {    1, 5113,     708,  6, }, // Africa/Luanda
    {    1, 4841,     714,  9, }, // Africa/Lubumbashi
    {    1, 4903,     723,  6, }, // Africa/Lusaka
    {    1, 5224,     729,  6, }, // Africa/Malabo
    {    1, 4889,     735,  6, }, // Africa/Maputo
    {    1, 5099,     741,  6, }, // Africa/Maseru
    {    1, 5084,     747,  7, }, // Africa/Mbabane
    {    1, 5015,     754,  8, }, // Africa/Mogadishu
    {    1, 6997,     762,  8, }, // Africa/Monrovia
    {    1, 3755,     770,  7, }, // Africa/Nairobi
    {    1, 5173,     777,  7, }, // Africa/Ndjamena
    {    1, 5256,     784,  6, }, // Africa/Niamey
    {    1, 6766,     790,  7, }, // Africa/Nouakchott
    {    1, 6690,     797,  8, }, // Africa/Ouagadougou
    {    1, 5127,     805, 10, }, // Africa/Porto-Novo
    {    1, 4337,     815,  8, }, // Africa/Sao_Tome
    {    1, 2732,     823,  7, }, // Africa/Tripoli
    {    1, 6503,     830,  5, }, // Africa/Tunis
    {    1, 4054,     835,  7, }, // Africa/Windhoek
    {    1,  132,     842,  4, }, // America/Adak
    {    1, 3183,     846,  7, }, // America/Anchorage
    {    1, 5524,     853,  7, }, // America/Anguilla
    {    1, 5541,     860,  7, }, // America/Antigua
    {    1, 4456,     867,  9, }, // America/Araguaina
    {    1,  166,     876, 12, }, // America/Argentina/Buenos_Aires
    {    1,   91,     888,  9, }, // America/Argentina/Catamarca
    {    1,  270,     897,  7, }, // America/Argentina/Cordoba
    {    1,  441,     904,  5, }, // America/Argentina/Jujuy
    {    1, 7425,     909,  8, }, // America/Argentina/La_Rioja
    {    1,  565,     917,  7, }, // America/Argentina/Mendoza
    {    1, 7452,     924, 11, }, // America/Argentina/Rio_Gallegos
    {    1, 7483,     935,  5, }, // America/Argentina/Salta
    {    1, 7507,     940,  8, }, // America/Argentina/San_Juan
    {    1, 5497,     948,  8, }, // America/Argentina/San_Luis
    {    1, 7534,     956,  7, }, // America/Argentina/Tucuman
    {    1, 7560,     963,  6, }, // America/Argentina/Ushuaia
    {    1, 5557,     969,  5, }, // America/Aruba
    {    1, 4154,     974,  7, }, // America/Asuncion
    {    1,  237,     981, 12, }, // America/Atikokan
    {    1, 3581,     993,  4, }, // America/Bahia
    {    1, 7586,     997, 16, }, // America/Bahia_Banderas
    {    1, 5571,    1013,  8, }, // America/Barbados
    {    1, 7609,    1021,  5, }, // America/Belem
    {    1, 5309,    1026,  5, }, // America/Belize
    {    1, 7623,    1031, 12, }, // America/Blanc-Sablon
    {    1, 7644,    1043,  9, }, // America/Boa_Vista
    {    1, 4263,    1052,  6, }, // America/Bogota
    {    1, 7662,    1058,  5, }, // America/Boise
    {    1, 7676,    1063, 12, }, // America/Cambridge_Bay
    {    1, 7698,    1075, 12, }, // America/Campo_Grande
    {    1, 3770,    1087,  6, }, // America/Cancun
    {    1, 4596,    1093,  7, }, // America/Caracas
    {    1, 4247,    1100,  7, }, // America/Cayenne
    {    1, 5398,    1107, 22, }, // America/Cayman
    {    1, 3251,    1129,  6, }, // America/Chicago
    {    1, 7719,    1135,  6, }, // America/Chihuahua
    {    1, 7737,      53, 13, }, // America/Ciudad_Juarez
    {    1, 5324,    1141, 10, }, // America/Costa_Rica
    {    1, 7759,    1151,  7, }, // America/Creston
    {    1, 3693,    1158,  6, }, // America/Cuiaba
    {    1, 5640,    1164,  8, }, // America/Curacao
    {    1, 7775,    1172, 12, }, // America/Danmarkshavn
    {    1, 7796,    1184,  6, }, // America/Dawson
    {    1, 7811,    1190, 11, }, // America/Dawson_Creek
    {    1,  805,    1201,  6, }, // America/Denver
    {    1, 3351,    1207,  7, }, // America/Detroit
    {    1, 5656,    1214,  8, }, // America/Dominica
    {    1,  893,    1222,  8, }, // America/Edmonton
    {    1, 7832,    1230,  8, }, // America/Eirunepe
    {    1, 5343,    1238,  8, }, // America/El_Salvador
    {    1, 7849,    1246, 11, }, // America/Fort_Nelson
    {    1, 7869,    1257,  9, }, // America/Fortaleza
    {    1, 7887,    1266,  9, }, // America/Glace_Bay
    {    1, 6844,    1275,  7, }, // America/Goose_Bay
    {    1, 4514,    1282, 10, }, // America/Grand_Turk
    {    1, 5687,    1292,  7, }, // America/Grenada
    {    1, 5703,    1299,  9, }, // America/Guadeloupe
    {    1, 3662,    1308,  9, }, // America/Guatemala
    {    1, 4743,    1317,  8, }, // America/Guayaquil
    {    1, 6895,    1325,  6, }, // America/Guyana
    {    1, 1923,    1331,  8, }, // America/Halifax
    {    1, 2221,    1339,  6, }, // America/Havana
    {    1, 7905,    1345,  8, }, // America/Hermosillo
    {    1,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {    1,  481,    1365, 13, }, // America/Indiana/Knox
    {    1, 7924,    1378, 16, }, // America/Indiana/Marengo
    {    1, 7948,    1394, 19, }, // America/Indiana/Petersburg
    {    1, 7975,    1413,  9, }, // America/Indiana/Tell_City
    {    1, 8001,    1422, 14, }, // America/Indiana/Vevay
    {    1, 8023,    1436,  9, }, // America/Indiana/Vincennes
    {    1, 8049,    1445,  7, }, // America/Indiana/Winamac
    {    1, 8073,    1452,  6, }, // America/Inuvik
    {    1,  660,    1458,  7, }, // America/Iqaluit
    {    1, 2665,    1465,  7, }, // America/Jamaica
    {    1, 5282,    1472,  4, }, // America/Juneau
    {    1,  521,    1476,  8, }, // America/Kentucky/Louisville
    {    1, 8088,    1484, 20, }, // America/Kentucky/Monticello
    {    1, 5621,    1504, 10, }, // America/Kralendijk
    {    1, 4278,    1514,  6, }, // America/La_Paz
    {    1, 7132,    1520,  4, }, // America/Lima
    {    1, 3390,    1524, 11, }, // America/Los_Angeles
    {    1, 5849,    1535, 18, }, // America/Lower_Princes
    {    1, 8116,    1553,  6, }, // America/Maceio
    {    1, 8131,    1559,  7, }, // America/Managua
    {    1, 1892,    1566,  6, }, // America/Manaus
    {    1, 5814,    1572,  6, }, // America/Marigot
    {    1, 5722,    1578,  9, }, // America/Martinique
    {    1, 8147,    1587,  9, }, // America/Matamoros
    {    1, 2779,    1596,  8, }, // America/Mazatlan
    {    1, 8165,    1604,  8, }, // America/Menominee
    {    1, 8183,    1612,  6, }, // America/Merida
    {    1, 8198,    1618, 10, }, // America/Metlakatla
    {    1, 2811,    1628,  6, }, // America/Mexico_City
    {    1, 4293,    1634,  7, }, // America/Miquelon
    {    1, 8217,    1641,  7, }, // America/Moncton
    {    1, 8233,    1648,  9, }, // America/Monterrey
    {    1, 4000,    1657, 10, }, // America/Montevideo
    {    1, 5741,    1667, 10, }, // America/Montserrat
    {    1, 5383,    1677,  6, }, // America/Nassau
    {    1, 3294,    1683,  8, }, // America/New_York
    {    1, 8251,    1691,  3, }, // America/Nome
    {    1, 1834,    1694,  7, }, // America/Noronha
    {    1, 8264,    1701, 22, }, // America/North_Dakota/Beulah
    {    1, 8292,    1723, 22, }, // America/North_Dakota/Center
    {    1, 8320,    1745, 27, }, // America/North_Dakota/New_Salem
    {    1,  393,    1772,  4, }, // America/Nuuk
    {    1, 8351,    1776,  7, }, // America/Ojinaga
    {    1, 5413,    1783,  6, }, // America/Panama
    {    1, 6106,    1789, 10, }, // America/Paramaribo
    {    1, 3224,    1799,  6, }, // America/Phoenix
    {    1, 3847,    1805, 12, }, // America/Port-au-Prince
    {    1, 5871,    1817, 13, }, // America/Port_of_Spain
    {    1, 8367,    1830, 11, }, // America/Porto_Velho
    {    1, 5760,    1841, 11, }, // America/Puerto_Rico
    {    1, 3932,    1852, 12, }, // America/Punta_Arenas
    {    1, 8387,    1864, 12, }, // America/Rankin_Inlet
    {    1, 8408,    1876,  6, }, // America/Recife
    {    1, 1979,    1882,  7, }, // America/Regina
    {    1, 8423,    1889,  8, }, // America/Resolute
    {    1,  695,    1897, 10, }, // America/Rio_Branco
    {    1, 8440,    1907,  8, }, // America/Santarem
    {    1, 2165,    1915,  8, }, // America/Santiago
    {    1, 6044,    1923, 13, }, // America/Santo_Domingo
    {    1, 1862,    1936,  9, }, // America/Sao_Paulo
    {    1, 6862,    1945, 11, }, // America/Scoresbysund
    {    1, 8457,    1956,  5, }, // America/Sitka
    {    1, 8471,    1961, 13, }, // America/St_Barthelemy
    {    1, 2045,    1974,  9, }, // America/St_Johns
    {    1, 5780,    1983,  9, }, // America/St_Kitts
    {    1, 5797,    1992, 11, }, // America/St_Lucia
    {    1,  855,    2003, 10, }, // America/St_Thomas
    {    1, 5830,    2013, 12, }, // America/St_Vincent
    {    1, 8493,    2025, 13, }, // America/Swift_Current
    {    1, 5363,    2038, 11, }, // America/Tegucigalpa
    {    1, 5673,    2049,  4, }, // America/Thule
    {    1,  313,    2053,  7, }, // America/Tijuana
    {    1,  608,    2060,  7, }, // America/Toronto
    {    1, 5605,    2067,  7, }, // America/Tortola
    {    1, 2077,    2074,  8, }, // America/Vancouver
    {    1, 2128,    2082,  8, }, // America/Whitehorse
    {    1,  734,    2090,  8, }, // America/Winnipeg
    {    1, 8515,    2098,  7, }, // America/Yakutat
    {    1, 5918,    2105,  5, }, // Antarctica/Casey
    {    1, 6027,    2110,  6, }, // Antarctica/Davis
    {    1, 6066,    2116, 15, }, // Antarctica/DumontDUrville
    {    1, 8531,    2131,  8, }, // Antarctica/Macquarie
    {    1, 7029,    2139,  6, }, // Antarctica/Mawson
    {    1, 7076,    2145,  9, }, // Antarctica/McMurdo
    {    1, 8552,    2154,  6, }, // Antarctica/Palmer
    {    1, 7189,    2160,  6, }, // Antarctica/Rothera
    {    1, 7258,    2166,  5, }, // Antarctica/Syowa
    {    1, 8570,    2171,  5, }, // Antarctica/Troll
    {    1, 7337,    2176, 10, }, // Antarctica/Vostok
    {    1, 1413,    2186,  6, }, // Arctic/Longyearbyen
    {    1, 5487,    2192,  4, }, // Asia/Aden
    {    1, 5297,    2196,  6, }, // Asia/Almaty
    {    1, 3870,    2202,  5, }, // Asia/Amman
    {    1, 5428,    2207,  6, }, // Asia/Anadyr
    {    1, 5440,    2213,  5, }, // Asia/Aqtau
    {    1, 5270,    2218,  6, }, // Asia/Aqtobe
    {    1,  964,    2224,  7, }, // Asia/Ashgabat
    {    1, 8587,    2231,  6, }, // Asia/Atyrau
    {    1, 3509,    2237,  6, }, // Asia/Baghdad
    {    1, 5451,    2243,  7, }, // Asia/Bahrain
    {    1, 3555,    2250,  4, }, // Asia/Baku
    {    1, 4368,    2254,  7, }, // Asia/Bangkok
    {    1, 3473,    2261,  7, }, // Asia/Barnaul
    {    1, 3988,    2268,  6, }, // Asia/Beirut
    {    1, 3680,    2274,  6, }, // Asia/Bishkek
    {    1, 5906,    2280,  6, }, // Asia/Brunei
    {    1, 4503,    2286,  4, }, // Asia/Chita
    {    1, 5963,    2290,  9, }, // Asia/Choibalsan
    {    1, 4413,    2299,  7, }, // Asia/Colombo
    {    1, 4442,    2306,  6, }, // Asia/Damascus
    {    1, 1060,    2312,  5, }, // Asia/Dhaka
    {    1, 6125,    2317,  4, }, // Asia/Dili
    {    1, 3498,    2321,  5, }, // Asia/Dubai
    {    1, 6092,    2326,  7, }, // Asia/Dushanbe
    {    1, 8599,    2333,  9, }, // Asia/Famagusta
    {    1, 8614,    2342,  4, }, // Asia/Gaza
    {    1, 4697,    2346,  6, }, // Asia/Hebron
    {    1, 1226,    2352,  7, }, // Asia/Ho_Chi_Minh
    {    1, 2576,    2359,  7, }, // Asia/Hong_Kong
    {    1, 4673,    2366,  4, }, // Asia/Hovd
    {    1, 4086,    2370,  7, }, // Asia/Irkutsk
    {    1, 6969,    2377,  7, }, // Asia/Jakarta
    {    1, 6955,    2384,  8, }, // Asia/Jayapura
    {    1, 1257,    2392,  9, }, // Asia/Jerusalem
    {    1, 3462,    2401,  5, }, // Asia/Kabul
    {    1, 4218,    2406, 25, }, // Asia/Kamchatka
    {    1, 4141,    2431,  6, }, // Asia/Karachi
    {    1, 1152,    2437,  8, }, // Asia/Kathmandu
    {    1, 8624,    2445,  7, }, // Asia/Khandyga
    {    1,  992,    2452,  8, }, // Asia/Kolkata
    {    1, 4099,    2460, 11, }, // Asia/Krasnoyarsk
    {    1, 4761,    2471, 12, }, // Asia/Kuala_Lumpur
    {    1, 5893,    2483,  6, }, // Asia/Kuching
    {    1, 5464,    2489,  6, }, // Asia/Kuwait
    {    1, 1178,    2495,  5, }, // Asia/Macau
    {    1, 3919,    2500,  7, }, // Asia/Magadan
    {    1, 1316,    2507,  7, }, // Asia/Makassar
    {    1, 7145,    2514,  6, }, // Asia/Manila
    {    1, 6883,    2520,  6, }, // Asia/Muscat
    {    1, 2446,    2526,  7, }, // Asia/Nicosia
    {    1, 8638,    2533, 11, }, // Asia/Novokuznetsk
    {    1, 4037,    2544, 11, }, // Asia/Novosibirsk
    {    1, 4131,    2555,  4, }, // Asia/Omsk
    {    1, 7108,    2559,  6, }, // Asia/Oral
    {    1, 6924,    2565,  7, }, // Asia/Phnom_Penh
    {    1, 8656,    2572,  9, }, // Asia/Pontianak
    {    1, 4116,    2581,  7, }, // Asia/Pyongyang
    {    1, 5476,    2588,  5, }, // Asia/Qatar
    {    1, 8671,    2593,  8, }, // Asia/Qostanay
    {    1, 4171,    2601,  9, }, // Asia/Qyzylorda
    {    1, 3486,    2610,  7, }, // Asia/Riyadh
    {    1, 4310,    2617, 13, }, // Asia/Sakhalin
    {    1, 7208,    2630,  9, }, // Asia/Samarkand
    {    1, 3116,    2639,  4, }, // Asia/Seoul
    {    1, 1020,    2643,  6, }, // Asia/Shanghai
    {    1, 3137,    2649,  8, }, // Asia/Singapore
    {    1, 4199,    2657, 13, }, // Asia/Srednekolymsk
    {    1, 3100,    2670,  6, }, // Asia/Taipei
    {    1, 4683,    2676,  7, }, // Asia/Tashkent
    {    1, 3817,    2683,  4, }, // Asia/Tbilisi
    {    1, 2638,    2687,  7, }, // Asia/Tehran
    {    1, 1284,    2694,  7, }, // Asia/Thimphu
    {    1, 2687,    2701,  5, }, // Asia/Tokyo
    {    1, 4474,    2706,  5, }, // Asia/Tomsk
    {    1, 1346,    2711, 10, }, // Asia/Ulaanbaatar
    {    1, 1126,    2721,  6, }, // Asia/Urumqi
    {    1, 8685,    2727,  8, }, // Asia/Ust-Nera
    {    1, 6940,    2735,  7, }, // Asia/Vientiane
    {    1, 4612,    2742, 11, }, // Asia/Vladivostok
    {    1, 4730,    2753,  7, }, // Asia/Yakutsk
    {    1, 1202,    2760,  6, }, // Asia/Yangon
    {    1, 3785,    2766, 12, }, // Asia/Yekaterinburg
    {    1, 3649,    2778,  6, }, // Asia/Yerevan
    {    1, 3565,    2784, 16, }, // Atlantic/Azores
    {    1, 5588,    2800, 18, }, // Atlantic/Bermuda
    {    1, 6591,    2818, 17, }, // Atlantic/Canary
    {    1, 3629,    2835, 10, }, // Atlantic/Cape_Verde
    {    1, 1379,    2845, 17, }, // Atlantic/Faroe
    {    1, 8699,    2862,  7, }, // Atlantic/Madeira
    {    1, 2614,    2869,  9, }, // Atlantic/Reykjavik
    {    1, 7235,    2878, 16, }, // Atlantic/South_Georgia
    {    1, 6784,    2894, 26, }, // Atlantic/St_Helena
    {    1, 6607,    2920,  6, }, // Atlantic/Stanley
    {    1, 1654,    2926,  8, }, // Australia/Adelaide
    {    1, 1619,    2934,  7, }, // Australia/Brisbane
    {    1, 1783,    2941, 11, }, // Australia/Broken_Hill
    {    1, 1567,    2952,  6, }, // Australia/Darwin
    {    1, 3539,    2958,  5, }, // Australia/Eucla
    {    1, 1500,    2963,  6, }, // Australia/Hobart
    {    1, 8716,    2969,  8, }, // Australia/Lindeman
    {    1, 1531,    2977,  8, }, // Australia/Lord_Howe
    {    1, 1711,    2985,  7, }, // Australia/Melbourne
    {    1, 1746,    2992,  4, }, // Australia/Perth
    {    1, 1447,    2996,  6, }, // Australia/Sydney
    {    1, 6390,    3002,  9, }, // Europe/Amsterdam
    {    1, 6149,    3011,  7, }, // Europe/Andorra
    {    1, 3522,    3018,  8, }, // Europe/Astrakhan
    {    1, 6577,    3026,  4, }, // Europe/Athens
    {    1, 6285,    3030,  7, }, // Europe/Belgrade
    {    1, 4659,    3037,  6, }, // Europe/Berlin
    {    1, 6437,    3043, 10, }, // Europe/Bratislava
    {    1, 6178,    3053,  8, }, // Europe/Brussels
    {    1, 3830,    3061,  8, }, // Europe/Bucharest
    {    1, 3708,    3069,  8, }, // Europe/Budapest
    {    1, 8751,    3077, 23, }, // Europe/Busingen
    {    1, 2475,    3100,  7, }, // Europe/Chisinau
    {    1, 6238,    3107, 10, }, // Europe/Copenhagen
    {    1, 2260,    3117,  6, }, // Europe/Dublin
    {    1, 6256,    3123,  9, }, // Europe/Gibraltar
    {    1, 8767,    3132,  6, }, // Europe/Guernsey
    {    1, 6561,    3138,  8, }, // Europe/Helsinki
    {    1, 8783,    3146,  9, }, // Europe/Isle_of_Man
    {    1, 1097,    3155,  7, }, // Europe/Istanbul
    {    1, 8802,    3162,  5, }, // Europe/Jersey
    {    1, 3881,    3167, 11, }, // Europe/Kaliningrad
    {    1, 8924,    3178,  5, }, // Europe/Kirov
    {    1, 2419,    3183,  4, }, // Europe/Kyiv
    {    1, 3078,    3187,  8, }, // Europe/Lisbon
    {    1, 6455,    3195,  9, }, // Europe/Ljubljana
    {    1, 2393,    3204,  6, }, // Europe/London
    {    1, 6314,    3210, 11, }, // Europe/Luxembourg
    {    1, 4794,    3221,  6, }, // Europe/Madrid
    {    1, 6346,    3227,  5, }, // Europe/Malta
    {    1, 6531,    3232,  9, }, // Europe/Mariehamn
    {    1, 3595,    3241,  5, }, // Europe/Minsk
    {    1, 6359,    3246,  6, }, // Europe/Monaco
    {    1, 3443,    3252,  6, }, // Europe/Moscow
    {    1, 6407,    3258,  4, }, // Europe/Oslo
    {    1, 4186,    3262,  5, }, // Europe/Paris
    {    1, 6373,    3267,  9, }, // Europe/Podgorica
    {    1, 6224,    3276,  5, }, // Europe/Prague
    {    1, 8816,    3281,  4, }, // Europe/Riga
    {    1, 6273,    3285,  3, }, // Europe/Rome
    {    1, 4233,    3288,  6, }, // Europe/Samara
    {    1, 6419,    3294, 10, }, // Europe/San_Marino
    {    1, 6194,    3304,  7, }, // Europe/Sarajevo
    {    1, 4353,    3311,  7, }, // Europe/Saratov
    {    1, 8828,    3318, 10, }, // Europe/Simferopol
    {    1, 6332,    3328,  5, }, // Europe/Skopje
    {    1, 6548,    3333,  5, }, // Europe/Sofia
    {    1, 6472,    3338,  8, }, // Europe/Stockholm
    {    1, 8846,    3346,  6, }, // Europe/Tallinn
    {    1, 6135,    3352,  6, }, // Europe/Tirane
    {    1, 8861,    3358,  9, }, // Europe/Ulyanovsk
    {    1, 6301,    3367,  5, }, // Europe/Vaduz
    {    1, 6516,    3372,  7, }, // Europe/Vatican
    {    1, 6164,    3379,  4, }, // Europe/Vienna
    {    1, 8878,    3383,  7, }, // Europe/Vilnius
    {    1, 4629,    3390,  9, }, // Europe/Volgograd
    {    1, 3055,    3399,  7, }, // Europe/Warsaw
    {    1, 6210,    3406,  6, }, // Europe/Zagreb
    {    1, 6489,    3412,  6, }, // Europe/Zurich
    {    1, 4980,    3418, 12, }, // Indian/Antananarivo
    {    1, 6910,    3430,  5, }, // Indian/Chagos
    {    1, 5979,    3435, 12, }, // Indian/Christmas
    {    1, 5996,    3447, 17, }, // Indian/Cocos
    {    1, 4931,    3464,  5, }, // Indian/Comoro
    {    1, 6624,    3469,  8, }, // Indian/Kerguelen
    {    1, 7223,    3477,  3, }, // Indian/Mahe
    {    1, 7013,    3480,  6, }, // Indian/Maldives
    {    1, 3971,    3486,  7, }, // Indian/Mauritius
    {    1, 5000,    3493,  7, }, // Indian/Mayotte
    {    1, 7174,    3500,  7, }, // Indian/Reunion
    {    1, 4324,    3507,  4, }, // Pacific/Apia
    {    1,  932,    3511,  6, }, // Pacific/Auckland
    {    1, 3608,    3517,  8, }, // Pacific/Bougainville
    {    1, 2863,    3525,  5, }, // Pacific/Chatham
    {    1, 3022,    3530,  4, }, // Pacific/Chuuk
    {    1, 2201,    3534, 11, }, // Pacific/Easter
    {    1, 7323,    3545,  5, }, // Pacific/Efate
    {    1, 7290,    3550,  7, }, // Pacific/Fakaofo
    {    1, 3804,    3557,  4, }, // Pacific/Fiji
    {    1, 7306,    3561,  8, }, // Pacific/Funafuti
    {    1, 6641,    3569, 21, }, // Pacific/Galapagos
    {    1, 6659,    3590, 14, }, // Pacific/Gambier
    {    1, 3724,    3604, 11, }, // Pacific/Guadalcanal
    {    1, 5935,    3615,  4, }, // Pacific/Guam
    {    1, 2897,    3619,  6, }, // Pacific/Kanton
    {    1, 3900,    3625, 10, }, // Pacific/Kiritimati
    {    1, 6982,    3635,  6, }, // Pacific/Kosrae
    {    1, 2708,    3641,  9, }, // Pacific/Kwajalein
    {    1, 4779,    3650,  6, }, // Pacific/Majuro
    {    1, 3953,    3656, 18, }, // Pacific/Marquesas
    {    1, 8909,    3674, 15, }, // Pacific/Midway
    {    1, 7047,    3689,  5, }, // Pacific/Nauru
    {    1, 7095,    3694,  4, }, // Pacific/Niue
    {    1, 4070,    3698,  7, }, // Pacific/Norfolk
    {    1, 7061,    3705,  5, }, // Pacific/Noumea
    {    1, 2991,    3710,  9, }, // Pacific/Pago_Pago
    {    1, 7118,    3719,  5, }, // Pacific/Palau
    {    1, 7157,    3724,  7, }, // Pacific/Pitcairn
    {    1, 2961,    3731,  6, }, // Pacific/Pohnpei
    {    1, 4709,    3737, 11, }, // Pacific/Port_Moresby
    {    1, 6009,    3748,  9, }, // Pacific/Rarotonga
    {    1, 5948,    3757,  6, }, // Pacific/Saipan
    {    1, 7275,    3763,  5, }, // Pacific/Tahiti
    {    1, 6675,    3768,  6, }, // Pacific/Tarawa
    {    1, 4485,    3774,  9, }, // Pacific/Tongatapu
    {    1, 7355,    3783,  4, }, // Pacific/Wake
    {    1, 7368,    3787,  6, }, // Pacific/Wallis
    {    2,   14,       0,  6, }, // Africa/Asmara Afar/Latin/Ethiopia
    {    2, 4337,       6,  8, }, // Africa/Sao_Tome
    {    2, 4154,      14,  8, }, // America/Asuncion
    {    2,  237,      22,  8, }, // America/Atikokan
    {    2, 7586,      30, 17, }, // America/Bahia_Banderas
    {    2, 3770,      47,  6, }, // America/Cancun
    {    2, 7737,      53, 13, }, // America/Ciudad_Juarez
    {    2, 5640,      66,  7, }, // America/Curacao
    {    2,  481,      73, 13, }, // America/Indiana/Knox
    {    2, 7924,      86, 16, }, // America/Indiana/Marengo
    {    2, 7948,     102, 19, }, // America/Indiana/Petersburg
    {    2, 7975,     121, 18, }, // America/Indiana/Tell_City
    {    2, 8001,     139, 14, }, // America/Indiana/Vevay
    {    2, 8023,     153, 18, }, // America/Indiana/Vincennes
    {    2, 8049,     171, 16, }, // America/Indiana/Winamac
    {    2, 8088,     187, 20, }, // America/Kentucky/Monticello
    {    2, 5849,     207, 22, }, // America/Lower_Princes
    {    2, 8183,     229,  6, }, // America/Merida
    {    2, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {    2, 8292,     255, 20, }, // America/North_Dakota/Center
    {    2, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {    2,  393,     298,  4, }, // America/Nuuk
    {    2, 6862,     302, 16, }, // America/Scoresbysund
    {    2, 8471,     318, 14, }, // America/St_Barthelemy
    {    2, 2045,     332, 10, }, // America/St_Johns
    {    2, 5780,     342,  9, }, // America/St_Kitts
    {    2, 5797,     351,  9, }, // America/St_Lucia
    {    2,  855,     360, 10, }, // America/St_Thomas
    {    2, 5830,     370, 11, }, // America/St_Vincent
    {    2, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {    2, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {    2, 1152,     413,  9, }, // Asia/Kathmandu
    {    2,  992,     422,  7, }, // Asia/Kolkata
    {    2, 1178,     429,  5, }, // Asia/Macau
    {    2, 1202,     442,  6, }, // Asia/Yangon
    {    2, 1379,     448,  5, }, // Atlantic/Faroe
    {    2, 6784,     453, 10, }, // Atlantic/St_Helena
    {    2, 2419,     463,  4, }, // Europe/Kyiv
    {    2, 7174,     467,  7, }, // Indian/Reunion
    {    2, 3022,     474,  5, }, // Pacific/Chuuk
    {    2, 2961,     479,  7, }, // Pacific/Pohnpei
    {    5, 4961,    3793, 11, }, // Africa/Addis_Ababa Afrikaans/Latin/South Africa
    {    5,   14,       0,  6, }, // Africa/Asmara
    {    5, 2242,    3804,  5, }, // Africa/Cairo
    {    5, 4945,    3809,  8, }, // Africa/Djibouti
    {    5, 4426,    3817,  7, }, // Africa/Khartoum
    {    5, 5015,    3824, 10, }, // Africa/Mogadishu
    {    5, 4337,       6,  8, }, // Africa/Sao_Tome
    {    5, 4154,      14,  8, }, // America/Asuncion
    {    5,  237,      22,  8, }, // America/Atikokan
    {    5, 7586,    3834, 14, }, // America/Bahia_Banderas
    {    5, 7676,    3848, 13, }, // America/Cambridge_Bay
    {    5, 3770,    3861,  6, }, // America/Cancun
    {    5, 5398,    3867,  7, }, // America/Cayman
    {    5, 7737,      53, 13, }, // America/Ciudad_Juarez
    {    5, 5640,      66,  7, }, // America/Curacao
    {    5, 7887,    3874,  9, }, // America/Glace_Bay
    {    5, 6844,    3883,  9, }, // America/Goose_Bay
    {    5,  481,      73, 13, }, // America/Indiana/Knox
    {    5, 7924,      86, 16, }, // America/Indiana/Marengo
    {    5, 7948,     102, 19, }, // America/Indiana/Petersburg
    {    5, 7975,     121, 18, }, // America/Indiana/Tell_City
    {    5, 8001,     139, 14, }, // America/Indiana/Vevay
    {    5, 8023,     153, 18, }, // America/Indiana/Vincennes
    {    5, 8049,     171, 16, }, // America/Indiana/Winamac
    {    5, 2665,    3892,  7, }, // America/Jamaica
    {    5, 8088,     187, 20, }, // America/Kentucky/Monticello
    {    5, 5849,     207, 22, }, // America/Lower_Princes
    {    5, 8183,    3899,  6, }, // America/Merida
    {    5, 2811,    3905, 11, }, // America/Mexico_City
    {    5, 8264,    3916, 20, }, // America/North_Dakota/Beulah
    {    5, 8292,    3936, 20, }, // America/North_Dakota/Center
    {    5, 8320,    3956, 23, }, // America/North_Dakota/New_Salem
    {    5,  393,     298,  4, }, // America/Nuuk
    {    5, 6862,     302, 16, }, // America/Scoresbysund
    {    5, 8471,    3979, 15, }, // America/St_Barthelemy
    {    5, 2045,    3994, 11, }, // America/St_Johns
    {    5, 5780,    4005, 10, }, // America/St_Kitts
    {    5, 5797,    4015, 10, }, // America/St_Lucia
    {    5,  855,    4025, 11, }, // America/St_Thomas
    {    5, 5830,    4036, 12, }, // America/St_Vincent
    {    5, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {    5, 7337,    4048,  6, }, // Antarctica/Vostok
    {    5,  964,    4054,  9, }, // Asia/Ashgabat
    {    5, 3509,    4063,  6, }, // Asia/Baghdad
    {    5, 5451,    4069,  7, }, // Asia/Bahrain
    {    5, 3555,    4076,  5, }, // Asia/Baku
    {    5, 3988,    4081,  7, }, // Asia/Beirut
    {    5, 3680,    4088,  7, }, // Asia/Bishkek
    {    5, 5906,    4095,  7, }, // Asia/Brunei
    {    5, 4442,    4102,  8, }, // Asia/Damascus
    {    5, 3498,    4110,  6, }, // Asia/Dubai
    {    5, 1226,    4116, 16, }, // Asia/Ho_Chi_Minh
    {    5, 2576,    4132,  8, }, // Asia/Hong_Kong
    {    5, 4086,    4140,  8, }, // Asia/Irkutsk
    {    5, 6969,    4148,  8, }, // Asia/Jakarta
    {    5, 3462,    4156,  6, }, // Asia/Kabul
    {    5, 4218,    4162, 10, }, // Asia/Kamchatka
    {    5, 4141,    4172,  8, }, // Asia/Karachi
    {    5, 1152,    4180,  9, }, // Asia/Kathmandu
    {    5,  992,     422,  7, }, // Asia/Kolkata
    {    5, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {    5, 4761,    4200, 15, }, // Asia/Kuala_Lumpur
    {    5, 5464,    4215,  7, }, // Asia/Kuwait
    {    5, 1178,    4222,  5, }, // Asia/Macau
    {    5, 6883,    4227,  6, }, // Asia/Muscat
    {    5, 5476,    4233,  5, }, // Asia/Qatar
    {    5, 8671,     434,  8, }, // Asia/Qostanay
    {    5, 3486,    4238,  5, }, // Asia/Riyadh
    {    5, 3116,    4243,  5, }, // Asia/Seoul
    {    5, 3137,    4248,  9, }, // Asia/Singapore
    {    5, 2687,    4257,  5, }, // Asia/Tokyo
    {    5, 4612,    4262, 11, }, // Asia/Vladivostok
    {    5, 4730,    4273,  8, }, // Asia/Yakutsk
    {    5, 1202,     442,  6, }, // Asia/Yangon
    {    5, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {    5, 3565,    4294,  5, }, // Atlantic/Azores
    {    5, 6591,    4299,  7, }, // Atlantic/Canary
    {    5, 3629,    4306, 10, }, // Atlantic/Cape_Verde
    {    5, 1379,     448,  5, }, // Atlantic/Faroe
    {    5, 7235,    4316, 12, }, // Atlantic/South_Georgia
    {    5, 6784,    4328, 11, }, // Atlantic/St_Helena
    {    5, 6577,    4339,  6, }, // Europe/Athens
    {    5, 4659,    4345,  6, }, // Europe/Berlin
    {    5, 6178,    4351,  7, }, // Europe/Brussels
    {    5, 3830,    4358,  9, }, // Europe/Bucharest
    {    5, 3708,    4367,  9, }, // Europe/Budapest
    {    5, 6238,    4376, 10, }, // Europe/Copenhagen
    {    5, 8783,    4386, 10, }, // Europe/Isle_of_Man
    {    5, 2419,    4396,  4, }, // Europe/Kyiv
    {    5, 3078,    4400,  8, }, // Europe/Lisbon
    {    5, 2393,    4408,  6, }, // Europe/London
    {    5, 6314,    4414,  9, }, // Europe/Luxembourg
    {    5, 3443,    4423,  6, }, // Europe/Moscow
    {    5, 4186,    4429,  5, }, // Europe/Paris
    {    5, 6224,    4434,  5, }, // Europe/Prague
    {    5, 4353,    4439,  7, }, // Europe/Saratov
    {    5, 6516,    4446, 12, }, // Europe/Vatican
    {    5, 6164,    4458,  4, }, // Europe/Vienna
    {    5, 4629,    4462,  9, }, // Europe/Volgograd
    {    5, 3055,    4471,  7, }, // Europe/Warsaw
    {    5, 6489,    4478,  6, }, // Europe/Zurich
    {    5, 5996,    4484,  5, }, // Indian/Cocos
    {    5, 4931,    4489,  6, }, // Indian/Comoro
    {    5, 7013,    4495,  8, }, // Indian/Maldives
    {    5, 7174,     467,  7, }, // Indian/Reunion
    {    5, 3022,     474,  5, }, // Pacific/Chuuk
    {    5, 2201,    4503,  4, }, // Pacific/Easter
    {    5, 3804,    4507,  5, }, // Pacific/Fiji
    {    5, 2929,    4512,  8, }, // Pacific/Honolulu
    {    5, 2897,    4520,  9, }, // Pacific/Kanton
    {    5, 7061,    4529,  6, }, // Pacific/Noumea
    {    5, 2961,     479,  7, }, // Pacific/Pohnpei
    {    5, 7368,    4535,  8, }, // Pacific/Wallis
    {    7,   14,       0,  6, }, // Africa/Asmara Aghem/Latin/Cameroon
    {    7, 4337,       6,  8, }, // Africa/Sao_Tome
    {    7, 4154,      14,  8, }, // America/Asuncion
    {    7,  237,      22,  8, }, // America/Atikokan
    {    7, 7586,      30, 17, }, // America/Bahia_Banderas
    {    7, 3770,      47,  6, }, // America/Cancun
    {    7, 7737,      53, 13, }, // America/Ciudad_Juarez
    {    7, 5640,      66,  7, }, // America/Curacao
    {    7,  481,      73, 13, }, // America/Indiana/Knox
    {    7, 7924,      86, 16, }, // America/Indiana/Marengo
    {    7, 7948,     102, 19, }, // America/Indiana/Petersburg
    {    7, 7975,     121, 18, }, // America/Indiana/Tell_City
    {    7, 8001,     139, 14, }, // America/Indiana/Vevay
    {    7, 8023,     153, 18, }, // America/Indiana/Vincennes
    {    7, 8049,     171, 16, }, // America/Indiana/Winamac
    {    7, 8088,     187, 20, }, // America/Kentucky/Monticello
    {    7, 5849,     207, 22, }, // America/Lower_Princes
    {    7, 8183,     229,  6, }, // America/Merida
    {    7, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {    7, 8292,     255, 20, }, // America/North_Dakota/Center
    {    7, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {    7,  393,     298,  4, }, // America/Nuuk
    {    7, 6862,     302, 16, }, // America/Scoresbysund
    {    7, 8471,     318, 14, }, // America/St_Barthelemy
    {    7, 2045,     332, 10, }, // America/St_Johns
    {    7, 5780,     342,  9, }, // America/St_Kitts
    {    7, 5797,     351,  9, }, // America/St_Lucia
    {    7,  855,     360, 10, }, // America/St_Thomas
    {    7, 5830,     370, 11, }, // America/St_Vincent
    {    7, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {    7, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {    7, 1152,     413,  9, }, // Asia/Kathmandu
    {    7,  992,     422,  7, }, // Asia/Kolkata
    {    7, 1178,     429,  5, }, // Asia/Macau
    {    7, 1202,     442,  6, }, // Asia/Yangon
    {    7, 1379,     448,  5, }, // Atlantic/Faroe
    {    7, 6784,     453, 10, }, // Atlantic/St_Helena
    {    7, 2419,     463,  4, }, // Europe/Kyiv
    {    7, 7174,     467,  7, }, // Indian/Reunion
    {    7, 3022,     474,  5, }, // Pacific/Chuuk
    {    7, 2961,     479,  7, }, // Pacific/Pohnpei
    {    8,   14,       0,  6, }, // Africa/Asmara Akan/Latin/Ghana
    {    8, 4337,       6,  8, }, // Africa/Sao_Tome
    {    8, 4154,      14,  8, }, // America/Asuncion
    {    8,  237,      22,  8, }, // America/Atikokan
    {    8, 7586,      30, 17, }, // America/Bahia_Banderas
    {    8, 3770,      47,  6, }, // America/Cancun
    {    8, 7737,      53, 13, }, // America/Ciudad_Juarez
    {    8, 5640,      66,  7, }, // America/Curacao
    {    8,  481,      73, 13, }, // America/Indiana/Knox
    {    8, 7924,      86, 16, }, // America/Indiana/Marengo
    {    8, 7948,     102, 19, }, // America/Indiana/Petersburg
    {    8, 7975,     121, 18, }, // America/Indiana/Tell_City
    {    8, 8001,     139, 14, }, // America/Indiana/Vevay
    {    8, 8023,     153, 18, }, // America/Indiana/Vincennes
    {    8, 8049,     171, 16, }, // America/Indiana/Winamac
    {    8, 8088,     187, 20, }, // America/Kentucky/Monticello
    {    8, 5849,     207, 22, }, // America/Lower_Princes
    {    8, 8183,     229,  6, }, // America/Merida
    {    8, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {    8, 8292,     255, 20, }, // America/North_Dakota/Center
    {    8, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {    8,  393,     298,  4, }, // America/Nuuk
    {    8, 6862,     302, 16, }, // America/Scoresbysund
    {    8, 8471,     318, 14, }, // America/St_Barthelemy
    {    8, 2045,     332, 10, }, // America/St_Johns
    {    8, 5780,     342,  9, }, // America/St_Kitts
    {    8, 5797,     351,  9, }, // America/St_Lucia
    {    8,  855,     360, 10, }, // America/St_Thomas
    {    8, 5830,     370, 11, }, // America/St_Vincent
    {    8, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {    8, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {    8, 1152,     413,  9, }, // Asia/Kathmandu
    {    8,  992,     422,  7, }, // Asia/Kolkata
    {    8, 1178,     429,  5, }, // Asia/Macau
    {    8, 1202,     442,  6, }, // Asia/Yangon
    {    8, 1379,     448,  5, }, // Atlantic/Faroe
    {    8, 6784,     453, 10, }, // Atlantic/St_Helena
    {    8, 2419,     463,  4, }, // Europe/Kyiv
    {    8, 7174,     467,  7, }, // Indian/Reunion
    {    8, 3022,     474,  5, }, // Pacific/Chuuk
    {    8, 2961,     479,  7, }, // Pacific/Pohnpei
    {    9,   14,       0,  6, }, // Africa/Asmara Akoose/Latin/Cameroon
    {    9, 4337,       6,  8, }, // Africa/Sao_Tome
    {    9, 4154,      14,  8, }, // America/Asuncion
    {    9,  237,      22,  8, }, // America/Atikokan
    {    9, 7586,      30, 17, }, // America/Bahia_Banderas
    {    9, 3770,      47,  6, }, // America/Cancun
    {    9, 7737,      53, 13, }, // America/Ciudad_Juarez
    {    9, 5640,      66,  7, }, // America/Curacao
    {    9,  481,      73, 13, }, // America/Indiana/Knox
    {    9, 7924,      86, 16, }, // America/Indiana/Marengo
    {    9, 7948,     102, 19, }, // America/Indiana/Petersburg
    {    9, 7975,     121, 18, }, // America/Indiana/Tell_City
    {    9, 8001,     139, 14, }, // America/Indiana/Vevay
    {    9, 8023,     153, 18, }, // America/Indiana/Vincennes
    {    9, 8049,     171, 16, }, // America/Indiana/Winamac
    {    9, 8088,     187, 20, }, // America/Kentucky/Monticello
    {    9, 5849,     207, 22, }, // America/Lower_Princes
    {    9, 8183,     229,  6, }, // America/Merida
    {    9, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {    9, 8292,     255, 20, }, // America/North_Dakota/Center
    {    9, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {    9,  393,     298,  4, }, // America/Nuuk
    {    9, 6862,     302, 16, }, // America/Scoresbysund
    {    9, 8471,     318, 14, }, // America/St_Barthelemy
    {    9, 2045,     332, 10, }, // America/St_Johns
    {    9, 5780,     342,  9, }, // America/St_Kitts
    {    9, 5797,     351,  9, }, // America/St_Lucia
    {    9,  855,     360, 10, }, // America/St_Thomas
    {    9, 5830,     370, 11, }, // America/St_Vincent
    {    9, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {    9, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {    9, 1152,     413,  9, }, // Asia/Kathmandu
    {    9,  992,     422,  7, }, // Asia/Kolkata
    {    9, 1178,     429,  5, }, // Asia/Macau
    {    9, 1202,     442,  6, }, // Asia/Yangon
    {    9, 1379,     448,  5, }, // Atlantic/Faroe
    {    9, 6784,     453, 10, }, // Atlantic/St_Helena
    {    9, 2419,     463,  4, }, // Europe/Kyiv
    {    9, 7174,     467,  7, }, // Indian/Reunion
    {    9, 3022,     474,  5, }, // Pacific/Chuuk
    {    9, 2961,     479,  7, }, // Pacific/Pohnpei
    {   10, 6751,    4543,  7, }, // Africa/Abidjan Albanian/Latin/Albania
    {   10, 6723,    4550,  4, }, // Africa/Accra
    {   10, 4961,    4554, 10, }, // Africa/Addis_Ababa
    {   10, 7383,    4564,  6, }, // Africa/Algiers
    {   10,   14,    4570,  6, }, // Africa/Asmara
    {   10, 6709,    4576,  7, }, // Africa/Banjul
    {   10, 7398,    4583,  5, }, // Africa/Bissau
    {   10, 4859,    4588,  8, }, // Africa/Blantyre
    {   10, 5189,    4596, 10, }, // Africa/Brazzaville
    {   10, 4824,    4606, 10, }, // Africa/Bujumbura
    {   10, 2242,    4616,  5, }, // Africa/Cairo
    {   10, 4019,    4621, 10, }, // Africa/Casablanca
    {   10, 7412,    4631,  6, }, // Africa/Ceuta
    {   10, 6736,    4637,  7, }, // Africa/Conakry
    {   10, 5032,    4644, 12, }, // Africa/Dar_es_Salaam
    {   10, 4945,    4656,  7, }, // Africa/Djibouti
    {   10, 5068,    4663,  7, }, // Africa/El_Aaiun
    {   10, 6816,    4670,  7, }, // Africa/Freetown
    {   10, 4381,    4677, 11, }, // Africa/Johannesburg
    {   10, 4401,    4688,  5, }, // Africa/Juba
    {   10, 4426,    4693,  6, }, // Africa/Khartoum
    {   10, 5238,    4699,  9, }, // Africa/Libreville
    {   10, 5256,    4708,  6, }, // Africa/Niamey
    {   10, 6766,    4714,  7, }, // Africa/Nouakchott
    {   10, 6690,    4721,  8, }, // Africa/Ouagadougou
    {   10, 4337,    4729,  8, }, // Africa/Sao_Tome
    {   10, 6503,    4737,  5, }, // Africa/Tunis
    {   10, 4054,    4742,  8, }, // Africa/Windhoek
    {   10, 3183,    4750,  9, }, // America/Anchorage
    {   10, 5524,    4759,  7, }, // America/Anguilla
    {   10, 4456,    4766, 10, }, // America/Araguaina
    {   10,  166,    4776, 12, }, // America/Argentina/Buenos_Aires
    {   10,   91,    4788,  9, }, // America/Argentina/Catamarca
    {   10,  270,    4797,  7, }, // America/Argentina/Cordoba
    {   10,  441,    4804,  5, }, // America/Argentina/Jujuy
    {   10, 7425,    4809,  8, }, // America/Argentina/La_Rioja
    {   10, 7452,    4817, 11, }, // America/Argentina/Rio_Gallegos
    {   10, 7483,    4828,  5, }, // America/Argentina/Salta
    {   10, 7507,    4833,  8, }, // America/Argentina/San_Juan
    {   10, 5497,    4841,  9, }, // America/Argentina/San_Luis
    {   10, 7534,    4850,  7, }, // America/Argentina/Tucuman
    {   10, 7560,    4857,  7, }, // America/Argentina/Ushuaia
    {   10, 5557,    4864,  5, }, // America/Aruba
    {   10, 4154,    4869,  8, }, // America/Asuncion
    {   10,  237,      22,  8, }, // America/Atikokan
    {   10, 7586,    4877, 14, }, // America/Bahia_Banderas
    {   10, 5309,    4891,  6, }, // America/Belize
    {   10, 7623,    4897, 12, }, // America/Blanc-Sablon
    {   10, 7644,    4909,  9, }, // America/Boa_Vista
    {   10, 4263,    4918,  6, }, // America/Bogota
    {   10, 7662,    4924,  5, }, // America/Boise
    {   10, 7676,    4929, 18, }, // America/Cambridge_Bay
    {   10, 7698,    4947, 12, }, // America/Campo_Grande
    {   10, 3770,    4959,  6, }, // America/Cancun
    {   10, 4596,    4965,  7, }, // America/Caracas
    {   10, 4247,    4972,  6, }, // America/Cayenne
    {   10, 5398,    4978,  6, }, // America/Cayman
    {   10, 3251,    4984,  6, }, // America/Chicago
    {   10, 7719,    4990,  7, }, // America/Chihuahua
    {   10, 7737,    4997, 13, }, // America/Ciudad_Juarez
    {   10, 5324,    5010, 10, }, // America/Costa_Rica
    {   10, 7759,    5020,  7, }, // America/Creston
    {   10, 3693,    5027,  6, }, // America/Cuiaba
    {   10, 5640,    5033,  7, }, // America/Curacao
    {   10, 7775,    5040, 13, }, // America/Danmarkshavn
    {   10, 7796,    5053,  6, }, // America/Dawson
    {   10, 7811,    5059, 16, }, // America/Dawson_Creek
    {   10, 5656,    5075,  8, }, // America/Dominica
    {   10, 7832,    5083,  7, }, // America/Eirunepe
    {   10, 5343,    5090,  8, }, // America/El_Salvador
    {   10, 7849,    5098, 11, }, // America/Fort_Nelson
    {   10, 7887,    5109, 14, }, // America/Glace_Bay
    {   10, 6844,    5123, 13, }, // America/Goose_Bay
    {   10, 4514,    5136, 12, }, // America/Grand_Turk
    {   10, 5687,    5148,  7, }, // America/Grenada
    {   10, 5703,    5155,  9, }, // America/Guadeloupe
    {   10, 3662,    5164,  9, }, // America/Guatemala
    {   10, 4743,    5173,  9, }, // America/Guayaquil
    {   10, 6895,    5182,  7, }, // America/Guyana
    {   10, 1923,    5189,  8, }, // America/Halifax
    {   10, 2221,    5197,  6, }, // America/Havana
    {   10, 7905,    5203,  9, }, // America/Hermosillo
    {   10,  481,    5212, 14, }, // America/Indiana/Knox
    {   10, 7924,      86, 16, }, // America/Indiana/Marengo
    {   10, 7948,    5226, 19, }, // America/Indiana/Petersburg
    {   10, 7975,    5245, 18, }, // America/Indiana/Tell_City
    {   10, 8001,    5263, 14, }, // America/Indiana/Vevay
    {   10, 8023,    5277, 17, }, // America/Indiana/Vincennes
    {   10, 8049,    5294, 16, }, // America/Indiana/Winamac
    {   10,  660,    5310,  7, }, // America/Iqaluit
    {   10, 2665,    5317,  8, }, // America/Jamaica
    {   10, 5282,    5325,  6, }, // America/Juneau
    {   10,  521,    5331,  8, }, // America/Kentucky/Louisville
    {   10, 8088,    5339, 18, }, // America/Kentucky/Monticello
    {   10, 7132,    5357,  4, }, // America/Lima
    {   10, 3390,    5361, 12, }, // America/Los_Angeles
    {   10, 5849,    5373, 21, }, // America/Lower_Princes
    {   10, 8116,    5394,  6, }, // America/Maceio
    {   10, 5722,    5400,  9, }, // America/Martinique
    {   10, 8165,    5409,  8, }, // America/Menominee
    {   10, 8183,    3899,  6, }, // America/Merida
    {   10, 2811,    5417, 17, }, // America/Mexico_City
    {   10, 4293,    5434,  7, }, // America/Miquelon
    {   10, 8217,    5441,  7, }, // America/Moncton
    {   10, 8233,    5448,  9, }, // America/Monterrey
    {   10, 5741,    5457,  9, }, // America/Montserrat
    {   10, 5383,    5466,  5, }, // America/Nassau
    {   10, 3294,    5471,  8, }, // America/New_York
    {   10, 1834,    5479,  7, }, // America/Noronha
    {   10, 8264,    5486, 22, }, // America/North_Dakota/Beulah
    {   10, 8292,    5508, 23, }, // America/North_Dakota/Center
    {   10, 8320,    5531, 26, }, // America/North_Dakota/New_Salem
    {   10,  393,    5557,  4, }, // America/Nuuk
    {   10, 8351,    5561,  7, }, // America/Ojinaga
    {   10, 3224,    5568,  6, }, // America/Phoenix
    {   10, 3847,    5574, 12, }, // America/Port-au-Prince
    {   10, 8367,    5586, 11, }, // America/Porto_Velho
    {   10, 5760,    5597, 10, }, // America/Puerto_Rico
    {   10, 3932,    5607, 12, }, // America/Punta_Arenas
    {   10, 8387,    5619, 11, }, // America/Rankin_Inlet
    {   10, 1979,    5630,  7, }, // America/Regina
    {   10,  695,    5637, 10, }, // America/Rio_Branco
    {   10, 6044,    5647, 13, }, // America/Santo_Domingo
    {   10, 1862,    5660,  9, }, // America/Sao_Paulo
    {   10, 6862,    5669, 12, }, // America/Scoresbysund
    {   10, 8471,    5681, 13, }, // America/St_Barthelemy
    {   10, 2045,    5694,  9, }, // America/St_Johns
    {   10, 5780,    5703,  9, }, // America/St_Kitts
    {   10, 5797,    5712, 10, }, // America/St_Lucia
    {   10,  855,    5722, 10, }, // America/St_Thomas
    {   10, 5830,    5732, 12, }, // America/St_Vincent
    {   10, 8493,    5744, 15, }, // America/Swift_Current
    {   10, 5363,    5759, 11, }, // America/Tegucigalpa
    {   10, 5673,    5770,  5, }, // America/Thule
    {   10,  313,    5775,  7, }, // America/Tijuana
    {   10, 5605,    5782,  7, }, // America/Tortola
    {   10, 2077,    5789,  8, }, // America/Vancouver
    {   10, 2128,    5797,  9, }, // America/Whitehorse
    {   10,  734,    5806,  7, }, // America/Winnipeg
    {   10, 8515,    5813,  7, }, // America/Yakutat
    {   10, 5918,    5820,  5, }, // Antarctica/Casey
    {   10, 6027,    5825,  6, }, // Antarctica/Davis
    {   10, 6066,    5831, 15, }, // Antarctica/DumontDUrville
    {   10, 8531,    5846,  7, }, // Antarctica/Macquarie
    {   10, 7029,    5853,  6, }, // Antarctica/Mawson
    {   10, 7076,    5859,  8, }, // Antarctica/McMurdo
    {   10, 7189,    5867,  7, }, // Antarctica/Rothera
    {   10, 7258,    5874,  5, }, // Antarctica/Syowa
    {   10, 1413,    5879, 11, }, // Arctic/Longyearbyen
    {   10, 5297,    5890,  6, }, // Asia/Almaty
    {   10, 3870,    5896,  4, }, // Asia/Amman
    {   10, 5428,    5900,  6, }, // Asia/Anadyr
    {   10, 5440,    5906,  5, }, // Asia/Aqtau
    {   10, 5270,    5911,  6, }, // Asia/Aqtobe
    {   10, 8587,    5917,  6, }, // Asia/Atyrau
    {   10, 3509,    4063,  6, }, // Asia/Baghdad
    {   10, 5451,    5923,  7, }, // Asia/Bahrain
    {   10, 3988,    5930,  6, }, // Asia/Beirut
    {   10, 4503,    5936,  4, }, // Asia/Chita
    {   10, 5963,    5940,  9, }, // Asia/Choibalsan
    {   10, 4413,    5949,  7, }, // Asia/Colombo
    {   10, 4442,    4102,  6, }, // Asia/Damascus
    {   10, 1060,    5956,  4, }, // Asia/Dhaka
    {   10, 8599,    5960,  9, }, // Asia/Famagusta
    {   10, 1226,    5969,  9, }, // Asia/Ho_Chi_Minh
    {   10, 2576,    5978,  9, }, // Asia/Hong_Kong
    {   10, 6969,    5987,  8, }, // Asia/Jakarta
    {   10, 6955,    5995,  9, }, // Asia/Jayapura
    {   10, 4218,    6004,  8, }, // Asia/Kamchatka
    {   10, 4141,    6012,  6, }, // Asia/Karachi
    {   10, 1152,    6018,  8, }, // Asia/Kathmandu
    {   10, 8624,    6026,  7, }, // Asia/Khandyga
    {   10,  992,    6033,  7, }, // Asia/Kolkata
    {   10, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {   10, 4761,    6040, 12, }, // Asia/Kuala_Lumpur
    {   10, 5893,    6052,  6, }, // Asia/Kuching
    {   10, 5464,    6058,  6, }, // Asia/Kuwait
    {   10, 1178,    6064,  5, }, // Asia/Macau
    {   10, 1316,    6069,  7, }, // Asia/Makassar
    {   10, 7145,    6076,  6, }, // Asia/Manila
    {   10, 6883,    4227,  6, }, // Asia/Muscat
    {   10, 2446,    6082,  7, }, // Asia/Nicosia
    {   10, 6924,    6089,  8, }, // Asia/Phnom_Penh
    {   10, 4116,    6097,  6, }, // Asia/Pyongyang
    {   10, 5476,    4233,  5, }, // Asia/Qatar
    {   10, 8671,    6103,  8, }, // Asia/Qostanay
    {   10, 4171,    6111,  9, }, // Asia/Qyzylorda
    {   10, 3486,    6120,  4, }, // Asia/Riyadh
    {   10, 4310,    6124,  7, }, // Asia/Sakhalin
    {   10, 3116,    6131,  4, }, // Asia/Seoul
    {   10, 1020,    6135,  7, }, // Asia/Shanghai
    {   10, 3137,    6142,  8, }, // Asia/Singapore
    {   10, 4199,    6150, 13, }, // Asia/Srednekolymsk
    {   10, 3100,    6163,  6, }, // Asia/Taipei
    {   10, 3817,    6169,  6, }, // Asia/Tbilisi
    {   10, 2638,    6175,  7, }, // Asia/Tehran
    {   10, 1284,    6182,  6, }, // Asia/Thimphu
    {   10, 2687,    4257,  5, }, // Asia/Tokyo
    {   10, 1346,    6188,  9, }, // Asia/Ulaanbaatar
    {   10, 6940,    6197,  9, }, // Asia/Vientiane
    {   10, 4730,    6206,  7, }, // Asia/Yakutsk
    {   10, 1202,    6213,  6, }, // Asia/Yangon
    {   10, 3785,    6219, 12, }, // Asia/Yekaterinburg
    {   10, 3649,    6231,  7, }, // Asia/Yerevan
    {   10, 3565,    6238,  5, }, // Atlantic/Azores
    {   10, 5588,    6243,  7, }, // Atlantic/Bermuda
    {   10, 6591,    4299,  6, }, // Atlantic/Canary
    {   10, 3629,    6250, 14, }, // Atlantic/Cape_Verde
    {   10, 1379,     448,  5, }, // Atlantic/Faroe
    {   10, 2614,    6264,  9, }, // Atlantic/Reykjavik
    {   10, 7235,    6273, 15, }, // Atlantic/South_Georgia
    {   10, 6784,    6288, 10, }, // Atlantic/St_Helena
    {   10, 6607,    6298,  6, }, // Atlantic/Stanley
    {   10, 1654,    6304,  8, }, // Australia/Adelaide
    {   10, 1619,    6312,  8, }, // Australia/Brisbane
    {   10, 1783,    6320, 11, }, // Australia/Broken_Hill
    {   10, 1567,    6331,  6, }, // Australia/Darwin
    {   10, 3539,    6337,  5, }, // Australia/Eucla
    {   10, 8716,    6342,  8, }, // Australia/Lindeman
    {   10, 1531,    6350,  9, }, // Australia/Lord_Howe
    {   10, 1711,    6359,  7, }, // Australia/Melbourne
    {   10, 1746,    6366,  5, }, // Australia/Perth
    {   10, 1447,    6371,  6, }, // Australia/Sydney
    {   10, 6149,    6377,  7, }, // Europe/Andorra
    {   10, 3522,    6384,  8, }, // Europe/Astrakhan
    {   10, 6577,    6392,  6, }, // Europe/Athens
    {   10, 6285,    6398,  7, }, // Europe/Belgrade
    {   10, 6437,    6405, 10, }, // Europe/Bratislava
    {   10, 6178,    6415,  7, }, // Europe/Brussels
    {   10, 3830,    6422,  9, }, // Europe/Bucharest
    {   10, 2475,    6431,  8, }, // Europe/Chisinau
    {   10, 6238,    4376, 10, }, // Europe/Copenhagen
    {   10, 6256,    6439, 10, }, // Europe/Gibraltar
    {   10, 8767,    6449,  7, }, // Europe/Guernsey
    {   10, 8783,    6456, 15, }, // Europe/Isle_of_Man
    {   10, 1097,    6471,  8, }, // Europe/Istanbul
    {   10, 8802,    6479,  7, }, // Europe/Jersey
    {   10, 2419,    6486,  4, }, // Europe/Kyiv
    {   10, 3078,    6490,  7, }, // Europe/Lisbon
    {   10, 6455,    6497,  7, }, // Europe/Ljubljana
    {   10, 2393,    6504,  6, }, // Europe/London
    {   10, 6314,    6510, 10, }, // Europe/Luxembourg
    {   10, 6346,    6520,  5, }, // Europe/Malta
    {   10, 6359,    6525,  6, }, // Europe/Monaco
    {   10, 3443,    6531,  5, }, // Europe/Moscow
    {   10, 6373,    6536,  9, }, // Europe/Podgorica
    {   10, 6224,    6545,  5, }, // Europe/Prague
    {   10, 8816,    6550,  4, }, // Europe/Riga
    {   10, 6273,    6554,  4, }, // Europe/Rome
    {   10, 6419,    6558, 10, }, // Europe/San_Marino
    {   10, 6194,    6568,  8, }, // Europe/Sarajevo
    {   10, 6332,    6576,  5, }, // Europe/Skopje
    {   10, 6548,    6581,  5, }, // Europe/Sofia
    {   10, 6472,    6586,  8, }, // Europe/Stockholm
    {   10, 8846,    6594,  5, }, // Europe/Tallinn
    {   10, 6135,    6599,  6, }, // Europe/Tirane
    {   10, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {   10, 6516,    6614,  7, }, // Europe/Vatican
    {   10, 6164,    6621,  5, }, // Europe/Vienna
    {   10, 3055,    6626,  8, }, // Europe/Warsaw
    {   10, 6489,    6634,  5, }, // Europe/Zurich
    {   10, 6910,    6639,  5, }, // Indian/Chagos
    {   10, 5979,    6644, 12, }, // Indian/Christmas
    {   10, 5996,    4484,  5, }, // Indian/Cocos
    {   10, 4931,    6656,  6, }, // Indian/Comoro
    {   10, 7013,    6662,  7, }, // Indian/Maldives
    {   10, 5000,    6669,  6, }, // Indian/Mayotte
    {   10, 7174,     467,  7, }, // Indian/Reunion
    {   10,  932,    6675,  6, }, // Pacific/Auckland
    {   10, 3608,    6681, 10, }, // Pacific/Bougainville
    {   10, 2863,    6691,  6, }, // Pacific/Chatham
    {   10, 3022,    6697,  3, }, // Pacific/Chuuk
    {   10, 2201,    6700,  6, }, // Pacific/Easter
    {   10, 3804,    6706,  5, }, // Pacific/Fiji
    {   10, 3724,    6711, 11, }, // Pacific/Guadalcanal
    {   10, 2929,    4512,  8, }, // Pacific/Honolulu
    {   10, 2897,    4520,  8, }, // Pacific/Kanton
    {   10, 3900,    6722,  9, }, // Pacific/Kiritimati
    {   10, 6982,    6731,  5, }, // Pacific/Kosrae
    {   10, 2708,    6736, 10, }, // Pacific/Kwajalein
    {   10, 4779,    6746,  6, }, // Pacific/Majuro
    {   10, 3953,    6752,  6, }, // Pacific/Marquesas
    {   10, 8909,    6758,  6, }, // Pacific/Midway
    {   10, 2991,    6764,  9, }, // Pacific/Pago_Pago
    {   10, 7157,    6773,  7, }, // Pacific/Pitcairn
    {   10, 2961,    6780,  7, }, // Pacific/Pohnpei
    {   10, 4709,    6787, 12, }, // Pacific/Port_Moresby
    {   10, 6009,    6799,  9, }, // Pacific/Rarotonga
    {   10, 6675,    6808,  6, }, // Pacific/Tarawa
    {   10, 7355,    6814,  4, }, // Pacific/Wake
    {   10, 7368,    6818,  6, }, // Pacific/Wallis
    {   13, 6751,    6824,  4, }, // Africa/Abidjan Amharic/Ethiopic/Ethiopia
    {   13, 6723,    6828,  3, }, // Africa/Accra
    {   13, 4961,    6831,  7, }, // Africa/Addis_Ababa
    {   13, 7383,    6838,  5, }, // Africa/Algiers
    {   13,   14,    6843,  4, }, // Africa/Asmara
    {   13,   44,    6847,  3, }, // Africa/Bamako
    {   13, 5159,    6850,  4, }, // Africa/Bangui
    {   13, 6709,    6854,  4, }, // Africa/Banjul
    {   13, 7398,    6858,  3, }, // Africa/Bissau
    {   13, 4859,    6861,  6, }, // Africa/Blantyre
    {   13, 5189,    6867,  5, }, // Africa/Brazzaville
    {   13, 4824,    6872,  5, }, // Africa/Bujumbura
    {   13, 2242,    6877,  3, }, // Africa/Cairo
    {   13, 4019,    6880,  6, }, // Africa/Casablanca
    {   13, 7412,    6886,  3, }, // Africa/Ceuta
    {   13, 6736,    6889,  4, }, // Africa/Conakry
    {   13, 6803,    6893,  3, }, // Africa/Dakar
    {   13, 5032,    6896,  6, }, // Africa/Dar_es_Salaam
    {   13, 4945,    6902,  3, }, // Africa/Djibouti
    {   13, 5145,    6905,  3, }, // Africa/Douala
    {   13, 5068,    6908,  7, }, // Africa/El_Aaiun
    {   13, 6816,    6915,  5, }, // Africa/Freetown
    {   13, 4808,    6920,  4, }, // Africa/Gaborone
    {   13, 4917,    6924,  3, }, // Africa/Harare
    {   13, 4381,    6927,  7, }, // Africa/Johannesburg
    {   13, 4401,    6934,  2, }, // Africa/Juba
    {   13, 5053,    6936,  4, }, // Africa/Kampala
    {   13, 4426,    6940,  4, }, // Africa/Khartoum
    {   13, 4875,    6944,  3, }, // Africa/Kigali
    {   13, 5208,    6947,  4, }, // Africa/Kinshasa
    {   13, 4646,    6951,  3, }, // Africa/Lagos
    {   13, 5238,    6954,  5, }, // Africa/Libreville
    {   13, 6832,    6959,  2, }, // Africa/Lome
    {   13, 5113,    6961,  4, }, // Africa/Luanda
    {   13, 4841,    6965,  5, }, // Africa/Lubumbashi
    {   13, 4903,    6970,  3, }, // Africa/Lusaka
    {   13, 5224,    6973,  3, }, // Africa/Malabo
    {   13, 4889,    6976,  3, }, // Africa/Maputo
    {   13, 5099,    6979,  3, }, // Africa/Maseru
    {   13, 5084,    6982,  4, }, // Africa/Mbabane
    {   13, 5015,    6986,  4, }, // Africa/Mogadishu
    {   13, 6997,    6990,  5, }, // Africa/Monrovia
    {   13, 3755,    6995,  4, }, // Africa/Nairobi
    {   13, 5173,    6999,  4, }, // Africa/Ndjamena
    {   13, 5256,    7003,  4, }, // Africa/Niamey
    {   13, 6766,    7007,  5, }, // Africa/Nouakchott
    {   13, 6690,    7012,  4, }, // Africa/Ouagadougou
    {   13, 5127,    7016,  8, }, // Africa/Porto-Novo
    {   13, 4337,    7024,  5, }, // Africa/Sao_Tome
    {   13, 2732,    7029,  4, }, // Africa/Tripoli
    {   13, 6503,    7033,  3, }, // Africa/Tunis
    {   13, 4054,    7036,  5, }, // Africa/Windhoek
    {   13,  132,    7041,  3, }, // America/Adak
    {   13, 3183,    7044,  5, }, // America/Anchorage
    {   13, 5524,    7049,  5, }, // America/Anguilla
    {   13, 5541,    7054,  4, }, // America/Antigua
    {   13, 4456,    7058,  5, }, // America/Araguaina
    {   13,  166,    7063,  8, }, // America/Argentina/Buenos_Aires
    {   13,   91,    7071,  5, }, // America/Argentina/Catamarca
    {   13,  270,    7076,  4, }, // America/Argentina/Cordoba
    {   13,  441,    7080,  3, }, // America/Argentina/Jujuy
    {   13, 7425,    7083,  5, }, // America/Argentina/La_Rioja
    {   13,  565,    7088,  4, }, // America/Argentina/Mendoza
    {   13, 7452,    7092,  7, }, // America/Argentina/Rio_Gallegos
    {   13, 7483,    7099,  3, }, // America/Argentina/Salta
    {   13, 7507,    7102,  6, }, // America/Argentina/San_Juan
    {   13, 5497,    7108,  6, }, // America/Argentina/San_Luis
    {   13, 7534,    7114,  4, }, // America/Argentina/Tucuman
    {   13, 7560,    7118,  4, }, // America/Argentina/Ushuaia
    {   13, 5557,    7122,  3, }, // America/Aruba
    {   13, 4154,    7125,  6, }, // America/Asuncion
    {   13,  237,    7131,  5, }, // America/Atikokan
    {   13, 3581,    7136,  3, }, // America/Bahia
    {   13, 7586,    7139,  9, }, // America/Bahia_Banderas
    {   13, 5571,    7148,  5, }, // America/Barbados
    {   13, 7609,    7153,  3, }, // America/Belem
    {   13, 5309,    7156,  3, }, // America/Belize
    {   13, 7623,    7159, 10, }, // America/Blanc-Sablon
    {   13, 7644,    7169,  6, }, // America/Boa_Vista
    {   13, 4263,    7175,  3, }, // America/Bogota
    {   13, 7662,    7178,  3, }, // America/Boise
    {   13, 7676,    7181,  8, }, // America/Cambridge_Bay
    {   13, 7698,    7189,  8, }, // America/Campo_Grande
    {   13, 3770,    7197,  4, }, // America/Cancun
    {   13, 4596,    7201,  4, }, // America/Caracas
    {   13, 4247,    7205,  4, }, // America/Cayenne
    {   13, 5398,    7209,  4, }, // America/Cayman
    {   13, 3251,    7213,  3, }, // America/Chicago
    {   13, 7719,    7216,  5, }, // America/Chihuahua
    {   13, 7737,    7221,  9, }, // America/Ciudad_Juarez
    {   13, 5324,    7230,  5, }, // America/Costa_Rica
    {   13, 7759,    7235,  5, }, // America/Creston
    {   13, 3693,    7240,  4, }, // America/Cuiaba
    {   13, 5640,    7244,  4, }, // America/Curacao
    {   13, 7775,    7248,  8, }, // America/Danmarkshavn
    {   13, 7796,    7256,  4, }, // America/Dawson
    {   13, 7811,    7260,  8, }, // America/Dawson_Creek
    {   13,  805,    7268,  4, }, // America/Denver
    {   13, 3351,    7272,  5, }, // America/Detroit
    {   13, 5656,    7277,  4, }, // America/Dominica
    {   13,  893,    7281,  6, }, // America/Edmonton
    {   13, 7832,    7287,  4, }, // America/Eirunepe
    {   13, 5343,    7291,  7, }, // America/El_Salvador
    {   13, 7849,    7298,  8, }, // America/Fort_Nelson
    {   13, 7869,    7306,  5, }, // America/Fortaleza
    {   13, 7887,    7311,  6, }, // America/Glace_Bay
    {   13, 6844,    7317,  5, }, // America/Goose_Bay
    {   13, 4514,    7322,  8, }, // America/Grand_Turk
    {   13, 5687,    7330,  4, }, // America/Grenada
    {   13, 5703,    7334,  4, }, // America/Guadeloupe
    {   13, 3662,    7338,  4, }, // America/Guatemala
    {   13, 4743,    7342,  5, }, // America/Guayaquil
    {   13, 6895,    7347,  3, }, // America/Guyana
    {   13, 1923,    7350,  5, }, // America/Halifax
    {   13, 2221,    7355,  3, }, // America/Havana
    {   13, 7905,    7358,  5, }, // America/Hermosillo
    {   13,  348,    7363,  8, }, // America/Indiana/Indianapolis
    {   13,  481,    7371, 10, }, // America/Indiana/Knox
    {   13, 7924,    7381, 11, }, // America/Indiana/Marengo
    {   13, 7948,    7392, 14, }, // America/Indiana/Petersburg
    {   13, 7975,    7406, 13, }, // America/Indiana/Tell_City
    {   13, 8001,    7419, 10, }, // America/Indiana/Vevay
    {   13, 8023,    7429, 12, }, // America/Indiana/Vincennes
    {   13, 8049,    7441, 11, }, // America/Indiana/Winamac
    {   13, 8073,    7452,  4, }, // America/Inuvik
    {   13,  660,    7456,  5, }, // America/Iqaluit
    {   13, 2665,    7461,  4, }, // America/Jamaica
    {   13, 5282,    7465,  3, }, // America/Juneau
    {   13,  521,    7468,  5, }, // America/Kentucky/Louisville
    {   13, 8088,    7473, 11, }, // America/Kentucky/Monticello
    {   13, 5621,    7484,  7, }, // America/Kralendijk
    {   13, 4278,    7491,  4, }, // America/La_Paz
    {   13, 7132,    7495,  2, }, // America/Lima
    {   13, 3390,    7497,  8, }, // America/Los_Angeles
    {   13, 5849,    7505, 12, }, // America/Lower_Princes
    {   13, 8116,    7517,  3, }, // America/Maceio
    {   13, 8131,    7520,  4, }, // America/Managua
    {   13, 1892,    7524,  4, }, // America/Manaus
    {   13, 5814,    7528,  4, }, // America/Marigot
    {   13, 5722,    7532,  5, }, // America/Martinique
    {   13, 8147,    7537,  5, }, // America/Matamoros
    {   13, 2779,    7542,  5, }, // America/Mazatlan
    {   13, 8165,    7547,  4, }, // America/Menominee
    {   13, 8183,    7551,  3, }, // America/Merida
    {   13, 8198,    7554,  6, }, // America/Metlakatla
    {   13, 2811,    7560,  8, }, // America/Mexico_City
    {   13, 4293,    7568,  5, }, // America/Miquelon
    {   13, 8217,    7573,  5, }, // America/Moncton
    {   13, 8233,    7578,  6, }, // America/Monterrey
    {   13, 4000,    7584,  6, }, // America/Montevideo
    {   13, 5741,    7590,  6, }, // America/Montserrat
    {   13, 5383,    7596,  3, }, // America/Nassau
    {   13, 3294,    7599,  5, }, // America/New_York
    {   13, 8251,    7604,  2, }, // America/Nome
    {   13, 1834,    7606,  3, }, // America/Noronha
    {   13, 8264,    7609, 13, }, // America/North_Dakota/Beulah
    {   13, 8292,    7622, 14, }, // America/North_Dakota/Center
    {   13, 8320,    7636, 16, }, // America/North_Dakota/New_Salem
    {   13,  393,    7652,  4, }, // America/Nuuk
    {   13, 8351,    7656,  4, }, // America/Ojinaga
    {   13, 5413,    7660,  3, }, // America/Panama
    {   13, 6106,    7663,  5, }, // America/Paramaribo
    {   13, 3224,    7668,  4, }, // America/Phoenix
    {   13, 3847,    7672, 10, }, // America/Port-au-Prince
    {   13, 5871,    7682,  9, }, // America/Port_of_Spain
    {   13, 8367,    7691,  7, }, // America/Porto_Velho
    {   13, 5760,    7698,  5, }, // America/Puerto_Rico
    {   13, 3932,    7703,  8, }, // America/Punta_Arenas
    {   13, 8387,    7711,  9, }, // America/Rankin_Inlet
    {   13, 8408,    7720,  3, }, // America/Recife
    {   13, 1979,    7723,  3, }, // America/Regina
    {   13, 8423,    7726,  5, }, // America/Resolute
    {   13,  695,    7731,  7, }, // America/Rio_Branco
    {   13, 8440,    7738,  5, }, // America/Santarem
    {   13, 2165,    7743,  5, }, // America/Santiago
    {   13, 6044,    7748,  8, }, // America/Santo_Domingo
    {   13, 1862,    7756,  5, }, // America/Sao_Paulo
    {   13, 6862,    7761,  9, }, // America/Scoresbysund
    {   13, 8457,    7770,  3, }, // America/Sitka
    {   13, 8471,    7773, 10, }, // America/St_Barthelemy
    {   13, 2045,    7783,  8, }, // America/St_Johns
    {   13, 5780,    7791,  7, }, // America/St_Kitts
    {   13, 5797,    7798,  8, }, // America/St_Lucia
    {   13,  855,    7806,  7, }, // America/St_Thomas
    {   13, 5830,    7813,  9, }, // America/St_Vincent
    {   13, 8493,    7822,  9, }, // America/Swift_Current
    {   13, 5363,    7831,  6, }, // America/Tegucigalpa
    {   13, 5673,    7837,  2, }, // America/Thule
    {   13,  313,    7839,  4, }, // America/Tijuana
    {   13,  608,    7843,  4, }, // America/Toronto
    {   13, 5605,    7847,  4, }, // America/Tortola
    {   13, 2077,    7851,  5, }, // America/Vancouver
    {   13, 2128,    7856,  6, }, // America/Whitehorse
    {   13,  734,    7862,  4, }, // America/Winnipeg
    {   13, 8515,    7866,  4, }, // America/Yakutat
    {   13, 5918,    7870,  3, }, // Antarctica/Casey
    {   13, 6027,    7873,  3, }, // Antarctica/Davis
    {   13, 6066,    7876, 10, }, // Antarctica/DumontDUrville
    {   13, 8531,    7886,  3, }, // Antarctica/Macquarie
    {   13, 7029,    7889,  4, }, // Antarctica/Mawson
    {   13, 7076,    7893,  5, }, // Antarctica/McMurdo
    {   13, 8552,    7898,  4, }, // Antarctica/Palmer
    {   13, 7189,    7902,  3, }, // Antarctica/Rothera
    {   13, 7258,    7905,  3, }, // Antarctica/Syowa
    {   13, 8570,    7908,  3, }, // Antarctica/Troll
    {   13, 7337,    7911,  4, }, // Antarctica/Vostok
    {   13, 1413,    7915,  8, }, // Arctic/Longyearbyen
    {   13, 5487,    7923,  3, }, // Asia/Aden
    {   13, 5297,    7926,  5, }, // Asia/Almaty
    {   13, 3870,    7931,  3, }, // Asia/Amman
    {   13, 5428,    7934,  5, }, // Asia/Anadyr
    {   13, 5440,    7939,  4, }, // Asia/Aqtau
    {   13, 5270,    7943,  4, }, // Asia/Aqtobe
    {   13,  964,    7947,  5, }, // Asia/Ashgabat
    {   13, 8587,    7952,  5, }, // Asia/Atyrau
    {   13, 3509,    7957,  4, }, // Asia/Baghdad
    {   13, 5451,    7961,  4, }, // Asia/Bahrain
    {   13, 3555,    7243,  2, }, // Asia/Baku
    {   13, 4368,    7965,  4, }, // Asia/Bangkok
    {   13, 3473,    7969,  5, }, // Asia/Barnaul
    {   13, 3988,    7974,  3, }, // Asia/Beirut
    {   13, 3680,    7977,  4, }, // Asia/Bishkek
    {   13, 5906,    7981,  4, }, // Asia/Brunei
    {   13, 4503,    7985,  2, }, // Asia/Chita
    {   13, 5963,    7987,  6, }, // Asia/Choibalsan
    {   13, 4413,    7993,  4, }, // Asia/Colombo
    {   13, 4442,    7997,  4, }, // Asia/Damascus
    {   13, 1060,    6893,  2, }, // Asia/Dhaka
    {   13, 6125,    8001,  2, }, // Asia/Dili
    {   13, 3498,    8003,  3, }, // Asia/Dubai
    {   13, 6092,    8006,  4, }, // Asia/Dushanbe
    {   13, 8599,    8010,  5, }, // Asia/Famagusta
    {   13, 8614,    8015,  2, }, // Asia/Gaza
    {   13, 4697,    8017,  4, }, // Asia/Hebron
    {   13, 1226,    8021, 11, }, // Asia/Ho_Chi_Minh
    {   13, 2576,    8032,  7, }, // Asia/Hong_Kong
    {   13, 4673,    8039,  3, }, // Asia/Hovd
    {   13, 4086,    8042,  6, }, // Asia/Irkutsk
    {   13, 6969,    8048,  4, }, // Asia/Jakarta
    {   13, 6955,    8052,  4, }, // Asia/Jayapura
    {   13, 1257,    8056,  6, }, // Asia/Jerusalem
    {   13, 3462,    8062,  3, }, // Asia/Kabul
    {   13, 4218,    8065,  5, }, // Asia/Kamchatka
    {   13, 4141,    8070,  3, }, // Asia/Karachi
    {   13, 1152,    8073,  5, }, // Asia/Kathmandu
    {   13, 8624,    8078,  4, }, // Asia/Khandyga
    {   13,  992,    8082,  4, }, // Asia/Kolkata
    {   13, 4099,    8086,  8, }, // Asia/Krasnoyarsk
    {   13, 4761,    8094,  8, }, // Asia/Kuala_Lumpur
    {   13, 5893,    8102,  4, }, // Asia/Kuching
    {   13, 5464,    8106,  3, }, // Asia/Kuwait
    {   13, 1178,    8109,  3, }, // Asia/Macau
    {   13, 3919,    8112,  4, }, // Asia/Magadan
    {   13, 1316,    8116,  4, }, // Asia/Makassar
    {   13, 7145,    8120,  3, }, // Asia/Manila
    {   13, 6883,    8123,  4, }, // Asia/Muscat
    {   13, 2446,    8127,  4, }, // Asia/Nicosia
    {   13, 8638,    8131,  7, }, // Asia/Novokuznetsk
    {   13, 4037,    8138,  7, }, // Asia/Novosibirsk
    {   13, 4131,    8145,  4, }, // Asia/Omsk
    {   13, 7108,    8149,  3, }, // Asia/Oral
    {   13, 6924,    8152,  7, }, // Asia/Phnom_Penh
    {   13, 8656,    8159,  6, }, // Asia/Pontianak
    {   13, 4116,    8165,  7, }, // Asia/Pyongyang
    {   13, 5476,    8172,  3, }, // Asia/Qatar
    {   13, 8671,    8175,  5, }, // Asia/Qostanay
    {   13, 4171,    8180,  6, }, // Asia/Qyzylorda
    {   13, 3486,    8186,  3, }, // Asia/Riyadh
    {   13, 4310,    8189,  5, }, // Asia/Sakhalin
    {   13, 7208,    8194,  6, }, // Asia/Samarkand
    {   13, 3116,    8200,  3, }, // Asia/Seoul
    {   13, 1020,    8203,  4, }, // Asia/Shanghai
    {   13, 3137,    8207,  5, }, // Asia/Singapore
    {   13, 4199,    8212, 10, }, // Asia/Srednekolymsk
    {   13, 3100,    8222,  4, }, // Asia/Taipei
    {   13, 4683,    8226,  5, }, // Asia/Tashkent
    {   13, 3817,    8231,  4, }, // Asia/Tbilisi
    {   13, 2638,    8235,  4, }, // Asia/Tehran
    {   13, 1284,    8239,  3, }, // Asia/Thimphu
    {   13, 2687,    8242,  3, }, // Asia/Tokyo
    {   13, 4474,    8245,  4, }, // Asia/Tomsk
    {   13, 1346,    8249,  8, }, // Asia/Ulaanbaatar
    {   13, 1126,    8257,  4, }, // Asia/Urumqi
    {   13, 8685,    8261,  6, }, // Asia/Ust-Nera
    {   13, 6940,    8267,  5, }, // Asia/Vientiane
    {   13, 4612,    8272,  7, }, // Asia/Vladivostok
    {   13, 4730,    8279,  5, }, // Asia/Yakutsk
    {   13, 1202,    8284,  4, }, // Asia/Yangon
    {   13, 3785,    8288,  8, }, // Asia/Yekaterinburg
    {   13, 3649,    8296,  4, }, // Asia/Yerevan
    {   13, 3565,    8300,  4, }, // Atlantic/Azores
    {   13, 5588,    8304,  4, }, // Atlantic/Bermuda
    {   13, 6591,    8308,  3, }, // Atlantic/Canary
    {   13, 3629,    8311,  6, }, // Atlantic/Cape_Verde
    {   13, 1379,    8317,  3, }, // Atlantic/Faroe
    {   13, 8699,    8320,  3, }, // Atlantic/Madeira
    {   13, 2614,    8323,  6, }, // Atlantic/Reykjavik
    {   13, 7235,    8329,  8, }, // Atlantic/South_Georgia
    {   13, 6784,    8337,  8, }, // Atlantic/St_Helena
    {   13, 6607,    8345,  5, }, // Atlantic/Stanley
    {   13, 1654,    8350,  5, }, // Australia/Adelaide
    {   13, 1619,    8355,  5, }, // Australia/Brisbane
    {   13, 1783,    8360,  7, }, // Australia/Broken_Hill
    {   13, 1567,    8367,  4, }, // Australia/Darwin
    {   13, 3539,    8371,  3, }, // Australia/Eucla
    {   13, 1500,    8374,  4, }, // Australia/Hobart
    {   13, 8716,    8378,  5, }, // Australia/Lindeman
    {   13, 1531,    8383,  6, }, // Australia/Lord_Howe
    {   13, 1711,    8389,  5, }, // Australia/Melbourne
    {   13, 1746,    8394,  3, }, // Australia/Perth
    {   13, 1447,    8397,  3, }, // Australia/Sydney
    {   13, 6390,    8400,  7, }, // Europe/Amsterdam
    {   13, 6149,    8407,  4, }, // Europe/Andorra
    {   13, 3522,    8411,  7, }, // Europe/Astrakhan
    {   13, 6577,    8418,  4, }, // Europe/Athens
    {   13, 6285,    8422,  5, }, // Europe/Belgrade
    {   13, 4659,    8427,  4, }, // Europe/Berlin
    {   13, 6437,    8431,  6, }, // Europe/Bratislava
    {   13, 6178,    8437,  5, }, // Europe/Brussels
    {   13, 3830,    8442,  5, }, // Europe/Bucharest
    {   13, 3708,    8447,  5, }, // Europe/Budapest
    {   13, 8751,    8452,  5, }, // Europe/Busingen
    {   13, 2475,    8457,  4, }, // Europe/Chisinau
    {   13, 6238,    8461,  6, }, // Europe/Copenhagen
    {   13, 2260,    8467,  4, }, // Europe/Dublin
    {   13, 6256,    8471,  6, }, // Europe/Gibraltar
    {   13, 8767,    8477,  4, }, // Europe/Guernsey
    {   13, 6561,    8481,  5, }, // Europe/Helsinki
    {   13, 8783,    8486, 10, }, // Europe/Isle_of_Man
    {   13, 1097,    8496,  6, }, // Europe/Istanbul
    {   13, 8802,    8502,  3, }, // Europe/Jersey
    {   13, 3881,    8505,  7, }, // Europe/Kaliningrad
    {   13, 8924,    8512,  3, }, // Europe/Kirov
    {   13, 2419,    8515,  3, }, // Europe/Kyiv
    {   13, 3078,    8518,  4, }, // Europe/Lisbon
    {   13, 6455,    8522,  6, }, // Europe/Ljubljana
    {   13, 2393,    8528,  4, }, // Europe/London
    {   13, 6314,    8532,  7, }, // Europe/Luxembourg
    {   13, 4794,    8539,  4, }, // Europe/Madrid
    {   13, 6346,    8543,  3, }, // Europe/Malta
    {   13, 6531,    8546,  5, }, // Europe/Mariehamn
    {   13, 3595,    8551,  4, }, // Europe/Minsk
    {   13, 6359,    8555,  3, }, // Europe/Monaco
    {   13, 3443,    8558,  3, }, // Europe/Moscow
    {   13, 6407,    8561,  3, }, // Europe/Oslo
    {   13, 4186,    8564,  3, }, // Europe/Paris
    {   13, 6373,    8567,  5, }, // Europe/Podgorica
    {   13, 6224,    8572,  3, }, // Europe/Prague
    {   13, 8816,    8575,  2, }, // Europe/Riga
    {   13, 6273,    8577,  2, }, // Europe/Rome
    {   13, 4233,    8579,  3, }, // Europe/Samara
    {   13, 6419,    8582,  6, }, // Europe/San_Marino
    {   13, 6194,    8588,  4, }, // Europe/Sarajevo
    {   13, 4353,    8592,  4, }, // Europe/Saratov
    {   13, 8828,    8596,  6, }, // Europe/Simferopol
    {   13, 6332,    8602,  4, }, // Europe/Skopje
    {   13, 6548,    8606,  3, }, // Europe/Sofia
    {   13, 6472,    8609,  6, }, // Europe/Stockholm
    {   13, 8846,    8615,  3, }, // Europe/Tallinn
    {   13, 6135,    8618,  3, }, // Europe/Tirane
    {   13, 8861,    8621,  7, }, // Europe/Ulyanovsk
    {   13, 6301,    8628,  3, }, // Europe/Vaduz
    {   13, 6516,    8631,  4, }, // Europe/Vatican
    {   13, 6164,    8635,  3, }, // Europe/Vienna
    {   13, 8878,    8638,  4, }, // Europe/Vilnius
    {   13, 4629,    8642,  5, }, // Europe/Volgograd
    {   13, 3055,    8647,  4, }, // Europe/Warsaw
    {   13, 6210,    8651,  4, }, // Europe/Zagreb
    {   13, 6489,    8655,  3, }, // Europe/Zurich
    {   13, 4980,    8658,  7, }, // Indian/Antananarivo
    {   13, 6910,    8665,  3, }, // Indian/Chagos
    {   13, 5979,    8668,  2, }, // Indian/Christmas
    {   13, 5996,    8670,  3, }, // Indian/Cocos
    {   13, 4931,    8673,  3, }, // Indian/Comoro
    {   13, 6624,    8676,  5, }, // Indian/Kerguelen
    {   13, 7223,    8681,  2, }, // Indian/Mahe
    {   13, 7013,    8683,  4, }, // Indian/Maldives
    {   13, 3971,    8687,  5, }, // Indian/Mauritius
    {   13, 5000,    8692,  3, }, // Indian/Mayotte
    {   13, 7174,    8695,  5, }, // Indian/Reunion
    {   13, 4324,    8700,  3, }, // Pacific/Apia
    {   13,  932,    8703,  5, }, // Pacific/Auckland
    {   13, 3608,    8708,  6, }, // Pacific/Bougainville
    {   13, 2863,    8714,  3, }, // Pacific/Chatham
    {   13, 3022,    8717,  2, }, // Pacific/Chuuk
    {   13, 2201,    8719,  3, }, // Pacific/Easter
    {   13, 7323,    8722,  3, }, // Pacific/Efate
    {   13, 7290,    8725,  4, }, // Pacific/Fakaofo
    {   13, 3804,    8729,  2, }, // Pacific/Fiji
    {   13, 7306,    8731,  4, }, // Pacific/Funafuti
    {   13, 6641,    8735,  5, }, // Pacific/Galapagos
    {   13, 6659,    8740,  5, }, // Pacific/Gambier
    {   13, 3724,    8745,  7, }, // Pacific/Guadalcanal
    {   13, 5935,    8752,  3, }, // Pacific/Guam
    {   13, 2929,    8755,  4, }, // Pacific/Honolulu
    {   13, 2897,    8759,  6, }, // Pacific/Kanton
    {   13, 3900,    8765,  5, }, // Pacific/Kiritimati
    {   13, 6982,    8770,  3, }, // Pacific/Kosrae
    {   13, 2708,    8773,  5, }, // Pacific/Kwajalein
    {   13, 4779,    8778,  3, }, // Pacific/Majuro
    {   13, 3953,    8781,  6, }, // Pacific/Marquesas
    {   13, 8909,    8787,  4, }, // Pacific/Midway
    {   13, 7047,    8791,  3, }, // Pacific/Nauru
    {   13, 7095,    8794,  2, }, // Pacific/Niue
    {   13, 4070,    8796,  5, }, // Pacific/Norfolk
    {   13, 7061,    8801,  4, }, // Pacific/Noumea
    {   13, 2991,    8805,  5, }, // Pacific/Pago_Pago
    {   13, 7118,    8810,  3, }, // Pacific/Palau
    {   13, 7157,    8813,  6, }, // Pacific/Pitcairn
    {   13, 2961,    8819,  4, }, // Pacific/Pohnpei
    {   13, 4709,    8823,  9, }, // Pacific/Port_Moresby
    {   13, 6009,    8832,  5, }, // Pacific/Rarotonga
    {   13, 5948,    8837,  4, }, // Pacific/Saipan
    {   13, 7275,    8841,  3, }, // Pacific/Tahiti
    {   13, 6675,    8844,  3, }, // Pacific/Tarawa
    {   13, 4485,    8847,  5, }, // Pacific/Tongatapu
    {   13, 7355,    8852,  2, }, // Pacific/Wake
    {   13, 7368,    8854,  3, }, // Pacific/Wallis
    {   14, 6751,    8857,  7, }, // Africa/Abidjan Arabic/Arabic/Egypt
    {   14, 6723,    8864,  4, }, // Africa/Accra
    {   14, 4961,    8868, 10, }, // Africa/Addis_Ababa
    {   14, 7383,    8878,  7, }, // Africa/Algiers
    {   14,   14,    8885,  5, }, // Africa/Asmara
    {   14,   44,    8890,  6, }, // Africa/Bamako
    {   14, 5159,    8896,  6, }, // Africa/Bangui
    {   14, 6709,    8902,  6, }, // Africa/Banjul
    {   14, 7398,    8908,  5, }, // Africa/Bissau
    {   14, 4859,    8913,  8, }, // Africa/Blantyre
    {   14, 5189,    8921,  8, }, // Africa/Brazzaville
    {   14, 4824,    8929,  9, }, // Africa/Bujumbura
    {   14, 2242,    8938,  7, }, // Africa/Cairo
    {   14, 4019,    8945, 13, }, // Africa/Casablanca
    {   14, 7412,    8958,  4, }, // Africa/Ceuta
    {   14, 6736,    8962,  7, }, // Africa/Conakry
    {   14, 6803,    8969,  5, }, // Africa/Dakar
    {   14, 5032,    8974, 10, }, // Africa/Dar_es_Salaam
    {   14, 4945,    8984,  6, }, // Africa/Djibouti
    {   14, 5145,    8990,  5, }, // Africa/Douala
    {   14, 5068,    8995,  6, }, // Africa/El_Aaiun
    {   14, 6816,    9001,  8, }, // Africa/Freetown
    {   14, 4808,    9009,  7, }, // Africa/Gaborone
    {   14, 4917,    9016,  5, }, // Africa/Harare
    {   14, 4381,    9021,  9, }, // Africa/Johannesburg
    {   14, 4401,    9030,  4, }, // Africa/Juba
    {   14, 5053,    9034,  7, }, // Africa/Kampala
    {   14, 4426,    9041,  7, }, // Africa/Khartoum
    {   14, 4875,    9048,  6, }, // Africa/Kigali
    {   14, 5208,    9054,  7, }, // Africa/Kinshasa
    {   14, 4646,    9061,  5, }, // Africa/Lagos
    {   14, 5238,    9066,  7, }, // Africa/Libreville
    {   14, 6832,    9073,  4, }, // Africa/Lome
    {   14, 5113,    9077,  6, }, // Africa/Luanda
    {   14, 4841,    9083,  8, }, // Africa/Lubumbashi
    {   14, 4903,    9091,  6, }, // Africa/Lusaka
    {   14, 5224,    9097,  6, }, // Africa/Malabo
    {   14, 4889,    9103,  6, }, // Africa/Maputo
    {   14, 5099,    9109,  6, }, // Africa/Maseru
    {   14, 5084,    9115,  7, }, // Africa/Mbabane
    {   14, 5015,    9122,  7, }, // Africa/Mogadishu
    {   14, 6997,    9129,  8, }, // Africa/Monrovia
    {   14, 3755,    9137,  6, }, // Africa/Nairobi
    {   14, 5173,    9143,  7, }, // Africa/Ndjamena
    {   14, 5256,    9150,  5, }, // Africa/Niamey
    {   14, 6766,    9155,  7, }, // Africa/Nouakchott
    {   14, 6690,    9162,  8, }, // Africa/Ouagadougou
    {   14, 5127,    9170, 10, }, // Africa/Porto-Novo
    {   14, 4337,    9180,  8, }, // Africa/Sao_Tome
    {   14, 2732,    9188,  6, }, // Africa/Tripoli
    {   14, 6503,    9194,  4, }, // Africa/Tunis
    {   14, 4054,    9198,  7, }, // Africa/Windhoek
    {   14,  132,    9205,  4, }, // America/Adak
    {   14, 3183,    9209,  7, }, // America/Anchorage
    {   14, 5524,    9216,  7, }, // America/Anguilla
    {   14, 5541,    9223,  7, }, // America/Antigua
    {   14, 4456,    9230,  9, }, // America/Araguaina
    {   14,  166,    9239, 11, }, // America/Argentina/Buenos_Aires
    {   14,   91,    9250,  9, }, // America/Argentina/Catamarca
    {   14,  270,    9259,  7, }, // America/Argentina/Cordoba
    {   14,  441,    9266,  4, }, // America/Argentina/Jujuy
    {   14, 7425,    9270,  8, }, // America/Argentina/La_Rioja
    {   14,  565,    9278,  7, }, // America/Argentina/Mendoza
    {   14, 7452,    9285, 11, }, // America/Argentina/Rio_Gallegos
    {   14, 7483,    9296,  5, }, // America/Argentina/Salta
    {   14, 7507,    9301,  8, }, // America/Argentina/San_Juan
    {   14, 5497,    9309,  8, }, // America/Argentina/San_Luis
    {   14, 7534,    9317,  6, }, // America/Argentina/Tucuman
    {   14, 7560,    9323,  4, }, // America/Argentina/Ushuaia
    {   14, 5557,    9327,  5, }, // America/Aruba
    {   14, 4154,    9332,  8, }, // America/Asuncion
    {   14,  237,    9340, 11, }, // America/Atikokan
    {   14, 3581,    9351,  5, }, // America/Bahia
    {   14, 7586,    9356, 14, }, // America/Bahia_Banderas
    {   14, 5571,    9370,  7, }, // America/Barbados
    {   14, 7609,    9377,  3, }, // America/Belem
    {   14, 5309,    9380,  4, }, // America/Belize
    {   14, 7623,    9384, 12, }, // America/Blanc-Sablon
    {   14, 7644,    9396,  9, }, // America/Boa_Vista
    {   14, 4263,    9405,  6, }, // America/Bogota
    {   14, 7662,    9411,  4, }, // America/Boise
    {   14, 7676,    9415, 12, }, // America/Cambridge_Bay
    {   14, 7698,    9427, 11, }, // America/Campo_Grande
    {   14, 3770,    9438,  6, }, // America/Cancun
    {   14, 4596,    9444,  7, }, // America/Caracas
    {   14, 4247,    9451,  5, }, // America/Cayenne
    {   14, 5398,    9456,  6, }, // America/Cayman
    {   14, 3251,    9462,  6, }, // America/Chicago
    {   14, 7719,    9468,  7, }, // America/Chihuahua
    {   14, 7737,    9475, 13, }, // America/Ciudad_Juarez
    {   14, 5324,    9488,  9, }, // America/Costa_Rica
    {   14, 7759,    9497,  7, }, // America/Creston
    {   14, 3693,    9504,  5, }, // America/Cuiaba
    {   14, 5640,    9509,  7, }, // America/Curacao
    {   14, 7775,    9516, 11, }, // America/Danmarkshavn
    {   14, 7796,    9527,  6, }, // America/Dawson
    {   14, 7811,    9533, 10, }, // America/Dawson_Creek
    {   14,  805,    9543,  4, }, // America/Denver
    {   14, 3351,    9547,  7, }, // America/Detroit
    {   14, 5656,    9554,  8, }, // America/Dominica
    {   14,  893,    9562,  9, }, // America/Edmonton
    {   14, 7832,    9571,  7, }, // America/Eirunepe
    {   14, 5343,    9578,  9, }, // America/El_Salvador
    {   14, 7849,    9587, 11, }, // America/Fort_Nelson
    {   14, 7869,    9598,  9, }, // America/Fortaleza
    {   14, 7887,    9607,  8, }, // America/Glace_Bay
    {   14, 6844,    9615,  7, }, // America/Goose_Bay
    {   14, 4514,    9622,  9, }, // America/Grand_Turk
    {   14, 5687,    9631,  7, }, // America/Grenada
    {   14, 5703,    9638,  7, }, // America/Guadeloupe
    {   14, 3662,    9645,  9, }, // America/Guatemala
    {   14, 4743,    9654,  9, }, // America/Guayaquil
    {   14, 6895,    9663,  5, }, // America/Guyana
    {   14, 1923,    9668,  8, }, // America/Halifax
    {   14, 2221,    9676,  6, }, // America/Havana
    {   14, 7905,    9682,  9, }, // America/Hermosillo
    {   14,  348,    9691, 12, }, // America/Indiana/Indianapolis
    {   14,  481,    9703,  5, }, // America/Indiana/Knox
    {   14, 7924,    9708,  6, }, // America/Indiana/Marengo
    {   14, 7948,    9714,  8, }, // America/Indiana/Petersburg
    {   14, 7975,    9722, 17, }, // America/Indiana/Tell_City
    {   14, 8001,    9739,  5, }, // America/Indiana/Vevay
    {   14, 8023,    9744,  7, }, // America/Indiana/Vincennes
    {   14, 8049,    9751,  7, }, // America/Indiana/Winamac
    {   14, 8073,    9758,  7, }, // America/Inuvik
    {   14,  660,    9765,  6, }, // America/Iqaluit
    {   14, 2665,    9771,  7, }, // America/Jamaica
    {   14, 5282,    9778,  4, }, // America/Juneau
    {   14,  521,    9782,  8, }, // America/Kentucky/Louisville
    {   14, 8088,    9790,  9, }, // America/Kentucky/Monticello
    {   14, 5621,    9799,  8, }, // America/Kralendijk
    {   14, 4278,    9807,  6, }, // America/La_Paz
    {   14, 7132,    9813,  4, }, // America/Lima
    {   14, 3390,    9817, 10, }, // America/Los_Angeles
    {   14, 5849,    9827, 16, }, // America/Lower_Princes
    {   14, 8116,    9843,  5, }, // America/Maceio
    {   14, 8131,    9848,  7, }, // America/Managua
    {   14, 1892,    9855,  6, }, // America/Manaus
    {   14, 5814,    9861,  7, }, // America/Marigot
    {   14, 5722,    9868, 10, }, // America/Martinique
    {   14, 8147,    9878,  9, }, // America/Matamoros
    {   14, 2779,    9887,  8, }, // America/Mazatlan
    {   14, 8165,    9895,  8, }, // America/Menominee
    {   14, 8183,    9903,  6, }, // America/Merida
    {   14, 8198,    9909, 10, }, // America/Metlakatla
    {   14, 2811,    9919, 11, }, // America/Mexico_City
    {   14, 4293,    9930,  7, }, // America/Miquelon
    {   14, 8217,    9937,  7, }, // America/Moncton
    {   14, 8233,    9944,  7, }, // America/Monterrey
    {   14, 4000,    9951,  9, }, // America/Montevideo
    {   14, 5741,    9960,  9, }, // America/Montserrat
    {   14, 5383,    9969,  4, }, // America/Nassau
    {   14, 3294,    9973,  7, }, // America/New_York
    {   14, 8251,    9897,  3, }, // America/Nome
    {   14, 1834,    9980,  7, }, // America/Noronha
    {   14, 8264,    9987, 22, }, // America/North_Dakota/Beulah
    {   14, 8292,   10009,  4, }, // America/North_Dakota/Center
    {   14, 8320,   10013,  9, }, // America/North_Dakota/New_Salem
    {   14,  393,   10022,  6, }, // America/Nuuk
    {   14, 8351,   10028,  8, }, // America/Ojinaga
    {   14, 5413,   10036,  4, }, // America/Panama
    {   14, 6106,   10040, 10, }, // America/Paramaribo
    {   14, 3224,   10050,  5, }, // America/Phoenix
    {   14, 3847,   10055, 12, }, // America/Port-au-Prince
    {   14, 5871,   10067, 13, }, // America/Port_of_Spain
    {   14, 8367,   10080, 10, }, // America/Porto_Velho
    {   14, 5760,   10090,  9, }, // America/Puerto_Rico
    {   14, 3932,   10099, 12, }, // America/Punta_Arenas
    {   14, 8387,   10111, 10, }, // America/Rankin_Inlet
    {   14, 8408,   10121,  5, }, // America/Recife
    {   14, 1979,   10126,  6, }, // America/Regina
    {   14, 8423,   10132,  7, }, // America/Resolute
    {   14,  695,   10139,  9, }, // America/Rio_Branco
    {   14, 8440,   10148,  8, }, // America/Santarem
    {   14, 2165,   10156,  8, }, // America/Santiago
    {   14, 6044,   10164, 13, }, // America/Santo_Domingo
    {   14, 1862,   10177,  9, }, // America/Sao_Paulo
    {   14, 6862,   10186, 10, }, // America/Scoresbysund
    {   14, 8457,   10196,  5, }, // America/Sitka
    {   14, 8471,   10201, 14, }, // America/St_Barthelemy
    {   14, 2045,   10215,  9, }, // America/St_Johns
    {   14, 5780,   10224,  9, }, // America/St_Kitts
    {   14, 5797,   10233, 10, }, // America/St_Lucia
    {   14,  855,   10243, 10, }, // America/St_Thomas
    {   14, 5830,   10253, 10, }, // America/St_Vincent
    {   14, 8493,   10263, 10, }, // America/Swift_Current
    {   14, 5363,   10273, 11, }, // America/Tegucigalpa
    {   14, 5673,   10284,  3, }, // America/Thule
    {   14,  313,   10287,  7, }, // America/Tijuana
    {   14,  608,   10294,  7, }, // America/Toronto
    {   14, 5605,   10301,  7, }, // America/Tortola
    {   14, 2077,   10308,  7, }, // America/Vancouver
    {   14, 2128,   10315,  9, }, // America/Whitehorse
    {   14,  734,   10324,  7, }, // America/Winnipeg
    {   14, 8515,   10331,  7, }, // America/Yakutat
    {   14, 5918,   10338,  5, }, // Antarctica/Casey
    {   14, 6027,   10343,  5, }, // Antarctica/Davis
    {   14, 6066,   10348, 16, }, // Antarctica/DumontDUrville
    {   14, 8531,   10364,  7, }, // Antarctica/Macquarie
    {   14, 7029,   10371,  6, }, // Antarctica/Mawson
    {   14, 7076,   10377,  9, }, // Antarctica/McMurdo
    {   14, 8552,   10386,  6, }, // Antarctica/Palmer
    {   14, 7189,   10392,  6, }, // Antarctica/Rothera
    {   14, 7258,   10398,  6, }, // Antarctica/Syowa
    {   14, 8570,   10404,  4, }, // Antarctica/Troll
    {   14, 7337,   10408,  6, }, // Antarctica/Vostok
    {   14, 1413,   10414,  9, }, // Arctic/Longyearbyen
    {   14, 5487,   10423,  3, }, // Asia/Aden
    {   14, 5297,   10426,  6, }, // Asia/Almaty
    {   14, 3870,   10432,  5, }, // Asia/Amman
    {   14, 5428,   10437,  5, }, // Asia/Anadyr
    {   14, 5440,   10442,  5, }, // Asia/Aqtau
    {   14, 5270,   10447,  5, }, // Asia/Aqtobe
    {   14,  964,   10452,  8, }, // Asia/Ashgabat
    {   14, 8587,   10460,  6, }, // Asia/Atyrau
    {   14, 3509,   10466,  5, }, // Asia/Baghdad
    {   14, 5451,   10471,  7, }, // Asia/Bahrain
    {   14, 3555,    9507,  4, }, // Asia/Baku
    {   14, 4368,   10478,  6, }, // Asia/Bangkok
    {   14, 3473,   10484,  7, }, // Asia/Barnaul
    {   14, 3988,   10491,  5, }, // Asia/Beirut
    {   14, 3680,   10496,  5, }, // Asia/Bishkek
    {   14, 5906,   10501,  6, }, // Asia/Brunei
    {   14, 4503,   10507,  5, }, // Asia/Chita
    {   14, 5963,   10512,  9, }, // Asia/Choibalsan
    {   14, 4413,   10521,  7, }, // Asia/Colombo
    {   14, 4442,   10528,  4, }, // Asia/Damascus
    {   14, 1060,    9437,  3, }, // Asia/Dhaka
    {   14, 6125,   10532,  4, }, // Asia/Dili
    {   14, 3498,   10536,  3, }, // Asia/Dubai
    {   14, 6092,   10539,  7, }, // Asia/Dushanbe
    {   14, 8599,   10546,  9, }, // Asia/Famagusta
    {   14, 8614,   10555,  3, }, // Asia/Gaza
    {   14, 4697,   10558, 21, }, // Asia/Hebron
    {   14, 1226,   10579, 16, }, // Asia/Ho_Chi_Minh
    {   14, 2576,   10595,  9, }, // Asia/Hong_Kong
    {   14, 4673,   10604,  4, }, // Asia/Hovd
    {   14, 4086,   10608,  8, }, // Asia/Irkutsk
    {   14, 6969,   10616,  6, }, // Asia/Jakarta
    {   14, 6955,   10622,  9, }, // Asia/Jayapura
    {   14, 1257,   10631,  5, }, // Asia/Jerusalem
    {   14, 3462,   10636,  5, }, // Asia/Kabul
    {   14, 4218,   10641,  9, }, // Asia/Kamchatka
    {   14, 4141,   10650,  6, }, // Asia/Karachi
    {   14, 1152,   10656,  8, }, // Asia/Kathmandu
    {   14, 8624,   10664,  7, }, // Asia/Khandyga
    {   14,  992,   10671,  6, }, // Asia/Kolkata
    {   14, 4099,   10677, 11, }, // Asia/Krasnoyarsk
    {   14, 4761,   10688, 12, }, // Asia/Kuala_Lumpur
    {   14, 5893,   10700,  6, }, // Asia/Kuching
    {   14, 5464,   10706,  6, }, // Asia/Kuwait
    {   14, 1178,   10712,  5, }, // Asia/Macau
    {   14, 3919,   10717,  5, }, // Asia/Magadan
    {   14, 1316,   10722,  6, }, // Asia/Makassar
    {   14, 7145,   10728,  6, }, // Asia/Manila
    {   14, 6883,   10734,  4, }, // Asia/Muscat
    {   14, 2446,   10738,  7, }, // Asia/Nicosia
    {   14, 8638,   10745, 11, }, // Asia/Novokuznetsk
    {   14, 4037,   10756, 10, }, // Asia/Novosibirsk
    {   14, 4131,   10766,  5, }, // Asia/Omsk
    {   14, 7108,   10771,  5, }, // Asia/Oral
    {   14, 6924,   10776,  8, }, // Asia/Phnom_Penh
    {   14, 8656,   10784,  8, }, // Asia/Pontianak
    {   14, 4116,   10792, 10, }, // Asia/Pyongyang
    {   14, 5476,   10802,  3, }, // Asia/Qatar
    {   14, 8671,   10805,  8, }, // Asia/Qostanay
    {   14, 4171,   10813,  9, }, // Asia/Qyzylorda
    {   14, 3486,   10822,  6, }, // Asia/Riyadh
    {   14, 4310,   10828,  6, }, // Asia/Sakhalin
    {   14, 7208,   10834,  6, }, // Asia/Samarkand
    {   14, 3116,   10840,  3, }, // Asia/Seoul
    {   14, 1020,   10843,  6, }, // Asia/Shanghai
    {   14, 3137,   10849,  8, }, // Asia/Singapore
    {   14, 4199,   10857, 12, }, // Asia/Srednekolymsk
    {   14, 3100,   10869,  6, }, // Asia/Taipei
    {   14, 4683,   10875,  5, }, // Asia/Tashkent
    {   14, 3817,   10880,  6, }, // Asia/Tbilisi
    {   14, 2638,   10886,  5, }, // Asia/Tehran
    {   14, 1284,   10891,  5, }, // Asia/Thimphu
    {   14, 2687,   10896,  5, }, // Asia/Tokyo
    {   14, 4474,   10901,  5, }, // Asia/Tomsk
    {   14, 1346,   10906,  9, }, // Asia/Ulaanbaatar
    {   14, 1126,   10915,  6, }, // Asia/Urumqi
    {   14, 8685,   10921,  9, }, // Asia/Ust-Nera
    {   14, 6940,   10930,  8, }, // Asia/Vientiane
    {   14, 4612,   10938, 10, }, // Asia/Vladivostok
    {   14, 4730,   10948,  6, }, // Asia/Yakutsk
    {   14, 1202,   10954,  6, }, // Asia/Yangon
    {   14, 3785,   10960, 10, }, // Asia/Yekaterinburg
    {   14, 3649,   10970,  6, }, // Asia/Yerevan
    {   14, 3565,   10976,  5, }, // Atlantic/Azores
    {   14, 5588,   10981,  6, }, // Atlantic/Bermuda
    {   14, 6591,   10987,  5, }, // Atlantic/Canary
    {   14, 3629,   10992, 12, }, // Atlantic/Cape_Verde
    {   14, 1379,   11004,  4, }, // Atlantic/Faroe
    {   14, 8699,   11008,  6, }, // Atlantic/Madeira
    {   14, 2614,   11014,  8, }, // Atlantic/Reykjavik
    {   14, 7235,   11022, 15, }, // Atlantic/South_Georgia
    {   14, 6784,   11037, 11, }, // Atlantic/St_Helena
    {   14, 6607,   11048,  7, }, // Atlantic/Stanley
    {   14, 1654,   11055,  6, }, // Australia/Adelaide
    {   14, 1619,   11061,  7, }, // Australia/Brisbane
    {   14, 1783,   11068,  9, }, // Australia/Broken_Hill
    {   14, 1567,   11077,  5, }, // Australia/Darwin
    {   14, 3539,   11082,  5, }, // Australia/Eucla
    {   14, 1500,   11087,  6, }, // Australia/Hobart
    {   14, 8716,   11093,  7, }, // Australia/Lindeman
    {   14, 1531,   11100,  8, }, // Australia/Lord_Howe
    {   14, 1711,   11108,  7, }, // Australia/Melbourne
    {   14, 1746,   11115,  4, }, // Australia/Perth
    {   14, 1447,   11119,  5, }, // Australia/Sydney
    {   14, 6390,   11124,  8, }, // Europe/Amsterdam
    {   14, 6149,   11132,  6, }, // Europe/Andorra
    {   14, 3522,   11138,  8, }, // Europe/Astrakhan
    {   14, 6577,   11146,  5, }, // Europe/Athens
    {   14, 6285,   11151,  6, }, // Europe/Belgrade
    {   14, 4659,   11157,  5, }, // Europe/Berlin
    {   14, 6437,   11162, 10, }, // Europe/Bratislava
    {   14, 6178,   11172,  6, }, // Europe/Brussels
    {   14, 3830,   11178,  7, }, // Europe/Bucharest
    {   14, 3708,   11185,  7, }, // Europe/Budapest
    {   14, 8751,   11192,  6, }, // Europe/Busingen
    {   14, 2475,   11198,  8, }, // Europe/Chisinau
    {   14, 6238,   11206,  8, }, // Europe/Copenhagen
    {   14, 2260,   11214,  4, }, // Europe/Dublin
    {   14, 6256,   11218,  8, }, // Europe/Gibraltar
    {   14, 8767,   11226,  6, }, // Europe/Guernsey
    {   14, 6561,   11232,  6, }, // Europe/Helsinki
    {   14, 8783,   11238,  9, }, // Europe/Isle_of_Man
    {   14, 1097,   11247,  7, }, // Europe/Istanbul
    {   14, 8802,   11254,  5, }, // Europe/Jersey
    {   14, 3881,   11259,  9, }, // Europe/Kaliningrad
    {   14, 8924,   11268,  5, }, // Europe/Kirov
    {   14, 2419,   11273,  4, }, // Europe/Kyiv
    {   14, 3078,   11277,  6, }, // Europe/Lisbon
    {   14, 6455,   11283,  9, }, // Europe/Ljubljana
    {   14, 2393,   11292,  4, }, // Europe/London
    {   14, 6314,   11296,  9, }, // Europe/Luxembourg
    {   14, 4794,   11305,  5, }, // Europe/Madrid
    {   14, 6346,   11310,  5, }, // Europe/Malta
    {   14, 6531,   11315,  8, }, // Europe/Mariehamn
    {   14, 3595,   11323,  5, }, // Europe/Minsk
    {   14, 6359,   11328,  6, }, // Europe/Monaco
    {   14, 3443,   11334,  5, }, // Europe/Moscow
    {   14, 6407,   11339,  5, }, // Europe/Oslo
    {   14, 4186,   11344,  5, }, // Europe/Paris
    {   14, 6373,   11349,  9, }, // Europe/Podgorica
    {   14, 6224,   11358,  4, }, // Europe/Prague
    {   14, 8816,   11362,  4, }, // Europe/Riga
    {   14, 6273,   11006,  4, }, // Europe/Rome
    {   14, 4233,   11366,  5, }, // Europe/Samara
    {   14, 6419,   11371, 10, }, // Europe/San_Marino
    {   14, 6194,   11381,  7, }, // Europe/Sarajevo
    {   14, 4353,   11388,  7, }, // Europe/Saratov
    {   14, 8828,   11395,  9, }, // Europe/Simferopol
    {   14, 6332,   11404,  5, }, // Europe/Skopje
    {   14, 6548,   11409,  5, }, // Europe/Sofia
    {   14, 6472,   11414,  8, }, // Europe/Stockholm
    {   14, 8846,   11422,  5, }, // Europe/Tallinn
    {   14, 6135,   11427,  6, }, // Europe/Tirane
    {   14, 8861,   11433, 10, }, // Europe/Ulyanovsk
    {   14, 6301,   11443,  5, }, // Europe/Vaduz
    {   14, 6516,   11448,  9, }, // Europe/Vatican
    {   14, 6164,   11457,  5, }, // Europe/Vienna
    {   14, 8878,   11462,  7, }, // Europe/Vilnius
    {   14, 4629,   11469,  8, }, // Europe/Volgograd
    {   14, 3055,   11477,  5, }, // Europe/Warsaw
    {   14, 6210,   11482,  4, }, // Europe/Zagreb
    {   14, 6489,   11486,  5, }, // Europe/Zurich
    {   14, 4980,   11491, 12, }, // Indian/Antananarivo
    {   14, 6910,   11503,  6, }, // Indian/Chagos
    {   14, 5979,   11509,  7, }, // Indian/Christmas
    {   14, 5996,   11516,  5, }, // Indian/Cocos
    {   14, 4931,   11521,  9, }, // Indian/Comoro
    {   14, 6624,   11530,  9, }, // Indian/Kerguelen
    {   14, 7223,   11539,  4, }, // Indian/Mahe
    {   14, 7013,   11543,  8, }, // Indian/Maldives
    {   14, 3971,   11551,  8, }, // Indian/Mauritius
    {   14, 5000,   11559,  5, }, // Indian/Mayotte
    {   14, 7174,   11564,  7, }, // Indian/Reunion
    {   14, 4324,   11571,  4, }, // Pacific/Apia
    {   14,  932,   11575,  7, }, // Pacific/Auckland
    {   14, 3608,   11582,  8, }, // Pacific/Bougainville
    {   14, 2863,   11590,  6, }, // Pacific/Chatham
    {   14, 3022,    9628,  3, }, // Pacific/Chuuk
    {   14, 2201,   11596,  4, }, // Pacific/Easter
    {   14, 7323,   11600,  5, }, // Pacific/Efate
    {   14, 7290,   11605,  7, }, // Pacific/Fakaofo
    {   14, 3804,   11612,  4, }, // Pacific/Fiji
    {   14, 7306,   11616,  8, }, // Pacific/Funafuti
    {   14, 6641,   11624,  8, }, // Pacific/Galapagos
    {   14, 6659,   11632,  6, }, // Pacific/Gambier
    {   14, 3724,   11638, 11, }, // Pacific/Guadalcanal
    {   14, 5935,    9852,  4, }, // Pacific/Guam
    {   14, 2929,   11649,  8, }, // Pacific/Honolulu
    {   14, 2897,   11657,  8, }, // Pacific/Kanton
    {   14, 3900,   11665, 11, }, // Pacific/Kiritimati
    {   14, 6982,   11676,  5, }, // Pacific/Kosrae
    {   14, 2708,   11681,  8, }, // Pacific/Kwajalein
    {   14, 4779,   11689,  6, }, // Pacific/Majuro
    {   14, 3953,   11695,  8, }, // Pacific/Marquesas
    {   14, 8909,   11703,  6, }, // Pacific/Midway
    {   14, 7047,   11709,  5, }, // Pacific/Nauru
    {   14, 7095,    9973,  4, }, // Pacific/Niue
    {   14, 4070,   11714,  7, }, // Pacific/Norfolk
    {   14, 7061,   11721,  5, }, // Pacific/Noumea
    {   14, 2991,   11726,  9, }, // Pacific/Pago_Pago
    {   14, 7118,   11735,  5, }, // Pacific/Palau
    {   14, 7157,   11740,  7, }, // Pacific/Pitcairn
    {   14, 2961,   11747,  5, }, // Pacific/Pohnpei
    {   14, 4709,   11752, 10, }, // Pacific/Port_Moresby
    {   14, 6009,   11762,  9, }, // Pacific/Rarotonga
    {   14, 5948,   11771,  6, }, // Pacific/Saipan
    {   14, 7275,   11777,  6, }, // Pacific/Tahiti
    {   14, 6675,   11783,  6, }, // Pacific/Tarawa
    {   14, 4485,   11789,  9, }, // Pacific/Tongatapu
    {   14, 7355,    9156,  3, }, // Pacific/Wake
    {   14, 7368,   11798,  5, }, // Pacific/Wallis
    {   42,   14,       0,  6, }, // Africa/Asmara Aragonese/Latin/Spain
    {   42, 4337,       6,  8, }, // Africa/Sao_Tome
    {   42, 4154,      14,  8, }, // America/Asuncion
    {   42,  237,      22,  8, }, // America/Atikokan
    {   42, 7586,      30, 17, }, // America/Bahia_Banderas
    {   42, 3770,      47,  6, }, // America/Cancun
    {   42, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   42, 5640,      66,  7, }, // America/Curacao
    {   42,  481,      73, 13, }, // America/Indiana/Knox
    {   42, 7924,      86, 16, }, // America/Indiana/Marengo
    {   42, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   42, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   42, 8001,     139, 14, }, // America/Indiana/Vevay
    {   42, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   42, 8049,     171, 16, }, // America/Indiana/Winamac
    {   42, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   42, 5849,     207, 22, }, // America/Lower_Princes
    {   42, 8183,     229,  6, }, // America/Merida
    {   42, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   42, 8292,     255, 20, }, // America/North_Dakota/Center
    {   42, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   42,  393,     298,  4, }, // America/Nuuk
    {   42, 6862,     302, 16, }, // America/Scoresbysund
    {   42, 8471,     318, 14, }, // America/St_Barthelemy
    {   42, 2045,     332, 10, }, // America/St_Johns
    {   42, 5780,     342,  9, }, // America/St_Kitts
    {   42, 5797,     351,  9, }, // America/St_Lucia
    {   42,  855,     360, 10, }, // America/St_Thomas
    {   42, 5830,     370, 11, }, // America/St_Vincent
    {   42, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   42, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   42, 1152,     413,  9, }, // Asia/Kathmandu
    {   42,  992,     422,  7, }, // Asia/Kolkata
    {   42, 1178,    4222,  5, }, // Asia/Macau
    {   42, 1202,     442,  6, }, // Asia/Yangon
    {   42, 1379,     448,  5, }, // Atlantic/Faroe
    {   42, 6784,     453, 10, }, // Atlantic/St_Helena
    {   42, 2419,     463,  4, }, // Europe/Kyiv
    {   42, 7174,     467,  7, }, // Indian/Reunion
    {   42, 3022,     474,  5, }, // Pacific/Chuuk
    {   42, 2961,     479,  7, }, // Pacific/Pohnpei
    {   43, 6751,   11803,  6, }, // Africa/Abidjan Armenian/Armenian/Armenia
    {   43, 6723,   11809,  4, }, // Africa/Accra
    {   43, 4961,   11813, 10, }, // Africa/Addis_Ababa
    {   43, 7383,   11823,  5, }, // Africa/Algiers
    {   43,   14,   11828,  6, }, // Africa/Asmara
    {   43,   44,   11834,  6, }, // Africa/Bamako
    {   43, 5159,   11840,  5, }, // Africa/Bangui
    {   43, 6709,   11845,  7, }, // Africa/Banjul
    {   43, 7398,   11852,  6, }, // Africa/Bissau
    {   43, 4859,   11858,  8, }, // Africa/Blantyre
    {   43, 5189,   11866,  8, }, // Africa/Brazzaville
    {   43, 4824,   11874, 12, }, // Africa/Bujumbura
    {   43, 2242,   11886,  6, }, // Africa/Cairo
    {   43, 4019,   11892, 10, }, // Africa/Casablanca
    {   43, 7412,   11902,  6, }, // Africa/Ceuta
    {   43, 6736,   11908,  7, }, // Africa/Conakry
    {   43, 6803,   11915,  5, }, // Africa/Dakar
    {   43, 5032,   11920, 12, }, // Africa/Dar_es_Salaam
    {   43, 4945,   11932,  7, }, // Africa/Djibouti
    {   43, 5145,   11939,  6, }, // Africa/Douala
    {   43, 5068,   11945,  8, }, // Africa/El_Aaiun
    {   43, 6816,   11953,  8, }, // Africa/Freetown
    {   43, 4808,   11961,  8, }, // Africa/Gaborone
    {   43, 4917,   11969,  6, }, // Africa/Harare
    {   43, 4381,   11975, 12, }, // Africa/Johannesburg
    {   43, 4401,   11987,  5, }, // Africa/Juba
    {   43, 5053,   11992,  7, }, // Africa/Kampala
    {   43, 4426,   11999,  7, }, // Africa/Khartoum
    {   43, 4875,   12006,  6, }, // Africa/Kigali
    {   43, 5208,   12012,  7, }, // Africa/Kinshasa
    {   43, 4646,   12019,  5, }, // Africa/Lagos
    {   43, 5238,   12024,  7, }, // Africa/Libreville
    {   43, 6832,   12031,  4, }, // Africa/Lome
    {   43, 5113,   12035,  7, }, // Africa/Luanda
    {   43, 4841,   12042, 11, }, // Africa/Lubumbashi
    {   43, 4903,   12053,  7, }, // Africa/Lusaka
    {   43, 5224,   12060,  6, }, // Africa/Malabo
    {   43, 4889,   12066,  8, }, // Africa/Maputo
    {   43, 5099,   12074,  7, }, // Africa/Maseru
    {   43, 5084,   12081,  7, }, // Africa/Mbabane
    {   43, 5015,   12088,  8, }, // Africa/Mogadishu
    {   43, 6997,   12096,  8, }, // Africa/Monrovia
    {   43, 3755,   12104,  7, }, // Africa/Nairobi
    {   43, 5173,   12111,  7, }, // Africa/Ndjamena
    {   43, 5256,   12118,  6, }, // Africa/Niamey
    {   43, 6766,   12124,  8, }, // Africa/Nouakchott
    {   43, 6690,   12132, 11, }, // Africa/Ouagadougou
    {   43, 5127,   12143, 10, }, // Africa/Porto-Novo
    {   43, 4337,   12153,  8, }, // Africa/Sao_Tome
    {   43, 2732,   12161,  7, }, // Africa/Tripoli
    {   43, 6503,   12168,  6, }, // Africa/Tunis
    {   43, 4054,   12174,  8, }, // Africa/Windhoek
    {   43,  132,   12182,  9, }, // America/Adak
    {   43, 3183,   12191,  7, }, // America/Anchorage
    {   43, 5524,   12198,  8, }, // America/Anguilla
    {   43, 5541,   12206,  8, }, // America/Antigua
    {   43, 4456,   12214, 10, }, // America/Araguaina
    {   43,  166,   12224, 13, }, // America/Argentina/Buenos_Aires
    {   43,   91,   12237,  9, }, // America/Argentina/Catamarca
    {   43,  270,   12246,  7, }, // America/Argentina/Cordoba
    {   43,  441,   12253,  7, }, // America/Argentina/Jujuy
    {   43, 7425,   12260,  8, }, // America/Argentina/La_Rioja
    {   43,  565,   12268,  7, }, // America/Argentina/Mendoza
    {   43, 7452,   12275, 12, }, // America/Argentina/Rio_Gallegos
    {   43, 7483,   12287,  5, }, // America/Argentina/Salta
    {   43, 7507,   12292,  9, }, // America/Argentina/San_Juan
    {   43, 5497,   12301,  9, }, // America/Argentina/San_Luis
    {   43, 7534,   12310,  9, }, // America/Argentina/Tucuman
    {   43, 7560,   12319,  8, }, // America/Argentina/Ushuaia
    {   43, 5557,   12327,  6, }, // America/Aruba
    {   43, 4154,   12333,  9, }, // America/Asuncion
    {   43,  237,   12342,  8, }, // America/Atikokan
    {   43, 3581,   12350,  5, }, // America/Bahia
    {   43, 7586,   12355, 14, }, // America/Bahia_Banderas
    {   43, 5571,   12369,  8, }, // America/Barbados
    {   43, 7609,   12377,  5, }, // America/Belem
    {   43, 5309,   12382,  5, }, // America/Belize
    {   43, 7623,   12387, 12, }, // America/Blanc-Sablon
    {   43, 7644,   12399,  9, }, // America/Boa_Vista
    {   43, 4263,   12408,  6, }, // America/Bogota
    {   43, 7662,   12414,  5, }, // America/Boise
    {   43, 7676,   12419, 11, }, // America/Cambridge_Bay
    {   43, 7698,   12430, 13, }, // America/Campo_Grande
    {   43, 3770,   12443,  7, }, // America/Cancun
    {   43, 4596,   12450,  7, }, // America/Caracas
    {   43, 4247,   12457,  5, }, // America/Cayenne
    {   43, 5398,   12462, 14, }, // America/Cayman
    {   43, 3251,   12476,  6, }, // America/Chicago
    {   43, 7719,   12482,  8, }, // America/Chihuahua
    {   43, 7737,   12490, 15, }, // America/Ciudad_Juarez
    {   43, 5324,   12505, 10, }, // America/Costa_Rica
    {   43, 7759,   12515,  7, }, // America/Creston
    {   43, 3693,   12522,  7, }, // America/Cuiaba
    {   43, 5640,   12529,  9, }, // America/Curacao
    {   43, 7775,   12538, 12, }, // America/Danmarkshavn
    {   43, 7796,   12550,  7, }, // America/Dawson
    {   43, 7811,   12557, 12, }, // America/Dawson_Creek
    {   43,  805,   12569,  6, }, // America/Denver
    {   43, 3351,   12575,  7, }, // America/Detroit
    {   43, 5656,   12582,  8, }, // America/Dominica
    {   43,  893,   12590,  8, }, // America/Edmonton
    {   43, 7832,   12598,  9, }, // America/Eirunepe
    {   43, 5343,   12607,  8, }, // America/El_Salvador
    {   43, 7849,   12615, 11, }, // America/Fort_Nelson
    {   43, 7869,   12626,  9, }, // America/Fortaleza
    {   43, 7887,   12635,  9, }, // America/Glace_Bay
    {   43, 6844,   12644,  8, }, // America/Goose_Bay
    {   43, 4514,   12652, 11, }, // America/Grand_Turk
    {   43, 5687,   12663,  7, }, // America/Grenada
    {   43, 5703,   12670, 10, }, // America/Guadeloupe
    {   43, 3662,   12680,  9, }, // America/Guatemala
    {   43, 4743,   12689,  9, }, // America/Guayaquil
    {   43, 6895,   12698,  6, }, // America/Guyana
    {   43, 1923,   12704,  8, }, // America/Halifax
    {   43, 2221,   12712,  6, }, // America/Havana
    {   43, 7905,   12718,  9, }, // America/Hermosillo
    {   43,  348,   12727, 12, }, // America/Indiana/Indianapolis
    {   43,  481,   12739, 13, }, // America/Indiana/Knox
    {   43, 7924,   12752, 16, }, // America/Indiana/Marengo
    {   43, 7948,   12768, 20, }, // America/Indiana/Petersburg
    {   43, 7975,   12788, 17, }, // America/Indiana/Tell_City
    {   43, 8001,   12805, 14, }, // America/Indiana/Vevay
    {   43, 8023,   12819, 16, }, // America/Indiana/Vincennes
    {   43, 8049,   12835, 16, }, // America/Indiana/Winamac
    {   43, 8073,   12851,  7, }, // America/Inuvik
    {   43,  660,   12858,  8, }, // America/Iqaluit
    {   43, 2665,   12866,  7, }, // America/Jamaica
    {   43, 5282,   12873,  5, }, // America/Juneau
    {   43,  521,   12878,  8, }, // America/Kentucky/Louisville
    {   43, 8088,   12886, 19, }, // America/Kentucky/Monticello
    {   43, 5621,   12905, 10, }, // America/Kralendijk
    {   43, 4278,   12915,  6, }, // America/La_Paz
    {   43, 7132,   12921,  4, }, // America/Lima
    {   43, 3390,   12925, 11, }, // America/Los_Angeles
    {   43, 5849,   12936, 22, }, // America/Lower_Princes
    {   43, 8116,   12958,  6, }, // America/Maceio
    {   43, 8131,   12964,  8, }, // America/Managua
    {   43, 1892,   12972,  7, }, // America/Manaus
    {   43, 5814,   12979,  6, }, // America/Marigot
    {   43, 5722,   12985,  9, }, // America/Martinique
    {   43, 8147,   12994,  9, }, // America/Matamoros
    {   43, 2779,   13003,  8, }, // America/Mazatlan
    {   43, 8165,   13011,  8, }, // America/Menominee
    {   43, 8183,   13019,  6, }, // America/Merida
    {   43, 8198,   13025, 10, }, // America/Metlakatla
    {   43, 2811,   13035,  6, }, // America/Mexico_City
    {   43, 4293,   13041,  7, }, // America/Miquelon
    {   43, 8217,   13048,  7, }, // America/Moncton
    {   43, 8233,   13055,  8, }, // America/Monterrey
    {   43, 4000,   13063, 10, }, // America/Montevideo
    {   43, 5741,   13073,  8, }, // America/Montserrat
    {   43, 5383,   13081,  6, }, // America/Nassau
    {   43, 3294,   13087,  9, }, // America/New_York
    {   43, 8251,   13096,  3, }, // America/Nome
    {   43, 1834,   13099,  7, }, // America/Noronha
    {   43, 8264,   13106, 25, }, // America/North_Dakota/Beulah
    {   43, 8292,   13131, 28, }, // America/North_Dakota/Center
    {   43, 8320,   13159, 30, }, // America/North_Dakota/New_Salem
    {   43,  393,   13189,  6, }, // America/Nuuk
    {   43, 8351,   13195,  7, }, // America/Ojinaga
    {   43, 5413,   13202,  6, }, // America/Panama
    {   43, 6106,   13208, 10, }, // America/Paramaribo
    {   43, 3224,   13218,  6, }, // America/Phoenix
    {   43, 3847,   13224, 12, }, // America/Port-au-Prince
    {   43, 5871,   13236, 13, }, // America/Port_of_Spain
    {   43, 8367,   13249, 13, }, // America/Porto_Velho
    {   43, 5760,   13262, 12, }, // America/Puerto_Rico
    {   43, 3932,   13274, 13, }, // America/Punta_Arenas
    {   43, 8387,   13287, 12, }, // America/Rankin_Inlet
    {   43, 8408,   13299,  6, }, // America/Recife
    {   43, 1979,   13305,  7, }, // America/Regina
    {   43, 8423,   13312,  9, }, // America/Resolute
    {   43,  695,   13321, 12, }, // America/Rio_Branco
    {   43, 8440,   13333,  8, }, // America/Santarem
    {   43, 2165,   13341,  8, }, // America/Santiago
    {   43, 6044,   13349, 13, }, // America/Santo_Domingo
    {   43, 1862,   13362, 11, }, // America/Sao_Paulo
    {   43, 6862,   13373, 11, }, // America/Scoresbysund
    {   43, 8457,   13384,  5, }, // America/Sitka
    {   43, 8471,   13389, 12, }, // America/St_Barthelemy
    {   43, 2045,   13401,  9, }, // America/St_Johns
    {   43, 5780,   13410,  9, }, // America/St_Kitts
    {   43, 5797,   13419, 12, }, // America/St_Lucia
    {   43,  855,   13431, 10, }, // America/St_Thomas
    {   43, 5830,   13441, 12, }, // America/St_Vincent
    {   43, 8493,   13453, 12, }, // America/Swift_Current
    {   43, 5363,   13465, 12, }, // America/Tegucigalpa
    {   43, 5673,   13477,  5, }, // America/Thule
    {   43,  313,   13482,  8, }, // America/Tijuana
    {   43,  608,   13490,  7, }, // America/Toronto
    {   43, 5605,   13497,  7, }, // America/Tortola
    {   43, 2077,   13504,  9, }, // America/Vancouver
    {   43, 2128,   13513,  9, }, // America/Whitehorse
    {   43,  734,   13522,  7, }, // America/Winnipeg
    {   43, 8515,   13529,  8, }, // America/Yakutat
    {   43, 5918,   13537,  5, }, // Antarctica/Casey
    {   43, 6027,   13542,  6, }, // Antarctica/Davis
    {   43, 6066,   13548, 17, }, // Antarctica/DumontDUrville
    {   43, 8531,   13565, 14, }, // Antarctica/Macquarie
    {   43, 7029,   13579,  7, }, // Antarctica/Mawson
    {   43, 7076,   13586,  9, }, // Antarctica/McMurdo
    {   43, 8552,   13595,  6, }, // Antarctica/Palmer
    {   43, 7189,   13601,  6, }, // Antarctica/Rothera
    {   43, 7258,   13607,  5, }, // Antarctica/Syowa
    {   43, 8570,   13612,  5, }, // Antarctica/Troll
    {   43, 7337,   13617,  6, }, // Antarctica/Vostok
    {   43, 1413,   13623,  7, }, // Arctic/Longyearbyen
    {   43, 5487,   13630,  4, }, // Asia/Aden
    {   43, 5297,   13634,  6, }, // Asia/Almaty
    {   43, 3870,   13640,  5, }, // Asia/Amman
    {   43, 5428,   13645,  6, }, // Asia/Anadyr
    {   43, 5440,   13651,  6, }, // Asia/Aqtau
    {   43, 5270,   13657,  6, }, // Asia/Aqtobe
    {   43,  964,   13663,  7, }, // Asia/Ashgabat
    {   43, 8587,   13670,  7, }, // Asia/Atyrau
    {   43, 3509,   13677,  6, }, // Asia/Baghdad
    {   43, 5451,   13683,  7, }, // Asia/Bahrain
    {   43, 3555,   13690,  5, }, // Asia/Baku
    {   43, 4368,   13695,  7, }, // Asia/Bangkok
    {   43, 3473,   13702,  8, }, // Asia/Barnaul
    {   43, 3988,   13710,  7, }, // Asia/Beirut
    {   43, 3680,   13717,  6, }, // Asia/Bishkek
    {   43, 5906,   13723,  7, }, // Asia/Brunei
    {   43, 4503,   13730,  4, }, // Asia/Chita
    {   43, 5963,   13734,  9, }, // Asia/Choibalsan
    {   43, 4413,   13743,  7, }, // Asia/Colombo
    {   43, 4442,   13750,  8, }, // Asia/Damascus
    {   43, 1060,   13758,  5, }, // Asia/Dhaka
    {   43, 6125,   13763,  4, }, // Asia/Dili
    {   43, 3498,   13767,  6, }, // Asia/Dubai
    {   43, 6092,   13773,  8, }, // Asia/Dushanbe
    {   43, 8599,   13781, 10, }, // Asia/Famagusta
    {   43, 8614,   13791,  4, }, // Asia/Gaza
    {   43, 4697,   13795,  6, }, // Asia/Hebron
    {   43, 1226,   13801,  7, }, // Asia/Ho_Chi_Minh
    {   43, 2576,   13808,  7, }, // Asia/Hong_Kong
    {   43, 4673,   13815,  4, }, // Asia/Hovd
    {   43, 4086,   13819,  8, }, // Asia/Irkutsk
    {   43, 6969,   13827,  7, }, // Asia/Jakarta
    {   43, 6955,   13834,  8, }, // Asia/Jayapura
    {   43, 1257,   13842,  9, }, // Asia/Jerusalem
    {   43, 3462,   13851,  6, }, // Asia/Kabul
    {   43, 4218,   13857,  8, }, // Asia/Kamchatka
    {   43, 4141,   13865,  6, }, // Asia/Karachi
    {   43, 1152,   13871,  9, }, // Asia/Kathmandu
    {   43, 8624,   13880,  7, }, // Asia/Khandyga
    {   43,  992,   13887,  8, }, // Asia/Kolkata
    {   43, 4099,   13895, 11, }, // Asia/Krasnoyarsk
    {   43, 4761,   13906, 15, }, // Asia/Kuala_Lumpur
    {   43, 5893,   13921,  7, }, // Asia/Kuching
    {   43, 5464,   13928,  7, }, // Asia/Kuwait
    {   43, 1178,   13935,  5, }, // Asia/Macau
    {   43, 3919,   13940,  7, }, // Asia/Magadan
    {   43, 1316,   13947,  7, }, // Asia/Makassar
    {   43, 7145,   13954,  6, }, // Asia/Manila
    {   43, 6883,   13960,  6, }, // Asia/Muscat
    {   43, 2446,   13966,  7, }, // Asia/Nicosia
    {   43, 8638,   13973, 12, }, // Asia/Novokuznetsk
    {   43, 4037,   13985, 11, }, // Asia/Novosibirsk
    {   43, 4131,   13996,  4, }, // Asia/Omsk
    {   43, 7108,   14000,  7, }, // Asia/Oral
    {   43, 6924,   14007,  7, }, // Asia/Phnom_Penh
    {   43, 8656,   14014,  9, }, // Asia/Pontianak
    {   43, 4116,   14023,  7, }, // Asia/Pyongyang
    {   43, 5476,   14030,  5, }, // Asia/Qatar
    {   43, 8671,   14035,  8, }, // Asia/Qostanay
    {   43, 4171,   14043,  9, }, // Asia/Qyzylorda
    {   43, 3486,   14052,  7, }, // Asia/Riyadh
    {   43, 4310,   14059,  7, }, // Asia/Sakhalin
    {   43, 7208,   14066,  9, }, // Asia/Samarkand
    {   43, 3116,   14075,  5, }, // Asia/Seoul
    {   43, 1020,   14080,  6, }, // Asia/Shanghai
    {   43, 3137,   14086,  9, }, // Asia/Singapore
    {   43, 4199,   14095, 13, }, // Asia/Srednekolymsk
    {   43, 3100,   14108,  6, }, // Asia/Taipei
    {   43, 4683,   14114,  7, }, // Asia/Tashkent
    {   43, 3817,   14121,  7, }, // Asia/Tbilisi
    {   43, 2638,   14128,  6, }, // Asia/Tehran
    {   43, 1284,   14134,  8, }, // Asia/Thimphu
    {   43, 2687,   14142,  5, }, // Asia/Tokyo
    {   43, 4474,   14147,  5, }, // Asia/Tomsk
    {   43, 1346,   14152, 11, }, // Asia/Ulaanbaatar
    {   43, 1126,   14163,  8, }, // Asia/Urumqi
    {   43, 8685,   14171,  9, }, // Asia/Ust-Nera
    {   43, 6940,   14180,  8, }, // Asia/Vientiane
    {   43, 4612,   14188, 11, }, // Asia/Vladivostok
    {   43, 4730,   14199,  8, }, // Asia/Yakutsk
    {   43, 1202,   14207,  7, }, // Asia/Yangon
    {   43, 3785,   14214, 13, }, // Asia/Yekaterinburg
    {   43, 3649,   14227,  5, }, // Asia/Yerevan
    {   43, 3565,   14232, 15, }, // Atlantic/Azores
    {   43, 5588,   14247, 10, }, // Atlantic/Bermuda
    {   43, 6591,   14257, 16, }, // Atlantic/Canary
    {   43, 3629,   14273, 10, }, // Atlantic/Cape_Verde
    {   43, 1379,   14283, 16, }, // Atlantic/Faroe
    {   43, 8699,   14299,  7, }, // Atlantic/Madeira
    {   43, 2614,   14306,  9, }, // Atlantic/Reykjavik
    {   43, 7235,   14315, 16, }, // Atlantic/South_Georgia
    {   43, 6784,   14331, 18, }, // Atlantic/St_Helena
    {   43, 6607,   14349,  6, }, // Atlantic/Stanley
    {   43, 1654,   14355,  8, }, // Australia/Adelaide
    {   43, 1619,   14363,  7, }, // Australia/Brisbane
    {   43, 1783,   14370, 10, }, // Australia/Broken_Hill
    {   43, 1567,   14380,  6, }, // Australia/Darwin
    {   43, 3539,   14386,  6, }, // Australia/Eucla
    {   43, 1500,   14392,  6, }, // Australia/Hobart
    {   43, 8716,   14398,  8, }, // Australia/Lindeman
    {   43, 1531,   14406, 14, }, // Australia/Lord_Howe
    {   43, 1711,   14420,  8, }, // Australia/Melbourne
    {   43, 1746,   14428,  4, }, // Australia/Perth
    {   43, 1447,   14432,  6, }, // Australia/Sydney
    {   43, 6390,   14438,  9, }, // Europe/Amsterdam
    {   43, 6149,   14447,  7, }, // Europe/Andorra
    {   43, 3522,   14454,  8, }, // Europe/Astrakhan
    {   43, 6577,   14462,  5, }, // Europe/Athens
    {   43, 6285,   14467,  7, }, // Europe/Belgrade
    {   43, 4659,   14474,  6, }, // Europe/Berlin
    {   43, 6437,   14480, 10, }, // Europe/Bratislava
    {   43, 6178,   14490,  8, }, // Europe/Brussels
    {   43, 3830,   14498,  9, }, // Europe/Bucharest
    {   43, 3708,   14507,  9, }, // Europe/Budapest
    {   43, 8751,   14516, 10, }, // Europe/Busingen
    {   43, 2475,   14526,  5, }, // Europe/Chisinau
    {   43, 6238,   14531, 10, }, // Europe/Copenhagen
    {   43, 2260,   14541,  7, }, // Europe/Dublin
    {   43, 6256,   14548,  9, }, // Europe/Gibraltar
    {   43, 8767,   14557,  6, }, // Europe/Guernsey
    {   43, 6561,   14563,  8, }, // Europe/Helsinki
    {   43, 8783,   14571,  8, }, // Europe/Isle_of_Man
    {   43, 1097,   14579,  8, }, // Europe/Istanbul
    {   43, 8802,   14587,  5, }, // Europe/Jersey
    {   43, 3881,   14592, 11, }, // Europe/Kaliningrad
    {   43, 8924,   14603,  5, }, // Europe/Kirov
    {   43, 2419,   14608,  3, }, // Europe/Kyiv
    {   43, 3078,   14611,  7, }, // Europe/Lisbon
    {   43, 6455,   14618, 10, }, // Europe/Ljubljana
    {   43, 2393,   14628,  6, }, // Europe/London
    {   43, 6314,   14634, 13, }, // Europe/Luxembourg
    {   43, 4794,   14647,  6, }, // Europe/Madrid
    {   43, 6346,   14653,  5, }, // Europe/Malta
    {   43, 6531,   14658,  9, }, // Europe/Mariehamn
    {   43, 3595,   14667,  5, }, // Europe/Minsk
    {   43, 6359,   14672,  6, }, // Europe/Monaco
    {   43, 3443,   14678,  6, }, // Europe/Moscow
    {   43, 6407,   14684,  4, }, // Europe/Oslo
    {   43, 4186,   14688,  5, }, // Europe/Paris
    {   43, 6373,   14693,  9, }, // Europe/Podgorica
    {   43, 6224,   14702,  5, }, // Europe/Prague
    {   43, 8816,   14707,  4, }, // Europe/Riga
    {   43, 6273,   14711,  4, }, // Europe/Rome
    {   43, 4233,   14715,  6, }, // Europe/Samara
    {   43, 6419,   14721, 10, }, // Europe/San_Marino
    {   43, 6194,   14731,  6, }, // Europe/Sarajevo
    {   43, 4353,   14737,  7, }, // Europe/Saratov
    {   43, 8828,   14744, 10, }, // Europe/Simferopol
    {   43, 6332,   14754,  6, }, // Europe/Skopje
    {   43, 6548,   14760,  5, }, // Europe/Sofia
    {   43, 6472,   14765,  8, }, // Europe/Stockholm
    {   43, 8846,   14773,  6, }, // Europe/Tallinn
    {   43, 6135,   14779,  6, }, // Europe/Tirane
    {   43, 8861,   14785, 10, }, // Europe/Ulyanovsk
    {   43, 6301,   14795,  6, }, // Europe/Vaduz
    {   43, 6516,   14801,  7, }, // Europe/Vatican
    {   43, 6164,   14808,  6, }, // Europe/Vienna
    {   43, 8878,   14814,  8, }, // Europe/Vilnius
    {   43, 4629,   14822,  9, }, // Europe/Volgograd
    {   43, 3055,   14831,  7, }, // Europe/Warsaw
    {   43, 6210,   14838,  6, }, // Europe/Zagreb
    {   43, 6489,   14844,  7, }, // Europe/Zurich
    {   43, 4980,   14851, 13, }, // Indian/Antananarivo
    {   43, 6910,   14864,  5, }, // Indian/Chagos
    {   43, 5979,   14869, 18, }, // Indian/Christmas
    {   43, 5996,   14887, 16, }, // Indian/Cocos
    {   43, 4931,   14903, 16, }, // Indian/Comoro
    {   43, 6624,   14919,  8, }, // Indian/Kerguelen
    {   43, 7223,   14927,  3, }, // Indian/Mahe
    {   43, 7013,   14930,  9, }, // Indian/Maldives
    {   43, 3971,   14939,  9, }, // Indian/Mauritius
    {   43, 5000,   14948,  5, }, // Indian/Mayotte
    {   43, 7174,   14953,  9, }, // Indian/Reunion
    {   43, 4324,   14962,  4, }, // Pacific/Apia
    {   43,  932,   14966,  6, }, // Pacific/Auckland
    {   43, 3608,   14972,  9, }, // Pacific/Bougainville
    {   43, 2863,   14981, 10, }, // Pacific/Chatham
    {   43, 3022,   14991,  6, }, // Pacific/Chuuk
    {   43, 2201,   14997, 10, }, // Pacific/Easter
    {   43, 7323,   15007,  5, }, // Pacific/Efate
    {   43, 7290,   15012,  7, }, // Pacific/Fakaofo
    {   43, 3804,   15019,  4, }, // Pacific/Fiji
    {   43, 7306,   15023, 10, }, // Pacific/Funafuti
    {   43, 6641,   15033, 20, }, // Pacific/Galapagos
    {   43, 6659,   15053, 14, }, // Pacific/Gambier
    {   43, 3724,   15067, 12, }, // Pacific/Guadalcanal
    {   43, 5935,   15079,  5, }, // Pacific/Guam
    {   43, 2929,   15084, 10, }, // Pacific/Honolulu
    {   43, 2897,   15094, 14, }, // Pacific/Kanton
    {   43, 3900,   15108, 10, }, // Pacific/Kiritimati
    {   43, 6982,   15118,  6, }, // Pacific/Kosrae
    {   43, 2708,   15124,  9, }, // Pacific/Kwajalein
    {   43, 4779,   15133,  7, }, // Pacific/Majuro
    {   43, 3953,   15140, 17, }, // Pacific/Marquesas
    {   43, 8909,   15157, 12, }, // Pacific/Midway
    {   43, 7047,   15169,  7, }, // Pacific/Nauru
    {   43, 7095,   15176,  5, }, // Pacific/Niue
    {   43, 4070,   15181,  7, }, // Pacific/Norfolk
    {   43, 7061,   15188,  6, }, // Pacific/Noumea
    {   43, 2991,   15194,  9, }, // Pacific/Pago_Pago
    {   43, 7118,   15203,  6, }, // Pacific/Palau
    {   43, 7157,   15209,  7, }, // Pacific/Pitcairn
    {   43, 2961,   15216,  6, }, // Pacific/Pohnpei
    {   43, 4709,   15222, 11, }, // Pacific/Port_Moresby
    {   43, 6009,   15233,  9, }, // Pacific/Rarotonga
    {   43, 5948,   15242,  6, }, // Pacific/Saipan
    {   43, 7275,   15248,  5, }, // Pacific/Tahiti
    {   43, 6675,   15253,  6, }, // Pacific/Tarawa
    {   43, 4485,   15259, 10, }, // Pacific/Tongatapu
    {   43, 7355,   15269, 10, }, // Pacific/Wake
    {   43, 7368,   15279,  6, }, // Pacific/Wallis
    {   44, 6751,   15285,  7, }, // Africa/Abidjan Assamese/Bangla/India
    {   44, 6723,   15292,  5, }, // Africa/Accra
    {   44, 4961,   15297, 12, }, // Africa/Addis_Ababa
    {   44, 7383,   15309, 10, }, // Africa/Algiers
    {   44,   14,   15319,  7, }, // Africa/Asmara
    {   44,   44,   15326,  6, }, // Africa/Bamako
    {   44, 5159,   15332,  6, }, // Africa/Bangui
    {   44, 6709,   15338,  7, }, // Africa/Banjul
    {   44, 7398,   15345,  5, }, // Africa/Bissau
    {   44, 4859,   15350, 12, }, // Africa/Blantyre
    {   44, 5189,   15362, 10, }, // Africa/Brazzaville
    {   44, 4824,   15372, 10, }, // Africa/Bujumbura
    {   44, 2242,   15382,  5, }, // Africa/Cairo
    {   44, 4019,   15387, 11, }, // Africa/Casablanca
    {   44, 7412,   15398,  5, }, // Africa/Ceuta
    {   44, 6736,   15403,  8, }, // Africa/Conakry
    {   44, 6803,   15411,  5, }, // Africa/Dakar
    {   44, 5032,   15416, 12, }, // Africa/Dar_es_Salaam
    {   44, 4945,   15428,  6, }, // Africa/Djibouti
    {   44, 5145,   15434,  6, }, // Africa/Douala
    {   44, 5068,   15440,  9, }, // Africa/El_Aaiun
    {   44, 6816,   15449,  8, }, // Africa/Freetown
    {   44, 4808,   15457,  7, }, // Africa/Gaborone
    {   44, 4917,   15464,  6, }, // Africa/Harare
    {   44, 4381,   15470, 12, }, // Africa/Johannesburg
    {   44, 4401,   15482,  4, }, // Africa/Juba
    {   44, 5053,   15486,  8, }, // Africa/Kampala
    {   44, 4426,   15494,  7, }, // Africa/Khartoum
    {   44, 4875,   15501,  6, }, // Africa/Kigali
    {   44, 5208,   15507,  7, }, // Africa/Kinshasa
    {   44, 4646,   15514,  5, }, // Africa/Lagos
    {   44, 5238,   15519, 10, }, // Africa/Libreville
    {   44, 6832,   15529,  3, }, // Africa/Lome
    {   44, 5113,   15532,  8, }, // Africa/Luanda
    {   44, 4841,   15540, 10, }, // Africa/Lubumbashi
    {   44, 4903,   15550,  6, }, // Africa/Lusaka
    {   44, 5224,   15556,  6, }, // Africa/Malabo
    {   44, 4889,   15562,  6, }, // Africa/Maputo
    {   44, 5099,   15568,  6, }, // Africa/Maseru
    {   44, 5084,   15574,  8, }, // Africa/Mbabane
    {   44, 5015,   15582,  8, }, // Africa/Mogadishu
    {   44, 6997,   15590, 10, }, // Africa/Monrovia
    {   44, 3755,   15600,  7, }, // Africa/Nairobi
    {   44, 5173,   15607,  9, }, // Africa/Ndjamena
    {   44, 5256,   15616,  7, }, // Africa/Niamey
    {   44, 6766,   15623,  7, }, // Africa/Nouakchott
    {   44, 6690,   15630,  7, }, // Africa/Ouagadougou
    {   44, 5127,   15637, 11, }, // Africa/Porto-Novo
    {   44, 4337,   15648,  8, }, // Africa/Sao_Tome
    {   44, 2732,   15656,  8, }, // Africa/Tripoli
    {   44, 6503,   15664,  5, }, // Africa/Tunis
    {   44, 4054,   15669,  8, }, // Africa/Windhoek
    {   44,  132,   15677,  4, }, // America/Adak
    {   44, 3183,   15681,  7, }, // America/Anchorage
    {   44, 5524,   15688,  7, }, // America/Anguilla
    {   44, 5541,   15695,  9, }, // America/Antigua
    {   44, 4456,   15704, 10, }, // America/Araguaina
    {   44,  166,   15714, 12, }, // America/Argentina/Buenos_Aires
    {   44,   91,   15726, 10, }, // America/Argentina/Catamarca
    {   44,  270,   15736,  8, }, // America/Argentina/Cordoba
    {   44,  441,   15744,  5, }, // America/Argentina/Jujuy
    {   44, 7425,   15749,  8, }, // America/Argentina/La_Rioja
    {   44,  565,   15757,  8, }, // America/Argentina/Mendoza
    {   44, 7452,   15765, 12, }, // America/Argentina/Rio_Gallegos
    {   44, 7483,   15777,  6, }, // America/Argentina/Salta
    {   44, 7507,   15783,  9, }, // America/Argentina/San_Juan
    {   44, 5497,   15792,  8, }, // America/Argentina/San_Luis
    {   44, 7534,   15800,  6, }, // America/Argentina/Tucuman
    {   44, 7560,   15806,  8, }, // America/Argentina/Ushuaia
    {   44, 5557,   15814,  5, }, // America/Aruba
    {   44, 4154,   15819,  9, }, // America/Asuncion
    {   44,  237,   15828,  8, }, // America/Atikokan
    {   44, 3581,   15836,  7, }, // America/Bahia
    {   44, 7586,   15843, 17, }, // America/Bahia_Banderas
    {   44, 5571,   15860,  9, }, // America/Barbados
    {   44, 7609,   15869,  5, }, // America/Belem
    {   44, 5309,   15874,  5, }, // America/Belize
    {   44, 7623,   15879, 13, }, // America/Blanc-Sablon
    {   44, 7644,   15892, 11, }, // America/Boa_Vista
    {   44, 4263,   15903,  6, }, // America/Bogota
    {   44, 7662,   15909,  5, }, // America/Boise
    {   44, 7676,   15914, 15, }, // America/Cambridge_Bay
    {   44, 7698,   15929, 15, }, // America/Campo_Grande
    {   44, 3770,   15944,  6, }, // America/Cancun
    {   44, 4596,   15950,  7, }, // America/Caracas
    {   44, 4247,   15957,  6, }, // America/Cayenne
    {   44, 5398,   15963,  6, }, // America/Cayman
    {   44, 3251,   15969,  6, }, // America/Chicago
    {   44, 7719,   15975,  9, }, // America/Chihuahua
    {   44, 7737,   15984, 12, }, // America/Ciudad_Juarez
    {   44, 5324,   15996, 11, }, // America/Costa_Rica
    {   44, 7759,   16007,  8, }, // America/Creston
    {   44, 3693,   16015,  6, }, // America/Cuiaba
    {   44, 5640,   16021,  7, }, // America/Curacao
    {   44, 7775,   16028, 12, }, // America/Danmarkshavn
    {   44, 7796,   16040,  5, }, // America/Dawson
    {   44, 7811,   16045, 11, }, // America/Dawson_Creek
    {   44,  805,   16056,  6, }, // America/Denver
    {   44, 3351,   16062,  8, }, // America/Detroit
    {   44, 5656,   16070,  7, }, // America/Dominica
    {   44,  893,   16077,  7, }, // America/Edmonton
    {   44, 7832,   16084,  7, }, // America/Eirunepe
    {   44, 5343,   16091, 11, }, // America/El_Salvador
    {   44, 7849,   16102, 11, }, // America/Fort_Nelson
    {   44, 7869,   16113, 10, }, // America/Fortaleza
    {   44, 7887,   16123, 12, }, // America/Glace_Bay
    {   44, 6844,   16135, 10, }, // America/Goose_Bay
    {   44, 4514,   16145, 13, }, // America/Grand_Turk
    {   44, 5687,   16158,  8, }, // America/Grenada
    {   44, 5703,   16166,  9, }, // America/Guadeloupe
    {   44, 3662,   16175, 10, }, // America/Guatemala
    {   44, 4743,   16185,  9, }, // America/Guayaquil
    {   44, 6895,   16194,  7, }, // America/Guyana
    {   44, 1923,   16201,  9, }, // America/Halifax
    {   44, 2221,   16210,  6, }, // America/Havana
    {   44, 7905,   16216, 12, }, // America/Hermosillo
    {   44,  348,   16228, 15, }, // America/Indiana/Indianapolis
    {   44,  481,   16243, 16, }, // America/Indiana/Knox
    {   44, 7924,   16259, 19, }, // America/Indiana/Marengo
    {   44, 7948,   16278, 24, }, // America/Indiana/Petersburg
    {   44, 7975,   16302, 20, }, // America/Indiana/Tell_City
    {   44, 8001,   16322, 16, }, // America/Indiana/Vevay
    {   44, 8023,   16338, 20, }, // America/Indiana/Vincennes
    {   44, 8049,   16358, 19, }, // America/Indiana/Winamac
    {   44, 8073,   16377,  6, }, // America/Inuvik
    {   44,  660,   16383,  7, }, // America/Iqaluit
    {   44, 2665,   16390,  7, }, // America/Jamaica
    {   44, 5282,   16397,  5, }, // America/Juneau
    {   44,  521,   16402,  8, }, // America/Kentucky/Louisville
    {   44, 8088,   16410, 19, }, // America/Kentucky/Monticello
    {   44, 5621,   16429, 13, }, // America/Kralendijk
    {   44, 4278,   16442,  6, }, // America/La_Paz
    {   44, 7132,   16448,  4, }, // America/Lima
    {   44, 3390,   16452, 11, }, // America/Los_Angeles
    {   44, 5849,   16463, 25, }, // America/Lower_Princes
    {   44, 8116,   16488,  5, }, // America/Maceio
    {   44, 8131,   16493,  8, }, // America/Managua
    {   44, 1892,   16501,  5, }, // America/Manaus
    {   44, 5814,   16506,  7, }, // America/Marigot
    {   44, 5722,   16513,  9, }, // America/Martinique
    {   44, 8147,   16522,  9, }, // America/Matamoros
    {   44, 2779,   16531, 10, }, // America/Mazatlan
    {   44, 8165,   16541,  8, }, // America/Menominee
    {   44, 8183,   16549,  6, }, // America/Merida
    {   44, 8198,   16555, 12, }, // America/Metlakatla
    {   44, 2811,   16567, 13, }, // America/Mexico_City
    {   44, 4293,   16580,  9, }, // America/Miquelon
    {   44, 8217,   16589,  7, }, // America/Moncton
    {   44, 8233,   16596,  9, }, // America/Monterrey
    {   44, 4000,   16605, 11, }, // America/Montevideo
    {   44, 5741,   16616,  9, }, // America/Montserrat
    {   44, 5383,   16625,  5, }, // America/Nassau
    {   44, 3294,   16630, 10, }, // America/New_York
    {   44, 8251,   16543,  3, }, // America/Nome
    {   44, 1834,   16640,  9, }, // America/Noronha
    {   44, 8264,   16649, 20, }, // America/North_Dakota/Beulah
    {   44, 8292,   16669, 21, }, // America/North_Dakota/Center
    {   44, 8320,   16690, 23, }, // America/North_Dakota/New_Salem
    {   44,  393,   16713,  3, }, // America/Nuuk
    {   44, 8351,   16716,  8, }, // America/Ojinaga
    {   44, 5413,   16724,  6, }, // America/Panama
    {   44, 6106,   16730, 10, }, // America/Paramaribo
    {   44, 3224,   16740,  7, }, // America/Phoenix
    {   44, 3847,   16747, 15, }, // America/Port-au-Prince
    {   44, 5871,   16762, 15, }, // America/Port_of_Spain
    {   44, 8367,   16777, 14, }, // America/Porto_Velho
    {   44, 5760,   16791, 13, }, // America/Puerto_Rico
    {   44, 3932,   16804, 13, }, // America/Punta_Arenas
    {   44, 8387,   16817, 12, }, // America/Rankin_Inlet
    {   44, 8408,   16829,  6, }, // America/Recife
    {   44, 1979,   16835,  6, }, // America/Regina
    {   44, 8423,   16841,  8, }, // America/Resolute
    {   44,  695,   16849, 12, }, // America/Rio_Branco
    {   44, 8440,   16861,  9, }, // America/Santarem
    {   44, 2165,   16870, 11, }, // America/Santiago
    {   44, 6044,   16881, 14, }, // America/Santo_Domingo
    {   44, 1862,   16895,  9, }, // America/Sao_Paulo
    {   44, 6862,   16904, 13, }, // America/Scoresbysund
    {   44, 8457,   16917,  8, }, // America/Sitka
    {   44, 8471,   16925, 16, }, // America/St_Barthelemy
    {   44, 2045,   16941, 11, }, // America/St_Johns
    {   44, 5780,   16952, 11, }, // America/St_Kitts
    {   44, 5797,   16963, 14, }, // America/St_Lucia
    {   44,  855,   16977, 11, }, // America/St_Thomas
    {   44, 5830,   16988, 15, }, // America/St_Vincent
    {   44, 8493,   17003, 14, }, // America/Swift_Current
    {   44, 5363,   17017, 12, }, // America/Tegucigalpa
    {   44, 5673,   17029,  6, }, // America/Thule
    {   44,  313,   17035,  8, }, // America/Tijuana
    {   44,  608,   17043,  8, }, // America/Toronto
    {   44, 5605,   17051,  8, }, // America/Tortola
    {   44, 2077,   17059,  8, }, // America/Vancouver
    {   44, 2128,   17067, 10, }, // America/Whitehorse
    {   44,  734,   17077,  7, }, // America/Winnipeg
    {   44, 8515,   17084,  8, }, // America/Yakutat
    {   44, 5918,   17092,  4, }, // Antarctica/Casey
    {   44, 6027,   17096,  5, }, // Antarctica/Davis
    {   44, 6066,   17101, 18, }, // Antarctica/DumontDUrville
    {   44, 8531,   17119, 11, }, // Antarctica/Macquarie
    {   44, 7029,   17130,  5, }, // Antarctica/Mawson
    {   44, 7076,   17135,  9, }, // Antarctica/McMurdo
    {   44, 8552,   17144,  4, }, // Antarctica/Palmer
    {   44, 7189,   17148,  6, }, // Antarctica/Rothera
    {   44, 7258,   17154,  6, }, // Antarctica/Syowa
    {   44, 8570,   17160,  5, }, // Antarctica/Troll
    {   44, 7337,   17165,  6, }, // Antarctica/Vostok
    {   44, 1413,   17171, 12, }, // Arctic/Longyearbyen
    {   44, 5487,   17183,  4, }, // Asia/Aden
    {   44, 5297,   17187,  6, }, // Asia/Almaty
    {   44, 3870,   17193,  6, }, // Asia/Amman
    {   44, 5428,   17199,  6, }, // Asia/Anadyr
    {   44, 5440,   17205, 10, }, // Asia/Aqtau
    {   44, 5270,   17215,  7, }, // Asia/Aqtobe
    {   44,  964,   17222, 11, }, // Asia/Ashgabat
    {   44, 8587,   17233,  5, }, // Asia/Atyrau
    {   44, 3509,   17238,  6, }, // Asia/Baghdad
    {   44, 5451,   17244,  7, }, // Asia/Bahrain
    {   44, 3555,   16019,  4, }, // Asia/Baku
    {   44, 4368,   17251,  5, }, // Asia/Bangkok
    {   44, 3473,   17256,  7, }, // Asia/Barnaul
    {   44, 3988,   17263,  6, }, // Asia/Beirut
    {   44, 3680,   17269, 10, }, // Asia/Bishkek
    {   44, 5906,   17279,  7, }, // Asia/Brunei
    {   44, 4503,   17286,  4, }, // Asia/Chita
    {   44, 5963,   17290,  9, }, // Asia/Choibalsan
    {   44, 4413,   17299,  6, }, // Asia/Colombo
    {   44, 4442,   17305,  9, }, // Asia/Damascus
    {   44, 1060,   17314,  4, }, // Asia/Dhaka
    {   44, 6125,   17318,  4, }, // Asia/Dili
    {   44, 3498,   17322,  5, }, // Asia/Dubai
    {   44, 6092,   17327,  7, }, // Asia/Dushanbe
    {   44, 8599,   17334, 10, }, // Asia/Famagusta
    {   44, 8614,   17344,  4, }, // Asia/Gaza
    {   44, 4697,   17348,  7, }, // Asia/Hebron
    {   44, 1226,   17355, 14, }, // Asia/Ho_Chi_Minh
    {   44, 2576,   17369,  5, }, // Asia/Hong_Kong
    {   44, 4673,   17374,  6, }, // Asia/Hovd
    {   44, 4086,   17380, 10, }, // Asia/Irkutsk
    {   44, 6969,   17390,  8, }, // Asia/Jakarta
    {   44, 6955,   17398,  9, }, // Asia/Jayapura
    {   44, 1257,   17407,  9, }, // Asia/Jerusalem
    {   44, 3462,   17416,  5, }, // Asia/Kabul
    {   44, 4218,   17421,  8, }, // Asia/Kamchatka
    {   44, 4141,   17429,  5, }, // Asia/Karachi
    {   44, 1152,   17434,  9, }, // Asia/Kathmandu
    {   44, 8624,   17443,  8, }, // Asia/Khandyga
    {   44,  992,   17451,  6, }, // Asia/Kolkata
    {   44, 4099,   17457, 15, }, // Asia/Krasnoyarsk
    {   44, 4761,   17472, 12, }, // Asia/Kuala_Lumpur
    {   44, 5893,   17484,  5, }, // Asia/Kuching
    {   44, 5464,   17489,  6, }, // Asia/Kuwait
    {   44, 1178,   17495,  5, }, // Asia/Macau
    {   44, 3919,   17500,  7, }, // Asia/Magadan
    {   44, 1316,   17507,  7, }, // Asia/Makassar
    {   44, 7145,   17514,  6, }, // Asia/Manila
    {   44, 6883,   17520,  7, }, // Asia/Muscat
    {   44, 2446,   17527,  9, }, // Asia/Nicosia
    {   44, 8638,   17536, 14, }, // Asia/Novokuznetsk
    {   44, 4037,   17550, 13, }, // Asia/Novosibirsk
    {   44, 4131,   17563,  5, }, // Asia/Omsk
    {   44, 7108,   17568,  5, }, // Asia/Oral
    {   44, 6924,   17573, 10, }, // Asia/Phnom_Penh
    {   44, 8656,   17583, 11, }, // Asia/Pontianak
    {   44, 4116,   17594,  8, }, // Asia/Pyongyang
    {   44, 5476,   17602,  5, }, // Asia/Qatar
    {   44, 8671,   17607,  8, }, // Asia/Qostanay
    {   44, 4171,   17615, 10, }, // Asia/Qyzylorda
    {   44, 3486,   17625,  6, }, // Asia/Riyadh
    {   44, 4310,   17631,  7, }, // Asia/Sakhalin
    {   44, 7208,   17638, 10, }, // Asia/Samarkand
    {   44, 3116,   17648,  4, }, // Asia/Seoul
    {   44, 1020,   17652,  6, }, // Asia/Shanghai
    {   44, 3137,   17658,  8, }, // Asia/Singapore
    {   44, 4199,   17666, 15, }, // Asia/Srednekolymsk
    {   44, 3100,   17681,  6, }, // Asia/Taipei
    {   44, 4683,   17687,  8, }, // Asia/Tashkent
    {   44, 3817,   17695,  7, }, // Asia/Tbilisi
    {   44, 2638,   17702,  6, }, // Asia/Tehran
    {   44, 1284,   17708,  6, }, // Asia/Thimphu
    {   44, 2687,   17714,  5, }, // Asia/Tokyo
    {   44, 4474,   17719,  8, }, // Asia/Tomsk
    {   44, 1346,   17727,  9, }, // Asia/Ulaanbaatar
    {   44, 1126,   17736,  8, }, // Asia/Urumqi
    {   44, 8685,   17744,  9, }, // Asia/Ust-Nera
    {   44, 6940,   17753, 13, }, // Asia/Vientiane
    {   44, 4612,   17766, 12, }, // Asia/Vladivostok
    {   44, 4730,   17778, 10, }, // Asia/Yakutsk
    {   44, 1202,   17788,  6, }, // Asia/Yangon
    {   44, 3785,   17794, 15, }, // Asia/Yekaterinburg
    {   44, 3649,   17809,  8, }, // Asia/Yerevan
    {   44, 3565,   17817,  6, }, // Atlantic/Azores
    {   44, 5588,   17823,  8, }, // Atlantic/Bermuda
    {   44, 6591,   17831,  6, }, // Atlantic/Canary
    {   44, 3629,   17837, 10, }, // Atlantic/Cape_Verde
    {   44, 1379,   17847,  4, }, // Atlantic/Faroe
    {   44, 8699,   17851,  6, }, // Atlantic/Madeira
    {   44, 2614,   17857,  9, }, // Atlantic/Reykjavik
    {   44, 7235,   17866, 15, }, // Atlantic/South_Georgia
    {   44, 6784,   17881, 13, }, // Atlantic/St_Helena
    {   44, 6607,   17894,  9, }, // Atlantic/Stanley
    {   44, 1654,   17903,  7, }, // Australia/Adelaide
    {   44, 1619,   17910,  8, }, // Australia/Brisbane
    {   44, 1783,   17918, 11, }, // Australia/Broken_Hill
    {   44, 1567,   17929,  6, }, // Australia/Darwin
    {   44, 3539,   17935,  6, }, // Australia/Eucla
    {   44, 1500,   17941,  7, }, // Australia/Hobart
    {   44, 8716,   17948,  8, }, // Australia/Lindeman
    {   44, 1531,   17956,  8, }, // Australia/Lord_Howe
    {   44, 1711,   17964,  8, }, // Australia/Melbourne
    {   44, 1746,   17972,  5, }, // Australia/Perth
    {   44, 1447,   17977,  5, }, // Australia/Sydney
    {   44, 6390,   17982, 10, }, // Europe/Amsterdam
    {   44, 6149,   17992,  7, }, // Europe/Andorra
    {   44, 3522,   17999, 10, }, // Europe/Astrakhan
    {   44, 6577,   18009,  6, }, // Europe/Athens
    {   44, 6285,   18015,  8, }, // Europe/Belgrade
    {   44, 4659,   18023,  7, }, // Europe/Berlin
    {   44, 6437,   18030, 11, }, // Europe/Bratislava
    {   44, 6178,   18041,  8, }, // Europe/Brussels
    {   44, 3830,   18049,  9, }, // Europe/Bucharest
    {   44, 3708,   18058,  9, }, // Europe/Budapest
    {   44, 8751,   18067,  8, }, // Europe/Busingen
    {   44, 2475,   18075,  7, }, // Europe/Chisinau
    {   44, 6238,   18082, 10, }, // Europe/Copenhagen
    {   44, 2260,   18092,  6, }, // Europe/Dublin
    {   44, 6256,   18098, 10, }, // Europe/Gibraltar
    {   44, 8767,   18108,  7, }, // Europe/Guernsey
    {   44, 6561,   18115,  8, }, // Europe/Helsinki
    {   44, 8783,   18123, 10, }, // Europe/Isle_of_Man
    {   44, 1097,   18133,  9, }, // Europe/Istanbul
    {   44, 8802,   18142,  6, }, // Europe/Jersey
    {   44, 3881,   18148, 12, }, // Europe/Kaliningrad
    {   44, 8924,   18160,  5, }, // Europe/Kirov
    {   44, 2419,   18165,  3, }, // Europe/Kyiv
    {   44, 3078,   18168,  5, }, // Europe/Lisbon
    {   44, 6455,   18173, 11, }, // Europe/Ljubljana
    {   44, 2393,   18184,  5, }, // Europe/London
    {   44, 6314,   18189, 12, }, // Europe/Luxembourg
    {   44, 4794,   18201,  7, }, // Europe/Madrid
    {   44, 6346,   18208,  6, }, // Europe/Malta
    {   44, 6531,   18214, 10, }, // Europe/Mariehamn
    {   44, 3595,   18224,  6, }, // Europe/Minsk
    {   44, 6359,   18230,  6, }, // Europe/Monaco
    {   44, 3443,   18236,  5, }, // Europe/Moscow
    {   44, 6407,   18241,  5, }, // Europe/Oslo
    {   44, 4186,   18246,  5, }, // Europe/Paris
    {   44, 6373,   18251,  9, }, // Europe/Podgorica
    {   44, 6224,   18260,  5, }, // Europe/Prague
    {   44, 8816,   18265,  4, }, // Europe/Riga
    {   44, 6273,   17849,  3, }, // Europe/Rome
    {   44, 4233,   18269,  6, }, // Europe/Samara
    {   44, 6419,   18275, 10, }, // Europe/San_Marino
    {   44, 6194,   18285,  8, }, // Europe/Sarajevo
    {   44, 4353,   18293,  7, }, // Europe/Saratov
    {   44, 8828,   18300, 11, }, // Europe/Simferopol
    {   44, 6332,   18311,  8, }, // Europe/Skopje
    {   44, 6548,   18319,  7, }, // Europe/Sofia
    {   44, 6472,   18326,  7, }, // Europe/Stockholm
    {   44, 8846,   18333,  5, }, // Europe/Tallinn
    {   44, 6135,   18338,  6, }, // Europe/Tirane
    {   44, 8861,   18344, 13, }, // Europe/Ulyanovsk
    {   44, 6301,   18357,  5, }, // Europe/Vaduz
    {   44, 6516,   18362,  7, }, // Europe/Vatican
    {   44, 6164,   18369,  7, }, // Europe/Vienna
    {   44, 8878,   18376,  9, }, // Europe/Vilnius
    {   44, 4629,   18385, 11, }, // Europe/Volgograd
    {   44, 3055,   18396,  6, }, // Europe/Warsaw
    {   44, 6210,   18402,  7, }, // Europe/Zagreb
    {   44, 6489,   18409,  5, }, // Europe/Zurich
    {   44, 4980,   18414, 13, }, // Indian/Antananarivo
    {   44, 6910,   18427,  5, }, // Indian/Chagos
    {   44, 5979,   18432, 10, }, // Indian/Christmas
    {   44, 5996,   18442,  5, }, // Indian/Cocos
    {   44, 4931,   18447,  6, }, // Indian/Comoro
    {   44, 6624,   18453,  8, }, // Indian/Kerguelen
    {   44, 7223,   18461,  4, }, // Indian/Mahe
    {   44, 7013,   18465,  8, }, // Indian/Maldives
    {   44, 3971,   18473,  6, }, // Indian/Mauritius
    {   44, 5000,   18479,  9, }, // Indian/Mayotte
    {   44, 7174,   18488,  9, }, // Indian/Reunion
    {   44, 4324,   18497,  6, }, // Pacific/Apia
    {   44,  932,   18503,  7, }, // Pacific/Auckland
    {   44, 3608,   18510, 10, }, // Pacific/Bougainville
    {   44, 2863,   18520,  5, }, // Pacific/Chatham
    {   44, 3022,   18525,  5, }, // Pacific/Chuuk
    {   44, 2201,   18530,  6, }, // Pacific/Easter
    {   44, 7323,   18536,  4, }, // Pacific/Efate
    {   44, 7290,   18540,  7, }, // Pacific/Fakaofo
    {   44, 3804,   18547,  4, }, // Pacific/Fiji
    {   44, 7306,   18551,  8, }, // Pacific/Funafuti
    {   44, 6641,   18559,  9, }, // Pacific/Galapagos
    {   44, 6659,   18568, 10, }, // Pacific/Gambier
    {   44, 3724,   18578, 11, }, // Pacific/Guadalcanal
    {   44, 5935,   16497,  5, }, // Pacific/Guam
    {   44, 2897,   18589,  9, }, // Pacific/Kanton
    {   44, 3900,   18598, 10, }, // Pacific/Kiritimati
    {   44, 6982,   18608,  8, }, // Pacific/Kosrae
    {   44, 2708,   18616,  9, }, // Pacific/Kwajalein
    {   44, 4779,   18625,  6, }, // Pacific/Majuro
    {   44, 3953,   18631,  8, }, // Pacific/Marquesas
    {   44, 8909,   18639,  5, }, // Pacific/Midway
    {   44, 7047,   18644,  4, }, // Pacific/Nauru
    {   44, 7095,   18648,  5, }, // Pacific/Niue
    {   44, 4070,   18653,  6, }, // Pacific/Norfolk
    {   44, 7061,   18659,  7, }, // Pacific/Noumea
    {   44, 2991,   18666,  9, }, // Pacific/Pago_Pago
    {   44, 7118,   18675,  5, }, // Pacific/Palau
    {   44, 7157,   18680,  9, }, // Pacific/Pitcairn
    {   44, 2961,   18689,  6, }, // Pacific/Pohnpei
    {   44, 4709,   18695, 15, }, // Pacific/Port_Moresby
    {   44, 6009,   18710, 10, }, // Pacific/Rarotonga
    {   44, 5948,   18720,  6, }, // Pacific/Saipan
    {   44, 7275,   18726,  6, }, // Pacific/Tahiti
    {   44, 6675,   18732,  6, }, // Pacific/Tarawa
    {   44, 4485,   18738,  8, }, // Pacific/Tongatapu
    {   44, 7355,   18746,  3, }, // Pacific/Wake
    {   44, 7368,   18749,  5, }, // Pacific/Wallis
    {   45, 4961,   18754, 10, }, // Africa/Addis_Ababa Asturian/Latin/Spain
    {   45, 7383,   18764,  5, }, // Africa/Algiers
    {   45,   14,       0,  6, }, // Africa/Asmara
    {   45, 2242,   18769,  8, }, // Africa/Cairo
    {   45, 4945,   18777,  6, }, // Africa/Djibouti
    {   45, 4381,   18783, 13, }, // Africa/Johannesburg
    {   45, 4426,   18796,  6, }, // Africa/Khartoum
    {   45, 6832,   18802,  4, }, // Africa/Lome
    {   45, 5015,   18806,  8, }, // Africa/Mogadishu
    {   45, 5173,   18814,  6, }, // Africa/Ndjamena
    {   45, 5256,   18820,  6, }, // Africa/Niamey
    {   45, 6766,   18826,  7, }, // Africa/Nouakchott
    {   45, 6690,   18833,  8, }, // Africa/Ouagadougou
    {   45, 4337,   18841, 10, }, // Africa/Sao_Tome
    {   45, 2732,   18851,  7, }, // Africa/Tripoli
    {   45, 6503,   18858,  5, }, // Africa/Tunis
    {   45, 5524,   18863,  7, }, // America/Anguilla
    {   45,  270,   18870,  7, }, // America/Argentina/Cordoba
    {   45, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {   45, 4154,      14,  8, }, // America/Asuncion
    {   45,  237,      22,  8, }, // America/Atikokan
    {   45, 3581,      30,  5, }, // America/Bahia
    {   45, 7586,   18884, 14, }, // America/Bahia_Banderas
    {   45, 4263,   18898,  6, }, // America/Bogota
    {   45, 3770,      47,  6, }, // America/Cancun
    {   45, 5398,   18904,  6, }, // America/Cayman
    {   45, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   45, 5640,      66,  7, }, // America/Curacao
    {   45, 5687,   18910,  7, }, // America/Grenada
    {   45, 5703,   18917,  9, }, // America/Guadeloupe
    {   45, 2221,   18926,  9, }, // America/Havana
    {   45,  481,      73, 13, }, // America/Indiana/Knox
    {   45, 7924,      86, 16, }, // America/Indiana/Marengo
    {   45, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   45, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   45, 8001,     139, 14, }, // America/Indiana/Vevay
    {   45, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   45, 8049,     171, 16, }, // America/Indiana/Winamac
    {   45, 2665,   18935,  7, }, // America/Jamaica
    {   45, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   45, 5849,     207, 22, }, // America/Lower_Princes
    {   45, 5722,   18942, 12, }, // America/Martinique
    {   45, 8183,     229,  6, }, // America/Merida
    {   45, 2811,   18954, 15, }, // America/Mexico_City
    {   45, 3294,   18969, 10, }, // America/New_York
    {   45, 8264,   18979, 24, }, // America/North_Dakota/Beulah
    {   45, 8292,   19003, 24, }, // America/North_Dakota/Center
    {   45, 8320,   19027, 29, }, // America/North_Dakota/New_Salem
    {   45,  393,     298,  4, }, // America/Nuuk
    {   45, 5413,   19056,  6, }, // America/Panama
    {   45, 3847,   19062, 15, }, // America/Port-au-Prince
    {   45, 5871,   19077, 13, }, // America/Port_of_Spain
    {   45, 5760,   19090, 11, }, // America/Puerto_Rico
    {   45, 6044,   19101, 13, }, // America/Santo_Domingo
    {   45, 6862,     302, 16, }, // America/Scoresbysund
    {   45, 8471,     318, 14, }, // America/St_Barthelemy
    {   45, 2045,     332, 10, }, // America/St_Johns
    {   45, 5780,     342,  9, }, // America/St_Kitts
    {   45, 5797,     351,  9, }, // America/St_Lucia
    {   45,  855,     360, 10, }, // America/St_Thomas
    {   45, 5830,     370, 11, }, // America/St_Vincent
    {   45, 5605,   19114,  7, }, // America/Tortola
    {   45, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   45, 3870,   19121,  4, }, // Asia/Amman
    {   45, 3509,   19125,  6, }, // Asia/Baghdad
    {   45, 5451,   19131,  7, }, // Asia/Bahrain
    {   45, 3555,   19138,  4, }, // Asia/Baku
    {   45, 5906,   19142,  6, }, // Asia/Brunei
    {   45, 4442,   19148,  7, }, // Asia/Damascus
    {   45, 6092,   19155,  7, }, // Asia/Dushanbe
    {   45, 4697,   19162,  6, }, // Asia/Hebron
    {   45, 1226,   19168, 20, }, // Asia/Ho_Chi_Minh
    {   45, 2576,   19188,  9, }, // Asia/Hong_Kong
    {   45, 6969,   19197,  7, }, // Asia/Jakarta
    {   45, 6955,   19204,  8, }, // Asia/Jayapura
    {   45, 1257,   19212,  9, }, // Asia/Jerusalem
    {   45, 1152,   19221,  8, }, // Asia/Kathmandu
    {   45,  992,   19229,  7, }, // Asia/Kolkata
    {   45, 1178,   19236,  5, }, // Asia/Macau
    {   45, 3919,   19241,  7, }, // Asia/Magadan
    {   45, 6883,   19248,  7, }, // Asia/Muscat
    {   45, 4116,   19255, 10, }, // Asia/Pyongyang
    {   45, 3486,   19265,  5, }, // Asia/Riyadh
    {   45, 4310,   19270,  7, }, // Asia/Sakhalin
    {   45, 7208,   19277, 10, }, // Asia/Samarkand
    {   45, 3116,   19287,  4, }, // Asia/Seoul
    {   45, 3137,   19291,  8, }, // Asia/Singapore
    {   45, 2638,   19299,  6, }, // Asia/Tehran
    {   45, 2687,   19305,  5, }, // Asia/Tokyo
    {   45, 1346,   19310, 10, }, // Asia/Ulaanbaatar
    {   45, 6940,   19320,  8, }, // Asia/Vientiane
    {   45, 1202,   19328,  6, }, // Asia/Yangon
    {   45, 3785,   19334, 14, }, // Asia/Yekaterinburg
    {   45, 3565,   19348, 10, }, // Atlantic/Azores
    {   45, 6591,   19358,  8, }, // Atlantic/Canary
    {   45, 3629,   19366, 10, }, // Atlantic/Cape_Verde
    {   45, 1379,   19376, 12, }, // Atlantic/Faroe
    {   45, 2614,   19388,  9, }, // Atlantic/Reykjavik
    {   45, 7235,   19397, 14, }, // Atlantic/South_Georgia
    {   45, 6784,   19411, 10, }, // Atlantic/St_Helena
    {   45, 1654,   19421,  8, }, // Australia/Adelaide
    {   45, 3522,   19429,  8, }, // Europe/Astrakhan
    {   45, 6577,   19437,  6, }, // Europe/Athens
    {   45, 6285,   19443,  7, }, // Europe/Belgrade
    {   45, 4659,   19450,  6, }, // Europe/Berlin
    {   45, 6178,   19456,  8, }, // Europe/Brussels
    {   45, 3830,   19464,  8, }, // Europe/Bucharest
    {   45, 6238,   19472, 10, }, // Europe/Copenhagen
    {   45, 2260,   19482,  6, }, // Europe/Dublin
    {   45, 6256,   19488,  9, }, // Europe/Gibraltar
    {   45, 6561,   19497,  8, }, // Europe/Helsinki
    {   45, 8783,   19505, 12, }, // Europe/Isle_of_Man
    {   45, 3881,   19517, 11, }, // Europe/Kaliningrad
    {   45, 2419,    6486,  4, }, // Europe/Kyiv
    {   45, 3078,   19528,  6, }, // Europe/Lisbon
    {   45, 6455,   19534,  9, }, // Europe/Ljubljana
    {   45, 2393,   19543,  7, }, // Europe/London
    {   45, 6314,   19550, 10, }, // Europe/Luxembourg
    {   45, 6359,   19560,  6, }, // Europe/Monaco
    {   45, 3443,   19566,  5, }, // Europe/Moscow
    {   45, 4186,   19571,  5, }, // Europe/Paris
    {   45, 6224,   19576,  5, }, // Europe/Prague
    {   45, 6273,   19581,  4, }, // Europe/Rome
    {   45, 6419,   19585,  9, }, // Europe/San_Marino
    {   45, 6548,   19594,  5, }, // Europe/Sofia
    {   45, 6472,   19599,  9, }, // Europe/Stockholm
    {   45, 6135,   19608,  6, }, // Europe/Tirane
    {   45, 6516,   19614, 11, }, // Europe/Vatican
    {   45, 6164,   19625,  5, }, // Europe/Vienna
    {   45, 4629,   19630,  9, }, // Europe/Volgograd
    {   45, 3055,   19639,  8, }, // Europe/Warsaw
    {   45, 3971,   19647,  8, }, // Indian/Mauritius
    {   45, 7174,   19655,  7, }, // Indian/Reunion
    {   45, 3022,     474,  5, }, // Pacific/Chuuk
    {   45, 2201,   19662,  6, }, // Pacific/Easter
    {   45, 3804,   19668,  4, }, // Pacific/Fiji
    {   45, 6641,   19672,  9, }, // Pacific/Galapagos
    {   45, 2929,    4512,  8, }, // Pacific/Honolulu
    {   45, 2897,    4520,  9, }, // Pacific/Kanton
    {   45, 2961,     479,  7, }, // Pacific/Pohnpei
    {   46,   14,       0,  6, }, // Africa/Asmara Asu/Latin/Tanzania
    {   46, 4337,       6,  8, }, // Africa/Sao_Tome
    {   46, 4154,      14,  8, }, // America/Asuncion
    {   46,  237,      22,  8, }, // America/Atikokan
    {   46, 7586,      30, 17, }, // America/Bahia_Banderas
    {   46, 3770,      47,  6, }, // America/Cancun
    {   46, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   46, 5640,      66,  7, }, // America/Curacao
    {   46,  481,      73, 13, }, // America/Indiana/Knox
    {   46, 7924,      86, 16, }, // America/Indiana/Marengo
    {   46, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   46, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   46, 8001,     139, 14, }, // America/Indiana/Vevay
    {   46, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   46, 8049,     171, 16, }, // America/Indiana/Winamac
    {   46, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   46, 5849,     207, 22, }, // America/Lower_Princes
    {   46, 8183,     229,  6, }, // America/Merida
    {   46, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   46, 8292,     255, 20, }, // America/North_Dakota/Center
    {   46, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   46,  393,     298,  4, }, // America/Nuuk
    {   46, 6862,     302, 16, }, // America/Scoresbysund
    {   46, 8471,     318, 14, }, // America/St_Barthelemy
    {   46, 2045,     332, 10, }, // America/St_Johns
    {   46, 5780,     342,  9, }, // America/St_Kitts
    {   46, 5797,     351,  9, }, // America/St_Lucia
    {   46,  855,     360, 10, }, // America/St_Thomas
    {   46, 5830,     370, 11, }, // America/St_Vincent
    {   46, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   46, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   46, 1152,     413,  9, }, // Asia/Kathmandu
    {   46,  992,     422,  7, }, // Asia/Kolkata
    {   46, 1178,     429,  5, }, // Asia/Macau
    {   46, 1202,     442,  6, }, // Asia/Yangon
    {   46, 1379,     448,  5, }, // Atlantic/Faroe
    {   46, 6784,     453, 10, }, // Atlantic/St_Helena
    {   46, 2419,     463,  4, }, // Europe/Kyiv
    {   46, 7174,     467,  7, }, // Indian/Reunion
    {   46, 3022,     474,  5, }, // Pacific/Chuuk
    {   46, 2961,     479,  7, }, // Pacific/Pohnpei
    {   47,   14,       0,  6, }, // Africa/Asmara Atsam/Latin/Nigeria
    {   47, 4337,       6,  8, }, // Africa/Sao_Tome
    {   47, 4154,      14,  8, }, // America/Asuncion
    {   47,  237,      22,  8, }, // America/Atikokan
    {   47, 7586,      30, 17, }, // America/Bahia_Banderas
    {   47, 3770,      47,  6, }, // America/Cancun
    {   47, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   47, 5640,      66,  7, }, // America/Curacao
    {   47,  481,      73, 13, }, // America/Indiana/Knox
    {   47, 7924,      86, 16, }, // America/Indiana/Marengo
    {   47, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   47, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   47, 8001,     139, 14, }, // America/Indiana/Vevay
    {   47, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   47, 8049,     171, 16, }, // America/Indiana/Winamac
    {   47, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   47, 5849,     207, 22, }, // America/Lower_Princes
    {   47, 8183,     229,  6, }, // America/Merida
    {   47, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   47, 8292,     255, 20, }, // America/North_Dakota/Center
    {   47, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   47,  393,     298,  4, }, // America/Nuuk
    {   47, 6862,     302, 16, }, // America/Scoresbysund
    {   47, 8471,     318, 14, }, // America/St_Barthelemy
    {   47, 2045,     332, 10, }, // America/St_Johns
    {   47, 5780,     342,  9, }, // America/St_Kitts
    {   47, 5797,     351,  9, }, // America/St_Lucia
    {   47,  855,     360, 10, }, // America/St_Thomas
    {   47, 5830,     370, 11, }, // America/St_Vincent
    {   47, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   47, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   47, 1152,     413,  9, }, // Asia/Kathmandu
    {   47,  992,     422,  7, }, // Asia/Kolkata
    {   47, 1178,     429,  5, }, // Asia/Macau
    {   47, 1202,     442,  6, }, // Asia/Yangon
    {   47, 1379,     448,  5, }, // Atlantic/Faroe
    {   47, 6784,     453, 10, }, // Atlantic/St_Helena
    {   47, 2419,     463,  4, }, // Europe/Kyiv
    {   47, 7174,     467,  7, }, // Indian/Reunion
    {   47, 3022,     474,  5, }, // Pacific/Chuuk
    {   47, 2961,     479,  7, }, // Pacific/Pohnpei
    {   48, 6751,   19681,  6, }, // Africa/Abidjan Azerbaijani/Latin/Azerbaijan
    {   48, 6723,   19687,  5, }, // Africa/Accra
    {   48, 4961,   19692, 11, }, // Africa/Addis_Ababa
    {   48, 7383,   19703,  8, }, // Africa/Algiers
    {   48,   14,   19711,  6, }, // Africa/Asmara
    {   48, 5159,   19717,  6, }, // Africa/Bangui
    {   48, 6709,   19723,  6, }, // Africa/Banjul
    {   48, 4859,    4588,  7, }, // Africa/Blantyre
    {   48, 5189,   19729,  9, }, // Africa/Brazzaville
    {   48, 2242,   19738,  6, }, // Africa/Cairo
    {   48, 4019,   19744, 10, }, // Africa/Casablanca
    {   48, 7412,   19754,  5, }, // Africa/Ceuta
    {   48, 6736,    4637,  7, }, // Africa/Conakry
    {   48, 5032,   19759, 12, }, // Africa/Dar_es_Salaam
    {   48, 4945,   19771,  6, }, // Africa/Djibouti
    {   48, 5145,   19777,  5, }, // Africa/Douala
    {   48, 5068,   19782,  7, }, // Africa/El_Aaiun
    {   48, 6816,    4670,  7, }, // Africa/Freetown
    {   48, 4808,   19789,  7, }, // Africa/Gaborone
    {   48, 4381,   19796, 11, }, // Africa/Johannesburg
    {   48, 4426,   19807,  6, }, // Africa/Khartoum
    {   48, 4875,   19813,  6, }, // Africa/Kigali
    {   48, 5208,   19819,  7, }, // Africa/Kinshasa
    {   48, 4646,   19826,  5, }, // Africa/Lagos
    {   48, 5238,    4699,  8, }, // Africa/Libreville
    {   48, 4841,   19831,  9, }, // Africa/Lubumbashi
    {   48, 5015,   19840,  8, }, // Africa/Mogadishu
    {   48, 5173,   19848,  7, }, // Africa/Ndjamena
    {   48, 6766,   19855,  8, }, // Africa/Nouakchott
    {   48, 6690,   19863,  7, }, // Africa/Ouagadougou
    {   48, 4337,   19870,  7, }, // Africa/Sao_Tome
    {   48, 4054,   19877,  7, }, // Africa/Windhoek
    {   48, 3183,   19884,  7, }, // America/Anchorage
    {   48, 5524,   19891,  7, }, // America/Anguilla
    {   48, 5541,   19898,  7, }, // America/Antigua
    {   48, 4456,   19905,  9, }, // America/Araguaina
    {   48,  166,   19914, 12, }, // America/Argentina/Buenos_Aires
    {   48,   91,    4788,  9, }, // America/Argentina/Catamarca
    {   48,  270,    4797,  7, }, // America/Argentina/Cordoba
    {   48, 7425,   19926,  5, }, // America/Argentina/La_Rioja
    {   48, 7452,   19931, 12, }, // America/Argentina/Rio_Gallegos
    {   48, 7507,   19943,  8, }, // America/Argentina/San_Juan
    {   48, 7534,    4850,  7, }, // America/Argentina/Tucuman
    {   48, 7560,   19951,  6, }, // America/Argentina/Ushuaia
    {   48, 4154,    4869,  8, }, // America/Asuncion
    {   48,  237,      22,  8, }, // America/Atikokan
    {   48, 3581,   19957,  6, }, // America/Bahia
    {   48, 7586,    3834, 14, }, // America/Bahia_Banderas
    {   48, 5309,    4891,  5, }, // America/Belize
    {   48, 7623,    4897, 12, }, // America/Blanc-Sablon
    {   48, 4263,   19963,  6, }, // America/Bogota
    {   48, 7662,   19969,  5, }, // America/Boise
    {   48, 7676,   19974, 15, }, // America/Cambridge_Bay
    {   48, 7698,   19989, 12, }, // America/Campo_Grande
    {   48, 3770,    4959,  6, }, // America/Cancun
    {   48, 4596,    4965,  7, }, // America/Caracas
    {   48, 4247,   20001,  5, }, // America/Cayenne
    {   48, 5398,   20006,  6, }, // America/Cayman
    {   48, 3251,   20012,  6, }, // America/Chicago
    {   48, 7719,   20018,  8, }, // America/Chihuahua
    {   48, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   48, 5324,   20026, 10, }, // America/Costa_Rica
    {   48, 7759,    5020,  7, }, // America/Creston
    {   48, 3693,   20036,  6, }, // America/Cuiaba
    {   48, 5640,    5033,  7, }, // America/Curacao
    {   48, 7775,   20042, 11, }, // America/Danmarkshavn
    {   48, 7796,    5053,  6, }, // America/Dawson
    {   48, 7811,   20053, 11, }, // America/Dawson_Creek
    {   48, 3351,   20064,  7, }, // America/Detroit
    {   48, 5656,   20071,  8, }, // America/Dominica
    {   48,  893,   20079,  9, }, // America/Edmonton
    {   48, 7832,   20088,  7, }, // America/Eirunepe
    {   48, 5343,    5090,  8, }, // America/El_Salvador
    {   48, 7887,   20095, 13, }, // America/Glace_Bay
    {   48, 6844,   20108, 11, }, // America/Goose_Bay
    {   48, 4514,   20119, 10, }, // America/Grand_Turk
    {   48, 5687,   20129,  7, }, // America/Grenada
    {   48, 5703,   20136,  9, }, // America/Guadeloupe
    {   48, 3662,   20145,  9, }, // America/Guatemala
    {   48, 4743,   20154,  8, }, // America/Guayaquil
    {   48, 6895,   20162,  6, }, // America/Guyana
    {   48, 1923,    5189,  8, }, // America/Halifax
    {   48, 7905,   20168,  9, }, // America/Hermosillo
    {   48,  348,   20177, 12, }, // America/Indiana/Indianapolis
    {   48,  481,   20189,  4, }, // America/Indiana/Knox
    {   48, 7924,   20193,  7, }, // America/Indiana/Marengo
    {   48, 7948,   20200, 10, }, // America/Indiana/Petersburg
    {   48, 7975,     121,  4, }, // America/Indiana/Tell_City
    {   48, 8001,   20210,  5, }, // America/Indiana/Vevay
    {   48, 8023,   20215,  6, }, // America/Indiana/Vincennes
    {   48, 8049,   20221,  7, }, // America/Indiana/Winamac
    {   48, 8073,   20228,  6, }, // America/Inuvik
    {   48,  660,   20234,  7, }, // America/Iqaluit
    {   48, 2665,   20241,  7, }, // America/Jamaica
    {   48, 5282,   20248,  4, }, // America/Juneau
    {   48,  521,   20252,  7, }, // America/Kentucky/Louisville
    {   48, 8088,   20259, 10, }, // America/Kentucky/Monticello
    {   48, 5621,   20269,  9, }, // America/Kralendijk
    {   48, 4278,   20278,  6, }, // America/La_Paz
    {   48, 3390,   20284, 11, }, // America/Los_Angeles
    {   48, 5849,   20295, 20, }, // America/Lower_Princes
    {   48, 8116,   20315,  6, }, // America/Maceio
    {   48, 8131,   20321,  7, }, // America/Managua
    {   48, 5814,   20328,  7, }, // America/Marigot
    {   48, 5722,    5400,  8, }, // America/Martinique
    {   48, 2779,   20335,  8, }, // America/Mazatlan
    {   48, 8165,    5409,  8, }, // America/Menominee
    {   48, 8183,    3899,  6, }, // America/Merida
    {   48, 2811,   20343,  6, }, // America/Mexico_City
    {   48, 4293,    5434,  7, }, // America/Miquelon
    {   48, 8217,    5441,  7, }, // America/Moncton
    {   48, 8233,   20349,  8, }, // America/Monterrey
    {   48, 5741,   20357,  8, }, // America/Montserrat
    {   48, 3294,   20365,  8, }, // America/New_York
    {   48, 8251,   20373,  3, }, // America/Nome
    {   48, 1834,   20376,  7, }, // America/Noronha
    {   48, 8264,   20383, 21, }, // America/North_Dakota/Beulah
    {   48, 8292,   20404, 20, }, // America/North_Dakota/Center
    {   48, 8320,   20424,  9, }, // America/North_Dakota/New_Salem
    {   48,  393,     298,  4, }, // America/Nuuk
    {   48, 8351,   20433,  7, }, // America/Ojinaga
    {   48, 3224,    5568,  6, }, // America/Phoenix
    {   48, 3847,    5574, 12, }, // America/Port-au-Prince
    {   48, 5871,   20440, 12, }, // America/Port_of_Spain
    {   48, 8367,   20452, 11, }, // America/Porto_Velho
    {   48, 5760,   20463, 11, }, // America/Puerto_Rico
    {   48, 8387,   20474, 13, }, // America/Rankin_Inlet
    {   48, 8408,   20487,  5, }, // America/Recife
    {   48, 1979,   20492,  6, }, // America/Regina
    {   48, 8423,   20498,  8, }, // America/Resolute
    {   48,  695,   20506, 10, }, // America/Rio_Branco
    {   48, 2165,   20516,  8, }, // America/Santiago
    {   48, 6044,   20524, 13, }, // America/Santo_Domingo
    {   48, 1862,   20537,  9, }, // America/Sao_Paulo
    {   48, 6862,   20546, 12, }, // America/Scoresbysund
    {   48, 8471,   20558, 14, }, // America/St_Barthelemy
    {   48, 2045,   20572,  9, }, // America/St_Johns
    {   48, 5780,   20581,  8, }, // America/St_Kitts
    {   48, 5797,   20589, 10, }, // America/St_Lucia
    {   48,  855,   20599,  9, }, // America/St_Thomas
    {   48, 5830,   20608, 11, }, // America/St_Vincent
    {   48, 8493,   20619, 12, }, // America/Swift_Current
    {   48, 5363,   20631, 11, }, // America/Tegucigalpa
    {   48, 5673,   20642,  3, }, // America/Thule
    {   48,  313,   20645,  7, }, // America/Tijuana
    {   48, 2077,    5789,  8, }, // America/Vancouver
    {   48, 2128,   20652,  8, }, // America/Whitehorse
    {   48,  734,   20660,  8, }, // America/Winnipeg
    {   48, 5918,   20668,  5, }, // Antarctica/Casey
    {   48, 6027,   20673,  6, }, // Antarctica/Davis
    {   48, 6066,     381, 14, }, // Antarctica/DumontDUrville
    {   48, 8531,   20679,  7, }, // Antarctica/Macquarie
    {   48, 7029,   20686,  6, }, // Antarctica/Mawson
    {   48, 7076,   20692,  9, }, // Antarctica/McMurdo
    {   48, 7258,   20701,  5, }, // Antarctica/Syowa
    {   48, 1413,   20706,  7, }, // Arctic/Longyearbyen
    {   48, 5297,   20713,  6, }, // Asia/Almaty
    {   48, 5428,   20719,  6, }, // Asia/Anadyr
    {   48, 5440,    5906,  5, }, // Asia/Aqtau
    {   48,  964,   20725,  7, }, // Asia/Ashgabat
    {   48, 8587,   20732,  6, }, // Asia/Atyrau
    {   48, 3509,   20738,  6, }, // Asia/Baghdad
    {   48, 5451,   20744,  7, }, // Asia/Bahrain
    {   48, 3555,   20751,  4, }, // Asia/Baku
    {   48, 4368,   20755,  7, }, // Asia/Bangkok
    {   48, 3988,   20762,  6, }, // Asia/Beirut
    {   48, 3680,   20768,  6, }, // Asia/Bishkek
    {   48, 5906,   20774,  6, }, // Asia/Brunei
    {   48, 4503,    5936,  4, }, // Asia/Chita
    {   48, 5963,   20780,  9, }, // Asia/Choibalsan
    {   48, 4413,    5949,  7, }, // Asia/Colombo
    {   48, 4442,   20789,  6, }, // Asia/Damascus
    {   48, 1060,   20795,  5, }, // Asia/Dhaka
    {   48, 3498,   20800,  5, }, // Asia/Dubai
    {   48, 6092,   20805,  7, }, // Asia/Dushanbe
    {   48, 8599,   20812,  9, }, // Asia/Famagusta
    {   48, 8614,   20821,  4, }, // Asia/Gaza
    {   48, 1226,   20825,  9, }, // Asia/Ho_Chi_Minh
    {   48, 2576,   20834,  9, }, // Asia/Hong_Kong
    {   48, 4086,   20843,  7, }, // Asia/Irkutsk
    {   48, 6969,   20850,  7, }, // Asia/Jakarta
    {   48, 6955,   20857,  8, }, // Asia/Jayapura
    {   48, 1257,   20865,  9, }, // Asia/Jerusalem
    {   48, 3462,   20874,  5, }, // Asia/Kabul
    {   48, 4218,   20879,  8, }, // Asia/Kamchatka
    {   48, 4141,    6012,  6, }, // Asia/Karachi
    {   48, 1152,    6018,  8, }, // Asia/Kathmandu
    {   48, 8624,   20887,  7, }, // Asia/Khandyga
    {   48,  992,   20894,  7, }, // Asia/Kolkata
    {   48, 5893,   20901,  6, }, // Asia/Kuching
    {   48, 5464,   20907,  6, }, // Asia/Kuwait
    {   48, 1178,    6064,  5, }, // Asia/Macau
    {   48, 3919,   20913,  7, }, // Asia/Magadan
    {   48, 1316,    6069,  7, }, // Asia/Makassar
    {   48, 6883,    4227,  6, }, // Asia/Muscat
    {   48, 2446,   20920,  8, }, // Asia/Nicosia
    {   48, 6924,   20928,  8, }, // Asia/Phnom_Penh
    {   48, 4116,   20936,  7, }, // Asia/Pyongyang
    {   48, 4171,   20943,  9, }, // Asia/Qyzylorda
    {   48, 3486,   19265,  5, }, // Asia/Riyadh
    {   48, 4310,   20952,  7, }, // Asia/Sakhalin
    {   48, 7208,   20959,  9, }, // Asia/Samarkand
    {   48, 3116,    6131,  4, }, // Asia/Seoul
    {   48, 1020,   20968,  6, }, // Asia/Shanghai
    {   48, 3137,   20974,  8, }, // Asia/Singapore
    {   48, 4199,   20982, 13, }, // Asia/Srednekolymsk
    {   48, 3100,   20995,  6, }, // Asia/Taipei
    {   48, 4683,   21001,  7, }, // Asia/Tashkent
    {   48, 1346,   21008,  9, }, // Asia/Ulaanbaatar
    {   48, 1126,   21017,  6, }, // Asia/Urumqi
    {   48, 6940,   21023,  8, }, // Asia/Vientiane
    {   48, 1202,   21031,  6, }, // Asia/Yangon
    {   48, 3785,   21037, 13, }, // Asia/Yekaterinburg
    {   48, 3565,    6238,  4, }, // Atlantic/Azores
    {   48, 5588,   21050, 14, }, // Atlantic/Bermuda
    {   48, 6591,    4299,  5, }, // Atlantic/Canary
    {   48, 3629,   21064, 10, }, // Atlantic/Cape_Verde
    {   48, 1379,   21074,  5, }, // Atlantic/Faroe
    {   48, 8699,   21079,  7, }, // Atlantic/Madeira
    {   48, 2614,   21086,  9, }, // Atlantic/Reykjavik
    {   48, 7235,   21095, 14, }, // Atlantic/South_Georgia
    {   48, 6784,   21109, 15, }, // Atlantic/St_Helena
    {   48, 6607,    6298,  6, }, // Atlantic/Stanley
    {   48, 1654,   19421,  8, }, // Australia/Adelaide
    {   48, 1619,   21124,  8, }, // Australia/Brisbane
    {   48, 1567,    6331,  6, }, // Australia/Darwin
    {   48, 3539,   21132,  5, }, // Australia/Eucla
    {   48, 1531,   21137,  8, }, // Australia/Lord_Howe
    {   48, 1711,    6359,  7, }, // Australia/Melbourne
    {   48, 1746,   21145,  4, }, // Australia/Perth
    {   48, 1447,   21149,  6, }, // Australia/Sydney
    {   48, 3522,   21155,  9, }, // Europe/Astrakhan
    {   48, 6577,   21164,  5, }, // Europe/Athens
    {   48, 6285,   21169,  7, }, // Europe/Belgrade
    {   48, 6178,   21176,  7, }, // Europe/Brussels
    {   48, 3830,   21183,  8, }, // Europe/Bucharest
    {   48, 3708,   21191,  8, }, // Europe/Budapest
    {   48, 2475,   21199,  8, }, // Europe/Chisinau
    {   48, 6238,    4376, 10, }, // Europe/Copenhagen
    {   48, 6256,   21207, 12, }, // Europe/Gibraltar
    {   48, 8767,   21219,  7, }, // Europe/Guernsey
    {   48, 8783,   21226,  9, }, // Europe/Isle_of_Man
    {   48, 1097,   21235,  8, }, // Europe/Istanbul
    {   48, 8802,   21243,  5, }, // Europe/Jersey
    {   48, 3881,   21248, 11, }, // Europe/Kaliningrad
    {   48, 2419,   21259,  5, }, // Europe/Kyiv
    {   48, 3078,    4400,  8, }, // Europe/Lisbon
    {   48, 6455,   21264,  9, }, // Europe/Ljubljana
    {   48, 6314,   21273, 10, }, // Europe/Luxembourg
    {   48, 6531,   21283,  7, }, // Europe/Mariehamn
    {   48, 6359,    6525,  6, }, // Europe/Monaco
    {   48, 3443,   21290,  6, }, // Europe/Moscow
    {   48, 6373,   21296, 10, }, // Europe/Podgorica
    {   48, 6224,   21306,  5, }, // Europe/Prague
    {   48, 8816,   21311,  4, }, // Europe/Riga
    {   48, 6273,   19581,  4, }, // Europe/Rome
    {   48, 6194,   21315,  8, }, // Europe/Sarajevo
    {   48, 6332,   21323,  6, }, // Europe/Skopje
    {   48, 6548,   21329,  6, }, // Europe/Sofia
    {   48, 6472,    6586,  8, }, // Europe/Stockholm
    {   48, 8846,   21335,  6, }, // Europe/Tallinn
    {   48, 6135,   19608,  6, }, // Europe/Tirane
    {   48, 6301,   21341,  6, }, // Europe/Vaduz
    {   48, 6516,    6614,  7, }, // Europe/Vatican
    {   48, 6164,   21347,  5, }, // Europe/Vienna
    {   48, 8878,   21352,  6, }, // Europe/Vilnius
    {   48, 4629,   21358,  9, }, // Europe/Volgograd
    {   48, 3055,   21367,  7, }, // Europe/Warsaw
    {   48, 6210,   21374,  6, }, // Europe/Zagreb
    {   48, 6489,   21380,  5, }, // Europe/Zurich
    {   48, 6910,   21385,  5, }, // Indian/Chagos
    {   48, 5979,   21390,  5, }, // Indian/Christmas
    {   48, 5996,    4484,  5, }, // Indian/Cocos
    {   48, 4931,   21395,  6, }, // Indian/Comoro
    {   48, 6624,   21401,  8, }, // Indian/Kerguelen
    {   48, 7013,    6662,  6, }, // Indian/Maldives
    {   48, 3971,   21409,  7, }, // Indian/Mauritius
    {   48, 5000,   21416,  5, }, // Indian/Mayotte
    {   48, 7174,     467,  7, }, // Indian/Reunion
    {   48,  932,   21421,  6, }, // Pacific/Auckland
    {   48, 3608,   21427,  8, }, // Pacific/Bougainville
    {   48, 2863,   21435,  5, }, // Pacific/Chatham
    {   48, 3022,   21440,  4, }, // Pacific/Chuuk
    {   48, 2201,   21444,  5, }, // Pacific/Easter
    {   48, 3804,   21449,  4, }, // Pacific/Fiji
    {   48, 6641,   21453,  9, }, // Pacific/Galapagos
    {   48, 6659,   21462,  8, }, // Pacific/Gambier
    {   48, 3724,   21470, 11, }, // Pacific/Guadalcanal
    {   48, 5935,   21481,  4, }, // Pacific/Guam
    {   48, 2929,    4512,  8, }, // Pacific/Honolulu
    {   48, 2897,   21485,  9, }, // Pacific/Kanton
    {   48, 3900,   21494,  8, }, // Pacific/Kiritimati
    {   48, 6982,   21502,  7, }, // Pacific/Kosrae
    {   48, 2708,   21509,  9, }, // Pacific/Kwajalein
    {   48, 4779,   21518,  6, }, // Pacific/Majuro
    {   48, 3953,   21524,  8, }, // Pacific/Marquesas
    {   48, 8909,   21532,  6, }, // Pacific/Midway
    {   48, 2991,   21538,  9, }, // Pacific/Pago_Pago
    {   48, 7157,    6773,  7, }, // Pacific/Pitcairn
    {   48, 2961,     479,  7, }, // Pacific/Pohnpei
    {   48, 4709,   21547, 12, }, // Pacific/Port_Moresby
    {   48, 6009,   21559,  9, }, // Pacific/Rarotonga
    {   48, 5948,   21568,  6, }, // Pacific/Saipan
    {   48, 6675,   21574,  6, }, // Pacific/Tarawa
    {   48, 4485,   21580,  9, }, // Pacific/Tongatapu
    {   48, 7355,   21589,  4, }, // Pacific/Wake
    {   48, 7368,    6818,  6, }, // Pacific/Wallis
    {   49,   14,       0,  6, }, // Africa/Asmara Azerbaijani/Arabic/Iran
    {   49, 4337,       6,  8, }, // Africa/Sao_Tome
    {   49, 4154,      14,  8, }, // America/Asuncion
    {   49, 7586,      30, 17, }, // America/Bahia_Banderas
    {   49, 3770,      47,  6, }, // America/Cancun
    {   49, 5640,      66,  7, }, // America/Curacao
    {   49,  481,      73, 13, }, // America/Indiana/Knox
    {   49, 7924,      86, 16, }, // America/Indiana/Marengo
    {   49, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   49, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   49, 8001,     139, 14, }, // America/Indiana/Vevay
    {   49, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   49, 8049,     171, 16, }, // America/Indiana/Winamac
    {   49, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   49, 5849,     207, 22, }, // America/Lower_Princes
    {   49, 8183,     229,  6, }, // America/Merida
    {   49, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   49, 8292,     255, 20, }, // America/North_Dakota/Center
    {   49, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   49, 6862,     302, 16, }, // America/Scoresbysund
    {   49, 8471,     318, 14, }, // America/St_Barthelemy
    {   49, 2045,     332, 10, }, // America/St_Johns
    {   49, 5780,     342,  9, }, // America/St_Kitts
    {   49, 5797,     351,  9, }, // America/St_Lucia
    {   49,  855,     360, 10, }, // America/St_Thomas
    {   49, 5830,     370, 11, }, // America/St_Vincent
    {   49, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   49, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   49, 1152,     413,  9, }, // Asia/Kathmandu
    {   49,  992,     422,  7, }, // Asia/Kolkata
    {   49, 1178,     429,  5, }, // Asia/Macau
    {   49, 1202,     442,  6, }, // Asia/Yangon
    {   49, 1379,     448,  5, }, // Atlantic/Faroe
    {   49, 6784,     453, 10, }, // Atlantic/St_Helena
    {   49, 2419,     463,  4, }, // Europe/Kyiv
    {   49, 3022,     474,  5, }, // Pacific/Chuuk
    {   52,   14,       0,  6, }, // Africa/Asmara Azerbaijani/Cyrillic/Azerbaijan
    {   52, 4337,       6,  8, }, // Africa/Sao_Tome
    {   52, 4154,      14,  8, }, // America/Asuncion
    {   52, 7586,      30, 17, }, // America/Bahia_Banderas
    {   52, 3770,      47,  6, }, // America/Cancun
    {   52, 5640,      66,  7, }, // America/Curacao
    {   52,  481,      73, 13, }, // America/Indiana/Knox
    {   52, 7924,      86, 16, }, // America/Indiana/Marengo
    {   52, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   52, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   52, 8001,     139, 14, }, // America/Indiana/Vevay
    {   52, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   52, 8049,     171, 16, }, // America/Indiana/Winamac
    {   52, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   52, 5849,     207, 22, }, // America/Lower_Princes
    {   52, 8183,     229,  6, }, // America/Merida
    {   52, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   52, 8292,     255, 20, }, // America/North_Dakota/Center
    {   52, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   52, 6862,     302, 16, }, // America/Scoresbysund
    {   52, 8471,     318, 14, }, // America/St_Barthelemy
    {   52, 2045,     332, 10, }, // America/St_Johns
    {   52, 5780,     342,  9, }, // America/St_Kitts
    {   52, 5797,     351,  9, }, // America/St_Lucia
    {   52,  855,     360, 10, }, // America/St_Thomas
    {   52, 5830,     370, 11, }, // America/St_Vincent
    {   52, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   52, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   52, 1152,     413,  9, }, // Asia/Kathmandu
    {   52,  992,     422,  7, }, // Asia/Kolkata
    {   52, 1178,     429,  5, }, // Asia/Macau
    {   52, 1202,     442,  6, }, // Asia/Yangon
    {   52, 1379,     448,  5, }, // Atlantic/Faroe
    {   52, 6784,     453, 10, }, // Atlantic/St_Helena
    {   52, 2419,     463,  4, }, // Europe/Kyiv
    {   52, 3022,     474,  5, }, // Pacific/Chuuk
    {   53,   14,       0,  6, }, // Africa/Asmara Bafia/Latin/Cameroon
    {   53, 4337,       6,  8, }, // Africa/Sao_Tome
    {   53, 4154,      14,  8, }, // America/Asuncion
    {   53,  237,      22,  8, }, // America/Atikokan
    {   53, 7586,      30, 17, }, // America/Bahia_Banderas
    {   53, 3770,      47,  6, }, // America/Cancun
    {   53, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   53, 5640,      66,  7, }, // America/Curacao
    {   53,  481,      73, 13, }, // America/Indiana/Knox
    {   53, 7924,      86, 16, }, // America/Indiana/Marengo
    {   53, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   53, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   53, 8001,     139, 14, }, // America/Indiana/Vevay
    {   53, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   53, 8049,     171, 16, }, // America/Indiana/Winamac
    {   53, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   53, 5849,     207, 22, }, // America/Lower_Princes
    {   53, 8183,     229,  6, }, // America/Merida
    {   53, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   53, 8292,     255, 20, }, // America/North_Dakota/Center
    {   53, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   53,  393,     298,  4, }, // America/Nuuk
    {   53, 6862,     302, 16, }, // America/Scoresbysund
    {   53, 8471,     318, 14, }, // America/St_Barthelemy
    {   53, 2045,     332, 10, }, // America/St_Johns
    {   53, 5780,     342,  9, }, // America/St_Kitts
    {   53, 5797,     351,  9, }, // America/St_Lucia
    {   53,  855,     360, 10, }, // America/St_Thomas
    {   53, 5830,     370, 11, }, // America/St_Vincent
    {   53, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   53, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   53, 1152,     413,  9, }, // Asia/Kathmandu
    {   53,  992,     422,  7, }, // Asia/Kolkata
    {   53, 1178,     429,  5, }, // Asia/Macau
    {   53, 1202,     442,  6, }, // Asia/Yangon
    {   53, 1379,     448,  5, }, // Atlantic/Faroe
    {   53, 6784,     453, 10, }, // Atlantic/St_Helena
    {   53, 2419,     463,  4, }, // Europe/Kyiv
    {   53, 7174,     467,  7, }, // Indian/Reunion
    {   53, 3022,     474,  5, }, // Pacific/Chuuk
    {   53, 2961,     479,  7, }, // Pacific/Pohnpei
    {   54,   14,       0,  6, }, // Africa/Asmara Bambara/Latin/Mali
    {   54, 4337,       6,  8, }, // Africa/Sao_Tome
    {   54, 4154,      14,  8, }, // America/Asuncion
    {   54,  237,      22,  8, }, // America/Atikokan
    {   54, 7586,      30, 17, }, // America/Bahia_Banderas
    {   54, 3770,      47,  6, }, // America/Cancun
    {   54, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   54, 5640,      66,  7, }, // America/Curacao
    {   54,  481,      73, 13, }, // America/Indiana/Knox
    {   54, 7924,      86, 16, }, // America/Indiana/Marengo
    {   54, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   54, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   54, 8001,     139, 14, }, // America/Indiana/Vevay
    {   54, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   54, 8049,     171, 16, }, // America/Indiana/Winamac
    {   54, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   54, 5849,     207, 22, }, // America/Lower_Princes
    {   54, 8183,     229,  6, }, // America/Merida
    {   54, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   54, 8292,     255, 20, }, // America/North_Dakota/Center
    {   54, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   54,  393,     298,  4, }, // America/Nuuk
    {   54, 6862,     302, 16, }, // America/Scoresbysund
    {   54, 8471,     318, 14, }, // America/St_Barthelemy
    {   54, 2045,     332, 10, }, // America/St_Johns
    {   54, 5780,     342,  9, }, // America/St_Kitts
    {   54, 5797,     351,  9, }, // America/St_Lucia
    {   54,  855,     360, 10, }, // America/St_Thomas
    {   54, 5830,     370, 11, }, // America/St_Vincent
    {   54, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   54, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   54, 1152,     413,  9, }, // Asia/Kathmandu
    {   54,  992,     422,  7, }, // Asia/Kolkata
    {   54, 1178,     429,  5, }, // Asia/Macau
    {   54, 1202,     442,  6, }, // Asia/Yangon
    {   54, 1379,     448,  5, }, // Atlantic/Faroe
    {   54, 6784,     453, 10, }, // Atlantic/St_Helena
    {   54, 2419,     463,  4, }, // Europe/Kyiv
    {   54, 7174,     467,  7, }, // Indian/Reunion
    {   54, 3022,     474,  5, }, // Pacific/Chuuk
    {   54, 2961,     479,  7, }, // Pacific/Pohnpei
    {   56, 6751,   21593,  7, }, // Africa/Abidjan Bangla/Bangla/Bangladesh
    {   56, 6723,   21600,  5, }, // Africa/Accra
    {   56, 4961,   21605, 12, }, // Africa/Addis_Ababa
    {   56, 7383,   21617, 10, }, // Africa/Algiers
    {   56,   14,   21627, 11, }, // Africa/Asmara
    {   56,   44,   21638,  8, }, // Africa/Bamako
    {   56, 5159,   21646,  7, }, // Africa/Bangui
    {   56, 6709,   21653,  7, }, // Africa/Banjul
    {   56, 7398,   21660,  5, }, // Africa/Bissau
    {   56, 4859,   21665, 13, }, // Africa/Blantyre
    {   56, 5189,   21678, 10, }, // Africa/Brazzaville
    {   56, 4824,   21688, 10, }, // Africa/Bujumbura
    {   56, 2242,   21698,  6, }, // Africa/Cairo
    {   56, 4019,   21704, 12, }, // Africa/Casablanca
    {   56, 7412,   21716,  5, }, // Africa/Ceuta
    {   56, 6736,   21721,  7, }, // Africa/Conakry
    {   56, 6803,   21728,  5, }, // Africa/Dakar
    {   56, 5032,   21733, 12, }, // Africa/Dar_es_Salaam
    {   56, 4945,   15428,  6, }, // Africa/Djibouti
    {   56, 5145,   21745,  7, }, // Africa/Douala
    {   56, 5068,   21752,  8, }, // Africa/El_Aaiun
    {   56, 6816,   21760,  8, }, // Africa/Freetown
    {   56, 4808,   21768,  9, }, // Africa/Gaborone
    {   56, 4917,   21777,  6, }, // Africa/Harare
    {   56, 4381,   21783, 12, }, // Africa/Johannesburg
    {   56, 4401,   15482,  4, }, // Africa/Juba
    {   56, 5053,   21795,  7, }, // Africa/Kampala
    {   56, 4426,   21802,  7, }, // Africa/Khartoum
    {   56, 4875,   21809,  6, }, // Africa/Kigali
    {   56, 5208,   21815,  7, }, // Africa/Kinshasa
    {   56, 4646,   21822,  5, }, // Africa/Lagos
    {   56, 5238,   21827,  9, }, // Africa/Libreville
    {   56, 6832,   21836,  4, }, // Africa/Lome
    {   56, 5113,   21840,  9, }, // Africa/Luanda
    {   56, 4841,   21849, 10, }, // Africa/Lubumbashi
    {   56, 4903,   21859,  6, }, // Africa/Lusaka
    {   56, 5224,   15556,  6, }, // Africa/Malabo
    {   56, 4889,   21865,  6, }, // Africa/Maputo
    {   56, 5099,   21871,  7, }, // Africa/Maseru
    {   56, 5084,   21878,  7, }, // Africa/Mbabane
    {   56, 5015,   21885,  9, }, // Africa/Mogadishu
    {   56, 6997,   21894,  9, }, // Africa/Monrovia
    {   56, 3755,   21903,  7, }, // Africa/Nairobi
    {   56, 5173,   21910,  7, }, // Africa/Ndjamena
    {   56, 5256,   21917,  7, }, // Africa/Niamey
    {   56, 6766,   21924,  8, }, // Africa/Nouakchott
    {   56, 6690,   21932, 11, }, // Africa/Ouagadougou
    {   56, 5127,   21943, 11, }, // Africa/Porto-Novo
    {   56, 4337,   21954,  7, }, // Africa/Sao_Tome
    {   56, 2732,   21961,  8, }, // Africa/Tripoli
    {   56, 6503,   21969,  6, }, // Africa/Tunis
    {   56, 4054,   21975,  6, }, // Africa/Windhoek
    {   56,  132,   21981,  3, }, // America/Adak
    {   56, 3183,   21984,  7, }, // America/Anchorage
    {   56, 5524,   21991, 11, }, // America/Anguilla
    {   56, 5541,   22002, 13, }, // America/Antigua
    {   56, 4456,   22015, 11, }, // America/Araguaina
    {   56,  166,   22026, 16, }, // America/Argentina/Buenos_Aires
    {   56,   91,   22042, 12, }, // America/Argentina/Catamarca
    {   56,  270,   22054,  7, }, // America/Argentina/Cordoba
    {   56,  441,   15744,  5, }, // America/Argentina/Jujuy
    {   56, 7425,   22061,  8, }, // America/Argentina/La_Rioja
    {   56,  565,   22069,  8, }, // America/Argentina/Mendoza
    {   56, 7452,   22077, 12, }, // America/Argentina/Rio_Gallegos
    {   56, 7483,   22089,  7, }, // America/Argentina/Salta
    {   56, 7507,   22096, 10, }, // America/Argentina/San_Juan
    {   56, 5497,   22106,  8, }, // America/Argentina/San_Luis
    {   56, 7534,   22114,  7, }, // America/Argentina/Tucuman
    {   56, 7560,   22121, 10, }, // America/Argentina/Ushuaia
    {   56, 5557,   22131,  5, }, // America/Aruba
    {   56, 4154,   22136,  9, }, // America/Asuncion
    {   56,  237,   15828,  8, }, // America/Atikokan
    {   56, 3581,   15836,  7, }, // America/Bahia
    {   56, 7586,   22145, 17, }, // America/Bahia_Banderas
    {   56, 5571,   22162,  9, }, // America/Barbados
    {   56, 7609,   15869,  5, }, // America/Belem
    {   56, 5309,   15874,  5, }, // America/Belize
    {   56, 7623,   22171, 15, }, // America/Blanc-Sablon
    {   56, 7644,   22186, 12, }, // America/Boa_Vista
    {   56, 4263,   15903,  6, }, // America/Bogota
    {   56, 7662,   22198,  5, }, // America/Boise
    {   56, 7676,   22203, 11, }, // America/Cambridge_Bay
    {   56, 7698,   22214, 15, }, // America/Campo_Grande
    {   56, 3770,   22229,  8, }, // America/Cancun
    {   56, 4596,   22237,  9, }, // America/Caracas
    {   56, 4247,   22246,  5, }, // America/Cayenne
    {   56, 5398,   22251,  5, }, // America/Cayman
    {   56, 3251,   22256,  6, }, // America/Chicago
    {   56, 7719,   22262, 12, }, // America/Chihuahua
    {   56, 7737,   22274, 15, }, // America/Ciudad_Juarez
    {   56, 5324,   22289, 10, }, // America/Costa_Rica
    {   56, 7759,   22299,  9, }, // America/Creston
    {   56, 3693,   22308,  8, }, // America/Cuiaba
    {   56, 5640,   22316,  7, }, // America/Curacao
    {   56, 7775,   22323, 14, }, // America/Danmarkshavn
    {   56, 7796,   22337,  4, }, // America/Dawson
    {   56, 7811,   22341, 10, }, // America/Dawson_Creek
    {   56,  805,   22351,  6, }, // America/Denver
    {   56, 3351,   22357,  8, }, // America/Detroit
    {   56, 5656,   22365,  8, }, // America/Dominica
    {   56,  893,   22373,  8, }, // America/Edmonton
    {   56, 7832,   22381,  8, }, // America/Eirunepe
    {   56, 5343,   22389, 11, }, // America/El_Salvador
    {   56, 7849,   22400, 11, }, // America/Fort_Nelson
    {   56, 7869,   22411, 10, }, // America/Fortaleza
    {   56, 7887,   22421,  8, }, // America/Glace_Bay
    {   56, 6844,   22429,  6, }, // America/Goose_Bay
    {   56, 4514,   22435, 15, }, // America/Grand_Turk
    {   56, 5687,   22450,  8, }, // America/Grenada
    {   56, 5703,   22458, 10, }, // America/Guadeloupe
    {   56, 3662,   22468, 11, }, // America/Guatemala
    {   56, 4743,   22479, 12, }, // America/Guayaquil
    {   56, 6895,   16194,  7, }, // America/Guyana
    {   56, 1923,   22491, 13, }, // America/Halifax
    {   56, 2221,   16210,  6, }, // America/Havana
    {   56, 7905,   22504, 11, }, // America/Hermosillo
    {   56,  348,   22515, 15, }, // America/Indiana/Indianapolis
    {   56,  481,   22530, 16, }, // America/Indiana/Knox
    {   56, 7924,   22546, 20, }, // America/Indiana/Marengo
    {   56, 7948,   22566, 25, }, // America/Indiana/Petersburg
    {   56, 7975,   22591, 19, }, // America/Indiana/Tell_City
    {   56, 8001,   22610, 18, }, // America/Indiana/Vevay
    {   56, 8023,   22628, 22, }, // America/Indiana/Vincennes
    {   56, 8049,   22650, 21, }, // America/Indiana/Winamac
    {   56, 8073,   22671,  8, }, // America/Inuvik
    {   56,  660,   22679, 10, }, // America/Iqaluit
    {   56, 2665,   16390,  7, }, // America/Jamaica
    {   56, 5282,   22689,  4, }, // America/Juneau
    {   56,  521,   22693,  7, }, // America/Kentucky/Louisville
    {   56, 8088,   22700, 19, }, // America/Kentucky/Monticello
    {   56, 5621,   22719, 13, }, // America/Kralendijk
    {   56, 4278,   16442,  6, }, // America/La_Paz
    {   56, 7132,   16448,  4, }, // America/Lima
    {   56, 3390,   22732, 14, }, // America/Los_Angeles
    {   56, 5849,   22746, 27, }, // America/Lower_Princes
    {   56, 8116,   22773,  7, }, // America/Maceio
    {   56, 8131,   22780,  9, }, // America/Managua
    {   56, 1892,   22789,  6, }, // America/Manaus
    {   56, 5814,   22795,  6, }, // America/Marigot
    {   56, 5722,   22801,  8, }, // America/Martinique
    {   56, 8147,   22809,  8, }, // America/Matamoros
    {   56, 2779,   22817,  8, }, // America/Mazatlan
    {   56, 8165,   22825,  8, }, // America/Menominee
    {   56, 8183,   22833,  6, }, // America/Merida
    {   56, 8198,   22839, 10, }, // America/Metlakatla
    {   56, 2811,   22849, 13, }, // America/Mexico_City
    {   56, 4293,   22862,  6, }, // America/Miquelon
    {   56, 8217,   22868,  7, }, // America/Moncton
    {   56, 8233,   22875,  7, }, // America/Monterrey
    {   56, 4000,   22882, 10, }, // America/Montevideo
    {   56, 5741,   22892,  9, }, // America/Montserrat
    {   56, 5383,   22901,  5, }, // America/Nassau
    {   56, 3294,   22906,  9, }, // America/New_York
    {   56, 8251,   16543,  3, }, // America/Nome
    {   56, 1834,   22915,  8, }, // America/Noronha
    {   56, 8264,   22923, 19, }, // America/North_Dakota/Beulah
    {   56, 8292,   22942, 18, }, // America/North_Dakota/Center
    {   56, 8320,   22960, 23, }, // America/North_Dakota/New_Salem
    {   56,  393,   16713,  3, }, // America/Nuuk
    {   56, 8351,   22983,  7, }, // America/Ojinaga
    {   56, 5413,   16724,  6, }, // America/Panama
    {   56, 6106,   22990, 12, }, // America/Paramaribo
    {   56, 3224,   16740,  7, }, // America/Phoenix
    {   56, 3847,   23002, 16, }, // America/Port-au-Prince
    {   56, 5871,   23018, 14, }, // America/Port_of_Spain
    {   56, 8367,   23032, 14, }, // America/Porto_Velho
    {   56, 5760,   23046, 14, }, // America/Puerto_Rico
    {   56, 3932,   23060, 13, }, // America/Punta_Arenas
    {   56, 8387,   23073, 16, }, // America/Rankin_Inlet
    {   56, 8408,   23089,  6, }, // America/Recife
    {   56, 1979,   23095,  6, }, // America/Regina
    {   56, 8423,   23101,  7, }, // America/Resolute
    {   56,  695,   23108, 12, }, // America/Rio_Branco
    {   56, 8440,   23120,  8, }, // America/Santarem
    {   56, 2165,   23128, 11, }, // America/Santiago
    {   56, 6044,   23139, 16, }, // America/Santo_Domingo
    {   56, 1862,   23155,  9, }, // America/Sao_Paulo
    {   56, 6862,   23164, 16, }, // America/Scoresbysund
    {   56, 8457,   23180,  5, }, // America/Sitka
    {   56, 8471,   23185, 16, }, // America/St_Barthelemy
    {   56, 2045,   23201, 10, }, // America/St_Johns
    {   56, 5780,   23211, 11, }, // America/St_Kitts
    {   56, 5797,   23222, 14, }, // America/St_Lucia
    {   56,  855,   23236, 11, }, // America/St_Thomas
    {   56, 5830,   23247, 15, }, // America/St_Vincent
    {   56, 8493,   23262, 14, }, // America/Swift_Current
    {   56, 5363,   23276, 11, }, // America/Tegucigalpa
    {   56, 5673,   23287,  4, }, // America/Thule
    {   56,  313,   23291,  9, }, // America/Tijuana
    {   56,  608,   23300,  7, }, // America/Toronto
    {   56, 5605,   23307,  6, }, // America/Tortola
    {   56, 2077,   23313, 11, }, // America/Vancouver
    {   56, 2128,   23324, 11, }, // America/Whitehorse
    {   56,  734,   23335,  7, }, // America/Winnipeg
    {   56, 8515,   23342,  9, }, // America/Yakutat
    {   56, 5918,   23351,  5, }, // Antarctica/Casey
    {   56, 6027,   23356,  5, }, // Antarctica/Davis
    {   56, 6066,   23361, 15, }, // Antarctica/DumontDUrville
    {   56, 8531,   23376, 14, }, // Antarctica/Macquarie
    {   56, 7029,   23390,  4, }, // Antarctica/Mawson
    {   56, 7076,   23394, 11, }, // Antarctica/McMurdo
    {   56, 8552,   23405,  6, }, // Antarctica/Palmer
    {   56, 7189,   23411,  5, }, // Antarctica/Rothera
    {   56, 7258,   23416,  6, }, // Antarctica/Syowa
    {   56, 8570,   23422,  4, }, // Antarctica/Troll
    {   56, 7337,   23426,  6, }, // Antarctica/Vostok
    {   56, 1413,   23432, 14, }, // Arctic/Longyearbyen
    {   56, 5487,   23446,  5, }, // Asia/Aden
    {   56, 5297,   17187,  6, }, // Asia/Almaty
    {   56, 3870,   17193,  6, }, // Asia/Amman
    {   56, 5428,   23451,  9, }, // Asia/Anadyr
    {   56, 5440,   23460,  5, }, // Asia/Aqtau
    {   56, 5270,   23465,  6, }, // Asia/Aqtobe
    {   56,  964,   23471,  7, }, // Asia/Ashgabat
    {   56, 8587,   23478,  6, }, // Asia/Atyrau
    {   56, 3509,   17238,  6, }, // Asia/Baghdad
    {   56, 5451,   23484,  7, }, // Asia/Bahrain
    {   56, 3555,   16019,  4, }, // Asia/Baku
    {   56, 4368,   23491,  7, }, // Asia/Bangkok
    {   56, 3473,   23498,  7, }, // Asia/Barnaul
    {   56, 3988,   23505,  6, }, // Asia/Beirut
    {   56, 3680,   23511,  6, }, // Asia/Bishkek
    {   56, 5906,   23517,  7, }, // Asia/Brunei
    {   56, 4503,   23524,  4, }, // Asia/Chita
    {   56, 5963,   23528, 13, }, // Asia/Choibalsan
    {   56, 4413,   17299,  6, }, // Asia/Colombo
    {   56, 4442,   23541,  9, }, // Asia/Damascus
    {   56, 1060,   17314,  4, }, // Asia/Dhaka
    {   56, 6125,   23550,  4, }, // Asia/Dili
    {   56, 3498,   23554,  5, }, // Asia/Dubai
    {   56, 6092,   23559,  7, }, // Asia/Dushanbe
    {   56, 8599,   23566, 10, }, // Asia/Famagusta
    {   56, 8614,   17344,  4, }, // Asia/Gaza
    {   56, 4697,   23576,  7, }, // Asia/Hebron
    {   56, 1226,   23583, 13, }, // Asia/Ho_Chi_Minh
    {   56, 2576,   17369,  5, }, // Asia/Hong_Kong
    {   56, 4673,   23596,  5, }, // Asia/Hovd
    {   56, 4086,   23601,  8, }, // Asia/Irkutsk
    {   56, 6969,   23609,  8, }, // Asia/Jakarta
    {   56, 6955,   23617,  8, }, // Asia/Jayapura
    {   56, 1257,   23625,  9, }, // Asia/Jerusalem
    {   56, 3462,   17416,  5, }, // Asia/Kabul
    {   56, 4218,   23634,  8, }, // Asia/Kamchatka
    {   56, 4141,   23642,  5, }, // Asia/Karachi
    {   56, 1152,   23647,  9, }, // Asia/Kathmandu
    {   56, 8624,   23656, 10, }, // Asia/Khandyga
    {   56,  992,   23666,  7, }, // Asia/Kolkata
    {   56, 4099,   23673, 16, }, // Asia/Krasnoyarsk
    {   56, 4761,   23689, 13, }, // Asia/Kuala_Lumpur
    {   56, 5893,   17484,  5, }, // Asia/Kuching
    {   56, 5464,   23702,  6, }, // Asia/Kuwait
    {   56, 1178,   23708,  7, }, // Asia/Macau
    {   56, 3919,   23715,  9, }, // Asia/Magadan
    {   56, 1316,   23724,  8, }, // Asia/Makassar
    {   56, 7145,   23732,  8, }, // Asia/Manila
    {   56, 6883,   23740,  5, }, // Asia/Muscat
    {   56, 2446,   23745,  9, }, // Asia/Nicosia
    {   56, 8638,   23754, 12, }, // Asia/Novokuznetsk
    {   56, 4037,   23766, 12, }, // Asia/Novosibirsk
    {   56, 4131,   23778,  6, }, // Asia/Omsk
    {   56, 7108,   23784,  4, }, // Asia/Oral
    {   56, 6924,   23788,  6, }, // Asia/Phnom_Penh
    {   56, 8656,   23794, 13, }, // Asia/Pontianak
    {   56, 4116,   23807,  9, }, // Asia/Pyongyang
    {   56, 5476,   23816,  5, }, // Asia/Qatar
    {   56, 8671,   23821,  9, }, // Asia/Qostanay
    {   56, 4171,   23830,  9, }, // Asia/Qyzylorda
    {   56, 3486,   23839,  6, }, // Asia/Riyadh
    {   56, 4310,   23845,  7, }, // Asia/Sakhalin
    {   56, 7208,   23852,  7, }, // Asia/Samarkand
    {   56, 3116,   23859,  4, }, // Asia/Seoul
    {   56, 1020,   23863,  6, }, // Asia/Shanghai
    {   56, 3137,   23869,  9, }, // Asia/Singapore
    {   56, 4199,   23878, 16, }, // Asia/Srednekolymsk
    {   56, 3100,   23894,  6, }, // Asia/Taipei
    {   56, 4683,   23900,  7, }, // Asia/Tashkent
    {   56, 3817,   23907,  8, }, // Asia/Tbilisi
    {   56, 2638,   23915,  7, }, // Asia/Tehran
    {   56, 1284,   17708,  6, }, // Asia/Thimphu
    {   56, 2687,   23922,  5, }, // Asia/Tokyo
    {   56, 4474,   23927,  6, }, // Asia/Tomsk
    {   56, 1346,   23933,  9, }, // Asia/Ulaanbaatar
    {   56, 1126,   23942,  6, }, // Asia/Urumqi
    {   56, 8685,   23948, 10, }, // Asia/Ust-Nera
    {   56, 6940,   23958, 12, }, // Asia/Vientiane
    {   56, 4612,   23970, 12, }, // Asia/Vladivostok
    {   56, 4730,   23982, 10, }, // Asia/Yakutsk
    {   56, 1202,   23992,  7, }, // Asia/Yangon
    {   56, 3785,   23999, 16, }, // Asia/Yekaterinburg
    {   56, 3649,   24015,  9, }, // Asia/Yerevan
    {   56, 3565,   24024,  6, }, // Atlantic/Azores
    {   56, 5588,   24030,  7, }, // Atlantic/Bermuda
    {   56, 6591,   24037,  6, }, // Atlantic/Canary
    {   56, 3629,   24043,  9, }, // Atlantic/Cape_Verde
    {   56, 1379,   24052,  6, }, // Atlantic/Faroe
    {   56, 8699,   24058,  7, }, // Atlantic/Madeira
    {   56, 2614,   24065,  8, }, // Atlantic/Reykjavik
    {   56, 7235,   24073, 15, }, // Atlantic/South_Georgia
    {   56, 6784,   24088, 13, }, // Atlantic/St_Helena
    {   56, 6607,   24101,  7, }, // Atlantic/Stanley
    {   56, 1654,   24108, 10, }, // Australia/Adelaide
    {   56, 1619,   24118,  8, }, // Australia/Brisbane
    {   56, 1783,   24126, 11, }, // Australia/Broken_Hill
    {   56, 1567,   24137,  6, }, // Australia/Darwin
    {   56, 3539,   17935,  6, }, // Australia/Eucla
    {   56, 1500,   24143,  7, }, // Australia/Hobart
    {   56, 8716,   24150, 10, }, // Australia/Lindeman
    {   56, 1531,   24160,  8, }, // Australia/Lord_Howe
    {   56, 1711,   24168,  8, }, // Australia/Melbourne
    {   56, 1746,   24176,  5, }, // Australia/Perth
    {   56, 1447,   24181,  5, }, // Australia/Sydney
    {   56, 6390,   24186, 10, }, // Europe/Amsterdam
    {   56, 6149,   24196, 10, }, // Europe/Andorra
    {   56, 3522,   24206,  9, }, // Europe/Astrakhan
    {   56, 6577,   18009,  6, }, // Europe/Athens
    {   56, 6285,   24215,  8, }, // Europe/Belgrade
    {   56, 4659,   24223,  7, }, // Europe/Berlin
    {   56, 6437,   24230, 12, }, // Europe/Bratislava
    {   56, 6178,   24242,  8, }, // Europe/Brussels
    {   56, 3830,   24250,  9, }, // Europe/Bucharest
    {   56, 3708,   24259,  9, }, // Europe/Budapest
    {   56, 8751,   24268,  8, }, // Europe/Busingen
    {   56, 2475,   24276,  8, }, // Europe/Chisinau
    {   56, 6238,   24284, 10, }, // Europe/Copenhagen
    {   56, 2260,   18092,  6, }, // Europe/Dublin
    {   56, 6256,   24294, 11, }, // Europe/Gibraltar
    {   56, 8767,   24305, 10, }, // Europe/Guernsey
    {   56, 6561,   24315,  9, }, // Europe/Helsinki
    {   56, 8783,   24324, 12, }, // Europe/Isle_of_Man
    {   56, 1097,   18133,  9, }, // Europe/Istanbul
    {   56, 8802,   24336,  6, }, // Europe/Jersey
    {   56, 3881,   24342, 12, }, // Europe/Kaliningrad
    {   56, 8924,   24354,  5, }, // Europe/Kirov
    {   56, 2419,   24359,  6, }, // Europe/Kyiv
    {   56, 3078,   24365,  5, }, // Europe/Lisbon
    {   56, 6455,   24370, 10, }, // Europe/Ljubljana
    {   56, 2393,   24380,  5, }, // Europe/London
    {   56, 6314,   24385, 12, }, // Europe/Luxembourg
    {   56, 4794,   24397,  7, }, // Europe/Madrid
    {   56, 6346,   18208,  6, }, // Europe/Malta
    {   56, 6531,   24404, 11, }, // Europe/Mariehamn
    {   56, 3595,   24415,  6, }, // Europe/Minsk
    {   56, 6359,   18230,  6, }, // Europe/Monaco
    {   56, 3443,   18236,  5, }, // Europe/Moscow
    {   56, 6407,   24421,  4, }, // Europe/Oslo
    {   56, 4186,   24425,  7, }, // Europe/Paris
    {   56, 6373,   24432,  9, }, // Europe/Podgorica
    {   56, 6224,   24441,  5, }, // Europe/Prague
    {   56, 8816,   24446,  4, }, // Europe/Riga
    {   56, 6273,   24056,  3, }, // Europe/Rome
    {   56, 4233,   24450,  6, }, // Europe/Samara
    {   56, 6419,   24456, 10, }, // Europe/San_Marino
    {   56, 6194,   24466,  8, }, // Europe/Sarajevo
    {   56, 4353,   24474,  7, }, // Europe/Saratov
    {   56, 8828,   24481, 10, }, // Europe/Simferopol
    {   56, 6332,   24491,  7, }, // Europe/Skopje
    {   56, 6548,   24498,  7, }, // Europe/Sofia
    {   56, 6472,   24505,  7, }, // Europe/Stockholm
    {   56, 8846,   24512,  6, }, // Europe/Tallinn
    {   56, 6135,   24518,  6, }, // Europe/Tirane
    {   56, 8861,   24524, 12, }, // Europe/Ulyanovsk
    {   56, 6301,   18357,  5, }, // Europe/Vaduz
    {   56, 6516,   24536,  7, }, // Europe/Vatican
    {   56, 6164,   18369,  7, }, // Europe/Vienna
    {   56, 8878,   24543,  7, }, // Europe/Vilnius
    {   56, 4629,   24550, 11, }, // Europe/Volgograd
    {   56, 3055,   24561,  6, }, // Europe/Warsaw
    {   56, 6210,   24567,  7, }, // Europe/Zagreb
    {   56, 6489,   24574,  5, }, // Europe/Zurich
    {   56, 4980,   24579, 13, }, // Indian/Antananarivo
    {   56, 6910,   24592,  5, }, // Indian/Chagos
    {   56, 5979,   24597,  8, }, // Indian/Christmas
    {   56, 5996,   24605,  5, }, // Indian/Cocos
    {   56, 4931,   24610,  6, }, // Indian/Comoro
    {   56, 6624,   24616,  9, }, // Indian/Kerguelen
    {   56, 7223,   18461,  4, }, // Indian/Mahe
    {   56, 7013,   18465,  8, }, // Indian/Maldives
    {   56, 3971,   24625,  6, }, // Indian/Mauritius
    {   56, 5000,   24631,  7, }, // Indian/Mayotte
    {   56, 7174,   24638,  9, }, // Indian/Reunion
    {   56, 4324,   18497,  6, }, // Pacific/Apia
    {   56,  932,   24647,  9, }, // Pacific/Auckland
    {   56, 3608,   24656,  9, }, // Pacific/Bougainville
    {   56, 2863,   24665,  7, }, // Pacific/Chatham
    {   56, 3022,   18527,  3, }, // Pacific/Chuuk
    {   56, 2201,   24672,  6, }, // Pacific/Easter
    {   56, 7323,   24678,  5, }, // Pacific/Efate
    {   56, 7290,   24683,  9, }, // Pacific/Fakaofo
    {   56, 3804,   18547,  4, }, // Pacific/Fiji
    {   56, 7306,   18551,  8, }, // Pacific/Funafuti
    {   56, 6641,   24692, 13, }, // Pacific/Galapagos
    {   56, 6659,   24705, 10, }, // Pacific/Gambier
    {   56, 3724,   24715, 13, }, // Pacific/Guadalcanal
    {   56, 5935,   22784,  6, }, // Pacific/Guam
    {   56, 2929,   24728,  7, }, // Pacific/Honolulu
    {   56, 2897,   24735, 10, }, // Pacific/Kanton
    {   56, 3900,   24745, 10, }, // Pacific/Kiritimati
    {   56, 6982,   24755,  7, }, // Pacific/Kosrae
    {   56, 2708,   24762, 11, }, // Pacific/Kwajalein
    {   56, 4779,   24773,  6, }, // Pacific/Majuro
    {   56, 3953,   24779,  9, }, // Pacific/Marquesas
    {   56, 8909,   24788,  7, }, // Pacific/Midway
    {   56, 7047,   24795,  5, }, // Pacific/Nauru
    {   56, 7095,   24800,  6, }, // Pacific/Niue
    {   56, 4070,   24806,  4, }, // Pacific/Norfolk
    {   56, 7061,   24810,  7, }, // Pacific/Noumea
    {   56, 2991,   24817, 13, }, // Pacific/Pago_Pago
    {   56, 7118,   18675,  5, }, // Pacific/Palau
    {   56, 7157,   24830, 11, }, // Pacific/Pitcairn
    {   56, 2961,   24841,  6, }, // Pacific/Pohnpei
    {   56, 4709,   24847, 12, }, // Pacific/Port_Moresby
    {   56, 6009,   24859, 11, }, // Pacific/Rarotonga
    {   56, 5948,   24870,  6, }, // Pacific/Saipan
    {   56, 7275,   18726,  6, }, // Pacific/Tahiti
    {   56, 6675,   24876,  8, }, // Pacific/Tarawa
    {   56, 4485,   24884, 10, }, // Pacific/Tongatapu
    {   56, 7355,   24894,  5, }, // Pacific/Wake
    {   56, 7368,   24899,  4, }, // Pacific/Wallis
    {   58,   14,       0,  6, }, // Africa/Asmara Basaa/Latin/Cameroon
    {   58, 4337,       6,  8, }, // Africa/Sao_Tome
    {   58, 4154,      14,  8, }, // America/Asuncion
    {   58,  237,      22,  8, }, // America/Atikokan
    {   58, 7586,      30, 17, }, // America/Bahia_Banderas
    {   58, 3770,      47,  6, }, // America/Cancun
    {   58, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   58, 5640,      66,  7, }, // America/Curacao
    {   58,  481,      73, 13, }, // America/Indiana/Knox
    {   58, 7924,      86, 16, }, // America/Indiana/Marengo
    {   58, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   58, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   58, 8001,     139, 14, }, // America/Indiana/Vevay
    {   58, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   58, 8049,     171, 16, }, // America/Indiana/Winamac
    {   58, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   58, 5849,     207, 22, }, // America/Lower_Princes
    {   58, 8183,     229,  6, }, // America/Merida
    {   58, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   58, 8292,     255, 20, }, // America/North_Dakota/Center
    {   58, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   58,  393,     298,  4, }, // America/Nuuk
    {   58, 6862,     302, 16, }, // America/Scoresbysund
    {   58, 8471,     318, 14, }, // America/St_Barthelemy
    {   58, 2045,     332, 10, }, // America/St_Johns
    {   58, 5780,     342,  9, }, // America/St_Kitts
    {   58, 5797,     351,  9, }, // America/St_Lucia
    {   58,  855,     360, 10, }, // America/St_Thomas
    {   58, 5830,     370, 11, }, // America/St_Vincent
    {   58, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   58, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   58, 1152,     413,  9, }, // Asia/Kathmandu
    {   58,  992,     422,  7, }, // Asia/Kolkata
    {   58, 1178,     429,  5, }, // Asia/Macau
    {   58, 1202,     442,  6, }, // Asia/Yangon
    {   58, 1379,     448,  5, }, // Atlantic/Faroe
    {   58, 6784,     453, 10, }, // Atlantic/St_Helena
    {   58, 2419,     463,  4, }, // Europe/Kyiv
    {   58, 7174,     467,  7, }, // Indian/Reunion
    {   58, 3022,     474,  5, }, // Pacific/Chuuk
    {   58, 2961,     479,  7, }, // Pacific/Pohnpei
    {   59,   14,       0,  6, }, // Africa/Asmara Bashkir/Cyrillic/Russia
    {   59, 4337,       6,  8, }, // Africa/Sao_Tome
    {   59, 4154,      14,  8, }, // America/Asuncion
    {   59,  237,      22,  8, }, // America/Atikokan
    {   59, 7586,      30, 17, }, // America/Bahia_Banderas
    {   59, 3770,      47,  6, }, // America/Cancun
    {   59, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   59, 5640,      66,  7, }, // America/Curacao
    {   59,  481,      73, 13, }, // America/Indiana/Knox
    {   59, 7924,      86, 16, }, // America/Indiana/Marengo
    {   59, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   59, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   59, 8001,     139, 14, }, // America/Indiana/Vevay
    {   59, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   59, 8049,     171, 16, }, // America/Indiana/Winamac
    {   59, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   59, 5849,     207, 22, }, // America/Lower_Princes
    {   59, 8183,     229,  6, }, // America/Merida
    {   59, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   59, 8292,     255, 20, }, // America/North_Dakota/Center
    {   59, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   59,  393,     298,  4, }, // America/Nuuk
    {   59, 6862,     302, 16, }, // America/Scoresbysund
    {   59, 8471,     318, 14, }, // America/St_Barthelemy
    {   59, 2045,     332, 10, }, // America/St_Johns
    {   59, 5780,     342,  9, }, // America/St_Kitts
    {   59, 5797,     351,  9, }, // America/St_Lucia
    {   59,  855,     360, 10, }, // America/St_Thomas
    {   59, 5830,     370, 11, }, // America/St_Vincent
    {   59, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   59, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   59, 1152,     413,  9, }, // Asia/Kathmandu
    {   59,  992,     422,  7, }, // Asia/Kolkata
    {   59, 1178,     429,  5, }, // Asia/Macau
    {   59, 1202,     442,  6, }, // Asia/Yangon
    {   59, 1379,     448,  5, }, // Atlantic/Faroe
    {   59, 6784,     453, 10, }, // Atlantic/St_Helena
    {   59, 2419,     463,  4, }, // Europe/Kyiv
    {   59, 7174,     467,  7, }, // Indian/Reunion
    {   59, 3022,     474,  5, }, // Pacific/Chuuk
    {   59, 2961,     479,  7, }, // Pacific/Pohnpei
    {   60, 6723,   19687,  5, }, // Africa/Accra Basque/Latin/Spain
    {   60, 4961,    3793, 11, }, // Africa/Addis_Ababa
    {   60, 7383,   24903,  5, }, // Africa/Algiers
    {   60,   14,       0,  6, }, // Africa/Asmara
    {   60, 5159,   24908,  5, }, // Africa/Bangui
    {   60, 2242,   24913,  5, }, // Africa/Cairo
    {   60, 6736,   24918,  7, }, // Africa/Conakry
    {   60, 4945,   24925,  7, }, // Africa/Djibouti
    {   60, 5145,   19777,  5, }, // Africa/Douala
    {   60, 5068,   24932,  5, }, // Africa/El_Aaiun
    {   60, 4426,   24937,  7, }, // Africa/Khartoum
    {   60, 5015,   24944,  8, }, // Africa/Mogadishu
    {   60, 5173,   24952,  9, }, // Africa/Ndjamena
    {   60, 5256,   24961,  6, }, // Africa/Niamey
    {   60, 6766,   18826,  7, }, // Africa/Nouakchott
    {   60, 6690,   24967,  8, }, // Africa/Ouagadougou
    {   60, 5127,   24975, 10, }, // Africa/Porto-Novo
    {   60, 4337,       6,  8, }, // Africa/Sao_Tome
    {   60, 5524,   24985,  7, }, // America/Anguilla
    {   60, 4456,   24992,  9, }, // America/Araguaina
    {   60,  270,   18870,  7, }, // America/Argentina/Cordoba
    {   60, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {   60, 4154,      14,  8, }, // America/Asuncion
    {   60,  237,      22,  8, }, // America/Atikokan
    {   60, 7586,      30, 17, }, // America/Bahia_Banderas
    {   60, 3770,      47,  6, }, // America/Cancun
    {   60, 5398,   25001,  6, }, // America/Cayman
    {   60, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   60, 3693,   25007,  6, }, // America/Cuiaba
    {   60, 5640,      66,  7, }, // America/Curacao
    {   60, 5656,   20071,  8, }, // America/Dominica
    {   60, 7832,   25013,  8, }, // America/Eirunepe
    {   60, 5703,   18917,  9, }, // America/Guadeloupe
    {   60, 2221,   18929,  6, }, // America/Havana
    {   60,  481,      73, 13, }, // America/Indiana/Knox
    {   60, 7924,      86, 16, }, // America/Indiana/Marengo
    {   60, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   60, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   60, 8001,     139, 14, }, // America/Indiana/Vevay
    {   60, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   60, 8049,     171, 16, }, // America/Indiana/Winamac
    {   60, 2665,    3892,  7, }, // America/Jamaica
    {   60, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   60, 5849,     207, 22, }, // America/Lower_Princes
    {   60, 8116,   25021,  6, }, // America/Maceio
    {   60, 5722,   25027,  9, }, // America/Martinique
    {   60, 2779,   25036,  8, }, // America/Mazatlan
    {   60, 8183,     229,  6, }, // America/Merida
    {   60, 2811,   25044, 12, }, // America/Mexico_City
    {   60, 4293,   25056,  8, }, // America/Miquelon
    {   60, 8264,   25064, 19, }, // America/North_Dakota/Beulah
    {   60, 8292,   25083, 19, }, // America/North_Dakota/Center
    {   60, 8320,   25102, 22, }, // America/North_Dakota/New_Salem
    {   60,  393,     298,  4, }, // America/Nuuk
    {   60, 5871,   25124, 13, }, // America/Port_of_Spain
    {   60, 8440,   25137,  8, }, // America/Santarem
    {   60, 1862,   25145,  9, }, // America/Sao_Paulo
    {   60, 6862,     302, 16, }, // America/Scoresbysund
    {   60, 8471,   25154, 16, }, // America/St_Barthelemy
    {   60, 2045,   25170, 12, }, // America/St_Johns
    {   60, 5780,   25182, 11, }, // America/St_Kitts
    {   60, 5797,   25193, 11, }, // America/St_Lucia
    {   60,  855,   25204, 12, }, // America/St_Thomas
    {   60, 5830,   25216, 13, }, // America/St_Vincent
    {   60, 5673,   25229,  7, }, // America/Thule
    {   60, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   60, 5440,    5906,  5, }, // Asia/Aqtau
    {   60, 5270,    5911,  6, }, // Asia/Aqtobe
    {   60,  964,   25236,  7, }, // Asia/Ashgabat
    {   60, 3509,    4063,  6, }, // Asia/Baghdad
    {   60, 3680,   25243,  6, }, // Asia/Bishkek
    {   60, 4503,   25249,  5, }, // Asia/Chita
    {   60, 5963,   25254, 10, }, // Asia/Choibalsan
    {   60, 4413,    5949,  7, }, // Asia/Colombo
    {   60, 4442,   25264,  7, }, // Asia/Damascus
    {   60, 6092,   25271,  7, }, // Asia/Dushanbe
    {   60, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   60, 4673,   25278,  5, }, // Asia/Hovd
    {   60, 4218,   25283,  9, }, // Asia/Kamchatka
    {   60, 4141,   25292,  7, }, // Asia/Karachi
    {   60, 1152,    6018,  8, }, // Asia/Kathmandu
    {   60, 8624,   25299,  8, }, // Asia/Khandyga
    {   60,  992,   25307,  7, }, // Asia/Kolkata
    {   60, 4099,   25314, 11, }, // Asia/Krasnoyarsk
    {   60, 1178,    4222,  5, }, // Asia/Macau
    {   60, 6883,   25325,  6, }, // Asia/Muscat
    {   60, 2446,   25331,  7, }, // Asia/Nicosia
    {   60, 4116,   25338,  9, }, // Asia/Pyongyang
    {   60, 8671,     434,  8, }, // Asia/Qostanay
    {   60, 4171,   25347,  9, }, // Asia/Qyzylorda
    {   60, 3486,    6120,  4, }, // Asia/Riyadh
    {   60, 7208,   25356, 10, }, // Asia/Samarkand
    {   60, 3116,    6131,  4, }, // Asia/Seoul
    {   60, 3137,   19291,  8, }, // Asia/Singapore
    {   60, 4199,    6150, 13, }, // Asia/Srednekolymsk
    {   60, 4683,   25366,  7, }, // Asia/Tashkent
    {   60, 2638,    6175,  7, }, // Asia/Tehran
    {   60, 2687,    4257,  5, }, // Asia/Tokyo
    {   60, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {   60, 1126,   25383,  6, }, // Asia/Urumqi
    {   60, 6940,   25389,  8, }, // Asia/Vientiane
    {   60, 4730,    6206,  7, }, // Asia/Yakutsk
    {   60, 1202,     442,  6, }, // Asia/Yangon
    {   60, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {   60, 3649,   25397,  6, }, // Asia/Yerevan
    {   60, 3565,   25403,  7, }, // Atlantic/Azores
    {   60, 6591,   25410,  8, }, // Atlantic/Canary
    {   60, 3629,   25418, 10, }, // Atlantic/Cape_Verde
    {   60, 1379,     448,  5, }, // Atlantic/Faroe
    {   60, 7235,   25428, 19, }, // Atlantic/South_Georgia
    {   60, 6784,   25447, 12, }, // Atlantic/St_Helena
    {   60, 6577,   25459,  6, }, // Europe/Athens
    {   60, 6285,   25465,  7, }, // Europe/Belgrade
    {   60, 6178,   25472,  7, }, // Europe/Brussels
    {   60, 3830,   25479,  8, }, // Europe/Bucharest
    {   60, 8751,   25487,  8, }, // Europe/Busingen
    {   60, 6238,    4376,  9, }, // Europe/Copenhagen
    {   60, 8767,   25495,  9, }, // Europe/Guernsey
    {   60, 8783,   25504, 11, }, // Europe/Isle_of_Man
    {   60, 2419,    6486,  4, }, // Europe/Kyiv
    {   60, 3078,   19528,  6, }, // Europe/Lisbon
    {   60, 2393,   19543,  7, }, // Europe/London
    {   60, 6314,   25515, 10, }, // Europe/Luxembourg
    {   60, 4794,   25525,  6, }, // Europe/Madrid
    {   60, 6359,    6525,  6, }, // Europe/Monaco
    {   60, 3443,   25531,  5, }, // Europe/Moscow
    {   60, 6224,   19576,  5, }, // Europe/Prague
    {   60, 6273,   25536,  6, }, // Europe/Rome
    {   60, 6135,   19608,  6, }, // Europe/Tirane
    {   60, 8861,   25542,  9, }, // Europe/Ulyanovsk
    {   60, 6516,   25551, 14, }, // Europe/Vatican
    {   60, 6164,   19625,  5, }, // Europe/Vienna
    {   60, 3055,   19639,  8, }, // Europe/Warsaw
    {   60, 6489,    4478,  6, }, // Europe/Zurich
    {   60, 7223,   25565,  4, }, // Indian/Mahe
    {   60, 7013,   25569,  8, }, // Indian/Maldives
    {   60, 3971,   25577,  8, }, // Indian/Mauritius
    {   60, 7174,     467,  7, }, // Indian/Reunion
    {   60, 3022,     474,  5, }, // Pacific/Chuuk
    {   60, 2201,   25585, 13, }, // Pacific/Easter
    {   60, 7323,   25598,  5, }, // Pacific/Efate
    {   60, 6641,   25603, 10, }, // Pacific/Galapagos
    {   60, 2929,    4512,  8, }, // Pacific/Honolulu
    {   60, 2897,    4520,  9, }, // Pacific/Kanton
    {   60, 3953,   25613,  8, }, // Pacific/Marquesas
    {   60, 7061,    4529,  6, }, // Pacific/Noumea
    {   60, 2961,     479,  7, }, // Pacific/Pohnpei
    {   61, 6751,   25621,  7, }, // Africa/Abidjan Belarusian/Cyrillic/Belarus
    {   61, 6723,   25628,  4, }, // Africa/Accra
    {   61, 4961,   25632, 10, }, // Africa/Addis_Ababa
    {   61, 7383,   25642,  5, }, // Africa/Algiers
    {   61,   14,   25647,  6, }, // Africa/Asmara
    {   61,   44,     519,  6, }, // Africa/Bamako
    {   61, 5159,   25653,  5, }, // Africa/Bangui
    {   61, 6709,   25658,  7, }, // Africa/Banjul
    {   61, 7398,   25665,  5, }, // Africa/Bissau
    {   61, 4859,   25670,  8, }, // Africa/Blantyre
    {   61, 5189,   25678,  9, }, // Africa/Brazzaville
    {   61, 4824,     558,  9, }, // Africa/Bujumbura
    {   61, 2242,   25687,  4, }, // Africa/Cairo
    {   61, 4019,     571, 10, }, // Africa/Casablanca
    {   61, 7412,     581,  5, }, // Africa/Ceuta
    {   61, 6736,   25691,  7, }, // Africa/Conakry
    {   61, 6803,     593,  5, }, // Africa/Dakar
    {   61, 5032,   25698, 12, }, // Africa/Dar_es_Salaam
    {   61, 4945,   25710,  7, }, // Africa/Djibouti
    {   61, 5145,     616,  5, }, // Africa/Douala
    {   61, 5068,   25717,  7, }, // Africa/El_Aaiun
    {   61, 6816,   25724,  7, }, // Africa/Freetown
    {   61, 4808,   25731,  8, }, // Africa/Gaborone
    {   61, 4917,   25739,  6, }, // Africa/Harare
    {   61, 4381,   25745, 10, }, // Africa/Johannesburg
    {   61, 4401,   25755,  5, }, // Africa/Juba
    {   61, 5053,     665,  7, }, // Africa/Kampala
    {   61, 4426,     672,  6, }, // Africa/Khartoum
    {   61, 4875,   25760,  6, }, // Africa/Kigali
    {   61, 5208,   25766,  7, }, // Africa/Kinshasa
    {   61, 4646,   25773,  5, }, // Africa/Lagos
    {   61, 5238,   25778,  9, }, // Africa/Libreville
    {   61, 6832,   25787,  4, }, // Africa/Lome
    {   61, 5113,     708,  6, }, // Africa/Luanda
    {   61, 4841,   25791,  9, }, // Africa/Lubumbashi
    {   61, 4903,     723,  6, }, // Africa/Lusaka
    {   61, 5224,   25800,  6, }, // Africa/Malabo
    {   61, 4889,     735,  6, }, // Africa/Maputo
    {   61, 5099,     741,  6, }, // Africa/Maseru
    {   61, 5084,   25806,  7, }, // Africa/Mbabane
    {   61, 5015,   25813,  8, }, // Africa/Mogadishu
    {   61, 6997,   25821,  8, }, // Africa/Monrovia
    {   61, 3755,   25829,  7, }, // Africa/Nairobi
    {   61, 5173,   25836,  8, }, // Africa/Ndjamena
    {   61, 5256,   25844,  6, }, // Africa/Niamey
    {   61, 6766,     790,  7, }, // Africa/Nouakchott
    {   61, 6690,     797,  8, }, // Africa/Ouagadougou
    {   61, 5127,   25850, 10, }, // Africa/Porto-Novo
    {   61, 4337,   25860,  8, }, // Africa/Sao_Tome
    {   61, 2732,   25868,  7, }, // Africa/Tripoli
    {   61, 6503,   25875,  5, }, // Africa/Tunis
    {   61, 4054,   25880,  7, }, // Africa/Windhoek
    {   61,  132,     842,  4, }, // America/Adak
    {   61, 3183,   25887,  8, }, // America/Anchorage
    {   61, 5524,   25895,  7, }, // America/Anguilla
    {   61, 5541,   25902,  7, }, // America/Antigua
    {   61, 4456,   25909,  9, }, // America/Araguaina
    {   61,  166,   25918, 12, }, // America/Argentina/Buenos_Aires
    {   61,   91,     888,  9, }, // America/Argentina/Catamarca
    {   61,  270,   25930,  7, }, // America/Argentina/Cordoba
    {   61,  441,   25937,  5, }, // America/Argentina/Jujuy
    {   61, 7425,   25942,  8, }, // America/Argentina/La_Rioja
    {   61,  565,     917,  7, }, // America/Argentina/Mendoza
    {   61, 7452,   25950, 12, }, // America/Argentina/Rio_Gallegos
    {   61, 7483,   25962,  6, }, // America/Argentina/Salta
    {   61, 7507,     940,  8, }, // America/Argentina/San_Juan
    {   61, 5497,   25968,  8, }, // America/Argentina/San_Luis
    {   61, 7534,     956,  7, }, // America/Argentina/Tucuman
    {   61, 7560,   25976,  5, }, // America/Argentina/Ushuaia
    {   61, 5557,     969,  5, }, // America/Aruba
    {   61, 4154,   25981,  8, }, // America/Asuncion
    {   61,  237,   25989,  8, }, // America/Atikokan
    {   61, 3581,   25997,  4, }, // America/Bahia
    {   61, 7586,   26001, 16, }, // America/Bahia_Banderas
    {   61, 5571,   26017,  8, }, // America/Barbados
    {   61, 7609,    1021,  5, }, // America/Belem
    {   61, 5309,   26025,  5, }, // America/Belize
    {   61, 7623,    1031, 12, }, // America/Blanc-Sablon
    {   61, 7644,   26030,  9, }, // America/Boa_Vista
    {   61, 4263,   26039,  6, }, // America/Bogota
    {   61, 7662,   26045,  5, }, // America/Boise
    {   61, 7676,   26050, 12, }, // America/Cambridge_Bay
    {   61, 7698,   26062, 12, }, // America/Campo_Grande
    {   61, 3770,    1087,  6, }, // America/Cancun
    {   61, 4596,    1093,  7, }, // America/Caracas
    {   61, 4247,   26074,  5, }, // America/Cayenne
    {   61, 5398,   26079, 17, }, // America/Cayman
    {   61, 3251,   26096,  6, }, // America/Chicago
    {   61, 7719,   26102,  6, }, // America/Chihuahua
    {   61, 7737,   26108, 13, }, // America/Ciudad_Juarez
    {   61, 5324,   26121, 10, }, // America/Costa_Rica
    {   61, 7759,   26131,  7, }, // America/Creston
    {   61, 3693,   26138,  5, }, // America/Cuiaba
    {   61, 5640,   26143,  7, }, // America/Curacao
    {   61, 7775,   26150, 12, }, // America/Danmarkshavn
    {   61, 7796,   26162,  6, }, // America/Dawson
    {   61, 7811,   26168, 10, }, // America/Dawson_Creek
    {   61,  805,   26178,  6, }, // America/Denver
    {   61, 3351,   26184,  7, }, // America/Detroit
    {   61, 5656,   26191,  8, }, // America/Dominica
    {   61,  893,   26199,  8, }, // America/Edmonton
    {   61, 7832,   26207,  8, }, // America/Eirunepe
    {   61, 5343,   26215,  9, }, // America/El_Salvador
    {   61, 7849,   26224, 12, }, // America/Fort_Nelson
    {   61, 7869,   26236,  9, }, // America/Fortaleza
    {   61, 7887,   26245,  9, }, // America/Glace_Bay
    {   61, 6844,   26254,  7, }, // America/Goose_Bay
    {   61, 4514,   26261, 10, }, // America/Grand_Turk
    {   61, 5687,   26271,  7, }, // America/Grenada
    {   61, 5703,   26278,  9, }, // America/Guadeloupe
    {   61, 3662,   26287,  9, }, // America/Guatemala
    {   61, 4743,   26296,  8, }, // America/Guayaquil
    {   61, 6895,   26304,  5, }, // America/Guyana
    {   61, 1923,   26309,  8, }, // America/Halifax
    {   61, 2221,    1339,  6, }, // America/Havana
    {   61, 7905,   26317,  9, }, // America/Hermosillo
    {   61,  348,   26326, 12, }, // America/Indiana/Indianapolis
    {   61,  481,   26338, 13, }, // America/Indiana/Knox
    {   61, 7924,   26351, 16, }, // America/Indiana/Marengo
    {   61, 7948,   26367, 19, }, // America/Indiana/Petersburg
    {   61, 7975,   26386, 17, }, // America/Indiana/Tell_City
    {   61, 8001,   26403, 13, }, // America/Indiana/Vevay
    {   61, 8023,   26416, 16, }, // America/Indiana/Vincennes
    {   61, 8049,   26432, 16, }, // America/Indiana/Winamac
    {   61, 8073,   26448,  6, }, // America/Inuvik
    {   61,  660,   26454,  7, }, // America/Iqaluit
    {   61, 2665,   26461,  6, }, // America/Jamaica
    {   61, 5282,   26467,  5, }, // America/Juneau
    {   61,  521,   26472,  7, }, // America/Kentucky/Louisville
    {   61, 8088,   26479, 18, }, // America/Kentucky/Monticello
    {   61, 5621,   26497, 10, }, // America/Kralendijk
    {   61, 4278,    1514,  6, }, // America/La_Paz
    {   61, 7132,   26507,  4, }, // America/Lima
    {   61, 3390,   26511, 12, }, // America/Los_Angeles
    {   61, 5849,   26523, 20, }, // America/Lower_Princes
    {   61, 8116,   26543,  5, }, // America/Maceio
    {   61, 8131,    1559,  7, }, // America/Managua
    {   61, 1892,   26548,  6, }, // America/Manaus
    {   61, 5814,   26554,  6, }, // America/Marigot
    {   61, 5722,   26560,  9, }, // America/Martinique
    {   61, 8147,   26569,  9, }, // America/Matamoros
    {   61, 2779,    1596,  8, }, // America/Mazatlan
    {   61, 8165,   26578,  8, }, // America/Menominee
    {   61, 8183,   26586,  6, }, // America/Merida
    {   61, 8198,    1618, 10, }, // America/Metlakatla
    {   61, 2811,   26592,  6, }, // America/Mexico_City
    {   61, 4293,   26598,  7, }, // America/Miquelon
    {   61, 8217,   26605,  7, }, // America/Moncton
    {   61, 8233,   26612,  8, }, // America/Monterrey
    {   61, 4000,   26620, 10, }, // America/Montevideo
    {   61, 5741,   26630,  9, }, // America/Montserrat
    {   61, 5383,   26639,  5, }, // America/Nassau
    {   61, 3294,   26644,  7, }, // America/New_York
    {   61, 8251,    1691,  3, }, // America/Nome
    {   61, 1834,   26651,  7, }, // America/Noronha
    {   61, 8264,   26658, 23, }, // America/North_Dakota/Beulah
    {   61, 8292,   26681, 24, }, // America/North_Dakota/Center
    {   61, 8320,   26705, 28, }, // America/North_Dakota/New_Salem
    {   61,  393,   26733,  3, }, // America/Nuuk
    {   61, 8351,   26736,  7, }, // America/Ojinaga
    {   61, 5413,    1783,  6, }, // America/Panama
    {   61, 6106,   26743, 10, }, // America/Paramaribo
    {   61, 3224,   26753,  6, }, // America/Phoenix
    {   61, 3847,   26759, 12, }, // America/Port-au-Prince
    {   61, 5871,   26771, 13, }, // America/Port_of_Spain
    {   61, 8367,   26784, 11, }, // America/Porto_Velho
    {   61, 5760,   26795, 11, }, // America/Puerto_Rico
    {   61, 3932,   26806, 12, }, // America/Punta_Arenas
    {   61, 8387,   26818, 12, }, // America/Rankin_Inlet
    {   61, 8408,   26830,  6, }, // America/Recife
    {   61, 1979,   26836,  8, }, // America/Regina
    {   61, 8423,   26844,  8, }, // America/Resolute
    {   61,  695,   26852, 10, }, // America/Rio_Branco
    {   61, 8440,   26862,  8, }, // America/Santarem
    {   61, 2165,   26870,  8, }, // America/Santiago
    {   61, 6044,   26878, 13, }, // America/Santo_Domingo
    {   61, 1862,   26891,  9, }, // America/Sao_Paulo
    {   61, 6862,   26900, 13, }, // America/Scoresbysund
    {   61, 8457,   26913,  5, }, // America/Sitka
    {   61, 8471,   26918, 13, }, // America/St_Barthelemy
    {   61, 2045,   26931, 10, }, // America/St_Johns
    {   61, 5780,   26941,  9, }, // America/St_Kitts
    {   61, 5797,   26950, 10, }, // America/St_Lucia
    {   61,  855,    2003, 10, }, // America/St_Thomas
    {   61, 5830,   26960, 12, }, // America/St_Vincent
    {   61, 8493,   26972, 12, }, // America/Swift_Current
    {   61, 5363,   26984, 12, }, // America/Tegucigalpa
    {   61, 5673,    2049,  4, }, // America/Thule
    {   61,  313,   26996,  7, }, // America/Tijuana
    {   61,  608,   27003,  7, }, // America/Toronto
    {   61, 5605,   27010,  7, }, // America/Tortola
    {   61, 2077,    2074,  8, }, // America/Vancouver
    {   61, 2128,   27017,  8, }, // America/Whitehorse
    {   61,  734,   27025,  7, }, // America/Winnipeg
    {   61, 8515,   27032,  6, }, // America/Yakutat
    {   61, 5918,   27038,  5, }, // Antarctica/Casey
    {   61, 6027,   27043,  6, }, // Antarctica/Davis
    {   61, 6066,   27049, 15, }, // Antarctica/DumontDUrville
    {   61, 8531,   27064,  7, }, // Antarctica/Macquarie
    {   61, 7029,   27071,  6, }, // Antarctica/Mawson
    {   61, 7076,   27077,  9, }, // Antarctica/McMurdo
    {   61, 8552,    2154,  6, }, // Antarctica/Palmer
    {   61, 7189,   27086,  6, }, // Antarctica/Rothera
    {   61, 7258,   27092,  4, }, // Antarctica/Syowa
    {   61, 8570,    2171,  4, }, // Antarctica/Troll
    {   61, 7337,   27096,  6, }, // Antarctica/Vostok
    {   61, 1413,   27102,  7, }, // Arctic/Longyearbyen
    {   61, 5487,   27109,  4, }, // Asia/Aden
    {   61, 5297,    2196,  6, }, // Asia/Almaty
    {   61, 3870,   27113, 12, }, // Asia/Amman
    {   61, 5428,    2207,  6, }, // Asia/Anadyr
    {   61, 5440,    2213,  5, }, // Asia/Aqtau
    {   61, 5270,   27125,  6, }, // Asia/Aqtobe
    {   61,  964,   27131,  7, }, // Asia/Ashgabat
    {   61, 8587,    2231,  6, }, // Asia/Atyrau
    {   61, 3509,    2237,  6, }, // Asia/Baghdad
    {   61, 5451,   27138,  7, }, // Asia/Bahrain
    {   61, 3555,    2250,  4, }, // Asia/Baku
    {   61, 4368,    2254,  7, }, // Asia/Bangkok
    {   61, 3473,    2261,  7, }, // Asia/Barnaul
    {   61, 3988,   27145,  6, }, // Asia/Beirut
    {   61, 3680,   27151,  6, }, // Asia/Bishkek
    {   61, 5906,   27157,  6, }, // Asia/Brunei
    {   61, 4503,   27163,  4, }, // Asia/Chita
    {   61, 5963,   27167,  9, }, // Asia/Choibalsan
    {   61, 4413,   27176,  7, }, // Asia/Colombo
    {   61, 4442,    2306,  6, }, // Asia/Damascus
    {   61, 1060,     593,  4, }, // Asia/Dhaka
    {   61, 6125,   27183,  4, }, // Asia/Dili
    {   61, 3498,   27187,  5, }, // Asia/Dubai
    {   61, 6092,   27192,  7, }, // Asia/Dushanbe
    {   61, 8599,    2333,  9, }, // Asia/Famagusta
    {   61, 8614,    2342,  4, }, // Asia/Gaza
    {   61, 4697,   27199,  6, }, // Asia/Hebron
    {   61, 1226,   27205,  7, }, // Asia/Ho_Chi_Minh
    {   61, 2576,   27212,  7, }, // Asia/Hong_Kong
    {   61, 4673,   27219,  4, }, // Asia/Hovd
    {   61, 4086,   27223,  6, }, // Asia/Irkutsk
    {   61, 6969,   27229,  8, }, // Asia/Jakarta
    {   61, 6955,   27237,  8, }, // Asia/Jayapura
    {   61, 1257,   27245,  9, }, // Asia/Jerusalem
    {   61, 3462,    2401,  5, }, // Asia/Kabul
    {   61, 4218,    2420,  8, }, // Asia/Kamchatka
    {   61, 4141,   27254,  6, }, // Asia/Karachi
    {   61, 1152,    2437,  8, }, // Asia/Kathmandu
    {   61, 8624,    2445,  7, }, // Asia/Khandyga
    {   61,  992,   27260,  8, }, // Asia/Kolkata
    {   61, 4099,   27268, 10, }, // Asia/Krasnoyarsk
    {   61, 4761,    2471, 12, }, // Asia/Kuala_Lumpur
    {   61, 5893,   27278,  6, }, // Asia/Kuching
    {   61, 5464,   27284,  6, }, // Asia/Kuwait
    {   61, 1178,   27290,  5, }, // Asia/Macau
    {   61, 3919,    2500,  7, }, // Asia/Magadan
    {   61, 1316,    2507,  7, }, // Asia/Makassar
    {   61, 7145,   27295,  6, }, // Asia/Manila
    {   61, 6883,    2520,  6, }, // Asia/Muscat
    {   61, 2446,   27301,  7, }, // Asia/Nicosia
    {   61, 8638,   27308, 11, }, // Asia/Novokuznetsk
    {   61, 4037,   27319, 11, }, // Asia/Novosibirsk
    {   61, 4131,    2555,  4, }, // Asia/Omsk
    {   61, 7108,   27330,  7, }, // Asia/Oral
    {   61, 6924,   27337,  8, }, // Asia/Phnom_Penh
    {   61, 8656,   27345,  9, }, // Asia/Pontianak
    {   61, 4116,   27354,  7, }, // Asia/Pyongyang
    {   61, 5476,    2588,  5, }, // Asia/Qatar
    {   61, 8671,   27361,  8, }, // Asia/Qostanay
    {   61, 4171,   27369,  9, }, // Asia/Qyzylorda
    {   61, 3486,   27378,  7, }, // Asia/Riyadh
    {   61, 4310,   27385,  7, }, // Asia/Sakhalin
    {   61, 7208,    2630,  9, }, // Asia/Samarkand
    {   61, 3116,    2639,  4, }, // Asia/Seoul
    {   61, 1020,   27392,  6, }, // Asia/Shanghai
    {   61, 3137,   27398,  8, }, // Asia/Singapore
    {   61, 4199,   27406, 14, }, // Asia/Srednekolymsk
    {   61, 3100,   27420,  6, }, // Asia/Taipei
    {   61, 4683,    2676,  7, }, // Asia/Tashkent
    {   61, 3817,   27426,  7, }, // Asia/Tbilisi
    {   61, 2638,   27433,  7, }, // Asia/Tehran
    {   61, 1284,   27440,  7, }, // Asia/Thimphu
    {   61, 2687,   27447,  5, }, // Asia/Tokyo
    {   61, 4474,    2706,  5, }, // Asia/Tomsk
    {   61, 1346,   27452, 10, }, // Asia/Ulaanbaatar
    {   61, 1126,   27462,  6, }, // Asia/Urumqi
    {   61, 8685,   27468,  9, }, // Asia/Ust-Nera
    {   61, 6940,   27477,  8, }, // Asia/Vientiane
    {   61, 4612,   27485, 12, }, // Asia/Vladivostok
    {   61, 4730,   27497,  5, }, // Asia/Yakutsk
    {   61, 1202,   27502,  6, }, // Asia/Yangon
    {   61, 3785,   27508, 12, }, // Asia/Yekaterinburg
    {   61, 3649,   27520,  6, }, // Asia/Yerevan
    {   61, 3565,   27526, 16, }, // Atlantic/Azores
    {   61, 5588,   27542, 18, }, // Atlantic/Bermuda
    {   61, 6591,   27560, 17, }, // Atlantic/Canary
    {   61, 3629,   27577, 10, }, // Atlantic/Cape_Verde
    {   61, 1379,   27587, 17, }, // Atlantic/Faroe
    {   61, 8699,   27604,  7, }, // Atlantic/Madeira
    {   61, 2614,   27611,  9, }, // Atlantic/Reykjavik
    {   61, 7235,   27620, 17, }, // Atlantic/South_Georgia
    {   61, 6784,   27637, 20, }, // Atlantic/St_Helena
    {   61, 6607,   27657, 11, }, // Atlantic/Stanley
    {   61, 1654,   27668,  8, }, // Australia/Adelaide
    {   61, 1619,   27676,  7, }, // Australia/Brisbane
    {   61, 1783,   27683, 10, }, // Australia/Broken_Hill
    {   61, 1567,   27693,  6, }, // Australia/Darwin
    {   61, 3539,   27699,  4, }, // Australia/Eucla
    {   61, 1500,    2963,  6, }, // Australia/Hobart
    {   61, 8716,   27703,  8, }, // Australia/Lindeman
    {   61, 1531,    2977,  8, }, // Australia/Lord_Howe
    {   61, 1711,   27711,  8, }, // Australia/Melbourne
    {   61, 1746,    2992,  4, }, // Australia/Perth
    {   61, 1447,   27719,  6, }, // Australia/Sydney
    {   61, 6390,   27725,  9, }, // Europe/Amsterdam
    {   61, 6149,   27734,  6, }, // Europe/Andorra
    {   61, 3522,   27740,  9, }, // Europe/Astrakhan
    {   61, 6577,   27749,  5, }, // Europe/Athens
    {   61, 6285,   27754,  7, }, // Europe/Belgrade
    {   61, 4659,   27761,  6, }, // Europe/Berlin
    {   61, 6437,   27767, 10, }, // Europe/Bratislava
    {   61, 6178,   27777,  7, }, // Europe/Brussels
    {   61, 3830,   27784,  8, }, // Europe/Bucharest
    {   61, 3708,    3069,  8, }, // Europe/Budapest
    {   61, 8751,   27792,  8, }, // Europe/Busingen
    {   61, 2475,   27800,  7, }, // Europe/Chisinau
    {   61, 6238,   27807, 10, }, // Europe/Copenhagen
    {   61, 2260,   27817,  6, }, // Europe/Dublin
    {   61, 6256,   27823,  9, }, // Europe/Gibraltar
    {   61, 8767,   27832,  6, }, // Europe/Guernsey
    {   61, 6561,   27838,  9, }, // Europe/Helsinki
    {   61, 8783,   27847, 11, }, // Europe/Isle_of_Man
    {   61, 1097,    3155,  7, }, // Europe/Istanbul
    {   61, 8802,   27858,  6, }, // Europe/Jersey
    {   61, 3881,   27864, 11, }, // Europe/Kaliningrad
    {   61, 8924,   27875,  5, }, // Europe/Kirov
    {   61, 2419,   27880,  4, }, // Europe/Kyiv
    {   61, 3078,   27884,  7, }, // Europe/Lisbon
    {   61, 6455,   27891,  7, }, // Europe/Ljubljana
    {   61, 2393,   27898,  6, }, // Europe/London
    {   61, 6314,   27904, 10, }, // Europe/Luxembourg
    {   61, 4794,   27914,  6, }, // Europe/Madrid
    {   61, 6346,   27920,  6, }, // Europe/Malta
    {   61, 6531,   27926, 12, }, // Europe/Mariehamn
    {   61, 3595,   27938,  5, }, // Europe/Minsk
    {   61, 6359,   27943,  6, }, // Europe/Monaco
    {   61, 3443,   27949,  6, }, // Europe/Moscow
    {   61, 6407,   27955,  4, }, // Europe/Oslo
    {   61, 4186,   27959,  5, }, // Europe/Paris
    {   61, 6373,   27964,  9, }, // Europe/Podgorica
    {   61, 6224,    3276,  5, }, // Europe/Prague
    {   61, 8816,   27973,  4, }, // Europe/Riga
    {   61, 6273,   27977,  3, }, // Europe/Rome
    {   61, 4233,    3288,  6, }, // Europe/Samara
    {   61, 6419,   27980, 10, }, // Europe/San_Marino
    {   61, 6194,   27990,  7, }, // Europe/Sarajevo
    {   61, 4353,   27997,  7, }, // Europe/Saratov
    {   61, 8828,   28004, 11, }, // Europe/Simferopol
    {   61, 6332,   28015,  6, }, // Europe/Skopje
    {   61, 6548,   28021,  5, }, // Europe/Sofia
    {   61, 6472,   28026,  9, }, // Europe/Stockholm
    {   61, 8846,   28035,  5, }, // Europe/Tallinn
    {   61, 6135,   28040,  6, }, // Europe/Tirane
    {   61, 8861,   28046,  9, }, // Europe/Ulyanovsk
    {   61, 6301,    3367,  5, }, // Europe/Vaduz
    {   61, 6516,   28055,  7, }, // Europe/Vatican
    {   61, 6164,    3379,  4, }, // Europe/Vienna
    {   61, 8878,   28062,  7, }, // Europe/Vilnius
    {   61, 4629,   28069,  9, }, // Europe/Volgograd
    {   61, 3055,    3399,  7, }, // Europe/Warsaw
    {   61, 6210,   28078,  6, }, // Europe/Zagreb
    {   61, 6489,   28084,  5, }, // Europe/Zurich
    {   61, 4980,   28089, 12, }, // Indian/Antananarivo
    {   61, 6910,   28101,  5, }, // Indian/Chagos
    {   61, 5979,   28106, 13, }, // Indian/Christmas
    {   61, 5996,   28119, 17, }, // Indian/Cocos
    {   61, 4931,   28136, 17, }, // Indian/Comoro
    {   61, 6624,    3469,  8, }, // Indian/Kerguelen
    {   61, 7223,   28153,  3, }, // Indian/Mahe
    {   61, 7013,   28156,  8, }, // Indian/Maldives
    {   61, 3971,   28164,  8, }, // Indian/Mauritius
    {   61, 5000,   28172,  5, }, // Indian/Mayotte
    {   61, 7174,   28177,  7, }, // Indian/Reunion
    {   61, 4324,   28184,  4, }, // Pacific/Apia
    {   61,  932,    3511,  6, }, // Pacific/Auckland
    {   61, 3608,   28188,  9, }, // Pacific/Bougainville
    {   61, 2863,   28197,  5, }, // Pacific/Chatham
    {   61, 3022,    3530,  4, }, // Pacific/Chuuk
    {   61, 2201,   28202, 17, }, // Pacific/Easter
    {   61, 7323,   28219,  5, }, // Pacific/Efate
    {   61, 7290,   28224,  7, }, // Pacific/Fakaofo
    {   61, 3804,   28231,  5, }, // Pacific/Fiji
    {   61, 7306,   28236,  8, }, // Pacific/Funafuti
    {   61, 6641,   28244,  9, }, // Pacific/Galapagos
    {   61, 6659,   28253, 14, }, // Pacific/Gambier
    {   61, 3724,    3604, 11, }, // Pacific/Guadalcanal
    {   61, 5935,    3615,  4, }, // Pacific/Guam
    {   61, 2929,   28267,  8, }, // Pacific/Honolulu
    {   61, 2897,   28275,  9, }, // Pacific/Kanton
    {   61, 3900,   28284, 10, }, // Pacific/Kiritimati
    {   61, 6982,   28294,  6, }, // Pacific/Kosrae
    {   61, 2708,   28300, 10, }, // Pacific/Kwajalein
    {   61, 4779,   28310,  7, }, // Pacific/Majuro
    {   61, 3953,   28317, 18, }, // Pacific/Marquesas
    {   61, 8909,   28335,  6, }, // Pacific/Midway
    {   61, 7047,    3689,  5, }, // Pacific/Nauru
    {   61, 7095,   28341,  4, }, // Pacific/Niue
    {   61, 4070,   28345,  7, }, // Pacific/Norfolk
    {   61, 7061,    3705,  5, }, // Pacific/Noumea
    {   61, 2991,   28352,  9, }, // Pacific/Pago_Pago
    {   61, 7118,    3719,  5, }, // Pacific/Palau
    {   61, 7157,   28361,  7, }, // Pacific/Pitcairn
    {   61, 2961,   28368,  6, }, // Pacific/Pohnpei
    {   61, 4709,   28374, 11, }, // Pacific/Port_Moresby
    {   61, 6009,   28385,  9, }, // Pacific/Rarotonga
    {   61, 5948,   28394,  6, }, // Pacific/Saipan
    {   61, 7275,   28400,  5, }, // Pacific/Tahiti
    {   61, 6675,    3768,  6, }, // Pacific/Tarawa
    {   61, 4485,   28405,  9, }, // Pacific/Tongatapu
    {   61, 7355,   28414,  4, }, // Pacific/Wake
    {   61, 7368,   28418,  5, }, // Pacific/Wallis
    {   62,   14,       0,  6, }, // Africa/Asmara Bemba/Latin/Zambia
    {   62, 4337,       6,  8, }, // Africa/Sao_Tome
    {   62, 4154,      14,  8, }, // America/Asuncion
    {   62,  237,      22,  8, }, // America/Atikokan
    {   62, 7586,      30, 17, }, // America/Bahia_Banderas
    {   62, 3770,      47,  6, }, // America/Cancun
    {   62, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   62, 5640,      66,  7, }, // America/Curacao
    {   62,  481,      73, 13, }, // America/Indiana/Knox
    {   62, 7924,      86, 16, }, // America/Indiana/Marengo
    {   62, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   62, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   62, 8001,     139, 14, }, // America/Indiana/Vevay
    {   62, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   62, 8049,     171, 16, }, // America/Indiana/Winamac
    {   62, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   62, 5849,     207, 22, }, // America/Lower_Princes
    {   62, 8183,     229,  6, }, // America/Merida
    {   62, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   62, 8292,     255, 20, }, // America/North_Dakota/Center
    {   62, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   62,  393,     298,  4, }, // America/Nuuk
    {   62, 6862,     302, 16, }, // America/Scoresbysund
    {   62, 8471,     318, 14, }, // America/St_Barthelemy
    {   62, 2045,     332, 10, }, // America/St_Johns
    {   62, 5780,     342,  9, }, // America/St_Kitts
    {   62, 5797,     351,  9, }, // America/St_Lucia
    {   62,  855,     360, 10, }, // America/St_Thomas
    {   62, 5830,     370, 11, }, // America/St_Vincent
    {   62, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   62, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   62, 1152,     413,  9, }, // Asia/Kathmandu
    {   62,  992,     422,  7, }, // Asia/Kolkata
    {   62, 1178,     429,  5, }, // Asia/Macau
    {   62, 1202,     442,  6, }, // Asia/Yangon
    {   62, 1379,     448,  5, }, // Atlantic/Faroe
    {   62, 6784,     453, 10, }, // Atlantic/St_Helena
    {   62, 2419,     463,  4, }, // Europe/Kyiv
    {   62, 7174,     467,  7, }, // Indian/Reunion
    {   62, 3022,     474,  5, }, // Pacific/Chuuk
    {   62, 2961,     479,  7, }, // Pacific/Pohnpei
    {   63,   14,       0,  6, }, // Africa/Asmara Bena/Latin/Tanzania
    {   63, 4337,       6,  8, }, // Africa/Sao_Tome
    {   63, 4154,      14,  8, }, // America/Asuncion
    {   63,  237,      22,  8, }, // America/Atikokan
    {   63, 7586,      30, 17, }, // America/Bahia_Banderas
    {   63, 3770,      47,  6, }, // America/Cancun
    {   63, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   63, 5640,      66,  7, }, // America/Curacao
    {   63,  481,      73, 13, }, // America/Indiana/Knox
    {   63, 7924,      86, 16, }, // America/Indiana/Marengo
    {   63, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   63, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   63, 8001,     139, 14, }, // America/Indiana/Vevay
    {   63, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   63, 8049,     171, 16, }, // America/Indiana/Winamac
    {   63, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   63, 5849,     207, 22, }, // America/Lower_Princes
    {   63, 8183,     229,  6, }, // America/Merida
    {   63, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   63, 8292,     255, 20, }, // America/North_Dakota/Center
    {   63, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   63,  393,     298,  4, }, // America/Nuuk
    {   63, 6862,     302, 16, }, // America/Scoresbysund
    {   63, 8471,     318, 14, }, // America/St_Barthelemy
    {   63, 2045,     332, 10, }, // America/St_Johns
    {   63, 5780,     342,  9, }, // America/St_Kitts
    {   63, 5797,     351,  9, }, // America/St_Lucia
    {   63,  855,     360, 10, }, // America/St_Thomas
    {   63, 5830,     370, 11, }, // America/St_Vincent
    {   63, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   63, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   63, 1152,     413,  9, }, // Asia/Kathmandu
    {   63,  992,     422,  7, }, // Asia/Kolkata
    {   63, 1178,     429,  5, }, // Asia/Macau
    {   63, 1202,     442,  6, }, // Asia/Yangon
    {   63, 1379,     448,  5, }, // Atlantic/Faroe
    {   63, 6784,     453, 10, }, // Atlantic/St_Helena
    {   63, 2419,     463,  4, }, // Europe/Kyiv
    {   63, 7174,     467,  7, }, // Indian/Reunion
    {   63, 3022,     474,  5, }, // Pacific/Chuuk
    {   63, 2961,     479,  7, }, // Pacific/Pohnpei
    {   64,   14,       0,  6, }, // Africa/Asmara Bhojpuri/Devanagari/India
    {   64, 4337,       6,  8, }, // Africa/Sao_Tome
    {   64, 4154,      14,  8, }, // America/Asuncion
    {   64,  237,      22,  8, }, // America/Atikokan
    {   64, 7586,      30, 17, }, // America/Bahia_Banderas
    {   64, 3770,      47,  6, }, // America/Cancun
    {   64, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   64, 5640,      66,  7, }, // America/Curacao
    {   64,  481,      73, 13, }, // America/Indiana/Knox
    {   64, 7924,      86, 16, }, // America/Indiana/Marengo
    {   64, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   64, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   64, 8001,     139, 14, }, // America/Indiana/Vevay
    {   64, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   64, 8049,     171, 16, }, // America/Indiana/Winamac
    {   64, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   64, 5849,     207, 22, }, // America/Lower_Princes
    {   64, 8183,     229,  6, }, // America/Merida
    {   64, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   64, 8292,     255, 20, }, // America/North_Dakota/Center
    {   64, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   64,  393,     298,  4, }, // America/Nuuk
    {   64, 6862,     302, 16, }, // America/Scoresbysund
    {   64, 8471,     318, 14, }, // America/St_Barthelemy
    {   64, 2045,     332, 10, }, // America/St_Johns
    {   64, 5780,     342,  9, }, // America/St_Kitts
    {   64, 5797,     351,  9, }, // America/St_Lucia
    {   64,  855,     360, 10, }, // America/St_Thomas
    {   64, 5830,     370, 11, }, // America/St_Vincent
    {   64, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   64, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   64, 1152,     413,  9, }, // Asia/Kathmandu
    {   64,  992,     422,  7, }, // Asia/Kolkata
    {   64, 1178,     429,  5, }, // Asia/Macau
    {   64, 1202,     442,  6, }, // Asia/Yangon
    {   64, 1379,     448,  5, }, // Atlantic/Faroe
    {   64, 6784,     453, 10, }, // Atlantic/St_Helena
    {   64, 2419,     463,  4, }, // Europe/Kyiv
    {   64, 7174,     467,  7, }, // Indian/Reunion
    {   64, 3022,     474,  5, }, // Pacific/Chuuk
    {   64, 2961,     479,  7, }, // Pacific/Pohnpei
    {   65,   14,       0,  6, }, // Africa/Asmara Blin/Ethiopic/Eritrea
    {   65, 4337,       6,  8, }, // Africa/Sao_Tome
    {   65, 4154,      14,  8, }, // America/Asuncion
    {   65,  237,      22,  8, }, // America/Atikokan
    {   65, 7586,      30, 17, }, // America/Bahia_Banderas
    {   65, 3770,      47,  6, }, // America/Cancun
    {   65, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   65, 5640,      66,  7, }, // America/Curacao
    {   65,  481,      73, 13, }, // America/Indiana/Knox
    {   65, 7924,      86, 16, }, // America/Indiana/Marengo
    {   65, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   65, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   65, 8001,     139, 14, }, // America/Indiana/Vevay
    {   65, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   65, 8049,     171, 16, }, // America/Indiana/Winamac
    {   65, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   65, 5849,     207, 22, }, // America/Lower_Princes
    {   65, 8183,     229,  6, }, // America/Merida
    {   65, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   65, 8292,     255, 20, }, // America/North_Dakota/Center
    {   65, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   65,  393,     298,  4, }, // America/Nuuk
    {   65, 6862,     302, 16, }, // America/Scoresbysund
    {   65, 8471,     318, 14, }, // America/St_Barthelemy
    {   65, 2045,     332, 10, }, // America/St_Johns
    {   65, 5780,     342,  9, }, // America/St_Kitts
    {   65, 5797,     351,  9, }, // America/St_Lucia
    {   65,  855,     360, 10, }, // America/St_Thomas
    {   65, 5830,     370, 11, }, // America/St_Vincent
    {   65, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   65, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   65, 1152,     413,  9, }, // Asia/Kathmandu
    {   65,  992,     422,  7, }, // Asia/Kolkata
    {   65, 1178,     429,  5, }, // Asia/Macau
    {   65, 1202,     442,  6, }, // Asia/Yangon
    {   65, 1379,     448,  5, }, // Atlantic/Faroe
    {   65, 6784,     453, 10, }, // Atlantic/St_Helena
    {   65, 2419,     463,  4, }, // Europe/Kyiv
    {   65, 7174,     467,  7, }, // Indian/Reunion
    {   65, 3022,     474,  5, }, // Pacific/Chuuk
    {   65, 2961,     479,  7, }, // Pacific/Pohnpei
    {   66, 6751,   28423,  7, }, // Africa/Abidjan Bodo/Devanagari/India
    {   66, 6723,   28430,  5, }, // Africa/Accra
    {   66, 4961,   28435, 12, }, // Africa/Addis_Ababa
    {   66, 7383,   28447,  9, }, // Africa/Algiers
    {   66,   14,   28456,  7, }, // Africa/Asmara
    {   66,   44,   28463,  6, }, // Africa/Bamako
    {   66, 5159,   28469,  6, }, // Africa/Bangui
    {   66, 6709,   28475,  6, }, // Africa/Banjul
    {   66, 7398,   28481,  7, }, // Africa/Bissau
    {   66, 4859,   28488, 10, }, // Africa/Blantyre
    {   66, 5189,   28498, 11, }, // Africa/Brazzaville
    {   66, 4824,   28509, 10, }, // Africa/Bujumbura
    {   66, 2242,   28519,  5, }, // Africa/Cairo
    {   66, 4019,   28524, 11, }, // Africa/Casablanca
    {   66, 7412,   28535,  5, }, // Africa/Ceuta
    {   66, 6736,   28540,  7, }, // Africa/Conakry
    {   66, 6803,   28547,  5, }, // Africa/Dakar
    {   66, 5032,   28552, 11, }, // Africa/Dar_es_Salaam
    {   66, 4945,   28563,  6, }, // Africa/Djibouti
    {   66, 5145,   28569,  5, }, // Africa/Douala
    {   66, 5068,   28574,  7, }, // Africa/El_Aaiun
    {   66, 6816,   28581,  8, }, // Africa/Freetown
    {   66, 4808,   28589,  7, }, // Africa/Gaborone
    {   66, 4917,   28596,  6, }, // Africa/Harare
    {   66, 4381,   28602, 11, }, // Africa/Johannesburg
    {   66, 4401,   28613,  4, }, // Africa/Juba
    {   66, 5053,   28617,  8, }, // Africa/Kampala
    {   66, 4426,   28625,  7, }, // Africa/Khartoum
    {   66, 4875,   28632,  6, }, // Africa/Kigali
    {   66, 5208,   28638,  8, }, // Africa/Kinshasa
    {   66, 4646,   28646,  5, }, // Africa/Lagos
    {   66, 5238,   28651,  9, }, // Africa/Libreville
    {   66, 6832,   28660,  3, }, // Africa/Lome
    {   66, 5113,   28663,  7, }, // Africa/Luanda
    {   66, 4841,   28670, 10, }, // Africa/Lubumbashi
    {   66, 4903,   28680,  6, }, // Africa/Lusaka
    {   66, 5224,   28686,  6, }, // Africa/Malabo
    {   66, 4889,   28692,  6, }, // Africa/Maputo
    {   66, 5099,   28698,  6, }, // Africa/Maseru
    {   66, 5084,   28704,  8, }, // Africa/Mbabane
    {   66, 5015,   28712,  7, }, // Africa/Mogadishu
    {   66, 6997,   28719,  9, }, // Africa/Monrovia
    {   66, 3755,   28728,  7, }, // Africa/Nairobi
    {   66, 5173,   28735,  8, }, // Africa/Ndjamena
    {   66, 5256,   28743,  5, }, // Africa/Niamey
    {   66, 6766,   28748,  8, }, // Africa/Nouakchott
    {   66, 6690,   28756,  7, }, // Africa/Ouagadougou
    {   66, 5127,   28763,  9, }, // Africa/Porto-Novo
    {   66, 4337,   28772,  7, }, // Africa/Sao_Tome
    {   66, 2732,   28779,  8, }, // Africa/Tripoli
    {   66, 6503,   28787,  7, }, // Africa/Tunis
    {   66, 4054,   28794,  8, }, // Africa/Windhoek
    {   66,  132,   28802,  4, }, // America/Adak
    {   66, 3183,   28806,  6, }, // America/Anchorage
    {   66, 5524,   28812,  8, }, // America/Anguilla
    {   66, 5541,   28820,  9, }, // America/Antigua
    {   66, 4456,   28829,  9, }, // America/Araguaina
    {   66,  166,   28838, 11, }, // America/Argentina/Buenos_Aires
    {   66,   91,   28849, 10, }, // America/Argentina/Catamarca
    {   66,  270,   28859,  6, }, // America/Argentina/Cordoba
    {   66,  441,   28865,  5, }, // America/Argentina/Jujuy
    {   66, 7425,   28870,  8, }, // America/Argentina/La_Rioja
    {   66,  565,   28878,  8, }, // America/Argentina/Mendoza
    {   66, 7452,   28886, 10, }, // America/Argentina/Rio_Gallegos
    {   66, 7483,   28896,  6, }, // America/Argentina/Salta
    {   66, 7507,   28902,  9, }, // America/Argentina/San_Juan
    {   66, 5497,   28911,  8, }, // America/Argentina/San_Luis
    {   66, 7534,   28919,  7, }, // America/Argentina/Tucuman
    {   66, 7560,   28926,  8, }, // America/Argentina/Ushuaia
    {   66, 5557,   28934,  5, }, // America/Aruba
    {   66, 4154,   28939,  8, }, // America/Asuncion
    {   66,  237,   28947,  7, }, // America/Atikokan
    {   66, 3581,   28954,  6, }, // America/Bahia
    {   66, 7586,   28960, 16, }, // America/Bahia_Banderas
    {   66, 5571,   28976,  9, }, // America/Barbados
    {   66, 7609,   28985,  5, }, // America/Belem
    {   66, 5309,   28990,  5, }, // America/Belize
    {   66, 7623,   28995, 14, }, // America/Blanc-Sablon
    {   66, 7644,   29009,  9, }, // America/Boa_Vista
    {   66, 4263,   29018,  5, }, // America/Bogota
    {   66, 7662,   29023,  4, }, // America/Boise
    {   66, 7676,   29027, 15, }, // America/Cambridge_Bay
    {   66, 7698,   29042, 13, }, // America/Campo_Grande
    {   66, 3770,   29055,  6, }, // America/Cancun
    {   66, 4596,   29061,  7, }, // America/Caracas
    {   66, 4247,   29068,  5, }, // America/Cayenne
    {   66, 5398,   29073,  5, }, // America/Cayman
    {   66, 3251,   29078,  6, }, // America/Chicago
    {   66, 7719,   29084,  7, }, // America/Chihuahua
    {   66, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   66, 5324,   29091, 10, }, // America/Costa_Rica
    {   66, 7759,   29101,  8, }, // America/Creston
    {   66, 3693,   29109,  6, }, // America/Cuiaba
    {   66, 5640,   29115,  7, }, // America/Curacao
    {   66, 7775,   29122, 13, }, // America/Danmarkshavn
    {   66, 7796,   29135,  6, }, // America/Dawson
    {   66, 7811,   29141, 12, }, // America/Dawson_Creek
    {   66,  805,   29153,  6, }, // America/Denver
    {   66, 3351,   29159,  8, }, // America/Detroit
    {   66, 5656,   29167,  7, }, // America/Dominica
    {   66,  893,   29174,  7, }, // America/Edmonton
    {   66, 7832,   29181,  8, }, // America/Eirunepe
    {   66, 5343,   29189, 11, }, // America/El_Salvador
    {   66, 7849,   29200, 10, }, // America/Fort_Nelson
    {   66, 7869,   29210,  9, }, // America/Fortaleza
    {   66, 7887,   29219,  9, }, // America/Glace_Bay
    {   66, 6844,   29228,  7, }, // America/Goose_Bay
    {   66, 4514,   29235, 13, }, // America/Grand_Turk
    {   66, 5687,   29248,  8, }, // America/Grenada
    {   66, 5703,   29256,  9, }, // America/Guadeloupe
    {   66, 3662,   29265, 10, }, // America/Guatemala
    {   66, 4743,   29275,  8, }, // America/Guayaquil
    {   66, 6895,   29283,  6, }, // America/Guyana
    {   66, 1923,   29289,  9, }, // America/Halifax
    {   66, 2221,   29298,  6, }, // America/Havana
    {   66, 7905,   29304, 12, }, // America/Hermosillo
    {   66,  348,   29316, 13, }, // America/Indiana/Indianapolis
    {   66,  481,   29329, 15, }, // America/Indiana/Knox
    {   66, 7924,   29344, 17, }, // America/Indiana/Marengo
    {   66, 7948,   29361, 23, }, // America/Indiana/Petersburg
    {   66, 7975,   29384, 20, }, // America/Indiana/Tell_City
    {   66, 8001,   29404, 15, }, // America/Indiana/Vevay
    {   66, 8023,   29419, 21, }, // America/Indiana/Vincennes
    {   66, 8049,   29440, 18, }, // America/Indiana/Winamac
    {   66, 8073,   29458,  6, }, // America/Inuvik
    {   66,  660,   29464,  7, }, // America/Iqaluit
    {   66, 2665,   29471,  7, }, // America/Jamaica
    {   66, 5282,   29478,  4, }, // America/Juneau
    {   66,  521,   29482,  6, }, // America/Kentucky/Louisville
    {   66, 8088,   29488, 19, }, // America/Kentucky/Monticello
    {   66, 5621,   29507, 10, }, // America/Kralendijk
    {   66, 4278,   29517,  6, }, // America/La_Paz
    {   66, 7132,   29523,  4, }, // America/Lima
    {   66, 3390,   29527, 11, }, // America/Los_Angeles
    {   66, 5849,   29538, 24, }, // America/Lower_Princes
    {   66, 8116,   29562,  6, }, // America/Maceio
    {   66, 8131,   29568,  6, }, // America/Managua
    {   66, 1892,   29574,  5, }, // America/Manaus
    {   66, 5814,   29579,  6, }, // America/Marigot
    {   66, 5722,   29585,  9, }, // America/Martinique
    {   66, 8147,   29594,  7, }, // America/Matamoros
    {   66, 2779,   29601,  8, }, // America/Mazatlan
    {   66, 8165,   29609,  8, }, // America/Menominee
    {   66, 8183,   29617,  6, }, // America/Merida
    {   66, 8198,   29623, 12, }, // America/Metlakatla
    {   66, 2811,   29635, 13, }, // America/Mexico_City
    {   66, 4293,   29648,  9, }, // America/Miquelon
    {   66, 8217,   29657,  6, }, // America/Moncton
    {   66, 8233,   29663,  9, }, // America/Monterrey
    {   66, 4000,   29672, 10, }, // America/Montevideo
    {   66, 5741,   29682,  9, }, // America/Montserrat
    {   66, 5383,   29691,  7, }, // America/Nassau
    {   66, 3294,   29698,  8, }, // America/New_York
    {   66, 8251,   29611,  3, }, // America/Nome
    {   66, 1834,   29706,  7, }, // America/Noronha
    {   66, 8264,   29713, 19, }, // America/North_Dakota/Beulah
    {   66, 8292,   29732, 20, }, // America/North_Dakota/Center
    {   66, 8320,   29752, 22, }, // America/North_Dakota/New_Salem
    {   66,  393,   29774,  3, }, // America/Nuuk
    {   66, 8351,   29777,  7, }, // America/Ojinaga
    {   66, 5413,   29784,  6, }, // America/Panama
    {   66, 6106,   29790, 10, }, // America/Paramaribo
    {   66, 3224,   29800,  7, }, // America/Phoenix
    {   66, 3847,   29807, 15, }, // America/Port-au-Prince
    {   66, 5871,   29822, 13, }, // America/Port_of_Spain
    {   66, 8367,   29835, 12, }, // America/Porto_Velho
    {   66, 5760,   29847, 12, }, // America/Puerto_Rico
    {   66, 3932,   29859, 13, }, // America/Punta_Arenas
    {   66, 8387,   29872, 12, }, // America/Rankin_Inlet
    {   66, 8408,   29884,  6, }, // America/Recife
    {   66, 1979,   29890,  6, }, // America/Regina
    {   66, 8423,   29896,  8, }, // America/Resolute
    {   66,  695,   29904, 12, }, // America/Rio_Branco
    {   66, 8440,   29916,  9, }, // America/Santarem
    {   66, 2165,   29925,  9, }, // America/Santiago
    {   66, 6044,   29934, 12, }, // America/Santo_Domingo
    {   66, 1862,   29946,  9, }, // America/Sao_Paulo
    {   66, 6862,   29955, 14, }, // America/Scoresbysund
    {   66, 8457,   29969,  6, }, // America/Sitka
    {   66, 8471,   29975, 16, }, // America/St_Barthelemy
    {   66, 2045,   29991, 10, }, // America/St_Johns
    {   66, 5780,   30001, 11, }, // America/St_Kitts
    {   66, 5797,   30012, 12, }, // America/St_Lucia
    {   66,  855,   30024, 10, }, // America/St_Thomas
    {   66, 5830,   30034, 15, }, // America/St_Vincent
    {   66, 8493,   30049, 15, }, // America/Swift_Current
    {   66, 5363,   30064, 12, }, // America/Tegucigalpa
    {   66, 5673,   30076,  5, }, // America/Thule
    {   66,  313,   30081,  7, }, // America/Tijuana
    {   66,  608,   30088,  6, }, // America/Toronto
    {   66, 5605,   30094,  7, }, // America/Tortola
    {   66, 2077,   30101,  7, }, // America/Vancouver
    {   66, 2128,   30108, 10, }, // America/Whitehorse
    {   66,  734,   30118,  7, }, // America/Winnipeg
    {   66, 8515,   30125,  7, }, // America/Yakutat
    {   66, 5918,   30132,  4, }, // Antarctica/Casey
    {   66, 6027,   30136,  5, }, // Antarctica/Davis
    {   66, 6066,   30141, 15, }, // Antarctica/DumontDUrville
    {   66, 8531,   30156,  8, }, // Antarctica/Macquarie
    {   66, 7029,   30164,  4, }, // Antarctica/Mawson
    {   66, 7076,   30168,  9, }, // Antarctica/McMurdo
    {   66, 8552,   30177,  6, }, // Antarctica/Palmer
    {   66, 7189,   30183,  5, }, // Antarctica/Rothera
    {   66, 7258,   30188,  5, }, // Antarctica/Syowa
    {   66, 8570,   30193,  5, }, // Antarctica/Troll
    {   66, 7337,   30198,  6, }, // Antarctica/Vostok
    {   66, 1413,   30204, 10, }, // Arctic/Longyearbyen
    {   66, 5487,   30214,  4, }, // Asia/Aden
    {   66, 5297,   30218,  7, }, // Asia/Almaty
    {   66, 3870,   30225,  6, }, // Asia/Amman
    {   66, 5428,   30231,  6, }, // Asia/Anadyr
    {   66, 5440,   30237,  6, }, // Asia/Aqtau
    {   66, 5270,   30243,  7, }, // Asia/Aqtobe
    {   66,  964,   30250,  8, }, // Asia/Ashgabat
    {   66, 8587,   30258,  6, }, // Asia/Atyrau
    {   66, 3509,   30264,  6, }, // Asia/Baghdad
    {   66, 5451,   30270,  6, }, // Asia/Bahrain
    {   66, 3555,   29113,  4, }, // Asia/Baku
    {   66, 4368,   30276,  7, }, // Asia/Bangkok
    {   66, 3473,   30283,  7, }, // Asia/Barnaul
    {   66, 3988,   30290,  5, }, // Asia/Beirut
    {   66, 3680,   30295,  7, }, // Asia/Bishkek
    {   66, 5906,   30302,  6, }, // Asia/Brunei
    {   66, 4503,   30308,  4, }, // Asia/Chita
    {   66, 5963,   30312,  9, }, // Asia/Choibalsan
    {   66, 4413,   30321,  6, }, // Asia/Colombo
    {   66, 4442,   30327,  9, }, // Asia/Damascus
    {   66, 1060,   30336,  4, }, // Asia/Dhaka
    {   66, 6125,   30340,  4, }, // Asia/Dili
    {   66, 3498,   30344,  5, }, // Asia/Dubai
    {   66, 6092,   30349,  8, }, // Asia/Dushanbe
    {   66, 8599,   30357, 10, }, // Asia/Famagusta
    {   66, 8614,   30367,  4, }, // Asia/Gaza
    {   66, 4697,   30371,  7, }, // Asia/Hebron
    {   66, 1226,   30378,  6, }, // Asia/Ho_Chi_Minh
    {   66, 2576,   30384,  4, }, // Asia/Hong_Kong
    {   66, 4673,   30388,  5, }, // Asia/Hovd
    {   66, 4086,   30393, 10, }, // Asia/Irkutsk
    {   66, 6969,   30403,  8, }, // Asia/Jakarta
    {   66, 6955,   30411,  7, }, // Asia/Jayapura
    {   66, 1257,   30418,  9, }, // Asia/Jerusalem
    {   66, 3462,   30427,  5, }, // Asia/Kabul
    {   66, 4218,   30432,  8, }, // Asia/Kamchatka
    {   66, 4141,   30440,  5, }, // Asia/Karachi
    {   66, 1152,   30445,  9, }, // Asia/Kathmandu
    {   66, 8624,   30454,  8, }, // Asia/Khandyga
    {   66,  992,   30462,  6, }, // Asia/Kolkata
    {   66, 4099,   30468, 15, }, // Asia/Krasnoyarsk
    {   66, 4761,   30483, 13, }, // Asia/Kuala_Lumpur
    {   66, 5893,   30496,  6, }, // Asia/Kuching
    {   66, 5464,   30502,  5, }, // Asia/Kuwait
    {   66, 1178,   30507,  5, }, // Asia/Macau
    {   66, 3919,   30512,  7, }, // Asia/Magadan
    {   66, 1316,   30519,  9, }, // Asia/Makassar
    {   66, 7145,   30528,  5, }, // Asia/Manila
    {   66, 6883,   30533,  7, }, // Asia/Muscat
    {   66, 2446,   30540,  8, }, // Asia/Nicosia
    {   66, 8638,   30548, 14, }, // Asia/Novokuznetsk
    {   66, 4037,   30562, 12, }, // Asia/Novosibirsk
    {   66, 4131,   30574,  6, }, // Asia/Omsk
    {   66, 7108,   30580,  4, }, // Asia/Oral
    {   66, 6924,   30584,  7, }, // Asia/Phnom_Penh
    {   66, 8656,   30591,  9, }, // Asia/Pontianak
    {   66, 4116,   30600,  7, }, // Asia/Pyongyang
    {   66, 5476,   30607,  5, }, // Asia/Qatar
    {   66, 8671,   30612,  7, }, // Asia/Qostanay
    {   66, 4171,   30619,  9, }, // Asia/Qyzylorda
    {   66, 3486,   30628,  5, }, // Asia/Riyadh
    {   66, 4310,   30633,  7, }, // Asia/Sakhalin
    {   66, 7208,   30640,  8, }, // Asia/Samarkand
    {   66, 3116,   30648,  4, }, // Asia/Seoul
    {   66, 1020,   30652,  6, }, // Asia/Shanghai
    {   66, 3137,   30658,  8, }, // Asia/Singapore
    {   66, 4199,   30666, 16, }, // Asia/Srednekolymsk
    {   66, 3100,   30682,  6, }, // Asia/Taipei
    {   66, 4683,   30688,  8, }, // Asia/Tashkent
    {   66, 3817,   30696,  8, }, // Asia/Tbilisi
    {   66, 2638,   30704,  6, }, // Asia/Tehran
    {   66, 1284,   30710,  6, }, // Asia/Thimphu
    {   66, 2687,   30716,  6, }, // Asia/Tokyo
    {   66, 4474,   30722,  6, }, // Asia/Tomsk
    {   66, 1346,   30728,  9, }, // Asia/Ulaanbaatar
    {   66, 1126,   30737,  6, }, // Asia/Urumqi
    {   66, 8685,   30743,  9, }, // Asia/Ust-Nera
    {   66, 6940,   30752,  9, }, // Asia/Vientiane
    {   66, 4612,   30761, 12, }, // Asia/Vladivostok
    {   66, 4730,   30773,  9, }, // Asia/Yakutsk
    {   66, 1202,   30782,  5, }, // Asia/Yangon
    {   66, 3785,   30787, 15, }, // Asia/Yekaterinburg
    {   66, 3649,   30802,  7, }, // Asia/Yerevan
    {   66, 3565,   30809,  6, }, // Atlantic/Azores
    {   66, 5588,   30815,  8, }, // Atlantic/Bermuda
    {   66, 6591,   30823,  6, }, // Atlantic/Canary
    {   66, 3629,   30829, 10, }, // Atlantic/Cape_Verde
    {   66, 1379,   30839,  4, }, // Atlantic/Faroe
    {   66, 8699,   30843,  6, }, // Atlantic/Madeira
    {   66, 2614,   30849,  8, }, // Atlantic/Reykjavik
    {   66, 7235,   30857, 12, }, // Atlantic/South_Georgia
    {   66, 6784,   30869, 12, }, // Atlantic/St_Helena
    {   66, 6607,   30881,  7, }, // Atlantic/Stanley
    {   66, 1654,   30888,  6, }, // Australia/Adelaide
    {   66, 1619,   30894,  8, }, // Australia/Brisbane
    {   66, 1783,   30902, 11, }, // Australia/Broken_Hill
    {   66, 1567,   30913,  7, }, // Australia/Darwin
    {   66, 3539,   30920,  6, }, // Australia/Eucla
    {   66, 1500,   30926,  6, }, // Australia/Hobart
    {   66, 8716,   30932,  8, }, // Australia/Lindeman
    {   66, 1531,   30940,  9, }, // Australia/Lord_Howe
    {   66, 1711,   30949,  8, }, // Australia/Melbourne
    {   66, 1746,   30957,  5, }, // Australia/Perth
    {   66, 1447,   30962,  5, }, // Australia/Sydney
    {   66, 6390,   30967, 10, }, // Europe/Amsterdam
    {   66, 6149,   30977,  7, }, // Europe/Andorra
    {   66, 3522,   30984, 10, }, // Europe/Astrakhan
    {   66, 6577,   30994,  6, }, // Europe/Athens
    {   66, 6285,   31000,  8, }, // Europe/Belgrade
    {   66, 4659,   31008,  7, }, // Europe/Berlin
    {   66, 6437,   31015, 12, }, // Europe/Bratislava
    {   66, 6178,   31027, 11, }, // Europe/Brussels
    {   66, 3830,   31038,  9, }, // Europe/Bucharest
    {   66, 3708,   31047,  9, }, // Europe/Budapest
    {   66, 8751,   31056,  8, }, // Europe/Busingen
    {   66, 2475,   31064,  6, }, // Europe/Chisinau
    {   66, 6238,   31070,  9, }, // Europe/Copenhagen
    {   66, 2260,   31079,  6, }, // Europe/Dublin
    {   66, 6256,   31085, 11, }, // Europe/Gibraltar
    {   66, 8767,   31096,  7, }, // Europe/Guernsey
    {   66, 6561,   31103,  9, }, // Europe/Helsinki
    {   66, 8783,   31112, 11, }, // Europe/Isle_of_Man
    {   66, 1097,   31123,  9, }, // Europe/Istanbul
    {   66, 8802,   31132,  6, }, // Europe/Jersey
    {   66, 3881,   31138, 12, }, // Europe/Kaliningrad
    {   66, 8924,   31150,  5, }, // Europe/Kirov
    {   66, 2419,   31155,  3, }, // Europe/Kyiv
    {   66, 3078,   31158,  6, }, // Europe/Lisbon
    {   66, 6455,   31164, 10, }, // Europe/Ljubljana
    {   66, 2393,   31174,  5, }, // Europe/London
    {   66, 6314,   31179, 13, }, // Europe/Luxembourg
    {   66, 4794,   31192,  7, }, // Europe/Madrid
    {   66, 6346,   31199,  6, }, // Europe/Malta
    {   66, 6531,   31205,  7, }, // Europe/Mariehamn
    {   66, 3595,   31212,  7, }, // Europe/Minsk
    {   66, 6359,   31219,  5, }, // Europe/Monaco
    {   66, 3443,   31224,  5, }, // Europe/Moscow
    {   66, 6407,   31229,  5, }, // Europe/Oslo
    {   66, 4186,   31234,  5, }, // Europe/Paris
    {   66, 6373,   31239,  8, }, // Europe/Podgorica
    {   66, 6224,   31247,  5, }, // Europe/Prague
    {   66, 8816,   31252,  4, }, // Europe/Riga
    {   66, 6273,   31256,  3, }, // Europe/Rome
    {   66, 4233,   31259,  6, }, // Europe/Samara
    {   66, 6419,   31265, 10, }, // Europe/San_Marino
    {   66, 6194,   31275,  8, }, // Europe/Sarajevo
    {   66, 4353,   31283,  7, }, // Europe/Saratov
    {   66, 8828,   31290, 11, }, // Europe/Simferopol
    {   66, 6332,   31301,  8, }, // Europe/Skopje
    {   66, 6548,   31309,  5, }, // Europe/Sofia
    {   66, 6472,   31314,  8, }, // Europe/Stockholm
    {   66, 8846,   31322,  5, }, // Europe/Tallinn
    {   66, 6135,   31327,  5, }, // Europe/Tirane
    {   66, 8861,   31332, 12, }, // Europe/Ulyanovsk
    {   66, 6301,   31344,  5, }, // Europe/Vaduz
    {   66, 6516,   31349,  7, }, // Europe/Vatican
    {   66, 6164,   31356,  8, }, // Europe/Vienna
    {   66, 8878,   31364,  8, }, // Europe/Vilnius
    {   66, 4629,   31372, 10, }, // Europe/Volgograd
    {   66, 3055,   31382,  5, }, // Europe/Warsaw
    {   66, 6210,   31387,  7, }, // Europe/Zagreb
    {   66, 6489,   31394,  5, }, // Europe/Zurich
    {   66, 4980,   31399, 13, }, // Indian/Antananarivo
    {   66, 6910,   31412,  5, }, // Indian/Chagos
    {   66, 5979,   31417, 16, }, // Indian/Christmas
    {   66, 5996,   31433,  4, }, // Indian/Cocos
    {   66, 4931,   31437,  5, }, // Indian/Comoro
    {   66, 6624,   31442,  9, }, // Indian/Kerguelen
    {   66, 7223,   31451,  4, }, // Indian/Mahe
    {   66, 7013,   31455,  8, }, // Indian/Maldives
    {   66, 3971,   31463,  8, }, // Indian/Mauritius
    {   66, 5000,   31471,  8, }, // Indian/Mayotte
    {   66, 7174,   31479,  9, }, // Indian/Reunion
    {   66, 4324,   31488,  5, }, // Pacific/Apia
    {   66,  932,   31493,  7, }, // Pacific/Auckland
    {   66, 3608,   31500,  8, }, // Pacific/Bougainville
    {   66, 2863,   31508,  5, }, // Pacific/Chatham
    {   66, 3022,   31513,  3, }, // Pacific/Chuuk
    {   66, 2201,   31516,  9, }, // Pacific/Easter
    {   66, 7323,   31525,  5, }, // Pacific/Efate
    {   66, 7290,   31530,  7, }, // Pacific/Fakaofo
    {   66, 3804,   31537,  4, }, // Pacific/Fiji
    {   66, 7306,   31541,  8, }, // Pacific/Funafuti
    {   66, 6641,   31549,  9, }, // Pacific/Galapagos
    {   66, 6659,   31558,  9, }, // Pacific/Gambier
    {   66, 3724,   31567, 12, }, // Pacific/Guadalcanal
    {   66, 5935,   31579,  5, }, // Pacific/Guam
    {   66, 2929,   31584,  6, }, // Pacific/Honolulu
    {   66, 2897,   31590, 10, }, // Pacific/Kanton
    {   66, 3900,   31600, 10, }, // Pacific/Kiritimati
    {   66, 6982,   31610,  5, }, // Pacific/Kosrae
    {   66, 2708,   31615,  9, }, // Pacific/Kwajalein
    {   66, 4779,   31624,  6, }, // Pacific/Majuro
    {   66, 3953,   31630,  9, }, // Pacific/Marquesas
    {   66, 8909,   31639,  5, }, // Pacific/Midway
    {   66, 7047,   31644,  5, }, // Pacific/Nauru
    {   66, 7095,   31649,  4, }, // Pacific/Niue
    {   66, 4070,   31653,  5, }, // Pacific/Norfolk
    {   66, 7061,   31658,  6, }, // Pacific/Noumea
    {   66, 2991,   31664,  9, }, // Pacific/Pago_Pago
    {   66, 7118,   31673,  5, }, // Pacific/Palau
    {   66, 7157,   31678,  8, }, // Pacific/Pitcairn
    {   66, 2961,   31686,  5, }, // Pacific/Pohnpei
    {   66, 4709,   31691, 11, }, // Pacific/Port_Moresby
    {   66, 6009,   31702,  8, }, // Pacific/Rarotonga
    {   66, 5948,   31710,  6, }, // Pacific/Saipan
    {   66, 7275,   31716,  6, }, // Pacific/Tahiti
    {   66, 6675,   31722,  6, }, // Pacific/Tarawa
    {   66, 4485,   31728,  8, }, // Pacific/Tongatapu
    {   66, 7355,   31736,  3, }, // Pacific/Wake
    {   66, 7368,   31739,  5, }, // Pacific/Wallis
    {   67,   14,       0,  6, }, // Africa/Asmara Bosnian/Latin/Bosnia and Herzegovina
    {   67, 2242,   24913,  5, }, // Africa/Cairo
    {   67, 4019,   31744, 10, }, // Africa/Casablanca
    {   67, 4945,   31754,  7, }, // Africa/Djibouti
    {   67, 4426,    4693,  6, }, // Africa/Khartoum
    {   67, 5015,   31761,  7, }, // Africa/Mogadishu
    {   67, 4337,       6,  8, }, // Africa/Sao_Tome
    {   67, 5524,   31768,  7, }, // America/Anguilla
    {   67, 5541,   31775,  7, }, // America/Antigua
    {   67, 4154,      14,  8, }, // America/Asuncion
    {   67,  237,      22,  8, }, // America/Atikokan
    {   67, 7586,    3834, 14, }, // America/Bahia_Banderas
    {   67, 3770,    3861,  6, }, // America/Cancun
    {   67, 5398,    4978,  6, }, // America/Cayman
    {   67, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   67, 5324,   31782,  9, }, // America/Costa_Rica
    {   67, 5640,    5033,  7, }, // America/Curacao
    {   67, 5656,   20071,  8, }, // America/Dominica
    {   67, 5343,    5090,  8, }, // America/El_Salvador
    {   67, 5703,   31791,  9, }, // America/Guadeloupe
    {   67, 3662,   31800,  9, }, // America/Guatemala
    {   67,  481,      73, 13, }, // America/Indiana/Knox
    {   67, 7924,      86, 16, }, // America/Indiana/Marengo
    {   67, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   67, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   67, 8001,     139, 14, }, // America/Indiana/Vevay
    {   67, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   67, 8049,     171, 16, }, // America/Indiana/Winamac
    {   67, 2665,   31809,  7, }, // America/Jamaica
    {   67, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   67, 5849,     207, 22, }, // America/Lower_Princes
    {   67, 8183,    3899,  6, }, // America/Merida
    {   67, 8264,   31816, 23, }, // America/North_Dakota/Beulah
    {   67, 8292,   31839, 23, }, // America/North_Dakota/Center
    {   67, 8320,   31862, 26, }, // America/North_Dakota/New_Salem
    {   67,  393,     298,  4, }, // America/Nuuk
    {   67, 5760,   31888,  9, }, // America/Puerto_Rico
    {   67, 6862,     302, 16, }, // America/Scoresbysund
    {   67, 8471,     318, 14, }, // America/St_Barthelemy
    {   67, 2045,     332, 10, }, // America/St_Johns
    {   67, 5780,     342,  9, }, // America/St_Kitts
    {   67, 5797,     351,  9, }, // America/St_Lucia
    {   67,  855,     360, 10, }, // America/St_Thomas
    {   67, 5830,     370, 11, }, // America/St_Vincent
    {   67, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   67, 5297,    5890,  6, }, // Asia/Almaty
    {   67, 3870,    5896,  4, }, // Asia/Amman
    {   67, 5428,    5900,  6, }, // Asia/Anadyr
    {   67, 5440,    5906,  5, }, // Asia/Aqtau
    {   67, 5270,   31897,  7, }, // Asia/Aqtobe
    {   67,  964,   31904,  7, }, // Asia/Ashgabat
    {   67, 8587,   31911,  6, }, // Asia/Atyrau
    {   67, 3509,    4063,  6, }, // Asia/Baghdad
    {   67, 5451,    4069,  7, }, // Asia/Bahrain
    {   67, 3988,    5930,  6, }, // Asia/Beirut
    {   67, 3680,   31917,  6, }, // Asia/Bishkek
    {   67, 5906,   31923,  7, }, // Asia/Brunei
    {   67, 5963,   31930,  9, }, // Asia/Choibalsan
    {   67, 4413,    5949,  7, }, // Asia/Colombo
    {   67, 4442,    4102,  6, }, // Asia/Damascus
    {   67, 1060,    5956,  4, }, // Asia/Dhaka
    {   67, 6092,   31939,  7, }, // Asia/Dushanbe
    {   67, 1226,   31946,  9, }, // Asia/Ho_Chi_Minh
    {   67, 6969,   31955,  8, }, // Asia/Jakarta
    {   67, 6955,   31963,  9, }, // Asia/Jayapura
    {   67, 1257,   31972,  9, }, // Asia/Jerusalem
    {   67, 4218,   31981,  8, }, // Asia/Kamchatka
    {   67, 4141,   31989,  6, }, // Asia/Karachi
    {   67, 1152,    6018,  8, }, // Asia/Kathmandu
    {   67, 8624,   31995,  7, }, // Asia/Khandyga
    {   67,  992,     422,  7, }, // Asia/Kolkata
    {   67, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {   67, 5893,   32002,  6, }, // Asia/Kuching
    {   67, 5464,    6058,  6, }, // Asia/Kuwait
    {   67, 1178,   32008,  5, }, // Asia/Macau
    {   67, 1316,    6069,  7, }, // Asia/Makassar
    {   67, 6883,    4227,  6, }, // Asia/Muscat
    {   67, 2446,   32013,  8, }, // Asia/Nicosia
    {   67, 8638,   32021, 12, }, // Asia/Novokuznetsk
    {   67, 6924,   20928,  8, }, // Asia/Phnom_Penh
    {   67, 4116,   32033,  9, }, // Asia/Pyongyang
    {   67, 5476,    4233,  5, }, // Asia/Qatar
    {   67, 8671,    6103,  8, }, // Asia/Qostanay
    {   67, 4171,    6111,  9, }, // Asia/Qyzylorda
    {   67, 3486,   32042,  5, }, // Asia/Riyadh
    {   67, 4310,   32047,  7, }, // Asia/Sakhalin
    {   67, 3116,    6131,  4, }, // Asia/Seoul
    {   67, 1020,   32054,  6, }, // Asia/Shanghai
    {   67, 3137,   19291,  8, }, // Asia/Singapore
    {   67, 3100,    6163,  6, }, // Asia/Taipei
    {   67, 4683,   32060,  7, }, // Asia/Tashkent
    {   67, 2638,    6175,  7, }, // Asia/Tehran
    {   67, 2687,    4257,  5, }, // Asia/Tokyo
    {   67, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {   67, 1126,   32067,  6, }, // Asia/Urumqi
    {   67, 6940,   32073, 10, }, // Asia/Vientiane
    {   67, 4730,    6206,  7, }, // Asia/Yakutsk
    {   67, 1202,    6213,  6, }, // Asia/Yangon
    {   67, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {   67, 3649,    6231,  7, }, // Asia/Yerevan
    {   67, 3565,   32083,  5, }, // Atlantic/Azores
    {   67, 6591,    4299,  6, }, // Atlantic/Canary
    {   67, 3629,   21064, 10, }, // Atlantic/Cape_Verde
    {   67, 1379,     448,  5, }, // Atlantic/Faroe
    {   67, 2614,   32088,  9, }, // Atlantic/Reykjavik
    {   67, 6784,   32097, 12, }, // Atlantic/St_Helena
    {   67, 1531,   21137,  8, }, // Australia/Lord_Howe
    {   67, 1711,    6359,  7, }, // Australia/Melbourne
    {   67, 1746,   21145,  4, }, // Australia/Perth
    {   67, 1447,    6371,  6, }, // Australia/Sydney
    {   67, 6149,   32109,  6, }, // Europe/Andorra
    {   67, 3522,   32115,  8, }, // Europe/Astrakhan
    {   67, 6577,   32123,  5, }, // Europe/Athens
    {   67, 6285,    6398,  7, }, // Europe/Belgrade
    {   67, 6178,   32128,  6, }, // Europe/Brussels
    {   67, 3830,   32134,  8, }, // Europe/Bucharest
    {   67, 3708,   32142, 10, }, // Europe/Budapest
    {   67, 2475,   32152,  8, }, // Europe/Chisinau
    {   67, 6238,    4376, 10, }, // Europe/Copenhagen
    {   67, 2260,   32160,  6, }, // Europe/Dublin
    {   67, 8767,   32166,  6, }, // Europe/Guernsey
    {   67, 8783,   32172, 10, }, // Europe/Isle_of_Man
    {   67, 3881,   32182, 12, }, // Europe/Kaliningrad
    {   67, 2419,   32194,  5, }, // Europe/Kyiv
    {   67, 3078,   32199,  7, }, // Europe/Lisbon
    {   67, 6314,    6510, 10, }, // Europe/Luxembourg
    {   67, 6359,    6525,  6, }, // Europe/Monaco
    {   67, 3443,   21290,  6, }, // Europe/Moscow
    {   67, 4186,   32206,  5, }, // Europe/Paris
    {   67, 6224,    6545,  4, }, // Europe/Prague
    {   67, 6273,   32211,  3, }, // Europe/Rome
    {   67, 8828,   32214, 11, }, // Europe/Simferopol
    {   67, 6332,   32225,  7, }, // Europe/Skopje
    {   67, 6548,   32232,  6, }, // Europe/Sofia
    {   67, 6472,   32238,  8, }, // Europe/Stockholm
    {   67, 8846,    6594,  5, }, // Europe/Tallinn
    {   67, 6135,   19608,  6, }, // Europe/Tirane
    {   67, 6516,    6614,  7, }, // Europe/Vatican
    {   67, 6164,   32246,  3, }, // Europe/Vienna
    {   67, 3055,   32249,  7, }, // Europe/Warsaw
    {   67, 6489,   32256,  5, }, // Europe/Zurich
    {   67, 5979,   32261, 14, }, // Indian/Christmas
    {   67, 5996,   32275, 15, }, // Indian/Cocos
    {   67, 7013,   32290,  7, }, // Indian/Maldives
    {   67, 3971,   32297, 10, }, // Indian/Mauritius
    {   67, 7174,     467,  7, }, // Indian/Reunion
    {   67, 3022,     474,  5, }, // Pacific/Chuuk
    {   67, 3804,   32307,  5, }, // Pacific/Fiji
    {   67, 2929,    4512,  8, }, // Pacific/Honolulu
    {   67, 2897,    4520,  9, }, // Pacific/Kanton
    {   67, 7157,    6773,  7, }, // Pacific/Pitcairn
    {   67, 2961,     479,  7, }, // Pacific/Pohnpei
    {   67, 7368,   32312,  5, }, // Pacific/Wallis
    {   68, 6751,     486,  6, }, // Africa/Abidjan Bosnian/Cyrillic/Bosnia and Herzegovina
    {   68, 6723,   25628,  4, }, // Africa/Accra
    {   68, 4961,   32317, 10, }, // Africa/Addis_Ababa
    {   68, 7383,     508,  5, }, // Africa/Algiers
    {   68,   14,     513,  6, }, // Africa/Asmara
    {   68,   44,     519,  6, }, // Africa/Bamako
    {   68, 5159,   32327,  6, }, // Africa/Bangui
    {   68, 6709,     530,  6, }, // Africa/Banjul
    {   68, 7398,   32333,  5, }, // Africa/Bissau
    {   68, 4859,   32338,  7, }, // Africa/Blantyre
    {   68, 5189,   32345,  8, }, // Africa/Brazzaville
    {   68, 4824,   32353,  9, }, // Africa/Bujumbura
    {   68, 2242,   32362,  5, }, // Africa/Cairo
    {   68, 4019,   32367, 10, }, // Africa/Casablanca
    {   68, 7412,     581,  5, }, // Africa/Ceuta
    {   68, 6736,     586,  7, }, // Africa/Conakry
    {   68, 6803,     593,  5, }, // Africa/Dakar
    {   68, 5032,     598, 12, }, // Africa/Dar_es_Salaam
    {   68, 4945,     610,  6, }, // Africa/Djibouti
    {   68, 5145,     616,  5, }, // Africa/Douala
    {   68, 5068,   32377,  7, }, // Africa/El_Aaiun
    {   68, 6816,     628,  7, }, // Africa/Freetown
    {   68, 4808,     635,  7, }, // Africa/Gaborone
    {   68, 4917,   32384,  6, }, // Africa/Harare
    {   68, 4381,   32390, 11, }, // Africa/Johannesburg
    {   68, 4401,     661,  4, }, // Africa/Juba
    {   68, 5053,     665,  7, }, // Africa/Kampala
    {   68, 4426,   32401,  6, }, // Africa/Khartoum
    {   68, 4875,     678,  6, }, // Africa/Kigali
    {   68, 5208,     684,  7, }, // Africa/Kinshasa
    {   68, 4646,     691,  5, }, // Africa/Lagos
    {   68, 5238,     696,  8, }, // Africa/Libreville
    {   68, 6832,     704,  4, }, // Africa/Lome
    {   68, 5113,     708,  6, }, // Africa/Luanda
    {   68, 4841,   32407,  9, }, // Africa/Lubumbashi
    {   68, 4903,     723,  6, }, // Africa/Lusaka
    {   68, 5224,     729,  6, }, // Africa/Malabo
    {   68, 4889,   32416,  6, }, // Africa/Maputo
    {   68, 5099,     741,  6, }, // Africa/Maseru
    {   68, 5084,     747,  7, }, // Africa/Mbabane
    {   68, 5015,     754,  7, }, // Africa/Mogadishu
    {   68, 6997,   32422,  9, }, // Africa/Monrovia
    {   68, 3755,   32431,  7, }, // Africa/Nairobi
    {   68, 5173,     777,  7, }, // Africa/Ndjamena
    {   68, 5256,   32438,  7, }, // Africa/Niamey
    {   68, 6766,     790,  7, }, // Africa/Nouakchott
    {   68, 6690,     797,  8, }, // Africa/Ouagadougou
    {   68, 5127,   32445, 10, }, // Africa/Porto-Novo
    {   68, 4337,   32455,  8, }, // Africa/Sao_Tome
    {   68, 2732,     823,  7, }, // Africa/Tripoli
    {   68, 6503,     830,  5, }, // Africa/Tunis
    {   68, 4054,     835,  7, }, // Africa/Windhoek
    {   68,  132,     842,  4, }, // America/Adak
    {   68, 3183,   32463,  7, }, // America/Anchorage
    {   68, 5524,   32470,  7, }, // America/Anguilla
    {   68, 5541,   32477,  7, }, // America/Antigua
    {   68, 4456,   32484, 10, }, // America/Araguaina
    {   68,  166,   32494, 12, }, // America/Argentina/Buenos_Aires
    {   68,   91,     888,  9, }, // America/Argentina/Catamarca
    {   68,  270,   32506,  7, }, // America/Argentina/Cordoba
    {   68,  441,   32513,  5, }, // America/Argentina/Jujuy
    {   68, 7425,   32518,  8, }, // America/Argentina/La_Rioja
    {   68,  565,   32526,  7, }, // America/Argentina/Mendoza
    {   68, 7452,   32533, 11, }, // America/Argentina/Rio_Gallegos
    {   68, 7483,     935,  5, }, // America/Argentina/Salta
    {   68, 7507,   32544,  8, }, // America/Argentina/San_Juan
    {   68, 5497,   32552,  7, }, // America/Argentina/San_Luis
    {   68, 7534,     956,  7, }, // America/Argentina/Tucuman
    {   68, 7560,   32559,  7, }, // America/Argentina/Ushuaia
    {   68, 5557,     969,  5, }, // America/Aruba
    {   68, 4154,   32566,  8, }, // America/Asuncion
    {   68,  237,   32574,  8, }, // America/Atikokan
    {   68, 3581,   32582,  5, }, // America/Bahia
    {   68, 7586,   32587, 14, }, // America/Bahia_Banderas
    {   68, 5571,    1013,  8, }, // America/Barbados
    {   68, 7609,   32601,  5, }, // America/Belem
    {   68, 5309,   32606,  6, }, // America/Belize
    {   68, 7623,   32612, 13, }, // America/Blanc-Sablon
    {   68, 7644,   32625,  9, }, // America/Boa_Vista
    {   68, 4263,    1052,  6, }, // America/Bogota
    {   68, 7662,   32634,  5, }, // America/Boise
    {   68, 7676,   32639, 11, }, // America/Cambridge_Bay
    {   68, 7698,   32650, 12, }, // America/Campo_Grande
    {   68, 3770,    1087,  6, }, // America/Cancun
    {   68, 4596,    1093,  7, }, // America/Caracas
    {   68, 4247,   32662,  5, }, // America/Cayenne
    {   68, 5398,   32667, 16, }, // America/Cayman
    {   68, 3251,    1129,  6, }, // America/Chicago
    {   68, 7719,   32683,  6, }, // America/Chihuahua
    {   68, 5324,   32689,  9, }, // America/Costa_Rica
    {   68, 7759,    1151,  7, }, // America/Creston
    {   68, 3693,    1158,  6, }, // America/Cuiaba
    {   68, 5640,   32698,  7, }, // America/Curacao
    {   68, 7775,   32705, 13, }, // America/Danmarkshavn
    {   68, 7796,   32718,  5, }, // America/Dawson
    {   68, 7811,   32723, 10, }, // America/Dawson_Creek
    {   68,  805,    1201,  6, }, // America/Denver
    {   68, 3351,    1207,  7, }, // America/Detroit
    {   68, 5656,    1214,  8, }, // America/Dominica
    {   68,  893,    1222,  8, }, // America/Edmonton
    {   68, 7832,    1230,  8, }, // America/Eirunepe
    {   68, 5343,    1238,  8, }, // America/El_Salvador
    {   68, 7849,    1246, 11, }, // America/Fort_Nelson
    {   68, 7869,    1257,  9, }, // America/Fortaleza
    {   68, 7887,   32733,  9, }, // America/Glace_Bay
    {   68, 6844,   32742,  7, }, // America/Goose_Bay
    {   68, 4514,   32749, 10, }, // America/Grand_Turk
    {   68, 5687,    1292,  7, }, // America/Grenada
    {   68, 5703,   32759,  9, }, // America/Guadeloupe
    {   68, 3662,    1308,  9, }, // America/Guatemala
    {   68, 4743,   32768,  8, }, // America/Guayaquil
    {   68, 6895,   32776,  5, }, // America/Guyana
    {   68, 1923,   32781,  8, }, // America/Halifax
    {   68, 2221,   32789,  6, }, // America/Havana
    {   68, 7905,   32795,  9, }, // America/Hermosillo
    {   68,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {   68,  481,   32804, 15, }, // America/Indiana/Knox
    {   68, 7924,   32819, 17, }, // America/Indiana/Marengo
    {   68, 7948,   32836, 20, }, // America/Indiana/Petersburg
    {   68, 7975,   32856, 18, }, // America/Indiana/Tell_City
    {   68, 8001,   32874, 15, }, // America/Indiana/Vevay
    {   68, 8023,   32889, 18, }, // America/Indiana/Vincennes
    {   68, 8049,   32907, 17, }, // America/Indiana/Winamac
    {   68, 8073,    1452,  6, }, // America/Inuvik
    {   68,  660,   32924,  8, }, // America/Iqaluit
    {   68, 2665,   32932,  7, }, // America/Jamaica
    {   68, 5282,   32939,  4, }, // America/Juneau
    {   68,  521,    1476,  7, }, // America/Kentucky/Louisville
    {   68, 8088,   32943, 18, }, // America/Kentucky/Monticello
    {   68, 5621,   32961, 10, }, // America/Kralendijk
    {   68, 4278,   32971,  6, }, // America/La_Paz
    {   68, 7132,    1520,  4, }, // America/Lima
    {   68, 3390,   32977, 11, }, // America/Los_Angeles
    {   68, 5849,   32988, 19, }, // America/Lower_Princes
    {   68, 8116,   33007,  6, }, // America/Maceio
    {   68, 8131,   33013,  7, }, // America/Managua
    {   68, 1892,    1566,  6, }, // America/Manaus
    {   68, 5814,    1572,  6, }, // America/Marigot
    {   68, 5722,    1578,  8, }, // America/Martinique
    {   68, 8147,    1587,  9, }, // America/Matamoros
    {   68, 2779,   33020,  8, }, // America/Mazatlan
    {   68, 8165,    1604,  8, }, // America/Menominee
    {   68, 8183,    1612,  6, }, // America/Merida
    {   68, 8198,    1618, 10, }, // America/Metlakatla
    {   68, 2811,   33028, 12, }, // America/Mexico_City
    {   68, 4293,    1634,  7, }, // America/Miquelon
    {   68, 8217,    1641,  7, }, // America/Moncton
    {   68, 8233,   33040,  8, }, // America/Monterrey
    {   68, 4000,    1657, 10, }, // America/Montevideo
    {   68, 5741,   33048,  9, }, // America/Montserrat
    {   68, 5383,   33057,  5, }, // America/Nassau
    {   68, 3294,   33062,  6, }, // America/New_York
    {   68, 8251,    1691,  3, }, // America/Nome
    {   68, 1834,   33068,  6, }, // America/Noronha
    {   68, 8264,   33074, 22, }, // America/North_Dakota/Beulah
    {   68, 8292,   33096, 22, }, // America/North_Dakota/Center
    {   68, 8320,   33118, 26, }, // America/North_Dakota/New_Salem
    {   68,  393,   26733,  3, }, // America/Nuuk
    {   68, 8351,    1776,  7, }, // America/Ojinaga
    {   68, 5413,    1783,  6, }, // America/Panama
    {   68, 6106,    1789, 10, }, // America/Paramaribo
    {   68, 3224,   33144,  6, }, // America/Phoenix
    {   68, 3847,    1805, 12, }, // America/Port-au-Prince
    {   68, 5871,   33150, 13, }, // America/Port_of_Spain
    {   68, 8367,   33163, 10, }, // America/Porto_Velho
    {   68, 5760,   33173,  9, }, // America/Puerto_Rico
    {   68, 3932,   33182, 12, }, // America/Punta_Arenas
    {   68, 8387,   33194, 12, }, // America/Rankin_Inlet
    {   68, 8408,   33206,  6, }, // America/Recife
    {   68, 1979,   33212,  6, }, // America/Regina
    {   68, 8423,   33218,  7, }, // America/Resolute
    {   68,  695,   33225, 10, }, // America/Rio_Branco
    {   68, 8440,   33235,  8, }, // America/Santarem
    {   68, 2165,   33243,  9, }, // America/Santiago
    {   68, 6044,   33252, 13, }, // America/Santo_Domingo
    {   68, 1862,   33265,  9, }, // America/Sao_Paulo
    {   68, 6862,   33274, 12, }, // America/Scoresbysund
    {   68, 8457,    1956,  5, }, // America/Sitka
    {   68, 8471,   33286, 14, }, // America/St_Barthelemy
    {   68, 2045,   33300,  7, }, // America/St_Johns
    {   68, 5780,   33307,  9, }, // America/St_Kitts
    {   68, 5797,   33316, 10, }, // America/St_Lucia
    {   68,  855,   33326,  8, }, // America/St_Thomas
    {   68, 5830,   33334, 12, }, // America/St_Vincent
    {   68, 8493,   33346, 12, }, // America/Swift_Current
    {   68, 5363,    2038, 11, }, // America/Tegucigalpa
    {   68, 5673,    2049,  4, }, // America/Thule
    {   68,  313,    2053,  7, }, // America/Tijuana
    {   68,  608,    2060,  7, }, // America/Toronto
    {   68, 5605,    2067,  7, }, // America/Tortola
    {   68, 2077,    2074,  8, }, // America/Vancouver
    {   68, 2128,   33358,  8, }, // America/Whitehorse
    {   68,  734,   33366,  7, }, // America/Winnipeg
    {   68, 8515,   33373,  7, }, // America/Yakutat
    {   68, 5918,   33380,  5, }, // Antarctica/Casey
    {   68, 6027,   33385,  6, }, // Antarctica/Davis
    {   68, 6066,   33391, 13, }, // Antarctica/DumontDUrville
    {   68, 8531,   33404,  7, }, // Antarctica/Macquarie
    {   68, 7029,   33411,  5, }, // Antarctica/Mawson
    {   68, 7076,   33416,  8, }, // Antarctica/McMurdo
    {   68, 8552,    2154,  6, }, // Antarctica/Palmer
    {   68, 7189,    2160,  6, }, // Antarctica/Rothera
    {   68, 7258,   33424,  4, }, // Antarctica/Syowa
    {   68, 8570,    2171,  4, }, // Antarctica/Troll
    {   68, 7337,   33428,  6, }, // Antarctica/Vostok
    {   68, 1413,   33434, 11, }, // Arctic/Longyearbyen
    {   68, 5487,    2192,  4, }, // Asia/Aden
    {   68, 5297,   33445,  6, }, // Asia/Almaty
    {   68, 3870,   27113,  4, }, // Asia/Amman
    {   68, 5428,   33451,  6, }, // Asia/Anadyr
    {   68, 5440,    2213,  5, }, // Asia/Aqtau
    {   68, 5270,   33457,  7, }, // Asia/Aqtobe
    {   68,  964,    2224,  7, }, // Asia/Ashgabat
    {   68, 8587,   33464,  6, }, // Asia/Atyrau
    {   68, 3509,    2237,  6, }, // Asia/Baghdad
    {   68, 5451,    2243,  7, }, // Asia/Bahrain
    {   68, 3555,    2250,  4, }, // Asia/Baku
    {   68, 4368,   33470,  6, }, // Asia/Bangkok
    {   68, 3473,    2261,  7, }, // Asia/Barnaul
    {   68, 3988,   33476,  6, }, // Asia/Beirut
    {   68, 3680,    2274,  6, }, // Asia/Bishkek
    {   68, 5906,   33482,  7, }, // Asia/Brunei
    {   68, 4503,    2286,  4, }, // Asia/Chita
    {   68, 5963,   33489,  9, }, // Asia/Choibalsan
    {   68, 4413,    2299,  7, }, // Asia/Colombo
    {   68, 4442,    2306,  6, }, // Asia/Damascus
    {   68, 1060,     593,  4, }, // Asia/Dhaka
    {   68, 6125,    2317,  4, }, // Asia/Dili
    {   68, 3498,    2321,  5, }, // Asia/Dubai
    {   68, 6092,    2326,  7, }, // Asia/Dushanbe
    {   68, 8599,    2333,  9, }, // Asia/Famagusta
    {   68, 8614,    2342,  4, }, // Asia/Gaza
    {   68, 4697,   33498,  6, }, // Asia/Hebron
    {   68, 1226,   33504,  9, }, // Asia/Ho_Chi_Minh
    {   68, 2576,   33513,  9, }, // Asia/Hong_Kong
    {   68, 4673,    2366,  4, }, // Asia/Hovd
    {   68, 4086,   33522,  6, }, // Asia/Irkutsk
    {   68, 6969,    2377,  7, }, // Asia/Jakarta
    {   68, 6955,   33528,  8, }, // Asia/Jayapura
    {   68, 1257,   33536,  9, }, // Asia/Jerusalem
    {   68, 3462,    2401,  5, }, // Asia/Kabul
    {   68, 4218,    2420,  8, }, // Asia/Kamchatka
    {   68, 4141,    2431,  6, }, // Asia/Karachi
    {   68, 1152,    2437,  8, }, // Asia/Kathmandu
    {   68, 8624,   33545,  7, }, // Asia/Khandyga
    {   68,  992,   33552,  7, }, // Asia/Kolkata
    {   68, 4099,   33559, 11, }, // Asia/Krasnoyarsk
    {   68, 4761,   33570, 12, }, // Asia/Kuala_Lumpur
    {   68, 5893,    2483,  6, }, // Asia/Kuching
    {   68, 5464,   33582,  6, }, // Asia/Kuwait
    {   68, 1178,   33588,  5, }, // Asia/Macau
    {   68, 3919,    2500,  7, }, // Asia/Magadan
    {   68, 1316,    2507,  7, }, // Asia/Makassar
    {   68, 7145,    2514,  6, }, // Asia/Manila
    {   68, 6883,   33593,  6, }, // Asia/Muscat
    {   68, 2446,   33599,  8, }, // Asia/Nicosia
    {   68, 8638,   33607, 11, }, // Asia/Novokuznetsk
    {   68, 4037,    2544, 11, }, // Asia/Novosibirsk
    {   68, 4131,    2555,  4, }, // Asia/Omsk
    {   68, 7108,   33618,  4, }, // Asia/Oral
    {   68, 6924,   33622,  8, }, // Asia/Phnom_Penh
    {   68, 8656,    2572,  9, }, // Asia/Pontianak
    {   68, 4116,   33630,  9, }, // Asia/Pyongyang
    {   68, 5476,    2588,  5, }, // Asia/Qatar
    {   68, 8671,   33639,  8, }, // Asia/Qostanay
    {   68, 4171,   33647,  9, }, // Asia/Qyzylorda
    {   68, 3486,   33656,  5, }, // Asia/Riyadh
    {   68, 4310,    2623,  7, }, // Asia/Sakhalin
    {   68, 7208,    2630,  9, }, // Asia/Samarkand
    {   68, 3116,    2639,  4, }, // Asia/Seoul
    {   68, 1020,   33661,  6, }, // Asia/Shanghai
    {   68, 3137,    2649,  8, }, // Asia/Singapore
    {   68, 4199,   33667, 13, }, // Asia/Srednekolymsk
    {   68, 3100,   33680,  6, }, // Asia/Taipei
    {   68, 4683,    2676,  7, }, // Asia/Tashkent
    {   68, 3817,   33686,  7, }, // Asia/Tbilisi
    {   68, 2638,   33693,  7, }, // Asia/Tehran
    {   68, 1284,   33700,  5, }, // Asia/Thimphu
    {   68, 2687,    2701,  5, }, // Asia/Tokyo
    {   68, 4474,    2706,  5, }, // Asia/Tomsk
    {   68, 1346,   33705, 10, }, // Asia/Ulaanbaatar
    {   68, 1126,   33715,  7, }, // Asia/Urumqi
    {   68, 8685,    2727,  8, }, // Asia/Ust-Nera
    {   68, 6940,   33722, 10, }, // Asia/Vientiane
    {   68, 4612,    2742, 11, }, // Asia/Vladivostok
    {   68, 4730,   33732,  7, }, // Asia/Yakutsk
    {   68, 1202,   27502,  6, }, // Asia/Yangon
    {   68, 3785,   33739, 13, }, // Asia/Yekaterinburg
    {   68, 3649,   33752,  7, }, // Asia/Yerevan
    {   68, 3565,   33759,  5, }, // Atlantic/Azores
    {   68, 5588,   33764,  7, }, // Atlantic/Bermuda
    {   68, 6591,   33771, 15, }, // Atlantic/Canary
    {   68, 3629,   33786, 18, }, // Atlantic/Cape_Verde
    {   68, 1379,   33804, 13, }, // Atlantic/Faroe
    {   68, 8699,    2862,  7, }, // Atlantic/Madeira
    {   68, 2614,   33817,  9, }, // Atlantic/Reykjavik
    {   68, 7235,   33826, 13, }, // Atlantic/South_Georgia
    {   68, 6784,   33839, 10, }, // Atlantic/St_Helena
    {   68, 6607,    2920,  6, }, // Atlantic/Stanley
    {   68, 1654,    2926,  8, }, // Australia/Adelaide
    {   68, 1619,   33849,  8, }, // Australia/Brisbane
    {   68, 1783,   33857, 10, }, // Australia/Broken_Hill
    {   68, 1567,    2952,  6, }, // Australia/Darwin
    {   68, 3539,    2958,  5, }, // Australia/Eucla
    {   68, 1500,    2963,  6, }, // Australia/Hobart
    {   68, 8716,    2969,  8, }, // Australia/Lindeman
    {   68, 1531,   33867,  8, }, // Australia/Lord_Howe
    {   68, 1711,    2985,  7, }, // Australia/Melbourne
    {   68, 1746,    2992,  4, }, // Australia/Perth
    {   68, 1447,   33875,  6, }, // Australia/Sydney
    {   68, 6390,    3002,  9, }, // Europe/Amsterdam
    {   68, 6149,   27734,  6, }, // Europe/Andorra
    {   68, 3522,    3018,  8, }, // Europe/Astrakhan
    {   68, 6577,   33881,  5, }, // Europe/Athens
    {   68, 6285,   33886,  7, }, // Europe/Belgrade
    {   68, 4659,    3037,  6, }, // Europe/Berlin
    {   68, 6437,    3043, 10, }, // Europe/Bratislava
    {   68, 6178,   33893,  6, }, // Europe/Brussels
    {   68, 3830,   33899,  8, }, // Europe/Bucharest
    {   68, 3708,   33907, 10, }, // Europe/Budapest
    {   68, 8751,   33917,  8, }, // Europe/Busingen
    {   68, 2475,   33925,  7, }, // Europe/Chisinau
    {   68, 6238,   33932, 10, }, // Europe/Copenhagen
    {   68, 2260,   33942,  6, }, // Europe/Dublin
    {   68, 6256,    3123,  9, }, // Europe/Gibraltar
    {   68, 8767,   33948,  6, }, // Europe/Guernsey
    {   68, 6561,    3138,  8, }, // Europe/Helsinki
    {   68, 8783,   33954, 10, }, // Europe/Isle_of_Man
    {   68, 1097,   33964,  8, }, // Europe/Istanbul
    {   68, 8802,    3162,  5, }, // Europe/Jersey
    {   68, 3881,   33972, 11, }, // Europe/Kaliningrad
    {   68, 8924,    3178,  5, }, // Europe/Kirov
    {   68, 2419,   33983,  5, }, // Europe/Kyiv
    {   68, 3078,   33988,  7, }, // Europe/Lisbon
    {   68, 6455,   33995,  7, }, // Europe/Ljubljana
    {   68, 2393,    3204,  6, }, // Europe/London
    {   68, 6314,   34002, 10, }, // Europe/Luxembourg
    {   68, 4794,    3221,  6, }, // Europe/Madrid
    {   68, 6346,    3227,  5, }, // Europe/Malta
    {   68, 6531,   34012,  8, }, // Europe/Mariehamn
    {   68, 3595,    3241,  5, }, // Europe/Minsk
    {   68, 6359,    3246,  6, }, // Europe/Monaco
    {   68, 3443,    3252,  6, }, // Europe/Moscow
    {   68, 6407,    3258,  4, }, // Europe/Oslo
    {   68, 4186,   34020,  5, }, // Europe/Paris
    {   68, 6373,    3267,  9, }, // Europe/Podgorica
    {   68, 6224,    3276,  4, }, // Europe/Prague
    {   68, 8816,    3281,  4, }, // Europe/Riga
    {   68, 6273,    3285,  3, }, // Europe/Rome
    {   68, 4233,    3288,  6, }, // Europe/Samara
    {   68, 6419,   34025, 10, }, // Europe/San_Marino
    {   68, 6194,   34035,  8, }, // Europe/Sarajevo
    {   68, 4353,    3311,  7, }, // Europe/Saratov
    {   68, 8828,   34043, 10, }, // Europe/Simferopol
    {   68, 6332,   34053,  6, }, // Europe/Skopje
    {   68, 6548,   34059,  6, }, // Europe/Sofia
    {   68, 6472,   34065,  8, }, // Europe/Stockholm
    {   68, 8846,   34073,  5, }, // Europe/Tallinn
    {   68, 6135,    3352,  6, }, // Europe/Tirane
    {   68, 8861,   34078,  8, }, // Europe/Ulyanovsk
    {   68, 6301,   34086,  5, }, // Europe/Vaduz
    {   68, 6516,    3372,  7, }, // Europe/Vatican
    {   68, 6164,   34091,  3, }, // Europe/Vienna
    {   68, 8878,   34094,  6, }, // Europe/Vilnius
    {   68, 4629,    3390,  9, }, // Europe/Volgograd
    {   68, 3055,    3399,  7, }, // Europe/Warsaw
    {   68, 6210,    3406,  6, }, // Europe/Zagreb
    {   68, 6489,   34100,  5, }, // Europe/Zurich
    {   68, 4980,   34105, 12, }, // Indian/Antananarivo
    {   68, 6910,    3430,  5, }, // Indian/Chagos
    {   68, 5979,   34117,  5, }, // Indian/Christmas
    {   68, 5996,    3447,  5, }, // Indian/Cocos
    {   68, 4931,   34122,  6, }, // Indian/Comoro
    {   68, 6624,    3469,  8, }, // Indian/Kerguelen
    {   68, 7223,   34128,  4, }, // Indian/Mahe
    {   68, 7013,   34132,  7, }, // Indian/Maldives
    {   68, 3971,   34139, 10, }, // Indian/Mauritius
    {   68, 5000,   34149,  5, }, // Indian/Mayotte
    {   68, 7174,   34154,  7, }, // Indian/Reunion
    {   68, 4324,   34161,  5, }, // Pacific/Apia
    {   68,  932,   34166,  6, }, // Pacific/Auckland
    {   68, 3608,    3517,  8, }, // Pacific/Bougainville
    {   68, 2863,   34172,  6, }, // Pacific/Chatham
    {   68, 3022,    3530,  4, }, // Pacific/Chuuk
    {   68, 2201,   34178, 14, }, // Pacific/Easter
    {   68, 7323,    3545,  4, }, // Pacific/Efate
    {   68, 7290,    3550,  7, }, // Pacific/Fakaofo
    {   68, 3804,    3557,  4, }, // Pacific/Fiji
    {   68, 7306,    3561,  8, }, // Pacific/Funafuti
    {   68, 6641,    3569,  9, }, // Pacific/Galapagos
    {   68, 6659,   34192,  7, }, // Pacific/Gambier
    {   68, 3724,   34199, 11, }, // Pacific/Guadalcanal
    {   68, 5935,    3615,  4, }, // Pacific/Guam
    {   68, 2929,   34210,  8, }, // Pacific/Honolulu
    {   68, 2897,   34218,  9, }, // Pacific/Kanton
    {   68, 3900,    3625, 10, }, // Pacific/Kiritimati
    {   68, 6982,   34227,  5, }, // Pacific/Kosrae
    {   68, 2708,   34232, 10, }, // Pacific/Kwajalein
    {   68, 4779,   34242,  6, }, // Pacific/Majuro
    {   68, 3953,    3656,  6, }, // Pacific/Marquesas
    {   68, 8909,   34248,  6, }, // Pacific/Midway
    {   68, 7047,    3689,  5, }, // Pacific/Nauru
    {   68, 7095,    3694,  4, }, // Pacific/Niue
    {   68, 4070,    3698,  7, }, // Pacific/Norfolk
    {   68, 7061,    3705,  5, }, // Pacific/Noumea
    {   68, 2991,   34254,  9, }, // Pacific/Pago_Pago
    {   68, 7118,    3719,  5, }, // Pacific/Palau
    {   68, 7157,    3724,  7, }, // Pacific/Pitcairn
    {   68, 2961,   34263,  6, }, // Pacific/Pohnpei
    {   68, 4709,   34269, 11, }, // Pacific/Port_Moresby
    {   68, 6009,    3748,  9, }, // Pacific/Rarotonga
    {   68, 5948,   34280,  6, }, // Pacific/Saipan
    {   68, 7275,   34286,  6, }, // Pacific/Tahiti
    {   68, 6675,    3768,  6, }, // Pacific/Tarawa
    {   68, 4485,    3774,  9, }, // Pacific/Tongatapu
    {   68, 7355,   34292,  4, }, // Pacific/Wake
    {   68, 7368,   34296,  5, }, // Pacific/Wallis
    {   69, 4961,   34301, 10, }, // Africa/Addis_Ababa Breton/Latin/France
    {   69, 7383,   24903,  5, }, // Africa/Algiers
    {   69,   14,       0,  6, }, // Africa/Asmara
    {   69, 2242,   34311,  5, }, // Africa/Cairo
    {   69, 4019,   34316, 25, }, // Africa/Casablanca
    {   69, 5068,   34341,  7, }, // Africa/El_Aaiun
    {   69, 6832,   18802,  4, }, // Africa/Lome
    {   69, 5015,   24944,  8, }, // Africa/Mogadishu
    {   69, 5173,   34348,  9, }, // Africa/Ndjamena
    {   69, 4337,       6,  8, }, // Africa/Sao_Tome
    {   69, 2732,   34357, 19, }, // Africa/Tripoli
    {   69, 6503,    4737,  5, }, // Africa/Tunis
    {   69, 4154,      14,  8, }, // America/Asuncion
    {   69,  237,      22,  8, }, // America/Atikokan
    {   69, 7586,      30, 17, }, // America/Bahia_Banderas
    {   69, 7609,   34376,  5, }, // America/Belem
    {   69, 4263,   18898,  6, }, // America/Bogota
    {   69, 3770,      47,  6, }, // America/Cancun
    {   69, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   69, 5640,      66,  7, }, // America/Curacao
    {   69, 5343,    5090,  8, }, // America/El_Salvador
    {   69, 5703,   34381,  9, }, // America/Guadeloupe
    {   69, 2221,   18926,  9, }, // America/Havana
    {   69,  481,      73, 13, }, // America/Indiana/Knox
    {   69, 7924,      86, 16, }, // America/Indiana/Marengo
    {   69, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   69, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   69, 8001,     139, 14, }, // America/Indiana/Vevay
    {   69, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   69, 8049,     171, 16, }, // America/Indiana/Winamac
    {   69, 2665,    3892,  7, }, // America/Jamaica
    {   69, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   69, 5849,     207, 22, }, // America/Lower_Princes
    {   69, 5722,    5400,  8, }, // America/Martinique
    {   69, 8183,     229,  6, }, // America/Merida
    {   69, 2811,   34390, 12, }, // America/Mexico_City
    {   69, 4293,    5434,  7, }, // America/Miquelon
    {   69, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   69, 8292,     255, 20, }, // America/North_Dakota/Center
    {   69, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   69,  393,   34402, 14, }, // America/Nuuk
    {   69, 5413,   19056,  6, }, // America/Panama
    {   69, 1862,   25145,  9, }, // America/Sao_Paulo
    {   69, 6862,     302, 16, }, // America/Scoresbysund
    {   69, 8471,   34416, 16, }, // America/St_Barthelemy
    {   69, 2045,   34432, 12, }, // America/St_Johns
    {   69, 5780,   25182, 11, }, // America/St_Kitts
    {   69, 5797,   34444, 11, }, // America/St_Lucia
    {   69,  855,     360, 10, }, // America/St_Thomas
    {   69, 5830,   34455, 11, }, // America/St_Vincent
    {   69, 5673,   34466,  5, }, // America/Thule
    {   69, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   69, 5428,   34471,  7, }, // Asia/Anadyr
    {   69, 5451,    4069,  7, }, // Asia/Bahrain
    {   69, 3555,   34478,  5, }, // Asia/Baku
    {   69, 3988,   34483,  6, }, // Asia/Beirut
    {   69, 4503,   34489,  6, }, // Asia/Chita
    {   69, 4413,   34495,  7, }, // Asia/Colombo
    {   69, 4442,    4102,  6, }, // Asia/Damascus
    {   69, 1226,   34502, 15, }, // Asia/Ho_Chi_Minh
    {   69, 1257,   31972,  9, }, // Asia/Jerusalem
    {   69, 3462,   34517,  6, }, // Asia/Kabul
    {   69, 4218,   34523, 10, }, // Asia/Kamchatka
    {   69, 1152,     413,  9, }, // Asia/Kathmandu
    {   69,  992,   34533,  8, }, // Asia/Kolkata
    {   69, 5464,   34541,  6, }, // Asia/Kuwait
    {   69, 1178,    4222,  5, }, // Asia/Macau
    {   69, 6883,   34547,  6, }, // Asia/Muscat
    {   69, 2446,   34553,  8, }, // Asia/Nicosia
    {   69, 6924,   34561, 10, }, // Asia/Phnom_Penh
    {   69, 4116,   34571, 10, }, // Asia/Pyongyang
    {   69, 3137,   34581,  9, }, // Asia/Singapore
    {   69, 4683,   34590,  8, }, // Asia/Tashkent
    {   69, 1126,   25383,  6, }, // Asia/Urumqi
    {   69, 6940,   34598,  9, }, // Asia/Vientiane
    {   69, 1202,     442,  6, }, // Asia/Yangon
    {   69, 3785,   34607, 14, }, // Asia/Yekaterinburg
    {   69, 3565,   34621,  6, }, // Atlantic/Azores
    {   69, 5588,   34627,  8, }, // Atlantic/Bermuda
    {   69, 6591,   34635,  8, }, // Atlantic/Canary
    {   69, 3629,   34643,  8, }, // Atlantic/Cape_Verde
    {   69, 1379,   34651,  5, }, // Atlantic/Faroe
    {   69, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {   69, 7235,   34665, 13, }, // Atlantic/South_Georgia
    {   69, 6784,   34678, 12, }, // Atlantic/St_Helena
    {   69, 6577,   19437,  4, }, // Europe/Athens
    {   69, 6285,    6398,  7, }, // Europe/Belgrade
    {   69, 6178,   25472,  6, }, // Europe/Brussels
    {   69, 3830,   25479,  8, }, // Europe/Bucharest
    {   69, 6238,    4376, 10, }, // Europe/Copenhagen
    {   69, 2260,   34690,  6, }, // Europe/Dublin
    {   69, 6256,   34696,  9, }, // Europe/Gibraltar
    {   69, 8767,   34705,  9, }, // Europe/Guernsey
    {   69, 8783,   34714,  5, }, // Europe/Isle_of_Man
    {   69, 8802,   34719,  8, }, // Europe/Jersey
    {   69, 2419,    6486,  4, }, // Europe/Kyiv
    {   69, 3078,   19528,  6, }, // Europe/Lisbon
    {   69, 2393,   34727,  7, }, // Europe/London
    {   69, 6314,   34734, 11, }, // Europe/Luxembourg
    {   69, 6531,   34745,  9, }, // Europe/Mariehamn
    {   69, 3595,   34754,  5, }, // Europe/Minsk
    {   69, 3443,   34759,  6, }, // Europe/Moscow
    {   69, 4186,   32206,  5, }, // Europe/Paris
    {   69, 6224,   34765,  5, }, // Europe/Prague
    {   69, 6273,   19581,  4, }, // Europe/Rome
    {   69, 6135,    6599,  6, }, // Europe/Tirane
    {   69, 6516,    6614,  7, }, // Europe/Vatican
    {   69, 3055,   19639,  8, }, // Europe/Warsaw
    {   69, 5996,   34770,  5, }, // Indian/Cocos
    {   69, 4931,   34775,  7, }, // Indian/Comoro
    {   69, 6624,   34782,  9, }, // Indian/Kerguelen
    {   69, 7013,   34791,  8, }, // Indian/Maldives
    {   69, 3971,   34799,  5, }, // Indian/Mauritius
    {   69, 7174,   34804,  7, }, // Indian/Reunion
    {   69, 3022,     474,  5, }, // Pacific/Chuuk
    {   69, 2201,   34811,  9, }, // Pacific/Easter
    {   69, 3804,    4507,  5, }, // Pacific/Fiji
    {   69, 6641,   19672,  9, }, // Pacific/Galapagos
    {   69, 2929,    4512,  8, }, // Pacific/Honolulu
    {   69, 3953,   34820,  6, }, // Pacific/Marquesas
    {   69, 2961,     479,  7, }, // Pacific/Pohnpei
    {   70, 6751,   34826,  7, }, // Africa/Abidjan Bulgarian/Cyrillic/Bulgaria
    {   70, 6723,   25628,  4, }, // Africa/Accra
    {   70, 4961,   32317, 10, }, // Africa/Addis_Ababa
    {   70, 7383,     508,  5, }, // Africa/Algiers
    {   70,   14,   25647,  6, }, // Africa/Asmara
    {   70,   44,     519,  6, }, // Africa/Bamako
    {   70, 5159,   32327,  6, }, // Africa/Bangui
    {   70, 6709,   25658,  7, }, // Africa/Banjul
    {   70, 7398,     536,  5, }, // Africa/Bissau
    {   70, 4859,   34833,  9, }, // Africa/Blantyre
    {   70, 5189,   32345,  8, }, // Africa/Brazzaville
    {   70, 4824,     558,  9, }, // Africa/Bujumbura
    {   70, 2242,   34842,  5, }, // Africa/Cairo
    {   70, 4019,   32367, 10, }, // Africa/Casablanca
    {   70, 7412,     581,  5, }, // Africa/Ceuta
    {   70, 6736,     586,  7, }, // Africa/Conakry
    {   70, 6803,     593,  5, }, // Africa/Dakar
    {   70, 5032,   34847, 12, }, // Africa/Dar_es_Salaam
    {   70, 4945,   34859,  7, }, // Africa/Djibouti
    {   70, 5145,     616,  5, }, // Africa/Douala
    {   70, 5068,   34866,  6, }, // Africa/El_Aaiun
    {   70, 6816,   34872,  8, }, // Africa/Freetown
    {   70, 4808,     635,  8, }, // Africa/Gaborone
    {   70, 4917,   32384,  6, }, // Africa/Harare
    {   70, 4381,   34880, 11, }, // Africa/Johannesburg
    {   70, 4401,   25755,  5, }, // Africa/Juba
    {   70, 5053,     665,  7, }, // Africa/Kampala
    {   70, 4426,     672,  6, }, // Africa/Khartoum
    {   70, 4875,     678,  6, }, // Africa/Kigali
    {   70, 5208,     684,  7, }, // Africa/Kinshasa
    {   70, 4646,     691,  5, }, // Africa/Lagos
    {   70, 5238,     696,  8, }, // Africa/Libreville
    {   70, 6832,     704,  4, }, // Africa/Lome
    {   70, 5113,     708,  6, }, // Africa/Luanda
    {   70, 4841,     714,  9, }, // Africa/Lubumbashi
    {   70, 4903,     723,  6, }, // Africa/Lusaka
    {   70, 5224,     729,  6, }, // Africa/Malabo
    {   70, 4889,   32416,  6, }, // Africa/Maputo
    {   70, 5099,     741,  6, }, // Africa/Maseru
    {   70, 5084,     747,  7, }, // Africa/Mbabane
    {   70, 5015,   34891,  8, }, // Africa/Mogadishu
    {   70, 6997,   34899,  8, }, // Africa/Monrovia
    {   70, 3755,   34907,  7, }, // Africa/Nairobi
    {   70, 5173,   25836,  8, }, // Africa/Ndjamena
    {   70, 5256,   34914,  6, }, // Africa/Niamey
    {   70, 6766,     790,  7, }, // Africa/Nouakchott
    {   70, 6690,     797,  8, }, // Africa/Ouagadougou
    {   70, 5127,   32445, 10, }, // Africa/Porto-Novo
    {   70, 4337,   32455,  8, }, // Africa/Sao_Tome
    {   70, 2732,     823,  7, }, // Africa/Tripoli
    {   70, 6503,     830,  5, }, // Africa/Tunis
    {   70, 4054,     835,  7, }, // Africa/Windhoek
    {   70,  132,     842,  4, }, // America/Adak
    {   70, 3183,   34920,  8, }, // America/Anchorage
    {   70, 5524,   34928,  7, }, // America/Anguilla
    {   70, 5541,     860,  7, }, // America/Antigua
    {   70, 4456,   34935,  9, }, // America/Araguaina
    {   70,  166,   34944, 12, }, // America/Argentina/Buenos_Aires
    {   70,   91,     888,  9, }, // America/Argentina/Catamarca
    {   70,  270,   32506,  7, }, // America/Argentina/Cordoba
    {   70,  441,   34956,  5, }, // America/Argentina/Jujuy
    {   70, 7425,   32518,  8, }, // America/Argentina/La_Rioja
    {   70,  565,     917,  7, }, // America/Argentina/Mendoza
    {   70, 7452,   34961, 11, }, // America/Argentina/Rio_Gallegos
    {   70, 7483,     935,  5, }, // America/Argentina/Salta
    {   70, 7507,   32544,  8, }, // America/Argentina/San_Juan
    {   70, 5497,   34972,  8, }, // America/Argentina/San_Luis
    {   70, 7534,     956,  7, }, // America/Argentina/Tucuman
    {   70, 7560,   25976,  5, }, // America/Argentina/Ushuaia
    {   70, 5557,     969,  5, }, // America/Aruba
    {   70, 4154,   32566,  8, }, // America/Asuncion
    {   70,  237,   32574,  8, }, // America/Atikokan
    {   70, 3581,   34980,  4, }, // America/Bahia
    {   70, 7586,   34984, 16, }, // America/Bahia_Banderas
    {   70, 5571,    1013,  8, }, // America/Barbados
    {   70, 7609,   32601,  5, }, // America/Belem
    {   70, 5309,    1026,  5, }, // America/Belize
    {   70, 7623,   35000, 11, }, // America/Blanc-Sablon
    {   70, 7644,   32625,  9, }, // America/Boa_Vista
    {   70, 4263,    1052,  6, }, // America/Bogota
    {   70, 7662,   35011,  5, }, // America/Boise
    {   70, 7676,   35016, 13, }, // America/Cambridge_Bay
    {   70, 7698,   32650, 12, }, // America/Campo_Grande
    {   70, 3770,    1087,  6, }, // America/Cancun
    {   70, 4596,    1093,  7, }, // America/Caracas
    {   70, 4247,   35029,  5, }, // America/Cayenne
    {   70, 5398,   35034, 17, }, // America/Cayman
    {   70, 3251,    1129,  6, }, // America/Chicago
    {   70, 7719,    1135,  6, }, // America/Chihuahua
    {   70, 7737,   35051, 13, }, // America/Ciudad_Juarez
    {   70, 5324,   35064, 10, }, // America/Costa_Rica
    {   70, 7759,   35074,  7, }, // America/Creston
    {   70, 3693,   35081,  5, }, // America/Cuiaba
    {   70, 5640,   35086,  7, }, // America/Curacao
    {   70, 7775,   35093, 12, }, // America/Danmarkshavn
    {   70, 7796,   35105,  6, }, // America/Dawson
    {   70, 7811,   35111, 12, }, // America/Dawson_Creek
    {   70,  805,   35123,  6, }, // America/Denver
    {   70, 3351,   35129,  7, }, // America/Detroit
    {   70, 5656,    1214,  8, }, // America/Dominica
    {   70,  893,   35136,  8, }, // America/Edmonton
    {   70, 7832,   35144,  8, }, // America/Eirunepe
    {   70, 5343,    1238,  8, }, // America/El_Salvador
    {   70, 7849,   35152, 11, }, // America/Fort_Nelson
    {   70, 7869,    1257,  9, }, // America/Fortaleza
    {   70, 7887,   35163,  9, }, // America/Glace_Bay
    {   70, 6844,   35172,  7, }, // America/Goose_Bay
    {   70, 4514,   35179, 10, }, // America/Grand_Turk
    {   70, 5687,    1292,  7, }, // America/Grenada
    {   70, 5703,    1299,  9, }, // America/Guadeloupe
    {   70, 3662,    1308,  9, }, // America/Guatemala
    {   70, 4743,   35189,  7, }, // America/Guayaquil
    {   70, 6895,   26304,  5, }, // America/Guyana
    {   70, 1923,   32781,  8, }, // America/Halifax
    {   70, 2221,   32789,  6, }, // America/Havana
    {   70, 7905,   35196,  9, }, // America/Hermosillo
    {   70,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {   70,  481,    1365,  4, }, // America/Indiana/Knox
    {   70, 7924,    1378,  7, }, // America/Indiana/Marengo
    {   70, 7948,   35205, 10, }, // America/Indiana/Petersburg
    {   70, 7975,   32856,  8, }, // America/Indiana/Tell_City
    {   70, 8001,   35215,  4, }, // America/Indiana/Vevay
    {   70, 8023,   35219,  7, }, // America/Indiana/Vincennes
    {   70, 8049,    1445,  7, }, // America/Indiana/Winamac
    {   70, 8073,    1452,  6, }, // America/Inuvik
    {   70,  660,   32924,  8, }, // America/Iqaluit
    {   70, 2665,   26461,  6, }, // America/Jamaica
    {   70, 5282,   35226,  5, }, // America/Juneau
    {   70,  521,    1476,  7, }, // America/Kentucky/Louisville
    {   70, 8088,   32943,  9, }, // America/Kentucky/Monticello
    {   70, 5621,   35231, 10, }, // America/Kralendijk
    {   70, 4278,   35241,  6, }, // America/La_Paz
    {   70, 7132,    1520,  4, }, // America/Lima
    {   70, 3390,   35247, 12, }, // America/Los_Angeles
    {   70, 5849,   35259, 20, }, // America/Lower_Princes
    {   70, 8116,   35279,  6, }, // America/Maceio
    {   70, 8131,    1559,  7, }, // America/Managua
    {   70, 1892,    1566,  6, }, // America/Manaus
    {   70, 5814,    1572,  6, }, // America/Marigot
    {   70, 5722,    1578,  9, }, // America/Martinique
    {   70, 8147,    1587,  9, }, // America/Matamoros
    {   70, 2779,    1596,  8, }, // America/Mazatlan
    {   70, 8165,    1604,  8, }, // America/Menominee
    {   70, 8183,    1612,  6, }, // America/Merida
    {   70, 8198,    1618, 10, }, // America/Metlakatla
    {   70, 2811,   33028, 12, }, // America/Mexico_City
    {   70, 4293,    1634,  7, }, // America/Miquelon
    {   70, 8217,    1641,  7, }, // America/Moncton
    {   70, 8233,   35285,  8, }, // America/Monterrey
    {   70, 4000,    1657, 10, }, // America/Montevideo
    {   70, 5741,   33048,  9, }, // America/Montserrat
    {   70, 5383,   33057,  5, }, // America/Nassau
    {   70, 3294,   35293,  7, }, // America/New_York
    {   70, 8251,   35300,  4, }, // America/Nome
    {   70, 1834,   35304,  6, }, // America/Noronha
    {   70, 8264,   35310,  4, }, // America/North_Dakota/Beulah
    {   70, 8292,   35314,  6, }, // America/North_Dakota/Center
    {   70, 8320,   35320,  9, }, // America/North_Dakota/New_Salem
    {   70,  393,    1772,  4, }, // America/Nuuk
    {   70, 8351,    1776,  7, }, // America/Ojinaga
    {   70, 5413,    1783,  6, }, // America/Panama
    {   70, 6106,    1789, 10, }, // America/Paramaribo
    {   70, 3224,    1799,  6, }, // America/Phoenix
    {   70, 3847,    1805, 12, }, // America/Port-au-Prince
    {   70, 5871,   35329, 13, }, // America/Port_of_Spain
    {   70, 8367,   35342, 11, }, // America/Porto_Velho
    {   70, 5760,   35353, 11, }, // America/Puerto_Rico
    {   70, 3932,   33182, 12, }, // America/Punta_Arenas
    {   70, 8387,   33194, 12, }, // America/Rankin_Inlet
    {   70, 8408,   33206,  6, }, // America/Recife
    {   70, 1979,   35364,  8, }, // America/Regina
    {   70, 8423,   35372,  7, }, // America/Resolute
    {   70,  695,   33225, 10, }, // America/Rio_Branco
    {   70, 8440,   33235,  8, }, // America/Santarem
    {   70, 2165,   35379,  7, }, // America/Santiago
    {   70, 6044,   33252, 13, }, // America/Santo_Domingo
    {   70, 1862,   35386,  9, }, // America/Sao_Paulo
    {   70, 6862,   35395, 10, }, // America/Scoresbysund
    {   70, 8457,    1956,  5, }, // America/Sitka
    {   70, 8471,   35405, 13, }, // America/St_Barthelemy
    {   70, 2045,   35418, 11, }, // America/St_Johns
    {   70, 5780,   35429, 10, }, // America/St_Kitts
    {   70, 5797,   35439, 11, }, // America/St_Lucia
    {   70,  855,   35450, 11, }, // America/St_Thomas
    {   70, 5830,   35461, 13, }, // America/St_Vincent
    {   70, 8493,   35474, 12, }, // America/Swift_Current
    {   70, 5363,    2038, 11, }, // America/Tegucigalpa
    {   70, 5673,    2049,  4, }, // America/Thule
    {   70,  313,    2053,  7, }, // America/Tijuana
    {   70,  608,    2060,  7, }, // America/Toronto
    {   70, 5605,    2067,  7, }, // America/Tortola
    {   70, 2077,   35486,  8, }, // America/Vancouver
    {   70, 2128,   27017,  8, }, // America/Whitehorse
    {   70,  734,   35494,  7, }, // America/Winnipeg
    {   70, 8515,   27032,  6, }, // America/Yakutat
    {   70, 5918,   35501,  5, }, // Antarctica/Casey
    {   70, 6027,   35506,  6, }, // Antarctica/Davis
    {   70, 6066,   35512, 12, }, // Antarctica/DumontDUrville
    {   70, 8531,    2131,  8, }, // Antarctica/Macquarie
    {   70, 7029,   35524,  6, }, // Antarctica/Mawson
    {   70, 7076,   35530,  8, }, // Antarctica/McMurdo
    {   70, 8552,   35538,  6, }, // Antarctica/Palmer
    {   70, 7189,    2160,  6, }, // Antarctica/Rothera
    {   70, 7258,   33424,  4, }, // Antarctica/Syowa
    {   70, 8570,    2171,  4, }, // Antarctica/Troll
    {   70, 7337,   33428,  6, }, // Antarctica/Vostok
    {   70, 1413,   35544, 10, }, // Arctic/Longyearbyen
    {   70, 5487,    2192,  4, }, // Asia/Aden
    {   70, 5297,   33445,  6, }, // Asia/Almaty
    {   70, 3870,   27113,  4, }, // Asia/Amman
    {   70, 5428,   33451,  6, }, // Asia/Anadyr
    {   70, 5440,    2213,  5, }, // Asia/Aqtau
    {   70, 5270,    2218,  6, }, // Asia/Aqtobe
    {   70,  964,    2224,  7, }, // Asia/Ashgabat
    {   70, 8587,   35554,  6, }, // Asia/Atyrau
    {   70, 3509,    2237,  6, }, // Asia/Baghdad
    {   70, 5451,   35560,  7, }, // Asia/Bahrain
    {   70, 3555,    2250,  4, }, // Asia/Baku
    {   70, 4368,   33470,  6, }, // Asia/Bangkok
    {   70, 3473,    2261,  7, }, // Asia/Barnaul
    {   70, 3988,   27145,  6, }, // Asia/Beirut
    {   70, 3680,    2274,  6, }, // Asia/Bishkek
    {   70, 5906,   27157,  6, }, // Asia/Brunei
    {   70, 4503,    2286,  4, }, // Asia/Chita
    {   70, 5963,   35567,  9, }, // Asia/Choibalsan
    {   70, 4413,    2299,  7, }, // Asia/Colombo
    {   70, 4442,    2306,  6, }, // Asia/Damascus
    {   70, 1060,     593,  4, }, // Asia/Dhaka
    {   70, 6125,    2317,  4, }, // Asia/Dili
    {   70, 3498,   27187,  5, }, // Asia/Dubai
    {   70, 6092,    2326,  7, }, // Asia/Dushanbe
    {   70, 8599,    2333,  9, }, // Asia/Famagusta
    {   70, 8614,    2342,  4, }, // Asia/Gaza
    {   70, 4697,   33498,  6, }, // Asia/Hebron
    {   70, 1226,    2352,  7, }, // Asia/Ho_Chi_Minh
    {   70, 2576,   35576,  7, }, // Asia/Hong_Kong
    {   70, 4673,    2366,  4, }, // Asia/Hovd
    {   70, 4086,    2370,  7, }, // Asia/Irkutsk
    {   70, 6969,   27229,  8, }, // Asia/Jakarta
    {   70, 6955,   27237,  8, }, // Asia/Jayapura
    {   70, 1257,   35583,  9, }, // Asia/Jerusalem
    {   70, 3462,    2401,  5, }, // Asia/Kabul
    {   70, 4218,    2420,  8, }, // Asia/Kamchatka
    {   70, 4141,    2431,  6, }, // Asia/Karachi
    {   70, 1152,    2437,  8, }, // Asia/Kathmandu
    {   70, 8624,   33545,  7, }, // Asia/Khandyga
    {   70,  992,   35592,  7, }, // Asia/Kolkata
    {   70, 4099,   35599, 10, }, // Asia/Krasnoyarsk
    {   70, 4761,   33570, 12, }, // Asia/Kuala_Lumpur
    {   70, 5893,    2483,  5, }, // Asia/Kuching
    {   70, 5464,   27284,  6, }, // Asia/Kuwait
    {   70, 1178,    2495,  5, }, // Asia/Macau
    {   70, 3919,    2500,  7, }, // Asia/Magadan
    {   70, 1316,    2507,  7, }, // Asia/Makassar
    {   70, 7145,    2514,  6, }, // Asia/Manila
    {   70, 6883,   33593,  6, }, // Asia/Muscat
    {   70, 2446,   35609,  7, }, // Asia/Nicosia
    {   70, 8638,    2533, 11, }, // Asia/Novokuznetsk
    {   70, 4037,    2544, 11, }, // Asia/Novosibirsk
    {   70, 4131,    2555,  4, }, // Asia/Omsk
    {   70, 7108,   35616,  4, }, // Asia/Oral
    {   70, 6924,   33622,  8, }, // Asia/Phnom_Penh
    {   70, 8656,    2572,  9, }, // Asia/Pontianak
    {   70, 4116,   35620,  6, }, // Asia/Pyongyang
    {   70, 5476,    2588,  5, }, // Asia/Qatar
    {   70, 8671,   35626,  8, }, // Asia/Qostanay
    {   70, 4171,   35634,  9, }, // Asia/Qyzylorda
    {   70, 3486,   35643,  4, }, // Asia/Riyadh
    {   70, 4310,    2623,  7, }, // Asia/Sakhalin
    {   70, 7208,    2630,  9, }, // Asia/Samarkand
    {   70, 3116,    2639,  4, }, // Asia/Seoul
    {   70, 1020,   27392,  6, }, // Asia/Shanghai
    {   70, 3137,    2649,  8, }, // Asia/Singapore
    {   70, 4199,   35647, 13, }, // Asia/Srednekolymsk
    {   70, 3100,   35660,  5, }, // Asia/Taipei
    {   70, 4683,    2676,  7, }, // Asia/Tashkent
    {   70, 3817,   33686,  7, }, // Asia/Tbilisi
    {   70, 2638,   33693,  7, }, // Asia/Tehran
    {   70, 1284,    2694,  7, }, // Asia/Thimphu
    {   70, 2687,    2701,  5, }, // Asia/Tokyo
    {   70, 4474,    2706,  5, }, // Asia/Tomsk
    {   70, 1346,   33705, 10, }, // Asia/Ulaanbaatar
    {   70, 1126,    2721,  6, }, // Asia/Urumqi
    {   70, 8685,    2727,  8, }, // Asia/Ust-Nera
    {   70, 6940,   35665,  7, }, // Asia/Vientiane
    {   70, 4612,    2742, 11, }, // Asia/Vladivostok
    {   70, 4730,   35672,  6, }, // Asia/Yakutsk
    {   70, 1202,   27502,  6, }, // Asia/Yangon
    {   70, 3785,    2766, 12, }, // Asia/Yekaterinburg
    {   70, 3649,    2778,  6, }, // Asia/Yerevan
    {   70, 3565,   35678, 15, }, // Atlantic/Azores
    {   70, 5588,   35693, 17, }, // Atlantic/Bermuda
    {   70, 6591,   35710, 16, }, // Atlantic/Canary
    {   70, 3629,   35726, 10, }, // Atlantic/Cape_Verde
    {   70, 1379,   35736, 17, }, // Atlantic/Faroe
    {   70, 8699,   35753,  7, }, // Atlantic/Madeira
    {   70, 2614,   35760,  8, }, // Atlantic/Reykjavik
    {   70, 7235,   35768, 13, }, // Atlantic/South_Georgia
    {   70, 6784,   35781, 11, }, // Atlantic/St_Helena
    {   70, 6607,    2920,  6, }, // Atlantic/Stanley
    {   70, 1654,    2926,  8, }, // Australia/Adelaide
    {   70, 1619,   35792,  8, }, // Australia/Brisbane
    {   70, 1783,   35800, 11, }, // Australia/Broken_Hill
    {   70, 1567,    2952,  6, }, // Australia/Darwin
    {   70, 3539,   27699,  4, }, // Australia/Eucla
    {   70, 1500,    2963,  6, }, // Australia/Hobart
    {   70, 8716,    2969,  8, }, // Australia/Lindeman
    {   70, 1531,   35811,  8, }, // Australia/Lord_Howe
    {   70, 1711,   35819,  7, }, // Australia/Melbourne
    {   70, 1746,   35826,  4, }, // Australia/Perth
    {   70, 1447,   35830,  5, }, // Australia/Sydney
    {   70, 6390,    3002,  9, }, // Europe/Amsterdam
    {   70, 6149,   27734,  6, }, // Europe/Andorra
    {   70, 3522,    3018,  8, }, // Europe/Astrakhan
    {   70, 6577,   33881,  5, }, // Europe/Athens
    {   70, 6285,    3030,  7, }, // Europe/Belgrade
    {   70, 4659,    3037,  6, }, // Europe/Berlin
    {   70, 6437,    3043, 10, }, // Europe/Bratislava
    {   70, 6178,   35835,  7, }, // Europe/Brussels
    {   70, 3830,   35842,  7, }, // Europe/Bucharest
    {   70, 3708,   35849,  8, }, // Europe/Budapest
    {   70, 8751,   35857,  8, }, // Europe/Busingen
    {   70, 2475,    3100,  7, }, // Europe/Chisinau
    {   70, 6238,   33932, 10, }, // Europe/Copenhagen
    {   70, 2260,   35865,  6, }, // Europe/Dublin
    {   70, 6256,    3123,  9, }, // Europe/Gibraltar
    {   70, 8767,   35871,  6, }, // Europe/Guernsey
    {   70, 6561,   35877,  8, }, // Europe/Helsinki
    {   70, 8783,   35885, 10, }, // Europe/Isle_of_Man
    {   70, 1097,   33964,  8, }, // Europe/Istanbul
    {   70, 8802,   35895,  6, }, // Europe/Jersey
    {   70, 3881,    3167, 11, }, // Europe/Kaliningrad
    {   70, 8924,    3178,  5, }, // Europe/Kirov
    {   70, 2419,    3183,  4, }, // Europe/Kyiv
    {   70, 3078,   33988,  7, }, // Europe/Lisbon
    {   70, 6455,   27891,  7, }, // Europe/Ljubljana
    {   70, 2393,    3204,  6, }, // Europe/London
    {   70, 6314,   27904, 10, }, // Europe/Luxembourg
    {   70, 4794,    3221,  6, }, // Europe/Madrid
    {   70, 6346,    3227,  5, }, // Europe/Malta
    {   70, 6531,    3232,  9, }, // Europe/Mariehamn
    {   70, 3595,    3241,  5, }, // Europe/Minsk
    {   70, 6359,    3246,  6, }, // Europe/Monaco
    {   70, 3443,    3252,  6, }, // Europe/Moscow
    {   70, 6407,    3258,  4, }, // Europe/Oslo
    {   70, 4186,    3262,  5, }, // Europe/Paris
    {   70, 6373,    3267,  9, }, // Europe/Podgorica
    {   70, 6224,    3276,  5, }, // Europe/Prague
    {   70, 8816,    3281,  4, }, // Europe/Riga
    {   70, 6273,    3285,  3, }, // Europe/Rome
    {   70, 4233,    3288,  6, }, // Europe/Samara
    {   70, 6419,   34025, 10, }, // Europe/San_Marino
    {   70, 6194,    3304,  7, }, // Europe/Sarajevo
    {   70, 4353,    3311,  7, }, // Europe/Saratov
    {   70, 8828,    3318, 10, }, // Europe/Simferopol
    {   70, 6332,   35901,  6, }, // Europe/Skopje
    {   70, 6548,   35907,  5, }, // Europe/Sofia
    {   70, 6472,   34065,  8, }, // Europe/Stockholm
    {   70, 8846,   34073,  5, }, // Europe/Tallinn
    {   70, 6135,    3352,  6, }, // Europe/Tirane
    {   70, 8861,   35912,  8, }, // Europe/Ulyanovsk
    {   70, 6301,    3367,  5, }, // Europe/Vaduz
    {   70, 6516,    3372,  7, }, // Europe/Vatican
    {   70, 6164,   35920,  5, }, // Europe/Vienna
    {   70, 8878,   35925,  6, }, // Europe/Vilnius
    {   70, 4629,    3390,  9, }, // Europe/Volgograd
    {   70, 3055,    3399,  7, }, // Europe/Warsaw
    {   70, 6210,    3406,  6, }, // Europe/Zagreb
    {   70, 6489,   35931,  5, }, // Europe/Zurich
    {   70, 4980,   34105, 12, }, // Indian/Antananarivo
    {   70, 6910,    3430,  5, }, // Indian/Chagos
    {   70, 5979,   35936,  9, }, // Indian/Christmas
    {   70, 5996,   35945, 16, }, // Indian/Cocos
    {   70, 4931,   35961, 16, }, // Indian/Comoro
    {   70, 6624,    3469,  8, }, // Indian/Kerguelen
    {   70, 7223,    3477,  3, }, // Indian/Mahe
    {   70, 7013,   34132,  7, }, // Indian/Maldives
    {   70, 3971,   35977,  8, }, // Indian/Mauritius
    {   70, 5000,   35985,  5, }, // Indian/Mayotte
    {   70, 7174,   35990,  7, }, // Indian/Reunion
    {   70, 4324,   35997,  4, }, // Pacific/Apia
    {   70,  932,   34166,  6, }, // Pacific/Auckland
    {   70, 3608,    3517,  8, }, // Pacific/Bougainville
    {   70, 2863,   36001,  5, }, // Pacific/Chatham
    {   70, 3022,   36006,  4, }, // Pacific/Chuuk
    {   70, 2201,   36010, 18, }, // Pacific/Easter
    {   70, 7323,    3545,  5, }, // Pacific/Efate
    {   70, 7290,    3550,  7, }, // Pacific/Fakaofo
    {   70, 3804,   36028,  5, }, // Pacific/Fiji
    {   70, 7306,    3561,  8, }, // Pacific/Funafuti
    {   70, 6641,    3569,  9, }, // Pacific/Galapagos
    {   70, 6659,   36033,  6, }, // Pacific/Gambier
    {   70, 3724,    3604, 11, }, // Pacific/Guadalcanal
    {   70, 5935,    3615,  4, }, // Pacific/Guam
    {   70, 2929,   34210,  8, }, // Pacific/Honolulu
    {   70, 2897,   36039,  9, }, // Pacific/Kanton
    {   70, 3900,    3625, 10, }, // Pacific/Kiritimati
    {   70, 6982,   36048,  6, }, // Pacific/Kosrae
    {   70, 2708,   36054,  9, }, // Pacific/Kwajalein
    {   70, 4779,   36063,  7, }, // Pacific/Majuro
    {   70, 3953,   36070, 16, }, // Pacific/Marquesas
    {   70, 8909,   36086,  6, }, // Pacific/Midway
    {   70, 7047,    3689,  5, }, // Pacific/Nauru
    {   70, 7095,    3694,  4, }, // Pacific/Niue
    {   70, 4070,    3698,  7, }, // Pacific/Norfolk
    {   70, 7061,    3705,  5, }, // Pacific/Noumea
    {   70, 2991,   34254,  9, }, // Pacific/Pago_Pago
    {   70, 7118,    3719,  5, }, // Pacific/Palau
    {   70, 7157,    3724,  7, }, // Pacific/Pitcairn
    {   70, 2961,   36092,  6, }, // Pacific/Pohnpei
    {   70, 4709,   36098, 11, }, // Pacific/Port_Moresby
    {   70, 6009,    3748,  9, }, // Pacific/Rarotonga
    {   70, 5948,   28394,  6, }, // Pacific/Saipan
    {   70, 7275,    3763,  5, }, // Pacific/Tahiti
    {   70, 6675,   36109,  6, }, // Pacific/Tarawa
    {   70, 4485,    3774,  9, }, // Pacific/Tongatapu
    {   70, 7355,   36115,  4, }, // Pacific/Wake
    {   70, 7368,   36119,  5, }, // Pacific/Wallis
    {   71, 6751,   36124,  8, }, // Africa/Abidjan Burmese/Myanmar/Myanmar
    {   71, 6723,   36132,  6, }, // Africa/Accra
    {   71, 4961,   36138, 12, }, // Africa/Addis_Ababa
    {   71, 7383,   36150, 15, }, // Africa/Algiers
    {   71,   14,   36165, 10, }, // Africa/Asmara
    {   71,   44,   36175,  7, }, // Africa/Bamako
    {   71, 5159,   36182,  7, }, // Africa/Bangui
    {   71, 6709,   36189,  9, }, // Africa/Banjul
    {   71, 7398,   36198,  7, }, // Africa/Bissau
    {   71, 4859,   36205, 14, }, // Africa/Blantyre
    {   71, 5189,   36219, 11, }, // Africa/Brazzaville
    {   71, 4824,   36230, 12, }, // Africa/Bujumbura
    {   71, 2242,   36242,  8, }, // Africa/Cairo
    {   71, 4019,   36250, 10, }, // Africa/Casablanca
    {   71, 7412,   36260,  7, }, // Africa/Ceuta
    {   71, 6736,   36267,  9, }, // Africa/Conakry
    {   71, 6803,   36276,  6, }, // Africa/Dakar
    {   71, 5032,   36282, 12, }, // Africa/Dar_es_Salaam
    {   71, 4945,   36294,  7, }, // Africa/Djibouti
    {   71, 5145,   36301,  9, }, // Africa/Douala
    {   71, 5068,   36310, 12, }, // Africa/El_Aaiun
    {   71, 6816,   36322, 12, }, // Africa/Freetown
    {   71, 4808,   36334, 11, }, // Africa/Gaborone
    {   71, 4917,   36345,  9, }, // Africa/Harare
    {   71, 4381,   36354, 15, }, // Africa/Johannesburg
    {   71, 4401,   36369,  5, }, // Africa/Juba
    {   71, 5053,   36374,  7, }, // Africa/Kampala
    {   71, 4426,   36381,  8, }, // Africa/Khartoum
    {   71, 4875,   36389,  7, }, // Africa/Kigali
    {   71, 5208,   36396,  8, }, // Africa/Kinshasa
    {   71, 4646,   36404,  7, }, // Africa/Lagos
    {   71, 5238,   36411, 10, }, // Africa/Libreville
    {   71, 6832,   36421,  5, }, // Africa/Lome
    {   71, 5113,   36426,  7, }, // Africa/Luanda
    {   71, 4841,   36433, 12, }, // Africa/Lubumbashi
    {   71, 4903,   36445,  6, }, // Africa/Lusaka
    {   71, 5224,   36451,  7, }, // Africa/Malabo
    {   71, 4889,   36458,  8, }, // Africa/Maputo
    {   71, 5099,   36466,  7, }, // Africa/Maseru
    {   71, 5084,   36473,  8, }, // Africa/Mbabane
    {   71, 5015,   36481, 13, }, // Africa/Mogadishu
    {   71, 6997,   36494, 14, }, // Africa/Monrovia
    {   71, 3755,   36508, 10, }, // Africa/Nairobi
    {   71, 5173,   36518, 10, }, // Africa/Ndjamena
    {   71, 5256,   36528,  6, }, // Africa/Niamey
    {   71, 6766,   36534, 15, }, // Africa/Nouakchott
    {   71, 6690,   36549,  9, }, // Africa/Ouagadougou
    {   71, 5127,   36558, 13, }, // Africa/Porto-Novo
    {   71, 4337,   36571, 10, }, // Africa/Sao_Tome
    {   71, 2732,   36581,  8, }, // Africa/Tripoli
    {   71, 6503,   36589,  6, }, // Africa/Tunis
    {   71, 4054,   36595,  9, }, // Africa/Windhoek
    {   71,  132,   36604,  5, }, // America/Adak
    {   71, 3183,   36609, 11, }, // America/Anchorage
    {   71, 5524,   36620,  7, }, // America/Anguilla
    {   71, 5541,   36627,  8, }, // America/Antigua
    {   71, 4456,   36635, 11, }, // America/Araguaina
    {   71,  166,   36646, 13, }, // America/Argentina/Buenos_Aires
    {   71,   91,   36659,  9, }, // America/Argentina/Catamarca
    {   71,  270,   36668,  8, }, // America/Argentina/Cordoba
    {   71,  441,   36676,  6, }, // America/Argentina/Jujuy
    {   71, 7425,   36682, 10, }, // America/Argentina/La_Rioja
    {   71,  565,   36692,  8, }, // America/Argentina/Mendoza
    {   71, 7452,   36700, 15, }, // America/Argentina/Rio_Gallegos
    {   71, 7483,   36715,  6, }, // America/Argentina/Salta
    {   71, 7507,   36721,  8, }, // America/Argentina/San_Juan
    {   71, 5497,   36729, 10, }, // America/Argentina/San_Luis
    {   71, 7534,   36739,  7, }, // America/Argentina/Tucuman
    {   71, 7560,   36746,  8, }, // America/Argentina/Ushuaia
    {   71, 5557,   36754,  7, }, // America/Aruba
    {   71, 4154,   36761, 11, }, // America/Asuncion
    {   71,  237,   36772, 10, }, // America/Atikokan
    {   71, 3581,   36782,  6, }, // America/Bahia
    {   71, 7586,   36788, 16, }, // America/Bahia_Banderas
    {   71, 5571,   36804, 11, }, // America/Barbados
    {   71, 7609,   36815,  7, }, // America/Belem
    {   71, 5309,   36822,  5, }, // America/Belize
    {   71, 7623,   36827, 15, }, // America/Blanc-Sablon
    {   71, 7644,   36842, 11, }, // America/Boa_Vista
    {   71, 4263,   36853,  8, }, // America/Bogota
    {   71, 7662,   36861,  8, }, // America/Boise
    {   71, 7676,   36869, 16, }, // America/Cambridge_Bay
    {   71, 7698,   36885, 14, }, // America/Campo_Grande
    {   71, 3770,   36899,  6, }, // America/Cancun
    {   71, 4596,   36905,  8, }, // America/Caracas
    {   71, 4247,   36913,  7, }, // America/Cayenne
    {   71, 5398,   36920,  5, }, // America/Cayman
    {   71, 3251,   36925,  8, }, // America/Chicago
    {   71, 7719,   36933, 11, }, // America/Chihuahua
    {   71, 7737,   36944, 18, }, // America/Ciudad_Juarez
    {   71, 5324,   36962, 11, }, // America/Costa_Rica
    {   71, 7759,   36973,  9, }, // America/Creston
    {   71, 3693,   36982,  7, }, // America/Cuiaba
    {   71, 5640,   36989, 10, }, // America/Curacao
    {   71, 7775,   36999, 13, }, // America/Danmarkshavn
    {   71, 7796,   37012,  7, }, // America/Dawson
    {   71, 7811,   37019, 13, }, // America/Dawson_Creek
    {   71,  805,   37032,  7, }, // America/Denver
    {   71, 3351,   37039, 10, }, // America/Detroit
    {   71, 5656,   37049,  9, }, // America/Dominica
    {   71,  893,   37058, 12, }, // America/Edmonton
    {   71, 7832,   37070,  8, }, // America/Eirunepe
    {   71, 5343,   37078, 12, }, // America/El_Salvador
    {   71, 7849,   37090, 15, }, // America/Fort_Nelson
    {   71, 7869,   37105, 12, }, // America/Fortaleza
    {   71, 7887,   37117,  8, }, // America/Glace_Bay
    {   71, 6844,   37125,  8, }, // America/Goose_Bay
    {   71, 4514,   37133, 10, }, // America/Grand_Turk
    {   71, 5687,   37143,  7, }, // America/Grenada
    {   71, 5703,   37150,  9, }, // America/Guadeloupe
    {   71, 3662,   37159,  9, }, // America/Guatemala
    {   71, 4743,   37168, 11, }, // America/Guayaquil
    {   71, 6895,   37179,  9, }, // America/Guyana
    {   71, 1923,   37188,  8, }, // America/Halifax
    {   71, 2221,   37196,  6, }, // America/Havana
    {   71, 7905,   37202, 12, }, // America/Hermosillo
    {   71,  348,   37214, 19, }, // America/Indiana/Indianapolis
    {   71,  481,   37233, 22, }, // America/Indiana/Knox
    {   71, 7924,   37255, 21, }, // America/Indiana/Marengo
    {   71, 7948,   37276, 24, }, // America/Indiana/Petersburg
    {   71, 7975,   37300, 23, }, // America/Indiana/Tell_City
    {   71, 8001,   37323, 17, }, // America/Indiana/Vevay
    {   71, 8023,   37340, 22, }, // America/Indiana/Vincennes
    {   71, 8049,   37362, 22, }, // America/Indiana/Winamac
    {   71, 8073,   37384,  8, }, // America/Inuvik
    {   71,  660,   37392, 10, }, // America/Iqaluit
    {   71, 2665,   37402,  6, }, // America/Jamaica
    {   71, 5282,   37408,  9, }, // America/Juneau
    {   71,  521,   37417,  9, }, // America/Kentucky/Louisville
    {   71, 8088,   37426, 23, }, // America/Kentucky/Monticello
    {   71, 5621,   37449, 14, }, // America/Kralendijk
    {   71, 4278,   37463,  6, }, // America/La_Paz
    {   71, 7132,   37469,  4, }, // America/Lima
    {   71, 3390,   37473, 18, }, // America/Los_Angeles
    {   71, 5849,   37491, 18, }, // America/Lower_Princes
    {   71, 8116,   37509,  7, }, // America/Maceio
    {   71, 8131,   37516,  7, }, // America/Managua
    {   71, 1892,   37523,  9, }, // America/Manaus
    {   71, 5814,   37532,  8, }, // America/Marigot
    {   71, 5722,   37540,  8, }, // America/Martinique
    {   71, 8147,   37548, 12, }, // America/Matamoros
    {   71, 2779,   37560,  8, }, // America/Mazatlan
    {   71, 8165,   37568, 10, }, // America/Menominee
    {   71, 8183,   37578,  6, }, // America/Merida
    {   71, 8198,   37584, 12, }, // America/Metlakatla
    {   71, 2811,   37596, 16, }, // America/Mexico_City
    {   71, 4293,   37612,  9, }, // America/Miquelon
    {   71, 8217,   37621, 10, }, // America/Moncton
    {   71, 8233,   37631,  9, }, // America/Monterrey
    {   71, 4000,   37640, 13, }, // America/Montevideo
    {   71, 5741,   37653, 15, }, // America/Montserrat
    {   71, 5383,   37668,  7, }, // America/Nassau
    {   71, 3294,   37675,  9, }, // America/New_York
    {   71, 8251,   37570,  5, }, // America/Nome
    {   71, 1834,   37684,  8, }, // America/Noronha
    {   71, 8264,   37692, 20, }, // America/North_Dakota/Beulah
    {   71, 8292,   37712, 20, }, // America/North_Dakota/Center
    {   71, 8320,   37732, 25, }, // America/North_Dakota/New_Salem
    {   71,  393,   37757,  4, }, // America/Nuuk
    {   71, 8351,   37761,  8, }, // America/Ojinaga
    {   71, 5413,   37769,  7, }, // America/Panama
    {   71, 6106,   37776, 11, }, // America/Paramaribo
    {   71, 3224,   37787,  6, }, // America/Phoenix
    {   71, 3847,   37793, 18, }, // America/Port-au-Prince
    {   71, 5871,   37811, 19, }, // America/Port_of_Spain
    {   71, 8367,   37830, 17, }, // America/Porto_Velho
    {   71, 5760,   37847, 12, }, // America/Puerto_Rico
    {   71, 3932,   37859, 14, }, // America/Punta_Arenas
    {   71, 8387,   37873, 14, }, // America/Rankin_Inlet
    {   71, 8408,   37887,  8, }, // America/Recife
    {   71, 1979,   37895,  8, }, // America/Regina
    {   71, 8423,   37903, 13, }, // America/Resolute
    {   71,  695,   37916, 13, }, // America/Rio_Branco
    {   71, 8440,   37929,  8, }, // America/Santarem
    {   71, 2165,   37937, 10, }, // America/Santiago
    {   71, 6044,   37947, 17, }, // America/Santo_Domingo
    {   71, 1862,   37964, 10, }, // America/Sao_Paulo
    {   71, 6862,   37974, 19, }, // America/Scoresbysund
    {   71, 8457,   37993,  5, }, // America/Sitka
    {   71, 8471,   37998, 13, }, // America/St_Barthelemy
    {   71, 2045,   38011, 11, }, // America/St_Johns
    {   71, 5780,   38022,  9, }, // America/St_Kitts
    {   71, 5797,   38031, 11, }, // America/St_Lucia
    {   71,  855,   38042, 13, }, // America/St_Thomas
    {   71, 5830,   38055, 13, }, // America/St_Vincent
    {   71, 8493,   38068, 16, }, // America/Swift_Current
    {   71, 5363,   38084, 11, }, // America/Tegucigalpa
    {   71, 5673,   38095,  4, }, // America/Thule
    {   71,  313,   38099,  7, }, // America/Tijuana
    {   71,  608,   38106,  9, }, // America/Toronto
    {   71, 5605,   38115,  8, }, // America/Tortola
    {   71, 2077,   38123,  9, }, // America/Vancouver
    {   71, 2128,   38132, 14, }, // America/Whitehorse
    {   71,  734,   38146,  9, }, // America/Winnipeg
    {   71, 8515,   38155,  7, }, // America/Yakutat
    {   71, 5918,   38162,  4, }, // Antarctica/Casey
    {   71, 6027,   38166,  6, }, // Antarctica/Davis
    {   71, 6066,   38172, 19, }, // Antarctica/DumontDUrville
    {   71, 8531,   38191,  9, }, // Antarctica/Macquarie
    {   71, 7029,   38200,  8, }, // Antarctica/Mawson
    {   71, 7076,   38208,  9, }, // Antarctica/McMurdo
    {   71, 8552,   38217,  6, }, // Antarctica/Palmer
    {   71, 7189,   38223,  7, }, // Antarctica/Rothera
    {   71, 7258,   38230,  7, }, // Antarctica/Syowa
    {   71, 8570,   38237,  6, }, // Antarctica/Troll
    {   71, 7337,   38243, 10, }, // Antarctica/Vostok
    {   71, 1413,   38253, 13, }, // Arctic/Longyearbyen
    {   71, 5487,   38266,  5, }, // Asia/Aden
    {   71, 5297,   38271,  8, }, // Asia/Almaty
    {   71, 3870,   38279,  7, }, // Asia/Amman
    {   71, 5428,   38286,  9, }, // Asia/Anadyr
    {   71, 5440,   38295,  7, }, // Asia/Aqtau
    {   71, 5270,   38302,  9, }, // Asia/Aqtobe
    {   71,  964,   38311, 11, }, // Asia/Ashgabat
    {   71, 8587,   38322,  7, }, // Asia/Atyrau
    {   71, 3509,   38329,  7, }, // Asia/Baghdad
    {   71, 5451,   38336,  7, }, // Asia/Bahrain
    {   71, 3555,   38343,  4, }, // Asia/Baku
    {   71, 4368,   38347,  8, }, // Asia/Bangkok
    {   71, 3473,   38355,  7, }, // Asia/Barnaul
    {   71, 3988,   38362,  6, }, // Asia/Beirut
    {   71, 3680,   38368,  8, }, // Asia/Bishkek
    {   71, 5906,   38376,  9, }, // Asia/Brunei
    {   71, 4503,   38385,  5, }, // Asia/Chita
    {   71, 5963,   38390, 10, }, // Asia/Choibalsan
    {   71, 4413,   38400,  8, }, // Asia/Colombo
    {   71, 4442,   38408,  8, }, // Asia/Damascus
    {   71, 1060,   38416,  5, }, // Asia/Dhaka
    {   71, 6125,   38421,  5, }, // Asia/Dili
    {   71, 3498,   38426,  8, }, // Asia/Dubai
    {   71, 6092,   38434,  9, }, // Asia/Dushanbe
    {   71, 8599,   38443,  9, }, // Asia/Famagusta
    {   71, 8614,   38452,  4, }, // Asia/Gaza
    {   71, 4697,   38456,  7, }, // Asia/Hebron
    {   71, 1226,   38463, 16, }, // Asia/Ho_Chi_Minh
    {   71, 2576,   38479, 10, }, // Asia/Hong_Kong
    {   71, 4673,   38489,  6, }, // Asia/Hovd
    {   71, 4086,   38495,  8, }, // Asia/Irkutsk
    {   71, 6969,   38503,  6, }, // Asia/Jakarta
    {   71, 6955,   38509,  9, }, // Asia/Jayapura
    {   71, 1257,   38518,  9, }, // Asia/Jerusalem
    {   71, 3462,   38527,  6, }, // Asia/Kabul
    {   71, 4218,   38533,  9, }, // Asia/Kamchatka
    {   71, 4141,   38542,  6, }, // Asia/Karachi
    {   71, 1152,   38548,  9, }, // Asia/Kathmandu
    {   71, 8624,   38557, 10, }, // Asia/Khandyga
    {   71,  992,   38567, 11, }, // Asia/Kolkata
    {   71, 4099,   38578, 13, }, // Asia/Krasnoyarsk
    {   71, 4761,   38591, 10, }, // Asia/Kuala_Lumpur
    {   71, 5893,   38601,  6, }, // Asia/Kuching
    {   71, 5464,   38607,  6, }, // Asia/Kuwait
    {   71, 1178,   38613,  6, }, // Asia/Macau
    {   71, 3919,   38619,  8, }, // Asia/Magadan
    {   71, 1316,   38627,  8, }, // Asia/Makassar
    {   71, 7145,   38635,  5, }, // Asia/Manila
    {   71, 6883,   38640, 10, }, // Asia/Muscat
    {   71, 2446,   38650,  9, }, // Asia/Nicosia
    {   71, 8638,   38659, 15, }, // Asia/Novokuznetsk
    {   71, 4037,   38674, 16, }, // Asia/Novosibirsk
    {   71, 4131,   38690,  9, }, // Asia/Omsk
    {   71, 7108,   38699,  8, }, // Asia/Oral
    {   71, 6924,   38707,  8, }, // Asia/Phnom_Penh
    {   71, 8656,   38715, 14, }, // Asia/Pontianak
    {   71, 4116,   38729,  8, }, // Asia/Pyongyang
    {   71, 5476,   36659,  4, }, // Asia/Qatar
    {   71, 8671,   38737,  9, }, // Asia/Qostanay
    {   71, 4171,   38746,  9, }, // Asia/Qyzylorda
    {   71, 3486,   38755,  7, }, // Asia/Riyadh
    {   71, 4310,   38762,  7, }, // Asia/Sakhalin
    {   71, 7208,   38769, 12, }, // Asia/Samarkand
    {   71, 3116,   38781,  6, }, // Asia/Seoul
    {   71, 1020,   38787, 10, }, // Asia/Shanghai
    {   71, 3137,   38797,  8, }, // Asia/Singapore
    {   71, 4199,   38805, 16, }, // Asia/Srednekolymsk
    {   71, 3100,   38821,  7, }, // Asia/Taipei
    {   71, 4683,   38828, 10, }, // Asia/Tashkent
    {   71, 3817,   38838,  8, }, // Asia/Tbilisi
    {   71, 2638,   38846,  7, }, // Asia/Tehran
    {   71, 1284,   38853,  6, }, // Asia/Thimphu
    {   71, 2687,   38859,  7, }, // Asia/Tokyo
    {   71, 4474,   38866, 10, }, // Asia/Tomsk
    {   71, 1346,   38876,  9, }, // Asia/Ulaanbaatar
    {   71, 1126,   38885,  9, }, // Asia/Urumqi
    {   71, 8685,   38894, 11, }, // Asia/Ust-Nera
    {   71, 6940,   38905, 10, }, // Asia/Vientiane
    {   71, 4612,   38915, 16, }, // Asia/Vladivostok
    {   71, 4730,   38931,  9, }, // Asia/Yakutsk
    {   71, 1202,   38940,  7, }, // Asia/Yangon
    {   71, 3785,   38947, 16, }, // Asia/Yekaterinburg
    {   71, 3649,   38963, 10, }, // Asia/Yerevan
    {   71, 3565,   38973, 11, }, // Atlantic/Azores
    {   71, 5588,   38984,  7, }, // Atlantic/Bermuda
    {   71, 6591,   38991,  5, }, // Atlantic/Canary
    {   71, 3629,   38996, 11, }, // Atlantic/Cape_Verde
    {   71, 1379,   39007,  5, }, // Atlantic/Faroe
    {   71, 8699,   39012,  6, }, // Atlantic/Madeira
    {   71, 2614,   39018, 10, }, // Atlantic/Reykjavik
    {   71, 7235,   39028, 15, }, // Atlantic/South_Georgia
    {   71, 6784,   39043, 13, }, // Atlantic/St_Helena
    {   71, 6607,   39056,  6, }, // Atlantic/Stanley
    {   71, 1654,   39062, 11, }, // Australia/Adelaide
    {   71, 1619,   39073, 10, }, // Australia/Brisbane
    {   71, 1783,   39083, 12, }, // Australia/Broken_Hill
    {   71, 1567,   39095,  5, }, // Australia/Darwin
    {   71, 3539,   39100,  6, }, // Australia/Eucla
    {   71, 1500,   39106,  9, }, // Australia/Hobart
    {   71, 8716,   39115, 10, }, // Australia/Lindeman
    {   71, 1531,   39125, 12, }, // Australia/Lord_Howe
    {   71, 1711,   39137,  9, }, // Australia/Melbourne
    {   71, 1746,   39146,  5, }, // Australia/Perth
    {   71, 1447,   39151,  6, }, // Australia/Sydney
    {   71, 6390,   39157,  9, }, // Europe/Amsterdam
    {   71, 6149,   39166,  8, }, // Europe/Andorra
    {   71, 3522,   39174, 11, }, // Europe/Astrakhan
    {   71, 6577,   39185,  5, }, // Europe/Athens
    {   71, 6285,   39190,  9, }, // Europe/Belgrade
    {   71, 4659,   39199,  5, }, // Europe/Berlin
    {   71, 6437,   39204, 13, }, // Europe/Bratislava
    {   71, 6178,   39217,  8, }, // Europe/Brussels
    {   71, 3830,   39225,  9, }, // Europe/Bucharest
    {   71, 3708,   39234,  9, }, // Europe/Budapest
    {   71, 8751,   39243, 10, }, // Europe/Busingen
    {   71, 2475,   39253, 11, }, // Europe/Chisinau
    {   71, 6238,   39264, 11, }, // Europe/Copenhagen
    {   71, 2260,   39275,  7, }, // Europe/Dublin
    {   71, 6256,   39282, 12, }, // Europe/Gibraltar
    {   71, 8767,   39294,  7, }, // Europe/Guernsey
    {   71, 6561,   39301,  9, }, // Europe/Helsinki
    {   71, 8783,   39310, 10, }, // Europe/Isle_of_Man
    {   71, 1097,   39320, 11, }, // Europe/Istanbul
    {   71, 8802,   39331,  5, }, // Europe/Jersey
    {   71, 3881,   39336, 11, }, // Europe/Kaliningrad
    {   71, 8924,   39347,  7, }, // Europe/Kirov
    {   71, 2419,   39354,  8, }, // Europe/Kyiv
    {   71, 3078,   39362,  9, }, // Europe/Lisbon
    {   71, 6455,   39371, 11, }, // Europe/Ljubljana
    {   71, 2393,   39382,  6, }, // Europe/London
    {   71, 6314,   39388,  8, }, // Europe/Luxembourg
    {   71, 4794,   39396,  5, }, // Europe/Madrid
    {   71, 6346,   39401,  8, }, // Europe/Malta
    {   71, 6531,   39409, 12, }, // Europe/Mariehamn
    {   71, 3595,   39421,  7, }, // Europe/Minsk
    {   71, 6359,   39428,  8, }, // Europe/Monaco
    {   71, 3443,   39436,  8, }, // Europe/Moscow
    {   71, 6407,   39444,  8, }, // Europe/Oslo
    {   71, 4186,   39452,  5, }, // Europe/Paris
    {   71, 6373,   39457, 11, }, // Europe/Podgorica
    {   71, 6224,   39468,  6, }, // Europe/Prague
    {   71, 8816,   39474,  4, }, // Europe/Riga
    {   71, 6273,   39478,  4, }, // Europe/Rome
    {   71, 4233,   39482,  7, }, // Europe/Samara
    {   71, 6419,   39489, 10, }, // Europe/San_Marino
    {   71, 6194,   39499,  9, }, // Europe/Sarajevo
    {   71, 4353,   39508, 10, }, // Europe/Saratov
    {   71, 8828,   39518, 12, }, // Europe/Simferopol
    {   71, 6332,   39530,  9, }, // Europe/Skopje
    {   71, 6548,   39539,  7, }, // Europe/Sofia
    {   71, 6472,   39546, 10, }, // Europe/Stockholm
    {   71, 8846,   39556,  6, }, // Europe/Tallinn
    {   71, 6135,   39562,  6, }, // Europe/Tirane
    {   71, 8861,   39568, 15, }, // Europe/Ulyanovsk
    {   71, 6301,   39583,  7, }, // Europe/Vaduz
    {   71, 6516,   39590,  7, }, // Europe/Vatican
    {   71, 6164,   39597,  7, }, // Europe/Vienna
    {   71, 8878,   39604, 13, }, // Europe/Vilnius
    {   71, 4629,   39617, 12, }, // Europe/Volgograd
    {   71, 3055,   39629,  5, }, // Europe/Warsaw
    {   71, 6210,   39634, 10, }, // Europe/Zagreb
    {   71, 6489,   39644,  9, }, // Europe/Zurich
    {   71, 4980,   39653, 15, }, // Indian/Antananarivo
    {   71, 6910,   39668,  8, }, // Indian/Chagos
    {   71, 5979,   39676,  8, }, // Indian/Christmas
    {   71, 5996,   39684,  9, }, // Indian/Cocos
    {   71, 4931,   39693,  9, }, // Indian/Comoro
    {   71, 6624,   39702,  7, }, // Indian/Kerguelen
    {   71, 7223,   39709,  4, }, // Indian/Mahe
    {   71, 7013,   39713, 10, }, // Indian/Maldives
    {   71, 3971,   39723,  8, }, // Indian/Mauritius
    {   71, 5000,   39731,  7, }, // Indian/Mayotte
    {   71, 7174,   39738,  9, }, // Indian/Reunion
    {   71, 4324,   39747,  8, }, // Pacific/Apia
    {   71,  932,   39755,  8, }, // Pacific/Auckland
    {   71, 3608,   39763, 14, }, // Pacific/Bougainville
    {   71, 2863,   39777,  7, }, // Pacific/Chatham
    {   71, 3022,   39784,  5, }, // Pacific/Chuuk
    {   71, 2201,   39789,  5, }, // Pacific/Easter
    {   71, 7323,   39794,  6, }, // Pacific/Efate
    {   71, 7290,   39800, 10, }, // Pacific/Fakaofo
    {   71, 3804,   39810,  5, }, // Pacific/Fiji
    {   71, 7306,   39815,  8, }, // Pacific/Funafuti
    {   71, 6641,   39823, 11, }, // Pacific/Galapagos
    {   71, 6659,   39834,  7, }, // Pacific/Gambier
    {   71, 3724,   39841, 10, }, // Pacific/Guadalcanal
    {   71, 5935,   39851,  5, }, // Pacific/Guam
    {   71, 2929,   39856, 10, }, // Pacific/Honolulu
    {   71, 2897,   39866,  9, }, // Pacific/Kanton
    {   71, 3900,   39875,  8, }, // Pacific/Kiritimati
    {   71, 6982,   39883, 10, }, // Pacific/Kosrae
    {   71, 2708,   39893, 10, }, // Pacific/Kwajalein
    {   71, 4779,   39903,  9, }, // Pacific/Majuro
    {   71, 3953,   39912, 10, }, // Pacific/Marquesas
    {   71, 8909,   39922,  8, }, // Pacific/Midway
    {   71, 7047,   39930,  6, }, // Pacific/Nauru
    {   71, 7095,   39936,  6, }, // Pacific/Niue
    {   71, 4070,   39942, 10, }, // Pacific/Norfolk
    {   71, 7061,   39952,  7, }, // Pacific/Noumea
    {   71, 2991,   39959, 10, }, // Pacific/Pago_Pago
    {   71, 7118,   39969,  6, }, // Pacific/Palau
    {   71, 7157,   39975, 10, }, // Pacific/Pitcairn
    {   71, 2961,   39985,  9, }, // Pacific/Pohnpei
    {   71, 4709,   39994, 17, }, // Pacific/Port_Moresby
    {   71, 6009,   40011, 12, }, // Pacific/Rarotonga
    {   71, 5948,   40023,  8, }, // Pacific/Saipan
    {   71, 7275,   40031,  5, }, // Pacific/Tahiti
    {   71, 6675,   40036,  6, }, // Pacific/Tarawa
    {   71, 4485,   40042, 10, }, // Pacific/Tongatapu
    {   71, 7355,   40052,  6, }, // Pacific/Wake
    {   71, 7368,   40058,  7, }, // Pacific/Wallis
    {   72, 6751,   40065,  3, }, // Africa/Abidjan Cantonese/Traditional Han/Hong Kong
    {   72, 6723,   40068,  3, }, // Africa/Accra
    {   72, 4961,   40071,  6, }, // Africa/Addis_Ababa
    {   72, 7383,   40077,  4, }, // Africa/Algiers
    {   72,   14,   40081,  4, }, // Africa/Asmara
    {   72,   44,   40085,  3, }, // Africa/Bamako
    {   72, 5159,   40088,  2, }, // Africa/Bangui
    {   72, 6709,   40090,  2, }, // Africa/Banjul
    {   72, 7398,   40092,  2, }, // Africa/Bissau
    {   72, 4859,   40094,  4, }, // Africa/Blantyre
    {   72, 5189,   40098,  5, }, // Africa/Brazzaville
    {   72, 4824,   40103,  4, }, // Africa/Bujumbura
    {   72, 2242,   40107,  2, }, // Africa/Cairo
    {   72, 4019,   40109,  5, }, // Africa/Casablanca
    {   72, 7412,   40114,  2, }, // Africa/Ceuta
    {   72, 6736,   40116,  4, }, // Africa/Conakry
    {   72, 6803,   40120,  3, }, // Africa/Dakar
    {   72, 5032,   40123,  3, }, // Africa/Dar_es_Salaam
    {   72, 4945,   40126,  3, }, // Africa/Djibouti
    {   72, 5145,   40129,  3, }, // Africa/Douala
    {   72, 5068,   40132,  3, }, // Africa/El_Aaiun
    {   72, 6816,   40135,  3, }, // Africa/Freetown
    {   72, 4808,   40138,  4, }, // Africa/Gaborone
    {   72, 4917,   40142,  3, }, // Africa/Harare
    {   72, 4381,   40145,  5, }, // Africa/Johannesburg
    {   72, 4401,   40150,  2, }, // Africa/Juba
    {   72, 5053,   40152,  3, }, // Africa/Kampala
    {   72, 4426,   40155,  3, }, // Africa/Khartoum
    {   72, 4875,   40158,  3, }, // Africa/Kigali
    {   72, 5208,   40161,  3, }, // Africa/Kinshasa
    {   72, 4646,   40164,  3, }, // Africa/Lagos
    {   72, 5238,   40167,  3, }, // Africa/Libreville
    {   72, 6832,   40170,  2, }, // Africa/Lome
    {   72, 5113,   40172,  3, }, // Africa/Luanda
    {   72, 4841,   40175,  4, }, // Africa/Lubumbashi
    {   72, 4903,   40179,  3, }, // Africa/Lusaka
    {   72, 5224,   40182,  3, }, // Africa/Malabo
    {   72, 4889,   40185,  3, }, // Africa/Maputo
    {   72, 5099,   40188,  3, }, // Africa/Maseru
    {   72, 5084,   40191,  3, }, // Africa/Mbabane
    {   72, 5015,   40194,  4, }, // Africa/Mogadishu
    {   72, 6997,   40198,  4, }, // Africa/Monrovia
    {   72, 3755,   40202,  3, }, // Africa/Nairobi
    {   72, 5173,   40205,  4, }, // Africa/Ndjamena
    {   72, 5256,   40209,  3, }, // Africa/Niamey
    {   72, 6766,   40212,  3, }, // Africa/Nouakchott
    {   72, 6690,   40215,  4, }, // Africa/Ouagadougou
    {   72, 5127,   40219,  4, }, // Africa/Porto-Novo
    {   72, 4337,   40223,  3, }, // Africa/Sao_Tome
    {   72, 2732,   40226,  4, }, // Africa/Tripoli
    {   72, 6503,   40230,  3, }, // Africa/Tunis
    {   72, 4054,   40233,  4, }, // Africa/Windhoek
    {   72,  132,   40237,  3, }, // America/Adak
    {   72, 3183,   40240,  4, }, // America/Anchorage
    {   72, 5524,   40244,  3, }, // America/Anguilla
    {   72, 5541,   40247,  3, }, // America/Antigua
    {   72, 4456,   40250,  4, }, // America/Araguaina
    {   72,  166,   40254,  7, }, // America/Argentina/Buenos_Aires
    {   72,   91,   40261,  4, }, // America/Argentina/Catamarca
    {   72,  270,   40265,  3, }, // America/Argentina/Cordoba
    {   72,  441,   40268,  3, }, // America/Argentina/Jujuy
    {   72, 7425,   40271,  3, }, // America/Argentina/La_Rioja
    {   72,  565,   40274,  3, }, // America/Argentina/Mendoza
    {   72, 7452,   40277,  6, }, // America/Argentina/Rio_Gallegos
    {   72, 7483,   40283,  3, }, // America/Argentina/Salta
    {   72, 7507,   40286,  3, }, // America/Argentina/San_Juan
    {   72, 5497,   40289,  3, }, // America/Argentina/San_Luis
    {   72, 7534,   40292,  3, }, // America/Argentina/Tucuman
    {   72, 7560,   40295,  4, }, // America/Argentina/Ushuaia
    {   72, 5557,   40299,  3, }, // America/Aruba
    {   72, 4154,   40302,  3, }, // America/Asuncion
    {   72,  237,   40305,  4, }, // America/Atikokan
    {   72, 3581,   40309,  3, }, // America/Bahia
    {   72, 7586,   40312,  7, }, // America/Bahia_Banderas
    {   72, 5571,   40319,  3, }, // America/Barbados
    {   72, 7609,   40322,  2, }, // America/Belem
    {   72, 5309,   40324,  3, }, // America/Belize
    {   72, 7623,   40327,  5, }, // America/Blanc-Sablon
    {   72, 7644,   40332,  4, }, // America/Boa_Vista
    {   72, 4263,   40336,  3, }, // America/Bogota
    {   72, 7662,   40339,  2, }, // America/Boise
    {   72, 7676,   40341,  3, }, // America/Cambridge_Bay
    {   72, 7698,   40344,  3, }, // America/Campo_Grande
    {   72, 3770,   40347,  2, }, // America/Cancun
    {   72, 4596,   40349,  4, }, // America/Caracas
    {   72, 4247,   40353,  2, }, // America/Cayenne
    {   72, 5398,   40355,  4, }, // America/Cayman
    {   72, 3251,   40359,  3, }, // America/Chicago
    {   72, 7719,   40362,  3, }, // America/Chihuahua
    {   72, 7737,   40365,  4, }, // America/Ciudad_Juarez
    {   72, 5324,   40369,  5, }, // America/Costa_Rica
    {   72, 7759,   40374,  4, }, // America/Creston
    {   72, 3693,   40378,  3, }, // America/Cuiaba
    {   72, 5640,   40381,  3, }, // America/Curacao
    {   72, 7775,   40384,  4, }, // America/Danmarkshavn
    {   72, 7796,   40388,  2, }, // America/Dawson
    {   72, 7811,   40390,  5, }, // America/Dawson_Creek
    {   72,  805,   40395,  2, }, // America/Denver
    {   72, 3351,   40397,  3, }, // America/Detroit
    {   72, 5656,   40400,  4, }, // America/Dominica
    {   72,  893,   40404,  4, }, // America/Edmonton
    {   72, 7832,   40408,  4, }, // America/Eirunepe
    {   72, 5343,   40412,  4, }, // America/El_Salvador
    {   72, 7849,   40416,  4, }, // America/Fort_Nelson
    {   72, 7869,   40420,  4, }, // America/Fortaleza
    {   72, 7887,   40424,  4, }, // America/Glace_Bay
    {   72, 6844,   40428,  2, }, // America/Goose_Bay
    {   72, 4514,   40430,  4, }, // America/Grand_Turk
    {   72, 5687,   40434,  4, }, // America/Grenada
    {   72, 5703,   40438,  4, }, // America/Guadeloupe
    {   72, 3662,   40442,  4, }, // America/Guatemala
    {   72, 4743,   40446,  4, }, // America/Guayaquil
    {   72, 6895,   40450,  3, }, // America/Guyana
    {   72, 1923,   40453,  5, }, // America/Halifax
    {   72, 2221,   40458,  3, }, // America/Havana
    {   72, 7905,   40461,  4, }, // America/Hermosillo
    {   72,  348,   40465,  7, }, // America/Indiana/Indianapolis
    {   72,  481,   40472,  8, }, // America/Indiana/Knox
    {   72, 7924,   40480,  8, }, // America/Indiana/Marengo
    {   72, 7948,   40488,  8, }, // America/Indiana/Petersburg
    {   72, 7975,   40496,  8, }, // America/Indiana/Tell_City
    {   72, 8001,   40504,  7, }, // America/Indiana/Vevay
    {   72, 8023,   40511,  8, }, // America/Indiana/Vincennes
    {   72, 8049,   40519,  9, }, // America/Indiana/Winamac
    {   72, 8073,   40528,  4, }, // America/Inuvik
    {   72,  660,   40532,  3, }, // America/Iqaluit
    {   72, 2665,   40535,  3, }, // America/Jamaica
    {   72, 5282,   40538,  2, }, // America/Juneau
    {   72,  521,   40540,  5, }, // America/Kentucky/Louisville
    {   72, 8088,   40545,  8, }, // America/Kentucky/Monticello
    {   72, 5621,   40553,  5, }, // America/Kralendijk
    {   72, 4278,   40558,  3, }, // America/La_Paz
    {   72, 7132,   40561,  2, }, // America/Lima
    {   72, 3390,   40563,  3, }, // America/Los_Angeles
    {   72, 5849,   40566,  4, }, // America/Lower_Princes
    {   72, 8116,   40570,  3, }, // America/Maceio
    {   72, 8131,   40573,  3, }, // America/Managua
    {   72, 1892,   40576,  3, }, // America/Manaus
    {   72, 5814,   40579,  4, }, // America/Marigot
    {   72, 5722,   40583,  4, }, // America/Martinique
    {   72, 8147,   40587,  5, }, // America/Matamoros
    {   72, 2779,   40592,  4, }, // America/Mazatlan
    {   72, 8165,   40596,  4, }, // America/Menominee
    {   72, 8183,   40600,  3, }, // America/Merida
    {   72, 8198,   40603,  6, }, // America/Metlakatla
    {   72, 2811,   40609,  4, }, // America/Mexico_City
    {   72, 4293,   40613,  3, }, // America/Miquelon
    {   72, 8217,   40616,  3, }, // America/Moncton
    {   72, 8233,   40619,  3, }, // America/Monterrey
    {   72, 4000,   40622,  4, }, // America/Montevideo
    {   72, 5741,   40626,  3, }, // America/Montserrat
    {   72, 5383,   40629,  2, }, // America/Nassau
    {   72, 3294,   40631,  2, }, // America/New_York
    {   72, 8251,   40633,  2, }, // America/Nome
    {   72, 1834,   40635,  3, }, // America/Noronha
    {   72, 8264,   40638,  8, }, // America/North_Dakota/Beulah
    {   72, 8292,   40646,  7, }, // America/North_Dakota/Center
    {   72, 8320,   40653,  8, }, // America/North_Dakota/New_Salem
    {   72,  393,   40661,  2, }, // America/Nuuk
    {   72, 8351,   40663,  4, }, // America/Ojinaga
    {   72, 5413,   40667,  3, }, // America/Panama
    {   72, 6106,   40670,  5, }, // America/Paramaribo
    {   72, 3224,   40675,  3, }, // America/Phoenix
    {   72, 3847,   40678,  3, }, // America/Port-au-Prince
    {   72, 5871,   40681,  4, }, // America/Port_of_Spain
    {   72, 8367,   40685,  3, }, // America/Porto_Velho
    {   72, 5760,   40688,  4, }, // America/Puerto_Rico
    {   72, 3932,   40692,  6, }, // America/Punta_Arenas
    {   72, 8387,   40698,  3, }, // America/Rankin_Inlet
    {   72, 8408,   40701,  3, }, // America/Recife
    {   72, 1979,   40704,  3, }, // America/Regina
    {   72, 8423,   40707,  4, }, // America/Resolute
    {   72,  695,   40711,  4, }, // America/Rio_Branco
    {   72, 8440,   40715,  3, }, // America/Santarem
    {   72, 2165,   40718,  4, }, // America/Santiago
    {   72, 6044,   40722,  4, }, // America/Santo_Domingo
    {   72, 1862,   40726,  3, }, // America/Sao_Paulo
    {   72, 6862,   40729,  7, }, // America/Scoresbysund
    {   72, 8457,   40736,  3, }, // America/Sitka
    {   72, 8471,   40739,  6, }, // America/St_Barthelemy
    {   72, 2045,   40745,  3, }, // America/St_Johns
    {   72, 5780,   40748,  3, }, // America/St_Kitts
    {   72, 5797,   40751,  4, }, // America/St_Lucia
    {   72,  855,   40755,  4, }, // America/St_Thomas
    {   72, 5830,   40759,  3, }, // America/St_Vincent
    {   72, 8493,   40762,  7, }, // America/Swift_Current
    {   72, 5363,   40769,  5, }, // America/Tegucigalpa
    {   72, 5673,   40774,  2, }, // America/Thule
    {   72,  313,   40776,  3, }, // America/Tijuana
    {   72,  608,   40779,  3, }, // America/Toronto
    {   72, 5605,   40782,  4, }, // America/Tortola
    {   72, 2077,   40786,  3, }, // America/Vancouver
    {   72, 2128,   40789,  4, }, // America/Whitehorse
    {   72,  734,   40793,  3, }, // America/Winnipeg
    {   72, 8515,   40796,  3, }, // America/Yakutat
    {   72, 5918,   40799,  2, }, // Antarctica/Casey
    {   72, 6027,   40801,  3, }, // Antarctica/Davis
    {   72, 6066,   40804,  5, }, // Antarctica/DumontDUrville
    {   72, 8531,   40809,  3, }, // Antarctica/Macquarie
    {   72, 7029,   40812,  2, }, // Antarctica/Mawson
    {   72, 7076,   40814,  4, }, // Antarctica/McMurdo
    {   72, 8552,   40818,  2, }, // Antarctica/Palmer
    {   72, 7189,   40820,  3, }, // Antarctica/Rothera
    {   72, 7258,   40823,  4, }, // Antarctica/Syowa
    {   72, 8570,   40827,  2, }, // Antarctica/Troll
    {   72, 7337,   40829,  4, }, // Antarctica/Vostok
    {   72, 1413,   40833,  5, }, // Arctic/Longyearbyen
    {   72, 5487,   40838,  2, }, // Asia/Aden
    {   72, 5297,   40840,  4, }, // Asia/Almaty
    {   72, 3870,   40844,  2, }, // Asia/Amman
    {   72, 5428,   40846,  3, }, // Asia/Anadyr
    {   72, 5440,   40849,  3, }, // Asia/Aqtau
    {   72, 5270,   40852,  4, }, // Asia/Aqtobe
    {   72,  964,   40856,  5, }, // Asia/Ashgabat
    {   72, 8587,   40861,  3, }, // Asia/Atyrau
    {   72, 3509,   40864,  3, }, // Asia/Baghdad
    {   72, 5451,   40867,  2, }, // Asia/Bahrain
    {   72, 3555,   40380,  2, }, // Asia/Baku
    {   72, 4368,   40869,  2, }, // Asia/Bangkok
    {   72, 3473,   40871,  4, }, // Asia/Barnaul
    {   72, 3988,   40875,  3, }, // Asia/Beirut
    {   72, 3680,   40878,  4, }, // Asia/Bishkek
    {   72, 5906,   40882,  2, }, // Asia/Brunei
    {   72, 4503,   40884,  2, }, // Asia/Chita
    {   72, 5963,   40886,  3, }, // Asia/Choibalsan
    {   72, 4413,   40889,  3, }, // Asia/Colombo
    {   72, 4442,   40892,  4, }, // Asia/Damascus
    {   72, 1060,   40896,  2, }, // Asia/Dhaka
    {   72, 6125,   40898,  2, }, // Asia/Dili
    {   72, 3498,   40900,  2, }, // Asia/Dubai
    {   72, 6092,   40902,  3, }, // Asia/Dushanbe
    {   72, 8599,   40905,  5, }, // Asia/Famagusta
    {   72, 8614,   40910,  2, }, // Asia/Gaza
    {   72, 4697,   40912,  3, }, // Asia/Hebron
    {   72, 1226,   40915,  4, }, // Asia/Ho_Chi_Minh
    {   72, 2576,   40919, 14, }, // Asia/Hong_Kong
    {   72, 4673,   40933,  3, }, // Asia/Hovd
    {   72, 4086,   40936,  5, }, // Asia/Irkutsk
    {   72, 6969,   40941,  3, }, // Asia/Jakarta
    {   72, 6955,   40944,  4, }, // Asia/Jayapura
    {   72, 1257,   40948,  4, }, // Asia/Jerusalem
    {   72, 3462,   40952,  3, }, // Asia/Kabul
    {   72, 4218,   40955,  3, }, // Asia/Kamchatka
    {   72, 4141,   40958,  3, }, // Asia/Karachi
    {   72, 1152,   40961,  4, }, // Asia/Kathmandu
    {   72, 8624,   40965,  3, }, // Asia/Khandyga
    {   72,  992,   40968,  4, }, // Asia/Kolkata
    {   72, 4099,   40972,  8, }, // Asia/Krasnoyarsk
    {   72, 4761,   40980,  3, }, // Asia/Kuala_Lumpur
    {   72, 5893,   40983,  2, }, // Asia/Kuching
    {   72, 5464,   40985,  3, }, // Asia/Kuwait
    {   72, 1178,   40988, 14, }, // Asia/Macau
    {   72, 3919,   41002,  3, }, // Asia/Magadan
    {   72, 1316,   41005,  4, }, // Asia/Makassar
    {   72, 7145,   41009,  3, }, // Asia/Manila
    {   72, 6883,   41012,  4, }, // Asia/Muscat
    {   72, 2446,   41016,  4, }, // Asia/Nicosia
    {   72, 8638,   41020,  6, }, // Asia/Novokuznetsk
    {   72, 4037,   41026,  5, }, // Asia/Novosibirsk
    {   72, 4131,   41031,  4, }, // Asia/Omsk
    {   72, 7108,   41035,  3, }, // Asia/Oral
    {   72, 6924,   41038,  2, }, // Asia/Phnom_Penh
    {   72, 8656,   41040,  2, }, // Asia/Pontianak
    {   72, 4116,   41042,  2, }, // Asia/Pyongyang
    {   72, 5476,   41044,  2, }, // Asia/Qatar
    {   72, 8671,   41046,  4, }, // Asia/Qostanay
    {   72, 4171,   41050,  6, }, // Asia/Qyzylorda
    {   72, 3486,   41056,  3, }, // Asia/Riyadh
    {   72, 4310,   41059,  3, }, // Asia/Sakhalin
    {   72, 7208,   41062,  4, }, // Asia/Samarkand
    {   72, 3116,   41066,  2, }, // Asia/Seoul
    {   72, 1020,   41068,  2, }, // Asia/Shanghai
    {   72, 3137,   41070,  3, }, // Asia/Singapore
    {   72, 4199,   41073,  6, }, // Asia/Srednekolymsk
    {   72, 3100,   41079,  2, }, // Asia/Taipei
    {   72, 4683,   41081,  3, }, // Asia/Tashkent
    {   72, 3817,   41084,  4, }, // Asia/Tbilisi
    {   72, 2638,   41088,  3, }, // Asia/Tehran
    {   72, 1284,   41091,  2, }, // Asia/Thimphu
    {   72, 2687,   41093,  2, }, // Asia/Tokyo
    {   72, 4474,   41095,  4, }, // Asia/Tomsk
    {   72, 1346,   41099,  4, }, // Asia/Ulaanbaatar
    {   72, 1126,   41103,  4, }, // Asia/Urumqi
    {   72, 8685,   41107,  4, }, // Asia/Ust-Nera
    {   72, 6940,   41111,  2, }, // Asia/Vientiane
    {   72, 4612,   41113,  3, }, // Asia/Vladivostok
    {   72, 4730,   41116,  4, }, // Asia/Yakutsk
    {   72, 1202,   41120,  2, }, // Asia/Yangon
    {   72, 3785,   41122,  5, }, // Asia/Yekaterinburg
    {   72, 3649,   41127,  3, }, // Asia/Yerevan
    {   72, 3565,   41130,  5, }, // Atlantic/Azores
    {   72, 5588,   41135,  3, }, // Atlantic/Bermuda
    {   72, 6591,   41138,  3, }, // Atlantic/Canary
    {   72, 3629,   41141,  3, }, // Atlantic/Cape_Verde
    {   72, 1379,   41144,  4, }, // Atlantic/Faroe
    {   72, 8699,   41148,  5, }, // Atlantic/Madeira
    {   72, 2614,   41153,  5, }, // Atlantic/Reykjavik
    {   72, 7235,   41158,  4, }, // Atlantic/South_Georgia
    {   72, 6784,   41162,  5, }, // Atlantic/St_Helena
    {   72, 6607,   41167,  3, }, // Atlantic/Stanley
    {   72, 1654,   41170,  4, }, // Australia/Adelaide
    {   72, 1619,   41174,  4, }, // Australia/Brisbane
    {   72, 1783,   41178,  5, }, // Australia/Broken_Hill
    {   72, 1567,   41183,  3, }, // Australia/Darwin
    {   72, 3539,   41186,  3, }, // Australia/Eucla
    {   72, 1500,   41189,  3, }, // Australia/Hobart
    {   72, 8716,   41192,  3, }, // Australia/Lindeman
    {   72, 1531,   41195,  4, }, // Australia/Lord_Howe
    {   72, 1711,   41199,  3, }, // Australia/Melbourne
    {   72, 1746,   41202,  2, }, // Australia/Perth
    {   72, 1447,   41204,  2, }, // Australia/Sydney
    {   72, 6390,   41206,  5, }, // Europe/Amsterdam
    {   72, 6149,   41211,  3, }, // Europe/Andorra
    {   72, 3522,   41214,  5, }, // Europe/Astrakhan
    {   72, 6577,   41219,  2, }, // Europe/Athens
    {   72, 6285,   41221,  4, }, // Europe/Belgrade
    {   72, 4659,   41225,  2, }, // Europe/Berlin
    {   72, 6437,   41227,  6, }, // Europe/Bratislava
    {   72, 6178,   41233,  4, }, // Europe/Brussels
    {   72, 3830,   41237,  5, }, // Europe/Bucharest
    {   72, 3708,   41242,  4, }, // Europe/Budapest
    {   72, 8751,   41246,  3, }, // Europe/Busingen
    {   72, 2475,   41249,  3, }, // Europe/Chisinau
    {   72, 6238,   41252,  4, }, // Europe/Copenhagen
    {   72, 2260,   41256,  3, }, // Europe/Dublin
    {   72, 6256,   41259,  4, }, // Europe/Gibraltar
    {   72, 8767,   41263,  3, }, // Europe/Guernsey
    {   72, 6561,   41266,  4, }, // Europe/Helsinki
    {   72, 8783,   41270,  2, }, // Europe/Isle_of_Man
    {   72, 1097,   41272,  4, }, // Europe/Istanbul
    {   72, 8802,   41276,  3, }, // Europe/Jersey
    {   72, 3881,   41279,  5, }, // Europe/Kaliningrad
    {   72, 8924,   41284,  3, }, // Europe/Kirov
    {   72, 2419,   41287,  2, }, // Europe/Kyiv
    {   72, 3078,   41289,  3, }, // Europe/Lisbon
    {   72, 6455,   41292,  4, }, // Europe/Ljubljana
    {   72, 2393,   41296,  2, }, // Europe/London
    {   72, 6314,   41298,  3, }, // Europe/Luxembourg
    {   72, 4794,   41301,  3, }, // Europe/Madrid
    {   72, 6346,   41304,  3, }, // Europe/Malta
    {   72, 6531,   41307,  3, }, // Europe/Mariehamn
    {   72, 3595,   41310,  3, }, // Europe/Minsk
    {   72, 6359,   41313,  3, }, // Europe/Monaco
    {   72, 3443,   41316,  3, }, // Europe/Moscow
    {   72, 6407,   41319,  3, }, // Europe/Oslo
    {   72, 4186,   41322,  2, }, // Europe/Paris
    {   72, 6373,   41324,  4, }, // Europe/Podgorica
    {   72, 6224,   41328,  3, }, // Europe/Prague
    {   72, 8816,   41331,  2, }, // Europe/Riga
    {   72, 6273,   41333,  2, }, // Europe/Rome
    {   72, 4233,   41335,  3, }, // Europe/Samara
    {   72, 6419,   41338,  4, }, // Europe/San_Marino
    {   72, 6194,   41342,  4, }, // Europe/Sarajevo
    {   72, 4353,   41346,  4, }, // Europe/Saratov
    {   72, 8828,   41350,  4, }, // Europe/Simferopol
    {   72, 6332,   41354,  4, }, // Europe/Skopje
    {   72, 6548,   41358,  3, }, // Europe/Sofia
    {   72, 6472,   41361,  5, }, // Europe/Stockholm
    {   72, 8846,   41366,  2, }, // Europe/Tallinn
    {   72, 6135,   41368,  3, }, // Europe/Tirane
    {   72, 8861,   41371,  7, }, // Europe/Ulyanovsk
    {   72, 6301,   41378,  3, }, // Europe/Vaduz
    {   72, 6516,   41381,  3, }, // Europe/Vatican
    {   72, 6164,   41384,  3, }, // Europe/Vienna
    {   72, 8878,   41387,  4, }, // Europe/Vilnius
    {   72, 4629,   41391,  5, }, // Europe/Volgograd
    {   72, 3055,   41396,  2, }, // Europe/Warsaw
    {   72, 6210,   41398,  4, }, // Europe/Zagreb
    {   72, 6489,   41402,  3, }, // Europe/Zurich
    {   72, 4980,   41405,  6, }, // Indian/Antananarivo
    {   72, 6910,   41411,  3, }, // Indian/Chagos
    {   72, 5979,   41414,  3, }, // Indian/Christmas
    {   72, 5996,   41417,  5, }, // Indian/Cocos
    {   72, 4931,   41422,  5, }, // Indian/Comoro
    {   72, 6624,   41427,  5, }, // Indian/Kerguelen
    {   72, 7223,   41432,  3, }, // Indian/Mahe
    {   72, 7013,   41435,  4, }, // Indian/Maldives
    {   72, 3971,   41439,  4, }, // Indian/Mauritius
    {   72, 5000,   41443,  4, }, // Indian/Mayotte
    {   72, 7174,   41447,  4, }, // Indian/Reunion
    {   72, 4324,   41451,  3, }, // Pacific/Apia
    {   72,  932,   41454,  3, }, // Pacific/Auckland
    {   72, 3608,   41457,  4, }, // Pacific/Bougainville
    {   72, 2863,   41461,  2, }, // Pacific/Chatham
    {   72, 3022,   41463,  2, }, // Pacific/Chuuk
    {   72, 2201,   41465,  3, }, // Pacific/Easter
    {   72, 7323,   41468,  3, }, // Pacific/Efate
    {   72, 7290,   41471,  3, }, // Pacific/Fakaofo
    {   72, 3804,   41474,  2, }, // Pacific/Fiji
    {   72, 7306,   41476,  4, }, // Pacific/Funafuti
    {   72, 6641,   41480,  6, }, // Pacific/Galapagos
    {   72, 6659,   41486,  5, }, // Pacific/Gambier
    {   72, 3724,   41491,  6, }, // Pacific/Guadalcanal
    {   72, 5935,   41497,  2, }, // Pacific/Guam
    {   72, 2929,   41499,  3, }, // Pacific/Honolulu
    {   72, 2897,   41502,  5, }, // Pacific/Kanton
    {   72, 3900,   41507,  6, }, // Pacific/Kiritimati
    {   72, 6982,   41513,  3, }, // Pacific/Kosrae
    {   72, 2708,   41516,  4, }, // Pacific/Kwajalein
    {   72, 4779,   41520,  3, }, // Pacific/Majuro
    {   72, 3953,   41523,  5, }, // Pacific/Marquesas
    {   72, 8909,   41528,  3, }, // Pacific/Midway
    {   72, 7047,   41531,  2, }, // Pacific/Nauru
    {   72, 7095,   41533,  3, }, // Pacific/Niue
    {   72, 4070,   41536,  3, }, // Pacific/Norfolk
    {   72, 7061,   41539,  3, }, // Pacific/Noumea
    {   72, 2991,   41542,  4, }, // Pacific/Pago_Pago
    {   72, 7118,   41546,  2, }, // Pacific/Palau
    {   72, 7157,   41548,  5, }, // Pacific/Pitcairn
    {   72, 2961,   41553,  3, }, // Pacific/Pohnpei
    {   72, 4709,   41556,  4, }, // Pacific/Port_Moresby
    {   72, 6009,   41560,  4, }, // Pacific/Rarotonga
    {   72, 5948,   41564,  2, }, // Pacific/Saipan
    {   72, 7275,   41566,  3, }, // Pacific/Tahiti
    {   72, 6675,   41569,  3, }, // Pacific/Tarawa
    {   72, 4485,   41572,  5, }, // Pacific/Tongatapu
    {   72, 7355,   41577,  2, }, // Pacific/Wake
    {   72, 7368,   41579,  3, }, // Pacific/Wallis
    {   73, 6751,   41582,  3, }, // Africa/Abidjan Cantonese/Simplified Han/China
    {   73, 4961,   41585,  6, }, // Africa/Addis_Ababa
    {   73, 7383,   41591,  4, }, // Africa/Algiers
    {   73,   14,   41595,  4, }, // Africa/Asmara
    {   73,   44,   41599,  3, }, // Africa/Bamako
    {   73, 7398,   41602,  2, }, // Africa/Bissau
    {   73, 4859,   41604,  4, }, // Africa/Blantyre
    {   73, 5189,   41608,  5, }, // Africa/Brazzaville
    {   73, 2242,   41613,  2, }, // Africa/Cairo
    {   73, 4019,   41615,  5, }, // Africa/Casablanca
    {   73, 7412,   41620,  2, }, // Africa/Ceuta
    {   73, 6803,   41622,  3, }, // Africa/Dakar
    {   73, 5032,   41625,  3, }, // Africa/Dar_es_Salaam
    {   73, 4381,   41628,  5, }, // Africa/Johannesburg
    {   73, 5113,   41633,  3, }, // Africa/Luanda
    {   73, 4841,   41636,  4, }, // Africa/Lubumbashi
    {   73, 5224,   41640,  3, }, // Africa/Malabo
    {   73, 4889,   41643,  3, }, // Africa/Maputo
    {   73, 5099,   41646,  3, }, // Africa/Maseru
    {   73, 6997,   41649,  4, }, // Africa/Monrovia
    {   73, 5173,   41653,  4, }, // Africa/Ndjamena
    {   73, 5256,   41657,  3, }, // Africa/Niamey
    {   73, 6766,   41660,  3, }, // Africa/Nouakchott
    {   73, 5127,   41663,  4, }, // Africa/Porto-Novo
    {   73, 4337,   41667,  3, }, // Africa/Sao_Tome
    {   73, 4054,   41670,  4, }, // Africa/Windhoek
    {   73,  132,   41674,  3, }, // America/Adak
    {   73,  166,   41677,  7, }, // America/Argentina/Buenos_Aires
    {   73,   91,   41684,  4, }, // America/Argentina/Catamarca
    {   73,  270,   41688,  3, }, // America/Argentina/Cordoba
    {   73,  565,   41691,  3, }, // America/Argentina/Mendoza
    {   73, 7452,   41694,  6, }, // America/Argentina/Rio_Gallegos
    {   73, 7483,   41700,  3, }, // America/Argentina/Salta
    {   73, 7507,   41703,  3, }, // America/Argentina/San_Juan
    {   73, 5497,   41706,  3, }, // America/Argentina/San_Luis
    {   73, 7534,   41709,  3, }, // America/Argentina/Tucuman
    {   73, 7560,   41712,  4, }, // America/Argentina/Ushuaia
    {   73, 4154,   41716,  3, }, // America/Asuncion
    {   73, 7586,   41719,  7, }, // America/Bahia_Banderas
    {   73, 5571,   41726,  3, }, // America/Barbados
    {   73, 7609,   41729,  2, }, // America/Belem
    {   73, 5309,   41731,  3, }, // America/Belize
    {   73, 7623,   41734,  5, }, // America/Blanc-Sablon
    {   73, 7644,   41739,  4, }, // America/Boa_Vista
    {   73, 7676,   41743,  3, }, // America/Cambridge_Bay
    {   73, 7698,   41746,  3, }, // America/Campo_Grande
    {   73, 4247,   41749,  2, }, // America/Cayenne
    {   73, 5398,   41751,  4, }, // America/Cayman
    {   73, 7719,   41755,  3, }, // America/Chihuahua
    {   73, 7737,   41758,  4, }, // America/Ciudad_Juarez
    {   73, 7759,   41762,  4, }, // America/Creston
    {   73, 5640,   41766,  3, }, // America/Curacao
    {   73, 7775,   41769,  4, }, // America/Danmarkshavn
    {   73, 7832,   41773,  4, }, // America/Eirunepe
    {   73, 5343,   41777,  4, }, // America/El_Salvador
    {   73, 7849,   41781,  4, }, // America/Fort_Nelson
    {   73, 7887,   41785,  4, }, // America/Glace_Bay
    {   73, 6844,   41789,  2, }, // America/Goose_Bay
    {   73, 4514,   41791,  4, }, // America/Grand_Turk
    {   73, 5687,   41795,  4, }, // America/Grenada
    {   73, 3662,   41799,  4, }, // America/Guatemala
    {   73, 4743,   41803,  4, }, // America/Guayaquil
    {   73, 6895,   41807,  3, }, // America/Guyana
    {   73, 7905,   41810,  4, }, // America/Hermosillo
    {   73,  481,   41814,  8, }, // America/Indiana/Knox
    {   73, 7924,   41822,  8, }, // America/Indiana/Marengo
    {   73, 7975,   41830,  8, }, // America/Indiana/Tell_City
    {   73, 8001,   41838,  7, }, // America/Indiana/Vevay
    {   73, 8023,   41845,  8, }, // America/Indiana/Vincennes
    {   73, 8049,   41853,  9, }, // America/Indiana/Winamac
    {   73, 8073,   41862,  4, }, // America/Inuvik
    {   73, 2665,   41866,  3, }, // America/Jamaica
    {   73, 5282,   41869,  2, }, // America/Juneau
    {   73,  521,   41871,  5, }, // America/Kentucky/Louisville
    {   73, 8088,   41876,  8, }, // America/Kentucky/Monticello
    {   73, 5621,   41884,  5, }, // America/Kralendijk
    {   73, 7132,   41889,  2, }, // America/Lima
    {   73, 3390,   41891,  3, }, // America/Los_Angeles
    {   73, 5849,   41894,  4, }, // America/Lower_Princes
    {   73, 8116,   41898,  3, }, // America/Maceio
    {   73, 8131,   41901,  3, }, // America/Managua
    {   73, 1892,   41904,  3, }, // America/Manaus
    {   73, 5814,   41907,  4, }, // America/Marigot
    {   73, 5722,   41911,  4, }, // America/Martinique
    {   73, 8147,   41915,  5, }, // America/Matamoros
    {   73, 2779,   41920,  4, }, // America/Mazatlan
    {   73, 8165,   41924,  4, }, // America/Menominee
    {   73, 8183,   41928,  3, }, // America/Merida
    {   73, 4293,   41931,  3, }, // America/Miquelon
    {   73, 8217,   41934,  3, }, // America/Moncton
    {   73, 4000,   41937,  4, }, // America/Montevideo
    {   73, 5741,   41941,  3, }, // America/Montserrat
    {   73, 3294,   41944,  2, }, // America/New_York
    {   73, 8251,   41946,  2, }, // America/Nome
    {   73, 1834,   41948,  3, }, // America/Noronha
    {   73, 8264,   41951,  8, }, // America/North_Dakota/Beulah
    {   73, 8292,   41959,  7, }, // America/North_Dakota/Center
    {   73, 8320,   41966,  8, }, // America/North_Dakota/New_Salem
    {   73, 8351,   41974,  4, }, // America/Ojinaga
    {   73, 5413,   41978,  3, }, // America/Panama
    {   73, 6106,   41981,  5, }, // America/Paramaribo
    {   73, 3224,   41986,  3, }, // America/Phoenix
    {   73, 8367,   41989,  3, }, // America/Porto_Velho
    {   73, 3932,   41992,  6, }, // America/Punta_Arenas
    {   73, 8387,   41998,  3, }, // America/Rankin_Inlet
    {   73, 1979,   42001,  3, }, // America/Regina
    {   73, 8423,   42004,  4, }, // America/Resolute
    {   73,  695,   42008,  4, }, // America/Rio_Branco
    {   73, 8440,   42012,  3, }, // America/Santarem
    {   73, 2165,   42015,  4, }, // America/Santiago
    {   73, 6044,   42019,  4, }, // America/Santo_Domingo
    {   73, 1862,   42023,  3, }, // America/Sao_Paulo
    {   73, 6862,   42026,  7, }, // America/Scoresbysund
    {   73, 8457,   42033,  3, }, // America/Sitka
    {   73, 8471,   42036,  6, }, // America/St_Barthelemy
    {   73, 2045,   42042,  3, }, // America/St_Johns
    {   73, 5780,   42045,  3, }, // America/St_Kitts
    {   73, 5797,   42048,  4, }, // America/St_Lucia
    {   73,  855,   42052,  4, }, // America/St_Thomas
    {   73, 5830,   42056,  3, }, // America/St_Vincent
    {   73, 8493,   42059,  7, }, // America/Swift_Current
    {   73,  313,   42066,  3, }, // America/Tijuana
    {   73,  608,   42069,  3, }, // America/Toronto
    {   73, 5605,   42072,  4, }, // America/Tortola
    {   73, 2077,   42076,  3, }, // America/Vancouver
    {   73, 2128,   42079,  4, }, // America/Whitehorse
    {   73,  734,   42083,  3, }, // America/Winnipeg
    {   73, 8515,   42086,  3, }, // America/Yakutat
    {   73, 5918,   42089,  2, }, // Antarctica/Casey
    {   73, 6027,   42091,  3, }, // Antarctica/Davis
    {   73, 6066,   42094,  5, }, // Antarctica/DumontDUrville
    {   73, 8531,   42099,  3, }, // Antarctica/Macquarie
    {   73, 7076,   42102,  4, }, // Antarctica/McMurdo
    {   73, 8552,   42106,  2, }, // Antarctica/Palmer
    {   73, 7189,   42108,  3, }, // Antarctica/Rothera
    {   73, 8570,   42111,  2, }, // Antarctica/Troll
    {   73, 5487,   42113,  2, }, // Asia/Aden
    {   73, 5297,   42115,  4, }, // Asia/Almaty
    {   73, 8587,   42119,  3, }, // Asia/Atyrau
    {   73, 3509,   42122,  3, }, // Asia/Baghdad
    {   73, 3555,   42125,  2, }, // Asia/Baku
    {   73, 3473,   42127,  4, }, // Asia/Barnaul
    {   73, 3988,   42131,  3, }, // Asia/Beirut
    {   73, 3680,   42134,  4, }, // Asia/Bishkek
    {   73, 5906,   42138,  2, }, // Asia/Brunei
    {   73, 5963,   42140,  3, }, // Asia/Choibalsan
    {   73, 4413,   42143,  3, }, // Asia/Colombo
    {   73, 4442,   42146,  4, }, // Asia/Damascus
    {   73, 1060,   42150,  2, }, // Asia/Dhaka
    {   73, 6092,   42152,  3, }, // Asia/Dushanbe
    {   73, 8599,   42155,  5, }, // Asia/Famagusta
    {   73, 8614,   42160,  2, }, // Asia/Gaza
    {   73, 2576,   42162, 14, }, // Asia/Hong_Kong
    {   73, 4086,   42176,  5, }, // Asia/Irkutsk
    {   73, 6969,   42181,  3, }, // Asia/Jakarta
    {   73, 6955,   42184,  4, }, // Asia/Jayapura
    {   73, 3462,   42188,  3, }, // Asia/Kabul
    {   73, 1152,   42191,  4, }, // Asia/Kathmandu
    {   73,  992,   42195,  4, }, // Asia/Kolkata
    {   73, 4099,   42199,  8, }, // Asia/Krasnoyarsk
    {   73, 5893,   42207,  2, }, // Asia/Kuching
    {   73, 1178,   42209, 14, }, // Asia/Macau
    {   73, 3919,   42223,  3, }, // Asia/Magadan
    {   73, 1316,   42226,  4, }, // Asia/Makassar
    {   73, 7145,   42230,  3, }, // Asia/Manila
    {   73, 6883,   42233,  4, }, // Asia/Muscat
    {   73, 2446,   42237,  4, }, // Asia/Nicosia
    {   73, 8638,   42241,  6, }, // Asia/Novokuznetsk
    {   73, 4037,   42247,  5, }, // Asia/Novosibirsk
    {   73, 7108,   42252,  3, }, // Asia/Oral
    {   73, 6924,   42255,  2, }, // Asia/Phnom_Penh
    {   73, 5476,   42257,  2, }, // Asia/Qatar
    {   73, 4171,   42259,  6, }, // Asia/Qyzylorda
    {   73, 4310,   42265,  3, }, // Asia/Sakhalin
    {   73, 7208,   42268,  4, }, // Asia/Samarkand
    {   73, 3116,   42272,  2, }, // Asia/Seoul
    {   73, 2638,   42274,  3, }, // Asia/Tehran
    {   73, 2687,   42277,  2, }, // Asia/Tokyo
    {   73, 1346,   42279,  4, }, // Asia/Ulaanbaatar
    {   73, 1126,   42283,  4, }, // Asia/Urumqi
    {   73, 8685,   42287,  4, }, // Asia/Ust-Nera
    {   73, 4612,   42291,  3, }, // Asia/Vladivostok
    {   73, 4730,   42294,  4, }, // Asia/Yakutsk
    {   73, 3785,   42298,  5, }, // Asia/Yekaterinburg
    {   73, 3649,   42303,  3, }, // Asia/Yerevan
    {   73, 3565,   42306,  5, }, // Atlantic/Azores
    {   73, 5588,   42311,  3, }, // Atlantic/Bermuda
    {   73, 6591,   42314,  3, }, // Atlantic/Canary
    {   73, 3629,   42317,  3, }, // Atlantic/Cape_Verde
    {   73, 1379,   42320,  4, }, // Atlantic/Faroe
    {   73, 8699,   42324,  5, }, // Atlantic/Madeira
    {   73, 2614,   42329,  5, }, // Atlantic/Reykjavik
    {   73, 7235,   42334,  4, }, // Atlantic/South_Georgia
    {   73, 6784,   42338,  5, }, // Atlantic/St_Helena
    {   73, 1783,   42343,  5, }, // Australia/Broken_Hill
    {   73, 1567,   42348,  3, }, // Australia/Darwin
    {   73, 1531,   42351,  4, }, // Australia/Lord_Howe
    {   73, 1711,   42355,  3, }, // Australia/Melbourne
    {   73, 6149,   42358,  3, }, // Europe/Andorra
    {   73, 6285,   42361,  4, }, // Europe/Belgrade
    {   73, 6178,   42365,  4, }, // Europe/Brussels
    {   73, 3708,   42369,  4, }, // Europe/Budapest
    {   73, 2260,   41256,  3, }, // Europe/Dublin
    {   73, 6256,   42373,  4, }, // Europe/Gibraltar
    {   73, 8767,   42377,  3, }, // Europe/Guernsey
    {   73, 6561,   42380,  4, }, // Europe/Helsinki
    {   73, 8783,   42384,  2, }, // Europe/Isle_of_Man
    {   73, 8802,   42386,  3, }, // Europe/Jersey
    {   73, 3881,   42389,  5, }, // Europe/Kaliningrad
    {   73, 2419,   42394,  2, }, // Europe/Kyiv
    {   73, 6455,   42396,  4, }, // Europe/Ljubljana
    {   73, 2393,   42400,  2, }, // Europe/London
    {   73, 6314,   42402,  3, }, // Europe/Luxembourg
    {   73, 4794,   42405,  3, }, // Europe/Madrid
    {   73, 6346,   42408,  3, }, // Europe/Malta
    {   73, 6531,   42411,  3, }, // Europe/Mariehamn
    {   73, 6359,   42414,  3, }, // Europe/Monaco
    {   73, 6407,   42417,  3, }, // Europe/Oslo
    {   73, 6273,   42420,  2, }, // Europe/Rome
    {   73, 4233,   42422,  3, }, // Europe/Samara
    {   73, 6419,   42425,  4, }, // Europe/San_Marino
    {   73, 4353,   42429,  4, }, // Europe/Saratov
    {   73, 6548,   42433,  3, }, // Europe/Sofia
    {   73, 6472,   42436,  5, }, // Europe/Stockholm
    {   73, 8861,   42441,  7, }, // Europe/Ulyanovsk
    {   73, 6301,   42448,  3, }, // Europe/Vaduz
    {   73, 6516,   42451,  3, }, // Europe/Vatican
    {   73, 6164,   42454,  3, }, // Europe/Vienna
    {   73, 8878,   42457,  4, }, // Europe/Vilnius
    {   73, 4629,   42461,  5, }, // Europe/Volgograd
    {   73, 3055,   42466,  2, }, // Europe/Warsaw
    {   73, 6489,   42468,  3, }, // Europe/Zurich
    {   73, 5979,   42471,  3, }, // Indian/Christmas
    {   73, 5996,   42474,  5, }, // Indian/Cocos
    {   73, 4931,   42479,  5, }, // Indian/Comoro
    {   73, 6624,   42484,  5, }, // Indian/Kerguelen
    {   73, 7223,   42489,  3, }, // Indian/Mahe
    {   73, 7013,   42492,  4, }, // Indian/Maldives
    {   73, 5000,   42496,  4, }, // Indian/Mayotte
    {   73, 7174,   42500,  4, }, // Indian/Reunion
    {   73, 4324,   42504,  3, }, // Pacific/Apia
    {   73,  932,   42507,  3, }, // Pacific/Auckland
    {   73, 3608,   42510,  4, }, // Pacific/Bougainville
    {   73, 2201,   42514,  3, }, // Pacific/Easter
    {   73, 3804,   42517,  2, }, // Pacific/Fiji
    {   73, 6641,   42519,  6, }, // Pacific/Galapagos
    {   73, 6659,   42525,  5, }, // Pacific/Gambier
    {   73, 3724,   42530,  6, }, // Pacific/Guadalcanal
    {   73, 5935,   42536,  2, }, // Pacific/Guam
    {   73, 2897,   42538,  5, }, // Pacific/Kanton
    {   73, 3900,   42543,  6, }, // Pacific/Kiritimati
    {   73, 2708,   42549,  4, }, // Pacific/Kwajalein
    {   73, 4779,   42553,  3, }, // Pacific/Majuro
    {   73, 3953,   42556,  5, }, // Pacific/Marquesas
    {   73, 8909,   42561,  3, }, // Pacific/Midway
    {   73, 7047,   42564,  2, }, // Pacific/Nauru
    {   73, 7095,   42566,  3, }, // Pacific/Niue
    {   73, 4070,   42569,  3, }, // Pacific/Norfolk
    {   73, 7061,   42572,  3, }, // Pacific/Noumea
    {   73, 7157,   42575,  5, }, // Pacific/Pitcairn
    {   73, 2961,   42580,  3, }, // Pacific/Pohnpei
    {   73, 6009,   42583,  4, }, // Pacific/Rarotonga
    {   73, 4485,   42587,  5, }, // Pacific/Tongatapu
    {   74, 4961,    3793, 11, }, // Africa/Addis_Ababa Catalan/Latin/Spain
    {   74, 7383,   42592,  5, }, // Africa/Algiers
    {   74,   14,       0,  6, }, // Africa/Asmara
    {   74, 2242,   42597,  9, }, // Africa/Cairo
    {   74, 5068,   42606,  8, }, // Africa/El_Aaiun
    {   74, 4426,   24937,  7, }, // Africa/Khartoum
    {   74, 5015,   42614, 10, }, // Africa/Mogadishu
    {   74, 6997,   42624,  8, }, // Africa/Monrovia
    {   74, 5173,   42632,  9, }, // Africa/Ndjamena
    {   74, 4337,       6,  8, }, // Africa/Sao_Tome
    {   74, 2732,   18851,  7, }, // Africa/Tripoli
    {   74, 4456,   24992,  9, }, // America/Araguaina
    {   74,  270,   18870,  7, }, // America/Argentina/Cordoba
    {   74, 7452,   42641, 12, }, // America/Argentina/Rio_Gallegos
    {   74, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {   74, 4154,      14,  8, }, // America/Asuncion
    {   74,  237,      22,  8, }, // America/Atikokan
    {   74, 7586,      30, 17, }, // America/Bahia_Banderas
    {   74, 7609,   34376,  5, }, // America/Belem
    {   74, 7623,   42653, 12, }, // America/Blanc-Sablon
    {   74, 4263,   42665,  6, }, // America/Bogota
    {   74, 3770,    3861,  6, }, // America/Cancun
    {   74, 4247,   42671,  6, }, // America/Cayenne
    {   74, 5398,   42677,  6, }, // America/Cayman
    {   74, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   74, 3693,   25007,  6, }, // America/Cuiaba
    {   74, 5640,      66,  7, }, // America/Curacao
    {   74, 7832,   25013,  8, }, // America/Eirunepe
    {   74,  481,      73, 13, }, // America/Indiana/Knox
    {   74, 7924,      86, 16, }, // America/Indiana/Marengo
    {   74, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   74, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   74, 8001,     139, 14, }, // America/Indiana/Vevay
    {   74, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   74, 8049,     171, 16, }, // America/Indiana/Winamac
    {   74, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   74, 5849,     207, 22, }, // America/Lower_Princes
    {   74, 8116,   25021,  6, }, // America/Maceio
    {   74, 5722,   18945,  9, }, // America/Martinique
    {   74, 2779,   25036,  8, }, // America/Mazatlan
    {   74, 8183,     229,  6, }, // America/Merida
    {   74, 2811,   42683, 15, }, // America/Mexico_City
    {   74, 3294,   42698,  9, }, // America/New_York
    {   74, 8264,   42707, 23, }, // America/North_Dakota/Beulah
    {   74, 8292,   42730, 23, }, // America/North_Dakota/Center
    {   74, 8320,   42753, 26, }, // America/North_Dakota/New_Salem
    {   74,  393,     298,  4, }, // America/Nuuk
    {   74, 5413,   42779,  6, }, // America/Panama
    {   74, 8440,   25137,  8, }, // America/Santarem
    {   74, 1862,   25145,  9, }, // America/Sao_Paulo
    {   74, 6862,   42785, 12, }, // America/Scoresbysund
    {   74, 8471,   34416, 16, }, // America/St_Barthelemy
    {   74, 2045,   25170, 12, }, // America/St_Johns
    {   74, 5780,   25182, 11, }, // America/St_Kitts
    {   74, 5797,   34444, 11, }, // America/St_Lucia
    {   74,  855,   42797, 12, }, // America/St_Thomas
    {   74, 5830,   25216, 13, }, // America/St_Vincent
    {   74, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   74, 5428,   42809,  6, }, // Asia/Anadyr
    {   74, 5440,   42815,  5, }, // Asia/Aqtau
    {   74, 5270,   42820,  6, }, // Asia/Aqtobe
    {   74,  964,   42826,  7, }, // Asia/Ashgabat
    {   74, 8587,   42833,  6, }, // Asia/Atyrau
    {   74, 3509,    4063,  6, }, // Asia/Baghdad
    {   74, 3555,   19138,  4, }, // Asia/Baku
    {   74, 3473,   42839,  7, }, // Asia/Barnaul
    {   74, 4503,   42846,  5, }, // Asia/Chita
    {   74, 4442,   19148,  6, }, // Asia/Damascus
    {   74, 6092,   42851,  8, }, // Asia/Dushanbe
    {   74, 1226,   42859, 11, }, // Asia/Ho_Chi_Minh
    {   74, 4673,   25278,  5, }, // Asia/Hovd
    {   74, 6955,   42870,  6, }, // Asia/Jayapura
    {   74, 4218,   25283,  9, }, // Asia/Kamchatka
    {   74, 1152,   19221,  8, }, // Asia/Kathmandu
    {   74,  992,   19229,  7, }, // Asia/Kolkata
    {   74, 4099,   25314, 11, }, // Asia/Krasnoyarsk
    {   74, 1178,    4222,  5, }, // Asia/Macau
    {   74, 6883,   34547,  6, }, // Asia/Muscat
    {   74, 2446,   42876,  7, }, // Asia/Nicosia
    {   74, 4037,   42883, 12, }, // Asia/Novosibirsk
    {   74, 8671,   42895,  8, }, // Asia/Qostanay
    {   74, 3486,    6120,  4, }, // Asia/Riyadh
    {   74, 4310,   42903,  8, }, // Asia/Sakhalin
    {   74, 7208,   19277, 10, }, // Asia/Samarkand
    {   74, 3116,   42911,  4, }, // Asia/Seoul
    {   74, 3137,   19291,  8, }, // Asia/Singapore
    {   74, 4199,    6150, 13, }, // Asia/Srednekolymsk
    {   74, 4683,   42915,  8, }, // Asia/Tashkent
    {   74, 2638,    6175,  7, }, // Asia/Tehran
    {   74, 2687,   42923,  6, }, // Asia/Tokyo
    {   74, 1126,   42929,  7, }, // Asia/Urumqi
    {   74, 8685,   42936,  9, }, // Asia/Ust-Nera
    {   74, 4730,   42945,  7, }, // Asia/Yakutsk
    {   74, 1202,     442,  6, }, // Asia/Yangon
    {   74, 3785,   42952, 13, }, // Asia/Yekaterinburg
    {   74, 3565,   42965,  6, }, // Atlantic/Azores
    {   74, 5588,   42971,  8, }, // Atlantic/Bermuda
    {   74, 6591,   42979, 14, }, // Atlantic/Canary
    {   74, 3629,   42993,  8, }, // Atlantic/Cape_Verde
    {   74, 1379,   43001, 11, }, // Atlantic/Faroe
    {   74, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {   74, 7235,   43012, 15, }, // Atlantic/South_Georgia
    {   74, 6784,   34678, 12, }, // Atlantic/St_Helena
    {   74, 3522,   43027,  8, }, // Europe/Astrakhan
    {   74, 6577,   19437,  6, }, // Europe/Athens
    {   74, 6285,   25465,  7, }, // Europe/Belgrade
    {   74, 4659,   19450,  6, }, // Europe/Berlin
    {   74, 6178,   43035, 11, }, // Europe/Brussels
    {   74, 3830,   19464,  8, }, // Europe/Bucharest
    {   74, 6238,   43046, 11, }, // Europe/Copenhagen
    {   74, 2260,   19482,  6, }, // Europe/Dublin
    {   74, 6561,   43057,  8, }, // Europe/Helsinki
    {   74, 8783,    4393,  3, }, // Europe/Isle_of_Man
    {   74, 8924,   43065,  5, }, // Europe/Kirov
    {   74, 2419,   43070,  4, }, // Europe/Kyiv
    {   74, 3078,   19528,  6, }, // Europe/Lisbon
    {   74, 2393,   19543,  7, }, // Europe/London
    {   74, 6314,    4414,  9, }, // Europe/Luxembourg
    {   74, 6359,   43074,  6, }, // Europe/Monaco
    {   74, 3443,   43080,  6, }, // Europe/Moscow
    {   74, 4186,   19571,  5, }, // Europe/Paris
    {   74, 6224,   19576,  5, }, // Europe/Prague
    {   74, 6273,   19581,  4, }, // Europe/Rome
    {   74, 4353,   43086,  7, }, // Europe/Saratov
    {   74, 8828,   43093, 10, }, // Europe/Simferopol
    {   74, 6472,   19599,  8, }, // Europe/Stockholm
    {   74, 6135,   19608,  6, }, // Europe/Tirane
    {   74, 8861,   43103,  9, }, // Europe/Ulyanovsk
    {   74, 6516,   43112,  6, }, // Europe/Vatican
    {   74, 6164,   19625,  5, }, // Europe/Vienna
    {   74, 8878,   43118,  7, }, // Europe/Vilnius
    {   74, 3055,   43125,  8, }, // Europe/Warsaw
    {   74, 6489,   43133,  5, }, // Europe/Zurich
    {   74, 3971,   19647,  7, }, // Indian/Mauritius
    {   74, 7174,   19655,  6, }, // Indian/Reunion
    {   74, 3022,     474,  5, }, // Pacific/Chuuk
    {   74, 2201,   43138, 14, }, // Pacific/Easter
    {   74, 6641,   19672,  9, }, // Pacific/Galapagos
    {   74, 2929,    4512,  8, }, // Pacific/Honolulu
    {   74, 2897,    4520,  9, }, // Pacific/Kanton
    {   74, 3953,   43152,  9, }, // Pacific/Marquesas
    {   74, 7061,    4529,  6, }, // Pacific/Noumea
    {   74, 2961,     479,  7, }, // Pacific/Pohnpei
    {   74, 7275,   43161,  6, }, // Pacific/Tahiti
    {   78,   14,       0,  6, }, // Africa/Asmara Cebuano/Latin/Philippines
    {   78, 4337,       6,  8, }, // Africa/Sao_Tome
    {   78, 4154,      14,  8, }, // America/Asuncion
    {   78,  237,      22,  8, }, // America/Atikokan
    {   78, 7586,      30, 17, }, // America/Bahia_Banderas
    {   78, 3770,      47,  6, }, // America/Cancun
    {   78, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   78, 5640,      66,  7, }, // America/Curacao
    {   78,  481,      73, 13, }, // America/Indiana/Knox
    {   78, 7924,      86, 16, }, // America/Indiana/Marengo
    {   78, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   78, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   78, 8001,     139, 14, }, // America/Indiana/Vevay
    {   78, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   78, 8049,     171, 16, }, // America/Indiana/Winamac
    {   78, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   78, 5849,     207, 22, }, // America/Lower_Princes
    {   78, 8183,     229,  6, }, // America/Merida
    {   78, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   78, 8292,     255, 20, }, // America/North_Dakota/Center
    {   78, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   78,  393,     298,  4, }, // America/Nuuk
    {   78, 6862,     302, 16, }, // America/Scoresbysund
    {   78, 8471,     318, 14, }, // America/St_Barthelemy
    {   78, 2045,     332, 10, }, // America/St_Johns
    {   78, 5780,     342,  9, }, // America/St_Kitts
    {   78, 5797,     351,  9, }, // America/St_Lucia
    {   78,  855,     360, 10, }, // America/St_Thomas
    {   78, 5830,     370, 11, }, // America/St_Vincent
    {   78, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   78, 1226,   43167, 22, }, // Asia/Ho_Chi_Minh
    {   78, 1152,     413,  9, }, // Asia/Kathmandu
    {   78,  992,     422,  7, }, // Asia/Kolkata
    {   78, 1178,     429,  5, }, // Asia/Macau
    {   78, 1202,     442,  6, }, // Asia/Yangon
    {   78, 1379,     448,  5, }, // Atlantic/Faroe
    {   78, 6784,     453, 10, }, // Atlantic/St_Helena
    {   78, 2419,     463,  4, }, // Europe/Kyiv
    {   78, 7174,     467,  7, }, // Indian/Reunion
    {   78, 3022,     474,  5, }, // Pacific/Chuuk
    {   78, 2961,     479,  7, }, // Pacific/Pohnpei
    {   79,   14,       0,  6, }, // Africa/Asmara Central Atlas Tamazight/Latin/Morocco
    {   79, 4337,       6,  8, }, // Africa/Sao_Tome
    {   79, 4154,      14,  8, }, // America/Asuncion
    {   79,  237,      22,  8, }, // America/Atikokan
    {   79, 7586,      30, 17, }, // America/Bahia_Banderas
    {   79, 3770,      47,  6, }, // America/Cancun
    {   79, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   79, 5640,      66,  7, }, // America/Curacao
    {   79,  481,      73, 13, }, // America/Indiana/Knox
    {   79, 7924,      86, 16, }, // America/Indiana/Marengo
    {   79, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   79, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   79, 8001,     139, 14, }, // America/Indiana/Vevay
    {   79, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   79, 8049,     171, 16, }, // America/Indiana/Winamac
    {   79, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   79, 5849,     207, 22, }, // America/Lower_Princes
    {   79, 8183,     229,  6, }, // America/Merida
    {   79, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   79, 8292,     255, 20, }, // America/North_Dakota/Center
    {   79, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   79,  393,     298,  4, }, // America/Nuuk
    {   79, 6862,     302, 16, }, // America/Scoresbysund
    {   79, 8471,     318, 14, }, // America/St_Barthelemy
    {   79, 2045,     332, 10, }, // America/St_Johns
    {   79, 5780,     342,  9, }, // America/St_Kitts
    {   79, 5797,     351,  9, }, // America/St_Lucia
    {   79,  855,     360, 10, }, // America/St_Thomas
    {   79, 5830,     370, 11, }, // America/St_Vincent
    {   79, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   79, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   79, 1152,     413,  9, }, // Asia/Kathmandu
    {   79,  992,     422,  7, }, // Asia/Kolkata
    {   79, 1178,     429,  5, }, // Asia/Macau
    {   79, 1202,     442,  6, }, // Asia/Yangon
    {   79, 1379,     448,  5, }, // Atlantic/Faroe
    {   79, 6784,     453, 10, }, // Atlantic/St_Helena
    {   79, 2419,     463,  4, }, // Europe/Kyiv
    {   79, 7174,     467,  7, }, // Indian/Reunion
    {   79, 3022,     474,  5, }, // Pacific/Chuuk
    {   79, 2961,     479,  7, }, // Pacific/Pohnpei
    {   80,   14,       0,  6, }, // Africa/Asmara Central Kurdish/Arabic/Iraq
    {   80, 4337,       6,  8, }, // Africa/Sao_Tome
    {   80, 4154,      14,  8, }, // America/Asuncion
    {   80,  237,      22,  8, }, // America/Atikokan
    {   80, 7586,      30, 17, }, // America/Bahia_Banderas
    {   80, 3770,      47,  6, }, // America/Cancun
    {   80, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   80, 5640,      66,  7, }, // America/Curacao
    {   80,  481,      73, 13, }, // America/Indiana/Knox
    {   80, 7924,      86, 16, }, // America/Indiana/Marengo
    {   80, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   80, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   80, 8001,     139, 14, }, // America/Indiana/Vevay
    {   80, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   80, 8049,     171, 16, }, // America/Indiana/Winamac
    {   80, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   80, 5849,     207, 22, }, // America/Lower_Princes
    {   80, 8183,     229,  6, }, // America/Merida
    {   80, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   80, 8292,     255, 20, }, // America/North_Dakota/Center
    {   80, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   80,  393,     298,  4, }, // America/Nuuk
    {   80, 6862,     302, 16, }, // America/Scoresbysund
    {   80, 8471,     318, 14, }, // America/St_Barthelemy
    {   80, 2045,     332, 10, }, // America/St_Johns
    {   80, 5780,     342,  9, }, // America/St_Kitts
    {   80, 5797,     351,  9, }, // America/St_Lucia
    {   80,  855,     360, 10, }, // America/St_Thomas
    {   80, 5830,     370, 11, }, // America/St_Vincent
    {   80, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   80, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   80, 1152,     413,  9, }, // Asia/Kathmandu
    {   80,  992,     422,  7, }, // Asia/Kolkata
    {   80, 1178,     429,  5, }, // Asia/Macau
    {   80, 1202,     442,  6, }, // Asia/Yangon
    {   80, 1379,     448,  5, }, // Atlantic/Faroe
    {   80, 6784,     453, 10, }, // Atlantic/St_Helena
    {   80, 2419,     463,  4, }, // Europe/Kyiv
    {   80, 7174,     467,  7, }, // Indian/Reunion
    {   80, 3022,     474,  5, }, // Pacific/Chuuk
    {   80, 2961,     479,  7, }, // Pacific/Pohnpei
    {   82, 6751,   43189, 16, }, // Africa/Abidjan Chakma/Chakma/Bangladesh
    {   82, 6723,   43205,  8, }, // Africa/Accra
    {   82, 4961,   43213, 21, }, // Africa/Addis_Ababa
    {   82, 7383,   43234, 20, }, // Africa/Algiers
    {   82,   14,   43254, 16, }, // Africa/Asmara
    {   82,   44,   43270, 12, }, // Africa/Bamako
    {   82, 5159,   43282, 12, }, // Africa/Bangui
    {   82, 6709,   43294, 12, }, // Africa/Banjul
    {   82, 7398,   43306,  8, }, // Africa/Bissau
    {   82, 4859,   43314, 24, }, // Africa/Blantyre
    {   82, 5189,   43338, 18, }, // Africa/Brazzaville
    {   82, 4824,   43356, 24, }, // Africa/Bujumbura
    {   82, 2242,   43380, 10, }, // Africa/Cairo
    {   82, 4019,   43390, 14, }, // Africa/Casablanca
    {   82, 7412,   43404, 10, }, // Africa/Ceuta
    {   82, 6736,   43414, 14, }, // Africa/Conakry
    {   82, 6803,   43428,  8, }, // Africa/Dakar
    {   82, 5032,   43436, 24, }, // Africa/Dar_es_Salaam
    {   82, 4945,   43460, 12, }, // Africa/Djibouti
    {   82, 5145,   43472,  8, }, // Africa/Douala
    {   82, 5068,   43480, 27, }, // Africa/El_Aaiun
    {   82, 6816,   43507, 18, }, // Africa/Freetown
    {   82, 4808,   43525, 18, }, // Africa/Gaborone
    {   82, 4917,   43543,  8, }, // Africa/Harare
    {   82, 4381,   43551, 24, }, // Africa/Johannesburg
    {   82, 4401,   43575,  6, }, // Africa/Juba
    {   82, 5053,   43581, 10, }, // Africa/Kampala
    {   82, 4426,   43591, 14, }, // Africa/Khartoum
    {   82, 4875,   43605, 10, }, // Africa/Kigali
    {   82, 5208,   43615, 12, }, // Africa/Kinshasa
    {   82, 4646,   43627, 10, }, // Africa/Lagos
    {   82, 5238,   43637, 20, }, // Africa/Libreville
    {   82, 6832,   43657,  8, }, // Africa/Lome
    {   82, 5113,   43665, 12, }, // Africa/Luanda
    {   82, 4841,   43677, 18, }, // Africa/Lubumbashi
    {   82, 4903,   43695,  8, }, // Africa/Lusaka
    {   82, 5224,   43703,  8, }, // Africa/Malabo
    {   82, 4889,   43711, 10, }, // Africa/Maputo
    {   82, 5099,   43721, 14, }, // Africa/Maseru
    {   82, 5084,   43735, 16, }, // Africa/Mbabane
    {   82, 5015,   43751, 14, }, // Africa/Mogadishu
    {   82, 6997,   43765, 18, }, // Africa/Monrovia
    {   82, 3755,   43783, 12, }, // Africa/Nairobi
    {   82, 5173,   43795, 18, }, // Africa/Ndjamena
    {   82, 5256,   43813, 10, }, // Africa/Niamey
    {   82, 6766,   43823, 18, }, // Africa/Nouakchott
    {   82, 6690,   43841, 18, }, // Africa/Ouagadougou
    {   82, 5127,   43859, 21, }, // Africa/Porto-Novo
    {   82, 4337,   43880, 15, }, // Africa/Sao_Tome
    {   82, 2732,   43895, 16, }, // Africa/Tripoli
    {   82, 6503,   43911, 16, }, // Africa/Tunis
    {   82, 4054,   43927, 20, }, // Africa/Windhoek
    {   82,  132,   43947,  8, }, // America/Adak
    {   82, 3183,   43955, 20, }, // America/Anchorage
    {   82, 5524,   43975, 20, }, // America/Anguilla
    {   82, 5541,   43995, 22, }, // America/Antigua
    {   82, 4456,   44017, 14, }, // America/Araguaina
    {   82,  166,   44031, 31, }, // America/Argentina/Buenos_Aires
    {   82,   91,   44062, 16, }, // America/Argentina/Catamarca
    {   82,  270,   44078, 14, }, // America/Argentina/Cordoba
    {   82,  441,   44092, 12, }, // America/Argentina/Jujuy
    {   82, 7425,   44104, 13, }, // America/Argentina/La_Rioja
    {   82,  565,   44117, 14, }, // America/Argentina/Mendoza
    {   82, 7452,   44131, 23, }, // America/Argentina/Rio_Gallegos
    {   82, 7483,   44154, 12, }, // America/Argentina/Salta
    {   82, 7507,   44166, 17, }, // America/Argentina/San_Juan
    {   82, 5497,   44183, 17, }, // America/Argentina/San_Luis
    {   82, 7534,   44200, 14, }, // America/Argentina/Tucuman
    {   82, 7560,   44214, 12, }, // America/Argentina/Ushuaia
    {   82, 5557,   44226, 10, }, // America/Aruba
    {   82, 4154,   44236, 22, }, // America/Asuncion
    {   82,  237,   44258, 23, }, // America/Atikokan
    {   82, 3581,   44281,  8, }, // America/Bahia
    {   82, 7586,   44289, 25, }, // America/Bahia_Banderas
    {   82, 5571,   44314, 16, }, // America/Barbados
    {   82, 7609,   44330, 12, }, // America/Belem
    {   82, 5309,   44342, 12, }, // America/Belize
    {   82, 7623,   44354, 27, }, // America/Blanc-Sablon
    {   82, 7644,   44381, 15, }, // America/Boa_Vista
    {   82, 4263,   44396, 10, }, // America/Bogota
    {   82, 7662,   44406, 12, }, // America/Boise
    {   82, 7676,   44418, 25, }, // America/Cambridge_Bay
    {   82, 7698,   44443, 25, }, // America/Campo_Grande
    {   82, 3770,   44468, 18, }, // America/Cancun
    {   82, 4596,   44486, 14, }, // America/Caracas
    {   82, 4247,   44500, 10, }, // America/Cayenne
    {   82, 5398,   44510, 10, }, // America/Cayman
    {   82, 3251,   44520, 10, }, // America/Chicago
    {   82, 7719,   44530, 16, }, // America/Chihuahua
    {   82, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   82, 5324,   44546, 16, }, // America/Costa_Rica
    {   82, 7759,   44562, 18, }, // America/Creston
    {   82, 3693,   44580, 10, }, // America/Cuiaba
    {   82, 5640,   44590, 12, }, // America/Curacao
    {   82, 7775,   44602, 30, }, // America/Danmarkshavn
    {   82, 7796,   44632, 12, }, // America/Dawson
    {   82, 7811,   44644, 25, }, // America/Dawson_Creek
    {   82,  805,   44669, 14, }, // America/Denver
    {   82, 3351,   44683, 20, }, // America/Detroit
    {   82, 5656,   44703, 14, }, // America/Dominica
    {   82,  893,   44717, 24, }, // America/Edmonton
    {   82, 7832,   44741, 16, }, // America/Eirunepe
    {   82, 5343,   44757, 25, }, // America/El_Salvador
    {   82, 7849,   44782, 29, }, // America/Fort_Nelson
    {   82, 7869,   44811, 14, }, // America/Fortaleza
    {   82, 7887,   44825, 15, }, // America/Glace_Bay
    {   82, 6844,   44840, 13, }, // America/Goose_Bay
    {   82, 4514,   44853, 31, }, // America/Grand_Turk
    {   82, 5687,   44884, 12, }, // America/Grenada
    {   82, 5703,   44896, 16, }, // America/Guadeloupe
    {   82, 3662,   44912, 12, }, // America/Guatemala
    {   82, 4743,   44924, 20, }, // America/Guayaquil
    {   82, 6895,   44944,  6, }, // America/Guyana
    {   82, 1923,   44950, 24, }, // America/Halifax
    {   82, 2221,   44974,  6, }, // America/Havana
    {   82, 7905,   44980, 22, }, // America/Hermosillo
    {   82,  348,   45002, 28, }, // America/Indiana/Indianapolis
    {   82,  481,   45030, 30, }, // America/Indiana/Knox
    {   82, 7924,   45060, 32, }, // America/Indiana/Marengo
    {   82, 7948,   45092, 42, }, // America/Indiana/Petersburg
    {   82, 7975,   45134, 34, }, // America/Indiana/Tell_City
    {   82, 8001,   45168, 30, }, // America/Indiana/Vevay
    {   82, 8023,   45198, 42, }, // America/Indiana/Vincennes
    {   82, 8049,   45240, 38, }, // America/Indiana/Winamac
    {   82, 8073,   45278, 18, }, // America/Inuvik
    {   82,  660,   45296, 20, }, // America/Iqaluit
    {   82, 2665,   45316,  8, }, // America/Jamaica
    {   82, 5282,   45324,  8, }, // America/Juneau
    {   82,  521,   45332, 18, }, // America/Kentucky/Louisville
    {   82, 8088,   45350, 36, }, // America/Kentucky/Monticello
    {   82, 5621,   45386, 26, }, // America/Kralendijk
    {   82, 4278,   45412,  9, }, // America/La_Paz
    {   82, 7132,   45421,  6, }, // America/Lima
    {   82, 3390,   45427, 31, }, // America/Los_Angeles
    {   82, 5849,   45458, 48, }, // America/Lower_Princes
    {   82, 8116,   45506, 10, }, // America/Maceio
    {   82, 8131,   45516,  8, }, // America/Managua
    {   82, 1892,   45524, 12, }, // America/Manaus
    {   82, 5814,   45536, 10, }, // America/Marigot
    {   82, 5722,   45546, 18, }, // America/Martinique
    {   82, 8147,   45564, 16, }, // America/Matamoros
    {   82, 2779,   45580, 14, }, // America/Mazatlan
    {   82, 8165,   45594, 16, }, // America/Menominee
    {   82, 8183,   45610, 10, }, // America/Merida
    {   82, 8198,   45620, 16, }, // America/Metlakatla
    {   82, 2811,   45636, 29, }, // America/Mexico_City
    {   82, 4293,   45665, 16, }, // America/Miquelon
    {   82, 8217,   45681, 16, }, // America/Moncton
    {   82, 8233,   45697, 16, }, // America/Monterrey
    {   82, 4000,   45713, 24, }, // America/Montevideo
    {   82, 5741,   45737, 22, }, // America/Montserrat
    {   82, 5383,   45759,  8, }, // America/Nassau
    {   82, 3294,   45767, 24, }, // America/New_York
    {   82, 8251,   45791,  8, }, // America/Nome
    {   82, 1834,   45799, 14, }, // America/Noronha
    {   82, 8264,   45813, 37, }, // America/North_Dakota/Beulah
    {   82, 8292,   45850, 41, }, // America/North_Dakota/Center
    {   82, 8320,   45891, 46, }, // America/North_Dakota/New_Salem
    {   82,  393,   45937,  8, }, // America/Nuuk
    {   82, 8351,   45945, 14, }, // America/Ojinaga
    {   82, 5413,   45959,  6, }, // America/Panama
    {   82, 6106,   45965, 20, }, // America/Paramaribo
    {   82, 3224,   45985, 16, }, // America/Phoenix
    {   82, 3847,   46001, 34, }, // America/Port-au-Prince
    {   82, 5871,   46035, 34, }, // America/Port_of_Spain
    {   82, 8367,   46069, 25, }, // America/Porto_Velho
    {   82, 5760,   46094, 25, }, // America/Puerto_Rico
    {   82, 8387,   46119, 35, }, // America/Rankin_Inlet
    {   82, 8408,   46154, 12, }, // America/Recife
    {   82, 1979,   46166, 10, }, // America/Regina
    {   82, 8423,   46176, 16, }, // America/Resolute
    {   82,  695,   46192, 23, }, // America/Rio_Branco
    {   82, 8440,   46215, 20, }, // America/Santarem
    {   82, 2165,   46235, 16, }, // America/Santiago
    {   82, 6044,   46251, 29, }, // America/Santo_Domingo
    {   82, 1862,   46280, 17, }, // America/Sao_Paulo
    {   82, 6862,   46297, 36, }, // America/Scoresbysund
    {   82, 8457,   46333, 10, }, // America/Sitka
    {   82, 8471,   46343, 32, }, // America/St_Barthelemy
    {   82, 2045,   46375, 25, }, // America/St_Johns
    {   82, 5780,   46400, 26, }, // America/St_Kitts
    {   82, 5797,   46426, 24, }, // America/St_Lucia
    {   82,  855,   46450, 23, }, // America/St_Thomas
    {   82, 5830,   46473, 33, }, // America/St_Vincent
    {   82, 8493,   46506, 29, }, // America/Swift_Current
    {   82, 5363,   46535, 20, }, // America/Tegucigalpa
    {   82, 5673,   46555,  8, }, // America/Thule
    {   82,  313,   46563, 12, }, // America/Tijuana
    {   82,  608,   46575, 16, }, // America/Toronto
    {   82, 5605,   46591, 14, }, // America/Tortola
    {   82, 2077,   46605, 20, }, // America/Vancouver
    {   82, 2128,   46625, 22, }, // America/Whitehorse
    {   82,  734,   46647, 20, }, // America/Winnipeg
    {   82, 8515,   46667, 16, }, // America/Yakutat
    {   82, 5918,   46683, 12, }, // Antarctica/Casey
    {   82, 6027,   46695, 12, }, // Antarctica/Davis
    {   82, 6066,   46707, 37, }, // Antarctica/DumontDUrville
    {   82, 8531,   46744, 22, }, // Antarctica/Macquarie
    {   82, 7029,   46766, 12, }, // Antarctica/Mawson
    {   82, 7076,   46778, 22, }, // Antarctica/McMurdo
    {   82, 8552,   46800, 12, }, // Antarctica/Palmer
    {   82, 7189,   46812, 10, }, // Antarctica/Rothera
    {   82, 7258,   46822,  6, }, // Antarctica/Syowa
    {   82, 8570,   46828, 12, }, // Antarctica/Troll
    {   82, 7337,   46840, 16, }, // Antarctica/Vostok
    {   82, 1413,   46856, 28, }, // Arctic/Longyearbyen
    {   82, 5487,   46884, 14, }, // Asia/Aden
    {   82, 5297,   46898, 12, }, // Asia/Almaty
    {   82, 3870,   46910, 14, }, // Asia/Amman
    {   82, 5428,   46924, 16, }, // Asia/Anadyr
    {   82, 5440,   46940, 12, }, // Asia/Aqtau
    {   82, 5270,   46952, 14, }, // Asia/Aqtobe
    {   82,  964,   46966, 16, }, // Asia/Ashgabat
    {   82, 3509,   46982, 12, }, // Asia/Baghdad
    {   82, 5451,   46994, 14, }, // Asia/Bahrain
    {   82, 3555,   44588,  6, }, // Asia/Baku
    {   82, 4368,   47008, 16, }, // Asia/Bangkok
    {   82, 3473,   47024, 14, }, // Asia/Barnaul
    {   82, 3988,   47038, 14, }, // Asia/Beirut
    {   82, 3680,   47052, 16, }, // Asia/Bishkek
    {   82, 5906,   47068, 14, }, // Asia/Brunei
    {   82, 4503,   47082,  6, }, // Asia/Chita
    {   82, 5963,   47088, 22, }, // Asia/Choibalsan
    {   82, 4413,   47110, 22, }, // Asia/Colombo
    {   82, 4442,   47132, 14, }, // Asia/Damascus
    {   82, 1060,   47146,  4, }, // Asia/Dhaka
    {   82, 6125,   47150,  8, }, // Asia/Dili
    {   82, 3498,   47158,  8, }, // Asia/Dubai
    {   82, 6092,   47166, 14, }, // Asia/Dushanbe
    {   82, 8614,   47180,  4, }, // Asia/Gaza
    {   82, 4697,   47184, 16, }, // Asia/Hebron
    {   82, 1226,   47200, 31, }, // Asia/Ho_Chi_Minh
    {   82, 2576,   47231, 13, }, // Asia/Hong_Kong
    {   82, 4673,   47244, 12, }, // Asia/Hovd
    {   82, 4086,   47256, 24, }, // Asia/Irkutsk
    {   82, 6969,   47280, 10, }, // Asia/Jakarta
    {   82, 6955,   47290, 12, }, // Asia/Jayapura
    {   82, 1257,   47302, 18, }, // Asia/Jerusalem
    {   82, 3462,   47320, 10, }, // Asia/Kabul
    {   82, 4218,   47330, 14, }, // Asia/Kamchatka
    {   82, 4141,   47344, 10, }, // Asia/Karachi
    {   82, 1152,   47354, 18, }, // Asia/Kathmandu
    {   82, 8624,   47372, 14, }, // Asia/Khandyga
    {   82,  992,   47386, 12, }, // Asia/Kolkata
    {   82, 4099,   47398, 28, }, // Asia/Krasnoyarsk
    {   82, 4761,   47426, 22, }, // Asia/Kuala_Lumpur
    {   82, 5893,   47448, 10, }, // Asia/Kuching
    {   82, 5464,   47458, 12, }, // Asia/Kuwait
    {   82, 1178,   47470, 12, }, // Asia/Macau
    {   82, 3919,   47482, 14, }, // Asia/Magadan
    {   82, 1316,   47496, 14, }, // Asia/Makassar
    {   82, 7145,   47510, 12, }, // Asia/Manila
    {   82, 6883,   47522, 12, }, // Asia/Muscat
    {   82, 2446,   47534, 14, }, // Asia/Nicosia
    {   82, 8638,   47548, 32, }, // Asia/Novokuznetsk
    {   82, 4037,   47580, 28, }, // Asia/Novosibirsk
    {   82, 4131,   47608, 16, }, // Asia/Omsk
    {   82, 7108,   47624, 10, }, // Asia/Oral
    {   82, 6924,   47634, 17, }, // Asia/Phnom_Penh
    {   82, 8656,   47651, 24, }, // Asia/Pontianak
    {   82, 4116,   47675, 21, }, // Asia/Pyongyang
    {   82, 5476,   47696,  8, }, // Asia/Qatar
    {   82, 4171,   47704, 18, }, // Asia/Qyzylorda
    {   82, 3486,   47722, 10, }, // Asia/Riyadh
    {   82, 4310,   47732, 12, }, // Asia/Sakhalin
    {   82, 7208,   47744, 24, }, // Asia/Samarkand
    {   82, 3116,   47768, 12, }, // Asia/Seoul
    {   82, 1020,   47780,  8, }, // Asia/Shanghai
    {   82, 3137,   47788, 18, }, // Asia/Singapore
    {   82, 4199,   47806, 40, }, // Asia/Srednekolymsk
    {   82, 3100,   47846, 12, }, // Asia/Taipei
    {   82, 4683,   47858, 18, }, // Asia/Tashkent
    {   82, 3817,   47876, 16, }, // Asia/Tbilisi
    {   82, 2638,   47892, 14, }, // Asia/Tehran
    {   82, 1284,   47906, 12, }, // Asia/Thimphu
    {   82, 2687,   47918, 12, }, // Asia/Tokyo
    {   82, 4474,   47930, 16, }, // Asia/Tomsk
    {   82, 1346,   47946, 18, }, // Asia/Ulaanbaatar
    {   82, 1126,   47964, 16, }, // Asia/Urumqi
    {   82, 8685,   47980, 19, }, // Asia/Ust-Nera
    {   82, 6940,   47999, 24, }, // Asia/Vientiane
    {   82, 4612,   48023, 26, }, // Asia/Vladivostok
    {   82, 4730,   48049, 18, }, // Asia/Yakutsk
    {   82, 1202,   48067, 16, }, // Asia/Yangon
    {   82, 3785,   48083, 32, }, // Asia/Yekaterinburg
    {   82, 3649,   48115, 16, }, // Asia/Yerevan
    {   82, 3565,   48131, 14, }, // Atlantic/Azores
    {   82, 5588,   48145, 12, }, // Atlantic/Bermuda
    {   82, 6591,   48157, 10, }, // Atlantic/Canary
    {   82, 3629,   48167, 19, }, // Atlantic/Cape_Verde
    {   82, 1379,   48186, 10, }, // Atlantic/Faroe
    {   82, 8699,   48196, 10, }, // Atlantic/Madeira
    {   82, 2614,   48206, 18, }, // Atlantic/Reykjavik
    {   82, 7235,   48224, 27, }, // Atlantic/South_Georgia
    {   82, 6784,   48251, 23, }, // Atlantic/St_Helena
    {   82, 6607,   48274, 14, }, // Atlantic/Stanley
    {   82, 1654,   48288, 20, }, // Australia/Adelaide
    {   82, 1619,   48308, 20, }, // Australia/Brisbane
    {   82, 1783,   48328, 25, }, // Australia/Broken_Hill
    {   82, 1567,   48353, 18, }, // Australia/Darwin
    {   82, 3539,   48371, 14, }, // Australia/Eucla
    {   82, 1500,   48385, 14, }, // Australia/Hobart
    {   82, 8716,   48399, 22, }, // Australia/Lindeman
    {   82, 1531,   48421, 19, }, // Australia/Lord_Howe
    {   82, 1711,   48440, 18, }, // Australia/Melbourne
    {   82, 1746,   48458, 10, }, // Australia/Perth
    {   82, 1447,   48468, 12, }, // Australia/Sydney
    {   82, 6390,   48480, 22, }, // Europe/Amsterdam
    {   82, 6149,   48502, 16, }, // Europe/Andorra
    {   82, 3522,   48518, 18, }, // Europe/Astrakhan
    {   82, 6577,   48536, 16, }, // Europe/Athens
    {   82, 6285,   48552, 20, }, // Europe/Belgrade
    {   82, 4659,   48572, 14, }, // Europe/Berlin
    {   82, 6437,   48586, 18, }, // Europe/Bratislava
    {   82, 6178,   48604, 18, }, // Europe/Brussels
    {   82, 3830,   48622, 18, }, // Europe/Bucharest
    {   82, 3708,   48640, 18, }, // Europe/Budapest
    {   82, 8751,   48658, 20, }, // Europe/Busingen
    {   82, 2475,   48678, 14, }, // Europe/Chisinau
    {   82, 6238,   48692, 24, }, // Europe/Copenhagen
    {   82, 2260,   48716, 14, }, // Europe/Dublin
    {   82, 6256,   48730, 20, }, // Europe/Gibraltar
    {   82, 8767,   48750, 16, }, // Europe/Guernsey
    {   82, 6561,   48766, 18, }, // Europe/Helsinki
    {   82, 8783,   48784, 28, }, // Europe/Isle_of_Man
    {   82, 1097,   48812, 22, }, // Europe/Istanbul
    {   82, 8802,   48834, 10, }, // Europe/Jersey
    {   82, 3881,   48844, 24, }, // Europe/Kaliningrad
    {   82, 8924,   48868, 12, }, // Europe/Kirov
    {   82, 2419,   48880, 12, }, // Europe/Kyiv
    {   82, 3078,   48892, 16, }, // Europe/Lisbon
    {   82, 6455,   48908, 16, }, // Europe/Ljubljana
    {   82, 2393,   48924, 16, }, // Europe/London
    {   82, 6314,   48940, 26, }, // Europe/Luxembourg
    {   82, 4794,   48966, 14, }, // Europe/Madrid
    {   82, 6346,   48980, 10, }, // Europe/Malta
    {   82, 6531,   48990, 22, }, // Europe/Mariehamn
    {   82, 3595,   49012, 16, }, // Europe/Minsk
    {   82, 6359,   49028, 10, }, // Europe/Monaco
    {   82, 3443,   49038, 12, }, // Europe/Moscow
    {   82, 6407,   49050, 12, }, // Europe/Oslo
    {   82, 4186,   49062, 14, }, // Europe/Paris
    {   82, 6373,   49076, 18, }, // Europe/Podgorica
    {   82, 6224,   49094, 10, }, // Europe/Prague
    {   82, 8816,   45538,  6, }, // Europe/Riga
    {   82, 6273,   49104,  8, }, // Europe/Rome
    {   82, 4233,   49112,  6, }, // Europe/Samara
    {   82, 6419,   49118, 19, }, // Europe/San_Marino
    {   82, 6194,   49137, 12, }, // Europe/Sarajevo
    {   82, 8828,   49149, 24, }, // Europe/Simferopol
    {   82, 6332,   49173, 16, }, // Europe/Skopje
    {   82, 6548,   49189, 10, }, // Europe/Sofia
    {   82, 6472,   49199, 20, }, // Europe/Stockholm
    {   82, 8846,   49219, 10, }, // Europe/Tallinn
    {   82, 6135,   49229,  8, }, // Europe/Tirane
    {   82, 8861,   49237, 26, }, // Europe/Ulyanovsk
    {   82, 6301,   49263, 10, }, // Europe/Vaduz
    {   82, 6516,   49273, 12, }, // Europe/Vatican
    {   82, 6164,   47999, 10, }, // Europe/Vienna
    {   82, 8878,   49285, 18, }, // Europe/Vilnius
    {   82, 4629,   49303, 22, }, // Europe/Volgograd
    {   82, 3055,   49325, 10, }, // Europe/Warsaw
    {   82, 6210,   49335, 14, }, // Europe/Zagreb
    {   82, 6489,   49349, 12, }, // Europe/Zurich
    {   82, 4980,   49361, 20, }, // Indian/Antananarivo
    {   82, 6910,   49381, 10, }, // Indian/Chagos
    {   82, 5979,   49391, 33, }, // Indian/Christmas
    {   82, 5996,   49424, 12, }, // Indian/Cocos
    {   82, 4931,   49436, 12, }, // Indian/Comoro
    {   82, 6624,   49448, 18, }, // Indian/Kerguelen
    {   82, 7223,   49466,  6, }, // Indian/Mahe
    {   82, 7013,   49472, 14, }, // Indian/Maldives
    {   82, 3971,   49486, 14, }, // Indian/Mauritius
    {   82, 5000,   49500, 10, }, // Indian/Mayotte
    {   82, 7174,   49510, 24, }, // Indian/Reunion
    {   82, 4324,   49534,  8, }, // Pacific/Apia
    {   82,  932,   49542, 22, }, // Pacific/Auckland
    {   82, 3608,   49564, 20, }, // Pacific/Bougainville
    {   82, 2863,   49584, 12, }, // Pacific/Chatham
    {   82, 3022,   49596, 12, }, // Pacific/Chuuk
    {   82, 2201,   49608, 14, }, // Pacific/Easter
    {   82, 7323,   49622, 10, }, // Pacific/Efate
    {   82, 7290,   49632, 16, }, // Pacific/Fakaofo
    {   82, 3804,   49648,  8, }, // Pacific/Fiji
    {   82, 7306,   49656, 14, }, // Pacific/Funafuti
    {   82, 6641,   49670, 22, }, // Pacific/Galapagos
    {   82, 6659,   49692, 18, }, // Pacific/Gambier
    {   82, 3724,   49710, 20, }, // Pacific/Guadalcanal
    {   82, 5935,   49730,  8, }, // Pacific/Guam
    {   82, 2929,   49738, 16, }, // Pacific/Honolulu
    {   82, 2897,   49754, 20, }, // Pacific/Kanton
    {   82, 3900,   49774, 18, }, // Pacific/Kiritimati
    {   82, 6982,   49792, 14, }, // Pacific/Kosrae
    {   82, 2708,   49806, 16, }, // Pacific/Kwajalein
    {   82, 4779,   49822, 10, }, // Pacific/Majuro
    {   82, 3953,   49832, 16, }, // Pacific/Marquesas
    {   82, 8909,   49848, 16, }, // Pacific/Midway
    {   82, 7047,   49864, 10, }, // Pacific/Nauru
    {   82, 7095,   49874, 12, }, // Pacific/Niue
    {   82, 4070,   49886, 16, }, // Pacific/Norfolk
    {   82, 7061,   49902, 10, }, // Pacific/Noumea
    {   82, 2991,   49912, 21, }, // Pacific/Pago_Pago
    {   82, 7118,   49933,  8, }, // Pacific/Palau
    {   82, 7157,   49941, 22, }, // Pacific/Pitcairn
    {   82, 2961,   49963, 10, }, // Pacific/Pohnpei
    {   82, 4709,   49973, 29, }, // Pacific/Port_Moresby
    {   82, 6009,   50002, 14, }, // Pacific/Rarotonga
    {   82, 5948,   50016, 10, }, // Pacific/Saipan
    {   82, 7275,   50026, 10, }, // Pacific/Tahiti
    {   82, 6675,   50036,  6, }, // Pacific/Tarawa
    {   82, 4485,   50042, 14, }, // Pacific/Tongatapu
    {   82, 7355,   50056, 12, }, // Pacific/Wake
    {   82, 7368,   50068, 12, }, // Pacific/Wallis
    {   84, 6751,   34826,  7, }, // Africa/Abidjan Chechen/Cyrillic/Russia
    {   84, 6723,     492,  5, }, // Africa/Accra
    {   84, 4961,     497, 11, }, // Africa/Addis_Ababa
    {   84, 7383,     508,  5, }, // Africa/Algiers
    {   84,   14,     513,  6, }, // Africa/Asmara
    {   84,   44,     519,  6, }, // Africa/Bamako
    {   84, 5159,     525,  5, }, // Africa/Bangui
    {   84, 6709,     530,  6, }, // Africa/Banjul
    {   84, 7398,     536,  5, }, // Africa/Bissau
    {   84, 4859,   25670,  8, }, // Africa/Blantyre
    {   84, 5189,   50080, 10, }, // Africa/Brazzaville
    {   84, 4824,     558,  9, }, // Africa/Bujumbura
    {   84, 2242,   50090,  8, }, // Africa/Cairo
    {   84, 4019,     571, 10, }, // Africa/Casablanca
    {   84, 7412,     581,  5, }, // Africa/Ceuta
    {   84, 6736,     586,  7, }, // Africa/Conakry
    {   84, 6803,     593,  5, }, // Africa/Dakar
    {   84, 5032,   25698, 12, }, // Africa/Dar_es_Salaam
    {   84, 4945,   34859,  7, }, // Africa/Djibouti
    {   84, 5145,     616,  5, }, // Africa/Douala
    {   84, 5068,   25717,  7, }, // Africa/El_Aaiun
    {   84, 6816,     628,  7, }, // Africa/Freetown
    {   84, 4808,     635,  8, }, // Africa/Gaborone
    {   84, 4917,   32384,  6, }, // Africa/Harare
    {   84, 4381,   50098, 12, }, // Africa/Johannesburg
    {   84, 4401,   25755,  5, }, // Africa/Juba
    {   84, 5053,     665,  7, }, // Africa/Kampala
    {   84, 4426,     672,  6, }, // Africa/Khartoum
    {   84, 4875,     678,  6, }, // Africa/Kigali
    {   84, 5208,     684,  7, }, // Africa/Kinshasa
    {   84, 4646,     691,  5, }, // Africa/Lagos
    {   84, 5238,   50110,  9, }, // Africa/Libreville
    {   84, 6832,     704,  4, }, // Africa/Lome
    {   84, 5113,     708,  6, }, // Africa/Luanda
    {   84, 4841,     714,  9, }, // Africa/Lubumbashi
    {   84, 4903,     723,  6, }, // Africa/Lusaka
    {   84, 5224,     729,  6, }, // Africa/Malabo
    {   84, 4889,     735,  6, }, // Africa/Maputo
    {   84, 5099,     741,  6, }, // Africa/Maseru
    {   84, 5084,     747,  7, }, // Africa/Mbabane
    {   84, 5015,     754,  8, }, // Africa/Mogadishu
    {   84, 6997,     762,  7, }, // Africa/Monrovia
    {   84, 3755,   34907,  7, }, // Africa/Nairobi
    {   84, 5173,   25836,  8, }, // Africa/Ndjamena
    {   84, 5256,   34914,  6, }, // Africa/Niamey
    {   84, 6766,     790,  7, }, // Africa/Nouakchott
    {   84, 6690,     797,  8, }, // Africa/Ouagadougou
    {   84, 5127,     805, 10, }, // Africa/Porto-Novo
    {   84, 4337,     815,  8, }, // Africa/Sao_Tome
    {   84, 2732,     823,  7, }, // Africa/Tripoli
    {   84, 6503,     830,  5, }, // Africa/Tunis
    {   84, 4054,     835,  7, }, // Africa/Windhoek
    {   84,  132,     842,  4, }, // America/Adak
    {   84, 3183,   50119,  8, }, // America/Anchorage
    {   84, 5524,   50127,  7, }, // America/Anguilla
    {   84, 5541,     860,  7, }, // America/Antigua
    {   84, 4456,     867,  9, }, // America/Araguaina
    {   84,  166,   50134, 12, }, // America/Argentina/Buenos_Aires
    {   84,   91,     888,  9, }, // America/Argentina/Catamarca
    {   84,  270,     897,  7, }, // America/Argentina/Cordoba
    {   84,  441,   25937,  5, }, // America/Argentina/Jujuy
    {   84, 7425,     909,  8, }, // America/Argentina/La_Rioja
    {   84,  565,     917,  7, }, // America/Argentina/Mendoza
    {   84, 7452,   50146, 12, }, // America/Argentina/Rio_Gallegos
    {   84, 7483,   25962,  6, }, // America/Argentina/Salta
    {   84, 7507,     940,  8, }, // America/Argentina/San_Juan
    {   84, 5497,     948,  8, }, // America/Argentina/San_Luis
    {   84, 7534,     956,  7, }, // America/Argentina/Tucuman
    {   84, 7560,   25976,  5, }, // America/Argentina/Ushuaia
    {   84, 5557,     969,  5, }, // America/Aruba
    {   84, 4154,   50158,  8, }, // America/Asuncion
    {   84,  237,     981, 12, }, // America/Atikokan
    {   84, 3581,     993,  3, }, // America/Bahia
    {   84, 7586,   50166, 16, }, // America/Bahia_Banderas
    {   84, 5571,    1013,  8, }, // America/Barbados
    {   84, 7609,    1021,  5, }, // America/Belem
    {   84, 5309,    1026,  5, }, // America/Belize
    {   84, 7623,    1031, 12, }, // America/Blanc-Sablon
    {   84, 7644,    1043,  9, }, // America/Boa_Vista
    {   84, 4263,    1052,  6, }, // America/Bogota
    {   84, 7662,   50182,  5, }, // America/Boise
    {   84, 7676,   50187, 13, }, // America/Cambridge_Bay
    {   84, 7698,    1075, 12, }, // America/Campo_Grande
    {   84, 3770,    1087,  6, }, // America/Cancun
    {   84, 4596,    1093,  7, }, // America/Caracas
    {   84, 4247,   50200,  7, }, // America/Cayenne
    {   84, 5398,   26079,  6, }, // America/Cayman
    {   84, 3251,    1129,  6, }, // America/Chicago
    {   84, 7719,    1135,  6, }, // America/Chihuahua
    {   84, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   84, 5324,    1141, 10, }, // America/Costa_Rica
    {   84, 7759,    1151,  7, }, // America/Creston
    {   84, 3693,   26138,  5, }, // America/Cuiaba
    {   84, 5640,   35086,  7, }, // America/Curacao
    {   84, 7775,    1172, 12, }, // America/Danmarkshavn
    {   84, 7796,    1184,  6, }, // America/Dawson
    {   84, 7811,    1190, 11, }, // America/Dawson_Creek
    {   84,  805,    1201,  6, }, // America/Denver
    {   84, 3351,   35129,  7, }, // America/Detroit
    {   84, 5656,    1214,  8, }, // America/Dominica
    {   84,  893,   50207,  8, }, // America/Edmonton
    {   84, 7832,   50215,  8, }, // America/Eirunepe
    {   84, 5343,   26215,  9, }, // America/El_Salvador
    {   84, 7849,   50223, 12, }, // America/Fort_Nelson
    {   84, 7869,    1257,  9, }, // America/Fortaleza
    {   84, 7887,   50235,  9, }, // America/Glace_Bay
    {   84, 6844,   50244,  7, }, // America/Goose_Bay
    {   84, 4514,   32749, 10, }, // America/Grand_Turk
    {   84, 5687,    1292,  7, }, // America/Grenada
    {   84, 5703,    1299,  9, }, // America/Guadeloupe
    {   84, 3662,    1308,  9, }, // America/Guatemala
    {   84, 4743,   50251,  8, }, // America/Guayaquil
    {   84, 6895,   50259,  6, }, // America/Guyana
    {   84, 1923,   50265,  8, }, // America/Halifax
    {   84, 2221,    1339,  6, }, // America/Havana
    {   84, 7905,   50273,  9, }, // America/Hermosillo
    {   84,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {   84,  481,    1365,  4, }, // America/Indiana/Knox
    {   84, 7924,    1378,  7, }, // America/Indiana/Marengo
    {   84, 7948,   32836, 10, }, // America/Indiana/Petersburg
    {   84, 7975,    1413,  9, }, // America/Indiana/Tell_City
    {   84, 8001,   50282,  5, }, // America/Indiana/Vevay
    {   84, 8023,   35219,  7, }, // America/Indiana/Vincennes
    {   84, 8049,   32907,  7, }, // America/Indiana/Winamac
    {   84, 8073,    1452,  6, }, // America/Inuvik
    {   84,  660,    1458,  7, }, // America/Iqaluit
    {   84, 2665,   26461,  6, }, // America/Jamaica
    {   84, 5282,   35226,  5, }, // America/Juneau
    {   84,  521,    1476,  8, }, // America/Kentucky/Louisville
    {   84, 8088,    1484, 10, }, // America/Kentucky/Monticello
    {   84, 5621,   35231, 10, }, // America/Kralendijk
    {   84, 4278,    1514,  6, }, // America/La_Paz
    {   84, 7132,    1520,  4, }, // America/Lima
    {   84, 3390,   50287, 12, }, // America/Los_Angeles
    {   84, 5849,   50299, 21, }, // America/Lower_Princes
    {   84, 8116,   35279,  6, }, // America/Maceio
    {   84, 8131,    1559,  7, }, // America/Managua
    {   84, 1892,    1566,  6, }, // America/Manaus
    {   84, 5814,    1572,  6, }, // America/Marigot
    {   84, 5722,    1578,  9, }, // America/Martinique
    {   84, 8147,    1587,  9, }, // America/Matamoros
    {   84, 2779,    1596,  8, }, // America/Mazatlan
    {   84, 8165,    1604,  8, }, // America/Menominee
    {   84, 8183,    1612,  6, }, // America/Merida
    {   84, 8198,    1618, 10, }, // America/Metlakatla
    {   84, 2811,    1628,  6, }, // America/Mexico_City
    {   84, 4293,    1634,  7, }, // America/Miquelon
    {   84, 8217,    1641,  7, }, // America/Moncton
    {   84, 8233,   50320,  9, }, // America/Monterrey
    {   84, 4000,    1657, 10, }, // America/Montevideo
    {   84, 5741,   50329,  9, }, // America/Montserrat
    {   84, 5383,    1677,  6, }, // America/Nassau
    {   84, 3294,   50338,  8, }, // America/New_York
    {   84, 8251,    1691,  3, }, // America/Nome
    {   84, 1834,   50346,  7, }, // America/Noronha
    {   84, 8264,   50353, 24, }, // America/North_Dakota/Beulah
    {   84, 8292,   50377, 22, }, // America/North_Dakota/Center
    {   84, 8320,   50399,  9, }, // America/North_Dakota/New_Salem
    {   84,  393,   50408,  6, }, // America/Nuuk
    {   84, 8351,    1776,  7, }, // America/Ojinaga
    {   84, 5413,    1783,  6, }, // America/Panama
    {   84, 6106,    1789, 10, }, // America/Paramaribo
    {   84, 3224,    1799,  6, }, // America/Phoenix
    {   84, 3847,    1805, 12, }, // America/Port-au-Prince
    {   84, 5871,   26771, 13, }, // America/Port_of_Spain
    {   84, 8367,   26784, 11, }, // America/Porto_Velho
    {   84, 5760,   50414, 11, }, // America/Puerto_Rico
    {   84, 3932,    1852, 12, }, // America/Punta_Arenas
    {   84, 8387,    1864, 12, }, // America/Rankin_Inlet
    {   84, 8408,    1876,  6, }, // America/Recife
    {   84, 1979,   50425,  8, }, // America/Regina
    {   84, 8423,   35372,  7, }, // America/Resolute
    {   84,  695,    1897, 10, }, // America/Rio_Branco
    {   84, 8440,    1907,  8, }, // America/Santarem
    {   84, 2165,   50433,  8, }, // America/Santiago
    {   84, 6044,    1923, 13, }, // America/Santo_Domingo
    {   84, 1862,    1936,  9, }, // America/Sao_Paulo
    {   84, 6862,    1945, 11, }, // America/Scoresbysund
    {   84, 8457,    1956,  5, }, // America/Sitka
    {   84, 8471,   50441, 13, }, // America/St_Barthelemy
    {   84, 2045,   26931, 10, }, // America/St_Johns
    {   84, 5780,    1983,  9, }, // America/St_Kitts
    {   84, 5797,   50454,  9, }, // America/St_Lucia
    {   84,  855,    2003, 10, }, // America/St_Thomas
    {   84, 5830,    2013, 12, }, // America/St_Vincent
    {   84, 8493,   50463, 12, }, // America/Swift_Current
    {   84, 5363,   50475, 12, }, // America/Tegucigalpa
    {   84, 5673,    2049,  4, }, // America/Thule
    {   84,  313,    2053,  7, }, // America/Tijuana
    {   84,  608,    2060,  7, }, // America/Toronto
    {   84, 5605,    2067,  7, }, // America/Tortola
    {   84, 2077,    2074,  8, }, // America/Vancouver
    {   84, 2128,   27017,  8, }, // America/Whitehorse
    {   84,  734,    2090,  8, }, // America/Winnipeg
    {   84, 8515,   27032,  6, }, // America/Yakutat
    {   84, 5918,   35501,  5, }, // Antarctica/Casey
    {   84, 6027,   35506,  6, }, // Antarctica/Davis
    {   84, 6066,   50487, 14, }, // Antarctica/DumontDUrville
    {   84, 8531,    2131,  8, }, // Antarctica/Macquarie
    {   84, 7029,    2139,  6, }, // Antarctica/Mawson
    {   84, 7076,    2145,  9, }, // Antarctica/McMurdo
    {   84, 8552,    2154,  6, }, // Antarctica/Palmer
    {   84, 7189,    2160,  6, }, // Antarctica/Rothera
    {   84, 7258,   27092,  4, }, // Antarctica/Syowa
    {   84, 8570,   50501,  6, }, // Antarctica/Troll
    {   84, 7337,   33428,  6, }, // Antarctica/Vostok
    {   84, 1413,   50507,  7, }, // Arctic/Longyearbyen
    {   84, 5487,    2192,  4, }, // Asia/Aden
    {   84, 5297,   50514,  8, }, // Asia/Almaty
    {   84, 3870,   50522,  6, }, // Asia/Amman
    {   84, 5428,   50528,  7, }, // Asia/Anadyr
    {   84, 5440,    2213,  5, }, // Asia/Aqtau
    {   84, 5270,    2218,  6, }, // Asia/Aqtobe
    {   84,  964,    2224,  7, }, // Asia/Ashgabat
    {   84, 8587,   50535,  6, }, // Asia/Atyrau
    {   84, 3509,   50541,  7, }, // Asia/Baghdad
    {   84, 5451,   35560,  7, }, // Asia/Bahrain
    {   84, 3555,   50548,  5, }, // Asia/Baku
    {   84, 4368,    2254,  7, }, // Asia/Bangkok
    {   84, 3473,    2261,  7, }, // Asia/Barnaul
    {   84, 3988,   27145,  6, }, // Asia/Beirut
    {   84, 3680,    2274,  6, }, // Asia/Bishkek
    {   84, 5906,   27157,  6, }, // Asia/Brunei
    {   84, 4503,    2286,  4, }, // Asia/Chita
    {   84, 5963,   35567,  9, }, // Asia/Choibalsan
    {   84, 4413,    2299,  7, }, // Asia/Colombo
    {   84, 4442,   50553,  7, }, // Asia/Damascus
    {   84, 1060,    2312,  5, }, // Asia/Dhaka
    {   84, 6125,    2317,  4, }, // Asia/Dili
    {   84, 3498,   27187,  5, }, // Asia/Dubai
    {   84, 6092,    2326,  7, }, // Asia/Dushanbe
    {   84, 8599,    2333,  9, }, // Asia/Famagusta
    {   84, 8614,    2342,  4, }, // Asia/Gaza
    {   84, 4697,    2346,  6, }, // Asia/Hebron
    {   84, 1226,    2352,  7, }, // Asia/Ho_Chi_Minh
    {   84, 2576,   50560,  7, }, // Asia/Hong_Kong
    {   84, 4673,    2366,  4, }, // Asia/Hovd
    {   84, 4086,    2370,  7, }, // Asia/Irkutsk
    {   84, 6969,   27229,  8, }, // Asia/Jakarta
    {   84, 6955,   50567,  7, }, // Asia/Jayapura
    {   84, 1257,   50574, 11, }, // Asia/Jerusalem
    {   84, 3462,    2401,  5, }, // Asia/Kabul
    {   84, 4218,   50585, 23, }, // Asia/Kamchatka
    {   84, 4141,    2431,  6, }, // Asia/Karachi
    {   84, 1152,    2437,  8, }, // Asia/Kathmandu
    {   84, 8624,    2445,  7, }, // Asia/Khandyga
    {   84,  992,   50608,  9, }, // Asia/Kolkata
    {   84, 4099,   35599, 10, }, // Asia/Krasnoyarsk
    {   84, 4761,    2471, 12, }, // Asia/Kuala_Lumpur
    {   84, 5893,    2483,  6, }, // Asia/Kuching
    {   84, 5464,   27284,  6, }, // Asia/Kuwait
    {   84, 1178,    2495,  5, }, // Asia/Macau
    {   84, 3919,    2500,  7, }, // Asia/Magadan
    {   84, 1316,    2507,  7, }, // Asia/Makassar
    {   84, 7145,    2514,  6, }, // Asia/Manila
    {   84, 6883,    2520,  6, }, // Asia/Muscat
    {   84, 2446,    2526,  6, }, // Asia/Nicosia
    {   84, 8638,    2533, 11, }, // Asia/Novokuznetsk
    {   84, 4037,    2544, 11, }, // Asia/Novosibirsk
    {   84, 4131,    2555,  4, }, // Asia/Omsk
    {   84, 7108,   33618,  4, }, // Asia/Oral
    {   84, 6924,   50617,  8, }, // Asia/Phnom_Penh
    {   84, 8656,    2572,  9, }, // Asia/Pontianak
    {   84, 4116,   27354,  7, }, // Asia/Pyongyang
    {   84, 5476,    2588,  5, }, // Asia/Qatar
    {   84, 4171,    2601,  9, }, // Asia/Qyzylorda
    {   84, 3486,   50625,  7, }, // Asia/Riyadh
    {   84, 4310,   50632, 12, }, // Asia/Sakhalin
    {   84, 7208,    2630,  9, }, // Asia/Samarkand
    {   84, 3116,    2639,  4, }, // Asia/Seoul
    {   84, 1020,   27392,  6, }, // Asia/Shanghai
    {   84, 3137,    2649,  8, }, // Asia/Singapore
    {   84, 4199,    2657, 13, }, // Asia/Srednekolymsk
    {   84, 3100,   27420,  6, }, // Asia/Taipei
    {   84, 4683,    2676,  7, }, // Asia/Tashkent
    {   84, 3817,   33686,  7, }, // Asia/Tbilisi
    {   84, 2638,   50644,  7, }, // Asia/Tehran
    {   84, 1284,   33700,  5, }, // Asia/Thimphu
    {   84, 2687,    2701,  5, }, // Asia/Tokyo
    {   84, 4474,    2706,  5, }, // Asia/Tomsk
    {   84, 1346,    2711, 10, }, // Asia/Ulaanbaatar
    {   84, 1126,    2721,  6, }, // Asia/Urumqi
    {   84, 8685,   50651,  9, }, // Asia/Ust-Nera
    {   84, 6940,   50660,  8, }, // Asia/Vientiane
    {   84, 4612,    2742, 11, }, // Asia/Vladivostok
    {   84, 4730,   35672,  6, }, // Asia/Yakutsk
    {   84, 1202,   27502,  6, }, // Asia/Yangon
    {   84, 3785,    2766, 12, }, // Asia/Yekaterinburg
    {   84, 3649,    2778,  6, }, // Asia/Yerevan
    {   84, 3565,   50668, 16, }, // Atlantic/Azores
    {   84, 5588,   50684,  8, }, // Atlantic/Bermuda
    {   84, 6591,   50692, 17, }, // Atlantic/Canary
    {   84, 3629,    2835, 10, }, // Atlantic/Cape_Verde
    {   84, 1379,   50709, 18, }, // Atlantic/Faroe
    {   84, 8699,   50727, 13, }, // Atlantic/Madeira
    {   84, 2614,   50740,  9, }, // Atlantic/Reykjavik
    {   84, 7235,   50749, 14, }, // Atlantic/South_Georgia
    {   84, 6784,   50763, 19, }, // Atlantic/St_Helena
    {   84, 6607,   50782,  6, }, // Atlantic/Stanley
    {   84, 1654,    2926,  8, }, // Australia/Adelaide
    {   84, 1619,    2934,  7, }, // Australia/Brisbane
    {   84, 1783,    2941, 11, }, // Australia/Broken_Hill
    {   84, 1567,    2952,  6, }, // Australia/Darwin
    {   84, 3539,   27699,  4, }, // Australia/Eucla
    {   84, 1500,    2963,  6, }, // Australia/Hobart
    {   84, 8716,    2969,  8, }, // Australia/Lindeman
    {   84, 1531,   50788, 14, }, // Australia/Lord_Howe
    {   84, 1711,   27711,  8, }, // Australia/Melbourne
    {   84, 1746,    2992,  4, }, // Australia/Perth
    {   84, 1447,   50802,  6, }, // Australia/Sydney
    {   84, 6390,    3002,  9, }, // Europe/Amsterdam
    {   84, 6149,    3011,  7, }, // Europe/Andorra
    {   84, 3522,   50808,  9, }, // Europe/Astrakhan
    {   84, 6577,   50817,  5, }, // Europe/Athens
    {   84, 6285,    3030,  7, }, // Europe/Belgrade
    {   84, 4659,    3037,  6, }, // Europe/Berlin
    {   84, 6437,    3043, 10, }, // Europe/Bratislava
    {   84, 6178,   50822,  8, }, // Europe/Brussels
    {   84, 3830,    3061,  8, }, // Europe/Bucharest
    {   84, 3708,    3069,  8, }, // Europe/Budapest
    {   84, 8751,   50830, 19, }, // Europe/Busingen
    {   84, 2475,    3100,  7, }, // Europe/Chisinau
    {   84, 6238,   50849, 10, }, // Europe/Copenhagen
    {   84, 2260,    3117,  6, }, // Europe/Dublin
    {   84, 6256,    3123,  9, }, // Europe/Gibraltar
    {   84, 8767,    3132,  6, }, // Europe/Guernsey
    {   84, 6561,   50859,  9, }, // Europe/Helsinki
    {   84, 8783,   50868,  9, }, // Europe/Isle_of_Man
    {   84, 1097,    3155,  7, }, // Europe/Istanbul
    {   84, 8802,   50877,  6, }, // Europe/Jersey
    {   84, 3881,    3167, 11, }, // Europe/Kaliningrad
    {   84, 8924,    3178,  5, }, // Europe/Kirov
    {   84, 2419,    3183,  4, }, // Europe/Kyiv
    {   84, 3078,    3187,  8, }, // Europe/Lisbon
    {   84, 6455,   27891,  7, }, // Europe/Ljubljana
    {   84, 2393,    3204,  6, }, // Europe/London
    {   84, 6314,   27904, 10, }, // Europe/Luxembourg
    {   84, 4794,    3221,  6, }, // Europe/Madrid
    {   84, 6346,   27920,  6, }, // Europe/Malta
    {   84, 6531,    3232,  9, }, // Europe/Mariehamn
    {   84, 3595,    3241,  5, }, // Europe/Minsk
    {   84, 6359,    3246,  6, }, // Europe/Monaco
    {   84, 3443,    3252,  6, }, // Europe/Moscow
    {   84, 6407,    3258,  4, }, // Europe/Oslo
    {   84, 4186,    3262,  5, }, // Europe/Paris
    {   84, 6373,    3267,  9, }, // Europe/Podgorica
    {   84, 6224,    3276,  5, }, // Europe/Prague
    {   84, 8816,    3281,  4, }, // Europe/Riga
    {   84, 6273,    3285,  3, }, // Europe/Rome
    {   84, 4233,    3288,  6, }, // Europe/Samara
    {   84, 6419,    3294, 10, }, // Europe/San_Marino
    {   84, 6194,    3304,  7, }, // Europe/Sarajevo
    {   84, 4353,    3311,  7, }, // Europe/Saratov
    {   84, 8828,   50883, 11, }, // Europe/Simferopol
    {   84, 6332,   50894,  6, }, // Europe/Skopje
    {   84, 6548,    3333,  4, }, // Europe/Sofia
    {   84, 6472,   50900,  9, }, // Europe/Stockholm
    {   84, 8846,    3346,  6, }, // Europe/Tallinn
    {   84, 6135,    3352,  6, }, // Europe/Tirane
    {   84, 8861,   50909,  9, }, // Europe/Ulyanovsk
    {   84, 6301,    3367,  5, }, // Europe/Vaduz
    {   84, 6516,    3372,  7, }, // Europe/Vatican
    {   84, 6164,    3379,  4, }, // Europe/Vienna
    {   84, 8878,   50918,  7, }, // Europe/Vilnius
    {   84, 4629,    3390,  9, }, // Europe/Volgograd
    {   84, 3055,    3399,  7, }, // Europe/Warsaw
    {   84, 6210,    3406,  6, }, // Europe/Zagreb
    {   84, 6489,   35931,  5, }, // Europe/Zurich
    {   84, 4980,    3418, 12, }, // Indian/Antananarivo
    {   84, 6910,    3430,  5, }, // Indian/Chagos
    {   84, 5979,   50925, 31, }, // Indian/Christmas
    {   84, 5996,   50956, 16, }, // Indian/Cocos
    {   84, 4931,   50972, 18, }, // Indian/Comoro
    {   84, 6624,    3469,  8, }, // Indian/Kerguelen
    {   84, 7223,   28153,  3, }, // Indian/Mahe
    {   84, 7013,   50990,  9, }, // Indian/Maldives
    {   84, 3971,    3486,  7, }, // Indian/Mauritius
    {   84, 5000,   50999,  7, }, // Indian/Mayotte
    {   84, 7174,   51006,  7, }, // Indian/Reunion
    {   84, 4324,    3507,  3, }, // Pacific/Apia
    {   84,  932,    3511,  6, }, // Pacific/Auckland
    {   84, 3608,   51013, 15, }, // Pacific/Bougainville
    {   84, 2863,   51028, 11, }, // Pacific/Chatham
    {   84, 3022,   51039, 12, }, // Pacific/Chuuk
    {   84, 2201,   51051, 16, }, // Pacific/Easter
    {   84, 7323,   51067,  5, }, // Pacific/Efate
    {   84, 7290,    3550,  7, }, // Pacific/Fakaofo
    {   84, 3804,   36028,  5, }, // Pacific/Fiji
    {   84, 7306,    3561,  8, }, // Pacific/Funafuti
    {   84, 6641,   51072, 21, }, // Pacific/Galapagos
    {   84, 6659,   51093, 15, }, // Pacific/Gambier
    {   84, 3724,   34199, 11, }, // Pacific/Guadalcanal
    {   84, 5935,    3615,  4, }, // Pacific/Guam
    {   84, 2929,   51108,  8, }, // Pacific/Honolulu
    {   84, 2897,   51116, 16, }, // Pacific/Kanton
    {   84, 3900,    3625, 10, }, // Pacific/Kiritimati
    {   84, 6982,    3635,  6, }, // Pacific/Kosrae
    {   84, 2708,   28300, 10, }, // Pacific/Kwajalein
    {   84, 4779,   36063,  7, }, // Pacific/Majuro
    {   84, 3953,   51132, 16, }, // Pacific/Marquesas
    {   84, 8909,   51148, 14, }, // Pacific/Midway
    {   84, 7047,    3689,  5, }, // Pacific/Nauru
    {   84, 7095,   51162,  4, }, // Pacific/Niue
    {   84, 4070,    3698,  7, }, // Pacific/Norfolk
    {   84, 7061,    3705,  5, }, // Pacific/Noumea
    {   84, 2991,    3710,  9, }, // Pacific/Pago_Pago
    {   84, 7118,    3719,  5, }, // Pacific/Palau
    {   84, 7157,    3724,  7, }, // Pacific/Pitcairn
    {   84, 2961,   51166, 14, }, // Pacific/Pohnpei
    {   84, 4709,    3737, 11, }, // Pacific/Port_Moresby
    {   84, 6009,    3748,  9, }, // Pacific/Rarotonga
    {   84, 5948,   28394,  6, }, // Pacific/Saipan
    {   84, 7275,   51180, 13, }, // Pacific/Tahiti
    {   84, 6675,    3768,  6, }, // Pacific/Tarawa
    {   84, 4485,    3774,  9, }, // Pacific/Tongatapu
    {   84, 7355,   51193, 10, }, // Pacific/Wake
    {   84, 7368,    3787,  6, }, // Pacific/Wallis
    {   85, 6751,   51203,  4, }, // Africa/Abidjan Cherokee/Cherokee/United States
    {   85, 6723,   51207,  2, }, // Africa/Accra
    {   85, 4961,   51209,  7, }, // Africa/Addis_Ababa
    {   85, 7383,   51216,  4, }, // Africa/Algiers
    {   85,   14,   51220,  4, }, // Africa/Asmara
    {   85,   44,   51224,  3, }, // Africa/Bamako
    {   85, 5159,   51227,  4, }, // Africa/Bangui
    {   85, 6709,   51231,  4, }, // Africa/Banjul
    {   85, 7398,   51235,  3, }, // Africa/Bissau
    {   85, 4859,   51238,  3, }, // Africa/Blantyre
    {   85, 5189,   51241,  4, }, // Africa/Brazzaville
    {   85, 4824,   51245,  5, }, // Africa/Bujumbura
    {   85, 2242,   51250,  3, }, // Africa/Cairo
    {   85, 4019,   51253,  8, }, // Africa/Casablanca
    {   85, 7412,   51261,  2, }, // Africa/Ceuta
    {   85, 6736,   51263,  3, }, // Africa/Conakry
    {   85, 6803,   51266,  3, }, // Africa/Dakar
    {   85, 5032,   51269,  8, }, // Africa/Dar_es_Salaam
    {   85, 4945,   51277,  3, }, // Africa/Djibouti
    {   85, 5145,   51280,  4, }, // Africa/Douala
    {   85, 5068,   51284,  7, }, // Africa/El_Aaiun
    {   85, 6816,   51291,  7, }, // Africa/Freetown
    {   85, 4808,   51298,  4, }, // Africa/Gaborone
    {   85, 4917,   51302,  3, }, // Africa/Harare
    {   85, 4381,   51305,  6, }, // Africa/Johannesburg
    {   85, 4401,   51311,  2, }, // Africa/Juba
    {   85, 5053,   51313,  4, }, // Africa/Kampala
    {   85, 4426,   51317,  3, }, // Africa/Khartoum
    {   85, 4875,   51320,  3, }, // Africa/Kigali
    {   85, 5208,   51323,  4, }, // Africa/Kinshasa
    {   85, 4646,   51327,  3, }, // Africa/Lagos
    {   85, 5238,   51330,  4, }, // Africa/Libreville
    {   85, 6832,   51334,  2, }, // Africa/Lome
    {   85, 5113,   51336,  4, }, // Africa/Luanda
    {   85, 4841,   51340,  4, }, // Africa/Lubumbashi
    {   85, 4903,   51344,  3, }, // Africa/Lusaka
    {   85, 5224,   51347,  3, }, // Africa/Malabo
    {   85, 4889,   51350,  3, }, // Africa/Maputo
    {   85, 5099,   51353,  3, }, // Africa/Maseru
    {   85, 5084,   51356,  3, }, // Africa/Mbabane
    {   85, 5015,   51359,  4, }, // Africa/Mogadishu
    {   85, 6997,   51363,  5, }, // Africa/Monrovia
    {   85, 3755,   51368,  4, }, // Africa/Nairobi
    {   85, 5173,   51372,  4, }, // Africa/Ndjamena
    {   85, 5256,   51376,  3, }, // Africa/Niamey
    {   85, 6766,   51379,  4, }, // Africa/Nouakchott
    {   85, 6690,   51383,  5, }, // Africa/Ouagadougou
    {   85, 5127,   51388, 11, }, // Africa/Porto-Novo
    {   85, 4337,   51399,  4, }, // Africa/Sao_Tome
    {   85, 2732,   51403,  3, }, // Africa/Tripoli
    {   85, 6503,   51406,  3, }, // Africa/Tunis
    {   85, 4054,   51409,  5, }, // Africa/Windhoek
    {   85,  132,   51414,  3, }, // America/Adak
    {   85, 3183,   51417,  5, }, // America/Anchorage
    {   85, 5524,   51422,  4, }, // America/Anguilla
    {   85, 5541,   51426,  3, }, // America/Antigua
    {   85, 4456,   51429,  6, }, // America/Araguaina
    {   85,  166,   51435,  7, }, // America/Argentina/Buenos_Aires
    {   85,   91,   51442,  4, }, // America/Argentina/Catamarca
    {   85,  270,   51446,  3, }, // America/Argentina/Cordoba
    {   85,  441,   51449,  3, }, // America/Argentina/Jujuy
    {   85, 7425,   51452,  5, }, // America/Argentina/La_Rioja
    {   85,  565,   51457,  4, }, // America/Argentina/Mendoza
    {   85, 7452,   51461,  8, }, // America/Argentina/Rio_Gallegos
    {   85, 7483,   51469,  3, }, // America/Argentina/Salta
    {   85, 7507,   51472,  5, }, // America/Argentina/San_Juan
    {   85, 5497,   51477,  8, }, // America/Argentina/San_Luis
    {   85, 7534,   51485,  4, }, // America/Argentina/Tucuman
    {   85, 7560,   51489,  4, }, // America/Argentina/Ushuaia
    {   85, 5557,   51493,  3, }, // America/Aruba
    {   85, 4154,   51496,  5, }, // America/Asuncion
    {   85,  237,   51501,  5, }, // America/Atikokan
    {   85, 3581,   51506,  3, }, // America/Bahia
    {   85, 7586,   51509,  9, }, // America/Bahia_Banderas
    {   85, 5571,   51518,  4, }, // America/Barbados
    {   85, 7609,   51522,  3, }, // America/Belem
    {   85, 5309,   51525,  3, }, // America/Belize
    {   85, 7623,   51528,  6, }, // America/Blanc-Sablon
    {   85, 7644,   51534,  9, }, // America/Boa_Vista
    {   85, 4263,   51543,  3, }, // America/Bogota
    {   85, 7662,   51546,  3, }, // America/Boise
    {   85, 7676,   51549, 10, }, // America/Cambridge_Bay
    {   85, 7698,   51559,  7, }, // America/Campo_Grande
    {   85, 3770,   51566,  4, }, // America/Cancun
    {   85, 4596,   51570,  4, }, // America/Caracas
    {   85, 4247,   51574,  3, }, // America/Cayenne
    {   85, 5398,   51577,  4, }, // America/Cayman
    {   85, 3251,   51581,  3, }, // America/Chicago
    {   85, 7719,   51584,  3, }, // America/Chihuahua
    {   85, 7737,   51587,  7, }, // America/Ciudad_Juarez
    {   85, 5324,   51594,  5, }, // America/Costa_Rica
    {   85, 7759,   51599,  4, }, // America/Creston
    {   85, 3693,   51603,  3, }, // America/Cuiaba
    {   85, 5640,   51606,  4, }, // America/Curacao
    {   85, 7775,   51610,  7, }, // America/Danmarkshavn
    {   85, 7796,   51617,  3, }, // America/Dawson
    {   85, 7811,   51620,  7, }, // America/Dawson_Creek
    {   85,  805,   51627,  8, }, // America/Denver
    {   85, 3351,   51635,  3, }, // America/Detroit
    {   85, 5656,   51638,  4, }, // America/Dominica
    {   85,  893,   51642,  6, }, // America/Edmonton
    {   85, 7832,   51648,  4, }, // America/Eirunepe
    {   85, 5343,   51652,  8, }, // America/El_Salvador
    {   85, 7849,   51660,  8, }, // America/Fort_Nelson
    {   85, 7869,   51668,  4, }, // America/Fortaleza
    {   85, 7887,   51672,  8, }, // America/Glace_Bay
    {   85, 6844,   51680,  8, }, // America/Goose_Bay
    {   85, 4514,   51688,  8, }, // America/Grand_Turk
    {   85, 5687,   51696,  3, }, // America/Grenada
    {   85, 5703,   51699,  4, }, // America/Guadeloupe
    {   85, 3662,   51703,  4, }, // America/Guatemala
    {   85, 4743,   51707,  4, }, // America/Guayaquil
    {   85, 6895,   51711,  3, }, // America/Guyana
    {   85, 1923,   51714,  5, }, // America/Halifax
    {   85, 2221,   51719,  3, }, // America/Havana
    {   85, 7905,   51722,  4, }, // America/Hermosillo
    {   85,  348,   51726,  9, }, // America/Indiana/Indianapolis
    {   85,  481,   51735, 10, }, // America/Indiana/Knox
    {   85, 7924,   51745, 11, }, // America/Indiana/Marengo
    {   85, 7948,   51756, 11, }, // America/Indiana/Petersburg
    {   85, 7975,   51767, 13, }, // America/Indiana/Tell_City
    {   85, 8001,   51780,  9, }, // America/Indiana/Vevay
    {   85, 8023,   51789, 12, }, // America/Indiana/Vincennes
    {   85, 8049,   51801, 11, }, // America/Indiana/Winamac
    {   85, 8073,   51812,  4, }, // America/Inuvik
    {   85,  660,   51816,  5, }, // America/Iqaluit
    {   85, 2665,   51821,  4, }, // America/Jamaica
    {   85, 5282,   51825,  2, }, // America/Juneau
    {   85,  521,   51827,  4, }, // America/Kentucky/Louisville
    {   85, 8088,   51831, 11, }, // America/Kentucky/Monticello
    {   85, 5621,   51842,  4, }, // America/Kralendijk
    {   85, 4278,   51846,  2, }, // America/La_Paz
    {   85, 7132,   51830,  2, }, // America/Lima
    {   85, 3390,   51848, 12, }, // America/Los_Angeles
    {   85, 5849,   51860, 17, }, // America/Lower_Princes
    {   85, 8116,   51877,  3, }, // America/Maceio
    {   85, 8131,   51880,  3, }, // America/Managua
    {   85, 1892,   51883,  4, }, // America/Manaus
    {   85, 5814,   51887,  4, }, // America/Marigot
    {   85, 5722,   51891,  4, }, // America/Martinique
    {   85, 8147,   51895,  5, }, // America/Matamoros
    {   85, 2779,   51900,  4, }, // America/Mazatlan
    {   85, 8165,   51904,  4, }, // America/Menominee
    {   85, 8183,   51908,  3, }, // America/Merida
    {   85, 8198,   51911,  4, }, // America/Metlakatla
    {   85, 2811,   51915,  9, }, // America/Mexico_City
    {   85, 4293,   51924,  4, }, // America/Miquelon
    {   85, 8217,   51928,  4, }, // America/Moncton
    {   85, 8233,   51932,  3, }, // America/Monterrey
    {   85, 4000,   51935,  6, }, // America/Montevideo
    {   85, 5741,   51941,  6, }, // America/Montserrat
    {   85, 5383,   51947,  3, }, // America/Nassau
    {   85, 3294,   51950,  3, }, // America/New_York
    {   85, 8251,   51953,  2, }, // America/Nome
    {   85, 1834,   51955,  4, }, // America/Noronha
    {   85, 8264,   51959, 12, }, // America/North_Dakota/Beulah
    {   85, 8292,   51971, 12, }, // America/North_Dakota/Center
    {   85, 8320,   51983, 15, }, // America/North_Dakota/New_Salem
    {   85,  393,   51998,  2, }, // America/Nuuk
    {   85, 8351,   52000,  4, }, // America/Ojinaga
    {   85, 5413,   52004,  3, }, // America/Panama
    {   85, 6106,   52007,  5, }, // America/Paramaribo
    {   85, 3224,   52012,  6, }, // America/Phoenix
    {   85, 3847,   52018, 17, }, // America/Port-au-Prince
    {   85, 5871,   52035, 15, }, // America/Port_of_Spain
    {   85, 8367,   52050, 12, }, // America/Porto_Velho
    {   85, 5760,   52062,  7, }, // America/Puerto_Rico
    {   85, 3932,   52069,  7, }, // America/Punta_Arenas
    {   85, 8387,   52076,  9, }, // America/Rankin_Inlet
    {   85, 8408,   52085,  3, }, // America/Recife
    {   85, 1979,   52088,  3, }, // America/Regina
    {   85, 8423,   52091,  4, }, // America/Resolute
    {   85,  695,   52095,  7, }, // America/Rio_Branco
    {   85, 8440,   52102,  5, }, // America/Santarem
    {   85, 2165,   52107,  5, }, // America/Santiago
    {   85, 6044,   52112,  7, }, // America/Santo_Domingo
    {   85, 1862,   52119,  5, }, // America/Sao_Paulo
    {   85, 6862,   52124,  6, }, // America/Scoresbysund
    {   85, 8457,   52130,  3, }, // America/Sitka
    {   85, 8471,   52133, 10, }, // America/St_Barthelemy
    {   85, 2045,   52143, 11, }, // America/St_Johns
    {   85, 5780,   52154,  8, }, // America/St_Kitts
    {   85, 5797,   52162,  8, }, // America/St_Lucia
    {   85,  855,   52170,  8, }, // America/St_Thomas
    {   85, 5830,   52178,  9, }, // America/St_Vincent
    {   85, 8493,   52187,  8, }, // America/Swift_Current
    {   85, 5363,   52195,  6, }, // America/Tegucigalpa
    {   85, 5673,   52201,  2, }, // America/Thule
    {   85,  313,   52203,  4, }, // America/Tijuana
    {   85,  608,   52207,  4, }, // America/Toronto
    {   85, 5605,   52211,  3, }, // America/Tortola
    {   85, 2077,   52214,  5, }, // America/Vancouver
    {   85, 2128,   52219,  7, }, // America/Whitehorse
    {   85,  734,   52226,  4, }, // America/Winnipeg
    {   85, 8515,   52230,  4, }, // America/Yakutat
    {   85, 5918,   51564,  2, }, // Antarctica/Casey
    {   85, 6027,   52234,  3, }, // Antarctica/Davis
    {   85, 6066,   52237, 11, }, // Antarctica/DumontDUrville
    {   85, 8531,   52248,  3, }, // Antarctica/Macquarie
    {   85, 7029,   52251,  3, }, // Antarctica/Mawson
    {   85, 7076,   52254,  4, }, // Antarctica/McMurdo
    {   85, 8552,   52258,  4, }, // Antarctica/Palmer
    {   85, 7189,   52262,  3, }, // Antarctica/Rothera
    {   85, 7258,   52265,  3, }, // Antarctica/Syowa
    {   85, 8570,   52268,  2, }, // Antarctica/Troll
    {   85, 7337,   52270,  4, }, // Antarctica/Vostok
    {   85, 1413,   52274, 17, }, // Arctic/Longyearbyen
    {   85, 5487,   52291,  3, }, // Asia/Aden
    {   85, 5297,   52294,  4, }, // Asia/Almaty
    {   85, 3870,   52298,  3, }, // Asia/Amman
    {   85, 5428,   52301,  4, }, // Asia/Anadyr
    {   85, 5440,   52305,  4, }, // Asia/Aqtau
    {   85, 5270,   52309,  4, }, // Asia/Aqtobe
    {   85,  964,   52313,  5, }, // Asia/Ashgabat
    {   85, 8587,   52318,  4, }, // Asia/Atyrau
    {   85, 3509,   52322,  4, }, // Asia/Baghdad
    {   85, 5451,   52326,  5, }, // Asia/Bahrain
    {   85, 3555,   51605,  2, }, // Asia/Baku
    {   85, 4368,   52331,  5, }, // Asia/Bangkok
    {   85, 3473,   52336,  4, }, // Asia/Barnaul
    {   85, 3988,   52340,  3, }, // Asia/Beirut
    {   85, 3680,   52343,  4, }, // Asia/Bishkek
    {   85, 5906,   52347,  3, }, // Asia/Brunei
    {   85, 4503,   52350,  2, }, // Asia/Chita
    {   85, 5963,   52352,  6, }, // Asia/Choibalsan
    {   85, 4413,   52358,  4, }, // Asia/Colombo
    {   85, 4442,   52362,  5, }, // Asia/Damascus
    {   85, 1060,   51266,  2, }, // Asia/Dhaka
    {   85, 6125,   52303,  2, }, // Asia/Dili
    {   85, 3498,   52367,  3, }, // Asia/Dubai
    {   85, 6092,   52370,  4, }, // Asia/Dushanbe
    {   85, 8599,   52374,  5, }, // Asia/Famagusta
    {   85, 8614,   52379,  2, }, // Asia/Gaza
    {   85, 4697,   52381,  3, }, // Asia/Hebron
    {   85, 1226,   52384, 10, }, // Asia/Ho_Chi_Minh
    {   85, 2576,   52394,  7, }, // Asia/Hong_Kong
    {   85, 4673,   52401,  3, }, // Asia/Hovd
    {   85, 4086,   52404,  4, }, // Asia/Irkutsk
    {   85, 6969,   52408,  3, }, // Asia/Jakarta
    {   85, 6955,   52411,  4, }, // Asia/Jayapura
    {   85, 1257,   52415,  5, }, // Asia/Jerusalem
    {   85, 3462,   52420,  3, }, // Asia/Kabul
    {   85, 4218,   52423,  4, }, // Asia/Kamchatka
    {   85, 4141,   52427,  3, }, // Asia/Karachi
    {   85, 1152,   52430,  5, }, // Asia/Kathmandu
    {   85, 8624,   52435,  4, }, // Asia/Khandyga
    {   85,  992,   52439,  4, }, // Asia/Kolkata
    {   85, 4099,   52443,  6, }, // Asia/Krasnoyarsk
    {   85, 4761,   52449,  7, }, // Asia/Kuala_Lumpur
    {   85, 5893,   52456,  4, }, // Asia/Kuching
    {   85, 5464,   52460,  3, }, // Asia/Kuwait
    {   85, 1178,   52463,  3, }, // Asia/Macau
    {   85, 3919,   52466,  4, }, // Asia/Magadan
    {   85, 1316,   52470,  4, }, // Asia/Makassar
    {   85, 7145,   52474,  3, }, // Asia/Manila
    {   85, 6883,   52477,  4, }, // Asia/Muscat
    {   85, 2446,   52481,  4, }, // Asia/Nicosia
    {   85, 8638,   52485,  7, }, // Asia/Novokuznetsk
    {   85, 4037,   52492,  6, }, // Asia/Novosibirsk
    {   85, 4131,   52498,  4, }, // Asia/Omsk
    {   85, 7108,   52502,  3, }, // Asia/Oral
    {   85, 6924,   52505,  5, }, // Asia/Phnom_Penh
    {   85, 8656,   52510,  6, }, // Asia/Pontianak
    {   85, 4116,   52516,  7, }, // Asia/Pyongyang
    {   85, 5476,   52523,  3, }, // Asia/Qatar
    {   85, 8671,   52526,  4, }, // Asia/Qostanay
    {   85, 4171,   52530,  5, }, // Asia/Qyzylorda
    {   85, 3486,   52535,  3, }, // Asia/Riyadh
    {   85, 4310,   52538,  4, }, // Asia/Sakhalin
    {   85, 7208,   52542,  5, }, // Asia/Samarkand
    {   85, 3116,   52547,  2, }, // Asia/Seoul
    {   85, 1020,   52549,  5, }, // Asia/Shanghai
    {   85, 3137,   52554,  5, }, // Asia/Singapore
    {   85, 4199,   52559,  9, }, // Asia/Srednekolymsk
    {   85, 3100,   52568,  3, }, // Asia/Taipei
    {   85, 4683,   52571,  5, }, // Asia/Tashkent
    {   85, 3817,   52576,  4, }, // Asia/Tbilisi
    {   85, 2638,   52580,  3, }, // Asia/Tehran
    {   85, 1284,   52583,  3, }, // Asia/Thimphu
    {   85, 2687,   52586,  3, }, // Asia/Tokyo
    {   85, 4474,   52589,  4, }, // Asia/Tomsk
    {   85, 1346,   52593,  5, }, // Asia/Ulaanbaatar
    {   85, 1126,   52598,  4, }, // Asia/Urumqi
    {   85, 8685,   52602,  6, }, // Asia/Ust-Nera
    {   85, 6940,   52608,  6, }, // Asia/Vientiane
    {   85, 4612,   52614,  7, }, // Asia/Vladivostok
    {   85, 4730,   52621,  4, }, // Asia/Yakutsk
    {   85, 1202,   52625,  4, }, // Asia/Yangon
    {   85, 3785,   52629,  7, }, // Asia/Yekaterinburg
    {   85, 3649,   52636,  4, }, // Asia/Yerevan
    {   85, 3565,   52640,  4, }, // Atlantic/Azores
    {   85, 5588,   52644,  3, }, // Atlantic/Bermuda
    {   85, 6591,   52647,  3, }, // Atlantic/Canary
    {   85, 3629,   52650, 11, }, // Atlantic/Cape_Verde
    {   85, 1379,   52661,  2, }, // Atlantic/Faroe
    {   85, 8699,   52663,  3, }, // Atlantic/Madeira
    {   85, 2614,   52666,  4, }, // Atlantic/Reykjavik
    {   85, 7235,   52670,  9, }, // Atlantic/South_Georgia
    {   85, 6784,   52679,  8, }, // Atlantic/St_Helena
    {   85, 6607,   52687,  4, }, // Atlantic/Stanley
    {   85, 1654,   52691,  4, }, // Australia/Adelaide
    {   85, 1619,   52695,  4, }, // Australia/Brisbane
    {   85, 1783,   52699,  8, }, // Australia/Broken_Hill
    {   85, 1567,   52707,  3, }, // Australia/Darwin
    {   85, 3539,   52710,  2, }, // Australia/Eucla
    {   85, 1500,   52712,  3, }, // Australia/Hobart
    {   85, 8716,   52715,  5, }, // Australia/Lindeman
    {   85, 1531,   52720,  8, }, // Australia/Lord_Howe
    {   85, 1711,   52728,  4, }, // Australia/Melbourne
    {   85, 1746,   52732,  3, }, // Australia/Perth
    {   85, 1447,   52735,  3, }, // Australia/Sydney
    {   85, 6390,   52738,  6, }, // Europe/Amsterdam
    {   85, 6149,   52744,  4, }, // Europe/Andorra
    {   85, 3522,   52748,  4, }, // Europe/Astrakhan
    {   85, 6577,   52752,  4, }, // Europe/Athens
    {   85, 6285,   52756,  4, }, // Europe/Belgrade
    {   85, 4659,   52760,  3, }, // Europe/Berlin
    {   85, 6437,   52763,  5, }, // Europe/Bratislava
    {   85, 6178,   52768,  4, }, // Europe/Brussels
    {   85, 3830,   52772,  5, }, // Europe/Bucharest
    {   85, 3708,   52777,  5, }, // Europe/Budapest
    {   85, 8751,   52782,  5, }, // Europe/Busingen
    {   85, 2475,   52787,  4, }, // Europe/Chisinau
    {   85, 6238,   52791,  6, }, // Europe/Copenhagen
    {   85, 2260,   52797,  3, }, // Europe/Dublin
    {   85, 6256,   52800,  4, }, // Europe/Gibraltar
    {   85, 8767,   52804,  3, }, // Europe/Guernsey
    {   85, 6561,   52807,  5, }, // Europe/Helsinki
    {   85, 8783,   52812, 15, }, // Europe/Isle_of_Man
    {   85, 1097,   52827,  6, }, // Europe/Istanbul
    {   85, 8802,   52833,  3, }, // Europe/Jersey
    {   85, 3881,   52836,  5, }, // Europe/Kaliningrad
    {   85, 8924,   52841,  3, }, // Europe/Kirov
    {   85, 2419,   52844,  2, }, // Europe/Kyiv
    {   85, 3078,   52846,  4, }, // Europe/Lisbon
    {   85, 6455,   52850,  3, }, // Europe/Ljubljana
    {   85, 2393,   52853,  4, }, // Europe/London
    {   85, 6314,   52857,  5, }, // Europe/Luxembourg
    {   85, 4794,   52862,  3, }, // Europe/Madrid
    {   85, 6346,   52865,  3, }, // Europe/Malta
    {   85, 6531,   52868,  4, }, // Europe/Mariehamn
    {   85, 3595,   52872,  4, }, // Europe/Minsk
    {   85, 6359,   52876,  3, }, // Europe/Monaco
    {   85, 3443,   52879,  4, }, // Europe/Moscow
    {   85, 6407,   52883,  3, }, // Europe/Oslo
    {   85, 4186,   52886,  3, }, // Europe/Paris
    {   85, 6373,   52889,  4, }, // Europe/Podgorica
    {   85, 6224,   51356,  2, }, // Europe/Prague
    {   85, 8816,   51710,  2, }, // Europe/Riga
    {   85, 6273,   52359,  2, }, // Europe/Rome
    {   85, 4233,   52893,  3, }, // Europe/Samara
    {   85, 6419,   52896,  3, }, // Europe/San_Marino
    {   85, 6194,   52899,  4, }, // Europe/Sarajevo
    {   85, 4353,   52903,  4, }, // Europe/Saratov
    {   85, 8828,   52907,  6, }, // Europe/Simferopol
    {   85, 6332,   52913,  3, }, // Europe/Skopje
    {   85, 6548,   52916,  3, }, // Europe/Sofia
    {   85, 6472,   52919,  5, }, // Europe/Stockholm
    {   85, 8846,   52924,  3, }, // Europe/Tallinn
    {   85, 6135,   52927,  3, }, // Europe/Tirane
    {   85, 8861,   52930,  7, }, // Europe/Ulyanovsk
    {   85, 6301,   52937,  3, }, // Europe/Vaduz
    {   85, 6516,   52940,  5, }, // Europe/Vatican
    {   85, 6164,   52945,  3, }, // Europe/Vienna
    {   85, 8878,   52948,  5, }, // Europe/Vilnius
    {   85, 4629,   52953,  5, }, // Europe/Volgograd
    {   85, 3055,   52958,  8, }, // Europe/Warsaw
    {   85, 6210,   52966,  3, }, // Europe/Zagreb
    {   85, 6489,   52969,  3, }, // Europe/Zurich
    {   85, 4980,   52972,  7, }, // Indian/Antananarivo
    {   85, 6910,   52979,  3, }, // Indian/Chagos
    {   85, 5979,   52982,  7, }, // Indian/Christmas
    {   85, 5996,   52989,  3, }, // Indian/Cocos
    {   85, 4931,   52992,  3, }, // Indian/Comoro
    {   85, 6624,   52995,  6, }, // Indian/Kerguelen
    {   85, 7223,   53001,  2, }, // Indian/Mahe
    {   85, 7013,   53003,  5, }, // Indian/Maldives
    {   85, 3971,   53008,  5, }, // Indian/Mauritius
    {   85, 5000,   53013,  3, }, // Indian/Mayotte
    {   85, 7174,   53016,  5, }, // Indian/Reunion
    {   85, 4324,   53021,  3, }, // Pacific/Apia
    {   85,  932,   53024,  5, }, // Pacific/Auckland
    {   85, 3608,   53029,  5, }, // Pacific/Bougainville
    {   85, 2863,   53034,  3, }, // Pacific/Chatham
    {   85, 3022,   53037,  2, }, // Pacific/Chuuk
    {   85, 2201,   53039,  7, }, // Pacific/Easter
    {   85, 7323,   53046,  3, }, // Pacific/Efate
    {   85, 7290,   53049,  4, }, // Pacific/Fakaofo
    {   85, 3804,   53053,  2, }, // Pacific/Fiji
    {   85, 7306,   53055,  4, }, // Pacific/Funafuti
    {   85, 6641,   53059, 12, }, // Pacific/Galapagos
    {   85, 6659,   53071,  4, }, // Pacific/Gambier
    {   85, 3724,   53075,  6, }, // Pacific/Guadalcanal
    {   85, 5935,   53081,  2, }, // Pacific/Guam
    {   85, 2929,   53083,  4, }, // Pacific/Honolulu
    {   85, 2897,   53087,  4, }, // Pacific/Kanton
    {   85, 3900,   53091,  5, }, // Pacific/Kiritimati
    {   85, 6982,   53096,  3, }, // Pacific/Kosrae
    {   85, 2708,   53099,  5, }, // Pacific/Kwajalein
    {   85, 4779,   53104,  3, }, // Pacific/Majuro
    {   85, 3953,   53107,  5, }, // Pacific/Marquesas
    {   85, 8909,   53112,  5, }, // Pacific/Midway
    {   85, 7047,   53117,  3, }, // Pacific/Nauru
    {   85, 7095,   52709,  2, }, // Pacific/Niue
    {   85, 4070,   53120,  5, }, // Pacific/Norfolk
    {   85, 7061,   53125,  4, }, // Pacific/Noumea
    {   85, 2991,   53129,  5, }, // Pacific/Pago_Pago
    {   85, 7118,   53134,  4, }, // Pacific/Palau
    {   85, 7157,   53138,  5, }, // Pacific/Pitcairn
    {   85, 2961,   53143,  3, }, // Pacific/Pohnpei
    {   85, 4709,   53146, 13, }, // Pacific/Port_Moresby
    {   85, 6009,   53159,  5, }, // Pacific/Rarotonga
    {   85, 5948,   53164,  4, }, // Pacific/Saipan
    {   85, 7275,   53168,  3, }, // Pacific/Tahiti
    {   85, 6675,   53171,  3, }, // Pacific/Tarawa
    {   85, 4485,   53174,  5, }, // Pacific/Tongatapu
    {   85, 7355,   53179,  3, }, // Pacific/Wake
    {   85, 7368,   53182,  3, }, // Pacific/Wallis
    {   86,   14,       0,  6, }, // Africa/Asmara Chickasaw/Latin/United States
    {   86, 4337,       6,  8, }, // Africa/Sao_Tome
    {   86, 4154,      14,  8, }, // America/Asuncion
    {   86,  237,      22,  8, }, // America/Atikokan
    {   86, 7586,      30, 17, }, // America/Bahia_Banderas
    {   86, 3770,      47,  6, }, // America/Cancun
    {   86, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   86, 5640,      66,  7, }, // America/Curacao
    {   86,  481,      73, 13, }, // America/Indiana/Knox
    {   86, 7924,      86, 16, }, // America/Indiana/Marengo
    {   86, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   86, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   86, 8001,     139, 14, }, // America/Indiana/Vevay
    {   86, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   86, 8049,     171, 16, }, // America/Indiana/Winamac
    {   86, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   86, 5849,     207, 22, }, // America/Lower_Princes
    {   86, 8183,     229,  6, }, // America/Merida
    {   86, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   86, 8292,     255, 20, }, // America/North_Dakota/Center
    {   86, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   86,  393,     298,  4, }, // America/Nuuk
    {   86, 6862,     302, 16, }, // America/Scoresbysund
    {   86, 8471,     318, 14, }, // America/St_Barthelemy
    {   86, 2045,     332, 10, }, // America/St_Johns
    {   86, 5780,     342,  9, }, // America/St_Kitts
    {   86, 5797,     351,  9, }, // America/St_Lucia
    {   86,  855,     360, 10, }, // America/St_Thomas
    {   86, 5830,     370, 11, }, // America/St_Vincent
    {   86, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   86, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   86, 1152,     413,  9, }, // Asia/Kathmandu
    {   86,  992,     422,  7, }, // Asia/Kolkata
    {   86, 1178,     429,  5, }, // Asia/Macau
    {   86, 1202,     442,  6, }, // Asia/Yangon
    {   86, 1379,     448,  5, }, // Atlantic/Faroe
    {   86, 6784,     453, 10, }, // Atlantic/St_Helena
    {   86, 2419,     463,  4, }, // Europe/Kyiv
    {   86, 7174,     467,  7, }, // Indian/Reunion
    {   86, 3022,     474,  5, }, // Pacific/Chuuk
    {   86, 2961,     479,  7, }, // Pacific/Pohnpei
    {   87,   14,       0,  6, }, // Africa/Asmara Chiga/Latin/Uganda
    {   87, 4337,       6,  8, }, // Africa/Sao_Tome
    {   87, 4154,      14,  8, }, // America/Asuncion
    {   87,  237,      22,  8, }, // America/Atikokan
    {   87, 7586,      30, 17, }, // America/Bahia_Banderas
    {   87, 3770,      47,  6, }, // America/Cancun
    {   87, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   87, 5640,      66,  7, }, // America/Curacao
    {   87,  481,      73, 13, }, // America/Indiana/Knox
    {   87, 7924,      86, 16, }, // America/Indiana/Marengo
    {   87, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   87, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   87, 8001,     139, 14, }, // America/Indiana/Vevay
    {   87, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   87, 8049,     171, 16, }, // America/Indiana/Winamac
    {   87, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   87, 5849,     207, 22, }, // America/Lower_Princes
    {   87, 8183,     229,  6, }, // America/Merida
    {   87, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   87, 8292,     255, 20, }, // America/North_Dakota/Center
    {   87, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   87,  393,     298,  4, }, // America/Nuuk
    {   87, 6862,     302, 16, }, // America/Scoresbysund
    {   87, 8471,     318, 14, }, // America/St_Barthelemy
    {   87, 2045,     332, 10, }, // America/St_Johns
    {   87, 5780,     342,  9, }, // America/St_Kitts
    {   87, 5797,     351,  9, }, // America/St_Lucia
    {   87,  855,     360, 10, }, // America/St_Thomas
    {   87, 5830,     370, 11, }, // America/St_Vincent
    {   87, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   87, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   87, 1152,     413,  9, }, // Asia/Kathmandu
    {   87,  992,     422,  7, }, // Asia/Kolkata
    {   87, 1178,     429,  5, }, // Asia/Macau
    {   87, 1202,     442,  6, }, // Asia/Yangon
    {   87, 1379,     448,  5, }, // Atlantic/Faroe
    {   87, 6784,     453, 10, }, // Atlantic/St_Helena
    {   87, 2419,     463,  4, }, // Europe/Kyiv
    {   87, 7174,     467,  7, }, // Indian/Reunion
    {   87, 3022,     474,  5, }, // Pacific/Chuuk
    {   87, 2961,     479,  7, }, // Pacific/Pohnpei
    {   88, 6751,   41582,  3, }, // Africa/Abidjan Chinese/Simplified Han/China
    {   88, 6723,   40068,  3, }, // Africa/Accra
    {   88, 4961,   53185,  6, }, // Africa/Addis_Ababa
    {   88, 7383,   41591,  4, }, // Africa/Algiers
    {   88,   14,   53191,  4, }, // Africa/Asmara
    {   88,   44,   41599,  3, }, // Africa/Bamako
    {   88, 5159,   40088,  2, }, // Africa/Bangui
    {   88, 6709,   53195,  3, }, // Africa/Banjul
    {   88, 7398,   41602,  2, }, // Africa/Bissau
    {   88, 4859,   41604,  4, }, // Africa/Blantyre
    {   88, 5189,   41608,  5, }, // Africa/Brazzaville
    {   88, 4824,   53198,  4, }, // Africa/Bujumbura
    {   88, 2242,   41613,  2, }, // Africa/Cairo
    {   88, 4019,   41615,  5, }, // Africa/Casablanca
    {   88, 7412,   41620,  2, }, // Africa/Ceuta
    {   88, 6736,   53202,  4, }, // Africa/Conakry
    {   88, 6803,   41622,  3, }, // Africa/Dakar
    {   88, 5032,   53206,  6, }, // Africa/Dar_es_Salaam
    {   88, 4945,   53212,  3, }, // Africa/Djibouti
    {   88, 5145,   40129,  3, }, // Africa/Douala
    {   88, 5068,   40132,  3, }, // Africa/El_Aaiun
    {   88, 6816,   53215,  3, }, // Africa/Freetown
    {   88, 4808,   53218,  4, }, // Africa/Gaborone
    {   88, 4917,   40142,  3, }, // Africa/Harare
    {   88, 4381,   53222,  5, }, // Africa/Johannesburg
    {   88, 4401,   40150,  2, }, // Africa/Juba
    {   88, 5053,   40152,  3, }, // Africa/Kampala
    {   88, 4426,   40155,  3, }, // Africa/Khartoum
    {   88, 4875,   40158,  3, }, // Africa/Kigali
    {   88, 5208,   53227,  3, }, // Africa/Kinshasa
    {   88, 4646,   53230,  3, }, // Africa/Lagos
    {   88, 5238,   53233,  4, }, // Africa/Libreville
    {   88, 6832,   40170,  2, }, // Africa/Lome
    {   88, 5113,   41633,  3, }, // Africa/Luanda
    {   88, 4841,   41636,  4, }, // Africa/Lubumbashi
    {   88, 4903,   53237,  3, }, // Africa/Lusaka
    {   88, 5224,   41640,  3, }, // Africa/Malabo
    {   88, 4889,   41643,  3, }, // Africa/Maputo
    {   88, 5099,   53240,  3, }, // Africa/Maseru
    {   88, 5084,   53243,  4, }, // Africa/Mbabane
    {   88, 5015,   53247,  4, }, // Africa/Mogadishu
    {   88, 6997,   41649,  4, }, // Africa/Monrovia
    {   88, 3755,   53251,  3, }, // Africa/Nairobi
    {   88, 5173,   41653,  4, }, // Africa/Ndjamena
    {   88, 5256,   41657,  3, }, // Africa/Niamey
    {   88, 6766,   53254,  5, }, // Africa/Nouakchott
    {   88, 6690,   40215,  4, }, // Africa/Ouagadougou
    {   88, 5127,   53259,  4, }, // Africa/Porto-Novo
    {   88, 4337,   41667,  3, }, // Africa/Sao_Tome
    {   88, 2732,   40226,  4, }, // Africa/Tripoli
    {   88, 6503,   40230,  3, }, // Africa/Tunis
    {   88, 4054,   41670,  4, }, // Africa/Windhoek
    {   88,  132,   53263,  3, }, // America/Adak
    {   88, 3183,   53266,  4, }, // America/Anchorage
    {   88, 5524,   53270,  3, }, // America/Anguilla
    {   88, 5541,   53273,  3, }, // America/Antigua
    {   88, 4456,   53276,  5, }, // America/Araguaina
    {   88,  166,   41677,  7, }, // America/Argentina/Buenos_Aires
    {   88,   91,   41684,  4, }, // America/Argentina/Catamarca
    {   88,  270,   53281,  4, }, // America/Argentina/Cordoba
    {   88,  441,   40268,  3, }, // America/Argentina/Jujuy
    {   88, 7425,   53285,  4, }, // America/Argentina/La_Rioja
    {   88,  565,   41691,  3, }, // America/Argentina/Mendoza
    {   88, 7452,   41694,  6, }, // America/Argentina/Rio_Gallegos
    {   88, 7483,   41700,  3, }, // America/Argentina/Salta
    {   88, 7507,   41703,  3, }, // America/Argentina/San_Juan
    {   88, 5497,   53289,  4, }, // America/Argentina/San_Luis
    {   88, 7534,   53293,  3, }, // America/Argentina/Tucuman
    {   88, 7560,   41712,  4, }, // America/Argentina/Ushuaia
    {   88, 5557,   53296,  3, }, // America/Aruba
    {   88, 4154,   41716,  3, }, // America/Asuncion
    {   88,  237,   40305,  4, }, // America/Atikokan
    {   88, 3581,   41719,  3, }, // America/Bahia
    {   88, 7586,   41719,  7, }, // America/Bahia_Banderas
    {   88, 5571,   53299,  4, }, // America/Barbados
    {   88, 7609,   41729,  2, }, // America/Belem
    {   88, 5309,   53303,  3, }, // America/Belize
    {   88, 7623,   53306,  6, }, // America/Blanc-Sablon
    {   88, 7644,   53312,  5, }, // America/Boa_Vista
    {   88, 4263,   40336,  3, }, // America/Bogota
    {   88, 7662,   53317,  3, }, // America/Boise
    {   88, 7676,   41743,  3, }, // America/Cambridge_Bay
    {   88, 7698,   53320,  3, }, // America/Campo_Grande
    {   88, 3770,   40347,  2, }, // America/Cancun
    {   88, 4596,   53323,  4, }, // America/Caracas
    {   88, 4247,   53327,  2, }, // America/Cayenne
    {   88, 5398,   41751,  2, }, // America/Cayman
    {   88, 3251,   40359,  3, }, // America/Chicago
    {   88, 7719,   53329,  3, }, // America/Chihuahua
    {   88, 7737,   41758,  4, }, // America/Ciudad_Juarez
    {   88, 5324,   53332,  5, }, // America/Costa_Rica
    {   88, 7759,   41762,  4, }, // America/Creston
    {   88, 3693,   53337,  3, }, // America/Cuiaba
    {   88, 5640,   41766,  3, }, // America/Curacao
    {   88, 7775,   41769,  4, }, // America/Danmarkshavn
    {   88, 7796,   40388,  2, }, // America/Dawson
    {   88, 7811,   40390,  5, }, // America/Dawson_Creek
    {   88,  805,   40395,  2, }, // America/Denver
    {   88, 3351,   40397,  3, }, // America/Detroit
    {   88, 5656,   53340,  4, }, // America/Dominica
    {   88,  893,   53344,  4, }, // America/Edmonton
    {   88, 7832,   53348,  4, }, // America/Eirunepe
    {   88, 5343,   41777,  4, }, // America/El_Salvador
    {   88, 7849,   41781,  4, }, // America/Fort_Nelson
    {   88, 7869,   53352,  4, }, // America/Fortaleza
    {   88, 7887,   53356,  4, }, // America/Glace_Bay
    {   88, 6844,   53360,  3, }, // America/Goose_Bay
    {   88, 4514,   40430,  3, }, // America/Grand_Turk
    {   88, 5687,   53363,  4, }, // America/Grenada
    {   88, 5703,   53367,  4, }, // America/Guadeloupe
    {   88, 3662,   53371,  4, }, // America/Guatemala
    {   88, 4743,   41803,  4, }, // America/Guayaquil
    {   88, 6895,   53375,  3, }, // America/Guyana
    {   88, 1923,   53378,  5, }, // America/Halifax
    {   88, 2221,   40458,  3, }, // America/Havana
    {   88, 7905,   41810,  4, }, // America/Hermosillo
    {   88,  348,   53383,  7, }, // America/Indiana/Indianapolis
    {   88,  481,   53390,  8, }, // America/Indiana/Knox
    {   88, 7924,   53398,  8, }, // America/Indiana/Marengo
    {   88, 7948,   53406,  9, }, // America/Indiana/Petersburg
    {   88, 7975,   53415,  8, }, // America/Indiana/Tell_City
    {   88, 8001,   53423,  8, }, // America/Indiana/Vevay
    {   88, 8023,   53431,  8, }, // America/Indiana/Vincennes
    {   88, 8049,   53439,  9, }, // America/Indiana/Winamac
    {   88, 8073,   53448,  4, }, // America/Inuvik
    {   88,  660,   40532,  3, }, // America/Iqaluit
    {   88, 2665,   41866,  3, }, // America/Jamaica
    {   88, 5282,   41869,  2, }, // America/Juneau
    {   88,  521,   41871,  5, }, // America/Kentucky/Louisville
    {   88, 8088,   53452,  8, }, // America/Kentucky/Monticello
    {   88, 5621,   41884,  5, }, // America/Kralendijk
    {   88, 4278,   40558,  3, }, // America/La_Paz
    {   88, 7132,   41889,  2, }, // America/Lima
    {   88, 3390,   41891,  3, }, // America/Los_Angeles
    {   88, 5849,   41894,  4, }, // America/Lower_Princes
    {   88, 8116,   53460,  3, }, // America/Maceio
    {   88, 8131,   53463,  3, }, // America/Managua
    {   88, 1892,   53466,  3, }, // America/Manaus
    {   88, 5814,   41907,  4, }, // America/Marigot
    {   88, 5722,   53469,  4, }, // America/Martinique
    {   88, 8147,   41915,  5, }, // America/Matamoros
    {   88, 2779,   41920,  4, }, // America/Mazatlan
    {   88, 8165,   53473,  4, }, // America/Menominee
    {   88, 8183,   41928,  3, }, // America/Merida
    {   88, 8198,   40603,  6, }, // America/Metlakatla
    {   88, 2811,   53477,  4, }, // America/Mexico_City
    {   88, 4293,   53481,  3, }, // America/Miquelon
    {   88, 8217,   41934,  3, }, // America/Moncton
    {   88, 8233,   53484,  3, }, // America/Monterrey
    {   88, 4000,   53487,  5, }, // America/Montevideo
    {   88, 5741,   53492,  5, }, // America/Montserrat
    {   88, 5383,   53497,  2, }, // America/Nassau
    {   88, 3294,   41944,  2, }, // America/New_York
    {   88, 8251,   41946,  2, }, // America/Nome
    {   88, 1834,   53499,  4, }, // America/Noronha
    {   88, 8264,   53503,  8, }, // America/North_Dakota/Beulah
    {   88, 8292,   53511,  7, }, // America/North_Dakota/Center
    {   88, 8320,   53518,  9, }, // America/North_Dakota/New_Salem
    {   88,  393,   40661,  2, }, // America/Nuuk
    {   88, 8351,   41974,  4, }, // America/Ojinaga
    {   88, 5413,   41978,  3, }, // America/Panama
    {   88, 6106,   53527,  5, }, // America/Paramaribo
    {   88, 3224,   41986,  3, }, // America/Phoenix
    {   88, 3847,   40678,  3, }, // America/Port-au-Prince
    {   88, 5871,   40681,  4, }, // America/Port_of_Spain
    {   88, 8367,   53532,  4, }, // America/Porto_Velho
    {   88, 5760,   40688,  4, }, // America/Puerto_Rico
    {   88, 3932,   41992,  6, }, // America/Punta_Arenas
    {   88, 8387,   41998,  3, }, // America/Rankin_Inlet
    {   88, 8408,   53536,  3, }, // America/Recife
    {   88, 1979,   42001,  3, }, // America/Regina
    {   88, 8423,   53539,  4, }, // America/Resolute
    {   88,  695,   53543,  5, }, // America/Rio_Branco
    {   88, 8440,   42012,  3, }, // America/Santarem
    {   88, 2165,   53548,  4, }, // America/Santiago
    {   88, 6044,   53552,  4, }, // America/Santo_Domingo
    {   88, 1862,   42023,  3, }, // America/Sao_Paulo
    {   88, 6862,   53556,  7, }, // America/Scoresbysund
    {   88, 8457,   42033,  3, }, // America/Sitka
    {   88, 8471,   53563,  6, }, // America/St_Barthelemy
    {   88, 2045,   53569,  4, }, // America/St_Johns
    {   88, 5780,   42045,  3, }, // America/St_Kitts
    {   88, 5797,   53573,  4, }, // America/St_Lucia
    {   88,  855,   42052,  4, }, // America/St_Thomas
    {   88, 5830,   53577,  4, }, // America/St_Vincent
    {   88, 8493,   42059,  7, }, // America/Swift_Current
    {   88, 5363,   53581,  6, }, // America/Tegucigalpa
    {   88, 5673,   53587,  2, }, // America/Thule
    {   88,  313,   53589,  3, }, // America/Tijuana
    {   88,  608,   42069,  3, }, // America/Toronto
    {   88, 5605,   42072,  4, }, // America/Tortola
    {   88, 2077,   42076,  3, }, // America/Vancouver
    {   88, 2128,   42079,  4, }, // America/Whitehorse
    {   88,  734,   42083,  3, }, // America/Winnipeg
    {   88, 8515,   53592,  4, }, // America/Yakutat
    {   88, 5918,   53596,  2, }, // Antarctica/Casey
    {   88, 6027,   42091,  3, }, // Antarctica/Davis
    {   88, 6066,   53598,  6, }, // Antarctica/DumontDUrville
    {   88, 8531,   53604,  3, }, // Antarctica/Macquarie
    {   88, 7029,   40812,  2, }, // Antarctica/Mawson
    {   88, 7076,   42102,  4, }, // Antarctica/McMurdo
    {   88, 8552,   53607,  3, }, // Antarctica/Palmer
    {   88, 7189,   42108,  3, }, // Antarctica/Rothera
    {   88, 7258,   40823,  2, }, // Antarctica/Syowa
    {   88, 8570,   53610,  3, }, // Antarctica/Troll
    {   88, 7337,   40829,  4, }, // Antarctica/Vostok
    {   88, 1413,   53613,  4, }, // Arctic/Longyearbyen
    {   88, 5487,   42113,  2, }, // Asia/Aden
    {   88, 5297,   42115,  4, }, // Asia/Almaty
    {   88, 3870,   40844,  2, }, // Asia/Amman
    {   88, 5428,   53617,  4, }, // Asia/Anadyr
    {   88, 5440,   40849,  3, }, // Asia/Aqtau
    {   88, 5270,   53621,  4, }, // Asia/Aqtobe
    {   88,  964,   53625,  5, }, // Asia/Ashgabat
    {   88, 8587,   42119,  3, }, // Asia/Atyrau
    {   88, 3509,   42122,  3, }, // Asia/Baghdad
    {   88, 5451,   40867,  2, }, // Asia/Bahrain
    {   88, 3555,   42125,  2, }, // Asia/Baku
    {   88, 4368,   40869,  2, }, // Asia/Bangkok
    {   88, 3473,   42127,  4, }, // Asia/Barnaul
    {   88, 3988,   42131,  3, }, // Asia/Beirut
    {   88, 3680,   42134,  4, }, // Asia/Bishkek
    {   88, 5906,   53630,  2, }, // Asia/Brunei
    {   88, 4503,   40884,  2, }, // Asia/Chita
    {   88, 5963,   42140,  3, }, // Asia/Choibalsan
    {   88, 4413,   53632,  3, }, // Asia/Colombo
    {   88, 4442,   42146,  4, }, // Asia/Damascus
    {   88, 1060,   42150,  2, }, // Asia/Dhaka
    {   88, 6125,   40898,  2, }, // Asia/Dili
    {   88, 3498,   53635,  2, }, // Asia/Dubai
    {   88, 6092,   53637,  3, }, // Asia/Dushanbe
    {   88, 8599,   42155,  5, }, // Asia/Famagusta
    {   88, 8614,   53640,  2, }, // Asia/Gaza
    {   88, 4697,   53642,  3, }, // Asia/Hebron
    {   88, 1226,   40915,  4, }, // Asia/Ho_Chi_Minh
    {   88, 2576,   40926,  2, }, // Asia/Hong_Kong
    {   88, 4673,   40933,  3, }, // Asia/Hovd
    {   88, 4086,   53645,  5, }, // Asia/Irkutsk
    {   88, 6969,   42181,  3, }, // Asia/Jakarta
    {   88, 6955,   53650,  4, }, // Asia/Jayapura
    {   88, 1257,   40948,  4, }, // Asia/Jerusalem
    {   88, 3462,   42188,  3, }, // Asia/Kabul
    {   88, 4218,   40955,  3, }, // Asia/Kamchatka
    {   88, 4141,   53654,  3, }, // Asia/Karachi
    {   88, 1152,   42191,  4, }, // Asia/Kathmandu
    {   88, 8624,   53657,  3, }, // Asia/Khandyga
    {   88,  992,   42195,  4, }, // Asia/Kolkata
    {   88, 4099,   42199,  8, }, // Asia/Krasnoyarsk
    {   88, 4761,   40980,  3, }, // Asia/Kuala_Lumpur
    {   88, 5893,   42207,  2, }, // Asia/Kuching
    {   88, 5464,   40985,  3, }, // Asia/Kuwait
    {   88, 1178,   42216,  2, }, // Asia/Macau
    {   88, 3919,   42223,  3, }, // Asia/Magadan
    {   88, 1316,   53660,  3, }, // Asia/Makassar
    {   88, 7145,   42230,  3, }, // Asia/Manila
    {   88, 6883,   53663,  4, }, // Asia/Muscat
    {   88, 2446,   53667,  4, }, // Asia/Nicosia
    {   88, 8638,   42241,  6, }, // Asia/Novokuznetsk
    {   88, 4037,   42247,  5, }, // Asia/Novosibirsk
    {   88, 4131,   41031,  4, }, // Asia/Omsk
    {   88, 7108,   42252,  3, }, // Asia/Oral
    {   88, 6924,   42255,  2, }, // Asia/Phnom_Penh
    {   88, 8656,   41040,  2, }, // Asia/Pontianak
    {   88, 4116,   41042,  2, }, // Asia/Pyongyang
    {   88, 5476,   53671,  3, }, // Asia/Qatar
    {   88, 8671,   53674,  4, }, // Asia/Qostanay
    {   88, 4171,   53678,  5, }, // Asia/Qyzylorda
    {   88, 3486,   53683,  3, }, // Asia/Riyadh
    {   88, 4310,   53686,  3, }, // Asia/Sakhalin
    {   88, 7208,   42268,  4, }, // Asia/Samarkand
    {   88, 3116,   42272,  2, }, // Asia/Seoul
    {   88, 1020,   41068,  2, }, // Asia/Shanghai
    {   88, 3137,   41070,  3, }, // Asia/Singapore
    {   88, 4199,   41073,  6, }, // Asia/Srednekolymsk
    {   88, 3100,   41079,  2, }, // Asia/Taipei
    {   88, 4683,   41081,  3, }, // Asia/Tashkent
    {   88, 3817,   41084,  4, }, // Asia/Tbilisi
    {   88, 2638,   42274,  3, }, // Asia/Tehran
    {   88, 1284,   41091,  2, }, // Asia/Thimphu
    {   88, 2687,   42277,  2, }, // Asia/Tokyo
    {   88, 4474,   41095,  4, }, // Asia/Tomsk
    {   88, 1346,   42279,  4, }, // Asia/Ulaanbaatar
    {   88, 1126,   42283,  4, }, // Asia/Urumqi
    {   88, 8685,   42287,  4, }, // Asia/Ust-Nera
    {   88, 6940,   53689,  2, }, // Asia/Vientiane
    {   88, 4612,   53691,  7, }, // Asia/Vladivostok
    {   88, 4730,   53698,  4, }, // Asia/Yakutsk
    {   88, 1202,   41120,  2, }, // Asia/Yangon
    {   88, 3785,   53702,  5, }, // Asia/Yekaterinburg
    {   88, 3649,   53707,  3, }, // Asia/Yerevan
    {   88, 3565,   42306,  5, }, // Atlantic/Azores
    {   88, 5588,   53710,  3, }, // Atlantic/Bermuda
    {   88, 6591,   53713,  3, }, // Atlantic/Canary
    {   88, 3629,   53716,  3, }, // Atlantic/Cape_Verde
    {   88, 1379,   42320,  2, }, // Atlantic/Faroe
    {   88, 8699,   53719,  3, }, // Atlantic/Madeira
    {   88, 2614,   53722,  5, }, // Atlantic/Reykjavik
    {   88, 7235,   42334,  4, }, // Atlantic/South_Georgia
    {   88, 6784,   42338,  4, }, // Atlantic/St_Helena
    {   88, 6607,   53727,  3, }, // Atlantic/Stanley
    {   88, 1654,   53730,  4, }, // Australia/Adelaide
    {   88, 1619,   53734,  4, }, // Australia/Brisbane
    {   88, 1783,   42343,  5, }, // Australia/Broken_Hill
    {   88, 1567,   42348,  3, }, // Australia/Darwin
    {   88, 3539,   41186,  3, }, // Australia/Eucla
    {   88, 1500,   53738,  3, }, // Australia/Hobart
    {   88, 8716,   41192,  3, }, // Australia/Lindeman
    {   88, 1531,   42351,  4, }, // Australia/Lord_Howe
    {   88, 1711,   42355,  3, }, // Australia/Melbourne
    {   88, 1746,   53741,  2, }, // Australia/Perth
    {   88, 1447,   53743,  2, }, // Australia/Sydney
    {   88, 6390,   41206,  5, }, // Europe/Amsterdam
    {   88, 6149,   42358,  3, }, // Europe/Andorra
    {   88, 3522,   41214,  5, }, // Europe/Astrakhan
    {   88, 6577,   41219,  2, }, // Europe/Athens
    {   88, 6285,   53745,  5, }, // Europe/Belgrade
    {   88, 4659,   41225,  2, }, // Europe/Berlin
    {   88, 6437,   53750,  6, }, // Europe/Bratislava
    {   88, 6178,   42365,  4, }, // Europe/Brussels
    {   88, 3830,   41237,  5, }, // Europe/Bucharest
    {   88, 3708,   42369,  4, }, // Europe/Budapest
    {   88, 8751,   41246,  3, }, // Europe/Busingen
    {   88, 2475,   53756,  4, }, // Europe/Chisinau
    {   88, 6238,   41252,  4, }, // Europe/Copenhagen
    {   88, 2260,   41256,  3, }, // Europe/Dublin
    {   88, 6256,   42373,  4, }, // Europe/Gibraltar
    {   88, 8767,   53760,  3, }, // Europe/Guernsey
    {   88, 6561,   42380,  4, }, // Europe/Helsinki
    {   88, 8783,   53763,  3, }, // Europe/Isle_of_Man
    {   88, 1097,   53766,  5, }, // Europe/Istanbul
    {   88, 8802,   42386,  3, }, // Europe/Jersey
    {   88, 3881,   42389,  5, }, // Europe/Kaliningrad
    {   88, 8924,   41284,  3, }, // Europe/Kirov
    {   88, 2419,   42394,  2, }, // Europe/Kyiv
    {   88, 3078,   41289,  3, }, // Europe/Lisbon
    {   88, 6455,   53771,  5, }, // Europe/Ljubljana
    {   88, 2393,   42400,  2, }, // Europe/London
    {   88, 6314,   42402,  3, }, // Europe/Luxembourg
    {   88, 4794,   42405,  3, }, // Europe/Madrid
    {   88, 6346,   53776,  3, }, // Europe/Malta
    {   88, 6531,   42411,  3, }, // Europe/Mariehamn
    {   88, 3595,   41310,  3, }, // Europe/Minsk
    {   88, 6359,   42414,  3, }, // Europe/Monaco
    {   88, 3443,   41316,  3, }, // Europe/Moscow
    {   88, 6407,   42417,  3, }, // Europe/Oslo
    {   88, 4186,   41322,  2, }, // Europe/Paris
    {   88, 6373,   53779,  5, }, // Europe/Podgorica
    {   88, 6224,   41328,  3, }, // Europe/Prague
    {   88, 8816,   41331,  2, }, // Europe/Riga
    {   88, 6273,   42420,  2, }, // Europe/Rome
    {   88, 4233,   53784,  3, }, // Europe/Samara
    {   88, 6419,   53787,  4, }, // Europe/San_Marino
    {   88, 6194,   53791,  4, }, // Europe/Sarajevo
    {   88, 4353,   42429,  4, }, // Europe/Saratov
    {   88, 8828,   53795,  5, }, // Europe/Simferopol
    {   88, 6332,   53800,  4, }, // Europe/Skopje
    {   88, 6548,   53804,  3, }, // Europe/Sofia
    {   88, 6472,   42436,  5, }, // Europe/Stockholm
    {   88, 8846,   41366,  2, }, // Europe/Tallinn
    {   88, 6135,   41368,  3, }, // Europe/Tirane
    {   88, 8861,   42441,  7, }, // Europe/Ulyanovsk
    {   88, 6301,   53807,  3, }, // Europe/Vaduz
    {   88, 6516,   42451,  3, }, // Europe/Vatican
    {   88, 6164,   42454,  3, }, // Europe/Vienna
    {   88, 8878,   42457,  4, }, // Europe/Vilnius
    {   88, 4629,   42461,  5, }, // Europe/Volgograd
    {   88, 3055,   42466,  2, }, // Europe/Warsaw
    {   88, 6210,   53810,  4, }, // Europe/Zagreb
    {   88, 6489,   42468,  3, }, // Europe/Zurich
    {   88, 4980,   41405,  6, }, // Indian/Antananarivo
    {   88, 6910,   41411,  3, }, // Indian/Chagos
    {   88, 5979,   42471,  3, }, // Indian/Christmas
    {   88, 5996,   53814,  3, }, // Indian/Cocos
    {   88, 4931,   42479,  3, }, // Indian/Comoro
    {   88, 6624,   42484,  4, }, // Indian/Kerguelen
    {   88, 7223,   42489,  3, }, // Indian/Mahe
    {   88, 7013,   53817,  4, }, // Indian/Maldives
    {   88, 3971,   53821,  4, }, // Indian/Mauritius
    {   88, 5000,   42496,  3, }, // Indian/Mayotte
    {   88, 7174,   53825,  3, }, // Indian/Reunion
    {   88, 4324,   42504,  3, }, // Pacific/Apia
    {   88,  932,   42507,  3, }, // Pacific/Auckland
    {   88, 3608,   42510,  4, }, // Pacific/Bougainville
    {   88, 2863,   53828,  3, }, // Pacific/Chatham
    {   88, 3022,   53831,  5, }, // Pacific/Chuuk
    {   88, 2201,   53836,  4, }, // Pacific/Easter
    {   88, 7323,   41468,  3, }, // Pacific/Efate
    {   88, 7290,   41471,  3, }, // Pacific/Fakaofo
    {   88, 3804,   42517,  2, }, // Pacific/Fiji
    {   88, 7306,   53840,  4, }, // Pacific/Funafuti
    {   88, 6641,   53844,  5, }, // Pacific/Galapagos
    {   88, 6659,   42525,  3, }, // Pacific/Gambier
    {   88, 3724,   53849,  6, }, // Pacific/Guadalcanal
    {   88, 5935,   42536,  2, }, // Pacific/Guam
    {   88, 2929,   41499,  3, }, // Pacific/Honolulu
    {   88, 2897,   53855,  4, }, // Pacific/Kanton
    {   88, 3900,   42543,  6, }, // Pacific/Kiritimati
    {   88, 6982,   53859,  3, }, // Pacific/Kosrae
    {   88, 2708,   53862,  3, }, // Pacific/Kwajalein
    {   88, 4779,   53865,  3, }, // Pacific/Majuro
    {   88, 3953,   53868,  4, }, // Pacific/Marquesas
    {   88, 8909,   42561,  3, }, // Pacific/Midway
    {   88, 7047,   53872,  2, }, // Pacific/Nauru
    {   88, 7095,   42566,  2, }, // Pacific/Niue
    {   88, 4070,   42569,  3, }, // Pacific/Norfolk
    {   88, 7061,   53874,  3, }, // Pacific/Noumea
    {   88, 2991,   53877,  4, }, // Pacific/Pago_Pago
    {   88, 7118,   53881,  2, }, // Pacific/Palau
    {   88, 7157,   53883,  4, }, // Pacific/Pitcairn
    {   88, 2961,   53887,  4, }, // Pacific/Pohnpei
    {   88, 4709,   53891,  5, }, // Pacific/Port_Moresby
    {   88, 6009,   42583,  4, }, // Pacific/Rarotonga
    {   88, 5948,   41564,  2, }, // Pacific/Saipan
    {   88, 7275,   53896,  3, }, // Pacific/Tahiti
    {   88, 6675,   41569,  3, }, // Pacific/Tarawa
    {   88, 4485,   42587,  4, }, // Pacific/Tongatapu
    {   88, 7355,   41577,  2, }, // Pacific/Wake
    {   88, 7368,   41579,  3, }, // Pacific/Wallis
    {   91, 6862,   53899,  6, }, // America/Scoresbysund Chinese/Simplified Han/Singapore
    {   92, 6751,   53905,  3, }, // Africa/Abidjan Chinese/Traditional Han/Hong Kong
    {   92,   14,   53908,  4, }, // Africa/Asmara
    {   92, 5159,   53912,  2, }, // Africa/Bangui
    {   92, 6736,   53914,  4, }, // Africa/Conakry
    {   92, 5032,   53918,  6, }, // Africa/Dar_es_Salaam
    {   92, 4381,   53924,  5, }, // Africa/Johannesburg
    {   92, 5208,   53929,  3, }, // Africa/Kinshasa
    {   92, 5238,   53932,  3, }, // Africa/Libreville
    {   92, 5113,   53935,  3, }, // Africa/Luanda
    {   92, 4903,   53938,  3, }, // Africa/Lusaka
    {   92, 5099,   53941,  3, }, // Africa/Maseru
    {   92, 3755,   53944,  3, }, // Africa/Nairobi
    {   92, 4054,   53947,  4, }, // Africa/Windhoek
    {   92,  132,   53951,  3, }, // America/Adak
    {   92, 4456,   53954,  5, }, // America/Araguaina
    {   92,  270,   53959,  4, }, // America/Argentina/Cordoba
    {   92, 7425,   53963,  4, }, // America/Argentina/La_Rioja
    {   92, 5557,   53967,  3, }, // America/Aruba
    {   92, 4154,   53970,  3, }, // America/Asuncion
    {   92, 3581,   53973,  3, }, // America/Bahia
    {   92, 7586,   53976,  8, }, // America/Bahia_Banderas
    {   92, 5309,   53984,  3, }, // America/Belize
    {   92, 7644,   53987,  5, }, // America/Boa_Vista
    {   92, 7719,   53992,  3, }, // America/Chihuahua
    {   92, 7737,   40365,  4, }, // America/Ciudad_Juarez
    {   92, 5324,   53995,  5, }, // America/Costa_Rica
    {   92, 3693,   54000,  3, }, // America/Cuiaba
    {   92, 7811,   54003,  3, }, // America/Dawson_Creek
    {   92,  893,   54006,  3, }, // America/Edmonton
    {   92, 7869,   54009,  4, }, // America/Fortaleza
    {   92, 7887,   54013,  4, }, // America/Glace_Bay
    {   92, 5687,   54017,  4, }, // America/Grenada
    {   92, 5703,   54021,  4, }, // America/Guadeloupe
    {   92, 3662,   54025,  4, }, // America/Guatemala
    {   92, 6895,   54029,  3, }, // America/Guyana
    {   92, 2221,   54032,  3, }, // America/Havana
    {   92,  348,   54035,  7, }, // America/Indiana/Indianapolis
    {   92,  481,   54042,  8, }, // America/Indiana/Knox
    {   92, 7924,   54050,  8, }, // America/Indiana/Marengo
    {   92, 7948,   54058,  8, }, // America/Indiana/Petersburg
    {   92, 7975,   54066,  8, }, // America/Indiana/Tell_City
    {   92, 8001,   54074,  7, }, // America/Indiana/Vevay
    {   92, 8023,   54081,  8, }, // America/Indiana/Vincennes
    {   92, 8049,   54089,  9, }, // America/Indiana/Winamac
    {   92, 8073,   54098,  4, }, // America/Inuvik
    {   92,  521,   54102,  4, }, // America/Kentucky/Louisville
    {   92, 8116,   54106,  3, }, // America/Maceio
    {   92, 8131,   54109,  3, }, // America/Managua
    {   92, 5722,   54112,  4, }, // America/Martinique
    {   92, 8165,   54116,  4, }, // America/Menominee
    {   92, 4000,   54120,  5, }, // America/Montevideo
    {   92, 5741,   54125,  5, }, // America/Montserrat
    {   92, 5383,   54130,  2, }, // America/Nassau
    {   92, 1834,   54132,  4, }, // America/Noronha
    {   92, 8264,   54136,  8, }, // America/North_Dakota/Beulah
    {   92, 8292,   54144,  8, }, // America/North_Dakota/Center
    {   92, 8320,   54152,  9, }, // America/North_Dakota/New_Salem
    {   92, 8367,   54161,  3, }, // America/Porto_Velho
    {   92,  695,   54164,  5, }, // America/Rio_Branco
    {   92, 2165,   54169,  4, }, // America/Santiago
    {   92, 6044,   54173,  4, }, // America/Santo_Domingo
    {   92, 8471,   54177,  5, }, // America/St_Barthelemy
    {   92, 2045,   54182,  4, }, // America/St_Johns
    {   92, 5797,   54186,  4, }, // America/St_Lucia
    {   92, 5830,   54190,  4, }, // America/St_Vincent
    {   92, 5363,   54194,  6, }, // America/Tegucigalpa
    {   92, 5673,   54200,  2, }, // America/Thule
    {   92,  313,   54202,  3, }, // America/Tijuana
    {   92, 2128,   54205,  3, }, // America/Whitehorse
    {   92, 8515,   54208,  4, }, // America/Yakutat
    {   92, 5918,   54212,  3, }, // Antarctica/Casey
    {   92, 6027,   54215,  4, }, // Antarctica/Davis
    {   92, 6066,   54219,  6, }, // Antarctica/DumontDUrville
    {   92, 8531,   54225,  3, }, // Antarctica/Macquarie
    {   92, 7029,   54228,  3, }, // Antarctica/Mawson
    {   92, 7076,   54231,  5, }, // Antarctica/McMurdo
    {   92, 8552,   54236,  3, }, // Antarctica/Palmer
    {   92, 7189,   54239,  4, }, // Antarctica/Rothera
    {   92, 7258,   54243,  3, }, // Antarctica/Syowa
    {   92, 8570,   54246,  4, }, // Antarctica/Troll
    {   92, 1413,   54250,  4, }, // Arctic/Longyearbyen
    {   92, 5428,   54254,  4, }, // Asia/Anadyr
    {   92, 3680,   54258,  4, }, // Asia/Bishkek
    {   92, 4413,   54262,  3, }, // Asia/Colombo
    {   92, 6092,   54265,  3, }, // Asia/Dushanbe
    {   92, 4697,   54268,  3, }, // Asia/Hebron
    {   92, 4086,   54271,  5, }, // Asia/Irkutsk
    {   92, 6955,   54276,  4, }, // Asia/Jayapura
    {   92, 1316,   54280,  2, }, // Asia/Makassar
    {   92, 6883,   54282,  4, }, // Asia/Muscat
    {   92, 2446,   54286,  4, }, // Asia/Nicosia
    {   92, 5476,   54290,  3, }, // Asia/Qatar
    {   92, 4730,   54293,  4, }, // Asia/Yakutsk
    {   92, 3785,   54297,  5, }, // Asia/Yekaterinburg
    {   92, 8699,   54302,  4, }, // Atlantic/Madeira
    {   92, 6607,   54306,  3, }, // Atlantic/Stanley
    {   92, 1654,   54309,  4, }, // Australia/Adelaide
    {   92, 1619,   54313,  4, }, // Australia/Brisbane
    {   92, 1500,   54317,  3, }, // Australia/Hobart
    {   92, 6285,   54320,  5, }, // Europe/Belgrade
    {   92, 6437,   54325,  6, }, // Europe/Bratislava
    {   92, 2475,   54331,  4, }, // Europe/Chisinau
    {   92, 8767,   54335,  3, }, // Europe/Guernsey
    {   92, 8783,   54338,  3, }, // Europe/Isle_of_Man
    {   92, 6455,   54341,  5, }, // Europe/Ljubljana
    {   92, 6346,   54346,  3, }, // Europe/Malta
    {   92, 6194,   54349,  4, }, // Europe/Sarajevo
    {   92, 6301,   54353,  3, }, // Europe/Vaduz
    {   92, 6210,   54356,  4, }, // Europe/Zagreb
    {   92, 6910,   54360,  5, }, // Indian/Chagos
    {   92, 5996,   54365,  5, }, // Indian/Cocos
    {   92, 4931,   41422,  3, }, // Indian/Comoro
    {   92, 6624,   54370,  6, }, // Indian/Kerguelen
    {   92, 7013,   54376,  4, }, // Indian/Maldives
    {   92, 3971,   54380,  4, }, // Indian/Mauritius
    {   92, 5000,   41443,  3, }, // Indian/Mayotte
    {   92, 7174,   41447,  3, }, // Indian/Reunion
    {   92, 3608,   54384,  5, }, // Pacific/Bougainville
    {   92, 7290,   54389,  5, }, // Pacific/Fakaofo
    {   92, 6659,   41486,  3, }, // Pacific/Gambier
    {   92, 3724,   54394,  7, }, // Pacific/Guadalcanal
    {   92, 2897,   54401,  5, }, // Pacific/Kanton
    {   92, 6982,   54406,  3, }, // Pacific/Kosrae
    {   92, 2708,   54409,  5, }, // Pacific/Kwajalein
    {   92, 4779,   54414,  3, }, // Pacific/Majuro
    {   92, 3953,   54417,  6, }, // Pacific/Marquesas
    {   92, 7047,   54423,  2, }, // Pacific/Nauru
    {   92, 7157,   54425,  5, }, // Pacific/Pitcairn
    {   92, 4485,   54430,  5, }, // Pacific/Tongatapu
    {   92, 7355,   54435,  3, }, // Pacific/Wake
    {   93, 6751,   53905,  3, }, // Africa/Abidjan Chinese/Traditional Han/Macao
    {   93,   14,   53908,  4, }, // Africa/Asmara
    {   93, 5159,   53912,  2, }, // Africa/Bangui
    {   93, 6736,   53914,  4, }, // Africa/Conakry
    {   93, 5032,   53918,  6, }, // Africa/Dar_es_Salaam
    {   93, 4381,   53924,  5, }, // Africa/Johannesburg
    {   93, 5208,   53929,  3, }, // Africa/Kinshasa
    {   93, 5238,   53932,  3, }, // Africa/Libreville
    {   93, 5113,   53935,  3, }, // Africa/Luanda
    {   93, 4903,   53938,  3, }, // Africa/Lusaka
    {   93, 5099,   53941,  3, }, // Africa/Maseru
    {   93, 3755,   53944,  3, }, // Africa/Nairobi
    {   93, 4054,   53947,  4, }, // Africa/Windhoek
    {   93,  132,   53951,  3, }, // America/Adak
    {   93, 4456,   53954,  5, }, // America/Araguaina
    {   93,  270,   53959,  4, }, // America/Argentina/Cordoba
    {   93, 7425,   53963,  4, }, // America/Argentina/La_Rioja
    {   93, 5557,   53967,  3, }, // America/Aruba
    {   93, 4154,   53970,  3, }, // America/Asuncion
    {   93, 3581,   53973,  3, }, // America/Bahia
    {   93, 7586,   53976,  8, }, // America/Bahia_Banderas
    {   93, 5309,   53984,  3, }, // America/Belize
    {   93, 7644,   53987,  5, }, // America/Boa_Vista
    {   93, 7719,   53992,  3, }, // America/Chihuahua
    {   93, 7737,   40365,  4, }, // America/Ciudad_Juarez
    {   93, 5324,   53995,  5, }, // America/Costa_Rica
    {   93, 3693,   54000,  3, }, // America/Cuiaba
    {   93, 7811,   54003,  3, }, // America/Dawson_Creek
    {   93,  893,   54006,  3, }, // America/Edmonton
    {   93, 7869,   54009,  4, }, // America/Fortaleza
    {   93, 7887,   54013,  4, }, // America/Glace_Bay
    {   93, 5687,   54017,  4, }, // America/Grenada
    {   93, 5703,   54021,  4, }, // America/Guadeloupe
    {   93, 3662,   54025,  4, }, // America/Guatemala
    {   93, 6895,   54029,  3, }, // America/Guyana
    {   93, 2221,   54032,  3, }, // America/Havana
    {   93,  348,   54035,  7, }, // America/Indiana/Indianapolis
    {   93,  481,   54042,  8, }, // America/Indiana/Knox
    {   93, 7924,   54050,  8, }, // America/Indiana/Marengo
    {   93, 7948,   54058,  8, }, // America/Indiana/Petersburg
    {   93, 7975,   54066,  8, }, // America/Indiana/Tell_City
    {   93, 8001,   54074,  7, }, // America/Indiana/Vevay
    {   93, 8023,   54081,  8, }, // America/Indiana/Vincennes
    {   93, 8049,   54089,  9, }, // America/Indiana/Winamac
    {   93, 8073,   54098,  4, }, // America/Inuvik
    {   93,  521,   54102,  4, }, // America/Kentucky/Louisville
    {   93, 8116,   54106,  3, }, // America/Maceio
    {   93, 8131,   54109,  3, }, // America/Managua
    {   93, 5722,   54112,  4, }, // America/Martinique
    {   93, 8165,   54116,  4, }, // America/Menominee
    {   93, 4000,   54120,  5, }, // America/Montevideo
    {   93, 5741,   54125,  5, }, // America/Montserrat
    {   93, 5383,   54130,  2, }, // America/Nassau
    {   93, 1834,   54132,  4, }, // America/Noronha
    {   93, 8264,   54136,  8, }, // America/North_Dakota/Beulah
    {   93, 8292,   54144,  8, }, // America/North_Dakota/Center
    {   93, 8320,   54152,  9, }, // America/North_Dakota/New_Salem
    {   93, 8367,   54161,  3, }, // America/Porto_Velho
    {   93,  695,   54164,  5, }, // America/Rio_Branco
    {   93, 2165,   54169,  4, }, // America/Santiago
    {   93, 6044,   54173,  4, }, // America/Santo_Domingo
    {   93, 8471,   54177,  5, }, // America/St_Barthelemy
    {   93, 2045,   54182,  4, }, // America/St_Johns
    {   93, 5797,   54186,  4, }, // America/St_Lucia
    {   93, 5830,   54190,  4, }, // America/St_Vincent
    {   93, 5363,   54194,  6, }, // America/Tegucigalpa
    {   93, 5673,   54200,  2, }, // America/Thule
    {   93,  313,   54202,  3, }, // America/Tijuana
    {   93, 2128,   54205,  3, }, // America/Whitehorse
    {   93, 8515,   54208,  4, }, // America/Yakutat
    {   93, 5918,   54212,  3, }, // Antarctica/Casey
    {   93, 6027,   54215,  4, }, // Antarctica/Davis
    {   93, 6066,   54219,  6, }, // Antarctica/DumontDUrville
    {   93, 8531,   54225,  3, }, // Antarctica/Macquarie
    {   93, 7029,   54228,  3, }, // Antarctica/Mawson
    {   93, 7076,   54231,  5, }, // Antarctica/McMurdo
    {   93, 8552,   54236,  3, }, // Antarctica/Palmer
    {   93, 7189,   54239,  4, }, // Antarctica/Rothera
    {   93, 7258,   54243,  3, }, // Antarctica/Syowa
    {   93, 8570,   54246,  4, }, // Antarctica/Troll
    {   93, 1413,   54250,  4, }, // Arctic/Longyearbyen
    {   93, 5428,   54254,  4, }, // Asia/Anadyr
    {   93, 3680,   54258,  4, }, // Asia/Bishkek
    {   93, 4413,   54262,  3, }, // Asia/Colombo
    {   93, 6092,   54265,  3, }, // Asia/Dushanbe
    {   93, 4697,   54268,  3, }, // Asia/Hebron
    {   93, 4086,   54271,  5, }, // Asia/Irkutsk
    {   93, 6955,   54276,  4, }, // Asia/Jayapura
    {   93, 1316,   54280,  2, }, // Asia/Makassar
    {   93, 6883,   54282,  4, }, // Asia/Muscat
    {   93, 2446,   54286,  4, }, // Asia/Nicosia
    {   93, 5476,   54290,  3, }, // Asia/Qatar
    {   93, 4730,   54293,  4, }, // Asia/Yakutsk
    {   93, 3785,   54297,  5, }, // Asia/Yekaterinburg
    {   93, 8699,   54302,  4, }, // Atlantic/Madeira
    {   93, 6607,   54306,  3, }, // Atlantic/Stanley
    {   93, 1654,   54309,  4, }, // Australia/Adelaide
    {   93, 1619,   54313,  4, }, // Australia/Brisbane
    {   93, 1500,   54317,  3, }, // Australia/Hobart
    {   93, 6285,   54320,  5, }, // Europe/Belgrade
    {   93, 6437,   54325,  6, }, // Europe/Bratislava
    {   93, 2475,   54331,  4, }, // Europe/Chisinau
    {   93, 8767,   54335,  3, }, // Europe/Guernsey
    {   93, 8783,   54338,  3, }, // Europe/Isle_of_Man
    {   93, 6455,   54341,  5, }, // Europe/Ljubljana
    {   93, 6346,   54346,  3, }, // Europe/Malta
    {   93, 6194,   54349,  4, }, // Europe/Sarajevo
    {   93, 6301,   54353,  3, }, // Europe/Vaduz
    {   93, 6210,   54356,  4, }, // Europe/Zagreb
    {   93, 6910,   54360,  5, }, // Indian/Chagos
    {   93, 5996,   54365,  5, }, // Indian/Cocos
    {   93, 4931,   41422,  3, }, // Indian/Comoro
    {   93, 6624,   54370,  6, }, // Indian/Kerguelen
    {   93, 7013,   54376,  4, }, // Indian/Maldives
    {   93, 3971,   54380,  4, }, // Indian/Mauritius
    {   93, 5000,   41443,  3, }, // Indian/Mayotte
    {   93, 7174,   41447,  3, }, // Indian/Reunion
    {   93, 3608,   54384,  5, }, // Pacific/Bougainville
    {   93, 7290,   54389,  5, }, // Pacific/Fakaofo
    {   93, 6659,   41486,  3, }, // Pacific/Gambier
    {   93, 3724,   54394,  7, }, // Pacific/Guadalcanal
    {   93, 2897,   54401,  5, }, // Pacific/Kanton
    {   93, 6982,   54406,  3, }, // Pacific/Kosrae
    {   93, 2708,   54409,  5, }, // Pacific/Kwajalein
    {   93, 4779,   54414,  3, }, // Pacific/Majuro
    {   93, 3953,   54417,  6, }, // Pacific/Marquesas
    {   93, 7047,   54423,  2, }, // Pacific/Nauru
    {   93, 7157,   54425,  5, }, // Pacific/Pitcairn
    {   93, 4485,   54430,  5, }, // Pacific/Tongatapu
    {   93, 7355,   54435,  3, }, // Pacific/Wake
    {   94, 6751,   40065,  3, }, // Africa/Abidjan Chinese/Traditional Han/Taiwan
    {   94, 4961,   40071,  6, }, // Africa/Addis_Ababa
    {   94, 7383,   40077,  4, }, // Africa/Algiers
    {   94,   14,   40081,  4, }, // Africa/Asmara
    {   94,   44,   40085,  3, }, // Africa/Bamako
    {   94, 6709,   40090,  2, }, // Africa/Banjul
    {   94, 7398,   40092,  2, }, // Africa/Bissau
    {   94, 4859,   40094,  4, }, // Africa/Blantyre
    {   94, 5189,   40098,  5, }, // Africa/Brazzaville
    {   94, 4824,   40103,  4, }, // Africa/Bujumbura
    {   94, 2242,   40107,  2, }, // Africa/Cairo
    {   94, 4019,   40109,  5, }, // Africa/Casablanca
    {   94, 7412,   40114,  2, }, // Africa/Ceuta
    {   94, 6736,   40116,  4, }, // Africa/Conakry
    {   94, 6803,   40120,  3, }, // Africa/Dakar
    {   94, 5032,   40123,  3, }, // Africa/Dar_es_Salaam
    {   94, 4945,   40126,  3, }, // Africa/Djibouti
    {   94, 6816,   40135,  3, }, // Africa/Freetown
    {   94, 4808,   40138,  4, }, // Africa/Gaborone
    {   94, 4381,   40145,  5, }, // Africa/Johannesburg
    {   94, 5208,   40161,  3, }, // Africa/Kinshasa
    {   94, 4646,   40164,  3, }, // Africa/Lagos
    {   94, 5238,   40167,  3, }, // Africa/Libreville
    {   94, 5113,   40172,  3, }, // Africa/Luanda
    {   94, 4841,   40175,  4, }, // Africa/Lubumbashi
    {   94, 4903,   40179,  3, }, // Africa/Lusaka
    {   94, 5224,   40182,  3, }, // Africa/Malabo
    {   94, 4889,   40185,  3, }, // Africa/Maputo
    {   94, 5099,   40188,  3, }, // Africa/Maseru
    {   94, 5084,   40191,  3, }, // Africa/Mbabane
    {   94, 5015,   40194,  4, }, // Africa/Mogadishu
    {   94, 6997,   40198,  4, }, // Africa/Monrovia
    {   94, 3755,   40202,  3, }, // Africa/Nairobi
    {   94, 5173,   40205,  4, }, // Africa/Ndjamena
    {   94, 5256,   40209,  3, }, // Africa/Niamey
    {   94, 6766,   40212,  3, }, // Africa/Nouakchott
    {   94, 5127,   40219,  4, }, // Africa/Porto-Novo
    {   94, 4337,   40223,  3, }, // Africa/Sao_Tome
    {   94, 4054,   40233,  4, }, // Africa/Windhoek
    {   94,  132,   40237,  3, }, // America/Adak
    {   94, 3183,   40240,  4, }, // America/Anchorage
    {   94, 5524,   54438,  3, }, // America/Anguilla
    {   94, 5541,   40247,  3, }, // America/Antigua
    {   94, 4456,   40250,  4, }, // America/Araguaina
    {   94,  166,   40254,  7, }, // America/Argentina/Buenos_Aires
    {   94,   91,   40261,  4, }, // America/Argentina/Catamarca
    {   94,  270,   40265,  3, }, // America/Argentina/Cordoba
    {   94, 7425,   40271,  3, }, // America/Argentina/La_Rioja
    {   94,  565,   40274,  3, }, // America/Argentina/Mendoza
    {   94, 7452,   40277,  6, }, // America/Argentina/Rio_Gallegos
    {   94, 7483,   40283,  3, }, // America/Argentina/Salta
    {   94, 7507,   40286,  3, }, // America/Argentina/San_Juan
    {   94, 5497,   40289,  3, }, // America/Argentina/San_Luis
    {   94, 7534,   40292,  3, }, // America/Argentina/Tucuman
    {   94, 7560,   40295,  4, }, // America/Argentina/Ushuaia
    {   94, 5557,   54441,  5, }, // America/Aruba
    {   94, 4154,   40302,  3, }, // America/Asuncion
    {   94, 3581,   40309,  3, }, // America/Bahia
    {   94, 7586,   40312,  7, }, // America/Bahia_Banderas
    {   94, 5571,   40319,  3, }, // America/Barbados
    {   94, 7609,   40322,  2, }, // America/Belem
    {   94, 5309,   40324,  3, }, // America/Belize
    {   94, 7623,   40327,  5, }, // America/Blanc-Sablon
    {   94, 7644,   40332,  4, }, // America/Boa_Vista
    {   94, 7662,   40339,  2, }, // America/Boise
    {   94, 7676,   40341,  3, }, // America/Cambridge_Bay
    {   94, 7698,   40344,  3, }, // America/Campo_Grande
    {   94, 4596,   40349,  4, }, // America/Caracas
    {   94, 4247,   40353,  2, }, // America/Cayenne
    {   94, 5398,   40355,  4, }, // America/Cayman
    {   94, 7719,   40362,  3, }, // America/Chihuahua
    {   94, 7737,   40365,  3, }, // America/Ciudad_Juarez
    {   94, 5324,   40369,  5, }, // America/Costa_Rica
    {   94, 7759,   40374,  4, }, // America/Creston
    {   94, 3693,   40378,  3, }, // America/Cuiaba
    {   94, 5640,   40381,  3, }, // America/Curacao
    {   94, 7775,   40384,  4, }, // America/Danmarkshavn
    {   94, 5656,   54446,  4, }, // America/Dominica
    {   94,  893,   40404,  4, }, // America/Edmonton
    {   94, 7832,   40408,  4, }, // America/Eirunepe
    {   94, 5343,   40412,  4, }, // America/El_Salvador
    {   94, 7849,   40416,  4, }, // America/Fort_Nelson
    {   94, 7869,   40420,  4, }, // America/Fortaleza
    {   94, 7887,   40424,  4, }, // America/Glace_Bay
    {   94, 6844,   40428,  2, }, // America/Goose_Bay
    {   94, 4514,   40430,  4, }, // America/Grand_Turk
    {   94, 5687,   40434,  4, }, // America/Grenada
    {   94, 5703,   40438,  4, }, // America/Guadeloupe
    {   94, 3662,   40442,  4, }, // America/Guatemala
    {   94, 4743,   40446,  4, }, // America/Guayaquil
    {   94, 6895,   40450,  3, }, // America/Guyana
    {   94, 1923,   40453,  5, }, // America/Halifax
    {   94, 7905,   40461,  4, }, // America/Hermosillo
    {   94,  348,   40465,  7, }, // America/Indiana/Indianapolis
    {   94,  481,   40472,  8, }, // America/Indiana/Knox
    {   94, 7924,   40480,  8, }, // America/Indiana/Marengo
    {   94, 7948,   40488,  8, }, // America/Indiana/Petersburg
    {   94, 7975,   40496,  8, }, // America/Indiana/Tell_City
    {   94, 8001,   40504,  7, }, // America/Indiana/Vevay
    {   94, 8023,   40511,  8, }, // America/Indiana/Vincennes
    {   94, 8049,   40519,  9, }, // America/Indiana/Winamac
    {   94, 8073,   40528,  4, }, // America/Inuvik
    {   94, 2665,   40535,  3, }, // America/Jamaica
    {   94, 5282,   40538,  2, }, // America/Juneau
    {   94,  521,   40540,  5, }, // America/Kentucky/Louisville
    {   94, 8088,   40545,  8, }, // America/Kentucky/Monticello
    {   94, 5621,   40553,  5, }, // America/Kralendijk
    {   94, 7132,   40561,  2, }, // America/Lima
    {   94, 3390,   40563,  3, }, // America/Los_Angeles
    {   94, 5849,   40566,  4, }, // America/Lower_Princes
    {   94, 8116,   40570,  3, }, // America/Maceio
    {   94, 8131,   40573,  3, }, // America/Managua
    {   94, 1892,   40576,  3, }, // America/Manaus
    {   94, 5814,   40579,  4, }, // America/Marigot
    {   94, 5722,   40583,  4, }, // America/Martinique
    {   94, 8147,   40587,  5, }, // America/Matamoros
    {   94, 2779,   40592,  4, }, // America/Mazatlan
    {   94, 8165,   40596,  4, }, // America/Menominee
    {   94, 8183,   40600,  3, }, // America/Merida
    {   94, 2811,   40609,  4, }, // America/Mexico_City
    {   94, 4293,   40613,  3, }, // America/Miquelon
    {   94, 8217,   40616,  3, }, // America/Moncton
    {   94, 8233,   40619,  3, }, // America/Monterrey
    {   94, 4000,   40622,  4, }, // America/Montevideo
    {   94, 5741,   40626,  3, }, // America/Montserrat
    {   94, 5383,   40629,  2, }, // America/Nassau
    {   94, 3294,   40631,  2, }, // America/New_York
    {   94, 8251,   40633,  2, }, // America/Nome
    {   94, 1834,   40635,  3, }, // America/Noronha
    {   94, 8264,   40638,  8, }, // America/North_Dakota/Beulah
    {   94, 8292,   40646,  7, }, // America/North_Dakota/Center
    {   94, 8320,   40653,  8, }, // America/North_Dakota/New_Salem
    {   94, 8351,   40663,  4, }, // America/Ojinaga
    {   94, 5413,   40667,  3, }, // America/Panama
    {   94, 6106,   40670,  5, }, // America/Paramaribo
    {   94, 3224,   40675,  3, }, // America/Phoenix
    {   94, 8367,   40685,  3, }, // America/Porto_Velho
    {   94, 3932,   40692,  6, }, // America/Punta_Arenas
    {   94, 8387,   40698,  3, }, // America/Rankin_Inlet
    {   94, 8408,   40701,  3, }, // America/Recife
    {   94, 1979,   40704,  3, }, // America/Regina
    {   94, 8423,   40707,  4, }, // America/Resolute
    {   94,  695,   40711,  4, }, // America/Rio_Branco
    {   94, 8440,   40715,  3, }, // America/Santarem
    {   94, 2165,   40718,  4, }, // America/Santiago
    {   94, 6044,   40722,  4, }, // America/Santo_Domingo
    {   94, 1862,   40726,  3, }, // America/Sao_Paulo
    {   94, 6862,   40729,  7, }, // America/Scoresbysund
    {   94, 8457,   40736,  3, }, // America/Sitka
    {   94, 8471,   40739,  6, }, // America/St_Barthelemy
    {   94, 2045,   40745,  3, }, // America/St_Johns
    {   94, 5780,   40748,  3, }, // America/St_Kitts
    {   94, 5797,   40751,  4, }, // America/St_Lucia
    {   94,  855,   40755,  4, }, // America/St_Thomas
    {   94, 5830,   40759,  3, }, // America/St_Vincent
    {   94, 8493,   40762,  7, }, // America/Swift_Current
    {   94, 5363,   40769,  5, }, // America/Tegucigalpa
    {   94, 5673,   40774,  2, }, // America/Thule
    {   94,  313,   40776,  3, }, // America/Tijuana
    {   94,  608,   40779,  3, }, // America/Toronto
    {   94, 5605,   40782,  4, }, // America/Tortola
    {   94, 2077,   40786,  3, }, // America/Vancouver
    {   94, 2128,   40789,  4, }, // America/Whitehorse
    {   94,  734,   40793,  3, }, // America/Winnipeg
    {   94, 8515,   40796,  3, }, // America/Yakutat
    {   94, 5918,   40799,  2, }, // Antarctica/Casey
    {   94, 6027,   40801,  3, }, // Antarctica/Davis
    {   94, 6066,   40804,  5, }, // Antarctica/DumontDUrville
    {   94, 8531,   40809,  3, }, // Antarctica/Macquarie
    {   94, 7076,   40814,  4, }, // Antarctica/McMurdo
    {   94, 8552,   40818,  2, }, // Antarctica/Palmer
    {   94, 7189,   40820,  3, }, // Antarctica/Rothera
    {   94, 7258,   40823,  4, }, // Antarctica/Syowa
    {   94, 8570,   40827,  2, }, // Antarctica/Troll
    {   94, 1413,   40833,  5, }, // Arctic/Longyearbyen
    {   94, 5487,   40838,  2, }, // Asia/Aden
    {   94, 5297,   40840,  4, }, // Asia/Almaty
    {   94, 5428,   40846,  3, }, // Asia/Anadyr
    {   94, 5270,   40852,  4, }, // Asia/Aqtobe
    {   94,  964,   40856,  5, }, // Asia/Ashgabat
    {   94, 8587,   40861,  3, }, // Asia/Atyrau
    {   94, 3509,   40864,  3, }, // Asia/Baghdad
    {   94, 3555,   40380,  2, }, // Asia/Baku
    {   94, 3473,   40871,  4, }, // Asia/Barnaul
    {   94, 3988,   40875,  3, }, // Asia/Beirut
    {   94, 3680,   40878,  4, }, // Asia/Bishkek
    {   94, 5906,   40882,  2, }, // Asia/Brunei
    {   94, 5963,   40886,  3, }, // Asia/Choibalsan
    {   94, 4413,   40889,  3, }, // Asia/Colombo
    {   94, 4442,   40892,  4, }, // Asia/Damascus
    {   94, 1060,   40896,  2, }, // Asia/Dhaka
    {   94, 3498,   40900,  2, }, // Asia/Dubai
    {   94, 6092,   40902,  3, }, // Asia/Dushanbe
    {   94, 8599,   40905,  5, }, // Asia/Famagusta
    {   94, 8614,   40910,  2, }, // Asia/Gaza
    {   94, 4697,   40912,  3, }, // Asia/Hebron
    {   94, 4086,   40936,  5, }, // Asia/Irkutsk
    {   94, 6969,   40941,  3, }, // Asia/Jakarta
    {   94, 6955,   40944,  4, }, // Asia/Jayapura
    {   94, 3462,   40952,  3, }, // Asia/Kabul
    {   94, 4141,   40958,  3, }, // Asia/Karachi
    {   94, 1152,   40961,  4, }, // Asia/Kathmandu
    {   94, 8624,   40965,  3, }, // Asia/Khandyga
    {   94,  992,   40968,  4, }, // Asia/Kolkata
    {   94, 4099,   40972,  8, }, // Asia/Krasnoyarsk
    {   94, 5893,   40983,  2, }, // Asia/Kuching
    {   94, 1178,   40995,  2, }, // Asia/Macau
    {   94, 3919,   41002,  3, }, // Asia/Magadan
    {   94, 1316,   41005,  4, }, // Asia/Makassar
    {   94, 7145,   41009,  3, }, // Asia/Manila
    {   94, 6883,   41012,  4, }, // Asia/Muscat
    {   94, 2446,   41016,  4, }, // Asia/Nicosia
    {   94, 8638,   41020,  6, }, // Asia/Novokuznetsk
    {   94, 4037,   41026,  5, }, // Asia/Novosibirsk
    {   94, 7108,   41035,  3, }, // Asia/Oral
    {   94, 6924,   41038,  2, }, // Asia/Phnom_Penh
    {   94, 5476,   41044,  2, }, // Asia/Qatar
    {   94, 8671,   54450,  4, }, // Asia/Qostanay
    {   94, 4171,   41050,  6, }, // Asia/Qyzylorda
    {   94, 3486,   41056,  3, }, // Asia/Riyadh
    {   94, 4310,   41059,  3, }, // Asia/Sakhalin
    {   94, 7208,   41062,  4, }, // Asia/Samarkand
    {   94, 3116,   41066,  2, }, // Asia/Seoul
    {   94, 2638,   41088,  3, }, // Asia/Tehran
    {   94, 2687,   41093,  2, }, // Asia/Tokyo
    {   94, 1346,   41099,  4, }, // Asia/Ulaanbaatar
    {   94, 1126,   41103,  4, }, // Asia/Urumqi
    {   94, 8685,   41107,  4, }, // Asia/Ust-Nera
    {   94, 6940,   41111,  2, }, // Asia/Vientiane
    {   94, 4612,   41113,  3, }, // Asia/Vladivostok
    {   94, 4730,   41116,  4, }, // Asia/Yakutsk
    {   94, 3785,   41122,  5, }, // Asia/Yekaterinburg
    {   94, 3649,   41127,  3, }, // Asia/Yerevan
    {   94, 3565,   41130,  5, }, // Atlantic/Azores
    {   94, 5588,   41135,  3, }, // Atlantic/Bermuda
    {   94, 6591,   41138,  3, }, // Atlantic/Canary
    {   94, 3629,   41141,  3, }, // Atlantic/Cape_Verde
    {   94, 1379,   41144,  4, }, // Atlantic/Faroe
    {   94, 8699,   41148,  5, }, // Atlantic/Madeira
    {   94, 2614,   41153,  5, }, // Atlantic/Reykjavik
    {   94, 7235,   41158,  4, }, // Atlantic/South_Georgia
    {   94, 6784,   41162,  5, }, // Atlantic/St_Helena
    {   94, 6607,   41167,  3, }, // Atlantic/Stanley
    {   94, 1654,   41170,  4, }, // Australia/Adelaide
    {   94, 1619,   41174,  4, }, // Australia/Brisbane
    {   94, 1783,   41178,  5, }, // Australia/Broken_Hill
    {   94, 1567,   41183,  3, }, // Australia/Darwin
    {   94, 1500,   41189,  3, }, // Australia/Hobart
    {   94, 1531,   41195,  4, }, // Australia/Lord_Howe
    {   94, 1711,   41199,  3, }, // Australia/Melbourne
    {   94, 1746,   41202,  2, }, // Australia/Perth
    {   94, 1447,   41204,  2, }, // Australia/Sydney
    {   94, 6149,   41211,  3, }, // Europe/Andorra
    {   94, 6285,   41221,  4, }, // Europe/Belgrade
    {   94, 6437,   41227,  6, }, // Europe/Bratislava
    {   94, 6178,   41233,  4, }, // Europe/Brussels
    {   94, 3708,   41242,  4, }, // Europe/Budapest
    {   94, 2475,   54454,  3, }, // Europe/Chisinau
    {   94, 2260,   41256,  3, }, // Europe/Dublin
    {   94, 6256,   41259,  4, }, // Europe/Gibraltar
    {   94, 8767,   41263,  3, }, // Europe/Guernsey
    {   94, 6561,   41266,  4, }, // Europe/Helsinki
    {   94, 8783,   41270,  2, }, // Europe/Isle_of_Man
    {   94, 1097,   41272,  4, }, // Europe/Istanbul
    {   94, 8802,   41276,  3, }, // Europe/Jersey
    {   94, 3881,   41279,  5, }, // Europe/Kaliningrad
    {   94, 2419,   41287,  2, }, // Europe/Kyiv
    {   94, 6455,   41292,  4, }, // Europe/Ljubljana
    {   94, 2393,   41296,  2, }, // Europe/London
    {   94, 6314,   41298,  3, }, // Europe/Luxembourg
    {   94, 4794,   41301,  3, }, // Europe/Madrid
    {   94, 6346,   41304,  3, }, // Europe/Malta
    {   94, 6531,   41307,  3, }, // Europe/Mariehamn
    {   94, 6359,   41313,  3, }, // Europe/Monaco
    {   94, 6407,   41319,  3, }, // Europe/Oslo
    {   94, 6373,   41324,  4, }, // Europe/Podgorica
    {   94, 6273,   41333,  2, }, // Europe/Rome
    {   94, 4233,   41335,  3, }, // Europe/Samara
    {   94, 6419,   41338,  4, }, // Europe/San_Marino
    {   94, 6194,   41342,  4, }, // Europe/Sarajevo
    {   94, 4353,   41346,  4, }, // Europe/Saratov
    {   94, 8828,   41350,  4, }, // Europe/Simferopol
    {   94, 6332,   41354,  4, }, // Europe/Skopje
    {   94, 6548,   41358,  3, }, // Europe/Sofia
    {   94, 6472,   41361,  5, }, // Europe/Stockholm
    {   94, 8861,   41371,  7, }, // Europe/Ulyanovsk
    {   94, 6301,   41378,  3, }, // Europe/Vaduz
    {   94, 6516,   41381,  3, }, // Europe/Vatican
    {   94, 6164,   41384,  3, }, // Europe/Vienna
    {   94, 8878,   41387,  4, }, // Europe/Vilnius
    {   94, 4629,   41391,  5, }, // Europe/Volgograd
    {   94, 3055,   41396,  2, }, // Europe/Warsaw
    {   94, 6210,   41398,  4, }, // Europe/Zagreb
    {   94, 6489,   41402,  3, }, // Europe/Zurich
    {   94, 4980,   54457,  6, }, // Indian/Antananarivo
    {   94, 5979,   41414,  3, }, // Indian/Christmas
    {   94, 5996,   41417,  5, }, // Indian/Cocos
    {   94, 4931,   41422,  5, }, // Indian/Comoro
    {   94, 6624,   41427,  5, }, // Indian/Kerguelen
    {   94, 7223,   41432,  3, }, // Indian/Mahe
    {   94, 7013,   41435,  4, }, // Indian/Maldives
    {   94, 3971,   41439,  4, }, // Indian/Mauritius
    {   94, 5000,   41443,  4, }, // Indian/Mayotte
    {   94, 7174,   41447,  4, }, // Indian/Reunion
    {   94, 4324,   41451,  3, }, // Pacific/Apia
    {   94,  932,   41454,  3, }, // Pacific/Auckland
    {   94, 3608,   41457,  4, }, // Pacific/Bougainville
    {   94, 2863,   41461,  2, }, // Pacific/Chatham
    {   94, 3022,   41463,  2, }, // Pacific/Chuuk
    {   94, 2201,   41465,  3, }, // Pacific/Easter
    {   94, 3804,   41474,  2, }, // Pacific/Fiji
    {   94, 7306,   41476,  4, }, // Pacific/Funafuti
    {   94, 6641,   41480,  6, }, // Pacific/Galapagos
    {   94, 6659,   41486,  5, }, // Pacific/Gambier
    {   94, 3724,   41491,  6, }, // Pacific/Guadalcanal
    {   94, 5935,   41497,  2, }, // Pacific/Guam
    {   94, 2897,   41502,  5, }, // Pacific/Kanton
    {   94, 3900,   41507,  6, }, // Pacific/Kiritimati
    {   94, 6982,   41513,  3, }, // Pacific/Kosrae
    {   94, 2708,   41516,  4, }, // Pacific/Kwajalein
    {   94, 4779,   41520,  3, }, // Pacific/Majuro
    {   94, 3953,   41523,  5, }, // Pacific/Marquesas
    {   94, 8909,   41528,  3, }, // Pacific/Midway
    {   94, 7047,   41531,  2, }, // Pacific/Nauru
    {   94, 7095,   41533,  3, }, // Pacific/Niue
    {   94, 4070,   41536,  3, }, // Pacific/Norfolk
    {   94, 7061,   41539,  3, }, // Pacific/Noumea
    {   94, 2991,   41542,  4, }, // Pacific/Pago_Pago
    {   94, 7118,   41546,  2, }, // Pacific/Palau
    {   94, 7157,   41548,  5, }, // Pacific/Pitcairn
    {   94, 2961,   41553,  3, }, // Pacific/Pohnpei
    {   94, 4709,   41556,  4, }, // Pacific/Port_Moresby
    {   94, 6009,   41560,  4, }, // Pacific/Rarotonga
    {   94, 7275,   41566,  3, }, // Pacific/Tahiti
    {   94, 4485,   41572,  5, }, // Pacific/Tongatapu
    {   95,   14,       0,  6, }, // Africa/Asmara Church/Cyrillic/Russia
    {   95, 4337,       6,  8, }, // Africa/Sao_Tome
    {   95, 4154,      14,  8, }, // America/Asuncion
    {   95,  237,      22,  8, }, // America/Atikokan
    {   95, 7586,      30, 17, }, // America/Bahia_Banderas
    {   95, 3770,      47,  6, }, // America/Cancun
    {   95, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   95, 5640,      66,  7, }, // America/Curacao
    {   95,  481,      73, 13, }, // America/Indiana/Knox
    {   95, 7924,      86, 16, }, // America/Indiana/Marengo
    {   95, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   95, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   95, 8001,     139, 14, }, // America/Indiana/Vevay
    {   95, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   95, 8049,     171, 16, }, // America/Indiana/Winamac
    {   95, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   95, 5849,     207, 22, }, // America/Lower_Princes
    {   95, 8183,     229,  6, }, // America/Merida
    {   95, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   95, 8292,     255, 20, }, // America/North_Dakota/Center
    {   95, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   95,  393,     298,  4, }, // America/Nuuk
    {   95, 6862,     302, 16, }, // America/Scoresbysund
    {   95, 8471,     318, 14, }, // America/St_Barthelemy
    {   95, 2045,     332, 10, }, // America/St_Johns
    {   95, 5780,     342,  9, }, // America/St_Kitts
    {   95, 5797,     351,  9, }, // America/St_Lucia
    {   95,  855,     360, 10, }, // America/St_Thomas
    {   95, 5830,     370, 11, }, // America/St_Vincent
    {   95, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   95, 5297,   54463,  8, }, // Asia/Almaty
    {   95, 5428,   54471,  9, }, // Asia/Anadyr
    {   95, 5440,   54480,  7, }, // Asia/Aqtau
    {   95, 5270,   54487, 12, }, // Asia/Aqtobe
    {   95, 3680,   54499,  8, }, // Asia/Bishkek
    {   95, 4503,   54507,  5, }, // Asia/Chita
    {   95, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   95, 4086,   54512, 10, }, // Asia/Irkutsk
    {   95, 4218,   54522, 27, }, // Asia/Kamchatka
    {   95, 1152,     413,  9, }, // Asia/Kathmandu
    {   95, 8624,   54549,  8, }, // Asia/Khandyga
    {   95,  992,     422,  7, }, // Asia/Kolkata
    {   95, 4099,   54557, 12, }, // Asia/Krasnoyarsk
    {   95, 1178,     429,  5, }, // Asia/Macau
    {   95, 3919,   54569,  9, }, // Asia/Magadan
    {   95, 8638,   54578, 13, }, // Asia/Novokuznetsk
    {   95, 4037,   54591, 13, }, // Asia/Novosibirsk
    {   95, 4131,   54604,  7, }, // Asia/Omsk
    {   95, 7108,   54611, 11, }, // Asia/Oral
    {   95, 4171,   54622, 10, }, // Asia/Qyzylorda
    {   95, 4310,   54632,  9, }, // Asia/Sakhalin
    {   95, 4199,   54641, 15, }, // Asia/Srednekolymsk
    {   95, 8685,   54656, 13, }, // Asia/Ust-Nera
    {   95, 4612,   54669, 13, }, // Asia/Vladivostok
    {   95, 4730,   54682,  9, }, // Asia/Yakutsk
    {   95, 1202,     442,  6, }, // Asia/Yangon
    {   95, 3785,   54691, 15, }, // Asia/Yekaterinburg
    {   95, 1379,     448,  5, }, // Atlantic/Faroe
    {   95, 6784,     453, 10, }, // Atlantic/St_Helena
    {   95, 3881,   54706, 13, }, // Europe/Kaliningrad
    {   95, 2419,   54719,  6, }, // Europe/Kyiv
    {   95, 3595,   54725,  7, }, // Europe/Minsk
    {   95, 3443,   54732,  7, }, // Europe/Moscow
    {   95, 4233,   54739,  7, }, // Europe/Samara
    {   95, 8828,   54746, 12, }, // Europe/Simferopol
    {   95, 4629,   54758, 11, }, // Europe/Volgograd
    {   95, 7174,     467,  7, }, // Indian/Reunion
    {   95, 3022,     474,  5, }, // Pacific/Chuuk
    {   95, 2961,     479,  7, }, // Pacific/Pohnpei
    {   96, 6751,   34826,  7, }, // Africa/Abidjan Chuvash/Cyrillic/Russia
    {   96, 6723,     492,  5, }, // Africa/Accra
    {   96, 4961,     497, 11, }, // Africa/Addis_Ababa
    {   96, 7383,     508,  5, }, // Africa/Algiers
    {   96,   14,   54769,  6, }, // Africa/Asmara
    {   96,   44,     519,  6, }, // Africa/Bamako
    {   96, 5159,     525,  5, }, // Africa/Bangui
    {   96, 6709,     530,  6, }, // Africa/Banjul
    {   96, 7398,     536,  5, }, // Africa/Bissau
    {   96, 4859,   25670,  8, }, // Africa/Blantyre
    {   96, 5189,   50080, 10, }, // Africa/Brazzaville
    {   96, 4824,     558,  9, }, // Africa/Bujumbura
    {   96, 2242,     567,  4, }, // Africa/Cairo
    {   96, 4019,     571, 10, }, // Africa/Casablanca
    {   96, 7412,     581,  5, }, // Africa/Ceuta
    {   96, 6736,     586,  7, }, // Africa/Conakry
    {   96, 6803,     593,  5, }, // Africa/Dakar
    {   96, 5032,   25698, 12, }, // Africa/Dar_es_Salaam
    {   96, 4945,   34859,  7, }, // Africa/Djibouti
    {   96, 5145,     616,  5, }, // Africa/Douala
    {   96, 5068,   25717,  7, }, // Africa/El_Aaiun
    {   96, 6816,     628,  7, }, // Africa/Freetown
    {   96, 4808,     635,  8, }, // Africa/Gaborone
    {   96, 4917,   32384,  6, }, // Africa/Harare
    {   96, 4381,   50098, 12, }, // Africa/Johannesburg
    {   96, 4401,   25755,  5, }, // Africa/Juba
    {   96, 5053,     665,  7, }, // Africa/Kampala
    {   96, 4426,     672,  6, }, // Africa/Khartoum
    {   96, 4875,     678,  6, }, // Africa/Kigali
    {   96, 5208,     684,  7, }, // Africa/Kinshasa
    {   96, 4646,     691,  5, }, // Africa/Lagos
    {   96, 5238,   50110,  9, }, // Africa/Libreville
    {   96, 6832,     704,  4, }, // Africa/Lome
    {   96, 5113,     708,  6, }, // Africa/Luanda
    {   96, 4841,     714,  9, }, // Africa/Lubumbashi
    {   96, 4903,     723,  6, }, // Africa/Lusaka
    {   96, 5224,     729,  6, }, // Africa/Malabo
    {   96, 4889,     735,  6, }, // Africa/Maputo
    {   96, 5099,     741,  6, }, // Africa/Maseru
    {   96, 5084,     747,  7, }, // Africa/Mbabane
    {   96, 5015,     754,  8, }, // Africa/Mogadishu
    {   96, 6997,     762,  7, }, // Africa/Monrovia
    {   96, 3755,   34907,  7, }, // Africa/Nairobi
    {   96, 5173,   25836,  8, }, // Africa/Ndjamena
    {   96, 5256,   34914,  6, }, // Africa/Niamey
    {   96, 6766,     790,  7, }, // Africa/Nouakchott
    {   96, 6690,     797,  8, }, // Africa/Ouagadougou
    {   96, 5127,     805, 10, }, // Africa/Porto-Novo
    {   96, 4337,   54775, 17, }, // Africa/Sao_Tome
    {   96, 2732,     823,  7, }, // Africa/Tripoli
    {   96, 6503,     830,  5, }, // Africa/Tunis
    {   96, 4054,     835,  7, }, // Africa/Windhoek
    {   96,  132,     842,  4, }, // America/Adak
    {   96, 3183,   50119,  8, }, // America/Anchorage
    {   96, 5524,   50127,  7, }, // America/Anguilla
    {   96, 5541,     860,  7, }, // America/Antigua
    {   96, 4456,     867,  9, }, // America/Araguaina
    {   96,  166,   50134, 12, }, // America/Argentina/Buenos_Aires
    {   96,   91,     888,  9, }, // America/Argentina/Catamarca
    {   96,  270,     897,  7, }, // America/Argentina/Cordoba
    {   96,  441,   25937,  5, }, // America/Argentina/Jujuy
    {   96, 7425,     909,  8, }, // America/Argentina/La_Rioja
    {   96,  565,     917,  7, }, // America/Argentina/Mendoza
    {   96, 7452,   50146, 12, }, // America/Argentina/Rio_Gallegos
    {   96, 7483,   25962,  6, }, // America/Argentina/Salta
    {   96, 7507,     940,  8, }, // America/Argentina/San_Juan
    {   96, 5497,   54792, 17, }, // America/Argentina/San_Luis
    {   96, 7534,     956,  7, }, // America/Argentina/Tucuman
    {   96, 7560,   25976,  5, }, // America/Argentina/Ushuaia
    {   96, 5557,     969,  5, }, // America/Aruba
    {   96, 4154,   50158,  8, }, // America/Asuncion
    {   96,  237,     981, 12, }, // America/Atikokan
    {   96, 3581,   34980,  4, }, // America/Bahia
    {   96, 7586,   50166, 16, }, // America/Bahia_Banderas
    {   96, 5571,    1013,  8, }, // America/Barbados
    {   96, 7609,    1021,  5, }, // America/Belem
    {   96, 5309,    1026,  5, }, // America/Belize
    {   96, 7623,    1031, 12, }, // America/Blanc-Sablon
    {   96, 7644,    1043,  9, }, // America/Boa_Vista
    {   96, 4263,    1052,  6, }, // America/Bogota
    {   96, 7662,   50182,  5, }, // America/Boise
    {   96, 7676,   50187, 13, }, // America/Cambridge_Bay
    {   96, 7698,    1075, 12, }, // America/Campo_Grande
    {   96, 3770,    1087,  6, }, // America/Cancun
    {   96, 4596,    1093,  7, }, // America/Caracas
    {   96, 4247,   50200,  7, }, // America/Cayenne
    {   96, 5398,   54809, 16, }, // America/Cayman
    {   96, 3251,    1129,  6, }, // America/Chicago
    {   96, 7719,    1135,  6, }, // America/Chihuahua
    {   96, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   96, 5324,    1141, 10, }, // America/Costa_Rica
    {   96, 7759,    1151,  7, }, // America/Creston
    {   96, 3693,   26138,  5, }, // America/Cuiaba
    {   96, 5640,   35086,  7, }, // America/Curacao
    {   96, 7775,    1172, 12, }, // America/Danmarkshavn
    {   96, 7796,    1184,  6, }, // America/Dawson
    {   96, 7811,    1190, 11, }, // America/Dawson_Creek
    {   96,  805,    1201,  6, }, // America/Denver
    {   96, 3351,   35129,  7, }, // America/Detroit
    {   96, 5656,    1214,  8, }, // America/Dominica
    {   96,  893,   50207,  8, }, // America/Edmonton
    {   96, 7832,   50215,  8, }, // America/Eirunepe
    {   96, 5343,   26215,  9, }, // America/El_Salvador
    {   96, 7849,   50223, 12, }, // America/Fort_Nelson
    {   96, 7869,    1257,  9, }, // America/Fortaleza
    {   96, 7887,   50235,  9, }, // America/Glace_Bay
    {   96, 6844,   50244,  7, }, // America/Goose_Bay
    {   96, 4514,    1282, 10, }, // America/Grand_Turk
    {   96, 5687,    1292,  7, }, // America/Grenada
    {   96, 5703,    1299,  9, }, // America/Guadeloupe
    {   96, 3662,    1308,  9, }, // America/Guatemala
    {   96, 4743,   50251,  8, }, // America/Guayaquil
    {   96, 6895,   50259,  6, }, // America/Guyana
    {   96, 1923,   50265,  8, }, // America/Halifax
    {   96, 2221,    1339,  6, }, // America/Havana
    {   96, 7905,   50273,  9, }, // America/Hermosillo
    {   96,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {   96,  481,    1365, 13, }, // America/Indiana/Knox
    {   96, 7924,    1378, 16, }, // America/Indiana/Marengo
    {   96, 7948,    1394, 19, }, // America/Indiana/Petersburg
    {   96, 7975,   54825, 18, }, // America/Indiana/Tell_City
    {   96, 8001,   54843, 14, }, // America/Indiana/Vevay
    {   96, 8023,   54857, 18, }, // America/Indiana/Vincennes
    {   96, 8049,   54875, 16, }, // America/Indiana/Winamac
    {   96, 8073,    1452,  6, }, // America/Inuvik
    {   96,  660,    1458,  7, }, // America/Iqaluit
    {   96, 2665,   26461,  6, }, // America/Jamaica
    {   96, 5282,   35226,  5, }, // America/Juneau
    {   96,  521,    1476,  8, }, // America/Kentucky/Louisville
    {   96, 8088,    1484, 20, }, // America/Kentucky/Monticello
    {   96, 5621,   35231, 10, }, // America/Kralendijk
    {   96, 4278,    1514,  6, }, // America/La_Paz
    {   96, 7132,    1520,  4, }, // America/Lima
    {   96, 3390,   50287, 12, }, // America/Los_Angeles
    {   96, 5849,   54891, 18, }, // America/Lower_Princes
    {   96, 8116,   35279,  6, }, // America/Maceio
    {   96, 8131,    1559,  7, }, // America/Managua
    {   96, 1892,    1566,  6, }, // America/Manaus
    {   96, 5814,    1572,  6, }, // America/Marigot
    {   96, 5722,    1578,  9, }, // America/Martinique
    {   96, 8147,    1587,  9, }, // America/Matamoros
    {   96, 2779,    1596,  8, }, // America/Mazatlan
    {   96, 8165,    1604,  8, }, // America/Menominee
    {   96, 8183,    1612,  6, }, // America/Merida
    {   96, 8198,    1618, 10, }, // America/Metlakatla
    {   96, 2811,    1628,  6, }, // America/Mexico_City
    {   96, 4293,    1634,  7, }, // America/Miquelon
    {   96, 8217,    1641,  7, }, // America/Moncton
    {   96, 8233,   50320,  9, }, // America/Monterrey
    {   96, 4000,    1657, 10, }, // America/Montevideo
    {   96, 5741,    1667, 10, }, // America/Montserrat
    {   96, 5383,    1677,  6, }, // America/Nassau
    {   96, 3294,   50338,  8, }, // America/New_York
    {   96, 8251,    1691,  3, }, // America/Nome
    {   96, 1834,   54909,  7, }, // America/Noronha
    {   96, 8264,   54916, 20, }, // America/North_Dakota/Beulah
    {   96, 8292,   54936, 20, }, // America/North_Dakota/Center
    {   96, 8320,   54956, 25, }, // America/North_Dakota/New_Salem
    {   96,  393,    1772,  4, }, // America/Nuuk
    {   96, 8351,    1776,  7, }, // America/Ojinaga
    {   96, 5413,    1783,  6, }, // America/Panama
    {   96, 6106,    1789, 10, }, // America/Paramaribo
    {   96, 3224,    1799,  6, }, // America/Phoenix
    {   96, 3847,    1805, 12, }, // America/Port-au-Prince
    {   96, 5871,   26771, 13, }, // America/Port_of_Spain
    {   96, 8367,   26784, 11, }, // America/Porto_Velho
    {   96, 5760,   50414, 11, }, // America/Puerto_Rico
    {   96, 3932,    1852, 12, }, // America/Punta_Arenas
    {   96, 8387,    1864, 12, }, // America/Rankin_Inlet
    {   96, 8408,    1876,  6, }, // America/Recife
    {   96, 1979,   50425,  8, }, // America/Regina
    {   96, 8423,   54981,  8, }, // America/Resolute
    {   96,  695,    1897, 10, }, // America/Rio_Branco
    {   96, 8440,    1907,  8, }, // America/Santarem
    {   96, 2165,   50433,  8, }, // America/Santiago
    {   96, 6044,    1923, 13, }, // America/Santo_Domingo
    {   96, 1862,   54989, 19, }, // America/Sao_Paulo
    {   96, 6862,    1945, 11, }, // America/Scoresbysund
    {   96, 8457,    1956,  5, }, // America/Sitka
    {   96, 8471,    1961, 13, }, // America/St_Barthelemy
    {   96, 2045,   26931, 10, }, // America/St_Johns
    {   96, 5780,    1983,  9, }, // America/St_Kitts
    {   96, 5797,   55008, 10, }, // America/St_Lucia
    {   96,  855,    2003, 10, }, // America/St_Thomas
    {   96, 5830,    2013, 12, }, // America/St_Vincent
    {   96, 8493,    2025, 13, }, // America/Swift_Current
    {   96, 5363,   50475, 12, }, // America/Tegucigalpa
    {   96, 5673,    2049,  4, }, // America/Thule
    {   96,  313,    2053,  7, }, // America/Tijuana
    {   96,  608,    2060,  7, }, // America/Toronto
    {   96, 5605,    2067,  7, }, // America/Tortola
    {   96, 2077,    2074,  8, }, // America/Vancouver
    {   96, 2128,   27017,  8, }, // America/Whitehorse
    {   96,  734,    2090,  8, }, // America/Winnipeg
    {   96, 8515,   27032,  6, }, // America/Yakutat
    {   96, 5918,   35501,  5, }, // Antarctica/Casey
    {   96, 6027,   35506,  6, }, // Antarctica/Davis
    {   96, 6066,   55018, 29, }, // Antarctica/DumontDUrville
    {   96, 8531,    2131,  8, }, // Antarctica/Macquarie
    {   96, 7029,    2139,  6, }, // Antarctica/Mawson
    {   96, 7076,    2145,  9, }, // Antarctica/McMurdo
    {   96, 8552,    2154,  6, }, // Antarctica/Palmer
    {   96, 7189,    2160,  6, }, // Antarctica/Rothera
    {   96, 7258,   27092,  4, }, // Antarctica/Syowa
    {   96, 8570,   50501,  6, }, // Antarctica/Troll
    {   96, 7337,   33428,  6, }, // Antarctica/Vostok
    {   96, 1413,   50507,  7, }, // Arctic/Longyearbyen
    {   96, 5487,    2192,  4, }, // Asia/Aden
    {   96, 5297,    2196,  6, }, // Asia/Almaty
    {   96, 3870,    2202,  5, }, // Asia/Amman
    {   96, 5428,   50528,  7, }, // Asia/Anadyr
    {   96, 5440,    2213,  5, }, // Asia/Aqtau
    {   96, 5270,    2218,  6, }, // Asia/Aqtobe
    {   96,  964,    2224,  7, }, // Asia/Ashgabat
    {   96, 8587,    2231,  6, }, // Asia/Atyrau
    {   96, 3509,    2237,  6, }, // Asia/Baghdad
    {   96, 5451,   35560,  7, }, // Asia/Bahrain
    {   96, 3555,    2250,  4, }, // Asia/Baku
    {   96, 4368,    2254,  7, }, // Asia/Bangkok
    {   96, 3473,    2261,  7, }, // Asia/Barnaul
    {   96, 3988,   27145,  6, }, // Asia/Beirut
    {   96, 3680,    2274,  6, }, // Asia/Bishkek
    {   96, 5906,   27157,  6, }, // Asia/Brunei
    {   96, 4503,    2286,  4, }, // Asia/Chita
    {   96, 5963,   35567,  9, }, // Asia/Choibalsan
    {   96, 4413,    2299,  7, }, // Asia/Colombo
    {   96, 4442,    2306,  6, }, // Asia/Damascus
    {   96, 1060,    2312,  5, }, // Asia/Dhaka
    {   96, 6125,    2317,  4, }, // Asia/Dili
    {   96, 3498,   27187,  5, }, // Asia/Dubai
    {   96, 6092,    2326,  7, }, // Asia/Dushanbe
    {   96, 8599,    2333,  9, }, // Asia/Famagusta
    {   96, 8614,    2342,  4, }, // Asia/Gaza
    {   96, 4697,    2346,  6, }, // Asia/Hebron
    {   96, 1226,    2352,  7, }, // Asia/Ho_Chi_Minh
    {   96, 2576,   50560,  7, }, // Asia/Hong_Kong
    {   96, 4673,    2366,  4, }, // Asia/Hovd
    {   96, 4086,    2370,  7, }, // Asia/Irkutsk
    {   96, 6969,   27229,  8, }, // Asia/Jakarta
    {   96, 6955,   27237,  8, }, // Asia/Jayapura
    {   96, 1257,    2392,  9, }, // Asia/Jerusalem
    {   96, 3462,    2401,  5, }, // Asia/Kabul
    {   96, 4218,   50585, 23, }, // Asia/Kamchatka
    {   96, 4141,    2431,  6, }, // Asia/Karachi
    {   96, 1152,    2437,  8, }, // Asia/Kathmandu
    {   96, 8624,    2445,  7, }, // Asia/Khandyga
    {   96,  992,   50608,  9, }, // Asia/Kolkata
    {   96, 4099,   35599, 10, }, // Asia/Krasnoyarsk
    {   96, 4761,    2471, 12, }, // Asia/Kuala_Lumpur
    {   96, 5893,    2483,  6, }, // Asia/Kuching
    {   96, 5464,   27284,  6, }, // Asia/Kuwait
    {   96, 1178,    2495,  5, }, // Asia/Macau
    {   96, 3919,    2500,  7, }, // Asia/Magadan
    {   96, 1316,    2507,  7, }, // Asia/Makassar
    {   96, 7145,    2514,  6, }, // Asia/Manila
    {   96, 6883,    2520,  6, }, // Asia/Muscat
    {   96, 2446,    2526,  6, }, // Asia/Nicosia
    {   96, 8638,    2533, 11, }, // Asia/Novokuznetsk
    {   96, 4037,    2544, 11, }, // Asia/Novosibirsk
    {   96, 4131,    2555,  4, }, // Asia/Omsk
    {   96, 7108,   27330,  7, }, // Asia/Oral
    {   96, 6924,   50617,  8, }, // Asia/Phnom_Penh
    {   96, 8656,    2572,  9, }, // Asia/Pontianak
    {   96, 4116,   27354,  7, }, // Asia/Pyongyang
    {   96, 5476,    2588,  5, }, // Asia/Qatar
    {   96, 8671,   35626,  8, }, // Asia/Qostanay
    {   96, 4171,    2601,  9, }, // Asia/Qyzylorda
    {   96, 3486,   50625,  7, }, // Asia/Riyadh
    {   96, 4310,   55047, 14, }, // Asia/Sakhalin
    {   96, 7208,    2630,  9, }, // Asia/Samarkand
    {   96, 3116,    2639,  4, }, // Asia/Seoul
    {   96, 1020,   27392,  6, }, // Asia/Shanghai
    {   96, 3137,    2649,  8, }, // Asia/Singapore
    {   96, 4199,    2657, 13, }, // Asia/Srednekolymsk
    {   96, 3100,   27420,  6, }, // Asia/Taipei
    {   96, 4683,    2676,  7, }, // Asia/Tashkent
    {   96, 3817,   33686,  7, }, // Asia/Tbilisi
    {   96, 2638,    2687,  7, }, // Asia/Tehran
    {   96, 1284,    2694,  7, }, // Asia/Thimphu
    {   96, 2687,    2701,  5, }, // Asia/Tokyo
    {   96, 4474,    2706,  5, }, // Asia/Tomsk
    {   96, 1346,    2711, 10, }, // Asia/Ulaanbaatar
    {   96, 1126,    2721,  6, }, // Asia/Urumqi
    {   96, 8685,   50651,  9, }, // Asia/Ust-Nera
    {   96, 6940,   50660,  8, }, // Asia/Vientiane
    {   96, 4612,    2742, 11, }, // Asia/Vladivostok
    {   96, 4730,   35672,  6, }, // Asia/Yakutsk
    {   96, 1202,   55061,  5, }, // Asia/Yangon
    {   96, 3785,    2766, 12, }, // Asia/Yekaterinburg
    {   96, 3649,    2778,  6, }, // Asia/Yerevan
    {   96, 3565,   55066, 14, }, // Atlantic/Azores
    {   96, 5588,   55080, 16, }, // Atlantic/Bermuda
    {   96, 6591,   55096, 15, }, // Atlantic/Canary
    {   96, 3629,    2835, 10, }, // Atlantic/Cape_Verde
    {   96, 1379,   55111, 15, }, // Atlantic/Faroe
    {   96, 8699,   35753,  7, }, // Atlantic/Madeira
    {   96, 2614,   50740,  9, }, // Atlantic/Reykjavik
    {   96, 7235,   55126, 13, }, // Atlantic/South_Georgia
    {   96, 6784,   55139, 20, }, // Atlantic/St_Helena
    {   96, 6607,   50782,  6, }, // Atlantic/Stanley
    {   96, 1654,    2926,  8, }, // Australia/Adelaide
    {   96, 1619,    2934,  7, }, // Australia/Brisbane
    {   96, 1783,    2941, 11, }, // Australia/Broken_Hill
    {   96, 1567,    2952,  6, }, // Australia/Darwin
    {   96, 3539,   27699,  4, }, // Australia/Eucla
    {   96, 1500,    2963,  6, }, // Australia/Hobart
    {   96, 8716,    2969,  8, }, // Australia/Lindeman
    {   96, 1531,    2977,  8, }, // Australia/Lord_Howe
    {   96, 1711,   27711,  8, }, // Australia/Melbourne
    {   96, 1746,    2992,  4, }, // Australia/Perth
    {   96, 1447,   50802,  6, }, // Australia/Sydney
    {   96, 6390,    3002,  9, }, // Europe/Amsterdam
    {   96, 6149,    3011,  7, }, // Europe/Andorra
    {   96, 3522,   27740,  9, }, // Europe/Astrakhan
    {   96, 6577,   55159,  7, }, // Europe/Athens
    {   96, 6285,    3030,  7, }, // Europe/Belgrade
    {   96, 4659,    3037,  6, }, // Europe/Berlin
    {   96, 6437,    3043, 10, }, // Europe/Bratislava
    {   96, 6178,   50822,  8, }, // Europe/Brussels
    {   96, 3830,    3061,  8, }, // Europe/Bucharest
    {   96, 3708,    3069,  8, }, // Europe/Budapest
    {   96, 8751,   35857,  8, }, // Europe/Busingen
    {   96, 2475,    3100,  7, }, // Europe/Chisinau
    {   96, 6238,   50849, 10, }, // Europe/Copenhagen
    {   96, 2260,    3117,  6, }, // Europe/Dublin
    {   96, 6256,    3123,  9, }, // Europe/Gibraltar
    {   96, 8767,    3132,  6, }, // Europe/Guernsey
    {   96, 6561,   50859,  9, }, // Europe/Helsinki
    {   96, 8783,   55166, 10, }, // Europe/Isle_of_Man
    {   96, 1097,    3155,  7, }, // Europe/Istanbul
    {   96, 8802,   50877,  6, }, // Europe/Jersey
    {   96, 3881,    3167, 11, }, // Europe/Kaliningrad
    {   96, 8924,    3178,  5, }, // Europe/Kirov
    {   96, 2419,    3183,  4, }, // Europe/Kyiv
    {   96, 3078,    3187,  8, }, // Europe/Lisbon
    {   96, 6455,   27891,  7, }, // Europe/Ljubljana
    {   96, 2393,    3204,  6, }, // Europe/London
    {   96, 6314,   27904, 10, }, // Europe/Luxembourg
    {   96, 4794,    3221,  6, }, // Europe/Madrid
    {   96, 6346,   27920,  6, }, // Europe/Malta
    {   96, 6531,    3232,  9, }, // Europe/Mariehamn
    {   96, 3595,    3241,  5, }, // Europe/Minsk
    {   96, 6359,    3246,  6, }, // Europe/Monaco
    {   96, 3443,   55176,  6, }, // Europe/Moscow
    {   96, 6407,    3258,  4, }, // Europe/Oslo
    {   96, 4186,    3262,  5, }, // Europe/Paris
    {   96, 6373,    3267,  9, }, // Europe/Podgorica
    {   96, 6224,    3276,  5, }, // Europe/Prague
    {   96, 8816,    3281,  4, }, // Europe/Riga
    {   96, 6273,    3285,  3, }, // Europe/Rome
    {   96, 4233,    3288,  6, }, // Europe/Samara
    {   96, 6419,    3294, 10, }, // Europe/San_Marino
    {   96, 6194,    3304,  7, }, // Europe/Sarajevo
    {   96, 4353,    3311,  7, }, // Europe/Saratov
    {   96, 8828,   50883, 11, }, // Europe/Simferopol
    {   96, 6332,   50894,  6, }, // Europe/Skopje
    {   96, 6548,    3333,  4, }, // Europe/Sofia
    {   96, 6472,   50900,  9, }, // Europe/Stockholm
    {   96, 8846,    3346,  6, }, // Europe/Tallinn
    {   96, 6135,    3352,  6, }, // Europe/Tirane
    {   96, 8861,   50909,  9, }, // Europe/Ulyanovsk
    {   96, 6301,    3367,  5, }, // Europe/Vaduz
    {   96, 6516,    3372,  7, }, // Europe/Vatican
    {   96, 6164,    3379,  4, }, // Europe/Vienna
    {   96, 8878,   50918,  7, }, // Europe/Vilnius
    {   96, 4629,    3390,  9, }, // Europe/Volgograd
    {   96, 3055,    3399,  7, }, // Europe/Warsaw
    {   96, 6210,    3406,  6, }, // Europe/Zagreb
    {   96, 6489,   35931,  5, }, // Europe/Zurich
    {   96, 4980,    3418, 12, }, // Indian/Antananarivo
    {   96, 6910,    3430,  5, }, // Indian/Chagos
    {   96, 5979,   55182, 13, }, // Indian/Christmas
    {   96, 5996,   55195, 15, }, // Indian/Cocos
    {   96, 4931,   55210,  8, }, // Indian/Comoro
    {   96, 6624,    3469,  8, }, // Indian/Kerguelen
    {   96, 7223,   28153,  3, }, // Indian/Mahe
    {   96, 7013,   55218, 10, }, // Indian/Maldives
    {   96, 3971,   55228,  8, }, // Indian/Mauritius
    {   96, 5000,   55236,  7, }, // Indian/Mayotte
    {   96, 7174,   51006,  7, }, // Indian/Reunion
    {   96, 4324,    3507,  4, }, // Pacific/Apia
    {   96,  932,    3511,  6, }, // Pacific/Auckland
    {   96, 3608,   51013,  9, }, // Pacific/Bougainville
    {   96, 2863,    3525,  5, }, // Pacific/Chatham
    {   96, 3022,    3530,  4, }, // Pacific/Chuuk
    {   96, 2201,   55243, 13, }, // Pacific/Easter
    {   96, 7323,   51067,  5, }, // Pacific/Efate
    {   96, 7290,    3550,  7, }, // Pacific/Fakaofo
    {   96, 3804,   36028,  5, }, // Pacific/Fiji
    {   96, 7306,    3561,  8, }, // Pacific/Funafuti
    {   96, 6641,   55256, 19, }, // Pacific/Galapagos
    {   96, 6659,   55275, 16, }, // Pacific/Gambier
    {   96, 3724,    3604, 11, }, // Pacific/Guadalcanal
    {   96, 5935,    3615,  4, }, // Pacific/Guam
    {   96, 2897,    3619,  6, }, // Pacific/Kanton
    {   96, 3900,    3625, 10, }, // Pacific/Kiritimati
    {   96, 6982,    3635,  6, }, // Pacific/Kosrae
    {   96, 2708,   28300, 10, }, // Pacific/Kwajalein
    {   96, 4779,   36063,  7, }, // Pacific/Majuro
    {   96, 3953,   55291, 18, }, // Pacific/Marquesas
    {   96, 8909,   51148,  6, }, // Pacific/Midway
    {   96, 7047,    3689,  5, }, // Pacific/Nauru
    {   96, 7095,   51162,  4, }, // Pacific/Niue
    {   96, 4070,    3698,  7, }, // Pacific/Norfolk
    {   96, 7061,    3705,  5, }, // Pacific/Noumea
    {   96, 2991,    3710,  9, }, // Pacific/Pago_Pago
    {   96, 7118,    3719,  5, }, // Pacific/Palau
    {   96, 7157,   55309,  7, }, // Pacific/Pitcairn
    {   96, 2961,    3731,  6, }, // Pacific/Pohnpei
    {   96, 4709,    3737, 11, }, // Pacific/Port_Moresby
    {   96, 6009,    3748,  9, }, // Pacific/Rarotonga
    {   96, 5948,   28394,  6, }, // Pacific/Saipan
    {   96, 7275,    3763,  5, }, // Pacific/Tahiti
    {   96, 6675,    3768,  6, }, // Pacific/Tarawa
    {   96, 4485,    3774,  9, }, // Pacific/Tongatapu
    {   96, 7355,   28414,  4, }, // Pacific/Wake
    {   96, 7368,    3787,  6, }, // Pacific/Wallis
    {   97,   14,       0,  6, }, // Africa/Asmara Colognian/Latin/Germany
    {   97, 2242,   55316,  6, }, // Africa/Cairo
    {   97, 4646,   55322,  6, }, // Africa/Lagos
    {   97, 4337,       6,  8, }, // Africa/Sao_Tome
    {   97, 4054,   55328,  7, }, // Africa/Windhoek
    {   97, 4154,      14,  8, }, // America/Asuncion
    {   97,  237,      22,  8, }, // America/Atikokan
    {   97, 7586,      30, 17, }, // America/Bahia_Banderas
    {   97, 3770,      47,  6, }, // America/Cancun
    {   97, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   97, 5324,   55335, 10, }, // America/Costa_Rica
    {   97, 5640,      66,  7, }, // America/Curacao
    {   97, 4743,   55345,  9, }, // America/Guayaquil
    {   97,  481,   55354, 15, }, // America/Indiana/Knox
    {   97, 7924,   55369, 18, }, // America/Indiana/Marengo
    {   97, 7948,   55387, 21, }, // America/Indiana/Petersburg
    {   97, 7975,   55408, 20, }, // America/Indiana/Tell_City
    {   97, 8001,   55428, 16, }, // America/Indiana/Vevay
    {   97, 8023,   55444, 20, }, // America/Indiana/Vincennes
    {   97, 8049,   55464, 18, }, // America/Indiana/Winamac
    {   97, 8088,   55482, 22, }, // America/Kentucky/Monticello
    {   97, 5849,     207, 22, }, // America/Lower_Princes
    {   97, 8183,     229,  6, }, // America/Merida
    {   97, 2811,   55504, 14, }, // America/Mexico_City
    {   97, 8264,   55518, 21, }, // America/North_Dakota/Beulah
    {   97, 8292,   55539, 21, }, // America/North_Dakota/Center
    {   97, 8320,   55560, 24, }, // America/North_Dakota/New_Salem
    {   97,  393,     298,  4, }, // America/Nuuk
    {   97, 6862,     302, 16, }, // America/Scoresbysund
    {   97, 8471,   55584, 15, }, // America/St_Barthelemy
    {   97, 2045,   55599, 11, }, // America/St_Johns
    {   97, 5780,   55610, 10, }, // America/St_Kitts
    {   97, 5797,   55620, 16, }, // America/St_Lucia
    {   97,  855,   55636, 11, }, // America/St_Thomas
    {   97, 5830,   55647, 15, }, // America/St_Vincent
    {   97, 8515,    5813,  7, }, // America/Yakutat
    {   97, 6066,   55662, 27, }, // Antarctica/DumontDUrville
    {   97, 7258,   55689, 36, }, // Antarctica/Syowa
    {   97, 7337,    4048,  6, }, // Antarctica/Vostok
    {   97, 5270,   55725,  6, }, // Asia/Aqtobe
    {   97,  964,   55731,  9, }, // Asia/Ashgabat
    {   97, 3509,    4063,  6, }, // Asia/Baghdad
    {   97, 3680,   55740,  8, }, // Asia/Bishkek
    {   97, 4442,    4102,  8, }, // Asia/Damascus
    {   97, 6092,   55748,  9, }, // Asia/Dushanbe
    {   97, 8614,   55757,  5, }, // Asia/Gaza
    {   97, 1226,   55762, 30, }, // Asia/Ho_Chi_Minh
    {   97, 2576,    4132,  8, }, // Asia/Hong_Kong
    {   97, 4218,   55792, 11, }, // Asia/Kamchatka
    {   97, 1152,     413,  9, }, // Asia/Kathmandu
    {   97,  992,     422,  7, }, // Asia/Kolkata
    {   97, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {   97, 1178,     429,  5, }, // Asia/Macau
    {   97, 2446,   55803,  8, }, // Asia/Nicosia
    {   97, 6924,   55811,  9, }, // Asia/Phnom_Penh
    {   97, 4171,   55820,  9, }, // Asia/Qyzylorda
    {   97, 3486,   32042,  5, }, // Asia/Riyadh
    {   97, 1020,   55829,  9, }, // Asia/Shanghai
    {   97, 3137,   55838, 10, }, // Asia/Singapore
    {   97, 4683,   55848,  9, }, // Asia/Tashkent
    {   97, 3817,   55857,  6, }, // Asia/Tbilisi
    {   97, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {   97, 1126,   55863, 11, }, // Asia/Urumqi
    {   97, 4612,    4262, 11, }, // Asia/Vladivostok
    {   97, 4730,    6206,  7, }, // Asia/Yakutsk
    {   97, 1202,   55874,  6, }, // Asia/Yangon
    {   97, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {   97, 3649,   55880,  6, }, // Asia/Yerevan
    {   97, 3565,   55886,  6, }, // Atlantic/Azores
    {   97, 6591,   55892,  8, }, // Atlantic/Canary
    {   97, 3629,   55900,  9, }, // Atlantic/Cape_Verde
    {   97, 1379,   55909,  5, }, // Atlantic/Faroe
    {   97, 6784,   55914, 11, }, // Atlantic/St_Helena
    {   97, 6577,    4339,  5, }, // Europe/Athens
    {   97, 6285,   55925,  7, }, // Europe/Belgrade
    {   97, 6178,   21176,  7, }, // Europe/Brussels
    {   97, 3830,   25479,  8, }, // Europe/Bucharest
    {   97, 6238,   55932, 10, }, // Europe/Copenhagen
    {   97, 6256,   55942,  9, }, // Europe/Gibraltar
    {   97, 8767,   55951,  7, }, // Europe/Guernsey
    {   97, 8783,   55958,  9, }, // Europe/Isle_of_Man
    {   97, 1097,   55967,  8, }, // Europe/Istanbul
    {   97, 8802,   55975,  6, }, // Europe/Jersey
    {   97, 3881,   55981, 13, }, // Europe/Kaliningrad
    {   97, 2419,     463,  4, }, // Europe/Kyiv
    {   97, 3078,    4400,  8, }, // Europe/Lisbon
    {   97, 6314,   55994, 11, }, // Europe/Luxembourg
    {   97, 3443,   56005,  6, }, // Europe/Moscow
    {   97, 6224,    6545,  4, }, // Europe/Prague
    {   97, 8816,   56011,  5, }, // Europe/Riga
    {   97, 6273,    6554,  3, }, // Europe/Rome
    {   97, 6135,   56016,  7, }, // Europe/Tirane
    {   97, 6516,   56023, 12, }, // Europe/Vatican
    {   97, 6164,   56035,  4, }, // Europe/Vienna
    {   97, 4629,   56039,  9, }, // Europe/Volgograd
    {   97, 3055,   56048,  8, }, // Europe/Warsaw
    {   97, 6210,   56056,  6, }, // Europe/Zagreb
    {   97, 6489,   56062,  7, }, // Europe/Zurich
    {   97, 7013,   56069,  8, }, // Indian/Maldives
    {   97, 7174,     467,  7, }, // Indian/Reunion
    {   97, 3022,     474,  5, }, // Pacific/Chuuk
    {   97, 6641,   56077, 10, }, // Pacific/Galapagos
    {   97, 5935,   56087,  4, }, // Pacific/Guam
    {   97, 2929,   56091,  8, }, // Pacific/Honolulu
    {   97, 7157,    6773,  7, }, // Pacific/Pitcairn
    {   97, 2961,     479,  7, }, // Pacific/Pohnpei
    {   98,   14,       0,  6, }, // Africa/Asmara Cornish/Latin/United Kingdom
    {   98, 4337,       6,  8, }, // Africa/Sao_Tome
    {   98, 4154,      14,  8, }, // America/Asuncion
    {   98,  237,      22,  8, }, // America/Atikokan
    {   98, 7586,      30, 17, }, // America/Bahia_Banderas
    {   98, 3770,      47,  6, }, // America/Cancun
    {   98, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   98, 5640,      66,  7, }, // America/Curacao
    {   98,  481,      73, 13, }, // America/Indiana/Knox
    {   98, 7924,      86, 16, }, // America/Indiana/Marengo
    {   98, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   98, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   98, 8001,     139, 14, }, // America/Indiana/Vevay
    {   98, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   98, 8049,     171, 16, }, // America/Indiana/Winamac
    {   98, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   98, 5849,     207, 22, }, // America/Lower_Princes
    {   98, 8183,     229,  6, }, // America/Merida
    {   98, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   98, 8292,     255, 20, }, // America/North_Dakota/Center
    {   98, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   98,  393,     298,  4, }, // America/Nuuk
    {   98, 6862,     302, 16, }, // America/Scoresbysund
    {   98, 8471,     318, 14, }, // America/St_Barthelemy
    {   98, 2045,     332, 10, }, // America/St_Johns
    {   98, 5780,     342,  9, }, // America/St_Kitts
    {   98, 5797,     351,  9, }, // America/St_Lucia
    {   98,  855,     360, 10, }, // America/St_Thomas
    {   98, 5830,     370, 11, }, // America/St_Vincent
    {   98, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   98, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   98, 1152,     413,  9, }, // Asia/Kathmandu
    {   98,  992,     422,  7, }, // Asia/Kolkata
    {   98, 1178,     429,  5, }, // Asia/Macau
    {   98, 1202,     442,  6, }, // Asia/Yangon
    {   98, 1379,     448,  5, }, // Atlantic/Faroe
    {   98, 6784,     453, 10, }, // Atlantic/St_Helena
    {   98, 2419,     463,  4, }, // Europe/Kyiv
    {   98, 7174,     467,  7, }, // Indian/Reunion
    {   98, 3022,     474,  5, }, // Pacific/Chuuk
    {   98, 2961,     479,  7, }, // Pacific/Pohnpei
    {   99,   14,       0,  6, }, // Africa/Asmara Corsican/Latin/France
    {   99, 4337,       6,  8, }, // Africa/Sao_Tome
    {   99, 4154,      14,  8, }, // America/Asuncion
    {   99,  237,      22,  8, }, // America/Atikokan
    {   99, 7586,      30, 17, }, // America/Bahia_Banderas
    {   99, 3770,      47,  6, }, // America/Cancun
    {   99, 7737,      53, 13, }, // America/Ciudad_Juarez
    {   99, 5640,      66,  7, }, // America/Curacao
    {   99,  481,      73, 13, }, // America/Indiana/Knox
    {   99, 7924,      86, 16, }, // America/Indiana/Marengo
    {   99, 7948,     102, 19, }, // America/Indiana/Petersburg
    {   99, 7975,     121, 18, }, // America/Indiana/Tell_City
    {   99, 8001,     139, 14, }, // America/Indiana/Vevay
    {   99, 8023,     153, 18, }, // America/Indiana/Vincennes
    {   99, 8049,     171, 16, }, // America/Indiana/Winamac
    {   99, 8088,     187, 20, }, // America/Kentucky/Monticello
    {   99, 5849,     207, 22, }, // America/Lower_Princes
    {   99, 8183,     229,  6, }, // America/Merida
    {   99, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {   99, 8292,     255, 20, }, // America/North_Dakota/Center
    {   99, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {   99,  393,     298,  4, }, // America/Nuuk
    {   99, 6862,     302, 16, }, // America/Scoresbysund
    {   99, 8471,     318, 14, }, // America/St_Barthelemy
    {   99, 2045,     332, 10, }, // America/St_Johns
    {   99, 5780,     342,  9, }, // America/St_Kitts
    {   99, 5797,     351,  9, }, // America/St_Lucia
    {   99,  855,     360, 10, }, // America/St_Thomas
    {   99, 5830,     370, 11, }, // America/St_Vincent
    {   99, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {   99, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {   99, 1152,     413,  9, }, // Asia/Kathmandu
    {   99,  992,     422,  7, }, // Asia/Kolkata
    {   99, 1178,     429,  5, }, // Asia/Macau
    {   99, 1202,     442,  6, }, // Asia/Yangon
    {   99, 1379,     448,  5, }, // Atlantic/Faroe
    {   99, 6784,     453, 10, }, // Atlantic/St_Helena
    {   99, 2419,     463,  4, }, // Europe/Kyiv
    {   99, 7174,     467,  7, }, // Indian/Reunion
    {   99, 3022,     474,  5, }, // Pacific/Chuuk
    {   99, 2961,     479,  7, }, // Pacific/Pohnpei
    {  100, 4961,    3793, 11, }, // Africa/Addis_Ababa Croatian/Latin/Croatia
    {  100, 7383,   56099,  5, }, // Africa/Algiers
    {  100,   14,       0,  6, }, // Africa/Asmara
    {  100, 2242,   24913,  5, }, // Africa/Cairo
    {  100, 4945,   31754,  7, }, // Africa/Djibouti
    {  100, 6832,   18802,  4, }, // Africa/Lome
    {  100, 4337,       6,  8, }, // Africa/Sao_Tome
    {  100, 5524,   31768,  7, }, // America/Anguilla
    {  100, 4154,      14,  8, }, // America/Asuncion
    {  100,  237,      22,  8, }, // America/Atikokan
    {  100, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  100, 3770,    3861,  6, }, // America/Cancun
    {  100, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  100, 5324,   31782,  9, }, // America/Costa_Rica
    {  100, 5640,      66,  7, }, // America/Curacao
    {  100, 5656,   20071,  8, }, // America/Dominica
    {  100, 5343,    5090,  8, }, // America/El_Salvador
    {  100, 5703,   18917,  9, }, // America/Guadeloupe
    {  100, 3662,   31800,  9, }, // America/Guatemala
    {  100, 6895,   56104,  7, }, // America/Guyana
    {  100,  481,      73, 13, }, // America/Indiana/Knox
    {  100, 7924,      86, 16, }, // America/Indiana/Marengo
    {  100, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  100, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  100, 8001,     139, 14, }, // America/Indiana/Vevay
    {  100, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  100, 8049,     171, 16, }, // America/Indiana/Winamac
    {  100, 2665,   31809,  7, }, // America/Jamaica
    {  100, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  100, 5849,     207, 22, }, // America/Lower_Princes
    {  100, 8183,    3899,  6, }, // America/Merida
    {  100, 2811,   56111, 16, }, // America/Mexico_City
    {  100, 8264,   31816, 23, }, // America/North_Dakota/Beulah
    {  100, 8292,   31839, 23, }, // America/North_Dakota/Center
    {  100, 8320,   31862, 26, }, // America/North_Dakota/New_Salem
    {  100,  393,     298,  4, }, // America/Nuuk
    {  100, 5760,   31888,  9, }, // America/Puerto_Rico
    {  100, 6862,     302, 16, }, // America/Scoresbysund
    {  100, 8471,   34416, 16, }, // America/St_Barthelemy
    {  100, 2045,     332, 10, }, // America/St_Johns
    {  100, 5780,     342,  9, }, // America/St_Kitts
    {  100, 5797,     351,  9, }, // America/St_Lucia
    {  100,  855,     360, 10, }, // America/St_Thomas
    {  100, 5830,     370, 11, }, // America/St_Vincent
    {  100, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  100, 5297,   56127,  8, }, // Asia/Almaty
    {  100, 5428,    5900,  6, }, // Asia/Anadyr
    {  100, 5440,    5906,  5, }, // Asia/Aqtau
    {  100, 5270,    5911,  6, }, // Asia/Aqtobe
    {  100,  964,   56135,  7, }, // Asia/Ashgabat
    {  100, 3509,    4063,  6, }, // Asia/Baghdad
    {  100, 5451,    4069,  7, }, // Asia/Bahrain
    {  100, 3680,   31917,  6, }, // Asia/Bishkek
    {  100, 5906,   31923,  6, }, // Asia/Brunei
    {  100, 4503,   56142,  4, }, // Asia/Chita
    {  100, 4442,    4102,  6, }, // Asia/Damascus
    {  100, 6092,   31939,  7, }, // Asia/Dushanbe
    {  100, 1226,   31946,  9, }, // Asia/Ho_Chi_Minh
    {  100, 6969,   31955,  8, }, // Asia/Jakarta
    {  100, 1257,   31972,  9, }, // Asia/Jerusalem
    {  100, 4218,   31981,  8, }, // Asia/Kamchatka
    {  100, 1152,     413,  9, }, // Asia/Kathmandu
    {  100, 8624,   31995,  7, }, // Asia/Khandyga
    {  100,  992,     422,  7, }, // Asia/Kolkata
    {  100, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  100, 5464,    6058,  6, }, // Asia/Kuwait
    {  100, 1178,    6064,  5, }, // Asia/Macau
    {  100, 2446,   32013,  8, }, // Asia/Nicosia
    {  100, 8638,   32021, 12, }, // Asia/Novokuznetsk
    {  100, 4116,   32033,  9, }, // Asia/Pyongyang
    {  100, 5476,    4233,  5, }, // Asia/Qatar
    {  100, 8671,     434,  8, }, // Asia/Qostanay
    {  100, 4171,    6111,  9, }, // Asia/Qyzylorda
    {  100, 3486,   32042,  5, }, // Asia/Riyadh
    {  100, 4310,   32047,  7, }, // Asia/Sakhalin
    {  100, 1020,   32054,  6, }, // Asia/Shanghai
    {  100, 3137,   19291,  8, }, // Asia/Singapore
    {  100, 4199,    6150, 13, }, // Asia/Srednekolymsk
    {  100, 4683,   32060,  7, }, // Asia/Tashkent
    {  100, 2638,    6175,  7, }, // Asia/Tehran
    {  100, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  100, 4730,    6206,  7, }, // Asia/Yakutsk
    {  100, 1202,   56146,  7, }, // Asia/Yangon
    {  100, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  100, 3649,   25397,  6, }, // Asia/Yerevan
    {  100, 3565,   56153, 13, }, // Atlantic/Azores
    {  100, 6591,    4299,  6, }, // Atlantic/Canary
    {  100, 1379,   56166, 14, }, // Atlantic/Faroe
    {  100, 7235,   56180, 14, }, // Atlantic/South_Georgia
    {  100, 6784,     453, 10, }, // Atlantic/St_Helena
    {  100, 6149,   32109,  6, }, // Europe/Andorra
    {  100, 3522,   32115,  8, }, // Europe/Astrakhan
    {  100, 6577,   25459,  5, }, // Europe/Athens
    {  100, 6285,    6398,  7, }, // Europe/Belgrade
    {  100, 6178,   56194,  9, }, // Europe/Brussels
    {  100, 3830,   32134,  8, }, // Europe/Bucharest
    {  100, 3708,   32142, 10, }, // Europe/Budapest
    {  100, 2475,   32152,  8, }, // Europe/Chisinau
    {  100, 6238,    4376, 10, }, // Europe/Copenhagen
    {  100, 8783,   56203,  8, }, // Europe/Isle_of_Man
    {  100, 3881,   32182, 12, }, // Europe/Kaliningrad
    {  100, 2419,   32194,  5, }, // Europe/Kyiv
    {  100, 3078,   32199,  7, }, // Europe/Lisbon
    {  100, 6314,    6510, 10, }, // Europe/Luxembourg
    {  100, 6359,    6525,  6, }, // Europe/Monaco
    {  100, 3443,   21290,  6, }, // Europe/Moscow
    {  100, 4186,   32206,  5, }, // Europe/Paris
    {  100, 6224,    6545,  4, }, // Europe/Prague
    {  100, 6273,   32211,  3, }, // Europe/Rome
    {  100, 8828,   32214, 11, }, // Europe/Simferopol
    {  100, 6332,   32225,  7, }, // Europe/Skopje
    {  100, 6548,   32232,  6, }, // Europe/Sofia
    {  100, 6135,   19608,  6, }, // Europe/Tirane
    {  100, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  100, 6516,    6614,  7, }, // Europe/Vatican
    {  100, 6164,   32246,  3, }, // Europe/Vienna
    {  100, 3055,   32249,  7, }, // Europe/Warsaw
    {  100, 6489,    4478,  6, }, // Europe/Zurich
    {  100, 7013,   32290,  7, }, // Indian/Maldives
    {  100, 3971,   32297, 10, }, // Indian/Mauritius
    {  100, 7174,     467,  7, }, // Indian/Reunion
    {  100, 3022,     474,  5, }, // Pacific/Chuuk
    {  100, 3804,   32307,  5, }, // Pacific/Fiji
    {  100, 2929,    4512,  8, }, // Pacific/Honolulu
    {  100, 2897,    4520,  9, }, // Pacific/Kanton
    {  100, 3953,   56211, 18, }, // Pacific/Marquesas
    {  100, 2961,     479,  7, }, // Pacific/Pohnpei
    {  102, 6751,   56229,  7, }, // Africa/Abidjan Czech/Latin/Czechia
    {  102, 4961,    3793, 11, }, // Africa/Addis_Ababa
    {  102, 7383,   56236,  5, }, // Africa/Algiers
    {  102,   14,       0,  6, }, // Africa/Asmara
    {  102, 2242,   56241,  6, }, // Africa/Cairo
    {  102, 4945,   31754,  7, }, // Africa/Djibouti
    {  102, 4426,   56247,  7, }, // Africa/Khartoum
    {  102, 6832,   18802,  4, }, // Africa/Lome
    {  102, 5015,   56254,  8, }, // Africa/Mogadishu
    {  102, 5173,   56262,  8, }, // Africa/Ndjamena
    {  102, 6766,   56270,  8, }, // Africa/Nouakchott
    {  102, 4337,   56278, 11, }, // Africa/Sao_Tome
    {  102, 2732,   56289,  8, }, // Africa/Tripoli
    {  102,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  102, 4154,      14,  8, }, // America/Asuncion
    {  102,  237,      22,  8, }, // America/Atikokan
    {  102, 3581,      30,  5, }, // America/Bahia
    {  102, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  102, 7609,   34376,  5, }, // America/Belem
    {  102, 4263,   18898,  6, }, // America/Bogota
    {  102, 3770,      47,  6, }, // America/Cancun
    {  102, 5398,   56297, 17, }, // America/Cayman
    {  102, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  102, 5324,   31782,  9, }, // America/Costa_Rica
    {  102, 5640,      66,  7, }, // America/Curacao
    {  102, 5656,   20071,  8, }, // America/Dominica
    {  102, 5343,    5090,  8, }, // America/El_Salvador
    {  102,  481,      73, 13, }, // America/Indiana/Knox
    {  102, 7924,      86, 16, }, // America/Indiana/Marengo
    {  102, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  102, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  102, 8001,     139, 14, }, // America/Indiana/Vevay
    {  102, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  102, 8049,     171, 16, }, // America/Indiana/Winamac
    {  102, 2665,   31809,  7, }, // America/Jamaica
    {  102, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  102, 5849,     207, 22, }, // America/Lower_Princes
    {  102, 5722,    5400,  8, }, // America/Martinique
    {  102, 2779,   25036,  8, }, // America/Mazatlan
    {  102, 8183,    3899,  6, }, // America/Merida
    {  102, 2811,   56111, 16, }, // America/Mexico_City
    {  102, 8264,   56314, 22, }, // America/North_Dakota/Beulah
    {  102, 8292,   56336, 22, }, // America/North_Dakota/Center
    {  102, 8320,   56358, 25, }, // America/North_Dakota/New_Salem
    {  102,  393,     298,  4, }, // America/Nuuk
    {  102, 5760,   31888,  9, }, // America/Puerto_Rico
    {  102, 8440,   25137,  8, }, // America/Santarem
    {  102, 1862,   25145,  9, }, // America/Sao_Paulo
    {  102, 6862,     302, 16, }, // America/Scoresbysund
    {  102, 8471,   56383, 16, }, // America/St_Barthelemy
    {  102, 2045,     332, 10, }, // America/St_Johns
    {  102, 5780,   56399, 13, }, // America/St_Kitts
    {  102, 5797,   56412, 11, }, // America/St_Lucia
    {  102,  855,   56423, 21, }, // America/St_Thomas
    {  102, 5830,   56444, 13, }, // America/St_Vincent
    {  102, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  102, 3870,   56457,  5, }, // Asia/Amman
    {  102, 5440,    5906,  5, }, // Asia/Aqtau
    {  102, 5270,    5911,  6, }, // Asia/Aqtobe
    {  102,  964,   56462,  8, }, // Asia/Ashgabat
    {  102, 3509,   56470,  6, }, // Asia/Baghdad
    {  102, 5451,   56476,  7, }, // Asia/Bahrain
    {  102, 3988,   56483,  6, }, // Asia/Beirut
    {  102, 3680,   31917,  6, }, // Asia/Bishkek
    {  102, 5906,   31923,  6, }, // Asia/Brunei
    {  102, 4503,   56142,  4, }, // Asia/Chita
    {  102, 5963,   31930,  9, }, // Asia/Choibalsan
    {  102, 4413,    5949,  7, }, // Asia/Colombo
    {  102, 4442,   56489,  7, }, // Asia/Damascus
    {  102, 1060,   56496,  5, }, // Asia/Dhaka
    {  102, 3498,   56501,  5, }, // Asia/Dubai
    {  102, 6092,   31939,  7, }, // Asia/Dushanbe
    {  102, 1226,   56506, 18, }, // Asia/Ho_Chi_Minh
    {  102, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  102, 1257,   56524,  9, }, // Asia/Jerusalem
    {  102, 3462,   56533,  5, }, // Asia/Kabul
    {  102, 4218,   31981,  8, }, // Asia/Kamchatka
    {  102, 4141,   56538,  6, }, // Asia/Karachi
    {  102, 1152,   56544,  9, }, // Asia/Kathmandu
    {  102, 8624,   56553,  8, }, // Asia/Khandyga
    {  102,  992,   56561,  7, }, // Asia/Kolkata
    {  102, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  102, 5893,   32002,  6, }, // Asia/Kuching
    {  102, 5464,    6058,  6, }, // Asia/Kuwait
    {  102, 1178,     429,  5, }, // Asia/Macau
    {  102, 6883,   25325,  6, }, // Asia/Muscat
    {  102, 2446,   56568,  7, }, // Asia/Nicosia
    {  102, 8638,   56575, 11, }, // Asia/Novokuznetsk
    {  102, 7108,   56586,  6, }, // Asia/Oral
    {  102, 6924,   56592,  9, }, // Asia/Phnom_Penh
    {  102, 4116,   56601, 11, }, // Asia/Pyongyang
    {  102, 5476,    4233,  5, }, // Asia/Qatar
    {  102, 8671,    6103,  8, }, // Asia/Qostanay
    {  102, 4171,   25347,  9, }, // Asia/Qyzylorda
    {  102, 3486,   56612,  5, }, // Asia/Riyadh
    {  102, 4310,   56617,  8, }, // Asia/Sakhalin
    {  102, 3116,   56625,  4, }, // Asia/Seoul
    {  102, 1020,   56629,  7, }, // Asia/Shanghai
    {  102, 3137,   19291,  8, }, // Asia/Singapore
    {  102, 4199,   56636, 13, }, // Asia/Srednekolymsk
    {  102, 3100,   56649,  9, }, // Asia/Taipei
    {  102, 4683,   32060,  7, }, // Asia/Tashkent
    {  102, 2638,   56658,  7, }, // Asia/Tehran
    {  102, 1284,   56665,  6, }, // Asia/Thimphu
    {  102, 2687,    4257,  5, }, // Asia/Tokyo
    {  102, 1346,   56671,  9, }, // Asia/Ulaanbaatar
    {  102, 1126,   32067,  6, }, // Asia/Urumqi
    {  102, 4730,    6206,  7, }, // Asia/Yakutsk
    {  102, 1202,   19328,  6, }, // Asia/Yangon
    {  102, 3785,   56680, 13, }, // Asia/Yekaterinburg
    {  102, 3649,    6231,  7, }, // Asia/Yerevan
    {  102, 3565,   56693, 15, }, // Atlantic/Azores
    {  102, 5588,   56708,  7, }, // Atlantic/Bermuda
    {  102, 6591,   56715, 16, }, // Atlantic/Canary
    {  102, 3629,   56731,  8, }, // Atlantic/Cape_Verde
    {  102, 1379,   56739, 15, }, // Atlantic/Faroe
    {  102, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  102, 7235,   56754, 13, }, // Atlantic/South_Georgia
    {  102, 6784,   56767, 12, }, // Atlantic/St_Helena
    {  102, 3522,   56779,  9, }, // Europe/Astrakhan
    {  102, 6577,   56788,  6, }, // Europe/Athens
    {  102, 6285,   56794,  8, }, // Europe/Belgrade
    {  102, 4659,   19450,  6, }, // Europe/Berlin
    {  102, 6178,   25472,  6, }, // Europe/Brussels
    {  102, 3830,   56802,  8, }, // Europe/Bucharest
    {  102, 3708,   56810,  8, }, // Europe/Budapest
    {  102, 2475,   56818,  7, }, // Europe/Chisinau
    {  102, 6238,   56825,  5, }, // Europe/Copenhagen
    {  102, 6561,   56830,  8, }, // Europe/Helsinki
    {  102, 8783,   56838, 10, }, // Europe/Isle_of_Man
    {  102, 2419,   56848,  5, }, // Europe/Kyiv
    {  102, 3078,   32199,  7, }, // Europe/Lisbon
    {  102, 6455,   56853,  6, }, // Europe/Ljubljana
    {  102, 2393,   56859,  6, }, // Europe/London
    {  102, 6314,   56865,  9, }, // Europe/Luxembourg
    {  102, 6359,    6525,  6, }, // Europe/Monaco
    {  102, 3443,   21290,  6, }, // Europe/Moscow
    {  102, 4186,   56874,  5, }, // Europe/Paris
    {  102, 6224,   34765,  5, }, // Europe/Prague
    {  102, 6273,   56879,  3, }, // Europe/Rome
    {  102, 6548,   56882,  5, }, // Europe/Sofia
    {  102, 6135,   19608,  6, }, // Europe/Tirane
    {  102, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  102, 6516,   56887,  7, }, // Europe/Vatican
    {  102, 6164,   56894,  5, }, // Europe/Vienna
    {  102, 3055,   32249,  7, }, // Europe/Warsaw
    {  102, 6210,   56899,  6, }, // Europe/Zagreb
    {  102, 6489,   56905,  6, }, // Europe/Zurich
    {  102, 5979,   56911, 14, }, // Indian/Christmas
    {  102, 5996,   56925, 16, }, // Indian/Cocos
    {  102, 4931,   56941,  6, }, // Indian/Comoro
    {  102, 6624,   56947, 20, }, // Indian/Kerguelen
    {  102, 7223,   25565,  4, }, // Indian/Mahe
    {  102, 7013,   56967,  8, }, // Indian/Maldives
    {  102, 3971,   56975,  9, }, // Indian/Mauritius
    {  102, 7174,     467,  7, }, // Indian/Reunion
    {  102, 2863,   56984, 18, }, // Pacific/Chatham
    {  102, 3022,   57002, 16, }, // Pacific/Chuuk
    {  102, 2201,   57018, 18, }, // Pacific/Easter
    {  102, 7323,   25598,  5, }, // Pacific/Efate
    {  102, 3804,   32307,  5, }, // Pacific/Fiji
    {  102, 6641,   57036,  8, }, // Pacific/Galapagos
    {  102, 6659,   57044, 18, }, // Pacific/Gambier
    {  102, 2929,    4512,  8, }, // Pacific/Honolulu
    {  102, 2897,    4520,  9, }, // Pacific/Kanton
    {  102, 3953,   57062,  7, }, // Pacific/Marquesas
    {  102, 7061,    4529,  6, }, // Pacific/Noumea
    {  102, 7157,   57069, 19, }, // Pacific/Pitcairn
    {  102, 2961,     479,  7, }, // Pacific/Pohnpei
    {  103, 4961,    3793, 11, }, // Africa/Addis_Ababa Danish/Latin/Denmark
    {  103, 7383,   57088,  6, }, // Africa/Algiers
    {  103,   14,       0,  6, }, // Africa/Asmara
    {  103, 4337,       6,  8, }, // Africa/Sao_Tome
    {  103, 4154,      14,  8, }, // America/Asuncion
    {  103,  237,      22,  8, }, // America/Atikokan
    {  103, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  103, 3770,    3861,  6, }, // America/Cancun
    {  103, 5398,   57094, 11, }, // America/Cayman
    {  103, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  103, 5640,      66,  7, }, // America/Curacao
    {  103,  481,      73, 13, }, // America/Indiana/Knox
    {  103, 7924,      86, 16, }, // America/Indiana/Marengo
    {  103, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  103, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  103, 8001,     139, 14, }, // America/Indiana/Vevay
    {  103, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  103, 8049,     171, 16, }, // America/Indiana/Winamac
    {  103, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  103, 5849,     207, 22, }, // America/Lower_Princes
    {  103, 8183,    3899,  6, }, // America/Merida
    {  103, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  103, 8292,     255, 20, }, // America/North_Dakota/Center
    {  103, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  103,  393,     298,  4, }, // America/Nuuk
    {  103, 1862,   25145,  9, }, // America/Sao_Paulo
    {  103, 6862,     302, 16, }, // America/Scoresbysund
    {  103, 8471,   25154, 16, }, // America/St_Barthelemy
    {  103, 2045,     332, 10, }, // America/St_Johns
    {  103, 5780,     342,  9, }, // America/St_Kitts
    {  103, 5797,     351,  9, }, // America/St_Lucia
    {  103,  855,     360, 10, }, // America/St_Thomas
    {  103, 5830,     370, 11, }, // America/St_Vincent
    {  103, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  103, 5440,    5906,  5, }, // Asia/Aqtau
    {  103, 5270,    5911,  6, }, // Asia/Aqtobe
    {  103,  964,   57105,  9, }, // Asia/Ashgabat
    {  103, 3509,    4063,  6, }, // Asia/Baghdad
    {  103, 3680,    4088,  7, }, // Asia/Bishkek
    {  103, 5963,   57114, 11, }, // Asia/Choibalsan
    {  103, 4442,    4102,  8, }, // Asia/Damascus
    {  103, 6092,   57125,  8, }, // Asia/Dushanbe
    {  103, 1226,     397, 16, }, // Asia/Ho_Chi_Minh
    {  103, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  103, 4218,   57133,  9, }, // Asia/Kamchatka
    {  103, 1152,    6018,  8, }, // Asia/Kathmandu
    {  103,  992,     422,  7, }, // Asia/Kolkata
    {  103, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  103, 1178,     429,  5, }, // Asia/Macau
    {  103, 8638,   57142, 12, }, // Asia/Novokuznetsk
    {  103, 8671,     434,  8, }, // Asia/Qostanay
    {  103, 4171,   25347,  9, }, // Asia/Qyzylorda
    {  103, 4683,   57154,  8, }, // Asia/Tashkent
    {  103, 2638,    6175,  7, }, // Asia/Tehran
    {  103, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  103, 1126,   25383,  6, }, // Asia/Urumqi
    {  103, 4730,    6206,  7, }, // Asia/Yakutsk
    {  103, 1202,   56146,  7, }, // Asia/Yangon
    {  103, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  103, 3649,    6231,  7, }, // Asia/Yerevan
    {  103, 3565,   57162,  8, }, // Atlantic/Azores
    {  103, 6591,   57170, 16, }, // Atlantic/Canary
    {  103, 3629,   55900,  9, }, // Atlantic/Cape_Verde
    {  103, 1379,   57186,  8, }, // Atlantic/Faroe
    {  103, 6784,     453, 10, }, // Atlantic/St_Helena
    {  103, 6577,    4339,  5, }, // Europe/Athens
    {  103, 6285,    6398,  7, }, // Europe/Belgrade
    {  103, 6178,   56194,  9, }, // Europe/Brussels
    {  103, 3830,   25479,  8, }, // Europe/Bucharest
    {  103, 6238,   57194,  9, }, // Europe/Copenhagen
    {  103, 2419,    6486,  4, }, // Europe/Kyiv
    {  103, 3078,    4400,  8, }, // Europe/Lisbon
    {  103, 3443,   21290,  6, }, // Europe/Moscow
    {  103, 6224,    6545,  4, }, // Europe/Prague
    {  103, 6273,    6554,  3, }, // Europe/Rome
    {  103, 6135,   19608,  6, }, // Europe/Tirane
    {  103, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  103, 6516,   57203,  9, }, // Europe/Vatican
    {  103, 6164,   56035,  4, }, // Europe/Vienna
    {  103, 3055,   57212,  8, }, // Europe/Warsaw
    {  103, 6489,    4478,  6, }, // Europe/Zurich
    {  103, 5979,   57220,  7, }, // Indian/Christmas
    {  103, 4931,   57227,  9, }, // Indian/Comoro
    {  103, 7013,   57236, 10, }, // Indian/Maldives
    {  103, 7174,     467,  7, }, // Indian/Reunion
    {  103, 3022,     474,  5, }, // Pacific/Chuuk
    {  103, 2201,   57246,  8, }, // Pacific/Easter
    {  103, 2929,    4512,  8, }, // Pacific/Honolulu
    {  103, 2897,    4520,  9, }, // Pacific/Kanton
    {  103, 2961,     479,  7, }, // Pacific/Pohnpei
    {  105,   14,       0,  6, }, // Africa/Asmara Divehi/Thaana/Maldives
    {  105, 4337,       6,  8, }, // Africa/Sao_Tome
    {  105, 4154,      14,  8, }, // America/Asuncion
    {  105,  237,      22,  8, }, // America/Atikokan
    {  105, 7586,      30, 17, }, // America/Bahia_Banderas
    {  105, 3770,      47,  6, }, // America/Cancun
    {  105, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  105, 5640,      66,  7, }, // America/Curacao
    {  105,  481,      73, 13, }, // America/Indiana/Knox
    {  105, 7924,      86, 16, }, // America/Indiana/Marengo
    {  105, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  105, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  105, 8001,     139, 14, }, // America/Indiana/Vevay
    {  105, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  105, 8049,     171, 16, }, // America/Indiana/Winamac
    {  105, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  105, 5849,     207, 22, }, // America/Lower_Princes
    {  105, 8183,     229,  6, }, // America/Merida
    {  105, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  105, 8292,     255, 20, }, // America/North_Dakota/Center
    {  105, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  105,  393,     298,  4, }, // America/Nuuk
    {  105, 6862,     302, 16, }, // America/Scoresbysund
    {  105, 8471,     318, 14, }, // America/St_Barthelemy
    {  105, 2045,     332, 10, }, // America/St_Johns
    {  105, 5780,     342,  9, }, // America/St_Kitts
    {  105, 5797,     351,  9, }, // America/St_Lucia
    {  105,  855,     360, 10, }, // America/St_Thomas
    {  105, 5830,     370, 11, }, // America/St_Vincent
    {  105, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  105, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  105, 1152,     413,  9, }, // Asia/Kathmandu
    {  105,  992,     422,  7, }, // Asia/Kolkata
    {  105, 1178,     429,  5, }, // Asia/Macau
    {  105, 1202,     442,  6, }, // Asia/Yangon
    {  105, 1379,     448,  5, }, // Atlantic/Faroe
    {  105, 6784,     453, 10, }, // Atlantic/St_Helena
    {  105, 2419,     463,  4, }, // Europe/Kyiv
    {  105, 7174,     467,  7, }, // Indian/Reunion
    {  105, 3022,     474,  5, }, // Pacific/Chuuk
    {  105, 2961,     479,  7, }, // Pacific/Pohnpei
    {  106,   14,       0,  6, }, // Africa/Asmara Dogri/Devanagari/India
    {  106, 4337,       6,  8, }, // Africa/Sao_Tome
    {  106, 4154,      14,  8, }, // America/Asuncion
    {  106,  237,      22,  8, }, // America/Atikokan
    {  106, 7586,      30, 17, }, // America/Bahia_Banderas
    {  106, 3770,      47,  6, }, // America/Cancun
    {  106, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  106, 5640,      66,  7, }, // America/Curacao
    {  106,  481,      73, 13, }, // America/Indiana/Knox
    {  106, 7924,      86, 16, }, // America/Indiana/Marengo
    {  106, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  106, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  106, 8001,     139, 14, }, // America/Indiana/Vevay
    {  106, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  106, 8049,     171, 16, }, // America/Indiana/Winamac
    {  106, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  106, 5849,     207, 22, }, // America/Lower_Princes
    {  106, 8183,     229,  6, }, // America/Merida
    {  106, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  106, 8292,     255, 20, }, // America/North_Dakota/Center
    {  106, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  106,  393,     298,  4, }, // America/Nuuk
    {  106, 6862,     302, 16, }, // America/Scoresbysund
    {  106, 8471,     318, 14, }, // America/St_Barthelemy
    {  106, 2045,     332, 10, }, // America/St_Johns
    {  106, 5780,     342,  9, }, // America/St_Kitts
    {  106, 5797,     351,  9, }, // America/St_Lucia
    {  106,  855,     360, 10, }, // America/St_Thomas
    {  106, 5830,     370, 11, }, // America/St_Vincent
    {  106, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  106, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  106, 1152,     413,  9, }, // Asia/Kathmandu
    {  106,  992,     422,  7, }, // Asia/Kolkata
    {  106, 1178,     429,  5, }, // Asia/Macau
    {  106, 1202,     442,  6, }, // Asia/Yangon
    {  106, 1379,     448,  5, }, // Atlantic/Faroe
    {  106, 6784,     453, 10, }, // Atlantic/St_Helena
    {  106, 2419,     463,  4, }, // Europe/Kyiv
    {  106, 7174,     467,  7, }, // Indian/Reunion
    {  106, 3022,     474,  5, }, // Pacific/Chuuk
    {  106, 2961,     479,  7, }, // Pacific/Pohnpei
    {  107,   14,       0,  6, }, // Africa/Asmara Duala/Latin/Cameroon
    {  107, 4337,       6,  8, }, // Africa/Sao_Tome
    {  107, 4154,      14,  8, }, // America/Asuncion
    {  107,  237,      22,  8, }, // America/Atikokan
    {  107, 7586,      30, 17, }, // America/Bahia_Banderas
    {  107, 3770,      47,  6, }, // America/Cancun
    {  107, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  107, 5640,      66,  7, }, // America/Curacao
    {  107,  481,      73, 13, }, // America/Indiana/Knox
    {  107, 7924,      86, 16, }, // America/Indiana/Marengo
    {  107, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  107, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  107, 8001,     139, 14, }, // America/Indiana/Vevay
    {  107, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  107, 8049,     171, 16, }, // America/Indiana/Winamac
    {  107, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  107, 5849,     207, 22, }, // America/Lower_Princes
    {  107, 8183,     229,  6, }, // America/Merida
    {  107, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  107, 8292,     255, 20, }, // America/North_Dakota/Center
    {  107, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  107,  393,     298,  4, }, // America/Nuuk
    {  107, 6862,     302, 16, }, // America/Scoresbysund
    {  107, 8471,     318, 14, }, // America/St_Barthelemy
    {  107, 2045,     332, 10, }, // America/St_Johns
    {  107, 5780,     342,  9, }, // America/St_Kitts
    {  107, 5797,     351,  9, }, // America/St_Lucia
    {  107,  855,     360, 10, }, // America/St_Thomas
    {  107, 5830,     370, 11, }, // America/St_Vincent
    {  107, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  107, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  107, 1152,     413,  9, }, // Asia/Kathmandu
    {  107,  992,     422,  7, }, // Asia/Kolkata
    {  107, 1178,     429,  5, }, // Asia/Macau
    {  107, 1202,     442,  6, }, // Asia/Yangon
    {  107, 1379,     448,  5, }, // Atlantic/Faroe
    {  107, 6784,     453, 10, }, // Atlantic/St_Helena
    {  107, 2419,     463,  4, }, // Europe/Kyiv
    {  107, 7174,     467,  7, }, // Indian/Reunion
    {  107, 3022,     474,  5, }, // Pacific/Chuuk
    {  107, 2961,     479,  7, }, // Pacific/Pohnpei
    {  108, 4961,    3793, 11, }, // Africa/Addis_Ababa Dutch/Latin/Netherlands
    {  108,   14,       0,  6, }, // Africa/Asmara
    {  108, 2242,   57254,  5, }, // Africa/Cairo
    {  108, 4426,   57259,  8, }, // Africa/Khartoum
    {  108, 6832,   18802,  4, }, // Africa/Lome
    {  108, 4337,   57267,  8, }, // Africa/Sao_Tome
    {  108,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  108, 7452,   42641, 12, }, // America/Argentina/Rio_Gallegos
    {  108, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  108, 4154,      14,  8, }, // America/Asuncion
    {  108,  237,      22,  8, }, // America/Atikokan
    {  108, 7586,      30, 17, }, // America/Bahia_Banderas
    {  108, 7609,   34376,  5, }, // America/Belem
    {  108, 3770,    3861,  6, }, // America/Cancun
    {  108, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  108, 3693,   25007,  6, }, // America/Cuiaba
    {  108, 5640,      66,  7, }, // America/Curacao
    {  108,  481,      73, 13, }, // America/Indiana/Knox
    {  108, 7924,      86, 16, }, // America/Indiana/Marengo
    {  108, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  108, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  108, 8001,     139, 14, }, // America/Indiana/Vevay
    {  108, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  108, 8049,     171, 16, }, // America/Indiana/Winamac
    {  108, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  108, 5849,   57275, 24, }, // America/Lower_Princes
    {  108, 8116,   25021,  6, }, // America/Maceio
    {  108, 2779,   25036,  8, }, // America/Mazatlan
    {  108, 8183,     229,  6, }, // America/Merida
    {  108, 2811,   57299, 11, }, // America/Mexico_City
    {  108, 8264,    3916, 20, }, // America/North_Dakota/Beulah
    {  108, 8292,    3936, 20, }, // America/North_Dakota/Center
    {  108, 8320,    3956, 23, }, // America/North_Dakota/New_Salem
    {  108,  393,     298,  4, }, // America/Nuuk
    {  108, 1862,   25145,  9, }, // America/Sao_Paulo
    {  108, 6862,     302, 16, }, // America/Scoresbysund
    {  108, 8471,   25154, 16, }, // America/St_Barthelemy
    {  108, 2045,   25170, 12, }, // America/St_Johns
    {  108, 5780,   25182, 11, }, // America/St_Kitts
    {  108, 5797,   34444, 11, }, // America/St_Lucia
    {  108,  855,   42797, 12, }, // America/St_Thomas
    {  108, 5830,   25216, 13, }, // America/St_Vincent
    {  108, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  108, 5297,   57310,  8, }, // Asia/Almaty
    {  108, 5270,   55725,  6, }, // Asia/Aqtobe
    {  108,  964,    4054,  9, }, // Asia/Ashgabat
    {  108, 8587,   57318,  6, }, // Asia/Atyrau
    {  108, 3509,    4063,  6, }, // Asia/Baghdad
    {  108, 5451,    4069,  7, }, // Asia/Bahrain
    {  108, 3555,    4076,  5, }, // Asia/Baku
    {  108, 3988,    4081,  7, }, // Asia/Beirut
    {  108, 3680,    4088,  7, }, // Asia/Bishkek
    {  108, 5963,   57114, 11, }, // Asia/Choibalsan
    {  108, 6092,   57324,  9, }, // Asia/Dushanbe
    {  108, 1226,   57333, 15, }, // Asia/Ho_Chi_Minh
    {  108, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  108, 4086,    4140,  8, }, // Asia/Irkutsk
    {  108, 1257,   31972,  9, }, // Asia/Jerusalem
    {  108, 4218,    4162, 10, }, // Asia/Kamchatka
    {  108, 1152,     413,  9, }, // Asia/Kathmandu
    {  108,  992,   34533,  8, }, // Asia/Kolkata
    {  108, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  108, 5464,    4215,  7, }, // Asia/Kuwait
    {  108, 1178,    4222,  5, }, // Asia/Macau
    {  108, 7145,   57348,  7, }, // Asia/Manila
    {  108, 3486,   19265,  5, }, // Asia/Riyadh
    {  108, 4310,   56617,  8, }, // Asia/Sakhalin
    {  108, 1020,   57355,  8, }, // Asia/Shanghai
    {  108, 4683,   57154,  8, }, // Asia/Tashkent
    {  108, 2638,    6175,  7, }, // Asia/Tehran
    {  108, 2687,    4257,  5, }, // Asia/Tokyo
    {  108, 4730,    4273,  8, }, // Asia/Yakutsk
    {  108, 1202,   56146,  7, }, // Asia/Yangon
    {  108, 3785,   57363, 15, }, // Asia/Yekaterinburg
    {  108, 3649,    6231,  7, }, // Asia/Yerevan
    {  108, 3565,   57378,  6, }, // Atlantic/Azores
    {  108, 6591,   57384, 19, }, // Atlantic/Canary
    {  108, 3629,   57403, 10, }, // Atlantic/Cape_Verde
    {  108, 1379,   57413,  7, }, // Atlantic/Faroe
    {  108, 7235,   57420, 12, }, // Atlantic/South_Georgia
    {  108, 6784,   57432, 11, }, // Atlantic/St_Helena
    {  108, 6577,    4339,  6, }, // Europe/Athens
    {  108, 6285,   57443,  8, }, // Europe/Belgrade
    {  108, 4659,   57451,  7, }, // Europe/Berlin
    {  108, 6178,    4351,  7, }, // Europe/Brussels
    {  108, 3830,    4358,  9, }, // Europe/Bucharest
    {  108, 3708,    4367,  9, }, // Europe/Budapest
    {  108, 6238,    4376, 10, }, // Europe/Copenhagen
    {  108, 1097,   57458,  9, }, // Europe/Istanbul
    {  108, 2419,    6486,  4, }, // Europe/Kyiv
    {  108, 3078,    4400,  8, }, // Europe/Lisbon
    {  108, 2393,    4408,  6, }, // Europe/London
    {  108, 6314,    4414,  9, }, // Europe/Luxembourg
    {  108, 3443,    4423,  6, }, // Europe/Moscow
    {  108, 4186,   57467,  6, }, // Europe/Paris
    {  108, 6224,    4434,  5, }, // Europe/Prague
    {  108, 6135,   19608,  6, }, // Europe/Tirane
    {  108, 6516,   57473, 12, }, // Europe/Vatican
    {  108, 6164,   57485,  5, }, // Europe/Vienna
    {  108, 4629,    4462,  9, }, // Europe/Volgograd
    {  108, 3055,   56048,  8, }, // Europe/Warsaw
    {  108, 6489,    4478,  6, }, // Europe/Zurich
    {  108, 6910,   57490, 14, }, // Indian/Chagos
    {  108, 5979,   57504, 15, }, // Indian/Christmas
    {  108, 5996,   57519, 13, }, // Indian/Cocos
    {  108, 7223,   25565,  4, }, // Indian/Mahe
    {  108, 7013,   57532,  8, }, // Indian/Maldives
    {  108, 7174,     467,  7, }, // Indian/Reunion
    {  108, 3022,     474,  5, }, // Pacific/Chuuk
    {  108, 2201,   57540, 10, }, // Pacific/Easter
    {  108, 6659,   57550, 12, }, // Pacific/Gambier
    {  108, 2929,    4512,  8, }, // Pacific/Honolulu
    {  108, 2897,    4520,  9, }, // Pacific/Kanton
    {  108, 3953,   57562, 17, }, // Pacific/Marquesas
    {  108, 7061,    4529,  6, }, // Pacific/Noumea
    {  108, 2961,     479,  7, }, // Pacific/Pohnpei
    {  115, 6723,   57579,  6, }, // Africa/Accra Dzongkha/Tibetan/Bhutan
    {  115, 4961,   57585, 14, }, // Africa/Addis_Ababa
    {  115, 7383,   57599,  8, }, // Africa/Algiers
    {  115,   14,   57607,  7, }, // Africa/Asmara
    {  115,   44,   57614,  7, }, // Africa/Bamako
    {  115, 5159,   57621,  6, }, // Africa/Bangui
    {  115, 6709,   57627,  8, }, // Africa/Banjul
    {  115, 7398,   57635,  8, }, // Africa/Bissau
    {  115, 5189,   57643,  9, }, // Africa/Brazzaville
    {  115, 4824,   57652, 13, }, // Africa/Bujumbura
    {  115, 2242,   57665,  7, }, // Africa/Cairo
    {  115, 6736,   57672,  8, }, // Africa/Conakry
    {  115, 6803,   57680,  5, }, // Africa/Dakar
    {  115, 4945,   57685,  9, }, // Africa/Djibouti
    {  115, 4808,   57694, 12, }, // Africa/Gaborone
    {  115, 4917,   57706,  7, }, // Africa/Harare
    {  115, 5053,   57713,  7, }, // Africa/Kampala
    {  115, 4426,   57720,  7, }, // Africa/Khartoum
    {  115, 4875,   57727,  8, }, // Africa/Kigali
    {  115, 5208,   57735,  7, }, // Africa/Kinshasa
    {  115, 5238,   57742, 10, }, // Africa/Libreville
    {  115, 6832,   57752,  5, }, // Africa/Lome
    {  115, 5113,   57757,  8, }, // Africa/Luanda
    {  115, 4903,   57765,  7, }, // Africa/Lusaka
    {  115, 5224,   57772,  7, }, // Africa/Malabo
    {  115, 4889,   57779,  7, }, // Africa/Maputo
    {  115, 5099,   57786,  6, }, // Africa/Maseru
    {  115, 5084,   57792, 12, }, // Africa/Mbabane
    {  115, 5015,   57804, 11, }, // Africa/Mogadishu
    {  115, 6997,   57815, 11, }, // Africa/Monrovia
    {  115, 3755,   57826, 10, }, // Africa/Nairobi
    {  115, 5256,   57836,  7, }, // Africa/Niamey
    {  115, 6766,   57843, 10, }, // Africa/Nouakchott
    {  115, 6690,   57853, 10, }, // Africa/Ouagadougou
    {  115, 4337,   57863,  9, }, // Africa/Sao_Tome
    {  115, 2732,   57872,  9, }, // Africa/Tripoli
    {  115, 6503,   57881,  7, }, // Africa/Tunis
    {  115, 4054,   57888,  8, }, // Africa/Windhoek
    {  115, 5541,   57896, 11, }, // America/Antigua
    {  115,  166,   57907, 18, }, // America/Argentina/Buenos_Aires
    {  115, 4154,   57925, 12, }, // America/Asuncion
    {  115,  237,   57937, 11, }, // America/Atikokan
    {  115, 7586,      30, 17, }, // America/Bahia_Banderas
    {  115, 5571,   57948, 10, }, // America/Barbados
    {  115, 5309,   57958,  7, }, // America/Belize
    {  115, 4263,   57965,  7, }, // America/Bogota
    {  115, 7676,   57972, 15, }, // America/Cambridge_Bay
    {  115, 3770,      47,  6, }, // America/Cancun
    {  115, 4596,   57987,  7, }, // America/Caracas
    {  115, 3251,   57994,  8, }, // America/Chicago
    {  115, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  115, 5324,   58002, 10, }, // America/Costa_Rica
    {  115, 5640,      66,  7, }, // America/Curacao
    {  115, 7796,   58012,  8, }, // America/Dawson
    {  115, 7811,   58020, 15, }, // America/Dawson_Creek
    {  115, 5656,   58035, 10, }, // America/Dominica
    {  115,  893,   58045, 12, }, // America/Edmonton
    {  115, 5343,   58057, 13, }, // America/El_Salvador
    {  115, 6844,   58070, 10, }, // America/Goose_Bay
    {  115, 3662,   58080,  9, }, // America/Guatemala
    {  115, 6895,   58089,  6, }, // America/Guyana
    {  115, 1923,   58095, 10, }, // America/Halifax
    {  115,  481,      73, 13, }, // America/Indiana/Knox
    {  115, 7924,      86, 16, }, // America/Indiana/Marengo
    {  115, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  115, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  115, 8001,     139, 14, }, // America/Indiana/Vevay
    {  115, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  115, 8049,     171, 16, }, // America/Indiana/Winamac
    {  115, 8073,   58105,  9, }, // America/Inuvik
    {  115, 2665,   58114,  8, }, // America/Jamaica
    {  115, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  115, 4278,   58122,  6, }, // America/La_Paz
    {  115, 7132,   58128,  4, }, // America/Lima
    {  115, 5849,     207, 22, }, // America/Lower_Princes
    {  115, 8183,     229,  6, }, // America/Merida
    {  115, 2811,   58132, 16, }, // America/Mexico_City
    {  115, 8217,   58148,  9, }, // America/Moncton
    {  115, 4000,   58157, 15, }, // America/Montevideo
    {  115, 5383,   58172,  5, }, // America/Nassau
    {  115, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  115, 8292,     255, 20, }, // America/North_Dakota/Center
    {  115, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  115,  393,     298,  4, }, // America/Nuuk
    {  115, 5413,   58177,  5, }, // America/Panama
    {  115, 6106,   58182, 12, }, // America/Paramaribo
    {  115, 8387,   58194, 16, }, // America/Rankin_Inlet
    {  115, 1979,   58210,  7, }, // America/Regina
    {  115, 8423,   58217, 11, }, // America/Resolute
    {  115, 2165,   58228, 12, }, // America/Santiago
    {  115, 6044,   58240, 18, }, // America/Santo_Domingo
    {  115, 6862,     302, 16, }, // America/Scoresbysund
    {  115, 8471,     318, 14, }, // America/St_Barthelemy
    {  115, 2045,   58258, 17, }, // America/St_Johns
    {  115, 5780,     342,  9, }, // America/St_Kitts
    {  115, 5797,     351,  9, }, // America/St_Lucia
    {  115,  855,     360, 10, }, // America/St_Thomas
    {  115, 5830,     370, 11, }, // America/St_Vincent
    {  115, 8493,   58275, 14, }, // America/Swift_Current
    {  115, 5363,   58289, 14, }, // America/Tegucigalpa
    {  115,  313,   58303,  8, }, // America/Tijuana
    {  115,  608,   58311,  9, }, // America/Toronto
    {  115, 8515,   58320,  7, }, // America/Yakutat
    {  115, 6027,   58327,  7, }, // Antarctica/Davis
    {  115, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  115, 7029,   58334,  8, }, // Antarctica/Mawson
    {  115, 8552,   58342,  6, }, // Antarctica/Palmer
    {  115, 7189,   58348,  8, }, // Antarctica/Rothera
    {  115, 7337,   58356,  8, }, // Antarctica/Vostok
    {  115, 5451,   58364,  7, }, // Asia/Bahrain
    {  115, 5906,   58371,  8, }, // Asia/Brunei
    {  115, 4413,   58379, 10, }, // Asia/Colombo
    {  115, 1060,   57680,  4, }, // Asia/Dhaka
    {  115, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  115, 6969,   58389,  7, }, // Asia/Jakarta
    {  115, 6955,   58396,  8, }, // Asia/Jayapura
    {  115, 3462,   58404,  6, }, // Asia/Kabul
    {  115, 4141,   58410,  6, }, // Asia/Karachi
    {  115, 1152,   58416, 10, }, // Asia/Kathmandu
    {  115,  992,   58426,  8, }, // Asia/Kolkata
    {  115, 4761,   58434, 12, }, // Asia/Kuala_Lumpur
    {  115, 1178,     429,  5, }, // Asia/Macau
    {  115, 8656,   58446, 11, }, // Asia/Pontianak
    {  115, 2638,   58457,  6, }, // Asia/Tehran
    {  115, 1284,   58463,  7, }, // Asia/Thimphu
    {  115, 6940,   58470, 13, }, // Asia/Vientiane
    {  115, 1202,   58483,  7, }, // Asia/Yangon
    {  115, 5588,   58490,  7, }, // Atlantic/Bermuda
    {  115, 3629,   58497,  7, }, // Atlantic/Cape_Verde
    {  115, 1379,   58504,  8, }, // Atlantic/Faroe
    {  115, 2614,   58512, 11, }, // Atlantic/Reykjavik
    {  115, 7235,   58523, 11, }, // Atlantic/South_Georgia
    {  115, 6784,     453, 10, }, // Atlantic/St_Helena
    {  115, 6390,   58534,  9, }, // Europe/Amsterdam
    {  115, 6149,   58543,  8, }, // Europe/Andorra
    {  115, 6577,   58551,  8, }, // Europe/Athens
    {  115, 4659,   58559,  7, }, // Europe/Berlin
    {  115, 6178,   58566,  7, }, // Europe/Brussels
    {  115, 3830,   58573, 10, }, // Europe/Bucharest
    {  115, 3708,   58583, 10, }, // Europe/Budapest
    {  115, 2475,   58593,  9, }, // Europe/Chisinau
    {  115, 6238,   58602, 13, }, // Europe/Copenhagen
    {  115, 2260,   58615,  6, }, // Europe/Dublin
    {  115, 6561,   58621, 10, }, // Europe/Helsinki
    {  115, 2419,     463,  4, }, // Europe/Kyiv
    {  115, 2393,   58631,  7, }, // Europe/London
    {  115, 6314,   58638,  9, }, // Europe/Luxembourg
    {  115, 4794,   58647,  8, }, // Europe/Madrid
    {  115, 3595,   58655,  5, }, // Europe/Minsk
    {  115, 6359,   58660,  7, }, // Europe/Monaco
    {  115, 3443,   58667,  7, }, // Europe/Moscow
    {  115, 6407,   58674,  6, }, // Europe/Oslo
    {  115, 4186,   58680,  6, }, // Europe/Paris
    {  115, 6224,   58686,  3, }, // Europe/Prague
    {  115, 8816,   58689,  5, }, // Europe/Riga
    {  115, 6194,   58694,  9, }, // Europe/Sarajevo
    {  115, 6548,   58703,  7, }, // Europe/Sofia
    {  115, 6472,   58710, 12, }, // Europe/Stockholm
    {  115, 8846,   58722,  7, }, // Europe/Tallinn
    {  115, 6135,   58729,  6, }, // Europe/Tirane
    {  115, 6301,   58735,  6, }, // Europe/Vaduz
    {  115, 6164,   58741,  7, }, // Europe/Vienna
    {  115, 8878,   58748, 10, }, // Europe/Vilnius
    {  115, 3055,   58758,  5, }, // Europe/Warsaw
    {  115, 6489,   58763,  6, }, // Europe/Zurich
    {  115, 4980,   58769, 15, }, // Indian/Antananarivo
    {  115, 5979,   58784, 10, }, // Indian/Christmas
    {  115, 5996,   58794,  7, }, // Indian/Cocos
    {  115, 4931,   58801,  8, }, // Indian/Comoro
    {  115, 6624,   58809, 10, }, // Indian/Kerguelen
    {  115, 7013,   58819,  8, }, // Indian/Maldives
    {  115, 3971,   58827,  9, }, // Indian/Mauritius
    {  115, 7174,     467,  7, }, // Indian/Reunion
    {  115, 4324,   58836,  6, }, // Pacific/Apia
    {  115, 3022,     474,  5, }, // Pacific/Chuuk
    {  115, 7290,   58842,  8, }, // Pacific/Fakaofo
    {  115, 7306,   58850, 10, }, // Pacific/Funafuti
    {  115, 6659,   58860,  8, }, // Pacific/Gambier
    {  115, 4779,   58868,  8, }, // Pacific/Majuro
    {  115, 8909,   58876,  6, }, // Pacific/Midway
    {  115, 7047,   58882,  6, }, // Pacific/Nauru
    {  115, 2991,   58888, 10, }, // Pacific/Pago_Pago
    {  115, 7118,   58898,  5, }, // Pacific/Palau
    {  115, 2961,     479,  7, }, // Pacific/Pohnpei
    {  115, 6009,   58903, 12, }, // Pacific/Rarotonga
    {  115, 5948,   58915,  8, }, // Pacific/Saipan
    {  115, 4485,   58923, 10, }, // Pacific/Tongatapu
    {  115, 7355,   58933,  3, }, // Pacific/Wake
    {  116,   14,       0,  6, }, // Africa/Asmara Embu/Latin/Kenya
    {  116, 4337,       6,  8, }, // Africa/Sao_Tome
    {  116, 4154,      14,  8, }, // America/Asuncion
    {  116,  237,      22,  8, }, // America/Atikokan
    {  116, 7586,      30, 17, }, // America/Bahia_Banderas
    {  116, 3770,      47,  6, }, // America/Cancun
    {  116, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  116, 5640,      66,  7, }, // America/Curacao
    {  116,  481,      73, 13, }, // America/Indiana/Knox
    {  116, 7924,      86, 16, }, // America/Indiana/Marengo
    {  116, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  116, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  116, 8001,     139, 14, }, // America/Indiana/Vevay
    {  116, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  116, 8049,     171, 16, }, // America/Indiana/Winamac
    {  116, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  116, 5849,     207, 22, }, // America/Lower_Princes
    {  116, 8183,     229,  6, }, // America/Merida
    {  116, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  116, 8292,     255, 20, }, // America/North_Dakota/Center
    {  116, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  116,  393,     298,  4, }, // America/Nuuk
    {  116, 6862,     302, 16, }, // America/Scoresbysund
    {  116, 8471,     318, 14, }, // America/St_Barthelemy
    {  116, 2045,     332, 10, }, // America/St_Johns
    {  116, 5780,     342,  9, }, // America/St_Kitts
    {  116, 5797,     351,  9, }, // America/St_Lucia
    {  116,  855,     360, 10, }, // America/St_Thomas
    {  116, 5830,     370, 11, }, // America/St_Vincent
    {  116, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  116, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  116, 1152,     413,  9, }, // Asia/Kathmandu
    {  116,  992,     422,  7, }, // Asia/Kolkata
    {  116, 1178,     429,  5, }, // Asia/Macau
    {  116, 1202,     442,  6, }, // Asia/Yangon
    {  116, 1379,     448,  5, }, // Atlantic/Faroe
    {  116, 6784,     453, 10, }, // Atlantic/St_Helena
    {  116, 2419,     463,  4, }, // Europe/Kyiv
    {  116, 7174,     467,  7, }, // Indian/Reunion
    {  116, 3022,     474,  5, }, // Pacific/Chuuk
    {  116, 2961,     479,  7, }, // Pacific/Pohnpei
    {  117,   14,       0,  6, }, // Africa/Asmara English/Latin/United States
    {  117, 4337,       6,  8, }, // Africa/Sao_Tome
    {  117, 4154,      14,  8, }, // America/Asuncion
    {  117,  237,      22,  8, }, // America/Atikokan
    {  117, 7586,      30, 17, }, // America/Bahia_Banderas
    {  117, 3770,      47,  6, }, // America/Cancun
    {  117, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  117, 5640,      66,  7, }, // America/Curacao
    {  117,  481,      73, 13, }, // America/Indiana/Knox
    {  117, 7924,      86, 16, }, // America/Indiana/Marengo
    {  117, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  117, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  117, 8001,     139, 14, }, // America/Indiana/Vevay
    {  117, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  117, 8049,     171, 16, }, // America/Indiana/Winamac
    {  117, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  117, 5849,     207, 22, }, // America/Lower_Princes
    {  117, 8183,     229,  6, }, // America/Merida
    {  117, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  117, 8292,     255, 20, }, // America/North_Dakota/Center
    {  117, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  117,  393,     298,  4, }, // America/Nuuk
    {  117, 6862,     302, 16, }, // America/Scoresbysund
    {  117, 8471,     318, 14, }, // America/St_Barthelemy
    {  117, 2045,     332, 10, }, // America/St_Johns
    {  117, 5780,     342,  9, }, // America/St_Kitts
    {  117, 5797,     351,  9, }, // America/St_Lucia
    {  117,  855,     360, 10, }, // America/St_Thomas
    {  117, 5830,     370, 11, }, // America/St_Vincent
    {  117, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  117, 1226,     397, 16, }, // Asia/Ho_Chi_Minh
    {  117, 1152,     413,  9, }, // Asia/Kathmandu
    {  117,  992,     422,  7, }, // Asia/Kolkata
    {  117, 1178,     429,  5, }, // Asia/Macau
    {  117, 8671,     434,  8, }, // Asia/Qostanay
    {  117, 1202,     442,  6, }, // Asia/Yangon
    {  117, 1379,     448,  5, }, // Atlantic/Faroe
    {  117, 6784,     453, 10, }, // Atlantic/St_Helena
    {  117, 2419,     463,  4, }, // Europe/Kyiv
    {  117, 7174,     467,  7, }, // Indian/Reunion
    {  117, 3022,     474,  5, }, // Pacific/Chuuk
    {  117, 2961,     479,  7, }, // Pacific/Pohnpei
    {  118,  132,   58936,  8, }, // America/Adak English/Deseret/United States
    {  118, 3183,   58944, 14, }, // America/Anchorage
    {  118, 7662,   58958, 10, }, // America/Boise
    {  118, 3251,   58968, 12, }, // America/Chicago
    {  118,  805,   58980, 12, }, // America/Denver
    {  118, 3351,   58992, 14, }, // America/Detroit
    {  118,  348,   59006, 24, }, // America/Indiana/Indianapolis
    {  118,  481,   59030, 24, }, // America/Indiana/Knox
    {  118, 7924,   59054, 30, }, // America/Indiana/Marengo
    {  118, 7948,   59084, 36, }, // America/Indiana/Petersburg
    {  118, 7975,   59120, 31, }, // America/Indiana/Tell_City
    {  118, 8001,   59151, 24, }, // America/Indiana/Vevay
    {  118, 8023,   59175, 30, }, // America/Indiana/Vincennes
    {  118, 8049,   59205, 30, }, // America/Indiana/Winamac
    {  118, 5282,   59235,  8, }, // America/Juneau
    {  118,  521,   59243, 14, }, // America/Kentucky/Louisville
    {  118, 8088,   59257, 34, }, // America/Kentucky/Monticello
    {  118, 3390,   59291, 19, }, // America/Los_Angeles
    {  118, 8165,   59310, 16, }, // America/Menominee
    {  118, 3294,   59326, 13, }, // America/New_York
    {  118, 8251,   59339,  6, }, // America/Nome
    {  118, 8292,   59345, 35, }, // America/North_Dakota/Center
    {  118, 8320,   59380, 38, }, // America/North_Dakota/New_Salem
    {  118, 3224,   59418, 12, }, // America/Phoenix
    {  118, 8515,   59430, 14, }, // America/Yakutat
    {  118, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  118, 2929,   59444, 16, }, // Pacific/Honolulu
    {  118, 8909,   59460, 10, }, // Pacific/Midway
    {  118, 7355,   59470,  6, }, // Pacific/Wake
    {  120, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Anguilla
    {  120, 2045,   59489,  9, }, // America/St_Johns
    {  120, 5780,   59498,  8, }, // America/St_Kitts
    {  120, 5797,   59506,  8, }, // America/St_Lucia
    {  120,  855,   59514,  9, }, // America/St_Thomas
    {  120, 5830,   59523, 10, }, // America/St_Vincent
    {  120, 5440,    5906,  5, }, // Asia/Aqtau
    {  120, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  120, 2897,    4520,  9, }, // Pacific/Kanton
    {  121, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Antigua and Barbuda
    {  121, 2045,   59489,  9, }, // America/St_Johns
    {  121, 5780,   59498,  8, }, // America/St_Kitts
    {  121, 5797,   59506,  8, }, // America/St_Lucia
    {  121,  855,   59514,  9, }, // America/St_Thomas
    {  121, 5830,   59523, 10, }, // America/St_Vincent
    {  121, 5440,    5906,  5, }, // Asia/Aqtau
    {  121, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  121, 2897,    4520,  9, }, // Pacific/Kanton
    {  122, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Australia
    {  122, 2045,   59489,  9, }, // America/St_Johns
    {  122, 5780,   59498,  8, }, // America/St_Kitts
    {  122, 5797,   59506,  8, }, // America/St_Lucia
    {  122,  855,   59514,  9, }, // America/St_Thomas
    {  122, 5830,   59523, 10, }, // America/St_Vincent
    {  122, 5440,    5906,  5, }, // Asia/Aqtau
    {  122, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  122, 2897,    4520,  9, }, // Pacific/Kanton
    {  123, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Austria
    {  123, 2045,   59489,  9, }, // America/St_Johns
    {  123, 5780,   59498,  8, }, // America/St_Kitts
    {  123, 5797,   59506,  8, }, // America/St_Lucia
    {  123,  855,   59514,  9, }, // America/St_Thomas
    {  123, 5830,   59523, 10, }, // America/St_Vincent
    {  123, 5440,    5906,  5, }, // Asia/Aqtau
    {  123, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  123, 2897,    4520,  9, }, // Pacific/Kanton
    {  124, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Bahamas
    {  124, 2045,   59489,  9, }, // America/St_Johns
    {  124, 5780,   59498,  8, }, // America/St_Kitts
    {  124, 5797,   59506,  8, }, // America/St_Lucia
    {  124,  855,   59514,  9, }, // America/St_Thomas
    {  124, 5830,   59523, 10, }, // America/St_Vincent
    {  124, 5440,    5906,  5, }, // Asia/Aqtau
    {  124, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  124, 2897,    4520,  9, }, // Pacific/Kanton
    {  125, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Barbados
    {  125, 2045,   59489,  9, }, // America/St_Johns
    {  125, 5780,   59498,  8, }, // America/St_Kitts
    {  125, 5797,   59506,  8, }, // America/St_Lucia
    {  125,  855,   59514,  9, }, // America/St_Thomas
    {  125, 5830,   59523, 10, }, // America/St_Vincent
    {  125, 5440,    5906,  5, }, // Asia/Aqtau
    {  125, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  125, 2897,    4520,  9, }, // Pacific/Kanton
    {  126, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Belgium
    {  126, 2045,   59489,  9, }, // America/St_Johns
    {  126, 5780,   59498,  8, }, // America/St_Kitts
    {  126, 5797,   59506,  8, }, // America/St_Lucia
    {  126,  855,   59514,  9, }, // America/St_Thomas
    {  126, 5830,   59523, 10, }, // America/St_Vincent
    {  126, 5440,    5906,  5, }, // Asia/Aqtau
    {  126, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  126, 2897,    4520,  9, }, // Pacific/Kanton
    {  127, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Belize
    {  127, 2045,   59489,  9, }, // America/St_Johns
    {  127, 5780,   59498,  8, }, // America/St_Kitts
    {  127, 5797,   59506,  8, }, // America/St_Lucia
    {  127,  855,   59514,  9, }, // America/St_Thomas
    {  127, 5830,   59523, 10, }, // America/St_Vincent
    {  127, 5440,    5906,  5, }, // Asia/Aqtau
    {  127, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  127, 2897,    4520,  9, }, // Pacific/Kanton
    {  128, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Bermuda
    {  128, 2045,   59489,  9, }, // America/St_Johns
    {  128, 5780,   59498,  8, }, // America/St_Kitts
    {  128, 5797,   59506,  8, }, // America/St_Lucia
    {  128,  855,   59514,  9, }, // America/St_Thomas
    {  128, 5830,   59523, 10, }, // America/St_Vincent
    {  128, 5440,    5906,  5, }, // Asia/Aqtau
    {  128, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  128, 2897,    4520,  9, }, // Pacific/Kanton
    {  129, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Botswana
    {  129, 2045,   59489,  9, }, // America/St_Johns
    {  129, 5780,   59498,  8, }, // America/St_Kitts
    {  129, 5797,   59506,  8, }, // America/St_Lucia
    {  129,  855,   59514,  9, }, // America/St_Thomas
    {  129, 5830,   59523, 10, }, // America/St_Vincent
    {  129, 5440,    5906,  5, }, // Asia/Aqtau
    {  129, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  129, 2897,    4520,  9, }, // Pacific/Kanton
    {  130, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/British Indian Ocean Territory
    {  130, 2045,   59489,  9, }, // America/St_Johns
    {  130, 5780,   59498,  8, }, // America/St_Kitts
    {  130, 5797,   59506,  8, }, // America/St_Lucia
    {  130,  855,   59514,  9, }, // America/St_Thomas
    {  130, 5830,   59523, 10, }, // America/St_Vincent
    {  130, 5440,    5906,  5, }, // Asia/Aqtau
    {  130, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  130, 2897,    4520,  9, }, // Pacific/Kanton
    {  131, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/British Virgin Islands
    {  131, 2045,   59489,  9, }, // America/St_Johns
    {  131, 5780,   59498,  8, }, // America/St_Kitts
    {  131, 5797,   59506,  8, }, // America/St_Lucia
    {  131,  855,   59514,  9, }, // America/St_Thomas
    {  131, 5830,   59523, 10, }, // America/St_Vincent
    {  131, 5440,    5906,  5, }, // Asia/Aqtau
    {  131, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  131, 2897,    4520,  9, }, // Pacific/Kanton
    {  133, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Cameroon
    {  133, 2045,   59489,  9, }, // America/St_Johns
    {  133, 5780,   59498,  8, }, // America/St_Kitts
    {  133, 5797,   59506,  8, }, // America/St_Lucia
    {  133,  855,   59514,  9, }, // America/St_Thomas
    {  133, 5830,   59523, 10, }, // America/St_Vincent
    {  133, 5440,    5906,  5, }, // Asia/Aqtau
    {  133, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  133, 2897,    4520,  9, }, // Pacific/Kanton
    {  134, 8471,   25154, 16, }, // America/St_Barthelemy English/Latin/Canada
    {  134, 2045,   25170, 12, }, // America/St_Johns
    {  134, 5780,   25182, 11, }, // America/St_Kitts
    {  134, 5797,   34444, 11, }, // America/St_Lucia
    {  134,  855,   42797, 12, }, // America/St_Thomas
    {  134, 5830,   25216, 13, }, // America/St_Vincent
    {  134, 5440,    5906,  5, }, // Asia/Aqtau
    {  134, 1202,   56146,  7, }, // Asia/Yangon
    {  134, 6784,   34678, 12, }, // Atlantic/St_Helena
    {  135, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Cayman Islands
    {  135, 2045,   59489,  9, }, // America/St_Johns
    {  135, 5780,   59498,  8, }, // America/St_Kitts
    {  135, 5797,   59506,  8, }, // America/St_Lucia
    {  135,  855,   59514,  9, }, // America/St_Thomas
    {  135, 5830,   59523, 10, }, // America/St_Vincent
    {  135, 5440,    5906,  5, }, // Asia/Aqtau
    {  135, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  135, 2897,    4520,  9, }, // Pacific/Kanton
    {  136, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Christmas Island
    {  136, 2045,   59489,  9, }, // America/St_Johns
    {  136, 5780,   59498,  8, }, // America/St_Kitts
    {  136, 5797,   59506,  8, }, // America/St_Lucia
    {  136,  855,   59514,  9, }, // America/St_Thomas
    {  136, 5830,   59523, 10, }, // America/St_Vincent
    {  136, 5440,    5906,  5, }, // Asia/Aqtau
    {  136, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  136, 2897,    4520,  9, }, // Pacific/Kanton
    {  137, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Cocos Islands
    {  137, 2045,   59489,  9, }, // America/St_Johns
    {  137, 5780,   59498,  8, }, // America/St_Kitts
    {  137, 5797,   59506,  8, }, // America/St_Lucia
    {  137,  855,   59514,  9, }, // America/St_Thomas
    {  137, 5830,   59523, 10, }, // America/St_Vincent
    {  137, 5440,    5906,  5, }, // Asia/Aqtau
    {  137, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  137, 2897,    4520,  9, }, // Pacific/Kanton
    {  138, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Cook Islands
    {  138, 2045,   59489,  9, }, // America/St_Johns
    {  138, 5780,   59498,  8, }, // America/St_Kitts
    {  138, 5797,   59506,  8, }, // America/St_Lucia
    {  138,  855,   59514,  9, }, // America/St_Thomas
    {  138, 5830,   59523, 10, }, // America/St_Vincent
    {  138, 5440,    5906,  5, }, // Asia/Aqtau
    {  138, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  138, 2897,    4520,  9, }, // Pacific/Kanton
    {  139, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Cyprus
    {  139, 2045,   59489,  9, }, // America/St_Johns
    {  139, 5780,   59498,  8, }, // America/St_Kitts
    {  139, 5797,   59506,  8, }, // America/St_Lucia
    {  139,  855,   59514,  9, }, // America/St_Thomas
    {  139, 5830,   59523, 10, }, // America/St_Vincent
    {  139, 5440,    5906,  5, }, // Asia/Aqtau
    {  139, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  139, 2897,    4520,  9, }, // Pacific/Kanton
    {  140, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Denmark
    {  140, 2045,   59489,  9, }, // America/St_Johns
    {  140, 5780,   59498,  8, }, // America/St_Kitts
    {  140, 5797,   59506,  8, }, // America/St_Lucia
    {  140,  855,   59514,  9, }, // America/St_Thomas
    {  140, 5830,   59523, 10, }, // America/St_Vincent
    {  140, 5440,    5906,  5, }, // Asia/Aqtau
    {  140, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  140, 2897,    4520,  9, }, // Pacific/Kanton
    {  141, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Diego Garcia
    {  141, 2045,   59489,  9, }, // America/St_Johns
    {  141, 5780,   59498,  8, }, // America/St_Kitts
    {  141, 5797,   59506,  8, }, // America/St_Lucia
    {  141,  855,   59514,  9, }, // America/St_Thomas
    {  141, 5830,   59523, 10, }, // America/St_Vincent
    {  141, 5440,    5906,  5, }, // Asia/Aqtau
    {  141, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  141, 2897,    4520,  9, }, // Pacific/Kanton
    {  142, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Dominica
    {  142, 2045,   59489,  9, }, // America/St_Johns
    {  142, 5780,   59498,  8, }, // America/St_Kitts
    {  142, 5797,   59506,  8, }, // America/St_Lucia
    {  142,  855,   59514,  9, }, // America/St_Thomas
    {  142, 5830,   59523, 10, }, // America/St_Vincent
    {  142, 5440,    5906,  5, }, // Asia/Aqtau
    {  142, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  142, 2897,    4520,  9, }, // Pacific/Kanton
    {  143, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Eritrea
    {  143, 2045,   59489,  9, }, // America/St_Johns
    {  143, 5780,   59498,  8, }, // America/St_Kitts
    {  143, 5797,   59506,  8, }, // America/St_Lucia
    {  143,  855,   59514,  9, }, // America/St_Thomas
    {  143, 5830,   59523, 10, }, // America/St_Vincent
    {  143, 5440,    5906,  5, }, // Asia/Aqtau
    {  143, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  143, 2897,    4520,  9, }, // Pacific/Kanton
    {  144, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Eswatini
    {  144, 2045,   59489,  9, }, // America/St_Johns
    {  144, 5780,   59498,  8, }, // America/St_Kitts
    {  144, 5797,   59506,  8, }, // America/St_Lucia
    {  144,  855,   59514,  9, }, // America/St_Thomas
    {  144, 5830,   59523, 10, }, // America/St_Vincent
    {  144, 5440,    5906,  5, }, // Asia/Aqtau
    {  144, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  144, 2897,    4520,  9, }, // Pacific/Kanton
    {  145, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Europe
    {  145, 2045,   59489,  9, }, // America/St_Johns
    {  145, 5780,   59498,  8, }, // America/St_Kitts
    {  145, 5797,   59506,  8, }, // America/St_Lucia
    {  145,  855,   59514,  9, }, // America/St_Thomas
    {  145, 5830,   59523, 10, }, // America/St_Vincent
    {  145, 5440,    5906,  5, }, // Asia/Aqtau
    {  145, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  145, 2897,    4520,  9, }, // Pacific/Kanton
    {  146, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Falkland Islands
    {  146, 2045,   59489,  9, }, // America/St_Johns
    {  146, 5780,   59498,  8, }, // America/St_Kitts
    {  146, 5797,   59506,  8, }, // America/St_Lucia
    {  146,  855,   59514,  9, }, // America/St_Thomas
    {  146, 5830,   59523, 10, }, // America/St_Vincent
    {  146, 5440,    5906,  5, }, // Asia/Aqtau
    {  146, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  146, 2897,    4520,  9, }, // Pacific/Kanton
    {  147, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Fiji
    {  147, 2045,   59489,  9, }, // America/St_Johns
    {  147, 5780,   59498,  8, }, // America/St_Kitts
    {  147, 5797,   59506,  8, }, // America/St_Lucia
    {  147,  855,   59514,  9, }, // America/St_Thomas
    {  147, 5830,   59523, 10, }, // America/St_Vincent
    {  147, 5440,    5906,  5, }, // Asia/Aqtau
    {  147, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  147, 2897,    4520,  9, }, // Pacific/Kanton
    {  148, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Finland
    {  148, 2045,   59489,  9, }, // America/St_Johns
    {  148, 5780,   59498,  8, }, // America/St_Kitts
    {  148, 5797,   59506,  8, }, // America/St_Lucia
    {  148,  855,   59514,  9, }, // America/St_Thomas
    {  148, 5830,   59523, 10, }, // America/St_Vincent
    {  148, 5440,    5906,  5, }, // Asia/Aqtau
    {  148, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  148, 2897,    4520,  9, }, // Pacific/Kanton
    {  149, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Gambia
    {  149, 2045,   59489,  9, }, // America/St_Johns
    {  149, 5780,   59498,  8, }, // America/St_Kitts
    {  149, 5797,   59506,  8, }, // America/St_Lucia
    {  149,  855,   59514,  9, }, // America/St_Thomas
    {  149, 5830,   59523, 10, }, // America/St_Vincent
    {  149, 5440,    5906,  5, }, // Asia/Aqtau
    {  149, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  149, 2897,    4520,  9, }, // Pacific/Kanton
    {  150, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Germany
    {  150, 2045,   59489,  9, }, // America/St_Johns
    {  150, 5780,   59498,  8, }, // America/St_Kitts
    {  150, 5797,   59506,  8, }, // America/St_Lucia
    {  150,  855,   59514,  9, }, // America/St_Thomas
    {  150, 5830,   59523, 10, }, // America/St_Vincent
    {  150, 5440,    5906,  5, }, // Asia/Aqtau
    {  150, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  150, 2897,    4520,  9, }, // Pacific/Kanton
    {  151, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Ghana
    {  151, 2045,   59489,  9, }, // America/St_Johns
    {  151, 5780,   59498,  8, }, // America/St_Kitts
    {  151, 5797,   59506,  8, }, // America/St_Lucia
    {  151,  855,   59514,  9, }, // America/St_Thomas
    {  151, 5830,   59523, 10, }, // America/St_Vincent
    {  151, 5440,    5906,  5, }, // Asia/Aqtau
    {  151, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  151, 2897,    4520,  9, }, // Pacific/Kanton
    {  152, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Gibraltar
    {  152, 2045,   59489,  9, }, // America/St_Johns
    {  152, 5780,   59498,  8, }, // America/St_Kitts
    {  152, 5797,   59506,  8, }, // America/St_Lucia
    {  152,  855,   59514,  9, }, // America/St_Thomas
    {  152, 5830,   59523, 10, }, // America/St_Vincent
    {  152, 5440,    5906,  5, }, // Asia/Aqtau
    {  152, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  152, 2897,    4520,  9, }, // Pacific/Kanton
    {  153, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Grenada
    {  153, 2045,   59489,  9, }, // America/St_Johns
    {  153, 5780,   59498,  8, }, // America/St_Kitts
    {  153, 5797,   59506,  8, }, // America/St_Lucia
    {  153,  855,   59514,  9, }, // America/St_Thomas
    {  153, 5830,   59523, 10, }, // America/St_Vincent
    {  153, 5440,    5906,  5, }, // Asia/Aqtau
    {  153, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  153, 2897,    4520,  9, }, // Pacific/Kanton
    {  155, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Guernsey
    {  155, 2045,   59489,  9, }, // America/St_Johns
    {  155, 5780,   59498,  8, }, // America/St_Kitts
    {  155, 5797,   59506,  8, }, // America/St_Lucia
    {  155,  855,   59514,  9, }, // America/St_Thomas
    {  155, 5830,   59523, 10, }, // America/St_Vincent
    {  155, 5440,    5906,  5, }, // Asia/Aqtau
    {  155, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  155, 2897,    4520,  9, }, // Pacific/Kanton
    {  156, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Guyana
    {  156, 2045,   59489,  9, }, // America/St_Johns
    {  156, 5780,   59498,  8, }, // America/St_Kitts
    {  156, 5797,   59506,  8, }, // America/St_Lucia
    {  156,  855,   59514,  9, }, // America/St_Thomas
    {  156, 5830,   59523, 10, }, // America/St_Vincent
    {  156, 5440,    5906,  5, }, // Asia/Aqtau
    {  156, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  156, 2897,    4520,  9, }, // Pacific/Kanton
    {  157, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Hong Kong
    {  157, 2045,   59489,  9, }, // America/St_Johns
    {  157, 5780,   59498,  8, }, // America/St_Kitts
    {  157, 5797,   59506,  8, }, // America/St_Lucia
    {  157,  855,   59514,  9, }, // America/St_Thomas
    {  157, 5830,   59523, 10, }, // America/St_Vincent
    {  157, 5440,    5906,  5, }, // Asia/Aqtau
    {  157, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  157, 2897,    4520,  9, }, // Pacific/Kanton
    {  158, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/India
    {  158, 2045,   59489,  9, }, // America/St_Johns
    {  158, 5780,   59498,  8, }, // America/St_Kitts
    {  158, 5797,   59506,  8, }, // America/St_Lucia
    {  158,  855,   59514,  9, }, // America/St_Thomas
    {  158, 5830,   59523, 10, }, // America/St_Vincent
    {  158, 5440,    5906,  5, }, // Asia/Aqtau
    {  158, 1202,   56146,  7, }, // Asia/Yangon
    {  158, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  158, 2897,    4520,  9, }, // Pacific/Kanton
    {  159, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Indonesia
    {  159, 2045,   59489,  9, }, // America/St_Johns
    {  159, 5780,   59498,  8, }, // America/St_Kitts
    {  159, 5797,   59506,  8, }, // America/St_Lucia
    {  159,  855,   59514,  9, }, // America/St_Thomas
    {  159, 5830,   59523, 10, }, // America/St_Vincent
    {  159, 5440,    5906,  5, }, // Asia/Aqtau
    {  159, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  159, 2897,    4520,  9, }, // Pacific/Kanton
    {  160, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Ireland
    {  160, 2045,   59489,  9, }, // America/St_Johns
    {  160, 5780,   59498,  8, }, // America/St_Kitts
    {  160, 5797,   59506,  8, }, // America/St_Lucia
    {  160,  855,   59514,  9, }, // America/St_Thomas
    {  160, 5830,   59523, 10, }, // America/St_Vincent
    {  160, 5440,    5906,  5, }, // Asia/Aqtau
    {  160, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  160, 2897,    4520,  9, }, // Pacific/Kanton
    {  161, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Isle of Man
    {  161, 2045,   59489,  9, }, // America/St_Johns
    {  161, 5780,   59498,  8, }, // America/St_Kitts
    {  161, 5797,   59506,  8, }, // America/St_Lucia
    {  161,  855,   59514,  9, }, // America/St_Thomas
    {  161, 5830,   59523, 10, }, // America/St_Vincent
    {  161, 5440,    5906,  5, }, // Asia/Aqtau
    {  161, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  161, 2897,    4520,  9, }, // Pacific/Kanton
    {  162, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Israel
    {  162, 2045,   59489,  9, }, // America/St_Johns
    {  162, 5780,   59498,  8, }, // America/St_Kitts
    {  162, 5797,   59506,  8, }, // America/St_Lucia
    {  162,  855,   59514,  9, }, // America/St_Thomas
    {  162, 5830,   59523, 10, }, // America/St_Vincent
    {  162, 5440,    5906,  5, }, // Asia/Aqtau
    {  162, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  162, 2897,    4520,  9, }, // Pacific/Kanton
    {  163, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Jamaica
    {  163, 2045,   59489,  9, }, // America/St_Johns
    {  163, 5780,   59498,  8, }, // America/St_Kitts
    {  163, 5797,   59506,  8, }, // America/St_Lucia
    {  163,  855,   59514,  9, }, // America/St_Thomas
    {  163, 5830,   59523, 10, }, // America/St_Vincent
    {  163, 5440,    5906,  5, }, // Asia/Aqtau
    {  163, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  163, 2897,    4520,  9, }, // Pacific/Kanton
    {  164, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Jersey
    {  164, 2045,   59489,  9, }, // America/St_Johns
    {  164, 5780,   59498,  8, }, // America/St_Kitts
    {  164, 5797,   59506,  8, }, // America/St_Lucia
    {  164,  855,   59514,  9, }, // America/St_Thomas
    {  164, 5830,   59523, 10, }, // America/St_Vincent
    {  164, 5440,    5906,  5, }, // Asia/Aqtau
    {  164, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  164, 2897,    4520,  9, }, // Pacific/Kanton
    {  165, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Kenya
    {  165, 2045,   59489,  9, }, // America/St_Johns
    {  165, 5780,   59498,  8, }, // America/St_Kitts
    {  165, 5797,   59506,  8, }, // America/St_Lucia
    {  165,  855,   59514,  9, }, // America/St_Thomas
    {  165, 5830,   59523, 10, }, // America/St_Vincent
    {  165, 5440,    5906,  5, }, // Asia/Aqtau
    {  165, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  165, 2897,    4520,  9, }, // Pacific/Kanton
    {  166, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Kiribati
    {  166, 2045,   59489,  9, }, // America/St_Johns
    {  166, 5780,   59498,  8, }, // America/St_Kitts
    {  166, 5797,   59506,  8, }, // America/St_Lucia
    {  166,  855,   59514,  9, }, // America/St_Thomas
    {  166, 5830,   59523, 10, }, // America/St_Vincent
    {  166, 5440,    5906,  5, }, // Asia/Aqtau
    {  166, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  166, 2897,    4520,  9, }, // Pacific/Kanton
    {  167, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Lesotho
    {  167, 2045,   59489,  9, }, // America/St_Johns
    {  167, 5780,   59498,  8, }, // America/St_Kitts
    {  167, 5797,   59506,  8, }, // America/St_Lucia
    {  167,  855,   59514,  9, }, // America/St_Thomas
    {  167, 5830,   59523, 10, }, // America/St_Vincent
    {  167, 5440,    5906,  5, }, // Asia/Aqtau
    {  167, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  167, 2897,    4520,  9, }, // Pacific/Kanton
    {  168, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Liberia
    {  168, 2045,   59489,  9, }, // America/St_Johns
    {  168, 5780,   59498,  8, }, // America/St_Kitts
    {  168, 5797,   59506,  8, }, // America/St_Lucia
    {  168,  855,   59514,  9, }, // America/St_Thomas
    {  168, 5830,   59523, 10, }, // America/St_Vincent
    {  168, 5440,    5906,  5, }, // Asia/Aqtau
    {  168, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  168, 2897,    4520,  9, }, // Pacific/Kanton
    {  169, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Macao
    {  169, 2045,   59489,  9, }, // America/St_Johns
    {  169, 5780,   59498,  8, }, // America/St_Kitts
    {  169, 5797,   59506,  8, }, // America/St_Lucia
    {  169,  855,   59514,  9, }, // America/St_Thomas
    {  169, 5830,   59523, 10, }, // America/St_Vincent
    {  169, 5440,    5906,  5, }, // Asia/Aqtau
    {  169, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  169, 2897,    4520,  9, }, // Pacific/Kanton
    {  170, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Madagascar
    {  170, 2045,   59489,  9, }, // America/St_Johns
    {  170, 5780,   59498,  8, }, // America/St_Kitts
    {  170, 5797,   59506,  8, }, // America/St_Lucia
    {  170,  855,   59514,  9, }, // America/St_Thomas
    {  170, 5830,   59523, 10, }, // America/St_Vincent
    {  170, 5440,    5906,  5, }, // Asia/Aqtau
    {  170, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  170, 2897,    4520,  9, }, // Pacific/Kanton
    {  171, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Malawi
    {  171, 2045,   59489,  9, }, // America/St_Johns
    {  171, 5780,   59498,  8, }, // America/St_Kitts
    {  171, 5797,   59506,  8, }, // America/St_Lucia
    {  171,  855,   59514,  9, }, // America/St_Thomas
    {  171, 5830,   59523, 10, }, // America/St_Vincent
    {  171, 5440,    5906,  5, }, // Asia/Aqtau
    {  171, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  171, 2897,    4520,  9, }, // Pacific/Kanton
    {  172, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Malaysia
    {  172, 2045,   59489,  9, }, // America/St_Johns
    {  172, 5780,   59498,  8, }, // America/St_Kitts
    {  172, 5797,   59506,  8, }, // America/St_Lucia
    {  172,  855,   59514,  9, }, // America/St_Thomas
    {  172, 5830,   59523, 10, }, // America/St_Vincent
    {  172, 5440,    5906,  5, }, // Asia/Aqtau
    {  172, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  172, 2897,    4520,  9, }, // Pacific/Kanton
    {  173, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Maldives
    {  173, 2045,   59489,  9, }, // America/St_Johns
    {  173, 5780,   59498,  8, }, // America/St_Kitts
    {  173, 5797,   59506,  8, }, // America/St_Lucia
    {  173,  855,   59514,  9, }, // America/St_Thomas
    {  173, 5830,   59523, 10, }, // America/St_Vincent
    {  173, 5440,    5906,  5, }, // Asia/Aqtau
    {  173, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  173, 2897,    4520,  9, }, // Pacific/Kanton
    {  174, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Malta
    {  174, 2045,   59489,  9, }, // America/St_Johns
    {  174, 5780,   59498,  8, }, // America/St_Kitts
    {  174, 5797,   59506,  8, }, // America/St_Lucia
    {  174,  855,   59514,  9, }, // America/St_Thomas
    {  174, 5830,   59523, 10, }, // America/St_Vincent
    {  174, 5440,    5906,  5, }, // Asia/Aqtau
    {  174, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  174, 2897,    4520,  9, }, // Pacific/Kanton
    {  176, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Mauritius
    {  176, 2045,   59489,  9, }, // America/St_Johns
    {  176, 5780,   59498,  8, }, // America/St_Kitts
    {  176, 5797,   59506,  8, }, // America/St_Lucia
    {  176,  855,   59514,  9, }, // America/St_Thomas
    {  176, 5830,   59523, 10, }, // America/St_Vincent
    {  176, 5440,    5906,  5, }, // Asia/Aqtau
    {  176, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  176, 2897,    4520,  9, }, // Pacific/Kanton
    {  177, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Micronesia
    {  177, 2045,   59489,  9, }, // America/St_Johns
    {  177, 5780,   59498,  8, }, // America/St_Kitts
    {  177, 5797,   59506,  8, }, // America/St_Lucia
    {  177,  855,   59514,  9, }, // America/St_Thomas
    {  177, 5830,   59523, 10, }, // America/St_Vincent
    {  177, 5440,    5906,  5, }, // Asia/Aqtau
    {  177, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  177, 2897,    4520,  9, }, // Pacific/Kanton
    {  178, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Montserrat
    {  178, 2045,   59489,  9, }, // America/St_Johns
    {  178, 5780,   59498,  8, }, // America/St_Kitts
    {  178, 5797,   59506,  8, }, // America/St_Lucia
    {  178,  855,   59514,  9, }, // America/St_Thomas
    {  178, 5830,   59523, 10, }, // America/St_Vincent
    {  178, 5440,    5906,  5, }, // Asia/Aqtau
    {  178, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  178, 2897,    4520,  9, }, // Pacific/Kanton
    {  179, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Namibia
    {  179, 2045,   59489,  9, }, // America/St_Johns
    {  179, 5780,   59498,  8, }, // America/St_Kitts
    {  179, 5797,   59506,  8, }, // America/St_Lucia
    {  179,  855,   59514,  9, }, // America/St_Thomas
    {  179, 5830,   59523, 10, }, // America/St_Vincent
    {  179, 5440,    5906,  5, }, // Asia/Aqtau
    {  179, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  179, 2897,    4520,  9, }, // Pacific/Kanton
    {  180, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Nauru
    {  180, 2045,   59489,  9, }, // America/St_Johns
    {  180, 5780,   59498,  8, }, // America/St_Kitts
    {  180, 5797,   59506,  8, }, // America/St_Lucia
    {  180,  855,   59514,  9, }, // America/St_Thomas
    {  180, 5830,   59523, 10, }, // America/St_Vincent
    {  180, 5440,    5906,  5, }, // Asia/Aqtau
    {  180, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  180, 2897,    4520,  9, }, // Pacific/Kanton
    {  181, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Netherlands
    {  181, 2045,   59489,  9, }, // America/St_Johns
    {  181, 5780,   59498,  8, }, // America/St_Kitts
    {  181, 5797,   59506,  8, }, // America/St_Lucia
    {  181,  855,   59514,  9, }, // America/St_Thomas
    {  181, 5830,   59523, 10, }, // America/St_Vincent
    {  181, 5440,    5906,  5, }, // Asia/Aqtau
    {  181, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  181, 2897,    4520,  9, }, // Pacific/Kanton
    {  182, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/New Zealand
    {  182, 2045,   59489,  9, }, // America/St_Johns
    {  182, 5780,   59498,  8, }, // America/St_Kitts
    {  182, 5797,   59506,  8, }, // America/St_Lucia
    {  182,  855,   59514,  9, }, // America/St_Thomas
    {  182, 5830,   59523, 10, }, // America/St_Vincent
    {  182, 5440,    5906,  5, }, // Asia/Aqtau
    {  182, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  182, 2897,    4520,  9, }, // Pacific/Kanton
    {  183, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Nigeria
    {  183, 2045,   59489,  9, }, // America/St_Johns
    {  183, 5780,   59498,  8, }, // America/St_Kitts
    {  183, 5797,   59506,  8, }, // America/St_Lucia
    {  183,  855,   59514,  9, }, // America/St_Thomas
    {  183, 5830,   59523, 10, }, // America/St_Vincent
    {  183, 5440,    5906,  5, }, // Asia/Aqtau
    {  183, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  183, 2897,    4520,  9, }, // Pacific/Kanton
    {  184, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Niue
    {  184, 2045,   59489,  9, }, // America/St_Johns
    {  184, 5780,   59498,  8, }, // America/St_Kitts
    {  184, 5797,   59506,  8, }, // America/St_Lucia
    {  184,  855,   59514,  9, }, // America/St_Thomas
    {  184, 5830,   59523, 10, }, // America/St_Vincent
    {  184, 5440,    5906,  5, }, // Asia/Aqtau
    {  184, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  184, 2897,    4520,  9, }, // Pacific/Kanton
    {  185, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Norfolk Island
    {  185, 2045,   59489,  9, }, // America/St_Johns
    {  185, 5780,   59498,  8, }, // America/St_Kitts
    {  185, 5797,   59506,  8, }, // America/St_Lucia
    {  185,  855,   59514,  9, }, // America/St_Thomas
    {  185, 5830,   59523, 10, }, // America/St_Vincent
    {  185, 5440,    5906,  5, }, // Asia/Aqtau
    {  185, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  185, 2897,    4520,  9, }, // Pacific/Kanton
    {  187, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Pakistan
    {  187, 2045,   59489,  9, }, // America/St_Johns
    {  187, 5780,   59498,  8, }, // America/St_Kitts
    {  187, 5797,   59506,  8, }, // America/St_Lucia
    {  187,  855,   59514,  9, }, // America/St_Thomas
    {  187, 5830,   59523, 10, }, // America/St_Vincent
    {  187, 5440,    5906,  5, }, // Asia/Aqtau
    {  187, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  187, 2897,    4520,  9, }, // Pacific/Kanton
    {  188, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Palau
    {  188, 2045,   59489,  9, }, // America/St_Johns
    {  188, 5780,   59498,  8, }, // America/St_Kitts
    {  188, 5797,   59506,  8, }, // America/St_Lucia
    {  188,  855,   59514,  9, }, // America/St_Thomas
    {  188, 5830,   59523, 10, }, // America/St_Vincent
    {  188, 5440,    5906,  5, }, // Asia/Aqtau
    {  188, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  188, 2897,    4520,  9, }, // Pacific/Kanton
    {  189, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Papua New Guinea
    {  189, 2045,   59489,  9, }, // America/St_Johns
    {  189, 5780,   59498,  8, }, // America/St_Kitts
    {  189, 5797,   59506,  8, }, // America/St_Lucia
    {  189,  855,   59514,  9, }, // America/St_Thomas
    {  189, 5830,   59523, 10, }, // America/St_Vincent
    {  189, 5440,    5906,  5, }, // Asia/Aqtau
    {  189, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  189, 2897,    4520,  9, }, // Pacific/Kanton
    {  191, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Pitcairn
    {  191, 2045,   59489,  9, }, // America/St_Johns
    {  191, 5780,   59498,  8, }, // America/St_Kitts
    {  191, 5797,   59506,  8, }, // America/St_Lucia
    {  191,  855,   59514,  9, }, // America/St_Thomas
    {  191, 5830,   59523, 10, }, // America/St_Vincent
    {  191, 5440,    5906,  5, }, // Asia/Aqtau
    {  191, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  191, 2897,    4520,  9, }, // Pacific/Kanton
    {  193, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Rwanda
    {  193, 2045,   59489,  9, }, // America/St_Johns
    {  193, 5780,   59498,  8, }, // America/St_Kitts
    {  193, 5797,   59506,  8, }, // America/St_Lucia
    {  193,  855,   59514,  9, }, // America/St_Thomas
    {  193, 5830,   59523, 10, }, // America/St_Vincent
    {  193, 5440,    5906,  5, }, // Asia/Aqtau
    {  193, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  193, 2897,    4520,  9, }, // Pacific/Kanton
    {  194, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Saint Helena
    {  194, 2045,   59489,  9, }, // America/St_Johns
    {  194, 5780,   59498,  8, }, // America/St_Kitts
    {  194, 5797,   59506,  8, }, // America/St_Lucia
    {  194,  855,   59514,  9, }, // America/St_Thomas
    {  194, 5830,   59523, 10, }, // America/St_Vincent
    {  194, 5440,    5906,  5, }, // Asia/Aqtau
    {  194, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  194, 2897,    4520,  9, }, // Pacific/Kanton
    {  195, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Saint Kitts and Nevis
    {  195, 2045,   59489,  9, }, // America/St_Johns
    {  195, 5780,   59498,  8, }, // America/St_Kitts
    {  195, 5797,   59506,  8, }, // America/St_Lucia
    {  195,  855,   59514,  9, }, // America/St_Thomas
    {  195, 5830,   59523, 10, }, // America/St_Vincent
    {  195, 5440,    5906,  5, }, // Asia/Aqtau
    {  195, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  195, 2897,    4520,  9, }, // Pacific/Kanton
    {  196, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Saint Lucia
    {  196, 2045,   59489,  9, }, // America/St_Johns
    {  196, 5780,   59498,  8, }, // America/St_Kitts
    {  196, 5797,   59506,  8, }, // America/St_Lucia
    {  196,  855,   59514,  9, }, // America/St_Thomas
    {  196, 5830,   59523, 10, }, // America/St_Vincent
    {  196, 5440,    5906,  5, }, // Asia/Aqtau
    {  196, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  196, 2897,    4520,  9, }, // Pacific/Kanton
    {  197, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Saint Vincent and Grenadines
    {  197, 2045,   59489,  9, }, // America/St_Johns
    {  197, 5780,   59498,  8, }, // America/St_Kitts
    {  197, 5797,   59506,  8, }, // America/St_Lucia
    {  197,  855,   59514,  9, }, // America/St_Thomas
    {  197, 5830,   59523, 10, }, // America/St_Vincent
    {  197, 5440,    5906,  5, }, // Asia/Aqtau
    {  197, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  197, 2897,    4520,  9, }, // Pacific/Kanton
    {  198, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Samoa
    {  198, 2045,   59489,  9, }, // America/St_Johns
    {  198, 5780,   59498,  8, }, // America/St_Kitts
    {  198, 5797,   59506,  8, }, // America/St_Lucia
    {  198,  855,   59514,  9, }, // America/St_Thomas
    {  198, 5830,   59523, 10, }, // America/St_Vincent
    {  198, 5440,    5906,  5, }, // Asia/Aqtau
    {  198, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  198, 2897,    4520,  9, }, // Pacific/Kanton
    {  199, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Seychelles
    {  199, 2045,   59489,  9, }, // America/St_Johns
    {  199, 5780,   59498,  8, }, // America/St_Kitts
    {  199, 5797,   59506,  8, }, // America/St_Lucia
    {  199,  855,   59514,  9, }, // America/St_Thomas
    {  199, 5830,   59523, 10, }, // America/St_Vincent
    {  199, 5440,    5906,  5, }, // Asia/Aqtau
    {  199, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  199, 2897,    4520,  9, }, // Pacific/Kanton
    {  200, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Sierra Leone
    {  200, 2045,   59489,  9, }, // America/St_Johns
    {  200, 5780,   59498,  8, }, // America/St_Kitts
    {  200, 5797,   59506,  8, }, // America/St_Lucia
    {  200,  855,   59514,  9, }, // America/St_Thomas
    {  200, 5830,   59523, 10, }, // America/St_Vincent
    {  200, 5440,    5906,  5, }, // Asia/Aqtau
    {  200, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  200, 2897,    4520,  9, }, // Pacific/Kanton
    {  201, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Singapore
    {  201, 2045,   59489,  9, }, // America/St_Johns
    {  201, 5780,   59498,  8, }, // America/St_Kitts
    {  201, 5797,   59506,  8, }, // America/St_Lucia
    {  201,  855,   59514,  9, }, // America/St_Thomas
    {  201, 5830,   59523, 10, }, // America/St_Vincent
    {  201, 5440,    5906,  5, }, // Asia/Aqtau
    {  201, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  201, 2897,    4520,  9, }, // Pacific/Kanton
    {  202, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Sint Maarten
    {  202, 2045,   59489,  9, }, // America/St_Johns
    {  202, 5780,   59498,  8, }, // America/St_Kitts
    {  202, 5797,   59506,  8, }, // America/St_Lucia
    {  202,  855,   59514,  9, }, // America/St_Thomas
    {  202, 5830,   59523, 10, }, // America/St_Vincent
    {  202, 5440,    5906,  5, }, // Asia/Aqtau
    {  202, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  202, 2897,    4520,  9, }, // Pacific/Kanton
    {  203, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Slovenia
    {  203, 2045,   59489,  9, }, // America/St_Johns
    {  203, 5780,   59498,  8, }, // America/St_Kitts
    {  203, 5797,   59506,  8, }, // America/St_Lucia
    {  203,  855,   59514,  9, }, // America/St_Thomas
    {  203, 5830,   59523, 10, }, // America/St_Vincent
    {  203, 5440,    5906,  5, }, // Asia/Aqtau
    {  203, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  203, 2897,    4520,  9, }, // Pacific/Kanton
    {  204, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Solomon Islands
    {  204, 2045,   59489,  9, }, // America/St_Johns
    {  204, 5780,   59498,  8, }, // America/St_Kitts
    {  204, 5797,   59506,  8, }, // America/St_Lucia
    {  204,  855,   59514,  9, }, // America/St_Thomas
    {  204, 5830,   59523, 10, }, // America/St_Vincent
    {  204, 5440,    5906,  5, }, // Asia/Aqtau
    {  204, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  204, 2897,    4520,  9, }, // Pacific/Kanton
    {  205, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/South Africa
    {  205, 2045,   59489,  9, }, // America/St_Johns
    {  205, 5780,   59498,  8, }, // America/St_Kitts
    {  205, 5797,   59506,  8, }, // America/St_Lucia
    {  205,  855,   59514,  9, }, // America/St_Thomas
    {  205, 5830,   59523, 10, }, // America/St_Vincent
    {  205, 5440,    5906,  5, }, // Asia/Aqtau
    {  205, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  205, 2897,    4520,  9, }, // Pacific/Kanton
    {  206, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/South Sudan
    {  206, 2045,   59489,  9, }, // America/St_Johns
    {  206, 5780,   59498,  8, }, // America/St_Kitts
    {  206, 5797,   59506,  8, }, // America/St_Lucia
    {  206,  855,   59514,  9, }, // America/St_Thomas
    {  206, 5830,   59523, 10, }, // America/St_Vincent
    {  206, 5440,    5906,  5, }, // Asia/Aqtau
    {  206, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  206, 2897,    4520,  9, }, // Pacific/Kanton
    {  207, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Sudan
    {  207, 2045,   59489,  9, }, // America/St_Johns
    {  207, 5780,   59498,  8, }, // America/St_Kitts
    {  207, 5797,   59506,  8, }, // America/St_Lucia
    {  207,  855,   59514,  9, }, // America/St_Thomas
    {  207, 5830,   59523, 10, }, // America/St_Vincent
    {  207, 5440,    5906,  5, }, // Asia/Aqtau
    {  207, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  207, 2897,    4520,  9, }, // Pacific/Kanton
    {  208, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Sweden
    {  208, 2045,   59489,  9, }, // America/St_Johns
    {  208, 5780,   59498,  8, }, // America/St_Kitts
    {  208, 5797,   59506,  8, }, // America/St_Lucia
    {  208,  855,   59514,  9, }, // America/St_Thomas
    {  208, 5830,   59523, 10, }, // America/St_Vincent
    {  208, 5440,    5906,  5, }, // Asia/Aqtau
    {  208, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  208, 2897,    4520,  9, }, // Pacific/Kanton
    {  209, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Switzerland
    {  209, 2045,   59489,  9, }, // America/St_Johns
    {  209, 5780,   59498,  8, }, // America/St_Kitts
    {  209, 5797,   59506,  8, }, // America/St_Lucia
    {  209,  855,   59514,  9, }, // America/St_Thomas
    {  209, 5830,   59523, 10, }, // America/St_Vincent
    {  209, 5440,    5906,  5, }, // Asia/Aqtau
    {  209, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  209, 2897,    4520,  9, }, // Pacific/Kanton
    {  210, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Tanzania
    {  210, 2045,   59489,  9, }, // America/St_Johns
    {  210, 5780,   59498,  8, }, // America/St_Kitts
    {  210, 5797,   59506,  8, }, // America/St_Lucia
    {  210,  855,   59514,  9, }, // America/St_Thomas
    {  210, 5830,   59523, 10, }, // America/St_Vincent
    {  210, 5440,    5906,  5, }, // Asia/Aqtau
    {  210, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  210, 2897,    4520,  9, }, // Pacific/Kanton
    {  211, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Tokelau
    {  211, 2045,   59489,  9, }, // America/St_Johns
    {  211, 5780,   59498,  8, }, // America/St_Kitts
    {  211, 5797,   59506,  8, }, // America/St_Lucia
    {  211,  855,   59514,  9, }, // America/St_Thomas
    {  211, 5830,   59523, 10, }, // America/St_Vincent
    {  211, 5440,    5906,  5, }, // Asia/Aqtau
    {  211, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  211, 2897,    4520,  9, }, // Pacific/Kanton
    {  212, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Tonga
    {  212, 2045,   59489,  9, }, // America/St_Johns
    {  212, 5780,   59498,  8, }, // America/St_Kitts
    {  212, 5797,   59506,  8, }, // America/St_Lucia
    {  212,  855,   59514,  9, }, // America/St_Thomas
    {  212, 5830,   59523, 10, }, // America/St_Vincent
    {  212, 5440,    5906,  5, }, // Asia/Aqtau
    {  212, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  212, 2897,    4520,  9, }, // Pacific/Kanton
    {  213, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Trinidad and Tobago
    {  213, 2045,   59489,  9, }, // America/St_Johns
    {  213, 5780,   59498,  8, }, // America/St_Kitts
    {  213, 5797,   59506,  8, }, // America/St_Lucia
    {  213,  855,   59514,  9, }, // America/St_Thomas
    {  213, 5830,   59523, 10, }, // America/St_Vincent
    {  213, 5440,    5906,  5, }, // Asia/Aqtau
    {  213, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  213, 2897,    4520,  9, }, // Pacific/Kanton
    {  214, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Turks and Caicos Islands
    {  214, 2045,   59489,  9, }, // America/St_Johns
    {  214, 5780,   59498,  8, }, // America/St_Kitts
    {  214, 5797,   59506,  8, }, // America/St_Lucia
    {  214,  855,   59514,  9, }, // America/St_Thomas
    {  214, 5830,   59523, 10, }, // America/St_Vincent
    {  214, 5440,    5906,  5, }, // Asia/Aqtau
    {  214, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  214, 2897,    4520,  9, }, // Pacific/Kanton
    {  215, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Tuvalu
    {  215, 2045,   59489,  9, }, // America/St_Johns
    {  215, 5780,   59498,  8, }, // America/St_Kitts
    {  215, 5797,   59506,  8, }, // America/St_Lucia
    {  215,  855,   59514,  9, }, // America/St_Thomas
    {  215, 5830,   59523, 10, }, // America/St_Vincent
    {  215, 5440,    5906,  5, }, // Asia/Aqtau
    {  215, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  215, 2897,    4520,  9, }, // Pacific/Kanton
    {  216, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Uganda
    {  216, 2045,   59489,  9, }, // America/St_Johns
    {  216, 5780,   59498,  8, }, // America/St_Kitts
    {  216, 5797,   59506,  8, }, // America/St_Lucia
    {  216,  855,   59514,  9, }, // America/St_Thomas
    {  216, 5830,   59523, 10, }, // America/St_Vincent
    {  216, 5440,    5906,  5, }, // Asia/Aqtau
    {  216, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  216, 2897,    4520,  9, }, // Pacific/Kanton
    {  218, 7586,    3834, 14, }, // America/Bahia_Banderas English/Latin/United Kingdom
    {  218, 3770,    3861,  6, }, // America/Cancun
    {  218, 8183,    3899,  6, }, // America/Merida
    {  218, 8471,   59476, 13, }, // America/St_Barthelemy
    {  218, 2045,   59489,  9, }, // America/St_Johns
    {  218, 5780,   59498,  8, }, // America/St_Kitts
    {  218, 5797,   59506,  8, }, // America/St_Lucia
    {  218,  855,   59514,  9, }, // America/St_Thomas
    {  218, 5830,   59523, 10, }, // America/St_Vincent
    {  218, 5440,    5906,  5, }, // Asia/Aqtau
    {  218, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  218, 2897,    4520,  9, }, // Pacific/Kanton
    {  221, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Vanuatu
    {  221, 2045,   59489,  9, }, // America/St_Johns
    {  221, 5780,   59498,  8, }, // America/St_Kitts
    {  221, 5797,   59506,  8, }, // America/St_Lucia
    {  221,  855,   59514,  9, }, // America/St_Thomas
    {  221, 5830,   59523, 10, }, // America/St_Vincent
    {  221, 5440,    5906,  5, }, // Asia/Aqtau
    {  221, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  221, 2897,    4520,  9, }, // Pacific/Kanton
    {  222, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/world
    {  222, 2045,   59489,  9, }, // America/St_Johns
    {  222, 5780,   59498,  8, }, // America/St_Kitts
    {  222, 5797,   59506,  8, }, // America/St_Lucia
    {  222,  855,   59514,  9, }, // America/St_Thomas
    {  222, 5830,   59523, 10, }, // America/St_Vincent
    {  222, 5440,    5906,  5, }, // Asia/Aqtau
    {  222, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  222, 2897,    4520,  9, }, // Pacific/Kanton
    {  223, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Zambia
    {  223, 2045,   59489,  9, }, // America/St_Johns
    {  223, 5780,   59498,  8, }, // America/St_Kitts
    {  223, 5797,   59506,  8, }, // America/St_Lucia
    {  223,  855,   59514,  9, }, // America/St_Thomas
    {  223, 5830,   59523, 10, }, // America/St_Vincent
    {  223, 5440,    5906,  5, }, // Asia/Aqtau
    {  223, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  223, 2897,    4520,  9, }, // Pacific/Kanton
    {  224, 8471,   59476, 13, }, // America/St_Barthelemy English/Latin/Zimbabwe
    {  224, 2045,   59489,  9, }, // America/St_Johns
    {  224, 5780,   59498,  8, }, // America/St_Kitts
    {  224, 5797,   59506,  8, }, // America/St_Lucia
    {  224,  855,   59514,  9, }, // America/St_Thomas
    {  224, 5830,   59523, 10, }, // America/St_Vincent
    {  224, 5440,    5906,  5, }, // Asia/Aqtau
    {  224, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  224, 2897,    4520,  9, }, // Pacific/Kanton
    {  225, 1226,     397, 11, }, // Asia/Ho_Chi_Minh English/Shavian/United Kingdom
    {  226,   14,       0,  6, }, // Africa/Asmara Erzya/Cyrillic/Russia
    {  226, 4337,       6,  8, }, // Africa/Sao_Tome
    {  226, 4154,      14,  8, }, // America/Asuncion
    {  226,  237,      22,  8, }, // America/Atikokan
    {  226, 7586,      30, 17, }, // America/Bahia_Banderas
    {  226, 7662,   35011,  5, }, // America/Boise
    {  226, 3770,      47,  6, }, // America/Cancun
    {  226, 3251,   59542,  6, }, // America/Chicago
    {  226, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  226, 5640,      66,  7, }, // America/Curacao
    {  226,  805,    1201,  6, }, // America/Denver
    {  226, 5687,    1292,  7, }, // America/Grenada
    {  226,  481,      73, 13, }, // America/Indiana/Knox
    {  226, 7924,      86, 16, }, // America/Indiana/Marengo
    {  226, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  226, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  226, 8001,     139, 14, }, // America/Indiana/Vevay
    {  226, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  226, 8049,     171, 16, }, // America/Indiana/Winamac
    {  226, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  226, 5849,     207, 22, }, // America/Lower_Princes
    {  226, 8183,     229,  6, }, // America/Merida
    {  226, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  226, 8292,     255, 20, }, // America/North_Dakota/Center
    {  226, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  226,  393,     298,  4, }, // America/Nuuk
    {  226, 5413,    1783,  6, }, // America/Panama
    {  226, 6862,     302, 16, }, // America/Scoresbysund
    {  226, 8471,     318, 14, }, // America/St_Barthelemy
    {  226, 2045,     332, 10, }, // America/St_Johns
    {  226, 5780,     342,  9, }, // America/St_Kitts
    {  226, 5797,     351,  9, }, // America/St_Lucia
    {  226,  855,     360, 10, }, // America/St_Thomas
    {  226, 5830,     370, 11, }, // America/St_Vincent
    {  226, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  226, 1413,   59548, 11, }, // Arctic/Longyearbyen
    {  226, 3509,    2237,  6, }, // Asia/Baghdad
    {  226, 3555,    2250,  4, }, // Asia/Baku
    {  226, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  226, 2576,   50560,  7, }, // Asia/Hong_Kong
    {  226, 1152,     413,  9, }, // Asia/Kathmandu
    {  226,  992,     422,  7, }, // Asia/Kolkata
    {  226, 1178,     429,  5, }, // Asia/Macau
    {  226, 4131,   59559,  6, }, // Asia/Omsk
    {  226, 4310,    2623,  7, }, // Asia/Sakhalin
    {  226, 7208,    2630,  9, }, // Asia/Samarkand
    {  226, 3116,    2639,  4, }, // Asia/Seoul
    {  226, 1020,   27392,  6, }, // Asia/Shanghai
    {  226, 3100,   59565,  6, }, // Asia/Taipei
    {  226, 4683,    2676,  7, }, // Asia/Tashkent
    {  226, 2687,    2701,  5, }, // Asia/Tokyo
    {  226, 4474,   59571,  7, }, // Asia/Tomsk
    {  226, 1346,   59578, 10, }, // Asia/Ulaanbaatar
    {  226, 1202,     442,  6, }, // Asia/Yangon
    {  226, 5588,   33764,  7, }, // Atlantic/Bermuda
    {  226, 1379,     448,  5, }, // Atlantic/Faroe
    {  226, 8699,   35753,  7, }, // Atlantic/Madeira
    {  226, 2614,   50740,  9, }, // Atlantic/Reykjavik
    {  226, 6784,     453, 10, }, // Atlantic/St_Helena
    {  226, 6390,    3002,  9, }, // Europe/Amsterdam
    {  226, 6149,    3011,  7, }, // Europe/Andorra
    {  226, 6577,   59588,  5, }, // Europe/Athens
    {  226, 6285,    3030,  7, }, // Europe/Belgrade
    {  226, 4659,    3037,  6, }, // Europe/Berlin
    {  226, 6437,    3043, 10, }, // Europe/Bratislava
    {  226, 6178,   50822,  8, }, // Europe/Brussels
    {  226, 3830,    3061,  8, }, // Europe/Bucharest
    {  226, 3708,    3069,  8, }, // Europe/Budapest
    {  226, 6238,   50849, 10, }, // Europe/Copenhagen
    {  226, 2260,    3117,  6, }, // Europe/Dublin
    {  226, 6561,   50859,  9, }, // Europe/Helsinki
    {  226, 8924,    3178,  5, }, // Europe/Kirov
    {  226, 2419,    3183,  4, }, // Europe/Kyiv
    {  226, 3078,    3187,  8, }, // Europe/Lisbon
    {  226, 6455,   27891,  7, }, // Europe/Ljubljana
    {  226, 2393,    3204,  6, }, // Europe/London
    {  226, 6314,   27904, 10, }, // Europe/Luxembourg
    {  226, 4794,    3221,  6, }, // Europe/Madrid
    {  226, 6346,   27920,  6, }, // Europe/Malta
    {  226, 3595,   59593,  7, }, // Europe/Minsk
    {  226, 6359,    3246,  6, }, // Europe/Monaco
    {  226, 3443,   59600,  6, }, // Europe/Moscow
    {  226, 6407,    3258,  4, }, // Europe/Oslo
    {  226, 4186,    3262,  5, }, // Europe/Paris
    {  226, 6373,    3267,  9, }, // Europe/Podgorica
    {  226, 6224,    3276,  5, }, // Europe/Prague
    {  226, 8816,    3281,  4, }, // Europe/Riga
    {  226, 6273,    3285,  3, }, // Europe/Rome
    {  226, 4233,    3288,  6, }, // Europe/Samara
    {  226, 6419,    3294, 10, }, // Europe/San_Marino
    {  226, 6194,    3304,  7, }, // Europe/Sarajevo
    {  226, 4353,    3311,  7, }, // Europe/Saratov
    {  226, 8828,   50883, 11, }, // Europe/Simferopol
    {  226, 6332,   50894,  6, }, // Europe/Skopje
    {  226, 6548,   35907,  5, }, // Europe/Sofia
    {  226, 6472,   50900,  9, }, // Europe/Stockholm
    {  226, 8846,    3346,  6, }, // Europe/Tallinn
    {  226, 6135,    3352,  6, }, // Europe/Tirane
    {  226, 6301,    3367,  5, }, // Europe/Vaduz
    {  226, 6516,    3372,  7, }, // Europe/Vatican
    {  226, 6164,    3379,  4, }, // Europe/Vienna
    {  226, 8878,   50918,  7, }, // Europe/Vilnius
    {  226, 3055,    3399,  7, }, // Europe/Warsaw
    {  226, 6210,    3406,  6, }, // Europe/Zagreb
    {  226, 6489,   35931,  5, }, // Europe/Zurich
    {  226, 7174,     467,  7, }, // Indian/Reunion
    {  226, 3022,     474,  5, }, // Pacific/Chuuk
    {  226, 2961,     479,  7, }, // Pacific/Pohnpei
    {  227, 6751,   59606,  7, }, // Africa/Abidjan Esperanto/Latin/world
    {  227, 6723,   59613,  5, }, // Africa/Accra
    {  227, 4961,   59618, 10, }, // Africa/Addis_Ababa
    {  227, 7383,   59628,  6, }, // Africa/Algiers
    {  227,   14,   59634,  6, }, // Africa/Asmara
    {  227, 5159,   59640,  5, }, // Africa/Bangui
    {  227, 6709,   59645,  7, }, // Africa/Banjul
    {  227, 7398,   59652,  6, }, // Africa/Bissau
    {  227, 5189,   59658,  9, }, // Africa/Brazzaville
    {  227, 4824,   59667,  9, }, // Africa/Bujumbura
    {  227, 2242,   24913,  5, }, // Africa/Cairo
    {  227, 4019,   59676, 10, }, // Africa/Casablanca
    {  227, 7412,   59686,  5, }, // Africa/Ceuta
    {  227, 6736,   59691,  8, }, // Africa/Conakry
    {  227, 6803,   59699,  6, }, // Africa/Dakar
    {  227, 5032,   59705, 10, }, // Africa/Dar_es_Salaam
    {  227, 4945,   59715,  7, }, // Africa/Djibouti
    {  227, 5145,   59722,  6, }, // Africa/Douala
    {  227, 5068,   59728,  5, }, // Africa/El_Aaiun
    {  227, 6816,   59733,  8, }, // Africa/Freetown
    {  227, 4808,   59741,  8, }, // Africa/Gaborone
    {  227, 4917,   59749,  7, }, // Africa/Harare
    {  227, 4381,   59756, 12, }, // Africa/Johannesburg
    {  227, 4401,   59768,  5, }, // Africa/Juba
    {  227, 5053,   59773,  7, }, // Africa/Kampala
    {  227, 4426,   59780,  7, }, // Africa/Khartoum
    {  227, 4875,   59787,  6, }, // Africa/Kigali
    {  227, 5208,   59793,  7, }, // Africa/Kinshasa
    {  227, 4646,   59800,  6, }, // Africa/Lagos
    {  227, 5238,   59806,  9, }, // Africa/Libreville
    {  227, 6832,   59815,  5, }, // Africa/Lome
    {  227, 5113,   59820,  6, }, // Africa/Luanda
    {  227, 4841,   59826,  9, }, // Africa/Lubumbashi
    {  227, 4903,   59835,  6, }, // Africa/Lusaka
    {  227, 5099,   59841,  7, }, // Africa/Maseru
    {  227, 5084,   59848,  7, }, // Africa/Mbabane
    {  227, 5015,   59855,  8, }, // Africa/Mogadishu
    {  227, 6997,   59863,  8, }, // Africa/Monrovia
    {  227, 3755,   59871,  8, }, // Africa/Nairobi
    {  227, 5173,   59879,  8, }, // Africa/Ndjamena
    {  227, 5256,   59887,  7, }, // Africa/Niamey
    {  227, 6766,   59894,  8, }, // Africa/Nouakchott
    {  227, 6690,   59902,  9, }, // Africa/Ouagadougou
    {  227, 4337,   59911,  8, }, // Africa/Sao_Tome
    {  227, 2732,   59919,  7, }, // Africa/Tripoli
    {  227, 6503,   59926,  6, }, // Africa/Tunis
    {  227, 4054,   59932,  8, }, // Africa/Windhoek
    {  227, 5524,   59940,  7, }, // America/Anguilla
    {  227, 5541,   59947,  7, }, // America/Antigua
    {  227, 4456,   24992,  9, }, // America/Araguaina
    {  227,  166,   59954,  7, }, // America/Argentina/Buenos_Aires
    {  227,  270,   59961,  7, }, // America/Argentina/Cordoba
    {  227,  441,   59968, 21, }, // America/Argentina/Jujuy
    {  227, 7452,   42641, 12, }, // America/Argentina/Rio_Gallegos
    {  227, 7483,   59989,  9, }, // America/Argentina/Salta
    {  227, 7534,   59998, 21, }, // America/Argentina/Tucuman
    {  227, 5557,   60019,  5, }, // America/Aruba
    {  227, 4154,   60024,  9, }, // America/Asuncion
    {  227,  237,      22,  8, }, // America/Atikokan
    {  227, 7586,      30, 17, }, // America/Bahia_Banderas
    {  227, 5571,   60033,  7, }, // America/Barbados
    {  227, 7609,   60040,  6, }, // America/Belem
    {  227, 5309,   60046,  6, }, // America/Belize
    {  227, 4263,   60052,  6, }, // America/Bogota
    {  227, 3770,   60058,  7, }, // America/Cancun
    {  227, 4596,   60065,  8, }, // America/Caracas
    {  227, 4247,   60073,  6, }, // America/Cayenne
    {  227, 5398,   60079,  8, }, // America/Cayman
    {  227, 3251,   60087,  6, }, // America/Chicago
    {  227, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  227, 5324,   60093,  9, }, // America/Costa_Rica
    {  227, 3693,   25007,  6, }, // America/Cuiaba
    {  227, 5640,   60102,  7, }, // America/Curacao
    {  227,  805,   60109,  7, }, // America/Denver
    {  227, 3351,   60116,  8, }, // America/Detroit
    {  227, 5656,   60124,  8, }, // America/Dominica
    {  227,  893,   60132,  9, }, // America/Edmonton
    {  227, 7832,   25013,  8, }, // America/Eirunepe
    {  227, 5343,   60141,  9, }, // America/El_Salvador
    {  227, 7869,   60150,  9, }, // America/Fortaleza
    {  227, 4514,   60159, 12, }, // America/Grand_Turk
    {  227, 5687,   60171,  7, }, // America/Grenada
    {  227, 5703,   60178,  9, }, // America/Guadeloupe
    {  227, 3662,   60187,  9, }, // America/Guatemala
    {  227, 6895,   60196,  6, }, // America/Guyana
    {  227, 1923,   60202,  9, }, // America/Halifax
    {  227, 2221,   60211,  6, }, // America/Havana
    {  227,  348,   60217, 21, }, // America/Indiana/Indianapolis
    {  227,  481,   60238, 14, }, // America/Indiana/Knox
    {  227, 7924,   60252, 17, }, // America/Indiana/Marengo
    {  227, 7948,   60269, 20, }, // America/Indiana/Petersburg
    {  227, 7975,   60289, 19, }, // America/Indiana/Tell_City
    {  227, 8001,   60308, 15, }, // America/Indiana/Vevay
    {  227, 8023,   60323, 19, }, // America/Indiana/Vincennes
    {  227, 8049,   60342, 17, }, // America/Indiana/Winamac
    {  227,  660,   60359,  8, }, // America/Iqaluit
    {  227, 2665,   60367,  7, }, // America/Jamaica
    {  227, 8088,   60374, 20, }, // America/Kentucky/Monticello
    {  227, 4278,   60394,  7, }, // America/La_Paz
    {  227, 7132,   60401,  4, }, // America/Lima
    {  227, 3390,   60405, 11, }, // America/Los_Angeles
    {  227, 5849,     207, 22, }, // America/Lower_Princes
    {  227, 8116,   25021,  6, }, // America/Maceio
    {  227, 8131,   60416,  7, }, // America/Managua
    {  227, 1892,   60423,  7, }, // America/Manaus
    {  227, 5814,   60430,  8, }, // America/Marigot
    {  227, 5722,   60438,  9, }, // America/Martinique
    {  227, 2779,   25036,  8, }, // America/Mazatlan
    {  227, 8183,   60447,  6, }, // America/Merida
    {  227, 2811,   60453, 10, }, // America/Mexico_City
    {  227, 4293,   60463,  8, }, // America/Miquelon
    {  227, 8217,   60471,  8, }, // America/Moncton
    {  227, 8233,   60479,  9, }, // America/Monterrey
    {  227, 5741,   60488,  9, }, // America/Montserrat
    {  227, 5383,   60497,  6, }, // America/Nassau
    {  227, 3294,   60503,  8, }, // America/New_York
    {  227, 1834,   60511, 19, }, // America/Noronha
    {  227, 8264,   60530, 20, }, // America/North_Dakota/Beulah
    {  227, 8292,   60550, 20, }, // America/North_Dakota/Center
    {  227, 8320,   60570, 23, }, // America/North_Dakota/New_Salem
    {  227,  393,   60593,  4, }, // America/Nuuk
    {  227, 5413,   60597,  6, }, // America/Panama
    {  227, 3224,   60603,  7, }, // America/Phoenix
    {  227, 3847,   60610, 11, }, // America/Port-au-Prince
    {  227, 5871,   60621, 10, }, // America/Port_of_Spain
    {  227, 5760,   60631, 10, }, // America/Puerto_Rico
    {  227, 8440,   25137,  8, }, // America/Santarem
    {  227, 6044,   60641, 14, }, // America/Santo_Domingo
    {  227, 1862,   60655,  9, }, // America/Sao_Paulo
    {  227, 6862,     302, 16, }, // America/Scoresbysund
    {  227, 8471,   60664, 17, }, // America/St_Barthelemy
    {  227, 2045,     332, 10, }, // America/St_Johns
    {  227, 5780,   60681, 17, }, // America/St_Kitts
    {  227, 5797,   60698, 12, }, // America/St_Lucia
    {  227,  855,   60710, 13, }, // America/St_Thomas
    {  227, 5830,   60723, 15, }, // America/St_Vincent
    {  227, 5363,   60738, 11, }, // America/Tegucigalpa
    {  227, 5673,   25229,  7, }, // America/Thule
    {  227, 2077,   60749,  9, }, // America/Vancouver
    {  227,  734,   60758,  8, }, // America/Winnipeg
    {  227, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  227, 8531,   60766, 14, }, // Antarctica/Macquarie
    {  227, 1413,   60780, 11, }, // Arctic/Longyearbyen
    {  227, 5487,   60791,  5, }, // Asia/Aden
    {  227, 5297,   60796,  6, }, // Asia/Almaty
    {  227, 3870,   60802,  5, }, // Asia/Amman
    {  227, 5428,    5900,  6, }, // Asia/Anadyr
    {  227, 5440,    5906,  5, }, // Asia/Aqtau
    {  227, 5270,    5911,  6, }, // Asia/Aqtobe
    {  227,  964,   60807,  8, }, // Asia/Ashgabat
    {  227, 8587,    5917,  6, }, // Asia/Atyrau
    {  227, 3509,   60815,  7, }, // Asia/Baghdad
    {  227, 5451,   60822,  7, }, // Asia/Bahrain
    {  227, 3555,   60829,  5, }, // Asia/Baku
    {  227, 4368,   60834,  7, }, // Asia/Bangkok
    {  227, 3988,   60841,  7, }, // Asia/Beirut
    {  227, 3680,   60848,  7, }, // Asia/Bishkek
    {  227, 5906,   60855,  7, }, // Asia/Brunei
    {  227, 4503,   60862,  4, }, // Asia/Chita
    {  227, 5963,   60866,  9, }, // Asia/Choibalsan
    {  227, 4442,   25264,  7, }, // Asia/Damascus
    {  227, 6125,   60875,  4, }, // Asia/Dili
    {  227, 3498,   60879,  9, }, // Asia/Dubai
    {  227, 6092,   60888,  8, }, // Asia/Dushanbe
    {  227, 8614,   60896,  9, }, // Asia/Gaza
    {  227, 4697,   60905,  7, }, // Asia/Hebron
    {  227, 1226,   60912,  8, }, // Asia/Ho_Chi_Minh
    {  227, 2576,   60920,  8, }, // Asia/Hong_Kong
    {  227, 4673,   60928,  4, }, // Asia/Hovd
    {  227, 4086,   60932,  8, }, // Asia/Irkutsk
    {  227, 6969,   60940,  7, }, // Asia/Jakarta
    {  227, 1257,   60947, 10, }, // Asia/Jerusalem
    {  227, 4218,   60957,  8, }, // Asia/Kamchatka
    {  227, 1152,     413,  9, }, // Asia/Kathmandu
    {  227, 8624,   60965,  7, }, // Asia/Khandyga
    {  227,  992,     422,  7, }, // Asia/Kolkata
    {  227, 4099,   60972, 12, }, // Asia/Krasnoyarsk
    {  227, 4761,   60984, 13, }, // Asia/Kuala_Lumpur
    {  227, 5464,   60997,  7, }, // Asia/Kuwait
    {  227, 1178,    6064,  5, }, // Asia/Macau
    {  227, 7145,   61004,  6, }, // Asia/Manila
    {  227, 6883,   61010,  7, }, // Asia/Muscat
    {  227, 2446,   61017,  7, }, // Asia/Nicosia
    {  227, 8638,   61024, 12, }, // Asia/Novokuznetsk
    {  227, 4037,   61036, 12, }, // Asia/Novosibirsk
    {  227, 4131,   61048,  5, }, // Asia/Omsk
    {  227, 7108,   61053,  5, }, // Asia/Oral
    {  227, 6924,   61058,  8, }, // Asia/Phnom_Penh
    {  227, 4116,   61066, 10, }, // Asia/Pyongyang
    {  227, 5476,   61076,  6, }, // Asia/Qatar
    {  227, 8671,    6103,  8, }, // Asia/Qostanay
    {  227, 4171,   61082,  9, }, // Asia/Qyzylorda
    {  227, 3486,   61091,  5, }, // Asia/Riyadh
    {  227, 4310,   61096,  8, }, // Asia/Sakhalin
    {  227, 7208,   61104, 10, }, // Asia/Samarkand
    {  227, 3116,   61114,  5, }, // Asia/Seoul
    {  227, 1020,   61119,  7, }, // Asia/Shanghai
    {  227, 3137,   61126,  9, }, // Asia/Singapore
    {  227, 4199,    6150, 13, }, // Asia/Srednekolymsk
    {  227, 3100,   61135,  6, }, // Asia/Taipei
    {  227, 4683,   61141,  8, }, // Asia/Tashkent
    {  227, 3817,   61149,  7, }, // Asia/Tbilisi
    {  227, 2687,    4257,  5, }, // Asia/Tokyo
    {  227, 1346,   61156, 10, }, // Asia/Ulaanbaatar
    {  227, 1126,   61166,  7, }, // Asia/Urumqi
    {  227, 8685,   61173,  7, }, // Asia/Ust-Nera
    {  227, 6940,   61180,  9, }, // Asia/Vientiane
    {  227, 4612,   61189, 12, }, // Asia/Vladivostok
    {  227, 4730,   61201,  8, }, // Asia/Yakutsk
    {  227, 1202,   61209,  7, }, // Asia/Yangon
    {  227, 3785,   61216, 14, }, // Asia/Yekaterinburg
    {  227, 3649,   61230,  7, }, // Asia/Yerevan
    {  227, 3565,   61237,  6, }, // Atlantic/Azores
    {  227, 5588,   61243,  8, }, // Atlantic/Bermuda
    {  227, 6591,   61251,  8, }, // Atlantic/Canary
    {  227, 3629,   61259,  9, }, // Atlantic/Cape_Verde
    {  227, 1379,   61268,  6, }, // Atlantic/Faroe
    {  227, 8699,   61274,  7, }, // Atlantic/Madeira
    {  227, 2614,   61281, 10, }, // Atlantic/Reykjavik
    {  227, 7235,   61291, 11, }, // Atlantic/South_Georgia
    {  227, 6784,   61302, 13, }, // Atlantic/St_Helena
    {  227, 6607,   61315,  8, }, // Atlantic/Stanley
    {  227, 1654,   61323,  8, }, // Australia/Adelaide
    {  227, 1619,   61331,  8, }, // Australia/Brisbane
    {  227, 1567,   61339,  7, }, // Australia/Darwin
    {  227, 1500,   61346,  7, }, // Australia/Hobart
    {  227, 1711,   61353,  8, }, // Australia/Melbourne
    {  227, 1746,   61361,  5, }, // Australia/Perth
    {  227, 1447,   61366,  7, }, // Australia/Sydney
    {  227, 6390,   61373, 10, }, // Europe/Amsterdam
    {  227, 6149,   61383,  6, }, // Europe/Andorra
    {  227, 3522,   61389,  9, }, // Europe/Astrakhan
    {  227, 6577,   61398,  5, }, // Europe/Athens
    {  227, 6285,   61403,  8, }, // Europe/Belgrade
    {  227, 4659,   61411,  7, }, // Europe/Berlin
    {  227, 6437,   61418, 10, }, // Europe/Bratislava
    {  227, 6178,   61428,  7, }, // Europe/Brussels
    {  227, 3830,   61435,  9, }, // Europe/Bucharest
    {  227, 3708,   61444,  9, }, // Europe/Budapest
    {  227, 8751,   61453, 21, }, // Europe/Busingen
    {  227, 2475,   61474,  8, }, // Europe/Chisinau
    {  227, 6238,   61482,  9, }, // Europe/Copenhagen
    {  227, 2260,   61491,  7, }, // Europe/Dublin
    {  227, 6256,   61498, 10, }, // Europe/Gibraltar
    {  227, 8767,   61508,  9, }, // Europe/Guernsey
    {  227, 6561,   61517,  8, }, // Europe/Helsinki
    {  227, 8783,   61525, 10, }, // Europe/Isle_of_Man
    {  227, 1097,   61535,  9, }, // Europe/Istanbul
    {  227, 8802,   61544,  7, }, // Europe/Jersey
    {  227, 3881,   61551, 12, }, // Europe/Kaliningrad
    {  227, 2419,   61563,  5, }, // Europe/Kyiv
    {  227, 3078,   61568,  7, }, // Europe/Lisbon
    {  227, 6455,   61575,  9, }, // Europe/Ljubljana
    {  227, 2393,   61584,  7, }, // Europe/London
    {  227, 6314,   61591, 11, }, // Europe/Luxembourg
    {  227, 4794,   61602,  7, }, // Europe/Madrid
    {  227, 6346,   61609,  5, }, // Europe/Malta
    {  227, 3595,   61614,  6, }, // Europe/Minsk
    {  227, 6359,    6525,  6, }, // Europe/Monaco
    {  227, 3443,   61620,  6, }, // Europe/Moscow
    {  227, 4186,   61626,  6, }, // Europe/Paris
    {  227, 6373,   61632,  9, }, // Europe/Podgorica
    {  227, 6224,   61641,  5, }, // Europe/Prague
    {  227, 8816,   61646,  4, }, // Europe/Riga
    {  227, 6273,   61650,  4, }, // Europe/Rome
    {  227, 6419,   61654,  9, }, // Europe/San_Marino
    {  227, 8828,   61663, 11, }, // Europe/Simferopol
    {  227, 6332,   61674,  6, }, // Europe/Skopje
    {  227, 6548,   61680,  5, }, // Europe/Sofia
    {  227, 6472,   61685,  9, }, // Europe/Stockholm
    {  227, 8846,   61694,  6, }, // Europe/Tallinn
    {  227, 6135,   61700,  6, }, // Europe/Tirane
    {  227, 8861,   61706, 10, }, // Europe/Ulyanovsk
    {  227, 6301,   61716,  6, }, // Europe/Vaduz
    {  227, 6516,   25551,  8, }, // Europe/Vatican
    {  227, 6164,   61722,  5, }, // Europe/Vienna
    {  227, 8878,   61727,  5, }, // Europe/Vilnius
    {  227, 4629,   61732, 10, }, // Europe/Volgograd
    {  227, 3055,   61742,  8, }, // Europe/Warsaw
    {  227, 6210,   61750,  7, }, // Europe/Zagreb
    {  227, 6489,   61757,  6, }, // Europe/Zurich
    {  227, 6910,   61763,  7, }, // Indian/Chagos
    {  227, 5979,   61770, 15, }, // Indian/Christmas
    {  227, 5996,   61785, 12, }, // Indian/Cocos
    {  227, 4931,   61797,  7, }, // Indian/Comoro
    {  227, 6624,   61804, 10, }, // Indian/Kerguelen
    {  227, 7223,   25565,  4, }, // Indian/Mahe
    {  227, 3971,   61814,  8, }, // Indian/Mauritius
    {  227, 5000,   61822,  6, }, // Indian/Mayotte
    {  227, 7174,   61828,  6, }, // Indian/Reunion
    {  227, 2863,   61834,  7, }, // Pacific/Chatham
    {  227, 3022,     474,  5, }, // Pacific/Chuuk
    {  227, 2201,   61841, 10, }, // Pacific/Easter
    {  227, 6641,   61851,  9, }, // Pacific/Galapagos
    {  227, 4070,   61860, 13, }, // Pacific/Norfolk
    {  227, 2961,     479,  7, }, // Pacific/Pohnpei
    {  228, 4961,    3793, 11, }, // Africa/Addis_Ababa Estonian/Latin/Estonia
    {  228, 7383,   61873,  6, }, // Africa/Algiers
    {  228,   14,       0,  6, }, // Africa/Asmara
    {  228, 2242,   24913,  5, }, // Africa/Cairo
    {  228, 4426,   61879,  6, }, // Africa/Khartoum
    {  228, 5173,   42632,  9, }, // Africa/Ndjamena
    {  228, 4337,       6,  8, }, // Africa/Sao_Tome
    {  228, 4456,   24992,  9, }, // America/Araguaina
    {  228,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  228, 7452,   42641, 12, }, // America/Argentina/Rio_Gallegos
    {  228, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  228, 4154,      14,  8, }, // America/Asuncion
    {  228,  237,      22,  8, }, // America/Atikokan
    {  228, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  228, 7609,   34376,  5, }, // America/Belem
    {  228, 4263,   18898,  6, }, // America/Bogota
    {  228, 3770,      47,  6, }, // America/Cancun
    {  228, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  228, 3693,   25007,  6, }, // America/Cuiaba
    {  228, 5640,      66,  7, }, // America/Curacao
    {  228, 7832,   25013,  8, }, // America/Eirunepe
    {  228, 2221,   61885,  7, }, // America/Havana
    {  228,  481,      73, 13, }, // America/Indiana/Knox
    {  228, 7924,      86, 16, }, // America/Indiana/Marengo
    {  228, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  228, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  228, 8001,     139, 14, }, // America/Indiana/Vevay
    {  228, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  228, 8049,     171, 16, }, // America/Indiana/Winamac
    {  228, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  228, 5849,     207, 22, }, // America/Lower_Princes
    {  228, 8116,   25021,  6, }, // America/Maceio
    {  228, 2779,   25036,  8, }, // America/Mazatlan
    {  228, 8183,     229,  6, }, // America/Merida
    {  228, 2811,   56121,  6, }, // America/Mexico_City
    {  228, 8264,   61892, 20, }, // America/North_Dakota/Beulah
    {  228, 8292,   61912, 20, }, // America/North_Dakota/Center
    {  228, 8320,   61932, 23, }, // America/North_Dakota/New_Salem
    {  228,  393,     298,  4, }, // America/Nuuk
    {  228, 8440,   25137,  8, }, // America/Santarem
    {  228, 1862,   25145,  9, }, // America/Sao_Paulo
    {  228, 6862,     302, 16, }, // America/Scoresbysund
    {  228, 8471,   25154, 16, }, // America/St_Barthelemy
    {  228, 2045,   25170, 12, }, // America/St_Johns
    {  228, 5780,   25182, 11, }, // America/St_Kitts
    {  228, 5797,   34444, 11, }, // America/St_Lucia
    {  228,  855,   42797, 12, }, // America/St_Thomas
    {  228, 5830,   25216, 13, }, // America/St_Vincent
    {  228, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  228, 5297,   61955,  6, }, // Asia/Almaty
    {  228, 5428,   61961,  6, }, // Asia/Anadyr
    {  228, 5440,    5906,  5, }, // Asia/Aqtau
    {  228, 5270,   61967,  6, }, // Asia/Aqtobe
    {  228,  964,   42826,  7, }, // Asia/Ashgabat
    {  228, 8587,   61973,  6, }, // Asia/Atyrau
    {  228, 3509,    4063,  6, }, // Asia/Baghdad
    {  228, 5451,    4069,  7, }, // Asia/Bahrain
    {  228, 3555,   61979,  5, }, // Asia/Baku
    {  228, 3680,   31917,  6, }, // Asia/Bishkek
    {  228, 4503,   61984,  5, }, // Asia/Chita
    {  228, 5963,   61989, 10, }, // Asia/Choibalsan
    {  228, 4442,    4102,  8, }, // Asia/Damascus
    {  228, 6092,   31939,  7, }, // Asia/Dushanbe
    {  228, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  228, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  228, 1257,   61999, 11, }, // Asia/Jerusalem
    {  228, 4218,   62010,  9, }, // Asia/Kamchatka
    {  228, 1152,    6018,  8, }, // Asia/Kathmandu
    {  228, 8624,   62019,  7, }, // Asia/Khandyga
    {  228,  992,     422,  7, }, // Asia/Kolkata
    {  228, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  228, 5464,   62026,  6, }, // Asia/Kuwait
    {  228, 1178,    4222,  5, }, // Asia/Macau
    {  228, 6883,   62032,  6, }, // Asia/Muscat
    {  228, 2446,   25331,  7, }, // Asia/Nicosia
    {  228, 5476,    4233,  5, }, // Asia/Qatar
    {  228, 8671,    6103,  8, }, // Asia/Qostanay
    {  228, 4171,   62038,  9, }, // Asia/Qyzylorda
    {  228, 3486,   62047,  8, }, // Asia/Riyadh
    {  228, 4310,   62055,  8, }, // Asia/Sakhalin
    {  228, 3116,   56625,  4, }, // Asia/Seoul
    {  228, 3137,   19291,  8, }, // Asia/Singapore
    {  228, 4199,   62063, 13, }, // Asia/Srednekolymsk
    {  228, 4683,   32060,  7, }, // Asia/Tashkent
    {  228, 3817,   62076,  8, }, // Asia/Tbilisi
    {  228, 2638,    6175,  7, }, // Asia/Tehran
    {  228, 2687,   62084,  5, }, // Asia/Tokyo
    {  228, 1126,   25383,  6, }, // Asia/Urumqi
    {  228, 4730,    6206,  7, }, // Asia/Yakutsk
    {  228, 1202,     442,  6, }, // Asia/Yangon
    {  228, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  228, 3649,    6231,  7, }, // Asia/Yerevan
    {  228, 3565,   62089,  8, }, // Atlantic/Azores
    {  228, 6591,   62097, 14, }, // Atlantic/Canary
    {  228, 3629,   62111, 15, }, // Atlantic/Cape_Verde
    {  228, 1379,   62126, 12, }, // Atlantic/Faroe
    {  228, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  228, 7235,   62138, 13, }, // Atlantic/South_Georgia
    {  228, 6784,   34678, 12, }, // Atlantic/St_Helena
    {  228, 3522,   32115,  8, }, // Europe/Astrakhan
    {  228, 6577,   62151,  6, }, // Europe/Athens
    {  228, 6285,   25465,  7, }, // Europe/Belgrade
    {  228, 4659,   62157,  7, }, // Europe/Berlin
    {  228, 6178,   21176,  7, }, // Europe/Brussels
    {  228, 3830,   25479,  8, }, // Europe/Bucharest
    {  228, 8751,   25487,  8, }, // Europe/Busingen
    {  228, 2475,   62164,  8, }, // Europe/Chisinau
    {  228, 6238,   62172, 11, }, // Europe/Copenhagen
    {  228, 6561,   62183,  8, }, // Europe/Helsinki
    {  228, 8783,   62191,  9, }, // Europe/Isle_of_Man
    {  228, 1097,   21235,  8, }, // Europe/Istanbul
    {  228, 2419,   62200,  5, }, // Europe/Kyiv
    {  228, 3078,    4400,  8, }, // Europe/Lisbon
    {  228, 6531,   62205, 13, }, // Europe/Mariehamn
    {  228, 3443,   21290,  6, }, // Europe/Moscow
    {  228, 4186,   62218,  6, }, // Europe/Paris
    {  228, 6224,   34765,  5, }, // Europe/Prague
    {  228, 8816,   62224,  4, }, // Europe/Riga
    {  228, 6273,   62228,  5, }, // Europe/Rome
    {  228, 6135,   19608,  6, }, // Europe/Tirane
    {  228, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  228, 6516,    6614,  7, }, // Europe/Vatican
    {  228, 6164,   62233,  4, }, // Europe/Vienna
    {  228, 3055,   62237,  8, }, // Europe/Warsaw
    {  228, 6489,    4478,  6, }, // Europe/Zurich
    {  228, 5979,   62245,  9, }, // Indian/Christmas
    {  228, 5996,   62254, 12, }, // Indian/Cocos
    {  228, 7013,   62266,  9, }, // Indian/Maldives
    {  228, 7174,     467,  7, }, // Indian/Reunion
    {  228, 3022,     474,  5, }, // Pacific/Chuuk
    {  228, 2201,   62275, 13, }, // Pacific/Easter
    {  228, 3804,   32307,  5, }, // Pacific/Fiji
    {  228, 6641,   19672,  9, }, // Pacific/Galapagos
    {  228, 2929,    4512,  8, }, // Pacific/Honolulu
    {  228, 2897,    4520,  9, }, // Pacific/Kanton
    {  228, 3953,   62288, 12, }, // Pacific/Marquesas
    {  228, 7118,   62300,  5, }, // Pacific/Palau
    {  228, 2961,     479,  7, }, // Pacific/Pohnpei
    {  229,   14,       0,  6, }, // Africa/Asmara Ewe/Latin/Ghana
    {  229, 4337,       6,  8, }, // Africa/Sao_Tome
    {  229, 4154,      14,  8, }, // America/Asuncion
    {  229,  237,      22,  8, }, // America/Atikokan
    {  229, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  229, 3770,    3861,  6, }, // America/Cancun
    {  229, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  229, 5640,      66,  7, }, // America/Curacao
    {  229,  481,      73, 13, }, // America/Indiana/Knox
    {  229, 7924,      86, 16, }, // America/Indiana/Marengo
    {  229, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  229, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  229, 8001,     139, 14, }, // America/Indiana/Vevay
    {  229, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  229, 8049,     171, 16, }, // America/Indiana/Winamac
    {  229, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  229, 5849,     207, 22, }, // America/Lower_Princes
    {  229, 8183,    3899,  6, }, // America/Merida
    {  229, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  229, 8292,     255, 20, }, // America/North_Dakota/Center
    {  229, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  229,  393,     298,  4, }, // America/Nuuk
    {  229, 6862,     302, 16, }, // America/Scoresbysund
    {  229, 8471,     318, 14, }, // America/St_Barthelemy
    {  229, 2045,     332, 10, }, // America/St_Johns
    {  229, 5780,     342,  9, }, // America/St_Kitts
    {  229, 5797,     351,  9, }, // America/St_Lucia
    {  229,  855,     360, 10, }, // America/St_Thomas
    {  229, 5830,     370, 11, }, // America/St_Vincent
    {  229, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  229, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  229, 1152,   62305, 24, }, // Asia/Kathmandu
    {  229,  992,     422,  7, }, // Asia/Kolkata
    {  229, 1178,    4222,  5, }, // Asia/Macau
    {  229, 1202,     442,  6, }, // Asia/Yangon
    {  229, 1379,     448,  5, }, // Atlantic/Faroe
    {  229, 6784,     453, 10, }, // Atlantic/St_Helena
    {  229, 2419,    6486,  4, }, // Europe/Kyiv
    {  229, 7174,     467,  7, }, // Indian/Reunion
    {  229, 3022,     474,  5, }, // Pacific/Chuuk
    {  229, 2929,    4512,  8, }, // Pacific/Honolulu
    {  229, 2897,    4520,  9, }, // Pacific/Kanton
    {  229, 2961,     479,  7, }, // Pacific/Pohnpei
    {  231,   14,       0,  6, }, // Africa/Asmara Ewondo/Latin/Cameroon
    {  231, 4337,       6,  8, }, // Africa/Sao_Tome
    {  231, 4154,      14,  8, }, // America/Asuncion
    {  231,  237,      22,  8, }, // America/Atikokan
    {  231, 7586,      30, 17, }, // America/Bahia_Banderas
    {  231, 3770,      47,  6, }, // America/Cancun
    {  231, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  231, 5640,      66,  7, }, // America/Curacao
    {  231,  481,      73, 13, }, // America/Indiana/Knox
    {  231, 7924,      86, 16, }, // America/Indiana/Marengo
    {  231, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  231, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  231, 8001,     139, 14, }, // America/Indiana/Vevay
    {  231, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  231, 8049,     171, 16, }, // America/Indiana/Winamac
    {  231, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  231, 5849,     207, 22, }, // America/Lower_Princes
    {  231, 8183,     229,  6, }, // America/Merida
    {  231, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  231, 8292,     255, 20, }, // America/North_Dakota/Center
    {  231, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  231,  393,     298,  4, }, // America/Nuuk
    {  231, 6862,     302, 16, }, // America/Scoresbysund
    {  231, 8471,     318, 14, }, // America/St_Barthelemy
    {  231, 2045,     332, 10, }, // America/St_Johns
    {  231, 5780,     342,  9, }, // America/St_Kitts
    {  231, 5797,     351,  9, }, // America/St_Lucia
    {  231,  855,     360, 10, }, // America/St_Thomas
    {  231, 5830,     370, 11, }, // America/St_Vincent
    {  231, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  231, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  231, 1152,     413,  9, }, // Asia/Kathmandu
    {  231,  992,     422,  7, }, // Asia/Kolkata
    {  231, 1178,     429,  5, }, // Asia/Macau
    {  231, 1202,     442,  6, }, // Asia/Yangon
    {  231, 1379,     448,  5, }, // Atlantic/Faroe
    {  231, 6784,     453, 10, }, // Atlantic/St_Helena
    {  231, 2419,     463,  4, }, // Europe/Kyiv
    {  231, 7174,     467,  7, }, // Indian/Reunion
    {  231, 3022,     474,  5, }, // Pacific/Chuuk
    {  231, 2961,     479,  7, }, // Pacific/Pohnpei
    {  232,   14,       0,  6, }, // Africa/Asmara Faroese/Latin/Faroe Islands
    {  232, 4945,   24925,  7, }, // Africa/Djibouti
    {  232, 4337,       6,  8, }, // Africa/Sao_Tome
    {  232, 4154,      14,  8, }, // America/Asuncion
    {  232,  237,      22,  8, }, // America/Atikokan
    {  232, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  232, 5309,   62329,  5, }, // America/Belize
    {  232, 3770,    3861,  6, }, // America/Cancun
    {  232, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  232, 5324,   20026, 10, }, // America/Costa_Rica
    {  232, 5640,      66,  7, }, // America/Curacao
    {  232,  481,      73, 13, }, // America/Indiana/Knox
    {  232, 7924,      86, 16, }, // America/Indiana/Marengo
    {  232, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  232, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  232, 8001,     139, 14, }, // America/Indiana/Vevay
    {  232, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  232, 8049,     171, 16, }, // America/Indiana/Winamac
    {  232, 2665,    3892,  7, }, // America/Jamaica
    {  232, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  232, 5849,     207, 22, }, // America/Lower_Princes
    {  232, 8183,    3899,  6, }, // America/Merida
    {  232, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  232, 8292,     255, 20, }, // America/North_Dakota/Center
    {  232, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  232,  393,     298,  4, }, // America/Nuuk
    {  232, 5760,   20463, 11, }, // America/Puerto_Rico
    {  232, 6862,     302, 16, }, // America/Scoresbysund
    {  232, 8471,     318, 14, }, // America/St_Barthelemy
    {  232, 2045,     332, 10, }, // America/St_Johns
    {  232, 5780,     342,  9, }, // America/St_Kitts
    {  232, 5797,     351,  9, }, // America/St_Lucia
    {  232,  855,     360, 10, }, // America/St_Thomas
    {  232, 5830,     370, 11, }, // America/St_Vincent
    {  232, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  232, 5451,   62334,  6, }, // Asia/Bahrain
    {  232, 8614,   62340,  4, }, // Asia/Gaza
    {  232, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  232, 1152,     413,  9, }, // Asia/Kathmandu
    {  232,  992,     422,  7, }, // Asia/Kolkata
    {  232, 5464,   62344,  6, }, // Asia/Kuwait
    {  232, 1178,    6064,  5, }, // Asia/Macau
    {  232, 8671,     434,  8, }, // Asia/Qostanay
    {  232, 3137,    6142,  8, }, // Asia/Singapore
    {  232, 2638,    6175,  7, }, // Asia/Tehran
    {  232, 1202,   56146,  7, }, // Asia/Yangon
    {  232, 3565,   62350,  9, }, // Atlantic/Azores
    {  232, 3629,   62359, 16, }, // Atlantic/Cape_Verde
    {  232, 1379,   62375,  7, }, // Atlantic/Faroe
    {  232, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  232, 7235,   62382, 19, }, // Atlantic/South_Georgia
    {  232, 6784,     453, 10, }, // Atlantic/St_Helena
    {  232, 6577,   19437,  4, }, // Europe/Athens
    {  232, 6285,    6398,  7, }, // Europe/Belgrade
    {  232, 6178,   56194,  9, }, // Europe/Brussels
    {  232, 3830,   25479,  8, }, // Europe/Bucharest
    {  232, 6238,   62401, 13, }, // Europe/Copenhagen
    {  232, 2419,    6486,  4, }, // Europe/Kyiv
    {  232, 3078,    4400,  8, }, // Europe/Lisbon
    {  232, 6314,   62414, 10, }, // Europe/Luxembourg
    {  232, 6359,    6525,  6, }, // Europe/Monaco
    {  232, 3443,   21290,  6, }, // Europe/Moscow
    {  232, 6224,    6545,  4, }, // Europe/Prague
    {  232, 6273,    6554,  3, }, // Europe/Rome
    {  232, 6472,   62424,  9, }, // Europe/Stockholm
    {  232, 6516,   62433,  9, }, // Europe/Vatican
    {  232, 6164,   56035,  4, }, // Europe/Vienna
    {  232, 3055,   62442,  8, }, // Europe/Warsaw
    {  232, 6489,    4478,  6, }, // Europe/Zurich
    {  232, 7013,   62450, 13, }, // Indian/Maldives
    {  232, 3971,   62463,  8, }, // Indian/Mauritius
    {  232, 7174,     467,  7, }, // Indian/Reunion
    {  232, 3022,     474,  5, }, // Pacific/Chuuk
    {  232, 2929,    4512,  8, }, // Pacific/Honolulu
    {  232, 2897,    4520,  9, }, // Pacific/Kanton
    {  232, 2961,     479,  7, }, // Pacific/Pohnpei
    {  234,   14,       0,  6, }, // Africa/Asmara Filipino/Latin/Philippines
    {  234, 4337,       6,  8, }, // Africa/Sao_Tome
    {  234, 4154,      14,  8, }, // America/Asuncion
    {  234,  237,      22,  8, }, // America/Atikokan
    {  234, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  234, 3770,    3861,  6, }, // America/Cancun
    {  234, 7737,   62471, 17, }, // America/Ciudad_Juarez
    {  234, 5640,      66,  7, }, // America/Curacao
    {  234,  481,      73, 13, }, // America/Indiana/Knox
    {  234, 7924,      86, 16, }, // America/Indiana/Marengo
    {  234, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  234, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  234, 8001,     139, 14, }, // America/Indiana/Vevay
    {  234, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  234, 8049,     171, 16, }, // America/Indiana/Winamac
    {  234, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  234, 5849,     207, 22, }, // America/Lower_Princes
    {  234, 8183,    3899,  6, }, // America/Merida
    {  234, 2811,   62488, 17, }, // America/Mexico_City
    {  234, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  234, 8292,     255, 20, }, // America/North_Dakota/Center
    {  234, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  234,  393,     298,  4, }, // America/Nuuk
    {  234, 5871,   62505, 18, }, // America/Port_of_Spain
    {  234, 8387,   62523, 25, }, // America/Rankin_Inlet
    {  234, 6862,     302, 16, }, // America/Scoresbysund
    {  234, 8471,     318, 14, }, // America/St_Barthelemy
    {  234, 2045,     332, 10, }, // America/St_Johns
    {  234, 5780,     342,  9, }, // America/St_Kitts
    {  234, 5797,     351,  9, }, // America/St_Lucia
    {  234,  855,     360, 10, }, // America/St_Thomas
    {  234, 5830,     370, 11, }, // America/St_Vincent
    {  234, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  234, 1226,   62548, 22, }, // Asia/Ho_Chi_Minh
    {  234, 1152,     413,  9, }, // Asia/Kathmandu
    {  234,  992,     422,  7, }, // Asia/Kolkata
    {  234, 1178,    4222,  5, }, // Asia/Macau
    {  234, 8671,     434,  8, }, // Asia/Qostanay
    {  234, 1202,   56146,  7, }, // Asia/Yangon
    {  234, 1379,     448,  5, }, // Atlantic/Faroe
    {  234, 6784,     453, 10, }, // Atlantic/St_Helena
    {  234, 2419,    6486,  4, }, // Europe/Kyiv
    {  234, 7174,     467,  7, }, // Indian/Reunion
    {  234, 3022,     474,  5, }, // Pacific/Chuuk
    {  234, 2929,    4512,  8, }, // Pacific/Honolulu
    {  234, 2897,    4520,  9, }, // Pacific/Kanton
    {  234, 2961,     479,  7, }, // Pacific/Pohnpei
    {  235, 4961,    3793, 11, }, // Africa/Addis_Ababa Finnish/Latin/Finland
    {  235, 7383,   42592,  5, }, // Africa/Algiers
    {  235,   14,       0,  6, }, // Africa/Asmara
    {  235, 2242,   24913,  5, }, // Africa/Cairo
    {  235, 5068,   62570,  8, }, // Africa/El_Aaiun
    {  235, 4426,   24937,  7, }, // Africa/Khartoum
    {  235, 6832,   18802,  4, }, // Africa/Lome
    {  235, 5173,   42632,  9, }, // Africa/Ndjamena
    {  235, 4337,       6,  8, }, // Africa/Sao_Tome
    {  235, 4456,   24992,  9, }, // America/Araguaina
    {  235,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  235, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  235, 4154,      14,  8, }, // America/Asuncion
    {  235,  237,      22,  8, }, // America/Atikokan
    {  235, 7586,      30, 17, }, // America/Bahia_Banderas
    {  235, 4263,   18898,  6, }, // America/Bogota
    {  235, 3770,      47,  6, }, // America/Cancun
    {  235, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  235, 3693,   25007,  6, }, // America/Cuiaba
    {  235, 5640,      66,  7, }, // America/Curacao
    {  235, 7832,   25013,  8, }, // America/Eirunepe
    {  235, 2221,   61885,  7, }, // America/Havana
    {  235,  481,      73, 13, }, // America/Indiana/Knox
    {  235, 7924,      86, 16, }, // America/Indiana/Marengo
    {  235, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  235, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  235, 8001,     139, 14, }, // America/Indiana/Vevay
    {  235, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  235, 8049,     171, 16, }, // America/Indiana/Winamac
    {  235, 2665,    3892,  7, }, // America/Jamaica
    {  235, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  235, 5849,     207, 22, }, // America/Lower_Princes
    {  235, 8116,   25021,  6, }, // America/Maceio
    {  235, 2779,   25036,  8, }, // America/Mazatlan
    {  235, 8183,     229,  6, }, // America/Merida
    {  235, 2811,   56111, 16, }, // America/Mexico_City
    {  235, 8264,   62578, 22, }, // America/North_Dakota/Beulah
    {  235, 8292,   62600, 22, }, // America/North_Dakota/Center
    {  235, 8320,   62622, 25, }, // America/North_Dakota/New_Salem
    {  235,  393,   34408,  7, }, // America/Nuuk
    {  235, 8440,   25137,  8, }, // America/Santarem
    {  235, 2165,   62647, 17, }, // America/Santiago
    {  235, 1862,   25145,  9, }, // America/Sao_Paulo
    {  235, 6862,   42785, 12, }, // America/Scoresbysund
    {  235, 8471,   25154, 16, }, // America/St_Barthelemy
    {  235, 2045,     332, 10, }, // America/St_Johns
    {  235, 5780,   25182, 11, }, // America/St_Kitts
    {  235, 5797,   34444, 11, }, // America/St_Lucia
    {  235,  855,   42797, 12, }, // America/St_Thomas
    {  235, 5830,   25216, 13, }, // America/St_Vincent
    {  235, 5673,   25229,  7, }, // America/Thule
    {  235, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  235, 8531,   62664, 15, }, // Antarctica/Macquarie
    {  235, 5440,   62679,  5, }, // Asia/Aqtau
    {  235, 5270,   55725,  6, }, // Asia/Aqtobe
    {  235,  964,   56135,  7, }, // Asia/Ashgabat
    {  235, 8587,   57318,  6, }, // Asia/Atyrau
    {  235, 3509,    4063,  6, }, // Asia/Baghdad
    {  235, 3680,   31917,  6, }, // Asia/Bishkek
    {  235, 4503,   61984,  5, }, // Asia/Chita
    {  235, 5963,   62684,  9, }, // Asia/Choibalsan
    {  235, 4442,   62693,  8, }, // Asia/Damascus
    {  235, 6092,   31939,  7, }, // Asia/Dushanbe
    {  235, 1226,   62701, 22, }, // Asia/Ho_Chi_Minh
    {  235, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  235, 4218,   62010,  9, }, // Asia/Kamchatka
    {  235, 1152,     413,  9, }, // Asia/Kathmandu
    {  235, 8624,   62723,  7, }, // Asia/Khandyga
    {  235,  992,   62730,  8, }, // Asia/Kolkata
    {  235, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  235, 1178,     429,  5, }, // Asia/Macau
    {  235, 6883,   34547,  6, }, // Asia/Muscat
    {  235, 2446,   25331,  7, }, // Asia/Nicosia
    {  235, 7108,   56586,  6, }, // Asia/Oral
    {  235, 4116,   32033,  9, }, // Asia/Pyongyang
    {  235, 8671,   62738,  8, }, // Asia/Qostanay
    {  235, 4171,   20943,  9, }, // Asia/Qyzylorda
    {  235, 3486,    6120,  4, }, // Asia/Riyadh
    {  235, 4310,   32047,  7, }, // Asia/Sakhalin
    {  235, 3116,   56625,  4, }, // Asia/Seoul
    {  235, 4683,   32060,  7, }, // Asia/Tashkent
    {  235, 2638,    6175,  7, }, // Asia/Tehran
    {  235, 2687,    4257,  5, }, // Asia/Tokyo
    {  235, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  235, 1126,   25383,  6, }, // Asia/Urumqi
    {  235, 4730,    6206,  7, }, // Asia/Yakutsk
    {  235, 1202,     442,  6, }, // Asia/Yangon
    {  235, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  235, 3649,    6231,  7, }, // Asia/Yerevan
    {  235, 3565,   62746,  6, }, // Atlantic/Azores
    {  235, 6591,   62752, 14, }, // Atlantic/Canary
    {  235, 3629,   55900,  9, }, // Atlantic/Cape_Verde
    {  235, 1379,   62766,  9, }, // Atlantic/Faroe
    {  235, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  235, 7235,   62775, 13, }, // Atlantic/South_Georgia
    {  235, 6784,   34678, 12, }, // Atlantic/St_Helena
    {  235, 3522,   32115,  8, }, // Europe/Astrakhan
    {  235, 6577,   62151,  6, }, // Europe/Athens
    {  235, 6285,   25465,  7, }, // Europe/Belgrade
    {  235, 4659,   62788,  8, }, // Europe/Berlin
    {  235, 6178,   62796,  7, }, // Europe/Brussels
    {  235, 3830,   25479,  8, }, // Europe/Bucharest
    {  235, 8751,   25487,  8, }, // Europe/Busingen
    {  235, 2475,   62803,  8, }, // Europe/Chisinau
    {  235, 6238,   62811, 12, }, // Europe/Copenhagen
    {  235, 8783,   62823,  8, }, // Europe/Isle_of_Man
    {  235, 2419,   62831,  5, }, // Europe/Kyiv
    {  235, 3078,    4400,  8, }, // Europe/Lisbon
    {  235, 2393,   62836,  6, }, // Europe/London
    {  235, 6314,    4414,  9, }, // Europe/Luxembourg
    {  235, 6531,   62205, 13, }, // Europe/Mariehamn
    {  235, 3443,   62842,  7, }, // Europe/Moscow
    {  235, 4186,   62849,  7, }, // Europe/Paris
    {  235, 6224,   34765,  5, }, // Europe/Prague
    {  235, 8816,   62856,  5, }, // Europe/Riga
    {  235, 6273,   62228,  5, }, // Europe/Rome
    {  235, 6472,   62861,  8, }, // Europe/Stockholm
    {  235, 8846,   62869,  8, }, // Europe/Tallinn
    {  235, 6135,   19608,  6, }, // Europe/Tirane
    {  235, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  235, 6516,   62877,  9, }, // Europe/Vatican
    {  235, 6164,   56035,  4, }, // Europe/Vienna
    {  235, 8878,   62886,  5, }, // Europe/Vilnius
    {  235, 3055,   62891,  7, }, // Europe/Warsaw
    {  235, 6489,    4478,  6, }, // Europe/Zurich
    {  235, 5979,   62898, 10, }, // Indian/Christmas
    {  235, 5996,   62908, 12, }, // Indian/Cocos
    {  235, 4931,   62920,  7, }, // Indian/Comoro
    {  235, 6624,   62927, 15, }, // Indian/Kerguelen
    {  235, 7223,   25565,  4, }, // Indian/Mahe
    {  235, 7013,   62942, 10, }, // Indian/Maldives
    {  235, 7174,     467,  7, }, // Indian/Reunion
    {  235, 2863,   62952, 13, }, // Pacific/Chatham
    {  235, 3022,     474,  5, }, // Pacific/Chuuk
    {  235, 2201,   62965, 13, }, // Pacific/Easter
    {  235, 3804,   32307,  5, }, // Pacific/Fiji
    {  235, 6659,   62978, 13, }, // Pacific/Gambier
    {  235, 2929,    4512,  8, }, // Pacific/Honolulu
    {  235, 2897,    4520,  9, }, // Pacific/Kanton
    {  235, 3953,   62991, 15, }, // Pacific/Marquesas
    {  235, 8909,   63006, 12, }, // Pacific/Midway
    {  235, 7061,    4529,  6, }, // Pacific/Noumea
    {  235, 2961,     479,  7, }, // Pacific/Pohnpei
    {  236, 4961,   63018, 11, }, // Africa/Addis_Ababa French/Latin/France
    {  236, 7383,   42592,  5, }, // Africa/Algiers
    {  236,   14,       0,  6, }, // Africa/Asmara
    {  236, 2242,   63029,  8, }, // Africa/Cairo
    {  236, 5068,   63037,  8, }, // Africa/El_Aaiun
    {  236, 6832,   18802,  4, }, // Africa/Lome
    {  236, 5015,   42614, 10, }, // Africa/Mogadishu
    {  236, 5173,   42632,  9, }, // Africa/Ndjamena
    {  236, 4337,       6,  8, }, // Africa/Sao_Tome
    {  236, 2732,   63045, 15, }, // Africa/Tripoli
    {  236, 4456,   24992,  9, }, // America/Araguaina
    {  236,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  236, 7452,   42641, 12, }, // America/Argentina/Rio_Gallegos
    {  236, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  236, 7560,   63060,  7, }, // America/Argentina/Ushuaia
    {  236, 4154,      14,  8, }, // America/Asuncion
    {  236,  237,      22,  8, }, // America/Atikokan
    {  236, 7586,   63067, 17, }, // America/Bahia_Banderas
    {  236, 5571,   63084, 10, }, // America/Barbados
    {  236, 7609,   34376,  5, }, // America/Belem
    {  236, 3770,      47,  6, }, // America/Cancun
    {  236, 5398,   63094,  7, }, // America/Cayman
    {  236, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  236, 3693,   25007,  6, }, // America/Cuiaba
    {  236, 5640,      66,  7, }, // America/Curacao
    {  236, 3351,   63101,  7, }, // America/Detroit
    {  236, 5656,   63108,  9, }, // America/Dominica
    {  236, 7832,   25013,  8, }, // America/Eirunepe
    {  236, 5687,   63117,  7, }, // America/Grenada
    {  236, 2221,   63124,  9, }, // America/Havana
    {  236,  481,   63133, 14, }, // America/Indiana/Knox
    {  236, 7924,   63147, 17, }, // America/Indiana/Marengo
    {  236, 7948,   63164, 20, }, // America/Indiana/Petersburg
    {  236, 7975,   63184, 19, }, // America/Indiana/Tell_City
    {  236, 8001,   63203, 15, }, // America/Indiana/Vevay
    {  236, 8023,   63218, 19, }, // America/Indiana/Vincennes
    {  236, 8049,   63237, 17, }, // America/Indiana/Winamac
    {  236, 2665,   63254,  8, }, // America/Jamaica
    {  236, 8088,   63262, 21, }, // America/Kentucky/Monticello
    {  236, 5849,     207, 22, }, // America/Lower_Princes
    {  236, 8116,   25021,  6, }, // America/Maceio
    {  236, 1892,   63283,  6, }, // America/Manaus
    {  236, 2779,   25036,  8, }, // America/Mazatlan
    {  236, 8183,     229,  6, }, // America/Merida
    {  236, 2811,   57299,  6, }, // America/Mexico_City
    {  236, 8264,   63289, 23, }, // America/North_Dakota/Beulah
    {  236, 8292,   63312, 23, }, // America/North_Dakota/Center
    {  236, 8320,   63335, 26, }, // America/North_Dakota/New_Salem
    {  236,  393,     298,  4, }, // America/Nuuk
    {  236, 5871,   63361, 14, }, // America/Port_of_Spain
    {  236, 5760,   63375, 10, }, // America/Puerto_Rico
    {  236, 8440,   25137,  8, }, // America/Santarem
    {  236, 6044,   63385, 14, }, // America/Santo_Domingo
    {  236, 1862,   25145,  9, }, // America/Sao_Paulo
    {  236, 6862,     302, 16, }, // America/Scoresbysund
    {  236, 8471,   25154, 16, }, // America/St_Barthelemy
    {  236, 2045,   63399, 25, }, // America/St_Johns
    {  236, 5780,   63424, 16, }, // America/St_Kitts
    {  236, 5797,   63440, 12, }, // America/St_Lucia
    {  236,  855,   25204, 12, }, // America/St_Thomas
    {  236, 5830,   63452, 13, }, // America/St_Vincent
    {  236, 5673,   63465,  5, }, // America/Thule
    {  236, 6066,   55662, 16, }, // Antarctica/DumontDUrville
    {  236, 7258,   63470,  5, }, // Antarctica/Syowa
    {  236, 5297,   56127,  8, }, // Asia/Almaty
    {  236, 5440,   63475,  6, }, // Asia/Aqtau
    {  236, 5270,   63481, 11, }, // Asia/Aqtobe
    {  236,  964,   63492,  8, }, // Asia/Ashgabat
    {  236, 8587,   63500,  7, }, // Asia/Atyrau
    {  236, 3509,    4063,  6, }, // Asia/Baghdad
    {  236, 5451,   63507,  7, }, // Asia/Bahrain
    {  236, 3555,   34478,  5, }, // Asia/Baku
    {  236, 3988,   63514,  8, }, // Asia/Beirut
    {  236, 3680,   63522,  7, }, // Asia/Bishkek
    {  236, 4503,   34489,  6, }, // Asia/Chita
    {  236, 5963,   63529, 11, }, // Asia/Choibalsan
    {  236, 4442,    4102,  5, }, // Asia/Damascus
    {  236, 3498,   63540,  5, }, // Asia/Dubai
    {  236, 6092,   63545,  9, }, // Asia/Dushanbe
    {  236, 8599,   63554, 10, }, // Asia/Famagusta
    {  236, 4697,   63564,  6, }, // Asia/Hebron
    {  236, 1226,   63570, 17, }, // Asia/Ho_Chi_Minh
    {  236, 4086,   63587,  8, }, // Asia/Irkutsk
    {  236, 1257,   63595,  9, }, // Asia/Jerusalem
    {  236, 3462,   34517,  6, }, // Asia/Kabul
    {  236, 4218,   34523, 10, }, // Asia/Kamchatka
    {  236, 1152,   63604,  9, }, // Asia/Kathmandu
    {  236,  992,   34533,  8, }, // Asia/Kolkata
    {  236, 4099,   63613, 11, }, // Asia/Krasnoyarsk
    {  236, 5464,   63624,  6, }, // Asia/Kuwait
    {  236, 1178,     429,  5, }, // Asia/Macau
    {  236, 1316,   63630,  8, }, // Asia/Makassar
    {  236, 7145,   63638,  7, }, // Asia/Manila
    {  236, 6883,   19248,  7, }, // Asia/Muscat
    {  236, 2446,   63645,  7, }, // Asia/Nicosia
    {  236, 4037,   42883, 12, }, // Asia/Novosibirsk
    {  236, 7108,   63652,  7, }, // Asia/Oral
    {  236, 8671,   63659,  8, }, // Asia/Qostanay
    {  236, 4171,   63667,  9, }, // Asia/Qyzylorda
    {  236, 3486,   19265,  5, }, // Asia/Riyadh
    {  236, 4310,   63676,  9, }, // Asia/Sakhalin
    {  236, 7208,   63685, 10, }, // Asia/Samarkand
    {  236, 3116,   63695,  5, }, // Asia/Seoul
    {  236, 3137,   34581,  9, }, // Asia/Singapore
    {  236, 4683,   63700,  8, }, // Asia/Tashkent
    {  236, 3817,   63708,  8, }, // Asia/Tbilisi
    {  236, 2638,   63716,  7, }, // Asia/Tehran
    {  236, 1346,   63723, 11, }, // Asia/Ulaanbaatar
    {  236, 1126,   25383,  6, }, // Asia/Urumqi
    {  236, 4730,   63734,  8, }, // Asia/Yakutsk
    {  236, 1202,   63742,  7, }, // Asia/Yangon
    {  236, 3785,   63749, 13, }, // Asia/Yekaterinburg
    {  236, 3649,   25397,  6, }, // Asia/Yerevan
    {  236, 3565,   42965,  6, }, // Atlantic/Azores
    {  236, 5588,   42971,  8, }, // Atlantic/Bermuda
    {  236, 6591,   63762, 13, }, // Atlantic/Canary
    {  236, 3629,   63775,  8, }, // Atlantic/Cape_Verde
    {  236, 1379,   63783, 10, }, // Atlantic/Faroe
    {  236, 8699,   63793,  6, }, // Atlantic/Madeira
    {  236, 7235,   63799, 14, }, // Atlantic/South_Georgia
    {  236, 6784,   63813, 13, }, // Atlantic/St_Helena
    {  236, 1654,   63826,  8, }, // Australia/Adelaide
    {  236, 6149,   63834,  7, }, // Europe/Andorra
    {  236, 6577,   63841,  7, }, // Europe/Athens
    {  236, 6178,   56194,  9, }, // Europe/Brussels
    {  236, 3830,   19464,  8, }, // Europe/Bucharest
    {  236, 8751,   25487,  8, }, // Europe/Busingen
    {  236, 6238,   19472, 10, }, // Europe/Copenhagen
    {  236, 8767,   25495,  9, }, // Europe/Guernsey
    {  236, 8783,   63848, 10, }, // Europe/Isle_of_Man
    {  236, 2419,    6486,  4, }, // Europe/Kyiv
    {  236, 3078,   63858,  8, }, // Europe/Lisbon
    {  236, 2393,   19543,  7, }, // Europe/London
    {  236, 6346,   63866,  5, }, // Europe/Malta
    {  236, 3443,   43080,  6, }, // Europe/Moscow
    {  236, 6419,   63871, 11, }, // Europe/San_Marino
    {  236, 6135,   19608,  6, }, // Europe/Tirane
    {  236, 8861,   63882, 10, }, // Europe/Ulyanovsk
    {  236, 6516,   63892, 10, }, // Europe/Vatican
    {  236, 6164,   63902,  6, }, // Europe/Vienna
    {  236, 3055,   63908,  8, }, // Europe/Warsaw
    {  236, 4931,   63916,  7, }, // Indian/Comoro
    {  236, 7223,   25565,  4, }, // Indian/Mahe
    {  236, 3971,   63923,  7, }, // Indian/Mauritius
    {  236, 7174,   63930, 10, }, // Indian/Reunion
    {  236, 3022,     474,  5, }, // Pacific/Chuuk
    {  236, 2201,   63940, 13, }, // Pacific/Easter
    {  236, 7323,   25598,  5, }, // Pacific/Efate
    {  236, 3804,    4507,  5, }, // Pacific/Fiji
    {  236, 6641,   19672,  9, }, // Pacific/Galapagos
    {  236, 2929,    4512,  8, }, // Pacific/Honolulu
    {  236, 2897,    4520,  9, }, // Pacific/Kanton
    {  236, 3953,   63953,  9, }, // Pacific/Marquesas
    {  236, 7061,    4529,  6, }, // Pacific/Noumea
    {  236, 7118,   63962,  6, }, // Pacific/Palau
    {  236, 2961,     479,  7, }, // Pacific/Pohnpei
    {  243, 5173,   63968,  8, }, // Africa/Ndjamena French/Latin/Canada
    {  243, 2732,   63976, 15, }, // Africa/Tripoli
    {  243, 5571,   63991, 12, }, // America/Barbados
    {  243, 5398,   64003, 12, }, // America/Cayman
    {  243, 8264,   64015, 23, }, // America/North_Dakota/Beulah
    {  243, 8292,   64038, 23, }, // America/North_Dakota/Center
    {  243, 8320,   64061, 25, }, // America/North_Dakota/New_Salem
    {  243, 2045,     332, 10, }, // America/St_Johns
    {  243, 5780,   64086, 26, }, // America/St_Kitts
    {  243,  855,   42797, 12, }, // America/St_Thomas
    {  243, 5363,   64112, 11, }, // America/Tegucigalpa
    {  243, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  243, 1060,   64123,  5, }, // Asia/Dhaka
    {  243, 1284,   64128,  8, }, // Asia/Thimphu
    {  243, 1379,   64136, 10, }, // Atlantic/Faroe
    {  243, 8783,   64146, 10, }, // Europe/Isle_of_Man
    {  243, 2393,   19543,  7, }, // Europe/London
    {  243, 6516,   19617,  7, }, // Europe/Vatican
    {  243, 2201,   64156, 13, }, // Pacific/Easter
    {  243, 7118,   64169,  5, }, // Pacific/Palau
    {  282,   14,       0,  6, }, // Africa/Asmara Friulian/Latin/Italy
    {  282, 4337,       6,  8, }, // Africa/Sao_Tome
    {  282, 4154,      14,  8, }, // America/Asuncion
    {  282,  237,      22,  8, }, // America/Atikokan
    {  282, 7586,      30, 17, }, // America/Bahia_Banderas
    {  282, 3770,      47,  6, }, // America/Cancun
    {  282, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  282, 5640,      66,  7, }, // America/Curacao
    {  282,  481,      73, 13, }, // America/Indiana/Knox
    {  282, 7924,      86, 16, }, // America/Indiana/Marengo
    {  282, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  282, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  282, 8001,     139, 14, }, // America/Indiana/Vevay
    {  282, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  282, 8049,     171, 16, }, // America/Indiana/Winamac
    {  282, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  282, 5849,     207, 22, }, // America/Lower_Princes
    {  282, 8183,     229,  6, }, // America/Merida
    {  282, 3294,   64174, 10, }, // America/New_York
    {  282, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  282, 8292,     255, 20, }, // America/North_Dakota/Center
    {  282, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  282,  393,     298,  4, }, // America/Nuuk
    {  282, 1862,   64184, 20, }, // America/Sao_Paulo
    {  282, 6862,     302, 16, }, // America/Scoresbysund
    {  282, 8471,     318, 14, }, // America/St_Barthelemy
    {  282, 2045,     332, 10, }, // America/St_Johns
    {  282, 5780,     342,  9, }, // America/St_Kitts
    {  282, 5797,     351,  9, }, // America/St_Lucia
    {  282,  855,     360, 10, }, // America/St_Thomas
    {  282, 5830,     370, 11, }, // America/St_Vincent
    {  282, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  282, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  282, 1152,     413,  9, }, // Asia/Kathmandu
    {  282,  992,     422,  7, }, // Asia/Kolkata
    {  282, 1178,     429,  5, }, // Asia/Macau
    {  282, 1202,     442,  6, }, // Asia/Yangon
    {  282, 3565,   64204,  6, }, // Atlantic/Azores
    {  282, 6591,   64210,  8, }, // Atlantic/Canary
    {  282, 1379,     448,  5, }, // Atlantic/Faroe
    {  282, 6784,     453, 10, }, // Atlantic/St_Helena
    {  282, 6285,   64218,  7, }, // Europe/Belgrade
    {  282, 2419,     463,  4, }, // Europe/Kyiv
    {  282, 3078,   64225,  7, }, // Europe/Lisbon
    {  282, 6455,   64232,  7, }, // Europe/Ljubljana
    {  282, 6314,   64239, 10, }, // Europe/Luxembourg
    {  282, 6346,   63866,  5, }, // Europe/Malta
    {  282, 3443,   64249,  6, }, // Europe/Moscow
    {  282, 6224,   64255,  6, }, // Europe/Prague
    {  282, 6419,   64261,  9, }, // Europe/San_Marino
    {  282, 6164,   64270,  5, }, // Europe/Vienna
    {  282, 7174,     467,  7, }, // Indian/Reunion
    {  282, 3022,     474,  5, }, // Pacific/Chuuk
    {  282, 2961,     479,  7, }, // Pacific/Pohnpei
    {  283,   14,       0,  6, }, // Africa/Asmara Fulah/Latin/Senegal
    {  283, 4337,       6,  8, }, // Africa/Sao_Tome
    {  283, 4154,      14,  8, }, // America/Asuncion
    {  283,  237,      22,  8, }, // America/Atikokan
    {  283, 7586,      30, 17, }, // America/Bahia_Banderas
    {  283, 3770,      47,  6, }, // America/Cancun
    {  283, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  283, 5640,      66,  7, }, // America/Curacao
    {  283,  481,      73, 13, }, // America/Indiana/Knox
    {  283, 7924,      86, 16, }, // America/Indiana/Marengo
    {  283, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  283, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  283, 8001,     139, 14, }, // America/Indiana/Vevay
    {  283, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  283, 8049,     171, 16, }, // America/Indiana/Winamac
    {  283, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  283, 5849,     207, 22, }, // America/Lower_Princes
    {  283, 8183,     229,  6, }, // America/Merida
    {  283, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  283, 8292,     255, 20, }, // America/North_Dakota/Center
    {  283, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  283,  393,     298,  4, }, // America/Nuuk
    {  283, 6862,     302, 16, }, // America/Scoresbysund
    {  283, 8471,     318, 14, }, // America/St_Barthelemy
    {  283, 2045,     332, 10, }, // America/St_Johns
    {  283, 5780,     342,  9, }, // America/St_Kitts
    {  283, 5797,     351,  9, }, // America/St_Lucia
    {  283,  855,     360, 10, }, // America/St_Thomas
    {  283, 5830,     370, 11, }, // America/St_Vincent
    {  283, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  283, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  283, 1152,     413,  9, }, // Asia/Kathmandu
    {  283,  992,     422,  7, }, // Asia/Kolkata
    {  283, 1178,     429,  5, }, // Asia/Macau
    {  283, 1202,     442,  6, }, // Asia/Yangon
    {  283, 1379,     448,  5, }, // Atlantic/Faroe
    {  283, 6784,     453, 10, }, // Atlantic/St_Helena
    {  283, 2419,     463,  4, }, // Europe/Kyiv
    {  283, 7174,     467,  7, }, // Indian/Reunion
    {  283, 3022,     474,  5, }, // Pacific/Chuuk
    {  283, 2961,     479,  7, }, // Pacific/Pohnpei
    {  289, 6751,   64275, 12, }, // Africa/Abidjan Fulah/Adlam/Guinea
    {  289, 6723,   64287, 10, }, // Africa/Accra
    {  289, 4961,   64297, 18, }, // Africa/Addis_Ababa
    {  289, 7383,   64315, 14, }, // Africa/Algiers
    {  289,   14,   64329, 12, }, // Africa/Asmara
    {  289,   44,   64341, 14, }, // Africa/Bamako
    {  289, 5159,   64355, 10, }, // Africa/Bangui
    {  289, 6709,   64365, 12, }, // Africa/Banjul
    {  289, 7398,   64377, 12, }, // Africa/Bissau
    {  289, 4859,   64389, 18, }, // Africa/Blantyre
    {  289, 5189,   64407, 16, }, // Africa/Brazzaville
    {  289, 4824,   64423, 18, }, // Africa/Bujumbura
    {  289, 2242,   64441, 10, }, // Africa/Cairo
    {  289, 4019,   64451, 26, }, // Africa/Casablanca
    {  289, 7412,   64477, 10, }, // Africa/Ceuta
    {  289, 6736,   64487, 18, }, // Africa/Conakry
    {  289, 6803,   64505, 12, }, // Africa/Dakar
    {  289, 5032,   64517, 25, }, // Africa/Dar_es_Salaam
    {  289, 4945,   64542, 14, }, // Africa/Djibouti
    {  289, 5145,   64556, 12, }, // Africa/Douala
    {  289, 5068,   64568, 14, }, // Africa/El_Aaiun
    {  289, 6816,   64582, 12, }, // Africa/Freetown
    {  289, 4808,   64594, 16, }, // Africa/Gaborone
    {  289, 4917,   64610, 12, }, // Africa/Harare
    {  289, 4381,   64622, 22, }, // Africa/Johannesburg
    {  289, 4401,   64644,  8, }, // Africa/Juba
    {  289, 5053,   64652, 14, }, // Africa/Kampala
    {  289, 4426,   64666, 14, }, // Africa/Khartoum
    {  289, 4875,   64680, 12, }, // Africa/Kigali
    {  289, 5208,   64692, 14, }, // Africa/Kinshasa
    {  289, 4646,   64706, 12, }, // Africa/Lagos
    {  289, 5238,   64718, 20, }, // Africa/Libreville
    {  289, 6832,   64738,  8, }, // Africa/Lome
    {  289, 5113,   64746, 16, }, // Africa/Luanda
    {  289, 4841,   64762, 18, }, // Africa/Lubumbashi
    {  289, 4903,   64780, 12, }, // Africa/Lusaka
    {  289, 5224,   64792, 14, }, // Africa/Malabo
    {  289, 4889,   64806, 12, }, // Africa/Maputo
    {  289, 5099,   64818, 12, }, // Africa/Maseru
    {  289, 5084,   64830, 18, }, // Africa/Mbabane
    {  289, 5015,   64848, 16, }, // Africa/Mogadishu
    {  289, 6997,   64864, 16, }, // Africa/Monrovia
    {  289, 3755,   64880, 14, }, // Africa/Nairobi
    {  289, 5173,   64894, 16, }, // Africa/Ndjamena
    {  289, 5256,   64910, 12, }, // Africa/Niamey
    {  289, 6766,   64922, 20, }, // Africa/Nouakchott
    {  289, 6690,   64942, 16, }, // Africa/Ouagadougou
    {  289, 5127,   64958, 21, }, // Africa/Porto-Novo
    {  289, 4337,   64979, 19, }, // Africa/Sao_Tome
    {  289, 2732,   64998, 14, }, // Africa/Tripoli
    {  289, 6503,   65012, 12, }, // Africa/Tunis
    {  289, 4054,   65024, 14, }, // Africa/Windhoek
    {  289,  132,   65038,  8, }, // America/Adak
    {  289, 3183,   65046, 18, }, // America/Anchorage
    {  289, 5524,   65064, 14, }, // America/Anguilla
    {  289, 5541,   65078, 12, }, // America/Antigua
    {  289, 4456,   65090, 18, }, // America/Araguaina
    {  289,  166,   65108, 29, }, // America/Argentina/Buenos_Aires
    {  289,   91,   65137, 20, }, // America/Argentina/Catamarca
    {  289,  270,   65157, 16, }, // America/Argentina/Cordoba
    {  289,  441,   65173, 10, }, // America/Argentina/Jujuy
    {  289, 7425,   65183, 17, }, // America/Argentina/La_Rioja
    {  289,  565,   65200, 16, }, // America/Argentina/Mendoza
    {  289, 7452,   65216, 21, }, // America/Argentina/Rio_Gallegos
    {  289, 7483,   65237, 10, }, // America/Argentina/Salta
    {  289, 7507,   65247, 19, }, // America/Argentina/San_Juan
    {  289, 5497,   65266, 19, }, // America/Argentina/San_Luis
    {  289, 7534,   65285, 16, }, // America/Argentina/Tucuman
    {  289, 7560,   65301, 14, }, // America/Argentina/Ushuaia
    {  289, 5557,   65315, 12, }, // America/Aruba
    {  289, 4154,   65327, 18, }, // America/Asuncion
    {  289,  237,   65345, 18, }, // America/Atikokan
    {  289, 3581,   65363, 12, }, // America/Bahia
    {  289, 7586,   65375, 33, }, // America/Bahia_Banderas
    {  289, 5571,   65408, 18, }, // America/Barbados
    {  289, 7609,   65426, 10, }, // America/Belem
    {  289, 5309,   65436, 12, }, // America/Belize
    {  289, 7623,   65448, 25, }, // America/Blanc-Sablon
    {  289, 7644,   65473, 19, }, // America/Boa_Vista
    {  289, 4263,   65492, 12, }, // America/Bogota
    {  289, 7662,   65504, 12, }, // America/Boise
    {  289, 7676,   65516, 25, }, // America/Cambridge_Bay
    {  289, 7698,   65541, 23, }, // America/Campo_Grande
    {  289, 3770,   65564, 14, }, // America/Cancun
    {  289, 4596,   65578, 14, }, // America/Caracas
    {  289, 4247,   65592, 12, }, // America/Cayenne
    {  289, 5398,   65604, 14, }, // America/Cayman
    {  289, 3251,   65618, 16, }, // America/Chicago
    {  289, 7719,   65634, 16, }, // America/Chihuahua
    {  289, 5324,   65650, 21, }, // America/Costa_Rica
    {  289, 7759,   65671, 14, }, // America/Creston
    {  289, 3693,   65685, 14, }, // America/Cuiaba
    {  289, 5640,   65699, 18, }, // America/Curacao
    {  289, 7775,   65717, 26, }, // America/Danmarkshavn
    {  289, 7796,   65743, 12, }, // America/Dawson
    {  289, 7811,   65755, 21, }, // America/Dawson_Creek
    {  289,  805,   65776, 12, }, // America/Denver
    {  289, 3351,   65788, 14, }, // America/Detroit
    {  289, 5656,   65802, 18, }, // America/Dominica
    {  289,  893,   65820, 16, }, // America/Edmonton
    {  289, 7832,   65836, 14, }, // America/Eirunepe
    {  289, 5343,   65850, 23, }, // America/El_Salvador
    {  289, 7849,   65873, 22, }, // America/Fort_Nelson
    {  289, 7869,   65895, 20, }, // America/Fortaleza
    {  289, 7887,   65915, 15, }, // America/Glace_Bay
    {  289, 6844,   65930, 15, }, // America/Goose_Bay
    {  289, 4514,   65945, 19, }, // America/Grand_Turk
    {  289, 5687,   65964, 16, }, // America/Grenada
    {  289, 5703,   65980, 24, }, // America/Guadeloupe
    {  289, 3662,   66004, 20, }, // America/Guatemala
    {  289, 4743,   66024, 14, }, // America/Guayaquil
    {  289, 6895,   66038, 14, }, // America/Guyana
    {  289, 1923,   66052, 18, }, // America/Halifax
    {  289, 2221,   66070, 14, }, // America/Havana
    {  289, 7905,   66084, 24, }, // America/Hermosillo
    {  289,  348,   66108, 28, }, // America/Indiana/Indianapolis
    {  289,  481,   66136, 28, }, // America/Indiana/Knox
    {  289, 7924,   66164, 34, }, // America/Indiana/Marengo
    {  289, 7948,   66198, 40, }, // America/Indiana/Petersburg
    {  289, 7975,   66238, 37, }, // America/Indiana/Tell_City
    {  289, 8001,   66275, 32, }, // America/Indiana/Vevay
    {  289, 8023,   66307, 34, }, // America/Indiana/Vincennes
    {  289, 8049,   66341, 34, }, // America/Indiana/Winamac
    {  289, 8073,   66375, 12, }, // America/Inuvik
    {  289,  660,   66387, 16, }, // America/Iqaluit
    {  289, 2665,   66403, 16, }, // America/Jamaica
    {  289, 5282,   66419, 12, }, // America/Juneau
    {  289,  521,   66431, 12, }, // America/Kentucky/Louisville
    {  289, 8088,   66443, 40, }, // America/Kentucky/Monticello
    {  289, 5621,   66483, 20, }, // America/Kralendijk
    {  289, 4278,   66503, 13, }, // America/La_Paz
    {  289, 7132,   66516, 10, }, // America/Lima
    {  289, 3390,   66526, 21, }, // America/Los_Angeles
    {  289, 5849,   66547, 36, }, // America/Lower_Princes
    {  289, 8116,   66583, 12, }, // America/Maceio
    {  289, 8131,   66595, 14, }, // America/Managua
    {  289, 1892,   66609, 12, }, // America/Manaus
    {  289, 5814,   66621, 14, }, // America/Marigot
    {  289, 5722,   66635, 16, }, // America/Martinique
    {  289, 8147,   66651, 20, }, // America/Matamoros
    {  289, 2779,   66671, 16, }, // America/Mazatlan
    {  289, 8165,   66687, 16, }, // America/Menominee
    {  289, 8183,   66703, 12, }, // America/Merida
    {  289, 8198,   66715, 20, }, // America/Metlakatla
    {  289, 2811,   66735, 27, }, // America/Mexico_City
    {  289, 4293,   66762, 14, }, // America/Miquelon
    {  289, 8217,   66776, 16, }, // America/Moncton
    {  289, 8233,   66792, 18, }, // America/Monterrey
    {  289, 4000,   66810, 24, }, // America/Montevideo
    {  289, 5741,   66834, 20, }, // America/Montserrat
    {  289, 5383,   66854, 10, }, // America/Nassau
    {  289, 3294,   66864, 15, }, // America/New_York
    {  289, 8251,   66879, 10, }, // America/Nome
    {  289, 1834,   66889, 14, }, // America/Noronha
    {  289, 8264,   66903, 33, }, // America/North_Dakota/Beulah
    {  289, 8292,   66936, 37, }, // America/North_Dakota/Center
    {  289, 8320,   66973, 42, }, // America/North_Dakota/New_Salem
    {  289,  393,   67015,  8, }, // America/Nuuk
    {  289, 8351,   67023, 14, }, // America/Ojinaga
    {  289, 5413,   67037, 16, }, // America/Panama
    {  289, 6106,   67053, 20, }, // America/Paramaribo
    {  289, 3224,   67073, 12, }, // America/Phoenix
    {  289, 3847,   67085, 22, }, // America/Port-au-Prince
    {  289, 5871,   67107, 26, }, // America/Port_of_Spain
    {  289, 8367,   67133, 23, }, // America/Porto_Velho
    {  289, 5760,   67156, 19, }, // America/Puerto_Rico
    {  289, 3932,   67175, 23, }, // America/Punta_Arenas
    {  289, 8387,   67198, 23, }, // America/Rankin_Inlet
    {  289, 8408,   67221, 12, }, // America/Recife
    {  289, 1979,   67233, 14, }, // America/Regina
    {  289, 8423,   67247, 16, }, // America/Resolute
    {  289,  695,   67263, 21, }, // America/Rio_Branco
    {  289, 8440,   67284, 18, }, // America/Santarem
    {  289, 2165,   67302, 20, }, // America/Santiago
    {  289, 6044,   67322, 27, }, // America/Santo_Domingo
    {  289, 1862,   67349, 19, }, // America/Sao_Paulo
    {  289, 6862,   67368, 30, }, // America/Scoresbysund
    {  289, 8457,   67398, 10, }, // America/Sitka
    {  289, 8471,   67408, 27, }, // America/St_Barthelemy
    {  289, 2045,   67435, 19, }, // America/St_Johns
    {  289, 5780,   67454, 21, }, // America/St_Kitts
    {  289, 5797,   67475, 19, }, // America/St_Lucia
    {  289,  855,   67494, 21, }, // America/St_Thomas
    {  289, 5830,   67515, 23, }, // America/St_Vincent
    {  289, 8493,   67538, 23, }, // America/Swift_Current
    {  289, 5363,   67561, 22, }, // America/Tegucigalpa
    {  289, 5673,   67583,  8, }, // America/Thule
    {  289,  313,   67591, 18, }, // America/Tijuana
    {  289,  608,   67609, 16, }, // America/Toronto
    {  289, 5605,   67625, 16, }, // America/Tortola
    {  289, 2077,   67641, 16, }, // America/Vancouver
    {  289, 2128,   67657, 18, }, // America/Whitehorse
    {  289,  734,   67675, 16, }, // America/Winnipeg
    {  289, 8515,   67691, 14, }, // America/Yakutat
    {  289, 5918,   67705, 10, }, // Antarctica/Casey
    {  289, 6027,   67715, 12, }, // Antarctica/Davis
    {  289, 6066,   67727, 27, }, // Antarctica/DumontDUrville
    {  289, 8531,   67754, 14, }, // Antarctica/Macquarie
    {  289, 7029,   67768, 12, }, // Antarctica/Mawson
    {  289, 7076,   67780, 18, }, // Antarctica/McMurdo
    {  289, 8552,   67798, 14, }, // Antarctica/Palmer
    {  289, 7189,   67812, 12, }, // Antarctica/Rothera
    {  289, 7258,   67824, 14, }, // Antarctica/Syowa
    {  289, 8570,   67838, 12, }, // Antarctica/Troll
    {  289, 7337,   67850, 12, }, // Antarctica/Vostok
    {  289, 1413,   67862, 24, }, // Arctic/Longyearbyen
    {  289, 5487,   67886,  8, }, // Asia/Aden
    {  289, 5297,   67894, 14, }, // Asia/Almaty
    {  289, 3870,   67908, 12, }, // Asia/Amman
    {  289, 5428,   67920, 14, }, // Asia/Anadyr
    {  289, 5440,   67934, 14, }, // Asia/Aqtau
    {  289, 5270,   67948, 14, }, // Asia/Aqtobe
    {  289,  964,   67962, 16, }, // Asia/Ashgabat
    {  289, 8587,   67978, 14, }, // Asia/Atyrau
    {  289, 3509,   67992, 16, }, // Asia/Baghdad
    {  289, 5451,   68008, 16, }, // Asia/Bahrain
    {  289, 3555,   68024, 10, }, // Asia/Baku
    {  289, 4368,   68034, 16, }, // Asia/Bangkok
    {  289, 3473,   68050, 16, }, // Asia/Barnaul
    {  289, 3988,   68066, 16, }, // Asia/Beirut
    {  289, 3680,   68082, 16, }, // Asia/Bishkek
    {  289, 5906,   68098, 12, }, // Asia/Brunei
    {  289, 4503,   68110, 10, }, // Asia/Chita
    {  289, 5963,   68120, 18, }, // Asia/Choibalsan
    {  289, 4413,   68138, 14, }, // Asia/Colombo
    {  289, 4442,   68152, 14, }, // Asia/Damascus
    {  289, 1060,   64505, 10, }, // Asia/Dhaka
    {  289, 6125,   68166, 10, }, // Asia/Dili
    {  289, 3498,   68176, 10, }, // Asia/Dubai
    {  289, 6092,   68186, 14, }, // Asia/Dushanbe
    {  289, 8599,   68200, 18, }, // Asia/Famagusta
    {  289, 8614,   68218, 10, }, // Asia/Gaza
    {  289, 4697,   68228, 14, }, // Asia/Hebron
    {  289, 1226,   68242, 31, }, // Asia/Ho_Chi_Minh
    {  289, 2576,   68273, 17, }, // Asia/Hong_Kong
    {  289, 4673,   68290, 10, }, // Asia/Hovd
    {  289, 4086,   68300, 18, }, // Asia/Irkutsk
    {  289, 6969,   68318, 16, }, // Asia/Jakarta
    {  289, 6955,   68334, 16, }, // Asia/Jayapura
    {  289, 1257,   68350, 12, }, // Asia/Jerusalem
    {  289, 3462,   68362, 10, }, // Asia/Kabul
    {  289, 4218,   68372, 18, }, // Asia/Kamchatka
    {  289, 4141,   68390, 14, }, // Asia/Karachi
    {  289, 1152,   68404, 18, }, // Asia/Kathmandu
    {  289, 8624,   68422, 14, }, // Asia/Khandyga
    {  289,  992,   68436, 14, }, // Asia/Kolkata
    {  289, 4099,   68450, 28, }, // Asia/Krasnoyarsk
    {  289, 4761,   68478, 27, }, // Asia/Kuala_Lumpur
    {  289, 5893,   68505, 10, }, // Asia/Kuching
    {  289, 5464,   68515, 14, }, // Asia/Kuwait
    {  289, 1178,   68529, 12, }, // Asia/Macau
    {  289, 3919,   68541, 14, }, // Asia/Magadan
    {  289, 1316,   68555, 16, }, // Asia/Makassar
    {  289, 7145,   68571, 12, }, // Asia/Manila
    {  289, 6883,   68583, 14, }, // Asia/Muscat
    {  289, 2446,   68597, 16, }, // Asia/Nicosia
    {  289, 8638,   68613, 28, }, // Asia/Novokuznetsk
    {  289, 4037,   68641, 24, }, // Asia/Novosibirsk
    {  289, 4131,   68665, 12, }, // Asia/Omsk
    {  289, 7108,   68677,  8, }, // Asia/Oral
    {  289, 6924,   68685, 17, }, // Asia/Phnom_Penh
    {  289, 8656,   68702, 20, }, // Asia/Pontianak
    {  289, 4116,   68722, 16, }, // Asia/Pyongyang
    {  289, 5476,   68738, 10, }, // Asia/Qatar
    {  289, 8671,   68748, 16, }, // Asia/Qostanay
    {  289, 4171,   68764, 18, }, // Asia/Qyzylorda
    {  289, 3486,   68782, 12, }, // Asia/Riyadh
    {  289, 4310,   68794, 16, }, // Asia/Sakhalin
    {  289, 7208,   68810, 20, }, // Asia/Samarkand
    {  289, 3116,   68830, 10, }, // Asia/Seoul
    {  289, 1020,   68840, 12, }, // Asia/Shanghai
    {  289, 3137,   68852, 18, }, // Asia/Singapore
    {  289, 4199,   68870, 24, }, // Asia/Srednekolymsk
    {  289, 3100,   68894, 12, }, // Asia/Taipei
    {  289, 4683,   68906, 16, }, // Asia/Tashkent
    {  289, 3817,   68922, 18, }, // Asia/Tbilisi
    {  289, 2638,   68940, 16, }, // Asia/Tehran
    {  289, 1284,   68956, 10, }, // Asia/Thimphu
    {  289, 2687,   68966, 12, }, // Asia/Tokyo
    {  289, 4474,   68978, 14, }, // Asia/Tomsk
    {  289, 1346,   68992, 18, }, // Asia/Ulaanbaatar
    {  289, 1126,   69010, 12, }, // Asia/Urumqi
    {  289, 8685,   69022, 17, }, // Asia/Ust-Nera
    {  289, 6940,   69039, 18, }, // Asia/Vientiane
    {  289, 4612,   69057, 28, }, // Asia/Vladivostok
    {  289, 4730,   69085, 18, }, // Asia/Yakutsk
    {  289, 1202,   69103, 14, }, // Asia/Yangon
    {  289, 3785,   69117, 28, }, // Asia/Yekaterinburg
    {  289, 3649,   69145, 16, }, // Asia/Yerevan
    {  289, 3565,   69161, 14, }, // Atlantic/Azores
    {  289, 5588,   69175, 16, }, // Atlantic/Bermuda
    {  289, 6591,   69191, 12, }, // Atlantic/Canary
    {  289, 3629,   69203, 21, }, // Atlantic/Cape_Verde
    {  289, 1379,   69224, 10, }, // Atlantic/Faroe
    {  289, 8699,   69234, 14, }, // Atlantic/Madeira
    {  289, 2614,   69248, 18, }, // Atlantic/Reykjavik
    {  289, 7235,   69266, 25, }, // Atlantic/South_Georgia
    {  289, 6784,   69291, 23, }, // Atlantic/St_Helena
    {  289, 6607,   69314, 16, }, // Atlantic/Stanley
    {  289, 1654,   69330, 16, }, // Australia/Adelaide
    {  289, 1619,   69346, 20, }, // Australia/Brisbane
    {  289, 1783,   69366, 21, }, // Australia/Broken_Hill
    {  289, 1567,   69387, 14, }, // Australia/Darwin
    {  289, 3539,   69401, 12, }, // Australia/Eucla
    {  289, 1500,   69413, 16, }, // Australia/Hobart
    {  289, 8716,   69429, 20, }, // Australia/Lindeman
    {  289, 1531,   69449, 19, }, // Australia/Lord_Howe
    {  289, 1711,   69468, 18, }, // Australia/Melbourne
    {  289, 1746,   69486, 10, }, // Australia/Perth
    {  289, 1447,   69496, 12, }, // Australia/Sydney
    {  289, 6390,   69508, 22, }, // Europe/Amsterdam
    {  289, 6149,   69530, 14, }, // Europe/Andorra
    {  289, 3522,   69544, 18, }, // Europe/Astrakhan
    {  289, 6577,   69562, 12, }, // Europe/Athens
    {  289, 6285,   69574, 18, }, // Europe/Belgrade
    {  289, 4659,   69592, 12, }, // Europe/Berlin
    {  289, 6437,   69604, 22, }, // Europe/Bratislava
    {  289, 6178,   69626, 16, }, // Europe/Brussels
    {  289, 3830,   69642, 18, }, // Europe/Bucharest
    {  289, 3708,   69660, 16, }, // Europe/Budapest
    {  289, 8751,   69676, 16, }, // Europe/Busingen
    {  289, 2475,   69692, 18, }, // Europe/Chisinau
    {  289, 6238,   69710, 18, }, // Europe/Copenhagen
    {  289, 2260,   69728, 14, }, // Europe/Dublin
    {  289, 6256,   69742, 20, }, // Europe/Gibraltar
    {  289, 8767,   69762, 16, }, // Europe/Guernsey
    {  289, 6561,   69778, 16, }, // Europe/Helsinki
    {  289, 8783,   69794, 23, }, // Europe/Isle_of_Man
    {  289, 1097,   69817, 16, }, // Europe/Istanbul
    {  289, 8802,   69833, 12, }, // Europe/Jersey
    {  289, 3881,   69845, 18, }, // Europe/Kaliningrad
    {  289, 8924,   69863, 14, }, // Europe/Kirov
    {  289, 2419,   69877, 12, }, // Europe/Kyiv
    {  289, 3078,   69889, 14, }, // Europe/Lisbon
    {  289, 6455,   69903, 20, }, // Europe/Ljubljana
    {  289, 2393,   69923, 12, }, // Europe/London
    {  289, 6314,   69935, 22, }, // Europe/Luxembourg
    {  289, 4794,   69957, 14, }, // Europe/Madrid
    {  289, 6346,   69971, 10, }, // Europe/Malta
    {  289, 6531,   69981, 26, }, // Europe/Mariehamn
    {  289, 3595,   70007, 14, }, // Europe/Minsk
    {  289, 6359,   70021, 14, }, // Europe/Monaco
    {  289, 3443,   70035, 10, }, // Europe/Moscow
    {  289, 6407,   70045, 10, }, // Europe/Oslo
    {  289, 4186,   70055,  8, }, // Europe/Paris
    {  289, 6373,   70063, 20, }, // Europe/Podgorica
    {  289, 6224,   70083, 14, }, // Europe/Prague
    {  289, 8816,   70097,  8, }, // Europe/Riga
    {  289, 6273,   70105, 10, }, // Europe/Rome
    {  289, 4233,   70115, 12, }, // Europe/Samara
    {  289, 6419,   70127, 19, }, // Europe/San_Marino
    {  289, 6194,   70146, 18, }, // Europe/Sarajevo
    {  289, 4353,   70164, 16, }, // Europe/Saratov
    {  289, 8828,   70180, 20, }, // Europe/Simferopol
    {  289, 6332,   70200, 18, }, // Europe/Skopje
    {  289, 6548,   70218, 12, }, // Europe/Sofia
    {  289, 6472,   70230, 18, }, // Europe/Stockholm
    {  289, 8846,   70248, 14, }, // Europe/Tallinn
    {  289, 6135,   70262, 12, }, // Europe/Tirane
    {  289, 8861,   70274, 22, }, // Europe/Ulyanovsk
    {  289, 6301,   70296, 14, }, // Europe/Vaduz
    {  289, 6516,   70310, 14, }, // Europe/Vatican
    {  289, 6164,   70324, 14, }, // Europe/Vienna
    {  289, 8878,   70338, 14, }, // Europe/Vilnius
    {  289, 4629,   70352, 20, }, // Europe/Volgograd
    {  289, 3055,   70372, 10, }, // Europe/Warsaw
    {  289, 6210,   70382, 14, }, // Europe/Zagreb
    {  289, 6489,   70396, 10, }, // Europe/Zurich
    {  289, 4980,   70406, 28, }, // Indian/Antananarivo
    {  289, 6910,   70434, 10, }, // Indian/Chagos
    {  289, 5979,   70444, 18, }, // Indian/Christmas
    {  289, 5996,   70462, 12, }, // Indian/Cocos
    {  289, 4931,   70474, 14, }, // Indian/Comoro
    {  289, 6624,   70488, 16, }, // Indian/Kerguelen
    {  289, 7223,   70504, 10, }, // Indian/Mahe
    {  289, 7013,   70514, 18, }, // Indian/Maldives
    {  289, 3971,   70532, 14, }, // Indian/Mauritius
    {  289, 5000,   70546, 14, }, // Indian/Mayotte
    {  289, 7174,   70560, 16, }, // Indian/Reunion
    {  289, 4324,   70576, 12, }, // Pacific/Apia
    {  289,  932,   70588, 14, }, // Pacific/Auckland
    {  289, 3608,   70602, 18, }, // Pacific/Bougainville
    {  289, 2863,   70620, 12, }, // Pacific/Chatham
    {  289, 3022,   70632, 10, }, // Pacific/Chuuk
    {  289, 2201,   70642, 10, }, // Pacific/Easter
    {  289, 7323,   70652, 10, }, // Pacific/Efate
    {  289, 7290,   70662, 14, }, // Pacific/Fakaofo
    {  289, 3804,   70676, 10, }, // Pacific/Fiji
    {  289, 7306,   70686, 18, }, // Pacific/Funafuti
    {  289, 6641,   70704, 18, }, // Pacific/Galapagos
    {  289, 6659,   70722, 16, }, // Pacific/Gambier
    {  289, 3724,   70738, 24, }, // Pacific/Guadalcanal
    {  289, 5935,   70762, 12, }, // Pacific/Guam
    {  289, 2897,   70774, 20, }, // Pacific/Kanton
    {  289, 3900,   70794, 20, }, // Pacific/Kiritimati
    {  289, 6982,   70814, 12, }, // Pacific/Kosrae
    {  289, 2708,   70826, 20, }, // Pacific/Kwajalein
    {  289, 4779,   70846, 14, }, // Pacific/Majuro
    {  289, 3953,   70860, 18, }, // Pacific/Marquesas
    {  289, 8909,   70878, 12, }, // Pacific/Midway
    {  289, 7047,   70890, 10, }, // Pacific/Nauru
    {  289, 7095,   70900, 10, }, // Pacific/Niue
    {  289, 4070,   70910, 16, }, // Pacific/Norfolk
    {  289, 7061,   70926, 12, }, // Pacific/Noumea
    {  289, 2991,   70938, 17, }, // Pacific/Pago_Pago
    {  289, 7118,   70955, 10, }, // Pacific/Palau
    {  289, 7157,   70965, 18, }, // Pacific/Pitcairn
    {  289, 2961,   70983, 12, }, // Pacific/Pohnpei
    {  289, 4709,   70995, 25, }, // Pacific/Port_Moresby
    {  289, 6009,   71020, 18, }, // Pacific/Rarotonga
    {  289, 5948,   71038, 14, }, // Pacific/Saipan
    {  289, 7275,   71052, 14, }, // Pacific/Tahiti
    {  289, 6675,   71066, 14, }, // Pacific/Tarawa
    {  289, 4485,   71080, 20, }, // Pacific/Tongatapu
    {  289, 7355,   71100, 10, }, // Pacific/Wake
    {  289, 7368,   71110, 14, }, // Pacific/Wallis
    {  307, 4961,   71124, 11, }, // Africa/Addis_Ababa Gaelic/Latin/United Kingdom
    {  307,   14,   71135,  6, }, // Africa/Asmara
    {  307, 5032,   71141, 12, }, // Africa/Dar_es_Salaam
    {  307, 4945,   71153, 10, }, // Africa/Djibouti
    {  307, 5068,   62570,  8, }, // Africa/El_Aaiun
    {  307, 4381,   71163, 11, }, // Africa/Johannesburg
    {  307, 6832,   18802,  4, }, // Africa/Lome
    {  307, 5173,   71174,  9, }, // Africa/Ndjamena
    {  307, 4337,       6,  8, }, // Africa/Sao_Tome
    {  307, 5524,   71183,  9, }, // America/Anguilla
    {  307, 5541,   71192,  8, }, // America/Antigua
    {  307, 4456,   24992,  9, }, // America/Araguaina
    {  307,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  307, 7452,   42641, 12, }, // America/Argentina/Rio_Gallegos
    {  307, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  307, 5557,   71200,  5, }, // America/Aruba
    {  307, 4154,      14,  8, }, // America/Asuncion
    {  307,  237,      22,  8, }, // America/Atikokan
    {  307, 7586,      30, 17, }, // America/Bahia_Banderas
    {  307, 7609,   34376,  5, }, // America/Belem
    {  307, 5309,   71205, 10, }, // America/Belize
    {  307, 4263,   18898,  6, }, // America/Bogota
    {  307, 3770,      47,  6, }, // America/Cancun
    {  307, 5398,   71215,  7, }, // America/Cayman
    {  307, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  307, 5324,   71222, 11, }, // America/Costa_Rica
    {  307, 3693,   25007,  6, }, // America/Cuiaba
    {  307, 5640,      66,  7, }, // America/Curacao
    {  307, 5656,   71233, 10, }, // America/Dominica
    {  307, 7832,   25013,  8, }, // America/Eirunepe
    {  307, 5343,   71243, 12, }, // America/El_Salvador
    {  307, 7887,   71255,  9, }, // America/Glace_Bay
    {  307, 4514,   71264, 12, }, // America/Grand_Turk
    {  307, 5687,   71276,  8, }, // America/Grenada
    {  307, 5703,   18917,  8, }, // America/Guadeloupe
    {  307, 3662,   71284,  9, }, // America/Guatemala
    {  307, 6895,   71293,  9, }, // America/Guyana
    {  307,  481,      73, 13, }, // America/Indiana/Knox
    {  307, 7924,      86, 16, }, // America/Indiana/Marengo
    {  307, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  307, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  307, 8001,     139, 14, }, // America/Indiana/Vevay
    {  307, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  307, 8049,     171, 16, }, // America/Indiana/Winamac
    {  307, 8073,   71302,  7, }, // America/Inuvik
    {  307, 2665,   71309,  8, }, // America/Jamaica
    {  307, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  307, 5849,     207, 22, }, // America/Lower_Princes
    {  307, 8116,   25021,  6, }, // America/Maceio
    {  307, 5722,   71317,  9, }, // America/Martinique
    {  307, 2779,   25036,  8, }, // America/Mazatlan
    {  307, 8183,     229,  6, }, // America/Merida
    {  307, 2811,   71326, 17, }, // America/Mexico_City
    {  307, 5741,   71343,  9, }, // America/Montserrat
    {  307, 3294,   71352, 13, }, // America/New_York
    {  307, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  307, 8292,     255, 20, }, // America/North_Dakota/Center
    {  307, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  307,  393,     298,  4, }, // America/Nuuk
    {  307, 5871,   71365, 15, }, // America/Port_of_Spain
    {  307, 5760,   71380, 11, }, // America/Puerto_Rico
    {  307, 8387,   71391, 11, }, // America/Rankin_Inlet
    {  307, 8423,   71402, 10, }, // America/Resolute
    {  307, 8440,   25137,  8, }, // America/Santarem
    {  307, 1862,   25145,  9, }, // America/Sao_Paulo
    {  307, 6862,     302, 16, }, // America/Scoresbysund
    {  307, 8471,   34416, 16, }, // America/St_Barthelemy
    {  307, 2045,     332, 10, }, // America/St_Johns
    {  307, 5780,   71412, 14, }, // America/St_Kitts
    {  307, 5797,   71426, 12, }, // America/St_Lucia
    {  307,  855,     360, 10, }, // America/St_Thomas
    {  307, 5830,   71438, 15, }, // America/St_Vincent
    {  307, 5673,   25229,  7, }, // America/Thule
    {  307, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  307, 8531,   71453, 17, }, // Antarctica/Macquarie
    {  307, 3870,   71470,  5, }, // Asia/Amman
    {  307, 5270,   55725,  6, }, // Asia/Aqtobe
    {  307,  964,   42826,  7, }, // Asia/Ashgabat
    {  307, 3509,   71475,  7, }, // Asia/Baghdad
    {  307, 5451,   71482,  8, }, // Asia/Bahrain
    {  307, 3680,   31917,  6, }, // Asia/Bishkek
    {  307, 5906,   71490,  8, }, // Asia/Brunei
    {  307, 6092,   31939,  7, }, // Asia/Dushanbe
    {  307, 8614,   71498,  4, }, // Asia/Gaza
    {  307, 1226,   71502, 19, }, // Asia/Ho_Chi_Minh
    {  307, 4673,   25278,  5, }, // Asia/Hovd
    {  307, 1257,   71521,  9, }, // Asia/Jerusalem
    {  307, 4141,   71530,  6, }, // Asia/Karachi
    {  307, 1152,     413,  9, }, // Asia/Kathmandu
    {  307,  992,     422,  7, }, // Asia/Kolkata
    {  307, 5464,   71536,  8, }, // Asia/Kuwait
    {  307, 1178,   71544,  7, }, // Asia/Macau
    {  307, 4116,   71551, 10, }, // Asia/Pyongyang
    {  307, 5476,   71561,  5, }, // Asia/Qatar
    {  307, 8671,   71566,  8, }, // Asia/Qostanay
    {  307, 4171,   20943,  9, }, // Asia/Qyzylorda
    {  307, 3137,   71574,  9, }, // Asia/Singapore
    {  307, 4683,   71583,  7, }, // Asia/Tashkent
    {  307, 3817,   71590,  8, }, // Asia/Tbilisi
    {  307, 2638,   71598,  6, }, // Asia/Tehran
    {  307, 2687,   62084,  5, }, // Asia/Tokyo
    {  307, 1126,   25383,  6, }, // Asia/Urumqi
    {  307, 6940,   71604, 10, }, // Asia/Vientiane
    {  307, 1202,    6213,  6, }, // Asia/Yangon
    {  307, 3565,   71614, 13, }, // Atlantic/Azores
    {  307, 5588,   71627,  8, }, // Atlantic/Bermuda
    {  307, 6591,   71635, 22, }, // Atlantic/Canary
    {  307, 3629,   71657, 13, }, // Atlantic/Cape_Verde
    {  307, 1379,   71670,  4, }, // Atlantic/Faroe
    {  307, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  307, 7235,   71674, 14, }, // Atlantic/South_Georgia
    {  307, 6784,   71688, 19, }, // Atlantic/St_Helena
    {  307, 1447,   71707,  5, }, // Australia/Sydney
    {  307, 6577,   71712,  9, }, // Europe/Athens
    {  307, 6178,   71721, 12, }, // Europe/Brussels
    {  307, 2475,   62164,  8, }, // Europe/Chisinau
    {  307, 6238,   71733,  6, }, // Europe/Copenhagen
    {  307, 2260,   71739, 17, }, // Europe/Dublin
    {  307, 6256,   71756, 10, }, // Europe/Gibraltar
    {  307, 8767,   71766, 10, }, // Europe/Guernsey
    {  307, 8783,   71776, 15, }, // Europe/Isle_of_Man
    {  307, 8802,   71791,  9, }, // Europe/Jersey
    {  307, 2419,    6486,  4, }, // Europe/Kyiv
    {  307, 2393,   71800, 20, }, // Europe/London
    {  307, 6314,   71820, 10, }, // Europe/Luxembourg
    {  307, 3443,   71830,  5, }, // Europe/Moscow
    {  307, 4186,   71835,  5, }, // Europe/Paris
    {  307, 6224,   71840,  4, }, // Europe/Prague
    {  307, 8816,   71844,  4, }, // Europe/Riga
    {  307, 6273,   71848,  8, }, // Europe/Rome
    {  307, 6548,   21329,  6, }, // Europe/Sofia
    {  307, 6135,    6599,  6, }, // Europe/Tirane
    {  307, 6516,   71856, 11, }, // Europe/Vatican
    {  307, 6489,    4478,  6, }, // Europe/Zurich
    {  307, 5979,   71867,  7, }, // Indian/Christmas
    {  307, 6624,   34782,  9, }, // Indian/Kerguelen
    {  307, 7223,   25565,  4, }, // Indian/Mahe
    {  307, 7013,   71874, 24, }, // Indian/Maldives
    {  307, 3971,   71898, 24, }, // Indian/Mauritius
    {  307, 7174,     467,  7, }, // Indian/Reunion
    {  307, 3022,     474,  5, }, // Pacific/Chuuk
    {  307, 2201,   71922,  8, }, // Pacific/Easter
    {  307, 3804,   71930,  4, }, // Pacific/Fiji
    {  307, 6641,   19672,  9, }, // Pacific/Galapagos
    {  307, 6659,   71934,  9, }, // Pacific/Gambier
    {  307, 2929,    4512,  8, }, // Pacific/Honolulu
    {  307, 2897,    4520,  9, }, // Pacific/Kanton
    {  307, 3953,   71943, 20, }, // Pacific/Marquesas
    {  307, 7047,   71963,  6, }, // Pacific/Nauru
    {  307, 7118,   71969,  6, }, // Pacific/Palau
    {  307, 7157,   71975, 14, }, // Pacific/Pitcairn
    {  307, 2961,     479,  7, }, // Pacific/Pohnpei
    {  307, 7368,   71989,  6, }, // Pacific/Wallis
    {  308, 6723,    4821,  2, }, // Africa/Accra Ga/Latin/Ghana
    {  308,   14,       0,  6, }, // Africa/Asmara
    {  308, 2242,   24913,  5, }, // Africa/Cairo
    {  308, 4019,   19744, 10, }, // Africa/Casablanca
    {  308, 6736,   24918,  7, }, // Africa/Conakry
    {  308, 6766,   71995, 10, }, // Africa/Nouakchott
    {  308, 4337,       6,  8, }, // Africa/Sao_Tome
    {  308, 3183,   72005,  8, }, // America/Anchorage
    {  308, 4154,      14,  8, }, // America/Asuncion
    {  308,  237,      22,  8, }, // America/Atikokan
    {  308, 7586,      30, 17, }, // America/Bahia_Banderas
    {  308, 7623,    4897, 12, }, // America/Blanc-Sablon
    {  308, 7676,   72013, 25, }, // America/Cambridge_Bay
    {  308, 3770,    4959,  6, }, // America/Cancun
    {  308, 5398,   20006,  6, }, // America/Cayman
    {  308, 3251,   72038,  7, }, // America/Chicago
    {  308, 7719,   72045,  9, }, // America/Chihuahua
    {  308, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  308, 5324,   20026, 10, }, // America/Costa_Rica
    {  308, 7759,    5020,  7, }, // America/Creston
    {  308, 5640,    5033,  7, }, // America/Curacao
    {  308, 7811,   72054, 21, }, // America/Dawson_Creek
    {  308, 5656,   20071,  8, }, // America/Dominica
    {  308, 7849,   72075, 10, }, // America/Fort_Nelson
    {  308, 7887,   72085, 21, }, // America/Glace_Bay
    {  308, 6844,   72106, 21, }, // America/Goose_Bay
    {  308, 4514,   72127, 10, }, // America/Grand_Turk
    {  308, 1923,   72137,  7, }, // America/Halifax
    {  308,  481,   72144, 13, }, // America/Indiana/Knox
    {  308, 7924,      86, 16, }, // America/Indiana/Marengo
    {  308, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  308, 7975,   72157, 19, }, // America/Indiana/Tell_City
    {  308, 8001,     139, 14, }, // America/Indiana/Vevay
    {  308, 8023,   72176, 18, }, // America/Indiana/Vincennes
    {  308, 8049,   72194, 16, }, // America/Indiana/Winamac
    {  308,  660,    5310,  7, }, // America/Iqaluit
    {  308, 2665,    3892,  7, }, // America/Jamaica
    {  308, 8088,   72210, 19, }, // America/Kentucky/Monticello
    {  308, 5849,   72229, 26, }, // America/Lower_Princes
    {  308, 5722,    5400,  8, }, // America/Martinique
    {  308, 8183,     229,  6, }, // America/Merida
    {  308, 2811,   72255, 10, }, // America/Mexico_City
    {  308, 4293,    5434,  7, }, // America/Miquelon
    {  308, 8217,    5441,  7, }, // America/Moncton
    {  308, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  308, 8292,   72265, 23, }, // America/North_Dakota/Center
    {  308, 8320,   72288, 26, }, // America/North_Dakota/New_Salem
    {  308,  393,     298,  4, }, // America/Nuuk
    {  308, 3847,   72314, 21, }, // America/Port-au-Prince
    {  308, 5871,   72335, 18, }, // America/Port_of_Spain
    {  308, 5760,   20463, 11, }, // America/Puerto_Rico
    {  308, 8387,   72353, 15, }, // America/Rankin_Inlet
    {  308, 6862,   72368, 15, }, // America/Scoresbysund
    {  308, 8471,     318, 14, }, // America/St_Barthelemy
    {  308, 2045,     332, 10, }, // America/St_Johns
    {  308, 5780,     342,  9, }, // America/St_Kitts
    {  308, 5797,   72383,  9, }, // America/St_Lucia
    {  308,  855,     360, 10, }, // America/St_Thomas
    {  308, 5830,   72392, 11, }, // America/St_Vincent
    {  308, 8493,   72403, 12, }, // America/Swift_Current
    {  308, 5363,    5759, 11, }, // America/Tegucigalpa
    {  308, 2077,   72415,  9, }, // America/Vancouver
    {  308, 2128,   72424, 10, }, // America/Whitehorse
    {  308, 5918,   72434,  5, }, // Antarctica/Casey
    {  308, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  308, 8531,   72439,  8, }, // Antarctica/Macquarie
    {  308, 7076,   72447,  8, }, // Antarctica/McMurdo
    {  308, 5440,    5906,  5, }, // Asia/Aqtau
    {  308, 5270,    5911,  6, }, // Asia/Aqtobe
    {  308, 5963,   72455, 10, }, // Asia/Choibalsan
    {  308, 4413,    5949,  7, }, // Asia/Colombo
    {  308, 4442,   25264,  7, }, // Asia/Damascus
    {  308, 1226,   72465, 20, }, // Asia/Ho_Chi_Minh
    {  308, 1257,   72485,  9, }, // Asia/Jerusalem
    {  308, 4141,   72494,  7, }, // Asia/Karachi
    {  308, 1152,     413,  9, }, // Asia/Kathmandu
    {  308,  992,     422,  7, }, // Asia/Kolkata
    {  308, 5893,   72501,  7, }, // Asia/Kuching
    {  308, 1178,    6064,  5, }, // Asia/Macau
    {  308, 6883,    4227,  6, }, // Asia/Muscat
    {  308, 2446,   25331,  7, }, // Asia/Nicosia
    {  308, 5476,    4233,  4, }, // Asia/Qatar
    {  308, 8671,     434,  8, }, // Asia/Qostanay
    {  308, 4171,   25347,  9, }, // Asia/Qyzylorda
    {  308, 1126,   72508,  6, }, // Asia/Urumqi
    {  308, 1202,     442,  6, }, // Asia/Yangon
    {  308, 3629,   21064, 10, }, // Atlantic/Cape_Verde
    {  308, 1379,     448,  5, }, // Atlantic/Faroe
    {  308, 6784,     453, 10, }, // Atlantic/St_Helena
    {  308, 1783,   72514, 15, }, // Australia/Broken_Hill
    {  308, 3539,   21132,  5, }, // Australia/Eucla
    {  308, 1531,   72529, 11, }, // Australia/Lord_Howe
    {  308, 2419,     463,  4, }, // Europe/Kyiv
    {  308, 6910,   72540,  6, }, // Indian/Chagos
    {  308, 5979,   72546,  7, }, // Indian/Christmas
    {  308, 5996,    4484,  5, }, // Indian/Cocos
    {  308, 4931,   21395,  6, }, // Indian/Comoro
    {  308, 7174,     467,  7, }, // Indian/Reunion
    {  308,  932,   72553,  6, }, // Pacific/Auckland
    {  308, 2863,   72559,  7, }, // Pacific/Chatham
    {  308, 3022,   72566,  5, }, // Pacific/Chuuk
    {  308, 3724,    6711, 11, }, // Pacific/Guadalcanal
    {  308, 3953,   72571,  9, }, // Pacific/Marquesas
    {  308, 7157,   72580,  8, }, // Pacific/Pitcairn
    {  308, 2961,     479,  7, }, // Pacific/Pohnpei
    {  309, 6723,   72588,  4, }, // Africa/Accra Galician/Latin/Spain
    {  309, 4961,   18754, 10, }, // Africa/Addis_Ababa
    {  309, 7383,   72592,  5, }, // Africa/Algiers
    {  309,   14,       0,  6, }, // Africa/Asmara
    {  309,   44,   72597,  6, }, // Africa/Bamako
    {  309, 2242,   72603,  7, }, // Africa/Cairo
    {  309, 4945,   24925,  7, }, // Africa/Djibouti
    {  309, 5068,   72610,  6, }, // Africa/El_Aaiun
    {  309, 4381,   72616, 12, }, // Africa/Johannesburg
    {  309, 4426,   72628,  7, }, // Africa/Khartoum
    {  309, 6832,   18802,  4, }, // Africa/Lome
    {  309, 4903,   72635,  6, }, // Africa/Lusaka
    {  309, 5015,   72641,  8, }, // Africa/Mogadishu
    {  309, 5173,   42632,  9, }, // Africa/Ndjamena
    {  309, 6690,   24967,  8, }, // Africa/Ouagadougou
    {  309, 4337,   72649,  8, }, // Africa/Sao_Tome
    {  309, 2732,   18851,  7, }, // Africa/Tripoli
    {  309, 6503,   72657,  5, }, // Africa/Tunis
    {  309, 5524,   18863,  7, }, // America/Anguilla
    {  309, 4456,   24992,  9, }, // America/Araguaina
    {  309,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  309, 7452,   42641, 12, }, // America/Argentina/Rio_Gallegos
    {  309, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  309, 4154,      14,  8, }, // America/Asuncion
    {  309,  237,      22,  8, }, // America/Atikokan
    {  309, 3581,   72662,  4, }, // America/Bahia
    {  309, 7586,      30, 17, }, // America/Bahia_Banderas
    {  309, 7609,   34376,  5, }, // America/Belem
    {  309, 4263,   18898,  6, }, // America/Bogota
    {  309, 3770,      47,  6, }, // America/Cancun
    {  309, 4247,   42671,  6, }, // America/Cayenne
    {  309, 5398,   72666, 12, }, // America/Cayman
    {  309, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  309, 3693,   25007,  6, }, // America/Cuiaba
    {  309, 5640,      66,  7, }, // America/Curacao
    {  309, 7832,   25013,  8, }, // America/Eirunepe
    {  309, 5343,   72678, 10, }, // America/El_Salvador
    {  309, 5687,   18910,  7, }, // America/Grenada
    {  309, 5703,   18917,  9, }, // America/Guadeloupe
    {  309, 6895,   72688,  6, }, // America/Guyana
    {  309, 2221,   72694,  8, }, // America/Havana
    {  309,  348,   72702, 12, }, // America/Indiana/Indianapolis
    {  309,  481,      73, 13, }, // America/Indiana/Knox
    {  309, 7924,      86, 16, }, // America/Indiana/Marengo
    {  309, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  309, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  309, 8001,     139, 14, }, // America/Indiana/Vevay
    {  309, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  309, 8049,     171, 16, }, // America/Indiana/Winamac
    {  309, 2665,   18935,  7, }, // America/Jamaica
    {  309, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  309, 4278,   72714,  5, }, // America/La_Paz
    {  309, 3390,   72719, 10, }, // America/Los_Angeles
    {  309, 5849,     207, 22, }, // America/Lower_Princes
    {  309, 8116,   25021,  6, }, // America/Maceio
    {  309, 5722,   18945,  9, }, // America/Martinique
    {  309, 2779,   25036,  8, }, // America/Mazatlan
    {  309, 8183,     229,  6, }, // America/Merida
    {  309, 2811,   72729, 16, }, // America/Mexico_City
    {  309, 3294,   42698,  9, }, // America/New_York
    {  309, 8264,   72745, 23, }, // America/North_Dakota/Beulah
    {  309, 8292,   72768, 23, }, // America/North_Dakota/Center
    {  309, 8320,   72791, 26, }, // America/North_Dakota/New_Salem
    {  309,  393,     298,  4, }, // America/Nuuk
    {  309, 5413,   19056,  6, }, // America/Panama
    {  309, 3847,   72817, 14, }, // America/Port-au-Prince
    {  309, 5871,   72831, 12, }, // America/Port_of_Spain
    {  309, 5760,   63375, 10, }, // America/Puerto_Rico
    {  309,  695,   72843, 10, }, // America/Rio_Branco
    {  309, 8440,   25137,  8, }, // America/Santarem
    {  309, 1862,   25145,  9, }, // America/Sao_Paulo
    {  309, 6862,     302, 16, }, // America/Scoresbysund
    {  309, 8471,   34416, 16, }, // America/St_Barthelemy
    {  309, 2045,   25170, 12, }, // America/St_Johns
    {  309, 5780,   25182, 11, }, // America/St_Kitts
    {  309, 5797,   72853, 11, }, // America/St_Lucia
    {  309,  855,   42797, 12, }, // America/St_Thomas
    {  309, 5830,   72864, 11, }, // America/St_Vincent
    {  309, 5605,   19114,  7, }, // America/Tortola
    {  309, 6066,   55662, 16, }, // Antarctica/DumontDUrville
    {  309, 7258,   63470,  5, }, // Antarctica/Syowa
    {  309, 5487,   72875,  4, }, // Asia/Aden
    {  309, 5297,    5890,  6, }, // Asia/Almaty
    {  309, 3870,   19121,  4, }, // Asia/Amman
    {  309, 5440,    5906,  5, }, // Asia/Aqtau
    {  309, 5270,    5911,  6, }, // Asia/Aqtobe
    {  309,  964,   72879,  9, }, // Asia/Ashgabat
    {  309, 3509,    4063,  6, }, // Asia/Baghdad
    {  309, 3555,   72888,  4, }, // Asia/Baku
    {  309, 4503,   72892,  5, }, // Asia/Chita
    {  309, 4442,   72897,  7, }, // Asia/Damascus
    {  309, 4697,   19162,  6, }, // Asia/Hebron
    {  309, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  309, 6969,   72904,  7, }, // Asia/Jakarta
    {  309, 1257,   19212,  9, }, // Asia/Jerusalem
    {  309, 3462,   72911,  5, }, // Asia/Kabul
    {  309, 1152,   19221,  8, }, // Asia/Kathmandu
    {  309, 8624,   56553,  8, }, // Asia/Khandyga
    {  309,  992,   19229,  7, }, // Asia/Kolkata
    {  309, 1178,    4222,  5, }, // Asia/Macau
    {  309, 6883,   19248,  7, }, // Asia/Muscat
    {  309, 8671,   42895,  8, }, // Asia/Qostanay
    {  309, 4171,   25347,  9, }, // Asia/Qyzylorda
    {  309, 3486,    6120,  4, }, // Asia/Riyadh
    {  309, 4310,   42903,  8, }, // Asia/Sakhalin
    {  309, 7208,   19277, 10, }, // Asia/Samarkand
    {  309, 3116,   19287,  4, }, // Asia/Seoul
    {  309, 3137,   19291,  8, }, // Asia/Singapore
    {  309, 4199,    6150, 13, }, // Asia/Srednekolymsk
    {  309, 2638,   56658,  7, }, // Asia/Tehran
    {  309, 1126,   25383,  6, }, // Asia/Urumqi
    {  309, 4730,   42945,  7, }, // Asia/Yakutsk
    {  309, 1202,     442,  6, }, // Asia/Yangon
    {  309, 3785,   72916, 13, }, // Asia/Yekaterinburg
    {  309, 3649,   72929,  7, }, // Asia/Yerevan
    {  309, 5588,   72936, 14, }, // Atlantic/Bermuda
    {  309, 6591,   72950, 14, }, // Atlantic/Canary
    {  309, 3629,   25418, 10, }, // Atlantic/Cape_Verde
    {  309, 1379,   19383,  5, }, // Atlantic/Faroe
    {  309, 2614,   72964, 10, }, // Atlantic/Reykjavik
    {  309, 7235,   72974, 14, }, // Atlantic/South_Georgia
    {  309, 6784,   25447, 12, }, // Atlantic/St_Helena
    {  309, 1654,   19421,  8, }, // Australia/Adelaide
    {  309, 1447,   21149,  6, }, // Australia/Sydney
    {  309, 6390,   72988,  9, }, // Europe/Amsterdam
    {  309, 3522,   72997,  8, }, // Europe/Astrakhan
    {  309, 6577,   25459,  6, }, // Europe/Athens
    {  309, 6285,   57443,  8, }, // Europe/Belgrade
    {  309, 4659,   19450,  6, }, // Europe/Berlin
    {  309, 6178,   73005,  8, }, // Europe/Brussels
    {  309, 3830,   19464,  8, }, // Europe/Bucharest
    {  309, 6238,   19472, 10, }, // Europe/Copenhagen
    {  309, 2260,   19482,  6, }, // Europe/Dublin
    {  309, 6256,   19488,  9, }, // Europe/Gibraltar
    {  309, 6561,   73013,  9, }, // Europe/Helsinki
    {  309, 8783,   73022, 11, }, // Europe/Isle_of_Man
    {  309, 3881,   61551, 12, }, // Europe/Kaliningrad
    {  309, 2419,   43070,  4, }, // Europe/Kyiv
    {  309, 3078,   19528,  6, }, // Europe/Lisbon
    {  309, 6455,   19534,  9, }, // Europe/Ljubljana
    {  309, 2393,   19543,  7, }, // Europe/London
    {  309, 6314,   73033, 10, }, // Europe/Luxembourg
    {  309, 6359,   73043,  6, }, // Europe/Monaco
    {  309, 3443,   73049,  7, }, // Europe/Moscow
    {  309, 4186,   19571,  5, }, // Europe/Paris
    {  309, 6224,   19576,  5, }, // Europe/Prague
    {  309, 6273,   19581,  4, }, // Europe/Rome
    {  309, 6194,   73056,  8, }, // Europe/Sarajevo
    {  309, 8828,   73064, 10, }, // Europe/Simferopol
    {  309, 6548,   19594,  5, }, // Europe/Sofia
    {  309, 6472,   73074,  9, }, // Europe/Stockholm
    {  309, 6135,   19608,  6, }, // Europe/Tirane
    {  309, 8861,   25542,  9, }, // Europe/Ulyanovsk
    {  309, 6516,   73083,  8, }, // Europe/Vatican
    {  309, 6164,   19625,  5, }, // Europe/Vienna
    {  309, 4629,   61732, 10, }, // Europe/Volgograd
    {  309, 3055,   19639,  8, }, // Europe/Warsaw
    {  309, 6489,    4478,  6, }, // Europe/Zurich
    {  309, 5979,   73091, 14, }, // Indian/Christmas
    {  309, 4931,   63916,  7, }, // Indian/Comoro
    {  309, 7223,   25565,  4, }, // Indian/Mahe
    {  309, 7013,   73105,  8, }, // Indian/Maldives
    {  309, 3971,   73113,  8, }, // Indian/Mauritius
    {  309, 7174,   19655,  7, }, // Indian/Reunion
    {  309, 3022,     474,  5, }, // Pacific/Chuuk
    {  309, 2201,   73121, 14, }, // Pacific/Easter
    {  309, 3804,   19668,  4, }, // Pacific/Fiji
    {  309, 6641,   73135, 15, }, // Pacific/Galapagos
    {  309, 2929,   73150,  8, }, // Pacific/Honolulu
    {  309, 2897,    4520,  9, }, // Pacific/Kanton
    {  309, 2961,     479,  7, }, // Pacific/Pohnpei
    {  309, 7275,   43161,  6, }, // Pacific/Tahiti
    {  310,   14,       0,  6, }, // Africa/Asmara Ganda/Latin/Uganda
    {  310, 4337,       6,  8, }, // Africa/Sao_Tome
    {  310, 4154,      14,  8, }, // America/Asuncion
    {  310,  237,      22,  8, }, // America/Atikokan
    {  310, 7586,      30, 17, }, // America/Bahia_Banderas
    {  310, 3770,      47,  6, }, // America/Cancun
    {  310, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  310, 5640,      66,  7, }, // America/Curacao
    {  310,  481,      73, 13, }, // America/Indiana/Knox
    {  310, 7924,      86, 16, }, // America/Indiana/Marengo
    {  310, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  310, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  310, 8001,     139, 14, }, // America/Indiana/Vevay
    {  310, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  310, 8049,     171, 16, }, // America/Indiana/Winamac
    {  310, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  310, 5849,     207, 22, }, // America/Lower_Princes
    {  310, 8183,     229,  6, }, // America/Merida
    {  310, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  310, 8292,     255, 20, }, // America/North_Dakota/Center
    {  310, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  310,  393,     298,  4, }, // America/Nuuk
    {  310, 6862,     302, 16, }, // America/Scoresbysund
    {  310, 8471,     318, 14, }, // America/St_Barthelemy
    {  310, 2045,     332, 10, }, // America/St_Johns
    {  310, 5780,     342,  9, }, // America/St_Kitts
    {  310, 5797,     351,  9, }, // America/St_Lucia
    {  310,  855,     360, 10, }, // America/St_Thomas
    {  310, 5830,     370, 11, }, // America/St_Vincent
    {  310, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  310, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  310, 1152,     413,  9, }, // Asia/Kathmandu
    {  310,  992,     422,  7, }, // Asia/Kolkata
    {  310, 1178,     429,  5, }, // Asia/Macau
    {  310, 1202,     442,  6, }, // Asia/Yangon
    {  310, 1379,     448,  5, }, // Atlantic/Faroe
    {  310, 6784,     453, 10, }, // Atlantic/St_Helena
    {  310, 2419,     463,  4, }, // Europe/Kyiv
    {  310, 7174,     467,  7, }, // Indian/Reunion
    {  310, 3022,     474,  5, }, // Pacific/Chuuk
    {  310, 2961,     479,  7, }, // Pacific/Pohnpei
    {  311,   14,       0,  6, }, // Africa/Asmara Geez/Ethiopic/Ethiopia
    {  311, 4337,       6,  8, }, // Africa/Sao_Tome
    {  311, 4154,      14,  8, }, // America/Asuncion
    {  311,  237,      22,  8, }, // America/Atikokan
    {  311, 7586,      30, 17, }, // America/Bahia_Banderas
    {  311, 3770,      47,  6, }, // America/Cancun
    {  311, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  311, 5640,      66,  7, }, // America/Curacao
    {  311,  481,      73, 13, }, // America/Indiana/Knox
    {  311, 7924,      86, 16, }, // America/Indiana/Marengo
    {  311, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  311, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  311, 8001,     139, 14, }, // America/Indiana/Vevay
    {  311, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  311, 8049,     171, 16, }, // America/Indiana/Winamac
    {  311, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  311, 5849,     207, 22, }, // America/Lower_Princes
    {  311, 8183,     229,  6, }, // America/Merida
    {  311, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  311, 8292,     255, 20, }, // America/North_Dakota/Center
    {  311, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  311,  393,     298,  4, }, // America/Nuuk
    {  311, 6862,     302, 16, }, // America/Scoresbysund
    {  311, 8471,     318, 14, }, // America/St_Barthelemy
    {  311, 2045,     332, 10, }, // America/St_Johns
    {  311, 5780,     342,  9, }, // America/St_Kitts
    {  311, 5797,     351,  9, }, // America/St_Lucia
    {  311,  855,     360, 10, }, // America/St_Thomas
    {  311, 5830,     370, 11, }, // America/St_Vincent
    {  311, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  311, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  311, 1152,     413,  9, }, // Asia/Kathmandu
    {  311,  992,     422,  7, }, // Asia/Kolkata
    {  311, 1178,     429,  5, }, // Asia/Macau
    {  311, 1202,     442,  6, }, // Asia/Yangon
    {  311, 1379,     448,  5, }, // Atlantic/Faroe
    {  311, 6784,     453, 10, }, // Atlantic/St_Helena
    {  311, 2419,     463,  4, }, // Europe/Kyiv
    {  311, 7174,     467,  7, }, // Indian/Reunion
    {  311, 3022,     474,  5, }, // Pacific/Chuuk
    {  311, 2961,     479,  7, }, // Pacific/Pohnpei
    {  313, 6751,   73158,  7, }, // Africa/Abidjan Georgian/Georgian/Georgia
    {  313, 6723,   73165,  4, }, // Africa/Accra
    {  313, 4961,   73169, 10, }, // Africa/Addis_Ababa
    {  313, 7383,   73179,  6, }, // Africa/Algiers
    {  313,   14,   73185,  6, }, // Africa/Asmara
    {  313,   44,   73191,  6, }, // Africa/Bamako
    {  313, 5159,   73197,  5, }, // Africa/Bangui
    {  313, 6709,   73202,  7, }, // Africa/Banjul
    {  313, 7398,   73209,  5, }, // Africa/Bissau
    {  313, 4859,   73214,  8, }, // Africa/Blantyre
    {  313, 5189,   73222,  9, }, // Africa/Brazzaville
    {  313, 4824,   73231,  9, }, // Africa/Bujumbura
    {  313, 2242,   73240,  5, }, // Africa/Cairo
    {  313, 4019,   73245, 10, }, // Africa/Casablanca
    {  313, 7412,   73255,  5, }, // Africa/Ceuta
    {  313, 6736,   73260,  7, }, // Africa/Conakry
    {  313, 6803,   73267,  6, }, // Africa/Dakar
    {  313, 5032,   73273, 13, }, // Africa/Dar_es_Salaam
    {  313, 4945,   73286,  6, }, // Africa/Djibouti
    {  313, 5145,   73292,  5, }, // Africa/Douala
    {  313, 5068,   73297,  9, }, // Africa/El_Aaiun
    {  313, 6816,   73306,  8, }, // Africa/Freetown
    {  313, 4808,   73314,  8, }, // Africa/Gaborone
    {  313, 4917,   73322,  6, }, // Africa/Harare
    {  313, 4381,   73328, 12, }, // Africa/Johannesburg
    {  313, 4401,   73340,  4, }, // Africa/Juba
    {  313, 5053,   73344,  7, }, // Africa/Kampala
    {  313, 4426,   73351,  7, }, // Africa/Khartoum
    {  313, 4875,   73358,  6, }, // Africa/Kigali
    {  313, 5208,   73364,  7, }, // Africa/Kinshasa
    {  313, 4646,   73371,  6, }, // Africa/Lagos
    {  313, 5238,   73377,  9, }, // Africa/Libreville
    {  313, 6832,   73386,  4, }, // Africa/Lome
    {  313, 5113,   73390,  6, }, // Africa/Luanda
    {  313, 4841,   73396,  9, }, // Africa/Lubumbashi
    {  313, 4903,   73405,  6, }, // Africa/Lusaka
    {  313, 5224,   73411,  6, }, // Africa/Malabo
    {  313, 4889,   73417,  6, }, // Africa/Maputo
    {  313, 5099,   73423,  6, }, // Africa/Maseru
    {  313, 5084,   73429,  7, }, // Africa/Mbabane
    {  313, 5015,   73436,  8, }, // Africa/Mogadishu
    {  313, 6997,   73444,  8, }, // Africa/Monrovia
    {  313, 3755,   73452,  7, }, // Africa/Nairobi
    {  313, 5173,   73459,  7, }, // Africa/Ndjamena
    {  313, 5256,   73466,  6, }, // Africa/Niamey
    {  313, 6766,   73472,  8, }, // Africa/Nouakchott
    {  313, 6690,   73480,  8, }, // Africa/Ouagadougou
    {  313, 5127,   73488, 10, }, // Africa/Porto-Novo
    {  313, 4337,   73498,  8, }, // Africa/Sao_Tome
    {  313, 2732,   73506,  7, }, // Africa/Tripoli
    {  313, 6503,   73513,  6, }, // Africa/Tunis
    {  313, 4054,   73519,  8, }, // Africa/Windhoek
    {  313,  132,   73527,  5, }, // America/Adak
    {  313, 3183,   73532,  8, }, // America/Anchorage
    {  313, 5524,   73540,  7, }, // America/Anguilla
    {  313, 5541,   73547,  7, }, // America/Antigua
    {  313, 4456,   73554,  9, }, // America/Araguaina
    {  313,  166,   73563, 13, }, // America/Argentina/Buenos_Aires
    {  313,   91,   73576,  9, }, // America/Argentina/Catamarca
    {  313,  270,   73585,  7, }, // America/Argentina/Cordoba
    {  313,  441,   73592,  5, }, // America/Argentina/Jujuy
    {  313, 7425,   73597,  8, }, // America/Argentina/La_Rioja
    {  313,  565,   73605,  7, }, // America/Argentina/Mendoza
    {  313, 7452,   73612, 12, }, // America/Argentina/Rio_Gallegos
    {  313, 7483,   73624,  5, }, // America/Argentina/Salta
    {  313, 7507,   73629,  9, }, // America/Argentina/San_Juan
    {  313, 5497,   73638,  9, }, // America/Argentina/San_Luis
    {  313, 7534,   73647,  8, }, // America/Argentina/Tucuman
    {  313, 7560,   73655,  6, }, // America/Argentina/Ushuaia
    {  313, 5557,   73661,  5, }, // America/Aruba
    {  313, 4154,   73666,  9, }, // America/Asuncion
    {  313,  237,   73675,  9, }, // America/Atikokan
    {  313, 3581,   73684,  4, }, // America/Bahia
    {  313, 7586,   73688, 15, }, // America/Bahia_Banderas
    {  313, 5571,   73703,  9, }, // America/Barbados
    {  313, 7609,   73712,  6, }, // America/Belem
    {  313, 5309,   73718,  6, }, // America/Belize
    {  313, 7623,   73724, 12, }, // America/Blanc-Sablon
    {  313, 7644,   73736,  9, }, // America/Boa_Vista
    {  313, 4263,   73745,  6, }, // America/Bogota
    {  313, 7662,   73751,  5, }, // America/Boise
    {  313, 7676,   73756, 11, }, // America/Cambridge_Bay
    {  313, 7698,   73767, 12, }, // America/Campo_Grande
    {  313, 3770,   73779,  7, }, // America/Cancun
    {  313, 4596,   73786,  8, }, // America/Caracas
    {  313, 4247,   73794,  6, }, // America/Cayenne
    {  313, 5398,   73800,  7, }, // America/Cayman
    {  313, 3251,   73807,  6, }, // America/Chicago
    {  313, 7719,   73813,  8, }, // America/Chihuahua
    {  313, 7737,   73821, 14, }, // America/Ciudad_Juarez
    {  313, 5324,   73835, 10, }, // America/Costa_Rica
    {  313, 7759,   73845,  8, }, // America/Creston
    {  313, 3693,   73853,  6, }, // America/Cuiaba
    {  313, 5640,   73859,  8, }, // America/Curacao
    {  313, 7775,   73867, 12, }, // America/Danmarkshavn
    {  313, 7796,   73879,  7, }, // America/Dawson
    {  313, 7811,   73886, 12, }, // America/Dawson_Creek
    {  313,  805,   73898,  7, }, // America/Denver
    {  313, 3351,   73905,  8, }, // America/Detroit
    {  313, 5656,   73913,  8, }, // America/Dominica
    {  313,  893,   73921,  9, }, // America/Edmonton
    {  313, 7832,   73930,  8, }, // America/Eirunepe
    {  313, 5343,   73938,  9, }, // America/El_Salvador
    {  313, 7849,   73947, 12, }, // America/Fort_Nelson
    {  313, 7869,   73959,  9, }, // America/Fortaleza
    {  313, 7887,   73968,  9, }, // America/Glace_Bay
    {  313, 6844,   73977,  7, }, // America/Goose_Bay
    {  313, 4514,   73984, 11, }, // America/Grand_Turk
    {  313, 5687,   73995,  7, }, // America/Grenada
    {  313, 5703,   74002,  9, }, // America/Guadeloupe
    {  313, 3662,   74011,  9, }, // America/Guatemala
    {  313, 4743,   74020,  9, }, // America/Guayaquil
    {  313, 6895,   74029,  6, }, // America/Guyana
    {  313, 1923,   74035,  9, }, // America/Halifax
    {  313, 2221,   74044,  6, }, // America/Havana
    {  313, 7905,   74050,  9, }, // America/Hermosillo
    {  313,  348,   74059, 13, }, // America/Indiana/Indianapolis
    {  313,  481,   74072, 14, }, // America/Indiana/Knox
    {  313, 7924,   74086, 16, }, // America/Indiana/Marengo
    {  313, 7948,   74102, 20, }, // America/Indiana/Petersburg
    {  313, 7975,   74122, 17, }, // America/Indiana/Tell_City
    {  313, 8001,   74139, 14, }, // America/Indiana/Vevay
    {  313, 8023,   74153, 16, }, // America/Indiana/Vincennes
    {  313, 8049,   74169, 17, }, // America/Indiana/Winamac
    {  313, 8073,   74186,  7, }, // America/Inuvik
    {  313,  660,   74193,  8, }, // America/Iqaluit
    {  313, 2665,   74201,  7, }, // America/Jamaica
    {  313, 5282,   74208,  4, }, // America/Juneau
    {  313,  521,   74212,  8, }, // America/Kentucky/Louisville
    {  313, 8088,   74220, 18, }, // America/Kentucky/Monticello
    {  313, 5621,   74238, 11, }, // America/Kralendijk
    {  313, 4278,   74249,  7, }, // America/La_Paz
    {  313, 7132,   74256,  4, }, // America/Lima
    {  313, 3390,   74260, 12, }, // America/Los_Angeles
    {  313, 5849,   74272, 19, }, // America/Lower_Princes
    {  313, 8116,   74291,  6, }, // America/Maceio
    {  313, 8131,   74297,  7, }, // America/Managua
    {  313, 1892,   74304,  7, }, // America/Manaus
    {  313, 5814,   74311,  6, }, // America/Marigot
    {  313, 5722,   74317,  9, }, // America/Martinique
    {  313, 8147,   74326, 10, }, // America/Matamoros
    {  313, 2779,   74336,  9, }, // America/Mazatlan
    {  313, 8165,   74345,  8, }, // America/Menominee
    {  313, 8183,   74353,  6, }, // America/Merida
    {  313, 8198,   74359, 10, }, // America/Metlakatla
    {  313, 2811,   74369,  6, }, // America/Mexico_City
    {  313, 4293,   74375,  8, }, // America/Miquelon
    {  313, 8217,   74383,  8, }, // America/Moncton
    {  313, 8233,   74391,  8, }, // America/Monterrey
    {  313, 4000,   74399, 10, }, // America/Montevideo
    {  313, 5741,   74409,  9, }, // America/Montserrat
    {  313, 5383,   74418,  5, }, // America/Nassau
    {  313, 3294,   74423,  9, }, // America/New_York
    {  313, 8251,   74347,  4, }, // America/Nome
    {  313, 1834,   74432,  7, }, // America/Noronha
    {  313, 8264,   74439, 23, }, // America/North_Dakota/Beulah
    {  313, 8292,   74462, 24, }, // America/North_Dakota/Center
    {  313, 8320,   74486, 28, }, // America/North_Dakota/New_Salem
    {  313,  393,   74514,  7, }, // America/Nuuk
    {  313, 8351,   74521,  7, }, // America/Ojinaga
    {  313, 5413,   74528,  6, }, // America/Panama
    {  313, 6106,   74534, 10, }, // America/Paramaribo
    {  313, 3224,   74544,  7, }, // America/Phoenix
    {  313, 3847,   74551, 13, }, // America/Port-au-Prince
    {  313, 5871,   74564, 14, }, // America/Port_of_Spain
    {  313, 8367,   74578, 11, }, // America/Porto_Velho
    {  313, 5760,   74589, 11, }, // America/Puerto_Rico
    {  313, 3932,   74600, 13, }, // America/Punta_Arenas
    {  313, 8387,   74613, 13, }, // America/Rankin_Inlet
    {  313, 8408,   74626,  6, }, // America/Recife
    {  313, 1979,   74632,  6, }, // America/Regina
    {  313, 8423,   74638,  8, }, // America/Resolute
    {  313,  695,   74646, 10, }, // America/Rio_Branco
    {  313, 8440,   74656,  9, }, // America/Santarem
    {  313, 2165,   74665,  8, }, // America/Santiago
    {  313, 6044,   74673, 13, }, // America/Santo_Domingo
    {  313, 1862,   74686,  9, }, // America/Sao_Paulo
    {  313, 6862,   74695, 12, }, // America/Scoresbysund
    {  313, 8457,   74707,  5, }, // America/Sitka
    {  313, 8471,   74712, 13, }, // America/St_Barthelemy
    {  313, 2045,   74725, 10, }, // America/St_Johns
    {  313, 5780,   74735, 10, }, // America/St_Kitts
    {  313, 5797,   74745, 10, }, // America/St_Lucia
    {  313,  855,   74755, 10, }, // America/St_Thomas
    {  313, 5830,   74765, 12, }, // America/St_Vincent
    {  313, 8493,   74777, 13, }, // America/Swift_Current
    {  313, 5363,   74790, 11, }, // America/Tegucigalpa
    {  313, 5673,   74801,  4, }, // America/Thule
    {  313,  313,   74805,  7, }, // America/Tijuana
    {  313,  608,   74812,  7, }, // America/Toronto
    {  313, 5605,   74819,  7, }, // America/Tortola
    {  313, 2077,   74826,  9, }, // America/Vancouver
    {  313, 2128,   74835,  9, }, // America/Whitehorse
    {  313,  734,   74844,  8, }, // America/Winnipeg
    {  313, 8515,   74852,  8, }, // America/Yakutat
    {  313, 5918,   74860,  5, }, // Antarctica/Casey
    {  313, 6027,   74865,  6, }, // Antarctica/Davis
    {  313, 6066,   74871, 15, }, // Antarctica/DumontDUrville
    {  313, 8531,   74886,  7, }, // Antarctica/Macquarie
    {  313, 7029,   74893,  7, }, // Antarctica/Mawson
    {  313, 7076,   74900,  8, }, // Antarctica/McMurdo
    {  313, 8552,   74908,  7, }, // Antarctica/Palmer
    {  313, 7189,   74915,  6, }, // Antarctica/Rothera
    {  313, 7258,   74921,  5, }, // Antarctica/Syowa
    {  313, 8570,   74926,  5, }, // Antarctica/Troll
    {  313, 7337,   74931,  7, }, // Antarctica/Vostok
    {  313, 1413,   74938, 11, }, // Arctic/Longyearbyen
    {  313, 5487,   74949,  5, }, // Asia/Aden
    {  313, 5297,   74954,  6, }, // Asia/Almaty
    {  313, 3870,   74960,  5, }, // Asia/Amman
    {  313, 5428,   74965,  7, }, // Asia/Anadyr
    {  313, 5440,   74972,  5, }, // Asia/Aqtau
    {  313, 5270,   74977,  6, }, // Asia/Aqtobe
    {  313,  964,   74983,  8, }, // Asia/Ashgabat
    {  313, 8587,   74991,  6, }, // Asia/Atyrau
    {  313, 3509,   74997,  7, }, // Asia/Baghdad
    {  313, 5451,   75004,  8, }, // Asia/Bahrain
    {  313, 3555,   75012,  4, }, // Asia/Baku
    {  313, 4368,   75016,  8, }, // Asia/Bangkok
    {  313, 3473,   75024,  8, }, // Asia/Barnaul
    {  313, 3988,   75032,  7, }, // Asia/Beirut
    {  313, 3680,   75039,  7, }, // Asia/Bishkek
    {  313, 5906,   75046,  6, }, // Asia/Brunei
    {  313, 4503,   75052,  4, }, // Asia/Chita
    {  313, 5963,   75056, 10, }, // Asia/Choibalsan
    {  313, 4413,   75066,  7, }, // Asia/Colombo
    {  313, 4442,   75073,  7, }, // Asia/Damascus
    {  313, 1060,   73267,  4, }, // Asia/Dhaka
    {  313, 6125,   75080,  4, }, // Asia/Dili
    {  313, 3498,   75084,  5, }, // Asia/Dubai
    {  313, 6092,   75089,  7, }, // Asia/Dushanbe
    {  313, 8599,   75096,  9, }, // Asia/Famagusta
    {  313, 8614,   75105,  4, }, // Asia/Gaza
    {  313, 4697,   75109,  7, }, // Asia/Hebron
    {  313, 1226,   75116,  8, }, // Asia/Ho_Chi_Minh
    {  313, 2576,   75124,  8, }, // Asia/Hong_Kong
    {  313, 4673,   75132,  5, }, // Asia/Hovd
    {  313, 4086,   75137,  8, }, // Asia/Irkutsk
    {  313, 6969,   75145,  7, }, // Asia/Jakarta
    {  313, 6955,   75152,  8, }, // Asia/Jayapura
    {  313, 1257,   75160, 10, }, // Asia/Jerusalem
    {  313, 3462,   75170,  6, }, // Asia/Kabul
    {  313, 4218,   75176,  8, }, // Asia/Kamchatka
    {  313, 4141,   75184,  6, }, // Asia/Karachi
    {  313, 1152,   75190,  8, }, // Asia/Kathmandu
    {  313, 8624,   75198,  7, }, // Asia/Khandyga
    {  313,  992,   75205,  7, }, // Asia/Kolkata
    {  313, 4099,   75212, 12, }, // Asia/Krasnoyarsk
    {  313, 4761,   75224, 13, }, // Asia/Kuala_Lumpur
    {  313, 5893,   75237,  7, }, // Asia/Kuching
    {  313, 5464,   75244,  7, }, // Asia/Kuwait
    {  313, 1178,   75251,  5, }, // Asia/Macau
    {  313, 3919,   75256,  8, }, // Asia/Magadan
    {  313, 1316,   75264,  8, }, // Asia/Makassar
    {  313, 7145,   75272,  6, }, // Asia/Manila
    {  313, 6883,   75278,  7, }, // Asia/Muscat
    {  313, 2446,   75285,  7, }, // Asia/Nicosia
    {  313, 8638,   75292, 12, }, // Asia/Novokuznetsk
    {  313, 4037,   75304, 12, }, // Asia/Novosibirsk
    {  313, 4131,   75316,  5, }, // Asia/Omsk
    {  313, 7108,   75321,  5, }, // Asia/Oral
    {  313, 6924,   75326,  8, }, // Asia/Phnom_Penh
    {  313, 8656,   75334, 10, }, // Asia/Pontianak
    {  313, 4116,   75344,  8, }, // Asia/Pyongyang
    {  313, 5476,   75352,  6, }, // Asia/Qatar
    {  313, 8671,   75358,  8, }, // Asia/Qostanay
    {  313, 4171,   75366,  9, }, // Asia/Qyzylorda
    {  313, 3486,   75375,  8, }, // Asia/Riyadh
    {  313, 4310,   75383,  8, }, // Asia/Sakhalin
    {  313, 7208,   75391, 10, }, // Asia/Samarkand
    {  313, 3116,   75401,  5, }, // Asia/Seoul
    {  313, 1020,   75406,  6, }, // Asia/Shanghai
    {  313, 3137,   75412,  9, }, // Asia/Singapore
    {  313, 4199,   75421, 14, }, // Asia/Srednekolymsk
    {  313, 3100,   75435,  6, }, // Asia/Taipei
    {  313, 4683,   75441,  8, }, // Asia/Tashkent
    {  313, 3817,   75449,  7, }, // Asia/Tbilisi
    {  313, 2638,   75456,  7, }, // Asia/Tehran
    {  313, 1284,   75463,  7, }, // Asia/Thimphu
    {  313, 2687,   75470,  5, }, // Asia/Tokyo
    {  313, 4474,   75475,  6, }, // Asia/Tomsk
    {  313, 1346,   75481, 10, }, // Asia/Ulaanbaatar
    {  313, 1126,   75491,  6, }, // Asia/Urumqi
    {  313, 8685,   75497,  8, }, // Asia/Ust-Nera
    {  313, 6940,   75505,  9, }, // Asia/Vientiane
    {  313, 4612,   75514, 12, }, // Asia/Vladivostok
    {  313, 4730,   75526,  8, }, // Asia/Yakutsk
    {  313, 1202,   75534,  7, }, // Asia/Yangon
    {  313, 3785,   75541, 13, }, // Asia/Yekaterinburg
    {  313, 3649,   75554,  7, }, // Asia/Yerevan
    {  313, 3565,   75561, 16, }, // Atlantic/Azores
    {  313, 5588,   75577,  7, }, // Atlantic/Bermuda
    {  313, 6591,   75584,  6, }, // Atlantic/Canary
    {  313, 3629,   75590, 10, }, // Atlantic/Cape_Verde
    {  313, 1379,   75600, 17, }, // Atlantic/Faroe
    {  313, 8699,   75617,  7, }, // Atlantic/Madeira
    {  313, 2614,   75624, 10, }, // Atlantic/Reykjavik
    {  313, 7235,   75634, 14, }, // Atlantic/South_Georgia
    {  313, 6784,   75648, 18, }, // Atlantic/St_Helena
    {  313, 6607,   75666,  6, }, // Atlantic/Stanley
    {  313, 1654,   75672,  8, }, // Australia/Adelaide
    {  313, 1619,   75680,  9, }, // Australia/Brisbane
    {  313, 1783,   75689, 12, }, // Australia/Broken_Hill
    {  313, 1567,   75701,  7, }, // Australia/Darwin
    {  313, 3539,   75708,  5, }, // Australia/Eucla
    {  313, 1500,   75713,  7, }, // Australia/Hobart
    {  313, 8716,   75720,  8, }, // Australia/Lindeman
    {  313, 1531,   75728,  9, }, // Australia/Lord_Howe
    {  313, 1711,   75737,  8, }, // Australia/Melbourne
    {  313, 1746,   75745,  5, }, // Australia/Perth
    {  313, 1447,   75750,  6, }, // Australia/Sydney
    {  313, 6390,   75756, 10, }, // Europe/Amsterdam
    {  313, 6149,   75766,  6, }, // Europe/Andorra
    {  313, 3522,   75772,  9, }, // Europe/Astrakhan
    {  313, 6577,   75781,  5, }, // Europe/Athens
    {  313, 6285,   75786,  8, }, // Europe/Belgrade
    {  313, 4659,   75794,  7, }, // Europe/Berlin
    {  313, 6437,   75801, 10, }, // Europe/Bratislava
    {  313, 6178,   75811,  8, }, // Europe/Brussels
    {  313, 3830,   75819,  9, }, // Europe/Bucharest
    {  313, 3708,   75828,  9, }, // Europe/Budapest
    {  313, 8751,   75837,  9, }, // Europe/Busingen
    {  313, 2475,   75846,  9, }, // Europe/Chisinau
    {  313, 6238,   75855, 11, }, // Europe/Copenhagen
    {  313, 2260,   75866,  7, }, // Europe/Dublin
    {  313, 6256,   75873, 10, }, // Europe/Gibraltar
    {  313, 8767,   75883,  6, }, // Europe/Guernsey
    {  313, 6561,   75889,  8, }, // Europe/Helsinki
    {  313, 8783,   75897, 13, }, // Europe/Isle_of_Man
    {  313, 1097,   75910,  8, }, // Europe/Istanbul
    {  313, 8802,   75918,  5, }, // Europe/Jersey
    {  313, 3881,   75923, 12, }, // Europe/Kaliningrad
    {  313, 8924,   75935,  6, }, // Europe/Kirov
    {  313, 2419,   75941,  5, }, // Europe/Kyiv
    {  313, 3078,   75946,  8, }, // Europe/Lisbon
    {  313, 6455,   75954,  9, }, // Europe/Ljubljana
    {  313, 2393,   75963,  7, }, // Europe/London
    {  313, 6314,   75970, 11, }, // Europe/Luxembourg
    {  313, 4794,   75981,  7, }, // Europe/Madrid
    {  313, 6346,   75988,  5, }, // Europe/Malta
    {  313, 6531,   75993,  9, }, // Europe/Mariehamn
    {  313, 3595,   76002,  6, }, // Europe/Minsk
    {  313, 6359,   76008,  6, }, // Europe/Monaco
    {  313, 3443,   76014,  7, }, // Europe/Moscow
    {  313, 6407,   76021,  4, }, // Europe/Oslo
    {  313, 4186,   76025,  6, }, // Europe/Paris
    {  313, 6373,   76031,  9, }, // Europe/Podgorica
    {  313, 6224,   76040,  5, }, // Europe/Prague
    {  313, 8816,   76045,  4, }, // Europe/Riga
    {  313, 6273,   76049,  4, }, // Europe/Rome
    {  313, 4233,   76053,  6, }, // Europe/Samara
    {  313, 6419,   76059, 10, }, // Europe/San_Marino
    {  313, 6194,   76069,  7, }, // Europe/Sarajevo
    {  313, 4353,   76076,  8, }, // Europe/Saratov
    {  313, 8828,   76084, 11, }, // Europe/Simferopol
    {  313, 6332,   76095,  6, }, // Europe/Skopje
    {  313, 6548,   76101,  5, }, // Europe/Sofia
    {  313, 6472,   76106,  9, }, // Europe/Stockholm
    {  313, 8846,   76115,  6, }, // Europe/Tallinn
    {  313, 6135,   76121,  6, }, // Europe/Tirane
    {  313, 8861,   76127, 10, }, // Europe/Ulyanovsk
    {  313, 6301,   76137,  6, }, // Europe/Vaduz
    {  313, 6516,   76143,  8, }, // Europe/Vatican
    {  313, 6164,   76151,  4, }, // Europe/Vienna
    {  313, 8878,   76155,  8, }, // Europe/Vilnius
    {  313, 4629,   76163, 10, }, // Europe/Volgograd
    {  313, 3055,   76173,  7, }, // Europe/Warsaw
    {  313, 6210,   76180,  7, }, // Europe/Zagreb
    {  313, 6489,   76187,  7, }, // Europe/Zurich
    {  313, 4980,   76194, 12, }, // Indian/Antananarivo
    {  313, 6910,   76206,  6, }, // Indian/Chagos
    {  313, 5979,   76212, 13, }, // Indian/Christmas
    {  313, 5996,   76225,  6, }, // Indian/Cocos
    {  313, 4931,   76231,  6, }, // Indian/Comoro
    {  313, 6624,   76237,  9, }, // Indian/Kerguelen
    {  313, 7223,   76246,  4, }, // Indian/Mahe
    {  313, 7013,   76250,  9, }, // Indian/Maldives
    {  313, 3971,   76259,  7, }, // Indian/Mauritius
    {  313, 5000,   76266,  6, }, // Indian/Mayotte
    {  313, 7174,   76272,  9, }, // Indian/Reunion
    {  313, 4324,   76281,  4, }, // Pacific/Apia
    {  313,  932,   76285,  7, }, // Pacific/Auckland
    {  313, 3608,   76292,  9, }, // Pacific/Bougainville
    {  313, 2863,   76301,  6, }, // Pacific/Chatham
    {  313, 3022,   76307,  5, }, // Pacific/Chuuk
    {  313, 2201,   76312,  6, }, // Pacific/Easter
    {  313, 7323,   76318,  5, }, // Pacific/Efate
    {  313, 7290,   76323,  7, }, // Pacific/Fakaofo
    {  313, 3804,   76330,  4, }, // Pacific/Fiji
    {  313, 7306,   76334,  8, }, // Pacific/Funafuti
    {  313, 6641,   76342, 10, }, // Pacific/Galapagos
    {  313, 6659,   76352,  8, }, // Pacific/Gambier
    {  313, 3724,   76360, 12, }, // Pacific/Guadalcanal
    {  313, 5935,   76372,  5, }, // Pacific/Guam
    {  313, 2929,   76377,  8, }, // Pacific/Honolulu
    {  313, 2897,   76385,  9, }, // Pacific/Kanton
    {  313, 3900,   76394, 10, }, // Pacific/Kiritimati
    {  313, 6982,   76404,  6, }, // Pacific/Kosrae
    {  313, 2708,   76410, 10, }, // Pacific/Kwajalein
    {  313, 4779,   76420,  6, }, // Pacific/Majuro
    {  313, 3953,   76426,  9, }, // Pacific/Marquesas
    {  313, 8909,   76435,  6, }, // Pacific/Midway
    {  313, 7047,   76441,  5, }, // Pacific/Nauru
    {  313, 7095,   76446,  4, }, // Pacific/Niue
    {  313, 4070,   76450,  8, }, // Pacific/Norfolk
    {  313, 7061,   76458,  5, }, // Pacific/Noumea
    {  313, 2991,   76463,  9, }, // Pacific/Pago_Pago
    {  313, 7118,   76472,  5, }, // Pacific/Palau
    {  313, 7157,   76477,  8, }, // Pacific/Pitcairn
    {  313, 2961,   76485,  6, }, // Pacific/Pohnpei
    {  313, 4709,   76491, 11, }, // Pacific/Port_Moresby
    {  313, 6009,   76502,  9, }, // Pacific/Rarotonga
    {  313, 5948,   76511,  7, }, // Pacific/Saipan
    {  313, 7275,   76518,  5, }, // Pacific/Tahiti
    {  313, 6675,   76523,  6, }, // Pacific/Tarawa
    {  313, 4485,   76529,  9, }, // Pacific/Tongatapu
    {  313, 7355,   76538,  5, }, // Pacific/Wake
    {  313, 7368,   76543,  6, }, // Pacific/Wallis
    {  314, 4961,    3793, 11, }, // Africa/Addis_Ababa German/Latin/Germany
    {  314, 7383,   57088,  6, }, // Africa/Algiers
    {  314,   14,       0,  6, }, // Africa/Asmara
    {  314, 2242,   24913,  5, }, // Africa/Cairo
    {  314, 5032,   76549, 10, }, // Africa/Dar_es_Salaam
    {  314, 4945,   76559,  9, }, // Africa/Djibouti
    {  314, 5068,   62570,  8, }, // Africa/El_Aaiun
    {  314, 4426,   24937,  7, }, // Africa/Khartoum
    {  314, 6832,   18802,  4, }, // Africa/Lome
    {  314, 5015,   76568, 10, }, // Africa/Mogadishu
    {  314, 5173,   42632,  9, }, // Africa/Ndjamena
    {  314, 5127,   24975, 10, }, // Africa/Porto-Novo
    {  314, 4337,       6,  8, }, // Africa/Sao_Tome
    {  314, 2732,   56289,  8, }, // Africa/Tripoli
    {  314,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  314, 4154,      14,  8, }, // America/Asuncion
    {  314,  237,      22,  8, }, // America/Atikokan
    {  314, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  314, 4263,   18898,  6, }, // America/Bogota
    {  314, 3770,      47,  6, }, // America/Cancun
    {  314, 5398,   76578, 12, }, // America/Cayman
    {  314, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  314, 5640,      66,  7, }, // America/Curacao
    {  314, 2221,   61885,  7, }, // America/Havana
    {  314,  481,      73, 13, }, // America/Indiana/Knox
    {  314, 7924,      86, 16, }, // America/Indiana/Marengo
    {  314, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  314, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  314, 8001,     139, 14, }, // America/Indiana/Vevay
    {  314, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  314, 8049,     171, 16, }, // America/Indiana/Winamac
    {  314, 2665,    3892,  7, }, // America/Jamaica
    {  314, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  314, 5849,     207, 22, }, // America/Lower_Princes
    {  314, 8183,    3899,  6, }, // America/Merida
    {  314, 2811,   76590, 12, }, // America/Mexico_City
    {  314, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  314, 8292,     255, 20, }, // America/North_Dakota/Center
    {  314, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  314,  393,     298,  4, }, // America/Nuuk
    {  314, 1862,   25145,  9, }, // America/Sao_Paulo
    {  314, 6862,     302, 16, }, // America/Scoresbysund
    {  314, 8471,   25154, 16, }, // America/St_Barthelemy
    {  314, 2045,     332, 10, }, // America/St_Johns
    {  314, 5780,     342,  9, }, // America/St_Kitts
    {  314, 5797,     351,  9, }, // America/St_Lucia
    {  314,  855,     360, 10, }, // America/St_Thomas
    {  314, 5830,     370, 11, }, // America/St_Vincent
    {  314, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  314, 7337,    4048,  6, }, // Antarctica/Vostok
    {  314, 5270,    5911,  6, }, // Asia/Aqtobe
    {  314,  964,   42826,  7, }, // Asia/Ashgabat
    {  314, 3509,    4063,  6, }, // Asia/Baghdad
    {  314, 3680,   55740,  8, }, // Asia/Bishkek
    {  314, 5906,   76602, 17, }, // Asia/Brunei
    {  314, 4503,   76619,  7, }, // Asia/Chita
    {  314, 5963,   76626, 12, }, // Asia/Choibalsan
    {  314, 4442,    4102,  8, }, // Asia/Damascus
    {  314, 6092,   55748,  9, }, // Asia/Dushanbe
    {  314, 1226,   76638, 17, }, // Asia/Ho_Chi_Minh
    {  314, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  314, 4673,   76655,  5, }, // Asia/Hovd
    {  314, 4218,   76660, 11, }, // Asia/Kamchatka
    {  314, 4141,   76671,  9, }, // Asia/Karachi
    {  314, 1152,     413,  9, }, // Asia/Kathmandu
    {  314, 8624,   56553,  8, }, // Asia/Khandyga
    {  314,  992,   62730,  8, }, // Asia/Kolkata
    {  314, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  314, 1178,    4222,  5, }, // Asia/Macau
    {  314, 6883,   25325,  6, }, // Asia/Muscat
    {  314, 2446,   25331,  7, }, // Asia/Nicosia
    {  314, 8638,   76680, 12, }, // Asia/Novokuznetsk
    {  314, 4037,   76692, 11, }, // Asia/Novosibirsk
    {  314, 4116,   76703,  9, }, // Asia/Pyongyang
    {  314, 5476,    4233,  5, }, // Asia/Qatar
    {  314, 8671,   42895,  8, }, // Asia/Qostanay
    {  314, 4171,   55820,  9, }, // Asia/Qyzylorda
    {  314, 3486,    6120,  4, }, // Asia/Riyadh
    {  314, 4310,   56617,  8, }, // Asia/Sakhalin
    {  314, 3137,   19291,  8, }, // Asia/Singapore
    {  314, 3100,   76712,  6, }, // Asia/Taipei
    {  314, 4683,   55848,  9, }, // Asia/Tashkent
    {  314, 3817,   76718,  6, }, // Asia/Tbilisi
    {  314, 2638,    6175,  7, }, // Asia/Tehran
    {  314, 2687,    4257,  5, }, // Asia/Tokyo
    {  314, 1126,   25383,  6, }, // Asia/Urumqi
    {  314, 4612,    4262, 11, }, // Asia/Vladivostok
    {  314, 4730,    6206,  7, }, // Asia/Yakutsk
    {  314, 1202,    6213,  6, }, // Asia/Yangon
    {  314, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  314, 3649,   55880,  6, }, // Asia/Yerevan
    {  314, 3565,   57378,  6, }, // Atlantic/Azores
    {  314, 6591,   76724,  7, }, // Atlantic/Canary
    {  314, 3629,   25418, 10, }, // Atlantic/Cape_Verde
    {  314, 1379,   76731,  6, }, // Atlantic/Faroe
    {  314, 2614,   76737, 11, }, // Atlantic/Reykjavik
    {  314, 7235,   76748, 11, }, // Atlantic/South_Georgia
    {  314, 6784,     453, 10, }, // Atlantic/St_Helena
    {  314, 3522,   76759,  9, }, // Europe/Astrakhan
    {  314, 6577,    4339,  5, }, // Europe/Athens
    {  314, 6285,   25465,  7, }, // Europe/Belgrade
    {  314, 6178,   21176,  7, }, // Europe/Brussels
    {  314, 3830,   25479,  8, }, // Europe/Bucharest
    {  314, 8751,   25487,  8, }, // Europe/Busingen
    {  314, 6238,    4376, 10, }, // Europe/Copenhagen
    {  314, 8924,   76768,  5, }, // Europe/Kirov
    {  314, 2419,   76773,  4, }, // Europe/Kyiv
    {  314, 3078,    4400,  8, }, // Europe/Lisbon
    {  314, 6314,    4414,  9, }, // Europe/Luxembourg
    {  314, 3443,   56005,  6, }, // Europe/Moscow
    {  314, 6224,    6545,  4, }, // Europe/Prague
    {  314, 6273,    6554,  3, }, // Europe/Rome
    {  314, 4353,   76777,  7, }, // Europe/Saratov
    {  314, 6135,   19608,  6, }, // Europe/Tirane
    {  314, 8861,   76784,  9, }, // Europe/Ulyanovsk
    {  314, 6516,    6614,  7, }, // Europe/Vatican
    {  314, 6164,   56035,  4, }, // Europe/Vienna
    {  314, 4629,    4462,  9, }, // Europe/Volgograd
    {  314, 3055,   56048,  8, }, // Europe/Warsaw
    {  314, 6489,    4478,  6, }, // Europe/Zurich
    {  314, 5979,   76793, 15, }, // Indian/Christmas
    {  314, 4931,   76808,  7, }, // Indian/Comoro
    {  314, 7013,   76815,  9, }, // Indian/Maldives
    {  314, 7174,     467,  7, }, // Indian/Reunion
    {  314, 3022,     474,  5, }, // Pacific/Chuuk
    {  314, 2201,   76824, 10, }, // Pacific/Easter
    {  314, 3804,   76834,  7, }, // Pacific/Fiji
    {  314, 2929,    4512,  8, }, // Pacific/Honolulu
    {  314, 2897,    4520,  9, }, // Pacific/Kanton
    {  314, 2961,     479,  7, }, // Pacific/Pohnpei
    {  320, 5906,   76602,  6, }, // Asia/Brunei German/Latin/Switzerland
    {  320, 1178,     429,  5, }, // Asia/Macau
    {  320, 4353,   76841,  7, }, // Europe/Saratov
    {  321, 6751,   76848,  8, }, // Africa/Abidjan Greek/Greek/Greece
    {  321, 6723,   76856,  4, }, // Africa/Accra
    {  321, 4961,   76860, 13, }, // Africa/Addis_Ababa
    {  321, 7383,   76873,  6, }, // Africa/Algiers
    {  321,   14,   76879,  6, }, // Africa/Asmara
    {  321,   44,   76885,  7, }, // Africa/Bamako
    {  321, 5159,   76892,  7, }, // Africa/Bangui
    {  321, 6709,   76899,  8, }, // Africa/Banjul
    {  321, 7398,   76907,  7, }, // Africa/Bissau
    {  321, 4859,   76914,  9, }, // Africa/Blantyre
    {  321, 5189,   76923,  9, }, // Africa/Brazzaville
    {  321, 4824,   76932, 13, }, // Africa/Bujumbura
    {  321, 2242,   76945,  5, }, // Africa/Cairo
    {  321, 4019,   76950, 11, }, // Africa/Casablanca
    {  321, 7412,   76961,  6, }, // Africa/Ceuta
    {  321, 6736,   76967,  7, }, // Africa/Conakry
    {  321, 6803,   76974,  6, }, // Africa/Dakar
    {  321, 5032,   76980, 13, }, // Africa/Dar_es_Salaam
    {  321, 4945,   76993,  9, }, // Africa/Djibouti
    {  321, 5145,   77002,  7, }, // Africa/Douala
    {  321, 5068,   77009,  9, }, // Africa/El_Aaiun
    {  321, 6816,   77018,  8, }, // Africa/Freetown
    {  321, 4808,   77026, 10, }, // Africa/Gaborone
    {  321, 4917,   77036,  6, }, // Africa/Harare
    {  321, 4381,   77042, 15, }, // Africa/Johannesburg
    {  321, 4401,   77057,  7, }, // Africa/Juba
    {  321, 5053,   77064,  7, }, // Africa/Kampala
    {  321, 4426,   77071,  7, }, // Africa/Khartoum
    {  321, 4875,   77078,  7, }, // Africa/Kigali
    {  321, 5208,   77085,  7, }, // Africa/Kinshasa
    {  321, 4646,   77092,  6, }, // Africa/Lagos
    {  321, 5238,   77098,  9, }, // Africa/Libreville
    {  321, 6832,   77107,  4, }, // Africa/Lome
    {  321, 5113,   77111,  7, }, // Africa/Luanda
    {  321, 4841,   77118, 12, }, // Africa/Lubumbashi
    {  321, 4903,   77130,  7, }, // Africa/Lusaka
    {  321, 5224,   77137,  7, }, // Africa/Malabo
    {  321, 4889,   77144,  7, }, // Africa/Maputo
    {  321, 5099,   77151,  7, }, // Africa/Maseru
    {  321, 5084,   77158,  8, }, // Africa/Mbabane
    {  321, 5015,   77166, 11, }, // Africa/Mogadishu
    {  321, 6997,   77177,  8, }, // Africa/Monrovia
    {  321, 3755,   77185,  8, }, // Africa/Nairobi
    {  321, 5173,   77193,  8, }, // Africa/Ndjamena
    {  321, 5256,   77201,  6, }, // Africa/Niamey
    {  321, 6766,   77207,  8, }, // Africa/Nouakchott
    {  321, 6690,   77215, 14, }, // Africa/Ouagadougou
    {  321, 5127,   77229, 10, }, // Africa/Porto-Novo
    {  321, 4337,   77239,  8, }, // Africa/Sao_Tome
    {  321, 2732,   77247,  7, }, // Africa/Tripoli
    {  321, 6503,   77254,  6, }, // Africa/Tunis
    {  321, 4054,   77260,  8, }, // Africa/Windhoek
    {  321,  132,   77268,  5, }, // America/Adak
    {  321, 3183,   77273,  8, }, // America/Anchorage
    {  321, 5524,   77281,  9, }, // America/Anguilla
    {  321, 5541,   77290,  9, }, // America/Antigua
    {  321, 4456,   77299, 11, }, // America/Araguaina
    {  321,  166,   77310, 14, }, // America/Argentina/Buenos_Aires
    {  321,   91,   77324,  9, }, // America/Argentina/Catamarca
    {  321,  270,   77333,  7, }, // America/Argentina/Cordoba
    {  321,  441,   77340,  7, }, // America/Argentina/Jujuy
    {  321, 7425,   77347,  8, }, // America/Argentina/La_Rioja
    {  321,  565,   77355,  7, }, // America/Argentina/Mendoza
    {  321, 7452,   77362, 14, }, // America/Argentina/Rio_Gallegos
    {  321, 7483,   77376,  5, }, // America/Argentina/Salta
    {  321, 7507,   77381,  9, }, // America/Argentina/San_Juan
    {  321, 5497,   77390,  9, }, // America/Argentina/San_Luis
    {  321, 7534,   77399,  9, }, // America/Argentina/Tucuman
    {  321, 7560,   77408,  8, }, // America/Argentina/Ushuaia
    {  321, 5557,   77416,  7, }, // America/Aruba
    {  321, 4154,   77423,  9, }, // America/Asuncion
    {  321,  237,   77432,  8, }, // America/Atikokan
    {  321, 3581,   77440,  5, }, // America/Bahia
    {  321, 7586,   77445, 19, }, // America/Bahia_Banderas
    {  321, 5571,   77464, 12, }, // America/Barbados
    {  321, 7609,   77476,  6, }, // America/Belem
    {  321, 5309,   77482,  6, }, // America/Belize
    {  321, 7623,   77488, 13, }, // America/Blanc-Sablon
    {  321, 7644,   77501, 10, }, // America/Boa_Vista
    {  321, 4263,   77511,  8, }, // America/Bogota
    {  321, 7662,   77519,  6, }, // America/Boise
    {  321, 7676,   77525, 13, }, // America/Cambridge_Bay
    {  321, 7698,   77538, 13, }, // America/Campo_Grande
    {  321, 3770,   77551,  7, }, // America/Cancun
    {  321, 4596,   77558,  7, }, // America/Caracas
    {  321, 4247,   77565,  6, }, // America/Cayenne
    {  321, 5398,   77571,  6, }, // America/Cayman
    {  321, 3251,   77577,  6, }, // America/Chicago
    {  321, 7719,   77583,  9, }, // America/Chihuahua
    {  321, 7737,   77592, 15, }, // America/Ciudad_Juarez
    {  321, 5324,   77607, 10, }, // America/Costa_Rica
    {  321, 7759,   77617,  7, }, // America/Creston
    {  321, 3693,   77624,  8, }, // America/Cuiaba
    {  321, 5640,   77632,  8, }, // America/Curacao
    {  321, 7775,   77640, 12, }, // America/Danmarkshavn
    {  321, 7796,   77652,  6, }, // America/Dawson
    {  321, 7811,   77658, 11, }, // America/Dawson_Creek
    {  321,  805,   77669,  7, }, // America/Denver
    {  321, 3351,   77676,  8, }, // America/Detroit
    {  321, 5656,   77684,  9, }, // America/Dominica
    {  321,  893,   77693,  9, }, // America/Edmonton
    {  321, 7832,   77702,  9, }, // America/Eirunepe
    {  321, 5343,   77711, 11, }, // America/El_Salvador
    {  321, 7849,   77722, 11, }, // America/Fort_Nelson
    {  321, 7869,   77733,  9, }, // America/Fortaleza
    {  321, 7887,   77742, 11, }, // America/Glace_Bay
    {  321, 6844,   77753, 10, }, // America/Goose_Bay
    {  321, 4514,   77763, 12, }, // America/Grand_Turk
    {  321, 5687,   77775,  7, }, // America/Grenada
    {  321, 5703,   77782, 11, }, // America/Guadeloupe
    {  321, 3662,   77793, 10, }, // America/Guatemala
    {  321, 4743,   77803, 11, }, // America/Guayaquil
    {  321, 6895,   77814,  7, }, // America/Guyana
    {  321, 1923,   77821,  7, }, // America/Halifax
    {  321, 2221,   77828,  5, }, // America/Havana
    {  321, 7905,   77833,  9, }, // America/Hermosillo
    {  321,  348,   77842, 12, }, // America/Indiana/Indianapolis
    {  321,  481,   77854, 12, }, // America/Indiana/Knox
    {  321, 7924,   77866, 17, }, // America/Indiana/Marengo
    {  321, 7948,   77883, 21, }, // America/Indiana/Petersburg
    {  321, 7975,   77904, 17, }, // America/Indiana/Tell_City
    {  321, 8001,   77921, 14, }, // America/Indiana/Vevay
    {  321, 8023,   77935, 15, }, // America/Indiana/Vincennes
    {  321, 8049,   77950, 18, }, // America/Indiana/Winamac
    {  321, 8073,   77968,  7, }, // America/Inuvik
    {  321,  660,   77975,  8, }, // America/Iqaluit
    {  321, 2665,   77983,  8, }, // America/Jamaica
    {  321, 5282,   77991,  6, }, // America/Juneau
    {  321,  521,   77997,  7, }, // America/Kentucky/Louisville
    {  321, 8088,   78004, 19, }, // America/Kentucky/Monticello
    {  321, 5621,   78023,  9, }, // America/Kralendijk
    {  321, 4278,   78032,  6, }, // America/La_Paz
    {  321, 7132,   78038,  4, }, // America/Lima
    {  321, 3390,   78042, 12, }, // America/Los_Angeles
    {  321, 5849,     207, 22, }, // America/Lower_Princes
    {  321, 8116,   78054,  6, }, // America/Maceio
    {  321, 8131,   78060,  9, }, // America/Managua
    {  321, 1892,   78069,  7, }, // America/Manaus
    {  321, 5814,   78076,  7, }, // America/Marigot
    {  321, 5722,   78083,  9, }, // America/Martinique
    {  321, 8147,   78092,  9, }, // America/Matamoros
    {  321, 2779,   78101,  8, }, // America/Mazatlan
    {  321, 8165,   78109,  8, }, // America/Menominee
    {  321, 8183,   78117,  6, }, // America/Merida
    {  321, 8198,   78123, 10, }, // America/Metlakatla
    {  321, 2811,   78133, 16, }, // America/Mexico_City
    {  321, 4293,   78149,  7, }, // America/Miquelon
    {  321, 8217,   78156,  7, }, // America/Moncton
    {  321, 8233,   78163,  8, }, // America/Monterrey
    {  321, 4000,   78171, 10, }, // America/Montevideo
    {  321, 5741,   78181,  8, }, // America/Montserrat
    {  321, 5383,   78189,  6, }, // America/Nassau
    {  321, 3294,   78195,  9, }, // America/New_York
    {  321, 8251,   78204,  4, }, // America/Nome
    {  321, 1834,   78208,  7, }, // America/Noronha
    {  321, 8264,   78215, 23, }, // America/North_Dakota/Beulah
    {  321, 8292,   78238, 22, }, // America/North_Dakota/Center
    {  321, 8320,   78260, 27, }, // America/North_Dakota/New_Salem
    {  321,  393,   78287,  6, }, // America/Nuuk
    {  321, 8351,   78293,  8, }, // America/Ojinaga
    {  321, 5413,   78301,  7, }, // America/Panama
    {  321, 6106,   78308, 11, }, // America/Paramaribo
    {  321, 3224,   78319,  6, }, // America/Phoenix
    {  321, 3847,   78325, 12, }, // America/Port-au-Prince
    {  321, 5871,   78337, 13, }, // America/Port_of_Spain
    {  321, 8367,   78350, 11, }, // America/Porto_Velho
    {  321, 5760,   78361, 12, }, // America/Puerto_Rico
    {  321, 3932,   78373, 13, }, // America/Punta_Arenas
    {  321, 8387,   78386, 12, }, // America/Rankin_Inlet
    {  321, 8408,   78398,  6, }, // America/Recife
    {  321, 1979,   78404,  8, }, // America/Regina
    {  321, 8423,   78412,  8, }, // America/Resolute
    {  321,  695,   78420, 11, }, // America/Rio_Branco
    {  321, 8440,   78431,  8, }, // America/Santarem
    {  321, 2165,   78439,  9, }, // America/Santiago
    {  321, 6044,   78448, 15, }, // America/Santo_Domingo
    {  321, 1862,   78463,  9, }, // America/Sao_Paulo
    {  321, 6862,   78472, 14, }, // America/Scoresbysund
    {  321, 8457,   78486,  5, }, // America/Sitka
    {  321, 8471,   78491, 18, }, // America/St_Barthelemy
    {  321, 2045,   78509,  9, }, // America/St_Johns
    {  321, 5780,   78518,  8, }, // America/St_Kitts
    {  321, 5797,   78526, 11, }, // America/St_Lucia
    {  321,  855,   78537, 11, }, // America/St_Thomas
    {  321, 5830,   78548, 15, }, // America/St_Vincent
    {  321, 8493,   78563, 13, }, // America/Swift_Current
    {  321, 5363,   78576, 14, }, // America/Tegucigalpa
    {  321, 5673,   78590,  5, }, // America/Thule
    {  321,  313,   78595,  8, }, // America/Tijuana
    {  321,  608,   78603,  7, }, // America/Toronto
    {  321, 5605,   78610,  7, }, // America/Tortola
    {  321, 2077,   78617,  9, }, // America/Vancouver
    {  321, 2128,   78626, 10, }, // America/Whitehorse
    {  321,  734,   78636, 10, }, // America/Winnipeg
    {  321, 8515,   78646,  9, }, // America/Yakutat
    {  321, 5918,   78655,  5, }, // Antarctica/Casey
    {  321, 6027,   78660,  7, }, // Antarctica/Davis
    {  321, 6066,   78667, 17, }, // Antarctica/DumontDUrville
    {  321, 8531,   78684,  8, }, // Antarctica/Macquarie
    {  321, 7029,   78692,  5, }, // Antarctica/Mawson
    {  321, 7076,   78697,  9, }, // Antarctica/McMurdo
    {  321, 8552,   78706,  6, }, // Antarctica/Palmer
    {  321, 7189,   78712,  6, }, // Antarctica/Rothera
    {  321, 7258,   78718,  5, }, // Antarctica/Syowa
    {  321, 8570,   78723,  4, }, // Antarctica/Troll
    {  321, 7337,   78727,  6, }, // Antarctica/Vostok
    {  321, 1413,   78733, 14, }, // Arctic/Longyearbyen
    {  321, 5487,   78747,  5, }, // Asia/Aden
    {  321, 5297,   78752,  6, }, // Asia/Almaty
    {  321, 3870,   78758,  5, }, // Asia/Amman
    {  321, 5428,   78763,  7, }, // Asia/Anadyr
    {  321, 5440,   78770,  6, }, // Asia/Aqtau
    {  321, 5270,   78776,  7, }, // Asia/Aqtobe
    {  321,  964,   78783,  8, }, // Asia/Ashgabat
    {  321, 8587,   78791,  7, }, // Asia/Atyrau
    {  321, 3509,   78798,  7, }, // Asia/Baghdad
    {  321, 5451,   78805,  8, }, // Asia/Bahrain
    {  321, 3555,   78813,  6, }, // Asia/Baku
    {  321, 4368,   78819,  8, }, // Asia/Bangkok
    {  321, 3473,   78827,  9, }, // Asia/Barnaul
    {  321, 3988,   78836,  7, }, // Asia/Beirut
    {  321, 3680,   78843,  7, }, // Asia/Bishkek
    {  321, 5906,   78850,  8, }, // Asia/Brunei
    {  321, 4503,   78858,  5, }, // Asia/Chita
    {  321, 5963,   78863, 11, }, // Asia/Choibalsan
    {  321, 4413,   78874,  7, }, // Asia/Colombo
    {  321, 4442,   78881,  8, }, // Asia/Damascus
    {  321, 1060,   78889,  5, }, // Asia/Dhaka
    {  321, 6125,   78894,  5, }, // Asia/Dili
    {  321, 3498,   78899,  8, }, // Asia/Dubai
    {  321, 6092,   78907, 10, }, // Asia/Dushanbe
    {  321, 8599,   78917, 10, }, // Asia/Famagusta
    {  321, 8614,   78927,  4, }, // Asia/Gaza
    {  321, 4697,   78931,  7, }, // Asia/Hebron
    {  321, 1226,   78938, 16, }, // Asia/Ho_Chi_Minh
    {  321, 2576,   78954, 11, }, // Asia/Hong_Kong
    {  321, 4673,   78965,  5, }, // Asia/Hovd
    {  321, 4086,   78970,  8, }, // Asia/Irkutsk
    {  321, 6969,   78978,  8, }, // Asia/Jakarta
    {  321, 6955,   78986, 11, }, // Asia/Jayapura
    {  321, 1257,   78997, 10, }, // Asia/Jerusalem
    {  321, 3462,   79007,  7, }, // Asia/Kabul
    {  321, 4218,   79014,  9, }, // Asia/Kamchatka
    {  321, 4141,   79023,  7, }, // Asia/Karachi
    {  321, 1152,   79030,  9, }, // Asia/Kathmandu
    {  321, 8624,   79039,  8, }, // Asia/Khandyga
    {  321,  992,   79047,  8, }, // Asia/Kolkata
    {  321, 4099,   79055, 12, }, // Asia/Krasnoyarsk
    {  321, 4761,   79067, 15, }, // Asia/Kuala_Lumpur
    {  321, 5893,   79082,  9, }, // Asia/Kuching
    {  321, 5464,   79091,  7, }, // Asia/Kuwait
    {  321, 1178,   79098,  5, }, // Asia/Macau
    {  321, 3919,   79103,  9, }, // Asia/Magadan
    {  321, 1316,   79112,  7, }, // Asia/Makassar
    {  321, 7145,   79119,  6, }, // Asia/Manila
    {  321, 6883,   79125,  6, }, // Asia/Muscat
    {  321, 2446,   79131,  8, }, // Asia/Nicosia
    {  321, 8638,   79139, 13, }, // Asia/Novokuznetsk
    {  321, 4037,   79152, 12, }, // Asia/Novosibirsk
    {  321, 4131,   79164,  4, }, // Asia/Omsk
    {  321, 7108,   79168,  4, }, // Asia/Oral
    {  321, 6924,   79172,  9, }, // Asia/Phnom_Penh
    {  321, 8656,   79181,  9, }, // Asia/Pontianak
    {  321, 4116,   79190, 12, }, // Asia/Pyongyang
    {  321, 5476,   79202,  5, }, // Asia/Qatar
    {  321, 8671,   79207,  8, }, // Asia/Qostanay
    {  321, 4171,   79215, 10, }, // Asia/Qyzylorda
    {  321, 3486,   79225,  5, }, // Asia/Riyadh
    {  321, 4310,   79230,  8, }, // Asia/Sakhalin
    {  321, 7208,   79238, 10, }, // Asia/Samarkand
    {  321, 3116,   79248,  5, }, // Asia/Seoul
    {  321, 1020,   79253,  7, }, // Asia/Shanghai
    {  321, 3137,   79260, 10, }, // Asia/Singapore
    {  321, 4199,   79270, 14, }, // Asia/Srednekolymsk
    {  321, 3100,   79284,  6, }, // Asia/Taipei
    {  321, 4683,   79290,  8, }, // Asia/Tashkent
    {  321, 3817,   79298,  7, }, // Asia/Tbilisi
    {  321, 2638,   79305,  8, }, // Asia/Tehran
    {  321, 1284,   79313,  6, }, // Asia/Thimphu
    {  321, 2687,   79319,  5, }, // Asia/Tokyo
    {  321, 4474,   79324,  5, }, // Asia/Tomsk
    {  321, 1346,   79329, 12, }, // Asia/Ulaanbaatar
    {  321, 1126,   79341,  9, }, // Asia/Urumqi
    {  321, 8685,   79350,  9, }, // Asia/Ust-Nera
    {  321, 6940,   79359,  8, }, // Asia/Vientiane
    {  321, 4612,   79367, 11, }, // Asia/Vladivostok
    {  321, 4730,   79378,  9, }, // Asia/Yakutsk
    {  321, 1202,   79387,  8, }, // Asia/Yangon
    {  321, 3785,   79395, 15, }, // Asia/Yekaterinburg
    {  321, 3649,   79410,  6, }, // Asia/Yerevan
    {  321, 3565,   79416,  6, }, // Atlantic/Azores
    {  321, 5588,   79422,  9, }, // Atlantic/Bermuda
    {  321, 6591,   79431,  7, }, // Atlantic/Canary
    {  321, 3629,   79438, 17, }, // Atlantic/Cape_Verde
    {  321, 1379,   79455,  6, }, // Atlantic/Faroe
    {  321, 8699,   79461,  6, }, // Atlantic/Madeira
    {  321, 2614,   79467,  9, }, // Atlantic/Reykjavik
    {  321, 7235,   79476, 13, }, // Atlantic/South_Georgia
    {  321, 6784,   79489,  9, }, // Atlantic/St_Helena
    {  321, 6607,   79498,  7, }, // Atlantic/Stanley
    {  321, 1654,   79505,  8, }, // Australia/Adelaide
    {  321, 1619,   79513, 10, }, // Australia/Brisbane
    {  321, 1783,   79523, 11, }, // Australia/Broken_Hill
    {  321, 1567,   79534,  9, }, // Australia/Darwin
    {  321, 3539,   79543,  7, }, // Australia/Eucla
    {  321, 1500,   79550,  7, }, // Australia/Hobart
    {  321, 8716,   79557,  8, }, // Australia/Lindeman
    {  321, 1531,   79565, 10, }, // Australia/Lord_Howe
    {  321, 1711,   79575,  9, }, // Australia/Melbourne
    {  321, 1746,   79584,  4, }, // Australia/Perth
    {  321, 1447,   79588,  6, }, // Australia/Sydney
    {  321, 6390,   79594, 10, }, // Europe/Amsterdam
    {  321, 6149,   79604,  6, }, // Europe/Andorra
    {  321, 3522,   79610,  8, }, // Europe/Astrakhan
    {  321, 6577,   79618,  5, }, // Europe/Athens
    {  321, 6285,   79623,  9, }, // Europe/Belgrade
    {  321, 4659,   79632,  8, }, // Europe/Berlin
    {  321, 6437,   79640, 11, }, // Europe/Bratislava
    {  321, 6178,   79651,  9, }, // Europe/Brussels
    {  321, 3830,   79660, 11, }, // Europe/Bucharest
    {  321, 3708,   79671, 10, }, // Europe/Budapest
    {  321, 8751,   79681, 10, }, // Europe/Busingen
    {  321, 2475,   79691,  8, }, // Europe/Chisinau
    {  321, 6238,   79699,  9, }, // Europe/Copenhagen
    {  321, 2260,   79708,  8, }, // Europe/Dublin
    {  321, 6256,   79716,  9, }, // Europe/Gibraltar
    {  321, 8767,   79725,  7, }, // Europe/Guernsey
    {  321, 6561,   79732,  7, }, // Europe/Helsinki
    {  321, 8783,   79739, 13, }, // Europe/Isle_of_Man
    {  321, 1097,   79752, 16, }, // Europe/Istanbul
    {  321, 8802,   79768,  7, }, // Europe/Jersey
    {  321, 3881,   79775, 13, }, // Europe/Kaliningrad
    {  321, 8924,   79788,  5, }, // Europe/Kirov
    {  321, 2419,   79793,  5, }, // Europe/Kyiv
    {  321, 3078,   79798,  8, }, // Europe/Lisbon
    {  321, 6455,   79806, 11, }, // Europe/Ljubljana
    {  321, 2393,   79817,  7, }, // Europe/London
    {  321, 6314,   79824, 12, }, // Europe/Luxembourg
    {  321, 4794,   79836,  7, }, // Europe/Madrid
    {  321, 6346,   79843,  5, }, // Europe/Malta
    {  321, 6531,   79848,  9, }, // Europe/Mariehamn
    {  321, 3595,   79857,  5, }, // Europe/Minsk
    {  321, 6359,   79862,  6, }, // Europe/Monaco
    {  321, 3443,   79868,  5, }, // Europe/Moscow
    {  321, 6407,   79873,  4, }, // Europe/Oslo
    {  321, 4186,   79877,  6, }, // Europe/Paris
    {  321, 6373,   79883, 12, }, // Europe/Podgorica
    {  321, 6224,   79895,  5, }, // Europe/Prague
    {  321, 8816,   79900,  4, }, // Europe/Riga
    {  321, 6273,   79904,  4, }, // Europe/Rome
    {  321, 4233,   79908,  6, }, // Europe/Samara
    {  321, 6419,   79914, 13, }, // Europe/San_Marino
    {  321, 6194,   79927,  8, }, // Europe/Sarajevo
    {  321, 4353,   79935,  7, }, // Europe/Saratov
    {  321, 8828,   79942, 11, }, // Europe/Simferopol
    {  321, 6332,   79953,  6, }, // Europe/Skopje
    {  321, 6548,   79959,  5, }, // Europe/Sofia
    {  321, 6472,   79964,  9, }, // Europe/Stockholm
    {  321, 8846,   79973,  5, }, // Europe/Tallinn
    {  321, 6135,   79978,  6, }, // Europe/Tirane
    {  321, 8861,   79984, 10, }, // Europe/Ulyanovsk
    {  321, 6301,   79994,  7, }, // Europe/Vaduz
    {  321, 6516,   80001,  8, }, // Europe/Vatican
    {  321, 6164,   80009,  6, }, // Europe/Vienna
    {  321, 8878,   80015,  8, }, // Europe/Vilnius
    {  321, 4629,   80023, 11, }, // Europe/Volgograd
    {  321, 3055,   80034,  8, }, // Europe/Warsaw
    {  321, 6210,   80042,  8, }, // Europe/Zagreb
    {  321, 6489,   80050,  6, }, // Europe/Zurich
    {  321, 4980,   80056, 12, }, // Indian/Antananarivo
    {  321, 6910,   80068,  7, }, // Indian/Chagos
    {  321, 5979,   80075, 19, }, // Indian/Christmas
    {  321, 5996,   80094,  5, }, // Indian/Cocos
    {  321, 4931,   80099,  6, }, // Indian/Comoro
    {  321, 6624,   80105,  9, }, // Indian/Kerguelen
    {  321, 7223,   80114,  4, }, // Indian/Mahe
    {  321, 7013,   80118,  8, }, // Indian/Maldives
    {  321, 3971,   80126,  9, }, // Indian/Mauritius
    {  321, 5000,   80135,  6, }, // Indian/Mayotte
    {  321, 7174,   80141,  7, }, // Indian/Reunion
    {  321, 4324,   80148,  4, }, // Pacific/Apia
    {  321,  932,   80152,  6, }, // Pacific/Auckland
    {  321, 3608,   80158, 11, }, // Pacific/Bougainville
    {  321, 2863,   80169,  6, }, // Pacific/Chatham
    {  321, 3022,   80175,  5, }, // Pacific/Chuuk
    {  321, 2201,   80180, 11, }, // Pacific/Easter
    {  321, 7323,   80191,  5, }, // Pacific/Efate
    {  321, 7290,   80196,  7, }, // Pacific/Fakaofo
    {  321, 3804,   80203,  5, }, // Pacific/Fiji
    {  321, 7306,   80208, 10, }, // Pacific/Funafuti
    {  321, 6641,   80218, 11, }, // Pacific/Galapagos
    {  321, 6659,   80229,  8, }, // Pacific/Gambier
    {  321, 3724,   80237, 14, }, // Pacific/Guadalcanal
    {  321, 5935,   80251,  6, }, // Pacific/Guam
    {  321, 2929,   80257, 10, }, // Pacific/Honolulu
    {  321, 2897,   80267, 10, }, // Pacific/Kanton
    {  321, 3900,   80277, 10, }, // Pacific/Kiritimati
    {  321, 6982,   80287,  6, }, // Pacific/Kosrae
    {  321, 2708,   80293, 11, }, // Pacific/Kwajalein
    {  321, 4779,   80304,  8, }, // Pacific/Majuro
    {  321, 3953,   80312,  8, }, // Pacific/Marquesas
    {  321, 8909,   80320,  9, }, // Pacific/Midway
    {  321, 7047,   80329,  7, }, // Pacific/Nauru
    {  321, 7095,   80336,  5, }, // Pacific/Niue
    {  321, 4070,   80341,  7, }, // Pacific/Norfolk
    {  321, 7061,   80348,  6, }, // Pacific/Noumea
    {  321, 2991,   80354, 11, }, // Pacific/Pago_Pago
    {  321, 7118,   80365,  6, }, // Pacific/Palau
    {  321, 7157,   80371,  7, }, // Pacific/Pitcairn
    {  321, 2961,   80378,  6, }, // Pacific/Pohnpei
    {  321, 4709,   80384, 13, }, // Pacific/Port_Moresby
    {  321, 6009,   80397, 10, }, // Pacific/Rarotonga
    {  321, 5948,   80407,  6, }, // Pacific/Saipan
    {  321, 7275,   80413,  5, }, // Pacific/Tahiti
    {  321, 6675,   80418,  7, }, // Pacific/Tarawa
    {  321, 4485,   80425, 11, }, // Pacific/Tongatapu
    {  321, 7355,   80436,  6, }, // Pacific/Wake
    {  321, 7368,   80442,  7, }, // Pacific/Wallis
    {  323,   14,       0,  6, }, // Africa/Asmara Guarani/Latin/Paraguay
    {  323, 4337,       6,  8, }, // Africa/Sao_Tome
    {  323, 4154,      14,  8, }, // America/Asuncion
    {  323,  237,      22,  8, }, // America/Atikokan
    {  323, 7586,      30, 17, }, // America/Bahia_Banderas
    {  323, 3770,      47,  6, }, // America/Cancun
    {  323, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  323, 5640,      66,  7, }, // America/Curacao
    {  323,  481,      73, 13, }, // America/Indiana/Knox
    {  323, 7924,      86, 16, }, // America/Indiana/Marengo
    {  323, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  323, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  323, 8001,     139, 14, }, // America/Indiana/Vevay
    {  323, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  323, 8049,     171, 16, }, // America/Indiana/Winamac
    {  323, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  323, 5849,     207, 22, }, // America/Lower_Princes
    {  323, 8183,     229,  6, }, // America/Merida
    {  323, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  323, 8292,     255, 20, }, // America/North_Dakota/Center
    {  323, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  323,  393,     298,  4, }, // America/Nuuk
    {  323, 6862,     302, 16, }, // America/Scoresbysund
    {  323, 8471,     318, 14, }, // America/St_Barthelemy
    {  323, 2045,     332, 10, }, // America/St_Johns
    {  323, 5780,     342,  9, }, // America/St_Kitts
    {  323, 5797,     351,  9, }, // America/St_Lucia
    {  323,  855,     360, 10, }, // America/St_Thomas
    {  323, 5830,     370, 11, }, // America/St_Vincent
    {  323, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  323, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  323, 1152,     413,  9, }, // Asia/Kathmandu
    {  323,  992,     422,  7, }, // Asia/Kolkata
    {  323, 1178,     429,  5, }, // Asia/Macau
    {  323, 1202,     442,  6, }, // Asia/Yangon
    {  323, 1379,     448,  5, }, // Atlantic/Faroe
    {  323, 6784,     453, 10, }, // Atlantic/St_Helena
    {  323, 2419,     463,  4, }, // Europe/Kyiv
    {  323, 7174,     467,  7, }, // Indian/Reunion
    {  323, 3022,     474,  5, }, // Pacific/Chuuk
    {  323, 2961,     479,  7, }, // Pacific/Pohnpei
    {  324, 6751,   80449,  8, }, // Africa/Abidjan Gujarati/Gujarati/India
    {  324, 6723,   80457,  4, }, // Africa/Accra
    {  324, 4961,   80461, 10, }, // Africa/Addis_Ababa
    {  324, 7383,   80471,  9, }, // Africa/Algiers
    {  324,   14,   80480,  7, }, // Africa/Asmara
    {  324,   44,   80487,  5, }, // Africa/Bamako
    {  324, 5159,   80492,  6, }, // Africa/Bangui
    {  324, 6709,   80498,  6, }, // Africa/Banjul
    {  324, 7398,   80504,  5, }, // Africa/Bissau
    {  324, 4859,   80509,  8, }, // Africa/Blantyre
    {  324, 5189,   80517, 10, }, // Africa/Brazzaville
    {  324, 4824,   80527,  9, }, // Africa/Bujumbura
    {  324, 2242,   80536,  4, }, // Africa/Cairo
    {  324, 4019,   80540, 11, }, // Africa/Casablanca
    {  324, 7412,   80551,  6, }, // Africa/Ceuta
    {  324, 6736,   80557,  8, }, // Africa/Conakry
    {  324, 6803,   80565,  4, }, // Africa/Dakar
    {  324, 5032,   80569, 11, }, // Africa/Dar_es_Salaam
    {  324, 4945,   80580,  6, }, // Africa/Djibouti
    {  324, 5145,   80586,  5, }, // Africa/Douala
    {  324, 5068,   80591,  6, }, // Africa/El_Aaiun
    {  324, 6816,   80597,  8, }, // Africa/Freetown
    {  324, 4808,   80605,  7, }, // Africa/Gaborone
    {  324, 4917,   80612,  5, }, // Africa/Harare
    {  324, 4381,   80617, 11, }, // Africa/Johannesburg
    {  324, 4401,   80628,  4, }, // Africa/Juba
    {  324, 5053,   80632,  7, }, // Africa/Kampala
    {  324, 4426,   80639,  7, }, // Africa/Khartoum
    {  324, 4875,   80646,  6, }, // Africa/Kigali
    {  324, 5208,   80652,  7, }, // Africa/Kinshasa
    {  324, 4646,   80659,  5, }, // Africa/Lagos
    {  324, 5238,   80664, 10, }, // Africa/Libreville
    {  324, 6832,   80674,  3, }, // Africa/Lome
    {  324, 5113,   80677,  9, }, // Africa/Luanda
    {  324, 4841,   80686, 10, }, // Africa/Lubumbashi
    {  324, 4903,   80696,  6, }, // Africa/Lusaka
    {  324, 5224,   80702,  5, }, // Africa/Malabo
    {  324, 4889,   80707,  6, }, // Africa/Maputo
    {  324, 5099,   80713,  5, }, // Africa/Maseru
    {  324, 5084,   80718,  7, }, // Africa/Mbabane
    {  324, 5015,   80725,  8, }, // Africa/Mogadishu
    {  324, 6997,   80733,  9, }, // Africa/Monrovia
    {  324, 3755,   80742,  6, }, // Africa/Nairobi
    {  324, 5173,   80748,  7, }, // Africa/Ndjamena
    {  324, 5256,   80755,  6, }, // Africa/Niamey
    {  324, 6766,   80761,  6, }, // Africa/Nouakchott
    {  324, 6690,   80767,  8, }, // Africa/Ouagadougou
    {  324, 5127,   80775, 11, }, // Africa/Porto-Novo
    {  324, 4337,   80786,  7, }, // Africa/Sao_Tome
    {  324, 2732,   80793,  8, }, // Africa/Tripoli
    {  324, 6503,   80801,  7, }, // Africa/Tunis
    {  324, 4054,   80808,  7, }, // Africa/Windhoek
    {  324,  132,   80815,  3, }, // America/Adak
    {  324, 3183,   80818,  8, }, // America/Anchorage
    {  324, 5524,   80826,  8, }, // America/Anguilla
    {  324, 5541,   80834,  8, }, // America/Antigua
    {  324, 4456,   80842,  8, }, // America/Araguaina
    {  324,  166,   80850, 12, }, // America/Argentina/Buenos_Aires
    {  324,   91,   80862, 10, }, // America/Argentina/Catamarca
    {  324,  270,   80872,  8, }, // America/Argentina/Cordoba
    {  324,  441,   80880,  5, }, // America/Argentina/Jujuy
    {  324, 7425,   80885,  8, }, // America/Argentina/La_Rioja
    {  324,  565,   80893,  8, }, // America/Argentina/Mendoza
    {  324, 7452,   80901, 11, }, // America/Argentina/Rio_Gallegos
    {  324, 7483,   80912,  6, }, // America/Argentina/Salta
    {  324, 7507,   80918,  8, }, // America/Argentina/San_Juan
    {  324, 5497,   80926,  8, }, // America/Argentina/San_Luis
    {  324, 7534,   80934,  7, }, // America/Argentina/Tucuman
    {  324, 7560,   80941,  5, }, // America/Argentina/Ushuaia
    {  324, 5557,   80946,  5, }, // America/Aruba
    {  324, 4154,   80951,  8, }, // America/Asuncion
    {  324,  237,   80959,  8, }, // America/Atikokan
    {  324, 3581,   80967,  6, }, // America/Bahia
    {  324, 7586,   80973, 14, }, // America/Bahia_Banderas
    {  324, 5571,   80987,  8, }, // America/Barbados
    {  324, 7609,   80995,  5, }, // America/Belem
    {  324, 5309,   81000,  5, }, // America/Belize
    {  324, 7623,   81005, 13, }, // America/Blanc-Sablon
    {  324, 7644,   81018, 10, }, // America/Boa_Vista
    {  324, 4263,   81028,  6, }, // America/Bogota
    {  324, 7662,   81034,  4, }, // America/Boise
    {  324, 7676,   81038, 12, }, // America/Cambridge_Bay
    {  324, 7698,   81050, 13, }, // America/Campo_Grande
    {  324, 3770,   81063,  7, }, // America/Cancun
    {  324, 4596,   81070,  6, }, // America/Caracas
    {  324, 4247,   81076,  6, }, // America/Cayenne
    {  324, 5398,   81082,  5, }, // America/Cayman
    {  324, 3251,   81087,  6, }, // America/Chicago
    {  324, 7719,   81093,  8, }, // America/Chihuahua
    {  324, 7737,   81101, 14, }, // America/Ciudad_Juarez
    {  324, 5324,   81115, 11, }, // America/Costa_Rica
    {  324, 7759,   81126,  8, }, // America/Creston
    {  324, 3693,   81134,  7, }, // America/Cuiaba
    {  324, 5640,   81141,  7, }, // America/Curacao
    {  324, 7775,   81148, 11, }, // America/Danmarkshavn
    {  324, 7796,   81159,  4, }, // America/Dawson
    {  324, 7811,   81163, 10, }, // America/Dawson_Creek
    {  324,  805,   81173,  6, }, // America/Denver
    {  324, 3351,   81179,  8, }, // America/Detroit
    {  324, 5656,   81187,  8, }, // America/Dominica
    {  324,  893,   81195,  8, }, // America/Edmonton
    {  324, 7832,   81203,  6, }, // America/Eirunepe
    {  324, 5343,   81209, 12, }, // America/El_Salvador
    {  324, 7849,   81221, 12, }, // America/Fort_Nelson
    {  324, 7869,   81233, 10, }, // America/Fortaleza
    {  324, 7887,   81243,  8, }, // America/Glace_Bay
    {  324, 6844,   81251,  6, }, // America/Goose_Bay
    {  324, 4514,   81257, 12, }, // America/Grand_Turk
    {  324, 5687,   81269,  8, }, // America/Grenada
    {  324, 5703,   81277,  9, }, // America/Guadeloupe
    {  324, 3662,   81286, 10, }, // America/Guatemala
    {  324, 4743,   81296,  9, }, // America/Guayaquil
    {  324, 6895,   81305,  5, }, // America/Guyana
    {  324, 1923,   81310,  9, }, // America/Halifax
    {  324, 2221,   81319,  5, }, // America/Havana
    {  324, 7905,   81324,  8, }, // America/Hermosillo
    {  324,  348,   81332, 14, }, // America/Indiana/Indianapolis
    {  324,  481,   81346, 16, }, // America/Indiana/Knox
    {  324, 7924,   81362, 18, }, // America/Indiana/Marengo
    {  324, 7948,   81380, 21, }, // America/Indiana/Petersburg
    {  324, 7975,   81401, 19, }, // America/Indiana/Tell_City
    {  324, 8001,   81420, 15, }, // America/Indiana/Vevay
    {  324, 8023,   81435, 20, }, // America/Indiana/Vincennes
    {  324, 8049,   81455, 18, }, // America/Indiana/Winamac
    {  324, 8073,   81473,  6, }, // America/Inuvik
    {  324,  660,   81479,  7, }, // America/Iqaluit
    {  324, 2665,   81486,  5, }, // America/Jamaica
    {  324, 5282,   81491,  5, }, // America/Juneau
    {  324,  521,   81496,  6, }, // America/Kentucky/Louisville
    {  324, 8088,   81502, 20, }, // America/Kentucky/Monticello
    {  324, 5621,   81522, 11, }, // America/Kralendijk
    {  324, 4278,   81533,  6, }, // America/La_Paz
    {  324, 7132,   81539,  4, }, // America/Lima
    {  324, 3390,   81543, 11, }, // America/Los_Angeles
    {  324, 5849,   81554, 24, }, // America/Lower_Princes
    {  324, 8116,   81578,  5, }, // America/Maceio
    {  324, 8131,   81583,  7, }, // America/Managua
    {  324, 1892,   81590,  4, }, // America/Manaus
    {  324, 5814,   81594,  7, }, // America/Marigot
    {  324, 5722,   81601,  9, }, // America/Martinique
    {  324, 8147,   81610,  9, }, // America/Matamoros
    {  324, 2779,   81619,  7, }, // America/Mazatlan
    {  324, 8165,   81626,  8, }, // America/Menominee
    {  324, 8183,   81634,  6, }, // America/Merida
    {  324, 8198,   81640, 12, }, // America/Metlakatla
    {  324, 2811,   81652, 13, }, // America/Mexico_City
    {  324, 4293,   81665,  9, }, // America/Miquelon
    {  324, 8217,   81674,  7, }, // America/Moncton
    {  324, 8233,   81681,  8, }, // America/Monterrey
    {  324, 4000,   81689, 10, }, // America/Montevideo
    {  324, 5741,   81699,  9, }, // America/Montserrat
    {  324, 5383,   81708,  5, }, // America/Nassau
    {  324, 3294,   81713,  9, }, // America/New_York
    {  324, 8251,   81628,  3, }, // America/Nome
    {  324, 1834,   81722,  8, }, // America/Noronha
    {  324, 8264,   81730, 21, }, // America/North_Dakota/Beulah
    {  324, 8292,   81751, 20, }, // America/North_Dakota/Center
    {  324, 8320,   81771, 23, }, // America/North_Dakota/New_Salem
    {  324,  393,   81794,  3, }, // America/Nuuk
    {  324, 8351,   81797,  7, }, // America/Ojinaga
    {  324, 5413,   81804,  5, }, // America/Panama
    {  324, 6106,   81809, 10, }, // America/Paramaribo
    {  324, 3224,   81819,  7, }, // America/Phoenix
    {  324, 3847,   81826, 15, }, // America/Port-au-Prince
    {  324, 5871,   81841, 14, }, // America/Port_of_Spain
    {  324, 8367,   81855, 13, }, // America/Porto_Velho
    {  324, 5760,   81868, 14, }, // America/Puerto_Rico
    {  324, 3932,   81882, 13, }, // America/Punta_Arenas
    {  324, 8387,   81895, 13, }, // America/Rankin_Inlet
    {  324, 8408,   81908,  5, }, // America/Recife
    {  324, 1979,   81913,  6, }, // America/Regina
    {  324, 8423,   81919,  7, }, // America/Resolute
    {  324,  695,   81926, 12, }, // America/Rio_Branco
    {  324, 8440,   81938,  9, }, // America/Santarem
    {  324, 2165,   81947,  9, }, // America/Santiago
    {  324, 6044,   81956, 13, }, // America/Santo_Domingo
    {  324, 1862,   81969,  9, }, // America/Sao_Paulo
    {  324, 6862,   81978, 14, }, // America/Scoresbysund
    {  324, 8457,   81992,  6, }, // America/Sitka
    {  324, 8471,   81998, 15, }, // America/St_Barthelemy
    {  324, 2045,   82013, 12, }, // America/St_Johns
    {  324, 5780,   82025, 10, }, // America/St_Kitts
    {  324, 5797,   82035, 11, }, // America/St_Lucia
    {  324,  855,   82046,  9, }, // America/St_Thomas
    {  324, 5830,   82055, 14, }, // America/St_Vincent
    {  324, 8493,   82069, 12, }, // America/Swift_Current
    {  324, 5363,   82081, 11, }, // America/Tegucigalpa
    {  324, 5673,   82092,  4, }, // America/Thule
    {  324,  313,   82096,  7, }, // America/Tijuana
    {  324,  608,   82103,  7, }, // America/Toronto
    {  324, 5605,   82110,  8, }, // America/Tortola
    {  324, 2077,   82118,  8, }, // America/Vancouver
    {  324, 2128,   82126, 11, }, // America/Whitehorse
    {  324,  734,   82137,  9, }, // America/Winnipeg
    {  324, 8515,   82146,  5, }, // America/Yakutat
    {  324, 5918,   82151,  4, }, // Antarctica/Casey
    {  324, 6027,   82155,  5, }, // Antarctica/Davis
    {  324, 6066,   82160, 17, }, // Antarctica/DumontDUrville
    {  324, 8531,   82177,  8, }, // Antarctica/Macquarie
    {  324, 7029,   82185,  4, }, // Antarctica/Mawson
    {  324, 7076,   82189,  8, }, // Antarctica/McMurdo
    {  324, 8552,   82197,  6, }, // Antarctica/Palmer
    {  324, 7189,   82203,  6, }, // Antarctica/Rothera
    {  324, 7258,   82209,  6, }, // Antarctica/Syowa
    {  324, 8570,   82215,  5, }, // Antarctica/Troll
    {  324, 7337,   82220,  7, }, // Antarctica/Vostok
    {  324, 1413,   82227, 12, }, // Arctic/Longyearbyen
    {  324, 5487,   82239,  4, }, // Asia/Aden
    {  324, 5297,   82243,  7, }, // Asia/Almaty
    {  324, 3870,   82250,  6, }, // Asia/Amman
    {  324, 5428,   82256,  5, }, // Asia/Anadyr
    {  324, 5440,   82261,  5, }, // Asia/Aqtau
    {  324, 5270,   82266,  6, }, // Asia/Aqtobe
    {  324,  964,   82272,  6, }, // Asia/Ashgabat
    {  324, 8587,   82278,  7, }, // Asia/Atyrau
    {  324, 3509,   82285,  5, }, // Asia/Baghdad
    {  324, 5451,   82290,  6, }, // Asia/Bahrain
    {  324, 3555,   81139,  4, }, // Asia/Baku
    {  324, 4368,   82296,  7, }, // Asia/Bangkok
    {  324, 3473,   82303,  6, }, // Asia/Barnaul
    {  324, 3988,   82309,  5, }, // Asia/Beirut
    {  324, 3680,   82314,  6, }, // Asia/Bishkek
    {  324, 5906,   82320,  7, }, // Asia/Brunei
    {  324, 4503,   82327,  4, }, // Asia/Chita
    {  324, 5963,   82331,  9, }, // Asia/Choibalsan
    {  324, 4413,   82340,  6, }, // Asia/Colombo
    {  324, 4442,   82346,  7, }, // Asia/Damascus
    {  324, 1060,   82353,  4, }, // Asia/Dhaka
    {  324, 6125,   82357,  4, }, // Asia/Dili
    {  324, 3498,   82361,  4, }, // Asia/Dubai
    {  324, 6092,   82365,  8, }, // Asia/Dushanbe
    {  324, 8599,   82373, 10, }, // Asia/Famagusta
    {  324, 8614,   82383,  4, }, // Asia/Gaza
    {  324, 4697,   82387,  7, }, // Asia/Hebron
    {  324, 1226,   82394, 14, }, // Asia/Ho_Chi_Minh
    {  324, 2576,   82408,  8, }, // Asia/Hong_Kong
    {  324, 4673,   82416,  5, }, // Asia/Hovd
    {  324, 4086,   82421, 10, }, // Asia/Irkutsk
    {  324, 6969,   82431,  7, }, // Asia/Jakarta
    {  324, 6955,   82438,  7, }, // Asia/Jayapura
    {  324, 1257,   82445,  8, }, // Asia/Jerusalem
    {  324, 3462,   82453,  5, }, // Asia/Kabul
    {  324, 4218,   82458,  8, }, // Asia/Kamchatka
    {  324, 4141,   82466,  5, }, // Asia/Karachi
    {  324, 1152,   82471,  7, }, // Asia/Kathmandu
    {  324, 8624,   82478,  6, }, // Asia/Khandyga
    {  324,  992,   82484,  7, }, // Asia/Kolkata
    {  324, 4099,   82491, 15, }, // Asia/Krasnoyarsk
    {  324, 4761,   82506, 13, }, // Asia/Kuala_Lumpur
    {  324, 5893,   82519,  6, }, // Asia/Kuching
    {  324, 5464,   82525,  5, }, // Asia/Kuwait
    {  324, 1178,   82530,  4, }, // Asia/Macau
    {  324, 3919,   82534,  6, }, // Asia/Magadan
    {  324, 1316,   82540,  7, }, // Asia/Makassar
    {  324, 7145,   82547,  5, }, // Asia/Manila
    {  324, 6883,   82552,  5, }, // Asia/Muscat
    {  324, 2446,   82557,  8, }, // Asia/Nicosia
    {  324, 8638,   82565, 15, }, // Asia/Novokuznetsk
    {  324, 4037,   82580, 15, }, // Asia/Novosibirsk
    {  324, 4131,   82595,  6, }, // Asia/Omsk
    {  324, 7108,   82601,  3, }, // Asia/Oral
    {  324, 6924,   82604, 11, }, // Asia/Phnom_Penh
    {  324, 8656,   82615, 10, }, // Asia/Pontianak
    {  324, 4116,   82625, 10, }, // Asia/Pyongyang
    {  324, 5476,   82635,  4, }, // Asia/Qatar
    {  324, 8671,   82639,  8, }, // Asia/Qostanay
    {  324, 4171,   82647, 10, }, // Asia/Qyzylorda
    {  324, 3486,   82657,  5, }, // Asia/Riyadh
    {  324, 4310,   82662,  6, }, // Asia/Sakhalin
    {  324, 7208,   82668,  7, }, // Asia/Samarkand
    {  324, 3116,   82675,  4, }, // Asia/Seoul
    {  324, 1020,   82679,  6, }, // Asia/Shanghai
    {  324, 3137,   82685,  8, }, // Asia/Singapore
    {  324, 4199,   82693, 17, }, // Asia/Srednekolymsk
    {  324, 3100,   82710,  6, }, // Asia/Taipei
    {  324, 4683,   82716,  8, }, // Asia/Tashkent
    {  324, 3817,   82724,  6, }, // Asia/Tbilisi
    {  324, 2638,   82730,  6, }, // Asia/Tehran
    {  324, 1284,   82736,  6, }, // Asia/Thimphu
    {  324, 2687,   82742,  6, }, // Asia/Tokyo
    {  324, 4474,   82748,  6, }, // Asia/Tomsk
    {  324, 1346,   82754,  8, }, // Asia/Ulaanbaatar
    {  324, 1126,   82762,  7, }, // Asia/Urumqi
    {  324, 8685,   82769,  9, }, // Asia/Ust-Nera
    {  324, 6940,   82778, 10, }, // Asia/Vientiane
    {  324, 4612,   82788, 13, }, // Asia/Vladivostok
    {  324, 4730,   82801,  7, }, // Asia/Yakutsk
    {  324, 1202,   82808,  5, }, // Asia/Yangon
    {  324, 3785,   82813, 13, }, // Asia/Yekaterinburg
    {  324, 3649,   82826,  7, }, // Asia/Yerevan
    {  324, 3565,   82833,  6, }, // Atlantic/Azores
    {  324, 5588,   82839,  7, }, // Atlantic/Bermuda
    {  324, 6591,   82846,  6, }, // Atlantic/Canary
    {  324, 3629,   82852,  9, }, // Atlantic/Cape_Verde
    {  324, 1379,   82861,  4, }, // Atlantic/Faroe
    {  324, 8699,   82865,  5, }, // Atlantic/Madeira
    {  324, 2614,   82870,  8, }, // Atlantic/Reykjavik
    {  324, 7235,   82878, 17, }, // Atlantic/South_Georgia
    {  324, 6784,   82895, 11, }, // Atlantic/St_Helena
    {  324, 6607,   82906,  7, }, // Atlantic/Stanley
    {  324, 1654,   82913,  7, }, // Australia/Adelaide
    {  324, 1619,   82920,  8, }, // Australia/Brisbane
    {  324, 1783,   82928, 10, }, // Australia/Broken_Hill
    {  324, 1567,   82938,  7, }, // Australia/Darwin
    {  324, 3539,   82945,  5, }, // Australia/Eucla
    {  324, 1500,   82950,  7, }, // Australia/Hobart
    {  324, 8716,   82957,  9, }, // Australia/Lindeman
    {  324, 1531,   82966,  9, }, // Australia/Lord_Howe
    {  324, 1711,   82975,  8, }, // Australia/Melbourne
    {  324, 1746,   82983,  4, }, // Australia/Perth
    {  324, 1447,   82987,  5, }, // Australia/Sydney
    {  324, 6390,   82992, 10, }, // Europe/Amsterdam
    {  324, 6149,   83002,  6, }, // Europe/Andorra
    {  324, 3522,   83008,  8, }, // Europe/Astrakhan
    {  324, 6577,   83016,  5, }, // Europe/Athens
    {  324, 6285,   83021,  8, }, // Europe/Belgrade
    {  324, 4659,   83029,  6, }, // Europe/Berlin
    {  324, 6437,   83035, 11, }, // Europe/Bratislava
    {  324, 6178,   83046,  8, }, // Europe/Brussels
    {  324, 3830,   83054,  8, }, // Europe/Bucharest
    {  324, 3708,   83062,  9, }, // Europe/Budapest
    {  324, 8751,   83071,  8, }, // Europe/Busingen
    {  324, 2475,   83079,  7, }, // Europe/Chisinau
    {  324, 6238,   83086,  8, }, // Europe/Copenhagen
    {  324, 2260,   83094,  5, }, // Europe/Dublin
    {  324, 6256,   83099, 10, }, // Europe/Gibraltar
    {  324, 8767,   83109,  6, }, // Europe/Guernsey
    {  324, 6561,   83115,  8, }, // Europe/Helsinki
    {  324, 8783,   83123, 10, }, // Europe/Isle_of_Man
    {  324, 1097,   83133,  9, }, // Europe/Istanbul
    {  324, 8802,   83142,  5, }, // Europe/Jersey
    {  324, 3881,   83147, 12, }, // Europe/Kaliningrad
    {  324, 8924,   83159,  5, }, // Europe/Kirov
    {  324, 2419,   83164,  3, }, // Europe/Kyiv
    {  324, 3078,   83167,  6, }, // Europe/Lisbon
    {  324, 6455,   83173, 10, }, // Europe/Ljubljana
    {  324, 2393,   83183,  4, }, // Europe/London
    {  324, 6314,   83187,  9, }, // Europe/Luxembourg
    {  324, 4794,   83196,  7, }, // Europe/Madrid
    {  324, 6346,   83203,  6, }, // Europe/Malta
    {  324, 6531,   83209,  7, }, // Europe/Mariehamn
    {  324, 3595,   83216,  6, }, // Europe/Minsk
    {  324, 6359,   83222,  6, }, // Europe/Monaco
    {  324, 3443,   83228,  6, }, // Europe/Moscow
    {  324, 6407,   83234,  5, }, // Europe/Oslo
    {  324, 4186,   83239,  5, }, // Europe/Paris
    {  324, 6373,   83244,  9, }, // Europe/Podgorica
    {  324, 6224,   83253,  5, }, // Europe/Prague
    {  324, 8816,   83258,  4, }, // Europe/Riga
    {  324, 6273,   81987,  3, }, // Europe/Rome
    {  324, 4233,   83262,  5, }, // Europe/Samara
    {  324, 6419,   83267, 10, }, // Europe/San_Marino
    {  324, 6194,   83277,  8, }, // Europe/Sarajevo
    {  324, 4353,   83285,  7, }, // Europe/Saratov
    {  324, 8828,   83292, 11, }, // Europe/Simferopol
    {  324, 6332,   83303,  7, }, // Europe/Skopje
    {  324, 6548,   83310,  6, }, // Europe/Sofia
    {  324, 6472,   83316,  8, }, // Europe/Stockholm
    {  324, 8846,   83324,  6, }, // Europe/Tallinn
    {  324, 6135,   83330,  6, }, // Europe/Tirane
    {  324, 8861,   83336, 10, }, // Europe/Ulyanovsk
    {  324, 6301,   83346,  5, }, // Europe/Vaduz
    {  324, 6516,   83351,  6, }, // Europe/Vatican
    {  324, 6164,   83357,  6, }, // Europe/Vienna
    {  324, 8878,   83363,  8, }, // Europe/Vilnius
    {  324, 4629,   83371, 11, }, // Europe/Volgograd
    {  324, 3055,   83382,  5, }, // Europe/Warsaw
    {  324, 6210,   83387,  6, }, // Europe/Zagreb
    {  324, 6489,   83393,  5, }, // Europe/Zurich
    {  324, 4980,   83398, 12, }, // Indian/Antananarivo
    {  324, 6910,   83410,  5, }, // Indian/Chagos
    {  324, 5979,   83415,  7, }, // Indian/Christmas
    {  324, 5996,   83422,  5, }, // Indian/Cocos
    {  324, 4931,   83427,  6, }, // Indian/Comoro
    {  324, 6624,   83433, 10, }, // Indian/Kerguelen
    {  324, 7223,   83443,  4, }, // Indian/Mahe
    {  324, 7013,   83447,  8, }, // Indian/Maldives
    {  324, 3971,   83455,  8, }, // Indian/Mauritius
    {  324, 5000,   83463,  5, }, // Indian/Mayotte
    {  324, 7174,   83468,  8, }, // Indian/Reunion
    {  324, 4324,   83476,  5, }, // Pacific/Apia
    {  324,  932,   83481,  7, }, // Pacific/Auckland
    {  324, 3608,   83488, 10, }, // Pacific/Bougainville
    {  324, 2863,   83498,  3, }, // Pacific/Chatham
    {  324, 3022,   83501,  3, }, // Pacific/Chuuk
    {  324, 2201,   83504,  5, }, // Pacific/Easter
    {  324, 7323,   83509,  4, }, // Pacific/Efate
    {  324, 7290,   83513,  7, }, // Pacific/Fakaofo
    {  324, 3804,   83520,  4, }, // Pacific/Fiji
    {  324, 7306,   83524,  8, }, // Pacific/Funafuti
    {  324, 6641,   83532,  9, }, // Pacific/Galapagos
    {  324, 6659,   83541,  8, }, // Pacific/Gambier
    {  324, 3724,   83549,  8, }, // Pacific/Guadalcanal
    {  324, 5935,   83557,  5, }, // Pacific/Guam
    {  324, 2929,   83562,  8, }, // Pacific/Honolulu
    {  324, 2897,   83570,  8, }, // Pacific/Kanton
    {  324, 3900,   83578, 10, }, // Pacific/Kiritimati
    {  324, 6982,   83588,  5, }, // Pacific/Kosrae
    {  324, 2708,   83593,  9, }, // Pacific/Kwajalein
    {  324, 4779,   83602,  6, }, // Pacific/Majuro
    {  324, 3953,   83608,  9, }, // Pacific/Marquesas
    {  324, 8909,   83617,  5, }, // Pacific/Midway
    {  324, 7047,   83622,  4, }, // Pacific/Nauru
    {  324, 7095,   83626,  4, }, // Pacific/Niue
    {  324, 4070,   83630,  6, }, // Pacific/Norfolk
    {  324, 7061,   83636,  5, }, // Pacific/Noumea
    {  324, 2991,   83641,  9, }, // Pacific/Pago_Pago
    {  324, 7118,   83650,  4, }, // Pacific/Palau
    {  324, 7157,   83654,  8, }, // Pacific/Pitcairn
    {  324, 2961,   83662,  6, }, // Pacific/Pohnpei
    {  324, 4709,   83668, 14, }, // Pacific/Port_Moresby
    {  324, 6009,   83682,  9, }, // Pacific/Rarotonga
    {  324, 5948,   83691,  5, }, // Pacific/Saipan
    {  324, 7275,   83696,  6, }, // Pacific/Tahiti
    {  324, 6675,   83702,  6, }, // Pacific/Tarawa
    {  324, 4485,   83708,  9, }, // Pacific/Tongatapu
    {  324, 7355,   83717,  3, }, // Pacific/Wake
    {  324, 7368,   83720,  5, }, // Pacific/Wallis
    {  325,   14,       0,  6, }, // Africa/Asmara Gusii/Latin/Kenya
    {  325, 4337,       6,  8, }, // Africa/Sao_Tome
    {  325, 4154,      14,  8, }, // America/Asuncion
    {  325,  237,      22,  8, }, // America/Atikokan
    {  325, 7586,      30, 17, }, // America/Bahia_Banderas
    {  325, 3770,      47,  6, }, // America/Cancun
    {  325, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  325, 5640,      66,  7, }, // America/Curacao
    {  325,  481,      73, 13, }, // America/Indiana/Knox
    {  325, 7924,      86, 16, }, // America/Indiana/Marengo
    {  325, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  325, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  325, 8001,     139, 14, }, // America/Indiana/Vevay
    {  325, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  325, 8049,     171, 16, }, // America/Indiana/Winamac
    {  325, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  325, 5849,     207, 22, }, // America/Lower_Princes
    {  325, 8183,     229,  6, }, // America/Merida
    {  325, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  325, 8292,     255, 20, }, // America/North_Dakota/Center
    {  325, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  325,  393,     298,  4, }, // America/Nuuk
    {  325, 6862,     302, 16, }, // America/Scoresbysund
    {  325, 8471,     318, 14, }, // America/St_Barthelemy
    {  325, 2045,     332, 10, }, // America/St_Johns
    {  325, 5780,     342,  9, }, // America/St_Kitts
    {  325, 5797,     351,  9, }, // America/St_Lucia
    {  325,  855,     360, 10, }, // America/St_Thomas
    {  325, 5830,     370, 11, }, // America/St_Vincent
    {  325, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  325, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  325, 1152,     413,  9, }, // Asia/Kathmandu
    {  325,  992,     422,  7, }, // Asia/Kolkata
    {  325, 1178,     429,  5, }, // Asia/Macau
    {  325, 1202,     442,  6, }, // Asia/Yangon
    {  325, 1379,     448,  5, }, // Atlantic/Faroe
    {  325, 6784,     453, 10, }, // Atlantic/St_Helena
    {  325, 2419,     463,  4, }, // Europe/Kyiv
    {  325, 7174,     467,  7, }, // Indian/Reunion
    {  325, 3022,     474,  5, }, // Pacific/Chuuk
    {  325, 2961,     479,  7, }, // Pacific/Pohnpei
    {  326,   14,       0,  6, }, // Africa/Asmara Hausa/Latin/Nigeria
    {  326, 4337,       6,  8, }, // Africa/Sao_Tome
    {  326, 4154,      14,  8, }, // America/Asuncion
    {  326,  237,      22,  8, }, // America/Atikokan
    {  326, 7586,      30, 17, }, // America/Bahia_Banderas
    {  326, 3770,      47,  6, }, // America/Cancun
    {  326, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  326, 5640,      66,  7, }, // America/Curacao
    {  326,  481,      73, 13, }, // America/Indiana/Knox
    {  326, 7924,      86, 16, }, // America/Indiana/Marengo
    {  326, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  326, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  326, 8001,     139, 14, }, // America/Indiana/Vevay
    {  326, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  326, 8049,     171, 16, }, // America/Indiana/Winamac
    {  326, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  326, 5849,     207, 22, }, // America/Lower_Princes
    {  326, 8183,     229,  6, }, // America/Merida
    {  326, 8264,   83725, 23, }, // America/North_Dakota/Beulah
    {  326, 8292,   83748, 23, }, // America/North_Dakota/Center
    {  326, 8320,   83771, 26, }, // America/North_Dakota/New_Salem
    {  326,  393,     298,  4, }, // America/Nuuk
    {  326, 6862,     302, 16, }, // America/Scoresbysund
    {  326, 8471,     318, 14, }, // America/St_Barthelemy
    {  326, 2045,     332, 10, }, // America/St_Johns
    {  326, 5780,     342,  9, }, // America/St_Kitts
    {  326, 5797,     351,  9, }, // America/St_Lucia
    {  326,  855,     360, 10, }, // America/St_Thomas
    {  326, 5830,     370, 11, }, // America/St_Vincent
    {  326, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  326, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  326, 1152,     413,  9, }, // Asia/Kathmandu
    {  326,  992,     422,  7, }, // Asia/Kolkata
    {  326, 1178,     429,  5, }, // Asia/Macau
    {  326, 1202,     442,  6, }, // Asia/Yangon
    {  326, 1379,     448,  5, }, // Atlantic/Faroe
    {  326, 6784,     453, 10, }, // Atlantic/St_Helena
    {  326, 2419,     463,  4, }, // Europe/Kyiv
    {  326, 7174,     467,  7, }, // Indian/Reunion
    {  326, 3022,     474,  5, }, // Pacific/Chuuk
    {  326, 2961,     479,  7, }, // Pacific/Pohnpei
    {  327, 8264,     235, 20, }, // America/North_Dakota/Beulah Hausa/Arabic/Nigeria
    {  327, 8292,     255, 20, }, // America/North_Dakota/Center
    {  327, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  328, 8264,     235, 20, }, // America/North_Dakota/Beulah Hausa/Arabic/Sudan
    {  328, 8292,     255, 20, }, // America/North_Dakota/Center
    {  328, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  331,   14,       0,  6, }, // Africa/Asmara Hawaiian/Latin/United States
    {  331, 4337,       6,  8, }, // Africa/Sao_Tome
    {  331, 4154,      14,  8, }, // America/Asuncion
    {  331,  237,      22,  8, }, // America/Atikokan
    {  331, 7586,      30, 17, }, // America/Bahia_Banderas
    {  331, 3770,      47,  6, }, // America/Cancun
    {  331, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  331, 5640,      66,  7, }, // America/Curacao
    {  331,  481,      73, 13, }, // America/Indiana/Knox
    {  331, 7924,      86, 16, }, // America/Indiana/Marengo
    {  331, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  331, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  331, 8001,     139, 14, }, // America/Indiana/Vevay
    {  331, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  331, 8049,     171, 16, }, // America/Indiana/Winamac
    {  331, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  331, 5849,     207, 22, }, // America/Lower_Princes
    {  331, 8183,     229,  6, }, // America/Merida
    {  331, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  331, 8292,     255, 20, }, // America/North_Dakota/Center
    {  331, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  331,  393,     298,  4, }, // America/Nuuk
    {  331, 6862,     302, 16, }, // America/Scoresbysund
    {  331, 8471,     318, 14, }, // America/St_Barthelemy
    {  331, 2045,     332, 10, }, // America/St_Johns
    {  331, 5780,     342,  9, }, // America/St_Kitts
    {  331, 5797,     351,  9, }, // America/St_Lucia
    {  331,  855,     360, 10, }, // America/St_Thomas
    {  331, 5830,     370, 11, }, // America/St_Vincent
    {  331, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  331, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  331, 1152,     413,  9, }, // Asia/Kathmandu
    {  331,  992,     422,  7, }, // Asia/Kolkata
    {  331, 1178,     429,  5, }, // Asia/Macau
    {  331, 1202,     442,  6, }, // Asia/Yangon
    {  331, 1379,     448,  5, }, // Atlantic/Faroe
    {  331, 6784,     453, 10, }, // Atlantic/St_Helena
    {  331, 2419,     463,  4, }, // Europe/Kyiv
    {  331, 7174,     467,  7, }, // Indian/Reunion
    {  331, 3022,     474,  5, }, // Pacific/Chuuk
    {  331, 2961,     479,  7, }, // Pacific/Pohnpei
    {  332, 6751,   83797,  7, }, // Africa/Abidjan Hebrew/Hebrew/Israel
    {  332, 6723,   83804,  4, }, // Africa/Accra
    {  332, 4961,   83808,  9, }, // Africa/Addis_Ababa
    {  332, 7383,   83817,  6, }, // Africa/Algiers
    {  332,   14,   83823,  5, }, // Africa/Asmara
    {  332,   44,   83828,  5, }, // Africa/Bamako
    {  332, 5159,   83833,  6, }, // Africa/Bangui
    {  332, 6709,   83839,  6, }, // Africa/Banjul
    {  332, 7398,   83845,  5, }, // Africa/Bissau
    {  332, 4859,   83850,  6, }, // Africa/Blantyre
    {  332, 5189,   83856,  7, }, // Africa/Brazzaville
    {  332, 4824,   83863, 10, }, // Africa/Bujumbura
    {  332, 2242,   83873,  4, }, // Africa/Cairo
    {  332, 4019,   83877,  7, }, // Africa/Casablanca
    {  332, 7412,   83884,  5, }, // Africa/Ceuta
    {  332, 6736,   83889,  7, }, // Africa/Conakry
    {  332, 6803,   83896,  4, }, // Africa/Dakar
    {  332, 5032,   83900, 10, }, // Africa/Dar_es_Salaam
    {  332, 4945,   83910,  7, }, // Africa/Djibouti
    {  332, 5145,   83917,  5, }, // Africa/Douala
    {  332, 5068,   83922,  7, }, // Africa/El_Aaiun
    {  332, 6816,   83929,  7, }, // Africa/Freetown
    {  332, 4808,   83936,  7, }, // Africa/Gaborone
    {  332, 4917,   83943,  5, }, // Africa/Harare
    {  332, 4381,   83948,  9, }, // Africa/Johannesburg
    {  332, 4401,   83957,  5, }, // Africa/Juba
    {  332, 5053,   83962,  6, }, // Africa/Kampala
    {  332, 4426,   83968,  5, }, // Africa/Khartoum
    {  332, 4875,   83973,  5, }, // Africa/Kigali
    {  332, 5208,   83978,  6, }, // Africa/Kinshasa
    {  332, 4646,   83984,  5, }, // Africa/Lagos
    {  332, 5238,   83989,  8, }, // Africa/Libreville
    {  332, 6832,   83997,  4, }, // Africa/Lome
    {  332, 5113,   84001,  6, }, // Africa/Luanda
    {  332, 4841,   84007,  9, }, // Africa/Lubumbashi
    {  332, 4903,   84016,  5, }, // Africa/Lusaka
    {  332, 5224,   84021,  4, }, // Africa/Malabo
    {  332, 4889,   84025,  6, }, // Africa/Maputo
    {  332, 5099,   84031,  4, }, // Africa/Maseru
    {  332, 5084,   84035,  5, }, // Africa/Mbabane
    {  332, 5015,   84040,  7, }, // Africa/Mogadishu
    {  332, 6997,   84047,  8, }, // Africa/Monrovia
    {  332, 3755,   84055,  7, }, // Africa/Nairobi
    {  332, 5173,   84062,  6, }, // Africa/Ndjamena
    {  332, 5256,   84068,  6, }, // Africa/Niamey
    {  332, 6766,   84074,  8, }, // Africa/Nouakchott
    {  332, 6690,   84082,  9, }, // Africa/Ouagadougou
    {  332, 5127,   84091, 10, }, // Africa/Porto-Novo
    {  332, 4337,   84101,  8, }, // Africa/Sao_Tome
    {  332, 2732,   84109,  7, }, // Africa/Tripoli
    {  332, 6503,   84116,  5, }, // Africa/Tunis
    {  332, 4054,   84121,  7, }, // Africa/Windhoek
    {  332,  132,   84128,  4, }, // America/Adak
    {  332, 3183,   84132,  7, }, // America/Anchorage
    {  332, 5524,   84139,  8, }, // America/Anguilla
    {  332, 5541,   84147,  8, }, // America/Antigua
    {  332, 4456,   84155,  9, }, // America/Araguaina
    {  332,  166,   84164, 12, }, // America/Argentina/Buenos_Aires
    {  332,   91,   84176,  6, }, // America/Argentina/Catamarca
    {  332,  270,   84182,  7, }, // America/Argentina/Cordoba
    {  332,  441,   84189,  5, }, // America/Argentina/Jujuy
    {  332, 7425,   84194,  8, }, // America/Argentina/La_Rioja
    {  332,  565,   84202,  6, }, // America/Argentina/Mendoza
    {  332, 7452,   84208, 11, }, // America/Argentina/Rio_Gallegos
    {  332, 7483,   84219,  4, }, // America/Argentina/Salta
    {  332, 7507,   84223,  7, }, // America/Argentina/San_Juan
    {  332, 5497,   84230,  8, }, // America/Argentina/San_Luis
    {  332, 7534,   84238,  6, }, // America/Argentina/Tucuman
    {  332, 7560,   84244,  8, }, // America/Argentina/Ushuaia
    {  332, 5557,   84252,  5, }, // America/Aruba
    {  332, 4154,   84257,  8, }, // America/Asuncion
    {  332,  237,   84265,  7, }, // America/Atikokan
    {  332, 3581,   84272,  5, }, // America/Bahia
    {  332, 7586,   84277, 11, }, // America/Bahia_Banderas
    {  332, 5571,   84288,  6, }, // America/Barbados
    {  332, 7609,   84294,  3, }, // America/Belem
    {  332, 5309,   84297,  4, }, // America/Belize
    {  332, 7623,   84301, 10, }, // America/Blanc-Sablon
    {  332, 7644,   84311, 10, }, // America/Boa_Vista
    {  332, 4263,   84321,  6, }, // America/Bogota
    {  332, 7662,   84327,  5, }, // America/Boise
    {  332, 7676,   84332, 14, }, // America/Cambridge_Bay
    {  332, 7698,   84346, 10, }, // America/Campo_Grande
    {  332, 3770,   84356,  5, }, // America/Cancun
    {  332, 4596,   84361,  5, }, // America/Caracas
    {  332, 4247,   84366,  5, }, // America/Cayenne
    {  332, 5398,   84371,  5, }, // America/Cayman
    {  332, 3251,   84376,  5, }, // America/Chicago
    {  332, 7719,   84381,  9, }, // America/Chihuahua
    {  332, 7737,   84390, 11, }, // America/Ciudad_Juarez
    {  332, 5324,   84401, 10, }, // America/Costa_Rica
    {  332, 7759,   84411,  6, }, // America/Creston
    {  332, 3693,   84417,  6, }, // America/Cuiaba
    {  332, 5640,   84423,  7, }, // America/Curacao
    {  332, 7775,   84430, 10, }, // America/Danmarkshavn
    {  332, 7796,   84440,  5, }, // America/Dawson
    {  332, 7811,   84445, 10, }, // America/Dawson_Creek
    {  332,  805,   84455,  5, }, // America/Denver
    {  332, 3351,   84460,  6, }, // America/Detroit
    {  332, 5656,   84466,  8, }, // America/Dominica
    {  332,  893,   84474,  8, }, // America/Edmonton
    {  332, 7832,   84482,  7, }, // America/Eirunepe
    {  332, 5343,   84489,  9, }, // America/El_Salvador
    {  332, 7849,   84498, 10, }, // America/Fort_Nelson
    {  332, 7869,   84508,  8, }, // America/Fortaleza
    {  332, 7887,   84516,  9, }, // America/Glace_Bay
    {  332, 6844,   84525,  7, }, // America/Goose_Bay
    {  332, 4514,   84532,  9, }, // America/Grand_Turk
    {  332, 5687,   84351,  5, }, // America/Grenada
    {  332, 5703,   84541,  7, }, // America/Guadeloupe
    {  332, 3662,   84548,  7, }, // America/Guatemala
    {  332, 4743,   84555,  8, }, // America/Guayaquil
    {  332, 6895,   84563,  5, }, // America/Guyana
    {  332, 1923,   84568,  6, }, // America/Halifax
    {  332, 2221,   84574,  6, }, // America/Havana
    {  332, 7905,   84580,  7, }, // America/Hermosillo
    {  332,  348,   84587, 12, }, // America/Indiana/Indianapolis
    {  332,  481,   84599, 14, }, // America/Indiana/Knox
    {  332, 7924,   84613, 15, }, // America/Indiana/Marengo
    {  332, 7948,   84628, 19, }, // America/Indiana/Petersburg
    {  332, 7975,   84647, 17, }, // America/Indiana/Tell_City
    {  332, 8001,   84664, 17, }, // America/Indiana/Vevay
    {  332, 8023,   84681, 16, }, // America/Indiana/Vincennes
    {  332, 8049,   84697, 16, }, // America/Indiana/Winamac
    {  332, 8073,   84713,  7, }, // America/Inuvik
    {  332,  660,   84720,  8, }, // America/Iqaluit
    {  332, 2665,   84728,  7, }, // America/Jamaica
    {  332, 5282,   84735,  5, }, // America/Juneau
    {  332,  521,   84740,  8, }, // America/Kentucky/Louisville
    {  332, 8088,   84748, 17, }, // America/Kentucky/Monticello
    {  332, 5621,   84765,  8, }, // America/Kralendijk
    {  332, 4278,   84773,  6, }, // America/La_Paz
    {  332, 7132,   84779,  4, }, // America/Lima
    {  332, 3390,   84783, 10, }, // America/Los_Angeles
    {  332, 5849,   84793, 20, }, // America/Lower_Princes
    {  332, 8116,   84813,  6, }, // America/Maceio
    {  332, 8131,   84819,  6, }, // America/Managua
    {  332, 1892,   84825,  5, }, // America/Manaus
    {  332, 5814,   84830,  5, }, // America/Marigot
    {  332, 5722,   84835,  7, }, // America/Martinique
    {  332, 8147,   84842,  7, }, // America/Matamoros
    {  332, 2779,   84849,  5, }, // America/Mazatlan
    {  332, 8165,   84854,  7, }, // America/Menominee
    {  332, 8183,   84861,  5, }, // America/Merida
    {  332, 8198,   84866,  7, }, // America/Metlakatla
    {  332, 2811,   84873, 11, }, // America/Mexico_City
    {  332, 4293,   84884,  6, }, // America/Miquelon
    {  332, 8217,   84890,  7, }, // America/Moncton
    {  332, 8233,   84897,  7, }, // America/Monterrey
    {  332, 4000,   84904, 10, }, // America/Montevideo
    {  332, 5741,   84914,  7, }, // America/Montserrat
    {  332, 5383,   84921,  4, }, // America/Nassau
    {  332, 3294,   84925,  8, }, // America/New_York
    {  332, 8251,   84933,  3, }, // America/Nome
    {  332, 1834,   84936,  7, }, // America/Noronha
    {  332, 8264,   84943, 20, }, // America/North_Dakota/Beulah
    {  332, 8292,   84963, 19, }, // America/North_Dakota/Center
    {  332, 8320,   84982, 24, }, // America/North_Dakota/New_Salem
    {  332,  393,   85006,  5, }, // America/Nuuk
    {  332, 8351,   85011,  9, }, // America/Ojinaga
    {  332, 5413,   85020,  4, }, // America/Panama
    {  332, 6106,   85024,  7, }, // America/Paramaribo
    {  332, 3224,   85031,  6, }, // America/Phoenix
    {  332, 3847,   85037, 13, }, // America/Port-au-Prince
    {  332, 5871,   85050, 14, }, // America/Port_of_Spain
    {  332, 8367,   85064, 10, }, // America/Porto_Velho
    {  332, 5760,   85074, 11, }, // America/Puerto_Rico
    {  332, 3932,   85085, 10, }, // America/Punta_Arenas
    {  332, 8387,   85095, 11, }, // America/Rankin_Inlet
    {  332, 8408,   85106,  5, }, // America/Recife
    {  332, 1979,   85111,  6, }, // America/Regina
    {  332, 8423,   85117,  6, }, // America/Resolute
    {  332,  695,   85123,  9, }, // America/Rio_Branco
    {  332, 8440,   85132,  5, }, // America/Santarem
    {  332, 2165,   85137,  7, }, // America/Santiago
    {  332, 6044,   85144, 12, }, // America/Santo_Domingo
    {  332, 1862,   85156,  9, }, // America/Sao_Paulo
    {  332, 6862,   85165, 11, }, // America/Scoresbysund
    {  332, 8457,   85176,  5, }, // America/Sitka
    {  332, 8471,   85181, 10, }, // America/St_Barthelemy
    {  332, 2045,   85191,  9, }, // America/St_Johns
    {  332, 5780,   85200,  8, }, // America/St_Kitts
    {  332, 5797,   85208,  9, }, // America/St_Lucia
    {  332,  855,   85217,  9, }, // America/St_Thomas
    {  332, 5830,   85226, 10, }, // America/St_Vincent
    {  332, 8493,   85236, 11, }, // America/Swift_Current
    {  332, 5363,   85247,  9, }, // America/Tegucigalpa
    {  332, 5673,   85256,  4, }, // America/Thule
    {  332,  313,   85260,  7, }, // America/Tijuana
    {  332,  608,   85267,  7, }, // America/Toronto
    {  332, 5605,   85274,  7, }, // America/Tortola
    {  332, 2077,   85281,  6, }, // America/Vancouver
    {  332, 2128,   85287,  9, }, // America/Whitehorse
    {  332,  734,   85296,  7, }, // America/Winnipeg
    {  332, 8515,   85303,  6, }, // America/Yakutat
    {  332, 5918,   85309,  5, }, // Antarctica/Casey
    {  332, 6027,   85314,  6, }, // Antarctica/Davis
    {  332, 6066,   85320, 15, }, // Antarctica/DumontDUrville
    {  332, 8531,   85335,  7, }, // Antarctica/Macquarie
    {  332, 7029,   85342,  5, }, // Antarctica/Mawson
    {  332, 7076,   85347,  7, }, // Antarctica/McMurdo
    {  332, 8552,   85354,  5, }, // Antarctica/Palmer
    {  332, 7189,   85359,  6, }, // Antarctica/Rothera
    {  332, 7258,   85365,  6, }, // Antarctica/Syowa
    {  332, 8570,   85371,  4, }, // Antarctica/Troll
    {  332, 7337,   85375,  6, }, // Antarctica/Vostok
    {  332, 1413,   85381, 10, }, // Arctic/Longyearbyen
    {  332, 5487,   85391,  3, }, // Asia/Aden
    {  332, 5297,   85394,  6, }, // Asia/Almaty
    {  332, 3870,   85400,  4, }, // Asia/Amman
    {  332, 5428,   85404,  5, }, // Asia/Anadyr
    {  332, 5440,   85409,  5, }, // Asia/Aqtau
    {  332, 5270,   85414,  6, }, // Asia/Aqtobe
    {  332,  964,   85420,  5, }, // Asia/Ashgabat
    {  332, 8587,   85425,  6, }, // Asia/Atyrau
    {  332, 3509,   85431,  4, }, // Asia/Baghdad
    {  332, 5451,   85435,  6, }, // Asia/Bahrain
    {  332, 3555,   85441,  4, }, // Asia/Baku
    {  332, 4368,   85445,  6, }, // Asia/Bangkok
    {  332, 3473,   85451,  6, }, // Asia/Barnaul
    {  332, 3988,   85457,  6, }, // Asia/Beirut
    {  332, 3680,   85463,  5, }, // Asia/Bishkek
    {  332, 5906,   85468,  6, }, // Asia/Brunei
    {  332, 4503,   85474,  5, }, // Asia/Chita
    {  332, 5963,   85479,  8, }, // Asia/Choibalsan
    {  332, 4413,   85487,  7, }, // Asia/Colombo
    {  332, 4442,   85494,  4, }, // Asia/Damascus
    {  332, 1060,   85498,  4, }, // Asia/Dhaka
    {  332, 6125,   85502,  4, }, // Asia/Dili
    {  332, 3498,   85506,  5, }, // Asia/Dubai
    {  332, 6092,   85511,  6, }, // Asia/Dushanbe
    {  332, 8599,   85517,  7, }, // Asia/Famagusta
    {  332, 8614,   85524,  3, }, // Asia/Gaza
    {  332, 4697,   85527,  5, }, // Asia/Hebron
    {  332, 1226,   85532, 15, }, // Asia/Ho_Chi_Minh
    {  332, 2576,   85547,  9, }, // Asia/Hong_Kong
    {  332, 4673,   85556,  4, }, // Asia/Hovd
    {  332, 4086,   85560,  8, }, // Asia/Irkutsk
    {  332, 6969,   85568,  6, }, // Asia/Jakarta
    {  332, 6955,   85574,  9, }, // Asia/Jayapura
    {  332, 1257,   85583,  7, }, // Asia/Jerusalem
    {  332, 3462,   85590,  5, }, // Asia/Kabul
    {  332, 4218,   85595,  7, }, // Asia/Kamchatka
    {  332, 4141,   85602,  6, }, // Asia/Karachi
    {  332, 1152,   85608,  6, }, // Asia/Kathmandu
    {  332, 8624,   85614,  6, }, // Asia/Khandyga
    {  332,  992,   85620,  6, }, // Asia/Kolkata
    {  332, 4099,   85626, 10, }, // Asia/Krasnoyarsk
    {  332, 4761,   85636, 12, }, // Asia/Kuala_Lumpur
    {  332, 5893,   85648,  7, }, // Asia/Kuching
    {  332, 5464,   85655,  5, }, // Asia/Kuwait
    {  332, 1178,   85660,  4, }, // Asia/Macau
    {  332, 3919,   85664,  4, }, // Asia/Magadan
    {  332, 1316,   85668,  7, }, // Asia/Makassar
    {  332, 7145,   85675,  5, }, // Asia/Manila
    {  332, 6883,   85680,  5, }, // Asia/Muscat
    {  332, 2446,   85685,  7, }, // Asia/Nicosia
    {  332, 8638,   85692, 11, }, // Asia/Novokuznetsk
    {  332, 4037,   85703, 11, }, // Asia/Novosibirsk
    {  332, 4131,   85714,  5, }, // Asia/Omsk
    {  332, 7108,   85719,  4, }, // Asia/Oral
    {  332, 6924,   85723,  7, }, // Asia/Phnom_Penh
    {  332, 8656,   85730,  8, }, // Asia/Pontianak
    {  332, 4116,   85738,  9, }, // Asia/Pyongyang
    {  332, 5476,   85747,  4, }, // Asia/Qatar
    {  332, 8671,   85751,  7, }, // Asia/Qostanay
    {  332, 4171,   85758,  9, }, // Asia/Qyzylorda
    {  332, 3486,   85767,  4, }, // Asia/Riyadh
    {  332, 4310,   85771,  5, }, // Asia/Sakhalin
    {  332, 7208,   85776,  6, }, // Asia/Samarkand
    {  332, 3116,   85782,  5, }, // Asia/Seoul
    {  332, 1020,   85787,  5, }, // Asia/Shanghai
    {  332, 3137,   85792,  7, }, // Asia/Singapore
    {  332, 4199,   85799, 13, }, // Asia/Srednekolymsk
    {  332, 3100,   85812,  6, }, // Asia/Taipei
    {  332, 4683,   85818,  5, }, // Asia/Tashkent
    {  332, 3817,   85823,  7, }, // Asia/Tbilisi
    {  332, 2638,   85830,  4, }, // Asia/Tehran
    {  332, 1284,   85834,  7, }, // Asia/Thimphu
    {  332, 2687,   85841,  5, }, // Asia/Tokyo
    {  332, 4474,   85846,  5, }, // Asia/Tomsk
    {  332, 1346,   85851, 11, }, // Asia/Ulaanbaatar
    {  332, 1126,   85862,  7, }, // Asia/Urumqi
    {  332, 8685,   85869,  8, }, // Asia/Ust-Nera
    {  332, 6940,   85877,  8, }, // Asia/Vientiane
    {  332, 4612,   85885, 10, }, // Asia/Vladivostok
    {  332, 4730,   85895,  6, }, // Asia/Yakutsk
    {  332, 1202,   85901,  5, }, // Asia/Yangon
    {  332, 3785,   85906, 10, }, // Asia/Yekaterinburg
    {  332, 3649,   85916,  6, }, // Asia/Yerevan
    {  332, 3565,   85922, 14, }, // Atlantic/Azores
    {  332, 5588,   85936,  6, }, // Atlantic/Bermuda
    {  332, 6591,   85942, 13, }, // Atlantic/Canary
    {  332, 3629,   85955,  7, }, // Atlantic/Cape_Verde
    {  332, 1379,   85962,  4, }, // Atlantic/Faroe
    {  332, 8699,   85966,  6, }, // Atlantic/Madeira
    {  332, 2614,   85972, 10, }, // Atlantic/Reykjavik
    {  332, 7235,   85982, 13, }, // Atlantic/South_Georgia
    {  332, 6784,   85995,  8, }, // Atlantic/St_Helena
    {  332, 6607,   86003,  5, }, // Atlantic/Stanley
    {  332, 1654,   86008,  6, }, // Australia/Adelaide
    {  332, 1619,   86014,  8, }, // Australia/Brisbane
    {  332, 1783,   86022,  9, }, // Australia/Broken_Hill
    {  332, 1567,   86031,  6, }, // Australia/Darwin
    {  332, 3539,   86037,  5, }, // Australia/Eucla
    {  332, 1500,   86042,  5, }, // Australia/Hobart
    {  332, 8716,   86047,  6, }, // Australia/Lindeman
    {  332, 1531,   86053, 12, }, // Australia/Lord_Howe
    {  332, 1711,   86065,  6, }, // Australia/Melbourne
    {  332, 1746,   86071,  4, }, // Australia/Perth
    {  332, 1447,   86075,  5, }, // Australia/Sydney
    {  332, 6390,   86080,  7, }, // Europe/Amsterdam
    {  332, 6149,   86087,  6, }, // Europe/Andorra
    {  332, 3522,   86093,  6, }, // Europe/Astrakhan
    {  332, 6577,   86099,  5, }, // Europe/Athens
    {  332, 6285,   86104,  5, }, // Europe/Belgrade
    {  332, 4659,   86109,  5, }, // Europe/Berlin
    {  332, 6437,   86114,  8, }, // Europe/Bratislava
    {  332, 6178,   86122,  5, }, // Europe/Brussels
    {  332, 3830,   86127,  6, }, // Europe/Bucharest
    {  332, 3708,   86133,  6, }, // Europe/Budapest
    {  332, 8751,   86139,  7, }, // Europe/Busingen
    {  332, 2475,   86146,  6, }, // Europe/Chisinau
    {  332, 6238,   86152,  7, }, // Europe/Copenhagen
    {  332, 2260,   86159,  5, }, // Europe/Dublin
    {  332, 6256,   86164,  7, }, // Europe/Gibraltar
    {  332, 8767,   86171,  5, }, // Europe/Guernsey
    {  332, 6561,   86176,  7, }, // Europe/Helsinki
    {  332, 8783,   86183,  7, }, // Europe/Isle_of_Man
    {  332, 1097,   86190,  8, }, // Europe/Istanbul
    {  332, 8802,   86198,  5, }, // Europe/Jersey
    {  332, 3881,   86203,  9, }, // Europe/Kaliningrad
    {  332, 8924,   86212,  5, }, // Europe/Kirov
    {  332, 2419,   86217,  4, }, // Europe/Kyiv
    {  332, 3078,   86221,  6, }, // Europe/Lisbon
    {  332, 6455,   86227,  8, }, // Europe/Ljubljana
    {  332, 2393,   86235,  6, }, // Europe/London
    {  332, 6314,   86241,  9, }, // Europe/Luxembourg
    {  332, 4794,   86250,  5, }, // Europe/Madrid
    {  332, 6346,   86255,  4, }, // Europe/Malta
    {  332, 6531,   86259,  8, }, // Europe/Mariehamn
    {  332, 3595,   86267,  5, }, // Europe/Minsk
    {  332, 6359,   86272,  5, }, // Europe/Monaco
    {  332, 3443,   86277,  6, }, // Europe/Moscow
    {  332, 6407,   86283,  5, }, // Europe/Oslo
    {  332, 4186,   86288,  4, }, // Europe/Paris
    {  332, 6373,   86292,  9, }, // Europe/Podgorica
    {  332, 6224,   86301,  4, }, // Europe/Prague
    {  332, 8816,   86305,  4, }, // Europe/Riga
    {  332, 6273,   86309,  4, }, // Europe/Rome
    {  332, 4233,   83824,  4, }, // Europe/Samara
    {  332, 6419,   86313,  8, }, // Europe/San_Marino
    {  332, 6194,   86321,  6, }, // Europe/Sarajevo
    {  332, 4353,   86327,  6, }, // Europe/Saratov
    {  332, 8828,   86333,  9, }, // Europe/Simferopol
    {  332, 6332,   86342,  6, }, // Europe/Skopje
    {  332, 6548,   86348,  5, }, // Europe/Sofia
    {  332, 6472,   86353,  8, }, // Europe/Stockholm
    {  332, 8846,   86361,  5, }, // Europe/Tallinn
    {  332, 6135,   86366,  5, }, // Europe/Tirane
    {  332, 8861,   86371, 10, }, // Europe/Ulyanovsk
    {  332, 6301,   86381,  5, }, // Europe/Vaduz
    {  332, 6516,   86386,  7, }, // Europe/Vatican
    {  332, 6164,   86393,  4, }, // Europe/Vienna
    {  332, 8878,   86397,  5, }, // Europe/Vilnius
    {  332, 4629,   86402,  8, }, // Europe/Volgograd
    {  332, 3055,   86410,  4, }, // Europe/Warsaw
    {  332, 6210,   86414,  5, }, // Europe/Zagreb
    {  332, 6489,   86419,  5, }, // Europe/Zurich
    {  332, 4980,   86424,  9, }, // Indian/Antananarivo
    {  332, 6910,   86433,  6, }, // Indian/Chagos
    {  332, 5979,   86439, 11, }, // Indian/Christmas
    {  332, 5996,   86450,  5, }, // Indian/Cocos
    {  332, 4931,   86455,  6, }, // Indian/Comoro
    {  332, 6624,   86461,  7, }, // Indian/Kerguelen
    {  332, 7223,   86468,  3, }, // Indian/Mahe
    {  332, 7013,   86471, 15, }, // Indian/Maldives
    {  332, 3971,   86486,  9, }, // Indian/Mauritius
    {  332, 5000,   86495,  5, }, // Indian/Mayotte
    {  332, 7174,   86500,  7, }, // Indian/Reunion
    {  332, 4324,   86507,  4, }, // Pacific/Apia
    {  332,  932,   86511,  6, }, // Pacific/Auckland
    {  332, 3608,   86517,  8, }, // Pacific/Bougainville
    {  332, 2863,   86525,  7, }, // Pacific/Chatham
    {  332, 3022,   86532,  4, }, // Pacific/Chuuk
    {  332, 2201,   86536,  8, }, // Pacific/Easter
    {  332, 7323,   86544,  4, }, // Pacific/Efate
    {  332, 7290,   86548,  6, }, // Pacific/Fakaofo
    {  332, 3804,   86554,  5, }, // Pacific/Fiji
    {  332, 7306,   86559,  7, }, // Pacific/Funafuti
    {  332, 6641,   86566,  7, }, // Pacific/Galapagos
    {  332, 6659,   86573, 10, }, // Pacific/Gambier
    {  332, 3724,   86583,  9, }, // Pacific/Guadalcanal
    {  332, 5935,   86592,  4, }, // Pacific/Guam
    {  332, 2929,   86596,  8, }, // Pacific/Honolulu
    {  332, 2897,   86604,  8, }, // Pacific/Kanton
    {  332, 3900,   86612, 10, }, // Pacific/Kiritimati
    {  332, 6982,   86622,  5, }, // Pacific/Kosrae
    {  332, 2708,   86627, 10, }, // Pacific/Kwajalein
    {  332, 4779,   86637,  7, }, // Pacific/Majuro
    {  332, 3953,   86644,  9, }, // Pacific/Marquesas
    {  332, 8909,   86653,  7, }, // Pacific/Midway
    {  332, 7047,   86660,  5, }, // Pacific/Nauru
    {  332, 7095,   86665,  5, }, // Pacific/Niue
    {  332, 4070,   86670,  6, }, // Pacific/Norfolk
    {  332, 7061,   86676,  5, }, // Pacific/Noumea
    {  332, 2991,   86681,  9, }, // Pacific/Pago_Pago
    {  332, 7118,   86690,  4, }, // Pacific/Palau
    {  332, 7157,   86694,  6, }, // Pacific/Pitcairn
    {  332, 2961,   86700,  6, }, // Pacific/Pohnpei
    {  332, 4709,   86706, 11, }, // Pacific/Port_Moresby
    {  332, 6009,   86717,  9, }, // Pacific/Rarotonga
    {  332, 5948,   86726,  6, }, // Pacific/Saipan
    {  332, 7275,   86732,  5, }, // Pacific/Tahiti
    {  332, 6675,   86737,  7, }, // Pacific/Tarawa
    {  332, 4485,   86744,  8, }, // Pacific/Tongatapu
    {  332, 7355,   86752,  4, }, // Pacific/Wake
    {  332, 7368,   86756,  5, }, // Pacific/Wallis
    {  333, 6751,   28423,  7, }, // Africa/Abidjan Hindi/Devanagari/India
    {  333, 6723,   86761,  5, }, // Africa/Accra
    {  333, 4961,   86766, 10, }, // Africa/Addis_Ababa
    {  333, 7383,   86776,  9, }, // Africa/Algiers
    {  333,   14,   86785,  7, }, // Africa/Asmara
    {  333,   44,   86792,  6, }, // Africa/Bamako
    {  333, 5159,   86798,  6, }, // Africa/Bangui
    {  333, 6709,   86804,  6, }, // Africa/Banjul
    {  333, 7398,   86810,  5, }, // Africa/Bissau
    {  333, 4859,   86815,  9, }, // Africa/Blantyre
    {  333, 5189,   86824, 11, }, // Africa/Brazzaville
    {  333, 4824,   86835,  9, }, // Africa/Bujumbura
    {  333, 2242,   86844,  5, }, // Africa/Cairo
    {  333, 4019,   28524, 11, }, // Africa/Casablanca
    {  333, 7412,   86849,  5, }, // Africa/Ceuta
    {  333, 6736,   86854,  8, }, // Africa/Conakry
    {  333, 6803,   86862,  4, }, // Africa/Dakar
    {  333, 5032,   86866, 11, }, // Africa/Dar_es_Salaam
    {  333, 4945,   86877,  6, }, // Africa/Djibouti
    {  333, 5145,   86883,  5, }, // Africa/Douala
    {  333, 5068,   86888,  8, }, // Africa/El_Aaiun
    {  333, 6816,   86896,  9, }, // Africa/Freetown
    {  333, 4808,   86905,  7, }, // Africa/Gaborone
    {  333, 4917,   86912,  5, }, // Africa/Harare
    {  333, 4381,   86917, 10, }, // Africa/Johannesburg
    {  333, 4401,   28613,  4, }, // Africa/Juba
    {  333, 5053,   86927,  6, }, // Africa/Kampala
    {  333, 4426,   28625,  7, }, // Africa/Khartoum
    {  333, 4875,   86933,  6, }, // Africa/Kigali
    {  333, 5208,   86939,  7, }, // Africa/Kinshasa
    {  333, 4646,   86946,  5, }, // Africa/Lagos
    {  333, 5238,   86951, 10, }, // Africa/Libreville
    {  333, 6832,   86961,  3, }, // Africa/Lome
    {  333, 5113,   86964,  6, }, // Africa/Luanda
    {  333, 4841,   86970,  9, }, // Africa/Lubumbashi
    {  333, 4903,   28680,  6, }, // Africa/Lusaka
    {  333, 5224,   86979,  5, }, // Africa/Malabo
    {  333, 4889,   86984,  6, }, // Africa/Maputo
    {  333, 5099,   28698,  6, }, // Africa/Maseru
    {  333, 5084,   86990,  7, }, // Africa/Mbabane
    {  333, 5015,   86997,  8, }, // Africa/Mogadishu
    {  333, 6997,   87005,  9, }, // Africa/Monrovia
    {  333, 3755,   87014,  6, }, // Africa/Nairobi
    {  333, 5173,   87020, 10, }, // Africa/Ndjamena
    {  333, 5256,   87030,  6, }, // Africa/Niamey
    {  333, 6766,   87036,  7, }, // Africa/Nouakchott
    {  333, 6690,   87043,  7, }, // Africa/Ouagadougou
    {  333, 5127,   87050, 11, }, // Africa/Porto-Novo
    {  333, 4337,   87061,  7, }, // Africa/Sao_Tome
    {  333, 2732,   87068,  8, }, // Africa/Tripoli
    {  333, 6503,   28787,  7, }, // Africa/Tunis
    {  333, 4054,   87076,  7, }, // Africa/Windhoek
    {  333,  132,   87083,  3, }, // America/Adak
    {  333, 3183,   28806,  6, }, // America/Anchorage
    {  333, 5524,   87086,  8, }, // America/Anguilla
    {  333, 5541,   87094,  7, }, // America/Antigua
    {  333, 4456,   87101,  9, }, // America/Araguaina
    {  333,  166,   87110, 11, }, // America/Argentina/Buenos_Aires
    {  333,   91,   28849, 10, }, // America/Argentina/Catamarca
    {  333,  270,   87121,  8, }, // America/Argentina/Cordoba
    {  333,  441,   87129,  5, }, // America/Argentina/Jujuy
    {  333, 7425,   87134,  8, }, // America/Argentina/La_Rioja
    {  333,  565,   87142,  8, }, // America/Argentina/Mendoza
    {  333, 7452,   87150, 12, }, // America/Argentina/Rio_Gallegos
    {  333, 7483,   28896,  6, }, // America/Argentina/Salta
    {  333, 7507,   87162,  9, }, // America/Argentina/San_Juan
    {  333, 5497,   87171,  7, }, // America/Argentina/San_Luis
    {  333, 7534,   87178,  6, }, // America/Argentina/Tucuman
    {  333, 7560,   87184,  7, }, // America/Argentina/Ushuaia
    {  333, 5557,   87191,  5, }, // America/Aruba
    {  333, 4154,   87196,  8, }, // America/Asuncion
    {  333,  237,   87204,  8, }, // America/Atikokan
    {  333, 3581,   87212,  5, }, // America/Bahia
    {  333, 7586,   87217, 15, }, // America/Bahia_Banderas
    {  333, 5571,   87232,  8, }, // America/Barbados
    {  333, 7609,   28985,  5, }, // America/Belem
    {  333, 5309,   87240,  6, }, // America/Belize
    {  333, 7623,   87246, 13, }, // America/Blanc-Sablon
    {  333, 7644,   87259, 10, }, // America/Boa_Vista
    {  333, 4263,   87269,  6, }, // America/Bogota
    {  333, 7662,   87275,  5, }, // America/Boise
    {  333, 7676,   87280, 15, }, // America/Cambridge_Bay
    {  333, 7698,   87295, 13, }, // America/Campo_Grande
    {  333, 3770,   87308,  6, }, // America/Cancun
    {  333, 4596,   87314,  6, }, // America/Caracas
    {  333, 4247,   29068,  5, }, // America/Cayenne
    {  333, 5398,   29073,  5, }, // America/Cayman
    {  333, 3251,   87320,  6, }, // America/Chicago
    {  333, 7719,   87326,  8, }, // America/Chihuahua
    {  333, 7737,   87334, 16, }, // America/Ciudad_Juarez
    {  333, 5324,   87350, 11, }, // America/Costa_Rica
    {  333, 7759,   29101,  8, }, // America/Creston
    {  333, 3693,   87361,  7, }, // America/Cuiaba
    {  333, 5640,   87368,  7, }, // America/Curacao
    {  333, 7775,   87375, 11, }, // America/Danmarkshavn
    {  333, 7796,   87386,  4, }, // America/Dawson
    {  333, 7811,   87390, 10, }, // America/Dawson_Creek
    {  333,  805,   87400,  5, }, // America/Denver
    {  333, 3351,   87405,  8, }, // America/Detroit
    {  333, 5656,   87413,  8, }, // America/Dominica
    {  333,  893,   87421,  6, }, // America/Edmonton
    {  333, 7832,   29182,  7, }, // America/Eirunepe
    {  333, 5343,   87427, 11, }, // America/El_Salvador
    {  333, 7849,   87438, 13, }, // America/Fort_Nelson
    {  333, 7869,   87451, 12, }, // America/Fortaleza
    {  333, 7887,   87463, 11, }, // America/Glace_Bay
    {  333, 6844,   87474,  9, }, // America/Goose_Bay
    {  333, 4514,   87483, 11, }, // America/Grand_Turk
    {  333, 5687,   29248,  8, }, // America/Grenada
    {  333, 5703,   87494,  9, }, // America/Guadeloupe
    {  333, 3662,   29265, 10, }, // America/Guatemala
    {  333, 4743,   87503,  9, }, // America/Guayaquil
    {  333, 6895,   87512,  5, }, // America/Guyana
    {  333, 1923,   87517, 10, }, // America/Halifax
    {  333, 2221,   87527,  5, }, // America/Havana
    {  333, 7905,   87532, 11, }, // America/Hermosillo
    {  333,  348,   87543, 13, }, // America/Indiana/Indianapolis
    {  333,  481,   87556, 15, }, // America/Indiana/Knox
    {  333, 7924,   87571, 17, }, // America/Indiana/Marengo
    {  333, 7948,   87588, 20, }, // America/Indiana/Petersburg
    {  333, 7975,   87608, 18, }, // America/Indiana/Tell_City
    {  333, 8001,   87626, 14, }, // America/Indiana/Vevay
    {  333, 8023,   87640, 18, }, // America/Indiana/Vincennes
    {  333, 8049,   87658, 17, }, // America/Indiana/Winamac
    {  333, 8073,   29458,  6, }, // America/Inuvik
    {  333,  660,   87675,  7, }, // America/Iqaluit
    {  333, 2665,   87682,  5, }, // America/Jamaica
    {  333, 5282,   87687,  7, }, // America/Juneau
    {  333,  521,   87694,  8, }, // America/Kentucky/Louisville
    {  333, 8088,   87702, 17, }, // America/Kentucky/Monticello
    {  333, 5621,   87719, 12, }, // America/Kralendijk
    {  333, 4278,   87731,  7, }, // America/La_Paz
    {  333, 7132,   29523,  4, }, // America/Lima
    {  333, 3390,   87738, 11, }, // America/Los_Angeles
    {  333, 5849,   87749, 22, }, // America/Lower_Princes
    {  333, 8116,   87771,  5, }, // America/Maceio
    {  333, 8131,   87776,  7, }, // America/Managua
    {  333, 1892,   87783,  4, }, // America/Manaus
    {  333, 5814,   87787,  7, }, // America/Marigot
    {  333, 5722,   87794,  9, }, // America/Martinique
    {  333, 8147,   87803,  9, }, // America/Matamoros
    {  333, 2779,   87812,  9, }, // America/Mazatlan
    {  333, 8165,   87821,  8, }, // America/Menominee
    {  333, 8183,   29617,  6, }, // America/Merida
    {  333, 8198,   87829, 12, }, // America/Metlakatla
    {  333, 2811,   87841, 13, }, // America/Mexico_City
    {  333, 4293,   87854,  7, }, // America/Miquelon
    {  333, 8217,   87861,  6, }, // America/Moncton
    {  333, 8233,   87867,  9, }, // America/Monterrey
    {  333, 4000,   87876, 11, }, // America/Montevideo
    {  333, 5741,   87887,  9, }, // America/Montserrat
    {  333, 5383,   87896,  5, }, // America/Nassau
    {  333, 3294,   87901,  9, }, // America/New_York
    {  333, 8251,   87823,  3, }, // America/Nome
    {  333, 1834,   87910,  8, }, // America/Noronha
    {  333, 8264,   87918, 20, }, // America/North_Dakota/Beulah
    {  333, 8292,   87938, 19, }, // America/North_Dakota/Center
    {  333, 8320,   87957, 24, }, // America/North_Dakota/New_Salem
    {  333,  393,   87981,  3, }, // America/Nuuk
    {  333, 8351,   87984,  9, }, // America/Ojinaga
    {  333, 5413,   87993,  5, }, // America/Panama
    {  333, 6106,   87998, 10, }, // America/Paramaribo
    {  333, 3224,   88008,  8, }, // America/Phoenix
    {  333, 3847,   88016, 14, }, // America/Port-au-Prince
    {  333, 5871,   88030, 15, }, // America/Port_of_Spain
    {  333, 8367,   88045, 13, }, // America/Porto_Velho
    {  333, 5760,   88058, 11, }, // America/Puerto_Rico
    {  333, 3932,   88069, 12, }, // America/Punta_Arenas
    {  333, 8387,   88081, 12, }, // America/Rankin_Inlet
    {  333, 8408,   88093,  7, }, // America/Recife
    {  333, 1979,   29890,  6, }, // America/Regina
    {  333, 8423,   88100,  9, }, // America/Resolute
    {  333,  695,   88109, 12, }, // America/Rio_Branco
    {  333, 8440,   88121,  8, }, // America/Santarem
    {  333, 2165,   88129,  9, }, // America/Santiago
    {  333, 6044,   88138, 13, }, // America/Santo_Domingo
    {  333, 1862,   88151,  9, }, // America/Sao_Paulo
    {  333, 6862,   88160, 13, }, // America/Scoresbysund
    {  333, 8457,   88173,  6, }, // America/Sitka
    {  333, 8471,   88179, 15, }, // America/St_Barthelemy
    {  333, 2045,   88194,  9, }, // America/St_Johns
    {  333, 5780,   88203, 10, }, // America/St_Kitts
    {  333, 5797,   88213, 11, }, // America/St_Lucia
    {  333,  855,   88224,  9, }, // America/St_Thomas
    {  333, 5830,   88233, 12, }, // America/St_Vincent
    {  333, 8493,   88245, 13, }, // America/Swift_Current
    {  333, 5363,   88258, 11, }, // America/Tegucigalpa
    {  333, 5673,   88269,  6, }, // America/Thule
    {  333,  313,   88275,  7, }, // America/Tijuana
    {  333,  608,   88282,  6, }, // America/Toronto
    {  333, 5605,   88288,  8, }, // America/Tortola
    {  333, 2077,   88296,  7, }, // America/Vancouver
    {  333, 2128,   88303, 11, }, // America/Whitehorse
    {  333,  734,   88314,  7, }, // America/Winnipeg
    {  333, 8515,   88321,  7, }, // America/Yakutat
    {  333, 5918,   30132,  4, }, // Antarctica/Casey
    {  333, 6027,   88328,  5, }, // Antarctica/Davis
    {  333, 6066,   88333, 19, }, // Antarctica/DumontDUrville
    {  333, 8531,   88352,  7, }, // Antarctica/Macquarie
    {  333, 7029,   88359,  4, }, // Antarctica/Mawson
    {  333, 7076,   88363,  9, }, // Antarctica/McMurdo
    {  333, 8552,   88372,  4, }, // Antarctica/Palmer
    {  333, 7189,   88376,  6, }, // Antarctica/Rothera
    {  333, 7258,   88382,  6, }, // Antarctica/Syowa
    {  333, 8570,   88388,  5, }, // Antarctica/Troll
    {  333, 7337,   88393,  7, }, // Antarctica/Vostok
    {  333, 1413,   88400, 13, }, // Arctic/Longyearbyen
    {  333, 5487,   88413,  4, }, // Asia/Aden
    {  333, 5297,   88417,  7, }, // Asia/Almaty
    {  333, 3870,   88424,  6, }, // Asia/Amman
    {  333, 5428,   88430,  6, }, // Asia/Anadyr
    {  333, 5440,   30237,  6, }, // Asia/Aqtau
    {  333, 5270,   88436,  6, }, // Asia/Aqtobe
    {  333,  964,   30250,  8, }, // Asia/Ashgabat
    {  333, 8587,   88442,  5, }, // Asia/Atyrau
    {  333, 3509,   88447,  5, }, // Asia/Baghdad
    {  333, 5451,   88452,  5, }, // Asia/Bahrain
    {  333, 3555,   87366,  4, }, // Asia/Baku
    {  333, 4368,   88457,  6, }, // Asia/Bangkok
    {  333, 3473,   88463,  6, }, // Asia/Barnaul
    {  333, 3988,   88469,  5, }, // Asia/Beirut
    {  333, 3680,   30295,  7, }, // Asia/Bishkek
    {  333, 5906,   88474,  7, }, // Asia/Brunei
    {  333, 4503,   88481,  6, }, // Asia/Chita
    {  333, 5963,   88487,  8, }, // Asia/Choibalsan
    {  333, 4413,   88495,  6, }, // Asia/Colombo
    {  333, 4442,   88501,  7, }, // Asia/Damascus
    {  333, 1060,   88508,  4, }, // Asia/Dhaka
    {  333, 6125,   88512,  4, }, // Asia/Dili
    {  333, 3498,   88516,  4, }, // Asia/Dubai
    {  333, 6092,   88520,  7, }, // Asia/Dushanbe
    {  333, 8599,   88527, 11, }, // Asia/Famagusta
    {  333, 8614,   88538,  5, }, // Asia/Gaza
    {  333, 4697,   88543,  7, }, // Asia/Hebron
    {  333, 1226,   88550, 16, }, // Asia/Ho_Chi_Minh
    {  333, 2576,   88566,  9, }, // Asia/Hong_Kong
    {  333, 4673,   88575,  5, }, // Asia/Hovd
    {  333, 4086,   88580, 10, }, // Asia/Irkutsk
    {  333, 6969,   88590,  7, }, // Asia/Jakarta
    {  333, 6955,   30411,  7, }, // Asia/Jayapura
    {  333, 1257,   88597,  6, }, // Asia/Jerusalem
    {  333, 3462,   88603,  5, }, // Asia/Kabul
    {  333, 4218,   88608,  7, }, // Asia/Kamchatka
    {  333, 4141,   30440,  5, }, // Asia/Karachi
    {  333, 1152,   88615,  8, }, // Asia/Kathmandu
    {  333, 8624,   88623,  7, }, // Asia/Khandyga
    {  333,  992,   88630,  7, }, // Asia/Kolkata
    {  333, 4099,   88637, 15, }, // Asia/Krasnoyarsk
    {  333, 4761,   88652, 10, }, // Asia/Kuala_Lumpur
    {  333, 5893,   30496,  6, }, // Asia/Kuching
    {  333, 5464,   30502,  5, }, // Asia/Kuwait
    {  333, 1178,   88662,  4, }, // Asia/Macau
    {  333, 3919,   30512,  7, }, // Asia/Magadan
    {  333, 1316,   88666,  6, }, // Asia/Makassar
    {  333, 7145,   30528,  5, }, // Asia/Manila
    {  333, 6883,   88672,  5, }, // Asia/Muscat
    {  333, 2446,   88677,  8, }, // Asia/Nicosia
    {  333, 8638,   88685, 16, }, // Asia/Novokuznetsk
    {  333, 4037,   88701, 13, }, // Asia/Novosibirsk
    {  333, 4131,   88714,  6, }, // Asia/Omsk
    {  333, 7108,   88720,  3, }, // Asia/Oral
    {  333, 6924,   88723,  9, }, // Asia/Phnom_Penh
    {  333, 8656,   88732,  9, }, // Asia/Pontianak
    {  333, 4116,   88741, 10, }, // Asia/Pyongyang
    {  333, 5476,   88751,  3, }, // Asia/Qatar
    {  333, 8671,   88754,  8, }, // Asia/Qostanay
    {  333, 4171,   88762, 11, }, // Asia/Qyzylorda
    {  333, 3486,   88773,  5, }, // Asia/Riyadh
    {  333, 4310,   88778,  6, }, // Asia/Sakhalin
    {  333, 7208,   88784,  6, }, // Asia/Samarkand
    {  333, 3116,   88790,  4, }, // Asia/Seoul
    {  333, 1020,   88794,  5, }, // Asia/Shanghai
    {  333, 3137,   30658,  8, }, // Asia/Singapore
    {  333, 4199,   88799, 16, }, // Asia/Srednekolymsk
    {  333, 3100,   88815,  6, }, // Asia/Taipei
    {  333, 4683,   88821,  6, }, // Asia/Tashkent
    {  333, 3817,   88827,  7, }, // Asia/Tbilisi
    {  333, 2638,   30704,  6, }, // Asia/Tehran
    {  333, 1284,   88834,  5, }, // Asia/Thimphu
    {  333, 2687,   88839,  6, }, // Asia/Tokyo
    {  333, 4474,   88845,  7, }, // Asia/Tomsk
    {  333, 1346,   88852,  8, }, // Asia/Ulaanbaatar
    {  333, 1126,   88860,  7, }, // Asia/Urumqi
    {  333, 8685,   88867, 10, }, // Asia/Ust-Nera
    {  333, 6940,   88877,  9, }, // Asia/Vientiane
    {  333, 4612,   88886, 13, }, // Asia/Vladivostok
    {  333, 4730,   30773,  9, }, // Asia/Yakutsk
    {  333, 1202,   30782,  5, }, // Asia/Yangon
    {  333, 3785,   88899, 13, }, // Asia/Yekaterinburg
    {  333, 3649,   88912,  7, }, // Asia/Yerevan
    {  333, 3565,   88919,  7, }, // Atlantic/Azores
    {  333, 5588,   88926,  6, }, // Atlantic/Bermuda
    {  333, 6591,   88932,  6, }, // Atlantic/Canary
    {  333, 3629,   88938,  8, }, // Atlantic/Cape_Verde
    {  333, 1379,   88946,  5, }, // Atlantic/Faroe
    {  333, 8699,   88951,  5, }, // Atlantic/Madeira
    {  333, 2614,   88956,  9, }, // Atlantic/Reykjavik
    {  333, 7235,   88965, 15, }, // Atlantic/South_Georgia
    {  333, 6784,   88980, 11, }, // Atlantic/St_Helena
    {  333, 6607,   88991,  7, }, // Atlantic/Stanley
    {  333, 1654,   88998,  6, }, // Australia/Adelaide
    {  333, 1619,   89004,  8, }, // Australia/Brisbane
    {  333, 1783,   89012, 10, }, // Australia/Broken_Hill
    {  333, 1567,   89022,  7, }, // Australia/Darwin
    {  333, 3539,   30920,  6, }, // Australia/Eucla
    {  333, 1500,   89029,  7, }, // Australia/Hobart
    {  333, 8716,   89036,  8, }, // Australia/Lindeman
    {  333, 1531,   89044, 10, }, // Australia/Lord_Howe
    {  333, 1711,   89054,  8, }, // Australia/Melbourne
    {  333, 1746,   89062,  4, }, // Australia/Perth
    {  333, 1447,   30962,  5, }, // Australia/Sydney
    {  333, 6390,   89066, 10, }, // Europe/Amsterdam
    {  333, 6149,   89076,  6, }, // Europe/Andorra
    {  333, 3522,   89082, 10, }, // Europe/Astrakhan
    {  333, 6577,   89092,  5, }, // Europe/Athens
    {  333, 6285,   31000,  8, }, // Europe/Belgrade
    {  333, 4659,   89097,  6, }, // Europe/Berlin
    {  333, 6437,   89103, 12, }, // Europe/Bratislava
    {  333, 6178,   89115,  9, }, // Europe/Brussels
    {  333, 3830,   89124, 10, }, // Europe/Bucharest
    {  333, 3708,   89134,  9, }, // Europe/Budapest
    {  333, 8751,   89143, 10, }, // Europe/Busingen
    {  333, 2475,   89153,  7, }, // Europe/Chisinau
    {  333, 6238,   89160,  9, }, // Europe/Copenhagen
    {  333, 2260,   89169,  5, }, // Europe/Dublin
    {  333, 6256,   89174, 10, }, // Europe/Gibraltar
    {  333, 8767,   89184,  6, }, // Europe/Guernsey
    {  333, 6561,   89190,  8, }, // Europe/Helsinki
    {  333, 8783,   89198, 11, }, // Europe/Isle_of_Man
    {  333, 1097,   89209,  9, }, // Europe/Istanbul
    {  333, 8802,   89218,  5, }, // Europe/Jersey
    {  333, 3881,   89223, 12, }, // Europe/Kaliningrad
    {  333, 8924,   89235,  5, }, // Europe/Kirov
    {  333, 2419,   89240,  3, }, // Europe/Kyiv
    {  333, 3078,   31158,  6, }, // Europe/Lisbon
    {  333, 6455,   89243, 11, }, // Europe/Ljubljana
    {  333, 2393,   89254,  4, }, // Europe/London
    {  333, 6314,   89258, 10, }, // Europe/Luxembourg
    {  333, 4794,   89268,  7, }, // Europe/Madrid
    {  333, 6346,   31199,  6, }, // Europe/Malta
    {  333, 6531,   89275,  9, }, // Europe/Mariehamn
    {  333, 3595,   89284,  6, }, // Europe/Minsk
    {  333, 6359,   89290,  6, }, // Europe/Monaco
    {  333, 3443,   89296,  6, }, // Europe/Moscow
    {  333, 6407,   89302,  5, }, // Europe/Oslo
    {  333, 4186,   31234,  5, }, // Europe/Paris
    {  333, 6373,   89307, 10, }, // Europe/Podgorica
    {  333, 6224,   89317,  5, }, // Europe/Prague
    {  333, 8816,   89322,  4, }, // Europe/Riga
    {  333, 6273,   88949,  3, }, // Europe/Rome
    {  333, 4233,   89326,  5, }, // Europe/Samara
    {  333, 6419,   89331, 10, }, // Europe/San_Marino
    {  333, 6194,   89341,  8, }, // Europe/Sarajevo
    {  333, 4353,   89349,  7, }, // Europe/Saratov
    {  333, 8828,   89356, 12, }, // Europe/Simferopol
    {  333, 6332,   89368,  8, }, // Europe/Skopje
    {  333, 6548,   89376,  7, }, // Europe/Sofia
    {  333, 6472,   89383,  8, }, // Europe/Stockholm
    {  333, 8846,   89391,  5, }, // Europe/Tallinn
    {  333, 6135,   89396,  6, }, // Europe/Tirane
    {  333, 8861,   89402, 12, }, // Europe/Ulyanovsk
    {  333, 6301,   89414,  6, }, // Europe/Vaduz
    {  333, 6516,   89420,  6, }, // Europe/Vatican
    {  333, 6164,   89426,  5, }, // Europe/Vienna
    {  333, 8878,   89431,  8, }, // Europe/Vilnius
    {  333, 4629,   89439, 11, }, // Europe/Volgograd
    {  333, 3055,   89450,  5, }, // Europe/Warsaw
    {  333, 6210,   89455,  8, }, // Europe/Zagreb
    {  333, 6489,   89463,  9, }, // Europe/Zurich
    {  333, 4980,   89472, 11, }, // Indian/Antananarivo
    {  333, 6910,   89483,  5, }, // Indian/Chagos
    {  333, 5979,   89488,  7, }, // Indian/Christmas
    {  333, 5996,   89495,  5, }, // Indian/Cocos
    {  333, 4931,   89500,  6, }, // Indian/Comoro
    {  333, 6624,   89506,  7, }, // Indian/Kerguelen
    {  333, 7223,   31451,  4, }, // Indian/Mahe
    {  333, 7013,   89513,  6, }, // Indian/Maldives
    {  333, 3971,   89519,  6, }, // Indian/Mauritius
    {  333, 5000,   89525,  8, }, // Indian/Mayotte
    {  333, 7174,   89533,  8, }, // Indian/Reunion
    {  333, 4324,   89541,  5, }, // Pacific/Apia
    {  333,  932,   89546,  6, }, // Pacific/Auckland
    {  333, 3608,   89552,  8, }, // Pacific/Bougainville
    {  333, 2863,   89560,  4, }, // Pacific/Chatham
    {  333, 3022,   89564,  2, }, // Pacific/Chuuk
    {  333, 2201,   89566,  5, }, // Pacific/Easter
    {  333, 7323,   89571,  5, }, // Pacific/Efate
    {  333, 7290,   89576,  9, }, // Pacific/Fakaofo
    {  333, 3804,   89585,  5, }, // Pacific/Fiji
    {  333, 7306,   89590, 12, }, // Pacific/Funafuti
    {  333, 6641,   89602,  9, }, // Pacific/Galapagos
    {  333, 6659,   89611,  7, }, // Pacific/Gambier
    {  333, 3724,   89618, 10, }, // Pacific/Guadalcanal
    {  333, 5935,   29571,  4, }, // Pacific/Guam
    {  333, 2929,   89628,  8, }, // Pacific/Honolulu
    {  333, 2897,   89636,  7, }, // Pacific/Kanton
    {  333, 3900,   31600, 10, }, // Pacific/Kiritimati
    {  333, 6982,   89643,  6, }, // Pacific/Kosrae
    {  333, 2708,   89649, 10, }, // Pacific/Kwajalein
    {  333, 4779,   89659,  6, }, // Pacific/Majuro
    {  333, 3953,   89665,  8, }, // Pacific/Marquesas
    {  333, 8909,   89673,  5, }, // Pacific/Midway
    {  333, 7047,   89678,  4, }, // Pacific/Nauru
    {  333, 7095,   89682,  4, }, // Pacific/Niue
    {  333, 4070,   89686,  7, }, // Pacific/Norfolk
    {  333, 7061,   89693,  6, }, // Pacific/Noumea
    {  333, 2991,   89699,  9, }, // Pacific/Pago_Pago
    {  333, 7118,   89708,  4, }, // Pacific/Palau
    {  333, 7157,   31678,  8, }, // Pacific/Pitcairn
    {  333, 2961,   89712,  6, }, // Pacific/Pohnpei
    {  333, 4709,   89718, 14, }, // Pacific/Port_Moresby
    {  333, 6009,   89732,  9, }, // Pacific/Rarotonga
    {  333, 5948,   89741,  6, }, // Pacific/Saipan
    {  333, 7275,   89747,  6, }, // Pacific/Tahiti
    {  333, 6675,   89753,  6, }, // Pacific/Tarawa
    {  333, 4485,   89759,  9, }, // Pacific/Tongatapu
    {  333, 7355,   31736,  3, }, // Pacific/Wake
    {  333, 7368,   31739,  5, }, // Pacific/Wallis
    {  334,   14,   89768,  6, }, // Africa/Asmara Hindi/Latin/India
    {  334, 5127,   24975, 10, }, // Africa/Porto-Novo
    {  334, 4337,   89774,  8, }, // Africa/Sao_Tome
    {  334, 4154,   60024,  8, }, // America/Asuncion
    {  334,  237,   89782, 13, }, // America/Atikokan
    {  334, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  334, 7623,   42653, 12, }, // America/Blanc-Sablon
    {  334, 3770,    3861,  6, }, // America/Cancun
    {  334, 7737,   89795, 13, }, // America/Ciudad_Juarez
    {  334, 5640,   89808,  7, }, // America/Curacao
    {  334,  481,      73, 13, }, // America/Indiana/Knox
    {  334, 7924,      86, 16, }, // America/Indiana/Marengo
    {  334, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  334, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  334, 8001,     139, 14, }, // America/Indiana/Vevay
    {  334, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  334, 8049,     171, 16, }, // America/Indiana/Winamac
    {  334, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  334, 5849,     207, 22, }, // America/Lower_Princes
    {  334, 8183,    3899,  6, }, // America/Merida
    {  334, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  334, 8292,     255, 20, }, // America/North_Dakota/Center
    {  334, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  334,  393,     298,  4, }, // America/Nuuk
    {  334, 6862,     302, 16, }, // America/Scoresbysund
    {  334, 8471,   89815, 13, }, // America/St_Barthelemy
    {  334, 2045,   59489,  9, }, // America/St_Johns
    {  334, 5780,   59498,  8, }, // America/St_Kitts
    {  334, 5797,   59506,  8, }, // America/St_Lucia
    {  334,  855,   59514,  9, }, // America/St_Thomas
    {  334, 5830,   59523, 10, }, // America/St_Vincent
    {  334, 6066,   89828, 14, }, // Antarctica/DumontDUrville
    {  334, 5440,   89842,  5, }, // Asia/Aqtau
    {  334, 1226,   89847,  6, }, // Asia/Ho_Chi_Minh
    {  334, 1152,     413,  9, }, // Asia/Kathmandu
    {  334,  992,     422,  7, }, // Asia/Kolkata
    {  334, 1178,    4222,  5, }, // Asia/Macau
    {  334, 8671,   89853,  8, }, // Asia/Qostanay
    {  334, 1202,   56146,  7, }, // Asia/Yangon
    {  334, 1379,   89861,  6, }, // Atlantic/Faroe
    {  334, 6784,   59533,  9, }, // Atlantic/St_Helena
    {  334, 2419,     463,  4, }, // Europe/Kyiv
    {  334, 7174,   89867,  7, }, // Indian/Reunion
    {  334, 3022,   89874,  4, }, // Pacific/Chuuk
    {  334, 2929,    4512,  8, }, // Pacific/Honolulu
    {  334, 2897,    4520,  9, }, // Pacific/Kanton
    {  334, 2961,    6780,  6, }, // Pacific/Pohnpei
    {  335, 4961,   89878, 12, }, // Africa/Addis_Ababa Hungarian/Latin/Hungary
    {  335, 7383,   89890,  5, }, // Africa/Algiers
    {  335,   14,   89768,  6, }, // Africa/Asmara
    {  335, 2242,   89895,  5, }, // Africa/Cairo
    {  335, 5032,   89900, 13, }, // Africa/Dar_es_Salaam
    {  335, 4945,   89913,  8, }, // Africa/Djibouti
    {  335, 5068,   89921,  7, }, // Africa/El_Aaiun
    {  335, 4426,   89928,  6, }, // Africa/Khartoum
    {  335, 5224,   89934,  6, }, // Africa/Malabo
    {  335, 4337,       6,  8, }, // Africa/Sao_Tome
    {  335, 6503,   89940,  6, }, // Africa/Tunis
    {  335, 4456,   24992,  9, }, // America/Araguaina
    {  335,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  335, 7452,   42641, 12, }, // America/Argentina/Rio_Gallegos
    {  335, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  335, 4154,      14,  8, }, // America/Asuncion
    {  335,  237,      22,  8, }, // America/Atikokan
    {  335, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  335, 7609,   34376,  5, }, // America/Belem
    {  335, 4263,   18898,  6, }, // America/Bogota
    {  335, 3770,      47,  6, }, // America/Cancun
    {  335, 5398,   89946, 15, }, // America/Cayman
    {  335, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  335, 3693,   25007,  6, }, // America/Cuiaba
    {  335, 5640,      66,  7, }, // America/Curacao
    {  335, 5656,   20071,  8, }, // America/Dominica
    {  335, 7832,   25013,  8, }, // America/Eirunepe
    {  335, 5343,    5090,  8, }, // America/El_Salvador
    {  335, 2221,   61885,  7, }, // America/Havana
    {  335,  481,      73, 13, }, // America/Indiana/Knox
    {  335, 7924,      86, 16, }, // America/Indiana/Marengo
    {  335, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  335, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  335, 8001,     139, 14, }, // America/Indiana/Vevay
    {  335, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  335, 8049,     171, 16, }, // America/Indiana/Winamac
    {  335, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  335, 5849,     207, 22, }, // America/Lower_Princes
    {  335, 8116,   25021,  6, }, // America/Maceio
    {  335, 2779,   25036,  8, }, // America/Mazatlan
    {  335, 8183,     229,  6, }, // America/Merida
    {  335, 2811,   89961, 11, }, // America/Mexico_City
    {  335, 8264,   89972, 20, }, // America/North_Dakota/Beulah
    {  335, 8292,   89992, 20, }, // America/North_Dakota/Center
    {  335, 8320,   90012, 23, }, // America/North_Dakota/New_Salem
    {  335,  393,     298,  4, }, // America/Nuuk
    {  335,  695,   72843, 10, }, // America/Rio_Branco
    {  335, 1862,   25145,  9, }, // America/Sao_Paulo
    {  335, 6862,     302, 16, }, // America/Scoresbysund
    {  335, 8471,   25154, 16, }, // America/St_Barthelemy
    {  335, 2045,     332, 10, }, // America/St_Johns
    {  335, 5780,     342,  9, }, // America/St_Kitts
    {  335, 5797,     351,  9, }, // America/St_Lucia
    {  335,  855,     360, 10, }, // America/St_Thomas
    {  335, 5830,     370, 11, }, // America/St_Vincent
    {  335, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  335, 7337,   90035,  7, }, // Antarctica/Vostok
    {  335, 5487,   90042,  4, }, // Asia/Aden
    {  335, 5297,   57310,  8, }, // Asia/Almaty
    {  335, 3870,   56457,  5, }, // Asia/Amman
    {  335, 5428,    5900,  6, }, // Asia/Anadyr
    {  335, 5440,    5906,  5, }, // Asia/Aqtau
    {  335, 5270,   61967,  6, }, // Asia/Aqtobe
    {  335,  964,   25236,  7, }, // Asia/Ashgabat
    {  335, 8587,    5917,  6, }, // Asia/Atyrau
    {  335, 3509,    4063,  6, }, // Asia/Baghdad
    {  335, 5451,    4069,  7, }, // Asia/Bahrain
    {  335, 3988,   56483,  6, }, // Asia/Beirut
    {  335, 3680,   90046,  6, }, // Asia/Bishkek
    {  335, 4503,   90052,  5, }, // Asia/Chita
    {  335, 5963,   90057, 11, }, // Asia/Choibalsan
    {  335, 4442,   90068, 10, }, // Asia/Damascus
    {  335, 1060,   90078,  5, }, // Asia/Dhaka
    {  335, 3498,   56501,  5, }, // Asia/Dubai
    {  335, 6092,   90083,  7, }, // Asia/Dushanbe
    {  335, 8614,   90090,  4, }, // Asia/Gaza
    {  335, 1226,   90094, 16, }, // Asia/Ho_Chi_Minh
    {  335, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  335, 4086,   90110,  8, }, // Asia/Irkutsk
    {  335, 1257,   90118, 10, }, // Asia/Jerusalem
    {  335, 4218,   90128,  9, }, // Asia/Kamchatka
    {  335, 4141,   90137,  7, }, // Asia/Karachi
    {  335, 1152,    6018,  8, }, // Asia/Kathmandu
    {  335, 8624,   31995,  7, }, // Asia/Khandyga
    {  335,  992,   62730,  8, }, // Asia/Kolkata
    {  335, 4099,   90144, 13, }, // Asia/Krasnoyarsk
    {  335, 5893,   90157,  7, }, // Asia/Kuching
    {  335, 5464,   62344,  6, }, // Asia/Kuwait
    {  335, 1178,   90164,  5, }, // Asia/Macau
    {  335, 3919,   19241,  7, }, // Asia/Magadan
    {  335, 1316,    6069,  7, }, // Asia/Makassar
    {  335, 6883,   90169,  7, }, // Asia/Muscat
    {  335, 8638,   90176, 12, }, // Asia/Novokuznetsk
    {  335, 4037,   90188, 13, }, // Asia/Novosibirsk
    {  335, 4131,   90201,  5, }, // Asia/Omsk
    {  335, 4116,   90206,  7, }, // Asia/Pyongyang
    {  335, 5476,    4233,  5, }, // Asia/Qatar
    {  335, 8671,   90213,  9, }, // Asia/Qostanay
    {  335, 4171,    6111,  9, }, // Asia/Qyzylorda
    {  335, 3486,   56612,  5, }, // Asia/Riyadh
    {  335, 4310,   90222,  8, }, // Asia/Sakhalin
    {  335, 7208,   90230, 10, }, // Asia/Samarkand
    {  335, 3116,   90240,  5, }, // Asia/Seoul
    {  335, 1020,   90245,  7, }, // Asia/Shanghai
    {  335, 3137,   90252,  9, }, // Asia/Singapore
    {  335, 4199,   90261, 15, }, // Asia/Srednekolymsk
    {  335, 3100,    6163,  6, }, // Asia/Taipei
    {  335, 4683,   90276,  7, }, // Asia/Tashkent
    {  335, 3817,   90283,  8, }, // Asia/Tbilisi
    {  335, 2638,   56658,  7, }, // Asia/Tehran
    {  335, 1284,   90291,  5, }, // Asia/Thimphu
    {  335, 2687,   90296,  5, }, // Asia/Tokyo
    {  335, 4474,   90301,  6, }, // Asia/Tomsk
    {  335, 1346,   90307,  9, }, // Asia/Ulaanbaatar
    {  335, 1126,   90316,  7, }, // Asia/Urumqi
    {  335, 8685,   90323, 11, }, // Asia/Ust-Nera
    {  335, 6940,   19320,  8, }, // Asia/Vientiane
    {  335, 4612,   90334, 13, }, // Asia/Vladivostok
    {  335, 4730,   90347,  8, }, // Asia/Yakutsk
    {  335, 1202,     442,  6, }, // Asia/Yangon
    {  335, 3785,   90355, 14, }, // Asia/Yekaterinburg
    {  335, 3649,   90369,  7, }, // Asia/Yerevan
    {  335, 3565,   90376, 14, }, // Atlantic/Azores
    {  335, 6591,   90390, 15, }, // Atlantic/Canary
    {  335, 3629,   90405, 18, }, // Atlantic/Cape_Verde
    {  335, 1379,   90423,  6, }, // Atlantic/Faroe
    {  335, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  335, 7235,   90429, 12, }, // Atlantic/South_Georgia
    {  335, 6784,   90441, 11, }, // Atlantic/St_Helena
    {  335, 6390,   90452, 10, }, // Europe/Amsterdam
    {  335, 3522,   90462,  9, }, // Europe/Astrakhan
    {  335, 6577,   56788,  5, }, // Europe/Athens
    {  335, 6285,   90471,  7, }, // Europe/Belgrade
    {  335, 6437,   90478,  7, }, // Europe/Bratislava
    {  335, 6178,   90485,  8, }, // Europe/Brussels
    {  335, 3830,   25479,  8, }, // Europe/Bucharest
    {  335, 8751,   25487,  8, }, // Europe/Busingen
    {  335, 6238,   90493, 10, }, // Europe/Copenhagen
    {  335, 6256,   90503,  9, }, // Europe/Gibraltar
    {  335, 8783,   90512, 10, }, // Europe/Isle_of_Man
    {  335, 1097,   90522,  9, }, // Europe/Istanbul
    {  335, 3881,   90531, 12, }, // Europe/Kaliningrad
    {  335, 2419,   32194,  5, }, // Europe/Kyiv
    {  335, 3078,   90543,  9, }, // Europe/Lisbon
    {  335, 6314,    4414,  9, }, // Europe/Luxembourg
    {  335, 6346,   90552,  5, }, // Europe/Malta
    {  335, 3595,   90557,  6, }, // Europe/Minsk
    {  335, 3443,   90563,  7, }, // Europe/Moscow
    {  335, 4186,   90570,  6, }, // Europe/Paris
    {  335, 6224,   90576,  5, }, // Europe/Prague
    {  335, 6273,   90581,  4, }, // Europe/Rome
    {  335, 4233,   90585,  7, }, // Europe/Samara
    {  335, 6194,   90592,  9, }, // Europe/Sarajevo
    {  335, 4353,   90601,  8, }, // Europe/Saratov
    {  335, 8828,   90609, 11, }, // Europe/Simferopol
    {  335, 6332,   90620,  7, }, // Europe/Skopje
    {  335, 6548,   90627,  6, }, // Europe/Sofia
    {  335, 8846,   21335,  6, }, // Europe/Tallinn
    {  335, 6135,   19608,  6, }, // Europe/Tirane
    {  335, 8861,   90633, 10, }, // Europe/Ulyanovsk
    {  335, 6516,   56887,  7, }, // Europe/Vatican
    {  335, 6164,   90643,  4, }, // Europe/Vienna
    {  335, 4629,   90647,  9, }, // Europe/Volgograd
    {  335, 3055,   90656,  5, }, // Europe/Warsaw
    {  335, 6210,   90661,  6, }, // Europe/Zagreb
    {  335, 6489,    4478,  6, }, // Europe/Zurich
    {  335, 5979,   90667, 16, }, // Indian/Christmas
    {  335, 5996,   90683, 13, }, // Indian/Cocos
    {  335, 4931,   90696,  6, }, // Indian/Comoro
    {  335, 7013,   90702, 15, }, // Indian/Maldives
    {  335, 7174,     467,  7, }, // Indian/Reunion
    {  335, 2863,   90717, 16, }, // Pacific/Chatham
    {  335, 3022,   89874,  4, }, // Pacific/Chuuk
    {  335, 2201,   90733, 15, }, // Pacific/Easter
    {  335, 3804,   90748,  6, }, // Pacific/Fiji
    {  335, 6641,   90754, 18, }, // Pacific/Galapagos
    {  335, 6659,   90772, 16, }, // Pacific/Gambier
    {  335, 2929,    4512,  8, }, // Pacific/Honolulu
    {  335, 2897,    4520,  9, }, // Pacific/Kanton
    {  335, 3900,   90788, 17, }, // Pacific/Kiritimati
    {  335, 6982,   90805, 15, }, // Pacific/Kosrae
    {  335, 2708,   90820, 16, }, // Pacific/Kwajalein
    {  335, 4779,   90836, 13, }, // Pacific/Majuro
    {  335, 3953,   90849, 18, }, // Pacific/Marquesas
    {  335, 8909,   90867, 15, }, // Pacific/Midway
    {  335, 7157,   90882, 17, }, // Pacific/Pitcairn
    {  335, 2961,   90899, 15, }, // Pacific/Pohnpei
    {  335, 7355,   90914, 11, }, // Pacific/Wake
    {  336, 7383,   90925, 11, }, // Africa/Algiers Icelandic/Latin/Iceland
    {  336,   14,       0,  6, }, // Africa/Asmara
    {  336, 7398,   90936,  5, }, // Africa/Bissau
    {  336, 2242,   90941,  5, }, // Africa/Cairo
    {  336, 4945,   90946,  7, }, // Africa/Djibouti
    {  336, 4381,   90953, 14, }, // Africa/Johannesburg
    {  336, 4337,   90967,  8, }, // Africa/Sao_Tome
    {  336, 2732,   90975,  7, }, // Africa/Tripoli
    {  336, 6503,   90982,  9, }, // Africa/Tunis
    {  336, 5524,   90991,  8, }, // America/Anguilla
    {  336, 5541,   90999,  7, }, // America/Antigua
    {  336, 5557,   91006,  5, }, // America/Aruba
    {  336, 4154,      14,  8, }, // America/Asuncion
    {  336,  237,      22,  8, }, // America/Atikokan
    {  336, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  336, 5309,   91011,  5, }, // America/Belize
    {  336, 3770,   91016,  6, }, // America/Cancun
    {  336, 5398,   91022, 12, }, // America/Cayman
    {  336, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  336, 5324,   91034,  9, }, // America/Costa_Rica
    {  336, 5640,      66,  7, }, // America/Curacao
    {  336, 5656,   91043,  8, }, // America/Dominica
    {  336, 5703,   91051,  8, }, // America/Guadeloupe
    {  336, 3662,   31800,  9, }, // America/Guatemala
    {  336, 6895,   91059,  7, }, // America/Guyana
    {  336,  481,      73, 13, }, // America/Indiana/Knox
    {  336, 7924,      86, 16, }, // America/Indiana/Marengo
    {  336, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  336, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  336, 8001,     139, 14, }, // America/Indiana/Vevay
    {  336, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  336, 8049,     171, 16, }, // America/Indiana/Winamac
    {  336, 2665,   91066,  7, }, // America/Jamaica
    {  336, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  336, 5849,     207, 22, }, // America/Lower_Princes
    {  336, 5722,   91073,  8, }, // America/Martinique
    {  336, 8183,    3899,  6, }, // America/Merida
    {  336, 2811,   91081, 10, }, // America/Mexico_City
    {  336, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  336, 8292,     255, 20, }, // America/North_Dakota/Center
    {  336, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  336,  393,     298,  4, }, // America/Nuuk
    {  336, 5760,   91091, 11, }, // America/Puerto_Rico
    {  336, 6862,     302, 16, }, // America/Scoresbysund
    {  336, 8471,   91102, 25, }, // America/St_Barthelemy
    {  336, 2045,     332, 10, }, // America/St_Johns
    {  336, 5780,   91127, 12, }, // America/St_Kitts
    {  336, 5797,   91139, 12, }, // America/St_Lucia
    {  336,  855,   91151, 13, }, // America/St_Thomas
    {  336, 5830,   91164, 14, }, // America/St_Vincent
    {  336, 5605,   91178,  7, }, // America/Tortola
    {  336, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  336, 3509,    4063,  6, }, // Asia/Baghdad
    {  336, 5451,   62334,  6, }, // Asia/Bahrain
    {  336, 3555,   19138,  4, }, // Asia/Baku
    {  336, 3988,   91185,  6, }, // Asia/Beirut
    {  336, 5906,   91191,  6, }, // Asia/Brunei
    {  336, 4413,   91197,  7, }, // Asia/Colombo
    {  336, 4442,    4102,  8, }, // Asia/Damascus
    {  336, 1060,   90078,  5, }, // Asia/Dhaka
    {  336, 1226,   91204, 16, }, // Asia/Ho_Chi_Minh
    {  336, 6969,    4148,  8, }, // Asia/Jakarta
    {  336, 1257,   91220,  9, }, // Asia/Jerusalem
    {  336, 3462,   91229,  5, }, // Asia/Kabul
    {  336, 4218,    4162, 10, }, // Asia/Kamchatka
    {  336, 1152,   19221,  8, }, // Asia/Kathmandu
    {  336,  992,   91234,  8, }, // Asia/Kolkata
    {  336, 4761,   91242, 12, }, // Asia/Kuala_Lumpur
    {  336, 5464,   91254,  6, }, // Asia/Kuwait
    {  336, 1178,   90164,  5, }, // Asia/Macau
    {  336, 2446,   91260,  7, }, // Asia/Nicosia
    {  336, 4116,   32033,  9, }, // Asia/Pyongyang
    {  336, 5476,    4233,  5, }, // Asia/Qatar
    {  336, 8671,     434,  8, }, // Asia/Qostanay
    {  336, 3486,   91267,  5, }, // Asia/Riyadh
    {  336, 3116,   19287,  4, }, // Asia/Seoul
    {  336, 1020,   91272,  7, }, // Asia/Shanghai
    {  336, 3137,   91279,  8, }, // Asia/Singapore
    {  336, 2638,    6175,  7, }, // Asia/Tehran
    {  336, 2687,   91287,  5, }, // Asia/Tokyo
    {  336, 1346,   91292, 10, }, // Asia/Ulaanbaatar
    {  336, 1202,   19328,  6, }, // Asia/Yangon
    {  336, 3565,   91302,  9, }, // Atlantic/Azores
    {  336, 5588,   91311,  7, }, // Atlantic/Bermuda
    {  336, 6591,   91318, 11, }, // Atlantic/Canary
    {  336, 3629,   91329, 14, }, // Atlantic/Cape_Verde
    {  336, 1379,   91343,  8, }, // Atlantic/Faroe
    {  336, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  336, 7235,   91351, 13, }, // Atlantic/South_Georgia
    {  336, 6784,   91364, 13, }, // Atlantic/St_Helena
    {  336, 6577,   91377,  5, }, // Europe/Athens
    {  336, 6285,   25465,  7, }, // Europe/Belgrade
    {  336, 4659,   19450,  6, }, // Europe/Berlin
    {  336, 6178,    4351,  7, }, // Europe/Brussels
    {  336, 3830,   91382,  8, }, // Europe/Bucharest
    {  336, 3708,   91390,  8, }, // Europe/Budapest
    {  336, 6238,   91398, 13, }, // Europe/Copenhagen
    {  336, 6256,   91411,  9, }, // Europe/Gibraltar
    {  336, 8783,   91420,  3, }, // Europe/Isle_of_Man
    {  336, 1097,   91423,  8, }, // Europe/Istanbul
    {  336, 2419,   91431, 10, }, // Europe/Kyiv
    {  336, 3078,    4400,  8, }, // Europe/Lisbon
    {  336, 2393,   91441,  8, }, // Europe/London
    {  336, 6314,   91449,  9, }, // Europe/Luxembourg
    {  336, 4794,   91458,  6, }, // Europe/Madrid
    {  336, 6531,   91464,  9, }, // Europe/Mariehamn
    {  336, 6359,   91473,  6, }, // Europe/Monaco
    {  336, 3443,   21290,  6, }, // Europe/Moscow
    {  336, 6407,   91479,  4, }, // Europe/Oslo
    {  336, 4186,   19571,  5, }, // Europe/Paris
    {  336, 6224,    6545,  4, }, // Europe/Prague
    {  336, 6273,   90581,  3, }, // Europe/Rome
    {  336, 6419,   91483, 10, }, // Europe/San_Marino
    {  336, 6194,   91493,  8, }, // Europe/Sarajevo
    {  336, 6548,   91501,  5, }, // Europe/Sofia
    {  336, 6472,   91506, 11, }, // Europe/Stockholm
    {  336, 6135,   91517,  6, }, // Europe/Tirane
    {  336, 6516,   91523,  9, }, // Europe/Vatican
    {  336, 6164,   91532,  3, }, // Europe/Vienna
    {  336, 8878,   91535,  7, }, // Europe/Vilnius
    {  336, 3055,   91542,  6, }, // Europe/Warsaw
    {  336, 5979,   91548,  6, }, // Indian/Christmas
    {  336, 5996,   91554, 10, }, // Indian/Cocos
    {  336, 7013,   91564, 11, }, // Indian/Maldives
    {  336, 3971,   91575,  8, }, // Indian/Mauritius
    {  336, 7174,     467,  7, }, // Indian/Reunion
    {  336, 3022,     474,  5, }, // Pacific/Chuuk
    {  336, 2201,   91583,  9, }, // Pacific/Easter
    {  336, 3804,   91592,  5, }, // Pacific/Fiji
    {  336, 5935,   91597,  4, }, // Pacific/Guam
    {  336, 2929,    4512,  8, }, // Pacific/Honolulu
    {  336, 2897,    4520,  9, }, // Pacific/Kanton
    {  336, 3953,   91601, 15, }, // Pacific/Marquesas
    {  336, 7047,   91616,  4, }, // Pacific/Nauru
    {  336, 7118,   91620,  4, }, // Pacific/Palau
    {  336, 2961,     479,  7, }, // Pacific/Pohnpei
    {  336, 7275,   91624,  6, }, // Pacific/Tahiti
    {  337,   14,       0,  6, }, // Africa/Asmara Ido/Latin/world
    {  337, 4337,       6,  8, }, // Africa/Sao_Tome
    {  337, 4154,      14,  8, }, // America/Asuncion
    {  337,  237,      22,  8, }, // America/Atikokan
    {  337, 7586,      30, 17, }, // America/Bahia_Banderas
    {  337, 3770,      47,  6, }, // America/Cancun
    {  337, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  337, 5640,      66,  7, }, // America/Curacao
    {  337,  481,      73, 13, }, // America/Indiana/Knox
    {  337, 7924,      86, 16, }, // America/Indiana/Marengo
    {  337, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  337, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  337, 8001,     139, 14, }, // America/Indiana/Vevay
    {  337, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  337, 8049,     171, 16, }, // America/Indiana/Winamac
    {  337, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  337, 5849,     207, 22, }, // America/Lower_Princes
    {  337, 8183,     229,  6, }, // America/Merida
    {  337, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  337, 8292,     255, 20, }, // America/North_Dakota/Center
    {  337, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  337,  393,     298,  4, }, // America/Nuuk
    {  337, 6862,     302, 16, }, // America/Scoresbysund
    {  337, 8471,     318, 14, }, // America/St_Barthelemy
    {  337, 2045,     332, 10, }, // America/St_Johns
    {  337, 5780,     342,  9, }, // America/St_Kitts
    {  337, 5797,     351,  9, }, // America/St_Lucia
    {  337,  855,     360, 10, }, // America/St_Thomas
    {  337, 5830,     370, 11, }, // America/St_Vincent
    {  337, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  337, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  337, 1152,     413,  9, }, // Asia/Kathmandu
    {  337,  992,     422,  7, }, // Asia/Kolkata
    {  337, 1178,     429,  5, }, // Asia/Macau
    {  337, 1202,     442,  6, }, // Asia/Yangon
    {  337, 1379,     448,  5, }, // Atlantic/Faroe
    {  337, 6784,     453, 10, }, // Atlantic/St_Helena
    {  337, 2419,     463,  4, }, // Europe/Kyiv
    {  337, 7174,     467,  7, }, // Indian/Reunion
    {  337, 3022,     474,  5, }, // Pacific/Chuuk
    {  337, 2961,     479,  7, }, // Pacific/Pohnpei
    {  338,   14,       0,  6, }, // Africa/Asmara Igbo/Latin/Nigeria
    {  338, 4337,       6,  8, }, // Africa/Sao_Tome
    {  338, 4154,      14,  8, }, // America/Asuncion
    {  338,  237,      22,  8, }, // America/Atikokan
    {  338, 7586,      30, 17, }, // America/Bahia_Banderas
    {  338, 3770,      47,  6, }, // America/Cancun
    {  338, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  338, 5640,      66,  7, }, // America/Curacao
    {  338,  481,      73, 13, }, // America/Indiana/Knox
    {  338, 7924,      86, 16, }, // America/Indiana/Marengo
    {  338, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  338, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  338, 8001,     139, 14, }, // America/Indiana/Vevay
    {  338, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  338, 8049,     171, 16, }, // America/Indiana/Winamac
    {  338, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  338, 5849,     207, 22, }, // America/Lower_Princes
    {  338, 8183,     229,  6, }, // America/Merida
    {  338, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  338, 8292,     255, 20, }, // America/North_Dakota/Center
    {  338, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  338,  393,     298,  4, }, // America/Nuuk
    {  338, 6862,     302, 16, }, // America/Scoresbysund
    {  338, 8471,     318, 14, }, // America/St_Barthelemy
    {  338, 2045,     332, 10, }, // America/St_Johns
    {  338, 5780,     342,  9, }, // America/St_Kitts
    {  338, 5797,     351,  9, }, // America/St_Lucia
    {  338,  855,     360, 10, }, // America/St_Thomas
    {  338, 5830,     370, 11, }, // America/St_Vincent
    {  338, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  338, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  338, 1152,     413,  9, }, // Asia/Kathmandu
    {  338,  992,     422,  7, }, // Asia/Kolkata
    {  338, 1178,     429,  5, }, // Asia/Macau
    {  338, 1202,     442,  6, }, // Asia/Yangon
    {  338, 1379,     448,  5, }, // Atlantic/Faroe
    {  338, 6784,     453, 10, }, // Atlantic/St_Helena
    {  338, 2419,     463,  4, }, // Europe/Kyiv
    {  338, 7174,     467,  7, }, // Indian/Reunion
    {  338, 3022,     474,  5, }, // Pacific/Chuuk
    {  338, 2961,     479,  7, }, // Pacific/Pohnpei
    {  339,   14,       0,  6, }, // Africa/Asmara Inari Sami/Latin/Finland
    {  339, 4337,       6,  8, }, // Africa/Sao_Tome
    {  339, 4154,      14,  8, }, // America/Asuncion
    {  339,  237,      22,  8, }, // America/Atikokan
    {  339, 7586,      30, 17, }, // America/Bahia_Banderas
    {  339, 3770,      47,  6, }, // America/Cancun
    {  339, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  339, 5640,      66,  7, }, // America/Curacao
    {  339,  481,      73, 13, }, // America/Indiana/Knox
    {  339, 7924,      86, 16, }, // America/Indiana/Marengo
    {  339, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  339, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  339, 8001,     139, 14, }, // America/Indiana/Vevay
    {  339, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  339, 8049,     171, 16, }, // America/Indiana/Winamac
    {  339, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  339, 5849,     207, 22, }, // America/Lower_Princes
    {  339, 8183,     229,  6, }, // America/Merida
    {  339, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  339, 8292,     255, 20, }, // America/North_Dakota/Center
    {  339, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  339,  393,     298,  4, }, // America/Nuuk
    {  339, 6862,     302, 16, }, // America/Scoresbysund
    {  339, 8471,     318, 14, }, // America/St_Barthelemy
    {  339, 2045,     332, 10, }, // America/St_Johns
    {  339, 5780,     342,  9, }, // America/St_Kitts
    {  339, 5797,     351,  9, }, // America/St_Lucia
    {  339,  855,     360, 10, }, // America/St_Thomas
    {  339, 5830,     370, 11, }, // America/St_Vincent
    {  339, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  339, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  339, 1152,     413,  9, }, // Asia/Kathmandu
    {  339,  992,     422,  7, }, // Asia/Kolkata
    {  339, 1178,     429,  5, }, // Asia/Macau
    {  339, 1202,     442,  6, }, // Asia/Yangon
    {  339, 1379,     448,  5, }, // Atlantic/Faroe
    {  339, 6784,     453, 10, }, // Atlantic/St_Helena
    {  339, 2419,     463,  4, }, // Europe/Kyiv
    {  339, 7174,     467,  7, }, // Indian/Reunion
    {  339, 3022,     474,  5, }, // Pacific/Chuuk
    {  339, 2961,     479,  7, }, // Pacific/Pohnpei
    {  340, 7383,   91630,  5, }, // Africa/Algiers Indonesian/Latin/Indonesia
    {  340,   14,       0,  6, }, // Africa/Asmara
    {  340, 2242,   24913,  5, }, // Africa/Cairo
    {  340, 4337,       6,  8, }, // Africa/Sao_Tome
    {  340, 5524,   18863,  7, }, // America/Anguilla
    {  340, 4154,      14,  8, }, // America/Asuncion
    {  340,  237,      22,  8, }, // America/Atikokan
    {  340, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  340, 3770,    3861,  6, }, // America/Cancun
    {  340, 7737,   89795, 13, }, // America/Ciudad_Juarez
    {  340, 5324,   20026, 10, }, // America/Costa_Rica
    {  340, 5640,      66,  7, }, // America/Curacao
    {  340, 5656,   20071,  8, }, // America/Dominica
    {  340,  481,      73, 13, }, // America/Indiana/Knox
    {  340, 7924,      86, 16, }, // America/Indiana/Marengo
    {  340, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  340, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  340, 8001,     139, 14, }, // America/Indiana/Vevay
    {  340, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  340, 8049,     171, 16, }, // America/Indiana/Winamac
    {  340, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  340, 5849,     207, 22, }, // America/Lower_Princes
    {  340, 5722,    5400,  8, }, // America/Martinique
    {  340, 8183,    3899,  6, }, // America/Merida
    {  340, 8264,   91635, 20, }, // America/North_Dakota/Beulah
    {  340, 8292,   91655, 20, }, // America/North_Dakota/Center
    {  340, 8320,   91675, 23, }, // America/North_Dakota/New_Salem
    {  340,  393,     298,  4, }, // America/Nuuk
    {  340, 6862,     302, 16, }, // America/Scoresbysund
    {  340, 8471,     318, 14, }, // America/St_Barthelemy
    {  340, 2045,     332, 10, }, // America/St_Johns
    {  340, 5780,     342,  9, }, // America/St_Kitts
    {  340, 5797,     351,  9, }, // America/St_Lucia
    {  340,  855,     360, 10, }, // America/St_Thomas
    {  340, 5830,     370, 11, }, // America/St_Vincent
    {  340, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  340, 5440,    5906,  5, }, // Asia/Aqtau
    {  340, 5270,    5911,  6, }, // Asia/Aqtobe
    {  340, 4413,    5949,  7, }, // Asia/Colombo
    {  340, 4442,    4102,  8, }, // Asia/Damascus
    {  340, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  340, 1257,   72485,  9, }, // Asia/Jerusalem
    {  340, 1152,     413,  9, }, // Asia/Kathmandu
    {  340,  992,     422,  7, }, // Asia/Kolkata
    {  340, 1178,   32008,  5, }, // Asia/Macau
    {  340, 6883,    4227,  6, }, // Asia/Muscat
    {  340, 2446,   25331,  7, }, // Asia/Nicosia
    {  340, 8671,     434,  8, }, // Asia/Qostanay
    {  340, 3137,   91698,  9, }, // Asia/Singapore
    {  340, 2638,    6175,  7, }, // Asia/Tehran
    {  340, 1202,   56146,  7, }, // Asia/Yangon
    {  340, 3629,   91707, 13, }, // Atlantic/Cape_Verde
    {  340, 1379,     448,  5, }, // Atlantic/Faroe
    {  340, 7235,   91720, 15, }, // Atlantic/South_Georgia
    {  340, 6784,     453, 10, }, // Atlantic/St_Helena
    {  340, 6577,   91735,  6, }, // Europe/Athens
    {  340, 6285,    6398,  7, }, // Europe/Belgrade
    {  340, 2475,   91741,  8, }, // Europe/Chisinau
    {  340, 6238,    4376, 10, }, // Europe/Copenhagen
    {  340, 8783,   91749,  9, }, // Europe/Isle_of_Man
    {  340, 2419,    6486,  4, }, // Europe/Kyiv
    {  340, 6314,    6510, 10, }, // Europe/Luxembourg
    {  340, 6359,    6525,  6, }, // Europe/Monaco
    {  340, 3443,   91758,  6, }, // Europe/Moscow
    {  340, 6224,   34765,  5, }, // Europe/Prague
    {  340, 6273,   19581,  4, }, // Europe/Rome
    {  340, 6516,    6614,  7, }, // Europe/Vatican
    {  340, 6164,     171,  4, }, // Europe/Vienna
    {  340, 3055,   91764,  7, }, // Europe/Warsaw
    {  340, 4931,   21395,  6, }, // Indian/Comoro
    {  340, 7013,   91771,  8, }, // Indian/Maldives
    {  340, 7174,     467,  7, }, // Indian/Reunion
    {  340, 3022,     474,  5, }, // Pacific/Chuuk
    {  340, 3724,    6711, 11, }, // Pacific/Guadalcanal
    {  340, 2929,    4512,  8, }, // Pacific/Honolulu
    {  340, 2897,    4520,  9, }, // Pacific/Kanton
    {  340, 2961,     479,  7, }, // Pacific/Pohnpei
    {  341,   14,       0,  6, }, // Africa/Asmara Interlingua/Latin/world
    {  341, 4945,   24925,  7, }, // Africa/Djibouti
    {  341, 4337,       6,  8, }, // Africa/Sao_Tome
    {  341, 4154,      14,  8, }, // America/Asuncion
    {  341,  237,      22,  8, }, // America/Atikokan
    {  341, 7586,   63067, 17, }, // America/Bahia_Banderas
    {  341, 3770,    3861,  6, }, // America/Cancun
    {  341, 5398,   42677,  6, }, // America/Cayman
    {  341, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  341, 5640,      66,  7, }, // America/Curacao
    {  341,  481,      73, 13, }, // America/Indiana/Knox
    {  341, 7924,      86, 16, }, // America/Indiana/Marengo
    {  341, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  341, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  341, 8001,     139, 14, }, // America/Indiana/Vevay
    {  341, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  341, 8049,     171, 16, }, // America/Indiana/Winamac
    {  341, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  341, 5849,     207, 22, }, // America/Lower_Princes
    {  341, 5722,   18945,  9, }, // America/Martinique
    {  341, 8183,    3899,  6, }, // America/Merida
    {  341, 2811,   91779, 16, }, // America/Mexico_City
    {  341, 3294,   91795,  9, }, // America/New_York
    {  341, 8264,   42707, 23, }, // America/North_Dakota/Beulah
    {  341, 8292,   42730, 23, }, // America/North_Dakota/Center
    {  341, 8320,   42753, 26, }, // America/North_Dakota/New_Salem
    {  341,  393,     298,  4, }, // America/Nuuk
    {  341, 5760,   63375, 10, }, // America/Puerto_Rico
    {  341, 6862,     302, 16, }, // America/Scoresbysund
    {  341, 8471,   91804, 18, }, // America/St_Barthelemy
    {  341, 2045,   91822, 28, }, // America/St_Johns
    {  341, 5780,   91850, 19, }, // America/St_Kitts
    {  341, 5797,   91869, 12, }, // America/St_Lucia
    {  341,  855,   91881, 13, }, // America/St_Thomas
    {  341, 5830,   91894, 15, }, // America/St_Vincent
    {  341, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  341, 5451,    4069,  7, }, // Asia/Bahrain
    {  341, 4442,   72897,  7, }, // Asia/Damascus
    {  341, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  341, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  341, 1152,     413,  9, }, // Asia/Kathmandu
    {  341, 8624,   56553,  8, }, // Asia/Khandyga
    {  341,  992,   34533,  8, }, // Asia/Kolkata
    {  341, 1178,     429,  5, }, // Asia/Macau
    {  341, 4310,   56617,  8, }, // Asia/Sakhalin
    {  341, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  341, 1202,     442,  6, }, // Asia/Yangon
    {  341, 3785,    6219, 12, }, // Asia/Yekaterinburg
    {  341, 6591,   72956,  8, }, // Atlantic/Canary
    {  341, 3629,   91909, 10, }, // Atlantic/Cape_Verde
    {  341, 1379,   19383,  5, }, // Atlantic/Faroe
    {  341, 6784,     453, 10, }, // Atlantic/St_Helena
    {  341, 6577,   91919,  7, }, // Europe/Athens
    {  341, 6285,   57443,  8, }, // Europe/Belgrade
    {  341, 6178,   56194,  9, }, // Europe/Brussels
    {  341, 3830,   19464,  8, }, // Europe/Bucharest
    {  341, 8751,   25487,  8, }, // Europe/Busingen
    {  341, 8783,   91926, 13, }, // Europe/Isle_of_Man
    {  341, 2419,    6486,  4, }, // Europe/Kyiv
    {  341, 3078,   91939,  7, }, // Europe/Lisbon
    {  341, 6314,    4414,  9, }, // Europe/Luxembourg
    {  341, 3443,   73049,  7, }, // Europe/Moscow
    {  341, 6224,   19576,  5, }, // Europe/Prague
    {  341, 6273,   19581,  4, }, // Europe/Rome
    {  341, 6135,   19608,  6, }, // Europe/Tirane
    {  341, 6516,   73083,  8, }, // Europe/Vatican
    {  341, 3055,   19639,  8, }, // Europe/Warsaw
    {  341, 5979,   91946,  5, }, // Indian/Christmas
    {  341, 7013,   73105,  8, }, // Indian/Maldives
    {  341, 3971,   91951,  8, }, // Indian/Mauritius
    {  341, 5000,   91959,  7, }, // Indian/Mayotte
    {  341, 7174,     467,  7, }, // Indian/Reunion
    {  341, 3022,     474,  5, }, // Pacific/Chuuk
    {  341, 2201,   91966,  6, }, // Pacific/Easter
    {  341, 6641,   19672,  9, }, // Pacific/Galapagos
    {  341, 2961,     479,  7, }, // Pacific/Pohnpei
    {  342, 4961,   63018, 11, }, // Africa/Addis_Ababa Interlingue/Latin/Estonia
    {  342, 7383,   42592,  5, }, // Africa/Algiers
    {  342,   14,       0,  6, }, // Africa/Asmara
    {  342, 5032,   91972, 13, }, // Africa/Dar_es_Salaam
    {  342, 4945,   24925,  7, }, // Africa/Djibouti
    {  342, 5068,   91985,  8, }, // Africa/El_Aaiun
    {  342, 4426,   24937,  7, }, // Africa/Khartoum
    {  342, 6690,   24967,  8, }, // Africa/Ouagadougou
    {  342, 4337,   91993,  8, }, // Africa/Sao_Tome
    {  342,  166,   92001, 12, }, // America/Argentina/Buenos_Aires
    {  342, 7425,   92013,  8, }, // America/Argentina/La_Rioja
    {  342, 7452,   92021, 12, }, // America/Argentina/Rio_Gallegos
    {  342, 7507,   92033,  9, }, // America/Argentina/San_Juan
    {  342, 5497,   92042,  9, }, // America/Argentina/San_Luis
    {  342, 4154,      14,  8, }, // America/Asuncion
    {  342,  237,      22,  8, }, // America/Atikokan
    {  342, 7586,   92051, 15, }, // America/Bahia_Banderas
    {  342, 7644,    4909,  9, }, // America/Boa_Vista
    {  342, 7676,   92066, 13, }, // America/Cambridge_Bay
    {  342, 7698,   92079, 12, }, // America/Campo_Grande
    {  342, 3770,      47,  6, }, // America/Cancun
    {  342, 7737,   92091, 13, }, // America/Ciudad_Juarez
    {  342, 5324,   92104, 10, }, // America/Costa_Rica
    {  342, 5640,      66,  7, }, // America/Curacao
    {  342, 7811,   92114, 12, }, // America/Dawson_Creek
    {  342, 5343,   92126, 11, }, // America/El_Salvador
    {  342, 7849,    5098, 11, }, // America/Fort_Nelson
    {  342, 7887,   92137,  9, }, // America/Glace_Bay
    {  342, 6844,   92146,  9, }, // America/Goose_Bay
    {  342, 4514,   92155, 10, }, // America/Grand_Turk
    {  342, 5703,   92165,  9, }, // America/Guadeloupe
    {  342,  481,      73, 13, }, // America/Indiana/Knox
    {  342, 7924,      86, 16, }, // America/Indiana/Marengo
    {  342, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  342, 7975,   92174, 18, }, // America/Indiana/Tell_City
    {  342, 8001,     139, 14, }, // America/Indiana/Vevay
    {  342, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  342, 8049,     171, 16, }, // America/Indiana/Winamac
    {  342, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  342, 4278,   60394,  6, }, // America/La_Paz
    {  342, 3390,   92192, 11, }, // America/Los_Angeles
    {  342, 5849,   92203, 22, }, // America/Lower_Princes
    {  342, 5722,   18945,  9, }, // America/Martinique
    {  342, 8183,     229,  6, }, // America/Merida
    {  342, 2811,   92225, 12, }, // America/Mexico_City
    {  342, 8264,   92237, 19, }, // America/North_Dakota/Beulah
    {  342, 8292,   92256, 19, }, // America/North_Dakota/Center
    {  342, 8320,   92275, 22, }, // America/North_Dakota/New_Salem
    {  342,  393,     298,  4, }, // America/Nuuk
    {  342, 5871,   25124, 13, }, // America/Port_of_Spain
    {  342, 8367,    5586, 11, }, // America/Porto_Velho
    {  342, 5760,   92297, 10, }, // America/Puerto_Rico
    {  342, 3932,    5607, 12, }, // America/Punta_Arenas
    {  342, 8387,   92307, 12, }, // America/Rankin_Inlet
    {  342,  695,   92319, 10, }, // America/Rio_Branco
    {  342, 6044,   92329, 11, }, // America/Santo_Domingo
    {  342, 1862,   92340,  9, }, // America/Sao_Paulo
    {  342, 6862,     302, 16, }, // America/Scoresbysund
    {  342, 8471,   92349, 15, }, // America/St_Barthelemy
    {  342, 2045,   92364, 10, }, // America/St_Johns
    {  342, 5780,   92374,  9, }, // America/St_Kitts
    {  342, 5797,   92383,  9, }, // America/St_Lucia
    {  342,  855,   92392, 10, }, // America/St_Thomas
    {  342, 5830,   92402, 11, }, // America/St_Vincent
    {  342, 8493,   92413, 13, }, // America/Swift_Current
    {  342, 6066,   55662, 16, }, // Antarctica/DumontDUrville
    {  342, 7258,   63470,  5, }, // Antarctica/Syowa
    {  342, 5297,    5890,  6, }, // Asia/Almaty
    {  342, 5440,    5906,  5, }, // Asia/Aqtau
    {  342, 5270,    5911,  6, }, // Asia/Aqtobe
    {  342, 8587,    5917,  6, }, // Asia/Atyrau
    {  342, 4442,   72897,  7, }, // Asia/Damascus
    {  342, 1226,   76638, 11, }, // Asia/Ho_Chi_Minh
    {  342, 2576,    5978,  9, }, // Asia/Hong_Kong
    {  342, 1152,     413,  9, }, // Asia/Kathmandu
    {  342,  992,     422,  7, }, // Asia/Kolkata
    {  342, 4761,    6040, 12, }, // Asia/Kuala_Lumpur
    {  342, 1178,     429,  5, }, // Asia/Macau
    {  342, 6924,   92426, 10, }, // Asia/Phnom_Penh
    {  342, 8671,     434,  8, }, // Asia/Qostanay
    {  342, 4171,    6111,  9, }, // Asia/Qyzylorda
    {  342, 3137,   19291,  8, }, // Asia/Singapore
    {  342, 2638,    6175,  7, }, // Asia/Tehran
    {  342, 2687,    4257,  5, }, // Asia/Tokyo
    {  342, 1202,     442,  6, }, // Asia/Yangon
    {  342, 6591,   72956,  7, }, // Atlantic/Canary
    {  342, 3629,   92436,  9, }, // Atlantic/Cape_Verde
    {  342, 1379,   19383,  5, }, // Atlantic/Faroe
    {  342, 7235,   92445, 11, }, // Atlantic/South_Georgia
    {  342, 6784,   92456, 11, }, // Atlantic/St_Helena
    {  342, 1783,   92467, 11, }, // Australia/Broken_Hill
    {  342, 1531,   92478, 15, }, // Australia/Lord_Howe
    {  342, 1746,   21145,  4, }, // Australia/Perth
    {  342, 6577,   19437,  4, }, // Europe/Athens
    {  342, 6285,   25465,  7, }, // Europe/Belgrade
    {  342, 6178,   56194,  9, }, // Europe/Brussels
    {  342, 3830,   92493,  9, }, // Europe/Bucharest
    {  342, 6238,   57194,  9, }, // Europe/Copenhagen
    {  342, 8783,   92502, 12, }, // Europe/Isle_of_Man
    {  342, 2419,    6486,  4, }, // Europe/Kyiv
    {  342, 3078,   19528,  6, }, // Europe/Lisbon
    {  342, 6314,    4414,  9, }, // Europe/Luxembourg
    {  342, 6359,   73043,  6, }, // Europe/Monaco
    {  342, 3443,   92514,  6, }, // Europe/Moscow
    {  342, 6224,   34765,  5, }, // Europe/Prague
    {  342, 6273,   19581,  4, }, // Europe/Rome
    {  342, 6419,    6558, 10, }, // Europe/San_Marino
    {  342, 6135,   19608,  6, }, // Europe/Tirane
    {  342, 3055,   57212,  8, }, // Europe/Warsaw
    {  342, 6489,    4478,  6, }, // Europe/Zurich
    {  342, 3971,   73113,  8, }, // Indian/Mauritius
    {  342, 7174,     467,  7, }, // Indian/Reunion
    {  342, 3022,     474,  5, }, // Pacific/Chuuk
    {  342, 2201,   92520,  5, }, // Pacific/Easter
    {  342, 3804,    4507,  5, }, // Pacific/Fiji
    {  342, 2991,    6764,  9, }, // Pacific/Pago_Pago
    {  342, 2961,     479,  7, }, // Pacific/Pohnpei
    {  342, 4709,   92525, 12, }, // Pacific/Port_Moresby
    {  343,   14,       0,  6, }, // Africa/Asmara Inuktitut/Canadian Aboriginal/Canada
    {  343, 4337,       6,  8, }, // Africa/Sao_Tome
    {  343, 4154,      14,  8, }, // America/Asuncion
    {  343,  237,      22,  8, }, // America/Atikokan
    {  343, 7586,      30, 17, }, // America/Bahia_Banderas
    {  343, 3770,      47,  6, }, // America/Cancun
    {  343, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  343, 5640,      66,  7, }, // America/Curacao
    {  343,  481,      73, 13, }, // America/Indiana/Knox
    {  343, 7924,      86, 16, }, // America/Indiana/Marengo
    {  343, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  343, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  343, 8001,     139, 14, }, // America/Indiana/Vevay
    {  343, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  343, 8049,     171, 16, }, // America/Indiana/Winamac
    {  343, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  343, 5849,     207, 22, }, // America/Lower_Princes
    {  343, 8183,     229,  6, }, // America/Merida
    {  343, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  343, 8292,     255, 20, }, // America/North_Dakota/Center
    {  343, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  343,  393,     298,  4, }, // America/Nuuk
    {  343, 6862,     302, 16, }, // America/Scoresbysund
    {  343, 8471,     318, 14, }, // America/St_Barthelemy
    {  343, 2045,     332, 10, }, // America/St_Johns
    {  343, 5780,     342,  9, }, // America/St_Kitts
    {  343, 5797,     351,  9, }, // America/St_Lucia
    {  343,  855,     360, 10, }, // America/St_Thomas
    {  343, 5830,     370, 11, }, // America/St_Vincent
    {  343, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  343, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  343, 1152,     413,  9, }, // Asia/Kathmandu
    {  343,  992,     422,  7, }, // Asia/Kolkata
    {  343, 1178,     429,  5, }, // Asia/Macau
    {  343, 1202,     442,  6, }, // Asia/Yangon
    {  343, 1379,     448,  5, }, // Atlantic/Faroe
    {  343, 6784,     453, 10, }, // Atlantic/St_Helena
    {  343, 2419,     463,  4, }, // Europe/Kyiv
    {  343, 7174,     467,  7, }, // Indian/Reunion
    {  343, 3022,     474,  5, }, // Pacific/Chuuk
    {  343, 2961,     479,  7, }, // Pacific/Pohnpei
    {  345, 7383,   92537, 20, }, // Africa/Algiers Irish/Latin/Ireland
    {  345,   14,       0,  6, }, // Africa/Asmara
    {  345, 2242,   92557,  6, }, // Africa/Cairo
    {  345, 6736,   92563,  8, }, // Africa/Conakry
    {  345, 6803,   92571,  5, }, // Africa/Dakar
    {  345, 5032,   92576,  9, }, // Africa/Dar_es_Salaam
    {  345, 5068,   92585,  6, }, // Africa/El_Aaiun
    {  345, 4426,   92591,  6, }, // Africa/Khartoum
    {  345, 6832,   18802,  4, }, // Africa/Lome
    {  345, 4889,   92597,  6, }, // Africa/Maputo
    {  345, 5015,   92603, 10, }, // Africa/Mogadishu
    {  345, 6766,   92613,  7, }, // Africa/Nouakchott
    {  345, 4337,       6,  8, }, // Africa/Sao_Tome
    {  345, 2732,   92620,  7, }, // Africa/Tripoli
    {  345, 6503,   92627,  6, }, // Africa/Tunis
    {  345, 5524,   92633,  7, }, // America/Anguilla
    {  345,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  345, 5557,   91006,  5, }, // America/Aruba
    {  345, 4154,      14,  8, }, // America/Asuncion
    {  345,  237,      22,  8, }, // America/Atikokan
    {  345, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  345, 5571,   92640,  8, }, // America/Barbados
    {  345, 7609,   34376,  5, }, // America/Belem
    {  345, 5309,   92648, 10, }, // America/Belize
    {  345, 4263,   92658,  6, }, // America/Bogota
    {  345, 3770,      47,  6, }, // America/Cancun
    {  345, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  345, 5324,   92664, 10, }, // America/Costa_Rica
    {  345, 5640,   92674,  6, }, // America/Curacao
    {  345, 5656,   71233,  9, }, // America/Dominica
    {  345, 5343,   92680, 13, }, // America/El_Salvador
    {  345, 5687,   92693,  8, }, // America/Grenada
    {  345, 5703,   92701,  9, }, // America/Guadeloupe
    {  345, 3662,   71284,  9, }, // America/Guatemala
    {  345, 6895,   92710,  9, }, // America/Guyana
    {  345,  481,      73, 13, }, // America/Indiana/Knox
    {  345, 7924,      86, 16, }, // America/Indiana/Marengo
    {  345, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  345, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  345, 8001,     139, 14, }, // America/Indiana/Vevay
    {  345, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  345, 8049,     171, 16, }, // America/Indiana/Winamac
    {  345, 2665,   92719,  7, }, // America/Jamaica
    {  345, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  345, 7132,   92726,  5, }, // America/Lima
    {  345, 5849,     207, 22, }, // America/Lower_Princes
    {  345, 8183,    3899,  6, }, // America/Merida
    {  345, 2811,   92731, 18, }, // America/Mexico_City
    {  345, 5741,   71343,  9, }, // America/Montserrat
    {  345, 3294,   92749, 11, }, // America/New_York
    {  345, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  345, 8292,     255, 20, }, // America/North_Dakota/Center
    {  345, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  345,  393,     298,  4, }, // America/Nuuk
    {  345, 5760,   92760, 10, }, // America/Puerto_Rico
    {  345, 1862,   25145,  9, }, // America/Sao_Paulo
    {  345, 6862,     302, 16, }, // America/Scoresbysund
    {  345, 8471,   34416, 16, }, // America/St_Barthelemy
    {  345, 2045,     332, 10, }, // America/St_Johns
    {  345, 5780,   92770, 13, }, // America/St_Kitts
    {  345, 5797,   34444, 11, }, // America/St_Lucia
    {  345,  855,     360, 10, }, // America/St_Thomas
    {  345, 5830,   92783, 12, }, // America/St_Vincent
    {  345, 5673,   92795, 10, }, // America/Thule
    {  345, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  345, 8531,   92805, 10, }, // Antarctica/Macquarie
    {  345, 5487,   92815,  5, }, // Asia/Aden
    {  345, 3509,    4063,  6, }, // Asia/Baghdad
    {  345, 5451,   92820,  7, }, // Asia/Bahrain
    {  345, 3555,   92827,  4, }, // Asia/Baku
    {  345, 3988,   92831,  7, }, // Asia/Beirut
    {  345, 5906,   92838,  6, }, // Asia/Brunei
    {  345, 4442,   92844, 10, }, // Asia/Damascus
    {  345, 4697,   92854,  7, }, // Asia/Hebron
    {  345, 1226,   71502, 19, }, // Asia/Ho_Chi_Minh
    {  345, 2576,   92861,  9, }, // Asia/Hong_Kong
    {  345, 6969,   92870,  7, }, // Asia/Jakarta
    {  345, 1257,   92877, 11, }, // Asia/Jerusalem
    {  345, 3462,   92888,  5, }, // Asia/Kabul
    {  345, 1152,     413,  9, }, // Asia/Kathmandu
    {  345,  992,   92893,  7, }, // Asia/Kolkata
    {  345, 5464,   92900,  5, }, // Asia/Kuwait
    {  345, 1178,     429,  5, }, // Asia/Macau
    {  345, 1316,   92905,  7, }, // Asia/Makassar
    {  345, 7145,   92912,  7, }, // Asia/Manila
    {  345, 2446,   92919, 10, }, // Asia/Nicosia
    {  345, 5476,   71561,  5, }, // Asia/Qatar
    {  345, 8671,     434,  8, }, // Asia/Qostanay
    {  345, 3116,   92929,  3, }, // Asia/Seoul
    {  345, 1020,   92932,  9, }, // Asia/Shanghai
    {  345, 3137,   92941,  9, }, // Asia/Singapore
    {  345, 2687,   92950,  6, }, // Asia/Tokyo
    {  345, 4730,   92956,  7, }, // Asia/Yakutsk
    {  345, 1202,   19328,  6, }, // Asia/Yangon
    {  345, 3649,   92963,  8, }, // Asia/Yerevan
    {  345, 3565,   92971,  9, }, // Atlantic/Azores
    {  345, 5588,   92980,  9, }, // Atlantic/Bermuda
    {  345, 6591,   92989, 22, }, // Atlantic/Canary
    {  345, 3629,   93011, 10, }, // Atlantic/Cape_Verde
    {  345, 1379,   93021, 13, }, // Atlantic/Faroe
    {  345, 8699,   93034,  8, }, // Atlantic/Madeira
    {  345, 2614,   93042,  8, }, // Atlantic/Reykjavik
    {  345, 7235,   93050, 18, }, // Atlantic/South_Georgia
    {  345, 6784,   93068, 10, }, // Atlantic/St_Helena
    {  345, 6390,   93078,  9, }, // Europe/Amsterdam
    {  345, 6149,   93087,  6, }, // Europe/Andorra
    {  345, 3522,   93093, 12, }, // Europe/Astrakhan
    {  345, 6577,   93105,  9, }, // Europe/Athens
    {  345, 6285,   93114,  8, }, // Europe/Belgrade
    {  345, 4659,   93122,  7, }, // Europe/Berlin
    {  345, 6437,   93129, 14, }, // Europe/Bratislava
    {  345, 6178,   93143, 12, }, // Europe/Brussels
    {  345, 3830,   93155,  9, }, // Europe/Bucharest
    {  345, 3708,   93164, 10, }, // Europe/Budapest
    {  345, 2475,   93174,  7, }, // Europe/Chisinau
    {  345, 6238,   93181, 10, }, // Europe/Copenhagen
    {  345, 2260,   93191, 17, }, // Europe/Dublin
    {  345, 6256,   93208, 10, }, // Europe/Gibraltar
    {  345, 8767,   93218,  7, }, // Europe/Guernsey
    {  345, 6561,   93225,  9, }, // Europe/Helsinki
    {  345, 8783,   93234, 14, }, // Europe/Isle_of_Man
    {  345, 1097,   93248,  9, }, // Europe/Istanbul
    {  345, 8802,   93257,  6, }, // Europe/Jersey
    {  345, 2419,   93263,  3, }, // Europe/Kyiv
    {  345, 3078,   93266,  8, }, // Europe/Lisbon
    {  345, 6455,   93274, 10, }, // Europe/Ljubljana
    {  345, 2393,   93284,  7, }, // Europe/London
    {  345, 6314,   93291, 10, }, // Europe/Luxembourg
    {  345, 4794,   93301,  7, }, // Europe/Madrid
    {  345, 6346,   90552,  5, }, // Europe/Malta
    {  345, 3595,   93308,  6, }, // Europe/Minsk
    {  345, 6359,   93314,  6, }, // Europe/Monaco
    {  345, 3443,   93320,  5, }, // Europe/Moscow
    {  345, 6407,   91479,  4, }, // Europe/Oslo
    {  345, 4186,   93325,  5, }, // Europe/Paris
    {  345, 6373,   93330, 11, }, // Europe/Podgorica
    {  345, 6224,   90576,  4, }, // Europe/Prague
    {  345, 8816,   93341,  4, }, // Europe/Riga
    {  345, 6273,   93345,  8, }, // Europe/Rome
    {  345, 6419,   93353, 11, }, // Europe/San_Marino
    {  345, 6194,   93364,  8, }, // Europe/Sarajevo
    {  345, 6332,   93372,  6, }, // Europe/Skopje
    {  345, 6548,   93378,  6, }, // Europe/Sofia
    {  345, 6472,   93384,  7, }, // Europe/Stockholm
    {  345, 8846,   93391,  8, }, // Europe/Tallinn
    {  345, 6135,   93399,  7, }, // Europe/Tirane
    {  345, 6301,   93406,  5, }, // Europe/Vaduz
    {  345, 6516,   93411, 11, }, // Europe/Vatican
    {  345, 6164,   91532,  3, }, // Europe/Vienna
    {  345, 8878,   93422,  7, }, // Europe/Vilnius
    {  345, 3055,   93429,  5, }, // Europe/Warsaw
    {  345, 6210,   93434,  6, }, // Europe/Zagreb
    {  345, 6489,    4478,  6, }, // Europe/Zurich
    {  345, 4980,   93440, 14, }, // Indian/Antananarivo
    {  345, 5979,   93454, 16, }, // Indian/Christmas
    {  345, 5996,   93470, 12, }, // Indian/Cocos
    {  345, 4931,   93482, 15, }, // Indian/Comoro
    {  345, 7013,   93497, 17, }, // Indian/Maldives
    {  345, 3971,   93514, 14, }, // Indian/Mauritius
    {  345, 7174,   63930, 10, }, // Indian/Reunion
    {  345, 3022,     474,  5, }, // Pacific/Chuuk
    {  345, 3804,   93528,  5, }, // Pacific/Fiji
    {  345, 2929,    4512,  8, }, // Pacific/Honolulu
    {  345, 2897,    4520,  9, }, // Pacific/Kanton
    {  345, 3953,   93533, 23, }, // Pacific/Marquesas
    {  345, 8909,   93556, 14, }, // Pacific/Midway
    {  345, 7047,   91616,  4, }, // Pacific/Nauru
    {  345, 2961,     479,  7, }, // Pacific/Pohnpei
    {  345, 7275,   93570,  7, }, // Pacific/Tahiti
    {  345, 7368,   93577,  6, }, // Pacific/Wallis
    {  347, 4961,    3793, 11, }, // Africa/Addis_Ababa Italian/Latin/Italy
    {  347, 7383,   93583,  6, }, // Africa/Algiers
    {  347,   14,       0,  6, }, // Africa/Asmara
    {  347, 2242,   93589,  8, }, // Africa/Cairo
    {  347, 4945,   93597,  6, }, // Africa/Djibouti
    {  347, 5068,   93603,  7, }, // Africa/El_Aaiun
    {  347, 4401,   93610,  5, }, // Africa/Juba
    {  347, 4426,   24937,  7, }, // Africa/Khartoum
    {  347, 6832,   18802,  4, }, // Africa/Lome
    {  347, 5015,   42614, 10, }, // Africa/Mogadishu
    {  347, 5173,   42632,  9, }, // Africa/Ndjamena
    {  347, 4337,       6,  8, }, // Africa/Sao_Tome
    {  347, 6503,   93615,  6, }, // Africa/Tunis
    {  347, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  347, 4154,      14,  8, }, // America/Asuncion
    {  347,  237,      22,  8, }, // America/Atikokan
    {  347, 7586,      30, 17, }, // America/Bahia_Banderas
    {  347, 4263,   18898,  6, }, // America/Bogota
    {  347, 3770,      47,  6, }, // America/Cancun
    {  347, 4247,   93621,  7, }, // America/Cayenne
    {  347, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  347, 5640,      66,  7, }, // America/Curacao
    {  347, 5703,   93628,  9, }, // America/Guadeloupe
    {  347, 2221,   93637,  7, }, // America/Havana
    {  347,  481,      73, 13, }, // America/Indiana/Knox
    {  347, 7924,      86, 16, }, // America/Indiana/Marengo
    {  347, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  347, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  347, 8001,     139, 14, }, // America/Indiana/Vevay
    {  347, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  347, 8049,     171, 16, }, // America/Indiana/Winamac
    {  347, 2665,   93644,  8, }, // America/Jamaica
    {  347, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  347, 5849,     207, 22, }, // America/Lower_Princes
    {  347, 5722,   18945,  9, }, // America/Martinique
    {  347, 8183,     229,  6, }, // America/Merida
    {  347, 2811,   93652, 17, }, // America/Mexico_City
    {  347, 8264,   93669, 23, }, // America/North_Dakota/Beulah
    {  347, 8292,   93692, 23, }, // America/North_Dakota/Center
    {  347, 8320,   93715, 26, }, // America/North_Dakota/New_Salem
    {  347,  393,     298,  4, }, // America/Nuuk
    {  347, 5760,   93741,  9, }, // America/Puerto_Rico
    {  347, 8440,   25137,  8, }, // America/Santarem
    {  347, 1862,   93750,  9, }, // America/Sao_Paulo
    {  347, 6862,     302, 16, }, // America/Scoresbysund
    {  347, 8471,   25154, 16, }, // America/St_Barthelemy
    {  347, 2045,     332, 10, }, // America/St_Johns
    {  347, 5780,     342,  9, }, // America/St_Kitts
    {  347, 5797,   93759, 11, }, // America/St_Lucia
    {  347,  855,   42797, 12, }, // America/St_Thomas
    {  347, 5830,   25216, 13, }, // America/St_Vincent
    {  347, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  347, 5428,   93770,  7, }, // Asia/Anadyr
    {  347, 5270,   55725,  6, }, // Asia/Aqtobe
    {  347, 5451,    4069,  7, }, // Asia/Bahrain
    {  347, 4503,   56142,  4, }, // Asia/Chita
    {  347, 4442,   72897,  7, }, // Asia/Damascus
    {  347, 1060,   64123,  5, }, // Asia/Dhaka
    {  347, 8599,   93777,  9, }, // Asia/Famagusta
    {  347, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  347, 6969,   93786,  8, }, // Asia/Jakarta
    {  347, 1257,   93794, 11, }, // Asia/Jerusalem
    {  347, 1152,     413,  9, }, // Asia/Kathmandu
    {  347, 8624,   56553,  8, }, // Asia/Khandyga
    {  347,  992,   34533,  8, }, // Asia/Kolkata
    {  347, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  347, 1178,     429,  5, }, // Asia/Macau
    {  347, 6883,   19248,  7, }, // Asia/Muscat
    {  347, 8638,   61024, 11, }, // Asia/Novokuznetsk
    {  347, 3486,   19265,  5, }, // Asia/Riyadh
    {  347, 4310,   56617,  8, }, // Asia/Sakhalin
    {  347, 7208,   19277, 10, }, // Asia/Samarkand
    {  347, 3116,    6131,  4, }, // Asia/Seoul
    {  347, 2638,    6175,  7, }, // Asia/Tehran
    {  347, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  347, 8685,   42936,  9, }, // Asia/Ust-Nera
    {  347, 4730,    6206,  7, }, // Asia/Yakutsk
    {  347, 1202,   56146,  7, }, // Asia/Yangon
    {  347, 3785,    6219, 12, }, // Asia/Yekaterinburg
    {  347, 3565,   93805,  7, }, // Atlantic/Azores
    {  347, 6591,   19358,  7, }, // Atlantic/Canary
    {  347, 3629,   91909, 10, }, // Atlantic/Cape_Verde
    {  347, 1379,   93812, 13, }, // Atlantic/Faroe
    {  347, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  347, 7235,   93825, 15, }, // Atlantic/South_Georgia
    {  347, 6784,   93840, 10, }, // Atlantic/St_Helena
    {  347, 6577,   19437,  5, }, // Europe/Athens
    {  347, 6285,   57443,  8, }, // Europe/Belgrade
    {  347, 4659,   61411,  7, }, // Europe/Berlin
    {  347, 6178,   56194,  9, }, // Europe/Brussels
    {  347, 3830,   19464,  8, }, // Europe/Bucharest
    {  347, 8751,   25487,  8, }, // Europe/Busingen
    {  347, 6238,   93850, 10, }, // Europe/Copenhagen
    {  347, 2260,   61491,  7, }, // Europe/Dublin
    {  347, 6256,   93860, 10, }, // Europe/Gibraltar
    {  347, 8783,   93870, 12, }, // Europe/Isle_of_Man
    {  347, 2419,    6486,  4, }, // Europe/Kyiv
    {  347, 3078,   91939,  7, }, // Europe/Lisbon
    {  347, 6455,   93882,  7, }, // Europe/Ljubljana
    {  347, 2393,   93889,  6, }, // Europe/London
    {  347, 6314,   93895, 11, }, // Europe/Luxembourg
    {  347, 3443,   93906,  5, }, // Europe/Moscow
    {  347, 4186,   93911,  6, }, // Europe/Paris
    {  347, 6224,   19576,  5, }, // Europe/Prague
    {  347, 6273,   19581,  4, }, // Europe/Rome
    {  347, 8828,   93917, 11, }, // Europe/Simferopol
    {  347, 6472,   93928,  9, }, // Europe/Stockholm
    {  347, 6135,   19608,  6, }, // Europe/Tirane
    {  347, 6516,   93937, 18, }, // Europe/Vatican
    {  347, 3055,   93955,  8, }, // Europe/Warsaw
    {  347, 6210,   93963,  8, }, // Europe/Zagreb
    {  347, 6489,   93971,  6, }, // Europe/Zurich
    {  347, 4931,    4489,  6, }, // Indian/Comoro
    {  347, 7013,    6662,  7, }, // Indian/Maldives
    {  347, 7174,   93977, 11, }, // Indian/Reunion
    {  347, 3022,     474,  5, }, // Pacific/Chuuk
    {  347, 2201,   43146,  6, }, // Pacific/Easter
    {  347, 3804,   93988,  4, }, // Pacific/Fiji
    {  347, 2929,    4512,  8, }, // Pacific/Honolulu
    {  347, 2897,    4520,  9, }, // Pacific/Kanton
    {  347, 3953,   93992,  8, }, // Pacific/Marquesas
    {  347, 2961,     479,  7, }, // Pacific/Pohnpei
    {  351, 6751,   94000,  5, }, // Africa/Abidjan Japanese/Japanese/Japan
    {  351, 6723,   94005,  3, }, // Africa/Accra
    {  351, 4961,   94008,  6, }, // Africa/Addis_Ababa
    {  351, 7383,   94014,  4, }, // Africa/Algiers
    {  351,   14,   94018,  4, }, // Africa/Asmara
    {  351,   44,   94022,  3, }, // Africa/Bamako
    {  351, 5159,   94025,  3, }, // Africa/Bangui
    {  351, 6709,   94028,  6, }, // Africa/Banjul
    {  351, 7398,   94034,  3, }, // Africa/Bissau
    {  351, 4859,   94037,  6, }, // Africa/Blantyre
    {  351, 5189,   94043,  5, }, // Africa/Brazzaville
    {  351, 4824,   94048,  6, }, // Africa/Bujumbura
    {  351, 2242,   94054,  3, }, // Africa/Cairo
    {  351, 4019,   94057,  6, }, // Africa/Casablanca
    {  351, 7412,   94063,  3, }, // Africa/Ceuta
    {  351, 6736,   94066,  4, }, // Africa/Conakry
    {  351, 6803,   94070,  4, }, // Africa/Dakar
    {  351, 5032,   94074,  8, }, // Africa/Dar_es_Salaam
    {  351, 4945,   94082,  3, }, // Africa/Djibouti
    {  351, 5145,   94085,  4, }, // Africa/Douala
    {  351, 5068,   94089,  4, }, // Africa/El_Aaiun
    {  351, 6816,   94093,  6, }, // Africa/Freetown
    {  351, 4808,   94099,  5, }, // Africa/Gaborone
    {  351, 4917,   94104,  3, }, // Africa/Harare
    {  351, 4381,   94107,  7, }, // Africa/Johannesburg
    {  351, 4401,   94114,  3, }, // Africa/Juba
    {  351, 5053,   94117,  4, }, // Africa/Kampala
    {  351, 4426,   94121,  5, }, // Africa/Khartoum
    {  351, 4875,   94126,  3, }, // Africa/Kigali
    {  351, 5208,   94129,  5, }, // Africa/Kinshasa
    {  351, 4646,   94134,  3, }, // Africa/Lagos
    {  351, 5238,   94137,  7, }, // Africa/Libreville
    {  351, 6832,   94144,  2, }, // Africa/Lome
    {  351, 5113,   94146,  4, }, // Africa/Luanda
    {  351, 4841,   94150,  5, }, // Africa/Lubumbashi
    {  351, 4903,   94155,  3, }, // Africa/Lusaka
    {  351, 5224,   94158,  3, }, // Africa/Malabo
    {  351, 4889,   94161,  3, }, // Africa/Maputo
    {  351, 5099,   94164,  3, }, // Africa/Maseru
    {  351, 5084,   94167,  5, }, // Africa/Mbabane
    {  351, 5015,   94172,  6, }, // Africa/Mogadishu
    {  351, 6997,   94178,  5, }, // Africa/Monrovia
    {  351, 3755,   94183,  4, }, // Africa/Nairobi
    {  351, 5173,   94187,  5, }, // Africa/Ndjamena
    {  351, 5256,   94192,  3, }, // Africa/Niamey
    {  351, 6766,   94195,  7, }, // Africa/Nouakchott
    {  351, 6690,   94202,  6, }, // Africa/Ouagadougou
    {  351, 5127,   94208,  5, }, // Africa/Porto-Novo
    {  351, 4337,   94213,  4, }, // Africa/Sao_Tome
    {  351, 2732,   94217,  4, }, // Africa/Tripoli
    {  351, 6503,   94221,  4, }, // Africa/Tunis
    {  351, 4054,   94225,  7, }, // Africa/Windhoek
    {  351,  132,   94232,  4, }, // America/Adak
    {  351, 3183,   94236,  6, }, // America/Anchorage
    {  351, 5524,   94242,  4, }, // America/Anguilla
    {  351, 5541,   94246,  6, }, // America/Antigua
    {  351, 4456,   94252,  6, }, // America/Araguaina
    {  351,  166,   94258,  8, }, // America/Argentina/Buenos_Aires
    {  351,   91,   94266,  5, }, // America/Argentina/Catamarca
    {  351,  270,   94271,  4, }, // America/Argentina/Cordoba
    {  351,  441,   94275,  3, }, // America/Argentina/Jujuy
    {  351, 7425,   94278,  4, }, // America/Argentina/La_Rioja
    {  351,  565,   94282,  5, }, // America/Argentina/Mendoza
    {  351, 7452,   94287,  7, }, // America/Argentina/Rio_Gallegos
    {  351, 7483,   94294,  3, }, // America/Argentina/Salta
    {  351, 7507,   94297,  5, }, // America/Argentina/San_Juan
    {  351, 5497,   94302,  5, }, // America/Argentina/San_Luis
    {  351, 7534,   94307,  5, }, // America/Argentina/Tucuman
    {  351, 7560,   94312,  6, }, // America/Argentina/Ushuaia
    {  351, 5557,   94318,  3, }, // America/Aruba
    {  351, 4154,   94321,  6, }, // America/Asuncion
    {  351,  237,   94327,  6, }, // America/Atikokan
    {  351, 3581,   94333,  4, }, // America/Bahia
    {  351, 7586,   94337,  7, }, // America/Bahia_Banderas
    {  351, 5571,   94344,  5, }, // America/Barbados
    {  351, 7609,   94349,  3, }, // America/Belem
    {  351, 5309,   94352,  4, }, // America/Belize
    {  351, 7623,   94356,  8, }, // America/Blanc-Sablon
    {  351, 7644,   94364,  5, }, // America/Boa_Vista
    {  351, 4263,   94369,  3, }, // America/Bogota
    {  351, 7662,   94372,  3, }, // America/Boise
    {  351, 7676,   94375,  8, }, // America/Cambridge_Bay
    {  351, 7698,   94383,  7, }, // America/Campo_Grande
    {  351, 3770,   94390,  4, }, // America/Cancun
    {  351, 4596,   94394,  4, }, // America/Caracas
    {  351, 4247,   94398,  5, }, // America/Cayenne
    {  351, 5398,   94403,  4, }, // America/Cayman
    {  351, 3251,   94407,  3, }, // America/Chicago
    {  351, 7719,   94410,  3, }, // America/Chihuahua
    {  351, 7737,   94413,  9, }, // America/Ciudad_Juarez
    {  351, 5324,   94422,  5, }, // America/Costa_Rica
    {  351, 7759,   94427,  5, }, // America/Creston
    {  351, 3693,   94432,  4, }, // America/Cuiaba
    {  351, 5640,   94436,  5, }, // America/Curacao
    {  351, 7775,   94441,  9, }, // America/Danmarkshavn
    {  351, 7796,   94450,  4, }, // America/Dawson
    {  351, 7811,   94454,  8, }, // America/Dawson_Creek
    {  351,  805,   94462,  4, }, // America/Denver
    {  351, 3351,   94466,  5, }, // America/Detroit
    {  351, 5656,   94471,  4, }, // America/Dominica
    {  351,  893,   94475,  6, }, // America/Edmonton
    {  351, 7832,   94481,  5, }, // America/Eirunepe
    {  351, 5343,   94486,  7, }, // America/El_Salvador
    {  351, 7849,   94493,  8, }, // America/Fort_Nelson
    {  351, 7869,   94501,  6, }, // America/Fortaleza
    {  351, 7887,   94507,  6, }, // America/Glace_Bay
    {  351, 6844,   94513,  5, }, // America/Goose_Bay
    {  351, 4514,   94518,  7, }, // America/Grand_Turk
    {  351, 5687,   94525,  4, }, // America/Grenada
    {  351, 5703,   94529,  6, }, // America/Guadeloupe
    {  351, 3662,   94535,  5, }, // America/Guatemala
    {  351, 4743,   94540,  5, }, // America/Guayaquil
    {  351, 6895,   94545,  4, }, // America/Guyana
    {  351, 1923,   94549,  6, }, // America/Halifax
    {  351, 2221,   94555,  3, }, // America/Havana
    {  351, 7905,   94558,  5, }, // America/Hermosillo
    {  351,  348,   94563,  9, }, // America/Indiana/Indianapolis
    {  351,  481,   94572, 11, }, // America/Indiana/Knox
    {  351, 7924,   94583, 11, }, // America/Indiana/Marengo
    {  351, 7948,   94594, 15, }, // America/Indiana/Petersburg
    {  351, 7975,   94609, 12, }, // America/Indiana/Tell_City
    {  351, 8001,   94621, 10, }, // America/Indiana/Vevay
    {  351, 8023,   94631, 12, }, // America/Indiana/Vincennes
    {  351, 8049,   94643, 13, }, // America/Indiana/Winamac
    {  351, 8073,   94656,  5, }, // America/Inuvik
    {  351,  660,   94661,  6, }, // America/Iqaluit
    {  351, 2665,   94667,  5, }, // America/Jamaica
    {  351, 5282,   94672,  4, }, // America/Juneau
    {  351,  521,   94676,  4, }, // America/Kentucky/Louisville
    {  351, 8088,   94680, 14, }, // America/Kentucky/Monticello
    {  351, 5621,   94694,  7, }, // America/Kralendijk
    {  351, 4278,   94701,  3, }, // America/La_Paz
    {  351, 7132,   94704,  2, }, // America/Lima
    {  351, 3390,   94706,  6, }, // America/Los_Angeles
    {  351, 5849,   94712, 17, }, // America/Lower_Princes
    {  351, 8116,   94729,  4, }, // America/Maceio
    {  351, 8131,   94733,  4, }, // America/Managua
    {  351, 1892,   94737,  4, }, // America/Manaus
    {  351, 5814,   94741,  3, }, // America/Marigot
    {  351, 5722,   94744,  7, }, // America/Martinique
    {  351, 8147,   94751,  5, }, // America/Matamoros
    {  351, 2779,   94756,  5, }, // America/Mazatlan
    {  351, 8165,   94761,  5, }, // America/Menominee
    {  351, 8183,   94766,  3, }, // America/Merida
    {  351, 8198,   94769,  6, }, // America/Metlakatla
    {  351, 2811,   94775,  8, }, // America/Mexico_City
    {  351, 4293,   94783,  5, }, // America/Miquelon
    {  351, 8217,   94788,  5, }, // America/Moncton
    {  351, 8233,   94793,  5, }, // America/Monterrey
    {  351, 4000,   94798,  6, }, // America/Montevideo
    {  351, 5741,   94804,  6, }, // America/Montserrat
    {  351, 5383,   94810,  4, }, // America/Nassau
    {  351, 3294,   94814,  6, }, // America/New_York
    {  351, 8251,   94820,  3, }, // America/Nome
    {  351, 1834,   94823,  5, }, // America/Noronha
    {  351, 8264,   94828, 12, }, // America/North_Dakota/Beulah
    {  351, 8292,   94840, 11, }, // America/North_Dakota/Center
    {  351, 8320,   94851, 14, }, // America/North_Dakota/New_Salem
    {  351,  393,   94865,  3, }, // America/Nuuk
    {  351, 8351,   94868,  4, }, // America/Ojinaga
    {  351, 5413,   94872,  3, }, // America/Panama
    {  351, 6106,   94875,  5, }, // America/Paramaribo
    {  351, 3224,   94880,  6, }, // America/Phoenix
    {  351, 3847,   94886,  8, }, // America/Port-au-Prince
    {  351, 5871,   94894,  9, }, // America/Port_of_Spain
    {  351, 8367,   94903,  7, }, // America/Porto_Velho
    {  351, 5760,   94910,  6, }, // America/Puerto_Rico
    {  351, 3932,   94916,  7, }, // America/Punta_Arenas
    {  351, 8387,   94923,  9, }, // America/Rankin_Inlet
    {  351, 8408,   94932,  4, }, // America/Recife
    {  351, 1979,   94936,  5, }, // America/Regina
    {  351, 8423,   94941,  6, }, // America/Resolute
    {  351,  695,   94947,  6, }, // America/Rio_Branco
    {  351, 8440,   94953,  5, }, // America/Santarem
    {  351, 2165,   94958,  5, }, // America/Santiago
    {  351, 6044,   94963,  7, }, // America/Santo_Domingo
    {  351, 1862,   94970,  5, }, // America/Sao_Paulo
    {  351, 6862,   94975,  9, }, // America/Scoresbysund
    {  351, 8457,   94984,  3, }, // America/Sitka
    {  351, 8471,   94987,  9, }, // America/St_Barthelemy
    {  351, 2045,   94996,  7, }, // America/St_Johns
    {  351, 5780,   95003, 10, }, // America/St_Kitts
    {  351, 5797,   95013,  6, }, // America/St_Lucia
    {  351,  855,   95019,  7, }, // America/St_Thomas
    {  351, 5830,   95026,  8, }, // America/St_Vincent
    {  351, 8493,   95034,  9, }, // America/Swift_Current
    {  351, 5363,   95043,  6, }, // America/Tegucigalpa
    {  351, 5673,   95049,  4, }, // America/Thule
    {  351,  313,   95053,  5, }, // America/Tijuana
    {  351,  608,   95058,  4, }, // America/Toronto
    {  351, 5605,   95062,  5, }, // America/Tortola
    {  351, 2077,   95067,  6, }, // America/Vancouver
    {  351, 2128,   95073,  7, }, // America/Whitehorse
    {  351,  734,   95080,  5, }, // America/Winnipeg
    {  351, 8515,   95085,  5, }, // America/Yakutat
    {  351, 5918,   95090,  6, }, // Antarctica/Casey
    {  351, 6027,   95096,  6, }, // Antarctica/Davis
    {  351, 6066,   95102, 13, }, // Antarctica/DumontDUrville
    {  351, 8531,   95115,  5, }, // Antarctica/Macquarie
    {  351, 7029,   95120,  6, }, // Antarctica/Mawson
    {  351, 7076,   95126,  7, }, // Antarctica/McMurdo
    {  351, 8552,   95133,  6, }, // Antarctica/Palmer
    {  351, 7189,   95139,  5, }, // Antarctica/Rothera
    {  351, 7258,   40823,  4, }, // Antarctica/Syowa
    {  351, 8570,   95144,  5, }, // Antarctica/Troll
    {  351, 7337,   95149,  7, }, // Antarctica/Vostok
    {  351, 1413,   95156, 10, }, // Arctic/Longyearbyen
    {  351, 5487,   95166,  3, }, // Asia/Aden
    {  351, 5297,   95169,  5, }, // Asia/Almaty
    {  351, 3870,   95174,  4, }, // Asia/Amman
    {  351, 5428,   95178,  5, }, // Asia/Anadyr
    {  351, 5440,   95183,  4, }, // Asia/Aqtau
    {  351, 5270,   95187,  4, }, // Asia/Aqtobe
    {  351,  964,   95191,  6, }, // Asia/Ashgabat
    {  351, 8587,   95197,  5, }, // Asia/Atyrau
    {  351, 3509,   95202,  5, }, // Asia/Baghdad
    {  351, 5451,   95207,  5, }, // Asia/Bahrain
    {  351, 3555,   95212,  3, }, // Asia/Baku
    {  351, 4368,   95215,  4, }, // Asia/Bangkok
    {  351, 3473,   95219,  5, }, // Asia/Barnaul
    {  351, 3988,   95224,  5, }, // Asia/Beirut
    {  351, 3680,   95229,  5, }, // Asia/Bishkek
    {  351, 5906,   95234,  4, }, // Asia/Brunei
    {  351, 4503,   95238,  2, }, // Asia/Chita
    {  351, 5963,   95240,  7, }, // Asia/Choibalsan
    {  351, 4413,   95247,  4, }, // Asia/Colombo
    {  351, 4442,   95251,  5, }, // Asia/Damascus
    {  351, 1060,   95256,  3, }, // Asia/Dhaka
    {  351, 6125,   95180,  3, }, // Asia/Dili
    {  351, 3498,   95259,  3, }, // Asia/Dubai
    {  351, 6092,   95262,  6, }, // Asia/Dushanbe
    {  351, 8599,   95268,  6, }, // Asia/Famagusta
    {  351, 8614,   95274,  2, }, // Asia/Gaza
    {  351, 4697,   95276,  4, }, // Asia/Hebron
    {  351, 1226,   95280,  5, }, // Asia/Ho_Chi_Minh
    {  351, 2576,   40926,  2, }, // Asia/Hong_Kong
    {  351, 4673,   95285,  3, }, // Asia/Hovd
    {  351, 4086,   95288,  6, }, // Asia/Irkutsk
    {  351, 6969,   95294,  5, }, // Asia/Jakarta
    {  351, 6955,   95299,  5, }, // Asia/Jayapura
    {  351, 1257,   95304,  5, }, // Asia/Jerusalem
    {  351, 3462,   95309,  4, }, // Asia/Kabul
    {  351, 4218,   95313,  6, }, // Asia/Kamchatka
    {  351, 4141,   95319,  3, }, // Asia/Karachi
    {  351, 1152,   95322,  5, }, // Asia/Kathmandu
    {  351, 8624,   95327,  6, }, // Asia/Khandyga
    {  351,  992,   95333,  4, }, // Asia/Kolkata
    {  351, 4099,   95337,  8, }, // Asia/Krasnoyarsk
    {  351, 4761,   95345,  8, }, // Asia/Kuala_Lumpur
    {  351, 5893,   95353,  3, }, // Asia/Kuching
    {  351, 5464,   95356,  5, }, // Asia/Kuwait
    {  351, 1178,   95361,  3, }, // Asia/Macau
    {  351, 3919,   95364,  4, }, // Asia/Magadan
    {  351, 1316,   95368,  5, }, // Asia/Makassar
    {  351, 7145,   95373,  3, }, // Asia/Manila
    {  351, 6883,   95376,  5, }, // Asia/Muscat
    {  351, 2446,   95381,  4, }, // Asia/Nicosia
    {  351, 8638,   95385,  8, }, // Asia/Novokuznetsk
    {  351, 4037,   95393,  8, }, // Asia/Novosibirsk
    {  351, 4131,   95401,  4, }, // Asia/Omsk
    {  351, 7108,   95405,  3, }, // Asia/Oral
    {  351, 6924,   95408,  5, }, // Asia/Phnom_Penh
    {  351, 8656,   95413,  8, }, // Asia/Pontianak
    {  351, 4116,   95421,  2, }, // Asia/Pyongyang
    {  351, 5476,   95423,  4, }, // Asia/Qatar
    {  351, 8671,   95427,  5, }, // Asia/Qostanay
    {  351, 4171,   95432,  5, }, // Asia/Qyzylorda
    {  351, 3486,   95437,  3, }, // Asia/Riyadh
    {  351, 4310,   95440,  4, }, // Asia/Sakhalin
    {  351, 7208,   95444,  6, }, // Asia/Samarkand
    {  351, 3116,   95450,  3, }, // Asia/Seoul
    {  351, 1020,   41068,  2, }, // Asia/Shanghai
    {  351, 3137,   95453,  6, }, // Asia/Singapore
    {  351, 4199,   95459,  9, }, // Asia/Srednekolymsk
    {  351, 3100,   41079,  2, }, // Asia/Taipei
    {  351, 4683,   95468,  5, }, // Asia/Tashkent
    {  351, 3817,   95473,  4, }, // Asia/Tbilisi
    {  351, 2638,   95477,  4, }, // Asia/Tehran
    {  351, 1284,   95481,  5, }, // Asia/Thimphu
    {  351, 2687,   41093,  2, }, // Asia/Tokyo
    {  351, 4474,   95486,  4, }, // Asia/Tomsk
    {  351, 1346,   95490,  7, }, // Asia/Ulaanbaatar
    {  351, 1126,   95497,  4, }, // Asia/Urumqi
    {  351, 8685,   95501,  5, }, // Asia/Ust-Nera
    {  351, 6940,   95506,  6, }, // Asia/Vientiane
    {  351, 4612,   95512,  7, }, // Asia/Vladivostok
    {  351, 4730,   95519,  5, }, // Asia/Yakutsk
    {  351, 1202,   95524,  4, }, // Asia/Yangon
    {  351, 3785,   95528,  8, }, // Asia/Yekaterinburg
    {  351, 3649,   95536,  4, }, // Asia/Yerevan
    {  351, 3565,   95540,  4, }, // Atlantic/Azores
    {  351, 5588,   95544,  5, }, // Atlantic/Bermuda
    {  351, 6591,   95549,  4, }, // Atlantic/Canary
    {  351, 3629,   95553,  6, }, // Atlantic/Cape_Verde
    {  351, 1379,   95559,  4, }, // Atlantic/Faroe
    {  351, 8699,   95563,  4, }, // Atlantic/Madeira
    {  351, 2614,   95567,  6, }, // Atlantic/Reykjavik
    {  351, 7235,   95573,  8, }, // Atlantic/South_Georgia
    {  351, 6784,   95581,  6, }, // Atlantic/St_Helena
    {  351, 6607,   95587,  5, }, // Atlantic/Stanley
    {  351, 1654,   95592,  5, }, // Australia/Adelaide
    {  351, 1619,   95597,  5, }, // Australia/Brisbane
    {  351, 1783,   95602,  7, }, // Australia/Broken_Hill
    {  351, 1567,   95609,  5, }, // Australia/Darwin
    {  351, 3539,   95614,  4, }, // Australia/Eucla
    {  351, 1500,   95618,  4, }, // Australia/Hobart
    {  351, 8716,   95622,  5, }, // Australia/Lindeman
    {  351, 1531,   95627,  5, }, // Australia/Lord_Howe
    {  351, 1711,   95632,  5, }, // Australia/Melbourne
    {  351, 1746,   95637,  3, }, // Australia/Perth
    {  351, 1447,   95640,  4, }, // Australia/Sydney
    {  351, 6390,   95644,  7, }, // Europe/Amsterdam
    {  351, 6149,   95651,  4, }, // Europe/Andorra
    {  351, 3522,   95655,  6, }, // Europe/Astrakhan
    {  351, 6577,   95661,  3, }, // Europe/Athens
    {  351, 6285,   95664,  6, }, // Europe/Belgrade
    {  351, 4659,   95670,  4, }, // Europe/Berlin
    {  351, 6437,   95674,  6, }, // Europe/Bratislava
    {  351, 6178,   95680,  6, }, // Europe/Brussels
    {  351, 3830,   95686,  5, }, // Europe/Bucharest
    {  351, 3708,   95691,  5, }, // Europe/Budapest
    {  351, 8751,   95696,  7, }, // Europe/Busingen
    {  351, 2475,   95703,  4, }, // Europe/Chisinau
    {  351, 6238,   95707,  7, }, // Europe/Copenhagen
    {  351, 2260,   95714,  4, }, // Europe/Dublin
    {  351, 6256,   95718,  6, }, // Europe/Gibraltar
    {  351, 8767,   95724,  5, }, // Europe/Guernsey
    {  351, 6561,   95729,  5, }, // Europe/Helsinki
    {  351, 8783,   95734,  3, }, // Europe/Isle_of_Man
    {  351, 1097,   95737,  7, }, // Europe/Istanbul
    {  351, 8802,   95744,  5, }, // Europe/Jersey
    {  351, 3881,   95749,  9, }, // Europe/Kaliningrad
    {  351, 8924,   95758,  4, }, // Europe/Kirov
    {  351, 2419,   95762,  3, }, // Europe/Kyiv
    {  351, 3078,   95765,  4, }, // Europe/Lisbon
    {  351, 6455,   95769,  6, }, // Europe/Ljubljana
    {  351, 2393,   95775,  4, }, // Europe/London
    {  351, 6314,   95779,  7, }, // Europe/Luxembourg
    {  351, 4794,   95786,  5, }, // Europe/Madrid
    {  351, 6346,   95791,  3, }, // Europe/Malta
    {  351, 6531,   95794,  6, }, // Europe/Mariehamn
    {  351, 3595,   95800,  4, }, // Europe/Minsk
    {  351, 6359,   95804,  3, }, // Europe/Monaco
    {  351, 3443,   95807,  4, }, // Europe/Moscow
    {  351, 6407,   95811,  3, }, // Europe/Oslo
    {  351, 4186,   95814,  2, }, // Europe/Paris
    {  351, 6373,   95816,  6, }, // Europe/Podgorica
    {  351, 6224,   95822,  3, }, // Europe/Prague
    {  351, 8816,   95825,  2, }, // Europe/Riga
    {  351, 6273,   95561,  3, }, // Europe/Rome
    {  351, 4233,   95827,  3, }, // Europe/Samara
    {  351, 6419,   95830,  5, }, // Europe/San_Marino
    {  351, 6194,   95835,  4, }, // Europe/Sarajevo
    {  351, 4353,   95839,  4, }, // Europe/Saratov
    {  351, 8828,   95843,  7, }, // Europe/Simferopol
    {  351, 6332,   95850,  4, }, // Europe/Skopje
    {  351, 6548,   95854,  4, }, // Europe/Sofia
    {  351, 6472,   95858,  7, }, // Europe/Stockholm
    {  351, 8846,   95865,  3, }, // Europe/Tallinn
    {  351, 6135,   95868,  4, }, // Europe/Tirane
    {  351, 8861,   95872,  7, }, // Europe/Ulyanovsk
    {  351, 6301,   95879,  6, }, // Europe/Vaduz
    {  351, 6516,   95885,  4, }, // Europe/Vatican
    {  351, 6164,   95889,  4, }, // Europe/Vienna
    {  351, 8878,   95893,  6, }, // Europe/Vilnius
    {  351, 4629,   95899,  7, }, // Europe/Volgograd
    {  351, 3055,   95906,  5, }, // Europe/Warsaw
    {  351, 6210,   95911,  4, }, // Europe/Zagreb
    {  351, 6489,   95915,  6, }, // Europe/Zurich
    {  351, 4980,   95921,  7, }, // Indian/Antananarivo
    {  351, 6910,   95928,  4, }, // Indian/Chagos
    {  351, 5979,   95932,  6, }, // Indian/Christmas
    {  351, 5996,   95938,  5, }, // Indian/Cocos
    {  351, 4931,   95943,  3, }, // Indian/Comoro
    {  351, 6624,   95946,  7, }, // Indian/Kerguelen
    {  351, 7223,   95953,  2, }, // Indian/Mahe
    {  351, 7013,   95955,  5, }, // Indian/Maldives
    {  351, 3971,   95960,  6, }, // Indian/Mauritius
    {  351, 5000,   95966,  4, }, // Indian/Mayotte
    {  351, 7174,   95970,  5, }, // Indian/Reunion
    {  351, 4324,   95975,  3, }, // Pacific/Apia
    {  351,  932,   95978,  6, }, // Pacific/Auckland
    {  351, 3608,   95984,  6, }, // Pacific/Bougainville
    {  351, 2863,   95990,  4, }, // Pacific/Chatham
    {  351, 3022,   95994,  4, }, // Pacific/Chuuk
    {  351, 2201,   95998,  6, }, // Pacific/Easter
    {  351, 7323,   96004,  5, }, // Pacific/Efate
    {  351, 7290,   96009,  6, }, // Pacific/Fakaofo
    {  351, 3804,   96015,  4, }, // Pacific/Fiji
    {  351, 7306,   96019,  5, }, // Pacific/Funafuti
    {  351, 6641,   96024,  5, }, // Pacific/Galapagos
    {  351, 6659,   96029,  6, }, // Pacific/Gambier
    {  351, 3724,   96035,  6, }, // Pacific/Guadalcanal
    {  351, 5935,   96041,  3, }, // Pacific/Guam
    {  351, 2929,   96044,  4, }, // Pacific/Honolulu
    {  351, 2897,   96048,  8, }, // Pacific/Kanton
    {  351, 3900,   96056,  8, }, // Pacific/Kiritimati
    {  351, 6982,   96064,  4, }, // Pacific/Kosrae
    {  351, 2708,   96068,  5, }, // Pacific/Kwajalein
    {  351, 4779,   96073,  4, }, // Pacific/Majuro
    {  351, 3953,   96077,  5, }, // Pacific/Marquesas
    {  351, 8909,   96082,  7, }, // Pacific/Midway
    {  351, 7047,   95221,  3, }, // Pacific/Nauru
    {  351, 7095,   96089,  3, }, // Pacific/Niue
    {  351, 4070,   96092,  7, }, // Pacific/Norfolk
    {  351, 7061,   96099,  3, }, // Pacific/Noumea
    {  351, 2991,   96102,  4, }, // Pacific/Pago_Pago
    {  351, 7118,   96106,  3, }, // Pacific/Palau
    {  351, 7157,   96109,  7, }, // Pacific/Pitcairn
    {  351, 2961,   96116,  5, }, // Pacific/Pohnpei
    {  351, 4709,   96121,  8, }, // Pacific/Port_Moresby
    {  351, 6009,   96129,  5, }, // Pacific/Rarotonga
    {  351, 5948,   96134,  4, }, // Pacific/Saipan
    {  351, 7275,   96138,  3, }, // Pacific/Tahiti
    {  351, 6675,   96141,  3, }, // Pacific/Tarawa
    {  351, 4485,   96144,  5, }, // Pacific/Tongatapu
    {  351, 7355,   96149,  5, }, // Pacific/Wake
    {  351, 7368,   96154,  6, }, // Pacific/Wallis
    {  352,   14,       0,  6, }, // Africa/Asmara Javanese/Latin/Indonesia
    {  352, 2242,   24913,  5, }, // Africa/Cairo
    {  352, 4019,   19744, 10, }, // Africa/Casablanca
    {  352, 6736,    4637,  7, }, // Africa/Conakry
    {  352, 4337,       6,  8, }, // Africa/Sao_Tome
    {  352,   91,    4788,  9, }, // America/Argentina/Catamarca
    {  352,  270,    4797,  7, }, // America/Argentina/Cordoba
    {  352,  565,   96160,  7, }, // America/Argentina/Mendoza
    {  352, 4154,      14,  8, }, // America/Asuncion
    {  352,  237,      22,  8, }, // America/Atikokan
    {  352, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  352, 5309,   96167,  6, }, // America/Belize
    {  352, 7676,   96173, 15, }, // America/Cambridge_Bay
    {  352, 7698,   96188, 12, }, // America/Campo_Grande
    {  352, 3770,    3861,  6, }, // America/Cancun
    {  352, 4596,    4965,  7, }, // America/Caracas
    {  352, 4247,   96200,  7, }, // America/Cayenne
    {  352, 5398,   42677,  6, }, // America/Cayman
    {  352, 7737,   96207, 13, }, // America/Ciudad_Juarez
    {  352, 5324,   20026, 10, }, // America/Costa_Rica
    {  352, 3693,   96220,  6, }, // America/Cuiaba
    {  352, 5640,      66,  7, }, // America/Curacao
    {  352, 5656,   20071,  8, }, // America/Dominica
    {  352, 7849,   96226, 14, }, // America/Fort_Nelson
    {  352, 7887,   96240, 11, }, // America/Glace_Bay
    {  352, 6844,   96251, 11, }, // America/Goose_Bay
    {  352, 1923,    5189,  7, }, // America/Halifax
    {  352,  481,   63133, 14, }, // America/Indiana/Knox
    {  352, 7924,   63147, 17, }, // America/Indiana/Marengo
    {  352, 7948,   63164, 20, }, // America/Indiana/Petersburg
    {  352, 7975,   63184, 19, }, // America/Indiana/Tell_City
    {  352, 8001,   63203, 15, }, // America/Indiana/Vevay
    {  352, 8023,   63218, 19, }, // America/Indiana/Vincennes
    {  352, 8049,   63237, 17, }, // America/Indiana/Winamac
    {  352, 8088,   63262, 21, }, // America/Kentucky/Monticello
    {  352, 5849,     207, 22, }, // America/Lower_Princes
    {  352, 5722,    5400,  8, }, // America/Martinique
    {  352, 8183,    3899,  6, }, // America/Merida
    {  352, 2811,   96262, 13, }, // America/Mexico_City
    {  352, 8264,   96275, 19, }, // America/North_Dakota/Beulah
    {  352, 8292,   96294, 19, }, // America/North_Dakota/Center
    {  352, 8320,   96313, 24, }, // America/North_Dakota/New_Salem
    {  352,  393,     298,  4, }, // America/Nuuk
    {  352, 5871,   96337, 17, }, // America/Port_of_Spain
    {  352, 5760,   20463, 11, }, // America/Puerto_Rico
    {  352, 6862,     302, 16, }, // America/Scoresbysund
    {  352, 8471,   96354, 16, }, // America/St_Barthelemy
    {  352, 2045,   96370, 10, }, // America/St_Johns
    {  352, 5780,   96380, 11, }, // America/St_Kitts
    {  352, 5797,   93759, 11, }, // America/St_Lucia
    {  352,  855,   96391, 12, }, // America/St_Thomas
    {  352, 5830,   96403, 13, }, // America/St_Vincent
    {  352, 8493,   96416, 11, }, // America/Swift_Current
    {  352, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  352, 4413,    5949,  7, }, // Asia/Colombo
    {  352, 4442,    4102,  8, }, // Asia/Damascus
    {  352, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  352, 1257,   72485,  9, }, // Asia/Jerusalem
    {  352, 1152,     413,  9, }, // Asia/Kathmandu
    {  352,  992,   25307,  7, }, // Asia/Kolkata
    {  352, 1178,    4222,  5, }, // Asia/Macau
    {  352, 1316,    6069,  7, }, // Asia/Makassar
    {  352, 8671,   62738,  8, }, // Asia/Qostanay
    {  352, 3137,   91698,  9, }, // Asia/Singapore
    {  352, 2638,    6175,  7, }, // Asia/Tehran
    {  352, 1202,     442,  6, }, // Asia/Yangon
    {  352, 6591,    4299,  6, }, // Atlantic/Canary
    {  352, 3629,   21064, 10, }, // Atlantic/Cape_Verde
    {  352, 1379,     448,  5, }, // Atlantic/Faroe
    {  352, 7235,   96427, 13, }, // Atlantic/South_Georgia
    {  352, 6784,   25447, 12, }, // Atlantic/St_Helena
    {  352, 6577,   91735,  6, }, // Europe/Athens
    {  352, 6238,    4376, 10, }, // Europe/Copenhagen
    {  352, 8783,   96440,  8, }, // Europe/Isle_of_Man
    {  352, 2419,    6486,  4, }, // Europe/Kyiv
    {  352, 6314,   96448, 10, }, // Europe/Luxembourg
    {  352, 6359,    6525,  6, }, // Europe/Monaco
    {  352, 6273,   19581,  4, }, // Europe/Rome
    {  352, 6516,    6614,  7, }, // Europe/Vatican
    {  352, 6910,   96458,  6, }, // Indian/Chagos
    {  352, 5979,   91946,  5, }, // Indian/Christmas
    {  352, 4931,   21395,  6, }, // Indian/Comoro
    {  352, 7013,   91771,  8, }, // Indian/Maldives
    {  352, 7174,     467,  7, }, // Indian/Reunion
    {  352, 3022,     474,  5, }, // Pacific/Chuuk
    {  352, 2201,   96464,  6, }, // Pacific/Easter
    {  352, 2897,    4520,  9, }, // Pacific/Kanton
    {  352, 2961,     479,  7, }, // Pacific/Pohnpei
    {  352, 4709,   96470, 17, }, // Pacific/Port_Moresby
    {  353,   14,       0,  6, }, // Africa/Asmara Jju/Latin/Nigeria
    {  353, 4337,       6,  8, }, // Africa/Sao_Tome
    {  353, 4154,      14,  8, }, // America/Asuncion
    {  353,  237,      22,  8, }, // America/Atikokan
    {  353, 7586,      30, 17, }, // America/Bahia_Banderas
    {  353, 3770,      47,  6, }, // America/Cancun
    {  353, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  353, 5640,      66,  7, }, // America/Curacao
    {  353,  481,      73, 13, }, // America/Indiana/Knox
    {  353, 7924,      86, 16, }, // America/Indiana/Marengo
    {  353, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  353, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  353, 8001,     139, 14, }, // America/Indiana/Vevay
    {  353, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  353, 8049,     171, 16, }, // America/Indiana/Winamac
    {  353, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  353, 5849,     207, 22, }, // America/Lower_Princes
    {  353, 8183,     229,  6, }, // America/Merida
    {  353, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  353, 8292,     255, 20, }, // America/North_Dakota/Center
    {  353, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  353,  393,     298,  4, }, // America/Nuuk
    {  353, 6862,     302, 16, }, // America/Scoresbysund
    {  353, 8471,     318, 14, }, // America/St_Barthelemy
    {  353, 2045,     332, 10, }, // America/St_Johns
    {  353, 5780,     342,  9, }, // America/St_Kitts
    {  353, 5797,     351,  9, }, // America/St_Lucia
    {  353,  855,     360, 10, }, // America/St_Thomas
    {  353, 5830,     370, 11, }, // America/St_Vincent
    {  353, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  353, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  353, 1152,     413,  9, }, // Asia/Kathmandu
    {  353,  992,     422,  7, }, // Asia/Kolkata
    {  353, 1178,     429,  5, }, // Asia/Macau
    {  353, 1202,     442,  6, }, // Asia/Yangon
    {  353, 1379,     448,  5, }, // Atlantic/Faroe
    {  353, 6784,     453, 10, }, // Atlantic/St_Helena
    {  353, 2419,     463,  4, }, // Europe/Kyiv
    {  353, 7174,     467,  7, }, // Indian/Reunion
    {  353, 3022,     474,  5, }, // Pacific/Chuuk
    {  353, 2961,     479,  7, }, // Pacific/Pohnpei
    {  354,   14,       0,  6, }, // Africa/Asmara Jola-Fonyi/Latin/Senegal
    {  354, 4337,       6,  8, }, // Africa/Sao_Tome
    {  354, 4154,      14,  8, }, // America/Asuncion
    {  354,  237,      22,  8, }, // America/Atikokan
    {  354, 7586,      30, 17, }, // America/Bahia_Banderas
    {  354, 3770,      47,  6, }, // America/Cancun
    {  354, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  354, 5640,      66,  7, }, // America/Curacao
    {  354,  481,      73, 13, }, // America/Indiana/Knox
    {  354, 7924,      86, 16, }, // America/Indiana/Marengo
    {  354, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  354, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  354, 8001,     139, 14, }, // America/Indiana/Vevay
    {  354, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  354, 8049,     171, 16, }, // America/Indiana/Winamac
    {  354, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  354, 5849,     207, 22, }, // America/Lower_Princes
    {  354, 8183,     229,  6, }, // America/Merida
    {  354, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  354, 8292,     255, 20, }, // America/North_Dakota/Center
    {  354, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  354,  393,     298,  4, }, // America/Nuuk
    {  354, 6862,     302, 16, }, // America/Scoresbysund
    {  354, 8471,     318, 14, }, // America/St_Barthelemy
    {  354, 2045,     332, 10, }, // America/St_Johns
    {  354, 5780,     342,  9, }, // America/St_Kitts
    {  354, 5797,     351,  9, }, // America/St_Lucia
    {  354,  855,     360, 10, }, // America/St_Thomas
    {  354, 5830,     370, 11, }, // America/St_Vincent
    {  354, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  354, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  354, 1152,     413,  9, }, // Asia/Kathmandu
    {  354,  992,     422,  7, }, // Asia/Kolkata
    {  354, 1178,     429,  5, }, // Asia/Macau
    {  354, 1202,     442,  6, }, // Asia/Yangon
    {  354, 1379,     448,  5, }, // Atlantic/Faroe
    {  354, 6784,     453, 10, }, // Atlantic/St_Helena
    {  354, 2419,     463,  4, }, // Europe/Kyiv
    {  354, 7174,     467,  7, }, // Indian/Reunion
    {  354, 3022,     474,  5, }, // Pacific/Chuuk
    {  354, 2961,     479,  7, }, // Pacific/Pohnpei
    {  355,   14,       0,  6, }, // Africa/Asmara Kabuverdianu/Latin/Cape Verde
    {  355, 4337,       6,  8, }, // Africa/Sao_Tome
    {  355,  132,   96487,  5, }, // America/Adak
    {  355, 3183,   96492,  8, }, // America/Anchorage
    {  355, 5524,   96500,  6, }, // America/Anguilla
    {  355, 4154,      14,  8, }, // America/Asuncion
    {  355,  237,      22,  8, }, // America/Atikokan
    {  355, 7586,   96506, 16, }, // America/Bahia_Banderas
    {  355, 5571,   96522,  8, }, // America/Barbados
    {  355, 5309,   96530,  6, }, // America/Belize
    {  355, 7623,    4897, 12, }, // America/Blanc-Sablon
    {  355, 7662,    4924,  4, }, // America/Boise
    {  355, 3770,    4959,  6, }, // America/Cancun
    {  355, 5398,   25001,  6, }, // America/Cayman
    {  355, 3251,   96536,  6, }, // America/Chicago
    {  355, 7719,   96542,  6, }, // America/Chihuahua
    {  355, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  355, 5324,   20026, 10, }, // America/Costa_Rica
    {  355, 7759,   96548,  7, }, // America/Creston
    {  355, 5640,   96555,  7, }, // America/Curacao
    {  355, 5656,   20071,  8, }, // America/Dominica
    {  355, 5687,   18910,  7, }, // America/Grenada
    {  355, 5703,   96562,  9, }, // America/Guadeloupe
    {  355, 3662,   96571,  9, }, // America/Guatemala
    {  355, 2221,   93639,  5, }, // America/Havana
    {  355,  348,   72702, 12, }, // America/Indiana/Indianapolis
    {  355,  481,      73, 13, }, // America/Indiana/Knox
    {  355, 7924,      86, 16, }, // America/Indiana/Marengo
    {  355, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  355, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  355, 8001,     139, 14, }, // America/Indiana/Vevay
    {  355, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  355, 8049,     171, 16, }, // America/Indiana/Winamac
    {  355, 2665,    3892,  7, }, // America/Jamaica
    {  355, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  355, 5849,     207, 22, }, // America/Lower_Princes
    {  355, 8131,   96580,  7, }, // America/Managua
    {  355, 5722,   25027,  9, }, // America/Martinique
    {  355, 8183,     229,  6, }, // America/Merida
    {  355, 2811,   96587, 16, }, // America/Mexico_City
    {  355, 4293,    5434,  7, }, // America/Miquelon
    {  355, 3294,   96603, 10, }, // America/New_York
    {  355, 8264,   96613, 23, }, // America/North_Dakota/Beulah
    {  355, 8292,   96636, 23, }, // America/North_Dakota/Center
    {  355, 8320,   96659, 26, }, // America/North_Dakota/New_Salem
    {  355,  393,     298,  4, }, // America/Nuuk
    {  355, 5413,   19056,  6, }, // America/Panama
    {  355, 5871,   96685, 15, }, // America/Port_of_Spain
    {  355, 5760,   96700, 10, }, // America/Puerto_Rico
    {  355, 6044,   96710, 13, }, // America/Santo_Domingo
    {  355, 6862,     302, 16, }, // America/Scoresbysund
    {  355, 8471,   96723, 14, }, // America/St_Barthelemy
    {  355, 2045,     332, 10, }, // America/St_Johns
    {  355, 5780,   96737, 13, }, // America/St_Kitts
    {  355, 5797,   96750, 11, }, // America/St_Lucia
    {  355,  855,     360, 10, }, // America/St_Thomas
    {  355, 5830,   96761, 11, }, // America/St_Vincent
    {  355,  608,   96772,  7, }, // America/Toronto
    {  355, 2077,    5789,  8, }, // America/Vancouver
    {  355,  734,   96779,  7, }, // America/Winnipeg
    {  355, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  355, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  355, 1152,     413,  9, }, // Asia/Kathmandu
    {  355,  992,     422,  7, }, // Asia/Kolkata
    {  355, 1178,     429,  5, }, // Asia/Macau
    {  355, 1202,     442,  6, }, // Asia/Yangon
    {  355, 1379,     448,  5, }, // Atlantic/Faroe
    {  355, 6784,     453, 10, }, // Atlantic/St_Helena
    {  355, 2419,     463,  4, }, // Europe/Kyiv
    {  355, 7174,     467,  7, }, // Indian/Reunion
    {  355, 3022,     474,  5, }, // Pacific/Chuuk
    {  355, 2961,     479,  7, }, // Pacific/Pohnpei
    {  356, 6751,   96786,  6, }, // Africa/Abidjan Kabyle/Latin/Algeria
    {  356, 6723,   96792,  4, }, // Africa/Accra
    {  356, 4961,   96796, 10, }, // Africa/Addis_Ababa
    {  356, 7383,   96806,  8, }, // Africa/Algiers
    {  356,   14,       0,  6, }, // Africa/Asmara
    {  356,   44,   96814,  6, }, // Africa/Bamako
    {  356, 5159,   24908,  5, }, // Africa/Bangui
    {  356, 7398,   96820,  5, }, // Africa/Bissau
    {  356, 4859,    4588,  7, }, // Africa/Blantyre
    {  356, 5189,   96825,  8, }, // Africa/Brazzaville
    {  356, 2242,   96833,  7, }, // Africa/Cairo
    {  356, 4019,   96840, 10, }, // Africa/Casablanca
    {  356, 7412,   96850,  5, }, // Africa/Ceuta
    {  356, 6736,   96855,  7, }, // Africa/Conakry
    {  356, 5032,   96862,  9, }, // Africa/Dar_es_Salaam
    {  356, 4945,   96871,  6, }, // Africa/Djibouti
    {  356, 5145,   96877,  5, }, // Africa/Douala
    {  356, 5068,   96882,  6, }, // Africa/El_Aaiun
    {  356, 6816,   96888,  7, }, // Africa/Freetown
    {  356, 4808,   96895,  7, }, // Africa/Gaborone
    {  356, 4917,   96902,  6, }, // Africa/Harare
    {  356, 4381,   96908, 11, }, // Africa/Johannesburg
    {  356, 4426,   96919,  6, }, // Africa/Khartoum
    {  356, 5208,   96925,  7, }, // Africa/Kinshasa
    {  356, 4646,   96932,  5, }, // Africa/Lagos
    {  356, 5238,   96937,  8, }, // Africa/Libreville
    {  356, 6832,   96945,  4, }, // Africa/Lome
    {  356, 5113,   96949,  6, }, // Africa/Luanda
    {  356, 4841,   96955,  9, }, // Africa/Lubumbashi
    {  356, 4903,   96964,  6, }, // Africa/Lusaka
    {  356, 5224,   96970,  6, }, // Africa/Malabo
    {  356, 4889,   96976,  6, }, // Africa/Maputo
    {  356, 5099,   96982,  6, }, // Africa/Maseru
    {  356, 5084,   59848,  6, }, // Africa/Mbabane
    {  356, 5015,   96988,  8, }, // Africa/Mogadishu
    {  356, 6997,   96996,  8, }, // Africa/Monrovia
    {  356, 3755,   97004,  7, }, // Africa/Nairobi
    {  356, 5173,   97011,  7, }, // Africa/Ndjamena
    {  356, 5256,   97018,  6, }, // Africa/Niamey
    {  356, 6766,   97024,  7, }, // Africa/Nouakchott
    {  356, 6690,   97031,  8, }, // Africa/Ouagadougou
    {  356, 5127,   97039, 10, }, // Africa/Porto-Novo
    {  356, 4337,   97049,  8, }, // Africa/Sao_Tome
    {  356, 2732,   97057,  7, }, // Africa/Tripoli
    {  356, 6503,   72657,  5, }, // Africa/Tunis
    {  356, 4054,   97064,  7, }, // Africa/Windhoek
    {  356, 3183,   97071,  7, }, // America/Anchorage
    {  356, 5524,   96500,  6, }, // America/Anguilla
    {  356, 5541,   97078,  6, }, // America/Antigua
    {  356,   91,    4788,  9, }, // America/Argentina/Catamarca
    {  356,  270,   97084,  7, }, // America/Argentina/Cordoba
    {  356,  565,   97091,  7, }, // America/Argentina/Mendoza
    {  356, 7507,   97098,  8, }, // America/Argentina/San_Juan
    {  356, 5497,   97106,  8, }, // America/Argentina/San_Luis
    {  356, 7534,    4850,  7, }, // America/Argentina/Tucuman
    {  356, 7560,   97114,  6, }, // America/Argentina/Ushuaia
    {  356, 4154,   97120,  8, }, // America/Asuncion
    {  356,  237,   97128,  8, }, // America/Atikokan
    {  356, 3581,   97136,  5, }, // America/Bahia
    {  356, 7586,   97141, 14, }, // America/Bahia_Banderas
    {  356, 5571,   96522,  8, }, // America/Barbados
    {  356, 5309,   97155,  5, }, // America/Belize
    {  356, 7623,   97160, 11, }, // America/Blanc-Sablon
    {  356, 4263,   97171,  6, }, // America/Bogota
    {  356, 7662,   97177,  4, }, // America/Boise
    {  356, 7676,   97181, 11, }, // America/Cambridge_Bay
    {  356, 7698,   97192, 12, }, // America/Campo_Grande
    {  356, 3770,    4959,  6, }, // America/Cancun
    {  356, 4596,    4965,  7, }, // America/Caracas
    {  356, 4247,   97204,  5, }, // America/Cayenne
    {  356, 5398,   20006,  6, }, // America/Cayman
    {  356, 3251,   97209,  6, }, // America/Chicago
    {  356, 7719,   97215,  6, }, // America/Chihuahua
    {  356, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  356, 5324,   97221, 10, }, // America/Costa_Rica
    {  356, 7759,   97231,  7, }, // America/Creston
    {  356, 5640,   97238,  7, }, // America/Curacao
    {  356, 7775,   97245, 12, }, // America/Danmarkshavn
    {  356, 7796,   97257,  6, }, // America/Dawson
    {  356, 7811,   97263, 11, }, // America/Dawson_Creek
    {  356, 3351,   97274,  6, }, // America/Detroit
    {  356, 5656,   97280,  7, }, // America/Dominica
    {  356,  893,   97287,  8, }, // America/Edmonton
    {  356, 5343,   97295,  8, }, // America/El_Salvador
    {  356, 7869,   97303,  9, }, // America/Fortaleza
    {  356, 5687,   97312,  7, }, // America/Grenada
    {  356, 5703,   97319,  9, }, // America/Guadeloupe
    {  356, 3662,   97328,  9, }, // America/Guatemala
    {  356, 4743,   97337,  7, }, // America/Guayaquil
    {  356, 6895,   97344,  5, }, // America/Guyana
    {  356,  481,      73, 13, }, // America/Indiana/Knox
    {  356, 7924,      86, 16, }, // America/Indiana/Marengo
    {  356, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  356, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  356, 8001,     139, 14, }, // America/Indiana/Vevay
    {  356, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  356, 8049,     171, 16, }, // America/Indiana/Winamac
    {  356, 2665,   97349,  7, }, // America/Jamaica
    {  356, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  356, 5849,     207, 22, }, // America/Lower_Princes
    {  356, 8131,   97356,  6, }, // America/Managua
    {  356, 5814,   97362,  7, }, // America/Marigot
    {  356, 5722,    5400,  8, }, // America/Martinique
    {  356, 8183,    3899,  6, }, // America/Merida
    {  356, 1834,   97369,  7, }, // America/Noronha
    {  356, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  356, 8292,     255, 20, }, // America/North_Dakota/Center
    {  356, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  356,  393,   60593,  3, }, // America/Nuuk
    {  356, 6106,   97376, 10, }, // America/Paramaribo
    {  356, 8367,   97386, 11, }, // America/Porto_Velho
    {  356, 8408,   97397,  5, }, // America/Recife
    {  356,  695,   97402, 10, }, // America/Rio_Branco
    {  356, 2165,   97412,  8, }, // America/Santiago
    {  356, 1862,   97420,  9, }, // America/Sao_Paulo
    {  356, 6862,     302, 16, }, // America/Scoresbysund
    {  356, 8471,     318, 14, }, // America/St_Barthelemy
    {  356, 2045,     332, 10, }, // America/St_Johns
    {  356, 5780,     342,  9, }, // America/St_Kitts
    {  356, 5797,     351,  9, }, // America/St_Lucia
    {  356,  855,     360, 10, }, // America/St_Thomas
    {  356, 5830,     370, 11, }, // America/St_Vincent
    {  356, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  356, 8531,   97429,  6, }, // Antarctica/Macquarie
    {  356, 7337,   97435,  6, }, // Antarctica/Vostok
    {  356, 1413,   97441, 11, }, // Arctic/Longyearbyen
    {  356, 5487,   97452,  5, }, // Asia/Aden
    {  356, 5297,    5890,  6, }, // Asia/Almaty
    {  356, 3870,   97457,  6, }, // Asia/Amman
    {  356, 5428,    5900,  6, }, // Asia/Anadyr
    {  356, 3509,   97463,  6, }, // Asia/Baghdad
    {  356, 5451,   97469,  6, }, // Asia/Bahrain
    {  356, 4368,   97475,  7, }, // Asia/Bangkok
    {  356, 3988,   34483,  6, }, // Asia/Beirut
    {  356, 5906,   97482,  6, }, // Asia/Brunei
    {  356, 5963,   97488,  9, }, // Asia/Choibalsan
    {  356, 4413,   97497,  7, }, // Asia/Colombo
    {  356, 4442,    4102,  5, }, // Asia/Damascus
    {  356, 3498,   20800,  5, }, // Asia/Dubai
    {  356, 8614,   97504,  5, }, // Asia/Gaza
    {  356, 4697,   97509,  6, }, // Asia/Hebron
    {  356, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  356, 2576,   97515,  9, }, // Asia/Hong_Kong
    {  356, 4673,   97524,  4, }, // Asia/Hovd
    {  356, 1257,   97528,  9, }, // Asia/Jerusalem
    {  356, 4141,   97537,  6, }, // Asia/Karachi
    {  356, 1152,    6018,  8, }, // Asia/Kathmandu
    {  356,  992,   97543,  7, }, // Asia/Kolkata
    {  356, 4761,   97550, 12, }, // Asia/Kuala_Lumpur
    {  356, 5464,   97562,  4, }, // Asia/Kuwait
    {  356, 1178,   97566,  5, }, // Asia/Macau
    {  356, 6883,   97571,  6, }, // Asia/Muscat
    {  356, 2446,   97577,  7, }, // Asia/Nicosia
    {  356, 4116,   97584,  9, }, // Asia/Pyongyang
    {  356, 5476,   97593,  5, }, // Asia/Qatar
    {  356, 8671,   97598,  8, }, // Asia/Qostanay
    {  356, 3486,   97606,  5, }, // Asia/Riyadh
    {  356, 3116,   97611,  5, }, // Asia/Seoul
    {  356, 1020,   97616,  6, }, // Asia/Shanghai
    {  356, 3137,   97622,  8, }, // Asia/Singapore
    {  356, 3100,   97630,  6, }, // Asia/Taipei
    {  356, 2687,   97636,  5, }, // Asia/Tokyo
    {  356, 1346,    6188,  9, }, // Asia/Ulaanbaatar
    {  356, 1202,   56146,  7, }, // Asia/Yangon
    {  356, 3649,   97641,  6, }, // Asia/Yerevan
    {  356, 3565,   97647,  6, }, // Atlantic/Azores
    {  356, 5588,   97653,  6, }, // Atlantic/Bermuda
    {  356, 6591,   97659, 17, }, // Atlantic/Canary
    {  356, 3629,   97676,  7, }, // Atlantic/Cape_Verde
    {  356, 1379,   97683,  5, }, // Atlantic/Faroe
    {  356, 8699,   97688,  6, }, // Atlantic/Madeira
    {  356, 7235,   97694, 15, }, // Atlantic/South_Georgia
    {  356, 6784,   97709, 10, }, // Atlantic/St_Helena
    {  356, 1654,   97719,  8, }, // Australia/Adelaide
    {  356, 1619,   61331,  7, }, // Australia/Brisbane
    {  356, 1711,   97727,  7, }, // Australia/Melbourne
    {  356, 1447,   71707,  5, }, // Australia/Sydney
    {  356, 6390,   97734, 10, }, // Europe/Amsterdam
    {  356, 6149,   97744,  6, }, // Europe/Andorra
    {  356, 6577,   32123,  5, }, // Europe/Athens
    {  356, 6285,   97750,  7, }, // Europe/Belgrade
    {  356, 6178,    6415,  7, }, // Europe/Brussels
    {  356, 3830,   21183,  8, }, // Europe/Bucharest
    {  356, 6238,   97757, 10, }, // Europe/Copenhagen
    {  356, 2260,   97767,  6, }, // Europe/Dublin
    {  356, 6256,   97773,  9, }, // Europe/Gibraltar
    {  356, 8767,   97782,  8, }, // Europe/Guernsey
    {  356, 6561,   97790,  8, }, // Europe/Helsinki
    {  356, 8783,   97798, 13, }, // Europe/Isle_of_Man
    {  356, 1097,   97811,  8, }, // Europe/Istanbul
    {  356, 8802,   97819,  6, }, // Europe/Jersey
    {  356, 2419,   97825,  4, }, // Europe/Kyiv
    {  356, 3078,   97829,  6, }, // Europe/Lisbon
    {  356, 2393,   97835,  6, }, // Europe/London
    {  356, 6314,    6510, 10, }, // Europe/Luxembourg
    {  356, 6531,   97841,  8, }, // Europe/Mariehamn
    {  356, 6359,   97849,  6, }, // Europe/Monaco
    {  356, 3443,   97855,  5, }, // Europe/Moscow
    {  356, 6407,   97860,  4, }, // Europe/Oslo
    {  356, 6224,    6545,  4, }, // Europe/Prague
    {  356, 6273,   97864,  3, }, // Europe/Rome
    {  356, 6419,   97867, 10, }, // Europe/San_Marino
    {  356, 6194,   97877,  8, }, // Europe/Sarajevo
    {  356, 6548,   97885,  5, }, // Europe/Sofia
    {  356, 6472,   97890,  8, }, // Europe/Stockholm
    {  356, 8846,   21335,  6, }, // Europe/Tallinn
    {  356, 6135,    6599,  5, }, // Europe/Tirane
    {  356, 6516,    6614,  7, }, // Europe/Vatican
    {  356, 6164,   97898,  6, }, // Europe/Vienna
    {  356, 8878,   97904,  7, }, // Europe/Vilnius
    {  356, 6489,   97911,  5, }, // Europe/Zurich
    {  356, 4980,   97916, 12, }, // Indian/Antananarivo
    {  356, 6910,   97928,  5, }, // Indian/Chagos
    {  356, 5979,   72546,  7, }, // Indian/Christmas
    {  356, 5996,   97933,  4, }, // Indian/Cocos
    {  356, 4931,   97937,  5, }, // Indian/Comoro
    {  356, 6624,   97942,  7, }, // Indian/Kerguelen
    {  356, 7223,   97949,  4, }, // Indian/Mahe
    {  356, 7013,    6662,  6, }, // Indian/Maldives
    {  356, 3971,   97953,  5, }, // Indian/Mauritius
    {  356, 5000,   97958,  5, }, // Indian/Mayotte
    {  356, 7174,   97963,  8, }, // Indian/Reunion
    {  356, 4324,   97971,  4, }, // Pacific/Apia
    {  356,  932,   97975,  6, }, // Pacific/Auckland
    {  356, 2863,   97981,  6, }, // Pacific/Chatham
    {  356, 3022,     474,  5, }, // Pacific/Chuuk
    {  356, 6641,   97987,  9, }, // Pacific/Galapagos
    {  356, 6659,   97996,  7, }, // Pacific/Gambier
    {  356, 2929,    4512,  8, }, // Pacific/Honolulu
    {  356, 2897,    4520,  9, }, // Pacific/Kanton
    {  356, 3953,   34820,  6, }, // Pacific/Marquesas
    {  356, 7095,   98003,  4, }, // Pacific/Niue
    {  356, 4070,   98007,  7, }, // Pacific/Norfolk
    {  356, 2961,     479,  7, }, // Pacific/Pohnpei
    {  357,   14,       0,  6, }, // Africa/Asmara Kako/Latin/Cameroon
    {  357, 4337,       6,  8, }, // Africa/Sao_Tome
    {  357, 4154,      14,  8, }, // America/Asuncion
    {  357,  237,      22,  8, }, // America/Atikokan
    {  357, 7586,      30, 17, }, // America/Bahia_Banderas
    {  357, 3770,      47,  6, }, // America/Cancun
    {  357, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  357, 5640,      66,  7, }, // America/Curacao
    {  357,  481,      73, 13, }, // America/Indiana/Knox
    {  357, 7924,      86, 16, }, // America/Indiana/Marengo
    {  357, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  357, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  357, 8001,     139, 14, }, // America/Indiana/Vevay
    {  357, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  357, 8049,     171, 16, }, // America/Indiana/Winamac
    {  357, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  357, 5849,     207, 22, }, // America/Lower_Princes
    {  357, 8183,     229,  6, }, // America/Merida
    {  357, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  357, 8292,     255, 20, }, // America/North_Dakota/Center
    {  357, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  357,  393,     298,  4, }, // America/Nuuk
    {  357, 6862,     302, 16, }, // America/Scoresbysund
    {  357, 8471,     318, 14, }, // America/St_Barthelemy
    {  357, 2045,     332, 10, }, // America/St_Johns
    {  357, 5780,     342,  9, }, // America/St_Kitts
    {  357, 5797,     351,  9, }, // America/St_Lucia
    {  357,  855,     360, 10, }, // America/St_Thomas
    {  357, 5830,     370, 11, }, // America/St_Vincent
    {  357, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  357, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  357, 1152,     413,  9, }, // Asia/Kathmandu
    {  357,  992,     422,  7, }, // Asia/Kolkata
    {  357, 1178,     429,  5, }, // Asia/Macau
    {  357, 1202,     442,  6, }, // Asia/Yangon
    {  357, 1379,     448,  5, }, // Atlantic/Faroe
    {  357, 6784,     453, 10, }, // Atlantic/St_Helena
    {  357, 2419,     463,  4, }, // Europe/Kyiv
    {  357, 7174,     467,  7, }, // Indian/Reunion
    {  357, 3022,     474,  5, }, // Pacific/Chuuk
    {  357, 2961,     479,  7, }, // Pacific/Pohnpei
    {  358,   14,       0,  6, }, // Africa/Asmara Kalaallisut/Latin/Greenland
    {  358, 4337,       6,  8, }, // Africa/Sao_Tome
    {  358, 4154,      14,  8, }, // America/Asuncion
    {  358,  237,      22,  8, }, // America/Atikokan
    {  358, 7586,      30, 17, }, // America/Bahia_Banderas
    {  358, 3770,      47,  6, }, // America/Cancun
    {  358, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  358, 5640,      66,  7, }, // America/Curacao
    {  358,  481,      73, 13, }, // America/Indiana/Knox
    {  358, 7924,      86, 16, }, // America/Indiana/Marengo
    {  358, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  358, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  358, 8001,     139, 14, }, // America/Indiana/Vevay
    {  358, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  358, 8049,     171, 16, }, // America/Indiana/Winamac
    {  358, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  358, 5849,     207, 22, }, // America/Lower_Princes
    {  358, 8183,     229,  6, }, // America/Merida
    {  358, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  358, 8292,     255, 20, }, // America/North_Dakota/Center
    {  358, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  358,  393,     298,  4, }, // America/Nuuk
    {  358, 6862,     302, 16, }, // America/Scoresbysund
    {  358, 8471,     318, 14, }, // America/St_Barthelemy
    {  358, 2045,     332, 10, }, // America/St_Johns
    {  358, 5780,     342,  9, }, // America/St_Kitts
    {  358, 5797,     351,  9, }, // America/St_Lucia
    {  358,  855,     360, 10, }, // America/St_Thomas
    {  358, 5830,     370, 11, }, // America/St_Vincent
    {  358, 5673,   25229,  7, }, // America/Thule
    {  358, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  358, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  358, 1152,     413,  9, }, // Asia/Kathmandu
    {  358,  992,     422,  7, }, // Asia/Kolkata
    {  358, 1178,     429,  5, }, // Asia/Macau
    {  358, 1202,     442,  6, }, // Asia/Yangon
    {  358, 1379,     448,  5, }, // Atlantic/Faroe
    {  358, 6784,     453, 10, }, // Atlantic/St_Helena
    {  358, 2419,     463,  4, }, // Europe/Kyiv
    {  358, 7174,     467,  7, }, // Indian/Reunion
    {  358, 3022,     474,  5, }, // Pacific/Chuuk
    {  358, 2961,     479,  7, }, // Pacific/Pohnpei
    {  359,   14,       0,  6, }, // Africa/Asmara Kalenjin/Latin/Kenya
    {  359, 4337,       6,  8, }, // Africa/Sao_Tome
    {  359, 4154,      14,  8, }, // America/Asuncion
    {  359,  237,      22,  8, }, // America/Atikokan
    {  359, 7586,      30, 17, }, // America/Bahia_Banderas
    {  359, 3770,      47,  6, }, // America/Cancun
    {  359, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  359, 5640,      66,  7, }, // America/Curacao
    {  359,  481,      73, 13, }, // America/Indiana/Knox
    {  359, 7924,      86, 16, }, // America/Indiana/Marengo
    {  359, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  359, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  359, 8001,     139, 14, }, // America/Indiana/Vevay
    {  359, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  359, 8049,     171, 16, }, // America/Indiana/Winamac
    {  359, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  359, 5849,     207, 22, }, // America/Lower_Princes
    {  359, 8183,     229,  6, }, // America/Merida
    {  359, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  359, 8292,     255, 20, }, // America/North_Dakota/Center
    {  359, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  359,  393,     298,  4, }, // America/Nuuk
    {  359, 6862,     302, 16, }, // America/Scoresbysund
    {  359, 8471,     318, 14, }, // America/St_Barthelemy
    {  359, 2045,     332, 10, }, // America/St_Johns
    {  359, 5780,     342,  9, }, // America/St_Kitts
    {  359, 5797,     351,  9, }, // America/St_Lucia
    {  359,  855,     360, 10, }, // America/St_Thomas
    {  359, 5830,     370, 11, }, // America/St_Vincent
    {  359, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  359, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  359, 1152,     413,  9, }, // Asia/Kathmandu
    {  359,  992,     422,  7, }, // Asia/Kolkata
    {  359, 1178,     429,  5, }, // Asia/Macau
    {  359, 1202,     442,  6, }, // Asia/Yangon
    {  359, 1379,     448,  5, }, // Atlantic/Faroe
    {  359, 6784,     453, 10, }, // Atlantic/St_Helena
    {  359, 2419,     463,  4, }, // Europe/Kyiv
    {  359, 7174,     467,  7, }, // Indian/Reunion
    {  359, 3022,     474,  5, }, // Pacific/Chuuk
    {  359, 2961,     479,  7, }, // Pacific/Pohnpei
    {  360,   14,       0,  6, }, // Africa/Asmara Kamba/Latin/Kenya
    {  360, 4337,       6,  8, }, // Africa/Sao_Tome
    {  360, 4154,      14,  8, }, // America/Asuncion
    {  360,  237,      22,  8, }, // America/Atikokan
    {  360, 7586,      30, 17, }, // America/Bahia_Banderas
    {  360, 3770,      47,  6, }, // America/Cancun
    {  360, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  360, 5640,      66,  7, }, // America/Curacao
    {  360,  481,      73, 13, }, // America/Indiana/Knox
    {  360, 7924,      86, 16, }, // America/Indiana/Marengo
    {  360, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  360, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  360, 8001,     139, 14, }, // America/Indiana/Vevay
    {  360, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  360, 8049,     171, 16, }, // America/Indiana/Winamac
    {  360, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  360, 5849,     207, 22, }, // America/Lower_Princes
    {  360, 8183,     229,  6, }, // America/Merida
    {  360, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  360, 8292,     255, 20, }, // America/North_Dakota/Center
    {  360, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  360,  393,     298,  4, }, // America/Nuuk
    {  360, 6862,     302, 16, }, // America/Scoresbysund
    {  360, 8471,     318, 14, }, // America/St_Barthelemy
    {  360, 2045,     332, 10, }, // America/St_Johns
    {  360, 5780,     342,  9, }, // America/St_Kitts
    {  360, 5797,     351,  9, }, // America/St_Lucia
    {  360,  855,     360, 10, }, // America/St_Thomas
    {  360, 5830,     370, 11, }, // America/St_Vincent
    {  360, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  360, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  360, 1152,     413,  9, }, // Asia/Kathmandu
    {  360,  992,     422,  7, }, // Asia/Kolkata
    {  360, 1178,     429,  5, }, // Asia/Macau
    {  360, 1202,     442,  6, }, // Asia/Yangon
    {  360, 1379,     448,  5, }, // Atlantic/Faroe
    {  360, 6784,     453, 10, }, // Atlantic/St_Helena
    {  360, 2419,     463,  4, }, // Europe/Kyiv
    {  360, 7174,     467,  7, }, // Indian/Reunion
    {  360, 3022,     474,  5, }, // Pacific/Chuuk
    {  360, 2961,     479,  7, }, // Pacific/Pohnpei
    {  361, 6751,   98014,  9, }, // Africa/Abidjan Kannada/Kannada/India
    {  361, 6723,   98023,  5, }, // Africa/Accra
    {  361, 4961,   98028, 11, }, // Africa/Addis_Ababa
    {  361, 7383,   98039,  9, }, // Africa/Algiers
    {  361,   14,   98048,  7, }, // Africa/Asmara
    {  361,   44,   98055,  5, }, // Africa/Bamako
    {  361, 5159,   98060,  5, }, // Africa/Bangui
    {  361, 6709,   98065,  6, }, // Africa/Banjul
    {  361, 7398,   98071,  8, }, // Africa/Bissau
    {  361, 4859,   98079, 11, }, // Africa/Blantyre
    {  361, 5189,   98090, 10, }, // Africa/Brazzaville
    {  361, 4824,   98100,  9, }, // Africa/Bujumbura
    {  361, 2242,   98109,  4, }, // Africa/Cairo
    {  361, 4019,   98113, 13, }, // Africa/Casablanca
    {  361, 7412,   98126,  6, }, // Africa/Ceuta
    {  361, 6736,   98132,  8, }, // Africa/Conakry
    {  361, 6803,   98140,  5, }, // Africa/Dakar
    {  361, 5032,   98145, 14, }, // Africa/Dar_es_Salaam
    {  361, 4945,   98159,  6, }, // Africa/Djibouti
    {  361, 5145,   98165,  4, }, // Africa/Douala
    {  361, 5068,   98169,  7, }, // Africa/El_Aaiun
    {  361, 6816,   98176,  8, }, // Africa/Freetown
    {  361, 4808,   98184,  9, }, // Africa/Gaborone
    {  361, 4917,   98193,  5, }, // Africa/Harare
    {  361, 4381,   98198, 14, }, // Africa/Johannesburg
    {  361, 4401,   98212,  4, }, // Africa/Juba
    {  361, 5053,   98216,  6, }, // Africa/Kampala
    {  361, 4426,   98222,  7, }, // Africa/Khartoum
    {  361, 4875,   98229,  6, }, // Africa/Kigali
    {  361, 5208,   98235,  8, }, // Africa/Kinshasa
    {  361, 4646,   98243,  6, }, // Africa/Lagos
    {  361, 5238,   98249, 10, }, // Africa/Libreville
    {  361, 6832,   98259,  4, }, // Africa/Lome
    {  361, 5113,   98263,  7, }, // Africa/Luanda
    {  361, 4841,   98270,  9, }, // Africa/Lubumbashi
    {  361, 4903,   98279,  6, }, // Africa/Lusaka
    {  361, 5224,   98285,  5, }, // Africa/Malabo
    {  361, 4889,   98290,  8, }, // Africa/Maputo
    {  361, 5099,   98298,  5, }, // Africa/Maseru
    {  361, 5084,   98303,  7, }, // Africa/Mbabane
    {  361, 5015,   98310,  8, }, // Africa/Mogadishu
    {  361, 6997,   98318, 10, }, // Africa/Monrovia
    {  361, 3755,   98328,  6, }, // Africa/Nairobi
    {  361, 5173,   98334,  7, }, // Africa/Ndjamena
    {  361, 5256,   98341,  6, }, // Africa/Niamey
    {  361, 6766,   98347, 11, }, // Africa/Nouakchott
    {  361, 6690,   98358,  7, }, // Africa/Ouagadougou
    {  361, 5127,   98365, 11, }, // Africa/Porto-Novo
    {  361, 4337,   98376,  9, }, // Africa/Sao_Tome
    {  361, 2732,   98385,  8, }, // Africa/Tripoli
    {  361, 6503,   98393,  6, }, // Africa/Tunis
    {  361, 4054,   98399,  9, }, // Africa/Windhoek
    {  361,  132,   98408,  6, }, // America/Adak
    {  361, 3183,   98414,  7, }, // America/Anchorage
    {  361, 5524,   98421, 10, }, // America/Anguilla
    {  361, 5541,   98431,  8, }, // America/Antigua
    {  361, 4456,   98439,  9, }, // America/Araguaina
    {  361,  166,   98448, 13, }, // America/Argentina/Buenos_Aires
    {  361,   91,   98461,  7, }, // America/Argentina/Catamarca
    {  361,  270,   98468,  8, }, // America/Argentina/Cordoba
    {  361,  441,   98476,  4, }, // America/Argentina/Jujuy
    {  361, 7425,   98480,  9, }, // America/Argentina/La_Rioja
    {  361,  565,   98489,  7, }, // America/Argentina/Mendoza
    {  361, 7452,   98496, 14, }, // America/Argentina/Rio_Gallegos
    {  361, 7483,   98510,  6, }, // America/Argentina/Salta
    {  361, 7507,   98516, 13, }, // America/Argentina/San_Juan
    {  361, 5497,   98529, 13, }, // America/Argentina/San_Luis
    {  361, 7534,   98542,  8, }, // America/Argentina/Tucuman
    {  361, 7560,   98550,  6, }, // America/Argentina/Ushuaia
    {  361, 5557,   98556,  5, }, // America/Aruba
    {  361, 4154,   98561, 11, }, // America/Asuncion
    {  361,  237,   98572,  9, }, // America/Atikokan
    {  361, 3581,   98581,  5, }, // America/Bahia
    {  361, 7586,   98586, 15, }, // America/Bahia_Banderas
    {  361, 5571,   98601,  9, }, // America/Barbados
    {  361, 7609,   98610,  5, }, // America/Belem
    {  361, 5309,   98615,  6, }, // America/Belize
    {  361, 7623,   98621, 15, }, // America/Blanc-Sablon
    {  361, 7644,   98636, 10, }, // America/Boa_Vista
    {  361, 4263,   98646,  5, }, // America/Bogota
    {  361, 7662,   98651,  6, }, // America/Boise
    {  361, 7676,   98657, 16, }, // America/Cambridge_Bay
    {  361, 7698,   98673, 13, }, // America/Campo_Grande
    {  361, 3770,   98686,  8, }, // America/Cancun
    {  361, 4596,   98694,  9, }, // America/Caracas
    {  361, 4247,   98703,  6, }, // America/Cayenne
    {  361, 5398,   98709,  5, }, // America/Cayman
    {  361, 3251,   98714,  6, }, // America/Chicago
    {  361, 7719,   98720,  5, }, // America/Chihuahua
    {  361, 7737,   98725, 15, }, // America/Ciudad_Juarez
    {  361, 5324,   98740, 11, }, // America/Costa_Rica
    {  361, 7759,   98751,  9, }, // America/Creston
    {  361, 3693,   98760,  7, }, // America/Cuiaba
    {  361, 5640,   98767,  8, }, // America/Curacao
    {  361, 7775,   98775, 17, }, // America/Danmarkshavn
    {  361, 7796,   98792,  5, }, // America/Dawson
    {  361, 7811,   98797, 12, }, // America/Dawson_Creek
    {  361,  805,   98809,  7, }, // America/Denver
    {  361, 3351,   98816, 10, }, // America/Detroit
    {  361, 5656,   98826,  8, }, // America/Dominica
    {  361,  893,   98834,  9, }, // America/Edmonton
    {  361, 7832,   98843,  7, }, // America/Eirunepe
    {  361, 5343,   98850, 13, }, // America/El_Salvador
    {  361, 7849,   98863, 14, }, // America/Fort_Nelson
    {  361, 7869,   98877,  9, }, // America/Fortaleza
    {  361, 7887,   98886,  9, }, // America/Glace_Bay
    {  361, 6844,   98895,  7, }, // America/Goose_Bay
    {  361, 4514,   98902, 15, }, // America/Grand_Turk
    {  361, 5687,   98917,  7, }, // America/Grenada
    {  361, 5703,   98924, 10, }, // America/Guadeloupe
    {  361, 3662,   98934, 10, }, // America/Guatemala
    {  361, 4743,   98944, 10, }, // America/Guayaquil
    {  361, 6895,   98954,  5, }, // America/Guyana
    {  361, 1923,   98959, 12, }, // America/Halifax
    {  361, 2221,   98971,  5, }, // America/Havana
    {  361, 7905,   98976, 11, }, // America/Hermosillo
    {  361,  348,   98987, 14, }, // America/Indiana/Indianapolis
    {  361,  481,   99001, 16, }, // America/Indiana/Knox
    {  361, 7924,   99017, 16, }, // America/Indiana/Marengo
    {  361, 7948,   99033, 23, }, // America/Indiana/Petersburg
    {  361, 7975,   99056, 19, }, // America/Indiana/Tell_City
    {  361, 8001,   99075, 16, }, // America/Indiana/Vevay
    {  361, 8023,   99091, 23, }, // America/Indiana/Vincennes
    {  361, 8049,   99114, 20, }, // America/Indiana/Winamac
    {  361, 8073,   99134,  7, }, // America/Inuvik
    {  361,  660,   99141,  9, }, // America/Iqaluit
    {  361, 2665,   99150,  5, }, // America/Jamaica
    {  361, 5282,   99155,  6, }, // America/Juneau
    {  361,  521,   99161, 13, }, // America/Kentucky/Louisville
    {  361, 8088,   99174, 20, }, // America/Kentucky/Monticello
    {  361, 5621,   99194, 13, }, // America/Kralendijk
    {  361, 4278,   99207,  7, }, // America/La_Paz
    {  361, 7132,   99214,  4, }, // America/Lima
    {  361, 3390,   99218, 12, }, // America/Los_Angeles
    {  361, 5849,   99230, 25, }, // America/Lower_Princes
    {  361, 8116,   99255,  6, }, // America/Maceio
    {  361, 8131,   99261,  7, }, // America/Managua
    {  361, 1892,   99268,  7, }, // America/Manaus
    {  361, 5814,   99275,  5, }, // America/Marigot
    {  361, 5722,   99280, 10, }, // America/Martinique
    {  361, 8147,   99290,  8, }, // America/Matamoros
    {  361, 2779,   99298,  8, }, // America/Mazatlan
    {  361, 8165,   99306,  8, }, // America/Menominee
    {  361, 8183,   99314,  6, }, // America/Merida
    {  361, 8198,   99320, 12, }, // America/Metlakatla
    {  361, 2811,   99332, 12, }, // America/Mexico_City
    {  361, 4293,   99344,  8, }, // America/Miquelon
    {  361, 8217,   99352,  8, }, // America/Moncton
    {  361, 8233,   99360,  9, }, // America/Monterrey
    {  361, 4000,   99369, 11, }, // America/Montevideo
    {  361, 5741,   99380, 12, }, // America/Montserrat
    {  361, 5383,   99392,  7, }, // America/Nassau
    {  361, 3294,   99399, 10, }, // America/New_York
    {  361, 8251,   99409,  4, }, // America/Nome
    {  361, 1834,   99413,  8, }, // America/Noronha
    {  361, 8264,   99421, 18, }, // America/North_Dakota/Beulah
    {  361, 8292,   99439, 18, }, // America/North_Dakota/Center
    {  361, 8320,   99457, 21, }, // America/North_Dakota/New_Salem
    {  361,  393,   99478,  4, }, // America/Nuuk
    {  361, 8351,   99482,  7, }, // America/Ojinaga
    {  361, 5413,   99489,  5, }, // America/Panama
    {  361, 6106,   99494,  8, }, // America/Paramaribo
    {  361, 3224,   99502,  8, }, // America/Phoenix
    {  361, 3847,   99510, 17, }, // America/Port-au-Prince
    {  361, 5871,   99527, 17, }, // America/Port_of_Spain
    {  361, 8367,   99544, 13, }, // America/Porto_Velho
    {  361, 5760,   99557, 13, }, // America/Puerto_Rico
    {  361, 3932,   99570, 13, }, // America/Punta_Arenas
    {  361, 8387,   99583, 16, }, // America/Rankin_Inlet
    {  361, 8408,   99599,  6, }, // America/Recife
    {  361, 1979,   99605,  6, }, // America/Regina
    {  361, 8423,   99611, 10, }, // America/Resolute
    {  361,  695,   99621, 14, }, // America/Rio_Branco
    {  361, 8440,   99635,  8, }, // America/Santarem
    {  361, 2165,   99643, 11, }, // America/Santiago
    {  361, 6044,   99654, 15, }, // America/Santo_Domingo
    {  361, 1862,   99669,  9, }, // America/Sao_Paulo
    {  361, 6862,   99678, 19, }, // America/Scoresbysund
    {  361, 8457,   99697,  6, }, // America/Sitka
    {  361, 8471,   99703, 16, }, // America/St_Barthelemy
    {  361, 2045,   99719, 12, }, // America/St_Johns
    {  361, 5780,   99731, 12, }, // America/St_Kitts
    {  361, 5797,   99743, 12, }, // America/St_Lucia
    {  361,  855,   99755, 11, }, // America/St_Thomas
    {  361, 5830,   99766, 15, }, // America/St_Vincent
    {  361, 8493,   99781, 15, }, // America/Swift_Current
    {  361, 5363,   99796, 11, }, // America/Tegucigalpa
    {  361, 5673,   99807,  4, }, // America/Thule
    {  361,  313,   99811,  8, }, // America/Tijuana
    {  361,  608,   99819,  7, }, // America/Toronto
    {  361, 5605,   99826,  8, }, // America/Tortola
    {  361, 2077,   99834, 11, }, // America/Vancouver
    {  361, 2128,   99845, 11, }, // America/Whitehorse
    {  361,  734,   99856, 10, }, // America/Winnipeg
    {  361, 8515,   99866,  8, }, // America/Yakutat
    {  361, 5918,   99874,  4, }, // Antarctica/Casey
    {  361, 6027,   99878,  6, }, // Antarctica/Davis
    {  361, 6066,   99884, 17, }, // Antarctica/DumontDUrville
    {  361, 8531,   99901, 10, }, // Antarctica/Macquarie
    {  361, 7029,   99911,  7, }, // Antarctica/Mawson
    {  361, 7076,   99918, 13, }, // Antarctica/McMurdo
    {  361, 8552,   99931,  7, }, // Antarctica/Palmer
    {  361, 7189,   99938,  6, }, // Antarctica/Rothera
    {  361, 7258,   99944,  6, }, // Antarctica/Syowa
    {  361, 8570,   99950,  6, }, // Antarctica/Troll
    {  361, 7337,   99956,  8, }, // Antarctica/Vostok
    {  361, 1413,   99964, 13, }, // Arctic/Longyearbyen
    {  361, 5487,   99977,  4, }, // Asia/Aden
    {  361, 5297,   99981,  7, }, // Asia/Almaty
    {  361, 3870,   99988,  7, }, // Asia/Amman
    {  361, 5428,   99995,  7, }, // Asia/Anadyr
    {  361, 5440,  100002,  7, }, // Asia/Aqtau
    {  361, 5270,  100009,  7, }, // Asia/Aqtobe
    {  361,  964,  100016,  8, }, // Asia/Ashgabat
    {  361, 8587,  100024,  7, }, // Asia/Atyrau
    {  361, 3509,  100031,  8, }, // Asia/Baghdad
    {  361, 5451,  100039,  7, }, // Asia/Bahrain
    {  361, 3555,  100046,  3, }, // Asia/Baku
    {  361, 4368,  100049,  9, }, // Asia/Bangkok
    {  361, 3473,  100058,  8, }, // Asia/Barnaul
    {  361, 3988,  100066,  5, }, // Asia/Beirut
    {  361, 3680,  100071,  8, }, // Asia/Bishkek
    {  361, 5906,  100079,  6, }, // Asia/Brunei
    {  361, 4503,  100085,  3, }, // Asia/Chita
    {  361, 5963,  100088, 12, }, // Asia/Choibalsan
    {  361, 4413,  100100,  6, }, // Asia/Colombo
    {  361, 4442,  100106,  8, }, // Asia/Damascus
    {  361, 1060,  100114,  4, }, // Asia/Dhaka
    {  361, 6125,  100118,  4, }, // Asia/Dili
    {  361, 3498,  100122,  4, }, // Asia/Dubai
    {  361, 6092,  100126,  7, }, // Asia/Dushanbe
    {  361, 8599,  100133,  9, }, // Asia/Famagusta
    {  361, 8614,  100142,  4, }, // Asia/Gaza
    {  361, 4697,  100146,  8, }, // Asia/Hebron
    {  361, 1226,  100154, 15, }, // Asia/Ho_Chi_Minh
    {  361, 2576,  100169, 11, }, // Asia/Hong_Kong
    {  361, 4673,  100180,  6, }, // Asia/Hovd
    {  361, 4086,  100186, 10, }, // Asia/Irkutsk
    {  361, 6969,  100196,  7, }, // Asia/Jakarta
    {  361, 6955,  100203,  5, }, // Asia/Jayapura
    {  361, 1257,  100208,  9, }, // Asia/Jerusalem
    {  361, 3462,  100217,  6, }, // Asia/Kabul
    {  361, 4218,  100223,  7, }, // Asia/Kamchatka
    {  361, 4141,  100230,  5, }, // Asia/Karachi
    {  361, 1152,  100235,  7, }, // Asia/Kathmandu
    {  361, 8624,  100242,  8, }, // Asia/Khandyga
    {  361,  992,  100250,  9, }, // Asia/Kolkata
    {  361, 4099,  100259, 16, }, // Asia/Krasnoyarsk
    {  361, 4761,  100275, 10, }, // Asia/Kuala_Lumpur
    {  361, 5893,  100285,  7, }, // Asia/Kuching
    {  361, 5464,  100292,  6, }, // Asia/Kuwait
    {  361, 1178,  100298,  5, }, // Asia/Macau
    {  361, 3919,  100303,  6, }, // Asia/Magadan
    {  361, 1316,  100309,  9, }, // Asia/Makassar
    {  361, 7145,  100318,  5, }, // Asia/Manila
    {  361, 6883,  100323,  6, }, // Asia/Muscat
    {  361, 2446,  100329,  8, }, // Asia/Nicosia
    {  361, 8638,  100337,  8, }, // Asia/Novokuznetsk
    {  361, 4037,  100345, 12, }, // Asia/Novosibirsk
    {  361, 4131,  100357,  7, }, // Asia/Omsk
    {  361, 7108,  100364,  4, }, // Asia/Oral
    {  361, 6924,  100368,  9, }, // Asia/Phnom_Penh
    {  361, 8656,  100377, 10, }, // Asia/Pontianak
    {  361, 4116,  100387, 13, }, // Asia/Pyongyang
    {  361, 5476,  100400,  5, }, // Asia/Qatar
    {  361, 8671,  100405,  8, }, // Asia/Qostanay
    {  361, 4171,  100413,  9, }, // Asia/Qyzylorda
    {  361, 3486,  100422,  6, }, // Asia/Riyadh
    {  361, 4310,  100428,  7, }, // Asia/Sakhalin
    {  361, 7208,  100435,  6, }, // Asia/Samarkand
    {  361, 3116,  100441,  6, }, // Asia/Seoul
    {  361, 1020,  100447,  5, }, // Asia/Shanghai
    {  361, 3137,  100452,  8, }, // Asia/Singapore
    {  361, 4199,  100460, 16, }, // Asia/Srednekolymsk
    {  361, 3100,  100476,  4, }, // Asia/Taipei
    {  361, 4683,  100480,  9, }, // Asia/Tashkent
    {  361, 3817,  100489,  8, }, // Asia/Tbilisi
    {  361, 2638,  100497,  8, }, // Asia/Tehran
    {  361, 1284,  100505,  5, }, // Asia/Thimphu
    {  361, 2687,  100510,  6, }, // Asia/Tokyo
    {  361, 4474,  100516,  9, }, // Asia/Tomsk
    {  361, 1346,  100525, 11, }, // Asia/Ulaanbaatar
    {  361, 1126,  100536,  8, }, // Asia/Urumqi
    {  361, 8685,  100544, 10, }, // Asia/Ust-Nera
    {  361, 6940,  100554, 11, }, // Asia/Vientiane
    {  361, 4612,  100565, 15, }, // Asia/Vladivostok
    {  361, 4730,  100580,  9, }, // Asia/Yakutsk
    {  361, 1202,  100589,  7, }, // Asia/Yangon
    {  361, 3785,  100596, 16, }, // Asia/Yekaterinburg
    {  361, 3649,  100612,  8, }, // Asia/Yerevan
    {  361, 3565,  100620,  7, }, // Atlantic/Azores
    {  361, 5588,  100627,  7, }, // Atlantic/Bermuda
    {  361, 6591,  100634,  7, }, // Atlantic/Canary
    {  361, 3629,  100641, 11, }, // Atlantic/Cape_Verde
    {  361, 1379,  100652,  6, }, // Atlantic/Faroe
    {  361, 8699,  100658,  6, }, // Atlantic/Madeira
    {  361, 2614,  100664, 10, }, // Atlantic/Reykjavik
    {  361, 7235,  100674, 15, }, // Atlantic/South_Georgia
    {  361, 6784,  100689, 12, }, // Atlantic/St_Helena
    {  361, 6607,  100701, 10, }, // Atlantic/Stanley
    {  361, 1654,  100711,  7, }, // Australia/Adelaide
    {  361, 1619,  100718, 11, }, // Australia/Brisbane
    {  361, 1783,  100729, 12, }, // Australia/Broken_Hill
    {  361, 1567,  100741,  8, }, // Australia/Darwin
    {  361, 3539,  100749,  6, }, // Australia/Eucla
    {  361, 1500,  100755,  9, }, // Australia/Hobart
    {  361, 8716,  100764,  9, }, // Australia/Lindeman
    {  361, 1531,  100773, 11, }, // Australia/Lord_Howe
    {  361, 1711,  100784,  9, }, // Australia/Melbourne
    {  361, 1746,  100793,  5, }, // Australia/Perth
    {  361, 1447,  100798,  6, }, // Australia/Sydney
    {  361, 6390,  100804, 16, }, // Europe/Amsterdam
    {  361, 6149,  100820,  6, }, // Europe/Andorra
    {  361, 3522,  100826, 11, }, // Europe/Astrakhan
    {  361, 6577,  100837,  7, }, // Europe/Athens
    {  361, 6285,  100844, 11, }, // Europe/Belgrade
    {  361, 4659,  100855,  7, }, // Europe/Berlin
    {  361, 6437,  100862, 11, }, // Europe/Bratislava
    {  361, 6178,  100873, 11, }, // Europe/Brussels
    {  361, 3830,  100884, 10, }, // Europe/Bucharest
    {  361, 3708,  100894, 10, }, // Europe/Budapest
    {  361, 8751,  100904,  9, }, // Europe/Busingen
    {  361, 2475,  100913,  8, }, // Europe/Chisinau
    {  361, 6238,  100921, 12, }, // Europe/Copenhagen
    {  361, 2260,  100933,  8, }, // Europe/Dublin
    {  361, 6256,  100941, 11, }, // Europe/Gibraltar
    {  361, 8767,  100952, 11, }, // Europe/Guernsey
    {  361, 6561,  100963,  9, }, // Europe/Helsinki
    {  361, 8783,  100972, 14, }, // Europe/Isle_of_Man
    {  361, 1097,  100986, 12, }, // Europe/Istanbul
    {  361, 8802,  100998,  6, }, // Europe/Jersey
    {  361, 3881,  101004, 14, }, // Europe/Kaliningrad
    {  361, 8924,  101018,  6, }, // Europe/Kirov
    {  361, 2419,  101024,  4, }, // Europe/Kyiv
    {  361, 3078,  101028,  7, }, // Europe/Lisbon
    {  361, 6455,  101035, 12, }, // Europe/Ljubljana
    {  361, 2393,  101047,  5, }, // Europe/London
    {  361, 6314,  101052, 11, }, // Europe/Luxembourg
    {  361, 4794,  101063, 10, }, // Europe/Madrid
    {  361, 6346,  101073,  6, }, // Europe/Malta
    {  361, 6531,  101079,  8, }, // Europe/Mariehamn
    {  361, 3595,  101087,  9, }, // Europe/Minsk
    {  361, 6359,  101096,  6, }, // Europe/Monaco
    {  361, 3443,  101102,  6, }, // Europe/Moscow
    {  361, 6407,  101108,  5, }, // Europe/Oslo
    {  361, 4186,  101113,  8, }, // Europe/Paris
    {  361, 6373,  101121,  9, }, // Europe/Podgorica
    {  361, 6224,  101130,  8, }, // Europe/Prague
    {  361, 8816,  101138,  4, }, // Europe/Riga
    {  361, 6273,  101142,  4, }, // Europe/Rome
    {  361, 4233,  100435,  3, }, // Europe/Samara
    {  361, 6419,  101146, 12, }, // Europe/San_Marino
    {  361, 6194,  101158,  6, }, // Europe/Sarajevo
    {  361, 4353,  101164,  7, }, // Europe/Saratov
    {  361, 8828,  101171, 12, }, // Europe/Simferopol
    {  361, 6332,  101183,  8, }, // Europe/Skopje
    {  361, 6548,  101191,  6, }, // Europe/Sofia
    {  361, 6472,  101197, 11, }, // Europe/Stockholm
    {  361, 8846,  101208,  8, }, // Europe/Tallinn
    {  361, 6135,  101216,  6, }, // Europe/Tirane
    {  361, 8861,  101222, 12, }, // Europe/Ulyanovsk
    {  361, 6301,  101234,  5, }, // Europe/Vaduz
    {  361, 6516,  101239,  9, }, // Europe/Vatican
    {  361, 6164,  101248,  8, }, // Europe/Vienna
    {  361, 8878,  101256,  9, }, // Europe/Vilnius
    {  361, 4629,  101265, 12, }, // Europe/Volgograd
    {  361, 3055,  101277,  6, }, // Europe/Warsaw
    {  361, 6210,  101283,  9, }, // Europe/Zagreb
    {  361, 6489,  101292,  8, }, // Europe/Zurich
    {  361, 4980,  101300, 11, }, // Indian/Antananarivo
    {  361, 6910,  101311,  6, }, // Indian/Chagos
    {  361, 5979,  101317,  9, }, // Indian/Christmas
    {  361, 5996,  101326,  6, }, // Indian/Cocos
    {  361, 4931,  101332,  6, }, // Indian/Comoro
    {  361, 6624,  101338, 12, }, // Indian/Kerguelen
    {  361, 7223,  101350,  4, }, // Indian/Mahe
    {  361, 7013,  101354, 10, }, // Indian/Maldives
    {  361, 3971,  101364,  7, }, // Indian/Mauritius
    {  361, 5000,  101371,  7, }, // Indian/Mayotte
    {  361, 7174,  101378,  9, }, // Indian/Reunion
    {  361, 4324,  101387,  5, }, // Pacific/Apia
    {  361,  932,  101392, 11, }, // Pacific/Auckland
    {  361, 3608,  101403, 13, }, // Pacific/Bougainville
    {  361, 2863,  101416,  5, }, // Pacific/Chatham
    {  361, 3022,  101421,  4, }, // Pacific/Chuuk
    {  361, 2201,  101425,  6, }, // Pacific/Easter
    {  361, 7323,  101431,  5, }, // Pacific/Efate
    {  361, 7290,  101436,  5, }, // Pacific/Fakaofo
    {  361, 3804,  101441,  4, }, // Pacific/Fiji
    {  361, 7306,  101445,  8, }, // Pacific/Funafuti
    {  361, 6641,  101453,  8, }, // Pacific/Galapagos
    {  361, 6659,  101461, 10, }, // Pacific/Gambier
    {  361, 3724,  101471, 13, }, // Pacific/Guadalcanal
    {  361, 5935,  101484,  6, }, // Pacific/Guam
    {  361, 2929,  101490,  7, }, // Pacific/Honolulu
    {  361, 2897,  101497,  8, }, // Pacific/Kanton
    {  361, 3900,  101505,  9, }, // Pacific/Kiritimati
    {  361, 6982,  101514,  6, }, // Pacific/Kosrae
    {  361, 2708,  101520,  9, }, // Pacific/Kwajalein
    {  361, 4779,  101529,  5, }, // Pacific/Majuro
    {  361, 3953,  101534, 11, }, // Pacific/Marquesas
    {  361, 8909,  101545,  7, }, // Pacific/Midway
    {  361, 7047,  101552,  4, }, // Pacific/Nauru
    {  361, 7095,  101556,  4, }, // Pacific/Niue
    {  361, 4070,  101560,  8, }, // Pacific/Norfolk
    {  361, 7061,  101568,  8, }, // Pacific/Noumea
    {  361, 2991,  101576,  9, }, // Pacific/Pago_Pago
    {  361, 7118,  101585,  6, }, // Pacific/Palau
    {  361, 7157,  101591, 11, }, // Pacific/Pitcairn
    {  361, 2961,  101602,  7, }, // Pacific/Pohnpei
    {  361, 4709,  101609, 16, }, // Pacific/Port_Moresby
    {  361, 6009,  101625,  8, }, // Pacific/Rarotonga
    {  361, 5948,  101633,  5, }, // Pacific/Saipan
    {  361, 7275,  101638,  5, }, // Pacific/Tahiti
    {  361, 6675,  101643,  5, }, // Pacific/Tarawa
    {  361, 4485,  101648,  9, }, // Pacific/Tongatapu
    {  361, 7355,  101657,  4, }, // Pacific/Wake
    {  361, 7368,  101661,  8, }, // Pacific/Wallis
    {  362, 6751,  101669,  8, }, // Africa/Abidjan Kashmiri/Arabic/India
    {  362, 6723,  101677,  5, }, // Africa/Accra
    {  362, 4961,  101682, 10, }, // Africa/Addis_Ababa
    {  362, 7383,  101692, 10, }, // Africa/Algiers
    {  362,   14,  101702,  7, }, // Africa/Asmara
    {  362,   44,  101709,  5, }, // Africa/Bamako
    {  362, 5159,  101714,  8, }, // Africa/Bangui
    {  362, 6709,  101722,  7, }, // Africa/Banjul
    {  362, 7398,  101729,  6, }, // Africa/Bissau
    {  362, 4859,  101735,  8, }, // Africa/Blantyre
    {  362, 5189,  101743,  8, }, // Africa/Brazzaville
    {  362, 4824,  101751,  9, }, // Africa/Bujumbura
    {  362, 2242,  101760,  5, }, // Africa/Cairo
    {  362, 4019,  101765, 10, }, // Africa/Casablanca
    {  362, 7412,  101775,  6, }, // Africa/Ceuta
    {  362, 6736,  101781,  7, }, // Africa/Conakry
    {  362, 6803,  101788,  5, }, // Africa/Dakar
    {  362, 5032,  101793,  9, }, // Africa/Dar_es_Salaam
    {  362, 4945,  101802,  8, }, // Africa/Djibouti
    {  362, 5145,  101810,  5, }, // Africa/Douala
    {  362, 5068,  101815,  7, }, // Africa/El_Aaiun
    {  362, 6816,  101822,  9, }, // Africa/Freetown
    {  362, 4808,  101831,  7, }, // Africa/Gaborone
    {  362, 4917,  101838,  7, }, // Africa/Harare
    {  362, 4381,  101845,  9, }, // Africa/Johannesburg
    {  362, 4401,    9030,  4, }, // Africa/Juba
    {  362, 5053,  101854,  7, }, // Africa/Kampala
    {  362, 4426,  101861,  7, }, // Africa/Khartoum
    {  362, 4875,  101868,  6, }, // Africa/Kigali
    {  362, 5208,  101874,  7, }, // Africa/Kinshasa
    {  362, 4646,  101881,  5, }, // Africa/Lagos
    {  362, 5238,  101886,  8, }, // Africa/Libreville
    {  362, 6832,    9073,  3, }, // Africa/Lome
    {  362, 5113,  101894,  7, }, // Africa/Luanda
    {  362, 4841,  101901,  9, }, // Africa/Lubumbashi
    {  362, 4903,  101910,  6, }, // Africa/Lusaka
    {  362, 5224,    9097,  6, }, // Africa/Malabo
    {  362, 4889,  101916,  7, }, // Africa/Maputo
    {  362, 5099,  101923,  7, }, // Africa/Maseru
    {  362, 5084,  101930,  5, }, // Africa/Mbabane
    {  362, 5015,  101935,  9, }, // Africa/Mogadishu
    {  362, 6997,  101944,  9, }, // Africa/Monrovia
    {  362, 3755,  101953,  6, }, // Africa/Nairobi
    {  362, 5173,  101959,  7, }, // Africa/Ndjamena
    {  362, 5256,  101966,  6, }, // Africa/Niamey
    {  362, 6766,  101972,  9, }, // Africa/Nouakchott
    {  362, 6690,  101981,  8, }, // Africa/Ouagadougou
    {  362, 5127,  101989, 10, }, // Africa/Porto-Novo
    {  362, 4337,  101999,  7, }, // Africa/Sao_Tome
    {  362, 2732,  102006,  6, }, // Africa/Tripoli
    {  362, 6503,  102012,  6, }, // Africa/Tunis
    {  362, 4054,  102018,  7, }, // Africa/Windhoek
    {  362,  132,  102025,  3, }, // America/Adak
    {  362, 3183,  102028,  8, }, // America/Anchorage
    {  362, 5524,  102036,  8, }, // America/Anguilla
    {  362, 5541,  102044,  9, }, // America/Antigua
    {  362, 4456,  102053, 10, }, // America/Araguaina
    {  362,  166,  102063, 13, }, // America/Argentina/Buenos_Aires
    {  362,   91,  102076,  9, }, // America/Argentina/Catamarca
    {  362,  270,  102085,  7, }, // America/Argentina/Cordoba
    {  362,  441,  102092,  6, }, // America/Argentina/Jujuy
    {  362, 7425,  102098,  9, }, // America/Argentina/La_Rioja
    {  362,  565,  102107,  7, }, // America/Argentina/Mendoza
    {  362, 7452,  102114, 12, }, // America/Argentina/Rio_Gallegos
    {  362, 7483,  102126,  5, }, // America/Argentina/Salta
    {  362, 7507,  102131,  9, }, // America/Argentina/San_Juan
    {  362, 5497,  102140, 10, }, // America/Argentina/San_Luis
    {  362, 7534,  102150,  8, }, // America/Argentina/Tucuman
    {  362, 7560,  102158, 10, }, // America/Argentina/Ushuaia
    {  362, 5557,  102168,  7, }, // America/Aruba
    {  362, 4154,  102175, 10, }, // America/Asuncion
    {  362,  237,  102185,  9, }, // America/Atikokan
    {  362, 3581,  102194,  6, }, // America/Bahia
    {  362, 7586,  102200, 12, }, // America/Bahia_Banderas
    {  362, 5571,  102212,  8, }, // America/Barbados
    {  362, 7609,  102220,  5, }, // America/Belem
    {  362, 5309,  102225,  6, }, // America/Belize
    {  362, 7623,  102231, 12, }, // America/Blanc-Sablon
    {  362, 7644,  102243,  9, }, // America/Boa_Vista
    {  362, 4263,  102252,  6, }, // America/Bogota
    {  362, 7662,  102258,  5, }, // America/Boise
    {  362, 7676,  102263, 13, }, // America/Cambridge_Bay
    {  362, 7698,  102276, 11, }, // America/Campo_Grande
    {  362, 3770,  102287,  6, }, // America/Cancun
    {  362, 4596,  102293,  7, }, // America/Caracas
    {  362, 4247,  102300,  5, }, // America/Cayenne
    {  362, 5398,  102305,  5, }, // America/Cayman
    {  362, 3251,  102310,  6, }, // America/Chicago
    {  362, 7719,  102316, 11, }, // America/Chihuahua
    {  362, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  362, 5324,  102327, 11, }, // America/Costa_Rica
    {  362, 7759,  102338,  6, }, // America/Creston
    {  362, 3693,  102344,  7, }, // America/Cuiaba
    {  362, 5640,  102351, 10, }, // America/Curacao
    {  362, 7775,  102361, 11, }, // America/Danmarkshavn
    {  362, 7796,  102372,  5, }, // America/Dawson
    {  362, 7811,  102377, 12, }, // America/Dawson_Creek
    {  362,  805,  102389,  6, }, // America/Denver
    {  362, 3351,  102395,  8, }, // America/Detroit
    {  362, 5656,  102403,  8, }, // America/Dominica
    {  362,  893,  102411,  9, }, // America/Edmonton
    {  362, 7832,  102420,  9, }, // America/Eirunepe
    {  362, 5343,  102429, 12, }, // America/El_Salvador
    {  362, 7849,  102441, 10, }, // America/Fort_Nelson
    {  362, 7869,  102451,  9, }, // America/Fortaleza
    {  362, 7887,  102460, 10, }, // America/Glace_Bay
    {  362, 6844,  102470, 10, }, // America/Goose_Bay
    {  362, 4514,  102480, 10, }, // America/Grand_Turk
    {  362, 5687,  102490,  7, }, // America/Grenada
    {  362, 5703,  102497,  6, }, // America/Guadeloupe
    {  362, 3662,  102503,  9, }, // America/Guatemala
    {  362, 4743,  102512, 10, }, // America/Guayaquil
    {  362, 6895,  102522,  6, }, // America/Guyana
    {  362, 1923,  102528,  8, }, // America/Halifax
    {  362, 2221,  102536,  5, }, // America/Havana
    {  362, 7905,  102541,  8, }, // America/Hermosillo
    {  362,  348,  102549, 13, }, // America/Indiana/Indianapolis
    {  362,  481,  102562,  4, }, // America/Indiana/Knox
    {  362, 7924,  102566, 15, }, // America/Indiana/Marengo
    {  362, 7948,  102581, 18, }, // America/Indiana/Petersburg
    {  362, 7975,  102599, 17, }, // America/Indiana/Tell_City
    {  362, 8001,  102616, 14, }, // America/Indiana/Vevay
    {  362, 8023,  102630, 17, }, // America/Indiana/Vincennes
    {  362, 8049,  102647, 16, }, // America/Indiana/Winamac
    {  362, 8073,  102663,  8, }, // America/Inuvik
    {  362,  660,  102671, 11, }, // America/Iqaluit
    {  362, 2665,  102682,  7, }, // America/Jamaica
    {  362, 5282,  102689,  5, }, // America/Juneau
    {  362,  521,  102694, 10, }, // America/Kentucky/Louisville
    {  362, 8088,  102704, 18, }, // America/Kentucky/Monticello
    {  362, 5621,  102722,  9, }, // America/Kralendijk
    {  362, 4278,  102731,  6, }, // America/La_Paz
    {  362, 7132,  102737,  4, }, // America/Lima
    {  362, 3390,  102741, 11, }, // America/Los_Angeles
    {  362, 5849,  102752, 16, }, // America/Lower_Princes
    {  362, 8116,  102768,  7, }, // America/Maceio
    {  362, 8131,  102775,  7, }, // America/Managua
    {  362, 1892,  102782,  5, }, // America/Manaus
    {  362, 5814,  102787,  7, }, // America/Marigot
    {  362, 5722,  102794,  8, }, // America/Martinique
    {  362, 8147,  102802,  9, }, // America/Matamoros
    {  362, 2779,  102811,  8, }, // America/Mazatlan
    {  362, 8165,  102819,  8, }, // America/Menominee
    {  362, 8183,  102827,  6, }, // America/Merida
    {  362, 8198,  102833, 11, }, // America/Metlakatla
    {  362, 2811,  102844, 12, }, // America/Mexico_City
    {  362, 4293,  102856,  8, }, // America/Miquelon
    {  362, 8217,  102864,  7, }, // America/Moncton
    {  362, 8233,  102871,  6, }, // America/Monterrey
    {  362, 4000,  102877, 10, }, // America/Montevideo
    {  362, 5741,  102887,  8, }, // America/Montserrat
    {  362, 5383,  102895,  5, }, // America/Nassau
    {  362, 3294,  102900,  8, }, // America/New_York
    {  362, 8251,    9897,  3, }, // America/Nome
    {  362, 1834,  102908,  7, }, // America/Noronha
    {  362, 8264,  102915, 21, }, // America/North_Dakota/Beulah
    {  362, 8292,  102936, 19, }, // America/North_Dakota/Center
    {  362, 8320,  102955, 24, }, // America/North_Dakota/New_Salem
    {  362,  393,  102979,  4, }, // America/Nuuk
    {  362, 8351,  102983,  9, }, // America/Ojinaga
    {  362, 5413,  102992,  6, }, // America/Panama
    {  362, 6106,  102998, 10, }, // America/Paramaribo
    {  362, 3224,  103008,  7, }, // America/Phoenix
    {  362, 3847,  103015, 13, }, // America/Port-au-Prince
    {  362, 5871,  103028, 11, }, // America/Port_of_Spain
    {  362, 8367,  103039, 10, }, // America/Porto_Velho
    {  362, 5760,  103049, 10, }, // America/Puerto_Rico
    {  362, 3932,  103059, 12, }, // America/Punta_Arenas
    {  362, 8387,  103071, 13, }, // America/Rankin_Inlet
    {  362, 8408,  103084,  5, }, // America/Recife
    {  362, 1979,  103089,  7, }, // America/Regina
    {  362, 8423,  103096,  9, }, // America/Resolute
    {  362,  695,  103105, 11, }, // America/Rio_Branco
    {  362, 8440,  103116,  8, }, // America/Santarem
    {  362, 2165,  103124,  8, }, // America/Santiago
    {  362, 6044,  103132, 13, }, // America/Santo_Domingo
    {  362, 1862,  103145,  8, }, // America/Sao_Paulo
    {  362, 6862,  103153, 13, }, // America/Scoresbysund
    {  362, 8457,  103166,  5, }, // America/Sitka
    {  362, 8471,  103171, 14, }, // America/St_Barthelemy
    {  362, 2045,  103185,  9, }, // America/St_Johns
    {  362, 5780,  103194,  9, }, // America/St_Kitts
    {  362, 5797,  103203, 12, }, // America/St_Lucia
    {  362,  855,  103215, 11, }, // America/St_Thomas
    {  362, 5830,  103226, 12, }, // America/St_Vincent
    {  362, 8493,  103238, 13, }, // America/Swift_Current
    {  362, 5363,  103251, 12, }, // America/Tegucigalpa
    {  362, 5673,  103263,  7, }, // America/Thule
    {  362,  313,  103270,  8, }, // America/Tijuana
    {  362,  608,  103278,  7, }, // America/Toronto
    {  362, 5605,  103285,  7, }, // America/Tortola
    {  362, 2077,  103292,  8, }, // America/Vancouver
    {  362, 2128,  103300, 11, }, // America/Whitehorse
    {  362,  734,  103311,  7, }, // America/Winnipeg
    {  362, 8515,  103318,  7, }, // America/Yakutat
    {  362, 5918,  103325,  4, }, // Antarctica/Casey
    {  362, 6027,  103329,  5, }, // Antarctica/Davis
    {  362, 6066,  103334, 15, }, // Antarctica/DumontDUrville
    {  362, 7029,  103349,  5, }, // Antarctica/Mawson
    {  362, 7076,  103354,  8, }, // Antarctica/McMurdo
    {  362, 8552,  103362,  5, }, // Antarctica/Palmer
    {  362, 7189,  103367,  7, }, // Antarctica/Rothera
    {  362, 7258,  103374,  5, }, // Antarctica/Syowa
    {  362, 7337,  103379,  6, }, // Antarctica/Vostok
    {  362, 1413,  103385, 14, }, // Arctic/Longyearbyen
    {  362, 5487,  103399,  5, }, // Asia/Aden
    {  362, 5297,  103404,  7, }, // Asia/Almaty
    {  362, 3870,  103411,  5, }, // Asia/Amman
    {  362, 5428,  103416,  7, }, // Asia/Anadyr
    {  362, 5440,  103423,  4, }, // Asia/Aqtau
    {  362, 5270,  103427,  6, }, // Asia/Aqtobe
    {  362,  964,  103433,  8, }, // Asia/Ashgabat
    {  362, 8587,  103441,  5, }, // Asia/Atyrau
    {  362, 3509,   10466,  5, }, // Asia/Baghdad
    {  362, 5451,  103446,  7, }, // Asia/Bahrain
    {  362, 3555,  103453,  5, }, // Asia/Baku
    {  362, 4368,  103458,  8, }, // Asia/Bangkok
    {  362, 3473,  103466,  5, }, // Asia/Barnaul
    {  362, 3988,  103471,  4, }, // Asia/Beirut
    {  362, 3680,  103475,  6, }, // Asia/Bishkek
    {  362, 5906,  103481,  7, }, // Asia/Brunei
    {  362, 4503,  103488,  4, }, // Asia/Chita
    {  362, 5963,  103492,  9, }, // Asia/Choibalsan
    {  362, 4413,  103501,  7, }, // Asia/Colombo
    {  362, 4442,  103508,  8, }, // Asia/Damascus
    {  362, 1060,  103516,  5, }, // Asia/Dhaka
    {  362, 6125,  103521,  4, }, // Asia/Dili
    {  362, 3498,  103525,  5, }, // Asia/Dubai
    {  362, 6092,  103530,  8, }, // Asia/Dushanbe
    {  362, 8599,  103538, 10, }, // Asia/Famagusta
    {  362, 8614,  103548,  3, }, // Asia/Gaza
    {  362, 4697,  103551,  6, }, // Asia/Hebron
    {  362, 1226,  103557,  5, }, // Asia/Ho_Chi_Minh
    {  362, 2576,  103562,  9, }, // Asia/Hong_Kong
    {  362, 4673,  103571,  4, }, // Asia/Hovd
    {  362, 4086,  103575,  7, }, // Asia/Irkutsk
    {  362, 6969,  103582,  6, }, // Asia/Jakarta
    {  362, 6955,  103588,  9, }, // Asia/Jayapura
    {  362, 1257,  103597,  6, }, // Asia/Jerusalem
    {  362, 3462,  103603,  5, }, // Asia/Kabul
    {  362, 4218,  103608,  8, }, // Asia/Kamchatka
    {  362, 4141,  103616,  6, }, // Asia/Karachi
    {  362, 1152,  103622, 10, }, // Asia/Kathmandu
    {  362, 8624,  103632,  8, }, // Asia/Khandyga
    {  362,  992,  103640,  8, }, // Asia/Kolkata
    {  362, 4099,  103648, 11, }, // Asia/Krasnoyarsk
    {  362, 4761,  103659, 11, }, // Asia/Kuala_Lumpur
    {  362, 5893,  103670,  6, }, // Asia/Kuching
    {  362, 5464,  103676,  5, }, // Asia/Kuwait
    {  362, 1178,  103681,  3, }, // Asia/Macau
    {  362, 3919,  103684,  7, }, // Asia/Magadan
    {  362, 1316,  103691,  7, }, // Asia/Makassar
    {  362, 7145,  103698,  7, }, // Asia/Manila
    {  362, 6883,   10734,  4, }, // Asia/Muscat
    {  362, 2446,  103705,  8, }, // Asia/Nicosia
    {  362, 8638,  103713, 12, }, // Asia/Novokuznetsk
    {  362, 4037,  103725, 11, }, // Asia/Novosibirsk
    {  362, 4131,  103736,  5, }, // Asia/Omsk
    {  362, 7108,  103741,  5, }, // Asia/Oral
    {  362, 6924,  103746,  9, }, // Asia/Phnom_Penh
    {  362, 8656,  103755,  9, }, // Asia/Pontianak
    {  362, 4116,  103764,  9, }, // Asia/Pyongyang
    {  362, 5476,   10802,  3, }, // Asia/Qatar
    {  362, 8671,  103773,  7, }, // Asia/Qostanay
    {  362, 4171,  103780,  8, }, // Asia/Qyzylorda
    {  362, 3486,  103788,  4, }, // Asia/Riyadh
    {  362, 4310,  103792,  8, }, // Asia/Sakhalin
    {  362, 7208,  103800,  9, }, // Asia/Samarkand
    {  362, 3116,  103809,  4, }, // Asia/Seoul
    {  362, 1020,  103813,  5, }, // Asia/Shanghai
    {  362, 3137,  103818,  8, }, // Asia/Singapore
    {  362, 4199,  103826, 13, }, // Asia/Srednekolymsk
    {  362, 3100,  103839,  6, }, // Asia/Taipei
    {  362, 4683,  103845,  7, }, // Asia/Tashkent
    {  362, 3817,  103852,  6, }, // Asia/Tbilisi
    {  362, 2638,  103858,  6, }, // Asia/Tehran
    {  362, 1284,  103864,  7, }, // Asia/Thimphu
    {  362, 2687,  103871,  5, }, // Asia/Tokyo
    {  362, 4474,  103876,  5, }, // Asia/Tomsk
    {  362, 1346,  103881,  9, }, // Asia/Ulaanbaatar
    {  362, 1126,  103890,  7, }, // Asia/Urumqi
    {  362, 8685,  103897,  9, }, // Asia/Ust-Nera
    {  362, 6940,  103906, 10, }, // Asia/Vientiane
    {  362, 4612,  103916, 10, }, // Asia/Vladivostok
    {  362, 4730,  103926,  5, }, // Asia/Yakutsk
    {  362, 1202,  103931,  6, }, // Asia/Yangon
    {  362, 3785,  103937, 13, }, // Asia/Yekaterinburg
    {  362, 3649,  103950,  7, }, // Asia/Yerevan
    {  362, 3565,  103957,  7, }, // Atlantic/Azores
    {  362, 5588,  103964,  7, }, // Atlantic/Bermuda
    {  362, 6591,  103971,  6, }, // Atlantic/Canary
    {  362, 3629,  103977,  8, }, // Atlantic/Cape_Verde
    {  362, 1379,  103985,  4, }, // Atlantic/Faroe
    {  362, 8699,  103989,  7, }, // Atlantic/Madeira
    {  362, 2614,  103996,  8, }, // Atlantic/Reykjavik
    {  362, 7235,  104004, 12, }, // Atlantic/South_Georgia
    {  362, 6784,  104016, 11, }, // Atlantic/St_Helena
    {  362, 6607,  104027,  6, }, // Atlantic/Stanley
    {  362, 1654,  104033,  7, }, // Australia/Adelaide
    {  362, 1619,  104040,  6, }, // Australia/Brisbane
    {  362, 1783,  104046, 10, }, // Australia/Broken_Hill
    {  362, 1567,  104056,  6, }, // Australia/Darwin
    {  362, 3539,  104062,  6, }, // Australia/Eucla
    {  362, 1500,  104068,  5, }, // Australia/Hobart
    {  362, 8716,  104073,  8, }, // Australia/Lindeman
    {  362, 1531,  104081,  9, }, // Australia/Lord_Howe
    {  362, 1711,  104090,  8, }, // Australia/Melbourne
    {  362, 1746,  104098,  5, }, // Australia/Perth
    {  362, 1447,  104103,  5, }, // Australia/Sydney
    {  362, 6390,  104108, 10, }, // Europe/Amsterdam
    {  362, 6149,  104118,  7, }, // Europe/Andorra
    {  362, 3522,  104125, 11, }, // Europe/Astrakhan
    {  362, 6577,  104136,  7, }, // Europe/Athens
    {  362, 6285,  104143,  7, }, // Europe/Belgrade
    {  362, 4659,  104150,  6, }, // Europe/Berlin
    {  362, 6437,  104156, 10, }, // Europe/Bratislava
    {  362, 6178,  104166,  6, }, // Europe/Brussels
    {  362, 3830,  104172,  8, }, // Europe/Bucharest
    {  362, 3708,  104180,  8, }, // Europe/Budapest
    {  362, 8751,  104188,  8, }, // Europe/Busingen
    {  362, 2475,  104196,  7, }, // Europe/Chisinau
    {  362, 6238,  104203,  9, }, // Europe/Copenhagen
    {  362, 2260,  104212,  6, }, // Europe/Dublin
    {  362, 6256,  104218,  9, }, // Europe/Gibraltar
    {  362, 8767,  104227,  7, }, // Europe/Guernsey
    {  362, 6561,  104234,  8, }, // Europe/Helsinki
    {  362, 8783,  104242, 10, }, // Europe/Isle_of_Man
    {  362, 1097,  104252,  9, }, // Europe/Istanbul
    {  362, 8802,  104261,  4, }, // Europe/Jersey
    {  362, 3881,  104265, 12, }, // Europe/Kaliningrad
    {  362, 8924,  104277,  4, }, // Europe/Kirov
    {  362, 2419,  104281,  4, }, // Europe/Kyiv
    {  362, 3078,  104285,  6, }, // Europe/Lisbon
    {  362, 6455,  104291, 13, }, // Europe/Ljubljana
    {  362, 2393,  104304,  6, }, // Europe/London
    {  362, 6314,  104310, 10, }, // Europe/Luxembourg
    {  362, 4794,  104320,  6, }, // Europe/Madrid
    {  362, 6346,  104326,  5, }, // Europe/Malta
    {  362, 6531,  104331,  8, }, // Europe/Mariehamn
    {  362, 3595,  104339,  5, }, // Europe/Minsk
    {  362, 6359,  104344,  6, }, // Europe/Monaco
    {  362, 3443,  104350,  5, }, // Europe/Moscow
    {  362, 6407,  104355,  5, }, // Europe/Oslo
    {  362, 4186,  104360,  5, }, // Europe/Paris
    {  362, 6373,  104365,  9, }, // Europe/Podgorica
    {  362, 6224,  104374,  4, }, // Europe/Prague
    {  362, 8816,  104378,  4, }, // Europe/Riga
    {  362, 6273,    9113,  3, }, // Europe/Rome
    {  362, 4233,  104382,  6, }, // Europe/Samara
    {  362, 6419,  104388,  9, }, // Europe/San_Marino
    {  362, 6194,  104397,  9, }, // Europe/Sarajevo
    {  362, 4353,  104406,  6, }, // Europe/Saratov
    {  362, 8828,  104412, 10, }, // Europe/Simferopol
    {  362, 6332,  104422,  7, }, // Europe/Skopje
    {  362, 6548,  104429,  6, }, // Europe/Sofia
    {  362, 6472,  104435,  9, }, // Europe/Stockholm
    {  362, 8846,  104444,  5, }, // Europe/Tallinn
    {  362, 6135,  104449,  5, }, // Europe/Tirane
    {  362, 8861,  104454, 10, }, // Europe/Ulyanovsk
    {  362, 6301,  104464,  5, }, // Europe/Vaduz
    {  362, 6516,  104469,  6, }, // Europe/Vatican
    {  362, 6164,  104475,  7, }, // Europe/Vienna
    {  362, 8878,  104482,  9, }, // Europe/Vilnius
    {  362, 4629,  104491,  9, }, // Europe/Volgograd
    {  362, 3055,  104500,  5, }, // Europe/Warsaw
    {  362, 6210,  104505,  5, }, // Europe/Zagreb
    {  362, 6489,  104510,  7, }, // Europe/Zurich
    {  362, 4980,  104517, 11, }, // Indian/Antananarivo
    {  362, 6910,  104528,  5, }, // Indian/Chagos
    {  362, 5979,  104533,  7, }, // Indian/Christmas
    {  362, 5996,  104540,  5, }, // Indian/Cocos
    {  362, 4931,  104545,  6, }, // Indian/Comoro
    {  362, 6624,  104551,  8, }, // Indian/Kerguelen
    {  362, 7223,  104559,  5, }, // Indian/Mahe
    {  362, 7013,  104564,  8, }, // Indian/Maldives
    {  362, 3971,  104572,  7, }, // Indian/Mauritius
    {  362, 5000,  104579,  4, }, // Indian/Mayotte
    {  362, 7174,  104583,  9, }, // Indian/Reunion
    {  362, 4324,  104592,  5, }, // Pacific/Apia
    {  362,  932,  104597,  6, }, // Pacific/Auckland
    {  362, 2863,  104603,  6, }, // Pacific/Chatham
    {  362, 3022,  104609,  3, }, // Pacific/Chuuk
    {  362, 2201,  104612,  7, }, // Pacific/Easter
    {  362, 7323,  104619,  7, }, // Pacific/Efate
    {  362, 7290,  104626,  7, }, // Pacific/Fakaofo
    {  362, 3804,  104633,  4, }, // Pacific/Fiji
    {  362, 7306,  104637,  9, }, // Pacific/Funafuti
    {  362, 6641,  104646,  8, }, // Pacific/Galapagos
    {  362, 6659,  104654,  8, }, // Pacific/Gambier
    {  362, 3724,  104662, 12, }, // Pacific/Guadalcanal
    {  362, 5935,  102779,  4, }, // Pacific/Guam
    {  362, 2929,  104674,  9, }, // Pacific/Honolulu
    {  362, 2897,  104683,  9, }, // Pacific/Kanton
    {  362, 3900,  104692, 10, }, // Pacific/Kiritimati
    {  362, 6982,  104702,  6, }, // Pacific/Kosrae
    {  362, 2708,  104708,  7, }, // Pacific/Kwajalein
    {  362, 4779,  104715,  7, }, // Pacific/Majuro
    {  362, 3953,  104722,  8, }, // Pacific/Marquesas
    {  362, 8909,  104730,  6, }, // Pacific/Midway
    {  362, 7047,  104736,  6, }, // Pacific/Nauru
    {  362, 7095,  102900,  3, }, // Pacific/Niue
    {  362, 4070,  104742,  6, }, // Pacific/Norfolk
    {  362, 7061,  104748,  5, }, // Pacific/Noumea
    {  362, 2991,  104753,  9, }, // Pacific/Pago_Pago
    {  362, 7118,  104762,  5, }, // Pacific/Palau
    {  362, 7157,  104767,  8, }, // Pacific/Pitcairn
    {  362, 2961,  104775,  6, }, // Pacific/Pohnpei
    {  362, 4709,  104781, 10, }, // Pacific/Port_Moresby
    {  362, 6009,  104791,  9, }, // Pacific/Rarotonga
    {  362, 5948,  104800,  6, }, // Pacific/Saipan
    {  362, 7275,  104806,  6, }, // Pacific/Tahiti
    {  362, 6675,  104812,  6, }, // Pacific/Tarawa
    {  362, 4485,  104818, 10, }, // Pacific/Tongatapu
    {  362, 7355,  104828,  3, }, // Pacific/Wake
    {  362, 7368,  104831,  5, }, // Pacific/Wallis
    {  363,   14,       0,  6, }, // Africa/Asmara Kashmiri/Devanagari/India
    {  363, 4337,       6,  8, }, // Africa/Sao_Tome
    {  363, 4154,      14,  8, }, // America/Asuncion
    {  363,  237,      22,  8, }, // America/Atikokan
    {  363, 7586,      30, 17, }, // America/Bahia_Banderas
    {  363, 3770,      47,  6, }, // America/Cancun
    {  363, 5640,      66,  7, }, // America/Curacao
    {  363,  481,      73, 13, }, // America/Indiana/Knox
    {  363, 7924,      86, 16, }, // America/Indiana/Marengo
    {  363, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  363, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  363, 8001,     139, 14, }, // America/Indiana/Vevay
    {  363, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  363, 8049,     171, 16, }, // America/Indiana/Winamac
    {  363, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  363, 5849,     207, 22, }, // America/Lower_Princes
    {  363, 8183,     229,  6, }, // America/Merida
    {  363, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  363, 8292,     255, 20, }, // America/North_Dakota/Center
    {  363, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  363,  393,     298,  4, }, // America/Nuuk
    {  363, 6862,     302, 16, }, // America/Scoresbysund
    {  363, 8471,     318, 14, }, // America/St_Barthelemy
    {  363, 2045,     332, 10, }, // America/St_Johns
    {  363, 5780,     342,  9, }, // America/St_Kitts
    {  363, 5797,     351,  9, }, // America/St_Lucia
    {  363,  855,     360, 10, }, // America/St_Thomas
    {  363, 5830,     370, 11, }, // America/St_Vincent
    {  363, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  363, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  363, 1152,     413,  9, }, // Asia/Kathmandu
    {  363,  992,     422,  7, }, // Asia/Kolkata
    {  363, 1178,     429,  5, }, // Asia/Macau
    {  363, 1202,     442,  6, }, // Asia/Yangon
    {  363, 1379,     448,  5, }, // Atlantic/Faroe
    {  363, 6784,     453, 10, }, // Atlantic/St_Helena
    {  363, 2419,     463,  4, }, // Europe/Kyiv
    {  363, 7174,     467,  7, }, // Indian/Reunion
    {  363, 3022,     474,  5, }, // Pacific/Chuuk
    {  363, 2961,     479,  7, }, // Pacific/Pohnpei
    {  364, 6751,   34826,  7, }, // Africa/Abidjan Kazakh/Cyrillic/Kazakhstan
    {  364, 6723,     492,  5, }, // Africa/Accra
    {  364, 4961,     497, 11, }, // Africa/Addis_Ababa
    {  364, 7383,     508,  5, }, // Africa/Algiers
    {  364,   14,   25647,  6, }, // Africa/Asmara
    {  364,   44,     519,  6, }, // Africa/Bamako
    {  364, 5159,     525,  5, }, // Africa/Bangui
    {  364, 6709,     530,  6, }, // Africa/Banjul
    {  364, 7398,     536,  5, }, // Africa/Bissau
    {  364, 4859,   25670,  8, }, // Africa/Blantyre
    {  364, 5189,   50080, 10, }, // Africa/Brazzaville
    {  364, 4824,     558,  9, }, // Africa/Bujumbura
    {  364, 2242,     567,  4, }, // Africa/Cairo
    {  364, 4019,     571, 10, }, // Africa/Casablanca
    {  364, 7412,     581,  5, }, // Africa/Ceuta
    {  364, 6736,     586,  7, }, // Africa/Conakry
    {  364, 6803,     593,  5, }, // Africa/Dakar
    {  364, 5032,   25698, 12, }, // Africa/Dar_es_Salaam
    {  364, 4945,   34859,  7, }, // Africa/Djibouti
    {  364, 5145,     616,  5, }, // Africa/Douala
    {  364, 5068,   25717,  7, }, // Africa/El_Aaiun
    {  364, 6816,     628,  7, }, // Africa/Freetown
    {  364, 4808,     635,  8, }, // Africa/Gaborone
    {  364, 4917,   32384,  6, }, // Africa/Harare
    {  364, 4381,   50098, 12, }, // Africa/Johannesburg
    {  364, 4401,   25755,  5, }, // Africa/Juba
    {  364, 5053,     665,  7, }, // Africa/Kampala
    {  364, 4426,     672,  6, }, // Africa/Khartoum
    {  364, 4875,     678,  6, }, // Africa/Kigali
    {  364, 5208,     684,  7, }, // Africa/Kinshasa
    {  364, 4646,     691,  5, }, // Africa/Lagos
    {  364, 5238,   50110,  9, }, // Africa/Libreville
    {  364, 6832,     704,  4, }, // Africa/Lome
    {  364, 5113,     708,  6, }, // Africa/Luanda
    {  364, 4841,     714,  9, }, // Africa/Lubumbashi
    {  364, 4903,     723,  6, }, // Africa/Lusaka
    {  364, 5224,     729,  6, }, // Africa/Malabo
    {  364, 4889,     735,  6, }, // Africa/Maputo
    {  364, 5099,     741,  6, }, // Africa/Maseru
    {  364, 5084,     747,  7, }, // Africa/Mbabane
    {  364, 5015,     754,  8, }, // Africa/Mogadishu
    {  364, 6997,   34899,  8, }, // Africa/Monrovia
    {  364, 3755,   34907,  7, }, // Africa/Nairobi
    {  364, 5173,   25836,  8, }, // Africa/Ndjamena
    {  364, 5256,   34914,  6, }, // Africa/Niamey
    {  364, 6766,     790,  7, }, // Africa/Nouakchott
    {  364, 6690,     797,  8, }, // Africa/Ouagadougou
    {  364, 5127,     805, 10, }, // Africa/Porto-Novo
    {  364, 4337,     815,  8, }, // Africa/Sao_Tome
    {  364, 2732,     823,  7, }, // Africa/Tripoli
    {  364, 6503,     830,  5, }, // Africa/Tunis
    {  364, 4054,     835,  7, }, // Africa/Windhoek
    {  364,  132,     842,  4, }, // America/Adak
    {  364, 3183,   50119,  8, }, // America/Anchorage
    {  364, 5524,   50127,  7, }, // America/Anguilla
    {  364, 5541,     860,  7, }, // America/Antigua
    {  364, 4456,     867,  9, }, // America/Araguaina
    {  364,  166,   50134, 12, }, // America/Argentina/Buenos_Aires
    {  364,   91,     888,  9, }, // America/Argentina/Catamarca
    {  364,  270,     897,  7, }, // America/Argentina/Cordoba
    {  364,  441,   25937,  5, }, // America/Argentina/Jujuy
    {  364, 7425,     909,  8, }, // America/Argentina/La_Rioja
    {  364,  565,     917,  7, }, // America/Argentina/Mendoza
    {  364, 7452,   50146, 12, }, // America/Argentina/Rio_Gallegos
    {  364, 7483,   25962,  6, }, // America/Argentina/Salta
    {  364, 7507,     940,  8, }, // America/Argentina/San_Juan
    {  364, 5497,     948,  8, }, // America/Argentina/San_Luis
    {  364, 7534,     956,  7, }, // America/Argentina/Tucuman
    {  364, 7560,  104836,  6, }, // America/Argentina/Ushuaia
    {  364, 5557,     969,  5, }, // America/Aruba
    {  364, 4154,   50158,  8, }, // America/Asuncion
    {  364,  237,   32574,  8, }, // America/Atikokan
    {  364, 3581,   34980,  4, }, // America/Bahia
    {  364, 7586,   50166, 16, }, // America/Bahia_Banderas
    {  364, 5571,    1013,  8, }, // America/Barbados
    {  364, 7609,    1021,  5, }, // America/Belem
    {  364, 5309,    1026,  5, }, // America/Belize
    {  364, 7623,    1031, 12, }, // America/Blanc-Sablon
    {  364, 7644,    1043,  9, }, // America/Boa_Vista
    {  364, 4263,    1052,  6, }, // America/Bogota
    {  364, 7662,   50182,  5, }, // America/Boise
    {  364, 7676,  104842, 12, }, // America/Cambridge_Bay
    {  364, 7698,    1075, 12, }, // America/Campo_Grande
    {  364, 3770,    1087,  6, }, // America/Cancun
    {  364, 4596,    1093,  7, }, // America/Caracas
    {  364, 4247,   50200,  7, }, // America/Cayenne
    {  364, 5398,  104854, 15, }, // America/Cayman
    {  364, 3251,    1129,  6, }, // America/Chicago
    {  364, 7719,    1135,  6, }, // America/Chihuahua
    {  364, 7737,  104869, 13, }, // America/Ciudad_Juarez
    {  364, 5324,    1141, 10, }, // America/Costa_Rica
    {  364, 7759,    1151,  7, }, // America/Creston
    {  364, 3693,   26138,  5, }, // America/Cuiaba
    {  364, 5640,   35086,  7, }, // America/Curacao
    {  364, 7775,   35093, 12, }, // America/Danmarkshavn
    {  364, 7796,    1184,  6, }, // America/Dawson
    {  364, 7811,  104882, 10, }, // America/Dawson_Creek
    {  364,  805,    1201,  6, }, // America/Denver
    {  364, 3351,   35129,  7, }, // America/Detroit
    {  364, 5656,    1214,  8, }, // America/Dominica
    {  364,  893,   50207,  8, }, // America/Edmonton
    {  364, 7832,   50215,  8, }, // America/Eirunepe
    {  364, 5343,   26215,  9, }, // America/El_Salvador
    {  364, 7849,  104892, 12, }, // America/Fort_Nelson
    {  364, 7869,    1257,  9, }, // America/Fortaleza
    {  364, 7887,   50235,  9, }, // America/Glace_Bay
    {  364, 6844,   50244,  7, }, // America/Goose_Bay
    {  364, 4514,    1282, 10, }, // America/Grand_Turk
    {  364, 5687,    1292,  7, }, // America/Grenada
    {  364, 5703,    1299,  9, }, // America/Guadeloupe
    {  364, 3662,    1308,  9, }, // America/Guatemala
    {  364, 4743,   50251,  8, }, // America/Guayaquil
    {  364, 6895,   50259,  6, }, // America/Guyana
    {  364, 1923,   50265,  8, }, // America/Halifax
    {  364, 2221,    1339,  6, }, // America/Havana
    {  364, 7905,   50273,  9, }, // America/Hermosillo
    {  364,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {  364,  481,    1365, 13, }, // America/Indiana/Knox
    {  364, 7924,    1378, 16, }, // America/Indiana/Marengo
    {  364, 7948,    1394, 19, }, // America/Indiana/Petersburg
    {  364, 7975,   54825, 18, }, // America/Indiana/Tell_City
    {  364, 8001,  104904, 14, }, // America/Indiana/Vevay
    {  364, 8023,   54857, 18, }, // America/Indiana/Vincennes
    {  364, 8049,   54875, 16, }, // America/Indiana/Winamac
    {  364, 8073,    1452,  6, }, // America/Inuvik
    {  364,  660,    1458,  7, }, // America/Iqaluit
    {  364, 2665,   26461,  6, }, // America/Jamaica
    {  364, 5282,   35226,  5, }, // America/Juneau
    {  364,  521,    1476,  8, }, // America/Kentucky/Louisville
    {  364, 8088,    1484, 20, }, // America/Kentucky/Monticello
    {  364, 5621,   35231, 10, }, // America/Kralendijk
    {  364, 4278,    1514,  6, }, // America/La_Paz
    {  364, 7132,    1520,  4, }, // America/Lima
    {  364, 3390,   50287, 12, }, // America/Los_Angeles
    {  364, 5849,   54891, 18, }, // America/Lower_Princes
    {  364, 8116,   35279,  6, }, // America/Maceio
    {  364, 8131,    1559,  7, }, // America/Managua
    {  364, 1892,    1566,  6, }, // America/Manaus
    {  364, 5814,    1572,  6, }, // America/Marigot
    {  364, 5722,    1578,  9, }, // America/Martinique
    {  364, 8147,    1587,  9, }, // America/Matamoros
    {  364, 2779,    1596,  8, }, // America/Mazatlan
    {  364, 8165,    1604,  8, }, // America/Menominee
    {  364, 8183,    1612,  6, }, // America/Merida
    {  364, 8198,    1618, 10, }, // America/Metlakatla
    {  364, 2811,    1628,  6, }, // America/Mexico_City
    {  364, 4293,    1634,  7, }, // America/Miquelon
    {  364, 8217,    1641,  7, }, // America/Moncton
    {  364, 8233,   50320,  9, }, // America/Monterrey
    {  364, 4000,    1657, 10, }, // America/Montevideo
    {  364, 5741,    1667, 10, }, // America/Montserrat
    {  364, 5383,    1677,  6, }, // America/Nassau
    {  364, 3294,   50338,  8, }, // America/New_York
    {  364, 8251,    1691,  3, }, // America/Nome
    {  364, 1834,   54909,  7, }, // America/Noronha
    {  364, 8264,  104918, 23, }, // America/North_Dakota/Beulah
    {  364, 8292,  104941, 24, }, // America/North_Dakota/Center
    {  364, 8320,  104965, 28, }, // America/North_Dakota/New_Salem
    {  364,  393,    1772,  4, }, // America/Nuuk
    {  364, 8351,    1776,  7, }, // America/Ojinaga
    {  364, 5413,    1783,  6, }, // America/Panama
    {  364, 6106,    1789, 10, }, // America/Paramaribo
    {  364, 3224,    1799,  6, }, // America/Phoenix
    {  364, 3847,    1805, 12, }, // America/Port-au-Prince
    {  364, 5871,   26771, 13, }, // America/Port_of_Spain
    {  364, 8367,   26784, 11, }, // America/Porto_Velho
    {  364, 5760,   50414, 11, }, // America/Puerto_Rico
    {  364, 3932,    1852, 12, }, // America/Punta_Arenas
    {  364, 8387,    1864, 12, }, // America/Rankin_Inlet
    {  364, 8408,    1876,  6, }, // America/Recife
    {  364, 1979,   50425,  8, }, // America/Regina
    {  364, 8423,   54981,  8, }, // America/Resolute
    {  364,  695,    1897, 10, }, // America/Rio_Branco
    {  364, 8440,    1907,  8, }, // America/Santarem
    {  364, 2165,   50433,  8, }, // America/Santiago
    {  364, 6044,    1923, 13, }, // America/Santo_Domingo
    {  364, 1862,    1936,  9, }, // America/Sao_Paulo
    {  364, 6862,  104993, 16, }, // America/Scoresbysund
    {  364, 8457,    1956,  5, }, // America/Sitka
    {  364, 8471,    1961, 13, }, // America/St_Barthelemy
    {  364, 2045,   26931, 10, }, // America/St_Johns
    {  364, 5780,    1983,  9, }, // America/St_Kitts
    {  364, 5797,   55008, 10, }, // America/St_Lucia
    {  364,  855,    2003, 10, }, // America/St_Thomas
    {  364, 5830,    2013, 12, }, // America/St_Vincent
    {  364, 8493,  105009, 13, }, // America/Swift_Current
    {  364, 5363,   50475, 12, }, // America/Tegucigalpa
    {  364, 5673,    2049,  4, }, // America/Thule
    {  364,  313,    2053,  7, }, // America/Tijuana
    {  364,  608,    2060,  7, }, // America/Toronto
    {  364, 5605,    2067,  7, }, // America/Tortola
    {  364, 2077,    2074,  8, }, // America/Vancouver
    {  364, 2128,   27017,  8, }, // America/Whitehorse
    {  364,  734,    2090,  8, }, // America/Winnipeg
    {  364, 8515,   27032,  6, }, // America/Yakutat
    {  364, 5918,   35501,  5, }, // Antarctica/Casey
    {  364, 6027,  105022,  6, }, // Antarctica/Davis
    {  364, 6066,   50487, 14, }, // Antarctica/DumontDUrville
    {  364, 8531,    2131,  8, }, // Antarctica/Macquarie
    {  364, 7029,    2139,  6, }, // Antarctica/Mawson
    {  364, 7076,    2145,  9, }, // Antarctica/McMurdo
    {  364, 8552,    2154,  6, }, // Antarctica/Palmer
    {  364, 7189,    2160,  6, }, // Antarctica/Rothera
    {  364, 7258,   27092,  4, }, // Antarctica/Syowa
    {  364, 8570,   50501,  6, }, // Antarctica/Troll
    {  364, 7337,   33428,  6, }, // Antarctica/Vostok
    {  364, 1413,   50507,  7, }, // Arctic/Longyearbyen
    {  364, 5487,    2192,  4, }, // Asia/Aden
    {  364, 5297,    2196,  6, }, // Asia/Almaty
    {  364, 3870,    2202,  5, }, // Asia/Amman
    {  364, 5428,    2207,  6, }, // Asia/Anadyr
    {  364, 5440,  105028,  5, }, // Asia/Aqtau
    {  364, 5270,  105033,  6, }, // Asia/Aqtobe
    {  364,  964,    2224,  7, }, // Asia/Ashgabat
    {  364, 8587,    2231,  6, }, // Asia/Atyrau
    {  364, 3509,  105039,  6, }, // Asia/Baghdad
    {  364, 5451,   35560,  7, }, // Asia/Bahrain
    {  364, 3555,    2250,  4, }, // Asia/Baku
    {  364, 4368,    2254,  7, }, // Asia/Bangkok
    {  364, 3473,  105045,  8, }, // Asia/Barnaul
    {  364, 3988,   27145,  6, }, // Asia/Beirut
    {  364, 3680,   27151,  6, }, // Asia/Bishkek
    {  364, 5906,   27157,  6, }, // Asia/Brunei
    {  364, 4503,    2286,  4, }, // Asia/Chita
    {  364, 5963,   35567,  9, }, // Asia/Choibalsan
    {  364, 4413,    2299,  7, }, // Asia/Colombo
    {  364, 4442,    2306,  6, }, // Asia/Damascus
    {  364, 1060,    2312,  5, }, // Asia/Dhaka
    {  364, 6125,    2317,  4, }, // Asia/Dili
    {  364, 3498,   27187,  5, }, // Asia/Dubai
    {  364, 6092,    2326,  7, }, // Asia/Dushanbe
    {  364, 8599,    2333,  9, }, // Asia/Famagusta
    {  364, 8614,    2342,  4, }, // Asia/Gaza
    {  364, 4697,    2346,  6, }, // Asia/Hebron
    {  364, 1226,    2352,  7, }, // Asia/Ho_Chi_Minh
    {  364, 2576,   50560,  7, }, // Asia/Hong_Kong
    {  364, 4673,    2366,  4, }, // Asia/Hovd
    {  364, 4086,    2370,  7, }, // Asia/Irkutsk
    {  364, 6969,   27229,  8, }, // Asia/Jakarta
    {  364, 6955,   27237,  8, }, // Asia/Jayapura
    {  364, 1257,    2392,  9, }, // Asia/Jerusalem
    {  364, 3462,    2401,  5, }, // Asia/Kabul
    {  364, 4218,    2420,  8, }, // Asia/Kamchatka
    {  364, 4141,    2431,  6, }, // Asia/Karachi
    {  364, 1152,    2437,  8, }, // Asia/Kathmandu
    {  364, 8624,    2445,  7, }, // Asia/Khandyga
    {  364,  992,   50608,  9, }, // Asia/Kolkata
    {  364, 4099,   35599, 10, }, // Asia/Krasnoyarsk
    {  364, 4761,    2471, 12, }, // Asia/Kuala_Lumpur
    {  364, 5893,    2483,  6, }, // Asia/Kuching
    {  364, 5464,   27284,  6, }, // Asia/Kuwait
    {  364, 1178,    2495,  5, }, // Asia/Macau
    {  364, 3919,    2500,  7, }, // Asia/Magadan
    {  364, 1316,    2507,  7, }, // Asia/Makassar
    {  364, 7145,    2514,  6, }, // Asia/Manila
    {  364, 6883,    2520,  6, }, // Asia/Muscat
    {  364, 2446,  105053,  7, }, // Asia/Nicosia
    {  364, 8638,    2533, 11, }, // Asia/Novokuznetsk
    {  364, 4037,  105060,  9, }, // Asia/Novosibirsk
    {  364, 4131,  105069,  4, }, // Asia/Omsk
    {  364, 7108,   33618,  4, }, // Asia/Oral
    {  364, 6924,   50617,  8, }, // Asia/Phnom_Penh
    {  364, 8656,    2572,  9, }, // Asia/Pontianak
    {  364, 4116,   27354,  7, }, // Asia/Pyongyang
    {  364, 5476,    2588,  5, }, // Asia/Qatar
    {  364, 8671,  105073,  8, }, // Asia/Qostanay
    {  364, 4171,  105081,  9, }, // Asia/Qyzylorda
    {  364, 3486,   50625,  7, }, // Asia/Riyadh
    {  364, 4310,    2623,  7, }, // Asia/Sakhalin
    {  364, 7208,  105090,  9, }, // Asia/Samarkand
    {  364, 3116,    2639,  4, }, // Asia/Seoul
    {  364, 1020,   27392,  6, }, // Asia/Shanghai
    {  364, 3137,    2649,  8, }, // Asia/Singapore
    {  364, 4199,    2657, 13, }, // Asia/Srednekolymsk
    {  364, 3100,   27420,  6, }, // Asia/Taipei
    {  364, 4683,    2676,  7, }, // Asia/Tashkent
    {  364, 3817,   33686,  7, }, // Asia/Tbilisi
    {  364, 2638,    2687,  7, }, // Asia/Tehran
    {  364, 1284,    2694,  7, }, // Asia/Thimphu
    {  364, 2687,    2701,  5, }, // Asia/Tokyo
    {  364, 4474,    2706,  5, }, // Asia/Tomsk
    {  364, 1346,  105099,  9, }, // Asia/Ulaanbaatar
    {  364, 1126,  105108,  6, }, // Asia/Urumqi
    {  364, 8685,   50651,  9, }, // Asia/Ust-Nera
    {  364, 6940,   50660,  8, }, // Asia/Vientiane
    {  364, 4612,    2742, 11, }, // Asia/Vladivostok
    {  364, 4730,   35672,  6, }, // Asia/Yakutsk
    {  364, 1202,   55061,  5, }, // Asia/Yangon
    {  364, 3785,    2766, 12, }, // Asia/Yekaterinburg
    {  364, 3649,    2778,  6, }, // Asia/Yerevan
    {  364, 3565,  105114, 13, }, // Atlantic/Azores
    {  364, 5588,  105127, 15, }, // Atlantic/Bermuda
    {  364, 6591,  105142, 14, }, // Atlantic/Canary
    {  364, 3629,    2835, 10, }, // Atlantic/Cape_Verde
    {  364, 1379,  105156, 14, }, // Atlantic/Faroe
    {  364, 8699,   35753,  7, }, // Atlantic/Madeira
    {  364, 2614,   50740,  9, }, // Atlantic/Reykjavik
    {  364, 7235,  105170, 16, }, // Atlantic/South_Georgia
    {  364, 6784,  105186, 17, }, // Atlantic/St_Helena
    {  364, 6607,   50782,  6, }, // Atlantic/Stanley
    {  364, 1654,    2926,  8, }, // Australia/Adelaide
    {  364, 1619,    2934,  7, }, // Australia/Brisbane
    {  364, 1783,    2941, 11, }, // Australia/Broken_Hill
    {  364, 1567,    2952,  6, }, // Australia/Darwin
    {  364, 3539,   27699,  4, }, // Australia/Eucla
    {  364, 1500,    2963,  6, }, // Australia/Hobart
    {  364, 8716,    2969,  8, }, // Australia/Lindeman
    {  364, 1531,    2977,  8, }, // Australia/Lord_Howe
    {  364, 1711,   27711,  8, }, // Australia/Melbourne
    {  364, 1746,    2992,  4, }, // Australia/Perth
    {  364, 1447,   50802,  6, }, // Australia/Sydney
    {  364, 6390,    3002,  9, }, // Europe/Amsterdam
    {  364, 6149,    3011,  7, }, // Europe/Andorra
    {  364, 3522,    3018,  8, }, // Europe/Astrakhan
    {  364, 6577,   50817,  5, }, // Europe/Athens
    {  364, 6285,    3030,  7, }, // Europe/Belgrade
    {  364, 4659,    3037,  6, }, // Europe/Berlin
    {  364, 6437,    3043, 10, }, // Europe/Bratislava
    {  364, 6178,   50822,  8, }, // Europe/Brussels
    {  364, 3830,    3061,  8, }, // Europe/Bucharest
    {  364, 3708,    3069,  8, }, // Europe/Budapest
    {  364, 8751,   50830, 19, }, // Europe/Busingen
    {  364, 2475,    3100,  7, }, // Europe/Chisinau
    {  364, 6238,   50849, 10, }, // Europe/Copenhagen
    {  364, 2260,    3117,  6, }, // Europe/Dublin
    {  364, 6256,    3123,  9, }, // Europe/Gibraltar
    {  364, 8767,    3132,  6, }, // Europe/Guernsey
    {  364, 6561,   50859,  9, }, // Europe/Helsinki
    {  364, 8783,  105203,  9, }, // Europe/Isle_of_Man
    {  364, 1097,  105212,  7, }, // Europe/Istanbul
    {  364, 8802,   50877,  6, }, // Europe/Jersey
    {  364, 3881,    3167, 11, }, // Europe/Kaliningrad
    {  364, 8924,    3178,  5, }, // Europe/Kirov
    {  364, 2419,    3183,  4, }, // Europe/Kyiv
    {  364, 3078,    3187,  8, }, // Europe/Lisbon
    {  364, 6455,   27891,  7, }, // Europe/Ljubljana
    {  364, 2393,    3204,  6, }, // Europe/London
    {  364, 6314,   27904, 10, }, // Europe/Luxembourg
    {  364, 4794,    3221,  6, }, // Europe/Madrid
    {  364, 6346,   27920,  6, }, // Europe/Malta
    {  364, 6531,    3232,  9, }, // Europe/Mariehamn
    {  364, 3595,    3241,  5, }, // Europe/Minsk
    {  364, 6359,    3246,  6, }, // Europe/Monaco
    {  364, 3443,  105219,  6, }, // Europe/Moscow
    {  364, 6407,    3258,  4, }, // Europe/Oslo
    {  364, 4186,    3262,  5, }, // Europe/Paris
    {  364, 6373,    3267,  9, }, // Europe/Podgorica
    {  364, 6224,    3276,  5, }, // Europe/Prague
    {  364, 8816,    3281,  4, }, // Europe/Riga
    {  364, 6273,    3285,  3, }, // Europe/Rome
    {  364, 4233,    3288,  6, }, // Europe/Samara
    {  364, 6419,    3294, 10, }, // Europe/San_Marino
    {  364, 6194,    3304,  7, }, // Europe/Sarajevo
    {  364, 4353,    3311,  7, }, // Europe/Saratov
    {  364, 8828,   50883, 11, }, // Europe/Simferopol
    {  364, 6332,   50894,  6, }, // Europe/Skopje
    {  364, 6548,   35907,  5, }, // Europe/Sofia
    {  364, 6472,   50900,  9, }, // Europe/Stockholm
    {  364, 8846,    3346,  6, }, // Europe/Tallinn
    {  364, 6135,    3352,  6, }, // Europe/Tirane
    {  364, 8861,   50909,  9, }, // Europe/Ulyanovsk
    {  364, 6301,    3367,  5, }, // Europe/Vaduz
    {  364, 6516,    3372,  7, }, // Europe/Vatican
    {  364, 6164,    3379,  4, }, // Europe/Vienna
    {  364, 8878,   50918,  7, }, // Europe/Vilnius
    {  364, 4629,    3390,  9, }, // Europe/Volgograd
    {  364, 3055,    3399,  7, }, // Europe/Warsaw
    {  364, 6210,    3406,  6, }, // Europe/Zagreb
    {  364, 6489,   35931,  5, }, // Europe/Zurich
    {  364, 4980,    3418, 12, }, // Indian/Antananarivo
    {  364, 6910,    3430,  5, }, // Indian/Chagos
    {  364, 5979,  105225, 15, }, // Indian/Christmas
    {  364, 5996,  105240, 14, }, // Indian/Cocos
    {  364, 4931,  105254, 14, }, // Indian/Comoro
    {  364, 6624,    3469,  8, }, // Indian/Kerguelen
    {  364, 7223,   28153,  3, }, // Indian/Mahe
    {  364, 7013,  105268, 16, }, // Indian/Maldives
    {  364, 3971,   55228,  8, }, // Indian/Mauritius
    {  364, 5000,   55236,  7, }, // Indian/Mayotte
    {  364, 7174,   51006,  7, }, // Indian/Reunion
    {  364, 4324,    3507,  4, }, // Pacific/Apia
    {  364,  932,    3511,  6, }, // Pacific/Auckland
    {  364, 3608,   51013,  9, }, // Pacific/Bougainville
    {  364, 2863,    3525,  5, }, // Pacific/Chatham
    {  364, 3022,    3530,  4, }, // Pacific/Chuuk
    {  364, 2201,  105284, 11, }, // Pacific/Easter
    {  364, 7323,   51067,  5, }, // Pacific/Efate
    {  364, 7290,    3550,  7, }, // Pacific/Fakaofo
    {  364, 3804,   36028,  5, }, // Pacific/Fiji
    {  364, 7306,    3561,  8, }, // Pacific/Funafuti
    {  364, 6641,    3569,  9, }, // Pacific/Galapagos
    {  364, 6659,   51093,  6, }, // Pacific/Gambier
    {  364, 3724,    3604, 11, }, // Pacific/Guadalcanal
    {  364, 5935,    3615,  4, }, // Pacific/Guam
    {  364, 2929,   51108,  8, }, // Pacific/Honolulu
    {  364, 2897,   51116,  9, }, // Pacific/Kanton
    {  364, 3900,    3625, 10, }, // Pacific/Kiritimati
    {  364, 6982,  105295,  6, }, // Pacific/Kosrae
    {  364, 2708,   28300, 10, }, // Pacific/Kwajalein
    {  364, 4779,   36063,  7, }, // Pacific/Majuro
    {  364, 3953,  105301, 15, }, // Pacific/Marquesas
    {  364, 8909,   51148,  6, }, // Pacific/Midway
    {  364, 7047,    3689,  5, }, // Pacific/Nauru
    {  364, 7095,   51162,  4, }, // Pacific/Niue
    {  364, 4070,    3698,  7, }, // Pacific/Norfolk
    {  364, 7061,    3705,  5, }, // Pacific/Noumea
    {  364, 2991,    3710,  9, }, // Pacific/Pago_Pago
    {  364, 7118,    3719,  5, }, // Pacific/Palau
    {  364, 7157,   55309,  7, }, // Pacific/Pitcairn
    {  364, 2961,   34263,  6, }, // Pacific/Pohnpei
    {  364, 4709,    3737, 11, }, // Pacific/Port_Moresby
    {  364, 6009,    3748,  9, }, // Pacific/Rarotonga
    {  364, 5948,   28394,  6, }, // Pacific/Saipan
    {  364, 7275,    3763,  5, }, // Pacific/Tahiti
    {  364, 6675,    3768,  6, }, // Pacific/Tarawa
    {  364, 4485,    3774,  9, }, // Pacific/Tongatapu
    {  364, 7355,   28414,  4, }, // Pacific/Wake
    {  364, 7368,    3787,  6, }, // Pacific/Wallis
    {  365,   14,       0,  6, }, // Africa/Asmara Kenyang/Latin/Cameroon
    {  365, 4337,       6,  8, }, // Africa/Sao_Tome
    {  365, 4154,      14,  8, }, // America/Asuncion
    {  365,  237,      22,  8, }, // America/Atikokan
    {  365, 7586,      30, 17, }, // America/Bahia_Banderas
    {  365, 3770,      47,  6, }, // America/Cancun
    {  365, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  365, 5640,      66,  7, }, // America/Curacao
    {  365,  481,      73, 13, }, // America/Indiana/Knox
    {  365, 7924,      86, 16, }, // America/Indiana/Marengo
    {  365, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  365, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  365, 8001,     139, 14, }, // America/Indiana/Vevay
    {  365, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  365, 8049,     171, 16, }, // America/Indiana/Winamac
    {  365, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  365, 5849,     207, 22, }, // America/Lower_Princes
    {  365, 8183,     229,  6, }, // America/Merida
    {  365, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  365, 8292,     255, 20, }, // America/North_Dakota/Center
    {  365, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  365,  393,     298,  4, }, // America/Nuuk
    {  365, 6862,     302, 16, }, // America/Scoresbysund
    {  365, 8471,     318, 14, }, // America/St_Barthelemy
    {  365, 2045,     332, 10, }, // America/St_Johns
    {  365, 5780,     342,  9, }, // America/St_Kitts
    {  365, 5797,     351,  9, }, // America/St_Lucia
    {  365,  855,     360, 10, }, // America/St_Thomas
    {  365, 5830,     370, 11, }, // America/St_Vincent
    {  365, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  365, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  365, 1152,     413,  9, }, // Asia/Kathmandu
    {  365,  992,     422,  7, }, // Asia/Kolkata
    {  365, 1178,     429,  5, }, // Asia/Macau
    {  365, 1202,     442,  6, }, // Asia/Yangon
    {  365, 1379,     448,  5, }, // Atlantic/Faroe
    {  365, 6784,     453, 10, }, // Atlantic/St_Helena
    {  365, 2419,     463,  4, }, // Europe/Kyiv
    {  365, 7174,     467,  7, }, // Indian/Reunion
    {  365, 3022,     474,  5, }, // Pacific/Chuuk
    {  365, 2961,     479,  7, }, // Pacific/Pohnpei
    {  366, 6751,  105316, 10, }, // Africa/Abidjan Khmer/Khmer/Cambodia
    {  366, 6723,  105326,  6, }, // Africa/Accra
    {  366, 4961,  105332, 11, }, // Africa/Addis_Ababa
    {  366, 7383,  105343,  8, }, // Africa/Algiers
    {  366,   14,  105351,  9, }, // Africa/Asmara
    {  366,   44,  105360,  7, }, // Africa/Bamako
    {  366, 5159,  105367,  7, }, // Africa/Bangui
    {  366, 6709,  105374,  9, }, // Africa/Banjul
    {  366, 7398,  105383,  7, }, // Africa/Bissau
    {  366, 4859,  105390, 11, }, // Africa/Blantyre
    {  366, 5189,  105401, 11, }, // Africa/Brazzaville
    {  366, 4824,  105412, 13, }, // Africa/Bujumbura
    {  366, 2242,  105425,  3, }, // Africa/Cairo
    {  366, 4019,  105428, 12, }, // Africa/Casablanca
    {  366, 7412,  105440,  4, }, // Africa/Ceuta
    {  366, 6736,  105444,  8, }, // Africa/Conakry
    {  366, 6803,  105452,  4, }, // Africa/Dakar
    {  366, 5032,  105456, 10, }, // Africa/Dar_es_Salaam
    {  366, 4945,  105466,  7, }, // Africa/Djibouti
    {  366, 5145,  105473,  7, }, // Africa/Douala
    {  366, 5068,  105480, 11, }, // Africa/El_Aaiun
    {  366, 6816,  105491,  9, }, // Africa/Freetown
    {  366, 4808,  105500, 10, }, // Africa/Gaborone
    {  366, 4917,  105510,  8, }, // Africa/Harare
    {  366, 4381,  105518, 13, }, // Africa/Johannesburg
    {  366, 4401,  105531,  4, }, // Africa/Juba
    {  366, 5053,  105535,  7, }, // Africa/Kampala
    {  366, 4426,  105542,  5, }, // Africa/Khartoum
    {  366, 4875,  105547,  8, }, // Africa/Kigali
    {  366, 5208,  105555,  9, }, // Africa/Kinshasa
    {  366, 4646,  105564,  7, }, // Africa/Lagos
    {  366, 5238,  105571,  9, }, // Africa/Libreville
    {  366, 6832,  105580,  5, }, // Africa/Lome
    {  366, 5113,  105585,  7, }, // Africa/Luanda
    {  366, 4841,  105592, 11, }, // Africa/Lubumbashi
    {  366, 4903,  105603,  6, }, // Africa/Lusaka
    {  366, 5224,  105609,  7, }, // Africa/Malabo
    {  366, 4889,  105616,  8, }, // Africa/Maputo
    {  366, 5099,  105624,  8, }, // Africa/Maseru
    {  366, 5084,  105632,  5, }, // Africa/Mbabane
    {  366, 5015,  105637, 13, }, // Africa/Mogadishu
    {  366, 6997,  105650, 12, }, // Africa/Monrovia
    {  366, 3755,  105662,  8, }, // Africa/Nairobi
    {  366, 5173,  105670,  8, }, // Africa/Ndjamena
    {  366, 5256,  105678,  8, }, // Africa/Niamey
    {  366, 6766,  105686,  6, }, // Africa/Nouakchott
    {  366, 6690,  105692, 12, }, // Africa/Ouagadougou
    {  366, 5127,  105704, 10, }, // Africa/Porto-Novo
    {  366, 4337,  105714,  7, }, // Africa/Sao_Tome
    {  366, 2732,  105721,  9, }, // Africa/Tripoli
    {  366, 6503,  105730,  6, }, // Africa/Tunis
    {  366, 4054,  105736,  6, }, // Africa/Windhoek
    {  366,  132,  105742,  5, }, // America/Adak
    {  366, 3183,  105747,  9, }, // America/Anchorage
    {  366, 5524,  105756,  9, }, // America/Anguilla
    {  366, 5541,  105765,  9, }, // America/Antigua
    {  366, 4456,  105774, 12, }, // America/Araguaina
    {  366,  166,  105786,  9, }, // America/Argentina/Buenos_Aires
    {  366,   91,  105795,  9, }, // America/Argentina/Catamarca
    {  366,  270,  105804,  6, }, // America/Argentina/Cordoba
    {  366,  441,  105810,  5, }, // America/Argentina/Jujuy
    {  366, 7425,  105815,  9, }, // America/Argentina/La_Rioja
    {  366,  565,  105824, 10, }, // America/Argentina/Mendoza
    {  366, 7452,  105834, 16, }, // America/Argentina/Rio_Gallegos
    {  366, 7483,  105850,  6, }, // America/Argentina/Salta
    {  366, 7507,  105856,  9, }, // America/Argentina/San_Juan
    {  366, 5497,  105865, 10, }, // America/Argentina/San_Luis
    {  366, 7534,  105875,  8, }, // America/Argentina/Tucuman
    {  366, 7560,  105883,  8, }, // America/Argentina/Ushuaia
    {  366, 5557,  105891,  7, }, // America/Aruba
    {  366, 4154,  105898, 10, }, // America/Asuncion
    {  366,  237,  105908,  9, }, // America/Atikokan
    {  366, 3581,  105917,  4, }, // America/Bahia
    {  366, 7586,  105921, 13, }, // America/Bahia_Banderas
    {  366, 5571,  105934,  7, }, // America/Barbados
    {  366, 7609,  105941,  5, }, // America/Belem
    {  366, 5309,  105946,  4, }, // America/Belize
    {  366, 7623,  105950, 13, }, // America/Blanc-Sablon
    {  366, 7644,  105963,  9, }, // America/Boa_Vista
    {  366, 4263,  105972,  8, }, // America/Bogota
    {  366, 7662,  105980,  6, }, // America/Boise
    {  366, 7676,  105986, 10, }, // America/Cambridge_Bay
    {  366, 7698,  105996, 14, }, // America/Campo_Grande
    {  366, 3770,  106010,  6, }, // America/Cancun
    {  366, 4596,  106016,  8, }, // America/Caracas
    {  366, 4247,  106024,  6, }, // America/Cayenne
    {  366, 5398,  106030,  5, }, // America/Cayman
    {  366, 3251,  106035,  9, }, // America/Chicago
    {  366, 7719,  106044,  6, }, // America/Chihuahua
    {  366, 7737,  106050, 19, }, // America/Ciudad_Juarez
    {  366, 5324,  106069, 10, }, // America/Costa_Rica
    {  366, 7759,  106079,  9, }, // America/Creston
    {  366, 3693,  106088,  7, }, // America/Cuiaba
    {  366, 5640,  106095,  7, }, // America/Curacao
    {  366, 7775,  106102, 14, }, // America/Danmarkshavn
    {  366, 7796,  106116,  6, }, // America/Dawson
    {  366, 7811,  106122, 12, }, // America/Dawson_Creek
    {  366,  805,  106134,  5, }, // America/Denver
    {  366, 3351,  106139, 10, }, // America/Detroit
    {  366, 5656,  106149,  7, }, // America/Dominica
    {  366,  893,  106156, 10, }, // America/Edmonton
    {  366, 7832,  106166, 11, }, // America/Eirunepe
    {  366, 5343,  106177, 13, }, // America/El_Salvador
    {  366, 7849,  106190, 11, }, // America/Fort_Nelson
    {  366, 7869,  106201, 14, }, // America/Fortaleza
    {  366, 7887,  106215,  8, }, // America/Glace_Bay
    {  366, 6844,  106223,  6, }, // America/Goose_Bay
    {  366, 4514,  106229, 11, }, // America/Grand_Turk
    {  366, 5687,  106240,  9, }, // America/Grenada
    {  366, 5703,  106249, 11, }, // America/Guadeloupe
    {  366, 3662,  106260, 11, }, // America/Guatemala
    {  366, 4743,  106271, 10, }, // America/Guayaquil
    {  366, 6895,  106281,  8, }, // America/Guyana
    {  366, 1923,  106289,  9, }, // America/Halifax
    {  366, 2221,  106298,  8, }, // America/Havana
    {  366, 7905,  106306, 11, }, // America/Hermosillo
    {  366,  348,  106317, 13, }, // America/Indiana/Indianapolis
    {  366,  481,  106330, 11, }, // America/Indiana/Knox
    {  366, 7924,  106341, 16, }, // America/Indiana/Marengo
    {  366, 7948,  106357, 17, }, // America/Indiana/Petersburg
    {  366, 7975,  106374, 18, }, // America/Indiana/Tell_City
    {  366, 8001,  106392, 16, }, // America/Indiana/Vevay
    {  366, 8023,  106408, 16, }, // America/Indiana/Vincennes
    {  366, 8049,  106424, 17, }, // America/Indiana/Winamac
    {  366, 8073,  106441,  8, }, // America/Inuvik
    {  366,  660,  106449, 11, }, // America/Iqaluit
    {  366, 2665,  106460, 11, }, // America/Jamaica
    {  366, 5282,  106471,  6, }, // America/Juneau
    {  366,  521,  106477,  8, }, // America/Kentucky/Louisville
    {  366, 8088,  106485, 17, }, // America/Kentucky/Monticello
    {  366, 5621,  106502, 10, }, // America/Kralendijk
    {  366, 4278,  106512,  6, }, // America/La_Paz
    {  366, 7132,  106518,  5, }, // America/Lima
    {  366, 3390,  106523, 12, }, // America/Los_Angeles
    {  366, 5849,  106535, 19, }, // America/Lower_Princes
    {  366, 8116,  106554,  8, }, // America/Maceio
    {  366, 8131,  106562,  9, }, // America/Managua
    {  366, 1892,  106571,  9, }, // America/Manaus
    {  366, 5814,  106580, 10, }, // America/Marigot
    {  366, 5722,  106590,  8, }, // America/Martinique
    {  366, 8147,  106598, 12, }, // America/Matamoros
    {  366, 2779,  106610, 11, }, // America/Mazatlan
    {  366, 8165,  106621,  9, }, // America/Menominee
    {  366, 8183,  106630,  7, }, // America/Merida
    {  366, 8198,  106637, 13, }, // America/Metlakatla
    {  366, 2811,  106650, 16, }, // America/Mexico_City
    {  366, 4293,  106666,  9, }, // America/Miquelon
    {  366, 8217,  106675,  7, }, // America/Moncton
    {  366, 8233,  106682,  9, }, // America/Monterrey
    {  366, 4000,  106691, 12, }, // America/Montevideo
    {  366, 5741,  106703, 12, }, // America/Montserrat
    {  366, 5383,  106715,  6, }, // America/Nassau
    {  366, 3294,  106721,  6, }, // America/New_York
    {  366, 8251,  106624,  3, }, // America/Nome
    {  366, 1834,  106727,  9, }, // America/Noronha
    {  366, 8264,  106736, 19, }, // America/North_Dakota/Beulah
    {  366, 8292,  106755, 20, }, // America/North_Dakota/Center
    {  366, 8320,  106775, 24, }, // America/North_Dakota/New_Salem
    {  366,  393,  106799,  3, }, // America/Nuuk
    {  366, 8351,  106802, 10, }, // America/Ojinaga
    {  366, 5413,  106812,  8, }, // America/Panama
    {  366, 6106,  106820, 13, }, // America/Paramaribo
    {  366, 3224,  106833,  5, }, // America/Phoenix
    {  366, 3847,  106838, 12, }, // America/Port-au-Prince
    {  366, 5871,  106850, 11, }, // America/Port_of_Spain
    {  366, 8367,  106861, 11, }, // America/Porto_Velho
    {  366, 5760,  106872,  9, }, // America/Puerto_Rico
    {  366, 3932,  106881, 13, }, // America/Punta_Arenas
    {  366, 8387,  106894, 16, }, // America/Rankin_Inlet
    {  366, 8408,  106910, 10, }, // America/Recife
    {  366, 1979,  106920,  9, }, // America/Regina
    {  366, 8423,  106929, 10, }, // America/Resolute
    {  366,  695,  106939, 12, }, // America/Rio_Branco
    {  366, 8440,  106951,  9, }, // America/Santarem
    {  366, 2165,  106960, 12, }, // America/Santiago
    {  366, 6044,  106972, 11, }, // America/Santo_Domingo
    {  366, 1862,  106983,  8, }, // America/Sao_Paulo
    {  366, 6862,  106991, 13, }, // America/Scoresbysund
    {  366, 8457,  107004,  6, }, // America/Sitka
    {  366, 8471,  107010, 14, }, // America/St_Barthelemy
    {  366, 2045,  107024,  7, }, // America/St_Johns
    {  366, 5780,  107031,  8, }, // America/St_Kitts
    {  366, 5797,  107039,  9, }, // America/St_Lucia
    {  366,  855,  107048, 12, }, // America/St_Thomas
    {  366, 5830,  107060, 11, }, // America/St_Vincent
    {  366, 8493,  107071, 11, }, // America/Swift_Current
    {  366, 5363,  107082, 18, }, // America/Tegucigalpa
    {  366, 5673,  107100,  4, }, // America/Thule
    {  366,  313,  107104, 10, }, // America/Tijuana
    {  366,  608,  107114,  8, }, // America/Toronto
    {  366, 5605,  107122,  6, }, // America/Tortola
    {  366, 2077,  107128,  9, }, // America/Vancouver
    {  366, 2128,  107137,  6, }, // America/Whitehorse
    {  366,  734,  107143,  7, }, // America/Winnipeg
    {  366, 8515,  107150,  8, }, // America/Yakutat
    {  366, 5918,  107158,  4, }, // Antarctica/Casey
    {  366, 6027,  107162,  5, }, // Antarctica/Davis
    {  366, 6066,  107167, 16, }, // Antarctica/DumontDUrville
    {  366, 8531,  107183, 10, }, // Antarctica/Macquarie
    {  366, 7029,  107193,  7, }, // Antarctica/Mawson
    {  366, 7076,  107200, 10, }, // Antarctica/McMurdo
    {  366, 8552,  107210,  7, }, // Antarctica/Palmer
    {  366, 7189,  107217,  8, }, // Antarctica/Rothera
    {  366, 7258,  107225,  9, }, // Antarctica/Syowa
    {  366, 8570,  107234,  5, }, // Antarctica/Troll
    {  366, 7337,  107239,  8, }, // Antarctica/Vostok
    {  366, 1413,  107247, 14, }, // Arctic/Longyearbyen
    {  366, 5487,  107261,  5, }, // Asia/Aden
    {  366, 5297,  107266,  9, }, // Asia/Almaty
    {  366, 3870,  107275,  7, }, // Asia/Amman
    {  366, 5428,  107282,  6, }, // Asia/Anadyr
    {  366, 5440,  107288,  5, }, // Asia/Aqtau
    {  366, 5270,  107293,  8, }, // Asia/Aqtobe
    {  366,  964,  107301, 10, }, // Asia/Ashgabat
    {  366, 8587,  107311,  7, }, // Asia/Atyrau
    {  366, 3509,  107318,  6, }, // Asia/Baghdad
    {  366, 5451,  107324,  6, }, // Asia/Bahrain
    {  366, 3555,  107330,  4, }, // Asia/Baku
    {  366, 4368,  107334,  5, }, // Asia/Bangkok
    {  366, 3473,  107339,  5, }, // Asia/Barnaul
    {  366, 3988,  107344,  5, }, // Asia/Beirut
    {  366, 3680,  107349,  8, }, // Asia/Bishkek
    {  366, 5906,  107357,  7, }, // Asia/Brunei
    {  366, 4503,  107364,  4, }, // Asia/Chita
    {  366, 5963,  107368,  9, }, // Asia/Choibalsan
    {  366, 4413,  107377,  7, }, // Asia/Colombo
    {  366, 4442,  107384,  6, }, // Asia/Damascus
    {  366, 1060,  107390,  6, }, // Asia/Dhaka
    {  366, 6125,  107396,  4, }, // Asia/Dili
    {  366, 3498,  107400,  5, }, // Asia/Dubai
    {  366, 6092,  107405, 10, }, // Asia/Dushanbe
    {  366, 8599,  107415, 16, }, // Asia/Famagusta
    {  366, 8614,  105428,  4, }, // Asia/Gaza
    {  366, 4697,  107431,  6, }, // Asia/Hebron
    {  366, 1226,  107437,  7, }, // Asia/Ho_Chi_Minh
    {  366, 2576,  107444,  6, }, // Asia/Hong_Kong
    {  366, 4673,  107450,  3, }, // Asia/Hovd
    {  366, 4086,  107453, 11, }, // Asia/Irkutsk
    {  366, 6969,  107464,  9, }, // Asia/Jakarta
    {  366, 6955,  107473, 10, }, // Asia/Jayapura
    {  366, 1257,  107483, 13, }, // Asia/Jerusalem
    {  366, 3462,  107496,  6, }, // Asia/Kabul
    {  366, 4218,  107502,  8, }, // Asia/Kamchatka
    {  366, 4141,  107510,  7, }, // Asia/Karachi
    {  366, 1152,  107517, 11, }, // Asia/Kathmandu
    {  366, 8624,  107528, 10, }, // Asia/Khandyga
    {  366,  992,  107538,  7, }, // Asia/Kolkata
    {  366, 4099,  107545, 11, }, // Asia/Krasnoyarsk
    {  366, 4761,  107556, 10, }, // Asia/Kuala_Lumpur
    {  366, 5893,  107566,  6, }, // Asia/Kuching
    {  366, 5464,  107572,  6, }, // Asia/Kuwait
    {  366, 1178,  107578,  6, }, // Asia/Macau
    {  366, 3919,  107584, 10, }, // Asia/Magadan
    {  366, 1316,  107594,  9, }, // Asia/Makassar
    {  366, 7145,  107603,  6, }, // Asia/Manila
    {  366, 6883,  107609,  8, }, // Asia/Muscat
    {  366, 2446,  107617,  7, }, // Asia/Nicosia
    {  366, 8638,  107624, 16, }, // Asia/Novokuznetsk
    {  366, 4037,  107640, 12, }, // Asia/Novosibirsk
    {  366, 4131,  107652,  8, }, // Asia/Omsk
    {  366, 7108,  107660,  7, }, // Asia/Oral
    {  366, 6924,  107667,  7, }, // Asia/Phnom_Penh
    {  366, 8656,  107674, 12, }, // Asia/Pontianak
    {  366, 4116,  107686,  9, }, // Asia/Pyongyang
    {  366, 5476,  105795,  4, }, // Asia/Qatar
    {  366, 8671,  107695,  8, }, // Asia/Qostanay
    {  366, 4171,  107703, 11, }, // Asia/Qyzylorda
    {  366, 3486,  107714,  6, }, // Asia/Riyadh
    {  366, 4310,  107720,  7, }, // Asia/Sakhalin
    {  366, 7208,  107727,  9, }, // Asia/Samarkand
    {  366, 3116,  107736,  6, }, // Asia/Seoul
    {  366, 1020,  107742,  5, }, // Asia/Shanghai
    {  366, 3137,  107747,  9, }, // Asia/Singapore
    {  366, 4199,  107756, 12, }, // Asia/Srednekolymsk
    {  366, 3100,  107768,  5, }, // Asia/Taipei
    {  366, 4683,  107773,  7, }, // Asia/Tashkent
    {  366, 3817,  107780,  8, }, // Asia/Tbilisi
    {  366, 2638,  107788,  8, }, // Asia/Tehran
    {  366, 1284,  107796,  5, }, // Asia/Thimphu
    {  366, 2687,  107801,  6, }, // Asia/Tokyo
    {  366, 4474,  107807,  7, }, // Asia/Tomsk
    {  366, 1346,  107814, 11, }, // Asia/Ulaanbaatar
    {  366, 1126,  107825, 10, }, // Asia/Urumqi
    {  366, 8685,  107835, 10, }, // Asia/Ust-Nera
    {  366, 6940,  107845,  9, }, // Asia/Vientiane
    {  366, 4612,  107854, 15, }, // Asia/Vladivostok
    {  366, 4730,  107869, 11, }, // Asia/Yakutsk
    {  366, 1202,  107880,  9, }, // Asia/Yangon
    {  366, 3785,  107889, 14, }, // Asia/Yekaterinburg
    {  366, 3649,  107903, 11, }, // Asia/Yerevan
    {  366, 3565,  107914,  8, }, // Atlantic/Azores
    {  366, 5588,  107922,  8, }, // Atlantic/Bermuda
    {  366, 6591,  107930,  6, }, // Atlantic/Canary
    {  366, 3629,  107936,  7, }, // Atlantic/Cape_Verde
    {  366, 1379,  107943,  7, }, // Atlantic/Faroe
    {  366, 8699,  107950,  8, }, // Atlantic/Madeira
    {  366, 2614,  107958, 10, }, // Atlantic/Reykjavik
    {  366, 7235,  107968, 19, }, // Atlantic/South_Georgia
    {  366, 6784,  107987, 10, }, // Atlantic/St_Helena
    {  366, 6607,  107997,  7, }, // Atlantic/Stanley
    {  366, 1654,  108004,  7, }, // Australia/Adelaide
    {  366, 1619,  108011,  9, }, // Australia/Brisbane
    {  366, 1783,  108020, 11, }, // Australia/Broken_Hill
    {  366, 1567,  108031,  5, }, // Australia/Darwin
    {  366, 3539,  108036,  8, }, // Australia/Eucla
    {  366, 1500,  108044,  5, }, // Australia/Hobart
    {  366, 8716,  108049,  9, }, // Australia/Lindeman
    {  366, 1531,  108058,  7, }, // Australia/Lord_Howe
    {  366, 1711,  108065,  7, }, // Australia/Melbourne
    {  366, 1746,  108072,  3, }, // Australia/Perth
    {  366, 1447,  108075,  6, }, // Australia/Sydney
    {  366, 6390,  108081, 10, }, // Europe/Amsterdam
    {  366, 6149,  108091,  8, }, // Europe/Andorra
    {  366, 3522,  108099, 12, }, // Europe/Astrakhan
    {  366, 6577,  108111,  5, }, // Europe/Athens
    {  366, 6285,  108116,  8, }, // Europe/Belgrade
    {  366, 4659,  108124,  7, }, // Europe/Berlin
    {  366, 6437,  108131, 13, }, // Europe/Bratislava
    {  366, 6178,  108144,  8, }, // Europe/Brussels
    {  366, 3830,  108152, 10, }, // Europe/Bucharest
    {  366, 3708,  108162, 10, }, // Europe/Budapest
    {  366, 8751,  108172, 12, }, // Europe/Busingen
    {  366, 2475,  108184,  7, }, // Europe/Chisinau
    {  366, 6238,  108191,  9, }, // Europe/Copenhagen
    {  366, 2260,  108200,  9, }, // Europe/Dublin
    {  366, 6256,  108209, 13, }, // Europe/Gibraltar
    {  366, 8767,  108222,  8, }, // Europe/Guernsey
    {  366, 6561,  108230,  9, }, // Europe/Helsinki
    {  366, 8783,  108239, 12, }, // Europe/Isle_of_Man
    {  366, 1097,  108251, 12, }, // Europe/Istanbul
    {  366, 8802,  108263,  5, }, // Europe/Jersey
    {  366, 3881,  108268, 14, }, // Europe/Kaliningrad
    {  366, 8924,  108282,  6, }, // Europe/Kirov
    {  366, 2419,  108288,  3, }, // Europe/Kyiv
    {  366, 3078,  108291,  6, }, // Europe/Lisbon
    {  366, 6455,  108297,  9, }, // Europe/Ljubljana
    {  366, 2393,  108306,  5, }, // Europe/London
    {  366, 6314,  108311,  7, }, // Europe/Luxembourg
    {  366, 4794,  108318,  8, }, // Europe/Madrid
    {  366, 6346,  108326,  7, }, // Europe/Malta
    {  366, 6531,  108333,  8, }, // Europe/Mariehamn
    {  366, 3595,  108341,  8, }, // Europe/Minsk
    {  366, 6359,  108349,  7, }, // Europe/Monaco
    {  366, 3443,  108356,  6, }, // Europe/Moscow
    {  366, 6407,  108362,  6, }, // Europe/Oslo
    {  366, 4186,  108368,  6, }, // Europe/Paris
    {  366, 6373,  108374, 12, }, // Europe/Podgorica
    {  366, 6224,  108386,  5, }, // Europe/Prague
    {  366, 8816,  108391,  6, }, // Europe/Riga
    {  366, 6273,  107947,  4, }, // Europe/Rome
    {  366, 4233,  108397,  8, }, // Europe/Samara
    {  366, 6419,  108405, 12, }, // Europe/San_Marino
    {  366, 6194,  108417, 11, }, // Europe/Sarajevo
    {  366, 4353,  108428,  8, }, // Europe/Saratov
    {  366, 8828,  108436, 15, }, // Europe/Simferopol
    {  366, 6332,  108451,  7, }, // Europe/Skopje
    {  366, 6548,  108458, 10, }, // Europe/Sofia
    {  366, 6472,  108468,  8, }, // Europe/Stockholm
    {  366, 8846,  108476,  5, }, // Europe/Tallinn
    {  366, 6135,  108481,  7, }, // Europe/Tirane
    {  366, 8861,  108488, 14, }, // Europe/Ulyanovsk
    {  366, 6301,  108502,  6, }, // Europe/Vaduz
    {  366, 6516,  108508,  8, }, // Europe/Vatican
    {  366, 6164,  108516,  5, }, // Europe/Vienna
    {  366, 8878,  108521,  8, }, // Europe/Vilnius
    {  366, 4629,  108529, 15, }, // Europe/Volgograd
    {  366, 3055,  108544,  7, }, // Europe/Warsaw
    {  366, 6210,  108551,  7, }, // Europe/Zagreb
    {  366, 6489,  108558,  8, }, // Europe/Zurich
    {  366, 4980,  108566, 12, }, // Indian/Antananarivo
    {  366, 6910,  106038,  6, }, // Indian/Chagos
    {  366, 5979,  108578,  9, }, // Indian/Christmas
    {  366, 5996,  108587,  5, }, // Indian/Cocos
    {  366, 4931,  108592,  5, }, // Indian/Comoro
    {  366, 6624,  108597, 10, }, // Indian/Kerguelen
    {  366, 7223,  108607,  5, }, // Indian/Mahe
    {  366, 7013,  108612,  8, }, // Indian/Maldives
    {  366, 3971,  108620,  6, }, // Indian/Mauritius
    {  366, 5000,  108626,  6, }, // Indian/Mayotte
    {  366, 7174,  108632,  9, }, // Indian/Reunion
    {  366, 4324,  108641,  4, }, // Pacific/Apia
    {  366,  932,  108645,  5, }, // Pacific/Auckland
    {  366, 3608,  108650, 10, }, // Pacific/Bougainville
    {  366, 2863,  108660,  5, }, // Pacific/Chatham
    {  366, 3022,  108665,  5, }, // Pacific/Chuuk
    {  366, 2201,  108670,  8, }, // Pacific/Easter
    {  366, 7323,  108678,  9, }, // Pacific/Efate
    {  366, 7290,  108687, 10, }, // Pacific/Fakaofo
    {  366, 3804,  108697,  6, }, // Pacific/Fiji
    {  366, 7306,  108703, 14, }, // Pacific/Funafuti
    {  366, 6641,  108717, 14, }, // Pacific/Galapagos
    {  366, 6659,  108731,  7, }, // Pacific/Gambier
    {  366, 3724,  108738, 14, }, // Pacific/Guadalcanal
    {  366, 5935,  108731,  5, }, // Pacific/Guam
    {  366, 2929,  108752,  8, }, // Pacific/Honolulu
    {  366, 2897,  108760, 11, }, // Pacific/Kanton
    {  366, 3900,  108771, 11, }, // Pacific/Kiritimati
    {  366, 6982,  108782,  6, }, // Pacific/Kosrae
    {  366, 2708,  108788, 11, }, // Pacific/Kwajalein
    {  366, 4779,  108799, 10, }, // Pacific/Majuro
    {  366, 3953,  108809, 10, }, // Pacific/Marquesas
    {  366, 8909,  108819,  6, }, // Pacific/Midway
    {  366, 7047,  108825,  4, }, // Pacific/Nauru
    {  366, 7095,  108829,  5, }, // Pacific/Niue
    {  366, 4070,  108834,  8, }, // Pacific/Norfolk
    {  366, 7061,  108842,  4, }, // Pacific/Noumea
    {  366, 2991,  108846, 15, }, // Pacific/Pago_Pago
    {  366, 7118,  108861,  5, }, // Pacific/Palau
    {  366, 7157,  108866,  9, }, // Pacific/Pitcairn
    {  366, 2961,  108875,  6, }, // Pacific/Pohnpei
    {  366, 4709,  108881, 12, }, // Pacific/Port_Moresby
    {  366, 6009,  108893, 13, }, // Pacific/Rarotonga
    {  366, 5948,  108906,  6, }, // Pacific/Saipan
    {  366, 7275,  108912,  6, }, // Pacific/Tahiti
    {  366, 6675,  108918,  8, }, // Pacific/Tarawa
    {  366, 4485,  108926, 11, }, // Pacific/Tongatapu
    {  366, 7355,  108937,  5, }, // Pacific/Wake
    {  366, 7368,  108942,  6, }, // Pacific/Wallis
    {  367,   14,       0,  6, }, // Africa/Asmara Kiche/Latin/Guatemala
    {  367, 4337,       6,  8, }, // Africa/Sao_Tome
    {  367, 4154,      14,  8, }, // America/Asuncion
    {  367,  237,      22,  8, }, // America/Atikokan
    {  367, 7586,      30, 17, }, // America/Bahia_Banderas
    {  367, 3770,      47,  6, }, // America/Cancun
    {  367, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  367, 5640,      66,  7, }, // America/Curacao
    {  367,  481,      73, 13, }, // America/Indiana/Knox
    {  367, 7924,      86, 16, }, // America/Indiana/Marengo
    {  367, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  367, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  367, 8001,     139, 14, }, // America/Indiana/Vevay
    {  367, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  367, 8049,     171, 16, }, // America/Indiana/Winamac
    {  367, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  367, 5849,     207, 22, }, // America/Lower_Princes
    {  367, 8183,     229,  6, }, // America/Merida
    {  367, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  367, 8292,     255, 20, }, // America/North_Dakota/Center
    {  367, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  367,  393,     298,  4, }, // America/Nuuk
    {  367, 6862,     302, 16, }, // America/Scoresbysund
    {  367, 8471,     318, 14, }, // America/St_Barthelemy
    {  367, 2045,     332, 10, }, // America/St_Johns
    {  367, 5780,     342,  9, }, // America/St_Kitts
    {  367, 5797,     351,  9, }, // America/St_Lucia
    {  367,  855,     360, 10, }, // America/St_Thomas
    {  367, 5830,     370, 11, }, // America/St_Vincent
    {  367, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  367, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  367, 1152,     413,  9, }, // Asia/Kathmandu
    {  367,  992,     422,  7, }, // Asia/Kolkata
    {  367, 1178,     429,  5, }, // Asia/Macau
    {  367, 1202,     442,  6, }, // Asia/Yangon
    {  367, 1379,     448,  5, }, // Atlantic/Faroe
    {  367, 6784,     453, 10, }, // Atlantic/St_Helena
    {  367, 2419,     463,  4, }, // Europe/Kyiv
    {  367, 7174,     467,  7, }, // Indian/Reunion
    {  367, 3022,     474,  5, }, // Pacific/Chuuk
    {  367, 2961,     479,  7, }, // Pacific/Pohnpei
    {  368,   14,       0,  6, }, // Africa/Asmara Kikuyu/Latin/Kenya
    {  368, 4337,       6,  8, }, // Africa/Sao_Tome
    {  368, 4154,      14,  8, }, // America/Asuncion
    {  368,  237,      22,  8, }, // America/Atikokan
    {  368, 7586,      30, 17, }, // America/Bahia_Banderas
    {  368, 3770,      47,  6, }, // America/Cancun
    {  368, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  368, 5640,      66,  7, }, // America/Curacao
    {  368,  481,      73, 13, }, // America/Indiana/Knox
    {  368, 7924,      86, 16, }, // America/Indiana/Marengo
    {  368, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  368, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  368, 8001,     139, 14, }, // America/Indiana/Vevay
    {  368, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  368, 8049,     171, 16, }, // America/Indiana/Winamac
    {  368, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  368, 5849,     207, 22, }, // America/Lower_Princes
    {  368, 8183,     229,  6, }, // America/Merida
    {  368, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  368, 8292,     255, 20, }, // America/North_Dakota/Center
    {  368, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  368,  393,     298,  4, }, // America/Nuuk
    {  368, 6862,     302, 16, }, // America/Scoresbysund
    {  368, 8471,     318, 14, }, // America/St_Barthelemy
    {  368, 2045,     332, 10, }, // America/St_Johns
    {  368, 5780,     342,  9, }, // America/St_Kitts
    {  368, 5797,     351,  9, }, // America/St_Lucia
    {  368,  855,     360, 10, }, // America/St_Thomas
    {  368, 5830,     370, 11, }, // America/St_Vincent
    {  368, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  368, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  368, 1152,     413,  9, }, // Asia/Kathmandu
    {  368,  992,     422,  7, }, // Asia/Kolkata
    {  368, 1178,     429,  5, }, // Asia/Macau
    {  368, 1202,     442,  6, }, // Asia/Yangon
    {  368, 1379,     448,  5, }, // Atlantic/Faroe
    {  368, 6784,     453, 10, }, // Atlantic/St_Helena
    {  368, 2419,     463,  4, }, // Europe/Kyiv
    {  368, 7174,     467,  7, }, // Indian/Reunion
    {  368, 3022,     474,  5, }, // Pacific/Chuuk
    {  368, 2961,     479,  7, }, // Pacific/Pohnpei
    {  369,   14,       0,  6, }, // Africa/Asmara Kinyarwanda/Latin/Rwanda
    {  369, 4337,       6,  8, }, // Africa/Sao_Tome
    {  369, 4154,      14,  8, }, // America/Asuncion
    {  369,  237,      22,  8, }, // America/Atikokan
    {  369, 7586,      30, 17, }, // America/Bahia_Banderas
    {  369, 3770,      47,  6, }, // America/Cancun
    {  369, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  369, 5640,      66,  7, }, // America/Curacao
    {  369,  481,      73, 13, }, // America/Indiana/Knox
    {  369, 7924,      86, 16, }, // America/Indiana/Marengo
    {  369, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  369, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  369, 8001,     139, 14, }, // America/Indiana/Vevay
    {  369, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  369, 8049,     171, 16, }, // America/Indiana/Winamac
    {  369, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  369, 5849,     207, 22, }, // America/Lower_Princes
    {  369, 8183,     229,  6, }, // America/Merida
    {  369, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  369, 8292,     255, 20, }, // America/North_Dakota/Center
    {  369, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  369,  393,     298,  4, }, // America/Nuuk
    {  369, 6862,     302, 16, }, // America/Scoresbysund
    {  369, 8471,     318, 14, }, // America/St_Barthelemy
    {  369, 2045,     332, 10, }, // America/St_Johns
    {  369, 5780,     342,  9, }, // America/St_Kitts
    {  369, 5797,     351,  9, }, // America/St_Lucia
    {  369,  855,     360, 10, }, // America/St_Thomas
    {  369, 5830,     370, 11, }, // America/St_Vincent
    {  369, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  369, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  369, 1152,     413,  9, }, // Asia/Kathmandu
    {  369,  992,     422,  7, }, // Asia/Kolkata
    {  369, 1178,     429,  5, }, // Asia/Macau
    {  369, 1202,     442,  6, }, // Asia/Yangon
    {  369, 1379,     448,  5, }, // Atlantic/Faroe
    {  369, 6784,     453, 10, }, // Atlantic/St_Helena
    {  369, 2419,     463,  4, }, // Europe/Kyiv
    {  369, 7174,     467,  7, }, // Indian/Reunion
    {  369, 3022,     474,  5, }, // Pacific/Chuuk
    {  369, 2961,     479,  7, }, // Pacific/Pohnpei
    {  370, 6751,   28423,  7, }, // Africa/Abidjan Konkani/Devanagari/India
    {  370, 6723,  108948,  4, }, // Africa/Accra
    {  370, 4961,  108952, 10, }, // Africa/Addis_Ababa
    {  370, 7383,  108962, 10, }, // Africa/Algiers
    {  370,   14,   86785,  7, }, // Africa/Asmara
    {  370,   44,   86792,  6, }, // Africa/Bamako
    {  370, 5159,  108972,  5, }, // Africa/Bangui
    {  370, 6709,  108977,  6, }, // Africa/Banjul
    {  370, 7398,  108983,  5, }, // Africa/Bissau
    {  370, 4859,  108988,  9, }, // Africa/Blantyre
    {  370, 5189,  108997, 10, }, // Africa/Brazzaville
    {  370, 4824,  109007,  9, }, // Africa/Bujumbura
    {  370, 2242,   86844,  5, }, // Africa/Cairo
    {  370, 4019,   28524, 11, }, // Africa/Casablanca
    {  370, 7412,   86849,  5, }, // Africa/Ceuta
    {  370, 6736,  109016,  8, }, // Africa/Conakry
    {  370, 6803,   86862,  4, }, // Africa/Dakar
    {  370, 5032,  109024, 10, }, // Africa/Dar_es_Salaam
    {  370, 4945,   86877,  6, }, // Africa/Djibouti
    {  370, 5145,  109034,  5, }, // Africa/Douala
    {  370, 5068,  109039,  7, }, // Africa/El_Aaiun
    {  370, 6816,  109046,  8, }, // Africa/Freetown
    {  370, 4808,   86905,  7, }, // Africa/Gaborone
    {  370, 4917,   86912,  5, }, // Africa/Harare
    {  370, 4381,  109054, 11, }, // Africa/Johannesburg
    {  370, 4401,   28613,  4, }, // Africa/Juba
    {  370, 5053,   28617,  8, }, // Africa/Kampala
    {  370, 4426,  109065,  7, }, // Africa/Khartoum
    {  370, 4875,   86933,  6, }, // Africa/Kigali
    {  370, 5208,  109072,  7, }, // Africa/Kinshasa
    {  370, 4646,   86946,  5, }, // Africa/Lagos
    {  370, 5238,   86951, 10, }, // Africa/Libreville
    {  370, 6832,   86961,  3, }, // Africa/Lome
    {  370, 5113,   86964,  6, }, // Africa/Luanda
    {  370, 4841,   86970,  9, }, // Africa/Lubumbashi
    {  370, 4903,   28680,  6, }, // Africa/Lusaka
    {  370, 5224,   86979,  5, }, // Africa/Malabo
    {  370, 4889,   86984,  6, }, // Africa/Maputo
    {  370, 5099,  109079,  6, }, // Africa/Maseru
    {  370, 5084,   86990,  7, }, // Africa/Mbabane
    {  370, 5015,  109085,  8, }, // Africa/Mogadishu
    {  370, 6997,   87005,  9, }, // Africa/Monrovia
    {  370, 3755,  109093,  7, }, // Africa/Nairobi
    {  370, 5173,  109100,  8, }, // Africa/Ndjamena
    {  370, 5256,   87030,  6, }, // Africa/Niamey
    {  370, 6766,  109108,  7, }, // Africa/Nouakchott
    {  370, 6690,  109115,  7, }, // Africa/Ouagadougou
    {  370, 5127,   87050, 11, }, // Africa/Porto-Novo
    {  370, 4337,   87061,  7, }, // Africa/Sao_Tome
    {  370, 2732,   87068,  8, }, // Africa/Tripoli
    {  370, 6503,  109122,  5, }, // Africa/Tunis
    {  370, 4054,   87076,  7, }, // Africa/Windhoek
    {  370,  132,  109127,  4, }, // America/Adak
    {  370, 3183,  109131,  5, }, // America/Anchorage
    {  370, 5524,  109136,  6, }, // America/Anguilla
    {  370, 5541,  109142,  7, }, // America/Antigua
    {  370, 4456,  109149,  8, }, // America/Araguaina
    {  370,  166,  109157, 12, }, // America/Argentina/Buenos_Aires
    {  370,   91,  109169,  9, }, // America/Argentina/Catamarca
    {  370,  270,  109178,  7, }, // America/Argentina/Cordoba
    {  370,  441,  109185,  5, }, // America/Argentina/Jujuy
    {  370, 7425,  109190,  9, }, // America/Argentina/La_Rioja
    {  370,  565,  109199,  8, }, // America/Argentina/Mendoza
    {  370, 7452,  109207, 11, }, // America/Argentina/Rio_Gallegos
    {  370, 7483,   28896,  6, }, // America/Argentina/Salta
    {  370, 7507,  109218,  8, }, // America/Argentina/San_Juan
    {  370, 5497,   28911,  8, }, // America/Argentina/San_Luis
    {  370, 7534,  109226,  7, }, // America/Argentina/Tucuman
    {  370, 7560,   87184,  7, }, // America/Argentina/Ushuaia
    {  370, 5557,  109233,  5, }, // America/Aruba
    {  370, 4154,  109238,  7, }, // America/Asuncion
    {  370,  237,  109245,  8, }, // America/Atikokan
    {  370, 3581,   87212,  5, }, // America/Bahia
    {  370, 7586,  109253, 12, }, // America/Bahia_Banderas
    {  370, 5571,   87232,  8, }, // America/Barbados
    {  370, 7609,   28985,  5, }, // America/Belem
    {  370, 5309,  109265,  5, }, // America/Belize
    {  370, 7623,  109270, 14, }, // America/Blanc-Sablon
    {  370, 7644,  109284, 10, }, // America/Boa_Vista
    {  370, 4263,   87269,  6, }, // America/Bogota
    {  370, 7662,  109294,  4, }, // America/Boise
    {  370, 7676,  109298, 11, }, // America/Cambridge_Bay
    {  370, 7698,  109309, 13, }, // America/Campo_Grande
    {  370, 3770,  109322,  6, }, // America/Cancun
    {  370, 4596,  109328,  5, }, // America/Caracas
    {  370, 4247,  109333,  4, }, // America/Cayenne
    {  370, 5398,   29073,  5, }, // America/Cayman
    {  370, 3251,   87320,  6, }, // America/Chicago
    {  370, 7719,   87326,  8, }, // America/Chihuahua
    {  370, 7737,  109337, 13, }, // America/Ciudad_Juarez
    {  370, 5324,  109350, 11, }, // America/Costa_Rica
    {  370, 7759,  109361,  9, }, // America/Creston
    {  370, 3693,  109370,  6, }, // America/Cuiaba
    {  370, 5640,   87368,  7, }, // America/Curacao
    {  370, 7775,  109376, 12, }, // America/Danmarkshavn
    {  370, 7796,  109388,  4, }, // America/Dawson
    {  370, 7811,  109392, 10, }, // America/Dawson_Creek
    {  370,  805,  109402,  7, }, // America/Denver
    {  370, 3351,  109409,  8, }, // America/Detroit
    {  370, 5656,  109417,  8, }, // America/Dominica
    {  370,  893,  109425,  7, }, // America/Edmonton
    {  370, 7832,   29182,  7, }, // America/Eirunepe
    {  370, 5343,  109432, 12, }, // America/El_Salvador
    {  370, 7849,  109444, 12, }, // America/Fort_Nelson
    {  370, 7869,  109456, 10, }, // America/Fortaleza
    {  370, 7887,   29219,  8, }, // America/Glace_Bay
    {  370, 6844,  109466,  6, }, // America/Goose_Bay
    {  370, 4514,  109472, 11, }, // America/Grand_Turk
    {  370, 5687,  109483,  8, }, // America/Grenada
    {  370, 5703,   87494,  9, }, // America/Guadeloupe
    {  370, 3662,   29265, 10, }, // America/Guatemala
    {  370, 4743,  109491,  9, }, // America/Guayaquil
    {  370, 6895,   29283,  6, }, // America/Guyana
    {  370, 1923,  109500,  9, }, // America/Halifax
    {  370, 2221,   87527,  5, }, // America/Havana
    {  370, 7905,  109509,  9, }, // America/Hermosillo
    {  370,  348,  109518, 13, }, // America/Indiana/Indianapolis
    {  370,  481,  109531, 15, }, // America/Indiana/Knox
    {  370, 7924,   87571, 17, }, // America/Indiana/Marengo
    {  370, 7948,  109546, 20, }, // America/Indiana/Petersburg
    {  370, 7975,   87608, 18, }, // America/Indiana/Tell_City
    {  370, 8001,  109566, 14, }, // America/Indiana/Vevay
    {  370, 8023,  109580, 17, }, // America/Indiana/Vincennes
    {  370, 8049,  109597, 17, }, // America/Indiana/Winamac
    {  370, 8073,  109614,  6, }, // America/Inuvik
    {  370,  660,  109620,  8, }, // America/Iqaluit
    {  370, 2665,   87682,  5, }, // America/Jamaica
    {  370, 5282,  109628,  4, }, // America/Juneau
    {  370,  521,  109632,  8, }, // America/Kentucky/Louisville
    {  370, 8088,  109640, 19, }, // America/Kentucky/Monticello
    {  370, 5621,  109659, 10, }, // America/Kralendijk
    {  370, 4278,  109669,  6, }, // America/La_Paz
    {  370, 7132,  109675,  4, }, // America/Lima
    {  370, 3390,  109679, 10, }, // America/Los_Angeles
    {  370, 5849,  109689, 22, }, // America/Lower_Princes
    {  370, 8116,  109711,  6, }, // America/Maceio
    {  370, 8131,  109717,  8, }, // America/Managua
    {  370, 1892,   87783,  4, }, // America/Manaus
    {  370, 5814,  109725,  7, }, // America/Marigot
    {  370, 5722,   87794,  9, }, // America/Martinique
    {  370, 8147,   87803,  9, }, // America/Matamoros
    {  370, 2779,  109732,  9, }, // America/Mazatlan
    {  370, 8165,   87821,  8, }, // America/Menominee
    {  370, 8183,   29617,  6, }, // America/Merida
    {  370, 8198,  109741, 12, }, // America/Metlakatla
    {  370, 2811,  109753, 12, }, // America/Mexico_City
    {  370, 4293,  109765,  9, }, // America/Miquelon
    {  370, 8217,  109774,  8, }, // America/Moncton
    {  370, 8233,  109782,  7, }, // America/Monterrey
    {  370, 4000,  109789, 13, }, // America/Montevideo
    {  370, 5741,  109802, 10, }, // America/Montserrat
    {  370, 5383,  109812,  4, }, // America/Nassau
    {  370, 3294,  109816, 10, }, // America/New_York
    {  370, 8251,  109826,  4, }, // America/Nome
    {  370, 1834,   87910,  8, }, // America/Noronha
    {  370, 8264,  109830, 23, }, // America/North_Dakota/Beulah
    {  370, 8292,  109853, 18, }, // America/North_Dakota/Center
    {  370, 8320,  109871, 24, }, // America/North_Dakota/New_Salem
    {  370,  393,   29774,  3, }, // America/Nuuk
    {  370, 8351,  109895,  7, }, // America/Ojinaga
    {  370, 5413,   87993,  5, }, // America/Panama
    {  370, 6106,  109902,  9, }, // America/Paramaribo
    {  370, 3224,  109911,  7, }, // America/Phoenix
    {  370, 3847,  109918, 15, }, // America/Port-au-Prince
    {  370, 5871,  109933, 14, }, // America/Port_of_Spain
    {  370, 8367,   88045, 13, }, // America/Porto_Velho
    {  370, 5760,  109947, 11, }, // America/Puerto_Rico
    {  370, 3932,  109958, 12, }, // America/Punta_Arenas
    {  370, 8387,  109970, 11, }, // America/Rankin_Inlet
    {  370, 8408,  109981,  5, }, // America/Recife
    {  370, 1979,  109986,  6, }, // America/Regina
    {  370, 8423,  109992,  9, }, // America/Resolute
    {  370,  695,   88109, 12, }, // America/Rio_Branco
    {  370, 8440,  110001,  8, }, // America/Santarem
    {  370, 2165,  110009,  8, }, // America/Santiago
    {  370, 6044,  110017, 13, }, // America/Santo_Domingo
    {  370, 1862,  110030,  9, }, // America/Sao_Paulo
    {  370, 6862,  110039, 13, }, // America/Scoresbysund
    {  370, 8457,   88173,  6, }, // America/Sitka
    {  370, 8471,   88179, 15, }, // America/St_Barthelemy
    {  370, 2045,  110052, 10, }, // America/St_Johns
    {  370, 5780,   88203, 10, }, // America/St_Kitts
    {  370, 5797,  110062, 11, }, // America/St_Lucia
    {  370,  855,   88224,  9, }, // America/St_Thomas
    {  370, 5830,  110073, 13, }, // America/St_Vincent
    {  370, 8493,  110086, 12, }, // America/Swift_Current
    {  370, 5363,  110098, 11, }, // America/Tegucigalpa
    {  370, 5673,  110109,  4, }, // America/Thule
    {  370,  313,   88275,  7, }, // America/Tijuana
    {  370,  608,  110113,  6, }, // America/Toronto
    {  370, 5605,  110119,  7, }, // America/Tortola
    {  370, 2077,  110126, 10, }, // America/Vancouver
    {  370, 2128,  110136, 11, }, // America/Whitehorse
    {  370,  734,   88314,  7, }, // America/Winnipeg
    {  370, 8515,  110147,  7, }, // America/Yakutat
    {  370, 5918,   30132,  4, }, // Antarctica/Casey
    {  370, 6027,  110154,  7, }, // Antarctica/Davis
    {  370, 6066,  110161, 14, }, // Antarctica/DumontDUrville
    {  370, 8531,  110175,  8, }, // Antarctica/Macquarie
    {  370, 7029,   88359,  4, }, // Antarctica/Mawson
    {  370, 7076,  110183,  9, }, // Antarctica/McMurdo
    {  370, 8552,  110192,  7, }, // Antarctica/Palmer
    {  370, 7189,   88376,  6, }, // Antarctica/Rothera
    {  370, 7258,   88382,  6, }, // Antarctica/Syowa
    {  370, 8570,   88388,  5, }, // Antarctica/Troll
    {  370, 7337,   88393,  7, }, // Antarctica/Vostok
    {  370, 1413,  110199, 13, }, // Arctic/Longyearbyen
    {  370, 5487,  110212,  3, }, // Asia/Aden
    {  370, 5297,   88417,  7, }, // Asia/Almaty
    {  370, 3870,   88424,  6, }, // Asia/Amman
    {  370, 5428,  110215,  6, }, // Asia/Anadyr
    {  370, 5440,  110221,  5, }, // Asia/Aqtau
    {  370, 5270,  110226,  7, }, // Asia/Aqtobe
    {  370,  964,  110233,  7, }, // Asia/Ashgabat
    {  370, 8587,  110240,  7, }, // Asia/Atyrau
    {  370, 3509,   88447,  5, }, // Asia/Baghdad
    {  370, 5451,  110247,  6, }, // Asia/Bahrain
    {  370, 3555,   29113,  4, }, // Asia/Baku
    {  370, 4368,  110253,  5, }, // Asia/Bangkok
    {  370, 3473,  110258,  5, }, // Asia/Barnaul
    {  370, 3988,  110263,  5, }, // Asia/Beirut
    {  370, 3680,   30295,  7, }, // Asia/Bishkek
    {  370, 5906,   88474,  7, }, // Asia/Brunei
    {  370, 4503,  110268,  4, }, // Asia/Chita
    {  370, 5963,  110272,  9, }, // Asia/Choibalsan
    {  370, 4413,   88495,  6, }, // Asia/Colombo
    {  370, 4442,  110281,  6, }, // Asia/Damascus
    {  370, 1060,   88508,  4, }, // Asia/Dhaka
    {  370, 6125,  110287,  4, }, // Asia/Dili
    {  370, 3498,  110291,  4, }, // Asia/Dubai
    {  370, 6092,   30349,  8, }, // Asia/Dushanbe
    {  370, 8599,  110295,  9, }, // Asia/Famagusta
    {  370, 8614,  110304,  4, }, // Asia/Gaza
    {  370, 4697,  110308,  7, }, // Asia/Hebron
    {  370, 1226,   88550, 11, }, // Asia/Ho_Chi_Minh
    {  370, 2576,   88566,  9, }, // Asia/Hong_Kong
    {  370, 4673,  110315,  6, }, // Asia/Hovd
    {  370, 4086,  110321,  9, }, // Asia/Irkutsk
    {  370, 6969,   88590,  7, }, // Asia/Jakarta
    {  370, 6955,  110330,  8, }, // Asia/Jayapura
    {  370, 1257,  110338,  9, }, // Asia/Jerusalem
    {  370, 3462,   88603,  5, }, // Asia/Kabul
    {  370, 4218,  110347,  9, }, // Asia/Kamchatka
    {  370, 4141,   30440,  5, }, // Asia/Karachi
    {  370, 1152,   88615,  8, }, // Asia/Kathmandu
    {  370, 8624,  110356,  8, }, // Asia/Khandyga
    {  370,  992,   88630,  7, }, // Asia/Kolkata
    {  370, 4099,   88637, 15, }, // Asia/Krasnoyarsk
    {  370, 4761,  110364, 11, }, // Asia/Kuala_Lumpur
    {  370, 5893,  110375,  6, }, // Asia/Kuching
    {  370, 5464,  110381,  5, }, // Asia/Kuwait
    {  370, 1178,  110386,  4, }, // Asia/Macau
    {  370, 3919,  110390,  6, }, // Asia/Magadan
    {  370, 1316,  110396,  8, }, // Asia/Makassar
    {  370, 7145,  110404,  5, }, // Asia/Manila
    {  370, 6883,   88672,  5, }, // Asia/Muscat
    {  370, 2446,   88677,  8, }, // Asia/Nicosia
    {  370, 8638,  110409, 14, }, // Asia/Novokuznetsk
    {  370, 4037,   88701, 13, }, // Asia/Novosibirsk
    {  370, 4131,   88714,  6, }, // Asia/Omsk
    {  370, 7108,   88720,  3, }, // Asia/Oral
    {  370, 6924,  110423, 11, }, // Asia/Phnom_Penh
    {  370, 8656,  110434, 10, }, // Asia/Pontianak
    {  370, 4116,   88741, 10, }, // Asia/Pyongyang
    {  370, 5476,  110444,  4, }, // Asia/Qatar
    {  370, 8671,  110448,  9, }, // Asia/Qostanay
    {  370, 4171,  110457, 12, }, // Asia/Qyzylorda
    {  370, 3486,   30628,  5, }, // Asia/Riyadh
    {  370, 4310,  110469,  5, }, // Asia/Sakhalin
    {  370, 7208,   88784,  6, }, // Asia/Samarkand
    {  370, 3116,   88102,  3, }, // Asia/Seoul
    {  370, 1020,  110474,  6, }, // Asia/Shanghai
    {  370, 3137,  110480,  8, }, // Asia/Singapore
    {  370, 4199,  110488, 16, }, // Asia/Srednekolymsk
    {  370, 3100,  110504,  5, }, // Asia/Taipei
    {  370, 4683,  110509,  7, }, // Asia/Tashkent
    {  370, 3817,  110516,  7, }, // Asia/Tbilisi
    {  370, 2638,   30704,  6, }, // Asia/Tehran
    {  370, 1284,  110523,  6, }, // Asia/Thimphu
    {  370, 2687,  110529,  6, }, // Asia/Tokyo
    {  370, 4474,  110535,  6, }, // Asia/Tomsk
    {  370, 1346,  110541,  8, }, // Asia/Ulaanbaatar
    {  370, 1126,  110549,  7, }, // Asia/Urumqi
    {  370, 8685,  110556,  9, }, // Asia/Ust-Nera
    {  370, 6940,  110565, 11, }, // Asia/Vientiane
    {  370, 4612,   88886, 13, }, // Asia/Vladivostok
    {  370, 4730,  110576,  8, }, // Asia/Yakutsk
    {  370, 1202,  110584,  6, }, // Asia/Yangon
    {  370, 3785,   88899, 13, }, // Asia/Yekaterinburg
    {  370, 3649,   88912,  7, }, // Asia/Yerevan
    {  370, 3565,  110590,  6, }, // Atlantic/Azores
    {  370, 5588,  110596,  7, }, // Atlantic/Bermuda
    {  370, 6591,  110603,  5, }, // Atlantic/Canary
    {  370, 3629,  110608,  9, }, // Atlantic/Cape_Verde
    {  370, 1379,  110617,  4, }, // Atlantic/Faroe
    {  370, 8699,  110621,  5, }, // Atlantic/Madeira
    {  370, 2614,  110626,  9, }, // Atlantic/Reykjavik
    {  370, 7235,   88965, 15, }, // Atlantic/South_Georgia
    {  370, 6784,  110635, 11, }, // Atlantic/St_Helena
    {  370, 6607,  110646,  8, }, // Atlantic/Stanley
    {  370, 1654,   88998,  6, }, // Australia/Adelaide
    {  370, 1619,  110654,  9, }, // Australia/Brisbane
    {  370, 1783,   89012, 10, }, // Australia/Broken_Hill
    {  370, 1567,   89022,  7, }, // Australia/Darwin
    {  370, 3539,  110663,  5, }, // Australia/Eucla
    {  370, 1500,   89029,  7, }, // Australia/Hobart
    {  370, 8716,  110668,  7, }, // Australia/Lindeman
    {  370, 1531,   89044, 10, }, // Australia/Lord_Howe
    {  370, 1711,  110675,  7, }, // Australia/Melbourne
    {  370, 1746,   89062,  4, }, // Australia/Perth
    {  370, 1447,   30962,  5, }, // Australia/Sydney
    {  370, 6390,  110682, 10, }, // Europe/Amsterdam
    {  370, 6149,   89076,  6, }, // Europe/Andorra
    {  370, 3522,  110692,  8, }, // Europe/Astrakhan
    {  370, 6577,  110700,  6, }, // Europe/Athens
    {  370, 6285,   31000,  8, }, // Europe/Belgrade
    {  370, 4659,   89097,  6, }, // Europe/Berlin
    {  370, 6437,  110706, 14, }, // Europe/Bratislava
    {  370, 6178,  110720,  9, }, // Europe/Brussels
    {  370, 3830,  110729,  9, }, // Europe/Bucharest
    {  370, 3708,   89134,  9, }, // Europe/Budapest
    {  370, 8751,  110738,  8, }, // Europe/Busingen
    {  370, 2475,  110746,  7, }, // Europe/Chisinau
    {  370, 6238,  110753,  8, }, // Europe/Copenhagen
    {  370, 2260,  110761,  6, }, // Europe/Dublin
    {  370, 6256,   89174, 10, }, // Europe/Gibraltar
    {  370, 8767,   89184,  6, }, // Europe/Guernsey
    {  370, 6561,   89190,  8, }, // Europe/Helsinki
    {  370, 8783,  110767, 11, }, // Europe/Isle_of_Man
    {  370, 1097,  110778,  8, }, // Europe/Istanbul
    {  370, 8802,   89218,  5, }, // Europe/Jersey
    {  370, 3881,  110786, 12, }, // Europe/Kaliningrad
    {  370, 8924,   89235,  5, }, // Europe/Kirov
    {  370, 2419,   89240,  3, }, // Europe/Kyiv
    {  370, 3078,   31158,  6, }, // Europe/Lisbon
    {  370, 6455,  110798,  9, }, // Europe/Ljubljana
    {  370, 2393,  110807,  4, }, // Europe/London
    {  370, 6314,  110811, 10, }, // Europe/Luxembourg
    {  370, 4794,  110821,  7, }, // Europe/Madrid
    {  370, 6346,   31199,  6, }, // Europe/Malta
    {  370, 6531,  110828,  9, }, // Europe/Mariehamn
    {  370, 3595,   89284,  6, }, // Europe/Minsk
    {  370, 6359,  110837,  6, }, // Europe/Monaco
    {  370, 3443,   89296,  6, }, // Europe/Moscow
    {  370, 6407,   89302,  5, }, // Europe/Oslo
    {  370, 4186,  110843,  5, }, // Europe/Paris
    {  370, 6373,  110848,  9, }, // Europe/Podgorica
    {  370, 6224,   89317,  5, }, // Europe/Prague
    {  370, 8816,   89322,  4, }, // Europe/Riga
    {  370, 6273,   88949,  3, }, // Europe/Rome
    {  370, 4233,   89326,  5, }, // Europe/Samara
    {  370, 6419,  110857, 10, }, // Europe/San_Marino
    {  370, 6194,  110867,  8, }, // Europe/Sarajevo
    {  370, 4353,  110875,  7, }, // Europe/Saratov
    {  370, 8828,  110882,  9, }, // Europe/Simferopol
    {  370, 6332,   89368,  8, }, // Europe/Skopje
    {  370, 6548,  110891,  6, }, // Europe/Sofia
    {  370, 6472,   89383,  8, }, // Europe/Stockholm
    {  370, 8846,  110897,  5, }, // Europe/Tallinn
    {  370, 6135,  110902,  6, }, // Europe/Tirane
    {  370, 8861,   89402, 12, }, // Europe/Ulyanovsk
    {  370, 6301,  110908,  4, }, // Europe/Vaduz
    {  370, 6516,  110912,  6, }, // Europe/Vatican
    {  370, 6164,  110918,  6, }, // Europe/Vienna
    {  370, 8878,   89431,  8, }, // Europe/Vilnius
    {  370, 4629,  110924, 13, }, // Europe/Volgograd
    {  370, 3055,  110937,  5, }, // Europe/Warsaw
    {  370, 6210,  110942,  7, }, // Europe/Zagreb
    {  370, 6489,  110949,  5, }, // Europe/Zurich
    {  370, 4980,  110954, 12, }, // Indian/Antananarivo
    {  370, 6910,  110966,  5, }, // Indian/Chagos
    {  370, 5979,   89488,  7, }, // Indian/Christmas
    {  370, 5996,   89495,  5, }, // Indian/Cocos
    {  370, 4931,   89500,  6, }, // Indian/Comoro
    {  370, 6624,  110971,  8, }, // Indian/Kerguelen
    {  370, 7223,   31451,  4, }, // Indian/Mahe
    {  370, 7013,   89513,  6, }, // Indian/Maldives
    {  370, 3971,  110979,  6, }, // Indian/Mauritius
    {  370, 5000,  110985,  5, }, // Indian/Mayotte
    {  370, 7174,  110990,  8, }, // Indian/Reunion
    {  370, 4324,  110998,  5, }, // Pacific/Apia
    {  370,  932,  111003,  5, }, // Pacific/Auckland
    {  370, 3608,  111008,  9, }, // Pacific/Bougainville
    {  370, 2863,  111017,  4, }, // Pacific/Chatham
    {  370, 3022,  111021,  3, }, // Pacific/Chuuk
    {  370, 2201,   89566,  5, }, // Pacific/Easter
    {  370, 7323,  111024,  4, }, // Pacific/Efate
    {  370, 7290,  111028,  7, }, // Pacific/Fakaofo
    {  370, 3804,   31537,  4, }, // Pacific/Fiji
    {  370, 7306,  111035,  8, }, // Pacific/Funafuti
    {  370, 6641,  111043,  9, }, // Pacific/Galapagos
    {  370, 6659,  111052,  8, }, // Pacific/Gambier
    {  370, 3724,  111060, 12, }, // Pacific/Guadalcanal
    {  370, 5935,   29571,  4, }, // Pacific/Guam
    {  370, 2897,  111072,  7, }, // Pacific/Kanton
    {  370, 3900,  111079, 10, }, // Pacific/Kiritimati
    {  370, 6982,  111089,  5, }, // Pacific/Kosrae
    {  370, 2708,  111094,  8, }, // Pacific/Kwajalein
    {  370, 4779,   89659,  6, }, // Pacific/Majuro
    {  370, 3953,  111102,  8, }, // Pacific/Marquesas
    {  370, 8909,   89673,  5, }, // Pacific/Midway
    {  370, 7047,  111110,  5, }, // Pacific/Nauru
    {  370, 7095,   89682,  4, }, // Pacific/Niue
    {  370, 4070,  111115,  6, }, // Pacific/Norfolk
    {  370, 7061,  111121,  5, }, // Pacific/Noumea
    {  370, 2991,   89699,  9, }, // Pacific/Pago_Pago
    {  370, 7118,   89708,  4, }, // Pacific/Palau
    {  370, 7157,  111126,  7, }, // Pacific/Pitcairn
    {  370, 2961,  111133,  8, }, // Pacific/Pohnpei
    {  370, 4709,  111141, 14, }, // Pacific/Port_Moresby
    {  370, 6009,   89732,  9, }, // Pacific/Rarotonga
    {  370, 5948,   89741,  6, }, // Pacific/Saipan
    {  370, 7275,   89747,  6, }, // Pacific/Tahiti
    {  370, 6675,  111155,  5, }, // Pacific/Tarawa
    {  370, 4485,   89759,  9, }, // Pacific/Tongatapu
    {  370, 7355,  111160,  3, }, // Pacific/Wake
    {  370, 7368,   31739,  5, }, // Pacific/Wallis
    {  371, 6751,  111163,  3, }, // Africa/Abidjan Korean/Korean/South Korea
    {  371, 6723,  111166,  3, }, // Africa/Accra
    {  371, 4961,  111169,  6, }, // Africa/Addis_Ababa
    {  371, 7383,  111175,  2, }, // Africa/Algiers
    {  371,   14,  111177,  4, }, // Africa/Asmara
    {  371,   44,  111181,  3, }, // Africa/Bamako
    {  371, 5159,  111184,  2, }, // Africa/Bangui
    {  371, 6709,  111186,  2, }, // Africa/Banjul
    {  371, 7398,  111188,  3, }, // Africa/Bissau
    {  371, 4859,  111191,  5, }, // Africa/Blantyre
    {  371, 5189,  111196,  4, }, // Africa/Brazzaville
    {  371, 4824,  111200,  4, }, // Africa/Bujumbura
    {  371, 2242,  111204,  3, }, // Africa/Cairo
    {  371, 4019,  111207,  5, }, // Africa/Casablanca
    {  371, 7412,  111212,  3, }, // Africa/Ceuta
    {  371, 6736,  111215,  4, }, // Africa/Conakry
    {  371, 6803,  111219,  3, }, // Africa/Dakar
    {  371, 5032,  111222,  6, }, // Africa/Dar_es_Salaam
    {  371, 4945,  111228,  3, }, // Africa/Djibouti
    {  371, 5145,  111231,  3, }, // Africa/Douala
    {  371, 5068,  111234,  4, }, // Africa/El_Aaiun
    {  371, 6816,  111238,  4, }, // Africa/Freetown
    {  371, 4808,  111242,  4, }, // Africa/Gaborone
    {  371, 4917,  111246,  3, }, // Africa/Harare
    {  371, 4381,  111249,  6, }, // Africa/Johannesburg
    {  371, 4401,  111255,  2, }, // Africa/Juba
    {  371, 5053,  111257,  3, }, // Africa/Kampala
    {  371, 4426,  111260,  3, }, // Africa/Khartoum
    {  371, 4875,  111263,  3, }, // Africa/Kigali
    {  371, 5208,  111266,  3, }, // Africa/Kinshasa
    {  371, 4646,  111269,  3, }, // Africa/Lagos
    {  371, 5238,  111272,  4, }, // Africa/Libreville
    {  371, 6832,  111276,  2, }, // Africa/Lome
    {  371, 5113,  111278,  3, }, // Africa/Luanda
    {  371, 4841,  111281,  4, }, // Africa/Lubumbashi
    {  371, 4903,  111285,  3, }, // Africa/Lusaka
    {  371, 5224,  111288,  3, }, // Africa/Malabo
    {  371, 4889,  111291,  3, }, // Africa/Maputo
    {  371, 5099,  111294,  3, }, // Africa/Maseru
    {  371, 5084,  111297,  4, }, // Africa/Mbabane
    {  371, 5015,  111301,  4, }, // Africa/Mogadishu
    {  371, 6997,  111305,  4, }, // Africa/Monrovia
    {  371, 3755,  111309,  4, }, // Africa/Nairobi
    {  371, 5173,  111313,  4, }, // Africa/Ndjamena
    {  371, 5256,  111317,  3, }, // Africa/Niamey
    {  371, 6766,  111320,  4, }, // Africa/Nouakchott
    {  371, 6690,  111324,  4, }, // Africa/Ouagadougou
    {  371, 5127,  111328,  5, }, // Africa/Porto-Novo
    {  371, 4337,  111333,  3, }, // Africa/Sao_Tome
    {  371, 2732,  111336,  4, }, // Africa/Tripoli
    {  371, 6503,  111340,  3, }, // Africa/Tunis
    {  371, 4054,  111343,  4, }, // Africa/Windhoek
    {  371,  132,  111347,  3, }, // America/Adak
    {  371, 3183,  111350,  4, }, // America/Anchorage
    {  371, 5524,  111354,  3, }, // America/Anguilla
    {  371, 5541,  111357,  3, }, // America/Antigua
    {  371, 4456,  111360,  5, }, // America/Araguaina
    {  371,  166,  111365,  9, }, // America/Argentina/Buenos_Aires
    {  371,   91,  111374,  5, }, // America/Argentina/Catamarca
    {  371,  270,  111379,  4, }, // America/Argentina/Cordoba
    {  371,  441,  111383,  3, }, // America/Argentina/Jujuy
    {  371, 7425,  111386,  5, }, // America/Argentina/La_Rioja
    {  371,  565,  111391,  3, }, // America/Argentina/Mendoza
    {  371, 7452,  111394,  7, }, // America/Argentina/Rio_Gallegos
    {  371, 7483,  111401,  2, }, // America/Argentina/Salta
    {  371, 7507,  111403,  3, }, // America/Argentina/San_Juan
    {  371, 5497,  111406,  4, }, // America/Argentina/San_Luis
    {  371, 7534,  111410,  3, }, // America/Argentina/Tucuman
    {  371, 7560,  111413,  5, }, // America/Argentina/Ushuaia
    {  371, 5557,  111418,  3, }, // America/Aruba
    {  371, 4154,  111421,  4, }, // America/Asuncion
    {  371,  237,  111425,  4, }, // America/Atikokan
    {  371, 3581,  111429,  3, }, // America/Bahia
    {  371, 7586,  111432,  8, }, // America/Bahia_Banderas
    {  371, 5571,  111440,  5, }, // America/Barbados
    {  371, 7609,  111445,  2, }, // America/Belem
    {  371, 5309,  111447,  3, }, // America/Belize
    {  371, 7623,  111450,  6, }, // America/Blanc-Sablon
    {  371, 7644,  111456,  6, }, // America/Boa_Vista
    {  371, 4263,  111462,  3, }, // America/Bogota
    {  371, 7662,  111465,  3, }, // America/Boise
    {  371, 7676,  111468,  8, }, // America/Cambridge_Bay
    {  371, 7698,  111476,  6, }, // America/Campo_Grande
    {  371, 3770,  111482,  2, }, // America/Cancun
    {  371, 4596,  111484,  4, }, // America/Caracas
    {  371, 4247,  111488,  2, }, // America/Cayenne
    {  371, 5398,  111490,  3, }, // America/Cayman
    {  371, 3251,  111493,  3, }, // America/Chicago
    {  371, 7719,  111496,  3, }, // America/Chihuahua
    {  371, 7737,  111499,  8, }, // America/Ciudad_Juarez
    {  371, 5324,  111507,  5, }, // America/Costa_Rica
    {  371, 7759,  111512,  4, }, // America/Creston
    {  371, 3693,  111516,  4, }, // America/Cuiaba
    {  371, 5640,  111520,  3, }, // America/Curacao
    {  371, 7775,  111523,  5, }, // America/Danmarkshavn
    {  371, 7796,  111528,  2, }, // America/Dawson
    {  371, 7811,  111530,  4, }, // America/Dawson_Creek
    {  371,  805,  111534,  2, }, // America/Denver
    {  371, 3351,  111536,  5, }, // America/Detroit
    {  371, 5656,  111541,  4, }, // America/Dominica
    {  371,  893,  111545,  4, }, // America/Edmonton
    {  371, 7832,  111549,  5, }, // America/Eirunepe
    {  371, 5343,  111554,  5, }, // America/El_Salvador
    {  371, 7849,  111559,  5, }, // America/Fort_Nelson
    {  371, 7869,  111564,  5, }, // America/Fortaleza
    {  371, 7887,  111569,  5, }, // America/Glace_Bay
    {  371, 6844,  111574,  4, }, // America/Goose_Bay
    {  371, 4514,  111578,  6, }, // America/Grand_Turk
    {  371, 5687,  111584,  4, }, // America/Grenada
    {  371, 5703,  111588,  4, }, // America/Guadeloupe
    {  371, 3662,  111592,  4, }, // America/Guatemala
    {  371, 4743,  111596,  3, }, // America/Guayaquil
    {  371, 6895,  111599,  4, }, // America/Guyana
    {  371, 1923,  111603,  4, }, // America/Halifax
    {  371, 2221,  111607,  3, }, // America/Havana
    {  371, 7905,  111610,  5, }, // America/Hermosillo
    {  371,  348,  111615,  7, }, // America/Indiana/Indianapolis
    {  371,  481,  111622,  9, }, // America/Indiana/Knox
    {  371, 7924,  111631, 10, }, // America/Indiana/Marengo
    {  371, 7948,  111641, 11, }, // America/Indiana/Petersburg
    {  371, 7975,  111652, 10, }, // America/Indiana/Tell_City
    {  371, 8001,  111662,  9, }, // America/Indiana/Vevay
    {  371, 8023,  111671, 10, }, // America/Indiana/Vincennes
    {  371, 8049,  111681, 10, }, // America/Indiana/Winamac
    {  371, 8073,  111691,  3, }, // America/Inuvik
    {  371,  660,  111694,  5, }, // America/Iqaluit
    {  371, 2665,  111699,  4, }, // America/Jamaica
    {  371, 5282,  111703,  2, }, // America/Juneau
    {  371,  521,  111705,  3, }, // America/Kentucky/Louisville
    {  371, 8088,  111708, 10, }, // America/Kentucky/Monticello
    {  371, 5621,  111718,  6, }, // America/Kralendijk
    {  371, 4278,  111724,  3, }, // America/La_Paz
    {  371, 7132,  111727,  2, }, // America/Lima
    {  371, 3390,  111729,  6, }, // America/Los_Angeles
    {  371, 5849,  111735,  9, }, // America/Lower_Princes
    {  371, 8116,  111744,  4, }, // America/Maceio
    {  371, 8131,  111748,  3, }, // America/Managua
    {  371, 1892,  111751,  4, }, // America/Manaus
    {  371, 5814,  111755,  3, }, // America/Marigot
    {  371, 5722,  111758,  4, }, // America/Martinique
    {  371, 8147,  111762,  5, }, // America/Matamoros
    {  371, 2779,  111767,  4, }, // America/Mazatlan
    {  371, 8165,  111771,  4, }, // America/Menominee
    {  371, 8183,  111775,  3, }, // America/Merida
    {  371, 8198,  111778,  6, }, // America/Metlakatla
    {  371, 2811,  111784,  6, }, // America/Mexico_City
    {  371, 4293,  111790,  3, }, // America/Miquelon
    {  371, 8217,  111793,  2, }, // America/Moncton
    {  371, 8233,  111795,  4, }, // America/Monterrey
    {  371, 4000,  111799,  5, }, // America/Montevideo
    {  371, 5741,  111804,  4, }, // America/Montserrat
    {  371, 5383,  111808,  2, }, // America/Nassau
    {  371, 3294,  111810,  2, }, // America/New_York
    {  371, 8251,  111812,  1, }, // America/Nome
    {  371, 1834,  111813,  3, }, // America/Noronha
    {  371, 8264,  111816, 10, }, // America/North_Dakota/Beulah
    {  371, 8292,  111826,  9, }, // America/North_Dakota/Center
    {  371, 8320,  111835, 11, }, // America/North_Dakota/New_Salem
    {  371,  393,  111846,  4, }, // America/Nuuk
    {  371, 8351,  111850,  4, }, // America/Ojinaga
    {  371, 5413,  111854,  3, }, // America/Panama
    {  371, 6106,  111857,  5, }, // America/Paramaribo
    {  371, 3224,  111862,  3, }, // America/Phoenix
    {  371, 3847,  111865,  6, }, // America/Port-au-Prince
    {  371, 5871,  111871,  7, }, // America/Port_of_Spain
    {  371, 8367,  111878,  5, }, // America/Porto_Velho
    {  371, 5760,  111883,  6, }, // America/Puerto_Rico
    {  371, 3932,  111889,  6, }, // America/Punta_Arenas
    {  371, 8387,  111895,  5, }, // America/Rankin_Inlet
    {  371, 8408,  111900,  3, }, // America/Recife
    {  371, 1979,  111903,  4, }, // America/Regina
    {  371, 8423,  111907,  4, }, // America/Resolute
    {  371,  695,  111911,  6, }, // America/Rio_Branco
    {  371, 8440,  111917,  3, }, // America/Santarem
    {  371, 2165,  111920,  4, }, // America/Santiago
    {  371, 6044,  111924,  5, }, // America/Santo_Domingo
    {  371, 1862,  111929,  4, }, // America/Sao_Paulo
    {  371, 6862,  111933,  8, }, // America/Scoresbysund
    {  371, 8457,  111941,  2, }, // America/Sitka
    {  371, 8471,  111943,  6, }, // America/St_Barthelemy
    {  371, 2045,  111949,  5, }, // America/St_Johns
    {  371, 5780,  111954,  5, }, // America/St_Kitts
    {  371, 5797,  111959,  6, }, // America/St_Lucia
    {  371,  855,  111965,  6, }, // America/St_Thomas
    {  371, 5830,  111971,  6, }, // America/St_Vincent
    {  371, 8493,  111977,  7, }, // America/Swift_Current
    {  371, 5363,  111984,  5, }, // America/Tegucigalpa
    {  371, 5673,  111989,  2, }, // America/Thule
    {  371,  313,  111991,  4, }, // America/Tijuana
    {  371,  608,  111995,  3, }, // America/Toronto
    {  371, 5605,  111998,  4, }, // America/Tortola
    {  371, 2077,  112002,  3, }, // America/Vancouver
    {  371, 2128,  112005,  5, }, // America/Whitehorse
    {  371,  734,  112010,  3, }, // America/Winnipeg
    {  371, 8515,  112013,  4, }, // America/Yakutat
    {  371, 5918,  112017,  3, }, // Antarctica/Casey
    {  371, 6027,  112020,  4, }, // Antarctica/Davis
    {  371, 6066,  112024,  6, }, // Antarctica/DumontDUrville
    {  371, 8531,  112030,  3, }, // Antarctica/Macquarie
    {  371, 7029,  112033,  2, }, // Antarctica/Mawson
    {  371, 7076,  112035,  3, }, // Antarctica/McMurdo
    {  371, 8552,  112038,  2, }, // Antarctica/Palmer
    {  371, 7189,  112040,  3, }, // Antarctica/Rothera
    {  371, 7258,  112043,  2, }, // Antarctica/Syowa
    {  371, 8570,  112045,  2, }, // Antarctica/Troll
    {  371, 7337,  112047,  4, }, // Antarctica/Vostok
    {  371, 1413,  112051,  5, }, // Arctic/Longyearbyen
    {  371, 5487,  112056,  2, }, // Asia/Aden
    {  371, 5297,  112058,  3, }, // Asia/Almaty
    {  371, 3870,  112061,  2, }, // Asia/Amman
    {  371, 5428,  112063,  4, }, // Asia/Anadyr
    {  371, 5440,  112067,  4, }, // Asia/Aqtau
    {  371, 5270,  112071,  3, }, // Asia/Aqtobe
    {  371,  964,  112074,  5, }, // Asia/Ashgabat
    {  371, 8587,  112079,  4, }, // Asia/Atyrau
    {  371, 3509,  112083,  4, }, // Asia/Baghdad
    {  371, 5451,  112087,  3, }, // Asia/Bahrain
    {  371, 3555,  112090,  2, }, // Asia/Baku
    {  371, 4368,  112092,  2, }, // Asia/Bangkok
    {  371, 3473,  112094,  4, }, // Asia/Barnaul
    {  371, 3988,  112098,  4, }, // Asia/Beirut
    {  371, 3680,  112102,  4, }, // Asia/Bishkek
    {  371, 5906,  112106,  4, }, // Asia/Brunei
    {  371, 4503,  112110,  2, }, // Asia/Chita
    {  371, 5963,  112112,  4, }, // Asia/Choibalsan
    {  371, 4413,  112116,  3, }, // Asia/Colombo
    {  371, 4442,  112119,  5, }, // Asia/Damascus
    {  371, 1060,  111219,  2, }, // Asia/Dhaka
    {  371, 6125,  112124,  2, }, // Asia/Dili
    {  371, 3498,  112126,  3, }, // Asia/Dubai
    {  371, 6092,  112129,  3, }, // Asia/Dushanbe
    {  371, 8599,  112132,  5, }, // Asia/Famagusta
    {  371, 8614,  112137,  2, }, // Asia/Gaza
    {  371, 4697,  112139,  3, }, // Asia/Hebron
    {  371, 1226,  112142,  3, }, // Asia/Ho_Chi_Minh
    {  371, 2576,  112145,  2, }, // Asia/Hong_Kong
    {  371, 4673,  112147,  3, }, // Asia/Hovd
    {  371, 4086,  112150,  5, }, // Asia/Irkutsk
    {  371, 6969,  112155,  4, }, // Asia/Jakarta
    {  371, 6955,  112159,  4, }, // Asia/Jayapura
    {  371, 1257,  112163,  4, }, // Asia/Jerusalem
    {  371, 3462,  112167,  2, }, // Asia/Kabul
    {  371, 4218,  112169,  3, }, // Asia/Kamchatka
    {  371, 4141,  112172,  3, }, // Asia/Karachi
    {  371, 1152,  112175,  4, }, // Asia/Kathmandu
    {  371, 8624,  112179,  3, }, // Asia/Khandyga
    {  371,  992,  112182,  3, }, // Asia/Kolkata
    {  371, 4099,  112185,  8, }, // Asia/Krasnoyarsk
    {  371, 4761,  112193,  6, }, // Asia/Kuala_Lumpur
    {  371, 5893,  112199,  2, }, // Asia/Kuching
    {  371, 5464,  112201,  4, }, // Asia/Kuwait
    {  371, 1178,  112205,  3, }, // Asia/Macau
    {  371, 3919,  112208,  3, }, // Asia/Magadan
    {  371, 1316,  112211,  4, }, // Asia/Makassar
    {  371, 7145,  112215,  3, }, // Asia/Manila
    {  371, 6883,  112218,  4, }, // Asia/Muscat
    {  371, 2446,  112222,  4, }, // Asia/Nicosia
    {  371, 8638,  112226,  7, }, // Asia/Novokuznetsk
    {  371, 4037,  112233,  7, }, // Asia/Novosibirsk
    {  371, 4131,  112240,  3, }, // Asia/Omsk
    {  371, 7108,  112243,  2, }, // Asia/Oral
    {  371, 6924,  112245,  3, }, // Asia/Phnom_Penh
    {  371, 8656,  112248,  5, }, // Asia/Pontianak
    {  371, 4116,  112253,  2, }, // Asia/Pyongyang
    {  371, 5476,  112255,  3, }, // Asia/Qatar
    {  371, 8671,  112258,  5, }, // Asia/Qostanay
    {  371, 4171,  112263,  5, }, // Asia/Qyzylorda
    {  371, 3486,  112268,  3, }, // Asia/Riyadh
    {  371, 4310,  112271,  3, }, // Asia/Sakhalin
    {  371, 7208,  112274,  5, }, // Asia/Samarkand
    {  371, 3116,  112279,  2, }, // Asia/Seoul
    {  371, 1020,  112281,  3, }, // Asia/Shanghai
    {  371, 3137,  112284,  4, }, // Asia/Singapore
    {  371, 4199,  112288,  8, }, // Asia/Srednekolymsk
    {  371, 3100,  112296,  4, }, // Asia/Taipei
    {  371, 4683,  112300,  4, }, // Asia/Tashkent
    {  371, 3817,  112304,  4, }, // Asia/Tbilisi
    {  371, 2638,  112308,  3, }, // Asia/Tehran
    {  371, 1284,  112311,  2, }, // Asia/Thimphu
    {  371, 2687,  112313,  2, }, // Asia/Tokyo
    {  371, 4474,  112315,  3, }, // Asia/Tomsk
    {  371, 1346,  112318,  5, }, // Asia/Ulaanbaatar
    {  371, 1126,  112323,  4, }, // Asia/Urumqi
    {  371, 8685,  112327,  5, }, // Asia/Ust-Nera
    {  371, 6940,  112332,  4, }, // Asia/Vientiane
    {  371, 4612,  112336,  7, }, // Asia/Vladivostok
    {  371, 4730,  112343,  4, }, // Asia/Yakutsk
    {  371, 1202,  112347,  2, }, // Asia/Yangon
    {  371, 3785,  112349,  7, }, // Asia/Yekaterinburg
    {  371, 3649,  112356,  3, }, // Asia/Yerevan
    {  371, 3565,  112359,  4, }, // Atlantic/Azores
    {  371, 5588,  112363,  3, }, // Atlantic/Bermuda
    {  371, 6591,  112366,  7, }, // Atlantic/Canary
    {  371, 3629,  112373,  6, }, // Atlantic/Cape_Verde
    {  371, 1379,  112379,  5, }, // Atlantic/Faroe
    {  371, 8699,  112384,  4, }, // Atlantic/Madeira
    {  371, 2614,  112388,  5, }, // Atlantic/Reykjavik
    {  371, 7235,  112393,  6, }, // Atlantic/South_Georgia
    {  371, 6784,  112399,  7, }, // Atlantic/St_Helena
    {  371, 6607,  112406,  3, }, // Atlantic/Stanley
    {  371, 1654,  112409,  5, }, // Australia/Adelaide
    {  371, 1619,  112414,  5, }, // Australia/Brisbane
    {  371, 1783,  112419,  4, }, // Australia/Broken_Hill
    {  371, 1567,  112423,  2, }, // Australia/Darwin
    {  371, 3539,  112425,  3, }, // Australia/Eucla
    {  371, 1500,  112428,  3, }, // Australia/Hobart
    {  371, 8716,  112431,  3, }, // Australia/Lindeman
    {  371, 1531,  112434,  5, }, // Australia/Lord_Howe
    {  371, 1711,  112439,  3, }, // Australia/Melbourne
    {  371, 1746,  112442,  2, }, // Australia/Perth
    {  371, 1447,  112444,  3, }, // Australia/Sydney
    {  371, 6390,  112447,  5, }, // Europe/Amsterdam
    {  371, 6149,  112452,  3, }, // Europe/Andorra
    {  371, 3522,  112455,  5, }, // Europe/Astrakhan
    {  371, 6577,  112460,  3, }, // Europe/Athens
    {  371, 6285,  112463,  5, }, // Europe/Belgrade
    {  371, 4659,  112468,  3, }, // Europe/Berlin
    {  371, 6437,  112471,  6, }, // Europe/Bratislava
    {  371, 6178,  112477,  3, }, // Europe/Brussels
    {  371, 3830,  112480,  5, }, // Europe/Bucharest
    {  371, 3708,  112485,  5, }, // Europe/Budapest
    {  371, 8751,  112490,  3, }, // Europe/Busingen
    {  371, 2475,  112493,  4, }, // Europe/Chisinau
    {  371, 6238,  112497,  4, }, // Europe/Copenhagen
    {  371, 2260,  112501,  3, }, // Europe/Dublin
    {  371, 6256,  112504,  4, }, // Europe/Gibraltar
    {  371, 8767,  112508,  2, }, // Europe/Guernsey
    {  371, 6561,  112510,  3, }, // Europe/Helsinki
    {  371, 8783,  112513,  2, }, // Europe/Isle_of_Man
    {  371, 1097,  112515,  4, }, // Europe/Istanbul
    {  371, 8802,  112519,  2, }, // Europe/Jersey
    {  371, 3881,  112521,  6, }, // Europe/Kaliningrad
    {  371, 8924,  112527,  3, }, // Europe/Kirov
    {  371, 2419,  112530,  3, }, // Europe/Kyiv
    {  371, 3078,  112533,  3, }, // Europe/Lisbon
    {  371, 6455,  112536,  4, }, // Europe/Ljubljana
    {  371, 2393,  112540,  2, }, // Europe/London
    {  371, 6314,  112542,  5, }, // Europe/Luxembourg
    {  371, 4794,  112547,  4, }, // Europe/Madrid
    {  371, 6346,  112551,  2, }, // Europe/Malta
    {  371, 6531,  112553,  4, }, // Europe/Mariehamn
    {  371, 3595,  112557,  3, }, // Europe/Minsk
    {  371, 6359,  112560,  3, }, // Europe/Monaco
    {  371, 3443,  112563,  4, }, // Europe/Moscow
    {  371, 6407,  112567,  3, }, // Europe/Oslo
    {  371, 4186,  112570,  2, }, // Europe/Paris
    {  371, 6373,  112572,  5, }, // Europe/Podgorica
    {  371, 6224,  112577,  3, }, // Europe/Prague
    {  371, 8816,  112580,  2, }, // Europe/Riga
    {  371, 6273,  112582,  2, }, // Europe/Rome
    {  371, 4233,  112584,  3, }, // Europe/Samara
    {  371, 6419,  112587,  4, }, // Europe/San_Marino
    {  371, 6194,  112591,  4, }, // Europe/Sarajevo
    {  371, 4353,  112595,  4, }, // Europe/Saratov
    {  371, 8828,  112599,  4, }, // Europe/Simferopol
    {  371, 6332,  112603,  3, }, // Europe/Skopje
    {  371, 6548,  112606,  3, }, // Europe/Sofia
    {  371, 6472,  112609,  4, }, // Europe/Stockholm
    {  371, 8846,  112613,  2, }, // Europe/Tallinn
    {  371, 6135,  112615,  3, }, // Europe/Tirane
    {  371, 8861,  112618,  7, }, // Europe/Ulyanovsk
    {  371, 6301,  112625,  3, }, // Europe/Vaduz
    {  371, 6516,  112628,  3, }, // Europe/Vatican
    {  371, 6164,  112631,  3, }, // Europe/Vienna
    {  371, 8878,  112634,  4, }, // Europe/Vilnius
    {  371, 4629,  112638,  5, }, // Europe/Volgograd
    {  371, 3055,  112643,  4, }, // Europe/Warsaw
    {  371, 6210,  112647,  4, }, // Europe/Zagreb
    {  371, 6489,  112651,  3, }, // Europe/Zurich
    {  371, 4980,  112654,  6, }, // Indian/Antananarivo
    {  371, 6910,  112660,  3, }, // Indian/Chagos
    {  371, 5979,  112663,  5, }, // Indian/Christmas
    {  371, 5996,  112668,  3, }, // Indian/Cocos
    {  371, 4931,  112671,  3, }, // Indian/Comoro
    {  371, 6624,  112674,  4, }, // Indian/Kerguelen
    {  371, 7223,  112678,  2, }, // Indian/Mahe
    {  371, 7013,  112680,  3, }, // Indian/Maldives
    {  371, 3971,  112683,  4, }, // Indian/Mauritius
    {  371, 5000,  112687,  3, }, // Indian/Mayotte
    {  371, 7174,  112690,  4, }, // Indian/Reunion
    {  371, 4324,  112694,  3, }, // Pacific/Apia
    {  371,  932,  112697,  4, }, // Pacific/Auckland
    {  371, 3608,  112701,  3, }, // Pacific/Bougainville
    {  371, 2863,  112704,  2, }, // Pacific/Chatham
    {  371, 3022,  112706,  3, }, // Pacific/Chuuk
    {  371, 2201,  112709,  5, }, // Pacific/Easter
    {  371, 7323,  112714,  3, }, // Pacific/Efate
    {  371, 7290,  112717,  4, }, // Pacific/Fakaofo
    {  371, 3804,  112721,  2, }, // Pacific/Fiji
    {  371, 7306,  112723,  4, }, // Pacific/Funafuti
    {  371, 6641,  112727,  5, }, // Pacific/Galapagos
    {  371, 6659,  112732,  3, }, // Pacific/Gambier
    {  371, 3724,  112735,  4, }, // Pacific/Guadalcanal
    {  371, 5935,  112739,  1, }, // Pacific/Guam
    {  371, 2929,  112740,  4, }, // Pacific/Honolulu
    {  371, 2897,  112744,  4, }, // Pacific/Kanton
    {  371, 3900,  112748,  5, }, // Pacific/Kiritimati
    {  371, 6982,  112753,  3, }, // Pacific/Kosrae
    {  371, 2708,  112756,  3, }, // Pacific/Kwajalein
    {  371, 4779,  112759,  3, }, // Pacific/Majuro
    {  371, 3953,  112762,  4, }, // Pacific/Marquesas
    {  371, 8909,  112766,  4, }, // Pacific/Midway
    {  371, 7047,  112770,  3, }, // Pacific/Nauru
    {  371, 7095,  112773,  3, }, // Pacific/Niue
    {  371, 4070,  112776,  2, }, // Pacific/Norfolk
    {  371, 7061,  112778,  3, }, // Pacific/Noumea
    {  371, 2991,  112781,  4, }, // Pacific/Pago_Pago
    {  371, 7118,  112785,  3, }, // Pacific/Palau
    {  371, 7157,  112788,  3, }, // Pacific/Pitcairn
    {  371, 2961,  112791,  3, }, // Pacific/Pohnpei
    {  371, 4709,  112794,  6, }, // Pacific/Port_Moresby
    {  371, 6009,  112800,  4, }, // Pacific/Rarotonga
    {  371, 5948,  112804,  3, }, // Pacific/Saipan
    {  371, 7275,  112807,  3, }, // Pacific/Tahiti
    {  371, 6675,  112810,  3, }, // Pacific/Tarawa
    {  371, 4485,  112813,  4, }, // Pacific/Tongatapu
    {  371, 7355,  112817,  3, }, // Pacific/Wake
    {  371, 7368,  112820,  3, }, // Pacific/Wallis
    {  374,   14,       0,  6, }, // Africa/Asmara Koyraboro Senni/Latin/Mali
    {  374, 4337,       6,  8, }, // Africa/Sao_Tome
    {  374, 4154,      14,  8, }, // America/Asuncion
    {  374,  237,      22,  8, }, // America/Atikokan
    {  374, 7586,      30, 17, }, // America/Bahia_Banderas
    {  374, 3770,      47,  6, }, // America/Cancun
    {  374, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  374, 5640,      66,  7, }, // America/Curacao
    {  374,  481,      73, 13, }, // America/Indiana/Knox
    {  374, 7924,      86, 16, }, // America/Indiana/Marengo
    {  374, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  374, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  374, 8001,     139, 14, }, // America/Indiana/Vevay
    {  374, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  374, 8049,     171, 16, }, // America/Indiana/Winamac
    {  374, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  374, 5849,     207, 22, }, // America/Lower_Princes
    {  374, 8183,     229,  6, }, // America/Merida
    {  374, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  374, 8292,     255, 20, }, // America/North_Dakota/Center
    {  374, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  374,  393,     298,  4, }, // America/Nuuk
    {  374, 6862,     302, 16, }, // America/Scoresbysund
    {  374, 8471,     318, 14, }, // America/St_Barthelemy
    {  374, 2045,     332, 10, }, // America/St_Johns
    {  374, 5780,     342,  9, }, // America/St_Kitts
    {  374, 5797,     351,  9, }, // America/St_Lucia
    {  374,  855,     360, 10, }, // America/St_Thomas
    {  374, 5830,     370, 11, }, // America/St_Vincent
    {  374, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  374, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  374, 1152,     413,  9, }, // Asia/Kathmandu
    {  374,  992,     422,  7, }, // Asia/Kolkata
    {  374, 1178,     429,  5, }, // Asia/Macau
    {  374, 1202,     442,  6, }, // Asia/Yangon
    {  374, 1379,     448,  5, }, // Atlantic/Faroe
    {  374, 6784,     453, 10, }, // Atlantic/St_Helena
    {  374, 2419,     463,  4, }, // Europe/Kyiv
    {  374, 7174,     467,  7, }, // Indian/Reunion
    {  374, 3022,     474,  5, }, // Pacific/Chuuk
    {  374, 2961,     479,  7, }, // Pacific/Pohnpei
    {  375,   14,       0,  6, }, // Africa/Asmara Koyra Chiini/Latin/Mali
    {  375, 4337,       6,  8, }, // Africa/Sao_Tome
    {  375, 4154,      14,  8, }, // America/Asuncion
    {  375,  237,      22,  8, }, // America/Atikokan
    {  375, 7586,      30, 17, }, // America/Bahia_Banderas
    {  375, 3770,      47,  6, }, // America/Cancun
    {  375, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  375, 5640,      66,  7, }, // America/Curacao
    {  375,  481,      73, 13, }, // America/Indiana/Knox
    {  375, 7924,      86, 16, }, // America/Indiana/Marengo
    {  375, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  375, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  375, 8001,     139, 14, }, // America/Indiana/Vevay
    {  375, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  375, 8049,     171, 16, }, // America/Indiana/Winamac
    {  375, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  375, 5849,     207, 22, }, // America/Lower_Princes
    {  375, 8183,     229,  6, }, // America/Merida
    {  375, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  375, 8292,     255, 20, }, // America/North_Dakota/Center
    {  375, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  375,  393,     298,  4, }, // America/Nuuk
    {  375, 6862,     302, 16, }, // America/Scoresbysund
    {  375, 8471,     318, 14, }, // America/St_Barthelemy
    {  375, 2045,     332, 10, }, // America/St_Johns
    {  375, 5780,     342,  9, }, // America/St_Kitts
    {  375, 5797,     351,  9, }, // America/St_Lucia
    {  375,  855,     360, 10, }, // America/St_Thomas
    {  375, 5830,     370, 11, }, // America/St_Vincent
    {  375, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  375, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  375, 1152,     413,  9, }, // Asia/Kathmandu
    {  375,  992,     422,  7, }, // Asia/Kolkata
    {  375, 1178,     429,  5, }, // Asia/Macau
    {  375, 1202,     442,  6, }, // Asia/Yangon
    {  375, 1379,     448,  5, }, // Atlantic/Faroe
    {  375, 6784,     453, 10, }, // Atlantic/St_Helena
    {  375, 2419,     463,  4, }, // Europe/Kyiv
    {  375, 7174,     467,  7, }, // Indian/Reunion
    {  375, 3022,     474,  5, }, // Pacific/Chuuk
    {  375, 2961,     479,  7, }, // Pacific/Pohnpei
    {  376,   14,       0,  6, }, // Africa/Asmara Kpelle/Latin/Liberia
    {  376, 4337,       6,  8, }, // Africa/Sao_Tome
    {  376, 4154,      14,  8, }, // America/Asuncion
    {  376,  237,      22,  8, }, // America/Atikokan
    {  376, 7586,      30, 17, }, // America/Bahia_Banderas
    {  376, 3770,      47,  6, }, // America/Cancun
    {  376, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  376, 5640,      66,  7, }, // America/Curacao
    {  376,  481,      73, 13, }, // America/Indiana/Knox
    {  376, 7924,      86, 16, }, // America/Indiana/Marengo
    {  376, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  376, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  376, 8001,     139, 14, }, // America/Indiana/Vevay
    {  376, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  376, 8049,     171, 16, }, // America/Indiana/Winamac
    {  376, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  376, 5849,     207, 22, }, // America/Lower_Princes
    {  376, 8183,     229,  6, }, // America/Merida
    {  376, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  376, 8292,     255, 20, }, // America/North_Dakota/Center
    {  376, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  376,  393,     298,  4, }, // America/Nuuk
    {  376, 6862,     302, 16, }, // America/Scoresbysund
    {  376, 8471,     318, 14, }, // America/St_Barthelemy
    {  376, 2045,     332, 10, }, // America/St_Johns
    {  376, 5780,     342,  9, }, // America/St_Kitts
    {  376, 5797,     351,  9, }, // America/St_Lucia
    {  376,  855,     360, 10, }, // America/St_Thomas
    {  376, 5830,     370, 11, }, // America/St_Vincent
    {  376, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  376, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  376, 1152,     413,  9, }, // Asia/Kathmandu
    {  376,  992,     422,  7, }, // Asia/Kolkata
    {  376, 1178,     429,  5, }, // Asia/Macau
    {  376, 1202,     442,  6, }, // Asia/Yangon
    {  376, 1379,     448,  5, }, // Atlantic/Faroe
    {  376, 6784,     453, 10, }, // Atlantic/St_Helena
    {  376, 2419,     463,  4, }, // Europe/Kyiv
    {  376, 7174,     467,  7, }, // Indian/Reunion
    {  376, 3022,     474,  5, }, // Pacific/Chuuk
    {  376, 2961,     479,  7, }, // Pacific/Pohnpei
    {  378, 6723,    4550,  4, }, // Africa/Accra Kurdish/Latin/Turkey
    {  378, 7383,  112823,  7, }, // Africa/Algiers
    {  378,   14,       0,  6, }, // Africa/Asmara
    {  378, 2242,  112830,  6, }, // Africa/Cairo
    {  378, 4019,   31744, 10, }, // Africa/Casablanca
    {  378, 7412,  112836,  5, }, // Africa/Ceuta
    {  378, 6736,  112841,  7, }, // Africa/Conakry
    {  378, 5032,  112848,  9, }, // Africa/Dar_es_Salaam
    {  378, 4945,  112857,  6, }, // Africa/Djibouti
    {  378, 4401,  112863,  4, }, // Africa/Juba
    {  378, 4426,  112867,  6, }, // Africa/Khartoum
    {  378, 4875,  112873,  6, }, // Africa/Kigali
    {  378, 5208,  112879,  7, }, // Africa/Kinshasa
    {  378, 5238,  112886,  8, }, // Africa/Libreville
    {  378, 5099,  112894,  6, }, // Africa/Maseru
    {  378, 5015,  112900,  8, }, // Africa/Mogadishu
    {  378, 3755,  112908,  7, }, // Africa/Nairobi
    {  378, 4337,       6,  8, }, // Africa/Sao_Tome
    {  378, 2732,  112915,  7, }, // Africa/Tripoli
    {  378, 6503,  112922,  5, }, // Africa/Tunis
    {  378, 5557,  112927,  5, }, // America/Aruba
    {  378, 4154,      14,  8, }, // America/Asuncion
    {  378,  237,      22,  8, }, // America/Atikokan
    {  378, 7586,  112932, 14, }, // America/Bahia_Banderas
    {  378, 5309,  112946,  6, }, // America/Belize
    {  378, 3770,  112952,  6, }, // America/Cancun
    {  378, 7737,  112958, 13, }, // America/Ciudad_Juarez
    {  378, 5324,  112971, 10, }, // America/Costa_Rica
    {  378, 5640,      66,  7, }, // America/Curacao
    {  378, 5656,  112981,  8, }, // America/Dominica
    {  378,  481,      73, 13, }, // America/Indiana/Knox
    {  378, 7924,      86, 16, }, // America/Indiana/Marengo
    {  378, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  378, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  378, 8001,     139, 14, }, // America/Indiana/Vevay
    {  378, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  378, 8049,     171, 16, }, // America/Indiana/Winamac
    {  378, 2665,  112989,  7, }, // America/Jamaica
    {  378, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  378, 5849,     207, 22, }, // America/Lower_Princes
    {  378, 8183,  112996,  6, }, // America/Merida
    {  378, 8264,  113002, 22, }, // America/North_Dakota/Beulah
    {  378, 8292,  113024, 22, }, // America/North_Dakota/Center
    {  378, 8320,  113046, 25, }, // America/North_Dakota/New_Salem
    {  378,  393,     298,  4, }, // America/Nuuk
    {  378, 5760,  113071, 10, }, // America/Puerto_Rico
    {  378, 6862,     302, 16, }, // America/Scoresbysund
    {  378, 8471,  113081, 16, }, // America/St_Barthelemy
    {  378, 2045,     332, 10, }, // America/St_Johns
    {  378, 5780,     342,  9, }, // America/St_Kitts
    {  378, 5797,     351,  9, }, // America/St_Lucia
    {  378,  855,     360, 10, }, // America/St_Thomas
    {  378, 5830,     370, 11, }, // America/St_Vincent
    {  378, 6027,  113097,  5, }, // Antarctica/Davis
    {  378, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  378, 5297,  113102,  6, }, // Asia/Almaty
    {  378, 3870,  113108,  4, }, // Asia/Amman
    {  378, 5428,    5900,  6, }, // Asia/Anadyr
    {  378, 5440,   62679,  5, }, // Asia/Aqtau
    {  378,  964,  113112,  7, }, // Asia/Ashgabat
    {  378, 8587,  113119,  6, }, // Asia/Atyrau
    {  378, 3509,  113125,  5, }, // Asia/Baghdad
    {  378, 5451,  113130,  7, }, // Asia/Bahrain
    {  378, 3555,  113137,  4, }, // Asia/Baku
    {  378, 3988,  113141,  6, }, // Asia/Beirut
    {  378, 3680,  113147,  6, }, // Asia/Bishkek
    {  378, 5906,  113153,  6, }, // Asia/Brunei
    {  378, 4503,  113159,  4, }, // Asia/Chita
    {  378, 5963,   20780,  9, }, // Asia/Choibalsan
    {  378, 4413,    5949,  7, }, // Asia/Colombo
    {  378, 4442,  113163,  3, }, // Asia/Damascus
    {  378, 1060,    5956,  4, }, // Asia/Dhaka
    {  378, 6125,  113166,  4, }, // Asia/Dili
    {  378, 3498,  113170,  5, }, // Asia/Dubai
    {  378, 6092,  113175,  7, }, // Asia/Dushanbe
    {  378, 8614,  113182,  4, }, // Asia/Gaza
    {  378, 1226,  113186, 19, }, // Asia/Ho_Chi_Minh
    {  378, 6969,   20850,  7, }, // Asia/Jakarta
    {  378, 6955,  113205,  8, }, // Asia/Jayapura
    {  378, 1257,  113213,  4, }, // Asia/Jerusalem
    {  378, 3462,  113217,  5, }, // Asia/Kabul
    {  378, 4218,   20879,  8, }, // Asia/Kamchatka
    {  378, 4141,  113222,  6, }, // Asia/Karachi
    {  378, 1152,  113228,  8, }, // Asia/Kathmandu
    {  378, 8624,  113236,  7, }, // Asia/Khandyga
    {  378,  992,     422,  7, }, // Asia/Kolkata
    {  378, 4761,  113243, 12, }, // Asia/Kuala_Lumpur
    {  378, 5893,  113255,  6, }, // Asia/Kuching
    {  378, 5464,  113261,  6, }, // Asia/Kuwait
    {  378, 1178,    6064,  5, }, // Asia/Macau
    {  378, 1316,    6069,  7, }, // Asia/Makassar
    {  378, 7145,  113267,  6, }, // Asia/Manila
    {  378, 6883,    4227,  6, }, // Asia/Muscat
    {  378, 2446,  113273,  7, }, // Asia/Nicosia
    {  378, 5476,  113280,  5, }, // Asia/Qatar
    {  378, 4171,  113285,  9, }, // Asia/Qyzylorda
    {  378, 3486,   19265,  5, }, // Asia/Riyadh
    {  378, 4310,  113294,  7, }, // Asia/Sakhalin
    {  378, 7208,  113301,  9, }, // Asia/Samarkand
    {  378, 3116,  113310,  4, }, // Asia/Seoul
    {  378, 1020,  113314,  7, }, // Asia/Shanghai
    {  378, 3137,  113321,  8, }, // Asia/Singapore
    {  378, 3100,  113329,  6, }, // Asia/Taipei
    {  378, 4683,  113335,  7, }, // Asia/Tashkent
    {  378, 3817,  113342,  6, }, // Asia/Tbilisi
    {  378, 1346,  113348,  9, }, // Asia/Ulaanbaatar
    {  378, 1126,  113357,  6, }, // Asia/Urumqi
    {  378, 8685,  113363,  8, }, // Asia/Ust-Nera
    {  378, 1202,     442,  6, }, // Asia/Yangon
    {  378, 3649,  113371,  5, }, // Asia/Yerevan
    {  378, 3565,  113376, 13, }, // Atlantic/Azores
    {  378, 5588,  113389,  7, }, // Atlantic/Bermuda
    {  378, 6591,  113396, 17, }, // Atlantic/Canary
    {  378, 3629,   55900,  9, }, // Atlantic/Cape_Verde
    {  378, 1379,     448,  5, }, // Atlantic/Faroe
    {  378, 7235,  113413, 15, }, // Atlantic/South_Georgia
    {  378, 6784,     453, 10, }, // Atlantic/St_Helena
    {  378, 1447,  113428,  6, }, // Australia/Sydney
    {  378, 6149,   32109,  6, }, // Europe/Andorra
    {  378, 3522,  113434,  8, }, // Europe/Astrakhan
    {  378, 6577,  113442,  5, }, // Europe/Athens
    {  378, 6285,   25465,  7, }, // Europe/Belgrade
    {  378, 4659,  113447,  6, }, // Europe/Berlin
    {  378, 6178,    6415,  7, }, // Europe/Brussels
    {  378, 3830,  113453,  6, }, // Europe/Bucharest
    {  378, 3708,  113459,  8, }, // Europe/Budapest
    {  378, 8751,  113467,  8, }, // Europe/Busingen
    {  378, 2475,  113475,  7, }, // Europe/Chisinau
    {  378, 6238,    4376,  8, }, // Europe/Copenhagen
    {  378, 2260,  113482,  6, }, // Europe/Dublin
    {  378, 6256,  113488,  9, }, // Europe/Gibraltar
    {  378, 6561,  113497,  8, }, // Europe/Helsinki
    {  378, 8783,  113505, 11, }, // Europe/Isle_of_Man
    {  378, 1097,  113516,  7, }, // Europe/Istanbul
    {  378, 3881,  113523, 11, }, // Europe/Kaliningrad
    {  378, 8924,  113534,  5, }, // Europe/Kirov
    {  378, 2419,  113539,  4, }, // Europe/Kyiv
    {  378, 3078,  113543,  6, }, // Europe/Lisbon
    {  378, 2393,   93889,  6, }, // Europe/London
    {  378, 6314,  113549, 10, }, // Europe/Luxembourg
    {  378, 4794,  113559,  6, }, // Europe/Madrid
    {  378, 3595,  113565,  5, }, // Europe/Minsk
    {  378, 6359,    6525,  6, }, // Europe/Monaco
    {  378, 3443,   62842,  7, }, // Europe/Moscow
    {  378, 4186,  113570,  5, }, // Europe/Paris
    {  378, 6373,  113575,  9, }, // Europe/Podgorica
    {  378, 6224,    6545,  4, }, // Europe/Prague
    {  378, 8816,  113584,  4, }, // Europe/Riga
    {  378, 6273,   19581,  4, }, // Europe/Rome
    {  378, 6419,  113588, 10, }, // Europe/San_Marino
    {  378, 6194,  113598, 10, }, // Europe/Sarajevo
    {  378, 6332,  113608,  5, }, // Europe/Skopje
    {  378, 6548,   97885,  5, }, // Europe/Sofia
    {  378, 6472,    6586,  8, }, // Europe/Stockholm
    {  378, 8846,  113613,  5, }, // Europe/Tallinn
    {  378, 6135,  113618,  5, }, // Europe/Tirane
    {  378, 6516,  113623,  7, }, // Europe/Vatican
    {  378, 6164,  113630,  6, }, // Europe/Vienna
    {  378, 8878,  113636,  6, }, // Europe/Vilnius
    {  378, 3055,  113642,  7, }, // Europe/Warsaw
    {  378, 6489,  113649,  5, }, // Europe/Zurich
    {  378, 4931,    6656,  5, }, // Indian/Comoro
    {  378, 7013,  113654,  6, }, // Indian/Maldives
    {  378, 7174,     467,  7, }, // Indian/Reunion
    {  378, 3022,     474,  5, }, // Pacific/Chuuk
    {  378, 2201,  113660, 17, }, // Pacific/Easter
    {  378, 3804,  113677,  4, }, // Pacific/Fiji
    {  378, 2961,     479,  7, }, // Pacific/Pohnpei
    {  378, 7275,  113681,  6, }, // Pacific/Tahiti
    {  379,   14,       0,  6, }, // Africa/Asmara Kwasio/Latin/Cameroon
    {  379, 4337,       6,  8, }, // Africa/Sao_Tome
    {  379, 4154,      14,  8, }, // America/Asuncion
    {  379,  237,      22,  8, }, // America/Atikokan
    {  379, 7586,      30, 17, }, // America/Bahia_Banderas
    {  379, 3770,      47,  6, }, // America/Cancun
    {  379, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  379, 5640,      66,  7, }, // America/Curacao
    {  379,  481,      73, 13, }, // America/Indiana/Knox
    {  379, 7924,      86, 16, }, // America/Indiana/Marengo
    {  379, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  379, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  379, 8001,     139, 14, }, // America/Indiana/Vevay
    {  379, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  379, 8049,     171, 16, }, // America/Indiana/Winamac
    {  379, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  379, 5849,     207, 22, }, // America/Lower_Princes
    {  379, 8183,     229,  6, }, // America/Merida
    {  379, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  379, 8292,     255, 20, }, // America/North_Dakota/Center
    {  379, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  379,  393,     298,  4, }, // America/Nuuk
    {  379, 6862,     302, 16, }, // America/Scoresbysund
    {  379, 8471,     318, 14, }, // America/St_Barthelemy
    {  379, 2045,     332, 10, }, // America/St_Johns
    {  379, 5780,     342,  9, }, // America/St_Kitts
    {  379, 5797,     351,  9, }, // America/St_Lucia
    {  379,  855,     360, 10, }, // America/St_Thomas
    {  379, 5830,     370, 11, }, // America/St_Vincent
    {  379, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  379, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  379, 1152,     413,  9, }, // Asia/Kathmandu
    {  379,  992,     422,  7, }, // Asia/Kolkata
    {  379, 1178,     429,  5, }, // Asia/Macau
    {  379, 1202,     442,  6, }, // Asia/Yangon
    {  379, 1379,     448,  5, }, // Atlantic/Faroe
    {  379, 6784,     453, 10, }, // Atlantic/St_Helena
    {  379, 2419,     463,  4, }, // Europe/Kyiv
    {  379, 7174,     467,  7, }, // Indian/Reunion
    {  379, 3022,     474,  5, }, // Pacific/Chuuk
    {  379, 2961,     479,  7, }, // Pacific/Pohnpei
    {  380, 6751,   34826,  7, }, // Africa/Abidjan Kyrgyz/Cyrillic/Kyrgyzstan
    {  380, 6723,     492,  5, }, // Africa/Accra
    {  380, 4961,     497, 11, }, // Africa/Addis_Ababa
    {  380, 7383,     508,  5, }, // Africa/Algiers
    {  380,   14,   25647,  6, }, // Africa/Asmara
    {  380,   44,     519,  6, }, // Africa/Bamako
    {  380, 5159,  113687,  4, }, // Africa/Bangui
    {  380, 6709,     530,  6, }, // Africa/Banjul
    {  380, 7398,     536,  5, }, // Africa/Bissau
    {  380, 4859,   25670,  8, }, // Africa/Blantyre
    {  380, 5189,  113691, 10, }, // Africa/Brazzaville
    {  380, 4824,     558,  9, }, // Africa/Bujumbura
    {  380, 2242,     567,  4, }, // Africa/Cairo
    {  380, 4019,     571, 10, }, // Africa/Casablanca
    {  380, 7412,     581,  5, }, // Africa/Ceuta
    {  380, 6736,     586,  7, }, // Africa/Conakry
    {  380, 6803,     593,  5, }, // Africa/Dakar
    {  380, 5032,  113701, 13, }, // Africa/Dar_es_Salaam
    {  380, 4945,  113714,  6, }, // Africa/Djibouti
    {  380, 5145,     616,  5, }, // Africa/Douala
    {  380, 5068,  113720,  7, }, // Africa/El_Aaiun
    {  380, 6816,     628,  7, }, // Africa/Freetown
    {  380, 4808,     635,  8, }, // Africa/Gaborone
    {  380, 4917,   32384,  6, }, // Africa/Harare
    {  380, 4381,   50098, 12, }, // Africa/Johannesburg
    {  380, 4401,  113727,  4, }, // Africa/Juba
    {  380, 5053,     665,  7, }, // Africa/Kampala
    {  380, 4426,   32401,  6, }, // Africa/Khartoum
    {  380, 4875,     678,  6, }, // Africa/Kigali
    {  380, 5208,     684,  7, }, // Africa/Kinshasa
    {  380, 4646,     691,  5, }, // Africa/Lagos
    {  380, 5238,  113731,  9, }, // Africa/Libreville
    {  380, 6832,     704,  4, }, // Africa/Lome
    {  380, 5113,     708,  6, }, // Africa/Luanda
    {  380, 4841,     714,  9, }, // Africa/Lubumbashi
    {  380, 4903,     723,  6, }, // Africa/Lusaka
    {  380, 5224,     729,  6, }, // Africa/Malabo
    {  380, 4889,   32416,  6, }, // Africa/Maputo
    {  380, 5099,     741,  6, }, // Africa/Maseru
    {  380, 5084,     747,  7, }, // Africa/Mbabane
    {  380, 5015,   34891,  8, }, // Africa/Mogadishu
    {  380, 6997,     762,  8, }, // Africa/Monrovia
    {  380, 3755,   34907,  7, }, // Africa/Nairobi
    {  380, 5173,  113740,  7, }, // Africa/Ndjamena
    {  380, 5256,  113747,  5, }, // Africa/Niamey
    {  380, 6766,     790,  7, }, // Africa/Nouakchott
    {  380, 6690,     797,  8, }, // Africa/Ouagadougou
    {  380, 5127,     805, 10, }, // Africa/Porto-Novo
    {  380, 4337,   32455,  8, }, // Africa/Sao_Tome
    {  380, 2732,     823,  7, }, // Africa/Tripoli
    {  380, 6503,     830,  5, }, // Africa/Tunis
    {  380, 4054,  113752,  7, }, // Africa/Windhoek
    {  380,  132,     842,  4, }, // America/Adak
    {  380, 3183,  113759,  7, }, // America/Anchorage
    {  380, 5524,   34928,  7, }, // America/Anguilla
    {  380, 5541,     860,  7, }, // America/Antigua
    {  380, 4456,     867,  9, }, // America/Araguaina
    {  380,  166,   50134, 12, }, // America/Argentina/Buenos_Aires
    {  380,   91,     888,  9, }, // America/Argentina/Catamarca
    {  380,  270,     897,  7, }, // America/Argentina/Cordoba
    {  380,  441,   25937,  5, }, // America/Argentina/Jujuy
    {  380, 7425,     909,  8, }, // America/Argentina/La_Rioja
    {  380,  565,   32526,  7, }, // America/Argentina/Mendoza
    {  380, 7452,  113766, 12, }, // America/Argentina/Rio_Gallegos
    {  380, 7483,     935,  5, }, // America/Argentina/Salta
    {  380, 7507,   32544,  8, }, // America/Argentina/San_Juan
    {  380, 5497,   32552,  7, }, // America/Argentina/San_Luis
    {  380, 7534,     956,  7, }, // America/Argentina/Tucuman
    {  380, 7560,  113778,  6, }, // America/Argentina/Ushuaia
    {  380, 5557,     969,  5, }, // America/Aruba
    {  380, 4154,   32566,  8, }, // America/Asuncion
    {  380,  237,   32574,  8, }, // America/Atikokan
    {  380, 3581,  113784,  5, }, // America/Bahia
    {  380, 7586,  113789, 14, }, // America/Bahia_Banderas
    {  380, 5571,    1013,  8, }, // America/Barbados
    {  380, 7609,   32601,  5, }, // America/Belem
    {  380, 5309,    1026,  5, }, // America/Belize
    {  380, 7623,    1031, 12, }, // America/Blanc-Sablon
    {  380, 7644,   32625,  9, }, // America/Boa_Vista
    {  380, 4263,    1052,  6, }, // America/Bogota
    {  380, 7662,   50182,  5, }, // America/Boise
    {  380, 7676,  113803, 11, }, // America/Cambridge_Bay
    {  380, 7698,   32650, 12, }, // America/Campo_Grande
    {  380, 3770,    1087,  6, }, // America/Cancun
    {  380, 4596,    1093,  7, }, // America/Caracas
    {  380, 4247,   35029,  5, }, // America/Cayenne
    {  380, 5398,   26079,  6, }, // America/Cayman
    {  380, 3251,    1129,  6, }, // America/Chicago
    {  380, 7719,  113814,  8, }, // America/Chihuahua
    {  380, 7737,  104869, 13, }, // America/Ciudad_Juarez
    {  380, 5324,    1141, 10, }, // America/Costa_Rica
    {  380, 7759,    1151,  7, }, // America/Creston
    {  380, 3693,  113822,  6, }, // America/Cuiaba
    {  380, 5640,   35086,  7, }, // America/Curacao
    {  380, 7775,   35093, 12, }, // America/Danmarkshavn
    {  380, 7796,    1184,  6, }, // America/Dawson
    {  380, 7811,  113828, 11, }, // America/Dawson_Creek
    {  380,  805,    1201,  6, }, // America/Denver
    {  380, 3351,   35129,  7, }, // America/Detroit
    {  380, 5656,    1214,  8, }, // America/Dominica
    {  380,  893,   50207,  8, }, // America/Edmonton
    {  380, 7832,   50215,  8, }, // America/Eirunepe
    {  380, 5343,  113839, 11, }, // America/El_Salvador
    {  380, 7849,   50223, 12, }, // America/Fort_Nelson
    {  380, 7869,    1257,  9, }, // America/Fortaleza
    {  380, 7887,   35163,  9, }, // America/Glace_Bay
    {  380, 6844,   35172,  7, }, // America/Goose_Bay
    {  380, 4514,  113850, 10, }, // America/Grand_Turk
    {  380, 5687,    1292,  7, }, // America/Grenada
    {  380, 5703,  113860,  9, }, // America/Guadeloupe
    {  380, 3662,    1308,  9, }, // America/Guatemala
    {  380, 4743,  113869,  8, }, // America/Guayaquil
    {  380, 6895,  113877,  6, }, // America/Guyana
    {  380, 1923,   50265,  8, }, // America/Halifax
    {  380, 2221,    1339,  6, }, // America/Havana
    {  380, 7905,  113883,  9, }, // America/Hermosillo
    {  380,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {  380,  481,    1365, 13, }, // America/Indiana/Knox
    {  380, 7924,    1378, 16, }, // America/Indiana/Marengo
    {  380, 7948,  113892, 19, }, // America/Indiana/Petersburg
    {  380, 7975,  113911, 18, }, // America/Indiana/Tell_City
    {  380, 8001,  104904, 14, }, // America/Indiana/Vevay
    {  380, 8023,  113929, 15, }, // America/Indiana/Vincennes
    {  380, 8049,   54875, 16, }, // America/Indiana/Winamac
    {  380, 8073,    1452,  6, }, // America/Inuvik
    {  380,  660,    1458,  7, }, // America/Iqaluit
    {  380, 2665,   26461,  6, }, // America/Jamaica
    {  380, 5282,   32939,  4, }, // America/Juneau
    {  380,  521,  113944,  8, }, // America/Kentucky/Louisville
    {  380, 8088,  113952, 20, }, // America/Kentucky/Monticello
    {  380, 5621,   35231, 10, }, // America/Kralendijk
    {  380, 4278,   35241,  6, }, // America/La_Paz
    {  380, 7132,    1520,  4, }, // America/Lima
    {  380, 3390,  113972, 11, }, // America/Los_Angeles
    {  380, 5849,  113983, 19, }, // America/Lower_Princes
    {  380, 8116,   35279,  6, }, // America/Maceio
    {  380, 8131,    1559,  7, }, // America/Managua
    {  380, 1892,    1566,  6, }, // America/Manaus
    {  380, 5814,    1572,  6, }, // America/Marigot
    {  380, 5722,    1578,  8, }, // America/Martinique
    {  380, 8147,    1587,  9, }, // America/Matamoros
    {  380, 2779,   33020,  8, }, // America/Mazatlan
    {  380, 8165,    1604,  8, }, // America/Menominee
    {  380, 8183,    1612,  6, }, // America/Merida
    {  380, 8198,    1618, 10, }, // America/Metlakatla
    {  380, 2811,  114002, 12, }, // America/Mexico_City
    {  380, 4293,    1634,  7, }, // America/Miquelon
    {  380, 8217,    1641,  7, }, // America/Moncton
    {  380, 8233,   35285,  8, }, // America/Monterrey
    {  380, 4000,    1657, 10, }, // America/Montevideo
    {  380, 5741,  114014,  8, }, // America/Montserrat
    {  380, 5383,    1677,  6, }, // America/Nassau
    {  380, 3294,   50338,  8, }, // America/New_York
    {  380, 8251,    1691,  3, }, // America/Nome
    {  380, 1834,    1694,  7, }, // America/Noronha
    {  380, 8264,  114022, 20, }, // America/North_Dakota/Beulah
    {  380, 8292,  114042, 24, }, // America/North_Dakota/Center
    {  380, 8320,  114066, 24, }, // America/North_Dakota/New_Salem
    {  380,  393,    1772,  4, }, // America/Nuuk
    {  380, 8351,    1776,  7, }, // America/Ojinaga
    {  380, 5413,    1783,  6, }, // America/Panama
    {  380, 6106,    1789, 10, }, // America/Paramaribo
    {  380, 3224,   33144,  6, }, // America/Phoenix
    {  380, 3847,    1805, 12, }, // America/Port-au-Prince
    {  380, 5871,  114090, 13, }, // America/Port_of_Spain
    {  380, 8367,  114103, 11, }, // America/Porto_Velho
    {  380, 5760,   50414, 11, }, // America/Puerto_Rico
    {  380, 3932,    1852, 12, }, // America/Punta_Arenas
    {  380, 8387,  114114, 12, }, // America/Rankin_Inlet
    {  380, 8408,    1876,  6, }, // America/Recife
    {  380, 1979,   33212,  6, }, // America/Regina
    {  380, 8423,  114126,  7, }, // America/Resolute
    {  380,  695,   33225, 10, }, // America/Rio_Branco
    {  380, 8440,   33235,  8, }, // America/Santarem
    {  380, 2165,    1915,  8, }, // America/Santiago
    {  380, 6044,   33252, 13, }, // America/Santo_Domingo
    {  380, 1862,   35386,  9, }, // America/Sao_Paulo
    {  380, 6862,  114133, 16, }, // America/Scoresbysund
    {  380, 8457,    1956,  5, }, // America/Sitka
    {  380, 8471,  114149, 14, }, // America/St_Barthelemy
    {  380, 2045,  114163,  9, }, // America/St_Johns
    {  380, 5780,    1983,  9, }, // America/St_Kitts
    {  380, 5797,  114172, 11, }, // America/St_Lucia
    {  380,  855,    2003, 10, }, // America/St_Thomas
    {  380, 5830,    2013, 12, }, // America/St_Vincent
    {  380, 8493,  114183, 13, }, // America/Swift_Current
    {  380, 5363,   50475, 12, }, // America/Tegucigalpa
    {  380, 5673,    2049,  4, }, // America/Thule
    {  380,  313,    2053,  7, }, // America/Tijuana
    {  380,  608,    2060,  7, }, // America/Toronto
    {  380, 5605,    2067,  7, }, // America/Tortola
    {  380, 2077,    2074,  8, }, // America/Vancouver
    {  380, 2128,   27017,  8, }, // America/Whitehorse
    {  380,  734,  114196,  8, }, // America/Winnipeg
    {  380, 8515,   27032,  6, }, // America/Yakutat
    {  380, 5918,   35501,  5, }, // Antarctica/Casey
    {  380, 6027,  114204,  5, }, // Antarctica/Davis
    {  380, 6066,  114209, 13, }, // Antarctica/DumontDUrville
    {  380, 8531,    2131,  8, }, // Antarctica/Macquarie
    {  380, 7029,    2139,  6, }, // Antarctica/Mawson
    {  380, 7076,  114222,  8, }, // Antarctica/McMurdo
    {  380, 8552,    2154,  6, }, // Antarctica/Palmer
    {  380, 7189,    2160,  6, }, // Antarctica/Rothera
    {  380, 7258,  114230,  4, }, // Antarctica/Syowa
    {  380, 8570,    2171,  5, }, // Antarctica/Troll
    {  380, 7337,   33428,  6, }, // Antarctica/Vostok
    {  380, 1413,  114234, 11, }, // Arctic/Longyearbyen
    {  380, 5487,    2192,  4, }, // Asia/Aden
    {  380, 5297,    2196,  6, }, // Asia/Almaty
    {  380, 3870,    2202,  5, }, // Asia/Amman
    {  380, 5428,    2207,  6, }, // Asia/Anadyr
    {  380, 5440,    2213,  5, }, // Asia/Aqtau
    {  380, 5270,    2218,  6, }, // Asia/Aqtobe
    {  380,  964,    2224,  7, }, // Asia/Ashgabat
    {  380, 8587,    2231,  6, }, // Asia/Atyrau
    {  380, 3509,    2237,  6, }, // Asia/Baghdad
    {  380, 5451,   35560,  7, }, // Asia/Bahrain
    {  380, 3555,    2250,  4, }, // Asia/Baku
    {  380, 4368,    2254,  7, }, // Asia/Bangkok
    {  380, 3473,    2261,  7, }, // Asia/Barnaul
    {  380, 3988,   27145,  6, }, // Asia/Beirut
    {  380, 3680,    2274,  6, }, // Asia/Bishkek
    {  380, 5906,   27157,  6, }, // Asia/Brunei
    {  380, 4503,    2286,  4, }, // Asia/Chita
    {  380, 5963,   35567,  9, }, // Asia/Choibalsan
    {  380, 4413,    2299,  7, }, // Asia/Colombo
    {  380, 4442,    2306,  6, }, // Asia/Damascus
    {  380, 1060,    2312,  5, }, // Asia/Dhaka
    {  380, 6125,    2317,  4, }, // Asia/Dili
    {  380, 3498,   27187,  5, }, // Asia/Dubai
    {  380, 6092,    2326,  7, }, // Asia/Dushanbe
    {  380, 8599,    2333,  9, }, // Asia/Famagusta
    {  380, 8614,    2342,  4, }, // Asia/Gaza
    {  380, 4697,   33498,  6, }, // Asia/Hebron
    {  380, 1226,   33504,  9, }, // Asia/Ho_Chi_Minh
    {  380, 2576,   50560,  7, }, // Asia/Hong_Kong
    {  380, 4673,    2366,  4, }, // Asia/Hovd
    {  380, 4086,    2370,  7, }, // Asia/Irkutsk
    {  380, 6969,  114245,  7, }, // Asia/Jakarta
    {  380, 6955,  114252,  8, }, // Asia/Jayapura
    {  380, 1257,    2392,  9, }, // Asia/Jerusalem
    {  380, 3462,    2401,  5, }, // Asia/Kabul
    {  380, 4218,    2420,  8, }, // Asia/Kamchatka
    {  380, 4141,    2431,  6, }, // Asia/Karachi
    {  380, 1152,    2437,  8, }, // Asia/Kathmandu
    {  380, 8624,  114260,  7, }, // Asia/Khandyga
    {  380,  992,   50608,  9, }, // Asia/Kolkata
    {  380, 4099,   35599, 10, }, // Asia/Krasnoyarsk
    {  380, 4761,   33570, 12, }, // Asia/Kuala_Lumpur
    {  380, 5893,  114267,  5, }, // Asia/Kuching
    {  380, 5464,   27284,  6, }, // Asia/Kuwait
    {  380, 1178,   33588,  5, }, // Asia/Macau
    {  380, 3919,    2500,  7, }, // Asia/Magadan
    {  380, 1316,  114272,  8, }, // Asia/Makassar
    {  380, 7145,    2514,  6, }, // Asia/Manila
    {  380, 6883,   33593,  6, }, // Asia/Muscat
    {  380, 2446,  105053,  7, }, // Asia/Nicosia
    {  380, 8638,    2533, 11, }, // Asia/Novokuznetsk
    {  380, 4037,    2544, 11, }, // Asia/Novosibirsk
    {  380, 4131,    2555,  4, }, // Asia/Omsk
    {  380, 7108,   33618,  4, }, // Asia/Oral
    {  380, 6924,   50617,  8, }, // Asia/Phnom_Penh
    {  380, 8656,    2572,  9, }, // Asia/Pontianak
    {  380, 4116,   27354,  7, }, // Asia/Pyongyang
    {  380, 5476,    2588,  5, }, // Asia/Qatar
    {  380, 8671,   35626,  8, }, // Asia/Qostanay
    {  380, 4171,    2601,  9, }, // Asia/Qyzylorda
    {  380, 3486,  114280,  5, }, // Asia/Riyadh
    {  380, 4310,    2623,  7, }, // Asia/Sakhalin
    {  380, 7208,    2630,  9, }, // Asia/Samarkand
    {  380, 3116,    2639,  4, }, // Asia/Seoul
    {  380, 1020,   27392,  6, }, // Asia/Shanghai
    {  380, 3137,    2649,  8, }, // Asia/Singapore
    {  380, 4199,    2657, 13, }, // Asia/Srednekolymsk
    {  380, 3100,   59565,  6, }, // Asia/Taipei
    {  380, 4683,    2676,  7, }, // Asia/Tashkent
    {  380, 3817,   33686,  7, }, // Asia/Tbilisi
    {  380, 2638,    2687,  7, }, // Asia/Tehran
    {  380, 1284,   33700,  5, }, // Asia/Thimphu
    {  380, 2687,    2701,  5, }, // Asia/Tokyo
    {  380, 4474,    2706,  5, }, // Asia/Tomsk
    {  380, 1346,    2711, 10, }, // Asia/Ulaanbaatar
    {  380, 1126,  114285,  6, }, // Asia/Urumqi
    {  380, 8685,   50651,  9, }, // Asia/Ust-Nera
    {  380, 6940,  114291,  8, }, // Asia/Vientiane
    {  380, 4612,    2742, 11, }, // Asia/Vladivostok
    {  380, 4730,   35672,  6, }, // Asia/Yakutsk
    {  380, 1202,   27502,  6, }, // Asia/Yangon
    {  380, 3785,    2766, 12, }, // Asia/Yekaterinburg
    {  380, 3649,    2778,  6, }, // Asia/Yerevan
    {  380, 3565,    2784,  4, }, // Atlantic/Azores
    {  380, 5588,   33764,  7, }, // Atlantic/Bermuda
    {  380, 6591,  114299,  6, }, // Atlantic/Canary
    {  380, 3629,  114305, 10, }, // Atlantic/Cape_Verde
    {  380, 1379,    2845,  5, }, // Atlantic/Faroe
    {  380, 8699,   35753,  7, }, // Atlantic/Madeira
    {  380, 2614,   50740,  9, }, // Atlantic/Reykjavik
    {  380, 7235,  114315, 13, }, // Atlantic/South_Georgia
    {  380, 6784,  114328, 10, }, // Atlantic/St_Helena
    {  380, 6607,   50782,  6, }, // Atlantic/Stanley
    {  380, 1654,    2926,  8, }, // Australia/Adelaide
    {  380, 1619,    2934,  7, }, // Australia/Brisbane
    {  380, 1783,  114338, 11, }, // Australia/Broken_Hill
    {  380, 1567,    2952,  6, }, // Australia/Darwin
    {  380, 3539,   27699,  4, }, // Australia/Eucla
    {  380, 1500,    2963,  6, }, // Australia/Hobart
    {  380, 8716,    2969,  8, }, // Australia/Lindeman
    {  380, 1531,   35811,  8, }, // Australia/Lord_Howe
    {  380, 1711,   27711,  8, }, // Australia/Melbourne
    {  380, 1746,    2992,  4, }, // Australia/Perth
    {  380, 1447,   50802,  6, }, // Australia/Sydney
    {  380, 6390,    3002,  9, }, // Europe/Amsterdam
    {  380, 6149,    3011,  7, }, // Europe/Andorra
    {  380, 3522,   27740,  9, }, // Europe/Astrakhan
    {  380, 6577,   59588,  5, }, // Europe/Athens
    {  380, 6285,    3030,  7, }, // Europe/Belgrade
    {  380, 4659,    3037,  6, }, // Europe/Berlin
    {  380, 6437,    3043, 10, }, // Europe/Bratislava
    {  380, 6178,   50822,  8, }, // Europe/Brussels
    {  380, 3830,    3061,  8, }, // Europe/Bucharest
    {  380, 3708,    3069,  8, }, // Europe/Budapest
    {  380, 8751,  114349,  8, }, // Europe/Busingen
    {  380, 2475,    3100,  7, }, // Europe/Chisinau
    {  380, 6238,   50849, 10, }, // Europe/Copenhagen
    {  380, 2260,    3117,  6, }, // Europe/Dublin
    {  380, 6256,    3123,  9, }, // Europe/Gibraltar
    {  380, 8767,    3132,  6, }, // Europe/Guernsey
    {  380, 6561,   50859,  9, }, // Europe/Helsinki
    {  380, 8783,  105203,  9, }, // Europe/Isle_of_Man
    {  380, 1097,    3155,  7, }, // Europe/Istanbul
    {  380, 8802,  114357,  5, }, // Europe/Jersey
    {  380, 3881,    3167, 11, }, // Europe/Kaliningrad
    {  380, 8924,    3178,  5, }, // Europe/Kirov
    {  380, 2419,    3183,  4, }, // Europe/Kyiv
    {  380, 3078,    3187,  8, }, // Europe/Lisbon
    {  380, 6455,   27891,  7, }, // Europe/Ljubljana
    {  380, 2393,    3204,  6, }, // Europe/London
    {  380, 6314,   27904, 10, }, // Europe/Luxembourg
    {  380, 4794,    3221,  6, }, // Europe/Madrid
    {  380, 6346,   27920,  6, }, // Europe/Malta
    {  380, 6531,    3232,  9, }, // Europe/Mariehamn
    {  380, 3595,    3241,  5, }, // Europe/Minsk
    {  380, 6359,    3246,  6, }, // Europe/Monaco
    {  380, 3443,    3252,  6, }, // Europe/Moscow
    {  380, 6407,    3258,  4, }, // Europe/Oslo
    {  380, 4186,    3262,  5, }, // Europe/Paris
    {  380, 6373,    3267,  9, }, // Europe/Podgorica
    {  380, 6224,    3276,  5, }, // Europe/Prague
    {  380, 8816,    3281,  4, }, // Europe/Riga
    {  380, 6273,    3285,  3, }, // Europe/Rome
    {  380, 4233,    3288,  6, }, // Europe/Samara
    {  380, 6419,   34025, 10, }, // Europe/San_Marino
    {  380, 6194,    3304,  7, }, // Europe/Sarajevo
    {  380, 4353,    3311,  7, }, // Europe/Saratov
    {  380, 8828,   50883, 11, }, // Europe/Simferopol
    {  380, 6332,  114362,  6, }, // Europe/Skopje
    {  380, 6548,   35907,  5, }, // Europe/Sofia
    {  380, 6472,   50900,  9, }, // Europe/Stockholm
    {  380, 8846,    3346,  6, }, // Europe/Tallinn
    {  380, 6135,    3352,  6, }, // Europe/Tirane
    {  380, 8861,   50909,  9, }, // Europe/Ulyanovsk
    {  380, 6301,  114368,  5, }, // Europe/Vaduz
    {  380, 6516,    3372,  7, }, // Europe/Vatican
    {  380, 6164,    3379,  4, }, // Europe/Vienna
    {  380, 8878,   50918,  7, }, // Europe/Vilnius
    {  380, 4629,    3390,  9, }, // Europe/Volgograd
    {  380, 3055,    3399,  7, }, // Europe/Warsaw
    {  380, 6210,    3406,  6, }, // Europe/Zagreb
    {  380, 6489,   35931,  5, }, // Europe/Zurich
    {  380, 4980,   34105, 12, }, // Indian/Antananarivo
    {  380, 6910,    3430,  5, }, // Indian/Chagos
    {  380, 5979,  114373,  7, }, // Indian/Christmas
    {  380, 5996,    3447,  5, }, // Indian/Cocos
    {  380, 4931,   34122,  6, }, // Indian/Comoro
    {  380, 6624,    3469,  8, }, // Indian/Kerguelen
    {  380, 7223,   28153,  3, }, // Indian/Mahe
    {  380, 7013,  105268, 16, }, // Indian/Maldives
    {  380, 3971,   55228,  8, }, // Indian/Mauritius
    {  380, 5000,   55236,  6, }, // Indian/Mayotte
    {  380, 7174,   35990,  7, }, // Indian/Reunion
    {  380, 4324,    3507,  4, }, // Pacific/Apia
    {  380,  932,  114380,  6, }, // Pacific/Auckland
    {  380, 3608,   51013,  9, }, // Pacific/Bougainville
    {  380, 2863,    3525,  5, }, // Pacific/Chatham
    {  380, 3022,  114386,  4, }, // Pacific/Chuuk
    {  380, 2201,  114390,  5, }, // Pacific/Easter
    {  380, 7323,   28219,  4, }, // Pacific/Efate
    {  380, 7290,    3550,  7, }, // Pacific/Fakaofo
    {  380, 3804,  114395,  4, }, // Pacific/Fiji
    {  380, 7306,    3561,  8, }, // Pacific/Funafuti
    {  380, 6641,    3569,  9, }, // Pacific/Galapagos
    {  380, 6659,   36033,  6, }, // Pacific/Gambier
    {  380, 3724,    3604, 11, }, // Pacific/Guadalcanal
    {  380, 5935,    3615,  4, }, // Pacific/Guam
    {  380, 2929,   51108,  8, }, // Pacific/Honolulu
    {  380, 2897,   51116,  9, }, // Pacific/Kanton
    {  380, 3900,    3625, 10, }, // Pacific/Kiritimati
    {  380, 6982,  114399,  6, }, // Pacific/Kosrae
    {  380, 2708,  114405,  9, }, // Pacific/Kwajalein
    {  380, 4779,  114414,  6, }, // Pacific/Majuro
    {  380, 3953,  114420,  8, }, // Pacific/Marquesas
    {  380, 8909,   51148,  6, }, // Pacific/Midway
    {  380, 7047,    3689,  5, }, // Pacific/Nauru
    {  380, 7095,   51162,  4, }, // Pacific/Niue
    {  380, 4070,    3698,  7, }, // Pacific/Norfolk
    {  380, 7061,    3705,  5, }, // Pacific/Noumea
    {  380, 2991,   34254,  9, }, // Pacific/Pago_Pago
    {  380, 7118,    3719,  5, }, // Pacific/Palau
    {  380, 7157,   55309,  7, }, // Pacific/Pitcairn
    {  380, 2961,   36092,  6, }, // Pacific/Pohnpei
    {  380, 4709,  114428, 12, }, // Pacific/Port_Moresby
    {  380, 6009,  114440,  9, }, // Pacific/Rarotonga
    {  380, 5948,   28394,  6, }, // Pacific/Saipan
    {  380, 7275,    3763,  5, }, // Pacific/Tahiti
    {  380, 6675,    3768,  6, }, // Pacific/Tarawa
    {  380, 4485,    3774,  9, }, // Pacific/Tongatapu
    {  380, 7355,   36115,  4, }, // Pacific/Wake
    {  380, 7368,   36119,  5, }, // Pacific/Wallis
    {  381,   14,       0,  6, }, // Africa/Asmara Lakota/Latin/United States
    {  381, 4337,       6,  8, }, // Africa/Sao_Tome
    {  381, 4154,      14,  8, }, // America/Asuncion
    {  381,  237,      22,  8, }, // America/Atikokan
    {  381, 7586,      30, 17, }, // America/Bahia_Banderas
    {  381, 3770,      47,  6, }, // America/Cancun
    {  381, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  381, 5640,      66,  7, }, // America/Curacao
    {  381,  481,      73, 13, }, // America/Indiana/Knox
    {  381, 7924,      86, 16, }, // America/Indiana/Marengo
    {  381, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  381, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  381, 8001,     139, 14, }, // America/Indiana/Vevay
    {  381, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  381, 8049,     171, 16, }, // America/Indiana/Winamac
    {  381, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  381, 5849,     207, 22, }, // America/Lower_Princes
    {  381, 8183,     229,  6, }, // America/Merida
    {  381, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  381, 8292,     255, 20, }, // America/North_Dakota/Center
    {  381, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  381,  393,     298,  4, }, // America/Nuuk
    {  381, 6862,     302, 16, }, // America/Scoresbysund
    {  381, 8471,     318, 14, }, // America/St_Barthelemy
    {  381, 2045,     332, 10, }, // America/St_Johns
    {  381, 5780,     342,  9, }, // America/St_Kitts
    {  381, 5797,     351,  9, }, // America/St_Lucia
    {  381,  855,     360, 10, }, // America/St_Thomas
    {  381, 5830,     370, 11, }, // America/St_Vincent
    {  381, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  381, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  381, 1152,     413,  9, }, // Asia/Kathmandu
    {  381,  992,     422,  7, }, // Asia/Kolkata
    {  381, 1178,     429,  5, }, // Asia/Macau
    {  381, 1202,     442,  6, }, // Asia/Yangon
    {  381, 1379,     448,  5, }, // Atlantic/Faroe
    {  381, 6784,     453, 10, }, // Atlantic/St_Helena
    {  381, 2419,     463,  4, }, // Europe/Kyiv
    {  381, 7174,     467,  7, }, // Indian/Reunion
    {  381, 3022,     474,  5, }, // Pacific/Chuuk
    {  381, 2961,     479,  7, }, // Pacific/Pohnpei
    {  382,   14,       0,  6, }, // Africa/Asmara Langi/Latin/Tanzania
    {  382, 4337,       6,  8, }, // Africa/Sao_Tome
    {  382, 4154,      14,  8, }, // America/Asuncion
    {  382,  237,      22,  8, }, // America/Atikokan
    {  382, 7586,      30, 17, }, // America/Bahia_Banderas
    {  382, 3770,      47,  6, }, // America/Cancun
    {  382, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  382, 5640,      66,  7, }, // America/Curacao
    {  382,  481,      73, 13, }, // America/Indiana/Knox
    {  382, 7924,      86, 16, }, // America/Indiana/Marengo
    {  382, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  382, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  382, 8001,     139, 14, }, // America/Indiana/Vevay
    {  382, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  382, 8049,     171, 16, }, // America/Indiana/Winamac
    {  382, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  382, 5849,     207, 22, }, // America/Lower_Princes
    {  382, 8183,     229,  6, }, // America/Merida
    {  382, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  382, 8292,     255, 20, }, // America/North_Dakota/Center
    {  382, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  382,  393,     298,  4, }, // America/Nuuk
    {  382, 6862,     302, 16, }, // America/Scoresbysund
    {  382, 8471,     318, 14, }, // America/St_Barthelemy
    {  382, 2045,     332, 10, }, // America/St_Johns
    {  382, 5780,     342,  9, }, // America/St_Kitts
    {  382, 5797,     351,  9, }, // America/St_Lucia
    {  382,  855,     360, 10, }, // America/St_Thomas
    {  382, 5830,     370, 11, }, // America/St_Vincent
    {  382, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  382, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  382, 1152,     413,  9, }, // Asia/Kathmandu
    {  382,  992,     422,  7, }, // Asia/Kolkata
    {  382, 1178,     429,  5, }, // Asia/Macau
    {  382, 1202,     442,  6, }, // Asia/Yangon
    {  382, 1379,     448,  5, }, // Atlantic/Faroe
    {  382, 6784,     453, 10, }, // Atlantic/St_Helena
    {  382, 2419,     463,  4, }, // Europe/Kyiv
    {  382, 7174,     467,  7, }, // Indian/Reunion
    {  382, 3022,     474,  5, }, // Pacific/Chuuk
    {  382, 2961,     479,  7, }, // Pacific/Pohnpei
    {  383, 6751,  114449,  8, }, // Africa/Abidjan Lao/Lao/Laos
    {  383, 6723,  114457,  6, }, // Africa/Accra
    {  383, 4961,  114463, 13, }, // Africa/Addis_Ababa
    {  383, 7383,  114476,  6, }, // Africa/Algiers
    {  383,   14,  114482,  7, }, // Africa/Asmara
    {  383,   44,  114489,  6, }, // Africa/Bamako
    {  383, 5159,  114495,  6, }, // Africa/Bangui
    {  383, 6709,  114501,  5, }, // Africa/Banjul
    {  383, 7398,  114506,  7, }, // Africa/Bissau
    {  383, 4859,  114513,  8, }, // Africa/Blantyre
    {  383, 5189,  114521,  8, }, // Africa/Brazzaville
    {  383, 4824,  114529,  9, }, // Africa/Bujumbura
    {  383, 2242,  114538,  4, }, // Africa/Cairo
    {  383, 4019,  114542, 10, }, // Africa/Casablanca
    {  383, 7412,  114552,  5, }, // Africa/Ceuta
    {  383, 6736,  114557,  7, }, // Africa/Conakry
    {  383, 6803,  114564,  4, }, // Africa/Dakar
    {  383, 5032,  114568, 10, }, // Africa/Dar_es_Salaam
    {  383, 4945,  114578,  6, }, // Africa/Djibouti
    {  383, 5145,  114584,  6, }, // Africa/Douala
    {  383, 5068,  114590,  9, }, // Africa/El_Aaiun
    {  383, 6816,  114599,  6, }, // Africa/Freetown
    {  383, 4808,  114605,  7, }, // Africa/Gaborone
    {  383, 4917,  114612,  6, }, // Africa/Harare
    {  383, 4381,  114618, 13, }, // Africa/Johannesburg
    {  383, 4401,  114631,  4, }, // Africa/Juba
    {  383, 5053,  114635,  6, }, // Africa/Kampala
    {  383, 4426,  114641,  5, }, // Africa/Khartoum
    {  383, 4875,  114646,  6, }, // Africa/Kigali
    {  383, 5208,  114652,  7, }, // Africa/Kinshasa
    {  383, 4646,  114659,  5, }, // Africa/Lagos
    {  383, 5238,  114664,  9, }, // Africa/Libreville
    {  383, 6832,  114673,  4, }, // Africa/Lome
    {  383, 5113,  114677,  5, }, // Africa/Luanda
    {  383, 4841,  114682,  8, }, // Africa/Lubumbashi
    {  383, 4903,  114690,  6, }, // Africa/Lusaka
    {  383, 5224,  114696,  6, }, // Africa/Malabo
    {  383, 4889,  114702,  6, }, // Africa/Maputo
    {  383, 5099,  114708,  6, }, // Africa/Maseru
    {  383, 5084,  114714,  8, }, // Africa/Mbabane
    {  383, 5015,  114722,  8, }, // Africa/Mogadishu
    {  383, 6997,  114730,  8, }, // Africa/Monrovia
    {  383, 3755,  114738,  6, }, // Africa/Nairobi
    {  383, 5173,  114744, 10, }, // Africa/Ndjamena
    {  383, 5256,  114754,  6, }, // Africa/Niamey
    {  383, 6766,  114760,  8, }, // Africa/Nouakchott
    {  383, 6690,  114768,  8, }, // Africa/Ouagadougou
    {  383, 5127,  114776,  9, }, // Africa/Porto-Novo
    {  383, 4337,  114785,  7, }, // Africa/Sao_Tome
    {  383, 2732,  114792,  7, }, // Africa/Tripoli
    {  383, 6503,  114799,  5, }, // Africa/Tunis
    {  383, 4054,  114804,  6, }, // Africa/Windhoek
    {  383,  132,  114810,  5, }, // America/Adak
    {  383, 3183,  114815,  9, }, // America/Anchorage
    {  383, 5524,  114824,  8, }, // America/Anguilla
    {  383, 5541,  114832,  8, }, // America/Antigua
    {  383, 4456,  114840,  9, }, // America/Araguaina
    {  383,  166,  114849, 12, }, // America/Argentina/Buenos_Aires
    {  383,   91,  114861,  8, }, // America/Argentina/Catamarca
    {  383,  270,  114869,  6, }, // America/Argentina/Cordoba
    {  383,  441,  114875,  5, }, // America/Argentina/Jujuy
    {  383, 7425,  114880,  9, }, // America/Argentina/La_Rioja
    {  383,  565,  114889,  9, }, // America/Argentina/Mendoza
    {  383, 7452,  114898, 13, }, // America/Argentina/Rio_Gallegos
    {  383, 7483,  114911,  5, }, // America/Argentina/Salta
    {  383, 7507,  114916,  6, }, // America/Argentina/San_Juan
    {  383, 5497,  114922,  8, }, // America/Argentina/San_Luis
    {  383, 7534,  114930,  7, }, // America/Argentina/Tucuman
    {  383, 7560,  114937,  7, }, // America/Argentina/Ushuaia
    {  383, 5557,  114944,  6, }, // America/Aruba
    {  383, 4154,  114950, 10, }, // America/Asuncion
    {  383,  237,  114960,  9, }, // America/Atikokan
    {  383, 3581,  114969,  5, }, // America/Bahia
    {  383, 7586,  114974, 15, }, // America/Bahia_Banderas
    {  383, 5571,  114989,  7, }, // America/Barbados
    {  383, 7609,  114996,  5, }, // America/Belem
    {  383, 5309,  115001,  5, }, // America/Belize
    {  383, 7623,  115006, 11, }, // America/Blanc-Sablon
    {  383, 7644,  115017,  9, }, // America/Boa_Vista
    {  383, 4263,  115026,  6, }, // America/Bogota
    {  383, 7662,  115032,  5, }, // America/Boise
    {  383, 7676,  115037, 10, }, // America/Cambridge_Bay
    {  383, 7698,  115047, 10, }, // America/Campo_Grande
    {  383, 3770,  115057,  6, }, // America/Cancun
    {  383, 4596,  115063,  7, }, // America/Caracas
    {  383, 4247,  115070,  5, }, // America/Cayenne
    {  383, 5398,  115075,  5, }, // America/Cayman
    {  383, 3251,  115080,  6, }, // America/Chicago
    {  383, 7719,  115086,  6, }, // America/Chihuahua
    {  383, 7737,  115092, 14, }, // America/Ciudad_Juarez
    {  383, 5324,  115106,  9, }, // America/Costa_Rica
    {  383, 7759,  115115,  7, }, // America/Creston
    {  383, 3693,  115122,  6, }, // America/Cuiaba
    {  383, 5640,  115128,  8, }, // America/Curacao
    {  383, 7775,  115136,  9, }, // America/Danmarkshavn
    {  383, 7796,  115145,  5, }, // America/Dawson
    {  383, 7811,  115150, 10, }, // America/Dawson_Creek
    {  383,  805,  115160,  7, }, // America/Denver
    {  383, 3351,  115167,  6, }, // America/Detroit
    {  383, 5656,  115173,  8, }, // America/Dominica
    {  383,  893,  115181,  9, }, // America/Edmonton
    {  383, 7832,  115190,  8, }, // America/Eirunepe
    {  383, 5343,  115198, 10, }, // America/El_Salvador
    {  383, 7849,  115208, 10, }, // America/Fort_Nelson
    {  383, 7869,  115218,  8, }, // America/Fortaleza
    {  383, 7887,  115226,  6, }, // America/Glace_Bay
    {  383, 6844,  115232,  5, }, // America/Goose_Bay
    {  383, 4514,  115237,  9, }, // America/Grand_Turk
    {  383, 5687,  115246,  7, }, // America/Grenada
    {  383, 5703,  115253,  8, }, // America/Guadeloupe
    {  383, 3662,  115261,  9, }, // America/Guatemala
    {  383, 4743,  115270,  8, }, // America/Guayaquil
    {  383, 6895,  115278,  6, }, // America/Guyana
    {  383, 1923,  115284,  8, }, // America/Halifax
    {  383, 2221,  115292,  6, }, // America/Havana
    {  383, 7905,  115298,  8, }, // America/Hermosillo
    {  383,  348,  115306, 14, }, // America/Indiana/Indianapolis
    {  383,  481,  115320, 14, }, // America/Indiana/Knox
    {  383, 7924,  115334, 17, }, // America/Indiana/Marengo
    {  383, 7948,  115351, 20, }, // America/Indiana/Petersburg
    {  383, 7975,  115371, 18, }, // America/Indiana/Tell_City
    {  383, 8001,  115389, 14, }, // America/Indiana/Vevay
    {  383, 8023,  115403, 19, }, // America/Indiana/Vincennes
    {  383, 8049,  115422, 18, }, // America/Indiana/Winamac
    {  383, 8073,  115440,  7, }, // America/Inuvik
    {  383,  660,  115447,  8, }, // America/Iqaluit
    {  383, 2665,  115455,  6, }, // America/Jamaica
    {  383, 5282,  115461,  4, }, // America/Juneau
    {  383,  521,  115465,  7, }, // America/Kentucky/Louisville
    {  383, 8088,  115472, 20, }, // America/Kentucky/Monticello
    {  383, 5621,  115492,  9, }, // America/Kralendijk
    {  383, 4278,  115501,  5, }, // America/La_Paz
    {  383, 7132,  115506,  4, }, // America/Lima
    {  383, 3390,  115510, 11, }, // America/Los_Angeles
    {  383, 5849,  115521, 19, }, // America/Lower_Princes
    {  383, 8116,  115540,  6, }, // America/Maceio
    {  383, 8131,  115546,  7, }, // America/Managua
    {  383, 1892,  115553,  8, }, // America/Manaus
    {  383, 5814,  115561,  7, }, // America/Marigot
    {  383, 5722,  115568,  7, }, // America/Martinique
    {  383, 8147,  115575,  9, }, // America/Matamoros
    {  383, 2779,  115584,  8, }, // America/Mazatlan
    {  383, 8165,  115592,  8, }, // America/Menominee
    {  383, 8183,  115600,  7, }, // America/Merida
    {  383, 8198,  115607, 11, }, // America/Metlakatla
    {  383, 2811,  115618, 13, }, // America/Mexico_City
    {  383, 4293,  115631,  8, }, // America/Miquelon
    {  383, 8217,  115639,  6, }, // America/Moncton
    {  383, 8233,  115645,  9, }, // America/Monterrey
    {  383, 4000,  115654, 11, }, // America/Montevideo
    {  383, 5741,  115665,  9, }, // America/Montserrat
    {  383, 5383,  115674,  5, }, // America/Nassau
    {  383, 3294,  115679,  6, }, // America/New_York
    {  383, 8251,  115685,  3, }, // America/Nome
    {  383, 1834,  115688,  7, }, // America/Noronha
    {  383, 8264,  115695, 17, }, // America/North_Dakota/Beulah
    {  383, 8292,  115712, 19, }, // America/North_Dakota/Center
    {  383, 8320,  115731, 21, }, // America/North_Dakota/New_Salem
    {  383,  393,  115752,  3, }, // America/Nuuk
    {  383, 8351,  115755,  8, }, // America/Ojinaga
    {  383, 5413,  115763,  6, }, // America/Panama
    {  383, 6106,  115769, 10, }, // America/Paramaribo
    {  383, 3224,  115779,  5, }, // America/Phoenix
    {  383, 3847,  115784,  8, }, // America/Port-au-Prince
    {  383, 5871,  115792, 11, }, // America/Port_of_Spain
    {  383, 8367,  115803, 10, }, // America/Porto_Velho
    {  383, 5760,  115813,  9, }, // America/Puerto_Rico
    {  383, 3932,  115822, 14, }, // America/Punta_Arenas
    {  383, 8387,  115836, 14, }, // America/Rankin_Inlet
    {  383, 8408,  115850,  6, }, // America/Recife
    {  383, 1979,  115856,  6, }, // America/Regina
    {  383, 8423,  115862,  7, }, // America/Resolute
    {  383,  695,  115869, 11, }, // America/Rio_Branco
    {  383, 8440,  115880,  8, }, // America/Santarem
    {  383, 2165,  115888,  9, }, // America/Santiago
    {  383, 6044,  115897, 12, }, // America/Santo_Domingo
    {  383, 1862,  115909, 11, }, // America/Sao_Paulo
    {  383, 6862,  115920, 12, }, // America/Scoresbysund
    {  383, 8457,  115932,  5, }, // America/Sitka
    {  383, 8471,  115937, 11, }, // America/St_Barthelemy
    {  383, 2045,  115948,  7, }, // America/St_Johns
    {  383, 5780,  115955,  9, }, // America/St_Kitts
    {  383, 5797,  115964,  9, }, // America/St_Lucia
    {  383,  855,  115973,  9, }, // America/St_Thomas
    {  383, 5830,  115982, 10, }, // America/St_Vincent
    {  383, 8493,  115992, 12, }, // America/Swift_Current
    {  383, 5363,  116004, 11, }, // America/Tegucigalpa
    {  383, 5673,  116015,  4, }, // America/Thule
    {  383,  313,  116019,  7, }, // America/Tijuana
    {  383,  608,  116026,  7, }, // America/Toronto
    {  383, 5605,  116033,  6, }, // America/Tortola
    {  383, 2077,  116039,  8, }, // America/Vancouver
    {  383, 2128,  116047,  7, }, // America/Whitehorse
    {  383,  734,  116054,  8, }, // America/Winnipeg
    {  383, 8515,  116062,  7, }, // America/Yakutat
    {  383, 5918,  116069,  4, }, // Antarctica/Casey
    {  383, 6027,  116073,  5, }, // Antarctica/Davis
    {  383, 6066,  116078, 14, }, // Antarctica/DumontDUrville
    {  383, 8531,  116092,  9, }, // Antarctica/Macquarie
    {  383, 7029,  116101,  7, }, // Antarctica/Mawson
    {  383, 7076,  116108,  9, }, // Antarctica/McMurdo
    {  383, 8552,  116117,  6, }, // Antarctica/Palmer
    {  383, 7189,  116123,  6, }, // Antarctica/Rothera
    {  383, 7258,  116129,  5, }, // Antarctica/Syowa
    {  383, 8570,  116134,  5, }, // Antarctica/Troll
    {  383, 7337,  116139,  7, }, // Antarctica/Vostok
    {  383, 1413,  116146,  9, }, // Arctic/Longyearbyen
    {  383, 5487,  116155,  6, }, // Asia/Aden
    {  383, 5297,  116161,  6, }, // Asia/Almaty
    {  383, 3870,  116167,  5, }, // Asia/Amman
    {  383, 5428,  116172,  6, }, // Asia/Anadyr
    {  383, 5440,  116178,  7, }, // Asia/Aqtau
    {  383, 5270,  116185,  7, }, // Asia/Aqtobe
    {  383,  964,  116192, 10, }, // Asia/Ashgabat
    {  383, 8587,  116202,  9, }, // Asia/Atyrau
    {  383, 3509,  116211,  6, }, // Asia/Baghdad
    {  383, 5451,  116217,  5, }, // Asia/Bahrain
    {  383, 3555,  116222,  4, }, // Asia/Baku
    {  383, 4368,  116226,  6, }, // Asia/Bangkok
    {  383, 3473,  116232,  5, }, // Asia/Barnaul
    {  383, 3988,  116237,  5, }, // Asia/Beirut
    {  383, 3680,  116242,  8, }, // Asia/Bishkek
    {  383, 5906,  116250,  5, }, // Asia/Brunei
    {  383, 4503,  116255,  5, }, // Asia/Chita
    {  383, 5963,  116260,  9, }, // Asia/Choibalsan
    {  383, 4413,  116269,  6, }, // Asia/Colombo
    {  383, 4442,  116275,  8, }, // Asia/Damascus
    {  383, 1060,  116283,  6, }, // Asia/Dhaka
    {  383, 6125,  116289,  4, }, // Asia/Dili
    {  383, 3498,  116293,  4, }, // Asia/Dubai
    {  383, 6092,  116297,  7, }, // Asia/Dushanbe
    {  383, 8599,  116304,  9, }, // Asia/Famagusta
    {  383, 8614,  116313,  4, }, // Asia/Gaza
    {  383, 4697,  116317,  6, }, // Asia/Hebron
    {  383, 1226,  116323,  7, }, // Asia/Ho_Chi_Minh
    {  383, 2576,  116330,  7, }, // Asia/Hong_Kong
    {  383, 4673,  116337,  3, }, // Asia/Hovd
    {  383, 4086,  116340,  7, }, // Asia/Irkutsk
    {  383, 6969,  116347,  6, }, // Asia/Jakarta
    {  383, 6955,  116353,  8, }, // Asia/Jayapura
    {  383, 1257,  116361,  9, }, // Asia/Jerusalem
    {  383, 3462,  116370,  4, }, // Asia/Kabul
    {  383, 4218,  116374,  8, }, // Asia/Kamchatka
    {  383, 4141,  116382,  6, }, // Asia/Karachi
    {  383, 1152,  116388,  8, }, // Asia/Kathmandu
    {  383, 8624,  116396,  7, }, // Asia/Khandyga
    {  383,  992,  116403,  7, }, // Asia/Kolkata
    {  383, 4099,  116410, 11, }, // Asia/Krasnoyarsk
    {  383, 4761,  116421, 10, }, // Asia/Kuala_Lumpur
    {  383, 5893,  116431,  5, }, // Asia/Kuching
    {  383, 5464,  116436,  5, }, // Asia/Kuwait
    {  383, 1178,  116441,  6, }, // Asia/Macau
    {  383, 3919,  116447,  7, }, // Asia/Magadan
    {  383, 1316,  116454,  7, }, // Asia/Makassar
    {  383, 7145,  116461,  6, }, // Asia/Manila
    {  383, 6883,  116467,  6, }, // Asia/Muscat
    {  383, 2446,  116473,  7, }, // Asia/Nicosia
    {  383, 8638,  116480, 11, }, // Asia/Novokuznetsk
    {  383, 4037,  116491, 11, }, // Asia/Novosibirsk
    {  383, 4131,  116502,  6, }, // Asia/Omsk
    {  383, 7108,  116508,  4, }, // Asia/Oral
    {  383, 6924,  116512,  9, }, // Asia/Phnom_Penh
    {  383, 8656,  116521,  9, }, // Asia/Pontianak
    {  383, 4116,  116530,  6, }, // Asia/Pyongyang
    {  383, 5476,  116536,  6, }, // Asia/Qatar
    {  383, 8671,  116542,  7, }, // Asia/Qostanay
    {  383, 4171,  116549,  8, }, // Asia/Qyzylorda
    {  383, 3486,  116557,  5, }, // Asia/Riyadh
    {  383, 4310,  116562,  7, }, // Asia/Sakhalin
    {  383, 7208,  116569,  7, }, // Asia/Samarkand
    {  383, 3116,  116576,  4, }, // Asia/Seoul
    {  383, 1020,  116580,  7, }, // Asia/Shanghai
    {  383, 3137,  116587,  7, }, // Asia/Singapore
    {  383, 4199,  116594, 13, }, // Asia/Srednekolymsk
    {  383, 3100,  116607,  4, }, // Asia/Taipei
    {  383, 4683,  116611,  8, }, // Asia/Tashkent
    {  383, 3817,  116619,  8, }, // Asia/Tbilisi
    {  383, 2638,  116627,  5, }, // Asia/Tehran
    {  383, 1284,  116632,  5, }, // Asia/Thimphu
    {  383, 2687,  116637,  5, }, // Asia/Tokyo
    {  383, 4474,  116642,  6, }, // Asia/Tomsk
    {  383, 1346,  116648, 11, }, // Asia/Ulaanbaatar
    {  383, 1126,  116659,  7, }, // Asia/Urumqi
    {  383, 8685,  116666,  8, }, // Asia/Ust-Nera
    {  383, 6940,  116674,  6, }, // Asia/Vientiane
    {  383, 4612,  116680, 12, }, // Asia/Vladivostok
    {  383, 4730,  116692,  7, }, // Asia/Yakutsk
    {  383, 1202,  116699,  6, }, // Asia/Yangon
    {  383, 3785,  116705, 14, }, // Asia/Yekaterinburg
    {  383, 3649,  116719,  7, }, // Asia/Yerevan
    {  383, 3565,  116726,  7, }, // Atlantic/Azores
    {  383, 5588,  116733,  8, }, // Atlantic/Bermuda
    {  383, 6591,  116741,  6, }, // Atlantic/Canary
    {  383, 3629,  116747,  7, }, // Atlantic/Cape_Verde
    {  383, 1379,  116754,  4, }, // Atlantic/Faroe
    {  383, 8699,  116758,  6, }, // Atlantic/Madeira
    {  383, 2614,  116764,  8, }, // Atlantic/Reykjavik
    {  383, 7235,  116772,  9, }, // Atlantic/South_Georgia
    {  383, 6784,  116781, 10, }, // Atlantic/St_Helena
    {  383, 6607,  116791,  7, }, // Atlantic/Stanley
    {  383, 1654,  116798,  7, }, // Australia/Adelaide
    {  383, 1619,  116805,  7, }, // Australia/Brisbane
    {  383, 1783,  116812, 11, }, // Australia/Broken_Hill
    {  383, 1567,  116823,  5, }, // Australia/Darwin
    {  383, 3539,  116828,  5, }, // Australia/Eucla
    {  383, 1500,  116833,  5, }, // Australia/Hobart
    {  383, 8716,  116838,  8, }, // Australia/Lindeman
    {  383, 1531,  116846,  7, }, // Australia/Lord_Howe
    {  383, 1711,  116853,  7, }, // Australia/Melbourne
    {  383, 1746,  116860,  4, }, // Australia/Perth
    {  383, 1447,  116864,  5, }, // Australia/Sydney
    {  383, 6390,  116869,  9, }, // Europe/Amsterdam
    {  383, 6149,  116878,  7, }, // Europe/Andorra
    {  383, 3522,  116885,  9, }, // Europe/Astrakhan
    {  383, 6577,  116894,  7, }, // Europe/Athens
    {  383, 6285,  116901,  7, }, // Europe/Belgrade
    {  383, 4659,  116908,  6, }, // Europe/Berlin
    {  383, 6437,  116914, 10, }, // Europe/Bratislava
    {  383, 6178,  116924,  7, }, // Europe/Brussels
    {  383, 3830,  116931,  9, }, // Europe/Bucharest
    {  383, 3708,  116940,  9, }, // Europe/Budapest
    {  383, 8751,  116949,  9, }, // Europe/Busingen
    {  383, 2475,  116958,  8, }, // Europe/Chisinau
    {  383, 6238,  116966, 10, }, // Europe/Copenhagen
    {  383, 2260,  116976,  7, }, // Europe/Dublin
    {  383, 6256,  116983,  8, }, // Europe/Gibraltar
    {  383, 8767,  116991,  6, }, // Europe/Guernsey
    {  383, 6561,  116997,  8, }, // Europe/Helsinki
    {  383, 8783,  117005,  7, }, // Europe/Isle_of_Man
    {  383, 1097,  117012, 10, }, // Europe/Istanbul
    {  383, 8802,  117022,  5, }, // Europe/Jersey
    {  383, 3881,  117027, 11, }, // Europe/Kaliningrad
    {  383, 8924,  117038,  5, }, // Europe/Kirov
    {  383, 2419,  117043,  3, }, // Europe/Kyiv
    {  383, 3078,  117046,  6, }, // Europe/Lisbon
    {  383, 6455,  117052,  9, }, // Europe/Ljubljana
    {  383, 2393,  117061,  6, }, // Europe/London
    {  383, 6314,  117067, 11, }, // Europe/Luxembourg
    {  383, 4794,  117078,  6, }, // Europe/Madrid
    {  383, 6346,  117084,  5, }, // Europe/Malta
    {  383, 6531,  117089,  9, }, // Europe/Mariehamn
    {  383, 3595,  117098,  6, }, // Europe/Minsk
    {  383, 6359,  117104,  6, }, // Europe/Monaco
    {  383, 3443,  117110,  5, }, // Europe/Moscow
    {  383, 6407,  117115,  5, }, // Europe/Oslo
    {  383, 4186,  117120,  5, }, // Europe/Paris
    {  383, 6373,  117125,  9, }, // Europe/Podgorica
    {  383, 6224,  117134,  4, }, // Europe/Prague
    {  383, 8816,  115111,  4, }, // Europe/Riga
    {  383, 6273,  116756,  3, }, // Europe/Rome
    {  383, 4233,  117138,  6, }, // Europe/Samara
    {  383, 6419,  117144,  9, }, // Europe/San_Marino
    {  383, 6194,  117153,  8, }, // Europe/Sarajevo
    {  383, 4353,  117161,  7, }, // Europe/Saratov
    {  383, 8828,  117168,  9, }, // Europe/Simferopol
    {  383, 6332,  117177,  7, }, // Europe/Skopje
    {  383, 6548,  117184,  5, }, // Europe/Sofia
    {  383, 6472,  117189,  8, }, // Europe/Stockholm
    {  383, 8846,  117197,  8, }, // Europe/Tallinn
    {  383, 6135,  117205,  5, }, // Europe/Tirane
    {  383, 8861,  117210, 12, }, // Europe/Ulyanovsk
    {  383, 6301,  117222,  5, }, // Europe/Vaduz
    {  383, 6516,  117227,  7, }, // Europe/Vatican
    {  383, 6164,  117234,  5, }, // Europe/Vienna
    {  383, 8878,  117239,  8, }, // Europe/Vilnius
    {  383, 4629,  117247,  9, }, // Europe/Volgograd
    {  383, 3055,  117256,  4, }, // Europe/Warsaw
    {  383, 6210,  117260,  6, }, // Europe/Zagreb
    {  383, 6489,  117266,  5, }, // Europe/Zurich
    {  383, 4980,  117271, 13, }, // Indian/Antananarivo
    {  383, 6910,  117284,  5, }, // Indian/Chagos
    {  383, 5979,  117289,  9, }, // Indian/Christmas
    {  383, 5996,  117298,  5, }, // Indian/Cocos
    {  383, 4931,  117303,  6, }, // Indian/Comoro
    {  383, 6624,  117309,  8, }, // Indian/Kerguelen
    {  383, 7223,  117317,  4, }, // Indian/Mahe
    {  383, 7013,  117321,  6, }, // Indian/Maldives
    {  383, 3971,  117327,  9, }, // Indian/Mauritius
    {  383, 5000,  117336,  8, }, // Indian/Mayotte
    {  383, 7174,  117344,  9, }, // Indian/Reunion
    {  383, 4324,  117353,  6, }, // Pacific/Apia
    {  383,  932,  117359,  6, }, // Pacific/Auckland
    {  383, 3608,  117365, 19, }, // Pacific/Bougainville
    {  383, 2863,  117384,  5, }, // Pacific/Chatham
    {  383, 3022,  117389,  3, }, // Pacific/Chuuk
    {  383, 2201,  117392,  6, }, // Pacific/Easter
    {  383, 7323,  117398,  6, }, // Pacific/Efate
    {  383, 7290,  117404,  7, }, // Pacific/Fakaofo
    {  383, 3804,  117411,  4, }, // Pacific/Fiji
    {  383, 7306,  117415,  8, }, // Pacific/Funafuti
    {  383, 6641,  117423,  9, }, // Pacific/Galapagos
    {  383, 6659,  117432,  6, }, // Pacific/Gambier
    {  383, 3724,  117438, 11, }, // Pacific/Guadalcanal
    {  383, 5935,  117449,  3, }, // Pacific/Guam
    {  383, 2929,  117452,  8, }, // Pacific/Honolulu
    {  383, 2897,  117460, 12, }, // Pacific/Kanton
    {  383, 3900,  117472, 10, }, // Pacific/Kiritimati
    {  383, 6982,  117482,  5, }, // Pacific/Kosrae
    {  383, 2708,  117487,  8, }, // Pacific/Kwajalein
    {  383, 4779,  117495,  6, }, // Pacific/Majuro
    {  383, 3953,  117501,  8, }, // Pacific/Marquesas
    {  383, 8909,  117509,  5, }, // Pacific/Midway
    {  383, 7047,  117514,  6, }, // Pacific/Nauru
    {  383, 7095,  117520,  6, }, // Pacific/Niue
    {  383, 4070,  117526,  7, }, // Pacific/Norfolk
    {  383, 7061,  117533,  6, }, // Pacific/Noumea
    {  383, 2991,  117539,  8, }, // Pacific/Pago_Pago
    {  383, 7118,  117547,  6, }, // Pacific/Palau
    {  383, 7157,  117553,  6, }, // Pacific/Pitcairn
    {  383, 2961,  117559,  7, }, // Pacific/Pohnpei
    {  383, 4709,  117566, 11, }, // Pacific/Port_Moresby
    {  383, 6009,  117577,  9, }, // Pacific/Rarotonga
    {  383, 5948,  117586,  5, }, // Pacific/Saipan
    {  383, 7275,  117591,  6, }, // Pacific/Tahiti
    {  383, 6675,  117597,  6, }, // Pacific/Tarawa
    {  383, 4485,  117603,  9, }, // Pacific/Tongatapu
    {  383, 7355,  117612,  3, }, // Pacific/Wake
    {  383, 7368,  117615,  5, }, // Pacific/Wallis
    {  384,   14,       0,  6, }, // Africa/Asmara Latin/Latin/Vatican City
    {  384, 4337,       6,  8, }, // Africa/Sao_Tome
    {  384, 4154,      14,  8, }, // America/Asuncion
    {  384,  237,      22,  8, }, // America/Atikokan
    {  384, 7586,      30, 17, }, // America/Bahia_Banderas
    {  384, 3770,      47,  6, }, // America/Cancun
    {  384, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  384, 5640,      66,  7, }, // America/Curacao
    {  384,  481,      73, 13, }, // America/Indiana/Knox
    {  384, 7924,      86, 16, }, // America/Indiana/Marengo
    {  384, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  384, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  384, 8001,     139, 14, }, // America/Indiana/Vevay
    {  384, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  384, 8049,     171, 16, }, // America/Indiana/Winamac
    {  384, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  384, 5849,     207, 22, }, // America/Lower_Princes
    {  384, 8183,     229,  6, }, // America/Merida
    {  384, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  384, 8292,     255, 20, }, // America/North_Dakota/Center
    {  384, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  384,  393,     298,  4, }, // America/Nuuk
    {  384, 6862,     302, 16, }, // America/Scoresbysund
    {  384, 8471,     318, 14, }, // America/St_Barthelemy
    {  384, 2045,     332, 10, }, // America/St_Johns
    {  384, 5780,     342,  9, }, // America/St_Kitts
    {  384, 5797,     351,  9, }, // America/St_Lucia
    {  384,  855,     360, 10, }, // America/St_Thomas
    {  384, 5830,     370, 11, }, // America/St_Vincent
    {  384, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  384, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  384, 1152,     413,  9, }, // Asia/Kathmandu
    {  384,  992,     422,  7, }, // Asia/Kolkata
    {  384, 1178,     429,  5, }, // Asia/Macau
    {  384, 1202,     442,  6, }, // Asia/Yangon
    {  384, 1379,     448,  5, }, // Atlantic/Faroe
    {  384, 6784,     453, 10, }, // Atlantic/St_Helena
    {  384, 2419,     463,  4, }, // Europe/Kyiv
    {  384, 7174,     467,  7, }, // Indian/Reunion
    {  384, 3022,     474,  5, }, // Pacific/Chuuk
    {  384, 2961,     479,  7, }, // Pacific/Pohnpei
    {  385, 6751,  117620,  8, }, // Africa/Abidjan Latvian/Latin/Latvia
    {  385, 6723,    4550,  4, }, // Africa/Accra
    {  385, 4961,  117628,  9, }, // Africa/Addis_Ababa
    {  385, 7383,  117637,  6, }, // Africa/Algiers
    {  385,   14,       0,  6, }, // Africa/Asmara
    {  385, 5159,   24908,  5, }, // Africa/Bangui
    {  385, 6709,  117643,  8, }, // Africa/Banjul
    {  385, 7398,  117651,  6, }, // Africa/Bissau
    {  385, 4859,  117657,  9, }, // Africa/Blantyre
    {  385, 5189,  117666,  9, }, // Africa/Brazzaville
    {  385, 4824,  117675,  9, }, // Africa/Bujumbura
    {  385, 2242,  117684,  5, }, // Africa/Cairo
    {  385, 4019,   19744, 10, }, // Africa/Casablanca
    {  385, 7412,   19754,  5, }, // Africa/Ceuta
    {  385, 6736,    4637,  7, }, // Africa/Conakry
    {  385, 6803,  117689,  6, }, // Africa/Dakar
    {  385, 5032,  117695, 10, }, // Africa/Dar_es_Salaam
    {  385, 4945,  117705,  9, }, // Africa/Djibouti
    {  385, 5145,   19777,  5, }, // Africa/Douala
    {  385, 5068,  117714,  5, }, // Africa/El_Aaiun
    {  385, 6816,  117719,  8, }, // Africa/Freetown
    {  385, 4381,  117727, 13, }, // Africa/Johannesburg
    {  385, 4401,  117740,  5, }, // Africa/Juba
    {  385, 4426,  117745,  7, }, // Africa/Khartoum
    {  385, 5208,  117752,  7, }, // Africa/Kinshasa
    {  385, 4646,  117759,  6, }, // Africa/Lagos
    {  385, 5238,  117765,  9, }, // Africa/Libreville
    {  385, 4841,  117774,  9, }, // Africa/Lubumbashi
    {  385, 4889,  117783,  6, }, // Africa/Maputo
    {  385, 5015,  117789,  8, }, // Africa/Mogadishu
    {  385, 6997,  117797,  9, }, // Africa/Monrovia
    {  385, 5173,   56262,  8, }, // Africa/Ndjamena
    {  385, 5256,  117806,  7, }, // Africa/Niamey
    {  385, 6766,  117813,  8, }, // Africa/Nouakchott
    {  385, 6690,   59902,  8, }, // Africa/Ouagadougou
    {  385, 5127,  117821,  9, }, // Africa/Porto-Novo
    {  385, 4337,   59911,  7, }, // Africa/Sao_Tome
    {  385, 2732,  117830,  7, }, // Africa/Tripoli
    {  385, 6503,  117837,  6, }, // Africa/Tunis
    {  385, 4054,  117843,  8, }, // Africa/Windhoek
    {  385,  132,  117851,  5, }, // America/Adak
    {  385, 3183,  117856,  8, }, // America/Anchorage
    {  385, 5524,  117864,  7, }, // America/Anguilla
    {  385, 5541,   31775,  7, }, // America/Antigua
    {  385, 4456,  117871,  9, }, // America/Araguaina
    {  385,  166,  117880, 12, }, // America/Argentina/Buenos_Aires
    {  385,   91,    4788,  9, }, // America/Argentina/Catamarca
    {  385,  270,  117892,  7, }, // America/Argentina/Cordoba
    {  385,  441,  117899,  6, }, // America/Argentina/Jujuy
    {  385, 7425,  117905,  7, }, // America/Argentina/La_Rioja
    {  385,  565,   96160,  7, }, // America/Argentina/Mendoza
    {  385, 7452,  117912, 12, }, // America/Argentina/Rio_Gallegos
    {  385, 7507,  117924,  8, }, // America/Argentina/San_Juan
    {  385, 5497,  117932,  8, }, // America/Argentina/San_Luis
    {  385, 7534,  117940,  8, }, // America/Argentina/Tucuman
    {  385, 7560,  117948,  6, }, // America/Argentina/Ushuaia
    {  385, 4154,  117954,  9, }, // America/Asuncion
    {  385,  237,  117963,  9, }, // America/Atikokan
    {  385, 3581,  117972,  5, }, // America/Bahia
    {  385, 7586,  117977, 13, }, // America/Bahia_Banderas
    {  385, 5571,  117990,  9, }, // America/Barbados
    {  385, 7609,  117999,  6, }, // America/Belem
    {  385, 5309,  118005,  6, }, // America/Belize
    {  385, 7623,  118011, 11, }, // America/Blanc-Sablon
    {  385, 7644,  118022,  8, }, // America/Boa_Vista
    {  385, 7662,  118030, 11, }, // America/Boise
    {  385, 7676,  118041, 12, }, // America/Cambridge_Bay
    {  385, 7698,  118053, 11, }, // America/Campo_Grande
    {  385, 3770,  118064,  7, }, // America/Cancun
    {  385, 4596,  118071,  8, }, // America/Caracas
    {  385, 4247,  118079,  7, }, // America/Cayenne
    {  385, 5398,  118086, 13, }, // America/Cayman
    {  385, 3251,  118099,  6, }, // America/Chicago
    {  385, 7719,  118105,  6, }, // America/Chihuahua
    {  385, 7737,  118111,  7, }, // America/Ciudad_Juarez
    {  385, 5324,   31782,  9, }, // America/Costa_Rica
    {  385, 7759,  118118,  8, }, // America/Creston
    {  385, 3693,    5027,  6, }, // America/Cuiaba
    {  385, 5640,  118126,  7, }, // America/Curacao
    {  385, 7775,  118133, 12, }, // America/Danmarkshavn
    {  385, 7796,  118145,  7, }, // America/Dawson
    {  385, 7811,  118152, 11, }, // America/Dawson_Creek
    {  385,  805,  118163,  7, }, // America/Denver
    {  385, 3351,  118170,  8, }, // America/Detroit
    {  385, 5656,   20071,  8, }, // America/Dominica
    {  385,  893,  118178,  9, }, // America/Edmonton
    {  385, 5343,  118187,  9, }, // America/El_Salvador
    {  385, 7849,  118196, 11, }, // America/Fort_Nelson
    {  385, 7887,  118207,  9, }, // America/Glace_Bay
    {  385, 6844,  118216,  7, }, // America/Goose_Bay
    {  385, 4514,  118223, 11, }, // America/Grand_Turk
    {  385, 5687,  118234,  7, }, // America/Grenada
    {  385, 5703,  118241,  9, }, // America/Guadeloupe
    {  385, 3662,   31800,  9, }, // America/Guatemala
    {  385, 4743,  118250,  9, }, // America/Guayaquil
    {  385, 6895,  118259,  6, }, // America/Guyana
    {  385, 1923,  118265,  9, }, // America/Halifax
    {  385, 7905,  118274,  9, }, // America/Hermosillo
    {  385,  348,  118283, 13, }, // America/Indiana/Indianapolis
    {  385,  481,  118296, 14, }, // America/Indiana/Knox
    {  385, 7924,  118310, 16, }, // America/Indiana/Marengo
    {  385, 7948,  118326, 20, }, // America/Indiana/Petersburg
    {  385, 7975,  118346, 18, }, // America/Indiana/Tell_City
    {  385, 8001,  118364, 15, }, // America/Indiana/Vevay
    {  385, 8023,  118379, 17, }, // America/Indiana/Vincennes
    {  385, 8049,  118396, 17, }, // America/Indiana/Winamac
    {  385, 8073,  118413,  7, }, // America/Inuvik
    {  385,  660,  118420,  8, }, // America/Iqaluit
    {  385, 2665,    3892,  7, }, // America/Jamaica
    {  385, 5282,  118428,  5, }, // America/Juneau
    {  385,  521,  118433,  7, }, // America/Kentucky/Louisville
    {  385, 8088,  118440, 18, }, // America/Kentucky/Monticello
    {  385, 5621,  118458, 11, }, // America/Kralendijk
    {  385, 4278,  118469,  6, }, // America/La_Paz
    {  385, 3390,  118475, 12, }, // America/Los_Angeles
    {  385, 5849,  118487, 19, }, // America/Lower_Princes
    {  385, 8116,  118506,  6, }, // America/Maceio
    {  385, 8131,  118512,  7, }, // America/Managua
    {  385, 1892,  118519,  7, }, // America/Manaus
    {  385, 5814,  118526,  8, }, // America/Marigot
    {  385, 5722,   25027,  9, }, // America/Martinique
    {  385, 8147,  118534, 10, }, // America/Matamoros
    {  385, 2779,  118544,  9, }, // America/Mazatlan
    {  385, 8165,  118553,  8, }, // America/Menominee
    {  385, 8183,    3899,  6, }, // America/Merida
    {  385, 2811,  118561,  6, }, // America/Mexico_City
    {  385, 4293,  118567,  8, }, // America/Miquelon
    {  385, 8217,  118575,  8, }, // America/Moncton
    {  385, 8233,  118583, 10, }, // America/Monterrey
    {  385, 5741,  118593, 11, }, // America/Montserrat
    {  385, 5383,  118604,  4, }, // America/Nassau
    {  385, 3294,  118608,  7, }, // America/New_York
    {  385, 8251,  118615,  4, }, // America/Nome
    {  385, 1834,  118619,  6, }, // America/Noronha
    {  385, 8264,  118625, 19, }, // America/North_Dakota/Beulah
    {  385, 8292,  118644, 21, }, // America/North_Dakota/Center
    {  385, 8320,  118665, 23, }, // America/North_Dakota/New_Salem
    {  385,  393,  118688,  4, }, // America/Nuuk
    {  385, 8351,    5561,  7, }, // America/Ojinaga
    {  385, 3224,  118692,  7, }, // America/Phoenix
    {  385, 3847,  118699, 11, }, // America/Port-au-Prince
    {  385, 5871,  118710, 12, }, // America/Port_of_Spain
    {  385, 8367,  118722,  9, }, // America/Porto_Velho
    {  385, 5760,   60631, 10, }, // America/Puerto_Rico
    {  385, 3932,  118731, 12, }, // America/Punta_Arenas
    {  385, 8387,  118743, 12, }, // America/Rankin_Inlet
    {  385, 8408,  118755,  6, }, // America/Recife
    {  385, 1979,  118761,  8, }, // America/Regina
    {  385, 8423,  118769,  8, }, // America/Resolute
    {  385,  695,  118777,  9, }, // America/Rio_Branco
    {  385, 8440,  118786,  9, }, // America/Santarem
    {  385, 2165,  118795,  8, }, // America/Santiago
    {  385, 6044,  118803, 12, }, // America/Santo_Domingo
    {  385, 1862,  118815,  8, }, // America/Sao_Paulo
    {  385, 6862,  118823, 13, }, // America/Scoresbysund
    {  385, 8471,  118836, 11, }, // America/St_Barthelemy
    {  385, 2045,  118847, 10, }, // America/St_Johns
    {  385, 5780,  118857,  9, }, // America/St_Kitts
    {  385, 5797,  118866, 10, }, // America/St_Lucia
    {  385,  855,  118876,  9, }, // America/St_Thomas
    {  385, 5830,  118885, 12, }, // America/St_Vincent
    {  385, 8493,  118897, 12, }, // America/Swift_Current
    {  385, 5363,    5759, 11, }, // America/Tegucigalpa
    {  385, 5673,  118909,  4, }, // America/Thule
    {  385,  313,    5775,  7, }, // America/Tijuana
    {  385, 2077,  118913,  9, }, // America/Vancouver
    {  385, 2128,  118922,  9, }, // America/Whitehorse
    {  385,  734,  118931,  8, }, // America/Winnipeg
    {  385, 8515,  118939,  8, }, // America/Yakutat
    {  385, 5918,  118947,  5, }, // Antarctica/Casey
    {  385, 6027,  118952,  7, }, // Antarctica/Davis
    {  385, 6066,  118959, 12, }, // Antarctica/DumontDUrville
    {  385, 8531,  118971,  7, }, // Antarctica/Macquarie
    {  385, 7029,  118978,  6, }, // Antarctica/Mawson
    {  385, 7076,  118984,  8, }, // Antarctica/McMurdo
    {  385, 8552,  118992,  7, }, // Antarctica/Palmer
    {  385, 7189,  118999,  6, }, // Antarctica/Rothera
    {  385, 7258,  119005,  4, }, // Antarctica/Syowa
    {  385, 8570,  119009,  6, }, // Antarctica/Troll
    {  385, 7337,  119015,  7, }, // Antarctica/Vostok
    {  385, 1413,  119022, 12, }, // Arctic/Longyearbyen
    {  385, 5487,  119034,  5, }, // Asia/Aden
    {  385, 5297,    5890,  6, }, // Asia/Almaty
    {  385, 3870,  119039,  6, }, // Asia/Amman
    {  385, 5428,  119045,  7, }, // Asia/Anadyr
    {  385, 5440,    5906,  5, }, // Asia/Aqtau
    {  385, 5270,  119052,  6, }, // Asia/Aqtobe
    {  385,  964,  119058,  8, }, // Asia/Ashgabat
    {  385, 8587,    5917,  6, }, // Asia/Atyrau
    {  385, 3509,  119066,  7, }, // Asia/Baghdad
    {  385, 5451,  119073,  8, }, // Asia/Bahrain
    {  385, 4368,  119081,  8, }, // Asia/Bangkok
    {  385, 3473,  119089,  8, }, // Asia/Barnaul
    {  385, 3988,  119097,  7, }, // Asia/Beirut
    {  385, 3680,  119104,  7, }, // Asia/Bishkek
    {  385, 5906,  119111,  7, }, // Asia/Brunei
    {  385, 4503,   56142,  4, }, // Asia/Chita
    {  385, 5963,  119118, 10, }, // Asia/Choibalsan
    {  385, 4413,    5949,  7, }, // Asia/Colombo
    {  385, 4442,  119128,  7, }, // Asia/Damascus
    {  385, 1060,    5956,  4, }, // Asia/Dhaka
    {  385, 3498,  119135,  7, }, // Asia/Dubai
    {  385, 6092,   31939,  7, }, // Asia/Dushanbe
    {  385, 4697,  119142,  7, }, // Asia/Hebron
    {  385, 1226,  119149,  8, }, // Asia/Ho_Chi_Minh
    {  385, 2576,  119157,  8, }, // Asia/Hong_Kong
    {  385, 4673,  119165,  5, }, // Asia/Hovd
    {  385, 4086,  119170,  8, }, // Asia/Irkutsk
    {  385, 6969,   31955,  8, }, // Asia/Jakarta
    {  385, 6955,   31963,  9, }, // Asia/Jayapura
    {  385, 1257,  119178, 10, }, // Asia/Jerusalem
    {  385, 3462,  119188,  6, }, // Asia/Kabul
    {  385, 4218,   31981,  8, }, // Asia/Kamchatka
    {  385, 4141,  119194,  6, }, // Asia/Karachi
    {  385, 1152,    6018,  8, }, // Asia/Kathmandu
    {  385, 8624,   31995,  7, }, // Asia/Khandyga
    {  385,  992,  119200,  7, }, // Asia/Kolkata
    {  385, 4099,  119207, 12, }, // Asia/Krasnoyarsk
    {  385, 4761,  119219, 12, }, // Asia/Kuala_Lumpur
    {  385, 5893,  119231,  6, }, // Asia/Kuching
    {  385, 5464,  119237,  7, }, // Asia/Kuwait
    {  385, 1178,    6064,  5, }, // Asia/Macau
    {  385, 3919,  119244,  8, }, // Asia/Magadan
    {  385, 1316,  119252,  8, }, // Asia/Makassar
    {  385, 6883,  119260,  7, }, // Asia/Muscat
    {  385, 2446,   55803,  8, }, // Asia/Nicosia
    {  385, 8638,  119267, 12, }, // Asia/Novokuznetsk
    {  385, 4037,  119279, 12, }, // Asia/Novosibirsk
    {  385, 4131,  119291,  5, }, // Asia/Omsk
    {  385, 7108,  119296,  5, }, // Asia/Oral
    {  385, 6924,  119301,  8, }, // Asia/Phnom_Penh
    {  385, 8656,  119309, 10, }, // Asia/Pontianak
    {  385, 4116,  119319,  8, }, // Asia/Pyongyang
    {  385, 5476,  119327,  6, }, // Asia/Qatar
    {  385, 8671,  119333,  9, }, // Asia/Qostanay
    {  385, 4171,    6111,  9, }, // Asia/Qyzylorda
    {  385, 3486,  119342,  6, }, // Asia/Riyadh
    {  385, 4310,  119348,  8, }, // Asia/Sakhalin
    {  385, 7208,   25356, 10, }, // Asia/Samarkand
    {  385, 3116,  119356,  5, }, // Asia/Seoul
    {  385, 1020,  119361,  7, }, // Asia/Shanghai
    {  385, 3137,  119368,  9, }, // Asia/Singapore
    {  385, 4199,  119377, 14, }, // Asia/Srednekolymsk
    {  385, 3100,  119391,  6, }, // Asia/Taipei
    {  385, 4683,  119397,  8, }, // Asia/Tashkent
    {  385, 2638,  119405,  8, }, // Asia/Tehran
    {  385, 2687,  119413,  6, }, // Asia/Tokyo
    {  385, 4474,  119419,  6, }, // Asia/Tomsk
    {  385, 1346,  119425, 10, }, // Asia/Ulaanbaatar
    {  385, 1126,   32067,  6, }, // Asia/Urumqi
    {  385, 8685,  119435,  8, }, // Asia/Ust-Nera
    {  385, 6940,  119443,  8, }, // Asia/Vientiane
    {  385, 4612,  119451, 12, }, // Asia/Vladivostok
    {  385, 4730,  119463,  8, }, // Asia/Yakutsk
    {  385, 1202,  119471,  7, }, // Asia/Yangon
    {  385, 3785,  119478, 14, }, // Asia/Yekaterinburg
    {  385, 3649,  119492,  7, }, // Asia/Yerevan
    {  385, 3565,  119499, 11, }, // Atlantic/Azores
    {  385, 6591,  119510, 14, }, // Atlantic/Canary
    {  385, 3629,  119524,  9, }, // Atlantic/Cape_Verde
    {  385, 1379,  119533, 10, }, // Atlantic/Faroe
    {  385, 2614,  119543, 10, }, // Atlantic/Reykjavik
    {  385, 7235,  119553, 16, }, // Atlantic/South_Georgia
    {  385, 6784,  119569, 15, }, // Atlantic/St_Helena
    {  385, 6607,    6298,  6, }, // Atlantic/Stanley
    {  385, 1654,   19421,  8, }, // Australia/Adelaide
    {  385, 1619,  119584,  8, }, // Australia/Brisbane
    {  385, 1783,  119592, 10, }, // Australia/Broken_Hill
    {  385, 1567,  119602,  7, }, // Australia/Darwin
    {  385, 3539,  119609,  5, }, // Australia/Eucla
    {  385, 1500,  119614,  7, }, // Australia/Hobart
    {  385, 8716,  119621,  9, }, // Australia/Lindeman
    {  385, 1531,  119630,  8, }, // Australia/Lord_Howe
    {  385, 1711,  119638,  8, }, // Australia/Melbourne
    {  385, 1746,  119646,  5, }, // Australia/Perth
    {  385, 1447,  119651,  7, }, // Australia/Sydney
    {  385, 6390,  119658, 10, }, // Europe/Amsterdam
    {  385, 6149,   32109,  6, }, // Europe/Andorra
    {  385, 3522,  119668,  9, }, // Europe/Astrakhan
    {  385, 6577,  119677,  6, }, // Europe/Athens
    {  385, 6285,  119683,  8, }, // Europe/Belgrade
    {  385, 4659,  119691,  7, }, // Europe/Berlin
    {  385, 6178,  119698,  7, }, // Europe/Brussels
    {  385, 3830,  119705,  9, }, // Europe/Bucharest
    {  385, 3708,  119714,  9, }, // Europe/Budapest
    {  385, 8751,  119723,  9, }, // Europe/Busingen
    {  385, 2475,  119732,  8, }, // Europe/Chisinau
    {  385, 6238,  119740, 11, }, // Europe/Copenhagen
    {  385, 2260,  119751,  7, }, // Europe/Dublin
    {  385, 6256,  119758, 10, }, // Europe/Gibraltar
    {  385, 8767,  119768,  8, }, // Europe/Guernsey
    {  385, 8783,  119776, 10, }, // Europe/Isle_of_Man
    {  385, 1097,  119786,  8, }, // Europe/Istanbul
    {  385, 8802,  119794,  8, }, // Europe/Jersey
    {  385, 3881,  119802, 12, }, // Europe/Kaliningrad
    {  385, 8924,  119814,  6, }, // Europe/Kirov
    {  385, 2419,  119820,  6, }, // Europe/Kyiv
    {  385, 3078,  119826,  8, }, // Europe/Lisbon
    {  385, 6455,  119834,  7, }, // Europe/Ljubljana
    {  385, 2393,  119841,  7, }, // Europe/London
    {  385, 6314,  119848, 11, }, // Europe/Luxembourg
    {  385, 4794,  119859,  7, }, // Europe/Madrid
    {  385, 6531,  119866, 10, }, // Europe/Mariehamn
    {  385, 3595,  119876,  6, }, // Europe/Minsk
    {  385, 6359,    6525,  6, }, // Europe/Monaco
    {  385, 3443,  119882,  7, }, // Europe/Moscow
    {  385, 4186,  119889,  6, }, // Europe/Paris
    {  385, 6224,  119895,  5, }, // Europe/Prague
    {  385, 8816,   71844,  4, }, // Europe/Riga
    {  385, 6273,   19581,  4, }, // Europe/Rome
    {  385, 6419,  119900,  9, }, // Europe/San_Marino
    {  385, 6194,  119909,  8, }, // Europe/Sarajevo
    {  385, 4353,  119917,  8, }, // Europe/Saratov
    {  385, 8828,  119925, 11, }, // Europe/Simferopol
    {  385, 6548,   32232,  6, }, // Europe/Sofia
    {  385, 6472,  119936,  9, }, // Europe/Stockholm
    {  385, 8846,  119945,  7, }, // Europe/Tallinn
    {  385, 6135,  119952,  6, }, // Europe/Tirane
    {  385, 8861,  119958, 10, }, // Europe/Ulyanovsk
    {  385, 6301,  119968,  6, }, // Europe/Vaduz
    {  385, 6516,  119974,  8, }, // Europe/Vatican
    {  385, 6164,  119982,  4, }, // Europe/Vienna
    {  385, 8878,  119986,  5, }, // Europe/Vilnius
    {  385, 4629,  119991, 10, }, // Europe/Volgograd
    {  385, 3055,   32249,  7, }, // Europe/Warsaw
    {  385, 6210,  120001,  7, }, // Europe/Zagreb
    {  385, 6489,  120008,  6, }, // Europe/Zurich
    {  385, 4980,   97916, 12, }, // Indian/Antananarivo
    {  385, 6910,  120014,  6, }, // Indian/Chagos
    {  385, 5979,  120020, 15, }, // Indian/Christmas
    {  385, 5996,  120035, 21, }, // Indian/Cocos
    {  385, 4931,  120056,  7, }, // Indian/Comoro
    {  385, 6624,  120063, 14, }, // Indian/Kerguelen
    {  385, 7223,  120077,  3, }, // Indian/Mahe
    {  385, 7013,  120080,  9, }, // Indian/Maldives
    {  385, 3971,  120089,  9, }, // Indian/Mauritius
    {  385, 5000,  120098,  6, }, // Indian/Mayotte
    {  385, 7174,  120104,  8, }, // Indian/Reunion
    {  385, 4324,  120112,  5, }, // Pacific/Apia
    {  385,  932,  120117,  7, }, // Pacific/Auckland
    {  385, 3608,  120124, 14, }, // Pacific/Bougainville
    {  385, 2863,  120138,  6, }, // Pacific/Chatham
    {  385, 3022,  120144,  4, }, // Pacific/Chuuk
    {  385, 2201,  120148, 14, }, // Pacific/Easter
    {  385, 3804,   32307,  5, }, // Pacific/Fiji
    {  385, 6641,  120162, 14, }, // Pacific/Galapagos
    {  385, 6659,  120176, 12, }, // Pacific/Gambier
    {  385, 3724,  120188, 12, }, // Pacific/Guadalcanal
    {  385, 5935,  120200,  5, }, // Pacific/Guam
    {  385, 2929,    4512,  8, }, // Pacific/Honolulu
    {  385, 2897,  120205, 11, }, // Pacific/Kanton
    {  385, 3900,  120216, 10, }, // Pacific/Kiritimati
    {  385, 6982,  120226,  7, }, // Pacific/Kosrae
    {  385, 2708,  120233, 11, }, // Pacific/Kwajalein
    {  385, 4779,  120244,  7, }, // Pacific/Majuro
    {  385, 3953,  120251, 13, }, // Pacific/Marquesas
    {  385, 8909,  120264,  7, }, // Pacific/Midway
    {  385, 4070,  120271,  8, }, // Pacific/Norfolk
    {  385, 7061,  120279,  5, }, // Pacific/Noumea
    {  385, 2991,  120284,  8, }, // Pacific/Pago_Pago
    {  385, 7157,  120292,  8, }, // Pacific/Pitcairn
    {  385, 2961,  120300,  7, }, // Pacific/Pohnpei
    {  385, 4709,  120307, 10, }, // Pacific/Port_Moresby
    {  385, 5948,  120317,  7, }, // Pacific/Saipan
    {  385, 7275,  120324,  5, }, // Pacific/Tahiti
    {  385, 6675,   21574,  6, }, // Pacific/Tarawa
    {  385, 7355,  120329, 10, }, // Pacific/Wake
    {  385, 7368,  120339,  6, }, // Pacific/Wallis
    {  386,   14,       0,  6, }, // Africa/Asmara Lingala/Latin/Congo - Kinshasa
    {  386, 4337,       6,  8, }, // Africa/Sao_Tome
    {  386, 4154,      14,  8, }, // America/Asuncion
    {  386,  237,      22,  8, }, // America/Atikokan
    {  386, 7586,      30, 17, }, // America/Bahia_Banderas
    {  386, 3770,      47,  6, }, // America/Cancun
    {  386, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  386, 5640,      66,  7, }, // America/Curacao
    {  386,  481,      73, 13, }, // America/Indiana/Knox
    {  386, 7924,      86, 16, }, // America/Indiana/Marengo
    {  386, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  386, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  386, 8001,     139, 14, }, // America/Indiana/Vevay
    {  386, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  386, 8049,     171, 16, }, // America/Indiana/Winamac
    {  386, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  386, 5849,     207, 22, }, // America/Lower_Princes
    {  386, 8183,     229,  6, }, // America/Merida
    {  386, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  386, 8292,     255, 20, }, // America/North_Dakota/Center
    {  386, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  386,  393,     298,  4, }, // America/Nuuk
    {  386, 6862,     302, 16, }, // America/Scoresbysund
    {  386, 8471,     318, 14, }, // America/St_Barthelemy
    {  386, 2045,     332, 10, }, // America/St_Johns
    {  386, 5780,     342,  9, }, // America/St_Kitts
    {  386, 5797,     351,  9, }, // America/St_Lucia
    {  386,  855,     360, 10, }, // America/St_Thomas
    {  386, 5830,     370, 11, }, // America/St_Vincent
    {  386, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  386, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  386, 1152,     413,  9, }, // Asia/Kathmandu
    {  386,  992,     422,  7, }, // Asia/Kolkata
    {  386, 1178,     429,  5, }, // Asia/Macau
    {  386, 1202,     442,  6, }, // Asia/Yangon
    {  386, 1379,     448,  5, }, // Atlantic/Faroe
    {  386, 6784,     453, 10, }, // Atlantic/St_Helena
    {  386, 2419,     463,  4, }, // Europe/Kyiv
    {  386, 7174,     467,  7, }, // Indian/Reunion
    {  386, 3022,     474,  5, }, // Pacific/Chuuk
    {  386, 2961,     479,  7, }, // Pacific/Pohnpei
    {  390, 6751,  120345,  9, }, // Africa/Abidjan Lithuanian/Latin/Lithuania
    {  390, 6723,    4550,  4, }, // Africa/Accra
    {  390, 4961,   34301, 10, }, // Africa/Addis_Ababa
    {  390, 7383,  120354,  7, }, // Africa/Algiers
    {  390,   14,       0,  6, }, // Africa/Asmara
    {  390,   44,  120361,  7, }, // Africa/Bamako
    {  390, 5159,  120368,  6, }, // Africa/Bangui
    {  390, 6709,  120374,  9, }, // Africa/Banjul
    {  390, 7398,    4583,  5, }, // Africa/Bissau
    {  390, 4859,  120383, 10, }, // Africa/Blantyre
    {  390, 5189,  120393, 10, }, // Africa/Brazzaville
    {  390, 4824,  120403,  9, }, // Africa/Bujumbura
    {  390, 2242,  120412,  6, }, // Africa/Cairo
    {  390, 4019,   19744, 10, }, // Africa/Casablanca
    {  390, 7412,   19754,  5, }, // Africa/Ceuta
    {  390, 6736,  120418,  8, }, // Africa/Conakry
    {  390, 6803,  120426,  7, }, // Africa/Dakar
    {  390, 5032,  120433, 14, }, // Africa/Dar_es_Salaam
    {  390, 4945,  120447,  8, }, // Africa/Djibouti
    {  390, 5145,   19777,  5, }, // Africa/Douala
    {  390, 5068,  120455,  9, }, // Africa/El_Aaiun
    {  390, 6816,  120464,  9, }, // Africa/Freetown
    {  390, 4808,  120473,  9, }, // Africa/Gaborone
    {  390, 4917,  120482,  6, }, // Africa/Harare
    {  390, 4381,  120488, 13, }, // Africa/Johannesburg
    {  390, 4401,  120501,  5, }, // Africa/Juba
    {  390, 4426,  120506,  9, }, // Africa/Khartoum
    {  390, 4875,  120515,  7, }, // Africa/Kigali
    {  390, 5208,  117752,  7, }, // Africa/Kinshasa
    {  390, 4646,  120522,  7, }, // Africa/Lagos
    {  390, 5238,  120529, 10, }, // Africa/Libreville
    {  390, 6832,  120539,  4, }, // Africa/Lome
    {  390, 4841,  120543, 10, }, // Africa/Lubumbashi
    {  390, 5224,  120553,  7, }, // Africa/Malabo
    {  390, 4889,  120560,  7, }, // Africa/Maputo
    {  390, 5084,  120567,  7, }, // Africa/Mbabane
    {  390, 5015,  120574,  9, }, // Africa/Mogadishu
    {  390, 6997,  117797,  9, }, // Africa/Monrovia
    {  390, 3755,  120583,  8, }, // Africa/Nairobi
    {  390, 5173,   56262,  8, }, // Africa/Ndjamena
    {  390, 5256,  120591,  8, }, // Africa/Niamey
    {  390, 6766,  120599,  9, }, // Africa/Nouakchott
    {  390, 6690,   59902,  8, }, // Africa/Ouagadougou
    {  390, 5127,  120608, 11, }, // Africa/Porto-Novo
    {  390, 4337,  120619,  8, }, // Africa/Sao_Tome
    {  390, 2732,   56289,  8, }, // Africa/Tripoli
    {  390, 6503,  120627,  7, }, // Africa/Tunis
    {  390, 4054,  120634,  9, }, // Africa/Windhoek
    {  390,  132,  120643,  7, }, // America/Adak
    {  390, 3183,  120650, 10, }, // America/Anchorage
    {  390, 5524,  120660,  8, }, // America/Anguilla
    {  390, 5541,   31775,  7, }, // America/Antigua
    {  390, 4456,  120668, 10, }, // America/Araguaina
    {  390,  166,  120678, 12, }, // America/Argentina/Buenos_Aires
    {  390,   91,    4788,  9, }, // America/Argentina/Catamarca
    {  390,  270,    4797,  7, }, // America/Argentina/Cordoba
    {  390,  441,  120690,  9, }, // America/Argentina/Jujuy
    {  390, 7425,  120699,  9, }, // America/Argentina/La_Rioja
    {  390,  565,   96160,  7, }, // America/Argentina/Mendoza
    {  390, 7452,  120708, 14, }, // America/Argentina/Rio_Gallegos
    {  390, 7483,  120722,  6, }, // America/Argentina/Salta
    {  390, 7507,  120728, 11, }, // America/Argentina/San_Juan
    {  390, 5497,  120739, 11, }, // America/Argentina/San_Luis
    {  390, 7534,  120750,  9, }, // America/Argentina/Tucuman
    {  390, 7560,  117948,  6, }, // America/Argentina/Ushuaia
    {  390, 4154,  120759, 10, }, // America/Asuncion
    {  390,  237,  120769, 10, }, // America/Atikokan
    {  390, 3581,  117972,  5, }, // America/Bahia
    {  390, 7586,  120779, 16, }, // America/Bahia_Banderas
    {  390, 5571,  120795, 10, }, // America/Barbados
    {  390, 7609,  120805,  7, }, // America/Belem
    {  390, 5309,  120812,  7, }, // America/Belize
    {  390, 7623,  120819, 14, }, // America/Blanc-Sablon
    {  390, 7644,  120833,  9, }, // America/Boa_Vista
    {  390, 7662,  118030,  6, }, // America/Boise
    {  390, 7676,  120842, 15, }, // America/Cambridge_Bay
    {  390, 7698,  120857, 12, }, // America/Campo_Grande
    {  390, 3770,  120869,  8, }, // America/Cancun
    {  390, 4596,  120877,  9, }, // America/Caracas
    {  390, 4247,  120886,  7, }, // America/Cayenne
    {  390, 5398,  120893,  8, }, // America/Cayman
    {  390, 3251,  120901,  6, }, // America/Chicago
    {  390, 7719,  120907,  8, }, // America/Chihuahua
    {  390, 7737,  120915, 16, }, // America/Ciudad_Juarez
    {  390, 5324,   20026, 10, }, // America/Costa_Rica
    {  390, 7759,  120931,  9, }, // America/Creston
    {  390, 3693,    5027,  6, }, // America/Cuiaba
    {  390, 5640,  120940,  8, }, // America/Curacao
    {  390, 7775,  120948, 14, }, // America/Danmarkshavn
    {  390, 7796,  120962,  7, }, // America/Dawson
    {  390, 7811,  120969, 12, }, // America/Dawson_Creek
    {  390,  805,  120981,  8, }, // America/Denver
    {  390, 3351,  120989,  9, }, // America/Detroit
    {  390, 5656,   20071,  8, }, // America/Dominica
    {  390,  893,  120998, 10, }, // America/Edmonton
    {  390, 7832,  121008,  8, }, // America/Eirunepe
    {  390, 5343,  121016, 10, }, // America/El_Salvador
    {  390, 7849,  121026, 13, }, // America/Fort_Nelson
    {  390, 7887,  121039, 11, }, // America/Glace_Bay
    {  390, 6844,  121050,  9, }, // America/Goose_Bay
    {  390, 4514,  121059, 12, }, // America/Grand_Turk
    {  390, 5703,  121071,  9, }, // America/Guadeloupe
    {  390, 3662,   31800,  9, }, // America/Guatemala
    {  390, 4743,  121080, 10, }, // America/Guayaquil
    {  390, 6895,   56104,  7, }, // America/Guyana
    {  390, 1923,  121090, 10, }, // America/Halifax
    {  390, 7905,  121100, 11, }, // America/Hermosillo
    {  390,  481,  121111, 15, }, // America/Indiana/Knox
    {  390, 7924,  121126, 17, }, // America/Indiana/Marengo
    {  390, 7948,  121143, 21, }, // America/Indiana/Petersburg
    {  390, 7975,  121164, 18, }, // America/Indiana/Tell_City
    {  390, 8001,  121182, 14, }, // America/Indiana/Vevay
    {  390, 8023,  121196, 17, }, // America/Indiana/Vincennes
    {  390, 8049,  121213, 18, }, // America/Indiana/Winamac
    {  390, 8073,  121231,  8, }, // America/Inuvik
    {  390,  660,  121239,  9, }, // America/Iqaluit
    {  390, 2665,    3892,  7, }, // America/Jamaica
    {  390, 5282,  121248,  6, }, // America/Juneau
    {  390,  521,  121254,  9, }, // America/Kentucky/Louisville
    {  390, 8088,  121263, 20, }, // America/Kentucky/Monticello
    {  390, 5621,  121283, 12, }, // America/Kralendijk
    {  390, 4278,  121295,  8, }, // America/La_Paz
    {  390, 3390,  121303, 12, }, // America/Los_Angeles
    {  390, 5849,  121315, 27, }, // America/Lower_Princes
    {  390, 8116,  118506,  6, }, // America/Maceio
    {  390, 8131,  118512,  7, }, // America/Managua
    {  390, 1892,  121342,  8, }, // America/Manaus
    {  390, 5814,   60430,  6, }, // America/Marigot
    {  390, 5722,   25027,  9, }, // America/Martinique
    {  390, 8147,  121350, 11, }, // America/Matamoros
    {  390, 2779,  121361, 10, }, // America/Mazatlan
    {  390, 8165,  121371,  9, }, // America/Menominee
    {  390, 8183,    3899,  6, }, // America/Merida
    {  390, 2811,  121380,  8, }, // America/Mexico_City
    {  390, 4293,  121388,  9, }, // America/Miquelon
    {  390, 8217,  121397,  9, }, // America/Moncton
    {  390, 8233,  121406, 10, }, // America/Monterrey
    {  390, 4000,  121416, 12, }, // America/Montevideo
    {  390, 5741,  121428, 11, }, // America/Montserrat
    {  390, 5383,  121439,  5, }, // America/Nassau
    {  390, 3294,  121444,  9, }, // America/New_York
    {  390, 8251,  121453,  5, }, // America/Nome
    {  390, 1834,    5479,  7, }, // America/Noronha
    {  390, 8264,  121458, 21, }, // America/North_Dakota/Beulah
    {  390, 8292,  121479, 24, }, // America/North_Dakota/Center
    {  390, 8320,  121503, 28, }, // America/North_Dakota/New_Salem
    {  390,  393,  121531,  5, }, // America/Nuuk
    {  390, 8351,  121536,  8, }, // America/Ojinaga
    {  390, 6106,  121544, 11, }, // America/Paramaribo
    {  390, 3224,  121555,  8, }, // America/Phoenix
    {  390, 3847,  121563, 14, }, // America/Port-au-Prince
    {  390, 5871,  121577, 15, }, // America/Port_of_Spain
    {  390, 8367,  121592, 12, }, // America/Porto_Velho
    {  390, 5760,  121604, 12, }, // America/Puerto_Rico
    {  390, 3932,  121616, 14, }, // America/Punta_Arenas
    {  390, 8387,  121630, 14, }, // America/Rankin_Inlet
    {  390, 8408,  121644,  6, }, // America/Recife
    {  390, 1979,  121650,  7, }, // America/Regina
    {  390, 8423,  121657,  9, }, // America/Resolute
    {  390,  695,  121666, 11, }, // America/Rio_Branco
    {  390, 8440,  121677, 10, }, // America/Santarem
    {  390, 2165,  121687,  9, }, // America/Santiago
    {  390, 6044,  121696, 14, }, // America/Santo_Domingo
    {  390, 1862,  121710, 10, }, // America/Sao_Paulo
    {  390, 6862,  121720, 14, }, // America/Scoresbysund
    {  390, 8471,  121734, 13, }, // America/St_Barthelemy
    {  390, 2045,  121747, 12, }, // America/St_Johns
    {  390, 5780,  121759, 11, }, // America/St_Kitts
    {  390, 5797,  121770, 11, }, // America/St_Lucia
    {  390,  855,  121781, 12, }, // America/St_Thomas
    {  390, 5830,  121793, 14, }, // America/St_Vincent
    {  390, 8493,  121807, 14, }, // America/Swift_Current
    {  390, 5363,    5759, 11, }, // America/Tegucigalpa
    {  390, 5673,  121821,  7, }, // America/Thule
    {  390,  313,  121828,  8, }, // America/Tijuana
    {  390,  608,  121836,  8, }, // America/Toronto
    {  390, 2077,  121844, 10, }, // America/Vancouver
    {  390, 2128,  121854, 10, }, // America/Whitehorse
    {  390,  734,  121864,  9, }, // America/Winnipeg
    {  390, 8515,  121873,  9, }, // America/Yakutat
    {  390, 5918,  121882,  6, }, // Antarctica/Casey
    {  390, 6027,  121888,  8, }, // Antarctica/Davis
    {  390, 6066,  121896, 18, }, // Antarctica/DumontDUrville
    {  390, 8531,  121914,  8, }, // Antarctica/Macquarie
    {  390, 7029,  121922,  7, }, // Antarctica/Mawson
    {  390, 7076,  121929,  9, }, // Antarctica/McMurdo
    {  390, 8552,  121938,  8, }, // Antarctica/Palmer
    {  390, 7189,  118999,  6, }, // Antarctica/Rothera
    {  390, 7258,  121946,  5, }, // Antarctica/Syowa
    {  390, 8570,  121951,  6, }, // Antarctica/Troll
    {  390, 7337,  121957,  8, }, // Antarctica/Vostok
    {  390, 1413,  121965, 13, }, // Arctic/Longyearbyen
    {  390, 5487,  121978,  6, }, // Asia/Aden
    {  390, 5297,   56127,  8, }, // Asia/Almaty
    {  390, 3870,  121984,  6, }, // Asia/Amman
    {  390, 5428,  121990,  8, }, // Asia/Anadyr
    {  390, 5440,    5906,  5, }, // Asia/Aqtau
    {  390, 5270,  121998, 12, }, // Asia/Aqtobe
    {  390,  964,  122010, 10, }, // Asia/Ashgabat
    {  390, 3509,  122020,  8, }, // Asia/Baghdad
    {  390, 5451,  122028,  9, }, // Asia/Bahrain
    {  390, 4368,  122037,  8, }, // Asia/Bangkok
    {  390, 3473,  122045,  9, }, // Asia/Barnaul
    {  390, 3988,  122054,  8, }, // Asia/Beirut
    {  390, 3680,  122062,  8, }, // Asia/Bishkek
    {  390, 5906,  122070,  8, }, // Asia/Brunei
    {  390, 4503,   56142,  4, }, // Asia/Chita
    {  390, 5963,  122078, 11, }, // Asia/Choibalsan
    {  390, 4413,  122089,  8, }, // Asia/Colombo
    {  390, 4442,  122097,  8, }, // Asia/Damascus
    {  390, 1060,    5956,  4, }, // Asia/Dhaka
    {  390, 6125,  122105,  5, }, // Asia/Dili
    {  390, 3498,  122110,  7, }, // Asia/Dubai
    {  390, 6092,  122117,  7, }, // Asia/Dushanbe
    {  390, 8614,  122124, 12, }, // Asia/Gaza
    {  390, 4697,  122136,  8, }, // Asia/Hebron
    {  390, 1226,  122144,  9, }, // Asia/Ho_Chi_Minh
    {  390, 2576,  122153,  9, }, // Asia/Hong_Kong
    {  390, 4673,  122162,  6, }, // Asia/Hovd
    {  390, 4086,  122168,  9, }, // Asia/Irkutsk
    {  390, 6969,   31955,  8, }, // Asia/Jakarta
    {  390, 6955,   31963,  9, }, // Asia/Jayapura
    {  390, 1257,  122177,  8, }, // Asia/Jerusalem
    {  390, 3462,  122185,  7, }, // Asia/Kabul
    {  390, 4218,  122192,  9, }, // Asia/Kamchatka
    {  390, 4141,  122201,  7, }, // Asia/Karachi
    {  390, 1152,    6018,  8, }, // Asia/Kathmandu
    {  390, 8624,   56553,  8, }, // Asia/Khandyga
    {  390,  992,     422,  7, }, // Asia/Kolkata
    {  390, 4099,  122208, 13, }, // Asia/Krasnoyarsk
    {  390, 4761,  122221, 14, }, // Asia/Kuala_Lumpur
    {  390, 5893,  122235,  8, }, // Asia/Kuching
    {  390, 5464,  122243,  8, }, // Asia/Kuwait
    {  390, 1178,    6064,  5, }, // Asia/Macau
    {  390, 3919,  122251,  9, }, // Asia/Magadan
    {  390, 1316,  122260,  9, }, // Asia/Makassar
    {  390, 6883,  122269,  8, }, // Asia/Muscat
    {  390, 2446,   55803,  8, }, // Asia/Nicosia
    {  390, 8638,  122277, 13, }, // Asia/Novokuznetsk
    {  390, 4037,  122290, 13, }, // Asia/Novosibirsk
    {  390, 4131,  122303,  6, }, // Asia/Omsk
    {  390, 7108,  122309,  8, }, // Asia/Oral
    {  390, 6924,  122317,  9, }, // Asia/Phnom_Penh
    {  390, 8656,  122326, 11, }, // Asia/Pontianak
    {  390, 4116,  122337, 10, }, // Asia/Pyongyang
    {  390, 5476,  122347,  7, }, // Asia/Qatar
    {  390, 8671,  122354, 10, }, // Asia/Qostanay
    {  390, 4171,  122364,  9, }, // Asia/Qyzylorda
    {  390, 3486,  122373,  7, }, // Asia/Riyadh
    {  390, 4310,  122380, 10, }, // Asia/Sakhalin
    {  390, 7208,  122390, 11, }, // Asia/Samarkand
    {  390, 3116,  122401,  6, }, // Asia/Seoul
    {  390, 1020,  122407,  9, }, // Asia/Shanghai
    {  390, 3137,  122416, 10, }, // Asia/Singapore
    {  390, 4199,  122426, 15, }, // Asia/Srednekolymsk
    {  390, 3100,  122441,  8, }, // Asia/Taipei
    {  390, 4683,  122449,  9, }, // Asia/Tashkent
    {  390, 3817,  122458,  8, }, // Asia/Tbilisi
    {  390, 2638,  122466,  9, }, // Asia/Tehran
    {  390, 1284,   90291,  5, }, // Asia/Thimphu
    {  390, 2687,  122475,  7, }, // Asia/Tokyo
    {  390, 4474,  122482,  7, }, // Asia/Tomsk
    {  390, 1346,  122489, 12, }, // Asia/Ulaanbaatar
    {  390, 1126,   32067,  6, }, // Asia/Urumqi
    {  390, 8685,  122501,  8, }, // Asia/Ust-Nera
    {  390, 6940,  122509, 10, }, // Asia/Vientiane
    {  390, 4612,  122519, 13, }, // Asia/Vladivostok
    {  390, 4730,  122532,  9, }, // Asia/Yakutsk
    {  390, 1202,  122541,  8, }, // Asia/Yangon
    {  390, 3785,  122549, 15, }, // Asia/Yekaterinburg
    {  390, 3649,  122564,  9, }, // Asia/Yerevan
    {  390, 3565,  122573,  6, }, // Atlantic/Azores
    {  390, 6591,  122579, 12, }, // Atlantic/Canary
    {  390, 3629,  122591, 16, }, // Atlantic/Cape_Verde
    {  390, 1379,  122607,  7, }, // Atlantic/Faroe
    {  390, 2614,  122614, 11, }, // Atlantic/Reykjavik
    {  390, 7235,  122625, 15, }, // Atlantic/South_Georgia
    {  390, 6784,  122640, 11, }, // Atlantic/St_Helena
    {  390, 6607,  122651,  7, }, // Atlantic/Stanley
    {  390, 1654,  122658,  8, }, // Australia/Adelaide
    {  390, 1619,  122666,  9, }, // Australia/Brisbane
    {  390, 1783,  122675, 12, }, // Australia/Broken_Hill
    {  390, 1567,  122687,  8, }, // Australia/Darwin
    {  390, 3539,  119609,  5, }, // Australia/Eucla
    {  390, 1500,  122695,  8, }, // Australia/Hobart
    {  390, 8716,  122703, 10, }, // Australia/Lindeman
    {  390, 1531,  122713, 14, }, // Australia/Lord_Howe
    {  390, 1711,  122727,  9, }, // Australia/Melbourne
    {  390, 1746,  122736,  6, }, // Australia/Perth
    {  390, 1447,  122742,  8, }, // Australia/Sydney
    {  390, 6390,  122750, 11, }, // Europe/Amsterdam
    {  390, 6149,   32109,  6, }, // Europe/Andorra
    {  390, 3522,  122761, 10, }, // Europe/Astrakhan
    {  390, 6577,  122771,  6, }, // Europe/Athens
    {  390, 6285,  122777,  9, }, // Europe/Belgrade
    {  390, 4659,  122786,  8, }, // Europe/Berlin
    {  390, 6178,  122794,  9, }, // Europe/Brussels
    {  390, 3830,  122803, 10, }, // Europe/Bucharest
    {  390, 3708,  122813, 10, }, // Europe/Budapest
    {  390, 8751,  122823, 11, }, // Europe/Busingen
    {  390, 2475,  122834, 10, }, // Europe/Chisinau
    {  390, 6238,  122844,  9, }, // Europe/Copenhagen
    {  390, 2260,  122853,  8, }, // Europe/Dublin
    {  390, 6256,  122861, 11, }, // Europe/Gibraltar
    {  390, 8767,  122872,  7, }, // Europe/Guernsey
    {  390, 6561,  122879,  9, }, // Europe/Helsinki
    {  390, 8783,  122888,  9, }, // Europe/Isle_of_Man
    {  390, 1097,  122897,  9, }, // Europe/Istanbul
    {  390, 8802,  122906,  7, }, // Europe/Jersey
    {  390, 3881,  122913, 13, }, // Europe/Kaliningrad
    {  390, 8924,  122926,  7, }, // Europe/Kirov
    {  390, 2419,  122933,  7, }, // Europe/Kyiv
    {  390, 3078,  119826,  8, }, // Europe/Lisbon
    {  390, 6455,   19534,  9, }, // Europe/Ljubljana
    {  390, 2393,  122940,  8, }, // Europe/London
    {  390, 6314,  122948, 13, }, // Europe/Luxembourg
    {  390, 4794,  122961,  8, }, // Europe/Madrid
    {  390, 6531,  122969, 12, }, // Europe/Mariehamn
    {  390, 3595,  122981,  7, }, // Europe/Minsk
    {  390, 6359,  122988,  7, }, // Europe/Monaco
    {  390, 3443,  122995,  6, }, // Europe/Moscow
    {  390, 6407,  123001,  5, }, // Europe/Oslo
    {  390, 4186,  123006,  8, }, // Europe/Paris
    {  390, 6224,   34765,  5, }, // Europe/Prague
    {  390, 8816,  123014,  4, }, // Europe/Riga
    {  390, 6273,   19581,  4, }, // Europe/Rome
    {  390, 6419,  123018, 11, }, // Europe/San_Marino
    {  390, 6194,  123029,  9, }, // Europe/Sarajevo
    {  390, 4353,  123038,  9, }, // Europe/Saratov
    {  390, 8828,  123047, 12, }, // Europe/Simferopol
    {  390, 6332,  123059,  6, }, // Europe/Skopje
    {  390, 6548,   32232,  6, }, // Europe/Sofia
    {  390, 6472,  123065, 10, }, // Europe/Stockholm
    {  390, 8846,  123075,  7, }, // Europe/Tallinn
    {  390, 6135,   19608,  6, }, // Europe/Tirane
    {  390, 8861,  123082, 11, }, // Europe/Ulyanovsk
    {  390, 6301,  123093,  7, }, // Europe/Vaduz
    {  390, 6516,  123100,  9, }, // Europe/Vatican
    {  390, 6164,   19625,  5, }, // Europe/Vienna
    {  390, 4629,  123109, 11, }, // Europe/Volgograd
    {  390, 3055,  123120,  7, }, // Europe/Warsaw
    {  390, 6210,  123127,  8, }, // Europe/Zagreb
    {  390, 6489,  123135,  9, }, // Europe/Zurich
    {  390, 4980,  123144, 13, }, // Indian/Antananarivo
    {  390, 6910,  123157,  7, }, // Indian/Chagos
    {  390, 5979,  123164, 11, }, // Indian/Christmas
    {  390, 5996,  123175, 11, }, // Indian/Cocos
    {  390, 4931,  120056,  7, }, // Indian/Comoro
    {  390, 6624,  123186, 10, }, // Indian/Kerguelen
    {  390, 7223,  123196,  4, }, // Indian/Mahe
    {  390, 7013,  123200,  8, }, // Indian/Maldives
    {  390, 3971,   32297, 10, }, // Indian/Mauritius
    {  390, 5000,  123208,  7, }, // Indian/Mayotte
    {  390, 7174,  123215,  9, }, // Indian/Reunion
    {  390, 4324,  120112,  5, }, // Pacific/Apia
    {  390,  932,  123224,  8, }, // Pacific/Auckland
    {  390, 3608,  123232, 10, }, // Pacific/Bougainville
    {  390, 2863,  123242,  7, }, // Pacific/Chatham
    {  390, 3022,  123249,  5, }, // Pacific/Chuuk
    {  390, 2201,  123254, 11, }, // Pacific/Easter
    {  390, 7323,  123265,  5, }, // Pacific/Efate
    {  390, 7290,  123270,  8, }, // Pacific/Fakaofo
    {  390, 3804,  123278,  6, }, // Pacific/Fiji
    {  390, 7306,  123284,  9, }, // Pacific/Funafuti
    {  390, 6641,  123293,  9, }, // Pacific/Galapagos
    {  390, 6659,  123302,  8, }, // Pacific/Gambier
    {  390, 3724,  123310, 13, }, // Pacific/Guadalcanal
    {  390, 5935,  123323,  6, }, // Pacific/Guam
    {  390, 2929,    4512,  8, }, // Pacific/Honolulu
    {  390, 2897,  123329, 10, }, // Pacific/Kanton
    {  390, 3900,  123339, 11, }, // Pacific/Kiritimati
    {  390, 6982,  123350,  7, }, // Pacific/Kosrae
    {  390, 2708,  123357, 12, }, // Pacific/Kwajalein
    {  390, 4779,  123369,  8, }, // Pacific/Majuro
    {  390, 3953,  123377, 13, }, // Pacific/Marquesas
    {  390, 8909,  123390,  8, }, // Pacific/Midway
    {  390, 7095,  123398,  5, }, // Pacific/Niue
    {  390, 4070,  123403,  9, }, // Pacific/Norfolk
    {  390, 7061,  123412,  6, }, // Pacific/Noumea
    {  390, 2991,  123418, 10, }, // Pacific/Pago_Pago
    {  390, 7157,  123428, 13, }, // Pacific/Pitcairn
    {  390, 2961,  123441,  6, }, // Pacific/Pohnpei
    {  390, 4709,  123447, 12, }, // Pacific/Port_Moresby
    {  390, 5948,  123459,  8, }, // Pacific/Saipan
    {  390, 7275,  123467,  6, }, // Pacific/Tahiti
    {  390, 6675,   21574,  6, }, // Pacific/Tarawa
    {  390, 7355,  123473, 10, }, // Pacific/Wake
    {  390, 7368,  123483,  7, }, // Pacific/Wallis
    {  391,   14,       0,  6, }, // Africa/Asmara Lojban/Latin/world
    {  391, 4337,       6,  8, }, // Africa/Sao_Tome
    {  391, 4154,      14,  8, }, // America/Asuncion
    {  391,  237,      22,  8, }, // America/Atikokan
    {  391, 7586,      30, 17, }, // America/Bahia_Banderas
    {  391, 3770,      47,  6, }, // America/Cancun
    {  391, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  391, 5640,      66,  7, }, // America/Curacao
    {  391,  481,      73, 13, }, // America/Indiana/Knox
    {  391, 7924,      86, 16, }, // America/Indiana/Marengo
    {  391, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  391, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  391, 8001,     139, 14, }, // America/Indiana/Vevay
    {  391, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  391, 8049,     171, 16, }, // America/Indiana/Winamac
    {  391, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  391, 5849,     207, 22, }, // America/Lower_Princes
    {  391, 8183,     229,  6, }, // America/Merida
    {  391, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  391, 8292,     255, 20, }, // America/North_Dakota/Center
    {  391, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  391,  393,     298,  4, }, // America/Nuuk
    {  391, 6862,     302, 16, }, // America/Scoresbysund
    {  391, 8471,     318, 14, }, // America/St_Barthelemy
    {  391, 2045,     332, 10, }, // America/St_Johns
    {  391, 5780,     342,  9, }, // America/St_Kitts
    {  391, 5797,     351,  9, }, // America/St_Lucia
    {  391,  855,     360, 10, }, // America/St_Thomas
    {  391, 5830,     370, 11, }, // America/St_Vincent
    {  391, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  391, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  391, 1152,     413,  9, }, // Asia/Kathmandu
    {  391,  992,     422,  7, }, // Asia/Kolkata
    {  391, 1178,     429,  5, }, // Asia/Macau
    {  391, 1202,     442,  6, }, // Asia/Yangon
    {  391, 1379,     448,  5, }, // Atlantic/Faroe
    {  391, 6784,     453, 10, }, // Atlantic/St_Helena
    {  391, 2419,     463,  4, }, // Europe/Kyiv
    {  391, 7174,     467,  7, }, // Indian/Reunion
    {  391, 3022,     474,  5, }, // Pacific/Chuuk
    {  391, 2961,     479,  7, }, // Pacific/Pohnpei
    {  392, 6723,   19687,  5, }, // Africa/Accra Lower Sorbian/Latin/Germany
    {  392, 4961,    3793, 11, }, // Africa/Addis_Ababa
    {  392, 7383,   57088,  6, }, // Africa/Algiers
    {  392,   14,       0,  6, }, // Africa/Asmara
    {  392, 2242,   24913,  5, }, // Africa/Cairo
    {  392, 5032,   76549, 10, }, // Africa/Dar_es_Salaam
    {  392, 4945,   31754,  7, }, // Africa/Djibouti
    {  392, 5068,   62570,  8, }, // Africa/El_Aaiun
    {  392, 4426,   24937,  7, }, // Africa/Khartoum
    {  392, 6832,   18802,  4, }, // Africa/Lome
    {  392, 5015,   56254,  8, }, // Africa/Mogadishu
    {  392, 6690,   97031,  8, }, // Africa/Ouagadougou
    {  392, 5127,   24975, 10, }, // Africa/Porto-Novo
    {  392, 4337,       6,  8, }, // Africa/Sao_Tome
    {  392, 2732,   56289,  8, }, // Africa/Tripoli
    {  392, 4154,      14,  8, }, // America/Asuncion
    {  392,  237,      22,  8, }, // America/Atikokan
    {  392, 7586,      30, 17, }, // America/Bahia_Banderas
    {  392, 4263,   18898,  6, }, // America/Bogota
    {  392, 3770,      47,  6, }, // America/Cancun
    {  392, 5398,  123490, 15, }, // America/Cayman
    {  392, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  392, 5640,      66,  7, }, // America/Curacao
    {  392, 2221,   61885,  7, }, // America/Havana
    {  392,  481,      73, 13, }, // America/Indiana/Knox
    {  392, 7924,      86, 16, }, // America/Indiana/Marengo
    {  392, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  392, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  392, 8001,     139, 14, }, // America/Indiana/Vevay
    {  392, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  392, 8049,     171, 16, }, // America/Indiana/Winamac
    {  392, 2665,    3892,  7, }, // America/Jamaica
    {  392, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  392, 5849,     207, 22, }, // America/Lower_Princes
    {  392, 8183,     229,  6, }, // America/Merida
    {  392, 2811,  123505, 12, }, // America/Mexico_City
    {  392, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  392, 8292,     255, 20, }, // America/North_Dakota/Center
    {  392, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  392,  393,     298,  4, }, // America/Nuuk
    {  392, 5871,   25124, 13, }, // America/Port_of_Spain
    {  392, 1862,   25145,  9, }, // America/Sao_Paulo
    {  392, 6862,     302, 16, }, // America/Scoresbysund
    {  392, 8471,  123517, 13, }, // America/St_Barthelemy
    {  392, 2045,     332, 10, }, // America/St_Johns
    {  392, 5780,     342,  9, }, // America/St_Kitts
    {  392, 5797,     351,  9, }, // America/St_Lucia
    {  392,  855,     360, 10, }, // America/St_Thomas
    {  392, 5830,     370, 11, }, // America/St_Vincent
    {  392, 6066,  123530, 16, }, // Antarctica/DumontDUrville
    {  392, 7337,    4048,  6, }, // Antarctica/Vostok
    {  392, 5270,    5911,  6, }, // Asia/Aqtobe
    {  392, 3509,    4063,  6, }, // Asia/Baghdad
    {  392, 3680,   31917,  6, }, // Asia/Bishkek
    {  392, 4442,    4102,  8, }, // Asia/Damascus
    {  392, 6092,   31939,  7, }, // Asia/Dushanbe
    {  392, 1226,  123546, 17, }, // Asia/Ho_Chi_Minh
    {  392, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  392, 6955,  123563, 11, }, // Asia/Jayapura
    {  392, 4218,   31981,  8, }, // Asia/Kamchatka
    {  392, 4141,   31989,  6, }, // Asia/Karachi
    {  392, 1152,     413,  9, }, // Asia/Kathmandu
    {  392,  992,   62730,  8, }, // Asia/Kolkata
    {  392, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  392, 1178,     429,  5, }, // Asia/Macau
    {  392, 6883,    4227,  6, }, // Asia/Muscat
    {  392, 2446,   25331,  7, }, // Asia/Nicosia
    {  392, 8638,  123574, 13, }, // Asia/Novokuznetsk
    {  392, 4037,   76692, 11, }, // Asia/Novosibirsk
    {  392, 4116,   76703,  9, }, // Asia/Pyongyang
    {  392, 5476,    4233,  5, }, // Asia/Qatar
    {  392, 3486,    6120,  4, }, // Asia/Riyadh
    {  392, 4310,   56617,  8, }, // Asia/Sakhalin
    {  392, 3137,   19291,  8, }, // Asia/Singapore
    {  392, 3100,   76712,  6, }, // Asia/Taipei
    {  392, 4683,   32060,  7, }, // Asia/Tashkent
    {  392, 2638,    6175,  7, }, // Asia/Tehran
    {  392, 2687,    4257,  5, }, // Asia/Tokyo
    {  392, 4612,    4262, 11, }, // Asia/Vladivostok
    {  392, 4730,    6206,  7, }, // Asia/Yakutsk
    {  392, 1202,    6213,  6, }, // Asia/Yangon
    {  392, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  392, 3649,  123587,  7, }, // Asia/Yerevan
    {  392, 3565,  123594,  5, }, // Atlantic/Azores
    {  392, 5588,   56708,  7, }, // Atlantic/Bermuda
    {  392, 6591,  113404,  6, }, // Atlantic/Canary
    {  392, 3629,   55900,  9, }, // Atlantic/Cape_Verde
    {  392, 1379,  123599,  6, }, // Atlantic/Faroe
    {  392, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  392, 7235,  123605, 22, }, // Atlantic/South_Georgia
    {  392, 6784,     453, 10, }, // Atlantic/St_Helena
    {  392, 6577,    4339,  5, }, // Europe/Athens
    {  392, 6285,  123627,  8, }, // Europe/Belgrade
    {  392, 6178,   21176,  7, }, // Europe/Brussels
    {  392, 3830,   25479,  8, }, // Europe/Bucharest
    {  392, 8751,   25487,  8, }, // Europe/Busingen
    {  392, 2475,  123635,  7, }, // Europe/Chisinau
    {  392, 6238,    4376, 10, }, // Europe/Copenhagen
    {  392, 2419,   76773,  4, }, // Europe/Kyiv
    {  392, 3078,    4400,  8, }, // Europe/Lisbon
    {  392, 6314,    4414,  9, }, // Europe/Luxembourg
    {  392, 3443,   91758,  6, }, // Europe/Moscow
    {  392, 6224,   19576,  5, }, // Europe/Prague
    {  392, 6273,    6554,  3, }, // Europe/Rome
    {  392, 6135,   19608,  6, }, // Europe/Tirane
    {  392, 6516,    6614,  7, }, // Europe/Vatican
    {  392, 6164,   56035,  4, }, // Europe/Vienna
    {  392, 8878,  123642,  5, }, // Europe/Vilnius
    {  392, 4629,    4462,  9, }, // Europe/Volgograd
    {  392, 3055,  123647,  7, }, // Europe/Warsaw
    {  392, 6489,    4478,  6, }, // Europe/Zurich
    {  392, 5979,  123654, 12, }, // Indian/Christmas
    {  392, 4931,   56941,  6, }, // Indian/Comoro
    {  392, 7013,  123666,  8, }, // Indian/Maldives
    {  392, 7174,     467,  7, }, // Indian/Reunion
    {  392, 3022,     474,  5, }, // Pacific/Chuuk
    {  392, 2201,  123674, 13, }, // Pacific/Easter
    {  392, 3804,   32307,  5, }, // Pacific/Fiji
    {  392, 2961,     479,  7, }, // Pacific/Pohnpei
    {  393,   14,       0,  6, }, // Africa/Asmara Low German/Latin/Germany
    {  393, 4337,       6,  8, }, // Africa/Sao_Tome
    {  393, 4154,      14,  8, }, // America/Asuncion
    {  393,  237,      22,  8, }, // America/Atikokan
    {  393, 7586,      30, 17, }, // America/Bahia_Banderas
    {  393, 3770,      47,  6, }, // America/Cancun
    {  393, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  393, 5640,      66,  7, }, // America/Curacao
    {  393,  481,      73, 13, }, // America/Indiana/Knox
    {  393, 7924,      86, 16, }, // America/Indiana/Marengo
    {  393, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  393, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  393, 8001,     139, 14, }, // America/Indiana/Vevay
    {  393, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  393, 8049,     171, 16, }, // America/Indiana/Winamac
    {  393, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  393, 5849,     207, 22, }, // America/Lower_Princes
    {  393, 8183,     229,  6, }, // America/Merida
    {  393, 2811,   76590, 12, }, // America/Mexico_City
    {  393, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  393, 8292,    3936, 20, }, // America/North_Dakota/Center
    {  393, 8320,    3956, 23, }, // America/North_Dakota/New_Salem
    {  393,  393,     298,  4, }, // America/Nuuk
    {  393, 6862,     302, 16, }, // America/Scoresbysund
    {  393, 8471,     318, 14, }, // America/St_Barthelemy
    {  393, 2045,     332, 10, }, // America/St_Johns
    {  393, 5780,     342,  9, }, // America/St_Kitts
    {  393, 5797,     351,  9, }, // America/St_Lucia
    {  393,  855,     360, 10, }, // America/St_Thomas
    {  393, 5830,     370, 11, }, // America/St_Vincent
    {  393, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  393, 7337,    4048,  6, }, // Antarctica/Vostok
    {  393, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  393, 4218,  123687, 10, }, // Asia/Kamchatka
    {  393, 1152,     413,  9, }, // Asia/Kathmandu
    {  393,  992,     422,  7, }, // Asia/Kolkata
    {  393, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  393, 1178,     429,  5, }, // Asia/Macau
    {  393, 4037,   76692, 11, }, // Asia/Novosibirsk
    {  393, 4310,   56617,  8, }, // Asia/Sakhalin
    {  393, 1346,   21008,  9, }, // Asia/Ulaanbaatar
    {  393, 4612,    4262, 11, }, // Asia/Vladivostok
    {  393, 4730,    6206,  7, }, // Asia/Yakutsk
    {  393, 1202,     442,  6, }, // Asia/Yangon
    {  393, 3785,  123697, 13, }, // Asia/Yekaterinburg
    {  393, 3565,   57378,  6, }, // Atlantic/Azores
    {  393, 6591,   76724,  7, }, // Atlantic/Canary
    {  393, 1379,     448,  5, }, // Atlantic/Faroe
    {  393, 6784,     453, 10, }, // Atlantic/St_Helena
    {  393, 2419,     463,  4, }, // Europe/Kyiv
    {  393, 3443,   56005,  6, }, // Europe/Moscow
    {  393, 4629,    4462,  9, }, // Europe/Volgograd
    {  393, 7174,     467,  7, }, // Indian/Reunion
    {  393, 3022,     474,  5, }, // Pacific/Chuuk
    {  393, 2961,     479,  7, }, // Pacific/Pohnpei
    {  395,   14,       0,  6, }, // Africa/Asmara Luba-Katanga/Latin/Congo - Kinshasa
    {  395, 4337,       6,  8, }, // Africa/Sao_Tome
    {  395, 4154,      14,  8, }, // America/Asuncion
    {  395,  237,      22,  8, }, // America/Atikokan
    {  395, 7586,      30, 17, }, // America/Bahia_Banderas
    {  395, 3770,      47,  6, }, // America/Cancun
    {  395, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  395, 5640,      66,  7, }, // America/Curacao
    {  395,  481,      73, 13, }, // America/Indiana/Knox
    {  395, 7924,      86, 16, }, // America/Indiana/Marengo
    {  395, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  395, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  395, 8001,     139, 14, }, // America/Indiana/Vevay
    {  395, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  395, 8049,     171, 16, }, // America/Indiana/Winamac
    {  395, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  395, 5849,     207, 22, }, // America/Lower_Princes
    {  395, 8183,     229,  6, }, // America/Merida
    {  395, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  395, 8292,     255, 20, }, // America/North_Dakota/Center
    {  395, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  395,  393,     298,  4, }, // America/Nuuk
    {  395, 6862,     302, 16, }, // America/Scoresbysund
    {  395, 8471,     318, 14, }, // America/St_Barthelemy
    {  395, 2045,     332, 10, }, // America/St_Johns
    {  395, 5780,     342,  9, }, // America/St_Kitts
    {  395, 5797,     351,  9, }, // America/St_Lucia
    {  395,  855,     360, 10, }, // America/St_Thomas
    {  395, 5830,     370, 11, }, // America/St_Vincent
    {  395, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  395, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  395, 1152,     413,  9, }, // Asia/Kathmandu
    {  395,  992,     422,  7, }, // Asia/Kolkata
    {  395, 1178,     429,  5, }, // Asia/Macau
    {  395, 1202,     442,  6, }, // Asia/Yangon
    {  395, 1379,     448,  5, }, // Atlantic/Faroe
    {  395, 6784,     453, 10, }, // Atlantic/St_Helena
    {  395, 2419,     463,  4, }, // Europe/Kyiv
    {  395, 7174,     467,  7, }, // Indian/Reunion
    {  395, 3022,     474,  5, }, // Pacific/Chuuk
    {  395, 2961,     479,  7, }, // Pacific/Pohnpei
    {  396,   14,       0,  6, }, // Africa/Asmara Lule Sami/Latin/Sweden
    {  396, 4337,       6,  8, }, // Africa/Sao_Tome
    {  396, 4154,      14,  8, }, // America/Asuncion
    {  396,  237,      22,  8, }, // America/Atikokan
    {  396, 7586,      30, 17, }, // America/Bahia_Banderas
    {  396, 3770,      47,  6, }, // America/Cancun
    {  396, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  396, 5640,      66,  7, }, // America/Curacao
    {  396,  481,      73, 13, }, // America/Indiana/Knox
    {  396, 7924,      86, 16, }, // America/Indiana/Marengo
    {  396, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  396, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  396, 8001,     139, 14, }, // America/Indiana/Vevay
    {  396, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  396, 8049,     171, 16, }, // America/Indiana/Winamac
    {  396, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  396, 5849,     207, 22, }, // America/Lower_Princes
    {  396, 8183,     229,  6, }, // America/Merida
    {  396, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  396, 8292,     255, 20, }, // America/North_Dakota/Center
    {  396, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  396,  393,     298,  4, }, // America/Nuuk
    {  396, 6862,     302, 16, }, // America/Scoresbysund
    {  396, 8471,     318, 14, }, // America/St_Barthelemy
    {  396, 2045,     332, 10, }, // America/St_Johns
    {  396, 5780,     342,  9, }, // America/St_Kitts
    {  396, 5797,     351,  9, }, // America/St_Lucia
    {  396,  855,     360, 10, }, // America/St_Thomas
    {  396, 5830,     370, 11, }, // America/St_Vincent
    {  396, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  396, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  396, 1152,     413,  9, }, // Asia/Kathmandu
    {  396,  992,     422,  7, }, // Asia/Kolkata
    {  396, 1178,     429,  5, }, // Asia/Macau
    {  396, 1202,     442,  6, }, // Asia/Yangon
    {  396, 1379,     448,  5, }, // Atlantic/Faroe
    {  396, 6784,     453, 10, }, // Atlantic/St_Helena
    {  396, 2419,     463,  4, }, // Europe/Kyiv
    {  396, 7174,     467,  7, }, // Indian/Reunion
    {  396, 3022,     474,  5, }, // Pacific/Chuuk
    {  396, 2961,     479,  7, }, // Pacific/Pohnpei
    {  398,   14,       0,  6, }, // Africa/Asmara Luo/Latin/Kenya
    {  398, 4337,       6,  8, }, // Africa/Sao_Tome
    {  398, 4154,      14,  8, }, // America/Asuncion
    {  398,  237,      22,  8, }, // America/Atikokan
    {  398, 7586,      30, 17, }, // America/Bahia_Banderas
    {  398, 3770,      47,  6, }, // America/Cancun
    {  398, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  398, 5640,      66,  7, }, // America/Curacao
    {  398,  481,      73, 13, }, // America/Indiana/Knox
    {  398, 7924,      86, 16, }, // America/Indiana/Marengo
    {  398, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  398, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  398, 8001,     139, 14, }, // America/Indiana/Vevay
    {  398, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  398, 8049,     171, 16, }, // America/Indiana/Winamac
    {  398, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  398, 5849,     207, 22, }, // America/Lower_Princes
    {  398, 8183,     229,  6, }, // America/Merida
    {  398, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  398, 8292,     255, 20, }, // America/North_Dakota/Center
    {  398, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  398,  393,     298,  4, }, // America/Nuuk
    {  398, 6862,     302, 16, }, // America/Scoresbysund
    {  398, 8471,     318, 14, }, // America/St_Barthelemy
    {  398, 2045,     332, 10, }, // America/St_Johns
    {  398, 5780,     342,  9, }, // America/St_Kitts
    {  398, 5797,     351,  9, }, // America/St_Lucia
    {  398,  855,     360, 10, }, // America/St_Thomas
    {  398, 5830,     370, 11, }, // America/St_Vincent
    {  398, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  398, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  398, 1152,     413,  9, }, // Asia/Kathmandu
    {  398,  992,     422,  7, }, // Asia/Kolkata
    {  398, 1178,     429,  5, }, // Asia/Macau
    {  398, 1202,     442,  6, }, // Asia/Yangon
    {  398, 1379,     448,  5, }, // Atlantic/Faroe
    {  398, 6784,     453, 10, }, // Atlantic/St_Helena
    {  398, 2419,     463,  4, }, // Europe/Kyiv
    {  398, 7174,     467,  7, }, // Indian/Reunion
    {  398, 3022,     474,  5, }, // Pacific/Chuuk
    {  398, 2961,     479,  7, }, // Pacific/Pohnpei
    {  399, 4961,    3793, 11, }, // Africa/Addis_Ababa Luxembourgish/Latin/Luxembourg
    {  399, 7383,   42592,  5, }, // Africa/Algiers
    {  399,   14,       0,  6, }, // Africa/Asmara
    {  399, 2242,   24913,  5, }, // Africa/Cairo
    {  399, 4945,   76559,  9, }, // Africa/Djibouti
    {  399, 5068,   62570,  8, }, // Africa/El_Aaiun
    {  399, 4426,   24937,  7, }, // Africa/Khartoum
    {  399, 5015,   76568, 10, }, // Africa/Mogadishu
    {  399, 6690,   97031,  8, }, // Africa/Ouagadougou
    {  399, 4337,       6,  8, }, // Africa/Sao_Tome
    {  399, 4154,      14,  8, }, // America/Asuncion
    {  399,  237,      22,  8, }, // America/Atikokan
    {  399, 7586,      30, 17, }, // America/Bahia_Banderas
    {  399, 3770,      47,  6, }, // America/Cancun
    {  399, 5398,  123710, 13, }, // America/Cayman
    {  399, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  399, 5640,      66,  7, }, // America/Curacao
    {  399, 5343,    5090,  8, }, // America/El_Salvador
    {  399, 2221,   61885,  7, }, // America/Havana
    {  399,  481,      73, 13, }, // America/Indiana/Knox
    {  399, 7924,      86, 16, }, // America/Indiana/Marengo
    {  399, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  399, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  399, 8001,     139, 14, }, // America/Indiana/Vevay
    {  399, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  399, 8049,     171, 16, }, // America/Indiana/Winamac
    {  399, 2665,    3892,  7, }, // America/Jamaica
    {  399, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  399, 5849,     207, 22, }, // America/Lower_Princes
    {  399, 8183,     229,  6, }, // America/Merida
    {  399, 2811,   76590, 11, }, // America/Mexico_City
    {  399, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  399, 8292,     255, 20, }, // America/North_Dakota/Center
    {  399, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  399,  393,     298,  4, }, // America/Nuuk
    {  399, 5871,   25124, 13, }, // America/Port_of_Spain
    {  399, 6862,     302, 16, }, // America/Scoresbysund
    {  399, 8471,   25154, 16, }, // America/St_Barthelemy
    {  399, 2045,     332, 10, }, // America/St_Johns
    {  399, 5780,     342,  9, }, // America/St_Kitts
    {  399, 5797,     351,  9, }, // America/St_Lucia
    {  399,  855,     360, 10, }, // America/St_Thomas
    {  399, 5830,     370, 11, }, // America/St_Vincent
    {  399, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  399, 7337,    4048,  6, }, // Antarctica/Vostok
    {  399, 5270,   55725,  6, }, // Asia/Aqtobe
    {  399, 3509,    4063,  6, }, // Asia/Baghdad
    {  399, 3680,   55740,  8, }, // Asia/Bishkek
    {  399, 4442,    4102,  8, }, // Asia/Damascus
    {  399, 6092,   55748,  9, }, // Asia/Dushanbe
    {  399, 1226,   76638, 16, }, // Asia/Ho_Chi_Minh
    {  399, 6955,  123563, 11, }, // Asia/Jayapura
    {  399, 4218,   76660, 11, }, // Asia/Kamchatka
    {  399, 1152,     413,  9, }, // Asia/Kathmandu
    {  399,  992,   62730,  8, }, // Asia/Kolkata
    {  399, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  399, 1178,    4222,  5, }, // Asia/Macau
    {  399, 6883,    4227,  6, }, // Asia/Muscat
    {  399, 2446,   25331,  7, }, // Asia/Nicosia
    {  399, 4037,   76692, 11, }, // Asia/Novosibirsk
    {  399, 4116,   76703,  9, }, // Asia/Pyongyang
    {  399, 5476,    4233,  5, }, // Asia/Qatar
    {  399, 3486,    6120,  4, }, // Asia/Riyadh
    {  399, 4310,   56617,  8, }, // Asia/Sakhalin
    {  399, 3137,   19291,  8, }, // Asia/Singapore
    {  399, 3100,   76712,  6, }, // Asia/Taipei
    {  399, 4683,   55848,  9, }, // Asia/Tashkent
    {  399, 3817,   76718,  6, }, // Asia/Tbilisi
    {  399, 2638,    6175,  7, }, // Asia/Tehran
    {  399, 2687,    4257,  5, }, // Asia/Tokyo
    {  399, 4612,    4262, 11, }, // Asia/Vladivostok
    {  399, 4730,    6206,  7, }, // Asia/Yakutsk
    {  399, 1202,     442,  6, }, // Asia/Yangon
    {  399, 3785,  123723, 14, }, // Asia/Yekaterinburg
    {  399, 3649,  123737,  6, }, // Asia/Yerevan
    {  399, 3565,   57378,  6, }, // Atlantic/Azores
    {  399, 5588,   72942,  8, }, // Atlantic/Bermuda
    {  399, 6591,  123743, 17, }, // Atlantic/Canary
    {  399, 3629,   55900,  9, }, // Atlantic/Cape_Verde
    {  399, 1379,   76731,  6, }, // Atlantic/Faroe
    {  399, 7235,   76748, 11, }, // Atlantic/South_Georgia
    {  399, 6784,     453, 10, }, // Atlantic/St_Helena
    {  399, 6577,    4339,  5, }, // Europe/Athens
    {  399, 6285,   25465,  7, }, // Europe/Belgrade
    {  399, 6178,  123760,  8, }, // Europe/Brussels
    {  399, 3830,   25479,  8, }, // Europe/Bucharest
    {  399, 2475,  123768,  9, }, // Europe/Chisinau
    {  399, 6238,    4376, 10, }, // Europe/Copenhagen
    {  399, 2419,   76773,  4, }, // Europe/Kyiv
    {  399, 3078,    4400,  8, }, // Europe/Lisbon
    {  399, 6314,  123777, 10, }, // Europe/Luxembourg
    {  399, 3443,   56005,  6, }, // Europe/Moscow
    {  399, 6224,    6545,  4, }, // Europe/Prague
    {  399, 6273,  123787,  4, }, // Europe/Rome
    {  399, 6135,   19608,  6, }, // Europe/Tirane
    {  399, 6516,    6614,  7, }, // Europe/Vatican
    {  399, 6164,   56035,  4, }, // Europe/Vienna
    {  399, 8878,  123642,  5, }, // Europe/Vilnius
    {  399, 4629,    4462,  9, }, // Europe/Volgograd
    {  399, 3055,   56048,  8, }, // Europe/Warsaw
    {  399, 6489,  123791,  6, }, // Europe/Zurich
    {  399, 5979,  123797, 17, }, // Indian/Christmas
    {  399, 4931,   76808,  7, }, // Indian/Comoro
    {  399, 7013,   57532,  8, }, // Indian/Maldives
    {  399, 7174,     467,  7, }, // Indian/Reunion
    {  399, 3022,     474,  5, }, // Pacific/Chuuk
    {  399, 2201,  123814, 13, }, // Pacific/Easter
    {  399, 3804,   76834,  7, }, // Pacific/Fiji
    {  399, 2961,     479,  7, }, // Pacific/Pohnpei
    {  400,   14,       0,  6, }, // Africa/Asmara Luyia/Latin/Kenya
    {  400, 4337,       6,  8, }, // Africa/Sao_Tome
    {  400, 4154,      14,  8, }, // America/Asuncion
    {  400,  237,      22,  8, }, // America/Atikokan
    {  400, 7586,      30, 17, }, // America/Bahia_Banderas
    {  400, 3770,      47,  6, }, // America/Cancun
    {  400, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  400, 5640,      66,  7, }, // America/Curacao
    {  400,  481,      73, 13, }, // America/Indiana/Knox
    {  400, 7924,      86, 16, }, // America/Indiana/Marengo
    {  400, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  400, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  400, 8001,     139, 14, }, // America/Indiana/Vevay
    {  400, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  400, 8049,     171, 16, }, // America/Indiana/Winamac
    {  400, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  400, 5849,     207, 22, }, // America/Lower_Princes
    {  400, 8183,     229,  6, }, // America/Merida
    {  400, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  400, 8292,     255, 20, }, // America/North_Dakota/Center
    {  400, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  400,  393,     298,  4, }, // America/Nuuk
    {  400, 6862,     302, 16, }, // America/Scoresbysund
    {  400, 8471,     318, 14, }, // America/St_Barthelemy
    {  400, 2045,     332, 10, }, // America/St_Johns
    {  400, 5780,     342,  9, }, // America/St_Kitts
    {  400, 5797,     351,  9, }, // America/St_Lucia
    {  400,  855,     360, 10, }, // America/St_Thomas
    {  400, 5830,     370, 11, }, // America/St_Vincent
    {  400, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  400, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  400, 1152,     413,  9, }, // Asia/Kathmandu
    {  400,  992,     422,  7, }, // Asia/Kolkata
    {  400, 1178,     429,  5, }, // Asia/Macau
    {  400, 1202,     442,  6, }, // Asia/Yangon
    {  400, 1379,     448,  5, }, // Atlantic/Faroe
    {  400, 6784,     453, 10, }, // Atlantic/St_Helena
    {  400, 2419,     463,  4, }, // Europe/Kyiv
    {  400, 7174,     467,  7, }, // Indian/Reunion
    {  400, 3022,     474,  5, }, // Pacific/Chuuk
    {  400, 2961,     479,  7, }, // Pacific/Pohnpei
    {  401, 6751,     486,  6, }, // Africa/Abidjan Macedonian/Cyrillic/Macedonia
    {  401, 6723,   25628,  4, }, // Africa/Accra
    {  401, 4961,   32317, 10, }, // Africa/Addis_Ababa
    {  401, 7383,     508,  5, }, // Africa/Algiers
    {  401,   14,   25647,  6, }, // Africa/Asmara
    {  401,   44,     519,  6, }, // Africa/Bamako
    {  401, 5159,     525,  5, }, // Africa/Bangui
    {  401, 6709,  123827,  6, }, // Africa/Banjul
    {  401, 7398,     536,  5, }, // Africa/Bissau
    {  401, 4859,  123833,  8, }, // Africa/Blantyre
    {  401, 5189,   32345,  8, }, // Africa/Brazzaville
    {  401, 4824,   32353,  9, }, // Africa/Bujumbura
    {  401, 2242,   32362,  5, }, // Africa/Cairo
    {  401, 4019,   32367, 10, }, // Africa/Casablanca
    {  401, 7412,     581,  5, }, // Africa/Ceuta
    {  401, 6736,     586,  7, }, // Africa/Conakry
    {  401, 6803,     593,  5, }, // Africa/Dakar
    {  401, 5032,   34847, 12, }, // Africa/Dar_es_Salaam
    {  401, 4945,     610,  6, }, // Africa/Djibouti
    {  401, 5145,     616,  5, }, // Africa/Douala
    {  401, 5068,   32377,  7, }, // Africa/El_Aaiun
    {  401, 6816,     628,  7, }, // Africa/Freetown
    {  401, 4808,     635,  8, }, // Africa/Gaborone
    {  401, 4917,   32384,  6, }, // Africa/Harare
    {  401, 4381,   32390, 11, }, // Africa/Johannesburg
    {  401, 4401,     661,  4, }, // Africa/Juba
    {  401, 5053,     665,  7, }, // Africa/Kampala
    {  401, 4426,   32401,  6, }, // Africa/Khartoum
    {  401, 4875,     678,  6, }, // Africa/Kigali
    {  401, 5208,     684,  7, }, // Africa/Kinshasa
    {  401, 4646,     691,  5, }, // Africa/Lagos
    {  401, 5238,  123841,  7, }, // Africa/Libreville
    {  401, 6832,     704,  4, }, // Africa/Lome
    {  401, 5113,     708,  6, }, // Africa/Luanda
    {  401, 4841,     714,  9, }, // Africa/Lubumbashi
    {  401, 4903,     723,  6, }, // Africa/Lusaka
    {  401, 5224,     729,  6, }, // Africa/Malabo
    {  401, 4889,   32416,  6, }, // Africa/Maputo
    {  401, 5099,     741,  6, }, // Africa/Maseru
    {  401, 5084,     747,  7, }, // Africa/Mbabane
    {  401, 5015,   34891,  8, }, // Africa/Mogadishu
    {  401, 6997,   32422,  9, }, // Africa/Monrovia
    {  401, 3755,   32431,  7, }, // Africa/Nairobi
    {  401, 5173,     777,  7, }, // Africa/Ndjamena
    {  401, 5256,   32438,  7, }, // Africa/Niamey
    {  401, 6766,     790,  7, }, // Africa/Nouakchott
    {  401, 6690,     797,  8, }, // Africa/Ouagadougou
    {  401, 5127,   32445, 10, }, // Africa/Porto-Novo
    {  401, 4337,  123848,  9, }, // Africa/Sao_Tome
    {  401, 2732,     823,  7, }, // Africa/Tripoli
    {  401, 6503,     830,  5, }, // Africa/Tunis
    {  401, 4054,     835,  7, }, // Africa/Windhoek
    {  401,  132,     842,  4, }, // America/Adak
    {  401, 3183,   32463,  7, }, // America/Anchorage
    {  401, 5524,   32470,  7, }, // America/Anguilla
    {  401, 5541,  123857,  6, }, // America/Antigua
    {  401, 4456,   32484, 10, }, // America/Araguaina
    {  401,  166,   32494, 12, }, // America/Argentina/Buenos_Aires
    {  401,   91,     888,  9, }, // America/Argentina/Catamarca
    {  401,  270,   32506,  7, }, // America/Argentina/Cordoba
    {  401,  441,   32513,  5, }, // America/Argentina/Jujuy
    {  401, 7425,   32518,  8, }, // America/Argentina/La_Rioja
    {  401,  565,   32526,  7, }, // America/Argentina/Mendoza
    {  401, 7452,   34961, 11, }, // America/Argentina/Rio_Gallegos
    {  401, 7483,     935,  5, }, // America/Argentina/Salta
    {  401, 7507,   32544,  8, }, // America/Argentina/San_Juan
    {  401, 5497,   34972,  8, }, // America/Argentina/San_Luis
    {  401, 7534,     956,  7, }, // America/Argentina/Tucuman
    {  401, 7560,  123863,  6, }, // America/Argentina/Ushuaia
    {  401, 5557,     969,  5, }, // America/Aruba
    {  401, 4154,   32566,  8, }, // America/Asuncion
    {  401,  237,  123869, 12, }, // America/Atikokan
    {  401, 3581,  123881,  5, }, // America/Bahia
    {  401, 7586,  123886, 18, }, // America/Bahia_Banderas
    {  401, 5571,    1013,  8, }, // America/Barbados
    {  401, 7609,   32601,  5, }, // America/Belem
    {  401, 5309,   32606,  6, }, // America/Belize
    {  401, 7623,   32612, 13, }, // America/Blanc-Sablon
    {  401, 7644,   32625,  9, }, // America/Boa_Vista
    {  401, 4263,    1052,  6, }, // America/Bogota
    {  401, 7662,   32634,  5, }, // America/Boise
    {  401, 7676,   32639, 11, }, // America/Cambridge_Bay
    {  401, 7698,   32650, 12, }, // America/Campo_Grande
    {  401, 3770,    1087,  6, }, // America/Cancun
    {  401, 4596,    1093,  7, }, // America/Caracas
    {  401, 4247,  123904,  6, }, // America/Cayenne
    {  401, 5398,  123910, 17, }, // America/Cayman
    {  401, 3251,    1129,  6, }, // America/Chicago
    {  401, 7719,   32683,  6, }, // America/Chihuahua
    {  401, 7737,  123927, 13, }, // America/Ciudad_Juarez
    {  401, 5324,   32689,  9, }, // America/Costa_Rica
    {  401, 7759,    1151,  7, }, // America/Creston
    {  401, 3693,  123940,  6, }, // America/Cuiaba
    {  401, 5640,   32698,  7, }, // America/Curacao
    {  401, 7775,  123946, 10, }, // America/Danmarkshavn
    {  401, 7796,   32718,  5, }, // America/Dawson
    {  401, 7811,   32723, 10, }, // America/Dawson_Creek
    {  401,  805,    1201,  6, }, // America/Denver
    {  401, 3351,    1207,  7, }, // America/Detroit
    {  401, 5656,    1214,  8, }, // America/Dominica
    {  401,  893,    1222,  8, }, // America/Edmonton
    {  401, 7832,    1230,  8, }, // America/Eirunepe
    {  401, 5343,  123956, 11, }, // America/El_Salvador
    {  401, 7849,    1246, 11, }, // America/Fort_Nelson
    {  401, 7869,    1257,  9, }, // America/Fortaleza
    {  401, 7887,   32733,  9, }, // America/Glace_Bay
    {  401, 6844,   32742,  7, }, // America/Goose_Bay
    {  401, 4514,   32749, 10, }, // America/Grand_Turk
    {  401, 5687,    1292,  7, }, // America/Grenada
    {  401, 5703,   32759,  9, }, // America/Guadeloupe
    {  401, 3662,    1308,  9, }, // America/Guatemala
    {  401, 4743,   32768,  8, }, // America/Guayaquil
    {  401, 6895,  123967,  7, }, // America/Guyana
    {  401, 1923,   32781,  8, }, // America/Halifax
    {  401, 2221,   32789,  6, }, // America/Havana
    {  401, 7905,  123974,  8, }, // America/Hermosillo
    {  401,  348,  123982, 13, }, // America/Indiana/Indianapolis
    {  401,  481,  123995, 14, }, // America/Indiana/Knox
    {  401, 7924,   32819, 17, }, // America/Indiana/Marengo
    {  401, 7948,  124009, 20, }, // America/Indiana/Petersburg
    {  401, 7975,   32856, 18, }, // America/Indiana/Tell_City
    {  401, 8001,  124029, 15, }, // America/Indiana/Vevay
    {  401, 8023,  124044, 16, }, // America/Indiana/Vincennes
    {  401, 8049,   32907, 17, }, // America/Indiana/Winamac
    {  401, 8073,    1452,  6, }, // America/Inuvik
    {  401,  660,    1458,  7, }, // America/Iqaluit
    {  401, 2665,   32932,  7, }, // America/Jamaica
    {  401, 5282,    1472,  4, }, // America/Juneau
    {  401,  521,  113944,  7, }, // America/Kentucky/Louisville
    {  401, 8088,   32943, 18, }, // America/Kentucky/Monticello
    {  401, 5621,  124060, 10, }, // America/Kralendijk
    {  401, 4278,   32971,  6, }, // America/La_Paz
    {  401, 7132,    1520,  4, }, // America/Lima
    {  401, 3390,  124070, 11, }, // America/Los_Angeles
    {  401, 5849,  124081, 18, }, // America/Lower_Princes
    {  401, 8116,   33007,  6, }, // America/Maceio
    {  401, 8131,   33013,  7, }, // America/Managua
    {  401, 1892,    1566,  6, }, // America/Manaus
    {  401, 5814,    1572,  6, }, // America/Marigot
    {  401, 5722,    1578,  8, }, // America/Martinique
    {  401, 8147,    1587,  9, }, // America/Matamoros
    {  401, 2779,   33020,  8, }, // America/Mazatlan
    {  401, 8165,    1604,  8, }, // America/Menominee
    {  401, 8183,    1612,  6, }, // America/Merida
    {  401, 8198,    1618, 10, }, // America/Metlakatla
    {  401, 2811,   33028, 12, }, // America/Mexico_City
    {  401, 4293,  124099,  7, }, // America/Miquelon
    {  401, 8217,    1641,  7, }, // America/Moncton
    {  401, 8233,   33040,  8, }, // America/Monterrey
    {  401, 4000,    1657, 10, }, // America/Montevideo
    {  401, 5741,   33048,  9, }, // America/Montserrat
    {  401, 5383,   33057,  5, }, // America/Nassau
    {  401, 3294,   33062,  6, }, // America/New_York
    {  401, 8251,    1691,  3, }, // America/Nome
    {  401, 1834,   33068,  6, }, // America/Noronha
    {  401, 8264,  124106, 21, }, // America/North_Dakota/Beulah
    {  401, 8292,   33096, 22, }, // America/North_Dakota/Center
    {  401, 8320,  124127, 24, }, // America/North_Dakota/New_Salem
    {  401,  393,   26733,  3, }, // America/Nuuk
    {  401, 8351,    1776,  7, }, // America/Ojinaga
    {  401, 5413,    1783,  6, }, // America/Panama
    {  401, 6106,    1789, 10, }, // America/Paramaribo
    {  401, 3224,   33144,  6, }, // America/Phoenix
    {  401, 3847,  124151, 12, }, // America/Port-au-Prince
    {  401, 5871,  124163, 13, }, // America/Port_of_Spain
    {  401, 8367,   33163, 10, }, // America/Porto_Velho
    {  401, 5760,   33173,  9, }, // America/Puerto_Rico
    {  401, 3932,   33182, 12, }, // America/Punta_Arenas
    {  401, 8387,  124176, 12, }, // America/Rankin_Inlet
    {  401, 8408,    1876,  6, }, // America/Recife
    {  401, 1979,  124188,  7, }, // America/Regina
    {  401, 8423,  114126,  7, }, // America/Resolute
    {  401,  695,   33225, 10, }, // America/Rio_Branco
    {  401, 8440,   33235,  8, }, // America/Santarem
    {  401, 2165,   33243,  9, }, // America/Santiago
    {  401, 6044,   33252, 13, }, // America/Santo_Domingo
    {  401, 1862,   33265,  9, }, // America/Sao_Paulo
    {  401, 6862,   33274, 12, }, // America/Scoresbysund
    {  401, 8457,    1956,  5, }, // America/Sitka
    {  401, 8471,  124195, 15, }, // America/St_Barthelemy
    {  401, 2045,  124210,  9, }, // America/St_Johns
    {  401, 5780,  124219, 10, }, // America/St_Kitts
    {  401, 5797,  124229, 11, }, // America/St_Lucia
    {  401,  855,  124240, 10, }, // America/St_Thomas
    {  401, 5830,   33334, 12, }, // America/St_Vincent
    {  401, 8493,   33346, 12, }, // America/Swift_Current
    {  401, 5363,    2038, 11, }, // America/Tegucigalpa
    {  401, 5673,    2049,  4, }, // America/Thule
    {  401,  313,    2053,  7, }, // America/Tijuana
    {  401,  608,    2060,  7, }, // America/Toronto
    {  401, 5605,    2067,  7, }, // America/Tortola
    {  401, 2077,    2074,  8, }, // America/Vancouver
    {  401, 2128,   33358,  8, }, // America/Whitehorse
    {  401,  734,   33366,  7, }, // America/Winnipeg
    {  401, 8515,   33373,  7, }, // America/Yakutat
    {  401, 5918,  124250,  5, }, // Antarctica/Casey
    {  401, 6027,   33385,  6, }, // Antarctica/Davis
    {  401, 6066,  124255, 12, }, // Antarctica/DumontDUrville
    {  401, 8531,  124267,  7, }, // Antarctica/Macquarie
    {  401, 7029,   33411,  5, }, // Antarctica/Mawson
    {  401, 7076,   33416,  8, }, // Antarctica/McMurdo
    {  401, 8552,    2154,  6, }, // Antarctica/Palmer
    {  401, 7189,    2160,  6, }, // Antarctica/Rothera
    {  401, 7258,  124274,  6, }, // Antarctica/Syowa
    {  401, 8570,    2171,  4, }, // Antarctica/Troll
    {  401, 7337,   33428,  6, }, // Antarctica/Vostok
    {  401, 1413,  124280, 12, }, // Arctic/Longyearbyen
    {  401, 5487,    2192,  4, }, // Asia/Aden
    {  401, 5297,   33445,  6, }, // Asia/Almaty
    {  401, 3870,   27113,  4, }, // Asia/Amman
    {  401, 5428,   33451,  6, }, // Asia/Anadyr
    {  401, 5440,    2213,  5, }, // Asia/Aqtau
    {  401, 5270,    2218,  6, }, // Asia/Aqtobe
    {  401,  964,  124292,  7, }, // Asia/Ashgabat
    {  401, 8587,   50535,  6, }, // Asia/Atyrau
    {  401, 3509,    2237,  6, }, // Asia/Baghdad
    {  401, 5451,    2243,  7, }, // Asia/Bahrain
    {  401, 3555,    2250,  4, }, // Asia/Baku
    {  401, 4368,    2254,  7, }, // Asia/Bangkok
    {  401, 3473,    2261,  7, }, // Asia/Barnaul
    {  401, 3988,   33476,  6, }, // Asia/Beirut
    {  401, 3680,    2274,  6, }, // Asia/Bishkek
    {  401, 5906,   33482,  6, }, // Asia/Brunei
    {  401, 4503,    2286,  4, }, // Asia/Chita
    {  401, 5963,   33489,  9, }, // Asia/Choibalsan
    {  401, 4413,    2299,  7, }, // Asia/Colombo
    {  401, 4442,    2306,  6, }, // Asia/Damascus
    {  401, 1060,     593,  4, }, // Asia/Dhaka
    {  401, 6125,    2317,  4, }, // Asia/Dili
    {  401, 3498,    2321,  5, }, // Asia/Dubai
    {  401, 6092,    2326,  7, }, // Asia/Dushanbe
    {  401, 8599,    2333,  9, }, // Asia/Famagusta
    {  401, 8614,    2342,  4, }, // Asia/Gaza
    {  401, 4697,   33498,  6, }, // Asia/Hebron
    {  401, 1226,   33504,  9, }, // Asia/Ho_Chi_Minh
    {  401, 2576,   33513,  9, }, // Asia/Hong_Kong
    {  401, 4673,    2366,  4, }, // Asia/Hovd
    {  401, 4086,    2370,  7, }, // Asia/Irkutsk
    {  401, 6969,    2377,  7, }, // Asia/Jakarta
    {  401, 6955,   33528,  8, }, // Asia/Jayapura
    {  401, 1257,  124299,  8, }, // Asia/Jerusalem
    {  401, 3462,    2401,  5, }, // Asia/Kabul
    {  401, 4218,    2420,  8, }, // Asia/Kamchatka
    {  401, 4141,    2431,  6, }, // Asia/Karachi
    {  401, 1152,    2437,  8, }, // Asia/Kathmandu
    {  401, 8624,   33545,  7, }, // Asia/Khandyga
    {  401,  992,   33552,  7, }, // Asia/Kolkata
    {  401, 4099,   33559, 11, }, // Asia/Krasnoyarsk
    {  401, 4761,   33570, 12, }, // Asia/Kuala_Lumpur
    {  401, 5893,    2483,  6, }, // Asia/Kuching
    {  401, 5464,   33582,  6, }, // Asia/Kuwait
    {  401, 1178,    2495,  5, }, // Asia/Macau
    {  401, 3919,    2500,  7, }, // Asia/Magadan
    {  401, 1316,    2507,  7, }, // Asia/Makassar
    {  401, 7145,    2514,  6, }, // Asia/Manila
    {  401, 6883,   33593,  6, }, // Asia/Muscat
    {  401, 2446,   33599,  8, }, // Asia/Nicosia
    {  401, 8638,   33607, 11, }, // Asia/Novokuznetsk
    {  401, 4037,    2544, 11, }, // Asia/Novosibirsk
    {  401, 4131,    2555,  4, }, // Asia/Omsk
    {  401, 7108,   33618,  4, }, // Asia/Oral
    {  401, 6924,   33622,  8, }, // Asia/Phnom_Penh
    {  401, 8656,  124307, 10, }, // Asia/Pontianak
    {  401, 4116,   33630,  9, }, // Asia/Pyongyang
    {  401, 5476,    2588,  5, }, // Asia/Qatar
    {  401, 8671,   33639,  8, }, // Asia/Qostanay
    {  401, 4171,   33647,  9, }, // Asia/Qyzylorda
    {  401, 3486,   33656,  5, }, // Asia/Riyadh
    {  401, 4310,  124317,  7, }, // Asia/Sakhalin
    {  401, 7208,    2630,  9, }, // Asia/Samarkand
    {  401, 3116,    2639,  4, }, // Asia/Seoul
    {  401, 1020,   33661,  6, }, // Asia/Shanghai
    {  401, 3137,    2649,  8, }, // Asia/Singapore
    {  401, 4199,   35647, 13, }, // Asia/Srednekolymsk
    {  401, 3100,   33680,  6, }, // Asia/Taipei
    {  401, 4683,    2676,  7, }, // Asia/Tashkent
    {  401, 3817,   33686,  7, }, // Asia/Tbilisi
    {  401, 2638,   33693,  7, }, // Asia/Tehran
    {  401, 1284,   33700,  5, }, // Asia/Thimphu
    {  401, 2687,    2701,  5, }, // Asia/Tokyo
    {  401, 4474,    2706,  5, }, // Asia/Tomsk
    {  401, 1346,   33705, 10, }, // Asia/Ulaanbaatar
    {  401, 1126,    2721,  6, }, // Asia/Urumqi
    {  401, 8685,    2727,  8, }, // Asia/Ust-Nera
    {  401, 6940,  124324,  9, }, // Asia/Vientiane
    {  401, 4612,    2742, 11, }, // Asia/Vladivostok
    {  401, 4730,   33732,  7, }, // Asia/Yakutsk
    {  401, 1202,   27502,  6, }, // Asia/Yangon
    {  401, 3785,    2766, 12, }, // Asia/Yekaterinburg
    {  401, 3649,    2778,  6, }, // Asia/Yerevan
    {  401, 3565,  124333, 15, }, // Atlantic/Azores
    {  401, 5588,  124348,  7, }, // Atlantic/Bermuda
    {  401, 6591,  124355, 16, }, // Atlantic/Canary
    {  401, 3629,   35726, 10, }, // Atlantic/Cape_Verde
    {  401, 1379,  124371, 14, }, // Atlantic/Faroe
    {  401, 8699,    2862,  7, }, // Atlantic/Madeira
    {  401, 2614,   33817,  9, }, // Atlantic/Reykjavik
    {  401, 7235,   33826, 13, }, // Atlantic/South_Georgia
    {  401, 6784,   35781, 11, }, // Atlantic/St_Helena
    {  401, 6607,    2920,  6, }, // Atlantic/Stanley
    {  401, 1654,    2926,  8, }, // Australia/Adelaide
    {  401, 1619,   33849,  8, }, // Australia/Brisbane
    {  401, 1783,   33857, 10, }, // Australia/Broken_Hill
    {  401, 1567,    2952,  6, }, // Australia/Darwin
    {  401, 3539,  124385,  5, }, // Australia/Eucla
    {  401, 1500,    2963,  6, }, // Australia/Hobart
    {  401, 8716,    2969,  8, }, // Australia/Lindeman
    {  401, 1531,   35811,  8, }, // Australia/Lord_Howe
    {  401, 1711,    2985,  7, }, // Australia/Melbourne
    {  401, 1746,    2992,  4, }, // Australia/Perth
    {  401, 1447,   33875,  6, }, // Australia/Sydney
    {  401, 6390,    3002,  9, }, // Europe/Amsterdam
    {  401, 6149,   27734,  6, }, // Europe/Andorra
    {  401, 3522,    3018,  8, }, // Europe/Astrakhan
    {  401, 6577,   33881,  5, }, // Europe/Athens
    {  401, 6285,    3030,  7, }, // Europe/Belgrade
    {  401, 4659,    3037,  6, }, // Europe/Berlin
    {  401, 6437,    3043, 10, }, // Europe/Bratislava
    {  401, 6178,   33893,  6, }, // Europe/Brussels
    {  401, 3830,   33899,  8, }, // Europe/Bucharest
    {  401, 3708,   33907, 10, }, // Europe/Budapest
    {  401, 8751,  124390,  8, }, // Europe/Busingen
    {  401, 2475,  124398,  7, }, // Europe/Chisinau
    {  401, 6238,   33932, 10, }, // Europe/Copenhagen
    {  401, 2260,   33942,  6, }, // Europe/Dublin
    {  401, 6256,    3123,  9, }, // Europe/Gibraltar
    {  401, 8767,   33948,  6, }, // Europe/Guernsey
    {  401, 6561,    3138,  8, }, // Europe/Helsinki
    {  401, 8783,  124405, 10, }, // Europe/Isle_of_Man
    {  401, 1097,   33964,  8, }, // Europe/Istanbul
    {  401, 8802,  124415,  5, }, // Europe/Jersey
    {  401, 3881,    3167, 11, }, // Europe/Kaliningrad
    {  401, 8924,    3178,  5, }, // Europe/Kirov
    {  401, 2419,    3183,  4, }, // Europe/Kyiv
    {  401, 3078,   33988,  7, }, // Europe/Lisbon
    {  401, 6455,   33995,  7, }, // Europe/Ljubljana
    {  401, 2393,    3204,  6, }, // Europe/London
    {  401, 6314,   34002, 10, }, // Europe/Luxembourg
    {  401, 4794,    3221,  6, }, // Europe/Madrid
    {  401, 6346,    3227,  5, }, // Europe/Malta
    {  401, 6531,  124420, 11, }, // Europe/Mariehamn
    {  401, 3595,    3241,  5, }, // Europe/Minsk
    {  401, 6359,    3246,  6, }, // Europe/Monaco
    {  401, 3443,    3252,  6, }, // Europe/Moscow
    {  401, 6407,    3258,  4, }, // Europe/Oslo
    {  401, 4186,   34020,  5, }, // Europe/Paris
    {  401, 6373,    3267,  9, }, // Europe/Podgorica
    {  401, 6224,    3276,  5, }, // Europe/Prague
    {  401, 8816,    3281,  4, }, // Europe/Riga
    {  401, 6273,    3285,  3, }, // Europe/Rome
    {  401, 4233,    3288,  6, }, // Europe/Samara
    {  401, 6419,   34025, 10, }, // Europe/San_Marino
    {  401, 6194,    3304,  7, }, // Europe/Sarajevo
    {  401, 4353,    3311,  7, }, // Europe/Saratov
    {  401, 8828,    3318, 10, }, // Europe/Simferopol
    {  401, 6332,  124431,  6, }, // Europe/Skopje
    {  401, 6548,   34059,  6, }, // Europe/Sofia
    {  401, 6472,   34065,  8, }, // Europe/Stockholm
    {  401, 8846,   34073,  5, }, // Europe/Tallinn
    {  401, 6135,    3352,  6, }, // Europe/Tirane
    {  401, 8861,  124437,  9, }, // Europe/Ulyanovsk
    {  401, 6301,    3367,  5, }, // Europe/Vaduz
    {  401, 6516,    3372,  7, }, // Europe/Vatican
    {  401, 6164,   35920,  5, }, // Europe/Vienna
    {  401, 8878,  124446,  6, }, // Europe/Vilnius
    {  401, 4629,    3390,  9, }, // Europe/Volgograd
    {  401, 3055,    3399,  7, }, // Europe/Warsaw
    {  401, 6210,    3406,  6, }, // Europe/Zagreb
    {  401, 6489,   34100,  5, }, // Europe/Zurich
    {  401, 4980,   34105, 12, }, // Indian/Antananarivo
    {  401, 6910,    3430,  5, }, // Indian/Chagos
    {  401, 5979,  124452, 14, }, // Indian/Christmas
    {  401, 5996,  124466, 16, }, // Indian/Cocos
    {  401, 4931,   50972,  6, }, // Indian/Comoro
    {  401, 6624,    3469,  8, }, // Indian/Kerguelen
    {  401, 7223,   34128,  4, }, // Indian/Mahe
    {  401, 7013,   34132,  7, }, // Indian/Maldives
    {  401, 3971,  124482,  9, }, // Indian/Mauritius
    {  401, 5000,   34149,  5, }, // Indian/Mayotte
    {  401, 7174,  124491,  8, }, // Indian/Reunion
    {  401, 4324,   34161,  5, }, // Pacific/Apia
    {  401,  932,    3511,  6, }, // Pacific/Auckland
    {  401, 3608,  124499,  8, }, // Pacific/Bougainville
    {  401, 2863,   36001,  5, }, // Pacific/Chatham
    {  401, 3022,  124507,  3, }, // Pacific/Chuuk
    {  401, 2201,  124510, 18, }, // Pacific/Easter
    {  401, 7323,    3545,  5, }, // Pacific/Efate
    {  401, 7290,    3550,  7, }, // Pacific/Fakaofo
    {  401, 3804,    3557,  4, }, // Pacific/Fiji
    {  401, 7306,    3561,  8, }, // Pacific/Funafuti
    {  401, 6641,    3569,  9, }, // Pacific/Galapagos
    {  401, 6659,   34192,  7, }, // Pacific/Gambier
    {  401, 3724,   34199, 11, }, // Pacific/Guadalcanal
    {  401, 5935,  124528,  4, }, // Pacific/Guam
    {  401, 2929,   34210,  8, }, // Pacific/Honolulu
    {  401, 2897,   34218,  9, }, // Pacific/Kanton
    {  401, 3900,    3625, 10, }, // Pacific/Kiritimati
    {  401, 6982,    3635,  6, }, // Pacific/Kosrae
    {  401, 2708,  124532,  9, }, // Pacific/Kwajalein
    {  401, 4779,    3650,  6, }, // Pacific/Majuro
    {  401, 3953,  124541, 16, }, // Pacific/Marquesas
    {  401, 8909,   34248,  6, }, // Pacific/Midway
    {  401, 7047,    3689,  5, }, // Pacific/Nauru
    {  401, 7095,    3694,  4, }, // Pacific/Niue
    {  401, 4070,    3698,  7, }, // Pacific/Norfolk
    {  401, 7061,    3705,  5, }, // Pacific/Noumea
    {  401, 2991,   34254,  9, }, // Pacific/Pago_Pago
    {  401, 7118,    3719,  5, }, // Pacific/Palau
    {  401, 7157,  124557, 18, }, // Pacific/Pitcairn
    {  401, 2961,  124575,  6, }, // Pacific/Pohnpei
    {  401, 4709,   36098, 11, }, // Pacific/Port_Moresby
    {  401, 6009,    3748,  9, }, // Pacific/Rarotonga
    {  401, 5948,   34280,  6, }, // Pacific/Saipan
    {  401, 7275,   34286,  6, }, // Pacific/Tahiti
    {  401, 6675,    3768,  6, }, // Pacific/Tarawa
    {  401, 4485,    3774,  9, }, // Pacific/Tongatapu
    {  401, 7355,   34292,  4, }, // Pacific/Wake
    {  401, 7368,   34296,  5, }, // Pacific/Wallis
    {  402,   14,       0,  6, }, // Africa/Asmara Machame/Latin/Tanzania
    {  402, 4337,       6,  8, }, // Africa/Sao_Tome
    {  402, 4154,      14,  8, }, // America/Asuncion
    {  402,  237,      22,  8, }, // America/Atikokan
    {  402, 7586,      30, 17, }, // America/Bahia_Banderas
    {  402, 3770,      47,  6, }, // America/Cancun
    {  402, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  402, 5640,      66,  7, }, // America/Curacao
    {  402,  481,      73, 13, }, // America/Indiana/Knox
    {  402, 7924,      86, 16, }, // America/Indiana/Marengo
    {  402, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  402, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  402, 8001,     139, 14, }, // America/Indiana/Vevay
    {  402, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  402, 8049,     171, 16, }, // America/Indiana/Winamac
    {  402, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  402, 5849,     207, 22, }, // America/Lower_Princes
    {  402, 8183,     229,  6, }, // America/Merida
    {  402, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  402, 8292,     255, 20, }, // America/North_Dakota/Center
    {  402, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  402,  393,     298,  4, }, // America/Nuuk
    {  402, 6862,     302, 16, }, // America/Scoresbysund
    {  402, 8471,     318, 14, }, // America/St_Barthelemy
    {  402, 2045,     332, 10, }, // America/St_Johns
    {  402, 5780,     342,  9, }, // America/St_Kitts
    {  402, 5797,     351,  9, }, // America/St_Lucia
    {  402,  855,     360, 10, }, // America/St_Thomas
    {  402, 5830,     370, 11, }, // America/St_Vincent
    {  402, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  402, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  402, 1152,     413,  9, }, // Asia/Kathmandu
    {  402,  992,     422,  7, }, // Asia/Kolkata
    {  402, 1178,     429,  5, }, // Asia/Macau
    {  402, 1202,     442,  6, }, // Asia/Yangon
    {  402, 1379,     448,  5, }, // Atlantic/Faroe
    {  402, 6784,     453, 10, }, // Atlantic/St_Helena
    {  402, 2419,     463,  4, }, // Europe/Kyiv
    {  402, 7174,     467,  7, }, // Indian/Reunion
    {  402, 3022,     474,  5, }, // Pacific/Chuuk
    {  402, 2961,     479,  7, }, // Pacific/Pohnpei
    {  403, 6751,  124581,  6, }, // Africa/Abidjan Maithili/Devanagari/India
    {  403, 6723,  124587,  4, }, // Africa/Accra
    {  403, 4961,  124591, 10, }, // Africa/Addis_Ababa
    {  403, 7383,   86776,  9, }, // Africa/Algiers
    {  403,   14,  124601,  6, }, // Africa/Asmara
    {  403,   44,  124607,  5, }, // Africa/Bamako
    {  403, 5159,  124612,  5, }, // Africa/Bangui
    {  403, 6709,  124617,  5, }, // Africa/Banjul
    {  403, 7398,   86810,  5, }, // Africa/Bissau
    {  403, 4859,  124622,  9, }, // Africa/Blantyre
    {  403, 5189,  124631,  9, }, // Africa/Brazzaville
    {  403, 4824,  109007,  9, }, // Africa/Bujumbura
    {  403, 2242,  124640,  6, }, // Africa/Cairo
    {  403, 4019,   28524, 11, }, // Africa/Casablanca
    {  403, 7412,  124646,  5, }, // Africa/Ceuta
    {  403, 6736,   86854,  8, }, // Africa/Conakry
    {  403, 6803,   86862,  4, }, // Africa/Dakar
    {  403, 5032,  124651,  9, }, // Africa/Dar_es_Salaam
    {  403, 4945,   86877,  6, }, // Africa/Djibouti
    {  403, 5145,  124660,  5, }, // Africa/Douala
    {  403, 5068,  124665,  7, }, // Africa/El_Aaiun
    {  403, 6816,   28581,  8, }, // Africa/Freetown
    {  403, 4808,  124672,  7, }, // Africa/Gaborone
    {  403, 4917,   86912,  5, }, // Africa/Harare
    {  403, 4381,  124679, 11, }, // Africa/Johannesburg
    {  403, 4401,   28613,  4, }, // Africa/Juba
    {  403, 5053,  124690,  6, }, // Africa/Kampala
    {  403, 4426,  124696,  6, }, // Africa/Khartoum
    {  403, 4875,   86933,  6, }, // Africa/Kigali
    {  403, 5208,   86939,  7, }, // Africa/Kinshasa
    {  403, 4646,   86946,  5, }, // Africa/Lagos
    {  403, 5238,   86951,  9, }, // Africa/Libreville
    {  403, 6832,   86961,  3, }, // Africa/Lome
    {  403, 5113,   86964,  6, }, // Africa/Luanda
    {  403, 4841,   86970,  9, }, // Africa/Lubumbashi
    {  403, 4903,   28680,  6, }, // Africa/Lusaka
    {  403, 5224,   86979,  5, }, // Africa/Malabo
    {  403, 4889,  124702,  5, }, // Africa/Maputo
    {  403, 5099,  124707,  5, }, // Africa/Maseru
    {  403, 5084,  124712,  9, }, // Africa/Mbabane
    {  403, 5015,  109085,  8, }, // Africa/Mogadishu
    {  403, 6997,   87005,  9, }, // Africa/Monrovia
    {  403, 3755,   87014,  6, }, // Africa/Nairobi
    {  403, 5173,  124721,  8, }, // Africa/Ndjamena
    {  403, 5256,  124729,  7, }, // Africa/Niamey
    {  403, 6766,  124736,  6, }, // Africa/Nouakchott
    {  403, 6690,  124742,  7, }, // Africa/Ouagadougou
    {  403, 5127,   87050, 11, }, // Africa/Porto-Novo
    {  403, 4337,   87061,  7, }, // Africa/Sao_Tome
    {  403, 2732,   87068,  8, }, // Africa/Tripoli
    {  403, 6503,   28787,  7, }, // Africa/Tunis
    {  403, 4054,  124749,  7, }, // Africa/Windhoek
    {  403,  132,  124756,  3, }, // America/Adak
    {  403, 3183,  124759,  7, }, // America/Anchorage
    {  403, 5524,   87086,  8, }, // America/Anguilla
    {  403, 5541,  124766,  7, }, // America/Antigua
    {  403, 4456,  124773,  8, }, // America/Araguaina
    {  403,  166,  109157, 12, }, // America/Argentina/Buenos_Aires
    {  403,   91,   28849, 10, }, // America/Argentina/Catamarca
    {  403,  270,   87121,  8, }, // America/Argentina/Cordoba
    {  403,  441,  124781,  5, }, // America/Argentina/Jujuy
    {  403, 7425,  109190,  9, }, // America/Argentina/La_Rioja
    {  403,  565,  124786,  7, }, // America/Argentina/Mendoza
    {  403, 7452,  124793, 10, }, // America/Argentina/Rio_Gallegos
    {  403, 7483,  124803,  5, }, // America/Argentina/Salta
    {  403, 7507,  124808,  7, }, // America/Argentina/San_Juan
    {  403, 5497,  124815,  7, }, // America/Argentina/San_Luis
    {  403, 7534,   28919,  7, }, // America/Argentina/Tucuman
    {  403, 7560,  124822,  7, }, // America/Argentina/Ushuaia
    {  403, 5557,   87191,  5, }, // America/Aruba
    {  403, 4154,  124829,  8, }, // America/Asuncion
    {  403,  237,  124837,  7, }, // America/Atikokan
    {  403, 3581,   87212,  5, }, // America/Bahia
    {  403, 7586,  124844, 13, }, // America/Bahia_Banderas
    {  403, 5571,  124857,  7, }, // America/Barbados
    {  403, 7609,  124864,  5, }, // America/Belem
    {  403, 5309,   28990,  5, }, // America/Belize
    {  403, 7623,  124869, 12, }, // America/Blanc-Sablon
    {  403, 7644,  124881, 10, }, // America/Boa_Vista
    {  403, 4263,   87269,  6, }, // America/Bogota
    {  403, 7662,  124891,  5, }, // America/Boise
    {  403, 7676,  124896, 12, }, // America/Cambridge_Bay
    {  403, 7698,  124908, 14, }, // America/Campo_Grande
    {  403, 3770,  109322,  6, }, // America/Cancun
    {  403, 4596,   29061,  7, }, // America/Caracas
    {  403, 4247,   29068,  5, }, // America/Cayenne
    {  403, 5398,   29073,  5, }, // America/Cayman
    {  403, 3251,   87320,  6, }, // America/Chicago
    {  403, 7719,   87326,  8, }, // America/Chihuahua
    {  403, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  403, 5324,   87350, 11, }, // America/Costa_Rica
    {  403, 7759,   29101,  8, }, // America/Creston
    {  403, 3693,  124922,  6, }, // America/Cuiaba
    {  403, 5640,  124928,  9, }, // America/Curacao
    {  403, 7775,  124937,  7, }, // America/Danmarkshavn
    {  403, 7796,  124944,  5, }, // America/Dawson
    {  403, 7811,  124949, 11, }, // America/Dawson_Creek
    {  403,  805,  124960,  5, }, // America/Denver
    {  403, 3351,  124965,  8, }, // America/Detroit
    {  403, 5656,   87413,  8, }, // America/Dominica
    {  403,  893,  124973,  8, }, // America/Edmonton
    {  403, 7832,  124981,  7, }, // America/Eirunepe
    {  403, 5343,   87427, 11, }, // America/El_Salvador
    {  403, 7849,  124988, 11, }, // America/Fort_Nelson
    {  403, 7869,  124999, 10, }, // America/Fortaleza
    {  403, 7887,   29219,  8, }, // America/Glace_Bay
    {  403, 6844,   29228,  6, }, // America/Goose_Bay
    {  403, 4514,   87483, 11, }, // America/Grand_Turk
    {  403, 5687,   29248,  8, }, // America/Grenada
    {  403, 5703,  125009,  9, }, // America/Guadeloupe
    {  403, 3662,   29265, 10, }, // America/Guatemala
    {  403, 4743,  125018, 10, }, // America/Guayaquil
    {  403, 6895,   29283,  6, }, // America/Guyana
    {  403, 1923,   29289,  9, }, // America/Halifax
    {  403, 2221,   87527,  5, }, // America/Havana
    {  403, 7905,  125028,  8, }, // America/Hermosillo
    {  403,  348,   87543, 13, }, // America/Indiana/Indianapolis
    {  403,  481,  125036, 15, }, // America/Indiana/Knox
    {  403, 7924,  125051, 17, }, // America/Indiana/Marengo
    {  403, 7948,   87588, 20, }, // America/Indiana/Petersburg
    {  403, 7975,   87608, 18, }, // America/Indiana/Tell_City
    {  403, 8001,  125068, 14, }, // America/Indiana/Vevay
    {  403, 8023,  125082, 17, }, // America/Indiana/Vincennes
    {  403, 8049,  125099, 17, }, // America/Indiana/Winamac
    {  403, 8073,  125116,  6, }, // America/Inuvik
    {  403,  660,  125122,  7, }, // America/Iqaluit
    {  403, 2665,   87682,  5, }, // America/Jamaica
    {  403, 5282,  125129,  5, }, // America/Juneau
    {  403,  521,   87694,  8, }, // America/Kentucky/Louisville
    {  403, 8088,  125134, 18, }, // America/Kentucky/Monticello
    {  403, 5621,  125152, 10, }, // America/Kralendijk
    {  403, 4278,   29517,  6, }, // America/La_Paz
    {  403, 7132,   29523,  4, }, // America/Lima
    {  403, 3390,  125162, 11, }, // America/Los_Angeles
    {  403, 5849,  125173, 20, }, // America/Lower_Princes
    {  403, 8116,  125193,  5, }, // America/Maceio
    {  403, 8131,   29568,  6, }, // America/Managua
    {  403, 1892,   87783,  4, }, // America/Manaus
    {  403, 5814,  125198,  7, }, // America/Marigot
    {  403, 5722,   87794,  9, }, // America/Martinique
    {  403, 8147,   87803,  9, }, // America/Matamoros
    {  403, 2779,  125205,  7, }, // America/Mazatlan
    {  403, 8165,  125212,  8, }, // America/Menominee
    {  403, 8183,  125220,  6, }, // America/Merida
    {  403, 8198,  125226, 10, }, // America/Metlakatla
    {  403, 2811,  125236, 13, }, // America/Mexico_City
    {  403, 4293,  125249,  9, }, // America/Miquelon
    {  403, 8217,  125258,  8, }, // America/Moncton
    {  403, 8233,  125266,  8, }, // America/Monterrey
    {  403, 4000,  125274, 11, }, // America/Montevideo
    {  403, 5741,   87887,  9, }, // America/Montserrat
    {  403, 5383,  125285,  4, }, // America/Nassau
    {  403, 3294,  125289,  9, }, // America/New_York
    {  403, 8251,   87823,  3, }, // America/Nome
    {  403, 1834,   87910,  8, }, // America/Noronha
    {  403, 8264,  125298, 19, }, // America/North_Dakota/Beulah
    {  403, 8292,  125317, 18, }, // America/North_Dakota/Center
    {  403, 8320,  125335, 22, }, // America/North_Dakota/New_Salem
    {  403,  393,   87981,  3, }, // America/Nuuk
    {  403, 8351,  109895,  7, }, // America/Ojinaga
    {  403, 5413,   87993,  5, }, // America/Panama
    {  403, 6106,   87998, 10, }, // America/Paramaribo
    {  403, 3224,  109911,  7, }, // America/Phoenix
    {  403, 3847,  125357, 15, }, // America/Port-au-Prince
    {  403, 5871,  125372, 14, }, // America/Port_of_Spain
    {  403, 8367,  125386, 13, }, // America/Porto_Velho
    {  403, 5760,   88058, 11, }, // America/Puerto_Rico
    {  403, 3932,  125399, 12, }, // America/Punta_Arenas
    {  403, 8387,   88081, 12, }, // America/Rankin_Inlet
    {  403, 8408,  125411,  5, }, // America/Recife
    {  403, 1979,  125416,  6, }, // America/Regina
    {  403, 8423,  125422,  7, }, // America/Resolute
    {  403,  695,  125429, 12, }, // America/Rio_Branco
    {  403, 8440,  110001,  8, }, // America/Santarem
    {  403, 2165,   88129,  9, }, // America/Santiago
    {  403, 6044,  125441, 13, }, // America/Santo_Domingo
    {  403, 1862,   88151,  9, }, // America/Sao_Paulo
    {  403, 6862,  125454, 13, }, // America/Scoresbysund
    {  403, 8457,  125467,  5, }, // America/Sitka
    {  403, 8471,  125472, 13, }, // America/St_Barthelemy
    {  403, 2045,  125485, 10, }, // America/St_Johns
    {  403, 5780,   88203, 10, }, // America/St_Kitts
    {  403, 5797,  125495, 11, }, // America/St_Lucia
    {  403,  855,  125506,  9, }, // America/St_Thomas
    {  403, 5830,   88233, 12, }, // America/St_Vincent
    {  403, 8493,  125515, 13, }, // America/Swift_Current
    {  403, 5363,   30064, 12, }, // America/Tegucigalpa
    {  403, 5673,  125528,  4, }, // America/Thule
    {  403,  313,   88275,  7, }, // America/Tijuana
    {  403,  608,   88282,  6, }, // America/Toronto
    {  403, 5605,   88288,  8, }, // America/Tortola
    {  403, 2077,  125532,  7, }, // America/Vancouver
    {  403, 2128,  125539,  9, }, // America/Whitehorse
    {  403,  734,  125548,  7, }, // America/Winnipeg
    {  403, 8515,  125555,  6, }, // America/Yakutat
    {  403, 5918,   30132,  4, }, // Antarctica/Casey
    {  403, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  403, 8531,  125561,  8, }, // Antarctica/Macquarie
    {  403, 1413,  125569, 11, }, // Arctic/Longyearbyen
    {  403, 5487,  125580,  3, }, // Asia/Aden
    {  403, 5297,  125583,  7, }, // Asia/Almaty
    {  403, 3870,   88424,  6, }, // Asia/Amman
    {  403, 5428,  125590,  6, }, // Asia/Anadyr
    {  403, 5440,   30237,  6, }, // Asia/Aqtau
    {  403, 5270,   88436,  6, }, // Asia/Aqtobe
    {  403,  964,   30250,  8, }, // Asia/Ashgabat
    {  403, 8587,  125596,  6, }, // Asia/Atyrau
    {  403, 3509,   88447,  5, }, // Asia/Baghdad
    {  403, 5451,   88452,  5, }, // Asia/Bahrain
    {  403, 3555,   29113,  4, }, // Asia/Baku
    {  403, 4368,  125602,  6, }, // Asia/Bangkok
    {  403, 3473,   88463,  6, }, // Asia/Barnaul
    {  403, 3988,  110263,  5, }, // Asia/Beirut
    {  403, 3680,   30295,  7, }, // Asia/Bishkek
    {  403, 5906,   88474,  7, }, // Asia/Brunei
    {  403, 4503,  125608,  4, }, // Asia/Chita
    {  403, 5963,  125612,  8, }, // Asia/Choibalsan
    {  403, 4413,  125620,  7, }, // Asia/Colombo
    {  403, 4442,  125627,  6, }, // Asia/Damascus
    {  403, 1060,   88508,  4, }, // Asia/Dhaka
    {  403, 6125,   30340,  4, }, // Asia/Dili
    {  403, 3498,   88516,  4, }, // Asia/Dubai
    {  403, 6092,   88520,  7, }, // Asia/Dushanbe
    {  403, 8599,   30357, 10, }, // Asia/Famagusta
    {  403, 8614,   30367,  4, }, // Asia/Gaza
    {  403, 4697,  125633,  7, }, // Asia/Hebron
    {  403, 1226,   88550, 16, }, // Asia/Ho_Chi_Minh
    {  403, 2576,  125640,  9, }, // Asia/Hong_Kong
    {  403, 4673,  125649,  5, }, // Asia/Hovd
    {  403, 4086,  125654,  7, }, // Asia/Irkutsk
    {  403, 6969,   88590,  7, }, // Asia/Jakarta
    {  403, 6955,   30411,  7, }, // Asia/Jayapura
    {  403, 1257,  125661,  8, }, // Asia/Jerusalem
    {  403, 3462,   88603,  5, }, // Asia/Kabul
    {  403, 4218,  125669,  6, }, // Asia/Kamchatka
    {  403, 4141,   30440,  5, }, // Asia/Karachi
    {  403, 1152,   88615,  8, }, // Asia/Kathmandu
    {  403, 8624,  125675,  8, }, // Asia/Khandyga
    {  403,  992,   88630,  7, }, // Asia/Kolkata
    {  403, 4099,  125683, 15, }, // Asia/Krasnoyarsk
    {  403, 4761,  125698, 11, }, // Asia/Kuala_Lumpur
    {  403, 5893,  125709,  6, }, // Asia/Kuching
    {  403, 5464,   30502,  5, }, // Asia/Kuwait
    {  403, 1178,   88662,  4, }, // Asia/Macau
    {  403, 3919,  110390,  6, }, // Asia/Magadan
    {  403, 1316,  125715,  6, }, // Asia/Makassar
    {  403, 7145,   30528,  5, }, // Asia/Manila
    {  403, 6883,   88672,  5, }, // Asia/Muscat
    {  403, 2446,  125721,  8, }, // Asia/Nicosia
    {  403, 8638,  125729, 12, }, // Asia/Novokuznetsk
    {  403, 4037,  125741, 12, }, // Asia/Novosibirsk
    {  403, 4131,  125753,  5, }, // Asia/Omsk
    {  403, 7108,  125758,  4, }, // Asia/Oral
    {  403, 6924,  110425,  9, }, // Asia/Phnom_Penh
    {  403, 8656,  125762,  9, }, // Asia/Pontianak
    {  403, 4116,   88741, 10, }, // Asia/Pyongyang
    {  403, 5476,   88751,  3, }, // Asia/Qatar
    {  403, 8671,  125771,  8, }, // Asia/Qostanay
    {  403, 4171,  125779, 10, }, // Asia/Qyzylorda
    {  403, 3486,   88773,  5, }, // Asia/Riyadh
    {  403, 4310,  125789,  6, }, // Asia/Sakhalin
    {  403, 7208,   88784,  6, }, // Asia/Samarkand
    {  403, 3116,  125795,  5, }, // Asia/Seoul
    {  403, 1020,   30652,  6, }, // Asia/Shanghai
    {  403, 3137,   30658,  8, }, // Asia/Singapore
    {  403, 4199,  125800, 16, }, // Asia/Srednekolymsk
    {  403, 3100,  125816,  6, }, // Asia/Taipei
    {  403, 4683,   88821,  6, }, // Asia/Tashkent
    {  403, 3817,  125822,  7, }, // Asia/Tbilisi
    {  403, 2638,   30704,  6, }, // Asia/Tehran
    {  403, 1284,  125829,  6, }, // Asia/Thimphu
    {  403, 2687,   88839,  6, }, // Asia/Tokyo
    {  403, 4474,  125835,  5, }, // Asia/Tomsk
    {  403, 1346,  125840,  7, }, // Asia/Ulaanbaatar
    {  403, 1126,  125847,  6, }, // Asia/Urumqi
    {  403, 8685,  125853,  9, }, // Asia/Ust-Nera
    {  403, 6940,  125862,  9, }, // Asia/Vientiane
    {  403, 4612,  125871, 13, }, // Asia/Vladivostok
    {  403, 4730,  125884,  9, }, // Asia/Yakutsk
    {  403, 1202,  125893,  6, }, // Asia/Yangon
    {  403, 3785,  125899, 13, }, // Asia/Yekaterinburg
    {  403, 3649,  125912,  6, }, // Asia/Yerevan
    {  403, 3565,  125918,  6, }, // Atlantic/Azores
    {  403, 5588,   88926,  6, }, // Atlantic/Bermuda
    {  403, 6591,   30823,  6, }, // Atlantic/Canary
    {  403, 3629,  125924,  8, }, // Atlantic/Cape_Verde
    {  403, 1379,  125932,  4, }, // Atlantic/Faroe
    {  403, 8699,  125936,  5, }, // Atlantic/Madeira
    {  403, 2614,  125941,  8, }, // Atlantic/Reykjavik
    {  403, 7235,  125949, 15, }, // Atlantic/South_Georgia
    {  403, 6784,   88980, 11, }, // Atlantic/St_Helena
    {  403, 6607,   30881,  7, }, // Atlantic/Stanley
    {  403, 1654,   88998,  6, }, // Australia/Adelaide
    {  403, 1619,  125964,  8, }, // Australia/Brisbane
    {  403, 1783,  125972, 11, }, // Australia/Broken_Hill
    {  403, 1567,  125983,  7, }, // Australia/Darwin
    {  403, 3539,  125990,  6, }, // Australia/Eucla
    {  403, 1500,  125996,  6, }, // Australia/Hobart
    {  403, 8716,  126002,  7, }, // Australia/Lindeman
    {  403, 1531,  126009, 10, }, // Australia/Lord_Howe
    {  403, 1711,   89054,  8, }, // Australia/Melbourne
    {  403, 1746,   89062,  4, }, // Australia/Perth
    {  403, 1447,   30962,  5, }, // Australia/Sydney
    {  403, 6390,  126019,  9, }, // Europe/Amsterdam
    {  403, 6149,  126028,  6, }, // Europe/Andorra
    {  403, 3522,  126034, 10, }, // Europe/Astrakhan
    {  403, 6577,   89092,  5, }, // Europe/Athens
    {  403, 6285,   31000,  8, }, // Europe/Belgrade
    {  403, 4659,   89097,  6, }, // Europe/Berlin
    {  403, 6437,  126044, 11, }, // Europe/Bratislava
    {  403, 6178,  126055,  8, }, // Europe/Brussels
    {  403, 3830,  110729,  9, }, // Europe/Bucharest
    {  403, 3708,   89134,  9, }, // Europe/Budapest
    {  403, 8751,  126063,  7, }, // Europe/Busingen
    {  403, 2475,  126070,  7, }, // Europe/Chisinau
    {  403, 6238,  126077, 10, }, // Europe/Copenhagen
    {  403, 2260,   89169,  5, }, // Europe/Dublin
    {  403, 6256,   89174, 10, }, // Europe/Gibraltar
    {  403, 8767,  126087,  6, }, // Europe/Guernsey
    {  403, 6561,  126093,  8, }, // Europe/Helsinki
    {  403, 8783,  126101, 11, }, // Europe/Isle_of_Man
    {  403, 1097,  126112, 10, }, // Europe/Istanbul
    {  403, 8802,   89218,  5, }, // Europe/Jersey
    {  403, 3881,  126122, 11, }, // Europe/Kaliningrad
    {  403, 8924,   89235,  5, }, // Europe/Kirov
    {  403, 2419,   89240,  3, }, // Europe/Kyiv
    {  403, 3078,  126133,  5, }, // Europe/Lisbon
    {  403, 6455,  126138,  9, }, // Europe/Ljubljana
    {  403, 2393,   89254,  4, }, // Europe/London
    {  403, 6314,  126147,  9, }, // Europe/Luxembourg
    {  403, 4794,  126156,  7, }, // Europe/Madrid
    {  403, 6346,  126163,  5, }, // Europe/Malta
    {  403, 6531,  126168,  8, }, // Europe/Mariehamn
    {  403, 3595,  126176,  6, }, // Europe/Minsk
    {  403, 6359,  126182,  6, }, // Europe/Monaco
    {  403, 3443,  126188,  6, }, // Europe/Moscow
    {  403, 6407,   89302,  5, }, // Europe/Oslo
    {  403, 4186,   31234,  5, }, // Europe/Paris
    {  403, 6373,  126194,  9, }, // Europe/Podgorica
    {  403, 6224,   89317,  5, }, // Europe/Prague
    {  403, 8816,   31252,  4, }, // Europe/Riga
    {  403, 6273,   88949,  3, }, // Europe/Rome
    {  403, 4233,   89326,  5, }, // Europe/Samara
    {  403, 6419,  126203,  9, }, // Europe/San_Marino
    {  403, 6194,  126212,  7, }, // Europe/Sarajevo
    {  403, 4353,  126219,  7, }, // Europe/Saratov
    {  403, 8828,  126226, 11, }, // Europe/Simferopol
    {  403, 6332,  126237,  7, }, // Europe/Skopje
    {  403, 6548,  110891,  6, }, // Europe/Sofia
    {  403, 6472,  126244,  8, }, // Europe/Stockholm
    {  403, 8846,  126252,  5, }, // Europe/Tallinn
    {  403, 6135,  126257,  6, }, // Europe/Tirane
    {  403, 8861,  126263, 12, }, // Europe/Ulyanovsk
    {  403, 6301,  126275,  5, }, // Europe/Vaduz
    {  403, 6516,  126280,  6, }, // Europe/Vatican
    {  403, 6164,  126286,  5, }, // Europe/Vienna
    {  403, 8878,  126291,  7, }, // Europe/Vilnius
    {  403, 4629,  126298, 10, }, // Europe/Volgograd
    {  403, 3055,  126308,  5, }, // Europe/Warsaw
    {  403, 6210,  126313,  5, }, // Europe/Zagreb
    {  403, 6489,  126318,  7, }, // Europe/Zurich
    {  403, 4980,  126325, 12, }, // Indian/Antananarivo
    {  403, 6910,  126337,  4, }, // Indian/Chagos
    {  403, 5979,   89488,  7, }, // Indian/Christmas
    {  403, 5996,   89495,  5, }, // Indian/Cocos
    {  403, 4931,   89500,  6, }, // Indian/Comoro
    {  403, 6624,  126341,  9, }, // Indian/Kerguelen
    {  403, 7223,   31451,  4, }, // Indian/Mahe
    {  403, 7013,   89513,  6, }, // Indian/Maldives
    {  403, 3971,  126350,  6, }, // Indian/Mauritius
    {  403, 5000,  126356,  5, }, // Indian/Mayotte
    {  403, 7174,  126361,  8, }, // Indian/Reunion
    {  403,  932,  126369,  6, }, // Pacific/Auckland
    {  403, 2863,  126375,  4, }, // Pacific/Chatham
    {  403, 3022,     474,  5, }, // Pacific/Chuuk
    {  403, 2201,   89566,  5, }, // Pacific/Easter
    {  403, 6641,  126379,  9, }, // Pacific/Galapagos
    {  403, 4070,  126388,  6, }, // Pacific/Norfolk
    {  403, 2961,     479,  7, }, // Pacific/Pohnpei
    {  404,   14,       0,  6, }, // Africa/Asmara Makhuwa-Meetto/Latin/Mozambique
    {  404, 4337,       6,  8, }, // Africa/Sao_Tome
    {  404, 4154,      14,  8, }, // America/Asuncion
    {  404,  237,      22,  8, }, // America/Atikokan
    {  404, 7586,      30, 17, }, // America/Bahia_Banderas
    {  404, 3770,      47,  6, }, // America/Cancun
    {  404, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  404, 5640,      66,  7, }, // America/Curacao
    {  404,  481,      73, 13, }, // America/Indiana/Knox
    {  404, 7924,      86, 16, }, // America/Indiana/Marengo
    {  404, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  404, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  404, 8001,     139, 14, }, // America/Indiana/Vevay
    {  404, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  404, 8049,     171, 16, }, // America/Indiana/Winamac
    {  404, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  404, 5849,     207, 22, }, // America/Lower_Princes
    {  404, 8183,     229,  6, }, // America/Merida
    {  404, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  404, 8292,     255, 20, }, // America/North_Dakota/Center
    {  404, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  404,  393,     298,  4, }, // America/Nuuk
    {  404, 6862,     302, 16, }, // America/Scoresbysund
    {  404, 8471,     318, 14, }, // America/St_Barthelemy
    {  404, 2045,     332, 10, }, // America/St_Johns
    {  404, 5780,     342,  9, }, // America/St_Kitts
    {  404, 5797,     351,  9, }, // America/St_Lucia
    {  404,  855,     360, 10, }, // America/St_Thomas
    {  404, 5830,     370, 11, }, // America/St_Vincent
    {  404, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  404, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  404, 1152,     413,  9, }, // Asia/Kathmandu
    {  404,  992,     422,  7, }, // Asia/Kolkata
    {  404, 1178,     429,  5, }, // Asia/Macau
    {  404, 1202,     442,  6, }, // Asia/Yangon
    {  404, 1379,     448,  5, }, // Atlantic/Faroe
    {  404, 6784,     453, 10, }, // Atlantic/St_Helena
    {  404, 2419,     463,  4, }, // Europe/Kyiv
    {  404, 7174,     467,  7, }, // Indian/Reunion
    {  404, 3022,     474,  5, }, // Pacific/Chuuk
    {  404, 2961,     479,  7, }, // Pacific/Pohnpei
    {  405,   14,       0,  6, }, // Africa/Asmara Makonde/Latin/Tanzania
    {  405, 4337,       6,  8, }, // Africa/Sao_Tome
    {  405, 4154,      14,  8, }, // America/Asuncion
    {  405,  237,      22,  8, }, // America/Atikokan
    {  405, 7586,      30, 17, }, // America/Bahia_Banderas
    {  405, 3770,      47,  6, }, // America/Cancun
    {  405, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  405, 5640,      66,  7, }, // America/Curacao
    {  405,  481,      73, 13, }, // America/Indiana/Knox
    {  405, 7924,      86, 16, }, // America/Indiana/Marengo
    {  405, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  405, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  405, 8001,     139, 14, }, // America/Indiana/Vevay
    {  405, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  405, 8049,     171, 16, }, // America/Indiana/Winamac
    {  405, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  405, 5849,     207, 22, }, // America/Lower_Princes
    {  405, 8183,     229,  6, }, // America/Merida
    {  405, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  405, 8292,     255, 20, }, // America/North_Dakota/Center
    {  405, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  405,  393,     298,  4, }, // America/Nuuk
    {  405, 6862,     302, 16, }, // America/Scoresbysund
    {  405, 8471,     318, 14, }, // America/St_Barthelemy
    {  405, 2045,     332, 10, }, // America/St_Johns
    {  405, 5780,     342,  9, }, // America/St_Kitts
    {  405, 5797,     351,  9, }, // America/St_Lucia
    {  405,  855,     360, 10, }, // America/St_Thomas
    {  405, 5830,     370, 11, }, // America/St_Vincent
    {  405, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  405, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  405, 1152,     413,  9, }, // Asia/Kathmandu
    {  405,  992,     422,  7, }, // Asia/Kolkata
    {  405, 1178,     429,  5, }, // Asia/Macau
    {  405, 1202,     442,  6, }, // Asia/Yangon
    {  405, 1379,     448,  5, }, // Atlantic/Faroe
    {  405, 6784,     453, 10, }, // Atlantic/St_Helena
    {  405, 2419,     463,  4, }, // Europe/Kyiv
    {  405, 7174,     467,  7, }, // Indian/Reunion
    {  405, 3022,     474,  5, }, // Pacific/Chuuk
    {  405, 2961,     479,  7, }, // Pacific/Pohnpei
    {  406,   14,       0,  6, }, // Africa/Asmara Malagasy/Latin/Madagascar
    {  406, 4337,       6,  8, }, // Africa/Sao_Tome
    {  406, 4154,      14,  8, }, // America/Asuncion
    {  406,  237,      22,  8, }, // America/Atikokan
    {  406, 7586,      30, 17, }, // America/Bahia_Banderas
    {  406, 3770,      47,  6, }, // America/Cancun
    {  406, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  406, 5640,      66,  7, }, // America/Curacao
    {  406,  481,      73, 13, }, // America/Indiana/Knox
    {  406, 7924,      86, 16, }, // America/Indiana/Marengo
    {  406, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  406, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  406, 8001,     139, 14, }, // America/Indiana/Vevay
    {  406, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  406, 8049,     171, 16, }, // America/Indiana/Winamac
    {  406, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  406, 5849,     207, 22, }, // America/Lower_Princes
    {  406, 8183,     229,  6, }, // America/Merida
    {  406, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  406, 8292,     255, 20, }, // America/North_Dakota/Center
    {  406, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  406,  393,     298,  4, }, // America/Nuuk
    {  406, 6862,     302, 16, }, // America/Scoresbysund
    {  406, 8471,     318, 14, }, // America/St_Barthelemy
    {  406, 2045,     332, 10, }, // America/St_Johns
    {  406, 5780,     342,  9, }, // America/St_Kitts
    {  406, 5797,     351,  9, }, // America/St_Lucia
    {  406,  855,     360, 10, }, // America/St_Thomas
    {  406, 5830,     370, 11, }, // America/St_Vincent
    {  406, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  406, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  406, 1152,     413,  9, }, // Asia/Kathmandu
    {  406,  992,     422,  7, }, // Asia/Kolkata
    {  406, 1178,     429,  5, }, // Asia/Macau
    {  406, 1202,     442,  6, }, // Asia/Yangon
    {  406, 1379,     448,  5, }, // Atlantic/Faroe
    {  406, 6784,     453, 10, }, // Atlantic/St_Helena
    {  406, 2419,     463,  4, }, // Europe/Kyiv
    {  406, 7174,     467,  7, }, // Indian/Reunion
    {  406, 3022,     474,  5, }, // Pacific/Chuuk
    {  406, 2961,     479,  7, }, // Pacific/Pohnpei
    {  407, 6751,  126394, 10, }, // Africa/Abidjan Malayalam/Malayalam/India
    {  407, 6723,  126404,  4, }, // Africa/Accra
    {  407, 4961,  126408, 10, }, // Africa/Addis_Ababa
    {  407, 7383,  126418, 11, }, // Africa/Algiers
    {  407,   14,  126429,  7, }, // Africa/Asmara
    {  407,   44,  126436,  5, }, // Africa/Bamako
    {  407, 5159,  126441,  6, }, // Africa/Bangui
    {  407, 6709,  126447,  6, }, // Africa/Banjul
    {  407, 7398,  126453,  8, }, // Africa/Bissau
    {  407, 4859,  126461, 11, }, // Africa/Blantyre
    {  407, 5189,  126472,  8, }, // Africa/Brazzaville
    {  407, 4824,  126480,  9, }, // Africa/Bujumbura
    {  407, 2242,  126489,  7, }, // Africa/Cairo
    {  407, 4019,  126496, 10, }, // Africa/Casablanca
    {  407, 7412,  126506,  7, }, // Africa/Ceuta
    {  407, 6736,  126513,  8, }, // Africa/Conakry
    {  407, 6803,  126521,  7, }, // Africa/Dakar
    {  407, 5032,  126528, 12, }, // Africa/Dar_es_Salaam
    {  407, 4945,  126540, 10, }, // Africa/Djibouti
    {  407, 5145,  126550,  3, }, // Africa/Douala
    {  407, 5068,  126553,  9, }, // Africa/El_Aaiun
    {  407, 6816,  126562,  7, }, // Africa/Freetown
    {  407, 4808,  126569,  6, }, // Africa/Gaborone
    {  407, 4917,  126575,  5, }, // Africa/Harare
    {  407, 4381,  126580, 14, }, // Africa/Johannesburg
    {  407, 4401,  126594,  3, }, // Africa/Juba
    {  407, 5053,  126597,  6, }, // Africa/Kampala
    {  407, 4426,  126603,  7, }, // Africa/Khartoum
    {  407, 4875,  126610,  6, }, // Africa/Kigali
    {  407, 5208,  126616,  6, }, // Africa/Kinshasa
    {  407, 4646,  126622,  6, }, // Africa/Lagos
    {  407, 5238,  126628, 11, }, // Africa/Libreville
    {  407, 6832,  126639,  3, }, // Africa/Lome
    {  407, 5113,  126642,  6, }, // Africa/Luanda
    {  407, 4841,  126648,  9, }, // Africa/Lubumbashi
    {  407, 4903,  126657,  5, }, // Africa/Lusaka
    {  407, 5224,  126662,  5, }, // Africa/Malabo
    {  407, 4889,  126667, 10, }, // Africa/Maputo
    {  407, 5099,  126677,  5, }, // Africa/Maseru
    {  407, 5084,  126682, 10, }, // Africa/Mbabane
    {  407, 5015,  126692,  8, }, // Africa/Mogadishu
    {  407, 6997,  126700,  8, }, // Africa/Monrovia
    {  407, 3755,  126708,  8, }, // Africa/Nairobi
    {  407, 5173,  126716,  4, }, // Africa/Ndjamena
    {  407, 5256,  126720,  6, }, // Africa/Niamey
    {  407, 6766,  126726, 13, }, // Africa/Nouakchott
    {  407, 6690,  126739,  7, }, // Africa/Ouagadougou
    {  407, 5127,  126746, 13, }, // Africa/Porto-Novo
    {  407, 4337,  126759,  9, }, // Africa/Sao_Tome
    {  407, 2732,  126768,  8, }, // Africa/Tripoli
    {  407, 6503,  126776,  8, }, // Africa/Tunis
    {  407, 4054,  126784, 10, }, // Africa/Windhoek
    {  407,  132,  126794,  5, }, // America/Adak
    {  407, 3183,  126799,  8, }, // America/Anchorage
    {  407, 5524,  126807,  9, }, // America/Anguilla
    {  407, 5541,  126816,  7, }, // America/Antigua
    {  407, 4456,  126823,  7, }, // America/Araguaina
    {  407,  166,  126830, 13, }, // America/Argentina/Buenos_Aires
    {  407,   91,  126843, 14, }, // America/Argentina/Catamarca
    {  407,  270,  126857,  6, }, // America/Argentina/Cordoba
    {  407,  441,  126863,  6, }, // America/Argentina/Jujuy
    {  407, 7425,  126869,  8, }, // America/Argentina/La_Rioja
    {  407,  565,  126877,  6, }, // America/Argentina/Mendoza
    {  407, 7452,  126883, 14, }, // America/Argentina/Rio_Gallegos
    {  407, 7483,  126897,  6, }, // America/Argentina/Salta
    {  407, 7507,  126903,  9, }, // America/Argentina/San_Juan
    {  407, 5497,  126912, 10, }, // America/Argentina/San_Luis
    {  407, 7534,  126922,  9, }, // America/Argentina/Tucuman
    {  407, 7560,  126931,  4, }, // America/Argentina/Ushuaia
    {  407, 5557,  126935,  4, }, // America/Aruba
    {  407, 4154,  126939,  9, }, // America/Asuncion
    {  407,  237,  126948, 11, }, // America/Atikokan
    {  407, 3581,  126959,  4, }, // America/Bahia
    {  407, 7586,  126963, 12, }, // America/Bahia_Banderas
    {  407, 5571,  126975,  8, }, // America/Barbados
    {  407, 7609,  126983,  4, }, // America/Belem
    {  407, 5309,  126987,  6, }, // America/Belize
    {  407, 7623,  126993, 16, }, // America/Blanc-Sablon
    {  407, 7644,  127009, 11, }, // America/Boa_Vista
    {  407, 4263,  127020,  7, }, // America/Bogota
    {  407, 7662,  127027,  6, }, // America/Boise
    {  407, 7676,  127033, 15, }, // America/Cambridge_Bay
    {  407, 7698,  127048, 16, }, // America/Campo_Grande
    {  407, 3770,  127064,  6, }, // America/Cancun
    {  407, 4596,  127070,  8, }, // America/Caracas
    {  407, 4247,  127078,  5, }, // America/Cayenne
    {  407, 5398,  127083,  5, }, // America/Cayman
    {  407, 3251,  127088,  8, }, // America/Chicago
    {  407, 7719,  127096,  7, }, // America/Chihuahua
    {  407, 7737,  127103, 16, }, // America/Ciudad_Juarez
    {  407, 5324,  127119, 13, }, // America/Costa_Rica
    {  407, 7759,  127132, 10, }, // America/Creston
    {  407, 3693,  127142,  6, }, // America/Cuiaba
    {  407, 5640,  127148, 10, }, // America/Curacao
    {  407, 7775,  127158, 15, }, // America/Danmarkshavn
    {  407, 7796,  127173,  6, }, // America/Dawson
    {  407, 7811,  127179, 15, }, // America/Dawson_Creek
    {  407,  805,  127194,  6, }, // America/Denver
    {  407, 3351,  127200, 12, }, // America/Detroit
    {  407, 5656,  127212,  9, }, // America/Dominica
    {  407,  893,  127221,  8, }, // America/Edmonton
    {  407, 7832,  127229, 10, }, // America/Eirunepe
    {  407, 5343,  127239, 11, }, // America/El_Salvador
    {  407, 7849,  127250, 13, }, // America/Fort_Nelson
    {  407, 7869,  127263,  9, }, // America/Fortaleza
    {  407, 7887,  127272, 11, }, // America/Glace_Bay
    {  407, 6844,  127283,  7, }, // America/Goose_Bay
    {  407, 4514,  127290, 14, }, // America/Grand_Turk
    {  407, 5687,  127304,  6, }, // America/Grenada
    {  407, 5703,  127310, 11, }, // America/Guadeloupe
    {  407, 3662,  127321, 11, }, // America/Guatemala
    {  407, 4743,  127332, 10, }, // America/Guayaquil
    {  407, 6895,  127342,  4, }, // America/Guyana
    {  407, 1923,  127346, 10, }, // America/Halifax
    {  407, 2221,  127356,  4, }, // America/Havana
    {  407, 7905,  127360, 11, }, // America/Hermosillo
    {  407,  348,  127371, 14, }, // America/Indiana/Indianapolis
    {  407,  481,  127385, 15, }, // America/Indiana/Knox
    {  407, 7924,  127400, 16, }, // America/Indiana/Marengo
    {  407, 7948,  127416, 23, }, // America/Indiana/Petersburg
    {  407, 7975,  127439, 21, }, // America/Indiana/Tell_City
    {  407, 8001,  127460, 15, }, // America/Indiana/Vevay
    {  407, 8023,  127475, 17, }, // America/Indiana/Vincennes
    {  407, 8049,  127492, 17, }, // America/Indiana/Winamac
    {  407, 8073,  127509,  7, }, // America/Inuvik
    {  407,  660,  127516,  7, }, // America/Iqaluit
    {  407, 2665,  127523,  6, }, // America/Jamaica
    {  407, 5282,  127529,  4, }, // America/Juneau
    {  407,  521,  127533, 13, }, // America/Kentucky/Louisville
    {  407, 8088,  127546, 23, }, // America/Kentucky/Monticello
    {  407, 5621,  127569, 13, }, // America/Kralendijk
    {  407, 4278,  127582,  7, }, // America/La_Paz
    {  407, 7132,  127589,  3, }, // America/Lima
    {  407, 3390,  127592, 14, }, // America/Los_Angeles
    {  407, 5849,  127606, 23, }, // America/Lower_Princes
    {  407, 8116,  127629,  6, }, // America/Maceio
    {  407, 8131,  127635,  6, }, // America/Managua
    {  407, 1892,  127641,  5, }, // America/Manaus
    {  407, 5814,  127646,  6, }, // America/Marigot
    {  407, 5722,  127652, 11, }, // America/Martinique
    {  407, 8147,  127663, 11, }, // America/Matamoros
    {  407, 2779,  127674, 10, }, // America/Mazatlan
    {  407, 8165,  127684,  8, }, // America/Menominee
    {  407, 8183,  127692,  5, }, // America/Merida
    {  407, 8198,  127697, 12, }, // America/Metlakatla
    {  407, 2811,  127709, 17, }, // America/Mexico_City
    {  407, 4293,  127726,  7, }, // America/Miquelon
    {  407, 8217,  127733,  8, }, // America/Moncton
    {  407, 8233,  127741,  8, }, // America/Monterrey
    {  407, 4000,  127749, 13, }, // America/Montevideo
    {  407, 5741,  127762, 12, }, // America/Montserrat
    {  407, 5383,  127774,  4, }, // America/Nassau
    {  407, 3294,  127778, 11, }, // America/New_York
    {  407, 8251,  127789,  3, }, // America/Nome
    {  407, 1834,  127792,  5, }, // America/Noronha
    {  407, 8264,  127797, 23, }, // America/North_Dakota/Beulah
    {  407, 8292,  127820, 24, }, // America/North_Dakota/Center
    {  407, 8320,  127844, 30, }, // America/North_Dakota/New_Salem
    {  407,  393,  127874,  6, }, // America/Nuuk
    {  407, 8351,  127880,  5, }, // America/Ojinaga
    {  407, 5413,  127885,  4, }, // America/Panama
    {  407, 6106,  127889,  9, }, // America/Paramaribo
    {  407, 3224,  127898,  8, }, // America/Phoenix
    {  407, 3847,  127906, 14, }, // America/Port-au-Prince
    {  407, 5871,  127920, 22, }, // America/Port_of_Spain
    {  407, 8367,  127942, 14, }, // America/Porto_Velho
    {  407, 5760,  127956, 16, }, // America/Puerto_Rico
    {  407, 3932,  127972, 12, }, // America/Punta_Arenas
    {  407, 8387,  127984, 16, }, // America/Rankin_Inlet
    {  407, 8408,  128000,  6, }, // America/Recife
    {  407, 1979,  128006,  5, }, // America/Regina
    {  407, 8423,  128011, 11, }, // America/Resolute
    {  407,  695,  128022, 13, }, // America/Rio_Branco
    {  407, 8440,  128035,  8, }, // America/Santarem
    {  407, 2165,  128043, 10, }, // America/Santiago
    {  407, 6044,  128053, 14, }, // America/Santo_Domingo
    {  407, 1862,  128067,  8, }, // America/Sao_Paulo
    {  407, 6862,  128075, 21, }, // America/Scoresbysund
    {  407, 8457,  128096,  9, }, // America/Sitka
    {  407, 8471,  128105, 16, }, // America/St_Barthelemy
    {  407, 2045,  128121, 12, }, // America/St_Johns
    {  407, 5780,  128133, 15, }, // America/St_Kitts
    {  407, 5797,  128148, 12, }, // America/St_Lucia
    {  407,  855,  128160, 12, }, // America/St_Thomas
    {  407, 5830,  128172, 16, }, // America/St_Vincent
    {  407, 8493,  128188, 18, }, // America/Swift_Current
    {  407, 5363,  128206, 10, }, // America/Tegucigalpa
    {  407, 5673,  128216,  4, }, // America/Thule
    {  407,  313,  128220,  7, }, // America/Tijuana
    {  407,  608,  128227,  7, }, // America/Toronto
    {  407, 5605,  128234,  9, }, // America/Tortola
    {  407, 2077,  128243,  8, }, // America/Vancouver
    {  407, 2128,  128251, 12, }, // America/Whitehorse
    {  407,  734,  128263, 10, }, // America/Winnipeg
    {  407, 8515,  128273, 12, }, // America/Yakutat
    {  407, 5918,  128103,  4, }, // Antarctica/Casey
    {  407, 6027,  128285,  9, }, // Antarctica/Davis
    {  407, 6066,  128294, 19, }, // Antarctica/DumontDUrville
    {  407, 8531,  128313,  7, }, // Antarctica/Macquarie
    {  407, 7029,  128320,  6, }, // Antarctica/Mawson
    {  407, 7076,  128326,  8, }, // Antarctica/McMurdo
    {  407, 8552,  128334,  4, }, // Antarctica/Palmer
    {  407, 7189,  128338,  5, }, // Antarctica/Rothera
    {  407, 7258,  128343,  5, }, // Antarctica/Syowa
    {  407, 8570,  128348,  5, }, // Antarctica/Troll
    {  407, 7337,  128353, 10, }, // Antarctica/Vostok
    {  407, 1413,  128363, 11, }, // Arctic/Longyearbyen
    {  407, 5487,  128374,  4, }, // Asia/Aden
    {  407, 5297,  128378,  8, }, // Asia/Almaty
    {  407, 3870,  128386,  7, }, // Asia/Amman
    {  407, 5428,  128393,  6, }, // Asia/Anadyr
    {  407, 5440,  128399,  5, }, // Asia/Aqtau
    {  407, 5270,  128404,  8, }, // Asia/Aqtobe
    {  407,  964,  128412, 11, }, // Asia/Ashgabat
    {  407, 8587,  128423,  7, }, // Asia/Atyrau
    {  407, 3509,  128430,  9, }, // Asia/Baghdad
    {  407, 5451,  128439,  6, }, // Asia/Bahrain
    {  407, 3555,  128445,  6, }, // Asia/Baku
    {  407, 4368,  128451, 10, }, // Asia/Bangkok
    {  407, 3473,  128461,  5, }, // Asia/Barnaul
    {  407, 3988,  128466, 11, }, // Asia/Beirut
    {  407, 3680,  128477,  9, }, // Asia/Bishkek
    {  407, 5906,  128486,  6, }, // Asia/Brunei
    {  407, 4503,  128492,  5, }, // Asia/Chita
    {  407, 5963,  128497,  8, }, // Asia/Choibalsan
    {  407, 4413,  128505,  7, }, // Asia/Colombo
    {  407, 4442,  128512,  8, }, // Asia/Damascus
    {  407, 1060,  128520,  5, }, // Asia/Dhaka
    {  407, 6125,  128525,  4, }, // Asia/Dili
    {  407, 3498,  128529,  6, }, // Asia/Dubai
    {  407, 6092,  128535,  7, }, // Asia/Dushanbe
    {  407, 8599,  128542, 11, }, // Asia/Famagusta
    {  407, 8614,  128553,  3, }, // Asia/Gaza
    {  407, 4697,  128556,  8, }, // Asia/Hebron
    {  407, 1226,  128564, 16, }, // Asia/Ho_Chi_Minh
    {  407, 2576,  128580,  9, }, // Asia/Hong_Kong
    {  407, 4673,  128589,  4, }, // Asia/Hovd
    {  407, 4086,  128593,  7, }, // Asia/Irkutsk
    {  407, 6969,  128600,  9, }, // Asia/Jakarta
    {  407, 6955,  128609,  5, }, // Asia/Jayapura
    {  407, 1257,  128614,  8, }, // Asia/Jerusalem
    {  407, 3462,  128622,  5, }, // Asia/Kabul
    {  407, 4218,  128627,  7, }, // Asia/Kamchatka
    {  407, 4141,  128634,  7, }, // Asia/Karachi
    {  407, 1152,  128641, 10, }, // Asia/Kathmandu
    {  407, 8624,  128651,  6, }, // Asia/Khandyga
    {  407,  992,  128657, 10, }, // Asia/Kolkata
    {  407, 4099,  128667, 14, }, // Asia/Krasnoyarsk
    {  407, 4761,  128681, 13, }, // Asia/Kuala_Lumpur
    {  407, 5893,  128694,  6, }, // Asia/Kuching
    {  407, 5464,  128700,  8, }, // Asia/Kuwait
    {  407, 1178,  128708,  7, }, // Asia/Macau
    {  407, 3919,  128715,  5, }, // Asia/Magadan
    {  407, 1316,  128720,  6, }, // Asia/Makassar
    {  407, 7145,  128726,  4, }, // Asia/Manila
    {  407, 6883,  128730,  8, }, // Asia/Muscat
    {  407, 2446,  128738,  9, }, // Asia/Nicosia
    {  407, 8638,  128747, 14, }, // Asia/Novokuznetsk
    {  407, 4037,  128761, 13, }, // Asia/Novosibirsk
    {  407, 4131,  128774,  6, }, // Asia/Omsk
    {  407, 7108,  128780,  3, }, // Asia/Oral
    {  407, 6924,  128783,  9, }, // Asia/Phnom_Penh
    {  407, 8656,  128792, 11, }, // Asia/Pontianak
    {  407, 4116,  128803, 13, }, // Asia/Pyongyang
    {  407, 5476,  128816,  5, }, // Asia/Qatar
    {  407, 8671,  128821,  9, }, // Asia/Qostanay
    {  407, 4171,  128830,  8, }, // Asia/Qyzylorda
    {  407, 3486,  128838,  6, }, // Asia/Riyadh
    {  407, 4310,  128844,  6, }, // Asia/Sakhalin
    {  407, 7208,  128850, 10, }, // Asia/Samarkand
    {  407, 3116,  128860,  3, }, // Asia/Seoul
    {  407, 1020,  128863,  9, }, // Asia/Shanghai
    {  407, 3137,  128872,  9, }, // Asia/Singapore
    {  407, 4199,  128881, 18, }, // Asia/Srednekolymsk
    {  407, 3100,  128899,  9, }, // Asia/Taipei
    {  407, 4683,  128908, 12, }, // Asia/Tashkent
    {  407, 3817,  128920,  8, }, // Asia/Tbilisi
    {  407, 2638,  128928,  9, }, // Asia/Tehran
    {  407, 1284,  128937,  5, }, // Asia/Thimphu
    {  407, 2687,  128942,  8, }, // Asia/Tokyo
    {  407, 4474,  128950,  7, }, // Asia/Tomsk
    {  407, 1346,  128957, 10, }, // Asia/Ulaanbaatar
    {  407, 1126,  128967,  6, }, // Asia/Urumqi
    {  407, 8685,  128973,  8, }, // Asia/Ust-Nera
    {  407, 6940,  128981,  9, }, // Asia/Vientiane
    {  407, 4612,  128990, 16, }, // Asia/Vladivostok
    {  407, 4730,  129006, 14, }, // Asia/Yakutsk
    {  407, 1202,  129020,  7, }, // Asia/Yangon
    {  407, 3785,  129027, 15, }, // Asia/Yekaterinburg
    {  407, 3649,  129042,  6, }, // Asia/Yerevan
    {  407, 3565,  129048,  6, }, // Atlantic/Azores
    {  407, 5588,  129054,  5, }, // Atlantic/Bermuda
    {  407, 6591,  129059,  7, }, // Atlantic/Canary
    {  407, 3629,  129066, 10, }, // Atlantic/Cape_Verde
    {  407, 1379,  129076,  4, }, // Atlantic/Faroe
    {  407, 8699,  129080,  6, }, // Atlantic/Madeira
    {  407, 2614,  129086, 11, }, // Atlantic/Reykjavik
    {  407, 7235,  129097, 13, }, // Atlantic/South_Georgia
    {  407, 6784,  129110, 12, }, // Atlantic/St_Helena
    {  407, 6607,  129122, 10, }, // Atlantic/Stanley
    {  407, 1654,  129132, 10, }, // Australia/Adelaide
    {  407, 1619,  129142, 12, }, // Australia/Brisbane
    {  407, 1783,  129154, 12, }, // Australia/Broken_Hill
    {  407, 1567,  129166,  6, }, // Australia/Darwin
    {  407, 3539,  129172,  5, }, // Australia/Eucla
    {  407, 1500,  129177,  9, }, // Australia/Hobart
    {  407, 8716,  129186,  8, }, // Australia/Lindeman
    {  407, 1531,  129194,  9, }, // Australia/Lord_Howe
    {  407, 1711,  129203,  5, }, // Australia/Melbourne
    {  407, 1746,  129208,  7, }, // Australia/Perth
    {  407, 1447,  129215,  6, }, // Australia/Sydney
    {  407, 6390,  129221, 13, }, // Europe/Amsterdam
    {  407, 6149,  129234,  6, }, // Europe/Andorra
    {  407, 3522,  129240, 10, }, // Europe/Astrakhan
    {  407, 6577,  129250,  6, }, // Europe/Athens
    {  407, 6285,  129256,  9, }, // Europe/Belgrade
    {  407, 4659,  129265,  8, }, // Europe/Berlin
    {  407, 6437,  129273, 14, }, // Europe/Bratislava
    {  407, 6178,  129287, 10, }, // Europe/Brussels
    {  407, 3830,  129297, 14, }, // Europe/Bucharest
    {  407, 3708,  129311, 12, }, // Europe/Budapest
    {  407, 8751,  129323,  7, }, // Europe/Busingen
    {  407, 2475,  129330,  8, }, // Europe/Chisinau
    {  407, 6238,  129338, 12, }, // Europe/Copenhagen
    {  407, 2260,  129350,  6, }, // Europe/Dublin
    {  407, 6256,  129356, 11, }, // Europe/Gibraltar
    {  407, 8767,  129367,  5, }, // Europe/Guernsey
    {  407, 6561,  129372, 10, }, // Europe/Helsinki
    {  407, 8783,  129382, 12, }, // Europe/Isle_of_Man
    {  407, 1097,  129394, 11, }, // Europe/Istanbul
    {  407, 8802,  129405,  7, }, // Europe/Jersey
    {  407, 3881,  129412, 12, }, // Europe/Kaliningrad
    {  407, 8924,  129424,  4, }, // Europe/Kirov
    {  407, 2419,  129428,  4, }, // Europe/Kyiv
    {  407, 3078,  129432,  8, }, // Europe/Lisbon
    {  407, 6455,  129440, 10, }, // Europe/Ljubljana
    {  407, 2393,  129450,  6, }, // Europe/London
    {  407, 6314,  129456, 10, }, // Europe/Luxembourg
    {  407, 4794,  129466,  8, }, // Europe/Madrid
    {  407, 6346,  129474,  6, }, // Europe/Malta
    {  407, 6531,  129480,  9, }, // Europe/Mariehamn
    {  407, 3595,  129489,  8, }, // Europe/Minsk
    {  407, 6359,  129497,  8, }, // Europe/Monaco
    {  407, 3443,  129505,  6, }, // Europe/Moscow
    {  407, 6407,  129511,  5, }, // Europe/Oslo
    {  407, 4186,  129516,  6, }, // Europe/Paris
    {  407, 6373,  129522, 10, }, // Europe/Podgorica
    {  407, 6224,  129532,  6, }, // Europe/Prague
    {  407, 8816,  126818,  3, }, // Europe/Riga
    {  407, 6273,  129538,  3, }, // Europe/Rome
    {  407, 4233,  129541,  4, }, // Europe/Samara
    {  407, 6419,  129545, 10, }, // Europe/San_Marino
    {  407, 6194,  129555,  6, }, // Europe/Sarajevo
    {  407, 4353,  129561,  9, }, // Europe/Saratov
    {  407, 8828,  129570, 10, }, // Europe/Simferopol
    {  407, 6332,  129580, 10, }, // Europe/Skopje
    {  407, 6548,  129590,  5, }, // Europe/Sofia
    {  407, 6472,  129595, 13, }, // Europe/Stockholm
    {  407, 8846,  129608,  6, }, // Europe/Tallinn
    {  407, 6135,  129614,  6, }, // Europe/Tirane
    {  407, 8861,  129620, 17, }, // Europe/Ulyanovsk
    {  407, 6301,  129637,  6, }, // Europe/Vaduz
    {  407, 6516,  129643, 10, }, // Europe/Vatican
    {  407, 6164,  129653,  6, }, // Europe/Vienna
    {  407, 8878,  129659,  9, }, // Europe/Vilnius
    {  407, 4629,  129668, 11, }, // Europe/Volgograd
    {  407, 3055,  129679,  5, }, // Europe/Warsaw
    {  407, 6210,  129684,  8, }, // Europe/Zagreb
    {  407, 6489,  129692,  8, }, // Europe/Zurich
    {  407, 4980,  129700, 10, }, // Indian/Antananarivo
    {  407, 6910,  129710,  6, }, // Indian/Chagos
    {  407, 5979,  129716, 11, }, // Indian/Christmas
    {  407, 5996,  129727,  8, }, // Indian/Cocos
    {  407, 4931,  129735,  6, }, // Indian/Comoro
    {  407, 6624,  129741,  8, }, // Indian/Kerguelen
    {  407, 7223,  129749,  4, }, // Indian/Mahe
    {  407, 7013,  129753, 10, }, // Indian/Maldives
    {  407, 3971,  129763,  9, }, // Indian/Mauritius
    {  407, 5000,  129772,  7, }, // Indian/Mayotte
    {  407, 7174,  129779,  8, }, // Indian/Reunion
    {  407, 4324,  129787,  4, }, // Pacific/Apia
    {  407,  932,  129791, 12, }, // Pacific/Auckland
    {  407, 3608,  129803, 10, }, // Pacific/Bougainville
    {  407, 2863,  129813,  6, }, // Pacific/Chatham
    {  407, 3022,  129819,  3, }, // Pacific/Chuuk
    {  407, 2201,  129822,  7, }, // Pacific/Easter
    {  407, 7323,  129829,  5, }, // Pacific/Efate
    {  407, 7290,  129834,  9, }, // Pacific/Fakaofo
    {  407, 3804,  129843,  4, }, // Pacific/Fiji
    {  407, 7306,  129847, 10, }, // Pacific/Funafuti
    {  407, 6641,  129857, 11, }, // Pacific/Galapagos
    {  407, 6659,  129868,  8, }, // Pacific/Gambier
    {  407, 3724,  129876, 10, }, // Pacific/Guadalcanal
    {  407, 5935,  129886,  5, }, // Pacific/Guam
    {  407, 2929,  129891,  7, }, // Pacific/Honolulu
    {  407, 2897,  129898,  6, }, // Pacific/Kanton
    {  407, 3900,  129904, 14, }, // Pacific/Kiritimati
    {  407, 6982,  129918,  6, }, // Pacific/Kosrae
    {  407, 2708,  129924, 10, }, // Pacific/Kwajalein
    {  407, 4779,  129934,  5, }, // Pacific/Majuro
    {  407, 3953,  129939, 11, }, // Pacific/Marquesas
    {  407, 8909,  129950,  7, }, // Pacific/Midway
    {  407, 7047,  129957,  4, }, // Pacific/Nauru
    {  407, 7095,  129961,  4, }, // Pacific/Niue
    {  407, 4070,  129965, 10, }, // Pacific/Norfolk
    {  407, 7061,  129975,  5, }, // Pacific/Noumea
    {  407, 2991,  129980,  9, }, // Pacific/Pago_Pago
    {  407, 7118,  129989,  5, }, // Pacific/Palau
    {  407, 7157,  129994, 11, }, // Pacific/Pitcairn
    {  407, 2961,  130005,  5, }, // Pacific/Pohnpei
    {  407, 4709,  130010, 17, }, // Pacific/Port_Moresby
    {  407, 6009,  130027,  8, }, // Pacific/Rarotonga
    {  407, 5948,  130035, 11, }, // Pacific/Saipan
    {  407, 7275,  130046,  6, }, // Pacific/Tahiti
    {  407, 6675,  130052,  4, }, // Pacific/Tarawa
    {  407, 4485,  130056,  9, }, // Pacific/Tongatapu
    {  407, 7355,  130065,  6, }, // Pacific/Wake
    {  407, 7368,  130071,  8, }, // Pacific/Wallis
    {  408,   14,       0,  6, }, // Africa/Asmara Malay/Latin/Malaysia
    {  408, 2242,  130079,  7, }, // Africa/Cairo
    {  408, 4337,       6,  8, }, // Africa/Sao_Tome
    {  408, 4154,      14,  8, }, // America/Asuncion
    {  408,  237,      22,  8, }, // America/Atikokan
    {  408, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  408, 7676,   96173, 15, }, // America/Cambridge_Bay
    {  408, 3770,    3861,  6, }, // America/Cancun
    {  408, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  408, 5640,      66,  7, }, // America/Curacao
    {  408, 7887,   96240, 11, }, // America/Glace_Bay
    {  408, 6844,   96251, 11, }, // America/Goose_Bay
    {  408,  481,      73, 13, }, // America/Indiana/Knox
    {  408, 7924,      86, 16, }, // America/Indiana/Marengo
    {  408, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  408, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  408, 8001,     139, 14, }, // America/Indiana/Vevay
    {  408, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  408, 8049,     171, 16, }, // America/Indiana/Winamac
    {  408, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  408, 5849,     207, 22, }, // America/Lower_Princes
    {  408, 8183,    3899,  6, }, // America/Merida
    {  408, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  408, 8292,     255, 20, }, // America/North_Dakota/Center
    {  408, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  408,  393,     298,  4, }, // America/Nuuk
    {  408, 6862,     302, 16, }, // America/Scoresbysund
    {  408, 8471,  113081, 16, }, // America/St_Barthelemy
    {  408, 2045,     332, 10, }, // America/St_Johns
    {  408, 5780,     342,  9, }, // America/St_Kitts
    {  408, 5797,     351,  9, }, // America/St_Lucia
    {  408,  855,     360, 10, }, // America/St_Thomas
    {  408, 5830,     370, 11, }, // America/St_Vincent
    {  408, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  408, 4442,  130086,  7, }, // Asia/Damascus
    {  408, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  408, 1257,  130093, 14, }, // Asia/Jerusalem
    {  408, 1152,     413,  9, }, // Asia/Kathmandu
    {  408,  992,     422,  7, }, // Asia/Kolkata
    {  408, 1178,    4222,  5, }, // Asia/Macau
    {  408, 8671,     434,  8, }, // Asia/Qostanay
    {  408, 3137,   91698,  9, }, // Asia/Singapore
    {  408, 1202,     442,  6, }, // Asia/Yangon
    {  408, 1379,     448,  5, }, // Atlantic/Faroe
    {  408, 6784,     453, 10, }, // Atlantic/St_Helena
    {  408, 2419,    6486,  4, }, // Europe/Kyiv
    {  408, 7174,     467,  7, }, // Indian/Reunion
    {  408, 3022,     474,  5, }, // Pacific/Chuuk
    {  408, 2929,    4512,  8, }, // Pacific/Honolulu
    {  408, 2897,    4520,  9, }, // Pacific/Kanton
    {  408, 2961,     479,  7, }, // Pacific/Pohnpei
    {  409, 7586,      30, 17, }, // America/Bahia_Banderas Malay/Arabic/Brunei
    {  409, 3770,      47,  6, }, // America/Cancun
    {  409, 8183,     229,  6, }, // America/Merida
    {  409, 8471,     318, 14, }, // America/St_Barthelemy
    {  409, 1178,     429,  5, }, // Asia/Macau
    {  409, 3137,  130107,  8, }, // Asia/Singapore
    {  409, 2419,     463,  4, }, // Europe/Kyiv
    {  410, 7586,      30, 17, }, // America/Bahia_Banderas Malay/Arabic/Malaysia
    {  410, 3770,      47,  6, }, // America/Cancun
    {  410, 8183,     229,  6, }, // America/Merida
    {  410, 8471,     318, 14, }, // America/St_Barthelemy
    {  410, 1178,     429,  5, }, // Asia/Macau
    {  410, 3137,  130107,  8, }, // Asia/Singapore
    {  410, 2419,     463,  4, }, // Europe/Kyiv
    {  414, 7383,  130115,  8, }, // Africa/Algiers Maltese/Latin/Malta
    {  414,   14,       0,  6, }, // Africa/Asmara
    {  414, 4337,       6,  8, }, // Africa/Sao_Tome
    {  414, 4154,      14,  8, }, // America/Asuncion
    {  414,  237,      22,  8, }, // America/Atikokan
    {  414, 7586,      30, 17, }, // America/Bahia_Banderas
    {  414, 3770,      47,  6, }, // America/Cancun
    {  414, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  414, 5640,      66,  7, }, // America/Curacao
    {  414, 3662,  130123, 12, }, // America/Guatemala
    {  414,  481,      73, 13, }, // America/Indiana/Knox
    {  414, 7924,      86, 16, }, // America/Indiana/Marengo
    {  414, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  414, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  414, 8001,     139, 14, }, // America/Indiana/Vevay
    {  414, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  414, 8049,     171, 16, }, // America/Indiana/Winamac
    {  414, 2665,  130135, 10, }, // America/Jamaica
    {  414, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  414, 5849,     207, 22, }, // America/Lower_Princes
    {  414, 8183,     229,  6, }, // America/Merida
    {  414, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  414, 8292,     255, 20, }, // America/North_Dakota/Center
    {  414, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  414,  393,     298,  4, }, // America/Nuuk
    {  414, 6862,     302, 16, }, // America/Scoresbysund
    {  414, 8471,     318, 14, }, // America/St_Barthelemy
    {  414, 2045,     332, 10, }, // America/St_Johns
    {  414, 5780,     342,  9, }, // America/St_Kitts
    {  414, 5797,     351,  9, }, // America/St_Lucia
    {  414,  855,     360, 10, }, // America/St_Thomas
    {  414, 5830,     370, 11, }, // America/St_Vincent
    {  414, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  414, 3988,    5930,  6, }, // Asia/Beirut
    {  414, 4442,    4102,  7, }, // Asia/Damascus
    {  414, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  414, 1257,  130145, 10, }, // Asia/Jerusalem
    {  414, 1152,     413,  9, }, // Asia/Kathmandu
    {  414,  992,     422,  7, }, // Asia/Kolkata
    {  414, 5464,  130155, 18, }, // Asia/Kuwait
    {  414, 1178,    4222,  5, }, // Asia/Macau
    {  414, 2446,   55803,  8, }, // Asia/Nicosia
    {  414, 1202,   56146,  7, }, // Asia/Yangon
    {  414, 1379,     448,  5, }, // Atlantic/Faroe
    {  414, 7235,  130173, 24, }, // Atlantic/South_Georgia
    {  414, 6784,     453, 10, }, // Atlantic/St_Helena
    {  414, 6577,  130197,  5, }, // Europe/Athens
    {  414, 6285,   25465,  7, }, // Europe/Belgrade
    {  414, 6178,  130202,  8, }, // Europe/Brussels
    {  414, 6256,  130210,  7, }, // Europe/Gibraltar
    {  414, 2419,     463,  4, }, // Europe/Kyiv
    {  414, 3078,  130217,  7, }, // Europe/Lisbon
    {  414, 2393,   93889,  6, }, // Europe/London
    {  414, 6314,  130224, 14, }, // Europe/Luxembourg
    {  414, 6346,  130238,  8, }, // Europe/Malta
    {  414, 3443,   56005,  5, }, // Europe/Moscow
    {  414, 4186,  130246,  6, }, // Europe/Paris
    {  414, 6224,   19576,  5, }, // Europe/Prague
    {  414, 6273,  130252,  4, }, // Europe/Rome
    {  414, 6548,   32232,  6, }, // Europe/Sofia
    {  414, 6472,  130256,  9, }, // Europe/Stockholm
    {  414, 6135,   19608,  6, }, // Europe/Tirane
    {  414, 6516,  130265, 19, }, // Europe/Vatican
    {  414, 6164,  130284,  6, }, // Europe/Vienna
    {  414, 3055,  130290,  8, }, // Europe/Warsaw
    {  414, 7013,  130298, 10, }, // Indian/Maldives
    {  414, 7174,     467,  7, }, // Indian/Reunion
    {  414, 3022,     474,  5, }, // Pacific/Chuuk
    {  414, 2929,    4512,  8, }, // Pacific/Honolulu
    {  414, 2897,    4520,  9, }, // Pacific/Kanton
    {  414, 2961,     479,  7, }, // Pacific/Pohnpei
    {  415,   14,       0,  6, }, // Africa/Asmara Manipuri/Bangla/India
    {  415, 4337,       6,  8, }, // Africa/Sao_Tome
    {  415, 4154,      14,  8, }, // America/Asuncion
    {  415,  237,      22,  8, }, // America/Atikokan
    {  415, 7586,      30, 17, }, // America/Bahia_Banderas
    {  415, 3770,      47,  6, }, // America/Cancun
    {  415, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  415, 5640,      66,  7, }, // America/Curacao
    {  415,  481,      73, 13, }, // America/Indiana/Knox
    {  415, 7924,      86, 16, }, // America/Indiana/Marengo
    {  415, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  415, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  415, 8001,     139, 14, }, // America/Indiana/Vevay
    {  415, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  415, 8049,     171, 16, }, // America/Indiana/Winamac
    {  415, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  415, 5849,     207, 22, }, // America/Lower_Princes
    {  415, 8183,     229,  6, }, // America/Merida
    {  415, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  415, 8292,     255, 20, }, // America/North_Dakota/Center
    {  415, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  415,  393,     298,  4, }, // America/Nuuk
    {  415, 6862,     302, 16, }, // America/Scoresbysund
    {  415, 8471,     318, 14, }, // America/St_Barthelemy
    {  415, 2045,     332, 10, }, // America/St_Johns
    {  415, 5780,     342,  9, }, // America/St_Kitts
    {  415, 5797,     351,  9, }, // America/St_Lucia
    {  415,  855,     360, 10, }, // America/St_Thomas
    {  415, 5830,     370, 11, }, // America/St_Vincent
    {  415, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  415, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  415, 1152,     413,  9, }, // Asia/Kathmandu
    {  415,  992,     422,  7, }, // Asia/Kolkata
    {  415, 1178,     429,  5, }, // Asia/Macau
    {  415, 1202,     442,  6, }, // Asia/Yangon
    {  415, 1379,     448,  5, }, // Atlantic/Faroe
    {  415, 6784,     453, 10, }, // Atlantic/St_Helena
    {  415, 2419,     463,  4, }, // Europe/Kyiv
    {  415, 7174,     467,  7, }, // Indian/Reunion
    {  415, 3022,     474,  5, }, // Pacific/Chuuk
    {  415, 2961,     479,  7, }, // Pacific/Pohnpei
    {  417,   14,       0,  6, }, // Africa/Asmara Manx/Latin/Isle of Man
    {  417, 4337,       6,  8, }, // Africa/Sao_Tome
    {  417, 4154,      14,  8, }, // America/Asuncion
    {  417,  237,      22,  8, }, // America/Atikokan
    {  417, 7586,      30, 17, }, // America/Bahia_Banderas
    {  417, 3770,      47,  6, }, // America/Cancun
    {  417, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  417, 5640,      66,  7, }, // America/Curacao
    {  417,  481,      73, 13, }, // America/Indiana/Knox
    {  417, 7924,      86, 16, }, // America/Indiana/Marengo
    {  417, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  417, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  417, 8001,     139, 14, }, // America/Indiana/Vevay
    {  417, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  417, 8049,     171, 16, }, // America/Indiana/Winamac
    {  417, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  417, 5849,     207, 22, }, // America/Lower_Princes
    {  417, 8183,     229,  6, }, // America/Merida
    {  417, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  417, 8292,     255, 20, }, // America/North_Dakota/Center
    {  417, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  417,  393,     298,  4, }, // America/Nuuk
    {  417, 6862,     302, 16, }, // America/Scoresbysund
    {  417, 8471,     318, 14, }, // America/St_Barthelemy
    {  417, 2045,     332, 10, }, // America/St_Johns
    {  417, 5780,     342,  9, }, // America/St_Kitts
    {  417, 5797,     351,  9, }, // America/St_Lucia
    {  417,  855,     360, 10, }, // America/St_Thomas
    {  417, 5830,     370, 11, }, // America/St_Vincent
    {  417, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  417, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  417, 1152,     413,  9, }, // Asia/Kathmandu
    {  417,  992,     422,  7, }, // Asia/Kolkata
    {  417, 1178,     429,  5, }, // Asia/Macau
    {  417, 1202,     442,  6, }, // Asia/Yangon
    {  417, 1379,     448,  5, }, // Atlantic/Faroe
    {  417, 6784,     453, 10, }, // Atlantic/St_Helena
    {  417, 2419,     463,  4, }, // Europe/Kyiv
    {  417, 7174,     467,  7, }, // Indian/Reunion
    {  417, 3022,     474,  5, }, // Pacific/Chuuk
    {  417, 2961,     479,  7, }, // Pacific/Pohnpei
    {  418,   14,       0,  6, }, // Africa/Asmara Maori/Latin/New Zealand
    {  418, 2242,   24913,  5, }, // Africa/Cairo
    {  418, 3755,  130308,  8, }, // Africa/Nairobi
    {  418, 4337,       6,  8, }, // Africa/Sao_Tome
    {  418, 2732,  130316,  8, }, // Africa/Tripoli
    {  418, 6503,  130324,  6, }, // Africa/Tunis
    {  418, 5541,  130330, 12, }, // America/Antigua
    {  418, 4154,      14,  8, }, // America/Asuncion
    {  418,  237,      22,  8, }, // America/Atikokan
    {  418, 7586,      30, 17, }, // America/Bahia_Banderas
    {  418, 5571,  130342,  8, }, // America/Barbados
    {  418, 5309,  130350,  6, }, // America/Belize
    {  418, 7676,  130356, 12, }, // America/Cambridge_Bay
    {  418, 3770,      47,  6, }, // America/Cancun
    {  418, 5398,  130368,  6, }, // America/Cayman
    {  418, 3251,  130374,  6, }, // America/Chicago
    {  418, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  418, 5324,  130380,  9, }, // America/Costa_Rica
    {  418, 5640,      66,  7, }, // America/Curacao
    {  418, 5656,  130389,  8, }, // America/Dominica
    {  418, 5343,  130397,  8, }, // America/El_Salvador
    {  418, 6844,  130405,  9, }, // America/Goose_Bay
    {  418, 4514,  130414, 10, }, // America/Grand_Turk
    {  418, 5687,  130424,  8, }, // America/Grenada
    {  418, 3662,  130432,  9, }, // America/Guatemala
    {  418, 2221,  130441,  7, }, // America/Havana
    {  418,  481,      73, 13, }, // America/Indiana/Knox
    {  418, 7924,      86, 16, }, // America/Indiana/Marengo
    {  418, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  418, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  418, 8001,     139, 14, }, // America/Indiana/Vevay
    {  418, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  418, 8049,     171, 16, }, // America/Indiana/Winamac
    {  418, 2665,  130448,  7, }, // America/Jamaica
    {  418, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  418, 3390,  130455, 11, }, // America/Los_Angeles
    {  418, 5849,     207, 22, }, // America/Lower_Princes
    {  418, 5722,  130466,  8, }, // America/Martinique
    {  418, 8183,     229,  6, }, // America/Merida
    {  418, 2811,  130474, 15, }, // America/Mexico_City
    {  418, 3294,  130489, 12, }, // America/New_York
    {  418, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  418, 8292,     255, 20, }, // America/North_Dakota/Center
    {  418, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  418,  393,     298,  4, }, // America/Nuuk
    {  418, 5760,  130501,  9, }, // America/Puerto_Rico
    {  418, 6862,     302, 16, }, // America/Scoresbysund
    {  418, 8471,     318, 14, }, // America/St_Barthelemy
    {  418, 2045,  130510,  9, }, // America/St_Johns
    {  418, 5780,     342,  9, }, // America/St_Kitts
    {  418, 5797,  130519, 11, }, // America/St_Lucia
    {  418,  855,  130530, 11, }, // America/St_Thomas
    {  418, 5830,  130541, 11, }, // America/St_Vincent
    {  418,  608,  130552,  6, }, // America/Toronto
    {  418, 2077,  130558, 16, }, // America/Vancouver
    {  418, 6027,  130574,  6, }, // Antarctica/Davis
    {  418, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  418, 8531,  130580,  7, }, // Antarctica/Macquarie
    {  418, 3509,  130587,  8, }, // Asia/Baghdad
    {  418, 5451,  130595,  7, }, // Asia/Bahrain
    {  418, 4368,  130602,  9, }, // Asia/Bangkok
    {  418, 5906,  130611,  7, }, // Asia/Brunei
    {  418, 3498,  130618,  5, }, // Asia/Dubai
    {  418, 8614,  130623,  4, }, // Asia/Gaza
    {  418, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  418, 2576,  130627,  8, }, // Asia/Hong_Kong
    {  418, 6969,  130635,  7, }, // Asia/Jakarta
    {  418, 1257,  130642, 10, }, // Asia/Jerusalem
    {  418, 1152,  130652,  8, }, // Asia/Kathmandu
    {  418,  992,     422,  7, }, // Asia/Kolkata
    {  418, 4761,  130660, 10, }, // Asia/Kuala_Lumpur
    {  418, 5464,  130670,  7, }, // Asia/Kuwait
    {  418, 1178,   32008,  5, }, // Asia/Macau
    {  418, 7145,  130677,  6, }, // Asia/Manila
    {  418, 6924,  130683, 11, }, // Asia/Phnom_Penh
    {  418, 5476,  130694,  4, }, // Asia/Qatar
    {  418, 3486,  130698,  5, }, // Asia/Riyadh
    {  418, 3116,  130703,  5, }, // Asia/Seoul
    {  418, 1020,  130708,  8, }, // Asia/Shanghai
    {  418, 3137,  130716,  8, }, // Asia/Singapore
    {  418, 2638,  130724,  6, }, // Asia/Tehran
    {  418, 2687,  130730,  5, }, // Asia/Tokyo
    {  418, 1202,     442,  6, }, // Asia/Yangon
    {  418, 5588,  130735,  6, }, // Atlantic/Bermuda
    {  418, 3629,  130741, 17, }, // Atlantic/Cape_Verde
    {  418, 1379,     448,  5, }, // Atlantic/Faroe
    {  418, 7235,  130758, 17, }, // Atlantic/South_Georgia
    {  418, 6784,  130775, 11, }, // Atlantic/St_Helena
    {  418, 1654,  130786,  8, }, // Australia/Adelaide
    {  418, 1619,  130794,  8, }, // Australia/Brisbane
    {  418, 1567,  130802,  6, }, // Australia/Darwin
    {  418, 1500,  130808,  9, }, // Australia/Hobart
    {  418, 1711,  130817,  9, }, // Australia/Melbourne
    {  418, 1746,  130826,  5, }, // Australia/Perth
    {  418, 1447,  130831,  9, }, // Australia/Sydney
    {  418, 6390,  130840, 11, }, // Europe/Amsterdam
    {  418, 6149,  130851,  7, }, // Europe/Andorra
    {  418, 6577,  130858,  5, }, // Europe/Athens
    {  418, 4659,  130863,  7, }, // Europe/Berlin
    {  418, 6178,  130870,  6, }, // Europe/Brussels
    {  418, 3708,  130876,  8, }, // Europe/Budapest
    {  418, 6238,  130884, 11, }, // Europe/Copenhagen
    {  418, 2260,  130895,  8, }, // Europe/Dublin
    {  418, 6561,  130903,  6, }, // Europe/Helsinki
    {  418, 8783,  130909, 15, }, // Europe/Isle_of_Man
    {  418, 1097,  130924,  7, }, // Europe/Istanbul
    {  418, 2419,     463,  4, }, // Europe/Kyiv
    {  418, 3078,  130931,  6, }, // Europe/Lisbon
    {  418, 2393,  130937,  6, }, // Europe/London
    {  418, 6314,  130943,  7, }, // Europe/Luxembourg
    {  418, 4794,  130950,  6, }, // Europe/Madrid
    {  418, 6346,  130956,  6, }, // Europe/Malta
    {  418, 6359,  130962,  6, }, // Europe/Monaco
    {  418, 3443,  130968,  7, }, // Europe/Moscow
    {  418, 6407,  130975,  5, }, // Europe/Oslo
    {  418, 4186,  119889,  4, }, // Europe/Paris
    {  418, 6224,  130980,  6, }, // Europe/Prague
    {  418, 6273,  130986,  4, }, // Europe/Rome
    {  418, 6419,  130990, 11, }, // Europe/San_Marino
    {  418, 6472,  131001,  7, }, // Europe/Stockholm
    {  418, 6516,  131008, 14, }, // Europe/Vatican
    {  418, 6164,  131022,  6, }, // Europe/Vienna
    {  418, 6489,  131028,  6, }, // Europe/Zurich
    {  418, 5979,  131034, 10, }, // Indian/Christmas
    {  418, 5996,  131044,  6, }, // Indian/Cocos
    {  418, 4931,   21395,  6, }, // Indian/Comoro
    {  418, 7013,  131050,  8, }, // Indian/Maldives
    {  418, 3971,  131058,  6, }, // Indian/Mauritius
    {  418, 7174,     467,  7, }, // Indian/Reunion
    {  418, 4324,  131064,  4, }, // Pacific/Apia
    {  418,  932,  131068, 15, }, // Pacific/Auckland
    {  418, 2863,  131083,  6, }, // Pacific/Chatham
    {  418, 3022,     474,  5, }, // Pacific/Chuuk
    {  418, 3804,  131089,  5, }, // Pacific/Fiji
    {  418, 5935,  131094,  5, }, // Pacific/Guam
    {  418, 4070,  131099,  6, }, // Pacific/Norfolk
    {  418, 7061,  131105,  5, }, // Pacific/Noumea
    {  418, 2991,  131110, 11, }, // Pacific/Pago_Pago
    {  418, 7118,  131121,  5, }, // Pacific/Palau
    {  418, 2961,     479,  7, }, // Pacific/Pohnpei
    {  418, 4709,  131126, 12, }, // Pacific/Port_Moresby
    {  418, 7368,  131138,  6, }, // Pacific/Wallis
    {  419,   14,       0,  6, }, // Africa/Asmara Mapuche/Latin/Chile
    {  419, 4337,       6,  8, }, // Africa/Sao_Tome
    {  419, 4154,      14,  8, }, // America/Asuncion
    {  419,  237,      22,  8, }, // America/Atikokan
    {  419, 7586,      30, 17, }, // America/Bahia_Banderas
    {  419, 3770,      47,  6, }, // America/Cancun
    {  419, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  419, 5640,      66,  7, }, // America/Curacao
    {  419,  481,      73, 13, }, // America/Indiana/Knox
    {  419, 7924,      86, 16, }, // America/Indiana/Marengo
    {  419, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  419, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  419, 8001,     139, 14, }, // America/Indiana/Vevay
    {  419, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  419, 8049,     171, 16, }, // America/Indiana/Winamac
    {  419, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  419, 5849,     207, 22, }, // America/Lower_Princes
    {  419, 8183,     229,  6, }, // America/Merida
    {  419, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  419, 8292,     255, 20, }, // America/North_Dakota/Center
    {  419, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  419,  393,     298,  4, }, // America/Nuuk
    {  419, 6862,     302, 16, }, // America/Scoresbysund
    {  419, 8471,     318, 14, }, // America/St_Barthelemy
    {  419, 2045,     332, 10, }, // America/St_Johns
    {  419, 5780,     342,  9, }, // America/St_Kitts
    {  419, 5797,     351,  9, }, // America/St_Lucia
    {  419,  855,     360, 10, }, // America/St_Thomas
    {  419, 5830,     370, 11, }, // America/St_Vincent
    {  419, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  419, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  419, 1152,     413,  9, }, // Asia/Kathmandu
    {  419,  992,     422,  7, }, // Asia/Kolkata
    {  419, 1178,     429,  5, }, // Asia/Macau
    {  419, 1202,     442,  6, }, // Asia/Yangon
    {  419, 1379,     448,  5, }, // Atlantic/Faroe
    {  419, 6784,     453, 10, }, // Atlantic/St_Helena
    {  419, 2419,     463,  4, }, // Europe/Kyiv
    {  419, 7174,     467,  7, }, // Indian/Reunion
    {  419, 3022,     474,  5, }, // Pacific/Chuuk
    {  419, 2961,     479,  7, }, // Pacific/Pohnpei
    {  420, 6751,  131144,  8, }, // Africa/Abidjan Marathi/Devanagari/India
    {  420, 6723,  131152,  7, }, // Africa/Accra
    {  420, 4961,  131159, 10, }, // Africa/Addis_Ababa
    {  420, 7383,  131169,  9, }, // Africa/Algiers
    {  420,   14,  131178,  7, }, // Africa/Asmara
    {  420,   44,  131185,  5, }, // Africa/Bamako
    {  420, 5159,  131190,  6, }, // Africa/Bangui
    {  420, 6709,  124617,  5, }, // Africa/Banjul
    {  420, 7398,  108983,  5, }, // Africa/Bissau
    {  420, 4859,  131196,  8, }, // Africa/Blantyre
    {  420, 5189,  131204, 12, }, // Africa/Brazzaville
    {  420, 4824,  131216,  9, }, // Africa/Bujumbura
    {  420, 2242,  131225,  4, }, // Africa/Cairo
    {  420, 4019,  131229, 12, }, // Africa/Casablanca
    {  420, 7412,   86849,  5, }, // Africa/Ceuta
    {  420, 6736,  131241,  8, }, // Africa/Conakry
    {  420, 6803,  131249,  3, }, // Africa/Dakar
    {  420, 5032,  131252, 10, }, // Africa/Dar_es_Salaam
    {  420, 4945,   28563,  6, }, // Africa/Djibouti
    {  420, 5145,  131262,  6, }, // Africa/Douala
    {  420, 5068,  131268,  6, }, // Africa/El_Aaiun
    {  420, 6816,   28581,  8, }, // Africa/Freetown
    {  420, 4808,   86905,  7, }, // Africa/Gaborone
    {  420, 4917,   86912,  5, }, // Africa/Harare
    {  420, 4381,  109054, 11, }, // Africa/Johannesburg
    {  420, 4401,   28613,  4, }, // Africa/Juba
    {  420, 5053,   86927,  6, }, // Africa/Kampala
    {  420, 4426,  131274,  7, }, // Africa/Khartoum
    {  420, 4875,  131281,  6, }, // Africa/Kigali
    {  420, 5208,   28638,  8, }, // Africa/Kinshasa
    {  420, 4646,   86946,  5, }, // Africa/Lagos
    {  420, 5238,  131287,  9, }, // Africa/Libreville
    {  420, 6832,   86961,  3, }, // Africa/Lome
    {  420, 5113,   86964,  6, }, // Africa/Luanda
    {  420, 4841,  131296,  8, }, // Africa/Lubumbashi
    {  420, 4903,   28680,  6, }, // Africa/Lusaka
    {  420, 5224,   86979,  5, }, // Africa/Malabo
    {  420, 4889,   86984,  6, }, // Africa/Maputo
    {  420, 5099,  131304,  5, }, // Africa/Maseru
    {  420, 5084,  131309,  8, }, // Africa/Mbabane
    {  420, 5015,   86997,  8, }, // Africa/Mogadishu
    {  420, 6997,  131317, 11, }, // Africa/Monrovia
    {  420, 3755,   87014,  6, }, // Africa/Nairobi
    {  420, 5173,  131328,  8, }, // Africa/Ndjamena
    {  420, 5256,  124729,  6, }, // Africa/Niamey
    {  420, 6766,  131336,  8, }, // Africa/Nouakchott
    {  420, 6690,  131344,  7, }, // Africa/Ouagadougou
    {  420, 5127,  131351, 13, }, // Africa/Porto-Novo
    {  420, 4337,  131364,  8, }, // Africa/Sao_Tome
    {  420, 2732,   87068,  8, }, // Africa/Tripoli
    {  420, 6503,  131372,  6, }, // Africa/Tunis
    {  420, 4054,  131378,  8, }, // Africa/Windhoek
    {  420,  132,  131386,  4, }, // America/Adak
    {  420, 3183,  131390,  6, }, // America/Anchorage
    {  420, 5524,  131396,  8, }, // America/Anguilla
    {  420, 5541,  131404,  8, }, // America/Antigua
    {  420, 4456,  131412,  8, }, // America/Araguaina
    {  420,  166,  131420, 13, }, // America/Argentina/Buenos_Aires
    {  420,   91,  131433, 10, }, // America/Argentina/Catamarca
    {  420,  270,  131443,  8, }, // America/Argentina/Cordoba
    {  420,  441,  124781,  5, }, // America/Argentina/Jujuy
    {  420, 7425,  109190,  9, }, // America/Argentina/La_Rioja
    {  420,  565,  131451,  7, }, // America/Argentina/Mendoza
    {  420, 7452,  131458, 12, }, // America/Argentina/Rio_Gallegos
    {  420, 7483,  131470,  6, }, // America/Argentina/Salta
    {  420, 7507,  109218,  8, }, // America/Argentina/San_Juan
    {  420, 5497,  131476, 10, }, // America/Argentina/San_Luis
    {  420, 7534,   28919,  7, }, // America/Argentina/Tucuman
    {  420, 7560,  131486,  8, }, // America/Argentina/Ushuaia
    {  420, 5557,  109233,  5, }, // America/Aruba
    {  420, 4154,  131494, 10, }, // America/Asuncion
    {  420,  237,  131504,  8, }, // America/Atikokan
    {  420, 3581,   87212,  5, }, // America/Bahia
    {  420, 7586,  131512, 15, }, // America/Bahia_Banderas
    {  420, 5571,  131527,  9, }, // America/Barbados
    {  420, 7609,   28985,  5, }, // America/Belem
    {  420, 5309,  131536,  6, }, // America/Belize
    {  420, 7623,  131542, 13, }, // America/Blanc-Sablon
    {  420, 7644,  131555, 12, }, // America/Boa_Vista
    {  420, 4263,   87269,  6, }, // America/Bogota
    {  420, 7662,  131567,  5, }, // America/Boise
    {  420, 7676,  131572, 15, }, // America/Cambridge_Bay
    {  420, 7698,  131587, 13, }, // America/Campo_Grande
    {  420, 3770,  109322,  6, }, // America/Cancun
    {  420, 4596,  131600,  6, }, // America/Caracas
    {  420, 4247,  131606,  6, }, // America/Cayenne
    {  420, 5398,  131612,  4, }, // America/Cayman
    {  420, 3251,   87320,  6, }, // America/Chicago
    {  420, 7719,   87326,  8, }, // America/Chihuahua
    {  420, 7737,  131616, 13, }, // America/Ciudad_Juarez
    {  420, 5324,   87350, 11, }, // America/Costa_Rica
    {  420, 7759,   29101,  8, }, // America/Creston
    {  420, 3693,  124922,  6, }, // America/Cuiaba
    {  420, 5640,  131629,  9, }, // America/Curacao
    {  420, 7775,  109376, 12, }, // America/Danmarkshavn
    {  420, 7796,   87386,  4, }, // America/Dawson
    {  420, 7811,   87390, 10, }, // America/Dawson_Creek
    {  420,  805,  131638,  8, }, // America/Denver
    {  420, 3351,  131646,  8, }, // America/Detroit
    {  420, 5656,   87413,  8, }, // America/Dominica
    {  420,  893,  131654,  7, }, // America/Edmonton
    {  420, 7832,  131661,  8, }, // America/Eirunepe
    {  420, 5343,  109432, 12, }, // America/El_Salvador
    {  420, 7849,  109444, 12, }, // America/Fort_Nelson
    {  420, 7869,  109456, 10, }, // America/Fortaleza
    {  420, 7887,  131669, 12, }, // America/Glace_Bay
    {  420, 6844,  131681, 10, }, // America/Goose_Bay
    {  420, 4514,  131691, 10, }, // America/Grand_Turk
    {  420, 5687,  131701,  8, }, // America/Grenada
    {  420, 5703,  131709, 11, }, // America/Guadeloupe
    {  420, 3662,   29265, 10, }, // America/Guatemala
    {  420, 4743,  131720,  8, }, // America/Guayaquil
    {  420, 6895,   87512,  5, }, // America/Guyana
    {  420, 1923,  131728,  9, }, // America/Halifax
    {  420, 2221,   87527,  5, }, // America/Havana
    {  420, 7905,  109509,  9, }, // America/Hermosillo
    {  420,  348,   87543, 13, }, // America/Indiana/Indianapolis
    {  420,  481,  109531, 15, }, // America/Indiana/Knox
    {  420, 7924,   87571, 17, }, // America/Indiana/Marengo
    {  420, 7948,   87588, 20, }, // America/Indiana/Petersburg
    {  420, 7975,   87608, 18, }, // America/Indiana/Tell_City
    {  420, 8001,  131737, 14, }, // America/Indiana/Vevay
    {  420, 8023,  109580, 17, }, // America/Indiana/Vincennes
    {  420, 8049,  131751, 16, }, // America/Indiana/Winamac
    {  420, 8073,  109614,  6, }, // America/Inuvik
    {  420,  660,  131767,  7, }, // America/Iqaluit
    {  420, 2665,   87682,  5, }, // America/Jamaica
    {  420, 5282,  131774,  6, }, // America/Juneau
    {  420,  521,  131780, 11, }, // America/Kentucky/Louisville
    {  420, 8088,  131791, 18, }, // America/Kentucky/Monticello
    {  420, 5621,  131809, 12, }, // America/Kralendijk
    {  420, 4278,  109669,  6, }, // America/La_Paz
    {  420, 7132,   29523,  4, }, // America/Lima
    {  420, 3390,  131821, 11, }, // America/Los_Angeles
    {  420, 5849,  131832, 22, }, // America/Lower_Princes
    {  420, 8116,  131854,  6, }, // America/Maceio
    {  420, 8131,   87776,  7, }, // America/Managua
    {  420, 1892,   87783,  4, }, // America/Manaus
    {  420, 5814,  131860,  7, }, // America/Marigot
    {  420, 5722,   87794,  9, }, // America/Martinique
    {  420, 8147,   87803,  9, }, // America/Matamoros
    {  420, 2779,  131867,  8, }, // America/Mazatlan
    {  420, 8165,   87821,  8, }, // America/Menominee
    {  420, 8183,   29617,  6, }, // America/Merida
    {  420, 8198,  131875,  9, }, // America/Metlakatla
    {  420, 2811,   87841, 13, }, // America/Mexico_City
    {  420, 4293,  109765,  9, }, // America/Miquelon
    {  420, 8217,  131884,  6, }, // America/Moncton
    {  420, 8233,  131890,  8, }, // America/Monterrey
    {  420, 4000,  131898, 14, }, // America/Montevideo
    {  420, 5741,  131912, 11, }, // America/Montserrat
    {  420, 5383,  109812,  4, }, // America/Nassau
    {  420, 3294,   87901,  9, }, // America/New_York
    {  420, 8251,   87823,  3, }, // America/Nome
    {  420, 1834,   87910,  8, }, // America/Noronha
    {  420, 8264,  131923, 20, }, // America/North_Dakota/Beulah
    {  420, 8292,  131943, 17, }, // America/North_Dakota/Center
    {  420, 8320,  131960, 23, }, // America/North_Dakota/New_Salem
    {  420,  393,   29774,  3, }, // America/Nuuk
    {  420, 8351,  109895,  7, }, // America/Ojinaga
    {  420, 5413,   87993,  5, }, // America/Panama
    {  420, 6106,  131983,  8, }, // America/Paramaribo
    {  420, 3224,  131991,  7, }, // America/Phoenix
    {  420, 3847,  131998, 14, }, // America/Port-au-Prince
    {  420, 5871,  109933, 14, }, // America/Port_of_Spain
    {  420, 8367,   88045, 13, }, // America/Porto_Velho
    {  420, 5760,  132012, 14, }, // America/Puerto_Rico
    {  420, 3932,  132026, 12, }, // America/Punta_Arenas
    {  420, 8387,  132038, 12, }, // America/Rankin_Inlet
    {  420, 8408,   29884,  6, }, // America/Recife
    {  420, 1979,   29890,  6, }, // America/Regina
    {  420, 8423,  132050,  9, }, // America/Resolute
    {  420,  695,   88109, 12, }, // America/Rio_Branco
    {  420, 8440,  132059,  7, }, // America/Santarem
    {  420, 2165,  132066,  9, }, // America/Santiago
    {  420, 6044,  132075, 14, }, // America/Santo_Domingo
    {  420, 1862,  110030,  9, }, // America/Sao_Paulo
    {  420, 6862,  110039, 13, }, // America/Scoresbysund
    {  420, 8457,  132089,  5, }, // America/Sitka
    {  420, 8471,   88179, 15, }, // America/St_Barthelemy
    {  420, 2045,  110052, 10, }, // America/St_Johns
    {  420, 5780,   88203, 10, }, // America/St_Kitts
    {  420, 5797,  132094, 11, }, // America/St_Lucia
    {  420,  855,   88224,  9, }, // America/St_Thomas
    {  420, 5830,  110073, 13, }, // America/St_Vincent
    {  420, 8493,  110086, 12, }, // America/Swift_Current
    {  420, 5363,  132105, 12, }, // America/Tegucigalpa
    {  420, 5673,  110109,  4, }, // America/Thule
    {  420,  313,   88275,  7, }, // America/Tijuana
    {  420,  608,  132117,  7, }, // America/Toronto
    {  420, 5605,   88288,  8, }, // America/Tortola
    {  420, 2077,  132124, 11, }, // America/Vancouver
    {  420, 2128,  132135, 11, }, // America/Whitehorse
    {  420,  734,   88314,  7, }, // America/Winnipeg
    {  420, 8515,  132146,  6, }, // America/Yakutat
    {  420, 5918,  132152,  4, }, // Antarctica/Casey
    {  420, 6027,  132156,  7, }, // Antarctica/Davis
    {  420, 6066,  132163, 19, }, // Antarctica/DumontDUrville
    {  420, 8531,  132182,  8, }, // Antarctica/Macquarie
    {  420, 7029,   88359,  4, }, // Antarctica/Mawson
    {  420, 7076,  132190,  9, }, // Antarctica/McMurdo
    {  420, 8552,  132199,  5, }, // Antarctica/Palmer
    {  420, 7189,   88376,  6, }, // Antarctica/Rothera
    {  420, 7258,   88382,  6, }, // Antarctica/Syowa
    {  420, 8570,   88388,  5, }, // Antarctica/Troll
    {  420, 7337,  132204,  7, }, // Antarctica/Vostok
    {  420, 1413,  132211, 12, }, // Arctic/Longyearbyen
    {  420, 5487,   30214,  4, }, // Asia/Aden
    {  420, 5297,   88417,  7, }, // Asia/Almaty
    {  420, 3870,   88424,  6, }, // Asia/Amman
    {  420, 5428,  132223,  7, }, // Asia/Anadyr
    {  420, 5440,  132230,  7, }, // Asia/Aqtau
    {  420, 5270,  132237,  9, }, // Asia/Aqtobe
    {  420,  964,   30250,  8, }, // Asia/Ashgabat
    {  420, 8587,  132246,  5, }, // Asia/Atyrau
    {  420, 3509,   88447,  5, }, // Asia/Baghdad
    {  420, 5451,  132251,  6, }, // Asia/Bahrain
    {  420, 3555,   87366,  4, }, // Asia/Baku
    {  420, 4368,  110253,  5, }, // Asia/Bangkok
    {  420, 3473,  132257,  6, }, // Asia/Barnaul
    {  420, 3988,   30290,  5, }, // Asia/Beirut
    {  420, 3680,   30295,  7, }, // Asia/Bishkek
    {  420, 5906,  132263,  7, }, // Asia/Brunei
    {  420, 4503,  125608,  4, }, // Asia/Chita
    {  420, 5963,  110272,  9, }, // Asia/Choibalsan
    {  420, 4413,   88495,  6, }, // Asia/Colombo
    {  420, 4442,   88501,  7, }, // Asia/Damascus
    {  420, 1060,   88508,  4, }, // Asia/Dhaka
    {  420, 6125,   88512,  4, }, // Asia/Dili
    {  420, 3498,   88516,  4, }, // Asia/Dubai
    {  420, 6092,   30349,  8, }, // Asia/Dushanbe
    {  420, 8599,  132270,  9, }, // Asia/Famagusta
    {  420, 8614,  110304,  4, }, // Asia/Gaza
    {  420, 4697,  110308,  7, }, // Asia/Hebron
    {  420, 1226,  132279, 15, }, // Asia/Ho_Chi_Minh
    {  420, 2576,  132294,  8, }, // Asia/Hong_Kong
    {  420, 4673,   88575,  5, }, // Asia/Hovd
    {  420, 4086,   30393, 10, }, // Asia/Irkutsk
    {  420, 6969,   88590,  7, }, // Asia/Jakarta
    {  420, 6955,   30411,  7, }, // Asia/Jayapura
    {  420, 1257,  132302,  8, }, // Asia/Jerusalem
    {  420, 3462,   30427,  5, }, // Asia/Kabul
    {  420, 4218,  132310,  8, }, // Asia/Kamchatka
    {  420, 4141,   30440,  5, }, // Asia/Karachi
    {  420, 1152,   88615,  8, }, // Asia/Kathmandu
    {  420, 8624,  132318,  6, }, // Asia/Khandyga
    {  420,  992,   88630,  7, }, // Asia/Kolkata
    {  420, 4099,   88637, 15, }, // Asia/Krasnoyarsk
    {  420, 4761,  132324, 11, }, // Asia/Kuala_Lumpur
    {  420, 5893,  125709,  6, }, // Asia/Kuching
    {  420, 5464,  110381,  5, }, // Asia/Kuwait
    {  420, 1178,   88662,  4, }, // Asia/Macau
    {  420, 3919,  132335,  6, }, // Asia/Magadan
    {  420, 1316,  132341,  7, }, // Asia/Makassar
    {  420, 7145,  110404,  5, }, // Asia/Manila
    {  420, 6883,   88811,  5, }, // Asia/Muscat
    {  420, 2446,   88677,  8, }, // Asia/Nicosia
    {  420, 8638,  132348, 15, }, // Asia/Novokuznetsk
    {  420, 4037,   88701, 13, }, // Asia/Novosibirsk
    {  420, 4131,   88714,  6, }, // Asia/Omsk
    {  420, 7108,   88720,  3, }, // Asia/Oral
    {  420, 6924,  132363, 11, }, // Asia/Phnom_Penh
    {  420, 8656,  132374, 10, }, // Asia/Pontianak
    {  420, 4116,   88741, 10, }, // Asia/Pyongyang
    {  420, 5476,  110444,  4, }, // Asia/Qatar
    {  420, 8671,   88754,  8, }, // Asia/Qostanay
    {  420, 4171,  132384, 10, }, // Asia/Qyzylorda
    {  420, 3486,   30628,  5, }, // Asia/Riyadh
    {  420, 4310,   30633,  7, }, // Asia/Sakhalin
    {  420, 7208,   88784,  6, }, // Asia/Samarkand
    {  420, 3116,  132394,  4, }, // Asia/Seoul
    {  420, 1020,  110474,  6, }, // Asia/Shanghai
    {  420, 3137,  110480,  8, }, // Asia/Singapore
    {  420, 4199,  132398, 16, }, // Asia/Srednekolymsk
    {  420, 3100,  110504,  5, }, // Asia/Taipei
    {  420, 4683,  110509,  7, }, // Asia/Tashkent
    {  420, 3817,   30698,  6, }, // Asia/Tbilisi
    {  420, 2638,   30704,  6, }, // Asia/Tehran
    {  420, 1284,  132414,  5, }, // Asia/Thimphu
    {  420, 2687,  132419,  6, }, // Asia/Tokyo
    {  420, 4474,  132425,  5, }, // Asia/Tomsk
    {  420, 1346,   88852,  8, }, // Asia/Ulaanbaatar
    {  420, 1126,  110549,  7, }, // Asia/Urumqi
    {  420, 8685,  125853,  9, }, // Asia/Ust-Nera
    {  420, 6940,  132430, 11, }, // Asia/Vientiane
    {  420, 4612,  132441, 13, }, // Asia/Vladivostok
    {  420, 4730,  110576,  8, }, // Asia/Yakutsk
    {  420, 1202,   30782,  5, }, // Asia/Yangon
    {  420, 3785,  132454, 13, }, // Asia/Yekaterinburg
    {  420, 3649,  132467,  6, }, // Asia/Yerevan
    {  420, 3565,  110590,  6, }, // Atlantic/Azores
    {  420, 5588,  110596,  7, }, // Atlantic/Bermuda
    {  420, 6591,  110603,  5, }, // Atlantic/Canary
    {  420, 3629,  132473, 11, }, // Atlantic/Cape_Verde
    {  420, 1379,  132484,  4, }, // Atlantic/Faroe
    {  420, 8699,  132488,  6, }, // Atlantic/Madeira
    {  420, 2614,  132494, 10, }, // Atlantic/Reykjavik
    {  420, 7235,   88965, 15, }, // Atlantic/South_Georgia
    {  420, 6784,  132504, 12, }, // Atlantic/St_Helena
    {  420, 6607,  132516,  7, }, // Atlantic/Stanley
    {  420, 1654,  132523,  5, }, // Australia/Adelaide
    {  420, 1619,  110654,  9, }, // Australia/Brisbane
    {  420, 1783,   89012, 10, }, // Australia/Broken_Hill
    {  420, 1567,   89022,  7, }, // Australia/Darwin
    {  420, 3539,  110663,  5, }, // Australia/Eucla
    {  420, 1500,   89029,  7, }, // Australia/Hobart
    {  420, 8716,  132528,  7, }, // Australia/Lindeman
    {  420, 1531,   89044, 10, }, // Australia/Lord_Howe
    {  420, 1711,   89054,  8, }, // Australia/Melbourne
    {  420, 1746,   89062,  4, }, // Australia/Perth
    {  420, 1447,   30962,  5, }, // Australia/Sydney
    {  420, 6390,  132535, 11, }, // Europe/Amsterdam
    {  420, 6149,  132546,  6, }, // Europe/Andorra
    {  420, 3522,   30984, 10, }, // Europe/Astrakhan
    {  420, 6577,  110700,  6, }, // Europe/Athens
    {  420, 6285,   31000,  8, }, // Europe/Belgrade
    {  420, 4659,   89097,  6, }, // Europe/Berlin
    {  420, 6437,  132552, 14, }, // Europe/Bratislava
    {  420, 6178,  110720,  9, }, // Europe/Brussels
    {  420, 3830,  110729,  9, }, // Europe/Bucharest
    {  420, 3708,   89134,  9, }, // Europe/Budapest
    {  420, 8751,  110738,  8, }, // Europe/Busingen
    {  420, 2475,  132566,  6, }, // Europe/Chisinau
    {  420, 6238,   89160,  9, }, // Europe/Copenhagen
    {  420, 2260,  110761,  6, }, // Europe/Dublin
    {  420, 6256,   89174, 10, }, // Europe/Gibraltar
    {  420, 8767,  132572,  9, }, // Europe/Guernsey
    {  420, 6561,   89190,  8, }, // Europe/Helsinki
    {  420, 8783,  132581, 10, }, // Europe/Isle_of_Man
    {  420, 1097,  110778,  8, }, // Europe/Istanbul
    {  420, 8802,   89218,  5, }, // Europe/Jersey
    {  420, 3881,  132591, 11, }, // Europe/Kaliningrad
    {  420, 8924,   89235,  5, }, // Europe/Kirov
    {  420, 2419,   89240,  3, }, // Europe/Kyiv
    {  420, 3078,   31158,  6, }, // Europe/Lisbon
    {  420, 6455,  132602, 10, }, // Europe/Ljubljana
    {  420, 2393,  110807,  4, }, // Europe/London
    {  420, 6314,  132612, 10, }, // Europe/Luxembourg
    {  420, 4794,   31192,  7, }, // Europe/Madrid
    {  420, 6346,   31199,  6, }, // Europe/Malta
    {  420, 6531,  132622, 10, }, // Europe/Mariehamn
    {  420, 3595,   31212,  7, }, // Europe/Minsk
    {  420, 6359,  110837,  6, }, // Europe/Monaco
    {  420, 3443,   89296,  6, }, // Europe/Moscow
    {  420, 6407,  132632,  5, }, // Europe/Oslo
    {  420, 4186,  110843,  5, }, // Europe/Paris
    {  420, 6373,  110848,  9, }, // Europe/Podgorica
    {  420, 6224,   89317,  5, }, // Europe/Prague
    {  420, 8816,   31252,  4, }, // Europe/Riga
    {  420, 6273,   88949,  3, }, // Europe/Rome
    {  420, 4233,   89326,  5, }, // Europe/Samara
    {  420, 6419,  132637,  9, }, // Europe/San_Marino
    {  420, 6194,  132646, 10, }, // Europe/Sarajevo
    {  420, 4353,   89349,  7, }, // Europe/Saratov
    {  420, 8828,  132656, 10, }, // Europe/Simferopol
    {  420, 6332,  132666,  8, }, // Europe/Skopje
    {  420, 6548,  110891,  6, }, // Europe/Sofia
    {  420, 6472,   89383,  8, }, // Europe/Stockholm
    {  420, 8846,  132674,  5, }, // Europe/Tallinn
    {  420, 6135,  126257,  6, }, // Europe/Tirane
    {  420, 8861,   89402, 12, }, // Europe/Ulyanovsk
    {  420, 6301,  132679,  4, }, // Europe/Vaduz
    {  420, 6516,  132683,  8, }, // Europe/Vatican
    {  420, 6164,  132691,  9, }, // Europe/Vienna
    {  420, 8878,  132700,  7, }, // Europe/Vilnius
    {  420, 4629,  110924, 13, }, // Europe/Volgograd
    {  420, 3055,  132707,  6, }, // Europe/Warsaw
    {  420, 6210,  132713,  7, }, // Europe/Zagreb
    {  420, 6489,  132720,  5, }, // Europe/Zurich
    {  420, 4980,  110954, 12, }, // Indian/Antananarivo
    {  420, 6910,  110966,  5, }, // Indian/Chagos
    {  420, 5979,  132725,  7, }, // Indian/Christmas
    {  420, 5996,   89495,  5, }, // Indian/Cocos
    {  420, 4931,   89500,  6, }, // Indian/Comoro
    {  420, 6624,  132732,  9, }, // Indian/Kerguelen
    {  420, 7223,   31451,  4, }, // Indian/Mahe
    {  420, 7013,   89513,  6, }, // Indian/Maldives
    {  420, 3971,  110979,  6, }, // Indian/Mauritius
    {  420, 5000,  132741,  8, }, // Indian/Mayotte
    {  420, 7174,  110990,  8, }, // Indian/Reunion
    {  420, 4324,  110998,  5, }, // Pacific/Apia
    {  420,  932,  132749,  5, }, // Pacific/Auckland
    {  420, 3608,  132754, 10, }, // Pacific/Bougainville
    {  420, 2863,  132764,  6, }, // Pacific/Chatham
    {  420, 3022,   31513,  3, }, // Pacific/Chuuk
    {  420, 2201,   89566,  5, }, // Pacific/Easter
    {  420, 7323,  111024,  4, }, // Pacific/Efate
    {  420, 7290,  111028,  7, }, // Pacific/Fakaofo
    {  420, 3804,   31537,  4, }, // Pacific/Fiji
    {  420, 7306,  132770,  8, }, // Pacific/Funafuti
    {  420, 6641,  132778,  9, }, // Pacific/Galapagos
    {  420, 6659,  132787,  8, }, // Pacific/Gambier
    {  420, 3724,   89618, 10, }, // Pacific/Guadalcanal
    {  420, 5935,   29571,  4, }, // Pacific/Guam
    {  420, 2929,  132795,  8, }, // Pacific/Honolulu
    {  420, 2897,   89636,  7, }, // Pacific/Kanton
    {  420, 3900,  111079, 10, }, // Pacific/Kiritimati
    {  420, 6982,  132803,  5, }, // Pacific/Kosrae
    {  420, 2708,  132808, 10, }, // Pacific/Kwajalein
    {  420, 4779,  132818,  5, }, // Pacific/Majuro
    {  420, 3953,  132823, 10, }, // Pacific/Marquesas
    {  420, 8909,   89673,  5, }, // Pacific/Midway
    {  420, 7047,  132833,  4, }, // Pacific/Nauru
    {  420, 7095,  132837,  5, }, // Pacific/Niue
    {  420, 4070,  111115,  6, }, // Pacific/Norfolk
    {  420, 7061,  132842,  5, }, // Pacific/Noumea
    {  420, 2991,  132847,  9, }, // Pacific/Pago_Pago
    {  420, 7118,   89708,  4, }, // Pacific/Palau
    {  420, 7157,  132856,  8, }, // Pacific/Pitcairn
    {  420, 2961,   89712,  6, }, // Pacific/Pohnpei
    {  420, 4709,  132864, 14, }, // Pacific/Port_Moresby
    {  420, 6009,   89732,  9, }, // Pacific/Rarotonga
    {  420, 5948,  132878,  5, }, // Pacific/Saipan
    {  420, 7275,   89747,  6, }, // Pacific/Tahiti
    {  420, 6675,   31722,  6, }, // Pacific/Tarawa
    {  420, 4485,   89759,  9, }, // Pacific/Tongatapu
    {  420, 7355,   31736,  3, }, // Pacific/Wake
    {  420, 7368,   31739,  5, }, // Pacific/Wallis
    {  421,   14,       0,  6, }, // Africa/Asmara Masai/Latin/Kenya
    {  421, 4337,       6,  8, }, // Africa/Sao_Tome
    {  421, 4154,      14,  8, }, // America/Asuncion
    {  421,  237,      22,  8, }, // America/Atikokan
    {  421, 7586,      30, 17, }, // America/Bahia_Banderas
    {  421, 3770,      47,  6, }, // America/Cancun
    {  421, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  421, 5640,      66,  7, }, // America/Curacao
    {  421,  481,      73, 13, }, // America/Indiana/Knox
    {  421, 7924,      86, 16, }, // America/Indiana/Marengo
    {  421, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  421, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  421, 8001,     139, 14, }, // America/Indiana/Vevay
    {  421, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  421, 8049,     171, 16, }, // America/Indiana/Winamac
    {  421, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  421, 5849,     207, 22, }, // America/Lower_Princes
    {  421, 8183,     229,  6, }, // America/Merida
    {  421, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  421, 8292,     255, 20, }, // America/North_Dakota/Center
    {  421, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  421,  393,     298,  4, }, // America/Nuuk
    {  421, 6862,     302, 16, }, // America/Scoresbysund
    {  421, 8471,     318, 14, }, // America/St_Barthelemy
    {  421, 2045,     332, 10, }, // America/St_Johns
    {  421, 5780,     342,  9, }, // America/St_Kitts
    {  421, 5797,     351,  9, }, // America/St_Lucia
    {  421,  855,     360, 10, }, // America/St_Thomas
    {  421, 5830,     370, 11, }, // America/St_Vincent
    {  421, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  421, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  421, 1152,     413,  9, }, // Asia/Kathmandu
    {  421,  992,     422,  7, }, // Asia/Kolkata
    {  421, 1178,     429,  5, }, // Asia/Macau
    {  421, 1202,     442,  6, }, // Asia/Yangon
    {  421, 1379,     448,  5, }, // Atlantic/Faroe
    {  421, 6784,     453, 10, }, // Atlantic/St_Helena
    {  421, 2419,     463,  4, }, // Europe/Kyiv
    {  421, 7174,     467,  7, }, // Indian/Reunion
    {  421, 3022,     474,  5, }, // Pacific/Chuuk
    {  421, 2961,     479,  7, }, // Pacific/Pohnpei
    {  423,   14,       0,  6, }, // Africa/Asmara Mazanderani/Arabic/Iran
    {  423, 4337,       6,  8, }, // Africa/Sao_Tome
    {  423, 4154,      14,  8, }, // America/Asuncion
    {  423,  237,      22,  8, }, // America/Atikokan
    {  423, 7586,      30, 17, }, // America/Bahia_Banderas
    {  423, 3770,      47,  6, }, // America/Cancun
    {  423, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  423, 5640,      66,  7, }, // America/Curacao
    {  423,  481,      73, 13, }, // America/Indiana/Knox
    {  423, 7924,      86, 16, }, // America/Indiana/Marengo
    {  423, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  423, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  423, 8001,     139, 14, }, // America/Indiana/Vevay
    {  423, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  423, 8049,     171, 16, }, // America/Indiana/Winamac
    {  423, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  423, 5849,     207, 22, }, // America/Lower_Princes
    {  423, 8183,     229,  6, }, // America/Merida
    {  423, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  423, 8292,     255, 20, }, // America/North_Dakota/Center
    {  423, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  423,  393,     298,  4, }, // America/Nuuk
    {  423, 6862,     302, 16, }, // America/Scoresbysund
    {  423, 8471,     318, 14, }, // America/St_Barthelemy
    {  423, 2045,     332, 10, }, // America/St_Johns
    {  423, 5780,     342,  9, }, // America/St_Kitts
    {  423, 5797,     351,  9, }, // America/St_Lucia
    {  423,  855,     360, 10, }, // America/St_Thomas
    {  423, 5830,     370, 11, }, // America/St_Vincent
    {  423, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  423, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  423, 1152,     413,  9, }, // Asia/Kathmandu
    {  423,  992,     422,  7, }, // Asia/Kolkata
    {  423, 1178,     429,  5, }, // Asia/Macau
    {  423, 1202,     442,  6, }, // Asia/Yangon
    {  423, 1379,     448,  5, }, // Atlantic/Faroe
    {  423, 6784,     453, 10, }, // Atlantic/St_Helena
    {  423, 2419,     463,  4, }, // Europe/Kyiv
    {  423, 7174,     467,  7, }, // Indian/Reunion
    {  423, 3022,     474,  5, }, // Pacific/Chuuk
    {  423, 2961,     479,  7, }, // Pacific/Pohnpei
    {  424,   14,       0,  6, }, // Africa/Asmara Meru/Latin/Kenya
    {  424, 4337,       6,  8, }, // Africa/Sao_Tome
    {  424, 4154,      14,  8, }, // America/Asuncion
    {  424,  237,      22,  8, }, // America/Atikokan
    {  424, 7586,      30, 17, }, // America/Bahia_Banderas
    {  424, 3770,      47,  6, }, // America/Cancun
    {  424, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  424, 5640,      66,  7, }, // America/Curacao
    {  424,  481,      73, 13, }, // America/Indiana/Knox
    {  424, 7924,      86, 16, }, // America/Indiana/Marengo
    {  424, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  424, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  424, 8001,     139, 14, }, // America/Indiana/Vevay
    {  424, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  424, 8049,     171, 16, }, // America/Indiana/Winamac
    {  424, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  424, 5849,     207, 22, }, // America/Lower_Princes
    {  424, 8183,     229,  6, }, // America/Merida
    {  424, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  424, 8292,     255, 20, }, // America/North_Dakota/Center
    {  424, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  424,  393,     298,  4, }, // America/Nuuk
    {  424, 6862,     302, 16, }, // America/Scoresbysund
    {  424, 8471,     318, 14, }, // America/St_Barthelemy
    {  424, 2045,     332, 10, }, // America/St_Johns
    {  424, 5780,     342,  9, }, // America/St_Kitts
    {  424, 5797,     351,  9, }, // America/St_Lucia
    {  424,  855,     360, 10, }, // America/St_Thomas
    {  424, 5830,     370, 11, }, // America/St_Vincent
    {  424, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  424, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  424, 1152,     413,  9, }, // Asia/Kathmandu
    {  424,  992,     422,  7, }, // Asia/Kolkata
    {  424, 1178,     429,  5, }, // Asia/Macau
    {  424, 1202,     442,  6, }, // Asia/Yangon
    {  424, 1379,     448,  5, }, // Atlantic/Faroe
    {  424, 6784,     453, 10, }, // Atlantic/St_Helena
    {  424, 2419,     463,  4, }, // Europe/Kyiv
    {  424, 7174,     467,  7, }, // Indian/Reunion
    {  424, 3022,     474,  5, }, // Pacific/Chuuk
    {  424, 2961,     479,  7, }, // Pacific/Pohnpei
    {  425,   14,       0,  6, }, // Africa/Asmara Meta/Latin/Cameroon
    {  425, 4337,       6,  8, }, // Africa/Sao_Tome
    {  425, 4154,      14,  8, }, // America/Asuncion
    {  425,  237,      22,  8, }, // America/Atikokan
    {  425, 7586,      30, 17, }, // America/Bahia_Banderas
    {  425, 3770,      47,  6, }, // America/Cancun
    {  425, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  425, 5640,      66,  7, }, // America/Curacao
    {  425,  481,      73, 13, }, // America/Indiana/Knox
    {  425, 7924,      86, 16, }, // America/Indiana/Marengo
    {  425, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  425, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  425, 8001,     139, 14, }, // America/Indiana/Vevay
    {  425, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  425, 8049,     171, 16, }, // America/Indiana/Winamac
    {  425, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  425, 5849,     207, 22, }, // America/Lower_Princes
    {  425, 8183,     229,  6, }, // America/Merida
    {  425, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  425, 8292,     255, 20, }, // America/North_Dakota/Center
    {  425, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  425,  393,     298,  4, }, // America/Nuuk
    {  425, 6862,     302, 16, }, // America/Scoresbysund
    {  425, 8471,     318, 14, }, // America/St_Barthelemy
    {  425, 2045,     332, 10, }, // America/St_Johns
    {  425, 5780,     342,  9, }, // America/St_Kitts
    {  425, 5797,     351,  9, }, // America/St_Lucia
    {  425,  855,     360, 10, }, // America/St_Thomas
    {  425, 5830,     370, 11, }, // America/St_Vincent
    {  425, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  425, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  425, 1152,     413,  9, }, // Asia/Kathmandu
    {  425,  992,     422,  7, }, // Asia/Kolkata
    {  425, 1178,     429,  5, }, // Asia/Macau
    {  425, 1202,     442,  6, }, // Asia/Yangon
    {  425, 1379,     448,  5, }, // Atlantic/Faroe
    {  425, 6784,     453, 10, }, // Atlantic/St_Helena
    {  425, 2419,     463,  4, }, // Europe/Kyiv
    {  425, 7174,     467,  7, }, // Indian/Reunion
    {  425, 3022,     474,  5, }, // Pacific/Chuuk
    {  425, 2961,     479,  7, }, // Pacific/Pohnpei
    {  426,   14,       0,  6, }, // Africa/Asmara Mohawk/Latin/Canada
    {  426, 4337,       6,  8, }, // Africa/Sao_Tome
    {  426, 4154,      14,  8, }, // America/Asuncion
    {  426,  237,      22,  8, }, // America/Atikokan
    {  426, 7586,      30, 17, }, // America/Bahia_Banderas
    {  426, 3770,      47,  6, }, // America/Cancun
    {  426, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  426, 5640,      66,  7, }, // America/Curacao
    {  426,  481,      73, 13, }, // America/Indiana/Knox
    {  426, 7924,      86, 16, }, // America/Indiana/Marengo
    {  426, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  426, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  426, 8001,     139, 14, }, // America/Indiana/Vevay
    {  426, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  426, 8049,     171, 16, }, // America/Indiana/Winamac
    {  426, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  426, 5849,     207, 22, }, // America/Lower_Princes
    {  426, 8183,     229,  6, }, // America/Merida
    {  426, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  426, 8292,     255, 20, }, // America/North_Dakota/Center
    {  426, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  426,  393,     298,  4, }, // America/Nuuk
    {  426, 6862,     302, 16, }, // America/Scoresbysund
    {  426, 8471,     318, 14, }, // America/St_Barthelemy
    {  426, 2045,     332, 10, }, // America/St_Johns
    {  426, 5780,     342,  9, }, // America/St_Kitts
    {  426, 5797,     351,  9, }, // America/St_Lucia
    {  426,  855,     360, 10, }, // America/St_Thomas
    {  426, 5830,     370, 11, }, // America/St_Vincent
    {  426, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  426, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  426, 1152,     413,  9, }, // Asia/Kathmandu
    {  426,  992,     422,  7, }, // Asia/Kolkata
    {  426, 1178,     429,  5, }, // Asia/Macau
    {  426, 1202,     442,  6, }, // Asia/Yangon
    {  426, 1379,     448,  5, }, // Atlantic/Faroe
    {  426, 6784,     453, 10, }, // Atlantic/St_Helena
    {  426, 2419,     463,  4, }, // Europe/Kyiv
    {  426, 7174,     467,  7, }, // Indian/Reunion
    {  426, 3022,     474,  5, }, // Pacific/Chuuk
    {  426, 2961,     479,  7, }, // Pacific/Pohnpei
    {  427, 6751,  132883,  6, }, // Africa/Abidjan Mongolian/Cyrillic/Mongolia
    {  427, 6723,     492,  5, }, // Africa/Accra
    {  427, 4961,  132889, 11, }, // Africa/Addis_Ababa
    {  427, 7383,     508,  5, }, // Africa/Algiers
    {  427,   14,   25647,  6, }, // Africa/Asmara
    {  427,   44,     519,  6, }, // Africa/Bamako
    {  427, 5159,  132900,  6, }, // Africa/Bangui
    {  427, 6709,     530,  6, }, // Africa/Banjul
    {  427, 7398,  132906,  6, }, // Africa/Bissau
    {  427, 4859,   25670,  8, }, // Africa/Blantyre
    {  427, 5189,   50080, 10, }, // Africa/Brazzaville
    {  427, 4824,     558,  9, }, // Africa/Bujumbura
    {  427, 2242,     567,  4, }, // Africa/Cairo
    {  427, 4019,     571, 10, }, // Africa/Casablanca
    {  427, 7412,     581,  5, }, // Africa/Ceuta
    {  427, 6736,     586,  7, }, // Africa/Conakry
    {  427, 6803,     593,  5, }, // Africa/Dakar
    {  427, 5032,   25698, 12, }, // Africa/Dar_es_Salaam
    {  427, 4945,  113714,  6, }, // Africa/Djibouti
    {  427, 5145,     616,  5, }, // Africa/Douala
    {  427, 5068,   25717,  7, }, // Africa/El_Aaiun
    {  427, 6816,     628,  7, }, // Africa/Freetown
    {  427, 4808,     635,  8, }, // Africa/Gaborone
    {  427, 4917,   32384,  6, }, // Africa/Harare
    {  427, 4381,   50098, 12, }, // Africa/Johannesburg
    {  427, 4401,  113727,  4, }, // Africa/Juba
    {  427, 5053,     665,  7, }, // Africa/Kampala
    {  427, 4426,     672,  6, }, // Africa/Khartoum
    {  427, 4875,     678,  6, }, // Africa/Kigali
    {  427, 5208,     684,  7, }, // Africa/Kinshasa
    {  427, 4646,     691,  5, }, // Africa/Lagos
    {  427, 5238,   50110,  9, }, // Africa/Libreville
    {  427, 6832,     704,  4, }, // Africa/Lome
    {  427, 5113,     708,  6, }, // Africa/Luanda
    {  427, 4841,     714,  9, }, // Africa/Lubumbashi
    {  427, 4903,     723,  6, }, // Africa/Lusaka
    {  427, 5224,     729,  6, }, // Africa/Malabo
    {  427, 4889,   32416,  6, }, // Africa/Maputo
    {  427, 5099,     741,  6, }, // Africa/Maseru
    {  427, 5084,     747,  7, }, // Africa/Mbabane
    {  427, 5015,   34891,  8, }, // Africa/Mogadishu
    {  427, 6997,     762,  8, }, // Africa/Monrovia
    {  427, 3755,   34907,  7, }, // Africa/Nairobi
    {  427, 5173,  113740,  7, }, // Africa/Ndjamena
    {  427, 5256,   34914,  6, }, // Africa/Niamey
    {  427, 6766,     790,  7, }, // Africa/Nouakchott
    {  427, 6690,     797,  8, }, // Africa/Ouagadougou
    {  427, 5127,     805, 10, }, // Africa/Porto-Novo
    {  427, 4337,     815,  8, }, // Africa/Sao_Tome
    {  427, 2732,     823,  7, }, // Africa/Tripoli
    {  427, 6503,     830,  5, }, // Africa/Tunis
    {  427, 4054,     835,  7, }, // Africa/Windhoek
    {  427,  132,     842,  4, }, // America/Adak
    {  427, 3183,  132912,  7, }, // America/Anchorage
    {  427, 5524,   50127,  7, }, // America/Anguilla
    {  427, 5541,     860,  7, }, // America/Antigua
    {  427, 4456,  132919,  9, }, // America/Araguaina
    {  427,  166,   50134, 12, }, // America/Argentina/Buenos_Aires
    {  427,   91,     888,  9, }, // America/Argentina/Catamarca
    {  427,  270,     897,  7, }, // America/Argentina/Cordoba
    {  427,  441,   25937,  5, }, // America/Argentina/Jujuy
    {  427, 7425,     909,  8, }, // America/Argentina/La_Rioja
    {  427,  565,   32526,  7, }, // America/Argentina/Mendoza
    {  427, 7452,   50146, 12, }, // America/Argentina/Rio_Gallegos
    {  427, 7483,     935,  5, }, // America/Argentina/Salta
    {  427, 7507,     940,  8, }, // America/Argentina/San_Juan
    {  427, 5497,   32552,  7, }, // America/Argentina/San_Luis
    {  427, 7534,     956,  7, }, // America/Argentina/Tucuman
    {  427, 7560,  104836,  6, }, // America/Argentina/Ushuaia
    {  427, 5557,     969,  5, }, // America/Aruba
    {  427, 4154,   50158,  8, }, // America/Asuncion
    {  427,  237,   32574,  8, }, // America/Atikokan
    {  427, 3581,  132928,  4, }, // America/Bahia
    {  427, 7586,  132932, 14, }, // America/Bahia_Banderas
    {  427, 5571,    1013,  8, }, // America/Barbados
    {  427, 7609,   32601,  5, }, // America/Belem
    {  427, 5309,   32606,  6, }, // America/Belize
    {  427, 7623,   35000, 11, }, // America/Blanc-Sablon
    {  427, 7644,    1043,  9, }, // America/Boa_Vista
    {  427, 4263,    1052,  6, }, // America/Bogota
    {  427, 7662,    1058,  5, }, // America/Boise
    {  427, 7676,  132946, 16, }, // America/Cambridge_Bay
    {  427, 7698,  132962, 12, }, // America/Campo_Grande
    {  427, 3770,    1087,  6, }, // America/Cancun
    {  427, 4596,    1093,  7, }, // America/Caracas
    {  427, 4247,   50200,  7, }, // America/Cayenne
    {  427, 5398,   26079,  6, }, // America/Cayman
    {  427, 3251,    1129,  6, }, // America/Chicago
    {  427, 7719,  113814,  8, }, // America/Chihuahua
    {  427, 7737,  132974, 13, }, // America/Ciudad_Juarez
    {  427, 5324,    1141, 10, }, // America/Costa_Rica
    {  427, 7759,    1151,  7, }, // America/Creston
    {  427, 3693,   26138,  5, }, // America/Cuiaba
    {  427, 5640,   35086,  7, }, // America/Curacao
    {  427, 7775,  132987, 11, }, // America/Danmarkshavn
    {  427, 7796,    1184,  6, }, // America/Dawson
    {  427, 7811,  132998, 11, }, // America/Dawson_Creek
    {  427,  805,    1201,  6, }, // America/Denver
    {  427, 3351,   35129,  7, }, // America/Detroit
    {  427, 5656,    1214,  8, }, // America/Dominica
    {  427,  893,   50207,  8, }, // America/Edmonton
    {  427, 7832,   50215,  8, }, // America/Eirunepe
    {  427, 5343,  133009, 13, }, // America/El_Salvador
    {  427, 7849,  133022, 12, }, // America/Fort_Nelson
    {  427, 7869,    1257,  9, }, // America/Fortaleza
    {  427, 7887,  133034, 11, }, // America/Glace_Bay
    {  427, 6844,  133045, 10, }, // America/Goose_Bay
    {  427, 4514,   32749, 10, }, // America/Grand_Turk
    {  427, 5687,    1292,  7, }, // America/Grenada
    {  427, 5703,    1299,  8, }, // America/Guadeloupe
    {  427, 3662,    1308,  8, }, // America/Guatemala
    {  427, 4743,   50251,  8, }, // America/Guayaquil
    {  427, 6895,   50259,  6, }, // America/Guyana
    {  427, 1923,   32781,  8, }, // America/Halifax
    {  427, 2221,   32789,  6, }, // America/Havana
    {  427, 7905,  133055, 10, }, // America/Hermosillo
    {  427,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {  427,  481,    1365, 13, }, // America/Indiana/Knox
    {  427, 7924,    1378, 16, }, // America/Indiana/Marengo
    {  427, 7948,  113892, 19, }, // America/Indiana/Petersburg
    {  427, 7975,  133065, 17, }, // America/Indiana/Tell_City
    {  427, 8001,  104904, 14, }, // America/Indiana/Vevay
    {  427, 8023,  133082, 17, }, // America/Indiana/Vincennes
    {  427, 8049,  133099, 16, }, // America/Indiana/Winamac
    {  427, 8073,    1452,  6, }, // America/Inuvik
    {  427,  660,    1458,  7, }, // America/Iqaluit
    {  427, 2665,   26461,  6, }, // America/Jamaica
    {  427, 5282,   32939,  4, }, // America/Juneau
    {  427,  521,    1476,  8, }, // America/Kentucky/Louisville
    {  427, 8088,  133115, 19, }, // America/Kentucky/Monticello
    {  427, 5621,  133134,  9, }, // America/Kralendijk
    {  427, 4278,   32971,  6, }, // America/La_Paz
    {  427, 7132,    1520,  4, }, // America/Lima
    {  427, 3390,  113972, 11, }, // America/Los_Angeles
    {  427, 5849,  133143, 21, }, // America/Lower_Princes
    {  427, 8116,   35279,  6, }, // America/Maceio
    {  427, 8131,    1559,  7, }, // America/Managua
    {  427, 1892,    1566,  6, }, // America/Manaus
    {  427, 5814,  133164,  7, }, // America/Marigot
    {  427, 5722,    1578,  8, }, // America/Martinique
    {  427, 8147,    1587,  9, }, // America/Matamoros
    {  427, 2779,   33020,  8, }, // America/Mazatlan
    {  427, 8165,    1604,  8, }, // America/Menominee
    {  427, 8183,    1612,  6, }, // America/Merida
    {  427, 8198,    1618, 10, }, // America/Metlakatla
    {  427, 2811,    1628,  6, }, // America/Mexico_City
    {  427, 4293,    1634,  7, }, // America/Miquelon
    {  427, 8217,    1641,  7, }, // America/Moncton
    {  427, 8233,   35285,  8, }, // America/Monterrey
    {  427, 4000,    1657, 10, }, // America/Montevideo
    {  427, 5741,    1667, 10, }, // America/Montserrat
    {  427, 5383,    1677,  6, }, // America/Nassau
    {  427, 3294,   50338,  8, }, // America/New_York
    {  427, 8251,    1691,  3, }, // America/Nome
    {  427, 1834,  133171,  6, }, // America/Noronha
    {  427, 8264,  133177, 17, }, // America/North_Dakota/Beulah
    {  427, 8292,  133194, 16, }, // America/North_Dakota/Center
    {  427, 8320,  133210, 22, }, // America/North_Dakota/New_Salem
    {  427,  393,  133232,  4, }, // America/Nuuk
    {  427, 8351,  133236,  7, }, // America/Ojinaga
    {  427, 5413,    1783,  6, }, // America/Panama
    {  427, 6106,    1789, 10, }, // America/Paramaribo
    {  427, 3224,   33144,  6, }, // America/Phoenix
    {  427, 3847,  133243, 12, }, // America/Port-au-Prince
    {  427, 5871,  133255, 13, }, // America/Port_of_Spain
    {  427, 8367,  133268, 11, }, // America/Porto_Velho
    {  427, 5760,   50414, 11, }, // America/Puerto_Rico
    {  427, 3932,   33182, 11, }, // America/Punta_Arenas
    {  427, 8387,  114114, 12, }, // America/Rankin_Inlet
    {  427, 8408,    1876,  6, }, // America/Recife
    {  427, 1979,   33212,  6, }, // America/Regina
    {  427, 8423,   35372,  7, }, // America/Resolute
    {  427,  695,  133279, 10, }, // America/Rio_Branco
    {  427, 8440,   33235,  8, }, // America/Santarem
    {  427, 2165,   50433,  8, }, // America/Santiago
    {  427, 6044,   33252, 13, }, // America/Santo_Domingo
    {  427, 1862,    1936,  9, }, // America/Sao_Paulo
    {  427, 6862,    1945, 11, }, // America/Scoresbysund
    {  427, 8457,    1956,  5, }, // America/Sitka
    {  427, 8471,  133289, 14, }, // America/St_Barthelemy
    {  427, 2045,  133303,  9, }, // America/St_Johns
    {  427, 5780,  133312, 10, }, // America/St_Kitts
    {  427, 5797,  133322, 10, }, // America/St_Lucia
    {  427,  855,  124240, 10, }, // America/St_Thomas
    {  427, 5830,   33334, 12, }, // America/St_Vincent
    {  427, 8493,  114183, 13, }, // America/Swift_Current
    {  427, 5363,   50475, 12, }, // America/Tegucigalpa
    {  427, 5673,    2049,  4, }, // America/Thule
    {  427,  313,    2053,  7, }, // America/Tijuana
    {  427,  608,    2060,  7, }, // America/Toronto
    {  427, 5605,    2067,  7, }, // America/Tortola
    {  427, 2077,    2074,  8, }, // America/Vancouver
    {  427, 2128,  133332,  8, }, // America/Whitehorse
    {  427,  734,    2090,  8, }, // America/Winnipeg
    {  427, 8515,   27032,  6, }, // America/Yakutat
    {  427, 5918,  133340,  5, }, // Antarctica/Casey
    {  427, 6027,  114204,  5, }, // Antarctica/Davis
    {  427, 6066,  133345, 14, }, // Antarctica/DumontDUrville
    {  427, 8531,    2131,  8, }, // Antarctica/Macquarie
    {  427, 7029,    2139,  6, }, // Antarctica/Mawson
    {  427, 7076,  133359,  9, }, // Antarctica/McMurdo
    {  427, 8552,    2154,  6, }, // Antarctica/Palmer
    {  427, 7189,    2160,  6, }, // Antarctica/Rothera
    {  427, 7258,   27092,  4, }, // Antarctica/Syowa
    {  427, 8570,    2171,  5, }, // Antarctica/Troll
    {  427, 7337,   33428,  6, }, // Antarctica/Vostok
    {  427, 1413,  133368, 11, }, // Arctic/Longyearbyen
    {  427, 5487,    2192,  4, }, // Asia/Aden
    {  427, 5297,    2196,  6, }, // Asia/Almaty
    {  427, 3870,    2202,  5, }, // Asia/Amman
    {  427, 5428,    2207,  6, }, // Asia/Anadyr
    {  427, 5440,    2213,  5, }, // Asia/Aqtau
    {  427, 5270,  133379,  6, }, // Asia/Aqtobe
    {  427,  964,    2224,  7, }, // Asia/Ashgabat
    {  427, 8587,    2231,  6, }, // Asia/Atyrau
    {  427, 3509,    2237,  6, }, // Asia/Baghdad
    {  427, 5451,   35560,  7, }, // Asia/Bahrain
    {  427, 3555,    2250,  4, }, // Asia/Baku
    {  427, 4368,    2254,  7, }, // Asia/Bangkok
    {  427, 3473,    2261,  7, }, // Asia/Barnaul
    {  427, 3988,   27145,  6, }, // Asia/Beirut
    {  427, 3680,    2274,  6, }, // Asia/Bishkek
    {  427, 5906,   27157,  6, }, // Asia/Brunei
    {  427, 4503,    2286,  4, }, // Asia/Chita
    {  427, 5963,   35567,  9, }, // Asia/Choibalsan
    {  427, 4413,    2299,  7, }, // Asia/Colombo
    {  427, 4442,    2306,  6, }, // Asia/Damascus
    {  427, 1060,     593,  4, }, // Asia/Dhaka
    {  427, 6125,    2317,  4, }, // Asia/Dili
    {  427, 3498,   27187,  5, }, // Asia/Dubai
    {  427, 6092,    2326,  7, }, // Asia/Dushanbe
    {  427, 8599,    2333,  9, }, // Asia/Famagusta
    {  427, 8614,    2342,  4, }, // Asia/Gaza
    {  427, 4697,   33498,  6, }, // Asia/Hebron
    {  427, 1226,  133385, 13, }, // Asia/Ho_Chi_Minh
    {  427, 2576,   33513,  9, }, // Asia/Hong_Kong
    {  427, 4673,    2366,  4, }, // Asia/Hovd
    {  427, 4086,  133398,  5, }, // Asia/Irkutsk
    {  427, 6969,  114245,  7, }, // Asia/Jakarta
    {  427, 6955,  133403,  6, }, // Asia/Jayapura
    {  427, 1257,  133409,  8, }, // Asia/Jerusalem
    {  427, 3462,    2401,  5, }, // Asia/Kabul
    {  427, 4218,    2420,  8, }, // Asia/Kamchatka
    {  427, 4141,    2431,  6, }, // Asia/Karachi
    {  427, 1152,    2437,  8, }, // Asia/Kathmandu
    {  427, 8624,  114260,  7, }, // Asia/Khandyga
    {  427,  992,   50608,  9, }, // Asia/Kolkata
    {  427, 4099,   35599, 10, }, // Asia/Krasnoyarsk
    {  427, 4761,   33570, 12, }, // Asia/Kuala_Lumpur
    {  427, 5893,    2483,  5, }, // Asia/Kuching
    {  427, 5464,   27284,  6, }, // Asia/Kuwait
    {  427, 1178,    2495,  5, }, // Asia/Macau
    {  427, 3919,    2500,  7, }, // Asia/Magadan
    {  427, 1316,  114272,  8, }, // Asia/Makassar
    {  427, 7145,    2514,  6, }, // Asia/Manila
    {  427, 6883,   33593,  6, }, // Asia/Muscat
    {  427, 2446,  105053,  7, }, // Asia/Nicosia
    {  427, 8638,    2533, 11, }, // Asia/Novokuznetsk
    {  427, 4037,    2544, 11, }, // Asia/Novosibirsk
    {  427, 4131,    2555,  4, }, // Asia/Omsk
    {  427, 7108,   33618,  4, }, // Asia/Oral
    {  427, 6924,   50617,  8, }, // Asia/Phnom_Penh
    {  427, 8656,    2572,  9, }, // Asia/Pontianak
    {  427, 4116,  133417,  6, }, // Asia/Pyongyang
    {  427, 5476,    2588,  5, }, // Asia/Qatar
    {  427, 8671,   35626,  8, }, // Asia/Qostanay
    {  427, 4171,    2601,  8, }, // Asia/Qyzylorda
    {  427, 3486,   35643,  4, }, // Asia/Riyadh
    {  427, 4310,    2623,  7, }, // Asia/Sakhalin
    {  427, 7208,    2630,  9, }, // Asia/Samarkand
    {  427, 3116,  133423,  4, }, // Asia/Seoul
    {  427, 1020,   27392,  6, }, // Asia/Shanghai
    {  427, 3137,    2649,  8, }, // Asia/Singapore
    {  427, 4199,    2657, 13, }, // Asia/Srednekolymsk
    {  427, 3100,   59565,  6, }, // Asia/Taipei
    {  427, 4683,    2676,  7, }, // Asia/Tashkent
    {  427, 3817,   33686,  7, }, // Asia/Tbilisi
    {  427, 2638,    2687,  7, }, // Asia/Tehran
    {  427, 1284,    2694,  7, }, // Asia/Thimphu
    {  427, 2687,    2701,  5, }, // Asia/Tokyo
    {  427, 4474,    2706,  5, }, // Asia/Tomsk
    {  427, 1346,  133427, 11, }, // Asia/Ulaanbaatar
    {  427, 1126,  133438,  6, }, // Asia/Urumqi
    {  427, 8685,    2727,  8, }, // Asia/Ust-Nera
    {  427, 6940,   50660,  8, }, // Asia/Vientiane
    {  427, 4612,    2742, 11, }, // Asia/Vladivostok
    {  427, 4730,   35672,  6, }, // Asia/Yakutsk
    {  427, 1202,   27502,  6, }, // Asia/Yangon
    {  427, 3785,    2766, 12, }, // Asia/Yekaterinburg
    {  427, 3649,    2778,  6, }, // Asia/Yerevan
    {  427, 3565,    2784,  4, }, // Atlantic/Azores
    {  427, 5588,   33764,  7, }, // Atlantic/Bermuda
    {  427, 6591,  114299,  6, }, // Atlantic/Canary
    {  427, 3629,    2835, 10, }, // Atlantic/Cape_Verde
    {  427, 1379,    2845,  5, }, // Atlantic/Faroe
    {  427, 8699,   35753,  7, }, // Atlantic/Madeira
    {  427, 2614,   50740,  9, }, // Atlantic/Reykjavik
    {  427, 7235,  133444, 12, }, // Atlantic/South_Georgia
    {  427, 6784,  133456, 11, }, // Atlantic/St_Helena
    {  427, 6607,   50782,  6, }, // Atlantic/Stanley
    {  427, 1654,    2926,  8, }, // Australia/Adelaide
    {  427, 1619,    2934,  7, }, // Australia/Brisbane
    {  427, 1783,  133467, 11, }, // Australia/Broken_Hill
    {  427, 1567,    2952,  6, }, // Australia/Darwin
    {  427, 3539,  133478,  5, }, // Australia/Eucla
    {  427, 1500,    2963,  6, }, // Australia/Hobart
    {  427, 8716,  133483,  8, }, // Australia/Lindeman
    {  427, 1531,  133491,  8, }, // Australia/Lord_Howe
    {  427, 1711,   27711,  8, }, // Australia/Melbourne
    {  427, 1746,  133499,  4, }, // Australia/Perth
    {  427, 1447,   50802,  6, }, // Australia/Sydney
    {  427, 6390,    3002,  9, }, // Europe/Amsterdam
    {  427, 6149,    3011,  7, }, // Europe/Andorra
    {  427, 3522,   27740,  9, }, // Europe/Astrakhan
    {  427, 6577,    3026,  4, }, // Europe/Athens
    {  427, 6285,    3030,  7, }, // Europe/Belgrade
    {  427, 4659,    3037,  6, }, // Europe/Berlin
    {  427, 6437,    3043,  9, }, // Europe/Bratislava
    {  427, 6178,   50822,  8, }, // Europe/Brussels
    {  427, 3830,    3061,  8, }, // Europe/Bucharest
    {  427, 3708,    3069,  8, }, // Europe/Budapest
    {  427, 8751,   33917,  8, }, // Europe/Busingen
    {  427, 2475,  133503,  7, }, // Europe/Chisinau
    {  427, 6238,   50849, 10, }, // Europe/Copenhagen
    {  427, 2260,    3117,  6, }, // Europe/Dublin
    {  427, 6256,    3123,  9, }, // Europe/Gibraltar
    {  427, 8767,    3132,  6, }, // Europe/Guernsey
    {  427, 6561,   50859,  8, }, // Europe/Helsinki
    {  427, 8783,  133510,  8, }, // Europe/Isle_of_Man
    {  427, 1097,   33964,  8, }, // Europe/Istanbul
    {  427, 8802,  114357,  5, }, // Europe/Jersey
    {  427, 3881,    3167, 11, }, // Europe/Kaliningrad
    {  427, 8924,    3178,  5, }, // Europe/Kirov
    {  427, 2419,    3183,  4, }, // Europe/Kyiv
    {  427, 3078,    3187,  8, }, // Europe/Lisbon
    {  427, 6455,   27891,  7, }, // Europe/Ljubljana
    {  427, 2393,    3204,  6, }, // Europe/London
    {  427, 6314,   27904, 10, }, // Europe/Luxembourg
    {  427, 4794,    3221,  6, }, // Europe/Madrid
    {  427, 6346,   27920,  6, }, // Europe/Malta
    {  427, 6531,   34012,  8, }, // Europe/Mariehamn
    {  427, 3595,    3241,  5, }, // Europe/Minsk
    {  427, 6359,    3246,  6, }, // Europe/Monaco
    {  427, 3443,    3252,  6, }, // Europe/Moscow
    {  427, 6407,    3258,  4, }, // Europe/Oslo
    {  427, 4186,  133518,  5, }, // Europe/Paris
    {  427, 6373,    3267,  9, }, // Europe/Podgorica
    {  427, 6224,    3276,  5, }, // Europe/Prague
    {  427, 8816,    3281,  4, }, // Europe/Riga
    {  427, 6273,  133523,  3, }, // Europe/Rome
    {  427, 4233,    3288,  6, }, // Europe/Samara
    {  427, 6419,    3294, 10, }, // Europe/San_Marino
    {  427, 6194,    3304,  7, }, // Europe/Sarajevo
    {  427, 4353,    3311,  7, }, // Europe/Saratov
    {  427, 8828,   50883, 11, }, // Europe/Simferopol
    {  427, 6332,   50894,  6, }, // Europe/Skopje
    {  427, 6548,   35907,  5, }, // Europe/Sofia
    {  427, 6472,  133526,  8, }, // Europe/Stockholm
    {  427, 8846,    3346,  6, }, // Europe/Tallinn
    {  427, 6135,    3352,  6, }, // Europe/Tirane
    {  427, 8861,   50909,  9, }, // Europe/Ulyanovsk
    {  427, 6301,   34086,  5, }, // Europe/Vaduz
    {  427, 6516,    3372,  7, }, // Europe/Vatican
    {  427, 6164,    3379,  4, }, // Europe/Vienna
    {  427, 8878,   50918,  7, }, // Europe/Vilnius
    {  427, 4629,    3390,  9, }, // Europe/Volgograd
    {  427, 3055,    3399,  6, }, // Europe/Warsaw
    {  427, 6210,    3406,  6, }, // Europe/Zagreb
    {  427, 6489,   35931,  5, }, // Europe/Zurich
    {  427, 4980,   34105, 12, }, // Indian/Antananarivo
    {  427, 6910,    3430,  5, }, // Indian/Chagos
    {  427, 5979,  114373,  7, }, // Indian/Christmas
    {  427, 5996,    3447,  5, }, // Indian/Cocos
    {  427, 4931,   34122,  6, }, // Indian/Comoro
    {  427, 6624,    3469,  8, }, // Indian/Kerguelen
    {  427, 7223,   34128,  4, }, // Indian/Mahe
    {  427, 7013,   50990,  7, }, // Indian/Maldives
    {  427, 3971,   55228,  8, }, // Indian/Mauritius
    {  427, 5000,   55236,  7, }, // Indian/Mayotte
    {  427, 7174,   35990,  7, }, // Indian/Reunion
    {  427, 4324,    3507,  4, }, // Pacific/Apia
    {  427,  932,  133534,  7, }, // Pacific/Auckland
    {  427, 3608,   51013,  9, }, // Pacific/Bougainville
    {  427, 2863,    3525,  5, }, // Pacific/Chatham
    {  427, 3022,  133541,  4, }, // Pacific/Chuuk
    {  427, 2201,  114390,  5, }, // Pacific/Easter
    {  427, 7323,   51067,  5, }, // Pacific/Efate
    {  427, 7290,    3550,  7, }, // Pacific/Fakaofo
    {  427, 3804,  114395,  4, }, // Pacific/Fiji
    {  427, 7306,    3561,  8, }, // Pacific/Funafuti
    {  427, 6641,    3569,  9, }, // Pacific/Galapagos
    {  427, 6659,  133545,  7, }, // Pacific/Gambier
    {  427, 3724,    3604, 11, }, // Pacific/Guadalcanal
    {  427, 5935,    3615,  4, }, // Pacific/Guam
    {  427, 2929,   34210,  8, }, // Pacific/Honolulu
    {  427, 2897,   51116,  9, }, // Pacific/Kanton
    {  427, 3900,    3625, 10, }, // Pacific/Kiritimati
    {  427, 6982,  133552,  5, }, // Pacific/Kosrae
    {  427, 2708,  133557,  9, }, // Pacific/Kwajalein
    {  427, 4779,  114414,  6, }, // Pacific/Majuro
    {  427, 3953,  114420,  6, }, // Pacific/Marquesas
    {  427, 8909,   51148,  6, }, // Pacific/Midway
    {  427, 7047,    3689,  5, }, // Pacific/Nauru
    {  427, 7095,   51162,  4, }, // Pacific/Niue
    {  427, 4070,    3698,  7, }, // Pacific/Norfolk
    {  427, 7061,    3705,  5, }, // Pacific/Noumea
    {  427, 2991,   34254,  9, }, // Pacific/Pago_Pago
    {  427, 7118,    3719,  5, }, // Pacific/Palau
    {  427, 7157,   55309,  7, }, // Pacific/Pitcairn
    {  427, 2961,   36092,  6, }, // Pacific/Pohnpei
    {  427, 4709,    3737, 11, }, // Pacific/Port_Moresby
    {  427, 6009,    3748,  9, }, // Pacific/Rarotonga
    {  427, 5948,   28394,  6, }, // Pacific/Saipan
    {  427, 7275,    3763,  5, }, // Pacific/Tahiti
    {  427, 6675,    3768,  6, }, // Pacific/Tarawa
    {  427, 4485,    3774,  9, }, // Pacific/Tongatapu
    {  427, 7355,  133566,  3, }, // Pacific/Wake
    {  427, 7368,    3787,  6, }, // Pacific/Wallis
    {  428,   14,       0,  6, }, // Africa/Asmara Mongolian/Mongolian/China
    {  428, 4337,       6,  8, }, // Africa/Sao_Tome
    {  428, 4154,      14,  8, }, // America/Asuncion
    {  428,  237,      22,  8, }, // America/Atikokan
    {  428, 7586,      30, 17, }, // America/Bahia_Banderas
    {  428, 3770,      47,  6, }, // America/Cancun
    {  428, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  428, 5640,      66,  7, }, // America/Curacao
    {  428,  481,      73, 13, }, // America/Indiana/Knox
    {  428, 7924,      86, 16, }, // America/Indiana/Marengo
    {  428, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  428, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  428, 8001,     139, 14, }, // America/Indiana/Vevay
    {  428, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  428, 8049,     171, 16, }, // America/Indiana/Winamac
    {  428, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  428, 5849,     207, 22, }, // America/Lower_Princes
    {  428, 8183,     229,  6, }, // America/Merida
    {  428, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  428, 8292,     255, 20, }, // America/North_Dakota/Center
    {  428, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  428,  393,     298,  4, }, // America/Nuuk
    {  428, 6862,     302, 16, }, // America/Scoresbysund
    {  428, 8471,     318, 14, }, // America/St_Barthelemy
    {  428, 2045,     332, 10, }, // America/St_Johns
    {  428, 5780,     342,  9, }, // America/St_Kitts
    {  428, 5797,     351,  9, }, // America/St_Lucia
    {  428,  855,     360, 10, }, // America/St_Thomas
    {  428, 5830,     370, 11, }, // America/St_Vincent
    {  428, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  428, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  428, 1152,     413,  9, }, // Asia/Kathmandu
    {  428,  992,     422,  7, }, // Asia/Kolkata
    {  428, 1178,     429,  5, }, // Asia/Macau
    {  428, 1202,     442,  6, }, // Asia/Yangon
    {  428, 1379,     448,  5, }, // Atlantic/Faroe
    {  428, 6784,     453, 10, }, // Atlantic/St_Helena
    {  428, 2419,     463,  4, }, // Europe/Kyiv
    {  428, 7174,     467,  7, }, // Indian/Reunion
    {  428, 3022,     474,  5, }, // Pacific/Chuuk
    {  428, 2961,     479,  7, }, // Pacific/Pohnpei
    {  429, 5963,  133569,  9, }, // Asia/Choibalsan Mongolian/Mongolian/Mongolia
    {  429, 4673,  133578,  5, }, // Asia/Hovd
    {  429, 1346,  133583, 13, }, // Asia/Ulaanbaatar
    {  430,   14,       0,  6, }, // Africa/Asmara Morisyen/Latin/Mauritius
    {  430, 4337,       6,  8, }, // Africa/Sao_Tome
    {  430, 4154,      14,  8, }, // America/Asuncion
    {  430,  237,      22,  8, }, // America/Atikokan
    {  430, 7586,      30, 17, }, // America/Bahia_Banderas
    {  430, 3770,      47,  6, }, // America/Cancun
    {  430, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  430, 5640,      66,  7, }, // America/Curacao
    {  430,  481,      73, 13, }, // America/Indiana/Knox
    {  430, 7924,      86, 16, }, // America/Indiana/Marengo
    {  430, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  430, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  430, 8001,     139, 14, }, // America/Indiana/Vevay
    {  430, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  430, 8049,     171, 16, }, // America/Indiana/Winamac
    {  430, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  430, 5849,     207, 22, }, // America/Lower_Princes
    {  430, 8183,     229,  6, }, // America/Merida
    {  430, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  430, 8292,     255, 20, }, // America/North_Dakota/Center
    {  430, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  430,  393,     298,  4, }, // America/Nuuk
    {  430, 6862,     302, 16, }, // America/Scoresbysund
    {  430, 8471,     318, 14, }, // America/St_Barthelemy
    {  430, 2045,     332, 10, }, // America/St_Johns
    {  430, 5780,     342,  9, }, // America/St_Kitts
    {  430, 5797,     351,  9, }, // America/St_Lucia
    {  430,  855,     360, 10, }, // America/St_Thomas
    {  430, 5830,     370, 11, }, // America/St_Vincent
    {  430, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  430, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  430, 1152,     413,  9, }, // Asia/Kathmandu
    {  430,  992,     422,  7, }, // Asia/Kolkata
    {  430, 1178,     429,  5, }, // Asia/Macau
    {  430, 1202,     442,  6, }, // Asia/Yangon
    {  430, 1379,     448,  5, }, // Atlantic/Faroe
    {  430, 6784,     453, 10, }, // Atlantic/St_Helena
    {  430, 2419,     463,  4, }, // Europe/Kyiv
    {  430, 7174,     467,  7, }, // Indian/Reunion
    {  430, 3022,     474,  5, }, // Pacific/Chuuk
    {  430, 2961,     479,  7, }, // Pacific/Pohnpei
    {  431,   14,       0,  6, }, // Africa/Asmara Mundang/Latin/Cameroon
    {  431, 4337,       6,  8, }, // Africa/Sao_Tome
    {  431, 4154,      14,  8, }, // America/Asuncion
    {  431,  237,      22,  8, }, // America/Atikokan
    {  431, 7586,      30, 17, }, // America/Bahia_Banderas
    {  431, 3770,      47,  6, }, // America/Cancun
    {  431, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  431, 5640,      66,  7, }, // America/Curacao
    {  431,  481,      73, 13, }, // America/Indiana/Knox
    {  431, 7924,      86, 16, }, // America/Indiana/Marengo
    {  431, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  431, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  431, 8001,     139, 14, }, // America/Indiana/Vevay
    {  431, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  431, 8049,     171, 16, }, // America/Indiana/Winamac
    {  431, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  431, 5849,     207, 22, }, // America/Lower_Princes
    {  431, 8183,     229,  6, }, // America/Merida
    {  431, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  431, 8292,     255, 20, }, // America/North_Dakota/Center
    {  431, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  431,  393,     298,  4, }, // America/Nuuk
    {  431, 6862,     302, 16, }, // America/Scoresbysund
    {  431, 8471,     318, 14, }, // America/St_Barthelemy
    {  431, 2045,     332, 10, }, // America/St_Johns
    {  431, 5780,     342,  9, }, // America/St_Kitts
    {  431, 5797,     351,  9, }, // America/St_Lucia
    {  431,  855,     360, 10, }, // America/St_Thomas
    {  431, 5830,     370, 11, }, // America/St_Vincent
    {  431, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  431, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  431, 1152,     413,  9, }, // Asia/Kathmandu
    {  431,  992,     422,  7, }, // Asia/Kolkata
    {  431, 1178,     429,  5, }, // Asia/Macau
    {  431, 1202,     442,  6, }, // Asia/Yangon
    {  431, 1379,     448,  5, }, // Atlantic/Faroe
    {  431, 6784,     453, 10, }, // Atlantic/St_Helena
    {  431, 2419,     463,  4, }, // Europe/Kyiv
    {  431, 7174,     467,  7, }, // Indian/Reunion
    {  431, 3022,     474,  5, }, // Pacific/Chuuk
    {  431, 2961,     479,  7, }, // Pacific/Pohnpei
    {  432,   14,       0,  6, }, // Africa/Asmara Muscogee/Latin/United States
    {  432, 4337,       6,  8, }, // Africa/Sao_Tome
    {  432, 4154,      14,  8, }, // America/Asuncion
    {  432,  237,      22,  8, }, // America/Atikokan
    {  432, 7586,      30, 17, }, // America/Bahia_Banderas
    {  432, 3770,      47,  6, }, // America/Cancun
    {  432, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  432, 5640,      66,  7, }, // America/Curacao
    {  432,  481,      73, 13, }, // America/Indiana/Knox
    {  432, 7924,      86, 16, }, // America/Indiana/Marengo
    {  432, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  432, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  432, 8001,     139, 14, }, // America/Indiana/Vevay
    {  432, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  432, 8049,     171, 16, }, // America/Indiana/Winamac
    {  432, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  432, 5849,     207, 22, }, // America/Lower_Princes
    {  432, 8183,     229,  6, }, // America/Merida
    {  432, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  432, 8292,     255, 20, }, // America/North_Dakota/Center
    {  432, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  432,  393,     298,  4, }, // America/Nuuk
    {  432, 6862,     302, 16, }, // America/Scoresbysund
    {  432, 8471,     318, 14, }, // America/St_Barthelemy
    {  432, 2045,     332, 10, }, // America/St_Johns
    {  432, 5780,     342,  9, }, // America/St_Kitts
    {  432, 5797,     351,  9, }, // America/St_Lucia
    {  432,  855,     360, 10, }, // America/St_Thomas
    {  432, 5830,     370, 11, }, // America/St_Vincent
    {  432, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  432, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  432, 1152,     413,  9, }, // Asia/Kathmandu
    {  432,  992,     422,  7, }, // Asia/Kolkata
    {  432, 1178,     429,  5, }, // Asia/Macau
    {  432, 1202,     442,  6, }, // Asia/Yangon
    {  432, 1379,     448,  5, }, // Atlantic/Faroe
    {  432, 6784,     453, 10, }, // Atlantic/St_Helena
    {  432, 2419,     463,  4, }, // Europe/Kyiv
    {  432, 7174,     467,  7, }, // Indian/Reunion
    {  432, 3022,     474,  5, }, // Pacific/Chuuk
    {  432, 2961,     479,  7, }, // Pacific/Pohnpei
    {  433,   14,       0,  6, }, // Africa/Asmara Nama/Latin/Namibia
    {  433, 4337,       6,  8, }, // Africa/Sao_Tome
    {  433, 4154,      14,  8, }, // America/Asuncion
    {  433,  237,      22,  8, }, // America/Atikokan
    {  433, 7586,      30, 17, }, // America/Bahia_Banderas
    {  433, 3770,      47,  6, }, // America/Cancun
    {  433, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  433, 5640,      66,  7, }, // America/Curacao
    {  433,  481,      73, 13, }, // America/Indiana/Knox
    {  433, 7924,      86, 16, }, // America/Indiana/Marengo
    {  433, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  433, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  433, 8001,     139, 14, }, // America/Indiana/Vevay
    {  433, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  433, 8049,     171, 16, }, // America/Indiana/Winamac
    {  433, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  433, 5849,     207, 22, }, // America/Lower_Princes
    {  433, 8183,     229,  6, }, // America/Merida
    {  433, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  433, 8292,     255, 20, }, // America/North_Dakota/Center
    {  433, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  433,  393,     298,  4, }, // America/Nuuk
    {  433, 6862,     302, 16, }, // America/Scoresbysund
    {  433, 8471,     318, 14, }, // America/St_Barthelemy
    {  433, 2045,     332, 10, }, // America/St_Johns
    {  433, 5780,     342,  9, }, // America/St_Kitts
    {  433, 5797,     351,  9, }, // America/St_Lucia
    {  433,  855,     360, 10, }, // America/St_Thomas
    {  433, 5830,     370, 11, }, // America/St_Vincent
    {  433, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  433, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  433, 1152,     413,  9, }, // Asia/Kathmandu
    {  433,  992,     422,  7, }, // Asia/Kolkata
    {  433, 1178,     429,  5, }, // Asia/Macau
    {  433, 1202,     442,  6, }, // Asia/Yangon
    {  433, 1379,     448,  5, }, // Atlantic/Faroe
    {  433, 6784,     453, 10, }, // Atlantic/St_Helena
    {  433, 2419,     463,  4, }, // Europe/Kyiv
    {  433, 7174,     467,  7, }, // Indian/Reunion
    {  433, 3022,     474,  5, }, // Pacific/Chuuk
    {  433, 2961,     479,  7, }, // Pacific/Pohnpei
    {  434,   14,       0,  6, }, // Africa/Asmara Navajo/Latin/United States
    {  434, 4337,       6,  8, }, // Africa/Sao_Tome
    {  434, 4154,      14,  8, }, // America/Asuncion
    {  434,  237,      22,  8, }, // America/Atikokan
    {  434, 7586,      30, 17, }, // America/Bahia_Banderas
    {  434, 3770,      47,  6, }, // America/Cancun
    {  434, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  434, 5640,      66,  7, }, // America/Curacao
    {  434,  481,      73, 13, }, // America/Indiana/Knox
    {  434, 7924,      86, 16, }, // America/Indiana/Marengo
    {  434, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  434, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  434, 8001,     139, 14, }, // America/Indiana/Vevay
    {  434, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  434, 8049,     171, 16, }, // America/Indiana/Winamac
    {  434, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  434, 5849,     207, 22, }, // America/Lower_Princes
    {  434, 8183,     229,  6, }, // America/Merida
    {  434, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  434, 8292,     255, 20, }, // America/North_Dakota/Center
    {  434, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  434,  393,     298,  4, }, // America/Nuuk
    {  434, 6862,     302, 16, }, // America/Scoresbysund
    {  434, 8471,     318, 14, }, // America/St_Barthelemy
    {  434, 2045,     332, 10, }, // America/St_Johns
    {  434, 5780,     342,  9, }, // America/St_Kitts
    {  434, 5797,     351,  9, }, // America/St_Lucia
    {  434,  855,     360, 10, }, // America/St_Thomas
    {  434, 5830,     370, 11, }, // America/St_Vincent
    {  434, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  434, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  434, 1152,     413,  9, }, // Asia/Kathmandu
    {  434,  992,     422,  7, }, // Asia/Kolkata
    {  434, 1178,     429,  5, }, // Asia/Macau
    {  434, 1202,     442,  6, }, // Asia/Yangon
    {  434, 1379,     448,  5, }, // Atlantic/Faroe
    {  434, 6784,     453, 10, }, // Atlantic/St_Helena
    {  434, 2419,     463,  4, }, // Europe/Kyiv
    {  434, 7174,     467,  7, }, // Indian/Reunion
    {  434, 3022,     474,  5, }, // Pacific/Chuuk
    {  434, 2961,     479,  7, }, // Pacific/Pohnpei
    {  435, 6751,  133596,  8, }, // Africa/Abidjan Nepali/Devanagari/Nepal
    {  435, 6723,  133604,  5, }, // Africa/Accra
    {  435, 4961,  133609, 12, }, // Africa/Addis_Ababa
    {  435, 7383,  133621,  9, }, // Africa/Algiers
    {  435,   14,   28456,  7, }, // Africa/Asmara
    {  435,   44,  133630,  6, }, // Africa/Bamako
    {  435, 5159,  133636,  8, }, // Africa/Bangui
    {  435, 6709,   28475,  6, }, // Africa/Banjul
    {  435, 7398,  108983,  5, }, // Africa/Bissau
    {  435, 4859,  133644, 10, }, // Africa/Blantyre
    {  435, 5189,  133654, 12, }, // Africa/Brazzaville
    {  435, 4824,   28509, 10, }, // Africa/Bujumbura
    {  435, 2242,  133666,  5, }, // Africa/Cairo
    {  435, 4019,  133671, 12, }, // Africa/Casablanca
    {  435, 7412,   86849,  5, }, // Africa/Ceuta
    {  435, 6736,   86854,  8, }, // Africa/Conakry
    {  435, 6803,   28547,  5, }, // Africa/Dakar
    {  435, 5032,  133683, 12, }, // Africa/Dar_es_Salaam
    {  435, 4945,  133695,  7, }, // Africa/Djibouti
    {  435, 5145,  133702,  6, }, // Africa/Douala
    {  435, 5068,  133708,  9, }, // Africa/El_Aaiun
    {  435, 6816,  133717,  8, }, // Africa/Freetown
    {  435, 4808,  133725,  7, }, // Africa/Gaborone
    {  435, 4917,   86912,  5, }, // Africa/Harare
    {  435, 4381,  133732, 11, }, // Africa/Johannesburg
    {  435, 4401,   28613,  4, }, // Africa/Juba
    {  435, 5053,   28617,  8, }, // Africa/Kampala
    {  435, 4426,  133743,  7, }, // Africa/Khartoum
    {  435, 4875,   86933,  6, }, // Africa/Kigali
    {  435, 5208,   28638,  8, }, // Africa/Kinshasa
    {  435, 4646,   86946,  5, }, // Africa/Lagos
    {  435, 5238,  133750, 12, }, // Africa/Libreville
    {  435, 6832,   86961,  3, }, // Africa/Lome
    {  435, 5113,  133762,  8, }, // Africa/Luanda
    {  435, 4841,  133770, 10, }, // Africa/Lubumbashi
    {  435, 4903,   28680,  6, }, // Africa/Lusaka
    {  435, 5224,  133780,  6, }, // Africa/Malabo
    {  435, 4889,  133786,  6, }, // Africa/Maputo
    {  435, 5099,   28698,  6, }, // Africa/Maseru
    {  435, 5084,   86990,  7, }, // Africa/Mbabane
    {  435, 5015,  133792,  8, }, // Africa/Mogadishu
    {  435, 6997,  133800, 10, }, // Africa/Monrovia
    {  435, 3755,  133810,  7, }, // Africa/Nairobi
    {  435, 5173,  133817, 10, }, // Africa/Ndjamena
    {  435, 5256,  133827,  5, }, // Africa/Niamey
    {  435, 6766,  133832,  8, }, // Africa/Nouakchott
    {  435, 6690,  133840,  8, }, // Africa/Ouagadougou
    {  435, 5127,  133848, 11, }, // Africa/Porto-Novo
    {  435, 4337,  133859,  8, }, // Africa/Sao_Tome
    {  435, 2732,   87068,  8, }, // Africa/Tripoli
    {  435, 6503,  109122,  5, }, // Africa/Tunis
    {  435, 4054,  133867,  9, }, // Africa/Windhoek
    {  435,  132,  133876,  4, }, // America/Adak
    {  435, 3183,  133880,  8, }, // America/Anchorage
    {  435, 5524,  133888, 11, }, // America/Anguilla
    {  435, 5541,   28820,  9, }, // America/Antigua
    {  435, 4456,  133899,  9, }, // America/Araguaina
    {  435,  166,  133908, 12, }, // America/Argentina/Buenos_Aires
    {  435,   91,  133920, 10, }, // America/Argentina/Catamarca
    {  435,  270,  133930,  7, }, // America/Argentina/Cordoba
    {  435,  441,   28865,  5, }, // America/Argentina/Jujuy
    {  435, 7425,  109190,  9, }, // America/Argentina/La_Rioja
    {  435,  565,  133937,  8, }, // America/Argentina/Mendoza
    {  435, 7452,  133945, 14, }, // America/Argentina/Rio_Gallegos
    {  435, 7483,   28896,  6, }, // America/Argentina/Salta
    {  435, 7507,  133959,  9, }, // America/Argentina/San_Juan
    {  435, 5497,  133968,  8, }, // America/Argentina/San_Luis
    {  435, 7534,   28919,  7, }, // America/Argentina/Tucuman
    {  435, 7560,  133976,  6, }, // America/Argentina/Ushuaia
    {  435, 5557,  109233,  5, }, // America/Aruba
    {  435, 4154,  133982,  8, }, // America/Asuncion
    {  435,  237,  133990,  8, }, // America/Atikokan
    {  435, 3581,   28954,  6, }, // America/Bahia
    {  435, 7586,  133998, 16, }, // America/Bahia_Banderas
    {  435, 5571,  131527,  9, }, // America/Barbados
    {  435, 7609,   28985,  5, }, // America/Belem
    {  435, 5309,   28990,  5, }, // America/Belize
    {  435, 7623,  134014, 15, }, // America/Blanc-Sablon
    {  435, 7644,  134029, 10, }, // America/Boa_Vista
    {  435, 4263,   87269,  6, }, // America/Bogota
    {  435, 7662,  134039,  4, }, // America/Boise
    {  435, 7676,  134043, 14, }, // America/Cambridge_Bay
    {  435, 7698,  134057, 17, }, // America/Campo_Grande
    {  435, 3770,  109322,  6, }, // America/Cancun
    {  435, 4596,   29061,  7, }, // America/Caracas
    {  435, 4247,  134074,  8, }, // America/Cayenne
    {  435, 5398,  134082,  7, }, // America/Cayman
    {  435, 3251,   87320,  6, }, // America/Chicago
    {  435, 7719,  134089, 10, }, // America/Chihuahua
    {  435, 7737,  134099, 10, }, // America/Ciudad_Juarez
    {  435, 5324,  134109, 11, }, // America/Costa_Rica
    {  435, 7759,   29101,  8, }, // America/Creston
    {  435, 3693,  134120,  8, }, // America/Cuiaba
    {  435, 5640,   87368,  7, }, // America/Curacao
    {  435, 7775,  134128, 13, }, // America/Danmarkshavn
    {  435, 7796,  134141,  3, }, // America/Dawson
    {  435, 7811,  134144,  9, }, // America/Dawson_Creek
    {  435,  805,  134153,  6, }, // America/Denver
    {  435, 3351,  134159,  8, }, // America/Detroit
    {  435, 5656,   87413,  8, }, // America/Dominica
    {  435,  893,  134167,  9, }, // America/Edmonton
    {  435, 7832,  134176,  7, }, // America/Eirunepe
    {  435, 5343,  134183, 13, }, // America/El_Salvador
    {  435, 7849,  109444, 12, }, // America/Fort_Nelson
    {  435, 7869,  124999, 10, }, // America/Fortaleza
    {  435, 7887,   29219,  8, }, // America/Glace_Bay
    {  435, 6844,  134196,  6, }, // America/Goose_Bay
    {  435, 4514,  134202, 12, }, // America/Grand_Turk
    {  435, 5687,   29248,  8, }, // America/Grenada
    {  435, 5703,  134214,  9, }, // America/Guadeloupe
    {  435, 3662,   29265, 10, }, // America/Guatemala
    {  435, 4743,  134223,  9, }, // America/Guayaquil
    {  435, 6895,   29283,  6, }, // America/Guyana
    {  435, 1923,  134232, 13, }, // America/Halifax
    {  435, 2221,  134245,  5, }, // America/Havana
    {  435, 7905,  134250, 12, }, // America/Hermosillo
    {  435,  348,  134262, 14, }, // America/Indiana/Indianapolis
    {  435,  481,  134276, 15, }, // America/Indiana/Knox
    {  435, 7924,  134291, 19, }, // America/Indiana/Marengo
    {  435, 7948,  134310, 21, }, // America/Indiana/Petersburg
    {  435, 7975,  134331, 19, }, // America/Indiana/Tell_City
    {  435, 8001,  134350, 15, }, // America/Indiana/Vevay
    {  435, 8023,  134365,  9, }, // America/Indiana/Vincennes
    {  435, 8049,  134374, 18, }, // America/Indiana/Winamac
    {  435, 8073,  134392,  6, }, // America/Inuvik
    {  435,  660,  134398,  9, }, // America/Iqaluit
    {  435, 2665,  134407,  6, }, // America/Jamaica
    {  435, 5282,  134413,  5, }, // America/Juneau
    {  435,  521,  134418, 10, }, // America/Kentucky/Louisville
    {  435, 8088,  134428, 20, }, // America/Kentucky/Monticello
    {  435, 5621,  134448, 11, }, // America/Kralendijk
    {  435, 4278,   29517,  6, }, // America/La_Paz
    {  435, 7132,  109675,  4, }, // America/Lima
    {  435, 3390,  134459, 11, }, // America/Los_Angeles
    {  435, 5849,  134470, 23, }, // America/Lower_Princes
    {  435, 8116,  134493,  6, }, // America/Maceio
    {  435, 8131,  134499,  8, }, // America/Managua
    {  435, 1892,  134507,  6, }, // America/Manaus
    {  435, 5814,  134513,  8, }, // America/Marigot
    {  435, 5722,   87794,  9, }, // America/Martinique
    {  435, 8147,  134521, 12, }, // America/Matamoros
    {  435, 2779,  134533,  9, }, // America/Mazatlan
    {  435, 8165,   87821,  8, }, // America/Menominee
    {  435, 8183,   29617,  6, }, // America/Merida
    {  435, 8198,  134542, 11, }, // America/Metlakatla
    {  435, 2811,   87841, 13, }, // America/Mexico_City
    {  435, 4293,  134553,  8, }, // America/Miquelon
    {  435, 8217,  134561,  7, }, // America/Moncton
    {  435, 8233,  134568, 10, }, // America/Monterrey
    {  435, 4000,  134578, 12, }, // America/Montevideo
    {  435, 5741,  134590, 11, }, // America/Montserrat
    {  435, 5383,   29691,  7, }, // America/Nassau
    {  435, 3294,  134601,  9, }, // America/New_York
    {  435, 8251,   87823,  3, }, // America/Nome
    {  435, 1834,   87910,  8, }, // America/Noronha
    {  435, 8264,  134610, 19, }, // America/North_Dakota/Beulah
    {  435, 8292,  134629, 21, }, // America/North_Dakota/Center
    {  435, 8320,  134650, 24, }, // America/North_Dakota/New_Salem
    {  435,  393,   29774,  3, }, // America/Nuuk
    {  435, 8351,  109895,  7, }, // America/Ojinaga
    {  435, 5413,   29784,  6, }, // America/Panama
    {  435, 6106,  134674, 10, }, // America/Paramaribo
    {  435, 3224,  109911,  7, }, // America/Phoenix
    {  435, 3847,  134684, 16, }, // America/Port-au-Prince
    {  435, 5871,  134700, 15, }, // America/Port_of_Spain
    {  435, 8367,  134715, 13, }, // America/Porto_Velho
    {  435, 5760,  134728, 13, }, // America/Puerto_Rico
    {  435, 3932,  134741, 13, }, // America/Punta_Arenas
    {  435, 8387,  134754, 14, }, // America/Rankin_Inlet
    {  435, 8408,  134768,  6, }, // America/Recife
    {  435, 1979,   29890,  6, }, // America/Regina
    {  435, 8423,  134774,  7, }, // America/Resolute
    {  435,  695,  134781, 13, }, // America/Rio_Branco
    {  435, 8440,  134794,  9, }, // America/Santarem
    {  435, 2165,  134803,  9, }, // America/Santiago
    {  435, 6044,  134812, 15, }, // America/Santo_Domingo
    {  435, 1862,   88151,  9, }, // America/Sao_Paulo
    {  435, 6862,  134827, 17, }, // America/Scoresbysund
    {  435, 8457,   88173,  6, }, // America/Sitka
    {  435, 8471,  134844, 16, }, // America/St_Barthelemy
    {  435, 2045,  134860, 11, }, // America/St_Johns
    {  435, 5780,  134871, 12, }, // America/St_Kitts
    {  435, 5797,  134883, 12, }, // America/St_Lucia
    {  435,  855,  134895, 10, }, // America/St_Thomas
    {  435, 5830,  134905, 15, }, // America/St_Vincent
    {  435, 8493,  134920, 14, }, // America/Swift_Current
    {  435, 5363,  132105, 12, }, // America/Tegucigalpa
    {  435, 5673,  110109,  4, }, // America/Thule
    {  435,  313,   88275,  7, }, // America/Tijuana
    {  435,  608,  134934,  8, }, // America/Toronto
    {  435, 5605,  134942,  8, }, // America/Tortola
    {  435, 2077,  134950,  9, }, // America/Vancouver
    {  435, 2128,  134959, 10, }, // America/Whitehorse
    {  435,  734,  134969,  9, }, // America/Winnipeg
    {  435, 8515,  110147,  7, }, // America/Yakutat
    {  435, 5918,  134978,  4, }, // Antarctica/Casey
    {  435, 6027,   30136,  5, }, // Antarctica/Davis
    {  435, 6066,  134982, 20, }, // Antarctica/DumontDUrville
    {  435, 8531,  135002,  8, }, // Antarctica/Macquarie
    {  435, 7029,  135010,  5, }, // Antarctica/Mawson
    {  435, 7076,  135015,  9, }, // Antarctica/McMurdo
    {  435, 8552,  110192,  7, }, // Antarctica/Palmer
    {  435, 7189,   88376,  6, }, // Antarctica/Rothera
    {  435, 7258,  135024,  4, }, // Antarctica/Syowa
    {  435, 8570,   88388,  5, }, // Antarctica/Troll
    {  435, 7337,  135028,  7, }, // Antarctica/Vostok
    {  435, 1413,  135035, 11, }, // Arctic/Longyearbyen
    {  435, 5487,   30214,  4, }, // Asia/Aden
    {  435, 5297,   30218,  7, }, // Asia/Almaty
    {  435, 3870,   30225,  6, }, // Asia/Amman
    {  435, 5428,  135046,  7, }, // Asia/Anadyr
    {  435, 5440,  135053,  6, }, // Asia/Aqtau
    {  435, 5270,  135059,  6, }, // Asia/Aqtobe
    {  435,  964,  135065,  8, }, // Asia/Ashgabat
    {  435, 8587,  135073,  6, }, // Asia/Atyrau
    {  435, 3509,   88447,  5, }, // Asia/Baghdad
    {  435, 5451,  135079,  6, }, // Asia/Bahrain
    {  435, 3555,   87366,  4, }, // Asia/Baku
    {  435, 4368,  135085,  5, }, // Asia/Bangkok
    {  435, 3473,  110258,  5, }, // Asia/Barnaul
    {  435, 3988,  135090,  6, }, // Asia/Beirut
    {  435, 3680,  135096,  7, }, // Asia/Bishkek
    {  435, 5906,  135103,  7, }, // Asia/Brunei
    {  435, 4503,  125608,  4, }, // Asia/Chita
    {  435, 5963,  135110, 10, }, // Asia/Choibalsan
    {  435, 4413,  125620,  7, }, // Asia/Colombo
    {  435, 4442,  135120,  9, }, // Asia/Damascus
    {  435, 1060,   88508,  4, }, // Asia/Dhaka
    {  435, 6125,  135129,  6, }, // Asia/Dili
    {  435, 3498,  135135,  5, }, // Asia/Dubai
    {  435, 6092,  135140,  9, }, // Asia/Dushanbe
    {  435, 8599,   30357, 10, }, // Asia/Famagusta
    {  435, 8614,   30367,  4, }, // Asia/Gaza
    {  435, 4697,  135149,  7, }, // Asia/Hebron
    {  435, 1226,  135156, 15, }, // Asia/Ho_Chi_Minh
    {  435, 2576,  135171,  4, }, // Asia/Hong_Kong
    {  435, 4673,  135175,  5, }, // Asia/Hovd
    {  435, 4086,   88580, 10, }, // Asia/Irkutsk
    {  435, 6969,   30403,  8, }, // Asia/Jakarta
    {  435, 6955,   30411,  7, }, // Asia/Jayapura
    {  435, 1257,  132302,  8, }, // Asia/Jerusalem
    {  435, 3462,   88603,  5, }, // Asia/Kabul
    {  435, 4218,  135180,  8, }, // Asia/Kamchatka
    {  435, 4141,   30440,  5, }, // Asia/Karachi
    {  435, 1152,  135188, 10, }, // Asia/Kathmandu
    {  435, 8624,  135198,  8, }, // Asia/Khandyga
    {  435,  992,   88630,  7, }, // Asia/Kolkata
    {  435, 4099,   88637, 15, }, // Asia/Krasnoyarsk
    {  435, 4761,  135206, 11, }, // Asia/Kuala_Lumpur
    {  435, 5893,  135217,  5, }, // Asia/Kuching
    {  435, 5464,  110381,  5, }, // Asia/Kuwait
    {  435, 1178,  135222,  4, }, // Asia/Macau
    {  435, 3919,  135226,  7, }, // Asia/Magadan
    {  435, 1316,   30519,  9, }, // Asia/Makassar
    {  435, 7145,  110404,  5, }, // Asia/Manila
    {  435, 6883,  135233,  8, }, // Asia/Muscat
    {  435, 2446,   88677,  8, }, // Asia/Nicosia
    {  435, 8638,  135241, 12, }, // Asia/Novokuznetsk
    {  435, 4037,  135253, 13, }, // Asia/Novosibirsk
    {  435, 4131,   88714,  6, }, // Asia/Omsk
    {  435, 7108,   88720,  3, }, // Asia/Oral
    {  435, 6924,  135266,  9, }, // Asia/Phnom_Penh
    {  435, 8656,  135275, 10, }, // Asia/Pontianak
    {  435, 4116,  135285,  8, }, // Asia/Pyongyang
    {  435, 5476,  110444,  4, }, // Asia/Qatar
    {  435, 8671,  135293,  7, }, // Asia/Qostanay
    {  435, 4171,  125779, 10, }, // Asia/Qyzylorda
    {  435, 3486,   88773,  5, }, // Asia/Riyadh
    {  435, 4310,   30633,  7, }, // Asia/Sakhalin
    {  435, 7208,  135300,  8, }, // Asia/Samarkand
    {  435, 3116,   88790,  4, }, // Asia/Seoul
    {  435, 1020,  135308,  7, }, // Asia/Shanghai
    {  435, 3137,   30658,  8, }, // Asia/Singapore
    {  435, 4199,  135315, 17, }, // Asia/Srednekolymsk
    {  435, 3100,   30682,  5, }, // Asia/Taipei
    {  435, 4683,  135332,  9, }, // Asia/Tashkent
    {  435, 3817,   30696,  8, }, // Asia/Tbilisi
    {  435, 2638,  135341,  7, }, // Asia/Tehran
    {  435, 1284,  135348,  6, }, // Asia/Thimphu
    {  435, 2687,  132419,  6, }, // Asia/Tokyo
    {  435, 4474,  135354,  7, }, // Asia/Tomsk
    {  435, 1346,  135361,  8, }, // Asia/Ulaanbaatar
    {  435, 1126,   88860,  7, }, // Asia/Urumqi
    {  435, 8685,  135369,  9, }, // Asia/Ust-Nera
    {  435, 6940,  135378,  8, }, // Asia/Vientiane
    {  435, 4612,  135386, 13, }, // Asia/Vladivostok
    {  435, 4730,  135399,  9, }, // Asia/Yakutsk
    {  435, 1202,  135408,  7, }, // Asia/Yangon
    {  435, 3785,  135415, 11, }, // Asia/Yekaterinburg
    {  435, 3649,   30802,  7, }, // Asia/Yerevan
    {  435, 3565,  135426,  6, }, // Atlantic/Azores
    {  435, 5588,  110596,  7, }, // Atlantic/Bermuda
    {  435, 6591,  135432,  8, }, // Atlantic/Canary
    {  435, 3629,  135440,  9, }, // Atlantic/Cape_Verde
    {  435, 1379,  135449,  5, }, // Atlantic/Faroe
    {  435, 8699,  135454,  7, }, // Atlantic/Madeira
    {  435, 2614,  135461,  9, }, // Atlantic/Reykjavik
    {  435, 7235,  135470, 14, }, // Atlantic/South_Georgia
    {  435, 6784,  135484, 12, }, // Atlantic/St_Helena
    {  435, 6607,  135496, 10, }, // Atlantic/Stanley
    {  435, 1654,  135506,  6, }, // Australia/Adelaide
    {  435, 1619,  110654,  9, }, // Australia/Brisbane
    {  435, 1783,   89012, 10, }, // Australia/Broken_Hill
    {  435, 1567,   89022,  7, }, // Australia/Darwin
    {  435, 3539,  135512,  7, }, // Australia/Eucla
    {  435, 1500,  135519,  7, }, // Australia/Hobart
    {  435, 8716,  135526, 11, }, // Australia/Lindeman
    {  435, 1531,  135537,  9, }, // Australia/Lord_Howe
    {  435, 1711,  135546,  8, }, // Australia/Melbourne
    {  435, 1746,   89062,  4, }, // Australia/Perth
    {  435, 1447,  135554,  6, }, // Australia/Sydney
    {  435, 6390,  135560, 11, }, // Europe/Amsterdam
    {  435, 6149,  135571,  9, }, // Europe/Andorra
    {  435, 3522,  135580,  9, }, // Europe/Astrakhan
    {  435, 6577,   30994,  6, }, // Europe/Athens
    {  435, 6285,   31000,  8, }, // Europe/Belgrade
    {  435, 4659,   89097,  6, }, // Europe/Berlin
    {  435, 6437,  135589, 12, }, // Europe/Bratislava
    {  435, 6178,  126055,  8, }, // Europe/Brussels
    {  435, 3830,  135601,  9, }, // Europe/Bucharest
    {  435, 3708,   89134,  9, }, // Europe/Budapest
    {  435, 8751,  135610,  9, }, // Europe/Busingen
    {  435, 2475,  135619,  7, }, // Europe/Chisinau
    {  435, 6238,  135626, 11, }, // Europe/Copenhagen
    {  435, 2260,  110761,  6, }, // Europe/Dublin
    {  435, 6256,   31085, 11, }, // Europe/Gibraltar
    {  435, 8767,  135637,  8, }, // Europe/Guernsey
    {  435, 6561,  135645, 10, }, // Europe/Helsinki
    {  435, 8783,  135655, 13, }, // Europe/Isle_of_Man
    {  435, 1097,  135668,  9, }, // Europe/Istanbul
    {  435, 8802,   89218,  5, }, // Europe/Jersey
    {  435, 3881,  135677, 12, }, // Europe/Kaliningrad
    {  435, 8924,  135689,  5, }, // Europe/Kirov
    {  435, 2419,  135694,  3, }, // Europe/Kyiv
    {  435, 3078,  135697,  7, }, // Europe/Lisbon
    {  435, 6455,  135704,  9, }, // Europe/Ljubljana
    {  435, 2393,  135713,  5, }, // Europe/London
    {  435, 6314,  135718, 11, }, // Europe/Luxembourg
    {  435, 4794,  135729,  9, }, // Europe/Madrid
    {  435, 6346,   31199,  6, }, // Europe/Malta
    {  435, 6531,  135738, 13, }, // Europe/Mariehamn
    {  435, 3595,   31212,  7, }, // Europe/Minsk
    {  435, 6359,   89290,  6, }, // Europe/Monaco
    {  435, 3443,  135751,  5, }, // Europe/Moscow
    {  435, 6407,   89302,  5, }, // Europe/Oslo
    {  435, 4186,   31234,  5, }, // Europe/Paris
    {  435, 6373,  135756,  9, }, // Europe/Podgorica
    {  435, 6224,   89317,  5, }, // Europe/Prague
    {  435, 8816,   31252,  4, }, // Europe/Riga
    {  435, 6273,   88949,  3, }, // Europe/Rome
    {  435, 4233,   31259,  6, }, // Europe/Samara
    {  435, 6419,  135765, 10, }, // Europe/San_Marino
    {  435, 6194,  135775,  8, }, // Europe/Sarajevo
    {  435, 4353,  135783,  7, }, // Europe/Saratov
    {  435, 8828,  126226, 11, }, // Europe/Simferopol
    {  435, 6332,  135790,  7, }, // Europe/Skopje
    {  435, 6548,  110891,  6, }, // Europe/Sofia
    {  435, 6472,  135797,  9, }, // Europe/Stockholm
    {  435, 8846,  135806,  7, }, // Europe/Tallinn
    {  435, 6135,  126257,  6, }, // Europe/Tirane
    {  435, 8861,  135813, 12, }, // Europe/Ulyanovsk
    {  435, 6301,   31344,  5, }, // Europe/Vaduz
    {  435, 6516,  135825,  8, }, // Europe/Vatican
    {  435, 6164,  135833,  5, }, // Europe/Vienna
    {  435, 8878,  135838,  8, }, // Europe/Vilnius
    {  435, 4629,  135846, 11, }, // Europe/Volgograd
    {  435, 3055,  135857,  5, }, // Europe/Warsaw
    {  435, 6210,   31387,  7, }, // Europe/Zagreb
    {  435, 6489,  135862,  5, }, // Europe/Zurich
    {  435, 4980,  135867, 11, }, // Indian/Antananarivo
    {  435, 6910,  110966,  5, }, // Indian/Chagos
    {  435, 5979,  135878,  9, }, // Indian/Christmas
    {  435, 5996,   89495,  5, }, // Indian/Cocos
    {  435, 4931,   89500,  6, }, // Indian/Comoro
    {  435, 6624,  135887, 10, }, // Indian/Kerguelen
    {  435, 7223,   31451,  4, }, // Indian/Mahe
    {  435, 7013,  135897,  9, }, // Indian/Maldives
    {  435, 3971,  135906,  8, }, // Indian/Mauritius
    {  435, 5000,  132741,  8, }, // Indian/Mayotte
    {  435, 7174,  110990,  8, }, // Indian/Reunion
    {  435, 4324,  110998,  5, }, // Pacific/Apia
    {  435,  932,  135914,  9, }, // Pacific/Auckland
    {  435, 3608,  135923, 11, }, // Pacific/Bougainville
    {  435, 2863,   31508,  5, }, // Pacific/Chatham
    {  435, 3022,   31513,  3, }, // Pacific/Chuuk
    {  435, 2201,  135934,  5, }, // Pacific/Easter
    {  435, 7323,  135939,  5, }, // Pacific/Efate
    {  435, 7290,  111028,  7, }, // Pacific/Fakaofo
    {  435, 3804,   31537,  4, }, // Pacific/Fiji
    {  435, 7306,  135944,  8, }, // Pacific/Funafuti
    {  435, 6641,  135952,  8, }, // Pacific/Galapagos
    {  435, 6659,  135960, 10, }, // Pacific/Gambier
    {  435, 3724,  135970, 10, }, // Pacific/Guadalcanal
    {  435, 5935,   31579,  5, }, // Pacific/Guam
    {  435, 2929,   89628,  8, }, // Pacific/Honolulu
    {  435, 2897,  135980,  8, }, // Pacific/Kanton
    {  435, 3900,  135988, 10, }, // Pacific/Kiritimati
    {  435, 6982,  135998,  7, }, // Pacific/Kosrae
    {  435, 2708,  136005, 10, }, // Pacific/Kwajalein
    {  435, 4779,   89659,  6, }, // Pacific/Majuro
    {  435, 3953,   31630,  9, }, // Pacific/Marquesas
    {  435, 8909,   89673,  5, }, // Pacific/Midway
    {  435, 7047,  136015,  5, }, // Pacific/Nauru
    {  435, 7095,  136020,  4, }, // Pacific/Niue
    {  435, 4070,  136024,  7, }, // Pacific/Norfolk
    {  435, 7061,  136031,  6, }, // Pacific/Noumea
    {  435, 2991,   89699,  9, }, // Pacific/Pago_Pago
    {  435, 7118,  136037,  4, }, // Pacific/Palau
    {  435, 7157,  136041,  8, }, // Pacific/Pitcairn
    {  435, 2961,  136049,  6, }, // Pacific/Pohnpei
    {  435, 4709,   89718, 14, }, // Pacific/Port_Moresby
    {  435, 6009,  136055,  8, }, // Pacific/Rarotonga
    {  435, 5948,  136063,  5, }, // Pacific/Saipan
    {  435, 7275,   89747,  6, }, // Pacific/Tahiti
    {  435, 6675,  136068,  4, }, // Pacific/Tarawa
    {  435, 4485,  136072,  7, }, // Pacific/Tongatapu
    {  435, 7355,   31736,  3, }, // Pacific/Wake
    {  435, 7368,   31739,  5, }, // Pacific/Wallis
    {  437,   14,       0,  6, }, // Africa/Asmara Ngiemboon/Latin/Cameroon
    {  437, 4337,       6,  8, }, // Africa/Sao_Tome
    {  437, 4154,      14,  8, }, // America/Asuncion
    {  437,  237,      22,  8, }, // America/Atikokan
    {  437, 7586,      30, 17, }, // America/Bahia_Banderas
    {  437, 3770,      47,  6, }, // America/Cancun
    {  437, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  437, 5640,      66,  7, }, // America/Curacao
    {  437,  481,      73, 13, }, // America/Indiana/Knox
    {  437, 7924,      86, 16, }, // America/Indiana/Marengo
    {  437, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  437, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  437, 8001,     139, 14, }, // America/Indiana/Vevay
    {  437, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  437, 8049,     171, 16, }, // America/Indiana/Winamac
    {  437, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  437, 5849,     207, 22, }, // America/Lower_Princes
    {  437, 8183,     229,  6, }, // America/Merida
    {  437, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  437, 8292,     255, 20, }, // America/North_Dakota/Center
    {  437, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  437,  393,     298,  4, }, // America/Nuuk
    {  437, 6862,     302, 16, }, // America/Scoresbysund
    {  437, 8471,     318, 14, }, // America/St_Barthelemy
    {  437, 2045,     332, 10, }, // America/St_Johns
    {  437, 5780,     342,  9, }, // America/St_Kitts
    {  437, 5797,     351,  9, }, // America/St_Lucia
    {  437,  855,     360, 10, }, // America/St_Thomas
    {  437, 5830,     370, 11, }, // America/St_Vincent
    {  437, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  437, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  437, 1152,     413,  9, }, // Asia/Kathmandu
    {  437,  992,     422,  7, }, // Asia/Kolkata
    {  437, 1178,     429,  5, }, // Asia/Macau
    {  437, 1202,     442,  6, }, // Asia/Yangon
    {  437, 1379,     448,  5, }, // Atlantic/Faroe
    {  437, 6784,     453, 10, }, // Atlantic/St_Helena
    {  437, 2419,     463,  4, }, // Europe/Kyiv
    {  437, 7174,     467,  7, }, // Indian/Reunion
    {  437, 3022,     474,  5, }, // Pacific/Chuuk
    {  437, 2961,     479,  7, }, // Pacific/Pohnpei
    {  438,   14,       0,  6, }, // Africa/Asmara Ngomba/Latin/Cameroon
    {  438, 4337,       6,  8, }, // Africa/Sao_Tome
    {  438, 4154,      14,  8, }, // America/Asuncion
    {  438,  237,      22,  8, }, // America/Atikokan
    {  438, 7586,      30, 17, }, // America/Bahia_Banderas
    {  438, 3770,      47,  6, }, // America/Cancun
    {  438, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  438, 5640,      66,  7, }, // America/Curacao
    {  438,  481,      73, 13, }, // America/Indiana/Knox
    {  438, 7924,      86, 16, }, // America/Indiana/Marengo
    {  438, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  438, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  438, 8001,     139, 14, }, // America/Indiana/Vevay
    {  438, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  438, 8049,     171, 16, }, // America/Indiana/Winamac
    {  438, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  438, 5849,     207, 22, }, // America/Lower_Princes
    {  438, 8183,     229,  6, }, // America/Merida
    {  438, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  438, 8292,     255, 20, }, // America/North_Dakota/Center
    {  438, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  438,  393,     298,  4, }, // America/Nuuk
    {  438, 6862,     302, 16, }, // America/Scoresbysund
    {  438, 8471,     318, 14, }, // America/St_Barthelemy
    {  438, 2045,     332, 10, }, // America/St_Johns
    {  438, 5780,     342,  9, }, // America/St_Kitts
    {  438, 5797,     351,  9, }, // America/St_Lucia
    {  438,  855,     360, 10, }, // America/St_Thomas
    {  438, 5830,     370, 11, }, // America/St_Vincent
    {  438, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  438, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  438, 1152,     413,  9, }, // Asia/Kathmandu
    {  438,  992,     422,  7, }, // Asia/Kolkata
    {  438, 1178,     429,  5, }, // Asia/Macau
    {  438, 1202,     442,  6, }, // Asia/Yangon
    {  438, 1379,     448,  5, }, // Atlantic/Faroe
    {  438, 6784,     453, 10, }, // Atlantic/St_Helena
    {  438, 2419,     463,  4, }, // Europe/Kyiv
    {  438, 7174,     467,  7, }, // Indian/Reunion
    {  438, 3022,     474,  5, }, // Pacific/Chuuk
    {  438, 2961,     479,  7, }, // Pacific/Pohnpei
    {  439, 6751,  136079,  6, }, // Africa/Abidjan Nigerian Pidgin/Latin/Nigeria
    {  439, 6723,  136085,  4, }, // Africa/Accra
    {  439, 4961,  136089,  9, }, // Africa/Addis_Ababa
    {  439, 7383,  136098,  6, }, // Africa/Algiers
    {  439,   14,  136104,  6, }, // Africa/Asmara
    {  439,   44,  136110,  6, }, // Africa/Bamako
    {  439, 5159,  136116,  6, }, // Africa/Bangui
    {  439, 7398,    4583,  5, }, // Africa/Bissau
    {  439, 4859,  136122,  8, }, // Africa/Blantyre
    {  439, 5189,  136130,  8, }, // Africa/Brazzaville
    {  439, 4824,  136138,  9, }, // Africa/Bujumbura
    {  439, 2242,  136147,  5, }, // Africa/Cairo
    {  439, 4019,  136152, 10, }, // Africa/Casablanca
    {  439, 7412,  136162,  5, }, // Africa/Ceuta
    {  439, 6736,  136167,  7, }, // Africa/Conakry
    {  439, 6803,  136174,  5, }, // Africa/Dakar
    {  439, 5032,  136179, 13, }, // Africa/Dar_es_Salaam
    {  439, 4945,  136192,  6, }, // Africa/Djibouti
    {  439, 5145,  136198,  5, }, // Africa/Douala
    {  439, 5068,  136203,  7, }, // Africa/El_Aaiun
    {  439, 6816,  136210,  7, }, // Africa/Freetown
    {  439, 4808,  136217,  8, }, // Africa/Gaborone
    {  439, 4917,  136225,  6, }, // Africa/Harare
    {  439, 4381,  136231, 10, }, // Africa/Johannesburg
    {  439, 4401,  136241,  4, }, // Africa/Juba
    {  439, 5053,  136245,  7, }, // Africa/Kampala
    {  439, 4426,    4693,  6, }, // Africa/Khartoum
    {  439, 4875,  136252,  6, }, // Africa/Kigali
    {  439, 5208,  136258,  9, }, // Africa/Kinshasa
    {  439, 4646,  136267,  5, }, // Africa/Lagos
    {  439, 5238,  136272,  8, }, // Africa/Libreville
    {  439, 6832,  136280,  4, }, // Africa/Lome
    {  439, 5113,  136284,  6, }, // Africa/Luanda
    {  439, 4841,  136290, 10, }, // Africa/Lubumbashi
    {  439, 4903,  136300,  6, }, // Africa/Lusaka
    {  439, 5224,  136306,  6, }, // Africa/Malabo
    {  439, 4889,  136312,  6, }, // Africa/Maputo
    {  439, 5099,  136318,  7, }, // Africa/Maseru
    {  439, 5084,  136325,  7, }, // Africa/Mbabane
    {  439, 5015,  136332,  9, }, // Africa/Mogadishu
    {  439, 6997,  136341,  8, }, // Africa/Monrovia
    {  439, 3755,  136349,  7, }, // Africa/Nairobi
    {  439, 5173,  136356,  8, }, // Africa/Ndjamena
    {  439, 5256,  136364,  5, }, // Africa/Niamey
    {  439, 6766,  136369,  9, }, // Africa/Nouakchott
    {  439, 6690,  136378,  9, }, // Africa/Ouagadougou
    {  439, 5127,  136387,  9, }, // Africa/Porto-Novo
    {  439, 4337,  136396,  8, }, // Africa/Sao_Tome
    {  439, 2732,  136404,  8, }, // Africa/Tripoli
    {  439, 6503,   90982,  5, }, // Africa/Tunis
    {  439, 4054,  136412,  7, }, // Africa/Windhoek
    {  439,  132,  136419,  4, }, // America/Adak
    {  439, 3183,  136423,  8, }, // America/Anchorage
    {  439, 5524,  136431,  7, }, // America/Anguilla
    {  439, 5541,  136438,  8, }, // America/Antigua
    {  439, 4456,  136446, 10, }, // America/Araguaina
    {  439,  166,  136456, 12, }, // America/Argentina/Buenos_Aires
    {  439,   91,  136468,  8, }, // America/Argentina/Catamarca
    {  439,  270,  136476,  7, }, // America/Argentina/Cordoba
    {  439,  441,    4804,  5, }, // America/Argentina/Jujuy
    {  439, 7425,  136483,  9, }, // America/Argentina/La_Rioja
    {  439,  565,  136492,  7, }, // America/Argentina/Mendoza
    {  439, 7452,  136499, 12, }, // America/Argentina/Rio_Gallegos
    {  439, 7483,  136511,  5, }, // America/Argentina/Salta
    {  439, 7507,  136516,  8, }, // America/Argentina/San_Juan
    {  439, 7534,  136524,  7, }, // America/Argentina/Tucuman
    {  439, 7560,  136531,  6, }, // America/Argentina/Ushuaia
    {  439, 5557,   91006,  5, }, // America/Aruba
    {  439, 4154,  136537,  8, }, // America/Asuncion
    {  439,  237,  136545,  8, }, // America/Atikokan
    {  439, 3581,      30,  5, }, // America/Bahia
    {  439, 7586,  136553, 15, }, // America/Bahia_Banderas
    {  439, 5571,  136568,  8, }, // America/Barbados
    {  439, 7609,  136576,  5, }, // America/Belem
    {  439, 5309,  136581,  5, }, // America/Belize
    {  439, 7623,  136586, 12, }, // America/Blanc-Sablon
    {  439, 7644,  136598,  9, }, // America/Boa_Vista
    {  439, 7662,  136607,  5, }, // America/Boise
    {  439, 7676,  136612, 10, }, // America/Cambridge_Bay
    {  439, 7698,  136622, 12, }, // America/Campo_Grande
    {  439, 3770,    4959,  6, }, // America/Cancun
    {  439, 4596,  136634,  7, }, // America/Caracas
    {  439, 4247,  136641,  5, }, // America/Cayenne
    {  439, 5398,  136646,  5, }, // America/Cayman
    {  439, 3251,  136651,  7, }, // America/Chicago
    {  439, 7719,  136658,  9, }, // America/Chihuahua
    {  439, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  439, 5324,  136667, 10, }, // America/Costa_Rica
    {  439, 7759,  136677,  8, }, // America/Creston
    {  439, 3693,  136685,  7, }, // America/Cuiaba
    {  439, 5640,  136692,  8, }, // America/Curacao
    {  439, 7775,  136700, 12, }, // America/Danmarkshavn
    {  439, 7796,  136712,  6, }, // America/Dawson
    {  439, 7811,  136718, 11, }, // America/Dawson_Creek
    {  439,  805,  136729,  6, }, // America/Denver
    {  439, 3351,  136735,  7, }, // America/Detroit
    {  439, 5656,  136742,  8, }, // America/Dominica
    {  439,  893,  136750, 10, }, // America/Edmonton
    {  439, 7832,  136760,  8, }, // America/Eirunepe
    {  439, 5343,  136768, 10, }, // America/El_Salvador
    {  439, 7849,  136778, 11, }, // America/Fort_Nelson
    {  439, 7869,  136789,  9, }, // America/Fortaleza
    {  439, 7887,  136798,  7, }, // America/Glace_Bay
    {  439, 6844,  136805,  6, }, // America/Goose_Bay
    {  439, 4514,  136811,  9, }, // America/Grand_Turk
    {  439, 5687,  136820,  7, }, // America/Grenada
    {  439, 5703,  136827,  9, }, // America/Guadeloupe
    {  439, 3662,  136836,  9, }, // America/Guatemala
    {  439, 4743,  136845,  8, }, // America/Guayaquil
    {  439, 6895,  136853,  6, }, // America/Guyana
    {  439, 1923,  136859,  8, }, // America/Halifax
    {  439, 2221,  136867,  6, }, // America/Havana
    {  439, 7905,  136873,  9, }, // America/Hermosillo
    {  439,  348,  136882, 13, }, // America/Indiana/Indianapolis
    {  439,  481,  136895, 13, }, // America/Indiana/Knox
    {  439, 7924,  136908, 17, }, // America/Indiana/Marengo
    {  439, 7948,  136925, 17, }, // America/Indiana/Petersburg
    {  439, 7975,  136942, 18, }, // America/Indiana/Tell_City
    {  439, 8001,  136960, 14, }, // America/Indiana/Vevay
    {  439, 8023,  136974, 15, }, // America/Indiana/Vincennes
    {  439, 8049,  136989, 16, }, // America/Indiana/Winamac
    {  439, 8073,  137005,  6, }, // America/Inuvik
    {  439,  660,  137011,  7, }, // America/Iqaluit
    {  439, 2665,   91066,  7, }, // America/Jamaica
    {  439, 5282,  137018,  4, }, // America/Juneau
    {  439,  521,  137022,  6, }, // America/Kentucky/Louisville
    {  439, 8088,  137028, 22, }, // America/Kentucky/Monticello
    {  439, 5621,  137050, 11, }, // America/Kralendijk
    {  439, 7132,  137061,  4, }, // America/Lima
    {  439, 3390,  137065, 12, }, // America/Los_Angeles
    {  439, 5849,  137077, 20, }, // America/Lower_Princes
    {  439, 8116,  137097,  7, }, // America/Maceio
    {  439, 8131,   96580,  7, }, // America/Managua
    {  439, 1892,  137104,  6, }, // America/Manaus
    {  439, 5814,  137110,  7, }, // America/Marigot
    {  439, 5722,  137117,  7, }, // America/Martinique
    {  439, 8147,  137124,  9, }, // America/Matamoros
    {  439, 2779,  137133,  8, }, // America/Mazatlan
    {  439, 8165,  137141, 10, }, // America/Menominee
    {  439, 8183,  137151,  7, }, // America/Merida
    {  439, 8198,  137158, 10, }, // America/Metlakatla
    {  439, 2811,  137168, 13, }, // America/Mexico_City
    {  439, 4293,  137181,  7, }, // America/Miquelon
    {  439, 8217,  137188,  8, }, // America/Moncton
    {  439, 8233,  137196,  8, }, // America/Monterrey
    {  439, 4000,  137204, 10, }, // America/Montevideo
    {  439, 5741,  137214,  9, }, // America/Montserrat
    {  439, 5383,  137223,  6, }, // America/Nassau
    {  439, 3294,  137229,  7, }, // America/New_York
    {  439, 8251,  137236,  4, }, // America/Nome
    {  439, 1834,  137240,  7, }, // America/Noronha
    {  439, 8264,  137247, 18, }, // America/North_Dakota/Beulah
    {  439, 8292,  137265, 19, }, // America/North_Dakota/Center
    {  439, 8320,  137284, 22, }, // America/North_Dakota/New_Salem
    {  439,  393,     298,  4, }, // America/Nuuk
    {  439, 8351,  137306,  8, }, // America/Ojinaga
    {  439, 5413,  137314,  6, }, // America/Panama
    {  439, 6106,  137320, 10, }, // America/Paramaribo
    {  439, 3224,  137330,  6, }, // America/Phoenix
    {  439, 3847,  137336, 11, }, // America/Port-au-Prince
    {  439, 5871,  137347, 12, }, // America/Port_of_Spain
    {  439, 8367,  137359, 11, }, // America/Porto_Velho
    {  439, 5760,  137370, 12, }, // America/Puerto_Rico
    {  439, 3932,  137382, 13, }, // America/Punta_Arenas
    {  439, 8387,  137395, 12, }, // America/Rankin_Inlet
    {  439, 8408,  137407,  6, }, // America/Recife
    {  439, 1979,  137413,  6, }, // America/Regina
    {  439, 8423,  137419,  8, }, // America/Resolute
    {  439,  695,  137427, 10, }, // America/Rio_Branco
    {  439, 8440,  137437,  8, }, // America/Santarem
    {  439, 2165,  137445,  8, }, // America/Santiago
    {  439, 6044,  137453, 13, }, // America/Santo_Domingo
    {  439, 1862,  137466,  9, }, // America/Sao_Paulo
    {  439, 6862,  137475, 11, }, // America/Scoresbysund
    {  439, 8471,  137486, 14, }, // America/St_Barthelemy
    {  439, 2045,  137500,  8, }, // America/St_Johns
    {  439, 5780,  121759,  9, }, // America/St_Kitts
    {  439, 5797,  137508, 11, }, // America/St_Lucia
    {  439,  855,  137519, 10, }, // America/St_Thomas
    {  439, 5830,  137529, 12, }, // America/St_Vincent
    {  439, 8493,  137541, 13, }, // America/Swift_Current
    {  439, 5363,  137554, 11, }, // America/Tegucigalpa
    {  439, 5673,  137565,  4, }, // America/Thule
    {  439,  313,  137569,  8, }, // America/Tijuana
    {  439,  608,  137577,  7, }, // America/Toronto
    {  439, 5605,  137584,  6, }, // America/Tortola
    {  439, 2077,  137590,  7, }, // America/Vancouver
    {  439, 2128,  137597,  7, }, // America/Whitehorse
    {  439,  734,  137604,  7, }, // America/Winnipeg
    {  439, 8515,  137611,  7, }, // America/Yakutat
    {  439, 5918,  137618,  4, }, // Antarctica/Casey
    {  439, 6027,  137622,  5, }, // Antarctica/Davis
    {  439, 6066,  137627, 14, }, // Antarctica/DumontDUrville
    {  439, 8531,  137641,  7, }, // Antarctica/Macquarie
    {  439, 7029,  137648,  6, }, // Antarctica/Mawson
    {  439, 7076,  137654,  7, }, // Antarctica/McMurdo
    {  439, 8552,  137661,  4, }, // Antarctica/Palmer
    {  439, 7189,  137665,  7, }, // Antarctica/Rothera
    {  439, 7258,  137672,  5, }, // Antarctica/Syowa
    {  439, 8570,  119009,  4, }, // Antarctica/Troll
    {  439, 7337,  137677,  7, }, // Antarctica/Vostok
    {  439, 1413,  137684, 12, }, // Arctic/Longyearbyen
    {  439, 5487,  137696,  4, }, // Asia/Aden
    {  439, 5297,  137700,  6, }, // Asia/Almaty
    {  439, 3870,    5896,  4, }, // Asia/Amman
    {  439, 5428,  137706,  7, }, // Asia/Anadyr
    {  439, 5440,  137713,  5, }, // Asia/Aqtau
    {  439, 5270,  137718,  6, }, // Asia/Aqtobe
    {  439,  964,  137724,  8, }, // Asia/Ashgabat
    {  439, 8587,  137732,  6, }, // Asia/Atyrau
    {  439, 3509,  137738,  6, }, // Asia/Baghdad
    {  439, 5451,  137744,  6, }, // Asia/Bahrain
    {  439, 4368,  137750,  6, }, // Asia/Bangkok
    {  439, 3473,  137756,  7, }, // Asia/Barnaul
    {  439, 3988,  137763,  5, }, // Asia/Beirut
    {  439, 3680,  137768,  7, }, // Asia/Bishkek
    {  439, 5906,  137775,  6, }, // Asia/Brunei
    {  439, 4503,  137781,  5, }, // Asia/Chita
    {  439, 5963,  137786, 10, }, // Asia/Choibalsan
    {  439, 4413,  137796,  7, }, // Asia/Colombo
    {  439, 4442,  137803,  8, }, // Asia/Damascus
    {  439, 1060,  137811,  4, }, // Asia/Dhaka
    {  439, 6125,  137815,  4, }, // Asia/Dili
    {  439, 6092,  137819,  8, }, // Asia/Dushanbe
    {  439, 8599,  137827,  7, }, // Asia/Famagusta
    {  439, 8614,   90090,  4, }, // Asia/Gaza
    {  439, 4697,  137834,  7, }, // Asia/Hebron
    {  439, 1226,  137841, 15, }, // Asia/Ho_Chi_Minh
    {  439, 2576,  137856,  9, }, // Asia/Hong_Kong
    {  439, 4086,  137865,  7, }, // Asia/Irkutsk
    {  439, 6969,  137872,  6, }, // Asia/Jakarta
    {  439, 6955,  137878,  8, }, // Asia/Jayapura
    {  439, 1257,  137886,  9, }, // Asia/Jerusalem
    {  439, 4218,  137895,  9, }, // Asia/Kamchatka
    {  439, 4141,  137904,  7, }, // Asia/Karachi
    {  439, 1152,  137911,  8, }, // Asia/Kathmandu
    {  439, 8624,  137919,  7, }, // Asia/Khandyga
    {  439,  992,  137926,  7, }, // Asia/Kolkata
    {  439, 4099,  137933, 10, }, // Asia/Krasnoyarsk
    {  439, 4761,  137943, 11, }, // Asia/Kuala_Lumpur
    {  439, 5464,  137954,  5, }, // Asia/Kuwait
    {  439, 1178,  137959,  5, }, // Asia/Macau
    {  439, 3919,  137964,  7, }, // Asia/Magadan
    {  439, 1316,  137971,  6, }, // Asia/Makassar
    {  439, 7145,  137977,  6, }, // Asia/Manila
    {  439, 6883,  137983,  6, }, // Asia/Muscat
    {  439, 2446,   25331,  7, }, // Asia/Nicosia
    {  439, 8638,  137989, 12, }, // Asia/Novokuznetsk
    {  439, 4037,  138001, 10, }, // Asia/Novosibirsk
    {  439, 4131,  138011,  4, }, // Asia/Omsk
    {  439, 7108,  138015,  5, }, // Asia/Oral
    {  439, 6924,  138020,  9, }, // Asia/Phnom_Penh
    {  439, 8656,  138029,  9, }, // Asia/Pontianak
    {  439, 4116,  138038,  9, }, // Asia/Pyongyang
    {  439, 5476,  138047,  4, }, // Asia/Qatar
    {  439, 8671,  138051,  8, }, // Asia/Qostanay
    {  439, 4171,  138059,  9, }, // Asia/Qyzylorda
    {  439, 3486,  138068,  5, }, // Asia/Riyadh
    {  439, 4310,  138073,  8, }, // Asia/Sakhalin
    {  439, 7208,  138081,  8, }, // Asia/Samarkand
    {  439, 3116,  138089,  3, }, // Asia/Seoul
    {  439, 1020,  138092,  8, }, // Asia/Shanghai
    {  439, 3137,  138100,  7, }, // Asia/Singapore
    {  439, 4199,  138107, 14, }, // Asia/Srednekolymsk
    {  439, 3100,  138121,  6, }, // Asia/Taipei
    {  439, 4683,  138127,  8, }, // Asia/Tashkent
    {  439, 3817,  138135,  7, }, // Asia/Tbilisi
    {  439, 2638,  138142,  5, }, // Asia/Tehran
    {  439, 1284,  138147,  5, }, // Asia/Thimphu
    {  439, 2687,  138152,  5, }, // Asia/Tokyo
    {  439, 1346,  138157,  8, }, // Asia/Ulaanbaatar
    {  439, 1126,  138165,  8, }, // Asia/Urumqi
    {  439, 8685,  138173,  9, }, // Asia/Ust-Nera
    {  439, 6940,  138182,  9, }, // Asia/Vientiane
    {  439, 4612,  138191, 11, }, // Asia/Vladivostok
    {  439, 4730,  138202,  7, }, // Asia/Yakutsk
    {  439, 1202,  138209,  6, }, // Asia/Yangon
    {  439, 3785,  138215, 12, }, // Asia/Yekaterinburg
    {  439, 3649,  138227,  8, }, // Asia/Yerevan
    {  439, 3565,  138235,  4, }, // Atlantic/Azores
    {  439, 5588,  138239,  7, }, // Atlantic/Bermuda
    {  439, 6591,  138246,  6, }, // Atlantic/Canary
    {  439, 3629,  138252,  7, }, // Atlantic/Cape_Verde
    {  439, 1379,  138259,  6, }, // Atlantic/Faroe
    {  439, 8699,  138265,  6, }, // Atlantic/Madeira
    {  439, 2614,  138271,  9, }, // Atlantic/Reykjavik
    {  439, 7235,  138280, 11, }, // Atlantic/South_Georgia
    {  439, 6784,  138291, 12, }, // Atlantic/St_Helena
    {  439, 6607,  138303,  6, }, // Atlantic/Stanley
    {  439, 1654,  138309,  6, }, // Australia/Adelaide
    {  439, 1619,  138315,  7, }, // Australia/Brisbane
    {  439, 1783,  138322, 10, }, // Australia/Broken_Hill
    {  439, 1567,  138332,  6, }, // Australia/Darwin
    {  439, 3539,  138338,  5, }, // Australia/Eucla
    {  439, 1500,  138343,  5, }, // Australia/Hobart
    {  439, 8716,  138348,  8, }, // Australia/Lindeman
    {  439, 1531,  138356,  7, }, // Australia/Lord_Howe
    {  439, 1711,  138363,  7, }, // Australia/Melbourne
    {  439, 1746,  138370,  4, }, // Australia/Perth
    {  439, 1447,  138374,  5, }, // Australia/Sydney
    {  439, 6390,  138379,  8, }, // Europe/Amsterdam
    {  439, 6149,  138387,  7, }, // Europe/Andorra
    {  439, 3522,  138394,  8, }, // Europe/Astrakhan
    {  439, 6577,  138402,  5, }, // Europe/Athens
    {  439, 6285,  138407,  8, }, // Europe/Belgrade
    {  439, 4659,  138415,  5, }, // Europe/Berlin
    {  439, 6437,  138420, 10, }, // Europe/Bratislava
    {  439, 6178,  138430,  7, }, // Europe/Brussels
    {  439, 3830,  138437,  8, }, // Europe/Bucharest
    {  439, 3708,  138445,  8, }, // Europe/Budapest
    {  439, 8751,  138453,  8, }, // Europe/Busingen
    {  439, 2475,  138461,  8, }, // Europe/Chisinau
    {  439, 6238,  138469, 10, }, // Europe/Copenhagen
    {  439, 2260,  138479,  7, }, // Europe/Dublin
    {  439, 6256,  138486,  9, }, // Europe/Gibraltar
    {  439, 8767,  138495,  6, }, // Europe/Guernsey
    {  439, 6561,  138501,  8, }, // Europe/Helsinki
    {  439, 8783,  138509, 10, }, // Europe/Isle_of_Man
    {  439, 1097,  138519,  8, }, // Europe/Istanbul
    {  439, 8802,  138527,  5, }, // Europe/Jersey
    {  439, 3881,  138532, 11, }, // Europe/Kaliningrad
    {  439, 8924,  138543,  5, }, // Europe/Kirov
    {  439, 2419,  138548,  4, }, // Europe/Kyiv
    {  439, 3078,  138552,  6, }, // Europe/Lisbon
    {  439, 6455,  138558,  8, }, // Europe/Ljubljana
    {  439, 2393,  138566,  7, }, // Europe/London
    {  439, 6314,  138573, 11, }, // Europe/Luxembourg
    {  439, 6346,  138584,  6, }, // Europe/Malta
    {  439, 6531,  138590,  9, }, // Europe/Mariehamn
    {  439, 6359,  138599,  7, }, // Europe/Monaco
    {  439, 3443,  138606,  6, }, // Europe/Moscow
    {  439, 6407,  138612,  5, }, // Europe/Oslo
    {  439, 4186,  138617,  5, }, // Europe/Paris
    {  439, 6373,  138622,  9, }, // Europe/Podgorica
    {  439, 6224,    6545,  4, }, // Europe/Prague
    {  439, 8816,  138631,  4, }, // Europe/Riga
    {  439, 6273,    6554,  3, }, // Europe/Rome
    {  439, 4233,  138635,  6, }, // Europe/Samara
    {  439, 6419,  138641, 10, }, // Europe/San_Marino
    {  439, 6194,  138651,  9, }, // Europe/Sarajevo
    {  439, 4353,  138660,  7, }, // Europe/Saratov
    {  439, 8828,  138667, 10, }, // Europe/Simferopol
    {  439, 6332,  138677,  7, }, // Europe/Skopje
    {  439, 6548,   19594,  5, }, // Europe/Sofia
    {  439, 6472,  138684,  8, }, // Europe/Stockholm
    {  439, 8846,  138692,  5, }, // Europe/Tallinn
    {  439, 6135,  138697,  6, }, // Europe/Tirane
    {  439, 8861,  138703,  9, }, // Europe/Ulyanovsk
    {  439, 6516,  138712,  7, }, // Europe/Vatican
    {  439, 6164,  138719,  6, }, // Europe/Vienna
    {  439, 8878,   43118,  7, }, // Europe/Vilnius
    {  439, 4629,  138725,  9, }, // Europe/Volgograd
    {  439, 3055,  138734,  5, }, // Europe/Warsaw
    {  439, 6210,  138739,  6, }, // Europe/Zagreb
    {  439, 6489,  138745,  5, }, // Europe/Zurich
    {  439, 4980,  138750, 12, }, // Indian/Antananarivo
    {  439, 6910,  138762,  6, }, // Indian/Chagos
    {  439, 5979,  138768,  7, }, // Indian/Christmas
    {  439, 5996,   91554,  5, }, // Indian/Cocos
    {  439, 4931,  138775,  9, }, // Indian/Comoro
    {  439, 6624,  138784,  9, }, // Indian/Kerguelen
    {  439, 7223,  138793,  5, }, // Indian/Mahe
    {  439, 7013,  138798,  8, }, // Indian/Maldives
    {  439, 3971,  138806,  8, }, // Indian/Mauritius
    {  439, 5000,  138814,  5, }, // Indian/Mayotte
    {  439, 7174,  138819,  8, }, // Indian/Reunion
    {  439, 4324,  138827,  4, }, // Pacific/Apia
    {  439,  932,  138831,  7, }, // Pacific/Auckland
    {  439, 3608,  138838, 10, }, // Pacific/Bougainville
    {  439, 2863,  138848,  6, }, // Pacific/Chatham
    {  439, 3022,  138854,  4, }, // Pacific/Chuuk
    {  439, 2201,  138858,  4, }, // Pacific/Easter
    {  439, 7323,  138862,  5, }, // Pacific/Efate
    {  439, 7290,  138867,  7, }, // Pacific/Fakaofo
    {  439, 3804,  138874,  4, }, // Pacific/Fiji
    {  439, 7306,  138878,  8, }, // Pacific/Funafuti
    {  439, 6641,  138886,  9, }, // Pacific/Galapagos
    {  439, 6659,  138895,  7, }, // Pacific/Gambier
    {  439, 3724,  138902, 11, }, // Pacific/Guadalcanal
    {  439, 2897,  138913, 10, }, // Pacific/Kanton
    {  439, 3900,  138923,  9, }, // Pacific/Kiritimati
    {  439, 6982,  138932,  7, }, // Pacific/Kosrae
    {  439, 2708,  138939,  8, }, // Pacific/Kwajalein
    {  439, 4779,  138947,  6, }, // Pacific/Majuro
    {  439, 3953,  138953,  9, }, // Pacific/Marquesas
    {  439, 8909,  138962,  5, }, // Pacific/Midway
    {  439, 7047,  138967,  5, }, // Pacific/Nauru
    {  439, 7095,  137229,  3, }, // Pacific/Niue
    {  439, 4070,  138972,  9, }, // Pacific/Norfolk
    {  439, 7061,  138981,  6, }, // Pacific/Noumea
    {  439, 2991,  138987,  9, }, // Pacific/Pago_Pago
    {  439, 7118,  138996,  5, }, // Pacific/Palau
    {  439, 7157,  139001,  6, }, // Pacific/Pitcairn
    {  439, 2961,  139007,  7, }, // Pacific/Pohnpei
    {  439, 4709,  139014, 12, }, // Pacific/Port_Moresby
    {  439, 6009,  139026,  9, }, // Pacific/Rarotonga
    {  439, 7275,  139035,  6, }, // Pacific/Tahiti
    {  439, 6675,  139041,  6, }, // Pacific/Tarawa
    {  439, 4485,  139047,  9, }, // Pacific/Tongatapu
    {  439, 7355,  139056,  3, }, // Pacific/Wake
    {  439, 7368,  139059,  4, }, // Pacific/Wallis
    {  440, 6751,  139063,  7, }, // Africa/Abidjan Nko/Nko/Guinea
    {  440, 6723,  139070,  5, }, // Africa/Accra
    {  440, 4961,  139075, 14, }, // Africa/Addis_Ababa
    {  440, 7383,  139089,  6, }, // Africa/Algiers
    {  440,   14,  139095,  7, }, // Africa/Asmara
    {  440,   44,  139102,  7, }, // Africa/Bamako
    {  440, 5159,  139109,  7, }, // Africa/Bangui
    {  440, 6709,  139116,  6, }, // Africa/Banjul
    {  440, 7398,  139122,  7, }, // Africa/Bissau
    {  440, 4859,  139129,  9, }, // Africa/Blantyre
    {  440, 5189,  139138,  9, }, // Africa/Brazzaville
    {  440, 4824,  139147, 11, }, // Africa/Bujumbura
    {  440, 2242,  139158,  4, }, // Africa/Cairo
    {  440, 4019,  139162, 11, }, // Africa/Casablanca
    {  440, 6736,  139173,  8, }, // Africa/Conakry
    {  440, 6803,  139181,  9, }, // Africa/Dakar
    {  440, 5032,  139190, 12, }, // Africa/Dar_es_Salaam
    {  440, 4945,  139202,  7, }, // Africa/Djibouti
    {  440, 5145,  139209,  6, }, // Africa/Douala
    {  440, 5068,  139215,  6, }, // Africa/El_Aaiun
    {  440, 6816,  139221,  7, }, // Africa/Freetown
    {  440, 4808,  139228,  8, }, // Africa/Gaborone
    {  440, 4917,  139236,  6, }, // Africa/Harare
    {  440, 4381,  139242, 13, }, // Africa/Johannesburg
    {  440, 4401,  139255,  5, }, // Africa/Juba
    {  440, 5053,  139260,  7, }, // Africa/Kampala
    {  440, 4426,  139267,  7, }, // Africa/Khartoum
    {  440, 4875,  139274,  8, }, // Africa/Kigali
    {  440, 5208,  139282,  8, }, // Africa/Kinshasa
    {  440, 4646,  139290,  6, }, // Africa/Lagos
    {  440, 5238,  139296,  9, }, // Africa/Libreville
    {  440, 6832,  139305,  5, }, // Africa/Lome
    {  440, 5113,  139310,  8, }, // Africa/Luanda
    {  440, 4841,  139318, 10, }, // Africa/Lubumbashi
    {  440, 4903,  139328,  6, }, // Africa/Lusaka
    {  440, 5224,  139334,  6, }, // Africa/Malabo
    {  440, 4889,  139340,  7, }, // Africa/Maputo
    {  440, 5099,  139347,  7, }, // Africa/Maseru
    {  440, 5084,  139354,  6, }, // Africa/Mbabane
    {  440, 5015,  139360, 10, }, // Africa/Mogadishu
    {  440, 6997,  139370, 11, }, // Africa/Monrovia
    {  440, 3755,  139381,  8, }, // Africa/Nairobi
    {  440, 5173,  139389,  8, }, // Africa/Ndjamena
    {  440, 5256,  139397,  5, }, // Africa/Niamey
    {  440, 6766,  139402,  9, }, // Africa/Nouakchott
    {  440, 6690,  139411, 10, }, // Africa/Ouagadougou
    {  440, 5127,  139421, 13, }, // Africa/Porto-Novo
    {  440, 4337,  139434,  9, }, // Africa/Sao_Tome
    {  440, 2732,  139443, 16, }, // Africa/Tripoli
    {  440, 6503,  139459,  5, }, // Africa/Tunis
    {  440, 4054,  139464,  6, }, // Africa/Windhoek
    {  440, 4154,      14,  8, }, // America/Asuncion
    {  440,  237,      22,  8, }, // America/Atikokan
    {  440, 7586,      30, 17, }, // America/Bahia_Banderas
    {  440, 3770,      47,  6, }, // America/Cancun
    {  440, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  440, 5640,      66,  7, }, // America/Curacao
    {  440,  481,      73, 13, }, // America/Indiana/Knox
    {  440, 7924,      86, 16, }, // America/Indiana/Marengo
    {  440, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  440, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  440, 8001,     139, 14, }, // America/Indiana/Vevay
    {  440, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  440, 8049,     171, 16, }, // America/Indiana/Winamac
    {  440, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  440, 5849,     207, 22, }, // America/Lower_Princes
    {  440, 8183,     229,  6, }, // America/Merida
    {  440, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  440, 8292,     255, 20, }, // America/North_Dakota/Center
    {  440, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  440,  393,     298,  4, }, // America/Nuuk
    {  440, 6862,     302, 16, }, // America/Scoresbysund
    {  440, 8471,     318, 14, }, // America/St_Barthelemy
    {  440, 2045,     332, 10, }, // America/St_Johns
    {  440, 5780,     342,  9, }, // America/St_Kitts
    {  440, 5797,     351,  9, }, // America/St_Lucia
    {  440,  855,     360, 10, }, // America/St_Thomas
    {  440, 5830,     370, 11, }, // America/St_Vincent
    {  440, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  440, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  440, 1152,     413,  9, }, // Asia/Kathmandu
    {  440,  992,     422,  7, }, // Asia/Kolkata
    {  440, 1178,     429,  5, }, // Asia/Macau
    {  440, 1202,     442,  6, }, // Asia/Yangon
    {  440, 3629,  139470, 14, }, // Atlantic/Cape_Verde
    {  440, 1379,     448,  5, }, // Atlantic/Faroe
    {  440, 6784,  139484, 10, }, // Atlantic/St_Helena
    {  440, 2419,     463,  4, }, // Europe/Kyiv
    {  440, 4980,  139494, 13, }, // Indian/Antananarivo
    {  440, 6910,  139507,  6, }, // Indian/Chagos
    {  440, 4931,  139513,  6, }, // Indian/Comoro
    {  440, 6624,  139519, 11, }, // Indian/Kerguelen
    {  440, 7223,  139530,  5, }, // Indian/Mahe
    {  440, 3971,  139535,  5, }, // Indian/Mauritius
    {  440, 5000,  139540,  5, }, // Indian/Mayotte
    {  440, 7174,  139545,  8, }, // Indian/Reunion
    {  440, 3022,     474,  5, }, // Pacific/Chuuk
    {  440, 2961,     479,  7, }, // Pacific/Pohnpei
    {  441,   14,       0,  6, }, // Africa/Asmara Northern Luri/Arabic/Iran
    {  441, 4337,       6,  8, }, // Africa/Sao_Tome
    {  441, 4154,      14,  8, }, // America/Asuncion
    {  441,  237,      22,  8, }, // America/Atikokan
    {  441, 7586,      30, 17, }, // America/Bahia_Banderas
    {  441, 3770,      47,  6, }, // America/Cancun
    {  441, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  441, 5640,      66,  7, }, // America/Curacao
    {  441,  481,      73, 13, }, // America/Indiana/Knox
    {  441, 7924,      86, 16, }, // America/Indiana/Marengo
    {  441, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  441, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  441, 8001,     139, 14, }, // America/Indiana/Vevay
    {  441, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  441, 8049,     171, 16, }, // America/Indiana/Winamac
    {  441, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  441, 5849,     207, 22, }, // America/Lower_Princes
    {  441, 8183,     229,  6, }, // America/Merida
    {  441, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  441, 8292,     255, 20, }, // America/North_Dakota/Center
    {  441, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  441,  393,     298,  4, }, // America/Nuuk
    {  441, 6862,     302, 16, }, // America/Scoresbysund
    {  441, 8471,     318, 14, }, // America/St_Barthelemy
    {  441, 2045,     332, 10, }, // America/St_Johns
    {  441, 5780,     342,  9, }, // America/St_Kitts
    {  441, 5797,     351,  9, }, // America/St_Lucia
    {  441,  855,     360, 10, }, // America/St_Thomas
    {  441, 5830,     370, 11, }, // America/St_Vincent
    {  441, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  441, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  441, 1152,     413,  9, }, // Asia/Kathmandu
    {  441,  992,     422,  7, }, // Asia/Kolkata
    {  441, 1178,     429,  5, }, // Asia/Macau
    {  441, 1202,     442,  6, }, // Asia/Yangon
    {  441, 1379,     448,  5, }, // Atlantic/Faroe
    {  441, 6784,     453, 10, }, // Atlantic/St_Helena
    {  441, 2419,     463,  4, }, // Europe/Kyiv
    {  441, 7174,     467,  7, }, // Indian/Reunion
    {  441, 3022,     474,  5, }, // Pacific/Chuuk
    {  441, 2961,     479,  7, }, // Pacific/Pohnpei
    {  443,   14,       0,  6, }, // Africa/Asmara Northern Sami/Latin/Norway
    {  443, 4337,       6,  8, }, // Africa/Sao_Tome
    {  443, 4154,      14,  8, }, // America/Asuncion
    {  443,  237,      22,  8, }, // America/Atikokan
    {  443, 7586,      30, 17, }, // America/Bahia_Banderas
    {  443, 3770,      47,  6, }, // America/Cancun
    {  443, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  443, 5640,      66,  7, }, // America/Curacao
    {  443,  481,      73, 13, }, // America/Indiana/Knox
    {  443, 7924,      86, 16, }, // America/Indiana/Marengo
    {  443, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  443, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  443, 8001,     139, 14, }, // America/Indiana/Vevay
    {  443, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  443, 8049,     171, 16, }, // America/Indiana/Winamac
    {  443, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  443, 5849,     207, 22, }, // America/Lower_Princes
    {  443, 8183,     229,  6, }, // America/Merida
    {  443, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  443, 8292,     255, 20, }, // America/North_Dakota/Center
    {  443, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  443,  393,     298,  4, }, // America/Nuuk
    {  443, 1862,   25145,  9, }, // America/Sao_Paulo
    {  443, 6862,     302, 16, }, // America/Scoresbysund
    {  443, 8471,   34416, 16, }, // America/St_Barthelemy
    {  443, 2045,     332, 10, }, // America/St_Johns
    {  443, 5780,     342,  9, }, // America/St_Kitts
    {  443, 5797,     351,  9, }, // America/St_Lucia
    {  443,  855,     360, 10, }, // America/St_Thomas
    {  443, 5830,     370, 11, }, // America/St_Vincent
    {  443, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  443, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  443, 1152,     413,  9, }, // Asia/Kathmandu
    {  443,  992,     422,  7, }, // Asia/Kolkata
    {  443, 1178,     429,  5, }, // Asia/Macau
    {  443, 1202,     442,  6, }, // Asia/Yangon
    {  443, 1379,     448,  5, }, // Atlantic/Faroe
    {  443, 6784,     453, 10, }, // Atlantic/St_Helena
    {  443, 2419,     463,  4, }, // Europe/Kyiv
    {  443, 7174,     467,  7, }, // Indian/Reunion
    {  443, 3022,     474,  5, }, // Pacific/Chuuk
    {  443, 2961,     479,  7, }, // Pacific/Pohnpei
    {  444, 2242,   24913,  5, }, // Africa/Cairo Northern Sami/Latin/Finland
    {  444, 7775,  139553, 15, }, // America/Danmarkshavn
    {  444, 8264,  139568, 20, }, // America/North_Dakota/Beulah
    {  444, 8292,  139588, 22, }, // America/North_Dakota/Center
    {  444, 8320,  139610, 23, }, // America/North_Dakota/New_Salem
    {  444, 4442,   62693,  8, }, // Asia/Damascus
    {  444, 4086,  139633,  6, }, // Asia/Irkutsk
    {  444, 4218,   31981,  8, }, // Asia/Kamchatka
    {  444, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  444, 8638,  139639, 11, }, // Asia/Novokuznetsk
    {  444, 4310,   32047,  7, }, // Asia/Sakhalin
    {  444, 2638,    6175,  7, }, // Asia/Tehran
    {  444, 1346,  139650, 10, }, // Asia/Ulaanbaatar
    {  444, 4730,  139660,  6, }, // Asia/Yakutsk
    {  444, 1202,   56146,  7, }, // Asia/Yangon
    {  444, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  444, 3649,    6231,  7, }, // Asia/Yerevan
    {  444, 3565,  139666,  6, }, // Atlantic/Azores
    {  444, 6591,  139672,  7, }, // Atlantic/Canary
    {  444, 3629,   55900,  9, }, // Atlantic/Cape_Verde
    {  444, 7235,  139679, 13, }, // Atlantic/South_Georgia
    {  444, 6577,   91735,  6, }, // Europe/Athens
    {  444, 6285,   25465,  7, }, // Europe/Belgrade
    {  444, 6178,   21176,  7, }, // Europe/Brussels
    {  444, 3830,   25479,  8, }, // Europe/Bucharest
    {  444, 6238,   57194,  9, }, // Europe/Copenhagen
    {  444, 6561,  139692,  7, }, // Europe/Helsinki
    {  444, 8783,  139699,  8, }, // Europe/Isle_of_Man
    {  444, 3078,   19528,  6, }, // Europe/Lisbon
    {  444, 6314,   25515,  9, }, // Europe/Luxembourg
    {  444, 3443,   21290,  6, }, // Europe/Moscow
    {  444, 6224,   34765,  5, }, // Europe/Prague
    {  444, 6273,   19581,  4, }, // Europe/Rome
    {  444, 6135,   19608,  6, }, // Europe/Tirane
    {  444, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  444, 6164,   56035,  4, }, // Europe/Vienna
    {  444, 3055,   91764,  7, }, // Europe/Warsaw
    {  444, 6489,    4478,  6, }, // Europe/Zurich
    {  444, 5979,  139707, 11, }, // Indian/Christmas
    {  444, 5996,    4484,  5, }, // Indian/Cocos
    {  444, 7013,  139718, 11, }, // Indian/Maldives
    {  444, 3953,  139729, 11, }, // Pacific/Marquesas
    {  446,   14,       0,  6, }, // Africa/Asmara Northern Sotho/Latin/South Africa
    {  446, 4337,       6,  8, }, // Africa/Sao_Tome
    {  446, 4154,      14,  8, }, // America/Asuncion
    {  446,  237,      22,  8, }, // America/Atikokan
    {  446, 7586,      30, 17, }, // America/Bahia_Banderas
    {  446, 3770,      47,  6, }, // America/Cancun
    {  446, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  446, 5640,      66,  7, }, // America/Curacao
    {  446,  481,      73, 13, }, // America/Indiana/Knox
    {  446, 7924,      86, 16, }, // America/Indiana/Marengo
    {  446, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  446, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  446, 8001,     139, 14, }, // America/Indiana/Vevay
    {  446, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  446, 8049,     171, 16, }, // America/Indiana/Winamac
    {  446, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  446, 5849,     207, 22, }, // America/Lower_Princes
    {  446, 8183,     229,  6, }, // America/Merida
    {  446, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  446, 8292,     255, 20, }, // America/North_Dakota/Center
    {  446, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  446,  393,     298,  4, }, // America/Nuuk
    {  446, 6862,     302, 16, }, // America/Scoresbysund
    {  446, 8471,     318, 14, }, // America/St_Barthelemy
    {  446, 2045,     332, 10, }, // America/St_Johns
    {  446, 5780,     342,  9, }, // America/St_Kitts
    {  446, 5797,     351,  9, }, // America/St_Lucia
    {  446,  855,     360, 10, }, // America/St_Thomas
    {  446, 5830,     370, 11, }, // America/St_Vincent
    {  446, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  446, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  446, 1152,     413,  9, }, // Asia/Kathmandu
    {  446,  992,     422,  7, }, // Asia/Kolkata
    {  446, 1178,     429,  5, }, // Asia/Macau
    {  446, 1202,     442,  6, }, // Asia/Yangon
    {  446, 1379,     448,  5, }, // Atlantic/Faroe
    {  446, 6784,     453, 10, }, // Atlantic/St_Helena
    {  446, 2419,     463,  4, }, // Europe/Kyiv
    {  446, 7174,     467,  7, }, // Indian/Reunion
    {  446, 3022,     474,  5, }, // Pacific/Chuuk
    {  446, 2961,     479,  7, }, // Pacific/Pohnpei
    {  447,   14,       0,  6, }, // Africa/Asmara North Ndebele/Latin/Zimbabwe
    {  447, 4337,       6,  8, }, // Africa/Sao_Tome
    {  447, 4154,      14,  8, }, // America/Asuncion
    {  447,  237,      22,  8, }, // America/Atikokan
    {  447, 7586,      30, 17, }, // America/Bahia_Banderas
    {  447, 3770,      47,  6, }, // America/Cancun
    {  447, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  447, 5640,      66,  7, }, // America/Curacao
    {  447,  481,      73, 13, }, // America/Indiana/Knox
    {  447, 7924,      86, 16, }, // America/Indiana/Marengo
    {  447, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  447, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  447, 8001,     139, 14, }, // America/Indiana/Vevay
    {  447, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  447, 8049,     171, 16, }, // America/Indiana/Winamac
    {  447, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  447, 5849,     207, 22, }, // America/Lower_Princes
    {  447, 8183,     229,  6, }, // America/Merida
    {  447, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  447, 8292,     255, 20, }, // America/North_Dakota/Center
    {  447, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  447,  393,     298,  4, }, // America/Nuuk
    {  447, 6862,     302, 16, }, // America/Scoresbysund
    {  447, 8471,     318, 14, }, // America/St_Barthelemy
    {  447, 2045,     332, 10, }, // America/St_Johns
    {  447, 5780,     342,  9, }, // America/St_Kitts
    {  447, 5797,     351,  9, }, // America/St_Lucia
    {  447,  855,     360, 10, }, // America/St_Thomas
    {  447, 5830,     370, 11, }, // America/St_Vincent
    {  447, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  447, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  447, 1152,     413,  9, }, // Asia/Kathmandu
    {  447,  992,     422,  7, }, // Asia/Kolkata
    {  447, 1178,     429,  5, }, // Asia/Macau
    {  447, 1202,     442,  6, }, // Asia/Yangon
    {  447, 1379,     448,  5, }, // Atlantic/Faroe
    {  447, 6784,     453, 10, }, // Atlantic/St_Helena
    {  447, 2419,     463,  4, }, // Europe/Kyiv
    {  447, 7174,     467,  7, }, // Indian/Reunion
    {  447, 3022,     474,  5, }, // Pacific/Chuuk
    {  447, 2961,     479,  7, }, // Pacific/Pohnpei
    {  448, 4961,    3793, 11, }, // Africa/Addis_Ababa Norwegian Bokmal/Latin/Norway
    {  448, 7383,   42592,  5, }, // Africa/Algiers
    {  448,   14,       0,  6, }, // Africa/Asmara
    {  448, 2242,   24913,  5, }, // Africa/Cairo
    {  448, 5032,   91972, 13, }, // Africa/Dar_es_Salaam
    {  448, 5068,   62570,  8, }, // Africa/El_Aaiun
    {  448, 6832,   18802,  4, }, // Africa/Lome
    {  448, 4337,       6,  8, }, // Africa/Sao_Tome
    {  448, 4456,   24992,  9, }, // America/Araguaina
    {  448,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  448, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  448, 4154,      14,  8, }, // America/Asuncion
    {  448,  237,      22,  8, }, // America/Atikokan
    {  448, 7586,   18884, 14, }, // America/Bahia_Banderas
    {  448, 7609,   34376,  5, }, // America/Belem
    {  448, 4263,   18898,  6, }, // America/Bogota
    {  448, 3770,      47,  6, }, // America/Cancun
    {  448, 5398,  139740, 11, }, // America/Cayman
    {  448, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  448, 3693,   25007,  6, }, // America/Cuiaba
    {  448, 5640,      66,  7, }, // America/Curacao
    {  448,  481,      73, 13, }, // America/Indiana/Knox
    {  448, 7924,      86, 16, }, // America/Indiana/Marengo
    {  448, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  448, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  448, 8001,     139, 14, }, // America/Indiana/Vevay
    {  448, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  448, 8049,     171, 16, }, // America/Indiana/Winamac
    {  448, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  448, 5849,     207, 22, }, // America/Lower_Princes
    {  448, 8116,   25021,  6, }, // America/Maceio
    {  448, 8183,     229,  6, }, // America/Merida
    {  448, 2811,  139751,  9, }, // America/Mexico_City
    {  448, 8264,   92237, 19, }, // America/North_Dakota/Beulah
    {  448, 8292,   92256, 19, }, // America/North_Dakota/Center
    {  448, 8320,   92275, 22, }, // America/North_Dakota/New_Salem
    {  448,  393,     298,  4, }, // America/Nuuk
    {  448, 8440,   25137,  8, }, // America/Santarem
    {  448, 1862,   25145,  9, }, // America/Sao_Paulo
    {  448, 6862,     302, 16, }, // America/Scoresbysund
    {  448, 8471,   25154, 16, }, // America/St_Barthelemy
    {  448, 2045,     332, 10, }, // America/St_Johns
    {  448, 5780,     342,  9, }, // America/St_Kitts
    {  448, 5797,     351,  9, }, // America/St_Lucia
    {  448,  855,     360, 10, }, // America/St_Thomas
    {  448, 5830,     370, 11, }, // America/St_Vincent
    {  448, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  448, 5440,    5906,  5, }, // Asia/Aqtau
    {  448, 5270,   55725,  6, }, // Asia/Aqtobe
    {  448,  964,   57105,  9, }, // Asia/Ashgabat
    {  448, 3509,    4063,  6, }, // Asia/Baghdad
    {  448, 3680,    4088,  7, }, // Asia/Bishkek
    {  448, 4503,  139760,  6, }, // Asia/Chita
    {  448, 5963,  139766, 10, }, // Asia/Choibalsan
    {  448, 4442,    4102,  8, }, // Asia/Damascus
    {  448, 6092,   57125,  8, }, // Asia/Dushanbe
    {  448, 1226,  139776, 16, }, // Asia/Ho_Chi_Minh
    {  448, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  448, 6955,  139792,  8, }, // Asia/Jayapura
    {  448, 4218,    4162, 10, }, // Asia/Kamchatka
    {  448, 1152,    6018,  8, }, // Asia/Kathmandu
    {  448,  992,     422,  7, }, // Asia/Kolkata
    {  448, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  448, 1178,     429,  5, }, // Asia/Macau
    {  448, 6883,    4227,  6, }, // Asia/Muscat
    {  448, 2446,   25331,  7, }, // Asia/Nicosia
    {  448, 8671,    6103,  8, }, // Asia/Qostanay
    {  448, 4171,   25347,  9, }, // Asia/Qyzylorda
    {  448, 4683,   57154,  8, }, // Asia/Tashkent
    {  448, 2638,    6175,  7, }, // Asia/Tehran
    {  448, 1284,    6182,  6, }, // Asia/Thimphu
    {  448, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  448, 1126,   25383,  6, }, // Asia/Urumqi
    {  448, 4730,    6206,  7, }, // Asia/Yakutsk
    {  448, 1202,     442,  6, }, // Asia/Yangon
    {  448, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  448, 3649,    6231,  7, }, // Asia/Yerevan
    {  448, 3565,  139800,  7, }, // Atlantic/Azores
    {  448, 6591,  139807, 11, }, // Atlantic/Canary
    {  448, 3629,  139818, 10, }, // Atlantic/Cape_Verde
    {  448, 1379,  139828,  8, }, // Atlantic/Faroe
    {  448, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  448, 7235,  139836, 11, }, // Atlantic/South_Georgia
    {  448, 6784,     453, 10, }, // Atlantic/St_Helena
    {  448, 6577,    4339,  5, }, // Europe/Athens
    {  448, 6285,    6398,  7, }, // Europe/Belgrade
    {  448, 6178,    4351,  7, }, // Europe/Brussels
    {  448, 3830,  139847,  9, }, // Europe/Bucharest
    {  448, 8751,   25487,  8, }, // Europe/Busingen
    {  448, 2475,   62164,  8, }, // Europe/Chisinau
    {  448, 6238,   57194,  9, }, // Europe/Copenhagen
    {  448, 6561,  139856, 11, }, // Europe/Helsinki
    {  448, 8783,    4393,  3, }, // Europe/Isle_of_Man
    {  448, 2419,     463,  4, }, // Europe/Kyiv
    {  448, 3078,   19528,  6, }, // Europe/Lisbon
    {  448, 6314,    4414,  9, }, // Europe/Luxembourg
    {  448, 3443,   21290,  6, }, // Europe/Moscow
    {  448, 6224,   34765,  5, }, // Europe/Prague
    {  448, 6273,   19581,  4, }, // Europe/Rome
    {  448, 6135,   19608,  6, }, // Europe/Tirane
    {  448, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  448, 6516,  139867, 13, }, // Europe/Vatican
    {  448, 6164,   56035,  4, }, // Europe/Vienna
    {  448, 3055,   57212,  8, }, // Europe/Warsaw
    {  448, 6489,    4478,  6, }, // Europe/Zurich
    {  448, 5979,  139880, 12, }, // Indian/Christmas
    {  448, 5996,  139892, 10, }, // Indian/Cocos
    {  448, 4931,  139902,  8, }, // Indian/Comoro
    {  448, 7223,   25565,  4, }, // Indian/Mahe
    {  448, 7013,  139910,  9, }, // Indian/Maldives
    {  448, 7174,     467,  7, }, // Indian/Reunion
    {  448, 3022,     474,  5, }, // Pacific/Chuuk
    {  448, 2201,  139919,  8, }, // Pacific/Easter
    {  448, 6641,  139927, 14, }, // Pacific/Galapagos
    {  448, 2929,    4512,  8, }, // Pacific/Honolulu
    {  448, 2897,    4520,  9, }, // Pacific/Kanton
    {  448, 4070,  139941, 10, }, // Pacific/Norfolk
    {  448, 7061,    4529,  6, }, // Pacific/Noumea
    {  448, 2961,     479,  7, }, // Pacific/Pohnpei
    {  450, 4961,    3793, 11, }, // Africa/Addis_Ababa Norwegian Nynorsk/Latin/Norway
    {  450, 7383,   42592,  5, }, // Africa/Algiers
    {  450,   14,       0,  6, }, // Africa/Asmara
    {  450, 2242,   24913,  5, }, // Africa/Cairo
    {  450, 5032,   91972, 13, }, // Africa/Dar_es_Salaam
    {  450, 5068,   62570,  8, }, // Africa/El_Aaiun
    {  450, 6832,   18802,  4, }, // Africa/Lome
    {  450, 4337,       6,  8, }, // Africa/Sao_Tome
    {  450, 4456,   24992,  9, }, // America/Araguaina
    {  450,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  450, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  450, 4154,      14,  8, }, // America/Asuncion
    {  450,  237,      22,  8, }, // America/Atikokan
    {  450, 7586,   18884, 14, }, // America/Bahia_Banderas
    {  450, 7609,   34376,  5, }, // America/Belem
    {  450, 4263,   18898,  6, }, // America/Bogota
    {  450, 3770,      47,  6, }, // America/Cancun
    {  450, 5398,  139951, 11, }, // America/Cayman
    {  450, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  450, 3693,  139962,  6, }, // America/Cuiaba
    {  450, 5640,      66,  7, }, // America/Curacao
    {  450,  481,      73, 13, }, // America/Indiana/Knox
    {  450, 7924,      86, 16, }, // America/Indiana/Marengo
    {  450, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  450, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  450, 8001,     139, 14, }, // America/Indiana/Vevay
    {  450, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  450, 8049,     171, 16, }, // America/Indiana/Winamac
    {  450, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  450, 5849,     207, 22, }, // America/Lower_Princes
    {  450, 8116,   25021,  6, }, // America/Maceio
    {  450, 8183,    3899,  6, }, // America/Merida
    {  450, 2811,  139751,  9, }, // America/Mexico_City
    {  450, 8264,   92237, 19, }, // America/North_Dakota/Beulah
    {  450, 8292,   92256, 19, }, // America/North_Dakota/Center
    {  450, 8320,   92275, 22, }, // America/North_Dakota/New_Salem
    {  450,  393,     298,  4, }, // America/Nuuk
    {  450, 8440,   25137,  8, }, // America/Santarem
    {  450, 1862,   25145,  9, }, // America/Sao_Paulo
    {  450, 6862,     302, 16, }, // America/Scoresbysund
    {  450, 8471,   25154, 16, }, // America/St_Barthelemy
    {  450, 2045,     332, 10, }, // America/St_Johns
    {  450, 5780,     342,  9, }, // America/St_Kitts
    {  450, 5797,     351,  9, }, // America/St_Lucia
    {  450,  855,     360, 10, }, // America/St_Thomas
    {  450, 5830,     370, 11, }, // America/St_Vincent
    {  450, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  450, 5440,    5906,  5, }, // Asia/Aqtau
    {  450, 5270,    5911,  6, }, // Asia/Aqtobe
    {  450,  964,  139968,  8, }, // Asia/Ashgabat
    {  450, 3509,    4063,  6, }, // Asia/Baghdad
    {  450, 3680,    4088,  7, }, // Asia/Bishkek
    {  450, 4503,  139760,  6, }, // Asia/Chita
    {  450, 5963,   57114, 11, }, // Asia/Choibalsan
    {  450, 4442,    4102,  8, }, // Asia/Damascus
    {  450, 6092,   57125,  8, }, // Asia/Dushanbe
    {  450, 1226,  139776, 16, }, // Asia/Ho_Chi_Minh
    {  450, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  450, 4673,   25278,  5, }, // Asia/Hovd
    {  450, 6955,  139792,  8, }, // Asia/Jayapura
    {  450, 4218,    4162, 10, }, // Asia/Kamchatka
    {  450, 1152,    6018,  8, }, // Asia/Kathmandu
    {  450,  992,     422,  7, }, // Asia/Kolkata
    {  450, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  450, 1178,     429,  5, }, // Asia/Macau
    {  450, 6883,    4227,  6, }, // Asia/Muscat
    {  450, 2446,   25331,  7, }, // Asia/Nicosia
    {  450, 8671,    6103,  8, }, // Asia/Qostanay
    {  450, 4171,   25347,  9, }, // Asia/Qyzylorda
    {  450, 4683,   57154,  8, }, // Asia/Tashkent
    {  450, 2638,    6175,  7, }, // Asia/Tehran
    {  450, 1284,    6182,  6, }, // Asia/Thimphu
    {  450, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  450, 1126,   25383,  6, }, // Asia/Urumqi
    {  450, 4730,    6206,  7, }, // Asia/Yakutsk
    {  450, 1202,  139976,  6, }, // Asia/Yangon
    {  450, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  450, 3649,    6231,  7, }, // Asia/Yerevan
    {  450, 3565,  139982,  7, }, // Atlantic/Azores
    {  450, 6591,  139989, 11, }, // Atlantic/Canary
    {  450, 3629,  139818, 10, }, // Atlantic/Cape_Verde
    {  450, 1379,  140000,  8, }, // Atlantic/Faroe
    {  450, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  450, 7235,  139836, 11, }, // Atlantic/South_Georgia
    {  450, 6784,     453, 10, }, // Atlantic/St_Helena
    {  450, 6577,    4339,  5, }, // Europe/Athens
    {  450, 6285,    6398,  7, }, // Europe/Belgrade
    {  450, 6178,    4351,  7, }, // Europe/Brussels
    {  450, 3830,  139847,  9, }, // Europe/Bucharest
    {  450, 8751,   25487,  8, }, // Europe/Busingen
    {  450, 2475,   62164,  8, }, // Europe/Chisinau
    {  450, 6238,   57194,  9, }, // Europe/Copenhagen
    {  450, 6561,  139856, 11, }, // Europe/Helsinki
    {  450, 8783,    4393,  3, }, // Europe/Isle_of_Man
    {  450, 2419,     463,  4, }, // Europe/Kyiv
    {  450, 3078,   19528,  6, }, // Europe/Lisbon
    {  450, 6314,    4414,  9, }, // Europe/Luxembourg
    {  450, 3443,   21290,  6, }, // Europe/Moscow
    {  450, 6224,   34765,  5, }, // Europe/Prague
    {  450, 6273,   19581,  4, }, // Europe/Rome
    {  450, 6135,   19608,  6, }, // Europe/Tirane
    {  450, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  450, 6516,  139867, 13, }, // Europe/Vatican
    {  450, 6164,   56035,  4, }, // Europe/Vienna
    {  450, 3055,   57212,  8, }, // Europe/Warsaw
    {  450, 6489,    4478,  6, }, // Europe/Zurich
    {  450, 5979,  139880, 12, }, // Indian/Christmas
    {  450, 5996,  140008, 10, }, // Indian/Cocos
    {  450, 4931,  140018,  8, }, // Indian/Comoro
    {  450, 7223,   25565,  4, }, // Indian/Mahe
    {  450, 7013,  140026,  9, }, // Indian/Maldives
    {  450, 7174,     467,  7, }, // Indian/Reunion
    {  450, 3022,     474,  5, }, // Pacific/Chuuk
    {  450, 2201,  139919,  8, }, // Pacific/Easter
    {  450, 6641,  140035, 14, }, // Pacific/Galapagos
    {  450, 2929,    4512,  8, }, // Pacific/Honolulu
    {  450, 2897,    4520,  9, }, // Pacific/Kanton
    {  450, 4070,  139941, 10, }, // Pacific/Norfolk
    {  450, 7061,    4529,  6, }, // Pacific/Noumea
    {  450, 2961,     479,  7, }, // Pacific/Pohnpei
    {  451,   14,       0,  6, }, // Africa/Asmara Nuer/Latin/South Sudan
    {  451, 4337,       6,  8, }, // Africa/Sao_Tome
    {  451, 4154,      14,  8, }, // America/Asuncion
    {  451,  237,      22,  8, }, // America/Atikokan
    {  451, 7586,      30, 17, }, // America/Bahia_Banderas
    {  451, 3770,      47,  6, }, // America/Cancun
    {  451, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  451, 5640,      66,  7, }, // America/Curacao
    {  451,  481,      73, 13, }, // America/Indiana/Knox
    {  451, 7924,      86, 16, }, // America/Indiana/Marengo
    {  451, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  451, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  451, 8001,     139, 14, }, // America/Indiana/Vevay
    {  451, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  451, 8049,     171, 16, }, // America/Indiana/Winamac
    {  451, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  451, 5849,     207, 22, }, // America/Lower_Princes
    {  451, 8183,     229,  6, }, // America/Merida
    {  451, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  451, 8292,     255, 20, }, // America/North_Dakota/Center
    {  451, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  451,  393,     298,  4, }, // America/Nuuk
    {  451, 6862,     302, 16, }, // America/Scoresbysund
    {  451, 8471,     318, 14, }, // America/St_Barthelemy
    {  451, 2045,     332, 10, }, // America/St_Johns
    {  451, 5780,     342,  9, }, // America/St_Kitts
    {  451, 5797,     351,  9, }, // America/St_Lucia
    {  451,  855,     360, 10, }, // America/St_Thomas
    {  451, 5830,     370, 11, }, // America/St_Vincent
    {  451, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  451, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  451, 1152,     413,  9, }, // Asia/Kathmandu
    {  451,  992,     422,  7, }, // Asia/Kolkata
    {  451, 1178,     429,  5, }, // Asia/Macau
    {  451, 1202,     442,  6, }, // Asia/Yangon
    {  451, 1379,     448,  5, }, // Atlantic/Faroe
    {  451, 6784,     453, 10, }, // Atlantic/St_Helena
    {  451, 2419,     463,  4, }, // Europe/Kyiv
    {  451, 7174,     467,  7, }, // Indian/Reunion
    {  451, 3022,     474,  5, }, // Pacific/Chuuk
    {  451, 2961,     479,  7, }, // Pacific/Pohnpei
    {  452,   14,       0,  6, }, // Africa/Asmara Nyanja/Latin/Malawi
    {  452, 4337,       6,  8, }, // Africa/Sao_Tome
    {  452, 4154,      14,  8, }, // America/Asuncion
    {  452,  237,      22,  8, }, // America/Atikokan
    {  452, 7586,      30, 17, }, // America/Bahia_Banderas
    {  452, 3770,      47,  6, }, // America/Cancun
    {  452, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  452, 5640,      66,  7, }, // America/Curacao
    {  452,  481,      73, 13, }, // America/Indiana/Knox
    {  452, 7924,      86, 16, }, // America/Indiana/Marengo
    {  452, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  452, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  452, 8001,     139, 14, }, // America/Indiana/Vevay
    {  452, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  452, 8049,     171, 16, }, // America/Indiana/Winamac
    {  452, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  452, 5849,     207, 22, }, // America/Lower_Princes
    {  452, 8183,     229,  6, }, // America/Merida
    {  452, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  452, 8292,     255, 20, }, // America/North_Dakota/Center
    {  452, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  452,  393,     298,  4, }, // America/Nuuk
    {  452, 6862,     302, 16, }, // America/Scoresbysund
    {  452, 8471,     318, 14, }, // America/St_Barthelemy
    {  452, 2045,     332, 10, }, // America/St_Johns
    {  452, 5780,     342,  9, }, // America/St_Kitts
    {  452, 5797,     351,  9, }, // America/St_Lucia
    {  452,  855,     360, 10, }, // America/St_Thomas
    {  452, 5830,     370, 11, }, // America/St_Vincent
    {  452, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  452, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  452, 1152,     413,  9, }, // Asia/Kathmandu
    {  452,  992,     422,  7, }, // Asia/Kolkata
    {  452, 1178,     429,  5, }, // Asia/Macau
    {  452, 1202,     442,  6, }, // Asia/Yangon
    {  452, 1379,     448,  5, }, // Atlantic/Faroe
    {  452, 6784,     453, 10, }, // Atlantic/St_Helena
    {  452, 2419,     463,  4, }, // Europe/Kyiv
    {  452, 7174,     467,  7, }, // Indian/Reunion
    {  452, 3022,     474,  5, }, // Pacific/Chuuk
    {  452, 2961,     479,  7, }, // Pacific/Pohnpei
    {  453,   14,       0,  6, }, // Africa/Asmara Nyankole/Latin/Uganda
    {  453, 4337,       6,  8, }, // Africa/Sao_Tome
    {  453, 4154,      14,  8, }, // America/Asuncion
    {  453,  237,      22,  8, }, // America/Atikokan
    {  453, 7586,      30, 17, }, // America/Bahia_Banderas
    {  453, 3770,      47,  6, }, // America/Cancun
    {  453, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  453, 5640,      66,  7, }, // America/Curacao
    {  453,  481,      73, 13, }, // America/Indiana/Knox
    {  453, 7924,      86, 16, }, // America/Indiana/Marengo
    {  453, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  453, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  453, 8001,     139, 14, }, // America/Indiana/Vevay
    {  453, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  453, 8049,     171, 16, }, // America/Indiana/Winamac
    {  453, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  453, 5849,     207, 22, }, // America/Lower_Princes
    {  453, 8183,     229,  6, }, // America/Merida
    {  453, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  453, 8292,     255, 20, }, // America/North_Dakota/Center
    {  453, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  453,  393,     298,  4, }, // America/Nuuk
    {  453, 6862,     302, 16, }, // America/Scoresbysund
    {  453, 8471,     318, 14, }, // America/St_Barthelemy
    {  453, 2045,     332, 10, }, // America/St_Johns
    {  453, 5780,     342,  9, }, // America/St_Kitts
    {  453, 5797,     351,  9, }, // America/St_Lucia
    {  453,  855,     360, 10, }, // America/St_Thomas
    {  453, 5830,     370, 11, }, // America/St_Vincent
    {  453, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  453, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  453, 1152,     413,  9, }, // Asia/Kathmandu
    {  453,  992,     422,  7, }, // Asia/Kolkata
    {  453, 1178,     429,  5, }, // Asia/Macau
    {  453, 1202,     442,  6, }, // Asia/Yangon
    {  453, 1379,     448,  5, }, // Atlantic/Faroe
    {  453, 6784,     453, 10, }, // Atlantic/St_Helena
    {  453, 2419,     463,  4, }, // Europe/Kyiv
    {  453, 7174,     467,  7, }, // Indian/Reunion
    {  453, 3022,     474,  5, }, // Pacific/Chuuk
    {  453, 2961,     479,  7, }, // Pacific/Pohnpei
    {  454, 7383,  140049,  7, }, // Africa/Algiers Occitan/Latin/France
    {  454,   14,       0,  6, }, // Africa/Asmara
    {  454, 4945,  140056,  6, }, // Africa/Djibouti
    {  454, 5145,  140062,  5, }, // Africa/Douala
    {  454, 4337,       6,  8, }, // Africa/Sao_Tome
    {  454, 4154,      14,  8, }, // America/Asuncion
    {  454,  237,      22,  8, }, // America/Atikokan
    {  454, 7586,      30, 17, }, // America/Bahia_Banderas
    {  454, 3770,      47,  6, }, // America/Cancun
    {  454, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  454, 5640,      66,  7, }, // America/Curacao
    {  454,  481,      73, 13, }, // America/Indiana/Knox
    {  454, 7924,      86, 16, }, // America/Indiana/Marengo
    {  454, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  454, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  454, 8001,     139, 14, }, // America/Indiana/Vevay
    {  454, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  454, 8049,     171, 16, }, // America/Indiana/Winamac
    {  454, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  454, 5849,     207, 22, }, // America/Lower_Princes
    {  454, 8183,     229,  6, }, // America/Merida
    {  454, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  454, 8292,     255, 20, }, // America/North_Dakota/Center
    {  454, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  454,  393,     298,  4, }, // America/Nuuk
    {  454, 6862,     302, 16, }, // America/Scoresbysund
    {  454, 8471,     318, 14, }, // America/St_Barthelemy
    {  454, 2045,     332, 10, }, // America/St_Johns
    {  454, 5780,     342,  9, }, // America/St_Kitts
    {  454, 5797,     351,  9, }, // America/St_Lucia
    {  454,  855,     360, 10, }, // America/St_Thomas
    {  454, 5830,     370, 11, }, // America/St_Vincent
    {  454, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  454, 4368,  140067,  7, }, // Asia/Bangkok
    {  454, 1060,   64123,  5, }, // Asia/Dhaka
    {  454, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  454, 3462,  140074,  5, }, // Asia/Kabul
    {  454, 1152,     413,  9, }, // Asia/Kathmandu
    {  454,  992,   19229,  7, }, // Asia/Kolkata
    {  454, 1178,     429,  5, }, // Asia/Macau
    {  454, 4310,  140079,  7, }, // Asia/Sakhalin
    {  454, 3137,    6142,  8, }, // Asia/Singapore
    {  454, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  454, 1202,     442,  6, }, // Asia/Yangon
    {  454, 3785,  140086, 12, }, // Asia/Yekaterinburg
    {  454, 3565,  140098,  6, }, // Atlantic/Azores
    {  454, 6591,   42985,  8, }, // Atlantic/Canary
    {  454, 3629,   42993,  8, }, // Atlantic/Cape_Verde
    {  454, 1379,     448,  5, }, // Atlantic/Faroe
    {  454, 6784,     453, 10, }, // Atlantic/St_Helena
    {  454, 1654,  140104,  8, }, // Australia/Adelaide
    {  454, 6149,  140112,  7, }, // Europe/Andorra
    {  454, 6577,   19437,  6, }, // Europe/Athens
    {  454, 6285,   25465,  7, }, // Europe/Belgrade
    {  454, 6178,  140119,  9, }, // Europe/Brussels
    {  454, 3830,  140128,  8, }, // Europe/Bucharest
    {  454, 3708,  140136,  8, }, // Europe/Budapest
    {  454, 2419,     463,  4, }, // Europe/Kyiv
    {  454, 3078,   91939,  7, }, // Europe/Lisbon
    {  454, 2393,   19543,  7, }, // Europe/London
    {  454, 6314,  140144,  9, }, // Europe/Luxembourg
    {  454, 6359,  140153,  7, }, // Europe/Monaco
    {  454, 3443,  140160,  6, }, // Europe/Moscow
    {  454, 4186,   19571,  5, }, // Europe/Paris
    {  454, 6224,   19576,  5, }, // Europe/Prague
    {  454, 6273,   19581,  4, }, // Europe/Rome
    {  454, 7013,  140166,  8, }, // Indian/Maldives
    {  454, 7174,     467,  7, }, // Indian/Reunion
    {  454, 3022,     474,  5, }, // Pacific/Chuuk
    {  454, 2961,     479,  7, }, // Pacific/Pohnpei
    {  455, 6751,  140174,  6, }, // Africa/Abidjan Occitan/Latin/Spain
    {  455, 6723,   72588,  4, }, // Africa/Accra
    {  455, 4961,   18754, 10, }, // Africa/Addis_Ababa
    {  455, 7383,  140180,  5, }, // Africa/Algiers
    {  455, 7398,    4583,  5, }, // Africa/Bissau
    {  455, 2242,  140185,  9, }, // Africa/Cairo
    {  455, 4945,  140194,  6, }, // Africa/Djibouti
    {  455, 4426,  140200,  6, }, // Africa/Khartoum
    {  455, 6832,   18802,  4, }, // Africa/Lome
    {  455, 5015,   42614, 10, }, // Africa/Mogadishu
    {  455, 5173,  140206,  6, }, // Africa/Ndjamena
    {  455, 6766,  140212,  8, }, // Africa/Nouakchott
    {  455, 6690,   18833,  8, }, // Africa/Ouagadougou
    {  455, 5127,  117821,  9, }, // Africa/Porto-Novo
    {  455, 4337,  140220,  9, }, // Africa/Sao_Tome
    {  455, 2732,   18851,  7, }, // Africa/Tripoli
    {  455, 6503,   18858,  5, }, // Africa/Tunis
    {  455, 4456,   24992,  9, }, // America/Araguaina
    {  455,  270,  140229,  6, }, // America/Argentina/Cordoba
    {  455, 7452,   42641, 12, }, // America/Argentina/Rio_Gallegos
    {  455, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  455, 7586,   63067, 17, }, // America/Bahia_Banderas
    {  455, 7609,  140235,  5, }, // America/Belem
    {  455, 5309,  140240,  6, }, // America/Belize
    {  455, 4263,   18898,  6, }, // America/Bogota
    {  455, 4247,  140246,  6, }, // America/Cayenne
    {  455, 5398,   42677,  6, }, // America/Cayman
    {  455, 5324,  140252, 10, }, // America/Costa_Rica
    {  455, 3693,   25007,  6, }, // America/Cuiaba
    {  455, 7832,   25013,  8, }, // America/Eirunepe
    {  455, 4514,  140262, 10, }, // America/Grand_Turk
    {  455, 5687,   18910,  7, }, // America/Grenada
    {  455, 5703,   18917,  9, }, // America/Guadeloupe
    {  455, 2221,  140272, 10, }, // America/Havana
    {  455, 8116,   25021,  6, }, // America/Maceio
    {  455, 1892,  140282,  8, }, // America/Manaus
    {  455, 5722,   18945,  9, }, // America/Martinique
    {  455, 2811,  140290, 15, }, // America/Mexico_City
    {  455, 3294,  140305,  9, }, // America/New_York
    {  455, 8264,  140314, 24, }, // America/North_Dakota/Beulah
    {  455, 8292,  140338, 24, }, // America/North_Dakota/Center
    {  455, 8320,  140362, 27, }, // America/North_Dakota/New_Salem
    {  455, 5413,   42779,  6, }, // America/Panama
    {  455, 3847,  140389, 15, }, // America/Port-au-Prince
    {  455, 5871,  140404, 14, }, // America/Port_of_Spain
    {  455, 8440,   25137,  8, }, // America/Santarem
    {  455, 2165,   62647, 17, }, // America/Santiago
    {  455, 1862,  140418,  9, }, // America/Sao_Paulo
    {  455, 8471,  140427, 14, }, // America/St_Barthelemy
    {  455, 2045,  140441, 22, }, // America/St_Johns
    {  455, 5780,  140463, 14, }, // America/St_Kitts
    {  455, 5797,   72853, 11, }, // America/St_Lucia
    {  455, 5830,  140477, 11, }, // America/St_Vincent
    {  455, 5605,  140488,  7, }, // America/Tortola
    {  455, 5428,  140495,  6, }, // Asia/Anadyr
    {  455, 5440,    5906,  5, }, // Asia/Aqtau
    {  455, 5270,    5911,  6, }, // Asia/Aqtobe
    {  455,  964,  140501,  7, }, // Asia/Ashgabat
    {  455, 3509,    4063,  6, }, // Asia/Baghdad
    {  455, 5451,  140508,  6, }, // Asia/Bahrain
    {  455, 3555,   19138,  4, }, // Asia/Baku
    {  455, 3473,  140514,  7, }, // Asia/Barnaul
    {  455, 4503,   72892,  5, }, // Asia/Chita
    {  455, 4442,   72897,  7, }, // Asia/Damascus
    {  455, 6092,  140521,  7, }, // Asia/Dushanbe
    {  455, 1226,  140528, 18, }, // Asia/Ho_Chi_Minh
    {  455, 1257,  140546,  9, }, // Asia/Jerusalem
    {  455, 1152,   19221,  8, }, // Asia/Kathmandu
    {  455, 1316,    6069,  7, }, // Asia/Makassar
    {  455, 6883,   19248,  7, }, // Asia/Muscat
    {  455, 5476,   71561,  5, }, // Asia/Qatar
    {  455, 8671,   62738,  8, }, // Asia/Qostanay
    {  455, 4171,   25347,  9, }, // Asia/Qyzylorda
    {  455, 3486,    6120,  4, }, // Asia/Riyadh
    {  455, 4310,  140555,  7, }, // Asia/Sakhalin
    {  455, 7208,   19277, 10, }, // Asia/Samarkand
    {  455, 3116,  140562,  4, }, // Asia/Seoul
    {  455, 4199,    6150, 13, }, // Asia/Srednekolymsk
    {  455, 4683,   90276,  7, }, // Asia/Tashkent
    {  455, 3817,  140566,  7, }, // Asia/Tbilisi
    {  455, 2638,    6175,  7, }, // Asia/Tehran
    {  455, 1284,  140573,  5, }, // Asia/Thimphu
    {  455, 2687,    4257,  5, }, // Asia/Tokyo
    {  455, 1126,   25383,  6, }, // Asia/Urumqi
    {  455, 3649,  140578,  6, }, // Asia/Yerevan
    {  455, 5588,   42971,  8, }, // Atlantic/Bermuda
    {  455, 3629,   25418, 10, }, // Atlantic/Cape_Verde
    {  455, 1379,  140584, 11, }, // Atlantic/Faroe
    {  455, 2614,   19388,  9, }, // Atlantic/Reykjavik
    {  455, 7235,  140595, 16, }, // Atlantic/South_Georgia
    {  455, 6784,  140611, 11, }, // Atlantic/St_Helena
    {  455, 1447,  140622,  6, }, // Australia/Sydney
    {  455, 3522,   43027,  8, }, // Europe/Astrakhan
    {  455, 3830,   19464,  8, }, // Europe/Bucharest
    {  455, 8783,  140628, 11, }, // Europe/Isle_of_Man
    {  455, 1097,  140639,  8, }, // Europe/Istanbul
    {  455, 8924,   43065,  5, }, // Europe/Kirov
    {  455, 2419,    6486,  4, }, // Europe/Kyiv
    {  455, 3078,   19528,  6, }, // Europe/Lisbon
    {  455, 6455,   19534,  9, }, // Europe/Ljubljana
    {  455, 2393,  140647,  7, }, // Europe/London
    {  455, 6419,  140654, 11, }, // Europe/San_Marino
    {  455, 4353,  140665,  7, }, // Europe/Saratov
    {  455, 8828,   43093, 10, }, // Europe/Simferopol
    {  455, 6332,  140672,  6, }, // Europe/Skopje
    {  455, 6548,   19594,  5, }, // Europe/Sofia
    {  455, 6472,   19599,  8, }, // Europe/Stockholm
    {  455, 8846,   21335,  6, }, // Europe/Tallinn
    {  455, 6135,   19608,  6, }, // Europe/Tirane
    {  455, 8861,  140678,  9, }, // Europe/Ulyanovsk
    {  455, 6516,  140687, 11, }, // Europe/Vatican
    {  455, 6164,   19625,  5, }, // Europe/Vienna
    {  455, 8878,   62886,  5, }, // Europe/Vilnius
    {  455, 3055,   43125,  8, }, // Europe/Warsaw
    {  455, 6489,  140698,  6, }, // Europe/Zurich
    {  455, 5979,  140704,  5, }, // Indian/Christmas
    {  455, 4931,   63916,  7, }, // Indian/Comoro
    {  455, 7223,   25565,  4, }, // Indian/Mahe
    {  455, 3971,   19647,  7, }, // Indian/Mauritius
    {  455, 2201,  140709, 14, }, // Pacific/Easter
    {  455, 3804,  140723,  4, }, // Pacific/Fiji
    {  455, 6641,   19672,  9, }, // Pacific/Galapagos
    {  455, 3953,   43152,  9, }, // Pacific/Marquesas
    {  455, 7118,   63962,  6, }, // Pacific/Palau
    {  455, 7275,   43161,  6, }, // Pacific/Tahiti
    {  456, 6751,  140727,  7, }, // Africa/Abidjan Odia/Odia/India
    {  456, 6723,  140734,  5, }, // Africa/Accra
    {  456, 4961,  140739, 12, }, // Africa/Addis_Ababa
    {  456, 7383,  140751,  8, }, // Africa/Algiers
    {  456,   14,  140759,  5, }, // Africa/Asmara
    {  456,   44,  140764,  6, }, // Africa/Bamako
    {  456, 5159,  140770,  7, }, // Africa/Bangui
    {  456, 6709,  140777,  7, }, // Africa/Banjul
    {  456, 7398,  140784,  5, }, // Africa/Bissau
    {  456, 4859,  140789, 13, }, // Africa/Blantyre
    {  456, 5189,  140802, 12, }, // Africa/Brazzaville
    {  456, 4824,  140814, 10, }, // Africa/Bujumbura
    {  456, 2242,  140824,  5, }, // Africa/Cairo
    {  456, 4019,  140829, 12, }, // Africa/Casablanca
    {  456, 7412,  140841,  5, }, // Africa/Ceuta
    {  456, 6736,  140846,  8, }, // Africa/Conakry
    {  456, 6803,  140854,  4, }, // Africa/Dakar
    {  456, 5032,  140858, 15, }, // Africa/Dar_es_Salaam
    {  456, 4945,  140873,  6, }, // Africa/Djibouti
    {  456, 5145,  140879,  4, }, // Africa/Douala
    {  456, 5068,  140883,  9, }, // Africa/El_Aaiun
    {  456, 6816,  140892, 10, }, // Africa/Freetown
    {  456, 4808,  140902,  9, }, // Africa/Gaborone
    {  456, 4917,  140911,  5, }, // Africa/Harare
    {  456, 4381,  140916, 11, }, // Africa/Johannesburg
    {  456, 4401,  140927,  4, }, // Africa/Juba
    {  456, 5053,  140931,  8, }, // Africa/Kampala
    {  456, 4426,  140939,  7, }, // Africa/Khartoum
    {  456, 4875,  140946,  6, }, // Africa/Kigali
    {  456, 5208,  140952,  8, }, // Africa/Kinshasa
    {  456, 4646,  140960,  7, }, // Africa/Lagos
    {  456, 5238,  140967, 12, }, // Africa/Libreville
    {  456, 6832,  140979,  5, }, // Africa/Lome
    {  456, 5113,  140984,  7, }, // Africa/Luanda
    {  456, 4841,  140991, 10, }, // Africa/Lubumbashi
    {  456, 4903,  141001,  6, }, // Africa/Lusaka
    {  456, 5224,  141007,  6, }, // Africa/Malabo
    {  456, 4889,  141013,  6, }, // Africa/Maputo
    {  456, 5099,  141019,  6, }, // Africa/Maseru
    {  456, 5084,  141025,  7, }, // Africa/Mbabane
    {  456, 5015,  141032,  8, }, // Africa/Mogadishu
    {  456, 6997,  141040,  7, }, // Africa/Monrovia
    {  456, 3755,  141047,  7, }, // Africa/Nairobi
    {  456, 5173,  141054,  6, }, // Africa/Ndjamena
    {  456, 5256,  141060,  5, }, // Africa/Niamey
    {  456, 6766,  141065,  7, }, // Africa/Nouakchott
    {  456, 6690,  141072,  8, }, // Africa/Ouagadougou
    {  456, 5127,  141080,  9, }, // Africa/Porto-Novo
    {  456, 4337,  141089,  8, }, // Africa/Sao_Tome
    {  456, 2732,  141097,  8, }, // Africa/Tripoli
    {  456, 6503,  141105,  7, }, // Africa/Tunis
    {  456, 4054,  141112, 10, }, // Africa/Windhoek
    {  456,  132,  141122,  5, }, // America/Adak
    {  456, 3183,  141127,  8, }, // America/Anchorage
    {  456, 5524,  141135,  8, }, // America/Anguilla
    {  456, 5541,  141143,  8, }, // America/Antigua
    {  456, 4456,  141151,  8, }, // America/Araguaina
    {  456,  166,  141159, 14, }, // America/Argentina/Buenos_Aires
    {  456,   91,  141173, 10, }, // America/Argentina/Catamarca
    {  456,  270,  141183,  8, }, // America/Argentina/Cordoba
    {  456,  441,  141191,  5, }, // America/Argentina/Jujuy
    {  456, 7425,  141196,  8, }, // America/Argentina/La_Rioja
    {  456,  565,  141204,  8, }, // America/Argentina/Mendoza
    {  456, 7452,  141212, 14, }, // America/Argentina/Rio_Gallegos
    {  456, 7483,  141226,  5, }, // America/Argentina/Salta
    {  456, 7507,  141231, 10, }, // America/Argentina/San_Juan
    {  456, 5497,  141241, 12, }, // America/Argentina/San_Luis
    {  456, 7534,  141253,  9, }, // America/Argentina/Tucuman
    {  456, 7560,  141262,  5, }, // America/Argentina/Ushuaia
    {  456, 5557,  141267,  5, }, // America/Aruba
    {  456, 4154,  141272,  9, }, // America/Asuncion
    {  456,  237,  141281,  9, }, // America/Atikokan
    {  456, 3581,  141290,  5, }, // America/Bahia
    {  456, 7586,  141295, 18, }, // America/Bahia_Banderas
    {  456, 5571,  141313, 10, }, // America/Barbados
    {  456, 7609,  141323,  5, }, // America/Belem
    {  456, 5309,  141328,  6, }, // America/Belize
    {  456, 7623,  141334, 15, }, // America/Blanc-Sablon
    {  456, 7644,  141349, 10, }, // America/Boa_Vista
    {  456, 4263,  141359,  6, }, // America/Bogota
    {  456, 7662,  141365,  5, }, // America/Boise
    {  456, 7676,  141370, 13, }, // America/Cambridge_Bay
    {  456, 7698,  141383, 15, }, // America/Campo_Grande
    {  456, 3770,  141398,  7, }, // America/Cancun
    {  456, 4596,  141405,  8, }, // America/Caracas
    {  456, 4247,  141413,  8, }, // America/Cayenne
    {  456, 5398,  141421,  9, }, // America/Cayman
    {  456, 3251,  141430,  6, }, // America/Chicago
    {  456, 7719,  141436,  8, }, // America/Chihuahua
    {  456, 7737,  141444, 15, }, // America/Ciudad_Juarez
    {  456, 5324,  141459, 11, }, // America/Costa_Rica
    {  456, 7759,  141470, 10, }, // America/Creston
    {  456, 3693,  141480,  5, }, // America/Cuiaba
    {  456, 5640,  141485,  6, }, // America/Curacao
    {  456, 7775,  141491, 13, }, // America/Danmarkshavn
    {  456, 7796,  141504,  5, }, // America/Dawson
    {  456, 7811,  141509, 12, }, // America/Dawson_Creek
    {  456,  805,  141521,  7, }, // America/Denver
    {  456, 3351,  141528,  9, }, // America/Detroit
    {  456, 5656,  141537,  8, }, // America/Dominica
    {  456,  893,  141545,  9, }, // America/Edmonton
    {  456, 7832,  141554,  7, }, // America/Eirunepe
    {  456, 5343,  141561, 13, }, // America/El_Salvador
    {  456, 7849,  141574, 13, }, // America/Fort_Nelson
    {  456, 7869,  141587, 10, }, // America/Fortaleza
    {  456, 7887,  141597,  9, }, // America/Glace_Bay
    {  456, 6844,  141606,  7, }, // America/Goose_Bay
    {  456, 4514,  141613, 14, }, // America/Grand_Turk
    {  456, 5687,  141627,  8, }, // America/Grenada
    {  456, 5703,  141635, 10, }, // America/Guadeloupe
    {  456, 3662,  141645,  9, }, // America/Guatemala
    {  456, 4743,  141654,  9, }, // America/Guayaquil
    {  456, 6895,  141663,  5, }, // America/Guyana
    {  456, 1923,  141668, 12, }, // America/Halifax
    {  456, 2221,  141680,  6, }, // America/Havana
    {  456, 7905,  141686,  9, }, // America/Hermosillo
    {  456,  348,  141695, 14, }, // America/Indiana/Indianapolis
    {  456,  481,  141709, 17, }, // America/Indiana/Knox
    {  456, 7924,  141726, 17, }, // America/Indiana/Marengo
    {  456, 7948,  141743, 22, }, // America/Indiana/Petersburg
    {  456, 7975,  141765, 19, }, // America/Indiana/Tell_City
    {  456, 8001,  141784, 15, }, // America/Indiana/Vevay
    {  456, 8023,  141799, 21, }, // America/Indiana/Vincennes
    {  456, 8049,  141820, 18, }, // America/Indiana/Winamac
    {  456, 8073,  141838,  7, }, // America/Inuvik
    {  456,  660,  141845, 10, }, // America/Iqaluit
    {  456, 2665,  141855,  7, }, // America/Jamaica
    {  456, 5282,  141862,  6, }, // America/Juneau
    {  456,  521,  141868, 10, }, // America/Kentucky/Louisville
    {  456, 8088,  141878, 22, }, // America/Kentucky/Monticello
    {  456, 5621,  141900, 12, }, // America/Kralendijk
    {  456, 4278,  141912,  8, }, // America/La_Paz
    {  456, 7132,  141920,  4, }, // America/Lima
    {  456, 3390,  141924, 13, }, // America/Los_Angeles
    {  456, 5849,  141937, 23, }, // America/Lower_Princes
    {  456, 8116,  141960,  5, }, // America/Maceio
    {  456, 8131,  141965,  7, }, // America/Managua
    {  456, 1892,  141972,  8, }, // America/Manaus
    {  456, 5814,  141980,  8, }, // America/Marigot
    {  456, 5722,  141988, 12, }, // America/Martinique
    {  456, 8147,  142000, 10, }, // America/Matamoros
    {  456, 2779,  142010,  9, }, // America/Mazatlan
    {  456, 8165,  142019,  8, }, // America/Menominee
    {  456, 8183,  142027,  6, }, // America/Merida
    {  456, 8198,  142033, 10, }, // America/Metlakatla
    {  456, 2811,  142043, 13, }, // America/Mexico_City
    {  456, 4293,  142056, 10, }, // America/Miquelon
    {  456, 8217,  142066,  8, }, // America/Moncton
    {  456, 8233,  142074,  8, }, // America/Monterrey
    {  456, 4000,  142082, 10, }, // America/Montevideo
    {  456, 5741,  142092, 11, }, // America/Montserrat
    {  456, 5383,  142103,  5, }, // America/Nassau
    {  456, 3294,  142108, 11, }, // America/New_York
    {  456, 8251,  142119,  4, }, // America/Nome
    {  456, 1834,  142123,  8, }, // America/Noronha
    {  456, 8264,  142131, 20, }, // America/North_Dakota/Beulah
    {  456, 8292,  142151, 21, }, // America/North_Dakota/Center
    {  456, 8320,  142172, 25, }, // America/North_Dakota/New_Salem
    {  456,  393,  142197,  5, }, // America/Nuuk
    {  456, 8351,  142202,  7, }, // America/Ojinaga
    {  456, 5413,  142209,  6, }, // America/Panama
    {  456, 6106,  142215, 10, }, // America/Paramaribo
    {  456, 3224,  142225,  8, }, // America/Phoenix
    {  456, 3847,  142233, 18, }, // America/Port-au-Prince
    {  456, 5871,  142251, 17, }, // America/Port_of_Spain
    {  456, 8367,  142268, 11, }, // America/Porto_Velho
    {  456, 5760,  142279, 12, }, // America/Puerto_Rico
    {  456, 3932,  142291, 15, }, // America/Punta_Arenas
    {  456, 8387,  142306, 14, }, // America/Rankin_Inlet
    {  456, 8408,  142320,  6, }, // America/Recife
    {  456, 1979,  142326,  6, }, // America/Regina
    {  456, 8423,  142332,  8, }, // America/Resolute
    {  456,  695,  142340, 12, }, // America/Rio_Branco
    {  456, 8440,  142352, 11, }, // America/Santarem
    {  456, 2165,  142363,  9, }, // America/Santiago
    {  456, 6044,  142372, 15, }, // America/Santo_Domingo
    {  456, 1862,  142387,  9, }, // America/Sao_Paulo
    {  456, 6862,  142396, 17, }, // America/Scoresbysund
    {  456, 8457,  142413,  5, }, // America/Sitka
    {  456, 8471,  142418, 18, }, // America/St_Barthelemy
    {  456, 2045,  142436, 12, }, // America/St_Johns
    {  456, 5780,  142448, 13, }, // America/St_Kitts
    {  456, 5797,  142461, 13, }, // America/St_Lucia
    {  456,  855,  142474, 14, }, // America/St_Thomas
    {  456, 5830,  142488, 17, }, // America/St_Vincent
    {  456, 8493,  142505, 15, }, // America/Swift_Current
    {  456, 5363,  142520, 11, }, // America/Tegucigalpa
    {  456, 5673,  142531,  4, }, // America/Thule
    {  456,  313,  142535,  7, }, // America/Tijuana
    {  456,  608,  142542,  8, }, // America/Toronto
    {  456, 5605,  142550,  7, }, // America/Tortola
    {  456, 2077,  142557,  9, }, // America/Vancouver
    {  456, 2128,  142566, 11, }, // America/Whitehorse
    {  456,  734,  142577,  8, }, // America/Winnipeg
    {  456, 8515,  142585,  8, }, // America/Yakutat
    {  456, 5918,  140839,  4, }, // Antarctica/Casey
    {  456, 6027,  142593,  7, }, // Antarctica/Davis
    {  456, 6066,  142600, 19, }, // Antarctica/DumontDUrville
    {  456, 8531,  142619,  8, }, // Antarctica/Macquarie
    {  456, 7029,  142627,  6, }, // Antarctica/Mawson
    {  456, 7076,  142633, 13, }, // Antarctica/McMurdo
    {  456, 8552,  142646,  7, }, // Antarctica/Palmer
    {  456, 7189,  142653,  6, }, // Antarctica/Rothera
    {  456, 7258,  142659,  4, }, // Antarctica/Syowa
    {  456, 8570,  142663,  6, }, // Antarctica/Troll
    {  456, 7337,  142669,  9, }, // Antarctica/Vostok
    {  456, 1413,  142678, 13, }, // Arctic/Longyearbyen
    {  456, 5487,  142691,  6, }, // Asia/Aden
    {  456, 5297,  142697,  6, }, // Asia/Almaty
    {  456, 3870,  142703,  6, }, // Asia/Amman
    {  456, 5428,  142709,  8, }, // Asia/Anadyr
    {  456, 5440,  142717,  5, }, // Asia/Aqtau
    {  456, 5270,  142722,  7, }, // Asia/Aqtobe
    {  456,  964,  142729, 11, }, // Asia/Ashgabat
    {  456, 8587,  142740,  6, }, // Asia/Atyrau
    {  456, 3509,  142746,  8, }, // Asia/Baghdad
    {  456, 5451,  142754,  8, }, // Asia/Bahrain
    {  456, 3555,  141483,  4, }, // Asia/Baku
    {  456, 4368,  142762, 11, }, // Asia/Bangkok
    {  456, 3473,  142773,  8, }, // Asia/Barnaul
    {  456, 3988,  142781,  7, }, // Asia/Beirut
    {  456, 3680,  142788,  8, }, // Asia/Bishkek
    {  456, 5906,  142796,  7, }, // Asia/Brunei
    {  456, 4503,  142803,  4, }, // Asia/Chita
    {  456, 5963,  142807, 11, }, // Asia/Choibalsan
    {  456, 4413,  142818,  6, }, // Asia/Colombo
    {  456, 4442,  142824,  9, }, // Asia/Damascus
    {  456, 1060,  142833,  4, }, // Asia/Dhaka
    {  456, 6125,  142837,  6, }, // Asia/Dili
    {  456, 3498,  142843,  5, }, // Asia/Dubai
    {  456, 6092,  142848,  7, }, // Asia/Dushanbe
    {  456, 8599,  142855, 10, }, // Asia/Famagusta
    {  456, 8614,  142865,  4, }, // Asia/Gaza
    {  456, 4697,  142869,  8, }, // Asia/Hebron
    {  456, 1226,  142877, 16, }, // Asia/Ho_Chi_Minh
    {  456, 2576,  142893,  5, }, // Asia/Hong_Kong
    {  456, 4673,  142898,  6, }, // Asia/Hovd
    {  456, 4086,  142904,  8, }, // Asia/Irkutsk
    {  456, 6969,  142912,  9, }, // Asia/Jakarta
    {  456, 6955,  142921,  6, }, // Asia/Jayapura
    {  456, 1257,  142927,  8, }, // Asia/Jerusalem
    {  456, 3462,  142935,  5, }, // Asia/Kabul
    {  456, 4218,  142940,  8, }, // Asia/Kamchatka
    {  456, 4141,  142948,  5, }, // Asia/Karachi
    {  456, 1152,  142953,  9, }, // Asia/Kathmandu
    {  456, 8624,  142962,  9, }, // Asia/Khandyga
    {  456,  992,  142971,  7, }, // Asia/Kolkata
    {  456, 4099,  142978, 13, }, // Asia/Krasnoyarsk
    {  456, 4761,  142991, 12, }, // Asia/Kuala_Lumpur
    {  456, 5893,  143003,  5, }, // Asia/Kuching
    {  456, 5464,  143008,  4, }, // Asia/Kuwait
    {  456, 1178,  143012,  5, }, // Asia/Macau
    {  456, 3919,  143017,  8, }, // Asia/Magadan
    {  456, 1316,  143025,  9, }, // Asia/Makassar
    {  456, 7145,  143034,  6, }, // Asia/Manila
    {  456, 6883,  143040,  8, }, // Asia/Muscat
    {  456, 2446,  143048,  7, }, // Asia/Nicosia
    {  456, 8638,  143055, 13, }, // Asia/Novokuznetsk
    {  456, 4037,  143068, 12, }, // Asia/Novosibirsk
    {  456, 4131,  143080,  5, }, // Asia/Omsk
    {  456, 7108,  143085,  6, }, // Asia/Oral
    {  456, 6924,  143091, 11, }, // Asia/Phnom_Penh
    {  456, 8656,  143102, 12, }, // Asia/Pontianak
    {  456, 4116,  143114, 11, }, // Asia/Pyongyang
    {  456, 5476,  143125,  5, }, // Asia/Qatar
    {  456, 8671,  143130,  7, }, // Asia/Qostanay
    {  456, 4171,  143137, 10, }, // Asia/Qyzylorda
    {  456, 3486,  143147,  4, }, // Asia/Riyadh
    {  456, 4310,  143151,  7, }, // Asia/Sakhalin
    {  456, 7208,  143158,  7, }, // Asia/Samarkand
    {  456, 3116,  143165,  4, }, // Asia/Seoul
    {  456, 1020,  143169,  5, }, // Asia/Shanghai
    {  456, 3137,  143174, 11, }, // Asia/Singapore
    {  456, 4199,  143185, 15, }, // Asia/Srednekolymsk
    {  456, 3100,  143200,  6, }, // Asia/Taipei
    {  456, 4683,  143206,  8, }, // Asia/Tashkent
    {  456, 3817,  143214,  8, }, // Asia/Tbilisi
    {  456, 2638,  143222,  8, }, // Asia/Tehran
    {  456, 1284,  143230,  6, }, // Asia/Thimphu
    {  456, 2687,  143236,  5, }, // Asia/Tokyo
    {  456, 4474,  143241,  6, }, // Asia/Tomsk
    {  456, 1346,  143247, 12, }, // Asia/Ulaanbaatar
    {  456, 1126,  143259,  6, }, // Asia/Urumqi
    {  456, 8685,  143265, 10, }, // Asia/Ust-Nera
    {  456, 6940,  143275, 11, }, // Asia/Vientiane
    {  456, 4612,  143286, 14, }, // Asia/Vladivostok
    {  456, 4730,  143300,  8, }, // Asia/Yakutsk
    {  456, 1202,  143308,  9, }, // Asia/Yangon
    {  456, 3785,  143317, 14, }, // Asia/Yekaterinburg
    {  456, 3649,  143331,  8, }, // Asia/Yerevan
    {  456, 3565,  143339,  7, }, // Atlantic/Azores
    {  456, 5588,  143346,  7, }, // Atlantic/Bermuda
    {  456, 6591,  143353,  6, }, // Atlantic/Canary
    {  456, 3629,  143359, 11, }, // Atlantic/Cape_Verde
    {  456, 1379,  143370,  5, }, // Atlantic/Faroe
    {  456, 8699,  143375,  7, }, // Atlantic/Madeira
    {  456, 2614,  143382, 10, }, // Atlantic/Reykjavik
    {  456, 7235,  143392, 13, }, // Atlantic/South_Georgia
    {  456, 6784,  143405, 14, }, // Atlantic/St_Helena
    {  456, 6607,  143419,  7, }, // Atlantic/Stanley
    {  456, 1654,  143426,  8, }, // Australia/Adelaide
    {  456, 1619,  143434, 10, }, // Australia/Brisbane
    {  456, 1783,  143444, 15, }, // Australia/Broken_Hill
    {  456, 1567,  143459,  8, }, // Australia/Darwin
    {  456, 3539,  143467,  5, }, // Australia/Eucla
    {  456, 1500,  143472,  9, }, // Australia/Hobart
    {  456, 8716,  143481, 13, }, // Australia/Lindeman
    {  456, 1531,  143494,  9, }, // Australia/Lord_Howe
    {  456, 1711,  143503, 10, }, // Australia/Melbourne
    {  456, 1746,  143513,  6, }, // Australia/Perth
    {  456, 1447,  143519,  5, }, // Australia/Sydney
    {  456, 6390,  143524, 12, }, // Europe/Amsterdam
    {  456, 6149,  143536,  6, }, // Europe/Andorra
    {  456, 3522,  143542, 11, }, // Europe/Astrakhan
    {  456, 6577,  143553,  6, }, // Europe/Athens
    {  456, 6285,  143559,  9, }, // Europe/Belgrade
    {  456, 4659,  143568,  7, }, // Europe/Berlin
    {  456, 6437,  143575, 11, }, // Europe/Bratislava
    {  456, 6178,  143586,  8, }, // Europe/Brussels
    {  456, 3830,  143594, 10, }, // Europe/Bucharest
    {  456, 3708,  143604, 10, }, // Europe/Budapest
    {  456, 8751,  143614,  9, }, // Europe/Busingen
    {  456, 2475,  143623,  6, }, // Europe/Chisinau
    {  456, 6238,  143629, 11, }, // Europe/Copenhagen
    {  456, 2260,  143640,  6, }, // Europe/Dublin
    {  456, 6256,  143646, 11, }, // Europe/Gibraltar
    {  456, 8767,  143657,  7, }, // Europe/Guernsey
    {  456, 6561,  143664,  8, }, // Europe/Helsinki
    {  456, 8783,  143672, 13, }, // Europe/Isle_of_Man
    {  456, 1097,  143685,  9, }, // Europe/Istanbul
    {  456, 8802,  143694,  5, }, // Europe/Jersey
    {  456, 3881,  143699, 13, }, // Europe/Kaliningrad
    {  456, 8924,  143712,  5, }, // Europe/Kirov
    {  456, 2419,  143717,  5, }, // Europe/Kyiv
    {  456, 3078,  143722,  7, }, // Europe/Lisbon
    {  456, 6455,  143729, 10, }, // Europe/Ljubljana
    {  456, 2393,  143739,  6, }, // Europe/London
    {  456, 6314,  143745,  9, }, // Europe/Luxembourg
    {  456, 4794,  143754,  8, }, // Europe/Madrid
    {  456, 6346,  143762,  5, }, // Europe/Malta
    {  456, 6531,  143767, 10, }, // Europe/Mariehamn
    {  456, 3595,  143777,  6, }, // Europe/Minsk
    {  456, 6359,  143783,  6, }, // Europe/Monaco
    {  456, 3443,  143789,  5, }, // Europe/Moscow
    {  456, 6407,  143794,  4, }, // Europe/Oslo
    {  456, 4186,  143798,  6, }, // Europe/Paris
    {  456, 6373,  143804,  9, }, // Europe/Podgorica
    {  456, 6224,  143813,  6, }, // Europe/Prague
    {  456, 8816,  143819,  4, }, // Europe/Riga
    {  456, 6273,  143823,  4, }, // Europe/Rome
    {  456, 4233,  143827,  6, }, // Europe/Samara
    {  456, 6419,  143833, 11, }, // Europe/San_Marino
    {  456, 6194,  143844,  8, }, // Europe/Sarajevo
    {  456, 4353,  143852,  8, }, // Europe/Saratov
    {  456, 8828,  143860, 11, }, // Europe/Simferopol
    {  456, 6332,  143871,  7, }, // Europe/Skopje
    {  456, 6548,  143878,  5, }, // Europe/Sofia
    {  456, 6472,  143883,  9, }, // Europe/Stockholm
    {  456, 8846,  143892,  7, }, // Europe/Tallinn
    {  456, 6135,  143899,  6, }, // Europe/Tirane
    {  456, 8861,  143905, 11, }, // Europe/Ulyanovsk
    {  456, 6301,  143916,  5, }, // Europe/Vaduz
    {  456, 6516,  143921,  8, }, // Europe/Vatican
    {  456, 6164,  143929,  5, }, // Europe/Vienna
    {  456, 8878,  143934,  8, }, // Europe/Vilnius
    {  456, 4629,  143942, 11, }, // Europe/Volgograd
    {  456, 3055,  143953,  6, }, // Europe/Warsaw
    {  456, 6210,  143959,  8, }, // Europe/Zagreb
    {  456, 6489,  143967,  6, }, // Europe/Zurich
    {  456, 4980,  143973, 13, }, // Indian/Antananarivo
    {  456, 6910,  143986,  7, }, // Indian/Chagos
    {  456, 5979,  143993, 10, }, // Indian/Christmas
    {  456, 5996,  144003,  7, }, // Indian/Cocos
    {  456, 4931,  144010,  6, }, // Indian/Comoro
    {  456, 6624,  144016,  9, }, // Indian/Kerguelen
    {  456, 7223,  144025,  4, }, // Indian/Mahe
    {  456, 7013,  144029,  8, }, // Indian/Maldives
    {  456, 3971,  144037,  7, }, // Indian/Mauritius
    {  456, 5000,  144044,  6, }, // Indian/Mayotte
    {  456, 7174,  144050, 10, }, // Indian/Reunion
    {  456, 4324,  144060,  4, }, // Pacific/Apia
    {  456,  932,  144064,  7, }, // Pacific/Auckland
    {  456, 3608,  144071, 13, }, // Pacific/Bougainville
    {  456, 2863,  144084,  7, }, // Pacific/Chatham
    {  456, 3022,  144091,  5, }, // Pacific/Chuuk
    {  456, 2201,  144096,  7, }, // Pacific/Easter
    {  456, 7323,  144103,  6, }, // Pacific/Efate
    {  456, 7290,  144109,  7, }, // Pacific/Fakaofo
    {  456, 3804,  144116,  4, }, // Pacific/Fiji
    {  456, 7306,  144120,  8, }, // Pacific/Funafuti
    {  456, 6641,  144128,  9, }, // Pacific/Galapagos
    {  456, 6659,  144137, 10, }, // Pacific/Gambier
    {  456, 3724,  144147, 11, }, // Pacific/Guadalcanal
    {  456, 5935,  141969,  4, }, // Pacific/Guam
    {  456, 2929,  144158,  8, }, // Pacific/Honolulu
    {  456, 2897,  144166, 10, }, // Pacific/Kanton
    {  456, 3900,  144176, 10, }, // Pacific/Kiritimati
    {  456, 6982,  144186,  6, }, // Pacific/Kosrae
    {  456, 2708,  144192, 11, }, // Pacific/Kwajalein
    {  456, 4779,  144203,  6, }, // Pacific/Majuro
    {  456, 3953,  144209, 13, }, // Pacific/Marquesas
    {  456, 8909,  144222,  7, }, // Pacific/Midway
    {  456, 7047,  144229,  5, }, // Pacific/Nauru
    {  456, 7095,  144234,  4, }, // Pacific/Niue
    {  456, 4070,  144238,  7, }, // Pacific/Norfolk
    {  456, 7061,  144245,  5, }, // Pacific/Noumea
    {  456, 2991,  144250,  9, }, // Pacific/Pago_Pago
    {  456, 7118,  144259,  5, }, // Pacific/Palau
    {  456, 7157,  144264, 10, }, // Pacific/Pitcairn
    {  456, 2961,  144274,  7, }, // Pacific/Pohnpei
    {  456, 4709,  144281, 15, }, // Pacific/Port_Moresby
    {  456, 6009,  144296, 10, }, // Pacific/Rarotonga
    {  456, 5948,  144306,  7, }, // Pacific/Saipan
    {  456, 7275,  144313,  6, }, // Pacific/Tahiti
    {  456, 6675,  144319,  5, }, // Pacific/Tarawa
    {  456, 4485,  144324, 10, }, // Pacific/Tongatapu
    {  456, 7355,  144334,  4, }, // Pacific/Wake
    {  456, 7368,  144338,  7, }, // Pacific/Wallis
    {  457,   14,       0,  6, }, // Africa/Asmara Oromo/Latin/Ethiopia
    {  457, 4337,       6,  8, }, // Africa/Sao_Tome
    {  457, 4154,      14,  8, }, // America/Asuncion
    {  457,  237,      22,  8, }, // America/Atikokan
    {  457, 7586,      30, 17, }, // America/Bahia_Banderas
    {  457, 3770,      47,  6, }, // America/Cancun
    {  457, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  457, 5640,      66,  7, }, // America/Curacao
    {  457,  481,      73, 13, }, // America/Indiana/Knox
    {  457, 7924,      86, 16, }, // America/Indiana/Marengo
    {  457, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  457, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  457, 8001,     139, 14, }, // America/Indiana/Vevay
    {  457, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  457, 8049,     171, 16, }, // America/Indiana/Winamac
    {  457, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  457, 5849,     207, 22, }, // America/Lower_Princes
    {  457, 8183,     229,  6, }, // America/Merida
    {  457, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  457, 8292,     255, 20, }, // America/North_Dakota/Center
    {  457, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  457,  393,     298,  4, }, // America/Nuuk
    {  457, 6862,     302, 16, }, // America/Scoresbysund
    {  457, 8471,     318, 14, }, // America/St_Barthelemy
    {  457, 2045,     332, 10, }, // America/St_Johns
    {  457, 5780,     342,  9, }, // America/St_Kitts
    {  457, 5797,     351,  9, }, // America/St_Lucia
    {  457,  855,     360, 10, }, // America/St_Thomas
    {  457, 5830,     370, 11, }, // America/St_Vincent
    {  457, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  457, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  457, 1152,     413,  9, }, // Asia/Kathmandu
    {  457,  992,     422,  7, }, // Asia/Kolkata
    {  457, 1178,     429,  5, }, // Asia/Macau
    {  457, 1202,     442,  6, }, // Asia/Yangon
    {  457, 1379,     448,  5, }, // Atlantic/Faroe
    {  457, 6784,     453, 10, }, // Atlantic/St_Helena
    {  457, 2419,     463,  4, }, // Europe/Kyiv
    {  457, 7174,     467,  7, }, // Indian/Reunion
    {  457, 3022,     474,  5, }, // Pacific/Chuuk
    {  457, 2961,     479,  7, }, // Pacific/Pohnpei
    {  459,   14,       0,  6, }, // Africa/Asmara Osage/Osage/United States
    {  459, 4337,       6,  8, }, // Africa/Sao_Tome
    {  459, 4154,      14,  8, }, // America/Asuncion
    {  459,  237,      22,  8, }, // America/Atikokan
    {  459, 7586,      30, 17, }, // America/Bahia_Banderas
    {  459, 3770,      47,  6, }, // America/Cancun
    {  459, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  459, 5640,      66,  7, }, // America/Curacao
    {  459,  481,      73, 13, }, // America/Indiana/Knox
    {  459, 7924,      86, 16, }, // America/Indiana/Marengo
    {  459, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  459, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  459, 8001,     139, 14, }, // America/Indiana/Vevay
    {  459, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  459, 8049,     171, 16, }, // America/Indiana/Winamac
    {  459, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  459, 5849,     207, 22, }, // America/Lower_Princes
    {  459, 8183,     229,  6, }, // America/Merida
    {  459, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  459, 8292,     255, 20, }, // America/North_Dakota/Center
    {  459, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  459,  393,     298,  4, }, // America/Nuuk
    {  459, 6862,     302, 16, }, // America/Scoresbysund
    {  459, 8471,     318, 14, }, // America/St_Barthelemy
    {  459, 2045,     332, 10, }, // America/St_Johns
    {  459, 5780,     342,  9, }, // America/St_Kitts
    {  459, 5797,     351,  9, }, // America/St_Lucia
    {  459,  855,     360, 10, }, // America/St_Thomas
    {  459, 5830,     370, 11, }, // America/St_Vincent
    {  459, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  459, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  459, 1152,     413,  9, }, // Asia/Kathmandu
    {  459,  992,     422,  7, }, // Asia/Kolkata
    {  459, 1178,     429,  5, }, // Asia/Macau
    {  459, 1202,     442,  6, }, // Asia/Yangon
    {  459, 1379,     448,  5, }, // Atlantic/Faroe
    {  459, 6784,     453, 10, }, // Atlantic/St_Helena
    {  459, 2419,     463,  4, }, // Europe/Kyiv
    {  459, 7174,     467,  7, }, // Indian/Reunion
    {  459, 3022,     474,  5, }, // Pacific/Chuuk
    {  459, 2961,     479,  7, }, // Pacific/Pohnpei
    {  461,   14,       0,  6, }, // Africa/Asmara Ossetic/Cyrillic/Russia
    {  461, 4337,       6,  8, }, // Africa/Sao_Tome
    {  461, 4154,      14,  8, }, // America/Asuncion
    {  461,  237,      22,  8, }, // America/Atikokan
    {  461, 7586,      30, 17, }, // America/Bahia_Banderas
    {  461, 3770,      47,  6, }, // America/Cancun
    {  461, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  461, 5640,      66,  7, }, // America/Curacao
    {  461,  481,      73, 13, }, // America/Indiana/Knox
    {  461, 7924,      86, 16, }, // America/Indiana/Marengo
    {  461, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  461, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  461, 8001,     139, 14, }, // America/Indiana/Vevay
    {  461, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  461, 8049,     171, 16, }, // America/Indiana/Winamac
    {  461, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  461, 5849,     207, 22, }, // America/Lower_Princes
    {  461, 8183,     229,  6, }, // America/Merida
    {  461, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  461, 8292,     255, 20, }, // America/North_Dakota/Center
    {  461, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  461,  393,     298,  4, }, // America/Nuuk
    {  461, 6862,     302, 16, }, // America/Scoresbysund
    {  461, 8471,     318, 14, }, // America/St_Barthelemy
    {  461, 2045,     332, 10, }, // America/St_Johns
    {  461, 5780,     342,  9, }, // America/St_Kitts
    {  461, 5797,     351,  9, }, // America/St_Lucia
    {  461,  855,     360, 10, }, // America/St_Thomas
    {  461, 5830,     370, 11, }, // America/St_Vincent
    {  461, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  461, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  461, 1152,     413,  9, }, // Asia/Kathmandu
    {  461,  992,     422,  7, }, // Asia/Kolkata
    {  461, 1178,     429,  5, }, // Asia/Macau
    {  461, 3817,   33686,  6, }, // Asia/Tbilisi
    {  461, 1202,     442,  6, }, // Asia/Yangon
    {  461, 1379,     448,  5, }, // Atlantic/Faroe
    {  461, 6784,     453, 10, }, // Atlantic/St_Helena
    {  461, 2419,     463,  4, }, // Europe/Kyiv
    {  461, 3595,    3241,  5, }, // Europe/Minsk
    {  461, 3443,  144345,  6, }, // Europe/Moscow
    {  461, 7174,     467,  7, }, // Indian/Reunion
    {  461, 3022,     474,  5, }, // Pacific/Chuuk
    {  461, 2961,     479,  7, }, // Pacific/Pohnpei
    {  462,   14,       0,  6, }, // Africa/Asmara Papiamento/Latin/Curacao
    {  462, 4337,       6,  8, }, // Africa/Sao_Tome
    {  462, 4154,      14,  8, }, // America/Asuncion
    {  462,  237,      22,  8, }, // America/Atikokan
    {  462, 7586,      30, 17, }, // America/Bahia_Banderas
    {  462, 3770,      47,  6, }, // America/Cancun
    {  462, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  462, 5640,  144351,  6, }, // America/Curacao
    {  462,  481,      73, 13, }, // America/Indiana/Knox
    {  462, 7924,      86, 16, }, // America/Indiana/Marengo
    {  462, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  462, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  462, 8001,     139, 14, }, // America/Indiana/Vevay
    {  462, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  462, 8049,     171, 16, }, // America/Indiana/Winamac
    {  462, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  462, 5849,     207, 22, }, // America/Lower_Princes
    {  462, 8183,     229,  6, }, // America/Merida
    {  462, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  462, 8292,     255, 20, }, // America/North_Dakota/Center
    {  462, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  462,  393,     298,  4, }, // America/Nuuk
    {  462, 5413,   19056,  6, }, // America/Panama
    {  462, 6044,  144357, 19, }, // America/Santo_Domingo
    {  462, 6862,     302, 16, }, // America/Scoresbysund
    {  462, 8471,     318, 14, }, // America/St_Barthelemy
    {  462, 2045,    3994, 11, }, // America/St_Johns
    {  462, 5780,    4005, 10, }, // America/St_Kitts
    {  462, 5797,    4015, 10, }, // America/St_Lucia
    {  462,  855,    4025, 11, }, // America/St_Thomas
    {  462, 5830,    4036, 12, }, // America/St_Vincent
    {  462, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  462, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  462, 1257,  144376,  9, }, // Asia/Jerusalem
    {  462, 1152,     413,  9, }, // Asia/Kathmandu
    {  462,  992,     422,  7, }, // Asia/Kolkata
    {  462, 1178,     429,  5, }, // Asia/Macau
    {  462, 1202,     442,  6, }, // Asia/Yangon
    {  462, 1379,     448,  5, }, // Atlantic/Faroe
    {  462, 6784,     453, 10, }, // Atlantic/St_Helena
    {  462, 2419,     463,  4, }, // Europe/Kyiv
    {  462, 6273,   19581,  4, }, // Europe/Rome
    {  462, 7174,     467,  7, }, // Indian/Reunion
    {  462, 3022,     474,  5, }, // Pacific/Chuuk
    {  462, 2961,     479,  7, }, // Pacific/Pohnpei
    {  464, 6751,  144385,  6, }, // Africa/Abidjan Pashto/Arabic/Afghanistan
    {  464, 6723,  144391,  4, }, // Africa/Accra
    {  464, 4961,  144395, 10, }, // Africa/Addis_Ababa
    {  464, 7383,  144405,  7, }, // Africa/Algiers
    {  464,   14,  144412,  6, }, // Africa/Asmara
    {  464,   44,  144418,  6, }, // Africa/Bamako
    {  464, 5159,  144424,  6, }, // Africa/Bangui
    {  464, 6709,    8902,  6, }, // Africa/Banjul
    {  464, 7398,  144430,  4, }, // Africa/Bissau
    {  464, 4859,  144434,  7, }, // Africa/Blantyre
    {  464, 5189,  144441,  8, }, // Africa/Brazzaville
    {  464, 4824,  144449,  7, }, // Africa/Bujumbura
    {  464, 2242,  144456,  5, }, // Africa/Cairo
    {  464, 4019,  144461, 10, }, // Africa/Casablanca
    {  464, 7412,  144471,  5, }, // Africa/Ceuta
    {  464, 6736,  144476,  6, }, // Africa/Conakry
    {  464, 6803,  144482,  5, }, // Africa/Dakar
    {  464, 5032,    8974, 10, }, // Africa/Dar_es_Salaam
    {  464, 4945,  144487,  5, }, // Africa/Djibouti
    {  464, 5145,    8990,  5, }, // Africa/Douala
    {  464, 5068,  144492,  5, }, // Africa/El_Aaiun
    {  464, 6816,  144497,  6, }, // Africa/Freetown
    {  464, 4808,  144503,  6, }, // Africa/Gaborone
    {  464, 4917,  144509,  5, }, // Africa/Harare
    {  464, 4381,  144514,  9, }, // Africa/Johannesburg
    {  464, 4401,    9030,  4, }, // Africa/Juba
    {  464, 5053,  144523,  6, }, // Africa/Kampala
    {  464, 4426,  144529,  5, }, // Africa/Khartoum
    {  464, 4875,  144534,  6, }, // Africa/Kigali
    {  464, 5208,  144540,  7, }, // Africa/Kinshasa
    {  464, 4646,  144547,  5, }, // Africa/Lagos
    {  464, 5238,  144552,  6, }, // Africa/Libreville
    {  464, 6832,  144558,  4, }, // Africa/Lome
    {  464, 5113,  144562,  5, }, // Africa/Luanda
    {  464, 4841,  144567,  7, }, // Africa/Lubumbashi
    {  464, 4903,  101910,  6, }, // Africa/Lusaka
    {  464, 5224,    9097,  6, }, // Africa/Malabo
    {  464, 4889,  144574,  6, }, // Africa/Maputo
    {  464, 5099,  144580,  5, }, // Africa/Maseru
    {  464, 5084,  144585,  7, }, // Africa/Mbabane
    {  464, 5015,  144592,  7, }, // Africa/Mogadishu
    {  464, 6997,  144599,  8, }, // Africa/Monrovia
    {  464, 3755,  144607,  7, }, // Africa/Nairobi
    {  464, 5173,  144614,  7, }, // Africa/Ndjamena
    {  464, 5256,  144621,  4, }, // Africa/Niamey
    {  464, 6766,  144625,  6, }, // Africa/Nouakchott
    {  464, 6690,  144631,  8, }, // Africa/Ouagadougou
    {  464, 5127,  144639, 11, }, // Africa/Porto-Novo
    {  464, 4337,  144650,  7, }, // Africa/Sao_Tome
    {  464, 2732,  144657,  7, }, // Africa/Tripoli
    {  464, 6503,    9194,  4, }, // Africa/Tunis
    {  464, 4054,  144664,  6, }, // Africa/Windhoek
    {  464,  132,  144670,  4, }, // America/Adak
    {  464, 3183,  144674,  7, }, // America/Anchorage
    {  464, 5524,  144681,  6, }, // America/Anguilla
    {  464, 5541,  144687,  5, }, // America/Antigua
    {  464, 4456,  144692,  7, }, // America/Araguaina
    {  464,  166,  144699, 11, }, // America/Argentina/Buenos_Aires
    {  464,   91,  144710,  8, }, // America/Argentina/Catamarca
    {  464,  270,  144718,  7, }, // America/Argentina/Cordoba
    {  464,  441,  144725,  5, }, // America/Argentina/Jujuy
    {  464, 7425,  144730,  7, }, // America/Argentina/La_Rioja
    {  464,  565,  144737,  7, }, // America/Argentina/Mendoza
    {  464, 7452,  144744, 11, }, // America/Argentina/Rio_Gallegos
    {  464, 7483,  144755,  5, }, // America/Argentina/Salta
    {  464, 7507,  144760,  8, }, // America/Argentina/San_Juan
    {  464, 5497,  144768,  8, }, // America/Argentina/San_Luis
    {  464, 7534,  144776,  6, }, // America/Argentina/Tucuman
    {  464, 7560,  144782,  7, }, // America/Argentina/Ushuaia
    {  464, 5557,  144789,  5, }, // America/Aruba
    {  464, 4154,  144794,  8, }, // America/Asuncion
    {  464,  237,  144802,  7, }, // America/Atikokan
    {  464, 3581,  144809,  4, }, // America/Bahia
    {  464, 7586,  144813, 12, }, // America/Bahia_Banderas
    {  464, 5571,  144825,  8, }, // America/Barbados
    {  464, 7609,    9377,  3, }, // America/Belem
    {  464, 5309,  144833,  4, }, // America/Belize
    {  464, 7623,  144837, 12, }, // America/Blanc-Sablon
    {  464, 7644,  144849,  9, }, // America/Boa_Vista
    {  464, 4263,  144858,  5, }, // America/Bogota
    {  464, 7662,  144863,  3, }, // America/Boise
    {  464, 7676,  144866,  9, }, // America/Cambridge_Bay
    {  464, 7698,  144875,  9, }, // America/Campo_Grande
    {  464, 3770,  144884,  5, }, // America/Cancun
    {  464, 4596,  144889,  7, }, // America/Caracas
    {  464, 4247,  144896,  4, }, // America/Cayenne
    {  464, 5398,  144900,  4, }, // America/Cayman
    {  464, 3251,  144904,  6, }, // America/Chicago
    {  464, 7719,  144910,  7, }, // America/Chihuahua
    {  464, 7737,  144917, 13, }, // America/Ciudad_Juarez
    {  464, 5324,  144930,  9, }, // America/Costa_Rica
    {  464, 7759,  144939,  6, }, // America/Creston
    {  464, 3693,  144945,  6, }, // America/Cuiaba
    {  464, 5640,  144951,  7, }, // America/Curacao
    {  464, 7775,  144958,  9, }, // America/Danmarkshavn
    {  464, 7796,    9533,  5, }, // America/Dawson
    {  464, 7811,  144967,  9, }, // America/Dawson_Creek
    {  464,  805,  144976,  4, }, // America/Denver
    {  464, 3351,  144980,  7, }, // America/Detroit
    {  464, 5656,  144987,  8, }, // America/Dominica
    {  464,  893,  144995,  8, }, // America/Edmonton
    {  464, 7832,  145003,  7, }, // America/Eirunepe
    {  464, 5343,  145010, 11, }, // America/El_Salvador
    {  464, 7849,  145021, 10, }, // America/Fort_Nelson
    {  464, 7869,  145031,  8, }, // America/Fortaleza
    {  464, 7887,  145039,  8, }, // America/Glace_Bay
    {  464, 6844,  145047,  6, }, // America/Goose_Bay
    {  464, 4514,  145053,  7, }, // America/Grand_Turk
    {  464, 5687,  145060,  6, }, // America/Grenada
    {  464, 5703,  145066,  6, }, // America/Guadeloupe
    {  464, 3662,  145072,  8, }, // America/Guatemala
    {  464, 4743,  145080,  6, }, // America/Guayaquil
    {  464, 6895,  145086,  5, }, // America/Guyana
    {  464, 1923,  145091,  6, }, // America/Halifax
    {  464, 2221,  145097,  5, }, // America/Havana
    {  464, 7905,  145102,  8, }, // America/Hermosillo
    {  464,  348,  145110, 13, }, // America/Indiana/Indianapolis
    {  464,  481,  145123, 12, }, // America/Indiana/Knox
    {  464, 7924,  145135, 15, }, // America/Indiana/Marengo
    {  464, 7948,  145150, 16, }, // America/Indiana/Petersburg
    {  464, 7975,  145166, 15, }, // America/Indiana/Tell_City
    {  464, 8001,  145181, 13, }, // America/Indiana/Vevay
    {  464, 8023,  145194, 16, }, // America/Indiana/Vincennes
    {  464, 8049,  145210, 14, }, // America/Indiana/Winamac
    {  464, 8073,  145224,  4, }, // America/Inuvik
    {  464,  660,  145228,  5, }, // America/Iqaluit
    {  464, 2665,  145233,  6, }, // America/Jamaica
    {  464, 5282,  145239,  4, }, // America/Juneau
    {  464,  521,  145243,  8, }, // America/Kentucky/Louisville
    {  464, 8088,  145251, 17, }, // America/Kentucky/Monticello
    {  464, 5621,  145268,  8, }, // America/Kralendijk
    {  464, 4278,  102731,  6, }, // America/La_Paz
    {  464, 7132,  145276,  4, }, // America/Lima
    {  464, 3390,  145280, 10, }, // America/Los_Angeles
    {  464, 5849,  145290, 16, }, // America/Lower_Princes
    {  464, 8116,  145306,  5, }, // America/Maceio
    {  464, 8131,  145311,  5, }, // America/Managua
    {  464, 1892,  145316,  5, }, // America/Manaus
    {  464, 5814,  145321,  7, }, // America/Marigot
    {  464, 5722,  145328,  8, }, // America/Martinique
    {  464, 8147,  145336,  9, }, // America/Matamoros
    {  464, 2779,  145345,  7, }, // America/Mazatlan
    {  464, 8165,  145352,  7, }, // America/Menominee
    {  464, 8183,  145359,  6, }, // America/Merida
    {  464, 8198,  145365, 10, }, // America/Metlakatla
    {  464, 2811,  145375, 10, }, // America/Mexico_City
    {  464, 4293,  145385,  8, }, // America/Miquelon
    {  464, 8217,  145393,  7, }, // America/Moncton
    {  464, 8233,  145400,  6, }, // America/Monterrey
    {  464, 4000,  145406,  9, }, // America/Montevideo
    {  464, 5741,  145415,  9, }, // America/Montserrat
    {  464, 5383,  145424,  5, }, // America/Nassau
    {  464, 3294,  145429,  7, }, // America/New_York
    {  464, 8251,    9897,  3, }, // America/Nome
    {  464, 1834,  145436,  7, }, // America/Noronha
    {  464, 8264,  145443, 19, }, // America/North_Dakota/Beulah
    {  464, 8292,  145462, 18, }, // America/North_Dakota/Center
    {  464, 8320,  145480, 22, }, // America/North_Dakota/New_Salem
    {  464,  393,  145502,  4, }, // America/Nuuk
    {  464, 8351,  145506,  6, }, // America/Ojinaga
    {  464, 5413,  145512,  6, }, // America/Panama
    {  464, 6106,  145518,  9, }, // America/Paramaribo
    {  464, 3224,  145527,  5, }, // America/Phoenix
    {  464, 3847,  145532, 15, }, // America/Port-au-Prince
    {  464, 5871,  145547, 14, }, // America/Port_of_Spain
    {  464, 8367,  145561, 11, }, // America/Porto_Velho
    {  464, 5760,  145572, 10, }, // America/Puerto_Rico
    {  464, 3932,  145582, 11, }, // America/Punta_Arenas
    {  464, 8387,  145593, 10, }, // America/Rankin_Inlet
    {  464, 8408,  145603,  6, }, // America/Recife
    {  464, 1979,  145609,  6, }, // America/Regina
    {  464, 8423,  145615,  7, }, // America/Resolute
    {  464,  695,  145622, 10, }, // America/Rio_Branco
    {  464, 8440,  145632,  6, }, // America/Santarem
    {  464, 2165,  145638,  7, }, // America/Santiago
    {  464, 6044,  145645, 12, }, // America/Santo_Domingo
    {  464, 1862,  145657,  9, }, // America/Sao_Paulo
    {  464, 6862,  145666, 11, }, // America/Scoresbysund
    {  464, 8457,  145677,  5, }, // America/Sitka
    {  464, 8471,  145682, 13, }, // America/St_Barthelemy
    {  464, 2045,  145695,  9, }, // America/St_Johns
    {  464, 5780,  145704,  8, }, // America/St_Kitts
    {  464, 5797,  145712, 10, }, // America/St_Lucia
    {  464,  855,  145722, 10, }, // America/St_Thomas
    {  464, 5830,  145732, 10, }, // America/St_Vincent
    {  464, 8493,  145742, 10, }, // America/Swift_Current
    {  464, 5363,  145752,  9, }, // America/Tegucigalpa
    {  464, 5673,   10304,  3, }, // America/Thule
    {  464,  313,  145761,  6, }, // America/Tijuana
    {  464,  608,  145767,  6, }, // America/Toronto
    {  464, 5605,  145773,  7, }, // America/Tortola
    {  464, 2077,  145780,  7, }, // America/Vancouver
    {  464, 2128,  145787,  9, }, // America/Whitehorse
    {  464,  734,  145796,  6, }, // America/Winnipeg
    {  464, 8515,  145802,  6, }, // America/Yakutat
    {  464, 5918,  145808,  4, }, // Antarctica/Casey
    {  464, 6027,  145812,  4, }, // Antarctica/Davis
    {  464, 6066,  145816, 14, }, // Antarctica/DumontDUrville
    {  464, 8531,  145830,  6, }, // Antarctica/Macquarie
    {  464, 7029,  145836,  5, }, // Antarctica/Mawson
    {  464, 7076,  145841,  6, }, // Antarctica/McMurdo
    {  464, 8552,  145847,  5, }, // Antarctica/Palmer
    {  464, 7189,  145852,  5, }, // Antarctica/Rothera
    {  464, 7258,  145857,  4, }, // Antarctica/Syowa
    {  464, 8570,  145861,  4, }, // Antarctica/Troll
    {  464, 7337,  145865,  6, }, // Antarctica/Vostok
    {  464, 1413,  145871,  9, }, // Arctic/Longyearbyen
    {  464, 5487,  145880,  3, }, // Asia/Aden
    {  464, 5297,  145883,  6, }, // Asia/Almaty
    {  464, 3870,  145889,  5, }, // Asia/Amman
    {  464, 5428,  145894,  6, }, // Asia/Anadyr
    {  464, 5440,  145900,  5, }, // Asia/Aqtau
    {  464, 5270,  145905,  5, }, // Asia/Aqtobe
    {  464,  964,  145910,  8, }, // Asia/Ashgabat
    {  464, 8587,  145918,  6, }, // Asia/Atyrau
    {  464, 3509,   10466,  5, }, // Asia/Baghdad
    {  464, 5451,  145924,  5, }, // Asia/Bahrain
    {  464, 3555,  144949,  4, }, // Asia/Baku
    {  464, 4368,  145929,  5, }, // Asia/Bangkok
    {  464, 3473,  103466,  5, }, // Asia/Barnaul
    {  464, 3988,  145934,  5, }, // Asia/Beirut
    {  464, 3680,  145939,  5, }, // Asia/Bishkek
    {  464, 5906,  145944,  6, }, // Asia/Brunei
    {  464, 4503,  145950,  4, }, // Asia/Chita
    {  464, 5963,  145954,  8, }, // Asia/Choibalsan
    {  464, 4413,  145962,  6, }, // Asia/Colombo
    {  464, 4442,   10528,  4, }, // Asia/Damascus
    {  464, 1060,  145968,  5, }, // Asia/Dhaka
    {  464, 6125,  145973,  3, }, // Asia/Dili
    {  464, 3498,  145976,  4, }, // Asia/Dubai
    {  464, 6092,  145980,  6, }, // Asia/Dushanbe
    {  464, 8599,  145986,  8, }, // Asia/Famagusta
    {  464, 8614,  145994,  3, }, // Asia/Gaza
    {  464, 4697,  145997,  5, }, // Asia/Hebron
    {  464, 1226,  146002, 12, }, // Asia/Ho_Chi_Minh
    {  464, 2576,  146014,  9, }, // Asia/Hong_Kong
    {  464, 4673,  146023,  4, }, // Asia/Hovd
    {  464, 4086,  146027,  7, }, // Asia/Irkutsk
    {  464, 6969,  103582,  6, }, // Asia/Jakarta
    {  464, 6955,  146034,  8, }, // Asia/Jayapura
    {  464, 1257,  146042,  6, }, // Asia/Jerusalem
    {  464, 3462,  146048,  4, }, // Asia/Kabul
    {  464, 4218,  146052,  8, }, // Asia/Kamchatka
    {  464, 4141,  146060,  5, }, // Asia/Karachi
    {  464, 1152,  146065,  6, }, // Asia/Kathmandu
    {  464, 8624,  146071,  6, }, // Asia/Khandyga
    {  464,  992,  146077,  6, }, // Asia/Kolkata
    {  464, 4099,  146083, 11, }, // Asia/Krasnoyarsk
    {  464, 4761,  146094,  9, }, // Asia/Kuala_Lumpur
    {  464, 5893,  146103,  5, }, // Asia/Kuching
    {  464, 5464,  146108,  4, }, // Asia/Kuwait
    {  464, 1178,  146112,  4, }, // Asia/Macau
    {  464, 3919,  146116,  5, }, // Asia/Magadan
    {  464, 1316,  146121,  6, }, // Asia/Makassar
    {  464, 7145,  146127,  5, }, // Asia/Manila
    {  464, 6883,   10734,  4, }, // Asia/Muscat
    {  464, 2446,  146132,  7, }, // Asia/Nicosia
    {  464, 8638,  146139, 10, }, // Asia/Novokuznetsk
    {  464, 4037,  146149,  9, }, // Asia/Novosibirsk
    {  464, 4131,  103736,  5, }, // Asia/Omsk
    {  464, 7108,  146158,  4, }, // Asia/Oral
    {  464, 6924,  146162,  7, }, // Asia/Phnom_Penh
    {  464, 8656,  146169,  8, }, // Asia/Pontianak
    {  464, 4116,  146177, 10, }, // Asia/Pyongyang
    {  464, 5476,   10802,  3, }, // Asia/Qatar
    {  464, 8671,  146187,  7, }, // Asia/Qostanay
    {  464, 4171,  146194, 10, }, // Asia/Qyzylorda
    {  464, 3486,   10824,  4, }, // Asia/Riyadh
    {  464, 4310,  146204,  6, }, // Asia/Sakhalin
    {  464, 7208,   10834,  6, }, // Asia/Samarkand
    {  464, 3116,  103809,  4, }, // Asia/Seoul
    {  464, 1020,  146210,  7, }, // Asia/Shanghai
    {  464, 3137,  146217,  8, }, // Asia/Singapore
    {  464, 4199,  146225, 13, }, // Asia/Srednekolymsk
    {  464, 3100,  146238,  5, }, // Asia/Taipei
    {  464, 4683,  146243,  6, }, // Asia/Tashkent
    {  464, 3817,  146249,  6, }, // Asia/Tbilisi
    {  464, 2638,  146255,  5, }, // Asia/Tehran
    {  464, 1284,  146260,  6, }, // Asia/Thimphu
    {  464, 2687,  146266,  5, }, // Asia/Tokyo
    {  464, 4474,    9107,  5, }, // Asia/Tomsk
    {  464, 1346,  146271, 10, }, // Asia/Ulaanbaatar
    {  464, 1126,  146281,  7, }, // Asia/Urumqi
    {  464, 8685,  146288,  8, }, // Asia/Ust-Nera
    {  464, 6940,  146296,  6, }, // Asia/Vientiane
    {  464, 4612,  146302, 10, }, // Asia/Vladivostok
    {  464, 4730,  146312,  7, }, // Asia/Yakutsk
    {  464, 1202,  146319,  6, }, // Asia/Yangon
    {  464, 3785,  146325, 10, }, // Asia/Yekaterinburg
    {  464, 3649,  146335,  6, }, // Asia/Yerevan
    {  464, 3565,  146341,  6, }, // Atlantic/Azores
    {  464, 5588,   10981,  6, }, // Atlantic/Bermuda
    {  464, 6591,  146347,  5, }, // Atlantic/Canary
    {  464, 3629,  146352,  7, }, // Atlantic/Cape_Verde
    {  464, 1379,   11004,  4, }, // Atlantic/Faroe
    {  464, 8699,  146359,  5, }, // Atlantic/Madeira
    {  464, 2614,  146364,  8, }, // Atlantic/Reykjavik
    {  464, 7235,  146372, 12, }, // Atlantic/South_Georgia
    {  464, 6784,  146384, 11, }, // Atlantic/St_Helena
    {  464, 6607,  146395,  5, }, // Atlantic/Stanley
    {  464, 1654,  146400,  7, }, // Australia/Adelaide
    {  464, 1619,  146407,  6, }, // Australia/Brisbane
    {  464, 1783,  146413,  8, }, // Australia/Broken_Hill
    {  464, 1567,  146421,  5, }, // Australia/Darwin
    {  464, 3539,  146426,  6, }, // Australia/Eucla
    {  464, 1500,  146432,  6, }, // Australia/Hobart
    {  464, 8716,  146438,  8, }, // Australia/Lindeman
    {  464, 1531,  146446,  8, }, // Australia/Lord_Howe
    {  464, 1711,  146454,  7, }, // Australia/Melbourne
    {  464, 1746,  146461,  3, }, // Australia/Perth
    {  464, 1447,  146464,  4, }, // Australia/Sydney
    {  464, 6390,  146468,  8, }, // Europe/Amsterdam
    {  464, 6149,  146476,  6, }, // Europe/Andorra
    {  464, 3522,  146482,  9, }, // Europe/Astrakhan
    {  464, 6577,  146491,  5, }, // Europe/Athens
    {  464, 6285,  146496,  5, }, // Europe/Belgrade
    {  464, 4659,  146501,  4, }, // Europe/Berlin
    {  464, 6437,  146505,  9, }, // Europe/Bratislava
    {  464, 6178,  146514,  6, }, // Europe/Brussels
    {  464, 3830,  146520,  6, }, // Europe/Bucharest
    {  464, 3708,  146526,  6, }, // Europe/Budapest
    {  464, 8751,  146532,  7, }, // Europe/Busingen
    {  464, 2475,  146539,  6, }, // Europe/Chisinau
    {  464, 6238,  146545,  9, }, // Europe/Copenhagen
    {  464, 2260,  146554,  4, }, // Europe/Dublin
    {  464, 6256,  146558, 10, }, // Europe/Gibraltar
    {  464, 8767,  146568,  5, }, // Europe/Guernsey
    {  464, 6561,  146573,  7, }, // Europe/Helsinki
    {  464, 8783,  146580, 12, }, // Europe/Isle_of_Man
    {  464, 1097,  146592,  7, }, // Europe/Istanbul
    {  464, 8802,  104261,  4, }, // Europe/Jersey
    {  464, 3881,  146599, 10, }, // Europe/Kaliningrad
    {  464, 8924,  146609,  5, }, // Europe/Kirov
    {  464, 2419,  146614,  3, }, // Europe/Kyiv
    {  464, 3078,  146617,  6, }, // Europe/Lisbon
    {  464, 6455,  146623,  7, }, // Europe/Ljubljana
    {  464, 2393,   11292,  4, }, // Europe/London
    {  464, 6314,  146630, 10, }, // Europe/Luxembourg
    {  464, 4794,  146640,  6, }, // Europe/Madrid
    {  464, 6346,  146646,  5, }, // Europe/Malta
    {  464, 6531,  146651,  7, }, // Europe/Mariehamn
    {  464, 3595,  146658,  4, }, // Europe/Minsk
    {  464, 6359,  104344,  6, }, // Europe/Monaco
    {  464, 3443,  104350,  5, }, // Europe/Moscow
    {  464, 6407,  104355,  5, }, // Europe/Oslo
    {  464, 4186,  146662,  5, }, // Europe/Paris
    {  464, 6373,  146667,  9, }, // Europe/Podgorica
    {  464, 6224,  146676,  4, }, // Europe/Prague
    {  464, 8816,  145323,  4, }, // Europe/Riga
    {  464, 6273,    9113,  3, }, // Europe/Rome
    {  464, 4233,  146680,  5, }, // Europe/Samara
    {  464, 6419,  146685, 10, }, // Europe/San_Marino
    {  464, 6194,  146695,  8, }, // Europe/Sarajevo
    {  464, 4353,  146703,  5, }, // Europe/Saratov
    {  464, 8828,  146708,  9, }, // Europe/Simferopol
    {  464, 6332,  146717,  5, }, // Europe/Skopje
    {  464, 6548,  146722,  5, }, // Europe/Sofia
    {  464, 6472,  146727,  7, }, // Europe/Stockholm
    {  464, 8846,   11422,  5, }, // Europe/Tallinn
    {  464, 6135,   11427,  5, }, // Europe/Tirane
    {  464, 8861,  146734,  8, }, // Europe/Ulyanovsk
    {  464, 6301,  146742,  5, }, // Europe/Vaduz
    {  464, 6516,  146747,  7, }, // Europe/Vatican
    {  464, 6164,  146754,  5, }, // Europe/Vienna
    {  464, 8878,  146759,  7, }, // Europe/Vilnius
    {  464, 4629,  146766,  9, }, // Europe/Volgograd
    {  464, 3055,  104500,  5, }, // Europe/Warsaw
    {  464, 6210,  146775,  4, }, // Europe/Zagreb
    {  464, 6489,  146779,  4, }, // Europe/Zurich
    {  464, 4980,  146783, 11, }, // Indian/Antananarivo
    {  464, 6910,  146794,  5, }, // Indian/Chagos
    {  464, 5979,  146799,  6, }, // Indian/Christmas
    {  464, 5996,  146805,  5, }, // Indian/Cocos
    {  464, 4931,  104545,  6, }, // Indian/Comoro
    {  464, 6624,  146810,  7, }, // Indian/Kerguelen
    {  464, 7223,   11539,  4, }, // Indian/Mahe
    {  464, 7013,  146817,  6, }, // Indian/Maldives
    {  464, 3971,  146823,  7, }, // Indian/Mauritius
    {  464, 5000,  146830,  5, }, // Indian/Mayotte
    {  464, 7174,  146835,  8, }, // Indian/Reunion
    {  464, 4324,  146843,  4, }, // Pacific/Apia
    {  464,  932,  146847,  5, }, // Pacific/Auckland
    {  464, 3608,  146852,  8, }, // Pacific/Bougainville
    {  464, 2863,  146860,  5, }, // Pacific/Chatham
    {  464, 3022,  146865,  3, }, // Pacific/Chuuk
    {  464, 2201,  146868,  5, }, // Pacific/Easter
    {  464, 7323,  146873,  4, }, // Pacific/Efate
    {  464, 7290,  146877,  6, }, // Pacific/Fakaofo
    {  464, 3804,  146883,  3, }, // Pacific/Fiji
    {  464, 7306,   11616,  8, }, // Pacific/Funafuti
    {  464, 6641,  146886,  7, }, // Pacific/Galapagos
    {  464, 6659,  146893,  6, }, // Pacific/Gambier
    {  464, 3724,  146899, 10, }, // Pacific/Guadalcanal
    {  464, 5935,  145313,  4, }, // Pacific/Guam
    {  464, 2929,  146909,  8, }, // Pacific/Honolulu
    {  464, 2897,  146917,  7, }, // Pacific/Kanton
    {  464, 3900,  146924,  9, }, // Pacific/Kiritimati
    {  464, 6982,  146933,  6, }, // Pacific/Kosrae
    {  464, 2708,  146939,  7, }, // Pacific/Kwajalein
    {  464, 4779,  146946,  5, }, // Pacific/Majuro
    {  464, 3953,  146951,  8, }, // Pacific/Marquesas
    {  464, 8909,  146959,  5, }, // Pacific/Midway
    {  464, 7047,  146964,  5, }, // Pacific/Nauru
    {  464, 7095,  146969,  4, }, // Pacific/Niue
    {  464, 4070,  146973,  5, }, // Pacific/Norfolk
    {  464, 7061,   11721,  5, }, // Pacific/Noumea
    {  464, 2991,  146978,  9, }, // Pacific/Pago_Pago
    {  464, 7118,  146987,  4, }, // Pacific/Palau
    {  464, 7157,  146991,  8, }, // Pacific/Pitcairn
    {  464, 2961,  146999,  5, }, // Pacific/Pohnpei
    {  464, 4709,  147004, 11, }, // Pacific/Port_Moresby
    {  464, 6009,  147015,  9, }, // Pacific/Rarotonga
    {  464, 5948,  147024,  5, }, // Pacific/Saipan
    {  464, 7275,  147029,  5, }, // Pacific/Tahiti
    {  464, 6675,   11783,  6, }, // Pacific/Tarawa
    {  464, 4485,  147034,  9, }, // Pacific/Tongatapu
    {  464, 7355,  104828,  3, }, // Pacific/Wake
    {  464, 7368,  147043,  4, }, // Pacific/Wallis
    {  465, 4917,  147047,  5, }, // Africa/Harare Pashto/Arabic/Pakistan
    {  465, 5849,  147052, 25, }, // America/Lower_Princes
    {  466, 6751,  147077,  6, }, // Africa/Abidjan Persian/Arabic/Iran
    {  466, 6723,  144391,  4, }, // Africa/Accra
    {  466, 4961,  147083, 10, }, // Africa/Addis_Ababa
    {  466, 7383,  147093,  7, }, // Africa/Algiers
    {  466,   14,  147100,  5, }, // Africa/Asmara
    {  466,   44,  147105,  6, }, // Africa/Bamako
    {  466, 5159,  147111,  5, }, // Africa/Bangui
    {  466, 6709,    8902,  6, }, // Africa/Banjul
    {  466, 7398,  147116,  6, }, // Africa/Bissau
    {  466, 4859,  147122,  8, }, // Africa/Blantyre
    {  466, 5189,  147130,  7, }, // Africa/Brazzaville
    {  466, 4824,    8929,  9, }, // Africa/Bujumbura
    {  466, 2242,  144456,  5, }, // Africa/Cairo
    {  466, 4019,  147137, 10, }, // Africa/Casablanca
    {  466, 7412,  147147,  4, }, // Africa/Ceuta
    {  466, 6736,  101781,  7, }, // Africa/Conakry
    {  466, 6803,  147151,  5, }, // Africa/Dakar
    {  466, 5032,  101793,  9, }, // Africa/Dar_es_Salaam
    {  466, 4945,  147156,  6, }, // Africa/Djibouti
    {  466, 5145,  147162,  5, }, // Africa/Douala
    {  466, 5068,  147167,  6, }, // Africa/El_Aaiun
    {  466, 6816,  147173,  8, }, // Africa/Freetown
    {  466, 4808,  147181,  7, }, // Africa/Gaborone
    {  466, 4917,  147188,  5, }, // Africa/Harare
    {  466, 4381,  147193, 10, }, // Africa/Johannesburg
    {  466, 4401,    9030,  4, }, // Africa/Juba
    {  466, 5053,  147203,  7, }, // Africa/Kampala
    {  466, 4426,  147210,  6, }, // Africa/Khartoum
    {  466, 4875,  147216,  6, }, // Africa/Kigali
    {  466, 5208,  147222,  7, }, // Africa/Kinshasa
    {  466, 4646,  144547,  5, }, // Africa/Lagos
    {  466, 5238,  147229,  7, }, // Africa/Libreville
    {  466, 6832,  147236,  4, }, // Africa/Lome
    {  466, 5113,    9077,  6, }, // Africa/Luanda
    {  466, 4841,  147240,  9, }, // Africa/Lubumbashi
    {  466, 4903,  147249,  6, }, // Africa/Lusaka
    {  466, 5224,    9097,  6, }, // Africa/Malabo
    {  466, 4889,  147255,  6, }, // Africa/Maputo
    {  466, 5099,  147261,  5, }, // Africa/Maseru
    {  466, 5084,  147266,  7, }, // Africa/Mbabane
    {  466, 5015,  147273,  8, }, // Africa/Mogadishu
    {  466, 6997,  147281,  8, }, // Africa/Monrovia
    {  466, 3755,  147289,  7, }, // Africa/Nairobi
    {  466, 5173,  147296,  7, }, // Africa/Ndjamena
    {  466, 5256,  147303,  5, }, // Africa/Niamey
    {  466, 6766,  147308,  7, }, // Africa/Nouakchott
    {  466, 6690,  147315,  9, }, // Africa/Ouagadougou
    {  466, 5127,  147324, 10, }, // Africa/Porto-Novo
    {  466, 4337,  147334,  8, }, // Africa/Sao_Tome
    {  466, 2732,    9188,  6, }, // Africa/Tripoli
    {  466, 6503,    9194,  4, }, // Africa/Tunis
    {  466, 4054,  147342,  7, }, // Africa/Windhoek
    {  466,  132,  147349,  4, }, // America/Adak
    {  466, 3183,  147353,  7, }, // America/Anchorage
    {  466, 5524,  147360,  8, }, // America/Anguilla
    {  466, 5541,  147368,  7, }, // America/Antigua
    {  466, 4456,  147375,  9, }, // America/Araguaina
    {  466,  166,  147384, 11, }, // America/Argentina/Buenos_Aires
    {  466,   91,  147395,  9, }, // America/Argentina/Catamarca
    {  466,  270,  147404,  7, }, // America/Argentina/Cordoba
    {  466,  441,  147411,  5, }, // America/Argentina/Jujuy
    {  466, 7425,  147416,  7, }, // America/Argentina/La_Rioja
    {  466,  565,  147423,  6, }, // America/Argentina/Mendoza
    {  466, 7452,  147429,  9, }, // America/Argentina/Rio_Gallegos
    {  466, 7483,  147438,  5, }, // America/Argentina/Salta
    {  466, 7507,  147443,  7, }, // America/Argentina/San_Juan
    {  466, 5497,  147450,  8, }, // America/Argentina/San_Luis
    {  466, 7534,  147458,  6, }, // America/Argentina/Tucuman
    {  466, 7560,  144782,  7, }, // America/Argentina/Ushuaia
    {  466, 5557,  147464,  5, }, // America/Aruba
    {  466, 4154,  147469,  8, }, // America/Asuncion
    {  466,  237,  147477,  7, }, // America/Atikokan
    {  466, 3581,  147484,  4, }, // America/Bahia
    {  466, 7586,  147488, 13, }, // America/Bahia_Banderas
    {  466, 5571,  144825,  8, }, // America/Barbados
    {  466, 7609,    9377,  3, }, // America/Belem
    {  466, 5309,  144833,  4, }, // America/Belize
    {  466, 7623,  147501, 11, }, // America/Blanc-Sablon
    {  466, 7644,  147512,  9, }, // America/Boa_Vista
    {  466, 4263,  147521,  6, }, // America/Bogota
    {  466, 7662,  147527,  5, }, // America/Boise
    {  466, 7676,  147532,  9, }, // America/Cambridge_Bay
    {  466, 7698,  147541, 11, }, // America/Campo_Grande
    {  466, 3770,  147552,  6, }, // America/Cancun
    {  466, 4596,  144889,  7, }, // America/Caracas
    {  466, 4247,  144673,  4, }, // America/Cayenne
    {  466, 5398,  144900,  4, }, // America/Cayman
    {  466, 3251,  147558,  6, }, // America/Chicago
    {  466, 7719,  147564,  6, }, // America/Chihuahua
    {  466, 7737,  147570, 12, }, // America/Ciudad_Juarez
    {  466, 5324,  147582,  9, }, // America/Costa_Rica
    {  466, 7759,  147591,  6, }, // America/Creston
    {  466, 3693,  147597,  6, }, // America/Cuiaba
    {  466, 5640,  147603,  8, }, // America/Curacao
    {  466, 7775,  147611, 13, }, // America/Danmarkshavn
    {  466, 7796,    9533,  5, }, // America/Dawson
    {  466, 7811,  147624, 10, }, // America/Dawson_Creek
    {  466,  805,  147634,  4, }, // America/Denver
    {  466, 3351,  147638,  7, }, // America/Detroit
    {  466, 5656,  144987,  8, }, // America/Dominica
    {  466,  893,  147645,  8, }, // America/Edmonton
    {  466, 7832,  147653,  7, }, // America/Eirunepe
    {  466, 5343,  147660, 10, }, // America/El_Salvador
    {  466, 7849,  147670, 10, }, // America/Fort_Nelson
    {  466, 7869,  147680,  8, }, // America/Fortaleza
    {  466, 7887,  147688,  7, }, // America/Glace_Bay
    {  466, 6844,  147695,  6, }, // America/Goose_Bay
    {  466, 4514,  147701, 10, }, // America/Grand_Turk
    {  466, 5687,  147711,  6, }, // America/Grenada
    {  466, 5703,  147717,  7, }, // America/Guadeloupe
    {  466, 3662,  147724,  8, }, // America/Guatemala
    {  466, 4743,  147732,  8, }, // America/Guayaquil
    {  466, 6895,  147740,  5, }, // America/Guyana
    {  466, 1923,  147745,  6, }, // America/Halifax
    {  466, 2221,  147751,  6, }, // America/Havana
    {  466, 7905,  147757,  7, }, // America/Hermosillo
    {  466,  348,  147764, 13, }, // America/Indiana/Indianapolis
    {  466,  481,  147777, 14, }, // America/Indiana/Knox
    {  466, 7924,  147791, 16, }, // America/Indiana/Marengo
    {  466, 7948,  147807, 18, }, // America/Indiana/Petersburg
    {  466, 7975,  147825, 17, }, // America/Indiana/Tell_City
    {  466, 8001,  147842, 14, }, // America/Indiana/Vevay
    {  466, 8023,  147856, 15, }, // America/Indiana/Vincennes
    {  466, 8049,  147871, 17, }, // America/Indiana/Winamac
    {  466, 8073,  147888,  7, }, // America/Inuvik
    {  466,  660,  147895,  7, }, // America/Iqaluit
    {  466, 2665,  147902,  8, }, // America/Jamaica
    {  466, 5282,  145239,  4, }, // America/Juneau
    {  466,  521,  147910,  8, }, // America/Kentucky/Louisville
    {  466, 8088,  147918, 16, }, // America/Kentucky/Monticello
    {  466, 5621,  147934,  8, }, // America/Kralendijk
    {  466, 4278,  147942,  5, }, // America/La_Paz
    {  466, 7132,  145276,  4, }, // America/Lima
    {  466, 3390,  147947,  9, }, // America/Los_Angeles
    {  466, 5849,  147956, 23, }, // America/Lower_Princes
    {  466, 8116,  147979,  5, }, // America/Maceio
    {  466, 8131,  147984,  7, }, // America/Managua
    {  466, 1892,    9855,  6, }, // America/Manaus
    {  466, 5814,  147991,  7, }, // America/Marigot
    {  466, 5722,  147998,  8, }, // America/Martinique
    {  466, 8147,    9878,  9, }, // America/Matamoros
    {  466, 2779,  148006,  8, }, // America/Mazatlan
    {  466, 8165,  148014,  7, }, // America/Menominee
    {  466, 8183,  148021,  5, }, // America/Merida
    {  466, 8198,  148026, 10, }, // America/Metlakatla
    {  466, 2811,  148036, 10, }, // America/Mexico_City
    {  466, 4293,  148046,  6, }, // America/Miquelon
    {  466, 8217,  148052,  7, }, // America/Moncton
    {  466, 8233,  148059,  6, }, // America/Monterrey
    {  466, 4000,  148065, 11, }, // America/Montevideo
    {  466, 5741,  148076,  8, }, // America/Montserrat
    {  466, 5383,  148084,  6, }, // America/Nassau
    {  466, 3294,  148090,  7, }, // America/New_York
    {  466, 8251,    9897,  3, }, // America/Nome
    {  466, 1834,  148097,  7, }, // America/Noronha
    {  466, 8264,  148104, 20, }, // America/North_Dakota/Beulah
    {  466, 8292,  148124, 19, }, // America/North_Dakota/Center
    {  466, 8320,  148143, 22, }, // America/North_Dakota/New_Salem
    {  466,  393,  145502,  4, }, // America/Nuuk
    {  466, 8351,  148165,  7, }, // America/Ojinaga
    {  466, 5413,  145512,  6, }, // America/Panama
    {  466, 6106,  148172, 10, }, // America/Paramaribo
    {  466, 3224,  145527,  5, }, // America/Phoenix
    {  466, 3847,  148182,  9, }, // America/Port-au-Prince
    {  466, 5871,  148191, 12, }, // America/Port_of_Spain
    {  466, 8367,  148203,  9, }, // America/Porto_Velho
    {  466, 5760,  148212,  9, }, // America/Puerto_Rico
    {  466, 3932,  148221, 11, }, // America/Punta_Arenas
    {  466, 8387,  148232, 11, }, // America/Rankin_Inlet
    {  466, 8408,  148243,  6, }, // America/Recife
    {  466, 1979,  148249,  6, }, // America/Regina
    {  466, 8423,  148255,  6, }, // America/Resolute
    {  466,  695,  148261,  9, }, // America/Rio_Branco
    {  466, 8440,  148270,  6, }, // America/Santarem
    {  466, 2165,  148276,  8, }, // America/Santiago
    {  466, 6044,  148284, 13, }, // America/Santo_Domingo
    {  466, 1862,  148297, 10, }, // America/Sao_Paulo
    {  466, 6862,  148307, 12, }, // America/Scoresbysund
    {  466, 8457,  148319,  5, }, // America/Sitka
    {  466, 8471,  148324, 11, }, // America/St_Barthelemy
    {  466, 2045,  148335,  7, }, // America/St_Johns
    {  466, 5780,  148342,  8, }, // America/St_Kitts
    {  466, 5797,  148350,  9, }, // America/St_Lucia
    {  466,  855,  148359,  9, }, // America/St_Thomas
    {  466, 5830,  148368, 10, }, // America/St_Vincent
    {  466, 8493,  148378, 11, }, // America/Swift_Current
    {  466, 5363,  148389, 10, }, // America/Tegucigalpa
    {  466, 5673,   10304,  3, }, // America/Thule
    {  466,  313,  148399,  7, }, // America/Tijuana
    {  466,  608,  148406,  6, }, // America/Toronto
    {  466, 5605,   10301,  7, }, // America/Tortola
    {  466, 2077,  148412,  6, }, // America/Vancouver
    {  466, 2128,  148418,  9, }, // America/Whitehorse
    {  466,  734,  148427,  6, }, // America/Winnipeg
    {  466, 8515,  148433,  7, }, // America/Yakutat
    {  466, 5918,  103325,  4, }, // Antarctica/Casey
    {  466, 6027,  148440,  5, }, // Antarctica/Davis
    {  466, 6066,  148445, 12, }, // Antarctica/DumontDUrville
    {  466, 8531,  148457,  6, }, // Antarctica/Macquarie
    {  466, 7029,   10371,  6, }, // Antarctica/Mawson
    {  466, 7076,  148463,  8, }, // Antarctica/McMurdo
    {  466, 8552,  145847,  5, }, // Antarctica/Palmer
    {  466, 7189,  148471,  5, }, // Antarctica/Rothera
    {  466, 7258,  148476,  4, }, // Antarctica/Syowa
    {  466, 8570,   10404,  4, }, // Antarctica/Troll
    {  466, 7337,  103380,  5, }, // Antarctica/Vostok
    {  466, 1413,  148480, 10, }, // Arctic/Longyearbyen
    {  466, 5487,   10423,  3, }, // Asia/Aden
    {  466, 5297,  148490,  6, }, // Asia/Almaty
    {  466, 3870,  148496,  5, }, // Asia/Amman
    {  466, 5428,  148501,  6, }, // Asia/Anadyr
    {  466, 5440,  148507,  5, }, // Asia/Aqtau
    {  466, 5270,  148512,  6, }, // Asia/Aqtobe
    {  466,  964,  148518,  8, }, // Asia/Ashgabat
    {  466, 8587,  148526,  6, }, // Asia/Atyrau
    {  466, 3509,   10466,  5, }, // Asia/Baghdad
    {  466, 5451,  145924,  5, }, // Asia/Bahrain
    {  466, 3555,  144949,  4, }, // Asia/Baku
    {  466, 4368,  148532,  6, }, // Asia/Bangkok
    {  466, 3473,  148538,  8, }, // Asia/Barnaul
    {  466, 3988,  145934,  5, }, // Asia/Beirut
    {  466, 3680,  148546,  5, }, // Asia/Bishkek
    {  466, 5906,  148551,  6, }, // Asia/Brunei
    {  466, 4503,  148557,  4, }, // Asia/Chita
    {  466, 5963,  148561,  9, }, // Asia/Choibalsan
    {  466, 4413,  148570,  5, }, // Asia/Colombo
    {  466, 4442,   10528,  4, }, // Asia/Damascus
    {  466, 1060,  144671,  4, }, // Asia/Dhaka
    {  466, 6125,  148575,  4, }, // Asia/Dili
    {  466, 3498,  148579,  3, }, // Asia/Dubai
    {  466, 6092,  148582,  6, }, // Asia/Dushanbe
    {  466, 8599,  148588,  9, }, // Asia/Famagusta
    {  466, 8614,  145994,  3, }, // Asia/Gaza
    {  466, 4697,  148597,  6, }, // Asia/Hebron
    {  466, 1226,  148603, 13, }, // Asia/Ho_Chi_Minh
    {  466, 2576,  148616,  7, }, // Asia/Hong_Kong
    {  466, 4673,  148623,  4, }, // Asia/Hovd
    {  466, 4086,  148627,  8, }, // Asia/Irkutsk
    {  466, 6969,  148635,  7, }, // Asia/Jakarta
    {  466, 6955,  148642,  8, }, // Asia/Jayapura
    {  466, 1257,  148650,  7, }, // Asia/Jerusalem
    {  466, 3462,  146048,  4, }, // Asia/Kabul
    {  466, 4218,  146052,  8, }, // Asia/Kamchatka
    {  466, 4141,  148657,  5, }, // Asia/Karachi
    {  466, 1152,  148662,  8, }, // Asia/Kathmandu
    {  466, 8624,  148670,  7, }, // Asia/Khandyga
    {  466,  992,  148677,  5, }, // Asia/Kolkata
    {  466, 4099,  148682, 11, }, // Asia/Krasnoyarsk
    {  466, 4761,  148693, 11, }, // Asia/Kuala_Lumpur
    {  466, 5893,  148704,  6, }, // Asia/Kuching
    {  466, 5464,  148710,  4, }, // Asia/Kuwait
    {  466, 1178,  148714,  6, }, // Asia/Macau
    {  466, 3919,  148720,  7, }, // Asia/Magadan
    {  466, 1316,  148727,  7, }, // Asia/Makassar
    {  466, 7145,  148734,  5, }, // Asia/Manila
    {  466, 6883,   10734,  4, }, // Asia/Muscat
    {  466, 2446,  148739,  7, }, // Asia/Nicosia
    {  466, 8638,  148746, 11, }, // Asia/Novokuznetsk
    {  466, 4037,  148757, 11, }, // Asia/Novosibirsk
    {  466, 4131,  103736,  5, }, // Asia/Omsk
    {  466, 7108,  148768,  5, }, // Asia/Oral
    {  466, 6924,  148773,  7, }, // Asia/Phnom_Penh
    {  466, 8656,  148780,  9, }, // Asia/Pontianak
    {  466, 4116,  148789, 10, }, // Asia/Pyongyang
    {  466, 5476,   10802,  3, }, // Asia/Qatar
    {  466, 8671,  148799,  8, }, // Asia/Qostanay
    {  466, 4171,  148807, 11, }, // Asia/Qyzylorda
    {  466, 3486,  103788,  4, }, // Asia/Riyadh
    {  466, 4310,  148818,  7, }, // Asia/Sakhalin
    {  466, 7208,   10834,  6, }, // Asia/Samarkand
    {  466, 3116,  148825,  4, }, // Asia/Seoul
    {  466, 1020,  148829,  7, }, // Asia/Shanghai
    {  466, 3137,  148836,  7, }, // Asia/Singapore
    {  466, 4199,  148843, 12, }, // Asia/Srednekolymsk
    {  466, 3100,  148855,  5, }, // Asia/Taipei
    {  466, 4683,  148860,  6, }, // Asia/Tashkent
    {  466, 3817,  148866,  5, }, // Asia/Tbilisi
    {  466, 2638,  146255,  5, }, // Asia/Tehran
    {  466, 1284,  148871,  5, }, // Asia/Thimphu
    {  466, 2687,  148876,  5, }, // Asia/Tokyo
    {  466, 4474,  148881,  5, }, // Asia/Tomsk
    {  466, 1346,  148886, 11, }, // Asia/Ulaanbaatar
    {  466, 1126,  148897,  6, }, // Asia/Urumqi
    {  466, 8685,  148903,  8, }, // Asia/Ust-Nera
    {  466, 6940,  148911,  7, }, // Asia/Vientiane
    {  466, 4612,  148918, 11, }, // Asia/Vladivostok
    {  466, 4730,  148929,  7, }, // Asia/Yakutsk
    {  466, 1202,  148936,  6, }, // Asia/Yangon
    {  466, 3785,  148942, 10, }, // Asia/Yekaterinburg
    {  466, 3649,  148952,  6, }, // Asia/Yerevan
    {  466, 3565,  148958,  4, }, // Atlantic/Azores
    {  466, 5588,   10981,  6, }, // Atlantic/Bermuda
    {  466, 6591,  148962,  5, }, // Atlantic/Canary
    {  466, 3629,  148967,  7, }, // Atlantic/Cape_Verde
    {  466, 1379,   11004,  4, }, // Atlantic/Faroe
    {  466, 8699,  148974,  6, }, // Atlantic/Madeira
    {  466, 2614,  148980,  8, }, // Atlantic/Reykjavik
    {  466, 7235,  148988, 13, }, // Atlantic/South_Georgia
    {  466, 6784,  149001,  8, }, // Atlantic/St_Helena
    {  466, 6607,  149009,  7, }, // Atlantic/Stanley
    {  466, 1654,  149016,  6, }, // Australia/Adelaide
    {  466, 1619,  149022,  7, }, // Australia/Brisbane
    {  466, 1783,  149029,  9, }, // Australia/Broken_Hill
    {  466, 1567,  149038,  6, }, // Australia/Darwin
    {  466, 3539,  149044,  5, }, // Australia/Eucla
    {  466, 1500,   11087,  6, }, // Australia/Hobart
    {  466, 8716,  149049,  6, }, // Australia/Lindeman
    {  466, 1531,  149055,  6, }, // Australia/Lord_Howe
    {  466, 1711,  149061,  6, }, // Australia/Melbourne
    {  466, 1746,  146461,  3, }, // Australia/Perth
    {  466, 1447,  149067,  5, }, // Australia/Sydney
    {  466, 6390,  149072,  8, }, // Europe/Amsterdam
    {  466, 6149,  149080,  6, }, // Europe/Andorra
    {  466, 3522,  149086,  8, }, // Europe/Astrakhan
    {  466, 6577,  149094,  3, }, // Europe/Athens
    {  466, 6285,  149097,  6, }, // Europe/Belgrade
    {  466, 4659,  149103,  5, }, // Europe/Berlin
    {  466, 6437,  149108, 10, }, // Europe/Bratislava
    {  466, 6178,  149118,  6, }, // Europe/Brussels
    {  466, 3830,  146520,  6, }, // Europe/Bucharest
    {  466, 3708,  149124,  7, }, // Europe/Budapest
    {  466, 8751,  149131,  6, }, // Europe/Busingen
    {  466, 2475,  149137,  7, }, // Europe/Chisinau
    {  466, 6238,  149144,  6, }, // Europe/Copenhagen
    {  466, 2260,  149150,  6, }, // Europe/Dublin
    {  466, 6256,  149156, 10, }, // Europe/Gibraltar
    {  466, 8767,  149166,  5, }, // Europe/Guernsey
    {  466, 6561,  149171,  7, }, // Europe/Helsinki
    {  466, 8783,  149178,  9, }, // Europe/Isle_of_Man
    {  466, 1097,  149187,  8, }, // Europe/Istanbul
    {  466, 8802,  149195,  4, }, // Europe/Jersey
    {  466, 3881,  149199, 11, }, // Europe/Kaliningrad
    {  466, 8924,  146609,  5, }, // Europe/Kirov
    {  466, 2419,  146614,  3, }, // Europe/Kyiv
    {  466, 3078,  146617,  6, }, // Europe/Lisbon
    {  466, 6455,  149210,  9, }, // Europe/Ljubljana
    {  466, 2393,   11292,  4, }, // Europe/London
    {  466, 6314,  149219, 10, }, // Europe/Luxembourg
    {  466, 4794,  149229,  6, }, // Europe/Madrid
    {  466, 6346,  149235,  4, }, // Europe/Malta
    {  466, 6531,  149239, 10, }, // Europe/Mariehamn
    {  466, 3595,  149249,  5, }, // Europe/Minsk
    {  466, 6359,  104344,  6, }, // Europe/Monaco
    {  466, 3443,  146803,  4, }, // Europe/Moscow
    {  466, 6407,  149254,  4, }, // Europe/Oslo
    {  466, 4186,  146662,  5, }, // Europe/Paris
    {  466, 6373,  149258, 10, }, // Europe/Podgorica
    {  466, 6224,  104374,  4, }, // Europe/Prague
    {  466, 8816,  147993,  4, }, // Europe/Riga
    {  466, 6273,    9684,  2, }, // Europe/Rome
    {  466, 4233,  149268,  6, }, // Europe/Samara
    {  466, 6419,  149274, 10, }, // Europe/San_Marino
    {  466, 6194,  149284,  7, }, // Europe/Sarajevo
    {  466, 4353,   11388,  7, }, // Europe/Saratov
    {  466, 8828,  149291,  8, }, // Europe/Simferopol
    {  466, 6332,  149299,  7, }, // Europe/Skopje
    {  466, 6548,  149306,  5, }, // Europe/Sofia
    {  466, 6472,  149311,  7, }, // Europe/Stockholm
    {  466, 8846,  149318,  5, }, // Europe/Tallinn
    {  466, 6135,  149323,  6, }, // Europe/Tirane
    {  466, 8861,  149329, 10, }, // Europe/Ulyanovsk
    {  466, 6301,  149339,  6, }, // Europe/Vaduz
    {  466, 6516,  149345,  7, }, // Europe/Vatican
    {  466, 6164,  102059,  3, }, // Europe/Vienna
    {  466, 8878,  149352,  7, }, // Europe/Vilnius
    {  466, 4629,  149359,  8, }, // Europe/Volgograd
    {  466, 3055,  149367,  4, }, // Europe/Warsaw
    {  466, 6210,  149371,  5, }, // Europe/Zagreb
    {  466, 6489,  149376,  5, }, // Europe/Zurich
    {  466, 4980,  149381, 12, }, // Indian/Antananarivo
    {  466, 6910,  149393,  5, }, // Indian/Chagos
    {  466, 5979,  149398,  6, }, // Indian/Christmas
    {  466, 5996,  104540,  5, }, // Indian/Cocos
    {  466, 4931,  104545,  6, }, // Indian/Comoro
    {  466, 6624,  149404,  6, }, // Indian/Kerguelen
    {  466, 7223,  149410,  4, }, // Indian/Mahe
    {  466, 7013,  149414,  6, }, // Indian/Maldives
    {  466, 3971,  149420,  5, }, // Indian/Mauritius
    {  466, 5000,  149425,  5, }, // Indian/Mayotte
    {  466, 7174,  149430,  7, }, // Indian/Reunion
    {  466, 4324,  149437,  4, }, // Pacific/Apia
    {  466,  932,  149441,  6, }, // Pacific/Auckland
    {  466, 3608,  149447,  7, }, // Pacific/Bougainville
    {  466, 2863,  149454,  3, }, // Pacific/Chatham
    {  466, 3022,  146865,  3, }, // Pacific/Chuuk
    {  466, 2201,  149457,  5, }, // Pacific/Easter
    {  466, 7323,  149462,  5, }, // Pacific/Efate
    {  466, 7290,  149467,  8, }, // Pacific/Fakaofo
    {  466, 3804,  149475,  4, }, // Pacific/Fiji
    {  466, 7306,  149479,  8, }, // Pacific/Funafuti
    {  466, 6641,  149487,  9, }, // Pacific/Galapagos
    {  466, 6659,  149496,  6, }, // Pacific/Gambier
    {  466, 3724,  149502, 12, }, // Pacific/Guadalcanal
    {  466, 5935,  102779,  4, }, // Pacific/Guam
    {  466, 2929,   11649,  8, }, // Pacific/Honolulu
    {  466, 2897,  149514,  7, }, // Pacific/Kanton
    {  466, 3900,  149521,  9, }, // Pacific/Kiritimati
    {  466, 6982,  149530,  6, }, // Pacific/Kosrae
    {  466, 2708,  149536,  8, }, // Pacific/Kwajalein
    {  466, 4779,   11689,  6, }, // Pacific/Majuro
    {  466, 3953,  149544,  7, }, // Pacific/Marquesas
    {  466, 8909,  149551,  5, }, // Pacific/Midway
    {  466, 7047,  149556,  6, }, // Pacific/Nauru
    {  466, 7095,  149562,  5, }, // Pacific/Niue
    {  466, 4070,  149567,  7, }, // Pacific/Norfolk
    {  466, 7061,  149574,  5, }, // Pacific/Noumea
    {  466, 2991,  149579,  8, }, // Pacific/Pago_Pago
    {  466, 7118,  149587,  6, }, // Pacific/Palau
    {  466, 7157,  149593,  7, }, // Pacific/Pitcairn
    {  466, 2961,  149600,  5, }, // Pacific/Pohnpei
    {  466, 4709,  149605, 11, }, // Pacific/Port_Moresby
    {  466, 6009,  149616,  9, }, // Pacific/Rarotonga
    {  466, 5948,  149625,  6, }, // Pacific/Saipan
    {  466, 7275,  149631,  6, }, // Pacific/Tahiti
    {  466, 6675,   11783,  6, }, // Pacific/Tarawa
    {  466, 4485,  149637,  9, }, // Pacific/Tongatapu
    {  466, 7355,  104828,  3, }, // Pacific/Wake
    {  466, 7368,  149646,  5, }, // Pacific/Wallis
    {  468, 6751,  149651,  7, }, // Africa/Abidjan Polish/Latin/Poland
    {  468, 6723,    4550,  4, }, // Africa/Accra
    {  468, 4961,    3793, 11, }, // Africa/Addis_Ababa
    {  468, 7383,   57088,  6, }, // Africa/Algiers
    {  468,   14,       0,  6, }, // Africa/Asmara
    {  468, 5159,   24908,  5, }, // Africa/Bangui
    {  468, 6709,  149658,  7, }, // Africa/Banjul
    {  468, 4824,  149665,  9, }, // Africa/Bujumbura
    {  468, 2242,   24913,  4, }, // Africa/Cairo
    {  468, 6736,   24918,  7, }, // Africa/Conakry
    {  468, 4945,  149674,  7, }, // Africa/Djibouti
    {  468, 5145,   19777,  5, }, // Africa/Douala
    {  468, 5068,  149681,  7, }, // Africa/El_Aaiun
    {  468, 4401,  149688,  5, }, // Africa/Juba
    {  468, 4426,  120506,  7, }, // Africa/Khartoum
    {  468, 5208,  149693,  8, }, // Africa/Kinshasa
    {  468, 6832,   18802,  4, }, // Africa/Lome
    {  468, 5015,  149701,  9, }, // Africa/Mogadishu
    {  468, 5173,  149710,  8, }, // Africa/Ndjamena
    {  468, 6766,  149718,  9, }, // Africa/Nouakchott
    {  468, 6690,   97031,  8, }, // Africa/Ouagadougou
    {  468, 5127,   24975, 10, }, // Africa/Porto-Novo
    {  468, 4337,       6,  8, }, // Africa/Sao_Tome
    {  468, 2732,  149727,  8, }, // Africa/Tripoli
    {  468, 4054,   55328,  7, }, // Africa/Windhoek
    {  468,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  468, 4154,      14,  8, }, // America/Asuncion
    {  468,  237,      22,  8, }, // America/Atikokan
    {  468, 3581,    5090,  8, }, // America/Bahia
    {  468, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  468, 7609,   34376,  5, }, // America/Belem
    {  468, 3770,      47,  6, }, // America/Cancun
    {  468, 4247,  118079,  7, }, // America/Cayenne
    {  468, 5398,  149735,  7, }, // America/Cayman
    {  468, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  468, 5324,  149742,  9, }, // America/Costa_Rica
    {  468, 3693,   25007,  6, }, // America/Cuiaba
    {  468, 5640,      66,  7, }, // America/Curacao
    {  468, 5656,   20071,  8, }, // America/Dominica
    {  468, 5343,  149751,  8, }, // America/El_Salvador
    {  468, 5703,  149759,  9, }, // America/Guadeloupe
    {  468, 3662,  130126,  9, }, // America/Guatemala
    {  468, 6895,  149768,  6, }, // America/Guyana
    {  468, 2221,  149774,  6, }, // America/Havana
    {  468,  481,      73, 13, }, // America/Indiana/Knox
    {  468, 7924,      86, 16, }, // America/Indiana/Marengo
    {  468, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  468, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  468, 8001,     139, 14, }, // America/Indiana/Vevay
    {  468, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  468, 8049,     171, 16, }, // America/Indiana/Winamac
    {  468, 2665,   31809,  7, }, // America/Jamaica
    {  468, 8088,     187, 10, }, // America/Kentucky/Monticello
    {  468, 5849,     207, 22, }, // America/Lower_Princes
    {  468, 8116,   25021,  6, }, // America/Maceio
    {  468, 5722,  149780,  9, }, // America/Martinique
    {  468, 8183,    3899,  6, }, // America/Merida
    {  468, 2811,  149789, 15, }, // America/Mexico_City
    {  468, 3294,  149804,  9, }, // America/New_York
    {  468, 8264,  149813, 23, }, // America/North_Dakota/Beulah
    {  468, 8292,  149836, 23, }, // America/North_Dakota/Center
    {  468, 8320,  149859, 26, }, // America/North_Dakota/New_Salem
    {  468,  393,     298,  4, }, // America/Nuuk
    {  468, 5871,   25124, 13, }, // America/Port_of_Spain
    {  468, 5760,  149885,  9, }, // America/Puerto_Rico
    {  468, 6862,     302, 16, }, // America/Scoresbysund
    {  468, 8471,   25154, 16, }, // America/St_Barthelemy
    {  468, 2045,     332, 10, }, // America/St_Johns
    {  468, 5780,   25182, 11, }, // America/St_Kitts
    {  468, 5797,   34444, 11, }, // America/St_Lucia
    {  468,  855,   42797, 12, }, // America/St_Thomas
    {  468, 5830,   25216, 13, }, // America/St_Vincent
    {  468, 5673,   25229,  7, }, // America/Thule
    {  468, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  468, 7337,    4048,  6, }, // Antarctica/Vostok
    {  468, 5297,  149894,  6, }, // Asia/Almaty
    {  468, 5440,    5906,  5, }, // Asia/Aqtau
    {  468, 5270,  149900, 10, }, // Asia/Aqtobe
    {  468,  964,  149910,  9, }, // Asia/Ashgabat
    {  468, 3509,    4063,  6, }, // Asia/Baghdad
    {  468, 5451,   56476,  7, }, // Asia/Bahrain
    {  468, 3473,  149919,  7, }, // Asia/Barnaul
    {  468, 3988,    5930,  6, }, // Asia/Beirut
    {  468, 3680,  149926,  7, }, // Asia/Bishkek
    {  468, 4503,  149933,  5, }, // Asia/Chita
    {  468, 5963,  149938, 10, }, // Asia/Choibalsan
    {  468, 4413,    5949,  7, }, // Asia/Colombo
    {  468, 4442,  149948,  8, }, // Asia/Damascus
    {  468, 3498,   56501,  5, }, // Asia/Dubai
    {  468, 6092,  149956,  8, }, // Asia/Dushanbe
    {  468, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  468, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  468, 4673,  149964,  5, }, // Asia/Hovd
    {  468, 4086,  139633,  6, }, // Asia/Irkutsk
    {  468, 6969,  149969,  8, }, // Asia/Jakarta
    {  468, 1257,  149977, 10, }, // Asia/Jerusalem
    {  468, 4218,  149987,  9, }, // Asia/Kamchatka
    {  468, 4141,  149996,  7, }, // Asia/Karachi
    {  468, 1152,    6018,  8, }, // Asia/Kathmandu
    {  468, 8624,   56553,  8, }, // Asia/Khandyga
    {  468,  992,   25307,  7, }, // Asia/Kolkata
    {  468, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  468, 5464,  150003,  6, }, // Asia/Kuwait
    {  468, 1178,   32008,  5, }, // Asia/Macau
    {  468, 6883,   25325,  6, }, // Asia/Muscat
    {  468, 2446,  150009,  7, }, // Asia/Nicosia
    {  468, 8638,  150016, 12, }, // Asia/Novokuznetsk
    {  468, 4037,  150028, 11, }, // Asia/Novosibirsk
    {  468, 7108,   56586,  6, }, // Asia/Oral
    {  468, 4116,   32033,  9, }, // Asia/Pyongyang
    {  468, 5476,    4233,  5, }, // Asia/Qatar
    {  468, 8671,  150039,  8, }, // Asia/Qostanay
    {  468, 4171,  150047,  9, }, // Asia/Qyzylorda
    {  468, 3486,   32042,  5, }, // Asia/Riyadh
    {  468, 4310,   56617,  8, }, // Asia/Sakhalin
    {  468, 7208,   25356, 10, }, // Asia/Samarkand
    {  468, 3116,    6131,  4, }, // Asia/Seoul
    {  468, 1020,  150056,  8, }, // Asia/Shanghai
    {  468, 3137,   19291,  8, }, // Asia/Singapore
    {  468, 4199,  150064, 15, }, // Asia/Srednekolymsk
    {  468, 3100,    6163,  6, }, // Asia/Taipei
    {  468, 4683,  150079,  9, }, // Asia/Tashkent
    {  468, 2638,    6175,  7, }, // Asia/Tehran
    {  468, 2687,    4257,  5, }, // Asia/Tokyo
    {  468, 1346,  150088, 10, }, // Asia/Ulaanbaatar
    {  468, 1126,  150098,  7, }, // Asia/Urumqi
    {  468, 8685,  150105,  9, }, // Asia/Ust-Nera
    {  468, 6940,  150114,  8, }, // Asia/Vientiane
    {  468, 4612,  150122, 11, }, // Asia/Vladivostok
    {  468, 4730,  139660,  6, }, // Asia/Yakutsk
    {  468, 1202,    6213,  6, }, // Asia/Yangon
    {  468, 3785,  150133, 13, }, // Asia/Yekaterinburg
    {  468, 3649,  150146,  6, }, // Asia/Yerevan
    {  468, 3565,  150152,  5, }, // Atlantic/Azores
    {  468, 5588,   56708,  7, }, // Atlantic/Bermuda
    {  468, 6591,  150157, 17, }, // Atlantic/Canary
    {  468, 3629,  150174, 29, }, // Atlantic/Cape_Verde
    {  468, 1379,  150203, 11, }, // Atlantic/Faroe
    {  468, 8699,  150214,  6, }, // Atlantic/Madeira
    {  468, 7235,  150220, 18, }, // Atlantic/South_Georgia
    {  468, 6784,  150238, 13, }, // Atlantic/St_Helena
    {  468, 6149,   32109,  6, }, // Europe/Andorra
    {  468, 3522,  150251,  9, }, // Europe/Astrakhan
    {  468, 6577,  150260,  5, }, // Europe/Athens
    {  468, 6285,   25465,  7, }, // Europe/Belgrade
    {  468, 6437,  150265, 10, }, // Europe/Bratislava
    {  468, 6178,  150275,  8, }, // Europe/Brussels
    {  468, 3830,  150283,  9, }, // Europe/Bucharest
    {  468, 3708,  150292,  9, }, // Europe/Budapest
    {  468, 8751,   61453, 21, }, // Europe/Busingen
    {  468, 2475,  150301,  9, }, // Europe/Chisinau
    {  468, 6238,  122844,  9, }, // Europe/Copenhagen
    {  468, 8783,  150310,  9, }, // Europe/Isle_of_Man
    {  468, 1097,  150319,  7, }, // Europe/Istanbul
    {  468, 8924,   76768,  5, }, // Europe/Kirov
    {  468, 2419,  150326,  5, }, // Europe/Kyiv
    {  468, 3078,  150331,  7, }, // Europe/Lisbon
    {  468, 6455,  150338,  7, }, // Europe/Ljubljana
    {  468, 2393,  150345,  6, }, // Europe/London
    {  468, 6314,    6510, 10, }, // Europe/Luxembourg
    {  468, 4794,  150351,  6, }, // Europe/Madrid
    {  468, 6531,   62205, 13, }, // Europe/Mariehamn
    {  468, 3595,  150357,  5, }, // Europe/Minsk
    {  468, 6359,    6525,  6, }, // Europe/Monaco
    {  468, 3443,   91758,  6, }, // Europe/Moscow
    {  468, 4186,  150362,  5, }, // Europe/Paris
    {  468, 6224,   19576,  5, }, // Europe/Prague
    {  468, 8816,  123014,  4, }, // Europe/Riga
    {  468, 6273,  150367,  4, }, // Europe/Rome
    {  468, 6194,  150371,  8, }, // Europe/Sarajevo
    {  468, 4353,  150379,  7, }, // Europe/Saratov
    {  468, 8828,  150386, 10, }, // Europe/Simferopol
    {  468, 6472,  150396,  9, }, // Europe/Stockholm
    {  468, 8846,   21335,  6, }, // Europe/Tallinn
    {  468, 6135,   19608,  6, }, // Europe/Tirane
    {  468, 8861,   76784,  9, }, // Europe/Ulyanovsk
    {  468, 6516,  150405,  7, }, // Europe/Vatican
    {  468, 6164,  150412,  6, }, // Europe/Vienna
    {  468, 8878,  150418,  5, }, // Europe/Vilnius
    {  468, 4629,  150423,  9, }, // Europe/Volgograd
    {  468, 3055,   57212,  8, }, // Europe/Warsaw
    {  468, 6210,  150432,  7, }, // Europe/Zagreb
    {  468, 6489,  150439,  6, }, // Europe/Zurich
    {  468, 4980,  150445, 12, }, // Indian/Antananarivo
    {  468, 6910,  150457,  6, }, // Indian/Chagos
    {  468, 5979,  150463, 23, }, // Indian/Christmas
    {  468, 5996,  150486, 14, }, // Indian/Cocos
    {  468, 4931,   56941,  6, }, // Indian/Comoro
    {  468, 6624,  150500, 16, }, // Indian/Kerguelen
    {  468, 7223,   25565,  4, }, // Indian/Mahe
    {  468, 7013,  123666,  8, }, // Indian/Maldives
    {  468, 5000,  150516,  7, }, // Indian/Mayotte
    {  468, 7174,     467,  7, }, // Indian/Reunion
    {  468, 3608,  150523, 20, }, // Pacific/Bougainville
    {  468, 3022,     474,  5, }, // Pacific/Chuuk
    {  468, 2201,  150543, 17, }, // Pacific/Easter
    {  468, 3804,  150560,  5, }, // Pacific/Fiji
    {  468, 6659,  150565, 14, }, // Pacific/Gambier
    {  468, 2929,    4512,  8, }, // Pacific/Honolulu
    {  468, 2897,    4520,  9, }, // Pacific/Kanton
    {  468, 3953,  150579,  7, }, // Pacific/Marquesas
    {  468, 7061,  120279,  5, }, // Pacific/Noumea
    {  468, 2961,     479,  7, }, // Pacific/Pohnpei
    {  469, 6723,   72588,  4, }, // Africa/Accra Portuguese/Latin/Brazil
    {  469, 4961,   34301, 10, }, // Africa/Addis_Ababa
    {  469, 7383,  140180,  5, }, // Africa/Algiers
    {  469,   14,       0,  6, }, // Africa/Asmara
    {  469, 6736,  150586,  7, }, // Africa/Conakry
    {  469, 4945,   24925,  7, }, // Africa/Djibouti
    {  469, 5068,   62570,  8, }, // Africa/El_Aaiun
    {  469, 4381,  150593, 11, }, // Africa/Johannesburg
    {  469, 4426,  150604,  6, }, // Africa/Khartoum
    {  469, 6832,   18802,  4, }, // Africa/Lome
    {  469, 5015,  150610, 10, }, // Africa/Mogadishu
    {  469, 6997,  136341,  8, }, // Africa/Monrovia
    {  469, 3755,  150620,  7, }, // Africa/Nairobi
    {  469, 5173,   42632,  9, }, // Africa/Ndjamena
    {  469, 5127,   24975, 10, }, // Africa/Porto-Novo
    {  469, 4337,       6,  8, }, // Africa/Sao_Tome
    {  469, 2732,   18851,  7, }, // Africa/Tripoli
    {  469, 6503,   90982,  5, }, // Africa/Tunis
    {  469, 5524,   18863,  7, }, // America/Anguilla
    {  469, 5541,  150627,  7, }, // America/Antigua
    {  469, 4456,   24992,  9, }, // America/Araguaina
    {  469,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  469, 7534,  150634,  6, }, // America/Argentina/Tucuman
    {  469, 4154,  150640,  8, }, // America/Asuncion
    {  469,  237,      22,  8, }, // America/Atikokan
    {  469, 7586,   63067, 17, }, // America/Bahia_Banderas
    {  469, 7609,   34376,  5, }, // America/Belem
    {  469, 4263,   18898,  6, }, // America/Bogota
    {  469, 3770,      47,  6, }, // America/Cancun
    {  469, 4247,   42671,  6, }, // America/Cayenne
    {  469, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  469, 3693,   25007,  6, }, // America/Cuiaba
    {  469, 5640,      66,  7, }, // America/Curacao
    {  469, 7832,   25013,  8, }, // America/Eirunepe
    {  469, 5687,   18910,  7, }, // America/Grenada
    {  469, 5703,   18917,  9, }, // America/Guadeloupe
    {  469, 4743,  150648,  9, }, // America/Guayaquil
    {  469, 6895,  150657,  6, }, // America/Guyana
    {  469,  348,   72702, 12, }, // America/Indiana/Indianapolis
    {  469,  481,      73, 13, }, // America/Indiana/Knox
    {  469, 7924,      86, 16, }, // America/Indiana/Marengo
    {  469, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  469, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  469, 8001,     139, 14, }, // America/Indiana/Vevay
    {  469, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  469, 8049,     171, 16, }, // America/Indiana/Winamac
    {  469, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  469, 5849,     207, 22, }, // America/Lower_Princes
    {  469, 8116,   25021,  6, }, // America/Maceio
    {  469, 8131,   96580,  7, }, // America/Managua
    {  469, 5722,   18945,  9, }, // America/Martinique
    {  469, 8183,     229,  6, }, // America/Merida
    {  469, 2811,  150663, 16, }, // America/Mexico_City
    {  469, 4000,  150679, 10, }, // America/Montevideo
    {  469, 3294,   42698,  9, }, // America/New_York
    {  469, 1834,   60511, 19, }, // America/Noronha
    {  469, 8264,  150689, 23, }, // America/North_Dakota/Beulah
    {  469, 8292,  150712, 23, }, // America/North_Dakota/Center
    {  469, 8320,  150735, 26, }, // America/North_Dakota/New_Salem
    {  469,  393,     298,  4, }, // America/Nuuk
    {  469, 5413,   19056,  6, }, // America/Panama
    {  469, 3847,   72817, 14, }, // America/Port-au-Prince
    {  469, 5760,   63375, 10, }, // America/Puerto_Rico
    {  469, 8440,   25137,  8, }, // America/Santarem
    {  469, 1862,   25145,  9, }, // America/Sao_Paulo
    {  469, 6862,     302, 16, }, // America/Scoresbysund
    {  469, 8471,  150761, 14, }, // America/St_Barthelemy
    {  469, 2045,   25170, 12, }, // America/St_Johns
    {  469, 5780,  150775, 13, }, // America/St_Kitts
    {  469, 5797,  150788, 11, }, // America/St_Lucia
    {  469,  855,   42797, 12, }, // America/St_Thomas
    {  469, 5830,  150799, 11, }, // America/St_Vincent
    {  469, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  469, 5487,   90042,  4, }, // Asia/Aden
    {  469, 3870,  150810,  3, }, // Asia/Amman
    {  469, 5440,    5906,  5, }, // Asia/Aqtau
    {  469, 5270,    5911,  6, }, // Asia/Aqtobe
    {  469,  964,  150813,  8, }, // Asia/Ashgabat
    {  469, 3509,   56470,  5, }, // Asia/Baghdad
    {  469, 5451,    4069,  7, }, // Asia/Bahrain
    {  469, 3988,  150821,  7, }, // Asia/Beirut
    {  469, 4442,   72897,  7, }, // Asia/Damascus
    {  469, 1060,   64123,  5, }, // Asia/Dhaka
    {  469, 6092,  150828,  8, }, // Asia/Dushanbe
    {  469, 1226,  150836, 21, }, // Asia/Ho_Chi_Minh
    {  469, 6969,  150857,  7, }, // Asia/Jakarta
    {  469, 1257,  150864,  9, }, // Asia/Jerusalem
    {  469, 3462,   72911,  5, }, // Asia/Kabul
    {  469, 1152,    6018,  8, }, // Asia/Kathmandu
    {  469,  992,  150873,  7, }, // Asia/Kolkata
    {  469, 1178,    4222,  5, }, // Asia/Macau
    {  469, 6883,   19248,  7, }, // Asia/Muscat
    {  469, 2446,  150880,  7, }, // Asia/Nicosia
    {  469, 5476,   71561,  5, }, // Asia/Qatar
    {  469, 3486,  150887,  5, }, // Asia/Riyadh
    {  469, 4310,  150892,  8, }, // Asia/Sakhalin
    {  469, 7208,   19277, 10, }, // Asia/Samarkand
    {  469, 3116,    6131,  4, }, // Asia/Seoul
    {  469, 1020,  150900,  6, }, // Asia/Shanghai
    {  469, 3137,   91698,  9, }, // Asia/Singapore
    {  469, 2638,  150906,  5, }, // Asia/Tehran
    {  469, 2687,  150911,  6, }, // Asia/Tokyo
    {  469, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  469, 1202,  150917,  6, }, // Asia/Yangon
    {  469, 3785,  150923, 13, }, // Asia/Yekaterinburg
    {  469, 3565,   42965,  6, }, // Atlantic/Azores
    {  469, 5588,   72942,  8, }, // Atlantic/Bermuda
    {  469, 6591,  150936,  8, }, // Atlantic/Canary
    {  469, 3629,   25418, 10, }, // Atlantic/Cape_Verde
    {  469, 1379,  150944, 11, }, // Atlantic/Faroe
    {  469, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  469, 7235,  150955, 14, }, // Atlantic/South_Georgia
    {  469, 6784,   25447, 12, }, // Atlantic/St_Helena
    {  469, 6390,  150969,  8, }, // Europe/Amsterdam
    {  469, 3522,  150977,  7, }, // Europe/Astrakhan
    {  469, 6577,   25459,  6, }, // Europe/Athens
    {  469, 6285,   57443,  8, }, // Europe/Belgrade
    {  469, 4659,  150984,  6, }, // Europe/Berlin
    {  469, 6178,   73005,  8, }, // Europe/Brussels
    {  469, 3830,  150990,  9, }, // Europe/Bucharest
    {  469, 3708,  150999,  9, }, // Europe/Budapest
    {  469, 8751,   25487,  8, }, // Europe/Busingen
    {  469, 6238,   19472, 10, }, // Europe/Copenhagen
    {  469, 6561,  151008,  9, }, // Europe/Helsinki
    {  469, 8783,  151017, 11, }, // Europe/Isle_of_Man
    {  469, 1097,   55967,  8, }, // Europe/Istanbul
    {  469, 3881,   61551, 12, }, // Europe/Kaliningrad
    {  469, 2419,    6486,  4, }, // Europe/Kyiv
    {  469, 3078,   19528,  6, }, // Europe/Lisbon
    {  469, 6455,   19534,  9, }, // Europe/Ljubljana
    {  469, 2393,   19543,  7, }, // Europe/London
    {  469, 6314,   73033, 10, }, // Europe/Luxembourg
    {  469, 4794,   25525,  5, }, // Europe/Madrid
    {  469, 6359,  151028,  6, }, // Europe/Monaco
    {  469, 3443,   43080,  6, }, // Europe/Moscow
    {  469, 6224,   19576,  5, }, // Europe/Prague
    {  469, 6273,   19581,  4, }, // Europe/Rome
    {  469, 6548,  151034,  5, }, // Europe/Sofia
    {  469, 6472,   73074,  9, }, // Europe/Stockholm
    {  469, 6135,   19608,  6, }, // Europe/Tirane
    {  469, 8861,   25542,  9, }, // Europe/Ulyanovsk
    {  469, 6516,   73083,  8, }, // Europe/Vatican
    {  469, 6164,   19625,  5, }, // Europe/Vienna
    {  469, 4629,   61732, 10, }, // Europe/Volgograd
    {  469, 3055,  151039,  8, }, // Europe/Warsaw
    {  469, 6489,  151047,  7, }, // Europe/Zurich
    {  469, 4931,   63916,  7, }, // Indian/Comoro
    {  469, 7223,   25565,  4, }, // Indian/Mahe
    {  469, 7013,   73105,  8, }, // Indian/Maldives
    {  469, 3971,  151054,  8, }, // Indian/Mauritius
    {  469, 7174,  151062,  7, }, // Indian/Reunion
    {  469, 2863,  151069,  7, }, // Pacific/Chatham
    {  469, 3022,     474,  5, }, // Pacific/Chuuk
    {  469, 2201,  151076, 14, }, // Pacific/Easter
    {  469, 7323,   25598,  5, }, // Pacific/Efate
    {  469, 6641,   19672,  9, }, // Pacific/Galapagos
    {  469, 2929,    4512,  8, }, // Pacific/Honolulu
    {  469, 2897,    4520,  9, }, // Pacific/Kanton
    {  469, 7061,    4529,  6, }, // Pacific/Noumea
    {  469, 2961,     479,  7, }, // Pacific/Pohnpei
    {  469, 7275,  120324,  5, }, // Pacific/Tahiti
    {  469, 6675,  151090,  6, }, // Pacific/Tarawa
    {  470, 4961,  151096, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Angola
    {  470,   44,   72597,  6, }, // Africa/Bamako
    {  470, 6803,  151106,  5, }, // Africa/Dakar
    {  470, 4945,  151111,  6, }, // Africa/Djibouti
    {  470, 5053,  151117,  7, }, // Africa/Kampala
    {  470, 4903,   72635,  6, }, // Africa/Lusaka
    {  470, 3755,  120583,  7, }, // Africa/Nairobi
    {  470, 5173,   63968,  8, }, // Africa/Ndjamena
    {  470, 5256,   24961,  6, }, // Africa/Niamey
    {  470, 5127,  151124, 10, }, // Africa/Porto-Novo
    {  470, 2732,   34368,  7, }, // Africa/Tripoli
    {  470, 6503,   72657,  5, }, // Africa/Tunis
    {  470, 4456,  151134,  9, }, // America/Araguaina
    {  470, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  470, 3581,   72662,  4, }, // America/Bahia
    {  470, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  470, 3770,    3861,  6, }, // America/Cancun
    {  470, 5398,  151143,  6, }, // America/Cayman
    {  470, 5640,  151149,  7, }, // America/Curacao
    {  470, 5656,  151156,  8, }, // America/Dominica
    {  470, 5343,    5090,  8, }, // America/El_Salvador
    {  470, 4000,  151164, 10, }, // America/Montevideo
    {  470, 5741,  151174, 10, }, // America/Montserrat
    {  470, 3294,  151184, 11, }, // America/New_York
    {  470, 3847,  151195, 14, }, // America/Port-au-Prince
    {  470, 5871,  151209, 16, }, // America/Port_of_Spain
    {  470, 2045,     332, 10, }, // America/St_Johns
    {  470,  855,     360, 10, }, // America/St_Thomas
    {  470, 5487,  151225,  4, }, // Asia/Aden
    {  470, 5440,   89842,  5, }, // Asia/Aqtau
    {  470, 5270,  151229,  6, }, // Asia/Aqtobe
    {  470, 3509,  151235,  7, }, // Asia/Baghdad
    {  470, 5451,  151242,  5, }, // Asia/Bahrain
    {  470, 4368,  151247, 11, }, // Asia/Bangkok
    {  470, 1060,  151106,  4, }, // Asia/Dhaka
    {  470, 4141,  151258,  7, }, // Asia/Karachi
    {  470, 1152,  151265,  8, }, // Asia/Kathmandu
    {  470, 5464,   34541,  6, }, // Asia/Kuwait
    {  470, 1316,   63630,  8, }, // Asia/Makassar
    {  470, 8671,     434,  8, }, // Asia/Qostanay
    {  470, 3100,  151273,  5, }, // Asia/Taipei
    {  470, 2638,  151278,  6, }, // Asia/Tehran
    {  470, 1284,  151284,  6, }, // Asia/Thimphu
    {  470, 1202,     442,  6, }, // Asia/Yangon
    {  470, 3649,   25397,  6, }, // Asia/Yerevan
    {  470, 1379,  150950,  5, }, // Atlantic/Faroe
    {  470, 2614,  151290, 12, }, // Atlantic/Reykjavik
    {  470, 1531,  151302, 17, }, // Australia/Lord_Howe
    {  470, 6390,  151319, 10, }, // Europe/Amsterdam
    {  470, 8751,  151329,  8, }, // Europe/Busingen
    {  470, 6238,  151337,  9, }, // Europe/Copenhagen
    {  470, 6561,  151346, 10, }, // Europe/Helsinki
    {  470, 3881,  151356, 12, }, // Europe/Kaliningrad
    {  470, 2393,   19543,  7, }, // Europe/London
    {  470, 4794,   61602,  6, }, // Europe/Madrid
    {  470, 6359,   73043,  6, }, // Europe/Monaco
    {  470, 3443,  151368,  7, }, // Europe/Moscow
    {  470, 6419,  151375, 11, }, // Europe/San_Marino
    {  470, 8846,  151386,  5, }, // Europe/Tallinn
    {  470, 5979,  151391, 13, }, // Indian/Christmas
    {  470, 5996,  151404, 11, }, // Indian/Cocos
    {  470, 7223,  151415,  4, }, // Indian/Mahe
    {  470, 3971,  151419,  8, }, // Indian/Mauritius
    {  470, 2863,   56984,  7, }, // Pacific/Chatham
    {  470, 2201,  151427, 14, }, // Pacific/Easter
    {  470, 7323,  151441,  5, }, // Pacific/Efate
    {  470, 7157,  151446, 14, }, // Pacific/Pitcairn
    {  470, 6675,  151460,  6, }, // Pacific/Tarawa
    {  471, 4961,  151096, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Cape Verde
    {  471,   44,   72597,  6, }, // Africa/Bamako
    {  471, 6803,  151106,  5, }, // Africa/Dakar
    {  471, 4945,  151111,  6, }, // Africa/Djibouti
    {  471, 5053,  151117,  7, }, // Africa/Kampala
    {  471, 4903,   72635,  6, }, // Africa/Lusaka
    {  471, 3755,  120583,  7, }, // Africa/Nairobi
    {  471, 5173,   63968,  8, }, // Africa/Ndjamena
    {  471, 5256,   24961,  6, }, // Africa/Niamey
    {  471, 5127,  151124, 10, }, // Africa/Porto-Novo
    {  471, 2732,   34368,  7, }, // Africa/Tripoli
    {  471, 6503,   72657,  5, }, // Africa/Tunis
    {  471, 4456,  151134,  9, }, // America/Araguaina
    {  471, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  471, 3581,   72662,  4, }, // America/Bahia
    {  471, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  471, 3770,    3861,  6, }, // America/Cancun
    {  471, 5398,  151143,  6, }, // America/Cayman
    {  471, 5640,  151149,  7, }, // America/Curacao
    {  471, 5656,  151156,  8, }, // America/Dominica
    {  471, 5343,    5090,  8, }, // America/El_Salvador
    {  471, 4000,  151164, 10, }, // America/Montevideo
    {  471, 5741,  151174, 10, }, // America/Montserrat
    {  471, 3294,  151184, 11, }, // America/New_York
    {  471, 3847,  151195, 14, }, // America/Port-au-Prince
    {  471, 5871,  151209, 16, }, // America/Port_of_Spain
    {  471, 2045,     332, 10, }, // America/St_Johns
    {  471,  855,     360, 10, }, // America/St_Thomas
    {  471, 5487,  151225,  4, }, // Asia/Aden
    {  471, 5440,   89842,  5, }, // Asia/Aqtau
    {  471, 5270,  151229,  6, }, // Asia/Aqtobe
    {  471, 3509,  151235,  7, }, // Asia/Baghdad
    {  471, 5451,  151242,  5, }, // Asia/Bahrain
    {  471, 4368,  151247, 11, }, // Asia/Bangkok
    {  471, 1060,  151106,  4, }, // Asia/Dhaka
    {  471, 4141,  151258,  7, }, // Asia/Karachi
    {  471, 1152,  151265,  8, }, // Asia/Kathmandu
    {  471, 5464,   34541,  6, }, // Asia/Kuwait
    {  471, 1316,   63630,  8, }, // Asia/Makassar
    {  471, 8671,     434,  8, }, // Asia/Qostanay
    {  471, 3100,  151273,  5, }, // Asia/Taipei
    {  471, 2638,  151278,  6, }, // Asia/Tehran
    {  471, 1284,  151284,  6, }, // Asia/Thimphu
    {  471, 1202,     442,  6, }, // Asia/Yangon
    {  471, 3649,   25397,  6, }, // Asia/Yerevan
    {  471, 1379,  150950,  5, }, // Atlantic/Faroe
    {  471, 2614,  151290, 12, }, // Atlantic/Reykjavik
    {  471, 1531,  151302, 17, }, // Australia/Lord_Howe
    {  471, 6390,  151319, 10, }, // Europe/Amsterdam
    {  471, 8751,  151329,  8, }, // Europe/Busingen
    {  471, 6238,  151337,  9, }, // Europe/Copenhagen
    {  471, 6561,  151346, 10, }, // Europe/Helsinki
    {  471, 3881,  151356, 12, }, // Europe/Kaliningrad
    {  471, 2393,   19543,  7, }, // Europe/London
    {  471, 4794,   61602,  6, }, // Europe/Madrid
    {  471, 6359,   73043,  6, }, // Europe/Monaco
    {  471, 3443,  151368,  7, }, // Europe/Moscow
    {  471, 6419,  151375, 11, }, // Europe/San_Marino
    {  471, 8846,  151386,  5, }, // Europe/Tallinn
    {  471, 5979,  151391, 13, }, // Indian/Christmas
    {  471, 5996,  151404, 11, }, // Indian/Cocos
    {  471, 7223,  151415,  4, }, // Indian/Mahe
    {  471, 3971,  151419,  8, }, // Indian/Mauritius
    {  471, 2863,   56984,  7, }, // Pacific/Chatham
    {  471, 2201,  151427, 14, }, // Pacific/Easter
    {  471, 7323,  151441,  5, }, // Pacific/Efate
    {  471, 7157,  151446, 14, }, // Pacific/Pitcairn
    {  471, 6675,  151460,  6, }, // Pacific/Tarawa
    {  472, 4961,  151096, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Equatorial Guinea
    {  472,   44,   72597,  6, }, // Africa/Bamako
    {  472, 6803,  151106,  5, }, // Africa/Dakar
    {  472, 4945,  151111,  6, }, // Africa/Djibouti
    {  472, 5053,  151117,  7, }, // Africa/Kampala
    {  472, 4903,   72635,  6, }, // Africa/Lusaka
    {  472, 3755,  120583,  7, }, // Africa/Nairobi
    {  472, 5173,   63968,  8, }, // Africa/Ndjamena
    {  472, 5256,   24961,  6, }, // Africa/Niamey
    {  472, 5127,  151124, 10, }, // Africa/Porto-Novo
    {  472, 2732,   34368,  7, }, // Africa/Tripoli
    {  472, 6503,   72657,  5, }, // Africa/Tunis
    {  472, 4456,  151134,  9, }, // America/Araguaina
    {  472, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  472, 3581,   72662,  4, }, // America/Bahia
    {  472, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  472, 3770,    3861,  6, }, // America/Cancun
    {  472, 5398,  151143,  6, }, // America/Cayman
    {  472, 5640,  151149,  7, }, // America/Curacao
    {  472, 5656,  151156,  8, }, // America/Dominica
    {  472, 5343,    5090,  8, }, // America/El_Salvador
    {  472, 4000,  151164, 10, }, // America/Montevideo
    {  472, 5741,  151174, 10, }, // America/Montserrat
    {  472, 3294,  151184, 11, }, // America/New_York
    {  472, 3847,  151195, 14, }, // America/Port-au-Prince
    {  472, 5871,  151209, 16, }, // America/Port_of_Spain
    {  472, 2045,     332, 10, }, // America/St_Johns
    {  472,  855,     360, 10, }, // America/St_Thomas
    {  472, 5487,  151225,  4, }, // Asia/Aden
    {  472, 5440,   89842,  5, }, // Asia/Aqtau
    {  472, 5270,  151229,  6, }, // Asia/Aqtobe
    {  472, 3509,  151235,  7, }, // Asia/Baghdad
    {  472, 5451,  151242,  5, }, // Asia/Bahrain
    {  472, 4368,  151247, 11, }, // Asia/Bangkok
    {  472, 1060,  151106,  4, }, // Asia/Dhaka
    {  472, 4141,  151258,  7, }, // Asia/Karachi
    {  472, 1152,  151265,  8, }, // Asia/Kathmandu
    {  472, 5464,   34541,  6, }, // Asia/Kuwait
    {  472, 1316,   63630,  8, }, // Asia/Makassar
    {  472, 8671,     434,  8, }, // Asia/Qostanay
    {  472, 3100,  151273,  5, }, // Asia/Taipei
    {  472, 2638,  151278,  6, }, // Asia/Tehran
    {  472, 1284,  151284,  6, }, // Asia/Thimphu
    {  472, 1202,     442,  6, }, // Asia/Yangon
    {  472, 3649,   25397,  6, }, // Asia/Yerevan
    {  472, 1379,  150950,  5, }, // Atlantic/Faroe
    {  472, 2614,  151290, 12, }, // Atlantic/Reykjavik
    {  472, 1531,  151302, 17, }, // Australia/Lord_Howe
    {  472, 6390,  151319, 10, }, // Europe/Amsterdam
    {  472, 8751,  151329,  8, }, // Europe/Busingen
    {  472, 6238,  151337,  9, }, // Europe/Copenhagen
    {  472, 6561,  151346, 10, }, // Europe/Helsinki
    {  472, 3881,  151356, 12, }, // Europe/Kaliningrad
    {  472, 2393,   19543,  7, }, // Europe/London
    {  472, 4794,   61602,  6, }, // Europe/Madrid
    {  472, 6359,   73043,  6, }, // Europe/Monaco
    {  472, 3443,  151368,  7, }, // Europe/Moscow
    {  472, 6419,  151375, 11, }, // Europe/San_Marino
    {  472, 8846,  151386,  5, }, // Europe/Tallinn
    {  472, 5979,  151391, 13, }, // Indian/Christmas
    {  472, 5996,  151404, 11, }, // Indian/Cocos
    {  472, 7223,  151415,  4, }, // Indian/Mahe
    {  472, 3971,  151419,  8, }, // Indian/Mauritius
    {  472, 2863,   56984,  7, }, // Pacific/Chatham
    {  472, 2201,  151427, 14, }, // Pacific/Easter
    {  472, 7323,  151441,  5, }, // Pacific/Efate
    {  472, 7157,  151446, 14, }, // Pacific/Pitcairn
    {  472, 6675,  151460,  6, }, // Pacific/Tarawa
    {  473, 4961,  151096, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Guinea-Bissau
    {  473,   44,   72597,  6, }, // Africa/Bamako
    {  473, 6803,  151106,  5, }, // Africa/Dakar
    {  473, 4945,  151111,  6, }, // Africa/Djibouti
    {  473, 5053,  151117,  7, }, // Africa/Kampala
    {  473, 4903,   72635,  6, }, // Africa/Lusaka
    {  473, 3755,  120583,  7, }, // Africa/Nairobi
    {  473, 5173,   63968,  8, }, // Africa/Ndjamena
    {  473, 5256,   24961,  6, }, // Africa/Niamey
    {  473, 5127,  151124, 10, }, // Africa/Porto-Novo
    {  473, 2732,   34368,  7, }, // Africa/Tripoli
    {  473, 6503,   72657,  5, }, // Africa/Tunis
    {  473, 4456,  151134,  9, }, // America/Araguaina
    {  473, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  473, 3581,   72662,  4, }, // America/Bahia
    {  473, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  473, 3770,    3861,  6, }, // America/Cancun
    {  473, 5398,  151143,  6, }, // America/Cayman
    {  473, 5640,  151149,  7, }, // America/Curacao
    {  473, 5656,  151156,  8, }, // America/Dominica
    {  473, 5343,    5090,  8, }, // America/El_Salvador
    {  473, 4000,  151164, 10, }, // America/Montevideo
    {  473, 5741,  151174, 10, }, // America/Montserrat
    {  473, 3294,  151184, 11, }, // America/New_York
    {  473, 3847,  151195, 14, }, // America/Port-au-Prince
    {  473, 5871,  151209, 16, }, // America/Port_of_Spain
    {  473, 2045,     332, 10, }, // America/St_Johns
    {  473,  855,     360, 10, }, // America/St_Thomas
    {  473, 5487,  151225,  4, }, // Asia/Aden
    {  473, 5440,   89842,  5, }, // Asia/Aqtau
    {  473, 5270,  151229,  6, }, // Asia/Aqtobe
    {  473, 3509,  151235,  7, }, // Asia/Baghdad
    {  473, 5451,  151242,  5, }, // Asia/Bahrain
    {  473, 4368,  151247, 11, }, // Asia/Bangkok
    {  473, 1060,  151106,  4, }, // Asia/Dhaka
    {  473, 4141,  151258,  7, }, // Asia/Karachi
    {  473, 1152,  151265,  8, }, // Asia/Kathmandu
    {  473, 5464,   34541,  6, }, // Asia/Kuwait
    {  473, 1316,   63630,  8, }, // Asia/Makassar
    {  473, 8671,     434,  8, }, // Asia/Qostanay
    {  473, 3100,  151273,  5, }, // Asia/Taipei
    {  473, 2638,  151278,  6, }, // Asia/Tehran
    {  473, 1284,  151284,  6, }, // Asia/Thimphu
    {  473, 1202,     442,  6, }, // Asia/Yangon
    {  473, 3649,   25397,  6, }, // Asia/Yerevan
    {  473, 1379,  150950,  5, }, // Atlantic/Faroe
    {  473, 2614,  151290, 12, }, // Atlantic/Reykjavik
    {  473, 1531,  151302, 17, }, // Australia/Lord_Howe
    {  473, 6390,  151319, 10, }, // Europe/Amsterdam
    {  473, 8751,  151329,  8, }, // Europe/Busingen
    {  473, 6238,  151337,  9, }, // Europe/Copenhagen
    {  473, 6561,  151346, 10, }, // Europe/Helsinki
    {  473, 3881,  151356, 12, }, // Europe/Kaliningrad
    {  473, 2393,   19543,  7, }, // Europe/London
    {  473, 4794,   61602,  6, }, // Europe/Madrid
    {  473, 6359,   73043,  6, }, // Europe/Monaco
    {  473, 3443,  151368,  7, }, // Europe/Moscow
    {  473, 6419,  151375, 11, }, // Europe/San_Marino
    {  473, 8846,  151386,  5, }, // Europe/Tallinn
    {  473, 5979,  151391, 13, }, // Indian/Christmas
    {  473, 5996,  151404, 11, }, // Indian/Cocos
    {  473, 7223,  151415,  4, }, // Indian/Mahe
    {  473, 3971,  151419,  8, }, // Indian/Mauritius
    {  473, 2863,   56984,  7, }, // Pacific/Chatham
    {  473, 2201,  151427, 14, }, // Pacific/Easter
    {  473, 7323,  151441,  5, }, // Pacific/Efate
    {  473, 7157,  151446, 14, }, // Pacific/Pitcairn
    {  473, 6675,  151460,  6, }, // Pacific/Tarawa
    {  474, 4961,  151096, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Luxembourg
    {  474,   44,   72597,  6, }, // Africa/Bamako
    {  474, 6803,  151106,  5, }, // Africa/Dakar
    {  474, 4945,  151111,  6, }, // Africa/Djibouti
    {  474, 5053,  151117,  7, }, // Africa/Kampala
    {  474, 4903,   72635,  6, }, // Africa/Lusaka
    {  474, 3755,  120583,  7, }, // Africa/Nairobi
    {  474, 5173,   63968,  8, }, // Africa/Ndjamena
    {  474, 5256,   24961,  6, }, // Africa/Niamey
    {  474, 5127,  151124, 10, }, // Africa/Porto-Novo
    {  474, 2732,   34368,  7, }, // Africa/Tripoli
    {  474, 6503,   72657,  5, }, // Africa/Tunis
    {  474, 4456,  151134,  9, }, // America/Araguaina
    {  474, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  474, 3581,   72662,  4, }, // America/Bahia
    {  474, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  474, 3770,    3861,  6, }, // America/Cancun
    {  474, 5398,  151143,  6, }, // America/Cayman
    {  474, 5640,  151149,  7, }, // America/Curacao
    {  474, 5656,  151156,  8, }, // America/Dominica
    {  474, 5343,    5090,  8, }, // America/El_Salvador
    {  474, 4000,  151164, 10, }, // America/Montevideo
    {  474, 5741,  151174, 10, }, // America/Montserrat
    {  474, 3294,  151184, 11, }, // America/New_York
    {  474, 3847,  151195, 14, }, // America/Port-au-Prince
    {  474, 5871,  151209, 16, }, // America/Port_of_Spain
    {  474, 2045,     332, 10, }, // America/St_Johns
    {  474,  855,     360, 10, }, // America/St_Thomas
    {  474, 5487,  151225,  4, }, // Asia/Aden
    {  474, 5440,   89842,  5, }, // Asia/Aqtau
    {  474, 5270,  151229,  6, }, // Asia/Aqtobe
    {  474, 3509,  151235,  7, }, // Asia/Baghdad
    {  474, 5451,  151242,  5, }, // Asia/Bahrain
    {  474, 4368,  151247, 11, }, // Asia/Bangkok
    {  474, 1060,  151106,  4, }, // Asia/Dhaka
    {  474, 4141,  151258,  7, }, // Asia/Karachi
    {  474, 1152,  151265,  8, }, // Asia/Kathmandu
    {  474, 5464,   34541,  6, }, // Asia/Kuwait
    {  474, 1316,   63630,  8, }, // Asia/Makassar
    {  474, 8671,     434,  8, }, // Asia/Qostanay
    {  474, 3100,  151273,  5, }, // Asia/Taipei
    {  474, 2638,  151278,  6, }, // Asia/Tehran
    {  474, 1284,  151284,  6, }, // Asia/Thimphu
    {  474, 1202,     442,  6, }, // Asia/Yangon
    {  474, 3649,   25397,  6, }, // Asia/Yerevan
    {  474, 1379,  150950,  5, }, // Atlantic/Faroe
    {  474, 2614,  151290, 12, }, // Atlantic/Reykjavik
    {  474, 1531,  151302, 17, }, // Australia/Lord_Howe
    {  474, 6390,  151319, 10, }, // Europe/Amsterdam
    {  474, 8751,  151329,  8, }, // Europe/Busingen
    {  474, 6238,  151337,  9, }, // Europe/Copenhagen
    {  474, 6561,  151346, 10, }, // Europe/Helsinki
    {  474, 3881,  151356, 12, }, // Europe/Kaliningrad
    {  474, 2393,   19543,  7, }, // Europe/London
    {  474, 4794,   61602,  6, }, // Europe/Madrid
    {  474, 6359,   73043,  6, }, // Europe/Monaco
    {  474, 3443,  151368,  7, }, // Europe/Moscow
    {  474, 6419,  151375, 11, }, // Europe/San_Marino
    {  474, 8846,  151386,  5, }, // Europe/Tallinn
    {  474, 5979,  151391, 13, }, // Indian/Christmas
    {  474, 5996,  151404, 11, }, // Indian/Cocos
    {  474, 7223,  151415,  4, }, // Indian/Mahe
    {  474, 3971,  151419,  8, }, // Indian/Mauritius
    {  474, 2863,   56984,  7, }, // Pacific/Chatham
    {  474, 2201,  151427, 14, }, // Pacific/Easter
    {  474, 7323,  151441,  5, }, // Pacific/Efate
    {  474, 7157,  151446, 14, }, // Pacific/Pitcairn
    {  474, 6675,  151460,  6, }, // Pacific/Tarawa
    {  475, 4961,  151096, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Macao
    {  475,   44,   72597,  6, }, // Africa/Bamako
    {  475, 6803,  151106,  5, }, // Africa/Dakar
    {  475, 4945,  151111,  6, }, // Africa/Djibouti
    {  475, 5053,  151117,  7, }, // Africa/Kampala
    {  475, 4903,   72635,  6, }, // Africa/Lusaka
    {  475, 3755,  120583,  7, }, // Africa/Nairobi
    {  475, 5173,   63968,  8, }, // Africa/Ndjamena
    {  475, 5256,   24961,  6, }, // Africa/Niamey
    {  475, 5127,  151124, 10, }, // Africa/Porto-Novo
    {  475, 2732,   34368,  7, }, // Africa/Tripoli
    {  475, 6503,   72657,  5, }, // Africa/Tunis
    {  475, 4456,  151134,  9, }, // America/Araguaina
    {  475, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  475, 3581,   72662,  4, }, // America/Bahia
    {  475, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  475, 3770,    3861,  6, }, // America/Cancun
    {  475, 5398,  151143,  6, }, // America/Cayman
    {  475, 5640,  151149,  7, }, // America/Curacao
    {  475, 5656,  151156,  8, }, // America/Dominica
    {  475, 5343,    5090,  8, }, // America/El_Salvador
    {  475, 4000,  151164, 10, }, // America/Montevideo
    {  475, 5741,  151174, 10, }, // America/Montserrat
    {  475, 3294,  151184, 11, }, // America/New_York
    {  475, 3847,  151195, 14, }, // America/Port-au-Prince
    {  475, 5871,  151209, 16, }, // America/Port_of_Spain
    {  475, 2045,     332, 10, }, // America/St_Johns
    {  475,  855,     360, 10, }, // America/St_Thomas
    {  475, 5487,  151225,  4, }, // Asia/Aden
    {  475, 5440,   89842,  5, }, // Asia/Aqtau
    {  475, 5270,  151229,  6, }, // Asia/Aqtobe
    {  475, 3509,  151235,  7, }, // Asia/Baghdad
    {  475, 5451,  151242,  5, }, // Asia/Bahrain
    {  475, 4368,  151247, 11, }, // Asia/Bangkok
    {  475, 1060,  151106,  4, }, // Asia/Dhaka
    {  475, 4141,  151258,  7, }, // Asia/Karachi
    {  475, 1152,  151265,  8, }, // Asia/Kathmandu
    {  475, 5464,   34541,  6, }, // Asia/Kuwait
    {  475, 1316,   63630,  8, }, // Asia/Makassar
    {  475, 8671,     434,  8, }, // Asia/Qostanay
    {  475, 3100,  151273,  5, }, // Asia/Taipei
    {  475, 2638,  151278,  6, }, // Asia/Tehran
    {  475, 1284,  151284,  6, }, // Asia/Thimphu
    {  475, 1202,     442,  6, }, // Asia/Yangon
    {  475, 3649,   25397,  6, }, // Asia/Yerevan
    {  475, 1379,  150950,  5, }, // Atlantic/Faroe
    {  475, 2614,  151290, 12, }, // Atlantic/Reykjavik
    {  475, 1531,  151302, 17, }, // Australia/Lord_Howe
    {  475, 6390,  151319, 10, }, // Europe/Amsterdam
    {  475, 8751,  151329,  8, }, // Europe/Busingen
    {  475, 6238,  151337,  9, }, // Europe/Copenhagen
    {  475, 6561,  151346, 10, }, // Europe/Helsinki
    {  475, 3881,  151356, 12, }, // Europe/Kaliningrad
    {  475, 2393,   19543,  7, }, // Europe/London
    {  475, 4794,   61602,  6, }, // Europe/Madrid
    {  475, 6359,   73043,  6, }, // Europe/Monaco
    {  475, 3443,  151368,  7, }, // Europe/Moscow
    {  475, 6419,  151375, 11, }, // Europe/San_Marino
    {  475, 8846,  151386,  5, }, // Europe/Tallinn
    {  475, 5979,  151391, 13, }, // Indian/Christmas
    {  475, 5996,  151404, 11, }, // Indian/Cocos
    {  475, 7223,  151415,  4, }, // Indian/Mahe
    {  475, 3971,  151419,  8, }, // Indian/Mauritius
    {  475, 2863,   56984,  7, }, // Pacific/Chatham
    {  475, 2201,  151427, 14, }, // Pacific/Easter
    {  475, 7323,  151441,  5, }, // Pacific/Efate
    {  475, 7157,  151446, 14, }, // Pacific/Pitcairn
    {  475, 6675,  151460,  6, }, // Pacific/Tarawa
    {  476, 4961,  151096, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Mozambique
    {  476,   44,   72597,  6, }, // Africa/Bamako
    {  476, 6803,  151106,  5, }, // Africa/Dakar
    {  476, 4945,  151111,  6, }, // Africa/Djibouti
    {  476, 5053,  151117,  7, }, // Africa/Kampala
    {  476, 4903,   72635,  6, }, // Africa/Lusaka
    {  476, 3755,  120583,  7, }, // Africa/Nairobi
    {  476, 5173,   63968,  8, }, // Africa/Ndjamena
    {  476, 5256,   24961,  6, }, // Africa/Niamey
    {  476, 5127,  151124, 10, }, // Africa/Porto-Novo
    {  476, 2732,   34368,  7, }, // Africa/Tripoli
    {  476, 6503,   72657,  5, }, // Africa/Tunis
    {  476, 4456,  151134,  9, }, // America/Araguaina
    {  476, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  476, 3581,   72662,  4, }, // America/Bahia
    {  476, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  476, 3770,    3861,  6, }, // America/Cancun
    {  476, 5398,  151143,  6, }, // America/Cayman
    {  476, 5640,  151149,  7, }, // America/Curacao
    {  476, 5656,  151156,  8, }, // America/Dominica
    {  476, 5343,    5090,  8, }, // America/El_Salvador
    {  476, 4000,  151164, 10, }, // America/Montevideo
    {  476, 5741,  151174, 10, }, // America/Montserrat
    {  476, 3294,  151184, 11, }, // America/New_York
    {  476, 3847,  151195, 14, }, // America/Port-au-Prince
    {  476, 5871,  151209, 16, }, // America/Port_of_Spain
    {  476, 2045,     332, 10, }, // America/St_Johns
    {  476,  855,     360, 10, }, // America/St_Thomas
    {  476, 5487,  151225,  4, }, // Asia/Aden
    {  476, 5440,   89842,  5, }, // Asia/Aqtau
    {  476, 5270,  151229,  6, }, // Asia/Aqtobe
    {  476, 3509,  151235,  7, }, // Asia/Baghdad
    {  476, 5451,  151242,  5, }, // Asia/Bahrain
    {  476, 4368,  151247, 11, }, // Asia/Bangkok
    {  476, 1060,  151106,  4, }, // Asia/Dhaka
    {  476, 4141,  151258,  7, }, // Asia/Karachi
    {  476, 1152,  151265,  8, }, // Asia/Kathmandu
    {  476, 5464,   34541,  6, }, // Asia/Kuwait
    {  476, 1316,   63630,  8, }, // Asia/Makassar
    {  476, 8671,     434,  8, }, // Asia/Qostanay
    {  476, 3100,  151273,  5, }, // Asia/Taipei
    {  476, 2638,  151278,  6, }, // Asia/Tehran
    {  476, 1284,  151284,  6, }, // Asia/Thimphu
    {  476, 1202,     442,  6, }, // Asia/Yangon
    {  476, 3649,   25397,  6, }, // Asia/Yerevan
    {  476, 1379,  150950,  5, }, // Atlantic/Faroe
    {  476, 2614,  151290, 12, }, // Atlantic/Reykjavik
    {  476, 1531,  151302, 17, }, // Australia/Lord_Howe
    {  476, 6390,  151319, 10, }, // Europe/Amsterdam
    {  476, 8751,  151329,  8, }, // Europe/Busingen
    {  476, 6238,  151337,  9, }, // Europe/Copenhagen
    {  476, 6561,  151346, 10, }, // Europe/Helsinki
    {  476, 3881,  151356, 12, }, // Europe/Kaliningrad
    {  476, 2393,   19543,  7, }, // Europe/London
    {  476, 4794,   61602,  6, }, // Europe/Madrid
    {  476, 6359,   73043,  6, }, // Europe/Monaco
    {  476, 3443,  151368,  7, }, // Europe/Moscow
    {  476, 6419,  151375, 11, }, // Europe/San_Marino
    {  476, 8846,  151386,  5, }, // Europe/Tallinn
    {  476, 5979,  151391, 13, }, // Indian/Christmas
    {  476, 5996,  151404, 11, }, // Indian/Cocos
    {  476, 7223,  151415,  4, }, // Indian/Mahe
    {  476, 3971,  151419,  8, }, // Indian/Mauritius
    {  476, 2863,   56984,  7, }, // Pacific/Chatham
    {  476, 2201,  151427, 14, }, // Pacific/Easter
    {  476, 7323,  151441,  5, }, // Pacific/Efate
    {  476, 7157,  151446, 14, }, // Pacific/Pitcairn
    {  476, 6675,  151460,  6, }, // Pacific/Tarawa
    {  477, 4961,  151096, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Portugal
    {  477,   44,   72597,  6, }, // Africa/Bamako
    {  477, 6803,  151106,  5, }, // Africa/Dakar
    {  477, 4945,  151111,  6, }, // Africa/Djibouti
    {  477, 5053,  151117,  7, }, // Africa/Kampala
    {  477, 4903,   72635,  6, }, // Africa/Lusaka
    {  477, 3755,  120583,  7, }, // Africa/Nairobi
    {  477, 5173,   63968,  8, }, // Africa/Ndjamena
    {  477, 5256,   24961,  6, }, // Africa/Niamey
    {  477, 5127,  151124, 10, }, // Africa/Porto-Novo
    {  477, 2732,   34368,  7, }, // Africa/Tripoli
    {  477, 6503,   72657,  5, }, // Africa/Tunis
    {  477, 4456,  151134,  9, }, // America/Araguaina
    {  477, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  477, 3581,   72662,  4, }, // America/Bahia
    {  477, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  477, 3770,    3861,  6, }, // America/Cancun
    {  477, 5398,  151143,  6, }, // America/Cayman
    {  477, 5640,  151149,  7, }, // America/Curacao
    {  477, 5656,  151156,  8, }, // America/Dominica
    {  477, 5343,    5090,  8, }, // America/El_Salvador
    {  477, 4000,  151164, 10, }, // America/Montevideo
    {  477, 5741,  151174, 10, }, // America/Montserrat
    {  477, 3294,  151184, 11, }, // America/New_York
    {  477, 3847,  151195, 14, }, // America/Port-au-Prince
    {  477, 5871,  151209, 16, }, // America/Port_of_Spain
    {  477, 2045,     332, 10, }, // America/St_Johns
    {  477,  855,     360, 10, }, // America/St_Thomas
    {  477, 5487,  151225,  4, }, // Asia/Aden
    {  477, 5440,   89842,  5, }, // Asia/Aqtau
    {  477, 5270,  151229,  6, }, // Asia/Aqtobe
    {  477, 3509,  151235,  7, }, // Asia/Baghdad
    {  477, 5451,  151242,  5, }, // Asia/Bahrain
    {  477, 4368,  151247, 11, }, // Asia/Bangkok
    {  477, 1060,  151106,  4, }, // Asia/Dhaka
    {  477, 4141,  151258,  7, }, // Asia/Karachi
    {  477, 1152,  151265,  8, }, // Asia/Kathmandu
    {  477, 5464,   34541,  6, }, // Asia/Kuwait
    {  477, 1316,   63630,  8, }, // Asia/Makassar
    {  477, 8671,     434,  8, }, // Asia/Qostanay
    {  477, 3100,  151273,  5, }, // Asia/Taipei
    {  477, 2638,  151278,  6, }, // Asia/Tehran
    {  477, 1284,  151284,  6, }, // Asia/Thimphu
    {  477, 1202,     442,  6, }, // Asia/Yangon
    {  477, 3649,   25397,  6, }, // Asia/Yerevan
    {  477, 1379,  150950,  5, }, // Atlantic/Faroe
    {  477, 2614,  151290, 12, }, // Atlantic/Reykjavik
    {  477, 1531,  151302, 17, }, // Australia/Lord_Howe
    {  477, 6390,  151319, 10, }, // Europe/Amsterdam
    {  477, 8751,  151329,  8, }, // Europe/Busingen
    {  477, 6238,  151337,  9, }, // Europe/Copenhagen
    {  477, 6561,  151346, 10, }, // Europe/Helsinki
    {  477, 3881,  151356, 12, }, // Europe/Kaliningrad
    {  477, 2393,   19543,  7, }, // Europe/London
    {  477, 4794,   61602,  6, }, // Europe/Madrid
    {  477, 6359,   73043,  6, }, // Europe/Monaco
    {  477, 3443,  151368,  7, }, // Europe/Moscow
    {  477, 6419,  151375, 11, }, // Europe/San_Marino
    {  477, 8846,  151386,  5, }, // Europe/Tallinn
    {  477, 5979,  151391, 13, }, // Indian/Christmas
    {  477, 5996,  151404, 11, }, // Indian/Cocos
    {  477, 7223,  151415,  4, }, // Indian/Mahe
    {  477, 3971,  151419,  8, }, // Indian/Mauritius
    {  477, 2863,   56984,  7, }, // Pacific/Chatham
    {  477, 2201,  151427, 14, }, // Pacific/Easter
    {  477, 7323,  151441,  5, }, // Pacific/Efate
    {  477, 7157,  151446, 14, }, // Pacific/Pitcairn
    {  477, 6675,  151460,  6, }, // Pacific/Tarawa
    {  478, 4961,  151096, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Sao Tome and Principe
    {  478,   44,   72597,  6, }, // Africa/Bamako
    {  478, 6803,  151106,  5, }, // Africa/Dakar
    {  478, 4945,  151111,  6, }, // Africa/Djibouti
    {  478, 5053,  151117,  7, }, // Africa/Kampala
    {  478, 4903,   72635,  6, }, // Africa/Lusaka
    {  478, 3755,  120583,  7, }, // Africa/Nairobi
    {  478, 5173,   63968,  8, }, // Africa/Ndjamena
    {  478, 5256,   24961,  6, }, // Africa/Niamey
    {  478, 5127,  151124, 10, }, // Africa/Porto-Novo
    {  478, 2732,   34368,  7, }, // Africa/Tripoli
    {  478, 6503,   72657,  5, }, // Africa/Tunis
    {  478, 4456,  151134,  9, }, // America/Araguaina
    {  478, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  478, 3581,   72662,  4, }, // America/Bahia
    {  478, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  478, 3770,    3861,  6, }, // America/Cancun
    {  478, 5398,  151143,  6, }, // America/Cayman
    {  478, 5640,  151149,  7, }, // America/Curacao
    {  478, 5656,  151156,  8, }, // America/Dominica
    {  478, 5343,    5090,  8, }, // America/El_Salvador
    {  478, 4000,  151164, 10, }, // America/Montevideo
    {  478, 5741,  151174, 10, }, // America/Montserrat
    {  478, 3294,  151184, 11, }, // America/New_York
    {  478, 3847,  151195, 14, }, // America/Port-au-Prince
    {  478, 5871,  151209, 16, }, // America/Port_of_Spain
    {  478, 2045,     332, 10, }, // America/St_Johns
    {  478,  855,     360, 10, }, // America/St_Thomas
    {  478, 5487,  151225,  4, }, // Asia/Aden
    {  478, 5440,   89842,  5, }, // Asia/Aqtau
    {  478, 5270,  151229,  6, }, // Asia/Aqtobe
    {  478, 3509,  151235,  7, }, // Asia/Baghdad
    {  478, 5451,  151242,  5, }, // Asia/Bahrain
    {  478, 4368,  151247, 11, }, // Asia/Bangkok
    {  478, 1060,  151106,  4, }, // Asia/Dhaka
    {  478, 4141,  151258,  7, }, // Asia/Karachi
    {  478, 1152,  151265,  8, }, // Asia/Kathmandu
    {  478, 5464,   34541,  6, }, // Asia/Kuwait
    {  478, 1316,   63630,  8, }, // Asia/Makassar
    {  478, 8671,     434,  8, }, // Asia/Qostanay
    {  478, 3100,  151273,  5, }, // Asia/Taipei
    {  478, 2638,  151278,  6, }, // Asia/Tehran
    {  478, 1284,  151284,  6, }, // Asia/Thimphu
    {  478, 1202,     442,  6, }, // Asia/Yangon
    {  478, 3649,   25397,  6, }, // Asia/Yerevan
    {  478, 1379,  150950,  5, }, // Atlantic/Faroe
    {  478, 2614,  151290, 12, }, // Atlantic/Reykjavik
    {  478, 1531,  151302, 17, }, // Australia/Lord_Howe
    {  478, 6390,  151319, 10, }, // Europe/Amsterdam
    {  478, 8751,  151329,  8, }, // Europe/Busingen
    {  478, 6238,  151337,  9, }, // Europe/Copenhagen
    {  478, 6561,  151346, 10, }, // Europe/Helsinki
    {  478, 3881,  151356, 12, }, // Europe/Kaliningrad
    {  478, 2393,   19543,  7, }, // Europe/London
    {  478, 4794,   61602,  6, }, // Europe/Madrid
    {  478, 6359,   73043,  6, }, // Europe/Monaco
    {  478, 3443,  151368,  7, }, // Europe/Moscow
    {  478, 6419,  151375, 11, }, // Europe/San_Marino
    {  478, 8846,  151386,  5, }, // Europe/Tallinn
    {  478, 5979,  151391, 13, }, // Indian/Christmas
    {  478, 5996,  151404, 11, }, // Indian/Cocos
    {  478, 7223,  151415,  4, }, // Indian/Mahe
    {  478, 3971,  151419,  8, }, // Indian/Mauritius
    {  478, 2863,   56984,  7, }, // Pacific/Chatham
    {  478, 2201,  151427, 14, }, // Pacific/Easter
    {  478, 7323,  151441,  5, }, // Pacific/Efate
    {  478, 7157,  151446, 14, }, // Pacific/Pitcairn
    {  478, 6675,  151460,  6, }, // Pacific/Tarawa
    {  479, 4961,  151096, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Switzerland
    {  479,   44,   72597,  6, }, // Africa/Bamako
    {  479, 6803,  151106,  5, }, // Africa/Dakar
    {  479, 4945,  151111,  6, }, // Africa/Djibouti
    {  479, 5053,  151117,  7, }, // Africa/Kampala
    {  479, 4903,   72635,  6, }, // Africa/Lusaka
    {  479, 3755,  120583,  7, }, // Africa/Nairobi
    {  479, 5173,   63968,  8, }, // Africa/Ndjamena
    {  479, 5256,   24961,  6, }, // Africa/Niamey
    {  479, 5127,  151124, 10, }, // Africa/Porto-Novo
    {  479, 2732,   34368,  7, }, // Africa/Tripoli
    {  479, 6503,   72657,  5, }, // Africa/Tunis
    {  479, 4456,  151134,  9, }, // America/Araguaina
    {  479, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  479, 3581,   72662,  4, }, // America/Bahia
    {  479, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  479, 3770,    3861,  6, }, // America/Cancun
    {  479, 5398,  151143,  6, }, // America/Cayman
    {  479, 5640,  151149,  7, }, // America/Curacao
    {  479, 5656,  151156,  8, }, // America/Dominica
    {  479, 5343,    5090,  8, }, // America/El_Salvador
    {  479, 4000,  151164, 10, }, // America/Montevideo
    {  479, 5741,  151174, 10, }, // America/Montserrat
    {  479, 3294,  151184, 11, }, // America/New_York
    {  479, 3847,  151195, 14, }, // America/Port-au-Prince
    {  479, 5871,  151209, 16, }, // America/Port_of_Spain
    {  479, 2045,     332, 10, }, // America/St_Johns
    {  479,  855,     360, 10, }, // America/St_Thomas
    {  479, 5487,  151225,  4, }, // Asia/Aden
    {  479, 5440,   89842,  5, }, // Asia/Aqtau
    {  479, 5270,  151229,  6, }, // Asia/Aqtobe
    {  479, 3509,  151235,  7, }, // Asia/Baghdad
    {  479, 5451,  151242,  5, }, // Asia/Bahrain
    {  479, 4368,  151247, 11, }, // Asia/Bangkok
    {  479, 1060,  151106,  4, }, // Asia/Dhaka
    {  479, 4141,  151258,  7, }, // Asia/Karachi
    {  479, 1152,  151265,  8, }, // Asia/Kathmandu
    {  479, 5464,   34541,  6, }, // Asia/Kuwait
    {  479, 1316,   63630,  8, }, // Asia/Makassar
    {  479, 8671,     434,  8, }, // Asia/Qostanay
    {  479, 3100,  151273,  5, }, // Asia/Taipei
    {  479, 2638,  151278,  6, }, // Asia/Tehran
    {  479, 1284,  151284,  6, }, // Asia/Thimphu
    {  479, 1202,     442,  6, }, // Asia/Yangon
    {  479, 3649,   25397,  6, }, // Asia/Yerevan
    {  479, 1379,  150950,  5, }, // Atlantic/Faroe
    {  479, 2614,  151290, 12, }, // Atlantic/Reykjavik
    {  479, 1531,  151302, 17, }, // Australia/Lord_Howe
    {  479, 6390,  151319, 10, }, // Europe/Amsterdam
    {  479, 8751,  151329,  8, }, // Europe/Busingen
    {  479, 6238,  151337,  9, }, // Europe/Copenhagen
    {  479, 6561,  151346, 10, }, // Europe/Helsinki
    {  479, 3881,  151356, 12, }, // Europe/Kaliningrad
    {  479, 2393,   19543,  7, }, // Europe/London
    {  479, 4794,   61602,  6, }, // Europe/Madrid
    {  479, 6359,   73043,  6, }, // Europe/Monaco
    {  479, 3443,  151368,  7, }, // Europe/Moscow
    {  479, 6419,  151375, 11, }, // Europe/San_Marino
    {  479, 8846,  151386,  5, }, // Europe/Tallinn
    {  479, 5979,  151391, 13, }, // Indian/Christmas
    {  479, 5996,  151404, 11, }, // Indian/Cocos
    {  479, 7223,  151415,  4, }, // Indian/Mahe
    {  479, 3971,  151419,  8, }, // Indian/Mauritius
    {  479, 2863,   56984,  7, }, // Pacific/Chatham
    {  479, 2201,  151427, 14, }, // Pacific/Easter
    {  479, 7323,  151441,  5, }, // Pacific/Efate
    {  479, 7157,  151446, 14, }, // Pacific/Pitcairn
    {  479, 6675,  151460,  6, }, // Pacific/Tarawa
    {  480, 4961,  151096, 10, }, // Africa/Addis_Ababa Portuguese/Latin/Timor-Leste
    {  480,   44,   72597,  6, }, // Africa/Bamako
    {  480, 6803,  151106,  5, }, // Africa/Dakar
    {  480, 4945,  151111,  6, }, // Africa/Djibouti
    {  480, 5053,  151117,  7, }, // Africa/Kampala
    {  480, 4903,   72635,  6, }, // Africa/Lusaka
    {  480, 3755,  120583,  7, }, // Africa/Nairobi
    {  480, 5173,   63968,  8, }, // Africa/Ndjamena
    {  480, 5256,   24961,  6, }, // Africa/Niamey
    {  480, 5127,  151124, 10, }, // Africa/Porto-Novo
    {  480, 2732,   34368,  7, }, // Africa/Tripoli
    {  480, 6503,   72657,  5, }, // Africa/Tunis
    {  480, 4456,  151134,  9, }, // America/Araguaina
    {  480, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  480, 3581,   72662,  4, }, // America/Bahia
    {  480, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  480, 3770,    3861,  6, }, // America/Cancun
    {  480, 5398,  151143,  6, }, // America/Cayman
    {  480, 5640,  151149,  7, }, // America/Curacao
    {  480, 5656,  151156,  8, }, // America/Dominica
    {  480, 5343,    5090,  8, }, // America/El_Salvador
    {  480, 4000,  151164, 10, }, // America/Montevideo
    {  480, 5741,  151174, 10, }, // America/Montserrat
    {  480, 3294,  151184, 11, }, // America/New_York
    {  480, 3847,  151195, 14, }, // America/Port-au-Prince
    {  480, 5871,  151209, 16, }, // America/Port_of_Spain
    {  480, 2045,     332, 10, }, // America/St_Johns
    {  480,  855,     360, 10, }, // America/St_Thomas
    {  480, 5487,  151225,  4, }, // Asia/Aden
    {  480, 5440,   89842,  5, }, // Asia/Aqtau
    {  480, 5270,  151229,  6, }, // Asia/Aqtobe
    {  480, 3509,  151235,  7, }, // Asia/Baghdad
    {  480, 5451,  151242,  5, }, // Asia/Bahrain
    {  480, 4368,  151247, 11, }, // Asia/Bangkok
    {  480, 1060,  151106,  4, }, // Asia/Dhaka
    {  480, 4141,  151258,  7, }, // Asia/Karachi
    {  480, 1152,  151265,  8, }, // Asia/Kathmandu
    {  480, 5464,   34541,  6, }, // Asia/Kuwait
    {  480, 1316,   63630,  8, }, // Asia/Makassar
    {  480, 8671,     434,  8, }, // Asia/Qostanay
    {  480, 3100,  151273,  5, }, // Asia/Taipei
    {  480, 2638,  151278,  6, }, // Asia/Tehran
    {  480, 1284,  151284,  6, }, // Asia/Thimphu
    {  480, 1202,     442,  6, }, // Asia/Yangon
    {  480, 3649,   25397,  6, }, // Asia/Yerevan
    {  480, 1379,  150950,  5, }, // Atlantic/Faroe
    {  480, 2614,  151290, 12, }, // Atlantic/Reykjavik
    {  480, 1531,  151302, 17, }, // Australia/Lord_Howe
    {  480, 6390,  151319, 10, }, // Europe/Amsterdam
    {  480, 8751,  151329,  8, }, // Europe/Busingen
    {  480, 6238,  151337,  9, }, // Europe/Copenhagen
    {  480, 6561,  151346, 10, }, // Europe/Helsinki
    {  480, 3881,  151356, 12, }, // Europe/Kaliningrad
    {  480, 2393,   19543,  7, }, // Europe/London
    {  480, 4794,   61602,  6, }, // Europe/Madrid
    {  480, 6359,   73043,  6, }, // Europe/Monaco
    {  480, 3443,  151368,  7, }, // Europe/Moscow
    {  480, 6419,  151375, 11, }, // Europe/San_Marino
    {  480, 8846,  151386,  5, }, // Europe/Tallinn
    {  480, 5979,  151391, 13, }, // Indian/Christmas
    {  480, 5996,  151404, 11, }, // Indian/Cocos
    {  480, 7223,  151415,  4, }, // Indian/Mahe
    {  480, 3971,  151419,  8, }, // Indian/Mauritius
    {  480, 2863,   56984,  7, }, // Pacific/Chatham
    {  480, 2201,  151427, 14, }, // Pacific/Easter
    {  480, 7323,  151441,  5, }, // Pacific/Efate
    {  480, 7157,  151446, 14, }, // Pacific/Pitcairn
    {  480, 6675,  151460,  6, }, // Pacific/Tarawa
    {  481,   14,       0,  6, }, // Africa/Asmara Prussian/Latin/Poland
    {  481, 4337,       6,  8, }, // Africa/Sao_Tome
    {  481, 4154,      14,  8, }, // America/Asuncion
    {  481,  237,      22,  8, }, // America/Atikokan
    {  481, 7586,      30, 17, }, // America/Bahia_Banderas
    {  481, 3770,      47,  6, }, // America/Cancun
    {  481, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  481, 5640,      66,  7, }, // America/Curacao
    {  481,  481,      73, 13, }, // America/Indiana/Knox
    {  481, 7924,      86, 16, }, // America/Indiana/Marengo
    {  481, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  481, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  481, 8001,     139, 14, }, // America/Indiana/Vevay
    {  481, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  481, 8049,     171, 16, }, // America/Indiana/Winamac
    {  481, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  481, 5849,     207, 22, }, // America/Lower_Princes
    {  481, 8183,     229,  6, }, // America/Merida
    {  481, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  481, 8292,     255, 20, }, // America/North_Dakota/Center
    {  481, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  481,  393,     298,  4, }, // America/Nuuk
    {  481, 6862,     302, 16, }, // America/Scoresbysund
    {  481, 8471,     318, 14, }, // America/St_Barthelemy
    {  481, 2045,     332, 10, }, // America/St_Johns
    {  481, 5780,     342,  9, }, // America/St_Kitts
    {  481, 5797,     351,  9, }, // America/St_Lucia
    {  481,  855,     360, 10, }, // America/St_Thomas
    {  481, 5830,     370, 11, }, // America/St_Vincent
    {  481, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  481, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  481, 1152,     413,  9, }, // Asia/Kathmandu
    {  481,  992,     422,  7, }, // Asia/Kolkata
    {  481, 1178,     429,  5, }, // Asia/Macau
    {  481, 1202,     442,  6, }, // Asia/Yangon
    {  481, 1379,     448,  5, }, // Atlantic/Faroe
    {  481, 6784,     453, 10, }, // Atlantic/St_Helena
    {  481, 2419,     463,  4, }, // Europe/Kyiv
    {  481, 7174,     467,  7, }, // Indian/Reunion
    {  481, 3022,     474,  5, }, // Pacific/Chuuk
    {  481, 2961,     479,  7, }, // Pacific/Pohnpei
    {  482, 6751,  151466,  7, }, // Africa/Abidjan Punjabi/Gurmukhi/India
    {  482, 6723,  151473,  5, }, // Africa/Accra
    {  482, 4961,  151478, 10, }, // Africa/Addis_Ababa
    {  482, 7383,  151488,  7, }, // Africa/Algiers
    {  482,   14,  151495,  6, }, // Africa/Asmara
    {  482,   44,  151501,  5, }, // Africa/Bamako
    {  482, 5159,  151506,  6, }, // Africa/Bangui
    {  482, 6709,  151512,  6, }, // Africa/Banjul
    {  482, 7398,  151518,  5, }, // Africa/Bissau
    {  482, 4859,  151523,  7, }, // Africa/Blantyre
    {  482, 5189,  151530, 11, }, // Africa/Brazzaville
    {  482, 4824,  151541,  9, }, // Africa/Bujumbura
    {  482, 2242,  151550,  4, }, // Africa/Cairo
    {  482, 4019,  151554, 10, }, // Africa/Casablanca
    {  482, 7412,  151564,  5, }, // Africa/Ceuta
    {  482, 6736,  151569,  7, }, // Africa/Conakry
    {  482, 6803,  151576,  4, }, // Africa/Dakar
    {  482, 5032,  151580, 11, }, // Africa/Dar_es_Salaam
    {  482, 4945,  151591,  7, }, // Africa/Djibouti
    {  482, 5145,  151598,  5, }, // Africa/Douala
    {  482, 5068,  151603,  7, }, // Africa/El_Aaiun
    {  482, 6816,  151610,  7, }, // Africa/Freetown
    {  482, 4808,  151617,  7, }, // Africa/Gaborone
    {  482, 4917,  151624,  5, }, // Africa/Harare
    {  482, 4381,  151629,  8, }, // Africa/Johannesburg
    {  482, 4401,  151637,  4, }, // Africa/Juba
    {  482, 5053,  151641,  6, }, // Africa/Kampala
    {  482, 4426,  151647,  6, }, // Africa/Khartoum
    {  482, 4875,  151653,  6, }, // Africa/Kigali
    {  482, 5208,  151659,  8, }, // Africa/Kinshasa
    {  482, 4646,  151667,  5, }, // Africa/Lagos
    {  482, 5238,  151672,  8, }, // Africa/Libreville
    {  482, 6832,  151680,  3, }, // Africa/Lome
    {  482, 5113,  151683,  6, }, // Africa/Luanda
    {  482, 4841,  151689, 10, }, // Africa/Lubumbashi
    {  482, 4903,  151699,  6, }, // Africa/Lusaka
    {  482, 5224,  151705,  6, }, // Africa/Malabo
    {  482, 4889,  151711,  6, }, // Africa/Maputo
    {  482, 5099,  151717,  5, }, // Africa/Maseru
    {  482, 5084,  151722,  8, }, // Africa/Mbabane
    {  482, 5015,  151730,  9, }, // Africa/Mogadishu
    {  482, 6997,  151739,  8, }, // Africa/Monrovia
    {  482, 3755,  151747,  6, }, // Africa/Nairobi
    {  482, 5173,  151753,  8, }, // Africa/Ndjamena
    {  482, 5256,  151761,  5, }, // Africa/Niamey
    {  482, 6766,  151766,  8, }, // Africa/Nouakchott
    {  482, 6690,  151774,  8, }, // Africa/Ouagadougou
    {  482, 5127,  151782, 10, }, // Africa/Porto-Novo
    {  482, 4337,  151792,  7, }, // Africa/Sao_Tome
    {  482, 2732,  151799,  8, }, // Africa/Tripoli
    {  482, 6503,  151807,  5, }, // Africa/Tunis
    {  482, 4054,  151812,  8, }, // Africa/Windhoek
    {  482,  132,  151820,  3, }, // America/Adak
    {  482, 3183,  151823,  6, }, // America/Anchorage
    {  482, 5524,  151829,  7, }, // America/Anguilla
    {  482, 5541,  151836,  7, }, // America/Antigua
    {  482, 4456,  151843,  8, }, // America/Araguaina
    {  482,  166,  151851, 10, }, // America/Argentina/Buenos_Aires
    {  482,   91,  151861,  9, }, // America/Argentina/Catamarca
    {  482,  270,  151870,  7, }, // America/Argentina/Cordoba
    {  482,  441,  151877,  5, }, // America/Argentina/Jujuy
    {  482, 7425,  151882,  8, }, // America/Argentina/La_Rioja
    {  482,  565,  151890,  8, }, // America/Argentina/Mendoza
    {  482, 7452,  151898, 11, }, // America/Argentina/Rio_Gallegos
    {  482, 7483,  151909,  5, }, // America/Argentina/Salta
    {  482, 7507,  151914,  8, }, // America/Argentina/San_Juan
    {  482, 5497,  151922,  8, }, // America/Argentina/San_Luis
    {  482, 7534,  151930,  6, }, // America/Argentina/Tucuman
    {  482, 7560,  151936,  7, }, // America/Argentina/Ushuaia
    {  482, 5557,  151943,  5, }, // America/Aruba
    {  482, 4154,  151948,  8, }, // America/Asuncion
    {  482,  237,  151956,  7, }, // America/Atikokan
    {  482, 3581,  151963,  5, }, // America/Bahia
    {  482, 7586,  151968, 13, }, // America/Bahia_Banderas
    {  482, 5571,  151981,  8, }, // America/Barbados
    {  482, 7609,  151989,  5, }, // America/Belem
    {  482, 5309,  151994,  6, }, // America/Belize
    {  482, 7623,  152000, 12, }, // America/Blanc-Sablon
    {  482, 7644,  152012,  9, }, // America/Boa_Vista
    {  482, 4263,  152021,  6, }, // America/Bogota
    {  482, 7662,  152027,  4, }, // America/Boise
    {  482, 7676,  152031, 11, }, // America/Cambridge_Bay
    {  482, 7698,  152042, 13, }, // America/Campo_Grande
    {  482, 3770,  152055,  6, }, // America/Cancun
    {  482, 4596,  152061,  7, }, // America/Caracas
    {  482, 4247,  152068,  6, }, // America/Cayenne
    {  482, 5398,  152074,  5, }, // America/Cayman
    {  482, 3251,  152079,  7, }, // America/Chicago
    {  482, 7719,  152086,  8, }, // America/Chihuahua
    {  482, 7737,  152094, 14, }, // America/Ciudad_Juarez
    {  482, 5324,  152108, 10, }, // America/Costa_Rica
    {  482, 7759,  152118,  7, }, // America/Creston
    {  482, 3693,  152125,  6, }, // America/Cuiaba
    {  482, 5640,  152131,  7, }, // America/Curacao
    {  482, 7775,  152138, 11, }, // America/Danmarkshavn
    {  482, 7796,  152149,  5, }, // America/Dawson
    {  482, 7811,  152154, 11, }, // America/Dawson_Creek
    {  482,  805,  152165,  5, }, // America/Denver
    {  482, 3351,  152170,  7, }, // America/Detroit
    {  482, 5656,  152177,  8, }, // America/Dominica
    {  482,  893,  152185,  7, }, // America/Edmonton
    {  482, 7832,  152192,  8, }, // America/Eirunepe
    {  482, 5343,  152200, 10, }, // America/El_Salvador
    {  482, 7849,  152210, 10, }, // America/Fort_Nelson
    {  482, 7869,  152220, 10, }, // America/Fortaleza
    {  482, 7887,  152230,  7, }, // America/Glace_Bay
    {  482, 6844,  152237,  7, }, // America/Goose_Bay
    {  482, 4514,  152244, 10, }, // America/Grand_Turk
    {  482, 5687,  152254,  8, }, // America/Grenada
    {  482, 5703,  152262,  8, }, // America/Guadeloupe
    {  482, 3662,  152270,  9, }, // America/Guatemala
    {  482, 4743,  152279,  8, }, // America/Guayaquil
    {  482, 6895,  152287,  5, }, // America/Guyana
    {  482, 1923,  152292,  8, }, // America/Halifax
    {  482, 2221,  152300,  5, }, // America/Havana
    {  482, 7905,  152305,  9, }, // America/Hermosillo
    {  482,  348,  152314, 12, }, // America/Indiana/Indianapolis
    {  482,  481,  152326, 13, }, // America/Indiana/Knox
    {  482, 7924,  152339, 16, }, // America/Indiana/Marengo
    {  482, 7948,  152355, 17, }, // America/Indiana/Petersburg
    {  482, 7975,  152372, 17, }, // America/Indiana/Tell_City
    {  482, 8001,  152389, 13, }, // America/Indiana/Vevay
    {  482, 8023,  152402, 17, }, // America/Indiana/Vincennes
    {  482, 8049,  152419, 15, }, // America/Indiana/Winamac
    {  482, 8073,  152434,  6, }, // America/Inuvik
    {  482,  660,  152440,  7, }, // America/Iqaluit
    {  482, 2665,  152447,  6, }, // America/Jamaica
    {  482, 5282,  152453,  5, }, // America/Juneau
    {  482,  521,  152458,  8, }, // America/Kentucky/Louisville
    {  482, 8088,  152466, 17, }, // America/Kentucky/Monticello
    {  482, 5621,  152483, 11, }, // America/Kralendijk
    {  482, 4278,  152494,  7, }, // America/La_Paz
    {  482, 7132,  152501,  4, }, // America/Lima
    {  482, 3390,  152505,  9, }, // America/Los_Angeles
    {  482, 5849,  152514, 20, }, // America/Lower_Princes
    {  482, 8116,  152534,  5, }, // America/Maceio
    {  482, 8131,  152539,  6, }, // America/Managua
    {  482, 1892,  152545,  4, }, // America/Manaus
    {  482, 5814,  152549,  7, }, // America/Marigot
    {  482, 5722,  152556,  8, }, // America/Martinique
    {  482, 8147,  152564,  8, }, // America/Matamoros
    {  482, 2779,  152572,  8, }, // America/Mazatlan
    {  482, 8165,  152580,  8, }, // America/Menominee
    {  482, 8183,  152588,  6, }, // America/Merida
    {  482, 8198,  152594, 10, }, // America/Metlakatla
    {  482, 2811,  152604, 12, }, // America/Mexico_City
    {  482, 4293,  152616,  6, }, // America/Miquelon
    {  482, 8217,  152622,  6, }, // America/Moncton
    {  482, 8233,  152628,  7, }, // America/Monterrey
    {  482, 4000,  152635, 10, }, // America/Montevideo
    {  482, 5741,  152645,  9, }, // America/Montserrat
    {  482, 5383,  152654,  5, }, // America/Nassau
    {  482, 3294,  152659,  8, }, // America/New_York
    {  482, 8251,  152582,  3, }, // America/Nome
    {  482, 1834,  152667,  7, }, // America/Noronha
    {  482, 8264,  152674, 18, }, // America/North_Dakota/Beulah
    {  482, 8292,  152692, 18, }, // America/North_Dakota/Center
    {  482, 8320,  152710, 21, }, // America/North_Dakota/New_Salem
    {  482,  393,  152731,  3, }, // America/Nuuk
    {  482, 8351,  152734,  7, }, // America/Ojinaga
    {  482, 5413,  152741,  5, }, // America/Panama
    {  482, 6106,  152746,  9, }, // America/Paramaribo
    {  482, 3224,  152755,  6, }, // America/Phoenix
    {  482, 3847,  152761, 13, }, // America/Port-au-Prince
    {  482, 5871,  152774, 12, }, // America/Port_of_Spain
    {  482, 8367,  152786, 12, }, // America/Porto_Velho
    {  482, 5760,  152798, 11, }, // America/Puerto_Rico
    {  482, 3932,  152809, 10, }, // America/Punta_Arenas
    {  482, 8387,  152819, 12, }, // America/Rankin_Inlet
    {  482, 8408,  152831,  6, }, // America/Recife
    {  482, 1979,  152837,  6, }, // America/Regina
    {  482, 8423,  152843,  9, }, // America/Resolute
    {  482,  695,  152852, 11, }, // America/Rio_Branco
    {  482, 8440,  152863,  7, }, // America/Santarem
    {  482, 2165,  152870,  8, }, // America/Santiago
    {  482, 6044,  152878, 13, }, // America/Santo_Domingo
    {  482, 1862,  152891,  8, }, // America/Sao_Paulo
    {  482, 6862,  152899, 13, }, // America/Scoresbysund
    {  482, 8457,  152912,  5, }, // America/Sitka
    {  482, 8471,  152917, 14, }, // America/St_Barthelemy
    {  482, 2045,  152931, 10, }, // America/St_Johns
    {  482, 5780,  152941,  9, }, // America/St_Kitts
    {  482, 5797,  152950, 10, }, // America/St_Lucia
    {  482,  855,  152960,  9, }, // America/St_Thomas
    {  482, 5830,  152969, 12, }, // America/St_Vincent
    {  482, 8493,  152981, 10, }, // America/Swift_Current
    {  482, 5363,  152991, 10, }, // America/Tegucigalpa
    {  482, 5673,  153001,  4, }, // America/Thule
    {  482,  313,  153005,  7, }, // America/Tijuana
    {  482,  608,  153012,  7, }, // America/Toronto
    {  482, 5605,  153019,  7, }, // America/Tortola
    {  482, 2077,  153026,  7, }, // America/Vancouver
    {  482, 2128,  153033,  8, }, // America/Whitehorse
    {  482,  734,  153041,  7, }, // America/Winnipeg
    {  482, 8515,  153048,  5, }, // America/Yakutat
    {  482, 5918,  152915,  4, }, // Antarctica/Casey
    {  482, 6027,  153053,  5, }, // Antarctica/Davis
    {  482, 6066,  153058, 16, }, // Antarctica/DumontDUrville
    {  482, 8531,  153074,  8, }, // Antarctica/Macquarie
    {  482, 7029,  153082,  4, }, // Antarctica/Mawson
    {  482, 7076,  153086,  8, }, // Antarctica/McMurdo
    {  482, 8552,  153094,  4, }, // Antarctica/Palmer
    {  482, 7189,  153098,  6, }, // Antarctica/Rothera
    {  482, 7258,  153104,  5, }, // Antarctica/Syowa
    {  482, 8570,  153109,  4, }, // Antarctica/Troll
    {  482, 7337,  153113,  6, }, // Antarctica/Vostok
    {  482, 1413,  153119, 11, }, // Arctic/Longyearbyen
    {  482, 5487,  153130,  4, }, // Asia/Aden
    {  482, 5297,  153134,  6, }, // Asia/Almaty
    {  482, 3870,  153140,  4, }, // Asia/Amman
    {  482, 5428,  153144,  7, }, // Asia/Anadyr
    {  482, 5440,  153151,  4, }, // Asia/Aqtau
    {  482, 5270,  153155,  6, }, // Asia/Aqtobe
    {  482,  964,  153161,  8, }, // Asia/Ashgabat
    {  482, 8587,  153169,  6, }, // Asia/Atyrau
    {  482, 3509,  153175,  5, }, // Asia/Baghdad
    {  482, 5451,  153180,  6, }, // Asia/Bahrain
    {  482, 3555,  153186,  4, }, // Asia/Baku
    {  482, 4368,  153190,  6, }, // Asia/Bangkok
    {  482, 3473,  153196,  5, }, // Asia/Barnaul
    {  482, 3988,  153201,  5, }, // Asia/Beirut
    {  482, 3680,  153206,  7, }, // Asia/Bishkek
    {  482, 5906,  153213,  6, }, // Asia/Brunei
    {  482, 4503,  153219,  4, }, // Asia/Chita
    {  482, 5963,  153223,  9, }, // Asia/Choibalsan
    {  482, 4413,  153232,  6, }, // Asia/Colombo
    {  482, 4442,  153238,  6, }, // Asia/Damascus
    {  482, 1060,  153244,  4, }, // Asia/Dhaka
    {  482, 6125,  153248,  4, }, // Asia/Dili
    {  482, 3498,  153252,  4, }, // Asia/Dubai
    {  482, 6092,  153256,  8, }, // Asia/Dushanbe
    {  482, 8599,  153264,  9, }, // Asia/Famagusta
    {  482, 8614,  153273,  5, }, // Asia/Gaza
    {  482, 4697,  153278,  6, }, // Asia/Hebron
    {  482, 1226,  153284, 16, }, // Asia/Ho_Chi_Minh
    {  482, 2576,  153300,  9, }, // Asia/Hong_Kong
    {  482, 4673,  153309,  4, }, // Asia/Hovd
    {  482, 4086,  153313,  7, }, // Asia/Irkutsk
    {  482, 6969,  153320,  6, }, // Asia/Jakarta
    {  482, 6955,  153326,  7, }, // Asia/Jayapura
    {  482, 1257,  153333,  7, }, // Asia/Jerusalem
    {  482, 3462,  153340,  5, }, // Asia/Kabul
    {  482, 4218,  153345,  6, }, // Asia/Kamchatka
    {  482, 4141,  153351,  5, }, // Asia/Karachi
    {  482, 1152,  153356,  8, }, // Asia/Kathmandu
    {  482, 8624,  153364,  7, }, // Asia/Khandyga
    {  482,  992,  153371,  7, }, // Asia/Kolkata
    {  482, 4099,  153378,  9, }, // Asia/Krasnoyarsk
    {  482, 4761,  153387, 10, }, // Asia/Kuala_Lumpur
    {  482, 5893,  153397,  6, }, // Asia/Kuching
    {  482, 5464,  153403,  5, }, // Asia/Kuwait
    {  482, 1178,  153408,  4, }, // Asia/Macau
    {  482, 3919,  153412,  6, }, // Asia/Magadan
    {  482, 1316,  153418,  5, }, // Asia/Makassar
    {  482, 7145,  153423,  5, }, // Asia/Manila
    {  482, 6883,  153428,  4, }, // Asia/Muscat
    {  482, 2446,  153432,  7, }, // Asia/Nicosia
    {  482, 8638,  153439, 13, }, // Asia/Novokuznetsk
    {  482, 4037,  153452, 11, }, // Asia/Novosibirsk
    {  482, 4131,  153463,  4, }, // Asia/Omsk
    {  482, 7108,  153467,  3, }, // Asia/Oral
    {  482, 6924,  153470,  9, }, // Asia/Phnom_Penh
    {  482, 8656,  153479,  9, }, // Asia/Pontianak
    {  482, 4116,  153488,  9, }, // Asia/Pyongyang
    {  482, 5476,  153497,  3, }, // Asia/Qatar
    {  482, 8671,  153500,  8, }, // Asia/Qostanay
    {  482, 4171,  153508, 10, }, // Asia/Qyzylorda
    {  482, 3486,  153518,  4, }, // Asia/Riyadh
    {  482, 4310,  153522,  5, }, // Asia/Sakhalin
    {  482, 7208,  153527,  6, }, // Asia/Samarkand
    {  482, 3116,  153533,  4, }, // Asia/Seoul
    {  482, 1020,  153537,  6, }, // Asia/Shanghai
    {  482, 3137,  153543,  8, }, // Asia/Singapore
    {  482, 4199,  153551, 12, }, // Asia/Srednekolymsk
    {  482, 3100,  153563,  4, }, // Asia/Taipei
    {  482, 4683,  153567,  7, }, // Asia/Tashkent
    {  482, 3817,  153574,  7, }, // Asia/Tbilisi
    {  482, 2638,  153581,  6, }, // Asia/Tehran
    {  482, 1284,  153587,  5, }, // Asia/Thimphu
    {  482, 2687,  153592,  5, }, // Asia/Tokyo
    {  482, 4474,  153597,  5, }, // Asia/Tomsk
    {  482, 1346,  153602,  8, }, // Asia/Ulaanbaatar
    {  482, 1126,  153610,  6, }, // Asia/Urumqi
    {  482, 8685,  153616,  8, }, // Asia/Ust-Nera
    {  482, 6940,  153624,  9, }, // Asia/Vientiane
    {  482, 4612,  153633, 10, }, // Asia/Vladivostok
    {  482, 4730,  153643,  6, }, // Asia/Yakutsk
    {  482, 1202,  153649,  5, }, // Asia/Yangon
    {  482, 3785,  153654, 10, }, // Asia/Yekaterinburg
    {  482, 3649,  153664,  6, }, // Asia/Yerevan
    {  482, 3565,  153670,  6, }, // Atlantic/Azores
    {  482, 5588,  153676,  6, }, // Atlantic/Bermuda
    {  482, 6591,  153682,  6, }, // Atlantic/Canary
    {  482, 3629,  153688,  7, }, // Atlantic/Cape_Verde
    {  482, 1379,  153695,  4, }, // Atlantic/Faroe
    {  482, 8699,  153699,  6, }, // Atlantic/Madeira
    {  482, 2614,  153705,  8, }, // Atlantic/Reykjavik
    {  482, 7235,  153713, 12, }, // Atlantic/South_Georgia
    {  482, 6784,  153725, 11, }, // Atlantic/St_Helena
    {  482, 6607,  153736,  6, }, // Atlantic/Stanley
    {  482, 1654,  153742,  6, }, // Australia/Adelaide
    {  482, 1619,  153748,  8, }, // Australia/Brisbane
    {  482, 1783,  153756, 10, }, // Australia/Broken_Hill
    {  482, 1567,  153766,  6, }, // Australia/Darwin
    {  482, 3539,  153772,  5, }, // Australia/Eucla
    {  482, 1500,  153777,  6, }, // Australia/Hobart
    {  482, 8716,  153783,  7, }, // Australia/Lindeman
    {  482, 1531,  153790,  9, }, // Australia/Lord_Howe
    {  482, 1711,  153799,  7, }, // Australia/Melbourne
    {  482, 1746,  153806,  3, }, // Australia/Perth
    {  482, 1447,  153809,  5, }, // Australia/Sydney
    {  482, 6390,  153814,  7, }, // Europe/Amsterdam
    {  482, 6149,  153821,  6, }, // Europe/Andorra
    {  482, 3522,  153827,  9, }, // Europe/Astrakhan
    {  482, 6577,  153836,  4, }, // Europe/Athens
    {  482, 6285,  153840,  8, }, // Europe/Belgrade
    {  482, 4659,  153848,  5, }, // Europe/Berlin
    {  482, 6437,  153853, 11, }, // Europe/Bratislava
    {  482, 6178,  153864,  6, }, // Europe/Brussels
    {  482, 3830,  153870,  8, }, // Europe/Bucharest
    {  482, 3708,  153878,  8, }, // Europe/Budapest
    {  482, 8751,  153886,  8, }, // Europe/Busingen
    {  482, 2475,  153894,  6, }, // Europe/Chisinau
    {  482, 6238,  153900,  8, }, // Europe/Copenhagen
    {  482, 2260,  153908,  5, }, // Europe/Dublin
    {  482, 6256,  153913,  8, }, // Europe/Gibraltar
    {  482, 8767,  153921,  5, }, // Europe/Guernsey
    {  482, 6561,  153926,  8, }, // Europe/Helsinki
    {  482, 8783,  153934, 10, }, // Europe/Isle_of_Man
    {  482, 1097,  153944,  8, }, // Europe/Istanbul
    {  482, 8802,  153952,  4, }, // Europe/Jersey
    {  482, 3881,  153956, 11, }, // Europe/Kaliningrad
    {  482, 8924,  153967,  5, }, // Europe/Kirov
    {  482, 2419,  153972,  3, }, // Europe/Kyiv
    {  482, 3078,  153975,  5, }, // Europe/Lisbon
    {  482, 6455,  153980,  9, }, // Europe/Ljubljana
    {  482, 2393,  153989,  4, }, // Europe/London
    {  482, 6314,  153993,  8, }, // Europe/Luxembourg
    {  482, 4794,  154001,  7, }, // Europe/Madrid
    {  482, 6346,  154008,  5, }, // Europe/Malta
    {  482, 6531,  154013, 10, }, // Europe/Mariehamn
    {  482, 3595,  154023,  5, }, // Europe/Minsk
    {  482, 6359,  154028,  6, }, // Europe/Monaco
    {  482, 3443,  154034,  5, }, // Europe/Moscow
    {  482, 6407,  154039,  4, }, // Europe/Oslo
    {  482, 4186,  154043,  5, }, // Europe/Paris
    {  482, 6373,  154048,  9, }, // Europe/Podgorica
    {  482, 6224,  154057,  5, }, // Europe/Prague
    {  482, 8816,  154062,  4, }, // Europe/Riga
    {  482, 6273,  153697,  3, }, // Europe/Rome
    {  482, 4233,  151496,  5, }, // Europe/Samara
    {  482, 6419,  154066,  9, }, // Europe/San_Marino
    {  482, 6194,  154075,  8, }, // Europe/Sarajevo
    {  482, 4353,  154083,  7, }, // Europe/Saratov
    {  482, 8828,  154090,  9, }, // Europe/Simferopol
    {  482, 6332,  154099,  6, }, // Europe/Skopje
    {  482, 6548,  154105,  5, }, // Europe/Sofia
    {  482, 6472,  154110,  7, }, // Europe/Stockholm
    {  482, 8846,  154117,  5, }, // Europe/Tallinn
    {  482, 6135,  154122,  6, }, // Europe/Tirane
    {  482, 8861,  154128,  9, }, // Europe/Ulyanovsk
    {  482, 6301,  154137,  6, }, // Europe/Vaduz
    {  482, 6516,  154143,  6, }, // Europe/Vatican
    {  482, 6164,  154149,  5, }, // Europe/Vienna
    {  482, 8878,  154154,  7, }, // Europe/Vilnius
    {  482, 4629,  154161, 10, }, // Europe/Volgograd
    {  482, 3055,  154171,  6, }, // Europe/Warsaw
    {  482, 6210,  154177,  6, }, // Europe/Zagreb
    {  482, 6489,  154183,  6, }, // Europe/Zurich
    {  482, 4980,  154189, 11, }, // Indian/Antananarivo
    {  482, 6910,  154200,  5, }, // Indian/Chagos
    {  482, 5979,  154205,  7, }, // Indian/Christmas
    {  482, 5996,  154212,  6, }, // Indian/Cocos
    {  482, 4931,  154218,  6, }, // Indian/Comoro
    {  482, 6624,  154224,  8, }, // Indian/Kerguelen
    {  482, 7223,  154232,  4, }, // Indian/Mahe
    {  482, 7013,  154236,  6, }, // Indian/Maldives
    {  482, 3971,  154242,  7, }, // Indian/Mauritius
    {  482, 5000,  154249,  6, }, // Indian/Mayotte
    {  482, 7174,  154255,  8, }, // Indian/Reunion
    {  482, 4324,  154263,  4, }, // Pacific/Apia
    {  482,  932,  154267,  6, }, // Pacific/Auckland
    {  482, 3608,  154273,  8, }, // Pacific/Bougainville
    {  482, 2863,  154281,  4, }, // Pacific/Chatham
    {  482, 3022,  154285,  3, }, // Pacific/Chuuk
    {  482, 2201,  154288,  4, }, // Pacific/Easter
    {  482, 7323,  154292,  4, }, // Pacific/Efate
    {  482, 7290,  154296,  6, }, // Pacific/Fakaofo
    {  482, 3804,  154302,  5, }, // Pacific/Fiji
    {  482, 7306,  154307,  8, }, // Pacific/Funafuti
    {  482, 6641,  154315,  7, }, // Pacific/Galapagos
    {  482, 6659,  154322,  7, }, // Pacific/Gambier
    {  482, 3724,  154329,  9, }, // Pacific/Guadalcanal
    {  482, 5935,  152542,  4, }, // Pacific/Guam
    {  482, 2929,  154338,  8, }, // Pacific/Honolulu
    {  482, 2897,  154346,  7, }, // Pacific/Kanton
    {  482, 3900,  154353,  9, }, // Pacific/Kiritimati
    {  482, 6982,  154362,  7, }, // Pacific/Kosrae
    {  482, 2708,  154369,  8, }, // Pacific/Kwajalein
    {  482, 4779,  154377,  6, }, // Pacific/Majuro
    {  482, 3953,  154383,  8, }, // Pacific/Marquesas
    {  482, 8909,  154391,  5, }, // Pacific/Midway
    {  482, 7047,  154396,  5, }, // Pacific/Nauru
    {  482, 7095,  154401,  4, }, // Pacific/Niue
    {  482, 4070,  154405,  6, }, // Pacific/Norfolk
    {  482, 7061,  154411,  5, }, // Pacific/Noumea
    {  482, 2991,  154416,  9, }, // Pacific/Pago_Pago
    {  482, 7118,  154425,  4, }, // Pacific/Palau
    {  482, 7157,  154429,  7, }, // Pacific/Pitcairn
    {  482, 2961,  154436,  7, }, // Pacific/Pohnpei
    {  482, 4709,  154443, 12, }, // Pacific/Port_Moresby
    {  482, 6009,  154455,  9, }, // Pacific/Rarotonga
    {  482, 5948,  154464,  5, }, // Pacific/Saipan
    {  482, 7275,  154469,  5, }, // Pacific/Tahiti
    {  482, 6675,  154474,  6, }, // Pacific/Tarawa
    {  482, 4485,  154480,  9, }, // Pacific/Tongatapu
    {  482, 7355,  154489,  3, }, // Pacific/Wake
    {  482, 7368,  154492,  5, }, // Pacific/Wallis
    {  483,   14,       0,  6, }, // Africa/Asmara Punjabi/Arabic/Pakistan
    {  483, 4337,       6,  8, }, // Africa/Sao_Tome
    {  483, 4154,      14,  8, }, // America/Asuncion
    {  483,  237,      22,  8, }, // America/Atikokan
    {  483, 7586,      30, 17, }, // America/Bahia_Banderas
    {  483, 3770,      47,  6, }, // America/Cancun
    {  483, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  483, 5640,      66,  7, }, // America/Curacao
    {  483,  481,      73, 13, }, // America/Indiana/Knox
    {  483, 7924,      86, 16, }, // America/Indiana/Marengo
    {  483, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  483, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  483, 8001,     139, 14, }, // America/Indiana/Vevay
    {  483, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  483, 8049,     171, 16, }, // America/Indiana/Winamac
    {  483, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  483, 5849,     207, 22, }, // America/Lower_Princes
    {  483, 8183,     229,  6, }, // America/Merida
    {  483, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  483, 8292,     255, 20, }, // America/North_Dakota/Center
    {  483, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  483,  393,     298,  4, }, // America/Nuuk
    {  483, 6862,     302, 16, }, // America/Scoresbysund
    {  483, 8471,     318, 14, }, // America/St_Barthelemy
    {  483, 2045,     332, 10, }, // America/St_Johns
    {  483, 5780,     342,  9, }, // America/St_Kitts
    {  483, 5797,     351,  9, }, // America/St_Lucia
    {  483,  855,     360, 10, }, // America/St_Thomas
    {  483, 5830,     370, 11, }, // America/St_Vincent
    {  483, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  483, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  483, 1152,     413,  9, }, // Asia/Kathmandu
    {  483,  992,     422,  7, }, // Asia/Kolkata
    {  483, 1178,     429,  5, }, // Asia/Macau
    {  483, 1202,     442,  6, }, // Asia/Yangon
    {  483, 1379,     448,  5, }, // Atlantic/Faroe
    {  483, 6784,     453, 10, }, // Atlantic/St_Helena
    {  483, 2419,     463,  4, }, // Europe/Kyiv
    {  483, 7174,     467,  7, }, // Indian/Reunion
    {  483, 3022,     474,  5, }, // Pacific/Chuuk
    {  483, 2961,     479,  7, }, // Pacific/Pohnpei
    {  484, 6751,  154497,  6, }, // Africa/Abidjan Quechua/Latin/Peru
    {  484, 6723,   72588,  4, }, // Africa/Accra
    {  484, 4961,   18754, 10, }, // Africa/Addis_Ababa
    {  484, 7383,  140180,  5, }, // Africa/Algiers
    {  484,   14,       0,  6, }, // Africa/Asmara
    {  484, 2242,  154503,  8, }, // Africa/Cairo
    {  484, 4945,  140194,  6, }, // Africa/Djibouti
    {  484, 5145,   19777,  5, }, // Africa/Douala
    {  484, 4381,  154511, 13, }, // Africa/Johannesburg
    {  484, 4426,  140200,  6, }, // Africa/Khartoum
    {  484, 5015,   42614, 10, }, // Africa/Mogadishu
    {  484, 4337,       6,  8, }, // Africa/Sao_Tome
    {  484, 6503,  154524,  5, }, // Africa/Tunis
    {  484, 5524,   18863,  7, }, // America/Anguilla
    {  484, 4154,      14,  8, }, // America/Asuncion
    {  484,  237,      22,  8, }, // America/Atikokan
    {  484, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  484, 5309,  140240,  6, }, // America/Belize
    {  484, 3770,    3861,  6, }, // America/Cancun
    {  484, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  484, 5640,  154529,  7, }, // America/Curacao
    {  484, 5687,   18910,  7, }, // America/Grenada
    {  484, 5703,   18917,  9, }, // America/Guadeloupe
    {  484, 2221,   18926,  9, }, // America/Havana
    {  484,  481,      73, 13, }, // America/Indiana/Knox
    {  484, 7924,      86, 16, }, // America/Indiana/Marengo
    {  484, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  484, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  484, 8001,     139, 14, }, // America/Indiana/Vevay
    {  484, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  484, 8049,     171, 16, }, // America/Indiana/Winamac
    {  484, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  484, 5849,     207, 22, }, // America/Lower_Princes
    {  484, 5722,   18945,  9, }, // America/Martinique
    {  484, 8183,    3899,  6, }, // America/Merida
    {  484, 2811,  154536, 16, }, // America/Mexico_City
    {  484, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  484, 8292,     255, 20, }, // America/North_Dakota/Center
    {  484, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  484,  393,     298,  4, }, // America/Nuuk
    {  484, 3847,  140389, 15, }, // America/Port-au-Prince
    {  484, 5871,  154552, 13, }, // America/Port_of_Spain
    {  484, 6862,     302, 16, }, // America/Scoresbysund
    {  484, 8471,  154565, 13, }, // America/St_Barthelemy
    {  484, 2045,  154578, 21, }, // America/St_Johns
    {  484, 5780,  154599, 13, }, // America/St_Kitts
    {  484, 5797,   93759, 11, }, // America/St_Lucia
    {  484,  855,  154612, 11, }, // America/St_Thomas
    {  484, 5830,   72864, 11, }, // America/St_Vincent
    {  484, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  484, 3870,   19121,  4, }, // Asia/Amman
    {  484, 5440,    5906,  5, }, // Asia/Aqtau
    {  484, 5270,    5911,  6, }, // Asia/Aqtobe
    {  484,  964,  140501,  7, }, // Asia/Ashgabat
    {  484, 3509,    4063,  6, }, // Asia/Baghdad
    {  484, 5451,  140508,  6, }, // Asia/Bahrain
    {  484, 4442,   72897,  7, }, // Asia/Damascus
    {  484, 6092,  140521,  7, }, // Asia/Dushanbe
    {  484, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  484, 6969,  154623,  7, }, // Asia/Jakarta
    {  484, 1152,   19221,  8, }, // Asia/Kathmandu
    {  484,  992,     422,  7, }, // Asia/Kolkata
    {  484, 1178,     429,  5, }, // Asia/Macau
    {  484, 1316,   92905,  7, }, // Asia/Makassar
    {  484, 6883,   19248,  7, }, // Asia/Muscat
    {  484, 4116,  154630,  8, }, // Asia/Pyongyang
    {  484, 5476,   71561,  5, }, // Asia/Qatar
    {  484, 8671,  154638,  8, }, // Asia/Qostanay
    {  484, 4171,   25347,  9, }, // Asia/Qyzylorda
    {  484, 3486,    6120,  4, }, // Asia/Riyadh
    {  484, 7208,   19277, 10, }, // Asia/Samarkand
    {  484, 3116,   19287,  4, }, // Asia/Seoul
    {  484, 1020,  154646,  7, }, // Asia/Shanghai
    {  484, 3137,   19291,  8, }, // Asia/Singapore
    {  484, 4683,   90276,  7, }, // Asia/Tashkent
    {  484, 3817,   76718,  6, }, // Asia/Tbilisi
    {  484, 2638,   56658,  7, }, // Asia/Tehran
    {  484, 2687,    4257,  5, }, // Asia/Tokyo
    {  484, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  484, 1202,   19328,  6, }, // Asia/Yangon
    {  484, 3785,   72916, 13, }, // Asia/Yekaterinburg
    {  484, 3649,  140578,  6, }, // Asia/Yerevan
    {  484, 6591,   72956,  8, }, // Atlantic/Canary
    {  484, 3629,   25418, 10, }, // Atlantic/Cape_Verde
    {  484, 1379,     448,  5, }, // Atlantic/Faroe
    {  484, 7235,  154653, 15, }, // Atlantic/South_Georgia
    {  484, 6784,  140611, 11, }, // Atlantic/St_Helena
    {  484, 1654,   19421,  8, }, // Australia/Adelaide
    {  484, 1447,   21149,  6, }, // Australia/Sydney
    {  484, 6178,  154668,  8, }, // Europe/Brussels
    {  484, 3830,   19464,  8, }, // Europe/Bucharest
    {  484, 6238,   19472, 10, }, // Europe/Copenhagen
    {  484, 8783,  140628, 11, }, // Europe/Isle_of_Man
    {  484, 1097,  140639,  8, }, // Europe/Istanbul
    {  484, 3881,   61551, 12, }, // Europe/Kaliningrad
    {  484, 2419,    6486,  4, }, // Europe/Kyiv
    {  484, 3078,   19528,  6, }, // Europe/Lisbon
    {  484, 6455,   19534,  9, }, // Europe/Ljubljana
    {  484, 2393,   19543,  7, }, // Europe/London
    {  484, 6314,   73033, 10, }, // Europe/Luxembourg
    {  484, 3443,   19566,  5, }, // Europe/Moscow
    {  484, 6224,   19576,  5, }, // Europe/Prague
    {  484, 6273,   19581,  4, }, // Europe/Rome
    {  484, 6472,   73074,  9, }, // Europe/Stockholm
    {  484, 6135,   19608,  6, }, // Europe/Tirane
    {  484, 6516,  154676, 11, }, // Europe/Vatican
    {  484, 6164,   19625,  5, }, // Europe/Vienna
    {  484, 4629,   61732, 10, }, // Europe/Volgograd
    {  484, 3055,   19639,  8, }, // Europe/Warsaw
    {  484, 7013,   73105,  8, }, // Indian/Maldives
    {  484, 7174,     467,  7, }, // Indian/Reunion
    {  484, 3022,     474,  5, }, // Pacific/Chuuk
    {  484, 2929,    4512,  8, }, // Pacific/Honolulu
    {  484, 2897,    4520,  9, }, // Pacific/Kanton
    {  484, 2961,     479,  7, }, // Pacific/Pohnpei
    {  487, 4961,    3793, 11, }, // Africa/Addis_Ababa Romanian/Latin/Romania
    {  487, 7383,   42592,  5, }, // Africa/Algiers
    {  487,   14,       0,  6, }, // Africa/Asmara
    {  487, 5015,   42614, 10, }, // Africa/Mogadishu
    {  487, 5173,   42632,  9, }, // Africa/Ndjamena
    {  487, 4337,   57267,  8, }, // Africa/Sao_Tome
    {  487, 4154,      14,  8, }, // America/Asuncion
    {  487,  237,      22,  8, }, // America/Atikokan
    {  487, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  487, 3770,    3861,  6, }, // America/Cancun
    {  487, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  487, 5640,      66,  7, }, // America/Curacao
    {  487, 5703,   92165,  9, }, // America/Guadeloupe
    {  487,  481,      73, 13, }, // America/Indiana/Knox
    {  487, 7924,      86, 16, }, // America/Indiana/Marengo
    {  487, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  487, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  487, 8001,     139, 14, }, // America/Indiana/Vevay
    {  487, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  487, 8049,     171, 16, }, // America/Indiana/Winamac
    {  487, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  487, 5849,     207, 22, }, // America/Lower_Princes
    {  487, 5722,   18945,  9, }, // America/Martinique
    {  487, 8183,    3899,  6, }, // America/Merida
    {  487, 2811,  154536, 16, }, // America/Mexico_City
    {  487, 8264,  154687, 22, }, // America/North_Dakota/Beulah
    {  487, 8292,  154709, 22, }, // America/North_Dakota/Center
    {  487, 8320,  154731, 25, }, // America/North_Dakota/New_Salem
    {  487,  393,     298,  4, }, // America/Nuuk
    {  487, 6862,     302, 16, }, // America/Scoresbysund
    {  487, 8471,   34416, 16, }, // America/St_Barthelemy
    {  487, 2045,     332, 10, }, // America/St_Johns
    {  487, 5780,     342,  9, }, // America/St_Kitts
    {  487, 5797,     351,  9, }, // America/St_Lucia
    {  487,  855,     360, 10, }, // America/St_Thomas
    {  487, 5830,     370, 11, }, // America/St_Vincent
    {  487, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  487, 7258,   63470,  5, }, // Antarctica/Syowa
    {  487, 5297,  113102,  6, }, // Asia/Almaty
    {  487, 5428,    5900,  6, }, // Asia/Anadyr
    {  487, 5440,    5906,  5, }, // Asia/Aqtau
    {  487, 5270,    5911,  6, }, // Asia/Aqtobe
    {  487,  964,  154756,  7, }, // Asia/Ashgabat
    {  487, 8587,  154763,  6, }, // Asia/Atyrau
    {  487, 3509,    4063,  6, }, // Asia/Baghdad
    {  487, 3680,  154769,  6, }, // Asia/Bishkek
    {  487, 4503,   91779,  4, }, // Asia/Chita
    {  487, 4442,   19148,  6, }, // Asia/Damascus
    {  487, 1060,   64123,  5, }, // Asia/Dhaka
    {  487, 6092,  154775,  7, }, // Asia/Dushanbe
    {  487, 1226,  154782,  9, }, // Asia/Ho_Chi_Minh
    {  487, 4086,  154791,  6, }, // Asia/Irkutsk
    {  487, 1257,  154797,  9, }, // Asia/Jerusalem
    {  487, 4218,  154806,  9, }, // Asia/Kamchatka
    {  487, 1152,     413,  9, }, // Asia/Kathmandu
    {  487,  992,   34533,  8, }, // Asia/Kolkata
    {  487, 4099,   25314, 11, }, // Asia/Krasnoyarsk
    {  487, 5464,  154815,  6, }, // Asia/Kuwait
    {  487, 1178,     429,  5, }, // Asia/Macau
    {  487, 8638,  154821, 11, }, // Asia/Novokuznetsk
    {  487, 7108,   56586,  6, }, // Asia/Oral
    {  487, 4116,  154832,  7, }, // Asia/Pyongyang
    {  487, 8671,     434,  8, }, // Asia/Qostanay
    {  487, 4171,   25347,  9, }, // Asia/Qyzylorda
    {  487, 3486,    6120,  4, }, // Asia/Riyadh
    {  487, 4310,   32047,  7, }, // Asia/Sakhalin
    {  487, 4199,    6150, 13, }, // Asia/Srednekolymsk
    {  487, 4683,  154839,  7, }, // Asia/Tashkent
    {  487, 2638,    6175,  7, }, // Asia/Tehran
    {  487, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  487, 4730,  154846,  6, }, // Asia/Yakutsk
    {  487, 1202,     442,  6, }, // Asia/Yangon
    {  487, 3785,    6219, 12, }, // Asia/Yekaterinburg
    {  487, 3649,   25397,  6, }, // Asia/Yerevan
    {  487, 3565,    6238,  5, }, // Atlantic/Azores
    {  487, 6591,  154852,  6, }, // Atlantic/Canary
    {  487, 3629,  154858, 11, }, // Atlantic/Cape_Verde
    {  487, 1379,   19383,  5, }, // Atlantic/Faroe
    {  487, 7235,  154869, 14, }, // Atlantic/South_Georgia
    {  487, 6784,  154883,  9, }, // Atlantic/St_Helena
    {  487, 3522,   32115,  8, }, // Europe/Astrakhan
    {  487, 6577,   25459,  5, }, // Europe/Athens
    {  487, 6285,   25465,  7, }, // Europe/Belgrade
    {  487, 6178,   56194,  9, }, // Europe/Brussels
    {  487, 3830,  139847,  9, }, // Europe/Bucharest
    {  487, 3708,  154892,  9, }, // Europe/Budapest
    {  487, 2475,   62164,  8, }, // Europe/Chisinau
    {  487, 6238,  151337,  9, }, // Europe/Copenhagen
    {  487, 8783,  154901, 10, }, // Europe/Isle_of_Man
    {  487, 2419,    6486,  4, }, // Europe/Kyiv
    {  487, 3078,  119826,  8, }, // Europe/Lisbon
    {  487, 2393,   93889,  6, }, // Europe/London
    {  487, 6314,    4414,  9, }, // Europe/Luxembourg
    {  487, 3443,   73049,  7, }, // Europe/Moscow
    {  487, 6373,  154911,  9, }, // Europe/Podgorica
    {  487, 6224,   19576,  5, }, // Europe/Prague
    {  487, 6273,   19581,  4, }, // Europe/Rome
    {  487, 6135,   19608,  6, }, // Europe/Tirane
    {  487, 8861,   25542,  9, }, // Europe/Ulyanovsk
    {  487, 6164,   19625,  5, }, // Europe/Vienna
    {  487, 3055,  154920,  8, }, // Europe/Warsaw
    {  487, 6489,    4478,  6, }, // Europe/Zurich
    {  487, 4931,    4489,  6, }, // Indian/Comoro
    {  487, 7013,    6662,  7, }, // Indian/Maldives
    {  487, 7174,     467,  7, }, // Indian/Reunion
    {  487, 3022,     474,  5, }, // Pacific/Chuuk
    {  487, 2201,  154928, 15, }, // Pacific/Easter
    {  487, 2929,    4512,  8, }, // Pacific/Honolulu
    {  487, 2897,    4520,  9, }, // Pacific/Kanton
    {  487, 3953,  154943,  8, }, // Pacific/Marquesas
    {  487, 7157,  154951, 15, }, // Pacific/Pitcairn
    {  487, 2961,     479,  7, }, // Pacific/Pohnpei
    {  489, 4961,    3793, 11, }, // Africa/Addis_Ababa Romansh/Latin/Switzerland
    {  489, 7383,   57088,  6, }, // Africa/Algiers
    {  489,   14,       0,  6, }, // Africa/Asmara
    {  489, 5032,   76549, 10, }, // Africa/Dar_es_Salaam
    {  489, 4945,   76559,  9, }, // Africa/Djibouti
    {  489, 5068,   62570,  8, }, // Africa/El_Aaiun
    {  489, 4426,   24937,  7, }, // Africa/Khartoum
    {  489, 6832,   18802,  4, }, // Africa/Lome
    {  489, 5015,   76568, 10, }, // Africa/Mogadishu
    {  489, 4337,       6,  8, }, // Africa/Sao_Tome
    {  489, 3183,  154966,  6, }, // America/Anchorage
    {  489, 5524,  154972, 10, }, // America/Anguilla
    {  489, 4456,   24992,  9, }, // America/Araguaina
    {  489,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  489,  441,   59968, 21, }, // America/Argentina/Jujuy
    {  489, 7507,  154982, 20, }, // America/Argentina/San_Juan
    {  489, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  489, 7560,   63060,  7, }, // America/Argentina/Ushuaia
    {  489, 5557,  155002, 10, }, // America/Aruba
    {  489, 4154,      14,  8, }, // America/Asuncion
    {  489,  237,      22,  8, }, // America/Atikokan
    {  489, 7586,      30, 17, }, // America/Bahia_Banderas
    {  489, 7609,   34376,  5, }, // America/Belem
    {  489, 5309,  155012,  8, }, // America/Belize
    {  489, 3770,      47,  6, }, // America/Cancun
    {  489, 5398,  155020, 13, }, // America/Cayman
    {  489, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  489, 3693,   25007,  6, }, // America/Cuiaba
    {  489, 5640,      66,  7, }, // America/Curacao
    {  489, 7832,   25013,  8, }, // America/Eirunepe
    {  489, 5343,    5090,  8, }, // America/El_Salvador
    {  489, 4514,  155033, 13, }, // America/Grand_Turk
    {  489, 5703,  155046, 11, }, // America/Guadeloupe
    {  489,  481,      73,  4, }, // America/Indiana/Knox
    {  489, 7924,      86,  7, }, // America/Indiana/Marengo
    {  489, 7948,     102, 10, }, // America/Indiana/Petersburg
    {  489, 7975,     121,  9, }, // America/Indiana/Tell_City
    {  489, 8001,     139,  5, }, // America/Indiana/Vevay
    {  489, 8023,     153,  9, }, // America/Indiana/Vincennes
    {  489, 8049,     171,  7, }, // America/Indiana/Winamac
    {  489, 2665,   93644,  8, }, // America/Jamaica
    {  489, 8088,     187, 10, }, // America/Kentucky/Monticello
    {  489, 5849,     207, 22, }, // America/Lower_Princes
    {  489, 2779,   25036,  8, }, // America/Mazatlan
    {  489, 8183,     229,  6, }, // America/Merida
    {  489, 2811,  155057, 15, }, // America/Mexico_City
    {  489, 4293,  155072, 12, }, // America/Miquelon
    {  489, 5741,  155084,  6, }, // America/Montserrat
    {  489, 1834,   60511, 19, }, // America/Noronha
    {  489, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  489, 8292,  155090, 22, }, // America/North_Dakota/Center
    {  489, 8320,  155112, 24, }, // America/North_Dakota/New_Salem
    {  489,  393,   34408,  7, }, // America/Nuuk
    {  489, 1862,   25145,  9, }, // America/Sao_Paulo
    {  489, 6862,     302, 16, }, // America/Scoresbysund
    {  489, 8471,     318, 14, }, // America/St_Barthelemy
    {  489, 2045,   25170, 12, }, // America/St_Johns
    {  489, 5780,   25182, 11, }, // America/St_Kitts
    {  489, 5797,   93759, 11, }, // America/St_Lucia
    {  489,  855,   42797, 12, }, // America/St_Thomas
    {  489, 5830,   25216, 13, }, // America/St_Vincent
    {  489, 5605,  155136,  9, }, // America/Tortola
    {  489, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  489, 7076,  155145,  9, }, // Antarctica/McMurdo
    {  489, 7258,   63470,  5, }, // Antarctica/Syowa
    {  489, 5270,   55725,  6, }, // Asia/Aqtobe
    {  489,  964,   42826,  7, }, // Asia/Ashgabat
    {  489, 3509,    4063,  6, }, // Asia/Baghdad
    {  489, 3680,   55740,  8, }, // Asia/Bishkek
    {  489, 5906,  155154, 19, }, // Asia/Brunei
    {  489, 5963,   76626, 12, }, // Asia/Choibalsan
    {  489, 6092,   55748,  9, }, // Asia/Dushanbe
    {  489, 1226,  155173, 20, }, // Asia/Ho_Chi_Minh
    {  489, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  489, 4218,   76660, 11, }, // Asia/Kamchatka
    {  489, 4141,   76671,  9, }, // Asia/Karachi
    {  489, 1152,     413,  9, }, // Asia/Kathmandu
    {  489,  992,     422,  7, }, // Asia/Kolkata
    {  489, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  489, 1178,     429,  5, }, // Asia/Macau
    {  489, 1316,   63630,  8, }, // Asia/Makassar
    {  489, 6883,   19248,  7, }, // Asia/Muscat
    {  489, 4037,   76692, 11, }, // Asia/Novosibirsk
    {  489, 3486,   19265,  5, }, // Asia/Riyadh
    {  489, 4310,   56617,  8, }, // Asia/Sakhalin
    {  489, 7208,   19277, 10, }, // Asia/Samarkand
    {  489, 3116,    6131,  4, }, // Asia/Seoul
    {  489, 3137,   19291,  8, }, // Asia/Singapore
    {  489, 4683,   55848,  9, }, // Asia/Tashkent
    {  489, 2638,    6175,  7, }, // Asia/Tehran
    {  489, 2687,    4257,  5, }, // Asia/Tokyo
    {  489, 1126,   25383,  6, }, // Asia/Urumqi
    {  489, 4730,    6206,  7, }, // Asia/Yakutsk
    {  489, 1202,     442,  6, }, // Asia/Yangon
    {  489, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  489, 3649,    6231,  7, }, // Asia/Yerevan
    {  489, 3565,  155193,  6, }, // Atlantic/Azores
    {  489, 5588,   72942,  8, }, // Atlantic/Bermuda
    {  489, 6591,  155199, 15, }, // Atlantic/Canary
    {  489, 3629,   42993,  8, }, // Atlantic/Cape_Verde
    {  489, 1379,  155214, 12, }, // Atlantic/Faroe
    {  489, 7235,  155226, 15, }, // Atlantic/South_Georgia
    {  489, 6784,  155241, 11, }, // Atlantic/St_Helena
    {  489, 6607,  155252, 12, }, // Atlantic/Stanley
    {  489, 6577,    4339,  5, }, // Europe/Athens
    {  489, 6178,   56194,  9, }, // Europe/Brussels
    {  489, 3830,   19464,  8, }, // Europe/Bucharest
    {  489, 8767,  155264, 16, }, // Europe/Guernsey
    {  489, 8783,  155280,  7, }, // Europe/Isle_of_Man
    {  489, 8802,  155287, 12, }, // Europe/Jersey
    {  489, 2419,     463,  4, }, // Europe/Kyiv
    {  489, 3078,    4400,  8, }, // Europe/Lisbon
    {  489, 2393,   93889,  6, }, // Europe/London
    {  489, 6314,    4414,  9, }, // Europe/Luxembourg
    {  489, 3443,  155299,  6, }, // Europe/Moscow
    {  489, 6273,   19581,  4, }, // Europe/Rome
    {  489, 6135,   19608,  6, }, // Europe/Tirane
    {  489, 3055,   19639,  8, }, // Europe/Warsaw
    {  489, 6489,  155305,  6, }, // Europe/Zurich
    {  489, 5979,  155311, 16, }, // Indian/Christmas
    {  489, 5996,  155327, 11, }, // Indian/Cocos
    {  489, 4931,  155338,  7, }, // Indian/Comoro
    {  489, 7013,   73105,  8, }, // Indian/Maldives
    {  489, 7174,     467,  7, }, // Indian/Reunion
    {  489, 3022,  155345,  4, }, // Pacific/Chuuk
    {  489, 2201,  155349, 14, }, // Pacific/Easter
    {  489, 3804,   76834,  7, }, // Pacific/Fiji
    {  489, 6641,   19672,  9, }, // Pacific/Galapagos
    {  489, 6659,  155363,  7, }, // Pacific/Gambier
    {  489, 3724,  155370,  7, }, // Pacific/Guadalcanal
    {  489, 5935,  155377,  7, }, // Pacific/Guam
    {  489, 6982,  155384,  5, }, // Pacific/Kosrae
    {  489, 7047,  155389,  5, }, // Pacific/Nauru
    {  489, 7095,  155394,  5, }, // Pacific/Niue
    {  489, 4070,  155399,  8, }, // Pacific/Norfolk
    {  489, 7061,    4529,  6, }, // Pacific/Noumea
    {  489, 7118,  155407,  7, }, // Pacific/Palau
    {  489, 2961,  155414,  7, }, // Pacific/Pohnpei
    {  489, 6675,  155421, 12, }, // Pacific/Tarawa
    {  489, 4485,  155433, 10, }, // Pacific/Tongatapu
    {  489, 7368,  155443,  8, }, // Pacific/Wallis
    {  490,   14,       0,  6, }, // Africa/Asmara Rombo/Latin/Tanzania
    {  490, 4337,       6,  8, }, // Africa/Sao_Tome
    {  490, 4154,      14,  8, }, // America/Asuncion
    {  490,  237,      22,  8, }, // America/Atikokan
    {  490, 7586,      30, 17, }, // America/Bahia_Banderas
    {  490, 3770,      47,  6, }, // America/Cancun
    {  490, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  490, 5640,      66,  7, }, // America/Curacao
    {  490,  481,      73, 13, }, // America/Indiana/Knox
    {  490, 7924,      86, 16, }, // America/Indiana/Marengo
    {  490, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  490, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  490, 8001,     139, 14, }, // America/Indiana/Vevay
    {  490, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  490, 8049,     171, 16, }, // America/Indiana/Winamac
    {  490, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  490, 5849,     207, 22, }, // America/Lower_Princes
    {  490, 8183,     229,  6, }, // America/Merida
    {  490, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  490, 8292,     255, 20, }, // America/North_Dakota/Center
    {  490, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  490,  393,     298,  4, }, // America/Nuuk
    {  490, 6862,     302, 16, }, // America/Scoresbysund
    {  490, 8471,     318, 14, }, // America/St_Barthelemy
    {  490, 2045,     332, 10, }, // America/St_Johns
    {  490, 5780,     342,  9, }, // America/St_Kitts
    {  490, 5797,     351,  9, }, // America/St_Lucia
    {  490,  855,     360, 10, }, // America/St_Thomas
    {  490, 5830,     370, 11, }, // America/St_Vincent
    {  490, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  490, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  490, 1152,     413,  9, }, // Asia/Kathmandu
    {  490,  992,     422,  7, }, // Asia/Kolkata
    {  490, 1178,     429,  5, }, // Asia/Macau
    {  490, 1202,     442,  6, }, // Asia/Yangon
    {  490, 1379,     448,  5, }, // Atlantic/Faroe
    {  490, 6784,     453, 10, }, // Atlantic/St_Helena
    {  490, 2419,     463,  4, }, // Europe/Kyiv
    {  490, 7174,     467,  7, }, // Indian/Reunion
    {  490, 3022,     474,  5, }, // Pacific/Chuuk
    {  490, 2961,     479,  7, }, // Pacific/Pohnpei
    {  491,   14,       0,  6, }, // Africa/Asmara Rundi/Latin/Burundi
    {  491, 4337,       6,  8, }, // Africa/Sao_Tome
    {  491, 4154,      14,  8, }, // America/Asuncion
    {  491,  237,      22,  8, }, // America/Atikokan
    {  491, 7586,      30, 17, }, // America/Bahia_Banderas
    {  491, 3770,      47,  6, }, // America/Cancun
    {  491, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  491, 5640,      66,  7, }, // America/Curacao
    {  491,  481,      73, 13, }, // America/Indiana/Knox
    {  491, 7924,      86, 16, }, // America/Indiana/Marengo
    {  491, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  491, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  491, 8001,     139, 14, }, // America/Indiana/Vevay
    {  491, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  491, 8049,     171, 16, }, // America/Indiana/Winamac
    {  491, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  491, 5849,     207, 22, }, // America/Lower_Princes
    {  491, 8183,     229,  6, }, // America/Merida
    {  491, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  491, 8292,     255, 20, }, // America/North_Dakota/Center
    {  491, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  491,  393,     298,  4, }, // America/Nuuk
    {  491, 6862,     302, 16, }, // America/Scoresbysund
    {  491, 8471,     318, 14, }, // America/St_Barthelemy
    {  491, 2045,     332, 10, }, // America/St_Johns
    {  491, 5780,     342,  9, }, // America/St_Kitts
    {  491, 5797,     351,  9, }, // America/St_Lucia
    {  491,  855,     360, 10, }, // America/St_Thomas
    {  491, 5830,     370, 11, }, // America/St_Vincent
    {  491, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  491, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  491, 1152,     413,  9, }, // Asia/Kathmandu
    {  491,  992,     422,  7, }, // Asia/Kolkata
    {  491, 1178,     429,  5, }, // Asia/Macau
    {  491, 1202,     442,  6, }, // Asia/Yangon
    {  491, 1379,     448,  5, }, // Atlantic/Faroe
    {  491, 6784,     453, 10, }, // Atlantic/St_Helena
    {  491, 2419,     463,  4, }, // Europe/Kyiv
    {  491, 7174,     467,  7, }, // Indian/Reunion
    {  491, 3022,     474,  5, }, // Pacific/Chuuk
    {  491, 2961,     479,  7, }, // Pacific/Pohnpei
    {  492, 6751,   34826,  7, }, // Africa/Abidjan Russian/Cyrillic/Russia
    {  492, 6723,     492,  5, }, // Africa/Accra
    {  492, 4961,     497, 11, }, // Africa/Addis_Ababa
    {  492, 7383,     508,  5, }, // Africa/Algiers
    {  492,   14,   54769,  6, }, // Africa/Asmara
    {  492,   44,     519,  6, }, // Africa/Bamako
    {  492, 5159,     525,  5, }, // Africa/Bangui
    {  492, 6709,     530,  6, }, // Africa/Banjul
    {  492, 7398,     536,  5, }, // Africa/Bissau
    {  492, 4859,   25670,  8, }, // Africa/Blantyre
    {  492, 5189,   50080, 10, }, // Africa/Brazzaville
    {  492, 4824,     558,  9, }, // Africa/Bujumbura
    {  492, 2242,     567,  4, }, // Africa/Cairo
    {  492, 4019,     571, 10, }, // Africa/Casablanca
    {  492, 7412,     581,  5, }, // Africa/Ceuta
    {  492, 6736,     586,  7, }, // Africa/Conakry
    {  492, 6803,     593,  5, }, // Africa/Dakar
    {  492, 5032,   25698, 12, }, // Africa/Dar_es_Salaam
    {  492, 4945,   34859,  7, }, // Africa/Djibouti
    {  492, 5145,     616,  5, }, // Africa/Douala
    {  492, 5068,   25717,  7, }, // Africa/El_Aaiun
    {  492, 6816,     628,  7, }, // Africa/Freetown
    {  492, 4808,     635,  8, }, // Africa/Gaborone
    {  492, 4917,   32384,  6, }, // Africa/Harare
    {  492, 4381,   50098, 12, }, // Africa/Johannesburg
    {  492, 4401,   25755,  5, }, // Africa/Juba
    {  492, 5053,     665,  7, }, // Africa/Kampala
    {  492, 4426,     672,  6, }, // Africa/Khartoum
    {  492, 4875,     678,  6, }, // Africa/Kigali
    {  492, 5208,     684,  7, }, // Africa/Kinshasa
    {  492, 4646,     691,  5, }, // Africa/Lagos
    {  492, 5238,   50110,  9, }, // Africa/Libreville
    {  492, 6832,     704,  4, }, // Africa/Lome
    {  492, 5113,     708,  6, }, // Africa/Luanda
    {  492, 4841,     714,  9, }, // Africa/Lubumbashi
    {  492, 4903,     723,  6, }, // Africa/Lusaka
    {  492, 5224,     729,  6, }, // Africa/Malabo
    {  492, 4889,     735,  6, }, // Africa/Maputo
    {  492, 5099,     741,  6, }, // Africa/Maseru
    {  492, 5084,     747,  7, }, // Africa/Mbabane
    {  492, 5015,     754,  8, }, // Africa/Mogadishu
    {  492, 6997,   34899,  8, }, // Africa/Monrovia
    {  492, 3755,   34907,  7, }, // Africa/Nairobi
    {  492, 5173,   25836,  8, }, // Africa/Ndjamena
    {  492, 5256,   34914,  6, }, // Africa/Niamey
    {  492, 6766,     790,  7, }, // Africa/Nouakchott
    {  492, 6690,     797,  8, }, // Africa/Ouagadougou
    {  492, 5127,     805, 10, }, // Africa/Porto-Novo
    {  492, 4337,     815,  8, }, // Africa/Sao_Tome
    {  492, 2732,     823,  7, }, // Africa/Tripoli
    {  492, 6503,     830,  5, }, // Africa/Tunis
    {  492, 4054,     835,  7, }, // Africa/Windhoek
    {  492,  132,     842,  4, }, // America/Adak
    {  492, 3183,   50119,  8, }, // America/Anchorage
    {  492, 5524,   50127,  7, }, // America/Anguilla
    {  492, 5541,     860,  7, }, // America/Antigua
    {  492, 4456,     867,  9, }, // America/Araguaina
    {  492,  166,   50134, 12, }, // America/Argentina/Buenos_Aires
    {  492,   91,     888,  9, }, // America/Argentina/Catamarca
    {  492,  270,     897,  7, }, // America/Argentina/Cordoba
    {  492,  441,   25937,  5, }, // America/Argentina/Jujuy
    {  492, 7425,     909,  8, }, // America/Argentina/La_Rioja
    {  492,  565,     917,  7, }, // America/Argentina/Mendoza
    {  492, 7452,   50146, 12, }, // America/Argentina/Rio_Gallegos
    {  492, 7483,   25962,  6, }, // America/Argentina/Salta
    {  492, 7507,     940,  8, }, // America/Argentina/San_Juan
    {  492, 5497,     948,  8, }, // America/Argentina/San_Luis
    {  492, 7534,     956,  7, }, // America/Argentina/Tucuman
    {  492, 7560,   25976,  5, }, // America/Argentina/Ushuaia
    {  492, 5557,     969,  5, }, // America/Aruba
    {  492, 4154,   50158,  8, }, // America/Asuncion
    {  492,  237,     981, 12, }, // America/Atikokan
    {  492, 3581,   34980,  4, }, // America/Bahia
    {  492, 7586,   50166, 16, }, // America/Bahia_Banderas
    {  492, 5571,    1013,  8, }, // America/Barbados
    {  492, 7609,    1021,  5, }, // America/Belem
    {  492, 5309,    1026,  5, }, // America/Belize
    {  492, 7623,    1031, 12, }, // America/Blanc-Sablon
    {  492, 7644,    1043,  9, }, // America/Boa_Vista
    {  492, 4263,    1052,  6, }, // America/Bogota
    {  492, 7662,   50182,  5, }, // America/Boise
    {  492, 7676,   50187, 13, }, // America/Cambridge_Bay
    {  492, 7698,    1075, 12, }, // America/Campo_Grande
    {  492, 3770,    1087,  6, }, // America/Cancun
    {  492, 4596,    1093,  7, }, // America/Caracas
    {  492, 4247,   50200,  7, }, // America/Cayenne
    {  492, 5398,  155451, 14, }, // America/Cayman
    {  492, 3251,    1129,  6, }, // America/Chicago
    {  492, 7719,    1135,  6, }, // America/Chihuahua
    {  492, 7737,  104869, 13, }, // America/Ciudad_Juarez
    {  492, 5324,    1141, 10, }, // America/Costa_Rica
    {  492, 7759,    1151,  7, }, // America/Creston
    {  492, 3693,   26138,  5, }, // America/Cuiaba
    {  492, 5640,   35086,  7, }, // America/Curacao
    {  492, 7775,    1172, 12, }, // America/Danmarkshavn
    {  492, 7796,    1184,  6, }, // America/Dawson
    {  492, 7811,    1190, 11, }, // America/Dawson_Creek
    {  492,  805,    1201,  6, }, // America/Denver
    {  492, 3351,   35129,  7, }, // America/Detroit
    {  492, 5656,    1214,  8, }, // America/Dominica
    {  492,  893,   50207,  8, }, // America/Edmonton
    {  492, 7832,   50215,  8, }, // America/Eirunepe
    {  492, 5343,   26215,  9, }, // America/El_Salvador
    {  492, 7849,   50223, 12, }, // America/Fort_Nelson
    {  492, 7869,    1257,  9, }, // America/Fortaleza
    {  492, 7887,   50235,  9, }, // America/Glace_Bay
    {  492, 6844,   50244,  7, }, // America/Goose_Bay
    {  492, 4514,    1282, 10, }, // America/Grand_Turk
    {  492, 5687,    1292,  7, }, // America/Grenada
    {  492, 5703,    1299,  9, }, // America/Guadeloupe
    {  492, 3662,    1308,  9, }, // America/Guatemala
    {  492, 4743,   50251,  8, }, // America/Guayaquil
    {  492, 6895,   50259,  6, }, // America/Guyana
    {  492, 1923,   50265,  8, }, // America/Halifax
    {  492, 2221,    1339,  6, }, // America/Havana
    {  492, 7905,   50273,  9, }, // America/Hermosillo
    {  492,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {  492,  481,    1365, 13, }, // America/Indiana/Knox
    {  492, 7924,    1378, 16, }, // America/Indiana/Marengo
    {  492, 7948,    1394, 19, }, // America/Indiana/Petersburg
    {  492, 7975,    1413,  9, }, // America/Indiana/Tell_City
    {  492, 8001,   54843, 14, }, // America/Indiana/Vevay
    {  492, 8023,    1436,  9, }, // America/Indiana/Vincennes
    {  492, 8049,    1445,  7, }, // America/Indiana/Winamac
    {  492, 8073,    1452,  6, }, // America/Inuvik
    {  492,  660,    1458,  7, }, // America/Iqaluit
    {  492, 2665,   26461,  6, }, // America/Jamaica
    {  492, 5282,   35226,  5, }, // America/Juneau
    {  492,  521,    1476,  8, }, // America/Kentucky/Louisville
    {  492, 8088,    1484, 20, }, // America/Kentucky/Monticello
    {  492, 5621,   35231, 10, }, // America/Kralendijk
    {  492, 4278,    1514,  6, }, // America/La_Paz
    {  492, 7132,    1520,  4, }, // America/Lima
    {  492, 3390,   50287, 12, }, // America/Los_Angeles
    {  492, 5849,   54891, 18, }, // America/Lower_Princes
    {  492, 8116,   35279,  6, }, // America/Maceio
    {  492, 8131,    1559,  7, }, // America/Managua
    {  492, 1892,    1566,  6, }, // America/Manaus
    {  492, 5814,    1572,  6, }, // America/Marigot
    {  492, 5722,    1578,  9, }, // America/Martinique
    {  492, 8147,    1587,  9, }, // America/Matamoros
    {  492, 2779,    1596,  8, }, // America/Mazatlan
    {  492, 8165,    1604,  8, }, // America/Menominee
    {  492, 8183,    1612,  6, }, // America/Merida
    {  492, 8198,    1618, 10, }, // America/Metlakatla
    {  492, 2811,    1628,  6, }, // America/Mexico_City
    {  492, 4293,    1634,  7, }, // America/Miquelon
    {  492, 8217,    1641,  7, }, // America/Moncton
    {  492, 8233,   50320,  9, }, // America/Monterrey
    {  492, 4000,    1657, 10, }, // America/Montevideo
    {  492, 5741,    1667, 10, }, // America/Montserrat
    {  492, 5383,    1677,  6, }, // America/Nassau
    {  492, 3294,   50338,  8, }, // America/New_York
    {  492, 8251,    1691,  3, }, // America/Nome
    {  492, 1834,   54909,  7, }, // America/Noronha
    {  492, 8264,  155465, 22, }, // America/North_Dakota/Beulah
    {  492, 8292,  155487, 22, }, // America/North_Dakota/Center
    {  492, 8320,  155509, 27, }, // America/North_Dakota/New_Salem
    {  492,  393,    1772,  4, }, // America/Nuuk
    {  492, 8351,    1776,  7, }, // America/Ojinaga
    {  492, 5413,    1783,  6, }, // America/Panama
    {  492, 6106,    1789, 10, }, // America/Paramaribo
    {  492, 3224,    1799,  6, }, // America/Phoenix
    {  492, 3847,    1805, 12, }, // America/Port-au-Prince
    {  492, 5871,   26771, 13, }, // America/Port_of_Spain
    {  492, 8367,   26784, 11, }, // America/Porto_Velho
    {  492, 5760,   50414, 11, }, // America/Puerto_Rico
    {  492, 3932,    1852, 12, }, // America/Punta_Arenas
    {  492, 8387,    1864, 12, }, // America/Rankin_Inlet
    {  492, 8408,    1876,  6, }, // America/Recife
    {  492, 1979,   50425,  8, }, // America/Regina
    {  492, 8423,   54981,  8, }, // America/Resolute
    {  492,  695,    1897, 10, }, // America/Rio_Branco
    {  492, 8440,    1907,  8, }, // America/Santarem
    {  492, 2165,   50433,  8, }, // America/Santiago
    {  492, 6044,    1923, 13, }, // America/Santo_Domingo
    {  492, 1862,    1936,  9, }, // America/Sao_Paulo
    {  492, 6862,    1945, 11, }, // America/Scoresbysund
    {  492, 8457,    1956,  5, }, // America/Sitka
    {  492, 8471,    1961, 13, }, // America/St_Barthelemy
    {  492, 2045,   26931, 10, }, // America/St_Johns
    {  492, 5780,    1983,  9, }, // America/St_Kitts
    {  492, 5797,   55008, 10, }, // America/St_Lucia
    {  492,  855,    2003, 10, }, // America/St_Thomas
    {  492, 5830,    2013, 12, }, // America/St_Vincent
    {  492, 8493,    2025, 13, }, // America/Swift_Current
    {  492, 5363,   50475, 12, }, // America/Tegucigalpa
    {  492, 5673,    2049,  4, }, // America/Thule
    {  492,  313,    2053,  7, }, // America/Tijuana
    {  492,  608,    2060,  7, }, // America/Toronto
    {  492, 5605,    2067,  7, }, // America/Tortola
    {  492, 2077,    2074,  8, }, // America/Vancouver
    {  492, 2128,   27017,  8, }, // America/Whitehorse
    {  492,  734,    2090,  8, }, // America/Winnipeg
    {  492, 8515,   27032,  6, }, // America/Yakutat
    {  492, 5918,   35501,  5, }, // Antarctica/Casey
    {  492, 6027,   35506,  6, }, // Antarctica/Davis
    {  492, 6066,   50487, 14, }, // Antarctica/DumontDUrville
    {  492, 8531,    2131,  8, }, // Antarctica/Macquarie
    {  492, 7029,    2139,  6, }, // Antarctica/Mawson
    {  492, 7076,    2145,  9, }, // Antarctica/McMurdo
    {  492, 8552,    2154,  6, }, // Antarctica/Palmer
    {  492, 7189,    2160,  6, }, // Antarctica/Rothera
    {  492, 7258,   27092,  4, }, // Antarctica/Syowa
    {  492, 8570,   50501,  6, }, // Antarctica/Troll
    {  492, 7337,   33428,  6, }, // Antarctica/Vostok
    {  492, 1413,   50507,  7, }, // Arctic/Longyearbyen
    {  492, 5487,    2192,  4, }, // Asia/Aden
    {  492, 5297,    2196,  6, }, // Asia/Almaty
    {  492, 3870,    2202,  5, }, // Asia/Amman
    {  492, 5428,   50528,  7, }, // Asia/Anadyr
    {  492, 5440,    2213,  5, }, // Asia/Aqtau
    {  492, 5270,    2218,  6, }, // Asia/Aqtobe
    {  492,  964,    2224,  7, }, // Asia/Ashgabat
    {  492, 8587,    2231,  6, }, // Asia/Atyrau
    {  492, 3509,    2237,  6, }, // Asia/Baghdad
    {  492, 5451,   35560,  7, }, // Asia/Bahrain
    {  492, 3555,    2250,  4, }, // Asia/Baku
    {  492, 4368,    2254,  7, }, // Asia/Bangkok
    {  492, 3473,    2261,  7, }, // Asia/Barnaul
    {  492, 3988,   27145,  6, }, // Asia/Beirut
    {  492, 3680,    2274,  6, }, // Asia/Bishkek
    {  492, 5906,   27157,  6, }, // Asia/Brunei
    {  492, 4503,    2286,  4, }, // Asia/Chita
    {  492, 5963,   35567,  9, }, // Asia/Choibalsan
    {  492, 4413,    2299,  7, }, // Asia/Colombo
    {  492, 4442,    2306,  6, }, // Asia/Damascus
    {  492, 1060,    2312,  5, }, // Asia/Dhaka
    {  492, 6125,    2317,  4, }, // Asia/Dili
    {  492, 3498,   27187,  5, }, // Asia/Dubai
    {  492, 6092,    2326,  7, }, // Asia/Dushanbe
    {  492, 8599,    2333,  9, }, // Asia/Famagusta
    {  492, 8614,    2342,  4, }, // Asia/Gaza
    {  492, 4697,    2346,  6, }, // Asia/Hebron
    {  492, 1226,    2352,  7, }, // Asia/Ho_Chi_Minh
    {  492, 2576,   50560,  7, }, // Asia/Hong_Kong
    {  492, 4673,    2366,  4, }, // Asia/Hovd
    {  492, 4086,    2370,  7, }, // Asia/Irkutsk
    {  492, 6969,   27229,  8, }, // Asia/Jakarta
    {  492, 6955,   27237,  8, }, // Asia/Jayapura
    {  492, 1257,    2392,  9, }, // Asia/Jerusalem
    {  492, 3462,    2401,  5, }, // Asia/Kabul
    {  492, 4218,  155536, 24, }, // Asia/Kamchatka
    {  492, 4141,    2431,  6, }, // Asia/Karachi
    {  492, 1152,    2437,  8, }, // Asia/Kathmandu
    {  492, 8624,    2445,  7, }, // Asia/Khandyga
    {  492,  992,   50608,  9, }, // Asia/Kolkata
    {  492, 4099,   35599, 10, }, // Asia/Krasnoyarsk
    {  492, 4761,    2471, 12, }, // Asia/Kuala_Lumpur
    {  492, 5893,    2483,  6, }, // Asia/Kuching
    {  492, 5464,   27284,  6, }, // Asia/Kuwait
    {  492, 1178,    2495,  5, }, // Asia/Macau
    {  492, 3919,    2500,  7, }, // Asia/Magadan
    {  492, 1316,    2507,  7, }, // Asia/Makassar
    {  492, 7145,    2514,  6, }, // Asia/Manila
    {  492, 6883,    2520,  6, }, // Asia/Muscat
    {  492, 2446,  105053,  7, }, // Asia/Nicosia
    {  492, 8638,    2533, 11, }, // Asia/Novokuznetsk
    {  492, 4037,    2544, 11, }, // Asia/Novosibirsk
    {  492, 4131,    2555,  4, }, // Asia/Omsk
    {  492, 7108,   27330,  7, }, // Asia/Oral
    {  492, 6924,   50617,  8, }, // Asia/Phnom_Penh
    {  492, 8656,    2572,  9, }, // Asia/Pontianak
    {  492, 4116,   27354,  7, }, // Asia/Pyongyang
    {  492, 5476,    2588,  5, }, // Asia/Qatar
    {  492, 8671,   35626,  8, }, // Asia/Qostanay
    {  492, 4171,    2601,  9, }, // Asia/Qyzylorda
    {  492, 3486,   50625,  7, }, // Asia/Riyadh
    {  492, 4310,  155560, 11, }, // Asia/Sakhalin
    {  492, 7208,    2630,  9, }, // Asia/Samarkand
    {  492, 3116,    2639,  4, }, // Asia/Seoul
    {  492, 1020,   27392,  6, }, // Asia/Shanghai
    {  492, 3137,    2649,  8, }, // Asia/Singapore
    {  492, 4199,    2657, 13, }, // Asia/Srednekolymsk
    {  492, 3100,   27420,  6, }, // Asia/Taipei
    {  492, 4683,    2676,  7, }, // Asia/Tashkent
    {  492, 3817,   33686,  7, }, // Asia/Tbilisi
    {  492, 2638,    2687,  7, }, // Asia/Tehran
    {  492, 1284,    2694,  7, }, // Asia/Thimphu
    {  492, 2687,    2701,  5, }, // Asia/Tokyo
    {  492, 4474,    2706,  5, }, // Asia/Tomsk
    {  492, 1346,    2711, 10, }, // Asia/Ulaanbaatar
    {  492, 1126,    2721,  6, }, // Asia/Urumqi
    {  492, 8685,   50651,  9, }, // Asia/Ust-Nera
    {  492, 6940,   50660,  8, }, // Asia/Vientiane
    {  492, 4612,    2742, 11, }, // Asia/Vladivostok
    {  492, 4730,   35672,  6, }, // Asia/Yakutsk
    {  492, 1202,   55061,  5, }, // Asia/Yangon
    {  492, 3785,    2766, 12, }, // Asia/Yekaterinburg
    {  492, 3649,    2778,  6, }, // Asia/Yerevan
    {  492, 3565,  155571, 13, }, // Atlantic/Azores
    {  492, 5588,  155584, 15, }, // Atlantic/Bermuda
    {  492, 6591,  155599, 14, }, // Atlantic/Canary
    {  492, 3629,    2835, 10, }, // Atlantic/Cape_Verde
    {  492, 1379,  155613, 14, }, // Atlantic/Faroe
    {  492, 8699,   35753,  7, }, // Atlantic/Madeira
    {  492, 2614,   50740,  9, }, // Atlantic/Reykjavik
    {  492, 7235,  155627, 13, }, // Atlantic/South_Georgia
    {  492, 6784,  155640, 16, }, // Atlantic/St_Helena
    {  492, 6607,   50782,  6, }, // Atlantic/Stanley
    {  492, 1654,    2926,  8, }, // Australia/Adelaide
    {  492, 1619,    2934,  7, }, // Australia/Brisbane
    {  492, 1783,    2941, 11, }, // Australia/Broken_Hill
    {  492, 1567,    2952,  6, }, // Australia/Darwin
    {  492, 3539,   27699,  4, }, // Australia/Eucla
    {  492, 1500,    2963,  6, }, // Australia/Hobart
    {  492, 8716,    2969,  8, }, // Australia/Lindeman
    {  492, 1531,    2977,  8, }, // Australia/Lord_Howe
    {  492, 1711,   27711,  8, }, // Australia/Melbourne
    {  492, 1746,    2992,  4, }, // Australia/Perth
    {  492, 1447,   50802,  6, }, // Australia/Sydney
    {  492, 6390,    3002,  9, }, // Europe/Amsterdam
    {  492, 6149,    3011,  7, }, // Europe/Andorra
    {  492, 3522,   27740,  9, }, // Europe/Astrakhan
    {  492, 6577,   59588,  5, }, // Europe/Athens
    {  492, 6285,    3030,  7, }, // Europe/Belgrade
    {  492, 4659,    3037,  6, }, // Europe/Berlin
    {  492, 6437,    3043, 10, }, // Europe/Bratislava
    {  492, 6178,   50822,  8, }, // Europe/Brussels
    {  492, 3830,    3061,  8, }, // Europe/Bucharest
    {  492, 3708,    3069,  8, }, // Europe/Budapest
    {  492, 8751,  155656, 25, }, // Europe/Busingen
    {  492, 2475,    3100,  7, }, // Europe/Chisinau
    {  492, 6238,   50849, 10, }, // Europe/Copenhagen
    {  492, 2260,    3117,  6, }, // Europe/Dublin
    {  492, 6256,    3123,  9, }, // Europe/Gibraltar
    {  492, 8767,    3132,  6, }, // Europe/Guernsey
    {  492, 6561,   50859,  9, }, // Europe/Helsinki
    {  492, 8783,  155681,  7, }, // Europe/Isle_of_Man
    {  492, 1097,    3155,  7, }, // Europe/Istanbul
    {  492, 8802,   50877,  6, }, // Europe/Jersey
    {  492, 3881,    3167, 11, }, // Europe/Kaliningrad
    {  492, 8924,    3178,  5, }, // Europe/Kirov
    {  492, 2419,    3183,  4, }, // Europe/Kyiv
    {  492, 3078,    3187,  8, }, // Europe/Lisbon
    {  492, 6455,   27891,  7, }, // Europe/Ljubljana
    {  492, 2393,    3204,  6, }, // Europe/London
    {  492, 6314,   27904, 10, }, // Europe/Luxembourg
    {  492, 4794,    3221,  6, }, // Europe/Madrid
    {  492, 6346,   27920,  6, }, // Europe/Malta
    {  492, 6531,    3232,  9, }, // Europe/Mariehamn
    {  492, 3595,    3241,  5, }, // Europe/Minsk
    {  492, 6359,    3246,  6, }, // Europe/Monaco
    {  492, 3443,    3252,  6, }, // Europe/Moscow
    {  492, 6407,    3258,  4, }, // Europe/Oslo
    {  492, 4186,    3262,  5, }, // Europe/Paris
    {  492, 6373,    3267,  9, }, // Europe/Podgorica
    {  492, 6224,    3276,  5, }, // Europe/Prague
    {  492, 8816,    3281,  4, }, // Europe/Riga
    {  492, 6273,    3285,  3, }, // Europe/Rome
    {  492, 4233,    3288,  6, }, // Europe/Samara
    {  492, 6419,    3294, 10, }, // Europe/San_Marino
    {  492, 6194,    3304,  7, }, // Europe/Sarajevo
    {  492, 4353,    3311,  7, }, // Europe/Saratov
    {  492, 8828,   50883, 11, }, // Europe/Simferopol
    {  492, 6332,   50894,  6, }, // Europe/Skopje
    {  492, 6548,   35907,  5, }, // Europe/Sofia
    {  492, 6472,   50900,  9, }, // Europe/Stockholm
    {  492, 8846,    3346,  6, }, // Europe/Tallinn
    {  492, 6135,    3352,  6, }, // Europe/Tirane
    {  492, 8861,   50909,  9, }, // Europe/Ulyanovsk
    {  492, 6301,    3367,  5, }, // Europe/Vaduz
    {  492, 6516,    3372,  7, }, // Europe/Vatican
    {  492, 6164,    3379,  4, }, // Europe/Vienna
    {  492, 8878,   50918,  7, }, // Europe/Vilnius
    {  492, 4629,    3390,  9, }, // Europe/Volgograd
    {  492, 3055,    3399,  7, }, // Europe/Warsaw
    {  492, 6210,    3406,  6, }, // Europe/Zagreb
    {  492, 6489,   35931,  5, }, // Europe/Zurich
    {  492, 4980,    3418, 12, }, // Indian/Antananarivo
    {  492, 6910,    3430,  5, }, // Indian/Chagos
    {  492, 5979,  155688, 13, }, // Indian/Christmas
    {  492, 5996,  155701, 14, }, // Indian/Cocos
    {  492, 4931,  155715,  6, }, // Indian/Comoro
    {  492, 6624,    3469,  8, }, // Indian/Kerguelen
    {  492, 7223,   28153,  3, }, // Indian/Mahe
    {  492, 7013,  155721,  8, }, // Indian/Maldives
    {  492, 3971,   55228,  8, }, // Indian/Mauritius
    {  492, 5000,   55236,  7, }, // Indian/Mayotte
    {  492, 7174,   51006,  7, }, // Indian/Reunion
    {  492, 4324,    3507,  4, }, // Pacific/Apia
    {  492,  932,    3511,  6, }, // Pacific/Auckland
    {  492, 3608,   51013,  9, }, // Pacific/Bougainville
    {  492, 2863,    3525,  5, }, // Pacific/Chatham
    {  492, 3022,    3530,  4, }, // Pacific/Chuuk
    {  492, 2201,  155729,  9, }, // Pacific/Easter
    {  492, 7323,   51067,  5, }, // Pacific/Efate
    {  492, 7290,    3550,  7, }, // Pacific/Fakaofo
    {  492, 3804,   36028,  5, }, // Pacific/Fiji
    {  492, 7306,    3561,  8, }, // Pacific/Funafuti
    {  492, 6641,  155738, 18, }, // Pacific/Galapagos
    {  492, 6659,  155756, 11, }, // Pacific/Gambier
    {  492, 3724,    3604, 11, }, // Pacific/Guadalcanal
    {  492, 5935,    3615,  4, }, // Pacific/Guam
    {  492, 2929,   51108,  8, }, // Pacific/Honolulu
    {  492, 2897,  155767, 13, }, // Pacific/Kanton
    {  492, 3900,    3625, 10, }, // Pacific/Kiritimati
    {  492, 6982,    3635,  6, }, // Pacific/Kosrae
    {  492, 2708,   28300, 10, }, // Pacific/Kwajalein
    {  492, 4779,   36063,  7, }, // Pacific/Majuro
    {  492, 3953,  155780, 15, }, // Pacific/Marquesas
    {  492, 8909,  155795, 11, }, // Pacific/Midway
    {  492, 7047,    3689,  5, }, // Pacific/Nauru
    {  492, 7095,   51162,  4, }, // Pacific/Niue
    {  492, 4070,    3698,  7, }, // Pacific/Norfolk
    {  492, 7061,    3705,  5, }, // Pacific/Noumea
    {  492, 2991,    3710,  9, }, // Pacific/Pago_Pago
    {  492, 7118,    3719,  5, }, // Pacific/Palau
    {  492, 7157,   55309,  7, }, // Pacific/Pitcairn
    {  492, 2961,    3731,  6, }, // Pacific/Pohnpei
    {  492, 4709,    3737, 11, }, // Pacific/Port_Moresby
    {  492, 6009,    3748,  9, }, // Pacific/Rarotonga
    {  492, 5948,   28394,  6, }, // Pacific/Saipan
    {  492, 7275,    3763,  5, }, // Pacific/Tahiti
    {  492, 6675,    3768,  6, }, // Pacific/Tarawa
    {  492, 4485,    3774,  9, }, // Pacific/Tongatapu
    {  492, 7355,   28414,  4, }, // Pacific/Wake
    {  492, 7368,    3787,  6, }, // Pacific/Wallis
    {  498,   14,       0,  6, }, // Africa/Asmara Rwa/Latin/Tanzania
    {  498, 4337,       6,  8, }, // Africa/Sao_Tome
    {  498, 4154,      14,  8, }, // America/Asuncion
    {  498,  237,      22,  8, }, // America/Atikokan
    {  498, 7586,      30, 17, }, // America/Bahia_Banderas
    {  498, 3770,      47,  6, }, // America/Cancun
    {  498, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  498, 5640,      66,  7, }, // America/Curacao
    {  498,  481,      73, 13, }, // America/Indiana/Knox
    {  498, 7924,      86, 16, }, // America/Indiana/Marengo
    {  498, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  498, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  498, 8001,     139, 14, }, // America/Indiana/Vevay
    {  498, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  498, 8049,     171, 16, }, // America/Indiana/Winamac
    {  498, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  498, 5849,     207, 22, }, // America/Lower_Princes
    {  498, 8183,     229,  6, }, // America/Merida
    {  498, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  498, 8292,     255, 20, }, // America/North_Dakota/Center
    {  498, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  498,  393,     298,  4, }, // America/Nuuk
    {  498, 6862,     302, 16, }, // America/Scoresbysund
    {  498, 8471,     318, 14, }, // America/St_Barthelemy
    {  498, 2045,     332, 10, }, // America/St_Johns
    {  498, 5780,     342,  9, }, // America/St_Kitts
    {  498, 5797,     351,  9, }, // America/St_Lucia
    {  498,  855,     360, 10, }, // America/St_Thomas
    {  498, 5830,     370, 11, }, // America/St_Vincent
    {  498, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  498, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  498, 1152,     413,  9, }, // Asia/Kathmandu
    {  498,  992,     422,  7, }, // Asia/Kolkata
    {  498, 1178,     429,  5, }, // Asia/Macau
    {  498, 1202,     442,  6, }, // Asia/Yangon
    {  498, 1379,     448,  5, }, // Atlantic/Faroe
    {  498, 6784,     453, 10, }, // Atlantic/St_Helena
    {  498, 2419,     463,  4, }, // Europe/Kyiv
    {  498, 7174,     467,  7, }, // Indian/Reunion
    {  498, 3022,     474,  5, }, // Pacific/Chuuk
    {  498, 2961,     479,  7, }, // Pacific/Pohnpei
    {  499,   14,       0,  6, }, // Africa/Asmara Saho/Latin/Eritrea
    {  499, 4337,       6,  8, }, // Africa/Sao_Tome
    {  499, 4154,      14,  8, }, // America/Asuncion
    {  499,  237,      22,  8, }, // America/Atikokan
    {  499, 7586,      30, 17, }, // America/Bahia_Banderas
    {  499, 3770,      47,  6, }, // America/Cancun
    {  499, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  499, 5640,      66,  7, }, // America/Curacao
    {  499,  481,      73, 13, }, // America/Indiana/Knox
    {  499, 7924,      86, 16, }, // America/Indiana/Marengo
    {  499, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  499, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  499, 8001,     139, 14, }, // America/Indiana/Vevay
    {  499, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  499, 8049,     171, 16, }, // America/Indiana/Winamac
    {  499, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  499, 5849,     207, 22, }, // America/Lower_Princes
    {  499, 8183,     229,  6, }, // America/Merida
    {  499, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  499, 8292,     255, 20, }, // America/North_Dakota/Center
    {  499, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  499,  393,     298,  4, }, // America/Nuuk
    {  499, 6862,     302, 16, }, // America/Scoresbysund
    {  499, 8471,     318, 14, }, // America/St_Barthelemy
    {  499, 2045,     332, 10, }, // America/St_Johns
    {  499, 5780,     342,  9, }, // America/St_Kitts
    {  499, 5797,     351,  9, }, // America/St_Lucia
    {  499,  855,     360, 10, }, // America/St_Thomas
    {  499, 5830,     370, 11, }, // America/St_Vincent
    {  499, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  499, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  499, 1152,     413,  9, }, // Asia/Kathmandu
    {  499,  992,     422,  7, }, // Asia/Kolkata
    {  499, 1178,     429,  5, }, // Asia/Macau
    {  499, 1202,     442,  6, }, // Asia/Yangon
    {  499, 1379,     448,  5, }, // Atlantic/Faroe
    {  499, 6784,     453, 10, }, // Atlantic/St_Helena
    {  499, 2419,     463,  4, }, // Europe/Kyiv
    {  499, 7174,     467,  7, }, // Indian/Reunion
    {  499, 3022,     474,  5, }, // Pacific/Chuuk
    {  499, 2961,     479,  7, }, // Pacific/Pohnpei
    {  500,   14,       0,  6, }, // Africa/Asmara Sakha/Cyrillic/Russia
    {  500, 4337,       6,  8, }, // Africa/Sao_Tome
    {  500, 4154,      14,  8, }, // America/Asuncion
    {  500,  237,      22,  8, }, // America/Atikokan
    {  500, 7586,      30, 17, }, // America/Bahia_Banderas
    {  500, 3770,      47,  6, }, // America/Cancun
    {  500, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  500, 5640,      66,  7, }, // America/Curacao
    {  500,  481,      73, 13, }, // America/Indiana/Knox
    {  500, 7924,      86, 16, }, // America/Indiana/Marengo
    {  500, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  500, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  500, 8001,     139, 14, }, // America/Indiana/Vevay
    {  500, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  500, 8049,     171, 16, }, // America/Indiana/Winamac
    {  500, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  500, 5849,     207, 22, }, // America/Lower_Princes
    {  500, 8183,     229,  6, }, // America/Merida
    {  500, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  500, 8292,     255, 20, }, // America/North_Dakota/Center
    {  500, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  500,  393,     298,  4, }, // America/Nuuk
    {  500, 6862,     302, 16, }, // America/Scoresbysund
    {  500, 8471,     318, 14, }, // America/St_Barthelemy
    {  500, 2045,     332, 10, }, // America/St_Johns
    {  500, 5780,     342,  9, }, // America/St_Kitts
    {  500, 5797,     351,  9, }, // America/St_Lucia
    {  500,  855,     360, 10, }, // America/St_Thomas
    {  500, 5830,     370, 11, }, // America/St_Vincent
    {  500, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  500, 5297,    2196,  6, }, // Asia/Almaty
    {  500, 5428,  155806,  7, }, // Asia/Anadyr
    {  500,  964,  155813,  8, }, // Asia/Ashgabat
    {  500, 3509,  155821,  7, }, // Asia/Baghdad
    {  500, 3555,  155828,  5, }, // Asia/Baku
    {  500, 3473,    2261,  7, }, // Asia/Barnaul
    {  500, 4503,  155833,  5, }, // Asia/Chita
    {  500, 5963,   35567,  9, }, // Asia/Choibalsan
    {  500, 4413,    2299,  7, }, // Asia/Colombo
    {  500, 4442,  155838,  6, }, // Asia/Damascus
    {  500, 3498,  155844,  6, }, // Asia/Dubai
    {  500, 1226,  155850, 10, }, // Asia/Ho_Chi_Minh
    {  500, 4086,  155860,  9, }, // Asia/Irkutsk
    {  500, 3462,  155869,  6, }, // Asia/Kabul
    {  500, 4218,  155875,  9, }, // Asia/Kamchatka
    {  500, 1152,     413,  9, }, // Asia/Kathmandu
    {  500, 8624,  155884,  8, }, // Asia/Khandyga
    {  500,  992,     422,  7, }, // Asia/Kolkata
    {  500, 4099,  155892, 12, }, // Asia/Krasnoyarsk
    {  500, 1178,     429,  5, }, // Asia/Macau
    {  500, 3919,  155904,  8, }, // Asia/Magadan
    {  500, 4037,  155912, 13, }, // Asia/Novosibirsk
    {  500, 4131,  155925,  6, }, // Asia/Omsk
    {  500, 4310,  155931,  8, }, // Asia/Sakhalin
    {  500, 7208,  155939,  9, }, // Asia/Samarkand
    {  500, 4199,  155948, 11, }, // Asia/Srednekolymsk
    {  500, 1346,  155959, 11, }, // Asia/Ulaanbaatar
    {  500, 1126,  155970,  6, }, // Asia/Urumqi
    {  500, 8685,  155976,  9, }, // Asia/Ust-Nera
    {  500, 4730,  155985, 10, }, // Asia/Yakutsk
    {  500, 1202,     442,  6, }, // Asia/Yangon
    {  500, 3785,  155995, 12, }, // Asia/Yekaterinburg
    {  500, 1379,     448,  5, }, // Atlantic/Faroe
    {  500, 6784,     453, 10, }, // Atlantic/St_Helena
    {  500, 3522,  156007,  9, }, // Europe/Astrakhan
    {  500, 1097,  156016,  8, }, // Europe/Istanbul
    {  500, 3881,    3167, 11, }, // Europe/Kaliningrad
    {  500, 2419,     463,  4, }, // Europe/Kyiv
    {  500, 3443,  156024,  7, }, // Europe/Moscow
    {  500, 4233,  156031,  7, }, // Europe/Samara
    {  500, 8828,   50883, 11, }, // Europe/Simferopol
    {  500, 8861,  156038, 11, }, // Europe/Ulyanovsk
    {  500, 7174,     467,  7, }, // Indian/Reunion
    {  500, 3022,     474,  5, }, // Pacific/Chuuk
    {  500, 2961,     479,  7, }, // Pacific/Pohnpei
    {  501,   14,       0,  6, }, // Africa/Asmara Samburu/Latin/Kenya
    {  501, 4337,       6,  8, }, // Africa/Sao_Tome
    {  501, 4154,      14,  8, }, // America/Asuncion
    {  501,  237,      22,  8, }, // America/Atikokan
    {  501, 7586,      30, 17, }, // America/Bahia_Banderas
    {  501, 3770,      47,  6, }, // America/Cancun
    {  501, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  501, 5640,      66,  7, }, // America/Curacao
    {  501,  481,      73, 13, }, // America/Indiana/Knox
    {  501, 7924,      86, 16, }, // America/Indiana/Marengo
    {  501, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  501, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  501, 8001,     139, 14, }, // America/Indiana/Vevay
    {  501, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  501, 8049,     171, 16, }, // America/Indiana/Winamac
    {  501, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  501, 5849,     207, 22, }, // America/Lower_Princes
    {  501, 8183,     229,  6, }, // America/Merida
    {  501, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  501, 8292,     255, 20, }, // America/North_Dakota/Center
    {  501, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  501,  393,     298,  4, }, // America/Nuuk
    {  501, 6862,     302, 16, }, // America/Scoresbysund
    {  501, 8471,     318, 14, }, // America/St_Barthelemy
    {  501, 2045,     332, 10, }, // America/St_Johns
    {  501, 5780,     342,  9, }, // America/St_Kitts
    {  501, 5797,     351,  9, }, // America/St_Lucia
    {  501,  855,     360, 10, }, // America/St_Thomas
    {  501, 5830,     370, 11, }, // America/St_Vincent
    {  501, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  501, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  501, 1152,     413,  9, }, // Asia/Kathmandu
    {  501,  992,     422,  7, }, // Asia/Kolkata
    {  501, 1178,     429,  5, }, // Asia/Macau
    {  501, 1202,     442,  6, }, // Asia/Yangon
    {  501, 1379,     448,  5, }, // Atlantic/Faroe
    {  501, 6784,     453, 10, }, // Atlantic/St_Helena
    {  501, 2419,     463,  4, }, // Europe/Kyiv
    {  501, 7174,     467,  7, }, // Indian/Reunion
    {  501, 3022,     474,  5, }, // Pacific/Chuuk
    {  501, 2961,     479,  7, }, // Pacific/Pohnpei
    {  502,   14,       0,  6, }, // Africa/Asmara Sango/Latin/Central African Republic
    {  502, 4337,       6,  8, }, // Africa/Sao_Tome
    {  502, 4154,      14,  8, }, // America/Asuncion
    {  502,  237,      22,  8, }, // America/Atikokan
    {  502, 7586,      30, 17, }, // America/Bahia_Banderas
    {  502, 3770,      47,  6, }, // America/Cancun
    {  502, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  502, 5640,      66,  7, }, // America/Curacao
    {  502,  481,      73, 13, }, // America/Indiana/Knox
    {  502, 7924,      86, 16, }, // America/Indiana/Marengo
    {  502, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  502, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  502, 8001,     139, 14, }, // America/Indiana/Vevay
    {  502, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  502, 8049,     171, 16, }, // America/Indiana/Winamac
    {  502, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  502, 5849,     207, 22, }, // America/Lower_Princes
    {  502, 8183,     229,  6, }, // America/Merida
    {  502, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  502, 8292,     255, 20, }, // America/North_Dakota/Center
    {  502, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  502,  393,     298,  4, }, // America/Nuuk
    {  502, 6862,     302, 16, }, // America/Scoresbysund
    {  502, 8471,     318, 14, }, // America/St_Barthelemy
    {  502, 2045,     332, 10, }, // America/St_Johns
    {  502, 5780,     342,  9, }, // America/St_Kitts
    {  502, 5797,     351,  9, }, // America/St_Lucia
    {  502,  855,     360, 10, }, // America/St_Thomas
    {  502, 5830,     370, 11, }, // America/St_Vincent
    {  502, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  502, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  502, 1152,     413,  9, }, // Asia/Kathmandu
    {  502,  992,     422,  7, }, // Asia/Kolkata
    {  502, 1178,     429,  5, }, // Asia/Macau
    {  502, 1202,     442,  6, }, // Asia/Yangon
    {  502, 1379,     448,  5, }, // Atlantic/Faroe
    {  502, 6784,     453, 10, }, // Atlantic/St_Helena
    {  502, 2419,     463,  4, }, // Europe/Kyiv
    {  502, 7174,     467,  7, }, // Indian/Reunion
    {  502, 3022,     474,  5, }, // Pacific/Chuuk
    {  502, 2961,     479,  7, }, // Pacific/Pohnpei
    {  503,   14,       0,  6, }, // Africa/Asmara Sangu/Latin/Tanzania
    {  503, 4337,       6,  8, }, // Africa/Sao_Tome
    {  503, 4154,      14,  8, }, // America/Asuncion
    {  503,  237,      22,  8, }, // America/Atikokan
    {  503, 7586,      30, 17, }, // America/Bahia_Banderas
    {  503, 3770,      47,  6, }, // America/Cancun
    {  503, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  503, 5640,      66,  7, }, // America/Curacao
    {  503,  481,      73, 13, }, // America/Indiana/Knox
    {  503, 7924,      86, 16, }, // America/Indiana/Marengo
    {  503, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  503, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  503, 8001,     139, 14, }, // America/Indiana/Vevay
    {  503, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  503, 8049,     171, 16, }, // America/Indiana/Winamac
    {  503, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  503, 5849,     207, 22, }, // America/Lower_Princes
    {  503, 8183,     229,  6, }, // America/Merida
    {  503, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  503, 8292,     255, 20, }, // America/North_Dakota/Center
    {  503, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  503,  393,     298,  4, }, // America/Nuuk
    {  503, 6862,     302, 16, }, // America/Scoresbysund
    {  503, 8471,     318, 14, }, // America/St_Barthelemy
    {  503, 2045,     332, 10, }, // America/St_Johns
    {  503, 5780,     342,  9, }, // America/St_Kitts
    {  503, 5797,     351,  9, }, // America/St_Lucia
    {  503,  855,     360, 10, }, // America/St_Thomas
    {  503, 5830,     370, 11, }, // America/St_Vincent
    {  503, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  503, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  503, 1152,     413,  9, }, // Asia/Kathmandu
    {  503,  992,     422,  7, }, // Asia/Kolkata
    {  503, 1178,     429,  5, }, // Asia/Macau
    {  503, 1202,     442,  6, }, // Asia/Yangon
    {  503, 1379,     448,  5, }, // Atlantic/Faroe
    {  503, 6784,     453, 10, }, // Atlantic/St_Helena
    {  503, 2419,     463,  4, }, // Europe/Kyiv
    {  503, 7174,     467,  7, }, // Indian/Reunion
    {  503, 3022,     474,  5, }, // Pacific/Chuuk
    {  503, 2961,     479,  7, }, // Pacific/Pohnpei
    {  504,   14,       0,  6, }, // Africa/Asmara Sanskrit/Devanagari/India
    {  504, 4337,       6,  8, }, // Africa/Sao_Tome
    {  504, 4154,      14,  8, }, // America/Asuncion
    {  504,  237,      22,  8, }, // America/Atikokan
    {  504, 7586,      30, 17, }, // America/Bahia_Banderas
    {  504, 3770,      47,  6, }, // America/Cancun
    {  504, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  504, 5640,      66,  7, }, // America/Curacao
    {  504,  481,      73, 13, }, // America/Indiana/Knox
    {  504, 7924,      86, 16, }, // America/Indiana/Marengo
    {  504, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  504, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  504, 8001,     139, 14, }, // America/Indiana/Vevay
    {  504, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  504, 8049,     171, 16, }, // America/Indiana/Winamac
    {  504, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  504, 5849,     207, 22, }, // America/Lower_Princes
    {  504, 8183,     229,  6, }, // America/Merida
    {  504, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  504, 8292,     255, 20, }, // America/North_Dakota/Center
    {  504, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  504,  393,     298,  4, }, // America/Nuuk
    {  504, 6862,     302, 16, }, // America/Scoresbysund
    {  504, 8471,     318, 14, }, // America/St_Barthelemy
    {  504, 2045,     332, 10, }, // America/St_Johns
    {  504, 5780,     342,  9, }, // America/St_Kitts
    {  504, 5797,     351,  9, }, // America/St_Lucia
    {  504,  855,     360, 10, }, // America/St_Thomas
    {  504, 5830,     370, 11, }, // America/St_Vincent
    {  504, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  504, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  504, 1152,     413,  9, }, // Asia/Kathmandu
    {  504,  992,     422,  7, }, // Asia/Kolkata
    {  504, 1178,     429,  5, }, // Asia/Macau
    {  504, 1202,     442,  6, }, // Asia/Yangon
    {  504, 1379,     448,  5, }, // Atlantic/Faroe
    {  504, 6784,     453, 10, }, // Atlantic/St_Helena
    {  504, 2419,     463,  4, }, // Europe/Kyiv
    {  504, 7174,     467,  7, }, // Indian/Reunion
    {  504, 3022,     474,  5, }, // Pacific/Chuuk
    {  504, 2961,     479,  7, }, // Pacific/Pohnpei
    {  505, 6751,  156049,  8, }, // Africa/Abidjan Santali/Ol Chiki/India
    {  505,   14,       0,  6, }, // Africa/Asmara
    {  505, 4337,       6,  8, }, // Africa/Sao_Tome
    {  505,  132,  156057,  4, }, // America/Adak
    {  505, 3183,  156061,  9, }, // America/Anchorage
    {  505, 5524,  156070,  7, }, // America/Anguilla
    {  505, 5541,  156077,  8, }, // America/Antigua
    {  505, 5557,  156085,  5, }, // America/Aruba
    {  505, 4154,      14,  8, }, // America/Asuncion
    {  505,  237,  156090,  8, }, // America/Atikokan
    {  505, 7586,  156098, 15, }, // America/Bahia_Banderas
    {  505, 5571,  156113,  8, }, // America/Barbados
    {  505, 5309,  156121,  6, }, // America/Belize
    {  505, 7623,  156127, 12, }, // America/Blanc-Sablon
    {  505, 7662,  156139,  5, }, // America/Boise
    {  505, 7676,  156144, 12, }, // America/Cambridge_Bay
    {  505, 3770,  156156,  6, }, // America/Cancun
    {  505, 5398,  156162,  5, }, // America/Cayman
    {  505, 3251,  156167,  6, }, // America/Chicago
    {  505, 7719,  156173, 11, }, // America/Chihuahua
    {  505, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  505, 5324,  156184, 10, }, // America/Costa_Rica
    {  505, 7759,  156194,  7, }, // America/Creston
    {  505, 5640,  156201,  8, }, // America/Curacao
    {  505, 7775,  156209, 11, }, // America/Danmarkshavn
    {  505, 7796,  156220,  6, }, // America/Dawson
    {  505, 7811,  156226, 11, }, // America/Dawson_Creek
    {  505,  805,  156237,  6, }, // America/Denver
    {  505, 3351,  156243,  8, }, // America/Detroit
    {  505, 5656,  156251,  8, }, // America/Dominica
    {  505,  893,  156259,  8, }, // America/Edmonton
    {  505, 5343,  156267, 11, }, // America/El_Salvador
    {  505, 7849,  156278, 11, }, // America/Fort_Nelson
    {  505, 7887,  156289,  7, }, // America/Glace_Bay
    {  505, 6844,  156296,  6, }, // America/Goose_Bay
    {  505, 4514,  156302, 11, }, // America/Grand_Turk
    {  505, 5687,  156313,  8, }, // America/Grenada
    {  505, 5703,  156321,  9, }, // America/Guadeloupe
    {  505, 3662,  156330, 10, }, // America/Guatemala
    {  505, 1923,  156340,  9, }, // America/Halifax
    {  505, 2221,  156349,  6, }, // America/Havana
    {  505, 7905,  156355, 10, }, // America/Hermosillo
    {  505,  348,  156365, 13, }, // America/Indiana/Indianapolis
    {  505,  481,  156378, 14, }, // America/Indiana/Knox
    {  505, 7924,  156392, 17, }, // America/Indiana/Marengo
    {  505, 7948,  156409, 21, }, // America/Indiana/Petersburg
    {  505, 7975,  156430, 18, }, // America/Indiana/Tell_City
    {  505, 8001,  156448, 17, }, // America/Indiana/Vevay
    {  505, 8023,  156465, 18, }, // America/Indiana/Vincennes
    {  505, 8049,  156483, 17, }, // America/Indiana/Winamac
    {  505, 8073,  156500,  6, }, // America/Inuvik
    {  505,  660,  156506,  8, }, // America/Iqaluit
    {  505, 2665,  156514,  7, }, // America/Jamaica
    {  505, 5282,  156521,  7, }, // America/Juneau
    {  505,  521,  156528,  9, }, // America/Kentucky/Louisville
    {  505, 8088,  156537, 18, }, // America/Kentucky/Monticello
    {  505, 5621,  156555, 12, }, // America/Kralendijk
    {  505, 3390,  156567, 11, }, // America/Los_Angeles
    {  505, 5849,  156578, 20, }, // America/Lower_Princes
    {  505, 8131,  156598,  8, }, // America/Managua
    {  505, 5814,  156606,  7, }, // America/Marigot
    {  505, 5722,  156613,  7, }, // America/Martinique
    {  505, 8147,  156620,  9, }, // America/Matamoros
    {  505, 2779,  156629,  8, }, // America/Mazatlan
    {  505, 8165,  156637,  8, }, // America/Menominee
    {  505, 8183,  156645,  6, }, // America/Merida
    {  505, 8198,  156651, 10, }, // America/Metlakatla
    {  505, 2811,  156661, 13, }, // America/Mexico_City
    {  505, 4293,  156674,  8, }, // America/Miquelon
    {  505, 8217,  156682,  7, }, // America/Moncton
    {  505, 8233,  156689,  8, }, // America/Monterrey
    {  505, 5741,  156697,  9, }, // America/Montserrat
    {  505, 5383,  156706,  5, }, // America/Nassau
    {  505, 3294,  156711,  8, }, // America/New_York
    {  505, 8251,  156719,  4, }, // America/Nome
    {  505, 8264,  156723, 19, }, // America/North_Dakota/Beulah
    {  505, 8292,  156742, 19, }, // America/North_Dakota/Center
    {  505, 8320,  156761, 22, }, // America/North_Dakota/New_Salem
    {  505,  393,  156783,  3, }, // America/Nuuk
    {  505, 8351,  156786,  7, }, // America/Ojinaga
    {  505, 5413,  156793,  6, }, // America/Panama
    {  505, 3224,  156799,  7, }, // America/Phoenix
    {  505, 3847,  156806, 13, }, // America/Port-au-Prince
    {  505, 5871,  156819, 13, }, // America/Port_of_Spain
    {  505, 5760,  156832, 11, }, // America/Puerto_Rico
    {  505, 8387,  156843, 12, }, // America/Rankin_Inlet
    {  505, 1979,  156855,  8, }, // America/Regina
    {  505, 8423,  156863,  8, }, // America/Resolute
    {  505, 6044,  156871, 13, }, // America/Santo_Domingo
    {  505, 6862,  156884, 11, }, // America/Scoresbysund
    {  505, 8457,  156895,  5, }, // America/Sitka
    {  505, 8471,  156900, 15, }, // America/St_Barthelemy
    {  505, 2045,  156915, 10, }, // America/St_Johns
    {  505, 5780,  156925, 10, }, // America/St_Kitts
    {  505, 5797,  156935, 12, }, // America/St_Lucia
    {  505,  855,  156947, 12, }, // America/St_Thomas
    {  505, 5830,  156959, 12, }, // America/St_Vincent
    {  505, 8493,  156971, 13, }, // America/Swift_Current
    {  505, 5363,  156984, 11, }, // America/Tegucigalpa
    {  505, 5673,  156995,  5, }, // America/Thule
    {  505,  313,  157000,  8, }, // America/Tijuana
    {  505,  608,  157008,  7, }, // America/Toronto
    {  505, 5605,  157015,  7, }, // America/Tortola
    {  505, 2077,  157022,  9, }, // America/Vancouver
    {  505, 2128,  157031,  9, }, // America/Whitehorse
    {  505,  734,  157040,  8, }, // America/Winnipeg
    {  505, 8515,  157048,  7, }, // America/Yakutat
    {  505, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  505, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  505, 1152,     413,  9, }, // Asia/Kathmandu
    {  505,  992,     422,  7, }, // Asia/Kolkata
    {  505, 1178,     429,  5, }, // Asia/Macau
    {  505, 1202,     442,  6, }, // Asia/Yangon
    {  505, 5588,  157055,  9, }, // Atlantic/Bermuda
    {  505, 1379,     448,  5, }, // Atlantic/Faroe
    {  505, 6784,     453, 10, }, // Atlantic/St_Helena
    {  505, 2419,     463,  4, }, // Europe/Kyiv
    {  505, 7174,     467,  7, }, // Indian/Reunion
    {  505, 3022,     474,  5, }, // Pacific/Chuuk
    {  505, 2961,     479,  7, }, // Pacific/Pohnpei
    {  506,  237,      22,  8, }, // America/Atikokan Santali/Devanagari/India
    {  506, 7586,      30, 17, }, // America/Bahia_Banderas
    {  506, 3770,      47,  6, }, // America/Cancun
    {  506, 5640,      66,  7, }, // America/Curacao
    {  506,  481,      73, 13, }, // America/Indiana/Knox
    {  506, 7924,      86, 16, }, // America/Indiana/Marengo
    {  506, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  506, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  506, 8001,     139, 14, }, // America/Indiana/Vevay
    {  506, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  506, 8049,     171, 16, }, // America/Indiana/Winamac
    {  506, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  506, 5849,     207, 22, }, // America/Lower_Princes
    {  506, 8183,     229,  6, }, // America/Merida
    {  506, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  506, 8292,     255, 20, }, // America/North_Dakota/Center
    {  506, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  506,  393,     298,  4, }, // America/Nuuk
    {  506, 6862,     302, 16, }, // America/Scoresbysund
    {  506, 8471,     318, 14, }, // America/St_Barthelemy
    {  506, 2045,     332, 10, }, // America/St_Johns
    {  506, 5780,     342,  9, }, // America/St_Kitts
    {  506, 5797,     351,  9, }, // America/St_Lucia
    {  506,  855,     360, 10, }, // America/St_Thomas
    {  506, 5830,     370, 11, }, // America/St_Vincent
    {  507, 4961,    3793, 11, }, // Africa/Addis_Ababa Sardinian/Latin/Italy
    {  507, 7383,   93583,  6, }, // Africa/Algiers
    {  507,   14,       0,  6, }, // Africa/Asmara
    {  507, 2242,  157064,  8, }, // Africa/Cairo
    {  507, 5015,  157072, 10, }, // Africa/Mogadishu
    {  507, 6997,   42624,  8, }, // Africa/Monrovia
    {  507, 5127,  157082,  9, }, // Africa/Porto-Novo
    {  507, 4337,       6,  8, }, // Africa/Sao_Tome
    {  507, 2732,  157091,  7, }, // Africa/Tripoli
    {  507, 6503,  157098,  6, }, // Africa/Tunis
    {  507, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  507, 4154,      14,  8, }, // America/Asuncion
    {  507,  237,      22,  8, }, // America/Atikokan
    {  507, 7586,      30, 17, }, // America/Bahia_Banderas
    {  507, 3770,      47,  6, }, // America/Cancun
    {  507, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  507, 5640,      66,  7, }, // America/Curacao
    {  507, 2221,  157104,  7, }, // America/Havana
    {  507,  481,      73, 13, }, // America/Indiana/Knox
    {  507, 7924,      86, 16, }, // America/Indiana/Marengo
    {  507, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  507, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  507, 8001,     139, 14, }, // America/Indiana/Vevay
    {  507, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  507, 8049,     171, 16, }, // America/Indiana/Winamac
    {  507, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  507, 5849,     207, 22, }, // America/Lower_Princes
    {  507, 5722,   18945,  9, }, // America/Martinique
    {  507, 8183,     229,  6, }, // America/Merida
    {  507, 2811,  157111, 21, }, // America/Mexico_City
    {  507, 3294,  157132,  8, }, // America/New_York
    {  507, 8264,  157140, 25, }, // America/North_Dakota/Beulah
    {  507, 8292,  157165, 25, }, // America/North_Dakota/Center
    {  507, 8320,  157190, 28, }, // America/North_Dakota/New_Salem
    {  507,  393,     298,  4, }, // America/Nuuk
    {  507, 5413,  157218,  6, }, // America/Panama
    {  507, 5871,  157224, 16, }, // America/Port_of_Spain
    {  507, 6862,     302, 16, }, // America/Scoresbysund
    {  507, 8471,  157240, 16, }, // America/St_Barthelemy
    {  507, 2045,  157256, 13, }, // America/St_Johns
    {  507, 5780,  157269, 14, }, // America/St_Kitts
    {  507, 5797,  157283, 12, }, // America/St_Lucia
    {  507,  855,  157295, 13, }, // America/St_Thomas
    {  507, 5830,  157308, 14, }, // America/St_Vincent
    {  507, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  507, 5440,    5906,  5, }, // Asia/Aqtau
    {  507, 5270,    5911,  6, }, // Asia/Aqtobe
    {  507, 5451,    4069,  7, }, // Asia/Bahrain
    {  507, 4503,   56142,  4, }, // Asia/Chita
    {  507, 4442,   19148,  7, }, // Asia/Damascus
    {  507, 1060,  151106,  4, }, // Asia/Dhaka
    {  507, 1226,  157322, 22, }, // Asia/Ho_Chi_Minh
    {  507, 6969,   93786,  8, }, // Asia/Jakarta
    {  507, 1257,   93794, 11, }, // Asia/Jerusalem
    {  507, 1152,     413,  9, }, // Asia/Kathmandu
    {  507,  992,   19229,  7, }, // Asia/Kolkata
    {  507, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  507, 1178,     429,  5, }, // Asia/Macau
    {  507, 6883,   19248,  7, }, // Asia/Muscat
    {  507, 5476,  157344,  5, }, // Asia/Qatar
    {  507, 4171,   25347,  9, }, // Asia/Qyzylorda
    {  507, 3486,  157349,  5, }, // Asia/Riyadh
    {  507, 7208,   19277, 10, }, // Asia/Samarkand
    {  507, 3116,  157354,  4, }, // Asia/Seoul
    {  507, 2638,  157358,  7, }, // Asia/Tehran
    {  507, 1346,  157365, 10, }, // Asia/Ulaanbaatar
    {  507, 1202,     442,  6, }, // Asia/Yangon
    {  507, 3565,  157375,  7, }, // Atlantic/Azores
    {  507, 6591,  157382, 15, }, // Atlantic/Canary
    {  507, 3629,  157397, 10, }, // Atlantic/Cape_Verde
    {  507, 1379,  157407, 14, }, // Atlantic/Faroe
    {  507, 7235,  157421, 17, }, // Atlantic/South_Georgia
    {  507, 6784,  157438, 10, }, // Atlantic/St_Helena
    {  507, 6577,   19437,  5, }, // Europe/Athens
    {  507, 6285,  157448,  8, }, // Europe/Belgrade
    {  507, 4659,  157456,  7, }, // Europe/Berlin
    {  507, 6178,   56194,  9, }, // Europe/Brussels
    {  507, 3830,  157463,  8, }, // Europe/Bucharest
    {  507, 3708,  157471,  8, }, // Europe/Budapest
    {  507, 8751,   25487,  8, }, // Europe/Busingen
    {  507, 2260,  157479,  7, }, // Europe/Dublin
    {  507, 6256,   93860, 10, }, // Europe/Gibraltar
    {  507, 8783,  157486, 12, }, // Europe/Isle_of_Man
    {  507, 1097,  157498,  8, }, // Europe/Istanbul
    {  507, 2419,     463,  4, }, // Europe/Kyiv
    {  507, 3078,   91939,  7, }, // Europe/Lisbon
    {  507, 2393,   93889,  6, }, // Europe/London
    {  507, 6314,  130227, 11, }, // Europe/Luxembourg
    {  507, 6359,  157506,  6, }, // Europe/Monaco
    {  507, 3443,   93906,  5, }, // Europe/Moscow
    {  507, 4186,   93911,  6, }, // Europe/Paris
    {  507, 6224,   19576,  5, }, // Europe/Prague
    {  507, 6273,   19581,  4, }, // Europe/Rome
    {  507, 6419,  157512, 12, }, // Europe/San_Marino
    {  507, 6472,  157524,  9, }, // Europe/Stockholm
    {  507, 6135,   19608,  6, }, // Europe/Tirane
    {  507, 6516,  157533, 22, }, // Europe/Vatican
    {  507, 3055,  157555,  8, }, // Europe/Warsaw
    {  507, 6210,  157563,  8, }, // Europe/Zagreb
    {  507, 6489,  157571,  6, }, // Europe/Zurich
    {  507, 5979,  157577, 10, }, // Indian/Christmas
    {  507, 7013,   73105,  8, }, // Indian/Maldives
    {  507, 3971,  157587, 10, }, // Indian/Mauritius
    {  507, 5000,  157597,  6, }, // Indian/Mayotte
    {  507, 7174,  157603,  8, }, // Indian/Reunion
    {  507, 3022,     474,  5, }, // Pacific/Chuuk
    {  507, 5935,  157611,  4, }, // Pacific/Guam
    {  507, 2897,    4520,  9, }, // Pacific/Kanton
    {  507, 3953,  157615,  9, }, // Pacific/Marquesas
    {  507, 2961,     479,  7, }, // Pacific/Pohnpei
    {  508,   14,       0,  6, }, // Africa/Asmara Sena/Latin/Mozambique
    {  508, 4337,       6,  8, }, // Africa/Sao_Tome
    {  508, 4154,      14,  8, }, // America/Asuncion
    {  508,  237,      22,  8, }, // America/Atikokan
    {  508, 7586,      30, 17, }, // America/Bahia_Banderas
    {  508, 3770,      47,  6, }, // America/Cancun
    {  508, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  508, 5640,      66,  7, }, // America/Curacao
    {  508,  481,      73, 13, }, // America/Indiana/Knox
    {  508, 7924,      86, 16, }, // America/Indiana/Marengo
    {  508, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  508, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  508, 8001,     139, 14, }, // America/Indiana/Vevay
    {  508, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  508, 8049,     171, 16, }, // America/Indiana/Winamac
    {  508, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  508, 5849,     207, 22, }, // America/Lower_Princes
    {  508, 8183,     229,  6, }, // America/Merida
    {  508, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  508, 8292,     255, 20, }, // America/North_Dakota/Center
    {  508, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  508,  393,     298,  4, }, // America/Nuuk
    {  508, 6862,     302, 16, }, // America/Scoresbysund
    {  508, 8471,     318, 14, }, // America/St_Barthelemy
    {  508, 2045,     332, 10, }, // America/St_Johns
    {  508, 5780,     342,  9, }, // America/St_Kitts
    {  508, 5797,     351,  9, }, // America/St_Lucia
    {  508,  855,     360, 10, }, // America/St_Thomas
    {  508, 5830,     370, 11, }, // America/St_Vincent
    {  508, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  508, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  508, 1152,     413,  9, }, // Asia/Kathmandu
    {  508,  992,     422,  7, }, // Asia/Kolkata
    {  508, 1178,     429,  5, }, // Asia/Macau
    {  508, 1202,     442,  6, }, // Asia/Yangon
    {  508, 1379,     448,  5, }, // Atlantic/Faroe
    {  508, 6784,     453, 10, }, // Atlantic/St_Helena
    {  508, 2419,     463,  4, }, // Europe/Kyiv
    {  508, 7174,     467,  7, }, // Indian/Reunion
    {  508, 3022,     474,  5, }, // Pacific/Chuuk
    {  508, 2961,     479,  7, }, // Pacific/Pohnpei
    {  509, 6751,     486,  6, }, // Africa/Abidjan Serbian/Cyrillic/Serbia
    {  509, 6723,   25628,  4, }, // Africa/Accra
    {  509, 4961,   32317, 10, }, // Africa/Addis_Ababa
    {  509, 7383,     508,  5, }, // Africa/Algiers
    {  509,   14,     513,  6, }, // Africa/Asmara
    {  509,   44,     519,  6, }, // Africa/Bamako
    {  509, 5159,   32327,  6, }, // Africa/Bangui
    {  509, 6709,     530,  6, }, // Africa/Banjul
    {  509, 7398,   32333,  5, }, // Africa/Bissau
    {  509, 4859,   32338,  7, }, // Africa/Blantyre
    {  509, 5189,   32345,  8, }, // Africa/Brazzaville
    {  509, 4824,   32353,  9, }, // Africa/Bujumbura
    {  509, 2242,   32362,  5, }, // Africa/Cairo
    {  509, 4019,   32367, 10, }, // Africa/Casablanca
    {  509, 7412,     581,  5, }, // Africa/Ceuta
    {  509, 6736,     586,  7, }, // Africa/Conakry
    {  509, 6803,     593,  5, }, // Africa/Dakar
    {  509, 5032,     598, 12, }, // Africa/Dar_es_Salaam
    {  509, 4945,     610,  6, }, // Africa/Djibouti
    {  509, 5145,     616,  5, }, // Africa/Douala
    {  509, 5068,   32377,  7, }, // Africa/El_Aaiun
    {  509, 6816,     628,  7, }, // Africa/Freetown
    {  509, 4808,     635,  7, }, // Africa/Gaborone
    {  509, 4917,   32384,  6, }, // Africa/Harare
    {  509, 4381,   32390, 11, }, // Africa/Johannesburg
    {  509, 4401,     661,  4, }, // Africa/Juba
    {  509, 5053,     665,  7, }, // Africa/Kampala
    {  509, 4426,   32401,  6, }, // Africa/Khartoum
    {  509, 4875,     678,  6, }, // Africa/Kigali
    {  509, 5208,     684,  7, }, // Africa/Kinshasa
    {  509, 4646,     691,  5, }, // Africa/Lagos
    {  509, 5238,     696,  8, }, // Africa/Libreville
    {  509, 6832,     704,  4, }, // Africa/Lome
    {  509, 5113,     708,  6, }, // Africa/Luanda
    {  509, 4841,     714,  9, }, // Africa/Lubumbashi
    {  509, 4903,     723,  6, }, // Africa/Lusaka
    {  509, 5224,     729,  6, }, // Africa/Malabo
    {  509, 4889,   32416,  6, }, // Africa/Maputo
    {  509, 5099,     741,  6, }, // Africa/Maseru
    {  509, 5084,     747,  7, }, // Africa/Mbabane
    {  509, 5015,     754,  7, }, // Africa/Mogadishu
    {  509, 6997,   32422,  9, }, // Africa/Monrovia
    {  509, 3755,   32431,  7, }, // Africa/Nairobi
    {  509, 5173,     777,  7, }, // Africa/Ndjamena
    {  509, 5256,   32438,  7, }, // Africa/Niamey
    {  509, 6766,     790,  7, }, // Africa/Nouakchott
    {  509, 6690,     797,  8, }, // Africa/Ouagadougou
    {  509, 5127,   32445, 10, }, // Africa/Porto-Novo
    {  509, 4337,   32455,  8, }, // Africa/Sao_Tome
    {  509, 2732,     823,  7, }, // Africa/Tripoli
    {  509, 6503,     830,  5, }, // Africa/Tunis
    {  509, 4054,     835,  7, }, // Africa/Windhoek
    {  509,  132,     842,  4, }, // America/Adak
    {  509, 3183,   32463,  7, }, // America/Anchorage
    {  509, 5524,   32470,  7, }, // America/Anguilla
    {  509, 5541,   32477,  7, }, // America/Antigua
    {  509, 4456,   32484, 10, }, // America/Araguaina
    {  509,  166,  157624, 12, }, // America/Argentina/Buenos_Aires
    {  509,   91,     888,  9, }, // America/Argentina/Catamarca
    {  509,  270,   32506,  7, }, // America/Argentina/Cordoba
    {  509,  441,     904,  5, }, // America/Argentina/Jujuy
    {  509, 7425,   32518,  8, }, // America/Argentina/La_Rioja
    {  509,  565,     917,  7, }, // America/Argentina/Mendoza
    {  509, 7452,   32533, 11, }, // America/Argentina/Rio_Gallegos
    {  509, 7483,     935,  5, }, // America/Argentina/Salta
    {  509, 7507,   32544,  8, }, // America/Argentina/San_Juan
    {  509, 5497,   32552,  7, }, // America/Argentina/San_Luis
    {  509, 7534,     956,  7, }, // America/Argentina/Tucuman
    {  509, 7560,   32559,  7, }, // America/Argentina/Ushuaia
    {  509, 5557,     969,  5, }, // America/Aruba
    {  509, 4154,   32566,  8, }, // America/Asuncion
    {  509,  237,  157636, 12, }, // America/Atikokan
    {  509, 3581,   32582,  5, }, // America/Bahia
    {  509, 7586,   32587, 14, }, // America/Bahia_Banderas
    {  509, 5571,    1013,  8, }, // America/Barbados
    {  509, 7609,   32601,  5, }, // America/Belem
    {  509, 5309,   32606,  6, }, // America/Belize
    {  509, 7623,   32612, 13, }, // America/Blanc-Sablon
    {  509, 7644,   32625,  9, }, // America/Boa_Vista
    {  509, 4263,    1052,  6, }, // America/Bogota
    {  509, 7662,   32634,  5, }, // America/Boise
    {  509, 7676,   32639, 11, }, // America/Cambridge_Bay
    {  509, 7698,   32650, 12, }, // America/Campo_Grande
    {  509, 3770,    1087,  6, }, // America/Cancun
    {  509, 4596,    1093,  7, }, // America/Caracas
    {  509, 4247,   32662,  5, }, // America/Cayenne
    {  509, 5398,  157648, 16, }, // America/Cayman
    {  509, 3251,    1129,  6, }, // America/Chicago
    {  509, 7719,  113814,  8, }, // America/Chihuahua
    {  509, 7737,  123927, 13, }, // America/Ciudad_Juarez
    {  509, 5324,   32689,  9, }, // America/Costa_Rica
    {  509, 7759,    1151,  7, }, // America/Creston
    {  509, 3693,    1158,  6, }, // America/Cuiaba
    {  509, 5640,   32698,  7, }, // America/Curacao
    {  509, 7775,   32705, 13, }, // America/Danmarkshavn
    {  509, 7796,   32718,  5, }, // America/Dawson
    {  509, 7811,   32723, 10, }, // America/Dawson_Creek
    {  509,  805,    1201,  6, }, // America/Denver
    {  509, 3351,    1207,  7, }, // America/Detroit
    {  509, 5656,    1214,  8, }, // America/Dominica
    {  509,  893,    1222,  8, }, // America/Edmonton
    {  509, 7832,    1230,  8, }, // America/Eirunepe
    {  509, 5343,    1238,  8, }, // America/El_Salvador
    {  509, 7849,    1246, 11, }, // America/Fort_Nelson
    {  509, 7869,    1257,  9, }, // America/Fortaleza
    {  509, 7887,   32733,  9, }, // America/Glace_Bay
    {  509, 6844,   32742,  7, }, // America/Goose_Bay
    {  509, 4514,   32749, 10, }, // America/Grand_Turk
    {  509, 5687,    1292,  7, }, // America/Grenada
    {  509, 5703,   32759,  9, }, // America/Guadeloupe
    {  509, 3662,    1308,  9, }, // America/Guatemala
    {  509, 4743,   32768,  8, }, // America/Guayaquil
    {  509, 6895,  123967,  7, }, // America/Guyana
    {  509, 1923,   32781,  8, }, // America/Halifax
    {  509, 2221,   32789,  6, }, // America/Havana
    {  509, 7905,   32795,  9, }, // America/Hermosillo
    {  509,  348,    1353, 12, }, // America/Indiana/Indianapolis
    {  509,  481,  123995, 14, }, // America/Indiana/Knox
    {  509, 7924,   32819, 17, }, // America/Indiana/Marengo
    {  509, 7948,  124009, 20, }, // America/Indiana/Petersburg
    {  509, 7975,   32856, 18, }, // America/Indiana/Tell_City
    {  509, 8001,   32874, 15, }, // America/Indiana/Vevay
    {  509, 8023,   32889, 18, }, // America/Indiana/Vincennes
    {  509, 8049,   32907, 17, }, // America/Indiana/Winamac
    {  509, 8073,    1452,  6, }, // America/Inuvik
    {  509,  660,   32924,  8, }, // America/Iqaluit
    {  509, 2665,   32932,  7, }, // America/Jamaica
    {  509, 5282,   32939,  4, }, // America/Juneau
    {  509,  521,  157664,  7, }, // America/Kentucky/Louisville
    {  509, 8088,   32943, 18, }, // America/Kentucky/Monticello
    {  509, 5621,  124060, 10, }, // America/Kralendijk
    {  509, 4278,   32971,  6, }, // America/La_Paz
    {  509, 7132,    1520,  4, }, // America/Lima
    {  509, 3390,   32977, 11, }, // America/Los_Angeles
    {  509, 5849,  157671, 21, }, // America/Lower_Princes
    {  509, 8116,   33007,  6, }, // America/Maceio
    {  509, 8131,   33013,  7, }, // America/Managua
    {  509, 1892,    1566,  6, }, // America/Manaus
    {  509, 5814,    1572,  6, }, // America/Marigot
    {  509, 5722,    1578,  8, }, // America/Martinique
    {  509, 8147,    1587,  9, }, // America/Matamoros
    {  509, 2779,   33020,  8, }, // America/Mazatlan
    {  509, 8165,    1604,  8, }, // America/Menominee
    {  509, 8183,    1612,  6, }, // America/Merida
    {  509, 8198,    1618, 10, }, // America/Metlakatla
    {  509, 2811,   33028, 12, }, // America/Mexico_City
    {  509, 4293,    1634,  7, }, // America/Miquelon
    {  509, 8217,    1641,  7, }, // America/Moncton
    {  509, 8233,   33040,  8, }, // America/Monterrey
    {  509, 4000,    1657, 10, }, // America/Montevideo
    {  509, 5741,   33048,  9, }, // America/Montserrat
    {  509, 5383,   33057,  5, }, // America/Nassau
    {  509, 3294,   33062,  6, }, // America/New_York
    {  509, 8251,    1691,  3, }, // America/Nome
    {  509, 1834,   33068,  6, }, // America/Noronha
    {  509, 8264,   33074, 22, }, // America/North_Dakota/Beulah
    {  509, 8292,   33096, 22, }, // America/North_Dakota/Center
    {  509, 8320,   33118, 26, }, // America/North_Dakota/New_Salem
    {  509,  393,  157692,  6, }, // America/Nuuk
    {  509, 8351,    1776,  7, }, // America/Ojinaga
    {  509, 5413,    1783,  6, }, // America/Panama
    {  509, 6106,    1789, 10, }, // America/Paramaribo
    {  509, 3224,    1799,  6, }, // America/Phoenix
    {  509, 3847,  124151, 12, }, // America/Port-au-Prince
    {  509, 5871,   33150, 13, }, // America/Port_of_Spain
    {  509, 8367,   33163, 10, }, // America/Porto_Velho
    {  509, 5760,  157698, 10, }, // America/Puerto_Rico
    {  509, 3932,   33182, 12, }, // America/Punta_Arenas
    {  509, 8387,   33194, 12, }, // America/Rankin_Inlet
    {  509, 8408,   33206,  6, }, // America/Recife
    {  509, 1979,   33212,  6, }, // America/Regina
    {  509, 8423,   33218,  7, }, // America/Resolute
    {  509,  695,   33225, 10, }, // America/Rio_Branco
    {  509, 8440,   33235,  8, }, // America/Santarem
    {  509, 2165,  157708,  8, }, // America/Santiago
    {  509, 6044,   33252, 13, }, // America/Santo_Domingo
    {  509, 1862,   33265,  9, }, // America/Sao_Paulo
    {  509, 6862,  157716, 12, }, // America/Scoresbysund
    {  509, 8457,    1956,  5, }, // America/Sitka
    {  509, 8471,   33286, 14, }, // America/St_Barthelemy
    {  509, 2045,   33300,  7, }, // America/St_Johns
    {  509, 5780,   33307,  9, }, // America/St_Kitts
    {  509, 5797,   33316, 10, }, // America/St_Lucia
    {  509,  855,   33326,  8, }, // America/St_Thomas
    {  509, 5830,   33334, 12, }, // America/St_Vincent
    {  509, 8493,   33346, 12, }, // America/Swift_Current
    {  509, 5363,    2038, 11, }, // America/Tegucigalpa
    {  509, 5673,    2049,  3, }, // America/Thule
    {  509,  313,    2053,  7, }, // America/Tijuana
    {  509,  608,    2060,  7, }, // America/Toronto
    {  509, 5605,    2067,  7, }, // America/Tortola
    {  509, 2077,    2074,  8, }, // America/Vancouver
    {  509, 2128,   33358,  8, }, // America/Whitehorse
    {  509,  734,   33366,  7, }, // America/Winnipeg
    {  509, 8515,   33373,  7, }, // America/Yakutat
    {  509, 5918,  124250,  5, }, // Antarctica/Casey
    {  509, 6027,   33385,  6, }, // Antarctica/Davis
    {  509, 6066,   33391, 13, }, // Antarctica/DumontDUrville
    {  509, 8531,   33404,  7, }, // Antarctica/Macquarie
    {  509, 7029,   33411,  5, }, // Antarctica/Mawson
    {  509, 7076,   33416,  8, }, // Antarctica/McMurdo
    {  509, 8552,    2154,  6, }, // Antarctica/Palmer
    {  509, 7189,    2160,  6, }, // Antarctica/Rothera
    {  509, 7258,   33424,  4, }, // Antarctica/Syowa
    {  509, 8570,    2171,  4, }, // Antarctica/Troll
    {  509, 7337,   33428,  6, }, // Antarctica/Vostok
    {  509, 1413,   33434, 11, }, // Arctic/Longyearbyen
    {  509, 5487,    2192,  4, }, // Asia/Aden
    {  509, 5297,   33445,  6, }, // Asia/Almaty
    {  509, 3870,   27113,  4, }, // Asia/Amman
    {  509, 5428,   33451,  6, }, // Asia/Anadyr
    {  509, 5440,    2213,  5, }, // Asia/Aqtau
    {  509, 5270,   33457,  7, }, // Asia/Aqtobe
    {  509,  964,    2224,  7, }, // Asia/Ashgabat
    {  509, 8587,   50535,  6, }, // Asia/Atyrau
    {  509, 3509,    2237,  6, }, // Asia/Baghdad
    {  509, 5451,    2243,  7, }, // Asia/Bahrain
    {  509, 3555,    2250,  4, }, // Asia/Baku
    {  509, 4368,    2254,  7, }, // Asia/Bangkok
    {  509, 3473,    2261,  7, }, // Asia/Barnaul
    {  509, 3988,   33476,  6, }, // Asia/Beirut
    {  509, 3680,    2274,  6, }, // Asia/Bishkek
    {  509, 5906,   33482,  6, }, // Asia/Brunei
    {  509, 4503,    2286,  4, }, // Asia/Chita
    {  509, 5963,   33489,  9, }, // Asia/Choibalsan
    {  509, 4413,    2299,  7, }, // Asia/Colombo
    {  509, 4442,    2306,  6, }, // Asia/Damascus
    {  509, 1060,     593,  4, }, // Asia/Dhaka
    {  509, 6125,    2317,  4, }, // Asia/Dili
    {  509, 3498,    2321,  5, }, // Asia/Dubai
    {  509, 6092,    2326,  7, }, // Asia/Dushanbe
    {  509, 8599,    2333,  9, }, // Asia/Famagusta
    {  509, 8614,    2342,  4, }, // Asia/Gaza
    {  509, 4697,   33498,  6, }, // Asia/Hebron
    {  509, 1226,   33504,  9, }, // Asia/Ho_Chi_Minh
    {  509, 2576,  157728,  8, }, // Asia/Hong_Kong
    {  509, 4673,    2366,  4, }, // Asia/Hovd
    {  509, 4086,   33522,  6, }, // Asia/Irkutsk
    {  509, 6969,    2377,  7, }, // Asia/Jakarta
    {  509, 6955,   33528,  8, }, // Asia/Jayapura
    {  509, 1257,   33536,  9, }, // Asia/Jerusalem
    {  509, 3462,    2401,  5, }, // Asia/Kabul
    {  509, 4218,    2420,  8, }, // Asia/Kamchatka
    {  509, 4141,    2431,  6, }, // Asia/Karachi
    {  509, 1152,    2437,  8, }, // Asia/Kathmandu
    {  509, 8624,   33545,  7, }, // Asia/Khandyga
    {  509,  992,   33552,  7, }, // Asia/Kolkata
    {  509, 4099,   33559, 11, }, // Asia/Krasnoyarsk
    {  509, 4761,   33570, 12, }, // Asia/Kuala_Lumpur
    {  509, 5893,    2483,  6, }, // Asia/Kuching
    {  509, 5464,   33582,  6, }, // Asia/Kuwait
    {  509, 1178,    2495,  5, }, // Asia/Macau
    {  509, 3919,    2500,  7, }, // Asia/Magadan
    {  509, 1316,    2507,  7, }, // Asia/Makassar
    {  509, 7145,    2514,  6, }, // Asia/Manila
    {  509, 6883,   33593,  6, }, // Asia/Muscat
    {  509, 2446,   33599,  8, }, // Asia/Nicosia
    {  509, 8638,   33607, 11, }, // Asia/Novokuznetsk
    {  509, 4037,    2544, 11, }, // Asia/Novosibirsk
    {  509, 4131,    2555,  4, }, // Asia/Omsk
    {  509, 7108,   33618,  4, }, // Asia/Oral
    {  509, 6924,   33622,  8, }, // Asia/Phnom_Penh
    {  509, 8656,  124307, 10, }, // Asia/Pontianak
    {  509, 4116,   33630,  9, }, // Asia/Pyongyang
    {  509, 5476,    2588,  5, }, // Asia/Qatar
    {  509, 8671,   33639,  8, }, // Asia/Qostanay
    {  509, 4171,   33647,  9, }, // Asia/Qyzylorda
    {  509, 3486,   33656,  5, }, // Asia/Riyadh
    {  509, 4310,    2623,  7, }, // Asia/Sakhalin
    {  509, 7208,    2630,  9, }, // Asia/Samarkand
    {  509, 3116,    2639,  4, }, // Asia/Seoul
    {  509, 1020,   33661,  6, }, // Asia/Shanghai
    {  509, 3137,    2649,  8, }, // Asia/Singapore
    {  509, 4199,   33667, 13, }, // Asia/Srednekolymsk
    {  509, 3100,   33680,  6, }, // Asia/Taipei
    {  509, 4683,    2676,  7, }, // Asia/Tashkent
    {  509, 3817,   33686,  7, }, // Asia/Tbilisi
    {  509, 2638,   33693,  7, }, // Asia/Tehran
    {  509, 1284,   33700,  5, }, // Asia/Thimphu
    {  509, 2687,    2701,  5, }, // Asia/Tokyo
    {  509, 4474,    2706,  5, }, // Asia/Tomsk
    {  509, 1346,   33705, 10, }, // Asia/Ulaanbaatar
    {  509, 1126,  157736,  6, }, // Asia/Urumqi
    {  509, 8685,    2727,  8, }, // Asia/Ust-Nera
    {  509, 6940,   33722, 10, }, // Asia/Vientiane
    {  509, 4612,    2742, 11, }, // Asia/Vladivostok
    {  509, 4730,   33732,  7, }, // Asia/Yakutsk
    {  509, 1202,   27502,  6, }, // Asia/Yangon
    {  509, 3785,   33739, 13, }, // Asia/Yekaterinburg
    {  509, 3649,   33752,  7, }, // Asia/Yerevan
    {  509, 3565,   33759,  5, }, // Atlantic/Azores
    {  509, 5588,   33764,  7, }, // Atlantic/Bermuda
    {  509, 6591,   33771, 15, }, // Atlantic/Canary
    {  509, 3629,   33786, 18, }, // Atlantic/Cape_Verde
    {  509, 1379,  157742, 13, }, // Atlantic/Faroe
    {  509, 8699,    2862,  7, }, // Atlantic/Madeira
    {  509, 2614,   33817,  9, }, // Atlantic/Reykjavik
    {  509, 7235,   33826, 13, }, // Atlantic/South_Georgia
    {  509, 6784,  157755, 12, }, // Atlantic/St_Helena
    {  509, 6607,    2920,  6, }, // Atlantic/Stanley
    {  509, 1654,  157767,  7, }, // Australia/Adelaide
    {  509, 1619,   33849,  8, }, // Australia/Brisbane
    {  509, 1783,   33857, 10, }, // Australia/Broken_Hill
    {  509, 1567,    2952,  6, }, // Australia/Darwin
    {  509, 3539,    2958,  5, }, // Australia/Eucla
    {  509, 1500,    2963,  6, }, // Australia/Hobart
    {  509, 8716,    2969,  8, }, // Australia/Lindeman
    {  509, 1531,   35811,  8, }, // Australia/Lord_Howe
    {  509, 1711,    2985,  7, }, // Australia/Melbourne
    {  509, 1746,    2992,  4, }, // Australia/Perth
    {  509, 1447,   33875,  6, }, // Australia/Sydney
    {  509, 6390,    3002,  9, }, // Europe/Amsterdam
    {  509, 6149,   27734,  6, }, // Europe/Andorra
    {  509, 3522,  157774,  8, }, // Europe/Astrakhan
    {  509, 6577,   33881,  5, }, // Europe/Athens
    {  509, 6285,   33886,  7, }, // Europe/Belgrade
    {  509, 4659,    3037,  6, }, // Europe/Berlin
    {  509, 6437,    3043, 10, }, // Europe/Bratislava
    {  509, 6178,   33893,  6, }, // Europe/Brussels
    {  509, 3830,   33899,  8, }, // Europe/Bucharest
    {  509, 3708,   33907, 10, }, // Europe/Budapest
    {  509, 8751,  157782,  8, }, // Europe/Busingen
    {  509, 2475,   33925,  7, }, // Europe/Chisinau
    {  509, 6238,   33932, 10, }, // Europe/Copenhagen
    {  509, 2260,   33942,  6, }, // Europe/Dublin
    {  509, 6256,    3123,  9, }, // Europe/Gibraltar
    {  509, 8767,   33948,  6, }, // Europe/Guernsey
    {  509, 6561,    3138,  8, }, // Europe/Helsinki
    {  509, 8783,  157790, 10, }, // Europe/Isle_of_Man
    {  509, 1097,   33964,  8, }, // Europe/Istanbul
    {  509, 8802,  124415,  5, }, // Europe/Jersey
    {  509, 3881,   33972, 11, }, // Europe/Kaliningrad
    {  509, 8924,    3178,  5, }, // Europe/Kirov
    {  509, 2419,   33983,  5, }, // Europe/Kyiv
    {  509, 3078,   33988,  7, }, // Europe/Lisbon
    {  509, 6455,   33995,  7, }, // Europe/Ljubljana
    {  509, 2393,    3204,  6, }, // Europe/London
    {  509, 6314,   34002, 10, }, // Europe/Luxembourg
    {  509, 4794,    3221,  6, }, // Europe/Madrid
    {  509, 6346,    3227,  5, }, // Europe/Malta
    {  509, 6531,   34012,  8, }, // Europe/Mariehamn
    {  509, 3595,    3241,  5, }, // Europe/Minsk
    {  509, 6359,    3246,  6, }, // Europe/Monaco
    {  509, 3443,    3252,  6, }, // Europe/Moscow
    {  509, 6407,    3258,  4, }, // Europe/Oslo
    {  509, 4186,   34020,  5, }, // Europe/Paris
    {  509, 6373,    3267,  9, }, // Europe/Podgorica
    {  509, 6224,    3276,  4, }, // Europe/Prague
    {  509, 8816,    3281,  4, }, // Europe/Riga
    {  509, 6273,    3285,  3, }, // Europe/Rome
    {  509, 4233,    3288,  6, }, // Europe/Samara
    {  509, 6419,   34025, 10, }, // Europe/San_Marino
    {  509, 6194,   34035,  8, }, // Europe/Sarajevo
    {  509, 4353,    3311,  7, }, // Europe/Saratov
    {  509, 8828,   34043, 10, }, // Europe/Simferopol
    {  509, 6332,   34053,  6, }, // Europe/Skopje
    {  509, 6548,   34059,  6, }, // Europe/Sofia
    {  509, 6472,   34065,  8, }, // Europe/Stockholm
    {  509, 8846,   34073,  5, }, // Europe/Tallinn
    {  509, 6135,    3352,  6, }, // Europe/Tirane
    {  509, 8861,   34078,  8, }, // Europe/Ulyanovsk
    {  509, 6301,   34086,  5, }, // Europe/Vaduz
    {  509, 6516,    3372,  7, }, // Europe/Vatican
    {  509, 6164,   34091,  3, }, // Europe/Vienna
    {  509, 8878,  157800,  6, }, // Europe/Vilnius
    {  509, 4629,    3390,  9, }, // Europe/Volgograd
    {  509, 3055,    3399,  7, }, // Europe/Warsaw
    {  509, 6210,    3406,  6, }, // Europe/Zagreb
    {  509, 6489,   34100,  5, }, // Europe/Zurich
    {  509, 4980,   34105, 12, }, // Indian/Antananarivo
    {  509, 6910,    3430,  5, }, // Indian/Chagos
    {  509, 5979,   34117,  5, }, // Indian/Christmas
    {  509, 5996,    3447,  5, }, // Indian/Cocos
    {  509, 4931,   34122,  6, }, // Indian/Comoro
    {  509, 6624,    3469,  8, }, // Indian/Kerguelen
    {  509, 7223,   34128,  4, }, // Indian/Mahe
    {  509, 7013,   34132,  7, }, // Indian/Maldives
    {  509, 3971,   34139, 10, }, // Indian/Mauritius
    {  509, 5000,   34149,  5, }, // Indian/Mayotte
    {  509, 7174,   34154,  7, }, // Indian/Reunion
    {  509, 4324,   34161,  5, }, // Pacific/Apia
    {  509,  932,   34166,  6, }, // Pacific/Auckland
    {  509, 3608,  124499,  8, }, // Pacific/Bougainville
    {  509, 2863,   36001,  5, }, // Pacific/Chatham
    {  509, 3022,    3530,  4, }, // Pacific/Chuuk
    {  509, 2201,   34178, 14, }, // Pacific/Easter
    {  509, 7323,    3545,  4, }, // Pacific/Efate
    {  509, 7290,    3550,  7, }, // Pacific/Fakaofo
    {  509, 3804,    3557,  4, }, // Pacific/Fiji
    {  509, 7306,    3561,  8, }, // Pacific/Funafuti
    {  509, 6641,    3569,  9, }, // Pacific/Galapagos
    {  509, 6659,   34192,  7, }, // Pacific/Gambier
    {  509, 3724,   34199, 11, }, // Pacific/Guadalcanal
    {  509, 5935,    3615,  4, }, // Pacific/Guam
    {  509, 2929,   34210,  8, }, // Pacific/Honolulu
    {  509, 2897,   34218,  9, }, // Pacific/Kanton
    {  509, 3900,    3625, 10, }, // Pacific/Kiritimati
    {  509, 6982,   34227,  5, }, // Pacific/Kosrae
    {  509, 2708,   34232, 10, }, // Pacific/Kwajalein
    {  509, 4779,   34242,  6, }, // Pacific/Majuro
    {  509, 3953,    3656,  6, }, // Pacific/Marquesas
    {  509, 8909,   34248,  6, }, // Pacific/Midway
    {  509, 7047,    3689,  5, }, // Pacific/Nauru
    {  509, 7095,    3694,  4, }, // Pacific/Niue
    {  509, 4070,    3698,  7, }, // Pacific/Norfolk
    {  509, 7061,    3705,  5, }, // Pacific/Noumea
    {  509, 2991,   34254,  9, }, // Pacific/Pago_Pago
    {  509, 7118,    3719,  5, }, // Pacific/Palau
    {  509, 7157,    3724,  7, }, // Pacific/Pitcairn
    {  509, 2961,   34263,  6, }, // Pacific/Pohnpei
    {  509, 4709,   34269, 11, }, // Pacific/Port_Moresby
    {  509, 6009,    3748,  9, }, // Pacific/Rarotonga
    {  509, 5948,   34280,  6, }, // Pacific/Saipan
    {  509, 7275,   34286,  6, }, // Pacific/Tahiti
    {  509, 6675,    3768,  6, }, // Pacific/Tarawa
    {  509, 4485,    3774,  9, }, // Pacific/Tongatapu
    {  509, 7355,   34292,  4, }, // Pacific/Wake
    {  509, 7368,   34296,  5, }, // Pacific/Wallis
    {  510,  348,  123982, 13, }, // America/Indiana/Indianapolis Serbian/Cyrillic/Bosnia and Herzegovina
    {  510, 8001,  157806, 14, }, // America/Indiana/Vevay
    {  510, 8023,  157820, 17, }, // America/Indiana/Vincennes
    {  510,  521,  157664,  6, }, // America/Kentucky/Louisville
    {  510, 8264,  157837, 22, }, // America/North_Dakota/Beulah
    {  510, 8292,  157859, 23, }, // America/North_Dakota/Center
    {  510, 8320,  157882, 27, }, // America/North_Dakota/New_Salem
    {  510, 3847,    1805, 12, }, // America/Port-au-Prince
    {  510, 5871,  124163, 13, }, // America/Port_of_Spain
    {  510, 5760,   33173,  9, }, // America/Puerto_Rico
    {  510, 1979,  124188,  7, }, // America/Regina
    {  510, 8423,  114126,  7, }, // America/Resolute
    {  510, 6862,   33274, 12, }, // America/Scoresbysund
    {  510, 8471,   35405, 13, }, // America/St_Barthelemy
    {  510, 2045,  124210,  9, }, // America/St_Johns
    {  510,  855,  124240, 10, }, // America/St_Thomas
    {  510, 8493,  157909, 12, }, // America/Swift_Current
    {  510, 6066,  157921, 13, }, // Antarctica/DumontDUrville
    {  510, 8531,  124267,  7, }, // Antarctica/Macquarie
    {  510, 1413,  157934,  7, }, // Arctic/Longyearbyen
    {  510, 2260,   33942,  6, }, // Europe/Dublin
    {  510, 2393,    3204,  6, }, // Europe/London
    {  510, 7095,  157941,  5, }, // Pacific/Niue
    {  513, 6751,   56229,  7, }, // Africa/Abidjan Serbian/Latin/Bosnia and Herzegovina
    {  513, 6723,    4550,  4, }, // Africa/Accra
    {  513, 4961,   34301, 10, }, // Africa/Addis_Ababa
    {  513, 7383,   56099,  5, }, // Africa/Algiers
    {  513,   14,   89768,  6, }, // Africa/Asmara
    {  513,   44,  157946,  6, }, // Africa/Bamako
    {  513, 5159,  157952,  6, }, // Africa/Bangui
    {  513, 6709,  157958,  6, }, // Africa/Banjul
    {  513, 7398,  157964,  5, }, // Africa/Bissau
    {  513, 4859,    4588,  7, }, // Africa/Blantyre
    {  513, 5189,    4596,  8, }, // Africa/Brazzaville
    {  513, 4824,  157969, 10, }, // Africa/Bujumbura
    {  513, 2242,   24913,  5, }, // Africa/Cairo
    {  513, 4019,   31744, 10, }, // Africa/Casablanca
    {  513, 7412,   19754,  5, }, // Africa/Ceuta
    {  513, 6736,    4637,  7, }, // Africa/Conakry
    {  513, 6803,   59699,  5, }, // Africa/Dakar
    {  513, 5032,    4644, 12, }, // Africa/Dar_es_Salaam
    {  513, 4945,   31754,  7, }, // Africa/Djibouti
    {  513, 5145,   19777,  5, }, // Africa/Douala
    {  513, 5068,    4663,  7, }, // Africa/El_Aaiun
    {  513, 6816,    4670,  7, }, // Africa/Freetown
    {  513, 4808,   59741,  7, }, // Africa/Gaborone
    {  513, 4917,   59749,  6, }, // Africa/Harare
    {  513, 4381,    4677, 11, }, // Africa/Johannesburg
    {  513, 4401,  120501,  5, }, // Africa/Juba
    {  513, 5053,  157979,  7, }, // Africa/Kampala
    {  513, 4426,    4693,  6, }, // Africa/Khartoum
    {  513, 4875,  120515,  6, }, // Africa/Kigali
    {  513, 5208,  117752,  7, }, // Africa/Kinshasa
    {  513, 4646,   59800,  5, }, // Africa/Lagos
    {  513, 5238,    4699,  8, }, // Africa/Libreville
    {  513, 6832,   59815,  4, }, // Africa/Lome
    {  513, 5113,  157986,  6, }, // Africa/Luanda
    {  513, 4841,  117774,  9, }, // Africa/Lubumbashi
    {  513, 4903,  157992,  6, }, // Africa/Lusaka
    {  513, 5224,  157998,  6, }, // Africa/Malabo
    {  513, 4889,  158004,  6, }, // Africa/Maputo
    {  513, 5099,   59841,  6, }, // Africa/Maseru
    {  513, 5084,  158010,  7, }, // Africa/Mbabane
    {  513, 5015,   31761,  7, }, // Africa/Mogadishu
    {  513, 6997,  117797,  9, }, // Africa/Monrovia
    {  513, 3755,   59871,  7, }, // Africa/Nairobi
    {  513, 5173,   56262,  8, }, // Africa/Ndjamena
    {  513, 5256,  158017,  7, }, // Africa/Niamey
    {  513, 6766,   56270,  7, }, // Africa/Nouakchott
    {  513, 6690,   24967,  8, }, // Africa/Ouagadougou
    {  513, 5127,   24975, 10, }, // Africa/Porto-Novo
    {  513, 4337,   89774,  8, }, // Africa/Sao_Tome
    {  513, 2732,   34368,  7, }, // Africa/Tripoli
    {  513, 6503,   89940,  5, }, // Africa/Tunis
    {  513, 4054,   19877,  7, }, // Africa/Windhoek
    {  513,  132,   96487,  4, }, // America/Adak
    {  513, 3183,  158024,  8, }, // America/Anchorage
    {  513, 5524,   31768,  7, }, // America/Anguilla
    {  513, 5541,   31775,  7, }, // America/Antigua
    {  513, 4456,  120668, 10, }, // America/Araguaina
    {  513,  166,  158032, 12, }, // America/Argentina/Buenos_Aires
    {  513,   91,    4788,  9, }, // America/Argentina/Catamarca
    {  513,  270,    4797,  7, }, // America/Argentina/Cordoba
    {  513,  441,  158044,  5, }, // America/Argentina/Jujuy
    {  513, 7425,    4809,  8, }, // America/Argentina/La_Rioja
    {  513,  565,   96160,  7, }, // America/Argentina/Mendoza
    {  513, 7452,  120708, 12, }, // America/Argentina/Rio_Gallegos
    {  513, 7483,   59989,  5, }, // America/Argentina/Salta
    {  513, 7507,  158049,  8, }, // America/Argentina/San_Juan
    {  513, 5497,  158057,  7, }, // America/Argentina/San_Luis
    {  513, 7534,    4850,  7, }, // America/Argentina/Tucuman
    {  513, 7560,  158064,  7, }, // America/Argentina/Ushuaia
    {  513, 5557,  158071,  5, }, // America/Aruba
    {  513, 4154,    4869,  8, }, // America/Asuncion
    {  513,  237,  158076, 12, }, // America/Atikokan
    {  513, 3581,  117972,  5, }, // America/Bahia
    {  513, 7586,  158088, 14, }, // America/Bahia_Banderas
    {  513, 5571,  117990,  8, }, // America/Barbados
    {  513, 7609,   60040,  5, }, // America/Belem
    {  513, 5309,  158102,  6, }, // America/Belize
    {  513, 7623,  158108, 13, }, // America/Blanc-Sablon
    {  513, 7644,  158121,  9, }, // America/Boa_Vista
    {  513, 4263,  158130,  6, }, // America/Bogota
    {  513, 7662,  158136,  5, }, // America/Boise
    {  513, 7676,  158141, 12, }, // America/Cambridge_Bay
    {  513, 7698,   96188, 12, }, // America/Campo_Grande
    {  513, 3770,    4959,  6, }, // America/Cancun
    {  513, 4596,    4965,  7, }, // America/Caracas
    {  513, 4247,    4972,  5, }, // America/Cayenne
    {  513, 5398,  158153, 16, }, // America/Cayman
    {  513, 3251,  158169,  6, }, // America/Chicago
    {  513, 7719,  120907,  8, }, // America/Chihuahua
    {  513, 7737,  158175, 13, }, // America/Ciudad_Juarez
    {  513, 5324,   31782,  9, }, // America/Costa_Rica
    {  513, 7759,    5020,  7, }, // America/Creston
    {  513, 3693,   96220,  6, }, // America/Cuiaba
    {  513, 5640,    5033,  7, }, // America/Curacao
    {  513, 7775,  158188, 13, }, // America/Danmarkshavn
    {  513, 7796,  120962,  5, }, // America/Dawson
    {  513, 7811,  120969, 10, }, // America/Dawson_Creek
    {  513,  805,   60109,  6, }, // America/Denver
    {  513, 3351,  118170,  7, }, // America/Detroit
    {  513, 5656,   20071,  8, }, // America/Dominica
    {  513,  893,   60132,  8, }, // America/Edmonton
    {  513, 7832,  158201,  8, }, // America/Eirunepe
    {  513, 5343,    5090,  8, }, // America/El_Salvador
    {  513, 7849,  121026, 11, }, // America/Fort_Nelson
    {  513, 7869,  158209,  9, }, // America/Fortaleza
    {  513, 7887,  158218,  9, }, // America/Glace_Bay
    {  513, 6844,  158227,  7, }, // America/Goose_Bay
    {  513, 4514,  158234, 10, }, // America/Grand_Turk
    {  513, 5687,  158244,  7, }, // America/Grenada
    {  513, 5703,   31791,  9, }, // America/Guadeloupe
    {  513, 3662,   31800,  9, }, // America/Guatemala
    {  513, 4743,  118250,  8, }, // America/Guayaquil
    {  513, 6895,   56104,  7, }, // America/Guyana
    {  513, 1923,    5189,  8, }, // America/Halifax
    {  513, 2221,  140276,  6, }, // America/Havana
    {  513, 7905,  158251, 10, }, // America/Hermosillo
    {  513,  348,  158261, 13, }, // America/Indiana/Indianapolis
    {  513,  481,  158274, 14, }, // America/Indiana/Knox
    {  513, 7924,  158288, 17, }, // America/Indiana/Marengo
    {  513, 7948,  158305, 20, }, // America/Indiana/Petersburg
    {  513, 7975,  158325, 18, }, // America/Indiana/Tell_City
    {  513, 8001,  158343, 14, }, // America/Indiana/Vevay
    {  513, 8023,  158357, 17, }, // America/Indiana/Vincennes
    {  513, 8049,  158374, 17, }, // America/Indiana/Winamac
    {  513, 8073,  118413,  6, }, // America/Inuvik
    {  513,  660,  158391,  8, }, // America/Iqaluit
    {  513, 2665,   31809,  7, }, // America/Jamaica
    {  513, 5282,  158399,  4, }, // America/Juneau
    {  513,  521,  158403,  6, }, // America/Kentucky/Louisville
    {  513, 8088,  158409, 18, }, // America/Kentucky/Monticello
    {  513, 5621,  158427, 10, }, // America/Kralendijk
    {  513, 4278,  158437,  6, }, // America/La_Paz
    {  513, 7132,   20446,  4, }, // America/Lima
    {  513, 3390,  158443, 11, }, // America/Los_Angeles
    {  513, 5849,  158454, 21, }, // America/Lower_Princes
    {  513, 8116,  118506,  6, }, // America/Maceio
    {  513, 8131,  118512,  7, }, // America/Managua
    {  513, 1892,  118519,  6, }, // America/Manaus
    {  513, 5814,   60430,  6, }, // America/Marigot
    {  513, 5722,    5400,  8, }, // America/Martinique
    {  513, 8147,  118534,  9, }, // America/Matamoros
    {  513, 2779,  158475,  8, }, // America/Mazatlan
    {  513, 8165,    5409,  8, }, // America/Menominee
    {  513, 8183,    3899,  6, }, // America/Merida
    {  513, 8198,  158483, 10, }, // America/Metlakatla
    {  513, 2811,  158493, 12, }, // America/Mexico_City
    {  513, 4293,    5434,  7, }, // America/Miquelon
    {  513, 8217,    5441,  7, }, // America/Moncton
    {  513, 8233,   60479,  8, }, // America/Monterrey
    {  513, 4000,  158505, 10, }, // America/Montevideo
    {  513, 5741,    5457,  9, }, // America/Montserrat
    {  513, 5383,  121439,  5, }, // America/Nassau
    {  513, 3294,  158515,  7, }, // America/New_York
    {  513, 8251,   20373,  3, }, // America/Nome
    {  513, 1834,    5479,  7, }, // America/Noronha
    {  513, 8264,  158522, 22, }, // America/North_Dakota/Beulah
    {  513, 8292,  158544, 23, }, // America/North_Dakota/Center
    {  513, 8320,  158567, 27, }, // America/North_Dakota/New_Salem
    {  513,  393,  158594,  6, }, // America/Nuuk
    {  513, 8351,    5561,  7, }, // America/Ojinaga
    {  513, 5413,  158600,  6, }, // America/Panama
    {  513, 6106,  158606, 10, }, // America/Paramaribo
    {  513, 3224,  121555,  6, }, // America/Phoenix
    {  513, 3847,  158616, 12, }, // America/Port-au-Prince
    {  513, 5871,  158628, 13, }, // America/Port_of_Spain
    {  513, 8367,  158641, 11, }, // America/Porto_Velho
    {  513, 5760,   31888,  9, }, // America/Puerto_Rico
    {  513, 3932,  121616, 12, }, // America/Punta_Arenas
    {  513, 8387,  121630, 12, }, // America/Rankin_Inlet
    {  513, 8408,  158652,  6, }, // America/Recife
    {  513, 1979,  158658,  8, }, // America/Regina
    {  513, 8423,  158666,  7, }, // America/Resolute
    {  513,  695,   20506, 10, }, // America/Rio_Branco
    {  513, 8440,  158673,  8, }, // America/Santarem
    {  513, 2165,  118795,  8, }, // America/Santiago
    {  513, 6044,  158681, 13, }, // America/Santo_Domingo
    {  513, 1862,  158694,  9, }, // America/Sao_Paulo
    {  513, 6862,  118823, 12, }, // America/Scoresbysund
    {  513, 8457,  158703,  5, }, // America/Sitka
    {  513, 8471,  121734, 13, }, // America/St_Barthelemy
    {  513, 2045,  121747, 10, }, // America/St_Johns
    {  513, 5780,  121759,  9, }, // America/St_Kitts
    {  513, 5797,  158708, 10, }, // America/St_Lucia
    {  513,  855,  121781, 10, }, // America/St_Thomas
    {  513, 5830,  158718, 12, }, // America/St_Vincent
    {  513, 8493,  121807, 12, }, // America/Swift_Current
    {  513, 5363,    5759, 11, }, // America/Tegucigalpa
    {  513, 5673,   20642,  3, }, // America/Thule
    {  513,  313,    5775,  7, }, // America/Tijuana
    {  513,  608,  158730,  7, }, // America/Toronto
    {  513, 5605,  158737,  7, }, // America/Tortola
    {  513, 2077,    5789,  8, }, // America/Vancouver
    {  513, 2128,  158744,  8, }, // America/Whitehorse
    {  513,  734,   60758,  7, }, // America/Winnipeg
    {  513, 8515,    5813,  7, }, // America/Yakutat
    {  513, 5918,    5820,  5, }, // Antarctica/Casey
    {  513, 6027,    5825,  6, }, // Antarctica/Davis
    {  513, 6066,  158752, 13, }, // Antarctica/DumontDUrville
    {  513, 8531,  118971,  7, }, // Antarctica/Macquarie
    {  513, 7029,  118978,  5, }, // Antarctica/Mawson
    {  513, 7076,  158765,  8, }, // Antarctica/McMurdo
    {  513, 8552,  121938,  6, }, // Antarctica/Palmer
    {  513, 7189,  118999,  6, }, // Antarctica/Rothera
    {  513, 7258,  119005,  4, }, // Antarctica/Syowa
    {  513, 8570,  119009,  4, }, // Antarctica/Troll
    {  513, 7337,  119015,  6, }, // Antarctica/Vostok
    {  513, 1413,  158773,  7, }, // Arctic/Longyearbyen
    {  513, 5487,   60791,  4, }, // Asia/Aden
    {  513, 5297,    5890,  6, }, // Asia/Almaty
    {  513, 3870,    5896,  4, }, // Asia/Amman
    {  513, 5428,    5900,  6, }, // Asia/Anadyr
    {  513, 5440,    5906,  5, }, // Asia/Aqtau
    {  513, 5270,   31897,  7, }, // Asia/Aqtobe
    {  513,  964,   31904,  7, }, // Asia/Ashgabat
    {  513, 8587,    5917,  6, }, // Asia/Atyrau
    {  513, 3509,    4063,  6, }, // Asia/Baghdad
    {  513, 5451,    4069,  7, }, // Asia/Bahrain
    {  513, 3555,   60829,  4, }, // Asia/Baku
    {  513, 4368,  119081,  7, }, // Asia/Bangkok
    {  513, 3473,  119089,  7, }, // Asia/Barnaul
    {  513, 3988,    5930,  6, }, // Asia/Beirut
    {  513, 3680,   31917,  6, }, // Asia/Bishkek
    {  513, 5906,   31923,  6, }, // Asia/Brunei
    {  513, 4503,   56142,  4, }, // Asia/Chita
    {  513, 5963,   31930,  9, }, // Asia/Choibalsan
    {  513, 4413,    5949,  7, }, // Asia/Colombo
    {  513, 4442,    4102,  6, }, // Asia/Damascus
    {  513, 1060,    5956,  4, }, // Asia/Dhaka
    {  513, 6125,  122105,  4, }, // Asia/Dili
    {  513, 3498,  119135,  5, }, // Asia/Dubai
    {  513, 6092,   31939,  7, }, // Asia/Dushanbe
    {  513, 8599,  158780,  9, }, // Asia/Famagusta
    {  513, 8614,   60896,  4, }, // Asia/Gaza
    {  513, 4697,   60905,  6, }, // Asia/Hebron
    {  513, 1226,   31946,  9, }, // Asia/Ho_Chi_Minh
    {  513, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  513, 4673,  119165,  4, }, // Asia/Hovd
    {  513, 4086,  139633,  6, }, // Asia/Irkutsk
    {  513, 6969,   31955,  8, }, // Asia/Jakarta
    {  513, 6955,   31963,  9, }, // Asia/Jayapura
    {  513, 1257,  158789,  9, }, // Asia/Jerusalem
    {  513, 3462,  119188,  5, }, // Asia/Kabul
    {  513, 4218,   31981,  8, }, // Asia/Kamchatka
    {  513, 4141,   31989,  6, }, // Asia/Karachi
    {  513, 1152,    6018,  8, }, // Asia/Kathmandu
    {  513, 8624,   31995,  7, }, // Asia/Khandyga
    {  513,  992,   25307,  7, }, // Asia/Kolkata
    {  513, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  513, 4761,  158798, 12, }, // Asia/Kuala_Lumpur
    {  513, 5893,   32002,  6, }, // Asia/Kuching
    {  513, 5464,    6058,  6, }, // Asia/Kuwait
    {  513, 1178,    6064,  5, }, // Asia/Macau
    {  513, 3919,  119244,  7, }, // Asia/Magadan
    {  513, 1316,    6069,  7, }, // Asia/Makassar
    {  513, 7145,  158810,  6, }, // Asia/Manila
    {  513, 6883,    4227,  6, }, // Asia/Muscat
    {  513, 2446,   32013,  8, }, // Asia/Nicosia
    {  513, 8638,   32021, 12, }, // Asia/Novokuznetsk
    {  513, 4037,   61036, 11, }, // Asia/Novosibirsk
    {  513, 4131,   61048,  4, }, // Asia/Omsk
    {  513, 7108,   61053,  4, }, // Asia/Oral
    {  513, 6924,   20928,  8, }, // Asia/Phnom_Penh
    {  513, 8656,  158816, 10, }, // Asia/Pontianak
    {  513, 4116,   32033,  9, }, // Asia/Pyongyang
    {  513, 5476,    4233,  5, }, // Asia/Qatar
    {  513, 8671,    6103,  8, }, // Asia/Qostanay
    {  513, 4171,    6111,  9, }, // Asia/Qyzylorda
    {  513, 3486,   32042,  5, }, // Asia/Riyadh
    {  513, 4310,   32047,  7, }, // Asia/Sakhalin
    {  513, 7208,   25356,  9, }, // Asia/Samarkand
    {  513, 3116,    6131,  4, }, // Asia/Seoul
    {  513, 1020,   32054,  6, }, // Asia/Shanghai
    {  513, 3137,   19291,  8, }, // Asia/Singapore
    {  513, 4199,  158826, 14, }, // Asia/Srednekolymsk
    {  513, 3100,    6163,  6, }, // Asia/Taipei
    {  513, 4683,   32060,  7, }, // Asia/Tashkent
    {  513, 3817,  122458,  7, }, // Asia/Tbilisi
    {  513, 2638,    6175,  7, }, // Asia/Tehran
    {  513, 1284,   90291,  5, }, // Asia/Thimphu
    {  513, 2687,    4257,  5, }, // Asia/Tokyo
    {  513, 4474,  119419,  5, }, // Asia/Tomsk
    {  513, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  513, 1126,  158840,  6, }, // Asia/Urumqi
    {  513, 8685,  158846,  8, }, // Asia/Ust-Nera
    {  513, 6940,   32073, 10, }, // Asia/Vientiane
    {  513, 4612,   61189, 11, }, // Asia/Vladivostok
    {  513, 4730,    6206,  7, }, // Asia/Yakutsk
    {  513, 1202,    6213,  6, }, // Asia/Yangon
    {  513, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  513, 3649,    6231,  7, }, // Asia/Yerevan
    {  513, 3565,   32083,  5, }, // Atlantic/Azores
    {  513, 5588,   72942,  7, }, // Atlantic/Bermuda
    {  513, 6591,  158854, 15, }, // Atlantic/Canary
    {  513, 3629,  158869, 18, }, // Atlantic/Cape_Verde
    {  513, 1379,  158887, 13, }, // Atlantic/Faroe
    {  513, 8699,  158900,  7, }, // Atlantic/Madeira
    {  513, 2614,   32088,  9, }, // Atlantic/Reykjavik
    {  513, 7235,  158907, 15, }, // Atlantic/South_Georgia
    {  513, 6784,  158922, 12, }, // Atlantic/St_Helena
    {  513, 6607,    6298,  6, }, // Atlantic/Stanley
    {  513, 1654,  158934,  7, }, // Australia/Adelaide
    {  513, 1619,  158941,  8, }, // Australia/Brisbane
    {  513, 1783,  122675, 10, }, // Australia/Broken_Hill
    {  513, 1567,    6331,  6, }, // Australia/Darwin
    {  513, 3539,  158949,  5, }, // Australia/Eucla
    {  513, 1500,   61346,  6, }, // Australia/Hobart
    {  513, 8716,  119621,  8, }, // Australia/Lindeman
    {  513, 1531,   21137,  8, }, // Australia/Lord_Howe
    {  513, 1711,    6359,  7, }, // Australia/Melbourne
    {  513, 1746,   21145,  4, }, // Australia/Perth
    {  513, 1447,    6371,  6, }, // Australia/Sydney
    {  513, 6390,   61373,  9, }, // Europe/Amsterdam
    {  513, 6149,   32109,  6, }, // Europe/Andorra
    {  513, 3522,    6384,  8, }, // Europe/Astrakhan
    {  513, 6577,   32123,  5, }, // Europe/Athens
    {  513, 6285,    6398,  7, }, // Europe/Belgrade
    {  513, 4659,   61411,  6, }, // Europe/Berlin
    {  513, 6437,  158954, 10, }, // Europe/Bratislava
    {  513, 6178,   32128,  6, }, // Europe/Brussels
    {  513, 3830,   32134,  8, }, // Europe/Bucharest
    {  513, 3708,   32142, 10, }, // Europe/Budapest
    {  513, 8751,  158964,  8, }, // Europe/Busingen
    {  513, 2475,   32152,  8, }, // Europe/Chisinau
    {  513, 6238,    4376, 10, }, // Europe/Copenhagen
    {  513, 2260,   32160,  6, }, // Europe/Dublin
    {  513, 6256,  122861,  9, }, // Europe/Gibraltar
    {  513, 8767,   32166,  6, }, // Europe/Guernsey
    {  513, 6561,  122879,  8, }, // Europe/Helsinki
    {  513, 8783,   32172, 10, }, // Europe/Isle_of_Man
    {  513, 1097,   61535,  8, }, // Europe/Istanbul
    {  513, 8802,  158972,  6, }, // Europe/Jersey
    {  513, 3881,   32182, 12, }, // Europe/Kaliningrad
    {  513, 8924,  119814,  5, }, // Europe/Kirov
    {  513, 2419,   32194,  5, }, // Europe/Kyiv
    {  513, 3078,   32199,  7, }, // Europe/Lisbon
    {  513, 6455,  158978,  9, }, // Europe/Ljubljana
    {  513, 2393,   61584,  6, }, // Europe/London
    {  513, 6314,    6510, 10, }, // Europe/Luxembourg
    {  513, 4794,   61602,  6, }, // Europe/Madrid
    {  513, 6346,  158987,  5, }, // Europe/Malta
    {  513, 6531,   97841,  8, }, // Europe/Mariehamn
    {  513, 3595,   61614,  5, }, // Europe/Minsk
    {  513, 6359,    6525,  6, }, // Europe/Monaco
    {  513, 3443,   21290,  6, }, // Europe/Moscow
    {  513, 6407,  158992,  4, }, // Europe/Oslo
    {  513, 4186,   32206,  5, }, // Europe/Paris
    {  513, 6373,  158996,  9, }, // Europe/Podgorica
    {  513, 6224,    6545,  4, }, // Europe/Prague
    {  513, 8816,  159005,  4, }, // Europe/Riga
    {  513, 6273,   32211,  3, }, // Europe/Rome
    {  513, 4233,  159009,  6, }, // Europe/Samara
    {  513, 6419,  159015, 10, }, // Europe/San_Marino
    {  513, 6194,  159025,  8, }, // Europe/Sarajevo
    {  513, 4353,   76841,  7, }, // Europe/Saratov
    {  513, 8828,   32214, 11, }, // Europe/Simferopol
    {  513, 6332,   32225,  7, }, // Europe/Skopje
    {  513, 6548,   32232,  6, }, // Europe/Sofia
    {  513, 6472,    6586,  8, }, // Europe/Stockholm
    {  513, 8846,    6594,  5, }, // Europe/Tallinn
    {  513, 6135,   19608,  6, }, // Europe/Tirane
    {  513, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  513, 6301,   61716,  5, }, // Europe/Vaduz
    {  513, 6516,    6614,  7, }, // Europe/Vatican
    {  513, 6164,   32246,  3, }, // Europe/Vienna
    {  513, 8878,  159033,  7, }, // Europe/Vilnius
    {  513, 4629,   61732,  9, }, // Europe/Volgograd
    {  513, 3055,   32249,  7, }, // Europe/Warsaw
    {  513, 6210,   61750,  6, }, // Europe/Zagreb
    {  513, 6489,   32256,  5, }, // Europe/Zurich
    {  513, 4980,  159040, 12, }, // Indian/Antananarivo
    {  513, 6910,  120014,  5, }, // Indian/Chagos
    {  513, 5979,   32261,  5, }, // Indian/Christmas
    {  513, 5996,    4484,  5, }, // Indian/Cocos
    {  513, 4931,   21395,  6, }, // Indian/Comoro
    {  513, 6624,   21401,  8, }, // Indian/Kerguelen
    {  513, 7223,  151415,  4, }, // Indian/Mahe
    {  513, 7013,   32290,  7, }, // Indian/Maldives
    {  513, 3971,   32297, 10, }, // Indian/Mauritius
    {  513, 5000,    6669,  5, }, // Indian/Mayotte
    {  513, 7174,   89867,  7, }, // Indian/Reunion
    {  513, 4324,  120112,  5, }, // Pacific/Apia
    {  513,  932,    6675,  6, }, // Pacific/Auckland
    {  513, 3608,  159052,  8, }, // Pacific/Bougainville
    {  513, 2863,  123242,  5, }, // Pacific/Chatham
    {  513, 3022,   89874,  4, }, // Pacific/Chuuk
    {  513, 2201,  159060, 15, }, // Pacific/Easter
    {  513, 7323,  123265,  4, }, // Pacific/Efate
    {  513, 7290,  159075,  7, }, // Pacific/Fakaofo
    {  513, 3804,   32307,  5, }, // Pacific/Fiji
    {  513, 7306,  123284,  8, }, // Pacific/Funafuti
    {  513, 6641,   90754,  9, }, // Pacific/Galapagos
    {  513, 6659,  159082,  7, }, // Pacific/Gambier
    {  513, 3724,  120188, 11, }, // Pacific/Guadalcanal
    {  513, 5935,  120200,  4, }, // Pacific/Guam
    {  513, 2929,    4512,  8, }, // Pacific/Honolulu
    {  513, 2897,  120205,  9, }, // Pacific/Kanton
    {  513, 3900,   90788, 10, }, // Pacific/Kiritimati
    {  513, 6982,  159089,  5, }, // Pacific/Kosrae
    {  513, 2708,  159094, 11, }, // Pacific/Kwajalein
    {  513, 4779,   90836,  6, }, // Pacific/Majuro
    {  513, 3953,   34820,  6, }, // Pacific/Marquesas
    {  513, 8909,  120264,  6, }, // Pacific/Midway
    {  513, 7047,  159105,  5, }, // Pacific/Nauru
    {  513, 7095,  159110,  5, }, // Pacific/Niue
    {  513, 4070,   61860,  7, }, // Pacific/Norfolk
    {  513, 7061,  120279,  5, }, // Pacific/Noumea
    {  513, 2991,  159115,  9, }, // Pacific/Pago_Pago
    {  513, 7118,   64169,  5, }, // Pacific/Palau
    {  513, 7157,    6773,  7, }, // Pacific/Pitcairn
    {  513, 2961,    6780,  6, }, // Pacific/Pohnpei
    {  513, 4709,  159124, 11, }, // Pacific/Port_Moresby
    {  513, 6009,  159135,  9, }, // Pacific/Rarotonga
    {  513, 5948,  159144,  6, }, // Pacific/Saipan
    {  513, 7275,  159150,  6, }, // Pacific/Tahiti
    {  513, 6675,   21574,  6, }, // Pacific/Tarawa
    {  513, 4485,  159156,  9, }, // Pacific/Tongatapu
    {  513, 7355,  159165,  4, }, // Pacific/Wake
    {  513, 7368,   32312,  5, }, // Pacific/Wallis
    {  514, 6751,   56229,  7, }, // Africa/Abidjan Serbian/Latin/Kosovo
    {  514, 6723,    4550,  4, }, // Africa/Accra
    {  514, 4961,   34301, 10, }, // Africa/Addis_Ababa
    {  514, 7383,   56099,  5, }, // Africa/Algiers
    {  514,   14,   89768,  6, }, // Africa/Asmara
    {  514,   44,  157946,  6, }, // Africa/Bamako
    {  514, 5159,  157952,  6, }, // Africa/Bangui
    {  514, 6709,  157958,  6, }, // Africa/Banjul
    {  514, 7398,  157964,  5, }, // Africa/Bissau
    {  514, 4859,    4588,  7, }, // Africa/Blantyre
    {  514, 5189,    4596,  8, }, // Africa/Brazzaville
    {  514, 4824,  157969, 10, }, // Africa/Bujumbura
    {  514, 2242,   24913,  5, }, // Africa/Cairo
    {  514, 4019,   31744, 10, }, // Africa/Casablanca
    {  514, 7412,   19754,  5, }, // Africa/Ceuta
    {  514, 6736,    4637,  7, }, // Africa/Conakry
    {  514, 6803,   59699,  5, }, // Africa/Dakar
    {  514, 5032,    4644, 12, }, // Africa/Dar_es_Salaam
    {  514, 4945,   31754,  7, }, // Africa/Djibouti
    {  514, 5145,   19777,  5, }, // Africa/Douala
    {  514, 5068,    4663,  7, }, // Africa/El_Aaiun
    {  514, 6816,    4670,  7, }, // Africa/Freetown
    {  514, 4808,   59741,  7, }, // Africa/Gaborone
    {  514, 4917,   59749,  6, }, // Africa/Harare
    {  514, 4381,    4677, 11, }, // Africa/Johannesburg
    {  514, 4401,  120501,  5, }, // Africa/Juba
    {  514, 5053,  157979,  7, }, // Africa/Kampala
    {  514, 4426,    4693,  6, }, // Africa/Khartoum
    {  514, 4875,  120515,  6, }, // Africa/Kigali
    {  514, 5208,  117752,  7, }, // Africa/Kinshasa
    {  514, 4646,   59800,  5, }, // Africa/Lagos
    {  514, 5238,    4699,  8, }, // Africa/Libreville
    {  514, 6832,   59815,  4, }, // Africa/Lome
    {  514, 5113,  157986,  6, }, // Africa/Luanda
    {  514, 4841,  117774,  9, }, // Africa/Lubumbashi
    {  514, 4903,  157992,  6, }, // Africa/Lusaka
    {  514, 5224,  157998,  6, }, // Africa/Malabo
    {  514, 4889,  158004,  6, }, // Africa/Maputo
    {  514, 5099,   59841,  6, }, // Africa/Maseru
    {  514, 5084,  158010,  7, }, // Africa/Mbabane
    {  514, 5015,   31761,  7, }, // Africa/Mogadishu
    {  514, 6997,  117797,  9, }, // Africa/Monrovia
    {  514, 3755,   59871,  7, }, // Africa/Nairobi
    {  514, 5173,   56262,  8, }, // Africa/Ndjamena
    {  514, 5256,  158017,  7, }, // Africa/Niamey
    {  514, 6766,   56270,  7, }, // Africa/Nouakchott
    {  514, 6690,   24967,  8, }, // Africa/Ouagadougou
    {  514, 5127,   24975, 10, }, // Africa/Porto-Novo
    {  514, 4337,   89774,  8, }, // Africa/Sao_Tome
    {  514, 2732,   34368,  7, }, // Africa/Tripoli
    {  514, 6503,   89940,  5, }, // Africa/Tunis
    {  514, 4054,   19877,  7, }, // Africa/Windhoek
    {  514,  132,   96487,  4, }, // America/Adak
    {  514, 3183,  158024,  8, }, // America/Anchorage
    {  514, 5524,   31768,  7, }, // America/Anguilla
    {  514, 5541,   31775,  7, }, // America/Antigua
    {  514, 4456,  120668, 10, }, // America/Araguaina
    {  514,  166,  158032, 12, }, // America/Argentina/Buenos_Aires
    {  514,   91,    4788,  9, }, // America/Argentina/Catamarca
    {  514,  270,    4797,  7, }, // America/Argentina/Cordoba
    {  514,  441,  158044,  5, }, // America/Argentina/Jujuy
    {  514, 7425,    4809,  8, }, // America/Argentina/La_Rioja
    {  514,  565,   96160,  7, }, // America/Argentina/Mendoza
    {  514, 7452,  120708, 12, }, // America/Argentina/Rio_Gallegos
    {  514, 7483,   59989,  5, }, // America/Argentina/Salta
    {  514, 7507,  158049,  8, }, // America/Argentina/San_Juan
    {  514, 5497,  158057,  7, }, // America/Argentina/San_Luis
    {  514, 7534,    4850,  7, }, // America/Argentina/Tucuman
    {  514, 7560,  158064,  7, }, // America/Argentina/Ushuaia
    {  514, 5557,  158071,  5, }, // America/Aruba
    {  514, 4154,    4869,  8, }, // America/Asuncion
    {  514,  237,  158076, 12, }, // America/Atikokan
    {  514, 3581,  117972,  5, }, // America/Bahia
    {  514, 7586,  158088, 14, }, // America/Bahia_Banderas
    {  514, 5571,  117990,  8, }, // America/Barbados
    {  514, 7609,   60040,  5, }, // America/Belem
    {  514, 5309,  158102,  6, }, // America/Belize
    {  514, 7623,  158108, 13, }, // America/Blanc-Sablon
    {  514, 7644,  158121,  9, }, // America/Boa_Vista
    {  514, 4263,  158130,  6, }, // America/Bogota
    {  514, 7662,  158136,  5, }, // America/Boise
    {  514, 7676,  158141, 12, }, // America/Cambridge_Bay
    {  514, 7698,   96188, 12, }, // America/Campo_Grande
    {  514, 3770,    4959,  6, }, // America/Cancun
    {  514, 4596,    4965,  7, }, // America/Caracas
    {  514, 4247,    4972,  5, }, // America/Cayenne
    {  514, 5398,  158153, 16, }, // America/Cayman
    {  514, 3251,  158169,  6, }, // America/Chicago
    {  514, 7719,  120907,  8, }, // America/Chihuahua
    {  514, 7737,  158175, 13, }, // America/Ciudad_Juarez
    {  514, 5324,   31782,  9, }, // America/Costa_Rica
    {  514, 7759,    5020,  7, }, // America/Creston
    {  514, 3693,   96220,  6, }, // America/Cuiaba
    {  514, 5640,    5033,  7, }, // America/Curacao
    {  514, 7775,  158188, 13, }, // America/Danmarkshavn
    {  514, 7796,  120962,  5, }, // America/Dawson
    {  514, 7811,  120969, 10, }, // America/Dawson_Creek
    {  514,  805,   60109,  6, }, // America/Denver
    {  514, 3351,  118170,  7, }, // America/Detroit
    {  514, 5656,   20071,  8, }, // America/Dominica
    {  514,  893,   60132,  8, }, // America/Edmonton
    {  514, 7832,  158201,  8, }, // America/Eirunepe
    {  514, 5343,    5090,  8, }, // America/El_Salvador
    {  514, 7849,  121026, 11, }, // America/Fort_Nelson
    {  514, 7869,  158209,  9, }, // America/Fortaleza
    {  514, 7887,  158218,  9, }, // America/Glace_Bay
    {  514, 6844,  158227,  7, }, // America/Goose_Bay
    {  514, 4514,  158234, 10, }, // America/Grand_Turk
    {  514, 5687,  158244,  7, }, // America/Grenada
    {  514, 5703,   31791,  9, }, // America/Guadeloupe
    {  514, 3662,   31800,  9, }, // America/Guatemala
    {  514, 4743,  118250,  8, }, // America/Guayaquil
    {  514, 6895,   56104,  7, }, // America/Guyana
    {  514, 1923,    5189,  8, }, // America/Halifax
    {  514, 2221,  140276,  6, }, // America/Havana
    {  514, 7905,  158251, 10, }, // America/Hermosillo
    {  514,  348,  118283, 12, }, // America/Indiana/Indianapolis
    {  514,  481,  158274, 14, }, // America/Indiana/Knox
    {  514, 7924,  158288, 17, }, // America/Indiana/Marengo
    {  514, 7948,  158305, 20, }, // America/Indiana/Petersburg
    {  514, 7975,  158325, 18, }, // America/Indiana/Tell_City
    {  514, 8001,  159169, 15, }, // America/Indiana/Vevay
    {  514, 8023,  159184, 18, }, // America/Indiana/Vincennes
    {  514, 8049,  158374, 17, }, // America/Indiana/Winamac
    {  514, 8073,  118413,  6, }, // America/Inuvik
    {  514,  660,  158391,  8, }, // America/Iqaluit
    {  514, 2665,   31809,  7, }, // America/Jamaica
    {  514, 5282,  158399,  4, }, // America/Juneau
    {  514,  521,  159202,  7, }, // America/Kentucky/Louisville
    {  514, 8088,  158409, 18, }, // America/Kentucky/Monticello
    {  514, 5621,  158427, 10, }, // America/Kralendijk
    {  514, 4278,  158437,  6, }, // America/La_Paz
    {  514, 7132,   20446,  4, }, // America/Lima
    {  514, 3390,  158443, 11, }, // America/Los_Angeles
    {  514, 5849,  158454, 21, }, // America/Lower_Princes
    {  514, 8116,  118506,  6, }, // America/Maceio
    {  514, 8131,  118512,  7, }, // America/Managua
    {  514, 1892,  118519,  6, }, // America/Manaus
    {  514, 5814,   60430,  6, }, // America/Marigot
    {  514, 5722,    5400,  8, }, // America/Martinique
    {  514, 8147,  118534,  9, }, // America/Matamoros
    {  514, 2779,  158475,  8, }, // America/Mazatlan
    {  514, 8165,    5409,  8, }, // America/Menominee
    {  514, 8183,    3899,  6, }, // America/Merida
    {  514, 8198,  158483, 10, }, // America/Metlakatla
    {  514, 2811,  158493, 12, }, // America/Mexico_City
    {  514, 4293,    5434,  7, }, // America/Miquelon
    {  514, 8217,    5441,  7, }, // America/Moncton
    {  514, 8233,   60479,  8, }, // America/Monterrey
    {  514, 4000,  158505, 10, }, // America/Montevideo
    {  514, 5741,    5457,  9, }, // America/Montserrat
    {  514, 5383,  121439,  5, }, // America/Nassau
    {  514, 3294,  158515,  7, }, // America/New_York
    {  514, 8251,   20373,  3, }, // America/Nome
    {  514, 1834,    5479,  7, }, // America/Noronha
    {  514, 8264,  159209, 22, }, // America/North_Dakota/Beulah
    {  514, 8292,  159231, 22, }, // America/North_Dakota/Center
    {  514, 8320,  159253, 26, }, // America/North_Dakota/New_Salem
    {  514,  393,  158594,  6, }, // America/Nuuk
    {  514, 8351,    5561,  7, }, // America/Ojinaga
    {  514, 5413,  158600,  6, }, // America/Panama
    {  514, 6106,  158606, 10, }, // America/Paramaribo
    {  514, 3224,  121555,  6, }, // America/Phoenix
    {  514, 3847,  121563, 12, }, // America/Port-au-Prince
    {  514, 5871,  159279, 13, }, // America/Port_of_Spain
    {  514, 8367,  158641, 11, }, // America/Porto_Velho
    {  514, 5760,  159292, 10, }, // America/Puerto_Rico
    {  514, 3932,  121616, 12, }, // America/Punta_Arenas
    {  514, 8387,  121630, 12, }, // America/Rankin_Inlet
    {  514, 8408,  158652,  6, }, // America/Recife
    {  514, 1979,  159302,  6, }, // America/Regina
    {  514, 8423,  121657,  7, }, // America/Resolute
    {  514,  695,   20506, 10, }, // America/Rio_Branco
    {  514, 8440,  158673,  8, }, // America/Santarem
    {  514, 2165,  118795,  8, }, // America/Santiago
    {  514, 6044,  158681, 13, }, // America/Santo_Domingo
    {  514, 1862,  158694,  9, }, // America/Sao_Paulo
    {  514, 6862,  159308, 12, }, // America/Scoresbysund
    {  514, 8457,  158703,  5, }, // America/Sitka
    {  514, 8471,  159320, 14, }, // America/St_Barthelemy
    {  514, 2045,  159334,  8, }, // America/St_Johns
    {  514, 5780,  121759,  9, }, // America/St_Kitts
    {  514, 5797,  158708, 10, }, // America/St_Lucia
    {  514,  855,  159342,  8, }, // America/St_Thomas
    {  514, 5830,  158718, 12, }, // America/St_Vincent
    {  514, 8493,   20619, 12, }, // America/Swift_Current
    {  514, 5363,    5759, 11, }, // America/Tegucigalpa
    {  514, 5673,   20642,  3, }, // America/Thule
    {  514,  313,    5775,  7, }, // America/Tijuana
    {  514,  608,  158730,  7, }, // America/Toronto
    {  514, 5605,  158737,  7, }, // America/Tortola
    {  514, 2077,    5789,  8, }, // America/Vancouver
    {  514, 2128,  158744,  8, }, // America/Whitehorse
    {  514,  734,   60758,  7, }, // America/Winnipeg
    {  514, 8515,    5813,  7, }, // America/Yakutat
    {  514, 5918,    5820,  5, }, // Antarctica/Casey
    {  514, 6027,    5825,  6, }, // Antarctica/Davis
    {  514, 6066,  159350, 13, }, // Antarctica/DumontDUrville
    {  514, 8531,  159363,  7, }, // Antarctica/Macquarie
    {  514, 7029,  118978,  5, }, // Antarctica/Mawson
    {  514, 7076,  158765,  8, }, // Antarctica/McMurdo
    {  514, 8552,  121938,  6, }, // Antarctica/Palmer
    {  514, 7189,  118999,  6, }, // Antarctica/Rothera
    {  514, 7258,  119005,  4, }, // Antarctica/Syowa
    {  514, 8570,  119009,  4, }, // Antarctica/Troll
    {  514, 7337,  119015,  6, }, // Antarctica/Vostok
    {  514, 1413,  159370, 11, }, // Arctic/Longyearbyen
    {  514, 5487,   60791,  4, }, // Asia/Aden
    {  514, 5297,    5890,  6, }, // Asia/Almaty
    {  514, 3870,    5896,  4, }, // Asia/Amman
    {  514, 5428,    5900,  6, }, // Asia/Anadyr
    {  514, 5440,    5906,  5, }, // Asia/Aqtau
    {  514, 5270,   31897,  7, }, // Asia/Aqtobe
    {  514,  964,   31904,  7, }, // Asia/Ashgabat
    {  514, 8587,    5917,  6, }, // Asia/Atyrau
    {  514, 3509,    4063,  6, }, // Asia/Baghdad
    {  514, 5451,    4069,  7, }, // Asia/Bahrain
    {  514, 3555,   60829,  4, }, // Asia/Baku
    {  514, 4368,  119081,  7, }, // Asia/Bangkok
    {  514, 3473,  119089,  7, }, // Asia/Barnaul
    {  514, 3988,    5930,  6, }, // Asia/Beirut
    {  514, 3680,   31917,  6, }, // Asia/Bishkek
    {  514, 5906,   31923,  6, }, // Asia/Brunei
    {  514, 4503,   56142,  4, }, // Asia/Chita
    {  514, 5963,   31930,  9, }, // Asia/Choibalsan
    {  514, 4413,    5949,  7, }, // Asia/Colombo
    {  514, 4442,    4102,  6, }, // Asia/Damascus
    {  514, 1060,    5956,  4, }, // Asia/Dhaka
    {  514, 6125,  122105,  4, }, // Asia/Dili
    {  514, 3498,  119135,  5, }, // Asia/Dubai
    {  514, 6092,   31939,  7, }, // Asia/Dushanbe
    {  514, 8599,  158780,  9, }, // Asia/Famagusta
    {  514, 8614,   60896,  4, }, // Asia/Gaza
    {  514, 4697,   60905,  6, }, // Asia/Hebron
    {  514, 1226,   31946,  9, }, // Asia/Ho_Chi_Minh
    {  514, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  514, 4673,  119165,  4, }, // Asia/Hovd
    {  514, 4086,  139633,  6, }, // Asia/Irkutsk
    {  514, 6969,   31955,  8, }, // Asia/Jakarta
    {  514, 6955,   31963,  9, }, // Asia/Jayapura
    {  514, 1257,  158789,  9, }, // Asia/Jerusalem
    {  514, 3462,  119188,  5, }, // Asia/Kabul
    {  514, 4218,   31981,  8, }, // Asia/Kamchatka
    {  514, 4141,   31989,  6, }, // Asia/Karachi
    {  514, 1152,    6018,  8, }, // Asia/Kathmandu
    {  514, 8624,   31995,  7, }, // Asia/Khandyga
    {  514,  992,   25307,  7, }, // Asia/Kolkata
    {  514, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  514, 4761,  158798, 12, }, // Asia/Kuala_Lumpur
    {  514, 5893,   32002,  6, }, // Asia/Kuching
    {  514, 5464,    6058,  6, }, // Asia/Kuwait
    {  514, 1178,    6064,  5, }, // Asia/Macau
    {  514, 3919,  119244,  7, }, // Asia/Magadan
    {  514, 1316,    6069,  7, }, // Asia/Makassar
    {  514, 7145,  158810,  6, }, // Asia/Manila
    {  514, 6883,    4227,  6, }, // Asia/Muscat
    {  514, 2446,   32013,  8, }, // Asia/Nicosia
    {  514, 8638,   32021, 12, }, // Asia/Novokuznetsk
    {  514, 4037,   61036, 11, }, // Asia/Novosibirsk
    {  514, 4131,   61048,  4, }, // Asia/Omsk
    {  514, 7108,   61053,  4, }, // Asia/Oral
    {  514, 6924,   20928,  8, }, // Asia/Phnom_Penh
    {  514, 8656,  158816, 10, }, // Asia/Pontianak
    {  514, 4116,   32033,  9, }, // Asia/Pyongyang
    {  514, 5476,    4233,  5, }, // Asia/Qatar
    {  514, 8671,    6103,  8, }, // Asia/Qostanay
    {  514, 4171,    6111,  9, }, // Asia/Qyzylorda
    {  514, 3486,   32042,  5, }, // Asia/Riyadh
    {  514, 4310,   32047,  7, }, // Asia/Sakhalin
    {  514, 7208,   25356,  9, }, // Asia/Samarkand
    {  514, 3116,    6131,  4, }, // Asia/Seoul
    {  514, 1020,   32054,  6, }, // Asia/Shanghai
    {  514, 3137,   19291,  8, }, // Asia/Singapore
    {  514, 4199,  158826, 14, }, // Asia/Srednekolymsk
    {  514, 3100,    6163,  6, }, // Asia/Taipei
    {  514, 4683,   32060,  7, }, // Asia/Tashkent
    {  514, 3817,  122458,  7, }, // Asia/Tbilisi
    {  514, 2638,    6175,  7, }, // Asia/Tehran
    {  514, 1284,   90291,  5, }, // Asia/Thimphu
    {  514, 2687,    4257,  5, }, // Asia/Tokyo
    {  514, 4474,  119419,  5, }, // Asia/Tomsk
    {  514, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  514, 1126,  158840,  6, }, // Asia/Urumqi
    {  514, 8685,  158846,  8, }, // Asia/Ust-Nera
    {  514, 6940,   32073, 10, }, // Asia/Vientiane
    {  514, 4612,   61189, 11, }, // Asia/Vladivostok
    {  514, 4730,    6206,  7, }, // Asia/Yakutsk
    {  514, 1202,    6213,  6, }, // Asia/Yangon
    {  514, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  514, 3649,    6231,  7, }, // Asia/Yerevan
    {  514, 3565,   32083,  5, }, // Atlantic/Azores
    {  514, 5588,   72942,  7, }, // Atlantic/Bermuda
    {  514, 6591,  158854, 15, }, // Atlantic/Canary
    {  514, 3629,  158869, 18, }, // Atlantic/Cape_Verde
    {  514, 1379,  158887, 13, }, // Atlantic/Faroe
    {  514, 8699,  158900,  7, }, // Atlantic/Madeira
    {  514, 2614,   32088,  9, }, // Atlantic/Reykjavik
    {  514, 7235,  158907, 15, }, // Atlantic/South_Georgia
    {  514, 6784,  158922, 12, }, // Atlantic/St_Helena
    {  514, 6607,    6298,  6, }, // Atlantic/Stanley
    {  514, 1654,  158934,  7, }, // Australia/Adelaide
    {  514, 1619,  158941,  8, }, // Australia/Brisbane
    {  514, 1783,  122675, 10, }, // Australia/Broken_Hill
    {  514, 1567,    6331,  6, }, // Australia/Darwin
    {  514, 3539,  158949,  5, }, // Australia/Eucla
    {  514, 1500,   61346,  6, }, // Australia/Hobart
    {  514, 8716,  119621,  8, }, // Australia/Lindeman
    {  514, 1531,   21137,  8, }, // Australia/Lord_Howe
    {  514, 1711,    6359,  7, }, // Australia/Melbourne
    {  514, 1746,   21145,  4, }, // Australia/Perth
    {  514, 1447,    6371,  6, }, // Australia/Sydney
    {  514, 6390,   61373,  9, }, // Europe/Amsterdam
    {  514, 6149,   32109,  6, }, // Europe/Andorra
    {  514, 3522,    6384,  8, }, // Europe/Astrakhan
    {  514, 6577,   32123,  5, }, // Europe/Athens
    {  514, 6285,    6398,  7, }, // Europe/Belgrade
    {  514, 4659,   61411,  6, }, // Europe/Berlin
    {  514, 6437,  158954, 10, }, // Europe/Bratislava
    {  514, 6178,   32128,  6, }, // Europe/Brussels
    {  514, 3830,   32134,  8, }, // Europe/Bucharest
    {  514, 3708,   32142, 10, }, // Europe/Budapest
    {  514, 8751,  158964,  8, }, // Europe/Busingen
    {  514, 2475,   32152,  8, }, // Europe/Chisinau
    {  514, 6238,    4376, 10, }, // Europe/Copenhagen
    {  514, 2260,   32160,  6, }, // Europe/Dublin
    {  514, 6256,  122861,  9, }, // Europe/Gibraltar
    {  514, 8767,   32166,  6, }, // Europe/Guernsey
    {  514, 6561,  122879,  8, }, // Europe/Helsinki
    {  514, 8783,   32172, 10, }, // Europe/Isle_of_Man
    {  514, 1097,   61535,  8, }, // Europe/Istanbul
    {  514, 8802,  158972,  6, }, // Europe/Jersey
    {  514, 3881,   32182, 12, }, // Europe/Kaliningrad
    {  514, 8924,  119814,  5, }, // Europe/Kirov
    {  514, 2419,   32194,  5, }, // Europe/Kyiv
    {  514, 3078,   32199,  7, }, // Europe/Lisbon
    {  514, 6455,  158978,  9, }, // Europe/Ljubljana
    {  514, 2393,   61584,  6, }, // Europe/London
    {  514, 6314,    6510, 10, }, // Europe/Luxembourg
    {  514, 4794,   61602,  6, }, // Europe/Madrid
    {  514, 6346,  158987,  5, }, // Europe/Malta
    {  514, 6531,   97841,  8, }, // Europe/Mariehamn
    {  514, 3595,   61614,  5, }, // Europe/Minsk
    {  514, 6359,    6525,  6, }, // Europe/Monaco
    {  514, 3443,   21290,  6, }, // Europe/Moscow
    {  514, 6407,  158992,  4, }, // Europe/Oslo
    {  514, 4186,   32206,  5, }, // Europe/Paris
    {  514, 6373,  158996,  9, }, // Europe/Podgorica
    {  514, 6224,    6545,  4, }, // Europe/Prague
    {  514, 8816,  159005,  4, }, // Europe/Riga
    {  514, 6273,   32211,  3, }, // Europe/Rome
    {  514, 4233,  159009,  6, }, // Europe/Samara
    {  514, 6419,  159015, 10, }, // Europe/San_Marino
    {  514, 6194,  159025,  8, }, // Europe/Sarajevo
    {  514, 4353,   76841,  7, }, // Europe/Saratov
    {  514, 8828,   32214, 11, }, // Europe/Simferopol
    {  514, 6332,   32225,  7, }, // Europe/Skopje
    {  514, 6548,   32232,  6, }, // Europe/Sofia
    {  514, 6472,    6586,  8, }, // Europe/Stockholm
    {  514, 8846,    6594,  5, }, // Europe/Tallinn
    {  514, 6135,   19608,  6, }, // Europe/Tirane
    {  514, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  514, 6301,   61716,  5, }, // Europe/Vaduz
    {  514, 6516,    6614,  7, }, // Europe/Vatican
    {  514, 6164,   32246,  3, }, // Europe/Vienna
    {  514, 8878,  159033,  7, }, // Europe/Vilnius
    {  514, 4629,   61732,  9, }, // Europe/Volgograd
    {  514, 3055,   32249,  7, }, // Europe/Warsaw
    {  514, 6210,   61750,  6, }, // Europe/Zagreb
    {  514, 6489,   32256,  5, }, // Europe/Zurich
    {  514, 4980,  159040, 12, }, // Indian/Antananarivo
    {  514, 6910,  120014,  5, }, // Indian/Chagos
    {  514, 5979,   32261,  5, }, // Indian/Christmas
    {  514, 5996,    4484,  5, }, // Indian/Cocos
    {  514, 4931,   21395,  6, }, // Indian/Comoro
    {  514, 6624,   21401,  8, }, // Indian/Kerguelen
    {  514, 7223,  151415,  4, }, // Indian/Mahe
    {  514, 7013,   32290,  7, }, // Indian/Maldives
    {  514, 3971,   32297, 10, }, // Indian/Mauritius
    {  514, 5000,    6669,  5, }, // Indian/Mayotte
    {  514, 7174,   89867,  7, }, // Indian/Reunion
    {  514, 4324,  120112,  5, }, // Pacific/Apia
    {  514,  932,    6675,  6, }, // Pacific/Auckland
    {  514, 3608,  159052,  8, }, // Pacific/Bougainville
    {  514, 2863,  123242,  5, }, // Pacific/Chatham
    {  514, 3022,   89874,  4, }, // Pacific/Chuuk
    {  514, 2201,  159060, 15, }, // Pacific/Easter
    {  514, 7323,  123265,  4, }, // Pacific/Efate
    {  514, 7290,  159075,  7, }, // Pacific/Fakaofo
    {  514, 3804,   32307,  5, }, // Pacific/Fiji
    {  514, 7306,  123284,  8, }, // Pacific/Funafuti
    {  514, 6641,   90754,  9, }, // Pacific/Galapagos
    {  514, 6659,  159082,  7, }, // Pacific/Gambier
    {  514, 3724,  120188, 11, }, // Pacific/Guadalcanal
    {  514, 5935,  120200,  4, }, // Pacific/Guam
    {  514, 2929,    4512,  8, }, // Pacific/Honolulu
    {  514, 2897,  120205,  9, }, // Pacific/Kanton
    {  514, 3900,   90788, 10, }, // Pacific/Kiritimati
    {  514, 6982,  159089,  5, }, // Pacific/Kosrae
    {  514, 2708,  159094, 11, }, // Pacific/Kwajalein
    {  514, 4779,   90836,  6, }, // Pacific/Majuro
    {  514, 3953,   34820,  6, }, // Pacific/Marquesas
    {  514, 8909,  120264,  6, }, // Pacific/Midway
    {  514, 7047,  159105,  5, }, // Pacific/Nauru
    {  514, 7095,  159381,  4, }, // Pacific/Niue
    {  514, 4070,   61860,  7, }, // Pacific/Norfolk
    {  514, 7061,  120279,  5, }, // Pacific/Noumea
    {  514, 2991,  159115,  9, }, // Pacific/Pago_Pago
    {  514, 7118,   64169,  5, }, // Pacific/Palau
    {  514, 7157,    6773,  7, }, // Pacific/Pitcairn
    {  514, 2961,    6780,  6, }, // Pacific/Pohnpei
    {  514, 4709,  159124, 11, }, // Pacific/Port_Moresby
    {  514, 6009,  159135,  9, }, // Pacific/Rarotonga
    {  514, 5948,  159144,  6, }, // Pacific/Saipan
    {  514, 7275,  159150,  6, }, // Pacific/Tahiti
    {  514, 6675,   21574,  6, }, // Pacific/Tarawa
    {  514, 4485,  159156,  9, }, // Pacific/Tongatapu
    {  514, 7355,  159165,  4, }, // Pacific/Wake
    {  514, 7368,   32312,  5, }, // Pacific/Wallis
    {  515, 6751,   56229,  7, }, // Africa/Abidjan Serbian/Latin/Montenegro
    {  515, 6723,    4550,  4, }, // Africa/Accra
    {  515, 4961,   34301, 10, }, // Africa/Addis_Ababa
    {  515, 7383,   56099,  5, }, // Africa/Algiers
    {  515,   14,   89768,  6, }, // Africa/Asmara
    {  515,   44,  157946,  6, }, // Africa/Bamako
    {  515, 5159,  157952,  6, }, // Africa/Bangui
    {  515, 6709,  157958,  6, }, // Africa/Banjul
    {  515, 7398,  157964,  5, }, // Africa/Bissau
    {  515, 4859,    4588,  7, }, // Africa/Blantyre
    {  515, 5189,    4596,  8, }, // Africa/Brazzaville
    {  515, 4824,  157969, 10, }, // Africa/Bujumbura
    {  515, 2242,   24913,  5, }, // Africa/Cairo
    {  515, 4019,   31744, 10, }, // Africa/Casablanca
    {  515, 7412,   19754,  5, }, // Africa/Ceuta
    {  515, 6736,    4637,  7, }, // Africa/Conakry
    {  515, 6803,   59699,  5, }, // Africa/Dakar
    {  515, 5032,    4644, 12, }, // Africa/Dar_es_Salaam
    {  515, 4945,   31754,  7, }, // Africa/Djibouti
    {  515, 5145,   19777,  5, }, // Africa/Douala
    {  515, 5068,    4663,  7, }, // Africa/El_Aaiun
    {  515, 6816,    4670,  7, }, // Africa/Freetown
    {  515, 4808,   59741,  7, }, // Africa/Gaborone
    {  515, 4917,   59749,  6, }, // Africa/Harare
    {  515, 4381,    4677, 11, }, // Africa/Johannesburg
    {  515, 4401,  120501,  5, }, // Africa/Juba
    {  515, 5053,  157979,  7, }, // Africa/Kampala
    {  515, 4426,    4693,  6, }, // Africa/Khartoum
    {  515, 4875,  120515,  6, }, // Africa/Kigali
    {  515, 5208,  117752,  7, }, // Africa/Kinshasa
    {  515, 4646,   59800,  5, }, // Africa/Lagos
    {  515, 5238,    4699,  8, }, // Africa/Libreville
    {  515, 6832,   59815,  4, }, // Africa/Lome
    {  515, 5113,  157986,  6, }, // Africa/Luanda
    {  515, 4841,  117774,  9, }, // Africa/Lubumbashi
    {  515, 4903,  157992,  6, }, // Africa/Lusaka
    {  515, 5224,  157998,  6, }, // Africa/Malabo
    {  515, 4889,  158004,  6, }, // Africa/Maputo
    {  515, 5099,   59841,  6, }, // Africa/Maseru
    {  515, 5084,  158010,  7, }, // Africa/Mbabane
    {  515, 5015,   31761,  7, }, // Africa/Mogadishu
    {  515, 6997,  117797,  9, }, // Africa/Monrovia
    {  515, 3755,   59871,  7, }, // Africa/Nairobi
    {  515, 5173,   56262,  8, }, // Africa/Ndjamena
    {  515, 5256,  158017,  7, }, // Africa/Niamey
    {  515, 6766,   56270,  7, }, // Africa/Nouakchott
    {  515, 6690,   24967,  8, }, // Africa/Ouagadougou
    {  515, 5127,   24975, 10, }, // Africa/Porto-Novo
    {  515, 4337,   89774,  8, }, // Africa/Sao_Tome
    {  515, 2732,   34368,  7, }, // Africa/Tripoli
    {  515, 6503,   89940,  5, }, // Africa/Tunis
    {  515, 4054,   19877,  7, }, // Africa/Windhoek
    {  515,  132,   96487,  4, }, // America/Adak
    {  515, 3183,  158024,  8, }, // America/Anchorage
    {  515, 5524,   31768,  7, }, // America/Anguilla
    {  515, 5541,   31775,  7, }, // America/Antigua
    {  515, 4456,  120668, 10, }, // America/Araguaina
    {  515,  166,  158032, 12, }, // America/Argentina/Buenos_Aires
    {  515,   91,    4788,  9, }, // America/Argentina/Catamarca
    {  515,  270,    4797,  7, }, // America/Argentina/Cordoba
    {  515,  441,  158044,  5, }, // America/Argentina/Jujuy
    {  515, 7425,    4809,  8, }, // America/Argentina/La_Rioja
    {  515,  565,   96160,  7, }, // America/Argentina/Mendoza
    {  515, 7452,  120708, 12, }, // America/Argentina/Rio_Gallegos
    {  515, 7483,   59989,  5, }, // America/Argentina/Salta
    {  515, 7507,  158049,  8, }, // America/Argentina/San_Juan
    {  515, 5497,  158057,  7, }, // America/Argentina/San_Luis
    {  515, 7534,    4850,  7, }, // America/Argentina/Tucuman
    {  515, 7560,  158064,  7, }, // America/Argentina/Ushuaia
    {  515, 5557,  158071,  5, }, // America/Aruba
    {  515, 4154,    4869,  8, }, // America/Asuncion
    {  515,  237,  158076, 12, }, // America/Atikokan
    {  515, 3581,  117972,  5, }, // America/Bahia
    {  515, 7586,  158088, 14, }, // America/Bahia_Banderas
    {  515, 5571,  117990,  8, }, // America/Barbados
    {  515, 7609,   60040,  5, }, // America/Belem
    {  515, 5309,  158102,  6, }, // America/Belize
    {  515, 7623,  158108, 13, }, // America/Blanc-Sablon
    {  515, 7644,  158121,  9, }, // America/Boa_Vista
    {  515, 4263,  158130,  6, }, // America/Bogota
    {  515, 7662,  158136,  5, }, // America/Boise
    {  515, 7676,  158141, 12, }, // America/Cambridge_Bay
    {  515, 7698,   96188, 12, }, // America/Campo_Grande
    {  515, 3770,    4959,  6, }, // America/Cancun
    {  515, 4596,    4965,  7, }, // America/Caracas
    {  515, 4247,    4972,  5, }, // America/Cayenne
    {  515, 5398,  158153, 16, }, // America/Cayman
    {  515, 3251,  158169,  6, }, // America/Chicago
    {  515, 7719,  120907,  8, }, // America/Chihuahua
    {  515, 7737,  158175, 13, }, // America/Ciudad_Juarez
    {  515, 5324,   31782,  9, }, // America/Costa_Rica
    {  515, 7759,    5020,  7, }, // America/Creston
    {  515, 3693,   96220,  6, }, // America/Cuiaba
    {  515, 5640,    5033,  7, }, // America/Curacao
    {  515, 7775,  158188, 13, }, // America/Danmarkshavn
    {  515, 7796,  120962,  5, }, // America/Dawson
    {  515, 7811,  120969, 10, }, // America/Dawson_Creek
    {  515,  805,   60109,  6, }, // America/Denver
    {  515, 3351,  118170,  7, }, // America/Detroit
    {  515, 5656,   20071,  8, }, // America/Dominica
    {  515,  893,   60132,  8, }, // America/Edmonton
    {  515, 7832,  158201,  8, }, // America/Eirunepe
    {  515, 5343,    5090,  8, }, // America/El_Salvador
    {  515, 7849,  121026, 11, }, // America/Fort_Nelson
    {  515, 7869,  158209,  9, }, // America/Fortaleza
    {  515, 7887,  158218,  9, }, // America/Glace_Bay
    {  515, 6844,  158227,  7, }, // America/Goose_Bay
    {  515, 4514,  158234, 10, }, // America/Grand_Turk
    {  515, 5687,  158244,  7, }, // America/Grenada
    {  515, 5703,   31791,  9, }, // America/Guadeloupe
    {  515, 3662,   31800,  9, }, // America/Guatemala
    {  515, 4743,  118250,  8, }, // America/Guayaquil
    {  515, 6895,   56104,  7, }, // America/Guyana
    {  515, 1923,    5189,  8, }, // America/Halifax
    {  515, 2221,  140276,  6, }, // America/Havana
    {  515, 7905,  158251, 10, }, // America/Hermosillo
    {  515,  348,  118283, 12, }, // America/Indiana/Indianapolis
    {  515,  481,  158274, 14, }, // America/Indiana/Knox
    {  515, 7924,  158288, 17, }, // America/Indiana/Marengo
    {  515, 7948,  158305, 20, }, // America/Indiana/Petersburg
    {  515, 7975,  158325, 18, }, // America/Indiana/Tell_City
    {  515, 8001,  159169, 15, }, // America/Indiana/Vevay
    {  515, 8023,  159184, 18, }, // America/Indiana/Vincennes
    {  515, 8049,  158374, 17, }, // America/Indiana/Winamac
    {  515, 8073,  118413,  6, }, // America/Inuvik
    {  515,  660,  158391,  8, }, // America/Iqaluit
    {  515, 2665,   31809,  7, }, // America/Jamaica
    {  515, 5282,  158399,  4, }, // America/Juneau
    {  515,  521,  159202,  7, }, // America/Kentucky/Louisville
    {  515, 8088,  158409, 18, }, // America/Kentucky/Monticello
    {  515, 5621,  158427, 10, }, // America/Kralendijk
    {  515, 4278,  158437,  6, }, // America/La_Paz
    {  515, 7132,   20446,  4, }, // America/Lima
    {  515, 3390,  158443, 11, }, // America/Los_Angeles
    {  515, 5849,  158454, 21, }, // America/Lower_Princes
    {  515, 8116,  118506,  6, }, // America/Maceio
    {  515, 8131,  118512,  7, }, // America/Managua
    {  515, 1892,  118519,  6, }, // America/Manaus
    {  515, 5814,   60430,  6, }, // America/Marigot
    {  515, 5722,    5400,  8, }, // America/Martinique
    {  515, 8147,  118534,  9, }, // America/Matamoros
    {  515, 2779,  158475,  8, }, // America/Mazatlan
    {  515, 8165,    5409,  8, }, // America/Menominee
    {  515, 8183,    3899,  6, }, // America/Merida
    {  515, 8198,  158483, 10, }, // America/Metlakatla
    {  515, 2811,  158493, 12, }, // America/Mexico_City
    {  515, 4293,    5434,  7, }, // America/Miquelon
    {  515, 8217,    5441,  7, }, // America/Moncton
    {  515, 8233,   60479,  8, }, // America/Monterrey
    {  515, 4000,  158505, 10, }, // America/Montevideo
    {  515, 5741,    5457,  9, }, // America/Montserrat
    {  515, 5383,  121439,  5, }, // America/Nassau
    {  515, 3294,  158515,  7, }, // America/New_York
    {  515, 8251,   20373,  3, }, // America/Nome
    {  515, 1834,    5479,  7, }, // America/Noronha
    {  515, 8264,  159209, 22, }, // America/North_Dakota/Beulah
    {  515, 8292,  159231, 22, }, // America/North_Dakota/Center
    {  515, 8320,  159253, 26, }, // America/North_Dakota/New_Salem
    {  515,  393,  158594,  6, }, // America/Nuuk
    {  515, 8351,    5561,  7, }, // America/Ojinaga
    {  515, 5413,  158600,  6, }, // America/Panama
    {  515, 6106,  158606, 10, }, // America/Paramaribo
    {  515, 3224,  121555,  6, }, // America/Phoenix
    {  515, 3847,  121563, 12, }, // America/Port-au-Prince
    {  515, 5871,  159279, 13, }, // America/Port_of_Spain
    {  515, 8367,  158641, 11, }, // America/Porto_Velho
    {  515, 5760,  159292, 10, }, // America/Puerto_Rico
    {  515, 3932,  121616, 12, }, // America/Punta_Arenas
    {  515, 8387,  121630, 12, }, // America/Rankin_Inlet
    {  515, 8408,  158652,  6, }, // America/Recife
    {  515, 1979,  159302,  6, }, // America/Regina
    {  515, 8423,  121657,  7, }, // America/Resolute
    {  515,  695,   20506, 10, }, // America/Rio_Branco
    {  515, 8440,  158673,  8, }, // America/Santarem
    {  515, 2165,  118795,  8, }, // America/Santiago
    {  515, 6044,  158681, 13, }, // America/Santo_Domingo
    {  515, 1862,  158694,  9, }, // America/Sao_Paulo
    {  515, 6862,  159308, 12, }, // America/Scoresbysund
    {  515, 8457,  158703,  5, }, // America/Sitka
    {  515, 8471,  159320, 14, }, // America/St_Barthelemy
    {  515, 2045,  159334,  8, }, // America/St_Johns
    {  515, 5780,  121759,  9, }, // America/St_Kitts
    {  515, 5797,  158708, 10, }, // America/St_Lucia
    {  515,  855,  159342,  8, }, // America/St_Thomas
    {  515, 5830,  158718, 12, }, // America/St_Vincent
    {  515, 8493,   20619, 12, }, // America/Swift_Current
    {  515, 5363,    5759, 11, }, // America/Tegucigalpa
    {  515, 5673,   20642,  3, }, // America/Thule
    {  515,  313,    5775,  7, }, // America/Tijuana
    {  515,  608,  158730,  7, }, // America/Toronto
    {  515, 5605,  158737,  7, }, // America/Tortola
    {  515, 2077,    5789,  8, }, // America/Vancouver
    {  515, 2128,  158744,  8, }, // America/Whitehorse
    {  515,  734,   60758,  7, }, // America/Winnipeg
    {  515, 8515,    5813,  7, }, // America/Yakutat
    {  515, 5918,    5820,  5, }, // Antarctica/Casey
    {  515, 6027,    5825,  6, }, // Antarctica/Davis
    {  515, 6066,  159350, 13, }, // Antarctica/DumontDUrville
    {  515, 8531,  159363,  7, }, // Antarctica/Macquarie
    {  515, 7029,  118978,  5, }, // Antarctica/Mawson
    {  515, 7076,  158765,  8, }, // Antarctica/McMurdo
    {  515, 8552,  121938,  6, }, // Antarctica/Palmer
    {  515, 7189,  118999,  6, }, // Antarctica/Rothera
    {  515, 7258,  119005,  4, }, // Antarctica/Syowa
    {  515, 8570,  119009,  4, }, // Antarctica/Troll
    {  515, 7337,  119015,  6, }, // Antarctica/Vostok
    {  515, 1413,  159370, 11, }, // Arctic/Longyearbyen
    {  515, 5487,   60791,  4, }, // Asia/Aden
    {  515, 5297,    5890,  6, }, // Asia/Almaty
    {  515, 3870,    5896,  4, }, // Asia/Amman
    {  515, 5428,    5900,  6, }, // Asia/Anadyr
    {  515, 5440,    5906,  5, }, // Asia/Aqtau
    {  515, 5270,   31897,  7, }, // Asia/Aqtobe
    {  515,  964,   31904,  7, }, // Asia/Ashgabat
    {  515, 8587,    5917,  6, }, // Asia/Atyrau
    {  515, 3509,    4063,  6, }, // Asia/Baghdad
    {  515, 5451,    4069,  7, }, // Asia/Bahrain
    {  515, 3555,   60829,  4, }, // Asia/Baku
    {  515, 4368,  119081,  7, }, // Asia/Bangkok
    {  515, 3473,  119089,  7, }, // Asia/Barnaul
    {  515, 3988,    5930,  6, }, // Asia/Beirut
    {  515, 3680,   31917,  6, }, // Asia/Bishkek
    {  515, 5906,   31923,  6, }, // Asia/Brunei
    {  515, 4503,   56142,  4, }, // Asia/Chita
    {  515, 5963,   31930,  9, }, // Asia/Choibalsan
    {  515, 4413,    5949,  7, }, // Asia/Colombo
    {  515, 4442,    4102,  6, }, // Asia/Damascus
    {  515, 1060,    5956,  4, }, // Asia/Dhaka
    {  515, 6125,  122105,  4, }, // Asia/Dili
    {  515, 3498,  119135,  5, }, // Asia/Dubai
    {  515, 6092,   31939,  7, }, // Asia/Dushanbe
    {  515, 8599,  158780,  9, }, // Asia/Famagusta
    {  515, 8614,   60896,  4, }, // Asia/Gaza
    {  515, 4697,   60905,  6, }, // Asia/Hebron
    {  515, 1226,   31946,  9, }, // Asia/Ho_Chi_Minh
    {  515, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  515, 4673,  119165,  4, }, // Asia/Hovd
    {  515, 4086,  139633,  6, }, // Asia/Irkutsk
    {  515, 6969,   31955,  8, }, // Asia/Jakarta
    {  515, 6955,   31963,  9, }, // Asia/Jayapura
    {  515, 1257,  158789,  9, }, // Asia/Jerusalem
    {  515, 3462,  119188,  5, }, // Asia/Kabul
    {  515, 4218,   31981,  8, }, // Asia/Kamchatka
    {  515, 4141,   31989,  6, }, // Asia/Karachi
    {  515, 1152,    6018,  8, }, // Asia/Kathmandu
    {  515, 8624,   31995,  7, }, // Asia/Khandyga
    {  515,  992,   25307,  7, }, // Asia/Kolkata
    {  515, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  515, 4761,  158798, 12, }, // Asia/Kuala_Lumpur
    {  515, 5893,   32002,  6, }, // Asia/Kuching
    {  515, 5464,    6058,  6, }, // Asia/Kuwait
    {  515, 1178,    6064,  5, }, // Asia/Macau
    {  515, 3919,  119244,  7, }, // Asia/Magadan
    {  515, 1316,    6069,  7, }, // Asia/Makassar
    {  515, 7145,  158810,  6, }, // Asia/Manila
    {  515, 6883,    4227,  6, }, // Asia/Muscat
    {  515, 2446,   32013,  8, }, // Asia/Nicosia
    {  515, 8638,   32021, 12, }, // Asia/Novokuznetsk
    {  515, 4037,   61036, 11, }, // Asia/Novosibirsk
    {  515, 4131,   61048,  4, }, // Asia/Omsk
    {  515, 7108,   61053,  4, }, // Asia/Oral
    {  515, 6924,   20928,  8, }, // Asia/Phnom_Penh
    {  515, 8656,  158816, 10, }, // Asia/Pontianak
    {  515, 4116,   32033,  9, }, // Asia/Pyongyang
    {  515, 5476,    4233,  5, }, // Asia/Qatar
    {  515, 8671,    6103,  8, }, // Asia/Qostanay
    {  515, 4171,    6111,  9, }, // Asia/Qyzylorda
    {  515, 3486,   32042,  5, }, // Asia/Riyadh
    {  515, 4310,   32047,  7, }, // Asia/Sakhalin
    {  515, 7208,   25356,  9, }, // Asia/Samarkand
    {  515, 3116,    6131,  4, }, // Asia/Seoul
    {  515, 1020,   32054,  6, }, // Asia/Shanghai
    {  515, 3137,   19291,  8, }, // Asia/Singapore
    {  515, 4199,  158826, 14, }, // Asia/Srednekolymsk
    {  515, 3100,    6163,  6, }, // Asia/Taipei
    {  515, 4683,   32060,  7, }, // Asia/Tashkent
    {  515, 3817,  122458,  7, }, // Asia/Tbilisi
    {  515, 2638,    6175,  7, }, // Asia/Tehran
    {  515, 1284,   90291,  5, }, // Asia/Thimphu
    {  515, 2687,    4257,  5, }, // Asia/Tokyo
    {  515, 4474,  119419,  5, }, // Asia/Tomsk
    {  515, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  515, 1126,  158840,  6, }, // Asia/Urumqi
    {  515, 8685,  158846,  8, }, // Asia/Ust-Nera
    {  515, 6940,   32073, 10, }, // Asia/Vientiane
    {  515, 4612,   61189, 11, }, // Asia/Vladivostok
    {  515, 4730,    6206,  7, }, // Asia/Yakutsk
    {  515, 1202,    6213,  6, }, // Asia/Yangon
    {  515, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  515, 3649,    6231,  7, }, // Asia/Yerevan
    {  515, 3565,   32083,  5, }, // Atlantic/Azores
    {  515, 5588,   72942,  7, }, // Atlantic/Bermuda
    {  515, 6591,  158854, 15, }, // Atlantic/Canary
    {  515, 3629,  158869, 18, }, // Atlantic/Cape_Verde
    {  515, 1379,  158887, 13, }, // Atlantic/Faroe
    {  515, 8699,  158900,  7, }, // Atlantic/Madeira
    {  515, 2614,   32088,  9, }, // Atlantic/Reykjavik
    {  515, 7235,  158907, 15, }, // Atlantic/South_Georgia
    {  515, 6784,  158922, 12, }, // Atlantic/St_Helena
    {  515, 6607,    6298,  6, }, // Atlantic/Stanley
    {  515, 1654,  158934,  7, }, // Australia/Adelaide
    {  515, 1619,  158941,  8, }, // Australia/Brisbane
    {  515, 1783,  122675, 10, }, // Australia/Broken_Hill
    {  515, 1567,    6331,  6, }, // Australia/Darwin
    {  515, 3539,  158949,  5, }, // Australia/Eucla
    {  515, 1500,   61346,  6, }, // Australia/Hobart
    {  515, 8716,  119621,  8, }, // Australia/Lindeman
    {  515, 1531,   21137,  8, }, // Australia/Lord_Howe
    {  515, 1711,    6359,  7, }, // Australia/Melbourne
    {  515, 1746,   21145,  4, }, // Australia/Perth
    {  515, 1447,    6371,  6, }, // Australia/Sydney
    {  515, 6390,   61373,  9, }, // Europe/Amsterdam
    {  515, 6149,   32109,  6, }, // Europe/Andorra
    {  515, 3522,    6384,  8, }, // Europe/Astrakhan
    {  515, 6577,   32123,  5, }, // Europe/Athens
    {  515, 6285,    6398,  7, }, // Europe/Belgrade
    {  515, 4659,   61411,  6, }, // Europe/Berlin
    {  515, 6437,  158954, 10, }, // Europe/Bratislava
    {  515, 6178,   32128,  6, }, // Europe/Brussels
    {  515, 3830,   32134,  8, }, // Europe/Bucharest
    {  515, 3708,   32142, 10, }, // Europe/Budapest
    {  515, 8751,  158964,  8, }, // Europe/Busingen
    {  515, 2475,   32152,  8, }, // Europe/Chisinau
    {  515, 6238,    4376, 10, }, // Europe/Copenhagen
    {  515, 2260,   32160,  6, }, // Europe/Dublin
    {  515, 6256,  122861,  9, }, // Europe/Gibraltar
    {  515, 8767,   32166,  6, }, // Europe/Guernsey
    {  515, 6561,  122879,  8, }, // Europe/Helsinki
    {  515, 8783,   32172, 10, }, // Europe/Isle_of_Man
    {  515, 1097,   61535,  8, }, // Europe/Istanbul
    {  515, 8802,  158972,  6, }, // Europe/Jersey
    {  515, 3881,   32182, 12, }, // Europe/Kaliningrad
    {  515, 8924,  119814,  5, }, // Europe/Kirov
    {  515, 2419,   32194,  5, }, // Europe/Kyiv
    {  515, 3078,   32199,  7, }, // Europe/Lisbon
    {  515, 6455,  158978,  9, }, // Europe/Ljubljana
    {  515, 2393,   61584,  6, }, // Europe/London
    {  515, 6314,    6510, 10, }, // Europe/Luxembourg
    {  515, 4794,   61602,  6, }, // Europe/Madrid
    {  515, 6346,  158987,  5, }, // Europe/Malta
    {  515, 6531,   97841,  8, }, // Europe/Mariehamn
    {  515, 3595,   61614,  5, }, // Europe/Minsk
    {  515, 6359,    6525,  6, }, // Europe/Monaco
    {  515, 3443,   21290,  6, }, // Europe/Moscow
    {  515, 6407,  158992,  4, }, // Europe/Oslo
    {  515, 4186,   32206,  5, }, // Europe/Paris
    {  515, 6373,  158996,  9, }, // Europe/Podgorica
    {  515, 6224,    6545,  4, }, // Europe/Prague
    {  515, 8816,  159005,  4, }, // Europe/Riga
    {  515, 6273,   32211,  3, }, // Europe/Rome
    {  515, 4233,  159009,  6, }, // Europe/Samara
    {  515, 6419,  159015, 10, }, // Europe/San_Marino
    {  515, 6194,  159025,  8, }, // Europe/Sarajevo
    {  515, 4353,   76841,  7, }, // Europe/Saratov
    {  515, 8828,   32214, 11, }, // Europe/Simferopol
    {  515, 6332,   32225,  7, }, // Europe/Skopje
    {  515, 6548,   32232,  6, }, // Europe/Sofia
    {  515, 6472,    6586,  8, }, // Europe/Stockholm
    {  515, 8846,    6594,  5, }, // Europe/Tallinn
    {  515, 6135,   19608,  6, }, // Europe/Tirane
    {  515, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  515, 6301,   61716,  5, }, // Europe/Vaduz
    {  515, 6516,    6614,  7, }, // Europe/Vatican
    {  515, 6164,   32246,  3, }, // Europe/Vienna
    {  515, 8878,  159033,  7, }, // Europe/Vilnius
    {  515, 4629,   61732,  9, }, // Europe/Volgograd
    {  515, 3055,   32249,  7, }, // Europe/Warsaw
    {  515, 6210,   61750,  6, }, // Europe/Zagreb
    {  515, 6489,   32256,  5, }, // Europe/Zurich
    {  515, 4980,  159040, 12, }, // Indian/Antananarivo
    {  515, 6910,  120014,  5, }, // Indian/Chagos
    {  515, 5979,   32261,  5, }, // Indian/Christmas
    {  515, 5996,    4484,  5, }, // Indian/Cocos
    {  515, 4931,   21395,  6, }, // Indian/Comoro
    {  515, 6624,   21401,  8, }, // Indian/Kerguelen
    {  515, 7223,  151415,  4, }, // Indian/Mahe
    {  515, 7013,   32290,  7, }, // Indian/Maldives
    {  515, 3971,   32297, 10, }, // Indian/Mauritius
    {  515, 5000,    6669,  5, }, // Indian/Mayotte
    {  515, 7174,   89867,  7, }, // Indian/Reunion
    {  515, 4324,  120112,  5, }, // Pacific/Apia
    {  515,  932,    6675,  6, }, // Pacific/Auckland
    {  515, 3608,  159052,  8, }, // Pacific/Bougainville
    {  515, 2863,  123242,  5, }, // Pacific/Chatham
    {  515, 3022,   89874,  4, }, // Pacific/Chuuk
    {  515, 2201,  159060, 15, }, // Pacific/Easter
    {  515, 7323,  123265,  4, }, // Pacific/Efate
    {  515, 7290,  159075,  7, }, // Pacific/Fakaofo
    {  515, 3804,   32307,  5, }, // Pacific/Fiji
    {  515, 7306,  123284,  8, }, // Pacific/Funafuti
    {  515, 6641,   90754,  9, }, // Pacific/Galapagos
    {  515, 6659,  159082,  7, }, // Pacific/Gambier
    {  515, 3724,  120188, 11, }, // Pacific/Guadalcanal
    {  515, 5935,  120200,  4, }, // Pacific/Guam
    {  515, 2929,    4512,  8, }, // Pacific/Honolulu
    {  515, 2897,  120205,  9, }, // Pacific/Kanton
    {  515, 3900,   90788, 10, }, // Pacific/Kiritimati
    {  515, 6982,  159089,  5, }, // Pacific/Kosrae
    {  515, 2708,  159094, 11, }, // Pacific/Kwajalein
    {  515, 4779,   90836,  6, }, // Pacific/Majuro
    {  515, 3953,   34820,  6, }, // Pacific/Marquesas
    {  515, 8909,  120264,  6, }, // Pacific/Midway
    {  515, 7047,  159105,  5, }, // Pacific/Nauru
    {  515, 7095,  159381,  4, }, // Pacific/Niue
    {  515, 4070,   61860,  7, }, // Pacific/Norfolk
    {  515, 7061,  120279,  5, }, // Pacific/Noumea
    {  515, 2991,  159115,  9, }, // Pacific/Pago_Pago
    {  515, 7118,   64169,  5, }, // Pacific/Palau
    {  515, 7157,    6773,  7, }, // Pacific/Pitcairn
    {  515, 2961,    6780,  6, }, // Pacific/Pohnpei
    {  515, 4709,  159124, 11, }, // Pacific/Port_Moresby
    {  515, 6009,  159135,  9, }, // Pacific/Rarotonga
    {  515, 5948,  159144,  6, }, // Pacific/Saipan
    {  515, 7275,  159150,  6, }, // Pacific/Tahiti
    {  515, 6675,   21574,  6, }, // Pacific/Tarawa
    {  515, 4485,  159156,  9, }, // Pacific/Tongatapu
    {  515, 7355,  159165,  4, }, // Pacific/Wake
    {  515, 7368,   32312,  5, }, // Pacific/Wallis
    {  516, 6751,   56229,  7, }, // Africa/Abidjan Serbian/Latin/Serbia
    {  516, 6723,    4550,  4, }, // Africa/Accra
    {  516, 4961,   34301, 10, }, // Africa/Addis_Ababa
    {  516, 7383,   56099,  5, }, // Africa/Algiers
    {  516,   14,   89768,  6, }, // Africa/Asmara
    {  516,   44,  157946,  6, }, // Africa/Bamako
    {  516, 5159,  157952,  6, }, // Africa/Bangui
    {  516, 6709,  157958,  6, }, // Africa/Banjul
    {  516, 7398,  157964,  5, }, // Africa/Bissau
    {  516, 4859,    4588,  7, }, // Africa/Blantyre
    {  516, 5189,    4596,  8, }, // Africa/Brazzaville
    {  516, 4824,  157969, 10, }, // Africa/Bujumbura
    {  516, 2242,   24913,  5, }, // Africa/Cairo
    {  516, 4019,   31744, 10, }, // Africa/Casablanca
    {  516, 7412,   19754,  5, }, // Africa/Ceuta
    {  516, 6736,    4637,  7, }, // Africa/Conakry
    {  516, 6803,   59699,  5, }, // Africa/Dakar
    {  516, 5032,    4644, 12, }, // Africa/Dar_es_Salaam
    {  516, 4945,   31754,  7, }, // Africa/Djibouti
    {  516, 5145,   19777,  5, }, // Africa/Douala
    {  516, 5068,    4663,  7, }, // Africa/El_Aaiun
    {  516, 6816,    4670,  7, }, // Africa/Freetown
    {  516, 4808,   59741,  7, }, // Africa/Gaborone
    {  516, 4917,   59749,  6, }, // Africa/Harare
    {  516, 4381,    4677, 11, }, // Africa/Johannesburg
    {  516, 4401,  120501,  5, }, // Africa/Juba
    {  516, 5053,  157979,  7, }, // Africa/Kampala
    {  516, 4426,    4693,  6, }, // Africa/Khartoum
    {  516, 4875,  120515,  6, }, // Africa/Kigali
    {  516, 5208,  117752,  7, }, // Africa/Kinshasa
    {  516, 4646,   59800,  5, }, // Africa/Lagos
    {  516, 5238,    4699,  8, }, // Africa/Libreville
    {  516, 6832,   59815,  4, }, // Africa/Lome
    {  516, 5113,  157986,  6, }, // Africa/Luanda
    {  516, 4841,  117774,  9, }, // Africa/Lubumbashi
    {  516, 4903,  157992,  6, }, // Africa/Lusaka
    {  516, 5224,  157998,  6, }, // Africa/Malabo
    {  516, 4889,  158004,  6, }, // Africa/Maputo
    {  516, 5099,   59841,  6, }, // Africa/Maseru
    {  516, 5084,  158010,  7, }, // Africa/Mbabane
    {  516, 5015,   31761,  7, }, // Africa/Mogadishu
    {  516, 6997,  117797,  9, }, // Africa/Monrovia
    {  516, 3755,   59871,  7, }, // Africa/Nairobi
    {  516, 5173,   56262,  8, }, // Africa/Ndjamena
    {  516, 5256,  158017,  7, }, // Africa/Niamey
    {  516, 6766,   56270,  7, }, // Africa/Nouakchott
    {  516, 6690,   24967,  8, }, // Africa/Ouagadougou
    {  516, 5127,   24975, 10, }, // Africa/Porto-Novo
    {  516, 4337,   89774,  8, }, // Africa/Sao_Tome
    {  516, 2732,   34368,  7, }, // Africa/Tripoli
    {  516, 6503,   89940,  5, }, // Africa/Tunis
    {  516, 4054,   19877,  7, }, // Africa/Windhoek
    {  516,  132,   96487,  4, }, // America/Adak
    {  516, 3183,  158024,  8, }, // America/Anchorage
    {  516, 5524,   31768,  7, }, // America/Anguilla
    {  516, 5541,   31775,  7, }, // America/Antigua
    {  516, 4456,  120668, 10, }, // America/Araguaina
    {  516,  166,  158032, 12, }, // America/Argentina/Buenos_Aires
    {  516,   91,    4788,  9, }, // America/Argentina/Catamarca
    {  516,  270,    4797,  7, }, // America/Argentina/Cordoba
    {  516,  441,  158044,  5, }, // America/Argentina/Jujuy
    {  516, 7425,    4809,  8, }, // America/Argentina/La_Rioja
    {  516,  565,   96160,  7, }, // America/Argentina/Mendoza
    {  516, 7452,  120708, 12, }, // America/Argentina/Rio_Gallegos
    {  516, 7483,   59989,  5, }, // America/Argentina/Salta
    {  516, 7507,  158049,  8, }, // America/Argentina/San_Juan
    {  516, 5497,  158057,  7, }, // America/Argentina/San_Luis
    {  516, 7534,    4850,  7, }, // America/Argentina/Tucuman
    {  516, 7560,  158064,  7, }, // America/Argentina/Ushuaia
    {  516, 5557,  158071,  5, }, // America/Aruba
    {  516, 4154,    4869,  8, }, // America/Asuncion
    {  516,  237,  158076, 12, }, // America/Atikokan
    {  516, 3581,  117972,  5, }, // America/Bahia
    {  516, 7586,  158088, 14, }, // America/Bahia_Banderas
    {  516, 5571,  117990,  8, }, // America/Barbados
    {  516, 7609,   60040,  5, }, // America/Belem
    {  516, 5309,  158102,  6, }, // America/Belize
    {  516, 7623,  158108, 13, }, // America/Blanc-Sablon
    {  516, 7644,  158121,  9, }, // America/Boa_Vista
    {  516, 4263,  158130,  6, }, // America/Bogota
    {  516, 7662,  158136,  5, }, // America/Boise
    {  516, 7676,  158141, 12, }, // America/Cambridge_Bay
    {  516, 7698,   96188, 12, }, // America/Campo_Grande
    {  516, 3770,    4959,  6, }, // America/Cancun
    {  516, 4596,    4965,  7, }, // America/Caracas
    {  516, 4247,    4972,  5, }, // America/Cayenne
    {  516, 5398,  158153, 16, }, // America/Cayman
    {  516, 3251,  158169,  6, }, // America/Chicago
    {  516, 7719,  120907,  8, }, // America/Chihuahua
    {  516, 7737,  158175, 13, }, // America/Ciudad_Juarez
    {  516, 5324,   31782,  9, }, // America/Costa_Rica
    {  516, 7759,    5020,  7, }, // America/Creston
    {  516, 3693,   96220,  6, }, // America/Cuiaba
    {  516, 5640,    5033,  7, }, // America/Curacao
    {  516, 7775,  158188, 13, }, // America/Danmarkshavn
    {  516, 7796,  120962,  5, }, // America/Dawson
    {  516, 7811,  120969, 10, }, // America/Dawson_Creek
    {  516,  805,   60109,  6, }, // America/Denver
    {  516, 3351,  118170,  7, }, // America/Detroit
    {  516, 5656,   20071,  8, }, // America/Dominica
    {  516,  893,   60132,  8, }, // America/Edmonton
    {  516, 7832,  158201,  8, }, // America/Eirunepe
    {  516, 5343,    5090,  8, }, // America/El_Salvador
    {  516, 7849,  121026, 11, }, // America/Fort_Nelson
    {  516, 7869,  158209,  9, }, // America/Fortaleza
    {  516, 7887,  158218,  9, }, // America/Glace_Bay
    {  516, 6844,  158227,  7, }, // America/Goose_Bay
    {  516, 4514,  158234, 10, }, // America/Grand_Turk
    {  516, 5687,  158244,  7, }, // America/Grenada
    {  516, 5703,   31791,  9, }, // America/Guadeloupe
    {  516, 3662,   31800,  9, }, // America/Guatemala
    {  516, 4743,  118250,  8, }, // America/Guayaquil
    {  516, 6895,   56104,  7, }, // America/Guyana
    {  516, 1923,    5189,  8, }, // America/Halifax
    {  516, 2221,  140276,  6, }, // America/Havana
    {  516, 7905,  158251, 10, }, // America/Hermosillo
    {  516,  348,  118283, 12, }, // America/Indiana/Indianapolis
    {  516,  481,  158274, 14, }, // America/Indiana/Knox
    {  516, 7924,  158288, 17, }, // America/Indiana/Marengo
    {  516, 7948,  158305, 20, }, // America/Indiana/Petersburg
    {  516, 7975,  158325, 18, }, // America/Indiana/Tell_City
    {  516, 8001,  159169, 15, }, // America/Indiana/Vevay
    {  516, 8023,  159184, 18, }, // America/Indiana/Vincennes
    {  516, 8049,  158374, 17, }, // America/Indiana/Winamac
    {  516, 8073,  118413,  6, }, // America/Inuvik
    {  516,  660,  158391,  8, }, // America/Iqaluit
    {  516, 2665,   31809,  7, }, // America/Jamaica
    {  516, 5282,  158399,  4, }, // America/Juneau
    {  516,  521,  159202,  7, }, // America/Kentucky/Louisville
    {  516, 8088,  158409, 18, }, // America/Kentucky/Monticello
    {  516, 5621,  158427, 10, }, // America/Kralendijk
    {  516, 4278,  158437,  6, }, // America/La_Paz
    {  516, 7132,   20446,  4, }, // America/Lima
    {  516, 3390,  158443, 11, }, // America/Los_Angeles
    {  516, 5849,  158454, 21, }, // America/Lower_Princes
    {  516, 8116,  118506,  6, }, // America/Maceio
    {  516, 8131,  118512,  7, }, // America/Managua
    {  516, 1892,  118519,  6, }, // America/Manaus
    {  516, 5814,   60430,  6, }, // America/Marigot
    {  516, 5722,    5400,  8, }, // America/Martinique
    {  516, 8147,  118534,  9, }, // America/Matamoros
    {  516, 2779,  158475,  8, }, // America/Mazatlan
    {  516, 8165,    5409,  8, }, // America/Menominee
    {  516, 8183,    3899,  6, }, // America/Merida
    {  516, 8198,  158483, 10, }, // America/Metlakatla
    {  516, 2811,  158493, 12, }, // America/Mexico_City
    {  516, 4293,    5434,  7, }, // America/Miquelon
    {  516, 8217,    5441,  7, }, // America/Moncton
    {  516, 8233,   60479,  8, }, // America/Monterrey
    {  516, 4000,  158505, 10, }, // America/Montevideo
    {  516, 5741,    5457,  9, }, // America/Montserrat
    {  516, 5383,  121439,  5, }, // America/Nassau
    {  516, 3294,  158515,  7, }, // America/New_York
    {  516, 8251,   20373,  3, }, // America/Nome
    {  516, 1834,    5479,  7, }, // America/Noronha
    {  516, 8264,  159209, 22, }, // America/North_Dakota/Beulah
    {  516, 8292,  159231, 22, }, // America/North_Dakota/Center
    {  516, 8320,  159253, 26, }, // America/North_Dakota/New_Salem
    {  516,  393,  158594,  6, }, // America/Nuuk
    {  516, 8351,    5561,  7, }, // America/Ojinaga
    {  516, 5413,  158600,  6, }, // America/Panama
    {  516, 6106,  158606, 10, }, // America/Paramaribo
    {  516, 3224,  121555,  6, }, // America/Phoenix
    {  516, 3847,  121563, 12, }, // America/Port-au-Prince
    {  516, 5871,  159279, 13, }, // America/Port_of_Spain
    {  516, 8367,  158641, 11, }, // America/Porto_Velho
    {  516, 5760,  159292, 10, }, // America/Puerto_Rico
    {  516, 3932,  121616, 12, }, // America/Punta_Arenas
    {  516, 8387,  121630, 12, }, // America/Rankin_Inlet
    {  516, 8408,  158652,  6, }, // America/Recife
    {  516, 1979,  159302,  6, }, // America/Regina
    {  516, 8423,  121657,  7, }, // America/Resolute
    {  516,  695,   20506, 10, }, // America/Rio_Branco
    {  516, 8440,  158673,  8, }, // America/Santarem
    {  516, 2165,  118795,  8, }, // America/Santiago
    {  516, 6044,  158681, 13, }, // America/Santo_Domingo
    {  516, 1862,  158694,  9, }, // America/Sao_Paulo
    {  516, 6862,  159308, 12, }, // America/Scoresbysund
    {  516, 8457,  158703,  5, }, // America/Sitka
    {  516, 8471,  159320, 14, }, // America/St_Barthelemy
    {  516, 2045,  159334,  8, }, // America/St_Johns
    {  516, 5780,  121759,  9, }, // America/St_Kitts
    {  516, 5797,  158708, 10, }, // America/St_Lucia
    {  516,  855,  159342,  8, }, // America/St_Thomas
    {  516, 5830,  158718, 12, }, // America/St_Vincent
    {  516, 8493,   20619, 12, }, // America/Swift_Current
    {  516, 5363,    5759, 11, }, // America/Tegucigalpa
    {  516, 5673,   20642,  3, }, // America/Thule
    {  516,  313,    5775,  7, }, // America/Tijuana
    {  516,  608,  158730,  7, }, // America/Toronto
    {  516, 5605,  158737,  7, }, // America/Tortola
    {  516, 2077,    5789,  8, }, // America/Vancouver
    {  516, 2128,  158744,  8, }, // America/Whitehorse
    {  516,  734,   60758,  7, }, // America/Winnipeg
    {  516, 8515,    5813,  7, }, // America/Yakutat
    {  516, 5918,    5820,  5, }, // Antarctica/Casey
    {  516, 6027,    5825,  6, }, // Antarctica/Davis
    {  516, 6066,  159350, 13, }, // Antarctica/DumontDUrville
    {  516, 8531,  159363,  7, }, // Antarctica/Macquarie
    {  516, 7029,  118978,  5, }, // Antarctica/Mawson
    {  516, 7076,  158765,  8, }, // Antarctica/McMurdo
    {  516, 8552,  121938,  6, }, // Antarctica/Palmer
    {  516, 7189,  118999,  6, }, // Antarctica/Rothera
    {  516, 7258,  119005,  4, }, // Antarctica/Syowa
    {  516, 8570,  119009,  4, }, // Antarctica/Troll
    {  516, 7337,  119015,  6, }, // Antarctica/Vostok
    {  516, 1413,  159370, 11, }, // Arctic/Longyearbyen
    {  516, 5487,   60791,  4, }, // Asia/Aden
    {  516, 5297,    5890,  6, }, // Asia/Almaty
    {  516, 3870,    5896,  4, }, // Asia/Amman
    {  516, 5428,    5900,  6, }, // Asia/Anadyr
    {  516, 5440,    5906,  5, }, // Asia/Aqtau
    {  516, 5270,   31897,  7, }, // Asia/Aqtobe
    {  516,  964,   31904,  7, }, // Asia/Ashgabat
    {  516, 8587,    5917,  6, }, // Asia/Atyrau
    {  516, 3509,    4063,  6, }, // Asia/Baghdad
    {  516, 5451,    4069,  7, }, // Asia/Bahrain
    {  516, 3555,   60829,  4, }, // Asia/Baku
    {  516, 4368,  119081,  7, }, // Asia/Bangkok
    {  516, 3473,  119089,  7, }, // Asia/Barnaul
    {  516, 3988,    5930,  6, }, // Asia/Beirut
    {  516, 3680,   31917,  6, }, // Asia/Bishkek
    {  516, 5906,   31923,  6, }, // Asia/Brunei
    {  516, 4503,   56142,  4, }, // Asia/Chita
    {  516, 5963,   31930,  9, }, // Asia/Choibalsan
    {  516, 4413,    5949,  7, }, // Asia/Colombo
    {  516, 4442,    4102,  6, }, // Asia/Damascus
    {  516, 1060,    5956,  4, }, // Asia/Dhaka
    {  516, 6125,  122105,  4, }, // Asia/Dili
    {  516, 3498,  119135,  5, }, // Asia/Dubai
    {  516, 6092,   31939,  7, }, // Asia/Dushanbe
    {  516, 8599,  158780,  9, }, // Asia/Famagusta
    {  516, 8614,   60896,  4, }, // Asia/Gaza
    {  516, 4697,   60905,  6, }, // Asia/Hebron
    {  516, 1226,   31946,  9, }, // Asia/Ho_Chi_Minh
    {  516, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  516, 4673,  119165,  4, }, // Asia/Hovd
    {  516, 4086,  139633,  6, }, // Asia/Irkutsk
    {  516, 6969,   31955,  8, }, // Asia/Jakarta
    {  516, 6955,   31963,  9, }, // Asia/Jayapura
    {  516, 1257,  158789,  9, }, // Asia/Jerusalem
    {  516, 3462,  119188,  5, }, // Asia/Kabul
    {  516, 4218,   31981,  8, }, // Asia/Kamchatka
    {  516, 4141,   31989,  6, }, // Asia/Karachi
    {  516, 1152,    6018,  8, }, // Asia/Kathmandu
    {  516, 8624,   31995,  7, }, // Asia/Khandyga
    {  516,  992,   25307,  7, }, // Asia/Kolkata
    {  516, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  516, 4761,  158798, 12, }, // Asia/Kuala_Lumpur
    {  516, 5893,   32002,  6, }, // Asia/Kuching
    {  516, 5464,    6058,  6, }, // Asia/Kuwait
    {  516, 1178,    6064,  5, }, // Asia/Macau
    {  516, 3919,  119244,  7, }, // Asia/Magadan
    {  516, 1316,    6069,  7, }, // Asia/Makassar
    {  516, 7145,  158810,  6, }, // Asia/Manila
    {  516, 6883,    4227,  6, }, // Asia/Muscat
    {  516, 2446,   32013,  8, }, // Asia/Nicosia
    {  516, 8638,   32021, 12, }, // Asia/Novokuznetsk
    {  516, 4037,   61036, 11, }, // Asia/Novosibirsk
    {  516, 4131,   61048,  4, }, // Asia/Omsk
    {  516, 7108,   61053,  4, }, // Asia/Oral
    {  516, 6924,   20928,  8, }, // Asia/Phnom_Penh
    {  516, 8656,  158816, 10, }, // Asia/Pontianak
    {  516, 4116,   32033,  9, }, // Asia/Pyongyang
    {  516, 5476,    4233,  5, }, // Asia/Qatar
    {  516, 8671,    6103,  8, }, // Asia/Qostanay
    {  516, 4171,    6111,  9, }, // Asia/Qyzylorda
    {  516, 3486,   32042,  5, }, // Asia/Riyadh
    {  516, 4310,   32047,  7, }, // Asia/Sakhalin
    {  516, 7208,   25356,  9, }, // Asia/Samarkand
    {  516, 3116,    6131,  4, }, // Asia/Seoul
    {  516, 1020,   32054,  6, }, // Asia/Shanghai
    {  516, 3137,   19291,  8, }, // Asia/Singapore
    {  516, 4199,  158826, 14, }, // Asia/Srednekolymsk
    {  516, 3100,    6163,  6, }, // Asia/Taipei
    {  516, 4683,   32060,  7, }, // Asia/Tashkent
    {  516, 3817,  122458,  7, }, // Asia/Tbilisi
    {  516, 2638,    6175,  7, }, // Asia/Tehran
    {  516, 1284,   90291,  5, }, // Asia/Thimphu
    {  516, 2687,    4257,  5, }, // Asia/Tokyo
    {  516, 4474,  119419,  5, }, // Asia/Tomsk
    {  516, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  516, 1126,  158840,  6, }, // Asia/Urumqi
    {  516, 8685,  158846,  8, }, // Asia/Ust-Nera
    {  516, 6940,   32073, 10, }, // Asia/Vientiane
    {  516, 4612,   61189, 11, }, // Asia/Vladivostok
    {  516, 4730,    6206,  7, }, // Asia/Yakutsk
    {  516, 1202,    6213,  6, }, // Asia/Yangon
    {  516, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  516, 3649,    6231,  7, }, // Asia/Yerevan
    {  516, 3565,   32083,  5, }, // Atlantic/Azores
    {  516, 5588,   72942,  7, }, // Atlantic/Bermuda
    {  516, 6591,  158854, 15, }, // Atlantic/Canary
    {  516, 3629,  158869, 18, }, // Atlantic/Cape_Verde
    {  516, 1379,  158887, 13, }, // Atlantic/Faroe
    {  516, 8699,  158900,  7, }, // Atlantic/Madeira
    {  516, 2614,   32088,  9, }, // Atlantic/Reykjavik
    {  516, 7235,  158907, 15, }, // Atlantic/South_Georgia
    {  516, 6784,  158922, 12, }, // Atlantic/St_Helena
    {  516, 6607,    6298,  6, }, // Atlantic/Stanley
    {  516, 1654,  158934,  7, }, // Australia/Adelaide
    {  516, 1619,  158941,  8, }, // Australia/Brisbane
    {  516, 1783,  122675, 10, }, // Australia/Broken_Hill
    {  516, 1567,    6331,  6, }, // Australia/Darwin
    {  516, 3539,  158949,  5, }, // Australia/Eucla
    {  516, 1500,   61346,  6, }, // Australia/Hobart
    {  516, 8716,  119621,  8, }, // Australia/Lindeman
    {  516, 1531,   21137,  8, }, // Australia/Lord_Howe
    {  516, 1711,    6359,  7, }, // Australia/Melbourne
    {  516, 1746,   21145,  4, }, // Australia/Perth
    {  516, 1447,    6371,  6, }, // Australia/Sydney
    {  516, 6390,   61373,  9, }, // Europe/Amsterdam
    {  516, 6149,   32109,  6, }, // Europe/Andorra
    {  516, 3522,    6384,  8, }, // Europe/Astrakhan
    {  516, 6577,   32123,  5, }, // Europe/Athens
    {  516, 6285,    6398,  7, }, // Europe/Belgrade
    {  516, 4659,   61411,  6, }, // Europe/Berlin
    {  516, 6437,  158954, 10, }, // Europe/Bratislava
    {  516, 6178,   32128,  6, }, // Europe/Brussels
    {  516, 3830,   32134,  8, }, // Europe/Bucharest
    {  516, 3708,   32142, 10, }, // Europe/Budapest
    {  516, 8751,  158964,  8, }, // Europe/Busingen
    {  516, 2475,   32152,  8, }, // Europe/Chisinau
    {  516, 6238,    4376, 10, }, // Europe/Copenhagen
    {  516, 2260,   32160,  6, }, // Europe/Dublin
    {  516, 6256,  122861,  9, }, // Europe/Gibraltar
    {  516, 8767,   32166,  6, }, // Europe/Guernsey
    {  516, 6561,  122879,  8, }, // Europe/Helsinki
    {  516, 8783,   32172, 10, }, // Europe/Isle_of_Man
    {  516, 1097,   61535,  8, }, // Europe/Istanbul
    {  516, 8802,  158972,  6, }, // Europe/Jersey
    {  516, 3881,   32182, 12, }, // Europe/Kaliningrad
    {  516, 8924,  119814,  5, }, // Europe/Kirov
    {  516, 2419,   32194,  5, }, // Europe/Kyiv
    {  516, 3078,   32199,  7, }, // Europe/Lisbon
    {  516, 6455,  158978,  9, }, // Europe/Ljubljana
    {  516, 2393,   61584,  6, }, // Europe/London
    {  516, 6314,    6510, 10, }, // Europe/Luxembourg
    {  516, 4794,   61602,  6, }, // Europe/Madrid
    {  516, 6346,  158987,  5, }, // Europe/Malta
    {  516, 6531,   97841,  8, }, // Europe/Mariehamn
    {  516, 3595,   61614,  5, }, // Europe/Minsk
    {  516, 6359,    6525,  6, }, // Europe/Monaco
    {  516, 3443,   21290,  6, }, // Europe/Moscow
    {  516, 6407,  158992,  4, }, // Europe/Oslo
    {  516, 4186,   32206,  5, }, // Europe/Paris
    {  516, 6373,  158996,  9, }, // Europe/Podgorica
    {  516, 6224,    6545,  4, }, // Europe/Prague
    {  516, 8816,  159005,  4, }, // Europe/Riga
    {  516, 6273,   32211,  3, }, // Europe/Rome
    {  516, 4233,  159009,  6, }, // Europe/Samara
    {  516, 6419,  159015, 10, }, // Europe/San_Marino
    {  516, 6194,  159025,  8, }, // Europe/Sarajevo
    {  516, 4353,   76841,  7, }, // Europe/Saratov
    {  516, 8828,   32214, 11, }, // Europe/Simferopol
    {  516, 6332,   32225,  7, }, // Europe/Skopje
    {  516, 6548,   32232,  6, }, // Europe/Sofia
    {  516, 6472,    6586,  8, }, // Europe/Stockholm
    {  516, 8846,    6594,  5, }, // Europe/Tallinn
    {  516, 6135,   19608,  6, }, // Europe/Tirane
    {  516, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  516, 6301,   61716,  5, }, // Europe/Vaduz
    {  516, 6516,    6614,  7, }, // Europe/Vatican
    {  516, 6164,   32246,  3, }, // Europe/Vienna
    {  516, 8878,  159033,  7, }, // Europe/Vilnius
    {  516, 4629,   61732,  9, }, // Europe/Volgograd
    {  516, 3055,   32249,  7, }, // Europe/Warsaw
    {  516, 6210,   61750,  6, }, // Europe/Zagreb
    {  516, 6489,   32256,  5, }, // Europe/Zurich
    {  516, 4980,  159040, 12, }, // Indian/Antananarivo
    {  516, 6910,  120014,  5, }, // Indian/Chagos
    {  516, 5979,   32261,  5, }, // Indian/Christmas
    {  516, 5996,    4484,  5, }, // Indian/Cocos
    {  516, 4931,   21395,  6, }, // Indian/Comoro
    {  516, 6624,   21401,  8, }, // Indian/Kerguelen
    {  516, 7223,  151415,  4, }, // Indian/Mahe
    {  516, 7013,   32290,  7, }, // Indian/Maldives
    {  516, 3971,   32297, 10, }, // Indian/Mauritius
    {  516, 5000,    6669,  5, }, // Indian/Mayotte
    {  516, 7174,   89867,  7, }, // Indian/Reunion
    {  516, 4324,  120112,  5, }, // Pacific/Apia
    {  516,  932,    6675,  6, }, // Pacific/Auckland
    {  516, 3608,  159052,  8, }, // Pacific/Bougainville
    {  516, 2863,  123242,  5, }, // Pacific/Chatham
    {  516, 3022,   89874,  4, }, // Pacific/Chuuk
    {  516, 2201,  159060, 15, }, // Pacific/Easter
    {  516, 7323,  123265,  4, }, // Pacific/Efate
    {  516, 7290,  159075,  7, }, // Pacific/Fakaofo
    {  516, 3804,   32307,  5, }, // Pacific/Fiji
    {  516, 7306,  123284,  8, }, // Pacific/Funafuti
    {  516, 6641,   90754,  9, }, // Pacific/Galapagos
    {  516, 6659,  159082,  7, }, // Pacific/Gambier
    {  516, 3724,  120188, 11, }, // Pacific/Guadalcanal
    {  516, 5935,  120200,  4, }, // Pacific/Guam
    {  516, 2929,    4512,  8, }, // Pacific/Honolulu
    {  516, 2897,  120205,  9, }, // Pacific/Kanton
    {  516, 3900,   90788, 10, }, // Pacific/Kiritimati
    {  516, 6982,  159089,  5, }, // Pacific/Kosrae
    {  516, 2708,  159094, 11, }, // Pacific/Kwajalein
    {  516, 4779,   90836,  6, }, // Pacific/Majuro
    {  516, 3953,   34820,  6, }, // Pacific/Marquesas
    {  516, 8909,  120264,  6, }, // Pacific/Midway
    {  516, 7047,  159105,  5, }, // Pacific/Nauru
    {  516, 7095,  159381,  4, }, // Pacific/Niue
    {  516, 4070,   61860,  7, }, // Pacific/Norfolk
    {  516, 7061,  120279,  5, }, // Pacific/Noumea
    {  516, 2991,  159115,  9, }, // Pacific/Pago_Pago
    {  516, 7118,   64169,  5, }, // Pacific/Palau
    {  516, 7157,    6773,  7, }, // Pacific/Pitcairn
    {  516, 2961,    6780,  6, }, // Pacific/Pohnpei
    {  516, 4709,  159124, 11, }, // Pacific/Port_Moresby
    {  516, 6009,  159135,  9, }, // Pacific/Rarotonga
    {  516, 5948,  159144,  6, }, // Pacific/Saipan
    {  516, 7275,  159150,  6, }, // Pacific/Tahiti
    {  516, 6675,   21574,  6, }, // Pacific/Tarawa
    {  516, 4485,  159156,  9, }, // Pacific/Tongatapu
    {  516, 7355,  159165,  4, }, // Pacific/Wake
    {  516, 7368,   32312,  5, }, // Pacific/Wallis
    {  517,   14,       0,  6, }, // Africa/Asmara Shambala/Latin/Tanzania
    {  517, 4337,       6,  8, }, // Africa/Sao_Tome
    {  517, 4154,      14,  8, }, // America/Asuncion
    {  517,  237,      22,  8, }, // America/Atikokan
    {  517, 7586,      30, 17, }, // America/Bahia_Banderas
    {  517, 3770,      47,  6, }, // America/Cancun
    {  517, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  517, 5640,      66,  7, }, // America/Curacao
    {  517,  481,      73, 13, }, // America/Indiana/Knox
    {  517, 7924,      86, 16, }, // America/Indiana/Marengo
    {  517, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  517, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  517, 8001,     139, 14, }, // America/Indiana/Vevay
    {  517, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  517, 8049,     171, 16, }, // America/Indiana/Winamac
    {  517, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  517, 5849,     207, 22, }, // America/Lower_Princes
    {  517, 8183,     229,  6, }, // America/Merida
    {  517, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  517, 8292,     255, 20, }, // America/North_Dakota/Center
    {  517, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  517,  393,     298,  4, }, // America/Nuuk
    {  517, 6862,     302, 16, }, // America/Scoresbysund
    {  517, 8471,     318, 14, }, // America/St_Barthelemy
    {  517, 2045,     332, 10, }, // America/St_Johns
    {  517, 5780,     342,  9, }, // America/St_Kitts
    {  517, 5797,     351,  9, }, // America/St_Lucia
    {  517,  855,     360, 10, }, // America/St_Thomas
    {  517, 5830,     370, 11, }, // America/St_Vincent
    {  517, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  517, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  517, 1152,     413,  9, }, // Asia/Kathmandu
    {  517,  992,     422,  7, }, // Asia/Kolkata
    {  517, 1178,     429,  5, }, // Asia/Macau
    {  517, 1202,     442,  6, }, // Asia/Yangon
    {  517, 1379,     448,  5, }, // Atlantic/Faroe
    {  517, 6784,     453, 10, }, // Atlantic/St_Helena
    {  517, 2419,     463,  4, }, // Europe/Kyiv
    {  517, 7174,     467,  7, }, // Indian/Reunion
    {  517, 3022,     474,  5, }, // Pacific/Chuuk
    {  517, 2961,     479,  7, }, // Pacific/Pohnpei
    {  518,   14,       0,  6, }, // Africa/Asmara Shona/Latin/Zimbabwe
    {  518, 4337,       6,  8, }, // Africa/Sao_Tome
    {  518, 4154,      14,  8, }, // America/Asuncion
    {  518,  237,      22,  8, }, // America/Atikokan
    {  518, 7586,      30, 17, }, // America/Bahia_Banderas
    {  518, 3770,      47,  6, }, // America/Cancun
    {  518, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  518, 5640,      66,  7, }, // America/Curacao
    {  518,  481,      73, 13, }, // America/Indiana/Knox
    {  518, 7924,      86, 16, }, // America/Indiana/Marengo
    {  518, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  518, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  518, 8001,     139, 14, }, // America/Indiana/Vevay
    {  518, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  518, 8049,     171, 16, }, // America/Indiana/Winamac
    {  518, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  518, 5849,     207, 22, }, // America/Lower_Princes
    {  518, 8183,     229,  6, }, // America/Merida
    {  518, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  518, 8292,     255, 20, }, // America/North_Dakota/Center
    {  518, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  518,  393,     298,  4, }, // America/Nuuk
    {  518, 6862,     302, 16, }, // America/Scoresbysund
    {  518, 8471,     318, 14, }, // America/St_Barthelemy
    {  518, 2045,     332, 10, }, // America/St_Johns
    {  518, 5780,     342,  9, }, // America/St_Kitts
    {  518, 5797,     351,  9, }, // America/St_Lucia
    {  518,  855,     360, 10, }, // America/St_Thomas
    {  518, 5830,     370, 11, }, // America/St_Vincent
    {  518, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  518, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  518, 1152,     413,  9, }, // Asia/Kathmandu
    {  518,  992,     422,  7, }, // Asia/Kolkata
    {  518, 1178,     429,  5, }, // Asia/Macau
    {  518, 1202,     442,  6, }, // Asia/Yangon
    {  518, 1379,     448,  5, }, // Atlantic/Faroe
    {  518, 6784,     453, 10, }, // Atlantic/St_Helena
    {  518, 2419,     463,  4, }, // Europe/Kyiv
    {  518, 7174,     467,  7, }, // Indian/Reunion
    {  518, 3022,     474,  5, }, // Pacific/Chuuk
    {  518, 2961,     479,  7, }, // Pacific/Pohnpei
    {  519,   14,       0,  6, }, // Africa/Asmara Sichuan Yi/Yi/China
    {  519, 4337,       6,  8, }, // Africa/Sao_Tome
    {  519, 4154,      14,  8, }, // America/Asuncion
    {  519,  237,      22,  8, }, // America/Atikokan
    {  519, 7586,      30, 17, }, // America/Bahia_Banderas
    {  519, 3770,      47,  6, }, // America/Cancun
    {  519, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  519, 5640,      66,  7, }, // America/Curacao
    {  519,  481,      73, 13, }, // America/Indiana/Knox
    {  519, 7924,      86, 16, }, // America/Indiana/Marengo
    {  519, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  519, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  519, 8001,     139, 14, }, // America/Indiana/Vevay
    {  519, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  519, 8049,     171, 16, }, // America/Indiana/Winamac
    {  519, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  519, 5849,     207, 22, }, // America/Lower_Princes
    {  519, 8183,     229,  6, }, // America/Merida
    {  519, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  519, 8292,     255, 20, }, // America/North_Dakota/Center
    {  519, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  519,  393,     298,  4, }, // America/Nuuk
    {  519, 6862,     302, 16, }, // America/Scoresbysund
    {  519, 8471,     318, 14, }, // America/St_Barthelemy
    {  519, 2045,     332, 10, }, // America/St_Johns
    {  519, 5780,     342,  9, }, // America/St_Kitts
    {  519, 5797,     351,  9, }, // America/St_Lucia
    {  519,  855,     360, 10, }, // America/St_Thomas
    {  519, 5830,     370, 11, }, // America/St_Vincent
    {  519, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  519, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  519, 1152,     413,  9, }, // Asia/Kathmandu
    {  519,  992,     422,  7, }, // Asia/Kolkata
    {  519, 1178,     429,  5, }, // Asia/Macau
    {  519, 1202,     442,  6, }, // Asia/Yangon
    {  519, 1379,     448,  5, }, // Atlantic/Faroe
    {  519, 6784,     453, 10, }, // Atlantic/St_Helena
    {  519, 2419,     463,  4, }, // Europe/Kyiv
    {  519, 7174,     467,  7, }, // Indian/Reunion
    {  519, 3022,     474,  5, }, // Pacific/Chuuk
    {  519, 2961,     479,  7, }, // Pacific/Pohnpei
    {  520,   14,       0,  6, }, // Africa/Asmara Sicilian/Latin/Italy
    {  520, 4337,       6,  8, }, // Africa/Sao_Tome
    {  520, 4154,      14,  8, }, // America/Asuncion
    {  520,  237,      22,  8, }, // America/Atikokan
    {  520, 7586,      30, 17, }, // America/Bahia_Banderas
    {  520, 3770,      47,  6, }, // America/Cancun
    {  520, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  520, 5640,      66,  7, }, // America/Curacao
    {  520,  481,      73, 13, }, // America/Indiana/Knox
    {  520, 7924,      86, 16, }, // America/Indiana/Marengo
    {  520, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  520, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  520, 8001,     139, 14, }, // America/Indiana/Vevay
    {  520, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  520, 8049,     171, 16, }, // America/Indiana/Winamac
    {  520, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  520, 5849,     207, 22, }, // America/Lower_Princes
    {  520, 8183,     229,  6, }, // America/Merida
    {  520, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  520, 8292,     255, 20, }, // America/North_Dakota/Center
    {  520, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  520,  393,     298,  4, }, // America/Nuuk
    {  520, 6862,     302, 16, }, // America/Scoresbysund
    {  520, 8471,     318, 14, }, // America/St_Barthelemy
    {  520, 2045,     332, 10, }, // America/St_Johns
    {  520, 5780,     342,  9, }, // America/St_Kitts
    {  520, 5797,     351,  9, }, // America/St_Lucia
    {  520,  855,     360, 10, }, // America/St_Thomas
    {  520, 5830,     370, 11, }, // America/St_Vincent
    {  520, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  520, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  520, 1152,     413,  9, }, // Asia/Kathmandu
    {  520,  992,     422,  7, }, // Asia/Kolkata
    {  520, 1178,     429,  5, }, // Asia/Macau
    {  520, 1202,     442,  6, }, // Asia/Yangon
    {  520, 1379,     448,  5, }, // Atlantic/Faroe
    {  520, 6784,     453, 10, }, // Atlantic/St_Helena
    {  520, 2419,     463,  4, }, // Europe/Kyiv
    {  520, 7174,     467,  7, }, // Indian/Reunion
    {  520, 3022,     474,  5, }, // Pacific/Chuuk
    {  520, 2961,     479,  7, }, // Pacific/Pohnpei
    {  521,   14,       0,  6, }, // Africa/Asmara Sidamo/Latin/Ethiopia
    {  521, 4337,       6,  8, }, // Africa/Sao_Tome
    {  521, 4154,      14,  8, }, // America/Asuncion
    {  521,  237,      22,  8, }, // America/Atikokan
    {  521, 7586,      30, 17, }, // America/Bahia_Banderas
    {  521, 3770,      47,  6, }, // America/Cancun
    {  521, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  521, 5640,      66,  7, }, // America/Curacao
    {  521,  481,      73, 13, }, // America/Indiana/Knox
    {  521, 7924,      86, 16, }, // America/Indiana/Marengo
    {  521, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  521, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  521, 8001,     139, 14, }, // America/Indiana/Vevay
    {  521, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  521, 8049,     171, 16, }, // America/Indiana/Winamac
    {  521, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  521, 5849,     207, 22, }, // America/Lower_Princes
    {  521, 8183,     229,  6, }, // America/Merida
    {  521, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  521, 8292,     255, 20, }, // America/North_Dakota/Center
    {  521, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  521,  393,     298,  4, }, // America/Nuuk
    {  521, 6862,     302, 16, }, // America/Scoresbysund
    {  521, 8471,     318, 14, }, // America/St_Barthelemy
    {  521, 2045,     332, 10, }, // America/St_Johns
    {  521, 5780,     342,  9, }, // America/St_Kitts
    {  521, 5797,     351,  9, }, // America/St_Lucia
    {  521,  855,     360, 10, }, // America/St_Thomas
    {  521, 5830,     370, 11, }, // America/St_Vincent
    {  521, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  521, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  521, 1152,     413,  9, }, // Asia/Kathmandu
    {  521,  992,     422,  7, }, // Asia/Kolkata
    {  521, 1178,     429,  5, }, // Asia/Macau
    {  521, 1202,     442,  6, }, // Asia/Yangon
    {  521, 1379,     448,  5, }, // Atlantic/Faroe
    {  521, 6784,     453, 10, }, // Atlantic/St_Helena
    {  521, 2419,     463,  4, }, // Europe/Kyiv
    {  521, 7174,     467,  7, }, // Indian/Reunion
    {  521, 3022,     474,  5, }, // Pacific/Chuuk
    {  521, 2961,     479,  7, }, // Pacific/Pohnpei
    {  522, 6751,  149651,  7, }, // Africa/Abidjan Silesian/Latin/Poland
    {  522, 6723,    4550,  4, }, // Africa/Accra
    {  522, 4961,    3793, 11, }, // Africa/Addis_Ababa
    {  522, 7383,   42592,  5, }, // Africa/Algiers
    {  522,   14,       0,  6, }, // Africa/Asmara
    {  522, 5159,   24908,  5, }, // Africa/Bangui
    {  522, 6709,  149658,  7, }, // Africa/Banjul
    {  522, 4824,  149665,  9, }, // Africa/Bujumbura
    {  522, 2242,   24913,  4, }, // Africa/Cairo
    {  522, 6736,  159385,  7, }, // Africa/Conakry
    {  522, 4945,  149674,  7, }, // Africa/Djibouti
    {  522, 5145,   19777,  5, }, // Africa/Douala
    {  522, 5068,  149681,  7, }, // Africa/El_Aaiun
    {  522, 4401,  149688,  5, }, // Africa/Juba
    {  522, 4426,  120506,  7, }, // Africa/Khartoum
    {  522, 5208,  149693,  8, }, // Africa/Kinshasa
    {  522, 6832,   18802,  4, }, // Africa/Lome
    {  522, 5015,  149701,  9, }, // Africa/Mogadishu
    {  522, 5173,  149710,  8, }, // Africa/Ndjamena
    {  522, 6766,  149718,  9, }, // Africa/Nouakchott
    {  522, 6690,   97031,  8, }, // Africa/Ouagadougou
    {  522, 5127,   24975, 10, }, // Africa/Porto-Novo
    {  522, 4337,       6,  8, }, // Africa/Sao_Tome
    {  522, 2732,  149727,  8, }, // Africa/Tripoli
    {  522, 4054,   55328,  7, }, // Africa/Windhoek
    {  522,  565,  159392,  7, }, // America/Argentina/Mendoza
    {  522, 4154,  159399,  8, }, // America/Asuncion
    {  522,  237,      22,  8, }, // America/Atikokan
    {  522, 3581,    5090,  8, }, // America/Bahia
    {  522, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  522, 7609,   34376,  5, }, // America/Belem
    {  522, 3770,      47,  6, }, // America/Cancun
    {  522, 4247,  159407,  7, }, // America/Cayenne
    {  522, 5398,  149735,  7, }, // America/Cayman
    {  522, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  522, 5324,  149742,  9, }, // America/Costa_Rica
    {  522, 3693,   25007,  6, }, // America/Cuiaba
    {  522, 5640,      66,  7, }, // America/Curacao
    {  522, 5656,  159414,  8, }, // America/Dominica
    {  522, 5343,  149751,  8, }, // America/El_Salvador
    {  522, 5687,  159422,  7, }, // America/Grenada
    {  522, 5703,  149759,  9, }, // America/Guadeloupe
    {  522, 3662,  159429,  9, }, // America/Guatemala
    {  522, 6895,  149768,  6, }, // America/Guyana
    {  522, 2221,  149774,  6, }, // America/Havana
    {  522,  481,      73, 13, }, // America/Indiana/Knox
    {  522, 7924,      86, 16, }, // America/Indiana/Marengo
    {  522, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  522, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  522, 8001,     139, 14, }, // America/Indiana/Vevay
    {  522, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  522, 8049,     171, 16, }, // America/Indiana/Winamac
    {  522, 2665,   31809,  7, }, // America/Jamaica
    {  522, 8088,     187, 10, }, // America/Kentucky/Monticello
    {  522, 5849,     207, 22, }, // America/Lower_Princes
    {  522, 8116,  159438,  6, }, // America/Maceio
    {  522, 5722,  149780,  9, }, // America/Martinique
    {  522, 8165,  159444,  9, }, // America/Menominee
    {  522, 8183,    3899,  6, }, // America/Merida
    {  522, 2811,  149789, 15, }, // America/Mexico_City
    {  522, 5741,  159453, 10, }, // America/Montserrat
    {  522, 3294,  149804,  9, }, // America/New_York
    {  522, 8264,  159463, 23, }, // America/North_Dakota/Beulah
    {  522, 8292,  159486, 23, }, // America/North_Dakota/Center
    {  522, 8320,  159509, 26, }, // America/North_Dakota/New_Salem
    {  522,  393,     298,  4, }, // America/Nuuk
    {  522, 5871,   25124, 13, }, // America/Port_of_Spain
    {  522, 5760,  149885,  9, }, // America/Puerto_Rico
    {  522, 6862,     302, 16, }, // America/Scoresbysund
    {  522, 8471,  159535, 16, }, // America/St_Barthelemy
    {  522, 2045,     332, 10, }, // America/St_Johns
    {  522, 5780,   25182, 11, }, // America/St_Kitts
    {  522, 5797,   34444, 11, }, // America/St_Lucia
    {  522,  855,   42797, 12, }, // America/St_Thomas
    {  522, 5830,   25216, 13, }, // America/St_Vincent
    {  522, 5673,   25229,  7, }, // America/Thule
    {  522, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  522, 7337,    4048,  6, }, // Antarctica/Vostok
    {  522, 1413,  159551, 12, }, // Arctic/Longyearbyen
    {  522, 5487,  159563,  4, }, // Asia/Aden
    {  522, 5297,  149894,  6, }, // Asia/Almaty
    {  522, 5440,    5906,  5, }, // Asia/Aqtau
    {  522, 5270,  149900, 10, }, // Asia/Aqtobe
    {  522,  964,  149910,  9, }, // Asia/Ashgabat
    {  522, 3509,    4063,  6, }, // Asia/Baghdad
    {  522, 5451,   56476,  7, }, // Asia/Bahrain
    {  522, 3473,  149919,  7, }, // Asia/Barnaul
    {  522, 3988,    5930,  6, }, // Asia/Beirut
    {  522, 3680,  149926,  7, }, // Asia/Bishkek
    {  522, 4503,  149933,  5, }, // Asia/Chita
    {  522, 5963,  149938, 10, }, // Asia/Choibalsan
    {  522, 4413,  159567,  7, }, // Asia/Colombo
    {  522, 4442,  149948,  8, }, // Asia/Damascus
    {  522, 3498,   56501,  5, }, // Asia/Dubai
    {  522, 6092,  149956,  8, }, // Asia/Dushanbe
    {  522, 4697,  159574,  6, }, // Asia/Hebron
    {  522, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  522, 2576,  159580,  8, }, // Asia/Hong_Kong
    {  522, 4673,  149964,  5, }, // Asia/Hovd
    {  522, 4086,  139633,  6, }, // Asia/Irkutsk
    {  522, 6969,  149969,  8, }, // Asia/Jakarta
    {  522, 1257,  149977, 10, }, // Asia/Jerusalem
    {  522, 4218,  149987,  9, }, // Asia/Kamchatka
    {  522, 4141,  149996,  7, }, // Asia/Karachi
    {  522, 1152,    6018,  8, }, // Asia/Kathmandu
    {  522, 8624,   56553,  8, }, // Asia/Khandyga
    {  522,  992,   25307,  7, }, // Asia/Kolkata
    {  522, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  522, 5464,  150003,  6, }, // Asia/Kuwait
    {  522, 1178,   32008,  5, }, // Asia/Macau
    {  522, 6883,   25325,  6, }, // Asia/Muscat
    {  522, 2446,  150009,  7, }, // Asia/Nicosia
    {  522, 8638,  150016, 12, }, // Asia/Novokuznetsk
    {  522, 4037,  150028, 11, }, // Asia/Novosibirsk
    {  522, 4131,  159588,  4, }, // Asia/Omsk
    {  522, 7108,  159592,  4, }, // Asia/Oral
    {  522, 4116,  159596,  9, }, // Asia/Pyongyang
    {  522, 5476,    4233,  5, }, // Asia/Qatar
    {  522, 8671,  150039,  8, }, // Asia/Qostanay
    {  522, 4171,  150047,  9, }, // Asia/Qyzylorda
    {  522, 3486,   32042,  5, }, // Asia/Riyadh
    {  522, 4310,   56617,  8, }, // Asia/Sakhalin
    {  522, 7208,   25356, 10, }, // Asia/Samarkand
    {  522, 3116,    6131,  4, }, // Asia/Seoul
    {  522, 1020,  150056,  8, }, // Asia/Shanghai
    {  522, 3137,   19291,  8, }, // Asia/Singapore
    {  522, 4199,  150064, 15, }, // Asia/Srednekolymsk
    {  522, 3100,    6163,  6, }, // Asia/Taipei
    {  522, 4683,  159605,  8, }, // Asia/Tashkent
    {  522, 2638,    6175,  7, }, // Asia/Tehran
    {  522, 2687,    4257,  5, }, // Asia/Tokyo
    {  522, 4474,  159613,  5, }, // Asia/Tomsk
    {  522, 1346,  150088, 10, }, // Asia/Ulaanbaatar
    {  522, 1126,  150098,  7, }, // Asia/Urumqi
    {  522, 8685,  150105,  9, }, // Asia/Ust-Nera
    {  522, 6940,  159618,  8, }, // Asia/Vientiane
    {  522, 4612,  150122, 11, }, // Asia/Vladivostok
    {  522, 4730,  139660,  6, }, // Asia/Yakutsk
    {  522, 1202,    6213,  6, }, // Asia/Yangon
    {  522, 3785,  150133, 13, }, // Asia/Yekaterinburg
    {  522, 3649,  150146,  6, }, // Asia/Yerevan
    {  522, 3565,  150152,  5, }, // Atlantic/Azores
    {  522, 5588,   56708,  7, }, // Atlantic/Bermuda
    {  522, 6591,  159626, 16, }, // Atlantic/Canary
    {  522, 3629,  159642, 30, }, // Atlantic/Cape_Verde
    {  522, 1379,  159672, 11, }, // Atlantic/Faroe
    {  522, 8699,  150214,  6, }, // Atlantic/Madeira
    {  522, 7235,  159683, 18, }, // Atlantic/South_Georgia
    {  522, 6784,  159701, 14, }, // Atlantic/St_Helena
    {  522, 6149,   32109,  6, }, // Europe/Andorra
    {  522, 3522,  150251,  9, }, // Europe/Astrakhan
    {  522, 6577,  159715,  5, }, // Europe/Athens
    {  522, 6285,   25465,  7, }, // Europe/Belgrade
    {  522, 6437,  150265, 10, }, // Europe/Bratislava
    {  522, 6178,  150275,  8, }, // Europe/Brussels
    {  522, 3830,  150283,  9, }, // Europe/Bucharest
    {  522, 3708,  150292,  9, }, // Europe/Budapest
    {  522, 8751,   61453, 21, }, // Europe/Busingen
    {  522, 2475,  159720,  9, }, // Europe/Chisinau
    {  522, 6238,  159729,  9, }, // Europe/Copenhagen
    {  522, 8783,  150310,  9, }, // Europe/Isle_of_Man
    {  522, 1097,  150319,  7, }, // Europe/Istanbul
    {  522, 8924,   76768,  5, }, // Europe/Kirov
    {  522, 2419,  159738,  5, }, // Europe/Kyiv
    {  522, 3078,  159743,  7, }, // Europe/Lisbon
    {  522, 6455,  150338,  7, }, // Europe/Ljubljana
    {  522, 2393,  159750,  6, }, // Europe/London
    {  522, 6314,  159756, 10, }, // Europe/Luxembourg
    {  522, 4794,  150351,  6, }, // Europe/Madrid
    {  522, 6531,   62205, 13, }, // Europe/Mariehamn
    {  522, 3595,  150357,  5, }, // Europe/Minsk
    {  522, 6359,  159766,  6, }, // Europe/Monaco
    {  522, 3443,   91758,  6, }, // Europe/Moscow
    {  522, 6407,  159772,  4, }, // Europe/Oslo
    {  522, 4186,  150362,  5, }, // Europe/Paris
    {  522, 6224,   19576,  5, }, // Europe/Prague
    {  522, 8816,  123014,  4, }, // Europe/Riga
    {  522, 6273,  150367,  4, }, // Europe/Rome
    {  522, 6194,  150371,  8, }, // Europe/Sarajevo
    {  522, 4353,  159776,  7, }, // Europe/Saratov
    {  522, 8828,  150386, 10, }, // Europe/Simferopol
    {  522, 6472,  150396,  9, }, // Europe/Stockholm
    {  522, 8846,   21335,  6, }, // Europe/Tallinn
    {  522, 6135,   19608,  6, }, // Europe/Tirane
    {  522, 8861,   76784,  9, }, // Europe/Ulyanovsk
    {  522, 6516,  150405,  7, }, // Europe/Vatican
    {  522, 6164,  159783,  6, }, // Europe/Vienna
    {  522, 8878,  150418,  5, }, // Europe/Vilnius
    {  522, 4629,  150423,  9, }, // Europe/Volgograd
    {  522, 3055,   57212,  8, }, // Europe/Warsaw
    {  522, 6210,  150432,  7, }, // Europe/Zagreb
    {  522, 6489,  150439,  6, }, // Europe/Zurich
    {  522, 4980,  150445, 12, }, // Indian/Antananarivo
    {  522, 6910,  150457,  6, }, // Indian/Chagos
    {  522, 5979,  159789, 12, }, // Indian/Christmas
    {  522, 5996,  150486, 14, }, // Indian/Cocos
    {  522, 4931,  159801,  6, }, // Indian/Comoro
    {  522, 6624,  159807, 18, }, // Indian/Kerguelen
    {  522, 7223,   25565,  4, }, // Indian/Mahe
    {  522, 7013,  123666,  8, }, // Indian/Maldives
    {  522, 5000,  150516,  7, }, // Indian/Mayotte
    {  522, 7174,     467,  7, }, // Indian/Reunion
    {  522, 3608,  159825, 22, }, // Pacific/Bougainville
    {  522, 3022,     474,  5, }, // Pacific/Chuuk
    {  522, 2201,  159847, 17, }, // Pacific/Easter
    {  522, 3804,  150560,  5, }, // Pacific/Fiji
    {  522, 6659,  150565, 14, }, // Pacific/Gambier
    {  522, 2897,    4520,  9, }, // Pacific/Kanton
    {  522, 3953,  150579,  7, }, // Pacific/Marquesas
    {  522, 7061,  120279,  5, }, // Pacific/Noumea
    {  522, 2961,     479,  7, }, // Pacific/Pohnpei
    {  523, 6751,  159864,  7, }, // Africa/Abidjan Sindhi/Arabic/Pakistan
    {  523, 6723,  159871,  5, }, // Africa/Accra
    {  523, 4961,  159876, 10, }, // Africa/Addis_Ababa
    {  523, 7383,  159886,  6, }, // Africa/Algiers
    {  523,   14,  159892,  6, }, // Africa/Asmara
    {  523,   44,  159898,  6, }, // Africa/Bamako
    {  523, 5159,  159904,  4, }, // Africa/Bangui
    {  523, 6709,  159908,  6, }, // Africa/Banjul
    {  523, 7398,  159914,  5, }, // Africa/Bissau
    {  523, 4859,  159919,  7, }, // Africa/Blantyre
    {  523, 5189,  144441,  8, }, // Africa/Brazzaville
    {  523, 4824,  159926,  8, }, // Africa/Bujumbura
    {  523, 2242,  159934,  5, }, // Africa/Cairo
    {  523, 4019,  159939, 10, }, // Africa/Casablanca
    {  523, 7412,  159949,  5, }, // Africa/Ceuta
    {  523, 6736,  159954,  7, }, // Africa/Conakry
    {  523, 6803,  159961,  5, }, // Africa/Dakar
    {  523, 5032,  101793,  9, }, // Africa/Dar_es_Salaam
    {  523, 4945,  144487,  5, }, // Africa/Djibouti
    {  523, 5145,    8990,  5, }, // Africa/Douala
    {  523, 5068,  159966,  7, }, // Africa/El_Aaiun
    {  523, 6816,  159973,  9, }, // Africa/Freetown
    {  523, 4808,  159982,  7, }, // Africa/Gaborone
    {  523, 4917,    9016,  5, }, // Africa/Harare
    {  523, 4381,  159989,  9, }, // Africa/Johannesburg
    {  523, 4401,    9030,  4, }, // Africa/Juba
    {  523, 5053,  159998,  6, }, // Africa/Kampala
    {  523, 4426,    9043,  5, }, // Africa/Khartoum
    {  523, 4875,  160004,  6, }, // Africa/Kigali
    {  523, 5208,  160010,  6, }, // Africa/Kinshasa
    {  523, 4646,  144547,  5, }, // Africa/Lagos
    {  523, 5238,  160016,  8, }, // Africa/Libreville
    {  523, 6832,    9073,  3, }, // Africa/Lome
    {  523, 5113,  160024,  6, }, // Africa/Luanda
    {  523, 4841,  160030,  8, }, // Africa/Lubumbashi
    {  523, 4903,  160038,  6, }, // Africa/Lusaka
    {  523, 5224,  160044,  5, }, // Africa/Malabo
    {  523, 4889,  147255,  6, }, // Africa/Maputo
    {  523, 5099,    9109,  6, }, // Africa/Maseru
    {  523, 5084,  160049,  6, }, // Africa/Mbabane
    {  523, 5015,  160055,  8, }, // Africa/Mogadishu
    {  523, 6997,  147281,  8, }, // Africa/Monrovia
    {  523, 3755,    9137,  6, }, // Africa/Nairobi
    {  523, 5173,    9143,  7, }, // Africa/Ndjamena
    {  523, 5256,    9150,  5, }, // Africa/Niamey
    {  523, 6766,  160063,  7, }, // Africa/Nouakchott
    {  523, 6690,  160070, 12, }, // Africa/Ouagadougou
    {  523, 5127,  160082,  9, }, // Africa/Porto-Novo
    {  523, 4337,  160091,  8, }, // Africa/Sao_Tome
    {  523, 2732,  160099,  6, }, // Africa/Tripoli
    {  523, 6503,  160105,  5, }, // Africa/Tunis
    {  523, 4054,  160110,  6, }, // Africa/Windhoek
    {  523,  132,  160116,  3, }, // America/Adak
    {  523, 3183,  160119,  7, }, // America/Anchorage
    {  523, 5524,  160126,  7, }, // America/Anguilla
    {  523, 5541,  160133,  8, }, // America/Antigua
    {  523, 4456,  160141,  9, }, // America/Araguaina
    {  523,  166,  160150, 12, }, // America/Argentina/Buenos_Aires
    {  523,   91,  160162, 10, }, // America/Argentina/Catamarca
    {  523,  270,  160172,  7, }, // America/Argentina/Cordoba
    {  523,  441,  160179,  6, }, // America/Argentina/Jujuy
    {  523, 7425,  160185,  9, }, // America/Argentina/La_Rioja
    {  523,  565,  160194,  7, }, // America/Argentina/Mendoza
    {  523, 7452,  160201, 11, }, // America/Argentina/Rio_Gallegos
    {  523, 7483,  160212,  5, }, // America/Argentina/Salta
    {  523, 7507,  160217,  8, }, // America/Argentina/San_Juan
    {  523, 5497,  160225,  9, }, // America/Argentina/San_Luis
    {  523, 7534,  160234,  7, }, // America/Argentina/Tucuman
    {  523, 7560,  160241,  8, }, // America/Argentina/Ushuaia
    {  523, 5557,  147464,  5, }, // America/Aruba
    {  523, 4154,  160249,  8, }, // America/Asuncion
    {  523,  237,  160257,  7, }, // America/Atikokan
    {  523, 3581,  160264,  5, }, // America/Bahia
    {  523, 7586,  160269, 13, }, // America/Bahia_Banderas
    {  523, 5571,  160282,  8, }, // America/Barbados
    {  523, 7609,  160290,  4, }, // America/Belem
    {  523, 5309,  160294,  5, }, // America/Belize
    {  523, 7623,  160299, 11, }, // America/Blanc-Sablon
    {  523, 7644,  160310,  9, }, // America/Boa_Vista
    {  523, 4263,  160319,  5, }, // America/Bogota
    {  523, 7662,  160324,  4, }, // America/Boise
    {  523, 7676,  160328,  9, }, // America/Cambridge_Bay
    {  523, 7698,  160337, 12, }, // America/Campo_Grande
    {  523, 3770,  160349,  6, }, // America/Cancun
    {  523, 4596,  160355,  5, }, // America/Caracas
    {  523, 4247,  160360,  5, }, // America/Cayenne
    {  523, 5398,  160365,  5, }, // America/Cayman
    {  523, 3251,  160370,  5, }, // America/Chicago
    {  523, 7719,  160375, 10, }, // America/Chihuahua
    {  523, 7737,  160385, 11, }, // America/Ciudad_Juarez
    {  523, 5324,  160396, 10, }, // America/Costa_Rica
    {  523, 7759,  160406,  6, }, // America/Creston
    {  523, 3693,  160412,  7, }, // America/Cuiaba
    {  523, 5640,  160419,  9, }, // America/Curacao
    {  523, 7775,  160428, 11, }, // America/Danmarkshavn
    {  523, 7796,  160439,  5, }, // America/Dawson
    {  523, 7811,  160444, 10, }, // America/Dawson_Creek
    {  523,  805,  160454,  5, }, // America/Denver
    {  523, 3351,  160459,  8, }, // America/Detroit
    {  523, 5656,  160467,  8, }, // America/Dominica
    {  523,  893,  160475,  8, }, // America/Edmonton
    {  523, 7832,  160483,  7, }, // America/Eirunepe
    {  523, 5343,  160490, 11, }, // America/El_Salvador
    {  523, 7849,  160501, 10, }, // America/Fort_Nelson
    {  523, 7869,  160511,  9, }, // America/Fortaleza
    {  523, 7887,  160520,  7, }, // America/Glace_Bay
    {  523, 6844,  145047,  6, }, // America/Goose_Bay
    {  523, 4514,  160527,  9, }, // America/Grand_Turk
    {  523, 5687,  160536,  7, }, // America/Grenada
    {  523, 5703,  160543,  8, }, // America/Guadeloupe
    {  523, 3662,  160551, 10, }, // America/Guatemala
    {  523, 4743,  160561,  8, }, // America/Guayaquil
    {  523, 6895,  160569,  5, }, // America/Guyana
    {  523, 1923,  160574,  9, }, // America/Halifax
    {  523, 2221,  145097,  5, }, // America/Havana
    {  523, 7905,  160583,  7, }, // America/Hermosillo
    {  523,  348,  160590, 12, }, // America/Indiana/Indianapolis
    {  523,  481,  160602, 13, }, // America/Indiana/Knox
    {  523, 7924,  160615, 15, }, // America/Indiana/Marengo
    {  523, 7948,  160630, 17, }, // America/Indiana/Petersburg
    {  523, 7975,  160647, 16, }, // America/Indiana/Tell_City
    {  523, 8001,  160663, 13, }, // America/Indiana/Vevay
    {  523, 8023,  160676, 16, }, // America/Indiana/Vincennes
    {  523, 8049,  160692, 16, }, // America/Indiana/Winamac
    {  523, 8073,  160708,  5, }, // America/Inuvik
    {  523,  660,  160713,  7, }, // America/Iqaluit
    {  523, 2665,  160720,  6, }, // America/Jamaica
    {  523, 5282,  160726,  5, }, // America/Juneau
    {  523,  521,  160731,  8, }, // America/Kentucky/Louisville
    {  523, 8088,  160739, 17, }, // America/Kentucky/Monticello
    {  523, 5621,  160756,  8, }, // America/Kralendijk
    {  523, 4278,  160764,  5, }, // America/La_Paz
    {  523, 7132,    9813,  4, }, // America/Lima
    {  523, 3390,  160769, 10, }, // America/Los_Angeles
    {  523, 5849,  160779, 17, }, // America/Lower_Princes
    {  523, 8116,  160796,  6, }, // America/Maceio
    {  523, 8131,  147984,  7, }, // America/Managua
    {  523, 1892,  160802,  6, }, // America/Manaus
    {  523, 5814,  160808,  7, }, // America/Marigot
    {  523, 5722,  160815,  7, }, // America/Martinique
    {  523, 8147,  160822,  8, }, // America/Matamoros
    {  523, 2779,  160830,  7, }, // America/Mazatlan
    {  523, 8165,  160837,  8, }, // America/Menominee
    {  523, 8183,  160845,  6, }, // America/Merida
    {  523, 8198,  160851, 11, }, // America/Metlakatla
    {  523, 2811,  160862, 11, }, // America/Mexico_City
    {  523, 4293,  160873,  9, }, // America/Miquelon
    {  523, 8217,  160882,  6, }, // America/Moncton
    {  523, 8233,  160888,  7, }, // America/Monterrey
    {  523, 4000,  160895, 11, }, // America/Montevideo
    {  523, 5741,  160906,  8, }, // America/Montserrat
    {  523, 5383,  148084,  6, }, // America/Nassau
    {  523, 3294,  160914,  7, }, // America/New_York
    {  523, 8251,    9897,  3, }, // America/Nome
    {  523, 1834,  160921,  8, }, // America/Noronha
    {  523, 8264,  160929, 17, }, // America/North_Dakota/Beulah
    {  523, 8292,  160946, 16, }, // America/North_Dakota/Center
    {  523, 8320,  160962, 19, }, // America/North_Dakota/New_Salem
    {  523,  393,  160981,  4, }, // America/Nuuk
    {  523, 8351,  160985,  8, }, // America/Ojinaga
    {  523, 5413,  160993,  5, }, // America/Panama
    {  523, 6106,  160998, 10, }, // America/Paramaribo
    {  523, 3224,  161008,  6, }, // America/Phoenix
    {  523, 3847,  161014, 12, }, // America/Port-au-Prince
    {  523, 5871,  161026, 12, }, // America/Port_of_Spain
    {  523, 8367,  161038, 11, }, // America/Porto_Velho
    {  523, 5760,  161049, 10, }, // America/Puerto_Rico
    {  523, 3932,  161059, 11, }, // America/Punta_Arenas
    {  523, 8387,  161070, 11, }, // America/Rankin_Inlet
    {  523, 8408,  161081,  6, }, // America/Recife
    {  523, 1979,  161087,  6, }, // America/Regina
    {  523, 8423,  161093,  8, }, // America/Resolute
    {  523,  695,  161101, 10, }, // America/Rio_Branco
    {  523, 8440,  161111,  7, }, // America/Santarem
    {  523, 2165,  161118,  8, }, // America/Santiago
    {  523, 6044,  161126, 13, }, // America/Santo_Domingo
    {  523, 1862,  161139,  9, }, // America/Sao_Paulo
    {  523, 6862,  161148, 12, }, // America/Scoresbysund
    {  523, 8457,  161160,  4, }, // America/Sitka
    {  523, 8471,  161164, 12, }, // America/St_Barthelemy
    {  523, 2045,  161176,  9, }, // America/St_Johns
    {  523, 5780,  161185,  8, }, // America/St_Kitts
    {  523, 5797,  161193, 10, }, // America/St_Lucia
    {  523,  855,  161203,  9, }, // America/St_Thomas
    {  523, 5830,  161212, 11, }, // America/St_Vincent
    {  523, 8493,  161223, 10, }, // America/Swift_Current
    {  523, 5363,  161233, 10, }, // America/Tegucigalpa
    {  523, 5673,  161243,  4, }, // America/Thule
    {  523,  313,  161247,  7, }, // America/Tijuana
    {  523,  608,  161254,  6, }, // America/Toronto
    {  523, 5605,  161260,  7, }, // America/Tortola
    {  523, 2077,  161267,  7, }, // America/Vancouver
    {  523, 2128,  161274, 11, }, // America/Whitehorse
    {  523,  734,  161285,  7, }, // America/Winnipeg
    {  523, 8515,  161292,  7, }, // America/Yakutat
    {  523, 5918,  161299,  4, }, // Antarctica/Casey
    {  523, 6027,  161303,  4, }, // Antarctica/Davis
    {  523, 6066,  161307, 13, }, // Antarctica/DumontDUrville
    {  523, 8531,  161320,  7, }, // Antarctica/Macquarie
    {  523, 7029,  161327,  4, }, // Antarctica/Mawson
    {  523, 7076,  161331,  8, }, // Antarctica/McMurdo
    {  523, 8552,  161339,  4, }, // Antarctica/Palmer
    {  523, 7189,  161343,  5, }, // Antarctica/Rothera
    {  523, 7258,  161348,  5, }, // Antarctica/Syowa
    {  523, 8570,  161353,  4, }, // Antarctica/Troll
    {  523, 7337,  161357,  6, }, // Antarctica/Vostok
    {  523, 1413,  161363, 11, }, // Arctic/Longyearbyen
    {  523, 5487,   10423,  3, }, // Asia/Aden
    {  523, 5297,  161374,  6, }, // Asia/Almaty
    {  523, 3870,    9854,  4, }, // Asia/Amman
    {  523, 5428,  161380,  5, }, // Asia/Anadyr
    {  523, 5440,  161385,  6, }, // Asia/Aqtau
    {  523, 5270,  161391,  6, }, // Asia/Aqtobe
    {  523,  964,  161397,  7, }, // Asia/Ashgabat
    {  523, 8587,  161404,  7, }, // Asia/Atyrau
    {  523, 3509,   10466,  5, }, // Asia/Baghdad
    {  523, 5451,   10473,  5, }, // Asia/Bahrain
    {  523, 3555,  160177,  4, }, // Asia/Baku
    {  523, 4368,  161411,  6, }, // Asia/Bangkok
    {  523, 3473,  161417,  4, }, // Asia/Barnaul
    {  523, 3988,   10491,  5, }, // Asia/Beirut
    {  523, 3680,  161421,  5, }, // Asia/Bishkek
    {  523, 5906,  161426,  7, }, // Asia/Brunei
    {  523, 4503,  145950,  4, }, // Asia/Chita
    {  523, 5963,  161433, 10, }, // Asia/Choibalsan
    {  523, 4413,  161443,  6, }, // Asia/Colombo
    {  523, 4442,   10528,  4, }, // Asia/Damascus
    {  523, 1060,  161449,  4, }, // Asia/Dhaka
    {  523, 6125,  145973,  3, }, // Asia/Dili
    {  523, 3498,  161453,  4, }, // Asia/Dubai
    {  523, 6092,   10539,  7, }, // Asia/Dushanbe
    {  523, 8599,  148588,  9, }, // Asia/Famagusta
    {  523, 8614,  145994,  3, }, // Asia/Gaza
    {  523, 4697,   10558,  6, }, // Asia/Hebron
    {  523, 1226,  161457,  7, }, // Asia/Ho_Chi_Minh
    {  523, 2576,  161464,  9, }, // Asia/Hong_Kong
    {  523, 4673,  161473,  4, }, // Asia/Hovd
    {  523, 4086,  161477,  6, }, // Asia/Irkutsk
    {  523, 6969,  161483,  6, }, // Asia/Jakarta
    {  523, 6955,  161489,  7, }, // Asia/Jayapura
    {  523, 1257,  161496,  7, }, // Asia/Jerusalem
    {  523, 3462,  161503,  4, }, // Asia/Kabul
    {  523, 4218,  161507,  7, }, // Asia/Kamchatka
    {  523, 4141,  161514,  5, }, // Asia/Karachi
    {  523, 1152,  161519,  6, }, // Asia/Kathmandu
    {  523, 8624,  161525,  7, }, // Asia/Khandyga
    {  523,  992,  161532,  5, }, // Asia/Kolkata
    {  523, 4099,  161537, 11, }, // Asia/Krasnoyarsk
    {  523, 4761,  161548, 10, }, // Asia/Kuala_Lumpur
    {  523, 5893,  161558,  5, }, // Asia/Kuching
    {  523, 5464,  161563,  4, }, // Asia/Kuwait
    {  523, 1178,  161567,  5, }, // Asia/Macau
    {  523, 3919,  161572,  6, }, // Asia/Magadan
    {  523, 1316,  161578,  6, }, // Asia/Makassar
    {  523, 7145,  161584,  5, }, // Asia/Manila
    {  523, 6883,   10734,  4, }, // Asia/Muscat
    {  523, 2446,  161589,  7, }, // Asia/Nicosia
    {  523, 8638,  161596, 10, }, // Asia/Novokuznetsk
    {  523, 4037,  161606, 11, }, // Asia/Novosibirsk
    {  523, 4131,  161617,  5, }, // Asia/Omsk
    {  523, 7108,  161622,  5, }, // Asia/Oral
    {  523, 6924,  161627,  9, }, // Asia/Phnom_Penh
    {  523, 8656,  161636,  8, }, // Asia/Pontianak
    {  523, 4116,  161644, 10, }, // Asia/Pyongyang
    {  523, 5476,   10802,  3, }, // Asia/Qatar
    {  523, 8671,  161654,  7, }, // Asia/Qostanay
    {  523, 4171,  161661,  8, }, // Asia/Qyzylorda
    {  523, 3486,   10824,  4, }, // Asia/Riyadh
    {  523, 4310,  161669,  6, }, // Asia/Sakhalin
    {  523, 7208,   10834,  6, }, // Asia/Samarkand
    {  523, 3116,  161675,  4, }, // Asia/Seoul
    {  523, 1020,  161679,  7, }, // Asia/Shanghai
    {  523, 3137,  148836,  7, }, // Asia/Singapore
    {  523, 4199,  161686, 12, }, // Asia/Srednekolymsk
    {  523, 3100,  161698,  5, }, // Asia/Taipei
    {  523, 4683,  161703,  6, }, // Asia/Tashkent
    {  523, 3817,   10880,  6, }, // Asia/Tbilisi
    {  523, 2638,  146255,  5, }, // Asia/Tehran
    {  523, 1284,  161709,  4, }, // Asia/Thimphu
    {  523, 2687,  161713,  5, }, // Asia/Tokyo
    {  523, 4474,  161718,  4, }, // Asia/Tomsk
    {  523, 1346,  161722, 10, }, // Asia/Ulaanbaatar
    {  523, 1126,  161732,  6, }, // Asia/Urumqi
    {  523, 8685,  161738,  8, }, // Asia/Ust-Nera
    {  523, 6940,  161746,  7, }, // Asia/Vientiane
    {  523, 4612,  161753, 10, }, // Asia/Vladivostok
    {  523, 4730,  161763,  6, }, // Asia/Yakutsk
    {  523, 1202,  161769,  5, }, // Asia/Yangon
    {  523, 3785,  161774,  9, }, // Asia/Yekaterinburg
    {  523, 3649,  146335,  6, }, // Asia/Yerevan
    {  523, 3565,  161783,  5, }, // Atlantic/Azores
    {  523, 5588,   10981,  6, }, // Atlantic/Bermuda
    {  523, 6591,  161788,  5, }, // Atlantic/Canary
    {  523, 3629,  161793,  8, }, // Atlantic/Cape_Verde
    {  523, 1379,  161801,  4, }, // Atlantic/Faroe
    {  523, 8699,  161805,  6, }, // Atlantic/Madeira
    {  523, 2614,  161811,  6, }, // Atlantic/Reykjavik
    {  523, 7235,  161817, 10, }, // Atlantic/South_Georgia
    {  523, 6784,  161827, 11, }, // Atlantic/St_Helena
    {  523, 6607,  161838,  7, }, // Atlantic/Stanley
    {  523, 1654,  161845,  7, }, // Australia/Adelaide
    {  523, 1619,  161852,  6, }, // Australia/Brisbane
    {  523, 1783,  161858,  8, }, // Australia/Broken_Hill
    {  523, 1567,  161866,  5, }, // Australia/Darwin
    {  523, 3539,  161871,  5, }, // Australia/Eucla
    {  523, 1500,  161876,  6, }, // Australia/Hobart
    {  523, 8716,  161882,  6, }, // Australia/Lindeman
    {  523, 1531,  161888,  9, }, // Australia/Lord_Howe
    {  523, 1711,   11108,  7, }, // Australia/Melbourne
    {  523, 1746,  161897,  3, }, // Australia/Perth
    {  523, 1447,  161900,  4, }, // Australia/Sydney
    {  523, 6390,  161904,  9, }, // Europe/Amsterdam
    {  523, 6149,  146476,  6, }, // Europe/Andorra
    {  523, 3522,  149086,  8, }, // Europe/Astrakhan
    {  523, 6577,  161913,  5, }, // Europe/Athens
    {  523, 6285,   11151,  6, }, // Europe/Belgrade
    {  523, 4659,  146501,  4, }, // Europe/Berlin
    {  523, 6437,  161918,  9, }, // Europe/Bratislava
    {  523, 6178,  161927,  5, }, // Europe/Brussels
    {  523, 3830,  161932,  7, }, // Europe/Bucharest
    {  523, 3708,  161939,  7, }, // Europe/Budapest
    {  523, 8751,  161946,  6, }, // Europe/Busingen
    {  523, 2475,  161952,  7, }, // Europe/Chisinau
    {  523, 6238,  161959,  9, }, // Europe/Copenhagen
    {  523, 2260,  161968,  4, }, // Europe/Dublin
    {  523, 6256,  161972,  7, }, // Europe/Gibraltar
    {  523, 8767,  161979,  5, }, // Europe/Guernsey
    {  523, 6561,  161984,  7, }, // Europe/Helsinki
    {  523, 8783,  161991, 12, }, // Europe/Isle_of_Man
    {  523, 1097,  146592,  7, }, // Europe/Istanbul
    {  523, 8802,  162003,  4, }, // Europe/Jersey
    {  523, 3881,  162007, 10, }, // Europe/Kaliningrad
    {  523, 8924,  162017,  5, }, // Europe/Kirov
    {  523, 2419,  162022,  4, }, // Europe/Kyiv
    {  523, 3078,  162026,  4, }, // Europe/Lisbon
    {  523, 6455,  162030,  7, }, // Europe/Ljubljana
    {  523, 2393,  162037,  4, }, // Europe/London
    {  523, 6314,  162041,  7, }, // Europe/Luxembourg
    {  523, 4794,  162048,  5, }, // Europe/Madrid
    {  523, 6346,  162053,  5, }, // Europe/Malta
    {  523, 6531,  146651,  7, }, // Europe/Mariehamn
    {  523, 3595,  162058,  4, }, // Europe/Minsk
    {  523, 6359,  162062,  6, }, // Europe/Monaco
    {  523, 3443,  162068,  5, }, // Europe/Moscow
    {  523, 6407,  104355,  5, }, // Europe/Oslo
    {  523, 4186,  162073,  4, }, // Europe/Paris
    {  523, 6373,  162077,  9, }, // Europe/Podgorica
    {  523, 6224,  104374,  4, }, // Europe/Prague
    {  523, 8816,  148950,  3, }, // Europe/Riga
    {  523, 6273,    9113,  3, }, // Europe/Rome
    {  523, 4233,  146680,  5, }, // Europe/Samara
    {  523, 6419,  162086,  9, }, // Europe/San_Marino
    {  523, 6194,  162095,  7, }, // Europe/Sarajevo
    {  523, 4353,  162102,  5, }, // Europe/Saratov
    {  523, 8828,  162107,  8, }, // Europe/Simferopol
    {  523, 6332,  162115,  6, }, // Europe/Skopje
    {  523, 6548,  162121,  5, }, // Europe/Sofia
    {  523, 6472,  162126,  9, }, // Europe/Stockholm
    {  523, 8846,  162135,  4, }, // Europe/Tallinn
    {  523, 6135,  162139,  5, }, // Europe/Tirane
    {  523, 8861,  162144,  8, }, // Europe/Ulyanovsk
    {  523, 6301,  162152,  4, }, // Europe/Vaduz
    {  523, 6516,  162156,  6, }, // Europe/Vatican
    {  523, 6164,    9751,  4, }, // Europe/Vienna
    {  523, 8878,  162162,  6, }, // Europe/Vilnius
    {  523, 4629,  162168,  9, }, // Europe/Volgograd
    {  523, 3055,  104500,  5, }, // Europe/Warsaw
    {  523, 6210,   11482,  4, }, // Europe/Zagreb
    {  523, 6489,   11486,  5, }, // Europe/Zurich
    {  523, 4980,  162177, 12, }, // Indian/Antananarivo
    {  523, 6910,  104528,  5, }, // Indian/Chagos
    {  523, 5979,  162189,  5, }, // Indian/Christmas
    {  523, 5996,  162194,  5, }, // Indian/Cocos
    {  523, 4931,  162199,  6, }, // Indian/Comoro
    {  523, 6624,  162205,  8, }, // Indian/Kerguelen
    {  523, 7223,   11539,  4, }, // Indian/Mahe
    {  523, 7013,  146817,  6, }, // Indian/Maldives
    {  523, 3971,  162213,  6, }, // Indian/Mauritius
    {  523, 5000,  162219,  5, }, // Indian/Mayotte
    {  523, 7174,  146835,  8, }, // Indian/Reunion
    {  523, 4324,  162224,  4, }, // Pacific/Apia
    {  523,  932,  162228,  6, }, // Pacific/Auckland
    {  523, 3608,  162234, 11, }, // Pacific/Bougainville
    {  523, 2863,  162245,  4, }, // Pacific/Chatham
    {  523, 3022,  162249,  4, }, // Pacific/Chuuk
    {  523, 2201,  162253,  5, }, // Pacific/Easter
    {  523, 7323,  162258,  5, }, // Pacific/Efate
    {  523, 7290,  162263,  7, }, // Pacific/Fakaofo
    {  523, 3804,  146883,  3, }, // Pacific/Fiji
    {  523, 7306,  162270,  7, }, // Pacific/Funafuti
    {  523, 6641,  162277,  9, }, // Pacific/Galapagos
    {  523, 6659,  162286,  7, }, // Pacific/Gambier
    {  523, 3724,  162293, 11, }, // Pacific/Guadalcanal
    {  523, 5935,  102779,  4, }, // Pacific/Guam
    {  523, 2929,  162304,  9, }, // Pacific/Honolulu
    {  523, 2897,  162313,  8, }, // Pacific/Kanton
    {  523, 3900,  162321,  7, }, // Pacific/Kiritimati
    {  523, 6982,  162328,  7, }, // Pacific/Kosrae
    {  523, 2708,  162335,  7, }, // Pacific/Kwajalein
    {  523, 4779,  146946,  5, }, // Pacific/Majuro
    {  523, 3953,  162342,  7, }, // Pacific/Marquesas
    {  523, 8909,  162349,  4, }, // Pacific/Midway
    {  523, 7047,  162353,  5, }, // Pacific/Nauru
    {  523, 7095,  162358,  5, }, // Pacific/Niue
    {  523, 4070,  162363,  7, }, // Pacific/Norfolk
    {  523, 7061,  162370,  7, }, // Pacific/Noumea
    {  523, 2991,  162377,  9, }, // Pacific/Pago_Pago
    {  523, 7118,  162386,  5, }, // Pacific/Palau
    {  523, 7157,  162391,  6, }, // Pacific/Pitcairn
    {  523, 2961,  162397,  7, }, // Pacific/Pohnpei
    {  523, 4709,  162404, 11, }, // Pacific/Port_Moresby
    {  523, 6009,  162415,  9, }, // Pacific/Rarotonga
    {  523, 5948,  162424,  4, }, // Pacific/Saipan
    {  523, 7275,  162428,  5, }, // Pacific/Tahiti
    {  523, 6675,  162433,  5, }, // Pacific/Tarawa
    {  523, 4485,  162438,  9, }, // Pacific/Tongatapu
    {  523, 7355,  162447,  3, }, // Pacific/Wake
    {  523, 7368,  162450,  4, }, // Pacific/Wallis
    {  524,   14,       0,  6, }, // Africa/Asmara Sindhi/Devanagari/India
    {  524, 4337,       6,  8, }, // Africa/Sao_Tome
    {  524, 4154,      14,  8, }, // America/Asuncion
    {  524,  237,      22,  8, }, // America/Atikokan
    {  524, 7586,      30, 17, }, // America/Bahia_Banderas
    {  524, 3770,      47,  6, }, // America/Cancun
    {  524, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  524, 5640,      66,  7, }, // America/Curacao
    {  524,  481,      73, 13, }, // America/Indiana/Knox
    {  524, 7924,      86, 16, }, // America/Indiana/Marengo
    {  524, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  524, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  524, 8001,     139, 14, }, // America/Indiana/Vevay
    {  524, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  524, 8049,     171, 16, }, // America/Indiana/Winamac
    {  524, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  524, 5849,     207, 22, }, // America/Lower_Princes
    {  524, 8183,     229,  6, }, // America/Merida
    {  524, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  524, 8292,     255, 20, }, // America/North_Dakota/Center
    {  524, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  524,  393,     298,  4, }, // America/Nuuk
    {  524, 6862,     302, 16, }, // America/Scoresbysund
    {  524, 8471,     318, 14, }, // America/St_Barthelemy
    {  524, 2045,     332, 10, }, // America/St_Johns
    {  524, 5780,     342,  9, }, // America/St_Kitts
    {  524, 5797,     351,  9, }, // America/St_Lucia
    {  524,  855,     360, 10, }, // America/St_Thomas
    {  524, 5830,     370, 11, }, // America/St_Vincent
    {  524, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  524, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  524, 1152,     413,  9, }, // Asia/Kathmandu
    {  524,  992,     422,  7, }, // Asia/Kolkata
    {  524, 1178,     429,  5, }, // Asia/Macau
    {  524, 1202,     442,  6, }, // Asia/Yangon
    {  524, 1379,     448,  5, }, // Atlantic/Faroe
    {  524, 6784,     453, 10, }, // Atlantic/St_Helena
    {  524, 2419,     463,  4, }, // Europe/Kyiv
    {  524, 7174,     467,  7, }, // Indian/Reunion
    {  524, 3022,     474,  5, }, // Pacific/Chuuk
    {  524, 2961,     479,  7, }, // Pacific/Pohnpei
    {  525, 6751,  162454,  7, }, // Africa/Abidjan Sinhala/Sinhala/Sri Lanka
    {  525, 6723,  162461,  5, }, // Africa/Accra
    {  525, 4961,  162466, 11, }, // Africa/Addis_Ababa
    {  525, 7383,  162477, 10, }, // Africa/Algiers
    {  525,   14,  162487,  7, }, // Africa/Asmara
    {  525,   44,  162494,  4, }, // Africa/Bamako
    {  525, 5159,  162498,  7, }, // Africa/Bangui
    {  525, 6709,  162505,  7, }, // Africa/Banjul
    {  525, 7398,  162512,  4, }, // Africa/Bissau
    {  525, 4859,  162516,  9, }, // Africa/Blantyre
    {  525, 5189,  162525, 10, }, // Africa/Brazzaville
    {  525, 4824,  162535, 10, }, // Africa/Bujumbura
    {  525, 2242,  162545,  5, }, // Africa/Cairo
    {  525, 4019,  162550, 10, }, // Africa/Casablanca
    {  525, 7412,  162560,  6, }, // Africa/Ceuta
    {  525, 6736,  162566,  8, }, // Africa/Conakry
    {  525, 6803,  162574,  4, }, // Africa/Dakar
    {  525, 5032,  162578, 14, }, // Africa/Dar_es_Salaam
    {  525, 4945,  162592,  6, }, // Africa/Djibouti
    {  525, 5145,  162598,  5, }, // Africa/Douala
    {  525, 5068,  162603,  9, }, // Africa/El_Aaiun
    {  525, 6816,  162612, 10, }, // Africa/Freetown
    {  525, 4808,  162622,  7, }, // Africa/Gaborone
    {  525, 4917,  162629,  5, }, // Africa/Harare
    {  525, 4381,  162634, 13, }, // Africa/Johannesburg
    {  525, 4401,  162647,  4, }, // Africa/Juba
    {  525, 5053,  162651,  7, }, // Africa/Kampala
    {  525, 4426,  162658,  8, }, // Africa/Khartoum
    {  525, 4875,  162666,  6, }, // Africa/Kigali
    {  525, 5208,  162672,  8, }, // Africa/Kinshasa
    {  525, 4646,  162680,  6, }, // Africa/Lagos
    {  525, 5238,  162686,  9, }, // Africa/Libreville
    {  525, 6832,  162695,  4, }, // Africa/Lome
    {  525, 5113,  162699,  7, }, // Africa/Luanda
    {  525, 4841,  162706, 10, }, // Africa/Lubumbashi
    {  525, 4903,  162716,  6, }, // Africa/Lusaka
    {  525, 5224,  162722,  4, }, // Africa/Malabo
    {  525, 4889,  162726,  5, }, // Africa/Maputo
    {  525, 5099,  162731,  5, }, // Africa/Maseru
    {  525, 5084,  162736,  7, }, // Africa/Mbabane
    {  525, 5015,  162743,  7, }, // Africa/Mogadishu
    {  525, 6997,  162750, 10, }, // Africa/Monrovia
    {  525, 3755,  162760,  7, }, // Africa/Nairobi
    {  525, 5173,  162767,  7, }, // Africa/Ndjamena
    {  525, 5256,  162774,  6, }, // Africa/Niamey
    {  525, 6766,  162780,  9, }, // Africa/Nouakchott
    {  525, 6690,  162789,  7, }, // Africa/Ouagadougou
    {  525, 5127,  162796, 11, }, // Africa/Porto-Novo
    {  525, 4337,  162807,  8, }, // Africa/Sao_Tome
    {  525, 2732,  162815,  9, }, // Africa/Tripoli
    {  525, 6503,  162824,  8, }, // Africa/Tunis
    {  525, 4054,  162832, 11, }, // Africa/Windhoek
    {  525,  132,  162843,  4, }, // America/Adak
    {  525, 3183,  162847,  8, }, // America/Anchorage
    {  525, 5524,  162855, 10, }, // America/Anguilla
    {  525, 5541,  162865,  9, }, // America/Antigua
    {  525, 4456,  162874,  9, }, // America/Araguaina
    {  525,  166,  162883, 14, }, // America/Argentina/Buenos_Aires
    {  525,   91,  162897,  7, }, // America/Argentina/Catamarca
    {  525,  270,  162904,  8, }, // America/Argentina/Cordoba
    {  525,  441,  162912,  6, }, // America/Argentina/Jujuy
    {  525, 7425,  162918,  9, }, // America/Argentina/La_Rioja
    {  525,  565,  162927,  8, }, // America/Argentina/Mendoza
    {  525, 7452,  162935, 12, }, // America/Argentina/Rio_Gallegos
    {  525, 7483,  162947,  6, }, // America/Argentina/Salta
    {  525, 7507,  162953, 11, }, // America/Argentina/San_Juan
    {  525, 5497,  162964, 11, }, // America/Argentina/San_Luis
    {  525, 7534,  162975,  7, }, // America/Argentina/Tucuman
    {  525, 7560,  162982,  8, }, // America/Argentina/Ushuaia
    {  525, 5557,  162990,  5, }, // America/Aruba
    {  525, 4154,  162995, 11, }, // America/Asuncion
    {  525,  237,  163006,  8, }, // America/Atikokan
    {  525, 3581,  163014,  5, }, // America/Bahia
    {  525, 7586,  163019, 15, }, // America/Bahia_Banderas
    {  525, 5571,  163034,  7, }, // America/Barbados
    {  525, 7609,  163041,  6, }, // America/Belem
    {  525, 5309,  163047,  6, }, // America/Belize
    {  525, 7623,  163053, 16, }, // America/Blanc-Sablon
    {  525, 7644,  163069, 10, }, // America/Boa_Vista
    {  525, 4263,  163079,  6, }, // America/Bogota
    {  525, 7662,  163085,  6, }, // America/Boise
    {  525, 7676,  163091, 17, }, // America/Cambridge_Bay
    {  525, 7698,  163108, 15, }, // America/Campo_Grande
    {  525, 3770,  163123,  8, }, // America/Cancun
    {  525, 4596,  163131,  5, }, // America/Caracas
    {  525, 4247,  163136,  6, }, // America/Cayenne
    {  525, 5398,  163142,  5, }, // America/Cayman
    {  525, 3251,  163147,  6, }, // America/Chicago
    {  525, 7719,  163153,  8, }, // America/Chihuahua
    {  525, 7737,  163161, 17, }, // America/Ciudad_Juarez
    {  525, 5324,  163178,  9, }, // America/Costa_Rica
    {  525, 7759,  163187, 10, }, // America/Creston
    {  525, 3693,  163197,  7, }, // America/Cuiaba
    {  525, 5640,  163204,  7, }, // America/Curacao
    {  525, 7775,  163211, 15, }, // America/Danmarkshavn
    {  525, 7796,  163226,  5, }, // America/Dawson
    {  525, 7811,  163231, 13, }, // America/Dawson_Creek
    {  525,  805,  163244,  7, }, // America/Denver
    {  525, 3351,  163251,  8, }, // America/Detroit
    {  525, 5656,  163259,  8, }, // America/Dominica
    {  525,  893,  163267,  9, }, // America/Edmonton
    {  525, 7832,  163276,  9, }, // America/Eirunepe
    {  525, 5343,  163285, 12, }, // America/El_Salvador
    {  525, 7849,  163297, 12, }, // America/Fort_Nelson
    {  525, 7869,  163309,  9, }, // America/Fortaleza
    {  525, 7887,  163318, 12, }, // America/Glace_Bay
    {  525, 6844,  163330, 10, }, // America/Goose_Bay
    {  525, 4514,  163340, 14, }, // America/Grand_Turk
    {  525, 5687,  163354,  8, }, // America/Grenada
    {  525, 5703,  163362, 10, }, // America/Guadeloupe
    {  525, 3662,  163372,  8, }, // America/Guatemala
    {  525, 4743,  163380, 11, }, // America/Guayaquil
    {  525, 6895,  163391,  5, }, // America/Guyana
    {  525, 1923,  163396,  9, }, // America/Halifax
    {  525, 2221,  163405,  5, }, // America/Havana
    {  525, 7905,  163410,  8, }, // America/Hermosillo
    {  525,  348,  163418, 14, }, // America/Indiana/Indianapolis
    {  525,  481,  163432, 17, }, // America/Indiana/Knox
    {  525, 7924,  163449, 18, }, // America/Indiana/Marengo
    {  525, 7948,  163467, 23, }, // America/Indiana/Petersburg
    {  525, 7975,  163490, 20, }, // America/Indiana/Tell_City
    {  525, 8001,  163510, 15, }, // America/Indiana/Vevay
    {  525, 8023,  163525, 19, }, // America/Indiana/Vincennes
    {  525, 8049,  163544, 18, }, // America/Indiana/Winamac
    {  525, 8073,  163562,  7, }, // America/Inuvik
    {  525,  660,  163569,  9, }, // America/Iqaluit
    {  525, 2665,  163578,  8, }, // America/Jamaica
    {  525, 5282,  163586,  4, }, // America/Juneau
    {  525,  521,  163590, 10, }, // America/Kentucky/Louisville
    {  525, 8088,  163600, 19, }, // America/Kentucky/Monticello
    {  525, 5621,  163619, 14, }, // America/Kralendijk
    {  525, 4278,  163633,  6, }, // America/La_Paz
    {  525, 7132,  163639,  4, }, // America/Lima
    {  525, 3390,  163643, 13, }, // America/Los_Angeles
    {  525, 5849,  163656, 24, }, // America/Lower_Princes
    {  525, 8116,  163680,  7, }, // America/Maceio
    {  525, 8131,  163687,  5, }, // America/Managua
    {  525, 1892,  163692,  6, }, // America/Manaus
    {  525, 5814,  163698,  8, }, // America/Marigot
    {  525, 5722,  163706,  8, }, // America/Martinique
    {  525, 8147,  163714,  8, }, // America/Matamoros
    {  525, 2779,  163722,  8, }, // America/Mazatlan
    {  525, 8165,  163730,  8, }, // America/Menominee
    {  525, 8183,  163738,  6, }, // America/Merida
    {  525, 8198,  163744, 10, }, // America/Metlakatla
    {  525, 2811,  163754, 13, }, // America/Mexico_City
    {  525, 4293,  163767,  9, }, // America/Miquelon
    {  525, 8217,  163776,  9, }, // America/Moncton
    {  525, 8233,  163785,  8, }, // America/Monterrey
    {  525, 4000,  163793, 12, }, // America/Montevideo
    {  525, 5741,  163805, 12, }, // America/Montserrat
    {  525, 5383,  163817,  5, }, // America/Nassau
    {  525, 3294,  163822,  8, }, // America/New_York
    {  525, 8251,  163830,  4, }, // America/Nome
    {  525, 1834,  163834,  8, }, // America/Noronha
    {  525, 8264,  163842, 19, }, // America/North_Dakota/Beulah
    {  525, 8292,  163861, 19, }, // America/North_Dakota/Center
    {  525, 8320,  163880, 22, }, // America/North_Dakota/New_Salem
    {  525,  393,  163902,  4, }, // America/Nuuk
    {  525, 8351,  163906,  6, }, // America/Ojinaga
    {  525, 5413,  163912,  5, }, // America/Panama
    {  525, 6106,  163917,  9, }, // America/Paramaribo
    {  525, 3224,  163926,  8, }, // America/Phoenix
    {  525, 3847,  163934, 18, }, // America/Port-au-Prince
    {  525, 5871,  163952, 19, }, // America/Port_of_Spain
    {  525, 8367,  163971, 13, }, // America/Porto_Velho
    {  525, 5760,  163984, 10, }, // America/Puerto_Rico
    {  525, 3932,  163994, 14, }, // America/Punta_Arenas
    {  525, 8387,  164008, 16, }, // America/Rankin_Inlet
    {  525, 8408,  164024,  6, }, // America/Recife
    {  525, 1979,  164030,  6, }, // America/Regina
    {  525, 8423,  164036,  8, }, // America/Resolute
    {  525,  695,  164044, 13, }, // America/Rio_Branco
    {  525, 8440,  164057,  8, }, // America/Santarem
    {  525, 2165,  164065,  9, }, // America/Santiago
    {  525, 6044,  164074, 15, }, // America/Santo_Domingo
    {  525, 1862,  164089,  9, }, // America/Sao_Paulo
    {  525, 6862,  164098, 13, }, // America/Scoresbysund
    {  525, 8457,  164111,  6, }, // America/Sitka
    {  525, 8471,  164117, 14, }, // America/St_Barthelemy
    {  525, 2045,  164131, 12, }, // America/St_Johns
    {  525, 5780,  164143, 12, }, // America/St_Kitts
    {  525, 5797,  164155, 12, }, // America/St_Lucia
    {  525,  855,  164167, 11, }, // America/St_Thomas
    {  525, 5830,  164178, 15, }, // America/St_Vincent
    {  525, 8493,  164193, 15, }, // America/Swift_Current
    {  525, 5363,  164208, 11, }, // America/Tegucigalpa
    {  525, 5673,  164219,  4, }, // America/Thule
    {  525,  313,  164223,  7, }, // America/Tijuana
    {  525,  608,  164230,  7, }, // America/Toronto
    {  525, 5605,  164237,  8, }, // America/Tortola
    {  525, 2077,  164245,  9, }, // America/Vancouver
    {  525, 2128,  164254, 11, }, // America/Whitehorse
    {  525,  734,  164265,  8, }, // America/Winnipeg
    {  525, 8515,  164273,  6, }, // America/Yakutat
    {  525, 5918,  164279,  4, }, // Antarctica/Casey
    {  525, 6027,  164283,  6, }, // Antarctica/Davis
    {  525, 6066,  164289, 18, }, // Antarctica/DumontDUrville
    {  525, 8531,  164307,  8, }, // Antarctica/Macquarie
    {  525, 7029,  164315,  6, }, // Antarctica/Mawson
    {  525, 7076,  164321, 10, }, // Antarctica/McMurdo
    {  525, 8552,  164331,  7, }, // Antarctica/Palmer
    {  525, 7189,  164338,  6, }, // Antarctica/Rothera
    {  525, 7258,  164344,  6, }, // Antarctica/Syowa
    {  525, 8570,  164350,  7, }, // Antarctica/Troll
    {  525, 7337,  164357,  8, }, // Antarctica/Vostok
    {  525, 1413,  164365, 16, }, // Arctic/Longyearbyen
    {  525, 5487,  164381,  5, }, // Asia/Aden
    {  525, 5297,  164386,  6, }, // Asia/Almaty
    {  525, 3870,  164392,  7, }, // Asia/Amman
    {  525, 5428,  164399,  9, }, // Asia/Anadyr
    {  525, 5440,  164408,  5, }, // Asia/Aqtau
    {  525, 5270,  164413,  7, }, // Asia/Aqtobe
    {  525,  964,  164420,  8, }, // Asia/Ashgabat
    {  525, 8587,  164428,  6, }, // Asia/Atyrau
    {  525, 3509,  164434,  8, }, // Asia/Baghdad
    {  525, 5451,  164442,  6, }, // Asia/Bahrain
    {  525, 3555,  163202,  4, }, // Asia/Baku
    {  525, 4368,  164448,  7, }, // Asia/Bangkok
    {  525, 3473,  164455,  7, }, // Asia/Barnaul
    {  525, 3988,  164462,  6, }, // Asia/Beirut
    {  525, 3680,  164468,  8, }, // Asia/Bishkek
    {  525, 5906,  164476,  6, }, // Asia/Brunei
    {  525, 4503,  164482,  4, }, // Asia/Chita
    {  525, 5963,  164486, 11, }, // Asia/Choibalsan
    {  525, 4413,  164497,  4, }, // Asia/Colombo
    {  525, 4442,  164501,  8, }, // Asia/Damascus
    {  525, 1060,  164509,  3, }, // Asia/Dhaka
    {  525, 6125,  164512,  4, }, // Asia/Dili
    {  525, 3498,  164516,  6, }, // Asia/Dubai
    {  525, 6092,  164522,  7, }, // Asia/Dushanbe
    {  525, 8599,  164529,  9, }, // Asia/Famagusta
    {  525, 8614,  164538,  4, }, // Asia/Gaza
    {  525, 4697,  164542,  8, }, // Asia/Hebron
    {  525, 1226,  164550, 13, }, // Asia/Ho_Chi_Minh
    {  525, 2576,  164563,  6, }, // Asia/Hong_Kong
    {  525, 4673,  164569,  6, }, // Asia/Hovd
    {  525, 4086,  164575, 10, }, // Asia/Irkutsk
    {  525, 6969,  164585,  6, }, // Asia/Jakarta
    {  525, 6955,  164591,  5, }, // Asia/Jayapura
    {  525, 1257,  164596,  7, }, // Asia/Jerusalem
    {  525, 3462,  164603,  6, }, // Asia/Kabul
    {  525, 4218,  164609,  8, }, // Asia/Kamchatka
    {  525, 4141,  164617,  8, }, // Asia/Karachi
    {  525, 1152,  164625,  8, }, // Asia/Kathmandu
    {  525, 8624,  164633,  7, }, // Asia/Khandyga
    {  525,  992,  164640,  6, }, // Asia/Kolkata
    {  525, 4099,  164646, 16, }, // Asia/Krasnoyarsk
    {  525, 4761,  164662, 13, }, // Asia/Kuala_Lumpur
    {  525, 5893,  164675,  5, }, // Asia/Kuching
    {  525, 5464,  164680,  6, }, // Asia/Kuwait
    {  525, 1178,  164686,  6, }, // Asia/Macau
    {  525, 3919,  164692,  5, }, // Asia/Magadan
    {  525, 1316,  164697,  9, }, // Asia/Makassar
    {  525, 7145,  164706,  6, }, // Asia/Manila
    {  525, 6883,  164712,  6, }, // Asia/Muscat
    {  525, 2446,  164718,  8, }, // Asia/Nicosia
    {  525, 8638,  164726, 16, }, // Asia/Novokuznetsk
    {  525, 4037,  164742, 13, }, // Asia/Novosibirsk
    {  525, 4131,  164755,  7, }, // Asia/Omsk
    {  525, 7108,  164762,  5, }, // Asia/Oral
    {  525, 6924,  164767,  9, }, // Asia/Phnom_Penh
    {  525, 8656,  164776, 10, }, // Asia/Pontianak
    {  525, 4116,  164786, 14, }, // Asia/Pyongyang
    {  525, 5476,  164800,  5, }, // Asia/Qatar
    {  525, 8671,  164805,  8, }, // Asia/Qostanay
    {  525, 4171,  164813, 12, }, // Asia/Qyzylorda
    {  525, 3486,  164825,  6, }, // Asia/Riyadh
    {  525, 4310,  164831,  8, }, // Asia/Sakhalin
    {  525, 7208,  164839, 11, }, // Asia/Samarkand
    {  525, 3116,  164850,  4, }, // Asia/Seoul
    {  525, 1020,  164854,  7, }, // Asia/Shanghai
    {  525, 3137,  164861, 11, }, // Asia/Singapore
    {  525, 4199,  164872, 18, }, // Asia/Srednekolymsk
    {  525, 3100,  164890,  6, }, // Asia/Taipei
    {  525, 4683,  164896,  9, }, // Asia/Tashkent
    {  525, 3817,  164905,  8, }, // Asia/Tbilisi
    {  525, 2638,  164913,  8, }, // Asia/Tehran
    {  525, 1284,  164921,  6, }, // Asia/Thimphu
    {  525, 2687,  164927,  6, }, // Asia/Tokyo
    {  525, 4474,  164933,  8, }, // Asia/Tomsk
    {  525, 1346,  164941, 10, }, // Asia/Ulaanbaatar
    {  525, 1126,  164951,  7, }, // Asia/Urumqi
    {  525, 8685,  164958, 10, }, // Asia/Ust-Nera
    {  525, 6940,  164968, 11, }, // Asia/Vientiane
    {  525, 4612,  164979, 13, }, // Asia/Vladivostok
    {  525, 4730,  164992,  9, }, // Asia/Yakutsk
    {  525, 1202,  165001,  7, }, // Asia/Yangon
    {  525, 3785,  165008, 13, }, // Asia/Yekaterinburg
    {  525, 3649,  165021,  7, }, // Asia/Yerevan
    {  525, 3565,  165028,  5, }, // Atlantic/Azores
    {  525, 5588,  165033,  9, }, // Atlantic/Bermuda
    {  525, 6591,  165042,  5, }, // Atlantic/Canary
    {  525, 3629,  165047, 10, }, // Atlantic/Cape_Verde
    {  525, 1379,  165057,  4, }, // Atlantic/Faroe
    {  525, 8699,  165061,  7, }, // Atlantic/Madeira
    {  525, 2614,  165068, 12, }, // Atlantic/Reykjavik
    {  525, 7235,  165080, 15, }, // Atlantic/South_Georgia
    {  525, 6784,  165095, 10, }, // Atlantic/St_Helena
    {  525, 6607,  165105,  8, }, // Atlantic/Stanley
    {  525, 1654,  165113,  7, }, // Australia/Adelaide
    {  525, 1619,  165120, 11, }, // Australia/Brisbane
    {  525, 1783,  165131, 13, }, // Australia/Broken_Hill
    {  525, 1567,  165144,  6, }, // Australia/Darwin
    {  525, 3539,  165150,  7, }, // Australia/Eucla
    {  525, 1500,  165157,  8, }, // Australia/Hobart
    {  525, 8716,  165165,  9, }, // Australia/Lindeman
    {  525, 1531,  165174, 11, }, // Australia/Lord_Howe
    {  525, 1711,  165185, 10, }, // Australia/Melbourne
    {  525, 1746,  165195,  5, }, // Australia/Perth
    {  525, 1447,  165200,  6, }, // Australia/Sydney
    {  525, 6390,  165206, 12, }, // Europe/Amsterdam
    {  525, 6149,  165218,  8, }, // Europe/Andorra
    {  525, 3522,  165226, 11, }, // Europe/Astrakhan
    {  525, 6577,  165237,  7, }, // Europe/Athens
    {  525, 6285,  165244, 13, }, // Europe/Belgrade
    {  525, 4659,  165257,  7, }, // Europe/Berlin
    {  525, 6437,  165264, 11, }, // Europe/Bratislava
    {  525, 6178,  165275,  9, }, // Europe/Brussels
    {  525, 3830,  165284, 10, }, // Europe/Bucharest
    {  525, 3708,  165294, 10, }, // Europe/Budapest
    {  525, 8751,  165304, 10, }, // Europe/Busingen
    {  525, 2475,  165314,  6, }, // Europe/Chisinau
    {  525, 6238,  165320, 10, }, // Europe/Copenhagen
    {  525, 2260,  165330,  7, }, // Europe/Dublin
    {  525, 6256,  165337, 10, }, // Europe/Gibraltar
    {  525, 8767,  165347,  7, }, // Europe/Guernsey
    {  525, 6561,  165354, 10, }, // Europe/Helsinki
    {  525, 8783,  165364, 14, }, // Europe/Isle_of_Man
    {  525, 1097,  165378, 11, }, // Europe/Istanbul
    {  525, 8802,  165389,  5, }, // Europe/Jersey
    {  525, 3881,  165394, 14, }, // Europe/Kaliningrad
    {  525, 8924,  165408,  6, }, // Europe/Kirov
    {  525, 2419,  165414,  4, }, // Europe/Kyiv
    {  525, 3078,  165418,  7, }, // Europe/Lisbon
    {  525, 6455,  165425, 11, }, // Europe/Ljubljana
    {  525, 2393,  165436,  6, }, // Europe/London
    {  525, 6314,  165442, 12, }, // Europe/Luxembourg
    {  525, 4794,  165454,  8, }, // Europe/Madrid
    {  525, 6346,  165462,  6, }, // Europe/Malta
    {  525, 6531,  165468, 10, }, // Europe/Mariehamn
    {  525, 3595,  165478,  8, }, // Europe/Minsk
    {  525, 6359,  165486,  6, }, // Europe/Monaco
    {  525, 3443,  165492,  7, }, // Europe/Moscow
    {  525, 6407,  165499,  5, }, // Europe/Oslo
    {  525, 4186,  165504,  6, }, // Europe/Paris
    {  525, 6373,  165510, 10, }, // Europe/Podgorica
    {  525, 6224,  165520,  7, }, // Europe/Prague
    {  525, 8816,  165527,  4, }, // Europe/Riga
    {  525, 6273,  165531,  4, }, // Europe/Rome
    {  525, 4233,  165535,  4, }, // Europe/Samara
    {  525, 6419,  165539, 11, }, // Europe/San_Marino
    {  525, 6194,  165550,  7, }, // Europe/Sarajevo
    {  525, 4353,  165557,  6, }, // Europe/Saratov
    {  525, 8828,  165563, 12, }, // Europe/Simferopol
    {  525, 6332,  165575,  8, }, // Europe/Skopje
    {  525, 6548,  165583,  6, }, // Europe/Sofia
    {  525, 6472,  165589, 10, }, // Europe/Stockholm
    {  525, 8846,  165599,  5, }, // Europe/Tallinn
    {  525, 6135,  165604,  6, }, // Europe/Tirane
    {  525, 8861,  165610, 12, }, // Europe/Ulyanovsk
    {  525, 6301,  165622,  5, }, // Europe/Vaduz
    {  525, 6516,  165627,  8, }, // Europe/Vatican
    {  525, 6164,  165635,  5, }, // Europe/Vienna
    {  525, 8878,  165640, 10, }, // Europe/Vilnius
    {  525, 4629,  165650, 13, }, // Europe/Volgograd
    {  525, 3055,  165663,  5, }, // Europe/Warsaw
    {  525, 6210,  165668,  7, }, // Europe/Zagreb
    {  525, 6489,  165675,  6, }, // Europe/Zurich
    {  525, 4980,  165681, 11, }, // Indian/Antananarivo
    {  525, 6910,  165692,  6, }, // Indian/Chagos
    {  525, 5979,  165698, 10, }, // Indian/Christmas
    {  525, 5996,  165708,  6, }, // Indian/Cocos
    {  525, 4931,  165714,  6, }, // Indian/Comoro
    {  525, 6624,  165720, 11, }, // Indian/Kerguelen
    {  525, 7223,  165731,  4, }, // Indian/Mahe
    {  525, 7013,  165735, 10, }, // Indian/Maldives
    {  525, 3971,  165745,  7, }, // Indian/Mauritius
    {  525, 5000,  165752,  5, }, // Indian/Mayotte
    {  525, 7174,  165757,  9, }, // Indian/Reunion
    {  525, 4324,  165766,  5, }, // Pacific/Apia
    {  525,  932,  165771,  8, }, // Pacific/Auckland
    {  525, 3608,  165779,  9, }, // Pacific/Bougainville
    {  525, 2863,  165788,  5, }, // Pacific/Chatham
    {  525, 3022,  165793,  4, }, // Pacific/Chuuk
    {  525, 2201,  165797,  6, }, // Pacific/Easter
    {  525, 7323,  165803,  5, }, // Pacific/Efate
    {  525, 7290,  165808,  7, }, // Pacific/Fakaofo
    {  525, 3804,  165815,  4, }, // Pacific/Fiji
    {  525, 7306,  165819,  7, }, // Pacific/Funafuti
    {  525, 6641,  165826,  7, }, // Pacific/Galapagos
    {  525, 6659,  165833,  9, }, // Pacific/Gambier
    {  525, 3724,  165842, 12, }, // Pacific/Guadalcanal
    {  525, 5935,  165854,  6, }, // Pacific/Guam
    {  525, 2929,  165860,  8, }, // Pacific/Honolulu
    {  525, 2897,  165868,  9, }, // Pacific/Kanton
    {  525, 3900,  165877,  7, }, // Pacific/Kiritimati
    {  525, 6982,  165884,  6, }, // Pacific/Kosrae
    {  525, 2708,  165890, 11, }, // Pacific/Kwajalein
    {  525, 4779,  165901,  5, }, // Pacific/Majuro
    {  525, 3953,  165906, 10, }, // Pacific/Marquesas
    {  525, 8909,  165916,  6, }, // Pacific/Midway
    {  525, 7047,  165922,  5, }, // Pacific/Nauru
    {  525, 7095,  165644,  4, }, // Pacific/Niue
    {  525, 4070,  165927,  6, }, // Pacific/Norfolk
    {  525, 7061,  165933,  6, }, // Pacific/Noumea
    {  525, 2991,  165939,  7, }, // Pacific/Pago_Pago
    {  525, 7118,  165946,  5, }, // Pacific/Palau
    {  525, 7157,  165951, 10, }, // Pacific/Pitcairn
    {  525, 2961,  165961,  8, }, // Pacific/Pohnpei
    {  525, 4709,  165969, 15, }, // Pacific/Port_Moresby
    {  525, 6009,  165984,  9, }, // Pacific/Rarotonga
    {  525, 5948,  165993,  7, }, // Pacific/Saipan
    {  525, 7275,  166000,  5, }, // Pacific/Tahiti
    {  525, 6675,  166005,  5, }, // Pacific/Tarawa
    {  525, 4485,  166010,  9, }, // Pacific/Tongatapu
    {  525, 7355,  166019,  4, }, // Pacific/Wake
    {  525, 7368,  166023,  6, }, // Pacific/Wallis
    {  526,   14,       0,  6, }, // Africa/Asmara Skolt Sami/Latin/Finland
    {  526, 4337,       6,  8, }, // Africa/Sao_Tome
    {  526, 4154,      14,  8, }, // America/Asuncion
    {  526,  237,      22,  8, }, // America/Atikokan
    {  526, 7586,      30, 17, }, // America/Bahia_Banderas
    {  526, 3770,      47,  6, }, // America/Cancun
    {  526, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  526, 5640,      66,  7, }, // America/Curacao
    {  526,  481,      73, 13, }, // America/Indiana/Knox
    {  526, 7924,      86, 16, }, // America/Indiana/Marengo
    {  526, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  526, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  526, 8001,     139, 14, }, // America/Indiana/Vevay
    {  526, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  526, 8049,     171, 16, }, // America/Indiana/Winamac
    {  526, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  526, 5849,     207, 22, }, // America/Lower_Princes
    {  526, 8183,     229,  6, }, // America/Merida
    {  526, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  526, 8292,     255, 20, }, // America/North_Dakota/Center
    {  526, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  526,  393,     298,  4, }, // America/Nuuk
    {  526, 6862,     302, 16, }, // America/Scoresbysund
    {  526, 8471,     318, 14, }, // America/St_Barthelemy
    {  526, 2045,     332, 10, }, // America/St_Johns
    {  526, 5780,     342,  9, }, // America/St_Kitts
    {  526, 5797,     351,  9, }, // America/St_Lucia
    {  526,  855,     360, 10, }, // America/St_Thomas
    {  526, 5830,     370, 11, }, // America/St_Vincent
    {  526, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  526, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  526, 1152,     413,  9, }, // Asia/Kathmandu
    {  526,  992,     422,  7, }, // Asia/Kolkata
    {  526, 1178,     429,  5, }, // Asia/Macau
    {  526, 1202,     442,  6, }, // Asia/Yangon
    {  526, 1379,     448,  5, }, // Atlantic/Faroe
    {  526, 6784,     453, 10, }, // Atlantic/St_Helena
    {  526, 2419,     463,  4, }, // Europe/Kyiv
    {  526, 7174,     467,  7, }, // Indian/Reunion
    {  526, 3022,     474,  5, }, // Pacific/Chuuk
    {  526, 2961,     479,  7, }, // Pacific/Pohnpei
    {  527, 4961,    3793, 11, }, // Africa/Addis_Ababa Slovak/Latin/Slovakia
    {  527, 7383,   56236,  5, }, // Africa/Algiers
    {  527,   14,       0,  6, }, // Africa/Asmara
    {  527, 2242,   56241,  6, }, // Africa/Cairo
    {  527, 4945,   31754,  7, }, // Africa/Djibouti
    {  527, 5068,  166029,  8, }, // Africa/El_Aaiun
    {  527, 4426,   56247,  7, }, // Africa/Khartoum
    {  527, 6832,   18802,  4, }, // Africa/Lome
    {  527, 5015,  166037,  8, }, // Africa/Mogadishu
    {  527, 5173,   42632,  9, }, // Africa/Ndjamena
    {  527, 4337,  166045, 11, }, // Africa/Sao_Tome
    {  527, 2732,   56289,  8, }, // Africa/Tripoli
    {  527,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  527, 4154,      14,  8, }, // America/Asuncion
    {  527,  237,      22,  8, }, // America/Atikokan
    {  527, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  527, 7609,   34376,  5, }, // America/Belem
    {  527, 4263,   18898,  6, }, // America/Bogota
    {  527, 3770,      47,  6, }, // America/Cancun
    {  527, 5398,  166056, 16, }, // America/Cayman
    {  527, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  527, 5324,   31782,  9, }, // America/Costa_Rica
    {  527, 3693,   25007,  6, }, // America/Cuiaba
    {  527, 5640,      66,  7, }, // America/Curacao
    {  527, 5656,   20071,  8, }, // America/Dominica
    {  527, 7832,   25013,  8, }, // America/Eirunepe
    {  527, 5343,  166072,  8, }, // America/El_Salvador
    {  527,  481,      73, 13, }, // America/Indiana/Knox
    {  527, 7924,      86, 16, }, // America/Indiana/Marengo
    {  527, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  527, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  527, 8001,     139, 14, }, // America/Indiana/Vevay
    {  527, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  527, 8049,     171, 16, }, // America/Indiana/Winamac
    {  527, 2665,   31809,  7, }, // America/Jamaica
    {  527, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  527, 5849,     207, 22, }, // America/Lower_Princes
    {  527, 8116,   25021,  6, }, // America/Maceio
    {  527, 5722,    5400,  8, }, // America/Martinique
    {  527, 2779,   25036,  8, }, // America/Mazatlan
    {  527, 8183,     229,  6, }, // America/Merida
    {  527, 2811,   56121,  6, }, // America/Mexico_City
    {  527, 8264,  166080, 22, }, // America/North_Dakota/Beulah
    {  527, 8292,  166102, 22, }, // America/North_Dakota/Center
    {  527, 8320,  166124, 25, }, // America/North_Dakota/New_Salem
    {  527,  393,     298,  4, }, // America/Nuuk
    {  527, 5760,   31888,  9, }, // America/Puerto_Rico
    {  527, 8440,   25137,  8, }, // America/Santarem
    {  527, 1862,   25145,  9, }, // America/Sao_Paulo
    {  527, 6862,     302, 16, }, // America/Scoresbysund
    {  527, 8471,  166149, 16, }, // America/St_Barthelemy
    {  527, 2045,     332, 10, }, // America/St_Johns
    {  527, 5780,     342,  9, }, // America/St_Kitts
    {  527, 5797,  166165, 11, }, // America/St_Lucia
    {  527,  855,  166176,  9, }, // America/St_Thomas
    {  527, 5830,  166185, 11, }, // America/St_Vincent
    {  527, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  527, 7258,  166196,  4, }, // Antarctica/Syowa
    {  527, 3870,   56457,  5, }, // Asia/Amman
    {  527, 5440,    5906,  5, }, // Asia/Aqtau
    {  527, 5270,    5911,  6, }, // Asia/Aqtobe
    {  527,  964,   56462,  8, }, // Asia/Ashgabat
    {  527, 3509,    4063,  6, }, // Asia/Baghdad
    {  527, 5451,   56476,  7, }, // Asia/Bahrain
    {  527, 3988,   56483,  6, }, // Asia/Beirut
    {  527, 3680,   31917,  6, }, // Asia/Bishkek
    {  527, 5906,   31923,  6, }, // Asia/Brunei
    {  527, 4503,   56142,  4, }, // Asia/Chita
    {  527, 5963,   31930,  9, }, // Asia/Choibalsan
    {  527, 4413,    5949,  7, }, // Asia/Colombo
    {  527, 4442,    4102,  6, }, // Asia/Damascus
    {  527, 1060,   56496,  5, }, // Asia/Dhaka
    {  527, 3498,   56501,  5, }, // Asia/Dubai
    {  527, 6092,   31939,  7, }, // Asia/Dushanbe
    {  527, 1226,  166200, 16, }, // Asia/Ho_Chi_Minh
    {  527, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  527, 4673,  166216,  5, }, // Asia/Hovd
    {  527, 1257,   31972,  9, }, // Asia/Jerusalem
    {  527, 3462,   56533,  5, }, // Asia/Kabul
    {  527, 4218,   31981,  8, }, // Asia/Kamchatka
    {  527, 4141,  166221,  6, }, // Asia/Karachi
    {  527, 1152,  166227,  9, }, // Asia/Kathmandu
    {  527, 8624,   56553,  8, }, // Asia/Khandyga
    {  527,  992,   56561,  7, }, // Asia/Kolkata
    {  527, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  527, 5893,   32002,  6, }, // Asia/Kuching
    {  527, 5464,    6058,  6, }, // Asia/Kuwait
    {  527, 1178,     429,  5, }, // Asia/Macau
    {  527, 6883,   25325,  6, }, // Asia/Muscat
    {  527, 2446,  166236,  7, }, // Asia/Nicosia
    {  527, 8638,   61024, 11, }, // Asia/Novokuznetsk
    {  527, 7108,  166243,  6, }, // Asia/Oral
    {  527, 6924,  166249, 10, }, // Asia/Phnom_Penh
    {  527, 4116,   56601, 11, }, // Asia/Pyongyang
    {  527, 5476,    4233,  5, }, // Asia/Qatar
    {  527, 8671,    6103,  8, }, // Asia/Qostanay
    {  527, 4171,   25347,  9, }, // Asia/Qyzylorda
    {  527, 3486,   56612,  5, }, // Asia/Riyadh
    {  527, 4310,   56617,  8, }, // Asia/Sakhalin
    {  527, 3116,   56625,  4, }, // Asia/Seoul
    {  527, 1020,   56629,  7, }, // Asia/Shanghai
    {  527, 3137,   19291,  8, }, // Asia/Singapore
    {  527, 3100,   56649,  9, }, // Asia/Taipei
    {  527, 4683,   32060,  7, }, // Asia/Tashkent
    {  527, 2638,   56658,  7, }, // Asia/Tehran
    {  527, 2687,    4257,  5, }, // Asia/Tokyo
    {  527, 1346,  166259,  9, }, // Asia/Ulaanbaatar
    {  527, 1126,   32067,  6, }, // Asia/Urumqi
    {  527, 8685,  166268,  8, }, // Asia/Ust-Nera
    {  527, 6940,   25389,  8, }, // Asia/Vientiane
    {  527, 4730,    6206,  7, }, // Asia/Yakutsk
    {  527, 1202,   19328,  6, }, // Asia/Yangon
    {  527, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  527, 3649,    6231,  7, }, // Asia/Yerevan
    {  527, 3565,  150152,  5, }, // Atlantic/Azores
    {  527, 5588,   56708,  7, }, // Atlantic/Bermuda
    {  527, 6591,  166276, 16, }, // Atlantic/Canary
    {  527, 3629,   56731,  8, }, // Atlantic/Cape_Verde
    {  527, 1379,   56739, 15, }, // Atlantic/Faroe
    {  527, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  527, 7235,  166292, 13, }, // Atlantic/South_Georgia
    {  527, 6784,  166305, 12, }, // Atlantic/St_Helena
    {  527, 3522,  166317,  9, }, // Europe/Astrakhan
    {  527, 6577,  166326,  5, }, // Europe/Athens
    {  527, 6285,  166331,  8, }, // Europe/Belgrade
    {  527, 4659,   19450,  6, }, // Europe/Berlin
    {  527, 6178,   25472,  6, }, // Europe/Brussels
    {  527, 3830,   56802,  8, }, // Europe/Bucharest
    {  527, 3708,   56810,  8, }, // Europe/Budapest
    {  527, 8751,   25487,  8, }, // Europe/Busingen
    {  527, 2475,  166339,  7, }, // Europe/Chisinau
    {  527, 6238,   56825,  5, }, // Europe/Copenhagen
    {  527, 6256,   90503,  9, }, // Europe/Gibraltar
    {  527, 8783,   56838, 10, }, // Europe/Isle_of_Man
    {  527, 2419,   56848,  5, }, // Europe/Kyiv
    {  527, 3078,   32199,  7, }, // Europe/Lisbon
    {  527, 6455,  166346,  7, }, // Europe/Ljubljana
    {  527, 2393,   56859,  6, }, // Europe/London
    {  527, 6314,  166353, 11, }, // Europe/Luxembourg
    {  527, 6359,    6525,  6, }, // Europe/Monaco
    {  527, 3443,   21290,  6, }, // Europe/Moscow
    {  527, 4186,  166364,  5, }, // Europe/Paris
    {  527, 6224,   34765,  5, }, // Europe/Prague
    {  527, 6273,  166369,  3, }, // Europe/Rome
    {  527, 6419,  138641, 10, }, // Europe/San_Marino
    {  527, 6472,   32238,  8, }, // Europe/Stockholm
    {  527, 6135,   19608,  6, }, // Europe/Tirane
    {  527, 8861,  166372,  9, }, // Europe/Ulyanovsk
    {  527, 6516,   56887,  7, }, // Europe/Vatican
    {  527, 6164,  166381,  6, }, // Europe/Vienna
    {  527, 3055,   32249,  7, }, // Europe/Warsaw
    {  527, 6210,  166387,  6, }, // Europe/Zagreb
    {  527, 6489,    4478,  6, }, // Europe/Zurich
    {  527, 5979,  166393, 15, }, // Indian/Christmas
    {  527, 5996,   56925, 16, }, // Indian/Cocos
    {  527, 4931,   56941,  6, }, // Indian/Comoro
    {  527, 6624,  166408, 10, }, // Indian/Kerguelen
    {  527, 7223,   25565,  4, }, // Indian/Mahe
    {  527, 7013,  166418,  7, }, // Indian/Maldives
    {  527, 3971,  166425,  9, }, // Indian/Mauritius
    {  527, 7174,     467,  7, }, // Indian/Reunion
    {  527, 3022,     474,  5, }, // Pacific/Chuuk
    {  527, 2201,  166434, 17, }, // Pacific/Easter
    {  527, 3804,   32307,  5, }, // Pacific/Fiji
    {  527, 6641,   57036,  8, }, // Pacific/Galapagos
    {  527, 2929,    4512,  8, }, // Pacific/Honolulu
    {  527, 2897,    4520,  9, }, // Pacific/Kanton
    {  527, 3953,   57062,  7, }, // Pacific/Marquesas
    {  527, 7061,    4529,  6, }, // Pacific/Noumea
    {  527, 2961,     479,  7, }, // Pacific/Pohnpei
    {  528, 6751,   56229,  7, }, // Africa/Abidjan Slovenian/Latin/Slovenia
    {  528, 6723,    4550,  4, }, // Africa/Accra
    {  528, 4961,   34301, 10, }, // Africa/Addis_Ababa
    {  528, 7383,   56099,  5, }, // Africa/Algiers
    {  528,   14,       0,  6, }, // Africa/Asmara
    {  528, 2242,   24913,  5, }, // Africa/Cairo
    {  528, 4945,   31754,  7, }, // Africa/Djibouti
    {  528, 4426,    4693,  6, }, // Africa/Khartoum
    {  528, 5208,  117752,  7, }, // Africa/Kinshasa
    {  528, 4841,  117774,  9, }, // Africa/Lubumbashi
    {  528, 5015,   56254,  8, }, // Africa/Mogadishu
    {  528, 4337,       6,  8, }, // Africa/Sao_Tome
    {  528, 5524,   31768,  7, }, // America/Anguilla
    {  528, 4154,      14,  8, }, // America/Asuncion
    {  528,  237,      22,  8, }, // America/Atikokan
    {  528, 7586,   63067, 17, }, // America/Bahia_Banderas
    {  528, 3770,    3861,  6, }, // America/Cancun
    {  528, 5398,    4978,  6, }, // America/Cayman
    {  528, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  528, 5324,   31782,  9, }, // America/Costa_Rica
    {  528, 5640,      66,  7, }, // America/Curacao
    {  528, 5656,   20071,  8, }, // America/Dominica
    {  528, 5343,    5090,  8, }, // America/El_Salvador
    {  528, 3662,   31800,  9, }, // America/Guatemala
    {  528, 6895,   56104,  7, }, // America/Guyana
    {  528,  481,      73, 13, }, // America/Indiana/Knox
    {  528, 7924,      86, 16, }, // America/Indiana/Marengo
    {  528, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  528, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  528, 8001,     139, 14, }, // America/Indiana/Vevay
    {  528, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  528, 8049,     171, 16, }, // America/Indiana/Winamac
    {  528, 2665,   31809,  7, }, // America/Jamaica
    {  528, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  528, 5849,     207, 22, }, // America/Lower_Princes
    {  528, 5722,    5400,  8, }, // America/Martinique
    {  528, 8183,    3899,  6, }, // America/Merida
    {  528, 2811,  154536, 16, }, // America/Mexico_City
    {  528, 8264,  166451, 22, }, // America/North_Dakota/Beulah
    {  528, 8292,  166473, 22, }, // America/North_Dakota/Center
    {  528, 8320,  166495, 25, }, // America/North_Dakota/New_Salem
    {  528,  393,     298,  4, }, // America/Nuuk
    {  528, 5760,   31888,  9, }, // America/Puerto_Rico
    {  528, 6862,     302, 16, }, // America/Scoresbysund
    {  528, 8471,  159320, 14, }, // America/St_Barthelemy
    {  528, 2045,     332, 10, }, // America/St_Johns
    {  528, 5780,  166520, 11, }, // America/St_Kitts
    {  528, 5797,  158708, 10, }, // America/St_Lucia
    {  528,  855,  166531,  9, }, // America/St_Thomas
    {  528, 5830,  166540, 13, }, // America/St_Vincent
    {  528, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  528, 5297,    5890,  6, }, // Asia/Almaty
    {  528, 3870,    5896,  4, }, // Asia/Amman
    {  528, 5428,    5900,  6, }, // Asia/Anadyr
    {  528, 5440,    5906,  5, }, // Asia/Aqtau
    {  528, 5270,    5911,  6, }, // Asia/Aqtobe
    {  528,  964,   31904,  7, }, // Asia/Ashgabat
    {  528, 3509,    4063,  6, }, // Asia/Baghdad
    {  528, 5451,   56476,  7, }, // Asia/Bahrain
    {  528, 3988,    5930,  6, }, // Asia/Beirut
    {  528, 3680,   31917,  6, }, // Asia/Bishkek
    {  528, 5906,   31923,  6, }, // Asia/Brunei
    {  528, 4503,   56142,  4, }, // Asia/Chita
    {  528, 5963,   31930,  9, }, // Asia/Choibalsan
    {  528, 4413,    5949,  7, }, // Asia/Colombo
    {  528, 4442,    4102,  6, }, // Asia/Damascus
    {  528, 1060,    5956,  4, }, // Asia/Dhaka
    {  528, 3498,   56501,  5, }, // Asia/Dubai
    {  528, 6092,   31939,  7, }, // Asia/Dushanbe
    {  528, 1226,  166553,  8, }, // Asia/Ho_Chi_Minh
    {  528, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  528, 6969,   31955,  8, }, // Asia/Jakarta
    {  528, 1257,   31972,  9, }, // Asia/Jerusalem
    {  528, 4218,   31981,  8, }, // Asia/Kamchatka
    {  528, 4141,   31989,  6, }, // Asia/Karachi
    {  528, 1152,    6018,  8, }, // Asia/Kathmandu
    {  528, 8624,   31995,  7, }, // Asia/Khandyga
    {  528,  992,   25307,  7, }, // Asia/Kolkata
    {  528, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  528, 5464,    6058,  6, }, // Asia/Kuwait
    {  528, 1178,     429,  5, }, // Asia/Macau
    {  528, 1316,    6069,  7, }, // Asia/Makassar
    {  528, 6883,  166561,  6, }, // Asia/Muscat
    {  528, 2446,   32013,  8, }, // Asia/Nicosia
    {  528, 7108,   56586,  6, }, // Asia/Oral
    {  528, 4116,   32033,  9, }, // Asia/Pyongyang
    {  528, 5476,    4233,  5, }, // Asia/Qatar
    {  528, 8671,    6103,  8, }, // Asia/Qostanay
    {  528, 4171,  166567,  8, }, // Asia/Qyzylorda
    {  528, 3486,   32042,  5, }, // Asia/Riyadh
    {  528, 4310,   32047,  7, }, // Asia/Sakhalin
    {  528, 3116,    6131,  4, }, // Asia/Seoul
    {  528, 1020,   56629,  7, }, // Asia/Shanghai
    {  528, 3137,   19291,  8, }, // Asia/Singapore
    {  528, 4199,    6150, 13, }, // Asia/Srednekolymsk
    {  528, 3100,    6163,  6, }, // Asia/Taipei
    {  528, 4683,   32060,  7, }, // Asia/Tashkent
    {  528, 2638,    6175,  7, }, // Asia/Tehran
    {  528, 1284,   90291,  5, }, // Asia/Thimphu
    {  528, 2687,    4257,  5, }, // Asia/Tokyo
    {  528, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  528, 1126,   32067,  6, }, // Asia/Urumqi
    {  528, 4730,    6206,  7, }, // Asia/Yakutsk
    {  528, 1202,    6213,  6, }, // Asia/Yangon
    {  528, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  528, 3649,   25397,  6, }, // Asia/Yerevan
    {  528, 3565,   32083,  5, }, // Atlantic/Azores
    {  528, 5588,  166575,  7, }, // Atlantic/Bermuda
    {  528, 6591,  166582, 14, }, // Atlantic/Canary
    {  528, 3629,  166596, 17, }, // Atlantic/Cape_Verde
    {  528, 1379,  166613, 12, }, // Atlantic/Faroe
    {  528, 7235,  166625, 13, }, // Atlantic/South_Georgia
    {  528, 6784,  166638, 10, }, // Atlantic/St_Helena
    {  528, 6149,   32109,  6, }, // Europe/Andorra
    {  528, 3522,   32115,  8, }, // Europe/Astrakhan
    {  528, 6577,   19437,  5, }, // Europe/Athens
    {  528, 6285,    6398,  7, }, // Europe/Belgrade
    {  528, 6178,  166648,  7, }, // Europe/Brussels
    {  528, 3830,  122803,  9, }, // Europe/Bucharest
    {  528, 3708,   32142, 10, }, // Europe/Budapest
    {  528, 2475,   32152,  8, }, // Europe/Chisinau
    {  528, 6238,  166655,  9, }, // Europe/Copenhagen
    {  528, 8783,   56203,  8, }, // Europe/Isle_of_Man
    {  528, 2419,   32194,  5, }, // Europe/Kyiv
    {  528, 3078,  150331,  7, }, // Europe/Lisbon
    {  528, 6314,    6510, 10, }, // Europe/Luxembourg
    {  528, 6359,    6525,  6, }, // Europe/Monaco
    {  528, 3443,   21290,  6, }, // Europe/Moscow
    {  528, 4186,   32206,  5, }, // Europe/Paris
    {  528, 6224,   19576,  5, }, // Europe/Prague
    {  528, 6273,   32211,  3, }, // Europe/Rome
    {  528, 6548,   32232,  6, }, // Europe/Sofia
    {  528, 8846,    6594,  5, }, // Europe/Tallinn
    {  528, 6135,   19608,  6, }, // Europe/Tirane
    {  528, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  528, 6516,    6614,  7, }, // Europe/Vatican
    {  528, 6164,  166664,  5, }, // Europe/Vienna
    {  528, 8878,   62886,  5, }, // Europe/Vilnius
    {  528, 3055,   32249,  7, }, // Europe/Warsaw
    {  528, 6489,    4478,  6, }, // Europe/Zurich
    {  528, 5979,  166669, 12, }, // Indian/Christmas
    {  528, 5996,  166681, 14, }, // Indian/Cocos
    {  528, 4931,   62920,  6, }, // Indian/Comoro
    {  528, 7013,   32290,  7, }, // Indian/Maldives
    {  528, 7174,     467,  7, }, // Indian/Reunion
    {  528, 3022,     474,  5, }, // Pacific/Chuuk
    {  528, 2201,  166695, 16, }, // Pacific/Easter
    {  528, 3804,   32307,  5, }, // Pacific/Fiji
    {  528, 2929,    4512,  8, }, // Pacific/Honolulu
    {  528, 2897,    4520,  9, }, // Pacific/Kanton
    {  528, 2961,     479,  7, }, // Pacific/Pohnpei
    {  529,   14,       0,  6, }, // Africa/Asmara Soga/Latin/Uganda
    {  529, 4337,       6,  8, }, // Africa/Sao_Tome
    {  529, 4154,      14,  8, }, // America/Asuncion
    {  529,  237,      22,  8, }, // America/Atikokan
    {  529, 7586,      30, 17, }, // America/Bahia_Banderas
    {  529, 3770,      47,  6, }, // America/Cancun
    {  529, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  529, 5640,      66,  7, }, // America/Curacao
    {  529,  481,      73, 13, }, // America/Indiana/Knox
    {  529, 7924,      86, 16, }, // America/Indiana/Marengo
    {  529, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  529, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  529, 8001,     139, 14, }, // America/Indiana/Vevay
    {  529, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  529, 8049,     171, 16, }, // America/Indiana/Winamac
    {  529, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  529, 5849,     207, 22, }, // America/Lower_Princes
    {  529, 8183,     229,  6, }, // America/Merida
    {  529, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  529, 8292,     255, 20, }, // America/North_Dakota/Center
    {  529, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  529,  393,     298,  4, }, // America/Nuuk
    {  529, 6862,     302, 16, }, // America/Scoresbysund
    {  529, 8471,     318, 14, }, // America/St_Barthelemy
    {  529, 2045,     332, 10, }, // America/St_Johns
    {  529, 5780,     342,  9, }, // America/St_Kitts
    {  529, 5797,     351,  9, }, // America/St_Lucia
    {  529,  855,     360, 10, }, // America/St_Thomas
    {  529, 5830,     370, 11, }, // America/St_Vincent
    {  529, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  529, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  529, 1152,     413,  9, }, // Asia/Kathmandu
    {  529,  992,     422,  7, }, // Asia/Kolkata
    {  529, 1178,     429,  5, }, // Asia/Macau
    {  529, 1202,     442,  6, }, // Asia/Yangon
    {  529, 1379,     448,  5, }, // Atlantic/Faroe
    {  529, 6784,     453, 10, }, // Atlantic/St_Helena
    {  529, 2419,     463,  4, }, // Europe/Kyiv
    {  529, 7174,     467,  7, }, // Indian/Reunion
    {  529, 3022,     474,  5, }, // Pacific/Chuuk
    {  529, 2961,     479,  7, }, // Pacific/Pohnpei
    {  530, 6751,  166711,  8, }, // Africa/Abidjan Somali/Latin/Somalia
    {  530, 6723,    4550,  4, }, // Africa/Accra
    {  530, 7383,  166719,  8, }, // Africa/Algiers
    {  530,   14,       0,  6, }, // Africa/Asmara
    {  530,   44,  166727,  7, }, // Africa/Bamako
    {  530, 5159,  166734,  5, }, // Africa/Bangui
    {  530, 7398,  166739,  6, }, // Africa/Bissau
    {  530, 4859,  166745,  9, }, // Africa/Blantyre
    {  530, 5189,  166754,  9, }, // Africa/Brazzaville
    {  530, 2242,  166763,  7, }, // Africa/Cairo
    {  530, 4019,  166770, 12, }, // Africa/Casablanca
    {  530, 7412,   19754,  5, }, // Africa/Ceuta
    {  530, 6736,  166782,  7, }, // Africa/Conakry
    {  530, 5032,  166789, 10, }, // Africa/Dar_es_Salaam
    {  530, 4945,  166799,  7, }, // Africa/Djibouti
    {  530, 5145,  166806,  7, }, // Africa/Douala
    {  530, 5068,  166813,  9, }, // Africa/El_Aaiun
    {  530, 6816,  166822,  9, }, // Africa/Freetown
    {  530, 4808,  166831,  7, }, // Africa/Gaborone
    {  530, 4917,  166838,  7, }, // Africa/Harare
    {  530, 4381,  166845, 10, }, // Africa/Johannesburg
    {  530, 5053,  166855,  8, }, // Africa/Kampala
    {  530, 4426,  166863,  8, }, // Africa/Khartoum
    {  530, 4646,  166871,  7, }, // Africa/Lagos
    {  530, 5238,  166878,  8, }, // Africa/Libreville
    {  530, 6832,  166886,  4, }, // Africa/Lome
    {  530, 5113,  166890,  8, }, // Africa/Luanda
    {  530, 4841,  166898, 11, }, // Africa/Lubumbashi
    {  530, 4903,  166909,  7, }, // Africa/Lusaka
    {  530, 4889,  166916,  7, }, // Africa/Maputo
    {  530, 5099,  166923,  7, }, // Africa/Maseru
    {  530, 5084,  166930,  9, }, // Africa/Mbabane
    {  530, 5015,   24944,  8, }, // Africa/Mogadishu
    {  530, 6997,  166939,  9, }, // Africa/Monrovia
    {  530, 3755,  166948,  8, }, // Africa/Nairobi
    {  530, 5173,  166956,  8, }, // Africa/Ndjamena
    {  530, 5256,  158017,  6, }, // Africa/Niamey
    {  530, 6766,  166964,  7, }, // Africa/Nouakchott
    {  530, 6690,   97031,  8, }, // Africa/Ouagadougou
    {  530, 5127,  166971, 12, }, // Africa/Porto-Novo
    {  530, 4337,  166983,  8, }, // Africa/Sao_Tome
    {  530, 2732,  166991,  9, }, // Africa/Tripoli
    {  530, 6503,  167000,  6, }, // Africa/Tunis
    {  530, 4054,  167006,  8, }, // Africa/Windhoek
    {  530, 3183,  167014,  8, }, // America/Anchorage
    {  530, 5524,  167022,  8, }, // America/Anguilla
    {  530, 5541,  167030,  8, }, // America/Antigua
    {  530, 4456,  167038, 10, }, // America/Araguaina
    {  530,  166,  167048, 13, }, // America/Argentina/Buenos_Aires
    {  530,   91,  167061, 10, }, // America/Argentina/Catamarca
    {  530,  270,  167071,  8, }, // America/Argentina/Cordoba
    {  530, 7425,  167079,  9, }, // America/Argentina/La_Rioja
    {  530,  565,  167088,  9, }, // America/Argentina/Mendoza
    {  530, 7452,  167097, 12, }, // America/Argentina/Rio_Gallegos
    {  530, 7507,  167109, 10, }, // America/Argentina/San_Juan
    {  530, 5497,  167119,  9, }, // America/Argentina/San_Luis
    {  530, 7534,  167128,  9, }, // America/Argentina/Tucuman
    {  530, 7560,  167137,  7, }, // America/Argentina/Ushuaia
    {  530, 4154,  167144,  9, }, // America/Asuncion
    {  530,  237,  167153,  9, }, // America/Atikokan
    {  530, 3581,  167162,  7, }, // America/Bahia
    {  530, 7586,  167169, 15, }, // America/Bahia_Banderas
    {  530, 5309,  167184,  7, }, // America/Belize
    {  530, 7623,  167191, 13, }, // America/Blanc-Sablon
    {  530, 7644,  167204,  9, }, // America/Boa_Vista
    {  530, 7662,   19969,  5, }, // America/Boise
    {  530, 7676,  167213, 14, }, // America/Cambridge_Bay
    {  530, 7698,  167227, 14, }, // America/Campo_Grande
    {  530, 3770,  167241,  8, }, // America/Cancun
    {  530, 4596,  167249,  8, }, // America/Caracas
    {  530, 4247,  167257,  6, }, // America/Cayenne
    {  530, 5398,  167263,  7, }, // America/Cayman
    {  530, 3251,  167270,  7, }, // America/Chicago
    {  530, 7719,  167277,  8, }, // America/Chihuahua
    {  530, 7737,  167285, 16, }, // America/Ciudad_Juarez
    {  530, 5324,  167301, 11, }, // America/Costa_Rica
    {  530, 7759,  167312,  9, }, // America/Creston
    {  530, 3693,  167321,  7, }, // America/Cuiaba
    {  530, 5640,  167328,  8, }, // America/Curacao
    {  530, 7775,  167336, 13, }, // America/Danmarkshavn
    {  530, 7796,  167349,  6, }, // America/Dawson
    {  530, 7811,  167355, 13, }, // America/Dawson_Creek
    {  530,  805,  167368,  6, }, // America/Denver
    {  530, 3351,  167374,  8, }, // America/Detroit
    {  530, 5656,  167382,  9, }, // America/Dominica
    {  530, 7832,  167391,  8, }, // America/Eirunepe
    {  530, 5343,  167399, 12, }, // America/El_Salvador
    {  530, 7849,  167411, 11, }, // America/Fort_Nelson
    {  530, 7869,  167422,  9, }, // America/Fortaleza
    {  530, 7887,  167431, 11, }, // America/Glace_Bay
    {  530, 6844,  167442,  9, }, // America/Goose_Bay
    {  530, 4514,  167451, 11, }, // America/Grand_Turk
    {  530, 5687,  167462,  9, }, // America/Grenada
    {  530, 5703,  167471, 10, }, // America/Guadeloupe
    {  530, 3662,  167481, 11, }, // America/Guatemala
    {  530, 4743,  167492, 11, }, // America/Guayaquil
    {  530, 6895,  167503,  7, }, // America/Guyana
    {  530, 1923,  167510,  9, }, // America/Halifax
    {  530, 2221,  167519,  7, }, // America/Havana
    {  530, 7905,  167526,  9, }, // America/Hermosillo
    {  530,  348,  167535, 14, }, // America/Indiana/Indianapolis
    {  530,  481,  167549, 16, }, // America/Indiana/Knox
    {  530, 7924,  167565, 19, }, // America/Indiana/Marengo
    {  530, 7948,  167584, 20, }, // America/Indiana/Petersburg
    {  530, 7975,  167604, 19, }, // America/Indiana/Tell_City
    {  530, 8001,  167623, 18, }, // America/Indiana/Vevay
    {  530, 8023,  167641, 20, }, // America/Indiana/Vincennes
    {  530, 8049,  167661, 19, }, // America/Indiana/Winamac
    {  530, 8073,  167680,  7, }, // America/Inuvik
    {  530,  660,  167687,  8, }, // America/Iqaluit
    {  530, 2665,   97349,  7, }, // America/Jamaica
    {  530, 5282,  167695,  7, }, // America/Juneau
    {  530,  521,  167702,  8, }, // America/Kentucky/Louisville
    {  530, 8088,  167710, 19, }, // America/Kentucky/Monticello
    {  530, 5621,  167729, 11, }, // America/Kralendijk
    {  530, 4278,  167740,  8, }, // America/La_Paz
    {  530, 7132,  167748,  5, }, // America/Lima
    {  530, 3390,  167753, 12, }, // America/Los_Angeles
    {  530, 5849,  167765, 22, }, // America/Lower_Princes
    {  530, 8116,  167787,  8, }, // America/Maceio
    {  530, 8131,  167795,  9, }, // America/Managua
    {  530, 1892,  167804,  8, }, // America/Manaus
    {  530, 5814,  167812,  8, }, // America/Marigot
    {  530, 5722,  167820, 14, }, // America/Martinique
    {  530, 2779,  167834,  9, }, // America/Mazatlan
    {  530, 8165,  167843, 10, }, // America/Menominee
    {  530, 8183,  167853,  7, }, // America/Merida
    {  530, 8198,  167860, 11, }, // America/Metlakatla
    {  530, 2811,  158493, 12, }, // America/Mexico_City
    {  530, 4293,  167871,  9, }, // America/Miquelon
    {  530, 8217,  167880,  9, }, // America/Moncton
    {  530, 8233,  167889, 10, }, // America/Monterrey
    {  530, 4000,  167899, 13, }, // America/Montevideo
    {  530, 5741,  167912, 11, }, // America/Montserrat
    {  530, 5383,  167923,  6, }, // America/Nassau
    {  530, 3294,  167929, 10, }, // America/New_York
    {  530, 8251,  137236,  4, }, // America/Nome
    {  530, 8264,  167939, 27, }, // America/North_Dakota/Beulah
    {  530, 8292,  167966, 28, }, // America/North_Dakota/Center
    {  530, 8320,  167994, 31, }, // America/North_Dakota/New_Salem
    {  530,  393,     298,  4, }, // America/Nuuk
    {  530, 8351,  168025,  8, }, // America/Ojinaga
    {  530, 5413,  168033,  7, }, // America/Panama
    {  530, 3224,  168040,  8, }, // America/Phoenix
    {  530, 3847,  168048, 16, }, // America/Port-au-Prince
    {  530, 5871,  168064, 15, }, // America/Port_of_Spain
    {  530, 5760,  168079, 12, }, // America/Puerto_Rico
    {  530, 3932,  168091, 13, }, // America/Punta_Arenas
    {  530, 8387,  168104, 14, }, // America/Rankin_Inlet
    {  530, 8408,  168118,  6, }, // America/Recife
    {  530, 1979,  168124,  7, }, // America/Regina
    {  530, 8423,  168131,  8, }, // America/Resolute
    {  530,  695,  168139, 13, }, // America/Rio_Branco
    {  530, 8440,  168152,  9, }, // America/Santarem
    {  530, 2165,  168161, 10, }, // America/Santiago
    {  530, 6044,  168171, 14, }, // America/Santo_Domingo
    {  530, 1862,  168185, 10, }, // America/Sao_Paulo
    {  530, 6862,  168195, 13, }, // America/Scoresbysund
    {  530, 8457,  168208,  6, }, // America/Sitka
    {  530, 8471,  168214, 14, }, // America/St_Barthelemy
    {  530, 2045,  168228,  8, }, // America/St_Johns
    {  530, 5780,  168236,  9, }, // America/St_Kitts
    {  530, 5797,  168245, 10, }, // America/St_Lucia
    {  530,  855,  168255, 10, }, // America/St_Thomas
    {  530, 5830,  168265, 11, }, // America/St_Vincent
    {  530, 8493,  168276, 13, }, // America/Swift_Current
    {  530, 5363,  168289, 11, }, // America/Tegucigalpa
    {  530, 5673,  168300,  4, }, // America/Thule
    {  530,  313,  168304,  9, }, // America/Tijuana
    {  530, 5605,  168313,  8, }, // America/Tortola
    {  530, 2077,  168321,  9, }, // America/Vancouver
    {  530, 2128,  168330,  9, }, // America/Whitehorse
    {  530,  734,  168339,  7, }, // America/Winnipeg
    {  530, 8515,  168346,  8, }, // America/Yakutat
    {  530, 5918,  168354,  6, }, // Antarctica/Casey
    {  530, 6027,  168360,  5, }, // Antarctica/Davis
    {  530, 6066,  168365, 15, }, // Antarctica/DumontDUrville
    {  530, 8531,  168380, 10, }, // Antarctica/Macquarie
    {  530, 7076,   72447,  8, }, // Antarctica/McMurdo
    {  530, 8552,  168390,  6, }, // Antarctica/Palmer
    {  530, 7189,  118999,  6, }, // Antarctica/Rothera
    {  530, 7258,  168396,  6, }, // Antarctica/Syowa
    {  530, 8570,  168402,  6, }, // Antarctica/Troll
    {  530, 7337,  168408,  6, }, // Antarctica/Vostok
    {  530, 1413,  168414, 11, }, // Arctic/Longyearbyen
    {  530, 5487,  168425,  5, }, // Asia/Aden
    {  530, 5297,    5890,  6, }, // Asia/Almaty
    {  530, 3870,  168430,  6, }, // Asia/Amman
    {  530, 5428,  168436,  7, }, // Asia/Anadyr
    {  530, 5440,  168443,  5, }, // Asia/Aqtau
    {  530, 8587,  168448,  7, }, // Asia/Atyrau
    {  530, 3509,  168455,  7, }, // Asia/Baghdad
    {  530, 5451,  168462,  7, }, // Asia/Bahrain
    {  530, 4368,  168469,  8, }, // Asia/Bangkok
    {  530, 3473,  168477,  9, }, // Asia/Barnaul
    {  530, 3988,  168486,  7, }, // Asia/Beirut
    {  530, 5906,  168493,  7, }, // Asia/Brunei
    {  530, 4503,  168500,  5, }, // Asia/Chita
    {  530, 5963,  168505,  9, }, // Asia/Choibalsan
    {  530, 4413,    5949,  7, }, // Asia/Colombo
    {  530, 4442,  168514,  8, }, // Asia/Damascus
    {  530, 1060,  168522,  6, }, // Asia/Dhaka
    {  530, 3498,   20800,  5, }, // Asia/Dubai
    {  530, 8614,  168528,  4, }, // Asia/Gaza
    {  530, 1226,  168532, 15, }, // Asia/Ho_Chi_Minh
    {  530, 2576,  168547, 11, }, // Asia/Hong_Kong
    {  530, 4673,  168558,  5, }, // Asia/Hovd
    {  530, 4086,  168563,  8, }, // Asia/Irkutsk
    {  530, 6969,  168571,  8, }, // Asia/Jakarta
    {  530, 6955,  168579,  8, }, // Asia/Jayapura
    {  530, 1257,  168587, 11, }, // Asia/Jerusalem
    {  530, 3462,  168598,  7, }, // Asia/Kabul
    {  530, 4218,  168605,  8, }, // Asia/Kamchatka
    {  530, 4141,  168613,  7, }, // Asia/Karachi
    {  530, 1152,    6018,  8, }, // Asia/Kathmandu
    {  530, 8624,  168620,  9, }, // Asia/Khandyga
    {  530,  992,  168629,  8, }, // Asia/Kolkata
    {  530, 4099,  168637, 13, }, // Asia/Krasnoyarsk
    {  530, 4761,  168650, 13, }, // Asia/Kuala_Lumpur
    {  530, 5893,  168663,  6, }, // Asia/Kuching
    {  530, 5464,  113261,  6, }, // Asia/Kuwait
    {  530, 1178,  168669,  5, }, // Asia/Macau
    {  530, 3919,  168674,  7, }, // Asia/Magadan
    {  530, 1316,    6069,  7, }, // Asia/Makassar
    {  530, 7145,  168681,  7, }, // Asia/Manila
    {  530, 6883,    4227,  6, }, // Asia/Muscat
    {  530, 2446,   20920,  8, }, // Asia/Nicosia
    {  530, 8638,  168688, 13, }, // Asia/Novokuznetsk
    {  530, 4037,  168701, 12, }, // Asia/Novosibirsk
    {  530, 6924,  168713,  9, }, // Asia/Phnom_Penh
    {  530, 8656,  168722, 10, }, // Asia/Pontianak
    {  530, 4116,  168732, 10, }, // Asia/Pyongyang
    {  530, 5476,  168742,  6, }, // Asia/Qatar
    {  530, 8671,     434,  8, }, // Asia/Qostanay
    {  530, 4171,  168748,  9, }, // Asia/Qyzylorda
    {  530, 3486,  168757,  6, }, // Asia/Riyadh
    {  530, 7208,  168763,  9, }, // Asia/Samarkand
    {  530, 3116,   56625,  4, }, // Asia/Seoul
    {  530, 1020,  168772,  9, }, // Asia/Shanghai
    {  530, 3137,  168781,  9, }, // Asia/Singapore
    {  530, 4199,  168790, 16, }, // Asia/Srednekolymsk
    {  530, 3100,  168806,  6, }, // Asia/Taipei
    {  530, 4683,  168812,  9, }, // Asia/Tashkent
    {  530, 3817,  168821,  8, }, // Asia/Tbilisi
    {  530, 2638,  168829,  7, }, // Asia/Tehran
    {  530, 1284,  140573,  5, }, // Asia/Thimphu
    {  530, 6940,  168836, 12, }, // Asia/Vientiane
    {  530, 4612,  168848, 12, }, // Asia/Vladivostok
    {  530, 4730,  168346,  5, }, // Asia/Yakutsk
    {  530, 1202,     442,  6, }, // Asia/Yangon
    {  530, 3785,  168860, 13, }, // Asia/Yekaterinburg
    {  530, 3649,  168873,  7, }, // Asia/Yerevan
    {  530, 3565,  168880,  6, }, // Atlantic/Azores
    {  530, 5588,  168886,  8, }, // Atlantic/Bermuda
    {  530, 6591,    4299,  6, }, // Atlantic/Canary
    {  530, 3629,  168894, 11, }, // Atlantic/Cape_Verde
    {  530, 1379,  168905,  5, }, // Atlantic/Faroe
    {  530, 8699,   97688,  6, }, // Atlantic/Madeira
    {  530, 2614,  168910,  9, }, // Atlantic/Reykjavik
    {  530, 7235,  168919, 13, }, // Atlantic/South_Georgia
    {  530, 6784,     453, 10, }, // Atlantic/St_Helena
    {  530, 6607,  168932,  9, }, // Atlantic/Stanley
    {  530, 1654,  168941,  8, }, // Australia/Adelaide
    {  530, 1619,  168949,  8, }, // Australia/Brisbane
    {  530, 1783,  168957, 11, }, // Australia/Broken_Hill
    {  530, 3539,   21132,  5, }, // Australia/Eucla
    {  530, 1500,  168968,  7, }, // Australia/Hobart
    {  530, 1531,  168975,  7, }, // Australia/Lord_Howe
    {  530, 1711,  168982,  7, }, // Australia/Melbourne
    {  530, 1746,  168989,  4, }, // Australia/Perth
    {  530, 1447,   21149,  6, }, // Australia/Sydney
    {  530, 6390,  168993, 10, }, // Europe/Amsterdam
    {  530, 6149,  169003,  7, }, // Europe/Andorra
    {  530, 3522,  169010, 11, }, // Europe/Astrakhan
    {  530, 6577,  169021,  5, }, // Europe/Athens
    {  530, 6285,  169026, 10, }, // Europe/Belgrade
    {  530, 4659,  169036,  7, }, // Europe/Berlin
    {  530, 6437,  169043, 11, }, // Europe/Bratislava
    {  530, 6178,  169054,  9, }, // Europe/Brussels
    {  530, 3830,  169063,  8, }, // Europe/Bucharest
    {  530, 3708,  169071,  8, }, // Europe/Budapest
    {  530, 8751,  169079,  9, }, // Europe/Busingen
    {  530, 2475,  169088,  8, }, // Europe/Chisinau
    {  530, 6238,  169096, 11, }, // Europe/Copenhagen
    {  530, 2260,  169107,  8, }, // Europe/Dublin
    {  530, 8767,  169115,  8, }, // Europe/Guernsey
    {  530, 6561,  169123,  9, }, // Europe/Helsinki
    {  530, 8783,  169132, 12, }, // Europe/Isle_of_Man
    {  530, 1097,  169144,  9, }, // Europe/Istanbul
    {  530, 8802,  169153,  7, }, // Europe/Jersey
    {  530, 8924,  169160,  6, }, // Europe/Kirov
    {  530, 2419,  169166,  6, }, // Europe/Kyiv
    {  530, 6455,  169172,  9, }, // Europe/Ljubljana
    {  530, 2393,  169181,  6, }, // Europe/London
    {  530, 6314,  169187, 10, }, // Europe/Luxembourg
    {  530, 4794,  169197,  7, }, // Europe/Madrid
    {  530, 6531,  169204,  9, }, // Europe/Mariehamn
    {  530, 3595,  169213,  6, }, // Europe/Minsk
    {  530, 6359,    6525,  6, }, // Europe/Monaco
    {  530, 3443,  169219,  6, }, // Europe/Moscow
    {  530, 4186,  169225,  7, }, // Europe/Paris
    {  530, 6373,  169232,  9, }, // Europe/Podgorica
    {  530, 6224,  169241,  6, }, // Europe/Prague
    {  530, 8816,   56011,  5, }, // Europe/Riga
    {  530, 6273,   62228,  5, }, // Europe/Rome
    {  530, 6419,  169247, 11, }, // Europe/San_Marino
    {  530, 6194,  169258,  9, }, // Europe/Sarajevo
    {  530, 4353,  169267,  8, }, // Europe/Saratov
    {  530, 8828,  169275, 10, }, // Europe/Simferopol
    {  530, 6332,  169285,  8, }, // Europe/Skopje
    {  530, 6548,   21329,  6, }, // Europe/Sofia
    {  530, 6472,  169293,  9, }, // Europe/Stockholm
    {  530, 8846,  169302,  6, }, // Europe/Tallinn
    {  530, 6135,  169308,  7, }, // Europe/Tirane
    {  530, 8861,  169315, 10, }, // Europe/Ulyanovsk
    {  530, 6301,  169325,  6, }, // Europe/Vaduz
    {  530, 6516,  169331,  8, }, // Europe/Vatican
    {  530, 6164,  169339,  7, }, // Europe/Vienna
    {  530, 8878,  169346,  8, }, // Europe/Vilnius
    {  530, 4629,  169354,  9, }, // Europe/Volgograd
    {  530, 6210,  169363,  6, }, // Europe/Zagreb
    {  530, 6489,  169369,  7, }, // Europe/Zurich
    {  530, 4980,  169376, 12, }, // Indian/Antananarivo
    {  530, 6910,  169388,  5, }, // Indian/Chagos
    {  530, 5979,  169393,  9, }, // Indian/Christmas
    {  530, 5996,    4484,  5, }, // Indian/Cocos
    {  530, 4931,   21395,  6, }, // Indian/Comoro
    {  530, 6624,  169402,  8, }, // Indian/Kerguelen
    {  530, 7013,  169410,  8, }, // Indian/Maldives
    {  530, 3971,  169418, 11, }, // Indian/Mauritius
    {  530, 5000,  169429,  7, }, // Indian/Mayotte
    {  530, 7174,  169436, 10, }, // Indian/Reunion
    {  530, 4324,  140174,  5, }, // Pacific/Apia
    {  530,  932,  169446,  7, }, // Pacific/Auckland
    {  530, 3608,  169453, 10, }, // Pacific/Bougainville
    {  530, 2863,  169463,  5, }, // Pacific/Chatham
    {  530, 3022,  169468,  4, }, // Pacific/Chuuk
    {  530, 2201,  169472,  6, }, // Pacific/Easter
    {  530, 7290,  169478,  6, }, // Pacific/Fakaofo
    {  530, 6641,  169484,  9, }, // Pacific/Galapagos
    {  530, 6659,  169493,  7, }, // Pacific/Gambier
    {  530, 3724,  169500, 11, }, // Pacific/Guadalcanal
    {  530, 5935,  169511,  5, }, // Pacific/Guam
    {  530, 2897,  123329,  9, }, // Pacific/Kanton
    {  530, 3900,  169516, 11, }, // Pacific/Kiritimati
    {  530, 6982,  169527,  6, }, // Pacific/Kosrae
    {  530, 2708,  169533, 10, }, // Pacific/Kwajalein
    {  530, 4779,  169543,  5, }, // Pacific/Majuro
    {  530, 3953,  169548, 10, }, // Pacific/Marquesas
    {  530, 7047,  169558,  6, }, // Pacific/Nauru
    {  530, 7095,  167929,  5, }, // Pacific/Niue
    {  530, 4070,  169564,  7, }, // Pacific/Norfolk
    {  530, 7061,  169571,  7, }, // Pacific/Noumea
    {  530, 2991,  169578,  9, }, // Pacific/Pago_Pago
    {  530, 7118,  169587,  5, }, // Pacific/Palau
    {  530, 7157,  169592,  8, }, // Pacific/Pitcairn
    {  530, 2961,  169600,  6, }, // Pacific/Pohnpei
    {  530, 4709,  169606, 13, }, // Pacific/Port_Moresby
    {  530, 5948,  169619,  6, }, // Pacific/Saipan
    {  530, 4485,  169625,  9, }, // Pacific/Tongatapu
    {  530, 7368,  169634,  5, }, // Pacific/Wallis
    {  534,   14,       0,  6, }, // Africa/Asmara Southern Kurdish/Arabic/Iran
    {  534, 4337,       6,  8, }, // Africa/Sao_Tome
    {  534, 4154,      14,  8, }, // America/Asuncion
    {  534,  237,      22,  8, }, // America/Atikokan
    {  534, 7586,      30, 17, }, // America/Bahia_Banderas
    {  534, 3770,      47,  6, }, // America/Cancun
    {  534, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  534, 5640,      66,  7, }, // America/Curacao
    {  534,  481,      73, 13, }, // America/Indiana/Knox
    {  534, 7924,      86, 16, }, // America/Indiana/Marengo
    {  534, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  534, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  534, 8001,     139, 14, }, // America/Indiana/Vevay
    {  534, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  534, 8049,     171, 16, }, // America/Indiana/Winamac
    {  534, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  534, 5849,     207, 22, }, // America/Lower_Princes
    {  534, 8183,     229,  6, }, // America/Merida
    {  534, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  534, 8292,     255, 20, }, // America/North_Dakota/Center
    {  534, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  534,  393,     298,  4, }, // America/Nuuk
    {  534, 6862,     302, 16, }, // America/Scoresbysund
    {  534, 8471,     318, 14, }, // America/St_Barthelemy
    {  534, 2045,     332, 10, }, // America/St_Johns
    {  534, 5780,     342,  9, }, // America/St_Kitts
    {  534, 5797,     351,  9, }, // America/St_Lucia
    {  534,  855,     360, 10, }, // America/St_Thomas
    {  534, 5830,     370, 11, }, // America/St_Vincent
    {  534, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  534, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  534, 1152,     413,  9, }, // Asia/Kathmandu
    {  534,  992,     422,  7, }, // Asia/Kolkata
    {  534, 1178,     429,  5, }, // Asia/Macau
    {  534, 1202,     442,  6, }, // Asia/Yangon
    {  534, 1379,     448,  5, }, // Atlantic/Faroe
    {  534, 6784,     453, 10, }, // Atlantic/St_Helena
    {  534, 2419,     463,  4, }, // Europe/Kyiv
    {  534, 7174,     467,  7, }, // Indian/Reunion
    {  534, 3022,     474,  5, }, // Pacific/Chuuk
    {  534, 2961,     479,  7, }, // Pacific/Pohnpei
    {  536,   14,       0,  6, }, // Africa/Asmara Southern Sami/Latin/Sweden
    {  536, 4337,       6,  8, }, // Africa/Sao_Tome
    {  536, 4154,      14,  8, }, // America/Asuncion
    {  536,  237,      22,  8, }, // America/Atikokan
    {  536, 7586,      30, 17, }, // America/Bahia_Banderas
    {  536, 3770,      47,  6, }, // America/Cancun
    {  536, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  536, 5640,      66,  7, }, // America/Curacao
    {  536,  481,      73, 13, }, // America/Indiana/Knox
    {  536, 7924,      86, 16, }, // America/Indiana/Marengo
    {  536, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  536, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  536, 8001,     139, 14, }, // America/Indiana/Vevay
    {  536, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  536, 8049,     171, 16, }, // America/Indiana/Winamac
    {  536, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  536, 5849,     207, 22, }, // America/Lower_Princes
    {  536, 8183,     229,  6, }, // America/Merida
    {  536, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  536, 8292,     255, 20, }, // America/North_Dakota/Center
    {  536, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  536,  393,     298,  4, }, // America/Nuuk
    {  536, 6862,     302, 16, }, // America/Scoresbysund
    {  536, 8471,     318, 14, }, // America/St_Barthelemy
    {  536, 2045,     332, 10, }, // America/St_Johns
    {  536, 5780,     342,  9, }, // America/St_Kitts
    {  536, 5797,     351,  9, }, // America/St_Lucia
    {  536,  855,     360, 10, }, // America/St_Thomas
    {  536, 5830,     370, 11, }, // America/St_Vincent
    {  536, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  536, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  536, 1152,     413,  9, }, // Asia/Kathmandu
    {  536,  992,     422,  7, }, // Asia/Kolkata
    {  536, 1178,     429,  5, }, // Asia/Macau
    {  536, 1202,     442,  6, }, // Asia/Yangon
    {  536, 1379,     448,  5, }, // Atlantic/Faroe
    {  536, 6784,     453, 10, }, // Atlantic/St_Helena
    {  536, 2419,     463,  4, }, // Europe/Kyiv
    {  536, 7174,     467,  7, }, // Indian/Reunion
    {  536, 3022,     474,  5, }, // Pacific/Chuuk
    {  536, 2961,     479,  7, }, // Pacific/Pohnpei
    {  538,   14,       0,  6, }, // Africa/Asmara Southern Sotho/Latin/South Africa
    {  538, 4337,       6,  8, }, // Africa/Sao_Tome
    {  538, 4154,      14,  8, }, // America/Asuncion
    {  538,  237,      22,  8, }, // America/Atikokan
    {  538, 7586,      30, 17, }, // America/Bahia_Banderas
    {  538, 3770,      47,  6, }, // America/Cancun
    {  538, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  538, 5640,      66,  7, }, // America/Curacao
    {  538,  481,      73, 13, }, // America/Indiana/Knox
    {  538, 7924,      86, 16, }, // America/Indiana/Marengo
    {  538, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  538, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  538, 8001,     139, 14, }, // America/Indiana/Vevay
    {  538, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  538, 8049,     171, 16, }, // America/Indiana/Winamac
    {  538, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  538, 5849,     207, 22, }, // America/Lower_Princes
    {  538, 8183,     229,  6, }, // America/Merida
    {  538, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  538, 8292,     255, 20, }, // America/North_Dakota/Center
    {  538, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  538,  393,     298,  4, }, // America/Nuuk
    {  538, 6862,     302, 16, }, // America/Scoresbysund
    {  538, 8471,     318, 14, }, // America/St_Barthelemy
    {  538, 2045,     332, 10, }, // America/St_Johns
    {  538, 5780,     342,  9, }, // America/St_Kitts
    {  538, 5797,     351,  9, }, // America/St_Lucia
    {  538,  855,     360, 10, }, // America/St_Thomas
    {  538, 5830,     370, 11, }, // America/St_Vincent
    {  538, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  538, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  538, 1152,     413,  9, }, // Asia/Kathmandu
    {  538,  992,     422,  7, }, // Asia/Kolkata
    {  538, 1178,     429,  5, }, // Asia/Macau
    {  538, 1202,     442,  6, }, // Asia/Yangon
    {  538, 1379,     448,  5, }, // Atlantic/Faroe
    {  538, 6784,     453, 10, }, // Atlantic/St_Helena
    {  538, 2419,     463,  4, }, // Europe/Kyiv
    {  538, 7174,     467,  7, }, // Indian/Reunion
    {  538, 3022,     474,  5, }, // Pacific/Chuuk
    {  538, 2961,     479,  7, }, // Pacific/Pohnpei
    {  540,   14,       0,  6, }, // Africa/Asmara South Ndebele/Latin/South Africa
    {  540, 4337,       6,  8, }, // Africa/Sao_Tome
    {  540, 4154,      14,  8, }, // America/Asuncion
    {  540,  237,      22,  8, }, // America/Atikokan
    {  540, 7586,      30, 17, }, // America/Bahia_Banderas
    {  540, 3770,      47,  6, }, // America/Cancun
    {  540, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  540, 5640,      66,  7, }, // America/Curacao
    {  540,  481,      73, 13, }, // America/Indiana/Knox
    {  540, 7924,      86, 16, }, // America/Indiana/Marengo
    {  540, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  540, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  540, 8001,     139, 14, }, // America/Indiana/Vevay
    {  540, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  540, 8049,     171, 16, }, // America/Indiana/Winamac
    {  540, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  540, 5849,     207, 22, }, // America/Lower_Princes
    {  540, 8183,     229,  6, }, // America/Merida
    {  540, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  540, 8292,     255, 20, }, // America/North_Dakota/Center
    {  540, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  540,  393,     298,  4, }, // America/Nuuk
    {  540, 6862,     302, 16, }, // America/Scoresbysund
    {  540, 8471,     318, 14, }, // America/St_Barthelemy
    {  540, 2045,     332, 10, }, // America/St_Johns
    {  540, 5780,     342,  9, }, // America/St_Kitts
    {  540, 5797,     351,  9, }, // America/St_Lucia
    {  540,  855,     360, 10, }, // America/St_Thomas
    {  540, 5830,     370, 11, }, // America/St_Vincent
    {  540, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  540, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  540, 1152,     413,  9, }, // Asia/Kathmandu
    {  540,  992,     422,  7, }, // Asia/Kolkata
    {  540, 1178,     429,  5, }, // Asia/Macau
    {  540, 1202,     442,  6, }, // Asia/Yangon
    {  540, 1379,     448,  5, }, // Atlantic/Faroe
    {  540, 6784,     453, 10, }, // Atlantic/St_Helena
    {  540, 2419,     463,  4, }, // Europe/Kyiv
    {  540, 7174,     467,  7, }, // Indian/Reunion
    {  540, 3022,     474,  5, }, // Pacific/Chuuk
    {  540, 2961,     479,  7, }, // Pacific/Pohnpei
    {  541, 6736,  166782,  7, }, // Africa/Conakry Spanish/Latin/Spain
    {  541, 4945,  140194,  6, }, // Africa/Djibouti
    {  541, 2165,   62647, 17, }, // America/Santiago
    {  541, 8624,   25299,  8, }, // Asia/Khandyga
    {  541, 6591,   72956,  8, }, // Atlantic/Canary
    {  541, 3522,   19429,  8, }, // Europe/Astrakhan
    {  541, 2260,   19482,  6, }, // Europe/Dublin
    {  541, 8767,   25495,  9, }, // Europe/Guernsey
    {  541, 8924,   43065,  5, }, // Europe/Kirov
    {  541, 8861,  140678,  9, }, // Europe/Ulyanovsk
    {  541, 2929,   73150,  8, }, // Pacific/Honolulu
    {  542, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Argentina
    {  542, 5383,  169652,  5, }, // America/Nassau
    {  542,  855,  169657, 11, }, // America/St_Thomas
    {  542, 7355,  169668,  9, }, // Pacific/Wake
    {  543, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Belize
    {  543, 5383,  169652,  5, }, // America/Nassau
    {  543,  855,  169657, 11, }, // America/St_Thomas
    {  543, 7355,  169668,  9, }, // Pacific/Wake
    {  544, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Bolivia
    {  544, 5383,  169652,  5, }, // America/Nassau
    {  544,  855,  169657, 11, }, // America/St_Thomas
    {  544, 7355,  169668,  9, }, // Pacific/Wake
    {  545, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Brazil
    {  545, 5383,  169652,  5, }, // America/Nassau
    {  545,  855,  169657, 11, }, // America/St_Thomas
    {  545, 7355,  169668,  9, }, // Pacific/Wake
    {  548, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Chile
    {  548, 5383,  169652,  5, }, // America/Nassau
    {  548,  855,  169657, 11, }, // America/St_Thomas
    {  548, 7355,  169668,  9, }, // Pacific/Wake
    {  549, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Colombia
    {  549, 5383,  169652,  5, }, // America/Nassau
    {  549,  855,  169657, 11, }, // America/St_Thomas
    {  549, 7355,  169668,  9, }, // Pacific/Wake
    {  550, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Costa Rica
    {  550, 5383,  169652,  5, }, // America/Nassau
    {  550,  855,  169657, 11, }, // America/St_Thomas
    {  550, 7355,  169668,  9, }, // Pacific/Wake
    {  551, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Cuba
    {  551, 5383,  169652,  5, }, // America/Nassau
    {  551,  855,  169657, 11, }, // America/St_Thomas
    {  551, 7355,  169668,  9, }, // Pacific/Wake
    {  552, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Dominican Republic
    {  552, 5383,  169652,  5, }, // America/Nassau
    {  552,  855,  169657, 11, }, // America/St_Thomas
    {  552, 7355,  169668,  9, }, // Pacific/Wake
    {  553, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Ecuador
    {  553, 5383,  169652,  5, }, // America/Nassau
    {  553,  855,  169657, 11, }, // America/St_Thomas
    {  553, 7355,  169668,  9, }, // Pacific/Wake
    {  554, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/El Salvador
    {  554, 5383,  169652,  5, }, // America/Nassau
    {  554,  855,  169657, 11, }, // America/St_Thomas
    {  554, 7355,  169668,  9, }, // Pacific/Wake
    {  556, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Guatemala
    {  556, 5383,  169652,  5, }, // America/Nassau
    {  556,  855,  169657, 11, }, // America/St_Thomas
    {  556, 7355,  169668,  9, }, // Pacific/Wake
    {  557, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Honduras
    {  557, 5383,  169652,  5, }, // America/Nassau
    {  557,  855,  169657, 11, }, // America/St_Thomas
    {  557, 7355,  169668,  9, }, // Pacific/Wake
    {  558, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Latin America
    {  558, 5383,  169652,  5, }, // America/Nassau
    {  558,  855,  169657, 11, }, // America/St_Thomas
    {  558, 7355,  169668,  9, }, // Pacific/Wake
    {  559, 4824,  169677,  9, }, // Africa/Bujumbura Spanish/Latin/Mexico
    {  559, 5032,   89900, 13, }, // Africa/Dar_es_Salaam
    {  559, 5383,  169652,  5, }, // America/Nassau
    {  559,  695,  169686, 10, }, // America/Rio_Branco
    {  559, 5297,  169696,  6, }, // Asia/Almaty
    {  559, 5270,  169702,  6, }, // Asia/Aqtobe
    {  559, 8587,    5917,  6, }, // Asia/Atyrau
    {  559, 2929,    4512,  8, }, // Pacific/Honolulu
    {  560, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Nicaragua
    {  560, 5383,  169652,  5, }, // America/Nassau
    {  560,  855,  169657, 11, }, // America/St_Thomas
    {  560, 7355,  169668,  9, }, // Pacific/Wake
    {  561, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Panama
    {  561, 5383,  169652,  5, }, // America/Nassau
    {  561,  855,  169657, 11, }, // America/St_Thomas
    {  561, 7355,  169668,  9, }, // Pacific/Wake
    {  562, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Paraguay
    {  562, 5383,  169652,  5, }, // America/Nassau
    {  562,  855,  169657, 11, }, // America/St_Thomas
    {  562, 7355,  169668,  9, }, // Pacific/Wake
    {  563, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Peru
    {  563, 5383,  169652,  5, }, // America/Nassau
    {  563,  855,  169657, 11, }, // America/St_Thomas
    {  563, 7355,  169668,  9, }, // Pacific/Wake
    {  565, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Puerto Rico
    {  565, 5383,  169652,  5, }, // America/Nassau
    {  565,  855,  169657, 11, }, // America/St_Thomas
    {  565, 7355,  169668,  9, }, // Pacific/Wake
    {  566, 6751,  154497,  6, }, // Africa/Abidjan Spanish/Latin/United States
    {  566, 6723,   72588,  4, }, // Africa/Accra
    {  566, 4961,   18754, 10, }, // Africa/Addis_Ababa
    {  566, 7383,  140180,  5, }, // Africa/Algiers
    {  566,   14,       0,  6, }, // Africa/Asmara
    {  566, 7398,  169708,  5, }, // Africa/Bissau
    {  566, 2242,  154503,  8, }, // Africa/Cairo
    {  566, 6736,  169713,  7, }, // Africa/Conakry
    {  566, 5032,  169720, 12, }, // Africa/Dar_es_Salaam
    {  566, 4945,  169732,  6, }, // Africa/Djibouti
    {  566, 5145,   19777,  5, }, // Africa/Douala
    {  566, 5068,   62570,  8, }, // Africa/El_Aaiun
    {  566, 4381,  154511, 13, }, // Africa/Johannesburg
    {  566, 4426,  140200,  6, }, // Africa/Khartoum
    {  566, 6832,   18802,  4, }, // Africa/Lome
    {  566, 5015,   42614, 10, }, // Africa/Mogadishu
    {  566, 5173,  140206,  6, }, // Africa/Ndjamena
    {  566, 6766,  140212,  8, }, // Africa/Nouakchott
    {  566, 6690,   18833,  8, }, // Africa/Ouagadougou
    {  566, 5127,  117821,  9, }, // Africa/Porto-Novo
    {  566, 4337,  169738, 10, }, // Africa/Sao_Tome
    {  566, 2732,   18851,  7, }, // Africa/Tripoli
    {  566, 6503,   18858,  5, }, // Africa/Tunis
    {  566, 5524,   18863,  7, }, // America/Anguilla
    {  566, 4456,   24992,  9, }, // America/Araguaina
    {  566,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  566, 7452,   42641, 12, }, // America/Argentina/Rio_Gallegos
    {  566, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  566, 4154,      14,  8, }, // America/Asuncion
    {  566,  237,      22,  8, }, // America/Atikokan
    {  566, 3581,      30,  5, }, // America/Bahia
    {  566, 7586,      30, 17, }, // America/Bahia_Banderas
    {  566, 7609,  140235,  5, }, // America/Belem
    {  566, 5309,  140240,  6, }, // America/Belize
    {  566, 4263,   18898,  6, }, // America/Bogota
    {  566, 3770,      47,  6, }, // America/Cancun
    {  566, 4247,  140246,  6, }, // America/Cayenne
    {  566, 5398,   18904,  6, }, // America/Cayman
    {  566, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  566, 3693,   25007,  6, }, // America/Cuiaba
    {  566, 5640,  154529,  7, }, // America/Curacao
    {  566, 7832,   25013,  8, }, // America/Eirunepe
    {  566, 7849,  121026, 11, }, // America/Fort_Nelson
    {  566, 4514,  140262, 10, }, // America/Grand_Turk
    {  566, 5687,   18910,  7, }, // America/Grenada
    {  566, 5703,   18917,  9, }, // America/Guadeloupe
    {  566, 2221,   18926,  9, }, // America/Havana
    {  566,  348,   72702, 12, }, // America/Indiana/Indianapolis
    {  566,  481,      73, 13, }, // America/Indiana/Knox
    {  566, 7924,      86, 16, }, // America/Indiana/Marengo
    {  566, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  566, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  566, 8001,     139, 14, }, // America/Indiana/Vevay
    {  566, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  566, 8049,     171, 16, }, // America/Indiana/Winamac
    {  566, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  566, 3390,  169748, 11, }, // America/Los_Angeles
    {  566, 5849,     207, 22, }, // America/Lower_Princes
    {  566, 8116,   25021,  6, }, // America/Maceio
    {  566, 1892,   63283,  6, }, // America/Manaus
    {  566, 5722,   18945,  9, }, // America/Martinique
    {  566, 2779,   25036,  8, }, // America/Mazatlan
    {  566, 8183,     229,  6, }, // America/Merida
    {  566, 2811,   56111, 16, }, // America/Mexico_City
    {  566, 4293,  169759,  8, }, // America/Miquelon
    {  566, 5383,  169767,  6, }, // America/Nassau
    {  566, 3294,   18969, 10, }, // America/New_York
    {  566, 8264,   18979, 24, }, // America/North_Dakota/Beulah
    {  566, 8292,   19003, 24, }, // America/North_Dakota/Center
    {  566, 8320,  169773, 27, }, // America/North_Dakota/New_Salem
    {  566,  393,     298,  4, }, // America/Nuuk
    {  566, 5413,   19056,  6, }, // America/Panama
    {  566, 3847,  140389, 15, }, // America/Port-au-Prince
    {  566, 5871,  154552, 13, }, // America/Port_of_Spain
    {  566,  695,   72843, 10, }, // America/Rio_Branco
    {  566, 8440,   25137,  8, }, // America/Santarem
    {  566, 2165,   62647,  8, }, // America/Santiago
    {  566, 1862,   25145,  9, }, // America/Sao_Paulo
    {  566, 6862,     302, 16, }, // America/Scoresbysund
    {  566, 8471,  154565, 13, }, // America/St_Barthelemy
    {  566, 2045,  154578, 21, }, // America/St_Johns
    {  566, 5780,  169800, 13, }, // America/St_Kitts
    {  566, 5797,   72853, 11, }, // America/St_Lucia
    {  566,  855,     360, 10, }, // America/St_Thomas
    {  566, 5830,   72864, 11, }, // America/St_Vincent
    {  566, 5605,   19114,  7, }, // America/Tortola
    {  566, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  566, 5487,   72875,  4, }, // Asia/Aden
    {  566, 3870,   56457,  5, }, // Asia/Amman
    {  566, 5428,  140495,  6, }, // Asia/Anadyr
    {  566, 5440,    5906,  5, }, // Asia/Aqtau
    {  566, 5270,    5911,  6, }, // Asia/Aqtobe
    {  566,  964,  140501,  7, }, // Asia/Ashgabat
    {  566, 3509,    4063,  6, }, // Asia/Baghdad
    {  566, 5451,  140508,  6, }, // Asia/Bahrain
    {  566, 3555,   19138,  4, }, // Asia/Baku
    {  566, 3473,  140514,  7, }, // Asia/Barnaul
    {  566, 5906,   19142,  6, }, // Asia/Brunei
    {  566, 4503,   72892,  5, }, // Asia/Chita
    {  566, 4442,   72897,  7, }, // Asia/Damascus
    {  566, 1060,  151106,  4, }, // Asia/Dhaka
    {  566, 3498,  169813,  5, }, // Asia/Dubai
    {  566, 6092,  140521,  7, }, // Asia/Dushanbe
    {  566, 4697,   19162,  6, }, // Asia/Hebron
    {  566, 1226,  169818, 18, }, // Asia/Ho_Chi_Minh
    {  566, 6969,  154623,  7, }, // Asia/Jakarta
    {  566, 1257,  169836,  9, }, // Asia/Jerusalem
    {  566, 1152,   19221,  8, }, // Asia/Kathmandu
    {  566, 8624,  169845,  8, }, // Asia/Khandyga
    {  566,  992,   19229,  7, }, // Asia/Kolkata
    {  566, 1178,     429,  5, }, // Asia/Macau
    {  566, 3919,   19241,  7, }, // Asia/Magadan
    {  566, 1316,    6069,  7, }, // Asia/Makassar
    {  566, 6883,   19248,  7, }, // Asia/Muscat
    {  566, 4116,  154630,  8, }, // Asia/Pyongyang
    {  566, 5476,   71561,  5, }, // Asia/Qatar
    {  566, 8671,  154638,  8, }, // Asia/Qostanay
    {  566, 4171,   25347,  9, }, // Asia/Qyzylorda
    {  566, 3486,    6120,  4, }, // Asia/Riyadh
    {  566, 4310,  140555,  7, }, // Asia/Sakhalin
    {  566, 7208,   19277, 10, }, // Asia/Samarkand
    {  566, 3116,   19287,  4, }, // Asia/Seoul
    {  566, 1020,  169853,  8, }, // Asia/Shanghai
    {  566, 3137,   19291,  8, }, // Asia/Singapore
    {  566, 4199,    6150, 13, }, // Asia/Srednekolymsk
    {  566, 3100,  169861,  6, }, // Asia/Taipei
    {  566, 4683,   90276,  7, }, // Asia/Tashkent
    {  566, 3817,   76718,  6, }, // Asia/Tbilisi
    {  566, 2638,   56658,  7, }, // Asia/Tehran
    {  566, 1284,  140573,  5, }, // Asia/Thimphu
    {  566, 2687,    4257,  5, }, // Asia/Tokyo
    {  566, 1346,  169867, 10, }, // Asia/Ulaanbaatar
    {  566, 1126,   25383,  6, }, // Asia/Urumqi
    {  566, 6940,   19320,  8, }, // Asia/Vientiane
    {  566, 1202,  169877, 15, }, // Asia/Yangon
    {  566, 3785,  169892, 13, }, // Asia/Yekaterinburg
    {  566, 3649,  140578,  6, }, // Asia/Yerevan
    {  566, 5588,   72942,  8, }, // Atlantic/Bermuda
    {  566, 6591,  169905, 14, }, // Atlantic/Canary
    {  566, 3629,   25418, 10, }, // Atlantic/Cape_Verde
    {  566, 1379,  169919, 11, }, // Atlantic/Faroe
    {  566, 2614,   19388,  9, }, // Atlantic/Reykjavik
    {  566, 7235,  154653, 15, }, // Atlantic/South_Georgia
    {  566, 6784,  140611, 11, }, // Atlantic/St_Helena
    {  566, 1654,   19421,  8, }, // Australia/Adelaide
    {  566, 1447,  140622,  6, }, // Australia/Sydney
    {  566, 6390,   72988,  9, }, // Europe/Amsterdam
    {  566, 3522,  169930,  9, }, // Europe/Astrakhan
    {  566, 6577,   25459,  6, }, // Europe/Athens
    {  566, 6285,   57443,  8, }, // Europe/Belgrade
    {  566, 4659,   19450,  6, }, // Europe/Berlin
    {  566, 6178,  154668,  8, }, // Europe/Brussels
    {  566, 3830,   19464,  8, }, // Europe/Bucharest
    {  566, 8751,   25487,  8, }, // Europe/Busingen
    {  566, 2475,  138461,  8, }, // Europe/Chisinau
    {  566, 6238,   19472, 10, }, // Europe/Copenhagen
    {  566, 2260,   19482,  6, }, // Europe/Dublin
    {  566, 8767,  169939,  8, }, // Europe/Guernsey
    {  566, 8783,  140628, 11, }, // Europe/Isle_of_Man
    {  566, 1097,  140639,  8, }, // Europe/Istanbul
    {  566, 3881,   61551, 12, }, // Europe/Kaliningrad
    {  566, 8924,  119814,  5, }, // Europe/Kirov
    {  566, 2419,    6486,  4, }, // Europe/Kyiv
    {  566, 3078,   19528,  6, }, // Europe/Lisbon
    {  566, 6455,   19534,  9, }, // Europe/Ljubljana
    {  566, 2393,   19543,  7, }, // Europe/London
    {  566, 6314,   73033, 10, }, // Europe/Luxembourg
    {  566, 6359,   73043,  6, }, // Europe/Monaco
    {  566, 3443,   19566,  5, }, // Europe/Moscow
    {  566, 4186,   19571,  5, }, // Europe/Paris
    {  566, 6224,   19576,  5, }, // Europe/Prague
    {  566, 6273,   19581,  4, }, // Europe/Rome
    {  566, 4353,  140665,  7, }, // Europe/Saratov
    {  566, 8828,   73064, 10, }, // Europe/Simferopol
    {  566, 6332,  140672,  6, }, // Europe/Skopje
    {  566, 6548,   19594,  5, }, // Europe/Sofia
    {  566, 6472,   73074,  9, }, // Europe/Stockholm
    {  566, 8846,   21335,  6, }, // Europe/Tallinn
    {  566, 6135,   19608,  6, }, // Europe/Tirane
    {  566, 8861,  169947,  9, }, // Europe/Ulyanovsk
    {  566, 6516,  154676, 11, }, // Europe/Vatican
    {  566, 6164,   19625,  5, }, // Europe/Vienna
    {  566, 8878,   62886,  5, }, // Europe/Vilnius
    {  566, 4629,   61732, 10, }, // Europe/Volgograd
    {  566, 3055,   19639,  8, }, // Europe/Warsaw
    {  566, 6489,  140698,  6, }, // Europe/Zurich
    {  566, 5979,  169956,  7, }, // Indian/Christmas
    {  566, 4931,  155338,  7, }, // Indian/Comoro
    {  566, 7223,   25565,  4, }, // Indian/Mahe
    {  566, 7013,   73105,  8, }, // Indian/Maldives
    {  566, 3971,   73113,  8, }, // Indian/Mauritius
    {  566, 7174,   19655,  7, }, // Indian/Reunion
    {  566, 3022,     474,  5, }, // Pacific/Chuuk
    {  566, 2201,  169963, 14, }, // Pacific/Easter
    {  566, 3804,  140723,  4, }, // Pacific/Fiji
    {  566, 6641,   19672,  9, }, // Pacific/Galapagos
    {  566, 2929,    4512,  8, }, // Pacific/Honolulu
    {  566, 2897,    4520,  9, }, // Pacific/Kanton
    {  566, 7061,  120279,  5, }, // Pacific/Noumea
    {  566, 7118,   63962,  6, }, // Pacific/Palau
    {  566, 2961,     479,  7, }, // Pacific/Pohnpei
    {  566, 5948,  169977,  6, }, // Pacific/Saipan
    {  566, 7275,   43161,  6, }, // Pacific/Tahiti
    {  566, 7355,   90914,  4, }, // Pacific/Wake
    {  567, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Uruguay
    {  567, 5383,  169652,  5, }, // America/Nassau
    {  567,  855,  169657, 11, }, // America/St_Thomas
    {  567, 7355,  169668,  9, }, // Pacific/Wake
    {  568, 7849,  169639, 13, }, // America/Fort_Nelson Spanish/Latin/Venezuela
    {  568, 5383,  169652,  5, }, // America/Nassau
    {  568,  855,  169657, 11, }, // America/St_Thomas
    {  568, 7355,  169668,  9, }, // Pacific/Wake
    {  569,   14,       0,  6, }, // Africa/Asmara Standard Moroccan Tamazight/Tifinagh/Morocco
    {  569, 4337,       6,  8, }, // Africa/Sao_Tome
    {  569, 4154,      14,  8, }, // America/Asuncion
    {  569,  237,      22,  8, }, // America/Atikokan
    {  569, 7586,      30, 17, }, // America/Bahia_Banderas
    {  569, 3770,      47,  6, }, // America/Cancun
    {  569, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  569, 5640,      66,  7, }, // America/Curacao
    {  569,  481,      73, 13, }, // America/Indiana/Knox
    {  569, 7924,      86, 16, }, // America/Indiana/Marengo
    {  569, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  569, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  569, 8001,     139, 14, }, // America/Indiana/Vevay
    {  569, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  569, 8049,     171, 16, }, // America/Indiana/Winamac
    {  569, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  569, 5849,     207, 22, }, // America/Lower_Princes
    {  569, 8183,     229,  6, }, // America/Merida
    {  569, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  569, 8292,     255, 20, }, // America/North_Dakota/Center
    {  569, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  569,  393,     298,  4, }, // America/Nuuk
    {  569, 6862,     302, 16, }, // America/Scoresbysund
    {  569, 8471,     318, 14, }, // America/St_Barthelemy
    {  569, 2045,     332, 10, }, // America/St_Johns
    {  569, 5780,     342,  9, }, // America/St_Kitts
    {  569, 5797,     351,  9, }, // America/St_Lucia
    {  569,  855,     360, 10, }, // America/St_Thomas
    {  569, 5830,     370, 11, }, // America/St_Vincent
    {  569, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  569, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  569, 1152,     413,  9, }, // Asia/Kathmandu
    {  569,  992,     422,  7, }, // Asia/Kolkata
    {  569, 1178,     429,  5, }, // Asia/Macau
    {  569, 1202,     442,  6, }, // Asia/Yangon
    {  569, 1379,     448,  5, }, // Atlantic/Faroe
    {  569, 6784,     453, 10, }, // Atlantic/St_Helena
    {  569, 2419,     463,  4, }, // Europe/Kyiv
    {  569, 7174,     467,  7, }, // Indian/Reunion
    {  569, 3022,     474,  5, }, // Pacific/Chuuk
    {  569, 2961,     479,  7, }, // Pacific/Pohnpei
    {  570,   14,       0,  6, }, // Africa/Asmara Sundanese/Latin/Indonesia
    {  570, 4337,       6,  8, }, // Africa/Sao_Tome
    {  570, 4154,      14,  8, }, // America/Asuncion
    {  570,  237,      22,  8, }, // America/Atikokan
    {  570, 7586,      30, 17, }, // America/Bahia_Banderas
    {  570, 3770,      47,  6, }, // America/Cancun
    {  570, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  570, 5640,      66,  7, }, // America/Curacao
    {  570,  481,      73, 13, }, // America/Indiana/Knox
    {  570, 7924,      86, 16, }, // America/Indiana/Marengo
    {  570, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  570, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  570, 8001,     139, 14, }, // America/Indiana/Vevay
    {  570, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  570, 8049,     171, 16, }, // America/Indiana/Winamac
    {  570, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  570, 5849,     207, 22, }, // America/Lower_Princes
    {  570, 8183,     229,  6, }, // America/Merida
    {  570, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  570, 8292,     255, 20, }, // America/North_Dakota/Center
    {  570, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  570,  393,     298,  4, }, // America/Nuuk
    {  570, 6862,     302, 16, }, // America/Scoresbysund
    {  570, 8471,     318, 14, }, // America/St_Barthelemy
    {  570, 2045,     332, 10, }, // America/St_Johns
    {  570, 5780,     342,  9, }, // America/St_Kitts
    {  570, 5797,     351,  9, }, // America/St_Lucia
    {  570,  855,     360, 10, }, // America/St_Thomas
    {  570, 5830,     370, 11, }, // America/St_Vincent
    {  570, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  570, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  570, 1152,     413,  9, }, // Asia/Kathmandu
    {  570,  992,     422,  7, }, // Asia/Kolkata
    {  570, 1178,   32008,  5, }, // Asia/Macau
    {  570, 1202,     442,  6, }, // Asia/Yangon
    {  570, 1379,     448,  5, }, // Atlantic/Faroe
    {  570, 6784,     453, 10, }, // Atlantic/St_Helena
    {  570, 2419,     463,  4, }, // Europe/Kyiv
    {  570, 7174,     467,  7, }, // Indian/Reunion
    {  570, 3022,     474,  5, }, // Pacific/Chuuk
    {  570, 2961,     479,  7, }, // Pacific/Pohnpei
    {  571,   14,       0,  6, }, // Africa/Asmara Swahili/Latin/Tanzania
    {  571, 4337,       6,  8, }, // Africa/Sao_Tome
    {  571, 4154,      14,  8, }, // America/Asuncion
    {  571,  237,      22,  8, }, // America/Atikokan
    {  571, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  571, 3770,    3861,  6, }, // America/Cancun
    {  571, 7737,   89795, 13, }, // America/Ciudad_Juarez
    {  571, 5640,      66,  7, }, // America/Curacao
    {  571,  481,      73, 13, }, // America/Indiana/Knox
    {  571, 7924,      86, 16, }, // America/Indiana/Marengo
    {  571, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  571, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  571, 8001,     139, 14, }, // America/Indiana/Vevay
    {  571, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  571, 8049,     171, 16, }, // America/Indiana/Winamac
    {  571, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  571, 5849,     207, 22, }, // America/Lower_Princes
    {  571, 8183,    3899,  6, }, // America/Merida
    {  571, 2811,  169983, 14, }, // America/Mexico_City
    {  571, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  571, 8292,     255, 20, }, // America/North_Dakota/Center
    {  571, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  571,  393,     298,  4, }, // America/Nuuk
    {  571, 6862,     302, 16, }, // America/Scoresbysund
    {  571, 8471,     318, 14, }, // America/St_Barthelemy
    {  571, 2045,     332, 10, }, // America/St_Johns
    {  571, 5780,     342,  9, }, // America/St_Kitts
    {  571, 5797,     351,  9, }, // America/St_Lucia
    {  571,  855,     360, 10, }, // America/St_Thomas
    {  571, 5830,     370, 11, }, // America/St_Vincent
    {  571, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  571, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  571, 1152,     413,  9, }, // Asia/Kathmandu
    {  571,  992,     422,  7, }, // Asia/Kolkata
    {  571, 1178,    4222,  5, }, // Asia/Macau
    {  571, 8671,     434,  8, }, // Asia/Qostanay
    {  571, 1202,   56146,  7, }, // Asia/Yangon
    {  571, 1379,     448,  5, }, // Atlantic/Faroe
    {  571, 7235,  169997, 14, }, // Atlantic/South_Georgia
    {  571, 6784,     453, 10, }, // Atlantic/St_Helena
    {  571, 2419,    6486,  4, }, // Europe/Kyiv
    {  571, 7174,     467,  7, }, // Indian/Reunion
    {  571, 3022,     474,  5, }, // Pacific/Chuuk
    {  571, 2929,    4512,  8, }, // Pacific/Honolulu
    {  571, 2897,    4520,  9, }, // Pacific/Kanton
    {  571, 2961,     479,  7, }, // Pacific/Pohnpei
    {  573, 5571,  170011,  7, }, // America/Barbados Swahili/Latin/Kenya
    {  573, 5324,   31782,  9, }, // America/Costa_Rica
    {  573, 5640,  170018,  7, }, // America/Curacao
    {  573, 5656,   20071,  8, }, // America/Dominica
    {  573, 5343,  170025,  9, }, // America/El_Salvador
    {  573, 5703,   18917,  9, }, // America/Guadeloupe
    {  573, 2665,    3892,  7, }, // America/Jamaica
    {  573, 8264,  170034, 24, }, // America/North_Dakota/Beulah
    {  573, 8292,  170058, 24, }, // America/North_Dakota/Center
    {  573, 8320,  170082, 27, }, // America/North_Dakota/New_Salem
    {  573, 5871,  170109, 20, }, // America/Port_of_Spain
    {  573, 5760,  170129,  9, }, // America/Puerto_Rico
    {  573, 4413,    5949,  7, }, // Asia/Colombo
    {  573, 1226,  170138, 19, }, // Asia/Ho_Chi_Minh
    {  573, 1178,    6064,  5, }, // Asia/Macau
    {  573, 3137,  170157,  8, }, // Asia/Singapore
    {  573, 1202,     442,  6, }, // Asia/Yangon
    {  573, 6591,    4299,  6, }, // Atlantic/Canary
    {  573, 5979,  170165,  8, }, // Indian/Christmas
    {  573, 7013,   32290,  7, }, // Indian/Maldives
    {  575,   14,       0,  6, }, // Africa/Asmara Swati/Latin/South Africa
    {  575, 4337,       6,  8, }, // Africa/Sao_Tome
    {  575, 4154,      14,  8, }, // America/Asuncion
    {  575,  237,      22,  8, }, // America/Atikokan
    {  575, 7586,      30, 17, }, // America/Bahia_Banderas
    {  575, 3770,      47,  6, }, // America/Cancun
    {  575, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  575, 5640,      66,  7, }, // America/Curacao
    {  575,  481,      73, 13, }, // America/Indiana/Knox
    {  575, 7924,      86, 16, }, // America/Indiana/Marengo
    {  575, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  575, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  575, 8001,     139, 14, }, // America/Indiana/Vevay
    {  575, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  575, 8049,     171, 16, }, // America/Indiana/Winamac
    {  575, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  575, 5849,     207, 22, }, // America/Lower_Princes
    {  575, 8183,     229,  6, }, // America/Merida
    {  575, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  575, 8292,     255, 20, }, // America/North_Dakota/Center
    {  575, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  575,  393,     298,  4, }, // America/Nuuk
    {  575, 6862,     302, 16, }, // America/Scoresbysund
    {  575, 8471,     318, 14, }, // America/St_Barthelemy
    {  575, 2045,     332, 10, }, // America/St_Johns
    {  575, 5780,     342,  9, }, // America/St_Kitts
    {  575, 5797,     351,  9, }, // America/St_Lucia
    {  575,  855,     360, 10, }, // America/St_Thomas
    {  575, 5830,     370, 11, }, // America/St_Vincent
    {  575, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  575, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  575, 1152,     413,  9, }, // Asia/Kathmandu
    {  575,  992,     422,  7, }, // Asia/Kolkata
    {  575, 1178,     429,  5, }, // Asia/Macau
    {  575, 1202,     442,  6, }, // Asia/Yangon
    {  575, 1379,     448,  5, }, // Atlantic/Faroe
    {  575, 6784,     453, 10, }, // Atlantic/St_Helena
    {  575, 2419,     463,  4, }, // Europe/Kyiv
    {  575, 7174,     467,  7, }, // Indian/Reunion
    {  575, 3022,     474,  5, }, // Pacific/Chuuk
    {  575, 2961,     479,  7, }, // Pacific/Pohnpei
    {  577, 4961,    3793, 11, }, // Africa/Addis_Ababa Swedish/Latin/Sweden
    {  577, 7383,   42592,  5, }, // Africa/Algiers
    {  577,   14,       0,  6, }, // Africa/Asmara
    {  577, 2242,   24913,  5, }, // Africa/Cairo
    {  577, 5032,   89900, 13, }, // Africa/Dar_es_Salaam
    {  577, 5068,  166029,  8, }, // Africa/El_Aaiun
    {  577, 6832,   18802,  4, }, // Africa/Lome
    {  577, 5173,   42632,  9, }, // Africa/Ndjamena
    {  577, 4337,       6,  8, }, // Africa/Sao_Tome
    {  577, 4456,   24992,  9, }, // America/Araguaina
    {  577,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  577,  441,   59968, 21, }, // America/Argentina/Jujuy
    {  577, 7452,   42641, 12, }, // America/Argentina/Rio_Gallegos
    {  577, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  577, 4154,      14,  8, }, // America/Asuncion
    {  577,  237,      22,  8, }, // America/Atikokan
    {  577, 7586,      30, 17, }, // America/Bahia_Banderas
    {  577, 7609,   34376,  5, }, // America/Belem
    {  577, 4263,   18898,  6, }, // America/Bogota
    {  577, 3770,      47,  6, }, // America/Cancun
    {  577, 5398,  170173, 11, }, // America/Cayman
    {  577, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  577, 3693,   25007,  6, }, // America/Cuiaba
    {  577, 5640,      66,  7, }, // America/Curacao
    {  577, 7832,   25013,  8, }, // America/Eirunepe
    {  577, 2221,   61885,  7, }, // America/Havana
    {  577,  481,      73, 13, }, // America/Indiana/Knox
    {  577, 7924,      86, 16, }, // America/Indiana/Marengo
    {  577, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  577, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  577, 8001,     139, 14, }, // America/Indiana/Vevay
    {  577, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  577, 8049,     171, 16, }, // America/Indiana/Winamac
    {  577, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  577, 5849,     207, 22, }, // America/Lower_Princes
    {  577, 8116,   25021,  6, }, // America/Maceio
    {  577, 2779,   25036,  8, }, // America/Mazatlan
    {  577, 8183,     229,  6, }, // America/Merida
    {  577, 2811,  170184, 11, }, // America/Mexico_City
    {  577, 1834,   60511, 19, }, // America/Noronha
    {  577, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  577, 8292,     255, 20, }, // America/North_Dakota/Center
    {  577, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  577,  393,     298,  4, }, // America/Nuuk
    {  577, 8440,   25137,  8, }, // America/Santarem
    {  577, 1862,   25145,  9, }, // America/Sao_Paulo
    {  577, 6862,     302, 16, }, // America/Scoresbysund
    {  577, 8471,   25154, 16, }, // America/St_Barthelemy
    {  577, 2045,   25170, 12, }, // America/St_Johns
    {  577, 5780,   25182, 11, }, // America/St_Kitts
    {  577, 5797,   34444, 11, }, // America/St_Lucia
    {  577,  855,   42797, 12, }, // America/St_Thomas
    {  577, 5830,   25216, 13, }, // America/St_Vincent
    {  577, 5673,   25229,  7, }, // America/Thule
    {  577, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  577, 5440,    5906,  5, }, // Asia/Aqtau
    {  577, 5270,   55725,  6, }, // Asia/Aqtobe
    {  577,  964,    4054,  9, }, // Asia/Ashgabat
    {  577, 3509,    4063,  6, }, // Asia/Baghdad
    {  577, 3680,    4088,  7, }, // Asia/Bishkek
    {  577, 4503,  170195,  5, }, // Asia/Chita
    {  577, 5963,  170200, 10, }, // Asia/Choibalsan
    {  577, 4442,    4102,  8, }, // Asia/Damascus
    {  577, 6092,   57125,  8, }, // Asia/Dushanbe
    {  577, 1226,  170210, 18, }, // Asia/Ho_Chi_Minh
    {  577, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  577, 4673,  166216,  5, }, // Asia/Hovd
    {  577, 4218,   57133,  9, }, // Asia/Kamchatka
    {  577, 1152,    6018,  8, }, // Asia/Kathmandu
    {  577, 8624,   56553,  8, }, // Asia/Khandyga
    {  577,  992,     422,  7, }, // Asia/Kolkata
    {  577, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  577, 1178,     429,  5, }, // Asia/Macau
    {  577, 6883,    4227,  6, }, // Asia/Muscat
    {  577, 8671,    6103,  8, }, // Asia/Qostanay
    {  577, 4310,   56617,  8, }, // Asia/Sakhalin
    {  577, 4683,   57154,  8, }, // Asia/Tashkent
    {  577, 2638,    6175,  7, }, // Asia/Tehran
    {  577, 1126,   25383,  6, }, // Asia/Urumqi
    {  577, 4730,    6206,  7, }, // Asia/Yakutsk
    {  577, 1202,     442,  6, }, // Asia/Yangon
    {  577, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  577, 3649,    6231,  7, }, // Asia/Yerevan
    {  577, 3565,  170228,  8, }, // Atlantic/Azores
    {  577, 6591,  170236, 12, }, // Atlantic/Canary
    {  577, 3629,   55900,  9, }, // Atlantic/Cape_Verde
    {  577, 1379,  170248,  8, }, // Atlantic/Faroe
    {  577, 7235,  170256, 11, }, // Atlantic/South_Georgia
    {  577, 6784,  170267, 12, }, // Atlantic/St_Helena
    {  577, 6577,   19437,  4, }, // Europe/Athens
    {  577, 6285,   25465,  7, }, // Europe/Belgrade
    {  577, 6178,   62796,  7, }, // Europe/Brussels
    {  577, 3830,   25479,  8, }, // Europe/Bucharest
    {  577, 8751,   61453, 21, }, // Europe/Busingen
    {  577, 2475,   62164,  8, }, // Europe/Chisinau
    {  577, 6238,  170279,  9, }, // Europe/Copenhagen
    {  577, 6561,  139856, 11, }, // Europe/Helsinki
    {  577, 2419,    6486,  4, }, // Europe/Kyiv
    {  577, 3078,    4400,  8, }, // Europe/Lisbon
    {  577, 6314,    4414,  9, }, // Europe/Luxembourg
    {  577, 3443,   21290,  6, }, // Europe/Moscow
    {  577, 6224,    6545,  4, }, // Europe/Prague
    {  577, 6273,    6554,  3, }, // Europe/Rome
    {  577, 6135,   19608,  6, }, // Europe/Tirane
    {  577, 8861,    6605,  9, }, // Europe/Ulyanovsk
    {  577, 6516,  170288,  9, }, // Europe/Vatican
    {  577, 6164,   56035,  4, }, // Europe/Vienna
    {  577, 3055,   57212,  8, }, // Europe/Warsaw
    {  577, 6489,    4478,  6, }, // Europe/Zurich
    {  577, 5979,  170297,  5, }, // Indian/Christmas
    {  577, 5996,  170302, 10, }, // Indian/Cocos
    {  577, 4931,  170312,  9, }, // Indian/Comoro
    {  577, 6624,  170321, 14, }, // Indian/Kerguelen
    {  577, 7223,   25565,  4, }, // Indian/Mahe
    {  577, 7013,  170335, 10, }, // Indian/Maldives
    {  577, 7174,     467,  7, }, // Indian/Reunion
    {  577, 3022,     474,  5, }, // Pacific/Chuuk
    {  577, 2201,  170345,  6, }, // Pacific/Easter
    {  577, 6641,   19672,  9, }, // Pacific/Galapagos
    {  577, 6659,  170351, 12, }, // Pacific/Gambier
    {  577, 2929,    4512,  8, }, // Pacific/Honolulu
    {  577, 2897,    4520,  9, }, // Pacific/Kanton
    {  577, 3953,  170363, 14, }, // Pacific/Marquesas
    {  577, 8909,  170377, 11, }, // Pacific/Midway
    {  577, 7061,    4529,  6, }, // Pacific/Noumea
    {  577, 7157,  170388, 13, }, // Pacific/Pitcairn
    {  577, 2961,     479,  7, }, // Pacific/Pohnpei
    {  577, 7368,  170401,  8, }, // Pacific/Wallis
    {  580, 6723,   19687,  5, }, // Africa/Accra Swiss German/Latin/Switzerland
    {  580, 4961,    3793, 11, }, // Africa/Addis_Ababa
    {  580, 7383,   57088,  6, }, // Africa/Algiers
    {  580,   14,       0,  6, }, // Africa/Asmara
    {  580, 2242,   24913,  5, }, // Africa/Cairo
    {  580, 5032,   76549, 10, }, // Africa/Dar_es_Salaam
    {  580, 4945,   76559,  9, }, // Africa/Djibouti
    {  580, 5068,   62570,  8, }, // Africa/El_Aaiun
    {  580, 4426,   24937,  7, }, // Africa/Khartoum
    {  580, 5015,   76568, 10, }, // Africa/Mogadishu
    {  580, 6690,   97031,  8, }, // Africa/Ouagadougou
    {  580, 4337,       6,  8, }, // Africa/Sao_Tome
    {  580, 4154,      14,  8, }, // America/Asuncion
    {  580,  237,      22,  8, }, // America/Atikokan
    {  580, 7586,      30, 17, }, // America/Bahia_Banderas
    {  580, 3770,      47,  6, }, // America/Cancun
    {  580, 5398,  170409, 11, }, // America/Cayman
    {  580, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  580, 5640,      66,  7, }, // America/Curacao
    {  580, 5343,    5090,  8, }, // America/El_Salvador
    {  580, 2221,   61885,  7, }, // America/Havana
    {  580,  481,      73,  4, }, // America/Indiana/Knox
    {  580, 7924,      86, 16, }, // America/Indiana/Marengo
    {  580, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  580, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  580, 8001,     139, 14, }, // America/Indiana/Vevay
    {  580, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  580, 8049,     171, 16, }, // America/Indiana/Winamac
    {  580, 2665,    3892,  7, }, // America/Jamaica
    {  580, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  580, 5849,     207, 22, }, // America/Lower_Princes
    {  580, 8183,     229,  6, }, // America/Merida
    {  580, 2811,  170420, 14, }, // America/Mexico_City
    {  580, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  580, 8292,     255, 20, }, // America/North_Dakota/Center
    {  580, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  580,  393,     298,  4, }, // America/Nuuk
    {  580, 5871,   25124, 13, }, // America/Port_of_Spain
    {  580, 6862,     302, 16, }, // America/Scoresbysund
    {  580, 8471,     318, 14, }, // America/St_Barthelemy
    {  580, 2045,     332, 10, }, // America/St_Johns
    {  580, 5780,     342,  9, }, // America/St_Kitts
    {  580, 5797,     351,  9, }, // America/St_Lucia
    {  580,  855,     360, 10, }, // America/St_Thomas
    {  580, 5830,     370, 11, }, // America/St_Vincent
    {  580, 6066,  123530, 16, }, // Antarctica/DumontDUrville
    {  580, 7337,  170434,  8, }, // Antarctica/Vostok
    {  580, 5270,    5911,  6, }, // Asia/Aqtobe
    {  580, 3509,    4063,  6, }, // Asia/Baghdad
    {  580, 3680,   55740,  8, }, // Asia/Bishkek
    {  580, 4442,    4102,  8, }, // Asia/Damascus
    {  580, 6092,   55748,  9, }, // Asia/Dushanbe
    {  580, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  580, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  580, 4218,   76660, 11, }, // Asia/Kamchatka
    {  580, 1152,     413,  9, }, // Asia/Kathmandu
    {  580,  992,     422,  7, }, // Asia/Kolkata
    {  580, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  580, 1178,     429,  5, }, // Asia/Macau
    {  580, 6883,  170442,  8, }, // Asia/Muscat
    {  580, 2446,   25331,  7, }, // Asia/Nicosia
    {  580, 4037,   76692, 11, }, // Asia/Novosibirsk
    {  580, 4116,   76703,  9, }, // Asia/Pyongyang
    {  580, 5476,    4233,  5, }, // Asia/Qatar
    {  580, 3486,    6120,  4, }, // Asia/Riyadh
    {  580, 4310,   56617,  8, }, // Asia/Sakhalin
    {  580, 3137,   19291,  8, }, // Asia/Singapore
    {  580, 3100,   76712,  6, }, // Asia/Taipei
    {  580, 4683,   55848,  9, }, // Asia/Tashkent
    {  580, 3817,   76718,  6, }, // Asia/Tbilisi
    {  580, 2638,    6175,  7, }, // Asia/Tehran
    {  580, 2687,    4257,  5, }, // Asia/Tokyo
    {  580, 1346,  170450, 11, }, // Asia/Ulaanbaatar
    {  580, 4612,    4262, 11, }, // Asia/Vladivostok
    {  580, 4730,    6206,  7, }, // Asia/Yakutsk
    {  580, 1202,     442,  6, }, // Asia/Yangon
    {  580, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  580, 3649,  123737,  6, }, // Asia/Yerevan
    {  580, 3565,    6238,  5, }, // Atlantic/Azores
    {  580, 5588,   72942,  8, }, // Atlantic/Bermuda
    {  580, 6591,   76724,  6, }, // Atlantic/Canary
    {  580, 3629,   55900,  9, }, // Atlantic/Cape_Verde
    {  580, 1379,   76731,  6, }, // Atlantic/Faroe
    {  580, 7235,  170461, 12, }, // Atlantic/South_Georgia
    {  580, 6784,     453, 10, }, // Atlantic/St_Helena
    {  580, 6577,    4339,  5, }, // Europe/Athens
    {  580, 6178,   21176,  7, }, // Europe/Brussels
    {  580, 3830,  170473, 10, }, // Europe/Bucharest
    {  580, 2475,  123768,  9, }, // Europe/Chisinau
    {  580, 6238,  170483,  8, }, // Europe/Copenhagen
    {  580, 2419,   76773,  4, }, // Europe/Kyiv
    {  580, 3078,    4400,  8, }, // Europe/Lisbon
    {  580, 6314,    4414,  9, }, // Europe/Luxembourg
    {  580, 3443,   56005,  6, }, // Europe/Moscow
    {  580, 6273,    6554,  3, }, // Europe/Rome
    {  580, 6135,   19608,  6, }, // Europe/Tirane
    {  580, 6164,   56035,  4, }, // Europe/Vienna
    {  580, 8878,  123642,  5, }, // Europe/Vilnius
    {  580, 3055,   56048,  8, }, // Europe/Warsaw
    {  580, 6489,    4478,  4, }, // Europe/Zurich
    {  580, 5979,  170491, 15, }, // Indian/Christmas
    {  580, 4931,    6656,  6, }, // Indian/Comoro
    {  580, 7013,    4495,  8, }, // Indian/Maldives
    {  580, 7174,     467,  7, }, // Indian/Reunion
    {  580, 3022,     474,  5, }, // Pacific/Chuuk
    {  580, 2201,  170506, 12, }, // Pacific/Easter
    {  580, 3804,   76834,  7, }, // Pacific/Fiji
    {  580, 2961,     479,  7, }, // Pacific/Pohnpei
    {  583, 6751,  170518,  5, }, // Africa/Abidjan Syriac/Syriac/Iraq
    {  583, 6723,  170523,  4, }, // Africa/Accra
    {  583, 4961,  170527,  9, }, // Africa/Addis_Ababa
    {  583, 7383,  170536,  5, }, // Africa/Algiers
    {  583,   14,  170541,  6, }, // Africa/Asmara
    {  583,   44,  170547,  6, }, // Africa/Bamako
    {  583, 5159,  170553,  7, }, // Africa/Bangui
    {  583, 6709,  170560,  6, }, // Africa/Banjul
    {  583, 7398,  170566,  5, }, // Africa/Bissau
    {  583, 4859,  170571,  6, }, // Africa/Blantyre
    {  583, 5189,  170577,  8, }, // Africa/Brazzaville
    {  583, 4824,  170585,  9, }, // Africa/Bujumbura
    {  583, 2242,  170594,  6, }, // Africa/Cairo
    {  583, 4019,  170600,  9, }, // Africa/Casablanca
    {  583, 7412,  170609,  4, }, // Africa/Ceuta
    {  583, 6736,  170613,  7, }, // Africa/Conakry
    {  583, 6803,  170620,  5, }, // Africa/Dakar
    {  583, 5032,  170625,  8, }, // Africa/Dar_es_Salaam
    {  583, 4945,  170633,  6, }, // Africa/Djibouti
    {  583, 5145,  170639,  5, }, // Africa/Douala
    {  583, 5068,  170644,  7, }, // Africa/El_Aaiun
    {  583, 6816,  170651,  7, }, // Africa/Freetown
    {  583, 4808,  170658,  6, }, // Africa/Gaborone
    {  583, 4917,  170664,  5, }, // Africa/Harare
    {  583, 4381,  170669, 10, }, // Africa/Johannesburg
    {  583, 4401,  170679,  4, }, // Africa/Juba
    {  583, 5053,  170683,  7, }, // Africa/Kampala
    {  583, 4426,  170690,  5, }, // Africa/Khartoum
    {  583, 4875,  170695,  6, }, // Africa/Kigali
    {  583, 5208,  170701,  7, }, // Africa/Kinshasa
    {  583, 4646,  170708,  5, }, // Africa/Lagos
    {  583, 5238,  170713,  8, }, // Africa/Libreville
    {  583, 6832,  170721,  4, }, // Africa/Lome
    {  583, 5113,  170725,  6, }, // Africa/Luanda
    {  583, 4841,  170731,  8, }, // Africa/Lubumbashi
    {  583, 4903,  170739,  6, }, // Africa/Lusaka
    {  583, 5224,  170745,  6, }, // Africa/Malabo
    {  583, 4889,  170751,  6, }, // Africa/Maputo
    {  583, 5099,  170757,  6, }, // Africa/Maseru
    {  583, 5084,  170763,  7, }, // Africa/Mbabane
    {  583, 5015,  170770,  7, }, // Africa/Mogadishu
    {  583, 6997,  170777,  8, }, // Africa/Monrovia
    {  583, 3755,  170785,  6, }, // Africa/Nairobi
    {  583, 5173,  170791,  6, }, // Africa/Ndjamena
    {  583, 5256,  170797,  5, }, // Africa/Niamey
    {  583, 6766,  170802,  7, }, // Africa/Nouakchott
    {  583, 6690,  170809, 11, }, // Africa/Ouagadougou
    {  583, 5127,  170820, 10, }, // Africa/Porto-Novo
    {  583, 4337,  170830,  8, }, // Africa/Sao_Tome
    {  583, 2732,  170838,  8, }, // Africa/Tripoli
    {  583, 6503,  170846,  4, }, // Africa/Tunis
    {  583, 4054,  170850,  7, }, // Africa/Windhoek
    {  583,  132,  170857,  4, }, // America/Adak
    {  583, 3183,  170861,  6, }, // America/Anchorage
    {  583, 5524,  170867,  6, }, // America/Anguilla
    {  583, 5541,  170873,  7, }, // America/Antigua
    {  583, 4456,  170880,  9, }, // America/Araguaina
    {  583,  166,  170889, 12, }, // America/Argentina/Buenos_Aires
    {  583,   91,  170901,  8, }, // America/Argentina/Catamarca
    {  583,  270,  170909,  7, }, // America/Argentina/Cordoba
    {  583,  441,  170916,  5, }, // America/Argentina/Jujuy
    {  583, 7425,  170921,  8, }, // America/Argentina/La_Rioja
    {  583,  565,  170929,  6, }, // America/Argentina/Mendoza
    {  583, 7452,  170935,  9, }, // America/Argentina/Rio_Gallegos
    {  583, 7483,  170944,  4, }, // America/Argentina/Salta
    {  583, 7507,  170948,  7, }, // America/Argentina/San_Juan
    {  583, 5497,  170955,  8, }, // America/Argentina/San_Luis
    {  583, 7534,  170963,  7, }, // America/Argentina/Tucuman
    {  583, 7560,  170970,  7, }, // America/Argentina/Ushuaia
    {  583, 5557,  170977,  5, }, // America/Aruba
    {  583, 4154,  170982,  9, }, // America/Asuncion
    {  583,  237,  170991,  8, }, // America/Atikokan
    {  583, 3581,  170999,  5, }, // America/Bahia
    {  583, 7586,  171004, 12, }, // America/Bahia_Banderas
    {  583, 5571,  171016,  7, }, // America/Barbados
    {  583, 7609,  171023,  5, }, // America/Belem
    {  583, 5309,  171028,  5, }, // America/Belize
    {  583, 7623,  171033, 12, }, // America/Blanc-Sablon
    {  583, 7644,  171045,  9, }, // America/Boa_Vista
    {  583, 4263,  171054,  6, }, // America/Bogota
    {  583, 7662,  171060,  5, }, // America/Boise
    {  583, 7676,  171065,  9, }, // America/Cambridge_Bay
    {  583, 7698,  171074, 11, }, // America/Campo_Grande
    {  583, 3770,  171085,  6, }, // America/Cancun
    {  583, 4596,  171091,  6, }, // America/Caracas
    {  583, 4247,  171097,  5, }, // America/Cayenne
    {  583, 5398,  171102,  6, }, // America/Cayman
    {  583, 3251,  171108,  4, }, // America/Chicago
    {  583, 7719,  171112,  6, }, // America/Chihuahua
    {  583, 7737,  171118, 12, }, // America/Ciudad_Juarez
    {  583, 5324,  171130, 10, }, // America/Costa_Rica
    {  583, 7759,  171140,  7, }, // America/Creston
    {  583, 3693,  171147,  6, }, // America/Cuiaba
    {  583, 5640,  171153,  7, }, // America/Curacao
    {  583, 7775,  171160, 10, }, // America/Danmarkshavn
    {  583, 7796,  171170,  5, }, // America/Dawson
    {  583, 7811,  171175, 10, }, // America/Dawson_Creek
    {  583,  805,  171185,  4, }, // America/Denver
    {  583, 3351,  171189,  7, }, // America/Detroit
    {  583, 5656,  171196,  8, }, // America/Dominica
    {  583,  893,  171204,  8, }, // America/Edmonton
    {  583, 7832,  171212,  8, }, // America/Eirunepe
    {  583, 5343,  171220, 10, }, // America/El_Salvador
    {  583, 7849,  171230, 11, }, // America/Fort_Nelson
    {  583, 7869,  171241,  9, }, // America/Fortaleza
    {  583, 7887,  171250,  8, }, // America/Glace_Bay
    {  583, 6844,  171258,  7, }, // America/Goose_Bay
    {  583, 4514,  171265, 10, }, // America/Grand_Turk
    {  583, 5687,  171275,  6, }, // America/Grenada
    {  583, 5703,  171281,  9, }, // America/Guadeloupe
    {  583, 3662,  171290,  8, }, // America/Guatemala
    {  583, 4743,  171298,  9, }, // America/Guayaquil
    {  583, 6895,  171307,  6, }, // America/Guyana
    {  583, 1923,  171313,  7, }, // America/Halifax
    {  583, 2221,  171320,  6, }, // America/Havana
    {  583, 7905,  171326,  8, }, // America/Hermosillo
    {  583,  348,  171334, 12, }, // America/Indiana/Indianapolis
    {  583,  481,  171346, 13, }, // America/Indiana/Knox
    {  583, 7924,  171359, 15, }, // America/Indiana/Marengo
    {  583, 7948,  171374, 19, }, // America/Indiana/Petersburg
    {  583, 7975,  171393, 20, }, // America/Indiana/Tell_City
    {  583, 8001,  171413, 14, }, // America/Indiana/Vevay
    {  583, 8023,  171427, 17, }, // America/Indiana/Vincennes
    {  583, 8049,  171444, 16, }, // America/Indiana/Winamac
    {  583, 8073,  171460,  6, }, // America/Inuvik
    {  583,  660,  171466,  7, }, // America/Iqaluit
    {  583, 2665,  171473,  6, }, // America/Jamaica
    {  583, 5282,  171479,  4, }, // America/Juneau
    {  583,  521,  171483,  7, }, // America/Kentucky/Louisville
    {  583, 8088,  171490, 18, }, // America/Kentucky/Monticello
    {  583, 5621,  171508,  7, }, // America/Kralendijk
    {  583, 4278,  171515,  6, }, // America/La_Paz
    {  583, 7132,  171521,  4, }, // America/Lima
    {  583, 3390,  171525, 10, }, // America/Los_Angeles
    {  583, 5849,  171535, 19, }, // America/Lower_Princes
    {  583, 8116,  171554,  6, }, // America/Maceio
    {  583, 8131,  171560,  7, }, // America/Managua
    {  583, 1892,  171567,  6, }, // America/Manaus
    {  583, 5814,  171573,  6, }, // America/Marigot
    {  583, 5722,  171579,  8, }, // America/Martinique
    {  583, 8147,  171587,  9, }, // America/Matamoros
    {  583, 2779,  171596,  6, }, // America/Mazatlan
    {  583, 8165,  171602,  7, }, // America/Menominee
    {  583, 8183,  171609,  5, }, // America/Merida
    {  583, 8198,  171614,  8, }, // America/Metlakatla
    {  583, 2811,  171622, 14, }, // America/Mexico_City
    {  583, 4293,  171636,  6, }, // America/Miquelon
    {  583, 8217,  171642,  7, }, // America/Moncton
    {  583, 8233,  171649,  7, }, // America/Monterrey
    {  583, 4000,  171656, 10, }, // America/Montevideo
    {  583, 5741,  171666,  9, }, // America/Montserrat
    {  583, 5383,  171675,  5, }, // America/Nassau
    {  583, 3294,  171680,  8, }, // America/New_York
    {  583, 8251,  171603,  3, }, // America/Nome
    {  583, 1834,  171688,  7, }, // America/Noronha
    {  583, 8264,  171695, 21, }, // America/North_Dakota/Beulah
    {  583, 8292,  171716, 21, }, // America/North_Dakota/Center
    {  583, 8320,  171737, 26, }, // America/North_Dakota/New_Salem
    {  583,  393,  171346,  3, }, // America/Nuuk
    {  583, 8351,  171763,  7, }, // America/Ojinaga
    {  583, 5413,  171770,  4, }, // America/Panama
    {  583, 6106,  171774, 10, }, // America/Paramaribo
    {  583, 3224,  171784,  6, }, // America/Phoenix
    {  583, 3847,  171790, 13, }, // America/Port-au-Prince
    {  583, 5871,  171803, 12, }, // America/Port_of_Spain
    {  583, 8367,  171815, 11, }, // America/Porto_Velho
    {  583, 5760,  171826, 10, }, // America/Puerto_Rico
    {  583, 3932,  171836, 11, }, // America/Punta_Arenas
    {  583, 8387,  171847, 11, }, // America/Rankin_Inlet
    {  583, 8408,  171858,  6, }, // America/Recife
    {  583, 1979,  171864,  6, }, // America/Regina
    {  583, 8423,  171870,  7, }, // America/Resolute
    {  583,  695,  171877,  9, }, // America/Rio_Branco
    {  583, 8440,  171886,  7, }, // America/Santarem
    {  583, 2165,  171893,  8, }, // America/Santiago
    {  583, 6044,  171901, 12, }, // America/Santo_Domingo
    {  583, 1862,  171913,  9, }, // America/Sao_Paulo
    {  583, 6862,  171922, 10, }, // America/Scoresbysund
    {  583, 8457,  171932,  5, }, // America/Sitka
    {  583, 8471,  171937, 11, }, // America/St_Barthelemy
    {  583, 2045,  171948,  8, }, // America/St_Johns
    {  583, 5780,  171956,  8, }, // America/St_Kitts
    {  583, 5797,  171964,  9, }, // America/St_Lucia
    {  583,  855,  171973,  8, }, // America/St_Thomas
    {  583, 5830,  171981,  8, }, // America/St_Vincent
    {  583, 8493,  171989,  6, }, // America/Swift_Current
    {  583, 5363,  171995, 11, }, // America/Tegucigalpa
    {  583, 5673,  171943,  3, }, // America/Thule
    {  583,  313,  172006,  7, }, // America/Tijuana
    {  583,  608,  172013,  7, }, // America/Toronto
    {  583, 5605,  172020,  7, }, // America/Tortola
    {  583, 2077,  172027,  7, }, // America/Vancouver
    {  583, 2128,  172034, 11, }, // America/Whitehorse
    {  583,  734,  172045,  6, }, // America/Winnipeg
    {  583, 8515,  172051,  6, }, // America/Yakutat
    {  583, 5918,  172057,  5, }, // Antarctica/Casey
    {  583, 6027,  172062,  4, }, // Antarctica/Davis
    {  583, 6066,  172066, 14, }, // Antarctica/DumontDUrville
    {  583, 8531,  172080,  7, }, // Antarctica/Macquarie
    {  583, 7029,  172087,  6, }, // Antarctica/Mawson
    {  583, 7076,  172093,  7, }, // Antarctica/McMurdo
    {  583, 8552,  172100,  6, }, // Antarctica/Palmer
    {  583, 7189,  172106,  6, }, // Antarctica/Rothera
    {  583, 7258,  172112,  4, }, // Antarctica/Syowa
    {  583, 8570,  172116,  4, }, // Antarctica/Troll
    {  583, 7337,  172120,  6, }, // Antarctica/Vostok
    {  583, 1413,  172126, 11, }, // Arctic/Longyearbyen
    {  583, 5487,  172137,  3, }, // Asia/Aden
    {  583, 5297,  172140,  6, }, // Asia/Almaty
    {  583, 3870,  172146,  4, }, // Asia/Amman
    {  583, 5428,  172150,  6, }, // Asia/Anadyr
    {  583, 5440,  172156,  5, }, // Asia/Aqtau
    {  583, 5270,  172161,  5, }, // Asia/Aqtobe
    {  583,  964,  172166,  7, }, // Asia/Ashgabat
    {  583, 8587,  172173,  5, }, // Asia/Atyrau
    {  583, 3509,  172178,  4, }, // Asia/Baghdad
    {  583, 5451,  172182,  5, }, // Asia/Bahrain
    {  583, 3555,  171151,  4, }, // Asia/Baku
    {  583, 4368,  172187,  6, }, // Asia/Bangkok
    {  583, 3473,  172193,  7, }, // Asia/Barnaul
    {  583, 3988,  172200,  5, }, // Asia/Beirut
    {  583, 3680,  172205,  6, }, // Asia/Bishkek
    {  583, 5906,  172211,  6, }, // Asia/Brunei
    {  583, 4503,  172217,  5, }, // Asia/Chita
    {  583, 5963,  172222,  7, }, // Asia/Choibalsan
    {  583, 4413,  172229,  7, }, // Asia/Colombo
    {  583, 4442,  172236,  6, }, // Asia/Damascus
    {  583, 1060,  172242,  3, }, // Asia/Dhaka
    {  583, 6125,  172245,  4, }, // Asia/Dili
    {  583, 3498,  172249,  5, }, // Asia/Dubai
    {  583, 6092,  172254,  7, }, // Asia/Dushanbe
    {  583, 8599,  172261,  9, }, // Asia/Famagusta
    {  583, 8614,  172270,  3, }, // Asia/Gaza
    {  583, 4697,  172273,  5, }, // Asia/Hebron
    {  583, 1226,  172278, 17, }, // Asia/Ho_Chi_Minh
    {  583, 2576,  172295,  9, }, // Asia/Hong_Kong
    {  583, 4673,  172304,  4, }, // Asia/Hovd
    {  583, 4086,  172308,  8, }, // Asia/Irkutsk
    {  583, 6969,  172316,  7, }, // Asia/Jakarta
    {  583, 6955,  172323,  7, }, // Asia/Jayapura
    {  583, 1257,  172330,  6, }, // Asia/Jerusalem
    {  583, 3462,  172336,  5, }, // Asia/Kabul
    {  583, 4218,  172341,  9, }, // Asia/Kamchatka
    {  583, 4141,  172350,  5, }, // Asia/Karachi
    {  583, 1152,  172355,  8, }, // Asia/Kathmandu
    {  583, 8624,  172363,  7, }, // Asia/Khandyga
    {  583,  992,  172370,  6, }, // Asia/Kolkata
    {  583, 4099,  172376, 10, }, // Asia/Krasnoyarsk
    {  583, 4761,  172386, 12, }, // Asia/Kuala_Lumpur
    {  583, 5893,  172398,  6, }, // Asia/Kuching
    {  583, 5464,  172404,  4, }, // Asia/Kuwait
    {  583, 1178,  172080,  5, }, // Asia/Macau
    {  583, 3919,  172408,  7, }, // Asia/Magadan
    {  583, 1316,  172415,  7, }, // Asia/Makassar
    {  583, 7145,  172422,  6, }, // Asia/Manila
    {  583, 6883,  172428,  4, }, // Asia/Muscat
    {  583, 2446,  172432,  7, }, // Asia/Nicosia
    {  583, 8638,  172439, 12, }, // Asia/Novokuznetsk
    {  583, 4037,  172451, 10, }, // Asia/Novosibirsk
    {  583, 4131,  172461,  5, }, // Asia/Omsk
    {  583, 7108,  172466,  5, }, // Asia/Oral
    {  583, 6924,  172471,  7, }, // Asia/Phnom_Penh
    {  583, 8656,  172478,  9, }, // Asia/Pontianak
    {  583, 4116,  172487,  8, }, // Asia/Pyongyang
    {  583, 5476,  172495,  3, }, // Asia/Qatar
    {  583, 8671,  172498,  8, }, // Asia/Qostanay
    {  583, 4171,  172506,  9, }, // Asia/Qyzylorda
    {  583, 3486,  172515,  4, }, // Asia/Riyadh
    {  583, 4310,  172519,  7, }, // Asia/Sakhalin
    {  583, 7208,  172526,  6, }, // Asia/Samarkand
    {  583, 3116,  172532,  4, }, // Asia/Seoul
    {  583, 1020,  172536,  7, }, // Asia/Shanghai
    {  583, 3137,  172543,  8, }, // Asia/Singapore
    {  583, 4199,  172551, 13, }, // Asia/Srednekolymsk
    {  583, 3100,  172564,  6, }, // Asia/Taipei
    {  583, 4683,  172570,  6, }, // Asia/Tashkent
    {  583, 3817,  172576,  5, }, // Asia/Tbilisi
    {  583, 2638,  172581,  4, }, // Asia/Tehran
    {  583, 1284,  172585,  5, }, // Asia/Thimphu
    {  583, 2687,  172590,  5, }, // Asia/Tokyo
    {  583, 4474,  172595,  5, }, // Asia/Tomsk
    {  583, 1346,  172600, 10, }, // Asia/Ulaanbaatar
    {  583, 1126,  172610,  7, }, // Asia/Urumqi
    {  583, 8685,  172617,  9, }, // Asia/Ust-Nera
    {  583, 6940,  172626,  8, }, // Asia/Vientiane
    {  583, 4612,  172634, 11, }, // Asia/Vladivostok
    {  583, 4730,  172645,  7, }, // Asia/Yakutsk
    {  583, 1202,  172652,  5, }, // Asia/Yangon
    {  583, 3785,  172657, 11, }, // Asia/Yekaterinburg
    {  583, 3649,  172668,  5, }, // Asia/Yerevan
    {  583, 3565,  172673, 11, }, // Atlantic/Azores
    {  583, 5588,  172684,  8, }, // Atlantic/Bermuda
    {  583, 6591,  172692,  5, }, // Atlantic/Canary
    {  583, 3629,  172697,  9, }, // Atlantic/Cape_Verde
    {  583, 1379,  172706,  4, }, // Atlantic/Faroe
    {  583, 8699,  172710,  6, }, // Atlantic/Madeira
    {  583, 2614,  172716,  7, }, // Atlantic/Reykjavik
    {  583, 7235,  172723, 11, }, // Atlantic/South_Georgia
    {  583, 6784,  172734, 11, }, // Atlantic/St_Helena
    {  583, 6607,  172745,  6, }, // Atlantic/Stanley
    {  583, 1654,  172244,  6, }, // Australia/Adelaide
    {  583, 1619,  172751,  7, }, // Australia/Brisbane
    {  583, 1783,  172758, 10, }, // Australia/Broken_Hill
    {  583, 1567,  172768,  5, }, // Australia/Darwin
    {  583, 3539,  172773,  5, }, // Australia/Eucla
    {  583, 1500,  172778,  5, }, // Australia/Hobart
    {  583, 8716,  172783,  7, }, // Australia/Lindeman
    {  583, 1531,  172790,  8, }, // Australia/Lord_Howe
    {  583, 1711,  172798,  7, }, // Australia/Melbourne
    {  583, 1746,  172805,  4, }, // Australia/Perth
    {  583, 1447,  172809,  5, }, // Australia/Sydney
    {  583, 6390,  172814,  7, }, // Europe/Amsterdam
    {  583, 6149,  172821,  6, }, // Europe/Andorra
    {  583, 3522,  172827,  7, }, // Europe/Astrakhan
    {  583, 6577,  172834,  5, }, // Europe/Athens
    {  583, 6285,  172839,  5, }, // Europe/Belgrade
    {  583, 4659,  172844,  5, }, // Europe/Berlin
    {  583, 6437,  172849,  9, }, // Europe/Bratislava
    {  583, 6178,  172858,  6, }, // Europe/Brussels
    {  583, 3830,  172864,  7, }, // Europe/Bucharest
    {  583, 3708,  172871,  6, }, // Europe/Budapest
    {  583, 8751,  172877,  7, }, // Europe/Busingen
    {  583, 2475,  172884,  7, }, // Europe/Chisinau
    {  583, 6238,  172891,  8, }, // Europe/Copenhagen
    {  583, 2260,  172899,  4, }, // Europe/Dublin
    {  583, 6256,  172903,  6, }, // Europe/Gibraltar
    {  583, 8767,  172909,  6, }, // Europe/Guernsey
    {  583, 6561,  172915,  6, }, // Europe/Helsinki
    {  583, 8783,  172921, 10, }, // Europe/Isle_of_Man
    {  583, 1097,  172931,  7, }, // Europe/Istanbul
    {  583, 8802,  172938,  5, }, // Europe/Jersey
    {  583, 3881,  172943, 11, }, // Europe/Kaliningrad
    {  583, 8924,  172954,  5, }, // Europe/Kirov
    {  583, 2419,  172959,  4, }, // Europe/Kyiv
    {  583, 3078,  172963,  6, }, // Europe/Lisbon
    {  583, 6455,  172969,  9, }, // Europe/Ljubljana
    {  583, 2393,  172978,  6, }, // Europe/London
    {  583, 6314,  172984,  9, }, // Europe/Luxembourg
    {  583, 4794,  172993,  5, }, // Europe/Madrid
    {  583, 6346,  172998,  5, }, // Europe/Malta
    {  583, 6531,  173003,  7, }, // Europe/Mariehamn
    {  583, 3595,  173010,  5, }, // Europe/Minsk
    {  583, 6359,  173015,  6, }, // Europe/Monaco
    {  583, 3443,  173021,  5, }, // Europe/Moscow
    {  583, 6407,  173026,  5, }, // Europe/Oslo
    {  583, 4186,  173031,  5, }, // Europe/Paris
    {  583, 6373,  173036,  9, }, // Europe/Podgorica
    {  583, 6224,  173045,  4, }, // Europe/Prague
    {  583, 8816,  173049,  4, }, // Europe/Riga
    {  583, 6273,  173053,  4, }, // Europe/Rome
    {  583, 4233,  170542,  5, }, // Europe/Samara
    {  583, 6419,  173057,  8, }, // Europe/San_Marino
    {  583, 6194,  173065,  7, }, // Europe/Sarajevo
    {  583, 4353,  173072,  6, }, // Europe/Saratov
    {  583, 8828,  173078,  7, }, // Europe/Simferopol
    {  583, 6332,  173085,  6, }, // Europe/Skopje
    {  583, 6548,  173091,  5, }, // Europe/Sofia
    {  583, 6472,  173096,  8, }, // Europe/Stockholm
    {  583, 8846,  173104,  5, }, // Europe/Tallinn
    {  583, 6135,  173109,  5, }, // Europe/Tirane
    {  583, 8861,  173114,  9, }, // Europe/Ulyanovsk
    {  583, 6301,  173123,  4, }, // Europe/Vaduz
    {  583, 6516,  173127,  7, }, // Europe/Vatican
    {  583, 6164,  173134,  5, }, // Europe/Vienna
    {  583, 8878,  173139,  5, }, // Europe/Vilnius
    {  583, 4629,  173144,  9, }, // Europe/Volgograd
    {  583, 3055,  173153,  5, }, // Europe/Warsaw
    {  583, 6210,  173158,  5, }, // Europe/Zagreb
    {  583, 6489,  173163,  5, }, // Europe/Zurich
    {  583, 4980,  173168, 11, }, // Indian/Antananarivo
    {  583, 6910,  173179,  5, }, // Indian/Chagos
    {  583, 5979,  173184,  7, }, // Indian/Christmas
    {  583, 5996,  173191,  5, }, // Indian/Cocos
    {  583, 4931,  173196,  6, }, // Indian/Comoro
    {  583, 6624,  173202,  8, }, // Indian/Kerguelen
    {  583, 7223,  173210,  4, }, // Indian/Mahe
    {  583, 7013,  173214, 15, }, // Indian/Maldives
    {  583, 3971,  173229,  7, }, // Indian/Mauritius
    {  583, 5000,  173236,  5, }, // Indian/Mayotte
    {  583, 7174,  173241,  7, }, // Indian/Reunion
    {  583, 4324,  173248,  4, }, // Pacific/Apia
    {  583,  932,  173252,  6, }, // Pacific/Auckland
    {  583, 3608,  173258,  9, }, // Pacific/Bougainville
    {  583, 2863,  173267,  5, }, // Pacific/Chatham
    {  583, 3022,  173272,  4, }, // Pacific/Chuuk
    {  583, 2201,  173276,  4, }, // Pacific/Easter
    {  583, 7323,  173280,  4, }, // Pacific/Efate
    {  583, 7290,  173284,  7, }, // Pacific/Fakaofo
    {  583, 3804,  173291,  4, }, // Pacific/Fiji
    {  583, 7306,  173295,  8, }, // Pacific/Funafuti
    {  583, 6641,  173303,  8, }, // Pacific/Galapagos
    {  583, 6659,  173311,  5, }, // Pacific/Gambier
    {  583, 3724,  173316, 11, }, // Pacific/Guadalcanal
    {  583, 5935,  171564,  4, }, // Pacific/Guam
    {  583, 2897,  173327,  6, }, // Pacific/Kanton
    {  583, 3900,  173333,  8, }, // Pacific/Kiritimati
    {  583, 6982,  173341,  6, }, // Pacific/Kosrae
    {  583, 2708,  173347,  8, }, // Pacific/Kwajalein
    {  583, 4779,  173355,  6, }, // Pacific/Majuro
    {  583, 3953,  173361,  9, }, // Pacific/Marquesas
    {  583, 8909,  173370,  6, }, // Pacific/Midway
    {  583, 7047,  173376,  5, }, // Pacific/Nauru
    {  583, 7095,  173381,  4, }, // Pacific/Niue
    {  583, 4070,  173385,  6, }, // Pacific/Norfolk
    {  583, 7061,  173391,  5, }, // Pacific/Noumea
    {  583, 2991,  173396,  9, }, // Pacific/Pago_Pago
    {  583, 7118,  173405,  4, }, // Pacific/Palau
    {  583, 7157,  173409,  8, }, // Pacific/Pitcairn
    {  583, 2961,  173417,  7, }, // Pacific/Pohnpei
    {  583, 4709,  173424, 13, }, // Pacific/Port_Moresby
    {  583, 6009,  173437,  9, }, // Pacific/Rarotonga
    {  583, 5948,  173446,  6, }, // Pacific/Saipan
    {  583, 7275,  173452,  6, }, // Pacific/Tahiti
    {  583, 6675,  173458,  6, }, // Pacific/Tarawa
    {  583, 4485,  173464,  9, }, // Pacific/Tongatapu
    {  583, 7355,  173473,  4, }, // Pacific/Wake
    {  583, 7368,  173477,  5, }, // Pacific/Wallis
    {  585,   14,       0,  6, }, // Africa/Asmara Tachelhit/Tifinagh/Morocco
    {  585, 4337,       6,  8, }, // Africa/Sao_Tome
    {  585, 4154,      14,  8, }, // America/Asuncion
    {  585,  237,      22,  8, }, // America/Atikokan
    {  585, 7586,      30, 17, }, // America/Bahia_Banderas
    {  585, 3770,      47,  6, }, // America/Cancun
    {  585, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  585, 5640,      66,  7, }, // America/Curacao
    {  585,  481,      73, 13, }, // America/Indiana/Knox
    {  585, 7924,      86, 16, }, // America/Indiana/Marengo
    {  585, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  585, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  585, 8001,     139, 14, }, // America/Indiana/Vevay
    {  585, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  585, 8049,     171, 16, }, // America/Indiana/Winamac
    {  585, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  585, 5849,     207, 22, }, // America/Lower_Princes
    {  585, 8183,     229,  6, }, // America/Merida
    {  585, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  585, 8292,     255, 20, }, // America/North_Dakota/Center
    {  585, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  585,  393,     298,  4, }, // America/Nuuk
    {  585, 6862,     302, 16, }, // America/Scoresbysund
    {  585, 8471,     318, 14, }, // America/St_Barthelemy
    {  585, 2045,     332, 10, }, // America/St_Johns
    {  585, 5780,     342,  9, }, // America/St_Kitts
    {  585, 5797,     351,  9, }, // America/St_Lucia
    {  585,  855,     360, 10, }, // America/St_Thomas
    {  585, 5830,     370, 11, }, // America/St_Vincent
    {  585, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  585, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  585, 1152,     413,  9, }, // Asia/Kathmandu
    {  585,  992,     422,  7, }, // Asia/Kolkata
    {  585, 1178,     429,  5, }, // Asia/Macau
    {  585, 1202,     442,  6, }, // Asia/Yangon
    {  585, 1379,     448,  5, }, // Atlantic/Faroe
    {  585, 6784,     453, 10, }, // Atlantic/St_Helena
    {  585, 2419,     463,  4, }, // Europe/Kyiv
    {  585, 7174,     467,  7, }, // Indian/Reunion
    {  585, 3022,     474,  5, }, // Pacific/Chuuk
    {  585, 2961,     479,  7, }, // Pacific/Pohnpei
    {  587,   14,       0,  6, }, // Africa/Asmara Tai Dam/Tai Viet/Vietnam
    {  587, 4337,       6,  8, }, // Africa/Sao_Tome
    {  587, 4154,      14,  8, }, // America/Asuncion
    {  587,  237,      22,  8, }, // America/Atikokan
    {  587, 7586,      30, 17, }, // America/Bahia_Banderas
    {  587, 3770,      47,  6, }, // America/Cancun
    {  587, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  587, 5640,      66,  7, }, // America/Curacao
    {  587,  481,      73, 13, }, // America/Indiana/Knox
    {  587, 7924,      86, 16, }, // America/Indiana/Marengo
    {  587, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  587, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  587, 8001,     139, 14, }, // America/Indiana/Vevay
    {  587, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  587, 8049,     171, 16, }, // America/Indiana/Winamac
    {  587, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  587, 5849,     207, 22, }, // America/Lower_Princes
    {  587, 8183,     229,  6, }, // America/Merida
    {  587, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  587, 8292,     255, 20, }, // America/North_Dakota/Center
    {  587, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  587,  393,     298,  4, }, // America/Nuuk
    {  587, 6862,     302, 16, }, // America/Scoresbysund
    {  587, 8471,     318, 14, }, // America/St_Barthelemy
    {  587, 2045,     332, 10, }, // America/St_Johns
    {  587, 5780,     342,  9, }, // America/St_Kitts
    {  587, 5797,     351,  9, }, // America/St_Lucia
    {  587,  855,     360, 10, }, // America/St_Thomas
    {  587, 5830,     370, 11, }, // America/St_Vincent
    {  587, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  587, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  587, 1152,     413,  9, }, // Asia/Kathmandu
    {  587,  992,     422,  7, }, // Asia/Kolkata
    {  587, 1178,     429,  5, }, // Asia/Macau
    {  587, 1202,     442,  6, }, // Asia/Yangon
    {  587, 1379,     448,  5, }, // Atlantic/Faroe
    {  587, 6784,     453, 10, }, // Atlantic/St_Helena
    {  587, 2419,     463,  4, }, // Europe/Kyiv
    {  587, 7174,     467,  7, }, // Indian/Reunion
    {  587, 3022,     474,  5, }, // Pacific/Chuuk
    {  587, 2961,     479,  7, }, // Pacific/Pohnpei
    {  588,   14,       0,  6, }, // Africa/Asmara Taita/Latin/Kenya
    {  588, 4337,       6,  8, }, // Africa/Sao_Tome
    {  588, 4154,      14,  8, }, // America/Asuncion
    {  588,  237,      22,  8, }, // America/Atikokan
    {  588, 7586,      30, 17, }, // America/Bahia_Banderas
    {  588, 3770,      47,  6, }, // America/Cancun
    {  588, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  588, 5640,      66,  7, }, // America/Curacao
    {  588,  481,      73, 13, }, // America/Indiana/Knox
    {  588, 7924,      86, 16, }, // America/Indiana/Marengo
    {  588, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  588, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  588, 8001,     139, 14, }, // America/Indiana/Vevay
    {  588, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  588, 8049,     171, 16, }, // America/Indiana/Winamac
    {  588, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  588, 5849,     207, 22, }, // America/Lower_Princes
    {  588, 8183,     229,  6, }, // America/Merida
    {  588, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  588, 8292,     255, 20, }, // America/North_Dakota/Center
    {  588, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  588,  393,     298,  4, }, // America/Nuuk
    {  588, 6862,     302, 16, }, // America/Scoresbysund
    {  588, 8471,     318, 14, }, // America/St_Barthelemy
    {  588, 2045,     332, 10, }, // America/St_Johns
    {  588, 5780,     342,  9, }, // America/St_Kitts
    {  588, 5797,     351,  9, }, // America/St_Lucia
    {  588,  855,     360, 10, }, // America/St_Thomas
    {  588, 5830,     370, 11, }, // America/St_Vincent
    {  588, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  588, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  588, 1152,     413,  9, }, // Asia/Kathmandu
    {  588,  992,     422,  7, }, // Asia/Kolkata
    {  588, 1178,     429,  5, }, // Asia/Macau
    {  588, 1202,     442,  6, }, // Asia/Yangon
    {  588, 1379,     448,  5, }, // Atlantic/Faroe
    {  588, 6784,     453, 10, }, // Atlantic/St_Helena
    {  588, 2419,     463,  4, }, // Europe/Kyiv
    {  588, 7174,     467,  7, }, // Indian/Reunion
    {  588, 3022,     474,  5, }, // Pacific/Chuuk
    {  588, 2961,     479,  7, }, // Pacific/Pohnpei
    {  589,   14,       0,  6, }, // Africa/Asmara Tajik/Cyrillic/Tajikistan
    {  589, 4337,       6,  8, }, // Africa/Sao_Tome
    {  589, 4154,      14,  8, }, // America/Asuncion
    {  589,  237,      22,  8, }, // America/Atikokan
    {  589, 7586,      30, 17, }, // America/Bahia_Banderas
    {  589, 3770,      47,  6, }, // America/Cancun
    {  589, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  589, 5640,      66,  7, }, // America/Curacao
    {  589,  481,      73, 13, }, // America/Indiana/Knox
    {  589, 7924,      86, 16, }, // America/Indiana/Marengo
    {  589, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  589, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  589, 8001,     139, 14, }, // America/Indiana/Vevay
    {  589, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  589, 8049,     171, 16, }, // America/Indiana/Winamac
    {  589, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  589, 5849,     207, 22, }, // America/Lower_Princes
    {  589, 8183,     229,  6, }, // America/Merida
    {  589, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  589, 8292,     255, 20, }, // America/North_Dakota/Center
    {  589, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  589,  393,     298,  4, }, // America/Nuuk
    {  589, 6862,     302, 16, }, // America/Scoresbysund
    {  589, 8471,     318, 14, }, // America/St_Barthelemy
    {  589, 2045,     332, 10, }, // America/St_Johns
    {  589, 5780,     342,  9, }, // America/St_Kitts
    {  589, 5797,     351,  9, }, // America/St_Lucia
    {  589,  855,     360, 10, }, // America/St_Thomas
    {  589, 5830,     370, 11, }, // America/St_Vincent
    {  589, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  589, 6092,    2326,  7, }, // Asia/Dushanbe
    {  589, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  589, 1152,     413,  9, }, // Asia/Kathmandu
    {  589,  992,     422,  7, }, // Asia/Kolkata
    {  589, 1178,     429,  5, }, // Asia/Macau
    {  589, 1202,     442,  6, }, // Asia/Yangon
    {  589, 1379,     448,  5, }, // Atlantic/Faroe
    {  589, 6784,     453, 10, }, // Atlantic/St_Helena
    {  589, 2419,     463,  4, }, // Europe/Kyiv
    {  589, 7174,     467,  7, }, // Indian/Reunion
    {  589, 3022,     474,  5, }, // Pacific/Chuuk
    {  589, 2961,     479,  7, }, // Pacific/Pohnpei
    {  590, 6751,  173482,  9, }, // Africa/Abidjan Tamil/Tamil/India
    {  590, 6723,  173491,  5, }, // Africa/Accra
    {  590, 4961,  173496, 11, }, // Africa/Addis_Ababa
    {  590, 7383,  173507, 10, }, // Africa/Algiers
    {  590,   14,  173517,  7, }, // Africa/Asmara
    {  590,   44,  173524,  5, }, // Africa/Bamako
    {  590, 5159,  173529,  8, }, // Africa/Bangui
    {  590, 6709,  173537,  7, }, // Africa/Banjul
    {  590, 7398,  173544,  6, }, // Africa/Bissau
    {  590, 4859,  173550, 11, }, // Africa/Blantyre
    {  590, 5189,  173561, 12, }, // Africa/Brazzaville
    {  590, 4824,  173573, 10, }, // Africa/Bujumbura
    {  590, 2242,  173583,  6, }, // Africa/Cairo
    {  590, 4019,  173589, 12, }, // Africa/Casablanca
    {  590, 7412,  173601,  8, }, // Africa/Ceuta
    {  590, 6736,  173609,  7, }, // Africa/Conakry
    {  590, 6803,  173616,  5, }, // Africa/Dakar
    {  590, 5032,  173621, 14, }, // Africa/Dar_es_Salaam
    {  590, 4945,  173635,  8, }, // Africa/Djibouti
    {  590, 5145,  173643,  5, }, // Africa/Douala
    {  590, 5068,  173648,  9, }, // Africa/El_Aaiun
    {  590, 6816,  173657, 10, }, // Africa/Freetown
    {  590, 4808,  173667,  7, }, // Africa/Gaborone
    {  590, 4917,  173674,  5, }, // Africa/Harare
    {  590, 4381,  173679, 14, }, // Africa/Johannesburg
    {  590, 4401,  173693,  4, }, // Africa/Juba
    {  590, 5053,  173697,  7, }, // Africa/Kampala
    {  590, 4426,  173704,  8, }, // Africa/Khartoum
    {  590, 4875,  173712,  5, }, // Africa/Kigali
    {  590, 5208,  173717,  7, }, // Africa/Kinshasa
    {  590, 4646,  173724,  6, }, // Africa/Lagos
    {  590, 5238,  173730, 12, }, // Africa/Libreville
    {  590, 6832,  173742,  4, }, // Africa/Lome
    {  590, 5113,  173746,  8, }, // Africa/Luanda
    {  590, 4841,  173754, 10, }, // Africa/Lubumbashi
    {  590, 4903,  173764,  6, }, // Africa/Lusaka
    {  590, 5224,  173770,  5, }, // Africa/Malabo
    {  590, 4889,  173775,  5, }, // Africa/Maputo
    {  590, 5099,  173780,  5, }, // Africa/Maseru
    {  590, 5084,  173785,  5, }, // Africa/Mbabane
    {  590, 5015,  173790,  8, }, // Africa/Mogadishu
    {  590, 6997,  173798, 10, }, // Africa/Monrovia
    {  590, 3755,  173808,  6, }, // Africa/Nairobi
    {  590, 5173,  173814,  5, }, // Africa/Ndjamena
    {  590, 5256,  173819,  6, }, // Africa/Niamey
    {  590, 6766,  173825, 10, }, // Africa/Nouakchott
    {  590, 6690,  173835,  9, }, // Africa/Ouagadougou
    {  590, 5127,  173844, 13, }, // Africa/Porto-Novo
    {  590, 4337,  173857,  9, }, // Africa/Sao_Tome
    {  590, 2732,  173866,  8, }, // Africa/Tripoli
    {  590, 6503,  173874,  6, }, // Africa/Tunis
    {  590, 4054,  173880, 10, }, // Africa/Windhoek
    {  590,  132,  173890,  4, }, // America/Adak
    {  590, 3183,  173894,  9, }, // America/Anchorage
    {  590, 5524,  173903,  9, }, // America/Anguilla
    {  590, 5541,  173912,  9, }, // America/Antigua
    {  590, 4456,  173921, 10, }, // America/Araguaina
    {  590,  166,  173931, 14, }, // America/Argentina/Buenos_Aires
    {  590,   91,  173945,  8, }, // America/Argentina/Catamarca
    {  590,  270,  173953,  8, }, // America/Argentina/Cordoba
    {  590,  441,  173961,  6, }, // America/Argentina/Jujuy
    {  590, 7425,  173967,  9, }, // America/Argentina/La_Rioja
    {  590,  565,  173976,  8, }, // America/Argentina/Mendoza
    {  590, 7452,  173984, 13, }, // America/Argentina/Rio_Gallegos
    {  590, 7483,  173997,  6, }, // America/Argentina/Salta
    {  590, 7507,  174003, 11, }, // America/Argentina/San_Juan
    {  590, 5497,  174014, 11, }, // America/Argentina/San_Luis
    {  590, 7534,  174025,  7, }, // America/Argentina/Tucuman
    {  590, 7560,  174032,  7, }, // America/Argentina/Ushuaia
    {  590, 5557,  174039,  5, }, // America/Aruba
    {  590, 4154,  174044,  9, }, // America/Asuncion
    {  590,  237,  174053,  8, }, // America/Atikokan
    {  590, 3581,  174061,  6, }, // America/Bahia
    {  590, 7586,  174067, 15, }, // America/Bahia_Banderas
    {  590, 5571,  174082,  9, }, // America/Barbados
    {  590, 7609,  174091,  6, }, // America/Belem
    {  590, 5309,  174097,  6, }, // America/Belize
    {  590, 7623,  174103, 16, }, // America/Blanc-Sablon
    {  590, 7644,  174119, 11, }, // America/Boa_Vista
    {  590, 4263,  174130,  6, }, // America/Bogota
    {  590, 7662,  174136,  6, }, // America/Boise
    {  590, 7676,  174142, 20, }, // America/Cambridge_Bay
    {  590, 7698,  174162, 15, }, // America/Campo_Grande
    {  590, 3770,  174177,  7, }, // America/Cancun
    {  590, 4596,  174184,  6, }, // America/Caracas
    {  590, 4247,  174190,  6, }, // America/Cayenne
    {  590, 5398,  174196,  5, }, // America/Cayman
    {  590, 3251,  174201,  6, }, // America/Chicago
    {  590, 7719,  174207,  6, }, // America/Chihuahua
    {  590, 7737,  174213, 14, }, // America/Ciudad_Juarez
    {  590, 5324,  174227, 11, }, // America/Costa_Rica
    {  590, 7759,  174238, 10, }, // America/Creston
    {  590, 3693,  174248,  6, }, // America/Cuiaba
    {  590, 5640,  174254,  8, }, // America/Curacao
    {  590, 7775,  174262, 16, }, // America/Danmarkshavn
    {  590, 7796,  174278,  7, }, // America/Dawson
    {  590, 7811,  174285, 14, }, // America/Dawson_Creek
    {  590,  805,  174299,  7, }, // America/Denver
    {  590, 3351,  174306, 10, }, // America/Detroit
    {  590, 5656,  174316,  8, }, // America/Dominica
    {  590,  893,  174324, 11, }, // America/Edmonton
    {  590, 7832,  174335,  7, }, // America/Eirunepe
    {  590, 5343,  174342, 13, }, // America/El_Salvador
    {  590, 7849,  174355, 15, }, // America/Fort_Nelson
    {  590, 7869,  174370, 11, }, // America/Fortaleza
    {  590, 7887,  174381, 14, }, // America/Glace_Bay
    {  590, 6844,  174395,  7, }, // America/Goose_Bay
    {  590, 4514,  174402, 14, }, // America/Grand_Turk
    {  590, 5687,  174416,  6, }, // America/Grenada
    {  590, 5703,  174422,  9, }, // America/Guadeloupe
    {  590, 3662,  174431,  8, }, // America/Guatemala
    {  590, 4743,  174439,  9, }, // America/Guayaquil
    {  590, 6895,  174448,  5, }, // America/Guyana
    {  590, 1923,  174453, 10, }, // America/Halifax
    {  590, 2221,  174463,  5, }, // America/Havana
    {  590, 7905,  174468, 10, }, // America/Hermosillo
    {  590,  348,  174478, 15, }, // America/Indiana/Indianapolis
    {  590,  481,  174493, 17, }, // America/Indiana/Knox
    {  590, 7924,  174510, 18, }, // America/Indiana/Marengo
    {  590, 7948,  174528, 26, }, // America/Indiana/Petersburg
    {  590, 7975,  174554, 22, }, // America/Indiana/Tell_City
    {  590, 8001,  174576, 15, }, // America/Indiana/Vevay
    {  590, 8023,  174591, 21, }, // America/Indiana/Vincennes
    {  590, 8049,  174612, 19, }, // America/Indiana/Winamac
    {  590, 8073,  174631,  7, }, // America/Inuvik
    {  590,  660,  174638,  9, }, // America/Iqaluit
    {  590, 2665,  174647,  7, }, // America/Jamaica
    {  590, 5282,  174654,  6, }, // America/Juneau
    {  590,  521,  174660, 12, }, // America/Kentucky/Louisville
    {  590, 8088,  174672, 21, }, // America/Kentucky/Monticello
    {  590, 5621,  174693, 11, }, // America/Kralendijk
    {  590, 4278,  174704,  7, }, // America/La_Paz
    {  590, 7132,  174711,  4, }, // America/Lima
    {  590, 3390,  174715, 13, }, // America/Los_Angeles
    {  590, 5849,  174728, 27, }, // America/Lower_Princes
    {  590, 8116,  174755,  6, }, // America/Maceio
    {  590, 8131,  174761,  8, }, // America/Managua
    {  590, 1892,  174769,  5, }, // America/Manaus
    {  590, 5814,  174774,  8, }, // America/Marigot
    {  590, 5722,  174782, 12, }, // America/Martinique
    {  590, 8147,  174794,  8, }, // America/Matamoros
    {  590, 2779,  174802,  7, }, // America/Mazatlan
    {  590, 8165,  174809,  8, }, // America/Menominee
    {  590, 8183,  174817,  6, }, // America/Merida
    {  590, 8198,  174823, 10, }, // America/Metlakatla
    {  590, 2811,  174833, 14, }, // America/Mexico_City
    {  590, 4293,  174847,  9, }, // America/Miquelon
    {  590, 8217,  174856,  8, }, // America/Moncton
    {  590, 8233,  174864, 10, }, // America/Monterrey
    {  590, 4000,  174874, 12, }, // America/Montevideo
    {  590, 5741,  174886, 10, }, // America/Montserrat
    {  590, 5383,  174896,  4, }, // America/Nassau
    {  590, 3294,  174900, 10, }, // America/New_York
    {  590, 8251,  174910,  4, }, // America/Nome
    {  590, 1834,  174914,  7, }, // America/Noronha
    {  590, 8264,  174921, 22, }, // America/North_Dakota/Beulah
    {  590, 8292,  174943, 21, }, // America/North_Dakota/Center
    {  590, 8320,  174964, 24, }, // America/North_Dakota/New_Salem
    {  590,  393,  174988,  4, }, // America/Nuuk
    {  590, 8351,  174992,  6, }, // America/Ojinaga
    {  590, 5413,  174998,  5, }, // America/Panama
    {  590, 6106,  175003,  7, }, // America/Paramaribo
    {  590, 3224,  175010,  9, }, // America/Phoenix
    {  590, 3847,  175019, 19, }, // America/Port-au-Prince
    {  590, 5871,  175038, 20, }, // America/Port_of_Spain
    {  590, 8367,  175058, 15, }, // America/Porto_Velho
    {  590, 5760,  175073, 13, }, // America/Puerto_Rico
    {  590, 3932,  175086, 14, }, // America/Punta_Arenas
    {  590, 8387,  175100, 16, }, // America/Rankin_Inlet
    {  590, 8408,  175116,  7, }, // America/Recife
    {  590, 1979,  175123,  6, }, // America/Regina
    {  590, 8423,  175129,  7, }, // America/Resolute
    {  590,  695,  175136, 13, }, // America/Rio_Branco
    {  590, 8440,  175149,  9, }, // America/Santarem
    {  590, 2165,  175158, 10, }, // America/Santiago
    {  590, 6044,  175168, 15, }, // America/Santo_Domingo
    {  590, 1862,  175183, 10, }, // America/Sao_Paulo
    {  590, 6862,  175193, 15, }, // America/Scoresbysund
    {  590, 8457,  175208,  6, }, // America/Sitka
    {  590, 8471,  175214, 19, }, // America/St_Barthelemy
    {  590, 2045,  175233, 15, }, // America/St_Johns
    {  590, 5780,  175248, 15, }, // America/St_Kitts
    {  590, 5797,  175263, 15, }, // America/St_Lucia
    {  590,  855,  175278, 14, }, // America/St_Thomas
    {  590, 5830,  175292, 19, }, // America/St_Vincent
    {  590, 8493,  175311, 16, }, // America/Swift_Current
    {  590, 5363,  175327, 11, }, // America/Tegucigalpa
    {  590, 5673,  175338,  4, }, // America/Thule
    {  590,  313,  175342,  8, }, // America/Tijuana
    {  590,  608,  175350,  8, }, // America/Toronto
    {  590, 5605,  175358,  8, }, // America/Tortola
    {  590, 2077,  175366,  9, }, // America/Vancouver
    {  590, 2128,  175375, 12, }, // America/Whitehorse
    {  590,  734,  175387, 10, }, // America/Winnipeg
    {  590, 8515,  175397,  6, }, // America/Yakutat
    {  590, 5918,  175403,  4, }, // Antarctica/Casey
    {  590, 6027,  175407,  6, }, // Antarctica/Davis
    {  590, 6066,  175413, 21, }, // Antarctica/DumontDUrville
    {  590, 8531,  175434, 10, }, // Antarctica/Macquarie
    {  590, 7029,  175444,  5, }, // Antarctica/Mawson
    {  590, 7076,  175449,  9, }, // Antarctica/McMurdo
    {  590, 8552,  175458,  7, }, // Antarctica/Palmer
    {  590, 7189,  175465,  6, }, // Antarctica/Rothera
    {  590, 7258,  175471,  6, }, // Antarctica/Syowa
    {  590, 8570,  175477,  6, }, // Antarctica/Troll
    {  590, 7337,  175483,  8, }, // Antarctica/Vostok
    {  590, 1413,  175491, 13, }, // Arctic/Longyearbyen
    {  590, 5487,  175504,  4, }, // Asia/Aden
    {  590, 5297,  175508,  7, }, // Asia/Almaty
    {  590, 3870,  175515,  7, }, // Asia/Amman
    {  590, 5428,  175522,  6, }, // Asia/Anadyr
    {  590, 5440,  175528,  6, }, // Asia/Aqtau
    {  590, 5270,  175534,  7, }, // Asia/Aqtobe
    {  590,  964,  175541,  9, }, // Asia/Ashgabat
    {  590, 8587,  175550,  5, }, // Asia/Atyrau
    {  590, 3509,  175555,  8, }, // Asia/Baghdad
    {  590, 5451,  175563,  7, }, // Asia/Bahrain
    {  590, 3555,  175570,  6, }, // Asia/Baku
    {  590, 4368,  175576,  8, }, // Asia/Bangkok
    {  590, 3473,  175584,  8, }, // Asia/Barnaul
    {  590, 3988,  175592,  8, }, // Asia/Beirut
    {  590, 3680,  175600,  8, }, // Asia/Bishkek
    {  590, 5906,  175608,  6, }, // Asia/Brunei
    {  590, 4503,  175614,  4, }, // Asia/Chita
    {  590, 5963,  175618, 12, }, // Asia/Choibalsan
    {  590, 4413,  175630,  8, }, // Asia/Colombo
    {  590, 4442,  175638,  8, }, // Asia/Damascus
    {  590, 1060,  175646,  6, }, // Asia/Dhaka
    {  590, 6125,  175652,  4, }, // Asia/Dili
    {  590, 3498,  175656,  6, }, // Asia/Dubai
    {  590, 6092,  175662,  7, }, // Asia/Dushanbe
    {  590, 8599,  175669, 10, }, // Asia/Famagusta
    {  590, 8614,  173589,  4, }, // Asia/Gaza
    {  590, 4697,  175679,  8, }, // Asia/Hebron
    {  590, 1226,  175687, 17, }, // Asia/Ho_Chi_Minh
    {  590, 2576,  175704,  8, }, // Asia/Hong_Kong
    {  590, 4673,  175712,  6, }, // Asia/Hovd
    {  590, 4086,  175718, 11, }, // Asia/Irkutsk
    {  590, 6969,  175729,  9, }, // Asia/Jakarta
    {  590, 6955,  175738,  7, }, // Asia/Jayapura
    {  590, 1257,  175745,  9, }, // Asia/Jerusalem
    {  590, 3462,  175754,  6, }, // Asia/Kabul
    {  590, 4218,  175760,  9, }, // Asia/Kamchatka
    {  590, 4141,  175769,  7, }, // Asia/Karachi
    {  590, 1152,  175776, 10, }, // Asia/Kathmandu
    {  590, 8624,  175786,  8, }, // Asia/Khandyga
    {  590,  992,  175794,  9, }, // Asia/Kolkata
    {  590, 4099,  175803, 16, }, // Asia/Krasnoyarsk
    {  590, 4761,  175819, 11, }, // Asia/Kuala_Lumpur
    {  590, 5893,  175830,  6, }, // Asia/Kuching
    {  590, 5464,  175836,  6, }, // Asia/Kuwait
    {  590, 1178,  175842,  5, }, // Asia/Macau
    {  590, 3919,  175847,  5, }, // Asia/Magadan
    {  590, 1316,  175852,  7, }, // Asia/Makassar
    {  590, 7145,  175859,  5, }, // Asia/Manila
    {  590, 6883,  175864,  6, }, // Asia/Muscat
    {  590, 2446,  175870,  8, }, // Asia/Nicosia
    {  590, 8638,  175878, 16, }, // Asia/Novokuznetsk
    {  590, 4037,  175894, 14, }, // Asia/Novosibirsk
    {  590, 4131,  175908,  7, }, // Asia/Omsk
    {  590, 7108,  175915,  4, }, // Asia/Oral
    {  590, 6924,  175919, 12, }, // Asia/Phnom_Penh
    {  590, 8656,  175931, 12, }, // Asia/Pontianak
    {  590, 4116,  175943, 10, }, // Asia/Pyongyang
    {  590, 5476,  175953,  7, }, // Asia/Qatar
    {  590, 8671,  175960,  8, }, // Asia/Qostanay
    {  590, 4171,  175968, 10, }, // Asia/Qyzylorda
    {  590, 3486,  175978,  6, }, // Asia/Riyadh
    {  590, 4310,  175984,  6, }, // Asia/Sakhalin
    {  590, 7208,  175990, 10, }, // Asia/Samarkand
    {  590, 3116,  176000,  6, }, // Asia/Seoul
    {  590, 1020,  176006,  8, }, // Asia/Shanghai
    {  590, 3137,  176014, 11, }, // Asia/Singapore
    {  590, 4199,  176025, 18, }, // Asia/Srednekolymsk
    {  590, 3100,  176043,  6, }, // Asia/Taipei
    {  590, 4683,  176049,  9, }, // Asia/Tashkent
    {  590, 3817,  176058,  8, }, // Asia/Tbilisi
    {  590, 2638,  176066,  8, }, // Asia/Tehran
    {  590, 1284,  176074,  6, }, // Asia/Thimphu
    {  590, 2687,  176080,  8, }, // Asia/Tokyo
    {  590, 4474,  176088,  8, }, // Asia/Tomsk
    {  590, 1346,  176096, 12, }, // Asia/Ulaanbaatar
    {  590, 1126,  176108,  7, }, // Asia/Urumqi
    {  590, 8685,  176115, 10, }, // Asia/Ust-Nera
    {  590, 6940,  176125, 10, }, // Asia/Vientiane
    {  590, 4612,  176135, 14, }, // Asia/Vladivostok
    {  590, 4730,  176149,  9, }, // Asia/Yakutsk
    {  590, 1202,  176158,  7, }, // Asia/Yangon
    {  590, 3785,  176165, 15, }, // Asia/Yekaterinburg
    {  590, 3649,  176180,  6, }, // Asia/Yerevan
    {  590, 3565,  176186,  6, }, // Atlantic/Azores
    {  590, 5588,  176192,  8, }, // Atlantic/Bermuda
    {  590, 6591,  176200,  5, }, // Atlantic/Canary
    {  590, 3629,  176205, 11, }, // Atlantic/Cape_Verde
    {  590, 1379,  176216,  4, }, // Atlantic/Faroe
    {  590, 8699,  176220,  5, }, // Atlantic/Madeira
    {  590, 2614,  176225, 10, }, // Atlantic/Reykjavik
    {  590, 7235,  176235, 15, }, // Atlantic/South_Georgia
    {  590, 6784,  176250, 15, }, // Atlantic/St_Helena
    {  590, 6607,  176265,  8, }, // Atlantic/Stanley
    {  590, 1654,  176273,  9, }, // Australia/Adelaide
    {  590, 1619,  176282, 10, }, // Australia/Brisbane
    {  590, 1783,  176292, 14, }, // Australia/Broken_Hill
    {  590, 1567,  176306,  8, }, // Australia/Darwin
    {  590, 3539,  176314,  6, }, // Australia/Eucla
    {  590, 1500,  176320,  7, }, // Australia/Hobart
    {  590, 8716,  176327,  9, }, // Australia/Lindeman
    {  590, 1531,  176336, 11, }, // Australia/Lord_Howe
    {  590, 1711,  176347, 10, }, // Australia/Melbourne
    {  590, 1746,  176357,  6, }, // Australia/Perth
    {  590, 1447,  176363,  6, }, // Australia/Sydney
    {  590, 6390,  176369, 12, }, // Europe/Amsterdam
    {  590, 6149,  176381,  7, }, // Europe/Andorra
    {  590, 3522,  176388, 11, }, // Europe/Astrakhan
    {  590, 6577,  176399,  6, }, // Europe/Athens
    {  590, 6285,  176405, 10, }, // Europe/Belgrade
    {  590, 4659,  176415,  8, }, // Europe/Berlin
    {  590, 6437,  176423, 11, }, // Europe/Bratislava
    {  590, 6178,  176434,  9, }, // Europe/Brussels
    {  590, 3830,  176443, 10, }, // Europe/Bucharest
    {  590, 3708,  176453, 10, }, // Europe/Budapest
    {  590, 8751,  176463,  9, }, // Europe/Busingen
    {  590, 2475,  176472,  7, }, // Europe/Chisinau
    {  590, 6238,  176479, 10, }, // Europe/Copenhagen
    {  590, 2260,  176489,  7, }, // Europe/Dublin
    {  590, 6256,  176496, 11, }, // Europe/Gibraltar
    {  590, 8767,  176507,  6, }, // Europe/Guernsey
    {  590, 6561,  176513, 10, }, // Europe/Helsinki
    {  590, 8783,  176523, 13, }, // Europe/Isle_of_Man
    {  590, 1097,  176536, 11, }, // Europe/Istanbul
    {  590, 8802,  176547,  6, }, // Europe/Jersey
    {  590, 3881,  176553, 13, }, // Europe/Kaliningrad
    {  590, 8924,  176566,  6, }, // Europe/Kirov
    {  590, 2419,  176572,  4, }, // Europe/Kyiv
    {  590, 3078,  176576,  7, }, // Europe/Lisbon
    {  590, 6455,  176583,  8, }, // Europe/Ljubljana
    {  590, 2393,  176591,  6, }, // Europe/London
    {  590, 6314,  176597, 11, }, // Europe/Luxembourg
    {  590, 4794,  176608,  8, }, // Europe/Madrid
    {  590, 6346,  176616,  6, }, // Europe/Malta
    {  590, 6531,  176622,  6, }, // Europe/Mariehamn
    {  590, 3595,  176628,  8, }, // Europe/Minsk
    {  590, 6359,  176636,  8, }, // Europe/Monaco
    {  590, 3443,  176644,  6, }, // Europe/Moscow
    {  590, 6407,  176650,  5, }, // Europe/Oslo
    {  590, 4186,  176655,  6, }, // Europe/Paris
    {  590, 6373,  176661, 10, }, // Europe/Podgorica
    {  590, 6224,  176671,  6, }, // Europe/Prague
    {  590, 8816,  174234,  4, }, // Europe/Riga
    {  590, 6273,  176677,  4, }, // Europe/Rome
    {  590, 4233,  176681,  5, }, // Europe/Samara
    {  590, 6419,  176686, 10, }, // Europe/San_Marino
    {  590, 6194,  176696,  6, }, // Europe/Sarajevo
    {  590, 4353,  176702,  6, }, // Europe/Saratov
    {  590, 8828,  176708, 13, }, // Europe/Simferopol
    {  590, 6332,  176721,  8, }, // Europe/Skopje
    {  590, 6548,  176729,  7, }, // Europe/Sofia
    {  590, 6472,  176736, 10, }, // Europe/Stockholm
    {  590, 8846,  176746,  5, }, // Europe/Tallinn
    {  590, 6135,  176751,  6, }, // Europe/Tirane
    {  590, 8861,  176757, 11, }, // Europe/Ulyanovsk
    {  590, 6301,  176768,  5, }, // Europe/Vaduz
    {  590, 6516,  176773,  7, }, // Europe/Vatican
    {  590, 6164,  176780,  7, }, // Europe/Vienna
    {  590, 8878,  176787,  9, }, // Europe/Vilnius
    {  590, 4629,  176796, 12, }, // Europe/Volgograd
    {  590, 3055,  176808,  6, }, // Europe/Warsaw
    {  590, 6210,  176814,  7, }, // Europe/Zagreb
    {  590, 6489,  176821,  6, }, // Europe/Zurich
    {  590, 4980,  176827, 10, }, // Indian/Antananarivo
    {  590, 6910,  176837,  6, }, // Indian/Chagos
    {  590, 5979,  176843, 11, }, // Indian/Christmas
    {  590, 5996,  176854,  6, }, // Indian/Cocos
    {  590, 4931,  176860,  5, }, // Indian/Comoro
    {  590, 6624,  176865, 10, }, // Indian/Kerguelen
    {  590, 7223,  176875,  4, }, // Indian/Mahe
    {  590, 7013,  176879, 12, }, // Indian/Maldives
    {  590, 3971,  176891,  9, }, // Indian/Mauritius
    {  590, 5000,  176900,  7, }, // Indian/Mayotte
    {  590, 7174,  176907,  9, }, // Indian/Reunion
    {  590, 4324,  176916,  5, }, // Pacific/Apia
    {  590,  932,  176921,  9, }, // Pacific/Auckland
    {  590, 3608,  176930, 14, }, // Pacific/Bougainville
    {  590, 2863,  176944,  7, }, // Pacific/Chatham
    {  590, 3022,  176951,  4, }, // Pacific/Chuuk
    {  590, 2201,  176955,  6, }, // Pacific/Easter
    {  590, 7323,  176961,  6, }, // Pacific/Efate
    {  590, 7290,  176967,  7, }, // Pacific/Fakaofo
    {  590, 3804,  176974,  5, }, // Pacific/Fiji
    {  590, 7306,  176979, 10, }, // Pacific/Funafuti
    {  590, 6641,  176989,  7, }, // Pacific/Galapagos
    {  590, 6659,  176996,  9, }, // Pacific/Gambier
    {  590, 3724,  177005, 13, }, // Pacific/Guadalcanal
    {  590, 5935,  177018,  6, }, // Pacific/Guam
    {  590, 2929,  177024,  8, }, // Pacific/Honolulu
    {  590, 2897,  177032,  9, }, // Pacific/Kanton
    {  590, 3900,  177041, 12, }, // Pacific/Kiritimati
    {  590, 6982,  177053,  6, }, // Pacific/Kosrae
    {  590, 2708,  177059, 11, }, // Pacific/Kwajalein
    {  590, 4779,  177070,  5, }, // Pacific/Majuro
    {  590, 3953,  177075, 12, }, // Pacific/Marquesas
    {  590, 8909,  177087,  6, }, // Pacific/Midway
    {  590, 7047,  177093,  5, }, // Pacific/Nauru
    {  590, 7095,  177098,  4, }, // Pacific/Niue
    {  590, 4070,  177102,  9, }, // Pacific/Norfolk
    {  590, 7061,  177111,  8, }, // Pacific/Noumea
    {  590, 2991,  177119,  9, }, // Pacific/Pago_Pago
    {  590, 7118,  177128,  5, }, // Pacific/Palau
    {  590, 7157,  177133, 12, }, // Pacific/Pitcairn
    {  590, 2961,  177145,  9, }, // Pacific/Pohnpei
    {  590, 4709,  177154, 15, }, // Pacific/Port_Moresby
    {  590, 6009,  177169,  9, }, // Pacific/Rarotonga
    {  590, 5948,  177178,  5, }, // Pacific/Saipan
    {  590, 7275,  177183,  5, }, // Pacific/Tahiti
    {  590, 6675,  177188,  5, }, // Pacific/Tarawa
    {  590, 4485,  177193, 10, }, // Pacific/Tongatapu
    {  590, 7355,  177203,  4, }, // Pacific/Wake
    {  590, 7368,  177207,  6, }, // Pacific/Wallis
    {  594,   14,       0,  6, }, // Africa/Asmara Taroko/Latin/Taiwan
    {  594, 4337,       6,  8, }, // Africa/Sao_Tome
    {  594, 3183,  177213, 19, }, // America/Anchorage
    {  594, 4154,      14,  8, }, // America/Asuncion
    {  594,  237,      22,  8, }, // America/Atikokan
    {  594, 7586,      30, 17, }, // America/Bahia_Banderas
    {  594, 7698,  177232, 16, }, // America/Campo_Grande
    {  594, 3770,      47,  6, }, // America/Cancun
    {  594, 3251,  177248, 18, }, // America/Chicago
    {  594, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  594, 5640,      66,  7, }, // America/Curacao
    {  594,  805,  177266, 17, }, // America/Denver
    {  594,  348,  177283, 25, }, // America/Indiana/Indianapolis
    {  594,  481,      73, 13, }, // America/Indiana/Knox
    {  594, 7924,      86, 16, }, // America/Indiana/Marengo
    {  594, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  594, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  594, 8001,     139, 14, }, // America/Indiana/Vevay
    {  594, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  594, 8049,     171, 16, }, // America/Indiana/Winamac
    {  594, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  594, 3390,  177308, 19, }, // America/Los_Angeles
    {  594, 5849,     207, 22, }, // America/Lower_Princes
    {  594, 8183,     229,  6, }, // America/Merida
    {  594, 3294,  177327, 18, }, // America/New_York
    {  594, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  594, 8292,     255, 20, }, // America/North_Dakota/Center
    {  594, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  594,  393,     298,  4, }, // America/Nuuk
    {  594, 3224,  177345, 19, }, // America/Phoenix
    {  594,  695,  177364, 18, }, // America/Rio_Branco
    {  594, 1862,  177382, 20, }, // America/Sao_Paulo
    {  594, 6862,     302, 16, }, // America/Scoresbysund
    {  594, 8471,     318, 14, }, // America/St_Barthelemy
    {  594, 2045,     332, 10, }, // America/St_Johns
    {  594, 5780,     342,  9, }, // America/St_Kitts
    {  594, 5797,     351,  9, }, // America/St_Lucia
    {  594,  855,     360, 10, }, // America/St_Thomas
    {  594, 5830,     370, 11, }, // America/St_Vincent
    {  594, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  594, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  594, 1152,     413,  9, }, // Asia/Kathmandu
    {  594,  992,     422,  7, }, // Asia/Kolkata
    {  594, 1178,     429,  5, }, // Asia/Macau
    {  594, 1202,     442,  6, }, // Asia/Yangon
    {  594, 1379,     448,  5, }, // Atlantic/Faroe
    {  594, 6784,     453, 10, }, // Atlantic/St_Helena
    {  594, 2419,     463,  4, }, // Europe/Kyiv
    {  594, 7174,     467,  7, }, // Indian/Reunion
    {  594, 3022,     474,  5, }, // Pacific/Chuuk
    {  594, 2929,  177402, 20, }, // Pacific/Honolulu
    {  594, 2961,     479,  7, }, // Pacific/Pohnpei
    {  595,   14,       0,  6, }, // Africa/Asmara Tasawaq/Latin/Niger
    {  595, 4337,       6,  8, }, // Africa/Sao_Tome
    {  595, 4154,      14,  8, }, // America/Asuncion
    {  595,  237,      22,  8, }, // America/Atikokan
    {  595, 7586,      30, 17, }, // America/Bahia_Banderas
    {  595, 3770,      47,  6, }, // America/Cancun
    {  595, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  595, 5640,      66,  7, }, // America/Curacao
    {  595,  481,      73, 13, }, // America/Indiana/Knox
    {  595, 7924,      86, 16, }, // America/Indiana/Marengo
    {  595, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  595, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  595, 8001,     139, 14, }, // America/Indiana/Vevay
    {  595, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  595, 8049,     171, 16, }, // America/Indiana/Winamac
    {  595, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  595, 5849,     207, 22, }, // America/Lower_Princes
    {  595, 8183,     229,  6, }, // America/Merida
    {  595, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  595, 8292,     255, 20, }, // America/North_Dakota/Center
    {  595, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  595,  393,     298,  4, }, // America/Nuuk
    {  595, 6862,     302, 16, }, // America/Scoresbysund
    {  595, 8471,     318, 14, }, // America/St_Barthelemy
    {  595, 2045,     332, 10, }, // America/St_Johns
    {  595, 5780,     342,  9, }, // America/St_Kitts
    {  595, 5797,     351,  9, }, // America/St_Lucia
    {  595,  855,     360, 10, }, // America/St_Thomas
    {  595, 5830,     370, 11, }, // America/St_Vincent
    {  595, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  595, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  595, 1152,     413,  9, }, // Asia/Kathmandu
    {  595,  992,     422,  7, }, // Asia/Kolkata
    {  595, 1178,     429,  5, }, // Asia/Macau
    {  595, 1202,     442,  6, }, // Asia/Yangon
    {  595, 1379,     448,  5, }, // Atlantic/Faroe
    {  595, 6784,     453, 10, }, // Atlantic/St_Helena
    {  595, 2419,     463,  4, }, // Europe/Kyiv
    {  595, 7174,     467,  7, }, // Indian/Reunion
    {  595, 3022,     474,  5, }, // Pacific/Chuuk
    {  595, 2961,     479,  7, }, // Pacific/Pohnpei
    {  596,   14,       0,  6, }, // Africa/Asmara Tatar/Cyrillic/Russia
    {  596, 4337,       6,  8, }, // Africa/Sao_Tome
    {  596, 4154,      14,  8, }, // America/Asuncion
    {  596,  237,      22,  8, }, // America/Atikokan
    {  596, 7586,      30, 17, }, // America/Bahia_Banderas
    {  596, 3770,      47,  6, }, // America/Cancun
    {  596, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  596, 5640,      66,  7, }, // America/Curacao
    {  596,  481,      73, 13, }, // America/Indiana/Knox
    {  596, 7924,      86, 16, }, // America/Indiana/Marengo
    {  596, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  596, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  596, 8001,     139, 14, }, // America/Indiana/Vevay
    {  596, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  596, 8049,     171, 16, }, // America/Indiana/Winamac
    {  596, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  596, 5849,     207, 22, }, // America/Lower_Princes
    {  596, 8183,     229,  6, }, // America/Merida
    {  596, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  596, 8292,     255, 20, }, // America/North_Dakota/Center
    {  596, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  596,  393,     298,  4, }, // America/Nuuk
    {  596, 6862,     302, 16, }, // America/Scoresbysund
    {  596, 8471,     318, 14, }, // America/St_Barthelemy
    {  596, 2045,     332, 10, }, // America/St_Johns
    {  596, 5780,     342,  9, }, // America/St_Kitts
    {  596, 5797,     351,  9, }, // America/St_Lucia
    {  596,  855,     360, 10, }, // America/St_Thomas
    {  596, 5830,     370, 11, }, // America/St_Vincent
    {  596, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  596, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  596, 1152,     413,  9, }, // Asia/Kathmandu
    {  596,  992,     422,  7, }, // Asia/Kolkata
    {  596, 1178,     429,  5, }, // Asia/Macau
    {  596, 1202,     442,  6, }, // Asia/Yangon
    {  596, 1379,     448,  5, }, // Atlantic/Faroe
    {  596, 6784,     453, 10, }, // Atlantic/St_Helena
    {  596, 2419,     463,  4, }, // Europe/Kyiv
    {  596, 7174,     467,  7, }, // Indian/Reunion
    {  596, 3022,     474,  5, }, // Pacific/Chuuk
    {  596, 2961,     479,  7, }, // Pacific/Pohnpei
    {  597, 6751,  177422,  9, }, // Africa/Abidjan Telugu/Telugu/India
    {  597, 6723,  177431,  5, }, // Africa/Accra
    {  597, 4961,  177436, 12, }, // Africa/Addis_Ababa
    {  597, 7383,  177448, 10, }, // Africa/Algiers
    {  597,   14,  177458,  7, }, // Africa/Asmara
    {  597,   44,  177465,  6, }, // Africa/Bamako
    {  597, 5159,  177471,  7, }, // Africa/Bangui
    {  597, 6709,  177478,  6, }, // Africa/Banjul
    {  597, 7398,  177484,  8, }, // Africa/Bissau
    {  597, 4859,  177492, 10, }, // Africa/Blantyre
    {  597, 5189,  177502, 12, }, // Africa/Brazzaville
    {  597, 4824,  177514,  9, }, // Africa/Bujumbura
    {  597, 2242,  177523,  4, }, // Africa/Cairo
    {  597, 4019,  177527, 11, }, // Africa/Casablanca
    {  597, 7412,  177538,  6, }, // Africa/Ceuta
    {  597, 6736,  177544,  8, }, // Africa/Conakry
    {  597, 6803,  177552,  5, }, // Africa/Dakar
    {  597, 5032,  177557, 14, }, // Africa/Dar_es_Salaam
    {  597, 4945,  177571,  8, }, // Africa/Djibouti
    {  597, 5145,  177579,  4, }, // Africa/Douala
    {  597, 5068,  177583,  9, }, // Africa/El_Aaiun
    {  597, 6816,  177592,  8, }, // Africa/Freetown
    {  597, 4808,  177600,  7, }, // Africa/Gaborone
    {  597, 4917,  177607,  5, }, // Africa/Harare
    {  597, 4381,  177612, 14, }, // Africa/Johannesburg
    {  597, 4401,  177626,  4, }, // Africa/Juba
    {  597, 5053,  177630,  6, }, // Africa/Kampala
    {  597, 4426,  177636,  8, }, // Africa/Khartoum
    {  597, 4875,  177644,  5, }, // Africa/Kigali
    {  597, 5208,  177649,  8, }, // Africa/Kinshasa
    {  597, 4646,  177657,  6, }, // Africa/Lagos
    {  597, 5238,  177663, 11, }, // Africa/Libreville
    {  597, 6832,  177674,  4, }, // Africa/Lome
    {  597, 5113,  177678,  7, }, // Africa/Luanda
    {  597, 4841,  177685,  8, }, // Africa/Lubumbashi
    {  597, 4903,  177693,  6, }, // Africa/Lusaka
    {  597, 5224,  177699,  5, }, // Africa/Malabo
    {  597, 4889,  177704,  6, }, // Africa/Maputo
    {  597, 5099,  177710,  5, }, // Africa/Maseru
    {  597, 5084,  177715,  6, }, // Africa/Mbabane
    {  597, 5015,  177721,  8, }, // Africa/Mogadishu
    {  597, 6997,  177729, 10, }, // Africa/Monrovia
    {  597, 3755,  177739,  6, }, // Africa/Nairobi
    {  597, 5173,  177745,  8, }, // Africa/Ndjamena
    {  597, 5256,  177753,  6, }, // Africa/Niamey
    {  597, 6766,  177759, 10, }, // Africa/Nouakchott
    {  597, 6690,  177769,  9, }, // Africa/Ouagadougou
    {  597, 5127,  177778, 11, }, // Africa/Porto-Novo
    {  597, 4337,  177789,  9, }, // Africa/Sao_Tome
    {  597, 2732,  177798,  8, }, // Africa/Tripoli
    {  597, 6503,  177806,  8, }, // Africa/Tunis
    {  597, 4054,  177814,  9, }, // Africa/Windhoek
    {  597,  132,  177823,  5, }, // America/Adak
    {  597, 3183,  177828,  8, }, // America/Anchorage
    {  597, 5524,  177836, 10, }, // America/Anguilla
    {  597, 5541,  177846,  8, }, // America/Antigua
    {  597, 4456,  177854, 11, }, // America/Araguaina
    {  597,  166,  177865, 16, }, // America/Argentina/Buenos_Aires
    {  597,   91,  177881,  8, }, // America/Argentina/Catamarca
    {  597,  270,  177889,  8, }, // America/Argentina/Cordoba
    {  597,  441,  177897,  6, }, // America/Argentina/Jujuy
    {  597, 7425,  177903,  8, }, // America/Argentina/La_Rioja
    {  597,  565,  177911,  7, }, // America/Argentina/Mendoza
    {  597, 7452,  177918, 14, }, // America/Argentina/Rio_Gallegos
    {  597, 7483,  177932,  6, }, // America/Argentina/Salta
    {  597, 7507,  177938, 11, }, // America/Argentina/San_Juan
    {  597, 5497,  177949, 11, }, // America/Argentina/San_Luis
    {  597, 7534,  177960,  7, }, // America/Argentina/Tucuman
    {  597, 7560,  177967,  6, }, // America/Argentina/Ushuaia
    {  597, 5557,  177973,  5, }, // America/Aruba
    {  597, 4154,  177978, 11, }, // America/Asuncion
    {  597,  237,  177989,  8, }, // America/Atikokan
    {  597, 3581,  177997,  4, }, // America/Bahia
    {  597, 7586,  178001, 13, }, // America/Bahia_Banderas
    {  597, 5571,  178014,  9, }, // America/Barbados
    {  597, 7609,  178023,  6, }, // America/Belem
    {  597, 5309,  178029,  6, }, // America/Belize
    {  597, 7623,  178035, 15, }, // America/Blanc-Sablon
    {  597, 7644,  178050, 11, }, // America/Boa_Vista
    {  597, 4263,  178061,  5, }, // America/Bogota
    {  597, 7662,  178066,  6, }, // America/Boise
    {  597, 7676,  178072, 14, }, // America/Cambridge_Bay
    {  597, 7698,  178086, 13, }, // America/Campo_Grande
    {  597, 3770,  178099,  8, }, // America/Cancun
    {  597, 4596,  178107,  7, }, // America/Caracas
    {  597, 4247,  178114,  5, }, // America/Cayenne
    {  597, 5398,  178119,  6, }, // America/Cayman
    {  597, 3251,  178125,  6, }, // America/Chicago
    {  597, 7719,  178131,  6, }, // America/Chihuahua
    {  597, 7737,  178137, 17, }, // America/Ciudad_Juarez
    {  597, 5324,  178154, 11, }, // America/Costa_Rica
    {  597, 7759,  178165,  9, }, // America/Creston
    {  597, 3693,  178174,  5, }, // America/Cuiaba
    {  597, 5640,  178179,  7, }, // America/Curacao
    {  597, 7775,  178186, 15, }, // America/Danmarkshavn
    {  597, 7796,  178201,  5, }, // America/Dawson
    {  597, 7811,  178206, 12, }, // America/Dawson_Creek
    {  597,  805,  178218,  8, }, // America/Denver
    {  597, 3351,  178226, 10, }, // America/Detroit
    {  597, 5656,  178236,  8, }, // America/Dominica
    {  597,  893,  178244,  9, }, // America/Edmonton
    {  597, 7832,  178253,  7, }, // America/Eirunepe
    {  597, 5343,  178260, 13, }, // America/El_Salvador
    {  597, 7849,  178273, 14, }, // America/Fort_Nelson
    {  597, 7869,  178287,  9, }, // America/Fortaleza
    {  597, 7887,  178296,  9, }, // America/Glace_Bay
    {  597, 6844,  178305,  7, }, // America/Goose_Bay
    {  597, 4514,  178312, 13, }, // America/Grand_Turk
    {  597, 5687,  178325,  7, }, // America/Grenada
    {  597, 5703,  178332, 10, }, // America/Guadeloupe
    {  597, 3662,  178342, 10, }, // America/Guatemala
    {  597, 4743,  178352,  9, }, // America/Guayaquil
    {  597, 6895,  178361,  5, }, // America/Guyana
    {  597, 1923,  178366, 10, }, // America/Halifax
    {  597, 2221,  178376,  5, }, // America/Havana
    {  597, 7905,  178381, 12, }, // America/Hermosillo
    {  597,  348,  178393, 13, }, // America/Indiana/Indianapolis
    {  597,  481,  178406, 15, }, // America/Indiana/Knox
    {  597, 7924,  178421, 15, }, // America/Indiana/Marengo
    {  597, 7948,  178436, 22, }, // America/Indiana/Petersburg
    {  597, 7975,  178458, 18, }, // America/Indiana/Tell_City
    {  597, 8001,  178476, 14, }, // America/Indiana/Vevay
    {  597, 8023,  178490, 22, }, // America/Indiana/Vincennes
    {  597, 8049,  178512, 17, }, // America/Indiana/Winamac
    {  597, 8073,  178529,  7, }, // America/Inuvik
    {  597,  660,  178536,  9, }, // America/Iqaluit
    {  597, 2665,  178545,  5, }, // America/Jamaica
    {  597, 5282,  178550,  4, }, // America/Juneau
    {  597,  521,  178554,  8, }, // America/Kentucky/Louisville
    {  597, 8088,  178562, 24, }, // America/Kentucky/Monticello
    {  597, 5621,  178586, 12, }, // America/Kralendijk
    {  597, 4278,  178598,  7, }, // America/La_Paz
    {  597, 7132,  178605,  4, }, // America/Lima
    {  597, 3390,  178609, 12, }, // America/Los_Angeles
    {  597, 5849,  178621, 24, }, // America/Lower_Princes
    {  597, 8116,  178645,  6, }, // America/Maceio
    {  597, 8131,  178651,  7, }, // America/Managua
    {  597, 1892,  178658,  5, }, // America/Manaus
    {  597, 5814,  178663,  8, }, // America/Marigot
    {  597, 5722,  178671, 10, }, // America/Martinique
    {  597, 8147,  178681,  9, }, // America/Matamoros
    {  597, 2779,  178690, 10, }, // America/Mazatlan
    {  597, 8165,  178700,  8, }, // America/Menominee
    {  597, 8183,  178708,  6, }, // America/Merida
    {  597, 8198,  178714,  9, }, // America/Metlakatla
    {  597, 2811,  178723, 13, }, // America/Mexico_City
    {  597, 4293,  178736,  8, }, // America/Miquelon
    {  597, 8217,  178744, 11, }, // America/Moncton
    {  597, 8233,  178755,  9, }, // America/Monterrey
    {  597, 4000,  178764, 11, }, // America/Montevideo
    {  597, 5741,  178775, 11, }, // America/Montserrat
    {  597, 5383,  178786,  8, }, // America/Nassau
    {  597, 3294,  178794, 10, }, // America/New_York
    {  597, 8251,  178804,  4, }, // America/Nome
    {  597, 1834,  178808,  7, }, // America/Noronha
    {  597, 8264,  178815, 18, }, // America/North_Dakota/Beulah
    {  597, 8292,  178833, 19, }, // America/North_Dakota/Center
    {  597, 8320,  178852, 22, }, // America/North_Dakota/New_Salem
    {  597,  393,  178874,  4, }, // America/Nuuk
    {  597, 8351,  178878,  5, }, // America/Ojinaga
    {  597, 5413,  178883,  5, }, // America/Panama
    {  597, 6106,  178888,  9, }, // America/Paramaribo
    {  597, 3224,  178897,  8, }, // America/Phoenix
    {  597, 3847,  178905, 19, }, // America/Port-au-Prince
    {  597, 5871,  178924, 19, }, // America/Port_of_Spain
    {  597, 8367,  178943, 13, }, // America/Porto_Velho
    {  597, 5760,  178956, 13, }, // America/Puerto_Rico
    {  597, 3932,  178969, 12, }, // America/Punta_Arenas
    {  597, 8387,  178981, 17, }, // America/Rankin_Inlet
    {  597, 8408,  178998,  6, }, // America/Recife
    {  597, 1979,  179004,  6, }, // America/Regina
    {  597, 8423,  179010,  9, }, // America/Resolute
    {  597,  695,  179019, 12, }, // America/Rio_Branco
    {  597, 8440,  179031,  8, }, // America/Santarem
    {  597, 2165,  179039,  9, }, // America/Santiago
    {  597, 6044,  179048, 13, }, // America/Santo_Domingo
    {  597, 1862,  179061,  9, }, // America/Sao_Paulo
    {  597, 6862,  179070, 17, }, // America/Scoresbysund
    {  597, 8457,  179087,  6, }, // America/Sitka
    {  597, 8471,  179093, 17, }, // America/St_Barthelemy
    {  597, 2045,  179110, 14, }, // America/St_Johns
    {  597, 5780,  179124, 14, }, // America/St_Kitts
    {  597, 5797,  179138, 14, }, // America/St_Lucia
    {  597,  855,  179152, 13, }, // America/St_Thomas
    {  597, 5830,  179165, 17, }, // America/St_Vincent
    {  597, 8493,  179182, 15, }, // America/Swift_Current
    {  597, 5363,  179197, 11, }, // America/Tegucigalpa
    {  597, 5673,  179208,  4, }, // America/Thule
    {  597,  313,  179212,  8, }, // America/Tijuana
    {  597,  608,  179220,  6, }, // America/Toronto
    {  597, 5605,  179226,  8, }, // America/Tortola
    {  597, 2077,  179234,  9, }, // America/Vancouver
    {  597, 2128,  179243, 11, }, // America/Whitehorse
    {  597,  734,  179254, 10, }, // America/Winnipeg
    {  597, 8515,  179264,  7, }, // America/Yakutat
    {  597, 5918,  179271,  4, }, // Antarctica/Casey
    {  597, 6027,  179275,  6, }, // Antarctica/Davis
    {  597, 6066,  179281, 22, }, // Antarctica/DumontDUrville
    {  597, 8531,  179303,  5, }, // Antarctica/Macquarie
    {  597, 7029,  179308,  5, }, // Antarctica/Mawson
    {  597, 7076,  179313, 11, }, // Antarctica/McMurdo
    {  597, 8552,  179324,  7, }, // Antarctica/Palmer
    {  597, 7189,  179331,  6, }, // Antarctica/Rothera
    {  597, 7258,  179337,  6, }, // Antarctica/Syowa
    {  597, 8570,  179343,  6, }, // Antarctica/Troll
    {  597, 7337,  179349,  8, }, // Antarctica/Vostok
    {  597, 1413,  179357, 17, }, // Arctic/Longyearbyen
    {  597, 5487,  179374,  5, }, // Asia/Aden
    {  597, 5297,  179379,  7, }, // Asia/Almaty
    {  597, 3870,  179386,  6, }, // Asia/Amman
    {  597, 5428,  179392,  6, }, // Asia/Anadyr
    {  597, 5440,  179398,  7, }, // Asia/Aqtau
    {  597, 5270,  179405,  7, }, // Asia/Aqtobe
    {  597,  964,  179412, 10, }, // Asia/Ashgabat
    {  597, 8587,  179422,  5, }, // Asia/Atyrau
    {  597, 3509,  179427,  8, }, // Asia/Baghdad
    {  597, 5451,  179435,  9, }, // Asia/Bahrain
    {  597, 3555,  178177,  4, }, // Asia/Baku
    {  597, 4368,  179444,  9, }, // Asia/Bangkok
    {  597, 3473,  179453,  8, }, // Asia/Barnaul
    {  597, 3988,  179461,  5, }, // Asia/Beirut
    {  597, 3680,  179466,  8, }, // Asia/Bishkek
    {  597, 5906,  179474,  6, }, // Asia/Brunei
    {  597, 4503,  179480,  4, }, // Asia/Chita
    {  597, 5963,  179484, 11, }, // Asia/Choibalsan
    {  597, 4413,  179495,  6, }, // Asia/Colombo
    {  597, 4442,  179501,  8, }, // Asia/Damascus
    {  597, 1060,  179509,  4, }, // Asia/Dhaka
    {  597, 6125,  179513,  4, }, // Asia/Dili
    {  597, 3498,  179517,  6, }, // Asia/Dubai
    {  597, 6092,  179523,  7, }, // Asia/Dushanbe
    {  597, 8599,  179530,  9, }, // Asia/Famagusta
    {  597, 8614,  179539,  4, }, // Asia/Gaza
    {  597, 4697,  179543,  8, }, // Asia/Hebron
    {  597, 1226,  179551, 17, }, // Asia/Ho_Chi_Minh
    {  597, 2576,  179568,  8, }, // Asia/Hong_Kong
    {  597, 4673,  179576,  6, }, // Asia/Hovd
    {  597, 4086,  179582, 11, }, // Asia/Irkutsk
    {  597, 6969,  179593,  7, }, // Asia/Jakarta
    {  597, 6955,  179600,  5, }, // Asia/Jayapura
    {  597, 1257,  179605,  7, }, // Asia/Jerusalem
    {  597, 3462,  179612,  6, }, // Asia/Kabul
    {  597, 4218,  179618,  9, }, // Asia/Kamchatka
    {  597, 4141,  179627,  5, }, // Asia/Karachi
    {  597, 1152,  179632,  8, }, // Asia/Kathmandu
    {  597, 8624,  179640,  7, }, // Asia/Khandyga
    {  597,  992,  179647,  8, }, // Asia/Kolkata
    {  597, 4099,  179655, 14, }, // Asia/Krasnoyarsk
    {  597, 4761,  179669, 10, }, // Asia/Kuala_Lumpur
    {  597, 5893,  179679,  7, }, // Asia/Kuching
    {  597, 5464,  179686,  6, }, // Asia/Kuwait
    {  597, 1178,  179692,  5, }, // Asia/Macau
    {  597, 3919,  179697,  6, }, // Asia/Magadan
    {  597, 1316,  179703,  8, }, // Asia/Makassar
    {  597, 7145,  179711,  5, }, // Asia/Manila
    {  597, 6883,  179716,  6, }, // Asia/Muscat
    {  597, 2446,  179722,  8, }, // Asia/Nicosia
    {  597, 8638,  179730, 18, }, // Asia/Novokuznetsk
    {  597, 4037,  179748, 13, }, // Asia/Novosibirsk
    {  597, 4131,  179761,  7, }, // Asia/Omsk
    {  597, 7108,  179768,  4, }, // Asia/Oral
    {  597, 6924,  179772, 11, }, // Asia/Phnom_Penh
    {  597, 8656,  179783, 11, }, // Asia/Pontianak
    {  597, 4116,  179794, 10, }, // Asia/Pyongyang
    {  597, 5476,  179804,  5, }, // Asia/Qatar
    {  597, 8671,  179809,  8, }, // Asia/Qostanay
    {  597, 4171,  179817, 12, }, // Asia/Qyzylorda
    {  597, 3486,  179829,  6, }, // Asia/Riyadh
    {  597, 4310,  179835,  7, }, // Asia/Sakhalin
    {  597, 7208,  179842,  9, }, // Asia/Samarkand
    {  597, 3116,  179851,  6, }, // Asia/Seoul
    {  597, 1020,  179857,  5, }, // Asia/Shanghai
    {  597, 3137,  179862,  8, }, // Asia/Singapore
    {  597, 4199,  179870, 18, }, // Asia/Srednekolymsk
    {  597, 3100,  179888,  4, }, // Asia/Taipei
    {  597, 4683,  179892,  9, }, // Asia/Tashkent
    {  597, 3817,  179901,  8, }, // Asia/Tbilisi
    {  597, 2638,  179909,  8, }, // Asia/Tehran
    {  597, 1284,  179917,  5, }, // Asia/Thimphu
    {  597, 2687,  179922,  6, }, // Asia/Tokyo
    {  597, 4474,  179928,  8, }, // Asia/Tomsk
    {  597, 1346,  179936, 10, }, // Asia/Ulaanbaatar
    {  597, 1126,  179946,  8, }, // Asia/Urumqi
    {  597, 8685,  179954, 10, }, // Asia/Ust-Nera
    {  597, 6940,  179964,  9, }, // Asia/Vientiane
    {  597, 4612,  179973, 14, }, // Asia/Vladivostok
    {  597, 4730,  179987,  9, }, // Asia/Yakutsk
    {  597, 1202,  179996,  6, }, // Asia/Yangon
    {  597, 3785,  180002, 13, }, // Asia/Yekaterinburg
    {  597, 3649,  180015,  7, }, // Asia/Yerevan
    {  597, 3565,  180022,  7, }, // Atlantic/Azores
    {  597, 5588,  180029,  8, }, // Atlantic/Bermuda
    {  597, 6591,  180037,  5, }, // Atlantic/Canary
    {  597, 3629,  180042, 11, }, // Atlantic/Cape_Verde
    {  597, 1379,  180053,  4, }, // Atlantic/Faroe
    {  597, 8699,  180057,  5, }, // Atlantic/Madeira
    {  597, 2614,  180062, 10, }, // Atlantic/Reykjavik
    {  597, 7235,  180072, 15, }, // Atlantic/South_Georgia
    {  597, 6784,  180087, 14, }, // Atlantic/St_Helena
    {  597, 6607,  180101,  8, }, // Atlantic/Stanley
    {  597, 1654,  180109,  7, }, // Australia/Adelaide
    {  597, 1619,  180116, 13, }, // Australia/Brisbane
    {  597, 1783,  180129, 13, }, // Australia/Broken_Hill
    {  597, 1567,  180142,  8, }, // Australia/Darwin
    {  597, 3539,  180150,  6, }, // Australia/Eucla
    {  597, 1500,  180156,  7, }, // Australia/Hobart
    {  597, 8716,  180163,  9, }, // Australia/Lindeman
    {  597, 1531,  180172,  9, }, // Australia/Lord_Howe
    {  597, 1711,  180181, 10, }, // Australia/Melbourne
    {  597, 1746,  180191,  6, }, // Australia/Perth
    {  597, 1447,  180197,  6, }, // Australia/Sydney
    {  597, 6390,  180203, 14, }, // Europe/Amsterdam
    {  597, 6149,  180217,  6, }, // Europe/Andorra
    {  597, 3522,  180223, 10, }, // Europe/Astrakhan
    {  597, 6577,  180233,  7, }, // Europe/Athens
    {  597, 6285,  180240, 11, }, // Europe/Belgrade
    {  597, 4659,  180251,  8, }, // Europe/Berlin
    {  597, 6437,  180259, 12, }, // Europe/Bratislava
    {  597, 6178,  180271, 11, }, // Europe/Brussels
    {  597, 3830,  180282, 10, }, // Europe/Bucharest
    {  597, 3708,  180292, 10, }, // Europe/Budapest
    {  597, 8751,  180302,  7, }, // Europe/Busingen
    {  597, 2475,  180309,  8, }, // Europe/Chisinau
    {  597, 6238,  180317, 11, }, // Europe/Copenhagen
    {  597, 2260,  180328,  7, }, // Europe/Dublin
    {  597, 6256,  180335, 11, }, // Europe/Gibraltar
    {  597, 8767,  180346, 10, }, // Europe/Guernsey
    {  597, 6561,  180356,  9, }, // Europe/Helsinki
    {  597, 8783,  180365, 12, }, // Europe/Isle_of_Man
    {  597, 1097,  180377, 10, }, // Europe/Istanbul
    {  597, 8802,  180387,  6, }, // Europe/Jersey
    {  597, 3881,  180393, 13, }, // Europe/Kaliningrad
    {  597, 8924,  180406,  6, }, // Europe/Kirov
    {  597, 2419,  180412,  4, }, // Europe/Kyiv
    {  597, 3078,  180416,  7, }, // Europe/Lisbon
    {  597, 6455,  180423, 12, }, // Europe/Ljubljana
    {  597, 2393,  180435,  5, }, // Europe/London
    {  597, 6314,  180440, 11, }, // Europe/Luxembourg
    {  597, 4794,  180451,  8, }, // Europe/Madrid
    {  597, 6346,  180459,  6, }, // Europe/Malta
    {  597, 6531,  180465,  9, }, // Europe/Mariehamn
    {  597, 3595,  180474,  8, }, // Europe/Minsk
    {  597, 6359,  180482,  6, }, // Europe/Monaco
    {  597, 3443,  180488,  6, }, // Europe/Moscow
    {  597, 6407,  180494,  5, }, // Europe/Oslo
    {  597, 4186,  180499,  8, }, // Europe/Paris
    {  597, 6373,  180507, 10, }, // Europe/Podgorica
    {  597, 6224,  180517,  6, }, // Europe/Prague
    {  597, 8816,  180523,  4, }, // Europe/Riga
    {  597, 6273,  180527,  4, }, // Europe/Rome
    {  597, 4233,  180531,  4, }, // Europe/Samara
    {  597, 6419,  180535, 11, }, // Europe/San_Marino
    {  597, 6194,  180546,  7, }, // Europe/Sarajevo
    {  597, 4353,  180553,  6, }, // Europe/Saratov
    {  597, 8828,  180559, 13, }, // Europe/Simferopol
    {  597, 6332,  180572,  9, }, // Europe/Skopje
    {  597, 6548,  180581,  6, }, // Europe/Sofia
    {  597, 6472,  180587, 10, }, // Europe/Stockholm
    {  597, 8846,  180597,  8, }, // Europe/Tallinn
    {  597, 6135,  180605,  6, }, // Europe/Tirane
    {  597, 8861,  180611, 12, }, // Europe/Ulyanovsk
    {  597, 6301,  180623,  6, }, // Europe/Vaduz
    {  597, 6516,  180629,  7, }, // Europe/Vatican
    {  597, 6164,  180636,  7, }, // Europe/Vienna
    {  597, 8878,  180643,  9, }, // Europe/Vilnius
    {  597, 4629,  180652, 12, }, // Europe/Volgograd
    {  597, 3055,  180664,  6, }, // Europe/Warsaw
    {  597, 6210,  180670,  8, }, // Europe/Zagreb
    {  597, 6489,  180678,  8, }, // Europe/Zurich
    {  597, 4980,  180686, 12, }, // Indian/Antananarivo
    {  597, 6910,  180698,  6, }, // Indian/Chagos
    {  597, 5979,  180704,  9, }, // Indian/Christmas
    {  597, 5996,  180713,  6, }, // Indian/Cocos
    {  597, 4931,  180719,  6, }, // Indian/Comoro
    {  597, 6624,  180725, 12, }, // Indian/Kerguelen
    {  597, 7223,  180737,  4, }, // Indian/Mahe
    {  597, 7013,  180741, 10, }, // Indian/Maldives
    {  597, 3971,  180751,  7, }, // Indian/Mauritius
    {  597, 5000,  180758,  5, }, // Indian/Mayotte
    {  597, 7174,  180763,  9, }, // Indian/Reunion
    {  597, 4324,  180772,  5, }, // Pacific/Apia
    {  597,  932,  180777,  8, }, // Pacific/Auckland
    {  597, 3608,  180785, 15, }, // Pacific/Bougainville
    {  597, 2863,  180800,  5, }, // Pacific/Chatham
    {  597, 3022,  180805,  4, }, // Pacific/Chuuk
    {  597, 2201,  180809,  6, }, // Pacific/Easter
    {  597, 7323,  180815,  5, }, // Pacific/Efate
    {  597, 7290,  180820,  6, }, // Pacific/Fakaofo
    {  597, 3804,  180826,  4, }, // Pacific/Fiji
    {  597, 7306,  180830,  8, }, // Pacific/Funafuti
    {  597, 6641,  180838, 10, }, // Pacific/Galapagos
    {  597, 6659,  180848,  9, }, // Pacific/Gambier
    {  597, 3724,  180857, 13, }, // Pacific/Guadalcanal
    {  597, 5935,  180870,  6, }, // Pacific/Guam
    {  597, 2929,  180876,  8, }, // Pacific/Honolulu
    {  597, 2897,  180884,  9, }, // Pacific/Kanton
    {  597, 3900,  180893, 10, }, // Pacific/Kiritimati
    {  597, 6982,  180903,  6, }, // Pacific/Kosrae
    {  597, 2708,  180909, 10, }, // Pacific/Kwajalein
    {  597, 4779,  180919,  5, }, // Pacific/Majuro
    {  597, 3953,  180924, 12, }, // Pacific/Marquesas
    {  597, 8909,  180936,  6, }, // Pacific/Midway
    {  597, 7047,  180942,  4, }, // Pacific/Nauru
    {  597, 7095,  180946,  4, }, // Pacific/Niue
    {  597, 4070,  180950,  8, }, // Pacific/Norfolk
    {  597, 7061,  180958,  6, }, // Pacific/Noumea
    {  597, 2991,  180964,  9, }, // Pacific/Pago_Pago
    {  597, 7118,  180973,  6, }, // Pacific/Palau
    {  597, 7157,  180979, 11, }, // Pacific/Pitcairn
    {  597, 2961,  180990,  7, }, // Pacific/Pohnpei
    {  597, 4709,  180997, 15, }, // Pacific/Port_Moresby
    {  597, 6009,  181012,  8, }, // Pacific/Rarotonga
    {  597, 5948,  181020,  8, }, // Pacific/Saipan
    {  597, 7275,  181028,  5, }, // Pacific/Tahiti
    {  597, 6675,  181033,  5, }, // Pacific/Tarawa
    {  597, 4485,  181038,  9, }, // Pacific/Tongatapu
    {  597, 7355,  181047,  4, }, // Pacific/Wake
    {  597, 7368,  181051,  8, }, // Pacific/Wallis
    {  598,   14,       0,  6, }, // Africa/Asmara Teso/Latin/Uganda
    {  598, 4337,       6,  8, }, // Africa/Sao_Tome
    {  598, 4154,      14,  8, }, // America/Asuncion
    {  598,  237,      22,  8, }, // America/Atikokan
    {  598, 7586,      30, 17, }, // America/Bahia_Banderas
    {  598, 3770,      47,  6, }, // America/Cancun
    {  598, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  598, 5640,      66,  7, }, // America/Curacao
    {  598,  481,      73, 13, }, // America/Indiana/Knox
    {  598, 7924,      86, 16, }, // America/Indiana/Marengo
    {  598, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  598, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  598, 8001,     139, 14, }, // America/Indiana/Vevay
    {  598, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  598, 8049,     171, 16, }, // America/Indiana/Winamac
    {  598, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  598, 5849,     207, 22, }, // America/Lower_Princes
    {  598, 8183,     229,  6, }, // America/Merida
    {  598, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  598, 8292,     255, 20, }, // America/North_Dakota/Center
    {  598, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  598,  393,     298,  4, }, // America/Nuuk
    {  598, 6862,     302, 16, }, // America/Scoresbysund
    {  598, 8471,     318, 14, }, // America/St_Barthelemy
    {  598, 2045,     332, 10, }, // America/St_Johns
    {  598, 5780,     342,  9, }, // America/St_Kitts
    {  598, 5797,     351,  9, }, // America/St_Lucia
    {  598,  855,     360, 10, }, // America/St_Thomas
    {  598, 5830,     370, 11, }, // America/St_Vincent
    {  598, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  598, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  598, 1152,     413,  9, }, // Asia/Kathmandu
    {  598,  992,     422,  7, }, // Asia/Kolkata
    {  598, 1178,     429,  5, }, // Asia/Macau
    {  598, 1202,     442,  6, }, // Asia/Yangon
    {  598, 1379,     448,  5, }, // Atlantic/Faroe
    {  598, 6784,     453, 10, }, // Atlantic/St_Helena
    {  598, 2419,     463,  4, }, // Europe/Kyiv
    {  598, 7174,     467,  7, }, // Indian/Reunion
    {  598, 3022,     474,  5, }, // Pacific/Chuuk
    {  598, 2961,     479,  7, }, // Pacific/Pohnpei
    {  600, 6751,  181059,  7, }, // Africa/Abidjan Thai/Thai/Thailand
    {  600, 6723,  181066,  6, }, // Africa/Accra
    {  600, 4961,  181072, 12, }, // Africa/Addis_Ababa
    {  600, 7383,  181084,  9, }, // Africa/Algiers
    {  600,   14,  181093,  7, }, // Africa/Asmara
    {  600,   44,  181100,  6, }, // Africa/Bamako
    {  600, 5159,  181106,  5, }, // Africa/Bangui
    {  600, 6709,  181111,  6, }, // Africa/Banjul
    {  600, 7398,  181117,  6, }, // Africa/Bissau
    {  600, 4859,  181123,  8, }, // Africa/Blantyre
    {  600, 5189,  181131,  9, }, // Africa/Brazzaville
    {  600, 4824,  181140,  9, }, // Africa/Bujumbura
    {  600, 2242,  181149,  4, }, // Africa/Cairo
    {  600, 4019,  181153, 12, }, // Africa/Casablanca
    {  600, 7412,  181165,  5, }, // Africa/Ceuta
    {  600, 6736,  181170,  7, }, // Africa/Conakry
    {  600, 6803,  181177,  6, }, // Africa/Dakar
    {  600, 5032,  181183, 12, }, // Africa/Dar_es_Salaam
    {  600, 4945,  181195,  6, }, // Africa/Djibouti
    {  600, 5145,  181201,  6, }, // Africa/Douala
    {  600, 5068,  181207, 10, }, // Africa/El_Aaiun
    {  600, 6816,  181217,  8, }, // Africa/Freetown
    {  600, 4808,  181225,  8, }, // Africa/Gaborone
    {  600, 4917,  181233,  6, }, // Africa/Harare
    {  600, 4381,  181239, 14, }, // Africa/Johannesburg
    {  600, 4401,  181253,  4, }, // Africa/Juba
    {  600, 5053,  181257,  7, }, // Africa/Kampala
    {  600, 4426,  181264,  7, }, // Africa/Khartoum
    {  600, 4875,  181271,  6, }, // Africa/Kigali
    {  600, 5208,  181277,  7, }, // Africa/Kinshasa
    {  600, 4646,  181284,  5, }, // Africa/Lagos
    {  600, 5238,  181289,  9, }, // Africa/Libreville
    {  600, 6832,  181298,  4, }, // Africa/Lome
    {  600, 5113,  181302,  7, }, // Africa/Luanda
    {  600, 4841,  181309,  9, }, // Africa/Lubumbashi
    {  600, 4903,  181318,  6, }, // Africa/Lusaka
    {  600, 5224,  181324,  6, }, // Africa/Malabo
    {  600, 4889,  181330,  6, }, // Africa/Maputo
    {  600, 5099,  181336,  6, }, // Africa/Maseru
    {  600, 5084,  181342,  9, }, // Africa/Mbabane
    {  600, 5015,  181351,  8, }, // Africa/Mogadishu
    {  600, 6997,  181359,  9, }, // Africa/Monrovia
    {  600, 3755,  181368,  8, }, // Africa/Nairobi
    {  600, 5173,  181376, 10, }, // Africa/Ndjamena
    {  600, 5256,  181386,  8, }, // Africa/Niamey
    {  600, 6766,  181394,  8, }, // Africa/Nouakchott
    {  600, 6690,  181402,  8, }, // Africa/Ouagadougou
    {  600, 5127,  181410, 11, }, // Africa/Porto-Novo
    {  600, 4337,  181421,  7, }, // Africa/Sao_Tome
    {  600, 2732,  181428,  7, }, // Africa/Tripoli
    {  600, 6503,  181435,  5, }, // Africa/Tunis
    {  600, 4054,  181440,  8, }, // Africa/Windhoek
    {  600,  132,  181448,  5, }, // America/Adak
    {  600, 3183,  181453,  9, }, // America/Anchorage
    {  600, 5524,  181462,  8, }, // America/Anguilla
    {  600, 5541,  181470,  7, }, // America/Antigua
    {  600, 4456,  181477,  9, }, // America/Araguaina
    {  600,  166,  181486, 11, }, // America/Argentina/Buenos_Aires
    {  600,   91,  181497, 10, }, // America/Argentina/Catamarca
    {  600,  270,  181507,  8, }, // America/Argentina/Cordoba
    {  600,  441,  181515,  5, }, // America/Argentina/Jujuy
    {  600, 7425,  181520,  8, }, // America/Argentina/La_Rioja
    {  600,  565,  181528,  7, }, // America/Argentina/Mendoza
    {  600, 7452,  181535, 12, }, // America/Argentina/Rio_Gallegos
    {  600, 7483,  181547,  5, }, // America/Argentina/Salta
    {  600, 7507,  181552,  6, }, // America/Argentina/San_Juan
    {  600, 5497,  181558,  8, }, // America/Argentina/San_Luis
    {  600, 7534,  181566,  7, }, // America/Argentina/Tucuman
    {  600, 7560,  181573,  8, }, // America/Argentina/Ushuaia
    {  600, 5557,  181581,  6, }, // America/Aruba
    {  600, 4154,  181587, 10, }, // America/Asuncion
    {  600,  237,  181597, 14, }, // America/Atikokan
    {  600, 3581,  181611,  6, }, // America/Bahia
    {  600, 7586,  181617, 14, }, // America/Bahia_Banderas
    {  600, 5571,  181631,  9, }, // America/Barbados
    {  600, 7609,  181640,  5, }, // America/Belem
    {  600, 5309,  181645,  5, }, // America/Belize
    {  600, 7623,  181650, 13, }, // America/Blanc-Sablon
    {  600, 7644,  181663,  8, }, // America/Boa_Vista
    {  600, 4263,  181671,  6, }, // America/Bogota
    {  600, 7662,  181677,  5, }, // America/Boise
    {  600, 7676,  181682, 13, }, // America/Cambridge_Bay
    {  600, 7698,  181695, 11, }, // America/Campo_Grande
    {  600, 3770,  181706,  6, }, // America/Cancun
    {  600, 4596,  181712,  7, }, // America/Caracas
    {  600, 4247,  181719,  5, }, // America/Cayenne
    {  600, 5398,  181724,  7, }, // America/Cayman
    {  600, 3251,  181731,  6, }, // America/Chicago
    {  600, 7719,  181737,  6, }, // America/Chihuahua
    {  600, 7737,  181743, 12, }, // America/Ciudad_Juarez
    {  600, 5324,  181755,  9, }, // America/Costa_Rica
    {  600, 7759,  181764,  7, }, // America/Creston
    {  600, 3693,  181771,  6, }, // America/Cuiaba
    {  600, 5640,  181777,  7, }, // America/Curacao
    {  600, 7775,  181784, 14, }, // America/Danmarkshavn
    {  600, 7796,  181798,  7, }, // America/Dawson
    {  600, 7811,  181805, 12, }, // America/Dawson_Creek
    {  600,  805,  181817,  8, }, // America/Denver
    {  600, 3351,  181825,  8, }, // America/Detroit
    {  600, 5656,  181833,  8, }, // America/Dominica
    {  600,  893,  181841,  9, }, // America/Edmonton
    {  600, 7832,  181850,  8, }, // America/Eirunepe
    {  600, 5343,  181858, 12, }, // America/El_Salvador
    {  600, 7849,  181870, 11, }, // America/Fort_Nelson
    {  600, 7869,  181881, 10, }, // America/Fortaleza
    {  600, 7887,  181891,  8, }, // America/Glace_Bay
    {  600, 6844,  181899,  7, }, // America/Goose_Bay
    {  600, 4514,  181906, 12, }, // America/Grand_Turk
    {  600, 5687,  181918,  7, }, // America/Grenada
    {  600, 5703,  181925,  9, }, // America/Guadeloupe
    {  600, 3662,  181934,  9, }, // America/Guatemala
    {  600, 4743,  181943,  8, }, // America/Guayaquil
    {  600, 6895,  181951,  7, }, // America/Guyana
    {  600, 1923,  181958,  9, }, // America/Halifax
    {  600, 2221,  181967,  6, }, // America/Havana
    {  600, 7905,  181973, 10, }, // America/Hermosillo
    {  600,  348,  181983, 14, }, // America/Indiana/Indianapolis
    {  600,  481,  181997, 16, }, // America/Indiana/Knox
    {  600, 7924,  182013, 18, }, // America/Indiana/Marengo
    {  600, 7948,  182031, 25, }, // America/Indiana/Petersburg
    {  600, 7975,  182056, 20, }, // America/Indiana/Tell_City
    {  600, 8001,  182076, 17, }, // America/Indiana/Vevay
    {  600, 8023,  182093, 20, }, // America/Indiana/Vincennes
    {  600, 8049,  182113, 18, }, // America/Indiana/Winamac
    {  600, 8073,  182131,  7, }, // America/Inuvik
    {  600,  660,  182138,  8, }, // America/Iqaluit
    {  600, 2665,  182146,  6, }, // America/Jamaica
    {  600, 5282,  182152,  4, }, // America/Juneau
    {  600,  521,  182156,  9, }, // America/Kentucky/Louisville
    {  600, 8088,  182165, 20, }, // America/Kentucky/Monticello
    {  600, 5621,  182185, 11, }, // America/Kralendijk
    {  600, 4278,  182196,  5, }, // America/La_Paz
    {  600, 7132,  182201,  4, }, // America/Lima
    {  600, 3390,  182205, 11, }, // America/Los_Angeles
    {  600, 5849,  182216, 22, }, // America/Lower_Princes
    {  600, 8116,  182238,  6, }, // America/Maceio
    {  600, 8131,  182244,  7, }, // America/Managua
    {  600, 1892,  182251,  7, }, // America/Manaus
    {  600, 5814,  182258,  8, }, // America/Marigot
    {  600, 5722,  182266,  9, }, // America/Martinique
    {  600, 8147,  182275,  9, }, // America/Matamoros
    {  600, 2779,  182284,  8, }, // America/Mazatlan
    {  600, 8165,  182292,  8, }, // America/Menominee
    {  600, 8183,  182300,  6, }, // America/Merida
    {  600, 8198,  182306, 10, }, // America/Metlakatla
    {  600, 2811,  182316, 12, }, // America/Mexico_City
    {  600, 4293,  182328,  7, }, // America/Miquelon
    {  600, 8217,  182335,  6, }, // America/Moncton
    {  600, 8233,  182341, 11, }, // America/Monterrey
    {  600, 4000,  182352, 11, }, // America/Montevideo
    {  600, 5741,  182363, 11, }, // America/Montserrat
    {  600, 5383,  182374,  5, }, // America/Nassau
    {  600, 3294,  182379,  8, }, // America/New_York
    {  600, 8251,  182387,  3, }, // America/Nome
    {  600, 1834,  182390,  7, }, // America/Noronha
    {  600, 8264,  182397, 19, }, // America/North_Dakota/Beulah
    {  600, 8292,  182416, 21, }, // America/North_Dakota/Center
    {  600, 8320,  182437, 21, }, // America/North_Dakota/New_Salem
    {  600,  393,  182458,  6, }, // America/Nuuk
    {  600, 8351,  182464,  8, }, // America/Ojinaga
    {  600, 5413,  182472,  6, }, // America/Panama
    {  600, 6106,  182478, 10, }, // America/Paramaribo
    {  600, 3224,  182488,  7, }, // America/Phoenix
    {  600, 3847,  182495, 12, }, // America/Port-au-Prince
    {  600, 5871,  182507, 12, }, // America/Port_of_Spain
    {  600, 8367,  182519, 11, }, // America/Porto_Velho
    {  600, 5760,  182530,  9, }, // America/Puerto_Rico
    {  600, 3932,  182539, 12, }, // America/Punta_Arenas
    {  600, 8387,  182551, 13, }, // America/Rankin_Inlet
    {  600, 8408,  182564,  6, }, // America/Recife
    {  600, 1979,  182570,  6, }, // America/Regina
    {  600, 8423,  182576,  7, }, // America/Resolute
    {  600,  695,  182583, 10, }, // America/Rio_Branco
    {  600, 8440,  182593,  8, }, // America/Santarem
    {  600, 2165,  182601,  9, }, // America/Santiago
    {  600, 6044,  182610, 13, }, // America/Santo_Domingo
    {  600, 1862,  182623,  8, }, // America/Sao_Paulo
    {  600, 6862,  182631, 13, }, // America/Scoresbysund
    {  600, 8457,  182644,  5, }, // America/Sitka
    {  600, 8471,  182649, 15, }, // America/St_Barthelemy
    {  600, 2045,  182664, 10, }, // America/St_Johns
    {  600, 5780,  182674, 10, }, // America/St_Kitts
    {  600, 5797,  182684, 11, }, // America/St_Lucia
    {  600,  855,  182695, 10, }, // America/St_Thomas
    {  600, 5830,  182705, 13, }, // America/St_Vincent
    {  600, 8493,  182718, 16, }, // America/Swift_Current
    {  600, 5363,  182734, 11, }, // America/Tegucigalpa
    {  600, 5673,  182745,  4, }, // America/Thule
    {  600,  313,  182749,  7, }, // America/Tijuana
    {  600,  608,  182756,  7, }, // America/Toronto
    {  600, 5605,  182763,  8, }, // America/Tortola
    {  600, 2077,  182771, 10, }, // America/Vancouver
    {  600, 2128,  182781,  9, }, // America/Whitehorse
    {  600,  734,  182790,  8, }, // America/Winnipeg
    {  600, 8515,  182798,  7, }, // America/Yakutat
    {  600, 5918,  182805,  6, }, // Antarctica/Casey
    {  600, 6027,  182811,  5, }, // Antarctica/Davis
    {  600, 6066,  182816, 16, }, // Antarctica/DumontDUrville
    {  600, 8531,  182832,  8, }, // Antarctica/Macquarie
    {  600, 7029,  182840,  7, }, // Antarctica/Mawson
    {  600, 7076,  182847,  8, }, // Antarctica/McMurdo
    {  600, 8552,  182855,  9, }, // Antarctica/Palmer
    {  600, 7189,  182864,  6, }, // Antarctica/Rothera
    {  600, 7258,  182870,  6, }, // Antarctica/Syowa
    {  600, 8570,  182876,  4, }, // Antarctica/Troll
    {  600, 7337,  182880,  6, }, // Antarctica/Vostok
    {  600, 1413,  182886, 14, }, // Arctic/Longyearbyen
    {  600, 5487,  182900,  5, }, // Asia/Aden
    {  600, 5297,  182905,  7, }, // Asia/Almaty
    {  600, 3870,  182912,  6, }, // Asia/Amman
    {  600, 5428,  182918,  8, }, // Asia/Anadyr
    {  600, 5440,  182926,  7, }, // Asia/Aqtau
    {  600, 5270,  182933,  7, }, // Asia/Aqtobe
    {  600,  964,  182940,  8, }, // Asia/Ashgabat
    {  600, 8587,  182948,  6, }, // Asia/Atyrau
    {  600, 3509,  182954,  6, }, // Asia/Baghdad
    {  600, 5451,  182960,  7, }, // Asia/Bahrain
    {  600, 3555,  182967,  4, }, // Asia/Baku
    {  600, 4368,  182971,  7, }, // Asia/Bangkok
    {  600, 3473,  182978,  9, }, // Asia/Barnaul
    {  600, 3988,  182987,  5, }, // Asia/Beirut
    {  600, 3680,  182992,  6, }, // Asia/Bishkek
    {  600, 5906,  182998,  5, }, // Asia/Brunei
    {  600, 4503,  183003,  4, }, // Asia/Chita
    {  600, 5963,  183007,  9, }, // Asia/Choibalsan
    {  600, 4413,  183016,  7, }, // Asia/Colombo
    {  600, 4442,  183023,  8, }, // Asia/Damascus
    {  600, 1060,  181177,  4, }, // Asia/Dhaka
    {  600, 6125,  183031,  4, }, // Asia/Dili
    {  600, 3498,  183035,  4, }, // Asia/Dubai
    {  600, 6092,  183039,  7, }, // Asia/Dushanbe
    {  600, 8599,  183046,  9, }, // Asia/Famagusta
    {  600, 8614,  183055,  4, }, // Asia/Gaza
    {  600, 4697,  183059,  6, }, // Asia/Hebron
    {  600, 1226,  183065, 12, }, // Asia/Ho_Chi_Minh
    {  600, 2576,  183077,  6, }, // Asia/Hong_Kong
    {  600, 4673,  183083,  5, }, // Asia/Hovd
    {  600, 4086,  183088, 10, }, // Asia/Irkutsk
    {  600, 6969,  183098,  8, }, // Asia/Jakarta
    {  600, 6955,  183106,  8, }, // Asia/Jayapura
    {  600, 1257,  183114,  9, }, // Asia/Jerusalem
    {  600, 3462,  183123,  5, }, // Asia/Kabul
    {  600, 4218,  183128,  8, }, // Asia/Kamchatka
    {  600, 4141,  183136,  6, }, // Asia/Karachi
    {  600, 1152,  183142,  8, }, // Asia/Kathmandu
    {  600, 8624,  183150,  8, }, // Asia/Khandyga
    {  600,  992,  183158,  7, }, // Asia/Kolkata
    {  600, 4099,  183165, 13, }, // Asia/Krasnoyarsk
    {  600, 4761,  183178, 13, }, // Asia/Kuala_Lumpur
    {  600, 5893,  183191,  5, }, // Asia/Kuching
    {  600, 5464,  183196,  5, }, // Asia/Kuwait
    {  600, 1178,  183201,  6, }, // Asia/Macau
    {  600, 3919,  183207,  7, }, // Asia/Magadan
    {  600, 1316,  183214,  9, }, // Asia/Makassar
    {  600, 7145,  183223,  6, }, // Asia/Manila
    {  600, 6883,  183229,  6, }, // Asia/Muscat
    {  600, 2446,  183235,  8, }, // Asia/Nicosia
    {  600, 8638,  183243, 13, }, // Asia/Novokuznetsk
    {  600, 4037,  183256, 13, }, // Asia/Novosibirsk
    {  600, 4131,  183269,  6, }, // Asia/Omsk
    {  600, 7108,  183275,  5, }, // Asia/Oral
    {  600, 6924,  183280,  6, }, // Asia/Phnom_Penh
    {  600, 8656,  183286, 10, }, // Asia/Pontianak
    {  600, 4116,  183296,  8, }, // Asia/Pyongyang
    {  600, 5476,  183304,  6, }, // Asia/Qatar
    {  600, 8671,  183310,  9, }, // Asia/Qostanay
    {  600, 4171,  183319, 10, }, // Asia/Qyzylorda
    {  600, 3486,  183329,  7, }, // Asia/Riyadh
    {  600, 4310,  183336,  7, }, // Asia/Sakhalin
    {  600, 7208,  183343, 11, }, // Asia/Samarkand
    {  600, 3116,  182578,  3, }, // Asia/Seoul
    {  600, 1020,  183354,  9, }, // Asia/Shanghai
    {  600, 3137,  183363,  8, }, // Asia/Singapore
    {  600, 4199,  183371, 15, }, // Asia/Srednekolymsk
    {  600, 3100,  183386,  4, }, // Asia/Taipei
    {  600, 4683,  183390,  8, }, // Asia/Tashkent
    {  600, 3817,  183398,  7, }, // Asia/Tbilisi
    {  600, 2638,  183405,  7, }, // Asia/Tehran
    {  600, 1284,  183412,  5, }, // Asia/Thimphu
    {  600, 2687,  183417,  7, }, // Asia/Tokyo
    {  600, 4474,  183424,  6, }, // Asia/Tomsk
    {  600, 1346,  183430, 11, }, // Asia/Ulaanbaatar
    {  600, 1126,  183441,  7, }, // Asia/Urumqi
    {  600, 8685,  183448,  9, }, // Asia/Ust-Nera
    {  600, 6940,  183457, 11, }, // Asia/Vientiane
    {  600, 4612,  183468, 12, }, // Asia/Vladivostok
    {  600, 4730,  183480,  8, }, // Asia/Yakutsk
    {  600, 1202,  183488,  8, }, // Asia/Yangon
    {  600, 3785,  183496, 16, }, // Asia/Yekaterinburg
    {  600, 3649,  183512,  7, }, // Asia/Yerevan
    {  600, 3565,  183519,  7, }, // Atlantic/Azores
    {  600, 5588,  183526, 10, }, // Atlantic/Bermuda
    {  600, 6591,  183536,  6, }, // Atlantic/Canary
    {  600, 3629,  183542,  9, }, // Atlantic/Cape_Verde
    {  600, 1379,  183551,  4, }, // Atlantic/Faroe
    {  600, 8699,  183555,  6, }, // Atlantic/Madeira
    {  600, 2614,  183561,  8, }, // Atlantic/Reykjavik
    {  600, 7235,  183569, 14, }, // Atlantic/South_Georgia
    {  600, 6784,  183583, 11, }, // Atlantic/St_Helena
    {  600, 6607,  183594,  8, }, // Atlantic/Stanley
    {  600, 1654,  183602,  7, }, // Australia/Adelaide
    {  600, 1619,  183609,  7, }, // Australia/Brisbane
    {  600, 1783,  183616, 12, }, // Australia/Broken_Hill
    {  600, 1567,  183628,  7, }, // Australia/Darwin
    {  600, 3539,  183635,  5, }, // Australia/Eucla
    {  600, 1500,  183640,  7, }, // Australia/Hobart
    {  600, 8716,  183647,  8, }, // Australia/Lindeman
    {  600, 1531,  183655,  9, }, // Australia/Lord_Howe
    {  600, 1711,  183664,  9, }, // Australia/Melbourne
    {  600, 1746,  183673,  6, }, // Australia/Perth
    {  600, 1447,  183679,  7, }, // Australia/Sydney
    {  600, 6390,  183686, 10, }, // Europe/Amsterdam
    {  600, 6149,  183696,  9, }, // Europe/Andorra
    {  600, 3522,  183705,  9, }, // Europe/Astrakhan
    {  600, 6577,  183714,  7, }, // Europe/Athens
    {  600, 6285,  183721,  7, }, // Europe/Belgrade
    {  600, 4659,  183728,  8, }, // Europe/Berlin
    {  600, 6437,  183736, 10, }, // Europe/Bratislava
    {  600, 6178,  183746,  9, }, // Europe/Brussels
    {  600, 3830,  183755,  7, }, // Europe/Bucharest
    {  600, 3708,  183762,  7, }, // Europe/Budapest
    {  600, 8751,  183769, 10, }, // Europe/Busingen
    {  600, 2475,  183779,  7, }, // Europe/Chisinau
    {  600, 6238,  183786, 10, }, // Europe/Copenhagen
    {  600, 2260,  183796,  7, }, // Europe/Dublin
    {  600, 6256,  183803, 10, }, // Europe/Gibraltar
    {  600, 8767,  183813, 10, }, // Europe/Guernsey
    {  600, 6561,  183823,  8, }, // Europe/Helsinki
    {  600, 8783,  183831,  7, }, // Europe/Isle_of_Man
    {  600, 1097,  183838,  9, }, // Europe/Istanbul
    {  600, 8802,  183847,  9, }, // Europe/Jersey
    {  600, 3881,  183856, 11, }, // Europe/Kaliningrad
    {  600, 8924,  183867,  5, }, // Europe/Kirov
    {  600, 2419,  183872,  5, }, // Europe/Kyiv
    {  600, 3078,  183877,  6, }, // Europe/Lisbon
    {  600, 6455,  183883,  9, }, // Europe/Ljubljana
    {  600, 2393,  183892,  6, }, // Europe/London
    {  600, 6314,  183898, 12, }, // Europe/Luxembourg
    {  600, 4794,  183910,  6, }, // Europe/Madrid
    {  600, 6346,  183916,  5, }, // Europe/Malta
    {  600, 6531,  183921,  9, }, // Europe/Mariehamn
    {  600, 3595,  183930,  6, }, // Europe/Minsk
    {  600, 6359,  183936,  6, }, // Europe/Monaco
    {  600, 3443,  183942,  5, }, // Europe/Moscow
    {  600, 6407,  183947,  5, }, // Europe/Oslo
    {  600, 4186,  183952,  5, }, // Europe/Paris
    {  600, 6373,  183957, 10, }, // Europe/Podgorica
    {  600, 6224,  183967,  4, }, // Europe/Prague
    {  600, 8816,  181760,  4, }, // Europe/Riga
    {  600, 6273,  183553,  3, }, // Europe/Rome
    {  600, 4233,  183971,  6, }, // Europe/Samara
    {  600, 6419,  183977,  9, }, // Europe/San_Marino
    {  600, 6194,  183986,  8, }, // Europe/Sarajevo
    {  600, 4353,  183994,  7, }, // Europe/Saratov
    {  600, 8828,  184001, 11, }, // Europe/Simferopol
    {  600, 6332,  184012,  7, }, // Europe/Skopje
    {  600, 6548,  184019,  6, }, // Europe/Sofia
    {  600, 6472,  184025,  9, }, // Europe/Stockholm
    {  600, 8846,  184034,  8, }, // Europe/Tallinn
    {  600, 6135,  184042,  6, }, // Europe/Tirane
    {  600, 8861,  184048,  9, }, // Europe/Ulyanovsk
    {  600, 6301,  184057,  5, }, // Europe/Vaduz
    {  600, 6516,  184062,  7, }, // Europe/Vatican
    {  600, 6164,  184069,  7, }, // Europe/Vienna
    {  600, 8878,  184076,  8, }, // Europe/Vilnius
    {  600, 4629,  184084,  9, }, // Europe/Volgograd
    {  600, 3055,  184093,  6, }, // Europe/Warsaw
    {  600, 6210,  184099,  7, }, // Europe/Zagreb
    {  600, 6489,  184106,  5, }, // Europe/Zurich
    {  600, 4980,  184111, 13, }, // Indian/Antananarivo
    {  600, 6910,  184124,  5, }, // Indian/Chagos
    {  600, 5979,  184129,  9, }, // Indian/Christmas
    {  600, 5996,  184138,  5, }, // Indian/Cocos
    {  600, 4931,  184143,  6, }, // Indian/Comoro
    {  600, 6624,  184149, 10, }, // Indian/Kerguelen
    {  600, 7223,  184159,  4, }, // Indian/Mahe
    {  600, 7013,  184163,  8, }, // Indian/Maldives
    {  600, 3971,  184171,  9, }, // Indian/Mauritius
    {  600, 5000,  184180,  6, }, // Indian/Mayotte
    {  600, 7174,  184186,  8, }, // Indian/Reunion
    {  600, 4324,  184194,  6, }, // Pacific/Apia
    {  600,  932,  184200,  8, }, // Pacific/Auckland
    {  600, 3608,  184208, 10, }, // Pacific/Bougainville
    {  600, 2863,  184218,  5, }, // Pacific/Chatham
    {  600, 3022,  184223,  4, }, // Pacific/Chuuk
    {  600, 2201,  184227,  8, }, // Pacific/Easter
    {  600, 7323,  184235,  6, }, // Pacific/Efate
    {  600, 7290,  184241,  7, }, // Pacific/Fakaofo
    {  600, 3804,  184248,  4, }, // Pacific/Fiji
    {  600, 7306,  184252,  8, }, // Pacific/Funafuti
    {  600, 6641,  184260,  9, }, // Pacific/Galapagos
    {  600, 6659,  184269,  9, }, // Pacific/Gambier
    {  600, 3724,  184278, 11, }, // Pacific/Guadalcanal
    {  600, 5935,  184289,  3, }, // Pacific/Guam
    {  600, 2929,  184292,  8, }, // Pacific/Honolulu
    {  600, 2897,  184300, 13, }, // Pacific/Kanton
    {  600, 3900,  184313, 10, }, // Pacific/Kiritimati
    {  600, 6982,  184323,  5, }, // Pacific/Kosrae
    {  600, 2708,  184328,  8, }, // Pacific/Kwajalein
    {  600, 4779,  184336,  6, }, // Pacific/Majuro
    {  600, 3953,  184342,  9, }, // Pacific/Marquesas
    {  600, 8909,  184351,  7, }, // Pacific/Midway
    {  600, 7047,  184358,  6, }, // Pacific/Nauru
    {  600, 7095,  184364,  6, }, // Pacific/Niue
    {  600, 4070,  184370,  9, }, // Pacific/Norfolk
    {  600, 7061,  184379,  6, }, // Pacific/Noumea
    {  600, 2991,  184385,  9, }, // Pacific/Pago_Pago
    {  600, 7118,  184394,  5, }, // Pacific/Palau
    {  600, 7157,  184399,  8, }, // Pacific/Pitcairn
    {  600, 2961,  184407,  6, }, // Pacific/Pohnpei
    {  600, 4709,  184413, 12, }, // Pacific/Port_Moresby
    {  600, 6009,  184425,  9, }, // Pacific/Rarotonga
    {  600, 5948,  184434,  5, }, // Pacific/Saipan
    {  600, 7275,  184439,  6, }, // Pacific/Tahiti
    {  600, 6675,  184445,  6, }, // Pacific/Tarawa
    {  600, 4485,  184451,  9, }, // Pacific/Tongatapu
    {  600, 7355,  184460,  3, }, // Pacific/Wake
    {  600, 7368,  184463,  6, }, // Pacific/Wallis
    {  601,   14,       0,  6, }, // Africa/Asmara Tibetan/Tibetan/China
    {  601, 4337,       6,  8, }, // Africa/Sao_Tome
    {  601, 4154,      14,  8, }, // America/Asuncion
    {  601,  237,      22,  8, }, // America/Atikokan
    {  601, 7586,      30, 17, }, // America/Bahia_Banderas
    {  601, 3770,      47,  6, }, // America/Cancun
    {  601, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  601, 5640,      66,  7, }, // America/Curacao
    {  601,  481,      73, 13, }, // America/Indiana/Knox
    {  601, 7924,      86, 16, }, // America/Indiana/Marengo
    {  601, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  601, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  601, 8001,     139, 14, }, // America/Indiana/Vevay
    {  601, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  601, 8049,     171, 16, }, // America/Indiana/Winamac
    {  601, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  601, 5849,     207, 22, }, // America/Lower_Princes
    {  601, 8183,     229,  6, }, // America/Merida
    {  601, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  601, 8292,     255, 20, }, // America/North_Dakota/Center
    {  601, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  601,  393,     298,  4, }, // America/Nuuk
    {  601, 6862,     302, 16, }, // America/Scoresbysund
    {  601, 8471,     318, 14, }, // America/St_Barthelemy
    {  601, 2045,     332, 10, }, // America/St_Johns
    {  601, 5780,     342,  9, }, // America/St_Kitts
    {  601, 5797,     351,  9, }, // America/St_Lucia
    {  601,  855,     360, 10, }, // America/St_Thomas
    {  601, 5830,     370, 11, }, // America/St_Vincent
    {  601, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  601, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  601, 1152,     413,  9, }, // Asia/Kathmandu
    {  601,  992,     422,  7, }, // Asia/Kolkata
    {  601, 1178,     429,  5, }, // Asia/Macau
    {  601, 1202,     442,  6, }, // Asia/Yangon
    {  601, 1379,     448,  5, }, // Atlantic/Faroe
    {  601, 6784,     453, 10, }, // Atlantic/St_Helena
    {  601, 2419,     463,  4, }, // Europe/Kyiv
    {  601, 7174,     467,  7, }, // Indian/Reunion
    {  601, 3022,     474,  5, }, // Pacific/Chuuk
    {  601, 2961,     479,  7, }, // Pacific/Pohnpei
    {  603,   14,       0,  6, }, // Africa/Asmara Tigre/Ethiopic/Eritrea
    {  603, 4337,       6,  8, }, // Africa/Sao_Tome
    {  603, 4154,      14,  8, }, // America/Asuncion
    {  603,  237,      22,  8, }, // America/Atikokan
    {  603, 7586,      30, 17, }, // America/Bahia_Banderas
    {  603, 3770,      47,  6, }, // America/Cancun
    {  603, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  603, 5640,      66,  7, }, // America/Curacao
    {  603,  481,      73, 13, }, // America/Indiana/Knox
    {  603, 7924,      86, 16, }, // America/Indiana/Marengo
    {  603, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  603, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  603, 8001,     139, 14, }, // America/Indiana/Vevay
    {  603, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  603, 8049,     171, 16, }, // America/Indiana/Winamac
    {  603, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  603, 5849,     207, 22, }, // America/Lower_Princes
    {  603, 8183,     229,  6, }, // America/Merida
    {  603, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  603, 8292,     255, 20, }, // America/North_Dakota/Center
    {  603, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  603,  393,     298,  4, }, // America/Nuuk
    {  603, 6862,     302, 16, }, // America/Scoresbysund
    {  603, 8471,     318, 14, }, // America/St_Barthelemy
    {  603, 2045,     332, 10, }, // America/St_Johns
    {  603, 5780,     342,  9, }, // America/St_Kitts
    {  603, 5797,     351,  9, }, // America/St_Lucia
    {  603,  855,     360, 10, }, // America/St_Thomas
    {  603, 5830,     370, 11, }, // America/St_Vincent
    {  603, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  603, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  603, 1152,     413,  9, }, // Asia/Kathmandu
    {  603,  992,     422,  7, }, // Asia/Kolkata
    {  603, 1178,     429,  5, }, // Asia/Macau
    {  603, 1202,     442,  6, }, // Asia/Yangon
    {  603, 1379,     448,  5, }, // Atlantic/Faroe
    {  603, 6784,     453, 10, }, // Atlantic/St_Helena
    {  603, 2419,     463,  4, }, // Europe/Kyiv
    {  603, 7174,     467,  7, }, // Indian/Reunion
    {  603, 3022,     474,  5, }, // Pacific/Chuuk
    {  603, 2961,     479,  7, }, // Pacific/Pohnpei
    {  604, 6751,  184469,  4, }, // Africa/Abidjan Tigrinya/Ethiopic/Ethiopia
    {  604, 6723,  184473,  3, }, // Africa/Accra
    {  604, 4961,  184476,  7, }, // Africa/Addis_Ababa
    {  604, 7383,  184483,  5, }, // Africa/Algiers
    {  604,   14,  184488,  4, }, // Africa/Asmara
    {  604,   44,    6847,  3, }, // Africa/Bamako
    {  604, 5159,  184492,  3, }, // Africa/Bangui
    {  604, 6709,    6854,  4, }, // Africa/Banjul
    {  604, 7398,  184495,  3, }, // Africa/Bissau
    {  604, 4859,    6861,  6, }, // Africa/Blantyre
    {  604, 5189,    6867,  5, }, // Africa/Brazzaville
    {  604, 4824,  184498,  5, }, // Africa/Bujumbura
    {  604, 2242,    6877,  3, }, // Africa/Cairo
    {  604, 4019,    6880,  6, }, // Africa/Casablanca
    {  604, 7412,  184503,  3, }, // Africa/Ceuta
    {  604, 6736,  184506,  4, }, // Africa/Conakry
    {  604, 6803,    6893,  3, }, // Africa/Dakar
    {  604, 5032,  184510,  9, }, // Africa/Dar_es_Salaam
    {  604, 4945,    6902,  3, }, // Africa/Djibouti
    {  604, 5145,  184519,  3, }, // Africa/Douala
    {  604, 5068,  184522,  6, }, // Africa/El_Aaiun
    {  604, 6816,    6915,  5, }, // Africa/Freetown
    {  604, 4808,    6920,  4, }, // Africa/Gaborone
    {  604, 4917,  184528,  3, }, // Africa/Harare
    {  604, 4381,    6927,  7, }, // Africa/Johannesburg
    {  604, 4401,    6934,  2, }, // Africa/Juba
    {  604, 5053,    6936,  4, }, // Africa/Kampala
    {  604, 4426,    6940,  4, }, // Africa/Khartoum
    {  604, 4875,    6944,  3, }, // Africa/Kigali
    {  604, 5208,    6947,  4, }, // Africa/Kinshasa
    {  604, 4646,    6951,  3, }, // Africa/Lagos
    {  604, 5238,  184531,  5, }, // Africa/Libreville
    {  604, 6832,  184536,  2, }, // Africa/Lome
    {  604, 5113,  184538,  4, }, // Africa/Luanda
    {  604, 4841,    6965,  5, }, // Africa/Lubumbashi
    {  604, 4903,    6970,  3, }, // Africa/Lusaka
    {  604, 5224,  184542,  3, }, // Africa/Malabo
    {  604, 4889,  184545,  3, }, // Africa/Maputo
    {  604, 5099,  184548,  3, }, // Africa/Maseru
    {  604, 5084,  184551,  4, }, // Africa/Mbabane
    {  604, 5015,  184555,  4, }, // Africa/Mogadishu
    {  604, 6997,  184559,  5, }, // Africa/Monrovia
    {  604, 3755,    6995,  4, }, // Africa/Nairobi
    {  604, 5173,  184564,  4, }, // Africa/Ndjamena
    {  604, 5256,  184568,  4, }, // Africa/Niamey
    {  604, 6766,  184572,  5, }, // Africa/Nouakchott
    {  604, 6690,  184577,  4, }, // Africa/Ouagadougou
    {  604, 5127,  184581,  6, }, // Africa/Porto-Novo
    {  604, 4337,  184587,  5, }, // Africa/Sao_Tome
    {  604, 2732,    7029,  4, }, // Africa/Tripoli
    {  604, 6503,  184592,  3, }, // Africa/Tunis
    {  604, 4054,    7036,  5, }, // Africa/Windhoek
    {  604,  132,  184595,  3, }, // America/Adak
    {  604, 3183,  184598,  5, }, // America/Anchorage
    {  604, 5524,  184603,  4, }, // America/Anguilla
    {  604, 5541,  184607,  4, }, // America/Antigua
    {  604, 4456,  184611,  5, }, // America/Araguaina
    {  604,  166,  184616,  9, }, // America/Argentina/Buenos_Aires
    {  604,   91,    7071,  5, }, // America/Argentina/Catamarca
    {  604,  270,    7076,  4, }, // America/Argentina/Cordoba
    {  604,  441,  184625,  3, }, // America/Argentina/Jujuy
    {  604, 7425,  184628,  5, }, // America/Argentina/La_Rioja
    {  604,  565,  184633,  4, }, // America/Argentina/Mendoza
    {  604, 7452,  184637,  7, }, // America/Argentina/Rio_Gallegos
    {  604, 7483,    7099,  3, }, // America/Argentina/Salta
    {  604, 7507,  184644,  6, }, // America/Argentina/San_Juan
    {  604, 5497,  184650,  6, }, // America/Argentina/San_Luis
    {  604, 7534,    7114,  4, }, // America/Argentina/Tucuman
    {  604, 7560,  184656,  4, }, // America/Argentina/Ushuaia
    {  604, 5557,  184660,  3, }, // America/Aruba
    {  604, 4154,  184663,  6, }, // America/Asuncion
    {  604,  237,  184669,  5, }, // America/Atikokan
    {  604, 3581,  184674,  3, }, // America/Bahia
    {  604, 7586,  184677, 11, }, // America/Bahia_Banderas
    {  604, 5571,  184688,  5, }, // America/Barbados
    {  604, 7609,  184693,  3, }, // America/Belem
    {  604, 5309,  184696,  3, }, // America/Belize
    {  604, 7623,  184699,  9, }, // America/Blanc-Sablon
    {  604, 7644,  184708,  6, }, // America/Boa_Vista
    {  604, 4263,    7175,  3, }, // America/Bogota
    {  604, 7662,  184714,  3, }, // America/Boise
    {  604, 7676,  184717,  8, }, // America/Cambridge_Bay
    {  604, 7698,  184725,  8, }, // America/Campo_Grande
    {  604, 3770,    7197,  4, }, // America/Cancun
    {  604, 4596,    7201,  4, }, // America/Caracas
    {  604, 4247,  184733,  3, }, // America/Cayenne
    {  604, 5398,    7209,  4, }, // America/Cayman
    {  604, 3251,    7213,  3, }, // America/Chicago
    {  604, 7719,  184736,  3, }, // America/Chihuahua
    {  604, 7737,  184739,  9, }, // America/Ciudad_Juarez
    {  604, 5324,  184748,  6, }, // America/Costa_Rica
    {  604, 7759,    7235,  5, }, // America/Creston
    {  604, 3693,  184754,  3, }, // America/Cuiaba
    {  604, 5640,  184757,  4, }, // America/Curacao
    {  604, 7775,    7248,  8, }, // America/Danmarkshavn
    {  604, 7796,    7256,  4, }, // America/Dawson
    {  604, 7811,  184761,  8, }, // America/Dawson_Creek
    {  604,  805,  184769,  4, }, // America/Denver
    {  604, 3351,    7272,  5, }, // America/Detroit
    {  604, 5656,  184773,  4, }, // America/Dominica
    {  604,  893,    7281,  6, }, // America/Edmonton
    {  604, 7832,  184777,  5, }, // America/Eirunepe
    {  604, 5343,  184782,  8, }, // America/El_Salvador
    {  604, 7849,  184790,  8, }, // America/Fort_Nelson
    {  604, 7869,  184798,  5, }, // America/Fortaleza
    {  604, 7887,  184803,  6, }, // America/Glace_Bay
    {  604, 6844,  184809,  5, }, // America/Goose_Bay
    {  604, 4514,  184814,  8, }, // America/Grand_Turk
    {  604, 5687,  184822,  4, }, // America/Grenada
    {  604, 5703,  184826,  4, }, // America/Guadeloupe
    {  604, 3662,    7338,  4, }, // America/Guatemala
    {  604, 4743,  184830,  4, }, // America/Guayaquil
    {  604, 6895,    7347,  3, }, // America/Guyana
    {  604, 1923,    7350,  5, }, // America/Halifax
    {  604, 2221,    7355,  3, }, // America/Havana
    {  604, 7905,  184834,  5, }, // America/Hermosillo
    {  604,  348,  184839,  8, }, // America/Indiana/Indianapolis
    {  604,  481,  184847, 10, }, // America/Indiana/Knox
    {  604, 7924,  184857, 11, }, // America/Indiana/Marengo
    {  604, 7948,  184868, 14, }, // America/Indiana/Petersburg
    {  604, 7975,  184882, 12, }, // America/Indiana/Tell_City
    {  604, 8001,  184894,  9, }, // America/Indiana/Vevay
    {  604, 8023,  184903, 12, }, // America/Indiana/Vincennes
    {  604, 8049,  184915, 11, }, // America/Indiana/Winamac
    {  604, 8073,    7452,  4, }, // America/Inuvik
    {  604,  660,  184926,  5, }, // America/Iqaluit
    {  604, 2665,    7461,  4, }, // America/Jamaica
    {  604, 5282,  184931,  3, }, // America/Juneau
    {  604,  521,  184934,  4, }, // America/Kentucky/Louisville
    {  604, 8088,  184938, 11, }, // America/Kentucky/Monticello
    {  604, 5621,  184949,  7, }, // America/Kralendijk
    {  604, 4278,    7491,  4, }, // America/La_Paz
    {  604, 7132,    7495,  2, }, // America/Lima
    {  604, 3390,  184956,  8, }, // America/Los_Angeles
    {  604, 5849,  184964, 14, }, // America/Lower_Princes
    {  604, 8116,  184978,  3, }, // America/Maceio
    {  604, 8131,  184981,  3, }, // America/Managua
    {  604, 1892,  184984,  4, }, // America/Manaus
    {  604, 5814,  184988,  4, }, // America/Marigot
    {  604, 5722,    7532,  5, }, // America/Martinique
    {  604, 8147,    7537,  5, }, // America/Matamoros
    {  604, 2779,    7542,  5, }, // America/Mazatlan
    {  604, 8165,    7547,  4, }, // America/Menominee
    {  604, 8183,  184992,  3, }, // America/Merida
    {  604, 8198,    7554,  6, }, // America/Metlakatla
    {  604, 2811,  184995,  8, }, // America/Mexico_City
    {  604, 4293,  185003,  4, }, // America/Miquelon
    {  604, 8217,  185007,  5, }, // America/Moncton
    {  604, 8233,  185012,  5, }, // America/Monterrey
    {  604, 4000,  185017,  6, }, // America/Montevideo
    {  604, 5741,  185023,  6, }, // America/Montserrat
    {  604, 5383,    7596,  3, }, // America/Nassau
    {  604, 3294,  185029,  6, }, // America/New_York
    {  604, 8251,  185035,  3, }, // America/Nome
    {  604, 1834,  185038,  4, }, // America/Noronha
    {  604, 8264,  185042, 12, }, // America/North_Dakota/Beulah
    {  604, 8292,  185054, 13, }, // America/North_Dakota/Center
    {  604, 8320,  185067, 15, }, // America/North_Dakota/New_Salem
    {  604,  393,  185082,  2, }, // America/Nuuk
    {  604, 8351,  185084,  4, }, // America/Ojinaga
    {  604, 5413,    7660,  3, }, // America/Panama
    {  604, 6106,    7663,  5, }, // America/Paramaribo
    {  604, 3224,    7668,  4, }, // America/Phoenix
    {  604, 3847,  185088, 10, }, // America/Port-au-Prince
    {  604, 5871,  185098, 10, }, // America/Port_of_Spain
    {  604, 8367,  185108,  7, }, // America/Porto_Velho
    {  604, 5760,  185115,  6, }, // America/Puerto_Rico
    {  604, 3932,  185121,  8, }, // America/Punta_Arenas
    {  604, 8387,  185129,  9, }, // America/Rankin_Inlet
    {  604, 8408,  185138,  3, }, // America/Recife
    {  604, 1979,    7723,  3, }, // America/Regina
    {  604, 8423,  185141,  4, }, // America/Resolute
    {  604,  695,  185145,  7, }, // America/Rio_Branco
    {  604, 8440,  185152,  5, }, // America/Santarem
    {  604, 2165,  185157,  5, }, // America/Santiago
    {  604, 6044,    7748,  8, }, // America/Santo_Domingo
    {  604, 1862,  185162,  6, }, // America/Sao_Paulo
    {  604, 6862,  185168,  7, }, // America/Scoresbysund
    {  604, 8457,    7770,  3, }, // America/Sitka
    {  604, 8471,  185175,  9, }, // America/St_Barthelemy
    {  604, 2045,  185184,  8, }, // America/St_Johns
    {  604, 5780,    7791,  7, }, // America/St_Kitts
    {  604, 5797,  185192,  8, }, // America/St_Lucia
    {  604,  855,  185200,  8, }, // America/St_Thomas
    {  604, 5830,    7813,  9, }, // America/St_Vincent
    {  604, 8493,  185208,  9, }, // America/Swift_Current
    {  604, 5363,  185217,  6, }, // America/Tegucigalpa
    {  604, 5673,  185223,  2, }, // America/Thule
    {  604,  313,  185225,  4, }, // America/Tijuana
    {  604,  608,    7843,  4, }, // America/Toronto
    {  604, 5605,    7847,  4, }, // America/Tortola
    {  604, 2077,  185229,  5, }, // America/Vancouver
    {  604, 2128,  185234,  5, }, // America/Whitehorse
    {  604,  734,  185239,  4, }, // America/Winnipeg
    {  604, 8515,    7866,  4, }, // America/Yakutat
    {  604, 5918,  185243,  3, }, // Antarctica/Casey
    {  604, 6027,  185246,  3, }, // Antarctica/Davis
    {  604, 6066,  185249, 10, }, // Antarctica/DumontDUrville
    {  604, 8531,  185259,  3, }, // Antarctica/Macquarie
    {  604, 7029,  185262,  4, }, // Antarctica/Mawson
    {  604, 7076,  185266,  5, }, // Antarctica/McMurdo
    {  604, 8552,    7898,  4, }, // Antarctica/Palmer
    {  604, 7189,  185271,  3, }, // Antarctica/Rothera
    {  604, 7258,    7905,  3, }, // Antarctica/Syowa
    {  604, 8570,    7908,  3, }, // Antarctica/Troll
    {  604, 7337,    8275,  4, }, // Antarctica/Vostok
    {  604, 1413,  185274,  8, }, // Arctic/Longyearbyen
    {  604, 5487,  185282,  3, }, // Asia/Aden
    {  604, 5297,  185285,  4, }, // Asia/Almaty
    {  604, 3870,  185289,  3, }, // Asia/Amman
    {  604, 5428,  185292,  4, }, // Asia/Anadyr
    {  604, 5440,  185296,  4, }, // Asia/Aqtau
    {  604, 5270,  185300,  4, }, // Asia/Aqtobe
    {  604,  964,  185304,  5, }, // Asia/Ashgabat
    {  604, 8587,  185309,  4, }, // Asia/Atyrau
    {  604, 3509,  185313,  4, }, // Asia/Baghdad
    {  604, 5451,  185317,  4, }, // Asia/Bahrain
    {  604, 3555,    7243,  2, }, // Asia/Baku
    {  604, 4368,  185321,  5, }, // Asia/Bangkok
    {  604, 3473,  185326,  5, }, // Asia/Barnaul
    {  604, 3988,  185331,  4, }, // Asia/Beirut
    {  604, 3680,    7977,  4, }, // Asia/Bishkek
    {  604, 5906,  185335,  4, }, // Asia/Brunei
    {  604, 4503,    7985,  2, }, // Asia/Chita
    {  604, 5963,    7987,  6, }, // Asia/Choibalsan
    {  604, 4413,    7993,  4, }, // Asia/Colombo
    {  604, 4442,    7997,  4, }, // Asia/Damascus
    {  604, 1060,    6893,  2, }, // Asia/Dhaka
    {  604, 6125,    8001,  2, }, // Asia/Dili
    {  604, 3498,    8003,  3, }, // Asia/Dubai
    {  604, 6092,  185339,  4, }, // Asia/Dushanbe
    {  604, 8599,    8010,  5, }, // Asia/Famagusta
    {  604, 8614,  185343,  2, }, // Asia/Gaza
    {  604, 4697,    8017,  4, }, // Asia/Hebron
    {  604, 1226,  185345, 10, }, // Asia/Ho_Chi_Minh
    {  604, 2576,    8032,  7, }, // Asia/Hong_Kong
    {  604, 4673,    8039,  3, }, // Asia/Hovd
    {  604, 4086,    8042,  6, }, // Asia/Irkutsk
    {  604, 6969,    8048,  4, }, // Asia/Jakarta
    {  604, 6955,    8052,  4, }, // Asia/Jayapura
    {  604, 1257,    8057,  5, }, // Asia/Jerusalem
    {  604, 3462,    8062,  3, }, // Asia/Kabul
    {  604, 4218,    8065,  5, }, // Asia/Kamchatka
    {  604, 4141,    8070,  3, }, // Asia/Karachi
    {  604, 1152,    8073,  5, }, // Asia/Kathmandu
    {  604, 8624,    8078,  4, }, // Asia/Khandyga
    {  604,  992,    8082,  4, }, // Asia/Kolkata
    {  604, 4099,    8086,  8, }, // Asia/Krasnoyarsk
    {  604, 4761,  185355,  7, }, // Asia/Kuala_Lumpur
    {  604, 5893,    8102,  4, }, // Asia/Kuching
    {  604, 5464,    8106,  3, }, // Asia/Kuwait
    {  604, 1178,  185362,  3, }, // Asia/Macau
    {  604, 3919,    8112,  4, }, // Asia/Magadan
    {  604, 1316,    8116,  4, }, // Asia/Makassar
    {  604, 7145,    8120,  3, }, // Asia/Manila
    {  604, 6883,    8123,  4, }, // Asia/Muscat
    {  604, 2446,  185365,  4, }, // Asia/Nicosia
    {  604, 8638,  185369,  8, }, // Asia/Novokuznetsk
    {  604, 4037,  185377,  7, }, // Asia/Novosibirsk
    {  604, 4131,    8145,  4, }, // Asia/Omsk
    {  604, 7108,    8149,  3, }, // Asia/Oral
    {  604, 6924,  185384,  6, }, // Asia/Phnom_Penh
    {  604, 8656,  185390,  6, }, // Asia/Pontianak
    {  604, 4116,    8165,  7, }, // Asia/Pyongyang
    {  604, 5476,  185396,  3, }, // Asia/Qatar
    {  604, 8671,    8175,  5, }, // Asia/Qostanay
    {  604, 4171,  185399,  5, }, // Asia/Qyzylorda
    {  604, 3486,  185404,  3, }, // Asia/Riyadh
    {  604, 4310,  185407,  4, }, // Asia/Sakhalin
    {  604, 7208,    8194,  6, }, // Asia/Samarkand
    {  604, 3116,  185411,  3, }, // Asia/Seoul
    {  604, 1020,  185414,  5, }, // Asia/Shanghai
    {  604, 3137,    8207,  5, }, // Asia/Singapore
    {  604, 4199,  185419,  9, }, // Asia/Srednekolymsk
    {  604, 3100,  185428,  4, }, // Asia/Taipei
    {  604, 4683,  185432,  5, }, // Asia/Tashkent
    {  604, 3817,  185437,  4, }, // Asia/Tbilisi
    {  604, 2638,  185441,  4, }, // Asia/Tehran
    {  604, 1284,    8239,  3, }, // Asia/Thimphu
    {  604, 2687,  185445,  3, }, // Asia/Tokyo
    {  604, 4474,    8245,  4, }, // Asia/Tomsk
    {  604, 1346,  185448,  7, }, // Asia/Ulaanbaatar
    {  604, 1126,    8257,  4, }, // Asia/Urumqi
    {  604, 8685,    8261,  6, }, // Asia/Ust-Nera
    {  604, 6940,  185455,  6, }, // Asia/Vientiane
    {  604, 4612,    8272,  7, }, // Asia/Vladivostok
    {  604, 4730,    8279,  5, }, // Asia/Yakutsk
    {  604, 1202,    8284,  4, }, // Asia/Yangon
    {  604, 3785,    8288,  8, }, // Asia/Yekaterinburg
    {  604, 3649,  185461,  4, }, // Asia/Yerevan
    {  604, 3565,  185465,  4, }, // Atlantic/Azores
    {  604, 5588,  185469,  4, }, // Atlantic/Bermuda
    {  604, 6591,    8308,  3, }, // Atlantic/Canary
    {  604, 3629,  185473,  6, }, // Atlantic/Cape_Verde
    {  604, 1379,  185479,  7, }, // Atlantic/Faroe
    {  604, 8699,  185486,  4, }, // Atlantic/Madeira
    {  604, 2614,  185490,  6, }, // Atlantic/Reykjavik
    {  604, 7235,  185496,  8, }, // Atlantic/South_Georgia
    {  604, 6784,  185504,  8, }, // Atlantic/St_Helena
    {  604, 6607,  185512,  4, }, // Atlantic/Stanley
    {  604, 1654,  185516,  5, }, // Australia/Adelaide
    {  604, 1619,  185521,  5, }, // Australia/Brisbane
    {  604, 1783,    8360,  7, }, // Australia/Broken_Hill
    {  604, 1567,    8367,  4, }, // Australia/Darwin
    {  604, 3539,  185526,  3, }, // Australia/Eucla
    {  604, 1500,    8374,  4, }, // Australia/Hobart
    {  604, 8716,    8378,  5, }, // Australia/Lindeman
    {  604, 1531,  185529,  6, }, // Australia/Lord_Howe
    {  604, 1711,  185535,  5, }, // Australia/Melbourne
    {  604, 1746,    8394,  3, }, // Australia/Perth
    {  604, 1447,    8397,  3, }, // Australia/Sydney
    {  604, 6390,  185540,  7, }, // Europe/Amsterdam
    {  604, 6149,  185547,  4, }, // Europe/Andorra
    {  604, 3522,  185551,  6, }, // Europe/Astrakhan
    {  604, 6577,  185557,  4, }, // Europe/Athens
    {  604, 6285,  185561,  5, }, // Europe/Belgrade
    {  604, 4659,    8427,  4, }, // Europe/Berlin
    {  604, 6437,    8431,  6, }, // Europe/Bratislava
    {  604, 6178,  185566,  5, }, // Europe/Brussels
    {  604, 3830,  185571,  5, }, // Europe/Bucharest
    {  604, 3708,  185576,  5, }, // Europe/Budapest
    {  604, 8751,    8452,  5, }, // Europe/Busingen
    {  604, 2475,  185581,  4, }, // Europe/Chisinau
    {  604, 6238,    8461,  6, }, // Europe/Copenhagen
    {  604, 2260,    8467,  4, }, // Europe/Dublin
    {  604, 6256,    8471,  6, }, // Europe/Gibraltar
    {  604, 8767,  185585,  4, }, // Europe/Guernsey
    {  604, 6561,    8481,  5, }, // Europe/Helsinki
    {  604, 8783,  185589,  9, }, // Europe/Isle_of_Man
    {  604, 1097,    8496,  6, }, // Europe/Istanbul
    {  604, 8802,  185598,  3, }, // Europe/Jersey
    {  604, 3881,    8505,  7, }, // Europe/Kaliningrad
    {  604, 8924,    8512,  3, }, // Europe/Kirov
    {  604, 2419,  185601,  3, }, // Europe/Kyiv
    {  604, 3078,    8518,  4, }, // Europe/Lisbon
    {  604, 6455,  185604,  6, }, // Europe/Ljubljana
    {  604, 2393,  185610,  4, }, // Europe/London
    {  604, 6314,    8532,  7, }, // Europe/Luxembourg
    {  604, 4794,    8539,  4, }, // Europe/Madrid
    {  604, 6346,    8543,  3, }, // Europe/Malta
    {  604, 6531,  185614,  5, }, // Europe/Mariehamn
    {  604, 3595,    8551,  4, }, // Europe/Minsk
    {  604, 6359,    8555,  3, }, // Europe/Monaco
    {  604, 3443,    8558,  3, }, // Europe/Moscow
    {  604, 6407,    8561,  3, }, // Europe/Oslo
    {  604, 4186,    8564,  3, }, // Europe/Paris
    {  604, 6373,  185619,  5, }, // Europe/Podgorica
    {  604, 6224,    8572,  3, }, // Europe/Prague
    {  604, 8816,    8575,  2, }, // Europe/Riga
    {  604, 6273,  185485,  2, }, // Europe/Rome
    {  604, 4233,    8579,  3, }, // Europe/Samara
    {  604, 6419,    8582,  6, }, // Europe/San_Marino
    {  604, 6194,  185624,  4, }, // Europe/Sarajevo
    {  604, 4353,    8592,  4, }, // Europe/Saratov
    {  604, 8828,    8596,  6, }, // Europe/Simferopol
    {  604, 6332,    8602,  4, }, // Europe/Skopje
    {  604, 6548,  185628,  3, }, // Europe/Sofia
    {  604, 6472,    8609,  6, }, // Europe/Stockholm
    {  604, 8846,    8615,  3, }, // Europe/Tallinn
    {  604, 6135,  185631,  3, }, // Europe/Tirane
    {  604, 8861,    8621,  7, }, // Europe/Ulyanovsk
    {  604, 6301,    8628,  3, }, // Europe/Vaduz
    {  604, 6516,    8631,  4, }, // Europe/Vatican
    {  604, 6164,  185634,  3, }, // Europe/Vienna
    {  604, 8878,  185637,  5, }, // Europe/Vilnius
    {  604, 4629,  185642,  6, }, // Europe/Volgograd
    {  604, 3055,  185648,  4, }, // Europe/Warsaw
    {  604, 6210,  185652,  4, }, // Europe/Zagreb
    {  604, 6489,    8655,  3, }, // Europe/Zurich
    {  604, 4980,  185656,  7, }, // Indian/Antananarivo
    {  604, 6910,    8665,  3, }, // Indian/Chagos
    {  604, 5979,  185663,  5, }, // Indian/Christmas
    {  604, 5996,    8670,  3, }, // Indian/Cocos
    {  604, 4931,    8673,  3, }, // Indian/Comoro
    {  604, 6624,  185668,  5, }, // Indian/Kerguelen
    {  604, 7223,    8681,  2, }, // Indian/Mahe
    {  604, 7013,  185673,  5, }, // Indian/Maldives
    {  604, 3971,  185678,  5, }, // Indian/Mauritius
    {  604, 5000,  185683,  3, }, // Indian/Mayotte
    {  604, 7174,  185686,  5, }, // Indian/Reunion
    {  604, 4324,  185691,  3, }, // Pacific/Apia
    {  604,  932,    8703,  5, }, // Pacific/Auckland
    {  604, 3608,  185694,  5, }, // Pacific/Bougainville
    {  604, 2863,    8714,  3, }, // Pacific/Chatham
    {  604, 3022,    8717,  2, }, // Pacific/Chuuk
    {  604, 2201,  185699,  7, }, // Pacific/Easter
    {  604, 7323,  185706,  3, }, // Pacific/Efate
    {  604, 7290,    8725,  4, }, // Pacific/Fakaofo
    {  604, 3804,    8729,  2, }, // Pacific/Fiji
    {  604, 7306,  185709,  4, }, // Pacific/Funafuti
    {  604, 6641,    8735,  5, }, // Pacific/Galapagos
    {  604, 6659,  185713,  5, }, // Pacific/Gambier
    {  604, 3724,  185718,  6, }, // Pacific/Guadalcanal
    {  604, 5935,  185724,  2, }, // Pacific/Guam
    {  604, 2929,    8755,  4, }, // Pacific/Honolulu
    {  604, 2897,  185726,  6, }, // Pacific/Kanton
    {  604, 3900,    8765,  5, }, // Pacific/Kiritimati
    {  604, 6982,    8770,  3, }, // Pacific/Kosrae
    {  604, 2708,    8773,  5, }, // Pacific/Kwajalein
    {  604, 4779,  185732,  3, }, // Pacific/Majuro
    {  604, 3953,  185735,  5, }, // Pacific/Marquesas
    {  604, 8909,    8787,  4, }, // Pacific/Midway
    {  604, 7047,    8791,  3, }, // Pacific/Nauru
    {  604, 7095,    7466,  2, }, // Pacific/Niue
    {  604, 4070,    8796,  5, }, // Pacific/Norfolk
    {  604, 7061,  185740,  3, }, // Pacific/Noumea
    {  604, 2991,    8805,  5, }, // Pacific/Pago_Pago
    {  604, 7118,    8810,  3, }, // Pacific/Palau
    {  604, 7157,  185743,  5, }, // Pacific/Pitcairn
    {  604, 2961,  185748,  4, }, // Pacific/Pohnpei
    {  604, 4709,  185752,  8, }, // Pacific/Port_Moresby
    {  604, 6009,    8832,  5, }, // Pacific/Rarotonga
    {  604, 5948,  185760,  4, }, // Pacific/Saipan
    {  604, 7275,    8841,  3, }, // Pacific/Tahiti
    {  604, 6675,    8844,  3, }, // Pacific/Tarawa
    {  604, 4485,    8847,  5, }, // Pacific/Tongatapu
    {  604, 7355,  185764,  2, }, // Pacific/Wake
    {  604, 7368,    8854,  3, }, // Pacific/Wallis
    {  606,   14,       0,  6, }, // Africa/Asmara Tok Pisin/Latin/Papua New Guinea
    {  606, 4337,       6,  8, }, // Africa/Sao_Tome
    {  606, 4154,      14,  8, }, // America/Asuncion
    {  606,  237,      22,  8, }, // America/Atikokan
    {  606, 7586,      30, 17, }, // America/Bahia_Banderas
    {  606, 3770,      47,  6, }, // America/Cancun
    {  606, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  606, 5640,      66,  7, }, // America/Curacao
    {  606,  481,      73, 13, }, // America/Indiana/Knox
    {  606, 7924,      86, 16, }, // America/Indiana/Marengo
    {  606, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  606, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  606, 8001,     139, 14, }, // America/Indiana/Vevay
    {  606, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  606, 8049,     171, 16, }, // America/Indiana/Winamac
    {  606, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  606, 5849,     207, 22, }, // America/Lower_Princes
    {  606, 8183,     229,  6, }, // America/Merida
    {  606, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  606, 8292,     255, 20, }, // America/North_Dakota/Center
    {  606, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  606,  393,     298,  4, }, // America/Nuuk
    {  606, 6862,     302, 16, }, // America/Scoresbysund
    {  606, 8471,     318, 14, }, // America/St_Barthelemy
    {  606, 2045,     332, 10, }, // America/St_Johns
    {  606, 5780,     342,  9, }, // America/St_Kitts
    {  606, 5797,     351,  9, }, // America/St_Lucia
    {  606,  855,     360, 10, }, // America/St_Thomas
    {  606, 5830,     370, 11, }, // America/St_Vincent
    {  606, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  606, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  606, 1152,     413,  9, }, // Asia/Kathmandu
    {  606,  992,     422,  7, }, // Asia/Kolkata
    {  606, 1178,     429,  5, }, // Asia/Macau
    {  606, 1202,     442,  6, }, // Asia/Yangon
    {  606, 1379,     448,  5, }, // Atlantic/Faroe
    {  606, 6784,     453, 10, }, // Atlantic/St_Helena
    {  606, 2419,     463,  4, }, // Europe/Kyiv
    {  606, 7174,     467,  7, }, // Indian/Reunion
    {  606, 3022,     474,  5, }, // Pacific/Chuuk
    {  606, 2961,     479,  7, }, // Pacific/Pohnpei
    {  607,   14,       0,  6, }, // Africa/Asmara Tongan/Latin/Tonga
    {  607, 4337,   57267,  8, }, // Africa/Sao_Tome
    {  607, 4154,      14,  8, }, // America/Asuncion
    {  607,  237,      22,  8, }, // America/Atikokan
    {  607, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  607, 3770,    3861,  6, }, // America/Cancun
    {  607, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  607, 5640,      66,  7, }, // America/Curacao
    {  607,  481,      73, 13, }, // America/Indiana/Knox
    {  607, 7924,      86, 16, }, // America/Indiana/Marengo
    {  607, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  607, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  607, 8001,     139, 14, }, // America/Indiana/Vevay
    {  607, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  607, 8049,     171, 16, }, // America/Indiana/Winamac
    {  607, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  607, 5849,     207, 22, }, // America/Lower_Princes
    {  607, 8183,    3899,  6, }, // America/Merida
    {  607, 3294,  185766,  9, }, // America/New_York
    {  607, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  607, 8292,     255, 20, }, // America/North_Dakota/Center
    {  607, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  607,  393,     298,  4, }, // America/Nuuk
    {  607, 6862,     302, 16, }, // America/Scoresbysund
    {  607, 8471,     318, 14, }, // America/St_Barthelemy
    {  607, 2045,     332, 10, }, // America/St_Johns
    {  607, 5780,     342,  9, }, // America/St_Kitts
    {  607, 5797,     351,  9, }, // America/St_Lucia
    {  607,  855,     360, 10, }, // America/St_Thomas
    {  607, 5830,     370, 11, }, // America/St_Vincent
    {  607, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  607, 4697,  185775,  8, }, // Asia/Hebron
    {  607, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  607, 1257,  185783, 10, }, // Asia/Jerusalem
    {  607, 1152,     413,  9, }, // Asia/Kathmandu
    {  607,  992,     422,  7, }, // Asia/Kolkata
    {  607, 1178,    4222,  5, }, // Asia/Macau
    {  607, 1202,   56146,  7, }, // Asia/Yangon
    {  607, 1379,     448,  5, }, // Atlantic/Faroe
    {  607, 6784,     453, 10, }, // Atlantic/St_Helena
    {  607, 1654,  185793,  8, }, // Australia/Adelaide
    {  607, 1619,  185801, 10, }, // Australia/Brisbane
    {  607, 1711,  185811,  9, }, // Australia/Melbourne
    {  607, 1447,  185820,  4, }, // Australia/Sydney
    {  607, 6577,  185824,  8, }, // Europe/Athens
    {  607, 2419,    6486,  4, }, // Europe/Kyiv
    {  607, 2393,  185832,  8, }, // Europe/London
    {  607, 3443,  185840,  7, }, // Europe/Moscow
    {  607, 4186,  185847,  6, }, // Europe/Paris
    {  607, 6273,  185853,  4, }, // Europe/Rome
    {  607, 6516,   62433,  8, }, // Europe/Vatican
    {  607, 7174,     467,  7, }, // Indian/Reunion
    {  607,  932,  185857,  9, }, // Pacific/Auckland
    {  607, 3608,  185866, 10, }, // Pacific/Bougainville
    {  607, 3022,  185876,  4, }, // Pacific/Chuuk
    {  607, 2201,  185880,  7, }, // Pacific/Easter
    {  607, 3804,  185887,  4, }, // Pacific/Fiji
    {  607, 6659,  185891,  7, }, // Pacific/Gambier
    {  607, 3724,  185898, 11, }, // Pacific/Guadalcanal
    {  607, 5935,  185909,  5, }, // Pacific/Guam
    {  607, 2929,    4512,  8, }, // Pacific/Honolulu
    {  607, 2897,  185914, 10, }, // Pacific/Kanton
    {  607, 3900,  185924, 10, }, // Pacific/Kiritimati
    {  607, 6982,  185934,  7, }, // Pacific/Kosrae
    {  607, 2708,  185941,  9, }, // Pacific/Kwajalein
    {  607, 4779,  185950,  6, }, // Pacific/Majuro
    {  607, 3953,  185956,  9, }, // Pacific/Marquesas
    {  607, 8909,  185965,  7, }, // Pacific/Midway
    {  607, 7047,  185972,  5, }, // Pacific/Nauru
    {  607, 7095,  185977,  4, }, // Pacific/Niue
    {  607, 4070,  185981,  7, }, // Pacific/Norfolk
    {  607, 2991,  185988, 10, }, // Pacific/Pago_Pago
    {  607, 7157,  185998,  8, }, // Pacific/Pitcairn
    {  607, 2961,  186006,  6, }, // Pacific/Pohnpei
    {  607, 4709,  186012, 17, }, // Pacific/Port_Moresby
    {  607, 6009,  186029,  9, }, // Pacific/Rarotonga
    {  607, 5948,  186038,  8, }, // Pacific/Saipan
    {  607, 7275,  186046,  6, }, // Pacific/Tahiti
    {  607, 6675,  186052,  6, }, // Pacific/Tarawa
    {  607, 7355,  186058,  4, }, // Pacific/Wake
    {  607, 7368,  186062,  5, }, // Pacific/Wallis
    {  608,   14,       0,  6, }, // Africa/Asmara Tsonga/Latin/South Africa
    {  608, 4337,       6,  8, }, // Africa/Sao_Tome
    {  608, 4154,      14,  8, }, // America/Asuncion
    {  608,  237,      22,  8, }, // America/Atikokan
    {  608, 7586,      30, 17, }, // America/Bahia_Banderas
    {  608, 3770,      47,  6, }, // America/Cancun
    {  608, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  608, 5640,      66,  7, }, // America/Curacao
    {  608,  481,      73, 13, }, // America/Indiana/Knox
    {  608, 7924,      86, 16, }, // America/Indiana/Marengo
    {  608, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  608, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  608, 8001,     139, 14, }, // America/Indiana/Vevay
    {  608, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  608, 8049,     171, 16, }, // America/Indiana/Winamac
    {  608, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  608, 5849,     207, 22, }, // America/Lower_Princes
    {  608, 8183,     229,  6, }, // America/Merida
    {  608, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  608, 8292,     255, 20, }, // America/North_Dakota/Center
    {  608, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  608,  393,     298,  4, }, // America/Nuuk
    {  608, 6862,     302, 16, }, // America/Scoresbysund
    {  608, 8471,     318, 14, }, // America/St_Barthelemy
    {  608, 2045,     332, 10, }, // America/St_Johns
    {  608, 5780,     342,  9, }, // America/St_Kitts
    {  608, 5797,     351,  9, }, // America/St_Lucia
    {  608,  855,     360, 10, }, // America/St_Thomas
    {  608, 5830,     370, 11, }, // America/St_Vincent
    {  608, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  608, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  608, 1152,     413,  9, }, // Asia/Kathmandu
    {  608,  992,     422,  7, }, // Asia/Kolkata
    {  608, 1178,     429,  5, }, // Asia/Macau
    {  608, 1202,     442,  6, }, // Asia/Yangon
    {  608, 1379,     448,  5, }, // Atlantic/Faroe
    {  608, 6784,     453, 10, }, // Atlantic/St_Helena
    {  608, 2419,     463,  4, }, // Europe/Kyiv
    {  608, 7174,     467,  7, }, // Indian/Reunion
    {  608, 3022,     474,  5, }, // Pacific/Chuuk
    {  608, 2961,     479,  7, }, // Pacific/Pohnpei
    {  609,   14,       0,  6, }, // Africa/Asmara Tswana/Latin/South Africa
    {  609, 4337,       6,  8, }, // Africa/Sao_Tome
    {  609, 4154,      14,  8, }, // America/Asuncion
    {  609,  237,      22,  8, }, // America/Atikokan
    {  609, 7586,      30, 17, }, // America/Bahia_Banderas
    {  609, 3770,      47,  6, }, // America/Cancun
    {  609, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  609, 5640,      66,  7, }, // America/Curacao
    {  609,  481,      73, 13, }, // America/Indiana/Knox
    {  609, 7924,      86, 16, }, // America/Indiana/Marengo
    {  609, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  609, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  609, 8001,     139, 14, }, // America/Indiana/Vevay
    {  609, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  609, 8049,     171, 16, }, // America/Indiana/Winamac
    {  609, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  609, 5849,     207, 22, }, // America/Lower_Princes
    {  609, 8183,     229,  6, }, // America/Merida
    {  609, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  609, 8292,     255, 20, }, // America/North_Dakota/Center
    {  609, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  609,  393,     298,  4, }, // America/Nuuk
    {  609, 6862,     302, 16, }, // America/Scoresbysund
    {  609, 8471,     318, 14, }, // America/St_Barthelemy
    {  609, 2045,     332, 10, }, // America/St_Johns
    {  609, 5780,     342,  9, }, // America/St_Kitts
    {  609, 5797,     351,  9, }, // America/St_Lucia
    {  609,  855,     360, 10, }, // America/St_Thomas
    {  609, 5830,     370, 11, }, // America/St_Vincent
    {  609, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  609, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  609, 1152,     413,  9, }, // Asia/Kathmandu
    {  609,  992,     422,  7, }, // Asia/Kolkata
    {  609, 1178,     429,  5, }, // Asia/Macau
    {  609, 1202,     442,  6, }, // Asia/Yangon
    {  609, 1379,     448,  5, }, // Atlantic/Faroe
    {  609, 6784,     453, 10, }, // Atlantic/St_Helena
    {  609, 2419,     463,  4, }, // Europe/Kyiv
    {  609, 7174,     467,  7, }, // Indian/Reunion
    {  609, 3022,     474,  5, }, // Pacific/Chuuk
    {  609, 2961,     479,  7, }, // Pacific/Pohnpei
    {  611, 6723,    4550,  4, }, // Africa/Accra Turkish/Latin/Turkey
    {  611, 7383,  186067,  7, }, // Africa/Algiers
    {  611,   14,       0,  6, }, // Africa/Asmara
    {  611, 5189,   19729,  9, }, // Africa/Brazzaville
    {  611, 2242,  186074,  6, }, // Africa/Cairo
    {  611, 4019,   31744, 10, }, // Africa/Casablanca
    {  611, 7412,  112836,  5, }, // Africa/Ceuta
    {  611, 6736,    4637,  7, }, // Africa/Conakry
    {  611, 5032,  186080, 10, }, // Africa/Dar_es_Salaam
    {  611, 4945,   19771,  6, }, // Africa/Djibouti
    {  611, 5068,  186090,  5, }, // Africa/El_Aaiun
    {  611, 4401,  112863,  4, }, // Africa/Juba
    {  611, 4426,   61879,  6, }, // Africa/Khartoum
    {  611, 5208,   19819,  7, }, // Africa/Kinshasa
    {  611, 5238,    4699,  8, }, // Africa/Libreville
    {  611, 5015,  186095,  8, }, // Africa/Mogadishu
    {  611, 4337,       6,  8, }, // Africa/Sao_Tome
    {  611, 2732,   97057,  7, }, // Africa/Tripoli
    {  611, 6503,  186103,  5, }, // Africa/Tunis
    {  611, 4154,      14,  8, }, // America/Asuncion
    {  611,  237,      22,  8, }, // America/Atikokan
    {  611, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  611, 3770,    3861,  6, }, // America/Cancun
    {  611, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  611, 5324,   20026, 10, }, // America/Costa_Rica
    {  611, 5640,      66,  7, }, // America/Curacao
    {  611, 5656,   20071,  8, }, // America/Dominica
    {  611,  481,      73, 13, }, // America/Indiana/Knox
    {  611, 7924,      86, 16, }, // America/Indiana/Marengo
    {  611, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  611, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  611, 8001,     139, 14, }, // America/Indiana/Vevay
    {  611, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  611, 8049,     171, 16, }, // America/Indiana/Winamac
    {  611, 2665,    3892,  7, }, // America/Jamaica
    {  611, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  611, 5849,     207, 22, }, // America/Lower_Princes
    {  611, 8183,    3899,  6, }, // America/Merida
    {  611, 8264,  186108, 20, }, // America/North_Dakota/Beulah
    {  611, 8292,  186128, 20, }, // America/North_Dakota/Center
    {  611, 8320,  186148, 23, }, // America/North_Dakota/New_Salem
    {  611,  393,     298,  4, }, // America/Nuuk
    {  611, 5760,  159292, 10, }, // America/Puerto_Rico
    {  611, 6862,     302, 16, }, // America/Scoresbysund
    {  611, 8471,  113081, 16, }, // America/St_Barthelemy
    {  611, 2045,     332, 10, }, // America/St_Johns
    {  611, 5780,     342,  9, }, // America/St_Kitts
    {  611, 5797,     351,  9, }, // America/St_Lucia
    {  611,  855,     360, 10, }, // America/St_Thomas
    {  611, 5830,     370, 11, }, // America/St_Vincent
    {  611, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  611, 7258,   63470,  5, }, // Antarctica/Syowa
    {  611, 5297,   20713,  6, }, // Asia/Almaty
    {  611, 5428,   20719,  6, }, // Asia/Anadyr
    {  611, 5440,  186171,  5, }, // Asia/Aqtau
    {  611, 5270,   61967,  6, }, // Asia/Aqtobe
    {  611,  964,  186176,  7, }, // Asia/Ashgabat
    {  611, 8587,  186183,  6, }, // Asia/Atyrau
    {  611, 3509,  186189,  6, }, // Asia/Baghdad
    {  611, 5451,  186195,  7, }, // Asia/Bahrain
    {  611, 3555,  186202,  4, }, // Asia/Baku
    {  611, 3988,   20762,  6, }, // Asia/Beirut
    {  611, 3680,   20768,  6, }, // Asia/Bishkek
    {  611, 4503,    5936,  4, }, // Asia/Chita
    {  611, 5963,   20780,  9, }, // Asia/Choibalsan
    {  611, 4413,    5949,  7, }, // Asia/Colombo
    {  611, 4442,  113163,  3, }, // Asia/Damascus
    {  611, 1060,   90078,  5, }, // Asia/Dhaka
    {  611, 6092,  186206,  7, }, // Asia/Dushanbe
    {  611, 8599,  186213, 10, }, // Asia/Famagusta
    {  611, 8614,  186223,  5, }, // Asia/Gaza
    {  611, 4697,  186228,  8, }, // Asia/Hebron
    {  611, 1226,  186236, 17, }, // Asia/Ho_Chi_Minh
    {  611, 4086,   20843,  7, }, // Asia/Irkutsk
    {  611, 6969,   20850,  7, }, // Asia/Jakarta
    {  611, 1257,  186253,  5, }, // Asia/Jerusalem
    {  611, 3462,   20874,  5, }, // Asia/Kabul
    {  611, 4218,   20879,  8, }, // Asia/Kamchatka
    {  611, 4141,    6012,  6, }, // Asia/Karachi
    {  611, 1152,    6018,  8, }, // Asia/Kathmandu
    {  611, 8624,   31995,  7, }, // Asia/Khandyga
    {  611,  992,  186258,  7, }, // Asia/Kolkata
    {  611, 5893,    6052,  6, }, // Asia/Kuching
    {  611, 5464,  186265,  6, }, // Asia/Kuwait
    {  611, 1178,    6064,  5, }, // Asia/Macau
    {  611, 6883,   25325,  6, }, // Asia/Muscat
    {  611, 2446,  186271,  7, }, // Asia/Nicosia
    {  611, 5476,    4233,  5, }, // Asia/Qatar
    {  611, 8671,     434,  8, }, // Asia/Qostanay
    {  611, 4171,  186278,  9, }, // Asia/Qyzylorda
    {  611, 3486,   19265,  5, }, // Asia/Riyadh
    {  611, 4310,   32047,  7, }, // Asia/Sakhalin
    {  611, 7208,  113301,  9, }, // Asia/Samarkand
    {  611, 3116,    6131,  4, }, // Asia/Seoul
    {  611, 1020,  113314,  7, }, // Asia/Shanghai
    {  611, 3137,   19291,  8, }, // Asia/Singapore
    {  611, 4683,  113335,  7, }, // Asia/Tashkent
    {  611, 3817,   76718,  6, }, // Asia/Tbilisi
    {  611, 2638,  186287,  6, }, // Asia/Tehran
    {  611, 1346,  186293, 10, }, // Asia/Ulaanbaatar
    {  611, 1126,   21017,  6, }, // Asia/Urumqi
    {  611, 1202,     442,  6, }, // Asia/Yangon
    {  611, 3649,  123737,  6, }, // Asia/Yerevan
    {  611, 3565,  186303, 12, }, // Atlantic/Azores
    {  611, 6591,  186315, 15, }, // Atlantic/Canary
    {  611, 1379,     448,  5, }, // Atlantic/Faroe
    {  611, 8699,  186330, 15, }, // Atlantic/Madeira
    {  611, 7235,  186345, 13, }, // Atlantic/South_Georgia
    {  611, 6784,     453, 10, }, // Atlantic/St_Helena
    {  611, 1447,   21149,  6, }, // Australia/Sydney
    {  611, 3522,   32115,  8, }, // Europe/Astrakhan
    {  611, 6577,   32123,  5, }, // Europe/Athens
    {  611, 6285,   25465,  7, }, // Europe/Belgrade
    {  611, 6178,  186358,  7, }, // Europe/Brussels
    {  611, 3830,  186365,  6, }, // Europe/Bucharest
    {  611, 3708,  186371,  9, }, // Europe/Budapest
    {  611, 8751,   25487,  8, }, // Europe/Busingen
    {  611, 2475,  186380,  7, }, // Europe/Chisinau
    {  611, 6238,    4376,  8, }, // Europe/Copenhagen
    {  611, 6256,  186387, 11, }, // Europe/Gibraltar
    {  611, 8783,  186398,  9, }, // Europe/Isle_of_Man
    {  611, 1097,   21235,  8, }, // Europe/Istanbul
    {  611, 2419,    6486,  4, }, // Europe/Kyiv
    {  611, 3078,  150331,  6, }, // Europe/Lisbon
    {  611, 2393,   93889,  6, }, // Europe/London
    {  611, 6314,  186407, 10, }, // Europe/Luxembourg
    {  611, 6359,    6525,  6, }, // Europe/Monaco
    {  611, 3443,   62842,  7, }, // Europe/Moscow
    {  611, 6224,    6545,  4, }, // Europe/Prague
    {  611, 6273,   19581,  4, }, // Europe/Rome
    {  611, 6194,  113598, 10, }, // Europe/Sarajevo
    {  611, 6332,  186417,  5, }, // Europe/Skopje
    {  611, 6548,   97885,  5, }, // Europe/Sofia
    {  611, 6472,    6586,  8, }, // Europe/Stockholm
    {  611, 6135,    6599,  5, }, // Europe/Tirane
    {  611, 6516,    6614,  7, }, // Europe/Vatican
    {  611, 6164,  113630,  6, }, // Europe/Vienna
    {  611, 3055,  186422,  7, }, // Europe/Warsaw
    {  611, 6489,  186429,  5, }, // Europe/Zurich
    {  611, 4931,    6656,  5, }, // Indian/Comoro
    {  611, 7013,  186434,  9, }, // Indian/Maldives
    {  611, 7174,     467,  7, }, // Indian/Reunion
    {  611, 3022,     474,  5, }, // Pacific/Chuuk
    {  611, 2201,  186443, 14, }, // Pacific/Easter
    {  611, 2929,    4512,  8, }, // Pacific/Honolulu
    {  611, 2897,    4520,  9, }, // Pacific/Kanton
    {  611, 3953,  186457, 14, }, // Pacific/Marquesas
    {  611, 2961,     479,  7, }, // Pacific/Pohnpei
    {  613, 6751,  186471,  6, }, // Africa/Abidjan Turkmen/Latin/Turkmenistan
    {  613, 6723,   19687,  5, }, // Africa/Accra
    {  613, 4961,   63018, 11, }, // Africa/Addis_Ababa
    {  613, 7383,   56099,  5, }, // Africa/Algiers
    {  613,   14,       0,  6, }, // Africa/Asmara
    {  613, 5159,   24908,  5, }, // Africa/Bangui
    {  613, 7398,    4583,  5, }, // Africa/Bissau
    {  613, 4859,  186477,  8, }, // Africa/Blantyre
    {  613, 5189,  186485,  9, }, // Africa/Brazzaville
    {  613, 2242,   24913,  4, }, // Africa/Cairo
    {  613, 4019,   19744, 10, }, // Africa/Casablanca
    {  613, 7412,   19754,  5, }, // Africa/Ceuta
    {  613, 6736,    4637,  7, }, // Africa/Conakry
    {  613, 5032,    4644, 12, }, // Africa/Dar_es_Salaam
    {  613, 4945,  151111,  6, }, // Africa/Djibouti
    {  613, 5145,   19777,  5, }, // Africa/Douala
    {  613, 5068,  186494,  7, }, // Africa/El_Aaiun
    {  613, 6816,    4670,  7, }, // Africa/Freetown
    {  613, 4381,  186501, 12, }, // Africa/Johannesburg
    {  613, 4426,   61879,  6, }, // Africa/Khartoum
    {  613, 5208,   19819,  7, }, // Africa/Kinshasa
    {  613, 5238,  186513,  8, }, // Africa/Libreville
    {  613, 4841,   19831,  9, }, // Africa/Lubumbashi
    {  613, 5015,  186521,  8, }, // Africa/Mogadishu
    {  613, 6997,  186529,  9, }, // Africa/Monrovia
    {  613, 3755,  186538,  7, }, // Africa/Nairobi
    {  613, 5173,  186545,  6, }, // Africa/Ndjamena
    {  613, 5256,  186551,  6, }, // Africa/Niamey
    {  613, 6766,  186557,  7, }, // Africa/Nouakchott
    {  613, 6690,   24967,  8, }, // Africa/Ouagadougou
    {  613, 5127,  186564, 10, }, // Africa/Porto-Novo
    {  613, 4337,  186574,  8, }, // Africa/Sao_Tome
    {  613, 4054,   55328,  7, }, // Africa/Windhoek
    {  613,  132,  186582, 10, }, // America/Adak
    {  613, 3183,  120650,  8, }, // America/Anchorage
    {  613, 5524,  186592,  7, }, // America/Anguilla
    {  613, 4456,  186599,  9, }, // America/Araguaina
    {  613,  166,  186608, 12, }, // America/Argentina/Buenos_Aires
    {  613,   91,    4788,  9, }, // America/Argentina/Catamarca
    {  613,  270,  186620,  7, }, // America/Argentina/Cordoba
    {  613,  441,  186627,  5, }, // America/Argentina/Jujuy
    {  613, 7425,  186632,  8, }, // America/Argentina/La_Rioja
    {  613,  565,   96160,  7, }, // America/Argentina/Mendoza
    {  613, 7452,    4817, 11, }, // America/Argentina/Rio_Gallegos
    {  613, 7507,    4833,  8, }, // America/Argentina/San_Juan
    {  613, 5497,  186640,  8, }, // America/Argentina/San_Luis
    {  613, 7534,    4850,  7, }, // America/Argentina/Tucuman
    {  613, 7560,  186648,  6, }, // America/Argentina/Ushuaia
    {  613, 4154,  186654,  8, }, // America/Asuncion
    {  613,  237,      22,  8, }, // America/Atikokan
    {  613, 3581,  186662,  5, }, // America/Bahia
    {  613, 7586,  186667, 17, }, // America/Bahia_Banderas
    {  613, 7609,  117999,  5, }, // America/Belem
    {  613, 5309,    4891,  5, }, // America/Belize
    {  613, 7623,    4897, 12, }, // America/Blanc-Sablon
    {  613, 7644,  186684,  9, }, // America/Boa_Vista
    {  613, 7662,  186693,  5, }, // America/Boise
    {  613, 7676,  186698, 11, }, // America/Cambridge_Bay
    {  613, 7698,  186709, 12, }, // America/Campo_Grande
    {  613, 3770,    4959,  6, }, // America/Cancun
    {  613, 4596,    4965,  7, }, // America/Caracas
    {  613, 4247,  186721,  7, }, // America/Cayenne
    {  613, 5398,  186728, 14, }, // America/Cayman
    {  613, 3251,    4984,  6, }, // America/Chicago
    {  613, 7719,   20018,  8, }, // America/Chihuahua
    {  613, 7737,  186742, 13, }, // America/Ciudad_Juarez
    {  613, 5324,  186755, 10, }, // America/Costa_Rica
    {  613, 7759,    5020,  7, }, // America/Creston
    {  613, 3693,  186765,  6, }, // America/Cuiaba
    {  613, 5640,  186771,  8, }, // America/Curacao
    {  613, 7775,  186779, 12, }, // America/Danmarkshavn
    {  613, 7796,    5053,  6, }, // America/Dawson
    {  613, 7811,  186791, 11, }, // America/Dawson_Creek
    {  613,  805,  186802,  6, }, // America/Denver
    {  613, 3351,  186808,  7, }, // America/Detroit
    {  613, 5656,   20071,  8, }, // America/Dominica
    {  613, 7832,  186815,  8, }, // America/Eirunepe
    {  613, 5343,  149751,  8, }, // America/El_Salvador
    {  613, 7887,  186823,  9, }, // America/Glace_Bay
    {  613, 6844,  186832,  7, }, // America/Goose_Bay
    {  613, 4514,  186839, 10, }, // America/Grand_Turk
    {  613, 5703,  149759,  9, }, // America/Guadeloupe
    {  613, 3662,  130126,  9, }, // America/Guatemala
    {  613, 4743,  186849,  7, }, // America/Guayaquil
    {  613, 6895,  186856,  6, }, // America/Guyana
    {  613, 1923,  186862,  8, }, // America/Halifax
    {  613, 2221,  186870,  6, }, // America/Havana
    {  613, 7905,  186876,  8, }, // America/Hermosillo
    {  613,  481,  186884, 13, }, // America/Indiana/Knox
    {  613, 7924,      86, 16, }, // America/Indiana/Marengo
    {  613, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  613, 7975,    5245, 18, }, // America/Indiana/Tell_City
    {  613, 8001,  186897, 14, }, // America/Indiana/Vevay
    {  613, 8023,  186911, 16, }, // America/Indiana/Vincennes
    {  613, 8049,   72194, 16, }, // America/Indiana/Winamac
    {  613, 8073,  186927,  6, }, // America/Inuvik
    {  613,  660,    5310,  7, }, // America/Iqaluit
    {  613, 2665,  186933,  7, }, // America/Jamaica
    {  613, 5282,  118428,  5, }, // America/Juneau
    {  613,  521,  186940,  8, }, // America/Kentucky/Louisville
    {  613, 8088,  186948, 19, }, // America/Kentucky/Monticello
    {  613, 5621,  186967, 10, }, // America/Kralendijk
    {  613, 4278,  186977,  6, }, // America/La_Paz
    {  613, 3390,  186983, 11, }, // America/Los_Angeles
    {  613, 5849,  186994, 21, }, // America/Lower_Princes
    {  613, 8116,  187015,  6, }, // America/Maceio
    {  613, 5814,   60430,  6, }, // America/Marigot
    {  613, 5722,   25027,  9, }, // America/Martinique
    {  613, 8165,    5409,  8, }, // America/Menominee
    {  613, 8183,    3899,  6, }, // America/Merida
    {  613, 2811,  118561,  6, }, // America/Mexico_City
    {  613, 4293,    5434,  7, }, // America/Miquelon
    {  613, 8217,    5441,  7, }, // America/Moncton
    {  613, 8233,  187021,  9, }, // America/Monterrey
    {  613, 4000,  187030, 10, }, // America/Montevideo
    {  613, 5741,  151174,  9, }, // America/Montserrat
    {  613, 3294,  187040,  8, }, // America/New_York
    {  613, 8251,   20373,  3, }, // America/Nome
    {  613, 8264,  187048, 24, }, // America/North_Dakota/Beulah
    {  613, 8292,  187072, 24, }, // America/North_Dakota/Center
    {  613, 8320,  187096, 28, }, // America/North_Dakota/New_Salem
    {  613,  393,     298,  4, }, // America/Nuuk
    {  613, 8351,    5561,  7, }, // America/Ojinaga
    {  613, 3224,    5568,  6, }, // America/Phoenix
    {  613, 3847,  158616, 12, }, // America/Port-au-Prince
    {  613, 5871,  187124, 13, }, // America/Port_of_Spain
    {  613, 8367,  187137, 11, }, // America/Porto_Velho
    {  613, 5760,  187148, 11, }, // America/Puerto_Rico
    {  613, 3932,    5607, 12, }, // America/Punta_Arenas
    {  613, 8387,   92307, 12, }, // America/Rankin_Inlet
    {  613, 8408,  118755,  6, }, // America/Recife
    {  613, 1979,  187159,  7, }, // America/Regina
    {  613, 8423,  187166,  8, }, // America/Resolute
    {  613,  695,  187174, 10, }, // America/Rio_Branco
    {  613, 2165,  187184,  8, }, // America/Santiago
    {  613, 6044,    5647, 13, }, // America/Santo_Domingo
    {  613, 1862,  187192,  9, }, // America/Sao_Paulo
    {  613, 6862,  187201, 16, }, // America/Scoresbysund
    {  613, 8471,    5681, 13, }, // America/St_Barthelemy
    {  613, 2045,  187217,  9, }, // America/St_Johns
    {  613, 5780,  187226,  9, }, // America/St_Kitts
    {  613, 5797,  187235, 12, }, // America/St_Lucia
    {  613,  855,  187247, 10, }, // America/St_Thomas
    {  613, 5830,  187257, 12, }, // America/St_Vincent
    {  613, 8493,  187269, 12, }, // America/Swift_Current
    {  613, 5363,    5759, 11, }, // America/Tegucigalpa
    {  613, 5673,  118909,  4, }, // America/Thule
    {  613,  313,    5775,  7, }, // America/Tijuana
    {  613, 2077,  187281,  8, }, // America/Vancouver
    {  613, 2128,  187289,  8, }, // America/Whitehorse
    {  613, 8515,  187297,  7, }, // America/Yakutat
    {  613, 5918,  187304,  5, }, // Antarctica/Casey
    {  613, 6027,  187309,  6, }, // Antarctica/Davis
    {  613, 6066,  187315, 15, }, // Antarctica/DumontDUrville
    {  613, 8531,  187330,  8, }, // Antarctica/Macquarie
    {  613, 7029,   20686,  6, }, // Antarctica/Mawson
    {  613, 7076,  187338,  9, }, // Antarctica/McMurdo
    {  613, 7189,  118999,  6, }, // Antarctica/Rothera
    {  613, 7258,  187347,  5, }, // Antarctica/Syowa
    {  613, 8570,  119009,  4, }, // Antarctica/Troll
    {  613, 7337,    4048,  6, }, // Antarctica/Vostok
    {  613, 1413,  187352,  6, }, // Arctic/Longyearbyen
    {  613, 5440,    5906,  5, }, // Asia/Aqtau
    {  613, 5270,    5911,  6, }, // Asia/Aqtobe
    {  613,  964,   42826,  7, }, // Asia/Ashgabat
    {  613, 3509,  187358,  6, }, // Asia/Baghdad
    {  613, 5451,  187364,  7, }, // Asia/Bahrain
    {  613, 3988,  187371,  6, }, // Asia/Beirut
    {  613, 3680,   20768,  6, }, // Asia/Bishkek
    {  613, 5906,  187377,  6, }, // Asia/Brunei
    {  613, 4503,    5936,  4, }, // Asia/Chita
    {  613, 5963,  187383,  9, }, // Asia/Choibalsan
    {  613, 4413,    5949,  7, }, // Asia/Colombo
    {  613, 4442,    4102,  6, }, // Asia/Damascus
    {  613, 1060,   90078,  5, }, // Asia/Dhaka
    {  613, 3498,  187392,  5, }, // Asia/Dubai
    {  613, 6092,  186206,  7, }, // Asia/Dushanbe
    {  613, 4697,  187397,  6, }, // Asia/Hebron
    {  613, 1226,  187403,  7, }, // Asia/Ho_Chi_Minh
    {  613, 2576,  187410,  7, }, // Asia/Hong_Kong
    {  613, 4673,  187417,  4, }, // Asia/Hovd
    {  613, 6955,  187421,  8, }, // Asia/Jayapura
    {  613, 1257,  187429, 10, }, // Asia/Jerusalem
    {  613, 4218,   20879,  8, }, // Asia/Kamchatka
    {  613, 4141,    6012,  6, }, // Asia/Karachi
    {  613, 1152,    6018,  8, }, // Asia/Kathmandu
    {  613, 8624,   62723,  7, }, // Asia/Khandyga
    {  613,  992,   62730,  8, }, // Asia/Kolkata
    {  613, 4099,  187439, 11, }, // Asia/Krasnoyarsk
    {  613, 4761,    6040, 12, }, // Asia/Kuala_Lumpur
    {  613, 5893,    6052,  6, }, // Asia/Kuching
    {  613, 5464,  187450,  6, }, // Asia/Kuwait
    {  613, 1178,    6064,  5, }, // Asia/Macau
    {  613, 1316,    6069,  7, }, // Asia/Makassar
    {  613, 6883,   25325,  6, }, // Asia/Muscat
    {  613, 2446,  187456,  8, }, // Asia/Nicosia
    {  613, 8638,  187464, 11, }, // Asia/Novokuznetsk
    {  613, 4037,   76692, 11, }, // Asia/Novosibirsk
    {  613, 6924,   61058,  7, }, // Asia/Phnom_Penh
    {  613, 4116,  187475,  7, }, // Asia/Pyongyang
    {  613, 5476,    4233,  5, }, // Asia/Qatar
    {  613, 8671,  187482,  8, }, // Asia/Qostanay
    {  613, 4171,  187490,  9, }, // Asia/Qyzylorda
    {  613, 3486,  187499,  8, }, // Asia/Riyadh
    {  613, 4310,   32047,  7, }, // Asia/Sakhalin
    {  613, 3116,    6131,  4, }, // Asia/Seoul
    {  613, 1020,  187507,  6, }, // Asia/Shanghai
    {  613, 3137,   19291,  8, }, // Asia/Singapore
    {  613, 3100,  187513,  6, }, // Asia/Taipei
    {  613, 4683,  113335,  7, }, // Asia/Tashkent
    {  613, 2638,  187519,  6, }, // Asia/Tehran
    {  613, 1284,   90291,  5, }, // Asia/Thimphu
    {  613, 2687,    4257,  5, }, // Asia/Tokyo
    {  613, 1346,  139650, 10, }, // Asia/Ulaanbaatar
    {  613, 1126,   21017,  6, }, // Asia/Urumqi
    {  613, 6940,  187525,  8, }, // Asia/Vientiane
    {  613, 4612,    4262, 11, }, // Asia/Vladivostok
    {  613, 4730,  187533,  7, }, // Asia/Yakutsk
    {  613, 1202,  187540,  6, }, // Asia/Yangon
    {  613, 3785,  187546, 13, }, // Asia/Yekaterinburg
    {  613, 3649,  187559,  7, }, // Asia/Yerevan
    {  613, 3565,  187566, 12, }, // Atlantic/Azores
    {  613, 6591,  187578, 13, }, // Atlantic/Canary
    {  613, 3629,  187591, 10, }, // Atlantic/Cape_Verde
    {  613, 1379,  187601, 13, }, // Atlantic/Faroe
    {  613, 8699,  187614, 15, }, // Atlantic/Madeira
    {  613, 2614,  187629,  9, }, // Atlantic/Reykjavik
    {  613, 7235,  187638, 16, }, // Atlantic/South_Georgia
    {  613, 6784,  187654, 22, }, // Atlantic/St_Helena
    {  613, 6607,    6298,  6, }, // Atlantic/Stanley
    {  613, 1654,   19421,  8, }, // Australia/Adelaide
    {  613, 1619,  119584,  7, }, // Australia/Brisbane
    {  613, 1783,   92467, 10, }, // Australia/Broken_Hill
    {  613, 3539,  187676,  5, }, // Australia/Eucla
    {  613, 1531,  187681,  8, }, // Australia/Lord_Howe
    {  613, 1711,    6359,  7, }, // Australia/Melbourne
    {  613, 1746,   21145,  4, }, // Australia/Perth
    {  613, 1447,  187689,  6, }, // Australia/Sydney
    {  613, 3522,   32115,  8, }, // Europe/Astrakhan
    {  613, 6577,  187695,  5, }, // Europe/Athens
    {  613, 6285,   25465,  7, }, // Europe/Belgrade
    {  613, 6437,  187700, 10, }, // Europe/Bratislava
    {  613, 6178,  187710,  8, }, // Europe/Brussels
    {  613, 3830,  187718,  8, }, // Europe/Bucharest
    {  613, 3708,   21191,  8, }, // Europe/Budapest
    {  613, 8751,  187726,  9, }, // Europe/Busingen
    {  613, 2475,  187735,  8, }, // Europe/Chisinau
    {  613, 6238,  187743, 10, }, // Europe/Copenhagen
    {  613, 8767,  122872,  6, }, // Europe/Guernsey
    {  613, 8783,  187753,  9, }, // Europe/Isle_of_Man
    {  613, 1097,  119786,  7, }, // Europe/Istanbul
    {  613, 8802,  187762,  5, }, // Europe/Jersey
    {  613, 8924,   76768,  5, }, // Europe/Kirov
    {  613, 2419,  187767,  5, }, // Europe/Kyiv
    {  613, 3078,    4400,  8, }, // Europe/Lisbon
    {  613, 6455,  187772,  9, }, // Europe/Ljubljana
    {  613, 6314,  187781, 11, }, // Europe/Luxembourg
    {  613, 6531,  187792, 10, }, // Europe/Mariehamn
    {  613, 6359,    6525,  6, }, // Europe/Monaco
    {  613, 3443,   91758,  6, }, // Europe/Moscow
    {  613, 4186,  187802,  5, }, // Europe/Paris
    {  613, 6373,  187807,  9, }, // Europe/Podgorica
    {  613, 6224,   19576,  5, }, // Europe/Prague
    {  613, 6273,   32211,  3, }, // Europe/Rome
    {  613, 6419,    6558, 10, }, // Europe/San_Marino
    {  613, 6194,  187816,  8, }, // Europe/Sarajevo
    {  613, 4353,   76777,  7, }, // Europe/Saratov
    {  613, 6332,  187824,  6, }, // Europe/Skopje
    {  613, 6548,  187830,  6, }, // Europe/Sofia
    {  613, 6472,  187836,  8, }, // Europe/Stockholm
    {  613, 8846,   21335,  6, }, // Europe/Tallinn
    {  613, 6135,   19608,  6, }, // Europe/Tirane
    {  613, 8861,  187844,  9, }, // Europe/Ulyanovsk
    {  613, 6301,  187853,  5, }, // Europe/Vaduz
    {  613, 6516,  187858,  7, }, // Europe/Vatican
    {  613, 6164,  187865,  4, }, // Europe/Vienna
    {  613, 8878,  187869,  7, }, // Europe/Vilnius
    {  613, 4629,    4462,  9, }, // Europe/Volgograd
    {  613, 3055,  187876,  7, }, // Europe/Warsaw
    {  613, 6489,  187883,  6, }, // Europe/Zurich
    {  613, 4980,  187889, 12, }, // Indian/Antananarivo
    {  613, 6910,    6639,  5, }, // Indian/Chagos
    {  613, 5979,  187901,  9, }, // Indian/Christmas
    {  613, 5996,    4484,  5, }, // Indian/Cocos
    {  613, 4931,  187910, 13, }, // Indian/Comoro
    {  613, 6624,   21401,  8, }, // Indian/Kerguelen
    {  613, 7223,  187923,  4, }, // Indian/Mahe
    {  613, 7013,  187927,  9, }, // Indian/Maldives
    {  613, 3971,  187936,  8, }, // Indian/Mauritius
    {  613, 5000,  187944,  7, }, // Indian/Mayotte
    {  613, 7174,  187951,  8, }, // Indian/Reunion
    {  613,  932,  120117,  6, }, // Pacific/Auckland
    {  613, 3608,  187959,  8, }, // Pacific/Bougainville
    {  613, 2863,  187967,  5, }, // Pacific/Chatham
    {  613, 3022,   21440,  4, }, // Pacific/Chuuk
    {  613, 2201,  187972, 11, }, // Pacific/Easter
    {  613, 6641,  187983, 17, }, // Pacific/Galapagos
    {  613, 6659,  188000,  6, }, // Pacific/Gambier
    {  613, 3724,  188006, 11, }, // Pacific/Guadalcanal
    {  613, 2929,  188017,  8, }, // Pacific/Honolulu
    {  613, 2897,  120205,  9, }, // Pacific/Kanton
    {  613, 6982,  188025,  7, }, // Pacific/Kosrae
    {  613, 2708,  188032,  9, }, // Pacific/Kwajalein
    {  613, 3953,  188041, 14, }, // Pacific/Marquesas
    {  613, 8909,  188055,  6, }, // Pacific/Midway
    {  613, 7061,  120279,  5, }, // Pacific/Noumea
    {  613, 2991,    6764,  9, }, // Pacific/Pago_Pago
    {  613, 7157,    6773,  7, }, // Pacific/Pitcairn
    {  613, 2961,    6780,  6, }, // Pacific/Pohnpei
    {  613, 4709,  188061, 11, }, // Pacific/Port_Moresby
    {  613, 5948,  188072,  6, }, // Pacific/Saipan
    {  613, 7275,  120324,  5, }, // Pacific/Tahiti
    {  613, 7355,  188078,  4, }, // Pacific/Wake
    {  613, 7368,    6818,  6, }, // Pacific/Wallis
    {  614,   14,       0,  6, }, // Africa/Asmara Tyap/Latin/Nigeria
    {  614, 4337,       6,  8, }, // Africa/Sao_Tome
    {  614, 4154,      14,  8, }, // America/Asuncion
    {  614,  237,      22,  8, }, // America/Atikokan
    {  614, 7586,      30, 17, }, // America/Bahia_Banderas
    {  614, 3770,      47,  6, }, // America/Cancun
    {  614, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  614, 5640,      66,  7, }, // America/Curacao
    {  614,  481,      73, 13, }, // America/Indiana/Knox
    {  614, 7924,      86, 16, }, // America/Indiana/Marengo
    {  614, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  614, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  614, 8001,     139, 14, }, // America/Indiana/Vevay
    {  614, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  614, 8049,     171, 16, }, // America/Indiana/Winamac
    {  614, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  614, 5849,     207, 22, }, // America/Lower_Princes
    {  614, 8183,     229,  6, }, // America/Merida
    {  614, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  614, 8292,     255, 20, }, // America/North_Dakota/Center
    {  614, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  614,  393,     298,  4, }, // America/Nuuk
    {  614, 6862,     302, 16, }, // America/Scoresbysund
    {  614, 8471,     318, 14, }, // America/St_Barthelemy
    {  614, 2045,     332, 10, }, // America/St_Johns
    {  614, 5780,     342,  9, }, // America/St_Kitts
    {  614, 5797,     351,  9, }, // America/St_Lucia
    {  614,  855,     360, 10, }, // America/St_Thomas
    {  614, 5830,     370, 11, }, // America/St_Vincent
    {  614, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  614, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  614, 1152,     413,  9, }, // Asia/Kathmandu
    {  614,  992,     422,  7, }, // Asia/Kolkata
    {  614, 1178,     429,  5, }, // Asia/Macau
    {  614, 1202,     442,  6, }, // Asia/Yangon
    {  614, 1379,     448,  5, }, // Atlantic/Faroe
    {  614, 6784,     453, 10, }, // Atlantic/St_Helena
    {  614, 2419,     463,  4, }, // Europe/Kyiv
    {  614, 7174,     467,  7, }, // Indian/Reunion
    {  614, 3022,     474,  5, }, // Pacific/Chuuk
    {  614, 2961,     479,  7, }, // Pacific/Pohnpei
    {  615, 6751,   25621,  7, }, // Africa/Abidjan Ukrainian/Cyrillic/Ukraine
    {  615, 6723,     492,  5, }, // Africa/Accra
    {  615, 4961,     497, 11, }, // Africa/Addis_Ababa
    {  615, 7383,     508,  5, }, // Africa/Algiers
    {  615,   14,     513,  6, }, // Africa/Asmara
    {  615,   44,     519,  6, }, // Africa/Bamako
    {  615, 5159,  188082,  5, }, // Africa/Bangui
    {  615, 6709,     530,  6, }, // Africa/Banjul
    {  615, 7398,   25665,  5, }, // Africa/Bissau
    {  615, 4859,   25670,  8, }, // Africa/Blantyre
    {  615, 5189,  188087, 10, }, // Africa/Brazzaville
    {  615, 4824,     558,  9, }, // Africa/Bujumbura
    {  615, 2242,  188097,  4, }, // Africa/Cairo
    {  615, 4019,     571, 10, }, // Africa/Casablanca
    {  615, 7412,     581,  5, }, // Africa/Ceuta
    {  615, 6736,  188101,  7, }, // Africa/Conakry
    {  615, 6803,     593,  5, }, // Africa/Dakar
    {  615, 5032,     598, 12, }, // Africa/Dar_es_Salaam
    {  615, 4945,  188108,  7, }, // Africa/Djibouti
    {  615, 5145,     616,  5, }, // Africa/Douala
    {  615, 5068,  188115,  7, }, // Africa/El_Aaiun
    {  615, 6816,  188122,  7, }, // Africa/Freetown
    {  615, 4808,  188129,  8, }, // Africa/Gaborone
    {  615, 4917,   32384,  6, }, // Africa/Harare
    {  615, 4381,  188137, 12, }, // Africa/Johannesburg
    {  615, 4401,   25755,  5, }, // Africa/Juba
    {  615, 5053,     665,  7, }, // Africa/Kampala
    {  615, 4426,     672,  6, }, // Africa/Khartoum
    {  615, 4875,   25760,  6, }, // Africa/Kigali
    {  615, 5208,   25766,  7, }, // Africa/Kinshasa
    {  615, 4646,  188149,  5, }, // Africa/Lagos
    {  615, 5238,  188154,  9, }, // Africa/Libreville
    {  615, 6832,     704,  4, }, // Africa/Lome
    {  615, 5113,     708,  6, }, // Africa/Luanda
    {  615, 4841,  188163,  9, }, // Africa/Lubumbashi
    {  615, 4903,     723,  6, }, // Africa/Lusaka
    {  615, 5224,     729,  6, }, // Africa/Malabo
    {  615, 4889,     735,  6, }, // Africa/Maputo
    {  615, 5099,     741,  6, }, // Africa/Maseru
    {  615, 5084,     747,  7, }, // Africa/Mbabane
    {  615, 5015,  188172,  8, }, // Africa/Mogadishu
    {  615, 6997,  188180,  8, }, // Africa/Monrovia
    {  615, 3755,   25829,  7, }, // Africa/Nairobi
    {  615, 5173,   25836,  8, }, // Africa/Ndjamena
    {  615, 5256,  188188,  6, }, // Africa/Niamey
    {  615, 6766,  188194,  8, }, // Africa/Nouakchott
    {  615, 6690,  188202,  8, }, // Africa/Ouagadougou
    {  615, 5127,     805, 10, }, // Africa/Porto-Novo
    {  615, 4337,     815,  8, }, // Africa/Sao_Tome
    {  615, 2732,  188210,  7, }, // Africa/Tripoli
    {  615, 6503,   25875,  5, }, // Africa/Tunis
    {  615, 4054,  188217,  7, }, // Africa/Windhoek
    {  615,  132,     842,  4, }, // America/Adak
    {  615, 3183,   50119,  8, }, // America/Anchorage
    {  615, 5524,  188224,  7, }, // America/Anguilla
    {  615, 5541,  188231,  7, }, // America/Antigua
    {  615, 4456,  188238,  9, }, // America/Araguaina
    {  615,  166,  188247, 12, }, // America/Argentina/Buenos_Aires
    {  615,   91,     888,  9, }, // America/Argentina/Catamarca
    {  615,  270,     897,  7, }, // America/Argentina/Cordoba
    {  615,  441,   25937,  5, }, // America/Argentina/Jujuy
    {  615, 7425,  188259,  8, }, // America/Argentina/La_Rioja
    {  615,  565,     917,  7, }, // America/Argentina/Mendoza
    {  615, 7452,  188267, 12, }, // America/Argentina/Rio_Gallegos
    {  615, 7483,   25962,  6, }, // America/Argentina/Salta
    {  615, 7507,     940,  8, }, // America/Argentina/San_Juan
    {  615, 5497,  188279,  8, }, // America/Argentina/San_Luis
    {  615, 7534,     956,  7, }, // America/Argentina/Tucuman
    {  615, 7560,   25976,  5, }, // America/Argentina/Ushuaia
    {  615, 5557,     969,  5, }, // America/Aruba
    {  615, 4154,  188287,  9, }, // America/Asuncion
    {  615,  237,  188296,  8, }, // America/Atikokan
    {  615, 3581,  132928,  4, }, // America/Bahia
    {  615, 7586,  188304, 13, }, // America/Bahia_Banderas
    {  615, 5571,    1013,  8, }, // America/Barbados
    {  615, 7609,    1021,  5, }, // America/Belem
    {  615, 5309,   26025,  5, }, // America/Belize
    {  615, 7623,   35000, 11, }, // America/Blanc-Sablon
    {  615, 7644,   26030,  9, }, // America/Boa_Vista
    {  615, 4263,  188317,  6, }, // America/Bogota
    {  615, 7662,   50182,  5, }, // America/Boise
    {  615, 7676,  188323, 13, }, // America/Cambridge_Bay
    {  615, 7698,  188336, 12, }, // America/Campo_Grande
    {  615, 3770,    1087,  6, }, // America/Cancun
    {  615, 4596,    1093,  7, }, // America/Caracas
    {  615, 4247,  188348,  6, }, // America/Cayenne
    {  615, 5398,  188354, 17, }, // America/Cayman
    {  615, 3251,  188371,  6, }, // America/Chicago
    {  615, 7719,  188377,  6, }, // America/Chihuahua
    {  615, 7737,  188383, 13, }, // America/Ciudad_Juarez
    {  615, 5324,  188396, 10, }, // America/Costa_Rica
    {  615, 7759,    1151,  7, }, // America/Creston
    {  615, 3693,   26138,  5, }, // America/Cuiaba
    {  615, 5640,   35086,  7, }, // America/Curacao
    {  615, 7775,    1172, 12, }, // America/Danmarkshavn
    {  615, 7796,    1184,  6, }, // America/Dawson
    {  615, 7811,  188406, 11, }, // America/Dawson_Creek
    {  615,  805,    1201,  6, }, // America/Denver
    {  615, 3351,   35129,  7, }, // America/Detroit
    {  615, 5656,  188417,  8, }, // America/Dominica
    {  615,  893,    1222,  8, }, // America/Edmonton
    {  615, 7832,   35144,  8, }, // America/Eirunepe
    {  615, 5343,   26215,  9, }, // America/El_Salvador
    {  615, 7849,   50223, 12, }, // America/Fort_Nelson
    {  615, 7869,    1257,  9, }, // America/Fortaleza
    {  615, 7887,  188425,  9, }, // America/Glace_Bay
    {  615, 6844,  188434,  7, }, // America/Goose_Bay
    {  615, 4514,  188441, 10, }, // America/Grand_Turk
    {  615, 5687,  188451,  7, }, // America/Grenada
    {  615, 5703,  188458,  9, }, // America/Guadeloupe
    {  615, 3662,  188467,  9, }, // America/Guatemala
    {  615, 4743,  188476,  8, }, // America/Guayaquil
    {  615, 6895,  188484,  6, }, // America/Guyana
    {  615, 1923,   26309,  8, }, // America/Halifax
    {  615, 2221,    1339,  6, }, // America/Havana
    {  615, 7905,  188490, 10, }, // America/Hermosillo
    {  615,  348,  188500, 12, }, // America/Indiana/Indianapolis
    {  615,  481,  188512, 13, }, // America/Indiana/Knox
    {  615, 7924,  188525, 16, }, // America/Indiana/Marengo
    {  615, 7948,  188541, 19, }, // America/Indiana/Petersburg
    {  615, 7975,  188560, 18, }, // America/Indiana/Tell_City
    {  615, 8001,  188578, 14, }, // America/Indiana/Vevay
    {  615, 8023,  188592, 16, }, // America/Indiana/Vincennes
    {  615, 8049,  188608, 16, }, // America/Indiana/Winamac
    {  615, 8073,   26448,  6, }, // America/Inuvik
    {  615,  660,  188624,  7, }, // America/Iqaluit
    {  615, 2665,   26461,  6, }, // America/Jamaica
    {  615, 5282,   35226,  5, }, // America/Juneau
    {  615,  521,  188631,  8, }, // America/Kentucky/Louisville
    {  615, 8088,  188639, 20, }, // America/Kentucky/Monticello
    {  615, 5621,   35231, 10, }, // America/Kralendijk
    {  615, 4278,    1514,  6, }, // America/La_Paz
    {  615, 7132,   26507,  4, }, // America/Lima
    {  615, 3390,   50287, 12, }, // America/Los_Angeles
    {  615, 5849,  188659, 20, }, // America/Lower_Princes
    {  615, 8116,   35279,  6, }, // America/Maceio
    {  615, 8131,  188679,  7, }, // America/Managua
    {  615, 1892,    1566,  6, }, // America/Manaus
    {  615, 5814,  188686,  6, }, // America/Marigot
    {  615, 5722,  188692,  9, }, // America/Martinique
    {  615, 8147,    1587,  9, }, // America/Matamoros
    {  615, 2779,    1596,  8, }, // America/Mazatlan
    {  615, 8165,   26578,  8, }, // America/Menominee
    {  615, 8183,  188701,  6, }, // America/Merida
    {  615, 8198,    1618, 10, }, // America/Metlakatla
    {  615, 2811,  188707,  6, }, // America/Mexico_City
    {  615, 4293,   26598,  7, }, // America/Miquelon
    {  615, 8217,    1641,  7, }, // America/Moncton
    {  615, 8233,   35285,  8, }, // America/Monterrey
    {  615, 4000,  188713, 10, }, // America/Montevideo
    {  615, 5741,    1667, 10, }, // America/Montserrat
    {  615, 5383,   33057,  5, }, // America/Nassau
    {  615, 3294,   50338,  8, }, // America/New_York
    {  615, 8251,    1691,  3, }, // America/Nome
    {  615, 1834,   54909,  7, }, // America/Noronha
    {  615, 8264,  188723, 22, }, // America/North_Dakota/Beulah
    {  615, 8292,  188745, 22, }, // America/North_Dakota/Center
    {  615, 8320,  188767, 26, }, // America/North_Dakota/New_Salem
    {  615,  393,    1772,  4, }, // America/Nuuk
    {  615, 8351,  188793,  7, }, // America/Ojinaga
    {  615, 5413,    1783,  6, }, // America/Panama
    {  615, 6106,    1789, 10, }, // America/Paramaribo
    {  615, 3224,   26753,  6, }, // America/Phoenix
    {  615, 3847,    1805, 12, }, // America/Port-au-Prince
    {  615, 5871,   26771, 13, }, // America/Port_of_Spain
    {  615, 8367,   26784, 11, }, // America/Porto_Velho
    {  615, 5760,  188800, 11, }, // America/Puerto_Rico
    {  615, 3932,    1852, 12, }, // America/Punta_Arenas
    {  615, 8387,  188811, 12, }, // America/Rankin_Inlet
    {  615, 8408,  188823,  6, }, // America/Recife
    {  615, 1979,   50425,  8, }, // America/Regina
    {  615, 8423,   54981,  8, }, // America/Resolute
    {  615,  695,  188829, 10, }, // America/Rio_Branco
    {  615, 8440,    1907,  8, }, // America/Santarem
    {  615, 2165,  188839,  8, }, // America/Santiago
    {  615, 6044,  188847, 13, }, // America/Santo_Domingo
    {  615, 1862,    1936,  9, }, // America/Sao_Paulo
    {  615, 6862,  188860, 16, }, // America/Scoresbysund
    {  615, 8457,   26913,  5, }, // America/Sitka
    {  615, 8471,  188876, 13, }, // America/St_Barthelemy
    {  615, 2045,   26931, 10, }, // America/St_Johns
    {  615, 5780,  188889, 10, }, // America/St_Kitts
    {  615, 5797,   26950, 10, }, // America/St_Lucia
    {  615,  855,    2003, 10, }, // America/St_Thomas
    {  615, 5830,   26960, 12, }, // America/St_Vincent
    {  615, 8493,  188899, 13, }, // America/Swift_Current
    {  615, 5363,  188912, 12, }, // America/Tegucigalpa
    {  615, 5673,    2049,  4, }, // America/Thule
    {  615,  313,  188924,  7, }, // America/Tijuana
    {  615,  608,    2060,  7, }, // America/Toronto
    {  615, 5605,    2067,  7, }, // America/Tortola
    {  615, 2077,    2074,  8, }, // America/Vancouver
    {  615, 2128,  188931,  8, }, // America/Whitehorse
    {  615,  734,  188939,  8, }, // America/Winnipeg
    {  615, 8515,   27032,  6, }, // America/Yakutat
    {  615, 5918,  188947,  5, }, // Antarctica/Casey
    {  615, 6027,  188952,  5, }, // Antarctica/Davis
    {  615, 6066,  188957, 14, }, // Antarctica/DumontDUrville
    {  615, 8531,  188971,  8, }, // Antarctica/Macquarie
    {  615, 7029,    2139,  6, }, // Antarctica/Mawson
    {  615, 7076,    2145,  9, }, // Antarctica/McMurdo
    {  615, 8552,    2154,  6, }, // Antarctica/Palmer
    {  615, 7189,    2160,  6, }, // Antarctica/Rothera
    {  615, 7258,  188979,  5, }, // Antarctica/Syowa
    {  615, 8570,    2171,  5, }, // Antarctica/Troll
    {  615, 7337,   33428,  6, }, // Antarctica/Vostok
    {  615, 1413,  188984,  7, }, // Arctic/Longyearbyen
    {  615, 5487,    2192,  4, }, // Asia/Aden
    {  615, 5297,   33445,  6, }, // Asia/Almaty
    {  615, 3870,    2202,  5, }, // Asia/Amman
    {  615, 5428,   33451,  6, }, // Asia/Anadyr
    {  615, 5440,    2213,  5, }, // Asia/Aqtau
    {  615, 5270,    2218,  6, }, // Asia/Aqtobe
    {  615,  964,   27131,  7, }, // Asia/Ashgabat
    {  615, 8587,   50535,  6, }, // Asia/Atyrau
    {  615, 3509,    2237,  6, }, // Asia/Baghdad
    {  615, 5451,   35560,  7, }, // Asia/Bahrain
    {  615, 3555,    2250,  4, }, // Asia/Baku
    {  615, 4368,    2254,  7, }, // Asia/Bangkok
    {  615, 3473,    2261,  7, }, // Asia/Barnaul
    {  615, 3988,   27145,  6, }, // Asia/Beirut
    {  615, 3680,   27151,  6, }, // Asia/Bishkek
    {  615, 5906,   27157,  6, }, // Asia/Brunei
    {  615, 4503,    2286,  4, }, // Asia/Chita
    {  615, 5963,   35567,  9, }, // Asia/Choibalsan
    {  615, 4413,    2299,  7, }, // Asia/Colombo
    {  615, 4442,    2306,  6, }, // Asia/Damascus
    {  615, 1060,    2312,  5, }, // Asia/Dhaka
    {  615, 6125,  188991,  4, }, // Asia/Dili
    {  615, 3498,   27187,  5, }, // Asia/Dubai
    {  615, 6092,    2326,  7, }, // Asia/Dushanbe
    {  615, 8599,    2333,  9, }, // Asia/Famagusta
    {  615, 8614,    2342,  4, }, // Asia/Gaza
    {  615, 4697,    2346,  6, }, // Asia/Hebron
    {  615, 1226,  188995,  7, }, // Asia/Ho_Chi_Minh
    {  615, 2576,   50560,  7, }, // Asia/Hong_Kong
    {  615, 4673,    2366,  4, }, // Asia/Hovd
    {  615, 4086,  189002,  8, }, // Asia/Irkutsk
    {  615, 6969,   27229,  8, }, // Asia/Jakarta
    {  615, 6955,   50567,  7, }, // Asia/Jayapura
    {  615, 1257,  189010,  8, }, // Asia/Jerusalem
    {  615, 3462,    2401,  5, }, // Asia/Kabul
    {  615, 4218,    2420,  8, }, // Asia/Kamchatka
    {  615, 4141,  189018,  6, }, // Asia/Karachi
    {  615, 1152,    2437,  8, }, // Asia/Kathmandu
    {  615, 8624,   33545,  7, }, // Asia/Khandyga
    {  615,  992,   35592,  7, }, // Asia/Kolkata
    {  615, 4099,  189024, 11, }, // Asia/Krasnoyarsk
    {  615, 4761,    2471, 12, }, // Asia/Kuala_Lumpur
    {  615, 5893,    2483,  6, }, // Asia/Kuching
    {  615, 5464,   27284,  6, }, // Asia/Kuwait
    {  615, 1178,    2495,  5, }, // Asia/Macau
    {  615, 3919,    2500,  7, }, // Asia/Magadan
    {  615, 1316,  114272,  8, }, // Asia/Makassar
    {  615, 7145,   27295,  6, }, // Asia/Manila
    {  615, 6883,    2520,  6, }, // Asia/Muscat
    {  615, 2446,  189035,  7, }, // Asia/Nicosia
    {  615, 8638,  189042, 12, }, // Asia/Novokuznetsk
    {  615, 4037,  189054, 12, }, // Asia/Novosibirsk
    {  615, 4131,  189066,  5, }, // Asia/Omsk
    {  615, 7108,   33618,  4, }, // Asia/Oral
    {  615, 6924,   50617,  8, }, // Asia/Phnom_Penh
    {  615, 8656,  189071,  9, }, // Asia/Pontianak
    {  615, 4116,   27354,  7, }, // Asia/Pyongyang
    {  615, 5476,    2588,  5, }, // Asia/Qatar
    {  615, 8671,   35626,  8, }, // Asia/Qostanay
    {  615, 4171,   33647,  9, }, // Asia/Qyzylorda
    {  615, 3486,  189080,  7, }, // Asia/Riyadh
    {  615, 4310,   27385,  7, }, // Asia/Sakhalin
    {  615, 7208,    2630,  9, }, // Asia/Samarkand
    {  615, 3116,    2639,  4, }, // Asia/Seoul
    {  615, 1020,   27392,  6, }, // Asia/Shanghai
    {  615, 3137,   27398,  8, }, // Asia/Singapore
    {  615, 4199,  189087, 16, }, // Asia/Srednekolymsk
    {  615, 3100,  189103,  6, }, // Asia/Taipei
    {  615, 4683,    2676,  7, }, // Asia/Tashkent
    {  615, 3817,   27426,  7, }, // Asia/Tbilisi
    {  615, 2638,    2687,  7, }, // Asia/Tehran
    {  615, 1284,   27440,  7, }, // Asia/Thimphu
    {  615, 2687,  189109,  5, }, // Asia/Tokyo
    {  615, 4474,  189114,  6, }, // Asia/Tomsk
    {  615, 1346,    2711, 10, }, // Asia/Ulaanbaatar
    {  615, 1126,  189120,  6, }, // Asia/Urumqi
    {  615, 8685,   50651,  9, }, // Asia/Ust-Nera
    {  615, 6940,  189126,  8, }, // Asia/Vientiane
    {  615, 4612,    2742, 11, }, // Asia/Vladivostok
    {  615, 4730,  189134,  7, }, // Asia/Yakutsk
    {  615, 1202,   55061,  5, }, // Asia/Yangon
    {  615, 3785,  189141, 12, }, // Asia/Yekaterinburg
    {  615, 3649,  189153,  6, }, // Asia/Yerevan
    {  615, 3565,  189159, 16, }, // Atlantic/Azores
    {  615, 5588,  124348,  7, }, // Atlantic/Bermuda
    {  615, 6591,  189175, 17, }, // Atlantic/Canary
    {  615, 3629,    2835, 10, }, // Atlantic/Cape_Verde
    {  615, 1379,  189192, 17, }, // Atlantic/Faroe
    {  615, 8699,   35753,  7, }, // Atlantic/Madeira
    {  615, 2614,  189209,  9, }, // Atlantic/Reykjavik
    {  615, 7235,  189218, 17, }, // Atlantic/South_Georgia
    {  615, 6784,  189235, 19, }, // Atlantic/St_Helena
    {  615, 6607,  189254,  6, }, // Atlantic/Stanley
    {  615, 1654,  189260,  8, }, // Australia/Adelaide
    {  615, 1619,  189268,  7, }, // Australia/Brisbane
    {  615, 1783,  189275, 11, }, // Australia/Broken_Hill
    {  615, 1567,   27693,  6, }, // Australia/Darwin
    {  615, 3539,  189286,  5, }, // Australia/Eucla
    {  615, 1500,  189291,  6, }, // Australia/Hobart
    {  615, 8716,  189297,  8, }, // Australia/Lindeman
    {  615, 1531,    2977,  8, }, // Australia/Lord_Howe
    {  615, 1711,   27711,  8, }, // Australia/Melbourne
    {  615, 1746,    2992,  4, }, // Australia/Perth
    {  615, 1447,   27719,  6, }, // Australia/Sydney
    {  615, 6390,    3002,  9, }, // Europe/Amsterdam
    {  615, 6149,    3011,  7, }, // Europe/Andorra
    {  615, 3522,   27740,  9, }, // Europe/Astrakhan
    {  615, 6577,  189305,  5, }, // Europe/Athens
    {  615, 6285,    3030,  7, }, // Europe/Belgrade
    {  615, 4659,   27761,  6, }, // Europe/Berlin
    {  615, 6437,    3043, 10, }, // Europe/Bratislava
    {  615, 6178,   50822,  8, }, // Europe/Brussels
    {  615, 3830,    3061,  8, }, // Europe/Bucharest
    {  615, 3708,    3069,  8, }, // Europe/Budapest
    {  615, 8751,   27792,  8, }, // Europe/Busingen
    {  615, 2475,  189310,  7, }, // Europe/Chisinau
    {  615, 6238,   50849, 10, }, // Europe/Copenhagen
    {  615, 2260,   27817,  6, }, // Europe/Dublin
    {  615, 6256,   27823,  9, }, // Europe/Gibraltar
    {  615, 8767,   27832,  6, }, // Europe/Guernsey
    {  615, 6561,  189317,  9, }, // Europe/Helsinki
    {  615, 8783,  189326, 10, }, // Europe/Isle_of_Man
    {  615, 1097,    3155,  7, }, // Europe/Istanbul
    {  615, 8802,  189336,  6, }, // Europe/Jersey
    {  615, 3881,   27864, 11, }, // Europe/Kaliningrad
    {  615, 8924,  189342,  5, }, // Europe/Kirov
    {  615, 2419,  189347,  4, }, // Europe/Kyiv
    {  615, 3078,   27884,  7, }, // Europe/Lisbon
    {  615, 6455,   27891,  7, }, // Europe/Ljubljana
    {  615, 2393,    3204,  6, }, // Europe/London
    {  615, 6314,   27904, 10, }, // Europe/Luxembourg
    {  615, 4794,    3221,  6, }, // Europe/Madrid
    {  615, 6346,   27920,  6, }, // Europe/Malta
    {  615, 6531,  189351,  9, }, // Europe/Mariehamn
    {  615, 3595,  189360,  6, }, // Europe/Minsk
    {  615, 6359,    3246,  6, }, // Europe/Monaco
    {  615, 3443,    3252,  6, }, // Europe/Moscow
    {  615, 6407,    3258,  4, }, // Europe/Oslo
    {  615, 4186,    3262,  5, }, // Europe/Paris
    {  615, 6373,  189366,  9, }, // Europe/Podgorica
    {  615, 6224,    3276,  5, }, // Europe/Prague
    {  615, 8816,    3281,  4, }, // Europe/Riga
    {  615, 6273,    3285,  3, }, // Europe/Rome
    {  615, 4233,    3288,  6, }, // Europe/Samara
    {  615, 6419,    3294, 10, }, // Europe/San_Marino
    {  615, 6194,  189375,  7, }, // Europe/Sarajevo
    {  615, 4353,    3311,  7, }, // Europe/Saratov
    {  615, 8828,  189382, 11, }, // Europe/Simferopol
    {  615, 6332,  189393,  6, }, // Europe/Skopje
    {  615, 6548,  189399,  5, }, // Europe/Sofia
    {  615, 6472,   50900,  9, }, // Europe/Stockholm
    {  615, 8846,  189404,  7, }, // Europe/Tallinn
    {  615, 6135,    3352,  6, }, // Europe/Tirane
    {  615, 8861,  189411, 10, }, // Europe/Ulyanovsk
    {  615, 6301,    3367,  5, }, // Europe/Vaduz
    {  615, 6516,    3372,  7, }, // Europe/Vatican
    {  615, 6164,  189421,  6, }, // Europe/Vienna
    {  615, 8878,   28062,  7, }, // Europe/Vilnius
    {  615, 4629,    3390,  9, }, // Europe/Volgograd
    {  615, 3055,    3399,  7, }, // Europe/Warsaw
    {  615, 6210,    3406,  6, }, // Europe/Zagreb
    {  615, 6489,  189427,  5, }, // Europe/Zurich
    {  615, 4980,  189432, 12, }, // Indian/Antananarivo
    {  615, 6910,  189444,  5, }, // Indian/Chagos
    {  615, 5979,  189449, 13, }, // Indian/Christmas
    {  615, 5996,  189462, 16, }, // Indian/Cocos
    {  615, 4931,   50972,  6, }, // Indian/Comoro
    {  615, 6624,  189478,  8, }, // Indian/Kerguelen
    {  615, 7223,   34128,  4, }, // Indian/Mahe
    {  615, 7013,  189486,  8, }, // Indian/Maldives
    {  615, 3971,  189494,  8, }, // Indian/Mauritius
    {  615, 5000,   55236,  7, }, // Indian/Mayotte
    {  615, 7174,  189502,  8, }, // Indian/Reunion
    {  615, 4324,  189510,  4, }, // Pacific/Apia
    {  615,  932,    3511,  6, }, // Pacific/Auckland
    {  615, 3608,  189514,  9, }, // Pacific/Bougainville
    {  615, 2863,    3525,  5, }, // Pacific/Chatham
    {  615, 3022,  114386,  4, }, // Pacific/Chuuk
    {  615, 2201,  189523, 12, }, // Pacific/Easter
    {  615, 7323,    3545,  5, }, // Pacific/Efate
    {  615, 7290,    3550,  7, }, // Pacific/Fakaofo
    {  615, 3804,  189535,  5, }, // Pacific/Fiji
    {  615, 7306,  189540,  8, }, // Pacific/Funafuti
    {  615, 6641,  189548,  9, }, // Pacific/Galapagos
    {  615, 6659,  189557,  6, }, // Pacific/Gambier
    {  615, 3724,    3604, 11, }, // Pacific/Guadalcanal
    {  615, 5935,    3615,  4, }, // Pacific/Guam
    {  615, 2929,   51108,  8, }, // Pacific/Honolulu
    {  615, 2897,  189563,  9, }, // Pacific/Kanton
    {  615, 3900,  189572, 10, }, // Pacific/Kiritimati
    {  615, 6982,    3635,  6, }, // Pacific/Kosrae
    {  615, 2708,   28300, 10, }, // Pacific/Kwajalein
    {  615, 4779,   36063,  7, }, // Pacific/Majuro
    {  615, 3953,  189582, 17, }, // Pacific/Marquesas
    {  615, 8909,  189599,  6, }, // Pacific/Midway
    {  615, 7047,    3689,  5, }, // Pacific/Nauru
    {  615, 7095,  189605,  4, }, // Pacific/Niue
    {  615, 4070,    3698,  7, }, // Pacific/Norfolk
    {  615, 7061,    3705,  5, }, // Pacific/Noumea
    {  615, 2991,    3710,  9, }, // Pacific/Pago_Pago
    {  615, 7118,    3719,  5, }, // Pacific/Palau
    {  615, 7157,  189609,  7, }, // Pacific/Pitcairn
    {  615, 2961,   34263,  6, }, // Pacific/Pohnpei
    {  615, 4709,   28374, 11, }, // Pacific/Port_Moresby
    {  615, 6009,    3748,  9, }, // Pacific/Rarotonga
    {  615, 5948,   28394,  6, }, // Pacific/Saipan
    {  615, 7275,  189616,  5, }, // Pacific/Tahiti
    {  615, 6675,    3768,  6, }, // Pacific/Tarawa
    {  615, 4485,    3774,  9, }, // Pacific/Tongatapu
    {  615, 7355,  189621,  4, }, // Pacific/Wake
    {  615, 7368,  189625,  6, }, // Pacific/Wallis
    {  616, 6723,   19687,  5, }, // Africa/Accra Upper Sorbian/Latin/Germany
    {  616, 4961,    3793, 11, }, // Africa/Addis_Ababa
    {  616, 7383,   57088,  6, }, // Africa/Algiers
    {  616,   14,       0,  6, }, // Africa/Asmara
    {  616, 2242,   24913,  5, }, // Africa/Cairo
    {  616, 5032,   76549, 10, }, // Africa/Dar_es_Salaam
    {  616, 4945,  189631,  7, }, // Africa/Djibouti
    {  616, 5068,   62570,  8, }, // Africa/El_Aaiun
    {  616, 4426,   24937,  7, }, // Africa/Khartoum
    {  616, 6832,   18802,  4, }, // Africa/Lome
    {  616, 5015,   56254,  8, }, // Africa/Mogadishu
    {  616, 6690,   97031,  8, }, // Africa/Ouagadougou
    {  616, 5127,   24975, 10, }, // Africa/Porto-Novo
    {  616, 4337,       6,  8, }, // Africa/Sao_Tome
    {  616, 2732,   56289,  8, }, // Africa/Tripoli
    {  616, 4154,      14,  8, }, // America/Asuncion
    {  616,  237,      22,  8, }, // America/Atikokan
    {  616, 7586,      30, 17, }, // America/Bahia_Banderas
    {  616, 4263,   18898,  6, }, // America/Bogota
    {  616, 3770,      47,  6, }, // America/Cancun
    {  616, 5398,  189638, 14, }, // America/Cayman
    {  616, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  616, 5640,      66,  7, }, // America/Curacao
    {  616, 2221,   61885,  7, }, // America/Havana
    {  616,  481,      73, 13, }, // America/Indiana/Knox
    {  616, 7924,      86, 16, }, // America/Indiana/Marengo
    {  616, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  616, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  616, 8001,     139, 14, }, // America/Indiana/Vevay
    {  616, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  616, 8049,     171, 16, }, // America/Indiana/Winamac
    {  616, 2665,    3892,  7, }, // America/Jamaica
    {  616, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  616, 5849,     207, 22, }, // America/Lower_Princes
    {  616, 8183,     229,  6, }, // America/Merida
    {  616, 2811,  189652, 12, }, // America/Mexico_City
    {  616, 8264,  189664, 23, }, // America/North_Dakota/Beulah
    {  616, 8292,  189687, 23, }, // America/North_Dakota/Center
    {  616, 8320,  189710, 26, }, // America/North_Dakota/New_Salem
    {  616,  393,     298,  4, }, // America/Nuuk
    {  616, 5871,   25124, 13, }, // America/Port_of_Spain
    {  616, 1862,   25145,  9, }, // America/Sao_Paulo
    {  616, 6862,     302, 16, }, // America/Scoresbysund
    {  616, 8471,  123517, 13, }, // America/St_Barthelemy
    {  616, 2045,     332, 10, }, // America/St_Johns
    {  616, 5780,     342,  9, }, // America/St_Kitts
    {  616, 5797,     351,  9, }, // America/St_Lucia
    {  616,  855,     360, 10, }, // America/St_Thomas
    {  616, 5830,     370, 11, }, // America/St_Vincent
    {  616, 6066,  123530, 16, }, // Antarctica/DumontDUrville
    {  616, 7337,  189736, 17, }, // Antarctica/Vostok
    {  616, 5270,    5911,  6, }, // Asia/Aqtobe
    {  616, 3509,    4063,  6, }, // Asia/Baghdad
    {  616, 3680,   31917,  6, }, // Asia/Bishkek
    {  616, 4442,    4102,  8, }, // Asia/Damascus
    {  616, 6092,   31939,  7, }, // Asia/Dushanbe
    {  616, 1226,  189753, 20, }, // Asia/Ho_Chi_Minh
    {  616, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  616, 6955,  123563, 11, }, // Asia/Jayapura
    {  616, 4218,   31981,  8, }, // Asia/Kamchatka
    {  616, 4141,   31989,  6, }, // Asia/Karachi
    {  616, 1152,     413,  9, }, // Asia/Kathmandu
    {  616,  992,   62730,  8, }, // Asia/Kolkata
    {  616, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  616, 1178,     429,  5, }, // Asia/Macau
    {  616, 6883,    4227,  6, }, // Asia/Muscat
    {  616, 2446,   25331,  7, }, // Asia/Nicosia
    {  616, 8638,  189773, 12, }, // Asia/Novokuznetsk
    {  616, 4037,   76692, 11, }, // Asia/Novosibirsk
    {  616, 4116,   76703,  9, }, // Asia/Pyongyang
    {  616, 5476,    4233,  5, }, // Asia/Qatar
    {  616, 3486,    6120,  4, }, // Asia/Riyadh
    {  616, 4310,   56617,  8, }, // Asia/Sakhalin
    {  616, 3137,   19291,  8, }, // Asia/Singapore
    {  616, 3100,   76712,  6, }, // Asia/Taipei
    {  616, 4683,   32060,  7, }, // Asia/Tashkent
    {  616, 2638,    6175,  7, }, // Asia/Tehran
    {  616, 2687,    4257,  5, }, // Asia/Tokyo
    {  616, 4612,    4262, 11, }, // Asia/Vladivostok
    {  616, 4730,    6206,  7, }, // Asia/Yakutsk
    {  616, 1202,    6213,  6, }, // Asia/Yangon
    {  616, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  616, 3649,  189785,  8, }, // Asia/Yerevan
    {  616, 3565,  123594,  5, }, // Atlantic/Azores
    {  616, 5588,   56708,  7, }, // Atlantic/Bermuda
    {  616, 6591,  189793, 14, }, // Atlantic/Canary
    {  616, 3629,   55900,  9, }, // Atlantic/Cape_Verde
    {  616, 1379,  189807, 12, }, // Atlantic/Faroe
    {  616, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  616, 7235,  189819, 15, }, // Atlantic/South_Georgia
    {  616, 6784,     453, 10, }, // Atlantic/St_Helena
    {  616, 6577,    4339,  5, }, // Europe/Athens
    {  616, 6285,  189834,  8, }, // Europe/Belgrade
    {  616, 6178,   21176,  7, }, // Europe/Brussels
    {  616, 3830,   25479,  8, }, // Europe/Bucharest
    {  616, 8751,   25487,  8, }, // Europe/Busingen
    {  616, 2475,  189842,  8, }, // Europe/Chisinau
    {  616, 6238,    4376, 10, }, // Europe/Copenhagen
    {  616, 2419,   76773,  4, }, // Europe/Kyiv
    {  616, 3078,    4400,  8, }, // Europe/Lisbon
    {  616, 6314,    4414,  9, }, // Europe/Luxembourg
    {  616, 3443,   91758,  6, }, // Europe/Moscow
    {  616, 6224,   34765,  5, }, // Europe/Prague
    {  616, 6273,    6554,  3, }, // Europe/Rome
    {  616, 6135,   19608,  6, }, // Europe/Tirane
    {  616, 6516,    6614,  7, }, // Europe/Vatican
    {  616, 6164,   56035,  4, }, // Europe/Vienna
    {  616, 4629,    4462,  9, }, // Europe/Volgograd
    {  616, 3055,  123647,  7, }, // Europe/Warsaw
    {  616, 6489,    4478,  6, }, // Europe/Zurich
    {  616, 5979,  189850, 12, }, // Indian/Christmas
    {  616, 4931,   56941,  6, }, // Indian/Comoro
    {  616, 7013,  123666,  8, }, // Indian/Maldives
    {  616, 7174,     467,  7, }, // Indian/Reunion
    {  616, 3022,     474,  5, }, // Pacific/Chuuk
    {  616, 2201,  189862, 13, }, // Pacific/Easter
    {  616, 3804,  189875,  5, }, // Pacific/Fiji
    {  616, 2961,     479,  7, }, // Pacific/Pohnpei
    {  617, 6751,  189880,  7, }, // Africa/Abidjan Urdu/Arabic/Pakistan
    {  617, 6723,  189887,  5, }, // Africa/Accra
    {  617, 4961,  101682, 10, }, // Africa/Addis_Ababa
    {  617, 7383,  189892,  7, }, // Africa/Algiers
    {  617,   14,  189899,  6, }, // Africa/Asmara
    {  617,   44,  101709,  5, }, // Africa/Bamako
    {  617, 5159,  189905,  6, }, // Africa/Bangui
    {  617, 6709,  189911,  5, }, // Africa/Banjul
    {  617, 7398,  101729,  5, }, // Africa/Bissau
    {  617, 4859,  189916,  8, }, // Africa/Blantyre
    {  617, 5189,  189924,  9, }, // Africa/Brazzaville
    {  617, 4824,  189933,  6, }, // Africa/Bujumbura
    {  617, 2242,  189939,  5, }, // Africa/Cairo
    {  617, 4019,  189944, 11, }, // Africa/Casablanca
    {  617, 7412,  189955,  5, }, // Africa/Ceuta
    {  617, 6736,  189960,  6, }, // Africa/Conakry
    {  617, 6803,  189966,  4, }, // Africa/Dakar
    {  617, 5032,    8974, 10, }, // Africa/Dar_es_Salaam
    {  617, 4945,  189970,  5, }, // Africa/Djibouti
    {  617, 5145,  189975,  5, }, // Africa/Douala
    {  617, 5068,  147167,  6, }, // Africa/El_Aaiun
    {  617, 6816,  189980,  8, }, // Africa/Freetown
    {  617, 4808,  189988,  5, }, // Africa/Gaborone
    {  617, 4917,  189993,  5, }, // Africa/Harare
    {  617, 4381,  101845,  9, }, // Africa/Johannesburg
    {  617, 4401,    9030,  4, }, // Africa/Juba
    {  617, 5053,  189998,  7, }, // Africa/Kampala
    {  617, 4426,    9043,  5, }, // Africa/Khartoum
    {  617, 4875,  190005,  5, }, // Africa/Kigali
    {  617, 5208,  190010,  6, }, // Africa/Kinshasa
    {  617, 4646,  144547,  5, }, // Africa/Lagos
    {  617, 5238,  190016,  9, }, // Africa/Libreville
    {  617, 6832,    9073,  3, }, // Africa/Lome
    {  617, 5113,  190025,  6, }, // Africa/Luanda
    {  617, 4841,  160030,  8, }, // Africa/Lubumbashi
    {  617, 4903,  190031,  7, }, // Africa/Lusaka
    {  617, 5224,  160044,  5, }, // Africa/Malabo
    {  617, 4889,  190038,  5, }, // Africa/Maputo
    {  617, 5099,  190043,  5, }, // Africa/Maseru
    {  617, 5084,  190048,  6, }, // Africa/Mbabane
    {  617, 5015,  147273,  8, }, // Africa/Mogadishu
    {  617, 6997,  147281,  8, }, // Africa/Monrovia
    {  617, 3755,  101953,  6, }, // Africa/Nairobi
    {  617, 5173,  190054,  9, }, // Africa/Ndjamena
    {  617, 5256,  147303,  5, }, // Africa/Niamey
    {  617, 6766,  190063,  7, }, // Africa/Nouakchott
    {  617, 6690,  190070,  9, }, // Africa/Ouagadougou
    {  617, 5127,  101989, 10, }, // Africa/Porto-Novo
    {  617, 4337,  190079,  7, }, // Africa/Sao_Tome
    {  617, 2732,  190086,  7, }, // Africa/Tripoli
    {  617, 6503,  190093,  5, }, // Africa/Tunis
    {  617, 4054,  190098,  6, }, // Africa/Windhoek
    {  617,  132,  144670,  4, }, // America/Adak
    {  617, 3183,  144674,  7, }, // America/Anchorage
    {  617, 5524,  190104,  7, }, // America/Anguilla
    {  617, 5541,  190111,  7, }, // America/Antigua
    {  617, 4456,  160141,  9, }, // America/Araguaina
    {  617,  166,  190118, 10, }, // America/Argentina/Buenos_Aires
    {  617,   91,  102076,  9, }, // America/Argentina/Catamarca
    {  617,  270,  190128,  7, }, // America/Argentina/Cordoba
    {  617,  441,  190135,  6, }, // America/Argentina/Jujuy
    {  617, 7425,  160185,  9, }, // America/Argentina/La_Rioja
    {  617,  565,  190141,  7, }, // America/Argentina/Mendoza
    {  617, 7452,  160201, 11, }, // America/Argentina/Rio_Gallegos
    {  617, 7483,  102126,  5, }, // America/Argentina/Salta
    {  617, 7507,  160217,  8, }, // America/Argentina/San_Juan
    {  617, 5497,  190148,  8, }, // America/Argentina/San_Luis
    {  617, 7534,  190156,  7, }, // America/Argentina/Tucuman
    {  617, 7560,  160241,  8, }, // America/Argentina/Ushuaia
    {  617, 5557,  147464,  5, }, // America/Aruba
    {  617, 4154,  190163,  7, }, // America/Asuncion
    {  617,  237,  190170,  7, }, // America/Atikokan
    {  617, 3581,  102200,  5, }, // America/Bahia
    {  617, 7586,  190177, 11, }, // America/Bahia_Banderas
    {  617, 5571,  102212,  8, }, // America/Barbados
    {  617, 7609,  190188,  5, }, // America/Belem
    {  617, 5309,  190193,  6, }, // America/Belize
    {  617, 7623,  190199, 11, }, // America/Blanc-Sablon
    {  617, 7644,  190210,  8, }, // America/Boa_Vista
    {  617, 4263,  190218,  5, }, // America/Bogota
    {  617, 7662,  190223,  5, }, // America/Boise
    {  617, 7676,  190228, 15, }, // America/Cambridge_Bay
    {  617, 7698,  190243, 11, }, // America/Campo_Grande
    {  617, 3770,  190254,  6, }, // America/Cancun
    {  617, 4596,  190260,  6, }, // America/Caracas
    {  617, 4247,  190266,  5, }, // America/Cayenne
    {  617, 5398,  190271,  5, }, // America/Cayman
    {  617, 3251,  190276,  5, }, // America/Chicago
    {  617, 7719,  190281,  8, }, // America/Chihuahua
    {  617, 7737,  144917, 13, }, // America/Ciudad_Juarez
    {  617, 5324,  190289, 10, }, // America/Costa_Rica
    {  617, 7759,  190299,  7, }, // America/Creston
    {  617, 3693,  190306,  7, }, // America/Cuiaba
    {  617, 5640,  190313,  8, }, // America/Curacao
    {  617, 7775,  102361, 11, }, // America/Danmarkshavn
    {  617, 7796,  190321,  5, }, // America/Dawson
    {  617, 7811,  190326, 10, }, // America/Dawson_Creek
    {  617,  805,  190336,  5, }, // America/Denver
    {  617, 3351,  190341,  7, }, // America/Detroit
    {  617, 5656,  190348,  7, }, // America/Dominica
    {  617,  893,  190355,  8, }, // America/Edmonton
    {  617, 7832,  190363,  7, }, // America/Eirunepe
    {  617, 5343,  190370, 10, }, // America/El_Salvador
    {  617, 7849,  102441, 10, }, // America/Fort_Nelson
    {  617, 7869,  190380,  9, }, // America/Fortaleza
    {  617, 7887,  190389, 13, }, // America/Glace_Bay
    {  617, 6844,  190402, 12, }, // America/Goose_Bay
    {  617, 4514,  190414,  8, }, // America/Grand_Turk
    {  617, 5687,  190422,  6, }, // America/Grenada
    {  617, 5703,  190428,  8, }, // America/Guadeloupe
    {  617, 3662,  190436, 10, }, // America/Guatemala
    {  617, 4743,  190446,  9, }, // America/Guayaquil
    {  617, 6895,  190455,  5, }, // America/Guyana
    {  617, 1923,  190460,  8, }, // America/Halifax
    {  617, 2221,  190468,  5, }, // America/Havana
    {  617, 7905,  190473,  8, }, // America/Hermosillo
    {  617,  348,  190481, 11, }, // America/Indiana/Indianapolis
    {  617,  481,  190492, 14, }, // America/Indiana/Knox
    {  617, 7924,  190506, 15, }, // America/Indiana/Marengo
    {  617, 7948,  190521, 17, }, // America/Indiana/Petersburg
    {  617, 7975,  190538, 16, }, // America/Indiana/Tell_City
    {  617, 8001,  190554, 13, }, // America/Indiana/Vevay
    {  617, 8023,  190567, 16, }, // America/Indiana/Vincennes
    {  617, 8049,  190583, 16, }, // America/Indiana/Winamac
    {  617, 8073,  190599,  6, }, // America/Inuvik
    {  617,  660,  190605,  8, }, // America/Iqaluit
    {  617, 2665,  190613,  7, }, // America/Jamaica
    {  617, 5282,  190620,  6, }, // America/Juneau
    {  617,  521,  190626,  9, }, // America/Kentucky/Louisville
    {  617, 8088,  190635, 18, }, // America/Kentucky/Monticello
    {  617, 5621,  190653, 10, }, // America/Kralendijk
    {  617, 4278,  102731,  6, }, // America/La_Paz
    {  617, 7132,  145276,  4, }, // America/Lima
    {  617, 3390,  145280, 10, }, // America/Los_Angeles
    {  617, 5849,  190663, 17, }, // America/Lower_Princes
    {  617, 8116,  160796,  6, }, // America/Maceio
    {  617, 8131,  190680,  6, }, // America/Managua
    {  617, 1892,  190686,  5, }, // America/Manaus
    {  617, 5814,  102787,  7, }, // America/Marigot
    {  617, 5722,  190691,  7, }, // America/Martinique
    {  617, 8147,  102802,  9, }, // America/Matamoros
    {  617, 2779,  190698,  7, }, // America/Mazatlan
    {  617, 8165,  190705,  8, }, // America/Menominee
    {  617, 8183,  190713,  6, }, // America/Merida
    {  617, 8198,  102833, 11, }, // America/Metlakatla
    {  617, 2811,  190719, 11, }, // America/Mexico_City
    {  617, 4293,  190730,  8, }, // America/Miquelon
    {  617, 8217,  190738,  6, }, // America/Moncton
    {  617, 8233,  190744,  7, }, // America/Monterrey
    {  617, 4000,  190751, 11, }, // America/Montevideo
    {  617, 5741,  190762,  9, }, // America/Montserrat
    {  617, 5383,  102895,  4, }, // America/Nassau
    {  617, 3294,  190771,  8, }, // America/New_York
    {  617, 8251,    9897,  3, }, // America/Nome
    {  617, 1834,  102908,  7, }, // America/Noronha
    {  617, 8264,  190779, 19, }, // America/North_Dakota/Beulah
    {  617, 8292,  190798, 16, }, // America/North_Dakota/Center
    {  617, 8320,  190814, 21, }, // America/North_Dakota/New_Salem
    {  617,  393,  102562,  3, }, // America/Nuuk
    {  617, 8351,  190835,  8, }, // America/Ojinaga
    {  617, 5413,  190843,  5, }, // America/Panama
    {  617, 6106,  190848,  9, }, // America/Paramaribo
    {  617, 3224,  145527,  5, }, // America/Phoenix
    {  617, 3847,  190857, 12, }, // America/Port-au-Prince
    {  617, 5871,  190869, 13, }, // America/Port_of_Spain
    {  617, 8367,  190882, 11, }, // America/Porto_Velho
    {  617, 5760,  190893, 11, }, // America/Puerto_Rico
    {  617, 3932,  190904, 11, }, // America/Punta_Arenas
    {  617, 8387,  190915, 11, }, // America/Rankin_Inlet
    {  617, 8408,  190926,  6, }, // America/Recife
    {  617, 1979,  161087,  6, }, // America/Regina
    {  617, 8423,  190932,  8, }, // America/Resolute
    {  617,  695,  190940, 11, }, // America/Rio_Branco
    {  617, 8440,  190951,  7, }, // America/Santarem
    {  617, 2165,  190958,  7, }, // America/Santiago
    {  617, 6044,  190965, 12, }, // America/Santo_Domingo
    {  617, 1862,  103145,  8, }, // America/Sao_Paulo
    {  617, 6862,  190977, 15, }, // America/Scoresbysund
    {  617, 8457,  190992,  5, }, // America/Sitka
    {  617, 8471,  190997, 13, }, // America/St_Barthelemy
    {  617, 2045,  191010,  9, }, // America/St_Johns
    {  617, 5780,  191019,  8, }, // America/St_Kitts
    {  617, 5797,  191027, 10, }, // America/St_Lucia
    {  617,  855,  191037, 10, }, // America/St_Thomas
    {  617, 5830,  191047, 10, }, // America/St_Vincent
    {  617, 8493,  191057, 10, }, // America/Swift_Current
    {  617, 5363,  191067, 11, }, // America/Tegucigalpa
    {  617, 5673,  191078,  5, }, // America/Thule
    {  617,  313,  191083,  7, }, // America/Tijuana
    {  617,  608,  191090,  6, }, // America/Toronto
    {  617, 5605,  191096,  7, }, // America/Tortola
    {  617, 2077,  145780,  7, }, // America/Vancouver
    {  617, 2128,  191103, 10, }, // America/Whitehorse
    {  617,  734,  191113,  7, }, // America/Winnipeg
    {  617, 8515,  191120,  6, }, // America/Yakutat
    {  617, 5918,  103325,  4, }, // Antarctica/Casey
    {  617, 6027,  191126,  4, }, // Antarctica/Davis
    {  617, 6066,  191130, 16, }, // Antarctica/DumontDUrville
    {  617, 8531,  191146,  7, }, // Antarctica/Macquarie
    {  617, 7029,  191153,  5, }, // Antarctica/Mawson
    {  617, 7076,  191158,  8, }, // Antarctica/McMurdo
    {  617, 8552,  191166,  5, }, // Antarctica/Palmer
    {  617, 7189,  103367,  7, }, // Antarctica/Rothera
    {  617, 7258,  103374,  5, }, // Antarctica/Syowa
    {  617, 8570,  191171,  4, }, // Antarctica/Troll
    {  617, 7337,  103379,  6, }, // Antarctica/Vostok
    {  617, 1413,  191175, 13, }, // Arctic/Longyearbyen
    {  617, 5487,   10423,  3, }, // Asia/Aden
    {  617, 5297,  191188,  6, }, // Asia/Almaty
    {  617, 3870,  191194,  5, }, // Asia/Amman
    {  617, 5428,  191199,  5, }, // Asia/Anadyr
    {  617, 5440,  191204,  5, }, // Asia/Aqtau
    {  617, 5270,  191209,  5, }, // Asia/Aqtobe
    {  617,  964,  191214,  5, }, // Asia/Ashgabat
    {  617, 8587,  191219,  6, }, // Asia/Atyrau
    {  617, 3509,   10466,  5, }, // Asia/Baghdad
    {  617, 5451,  145924,  5, }, // Asia/Bahrain
    {  617, 3555,  144949,  4, }, // Asia/Baku
    {  617, 4368,  145929,  5, }, // Asia/Bangkok
    {  617, 3473,  191225,  5, }, // Asia/Barnaul
    {  617, 3988,  145934,  5, }, // Asia/Beirut
    {  617, 3680,  145939,  5, }, // Asia/Bishkek
    {  617, 5906,  148551,  6, }, // Asia/Brunei
    {  617, 4503,  148557,  4, }, // Asia/Chita
    {  617, 5963,  191230,  9, }, // Asia/Choibalsan
    {  617, 4413,  145962,  6, }, // Asia/Colombo
    {  617, 4442,   10528,  4, }, // Asia/Damascus
    {  617, 1060,  191239,  5, }, // Asia/Dhaka
    {  617, 6125,  191244,  3, }, // Asia/Dili
    {  617, 3498,  191247,  4, }, // Asia/Dubai
    {  617, 6092,  191251,  7, }, // Asia/Dushanbe
    {  617, 8599,  191258,  9, }, // Asia/Famagusta
    {  617, 8614,  103548,  3, }, // Asia/Gaza
    {  617, 4697,  103551,  6, }, // Asia/Hebron
    {  617, 1226,  191267, 13, }, // Asia/Ho_Chi_Minh
    {  617, 2576,  103562,  9, }, // Asia/Hong_Kong
    {  617, 4673,  191280,  6, }, // Asia/Hovd
    {  617, 4086,  191286,  6, }, // Asia/Irkutsk
    {  617, 6969,  191292,  6, }, // Asia/Jakarta
    {  617, 6955,  191298,  7, }, // Asia/Jayapura
    {  617, 1257,  103597,  6, }, // Asia/Jerusalem
    {  617, 3462,  146048,  4, }, // Asia/Kabul
    {  617, 4218,  191305,  7, }, // Asia/Kamchatka
    {  617, 4141,  148657,  5, }, // Asia/Karachi
    {  617, 1152,  191312,  8, }, // Asia/Kathmandu
    {  617, 8624,  191320,  6, }, // Asia/Khandyga
    {  617,  992,  191326,  7, }, // Asia/Kolkata
    {  617, 4099,  191333, 11, }, // Asia/Krasnoyarsk
    {  617, 4761,  191344, 11, }, // Asia/Kuala_Lumpur
    {  617, 5893,  191355,  6, }, // Asia/Kuching
    {  617, 5464,  148710,  4, }, // Asia/Kuwait
    {  617, 1178,  191361,  4, }, // Asia/Macau
    {  617, 3919,  191365,  6, }, // Asia/Magadan
    {  617, 1316,  191371,  5, }, // Asia/Makassar
    {  617, 7145,  146127,  5, }, // Asia/Manila
    {  617, 6883,   10734,  4, }, // Asia/Muscat
    {  617, 2446,  191376,  6, }, // Asia/Nicosia
    {  617, 8638,  191382, 12, }, // Asia/Novokuznetsk
    {  617, 4037,  103725, 11, }, // Asia/Novosibirsk
    {  617, 4131,  103736,  5, }, // Asia/Omsk
    {  617, 7108,  148768,  5, }, // Asia/Oral
    {  617, 6924,  146162,  7, }, // Asia/Phnom_Penh
    {  617, 8656,  191394,  8, }, // Asia/Pontianak
    {  617, 4116,  191402, 10, }, // Asia/Pyongyang
    {  617, 5476,   10802,  3, }, // Asia/Qatar
    {  617, 8671,  191412,  7, }, // Asia/Qostanay
    {  617, 4171,  191419,  9, }, // Asia/Qyzylorda
    {  617, 3486,  103788,  4, }, // Asia/Riyadh
    {  617, 4310,  191428,  6, }, // Asia/Sakhalin
    {  617, 7208,   10834,  6, }, // Asia/Samarkand
    {  617, 3116,  191434,  5, }, // Asia/Seoul
    {  617, 1020,  146210,  7, }, // Asia/Shanghai
    {  617, 3137,  148836,  7, }, // Asia/Singapore
    {  617, 4199,  191439, 13, }, // Asia/Srednekolymsk
    {  617, 3100,  191452,  5, }, // Asia/Taipei
    {  617, 4683,  146243,  6, }, // Asia/Tashkent
    {  617, 3817,  191457,  6, }, // Asia/Tbilisi
    {  617, 2638,  191463,  5, }, // Asia/Tehran
    {  617, 1284,  191468,  5, }, // Asia/Thimphu
    {  617, 2687,  103871,  5, }, // Asia/Tokyo
    {  617, 4474,  191473,  5, }, // Asia/Tomsk
    {  617, 1346,  191478, 11, }, // Asia/Ulaanbaatar
    {  617, 1126,  191489,  7, }, // Asia/Urumqi
    {  617, 8685,  191496,  9, }, // Asia/Ust-Nera
    {  617, 6940,  191505,  8, }, // Asia/Vientiane
    {  617, 4612,  191513, 11, }, // Asia/Vladivostok
    {  617, 4730,  191524,  6, }, // Asia/Yakutsk
    {  617, 1202,  161769,  5, }, // Asia/Yangon
    {  617, 3785,  191530, 11, }, // Asia/Yekaterinburg
    {  617, 3649,  191541,  6, }, // Asia/Yerevan
    {  617, 3565,  191547,  6, }, // Atlantic/Azores
    {  617, 5588,   10981,  6, }, // Atlantic/Bermuda
    {  617, 6591,  148240,  5, }, // Atlantic/Canary
    {  617, 3629,  191553,  8, }, // Atlantic/Cape_Verde
    {  617, 1379,   11004,  4, }, // Atlantic/Faroe
    {  617, 8699,  191561,  6, }, // Atlantic/Madeira
    {  617, 2614,  191567,  7, }, // Atlantic/Reykjavik
    {  617, 7235,  104004, 12, }, // Atlantic/South_Georgia
    {  617, 6784,  191574, 11, }, // Atlantic/St_Helena
    {  617, 6607,  191585,  7, }, // Atlantic/Stanley
    {  617, 1654,  191592,  7, }, // Australia/Adelaide
    {  617, 1619,  104040,  6, }, // Australia/Brisbane
    {  617, 1783,  191599,  9, }, // Australia/Broken_Hill
    {  617, 1567,  191608,  5, }, // Australia/Darwin
    {  617, 3539,  191613,  6, }, // Australia/Eucla
    {  617, 1500,  191619,  6, }, // Australia/Hobart
    {  617, 8716,  191625,  7, }, // Australia/Lindeman
    {  617, 1531,  191632,  9, }, // Australia/Lord_Howe
    {  617, 1711,  149061,  6, }, // Australia/Melbourne
    {  617, 1746,  191641,  4, }, // Australia/Perth
    {  617, 1447,  191645,  4, }, // Australia/Sydney
    {  617, 6390,  191649,  8, }, // Europe/Amsterdam
    {  617, 6149,  191657,  6, }, // Europe/Andorra
    {  617, 3522,  191663,  8, }, // Europe/Astrakhan
    {  617, 6577,  191671,  6, }, // Europe/Athens
    {  617, 6285,   11151,  6, }, // Europe/Belgrade
    {  617, 4659,  146501,  4, }, // Europe/Berlin
    {  617, 6437,  191677, 10, }, // Europe/Bratislava
    {  617, 6178,  161927,  5, }, // Europe/Brussels
    {  617, 3830,  191687,  6, }, // Europe/Bucharest
    {  617, 3708,  191693,  6, }, // Europe/Budapest
    {  617, 8751,  191699,  5, }, // Europe/Busingen
    {  617, 2475,  191704,  7, }, // Europe/Chisinau
    {  617, 6238,  104203,  9, }, // Europe/Copenhagen
    {  617, 2260,  191711,  4, }, // Europe/Dublin
    {  617, 6256,  146558, 10, }, // Europe/Gibraltar
    {  617, 8767,  149166,  5, }, // Europe/Guernsey
    {  617, 6561,  191715,  7, }, // Europe/Helsinki
    {  617, 8783,  104242, 10, }, // Europe/Isle_of_Man
    {  617, 1097,  146592,  7, }, // Europe/Istanbul
    {  617, 8802,  104261,  4, }, // Europe/Jersey
    {  617, 3881,  191722, 11, }, // Europe/Kaliningrad
    {  617, 8924,  146609,  5, }, // Europe/Kirov
    {  617, 2419,  101775,  3, }, // Europe/Kyiv
    {  617, 3078,  162026,  4, }, // Europe/Lisbon
    {  617, 6455,  149210,  9, }, // Europe/Ljubljana
    {  617, 2393,   11292,  4, }, // Europe/London
    {  617, 6314,  191733,  7, }, // Europe/Luxembourg
    {  617, 4794,  191740,  5, }, // Europe/Madrid
    {  617, 6346,  104326,  5, }, // Europe/Malta
    {  617, 6531,  191745,  7, }, // Europe/Mariehamn
    {  617, 3595,  104339,  5, }, // Europe/Minsk
    {  617, 6359,  104344,  6, }, // Europe/Monaco
    {  617, 3443,  104350,  5, }, // Europe/Moscow
    {  617, 6407,  104355,  5, }, // Europe/Oslo
    {  617, 4186,  191752,  4, }, // Europe/Paris
    {  617, 6373,  191756,  9, }, // Europe/Podgorica
    {  617, 6224,  104374,  4, }, // Europe/Prague
    {  617, 8816,  147993,  4, }, // Europe/Riga
    {  617, 6273,    9113,  3, }, // Europe/Rome
    {  617, 4233,  146680,  5, }, // Europe/Samara
    {  617, 6419,  191765, 11, }, // Europe/San_Marino
    {  617, 6194,  162095,  7, }, // Europe/Sarajevo
    {  617, 4353,  191776,  6, }, // Europe/Saratov
    {  617, 8828,  162107,  8, }, // Europe/Simferopol
    {  617, 6332,  191782,  7, }, // Europe/Skopje
    {  617, 6548,  191789,  5, }, // Europe/Sofia
    {  617, 6472,  191794,  9, }, // Europe/Stockholm
    {  617, 8846,  191803,  4, }, // Europe/Tallinn
    {  617, 6135,  191807,  6, }, // Europe/Tirane
    {  617, 8861,  191813,  8, }, // Europe/Ulyanovsk
    {  617, 6301,  191821,  5, }, // Europe/Vaduz
    {  617, 6516,  191826,  6, }, // Europe/Vatican
    {  617, 6164,  146754,  5, }, // Europe/Vienna
    {  617, 8878,  191832,  7, }, // Europe/Vilnius
    {  617, 4629,  162168,  9, }, // Europe/Volgograd
    {  617, 3055,  104500,  5, }, // Europe/Warsaw
    {  617, 6210,  191839,  6, }, // Europe/Zagreb
    {  617, 6489,  191845,  5, }, // Europe/Zurich
    {  617, 4980,  191850, 12, }, // Indian/Antananarivo
    {  617, 6910,  104528,  5, }, // Indian/Chagos
    {  617, 5979,  191862,  5, }, // Indian/Christmas
    {  617, 5996,  104540,  5, }, // Indian/Cocos
    {  617, 4931,  104545,  6, }, // Indian/Comoro
    {  617, 6624,  191867,  8, }, // Indian/Kerguelen
    {  617, 7223,  104559,  4, }, // Indian/Mahe
    {  617, 7013,  191875,  6, }, // Indian/Maldives
    {  617, 3971,  191881,  6, }, // Indian/Mauritius
    {  617, 5000,  191887,  5, }, // Indian/Mayotte
    {  617, 7174,  191892,  8, }, // Indian/Reunion
    {  617, 4324,  146843,  4, }, // Pacific/Apia
    {  617,  932,  104597,  6, }, // Pacific/Auckland
    {  617, 3608,  191900,  8, }, // Pacific/Bougainville
    {  617, 2863,  191908,  5, }, // Pacific/Chatham
    {  617, 3022,  191913,  4, }, // Pacific/Chuuk
    {  617, 2201,  104125,  5, }, // Pacific/Easter
    {  617, 7323,  191917,  5, }, // Pacific/Efate
    {  617, 7290,  191922,  6, }, // Pacific/Fakaofo
    {  617, 3804,  191928,  3, }, // Pacific/Fiji
    {  617, 7306,  191931, 10, }, // Pacific/Funafuti
    {  617, 6641,  191941,  9, }, // Pacific/Galapagos
    {  617, 6659,  191950,  7, }, // Pacific/Gambier
    {  617, 3724,  191957, 11, }, // Pacific/Guadalcanal
    {  617, 5935,  190683,  4, }, // Pacific/Guam
    {  617, 2929,  191968,  8, }, // Pacific/Honolulu
    {  617, 2897,  191976,  8, }, // Pacific/Kanton
    {  617, 3900,  191984,  9, }, // Pacific/Kiritimati
    {  617, 6982,  191993,  7, }, // Pacific/Kosrae
    {  617, 2708,  149536,  8, }, // Pacific/Kwajalein
    {  617, 4779,  146946,  5, }, // Pacific/Majuro
    {  617, 3953,  192000,  8, }, // Pacific/Marquesas
    {  617, 8909,  192008,  4, }, // Pacific/Midway
    {  617, 7047,  192012,  5, }, // Pacific/Nauru
    {  617, 7095,  190622,  4, }, // Pacific/Niue
    {  617, 4070,  104742,  6, }, // Pacific/Norfolk
    {  617, 7061,  192017,  5, }, // Pacific/Noumea
    {  617, 2991,  162377,  9, }, // Pacific/Pago_Pago
    {  617, 7118,  192022,  4, }, // Pacific/Palau
    {  617, 7157,  192026,  7, }, // Pacific/Pitcairn
    {  617, 2961,  192033,  7, }, // Pacific/Pohnpei
    {  617, 4709,  192040, 12, }, // Pacific/Port_Moresby
    {  617, 6009,  104791,  9, }, // Pacific/Rarotonga
    {  617, 5948,  192052,  6, }, // Pacific/Saipan
    {  617, 7275,  192058,  6, }, // Pacific/Tahiti
    {  617, 6675,  192064,  5, }, // Pacific/Tarawa
    {  617, 4485,  192069,  9, }, // Pacific/Tongatapu
    {  617, 7355,  104828,  3, }, // Pacific/Wake
    {  617, 7368,  192078,  5, }, // Pacific/Wallis
    {  618, 6723,  144391,  4, }, // Africa/Accra Urdu/Arabic/India
    {  618, 7676,  192083,  9, }, // America/Cambridge_Bay
    {  618, 7887,  192092,  7, }, // America/Glace_Bay
    {  618, 6844,  192099,  6, }, // America/Goose_Bay
    {  618, 4514,  192105,  9, }, // America/Grand_Turk
    {  618, 6862,  192114, 14, }, // America/Scoresbysund
    {  618, 5673,  192128,  5, }, // America/Thule
    {  618, 3708,  192133,  8, }, // Europe/Budapest
    {  619,   14,       0,  6, }, // Africa/Asmara Uyghur/Arabic/China
    {  619, 4337,  192141,  8, }, // Africa/Sao_Tome
    {  619, 4154,  192149, 10, }, // America/Asuncion
    {  619,  237,      22,  8, }, // America/Atikokan
    {  619, 7586,      30, 17, }, // America/Bahia_Banderas
    {  619, 3770,      47,  6, }, // America/Cancun
    {  619, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  619, 5640,  192159,  6, }, // America/Curacao
    {  619,  481,      73, 13, }, // America/Indiana/Knox
    {  619, 7924,      86, 16, }, // America/Indiana/Marengo
    {  619, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  619, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  619, 8001,     139, 14, }, // America/Indiana/Vevay
    {  619, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  619, 8049,     171, 16, }, // America/Indiana/Winamac
    {  619, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  619, 5849,     207, 22, }, // America/Lower_Princes
    {  619, 8183,     229,  6, }, // America/Merida
    {  619, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  619, 8292,     255, 20, }, // America/North_Dakota/Center
    {  619, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  619,  393,     298,  4, }, // America/Nuuk
    {  619, 6862,     302, 16, }, // America/Scoresbysund
    {  619, 8471,  192165, 18, }, // America/St_Barthelemy
    {  619, 2045,     332, 10, }, // America/St_Johns
    {  619, 5780,     342,  9, }, // America/St_Kitts
    {  619, 5797,     351,  9, }, // America/St_Lucia
    {  619,  855,     360, 10, }, // America/St_Thomas
    {  619, 5830,     370, 11, }, // America/St_Vincent
    {  619, 6066,  192183, 15, }, // Antarctica/DumontDUrville
    {  619, 1226,  192198, 14, }, // Asia/Ho_Chi_Minh
    {  619, 1152,     413,  9, }, // Asia/Kathmandu
    {  619,  992,     422,  7, }, // Asia/Kolkata
    {  619, 1178,     429,  5, }, // Asia/Macau
    {  619, 1202,     442,  6, }, // Asia/Yangon
    {  619, 1379,     448,  5, }, // Atlantic/Faroe
    {  619, 6784,     453, 10, }, // Atlantic/St_Helena
    {  619, 2419,     463,  4, }, // Europe/Kyiv
    {  619, 7174,  192212,  9, }, // Indian/Reunion
    {  619, 3022,     474,  5, }, // Pacific/Chuuk
    {  619, 2961,     479,  7, }, // Pacific/Pohnpei
    {  620, 6723,   19687,  5, }, // Africa/Accra Uzbek/Latin/Uzbekistan
    {  620, 4961,   63018, 11, }, // Africa/Addis_Ababa
    {  620, 7383,  192221,  6, }, // Africa/Algiers
    {  620,   14,   89768,  6, }, // Africa/Asmara
    {  620, 5159,   24908,  5, }, // Africa/Bangui
    {  620, 7398,    4583,  5, }, // Africa/Bissau
    {  620, 4859,  192227,  8, }, // Africa/Blantyre
    {  620, 5189,   19729,  9, }, // Africa/Brazzaville
    {  620, 2242,  192235,  6, }, // Africa/Cairo
    {  620, 4019,   19744, 10, }, // Africa/Casablanca
    {  620, 7412,   19754,  5, }, // Africa/Ceuta
    {  620, 6736,    4637,  7, }, // Africa/Conakry
    {  620, 5032,  192241, 12, }, // Africa/Dar_es_Salaam
    {  620, 4945,  151111,  6, }, // Africa/Djibouti
    {  620, 5145,   19777,  5, }, // Africa/Douala
    {  620, 5068,  192253,  7, }, // Africa/El_Aaiun
    {  620, 6816,    4670,  7, }, // Africa/Freetown
    {  620, 4917,  192260,  6, }, // Africa/Harare
    {  620, 4381,  192266, 12, }, // Africa/Johannesburg
    {  620, 4426,   19807,  6, }, // Africa/Khartoum
    {  620, 5238,    4699,  8, }, // Africa/Libreville
    {  620, 4889,  117783,  6, }, // Africa/Maputo
    {  620, 5015,  192278,  9, }, // Africa/Mogadishu
    {  620, 6997,  192287,  9, }, // Africa/Monrovia
    {  620, 3755,  192296,  7, }, // Africa/Nairobi
    {  620, 6690,   24967,  8, }, // Africa/Ouagadougou
    {  620, 4337,  186574,  8, }, // Africa/Sao_Tome
    {  620, 4054,  192303,  7, }, // Africa/Windhoek
    {  620,  132,  192310, 10, }, // America/Adak
    {  620, 3183,  192320,  7, }, // America/Anchorage
    {  620, 5524,   19891,  7, }, // America/Anguilla
    {  620,  166,  192327, 12, }, // America/Argentina/Buenos_Aires
    {  620,   91,    4788,  9, }, // America/Argentina/Catamarca
    {  620,  270,    4797,  7, }, // America/Argentina/Cordoba
    {  620, 7425,  192339,  8, }, // America/Argentina/La_Rioja
    {  620, 7452,  192347, 12, }, // America/Argentina/Rio_Gallegos
    {  620, 7507,  192359,  8, }, // America/Argentina/San_Juan
    {  620, 5497,  186640,  8, }, // America/Argentina/San_Luis
    {  620, 7534,    4850,  7, }, // America/Argentina/Tucuman
    {  620, 7560,  192367,  7, }, // America/Argentina/Ushuaia
    {  620, 4154,  192374,  7, }, // America/Asuncion
    {  620,  237,  192381, 12, }, // America/Atikokan
    {  620, 3581,  192393,  5, }, // America/Bahia
    {  620, 7586,  192398, 15, }, // America/Bahia_Banderas
    {  620, 5309,    4891,  5, }, // America/Belize
    {  620, 7623,    4897, 12, }, // America/Blanc-Sablon
    {  620, 7644,    4909,  9, }, // America/Boa_Vista
    {  620, 7662,   19969,  5, }, // America/Boise
    {  620, 7676,  192413, 11, }, // America/Cambridge_Bay
    {  620, 7698,  186709, 12, }, // America/Campo_Grande
    {  620, 3770,    4959,  6, }, // America/Cancun
    {  620, 4596,    4965,  7, }, // America/Caracas
    {  620, 4247,  192424,  7, }, // America/Cayenne
    {  620, 5398,  192431, 15, }, // America/Cayman
    {  620, 3251,  192446,  7, }, // America/Chicago
    {  620, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  620, 5324,  186755, 10, }, // America/Costa_Rica
    {  620, 7759,    5020,  7, }, // America/Creston
    {  620, 3693,   20036,  6, }, // America/Cuiaba
    {  620, 5640,  192453,  8, }, // America/Curacao
    {  620, 7775,  192461, 12, }, // America/Danmarkshavn
    {  620, 7796,    5053,  6, }, // America/Dawson
    {  620, 7811,  186791, 11, }, // America/Dawson_Creek
    {  620, 3351,   20064,  7, }, // America/Detroit
    {  620, 5656,   20071,  8, }, // America/Dominica
    {  620, 7832,  192473,  8, }, // America/Eirunepe
    {  620, 5343,    5090,  8, }, // America/El_Salvador
    {  620, 7887,  192481,  9, }, // America/Glace_Bay
    {  620, 6844,  192490,  7, }, // America/Goose_Bay
    {  620, 5703,  118241,  9, }, // America/Guadeloupe
    {  620, 3662,   31800,  9, }, // America/Guatemala
    {  620, 4743,  192497,  8, }, // America/Guayaquil
    {  620, 6895,  192505,  6, }, // America/Guyana
    {  620, 1923,  186862,  8, }, // America/Halifax
    {  620, 2221,  192511,  6, }, // America/Havana
    {  620, 7905,  192517,  9, }, // America/Hermosillo
    {  620,  481,  186884, 13, }, // America/Indiana/Knox
    {  620, 7924,      86, 16, }, // America/Indiana/Marengo
    {  620, 7948,  192526, 19, }, // America/Indiana/Petersburg
    {  620, 7975,    5245, 18, }, // America/Indiana/Tell_City
    {  620, 8001,  192545, 14, }, // America/Indiana/Vevay
    {  620, 8023,  192559, 16, }, // America/Indiana/Vincennes
    {  620, 8049,  192575, 16, }, // America/Indiana/Winamac
    {  620,  660,    5310,  7, }, // America/Iqaluit
    {  620, 2665,   20241,  7, }, // America/Jamaica
    {  620, 5282,  192591,  4, }, // America/Juneau
    {  620,  521,  192595,  8, }, // America/Kentucky/Louisville
    {  620, 8088,  192603, 20, }, // America/Kentucky/Monticello
    {  620, 5621,  192623, 10, }, // America/Kralendijk
    {  620, 4278,  186977,  6, }, // America/La_Paz
    {  620, 3390,  186983, 11, }, // America/Los_Angeles
    {  620, 5849,  192633, 21, }, // America/Lower_Princes
    {  620, 8116,   20315,  6, }, // America/Maceio
    {  620, 5814,   60430,  6, }, // America/Marigot
    {  620, 5722,   25027,  9, }, // America/Martinique
    {  620, 2779,  118544,  8, }, // America/Mazatlan
    {  620, 8165,    5409,  8, }, // America/Menominee
    {  620, 8183,    3899,  6, }, // America/Merida
    {  620, 2811,   20343,  6, }, // America/Mexico_City
    {  620, 4293,    5434,  7, }, // America/Miquelon
    {  620, 8217,    5441,  7, }, // America/Moncton
    {  620, 3294,  192654,  8, }, // America/New_York
    {  620, 8251,   20373,  3, }, // America/Nome
    {  620, 1834,   20376,  7, }, // America/Noronha
    {  620, 8264,  192662, 22, }, // America/North_Dakota/Beulah
    {  620, 8292,  192684, 23, }, // America/North_Dakota/Center
    {  620, 8320,  192707, 26, }, // America/North_Dakota/New_Salem
    {  620,  393,  192733,  6, }, // America/Nuuk
    {  620, 8351,  192739,  7, }, // America/Ojinaga
    {  620, 3224,    5568,  6, }, // America/Phoenix
    {  620, 3847,  158616, 12, }, // America/Port-au-Prince
    {  620, 5871,  192746, 13, }, // America/Port_of_Spain
    {  620, 8367,  192759, 11, }, // America/Porto_Velho
    {  620, 5760,  187148, 11, }, // America/Puerto_Rico
    {  620, 3932,    5607, 12, }, // America/Punta_Arenas
    {  620, 8387,   92307, 12, }, // America/Rankin_Inlet
    {  620, 8408,  118755,  6, }, // America/Recife
    {  620, 1979,  192770,  7, }, // America/Regina
    {  620, 8423,   20498,  8, }, // America/Resolute
    {  620,  695,  187174, 10, }, // America/Rio_Branco
    {  620, 2165,  192777,  8, }, // America/Santiago
    {  620, 6044,    5647, 13, }, // America/Santo_Domingo
    {  620, 1862,  187192,  9, }, // America/Sao_Paulo
    {  620, 6862,  192785, 16, }, // America/Scoresbysund
    {  620, 8471,    5681, 13, }, // America/St_Barthelemy
    {  620, 2045,  187217,  9, }, // America/St_Johns
    {  620, 5780,  187226,  9, }, // America/St_Kitts
    {  620, 5797,  192801, 12, }, // America/St_Lucia
    {  620,  855,  187247, 10, }, // America/St_Thomas
    {  620, 5830,  192813, 12, }, // America/St_Vincent
    {  620, 8493,  192825, 13, }, // America/Swift_Current
    {  620, 5363,    5759, 11, }, // America/Tegucigalpa
    {  620, 5673,  118909,  4, }, // America/Thule
    {  620,  313,   20645,  7, }, // America/Tijuana
    {  620, 2077,    5789,  8, }, // America/Vancouver
    {  620, 2128,  192838,  8, }, // America/Whitehorse
    {  620,  734,  192846,  8, }, // America/Winnipeg
    {  620, 5918,   20668,  5, }, // Antarctica/Casey
    {  620, 6027,   20673,  6, }, // Antarctica/Davis
    {  620, 6066,  192854, 15, }, // Antarctica/DumontDUrville
    {  620, 8531,  187330,  8, }, // Antarctica/Macquarie
    {  620, 7029,   20686,  6, }, // Antarctica/Mawson
    {  620, 7076,  187338,  9, }, // Antarctica/McMurdo
    {  620, 7189,  118999,  6, }, // Antarctica/Rothera
    {  620, 7258,   20701,  5, }, // Antarctica/Syowa
    {  620, 1413,  192869,  7, }, // Arctic/Longyearbyen
    {  620, 5487,  192876,  4, }, // Asia/Aden
    {  620, 5297,    5890,  6, }, // Asia/Almaty
    {  620, 3870,  192880,  5, }, // Asia/Amman
    {  620, 5428,    5900,  6, }, // Asia/Anadyr
    {  620, 5440,  192885,  5, }, // Asia/Aqtau
    {  620, 5270,  192890,  7, }, // Asia/Aqtobe
    {  620,  964,  192897,  8, }, // Asia/Ashgabat
    {  620, 8587,    5917,  6, }, // Asia/Atyrau
    {  620, 3509,  192905,  7, }, // Asia/Baghdad
    {  620, 5451,  192912,  7, }, // Asia/Bahrain
    {  620, 3555,  192919,  4, }, // Asia/Baku
    {  620, 3988,   34483,  6, }, // Asia/Beirut
    {  620, 5906,   20774,  6, }, // Asia/Brunei
    {  620, 5963,  139766, 10, }, // Asia/Choibalsan
    {  620, 4413,    5949,  7, }, // Asia/Colombo
    {  620, 4442,  192923,  7, }, // Asia/Damascus
    {  620, 1060,   90078,  5, }, // Asia/Dhaka
    {  620, 3498,   20800,  5, }, // Asia/Dubai
    {  620, 8614,  192930,  5, }, // Asia/Gaza
    {  620, 4697,  192935,  6, }, // Asia/Hebron
    {  620, 1226,  192941,  8, }, // Asia/Ho_Chi_Minh
    {  620, 2576,  187410,  7, }, // Asia/Hong_Kong
    {  620, 4673,  192949,  4, }, // Asia/Hovd
    {  620, 6955,  192953,  6, }, // Asia/Jayapura
    {  620, 1257,  192959,  6, }, // Asia/Jerusalem
    {  620, 3462,  192965,  5, }, // Asia/Kabul
    {  620, 1152,    6018,  8, }, // Asia/Kathmandu
    {  620, 8624,  192970,  7, }, // Asia/Khandyga
    {  620,  992,   62730,  8, }, // Asia/Kolkata
    {  620, 4761,    6040, 12, }, // Asia/Kuala_Lumpur
    {  620, 5464,  192977,  6, }, // Asia/Kuwait
    {  620, 1178,    6064,  5, }, // Asia/Macau
    {  620, 1316,    6069,  7, }, // Asia/Makassar
    {  620, 6883,   25325,  6, }, // Asia/Muscat
    {  620, 2446,   20920,  8, }, // Asia/Nicosia
    {  620, 7108,   56586,  6, }, // Asia/Oral
    {  620, 6924,   61058,  7, }, // Asia/Phnom_Penh
    {  620, 4116,   20936,  7, }, // Asia/Pyongyang
    {  620, 8671,   97598,  8, }, // Asia/Qostanay
    {  620, 4171,  192983, 10, }, // Asia/Qyzylorda
    {  620, 3486,  192993,  8, }, // Asia/Riyadh
    {  620, 4310,   20952,  7, }, // Asia/Sakhalin
    {  620, 7208,  193001,  9, }, // Asia/Samarkand
    {  620, 3116,    6131,  4, }, // Asia/Seoul
    {  620, 1020,  193010,  7, }, // Asia/Shanghai
    {  620, 3137,   19291,  8, }, // Asia/Singapore
    {  620, 4199,    6150, 13, }, // Asia/Srednekolymsk
    {  620, 3100,  193017,  6, }, // Asia/Taipei
    {  620, 4683,   34590,  8, }, // Asia/Tashkent
    {  620, 2638,  193023,  6, }, // Asia/Tehran
    {  620, 2687,    4257,  5, }, // Asia/Tokyo
    {  620, 1346,  139650, 10, }, // Asia/Ulaanbaatar
    {  620, 1126,  193029,  7, }, // Asia/Urumqi
    {  620, 6940,   21023,  8, }, // Asia/Vientiane
    {  620, 1202,    6213,  6, }, // Asia/Yangon
    {  620, 3565,  193036, 13, }, // Atlantic/Azores
    {  620, 5588,  193049, 16, }, // Atlantic/Bermuda
    {  620, 6591,  193065, 14, }, // Atlantic/Canary
    {  620, 3629,  193079, 10, }, // Atlantic/Cape_Verde
    {  620, 1379,  193089, 14, }, // Atlantic/Faroe
    {  620, 8699,  193103, 13, }, // Atlantic/Madeira
    {  620, 2614,   21086,  9, }, // Atlantic/Reykjavik
    {  620, 7235,  193116, 16, }, // Atlantic/South_Georgia
    {  620, 6784,  193132, 21, }, // Atlantic/St_Helena
    {  620, 6607,    6298,  6, }, // Atlantic/Stanley
    {  620, 1654,   19421,  8, }, // Australia/Adelaide
    {  620, 1619,  119584,  7, }, // Australia/Brisbane
    {  620, 1783,  193153, 11, }, // Australia/Broken_Hill
    {  620, 1567,    6331,  6, }, // Australia/Darwin
    {  620, 3539,  193164,  5, }, // Australia/Eucla
    {  620, 1500,  193169,  6, }, // Australia/Hobart
    {  620, 1531,  193175, 14, }, // Australia/Lord_Howe
    {  620, 1711,    6359,  7, }, // Australia/Melbourne
    {  620, 1746,   21145,  4, }, // Australia/Perth
    {  620, 1447,   21149,  6, }, // Australia/Sydney
    {  620, 3522,  113434,  8, }, // Europe/Astrakhan
    {  620, 6577,   21164,  5, }, // Europe/Athens
    {  620, 6285,   25465,  7, }, // Europe/Belgrade
    {  620, 6178,  193189,  8, }, // Europe/Brussels
    {  620, 3830,   21183,  8, }, // Europe/Bucharest
    {  620, 3708,  193197,  9, }, // Europe/Budapest
    {  620, 8751,  193206,  9, }, // Europe/Busingen
    {  620, 2475,  193215,  9, }, // Europe/Chisinau
    {  620, 6238,  187743, 10, }, // Europe/Copenhagen
    {  620, 8767,  122872,  6, }, // Europe/Guernsey
    {  620, 6561,  193224,  8, }, // Europe/Helsinki
    {  620, 8783,  193232,  9, }, // Europe/Isle_of_Man
    {  620, 8802,  187762,  5, }, // Europe/Jersey
    {  620, 2419,   21259,  5, }, // Europe/Kyiv
    {  620, 3078,    4400,  8, }, // Europe/Lisbon
    {  620, 6455,   21264,  9, }, // Europe/Ljubljana
    {  620, 6314,  193241, 11, }, // Europe/Luxembourg
    {  620, 6531,  193252, 10, }, // Europe/Mariehamn
    {  620, 6359,    6525,  6, }, // Europe/Monaco
    {  620, 3443,   21290,  6, }, // Europe/Moscow
    {  620, 4186,   57467,  5, }, // Europe/Paris
    {  620, 6373,  193262, 10, }, // Europe/Podgorica
    {  620, 6224,   19576,  5, }, // Europe/Prague
    {  620, 6273,   32211,  3, }, // Europe/Rome
    {  620, 6419,    6558, 10, }, // Europe/San_Marino
    {  620, 6194,   21315,  8, }, // Europe/Sarajevo
    {  620, 6332,   21323,  6, }, // Europe/Skopje
    {  620, 6548,   21329,  6, }, // Europe/Sofia
    {  620, 6472,  187836,  8, }, // Europe/Stockholm
    {  620, 8846,   21335,  6, }, // Europe/Tallinn
    {  620, 6135,   19608,  6, }, // Europe/Tirane
    {  620, 6301,   21341,  6, }, // Europe/Vaduz
    {  620, 6516,    6614,  7, }, // Europe/Vatican
    {  620, 6164,  193272,  4, }, // Europe/Vienna
    {  620, 8878,   97904,  7, }, // Europe/Vilnius
    {  620, 3055,  193276,  8, }, // Europe/Warsaw
    {  620, 6489,  193284,  6, }, // Europe/Zurich
    {  620, 4980,   97916, 12, }, // Indian/Antananarivo
    {  620, 5979,  193290, 15, }, // Indian/Christmas
    {  620, 5996,  193305, 14, }, // Indian/Cocos
    {  620, 4931,  193319, 14, }, // Indian/Comoro
    {  620, 6624,   21401,  8, }, // Indian/Kerguelen
    {  620, 7223,  120077,  3, }, // Indian/Mahe
    {  620, 7013,  193333, 15, }, // Indian/Maldives
    {  620, 3971,  193348,  8, }, // Indian/Mauritius
    {  620, 5000,  193356,  7, }, // Indian/Mayotte
    {  620, 7174,  193363,  8, }, // Indian/Reunion
    {  620,  932,  120117,  6, }, // Pacific/Auckland
    {  620, 3608,  120124,  8, }, // Pacific/Bougainville
    {  620, 2863,  193371, 12, }, // Pacific/Chatham
    {  620, 3022,  193383, 13, }, // Pacific/Chuuk
    {  620, 2201,  193396, 11, }, // Pacific/Easter
    {  620, 6659,  193407, 12, }, // Pacific/Gambier
    {  620, 3724,  120188, 11, }, // Pacific/Guadalcanal
    {  620, 2929,  188017,  8, }, // Pacific/Honolulu
    {  620, 2897,  193419, 15, }, // Pacific/Kanton
    {  620, 2708,   21509,  9, }, // Pacific/Kwajalein
    {  620, 3953,  193434, 15, }, // Pacific/Marquesas
    {  620, 8909,  193449, 15, }, // Pacific/Midway
    {  620, 7061,  120279,  5, }, // Pacific/Noumea
    {  620, 2991,    6764,  9, }, // Pacific/Pago_Pago
    {  620, 7157,    6773,  7, }, // Pacific/Pitcairn
    {  620, 2961,  193464, 12, }, // Pacific/Pohnpei
    {  620, 4709,  188061, 11, }, // Pacific/Port_Moresby
    {  620, 5948,   21568,  6, }, // Pacific/Saipan
    {  620, 7275,  193476, 11, }, // Pacific/Tahiti
    {  620, 6675,   21574,  6, }, // Pacific/Tarawa
    {  620, 7355,  193487, 10, }, // Pacific/Wake
    {  620, 7368,    6818,  6, }, // Pacific/Wallis
    {  621,   14,       0,  6, }, // Africa/Asmara Uzbek/Arabic/Afghanistan
    {  621, 4337,       6,  8, }, // Africa/Sao_Tome
    {  621, 4154,      14,  8, }, // America/Asuncion
    {  621,  237,      22,  8, }, // America/Atikokan
    {  621, 7586,      30, 17, }, // America/Bahia_Banderas
    {  621, 3770,      47,  6, }, // America/Cancun
    {  621, 5640,      66,  7, }, // America/Curacao
    {  621,  481,      73, 13, }, // America/Indiana/Knox
    {  621, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  621, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  621, 8001,     139, 14, }, // America/Indiana/Vevay
    {  621, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  621, 8049,     171, 16, }, // America/Indiana/Winamac
    {  621, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  621, 5849,     207, 22, }, // America/Lower_Princes
    {  621, 8183,     229,  6, }, // America/Merida
    {  621, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  621, 8292,     255, 20, }, // America/North_Dakota/Center
    {  621, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  621,  393,     298,  4, }, // America/Nuuk
    {  621, 6862,     302, 16, }, // America/Scoresbysund
    {  621, 8471,     318, 14, }, // America/St_Barthelemy
    {  621, 2045,     332, 10, }, // America/St_Johns
    {  621, 5780,     342,  9, }, // America/St_Kitts
    {  621, 5797,     351,  9, }, // America/St_Lucia
    {  621,  855,     360, 10, }, // America/St_Thomas
    {  621, 5830,     370, 11, }, // America/St_Vincent
    {  621, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  621, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  621, 3462,  146048,  4, }, // Asia/Kabul
    {  621, 1152,     413,  9, }, // Asia/Kathmandu
    {  621,  992,     422,  7, }, // Asia/Kolkata
    {  621, 1178,     429,  5, }, // Asia/Macau
    {  621, 1202,     442,  6, }, // Asia/Yangon
    {  621, 1379,     448,  5, }, // Atlantic/Faroe
    {  621, 6784,     453, 10, }, // Atlantic/St_Helena
    {  621, 2419,     463,  4, }, // Europe/Kyiv
    {  621, 7174,     467,  7, }, // Indian/Reunion
    {  621, 3022,     474,  5, }, // Pacific/Chuuk
    {  621, 2961,     479,  7, }, // Pacific/Pohnpei
    {  622,   14,       0,  6, }, // Africa/Asmara Uzbek/Cyrillic/Uzbekistan
    {  622, 4337,       6,  8, }, // Africa/Sao_Tome
    {  622, 4154,      14,  8, }, // America/Asuncion
    {  622,  237,      22,  8, }, // America/Atikokan
    {  622, 7586,      30, 17, }, // America/Bahia_Banderas
    {  622, 3770,      47,  6, }, // America/Cancun
    {  622, 5640,      66,  7, }, // America/Curacao
    {  622,  481,      73, 13, }, // America/Indiana/Knox
    {  622, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  622, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  622, 8001,     139, 14, }, // America/Indiana/Vevay
    {  622, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  622, 8049,     171, 16, }, // America/Indiana/Winamac
    {  622, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  622, 5849,  193497, 22, }, // America/Lower_Princes
    {  622, 8183,     229,  6, }, // America/Merida
    {  622, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  622, 8292,     255, 20, }, // America/North_Dakota/Center
    {  622, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  622,  393,     298,  4, }, // America/Nuuk
    {  622, 6862,     302, 16, }, // America/Scoresbysund
    {  622, 8471,     318, 14, }, // America/St_Barthelemy
    {  622, 2045,     332, 10, }, // America/St_Johns
    {  622, 5780,     342,  9, }, // America/St_Kitts
    {  622, 5797,     351,  9, }, // America/St_Lucia
    {  622,  855,     360, 10, }, // America/St_Thomas
    {  622, 5830,     370, 11, }, // America/St_Vincent
    {  622, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  622, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  622, 1152,     413,  9, }, // Asia/Kathmandu
    {  622,  992,     422,  7, }, // Asia/Kolkata
    {  622, 1178,     429,  5, }, // Asia/Macau
    {  622, 1202,     442,  6, }, // Asia/Yangon
    {  622, 1379,     448,  5, }, // Atlantic/Faroe
    {  622, 6784,     453, 10, }, // Atlantic/St_Helena
    {  622, 2419,     463,  4, }, // Europe/Kyiv
    {  622, 7174,     467,  7, }, // Indian/Reunion
    {  622, 3022,     474,  5, }, // Pacific/Chuuk
    {  622, 2961,     479,  7, }, // Pacific/Pohnpei
    {  623,   14,       0,  6, }, // Africa/Asmara Vai/Vai/Liberia
    {  623, 4337,       6,  8, }, // Africa/Sao_Tome
    {  623, 4154,      14,  8, }, // America/Asuncion
    {  623,  237,      22,  8, }, // America/Atikokan
    {  623, 7586,      30, 17, }, // America/Bahia_Banderas
    {  623, 3770,      47,  6, }, // America/Cancun
    {  623, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  623, 5640,      66,  7, }, // America/Curacao
    {  623,  481,      73, 13, }, // America/Indiana/Knox
    {  623, 7924,      86, 16, }, // America/Indiana/Marengo
    {  623, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  623, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  623, 8001,     139, 14, }, // America/Indiana/Vevay
    {  623, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  623, 8049,     171, 16, }, // America/Indiana/Winamac
    {  623, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  623, 5849,     207, 22, }, // America/Lower_Princes
    {  623, 8183,     229,  6, }, // America/Merida
    {  623, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  623, 8292,     255, 20, }, // America/North_Dakota/Center
    {  623, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  623,  393,     298,  4, }, // America/Nuuk
    {  623, 6862,     302, 16, }, // America/Scoresbysund
    {  623, 8471,     318, 14, }, // America/St_Barthelemy
    {  623, 2045,     332, 10, }, // America/St_Johns
    {  623, 5780,     342,  9, }, // America/St_Kitts
    {  623, 5797,     351,  9, }, // America/St_Lucia
    {  623,  855,     360, 10, }, // America/St_Thomas
    {  623, 5830,     370, 11, }, // America/St_Vincent
    {  623, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  623, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  623, 1152,     413,  9, }, // Asia/Kathmandu
    {  623,  992,     422,  7, }, // Asia/Kolkata
    {  623, 1178,     429,  5, }, // Asia/Macau
    {  623, 1202,     442,  6, }, // Asia/Yangon
    {  623, 1379,     448,  5, }, // Atlantic/Faroe
    {  623, 6784,     453, 10, }, // Atlantic/St_Helena
    {  623, 2419,     463,  4, }, // Europe/Kyiv
    {  623, 7174,     467,  7, }, // Indian/Reunion
    {  623, 3022,     474,  5, }, // Pacific/Chuuk
    {  623, 2961,     479,  7, }, // Pacific/Pohnpei
    {  625,   14,       0,  6, }, // Africa/Asmara Venda/Latin/South Africa
    {  625, 4337,       6,  8, }, // Africa/Sao_Tome
    {  625, 4154,      14,  8, }, // America/Asuncion
    {  625,  237,      22,  8, }, // America/Atikokan
    {  625, 7586,      30, 17, }, // America/Bahia_Banderas
    {  625, 3770,      47,  6, }, // America/Cancun
    {  625, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  625, 5640,      66,  7, }, // America/Curacao
    {  625,  481,      73, 13, }, // America/Indiana/Knox
    {  625, 7924,      86, 16, }, // America/Indiana/Marengo
    {  625, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  625, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  625, 8001,     139, 14, }, // America/Indiana/Vevay
    {  625, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  625, 8049,     171, 16, }, // America/Indiana/Winamac
    {  625, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  625, 5849,     207, 22, }, // America/Lower_Princes
    {  625, 8183,     229,  6, }, // America/Merida
    {  625, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  625, 8292,     255, 20, }, // America/North_Dakota/Center
    {  625, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  625,  393,     298,  4, }, // America/Nuuk
    {  625, 6862,     302, 16, }, // America/Scoresbysund
    {  625, 8471,     318, 14, }, // America/St_Barthelemy
    {  625, 2045,     332, 10, }, // America/St_Johns
    {  625, 5780,     342,  9, }, // America/St_Kitts
    {  625, 5797,     351,  9, }, // America/St_Lucia
    {  625,  855,     360, 10, }, // America/St_Thomas
    {  625, 5830,     370, 11, }, // America/St_Vincent
    {  625, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  625, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  625, 1152,     413,  9, }, // Asia/Kathmandu
    {  625,  992,     422,  7, }, // Asia/Kolkata
    {  625, 1178,     429,  5, }, // Asia/Macau
    {  625, 1202,     442,  6, }, // Asia/Yangon
    {  625, 1379,     448,  5, }, // Atlantic/Faroe
    {  625, 6784,     453, 10, }, // Atlantic/St_Helena
    {  625, 2419,     463,  4, }, // Europe/Kyiv
    {  625, 7174,     467,  7, }, // Indian/Reunion
    {  625, 3022,     474,  5, }, // Pacific/Chuuk
    {  625, 2961,     479,  7, }, // Pacific/Pohnpei
    {  626,   14,       0,  6, }, // Africa/Asmara Vietnamese/Latin/Vietnam
    {  626, 4337,       6,  8, }, // Africa/Sao_Tome
    {  626, 4154,      14,  8, }, // America/Asuncion
    {  626,  237,      22,  8, }, // America/Atikokan
    {  626, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  626, 3770,    3861,  6, }, // America/Cancun
    {  626, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  626, 5640,      66,  7, }, // America/Curacao
    {  626,  481,      73, 13, }, // America/Indiana/Knox
    {  626, 7924,      86, 16, }, // America/Indiana/Marengo
    {  626, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  626, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  626, 8001,     139, 14, }, // America/Indiana/Vevay
    {  626, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  626, 8049,     171, 16, }, // America/Indiana/Winamac
    {  626, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  626, 5849,     207, 22, }, // America/Lower_Princes
    {  626, 8183,    3899,  6, }, // America/Merida
    {  626, 8264,  193519, 18, }, // America/North_Dakota/Beulah
    {  626, 8292,  193537, 18, }, // America/North_Dakota/Center
    {  626, 8320,  193555, 21, }, // America/North_Dakota/New_Salem
    {  626,  393,     298,  4, }, // America/Nuuk
    {  626, 6862,     302, 16, }, // America/Scoresbysund
    {  626, 8471,     318, 14, }, // America/St_Barthelemy
    {  626, 2045,     332, 10, }, // America/St_Johns
    {  626, 5780,     342,  9, }, // America/St_Kitts
    {  626, 5797,     351,  9, }, // America/St_Lucia
    {  626,  855,     360, 10, }, // America/St_Thomas
    {  626, 5830,     370, 11, }, // America/St_Vincent
    {  626, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  626, 1226,  193576, 14, }, // Asia/Ho_Chi_Minh
    {  626, 2576,  193590,  9, }, // Asia/Hong_Kong
    {  626, 1152,     413,  9, }, // Asia/Kathmandu
    {  626,  992,     422,  7, }, // Asia/Kolkata
    {  626, 1178,  193599,  6, }, // Asia/Macau
    {  626, 4116,  193605, 11, }, // Asia/Pyongyang
    {  626, 8671,     434,  8, }, // Asia/Qostanay
    {  626, 1020,  193616, 10, }, // Asia/Shanghai
    {  626, 3100,  193626,  7, }, // Asia/Taipei
    {  626, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  626, 6940,  193633, 10, }, // Asia/Vientiane
    {  626, 1202,   56146,  7, }, // Asia/Yangon
    {  626, 1379,     448,  5, }, // Atlantic/Faroe
    {  626, 7235,  193643, 11, }, // Atlantic/South_Georgia
    {  626, 6784,     453, 10, }, // Atlantic/St_Helena
    {  626, 8783,  193654,  7, }, // Europe/Isle_of_Man
    {  626, 2419,    6486,  4, }, // Europe/Kyiv
    {  626, 3443,  193661, 10, }, // Europe/Moscow
    {  626, 6224,   34765,  5, }, // Europe/Prague
    {  626, 7174,     467,  7, }, // Indian/Reunion
    {  626, 3022,     474,  5, }, // Pacific/Chuuk
    {  626, 2929,    4512,  8, }, // Pacific/Honolulu
    {  626, 2897,    4520,  9, }, // Pacific/Kanton
    {  626, 2961,     479,  7, }, // Pacific/Pohnpei
    {  627,   14,       0,  6, }, // Africa/Asmara Volapuk/Latin/world
    {  627, 4337,       6,  8, }, // Africa/Sao_Tome
    {  627, 4154,      14,  8, }, // America/Asuncion
    {  627,  237,      22,  8, }, // America/Atikokan
    {  627, 7586,      30, 17, }, // America/Bahia_Banderas
    {  627, 3770,      47,  6, }, // America/Cancun
    {  627, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  627, 5640,      66,  7, }, // America/Curacao
    {  627,  481,      73, 13, }, // America/Indiana/Knox
    {  627, 7924,      86, 16, }, // America/Indiana/Marengo
    {  627, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  627, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  627, 8001,     139, 14, }, // America/Indiana/Vevay
    {  627, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  627, 8049,     171, 16, }, // America/Indiana/Winamac
    {  627, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  627, 5849,     207, 22, }, // America/Lower_Princes
    {  627, 8183,     229,  6, }, // America/Merida
    {  627, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  627, 8292,     255, 20, }, // America/North_Dakota/Center
    {  627, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  627,  393,     298,  4, }, // America/Nuuk
    {  627, 6862,     302, 16, }, // America/Scoresbysund
    {  627, 8471,     318, 14, }, // America/St_Barthelemy
    {  627, 2045,     332, 10, }, // America/St_Johns
    {  627, 5780,     342,  9, }, // America/St_Kitts
    {  627, 5797,     351,  9, }, // America/St_Lucia
    {  627,  855,     360, 10, }, // America/St_Thomas
    {  627, 5830,     370, 11, }, // America/St_Vincent
    {  627, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  627, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  627, 1152,     413,  9, }, // Asia/Kathmandu
    {  627,  992,     422,  7, }, // Asia/Kolkata
    {  627, 1178,     429,  5, }, // Asia/Macau
    {  627, 1202,     442,  6, }, // Asia/Yangon
    {  627, 1379,     448,  5, }, // Atlantic/Faroe
    {  627, 6784,     453, 10, }, // Atlantic/St_Helena
    {  627, 2419,     463,  4, }, // Europe/Kyiv
    {  627, 7174,     467,  7, }, // Indian/Reunion
    {  627, 3022,     474,  5, }, // Pacific/Chuuk
    {  627, 2961,     479,  7, }, // Pacific/Pohnpei
    {  628,   14,       0,  6, }, // Africa/Asmara Vunjo/Latin/Tanzania
    {  628, 4337,       6,  8, }, // Africa/Sao_Tome
    {  628, 4154,      14,  8, }, // America/Asuncion
    {  628,  237,      22,  8, }, // America/Atikokan
    {  628, 7586,      30, 17, }, // America/Bahia_Banderas
    {  628, 3770,      47,  6, }, // America/Cancun
    {  628, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  628, 5640,      66,  7, }, // America/Curacao
    {  628,  481,      73, 13, }, // America/Indiana/Knox
    {  628, 7924,      86, 16, }, // America/Indiana/Marengo
    {  628, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  628, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  628, 8001,     139, 14, }, // America/Indiana/Vevay
    {  628, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  628, 8049,     171, 16, }, // America/Indiana/Winamac
    {  628, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  628, 5849,     207, 22, }, // America/Lower_Princes
    {  628, 8183,     229,  6, }, // America/Merida
    {  628, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  628, 8292,     255, 20, }, // America/North_Dakota/Center
    {  628, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  628,  393,     298,  4, }, // America/Nuuk
    {  628, 6862,     302, 16, }, // America/Scoresbysund
    {  628, 8471,     318, 14, }, // America/St_Barthelemy
    {  628, 2045,     332, 10, }, // America/St_Johns
    {  628, 5780,     342,  9, }, // America/St_Kitts
    {  628, 5797,     351,  9, }, // America/St_Lucia
    {  628,  855,     360, 10, }, // America/St_Thomas
    {  628, 5830,     370, 11, }, // America/St_Vincent
    {  628, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  628, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  628, 1152,     413,  9, }, // Asia/Kathmandu
    {  628,  992,     422,  7, }, // Asia/Kolkata
    {  628, 1178,     429,  5, }, // Asia/Macau
    {  628, 1202,     442,  6, }, // Asia/Yangon
    {  628, 1379,     448,  5, }, // Atlantic/Faroe
    {  628, 6784,     453, 10, }, // Atlantic/St_Helena
    {  628, 2419,     463,  4, }, // Europe/Kyiv
    {  628, 7174,     467,  7, }, // Indian/Reunion
    {  628, 3022,     474,  5, }, // Pacific/Chuuk
    {  628, 2961,     479,  7, }, // Pacific/Pohnpei
    {  629,   14,       0,  6, }, // Africa/Asmara Walloon/Latin/Belgium
    {  629, 4337,       6,  8, }, // Africa/Sao_Tome
    {  629, 4154,      14,  8, }, // America/Asuncion
    {  629,  237,      22,  8, }, // America/Atikokan
    {  629, 7586,      30, 17, }, // America/Bahia_Banderas
    {  629, 3770,      47,  6, }, // America/Cancun
    {  629, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  629, 5640,      66,  7, }, // America/Curacao
    {  629,  481,      73, 13, }, // America/Indiana/Knox
    {  629, 7924,      86, 16, }, // America/Indiana/Marengo
    {  629, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  629, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  629, 8001,     139, 14, }, // America/Indiana/Vevay
    {  629, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  629, 8049,     171, 16, }, // America/Indiana/Winamac
    {  629, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  629, 5849,     207, 22, }, // America/Lower_Princes
    {  629, 8183,     229,  6, }, // America/Merida
    {  629, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  629, 8292,     255, 20, }, // America/North_Dakota/Center
    {  629, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  629,  393,     298,  4, }, // America/Nuuk
    {  629, 6862,     302, 16, }, // America/Scoresbysund
    {  629, 8471,     318, 14, }, // America/St_Barthelemy
    {  629, 2045,     332, 10, }, // America/St_Johns
    {  629, 5780,     342,  9, }, // America/St_Kitts
    {  629, 5797,     351,  9, }, // America/St_Lucia
    {  629,  855,     360, 10, }, // America/St_Thomas
    {  629, 5830,     370, 11, }, // America/St_Vincent
    {  629, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  629, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  629, 1152,     413,  9, }, // Asia/Kathmandu
    {  629,  992,     422,  7, }, // Asia/Kolkata
    {  629, 1178,     429,  5, }, // Asia/Macau
    {  629, 1202,     442,  6, }, // Asia/Yangon
    {  629, 1379,     448,  5, }, // Atlantic/Faroe
    {  629, 6784,     453, 10, }, // Atlantic/St_Helena
    {  629, 2419,     463,  4, }, // Europe/Kyiv
    {  629, 7174,     467,  7, }, // Indian/Reunion
    {  629, 3022,     474,  5, }, // Pacific/Chuuk
    {  629, 2961,     479,  7, }, // Pacific/Pohnpei
    {  630, 6723,    4550,  4, }, // Africa/Accra Walser/Latin/Switzerland
    {  630, 7383,   57088,  6, }, // Africa/Algiers
    {  630,   14,       0,  6, }, // Africa/Asmara
    {  630, 2242,   24913,  5, }, // Africa/Cairo
    {  630, 4945,  193671,  7, }, // Africa/Djibouti
    {  630, 4426,    4693,  6, }, // Africa/Khartoum
    {  630, 5015,   56254,  8, }, // Africa/Mogadishu
    {  630, 6690,   97031,  8, }, // Africa/Ouagadougou
    {  630, 4337,       6,  8, }, // Africa/Sao_Tome
    {  630, 2732,   56289,  8, }, // Africa/Tripoli
    {  630,  270,    4797,  7, }, // America/Argentina/Cordoba
    {  630, 4154,      14,  8, }, // America/Asuncion
    {  630,  237,      22,  8, }, // America/Atikokan
    {  630, 7586,      30, 17, }, // America/Bahia_Banderas
    {  630, 3770,      47,  6, }, // America/Cancun
    {  630, 5398,  193678, 11, }, // America/Cayman
    {  630, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  630, 5640,      66,  7, }, // America/Curacao
    {  630, 2221,  193689,  7, }, // America/Havana
    {  630,  481,      73, 13, }, // America/Indiana/Knox
    {  630, 7924,      86, 16, }, // America/Indiana/Marengo
    {  630, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  630, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  630, 8001,     139, 14, }, // America/Indiana/Vevay
    {  630, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  630, 8049,     171, 16, }, // America/Indiana/Winamac
    {  630, 2665,    3892,  7, }, // America/Jamaica
    {  630, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  630, 5849,     207, 22, }, // America/Lower_Princes
    {  630, 8183,     229,  6, }, // America/Merida
    {  630, 5741,   20357,  8, }, // America/Montserrat
    {  630, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  630, 8292,     255, 20, }, // America/North_Dakota/Center
    {  630, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  630,  393,     298,  4, }, // America/Nuuk
    {  630, 6862,     302, 16, }, // America/Scoresbysund
    {  630, 8471,     318, 14, }, // America/St_Barthelemy
    {  630, 2045,     332, 10, }, // America/St_Johns
    {  630, 5780,     342,  9, }, // America/St_Kitts
    {  630, 5797,     351,  9, }, // America/St_Lucia
    {  630,  855,     360, 10, }, // America/St_Thomas
    {  630, 5830,     370, 11, }, // America/St_Vincent
    {  630, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  630, 3509,    4063,  6, }, // Asia/Baghdad
    {  630, 4442,    4102,  8, }, // Asia/Damascus
    {  630, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  630, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  630, 1152,     413,  9, }, // Asia/Kathmandu
    {  630,  992,     422,  7, }, // Asia/Kolkata
    {  630, 1178,     429,  5, }, // Asia/Macau
    {  630, 3137,   19291,  8, }, // Asia/Singapore
    {  630, 2687,    4257,  5, }, // Asia/Tokyo
    {  630, 1202,     442,  6, }, // Asia/Yangon
    {  630, 1379,     448,  5, }, // Atlantic/Faroe
    {  630, 2614,  193696,  8, }, // Atlantic/Reykjavik
    {  630, 6784,     453, 10, }, // Atlantic/St_Helena
    {  630, 1447,  193704,  6, }, // Australia/Sydney
    {  630, 6390,  193710,  9, }, // Europe/Amsterdam
    {  630, 6285,   25465,  7, }, // Europe/Belgrade
    {  630, 6178,   21176,  7, }, // Europe/Brussels
    {  630, 3830,  122803,  8, }, // Europe/Bucharest
    {  630, 3708,  119714,  8, }, // Europe/Budapest
    {  630, 6238,  170483,  8, }, // Europe/Copenhagen
    {  630, 1097,  193719, 14, }, // Europe/Istanbul
    {  630, 3881,  193733, 10, }, // Europe/Kaliningrad
    {  630, 2419,     463,  4, }, // Europe/Kyiv
    {  630, 6455,  193743,  6, }, // Europe/Ljubljana
    {  630, 6224,    6545,  4, }, // Europe/Prague
    {  630, 6273,    6554,  3, }, // Europe/Rome
    {  630, 8846,  193749,  5, }, // Europe/Tallinn
    {  630, 6135,    6599,  5, }, // Europe/Tirane
    {  630, 6516,    6614,  7, }, // Europe/Vatican
    {  630, 6164,   56035,  4, }, // Europe/Vienna
    {  630, 8878,  123642,  5, }, // Europe/Vilnius
    {  630, 4629,    4462,  9, }, // Europe/Volgograd
    {  630, 3055,  193754,  6, }, // Europe/Warsaw
    {  630, 6489,  193760,  5, }, // Europe/Zurich
    {  630, 7174,     467,  7, }, // Indian/Reunion
    {  630, 3022,     474,  5, }, // Pacific/Chuuk
    {  630, 2961,     479,  7, }, // Pacific/Pohnpei
    {  631,   14,       0,  6, }, // Africa/Asmara Warlpiri/Latin/Australia
    {  631, 4337,       6,  8, }, // Africa/Sao_Tome
    {  631, 4154,      14,  8, }, // America/Asuncion
    {  631,  237,      22,  8, }, // America/Atikokan
    {  631, 7586,      30, 17, }, // America/Bahia_Banderas
    {  631, 3770,      47,  6, }, // America/Cancun
    {  631, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  631, 5640,      66,  7, }, // America/Curacao
    {  631,  481,      73, 13, }, // America/Indiana/Knox
    {  631, 7924,      86, 16, }, // America/Indiana/Marengo
    {  631, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  631, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  631, 8001,     139, 14, }, // America/Indiana/Vevay
    {  631, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  631, 8049,     171, 16, }, // America/Indiana/Winamac
    {  631, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  631, 5849,     207, 22, }, // America/Lower_Princes
    {  631, 8183,     229,  6, }, // America/Merida
    {  631, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  631, 8292,     255, 20, }, // America/North_Dakota/Center
    {  631, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  631,  393,     298,  4, }, // America/Nuuk
    {  631, 6862,     302, 16, }, // America/Scoresbysund
    {  631, 8471,     318, 14, }, // America/St_Barthelemy
    {  631, 2045,     332, 10, }, // America/St_Johns
    {  631, 5780,     342,  9, }, // America/St_Kitts
    {  631, 5797,     351,  9, }, // America/St_Lucia
    {  631,  855,     360, 10, }, // America/St_Thomas
    {  631, 5830,     370, 11, }, // America/St_Vincent
    {  631, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  631, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  631, 1152,     413,  9, }, // Asia/Kathmandu
    {  631,  992,     422,  7, }, // Asia/Kolkata
    {  631, 1178,     429,  5, }, // Asia/Macau
    {  631, 1202,     442,  6, }, // Asia/Yangon
    {  631, 1379,     448,  5, }, // Atlantic/Faroe
    {  631, 6784,     453, 10, }, // Atlantic/St_Helena
    {  631, 2419,     463,  4, }, // Europe/Kyiv
    {  631, 7174,     467,  7, }, // Indian/Reunion
    {  631, 3022,     474,  5, }, // Pacific/Chuuk
    {  631, 2961,     479,  7, }, // Pacific/Pohnpei
    {  632, 7383,   42592,  5, }, // Africa/Algiers Welsh/Latin/United Kingdom
    {  632,   14,       0,  6, }, // Africa/Asmara
    {  632, 4337,       6,  8, }, // Africa/Sao_Tome
    {  632, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  632, 4154,      14,  8, }, // America/Asuncion
    {  632,  237,      22,  8, }, // America/Atikokan
    {  632, 7586,  193765, 12, }, // America/Bahia_Banderas
    {  632, 7609,   34376,  5, }, // America/Belem
    {  632, 4263,   18898,  6, }, // America/Bogota
    {  632, 7676,  193777, 13, }, // America/Cambridge_Bay
    {  632, 3770,      47,  6, }, // America/Cancun
    {  632, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  632, 3693,   25007,  6, }, // America/Cuiaba
    {  632, 5640,      66,  7, }, // America/Curacao
    {  632, 7832,   25013,  8, }, // America/Eirunepe
    {  632,  481,      73, 13, }, // America/Indiana/Knox
    {  632, 7924,      86, 16, }, // America/Indiana/Marengo
    {  632, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  632, 7975,  193790, 19, }, // America/Indiana/Tell_City
    {  632, 8001,     139, 14, }, // America/Indiana/Vevay
    {  632, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  632, 8049,     171, 16, }, // America/Indiana/Winamac
    {  632, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  632, 5849,     207, 22, }, // America/Lower_Princes
    {  632, 8183,    3899,  6, }, // America/Merida
    {  632, 2811,  193809, 13, }, // America/Mexico_City
    {  632, 3294,  193822, 12, }, // America/New_York
    {  632, 8264,  193834, 22, }, // America/North_Dakota/Beulah
    {  632, 8292,  193856, 22, }, // America/North_Dakota/Center
    {  632, 8320,  193878, 25, }, // America/North_Dakota/New_Salem
    {  632,  393,     298,  4, }, // America/Nuuk
    {  632, 6862,     302, 16, }, // America/Scoresbysund
    {  632, 8471,     318, 14, }, // America/St_Barthelemy
    {  632, 2045,     332, 10, }, // America/St_Johns
    {  632, 5780,     342,  9, }, // America/St_Kitts
    {  632, 5797,     351,  9, }, // America/St_Lucia
    {  632,  855,     360, 10, }, // America/St_Thomas
    {  632, 5830,     370, 11, }, // America/St_Vincent
    {  632, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  632, 8614,   62340,  4, }, // Asia/Gaza
    {  632, 1226,  193903, 17, }, // Asia/Ho_Chi_Minh
    {  632, 1257,  193920,  9, }, // Asia/Jerusalem
    {  632, 1152,     413,  9, }, // Asia/Kathmandu
    {  632,  992,     422,  7, }, // Asia/Kolkata
    {  632, 1178,    4222,  5, }, // Asia/Macau
    {  632, 8671,     434,  8, }, // Asia/Qostanay
    {  632, 3817,   76718,  6, }, // Asia/Tbilisi
    {  632, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  632, 1202,     442,  6, }, // Asia/Yangon
    {  632, 6591,  193929, 19, }, // Atlantic/Canary
    {  632, 1379,  193948,  5, }, // Atlantic/Faroe
    {  632, 2614,   34656,  9, }, // Atlantic/Reykjavik
    {  632, 7235,  193953, 10, }, // Atlantic/South_Georgia
    {  632, 6784,     453, 10, }, // Atlantic/St_Helena
    {  632, 6178,  193963,  6, }, // Europe/Brussels
    {  632, 3830,  193969,  8, }, // Europe/Bucharest
    {  632, 2260,  193977,  5, }, // Europe/Dublin
    {  632, 8767,  193982, 11, }, // Europe/Guernsey
    {  632, 8783,  193993, 10, }, // Europe/Isle_of_Man
    {  632, 1097,  194003, 13, }, // Europe/Istanbul
    {  632, 2419,    6486,  4, }, // Europe/Kyiv
    {  632, 2393,  194016,  8, }, // Europe/London
    {  632, 6314,  194024, 10, }, // Europe/Luxembourg
    {  632, 6224,    6545,  4, }, // Europe/Prague
    {  632, 6273,  194034,  7, }, // Europe/Rome
    {  632, 6516,  194041,  9, }, // Europe/Vatican
    {  632, 6164,  194050,  6, }, // Europe/Vienna
    {  632, 5979,  194056, 14, }, // Indian/Christmas
    {  632, 7174,     467,  7, }, // Indian/Reunion
    {  632, 3022,     474,  5, }, // Pacific/Chuuk
    {  632, 2201,  194070, 11, }, // Pacific/Easter
    {  632, 2929,    4512,  8, }, // Pacific/Honolulu
    {  632, 2897,    4520,  9, }, // Pacific/Kanton
    {  632, 2961,     479,  7, }, // Pacific/Pohnpei
    {  633,   14,       0,  6, }, // Africa/Asmara Western Balochi/Arabic/Pakistan
    {  633, 4337,       6,  8, }, // Africa/Sao_Tome
    {  633, 4154,      14,  8, }, // America/Asuncion
    {  633,  237,      22,  8, }, // America/Atikokan
    {  633, 7586,      30, 17, }, // America/Bahia_Banderas
    {  633, 3770,      47,  6, }, // America/Cancun
    {  633, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  633, 5640,      66,  7, }, // America/Curacao
    {  633,  481,      73, 13, }, // America/Indiana/Knox
    {  633, 7924,      86, 16, }, // America/Indiana/Marengo
    {  633, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  633, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  633, 8001,     139, 14, }, // America/Indiana/Vevay
    {  633, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  633, 8049,     171, 16, }, // America/Indiana/Winamac
    {  633, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  633, 5849,     207, 22, }, // America/Lower_Princes
    {  633, 8183,     229,  6, }, // America/Merida
    {  633, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  633, 8292,     255, 20, }, // America/North_Dakota/Center
    {  633, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  633,  393,     298,  4, }, // America/Nuuk
    {  633, 6862,     302, 16, }, // America/Scoresbysund
    {  633, 8471,     318, 14, }, // America/St_Barthelemy
    {  633, 2045,     332, 10, }, // America/St_Johns
    {  633, 5780,     342,  9, }, // America/St_Kitts
    {  633, 5797,     351,  9, }, // America/St_Lucia
    {  633,  855,     360, 10, }, // America/St_Thomas
    {  633, 5830,     370, 11, }, // America/St_Vincent
    {  633, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  633, 5487,   10423,  3, }, // Asia/Aden
    {  633, 5297,  194081,  6, }, // Asia/Almaty
    {  633, 3870,    9854,  4, }, // Asia/Amman
    {  633, 5440,  194087,  5, }, // Asia/Aqtau
    {  633, 5270,  194092,  6, }, // Asia/Aqtobe
    {  633,  964,  194098,  8, }, // Asia/Ashgabat
    {  633, 3509,   10466,  5, }, // Asia/Baghdad
    {  633, 5451,  145924,  5, }, // Asia/Bahrain
    {  633, 3555,  144949,  4, }, // Asia/Baku
    {  633, 3680,  194106,  6, }, // Asia/Bishkek
    {  633, 4413,  194112,  7, }, // Asia/Colombo
    {  633, 4442,   10528,  4, }, // Asia/Damascus
    {  633, 1060,  194119,  4, }, // Asia/Dhaka
    {  633, 3498,  145976,  4, }, // Asia/Dubai
    {  633, 6092,  148582,  6, }, // Asia/Dushanbe
    {  633, 8614,  145994,  3, }, // Asia/Gaza
    {  633, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  633, 1257,  194123,  7, }, // Asia/Jerusalem
    {  633, 3462,  146048,  4, }, // Asia/Kabul
    {  633, 4141,  148657,  5, }, // Asia/Karachi
    {  633, 1152,  194130,  6, }, // Asia/Kathmandu
    {  633,  992,  148677,  5, }, // Asia/Kolkata
    {  633, 5464,  194136,  4, }, // Asia/Kuwait
    {  633, 1178,     429,  5, }, // Asia/Macau
    {  633, 6883,   10734,  4, }, // Asia/Muscat
    {  633, 7108,  148768,  5, }, // Asia/Oral
    {  633, 5476,   10802,  3, }, // Asia/Qatar
    {  633, 4171,  194140,  9, }, // Asia/Qyzylorda
    {  633, 3486,  103788,  4, }, // Asia/Riyadh
    {  633, 7208,   10834,  6, }, // Asia/Samarkand
    {  633, 3116,  103809,  4, }, // Asia/Seoul
    {  633, 4683,  194149,  7, }, // Asia/Tashkent
    {  633, 3817,  194156,  7, }, // Asia/Tbilisi
    {  633, 2638,  146255,  5, }, // Asia/Tehran
    {  633, 1284,  194163,  5, }, // Asia/Thimphu
    {  633, 2687,  148876,  5, }, // Asia/Tokyo
    {  633, 4474,  148881,  5, }, // Asia/Tomsk
    {  633, 1346,  148886, 11, }, // Asia/Ulaanbaatar
    {  633, 1126,  194168,  6, }, // Asia/Urumqi
    {  633, 1202,     442,  6, }, // Asia/Yangon
    {  633, 1379,     448,  5, }, // Atlantic/Faroe
    {  633, 6784,     453, 10, }, // Atlantic/St_Helena
    {  633, 1097,  149187,  8, }, // Europe/Istanbul
    {  633, 8924,  146609,  5, }, // Europe/Kirov
    {  633, 2419,     463,  4, }, // Europe/Kyiv
    {  633, 4233,  149268,  6, }, // Europe/Samara
    {  633, 7013,  194174,  6, }, // Indian/Maldives
    {  633, 7174,     467,  7, }, // Indian/Reunion
    {  633, 3022,     474,  5, }, // Pacific/Chuuk
    {  633, 2961,     479,  7, }, // Pacific/Pohnpei
    {  638, 4961,    3793, 11, }, // Africa/Addis_Ababa Western Frisian/Latin/Netherlands
    {  638,   14,       0,  6, }, // Africa/Asmara
    {  638, 2242,   57254,  5, }, // Africa/Cairo
    {  638, 4426,   57259,  8, }, // Africa/Khartoum
    {  638, 6832,   18802,  4, }, // Africa/Lome
    {  638, 4337,   57267,  8, }, // Africa/Sao_Tome
    {  638,  270,   18870,  7, }, // America/Argentina/Cordoba
    {  638, 7452,   42641, 12, }, // America/Argentina/Rio_Gallegos
    {  638, 7534,   18877,  7, }, // America/Argentina/Tucuman
    {  638, 4154,      14,  8, }, // America/Asuncion
    {  638,  237,      22,  8, }, // America/Atikokan
    {  638, 7586,      30, 17, }, // America/Bahia_Banderas
    {  638, 7609,   34376,  5, }, // America/Belem
    {  638, 4263,   18898,  6, }, // America/Bogota
    {  638, 3770,      47,  6, }, // America/Cancun
    {  638, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  638, 3693,   25007,  6, }, // America/Cuiaba
    {  638, 5640,      66,  7, }, // America/Curacao
    {  638, 5343,    5090,  8, }, // America/El_Salvador
    {  638,  481,      73, 13, }, // America/Indiana/Knox
    {  638, 7924,      86, 16, }, // America/Indiana/Marengo
    {  638, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  638, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  638, 8001,     139, 14, }, // America/Indiana/Vevay
    {  638, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  638, 8049,     171, 16, }, // America/Indiana/Winamac
    {  638, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  638, 5849,   57275, 24, }, // America/Lower_Princes
    {  638, 8116,   25021,  6, }, // America/Maceio
    {  638, 2779,   25036,  8, }, // America/Mazatlan
    {  638, 8183,     229,  6, }, // America/Merida
    {  638, 2811,  194180, 11, }, // America/Mexico_City
    {  638, 8264,  194191, 20, }, // America/North_Dakota/Beulah
    {  638, 8292,  194211, 20, }, // America/North_Dakota/Center
    {  638, 8320,  194231, 23, }, // America/North_Dakota/New_Salem
    {  638,  393,     298,  4, }, // America/Nuuk
    {  638, 8367,  194254, 11, }, // America/Porto_Velho
    {  638, 1862,   25145,  9, }, // America/Sao_Paulo
    {  638, 6862,     302, 16, }, // America/Scoresbysund
    {  638, 8471,   25154, 16, }, // America/St_Barthelemy
    {  638, 2045,     332, 10, }, // America/St_Johns
    {  638, 5780,     342,  9, }, // America/St_Kitts
    {  638, 5797,     351,  9, }, // America/St_Lucia
    {  638,  855,     360, 10, }, // America/St_Thomas
    {  638, 5830,     370, 11, }, // America/St_Vincent
    {  638, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  638, 5297,   57310,  8, }, // Asia/Almaty
    {  638, 5270,   55725,  6, }, // Asia/Aqtobe
    {  638,  964,    4054,  9, }, // Asia/Ashgabat
    {  638, 3509,    4063,  6, }, // Asia/Baghdad
    {  638, 5451,    4069,  7, }, // Asia/Bahrain
    {  638, 3555,    4076,  5, }, // Asia/Baku
    {  638, 3988,    4081,  7, }, // Asia/Beirut
    {  638, 3680,    4088,  7, }, // Asia/Bishkek
    {  638, 6092,   57125,  8, }, // Asia/Dushanbe
    {  638, 1226,   57333, 15, }, // Asia/Ho_Chi_Minh
    {  638, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  638, 4086,    4140,  8, }, // Asia/Irkutsk
    {  638, 1257,   31972,  9, }, // Asia/Jerusalem
    {  638, 4218,    4162, 10, }, // Asia/Kamchatka
    {  638, 1152,     413,  9, }, // Asia/Kathmandu
    {  638,  992,   34533,  8, }, // Asia/Kolkata
    {  638, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  638, 5464,    4215,  7, }, // Asia/Kuwait
    {  638, 1178,     429,  5, }, // Asia/Macau
    {  638, 7145,   57348,  7, }, // Asia/Manila
    {  638, 6924,   92426, 10, }, // Asia/Phnom_Penh
    {  638, 3486,   19265,  5, }, // Asia/Riyadh
    {  638, 4310,   56617,  8, }, // Asia/Sakhalin
    {  638, 1020,   57355,  8, }, // Asia/Shanghai
    {  638, 4683,   57154,  8, }, // Asia/Tashkent
    {  638, 2638,    6175,  7, }, // Asia/Tehran
    {  638, 2687,    4257,  5, }, // Asia/Tokyo
    {  638, 4730,    4273,  8, }, // Asia/Yakutsk
    {  638, 1202,     442,  6, }, // Asia/Yangon
    {  638, 3785,   57363, 15, }, // Asia/Yekaterinburg
    {  638, 3649,    6231,  7, }, // Asia/Yerevan
    {  638, 3565,   57378,  6, }, // Atlantic/Azores
    {  638, 6591,  194265, 18, }, // Atlantic/Canary
    {  638, 3629,   57403, 10, }, // Atlantic/Cape_Verde
    {  638, 1379,   57413,  7, }, // Atlantic/Faroe
    {  638, 7235,  194283, 11, }, // Atlantic/South_Georgia
    {  638, 6784,   57432, 11, }, // Atlantic/St_Helena
    {  638, 6577,    4339,  6, }, // Europe/Athens
    {  638, 6285,   57443,  8, }, // Europe/Belgrade
    {  638, 4659,    4345,  6, }, // Europe/Berlin
    {  638, 6178,    4351,  7, }, // Europe/Brussels
    {  638, 3830,    4358,  9, }, // Europe/Bucharest
    {  638, 3708,    4367,  9, }, // Europe/Budapest
    {  638, 6238,    4376, 10, }, // Europe/Copenhagen
    {  638, 6561,   56830,  8, }, // Europe/Helsinki
    {  638, 1097,   57458,  9, }, // Europe/Istanbul
    {  638, 2419,     463,  4, }, // Europe/Kyiv
    {  638, 3078,    4400,  8, }, // Europe/Lisbon
    {  638, 2393,    4408,  6, }, // Europe/London
    {  638, 6314,    4414,  9, }, // Europe/Luxembourg
    {  638, 6359,    6525,  6, }, // Europe/Monaco
    {  638, 3443,    4423,  6, }, // Europe/Moscow
    {  638, 4186,    4429,  5, }, // Europe/Paris
    {  638, 6224,  194294,  6, }, // Europe/Prague
    {  638, 6135,   19608,  6, }, // Europe/Tirane
    {  638, 6516,  194300, 12, }, // Europe/Vatican
    {  638, 6164,   57485,  5, }, // Europe/Vienna
    {  638, 4629,    4462,  9, }, // Europe/Volgograd
    {  638, 3055,   56048,  8, }, // Europe/Warsaw
    {  638, 6489,    4478,  6, }, // Europe/Zurich
    {  638, 6910,   57490, 14, }, // Indian/Chagos
    {  638, 5979,  194312, 10, }, // Indian/Christmas
    {  638, 5996,  194322, 13, }, // Indian/Cocos
    {  638, 7223,   25565,  4, }, // Indian/Mahe
    {  638, 7013,   57532,  8, }, // Indian/Maldives
    {  638, 7174,     467,  7, }, // Indian/Reunion
    {  638, 3022,     474,  5, }, // Pacific/Chuuk
    {  638, 2201,  194335, 11, }, // Pacific/Easter
    {  638, 6641,   19672,  9, }, // Pacific/Galapagos
    {  638, 6659,  194346, 15, }, // Pacific/Gambier
    {  638, 2897,  194361, 15, }, // Pacific/Kanton
    {  638, 3953,  194376, 17, }, // Pacific/Marquesas
    {  638, 7061,    4529,  6, }, // Pacific/Noumea
    {  638, 2961,     479,  7, }, // Pacific/Pohnpei
    {  639,   14,       0,  6, }, // Africa/Asmara Wolaytta/Ethiopic/Ethiopia
    {  639, 4337,       6,  8, }, // Africa/Sao_Tome
    {  639, 4154,      14,  8, }, // America/Asuncion
    {  639,  237,      22,  8, }, // America/Atikokan
    {  639, 7586,      30, 17, }, // America/Bahia_Banderas
    {  639, 3770,      47,  6, }, // America/Cancun
    {  639, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  639, 5640,      66,  7, }, // America/Curacao
    {  639,  481,      73, 13, }, // America/Indiana/Knox
    {  639, 7924,      86, 16, }, // America/Indiana/Marengo
    {  639, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  639, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  639, 8001,     139, 14, }, // America/Indiana/Vevay
    {  639, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  639, 8049,     171, 16, }, // America/Indiana/Winamac
    {  639, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  639, 5849,     207, 22, }, // America/Lower_Princes
    {  639, 8183,     229,  6, }, // America/Merida
    {  639, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  639, 8292,     255, 20, }, // America/North_Dakota/Center
    {  639, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  639,  393,     298,  4, }, // America/Nuuk
    {  639, 6862,     302, 16, }, // America/Scoresbysund
    {  639, 8471,     318, 14, }, // America/St_Barthelemy
    {  639, 2045,     332, 10, }, // America/St_Johns
    {  639, 5780,     342,  9, }, // America/St_Kitts
    {  639, 5797,     351,  9, }, // America/St_Lucia
    {  639,  855,     360, 10, }, // America/St_Thomas
    {  639, 5830,     370, 11, }, // America/St_Vincent
    {  639, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  639, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  639, 1152,     413,  9, }, // Asia/Kathmandu
    {  639,  992,     422,  7, }, // Asia/Kolkata
    {  639, 1178,     429,  5, }, // Asia/Macau
    {  639, 1202,     442,  6, }, // Asia/Yangon
    {  639, 1379,     448,  5, }, // Atlantic/Faroe
    {  639, 6784,     453, 10, }, // Atlantic/St_Helena
    {  639, 2419,     463,  4, }, // Europe/Kyiv
    {  639, 7174,     467,  7, }, // Indian/Reunion
    {  639, 3022,     474,  5, }, // Pacific/Chuuk
    {  639, 2961,     479,  7, }, // Pacific/Pohnpei
    {  640,   14,       0,  6, }, // Africa/Asmara Wolof/Latin/Senegal
    {  640, 4337,       6,  8, }, // Africa/Sao_Tome
    {  640, 4154,      14,  8, }, // America/Asuncion
    {  640,  237,      22,  8, }, // America/Atikokan
    {  640, 7586,      30, 17, }, // America/Bahia_Banderas
    {  640, 3770,      47,  6, }, // America/Cancun
    {  640, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  640, 5640,      66,  7, }, // America/Curacao
    {  640,  481,      73, 13, }, // America/Indiana/Knox
    {  640, 7924,      86, 16, }, // America/Indiana/Marengo
    {  640, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  640, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  640, 8001,     139, 14, }, // America/Indiana/Vevay
    {  640, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  640, 8049,     171, 16, }, // America/Indiana/Winamac
    {  640, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  640, 5849,     207, 22, }, // America/Lower_Princes
    {  640, 8183,     229,  6, }, // America/Merida
    {  640, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  640, 8292,     255, 20, }, // America/North_Dakota/Center
    {  640, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  640,  393,     298,  4, }, // America/Nuuk
    {  640, 6862,     302, 16, }, // America/Scoresbysund
    {  640, 8471,     318, 14, }, // America/St_Barthelemy
    {  640, 2045,     332, 10, }, // America/St_Johns
    {  640, 5780,     342,  9, }, // America/St_Kitts
    {  640, 5797,     351,  9, }, // America/St_Lucia
    {  640,  855,     360, 10, }, // America/St_Thomas
    {  640, 5830,     370, 11, }, // America/St_Vincent
    {  640, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  640, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  640, 1152,     413,  9, }, // Asia/Kathmandu
    {  640,  992,     422,  7, }, // Asia/Kolkata
    {  640, 1178,     429,  5, }, // Asia/Macau
    {  640, 1202,     442,  6, }, // Asia/Yangon
    {  640, 1379,     448,  5, }, // Atlantic/Faroe
    {  640, 6784,     453, 10, }, // Atlantic/St_Helena
    {  640, 2419,     463,  4, }, // Europe/Kyiv
    {  640, 7174,     467,  7, }, // Indian/Reunion
    {  640, 3022,     474,  5, }, // Pacific/Chuuk
    {  640, 2961,     479,  7, }, // Pacific/Pohnpei
    {  641,   14,       0,  6, }, // Africa/Asmara Xhosa/Latin/South Africa
    {  641, 4337,       6,  8, }, // Africa/Sao_Tome
    {  641, 4154,      14,  8, }, // America/Asuncion
    {  641,  237,      22,  8, }, // America/Atikokan
    {  641, 7586,    3834, 14, }, // America/Bahia_Banderas
    {  641, 3770,    3861,  6, }, // America/Cancun
    {  641, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  641, 5640,      66,  7, }, // America/Curacao
    {  641,  481,      73, 13, }, // America/Indiana/Knox
    {  641, 7924,      86, 16, }, // America/Indiana/Marengo
    {  641, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  641, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  641, 8001,     139, 14, }, // America/Indiana/Vevay
    {  641, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  641, 8049,     171, 16, }, // America/Indiana/Winamac
    {  641, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  641, 5849,     207, 22, }, // America/Lower_Princes
    {  641, 8183,    3899,  6, }, // America/Merida
    {  641, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  641, 8292,     255, 20, }, // America/North_Dakota/Center
    {  641, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  641,  393,     298,  4, }, // America/Nuuk
    {  641, 6862,     302, 16, }, // America/Scoresbysund
    {  641, 8471,     318, 14, }, // America/St_Barthelemy
    {  641, 2045,     332, 10, }, // America/St_Johns
    {  641, 5780,     342,  9, }, // America/St_Kitts
    {  641, 5797,     351,  9, }, // America/St_Lucia
    {  641,  855,     360, 10, }, // America/St_Thomas
    {  641, 5830,     370, 11, }, // America/St_Vincent
    {  641, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  641, 1226,     397, 16, }, // Asia/Ho_Chi_Minh
    {  641, 1152,     413,  9, }, // Asia/Kathmandu
    {  641,  992,     422,  7, }, // Asia/Kolkata
    {  641, 1178,     429,  5, }, // Asia/Macau
    {  641, 8671,     434,  8, }, // Asia/Qostanay
    {  641, 1202,     442,  6, }, // Asia/Yangon
    {  641, 1379,     448,  5, }, // Atlantic/Faroe
    {  641, 6784,     453, 10, }, // Atlantic/St_Helena
    {  641, 2419,     463,  4, }, // Europe/Kyiv
    {  641, 7174,     467,  7, }, // Indian/Reunion
    {  641, 3022,     474,  5, }, // Pacific/Chuuk
    {  641, 2961,     479,  7, }, // Pacific/Pohnpei
    {  642,   14,       0,  6, }, // Africa/Asmara Yangben/Latin/Cameroon
    {  642, 4337,       6,  8, }, // Africa/Sao_Tome
    {  642, 4154,      14,  8, }, // America/Asuncion
    {  642,  237,      22,  8, }, // America/Atikokan
    {  642, 7586,      30, 17, }, // America/Bahia_Banderas
    {  642, 3770,      47,  6, }, // America/Cancun
    {  642, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  642, 5640,      66,  7, }, // America/Curacao
    {  642,  481,      73, 13, }, // America/Indiana/Knox
    {  642, 7924,      86, 16, }, // America/Indiana/Marengo
    {  642, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  642, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  642, 8001,     139, 14, }, // America/Indiana/Vevay
    {  642, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  642, 8049,     171, 16, }, // America/Indiana/Winamac
    {  642, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  642, 5849,     207, 22, }, // America/Lower_Princes
    {  642, 8183,     229,  6, }, // America/Merida
    {  642, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  642, 8292,     255, 20, }, // America/North_Dakota/Center
    {  642, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  642,  393,     298,  4, }, // America/Nuuk
    {  642, 6862,     302, 16, }, // America/Scoresbysund
    {  642, 8471,     318, 14, }, // America/St_Barthelemy
    {  642, 2045,     332, 10, }, // America/St_Johns
    {  642, 5780,     342,  9, }, // America/St_Kitts
    {  642, 5797,     351,  9, }, // America/St_Lucia
    {  642,  855,     360, 10, }, // America/St_Thomas
    {  642, 5830,     370, 11, }, // America/St_Vincent
    {  642, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  642, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  642, 1152,     413,  9, }, // Asia/Kathmandu
    {  642,  992,     422,  7, }, // Asia/Kolkata
    {  642, 1178,     429,  5, }, // Asia/Macau
    {  642, 1202,     442,  6, }, // Asia/Yangon
    {  642, 1379,     448,  5, }, // Atlantic/Faroe
    {  642, 6784,     453, 10, }, // Atlantic/St_Helena
    {  642, 2419,     463,  4, }, // Europe/Kyiv
    {  642, 7174,     467,  7, }, // Indian/Reunion
    {  642, 3022,     474,  5, }, // Pacific/Chuuk
    {  642, 2961,     479,  7, }, // Pacific/Pohnpei
    {  643, 6751,  194393, 10, }, // Africa/Abidjan Yiddish/Hebrew/Ukraine
    {  643,   14,  194403,  9, }, // Africa/Asmara
    {  643, 4337,       6,  8, }, // Africa/Sao_Tome
    {  643, 6503,  194412,  5, }, // Africa/Tunis
    {  643, 4054,  194417,  8, }, // Africa/Windhoek
    {  643, 4154,      14,  8, }, // America/Asuncion
    {  643,  237,      22,  8, }, // America/Atikokan
    {  643, 7586,      30, 17, }, // America/Bahia_Banderas
    {  643, 3770,      47,  6, }, // America/Cancun
    {  643, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  643, 5640,      66,  7, }, // America/Curacao
    {  643,  481,      73, 13, }, // America/Indiana/Knox
    {  643, 7924,      86, 16, }, // America/Indiana/Marengo
    {  643, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  643, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  643, 8001,     139, 14, }, // America/Indiana/Vevay
    {  643, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  643, 8049,     171, 16, }, // America/Indiana/Winamac
    {  643, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  643, 5849,     207, 22, }, // America/Lower_Princes
    {  643, 8183,     229,  6, }, // America/Merida
    {  643, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  643, 8292,     255, 20, }, // America/North_Dakota/Center
    {  643, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  643,  393,     298,  4, }, // America/Nuuk
    {  643, 6862,     302, 16, }, // America/Scoresbysund
    {  643, 8471,     318, 14, }, // America/St_Barthelemy
    {  643, 2045,     332, 10, }, // America/St_Johns
    {  643, 5780,     342,  9, }, // America/St_Kitts
    {  643, 5797,     351,  9, }, // America/St_Lucia
    {  643,  855,     360, 10, }, // America/St_Thomas
    {  643, 5830,     370, 11, }, // America/St_Vincent
    {  643, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  643, 1226,  194425, 16, }, // Asia/Ho_Chi_Minh
    {  643, 1152,     413,  9, }, // Asia/Kathmandu
    {  643,  992,     422,  7, }, // Asia/Kolkata
    {  643, 1178,     429,  5, }, // Asia/Macau
    {  643, 1202,     442,  6, }, // Asia/Yangon
    {  643, 1379,     448,  5, }, // Atlantic/Faroe
    {  643, 6784,     453, 10, }, // Atlantic/St_Helena
    {  643, 2419,     463,  4, }, // Europe/Kyiv
    {  643, 7174,     467,  7, }, // Indian/Reunion
    {  643, 3022,     474,  5, }, // Pacific/Chuuk
    {  643, 2961,     479,  7, }, // Pacific/Pohnpei
    {  644,   14,       0,  6, }, // Africa/Asmara Yoruba/Latin/Nigeria
    {  644, 4337,       6,  8, }, // Africa/Sao_Tome
    {  644,  132,  194441,  9, }, // America/Adak
    {  644, 3183,  194450, 14, }, // America/Anchorage
    {  644, 5524,  194464, 11, }, // America/Anguilla
    {  644, 5541,  194475, 11, }, // America/Antigua
    {  644, 5557,  194486,  9, }, // America/Aruba
    {  644, 4154,      14,  8, }, // America/Asuncion
    {  644,  237,  194495, 12, }, // America/Atikokan
    {  644, 7586,  194507, 19, }, // America/Bahia_Banderas
    {  644, 5571,  194526, 15, }, // America/Barbados
    {  644, 5309,  194541, 12, }, // America/Belize
    {  644, 7623,  194553, 21, }, // America/Blanc-Sablon
    {  644, 7662,  194574, 10, }, // America/Boise
    {  644, 7676,  194584, 19, }, // America/Cambridge_Bay
    {  644, 3770,  194603, 10, }, // America/Cancun
    {  644, 5398,  194613, 10, }, // America/Cayman
    {  644, 3251,  194623, 11, }, // America/Chicago
    {  644, 7719,  194634, 12, }, // America/Chihuahua
    {  644, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  644, 5324,  194646, 14, }, // America/Costa_Rica
    {  644, 7759,  194660, 16, }, // America/Creston
    {  644, 5640,  194676, 11, }, // America/Curacao
    {  644, 7775,  194687, 15, }, // America/Danmarkshavn
    {  644, 7796,  194702, 10, }, // America/Dawson
    {  644, 7811,  194712, 16, }, // America/Dawson_Creek
    {  644,  805,  194728, 10, }, // America/Denver
    {  644, 3351,  194738, 11, }, // America/Detroit
    {  644, 5656,  194749, 12, }, // America/Dominica
    {  644,  893,  194761, 12, }, // America/Edmonton
    {  644, 5343,  194773, 14, }, // America/El_Salvador
    {  644, 7849,  194787, 15, }, // America/Fort_Nelson
    {  644, 7887,  194802, 13, }, // America/Glace_Bay
    {  644, 6844,  194815, 13, }, // America/Goose_Bay
    {  644, 4514,  194828, 14, }, // America/Grand_Turk
    {  644, 5687,  194842, 11, }, // America/Grenada
    {  644, 5703,  194853, 14, }, // America/Guadeloupe
    {  644, 3662,  194867, 13, }, // America/Guatemala
    {  644, 1923,  194880, 12, }, // America/Halifax
    {  644, 2221,  194892, 10, }, // America/Havana
    {  644, 7905,  194902, 13, }, // America/Hermosillo
    {  644,  348,  194915, 16, }, // America/Indiana/Indianapolis
    {  644,  481,  194931,  9, }, // America/Indiana/Knox
    {  644, 7924,  194940, 12, }, // America/Indiana/Marengo
    {  644, 7948,  194952, 14, }, // America/Indiana/Petersburg
    {  644, 7975,  194966, 13, }, // America/Indiana/Tell_City
    {  644, 8001,  194979,  9, }, // America/Indiana/Vevay
    {  644, 8023,  194988, 24, }, // America/Indiana/Vincennes
    {  644, 8049,  195012, 22, }, // America/Indiana/Winamac
    {  644, 8073,  195034, 10, }, // America/Inuvik
    {  644,  660,  195044, 11, }, // America/Iqaluit
    {  644, 2665,  195055, 11, }, // America/Jamaica
    {  644, 5282,  195066,  8, }, // America/Juneau
    {  644,  521,  195074, 12, }, // America/Kentucky/Louisville
    {  644, 8088,  195086, 14, }, // America/Kentucky/Monticello
    {  644, 5621,  195100, 17, }, // America/Kralendijk
    {  644, 3390,  195117, 15, }, // America/Los_Angeles
    {  644, 5849,  195132, 26, }, // America/Lower_Princes
    {  644, 8131,  195158, 11, }, // America/Managua
    {  644, 5814,  195169, 15, }, // America/Marigot
    {  644, 5722,  195184, 12, }, // America/Martinique
    {  644, 8147,  195196, 14, }, // America/Matamoros
    {  644, 2779,  195210, 13, }, // America/Mazatlan
    {  644, 8165,  195223, 12, }, // America/Menominee
    {  644, 8183,  195235, 10, }, // America/Merida
    {  644, 8198,  195245, 16, }, // America/Metlakatla
    {  644, 2811,  195261, 11, }, // America/Mexico_City
    {  644, 4293,  195272, 13, }, // America/Miquelon
    {  644, 8217,  195285, 11, }, // America/Moncton
    {  644, 8233,  195296, 11, }, // America/Monterrey
    {  644, 5741,  195307, 13, }, // America/Montserrat
    {  644, 5383,  195320,  9, }, // America/Nassau
    {  644, 3294,  195329, 12, }, // America/New_York
    {  644, 8251,  195341,  8, }, // America/Nome
    {  644, 8264,  195349, 25, }, // America/North_Dakota/Beulah
    {  644, 8292,  195374, 22, }, // America/North_Dakota/Center
    {  644, 8320,  195396, 29, }, // America/North_Dakota/New_Salem
    {  644,  393,  195425,  8, }, // America/Nuuk
    {  644, 8351,  195433, 11, }, // America/Ojinaga
    {  644, 5413,  195444, 10, }, // America/Panama
    {  644, 3224,  195454, 10, }, // America/Phoenix
    {  644, 3847,  195464, 18, }, // America/Port-au-Prince
    {  644, 5871,  195482, 19, }, // America/Port_of_Spain
    {  644, 5760,  195501, 15, }, // America/Puerto_Rico
    {  644, 8387,  195516, 16, }, // America/Rankin_Inlet
    {  644, 1979,  195532, 10, }, // America/Regina
    {  644, 8423,  195542, 12, }, // America/Resolute
    {  644, 6044,  195554, 16, }, // America/Santo_Domingo
    {  644, 6862,  195570, 15, }, // America/Scoresbysund
    {  644, 8457,  195585,  8, }, // America/Sitka
    {  644, 8471,  195593, 12, }, // America/St_Barthelemy
    {  644, 2045,  195605, 14, }, // America/St_Johns
    {  644, 5780,  195619, 13, }, // America/St_Kitts
    {  644, 5797,  195632, 12, }, // America/St_Lucia
    {  644,  855,  195644, 14, }, // America/St_Thomas
    {  644, 5830,  195658, 12, }, // America/St_Vincent
    {  644, 8493,  195670, 19, }, // America/Swift_Current
    {  644, 5363,  195689, 14, }, // America/Tegucigalpa
    {  644, 5673,  195703,  8, }, // America/Thule
    {  644,  313,  195711, 11, }, // America/Tijuana
    {  644,  608,  195722, 11, }, // America/Toronto
    {  644, 5605,  195733, 10, }, // America/Tortola
    {  644, 2077,  195743, 11, }, // America/Vancouver
    {  644, 2128,  195754, 13, }, // America/Whitehorse
    {  644,  734,  195767, 12, }, // America/Winnipeg
    {  644, 8515,  195779, 10, }, // America/Yakutat
    {  644, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  644, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  644, 1152,     413,  9, }, // Asia/Kathmandu
    {  644,  992,     422,  7, }, // Asia/Kolkata
    {  644, 1178,     429,  5, }, // Asia/Macau
    {  644, 1202,     442,  6, }, // Asia/Yangon
    {  644, 5588,  195789, 11, }, // Atlantic/Bermuda
    {  644, 1379,     448,  5, }, // Atlantic/Faroe
    {  644, 6784,     453, 10, }, // Atlantic/St_Helena
    {  644, 2419,     463,  4, }, // Europe/Kyiv
    {  644, 7174,     467,  7, }, // Indian/Reunion
    {  644, 3022,     474,  5, }, // Pacific/Chuuk
    {  644, 2961,     479,  7, }, // Pacific/Pohnpei
    {  645, 3183,  195800, 14, }, // America/Anchorage Yoruba/Latin/Benin
    {  645,  237,  195814, 12, }, // America/Atikokan
    {  645, 7586,  195826, 19, }, // America/Bahia_Banderas
    {  645, 5571,  195845, 15, }, // America/Barbados
    {  645, 5309,  195860, 12, }, // America/Belize
    {  645, 7623,  195872, 21, }, // America/Blanc-Sablon
    {  645, 7662,  195893, 10, }, // America/Boise
    {  645, 7719,  195903, 13, }, // America/Chihuahua
    {  645, 7759,  195916, 16, }, // America/Creston
    {  645, 5640,  195932, 12, }, // America/Curacao
    {  645,  481,  195944,  9, }, // America/Indiana/Knox
    {  645, 7924,  195953, 12, }, // America/Indiana/Marengo
    {  645, 8088,  195965, 14, }, // America/Kentucky/Monticello
    {  645, 5621,  195979, 17, }, // America/Kralendijk
    {  645, 5814,  195996, 15, }, // America/Marigot
    {  645, 2811,  196011, 11, }, // America/Mexico_City
    {  645, 4293,  196022, 13, }, // America/Miquelon
    {  645, 2045,  196035, 14, }, // America/St_Johns
    {  645,  855,  196049, 14, }, // America/St_Thomas
    {  645, 8493,  196063, 19, }, // America/Swift_Current
    {  645, 5588,  196082, 11, }, // Atlantic/Bermuda
    {  646,   14,       0,  6, }, // Africa/Asmara Zarma/Latin/Niger
    {  646, 4337,       6,  8, }, // Africa/Sao_Tome
    {  646, 4154,      14,  8, }, // America/Asuncion
    {  646,  237,      22,  8, }, // America/Atikokan
    {  646, 7586,      30, 17, }, // America/Bahia_Banderas
    {  646, 3770,      47,  6, }, // America/Cancun
    {  646, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  646, 5640,      66,  7, }, // America/Curacao
    {  646,  481,      73, 13, }, // America/Indiana/Knox
    {  646, 7924,      86, 16, }, // America/Indiana/Marengo
    {  646, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  646, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  646, 8001,     139, 14, }, // America/Indiana/Vevay
    {  646, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  646, 8049,     171, 16, }, // America/Indiana/Winamac
    {  646, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  646, 5849,     207, 22, }, // America/Lower_Princes
    {  646, 8183,     229,  6, }, // America/Merida
    {  646, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  646, 8292,     255, 20, }, // America/North_Dakota/Center
    {  646, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  646,  393,     298,  4, }, // America/Nuuk
    {  646, 6862,     302, 16, }, // America/Scoresbysund
    {  646, 8471,     318, 14, }, // America/St_Barthelemy
    {  646, 2045,     332, 10, }, // America/St_Johns
    {  646, 5780,     342,  9, }, // America/St_Kitts
    {  646, 5797,     351,  9, }, // America/St_Lucia
    {  646,  855,     360, 10, }, // America/St_Thomas
    {  646, 5830,     370, 11, }, // America/St_Vincent
    {  646, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  646, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  646, 1152,     413,  9, }, // Asia/Kathmandu
    {  646,  992,     422,  7, }, // Asia/Kolkata
    {  646, 1178,     429,  5, }, // Asia/Macau
    {  646, 1202,     442,  6, }, // Asia/Yangon
    {  646, 1379,     448,  5, }, // Atlantic/Faroe
    {  646, 6784,     453, 10, }, // Atlantic/St_Helena
    {  646, 2419,     463,  4, }, // Europe/Kyiv
    {  646, 7174,     467,  7, }, // Indian/Reunion
    {  646, 3022,     474,  5, }, // Pacific/Chuuk
    {  646, 2961,     479,  7, }, // Pacific/Pohnpei
    {  647,   14,       0,  6, }, // Africa/Asmara Zhuang/Latin/China
    {  647, 4337,       6,  8, }, // Africa/Sao_Tome
    {  647, 4154,      14,  8, }, // America/Asuncion
    {  647,  237,      22,  8, }, // America/Atikokan
    {  647, 7586,      30, 17, }, // America/Bahia_Banderas
    {  647, 3770,      47,  6, }, // America/Cancun
    {  647, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  647, 5640,      66,  7, }, // America/Curacao
    {  647,  481,      73, 13, }, // America/Indiana/Knox
    {  647, 7924,      86, 16, }, // America/Indiana/Marengo
    {  647, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  647, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  647, 8001,     139, 14, }, // America/Indiana/Vevay
    {  647, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  647, 8049,     171, 16, }, // America/Indiana/Winamac
    {  647, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  647, 5849,     207, 22, }, // America/Lower_Princes
    {  647, 8183,     229,  6, }, // America/Merida
    {  647, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  647, 8292,     255, 20, }, // America/North_Dakota/Center
    {  647, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  647,  393,     298,  4, }, // America/Nuuk
    {  647, 6862,     302, 16, }, // America/Scoresbysund
    {  647, 8471,     318, 14, }, // America/St_Barthelemy
    {  647, 2045,     332, 10, }, // America/St_Johns
    {  647, 5780,     342,  9, }, // America/St_Kitts
    {  647, 5797,     351,  9, }, // America/St_Lucia
    {  647,  855,     360, 10, }, // America/St_Thomas
    {  647, 5830,     370, 11, }, // America/St_Vincent
    {  647, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  647, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  647, 1152,     413,  9, }, // Asia/Kathmandu
    {  647,  992,     422,  7, }, // Asia/Kolkata
    {  647, 1178,     429,  5, }, // Asia/Macau
    {  647, 1202,     442,  6, }, // Asia/Yangon
    {  647, 1379,     448,  5, }, // Atlantic/Faroe
    {  647, 6784,     453, 10, }, // Atlantic/St_Helena
    {  647, 2419,     463,  4, }, // Europe/Kyiv
    {  647, 7174,     467,  7, }, // Indian/Reunion
    {  647, 3022,     474,  5, }, // Pacific/Chuuk
    {  647, 2961,     479,  7, }, // Pacific/Pohnpei
    {  648, 6751,  196093,  9, }, // Africa/Abidjan Zulu/Latin/South Africa
    {  648, 6723,  196102,  7, }, // Africa/Accra
    {  648, 4961,  196109, 13, }, // Africa/Addis_Ababa
    {  648, 7383,  196122,  9, }, // Africa/Algiers
    {  648,   14,  196131,  8, }, // Africa/Asmara
    {  648,   44,  196139,  8, }, // Africa/Bamako
    {  648, 5159,  196147,  8, }, // Africa/Bangui
    {  648, 6709,  196155,  8, }, // Africa/Banjul
    {  648, 7398,  196163,  8, }, // Africa/Bissau
    {  648, 4859,  196171, 10, }, // Africa/Blantyre
    {  648, 5189,  196181, 13, }, // Africa/Brazzaville
    {  648, 4824,  196194, 11, }, // Africa/Bujumbura
    {  648, 2242,  196205,  7, }, // Africa/Cairo
    {  648, 4019,  196212, 12, }, // Africa/Casablanca
    {  648, 7412,  196224,  7, }, // Africa/Ceuta
    {  648, 6736,  196231,  9, }, // Africa/Conakry
    {  648, 6803,  196240,  7, }, // Africa/Dakar
    {  648, 5032,  196247, 15, }, // Africa/Dar_es_Salaam
    {  648, 4945,  196262, 10, }, // Africa/Djibouti
    {  648, 5145,  196272,  8, }, // Africa/Douala
    {  648, 5068,  196280, 10, }, // Africa/El_Aaiun
    {  648, 6816,  196290, 10, }, // Africa/Freetown
    {  648, 4808,  196300, 10, }, // Africa/Gaborone
    {  648, 4917,  196310,  8, }, // Africa/Harare
    {  648, 4381,  196318, 14, }, // Africa/Johannesburg
    {  648, 4401,  196332,  5, }, // Africa/Juba
    {  648, 5053,  196337,  9, }, // Africa/Kampala
    {  648, 4426,  196346, 10, }, // Africa/Khartoum
    {  648, 4875,  196356,  8, }, // Africa/Kigali
    {  648, 5208,  196364, 10, }, // Africa/Kinshasa
    {  648, 4646,  196374,  7, }, // Africa/Lagos
    {  648, 5238,  196381, 12, }, // Africa/Libreville
    {  648, 6832,  196393,  6, }, // Africa/Lome
    {  648, 5113,  196399,  8, }, // Africa/Luanda
    {  648, 4841,  196407, 12, }, // Africa/Lubumbashi
    {  648, 4903,  196419,  7, }, // Africa/Lusaka
    {  648, 5224,  196426,  7, }, // Africa/Malabo
    {  648, 4889,  196433,  7, }, // Africa/Maputo
    {  648, 5099,  196440,  7, }, // Africa/Maseru
    {  648, 5084,  196447,  8, }, // Africa/Mbabane
    {  648, 5015,  196455, 11, }, // Africa/Mogadishu
    {  648, 6997,  196466, 10, }, // Africa/Monrovia
    {  648, 3755,  196476,  9, }, // Africa/Nairobi
    {  648, 5173,  196485, 10, }, // Africa/Ndjamena
    {  648, 5256,  196495,  8, }, // Africa/Niamey
    {  648, 6766,  196503, 12, }, // Africa/Nouakchott
    {  648, 6690,  196515, 13, }, // Africa/Ouagadougou
    {  648, 5127,  196528, 12, }, // Africa/Porto-Novo
    {  648, 4337,  196540, 10, }, // Africa/Sao_Tome
    {  648, 2732,  196550,  9, }, // Africa/Tripoli
    {  648, 6503,  196559,  7, }, // Africa/Tunis
    {  648, 4054,  196566, 10, }, // Africa/Windhoek
    {  648,  132,  196576,  6, }, // America/Adak
    {  648, 3183,  196582, 11, }, // America/Anchorage
    {  648, 5524,  196593, 10, }, // America/Anguilla
    {  648, 5541,  196603,  9, }, // America/Antigua
    {  648, 4456,  196612, 11, }, // America/Araguaina
    {  648,  166,  196623, 14, }, // America/Argentina/Buenos_Aires
    {  648,   91,  196637, 11, }, // America/Argentina/Catamarca
    {  648,  270,  196648,  9, }, // America/Argentina/Cordoba
    {  648,  441,  196657,  7, }, // America/Argentina/Jujuy
    {  648, 7425,  196664, 10, }, // America/Argentina/La_Rioja
    {  648,  565,  196674,  9, }, // America/Argentina/Mendoza
    {  648, 7452,  196683, 14, }, // America/Argentina/Rio_Gallegos
    {  648, 7483,  196697,  7, }, // America/Argentina/Salta
    {  648, 7507,  196704, 10, }, // America/Argentina/San_Juan
    {  648, 5497,  196714, 10, }, // America/Argentina/San_Luis
    {  648, 7534,  196724,  9, }, // America/Argentina/Tucuman
    {  648, 7560,  196733,  9, }, // America/Argentina/Ushuaia
    {  648, 5557,  196742,  7, }, // America/Aruba
    {  648, 4154,  196749, 10, }, // America/Asuncion
    {  648,  237,  196759, 10, }, // America/Atikokan
    {  648, 3581,  196769,  7, }, // America/Bahia
    {  648, 7586,  196776, 16, }, // America/Bahia_Banderas
    {  648, 5571,  196792, 10, }, // America/Barbados
    {  648, 7609,  196802,  7, }, // America/Belem
    {  648, 5309,  196809,  8, }, // America/Belize
    {  648, 7623,  196817, 14, }, // America/Blanc-Sablon
    {  648, 7644,  196831, 11, }, // America/Boa_Vista
    {  648, 4263,  196842,  8, }, // America/Bogota
    {  648, 7662,  196850,  7, }, // America/Boise
    {  648, 7676,  196857, 15, }, // America/Cambridge_Bay
    {  648, 7698,  196872, 14, }, // America/Campo_Grande
    {  648, 3770,  196886,  8, }, // America/Cancun
    {  648, 4596,  196894,  9, }, // America/Caracas
    {  648, 4247,  196903,  9, }, // America/Cayenne
    {  648, 5398,  196912,  8, }, // America/Cayman
    {  648, 3251,  196920,  9, }, // America/Chicago
    {  648, 7719,  196929, 11, }, // America/Chihuahua
    {  648, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  648, 5324,  196940, 12, }, // America/Costa_Rica
    {  648, 7759,  196952,  9, }, // America/Creston
    {  648, 3693,  196961,  8, }, // America/Cuiaba
    {  648, 5640,  196969,  9, }, // America/Curacao
    {  648, 7775,  196978, 14, }, // America/Danmarkshavn
    {  648, 7796,  196992,  8, }, // America/Dawson
    {  648, 7811,  197000, 14, }, // America/Dawson_Creek
    {  648,  805,  197014,  8, }, // America/Denver
    {  648, 3351,  197022,  9, }, // America/Detroit
    {  648, 5656,  197031, 10, }, // America/Dominica
    {  648,  893,  197041, 10, }, // America/Edmonton
    {  648, 7832,  197051, 10, }, // America/Eirunepe
    {  648, 5343,  197061, 13, }, // America/El_Salvador
    {  648, 7849,  197074, 13, }, // America/Fort_Nelson
    {  648, 7869,  197087, 11, }, // America/Fortaleza
    {  648, 7887,  197098, 11, }, // America/Glace_Bay
    {  648, 6844,  197109, 11, }, // America/Goose_Bay
    {  648, 4514,  197120, 12, }, // America/Grand_Turk
    {  648, 5687,  197132,  9, }, // America/Grenada
    {  648, 5703,  197141, 12, }, // America/Guadeloupe
    {  648, 3662,  197153, 11, }, // America/Guatemala
    {  648, 4743,  197164, 11, }, // America/Guayaquil
    {  648, 6895,  197175,  8, }, // America/Guyana
    {  648, 1923,  197183,  9, }, // America/Halifax
    {  648, 2221,  197192,  8, }, // America/Havana
    {  648, 7905,  197200, 12, }, // America/Hermosillo
    {  648,  348,  197212, 14, }, // America/Indiana/Indianapolis
    {  648,  481,  197226, 15, }, // America/Indiana/Knox
    {  648, 7924,  197241, 18, }, // America/Indiana/Marengo
    {  648, 7948,  197259, 21, }, // America/Indiana/Petersburg
    {  648, 7975,  197280, 20, }, // America/Indiana/Tell_City
    {  648, 8001,  197300, 16, }, // America/Indiana/Vevay
    {  648, 8023,  197316, 20, }, // America/Indiana/Vincennes
    {  648, 8049,  197336, 18, }, // America/Indiana/Winamac
    {  648, 8073,  197354,  8, }, // America/Inuvik
    {  648,  660,  197362,  9, }, // America/Iqaluit
    {  648, 2665,  197371,  9, }, // America/Jamaica
    {  648, 5282,  197380,  8, }, // America/Juneau
    {  648,  521,  197388, 12, }, // America/Kentucky/Louisville
    {  648, 8088,  197400, 22, }, // America/Kentucky/Monticello
    {  648, 5621,  197422, 12, }, // America/Kralendijk
    {  648, 4278,  197434,  8, }, // America/La_Paz
    {  648, 7132,  197442,  6, }, // America/Lima
    {  648, 3390,  197448, 13, }, // America/Los_Angeles
    {  648, 5849,  197461, 24, }, // America/Lower_Princes
    {  648, 8116,  197485,  8, }, // America/Maceio
    {  648, 8131,  197493,  9, }, // America/Managua
    {  648, 1892,  197502,  8, }, // America/Manaus
    {  648, 5814,  197510,  9, }, // America/Marigot
    {  648, 5722,  197519, 12, }, // America/Martinique
    {  648, 8147,  197531, 11, }, // America/Matamoros
    {  648, 2779,  197542, 10, }, // America/Mazatlan
    {  648, 8165,  197552, 11, }, // America/Menominee
    {  648, 8183,  197563,  8, }, // America/Merida
    {  648, 8198,  197571, 12, }, // America/Metlakatla
    {  648, 2811,  197583, 13, }, // America/Mexico_City
    {  648, 4293,  197596, 10, }, // America/Miquelon
    {  648, 8217,  197606,  9, }, // America/Moncton
    {  648, 8233,  197615, 11, }, // America/Monterrey
    {  648, 4000,  197626, 12, }, // America/Montevideo
    {  648, 5741,  197638, 12, }, // America/Montserrat
    {  648, 5383,  197650,  8, }, // America/Nassau
    {  648, 3294,  197658, 10, }, // America/New_York
    {  648, 8251,  197668,  6, }, // America/Nome
    {  648, 1834,  197674,  9, }, // America/Noronha
    {  648, 8264,  197683, 22, }, // America/North_Dakota/Beulah
    {  648, 8292,  197705, 22, }, // America/North_Dakota/Center
    {  648, 8320,  197727, 25, }, // America/North_Dakota/New_Salem
    {  648,  393,  197752,  6, }, // America/Nuuk
    {  648, 8351,  197758,  9, }, // America/Ojinaga
    {  648, 5413,  197767,  8, }, // America/Panama
    {  648, 6106,  197775, 12, }, // America/Paramaribo
    {  648, 3224,  197787,  9, }, // America/Phoenix
    {  648, 3847,  197796, 16, }, // America/Port-au-Prince
    {  648, 5871,  197812, 15, }, // America/Port_of_Spain
    {  648, 8367,  197827, 13, }, // America/Porto_Velho
    {  648, 5760,  197840, 13, }, // America/Puerto_Rico
    {  648, 3932,  197853, 14, }, // America/Punta_Arenas
    {  648, 8387,  197867, 14, }, // America/Rankin_Inlet
    {  648, 8408,  197881,  8, }, // America/Recife
    {  648, 1979,  197889,  8, }, // America/Regina
    {  648, 8423,  197897, 10, }, // America/Resolute
    {  648,  695,  197907, 12, }, // America/Rio_Branco
    {  648, 8440,  197919, 10, }, // America/Santarem
    {  648, 2165,  197929, 10, }, // America/Santiago
    {  648, 6044,  197939, 15, }, // America/Santo_Domingo
    {  648, 1862,  197954, 11, }, // America/Sao_Paulo
    {  648, 6862,  197965, 18, }, // America/Scoresbysund
    {  648, 8457,  197983,  7, }, // America/Sitka
    {  648, 8471,  197990, 16, }, // America/St_Barthelemy
    {  648, 2045,  198006, 12, }, // America/St_Johns
    {  648, 5780,  198018, 11, }, // America/St_Kitts
    {  648, 5797,  198029, 11, }, // America/St_Lucia
    {  648,  855,  198040, 12, }, // America/St_Thomas
    {  648, 5830,  198052, 13, }, // America/St_Vincent
    {  648, 8493,  198065, 15, }, // America/Swift_Current
    {  648, 5363,  198080, 13, }, // America/Tegucigalpa
    {  648, 5673,  198093,  7, }, // America/Thule
    {  648,  313,  198100,  9, }, // America/Tijuana
    {  648,  608,  198109,  9, }, // America/Toronto
    {  648, 5605,  198118,  9, }, // America/Tortola
    {  648, 2077,  198127, 11, }, // America/Vancouver
    {  648, 2128,  198138, 12, }, // America/Whitehorse
    {  648,  734,  198150, 10, }, // America/Winnipeg
    {  648, 8515,  198160,  9, }, // America/Yakutat
    {  648, 5918,  198169,  7, }, // Antarctica/Casey
    {  648, 6027,  198176,  7, }, // Antarctica/Davis
    {  648, 6066,  198183, 18, }, // Antarctica/DumontDUrville
    {  648, 8531,  198201, 11, }, // Antarctica/Macquarie
    {  648, 7029,  198212,  8, }, // Antarctica/Mawson
    {  648, 7076,  198220,  9, }, // Antarctica/McMurdo
    {  648, 8552,  198229,  8, }, // Antarctica/Palmer
    {  648, 7189,  198237,  9, }, // Antarctica/Rothera
    {  648, 7258,  198246,  7, }, // Antarctica/Syowa
    {  648, 8570,  198253,  7, }, // Antarctica/Troll
    {  648, 7337,  198260,  8, }, // Antarctica/Vostok
    {  648, 1413,  198268, 14, }, // Arctic/Longyearbyen
    {  648, 5487,  198282,  6, }, // Asia/Aden
    {  648, 5297,  198288,  8, }, // Asia/Almaty
    {  648, 3870,  198296,  7, }, // Asia/Amman
    {  648, 5428,  198303,  8, }, // Asia/Anadyr
    {  648, 5440,  198311,  7, }, // Asia/Aqtau
    {  648, 5270,  198318,  8, }, // Asia/Aqtobe
    {  648,  964,  198326, 10, }, // Asia/Ashgabat
    {  648, 3509,  198336,  9, }, // Asia/Baghdad
    {  648, 5451,  198345,  9, }, // Asia/Bahrain
    {  648, 3555,  198354,  6, }, // Asia/Baku
    {  648, 4368,  198360,  9, }, // Asia/Bangkok
    {  648, 3473,  198369,  9, }, // Asia/Barnaul
    {  648, 3988,  198378,  8, }, // Asia/Beirut
    {  648, 3680,  198386,  9, }, // Asia/Bishkek
    {  648, 5906,  198395,  8, }, // Asia/Brunei
    {  648, 4503,  198403,  7, }, // Asia/Chita
    {  648, 5963,  198410, 12, }, // Asia/Choibalsan
    {  648, 4413,  198422,  9, }, // Asia/Colombo
    {  648, 4442,  198431, 10, }, // Asia/Damascus
    {  648, 1060,  198441,  7, }, // Asia/Dhaka
    {  648, 6125,  198448,  6, }, // Asia/Dili
    {  648, 3498,  198454,  7, }, // Asia/Dubai
    {  648, 6092,  198461, 10, }, // Asia/Dushanbe
    {  648, 8614,  198471,  5, }, // Asia/Gaza
    {  648, 4697,  198476,  8, }, // Asia/Hebron
    {  648, 1226,  198484, 18, }, // Asia/Ho_Chi_Minh
    {  648, 2576,  198502, 11, }, // Asia/Hong_Kong
    {  648, 4673,  198513,  6, }, // Asia/Hovd
    {  648, 4086,  198519,  9, }, // Asia/Irkutsk
    {  648, 6969,  198528,  9, }, // Asia/Jakarta
    {  648, 6955,  198537, 10, }, // Asia/Jayapura
    {  648, 1257,  198547, 11, }, // Asia/Jerusalem
    {  648, 3462,  198558,  7, }, // Asia/Kabul
    {  648, 4218,  198565, 11, }, // Asia/Kamchatka
    {  648, 4141,  198576,  9, }, // Asia/Karachi
    {  648, 1152,  198585, 11, }, // Asia/Kathmandu
    {  648, 8624,  198596, 10, }, // Asia/Khandyga
    {  648,  992,  198606,  9, }, // Asia/Kolkata
    {  648, 4099,  198615, 13, }, // Asia/Krasnoyarsk
    {  648, 4761,  198628, 14, }, // Asia/Kuala_Lumpur
    {  648, 5893,  198642,  9, }, // Asia/Kuching
    {  648, 5464,  198651,  8, }, // Asia/Kuwait
    {  648, 1178,  198659,  7, }, // Asia/Macau
    {  648, 3919,  198666,  9, }, // Asia/Magadan
    {  648, 1316,  198675, 10, }, // Asia/Makassar
    {  648, 7145,  198685,  8, }, // Asia/Manila
    {  648, 6883,  198693,  8, }, // Asia/Muscat
    {  648, 2446,  198701,  9, }, // Asia/Nicosia
    {  648, 8638,  198710, 14, }, // Asia/Novokuznetsk
    {  648, 4037,  198724, 13, }, // Asia/Novosibirsk
    {  648, 4131,  198737,  6, }, // Asia/Omsk
    {  648, 7108,  198743,  6, }, // Asia/Oral
    {  648, 6924,  198749, 12, }, // Asia/Phnom_Penh
    {  648, 8656,  198761, 11, }, // Asia/Pontianak
    {  648, 4116,  198772, 11, }, // Asia/Pyongyang
    {  648, 5476,  198783,  7, }, // Asia/Qatar
    {  648, 8671,  198790, 10, }, // Asia/Qostanay
    {  648, 4171,  198800, 11, }, // Asia/Qyzylorda
    {  648, 3486,  198811,  8, }, // Asia/Riyadh
    {  648, 4310,  198819, 10, }, // Asia/Sakhalin
    {  648, 7208,  198829, 11, }, // Asia/Samarkand
    {  648, 3116,  198840,  7, }, // Asia/Seoul
    {  648, 1020,  198847, 10, }, // Asia/Shanghai
    {  648, 3137,  198857, 11, }, // Asia/Singapore
    {  648, 4199,  198868, 15, }, // Asia/Srednekolymsk
    {  648, 3100,  198883,  8, }, // Asia/Taipei
    {  648, 4683,  198891, 10, }, // Asia/Tashkent
    {  648, 3817,  198901,  9, }, // Asia/Tbilisi
    {  648, 2638,  198910,  8, }, // Asia/Tehran
    {  648, 1284,  198918,  9, }, // Asia/Thimphu
    {  648, 2687,  198927,  7, }, // Asia/Tokyo
    {  648, 4474,  198934,  7, }, // Asia/Tomsk
    {  648, 1346,  198941, 13, }, // Asia/Ulaanbaatar
    {  648, 1126,  198954,  8, }, // Asia/Urumqi
    {  648, 8685,  198962, 10, }, // Asia/Ust-Nera
    {  648, 6940,  198972, 11, }, // Asia/Vientiane
    {  648, 4612,  198983, 13, }, // Asia/Vladivostok
    {  648, 4730,  198996,  9, }, // Asia/Yakutsk
    {  648, 1202,  199005,  9, }, // Asia/Yangon
    {  648, 3785,  199014, 15, }, // Asia/Yekaterinburg
    {  648, 3649,  199029,  9, }, // Asia/Yerevan
    {  648, 3565,  199038,  8, }, // Atlantic/Azores
    {  648, 5588,  199046,  9, }, // Atlantic/Bermuda
    {  648, 6591,  199055,  8, }, // Atlantic/Canary
    {  648, 3629,  199063, 12, }, // Atlantic/Cape_Verde
    {  648, 1379,  199075,  7, }, // Atlantic/Faroe
    {  648, 8699,  199082,  9, }, // Atlantic/Madeira
    {  648, 2614,  199091, 11, }, // Atlantic/Reykjavik
    {  648, 7235,  199102, 15, }, // Atlantic/South_Georgia
    {  648, 6784,  199117, 12, }, // Atlantic/St_Helena
    {  648, 6607,  199129,  9, }, // Atlantic/Stanley
    {  648, 1654,  199138, 10, }, // Australia/Adelaide
    {  648, 1619,  199148, 10, }, // Australia/Brisbane
    {  648, 1783,  199158, 13, }, // Australia/Broken_Hill
    {  648, 1567,  199171,  8, }, // Australia/Darwin
    {  648, 3539,  199179,  7, }, // Australia/Eucla
    {  648, 1500,  199186,  8, }, // Australia/Hobart
    {  648, 8716,  199194, 10, }, // Australia/Lindeman
    {  648, 1531,  199204, 11, }, // Australia/Lord_Howe
    {  648, 1711,  199215, 11, }, // Australia/Melbourne
    {  648, 1746,  199226,  7, }, // Australia/Perth
    {  648, 1447,  199233,  8, }, // Australia/Sydney
    {  648, 6390,  199241, 11, }, // Europe/Amsterdam
    {  648, 6149,  199252,  9, }, // Europe/Andorra
    {  648, 3522,  199261, 11, }, // Europe/Astrakhan
    {  648, 6577,  199272,  8, }, // Europe/Athens
    {  648, 6285,  199280, 10, }, // Europe/Belgrade
    {  648, 4659,  199290,  8, }, // Europe/Berlin
    {  648, 6437,  199298, 12, }, // Europe/Bratislava
    {  648, 6178,  199310, 10, }, // Europe/Brussels
    {  648, 3830,  199320, 11, }, // Europe/Bucharest
    {  648, 3708,  199331, 10, }, // Europe/Budapest
    {  648, 8751,  199341, 10, }, // Europe/Busingen
    {  648, 2475,  199351, 10, }, // Europe/Chisinau
    {  648, 6238,  199361, 12, }, // Europe/Copenhagen
    {  648, 2260,  199373,  8, }, // Europe/Dublin
    {  648, 6256,  199381, 11, }, // Europe/Gibraltar
    {  648, 8767,  199392, 10, }, // Europe/Guernsey
    {  648, 6561,  199402, 10, }, // Europe/Helsinki
    {  648, 8783,  199412, 13, }, // Europe/Isle_of_Man
    {  648, 1097,  199425, 10, }, // Europe/Istanbul
    {  648, 8802,  199435,  8, }, // Europe/Jersey
    {  648, 3881,  199443, 13, }, // Europe/Kaliningrad
    {  648, 8924,  199456,  7, }, // Europe/Kirov
    {  648, 2419,  199463,  6, }, // Europe/Kyiv
    {  648, 3078,  199469,  8, }, // Europe/Lisbon
    {  648, 6455,  199477, 11, }, // Europe/Ljubljana
    {  648, 2393,  199488,  8, }, // Europe/London
    {  648, 6314,  199496, 12, }, // Europe/Luxembourg
    {  648, 4794,  199508,  8, }, // Europe/Madrid
    {  648, 6346,  199516,  7, }, // Europe/Malta
    {  648, 6531,  199523, 11, }, // Europe/Mariehamn
    {  648, 3595,  199534,  7, }, // Europe/Minsk
    {  648, 6359,  199541,  8, }, // Europe/Monaco
    {  648, 3443,  199549,  8, }, // Europe/Moscow
    {  648, 6407,  199557,  6, }, // Europe/Oslo
    {  648, 4186,  199563,  7, }, // Europe/Paris
    {  648, 6373,  199570, 11, }, // Europe/Podgorica
    {  648, 6224,  199581,  8, }, // Europe/Prague
    {  648, 8816,  199589,  6, }, // Europe/Riga
    {  648, 6273,  199595,  6, }, // Europe/Rome
    {  648, 4233,  199601,  8, }, // Europe/Samara
    {  648, 6419,  199609, 12, }, // Europe/San_Marino
    {  648, 6194,  199621, 10, }, // Europe/Sarajevo
    {  648, 4353,  199631,  9, }, // Europe/Saratov
    {  648, 8828,  199640, 12, }, // Europe/Simferopol
    {  648, 6332,  199652,  8, }, // Europe/Skopje
    {  648, 6548,  199660,  7, }, // Europe/Sofia
    {  648, 6472,  199667, 11, }, // Europe/Stockholm
    {  648, 8846,  199678,  9, }, // Europe/Tallinn
    {  648, 6135,  199687,  8, }, // Europe/Tirane
    {  648, 8861,  199695, 11, }, // Europe/Ulyanovsk
    {  648, 6301,  199706,  7, }, // Europe/Vaduz
    {  648, 6516,  199713,  9, }, // Europe/Vatican
    {  648, 6164,  199722,  8, }, // Europe/Vienna
    {  648, 8878,  199730,  9, }, // Europe/Vilnius
    {  648, 4629,  199739, 11, }, // Europe/Volgograd
    {  648, 3055,  199750,  8, }, // Europe/Warsaw
    {  648, 6210,  199758,  8, }, // Europe/Zagreb
    {  648, 6489,  199766,  8, }, // Europe/Zurich
    {  648, 4980,  199774, 14, }, // Indian/Antananarivo
    {  648, 6910,  199788,  8, }, // Indian/Chagos
    {  648, 5979,  199796, 10, }, // Indian/Christmas
    {  648, 5996,  199806,  7, }, // Indian/Cocos
    {  648, 4931,  199813,  8, }, // Indian/Comoro
    {  648, 6624,  199821, 11, }, // Indian/Kerguelen
    {  648, 7223,  199832,  5, }, // Indian/Mahe
    {  648, 7013,  199837, 10, }, // Indian/Maldives
    {  648, 3971,  199847, 11, }, // Indian/Mauritius
    {  648, 5000,  199858,  9, }, // Indian/Mayotte
    {  648, 7174,  199867,  9, }, // Indian/Reunion
    {  648, 4324,  199876,  6, }, // Pacific/Apia
    {  648,  932,  199882, 10, }, // Pacific/Auckland
    {  648, 3608,  199892, 14, }, // Pacific/Bougainville
    {  648, 2863,  199906,  9, }, // Pacific/Chatham
    {  648, 3022,  199915,  7, }, // Pacific/Chuuk
    {  648, 2201,  199922,  8, }, // Pacific/Easter
    {  648, 7323,  199930,  7, }, // Pacific/Efate
    {  648, 7290,  199937,  9, }, // Pacific/Fakaofo
    {  648, 3804,  199946,  6, }, // Pacific/Fiji
    {  648, 7306,  199952, 10, }, // Pacific/Funafuti
    {  648, 6641,  199962, 11, }, // Pacific/Galapagos
    {  648, 6659,  199973,  9, }, // Pacific/Gambier
    {  648, 3724,  199982, 13, }, // Pacific/Guadalcanal
    {  648, 5935,  199995,  6, }, // Pacific/Guam
    {  648, 2929,  200001, 10, }, // Pacific/Honolulu
    {  648, 2897,  200011, 11, }, // Pacific/Kanton
    {  648, 3900,  200022, 12, }, // Pacific/Kiritimati
    {  648, 6982,  200034,  8, }, // Pacific/Kosrae
    {  648, 2708,  200042, 11, }, // Pacific/Kwajalein
    {  648, 4779,  200053,  8, }, // Pacific/Majuro
    {  648, 3953,  200061, 11, }, // Pacific/Marquesas
    {  648, 8909,  200072,  8, }, // Pacific/Midway
    {  648, 7047,  200080,  7, }, // Pacific/Nauru
    {  648, 7095,  200087,  6, }, // Pacific/Niue
    {  648, 4070,  200093,  9, }, // Pacific/Norfolk
    {  648, 7061,  200102,  8, }, // Pacific/Noumea
    {  648, 2991,  200110, 11, }, // Pacific/Pago_Pago
    {  648, 7118,  200121,  7, }, // Pacific/Palau
    {  648, 7157,  200128, 10, }, // Pacific/Pitcairn
    {  648, 2961,  200138,  9, }, // Pacific/Pohnpei
    {  648, 4709,  200147, 14, }, // Pacific/Port_Moresby
    {  648, 6009,  200161, 11, }, // Pacific/Rarotonga
    {  648, 5948,  200172,  8, }, // Pacific/Saipan
    {  648, 7275,  200180,  8, }, // Pacific/Tahiti
    {  648, 6675,  200188,  8, }, // Pacific/Tarawa
    {  648, 4485,  200196, 11, }, // Pacific/Tongatapu
    {  648, 7355,  200207,  6, }, // Pacific/Wake
    {  648, 7368,  200213,  8, }, // Pacific/Wallis
    {  649, 6751,  200221,  7, }, // Africa/Abidjan Kaingang/Latin/Brazil
    {  649, 6723,    4550,  4, }, // Africa/Accra
    {  649, 4961,  200228, 11, }, // Africa/Addis_Ababa
    {  649, 7383,  200239,  5, }, // Africa/Algiers
    {  649,   14,  200244,  7, }, // Africa/Asmara
    {  649,   44,  200251,  6, }, // Africa/Bamako
    {  649, 5159,  200257,  4, }, // Africa/Bangui
    {  649, 6709,  200261,  6, }, // Africa/Banjul
    {  649, 7398,  200267,  5, }, // Africa/Bissau
    {  649, 4859,  200272,  8, }, // Africa/Blantyre
    {  649, 5189,  200280,  9, }, // Africa/Brazzaville
    {  649, 4824,  200289,  8, }, // Africa/Bujumbura
    {  649, 2242,    4616,  5, }, // Africa/Cairo
    {  649, 4019,  200297,  7, }, // Africa/Casablanca
    {  649, 7412,  200304,  6, }, // Africa/Ceuta
    {  649, 6736,  200310,  7, }, // Africa/Conakry
    {  649, 6803,  200317,  5, }, // Africa/Dakar
    {  649, 5032,  200322, 13, }, // Africa/Dar_es_Salaam
    {  649, 4945,  200335,  7, }, // Africa/Djibouti
    {  649, 5145,  200342,  6, }, // Africa/Douala
    {  649, 5068,  200348,  9, }, // Africa/El_Aaiun
    {  649, 6816,  200357,  7, }, // Africa/Freetown
    {  649, 4808,  200364,  8, }, // Africa/Gaborone
    {  649, 4917,   96902,  6, }, // Africa/Harare
    {  649, 4381,  200372, 13, }, // Africa/Johannesburg
    {  649, 4401,  200385,  4, }, // Africa/Juba
    {  649, 5053,  200389,  7, }, // Africa/Kampala
    {  649, 4426,  200396,  6, }, // Africa/Khartoum
    {  649, 4875,  200402,  6, }, // Africa/Kigali
    {  649, 5208,  200408,  6, }, // Africa/Kinshasa
    {  649, 4646,  200414,  4, }, // Africa/Lagos
    {  649, 5238,  200418,  9, }, // Africa/Libreville
    {  649, 6832,  200427,  4, }, // Africa/Lome
    {  649, 5113,  200431,  6, }, // Africa/Luanda
    {  649, 4841,  200437,  8, }, // Africa/Lubumbashi
    {  649, 4903,  200445,  6, }, // Africa/Lusaka
    {  649, 5224,  200451,  6, }, // Africa/Malabo
    {  649, 4889,  200457,  6, }, // Africa/Maputo
    {  649, 5099,  200463,  6, }, // Africa/Maseru
    {  649, 5084,  200469,  8, }, // Africa/Mbabane
    {  649, 5015,  200477, 10, }, // Africa/Mogadishu
    {  649, 6997,  200487,  9, }, // Africa/Monrovia
    {  649, 3755,  200496,  7, }, // Africa/Nairobi
    {  649, 5173,  200503, 11, }, // Africa/Ndjamena
    {  649, 5256,  200514,  6, }, // Africa/Niamey
    {  649, 6766,  200520,  9, }, // Africa/Nouakchott
    {  649, 6690,  200529, 10, }, // Africa/Ouagadougou
    {  649, 5127,  200539,  9, }, // Africa/Porto-Novo
    {  649, 4337,  200548,  7, }, // Africa/Sao_Tome
    {  649, 2732,  200555,  7, }, // Africa/Tripoli
    {  649, 6503,  200562,  6, }, // Africa/Tunis
    {  649, 4054,  200568,  8, }, // Africa/Windhoek
    {  649,  132,  200576,  5, }, // America/Adak
    {  649, 3183,  200581,  7, }, // America/Anchorage
    {  649, 5524,  200588,  5, }, // America/Anguilla
    {  649, 5541,  200593,  7, }, // America/Antigua
    {  649, 4456,  200600,  9, }, // America/Araguaina
    {  649,  166,  200609, 10, }, // America/Argentina/Buenos_Aires
    {  649,   91,  200619,  9, }, // America/Argentina/Catamarca
    {  649,  270,  200628,  7, }, // America/Argentina/Cordoba
    {  649,  441,  200635,  5, }, // America/Argentina/Jujuy
    {  649, 7425,  200640,  8, }, // America/Argentina/La_Rioja
    {  649,  565,  200648,  6, }, // America/Argentina/Mendoza
    {  649, 7452,  200654, 10, }, // America/Argentina/Rio_Gallegos
    {  649, 7483,  200664,  5, }, // America/Argentina/Salta
    {  649, 7507,  200669,  7, }, // America/Argentina/San_Juan
    {  649, 5497,  200676,  6, }, // America/Argentina/San_Luis
    {  649, 7534,  200682,  6, }, // America/Argentina/Tucuman
    {  649, 7560,  200688,  6, }, // America/Argentina/Ushuaia
    {  649, 5557,  200694,  5, }, // America/Aruba
    {  649, 4154,  200699,  6, }, // America/Asuncion
    {  649,  237,  200705,  8, }, // America/Atikokan
    {  649, 3581,  200713,  5, }, // America/Bahia
    {  649, 7586,  200718, 16, }, // America/Bahia_Banderas
    {  649, 5571,  200734,  7, }, // America/Barbados
    {  649, 7609,  200741,  5, }, // America/Belem
    {  649, 5309,  200746,  6, }, // America/Belize
    {  649, 7623,  200752, 12, }, // America/Blanc-Sablon
    {  649, 7644,  200764,  7, }, // America/Boa_Vista
    {  649, 4263,  200771,  6, }, // America/Bogota
    {  649, 7662,  200777,  5, }, // America/Boise
    {  649, 7676,  200782, 12, }, // America/Cambridge_Bay
    {  649, 7698,  200794,  6, }, // America/Campo_Grande
    {  649, 3770,  200800,  5, }, // America/Cancun
    {  649, 4596,  200805,  8, }, // America/Caracas
    {  649, 4247,  200813,  6, }, // America/Cayenne
    {  649, 5398,  200819,  6, }, // America/Cayman
    {  649, 3251,  200825,  6, }, // America/Chicago
    {  649, 7719,  200831, 10, }, // America/Chihuahua
    {  649, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  649, 5324,  200841, 11, }, // America/Costa_Rica
    {  649, 7759,  200852,  8, }, // America/Creston
    {  649, 3693,  200860,  6, }, // America/Cuiaba
    {  649, 5640,  200866,  8, }, // America/Curacao
    {  649, 7775,  200874, 13, }, // America/Danmarkshavn
    {  649, 7796,  200887,  6, }, // America/Dawson
    {  649, 7811,  200893, 12, }, // America/Dawson_Creek
    {  649,  805,  200905,  6, }, // America/Denver
    {  649, 3351,  200911,  9, }, // America/Detroit
    {  649, 5656,  200920,  8, }, // America/Dominica
    {  649,  893,  200928,  8, }, // America/Edmonton
    {  649, 7832,  200936,  8, }, // America/Eirunepe
    {  649, 5343,  200944, 11, }, // America/El_Salvador
    {  649, 7849,  200955, 11, }, // America/Fort_Nelson
    {  649, 7869,  200966,  9, }, // America/Fortaleza
    {  649, 7887,  200975,  9, }, // America/Glace_Bay
    {  649, 6844,  200984,  8, }, // America/Goose_Bay
    {  649, 4514,  200992,  9, }, // America/Grand_Turk
    {  649, 5687,  201001,  7, }, // America/Grenada
    {  649, 5703,  201008, 10, }, // America/Guadeloupe
    {  649, 3662,  201018, 10, }, // America/Guatemala
    {  649, 4743,  201028,  9, }, // America/Guayaquil
    {  649, 6895,  201037,  6, }, // America/Guyana
    {  649, 1923,  201043,  7, }, // America/Halifax
    {  649, 2221,  201050,  6, }, // America/Havana
    {  649, 7905,  201056,  9, }, // America/Hermosillo
    {  649,  348,  201065, 13, }, // America/Indiana/Indianapolis
    {  649,  481,  201078, 14, }, // America/Indiana/Knox
    {  649, 7924,  201092, 16, }, // America/Indiana/Marengo
    {  649, 7948,  201108, 19, }, // America/Indiana/Petersburg
    {  649, 7975,  201127, 17, }, // America/Indiana/Tell_City
    {  649, 8001,  201144, 14, }, // America/Indiana/Vevay
    {  649, 8023,  201158, 17, }, // America/Indiana/Vincennes
    {  649, 8049,  201175, 17, }, // America/Indiana/Winamac
    {  649, 8073,  201192,  7, }, // America/Inuvik
    {  649,  660,  201199,  8, }, // America/Iqaluit
    {  649, 2665,  201207,  7, }, // America/Jamaica
    {  649, 5282,  201214,  6, }, // America/Juneau
    {  649,  521,  201220,  9, }, // America/Kentucky/Louisville
    {  649, 8088,  201229, 17, }, // America/Kentucky/Monticello
    {  649, 5621,  201246,  9, }, // America/Kralendijk
    {  649, 4278,  201255,  7, }, // America/La_Paz
    {  649, 7132,  201262,  4, }, // America/Lima
    {  649, 3390,  201266, 11, }, // America/Los_Angeles
    {  649, 5849,  201277, 21, }, // America/Lower_Princes
    {  649, 8116,  201298,  6, }, // America/Maceio
    {  649, 8131,  201304,  8, }, // America/Managua
    {  649, 1892,  201312,  6, }, // America/Manaus
    {  649, 5814,  201318,  8, }, // America/Marigot
    {  649, 5722,  201326,  9, }, // America/Martinique
    {  649, 8147,  201335, 12, }, // America/Matamoros
    {  649, 2779,  201347,  8, }, // America/Mazatlan
    {  649, 8165,  201355,  8, }, // America/Menominee
    {  649, 8183,  201363,  6, }, // America/Merida
    {  649, 8198,  201369, 12, }, // America/Metlakatla
    {  649, 2811,  201381, 13, }, // America/Mexico_City
    {  649, 4293,  201394,  7, }, // America/Miquelon
    {  649, 8217,  201401,  6, }, // America/Moncton
    {  649, 8233,  201407,  7, }, // America/Monterrey
    {  649, 4000,  201414,  9, }, // America/Montevideo
    {  649, 5741,  201423,  9, }, // America/Montserrat
    {  649, 5383,  201432,  5, }, // America/Nassau
    {  649, 3294,  201437, 10, }, // America/New_York
    {  649, 8251,  201447,  4, }, // America/Nome
    {  649, 1834,  201451, 17, }, // America/Noronha
    {  649, 8264,  201468, 19, }, // America/North_Dakota/Beulah
    {  649, 8292,  201487, 20, }, // America/North_Dakota/Center
    {  649, 8320,  201507, 23, }, // America/North_Dakota/New_Salem
    {  649,  393,  201530,  3, }, // America/Nuuk
    {  649, 8351,  201533,  7, }, // America/Ojinaga
    {  649, 5413,  201540,  6, }, // America/Panama
    {  649, 6106,  201546, 10, }, // America/Paramaribo
    {  649, 3224,  201556,  7, }, // America/Phoenix
    {  649, 3847,  201563, 13, }, // America/Port-au-Prince
    {  649, 5871,  201576, 16, }, // America/Port_of_Spain
    {  649, 8367,  201592, 10, }, // America/Porto_Velho
    {  649, 5760,  159292, 10, }, // America/Puerto_Rico
    {  649, 3932,  201602, 10, }, // America/Punta_Arenas
    {  649, 8387,  201612, 11, }, // America/Rankin_Inlet
    {  649, 8408,  201623,  6, }, // America/Recife
    {  649, 1979,  201629,  6, }, // America/Regina
    {  649, 8423,  201635,  8, }, // America/Resolute
    {  649,  695,  201643,  9, }, // America/Rio_Branco
    {  649, 8440,  201652,  7, }, // America/Santarem
    {  649, 2165,  201659,  8, }, // America/Santiago
    {  649, 6044,  201667, 12, }, // America/Santo_Domingo
    {  649, 1862,  201679,  9, }, // America/Sao_Paulo
    {  649, 6862,  201688, 15, }, // America/Scoresbysund
    {  649, 8457,  201703,  5, }, // America/Sitka
    {  649, 8471,  201708, 12, }, // America/St_Barthelemy
    {  649, 2045,  201720,  8, }, // America/St_Johns
    {  649, 5780,  201728, 10, }, // America/St_Kitts
    {  649, 5797,  201738,  9, }, // America/St_Lucia
    {  649,  855,  201747,  8, }, // America/St_Thomas
    {  649, 5830,  201755,  9, }, // America/St_Vincent
    {  649, 8493,  201764, 15, }, // America/Swift_Current
    {  649, 5363,  201779, 11, }, // America/Tegucigalpa
    {  649, 5673,  201790,  4, }, // America/Thule
    {  649,  313,  201794,  6, }, // America/Tijuana
    {  649,  608,  201800,  6, }, // America/Toronto
    {  649, 5605,  201806,  7, }, // America/Tortola
    {  649, 2077,  201813,  8, }, // America/Vancouver
    {  649, 2128,  201821, 12, }, // America/Whitehorse
    {  649,  734,  201833,  7, }, // America/Winnipeg
    {  649, 8515,  201840,  9, }, // America/Yakutat
    {  649, 5918,  201849,  5, }, // Antarctica/Casey
    {  649, 6027,  201854,  6, }, // Antarctica/Davis
    {  649, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  649, 8531,  201860,  9, }, // Antarctica/Macquarie
    {  649, 7029,  201869,  6, }, // Antarctica/Mawson
    {  649, 7076,  201875,  9, }, // Antarctica/McMurdo
    {  649, 8552,  201884,  6, }, // Antarctica/Palmer
    {  649, 7189,  118999,  6, }, // Antarctica/Rothera
    {  649, 7258,  121946,  5, }, // Antarctica/Syowa
    {  649, 8570,  201890,  5, }, // Antarctica/Troll
    {  649, 7337,  201895,  8, }, // Antarctica/Vostok
    {  649, 1413,  201903, 14, }, // Arctic/Longyearbyen
    {  649, 5487,  201917,  4, }, // Asia/Aden
    {  649, 5297,  201921,  6, }, // Asia/Almaty
    {  649, 3870,  201927,  3, }, // Asia/Amman
    {  649, 5428,  201930,  7, }, // Asia/Anadyr
    {  649, 5440,  201937,  6, }, // Asia/Aqtau
    {  649, 5270,  201943,  7, }, // Asia/Aqtobe
    {  649,  964,  201950,  9, }, // Asia/Ashgabat
    {  649, 8587,  201959,  6, }, // Asia/Atyrau
    {  649, 3509,  201965,  6, }, // Asia/Baghdad
    {  649, 5451,  201971,  5, }, // Asia/Bahrain
    {  649, 3555,   20679,  4, }, // Asia/Baku
    {  649, 4368,  201976,  7, }, // Asia/Bangkok
    {  649, 3473,  201983,  7, }, // Asia/Barnaul
    {  649, 3988,  201990,  7, }, // Asia/Beirut
    {  649, 3680,  201997,  8, }, // Asia/Bishkek
    {  649, 5906,  202005,  6, }, // Asia/Brunei
    {  649, 4503,  202011,  4, }, // Asia/Chita
    {  649, 5963,  202015,  8, }, // Asia/Choibalsan
    {  649, 4413,  202023,  7, }, // Asia/Colombo
    {  649, 4442,  202030,  8, }, // Asia/Damascus
    {  649, 1060,  200317,  4, }, // Asia/Dhaka
    {  649, 6125,  202038,  4, }, // Asia/Dili
    {  649, 3498,  202042,  5, }, // Asia/Dubai
    {  649, 6092,  202047,  7, }, // Asia/Dushanbe
    {  649, 8599,  202054,  9, }, // Asia/Famagusta
    {  649, 8614,  202063,  4, }, // Asia/Gaza
    {  649, 4697,  202067,  6, }, // Asia/Hebron
    {  649, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  649, 2576,  202073,  7, }, // Asia/Hong_Kong
    {  649, 4673,  202080,  5, }, // Asia/Hovd
    {  649, 4086,  202085, 10, }, // Asia/Irkutsk
    {  649, 6955,  202095,  8, }, // Asia/Jayapura
    {  649, 1257,  202103,  9, }, // Asia/Jerusalem
    {  649, 3462,  202112,  5, }, // Asia/Kabul
    {  649, 4218,  202117,  8, }, // Asia/Kamchatka
    {  649, 4141,  202125,  6, }, // Asia/Karachi
    {  649, 1152,  202131,  9, }, // Asia/Kathmandu
    {  649, 8624,  202140,  6, }, // Asia/Khandyga
    {  649,  992,  202146,  7, }, // Asia/Kolkata
    {  649, 4099,  202153, 12, }, // Asia/Krasnoyarsk
    {  649, 4761,  202165, 12, }, // Asia/Kuala_Lumpur
    {  649, 5893,  202177,  5, }, // Asia/Kuching
    {  649, 5464,  202182,  7, }, // Asia/Kuwait
    {  649, 1178,  202189,  5, }, // Asia/Macau
    {  649, 3919,  202194,  7, }, // Asia/Magadan
    {  649, 7145,  202201,  6, }, // Asia/Manila
    {  649, 2446,  202207,  7, }, // Asia/Nicosia
    {  649, 8638,  202214, 16, }, // Asia/Novokuznetsk
    {  649, 4037,  202230, 13, }, // Asia/Novosibirsk
    {  649, 4131,  202243,  7, }, // Asia/Omsk
    {  649, 7108,  202250,  4, }, // Asia/Oral
    {  649, 6924,  202254,  9, }, // Asia/Phnom_Penh
    {  649, 8656,  202263, 10, }, // Asia/Pontianak
    {  649, 4116,  202273,  8, }, // Asia/Pyongyang
    {  649, 5476,    4233,  5, }, // Asia/Qatar
    {  649, 8671,  202281,  8, }, // Asia/Qostanay
    {  649, 4171,  202289,  9, }, // Asia/Qyzylorda
    {  649, 3486,  202298,  6, }, // Asia/Riyadh
    {  649, 4310,  202304,  8, }, // Asia/Sakhalin
    {  649, 7208,  202312, 10, }, // Asia/Samarkand
    {  649, 3116,  202322,  5, }, // Asia/Seoul
    {  649, 1020,  202327,  6, }, // Asia/Shanghai
    {  649, 3137,  202333,  8, }, // Asia/Singapore
    {  649, 4199,  202341, 15, }, // Asia/Srednekolymsk
    {  649, 3100,    6163,  6, }, // Asia/Taipei
    {  649, 4683,  202356,  9, }, // Asia/Tashkent
    {  649, 3817,  202365,  8, }, // Asia/Tbilisi
    {  649, 2638,  202373,  6, }, // Asia/Tehran
    {  649, 1284,  202379,  5, }, // Asia/Thimphu
    {  649, 2687,  202384,  6, }, // Asia/Tokyo
    {  649, 4474,  202390,  8, }, // Asia/Tomsk
    {  649, 1346,  202398, 10, }, // Asia/Ulaanbaatar
    {  649, 1126,  202408,  6, }, // Asia/Urumqi
    {  649, 8685,  202414, 10, }, // Asia/Ust-Nera
    {  649, 6940,  202424, 10, }, // Asia/Vientiane
    {  649, 4612,  202434, 13, }, // Asia/Vladivostok
    {  649, 4730,  202447, 11, }, // Asia/Yakutsk
    {  649, 1202,  202458,  6, }, // Asia/Yangon
    {  649, 3785,  202464, 13, }, // Asia/Yekaterinburg
    {  649, 3649,  202477,  6, }, // Asia/Yerevan
    {  649, 3565,  202483,  7, }, // Atlantic/Azores
    {  649, 5588,  202490,  7, }, // Atlantic/Bermuda
    {  649, 6591,  202497, 11, }, // Atlantic/Canary
    {  649, 3629,  202508,  7, }, // Atlantic/Cape_Verde
    {  649, 1379,     448,  5, }, // Atlantic/Faroe
    {  649, 8699,     413,  2, }, // Atlantic/Madeira
    {  649, 2614,  202515, 11, }, // Atlantic/Reykjavik
    {  649, 7235,  202526, 13, }, // Atlantic/South_Georgia
    {  649, 6784,  202539, 10, }, // Atlantic/St_Helena
    {  649, 6607,  202549,  7, }, // Atlantic/Stanley
    {  649, 1654,  202556,  8, }, // Australia/Adelaide
    {  649, 1619,  202564,  9, }, // Australia/Brisbane
    {  649, 1783,  202573, 10, }, // Australia/Broken_Hill
    {  649, 1567,  202583,  6, }, // Australia/Darwin
    {  649, 3539,  202589,  5, }, // Australia/Eucla
    {  649, 1500,  202594,  7, }, // Australia/Hobart
    {  649, 8716,  202601,  8, }, // Australia/Lindeman
    {  649, 1531,  202609, 10, }, // Australia/Lord_Howe
    {  649, 1711,  202619,  8, }, // Australia/Melbourne
    {  649, 1746,  202627,  5, }, // Australia/Perth
    {  649, 1447,  202632,  7, }, // Australia/Sydney
    {  649, 6390,  202639, 10, }, // Europe/Amsterdam
    {  649, 6149,  202649,  5, }, // Europe/Andorra
    {  649, 3522,  202654,  7, }, // Europe/Astrakhan
    {  649, 6577,  202661,  5, }, // Europe/Athens
    {  649, 6285,  202666,  8, }, // Europe/Belgrade
    {  649, 4659,  202674,  7, }, // Europe/Berlin
    {  649, 6437,  202681, 11, }, // Europe/Bratislava
    {  649, 6178,  202692,  7, }, // Europe/Brussels
    {  649, 3830,  202699, 10, }, // Europe/Bucharest
    {  649, 3708,  202709, 10, }, // Europe/Budapest
    {  649, 8751,  202719,  8, }, // Europe/Busingen
    {  649, 2475,  202727,  8, }, // Europe/Chisinau
    {  649, 6238,  202735,  9, }, // Europe/Copenhagen
    {  649, 2260,  202744,  6, }, // Europe/Dublin
    {  649, 6256,  202750,  8, }, // Europe/Gibraltar
    {  649, 8767,  202758,  8, }, // Europe/Guernsey
    {  649, 6561,  202766,  8, }, // Europe/Helsinki
    {  649, 8783,  202774, 15, }, // Europe/Isle_of_Man
    {  649, 1097,  202789,  8, }, // Europe/Istanbul
    {  649, 8802,  202797,  6, }, // Europe/Jersey
    {  649, 3881,  202803, 12, }, // Europe/Kaliningrad
    {  649, 8924,  202815,  6, }, // Europe/Kirov
    {  649, 2419,  202821,  6, }, // Europe/Kyiv
    {  649, 3078,  202827,  8, }, // Europe/Lisbon
    {  649, 6455,  202835,  8, }, // Europe/Ljubljana
    {  649, 2393,  202843,  6, }, // Europe/London
    {  649, 6314,  202849, 10, }, // Europe/Luxembourg
    {  649, 4794,  202859,  6, }, // Europe/Madrid
    {  649, 6346,  202865,  5, }, // Europe/Malta
    {  649, 6531,  202870,  8, }, // Europe/Mariehamn
    {  649, 3595,  202878,  7, }, // Europe/Minsk
    {  649, 6359,  202885,  6, }, // Europe/Monaco
    {  649, 3443,  202891,  7, }, // Europe/Moscow
    {  649, 6407,  202898,  5, }, // Europe/Oslo
    {  649, 4186,  202903,  6, }, // Europe/Paris
    {  649, 6373,  202909,  9, }, // Europe/Podgorica
    {  649, 6224,   19576,  5, }, // Europe/Prague
    {  649, 6273,  202918,  4, }, // Europe/Rome
    {  649, 4233,  202922,  6, }, // Europe/Samara
    {  649, 6419,  202928,  9, }, // Europe/San_Marino
    {  649, 6194,  202937, 17, }, // Europe/Sarajevo
    {  649, 4353,  202954,  8, }, // Europe/Saratov
    {  649, 8828,  202962, 10, }, // Europe/Simferopol
    {  649, 6332,  202972,  8, }, // Europe/Skopje
    {  649, 6548,  202980,  6, }, // Europe/Sofia
    {  649, 6472,  202986, 10, }, // Europe/Stockholm
    {  649, 8846,  202996,  5, }, // Europe/Tallinn
    {  649, 6135,  203001,  6, }, // Europe/Tirane
    {  649, 8861,  203007, 13, }, // Europe/Ulyanovsk
    {  649, 6301,  203020,  5, }, // Europe/Vaduz
    {  649, 6516,  203025,  8, }, // Europe/Vatican
    {  649, 6164,  203033,  5, }, // Europe/Vienna
    {  649, 8878,  203038,  9, }, // Europe/Vilnius
    {  649, 4629,  203047, 10, }, // Europe/Volgograd
    {  649, 3055,  203057,  9, }, // Europe/Warsaw
    {  649, 6210,  203066,  7, }, // Europe/Zagreb
    {  649, 6489,  203073,  6, }, // Europe/Zurich
    {  649, 4980,  203079, 11, }, // Indian/Antananarivo
    {  649, 6910,  203090,  4, }, // Indian/Chagos
    {  649, 5979,  203094,  9, }, // Indian/Christmas
    {  649, 5996,  203103,  7, }, // Indian/Cocos
    {  649, 4931,  203110,  6, }, // Indian/Comoro
    {  649, 6624,  203116,  8, }, // Indian/Kerguelen
    {  649, 7223,  203124,  4, }, // Indian/Mahe
    {  649, 7013,  203128,  7, }, // Indian/Maldives
    {  649, 3971,  203135,  7, }, // Indian/Mauritius
    {  649, 5000,  203142,  6, }, // Indian/Mayotte
    {  649, 7174,  203148,  6, }, // Indian/Reunion
    {  649, 4324,  120112,  5, }, // Pacific/Apia
    {  649,  932,  203154,  6, }, // Pacific/Auckland
    {  649, 3608,  203160,  9, }, // Pacific/Bougainville
    {  649, 2863,  203169,  7, }, // Pacific/Chatham
    {  649, 3022,     474,  5, }, // Pacific/Chuuk
    {  649, 2201,  203176, 17, }, // Pacific/Easter
    {  649, 7323,   25598,  5, }, // Pacific/Efate
    {  649, 7290,  203193,  7, }, // Pacific/Fakaofo
    {  649, 7306,  203200,  8, }, // Pacific/Funafuti
    {  649, 6641,  203208,  9, }, // Pacific/Galapagos
    {  649, 6659,  203217,  7, }, // Pacific/Gambier
    {  649, 3724,  203224, 12, }, // Pacific/Guadalcanal
    {  649, 5935,  203236,  5, }, // Pacific/Guam
    {  649, 2929,  203241,  8, }, // Pacific/Honolulu
    {  649, 2897,  203249,  9, }, // Pacific/Kanton
    {  649, 3900,  203258, 10, }, // Pacific/Kiritimati
    {  649, 6982,  203268,  8, }, // Pacific/Kosrae
    {  649, 2708,  203276,  9, }, // Pacific/Kwajalein
    {  649, 4779,  203285,  6, }, // Pacific/Majuro
    {  649, 3953,  203291, 11, }, // Pacific/Marquesas
    {  649, 8909,  203302,  8, }, // Pacific/Midway
    {  649, 7047,  203310,  6, }, // Pacific/Nauru
    {  649, 7095,  203316,  4, }, // Pacific/Niue
    {  649, 4070,  203320,  7, }, // Pacific/Norfolk
    {  649, 7061,  203327,  6, }, // Pacific/Noumea
    {  649, 7118,  203333,  5, }, // Pacific/Palau
    {  649, 7157,  203338,  8, }, // Pacific/Pitcairn
    {  649, 2961,     479,  7, }, // Pacific/Pohnpei
    {  649, 4709,  203346, 12, }, // Pacific/Port_Moresby
    {  649, 6009,  203358,  8, }, // Pacific/Rarotonga
    {  649, 5948,  203366,  5, }, // Pacific/Saipan
    {  649, 7275,  203371,  5, }, // Pacific/Tahiti
    {  649, 6675,  203376,  7, }, // Pacific/Tarawa
    {  649, 4485,  203383,  9, }, // Pacific/Tongatapu
    {  649, 7355,  203392,  4, }, // Pacific/Wake
    {  649, 7368,  203396,  6, }, // Pacific/Wallis
    {  650, 6751,  203402,  7, }, // Africa/Abidjan Nheengatu/Latin/Brazil
    {  650, 6723,  203409,  5, }, // Africa/Accra
    {  650, 4961,  203414, 11, }, // Africa/Addis_Ababa
    {  650, 7383,  203425,  4, }, // Africa/Algiers
    {  650,   14,  203429,  7, }, // Africa/Asmara
    {  650,   44,   96814,  6, }, // Africa/Bamako
    {  650, 5159,  203436,  4, }, // Africa/Bangui
    {  650, 6709,  203440,  4, }, // Africa/Banjul
    {  650, 7398,    4583,  5, }, // Africa/Bissau
    {  650, 4859,  203444,  8, }, // Africa/Blantyre
    {  650, 5189,  203452, 10, }, // Africa/Brazzaville
    {  650, 4824,  203462,  8, }, // Africa/Bujumbura
    {  650, 2242,  203470,  5, }, // Africa/Cairo
    {  650, 4019,  203475, 12, }, // Africa/Casablanca
    {  650, 7412,   19754,  5, }, // Africa/Ceuta
    {  650, 6736,  203487,  8, }, // Africa/Conakry
    {  650, 5032,  203495, 12, }, // Africa/Dar_es_Salaam
    {  650, 4945,  203507,  8, }, // Africa/Djibouti
    {  650, 5145,  203515,  6, }, // Africa/Douala
    {  650, 5068,  203521,  6, }, // Africa/El_Aaiun
    {  650, 6816,  203527,  6, }, // Africa/Freetown
    {  650, 4808,  203533,  8, }, // Africa/Gaborone
    {  650, 4381,  203541, 11, }, // Africa/Johannesburg
    {  650, 4401,  203552,  4, }, // Africa/Juba
    {  650, 5053,  203556,  6, }, // Africa/Kampala
    {  650, 4426,  203562,  4, }, // Africa/Khartoum
    {  650, 4875,  200402,  6, }, // Africa/Kigali
    {  650, 5208,  203566,  6, }, // Africa/Kinshasa
    {  650, 4646,  203572,  8, }, // Africa/Lagos
    {  650, 5238,  203580, 10, }, // Africa/Libreville
    {  650, 6832,  203590,  4, }, // Africa/Lome
    {  650, 5113,  203594,  5, }, // Africa/Luanda
    {  650, 4841,  203599,  8, }, // Africa/Lubumbashi
    {  650, 5224,   96970,  6, }, // Africa/Malabo
    {  650, 4889,  117783,  6, }, // Africa/Maputo
    {  650, 5084,  203607,  7, }, // Africa/Mbabane
    {  650, 5015,  203614, 10, }, // Africa/Mogadishu
    {  650, 6997,  203624,  8, }, // Africa/Monrovia
    {  650, 3755,  203632,  7, }, // Africa/Nairobi
    {  650, 5173,  203639, 11, }, // Africa/Ndjamena
    {  650, 5256,   24961,  6, }, // Africa/Niamey
    {  650, 6766,  203650, 10, }, // Africa/Nouakchott
    {  650, 6690,  203660,  9, }, // Africa/Ouagadougou
    {  650, 5127,  203669, 16, }, // Africa/Porto-Novo
    {  650, 4337,  203685,  7, }, // Africa/Sao_Tome
    {  650, 2732,  203692,  8, }, // Africa/Tripoli
    {  650, 6503,  203700,  8, }, // Africa/Tunis
    {  650, 4054,  203708,  7, }, // Africa/Windhoek
    {  650,  132,   96487,  5, }, // America/Adak
    {  650, 3183,  203715,  7, }, // America/Anchorage
    {  650, 5524,  203722,  5, }, // America/Anguilla
    {  650, 5541,  203727,  7, }, // America/Antigua
    {  650, 4456,  203734, 10, }, // America/Araguaina
    {  650,  166,  203744, 11, }, // America/Argentina/Buenos_Aires
    {  650,   91,  203755,  8, }, // America/Argentina/Catamarca
    {  650,  270,  203763,  6, }, // America/Argentina/Cordoba
    {  650,  441,  203769,  5, }, // America/Argentina/Jujuy
    {  650, 7425,  203774,  8, }, // America/Argentina/La_Rioja
    {  650,  565,  203782,  6, }, // America/Argentina/Mendoza
    {  650, 7452,  203788, 17, }, // America/Argentina/Rio_Gallegos
    {  650, 7483,  203805,  5, }, // America/Argentina/Salta
    {  650, 7507,  203810,  7, }, // America/Argentina/San_Juan
    {  650, 5497,  203817,  6, }, // America/Argentina/San_Luis
    {  650, 7534,  203823,  6, }, // America/Argentina/Tucuman
    {  650, 7560,  203829,  7, }, // America/Argentina/Ushuaia
    {  650, 4154,  203836,  7, }, // America/Asuncion
    {  650,  237,  203843,  7, }, // America/Atikokan
    {  650, 3581,  203850,  5, }, // America/Bahia
    {  650, 7586,  203855, 16, }, // America/Bahia_Banderas
    {  650, 5571,  203871,  8, }, // America/Barbados
    {  650, 7609,  203879,  4, }, // America/Belem
    {  650, 5309,  203883,  6, }, // America/Belize
    {  650, 7623,  203889, 15, }, // America/Blanc-Sablon
    {  650, 7644,  203904, 10, }, // America/Boa_Vista
    {  650, 4263,  203914,  6, }, // America/Bogota
    {  650, 7662,  203920,  6, }, // America/Boise
    {  650, 7676,  203926, 14, }, // America/Cambridge_Bay
    {  650, 7698,  203940,  9, }, // America/Campo_Grande
    {  650, 3770,  203949,  4, }, // America/Cancun
    {  650, 4596,    4965,  6, }, // America/Caracas
    {  650, 4247,  203953,  6, }, // America/Cayenne
    {  650, 5398,  203959,  5, }, // America/Cayman
    {  650, 3251,   96536,  6, }, // America/Chicago
    {  650, 7719,  203964,  6, }, // America/Chihuahua
    {  650, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  650, 5324,  203970,  9, }, // America/Costa_Rica
    {  650, 7759,  203979,  7, }, // America/Creston
    {  650, 3693,   20036,  6, }, // America/Cuiaba
    {  650, 5640,   96555,  7, }, // America/Curacao
    {  650, 7775,  203986, 13, }, // America/Danmarkshavn
    {  650, 7796,  203999,  5, }, // America/Dawson
    {  650, 7811,  204004, 12, }, // America/Dawson_Creek
    {  650,  805,  204016,  5, }, // America/Denver
    {  650, 5656,  204021,  8, }, // America/Dominica
    {  650,  893,  204029,  7, }, // America/Edmonton
    {  650, 7832,   25013,  8, }, // America/Eirunepe
    {  650, 5343,  204036,  9, }, // America/El_Salvador
    {  650, 7849,  204045, 10, }, // America/Fort_Nelson
    {  650, 7869,  204055,  8, }, // America/Fortaleza
    {  650, 7887,  204063, 19, }, // America/Glace_Bay
    {  650, 6844,  204082, 15, }, // America/Goose_Bay
    {  650, 4514,  204097, 11, }, // America/Grand_Turk
    {  650, 5687,  204108,  8, }, // America/Grenada
    {  650, 5703,  204116,  9, }, // America/Guadeloupe
    {  650, 3662,  204125,  9, }, // America/Guatemala
    {  650, 4743,  204134,  8, }, // America/Guayaquil
    {  650, 6895,  204142,  6, }, // America/Guyana
    {  650, 2221,  149774,  6, }, // America/Havana
    {  650, 7905,  204148,  8, }, // America/Hermosillo
    {  650,  348,  204156, 10, }, // America/Indiana/Indianapolis
    {  650,  481,  204166, 12, }, // America/Indiana/Knox
    {  650, 7924,  204178, 14, }, // America/Indiana/Marengo
    {  650, 7948,  204192, 20, }, // America/Indiana/Petersburg
    {  650, 7975,  204212, 22, }, // America/Indiana/Tell_City
    {  650, 8001,  204234, 13, }, // America/Indiana/Vevay
    {  650, 8023,  204247, 18, }, // America/Indiana/Vincennes
    {  650, 8049,  204265, 16, }, // America/Indiana/Winamac
    {  650, 8073,  204281,  7, }, // America/Inuvik
    {  650,  660,  204288,  8, }, // America/Iqaluit
    {  650, 2665,  204296,  7, }, // America/Jamaica
    {  650, 5282,  204303,  6, }, // America/Juneau
    {  650,  521,  204309, 10, }, // America/Kentucky/Louisville
    {  650, 8088,  204319, 16, }, // America/Kentucky/Monticello
    {  650, 5621,  204335, 10, }, // America/Kralendijk
    {  650, 7132,  204345,  4, }, // America/Lima
    {  650, 5849,  204349, 26, }, // America/Lower_Princes
    {  650, 8116,  204375,  6, }, // America/Maceio
    {  650, 8131,  204381,  8, }, // America/Managua
    {  650, 1892,  118519,  5, }, // America/Manaus
    {  650, 5814,  204389,  8, }, // America/Marigot
    {  650, 5722,  204397,  8, }, // America/Martinique
    {  650, 8147,  204405, 11, }, // America/Matamoros
    {  650, 2779,  204416,  8, }, // America/Mazatlan
    {  650, 8165,    5409,  8, }, // America/Menominee
    {  650, 8183,     229,  6, }, // America/Merida
    {  650, 8198,  204424, 12, }, // America/Metlakatla
    {  650, 2811,  204436, 13, }, // America/Mexico_City
    {  650, 4293,  204449,  6, }, // America/Miquelon
    {  650, 8217,  204455,  6, }, // America/Moncton
    {  650, 8233,  204461,  7, }, // America/Monterrey
    {  650, 4000,  204468, 10, }, // America/Montevideo
    {  650, 5741,  204478, 10, }, // America/Montserrat
    {  650, 5383,  121439,  5, }, // America/Nassau
    {  650, 3294,  204488, 12, }, // America/New_York
    {  650, 8251,  204500,  4, }, // America/Nome
    {  650, 1834,  204504, 13, }, // America/Noronha
    {  650, 8264,  204517, 21, }, // America/North_Dakota/Beulah
    {  650, 8292,  204538, 21, }, // America/North_Dakota/Center
    {  650, 8320,  204559, 28, }, // America/North_Dakota/New_Salem
    {  650,  393,  204587,  4, }, // America/Nuuk
    {  650, 8351,  204591,  7, }, // America/Ojinaga
    {  650, 5413,  204598,  6, }, // America/Panama
    {  650, 6106,   97376, 10, }, // America/Paramaribo
    {  650, 3224,  204604, 10, }, // America/Phoenix
    {  650, 3847,  204614, 18, }, // America/Port-au-Prince
    {  650, 5871,  204632, 17, }, // America/Port_of_Spain
    {  650, 8367,  204649, 20, }, // America/Porto_Velho
    {  650, 5760,  204669, 14, }, // America/Puerto_Rico
    {  650, 3932,  204683, 14, }, // America/Punta_Arenas
    {  650, 8387,  204697, 13, }, // America/Rankin_Inlet
    {  650, 8408,  118755,  6, }, // America/Recife
    {  650, 1979,  204710,  6, }, // America/Regina
    {  650, 8423,  204716,  8, }, // America/Resolute
    {  650,  695,  204724, 15, }, // America/Rio_Branco
    {  650, 8440,  204739,  6, }, // America/Santarem
    {  650, 2165,  204745,  8, }, // America/Santiago
    {  650, 6044,  204753, 11, }, // America/Santo_Domingo
    {  650, 1862,  204764,  8, }, // America/Sao_Paulo
    {  650, 6862,  204772, 11, }, // America/Scoresbysund
    {  650, 8457,  204783,  6, }, // America/Sitka
    {  650, 8471,  204789, 12, }, // America/St_Barthelemy
    {  650, 2045,  204801,  9, }, // America/St_Johns
    {  650, 5780,  204810, 12, }, // America/St_Kitts
    {  650, 5797,  204822, 11, }, // America/St_Lucia
    {  650,  855,  204833,  9, }, // America/St_Thomas
    {  650, 5830,  204842,  9, }, // America/St_Vincent
    {  650, 8493,  204851, 13, }, // America/Swift_Current
    {  650, 5363,  204864, 11, }, // America/Tegucigalpa
    {  650, 5673,  204875,  6, }, // America/Thule
    {  650,  313,  204881,  8, }, // America/Tijuana
    {  650,  608,  204889,  6, }, // America/Toronto
    {  650, 5605,  204895,  6, }, // America/Tortola
    {  650, 2077,  204901,  8, }, // America/Vancouver
    {  650, 2128,  204909, 15, }, // America/Whitehorse
    {  650,  734,  204924,  8, }, // America/Winnipeg
    {  650, 8515,  198162,  8, }, // America/Yakutat
    {  650, 5918,  204932,  5, }, // Antarctica/Casey
    {  650, 6027,  204937,  4, }, // Antarctica/Davis
    {  650, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  650, 8531,  204941,  9, }, // Antarctica/Macquarie
    {  650, 7029,  204950,  5, }, // Antarctica/Mawson
    {  650, 7076,  204955,  8, }, // Antarctica/McMurdo
    {  650, 8552,  204963,  7, }, // Antarctica/Palmer
    {  650, 7189,  118999,  6, }, // Antarctica/Rothera
    {  650, 7258,  168396,  6, }, // Antarctica/Syowa
    {  650, 7337,  204970,  8, }, // Antarctica/Vostok
    {  650, 1413,  204978, 18, }, // Arctic/Longyearbyen
    {  650, 5487,  204996,  3, }, // Asia/Aden
    {  650, 5297,  204999,  6, }, // Asia/Almaty
    {  650, 3870,  150810,  3, }, // Asia/Amman
    {  650, 5428,    5900,  5, }, // Asia/Anadyr
    {  650, 5440,  201937,  6, }, // Asia/Aqtau
    {  650, 5270,  205005,  7, }, // Asia/Aqtobe
    {  650,  964,  205012,  9, }, // Asia/Ashgabat
    {  650, 8587,    5917,  6, }, // Asia/Atyrau
    {  650, 3509,  205021,  5, }, // Asia/Baghdad
    {  650, 5451,  205026,  5, }, // Asia/Bahrain
    {  650, 4368,  205031,  8, }, // Asia/Bangkok
    {  650, 3473,  119089,  6, }, // Asia/Barnaul
    {  650, 3988,  150821,  7, }, // Asia/Beirut
    {  650, 3680,  205039,  8, }, // Asia/Bishkek
    {  650, 5906,  205047,  7, }, // Asia/Brunei
    {  650, 4503,  205054,  4, }, // Asia/Chita
    {  650, 5963,  205058,  8, }, // Asia/Choibalsan
    {  650, 4413,  205066,  6, }, // Asia/Colombo
    {  650, 4442,  205072,  8, }, // Asia/Damascus
    {  650, 1060,    5956,  4, }, // Asia/Dhaka
    {  650, 6125,  205080,  4, }, // Asia/Dili
    {  650, 6092,  205084,  6, }, // Asia/Dushanbe
    {  650, 8599,  205090,  9, }, // Asia/Famagusta
    {  650, 8614,   62340,  4, }, // Asia/Gaza
    {  650, 4697,  205099,  6, }, // Asia/Hebron
    {  650, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  650, 2576,  205105,  7, }, // Asia/Hong_Kong
    {  650, 4673,  205112,  6, }, // Asia/Hovd
    {  650, 4086,  205118, 10, }, // Asia/Irkutsk
    {  650, 6969,  205128,  6, }, // Asia/Jakarta
    {  650, 6955,  205134,  8, }, // Asia/Jayapura
    {  650, 1257,  205142,  9, }, // Asia/Jerusalem
    {  650, 3462,   91229,  4, }, // Asia/Kabul
    {  650, 4218,  205151,  8, }, // Asia/Kamchatka
    {  650, 4141,  205159,  6, }, // Asia/Karachi
    {  650, 1152,  205165,  8, }, // Asia/Kathmandu
    {  650, 8624,  205173,  6, }, // Asia/Khandyga
    {  650,  992,  205179,  7, }, // Asia/Kolkata
    {  650, 4099,  205186, 16, }, // Asia/Krasnoyarsk
    {  650, 4761,  205202, 12, }, // Asia/Kuala_Lumpur
    {  650, 5893,  205214,  6, }, // Asia/Kuching
    {  650, 5464,  198653,  7, }, // Asia/Kuwait
    {  650, 1178,   32008,  5, }, // Asia/Macau
    {  650, 3919,  205220,  6, }, // Asia/Magadan
    {  650, 7145,  130677,  6, }, // Asia/Manila
    {  650, 6883,  205226,  8, }, // Asia/Muscat
    {  650, 2446,   20920,  8, }, // Asia/Nicosia
    {  650, 8638,  205234, 19, }, // Asia/Novokuznetsk
    {  650, 4037,  205253, 17, }, // Asia/Novosibirsk
    {  650, 4131,  205270,  7, }, // Asia/Omsk
    {  650, 7108,  205277,  5, }, // Asia/Oral
    {  650, 6924,  205282,  8, }, // Asia/Phnom_Penh
    {  650, 8656,  205290,  9, }, // Asia/Pontianak
    {  650, 4116,  205299,  8, }, // Asia/Pyongyang
    {  650, 5476,  205307,  6, }, // Asia/Qatar
    {  650, 8671,  205313,  9, }, // Asia/Qostanay
    {  650, 4171,  205322,  8, }, // Asia/Qyzylorda
    {  650, 3486,  150887,  5, }, // Asia/Riyadh
    {  650, 4310,  205330,  8, }, // Asia/Sakhalin
    {  650, 7208,  205338,  8, }, // Asia/Samarkand
    {  650, 3116,   19287,  3, }, // Asia/Seoul
    {  650, 1020,  205346,  5, }, // Asia/Shanghai
    {  650, 3137,  202333,  8, }, // Asia/Singapore
    {  650, 4199,  205351, 18, }, // Asia/Srednekolymsk
    {  650, 4683,  205369,  8, }, // Asia/Tashkent
    {  650, 3817,  168821,  8, }, // Asia/Tbilisi
    {  650, 2638,  205377,  6, }, // Asia/Tehran
    {  650, 1284,  205383,  4, }, // Asia/Thimphu
    {  650, 2687,  205387,  6, }, // Asia/Tokyo
    {  650, 4474,  205393,  8, }, // Asia/Tomsk
    {  650, 1346,  205401, 10, }, // Asia/Ulaanbaatar
    {  650, 1126,  205411,  5, }, // Asia/Urumqi
    {  650, 8685,  205416, 10, }, // Asia/Ust-Nera
    {  650, 6940,  205426,  8, }, // Asia/Vientiane
    {  650, 4612,  205434, 14, }, // Asia/Vladivostok
    {  650, 4730,  205448, 10, }, // Asia/Yakutsk
    {  650, 1202,  205458,  4, }, // Asia/Yangon
    {  650, 3785,  205462, 11, }, // Asia/Yekaterinburg
    {  650, 3649,  205473,  6, }, // Asia/Yerevan
    {  650, 3565,  205479,  9, }, // Atlantic/Azores
    {  650, 5588,  205488,  6, }, // Atlantic/Bermuda
    {  650, 6591,  205494, 12, }, // Atlantic/Canary
    {  650, 3629,  205506, 12, }, // Atlantic/Cape_Verde
    {  650, 1379,  205518, 18, }, // Atlantic/Faroe
    {  650, 2614,  205536, 11, }, // Atlantic/Reykjavik
    {  650, 7235,  205547, 14, }, // Atlantic/South_Georgia
    {  650, 6784,  205561, 10, }, // Atlantic/St_Helena
    {  650, 6607,  205571,  8, }, // Atlantic/Stanley
    {  650, 1654,  205579,  8, }, // Australia/Adelaide
    {  650, 1619,  205587,  8, }, // Australia/Brisbane
    {  650, 1783,  205595, 17, }, // Australia/Broken_Hill
    {  650, 1567,  205612,  6, }, // Australia/Darwin
    {  650, 3539,  205618,  6, }, // Australia/Eucla
    {  650, 1500,  205624,  6, }, // Australia/Hobart
    {  650, 8716,  205630,  6, }, // Australia/Lindeman
    {  650, 1531,  205636, 11, }, // Australia/Lord_Howe
    {  650, 1711,  205647,  7, }, // Australia/Melbourne
    {  650, 1746,  205654,  6, }, // Australia/Perth
    {  650, 1447,  205660,  7, }, // Australia/Sydney
    {  650, 6390,  205667,  9, }, // Europe/Amsterdam
    {  650, 6149,  205676,  5, }, // Europe/Andorra
    {  650, 3522,  205681,  9, }, // Europe/Astrakhan
    {  650, 6577,  205690,  9, }, // Europe/Athens
    {  650, 6285,  205699,  9, }, // Europe/Belgrade
    {  650, 4659,  150984,  6, }, // Europe/Berlin
    {  650, 6437,  205708, 12, }, // Europe/Bratislava
    {  650, 6178,  205720,  8, }, // Europe/Brussels
    {  650, 3830,  205728,  8, }, // Europe/Bucharest
    {  650, 3708,  205736,  8, }, // Europe/Budapest
    {  650, 8751,  205744,  8, }, // Europe/Busingen
    {  650, 2475,  205752,  7, }, // Europe/Chisinau
    {  650, 6238,  205759,  8, }, // Europe/Copenhagen
    {  650, 2260,  205767,  6, }, // Europe/Dublin
    {  650, 6256,  205773,  9, }, // Europe/Gibraltar
    {  650, 8767,  205782,  9, }, // Europe/Guernsey
    {  650, 6561,  205791,  7, }, // Europe/Helsinki
    {  650, 8783,  205798, 10, }, // Europe/Isle_of_Man
    {  650, 1097,   55967,  8, }, // Europe/Istanbul
    {  650, 8802,  205808,  6, }, // Europe/Jersey
    {  650, 3881,  205814, 12, }, // Europe/Kaliningrad
    {  650, 8924,  205826,  6, }, // Europe/Kirov
    {  650, 2419,  205832,  6, }, // Europe/Kyiv
    {  650, 3078,   19528,  6, }, // Europe/Lisbon
    {  650, 6455,  205838, 10, }, // Europe/Ljubljana
    {  650, 2393,   19543,  7, }, // Europe/London
    {  650, 6314,  205848,  8, }, // Europe/Luxembourg
    {  650, 4794,   25525,  5, }, // Europe/Madrid
    {  650, 6346,  205856,  5, }, // Europe/Malta
    {  650, 6531,  205861,  7, }, // Europe/Mariehamn
    {  650, 3595,  205868,  6, }, // Europe/Minsk
    {  650, 6359,   97849,  6, }, // Europe/Monaco
    {  650, 3443,    4423,  6, }, // Europe/Moscow
    {  650, 6407,  205874,  5, }, // Europe/Oslo
    {  650, 6373,  205879, 10, }, // Europe/Podgorica
    {  650, 6224,   19576,  5, }, // Europe/Prague
    {  650, 6273,   19581,  4, }, // Europe/Rome
    {  650, 6419,  205889,  9, }, // Europe/San_Marino
    {  650, 6194,  205898,  8, }, // Europe/Sarajevo
    {  650, 4353,  205906,  8, }, // Europe/Saratov
    {  650, 8828,  205914,  8, }, // Europe/Simferopol
    {  650, 6332,  205922,  9, }, // Europe/Skopje
    {  650, 6548,  205931,  6, }, // Europe/Sofia
    {  650, 6472,   73074,  9, }, // Europe/Stockholm
    {  650, 8846,  202996,  4, }, // Europe/Tallinn
    {  650, 6135,   19608,  6, }, // Europe/Tirane
    {  650, 8861,  205937, 10, }, // Europe/Ulyanovsk
    {  650, 6301,  205947,  6, }, // Europe/Vaduz
    {  650, 6516,  205953,  8, }, // Europe/Vatican
    {  650, 6164,  205961,  6, }, // Europe/Vienna
    {  650, 8878,  205967,  9, }, // Europe/Vilnius
    {  650, 4629,  205976, 10, }, // Europe/Volgograd
    {  650, 3055,  205986, 10, }, // Europe/Warsaw
    {  650, 6210,  205996,  8, }, // Europe/Zagreb
    {  650, 6489,  206004,  6, }, // Europe/Zurich
    {  650, 4980,  206010, 10, }, // Indian/Antananarivo
    {  650, 6910,  206020,  8, }, // Indian/Chagos
    {  650, 5979,  206028, 12, }, // Indian/Christmas
    {  650, 5996,  206040,  8, }, // Indian/Cocos
    {  650, 4931,  206048, 10, }, // Indian/Comoro
    {  650, 6624,  206058,  7, }, // Indian/Kerguelen
    {  650, 7223,  206065,  3, }, // Indian/Mahe
    {  650, 7013,  206068, 11, }, // Indian/Maldives
    {  650, 3971,  206079,  9, }, // Indian/Mauritius
    {  650, 5000,  206088,  6, }, // Indian/Mayotte
    {  650, 7174,  206094, 13, }, // Indian/Reunion
    {  650, 4324,  206107,  5, }, // Pacific/Apia
    {  650,  932,  206112,  7, }, // Pacific/Auckland
    {  650, 3608,  206119,  9, }, // Pacific/Bougainville
    {  650, 2863,  206128,  6, }, // Pacific/Chatham
    {  650, 3022,     474,  5, }, // Pacific/Chuuk
    {  650, 2201,  206134, 16, }, // Pacific/Easter
    {  650, 7323,  206150,  5, }, // Pacific/Efate
    {  650, 7290,  206155,  7, }, // Pacific/Fakaofo
    {  650, 3804,  140723,  4, }, // Pacific/Fiji
    {  650, 6641,  206162, 12, }, // Pacific/Galapagos
    {  650, 6659,  206174,  7, }, // Pacific/Gambier
    {  650, 3724,  206181, 11, }, // Pacific/Guadalcanal
    {  650, 5935,  206192,  4, }, // Pacific/Guam
    {  650, 2929,  206196,  8, }, // Pacific/Honolulu
    {  650, 2897,  206204,  7, }, // Pacific/Kanton
    {  650, 6982,  206211,  7, }, // Pacific/Kosrae
    {  650, 2708,  206218,  8, }, // Pacific/Kwajalein
    {  650, 4779,  206226,  6, }, // Pacific/Majuro
    {  650, 3953,  206232, 10, }, // Pacific/Marquesas
    {  650, 8909,  206242,  7, }, // Pacific/Midway
    {  650, 7095,  206249,  4, }, // Pacific/Niue
    {  650, 4070,  206253,  8, }, // Pacific/Norfolk
    {  650, 7061,  206261,  6, }, // Pacific/Noumea
    {  650, 2991,  206267,  9, }, // Pacific/Pago_Pago
    {  650, 7118,  206276,  5, }, // Pacific/Palau
    {  650, 2961,     479,  7, }, // Pacific/Pohnpei
    {  650, 4709,  206281, 17, }, // Pacific/Port_Moresby
    {  650, 6009,  206298,  8, }, // Pacific/Rarotonga
    {  650, 5948,  206306,  5, }, // Pacific/Saipan
    {  650, 7275,  120324,  5, }, // Pacific/Tahiti
    {  650, 4485,  206311,  8, }, // Pacific/Tongatapu
    {  650, 7368,  206319,  8, }, // Pacific/Wallis
    {  651, 5780,  206327, 12, }, // America/St_Kitts Nheengatu/Latin/Colombia
    {  652, 5780,  206327, 12, }, // America/St_Kitts Nheengatu/Latin/Venezuela
    {  653,   14,       0,  6, }, // Africa/Asmara Haryanvi/Devanagari/India
    {  653, 4337,       6,  8, }, // Africa/Sao_Tome
    {  653, 4154,      14,  8, }, // America/Asuncion
    {  653,  237,      22,  8, }, // America/Atikokan
    {  653, 7586,      30, 17, }, // America/Bahia_Banderas
    {  653, 3770,      47,  6, }, // America/Cancun
    {  653, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  653, 5640,      66,  7, }, // America/Curacao
    {  653,  481,      73, 13, }, // America/Indiana/Knox
    {  653, 7924,      86, 16, }, // America/Indiana/Marengo
    {  653, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  653, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  653, 8001,     139, 14, }, // America/Indiana/Vevay
    {  653, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  653, 8049,     171, 16, }, // America/Indiana/Winamac
    {  653, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  653, 5849,     207, 22, }, // America/Lower_Princes
    {  653, 8183,     229,  6, }, // America/Merida
    {  653, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  653, 8292,     255, 20, }, // America/North_Dakota/Center
    {  653, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  653,  393,     298,  4, }, // America/Nuuk
    {  653, 6862,     302, 16, }, // America/Scoresbysund
    {  653, 8471,     318, 14, }, // America/St_Barthelemy
    {  653, 2045,     332, 10, }, // America/St_Johns
    {  653, 5780,     342,  9, }, // America/St_Kitts
    {  653, 5797,     351,  9, }, // America/St_Lucia
    {  653,  855,     360, 10, }, // America/St_Thomas
    {  653, 5830,     370, 11, }, // America/St_Vincent
    {  653, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  653, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  653, 1152,     413,  9, }, // Asia/Kathmandu
    {  653,  992,     422,  7, }, // Asia/Kolkata
    {  653, 1178,     429,  5, }, // Asia/Macau
    {  653, 1202,     442,  6, }, // Asia/Yangon
    {  653, 1379,     448,  5, }, // Atlantic/Faroe
    {  653, 6784,     453, 10, }, // Atlantic/St_Helena
    {  653, 2419,     463,  4, }, // Europe/Kyiv
    {  653, 7174,     467,  7, }, // Indian/Reunion
    {  653, 3022,     474,  5, }, // Pacific/Chuuk
    {  653, 2961,     479,  7, }, // Pacific/Pohnpei
    {  654, 7383,   57088,  6, }, // Africa/Algiers Northern Frisian/Latin/Germany
    {  654,   14,       0,  6, }, // Africa/Asmara
    {  654, 2242,   24913,  5, }, // Africa/Cairo
    {  654, 4337,       6,  8, }, // Africa/Sao_Tome
    {  654, 2732,   56289,  8, }, // Africa/Tripoli
    {  654, 5541,  206339,  8, }, // America/Antigua
    {  654, 5557,  206347,  6, }, // America/Aruba
    {  654, 4154,      14,  8, }, // America/Asuncion
    {  654,  237,      22,  8, }, // America/Atikokan
    {  654, 7586,   63067, 17, }, // America/Bahia_Banderas
    {  654, 5571,  206353,  9, }, // America/Barbados
    {  654, 3770,      47,  6, }, // America/Cancun
    {  654, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  654, 5640,      66,  7, }, // America/Curacao
    {  654, 5656,  206362,  9, }, // America/Dominica
    {  654, 5687,  206371,  8, }, // America/Grenada
    {  654, 3662,  206379, 10, }, // America/Guatemala
    {  654, 6895,  167503,  7, }, // America/Guyana
    {  654,  481,      73, 13, }, // America/Indiana/Knox
    {  654, 7924,      86, 16, }, // America/Indiana/Marengo
    {  654, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  654, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  654, 8001,     139, 14, }, // America/Indiana/Vevay
    {  654, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  654, 8049,     171, 16, }, // America/Indiana/Winamac
    {  654, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  654, 5849,     207, 22, }, // America/Lower_Princes
    {  654, 8183,     229,  6, }, // America/Merida
    {  654, 2811,  206389, 13, }, // America/Mexico_City
    {  654, 8264,  206402, 20, }, // America/North_Dakota/Beulah
    {  654, 8292,  206422, 20, }, // America/North_Dakota/Center
    {  654, 8320,  206442, 23, }, // America/North_Dakota/New_Salem
    {  654,  393,     298,  4, }, // America/Nuuk
    {  654, 5760,   20463, 11, }, // America/Puerto_Rico
    {  654, 6862,     302, 16, }, // America/Scoresbysund
    {  654, 8471,     318, 14, }, // America/St_Barthelemy
    {  654, 2045,     332, 10, }, // America/St_Johns
    {  654, 5780,     342,  9, }, // America/St_Kitts
    {  654, 5797,     351,  9, }, // America/St_Lucia
    {  654,  855,     360, 10, }, // America/St_Thomas
    {  654, 5830,     370, 11, }, // America/St_Vincent
    {  654, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  654, 7337,    4048,  6, }, // Antarctica/Vostok
    {  654,  964,  206465,  9, }, // Asia/Ashgabat
    {  654, 3509,    4063,  6, }, // Asia/Baghdad
    {  654, 5451,   71482,  8, }, // Asia/Bahrain
    {  654, 3680,   55740,  8, }, // Asia/Bishkek
    {  654, 5963,   76626, 12, }, // Asia/Choibalsan
    {  654, 4442,    4102,  8, }, // Asia/Damascus
    {  654, 6092,   55748,  9, }, // Asia/Dushanbe
    {  654, 1226,  206474, 17, }, // Asia/Ho_Chi_Minh
    {  654, 2576,    4132,  8, }, // Asia/Hong_Kong
    {  654, 4673,   76655,  5, }, // Asia/Hovd
    {  654, 4218,   76660, 11, }, // Asia/Kamchatka
    {  654, 4141,   76671,  9, }, // Asia/Karachi
    {  654, 1152,     413,  9, }, // Asia/Kathmandu
    {  654,  992,   25307,  7, }, // Asia/Kolkata
    {  654, 4099,    4189, 11, }, // Asia/Krasnoyarsk
    {  654, 5893,  206491,  9, }, // Asia/Kuching
    {  654, 1178,     429,  5, }, // Asia/Macau
    {  654, 6883,   25325,  6, }, // Asia/Muscat
    {  654, 2446,   25331,  7, }, // Asia/Nicosia
    {  654, 8638,  206500, 12, }, // Asia/Novokuznetsk
    {  654, 4037,   76692, 11, }, // Asia/Novosibirsk
    {  654, 4116,   76703,  9, }, // Asia/Pyongyang
    {  654, 5476,  206512,  6, }, // Asia/Qatar
    {  654, 3486,    6120,  4, }, // Asia/Riyadh
    {  654, 4310,   56617,  8, }, // Asia/Sakhalin
    {  654, 1020,   55829,  9, }, // Asia/Shanghai
    {  654, 3137,  206518,  9, }, // Asia/Singapore
    {  654, 3100,  206527,  6, }, // Asia/Taipei
    {  654, 4683,   55848,  9, }, // Asia/Tashkent
    {  654, 3817,   76718,  6, }, // Asia/Tbilisi
    {  654, 2638,    6175,  7, }, // Asia/Tehran
    {  654, 2687,    4257,  5, }, // Asia/Tokyo
    {  654, 1126,  206533,  9, }, // Asia/Urumqi
    {  654, 6940,  206542,  9, }, // Asia/Vientiane
    {  654, 4612,    4262, 11, }, // Asia/Vladivostok
    {  654, 4730,    6206,  7, }, // Asia/Yakutsk
    {  654, 1202,  206551,  7, }, // Asia/Yangon
    {  654, 3785,    4281, 13, }, // Asia/Yekaterinburg
    {  654, 3649,  123587,  7, }, // Asia/Yerevan
    {  654, 3565,  206558,  8, }, // Atlantic/Azores
    {  654, 5588,  206566,  8, }, // Atlantic/Bermuda
    {  654, 6591,  206574,  8, }, // Atlantic/Canary
    {  654, 3629,  206582,  9, }, // Atlantic/Cape_Verde
    {  654, 1379,  206591,  7, }, // Atlantic/Faroe
    {  654, 7235,  206598, 12, }, // Atlantic/South_Georgia
    {  654, 6784,     453, 10, }, // Atlantic/St_Helena
    {  654, 6149,   32109,  6, }, // Europe/Andorra
    {  654, 3522,   76759,  9, }, // Europe/Astrakhan
    {  654, 6577,  206610,  6, }, // Europe/Athens
    {  654, 6285,   25465,  7, }, // Europe/Belgrade
    {  654, 6178,   21176,  7, }, // Europe/Brussels
    {  654, 3830,   25479,  8, }, // Europe/Bucharest
    {  654, 6238,  206616, 11, }, // Europe/Copenhagen
    {  654, 6256,  206627, 10, }, // Europe/Gibraltar
    {  654, 8783,  206637,  9, }, // Europe/Isle_of_Man
    {  654, 2419,   76773,  4, }, // Europe/Kyiv
    {  654, 3078,   32199,  7, }, // Europe/Lisbon
    {  654, 6314,  206646, 11, }, // Europe/Luxembourg
    {  654, 6359,    6525,  6, }, // Europe/Monaco
    {  654, 3443,   56005,  6, }, // Europe/Moscow
    {  654, 4186,   62218,  6, }, // Europe/Paris
    {  654, 6224,    4434,  5, }, // Europe/Prague
    {  654, 8816,  206657,  5, }, // Europe/Riga
    {  654, 6273,   62228,  4, }, // Europe/Rome
    {  654, 4353,   76777,  7, }, // Europe/Saratov
    {  654, 6135,   19608,  6, }, // Europe/Tirane
    {  654, 8861,   76784,  9, }, // Europe/Ulyanovsk
    {  654, 6516,  206662, 13, }, // Europe/Vatican
    {  654, 6164,  206675,  4, }, // Europe/Vienna
    {  654, 4629,    4462,  9, }, // Europe/Volgograd
    {  654, 3055,   56048,  8, }, // Europe/Warsaw
    {  654, 6489,    4478,  6, }, // Europe/Zurich
    {  654, 5979,  206679,  9, }, // Indian/Christmas
    {  654, 5996,  206688, 13, }, // Indian/Cocos
    {  654, 7013,  206701, 10, }, // Indian/Maldives
    {  654, 7174,     467,  7, }, // Indian/Reunion
    {  654, 3022,     474,  5, }, // Pacific/Chuuk
    {  654, 2201,  206711, 13, }, // Pacific/Easter
    {  654, 3804,   76834,  7, }, // Pacific/Fiji
    {  654, 4070,  206724, 13, }, // Pacific/Norfolk
    {  654, 2961,     479,  7, }, // Pacific/Pohnpei
    {  655,   14,       0,  6, }, // Africa/Asmara Rajasthani/Devanagari/India
    {  655, 4337,       6,  8, }, // Africa/Sao_Tome
    {  655, 4154,      14,  8, }, // America/Asuncion
    {  655,  237,      22,  8, }, // America/Atikokan
    {  655, 7586,      30, 17, }, // America/Bahia_Banderas
    {  655, 3770,      47,  6, }, // America/Cancun
    {  655, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  655, 5640,      66,  7, }, // America/Curacao
    {  655,  481,      73, 13, }, // America/Indiana/Knox
    {  655, 7924,      86, 16, }, // America/Indiana/Marengo
    {  655, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  655, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  655, 8001,     139, 14, }, // America/Indiana/Vevay
    {  655, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  655, 8049,     171, 16, }, // America/Indiana/Winamac
    {  655, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  655, 5849,     207, 22, }, // America/Lower_Princes
    {  655, 8183,     229,  6, }, // America/Merida
    {  655, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  655, 8292,     255, 20, }, // America/North_Dakota/Center
    {  655, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  655,  393,     298,  4, }, // America/Nuuk
    {  655, 6862,     302, 16, }, // America/Scoresbysund
    {  655, 8471,     318, 14, }, // America/St_Barthelemy
    {  655, 2045,     332, 10, }, // America/St_Johns
    {  655, 5780,     342,  9, }, // America/St_Kitts
    {  655, 5797,     351,  9, }, // America/St_Lucia
    {  655,  855,     360, 10, }, // America/St_Thomas
    {  655, 5830,     370, 11, }, // America/St_Vincent
    {  655, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  655, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  655, 1152,     413,  9, }, // Asia/Kathmandu
    {  655,  992,     422,  7, }, // Asia/Kolkata
    {  655, 1178,     429,  5, }, // Asia/Macau
    {  655, 1202,     442,  6, }, // Asia/Yangon
    {  655, 1379,     448,  5, }, // Atlantic/Faroe
    {  655, 6784,     453, 10, }, // Atlantic/St_Helena
    {  655, 2419,     463,  4, }, // Europe/Kyiv
    {  655, 7174,     467,  7, }, // Indian/Reunion
    {  655, 3022,     474,  5, }, // Pacific/Chuuk
    {  655, 2961,     479,  7, }, // Pacific/Pohnpei
    {  656,   14,       0,  6, }, // Africa/Asmara Moksha/Cyrillic/Russia
    {  656, 4337,       6,  8, }, // Africa/Sao_Tome
    {  656, 4154,      14,  8, }, // America/Asuncion
    {  656,  237,      22,  8, }, // America/Atikokan
    {  656, 7586,      30, 17, }, // America/Bahia_Banderas
    {  656, 3770,      47,  6, }, // America/Cancun
    {  656, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  656, 5640,      66,  7, }, // America/Curacao
    {  656,  481,      73, 13, }, // America/Indiana/Knox
    {  656, 7924,      86, 16, }, // America/Indiana/Marengo
    {  656, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  656, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  656, 8001,     139, 14, }, // America/Indiana/Vevay
    {  656, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  656, 8049,     171, 16, }, // America/Indiana/Winamac
    {  656, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  656, 5849,     207, 22, }, // America/Lower_Princes
    {  656, 8183,     229,  6, }, // America/Merida
    {  656, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  656, 8292,     255, 20, }, // America/North_Dakota/Center
    {  656, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  656,  393,     298,  4, }, // America/Nuuk
    {  656, 6862,     302, 16, }, // America/Scoresbysund
    {  656, 8471,     318, 14, }, // America/St_Barthelemy
    {  656, 2045,     332, 10, }, // America/St_Johns
    {  656, 5780,     342,  9, }, // America/St_Kitts
    {  656, 5797,     351,  9, }, // America/St_Lucia
    {  656,  855,     360, 10, }, // America/St_Thomas
    {  656, 5830,     370, 11, }, // America/St_Vincent
    {  656, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  656, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  656, 1152,     413,  9, }, // Asia/Kathmandu
    {  656,  992,     422,  7, }, // Asia/Kolkata
    {  656, 1178,     429,  5, }, // Asia/Macau
    {  656, 1202,     442,  6, }, // Asia/Yangon
    {  656, 1379,     448,  5, }, // Atlantic/Faroe
    {  656, 6784,     453, 10, }, // Atlantic/St_Helena
    {  656, 2419,     463,  4, }, // Europe/Kyiv
    {  656, 7174,     467,  7, }, // Indian/Reunion
    {  656, 3022,     474,  5, }, // Pacific/Chuuk
    {  656, 2961,     479,  7, }, // Pacific/Pohnpei
    {  657,   14,       0,  6, }, // Africa/Asmara Toki Pona/Latin/world
    {  657, 4337,       6,  8, }, // Africa/Sao_Tome
    {  657, 4154,      14,  8, }, // America/Asuncion
    {  657,  237,      22,  8, }, // America/Atikokan
    {  657, 7586,      30, 17, }, // America/Bahia_Banderas
    {  657, 3770,      47,  6, }, // America/Cancun
    {  657, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  657, 5640,      66,  7, }, // America/Curacao
    {  657,  481,      73, 13, }, // America/Indiana/Knox
    {  657, 7924,      86, 16, }, // America/Indiana/Marengo
    {  657, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  657, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  657, 8001,     139, 14, }, // America/Indiana/Vevay
    {  657, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  657, 8049,     171, 16, }, // America/Indiana/Winamac
    {  657, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  657, 5849,     207, 22, }, // America/Lower_Princes
    {  657, 8183,     229,  6, }, // America/Merida
    {  657, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  657, 8292,     255, 20, }, // America/North_Dakota/Center
    {  657, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  657,  393,     298,  4, }, // America/Nuuk
    {  657, 6862,     302, 16, }, // America/Scoresbysund
    {  657, 8471,     318, 14, }, // America/St_Barthelemy
    {  657, 2045,     332, 10, }, // America/St_Johns
    {  657, 5780,     342,  9, }, // America/St_Kitts
    {  657, 5797,     351,  9, }, // America/St_Lucia
    {  657,  855,     360, 10, }, // America/St_Thomas
    {  657, 5830,     370, 11, }, // America/St_Vincent
    {  657, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  657, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  657, 1152,     413,  9, }, // Asia/Kathmandu
    {  657,  992,     422,  7, }, // Asia/Kolkata
    {  657, 1178,     429,  5, }, // Asia/Macau
    {  657, 1202,     442,  6, }, // Asia/Yangon
    {  657, 1379,     448,  5, }, // Atlantic/Faroe
    {  657, 6784,     453, 10, }, // Atlantic/St_Helena
    {  657, 2419,     463,  4, }, // Europe/Kyiv
    {  657, 7174,     467,  7, }, // Indian/Reunion
    {  657, 3022,     474,  5, }, // Pacific/Chuuk
    {  657, 2961,     479,  7, }, // Pacific/Pohnpei
    {  658,   14,       0,  6, }, // Africa/Asmara Pijin/Latin/Solomon Islands
    {  658, 4337,       6,  8, }, // Africa/Sao_Tome
    {  658, 4154,      14,  8, }, // America/Asuncion
    {  658,  237,      22,  8, }, // America/Atikokan
    {  658, 7586,      30, 17, }, // America/Bahia_Banderas
    {  658, 3770,      47,  6, }, // America/Cancun
    {  658, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  658, 5640,      66,  7, }, // America/Curacao
    {  658,  481,      73, 13, }, // America/Indiana/Knox
    {  658, 7924,      86, 16, }, // America/Indiana/Marengo
    {  658, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  658, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  658, 8001,     139, 14, }, // America/Indiana/Vevay
    {  658, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  658, 8049,     171, 16, }, // America/Indiana/Winamac
    {  658, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  658, 5849,     207, 22, }, // America/Lower_Princes
    {  658, 8183,     229,  6, }, // America/Merida
    {  658, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  658, 8292,     255, 20, }, // America/North_Dakota/Center
    {  658, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  658,  393,     298,  4, }, // America/Nuuk
    {  658, 6862,     302, 16, }, // America/Scoresbysund
    {  658, 8471,     318, 14, }, // America/St_Barthelemy
    {  658, 2045,     332, 10, }, // America/St_Johns
    {  658, 5780,     342,  9, }, // America/St_Kitts
    {  658, 5797,     351,  9, }, // America/St_Lucia
    {  658,  855,     360, 10, }, // America/St_Thomas
    {  658, 5830,     370, 11, }, // America/St_Vincent
    {  658, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  658, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  658, 1152,     413,  9, }, // Asia/Kathmandu
    {  658,  992,     422,  7, }, // Asia/Kolkata
    {  658, 1178,     429,  5, }, // Asia/Macau
    {  658, 1202,     442,  6, }, // Asia/Yangon
    {  658, 1379,     448,  5, }, // Atlantic/Faroe
    {  658, 6784,     453, 10, }, // Atlantic/St_Helena
    {  658, 2419,     463,  4, }, // Europe/Kyiv
    {  658, 7174,     467,  7, }, // Indian/Reunion
    {  658, 3022,     474,  5, }, // Pacific/Chuuk
    {  658, 2961,     479,  7, }, // Pacific/Pohnpei
    {  659,   14,       0,  6, }, // Africa/Asmara Obolo/Latin/Nigeria
    {  659, 4337,       6,  8, }, // Africa/Sao_Tome
    {  659, 4154,      14,  8, }, // America/Asuncion
    {  659,  237,      22,  8, }, // America/Atikokan
    {  659, 7586,      30, 17, }, // America/Bahia_Banderas
    {  659, 3770,      47,  6, }, // America/Cancun
    {  659, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  659, 5640,      66,  7, }, // America/Curacao
    {  659,  481,      73, 13, }, // America/Indiana/Knox
    {  659, 7924,      86, 16, }, // America/Indiana/Marengo
    {  659, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  659, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  659, 8001,     139, 14, }, // America/Indiana/Vevay
    {  659, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  659, 8049,     171, 16, }, // America/Indiana/Winamac
    {  659, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  659, 5849,     207, 22, }, // America/Lower_Princes
    {  659, 8183,     229,  6, }, // America/Merida
    {  659, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  659, 8292,     255, 20, }, // America/North_Dakota/Center
    {  659, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  659,  393,     298,  4, }, // America/Nuuk
    {  659, 6862,     302, 16, }, // America/Scoresbysund
    {  659, 8471,     318, 14, }, // America/St_Barthelemy
    {  659, 2045,     332, 10, }, // America/St_Johns
    {  659, 5780,     342,  9, }, // America/St_Kitts
    {  659, 5797,     351,  9, }, // America/St_Lucia
    {  659,  855,     360, 10, }, // America/St_Thomas
    {  659, 5830,     370, 11, }, // America/St_Vincent
    {  659, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  659, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  659, 1152,     413,  9, }, // Asia/Kathmandu
    {  659,  992,     422,  7, }, // Asia/Kolkata
    {  659, 1178,     429,  5, }, // Asia/Macau
    {  659, 1202,     442,  6, }, // Asia/Yangon
    {  659, 1379,     448,  5, }, // Atlantic/Faroe
    {  659, 6784,     453, 10, }, // Atlantic/St_Helena
    {  659, 2419,     463,  4, }, // Europe/Kyiv
    {  659, 7174,     467,  7, }, // Indian/Reunion
    {  659, 3022,     474,  5, }, // Pacific/Chuuk
    {  659, 2961,     479,  7, }, // Pacific/Pohnpei
    {  660,   14,       0,  6, }, // Africa/Asmara Baluchi/Arabic/Pakistan
    {  660, 4337,       6,  8, }, // Africa/Sao_Tome
    {  660, 4154,      14,  8, }, // America/Asuncion
    {  660,  237,      22,  8, }, // America/Atikokan
    {  660, 7586,      30, 17, }, // America/Bahia_Banderas
    {  660, 3770,      47,  6, }, // America/Cancun
    {  660, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  660, 5640,      66,  7, }, // America/Curacao
    {  660,  481,      73, 13, }, // America/Indiana/Knox
    {  660, 7924,      86, 16, }, // America/Indiana/Marengo
    {  660, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  660, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  660, 8001,     139, 14, }, // America/Indiana/Vevay
    {  660, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  660, 8049,     171, 16, }, // America/Indiana/Winamac
    {  660, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  660, 5849,     207, 22, }, // America/Lower_Princes
    {  660, 8183,     229,  6, }, // America/Merida
    {  660, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  660, 8292,     255, 20, }, // America/North_Dakota/Center
    {  660, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  660,  393,     298,  4, }, // America/Nuuk
    {  660, 6862,     302, 16, }, // America/Scoresbysund
    {  660, 8471,     318, 14, }, // America/St_Barthelemy
    {  660, 2045,     332, 10, }, // America/St_Johns
    {  660, 5780,     342,  9, }, // America/St_Kitts
    {  660, 5797,     351,  9, }, // America/St_Lucia
    {  660,  855,     360, 10, }, // America/St_Thomas
    {  660, 5830,     370, 11, }, // America/St_Vincent
    {  660, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  660, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  660, 1152,     413,  9, }, // Asia/Kathmandu
    {  660,  992,     422,  7, }, // Asia/Kolkata
    {  660, 1178,     429,  5, }, // Asia/Macau
    {  660, 1202,     442,  6, }, // Asia/Yangon
    {  660, 1379,     448,  5, }, // Atlantic/Faroe
    {  660, 6784,     453, 10, }, // Atlantic/St_Helena
    {  660, 2419,     463,  4, }, // Europe/Kyiv
    {  660, 7174,     467,  7, }, // Indian/Reunion
    {  660, 3022,     474,  5, }, // Pacific/Chuuk
    {  660, 2961,     479,  7, }, // Pacific/Pohnpei
    {  662, 4961,    3793, 11, }, // Africa/Addis_Ababa Ligurian/Latin/Italy
    {  662, 7383,  206737,  4, }, // Africa/Algiers
    {  662,   14,       0,  6, }, // Africa/Asmara
    {  662, 7412,  206741,  5, }, // Africa/Ceuta
    {  662, 6832,   18802,  4, }, // Africa/Lome
    {  662, 5015,   42614, 10, }, // Africa/Mogadishu
    {  662, 3755,  206746,  7, }, // Africa/Nairobi
    {  662, 5173,   42632,  9, }, // Africa/Ndjamena
    {  662, 4337,   57267,  8, }, // Africa/Sao_Tome
    {  662, 6503,  206753,  6, }, // Africa/Tunis
    {  662,  270,  206759,  7, }, // America/Argentina/Cordoba
    {  662, 4154,      14,  8, }, // America/Asuncion
    {  662,  237,      22,  8, }, // America/Atikokan
    {  662, 7586,      30, 17, }, // America/Bahia_Banderas
    {  662, 4263,   42665,  6, }, // America/Bogota
    {  662, 3770,      47,  6, }, // America/Cancun
    {  662, 4247,  206766,  6, }, // America/Cayenne
    {  662, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  662, 5324,  140252, 10, }, // America/Costa_Rica
    {  662, 5640,      66,  7, }, // America/Curacao
    {  662, 5687,   18910,  7, }, // America/Grenada
    {  662, 5703,  206772, 10, }, // America/Guadeloupe
    {  662,  481,      73, 13, }, // America/Indiana/Knox
    {  662, 7924,      86, 16, }, // America/Indiana/Marengo
    {  662, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  662, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  662, 8001,     139, 14, }, // America/Indiana/Vevay
    {  662, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  662, 8049,     171, 16, }, // America/Indiana/Winamac
    {  662, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  662, 5849,     207, 22, }, // America/Lower_Princes
    {  662, 5722,   18945,  9, }, // America/Martinique
    {  662, 8183,     229,  6, }, // America/Merida
    {  662, 2811,  206782, 16, }, // America/Mexico_City
    {  662, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  662, 8292,     255, 20, }, // America/North_Dakota/Center
    {  662, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  662,  393,     298,  4, }, // America/Nuuk
    {  662, 1862,  206798,  9, }, // America/Sao_Paulo
    {  662, 6862,     302, 16, }, // America/Scoresbysund
    {  662, 8471,  206807, 11, }, // America/St_Barthelemy
    {  662, 2045,     332, 10, }, // America/St_Johns
    {  662, 5780,  206818, 13, }, // America/St_Kitts
    {  662, 5797,  206831, 11, }, // America/St_Lucia
    {  662,  855,     360, 10, }, // America/St_Thomas
    {  662, 5830,  206842, 11, }, // America/St_Vincent
    {  662, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  662, 5451,    4069,  7, }, // Asia/Bahrain
    {  662, 4503,   91779,  4, }, // Asia/Chita
    {  662, 4442,   72897,  7, }, // Asia/Damascus
    {  662, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  662, 6969,   93786,  8, }, // Asia/Jakarta
    {  662, 1257,   93794, 11, }, // Asia/Jerusalem
    {  662, 1152,     413,  9, }, // Asia/Kathmandu
    {  662,  992,   34533,  8, }, // Asia/Kolkata
    {  662, 1178,     429,  5, }, // Asia/Macau
    {  662, 7208,   19277, 10, }, // Asia/Samarkand
    {  662, 3137,  206853,  8, }, // Asia/Singapore
    {  662, 1346,   25373, 10, }, // Asia/Ulaanbaatar
    {  662, 1202,     442,  6, }, // Asia/Yangon
    {  662, 3785,    6219, 12, }, // Asia/Yekaterinburg
    {  662, 3565,    6238,  5, }, // Atlantic/Azores
    {  662, 6591,  206861,  6, }, // Atlantic/Canary
    {  662, 3629,  206867, 11, }, // Atlantic/Cape_Verde
    {  662, 1379,   93818,  7, }, // Atlantic/Faroe
    {  662, 7235,  206878, 14, }, // Atlantic/South_Georgia
    {  662, 6784,   93840, 10, }, // Atlantic/St_Helena
    {  662, 6149,  140112,  7, }, // Europe/Andorra
    {  662, 6577,   19437,  5, }, // Europe/Athens
    {  662, 6285,  206892,  9, }, // Europe/Belgrade
    {  662, 6178,   56194,  9, }, // Europe/Brussels
    {  662, 2260,  206901,  6, }, // Europe/Dublin
    {  662, 6256,  206907,  7, }, // Europe/Gibraltar
    {  662, 8783,  206914, 11, }, // Europe/Isle_of_Man
    {  662, 2419,     463,  4, }, // Europe/Kyiv
    {  662, 3078,  206925,  7, }, // Europe/Lisbon
    {  662, 6455,  206932,  7, }, // Europe/Ljubljana
    {  662, 2393,   93889,  6, }, // Europe/London
    {  662, 6314,   73033, 10, }, // Europe/Luxembourg
    {  662, 6359,  206939,  6, }, // Europe/Monaco
    {  662, 3443,   93906,  5, }, // Europe/Moscow
    {  662, 6407,  206945,  4, }, // Europe/Oslo
    {  662, 4186,  206949,  7, }, // Europe/Paris
    {  662, 6224,   19576,  5, }, // Europe/Prague
    {  662, 6273,  206956,  5, }, // Europe/Rome
    {  662, 6419,   64261,  9, }, // Europe/San_Marino
    {  662, 6472,   93928,  9, }, // Europe/Stockholm
    {  662, 6135,  206961,  6, }, // Europe/Tirane
    {  662, 3055,   93955,  8, }, // Europe/Warsaw
    {  662, 6210,   93963,  8, }, // Europe/Zagreb
    {  662, 6489,   93971,  6, }, // Europe/Zurich
    {  662, 4931,  206967,  6, }, // Indian/Comoro
    {  662, 7013,    6662,  7, }, // Indian/Maldives
    {  662, 5000,  206973,  7, }, // Indian/Mayotte
    {  662, 7174,     467,  7, }, // Indian/Reunion
    {  662, 3022,     474,  5, }, // Pacific/Chuuk
    {  662, 2201,   43146,  6, }, // Pacific/Easter
    {  662, 3804,   93988,  4, }, // Pacific/Fiji
    {  662, 3953,  206980,  9, }, // Pacific/Marquesas
    {  662, 7061,    4529,  6, }, // Pacific/Noumea
    {  662, 2961,     479,  7, }, // Pacific/Pohnpei
    {  663,   14,       0,  6, }, // Africa/Asmara Rohingya/Hanifi/Myanmar
    {  663, 4337,       6,  8, }, // Africa/Sao_Tome
    {  663, 4154,      14,  8, }, // America/Asuncion
    {  663,  237,      22,  8, }, // America/Atikokan
    {  663, 7586,      30, 17, }, // America/Bahia_Banderas
    {  663, 3770,      47,  6, }, // America/Cancun
    {  663, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  663, 5640,      66,  7, }, // America/Curacao
    {  663,  481,      73, 13, }, // America/Indiana/Knox
    {  663, 7924,      86, 16, }, // America/Indiana/Marengo
    {  663, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  663, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  663, 8001,     139, 14, }, // America/Indiana/Vevay
    {  663, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  663, 8049,     171, 16, }, // America/Indiana/Winamac
    {  663, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  663, 5849,     207, 22, }, // America/Lower_Princes
    {  663, 8183,     229,  6, }, // America/Merida
    {  663, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  663, 8292,     255, 20, }, // America/North_Dakota/Center
    {  663, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  663,  393,     298,  4, }, // America/Nuuk
    {  663, 6862,     302, 16, }, // America/Scoresbysund
    {  663, 8471,     318, 14, }, // America/St_Barthelemy
    {  663, 2045,     332, 10, }, // America/St_Johns
    {  663, 5780,     342,  9, }, // America/St_Kitts
    {  663, 5797,     351,  9, }, // America/St_Lucia
    {  663,  855,     360, 10, }, // America/St_Thomas
    {  663, 5830,     370, 11, }, // America/St_Vincent
    {  663, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  663, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  663, 1152,     413,  9, }, // Asia/Kathmandu
    {  663,  992,     422,  7, }, // Asia/Kolkata
    {  663, 1178,     429,  5, }, // Asia/Macau
    {  663, 1202,     442,  6, }, // Asia/Yangon
    {  663, 1379,     448,  5, }, // Atlantic/Faroe
    {  663, 6784,     453, 10, }, // Atlantic/St_Helena
    {  663, 2419,     463,  4, }, // Europe/Kyiv
    {  663, 7174,     467,  7, }, // Indian/Reunion
    {  663, 3022,     474,  5, }, // Pacific/Chuuk
    {  663, 2961,     479,  7, }, // Pacific/Pohnpei
    {  665, 6751,  189880,  7, }, // Africa/Abidjan Torwali/Arabic/Pakistan
    {  665, 6723,  189887,  5, }, // Africa/Accra
    {  665, 4961,  101682, 10, }, // Africa/Addis_Ababa
    {  665, 7383,  189892,  7, }, // Africa/Algiers
    {  665,   14,  189899,  6, }, // Africa/Asmara
    {  665,   44,  101709,  5, }, // Africa/Bamako
    {  665, 5159,  189905,  6, }, // Africa/Bangui
    {  665, 6709,  189911,  5, }, // Africa/Banjul
    {  665, 7398,  101729,  5, }, // Africa/Bissau
    {  665, 4859,  189916,  8, }, // Africa/Blantyre
    {  665, 5189,  189924,  9, }, // Africa/Brazzaville
    {  665, 4824,  189933,  6, }, // Africa/Bujumbura
    {  665, 2242,  189939,  5, }, // Africa/Cairo
    {  665, 4019,  189944, 11, }, // Africa/Casablanca
    {  665, 7412,  189955,  5, }, // Africa/Ceuta
    {  665, 6736,  189960,  6, }, // Africa/Conakry
    {  665, 6803,  189966,  4, }, // Africa/Dakar
    {  665, 5032,    8974, 10, }, // Africa/Dar_es_Salaam
    {  665, 4945,  189970,  5, }, // Africa/Djibouti
    {  665, 5145,  189975,  5, }, // Africa/Douala
    {  665, 5068,  147167,  6, }, // Africa/El_Aaiun
    {  665, 6816,  189980,  8, }, // Africa/Freetown
    {  665, 4808,  189988,  5, }, // Africa/Gaborone
    {  665, 4917,  189993,  5, }, // Africa/Harare
    {  665, 4381,  101845,  9, }, // Africa/Johannesburg
    {  665, 4401,    9030,  4, }, // Africa/Juba
    {  665, 5053,  189998,  7, }, // Africa/Kampala
    {  665, 4426,    9043,  5, }, // Africa/Khartoum
    {  665, 4875,  190005,  5, }, // Africa/Kigali
    {  665, 5208,  190010,  6, }, // Africa/Kinshasa
    {  665, 4646,  144547,  5, }, // Africa/Lagos
    {  665, 5238,  190016,  9, }, // Africa/Libreville
    {  665, 6832,    9073,  3, }, // Africa/Lome
    {  665, 5113,  190025,  6, }, // Africa/Luanda
    {  665, 4841,  160030,  8, }, // Africa/Lubumbashi
    {  665, 4903,  190031,  7, }, // Africa/Lusaka
    {  665, 5224,  160044,  5, }, // Africa/Malabo
    {  665, 4889,  190038,  5, }, // Africa/Maputo
    {  665, 5099,  190043,  5, }, // Africa/Maseru
    {  665, 5084,  190048,  6, }, // Africa/Mbabane
    {  665, 5015,  147273,  8, }, // Africa/Mogadishu
    {  665, 6997,  147281,  8, }, // Africa/Monrovia
    {  665, 3755,  101953,  6, }, // Africa/Nairobi
    {  665, 5173,  190054,  9, }, // Africa/Ndjamena
    {  665, 5256,  147303,  5, }, // Africa/Niamey
    {  665, 6766,  190063,  7, }, // Africa/Nouakchott
    {  665, 6690,  190070,  9, }, // Africa/Ouagadougou
    {  665, 5127,  101989, 10, }, // Africa/Porto-Novo
    {  665, 4337,  190079,  7, }, // Africa/Sao_Tome
    {  665, 2732,  190086,  7, }, // Africa/Tripoli
    {  665, 6503,  190093,  5, }, // Africa/Tunis
    {  665, 4054,  190098,  6, }, // Africa/Windhoek
    {  665,  132,  144670,  4, }, // America/Adak
    {  665, 3183,  144674,  7, }, // America/Anchorage
    {  665, 5524,  190104,  7, }, // America/Anguilla
    {  665, 5541,  190111,  7, }, // America/Antigua
    {  665, 4456,  160141,  9, }, // America/Araguaina
    {  665,  166,  190118, 10, }, // America/Argentina/Buenos_Aires
    {  665,   91,  206989,  9, }, // America/Argentina/Catamarca
    {  665,  270,  190128,  7, }, // America/Argentina/Cordoba
    {  665,  441,  190135,  6, }, // America/Argentina/Jujuy
    {  665, 7425,  160185,  9, }, // America/Argentina/La_Rioja
    {  665,  565,  190141,  7, }, // America/Argentina/Mendoza
    {  665, 7452,  160201, 11, }, // America/Argentina/Rio_Gallegos
    {  665, 7483,  102126,  5, }, // America/Argentina/Salta
    {  665, 7507,  160217,  8, }, // America/Argentina/San_Juan
    {  665, 5497,  190148,  8, }, // America/Argentina/San_Luis
    {  665, 7534,  190156,  7, }, // America/Argentina/Tucuman
    {  665, 7560,  160241,  8, }, // America/Argentina/Ushuaia
    {  665, 5557,  147464,  5, }, // America/Aruba
    {  665, 4154,  190163,  7, }, // America/Asuncion
    {  665,  237,  190170,  7, }, // America/Atikokan
    {  665, 3581,  102200,  5, }, // America/Bahia
    {  665, 7586,  190177, 11, }, // America/Bahia_Banderas
    {  665, 5571,  102212,  8, }, // America/Barbados
    {  665, 7609,  190188,  5, }, // America/Belem
    {  665, 5309,  190193,  6, }, // America/Belize
    {  665, 7623,  190199, 11, }, // America/Blanc-Sablon
    {  665, 7644,  190210,  8, }, // America/Boa_Vista
    {  665, 4263,  190218,  5, }, // America/Bogota
    {  665, 7662,  190223,  5, }, // America/Boise
    {  665, 7676,  206998, 15, }, // America/Cambridge_Bay
    {  665, 7698,  190243, 11, }, // America/Campo_Grande
    {  665, 3770,  190254,  6, }, // America/Cancun
    {  665, 4596,  207013,  6, }, // America/Caracas
    {  665, 4247,  207019,  5, }, // America/Cayenne
    {  665, 5398,  190271,  5, }, // America/Cayman
    {  665, 3251,  190276,  5, }, // America/Chicago
    {  665, 7719,  190281,  8, }, // America/Chihuahua
    {  665, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  665, 5324,  190289, 10, }, // America/Costa_Rica
    {  665, 7759,  190299,  7, }, // America/Creston
    {  665, 3693,  190306,  7, }, // America/Cuiaba
    {  665, 5640,  190313,  8, }, // America/Curacao
    {  665, 7775,  102361, 11, }, // America/Danmarkshavn
    {  665, 7796,  190321,  5, }, // America/Dawson
    {  665, 7811,  190326, 10, }, // America/Dawson_Creek
    {  665,  805,  190336,  5, }, // America/Denver
    {  665, 3351,  190341,  7, }, // America/Detroit
    {  665, 5656,  190348,  7, }, // America/Dominica
    {  665,  893,  190355,  8, }, // America/Edmonton
    {  665, 7832,  190363,  7, }, // America/Eirunepe
    {  665, 5343,  190370, 10, }, // America/El_Salvador
    {  665, 7849,  102441, 10, }, // America/Fort_Nelson
    {  665, 7869,  190380,  9, }, // America/Fortaleza
    {  665, 7887,  207024, 13, }, // America/Glace_Bay
    {  665, 6844,  207037, 12, }, // America/Goose_Bay
    {  665, 4514,  190414,  8, }, // America/Grand_Turk
    {  665, 5687,  190422,  6, }, // America/Grenada
    {  665, 5703,  190428,  8, }, // America/Guadeloupe
    {  665, 3662,  190436, 10, }, // America/Guatemala
    {  665, 4743,  190446,  9, }, // America/Guayaquil
    {  665, 6895,  190455,  5, }, // America/Guyana
    {  665, 1923,  190460,  8, }, // America/Halifax
    {  665, 2221,  190468,  5, }, // America/Havana
    {  665, 7905,  190473,  8, }, // America/Hermosillo
    {  665,  348,  190481, 11, }, // America/Indiana/Indianapolis
    {  665,  481,  190492, 14, }, // America/Indiana/Knox
    {  665, 7924,  190506, 15, }, // America/Indiana/Marengo
    {  665, 7948,  190521, 17, }, // America/Indiana/Petersburg
    {  665, 7975,  190538, 16, }, // America/Indiana/Tell_City
    {  665, 8001,  190554, 13, }, // America/Indiana/Vevay
    {  665, 8023,  190567, 16, }, // America/Indiana/Vincennes
    {  665, 8049,  190583, 16, }, // America/Indiana/Winamac
    {  665, 8073,  190599,  6, }, // America/Inuvik
    {  665,  660,  190605,  8, }, // America/Iqaluit
    {  665, 2665,  190613,  7, }, // America/Jamaica
    {  665, 5282,  190620,  6, }, // America/Juneau
    {  665,  521,  190626,  9, }, // America/Kentucky/Louisville
    {  665, 8088,  190635, 18, }, // America/Kentucky/Monticello
    {  665, 5621,  190653, 10, }, // America/Kralendijk
    {  665, 4278,  102731,  6, }, // America/La_Paz
    {  665, 7132,  145276,  4, }, // America/Lima
    {  665, 3390,  145280, 10, }, // America/Los_Angeles
    {  665, 5849,  190663, 17, }, // America/Lower_Princes
    {  665, 8116,  160796,  6, }, // America/Maceio
    {  665, 8131,  190680,  6, }, // America/Managua
    {  665, 1892,  190686,  5, }, // America/Manaus
    {  665, 5814,  102787,  7, }, // America/Marigot
    {  665, 5722,  190691,  7, }, // America/Martinique
    {  665, 8147,  102802,  9, }, // America/Matamoros
    {  665, 2779,  190698,  7, }, // America/Mazatlan
    {  665, 8165,  190705,  8, }, // America/Menominee
    {  665, 8183,  190713,  6, }, // America/Merida
    {  665, 8198,  102833, 11, }, // America/Metlakatla
    {  665, 2811,  190719, 11, }, // America/Mexico_City
    {  665, 4293,  190730,  8, }, // America/Miquelon
    {  665, 8217,  190738,  6, }, // America/Moncton
    {  665, 8233,  190744,  7, }, // America/Monterrey
    {  665, 4000,  190751, 11, }, // America/Montevideo
    {  665, 5741,  190762,  9, }, // America/Montserrat
    {  665, 5383,  102895,  4, }, // America/Nassau
    {  665, 3294,  190771,  8, }, // America/New_York
    {  665, 8251,    9897,  3, }, // America/Nome
    {  665, 1834,  102908,  7, }, // America/Noronha
    {  665, 8264,  190779, 19, }, // America/North_Dakota/Beulah
    {  665, 8292,  190798, 16, }, // America/North_Dakota/Center
    {  665, 8320,  190814, 21, }, // America/North_Dakota/New_Salem
    {  665,  393,  102562,  3, }, // America/Nuuk
    {  665, 8351,  190835,  8, }, // America/Ojinaga
    {  665, 5413,  190843,  5, }, // America/Panama
    {  665, 6106,  190848,  9, }, // America/Paramaribo
    {  665, 3224,  145527,  5, }, // America/Phoenix
    {  665, 3847,  190857, 12, }, // America/Port-au-Prince
    {  665, 5871,  190869, 13, }, // America/Port_of_Spain
    {  665, 8367,  190882, 11, }, // America/Porto_Velho
    {  665, 5760,  190893, 11, }, // America/Puerto_Rico
    {  665, 3932,  190904, 11, }, // America/Punta_Arenas
    {  665, 8387,  190915, 11, }, // America/Rankin_Inlet
    {  665, 8408,  190926,  6, }, // America/Recife
    {  665, 1979,  161087,  6, }, // America/Regina
    {  665, 8423,  190932,  8, }, // America/Resolute
    {  665,  695,  190940, 11, }, // America/Rio_Branco
    {  665, 8440,  190951,  7, }, // America/Santarem
    {  665, 2165,  190958,  7, }, // America/Santiago
    {  665, 6044,  190965, 12, }, // America/Santo_Domingo
    {  665, 1862,  103145,  8, }, // America/Sao_Paulo
    {  665, 6862,  190977, 15, }, // America/Scoresbysund
    {  665, 8457,  190992,  5, }, // America/Sitka
    {  665, 8471,  190997, 13, }, // America/St_Barthelemy
    {  665, 2045,  191010,  9, }, // America/St_Johns
    {  665, 5780,  191019,  8, }, // America/St_Kitts
    {  665, 5797,  191027, 10, }, // America/St_Lucia
    {  665,  855,  191037, 10, }, // America/St_Thomas
    {  665, 5830,  191047, 10, }, // America/St_Vincent
    {  665, 8493,  191057, 10, }, // America/Swift_Current
    {  665, 5363,  191067, 11, }, // America/Tegucigalpa
    {  665, 5673,  191078,  5, }, // America/Thule
    {  665,  313,  191083,  7, }, // America/Tijuana
    {  665,  608,  191090,  6, }, // America/Toronto
    {  665, 5605,  191096,  7, }, // America/Tortola
    {  665, 2077,  145780,  7, }, // America/Vancouver
    {  665, 2128,  191103, 10, }, // America/Whitehorse
    {  665,  734,  191113,  7, }, // America/Winnipeg
    {  665, 8515,  191120,  6, }, // America/Yakutat
    {  665, 5918,  103325,  4, }, // Antarctica/Casey
    {  665, 6027,  191126,  4, }, // Antarctica/Davis
    {  665, 6066,  191130, 16, }, // Antarctica/DumontDUrville
    {  665, 8531,  191146,  7, }, // Antarctica/Macquarie
    {  665, 7029,  191153,  5, }, // Antarctica/Mawson
    {  665, 7076,  191158,  8, }, // Antarctica/McMurdo
    {  665, 8552,  191166,  5, }, // Antarctica/Palmer
    {  665, 7189,  103367,  7, }, // Antarctica/Rothera
    {  665, 7258,  103374,  5, }, // Antarctica/Syowa
    {  665, 8570,  191171,  4, }, // Antarctica/Troll
    {  665, 7337,  103379,  6, }, // Antarctica/Vostok
    {  665, 1413,  191175, 13, }, // Arctic/Longyearbyen
    {  665, 5487,   10423,  3, }, // Asia/Aden
    {  665, 5297,  191188,  6, }, // Asia/Almaty
    {  665, 3870,  191194,  5, }, // Asia/Amman
    {  665, 5428,  191199,  5, }, // Asia/Anadyr
    {  665, 5440,  191204,  5, }, // Asia/Aqtau
    {  665, 5270,  191209,  5, }, // Asia/Aqtobe
    {  665,  964,  191214,  5, }, // Asia/Ashgabat
    {  665, 8587,  191219,  6, }, // Asia/Atyrau
    {  665, 3509,   10466,  5, }, // Asia/Baghdad
    {  665, 5451,  145924,  5, }, // Asia/Bahrain
    {  665, 3555,  144949,  4, }, // Asia/Baku
    {  665, 4368,  145929,  5, }, // Asia/Bangkok
    {  665, 3473,  191225,  5, }, // Asia/Barnaul
    {  665, 3988,  145934,  5, }, // Asia/Beirut
    {  665, 3680,  145939,  5, }, // Asia/Bishkek
    {  665, 5906,  148551,  6, }, // Asia/Brunei
    {  665, 4503,  148557,  4, }, // Asia/Chita
    {  665, 5963,  191230,  9, }, // Asia/Choibalsan
    {  665, 4413,  145962,  6, }, // Asia/Colombo
    {  665, 4442,   10528,  4, }, // Asia/Damascus
    {  665, 1060,  191239,  5, }, // Asia/Dhaka
    {  665, 6125,  191244,  3, }, // Asia/Dili
    {  665, 3498,  191247,  4, }, // Asia/Dubai
    {  665, 6092,  191251,  7, }, // Asia/Dushanbe
    {  665, 8599,  191258,  9, }, // Asia/Famagusta
    {  665, 8614,  103548,  3, }, // Asia/Gaza
    {  665, 4697,  103551,  6, }, // Asia/Hebron
    {  665, 1226,  191267, 13, }, // Asia/Ho_Chi_Minh
    {  665, 2576,  207049,  9, }, // Asia/Hong_Kong
    {  665, 4673,  191280,  6, }, // Asia/Hovd
    {  665, 4086,  191286,  6, }, // Asia/Irkutsk
    {  665, 6969,  191292,  6, }, // Asia/Jakarta
    {  665, 6955,  191298,  7, }, // Asia/Jayapura
    {  665, 1257,  103597,  6, }, // Asia/Jerusalem
    {  665, 3462,  207058,  4, }, // Asia/Kabul
    {  665, 4218,  191305,  7, }, // Asia/Kamchatka
    {  665, 4141,  148657,  5, }, // Asia/Karachi
    {  665, 1152,  207062,  8, }, // Asia/Kathmandu
    {  665, 8624,  191320,  6, }, // Asia/Khandyga
    {  665,  992,  207070,  7, }, // Asia/Kolkata
    {  665, 4099,  191333, 11, }, // Asia/Krasnoyarsk
    {  665, 4761,  191344, 11, }, // Asia/Kuala_Lumpur
    {  665, 5893,  191355,  6, }, // Asia/Kuching
    {  665, 5464,  148710,  4, }, // Asia/Kuwait
    {  665, 1178,  207077,  4, }, // Asia/Macau
    {  665, 3919,  191365,  6, }, // Asia/Magadan
    {  665, 1316,  191371,  5, }, // Asia/Makassar
    {  665, 7145,  146127,  5, }, // Asia/Manila
    {  665, 6883,   10734,  4, }, // Asia/Muscat
    {  665, 2446,  191376,  6, }, // Asia/Nicosia
    {  665, 8638,  191382, 12, }, // Asia/Novokuznetsk
    {  665, 4037,  103725, 11, }, // Asia/Novosibirsk
    {  665, 4131,  103736,  5, }, // Asia/Omsk
    {  665, 7108,  148768,  5, }, // Asia/Oral
    {  665, 6924,  146162,  7, }, // Asia/Phnom_Penh
    {  665, 8656,  191394,  8, }, // Asia/Pontianak
    {  665, 4116,  191402, 10, }, // Asia/Pyongyang
    {  665, 5476,   10802,  3, }, // Asia/Qatar
    {  665, 8671,  191412,  7, }, // Asia/Qostanay
    {  665, 4171,  191419,  9, }, // Asia/Qyzylorda
    {  665, 3486,  103788,  4, }, // Asia/Riyadh
    {  665, 4310,  191428,  6, }, // Asia/Sakhalin
    {  665, 7208,   10834,  6, }, // Asia/Samarkand
    {  665, 3116,  191434,  5, }, // Asia/Seoul
    {  665, 1020,  146210,  7, }, // Asia/Shanghai
    {  665, 3137,  148836,  7, }, // Asia/Singapore
    {  665, 4199,  191439, 13, }, // Asia/Srednekolymsk
    {  665, 3100,  191452,  5, }, // Asia/Taipei
    {  665, 4683,  146243,  6, }, // Asia/Tashkent
    {  665, 3817,  191457,  6, }, // Asia/Tbilisi
    {  665, 2638,  191463,  5, }, // Asia/Tehran
    {  665, 1284,  191468,  5, }, // Asia/Thimphu
    {  665, 2687,  103871,  5, }, // Asia/Tokyo
    {  665, 4474,  191473,  5, }, // Asia/Tomsk
    {  665, 1346,  191478, 11, }, // Asia/Ulaanbaatar
    {  665, 1126,  191489,  7, }, // Asia/Urumqi
    {  665, 8685,  191496,  9, }, // Asia/Ust-Nera
    {  665, 6940,  191505,  8, }, // Asia/Vientiane
    {  665, 4612,  191513, 11, }, // Asia/Vladivostok
    {  665, 4730,  191524,  6, }, // Asia/Yakutsk
    {  665, 1202,  161769,  5, }, // Asia/Yangon
    {  665, 3785,  191530, 11, }, // Asia/Yekaterinburg
    {  665, 3649,  191541,  6, }, // Asia/Yerevan
    {  665, 3565,  191547,  6, }, // Atlantic/Azores
    {  665, 5588,   10981,  6, }, // Atlantic/Bermuda
    {  665, 6591,  148240,  5, }, // Atlantic/Canary
    {  665, 3629,  191553,  8, }, // Atlantic/Cape_Verde
    {  665, 1379,   11004,  4, }, // Atlantic/Faroe
    {  665, 8699,  191561,  6, }, // Atlantic/Madeira
    {  665, 2614,  191567,  7, }, // Atlantic/Reykjavik
    {  665, 7235,  104004, 12, }, // Atlantic/South_Georgia
    {  665, 6784,  191574, 11, }, // Atlantic/St_Helena
    {  665, 6607,  191585,  7, }, // Atlantic/Stanley
    {  665, 1654,  207081,  7, }, // Australia/Adelaide
    {  665, 1619,  104040,  6, }, // Australia/Brisbane
    {  665, 1783,  191599,  9, }, // Australia/Broken_Hill
    {  665, 1567,  191608,  5, }, // Australia/Darwin
    {  665, 3539,  191613,  6, }, // Australia/Eucla
    {  665, 1500,  191619,  6, }, // Australia/Hobart
    {  665, 8716,  191625,  7, }, // Australia/Lindeman
    {  665, 1531,  191632,  9, }, // Australia/Lord_Howe
    {  665, 1711,  149061,  6, }, // Australia/Melbourne
    {  665, 1746,  191641,  4, }, // Australia/Perth
    {  665, 1447,  191645,  4, }, // Australia/Sydney
    {  665, 6390,  191649,  8, }, // Europe/Amsterdam
    {  665, 6149,  191657,  6, }, // Europe/Andorra
    {  665, 3522,  191663,  8, }, // Europe/Astrakhan
    {  665, 6577,  191671,  6, }, // Europe/Athens
    {  665, 6285,   11151,  6, }, // Europe/Belgrade
    {  665, 4659,  146501,  4, }, // Europe/Berlin
    {  665, 6437,  191677, 10, }, // Europe/Bratislava
    {  665, 6178,  161927,  5, }, // Europe/Brussels
    {  665, 3830,  191687,  6, }, // Europe/Bucharest
    {  665, 3708,  191693,  6, }, // Europe/Budapest
    {  665, 8751,  191699,  5, }, // Europe/Busingen
    {  665, 2475,  191704,  7, }, // Europe/Chisinau
    {  665, 6238,  104203,  9, }, // Europe/Copenhagen
    {  665, 2260,  191711,  4, }, // Europe/Dublin
    {  665, 6256,  146558, 10, }, // Europe/Gibraltar
    {  665, 8767,  149166,  5, }, // Europe/Guernsey
    {  665, 6561,  191715,  7, }, // Europe/Helsinki
    {  665, 8783,  104242, 10, }, // Europe/Isle_of_Man
    {  665, 1097,  146592,  7, }, // Europe/Istanbul
    {  665, 8802,  104261,  4, }, // Europe/Jersey
    {  665, 3881,  191722, 11, }, // Europe/Kaliningrad
    {  665, 8924,  146609,  5, }, // Europe/Kirov
    {  665, 2419,  101775,  3, }, // Europe/Kyiv
    {  665, 3078,  162026,  4, }, // Europe/Lisbon
    {  665, 6455,  149210,  9, }, // Europe/Ljubljana
    {  665, 2393,   11292,  4, }, // Europe/London
    {  665, 6314,  191733,  7, }, // Europe/Luxembourg
    {  665, 4794,  191740,  5, }, // Europe/Madrid
    {  665, 6346,  104326,  5, }, // Europe/Malta
    {  665, 6531,  191745,  7, }, // Europe/Mariehamn
    {  665, 3595,  104339,  5, }, // Europe/Minsk
    {  665, 6359,  104344,  6, }, // Europe/Monaco
    {  665, 3443,  104350,  5, }, // Europe/Moscow
    {  665, 6407,  104355,  5, }, // Europe/Oslo
    {  665, 4186,  191752,  4, }, // Europe/Paris
    {  665, 6373,  191756,  9, }, // Europe/Podgorica
    {  665, 6224,  104374,  4, }, // Europe/Prague
    {  665, 8816,  147993,  4, }, // Europe/Riga
    {  665, 6273,    9113,  3, }, // Europe/Rome
    {  665, 4233,  146680,  5, }, // Europe/Samara
    {  665, 6419,  191765, 11, }, // Europe/San_Marino
    {  665, 6194,  162095,  7, }, // Europe/Sarajevo
    {  665, 4353,  191776,  6, }, // Europe/Saratov
    {  665, 8828,  162107,  8, }, // Europe/Simferopol
    {  665, 6332,  191782,  7, }, // Europe/Skopje
    {  665, 6548,  191789,  5, }, // Europe/Sofia
    {  665, 6472,  191794,  9, }, // Europe/Stockholm
    {  665, 8846,  191803,  4, }, // Europe/Tallinn
    {  665, 6135,  191807,  6, }, // Europe/Tirane
    {  665, 8861,  191813,  8, }, // Europe/Ulyanovsk
    {  665, 6301,  191821,  5, }, // Europe/Vaduz
    {  665, 6516,  191826,  6, }, // Europe/Vatican
    {  665, 6164,  146754,  5, }, // Europe/Vienna
    {  665, 8878,  191832,  7, }, // Europe/Vilnius
    {  665, 4629,  162168,  9, }, // Europe/Volgograd
    {  665, 3055,  104500,  5, }, // Europe/Warsaw
    {  665, 6210,  191839,  6, }, // Europe/Zagreb
    {  665, 6489,  191845,  5, }, // Europe/Zurich
    {  665, 4980,  191850, 12, }, // Indian/Antananarivo
    {  665, 6910,  104528,  5, }, // Indian/Chagos
    {  665, 5979,  191862,  5, }, // Indian/Christmas
    {  665, 5996,  104540,  5, }, // Indian/Cocos
    {  665, 4931,  104545,  6, }, // Indian/Comoro
    {  665, 6624,  191867,  8, }, // Indian/Kerguelen
    {  665, 7223,  104559,  4, }, // Indian/Mahe
    {  665, 7013,  191875,  6, }, // Indian/Maldives
    {  665, 3971,  191881,  6, }, // Indian/Mauritius
    {  665, 5000,  191887,  5, }, // Indian/Mayotte
    {  665, 7174,  191892,  8, }, // Indian/Reunion
    {  665, 4324,  146843,  4, }, // Pacific/Apia
    {  665,  932,  104597,  6, }, // Pacific/Auckland
    {  665, 3608,  191900,  8, }, // Pacific/Bougainville
    {  665, 2863,  191908,  5, }, // Pacific/Chatham
    {  665, 3022,  191913,  4, }, // Pacific/Chuuk
    {  665, 2201,  104125,  5, }, // Pacific/Easter
    {  665, 7323,  191917,  5, }, // Pacific/Efate
    {  665, 7290,  191922,  6, }, // Pacific/Fakaofo
    {  665, 3804,  191928,  3, }, // Pacific/Fiji
    {  665, 7306,  191931, 10, }, // Pacific/Funafuti
    {  665, 6641,  191941,  9, }, // Pacific/Galapagos
    {  665, 6659,  191950,  7, }, // Pacific/Gambier
    {  665, 3724,  191957, 11, }, // Pacific/Guadalcanal
    {  665, 5935,  190683,  4, }, // Pacific/Guam
    {  665, 2897,  191976,  8, }, // Pacific/Kanton
    {  665, 3900,  191984,  9, }, // Pacific/Kiritimati
    {  665, 6982,  191993,  7, }, // Pacific/Kosrae
    {  665, 2708,  149536,  8, }, // Pacific/Kwajalein
    {  665, 4779,  146946,  5, }, // Pacific/Majuro
    {  665, 3953,  192000,  8, }, // Pacific/Marquesas
    {  665, 8909,  192008,  4, }, // Pacific/Midway
    {  665, 7047,  192012,  5, }, // Pacific/Nauru
    {  665, 7095,  190622,  4, }, // Pacific/Niue
    {  665, 4070,  104742,  6, }, // Pacific/Norfolk
    {  665, 7061,  192017,  5, }, // Pacific/Noumea
    {  665, 2991,  162377,  9, }, // Pacific/Pago_Pago
    {  665, 7118,  192022,  4, }, // Pacific/Palau
    {  665, 7157,  192026,  7, }, // Pacific/Pitcairn
    {  665, 2961,  192033,  7, }, // Pacific/Pohnpei
    {  665, 4709,  192040, 12, }, // Pacific/Port_Moresby
    {  665, 6009,  104791,  9, }, // Pacific/Rarotonga
    {  665, 5948,  192052,  6, }, // Pacific/Saipan
    {  665, 7275,  192058,  6, }, // Pacific/Tahiti
    {  665, 6675,  192064,  5, }, // Pacific/Tarawa
    {  665, 4485,  192069,  9, }, // Pacific/Tongatapu
    {  665, 7355,  104828,  3, }, // Pacific/Wake
    {  665, 7368,  192078,  5, }, // Pacific/Wallis
    {  666, 6751,  207088,  6, }, // Africa/Abidjan Anii/Latin/Benin
    {  666, 6723,  207094,  5, }, // Africa/Accra
    {  666, 4961,  207099, 10, }, // Africa/Addis_Ababa
    {  666, 7383,  207109,  6, }, // Africa/Algiers
    {  666,   14,  207115,  7, }, // Africa/Asmara
    {  666,   44,  207122,  7, }, // Africa/Bamako
    {  666, 5159,  207129,  6, }, // Africa/Bangui
    {  666, 7398,  207135,  5, }, // Africa/Bissau
    {  666, 4859,  207140,  8, }, // Africa/Blantyre
    {  666, 5189,  207148,  8, }, // Africa/Brazzaville
    {  666, 4824,  207156, 10, }, // Africa/Bujumbura
    {  666, 2242,  207166,  6, }, // Africa/Cairo
    {  666, 4019,  207172, 11, }, // Africa/Casablanca
    {  666, 7412,  207183,  6, }, // Africa/Ceuta
    {  666, 6736,  207189,  8, }, // Africa/Conakry
    {  666, 6803,  207197,  6, }, // Africa/Dakar
    {  666, 5032,  207203, 10, }, // Africa/Dar_es_Salaam
    {  666, 4945,  207213,  7, }, // Africa/Djibouti
    {  666, 5145,  207220,  7, }, // Africa/Douala
    {  666, 5068,  207227,  7, }, // Africa/El_Aaiun
    {  666, 6816,  207234,  8, }, // Africa/Freetown
    {  666, 4917,  207242,  7, }, // Africa/Harare
    {  666, 4381,  207249, 11, }, // Africa/Johannesburg
    {  666, 4401,  207260,  5, }, // Africa/Juba
    {  666, 5053,  207265,  8, }, // Africa/Kampala
    {  666, 4426,  207273,  7, }, // Africa/Khartoum
    {  666, 4875,  196358,  7, }, // Africa/Kigali
    {  666, 5208,  207280,  9, }, // Africa/Kinshasa
    {  666, 4646,  207289,  5, }, // Africa/Lagos
    {  666, 5238,  207294,  8, }, // Africa/Libreville
    {  666, 6832,  207302,  4, }, // Africa/Lome
    {  666, 5113,  207306,  8, }, // Africa/Luanda
    {  666, 4841,  196409, 11, }, // Africa/Lubumbashi
    {  666, 4903,  207314,  7, }, // Africa/Lusaka
    {  666, 5224,  207321,  7, }, // Africa/Malabo
    {  666, 4889,  207328,  7, }, // Africa/Maputo
    {  666, 5099,  207335,  7, }, // Africa/Maseru
    {  666, 5084,  207342,  8, }, // Africa/Mbabane
    {  666, 5015,  207350, 10, }, // Africa/Mogadishu
    {  666, 6997,  207360, 10, }, // Africa/Monrovia
    {  666, 3755,  196478,  8, }, // Africa/Nairobi
    {  666, 5173,  207370,  8, }, // Africa/Ndjamena
    {  666, 5256,  207378,  5, }, // Africa/Niamey
    {  666, 6766,  207383,  8, }, // Africa/Nouakchott
    {  666, 6690,  207391,  8, }, // Africa/Ouagadougou
    {  666, 5127,  207399, 11, }, // Africa/Porto-Novo
    {  666, 4337,  207410, 13, }, // Africa/Sao_Tome
    {  666, 2732,  196552,  8, }, // Africa/Tripoli
    {  666, 6503,  207423,  6, }, // Africa/Tunis
    {  666, 4054,  207429,  7, }, // Africa/Windhoek
    {  666,  132,  207436,  4, }, // America/Adak
    {  666, 3183,  207440,  9, }, // America/Anchorage
    {  666, 5524,  207449,  9, }, // America/Anguilla
    {  666, 5541,  207458,  9, }, // America/Antigua
    {  666, 4456,  207467, 10, }, // America/Araguaina
    {  666,  166,  207477, 13, }, // America/Argentina/Buenos_Aires
    {  666,   91,  207490, 10, }, // America/Argentina/Catamarca
    {  666,  270,  207500,  8, }, // America/Argentina/Cordoba
    {  666,  441,  207508,  6, }, // America/Argentina/Jujuy
    {  666, 7425,  207514,  9, }, // America/Argentina/La_Rioja
    {  666,  565,  207523,  8, }, // America/Argentina/Mendoza
    {  666, 7452,  207531, 12, }, // America/Argentina/Rio_Gallegos
    {  666, 7507,  207543,  9, }, // America/Argentina/San_Juan
    {  666, 5497,  167119,  9, }, // America/Argentina/San_Luis
    {  666, 7534,    4850,  7, }, // America/Argentina/Tucuman
    {  666, 7560,  207552,  9, }, // America/Argentina/Ushuaia
    {  666, 5557,  207561,  6, }, // America/Aruba
    {  666, 4154,  207567,  9, }, // America/Asuncion
    {  666,  237,      22,  8, }, // America/Atikokan
    {  666, 3581,  207576,  7, }, // America/Bahia
    {  666, 7586,  207583, 21, }, // America/Bahia_Banderas
    {  666, 5571,  207604, 10, }, // America/Barbados
    {  666, 7609,  207614,  5, }, // America/Belem
    {  666, 5309,  167184,  6, }, // America/Belize
    {  666, 7623,  207619, 11, }, // America/Blanc-Sablon
    {  666, 7644,  207630, 11, }, // America/Boa_Vista
    {  666, 7662,  207641,  7, }, // America/Boise
    {  666, 7676,  207648, 13, }, // America/Cambridge_Bay
    {  666, 7698,  207661, 13, }, // America/Campo_Grande
    {  666, 3770,  207674,  6, }, // America/Cancun
    {  666, 4596,  167249,  8, }, // America/Caracas
    {  666, 4247,  207680,  6, }, // America/Cayenne
    {  666, 5398,  207686,  8, }, // America/Cayman
    {  666, 3251,  207694,  8, }, // America/Chicago
    {  666, 7719,  207702,  8, }, // America/Chihuahua
    {  666, 7737,  207710, 16, }, // America/Ciudad_Juarez
    {  666, 5324,  207726, 11, }, // America/Costa_Rica
    {  666, 7759,  207737,  7, }, // America/Creston
    {  666, 3693,  207744,  7, }, // America/Cuiaba
    {  666, 5640,  207751,  9, }, // America/Curacao
    {  666, 7775,  207760, 13, }, // America/Danmarkshavn
    {  666, 7796,  207773,  6, }, // America/Dawson
    {  666, 7811,  207779, 12, }, // America/Dawson_Creek
    {  666,  805,  207791,  5, }, // America/Denver
    {  666, 3351,  207796,  7, }, // America/Detroit
    {  666, 5656,  207803,  9, }, // America/Dominica
    {  666,  893,  207812,  8, }, // America/Edmonton
    {  666, 7832,  207820,  9, }, // America/Eirunepe
    {  666, 5343,  207829, 12, }, // America/El_Salvador
    {  666, 7849,  207841, 11, }, // America/Fort_Nelson
    {  666, 7869,  207852, 10, }, // America/Fortaleza
    {  666, 7887,  207862,  9, }, // America/Glace_Bay
    {  666, 6844,  207871,  8, }, // America/Goose_Bay
    {  666, 4514,  207879, 10, }, // America/Grand_Turk
    {  666, 5687,  207889,  8, }, // America/Grenada
    {  666, 5703,  207897, 11, }, // America/Guadeloupe
    {  666, 3662,  207908, 11, }, // America/Guatemala
    {  666, 4743,  207919,  9, }, // America/Guayaquil
    {  666, 6895,  207928,  7, }, // America/Guyana
    {  666, 1923,  207935,  8, }, // America/Halifax
    {  666, 2221,  207943,  7, }, // America/Havana
    {  666, 7905,  207950,  9, }, // America/Hermosillo
    {  666,  348,  207959, 13, }, // America/Indiana/Indianapolis
    {  666,  481,  207972, 14, }, // America/Indiana/Knox
    {  666, 7924,  207986, 18, }, // America/Indiana/Marengo
    {  666, 7948,  208004, 21, }, // America/Indiana/Petersburg
    {  666, 7975,  208025, 19, }, // America/Indiana/Tell_City
    {  666, 8001,  208044, 16, }, // America/Indiana/Vevay
    {  666, 8023,  208060, 17, }, // America/Indiana/Vincennes
    {  666, 8049,  208077, 19, }, // America/Indiana/Winamac
    {  666, 8073,  208096,  6, }, // America/Inuvik
    {  666,  660,  208102,  9, }, // America/Iqaluit
    {  666, 2665,  208111,  7, }, // America/Jamaica
    {  666, 5282,  208118,  5, }, // America/Juneau
    {  666,  521,  208123,  8, }, // America/Kentucky/Louisville
    {  666, 8088,  208131, 20, }, // America/Kentucky/Monticello
    {  666, 5621,  208151, 10, }, // America/Kralendijk
    {  666, 4278,   20278,  6, }, // America/La_Paz
    {  666, 7132,  208161,  5, }, // America/Lima
    {  666, 3390,  208166, 12, }, // America/Los_Angeles
    {  666, 5849,  208178, 17, }, // America/Lower_Princes
    {  666, 8116,  208195,  8, }, // America/Maceio
    {  666, 8131,  167795,  9, }, // America/Managua
    {  666, 1892,  208203,  6, }, // America/Manaus
    {  666, 5814,  208209,  7, }, // America/Marigot
    {  666, 5722,  208216, 10, }, // America/Martinique
    {  666, 8147,  208226, 10, }, // America/Matamoros
    {  666, 2779,  208236,  9, }, // America/Mazatlan
    {  666, 8183,  208245,  7, }, // America/Merida
    {  666, 8198,  208252, 13, }, // America/Metlakatla
    {  666, 2811,  208265, 11, }, // America/Mexico_City
    {  666, 4293,  208276,  7, }, // America/Miquelon
    {  666, 8217,  208283,  6, }, // America/Moncton
    {  666, 8233,  208289,  8, }, // America/Monterrey
    {  666, 4000,  208297, 12, }, // America/Montevideo
    {  666, 5741,  208309, 10, }, // America/Montserrat
    {  666, 5383,  121439,  5, }, // America/Nassau
    {  666, 3294,  208319,  8, }, // America/New_York
    {  666, 1834,  208327,  7, }, // America/Noronha
    {  666, 8264,  208334, 19, }, // America/North_Dakota/Beulah
    {  666, 8292,  208353, 19, }, // America/North_Dakota/Center
    {  666, 8320,  208372, 22, }, // America/North_Dakota/New_Salem
    {  666,  393,     298,  4, }, // America/Nuuk
    {  666, 8351,  168025,  8, }, // America/Ojinaga
    {  666, 5413,  208394,  7, }, // America/Panama
    {  666, 6106,  208401, 11, }, // America/Paramaribo
    {  666, 3224,  208412,  6, }, // America/Phoenix
    {  666, 3847,  208418, 12, }, // America/Port-au-Prince
    {  666, 5871,  208430, 13, }, // America/Port_of_Spain
    {  666, 8367,  208443, 11, }, // America/Porto_Velho
    {  666, 5760,  208454, 11, }, // America/Puerto_Rico
    {  666, 8387,  208465, 14, }, // America/Rankin_Inlet
    {  666, 8408,  158652,  6, }, // America/Recife
    {  666, 1979,  208479,  7, }, // America/Regina
    {  666, 8423,  208486,  8, }, // America/Resolute
    {  666,  695,  208494, 12, }, // America/Rio_Branco
    {  666, 8440,  208506,  8, }, // America/Santarem
    {  666, 2165,  208514, 10, }, // America/Santiago
    {  666, 6044,  208524, 13, }, // America/Santo_Domingo
    {  666, 1862,  208537, 10, }, // America/Sao_Paulo
    {  666, 6862,  208547, 14, }, // America/Scoresbysund
    {  666, 8457,  208561,  6, }, // America/Sitka
    {  666, 8471,  208567, 14, }, // America/St_Barthelemy
    {  666, 2045,  208581,  9, }, // America/St_Johns
    {  666, 5780,  208590,  8, }, // America/St_Kitts
    {  666, 5797,  208598, 10, }, // America/St_Lucia
    {  666,  855,  208608, 11, }, // America/St_Thomas
    {  666, 5830,  208619, 10, }, // America/St_Vincent
    {  666, 8493,  208629, 16, }, // America/Swift_Current
    {  666, 5363,    5759, 11, }, // America/Tegucigalpa
    {  666, 5673,  208645,  5, }, // America/Thule
    {  666,  313,  208650,  8, }, // America/Tijuana
    {  666,  608,  208658,  8, }, // America/Toronto
    {  666, 5605,  208666,  8, }, // America/Tortola
    {  666, 2077,  208674,  8, }, // America/Vancouver
    {  666, 2128,  208682,  8, }, // America/Whitehorse
    {  666,  734,  208690,  8, }, // America/Winnipeg
    {  666, 5918,  208698,  5, }, // Antarctica/Casey
    {  666, 6027,  208703,  5, }, // Antarctica/Davis
    {  666, 6066,  208708, 12, }, // Antarctica/DumontDUrville
    {  666, 8531,  208720,  7, }, // Antarctica/Macquarie
    {  666, 7029,  208727,  5, }, // Antarctica/Mawson
    {  666, 7076,  208732,  9, }, // Antarctica/McMurdo
    {  666, 8552,  208741,  7, }, // Antarctica/Palmer
    {  666, 7189,  208748,  7, }, // Antarctica/Rothera
    {  666, 7258,  208755,  7, }, // Antarctica/Syowa
    {  666, 8570,  208762,  4, }, // Antarctica/Troll
    {  666, 7337,  208766,  6, }, // Antarctica/Vostok
    {  666, 1413,  208772, 12, }, // Arctic/Longyearbyen
    {  666, 5487,  208784,  4, }, // Asia/Aden
    {  666, 5297,  208788,  7, }, // Asia/Almaty
    {  666, 3870,  208795,  5, }, // Asia/Amman
    {  666, 5428,  208800,  7, }, // Asia/Anadyr
    {  666, 5440,  208807,  5, }, // Asia/Aqtau
    {  666, 5270,  208812,  7, }, // Asia/Aqtobe
    {  666,  964,  208819,  9, }, // Asia/Ashgabat
    {  666, 8587,  208828,  6, }, // Asia/Atyrau
    {  666, 3509,  208834,  7, }, // Asia/Baghdad
    {  666, 5451,  208841,  6, }, // Asia/Bahrain
    {  666, 3555,   61979,  5, }, // Asia/Baku
    {  666, 4368,  208847,  6, }, // Asia/Bangkok
    {  666, 3473,  208853,  8, }, // Asia/Barnaul
    {  666, 3988,  208861,  7, }, // Asia/Beirut
    {  666, 3680,  208868,  7, }, // Asia/Bishkek
    {  666, 5906,  208875,  6, }, // Asia/Brunei
    {  666, 4503,  208881,  6, }, // Asia/Chita
    {  666, 5963,  208887, 10, }, // Asia/Choibalsan
    {  666, 4413,  208897,  8, }, // Asia/Colombo
    {  666, 4442,  208905, 10, }, // Asia/Damascus
    {  666, 1060,  207197,  5, }, // Asia/Dhaka
    {  666, 6125,  208915,  5, }, // Asia/Dili
    {  666, 3498,  208920,  5, }, // Asia/Dubai
    {  666, 6092,  208925,  9, }, // Asia/Dushanbe
    {  666, 8599,  208934, 10, }, // Asia/Famagusta
    {  666, 8614,  208944,  5, }, // Asia/Gaza
    {  666, 4697,  208949,  6, }, // Asia/Hebron
    {  666, 1226,  208955, 15, }, // Asia/Ho_Chi_Minh
    {  666, 2576,  208970,  7, }, // Asia/Hong_Kong
    {  666, 4673,  208977,  5, }, // Asia/Hovd
    {  666, 4086,   60932,  5, }, // Asia/Irkutsk
    {  666, 6969,  208982,  7, }, // Asia/Jakarta
    {  666, 6955,  208989,  9, }, // Asia/Jayapura
    {  666, 1257,  208998,  9, }, // Asia/Jerusalem
    {  666, 4218,  209007, 10, }, // Asia/Kamchatka
    {  666, 4141,  209017,  6, }, // Asia/Karachi
    {  666, 1152,  209023,  9, }, // Asia/Kathmandu
    {  666, 8624,  209032,  8, }, // Asia/Khandyga
    {  666,  992,  209040,  7, }, // Asia/Kolkata
    {  666, 4099,  209047, 10, }, // Asia/Krasnoyarsk
    {  666, 4761,  209057, 14, }, // Asia/Kuala_Lumpur
    {  666, 5893,  209071,  5, }, // Asia/Kuching
    {  666, 5464,  209076,  7, }, // Asia/Kuwait
    {  666, 1178,  209083,  7, }, // Asia/Macau
    {  666, 3919,  209090,  7, }, // Asia/Magadan
    {  666, 1316,  209097,  8, }, // Asia/Makassar
    {  666, 7145,  209105,  7, }, // Asia/Manila
    {  666, 6883,  209112,  7, }, // Asia/Muscat
    {  666, 2446,   20920,  8, }, // Asia/Nicosia
    {  666, 8638,  209119, 10, }, // Asia/Novokuznetsk
    {  666, 4037,  209129, 10, }, // Asia/Novosibirsk
    {  666, 4131,  209139,  6, }, // Asia/Omsk
    {  666, 7108,  209145,  5, }, // Asia/Oral
    {  666, 6924,  209150,  7, }, // Asia/Phnom_Penh
    {  666, 8656,  209157, 10, }, // Asia/Pontianak
    {  666, 4116,  209167,  8, }, // Asia/Pyongyang
    {  666, 5476,  206512,  5, }, // Asia/Qatar
    {  666, 8671,  209175,  8, }, // Asia/Qostanay
    {  666, 4171,  209183, 10, }, // Asia/Qyzylorda
    {  666, 3486,  209193,  6, }, // Asia/Riyadh
    {  666, 4310,    6124,  7, }, // Asia/Sakhalin
    {  666, 7208,  209199,  9, }, // Asia/Samarkand
    {  666, 3116,  209208,  5, }, // Asia/Seoul
    {  666, 1020,  209213,  7, }, // Asia/Shanghai
    {  666, 3137,  209220,  9, }, // Asia/Singapore
    {  666, 4199,  209229, 11, }, // Asia/Srednekolymsk
    {  666, 3100,  209240,  6, }, // Asia/Taipei
    {  666, 4683,  209246,  8, }, // Asia/Tashkent
    {  666, 3817,  209254,  8, }, // Asia/Tbilisi
    {  666, 2638,  209262,  7, }, // Asia/Tehran
    {  666, 1284,  209269,  5, }, // Asia/Thimphu
    {  666, 2687,  209274,  7, }, // Asia/Tokyo
    {  666, 4474,  209281,  7, }, // Asia/Tomsk
    {  666, 1346,  209288, 10, }, // Asia/Ulaanbaatar
    {  666, 1126,  209298,  7, }, // Asia/Urumqi
    {  666, 8685,  209305,  9, }, // Asia/Ust-Nera
    {  666, 6940,  209314, 10, }, // Asia/Vientiane
    {  666, 4612,  209324, 11, }, // Asia/Vladivostok
    {  666, 4730,  168346,  5, }, // Asia/Yakutsk
    {  666, 1202,  209335,  6, }, // Asia/Yangon
    {  666, 3785,  209341, 12, }, // Asia/Yekaterinburg
    {  666, 3649,  209353,  8, }, // Asia/Yerevan
    {  666, 3565,  209361,  5, }, // Atlantic/Azores
    {  666, 5588,  209366,  8, }, // Atlantic/Bermuda
    {  666, 6591,  209374,  7, }, // Atlantic/Canary
    {  666, 3629,  209381,  7, }, // Atlantic/Cape_Verde
    {  666, 1379,  209388,  5, }, // Atlantic/Faroe
    {  666, 8699,  209393,  8, }, // Atlantic/Madeira
    {  666, 2614,  209401,  9, }, // Atlantic/Reykjavik
    {  666, 7235,  209410, 25, }, // Atlantic/South_Georgia
    {  666, 6784,  209435, 10, }, // Atlantic/St_Helena
    {  666, 6607,  209445,  7, }, // Atlantic/Stanley
    {  666, 1654,  209452,  7, }, // Australia/Adelaide
    {  666, 1619,  209459,  7, }, // Australia/Brisbane
    {  666, 1783,  209466, 10, }, // Australia/Broken_Hill
    {  666, 1567,  209476,  6, }, // Australia/Darwin
    {  666, 3539,  209482,  5, }, // Australia/Eucla
    {  666, 1500,  209487,  5, }, // Australia/Hobart
    {  666, 8716,  209492,  8, }, // Australia/Lindeman
    {  666, 1531,  209500,  8, }, // Australia/Lord_Howe
    {  666, 1711,  209508,  7, }, // Australia/Melbourne
    {  666, 1746,  209515,  4, }, // Australia/Perth
    {  666, 1447,  209519,  6, }, // Australia/Sydney
    {  666, 6390,  209525,  9, }, // Europe/Amsterdam
    {  666, 6149,  209534,  7, }, // Europe/Andorra
    {  666, 3522,  209541,  9, }, // Europe/Astrakhan
    {  666, 6577,  209550,  4, }, // Europe/Athens
    {  666, 6285,  209554,  8, }, // Europe/Belgrade
    {  666, 4659,  209562,  6, }, // Europe/Berlin
    {  666, 6437,  209568, 10, }, // Europe/Bratislava
    {  666, 6178,  209578,  6, }, // Europe/Brussels
    {  666, 3830,  209584,  7, }, // Europe/Bucharest
    {  666, 3708,  209591,  7, }, // Europe/Budapest
    {  666, 8751,  209598,  8, }, // Europe/Busingen
    {  666, 2475,  209606,  8, }, // Europe/Chisinau
    {  666, 6238,  209614,  8, }, // Europe/Copenhagen
    {  666, 2260,  209622,  6, }, // Europe/Dublin
    {  666, 6256,  206627,  9, }, // Europe/Gibraltar
    {  666, 8767,  209628,  8, }, // Europe/Guernsey
    {  666, 6561,  209636,  9, }, // Europe/Helsinki
    {  666, 8783,  209645, 16, }, // Europe/Isle_of_Man
    {  666, 1097,  209661,  9, }, // Europe/Istanbul
    {  666, 8802,  205808,  6, }, // Europe/Jersey
    {  666, 3881,  209670, 12, }, // Europe/Kaliningrad
    {  666, 8924,  209682,  6, }, // Europe/Kirov
    {  666, 2419,  209688,  5, }, // Europe/Kyiv
    {  666, 3078,  209693,  6, }, // Europe/Lisbon
    {  666, 6455,  209699,  9, }, // Europe/Ljubljana
    {  666, 2393,  209708,  6, }, // Europe/London
    {  666, 6314,  209714,  9, }, // Europe/Luxembourg
    {  666, 4794,  209723,  7, }, // Europe/Madrid
    {  666, 6531,  209730,  9, }, // Europe/Mariehamn
    {  666, 3595,  209739,  4, }, // Europe/Minsk
    {  666, 6359,  209743,  7, }, // Europe/Monaco
    {  666, 3443,  209750,  6, }, // Europe/Moscow
    {  666, 6407,  209756,  5, }, // Europe/Oslo
    {  666, 4186,   62218,  5, }, // Europe/Paris
    {  666, 6373,  209761, 10, }, // Europe/Podgorica
    {  666, 6224,    4434,  5, }, // Europe/Prague
    {  666, 8816,  209771,  5, }, // Europe/Riga
    {  666, 6273,   62228,  4, }, // Europe/Rome
    {  666, 4233,  209776,  7, }, // Europe/Samara
    {  666, 6419,  209783, 11, }, // Europe/San_Marino
    {  666, 6194,  209794,  9, }, // Europe/Sarajevo
    {  666, 4353,  209803,  8, }, // Europe/Saratov
    {  666, 6332,  209811,  6, }, // Europe/Skopje
    {  666, 6548,   21329,  6, }, // Europe/Sofia
    {  666, 6472,  209817,  8, }, // Europe/Stockholm
    {  666, 8846,    6594,  5, }, // Europe/Tallinn
    {  666, 6135,  209825,  7, }, // Europe/Tirane
    {  666, 8861,  209832,  8, }, // Europe/Ulyanovsk
    {  666, 6301,  209840,  5, }, // Europe/Vaduz
    {  666, 6516,  209845,  7, }, // Europe/Vatican
    {  666, 6164,  209852,  6, }, // Europe/Vienna
    {  666, 8878,  209858,  8, }, // Europe/Vilnius
    {  666, 4629,  209866, 10, }, // Europe/Volgograd
    {  666, 3055,  209876,  6, }, // Europe/Warsaw
    {  666, 6210,  209882,  6, }, // Europe/Zagreb
    {  666, 6489,  209888,  6, }, // Europe/Zurich
    {  666, 4980,  209894, 13, }, // Indian/Antananarivo
    {  666, 6910,  209907,  6, }, // Indian/Chagos
    {  666, 5979,  209913,  5, }, // Indian/Christmas
    {  666, 5996,  209918,  6, }, // Indian/Cocos
    {  666, 4931,  209924,  7, }, // Indian/Comoro
    {  666, 6624,  209931,  9, }, // Indian/Kerguelen
    {  666, 7223,  209940,  5, }, // Indian/Mahe
    {  666, 7013,  209945,  8, }, // Indian/Maldives
    {  666, 3971,  209953,  6, }, // Indian/Mauritius
    {  666, 5000,  209959,  7, }, // Indian/Mayotte
    {  666, 7174,  209966,  8, }, // Indian/Reunion
    {  666, 4324,  206107,  5, }, // Pacific/Apia
    {  666,  932,  209974,  7, }, // Pacific/Auckland
    {  666, 3608,  209981,  9, }, // Pacific/Bougainville
    {  666, 2863,  209990,  6, }, // Pacific/Chatham
    {  666, 3022,  209996,  4, }, // Pacific/Chuuk
    {  666, 2201,   55967,  4, }, // Pacific/Easter
    {  666, 7323,  210000,  6, }, // Pacific/Efate
    {  666, 7290,  210006,  9, }, // Pacific/Fakaofo
    {  666, 7306,  199954,  9, }, // Pacific/Funafuti
    {  666, 6641,  210015,  9, }, // Pacific/Galapagos
    {  666, 6659,  210024,  8, }, // Pacific/Gambier
    {  666, 3724,  210032, 12, }, // Pacific/Guadalcanal
    {  666, 5935,  169511,  5, }, // Pacific/Guam
    {  666, 2897,  210044,  6, }, // Pacific/Kanton
    {  666, 3900,  210050,  9, }, // Pacific/Kiritimati
    {  666, 6982,  210059,  6, }, // Pacific/Kosrae
    {  666, 2708,  210065,  9, }, // Pacific/Kwajalein
    {  666, 4779,  210074,  7, }, // Pacific/Majuro
    {  666, 3953,   21524,  8, }, // Pacific/Marquesas
    {  666, 8909,  210081,  6, }, // Pacific/Midway
    {  666, 7047,  210087,  6, }, // Pacific/Nauru
    {  666, 7095,  210093,  6, }, // Pacific/Niue
    {  666, 4070,  210099,  7, }, // Pacific/Norfolk
    {  666, 7061,  206261,  6, }, // Pacific/Noumea
    {  666, 2991,  210106, 10, }, // Pacific/Pago_Pago
    {  666, 7118,  210116,  7, }, // Pacific/Palau
    {  666, 7157,  210123,  7, }, // Pacific/Pitcairn
    {  666, 2961,  210130,  7, }, // Pacific/Pohnpei
    {  666, 4709,  210137, 13, }, // Pacific/Port_Moresby
    {  666, 6009,  210150,  9, }, // Pacific/Rarotonga
    {  666, 5948,  210159,  6, }, // Pacific/Saipan
    {  666, 7275,  200182,  7, }, // Pacific/Tahiti
    {  666, 6675,  210165,  7, }, // Pacific/Tarawa
    {  666, 4485,  210172, 10, }, // Pacific/Tongatapu
    {  666, 7355,  210182,  4, }, // Pacific/Wake
    {  666, 7368,  169634,  5, }, // Pacific/Wallis
    {  667,   14,       0,  6, }, // Africa/Asmara Kangri/Devanagari/India
    {  667, 4337,       6,  8, }, // Africa/Sao_Tome
    {  667, 4154,      14,  8, }, // America/Asuncion
    {  667,  237,      22,  8, }, // America/Atikokan
    {  667, 7586,      30, 17, }, // America/Bahia_Banderas
    {  667, 3770,      47,  6, }, // America/Cancun
    {  667, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  667, 5640,      66,  7, }, // America/Curacao
    {  667,  481,      73, 13, }, // America/Indiana/Knox
    {  667, 7924,      86, 16, }, // America/Indiana/Marengo
    {  667, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  667, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  667, 8001,     139, 14, }, // America/Indiana/Vevay
    {  667, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  667, 8049,     171, 16, }, // America/Indiana/Winamac
    {  667, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  667, 5849,     207, 22, }, // America/Lower_Princes
    {  667, 8183,     229,  6, }, // America/Merida
    {  667, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  667, 8292,     255, 20, }, // America/North_Dakota/Center
    {  667, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  667,  393,     298,  4, }, // America/Nuuk
    {  667, 6862,     302, 16, }, // America/Scoresbysund
    {  667, 8471,     318, 14, }, // America/St_Barthelemy
    {  667, 2045,     332, 10, }, // America/St_Johns
    {  667, 5780,     342,  9, }, // America/St_Kitts
    {  667, 5797,     351,  9, }, // America/St_Lucia
    {  667,  855,     360, 10, }, // America/St_Thomas
    {  667, 5830,     370, 11, }, // America/St_Vincent
    {  667, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  667, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  667, 1152,     413,  9, }, // Asia/Kathmandu
    {  667,  992,     422,  7, }, // Asia/Kolkata
    {  667, 1178,     429,  5, }, // Asia/Macau
    {  667, 1202,     442,  6, }, // Asia/Yangon
    {  667, 1379,     448,  5, }, // Atlantic/Faroe
    {  667, 6784,     453, 10, }, // Atlantic/St_Helena
    {  667, 2419,     463,  4, }, // Europe/Kyiv
    {  667, 7174,     467,  7, }, // Indian/Reunion
    {  667, 3022,     474,  5, }, // Pacific/Chuuk
    {  667, 2961,     479,  7, }, // Pacific/Pohnpei
    {  668, 6751,  186471,  6, }, // Africa/Abidjan Venetian/Latin/Italy
    {  668, 6723,   72588,  4, }, // Africa/Accra
    {  668, 4961,   34301, 10, }, // Africa/Addis_Ababa
    {  668, 7383,  210186,  6, }, // Africa/Algiers
    {  668,   14,  210192,  6, }, // Africa/Asmara
    {  668, 5159,  210198,  6, }, // Africa/Bangui
    {  668, 7398,  210204,  5, }, // Africa/Bissau
    {  668, 4824,  210209,  9, }, // Africa/Bujumbura
    {  668, 2242,  154503,  8, }, // Africa/Cairo
    {  668, 4019,  210218, 10, }, // Africa/Casablanca
    {  668, 7412,  210228,  5, }, // Africa/Ceuta
    {  668, 6736,  150586,  7, }, // Africa/Conakry
    {  668, 5032,   19759, 12, }, // Africa/Dar_es_Salaam
    {  668, 4945,  151111,  6, }, // Africa/Djibouti
    {  668, 5145,   19777,  5, }, // Africa/Douala
    {  668, 5068,    4663,  7, }, // Africa/El_Aaiun
    {  668, 5053,  210233,  7, }, // Africa/Kampala
    {  668, 4426,  150604,  6, }, // Africa/Khartoum
    {  668, 6832,   18802,  4, }, // Africa/Lome
    {  668, 4841,  210240, 10, }, // Africa/Lubumbashi
    {  668, 5099,  210250,  6, }, // Africa/Maseru
    {  668, 5084,  210256,  7, }, // Africa/Mbabane
    {  668, 5015,  210263,  8, }, // Africa/Mogadishu
    {  668, 5173,  186545,  6, }, // Africa/Ndjamena
    {  668, 6766,  140212,  8, }, // Africa/Nouakchott
    {  668, 6690,  210271,  8, }, // Africa/Ouagadougou
    {  668, 5127,   24975, 10, }, // Africa/Porto-Novo
    {  668, 4337,  210279, 15, }, // Africa/Sao_Tome
    {  668, 2732,  157091,  7, }, // Africa/Tripoli
    {  668, 6503,  210294,  6, }, // Africa/Tunis
    {  668, 5524,   18863,  7, }, // America/Anguilla
    {  668,  270,  210300,  7, }, // America/Argentina/Cordoba
    {  668, 7452,  210307, 11, }, // America/Argentina/Rio_Gallegos
    {  668, 4154,    4869,  8, }, // America/Asuncion
    {  668,  237,      22,  8, }, // America/Atikokan
    {  668, 7586,  210318, 16, }, // America/Bahia_Banderas
    {  668, 4263,   42665,  6, }, // America/Bogota
    {  668, 7698,  210334, 12, }, // America/Campo_Grande
    {  668, 3770,    3861,  6, }, // America/Cancun
    {  668, 4247,  140246,  6, }, // America/Cayenne
    {  668, 5398,   42677,  6, }, // America/Cayman
    {  668, 7737,      60,  6, }, // America/Ciudad_Juarez
    {  668, 3693,  210346,  6, }, // America/Cuiaba
    {  668, 5640,      66,  7, }, // America/Curacao
    {  668,  805,  210352,  6, }, // America/Denver
    {  668, 5656,  210358,  8, }, // America/Dominica
    {  668, 7832,   25013,  8, }, // America/Eirunepe
    {  668, 5687,   18910,  7, }, // America/Grenada
    {  668, 5703,   93628,  9, }, // America/Guadeloupe
    {  668, 2221,   93637,  7, }, // America/Havana
    {  668,  348,  210366, 12, }, // America/Indiana/Indianapolis
    {  668,  481,  210378, 14, }, // America/Indiana/Knox
    {  668, 7924,  210392, 17, }, // America/Indiana/Marengo
    {  668, 7948,  210409, 20, }, // America/Indiana/Petersburg
    {  668, 7975,  210429, 19, }, // America/Indiana/Tell_City
    {  668, 8001,  210448, 15, }, // America/Indiana/Vevay
    {  668, 8023,  210463, 19, }, // America/Indiana/Vincennes
    {  668, 8049,  210482, 17, }, // America/Indiana/Winamac
    {  668, 2665,  210499,  7, }, // America/Jamaica
    {  668, 8088,  210506, 21, }, // America/Kentucky/Monticello
    {  668, 3390,  210527, 11, }, // America/Los_Angeles
    {  668, 5849,     207, 22, }, // America/Lower_Princes
    {  668, 8116,   25021,  6, }, // America/Maceio
    {  668, 5722,  210538,  9, }, // America/Martinique
    {  668, 8183,     229,  6, }, // America/Merida
    {  668, 2811,  210547, 16, }, // America/Mexico_City
    {  668, 4293,  210563,  8, }, // America/Miquelon
    {  668, 8233,   20349,  8, }, // America/Monterrey
    {  668, 5741,    5457,  9, }, // America/Montserrat
    {  668, 1834,   60511, 19, }, // America/Noronha
    {  668, 8264,  210571, 20, }, // America/North_Dakota/Beulah
    {  668, 8292,  210591, 20, }, // America/North_Dakota/Center
    {  668, 8320,  210611, 23, }, // America/North_Dakota/New_Salem
    {  668,  393,     298,  4, }, // America/Nuuk
    {  668, 5413,  157218,  6, }, // America/Panama
    {  668, 3847,  210634, 14, }, // America/Port-au-Prince
    {  668, 5871,  210648, 15, }, // America/Port_of_Spain
    {  668, 5760,   93741,  9, }, // America/Puerto_Rico
    {  668, 1862,  210663,  8, }, // America/Sao_Paulo
    {  668, 6862,     302, 16, }, // America/Scoresbysund
    {  668, 8471,  210671, 13, }, // America/St_Barthelemy
    {  668, 2045,  210684,  8, }, // America/St_Johns
    {  668, 5780,  210692, 12, }, // America/St_Kitts
    {  668, 5797,  210704,  8, }, // America/St_Lucia
    {  668,  855,  210285,  9, }, // America/St_Thomas
    {  668, 5830,  210712, 11, }, // America/St_Vincent
    {  668, 5605,  140488,  7, }, // America/Tortola
    {  668, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  668, 8531,  210723, 15, }, // Antarctica/Macquarie
    {  668, 3870,    5896,  4, }, // Asia/Amman
    {  668, 5440,  210738,  5, }, // Asia/Aqtau
    {  668, 5270,    5911,  6, }, // Asia/Aqtobe
    {  668, 8587,  210743,  6, }, // Asia/Atyrau
    {  668, 3509,    4063,  6, }, // Asia/Baghdad
    {  668, 5451,   62334,  6, }, // Asia/Bahrain
    {  668, 5906,   19142,  6, }, // Asia/Brunei
    {  668, 4413,  210749,  7, }, // Asia/Colombo
    {  668, 4442,   72897,  7, }, // Asia/Damascus
    {  668, 1060,  151106,  4, }, // Asia/Dhaka
    {  668, 3498,  210756,  5, }, // Asia/Dubai
    {  668, 8599,   93777,  9, }, // Asia/Famagusta
    {  668, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  668, 1257,  119178, 10, }, // Asia/Jerusalem
    {  668, 1152,  210761,  8, }, // Asia/Kathmandu
    {  668,  992,   19229,  7, }, // Asia/Kolkata
    {  668, 4761,  210769, 12, }, // Asia/Kuala_Lumpur
    {  668, 1178,  210781,  5, }, // Asia/Macau
    {  668, 1316,    6069,  7, }, // Asia/Makassar
    {  668, 6883,   19248,  7, }, // Asia/Muscat
    {  668, 6924,   20928,  8, }, // Asia/Phnom_Penh
    {  668, 5476,   71561,  5, }, // Asia/Qatar
    {  668, 8671,  210786,  8, }, // Asia/Qostanay
    {  668, 4171,    6111,  9, }, // Asia/Qyzylorda
    {  668, 3486,    6120,  4, }, // Asia/Riyadh
    {  668, 7208,   19277, 10, }, // Asia/Samarkand
    {  668, 3116,    6131,  4, }, // Asia/Seoul
    {  668, 1020,  210794,  8, }, // Asia/Shanghai
    {  668, 3100,  169861,  6, }, // Asia/Taipei
    {  668, 3817,  209254,  7, }, // Asia/Tbilisi
    {  668, 2638,    6175,  7, }, // Asia/Tehran
    {  668, 1284,  210802,  5, }, // Asia/Thimphu
    {  668, 1346,  210807, 10, }, // Asia/Ulaanbaatar
    {  668, 1126,  193029,  7, }, // Asia/Urumqi
    {  668, 6940,   25389,  8, }, // Asia/Vientiane
    {  668, 1202,     442,  6, }, // Asia/Yangon
    {  668, 3649,  210817,  7, }, // Asia/Yerevan
    {  668, 3565,  210824, 11, }, // Atlantic/Azores
    {  668, 6591,  210835, 13, }, // Atlantic/Canary
    {  668, 3629,  210848,  9, }, // Atlantic/Cape_Verde
    {  668, 1379,  210857, 11, }, // Atlantic/Faroe
    {  668, 8699,  210868, 13, }, // Atlantic/Madeira
    {  668, 2614,  210881,  8, }, // Atlantic/Reykjavik
    {  668, 7235,  210889, 16, }, // Atlantic/South_Georgia
    {  668, 6784,  210905, 14, }, // Atlantic/St_Helena
    {  668, 1654,  210919,  8, }, // Australia/Adelaide
    {  668, 1619,  210927,  8, }, // Australia/Brisbane
    {  668, 1531,  210935, 15, }, // Australia/Lord_Howe
    {  668, 6390,  210950,  9, }, // Europe/Amsterdam
    {  668, 6149,   32109,  6, }, // Europe/Andorra
    {  668, 6577,   19437,  5, }, // Europe/Athens
    {  668, 6285,   57443,  8, }, // Europe/Belgrade
    {  668, 6437,  210959, 10, }, // Europe/Bratislava
    {  668, 6178,   25472,  6, }, // Europe/Brussels
    {  668, 3830,   19464,  8, }, // Europe/Bucharest
    {  668, 8751,   25487,  8, }, // Europe/Busingen
    {  668, 6238,   93850, 10, }, // Europe/Copenhagen
    {  668, 6256,  210969,  9, }, // Europe/Gibraltar
    {  668, 8783,  210978, 12, }, // Europe/Isle_of_Man
    {  668, 1097,  157498,  8, }, // Europe/Istanbul
    {  668, 2419,    6486,  4, }, // Europe/Kyiv
    {  668, 3078,  150331,  7, }, // Europe/Lisbon
    {  668, 6455,  210990,  8, }, // Europe/Ljubljana
    {  668, 2393,   93889,  6, }, // Europe/London
    {  668, 6314,  210998, 10, }, // Europe/Luxembourg
    {  668, 6359,   43074,  6, }, // Europe/Monaco
    {  668, 6407,  211008,  4, }, // Europe/Oslo
    {  668, 4186,  211012,  6, }, // Europe/Paris
    {  668, 6373,  187807,  9, }, // Europe/Podgorica
    {  668, 6224,   19576,  5, }, // Europe/Prague
    {  668, 6273,   19581,  4, }, // Europe/Rome
    {  668, 6419,   64261,  9, }, // Europe/San_Marino
    {  668, 8828,  211018, 11, }, // Europe/Simferopol
    {  668, 6472,  211029,  8, }, // Europe/Stockholm
    {  668, 8846,    6594,  5, }, // Europe/Tallinn
    {  668, 6135,   19608,  6, }, // Europe/Tirane
    {  668, 6516,  211037,  7, }, // Europe/Vatican
    {  668, 6164,   19625,  5, }, // Europe/Vienna
    {  668, 3055,   93955,  8, }, // Europe/Warsaw
    {  668, 6489,   93971,  6, }, // Europe/Zurich
    {  668, 6910,  211044,  6, }, // Indian/Chagos
    {  668, 5979,  211050, 15, }, // Indian/Christmas
    {  668, 5996,  211065, 11, }, // Indian/Cocos
    {  668, 4931,  211076, 12, }, // Indian/Comoro
    {  668, 7223,   25565,  4, }, // Indian/Mahe
    {  668, 7013,  211088, 13, }, // Indian/Maldives
    {  668, 3971,  211101, 14, }, // Indian/Mauritius
    {  668, 5000,  211115, 12, }, // Indian/Mayotte
    {  668, 7174,  211127, 13, }, // Indian/Reunion
    {  668, 3608,  211140, 18, }, // Pacific/Bougainville
    {  668, 2863,  211158, 12, }, // Pacific/Chatham
    {  668, 3022,  211170, 11, }, // Pacific/Chuuk
    {  668, 2201,  211181, 15, }, // Pacific/Easter
    {  668, 7323,  211196, 11, }, // Pacific/Efate
    {  668, 7290,  211207, 13, }, // Pacific/Fakaofo
    {  668, 7306,  211220, 14, }, // Pacific/Funafuti
    {  668, 6641,  211234,  9, }, // Pacific/Galapagos
    {  668, 6659,  211243, 13, }, // Pacific/Gambier
    {  668, 3724,  211256, 17, }, // Pacific/Guadalcanal
    {  668, 2897,  211273, 12, }, // Pacific/Kanton
    {  668, 3900,  211285, 16, }, // Pacific/Kiritimati
    {  668, 6982,  211301, 12, }, // Pacific/Kosrae
    {  668, 2708,  211313, 15, }, // Pacific/Kwajalein
    {  668, 3953,  211328, 14, }, // Pacific/Marquesas
    {  668, 8909,  211342, 12, }, // Pacific/Midway
    {  668, 7047,  211354, 11, }, // Pacific/Nauru
    {  668, 7095,  211365, 10, }, // Pacific/Niue
    {  668, 4070,  211375, 13, }, // Pacific/Norfolk
    {  668, 7061,    4529,  6, }, // Pacific/Noumea
    {  668, 2991,  131110, 11, }, // Pacific/Pago_Pago
    {  668, 7118,  211388,  5, }, // Pacific/Palau
    {  668, 7157,  211393, 14, }, // Pacific/Pitcairn
    {  668, 2961,  211407, 12, }, // Pacific/Pohnpei
    {  668, 4709,  211419, 13, }, // Pacific/Port_Moresby
    {  668, 6009,  211432, 15, }, // Pacific/Rarotonga
    {  668, 5948,  211447, 12, }, // Pacific/Saipan
    {  668, 7275,  211459, 11, }, // Pacific/Tahiti
    {  668, 6675,  211470, 13, }, // Pacific/Tarawa
    {  668, 4485,  211483, 15, }, // Pacific/Tongatapu
    {  668, 7355,  211498, 10, }, // Pacific/Wake
    {  668, 7368,  211508, 12, }, // Pacific/Wallis
    {  669, 6751,  211520,  7, }, // Africa/Abidjan Kuvi/Latin/India
    {  669, 6723,  211527,  4, }, // Africa/Accra
    {  669, 4961,  211531, 10, }, // Africa/Addis_Ababa
    {  669, 7383,  211541,  7, }, // Africa/Algiers
    {  669,   14,  211548,  6, }, // Africa/Asmara
    {  669,   44,  211554,  6, }, // Africa/Bamako
    {  669, 5159,  211560,  6, }, // Africa/Bangui
    {  669, 6709,  211566,  6, }, // Africa/Banjul
    {  669, 7398,  211572,  6, }, // Africa/Bissau
    {  669, 4859,  211578,  8, }, // Africa/Blantyre
    {  669, 5189,  211586,  9, }, // Africa/Brazzaville
    {  669, 4824,  211595,  9, }, // Africa/Bujumbura
    {  669, 2242,  211604,  5, }, // Africa/Cairo
    {  669, 4019,  211609, 10, }, // Africa/Casablanca
    {  669, 7412,  211619,  5, }, // Africa/Ceuta
    {  669, 6736,  211624,  7, }, // Africa/Conakry
    {  669, 6803,  211631,  4, }, // Africa/Dakar
    {  669, 5032,  211635, 13, }, // Africa/Dar_es_Salaam
    {  669, 4945,   24926,  6, }, // Africa/Djibouti
    {  669, 5145,  211648,  5, }, // Africa/Douala
    {  669, 5068,  211653,  9, }, // Africa/El_Aaiun
    {  669, 6816,  211662,  7, }, // Africa/Freetown
    {  669, 4808,  211669,  7, }, // Africa/Gaborone
    {  669, 4917,  211676,  5, }, // Africa/Harare
    {  669, 4381,  211681, 12, }, // Africa/Johannesburg
    {  669, 4401,  211693,  4, }, // Africa/Juba
    {  669, 5053,  211697,  6, }, // Africa/Kampala
    {  669, 4426,  211703,  7, }, // Africa/Khartoum
    {  669, 4875,  211710,  6, }, // Africa/Kigali
    {  669, 5208,  211716,  7, }, // Africa/Kinshasa
    {  669, 4646,  211723,  5, }, // Africa/Lagos
    {  669, 5238,  211728,  9, }, // Africa/Libreville
    {  669, 6832,    5951,  3, }, // Africa/Lome
    {  669, 5113,  211737,  6, }, // Africa/Luanda
    {  669, 4841,  211743,  9, }, // Africa/Lubumbashi
    {  669, 4903,  211752,  6, }, // Africa/Lusaka
    {  669, 5224,  211758,  6, }, // Africa/Malabo
    {  669, 4889,  211764,  6, }, // Africa/Maputo
    {  669, 5099,  211770,  6, }, // Africa/Maseru
    {  669, 5084,  211776,  7, }, // Africa/Mbabane
    {  669, 5015,  211783,  8, }, // Africa/Mogadishu
    {  669, 6997,  211791,  8, }, // Africa/Monrovia
    {  669, 3755,  211799,  7, }, // Africa/Nairobi
    {  669, 5173,  211806,  7, }, // Africa/Ndjamena
    {  669, 5256,  211813,  6, }, // Africa/Niamey
    {  669, 6766,  211819,  6, }, // Africa/Nouakchott
    {  669, 6690,  211825,  7, }, // Africa/Ouagadougou
    {  669, 5127,  211832, 10, }, // Africa/Porto-Novo
    {  669, 4337,  211842,  7, }, // Africa/Sao_Tome
    {  669, 2732,  211849,  7, }, // Africa/Tripoli
    {  669, 6503,  211856,  5, }, // Africa/Tunis
    {  669, 4054,  211861,  7, }, // Africa/Windhoek
    {  669,  132,  211868,  4, }, // America/Adak
    {  669, 3183,  211872,  7, }, // America/Anchorage
    {  669, 5524,  211879,  8, }, // America/Anguilla
    {  669, 5541,  211887,  7, }, // America/Antigua
    {  669, 4456,  211894,  9, }, // America/Araguaina
    {  669,  166,  211903, 13, }, // America/Argentina/Buenos_Aires
    {  669,   91,  211916,  9, }, // America/Argentina/Catamarca
    {  669,  270,  211925,  7, }, // America/Argentina/Cordoba
    {  669,  441,  211932,  5, }, // America/Argentina/Jujuy
    {  669, 7425,  211937,  8, }, // America/Argentina/La_Rioja
    {  669,  565,  211945,  7, }, // America/Argentina/Mendoza
    {  669, 7452,  211952, 13, }, // America/Argentina/Rio_Gallegos
    {  669, 7483,  211965,  5, }, // America/Argentina/Salta
    {  669, 7507,  211970,  8, }, // America/Argentina/San_Juan
    {  669, 5497,  211978,  8, }, // America/Argentina/San_Luis
    {  669, 7534,  211986,  6, }, // America/Argentina/Tucuman
    {  669, 7560,  211992,  7, }, // America/Argentina/Ushuaia
    {  669, 5557,  211999,  5, }, // America/Aruba
    {  669, 4154,  212004,  8, }, // America/Asuncion
    {  669,  237,  212012,  8, }, // America/Atikokan
    {  669, 3581,  212020,  5, }, // America/Bahia
    {  669, 7586,  212025, 13, }, // America/Bahia_Banderas
    {  669, 5571,  212038,  8, }, // America/Barbados
    {  669, 7609,  212046,  5, }, // America/Belem
    {  669, 5309,  212051,  5, }, // America/Belize
    {  669, 7623,  212056, 12, }, // America/Blanc-Sablon
    {  669, 7644,  212068,  9, }, // America/Boa_Vista
    {  669, 4263,  212077,  6, }, // America/Bogota
    {  669, 7662,  212083,  5, }, // America/Boise
    {  669, 7676,  212088, 10, }, // America/Cambridge_Bay
    {  669, 7698,  212098, 12, }, // America/Campo_Grande
    {  669, 3770,  212110,  6, }, // America/Cancun
    {  669, 4596,  212116,  7, }, // America/Caracas
    {  669, 4247,  212123,  5, }, // America/Cayenne
    {  669, 5398,  212128,  6, }, // America/Cayman
    {  669, 3251,  212134,  6, }, // America/Chicago
    {  669, 7719,  212140,  9, }, // America/Chihuahua
    {  669, 7737,      53, 13, }, // America/Ciudad_Juarez
    {  669, 5324,  212149, 10, }, // America/Costa_Rica
    {  669, 7759,  212159,  7, }, // America/Creston
    {  669, 3693,  212166,  6, }, // America/Cuiaba
    {  669, 5640,  212172,  7, }, // America/Curacao
    {  669, 7775,  212179, 11, }, // America/Danmarkshavn
    {  669, 7796,  212190,  5, }, // America/Dawson
    {  669, 7811,  212195, 11, }, // America/Dawson_Creek
    {  669,  805,  212206,  6, }, // America/Denver
    {  669, 3351,  212212,  7, }, // America/Detroit
    {  669, 5656,  212219,  8, }, // America/Dominica
    {  669,  893,  212227,  8, }, // America/Edmonton
    {  669, 7832,  212235,  8, }, // America/Eirunepe
    {  669, 5343,  212243, 11, }, // America/El_Salvador
    {  669, 7849,  212254, 11, }, // America/Fort_Nelson
    {  669, 7869,  212265,  9, }, // America/Fortaleza
    {  669, 7887,  212274,  8, }, // America/Glace_Bay
    {  669, 6844,  212282,  7, }, // America/Goose_Bay
    {  669, 4514,  212289, 10, }, // America/Grand_Turk
    {  669, 5687,  212299,  7, }, // America/Grenada
    {  669, 5703,  212306, 10, }, // America/Guadeloupe
    {  669, 3662,  212316,  9, }, // America/Guatemala
    {  669, 4743,  212325,  8, }, // America/Guayaquil
    {  669, 6895,  212333,  6, }, // America/Guyana
    {  669, 1923,  212339,  8, }, // America/Halifax
    {  669, 2221,  212347,  6, }, // America/Havana
    {  669, 7905,    5204,  8, }, // America/Hermosillo
    {  669,  348,  212353, 12, }, // America/Indiana/Indianapolis
    {  669,  481,  212365, 14, }, // America/Indiana/Knox
    {  669, 7924,  212379, 17, }, // America/Indiana/Marengo
    {  669, 7948,  212396, 20, }, // America/Indiana/Petersburg
    {  669, 7975,  212416, 19, }, // America/Indiana/Tell_City
    {  669, 8001,  212435, 14, }, // America/Indiana/Vevay
    {  669, 8023,  212449, 17, }, // America/Indiana/Vincennes
    {  669, 8049,  212466, 17, }, // America/Indiana/Winamac
    {  669, 8073,  212483,  6, }, // America/Inuvik
    {  669,  660,  212489,  7, }, // America/Iqaluit
    {  669, 2665,  212496,  7, }, // America/Jamaica
    {  669, 5282,  212503,  5, }, // America/Juneau
    {  669,  521,  212508,  9, }, // America/Kentucky/Louisville
    {  669, 8088,  212517, 19, }, // America/Kentucky/Monticello
    {  669, 5621,  212536, 10, }, // America/Kralendijk
    {  669, 4278,  212546,  6, }, // America/La_Paz
    {  669, 7132,  149983,  4, }, // America/Lima
    {  669, 3390,  212552, 11, }, // America/Los_Angeles
    {  669, 5849,  212563, 21, }, // America/Lower_Princes
    {  669, 8116,  212584,  5, }, // America/Maceio
    {  669, 8131,  212589,  7, }, // America/Managua
    {  669, 1892,  212596,  6, }, // America/Manaus
    {  669, 5814,  212602,  7, }, // America/Marigot
    {  669, 5722,  212609,  8, }, // America/Martinique
    {  669, 8147,  212617,  9, }, // America/Matamoros
    {  669, 2779,  212626,  8, }, // America/Mazatlan
    {  669, 8165,  212634,  8, }, // America/Menominee
    {  669, 8183,  212642,  6, }, // America/Merida
    {  669, 8198,  212648, 10, }, // America/Metlakatla
    {  669, 2811,  212658, 12, }, // America/Mexico_City
    {  669, 4293,  212670,  7, }, // America/Miquelon
    {  669, 8217,  212677,  7, }, // America/Moncton
    {  669, 8233,  212684,  8, }, // America/Monterrey
    {  669, 4000,  212692, 10, }, // America/Montevideo
    {  669, 5741,  212702, 10, }, // America/Montserrat
    {  669, 5383,  212712,  5, }, // America/Nassau
    {  669, 3294,  212717,  9, }, // America/New_York
    {  669, 8251,    5411,  3, }, // America/Nome
    {  669, 1834,  212726,  7, }, // America/Noronha
    {  669, 8264,  212733, 20, }, // America/North_Dakota/Beulah
    {  669, 8292,  212753, 19, }, // America/North_Dakota/Center
    {  669, 8320,  212772, 23, }, // America/North_Dakota/New_Salem
    {  669,  393,  212795,  3, }, // America/Nuuk
    {  669, 8351,  212798,  7, }, // America/Ojinaga
    {  669, 5413,  212805,  6, }, // America/Panama
    {  669, 6106,  212811, 10, }, // America/Paramaribo
    {  669, 3224,  212821,  6, }, // America/Phoenix
    {  669, 3847,  212827, 14, }, // America/Port-au-Prince
    {  669, 5871,  212841, 13, }, // America/Port_of_Spain
    {  669, 8367,  212854, 11, }, // America/Porto_Velho
    {  669, 5760,  212865, 11, }, // America/Puerto_Rico
    {  669, 3932,  212876, 12, }, // America/Punta_Arenas
    {  669, 8387,  212888, 13, }, // America/Rankin_Inlet
    {  669, 8408,  212901,  6, }, // America/Recife
    {  669, 1979,  212907,  6, }, // America/Regina
    {  669, 8423,  212913,  8, }, // America/Resolute
    {  669,  695,  212921, 10, }, // America/Rio_Branco
    {  669, 8440,  212931,  6, }, // America/Santarem
    {  669, 2165,  212937,  8, }, // America/Santiago
    {  669, 6044,  212945, 13, }, // America/Santo_Domingo
    {  669, 1862,  212958,  9, }, // America/Sao_Paulo
    {  669, 6862,  212967, 11, }, // America/Scoresbysund
    {  669, 8457,  212978,  5, }, // America/Sitka
    {  669, 8471,  212983, 14, }, // America/St_Barthelemy
    {  669, 2045,  212997,  9, }, // America/St_Johns
    {  669, 5780,  213006, 10, }, // America/St_Kitts
    {  669, 5797,  213016, 10, }, // America/St_Lucia
    {  669,  855,  213026, 10, }, // America/St_Thomas
    {  669, 5830,  213036, 12, }, // America/St_Vincent
    {  669, 8493,  213048, 13, }, // America/Swift_Current
    {  669, 5363,  213061,  4, }, // America/Tegucigalpa
    {  669, 5673,  213065,  4, }, // America/Thule
    {  669,  313,  213069,  7, }, // America/Tijuana
    {  669,  608,  213076,  7, }, // America/Toronto
    {  669, 5605,  213083,  7, }, // America/Tortola
    {  669, 2077,  213090,  8, }, // America/Vancouver
    {  669, 2128,  213098,  9, }, // America/Whitehorse
    {  669,  734,  213107,  8, }, // America/Winnipeg
    {  669, 8515,  213115,  7, }, // America/Yakutat
    {  669, 5918,  213122,  5, }, // Antarctica/Casey
    {  669, 6027,  213127,  5, }, // Antarctica/Davis
    {  669, 6066,  213132, 17, }, // Antarctica/DumontDUrville
    {  669, 8531,  213149,  7, }, // Antarctica/Macquarie
    {  669, 7029,  213156,  6, }, // Antarctica/Mawson
    {  669, 7076,  213162,  9, }, // Antarctica/McMurdo
    {  669, 8552,  213171,  6, }, // Antarctica/Palmer
    {  669, 7189,  213177,  6, }, // Antarctica/Rothera
    {  669, 7258,  213183,  5, }, // Antarctica/Syowa
    {  669, 8570,  213188,  5, }, // Antarctica/Troll
    {  669, 7337,  213193,  6, }, // Antarctica/Vostok
    {  669, 1413,  213199, 10, }, // Arctic/Longyearbyen
    {  669, 5487,  213209,  4, }, // Asia/Aden
    {  669, 5297,  213213,  6, }, // Asia/Almaty
    {  669, 3870,  213219,  5, }, // Asia/Amman
    {  669, 5428,  213224,  6, }, // Asia/Anadyr
    {  669, 5440,  213230,  5, }, // Asia/Aqtau
    {  669, 5270,  213235,  7, }, // Asia/Aqtobe
    {  669,  964,  213242,  7, }, // Asia/Ashgabat
    {  669, 8587,  213249,  6, }, // Asia/Atyrau
    {  669, 3509,  213255,  6, }, // Asia/Baghdad
    {  669, 5451,  213261,  7, }, // Asia/Bahrain
    {  669, 3555,  213268,  4, }, // Asia/Baku
    {  669, 4368,  213272,  7, }, // Asia/Bangkok
    {  669, 3473,  213279,  7, }, // Asia/Barnaul
    {  669, 3988,  213286,  6, }, // Asia/Beirut
    {  669, 3680,  213292,  6, }, // Asia/Bishkek
    {  669, 5906,  213298,  6, }, // Asia/Brunei
    {  669, 4503,  213304,  4, }, // Asia/Chita
    {  669, 5963,  213308,  9, }, // Asia/Choibalsan
    {  669, 4413,  213317,  7, }, // Asia/Colombo
    {  669, 4442,  213324,  8, }, // Asia/Damascus
    {  669, 1060,  211869,  4, }, // Asia/Dhaka
    {  669, 6125,  213332,  4, }, // Asia/Dili
    {  669, 3498,  213336,  5, }, // Asia/Dubai
    {  669, 6092,  213341,  7, }, // Asia/Dushanbe
    {  669, 8599,  213348,  9, }, // Asia/Famagusta
    {  669, 8614,  213357,  4, }, // Asia/Gaza
    {  669, 4697,  213361,  6, }, // Asia/Hebron
    {  669, 1226,  213367, 15, }, // Asia/Ho_Chi_Minh
    {  669, 2576,  213382,  9, }, // Asia/Hong_Kong
    {  669, 4673,   25279,  4, }, // Asia/Hovd
    {  669, 4086,  213391,  7, }, // Asia/Irkutsk
    {  669, 6969,    4149,  7, }, // Asia/Jakarta
    {  669, 6955,  213398,  8, }, // Asia/Jayapura
    {  669, 1257,  213406,  9, }, // Asia/Jerusalem
    {  669, 3462,  213415,  5, }, // Asia/Kabul
    {  669, 4218,  213420,  8, }, // Asia/Kamchatka
    {  669, 4141,  213428,  6, }, // Asia/Karachi
    {  669, 1152,  213434,  8, }, // Asia/Kathmandu
    {  669, 8624,  213442,  7, }, // Asia/Khandyga
    {  669,  992,  213449,  7, }, // Asia/Kolkata
    {  669, 4099,  213456, 11, }, // Asia/Krasnoyarsk
    {  669, 4761,  213467, 12, }, // Asia/Kuala_Lumpur
    {  669, 5893,  213479,  6, }, // Asia/Kuching
    {  669, 5464,  213485,  6, }, // Asia/Kuwait
    {  669, 1178,  213491,  4, }, // Asia/Macau
    {  669, 3919,  213495,  7, }, // Asia/Magadan
    {  669, 1316,  213502,  7, }, // Asia/Makassar
    {  669, 7145,  213509,  6, }, // Asia/Manila
    {  669, 6883,  213515,  6, }, // Asia/Muscat
    {  669, 2446,  213521,  8, }, // Asia/Nicosia
    {  669, 8638,  213529, 12, }, // Asia/Novokuznetsk
    {  669, 4037,  213541, 11, }, // Asia/Novosibirsk
    {  669, 4131,  119420,  4, }, // Asia/Omsk
    {  669, 7108,   89783,  4, }, // Asia/Oral
    {  669, 6924,  213552,  9, }, // Asia/Phnom_Penh
    {  669, 8656,  213561,  9, }, // Asia/Pontianak
    {  669, 4116,  213570,  9, }, // Asia/Pyongyang
    {  669, 5476,  201375,  5, }, // Asia/Qatar
    {  669, 8671,  213579,  7, }, // Asia/Qostanay
    {  669, 4171,  213586,  8, }, // Asia/Qyzylorda
    {  669, 3486,  213594,  5, }, // Asia/Riyadh
    {  669, 4310,  213599,  7, }, // Asia/Sakhalin
    {  669, 7208,  213606,  9, }, // Asia/Samarkand
    {  669, 3116,  213615,  4, }, // Asia/Seoul
    {  669, 1020,  213619,  6, }, // Asia/Shanghai
    {  669, 3137,  213625,  8, }, // Asia/Singapore
    {  669, 4199,  213633, 13, }, // Asia/Srednekolymsk
    {  669, 3100,  213646,  6, }, // Asia/Taipei
    {  669, 4683,  213652,  7, }, // Asia/Tashkent
    {  669, 3817,  213659,  7, }, // Asia/Tbilisi
    {  669, 2638,  213666,  6, }, // Asia/Tehran
    {  669, 1284,  213672,  6, }, // Asia/Thimphu
    {  669, 2687,  213678,  5, }, // Asia/Tokyo
    {  669, 4474,  213683,  5, }, // Asia/Tomsk
    {  669, 1346,  213688,  9, }, // Asia/Ulaanbaatar
    {  669, 1126,  213697,  6, }, // Asia/Urumqi
    {  669, 8685,  213703,  9, }, // Asia/Ust-Nera
    {  669, 6940,  213712,  9, }, // Asia/Vientiane
    {  669, 4612,  213721, 11, }, // Asia/Vladivostok
    {  669, 4730,  213732,  7, }, // Asia/Yakutsk
    {  669, 1202,  213739,  6, }, // Asia/Yangon
    {  669, 3785,  213745, 13, }, // Asia/Yekaterinburg
    {  669, 3649,  213758,  7, }, // Asia/Yerevan
    {  669, 3565,  213765,  6, }, // Atlantic/Azores
    {  669, 5588,  213771,  7, }, // Atlantic/Bermuda
    {  669, 6591,  213778,  6, }, // Atlantic/Canary
    {  669, 3629,  213784,  9, }, // Atlantic/Cape_Verde
    {  669, 1379,  213793,  4, }, // Atlantic/Faroe
    {  669, 8699,  213797,  7, }, // Atlantic/Madeira
    {  669, 2614,  213804,  9, }, // Atlantic/Reykjavik
    {  669, 7235,  213813, 11, }, // Atlantic/South_Georgia
    {  669, 6784,  213824, 11, }, // Atlantic/St_Helena
    {  669, 6607,  213835,  6, }, // Atlantic/Stanley
    {  669, 1654,  213841,  8, }, // Australia/Adelaide
    {  669, 1619,  213849,  8, }, // Australia/Brisbane
    {  669, 1783,  213857, 11, }, // Australia/Broken_Hill
    {  669, 1567,  213868,  6, }, // Australia/Darwin
    {  669, 3539,  213874,  5, }, // Australia/Eucla
    {  669, 1500,  213879,  6, }, // Australia/Hobart
    {  669, 8716,  213885,  9, }, // Australia/Lindeman
    {  669, 1531,  213894,  9, }, // Australia/Lord_Howe
    {  669, 1711,  213903,  8, }, // Australia/Melbourne
    {  669, 1746,  213911,  4, }, // Australia/Perth
    {  669, 1447,  213915,  5, }, // Australia/Sydney
    {  669, 6390,  213920,  9, }, // Europe/Amsterdam
    {  669, 6149,  213929,  7, }, // Europe/Andorra
    {  669, 3522,  213936,  9, }, // Europe/Astrakhan
    {  669, 6577,  213945,  5, }, // Europe/Athens
    {  669, 6285,  213950,  8, }, // Europe/Belgrade
    {  669, 4659,  213958,  6, }, // Europe/Berlin
    {  669, 6437,  213964, 10, }, // Europe/Bratislava
    {  669, 6178,  213974,  8, }, // Europe/Brussels
    {  669, 3830,  213982,  8, }, // Europe/Bucharest
    {  669, 3708,  213990,  8, }, // Europe/Budapest
    {  669, 8751,  213998,  8, }, // Europe/Busingen
    {  669, 2475,  214006,  7, }, // Europe/Chisinau
    {  669, 6238,  214013, 10, }, // Europe/Copenhagen
    {  669, 2260,  214023,  5, }, // Europe/Dublin
    {  669, 6256,  214028,  9, }, // Europe/Gibraltar
    {  669, 8767,  214037,  6, }, // Europe/Guernsey
    {  669, 6561,  214043,  8, }, // Europe/Helsinki
    {  669, 8783,  214051, 11, }, // Europe/Isle_of_Man
    {  669, 1097,  214062,  8, }, // Europe/Istanbul
    {  669, 8802,  214070,  5, }, // Europe/Jersey
    {  669, 3881,  214075, 11, }, // Europe/Kaliningrad
    {  669, 8924,  214086,  5, }, // Europe/Kirov
    {  669, 2419,  214091,  4, }, // Europe/Kyiv
    {  669, 3078,  214095,  6, }, // Europe/Lisbon
    {  669, 6455,  214101,  9, }, // Europe/Ljubljana
    {  669, 2393,  214110,  4, }, // Europe/London
    {  669, 6314,  214114,  8, }, // Europe/Luxembourg
    {  669, 4794,  214122,  6, }, // Europe/Madrid
    {  669, 6346,  214128,  5, }, // Europe/Malta
    {  669, 6531,  214133,  8, }, // Europe/Mariehamn
    {  669, 3595,  214141,  5, }, // Europe/Minsk
    {  669, 6359,  214146,  6, }, // Europe/Monaco
    {  669, 3443,  214152,  5, }, // Europe/Moscow
    {  669, 6407,  214157,  4, }, // Europe/Oslo
    {  669, 4186,  214161,  5, }, // Europe/Paris
    {  669, 6373,  214166,  9, }, // Europe/Podgorica
    {  669, 6224,  214175,  4, }, // Europe/Prague
    {  669, 8816,  214179,  4, }, // Europe/Riga
    {  669, 6273,   25538,  3, }, // Europe/Rome
    {  669, 4233,  214183,  6, }, // Europe/Samara
    {  669, 6419,  214189, 10, }, // Europe/San_Marino
    {  669, 6194,  214199,  8, }, // Europe/Sarajevo
    {  669, 4353,  214207,  7, }, // Europe/Saratov
    {  669, 8828,  214214, 10, }, // Europe/Simferopol
    {  669, 6332,  214224,  5, }, // Europe/Skopje
    {  669, 6548,  214229,  6, }, // Europe/Sofia
    {  669, 6472,  214235,  7, }, // Europe/Stockholm
    {  669, 8846,  214242,  5, }, // Europe/Tallinn
    {  669, 6135,  214247,  6, }, // Europe/Tirane
    {  669, 8861,  214253,  9, }, // Europe/Ulyanovsk
    {  669, 6301,  214262,  5, }, // Europe/Vaduz
    {  669, 6516,  214267,  7, }, // Europe/Vatican
    {  669, 6164,  214274,  6, }, // Europe/Vienna
    {  669, 8878,  214280,  7, }, // Europe/Vilnius
    {  669, 4629,  214287,  9, }, // Europe/Volgograd
    {  669, 3055,  214296,  5, }, // Europe/Warsaw
    {  669, 6210,  214301, 11, }, // Europe/Zagreb
    {  669, 6489,  214312,  5, }, // Europe/Zurich
    {  669, 4980,  214317,  9, }, // Indian/Antananarivo
    {  669, 6910,  214326,  6, }, // Indian/Chagos
    {  669, 5979,  214332,  8, }, // Indian/Christmas
    {  669, 5996,  214340,  5, }, // Indian/Cocos
    {  669, 4931,  214345,  6, }, // Indian/Comoro
    {  669, 6624,  214351,  9, }, // Indian/Kerguelen
    {  669, 7223,  214360,  4, }, // Indian/Mahe
    {  669, 7013,  214364,  8, }, // Indian/Maldives
    {  669, 3971,  214372,  9, }, // Indian/Mauritius
    {  669, 5000,  214381,  5, }, // Indian/Mayotte
    {  669, 7174,  214386,  7, }, // Indian/Reunion
    {  669, 4324,  214393,  4, }, // Pacific/Apia
    {  669,  932,  214397,  6, }, // Pacific/Auckland
    {  669, 3608,  214403, 10, }, // Pacific/Bougainville
    {  669, 2863,  214413,  5, }, // Pacific/Chatham
    {  669, 3022,  214418,  3, }, // Pacific/Chuuk
    {  669, 2201,  214421,  5, }, // Pacific/Easter
    {  669, 7323,  214426,  5, }, // Pacific/Efate
    {  669, 7290,  214431,  7, }, // Pacific/Fakaofo
    {  669, 3804,  214438,  4, }, // Pacific/Fiji
    {  669, 7306,  214442,  8, }, // Pacific/Funafuti
    {  669, 6641,  214450,  9, }, // Pacific/Galapagos
    {  669, 6659,  214459,  7, }, // Pacific/Gambier
    {  669, 3724,  214466, 11, }, // Pacific/Guadalcanal
    {  669, 5935,  212593,  4, }, // Pacific/Guam
    {  669, 2897,  214477,  6, }, // Pacific/Kanton
    {  669, 3900,  214483, 10, }, // Pacific/Kiritimati
    {  669, 6982,  214493,  6, }, // Pacific/Kosrae
    {  669, 2708,  214499,  9, }, // Pacific/Kwajalein
    {  669, 4779,  214508,  6, }, // Pacific/Majuro
    {  669, 3953,  214514,  8, }, // Pacific/Marquesas
    {  669, 8909,  214522,  6, }, // Pacific/Midway
    {  669, 7047,  214528,  5, }, // Pacific/Nauru
    {  669, 7095,  214533,  4, }, // Pacific/Niue
    {  669, 4070,  214537,  6, }, // Pacific/Norfolk
    {  669, 7061,  214543,  5, }, // Pacific/Noumea
    {  669, 2991,  214548, 11, }, // Pacific/Pago_Pago
    {  669, 7118,  214559,  5, }, // Pacific/Palau
    {  669, 7157,  214564,  7, }, // Pacific/Pitcairn
    {  669, 2961,  214571,  6, }, // Pacific/Pohnpei
    {  669, 4709,  214577, 12, }, // Pacific/Port_Moresby
    {  669, 6009,  214589,  9, }, // Pacific/Rarotonga
    {  669, 5948,  214598,  6, }, // Pacific/Saipan
    {  669, 7275,  214604,  6, }, // Pacific/Tahiti
    {  669, 6675,  214610,  6, }, // Pacific/Tarawa
    {  669, 4485,  214616,  9, }, // Pacific/Tongatapu
    {  669, 7355,  214625,  3, }, // Pacific/Wake
    {  669, 7368,  214628,  6, }, // Pacific/Wallis
    {  670,   14,       0,  6, }, // Africa/Asmara Kuvi/Devanagari/India
    {  670, 4337,       6,  8, }, // Africa/Sao_Tome
    {  670, 4154,      14,  8, }, // America/Asuncion
    {  670,  237,      22,  8, }, // America/Atikokan
    {  670, 7586,      30, 17, }, // America/Bahia_Banderas
    {  670, 3770,      47,  6, }, // America/Cancun
    {  670, 5640,      66,  7, }, // America/Curacao
    {  670,  481,      73, 13, }, // America/Indiana/Knox
    {  670, 7924,      86, 16, }, // America/Indiana/Marengo
    {  670, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  670, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  670, 8001,     139, 14, }, // America/Indiana/Vevay
    {  670, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  670, 8049,     171, 16, }, // America/Indiana/Winamac
    {  670, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  670, 5849,     207, 22, }, // America/Lower_Princes
    {  670, 8183,     229,  6, }, // America/Merida
    {  670, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  670, 8292,     255, 20, }, // America/North_Dakota/Center
    {  670, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  670,  393,     298,  4, }, // America/Nuuk
    {  670, 6862,     302, 16, }, // America/Scoresbysund
    {  670, 8471,     318, 14, }, // America/St_Barthelemy
    {  670, 2045,     332, 10, }, // America/St_Johns
    {  670, 5780,     342,  9, }, // America/St_Kitts
    {  670, 5797,     351,  9, }, // America/St_Lucia
    {  670,  855,     360, 10, }, // America/St_Thomas
    {  670, 5830,     370, 11, }, // America/St_Vincent
    {  670, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  670, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  670, 1152,     413,  9, }, // Asia/Kathmandu
    {  670,  992,     422,  7, }, // Asia/Kolkata
    {  670, 1178,     429,  5, }, // Asia/Macau
    {  670, 1202,     442,  6, }, // Asia/Yangon
    {  670, 1379,     448,  5, }, // Atlantic/Faroe
    {  670, 6784,     453, 10, }, // Atlantic/St_Helena
    {  670, 2419,     463,  4, }, // Europe/Kyiv
    {  670, 7174,     467,  7, }, // Indian/Reunion
    {  670, 3022,     474,  5, }, // Pacific/Chuuk
    {  670, 2961,     479,  7, }, // Pacific/Pohnpei
    {  671,   14,       0,  6, }, // Africa/Asmara Kuvi/Odia/India
    {  671, 4337,       6,  8, }, // Africa/Sao_Tome
    {  671, 4154,      14,  8, }, // America/Asuncion
    {  671,  237,      22,  8, }, // America/Atikokan
    {  671, 7586,      30, 17, }, // America/Bahia_Banderas
    {  671, 3770,      47,  6, }, // America/Cancun
    {  671, 5640,      66,  7, }, // America/Curacao
    {  671,  481,      73, 13, }, // America/Indiana/Knox
    {  671, 7924,      86, 16, }, // America/Indiana/Marengo
    {  671, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  671, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  671, 8001,     139, 14, }, // America/Indiana/Vevay
    {  671, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  671, 8049,     171, 16, }, // America/Indiana/Winamac
    {  671, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  671, 5849,     207, 22, }, // America/Lower_Princes
    {  671, 8183,     229,  6, }, // America/Merida
    {  671, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  671, 8292,     255, 20, }, // America/North_Dakota/Center
    {  671, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  671,  393,     298,  4, }, // America/Nuuk
    {  671, 6862,     302, 16, }, // America/Scoresbysund
    {  671, 8471,     318, 14, }, // America/St_Barthelemy
    {  671, 2045,     332, 10, }, // America/St_Johns
    {  671, 5780,     342,  9, }, // America/St_Kitts
    {  671, 5797,     351,  9, }, // America/St_Lucia
    {  671,  855,     360, 10, }, // America/St_Thomas
    {  671, 5830,     370, 11, }, // America/St_Vincent
    {  671, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  671, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  671, 1152,     413,  9, }, // Asia/Kathmandu
    {  671,  992,     422,  7, }, // Asia/Kolkata
    {  671, 1178,     429,  5, }, // Asia/Macau
    {  671, 1202,     442,  6, }, // Asia/Yangon
    {  671, 1379,     448,  5, }, // Atlantic/Faroe
    {  671, 6784,     453, 10, }, // Atlantic/St_Helena
    {  671, 2419,     463,  4, }, // Europe/Kyiv
    {  671, 7174,     467,  7, }, // Indian/Reunion
    {  671, 3022,     474,  5, }, // Pacific/Chuuk
    {  671, 2961,     479,  7, }, // Pacific/Pohnpei
    {  672,   14,       0,  6, }, // Africa/Asmara Kuvi/Telugu/India
    {  672, 4337,       6,  8, }, // Africa/Sao_Tome
    {  672, 4154,      14,  8, }, // America/Asuncion
    {  672,  237,      22,  8, }, // America/Atikokan
    {  672, 7586,      30, 17, }, // America/Bahia_Banderas
    {  672, 3770,      47,  6, }, // America/Cancun
    {  672, 5640,      66,  7, }, // America/Curacao
    {  672,  481,      73, 13, }, // America/Indiana/Knox
    {  672, 7924,      86, 16, }, // America/Indiana/Marengo
    {  672, 7948,     102, 19, }, // America/Indiana/Petersburg
    {  672, 7975,     121, 18, }, // America/Indiana/Tell_City
    {  672, 8001,     139, 14, }, // America/Indiana/Vevay
    {  672, 8023,     153, 18, }, // America/Indiana/Vincennes
    {  672, 8049,     171, 16, }, // America/Indiana/Winamac
    {  672, 8088,     187, 20, }, // America/Kentucky/Monticello
    {  672, 5849,     207, 22, }, // America/Lower_Princes
    {  672, 8183,     229,  6, }, // America/Merida
    {  672, 8264,     235, 20, }, // America/North_Dakota/Beulah
    {  672, 8292,     255, 20, }, // America/North_Dakota/Center
    {  672, 8320,     275, 23, }, // America/North_Dakota/New_Salem
    {  672,  393,     298,  4, }, // America/Nuuk
    {  672, 6862,     302, 16, }, // America/Scoresbysund
    {  672, 8471,     318, 14, }, // America/St_Barthelemy
    {  672, 2045,     332, 10, }, // America/St_Johns
    {  672, 5780,     342,  9, }, // America/St_Kitts
    {  672, 5797,     351,  9, }, // America/St_Lucia
    {  672,  855,     360, 10, }, // America/St_Thomas
    {  672, 5830,     370, 11, }, // America/St_Vincent
    {  672, 6066,     381, 16, }, // Antarctica/DumontDUrville
    {  672, 1226,     397, 11, }, // Asia/Ho_Chi_Minh
    {  672, 1152,     413,  9, }, // Asia/Kathmandu
    {  672,  992,     422,  7, }, // Asia/Kolkata
    {  672, 1178,     429,  5, }, // Asia/Macau
    {  672, 1202,     442,  6, }, // Asia/Yangon
    {  672, 1379,     448,  5, }, // Atlantic/Faroe
    {  672, 6784,     453, 10, }, // Atlantic/St_Helena
    {  672, 2419,     463,  4, }, // Europe/Kyiv
    {  672, 7174,     467,  7, }, // Indian/Reunion
    {  672, 3022,     474,  5, }, // Pacific/Chuuk
    {  672, 2961,     479,  7, }, // Pacific/Pohnpei
}; // Exemplar city table

// Sorted by locale index, then iana name
static inline constexpr LocaleZoneNames localeZoneNameTable[] = {
    // locInd, ianaInd, (lngGen, lngStd, lngDst, srtGen, srtStd, srtDst){ind, sz}
    {    0, 2347,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC C/AnyScript/AnyTerritory
    {    0, 2260,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {    0, 2393,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {    0, 2929,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {    1, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Abkhazian/Cyrillic/Georgia
    {    1, 2260,       0,       0,      64,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {    1, 2393,       0,       0,      88,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {    2, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Afar/Latin/Ethiopia
    {    5, 2347,       0,     109,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Afrikaans/Latin/South Africa
    {    5, 2260,       0,       0,     138,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {    5, 2393,       0,       0,     156,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/London
    {    7, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Aghem/Latin/Cameroon
    {    8, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Akan/Latin/Ghana
    {    9, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Akoose/Latin/Cameroon
    {   10, 2347,       0,     171,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Albanian/Latin/Albania
    {   10, 2260,       0,       0,     198,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {   10, 2393,       0,       0,     223,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {   13, 2347,       0,     243,       0,    0,    0,    0,  0, 16,  0,  0,  3,  0, }, // Etc/UTC Amharic/Ethiopic/Ethiopia
    {   13, 2260,       0,       0,     259,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {   13, 2393,       0,       0,     279,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {   14, 2347,       0,     298,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Arabic/Arabic/Egypt
    {   14, 2260,       0,       0,     320,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {   14, 2393,       0,       0,     340,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {   42, 2347,       0,     361,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Aragonese/Latin/Spain
    {   43, 2347,       0,     388,       0,    0,    0,    0,  0, 34,  0,  0,  3,  0, }, // Etc/UTC Armenian/Armenian/Armenia
    {   43, 2260,       0,       0,     422,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/Dublin
    {   43, 2393,       0,       0,     449,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {   44, 2347,       0,     476,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Assamese/Bangla/India
    {   44, 2260,       0,       0,     498,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {   44, 2393,       0,       0,     512,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {   45, 2347,       0,     537,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Asturian/Latin/Spain
    {   45, 2260,       0,       0,     562,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {   45, 2393,       0,       0,     585,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {   45, 2929,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {   46, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Asu/Latin/Tanzania
    {   47, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Atsam/Latin/Nigeria
    {   48, 2347,       0,     608,       0,    0,    0,    0,  0, 36,  0,  0,  3,  0, }, // Etc/UTC Azerbaijani/Latin/Azerbaijan
    {   48, 2260,       0,       0,     644,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {   48, 2393,       0,       0,     663,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {   49, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Azerbaijani/Arabic/Iran
    {   52, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Azerbaijani/Cyrillic/Azerbaijan
    {   53, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bafia/Latin/Cameroon
    {   54, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bambara/Latin/Mali
    {   56, 2347,       0,     682,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Bangla/Bangla/Bangladesh
    {   56, 2260,       0,       0,     709,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {   56, 2393,       0,       0,     724,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {   58, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Basaa/Latin/Cameroon
    {   59, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bashkir/Cyrillic/Russia
    {   60, 2347,       0,     749,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Basque/Latin/Spain
    {   60, 2260,       0,       0,     776,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {   60, 2393,       0,       0,     801,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {   60, 2929,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {   61, 2347,       0,     822,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Belarusian/Cyrillic/Belarus
    {   61, 2260,       0,       0,     851,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   61, 2393,       0,       0,     875,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {   62, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bemba/Latin/Zambia
    {   63, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bena/Latin/Tanzania
    {   64, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Bhojpuri/Devanagari/India
    {   65, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Blin/Ethiopic/Eritrea
    {   66, 2347,       0,     894,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Bodo/Devanagari/India
    {   66, 2260,       0,       0,     916,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {   66, 2393,       0,       0,     932,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {   67, 2347,       0,     950,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Bosnian/Latin/Bosnia and Herzegovina
    {   67, 2260,       0,       0,     979,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   67, 2393,       0,       0,    1003,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {   68, 2347,       0,    1027,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Bosnian/Cyrillic/Bosnia and Herzegovina
    {   68, 2260,       0,       0,    1059,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   68, 2393,       0,       0,    1083,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {   69, 2347,       0,    1106,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Breton/Latin/France
    {   69, 2260,       0,       0,    1132,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {   69, 2393,       0,       0,    1152,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {   70, 2347,       0,    1172,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Bulgarian/Cyrillic/Bulgaria
    {   70, 2260,       0,       0,    1202,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {   70, 2393,       0,       0,    1228,    0,    0,    0,  0,  0, 28,  0,  0,  0, }, // Europe/London
    {   71, 2347,       0,    1256,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Burmese/Myanmar/Myanmar
    {   71, 2260,       0,       0,    1285,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   71, 2393,       0,       0,    1309,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {   72, 2347,       0,    1331,       0,    0,    0,    0,  0,  6,  0,  0,  3,  0, }, // Etc/UTC Cantonese/Traditional Han/Hong Kong
    {   72, 2260,       0,       0,    1337,    0,    0,    0,  0,  0,  7,  0,  0,  0, }, // Europe/Dublin
    {   72, 2393,       0,       0,    1344,    0,    0,    0,  0,  0,  6,  0,  0,  0, }, // Europe/London
    {   73, 2347,       0,    1350,       0,    0,    0,    0,  0,  6,  0,  0,  3,  0, }, // Etc/UTC Cantonese/Simplified Han/China
    {   73, 2260,       0,       0,    1356,    0,    0,    0,  0,  0,  7,  0,  0,  0, }, // Europe/Dublin
    {   73, 2393,       0,       0,    1363,    0,    0,    0,  0,  0,  6,  0,  0,  0, }, // Europe/London
    {   74, 2347,       0,    1369,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Catalan/Latin/Spain
    {   74, 2260,       0,       0,    1394,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {   74, 2393,       0,       0,    1418,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {   78, 2347,       0,    1440,       0,    0,    0,    0,  0, 34,  0,  0,  3,  0, }, // Etc/UTC Cebuano/Latin/Philippines
    {   78, 2260,       0,       0,    1474,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {   78, 2393,       0,       0,    1497,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {   79, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Central Atlas Tamazight/Latin/Morocco
    {   80, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Central Kurdish/Arabic/Iraq
    {   82, 2347,       0,    1524,       0,    0,    0,    0,  0, 46,  0,  0,  3,  0, }, // Etc/UTC Chakma/Chakma/Bangladesh
    {   82, 2260,       0,       0,    1570,    0,    0,    0,  0,  0, 36,  0,  0,  0, }, // Europe/Dublin
    {   82, 2393,       0,       0,    1606,    0,    0,    0,  0,  0, 42,  0,  0,  0, }, // Europe/London
    {   84, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Chechen/Cyrillic/Russia
    {   84, 2260,       0,       0,    1648,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {   84, 2393,       0,       0,    1669,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {   85, 2347,       0,    1691,       0,    0,    0,    0,  0, 14,  0,  0,  3,  0, }, // Etc/UTC Cherokee/Cherokee/United States
    {   85, 2260,       0,       0,    1705,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {   85, 2393,       0,       0,    1720,    0,    0,    0,  0,  0, 12,  0,  0,  0, }, // Europe/London
    {   85, 2929,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {   86, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Chickasaw/Latin/United States
    {   87, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Chiga/Latin/Uganda
    {   88, 2347,       0,    1350,       0,    0,    0,    0,  0,  5,  0,  0,  3,  0, }, // Etc/UTC Chinese/Simplified Han/China
    {   88, 2260,       0,       0,    1356,    0,    0,    0,  0,  0,  7,  0,  0,  0, }, // Europe/Dublin
    {   88, 2393,       0,       0,    1363,    0,    0,    0,  0,  0,  6,  0,  0,  0, }, // Europe/London
    {   94, 2347,       0,    1732,       0,    0,    0,    0,  0,  6,  0,  0,  3,  0, }, // Etc/UTC Chinese/Traditional Han/Taiwan
    {   94, 2260,       0,       0,    1337,    0,    0,    0,  0,  0,  7,  0,  0,  0, }, // Europe/Dublin
    {   94, 2393,       0,       0,    1344,    0,    0,    0,  0,  0,  6,  0,  0,  0, }, // Europe/London
    {   95, 2347,       0,    1738,       0,    0,    0,    0,  0, 37,  0,  0,  3,  0, }, // Etc/UTC Church/Cyrillic/Russia
    {   96, 2347,       0,    1775,       0,    0,    0,    0,  0, 35,  0,  0,  3,  0, }, // Etc/UTC Chuvash/Cyrillic/Russia
    {   96, 2260,       0,       0,    1810,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {   96, 2393,       0,       0,    1835,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {   97, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Colognian/Latin/Germany
    {   97, 2260,       0,       0,    1857,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {   97, 2393,       0,       0,    1879,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/London
    {   98, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Cornish/Latin/United Kingdom
    {   99, 2347,       0,    1909,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Corsican/Latin/France
    {  100, 2347,       0,    1936,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Croatian/Latin/Croatia
    {  100, 2260,       0,       0,    1965,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  100, 2393,       0,       0,    1989,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  102, 2347,       0,    2013,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Czech/Latin/Czechia
    {  102, 2260,       0,       0,    2037,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  102, 2393,       0,       0,    2052,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  102, 2929,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  103, 2347,       0,    2069,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Danish/Latin/Denmark
    {  103, 2260,       0,       0,    2093,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  103, 2393,       0,       0,    2107,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  105, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Divehi/Thaana/Maldives
    {  106, 2347,       0,    2124,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Dogri/Devanagari/India
    {  106, 2260,       0,       0,    2141,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  106, 2393,       0,       0,    2156,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  107, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Duala/Latin/Cameroon
    {  108, 2347,       0,    2180,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Dutch/Latin/Netherlands
    {  108, 2260,       0,       0,    2205,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  108, 2393,       0,       0,    2224,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/London
    {  108, 2929,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  115, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Dzongkha/Tibetan/Bhutan
    {  115, 2260,       0,       0,    2240,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  115, 2393,       0,       0,    2263,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  116, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Embu/Latin/Kenya
    {  117, 2347,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC English/Latin/United States
    {  117, 2260,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  117, 2393,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  117, 2929,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  118, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC English/Deseret/United States
    {  120, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Anguilla
    {  121, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Antigua and Barbuda
    {  122, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Australia
    {  123, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Austria
    {  124, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Bahamas
    {  125, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Barbados
    {  126, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Belgium
    {  127, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Belize
    {  128, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Bermuda
    {  129, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Botswana
    {  130, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/British Indian Ocean Territory
    {  131, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/British Virgin Islands
    {  133, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cameroon
    {  135, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cayman Islands
    {  136, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Christmas Island
    {  137, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cocos Islands
    {  138, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cook Islands
    {  139, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Cyprus
    {  140, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Denmark
    {  141, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Diego Garcia
    {  142, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Dominica
    {  143, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Eritrea
    {  144, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Eswatini
    {  145, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Europe
    {  146, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Falkland Islands
    {  147, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Fiji
    {  148, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Finland
    {  149, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Gambia
    {  150, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Germany
    {  151, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Ghana
    {  152, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Gibraltar
    {  153, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Grenada
    {  155, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Guernsey
    {  156, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Guyana
    {  157, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Hong Kong
    {  158, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/India
    {  159, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Indonesia
    {  160, 2260,       0,       0,      26,    0,    0,   12,  0,  0, 19,  0,  0,  3, }, // Europe/Dublin English/Latin/Ireland
    {  160, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  161, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Isle of Man
    {  162, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Israel
    {  163, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Jamaica
    {  164, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Jersey
    {  165, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Kenya
    {  166, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Kiribati
    {  167, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Lesotho
    {  168, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Liberia
    {  169, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Macao
    {  170, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Madagascar
    {  171, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Malawi
    {  172, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Malaysia
    {  173, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Maldives
    {  174, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Malta
    {  175, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Marshall Islands
    {  176, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Mauritius
    {  177, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Micronesia
    {  178, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Montserrat
    {  179, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Namibia
    {  180, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Nauru
    {  181, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Netherlands
    {  182, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/New Zealand
    {  183, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Nigeria
    {  184, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Niue
    {  185, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Norfolk Island
    {  186, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Northern Mariana Islands
    {  187, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Pakistan
    {  188, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Palau
    {  189, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Papua New Guinea
    {  191, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Pitcairn
    {  193, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Rwanda
    {  194, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Saint Helena
    {  195, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Saint Kitts and Nevis
    {  196, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Saint Lucia
    {  197, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Saint Vincent and Grenadines
    {  198, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Samoa
    {  199, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Seychelles
    {  200, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Sierra Leone
    {  201, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Singapore
    {  202, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Sint Maarten
    {  203, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Slovenia
    {  204, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Solomon Islands
    {  205, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/South Africa
    {  206, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/South Sudan
    {  207, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Sudan
    {  208, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Sweden
    {  209, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Switzerland
    {  210, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Tanzania
    {  211, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Tokelau
    {  212, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Tonga
    {  213, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Trinidad and Tobago
    {  214, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Turks and Caicos Islands
    {  215, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Tuvalu
    {  216, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Uganda
    {  217, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/United Arab Emirates
    {  218, 2393,       0,       0,      45,    0,    0,   15,  0,  0, 19,  0,  0,  3, }, // Europe/London English/Latin/United Kingdom
    {  218, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  221, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Vanuatu
    {  222, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/world
    {  223, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Zambia
    {  224, 2929,       0,       0,       0,    9,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu English/Latin/Zimbabwe
    {  225, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC English/Shavian/United Kingdom
    {  226, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Erzya/Cyrillic/Russia
    {  227, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Esperanto/Latin/world
    {  228, 2347,       0,    2287,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Estonian/Latin/Estonia
    {  228, 2260,       0,       0,    2312,    0,    0,    0,  0,  0, 12,  0,  0,  0, }, // Europe/Dublin
    {  228, 2393,       0,       0,    2324,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/London
    {  229, 2347,       0,    2337,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Ewe/Latin/Ghana
    {  229, 2260,       0,       0,    2359,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  229, 2393,       0,       0,    2383,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {  231, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ewondo/Latin/Cameroon
    {  232, 2347,       0,    2410,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Faroese/Latin/Faroe Islands
    {  232, 2260,       0,       0,    2428,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  232, 2393,       0,       0,    2443,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/London
    {  234, 2347,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Filipino/Latin/Philippines
    {  234, 2260,       0,       0,    2459,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/Dublin
    {  234, 2393,       0,       0,    2486,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {  235, 2347,       0,    2513,       0,    0,    0,    0,  0, 13,  0,  0,  3,  0, }, // Etc/UTC Finnish/Latin/Finland
    {  235, 2260,       0,       0,    2526,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  235, 2393,       0,       0,    2543,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  236, 2347,       0,    2562,       0,    0,   18,    0,  0, 25,  0,  0,  2,  0, }, // Etc/UTC French/Latin/France
    {  236, 2260,       0,       0,    2587,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  236, 2393,       0,       0,    2609,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  236, 2929,       0,       0,       0,   20,    3,    6,  0,  0,  0,  2,  3,  3, }, // Pacific/Honolulu
    {  243, 2260,       0,       0,    2632,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin French/Latin/Canada
    {  243, 2393,       0,       0,    2656,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  282, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Friulian/Latin/Italy
    {  283, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Fulah/Latin/Senegal
    {  289, 2347,       0,    2681,       0,    0,   22,    0,  0, 56,  0,  0,  6,  0, }, // Etc/UTC Fulah/Adlam/Guinea
    {  289, 2260,       0,       0,    2737,    0,    0,    0,  0,  0, 60,  0,  0,  0, }, // Europe/Dublin
    {  289, 2393,       0,       0,    2797,    0,    0,    0,  0,  0, 58,  0,  0,  0, }, // Europe/London
    {  289, 2929,       0,       0,       0,   28,   28,   34,  0,  0,  0,  6,  6,  6, }, // Pacific/Honolulu
    {  307, 2347,       0,    2855,       0,    0,    0,    0,  0, 34,  0,  0,  3,  0, }, // Etc/UTC Gaelic/Latin/United Kingdom
    {  307, 2260,       0,       0,    2889,    0,    0,   40,  0,  0, 19,  0,  0, 12, }, // Europe/Dublin
    {  307, 2393,       0,       0,    2908,    0,    0,   52,  0,  0, 25,  0,  0,  3, }, // Europe/London
    {  307, 2929,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  308, 2347,       0,    2933,       0,    0,    0,    0,  0, 36,  0,  0,  3,  0, }, // Etc/UTC Ga/Latin/Ghana
    {  308, 2260,       0,       0,    2969,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/Dublin
    {  308, 2393,       0,       0,    2996,    0,    0,    0,  0,  0, 42,  0,  0,  0, }, // Europe/London
    {  309, 2347,       0,    3038,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Galician/Latin/Spain
    {  309, 2260,       0,       0,    3063,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  309, 2393,       0,       0,    3086,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  310, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ganda/Latin/Uganda
    {  311, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Geez/Ethiopic/Ethiopia
    {  313, 2347,       0,    3109,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Georgian/Georgian/Georgia
    {  313, 2260,       0,       0,    3135,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  313, 2393,       0,       0,    3160,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  314, 2347,       0,    3183,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC German/Latin/Germany
    {  314, 2260,       0,       0,    3204,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  314, 2393,       0,       0,    3222,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  321, 2347,       0,    3242,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Greek/Greek/Greece
    {  321, 2260,       0,       0,    3268,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  321, 2393,       0,       0,    3291,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  323, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Guarani/Latin/Paraguay
    {  324, 2347,       0,    3311,       0,    0,    0,    0,  0, 20,  0,  0,  3,  0, }, // Etc/UTC Gujarati/Gujarati/India
    {  324, 2260,       0,       0,    3331,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  324, 2393,       0,       0,    3345,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  325, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Gusii/Latin/Kenya
    {  326, 2347,       0,    3364,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Hausa/Latin/Nigeria
    {  326, 2260,       0,       0,    3387,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  326, 2393,       0,       0,    3410,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {  327, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Hausa/Arabic/Nigeria
    {  328, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Hausa/Arabic/Sudan
    {  331, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Hawaiian/Latin/United States
    {  331, 2929,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  332, 2347,       0,    3437,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Hebrew/Hebrew/Israel
    {  332, 2260,       0,       0,    3456,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  332, 2393,       0,       0,    3471,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/London
    {  333, 2347,       0,    3487,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Hindi/Devanagari/India
    {  333, 2260,       0,       0,    3506,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  333, 2393,       0,       0,    3520,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  333, 2929,       0,       0,       0,   55,   55,    3,  0,  0,  0,  6,  6,  3, }, // Pacific/Honolulu
    {  334, 2347,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Hindi/Latin/India
    {  334, 2260,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  334, 2393,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  334, 2929,       0,       0,       0,    3,    9,    9,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  335, 2347,       0,    3544,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Hungarian/Latin/Hungary
    {  335, 2260,       0,       0,    3563,    0,    0,    0,  0,  0, 12,  0,  0,  0, }, // Europe/Dublin
    {  335, 2393,       0,       0,    3575,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/London
    {  336, 2347,       0,    3589,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Icelandic/Latin/Iceland
    {  336, 2260,       0,       0,    3615,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  336, 2393,       0,       0,    3634,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  337, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ido/Latin/world
    {  338, 2347,       0,    3655,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Igbo/Latin/Nigeria
    {  338, 2260,       0,       0,    3674,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  338, 2393,       0,       0,    3694,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  339, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Inari Sami/Latin/Finland
    {  340, 2347,       0,    3714,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Indonesian/Latin/Indonesia
    {  340, 2260,       0,       0,    3743,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  340, 2393,       0,       0,    3765,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  341, 2347,       0,    3790,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Interlingua/Latin/world
    {  341, 2260,       0,       0,    3818,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  341, 2393,       0,       0,    3839,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  342, 2347,       0,    3860,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Interlingue/Latin/Estonia
    {  342, 2260,       0,       0,    3886,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  342, 2393,       0,       0,    3911,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  343, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Inuktitut/Canadian Aboriginal/Canada
    {  345, 2347,       0,    3934,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Irish/Latin/Ireland
    {  345, 2260,       0,       0,    3952,    0,    0,   61,  0,  0, 27,  0,  0,  3, }, // Europe/Dublin
    {  345, 2393,       0,       0,    3979,    0,    0,   64,  0,  0, 25,  0,  0,  3, }, // Europe/London
    {  347, 2347,       0,    4004,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Italian/Latin/Italy
    {  347, 2260,       0,       0,    4031,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  347, 2393,       0,       0,    4054,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  351, 2347,       0,    4080,       0,    0,    0,    0,  0,  5,  0,  0,  3,  0, }, // Etc/UTC Japanese/Japanese/Japan
    {  351, 2260,       0,       0,    4085,    0,    0,    0,  0,  0,  9,  0,  0,  0, }, // Europe/Dublin
    {  351, 2393,       0,       0,    4094,    0,    0,    0,  0,  0,  5,  0,  0,  0, }, // Europe/London
    {  352, 2347,       0,    4099,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Javanese/Latin/Indonesia
    {  352, 2260,       0,       0,    4128,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  352, 2393,       0,       0,    4150,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  353, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Jju/Latin/Nigeria
    {  354, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Jola-Fonyi/Latin/Senegal
    {  355, 2347,       0,    4170,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kabuverdianu/Latin/Cape Verde
    {  355, 2260,       0,       0,    4193,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  355, 2393,       0,       0,    4214,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  356, 2347,       0,    4236,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kabyle/Latin/Algeria
    {  356, 2260,       0,       0,    4259,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  356, 2393,       0,       0,    4280,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  357, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kako/Latin/Cameroon
    {  358, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kalaallisut/Latin/Greenland
    {  359, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kalenjin/Latin/Kenya
    {  360, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kamba/Latin/Kenya
    {  361, 2347,       0,    4304,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Kannada/Kannada/India
    {  361, 2260,       0,       0,    4325,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  361, 2393,       0,       0,    4343,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  362, 2347,       0,    4362,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kashmiri/Arabic/India
    {  362, 2260,       0,       0,    4385,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  362, 2393,       0,       0,    4403,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  363, 2347,       0,    4422,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kashmiri/Devanagari/India
    {  364, 2347,       0,    4445,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Kazakh/Cyrillic/Kazakhstan
    {  364, 2260,       0,       0,    4477,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {  364, 2393,       0,       0,    4503,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  365, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kenyang/Latin/Cameroon
    {  366, 2347,       0,    4527,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Khmer/Khmer/Cambodia
    {  366, 2260,       0,       0,    4549,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  366, 2393,       0,       0,    4574,    0,    0,    0,  0,  0, 34,  0,  0,  0, }, // Europe/London
    {  367, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kiche/Latin/Guatemala
    {  368, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kikuyu/Latin/Kenya
    {  369, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kinyarwanda/Latin/Rwanda
    {  370, 2347,       0,    4608,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Konkani/Devanagari/India
    {  370, 2260,       0,       0,    4627,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  370, 2393,       0,       0,    4645,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  371, 2347,       0,    4664,       0,    0,    0,    0,  0,  6,  0,  0,  3,  0, }, // Etc/UTC Korean/Korean/South Korea
    {  371, 2260,       0,       0,    4670,    0,    0,    0,  0,  0,  8,  0,  0,  0, }, // Europe/Dublin
    {  371, 2393,       0,       0,    4678,    0,    0,    0,  0,  0,  9,  0,  0,  0, }, // Europe/London
    {  374, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Koyraboro Senni/Latin/Mali
    {  375, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Koyra Chiini/Latin/Mali
    {  376, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kpelle/Latin/Liberia
    {  378, 2347,       0,    4687,       0,    0,   67,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Kurdish/Latin/Turkey
    {  378, 2260,       0,       0,    4711,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {  378, 2393,       0,       0,    4737,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {  379, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kwasio/Latin/Cameroon
    {  380, 2347,       0,    4764,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Kyrgyz/Cyrillic/Kyrgyzstan
    {  380, 2260,       0,       0,    4789,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  380, 2393,       0,       0,    4811,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  381, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Lakota/Latin/United States
    {  382, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Langi/Latin/Tanzania
    {  383, 2347,       0,    4834,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Lao/Lao/Laos
    {  383, 2260,       0,       0,    4852,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  383, 2393,       0,       0,    4876,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  384, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Latin/Latin/Vatican City
    {  385, 2347,       0,    4900,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Latvian/Latin/Latvia
    {  385, 2260,       0,       0,    4931,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  385, 2393,       0,       0,    4950,    0,    0,    0,  0,  0, 28,  0,  0,  0, }, // Europe/London
    {  386, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Lingala/Latin/Congo - Kinshasa
    {  390, 2347,       0,    4978,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Lithuanian/Latin/Lithuania
    {  390, 2260,       0,       0,    5006,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  390, 2393,       0,       0,    5028,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  391, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Lojban/Latin/world
    {  392, 2347,       0,    5053,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Lower Sorbian/Latin/Germany
    {  392, 2260,       0,       0,    5079,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  392, 2393,       0,       0,    5098,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  393, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Low German/Latin/Germany
    {  395, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Luba-Katanga/Latin/Congo - Kinshasa
    {  396, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Lule Sami/Latin/Sweden
    {  398, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Luo/Latin/Kenya
    {  399, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Luxembourgish/Latin/Luxembourg
    {  399, 2260,       0,       0,    5119,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  399, 2393,       0,       0,    5136,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  400, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Luyia/Latin/Kenya
    {  401, 2347,       0,    5155,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Macedonian/Cyrillic/Macedonia
    {  401, 2260,       0,       0,    5185,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  401, 2393,       0,       0,    5207,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  402, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Machame/Latin/Tanzania
    {  403, 2347,       0,    3487,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Maithili/Devanagari/India
    {  404, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Makhuwa-Meetto/Latin/Mozambique
    {  405, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Makonde/Latin/Tanzania
    {  406, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Malagasy/Latin/Madagascar
    {  407, 2347,       0,    5228,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Malayalam/Malayalam/India
    {  407, 2260,       0,       0,    5257,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  407, 2393,       0,       0,    5280,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/London
    {  408, 2347,       0,    5307,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Malay/Latin/Malaysia
    {  408, 2260,       0,       0,    5330,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  408, 2393,       0,       0,    5350,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  409, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Malay/Arabic/Brunei
    {  410, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Malay/Arabic/Malaysia
    {  414, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Maltese/Latin/Malta
    {  415, 2347,       0,    5375,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Manipuri/Bangla/India
    {  416, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Manipuri/Meitei Mayek/India
    {  417, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Manx/Latin/Isle of Man
    {  418, 2347,       0,    5404,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Maori/Latin/New Zealand
    {  418, 2260,       0,       0,    5421,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  418, 2393,       0,       0,    5440,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  419, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mapuche/Latin/Chile
    {  420, 2347,       0,    4608,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Marathi/Devanagari/India
    {  420, 2260,       0,       0,    5459,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {  420, 2393,       0,       0,    5475,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  421, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Masai/Latin/Kenya
    {  423, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mazanderani/Arabic/Iran
    {  424, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Meru/Latin/Kenya
    {  425, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Meta/Latin/Cameroon
    {  426, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mohawk/Latin/Canada
    {  427, 2347,       0,    5494,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Mongolian/Cyrillic/Mongolia
    {  427, 2260,       0,       0,    5523,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  427, 2393,       0,       0,    5544,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {  428, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mongolian/Mongolian/China
    {  429, 2347,       0,    5562,       0,    0,    0,    0,  0, 33,  0,  0,  3,  0, }, // Etc/UTC Mongolian/Mongolian/Mongolia
    {  430, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Morisyen/Latin/Mauritius
    {  431, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Mundang/Latin/Cameroon
    {  432, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Muscogee/Latin/United States
    {  433, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Nama/Latin/Namibia
    {  434, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Navajo/Latin/United States
    {  435, 2347,       0,    5595,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Nepali/Devanagari/Nepal
    {  435, 2260,       0,       0,    3506,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  435, 2393,       0,       0,    5612,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  437, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ngiemboon/Latin/Cameroon
    {  438, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ngomba/Latin/Cameroon
    {  439, 2347,       0,    5636,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Nigerian Pidgin/Latin/Nigeria
    {  439, 2260,       0,       0,    5665,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  439, 2393,       0,       0,    5682,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  440, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Nko/Nko/Guinea
    {  441, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Northern Luri/Arabic/Iran
    {  443, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Northern Sami/Latin/Norway
    {  444, 2347,       0,    5705,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Northern Sami/Latin/Finland
    {  444, 2260,       0,       0,    5734,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  444, 2393,       0,       0,    5755,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  446, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Northern Sotho/Latin/South Africa
    {  447, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC North Ndebele/Latin/Zimbabwe
    {  448, 2347,       0,    5775,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Norwegian Bokmal/Latin/Norway
    {  448, 2260,       0,       0,    5798,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  448, 2393,       0,       0,    5812,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  448, 2929,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  450, 2347,       0,    5775,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Norwegian Nynorsk/Latin/Norway
    {  450, 2260,       0,       0,    5829,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/Dublin
    {  450, 2393,       0,       0,    5843,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  450, 2929,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  451, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Nuer/Latin/South Sudan
    {  452, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Nyanja/Latin/Malawi
    {  453, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Nyankole/Latin/Uganda
    {  454, 2347,       0,    5860,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Occitan/Latin/France
    {  454, 2260,       0,       0,    5885,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  454, 2393,       0,       0,    5909,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  455, 2347,       0,    5930,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Occitan/Latin/Spain
    {  455, 2260,       0,       0,    5954,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  455, 2393,       0,       0,    5909,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  456, 2347,       0,    5975,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Odia/Odia/India
    {  456, 2260,       0,       0,    5996,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  456, 2393,       0,       0,    6015,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  457, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Oromo/Latin/Ethiopia
    {  459, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Osage/Osage/United States
    {  461, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Ossetic/Cyrillic/Russia
    {  462, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Papiamento/Latin/Curacao
    {  464, 2347,       0,    6041,       0,    0,    0,    0,  0, 16,  0,  0,  3,  0, }, // Etc/UTC Pashto/Arabic/Afghanistan
    {  464, 2260,       0,       0,    6057,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  464, 2393,       0,       0,    6072,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  466, 2347,       0,    6091,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Persian/Arabic/Iran
    {  466, 2260,       0,       0,    6108,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  466, 2393,       0,       0,    6123,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  468, 2347,       0,    6144,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Polish/Latin/Poland
    {  468, 2260,       0,       0,    6173,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  468, 2393,       0,       0,    6194,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  469, 2347,       0,    6214,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Brazil
    {  469, 2260,       0,       0,    6242,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  469, 2393,       0,       0,    6265,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  470, 2347,       0,    6291,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Angola
    {  470, 2260,       0,       0,    6316,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  470, 2393,       0,       0,    6340,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  471, 2347,       0,    6291,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Cape Verde
    {  471, 2260,       0,       0,    6316,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  471, 2393,       0,       0,    6340,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  472, 2347,       0,    6291,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Equatorial Guinea
    {  472, 2260,       0,       0,    6316,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  472, 2393,       0,       0,    6340,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  473, 2347,       0,    6291,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Guinea-Bissau
    {  473, 2260,       0,       0,    6316,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  473, 2393,       0,       0,    6340,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  474, 2347,       0,    6291,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Luxembourg
    {  474, 2260,       0,       0,    6316,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  474, 2393,       0,       0,    6340,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  475, 2347,       0,    6291,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Macao
    {  475, 2260,       0,       0,    6316,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  475, 2393,       0,       0,    6340,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  476, 2347,       0,    6291,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Mozambique
    {  476, 2260,       0,       0,    6316,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  476, 2393,       0,       0,    6340,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  477, 2347,       0,    6291,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Portugal
    {  477, 2260,       0,       0,    6316,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  477, 2393,       0,       0,    6340,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  478, 2347,       0,    6291,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Sao Tome and Principe
    {  478, 2260,       0,       0,    6316,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  478, 2393,       0,       0,    6340,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  479, 2347,       0,    6291,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Switzerland
    {  479, 2260,       0,       0,    6316,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  479, 2393,       0,       0,    6340,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  480, 2347,       0,    6291,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Portuguese/Latin/Timor-Leste
    {  480, 2260,       0,       0,    6316,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  480, 2393,       0,       0,    6340,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  481, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Prussian/Latin/Poland
    {  482, 2347,       0,    6363,       0,    0,    0,    0,  0, 22,  0,  0,  3,  0, }, // Etc/UTC Punjabi/Gurmukhi/India
    {  482, 2260,       0,       0,    6385,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  482, 2393,       0,       0,    6402,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  483, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Punjabi/Arabic/Pakistan
    {  484, 2347,       0,    6425,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Quechua/Latin/Peru
    {  484, 2260,       0,       0,    6454,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  484, 2393,       0,       0,    6478,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  487, 2347,       0,    6502,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Romanian/Latin/Romania
    {  487, 2260,       0,       0,    6528,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  487, 2393,       0,       0,    6550,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  489, 2347,       0,    6571,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Romansh/Latin/Switzerland
    {  489, 2393,       0,       0,    6594,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  490, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rombo/Latin/Tanzania
    {  491, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rundi/Latin/Burundi
    {  492, 2347,       0,    6616,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Russian/Cyrillic/Russia
    {  492, 2260,       0,       0,    6648,    0,    0,    0,  0,  0, 27,  0,  0,  0, }, // Europe/Dublin
    {  492, 2393,       0,       0,    6675,    0,    0,    0,  0,  0, 28,  0,  0,  0, }, // Europe/London
    {  498, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rwa/Latin/Tanzania
    {  499, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Saho/Latin/Eritrea
    {  500, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sakha/Cyrillic/Russia
    {  501, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Samburu/Latin/Kenya
    {  502, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sango/Latin/Central African Republic
    {  503, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sangu/Latin/Tanzania
    {  504, 2347,       0,    6703,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Sanskrit/Devanagari/India
    {  505, 2347,       0,    6724,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Santali/Ol Chiki/India
    {  506, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Santali/Devanagari/India
    {  507, 2347,       0,    6750,       0,    0,   70,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Sardinian/Latin/Italy
    {  507, 2260,       0,       0,    6778,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  507, 2393,       0,       0,    6801,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  507, 2929,       0,       0,       0,   73,   73,   76,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  508, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sena/Latin/Mozambique
    {  509, 2347,       0,    6823,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Serbian/Cyrillic/Serbia
    {  509, 2260,       0,       0,    6853,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  509, 2393,       0,       0,    6876,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/London
    {  510, 2347,       0,    1027,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Serbian/Cyrillic/Bosnia and Herzegovina
    {  510, 2260,       0,       0,    6898,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  510, 2393,       0,       0,    6923,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  513, 2347,       0,    6947,       0,    0,    0,    0,  0, 32,  0,  0,  3,  0, }, // Etc/UTC Serbian/Latin/Bosnia and Herzegovina
    {  513, 2260,       0,       0,    6979,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  513, 2393,       0,       0,    7004,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/London
    {  514, 2347,       0,    7030,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Serbian/Latin/Kosovo
    {  514, 2260,       0,       0,    7060,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  514, 2393,       0,       0,    7083,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  515, 2347,       0,    7030,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Serbian/Latin/Montenegro
    {  515, 2260,       0,       0,    7060,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  515, 2393,       0,       0,    7083,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  516, 2347,       0,    7030,       0,    0,    0,    0,  0, 30,  0,  0,  3,  0, }, // Etc/UTC Serbian/Latin/Serbia
    {  516, 2260,       0,       0,    7060,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  516, 2393,       0,       0,    7083,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  517, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Shambala/Latin/Tanzania
    {  518, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Shona/Latin/Zimbabwe
    {  519, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sichuan Yi/Yi/China
    {  520, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sicilian/Latin/Italy
    {  521, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Sidamo/Latin/Ethiopia
    {  522, 2347,       0,    7106,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Silesian/Latin/Poland
    {  522, 2260,       0,       0,    7135,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/Dublin
    {  522, 2393,       0,       0,    7158,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  523, 2347,       0,    7179,       0,    0,   79,    0,  0, 15,  0,  0,  5,  0, }, // Etc/UTC Sindhi/Arabic/Pakistan
    {  523, 2260,       0,       0,    7194,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  523, 2393,       0,       0,    7212,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  524, 2347,       0,    7237,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Sindhi/Devanagari/India
    {  525, 2347,       0,    7254,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Sinhala/Sinhala/Sri Lanka
    {  525, 2260,       0,       0,    7272,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  525, 2393,       0,       0,    7292,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  526, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Skolt Sami/Latin/Finland
    {  527, 2347,       0,    7317,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Slovak/Latin/Slovakia
    {  527, 2260,       0,       0,    7341,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  527, 2393,       0,       0,    7361,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  527, 2929,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  528, 2347,       0,    7378,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Slovenian/Latin/Slovenia
    {  528, 2260,       0,       0,    7406,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  528, 2393,       0,       0,    7426,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  529, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Soga/Latin/Uganda
    {  530, 2347,       0,    7447,       0,    0,   84,    0,  0, 30,  0,  0, 11,  0, }, // Etc/UTC Somali/Latin/Somalia
    {  530, 2260,       0,       0,    7477,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/Dublin
    {  530, 2393,       0,       0,    7507,    0,    0,    0,  0,  0, 29,  0,  0,  0, }, // Europe/London
    {  534, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Southern Kurdish/Arabic/Iran
    {  536, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Southern Sami/Latin/Sweden
    {  538, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Southern Sotho/Latin/South Africa
    {  540, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC South Ndebele/Latin/South Africa
    {  541, 2347,       0,    7536,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/Spain
    {  541, 2260,       0,       0,    7563,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/Dublin
    {  566, 2347,       0,    3038,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Spanish/Latin/United States
    {  566, 2260,       0,       0,    7588,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  566, 2393,       0,       0,    7612,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/London
    {  566, 2929,       0,       0,       0,    3,    3,    6,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  569, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Standard Moroccan Tamazight/Tifinagh/Morocco
    {  570, 2347,       0,    3714,       0,    0,    0,    0,  0, 29,  0,  0,  3,  0, }, // Etc/UTC Sundanese/Latin/Indonesia
    {  571, 2347,       0,    7636,       0,    0,    0,    0,  0, 33,  0,  0,  3,  0, }, // Etc/UTC Swahili/Latin/Tanzania
    {  571, 2260,       0,       0,    7669,    0,    0,    0,  0,  0, 33,  0,  0,  0, }, // Europe/Dublin
    {  571, 2393,       0,       0,    7702,    0,    0,    0,  0,  0, 34,  0,  0,  0, }, // Europe/London
    {  573, 2347,       0,    7736,       0,    0,    0,    0,  0, 12,  0,  0,  3,  0, }, // Etc/UTC Swahili/Latin/Kenya
    {  575, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Swati/Latin/South Africa
    {  577, 2347,       0,    7748,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Swedish/Latin/Sweden
    {  577, 2260,       0,       0,    7774,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  577, 2393,       0,       0,    7792,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {  577, 2929,       0,       0,       0,   95,  106,  123,  0,  0,  0, 11, 17, 17, }, // Pacific/Honolulu
    {  580, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Swiss German/Latin/Switzerland
    {  583, 2347,       0,    7810,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Syriac/Syriac/Iraq
    {  583, 2260,       0,       0,    7831,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  583, 2393,       0,       0,    7853,    0,    0,    0,  0,  0, 23,  0,  0,  0, }, // Europe/London
    {  585, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tachelhit/Tifinagh/Morocco
    {  587, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tai Dam/Tai Viet/Vietnam
    {  588, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Taita/Latin/Kenya
    {  589, 2347,       0,    7876,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Tajik/Cyrillic/Tajikistan
    {  590, 2347,       0,    7902,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Tamil/Tamil/India
    {  590, 2260,       0,       0,    7928,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  590, 2393,       0,       0,    7947,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  594, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Taroko/Latin/Taiwan
    {  595, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tasawaq/Latin/Niger
    {  596, 2347,       0,    7968,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Tatar/Cyrillic/Russia
    {  597, 2347,       0,    7999,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Telugu/Telugu/India
    {  597, 2260,       0,       0,    8020,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  597, 2393,       0,       0,    8040,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  598, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Teso/Latin/Uganda
    {  600, 2347,       0,    8059,       0,    0,    0,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Thai/Thai/Thailand
    {  600, 2260,       0,       0,    8076,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  600, 2393,       0,       0,    8096,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  601, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tibetan/Tibetan/China
    {  603, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tigre/Ethiopic/Eritrea
    {  604, 2347,       0,    8113,       0,    0,    0,    0,  0, 14,  0,  0,  3,  0, }, // Etc/UTC Tigrinya/Ethiopic/Ethiopia
    {  604, 2260,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  604, 2393,       0,       0,    8127,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/London
    {  606, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tok Pisin/Latin/Papua New Guinea
    {  607, 2347,       0,    8140,       0,    0,  140,    0,  0, 17,  0,  0,  3,  0, }, // Etc/UTC Tongan/Latin/Tonga
    {  607, 2260,       0,       0,    8157,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/Dublin
    {  607, 2393,       0,       0,    8187,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/London
    {  607, 2929,       0,       0,       0,  143,  143,  146,  0,  0,  0,  3,  3,  3, }, // Pacific/Honolulu
    {  608, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tsonga/Latin/South Africa
    {  609, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tswana/Latin/South Africa
    {  611, 2347,       0,    8217,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Turkish/Latin/Turkey
    {  611, 2260,       0,       0,    8242,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  611, 2393,       0,       0,    8264,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  613, 2347,       0,    8283,       0,    0,    0,    0,  0, 28,  0,  0,  3,  0, }, // Etc/UTC Turkmen/Latin/Turkmenistan
    {  613, 2260,       0,       0,    8311,    0,    0,    0,  0,  0, 24,  0,  0,  0, }, // Europe/Dublin
    {  613, 2393,       0,       0,    8335,    0,    0,    0,  0,  0, 29,  0,  0,  0, }, // Europe/London
    {  614, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Tyap/Latin/Nigeria
    {  615, 2347,       0,    8364,       0,    0,    0,    0,  0, 33,  0,  0,  3,  0, }, // Etc/UTC Ukrainian/Cyrillic/Ukraine
    {  615, 2260,       0,       0,    8397,    0,    0,    0,  0,  0, 26,  0,  0,  0, }, // Europe/Dublin
    {  615, 2393,       0,       0,    8423,    0,    0,    0,  0,  0, 34,  0,  0,  0, }, // Europe/London
    {  616, 2347,       0,    8457,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Upper Sorbian/Latin/Germany
    {  616, 2260,       0,       0,    8481,    0,    0,    0,  0,  0, 15,  0,  0,  0, }, // Europe/Dublin
    {  616, 2393,       0,       0,    8496,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/London
    {  617, 2347,       0,    8514,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Urdu/Arabic/Pakistan
    {  617, 2260,       0,       0,    8538,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  617, 2393,       0,       0,    8556,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/London
    {  619, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Uyghur/Arabic/China
    {  619, 2260,       0,       0,    8569,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  619, 2393,       0,       0,    8590,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  620, 2347,       0,    8611,       0,    0,    0,    0,  0, 27,  0,  0,  3,  0, }, // Etc/UTC Uzbek/Latin/Uzbekistan
    {  620, 2260,       0,       0,    8638,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/Dublin
    {  620, 2393,       0,       0,    8659,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  621, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Uzbek/Arabic/Afghanistan
    {  622, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Uzbek/Cyrillic/Uzbekistan
    {  622, 2260,       0,       0,    8680,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  622, 2393,       0,       0,    8699,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  623, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Vai/Vai/Liberia
    {  625, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Venda/Latin/South Africa
    {  626, 2347,       0,    8718,       0,    0,    0,    0,  0, 20,  0,  0,  3,  0, }, // Etc/UTC Vietnamese/Latin/Vietnam
    {  626, 2260,       0,       0,    8738,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {  626, 2393,       0,       0,    8754,    0,    0,    0,  0,  0, 14,  0,  0,  0, }, // Europe/London
    {  626, 2929,       0,       0,       0,  149,    3,    6,  0,  0,  0,  7,  3,  3, }, // Pacific/Honolulu
    {  627, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Volapuk/Latin/world
    {  628, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Vunjo/Latin/Tanzania
    {  629, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Walloon/Latin/Belgium
    {  630, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Walser/Latin/Switzerland
    {  631, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Warlpiri/Latin/Australia
    {  632, 2347,       0,    8768,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Welsh/Latin/United Kingdom
    {  632, 2260,       0,       0,    8794,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  632, 2393,       0,       0,    8816,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  633, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Western Balochi/Arabic/Pakistan
    {  638, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Western Frisian/Latin/Netherlands
    {  638, 2260,       0,       0,    8833,    0,    0,    0,  0,  0, 16,  0,  0,  0, }, // Europe/Dublin
    {  638, 2393,       0,       0,    8849,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  639, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Wolaytta/Ethiopic/Ethiopia
    {  640, 2347,       0,    8866,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Wolof/Latin/Senegal
    {  641, 2347,       0,       0,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Xhosa/Latin/South Africa
    {  641, 2260,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  641, 2393,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  642, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Yangben/Latin/Cameroon
    {  643, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Yiddish/Hebrew/Ukraine
    {  644, 2347,       0,    8897,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Yoruba/Latin/Nigeria
    {  644, 2260,       0,       0,      26,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  644, 2393,       0,       0,      45,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  645, 2347,       0,    8916,       0,    0,    0,    0,  0, 19,  0,  0,  3,  0, }, // Etc/UTC Yoruba/Latin/Benin
    {  646, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Zarma/Latin/Niger
    {  647, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Zhuang/Latin/China
    {  648, 2347,       0,    8935,       0,    0,    0,    0,  0, 31,  0,  0,  3,  0, }, // Etc/UTC Zulu/Latin/South Africa
    {  648, 2260,       0,       0,    8966,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/Dublin
    {  648, 2393,       0,       0,    8996,    0,    0,    0,  0,  0, 32,  0,  0,  0, }, // Europe/London
    {  649, 2347,       0,    9028,       0,    0,    0,    0,  0, 23,  0,  0,  3,  0, }, // Etc/UTC Kaingang/Latin/Brazil
    {  649, 2260,       0,       0,    9051,    0,    0,    0,  0,  0, 12,  0,  0,  0, }, // Europe/Dublin
    {  649, 2393,       0,       0,    9063,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/London
    {  650, 2347,       0,    9082,       0,    0,    0,    0,  0, 34,  0,  0,  3,  0, }, // Etc/UTC Nheengatu/Latin/Brazil
    {  650, 2260,       0,       0,    9116,    0,    0,    0,  0,  0, 22,  0,  0,  0, }, // Europe/Dublin
    {  650, 2393,       0,       0,    9138,    0,    0,    0,  0,  0, 30,  0,  0,  0, }, // Europe/London
    {  653, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Haryanvi/Devanagari/India
    {  654, 2347,       0,    9168,       0,    0,    0,    0,  0, 18,  0,  0,  3,  0, }, // Etc/UTC Northern Frisian/Latin/Germany
    {  654, 2260,       0,       0,    9186,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  654, 2393,       0,       0,    9205,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/London
    {  655, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rajasthani/Devanagari/India
    {  656, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Moksha/Cyrillic/Russia
    {  657, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Toki Pona/Latin/world
    {  658, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Pijin/Latin/Solomon Islands
    {  659, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Obolo/Latin/Nigeria
    {  660, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Baluchi/Arabic/Pakistan
    {  662, 2347,       0,    9222,       0,    0,    0,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Ligurian/Latin/Italy
    {  662, 2260,       0,       0,    9248,    0,    0,    0,  0,  0, 19,  0,  0,  0, }, // Europe/Dublin
    {  662, 2393,       0,       0,    9267,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  663, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Rohingya/Hanifi/Myanmar
    {  665, 2347,       0,    8514,       0,    0,    0,    0,  0, 24,  0,  0,  3,  0, }, // Etc/UTC Torwali/Arabic/Pakistan
    {  665, 2260,       0,       0,    8538,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  665, 2393,       0,       0,    8556,    0,    0,    0,  0,  0, 13,  0,  0,  0, }, // Europe/London
    {  666, 2347,       0,    9287,       0,    0,  156,    0,  0, 26,  0,  0,  3,  0, }, // Etc/UTC Anii/Latin/Benin
    {  666, 2260,       0,       0,    9313,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  666, 2393,       0,       0,    9331,    0,    0,    0,  0,  0, 33,  0,  0,  0, }, // Europe/London
    {  667, 2347,       0,    9364,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Kangri/Devanagari/India
    {  667, 2260,       0,       0,    9385,    0,    0,    0,  0,  0, 17,  0,  0,  0, }, // Europe/Dublin
    {  667, 2393,       0,       0,    9402,    0,    0,    0,  0,  0, 25,  0,  0,  0, }, // Europe/London
    {  668, 2347,       0,    9427,       0,    0,    0,    0,  0, 25,  0,  0,  3,  0, }, // Etc/UTC Venetian/Latin/Italy
    {  668, 2260,       0,       0,    9452,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/Dublin
    {  668, 2393,       0,       0,    9472,    0,    0,    0,  0,  0, 20,  0,  0,  0, }, // Europe/London
    {  669, 2347,       0,    9492,       0,    0,    0,    0,  0, 21,  0,  0,  3,  0, }, // Etc/UTC Kuvi/Latin/India
    {  669, 2260,       0,       0,    9513,    0,    0,    0,  0,  0, 18,  0,  0,  0, }, // Europe/Dublin
    {  669, 2393,       0,       0,    9531,    0,    0,    0,  0,  0, 21,  0,  0,  0, }, // Europe/London
    {  670, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kuvi/Devanagari/India
    {  671, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kuvi/Odia/India
    {  672, 2347,       0,       0,       0,    0,    0,    0,  0,  0,  0,  0,  3,  0, }, // Etc/UTC Kuvi/Telugu/India
}; // Zone naming table

// Sorted by locale index, then meta key
static inline constexpr LocaleMetaZoneLongNames localeMetaZoneLongNameTable[] = {
    // locInd, metaKey, (generic, standard, DST){ind, sz}
    {    0,    0,       0,       9,      27,  9, 18, 16, }, // Acre C/AnyScript/AnyTerritory
    {    0,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan
    {    0,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central
    {    0,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {    0,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {    0,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {    0,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {    0,  128,     235,     246,     266, 11, 20, 18, }, // Almaty
    {    0,  135,     284,     295,     315, 11, 20, 18, }, // Amazon
    {    0,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {    0,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {    0,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {    0,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {    0,  207,     552,     563,     583, 11, 20, 18, }, // Anadyr
    {    0,  214,     601,     610,     628,  9, 18, 18, }, // Apia
    {    0,  219,     646,     656,     675, 10, 19, 17, }, // Aqtau
    {    0,  225,     692,     703,     723, 11, 20, 18, }, // Aqtobe
    {    0,  232,     741,     753,     774, 12, 21, 21, }, // Arabian
    {    0,  240,     795,     809,     832, 14, 23, 21, }, // Argentina
    {    0,  250,     853,     875,     906, 22, 31, 29, }, // Argentina_Western
    {    0,  268,     935,     947,     968, 12, 21, 19, }, // Armenia
    {    0,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {    0,  295,    1044,    1066,    1098, 22, 32, 32, }, // Australia_Central
    {    0,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {    0,  338,    1241,    1263,    1295, 22, 32, 32, }, // Australia_Eastern
    {    0,  356,    1327,    1349,    1381, 22, 32, 32, }, // Australia_Western
    {    0,  374,    1413,    1428,    1452, 15, 24, 22, }, // Azerbaijan
    {    0,  385,    1474,    1485,    1505, 11, 20, 18, }, // Azores
    {    0,  397,    1523,    1538,    1562, 15, 24, 22, }, // Bangladesh
    {    0,  415,       0,    1584,       0,  0, 11,  0, }, // Bhutan
    {    0,  422,       0,    1595,       0,  0, 12,  0, }, // Bolivia
    {    0,  437,    1607,    1620,    1642, 13, 22, 20, }, // Brasilia
    {    0,  454,       0,    1662,       0,  0, 22,  0, }, // Brunei
    {    0,  461,    1684,    1699,    1723, 15, 24, 22, }, // Cape_Verde
    {    0,  472,       0,    1745,       0,  0, 10,  0, }, // Casey
    {    0,  478,       0,    1755,       0,  0, 22,  0, }, // Chamorro
    {    0,  487,    1777,    1789,    1810, 12, 21, 21, }, // Chatham
    {    0,  495,    1831,    1841,    1860, 10, 19, 17, }, // Chile
    {    0,  501,    1877,    1887,    1906, 10, 19, 19, }, // China
    {    0,  507,    1925,    1940,    1964, 15, 24, 22, }, // Choibalsan
    {    0,  518,       0,    1986,       0,  0, 21,  0, }, // Christmas
    {    0,  528,       0,    2007,       0,  0, 18,  0, }, // Cocos
    {    0,  534,    2025,    2038,    2060, 13, 22, 20, }, // Colombia
    {    0,  543,    2080,    2097,    2123, 17, 26, 29, }, // Cook
    {    0,  548,    2152,    2161,    2179,  9, 18, 18, }, // Cuba
    {    0,  559,       0,    2197,       0,  0, 10,  0, }, // Davis
    {    0,  575,       0,    2207,       0,  0, 21,  0, }, // DumontDUrville
    {    0,  612,       0,    2228,       0,  0, 15,  0, }, // East_Timor
    {    0,  623,    2243,    2261,    2288, 18, 27, 25, }, // Easter
    {    0,  630,       0,    2313,       0,  0, 12,  0, }, // Ecuador
    {    0,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {    0,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {    0,  668,       0,    2483,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {    0,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {    0,  706,    2591,    2612,    2642, 21, 30, 28, }, // Falkland
    {    0,  715,    2670,    2679,    2697,  9, 18, 16, }, // Fiji
    {    0,  720,       0,    2713,       0,  0, 18,  0, }, // French_Guiana
    {    0,  734,       0,    2731,       0,  0, 32,  0, }, // French_Southern
    {    0,  757,       0,    2763,       0,  0, 14,  0, }, // Galapagos
    {    0,  767,       0,    2777,       0,  0, 12,  0, }, // Gambier
    {    0,  775,    2789,    2801,    2822, 12, 21, 19, }, // Georgia
    {    0,  783,       0,    2841,       0,  0, 20,  0, }, // Gilbert_Islands
    {    0,  799,       0,    2861,       0,  0, 19,  0, }, // GMT
    {    0,  813,    2880,    2894,    2917, 14, 23, 21, }, // Greenland
    {    0,  841,    2938,    2957,    2985, 19, 28, 26, }, // Greenland_Eastern
    {    0,  859,    3011,    3030,    3058, 19, 28, 26, }, // Greenland_Western
    {    0,  877,       0,    3084,       0,  0, 18,  0, }, // Guam
    {    0,  882,       0,    3102,       0,  0, 18,  0, }, // Gulf
    {    0,  887,       0,    3120,       0,  0, 11,  0, }, // Guyana
    {    0,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {    0,  910,    3209,    3223,    3246, 14, 23, 21, }, // Hong_Kong
    {    0,  920,    3267,    3276,    3294,  9, 18, 16, }, // Hovd
    {    0,  925,       0,    3310,       0,  0, 19,  0, }, // India
    {    0,  931,       0,    3329,       0,  0, 17,  0, }, // Indian_Ocean
    {    0,  944,       0,    3346,       0,  0, 14,  0, }, // Indochina
    {    0,  954,       0,    3360,       0,  0, 22,  0, }, // Indonesia_Central
    {    0,  972,       0,    3382,       0,  0, 22,  0, }, // Indonesia_Eastern
    {    0,  990,       0,    3404,       0,  0, 22,  0, }, // Indonesia_Western
    {    0, 1008,    3426,    3435,    3453,  9, 18, 18, }, // Iran
    {    0, 1019,    3471,    3483,    3504, 12, 21, 19, }, // Irkutsk
    {    0, 1027,    3523,    3534,    3554, 11, 20, 20, }, // Israel
    {    0, 1034,    3574,    3584,    3603, 10, 19, 19, }, // Japan
    {    0, 1040,    3622,    3651,    3689, 29, 38, 36, }, // Kamchatka
    {    0, 1058,       0,    3725,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {    0, 1077,       0,    3745,       0,  0, 20,  0, }, // Kazakhstan_Western
    {    0, 1106,    3765,    3776,    3796, 11, 20, 20, }, // Korea
    {    0, 1112,       0,    3816,       0,  0, 11,  0, }, // Kosrae
    {    0, 1119,    3827,    3843,    3868, 16, 25, 23, }, // Krasnoyarsk
    {    0, 1151,       0,    3891,       0,  0, 15,  0, }, // Kyrgystan
    {    0, 1161,       0,    3906,       0,  0, 10,  0, }, // Lanka
    {    0, 1175,       0,    3916,       0,  0, 17,  0, }, // Line_Islands
    {    0, 1188,    3933,    3947,    3970, 14, 23, 23, }, // Lord_Howe
    {    0, 1198,    3993,    4003,    4022, 10, 19, 17, }, // Macau
    {    0, 1204,    4039,    4051,    4072, 12, 21, 19, }, // Magadan
    {    0, 1219,       0,    4091,       0,  0, 13,  0, }, // Malaysia
    {    0, 1228,       0,    4104,       0,  0, 13,  0, }, // Maldives
    {    0, 1237,       0,    4117,       0,  0, 14,  0, }, // Marquesas
    {    0, 1247,       0,    4131,       0,  0, 21,  0, }, // Marshall_Islands
    {    0, 1264,    4152,    4166,    4189, 14, 23, 21, }, // Mauritius
    {    0, 1274,       0,    4210,       0,  0, 11,  0, }, // Mawson
    {    0, 1281,    4221,    4241,    4270, 20, 29, 29, }, // Mexico_Pacific
    {    0, 1296,    4299,    4315,    4340, 16, 25, 23, }, // Mongolia
    {    0, 1305,    4363,    4374,    4394, 11, 20, 18, }, // Moscow
    {    0, 1312,       0,    4412,       0,  0, 12,  0, }, // Myanmar
    {    0, 1320,       0,    4424,       0,  0, 10,  0, }, // Nauru
    {    0, 1326,       0,    4434,       0,  0, 10,  0, }, // Nepal
    {    0, 1332,    4444,    4462,    4489, 18, 27, 25, }, // New_Caledonia
    {    0, 1346,    4514,    4530,    4555, 16, 25, 25, }, // New_Zealand
    {    0, 1358,    4580,    4597,    4623, 17, 26, 26, }, // Newfoundland
    {    0, 1371,       0,    4649,       0,  0,  9,  0, }, // Niue
    {    0, 1376,    4658,    4677,    4705, 19, 28, 28, }, // Norfolk
    {    0, 1384,    4733,    4757,    4790, 24, 33, 31, }, // Noronha
    {    0, 1392,       0,    4821,       0,  0, 26,  0, }, // North_Mariana
    {    0, 1406,    4847,    4863,    4888, 16, 25, 23, }, // Novosibirsk
    {    0, 1418,    4911,    4920,    4938,  9, 18, 16, }, // Omsk
    {    0, 1428,    4954,    4967,    4989, 13, 22, 20, }, // Pakistan
    {    0, 1437,       0,    5009,       0,  0, 10,  0, }, // Palau
    {    0, 1443,       0,    5019,       0,  0, 21,  0, }, // Papua_New_Guinea
    {    0, 1460,    5040,    5053,    5075, 13, 22, 20, }, // Paraguay
    {    0, 1469,    5095,    5104,    5122,  9, 18, 16, }, // Peru
    {    0, 1474,    5138,    5153,    5177, 15, 24, 22, }, // Philippines
    {    0, 1486,       0,    5199,       0,  0, 20,  0, }, // Phoenix_Islands
    {    0, 1502,    5219,    5245,    5280, 26, 35, 35, }, // Pierre_Miquelon
    {    0, 1518,       0,    5315,       0,  0, 13,  0, }, // Pitcairn
    {    0, 1527,       0,    5328,       0,  0, 11,  0, }, // Ponape
    {    0, 1534,       0,    5339,       0,  0, 14,  0, }, // Pyongyang
    {    0, 1544,    5353,    5367,    5390, 14, 23, 21, }, // Qyzylorda
    {    0, 1554,       0,    5411,       0,  0, 12,  0, }, // Reunion
    {    0, 1562,       0,    5423,       0,  0, 12,  0, }, // Rothera
    {    0, 1570,    5435,    5448,    5470, 13, 22, 20, }, // Sakhalin
    {    0, 1579,    5490,    5501,    5521, 11, 20, 18, }, // Samara
    {    0, 1596,    5539,    5549,    5568, 10, 19, 19, }, // Samoa
    {    0, 1602,       0,    5587,       0,  0, 15,  0, }, // Seychelles
    {    0, 1624,       0,    5602,       0,  0, 23,  0, }, // Singapore
    {    0, 1634,       0,    5625,       0,  0, 20,  0, }, // Solomon
    {    0, 1642,       0,    5645,       0,  0, 18,  0, }, // South_Georgia
    {    0, 1656,       0,    5663,       0,  0, 13,  0, }, // Suriname
    {    0, 1676,       0,    5676,       0,  0, 10,  0, }, // Syowa
    {    0, 1682,       0,    5686,       0,  0, 11,  0, }, // Tahiti
    {    0, 1689,    5697,    5708,    5728, 11, 20, 20, }, // Taipei
    {    0, 1696,       0,    5748,       0,  0, 15,  0, }, // Tajikistan
    {    0, 1724,       0,    5763,       0,  0, 12,  0, }, // Tokelau
    {    0, 1732,    5775,    5785,    5804, 10, 19, 17, }, // Tonga
    {    0, 1738,       0,    5821,       0,  0, 10,  0, }, // Truk
    {    0, 1750,    5831,    5848,    5874, 17, 26, 24, }, // Turkmenistan
    {    0, 1763,       0,    5898,       0,  0, 11,  0, }, // Tuvalu
    {    0, 1777,    5909,    5921,    5942, 12, 21, 19, }, // Uruguay
    {    0, 1792,    5961,    5976,    6000, 15, 24, 22, }, // Uzbekistan
    {    0, 1803,    6022,    6034,    6055, 12, 21, 19, }, // Vanuatu
    {    0, 1811,       0,    6074,       0,  0, 14,  0, }, // Venezuela
    {    0, 1821,    6088,    6104,    6129, 16, 25, 23, }, // Vladivostok
    {    0, 1833,    6152,    6166,    6189, 14, 23, 21, }, // Volgograd
    {    0, 1843,       0,    6210,       0,  0, 11,  0, }, // Vostok
    {    0, 1850,       0,    6221,       0,  0, 16,  0, }, // Wake
    {    0, 1855,       0,    6237,       0,  0, 20,  0, }, // Wallis
    {    0, 1862,    6257,    6269,    6290, 12, 21, 19, }, // Yakutsk
    {    0, 1870,    6309,    6327,    6354, 18, 27, 25, }, // Yekaterinburg
    {    0, 1892,       0,    6379,       0,  0, 10,  0, }, // Yukon
    {    1,    0,    6389,    6399,    6420, 10, 21, 19, }, // Acre Abkhazian/Cyrillic/Georgia
    {    1,    5,       0,    6439,       0,  0, 10,  0, }, // Afghanistan
    {    1,   17,       0,    6449,       0,  0, 16,  0, }, // Africa_Central
    {    1,   32,       0,    6465,       0,  0, 19,  0, }, // Africa_Eastern
    {    1,   65,       0,    6484,       0,  0, 15,  0, }, // Africa_Southern
    {    1,   81,    6499,    6519,    6557, 20, 38, 36, }, // Africa_Western
    {    1,  107,    6593,    6600,    6625,  7, 25, 23, }, // Alaska
    {    1,  128,    6648,    6662,    6687, 14, 25, 23, }, // Almaty
    {    1,  135,    6710,    6718,    6744,  8, 26, 24, }, // Amazon
    {    1,  142,    6768,    6785,    6820, 17, 35, 33, }, // America_Central
    {    1,  158,    6853,    6873,    6911, 20, 38, 36, }, // America_Eastern
    {    1,  174,    6947,    6979,    7022, 32, 43, 41, }, // America_Mountain
    {    1,  191,    7063,    7082,    7112, 19, 30, 28, }, // America_Pacific
    {    1,  207,    7140,    7156,    7179, 16, 23, 21, }, // Anadyr
    {    1,  214,    7200,    7204,    7226,  4, 22, 20, }, // Apia
    {    1,  219,    7246,    7257,    7280, 11, 23, 20, }, // Aqtau
    {    1,  225,    7300,    7312,    7335, 12, 23, 21, }, // Aqtobe
    {    1,  232,    7356,    7372,    7406, 16, 34, 32, }, // Arabian
    {    1,  240,    7438,    7447,    7474,  9, 27, 25, }, // Argentina
    {    1,  250,    7499,    7522,    7563, 23, 41, 39, }, // Argentina_Western
    {    1,  268,    7602,    7612,    7640, 10, 28, 26, }, // Armenia
    {    1,  286,    7666,    7684,    7713, 18, 29, 27, }, // Atlantic
    {    1,  295,    7740,    7759,    7796, 19, 37, 35, }, // Australia_Central
    {    1,  313,    7831,    7871,    7922, 40, 51, 49, }, // Australia_CentralWestern
    {    1,  338,    7971,    7993,    8033, 22, 40, 38, }, // Australia_Eastern
    {    1,  356,    8071,    8094,    8135, 23, 41, 39, }, // Australia_Western
    {    1,  374,    8174,    8185,    8214, 11, 29, 27, }, // Azerbaijan
    {    1,  385,    8241,    8257,    8291, 16, 34, 32, }, // Azores
    {    1,  397,    8323,    8332,    8359,  9, 27, 25, }, // Bangladesh
    {    1,  415,       0,    8384,       0,  0,  5,  0, }, // Bhutan
    {    1,  422,       0,    8389,       0,  0,  7,  0, }, // Bolivia
    {    1,  437,    8396,    8404,    8430,  8, 26, 24, }, // Brasilia
    {    1,  454,       0,    8454,       0,  0, 17,  0, }, // Brunei
    {    1,  461,    8471,    8481,    8509, 10, 28, 26, }, // Cape_Verde
    {    1,  472,       0,    8535,       0,  0,  5,  0, }, // Casey
    {    1,  478,       0,    8540,       0,  0,  7,  0, }, // Chamorro
    {    1,  487,    8547,    8552,    8575,  5, 23, 21, }, // Chatham
    {    1,  495,    8596,    8600,    8622,  4, 22, 20, }, // Chile
    {    1,  501,    8642,    8647,    8670,  5, 23, 21, }, // China
    {    1,  507,    8691,    8700,    8727,  9, 27, 25, }, // Choibalsan
    {    1,  518,       0,    8752,       0,  0, 12,  0, }, // Christmas
    {    1,  528,       0,    8764,       0,  0, 17,  0, }, // Cocos
    {    1,  534,    8781,    8789,    8815,  8, 26, 24, }, // Colombia
    {    1,  543,    8839,    8858,    8895, 19, 37, 39, }, // Cook
    {    1,  548,    8934,    8938,    8960,  4, 22, 20, }, // Cuba
    {    1,  559,       0,    8980,       0,  0,  6,  0, }, // Davis
    {    1,  575,       0,    8986,       0,  0, 15,  0, }, // DumontDUrville
    {    1,  612,       0,    9001,       0,  0, 18,  0, }, // East_Timor
    {    1,  623,    9019,    9030,    9059, 11, 29, 27, }, // Easter
    {    1,  630,       0,    9086,       0,  0,  7,  0, }, // Ecuador
    {    1,  638,    9093,    9109,    9143, 16, 34, 32, }, // Europe_Central
    {    1,  653,    9175,    9194,    9231, 19, 37, 35, }, // Europe_Eastern
    {    1,  668,       0,    9266,       0,  0, 16,  0, }, // Europe_Further_Eastern
    {    1,  691,    9282,    9302,    9340, 20, 38, 36, }, // Europe_Western
    {    1,  706,    9376,    9396,    9434, 20, 38, 36, }, // Falkland
    {    1,  715,    9470,    9474,    9496,  4, 22, 20, }, // Fiji
    {    1,  720,       0,    9516,       0,  0, 17,  0, }, // French_Guiana
    {    1,  734,       0,    9533,       0,  0, 50,  0, }, // French_Southern
    {    1,  757,       0,    9583,       0,  0, 21,  0, }, // Galapagos
    {    1,  767,       0,    9604,       0,  0,  5,  0, }, // Gambier
    {    1,  775,    9609,    9618,    9645,  9, 27, 25, }, // Georgia
    {    1,  783,       0,    9670,       0,  0, 17,  0, }, // Gilbert_Islands
    {    1,  799,       0,    9687,       0,  0, 25,  0, }, // GMT
    {    1,  841,    9712,    9735,    9777, 23, 42, 39, }, // Greenland_Eastern
    {    1,  859,    9816,    9840,    9882, 24, 42, 40, }, // Greenland_Western
    {    1,  877,       0,    9922,       0,  0,  4,  0, }, // Guam
    {    1,  882,       0,    9926,       0,  0, 19,  0, }, // Gulf
    {    1,  887,       0,    9945,       0,  0,  6,  0, }, // Guyana
    {    1,  894,    9951,    9971,   10002, 20, 31, 29, }, // Hawaii_Aleutian
    {    1,  910,   10031,   10038,   10063,  7, 25, 23, }, // Hong_Kong
    {    1,  920,   10086,   10090,   10112,  4, 22, 20, }, // Hovd
    {    1,  925,       0,   10132,       0,  0,  5,  0, }, // India
    {    1,  931,       0,   10137,       0,  0, 15,  0, }, // Indian_Ocean
    {    1,  944,       0,   10152,       0,  0,  9,  0, }, // Indochina
    {    1,  954,       0,   10161,       0,  0, 19,  0, }, // Indonesia_Central
    {    1,  972,       0,   10180,       0,  0, 22,  0, }, // Indonesia_Eastern
    {    1,  990,       0,   10202,       0,  0, 23,  0, }, // Indonesia_Western
    {    1, 1008,   10225,   10229,   10251,  4, 22, 20, }, // Iran
    {    1, 1019,   10271,   10278,   10303,  7, 25, 23, }, // Irkutsk
    {    1, 1027,   10326,   10333,   10358,  7, 25, 23, }, // Israel
    {    1, 1034,   10381,   10388,   10413,  7, 25, 23, }, // Japan
    {    1, 1040,   10436,   10461,   10504, 25, 43, 41, }, // Kamchatka
    {    1, 1058,       0,   10545,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {    1, 1077,       0,   10567,       0,  0, 23,  0, }, // Kazakhstan_Western
    {    1, 1106,   10590,   10596,   10620,  6, 24, 22, }, // Korea
    {    1, 1112,       0,   10642,       0,  0,  6,  0, }, // Kosrae
    {    1, 1119,   10648,   10659,   10688, 11, 29, 27, }, // Krasnoyarsk
    {    1, 1151,       0,   10715,       0,  0,  8,  0, }, // Kyrgystan
    {    1, 1161,       0,   10723,       0,  0,  9,  0, }, // Lanka
    {    1, 1175,       0,   10732,       0,  0, 13,  0, }, // Line_Islands
    {    1, 1188,   10745,   10753,   10779,  8, 26, 24, }, // Lord_Howe
    {    1, 1198,   10803,   10808,   10831,  5, 23, 21, }, // Macau
    {    1, 1204,   10852,   10859,   10884,  7, 25, 23, }, // Magadan
    {    1, 1219,       0,   10907,       0,  0,  8,  0, }, // Malaysia
    {    1, 1228,       0,   10915,       0,  0,  7,  0, }, // Maldives
    {    1, 1237,       0,   10922,       0,  0, 18,  0, }, // Marquesas
    {    1, 1247,       0,   10940,       0,  0, 26,  0, }, // Marshall_Islands
    {    1, 1264,   10966,   10973,   10998,  7, 25, 23, }, // Mauritius
    {    1, 1274,       0,   11021,       0,  0,  6,  0, }, // Mawson
    {    1, 1281,   11027,   11057,   11098, 30, 41, 39, }, // Mexico_Pacific
    {    1, 1296,   11137,   11147,   11175, 10, 28, 26, }, // Mongolia
    {    1, 1305,   11201,   11207,   11231,  6, 24, 22, }, // Moscow
    {    1, 1312,       0,   11253,       0,  0,  6,  0, }, // Myanmar
    {    1, 1320,       0,   11259,       0,  0,  5,  0, }, // Nauru
    {    1, 1326,       0,   11264,       0,  0,  5,  0, }, // Nepal
    {    1, 1332,   11269,   11282,   11313, 13, 31, 29, }, // New_Caledonia
    {    1, 1346,   11342,   11354,   11384, 12, 30, 28, }, // New_Zealand
    {    1, 1358,   11412,   11424,   11454, 12, 30, 28, }, // Newfoundland
    {    1, 1371,       0,   11482,       0,  0,  4,  0, }, // Niue
    {    1, 1376,   11486,   11493,   11518,  7, 25, 23, }, // Norfolk
    {    1, 1384,   11541,   11560,   11597, 19, 37, 35, }, // Noronha
    {    1, 1392,       0,   11632,       0,  0, 27,  0, }, // North_Mariana
    {    1, 1406,   11659,   11670,   11699, 11, 29, 27, }, // Novosibirsk
    {    1, 1418,   11726,   11730,   11752,  4, 22, 20, }, // Omsk
    {    1, 1428,   11772,   11780,   11806,  8, 26, 24, }, // Pakistan
    {    1, 1437,       0,   11830,       0,  0,  5,  0, }, // Palau
    {    1, 1443,       0,   11835,       0,  0, 19,  0, }, // Papua_New_Guinea
    {    1, 1460,   11854,   11862,   11888,  8, 26, 24, }, // Paraguay
    {    1, 1469,   11912,   11916,   11938,  4, 22, 20, }, // Peru
    {    1, 1474,   11958,   11966,   11992,  8, 26, 24, }, // Philippines
    {    1, 1486,       0,   12016,       0,  0, 15,  0, }, // Phoenix_Islands
    {    1, 1502,   12031,   12049,   12085, 18, 36, 34, }, // Pierre_Miquelon
    {    1, 1518,       0,   12119,       0,  0,  7,  0, }, // Pitcairn
    {    1, 1527,       0,   12126,       0,  0,  6,  0, }, // Ponape
    {    1, 1534,       0,   12132,       0,  0,  7,  0, }, // Pyongyang
    {    1, 1544,   12139,   12149,   12177, 10, 28, 26, }, // Qyzylorda
    {    1, 1554,       0,   12203,       0,  0,  7,  0, }, // Reunion
    {    1, 1562,       0,   12210,       0,  0,  6,  0, }, // Rothera
    {    1, 1570,   12216,   12223,   12248,  7, 25, 23, }, // Sakhalin
    {    1, 1579,   12271,   12285,   12310, 14, 25, 23, }, // Samara
    {    1, 1596,   12333,   12338,   12361,  5, 23, 21, }, // Samoa
    {    1, 1602,       0,   12382,       0,  0, 25,  0, }, // Seychelles
    {    1, 1624,       0,   12407,       0,  0,  8,  0, }, // Singapore
    {    1, 1634,       0,   12415,       0,  0, 26,  0, }, // Solomon
    {    1, 1642,       0,   12441,       0,  0, 16,  0, }, // South_Georgia
    {    1, 1656,       0,   12457,       0,  0,  7,  0, }, // Suriname
    {    1, 1676,       0,   12464,       0,  0,  5,  0, }, // Syowa
    {    1, 1682,       0,   12469,       0,  0,  5,  0, }, // Tahiti
    {    1, 1689,   12474,   12480,   12504,  6, 24, 22, }, // Taipei
    {    1, 1696,       0,   12526,       0,  0, 10,  0, }, // Tajikistan
    {    1, 1724,       0,   12536,       0,  0,  7,  0, }, // Tokelau
    {    1, 1732,   12543,   12548,   12571,  5, 23, 21, }, // Tonga
    {    1, 1738,       0,   12592,       0,  0,  4,  0, }, // Truk
    {    1, 1750,   12596,   12608,   12638, 12, 30, 28, }, // Turkmenistan
    {    1, 1763,       0,   12666,       0,  0,  6,  0, }, // Tuvalu
    {    1, 1777,   12672,   12679,   12704,  7, 25, 23, }, // Uruguay
    {    1, 1792,   12727,   12738,   12767, 11, 29, 27, }, // Uzbekistan
    {    1, 1803,   12794,   12801,   12826,  7, 25, 23, }, // Vanuatu
    {    1, 1811,       0,   12849,       0,  0,  9,  0, }, // Venezuela
    {    1, 1821,   12858,   12869,   12898, 11, 29, 27, }, // Vladivostok
    {    1, 1833,   12925,   12934,   12961,  9, 27, 25, }, // Volgograd
    {    1, 1843,       0,   12986,       0,  0, 10,  0, }, // Vostok
    {    1, 1850,       0,   12996,       0,  0,  4,  0, }, // Wake
    {    1, 1855,       0,   13000,       0,  0, 15,  0, }, // Wallis
    {    1, 1862,   13015,   13022,   13047,  7, 25, 23, }, // Yakutsk
    {    1, 1870,   13070,   13082,   13112, 12, 30, 28, }, // Yekaterinburg
    {    1, 1892,       0,   13140,       0,  0,  5,  0, }, // Yukon
    {    5,    5,       0,   13145,       0,  0, 14,  0, }, // Afghanistan Afrikaans/Latin/South Africa
    {    5,   17,       0,   13159,       0,  0, 19,  0, }, // Africa_Central
    {    5,   32,       0,   13178,       0,  0, 14,  0, }, // Africa_Eastern
    {    5,   65,       0,   13192,       0,  0, 24,  0, }, // Africa_Southern
    {    5,   81,   13216,   13230,   13253, 14, 23, 19, }, // Africa_Western
    {    5,  107,   13272,   13282,   13301, 10, 19, 16, }, // Alaska
    {    5,  135,   13317,   13328,   13348, 11, 20, 16, }, // Amazon
    {    5,  142,   13364,   13394,   13433, 30, 39, 36, }, // America_Central
    {    5,  158,   13469,   13500,   13540, 31, 40, 37, }, // America_Eastern
    {    5,  174,   13577,   13602,   13637, 25, 35, 32, }, // America_Mountain
    {    5,  191,   13669,   13682,   13704, 13, 22, 19, }, // America_Pacific
    {    5,  207,   13723,   13733,   13752, 10, 19, 15, }, // Anadyr
    {    5,  214,   13767,   13775,   13792,  8, 17, 14, }, // Apia
    {    5,  232,   13806,   13818,   13839, 12, 21, 18, }, // Arabian
    {    5,  240,   13857,   13871,   13894, 14, 23, 19, }, // Argentina
    {    5,  250,   13913,   13931,   13958, 18, 27, 23, }, // Argentina_Western
    {    5,  268,   13981,   13992,   14012, 11, 20, 16, }, // Armenia
    {    5,  286,   14028,   14042,   14065, 14, 23, 20, }, // Atlantic
    {    5,  295,   14085,   14109,   14142, 24, 33, 30, }, // Australia_Central
    {    5,  313,   14172,   14206,   14250, 34, 44, 40, }, // Australia_CentralWestern
    {    5,  338,   14290,   14315,   14349, 25, 34, 31, }, // Australia_Eastern
    {    5,  356,   14380,   14405,   14439, 25, 34, 31, }, // Australia_Western
    {    5,  374,   14470,   14485,   14509, 15, 24, 20, }, // Azerbaijan
    {    5,  385,   14529,   14538,   14556,  9, 18, 14, }, // Azores
    {    5,  397,   14570,   14584,   14607, 14, 23, 19, }, // Bangladesh
    {    5,  415,       0,   14626,       0,  0, 11,  0, }, // Bhutan
    {    5,  422,       0,   14637,       0,  0, 11,  0, }, // Bolivia
    {    5,  437,   14648,   14660,   14681, 12, 21, 17, }, // Brasilia
    {    5,  454,       0,   14698,       0,  0, 22,  0, }, // Brunei
    {    5,  461,   14720,   14734,   14757, 14, 23, 19, }, // Cape_Verde
    {    5,  478,       0,   14776,       0,  0, 21,  0, }, // Chamorro
    {    5,  487,   14797,   14808,   14828, 11, 20, 17, }, // Chatham
    {    5,  495,   14845,   14854,   14872,  9, 18, 14, }, // Chile
    {    5,  501,   14886,   14895,   14913,  9, 18, 15, }, // China
    {    5,  507,   14928,   14942,   14965, 14, 23, 19, }, // Choibalsan
    {    5,  518,       0,   14984,       0,  0, 19,  0, }, // Christmas
    {    5,  528,       0,   15003,       0,  0, 16,  0, }, // Cocos
    {    5,  534,   15019,   15031,   15052, 12, 21, 17, }, // Colombia
    {    5,  543,   15069,   15084,   15108, 15, 24, 24, }, // Cook
    {    5,  548,   15132,   15140,   15157,  8, 17, 14, }, // Cuba
    {    5,  559,       0,   15171,       0,  0,  9,  0, }, // Davis
    {    5,  575,       0,   15180,       0,  0, 20,  0, }, // DumontDUrville
    {    5,  612,       0,   15200,       0,  0, 13,  0, }, // East_Timor
    {    5,  623,   15213,   15227,   15250, 14, 23, 19, }, // Easter
    {    5,  630,       0,   15269,       0,  0, 11,  0, }, // Ecuador
    {    5,  638,   15280,   15301,   15331, 21, 30, 26, }, // Europe_Central
    {    5,  653,   15357,   15373,   15398, 16, 25, 21, }, // Europe_Eastern
    {    5,  668,       0,   15419,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {    5,  691,   15442,   15458,   15483, 16, 25, 21, }, // Europe_Western
    {    5,  706,   15504,   15523,   15551, 19, 28, 24, }, // Falkland
    {    5,  715,   15575,   15584,   15602,  9, 18, 14, }, // Fiji
    {    5,  720,       0,   15616,       0,  0, 16,  0, }, // French_Guiana
    {    5,  734,       0,   15632,       0,  0, 33,  0, }, // French_Southern
    {    5,  757,       0,   15665,       0,  0, 13,  0, }, // Galapagos
    {    5,  767,       0,   15678,       0,  0, 11,  0, }, // Gambier
    {    5,  775,   15689,   15700,   15720, 11, 20, 16, }, // Georgia
    {    5,  783,       0,   15736,       0,  0, 18,  0, }, // Gilbert_Islands
    {    5,  799,       0,   15754,       0,  0, 13,  0, }, // GMT
    {    5,  841,   15767,   15784,   15810, 17, 26, 22, }, // Greenland_Eastern
    {    5,  859,   15832,   15849,   15875, 17, 26, 22, }, // Greenland_Western
    {    5,  882,       0,   15897,       0,  0, 26,  0, }, // Gulf
    {    5,  887,       0,   15622,       0,  0, 10,  0, }, // Guyana
    {    5,  894,   15923,   15941,   15968, 18, 27, 24, }, // Hawaii_Aleutian
    {    5,  910,   15992,   16004,   16025, 12, 21, 17, }, // Hong_Kong
    {    5,  920,   16042,   16050,   16067,  8, 17, 13, }, // Hovd
    {    5,  925,       0,   16080,       0,  0, 18,  0, }, // India
    {    5,  931,       0,   16098,       0,  0, 18,  0, }, // Indian_Ocean
    {    5,  944,       0,   16116,       0,  0, 13,  0, }, // Indochina
    {    5,  954,       0,   16129,       0,  0, 24,  0, }, // Indonesia_Central
    {    5,  972,       0,   16153,       0,  0, 17,  0, }, // Indonesia_Eastern
    {    5,  990,       0,   16170,       0,  0, 17,  0, }, // Indonesia_Western
    {    5, 1008,   16187,   16195,   16212,  8, 17, 14, }, // Iran
    {    5, 1019,   16226,   16238,   16259, 12, 21, 17, }, // Irkutsk
    {    5, 1027,   16276,   16286,   16305, 10, 19, 16, }, // Israel
    {    5, 1034,   16321,   16330,   16348,  9, 18, 15, }, // Japan
    {    5, 1040,   16363,   16391,   16428, 28, 37, 33, }, // Kamchatka
    {    5, 1058,       0,   16461,       0,  0, 17,  0, }, // Kazakhstan_Eastern
    {    5, 1077,       0,   16478,       0,  0, 17,  0, }, // Kazakhstan_Western
    {    5, 1106,   16495,   16508,   16530, 13, 22, 19, }, // Korea
    {    5, 1112,       0,   16549,       0,  0, 10,  0, }, // Kosrae
    {    5, 1119,   16559,   16574,   16598, 15, 24, 20, }, // Krasnoyarsk
    {    5, 1151,       0,   16618,       0,  0, 13,  0, }, // Kyrgystan
    {    5, 1175,       0,   16631,       0,  0, 16,  0, }, // Line_Islands
    {    5, 1188,   16647,   16660,   16682, 13, 22, 19, }, // Lord_Howe
    {    5, 1204,   16701,   16712,   16732, 11, 20, 16, }, // Magadan
    {    5, 1219,       0,   16748,       0,  0, 12,  0, }, // Malaysia
    {    5, 1228,       0,   16760,       0,  0, 12,  0, }, // Maldives
    {    5, 1237,       0,   16772,       0,  0, 13,  0, }, // Marquesas
    {    5, 1247,       0,   16785,       0,  0, 19,  0, }, // Marshall_Islands
    {    5, 1264,   16804,   16817,   16839, 13, 22, 18, }, // Mauritius
    {    5, 1274,       0,   16857,       0,  0, 10,  0, }, // Mawson
    {    5, 1281,   16867,   16892,   16926, 25, 34, 31, }, // Mexico_Pacific
    {    5, 1296,   16957,   16972,   16996, 15, 24, 20, }, // Mongolia
    {    5, 1305,   17016,   17026,   17045, 10, 19, 15, }, // Moscow
    {    5, 1312,       0,   17060,       0,  0, 11,  0, }, // Myanmar
    {    5, 1320,       0,   17071,       0,  0,  9,  0, }, // Nauru
    {    5, 1326,       0,   17080,       0,  0,  9,  0, }, // Nepal
    {    5, 1332,   17089,   17107,   17134, 18, 27, 23, }, // New_Caledonia
    {    5, 1346,   17157,   17173,   17198, 16, 25, 22, }, // New_Zealand
    {    5, 1358,   17220,   17236,   17261, 16, 25, 22, }, // Newfoundland
    {    5, 1371,       0,   17283,       0,  0,  8,  0, }, // Niue
    {    5, 1376,   17291,   17308,   17334, 17, 26, 23, }, // Norfolk
    {    5, 1384,   17357,   17380,   17412, 23, 32, 28, }, // Noronha
    {    5, 1406,   17440,   17455,   17479, 15, 24, 20, }, // Novosibirsk
    {    5, 1418,   17499,   17507,   17524,  8, 17, 13, }, // Omsk
    {    5, 1428,   17537,   17549,   17570, 12, 21, 17, }, // Pakistan
    {    5, 1437,       0,   17587,       0,  0,  9,  0, }, // Palau
    {    5, 1443,       0,   17596,       0,  0, 22,  0, }, // Papua_New_Guinea
    {    5, 1460,   17618,   17630,   17651, 12, 21, 17, }, // Paraguay
    {    5, 1469,   17668,   17676,   17693,  8, 17, 13, }, // Peru
    {    5, 1474,   17706,   17720,   17743, 14, 23, 19, }, // Philippines
    {    5, 1486,       0,   17762,       0,  0, 17,  0, }, // Phoenix_Islands
    {    5, 1502,   17779,   17806,   17842, 27, 36, 33, }, // Pierre_Miquelon
    {    5, 1518,       0,   17875,       0,  0, 12,  0, }, // Pitcairn
    {    5, 1527,       0,   17887,       0,  0, 10,  0, }, // Ponape
    {    5, 1534,       0,   17897,       0,  0, 13,  0, }, // Pyongyang
    {    5, 1554,       0,   17910,       0,  0, 11,  0, }, // Reunion
    {    5, 1562,       0,   17921,       0,  0, 11,  0, }, // Rothera
    {    5, 1570,   17932,   17944,   17965, 12, 21, 17, }, // Sakhalin
    {    5, 1579,   17982,   17992,   18011, 10, 19, 16, }, // Samara
    {    5, 1596,   18027,   18036,   18054,  9, 18, 15, }, // Samoa
    {    5, 1602,       0,   18069,       0,  0, 13,  0, }, // Seychelles
    {    5, 1624,       0,   18082,       0,  0, 22,  0, }, // Singapore
    {    5, 1634,       0,   18104,       0,  0, 19,  0, }, // Solomon
    {    5, 1642,       0,   18123,       0,  0, 16,  0, }, // South_Georgia
    {    5, 1656,       0,   18139,       0,  0, 12,  0, }, // Suriname
    {    5, 1676,       0,   18151,       0,  0,  9,  0, }, // Syowa
    {    5, 1682,       0,   18160,       0,  0, 10,  0, }, // Tahiti
    {    5, 1689,   18170,   18180,   18199, 10, 19, 16, }, // Taipei
    {    5, 1696,       0,   18215,       0,  0, 15,  0, }, // Tajikistan
    {    5, 1724,       0,   18230,       0,  0, 11,  0, }, // Tokelau
    {    5, 1732,   18241,   18250,   18268,  9, 18, 14, }, // Tonga
    {    5, 1738,       0,   18282,       0,  0,  9,  0, }, // Truk
    {    5, 1750,   18291,   18307,   18332, 16, 25, 21, }, // Turkmenistan
    {    5, 1763,       0,   18353,       0,  0, 10,  0, }, // Tuvalu
    {    5, 1777,   18363,   18374,   18394, 11, 20, 16, }, // Uruguay
    {    5, 1792,   18410,   18425,   18449, 15, 24, 20, }, // Uzbekistan
    {    5, 1803,   18469,   18480,   18500, 11, 20, 16, }, // Vanuatu
    {    5, 1811,       0,   18516,       0,  0, 13,  0, }, // Venezuela
    {    5, 1821,   18529,   18544,   18568, 15, 24, 20, }, // Vladivostok
    {    5, 1833,   18588,   18601,   18623, 13, 22, 18, }, // Volgograd
    {    5, 1843,       0,   18641,       0,  0, 10,  0, }, // Vostok
    {    5, 1850,       0,   18651,       0,  0, 15,  0, }, // Wake
    {    5, 1855,       0,   18666,       0,  0, 20,  0, }, // Wallis
    {    5, 1862,   18686,   18698,   18719, 12, 21, 17, }, // Yakutsk
    {    5, 1870,   18736,   18753,   18779, 17, 26, 22, }, // Yekaterinburg
    {    5, 1892,       0,   18801,       0,  0,  8,  0, }, // Yukon
    {   10,    0,   18809,   18827,   18855, 18, 28, 25, }, // Acre Albanian/Latin/Albania
    {   10,    5,       0,   18880,       0,  0, 18,  0, }, // Afghanistan
    {   10,   17,       0,   18898,       0,  0, 22,  0, }, // Africa_Central
    {   10,   32,       0,   18920,       0,  0, 21,  0, }, // Africa_Eastern
    {   10,   65,       0,   18941,       0,  0, 30,  0, }, // Africa_Southern
    {   10,   81,   18971,   18996,   19031, 25, 35, 32, }, // Africa_Western
    {   10,  107,   19063,   19076,   19099, 13, 23, 20, }, // Alaska
    {   10,  128,   19119,   19132,   19155, 13, 23, 20, }, // Almaty
    {   10,  135,   19175,   19189,   19213, 14, 24, 21, }, // Amazon
    {   10,  142,   19234,   19256,   19288, 22, 32, 29, }, // America_Central
    {   10,  158,   19317,   19338,   19369, 21, 31, 28, }, // America_Eastern
    {   10,  174,   19397,   19440,   19493, 43, 53, 50, }, // America_Mountain
    {   10,  191,   19543,   19593,   19653, 50, 60, 57, }, // America_Pacific
    {   10,  207,   19710,   19724,   19748, 14, 24, 21, }, // Anadyr
    {   10,  214,   19769,   19780,   19801, 11, 21, 18, }, // Apia
    {   10,  219,   19819,   19831,   19853, 12, 22, 19, }, // Aqtau
    {   10,  225,   19872,   19885,   19908, 13, 23, 20, }, // Aqtobe
    {   10,  232,   19928,   19937,   19956,  9, 19, 16, }, // Arabian
    {   10,  240,   19972,   19989,   20016, 17, 27, 24, }, // Argentina
    {   10,  250,   20040,   20069,   20108, 29, 39, 36, }, // Argentina_Western
    {   10,  268,   20144,   20158,   20182, 14, 24, 21, }, // Armenia
    {   10,  286,   20203,   20219,   20245, 16, 26, 23, }, // Atlantic
    {   10,  295,   20268,   20293,   20328, 25, 35, 32, }, // Australia_Central
    {   10,  313,   20360,   20397,   20444, 37, 47, 44, }, // Australia_CentralWestern
    {   10,  338,   20488,   20512,   20546, 24, 34, 31, }, // Australia_Eastern
    {   10,  356,   20577,   20605,   20643, 28, 38, 35, }, // Australia_Western
    {   10,  374,   20678,   20697,   20726, 19, 29, 26, }, // Azerbaijan
    {   10,  385,   20752,   20765,   20788, 13, 23, 20, }, // Azores
    {   10,  397,   20808,   20826,   20854, 18, 28, 25, }, // Bangladesh
    {   10,  415,       0,   20879,       0,  0, 13,  0, }, // Bhutan
    {   10,  422,       0,   20892,       0,  0, 14,  0, }, // Bolivia
    {   10,  437,   20906,   20921,   20946, 15, 25, 22, }, // Brasilia
    {   10,  454,       0,   20968,       0,  0, 24,  0, }, // Brunei
    {   10,  461,   20992,   21014,   21046, 22, 32, 29, }, // Cape_Verde
    {   10,  472,       0,   21075,       0,  0, 12,  0, }, // Casey
    {   10,  478,       0,   21087,       0,  0, 14,  0, }, // Chamorro
    {   10,  487,   21101,   21114,   21137, 13, 23, 20, }, // Chatham
    {   10,  495,   21157,   21168,   21189, 11, 21, 18, }, // Chile
    {   10,  501,   21207,   21218,   21239, 11, 21, 18, }, // China
    {   10,  507,   21257,   21274,   21301, 17, 27, 24, }, // Choibalsan
    {   10,  518,       0,   21325,       0,  0, 32,  0, }, // Christmas
    {   10,  528,       0,   21357,       0,  0, 19,  0, }, // Cocos
    {   10,  534,   21376,   21391,   21416, 15, 25, 22, }, // Colombia
    {   10,  543,   21438,   21455,   21482, 17, 27, 24, }, // Cook
    {   10,  548,   21506,   21517,   21538, 11, 21, 18, }, // Cuba
    {   10,  559,       0,   21556,       0,  0, 14,  0, }, // Davis
    {   10,  575,       0,   21570,       0,  0, 22,  0, }, // DumontDUrville
    {   10,  612,       0,   21592,       0,  0, 20,  0, }, // East_Timor
    {   10,  623,   21612,   21637,   21672, 25, 35, 32, }, // Easter
    {   10,  630,       0,   21704,       0,  0, 15,  0, }, // Ecuador
    {   10,  638,   21719,   21741,   21773, 22, 32, 29, }, // Europe_Central
    {   10,  653,   21802,   21823,   21854, 21, 31, 28, }, // Europe_Eastern
    {   10,  668,       0,   21882,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {   10,  691,   21913,   21938,   21973, 25, 35, 32, }, // Europe_Western
    {   10,  706,   22005,   22027,   22059, 22, 32, 29, }, // Falkland
    {   10,  715,   22088,   22100,   22122, 12, 22, 19, }, // Fiji
    {   10,  720,       0,   22141,       0,  0, 23,  0, }, // French_Guiana
    {   10,  734,       0,   22164,       0,  0, 48,  0, }, // French_Southern
    {   10,  757,       0,   22212,       0,  0, 17,  0, }, // Galapagos
    {   10,  767,       0,   22229,       0,  0, 15,  0, }, // Gambier
    {   10,  775,   22244,   22260,   22286, 16, 26, 23, }, // Georgia
    {   10,  783,       0,   22309,       0,  0, 21,  0, }, // Gilbert_Islands
    {   10,  799,       0,   22330,       0,  0, 15,  0, }, // GMT
    {   10,  841,   22345,   22369,   22403, 24, 34, 31, }, // Greenland_Eastern
    {   10,  859,   22434,   22462,   22500, 28, 38, 35, }, // Greenland_Western
    {   10,  877,       0,   22535,       0,  0, 12,  0, }, // Guam
    {   10,  882,       0,   22547,       0,  0, 12,  0, }, // Gulf
    {   10,  887,       0,   22141,       0,  0, 14,  0, }, // Guyana
    {   10,  894,   22559,   22587,   22625, 28, 38, 35, }, // Hawaii_Aleutian
    {   10,  910,   22660,   22677,   22704, 17, 27, 24, }, // Hong_Kong
    {   10,  920,   22728,   22740,   22762, 12, 22, 19, }, // Hovd
    {   10,  925,       0,   22781,       0,  0, 22,  0, }, // India
    {   10,  931,       0,   22803,       0,  0, 20,  0, }, // Indian_Ocean
    {   10,  944,       0,   22823,       0,  0, 15,  0, }, // Indochina
    {   10,  954,       0,   22838,       0,  0, 25,  0, }, // Indonesia_Central
    {   10,  972,       0,   22863,       0,  0, 24,  0, }, // Indonesia_Eastern
    {   10,  990,       0,   22887,       0,  0, 28,  0, }, // Indonesia_Western
    {   10, 1008,   22915,   22927,   22949, 12, 22, 19, }, // Iran
    {   10, 1019,   22968,   22983,   23008, 15, 25, 22, }, // Irkutsk
    {   10, 1027,   23030,   23044,   23068, 14, 24, 21, }, // Israel
    {   10, 1034,   23089,   23103,   23127, 14, 24, 21, }, // Japan
    {   10, 1040,   23148,   23178,   23218, 30, 40, 37, }, // Kamchatka
    {   10, 1058,       0,   23255,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {   10, 1077,       0,   23280,       0,  0, 29,  0, }, // Kazakhstan_Western
    {   10, 1106,   23309,   23320,   23341, 11, 21, 18, }, // Korea
    {   10, 1112,       0,   23359,       0,  0, 12,  0, }, // Kosrae
    {   10, 1119,   23371,   23390,   23419, 19, 29, 26, }, // Krasnoyarsk
    {   10, 1151,       0,   23445,       0,  0, 15,  0, }, // Kyrgystan
    {   10, 1161,       0,   23460,       0,  0, 13,  0, }, // Lanka
    {   10, 1175,       0,   23473,       0,  0, 35,  0, }, // Line_Islands
    {   10, 1188,   23508,   23524,   23550, 16, 26, 23, }, // Lord_Howe
    {   10, 1198,   23573,   23585,   23607, 12, 22, 19, }, // Macau
    {   10, 1204,   23626,   23641,   23666, 15, 25, 22, }, // Magadan
    {   10, 1219,       0,   23688,       0,  0, 15,  0, }, // Malaysia
    {   10, 1228,       0,   23703,       0,  0, 15,  0, }, // Maldives
    {   10, 1237,       0,   23718,       0,  0, 21,  0, }, // Marquesas
    {   10, 1247,       0,   23739,       0,  0, 22,  0, }, // Marshall_Islands
    {   10, 1264,   23761,   23778,   23805, 17, 27, 24, }, // Mauritius
    {   10, 1274,       0,   23829,       0,  0, 14,  0, }, // Mawson
    {   10, 1281,   23843,   23893,   23953, 50, 60, 57, }, // Mexico_Pacific
    {   10, 1296,   24010,   24028,   24056, 18, 28, 25, }, // Mongolia
    {   10, 1305,   24081,   24093,   24115, 12, 22, 19, }, // Moscow
    {   10, 1312,       0,   24134,       0,  0, 15,  0, }, // Myanmar
    {   10, 1320,       0,   24149,       0,  0, 13,  0, }, // Nauru
    {   10, 1326,       0,   24162,       0,  0, 13,  0, }, // Nepal
    {   10, 1332,   24175,   24197,   24229, 22, 32, 29, }, // New_Caledonia
    {   10, 1346,   24258,   24278,   24308, 20, 30, 27, }, // New_Zealand
    {   10, 1358,   24335,   24369,   24413, 34, 44, 41, }, // Newfoundland
    {   10, 1371,       0,   24454,       0,  0, 12,  0, }, // Niue
    {   10, 1376,   24466,   24488,   24520, 22, 32, 29, }, // Norfolk
    {   10, 1384,   24549,   24575,   24611, 26, 36, 33, }, // Noronha
    {   10, 1392,       0,   24644,       0,  0, 35,  0, }, // North_Mariana
    {   10, 1406,   24679,   24698,   24727, 19, 29, 26, }, // Novosibirsk
    {   10, 1418,   24753,   24765,   24787, 12, 22, 19, }, // Omsk
    {   10, 1428,   24806,   24822,   24848, 16, 26, 23, }, // Pakistan
    {   10, 1437,       0,   24871,       0,  0, 12,  0, }, // Palau
    {   10, 1443,       0,   24883,       0,  0, 25,  0, }, // Papua_New_Guinea
    {   10, 1460,   24908,   24923,   24948, 15, 25, 22, }, // Paraguay
    {   10, 1469,   24970,   24982,   25004, 12, 22, 19, }, // Peru
    {   10, 1474,   25023,   25039,   25065, 16, 26, 23, }, // Philippines
    {   10, 1486,       0,   25088,       0,  0, 20,  0, }, // Phoenix_Islands
    {   10, 1502,   25108,   25135,   25172, 27, 37, 34, }, // Pierre_Miquelon
    {   10, 1518,       0,   25206,       0,  0, 15,  0, }, // Pitcairn
    {   10, 1527,       0,   25221,       0,  0, 14,  0, }, // Ponape
    {   10, 1534,       0,   25235,       0,  0, 14,  0, }, // Pyongyang
    {   10, 1544,   25249,   25265,   25291, 16, 26, 23, }, // Qyzylorda
    {   10, 1554,       0,   25314,       0,  0, 15,  0, }, // Reunion
    {   10, 1562,       0,   25329,       0,  0, 14,  0, }, // Rothera
    {   10, 1570,   25343,   25358,   25383, 15, 25, 22, }, // Sakhalin
    {   10, 1579,   25405,   25418,   25441, 13, 23, 20, }, // Samara
    {   10, 1596,   25461,   25473,   25495, 12, 22, 19, }, // Samoa
    {   10, 1602,       0,   25514,       0,  0, 17,  0, }, // Seychelles
    {   10, 1624,       0,   25531,       0,  0, 16,  0, }, // Singapore
    {   10, 1634,       0,   25547,       0,  0, 21,  0, }, // Solomon
    {   10, 1642,       0,   25568,       0,  0, 23,  0, }, // South_Georgia
    {   10, 1656,       0,   25591,       0,  0, 15,  0, }, // Suriname
    {   10, 1676,       0,   25606,       0,  0, 12,  0, }, // Syowa
    {   10, 1682,       0,   25618,       0,  0, 13,  0, }, // Tahiti
    {   10, 1689,   25631,   25644,   25667, 13, 23, 20, }, // Taipei
    {   10, 1696,       0,   25687,       0,  0, 19,  0, }, // Tajikistan
    {   10, 1724,       0,   25706,       0,  0, 14,  0, }, // Tokelau
    {   10, 1732,   25720,   25732,   25754, 12, 22, 19, }, // Tonga
    {   10, 1738,       0,   25773,       0,  0, 11,  0, }, // Truk
    {   10, 1750,   25784,   25804,   25834, 20, 30, 27, }, // Turkmenistan
    {   10, 1763,       0,   25861,       0,  0, 14,  0, }, // Tuvalu
    {   10, 1777,   25875,   25889,   25913, 14, 24, 21, }, // Uruguay
    {   10, 1792,   25934,   25952,   25980, 18, 28, 25, }, // Uzbekistan
    {   10, 1803,   26005,   26020,   26045, 15, 25, 22, }, // Vanuatu
    {   10, 1811,       0,   26067,       0,  0, 16,  0, }, // Venezuela
    {   10, 1821,   26083,   26102,   26131, 19, 29, 26, }, // Vladivostok
    {   10, 1833,   26157,   26174,   26201, 17, 27, 24, }, // Volgograd
    {   10, 1843,       0,   26225,       0,  0, 14,  0, }, // Vostok
    {   10, 1850,       0,   26239,       0,  0, 19,  0, }, // Wake
    {   10, 1855,       0,   26258,       0,  0, 26,  0, }, // Wallis
    {   10, 1862,   26284,   26299,   26324, 15, 25, 22, }, // Yakutsk
    {   10, 1870,   26346,   26366,   26396, 20, 30, 27, }, // Yekaterinburg
    {   10, 1892,       0,   26423,       0,  0, 13,  0, }, // Yukon
    {   13,    5,       0,   26436,       0,  0, 12,  0, }, // Afghanistan Amharic/Ethiopic/Ethiopia
    {   13,   17,       0,   26448,       0,  0, 16,  0, }, // Africa_Central
    {   13,   32,       0,   26464,       0,  0, 14,  0, }, // Africa_Eastern
    {   13,   65,       0,   26478,       0,  0, 18,  0, }, // Africa_Southern
    {   13,   81,   26496,   26510,   26529, 14, 19, 19, }, // Africa_Western
    {   13,  107,   26548,   26563,   26584, 15, 21, 19, }, // Alaska
    {   13,  135,   26603,   26618,   26638, 15, 20, 19, }, // Amazon
    {   13,  142,   26657,   26683,   26714, 26, 31, 20, }, // America_Central
    {   13,  158,   26734,   26749,   26769, 15, 20, 19, }, // America_Eastern
    {   13,  174,   26788,   26803,   26823, 15, 20, 17, }, // America_Mountain
    {   13,  191,   26840,   26855,   26875, 15, 20, 19, }, // America_Pacific
    {   13,  207,   26894,   26910,   26926, 16, 16, 21, }, // Anadyr
    {   13,  214,   26947,   26955,   26968,  8, 13, 15, }, // Apia
    {   13,  232,   26983,   26992,   27006,  9, 14, 18, }, // Arabian
    {   13,  240,   27024,   27042,   27064, 18, 22, 21, }, // Argentina
    {   13,  250,   27085,   27108,   27136, 23, 28, 27, }, // Argentina_Western
    {   13,  268,   27163,   27173,   27188, 10, 15, 15, }, // Armenia
    {   13,  286,   27203,   27221,   27244, 18, 23, 21, }, // Atlantic
    {   13,  295,   27265,   27290,   27320, 25, 30, 28, }, // Australia_Central
    {   13,  313,   27348,   27378,   27413, 30, 35, 34, }, // Australia_CentralWestern
    {   13,  338,   27447,   27472,   27502, 25, 30, 28, }, // Australia_Eastern
    {   13,  356,   27530,   27554,   27583, 24, 29, 28, }, // Australia_Western
    {   13,  374,   27611,   27622,   27638, 11, 16, 16, }, // Azerbaijan
    {   13,  385,   27654,   27663,   27677,  9, 14, 14, }, // Azores
    {   13,  397,   27691,   27702,   27718, 11, 16, 16, }, // Bangladesh
    {   13,  415,       0,   27734,       0,  0,  8,  0, }, // Bhutan
    {   13,  422,       0,   27742,       0,  0,  9,  0, }, // Bolivia
    {   13,  437,   27751,   27767,   27788, 16, 21, 19, }, // Brasilia
    {   13,  454,       0,   27807,       0,  0, 15,  0, }, // Brunei
    {   13,  461,   27822,   27833,   27849, 11, 16, 16, }, // Cape_Verde
    {   13,  478,       0,   27865,       0,  0, 13,  0, }, // Chamorro
    {   13,  487,   27878,   27886,   27899,  8, 13, 17, }, // Chatham
    {   13,  495,   27916,   27923,   27935,  7, 12, 12, }, // Chile
    {   13,  501,   27947,   27955,   27968,  8, 13, 17, }, // China
    {   13,  507,   27985,   28001,   28024, 16, 23, 22, }, // Choibalsan
    {   13,  518,       0,   28046,       0,  0, 11,  0, }, // Christmas
    {   13,  528,       0,   28057,       0,  0, 13,  0, }, // Cocos
    {   13,  534,   28070,   28080,   28095, 10, 15, 15, }, // Colombia
    {   13,  543,   28110,   28122,   28139, 12, 17, 21, }, // Cook
    {   13,  548,   28160,   28166,   28178,  6, 12, 16, }, // Cuba
    {   13,  559,       0,   28194,       0,  0,  8,  0, }, // Davis
    {   13,  575,       0,   28202,       0,  0, 14,  0, }, // DumontDUrville
    {   13,  612,       0,   28216,       0,  0, 13,  0, }, // East_Timor
    {   13,  623,   28229,   28242,   28260, 13, 18, 18, }, // Easter
    {   13,  630,       0,   28278,       0,  0,  9,  0, }, // Ecuador
    {   13,  638,   28287,   28303,   28324, 16, 21, 21, }, // Europe_Central
    {   13,  653,   28345,   28360,   28380, 15, 20, 20, }, // Europe_Eastern
    {   13,  668,       0,   28400,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {   13,  691,   28418,   28433,   28453, 15, 20, 20, }, // Europe_Western
    {   13,  706,   28473,   28489,   28510, 16, 21, 21, }, // Falkland
    {   13,  715,   28531,   28538,   28550,  7, 12, 12, }, // Fiji
    {   13,  720,       0,   28562,       0,  0, 14,  0, }, // French_Guiana
    {   13,  734,       0,   28576,       0,  0, 26,  0, }, // French_Southern
    {   13,  757,       0,   28602,       0,  0, 10,  0, }, // Galapagos
    {   13,  767,       0,   28612,       0,  0, 10,  0, }, // Gambier
    {   13,  775,   28622,   28632,   28647, 10, 15, 15, }, // Georgia
    {   13,  783,       0,   28662,       0,  0, 15,  0, }, // Gilbert_Islands
    {   13,  799,       0,   28677,       0,  0, 15,  0, }, // GMT
    {   13,  841,   28692,   28708,   28729, 16, 21, 21, }, // Greenland_Eastern
    {   13,  859,   28750,   28766,   28787, 16, 21, 21, }, // Greenland_Western
    {   13,  882,       0,   28808,       0,  0, 16,  0, }, // Gulf
    {   13,  887,       0,   28824,       0,  0,  8,  0, }, // Guyana
    {   13,  894,   28832,   28851,   28875, 19, 24, 23, }, // Hawaii_Aleutian
    {   13,  910,   28898,   28910,   28927, 12, 17, 17, }, // Hong_Kong
    {   13,  920,   28944,   28958,   28978, 14, 20, 18, }, // Hovd
    {   13,  925,       0,   28996,       0,  0, 13,  0, }, // India
    {   13,  931,       0,   29009,       0,  0, 14,  0, }, // Indian_Ocean
    {   13,  944,       0,   29023,       0,  0, 11,  0, }, // Indochina
    {   13,  954,       0,   29034,       0,  0, 18,  0, }, // Indonesia_Central
    {   13,  972,       0,   29052,       0,  0, 17,  0, }, // Indonesia_Eastern
    {   13,  990,       0,   29069,       0,  0, 17,  0, }, // Indonesia_Western
    {   13, 1008,   29086,   29094,   29107,  8, 13, 17, }, // Iran
    {   13, 1019,   29124,   29141,   29164, 17, 23, 21, }, // Irkutsk
    {   13, 1027,   29185,   29195,   29210, 10, 15, 19, }, // Israel
    {   13, 1034,   29229,   29237,   29250,  8, 13, 17, }, // Japan
    {   13, 1040,   29267,   29283,   29311, 16, 28, 32, }, // Kamchatka
    {   13, 1058,       0,   29343,       0,  0, 16,  0, }, // Kazakhstan_Eastern
    {   13, 1077,       0,   29359,       0,  0, 16,  0, }, // Kazakhstan_Western
    {   13, 1106,   29375,   29383,   29396,  8, 13, 17, }, // Korea
    {   13, 1112,       0,   29413,       0,  0,  9,  0, }, // Kosrae
    {   13, 1119,   29422,   29441,   29466, 19, 25, 23, }, // Krasnoyarsk
    {   13, 1151,       0,   29489,       0,  0, 11,  0, }, // Kyrgystan
    {   13, 1175,       0,   29500,       0,  0, 13,  0, }, // Line_Islands
    {   13, 1188,   29513,   29531,   29554, 18, 23, 21, }, // Lord_Howe
    {   13, 1204,   29575,   29591,   29611, 16, 20, 18, }, // Magadan
    {   13, 1219,       0,   29629,       0,  0, 10,  0, }, // Malaysia
    {   13, 1228,       0,   29639,       0,  0, 10,  0, }, // Maldives
    {   13, 1237,       0,   29649,       0,  0, 10,  0, }, // Marquesas
    {   13, 1247,       0,   29659,       0,  0, 14,  0, }, // Marshall_Islands
    {   13, 1264,   29673,   29684,   29700, 11, 16, 16, }, // Mauritius
    {   13, 1274,       0,   29716,       0,  0,  9,  0, }, // Mawson
    {   13, 1281,   29725,   29745,   29770, 20, 25, 24, }, // Mexico_Pacific
    {   13, 1296,   29794,   29805,   29828, 11, 23, 22, }, // Mongolia
    {   13, 1305,   29850,   29864,   29883, 14, 19, 18, }, // Moscow
    {   13, 1312,       0,   29901,       0,  0, 10,  0, }, // Myanmar
    {   13, 1320,       0,   29911,       0,  0,  8,  0, }, // Nauru
    {   13, 1326,       0,   29919,       0,  0,  8,  0, }, // Nepal
    {   13, 1332,   29927,   29940,   29958, 13, 18, 18, }, // New_Caledonia
    {   13, 1346,   29976,   29988,   30005, 12, 17, 21, }, // New_Zealand
    {   13, 1358,   30026,   30047,   30073, 21, 26, 25, }, // Newfoundland
    {   13, 1371,       0,   30098,       0,  0,  8,  0, }, // Niue
    {   13, 1376,   30106,   30121,   30148, 15, 27, 26, }, // Norfolk
    {   13, 1384,   30174,   30189,   30217, 15, 28, 28, }, // Noronha
    {   13, 1406,   30245,   30264,   30288, 19, 24, 22, }, // Novosibirsk
    {   13, 1418,   30310,   30326,   30346, 16, 20, 19, }, // Omsk
    {   13, 1428,   30365,   30375,   30390, 10, 15, 15, }, // Pakistan
    {   13, 1437,       0,   30405,       0,  0,  8,  0, }, // Palau
    {   13, 1443,       0,   30413,       0,  0, 13,  0, }, // Papua_New_Guinea
    {   13, 1460,   30426,   30435,   30449,  9, 14, 14, }, // Paraguay
    {   13, 1469,   30463,   30470,   30482,  7, 12, 12, }, // Peru
    {   13, 1474,   30494,   30503,   30517,  9, 14, 14, }, // Philippines
    {   13, 1486,       0,   30531,       0,  0, 14,  0, }, // Phoenix_Islands
    {   13, 1502,   30545,   30565,   30590, 20, 25, 29, }, // Pierre_Miquelon
    {   13, 1518,       0,   30619,       0,  0, 11,  0, }, // Pitcairn
    {   13, 1527,       0,   30630,       0,  0,  8,  0, }, // Ponape
    {   13, 1534,       0,   30638,       0,  0, 12,  0, }, // Pyongyang
    {   13, 1554,       0,   30650,       0,  0, 10,  0, }, // Reunion
    {   13, 1562,       0,   30660,       0,  0,  8,  0, }, // Rothera
    {   13, 1570,   30668,   30684,   30705, 16, 21, 20, }, // Sakhalin
    {   13, 1579,   30725,   30739,   30758, 14, 19, 18, }, // Samara
    {   13, 1596,   30776,   30784,   30797,  8, 13, 12, }, // Samoa
    {   13, 1602,       0,   30809,       0,  0,  9,  0, }, // Seychelles
    {   13, 1624,       0,   30818,       0,  0, 15,  0, }, // Singapore
    {   13, 1634,       0,   30833,       0,  0, 14,  0, }, // Solomon
    {   13, 1642,       0,   30847,       0,  0, 14,  0, }, // South_Georgia
    {   13, 1656,       0,   30861,       0,  0,  9,  0, }, // Suriname
    {   13, 1676,       0,   30870,       0,  0,  8,  0, }, // Syowa
    {   13, 1682,       0,   30878,       0,  0,  8,  0, }, // Tahiti
    {   13, 1689,   30886,   30895,   30909,  9, 14, 18, }, // Taipei
    {   13, 1696,       0,   30927,       0,  0, 11,  0, }, // Tajikistan
    {   13, 1724,       0,   30938,       0,  0,  9,  0, }, // Tokelau
    {   13, 1732,   30947,   30955,   30968,  8, 13, 13, }, // Tonga
    {   13, 1738,       0,   30981,       0,  0,  7,  0, }, // Truk
    {   13, 1750,   30988,   31001,   31019, 13, 18, 18, }, // Turkmenistan
    {   13, 1763,       0,   31037,       0,  0,  8,  0, }, // Tuvalu
    {   13, 1777,   31045,   31054,   31068,  9, 14, 14, }, // Uruguay
    {   13, 1792,   31082,   31094,   31111, 12, 17, 17, }, // Uzbekistan
    {   13, 1803,   31128,   31136,   31149,  8, 13, 13, }, // Vanuatu
    {   13, 1811,       0,   31162,       0,  0, 10,  0, }, // Venezuela
    {   13, 1821,   31172,   31191,   31215, 19, 24, 23, }, // Vladivostok
    {   13, 1833,   31238,   31255,   31276, 17, 21, 20, }, // Volgograd
    {   13, 1843,       0,   31296,       0,  0,  9,  0, }, // Vostok
    {   13, 1850,       0,   31305,       0,  0, 11,  0, }, // Wake
    {   13, 1855,       0,   31316,       0,  0, 15,  0, }, // Wallis
    {   13, 1862,   31331,   31347,   31367, 16, 20, 20, }, // Yakutsk
    {   13, 1870,   31387,   31406,   31430, 19, 24, 23, }, // Yekaterinburg
    {   13, 1892,       0,   31453,       0,  0,  7,  0, }, // Yukon
    {   14,    5,       0,   31460,       0,  0, 15,  0, }, // Afghanistan Arabic/Arabic/Egypt
    {   14,   17,       0,   31475,       0,  0, 17,  0, }, // Africa_Central
    {   14,   32,       0,   31492,       0,  0, 17,  0, }, // Africa_Eastern
    {   14,   65,       0,   31509,       0,  0, 18,  0, }, // Africa_Southern
    {   14,   81,   31527,   31544,   31568, 17, 24, 24, }, // Africa_Western
    {   14,  107,   31592,   31604,   31626, 12, 22, 19, }, // Alaska
    {   14,  135,   31645,   31659,   31680, 14, 21, 21, }, // Amazon
    {   14,  142,   31701,   31733,   31772, 32, 39, 39, }, // America_Central
    {   14,  158,   31811,   31842,   31880, 31, 38, 38, }, // America_Eastern
    {   14,  174,   31918,   31949,   31987, 31, 38, 38, }, // America_Mountain
    {   14,  191,   32025,   32044,   32070, 19, 26, 26, }, // America_Pacific
    {   14,  207,   32096,   32108,   32127, 12, 19, 22, }, // Anadyr
    {   14,  214,   32149,   32159,   32179, 10, 20, 20, }, // Apia
    {   14,  232,   32199,   32213,   32234, 14, 21, 21, }, // Arabian
    {   14,  240,   32255,   32270,   32292, 15, 22, 22, }, // Argentina
    {   14,  250,   32314,   32333,   32359, 19, 26, 26, }, // Argentina_Western
    {   14,  268,   32385,   32398,   32418, 13, 20, 20, }, // Armenia
    {   14,  286,   32438,   32451,   32473, 13, 22, 22, }, // Atlantic
    {   14,  295,   32495,   32513,   32538, 18, 25, 25, }, // Australia_Central
    {   14,  313,   32563,   32585,   32614, 22, 29, 29, }, // Australia_CentralWestern
    {   14,  338,   32643,   32661,   32686, 18, 25, 25, }, // Australia_Eastern
    {   14,  356,   32711,   32729,   32754, 18, 25, 25, }, // Australia_Western
    {   14,  374,   32779,   32793,   32814, 14, 21, 21, }, // Azerbaijan
    {   14,  385,   32835,   32846,   32864, 11, 18, 18, }, // Azores
    {   14,  397,   32882,   32896,   32917, 14, 21, 21, }, // Bangladesh
    {   14,  415,       0,   32938,       0,  0, 11,  0, }, // Bhutan
    {   14,  422,       0,   32949,       0,  0, 13,  0, }, // Bolivia
    {   14,  437,   32962,   32976,   32997, 14, 21, 21, }, // Brasilia
    {   14,  454,       0,   33018,       0,  0, 12,  0, }, // Brunei
    {   14,  461,   33030,   33048,   33073, 18, 25, 25, }, // Cape_Verde
    {   14,  478,       0,   33098,       0,  0, 13,  0, }, // Chamorro
    {   14,  487,   33111,   33123,   33142, 12, 19, 19, }, // Chatham
    {   14,  495,   33161,   33172,   33190, 11, 18, 18, }, // Chile
    {   14,  501,   33208,   33219,   33237, 11, 18, 18, }, // China
    {   14,  507,   33255,   33270,   33292, 15, 22, 25, }, // Choibalsan
    {   14,  518,       0,   33317,       0,  0, 19,  0, }, // Christmas
    {   14,  528,       0,   33336,       0,  0, 15,  0, }, // Cocos
    {   14,  534,   33351,   33365,   33386, 14, 21, 21, }, // Colombia
    {   14,  543,   33407,   33421,   33441, 14, 20, 20, }, // Cook
    {   14,  548,   33461,   33471,   33488, 10, 17, 17, }, // Cuba
    {   14,  559,       0,   33505,       0,  0, 11,  0, }, // Davis
    {   14,  575,       0,   33516,       0,  0, 22,  0, }, // DumontDUrville
    {   14,  612,       0,   33538,       0,  0, 19,  0, }, // East_Timor
    {   14,  623,   33557,   33573,   33596, 16, 23, 23, }, // Easter
    {   14,  630,       0,   33619,       0,  0, 15,  0, }, // Ecuador
    {   14,  638,   33634,   33650,   33673, 16, 23, 23, }, // Europe_Central
    {   14,  653,   33696,   33712,   33735, 16, 23, 23, }, // Europe_Eastern
    {   14,  668,       0,   33758,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {   14,  691,   33787,   33803,   33826, 16, 23, 23, }, // Europe_Western
    {   14,  706,   33849,   33866,   33890, 17, 24, 24, }, // Falkland
    {   14,  715,   33914,   33924,   33941, 10, 17, 17, }, // Fiji
    {   14,  720,       0,   33958,       0,  0, 21,  0, }, // French_Guiana
    {   14,  734,       0,   33979,       0,  0, 47,  0, }, // French_Southern
    {   14,  757,       0,   34026,       0,  0, 14,  0, }, // Galapagos
    {   14,  767,       0,   34040,       0,  0, 12,  0, }, // Gambier
    {   14,  775,   34052,   34064,   34083, 12, 19, 19, }, // Georgia
    {   14,  783,       0,   34102,       0,  0, 16,  0, }, // Gilbert_Islands
    {   14,  799,       0,   34118,       0,  0, 12,  0, }, // GMT
    {   14,  841,   34130,   34148,   34173, 18, 25, 25, }, // Greenland_Eastern
    {   14,  859,   34198,   34216,   34241, 18, 25, 25, }, // Greenland_Western
    {   14,  877,       0,   34266,       0,  0, 10,  0, }, // Guam
    {   14,  882,       0,   34276,       0,  0, 12,  0, }, // Gulf
    {   14,  887,       0,   34288,       0,  0, 11,  0, }, // Guyana
    {   14,  894,   34299,   34318,   34344, 19, 26, 26, }, // Hawaii_Aleutian
    {   14,  910,   34370,   34385,   34407, 15, 22, 22, }, // Hong_Kong
    {   14,  920,   34429,   34439,   34456, 10, 17, 17, }, // Hovd
    {   14,  925,       0,   34473,       0,  0, 11,  0, }, // India
    {   14,  931,       0,   34484,       0,  0, 19,  0, }, // Indian_Ocean
    {   14,  944,       0,   34503,       0,  0, 19,  0, }, // Indochina
    {   14,  954,       0,   34522,       0,  0, 19,  0, }, // Indonesia_Central
    {   14,  972,       0,   34541,       0,  0, 19,  0, }, // Indonesia_Eastern
    {   14,  990,       0,   34560,       0,  0, 19,  0, }, // Indonesia_Western
    {   14, 1008,   34579,   34590,   34608, 11, 18, 18, }, // Iran
    {   14, 1019,   34626,   34639,   34659, 13, 20, 20, }, // Irkutsk
    {   14, 1027,   34679,   34692,   34712, 13, 20, 20, }, // Israel
    {   14, 1034,   34732,   34745,   34765, 13, 20, 20, }, // Japan
    {   14, 1040,   34785,   34799,   34829, 14, 30, 37, }, // Kamchatka
    {   14, 1058,       0,   34866,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {   14, 1077,       0,   34885,       0,  0, 19,  0, }, // Kazakhstan_Western
    {   14, 1106,   34904,   34915,   34933, 11, 18, 18, }, // Korea
    {   14, 1112,       0,   34951,       0,  0, 11,  0, }, // Kosrae
    {   14, 1119,   34962,   34979,   35003, 17, 24, 27, }, // Krasnoyarsk
    {   14, 1151,       0,   35030,       0,  0, 16,  0, }, // Kyrgystan
    {   14, 1175,       0,   35046,       0,  0, 14,  0, }, // Line_Islands
    {   14, 1188,   35060,   35074,   35095, 14, 21, 24, }, // Lord_Howe
    {   14, 1204,   35119,   35132,   35152, 13, 20, 20, }, // Magadan
    {   14, 1219,       0,   35172,       0,  0, 13,  0, }, // Malaysia
    {   14, 1228,       0,   35185,       0,  0, 18,  0, }, // Maldives
    {   14, 1237,       0,   35203,       0,  0, 14,  0, }, // Marquesas
    {   14, 1247,       0,   35217,       0,  0, 16,  0, }, // Marshall_Islands
    {   14, 1264,   35233,   35247,   35268, 14, 21, 21, }, // Mauritius
    {   14, 1274,       0,   35289,       0,  0, 12,  0, }, // Mawson
    {   14, 1281,   35301,   35328,   35362, 27, 34, 34, }, // Mexico_Pacific
    {   14, 1296,   35396,   35413,   35437, 17, 24, 24, }, // Mongolia
    {   14, 1305,   35461,   35472,   35490, 11, 18, 18, }, // Moscow
    {   14, 1312,       0,   35508,       0,  0, 13,  0, }, // Myanmar
    {   14, 1320,       0,   35521,       0,  0, 11,  0, }, // Nauru
    {   14, 1326,       0,   35532,       0,  0, 11,  0, }, // Nepal
    {   14, 1332,   35543,   35566,   35596, 23, 30, 30, }, // New_Caledonia
    {   14, 1346,   35626,   35641,   35663, 15, 22, 22, }, // New_Zealand
    {   14, 1358,   35685,   35703,   35728, 18, 25, 25, }, // Newfoundland
    {   14, 1371,       0,   35753,       0,  0, 10,  0, }, // Niue
    {   14, 1376,   35763,   35782,   35808, 19, 26, 26, }, // Norfolk
    {   14, 1384,   35834,   35859,   35890, 25, 31, 31, }, // Noronha
    {   14, 1392,       0,   35921,       0,  0, 26,  0, }, // North_Mariana
    {   14, 1406,   35947,   35964,   35988, 17, 24, 24, }, // Novosibirsk
    {   14, 1418,   36012,   36023,   36041, 11, 18, 18, }, // Omsk
    {   14, 1428,   36059,   36072,   36092, 13, 20, 20, }, // Pakistan
    {   14, 1437,       0,   36112,       0,  0, 11,  0, }, // Palau
    {   14, 1443,       0,   36123,       0,  0, 25,  0, }, // Papua_New_Guinea
    {   14, 1460,   36148,   36162,   36183, 14, 21, 21, }, // Paraguay
    {   14, 1469,   36204,   36214,   36231, 10, 17, 17, }, // Peru
    {   14, 1474,   36248,   36262,   36283, 14, 21, 21, }, // Philippines
    {   14, 1486,       0,   36304,       0,  0, 15,  0, }, // Phoenix_Islands
    {   14, 1502,   36319,   36343,   36374, 24, 31, 31, }, // Pierre_Miquelon
    {   14, 1518,       0,   36405,       0,  0, 13,  0, }, // Pitcairn
    {   14, 1527,       0,   36418,       0,  0, 12,  0, }, // Ponape
    {   14, 1534,       0,   36430,       0,  0, 16,  0, }, // Pyongyang
    {   14, 1554,       0,   36446,       0,  0, 13,  0, }, // Reunion
    {   14, 1562,       0,   36459,       0,  0, 12,  0, }, // Rothera
    {   14, 1570,   36471,   36484,   36504, 13, 20, 20, }, // Sakhalin
    {   14, 1579,   36524,   36536,   36547, 12, 11, 18, }, // Samara
    {   14, 1596,   36565,   36576,   36594, 11, 18, 18, }, // Samoa
    {   14, 1602,       0,   36612,       0,  0, 10,  0, }, // Seychelles
    {   14, 1624,       0,   36622,       0,  0, 14,  0, }, // Singapore
    {   14, 1634,       0,   36636,       0,  0, 16,  0, }, // Solomon
    {   14, 1642,       0,   36652,       0,  0, 17,  0, }, // South_Georgia
    {   14, 1656,       0,   36669,       0,  0, 13,  0, }, // Suriname
    {   14, 1676,       0,   36682,       0,  0, 12,  0, }, // Syowa
    {   14, 1682,       0,   36694,       0,  0, 12,  0, }, // Tahiti
    {   14, 1689,   36706,   36718,   36737, 12, 19, 19, }, // Taipei
    {   14, 1696,       0,   36756,       0,  0, 14,  0, }, // Tajikistan
    {   14, 1724,       0,   36770,       0,  0, 13,  0, }, // Tokelau
    {   14, 1732,   36783,   36794,   36812, 11, 18, 18, }, // Tonga
    {   14, 1738,       0,   36830,       0,  0,  9,  0, }, // Truk
    {   14, 1750,   36839,   36855,   36878, 16, 23, 23, }, // Turkmenistan
    {   14, 1763,       0,   36901,       0,  0, 12,  0, }, // Tuvalu
    {   14, 1777,   36913,   36927,   36948, 14, 21, 21, }, // Uruguay
    {   14, 1792,   36969,   36984,   37006, 15, 22, 22, }, // Uzbekistan
    {   14, 1803,   37028,   37041,   37061, 13, 20, 20, }, // Vanuatu
    {   14, 1811,       0,   37081,       0,  0, 13,  0, }, // Venezuela
    {   14, 1821,   37094,   37111,   37135, 17, 24, 24, }, // Vladivostok
    {   14, 1833,   37159,   37174,   37196, 15, 22, 22, }, // Volgograd
    {   14, 1843,       0,   37218,       0,  0, 12,  0, }, // Vostok
    {   14, 1850,       0,   37230,       0,  0, 15,  0, }, // Wake
    {   14, 1855,       0,   37245,       0,  0, 20,  0, }, // Wallis
    {   14, 1862,   37265,   37278,   37298, 13, 20, 20, }, // Yakutsk
    {   14, 1870,   37318,   37335,   37359, 17, 24, 24, }, // Yekaterinburg
    {   14, 1892,       0,   37383,       0,  0, 11,  0, }, // Yukon
    {   42,  142,   37394,   37426,   37467, 32, 41, 42, }, // America_Central Aragonese/Latin/Spain
    {   42,  158,   37509,   37542,   37584, 33, 42, 43, }, // America_Eastern
    {   42,  174,   37627,   37663,   37708, 36, 45, 46, }, // America_Mountain
    {   42,  191,   37754,   37791,   37837, 37, 46, 47, }, // America_Pacific
    {   42,  286,   37884,   37903,   37931, 19, 28, 29, }, // Atlantic
    {   42,  638,   37960,   37984,   38017, 24, 33, 34, }, // Europe_Central
    {   42,  653,   38051,   38074,   38106, 23, 32, 33, }, // Europe_Eastern
    {   42,  691,   38139,   38163,   38196, 24, 33, 34, }, // Europe_Western
    {   42,  799,       0,   38230,       0,  0, 31,  0, }, // GMT
    {   43,    5,       0,   38261,       0,  0, 18,  0, }, // Afghanistan Armenian/Armenian/Armenia
    {   43,   17,       0,   38279,       0,  0, 28,  0, }, // Africa_Central
    {   43,   32,       0,   38307,       0,  0, 25,  0, }, // Africa_Eastern
    {   43,   65,       0,   38332,       0,  0, 26,  0, }, // Africa_Southern
    {   43,   81,   38358,   38383,   38417, 25, 34, 34, }, // Africa_Western
    {   43,  107,   38451,   38468,   38494, 17, 26, 26, }, // Alaska
    {   43,  135,   38520,   38537,   38563, 17, 26, 26, }, // Amazon
    {   43,  142,   38589,   38618,   38656, 29, 38, 38, }, // America_Central
    {   43,  158,   38694,   38720,   38755, 26, 35, 35, }, // America_Eastern
    {   43,  174,   38790,   38812,   38843, 22, 31, 31, }, // America_Mountain
    {   43,  191,   38874,   38898,   38931, 24, 33, 33, }, // America_Pacific
    {   43,  214,   38964,   38978,   39001, 14, 23, 23, }, // Apia
    {   43,  232,   39024,   39049,   39083, 25, 34, 34, }, // Arabian
    {   43,  240,   39117,   39136,   39164, 19, 28, 28, }, // Argentina
    {   43,  250,   39192,   39220,   39257, 28, 37, 37, }, // Argentina_Western
    {   43,  268,   39294,   39311,   39337, 17, 26, 26, }, // Armenia
    {   43,  286,   39363,   39380,   39406, 17, 26, 26, }, // Atlantic
    {   43,  295,   39432,   39463,   39503, 31, 40, 40, }, // Australia_Central
    {   43,  313,   39543,   39583,   39632, 40, 49, 49, }, // Australia_CentralWestern
    {   43,  338,   39681,   39709,   39746, 28, 37, 37, }, // Australia_Eastern
    {   43,  356,   39783,   39811,   39848, 28, 37, 37, }, // Australia_Western
    {   43,  374,   39885,   39902,   39928, 17, 26, 26, }, // Azerbaijan
    {   43,  385,   39954,   39978,   40011, 24, 33, 33, }, // Azores
    {   43,  397,   40044,   40062,   40089, 18, 27, 27, }, // Bangladesh
    {   43,  415,       0,   40116,       0,  0, 15,  0, }, // Bhutan
    {   43,  422,       0,   40131,       0,  0, 17,  0, }, // Bolivia
    {   43,  437,   40148,   40166,   40193, 18, 27, 27, }, // Brasilia
    {   43,  454,       0,   40220,       0,  0, 16,  0, }, // Brunei
    {   43,  461,   40236,   40255,   40283, 19, 28, 28, }, // Cape_Verde
    {   43,  478,       0,   40311,       0,  0, 16,  0, }, // Chamorro
    {   43,  487,   40327,   40346,   40374, 19, 28, 28, }, // Chatham
    {   43,  495,   40402,   40415,   40437, 13, 22, 22, }, // Chile
    {   43,  501,   40459,   40476,   40502, 17, 26, 26, }, // China
    {   43,  507,   40528,   40546,   40573, 18, 27, 27, }, // Choibalsan
    {   43,  518,       0,   40600,       0,  0, 27,  0, }, // Christmas
    {   43,  528,       0,   40627,       0,  0, 25,  0, }, // Cocos
    {   43,  534,   40652,   40671,   40699, 19, 28, 28, }, // Colombia
    {   43,  543,   40727,   40749,   40780, 22, 31, 35, }, // Cook
    {   43,  548,   40815,   40830,   40854, 15, 24, 24, }, // Cuba
    {   43,  559,       0,   40878,       0,  0, 15,  0, }, // Davis
    {   43,  575,       0,   40893,       0,  0, 26,  0, }, // DumontDUrville
    {   43,  612,       0,   40919,       0,  0, 23,  0, }, // East_Timor
    {   43,  623,   40942,   40961,   40989, 19, 28, 28, }, // Easter
    {   43,  630,       0,   41017,       0,  0, 16,  0, }, // Ecuador
    {   43,  638,   41033,   41061,   41098, 28, 37, 37, }, // Europe_Central
    {   43,  653,   41135,   41160,   41194, 25, 34, 34, }, // Europe_Eastern
    {   43,  668,       0,   41228,       0,  0, 14,  0, }, // Europe_Further_Eastern
    {   43,  691,   41242,   41267,   41301, 25, 34, 34, }, // Europe_Western
    {   43,  706,   41335,   41363,   41400, 28, 37, 37, }, // Falkland
    {   43,  715,   41437,   41450,   41472, 13, 22, 22, }, // Fiji
    {   43,  720,       0,   41494,       0,  0, 27,  0, }, // French_Guiana
    {   43,  734,       0,   41521,       0,  0, 43,  0, }, // French_Southern
    {   43,  757,       0,   41564,       0,  0, 29,  0, }, // Galapagos
    {   43,  767,       0,   41593,       0,  0, 23,  0, }, // Gambier
    {   43,  775,   41616,   41632,   41657, 16, 25, 25, }, // Georgia
    {   43,  783,       0,   41682,       0,  0, 25,  0, }, // Gilbert_Islands
    {   43,  799,       0,   41707,       0,  0, 16,  0, }, // GMT
    {   43,  841,   41723,   41752,   41790, 29, 38, 38, }, // Greenland_Eastern
    {   43,  859,   41828,   41857,   41895, 29, 38, 38, }, // Greenland_Western
    {   43,  882,       0,   41933,       0,  0, 28,  0, }, // Gulf
    {   43,  887,       0,   41961,       0,  0, 16,  0, }, // Guyana
    {   43,  894,   41977,   42002,   42036, 25, 34, 34, }, // Hawaii_Aleutian
    {   43,  910,   42070,   42086,   42111, 16, 25, 25, }, // Hong_Kong
    {   43,  920,   42136,   42149,   42171, 13, 22, 22, }, // Hovd
    {   43,  925,       0,   42193,       0,  0, 27,  0, }, // India
    {   43,  931,       0,   42220,       0,  0, 26,  0, }, // Indian_Ocean
    {   43,  944,       0,   42246,       0,  0, 20,  0, }, // Indochina
    {   43,  954,       0,   42266,       0,  0, 31,  0, }, // Indonesia_Central
    {   43,  972,       0,   42297,       0,  0, 28,  0, }, // Indonesia_Eastern
    {   43,  990,       0,   42325,       0,  0, 28,  0, }, // Indonesia_Western
    {   43, 1008,   42353,   42366,   42388, 13, 22, 22, }, // Iran
    {   43, 1019,   42410,   42427,   42453, 17, 26, 26, }, // Irkutsk
    {   43, 1027,   42479,   42495,   42520, 16, 25, 25, }, // Israel
    {   43, 1034,   42545,   42562,   42588, 17, 26, 26, }, // Japan
    {   43, 1058,       0,   42614,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {   43, 1077,       0,   42641,       0,  0, 27,  0, }, // Kazakhstan_Western
    {   43, 1106,   42668,   42683,   42707, 15, 24, 24, }, // Korea
    {   43, 1112,       0,   42731,       0,  0, 16,  0, }, // Kosrae
    {   43, 1119,   42747,   42767,   42796, 20, 29, 29, }, // Krasnoyarsk
    {   43, 1151,       0,   42825,       0,  0, 17,  0, }, // Kyrgystan
    {   43, 1175,       0,   42842,       0,  0, 21,  0, }, // Line_Islands
    {   43, 1188,   42863,   42881,   42908, 18, 27, 27, }, // Lord_Howe
    {   43, 1204,   42935,   42951,   42976, 16, 25, 25, }, // Magadan
    {   43, 1219,       0,   43001,       0,  0, 18,  0, }, // Malaysia
    {   43, 1228,       0,   43019,       0,  0, 18,  0, }, // Maldives
    {   43, 1237,       0,   43037,       0,  0, 26,  0, }, // Marquesas
    {   43, 1247,       0,   43063,       0,  0, 26,  0, }, // Marshall_Islands
    {   43, 1264,   43089,   43107,   43134, 18, 27, 27, }, // Mauritius
    {   43, 1274,       0,   43161,       0,  0, 16,  0, }, // Mawson
    {   43, 1281,   43177,   43211,   43254, 34, 43, 43, }, // Mexico_Pacific
    {   43, 1296,   43297,   43317,   43346, 20, 29, 29, }, // Mongolia
    {   43, 1305,   43375,   43391,   43416, 16, 25, 25, }, // Moscow
    {   43, 1312,       0,   43441,       0,  0, 16,  0, }, // Myanmar
    {   43, 1320,       0,   43457,       0,  0, 16,  0, }, // Nauru
    {   43, 1326,       0,   43473,       0,  0, 14,  0, }, // Nepal
    {   43, 1332,   43487,   43510,   43542, 23, 32, 32, }, // New_Caledonia
    {   43, 1346,   43574,   43596,   43627, 22, 31, 31, }, // New_Zealand
    {   43, 1358,   43658,   43681,   43713, 23, 32, 32, }, // Newfoundland
    {   43, 1371,       0,   43745,       0,  0, 15,  0, }, // Niue
    {   43, 1376,   43760,   43781,   43811, 21, 30, 30, }, // Norfolk
    {   43, 1384,   43841,   43871,   43910, 30, 39, 39, }, // Noronha
    {   43, 1406,   43949,   43969,   43998, 20, 29, 29, }, // Novosibirsk
    {   43, 1418,   44027,   44040,   44062, 13, 22, 22, }, // Omsk
    {   43, 1428,   44084,   44101,   44127, 17, 26, 26, }, // Pakistan
    {   43, 1437,       0,   44153,       0,  0, 16,  0, }, // Palau
    {   43, 1443,       0,   44169,       0,  0, 27,  0, }, // Papua_New_Guinea
    {   43, 1460,   44196,   44213,   44239, 17, 26, 26, }, // Paraguay
    {   43, 1469,   44265,   44279,   44302, 14, 23, 23, }, // Peru
    {   43, 1474,   44325,   44344,   44372, 19, 28, 28, }, // Philippines
    {   43, 1486,       0,   44400,       0,  0, 23,  0, }, // Phoenix_Islands
    {   43, 1502,   44423,   44450,   44486, 27, 36, 36, }, // Pierre_Miquelon
    {   43, 1518,       0,   44522,       0,  0, 16,  0, }, // Pitcairn
    {   43, 1527,       0,   44538,       0,  0, 20,  0, }, // Ponape
    {   43, 1534,       0,   44558,       0,  0, 16,  0, }, // Pyongyang
    {   43, 1554,       0,   44574,       0,  0, 18,  0, }, // Reunion
    {   43, 1562,       0,   44592,       0,  0, 16,  0, }, // Rothera
    {   43, 1570,   44608,   44624,   44649, 16, 25, 25, }, // Sakhalin
    {   43, 1596,   44674,   44689,   44713, 15, 24, 24, }, // Samoa
    {   43, 1602,       0,   44737,       0,  0, 26,  0, }, // Seychelles
    {   43, 1624,       0,   44763,       0,  0, 18,  0, }, // Singapore
    {   43, 1634,       0,   44781,       0,  0, 25,  0, }, // Solomon
    {   43, 1642,       0,   44806,       0,  0, 26,  0, }, // South_Georgia
    {   43, 1656,       0,   44832,       0,  0, 17,  0, }, // Suriname
    {   43, 1676,       0,   44849,       0,  0, 15,  0, }, // Syowa
    {   43, 1682,       0,   44864,       0,  0, 14,  0, }, // Tahiti
    {   43, 1689,   44878,   44893,   44917, 15, 24, 24, }, // Taipei
    {   43, 1696,       0,   44941,       0,  0, 18,  0, }, // Tajikistan
    {   43, 1724,       0,   44959,       0,  0, 18,  0, }, // Tokelau
    {   43, 1732,   44977,   44992,   45016, 15, 24, 24, }, // Tonga
    {   43, 1738,       0,   45040,       0,  0, 14,  0, }, // Truk
    {   43, 1750,   45054,   45075,   45105, 21, 30, 30, }, // Turkmenistan
    {   43, 1763,       0,   45135,       0,  0, 18,  0, }, // Tuvalu
    {   43, 1777,   45153,   45171,   45198, 18, 27, 27, }, // Uruguay
    {   43, 1792,   45225,   45244,   45272, 19, 28, 28, }, // Uzbekistan
    {   43, 1803,   45300,   45319,   45347, 19, 28, 28, }, // Vanuatu
    {   43, 1811,       0,   45375,       0,  0, 20,  0, }, // Venezuela
    {   43, 1821,   45395,   45415,   45444, 20, 29, 29, }, // Vladivostok
    {   43, 1833,   45473,   45491,   45518, 18, 27, 27, }, // Volgograd
    {   43, 1843,       0,   45545,       0,  0, 15,  0, }, // Vostok
    {   43, 1850,       0,   45560,       0,  0, 19,  0, }, // Wake
    {   43, 1855,       0,   45579,       0,  0, 27,  0, }, // Wallis
    {   43, 1862,   45606,   45623,   45649, 17, 26, 26, }, // Yakutsk
    {   43, 1870,   45675,   45697,   45728, 22, 31, 31, }, // Yekaterinburg
    {   43, 1892,       0,   45759,       0,  0, 15,  0, }, // Yukon
    {   44,    5,       0,   45774,       0,  0, 17,  0, }, // Afghanistan Assamese/Bangla/India
    {   44,   17,       0,   45791,       0,  0, 18,  0, }, // Africa_Central
    {   44,   32,       0,   45809,       0,  0, 17,  0, }, // Africa_Eastern
    {   44,   65,       0,   45826,       0,  0, 24,  0, }, // Africa_Southern
    {   44,   81,   45850,   45870,   45894, 20, 24, 33, }, // Africa_Western
    {   44,  107,   45927,   45940,   45957, 13, 17, 20, }, // Alaska
    {   44,  135,   45977,   45988,   46003, 11, 15, 24, }, // Amazon
    {   44,  142,   46027,   46057,   46091, 30, 34, 37, }, // America_Central
    {   44,  158,   46128,   46155,   46186, 27, 31, 34, }, // America_Eastern
    {   44,  174,   46220,   46250,   46284, 30, 34, 37, }, // America_Mountain
    {   44,  191,   46321,   46349,   46381, 28, 32, 26, }, // America_Pacific
    {   44,  214,   46407,   46419,   46435, 12, 16, 19, }, // Apia
    {   44,  232,   46454,   46465,   46480, 11, 15, 18, }, // Arabian
    {   44,  240,   46498,   46515,   46536, 17, 21, 30, }, // Argentina
    {   44,  250,   46566,   46593,   46624, 27, 31, 40, }, // Argentina_Western
    {   44,  268,   46664,   46680,   46700, 16, 20, 29, }, // Armenia
    {   44,  286,   46729,   46743,   46761, 14, 18, 21, }, // Atlantic
    {   44,  295,   46782,   46805,   46838, 23, 33, 36, }, // Australia_Central
    {   44,  313,   46874,   46913,   46956, 39, 43, 46, }, // Australia_CentralWestern
    {   44,  338,   47002,   47028,   47058, 26, 30, 33, }, // Australia_Eastern
    {   44,  356,   47091,   47119,   47151, 28, 32, 35, }, // Australia_Western
    {   44,  374,   47186,   47202,   47222, 16, 20, 29, }, // Azerbaijan
    {   44,  385,   47251,   47262,   47278, 11, 16, 25, }, // Azores
    {   44,  397,   47303,   47317,   47335, 14, 18, 27, }, // Bangladesh
    {   44,  415,       0,   47362,       0,  0, 11,  0, }, // Bhutan
    {   44,  422,       0,   47373,       0,  0, 14,  0, }, // Bolivia
    {   44,  437,   47387,   47404,   47425, 17, 21, 30, }, // Brasilia
    {   44,  454,       0,   47455,       0,  0, 23,  0, }, // Brunei
    {   44,  461,   47478,   47494,   47514, 16, 20, 29, }, // Cape_Verde
    {   44,  478,       0,   47543,       0,  0, 16,  0, }, // Chamorro
    {   44,  487,   47559,   47570,   47585, 11, 15, 18, }, // Chatham
    {   44,  495,   47603,   47613,   47627, 10, 14, 23, }, // Chile
    {   44,  501,   47650,   47659,   47672,  9, 13, 16, }, // China
    {   44,  507,   47688,   47703,   47722, 15, 19, 28, }, // Choibalsan
    {   44,  518,       0,   47750,       0,  0, 22,  0, }, // Christmas
    {   44,  528,       0,   47772,       0,  0, 22,  0, }, // Cocos
    {   44,  534,   47794,   47809,   47828, 15, 19, 28, }, // Colombia
    {   44,  543,   47856,   47876,   47900, 20, 24, 38, }, // Cook
    {   44,  548,   47938,   47949,   47964, 11, 15, 18, }, // Cuba
    {   44,  559,       0,   47982,       0,  0, 11,  0, }, // Davis
    {   44,  575,       0,   47993,       0,  0, 23,  0, }, // DumontDUrville
    {   44,  612,       0,   48016,       0,  0, 15,  0, }, // East_Timor
    {   44,  623,   48031,   48049,   48071, 18, 22, 31, }, // Easter
    {   44,  630,       0,   48102,       0,  0, 13,  0, }, // Ecuador
    {   44,  638,   48115,   48133,   48155, 18, 22, 31, }, // Europe_Central
    {   44,  653,   48186,   48207,   48232, 21, 25, 34, }, // Europe_Eastern
    {   44,  668,       0,   48266,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {   44,  691,   48292,   48315,   48342, 23, 27, 36, }, // Europe_Western
    {   44,  706,   48378,   48402,   48430, 24, 28, 37, }, // Falkland
    {   44,  715,   48467,   48477,   48491, 10, 14, 23, }, // Fiji
    {   44,  720,       0,   48514,       0,  0, 20,  0, }, // French_Guiana
    {   44,  734,       0,   48534,       0,  0, 35,  0, }, // French_Southern
    {   44,  757,       0,   48569,       0,  0, 15,  0, }, // Galapagos
    {   44,  767,       0,   48584,       0,  0, 15,  0, }, // Gambier
    {   44,  775,   48599,   48613,   48631, 14, 18, 27, }, // Georgia
    {   44,  783,       0,   48658,       0,  0, 25,  0, }, // Gilbert_Islands
    {   44,  799,       0,   48683,       0,  0, 17,  0, }, // GMT
    {   44,  841,   48700,   48720,   48744, 20, 24, 33, }, // Greenland_Eastern
    {   44,  859,   48777,   48800,   48827, 23, 27, 36, }, // Greenland_Western
    {   44,  882,       0,   48863,       0,  0, 18,  0, }, // Gulf
    {   44,  887,       0,   48881,       0,  0, 13,  0, }, // Guyana
    {   44,  894,   48894,   48914,   48938, 20, 24, 27, }, // Hawaii_Aleutian
    {   44,  910,   48965,   48976,   48991, 11, 15, 24, }, // Hong_Kong
    {   44,  920,   49015,   49027,   49043, 12, 16, 25, }, // Hovd
    {   44,  925,       0,   49068,       0,  0, 16,  0, }, // India
    {   44,  931,       0,   49084,       0,  0, 20,  0, }, // Indian_Ocean
    {   44,  944,       0,   49104,       0,  0, 16,  0, }, // Indochina
    {   44,  954,       0,   49120,       0,  0, 23,  0, }, // Indonesia_Central
    {   44,  972,       0,   49143,       0,  0, 26,  0, }, // Indonesia_Eastern
    {   44,  990,       0,   49169,       0,  0, 28,  0, }, // Indonesia_Western
    {   44, 1008,   49197,   49207,   49221, 10, 14, 17, }, // Iran
    {   44, 1019,   49238,   49253,   49272, 15, 19, 28, }, // Irkutsk
    {   44, 1027,   49300,   49312,   49328, 12, 16, 19, }, // Israel
    {   44, 1034,   49347,   49358,   49373, 11, 15, 18, }, // Japan
    {   44, 1058,       0,   49391,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {   44, 1077,       0,   49411,       0,  0, 23,  0, }, // Kazakhstan_Western
    {   44, 1106,   49434,   49447,   49464, 13, 17, 20, }, // Korea
    {   44, 1112,       0,   49484,       0,  0, 13,  0, }, // Kosrae
    {   44, 1119,   49497,   49518,   49543, 21, 25, 33, }, // Krasnoyarsk
    {   44, 1151,       0,   49576,       0,  0, 18,  0, }, // Kyrgystan
    {   44, 1175,       0,   49594,       0,  0, 21,  0, }, // Line_Islands
    {   44, 1188,   49615,   49629,   49647, 14, 18, 21, }, // Lord_Howe
    {   44, 1204,   49668,   49681,   49698, 13, 17, 26, }, // Magadan
    {   44, 1219,       0,   49724,       0,  0, 17,  0, }, // Malaysia
    {   44, 1228,       0,   49741,       0,  0, 14,  0, }, // Maldives
    {   44, 1237,       0,   49755,       0,  0, 14,  0, }, // Marquesas
    {   44, 1247,       0,   49769,       0,  0, 26,  0, }, // Marshall_Islands
    {   44, 1264,   49795,   49807,   49823, 12, 16, 25, }, // Mauritius
    {   44, 1274,       0,   49848,       0,  0, 11,  0, }, // Mawson
    {   44, 1281,   49859,   49882,   49909, 23, 27, 30, }, // Mexico_Pacific
    {   44, 1296,   49939,   49954,   49973, 15, 19, 28, }, // Mongolia
    {   44, 1305,   50001,   50012,   50027, 11, 15, 24, }, // Moscow
    {   44, 1312,       0,   50051,       0,  0, 14,  0, }, // Myanmar
    {   44, 1320,       0,   50065,       0,  0, 11,  0, }, // Nauru
    {   44, 1326,       0,   50076,       0,  0, 11,  0, }, // Nepal
    {   44, 1332,   50087,   50108,   50133, 21, 25, 34, }, // New_Caledonia
    {   44, 1346,   50167,   50183,   50203, 16, 20, 23, }, // New_Zealand
    {   44, 1358,   50226,   50246,   50270, 20, 24, 27, }, // Newfoundland
    {   44, 1371,       0,   50297,       0,  0, 11,  0, }, // Niue
    {   44, 1376,   50308,   50326,   50350, 18, 24, 31, }, // Norfolk
    {   44, 1384,   50381,   50410,   50443, 29, 33, 42, }, // Noronha
    {   44, 1406,   50485,   50503,   50525, 18, 22, 31, }, // Novosibirsk
    {   44, 1418,   50556,   50567,   50582, 11, 15, 24, }, // Omsk
    {   44, 1428,   50606,   50621,   50640, 15, 19, 28, }, // Pakistan
    {   44, 1437,       0,   50668,       0,  0, 11,  0, }, // Palau
    {   44, 1443,       0,   50679,       0,  0, 21,  0, }, // Papua_New_Guinea
    {   44, 1460,   50700,   50714,   50732, 14, 18, 27, }, // Paraguay
    {   44, 1469,   50759,   50769,   50783, 10, 14, 23, }, // Peru
    {   44, 1474,   50806,   50820,   50838, 14, 18, 27, }, // Philippines
    {   44, 1486,       0,   50865,       0,  0, 24,  0, }, // Phoenix_Islands
    {   44, 1502,   50889,   50923,   50961, 34, 38, 41, }, // Pierre_Miquelon
    {   44, 1518,       0,   51002,       0,  0, 15,  0, }, // Pitcairn
    {   44, 1527,       0,   51017,       0,  0, 12,  0, }, // Ponape
    {   44, 1534,       0,   51029,       0,  0, 14,  0, }, // Pyongyang
    {   44, 1554,       0,   51043,       0,  0, 15,  0, }, // Reunion
    {   44, 1562,       0,   51058,       0,  0, 12,  0, }, // Rothera
    {   44, 1570,   51070,   51083,   51100, 13, 17, 26, }, // Sakhalin
    {   44, 1596,   51126,   51138,   51154, 12, 16, 19, }, // Samoa
    {   44, 1602,       0,   51173,       0,  0, 13,  0, }, // Seychelles
    {   44, 1624,       0,   51186,       0,  0, 18,  0, }, // Singapore
    {   44, 1634,       0,   51204,       0,  0, 24,  0, }, // Solomon
    {   44, 1642,       0,   51228,       0,  0, 21,  0, }, // South_Georgia
    {   44, 1656,       0,   51249,       0,  0, 13,  0, }, // Suriname
    {   44, 1676,       0,   51262,       0,  0, 11,  0, }, // Syowa
    {   44, 1682,       0,   51273,       0,  0, 12,  0, }, // Tahiti
    {   44, 1689,   51285,   51297,   51313, 12, 16, 19, }, // Taipei
    {   44, 1696,       0,   51332,       0,  0, 17,  0, }, // Tajikistan
    {   44, 1724,       0,   51349,       0,  0, 13,  0, }, // Tokelau
    {   44, 1732,   51362,   51372,   51386, 10, 14, 23, }, // Tonga
    {   44, 1738,       0,   51409,       0,  0, 11,  0, }, // Truk
    {   44, 1750,   51420,   51440,   51464, 20, 24, 33, }, // Turkmenistan
    {   44, 1763,       0,   51497,       0,  0, 13,  0, }, // Tuvalu
    {   44, 1777,   51510,   51523,   51540, 13, 17, 26, }, // Uruguay
    {   44, 1792,   51566,   51583,   51604, 17, 21, 30, }, // Uzbekistan
    {   44, 1803,   51634,   51646,   51662, 12, 16, 25, }, // Vanuatu
    {   44, 1811,       0,   51687,       0,  0, 16,  0, }, // Venezuela
    {   44, 1821,   51703,   51722,   51745, 19, 23, 32, }, // Vladivostok
    {   44, 1833,   51777,   51794,   51815, 17, 21, 30, }, // Volgograd
    {   44, 1843,       0,   51845,       0,  0, 12,  0, }, // Vostok
    {   44, 1850,       0,   51857,       0,  0, 15,  0, }, // Wake
    {   44, 1855,       0,   51872,       0,  0, 22,  0, }, // Wallis
    {   44, 1862,   51894,   51910,   51930, 16, 20, 29, }, // Yakutsk
    {   44, 1870,   51959,   51980,   52005, 21, 25, 34, }, // Yekaterinburg
    {   44, 1892,       0,   52039,       0,  0, 11,  0, }, // Yukon
    {   45,    0,   52050,   52061,   52081, 11, 20, 20, }, // Acre Asturian/Latin/Spain
    {   45,    5,       0,   52101,       0,  0, 17,  0, }, // Afghanistan
    {   45,   17,       0,   52118,       0,  0, 21,  0, }, // Africa_Central
    {   45,   32,       0,   52139,       0,  0, 22,  0, }, // Africa_Eastern
    {   45,   65,       0,   52161,       0,  0, 17,  0, }, // Africa_Southern
    {   45,   81,   52178,   52201,   52233, 23, 32, 32, }, // Africa_Western
    {   45,  107,   52265,   52278,   52300, 13, 22, 22, }, // Alaska
    {   45,  128,   52322,   52335,   52357, 13, 22, 22, }, // Almaty
    {   45,  135,   52379,   52396,   52422, 17, 26, 26, }, // Amazon
    {   45,  142,   52448,   52475,   52511, 27, 36, 36, }, // America_Central
    {   45,  158,   52547,   52575,   52612, 28, 37, 37, }, // America_Eastern
    {   45,  174,   52649,   52685,   52730, 36, 45, 45, }, // America_Mountain
    {   45,  191,   52775,   52807,   52848, 32, 41, 41, }, // America_Pacific
    {   45,  207,   52889,   52902,   52924, 13, 22, 22, }, // Anadyr
    {   45,  214,   52946,   52957,   52977, 11, 20, 20, }, // Apia
    {   45,  219,   52997,   53009,   53030, 12, 21, 21, }, // Aqtau
    {   45,  225,   53051,   53064,   53086, 13, 22, 22, }, // Aqtobe
    {   45,  232,   53108,   53121,   53143, 13, 22, 22, }, // Arabian
    {   45,  240,   53165,   53181,   53206, 16, 25, 25, }, // Argentina
    {   45,  250,   53231,   53258,   53294, 27, 36, 36, }, // Argentina_Western
    {   45,  268,   53330,   53344,   53367, 14, 23, 23, }, // Armenia
    {   45,  286,   53390,   53408,   53435, 18, 27, 27, }, // Atlantic
    {   45,  295,   53462,   53486,   53519, 24, 33, 33, }, // Australia_Central
    {   45,  313,   53552,   53586,   53629, 34, 43, 43, }, // Australia_CentralWestern
    {   45,  338,   53672,   53697,   53731, 25, 34, 34, }, // Australia_Eastern
    {   45,  356,   53765,   53791,   53826, 26, 35, 35, }, // Australia_Western
    {   45,  374,   53861,   53878,   53904, 17, 26, 26, }, // Azerbaijan
    {   45,  385,   53930,   53948,   53975, 18, 27, 27, }, // Azores
    {   45,  397,   54002,   54019,   54045, 17, 26, 26, }, // Bangladesh
    {   45,  415,       0,   54071,       0,  0, 13,  0, }, // Bhutan
    {   45,  422,       0,   54084,       0,  0, 15,  0, }, // Bolivia
    {   45,  437,   54099,   54115,   54140, 16, 25, 25, }, // Brasilia
    {   45,  454,       0,   54165,       0,  0, 25,  0, }, // Brunei
    {   45,  461,   54190,   54208,   54235, 18, 27, 27, }, // Cape_Verde
    {   45,  472,       0,   54262,       0,  0, 13,  0, }, // Casey
    {   45,  478,       0,   54275,       0,  0, 25,  0, }, // Chamorro
    {   45,  487,   54300,   54315,   54339, 15, 24, 24, }, // Chatham
    {   45,  495,   54363,   54376,   54398, 13, 22, 22, }, // Chile
    {   45,  501,   54420,   54433,   54455, 13, 22, 22, }, // China
    {   45,  507,   54477,   54495,   54522, 18, 27, 27, }, // Choibalsan
    {   45,  518,       0,   54549,       0,  0, 35,  0, }, // Christmas
    {   45,  528,       0,   54584,       0,  0, 24,  0, }, // Cocos
    {   45,  534,   54608,   54624,   54649, 16, 25, 25, }, // Colombia
    {   45,  543,   54674,   54697,   54729, 23, 32, 38, }, // Cook
    {   45,  548,   54767,   54779,   54800, 12, 21, 21, }, // Cuba
    {   45,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {   45,  575,       0,   54834,       0,  0, 24,  0, }, // DumontDUrville
    {   45,  612,       0,   54858,       0,  0, 22,  0, }, // East_Timor
    {   45,  623,   54880,   54906,   54941, 26, 35, 35, }, // Easter
    {   45,  630,       0,   54976,       0,  0, 14,  0, }, // Ecuador
    {   45,  638,   54990,   55011,   55041, 21, 30, 30, }, // Europe_Central
    {   45,  653,   55071,   55093,   55124, 22, 31, 31, }, // Europe_Eastern
    {   45,  668,       0,   55155,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {   45,  691,   55185,   55209,   55242, 24, 33, 33, }, // Europe_Western
    {   45,  706,   55275,   55302,   55338, 27, 36, 36, }, // Falkland
    {   45,  715,   55374,   55386,   55407, 12, 21, 21, }, // Fiji
    {   45,  720,       0,   55428,       0,  0, 26,  0, }, // French_Guiana
    {   45,  734,       0,   55454,       0,  0, 34,  0, }, // French_Southern
    {   45,  757,       0,   55488,       0,  0, 17,  0, }, // Galapagos
    {   45,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {   45,  775,   55520,   55534,   55557, 14, 23, 23, }, // Georgia
    {   45,  783,       0,   55580,       0,  0, 26,  0, }, // Gilbert_Islands
    {   45,  799,       0,   55606,       0,  0, 23,  0, }, // GMT
    {   45,  841,   55629,   55657,   55694, 28, 37, 37, }, // Greenland_Eastern
    {   45,  859,   55731,   55761,   55800, 30, 39, 39, }, // Greenland_Western
    {   45,  877,       0,   55839,       0,  0, 21,  0, }, // Guam
    {   45,  882,       0,   55860,       0,  0, 23,  0, }, // Gulf
    {   45,  887,       0,   55428,       0,  0, 17,  0, }, // Guyana
    {   45,  894,   55883,   55908,   55942, 25, 34, 34, }, // Hawaii_Aleutian
    {   45,  910,   55976,   55993,   56019, 17, 26, 26, }, // Hong_Kong
    {   45,  920,   56045,   56057,   56078, 12, 21, 21, }, // Hovd
    {   45,  925,       0,   56099,       0,  0, 25,  0, }, // India
    {   45,  931,       0,   56124,       0,  0, 22,  0, }, // Indian_Ocean
    {   45,  944,       0,   56146,       0,  0, 16,  0, }, // Indochina
    {   45,  954,       0,   56162,       0,  0, 24,  0, }, // Indonesia_Central
    {   45,  972,       0,   56186,       0,  0, 25,  0, }, // Indonesia_Eastern
    {   45,  990,       0,   56211,       0,  0, 26,  0, }, // Indonesia_Western
    {   45, 1008,   56237,   56248,   56268, 11, 20, 20, }, // Iran
    {   45, 1019,   56288,   56302,   56325, 14, 23, 23, }, // Irkutsk
    {   45, 1027,   56348,   56361,   56383, 13, 22, 22, }, // Israel
    {   45, 1034,   56405,   56418,   56440, 13, 22, 22, }, // Japan
    {   45, 1040,   56462,   56494,   56535, 32, 41, 41, }, // Kamchatka
    {   45, 1058,       0,   56576,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {   45, 1077,       0,   56603,       0,  0, 29,  0, }, // Kazakhstan_Western
    {   45, 1106,   56632,   56645,   56667, 13, 22, 22, }, // Korea
    {   45, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {   45, 1119,   56703,   56722,   56750, 19, 28, 28, }, // Krasnoyarsk
    {   45, 1151,       0,   56778,       0,  0, 19,  0, }, // Kyrgystan
    {   45, 1161,       0,   56797,       0,  0, 13,  0, }, // Lanka
    {   45, 1175,       0,   56810,       0,  0, 23,  0, }, // Line_Islands
    {   45, 1188,   56833,   56850,   56876, 17, 26, 26, }, // Lord_Howe
    {   45, 1198,   56902,   56915,   56937, 13, 22, 22, }, // Macau
    {   45, 1204,   56959,   56974,   56998, 15, 24, 24, }, // Magadan
    {   45, 1219,       0,   57022,       0,  0, 15,  0, }, // Malaysia
    {   45, 1228,       0,   57037,       0,  0, 20,  0, }, // Maldives
    {   45, 1237,       0,   57057,       0,  0, 21,  0, }, // Marquesas
    {   45, 1247,       0,   57078,       0,  0, 27,  0, }, // Marshall_Islands
    {   45, 1264,   57105,   57121,   57146, 16, 25, 25, }, // Mauritius
    {   45, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {   45, 1281,   57185,   57212,   57248, 27, 36, 36, }, // Mexico_Pacific
    {   45, 1296,   57284,   57301,   57327, 17, 26, 26, }, // Mongolia
    {   45, 1305,   57353,   57366,   57388, 13, 22, 22, }, // Moscow
    {   45, 1312,       0,   57410,       0,  0, 15,  0, }, // Myanmar
    {   45, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {   45, 1326,       0,   57438,       0,  0, 14,  0, }, // Nepal
    {   45, 1332,   57452,   57475,   57507, 23, 32, 32, }, // New_Caledonia
    {   45, 1346,   57539,   57560,   57590, 21, 30, 30, }, // New_Zealand
    {   45, 1358,   57620,   57640,   57669, 20, 29, 29, }, // Newfoundland
    {   45, 1371,       0,   57698,       0,  0, 12,  0, }, // Niue
    {   45, 1376,   57710,   57734,   57767, 24, 33, 33, }, // Norfolk
    {   45, 1384,   57800,   57827,   57863, 27, 36, 36, }, // Noronha
    {   45, 1392,       0,   57899,       0,  0, 37,  0, }, // North_Mariana
    {   45, 1406,   57936,   57955,   57983, 19, 28, 28, }, // Novosibirsk
    {   45, 1418,   58011,   58022,   58042, 11, 20, 20, }, // Omsk
    {   45, 1428,   58062,   58080,   58107, 18, 27, 27, }, // Pakistan
    {   45, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {   45, 1443,       0,   58147,       0,  0, 26,  0, }, // Papua_New_Guinea
    {   45, 1460,   58173,   58190,   58216, 17, 26, 26, }, // Paraguay
    {   45, 1469,   58242,   58255,   58277, 13, 22, 22, }, // Peru
    {   45, 1474,   58299,   58316,   58342, 17, 26, 26, }, // Philippines
    {   45, 1486,       0,   58368,       0,  0, 26,  0, }, // Phoenix_Islands
    {   45, 1502,   58394,   58425,   58465, 31, 40, 40, }, // Pierre_Miquelon
    {   45, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {   45, 1527,       0,   58521,       0,  0, 14,  0, }, // Ponape
    {   45, 1534,       0,   58535,       0,  0, 17,  0, }, // Pyongyang
    {   45, 1544,   58552,   58569,   58595, 17, 26, 26, }, // Qyzylorda
    {   45, 1554,       0,   58621,       0,  0, 15,  0, }, // Reunion
    {   45, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {   45, 1570,   58651,   58666,   58690, 15, 24, 24, }, // Sakhalin
    {   45, 1579,   58714,   58728,   58751, 14, 23, 23, }, // Samara
    {   45, 1596,   58774,   58787,   58809, 13, 22, 22, }, // Samoa
    {   45, 1602,       0,   58831,       0,  0, 20,  0, }, // Seychelles
    {   45, 1624,       0,   58851,       0,  0, 25,  0, }, // Singapore
    {   45, 1634,       0,   58876,       0,  0, 26,  0, }, // Solomon
    {   45, 1642,       0,   58902,       0,  0, 22,  0, }, // South_Georgia
    {   45, 1656,       0,   58924,       0,  0, 16,  0, }, // Suriname
    {   45, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {   45, 1682,       0,   58953,       0,  0, 14,  0, }, // Tahiti
    {   45, 1689,   58967,   58981,   59004, 14, 23, 23, }, // Taipei
    {   45, 1696,       0,   59027,       0,  0, 20,  0, }, // Tajikistan
    {   45, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {   45, 1732,   59062,   59075,   59097, 13, 22, 22, }, // Tonga
    {   45, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {   45, 1750,   59132,   59153,   59183, 21, 30, 30, }, // Turkmenistan
    {   45, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {   45, 1777,   59227,   59243,   59268, 16, 25, 25, }, // Uruguay
    {   45, 1792,   59293,   59313,   59342, 20, 29, 29, }, // Uzbekistan
    {   45, 1803,   59371,   59386,   59410, 15, 24, 24, }, // Vanuatu
    {   45, 1811,       0,   59434,       0,  0, 17,  0, }, // Venezuela
    {   45, 1821,   59451,   59470,   59498, 19, 28, 28, }, // Vladivostok
    {   45, 1833,   59526,   59543,   59569, 17, 26, 26, }, // Volgograd
    {   45, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {   45, 1850,       0,   59609,       0,  0, 21,  0, }, // Wake
    {   45, 1855,       0,   59630,       0,  0, 23,  0, }, // Wallis
    {   45, 1862,   59653,   59668,   59692, 15, 24, 24, }, // Yakutsk
    {   45, 1870,   59716,   59738,   59769, 22, 31, 31, }, // Yekaterinburg
    {   48,    5,       0,   59800,       0,  0, 16,  0, }, // Afghanistan Azerbaijani/Latin/Azerbaijan
    {   48,   17,       0,   59816,       0,  0, 20,  0, }, // Africa_Central
    {   48,   32,       0,   59836,       0,  0, 18,  0, }, // Africa_Eastern
    {   48,   65,       0,   59854,       0,  0, 19,  0, }, // Africa_Southern
    {   48,   81,   59873,   59891,   59918, 18, 27, 22, }, // Africa_Western
    {   48,  107,   59940,   59953,   59975, 13, 22, 17, }, // Alaska
    {   48,  135,   59992,   60004,   60025, 12, 21, 16, }, // Amazon
    {   48,  142,   60041,   60069,   60106, 28, 37, 32, }, // America_Central
    {   48,  158,   60138,   60164,   60199, 26, 35, 30, }, // America_Eastern
    {   48,  174,   60229,   60256,   60292, 27, 36, 31, }, // America_Mountain
    {   48,  191,   60323,   60355,   60396, 32, 41, 36, }, // America_Pacific
    {   48,  214,   60432,   60442,   60461, 10, 19, 14, }, // Apia
    {   48,  232,   60475,   60490,   60514, 15, 24, 19, }, // Arabian
    {   48,  240,   60533,   60548,   60572, 15, 24, 19, }, // Argentina
    {   48,  250,   60591,   60612,   60642, 21, 30, 25, }, // Argentina_Western
    {   48,  268,   60667,   60683,   60708, 16, 25, 20, }, // Armenia
    {   48,  286,   60728,   60741,   60763, 13, 22, 18, }, // Atlantic
    {   48,  295,   60781,   60805,   60838, 24, 33, 28, }, // Australia_Central
    {   48,  313,   60866,   60896,   60935, 30, 39, 34, }, // Australia_CentralWestern
    {   48,  338,   60969,   60991,   61022, 22, 31, 26, }, // Australia_Eastern
    {   48,  356,   60874,   60904,   60943, 22, 31, 26, }, // Australia_Western
    {   48,  374,   61048,   61064,   61089, 16, 25, 20, }, // Azerbaijan
    {   48,  385,   61109,   61119,   61138, 10, 19, 14, }, // Azores
    {   48,  397,   61152,   61167,   61191, 15, 24, 19, }, // Bangladesh
    {   48,  415,       0,   61210,       0,  0, 11,  0, }, // Bhutan
    {   48,  422,       0,   61221,       0,  0, 14,  0, }, // Bolivia
    {   48,  437,   61235,   61250,   61274, 15, 24, 19, }, // Brasilia
    {   48,  454,       0,   61293,       0,  0, 23,  0, }, // Brunei
    {   48,  461,   61316,   61332,   61357, 16, 25, 20, }, // Cape_Verde
    {   48,  478,       0,   61377,       0,  0, 13,  0, }, // Chamorro
    {   48,  487,   61390,   61402,   61423, 12, 21, 16, }, // Chatham
    {   48,  495,   61439,   61449,   61468, 10, 19, 14, }, // Chile
    {   48,  501,   61482,   61491,   61509,  9, 18, 13, }, // China
    {   48,  507,   61522,   61537,   61561, 15, 24, 19, }, // Choibalsan
    {   48,  518,       0,   61580,       0,  0, 17,  0, }, // Christmas
    {   48,  528,       0,   61597,       0,  0, 19,  0, }, // Cocos
    {   48,  534,   61616,   61631,   61655, 15, 24, 19, }, // Colombia
    {   48,  543,   61674,   61691,   61717, 17, 26, 27, }, // Cook
    {   48,  548,   61744,   61754,   61773, 10, 19, 14, }, // Cuba
    {   48,  559,       0,   61787,       0,  0, 11,  0, }, // Davis
    {   48,  575,       0,   61798,       0,  0, 19,  0, }, // DumontDUrville
    {   48,  612,       0,   61817,       0,  0, 17,  0, }, // East_Timor
    {   48,  623,   61834,   61851,   61877, 17, 26, 21, }, // Easter
    {   48,  630,       0,   61898,       0,  0, 13,  0, }, // Ecuador
    {   48,  638,   61911,   61931,   61960, 20, 29, 24, }, // Europe_Central
    {   48,  653,   61984,   62002,   62029, 18, 27, 22, }, // Europe_Eastern
    {   48,  668,       0,   62051,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {   48,  691,   62075,   62093,   62120, 18, 27, 22, }, // Europe_Western
    {   48,  706,   62142,   62164,   62195, 22, 31, 26, }, // Falkland
    {   48,  715,   62221,   62231,   62250, 10, 19, 14, }, // Fiji
    {   48,  720,       0,   62264,       0,  0, 22,  0, }, // French_Guiana
    {   48,  734,       0,   62286,       0,  0, 33,  0, }, // French_Southern
    {   48,  757,       0,   62319,       0,  0, 15,  0, }, // Galapagos
    {   48,  767,       0,   62334,       0,  0, 13,  0, }, // Gambier
    {   48,  775,   62347,   62362,   62386, 15, 24, 19, }, // Georgia
    {   48,  783,       0,   62405,       0,  0, 21,  0, }, // Gilbert_Islands
    {   48,  799,       0,   62426,       0,  0, 18,  0, }, // GMT
    {   48,  841,   62444,   62467,   62499, 23, 32, 27, }, // Greenland_Eastern
    {   48,  859,   62526,   62549,   62581, 23, 32, 27, }, // Greenland_Western
    {   48,  882,       0,   62608,       0,  0, 12,  0, }, // Gulf
    {   48,  887,       0,   62620,       0,  0, 12,  0, }, // Guyana
    {   48,  894,   62632,   62649,   62675, 17, 26, 21, }, // Hawaii_Aleutian
    {   48,  910,   62696,   62711,   62735, 15, 24, 19, }, // Hong_Kong
    {   48,  920,   62754,   62764,   62783, 10, 19, 14, }, // Hovd
    {   48,  925,       0,   62797,       0,  0, 15,  0, }, // India
    {   48,  931,       0,   62812,       0,  0, 17,  0, }, // Indian_Ocean
    {   48,  944,       0,   62829,       0,  0, 13,  0, }, // Indochina
    {   48,  954,       0,   62842,       0,  0, 24,  0, }, // Indonesia_Central
    {   48,  972,       0,   62866,       0,  0, 22,  0, }, // Indonesia_Eastern
    {   48,  990,       0,   62888,       0,  0, 22,  0, }, // Indonesia_Western
    {   48, 1008,   62910,   62920,   62939, 10, 19, 14, }, // Iran
    {   48, 1019,   62953,   62966,   62988, 13, 22, 17, }, // Irkutsk
    {   48, 1027,   63005,   63017,   63038, 12, 21, 16, }, // Israel
    {   48, 1034,   63054,   63068,   63091, 14, 23, 18, }, // Japan
    {   48, 1058,       0,   63109,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {   48, 1077,       0,   63131,       0,  0, 22,  0, }, // Kazakhstan_Western
    {   48, 1106,   63153,   63165,   63186, 12, 21, 16, }, // Korea
    {   48, 1112,       0,   63202,       0,  0, 11,  0, }, // Kosrae
    {   48, 1119,   63213,   63230,   63256, 17, 26, 21, }, // Krasnoyarsk
    {   48, 1151,       0,   63277,       0,  0, 17,  0, }, // Kyrgystan
    {   48, 1175,       0,   63294,       0,  0, 18,  0, }, // Line_Islands
    {   48, 1188,   63312,   63326,   63349, 14, 23, 18, }, // Lord_Howe
    {   48, 1204,   63367,   63380,   63402, 13, 22, 17, }, // Magadan
    {   48, 1219,       0,   63419,       0,  0, 15,  0, }, // Malaysia
    {   48, 1228,       0,   63434,       0,  0, 12,  0, }, // Maldives
    {   48, 1237,       0,   63446,       0,  0, 14,  0, }, // Marquesas
    {   48, 1247,       0,   63460,       0,  0, 20,  0, }, // Marshall_Islands
    {   48, 1264,   63480,   63493,   63515, 13, 22, 17, }, // Mauritius
    {   48, 1274,       0,   63532,       0,  0, 12,  0, }, // Mawson
    {   48, 1281,   63544,   63569,   63603, 25, 34, 29, }, // Mexico_Pacific
    {   48, 1296,   63632,   63647,   63671, 15, 24, 19, }, // Mongolia
    {   48, 1305,   63690,   63702,   63723, 12, 21, 16, }, // Moscow
    {   48, 1312,       0,   63739,       0,  0, 12,  0, }, // Myanmar
    {   48, 1320,       0,   63751,       0,  0, 11,  0, }, // Nauru
    {   48, 1326,       0,   63762,       0,  0, 11,  0, }, // Nepal
    {   48, 1332,   63773,   63794,   63824, 21, 30, 25, }, // New_Caledonia
    {   48, 1346,   63849,   63869,   63898, 20, 29, 24, }, // New_Zealand
    {   48, 1358,   63922,   63940,   63967, 18, 27, 22, }, // Newfoundland
    {   48, 1371,       0,   63989,       0,  0, 10,  0, }, // Niue
    {   48, 1376,   63999,   64018,   64046, 19, 28, 23, }, // Norfolk
    {   48, 1384,   64069,   64094,   64128, 25, 34, 29, }, // Noronha
    {   48, 1406,   64157,   64174,   64200, 17, 26, 21, }, // Novosibirsk
    {   48, 1418,   64221,   64231,   64250, 10, 19, 14, }, // Omsk
    {   48, 1428,   64264,   64278,   64301, 14, 23, 18, }, // Pakistan
    {   48, 1437,       0,   64319,       0,  0, 11,  0, }, // Palau
    {   48, 1443,       0,   64330,       0,  0, 24,  0, }, // Papua_New_Guinea
    {   48, 1460,   64354,   64368,   64391, 14, 23, 18, }, // Paraguay
    {   48, 1469,   64409,   64419,   64438, 10, 19, 14, }, // Peru
    {   48, 1474,   64452,   64466,   64489, 14, 23, 18, }, // Philippines
    {   48, 1486,       0,   64507,       0,  0, 20,  0, }, // Phoenix_Islands
    {   48, 1502,   64527,   64557,   64596, 30, 39, 34, }, // Pierre_Miquelon
    {   48, 1518,       0,   64630,       0,  0, 13,  0, }, // Pitcairn
    {   48, 1527,       0,   64643,       0,  0, 12,  0, }, // Ponape
    {   48, 1534,       0,   64655,       0,  0, 13,  0, }, // Pyongyang
    {   48, 1554,       0,   64668,       0,  0,  8,  0, }, // Reunion
    {   48, 1562,       0,   64676,       0,  0, 12,  0, }, // Rothera
    {   48, 1570,   64688,   64701,   64723, 13, 22, 17, }, // Sakhalin
    {   48, 1579,   64740,   64752,   64773, 12, 21, 16, }, // Samara
    {   48, 1596,   64789,   64800,   64820, 11, 20, 15, }, // Samoa
    {   48, 1602,       0,   64835,       0,  0, 20,  0, }, // Seychelles
    {   48, 1624,       0,   64855,       0,  0, 14,  0, }, // Singapore
    {   48, 1634,       0,   64869,       0,  0, 21,  0, }, // Solomon
    {   48, 1642,       0,   64890,       0,  0, 20,  0, }, // South_Georgia
    {   48, 1656,       0,   64910,       0,  0, 13,  0, }, // Suriname
    {   48, 1676,       0,   64923,       0,  0, 11,  0, }, // Syowa
    {   48, 1682,       0,   64934,       0,  0, 12,  0, }, // Tahiti
    {   48, 1689,   64946,   64958,   64979, 12, 21, 16, }, // Taipei
    {   48, 1696,       0,   64995,       0,  0, 16,  0, }, // Tajikistan
    {   48, 1724,       0,   65011,       0,  0, 13,  0, }, // Tokelau
    {   48, 1732,   65024,   65035,   65055, 11, 20, 15, }, // Tonga
    {   48, 1738,       0,   65070,       0,  0, 10,  0, }, // Truk
    {   48, 1750,   65080,   65098,   65125, 18, 27, 22, }, // Turkmenistan
    {   48, 1763,       0,   65147,       0,  0, 12,  0, }, // Tuvalu
    {   48, 1777,   65159,   65172,   65194, 13, 22, 17, }, // Uruguay
    {   48, 1792,   65211,   65227,   65252, 16, 25, 20, }, // Uzbekistan
    {   48, 1803,   65272,   65285,   65307, 13, 22, 17, }, // Vanuatu
    {   48, 1811,       0,   65324,       0,  0, 15,  0, }, // Venezuela
    {   48, 1821,   65339,   65356,   65382, 17, 26, 21, }, // Vladivostok
    {   48, 1833,   65403,   65418,   65442, 15, 24, 19, }, // Volgograd
    {   48, 1843,       0,   65461,       0,  0, 12,  0, }, // Vostok
    {   48, 1850,       0,   65473,       0,  0, 10,  0, }, // Wake
    {   48, 1855,       0,   65483,       0,  0, 22,  0, }, // Wallis
    {   48, 1862,   65505,   65518,   65540, 13, 22, 17, }, // Yakutsk
    {   48, 1870,   65557,   65576,   65604, 19, 28, 23, }, // Yekaterinburg
    {   48, 1892,       0,   65627,       0,  0, 11,  0, }, // Yukon
    {   56,    0,   65638,   65646,   65659,  8, 13, 19, }, // Acre Bangla/Bangla/Bangladesh
    {   56,    5,       0,   65678,       0,  0, 16,  0, }, // Afghanistan
    {   56,   17,       0,   65694,       0,  0, 17,  0, }, // Africa_Central
    {   56,   32,       0,   65711,       0,  0, 18,  0, }, // Africa_Eastern
    {   56,   65,       0,   65729,       0,  0, 24,  0, }, // Africa_Southern
    {   56,   81,   65753,   65772,   65796, 19, 24, 32, }, // Africa_Western
    {   56,  107,   65828,   65840,   65857, 12, 17, 20, }, // Alaska
    {   56,  128,   65877,   65889,   65906, 12, 17, 23, }, // Almaty
    {   56,  135,   65929,   65939,   65954, 10, 15, 23, }, // Amazon
    {   56,  142,   65977,   65992,   66012, 15, 20, 23, }, // America_Central
    {   56,  158,   66035,   66053,   66075, 18, 22, 25, }, // America_Eastern
    {   56,  174,   66100,   66121,   66147, 21, 26, 27, }, // America_Mountain
    {   56,  191,   66174,   66206,   66243, 32, 37, 38, }, // America_Pacific
    {   56,  207,   66281,   66294,   66312, 13, 18, 26, }, // Anadyr
    {   56,  214,   66338,   66349,   66365, 11, 16, 17, }, // Apia
    {   56,  219,   66382,   66392,   66407, 10, 15, 21, }, // Aqtau
    {   56,  225,   66428,   66438,   66453, 10, 15, 21, }, // Aqtobe
    {   56,  232,   66474,   66483,   66497,  9, 14, 17, }, // Arabian
    {   56,  240,   66514,   66530,   66551, 16, 21, 29, }, // Argentina
    {   56,  250,   66580,   66604,   66635, 24, 31, 36, }, // Argentina_Western
    {   56,  268,   66671,   66686,   66706, 15, 20, 28, }, // Armenia
    {   56,  286,   66734,   66750,   66769, 16, 19, 22, }, // Atlantic
    {   56,  295,   66791,   66818,   66850, 27, 32, 35, }, // Australia_Central
    {   56,  313,   66885,   66920,   66960, 35, 40, 43, }, // Australia_CentralWestern
    {   56,  338,   67003,   67025,   67052, 22, 27, 30, }, // Australia_Eastern
    {   56,  356,   67082,   67106,   67135, 24, 29, 32, }, // Australia_Western
    {   56,  374,   67167,   67182,   67202, 15, 20, 28, }, // Azerbaijan
    {   56,  385,   67230,   67241,   67257, 11, 16, 24, }, // Azores
    {   56,  397,   67281,   67294,   67312, 13, 18, 26, }, // Bangladesh
    {   56,  415,       0,   67338,       0,  0, 10,  0, }, // Bhutan
    {   56,  422,       0,   67348,       0,  0, 14,  0, }, // Bolivia
    {   56,  437,   67362,   67378,   67399, 16, 21, 29, }, // Brasilia
    {   56,  454,       0,   67428,       0,  0, 23,  0, }, // Brunei
    {   56,  461,   67451,   67465,   67484, 14, 19, 27, }, // Cape_Verde
    {   56,  478,       0,   67511,       0,  0, 16,  0, }, // Chamorro
    {   56,  487,   67527,   67539,   67556, 12, 17, 20, }, // Chatham
    {   56,  495,   67576,   67585,   67599,  9, 14, 22, }, // Chile
    {   56,  501,   67621,   67629,   67642,  8, 13, 16, }, // China
    {   56,  507,   67658,   67671,   67689, 13, 18, 26, }, // Choibalsan
    {   56,  518,       0,   67715,       0,  0, 19,  0, }, // Christmas
    {   56,  528,       0,   67734,       0,  0, 21,  0, }, // Cocos
    {   56,  534,   67755,   67771,   67792, 16, 21, 29, }, // Colombia
    {   56,  543,   67821,   67840,   67864, 19, 24, 39, }, // Cook
    {   56,  548,   67903,   67914,   67929, 11, 15, 18, }, // Cuba
    {   56,  559,       0,   67947,       0,  0, 10,  0, }, // Davis
    {   56,  575,       0,   67957,       0,  0, 22,  0, }, // DumontDUrville
    {   56,  612,       0,   67979,       0,  0, 15,  0, }, // East_Timor
    {   56,  623,   67994,   68011,   68033, 17, 22, 30, }, // Easter
    {   56,  630,       0,   68063,       0,  0, 13,  0, }, // Ecuador
    {   56,  638,   68076,   68094,   68117, 18, 23, 31, }, // Europe_Central
    {   56,  653,   68148,   68167,   68191, 19, 24, 32, }, // Europe_Eastern
    {   56,  668,       0,   68223,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {   56,  691,   68253,   68273,   68298, 20, 25, 33, }, // Europe_Western
    {   56,  706,   68331,   68356,   68386, 25, 30, 38, }, // Falkland
    {   56,  715,   68424,   68433,   68447,  9, 14, 22, }, // Fiji
    {   56,  720,       0,   68469,       0,  0, 18,  0, }, // French_Guiana
    {   56,  734,       0,   68487,       0,  0, 33,  0, }, // French_Southern
    {   56,  757,       0,   68520,       0,  0, 14,  0, }, // Galapagos
    {   56,  767,       0,   68534,       0,  0, 17,  0, }, // Gambier
    {   56,  775,   68551,   68564,   68582, 13, 18, 26, }, // Georgia
    {   56,  783,       0,   68608,       0,  0, 24,  0, }, // Gilbert_Islands
    {   56,  799,       0,   68632,       0,  0, 16,  0, }, // GMT
    {   56,  841,   68648,   68671,   68699, 23, 28, 36, }, // Greenland_Eastern
    {   56,  859,   68735,   68759,   68788, 24, 29, 37, }, // Greenland_Western
    {   56,  877,       0,   68825,       0,  0, 15,  0, }, // Guam
    {   56,  882,       0,   68840,       0,  0, 19,  0, }, // Gulf
    {   56,  887,       0,   68859,       0,  0, 12,  0, }, // Guyana
    {   56,  894,   68871,   68889,   68912, 18, 23, 26, }, // Hawaii_Aleutian
    {   56,  910,   68938,   68948,   68963, 10, 15, 23, }, // Hong_Kong
    {   56,  920,   68986,   68995,   69009,  9, 14, 22, }, // Hovd
    {   56,  925,       0,   69031,       0,  0, 17,  0, }, // India
    {   56,  931,       0,   69048,       0,  0, 20,  0, }, // Indian_Ocean
    {   56,  944,       0,   69068,       0,  0, 13,  0, }, // Indochina
    {   56,  954,       0,   69081,       0,  0, 28,  0, }, // Indonesia_Central
    {   56,  972,       0,   69109,       0,  0, 23,  0, }, // Indonesia_Eastern
    {   56,  990,       0,   69132,       0,  0, 25,  0, }, // Indonesia_Western
    {   56, 1008,   69157,   69166,   69180,  9, 14, 17, }, // Iran
    {   56, 1019,   69197,   69210,   69228, 13, 18, 26, }, // Irkutsk
    {   56, 1027,   69254,   69267,   69285, 13, 18, 21, }, // Israel
    {   56, 1034,   69306,   69316,   69331, 10, 15, 18, }, // Japan
    {   56, 1040,   69349,   69385,   69425, 36, 40, 47, }, // Kamchatka
    {   56, 1058,       0,   69472,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {   56, 1077,       0,   69494,       0,  0, 23,  0, }, // Kazakhstan_Western
    {   56, 1106,   69517,   69530,   69548, 13, 18, 21, }, // Korea
    {   56, 1112,       0,   69569,       0,  0, 11,  0, }, // Kosrae
    {   56, 1119,   69580,   69601,   69627, 21, 26, 34, }, // Krasnoyarsk
    {   56, 1151,       0,   69661,       0,  0, 15,  0, }, // Kyrgystan
    {   56, 1161,       0,   69676,       0,  0, 10,  0, }, // Lanka
    {   56, 1175,       0,   69686,       0,  0, 20,  0, }, // Line_Islands
    {   56, 1188,   69706,   69722,   69743, 16, 21, 24, }, // Lord_Howe
    {   56, 1198,   69767,   69777,   69791, 10, 14, 21, }, // Macau
    {   56, 1204,   69812,   69826,   69845, 14, 19, 27, }, // Magadan
    {   56, 1219,       0,   69872,       0,  0, 16,  0, }, // Malaysia
    {   56, 1228,       0,   69888,       0,  0, 13,  0, }, // Maldives
    {   56, 1237,       0,   69901,       0,  0, 14,  0, }, // Marquesas
    {   56, 1247,       0,   69915,       0,  0, 23,  0, }, // Marshall_Islands
    {   56, 1264,   69938,   69949,   69965, 11, 16, 24, }, // Mauritius
    {   56, 1274,       0,   69989,       0,  0,  8,  0, }, // Mawson
    {   56, 1281,   69997,   70031,   70069, 34, 38, 42, }, // Mexico_Pacific
    {   56, 1296,   70111,   70126,   70146, 15, 20, 28, }, // Mongolia
    {   56, 1305,   70174,   70184,   70199, 10, 15, 23, }, // Moscow
    {   56, 1312,       0,   70222,       0,  0, 14,  0, }, // Myanmar
    {   56, 1320,       0,   70236,       0,  0, 10,  0, }, // Nauru
    {   56, 1326,       0,   70246,       0,  0, 10,  0, }, // Nepal
    {   56, 1332,   70256,   70278,   70305, 22, 27, 35, }, // New_Caledonia
    {   56, 1346,   70340,   70357,   70379, 17, 22, 25, }, // New_Zealand
    {   56, 1358,   70404,   70425,   70451, 21, 26, 29, }, // Newfoundland
    {   56, 1371,       0,   70480,       0,  0,  9,  0, }, // Niue
    {   56, 1376,   70489,   70505,   70526, 16, 21, 24, }, // Norfolk
    {   56, 1384,   70550,   70576,   70607, 26, 31, 39, }, // Noronha
    {   56, 1392,       0,   70646,       0,  0, 27,  0, }, // North_Mariana
    {   56, 1406,   70673,   70691,   70714, 18, 23, 31, }, // Novosibirsk
    {   56, 1418,   70745,   70755,   70770, 10, 15, 23, }, // Omsk
    {   56, 1428,   70793,   70807,   70826, 14, 19, 27, }, // Pakistan
    {   56, 1437,       0,   70853,       0,  0, 10,  0, }, // Palau
    {   56, 1443,       0,   70863,       0,  0, 21,  0, }, // Papua_New_Guinea
    {   56, 1460,   70884,   70900,   70921, 16, 21, 29, }, // Paraguay
    {   56, 1469,   70950,   70959,   70973,  9, 14, 22, }, // Peru
    {   56, 1474,   70995,   71008,   71026, 13, 18, 26, }, // Philippines
    {   56, 1486,       0,   71052,       0,  0, 23,  0, }, // Phoenix_Islands
    {   56, 1502,   71075,   71101,   71132, 26, 31, 34, }, // Pierre_Miquelon
    {   56, 1518,       0,   71166,       0,  0, 16,  0, }, // Pitcairn
    {   56, 1527,       0,   71182,       0,  0, 11,  0, }, // Ponape
    {   56, 1534,       0,   71193,       0,  0, 15,  0, }, // Pyongyang
    {   56, 1544,   71208,   71223,   71242, 15, 19, 26, }, // Qyzylorda
    {   56, 1554,       0,   71268,       0,  0, 14,  0, }, // Reunion
    {   56, 1562,       0,   71282,       0,  0, 10,  0, }, // Rothera
    {   56, 1570,   71292,   71304,   71321, 12, 17, 25, }, // Sakhalin
    {   56, 1579,   71346,   71357,   71372, 11, 15, 22, }, // Samara
    {   56, 1596,   71394,   71406,   71423, 12, 17, 20, }, // Samoa
    {   56, 1602,       0,   71443,       0,  0, 11,  0, }, // Seychelles
    {   56, 1624,       0,   71454,       0,  0, 19,  0, }, // Singapore
    {   56, 1634,       0,   71473,       0,  0, 21,  0, }, // Solomon
    {   56, 1642,       0,   71494,       0,  0, 20,  0, }, // South_Georgia
    {   56, 1656,       0,   71514,       0,  0, 12,  0, }, // Suriname
    {   56, 1676,       0,   71526,       0,  0, 14,  0, }, // Syowa
    {   56, 1682,       0,   71540,       0,  0, 11,  0, }, // Tahiti
    {   56, 1689,   71551,   71562,   71578, 11, 16, 19, }, // Taipei
    {   56, 1696,       0,   71597,       0,  0, 16,  0, }, // Tajikistan
    {   56, 1724,       0,   71613,       0,  0, 12,  0, }, // Tokelau
    {   56, 1732,   71625,   71636,   71652, 11, 16, 24, }, // Tonga
    {   56, 1738,       0,   71676,       0,  0,  8,  0, }, // Truk
    {   56, 1750,   71684,   71703,   71727, 19, 24, 32, }, // Turkmenistan
    {   56, 1763,       0,   71759,       0,  0, 11,  0, }, // Tuvalu
    {   56, 1777,   71770,   71783,   71801, 13, 18, 26, }, // Uruguay
    {   56, 1792,   71827,   71843,   71864, 16, 21, 29, }, // Uzbekistan
    {   56, 1803,   71893,   71907,   71926, 14, 19, 27, }, // Vanuatu
    {   56, 1811,       0,   71953,       0,  0, 16,  0, }, // Venezuela
    {   56, 1821,   71969,   71985,   72006, 16, 21, 29, }, // Vladivostok
    {   56, 1833,   72035,   72049,   72068, 14, 19, 27, }, // Volgograd
    {   56, 1843,       0,   72095,       0,  0,  9,  0, }, // Vostok
    {   56, 1850,       0,   72104,       0,  0, 16,  0, }, // Wake
    {   56, 1855,       0,   72120,       0,  0, 23,  0, }, // Wallis
    {   56, 1862,   72143,   72159,   72180, 16, 21, 29, }, // Yakutsk
    {   56, 1870,   72209,   72230,   72256, 21, 26, 34, }, // Yekaterinburg
    {   56, 1892,       0,   72290,       0,  0, 10,  0, }, // Yukon
    {   60,    0,   72300,   72312,   72334, 12, 22, 18, }, // Acre Basque/Latin/Spain
    {   60,    5,       0,   72352,       0,  0, 18,  0, }, // Afghanistan
    {   60,   17,       0,   72370,       0,  0, 25,  0, }, // Africa_Central
    {   60,   32,       0,   72395,       0,  0, 24,  0, }, // Africa_Eastern
    {   60,   65,       0,   72419,       0,  0, 25,  0, }, // Africa_Southern
    {   60,   81,   72444,   72471,   72508, 27, 37, 33, }, // Africa_Western
    {   60,  107,   72541,   72555,   72579, 14, 24, 20, }, // Alaska
    {   60,  128,   72599,   72613,   72637, 14, 24, 20, }, // Almaty
    {   60,  135,   72657,   72673,   72699, 16, 26, 22, }, // Amazon
    {   60,  142,   72721,   72752,   72793, 31, 41, 37, }, // America_Central
    {   60,  158,   72830,   72860,   72900, 30, 40, 36, }, // America_Eastern
    {   60,  174,   72936,   72968,   73010, 32, 42, 38, }, // America_Mountain
    {   60,  191,   73048,   73079,   73120, 31, 41, 37, }, // America_Pacific
    {   60,  207,   73157,   73173,   73199, 16, 26, 22, }, // Anadyr
    {   60,  214,   73221,   73233,   73255, 12, 22, 18, }, // Apia
    {   60,  219,   73273,   73286,   73309, 13, 23, 19, }, // Aqtau
    {   60,  225,   73328,   73342,   73366, 14, 24, 20, }, // Aqtobe
    {   60,  232,   73386,   73400,   73424, 14, 24, 20, }, // Arabian
    {   60,  240,   73444,   73461,   73488, 17, 27, 23, }, // Argentina
    {   60,  250,   73511,   73539,   73577, 28, 38, 34, }, // Argentina_Western
    {   60,  268,   73611,   73626,   73651, 15, 25, 21, }, // Armenia
    {   60,  286,   73672,   73704,   73746, 32, 42, 38, }, // Atlantic
    {   60,  295,   73784,   73812,   73850, 28, 38, 34, }, // Australia_Central
    {   60,  313,   73884,   73919,   73964, 35, 45, 41, }, // Australia_CentralWestern
    {   60,  338,   74005,   74032,   74069, 27, 37, 33, }, // Australia_Eastern
    {   60,  356,   74102,   74132,   74172, 30, 40, 36, }, // Australia_Western
    {   60,  374,   74208,   74226,   74254, 18, 28, 24, }, // Azerbaijan
    {   60,  385,   74278,   74294,   74320, 16, 26, 22, }, // Azores
    {   60,  397,   74342,   74361,   74390, 19, 29, 25, }, // Bangladesh
    {   60,  415,       0,   74415,       0,  0, 14,  0, }, // Bhutan
    {   60,  422,       0,   74429,       0,  0, 15,  0, }, // Bolivia
    {   60,  437,   74444,   74460,   74486, 16, 26, 22, }, // Brasilia
    {   60,  454,       0,   74508,       0,  0, 25,  0, }, // Brunei
    {   60,  461,   74533,   74551,   74579, 18, 28, 24, }, // Cape_Verde
    {   60,  472,       0,   74603,       0,  0, 13,  0, }, // Casey
    {   60,  478,       0,   74616,       0,  0, 26,  0, }, // Chamorro
    {   60,  487,   74642,   74657,   74682, 15, 25, 21, }, // Chatham
    {   60,  495,   74703,   74716,   74739, 13, 23, 19, }, // Chile
    {   60,  501,   74758,   74771,   74794, 13, 23, 19, }, // China
    {   60,  507,   74813,   74831,   74859, 18, 28, 24, }, // Choibalsan
    {   60,  518,       0,   74883,       0,  0, 24,  0, }, // Christmas
    {   60,  528,       0,   74907,       0,  0, 23,  0, }, // Cocos
    {   60,  534,   74930,   74946,   74972, 16, 26, 22, }, // Colombia
    {   60,  543,   74994,   75016,   75048, 22, 32, 37, }, // Cook
    {   60,  548,   75085,   75097,   75119, 12, 22, 18, }, // Cuba
    {   60,  559,       0,   75137,       0,  0, 14,  0, }, // Davis
    {   60,  575,       0,   75151,       0,  0, 24,  0, }, // DumontDUrville
    {   60,  612,       0,   75175,       0,  0, 25,  0, }, // East_Timor
    {   60,  623,   75200,   75220,   75250, 20, 30, 26, }, // Easter
    {   60,  630,       0,   75276,       0,  0, 17,  0, }, // Ecuador
    {   60,  638,   75293,   75318,   75353, 25, 35, 31, }, // Europe_Central
    {   60,  653,   75384,   75408,   75442, 24, 34, 30, }, // Europe_Eastern
    {   60,  668,       0,   75472,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {   60,  691,   75503,   75530,   75567, 27, 37, 33, }, // Europe_Western
    {   60,  706,   75600,   75626,   75662, 26, 36, 32, }, // Falkland
    {   60,  715,   75694,   75706,   75728, 12, 22, 18, }, // Fiji
    {   60,  720,       0,   75746,       0,  0, 24,  0, }, // French_Guiana
    {   60,  734,       0,   75770,       0,  0, 57,  0, }, // French_Southern
    {   60,  757,       0,   75827,       0,  0, 19,  0, }, // Galapagos
    {   60,  767,       0,   75846,       0,  0, 19,  0, }, // Gambier
    {   60,  775,   75865,   75880,   75905, 15, 25, 21, }, // Georgia
    {   60,  783,       0,   75926,       0,  0, 25,  0, }, // Gilbert_Islands
    {   60,  799,       0,   75951,       0,  0, 31,  0, }, // GMT
    {   60,  841,   75982,   76011,   76050, 29, 39, 35, }, // Greenland_Eastern
    {   60,  859,   76085,   76117,   76159, 32, 42, 38, }, // Greenland_Western
    {   60,  877,       0,   76197,       0,  0, 23,  0, }, // Guam
    {   60,  882,       0,   76220,       0,  0, 23,  0, }, // Gulf
    {   60,  887,       0,   76243,       0,  0, 14,  0, }, // Guyana
    {   60,  894,   76257,   76290,   76333, 33, 43, 39, }, // Hawaii_Aleutian
    {   60,  910,   76372,   76388,   76414, 16, 26, 22, }, // Hong_Kong
    {   60,  920,   76436,   76450,   76474, 14, 24, 20, }, // Hovd
    {   60,  925,       0,   76494,       0,  0, 13,  0, }, // India
    {   60,  931,       0,   76507,       0,  0, 22,  0, }, // Indian_Ocean
    {   60,  944,       0,   76529,       0,  0, 17,  0, }, // Indochina
    {   60,  954,       0,   76546,       0,  0, 28,  0, }, // Indonesia_Central
    {   60,  972,       0,   76574,       0,  0, 27,  0, }, // Indonesia_Eastern
    {   60,  990,       0,   76601,       0,  0, 30,  0, }, // Indonesia_Western
    {   60, 1008,   76631,   76643,   76665, 12, 22, 18, }, // Iran
    {   60, 1019,   76683,   76699,   76725, 16, 26, 22, }, // Irkutsk
    {   60, 1027,   76747,   76761,   76785, 14, 24, 20, }, // Israel
    {   60, 1034,   76805,   76820,   76845, 15, 25, 21, }, // Japan
    {   60, 1040,   76866,   76898,   76940, 32, 42, 38, }, // Kamchatka
    {   60, 1058,       0,   76978,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {   60, 1077,       0,   77006,       0,  0, 31,  0, }, // Kazakhstan_Western
    {   60, 1106,   77037,   77050,   77073, 13, 23, 19, }, // Korea
    {   60, 1112,       0,   77092,       0,  0, 14,  0, }, // Kosrae
    {   60, 1119,   77106,   77126,   77156, 20, 30, 26, }, // Krasnoyarsk
    {   60, 1151,       0,   77182,       0,  0, 19,  0, }, // Kyrgystan
    {   60, 1161,       0,   77201,       0,  0, 13,  0, }, // Lanka
    {   60, 1175,       0,   77214,       0,  0, 22,  0, }, // Line_Islands
    {   60, 1188,   77236,   77253,   77280, 17, 27, 23, }, // Lord_Howe
    {   60, 1198,   77303,   77316,   77339, 13, 23, 19, }, // Macau
    {   60, 1204,   77358,   77373,   77398, 15, 25, 21, }, // Magadan
    {   60, 1219,       0,   77419,       0,  0, 16,  0, }, // Malaysia
    {   60, 1228,       0,   77435,       0,  0, 17,  0, }, // Maldives
    {   60, 1237,       0,   77452,       0,  0, 17,  0, }, // Marquesas
    {   60, 1247,       0,   77469,       0,  0, 26,  0, }, // Marshall_Islands
    {   60, 1264,   77495,   77511,   77537, 16, 26, 22, }, // Mauritius
    {   60, 1274,       0,   77559,       0,  0, 15,  0, }, // Mawson
    {   60, 1281,   77574,   77599,   77634, 25, 35, 31, }, // Mexico_Pacific
    {   60, 1296,   77665,   77685,   77715, 20, 30, 26, }, // Mongolia
    {   60, 1305,   77741,   77754,   77777, 13, 23, 19, }, // Moscow
    {   60, 1312,       0,   77796,       0,  0, 17,  0, }, // Myanmar
    {   60, 1320,       0,   77813,       0,  0, 13,  0, }, // Nauru
    {   60, 1326,       0,   77826,       0,  0, 13,  0, }, // Nepal
    {   60, 1332,   77839,   77862,   77895, 23, 33, 29, }, // New_Caledonia
    {   60, 1346,   77924,   77946,   77978, 22, 32, 28, }, // New_Zealand
    {   60, 1358,   78006,   78020,   78044, 14, 24, 20, }, // Newfoundland
    {   60, 1371,       0,   78064,       0,  0, 12,  0, }, // Niue
    {   60, 1376,   78076,   78101,   78136, 25, 35, 31, }, // Norfolk
    {   60, 1384,   78167,   78194,   78231, 27, 37, 33, }, // Noronha
    {   60, 1392,       0,   78264,       0,  0, 30,  0, }, // North_Mariana
    {   60, 1406,   78294,   78314,   78344, 20, 30, 26, }, // Novosibirsk
    {   60, 1418,   78370,   78383,   78406, 13, 23, 19, }, // Omsk
    {   60, 1428,   78425,   78441,   78467, 16, 26, 22, }, // Pakistan
    {   60, 1437,       0,   78489,       0,  0, 13,  0, }, // Palau
    {   60, 1443,       0,   78502,       0,  0, 25,  0, }, // Papua_New_Guinea
    {   60, 1460,   78527,   78543,   78569, 16, 26, 22, }, // Paraguay
    {   60, 1469,   78591,   78603,   78625, 12, 22, 18, }, // Peru
    {   60, 1474,   78643,   78661,   78689, 18, 28, 24, }, // Philippines
    {   60, 1486,       0,   78713,       0,  0, 25,  0, }, // Phoenix_Islands
    {   60, 1502,   78738,   78771,   78814, 33, 43, 39, }, // Pierre_Miquelon
    {   60, 1518,       0,   78853,       0,  0, 17,  0, }, // Pitcairn
    {   60, 1527,       0,   78870,       0,  0, 14,  0, }, // Ponape
    {   60, 1534,       0,   78884,       0,  0, 18,  0, }, // Pyongyang
    {   60, 1544,   78902,   78919,   78946, 17, 27, 23, }, // Qyzylorda
    {   60, 1554,       0,   78969,       0,  0, 16,  0, }, // Reunion
    {   60, 1562,       0,   78985,       0,  0, 15,  0, }, // Rothera
    {   60, 1570,   79000,   79016,   79042, 16, 26, 22, }, // Sakhalin
    {   60, 1579,   79064,   79078,   79102, 14, 24, 20, }, // Samara
    {   60, 1596,   79122,   79135,   79158, 13, 23, 19, }, // Samoa
    {   60, 1602,       0,   79177,       0,  0, 27,  0, }, // Seychelles
    {   60, 1624,       0,   79204,       0,  0, 28,  0, }, // Singapore
    {   60, 1634,       0,   79232,       0,  0, 25,  0, }, // Solomon
    {   60, 1642,       0,   79257,       0,  0, 28,  0, }, // South_Georgia
    {   60, 1656,       0,   79285,       0,  0, 15,  0, }, // Suriname
    {   60, 1676,       0,   79300,       0,  0, 13,  0, }, // Syowa
    {   60, 1682,       0,   79313,       0,  0, 14,  0, }, // Tahiti
    {   60, 1689,   79327,   79341,   79365, 14, 24, 20, }, // Taipei
    {   60, 1696,       0,   79385,       0,  0, 19,  0, }, // Tajikistan
    {   60, 1724,       0,   79404,       0,  0, 15,  0, }, // Tokelau
    {   60, 1732,   79419,   79432,   79455, 13, 23, 19, }, // Tonga
    {   60, 1738,       0,   79474,       0,  0, 14,  0, }, // Truk
    {   60, 1750,   79488,   79508,   79538, 20, 30, 26, }, // Turkmenistan
    {   60, 1763,       0,   79564,       0,  0, 14,  0, }, // Tuvalu
    {   60, 1777,   79578,   79593,   79618, 15, 25, 21, }, // Uruguay
    {   60, 1792,   79639,   79657,   79685, 18, 28, 24, }, // Uzbekistan
    {   60, 1803,   79709,   79724,   79749, 15, 25, 21, }, // Vanuatu
    {   60, 1811,       0,   79770,       0,  0, 17,  0, }, // Venezuela
    {   60, 1821,   79787,   79807,   79837, 20, 30, 26, }, // Vladivostok
    {   60, 1833,   79863,   79881,   79909, 18, 28, 24, }, // Volgograd
    {   60, 1843,       0,   79933,       0,  0, 15,  0, }, // Vostok
    {   60, 1850,       0,   79948,       0,  0, 19,  0, }, // Wake
    {   60, 1855,       0,   79967,       0,  0, 29,  0, }, // Wallis
    {   60, 1862,   79996,   80012,   80038, 16, 26, 22, }, // Yakutsk
    {   60, 1870,   80060,   80082,   80114, 22, 32, 28, }, // Yekaterinburg
    {   60, 1892,       0,   80142,       0,  0, 13,  0, }, // Yukon
    {   61,    5,       0,   80155,       0,  0, 17,  0, }, // Afghanistan Belarusian/Cyrillic/Belarus
    {   61,   17,       0,   80172,       0,  0, 24,  0, }, // Africa_Central
    {   61,   32,       0,   80196,       0,  0, 21,  0, }, // Africa_Eastern
    {   61,   65,       0,   80217,       0,  0, 22,  0, }, // Africa_Southern
    {   61,   81,   80239,   80260,   80292, 21, 32, 27, }, // Africa_Western
    {   61,  107,   80319,   80329,   80350, 10, 21, 16, }, // Alaska
    {   61,  135,   80366,   80379,   80403, 13, 24, 19, }, // Amazon
    {   61,  142,   80422,   80456,   80501, 34, 45, 40, }, // America_Central
    {   61,  158,   80541,   80572,   80614, 31, 42, 37, }, // America_Eastern
    {   61,  174,   80651,   80680,   80720, 29, 40, 35, }, // America_Mountain
    {   61,  191,   80755,   80771,   80798, 16, 27, 22, }, // America_Pacific
    {   61,  214,   80820,   80828,   80847,  8, 19, 14, }, // Apia
    {   61,  232,   80861,   80882,   80914, 21, 32, 27, }, // Arabian
    {   61,  240,   80941,   80956,   80982, 15, 26, 21, }, // Argentina
    {   61,  250,   81003,   81025,   81058, 22, 33, 28, }, // Argentina_Western
    {   61,  268,   81086,   81097,   81119, 11, 22, 17, }, // Armenia
    {   61,  286,   81136,   81150,   81175, 14, 25, 20, }, // Atlantic
    {   61,  295,   81195,   81220,   81256, 25, 36, 31, }, // Australia_Central
    {   61,  313,   81287,   81319,   81362, 32, 43, 38, }, // Australia_CentralWestern
    {   61,  338,   81400,   81422,   81455, 22, 33, 28, }, // Australia_Eastern
    {   61,  356,   81483,   81505,   81538, 22, 33, 28, }, // Australia_Western
    {   61,  374,   81566,   81582,   81609, 16, 27, 22, }, // Azerbaijan
    {   61,  385,   81631,   81652,   81684, 21, 32, 27, }, // Azores
    {   61,  397,   81711,   81724,   81748, 13, 24, 19, }, // Bangladesh
    {   61,  415,       0,   81767,       0,  0, 10,  0, }, // Bhutan
    {   61,  422,       0,   81777,       0,  0, 14,  0, }, // Bolivia
    {   61,  437,   81791,   81805,   81830, 14, 25, 20, }, // Brasilia
    {   61,  454,       0,   81850,       0,  0, 10,  0, }, // Brunei
    {   61,  461,   81860,   81874,   81899, 14, 25, 20, }, // Cape_Verde
    {   61,  478,       0,   81919,       0,  0, 10,  0, }, // Chamorro
    {   61,  487,   81929,   81939,   81960, 10, 21, 16, }, // Chatham
    {   61,  495,   81976,   81988,   82011, 12, 23, 18, }, // Chile
    {   61,  501,   82029,   82038,   82058,  9, 20, 15, }, // China
    {   61,  507,   82073,   82087,   82112, 14, 25, 20, }, // Choibalsan
    {   61,  518,       0,   82132,       0,  0, 18,  0, }, // Christmas
    {   61,  528,       0,   82150,       0,  0, 22,  0, }, // Cocos
    {   61,  534,   82172,   82187,   82213, 15, 26, 21, }, // Colombia
    {   61,  543,   82234,   82251,   82279, 17, 28, 26, }, // Cook
    {   61,  548,   82305,   82313,   82332,  8, 19, 14, }, // Cuba
    {   61,  559,       0,   82346,       0,  0, 18,  0, }, // Davis
    {   61,  575,       0,   82364,       0,  0, 27,  0, }, // DumontDUrville
    {   61,  612,       0,   82391,       0,  0, 20,  0, }, // East_Timor
    {   61,  623,   82411,   82433,   82466, 22, 33, 28, }, // Easter
    {   61,  630,       0,   82494,       0,  0, 14,  0, }, // Ecuador
    {   61,  638,   82508,   82532,   82567, 24, 35, 30, }, // Europe_Central
    {   61,  653,   82597,   82618,   82650, 21, 32, 27, }, // Europe_Eastern
    {   61,  668,       0,   82677,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {   61,  691,   82704,   82725,   82757, 21, 32, 27, }, // Europe_Western
    {   61,  706,   82784,   82809,   82845, 25, 36, 31, }, // Falkland
    {   61,  715,   82876,   82885,   82905,  9, 20, 15, }, // Fiji
    {   61,  720,       0,   82920,       0,  0, 22,  0, }, // French_Guiana
    {   61,  734,       0,   82942,       0,  0, 47,  0, }, // French_Southern
    {   61,  757,       0,   82989,       0,  0, 36,  0, }, // Galapagos
    {   61,  767,       0,   83025,       0,  0, 19,  0, }, // Gambier
    {   61,  775,   83044,   83057,   83081, 13, 24, 19, }, // Georgia
    {   61,  783,       0,   83100,       0,  0, 21,  0, }, // Gilbert_Islands
    {   61,  799,       0,   83121,       0,  0, 15,  0, }, // GMT
    {   61,  841,   83136,   83159,   83193, 23, 34, 29, }, // Greenland_Eastern
    {   61,  859,   83222,   83245,   83279, 23, 34, 29, }, // Greenland_Western
    {   61,  882,       0,   83308,       0,  0, 22,  0, }, // Gulf
    {   61,  887,       0,   83330,       0,  0,  9,  0, }, // Guyana
    {   61,  894,   83339,   83359,   83390, 20, 31, 26, }, // Hawaii_Aleutian
    {   61,  910,   83416,   83428,   83451, 12, 23, 18, }, // Hong_Kong
    {   61,  920,   83469,   83478,   83498,  9, 20, 15, }, // Hovd
    {   61,  925,       0,   83513,       0,  0,  9,  0, }, // India
    {   61,  931,       0,   83522,       0,  0, 21,  0, }, // Indian_Ocean
    {   61,  944,       0,   83543,       0,  0, 16,  0, }, // Indochina
    {   61,  954,       0,   83559,       0,  0, 26,  0, }, // Indonesia_Central
    {   61,  972,       0,   83585,       0,  0, 23,  0, }, // Indonesia_Eastern
    {   61,  990,       0,   83608,       0,  0, 23,  0, }, // Indonesia_Western
    {   61, 1008,   83631,   83642,   83664, 11, 22, 17, }, // Iran
    {   61, 1019,   83681,   83692,   83714, 11, 22, 17, }, // Irkutsk
    {   61, 1027,   83731,   83745,   83770, 14, 25, 20, }, // Israel
    {   61, 1034,   83790,   83800,   83821, 10, 21, 16, }, // Japan
    {   61, 1058,       0,   83837,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {   61, 1077,       0,   83860,       0,  0, 23,  0, }, // Kazakhstan_Western
    {   61, 1106,   83883,   83892,   83912,  9, 20, 15, }, // Korea
    {   61, 1112,       0,   83927,       0,  0, 19,  0, }, // Kosrae
    {   61, 1119,   83946,   83961,   83987, 15, 26, 21, }, // Krasnoyarsk
    {   61, 1151,       0,   84008,       0,  0, 15,  0, }, // Kyrgystan
    {   61, 1175,       0,   84023,       0,  0, 17,  0, }, // Line_Islands
    {   61, 1188,   84040,   84052,   84075, 12, 23, 18, }, // Lord_Howe
    {   61, 1204,   84093,   84107,   84132, 14, 25, 20, }, // Magadan
    {   61, 1219,       0,   84152,       0,  0, 12,  0, }, // Malaysia
    {   61, 1228,       0,   84164,       0,  0, 13,  0, }, // Maldives
    {   61, 1237,       0,   84177,       0,  0, 23,  0, }, // Marquesas
    {   61, 1247,       0,   84200,       0,  0, 23,  0, }, // Marshall_Islands
    {   61, 1264,   84223,   84235,   84258, 12, 23, 18, }, // Mauritius
    {   61, 1274,       0,   84276,       0,  0, 18,  0, }, // Mawson
    {   61, 1281,   84294,   84322,   84360, 28, 38, 34, }, // Mexico_Pacific
    {   61, 1296,   84394,   84409,   84435, 15, 26, 21, }, // Mongolia
    {   61, 1305,   84456,   84469,   84493, 13, 24, 19, }, // Moscow
    {   61, 1312,       0,   84512,       0,  0, 10,  0, }, // Myanmar
    {   61, 1320,       0,   84522,       0,  0,  9,  0, }, // Nauru
    {   61, 1326,       0,   84531,       0,  0, 13,  0, }, // Nepal
    {   61, 1332,   84544,   84563,   84593, 19, 30, 25, }, // New_Caledonia
    {   61, 1346,   84618,   84636,   84665, 18, 29, 24, }, // New_Zealand
    {   61, 1358,   84689,   84708,   84738, 19, 30, 25, }, // Newfoundland
    {   61, 1371,       0,   84763,       0,  0,  8,  0, }, // Niue
    {   61, 1376,   84771,   84791,   84822, 20, 31, 26, }, // Norfolk
    {   61, 1384,   84848,   84872,   84907, 24, 35, 30, }, // Noronha
    {   61, 1406,   84937,   84953,   84980, 16, 27, 22, }, // Novosibirsk
    {   61, 1418,   85002,   85011,   85031,  9, 20, 15, }, // Omsk
    {   61, 1428,   85046,   85061,   85087, 15, 26, 21, }, // Pakistan
    {   61, 1437,       0,   85108,       0,  0,  9,  0, }, // Palau
    {   61, 1443,       0,   85117,       0,  0, 22,  0, }, // Papua_New_Guinea
    {   61, 1460,   85139,   85151,   85174, 12, 23, 18, }, // Paraguay
    {   61, 1469,   85192,   85205,   85229, 13, 24, 19, }, // Peru
    {   61, 1474,   85248,   85262,   85287, 14, 25, 20, }, // Philippines
    {   61, 1486,       0,   85307,       0,  0, 19,  0, }, // Phoenix_Islands
    {   61, 1502,   85326,   85348,   85381, 22, 33, 39, }, // Pierre_Miquelon
    {   61, 1518,       0,   85420,       0,  0, 20,  0, }, // Pitcairn
    {   61, 1527,       0,   85440,       0,  0, 19,  0, }, // Ponape
    {   61, 1534,       0,   85459,       0,  0, 14,  0, }, // Pyongyang
    {   61, 1554,       0,   85473,       0,  0, 12,  0, }, // Reunion
    {   61, 1562,       0,   85485,       0,  0, 18,  0, }, // Rothera
    {   61, 1570,   85503,   85517,   85542, 14, 25, 20, }, // Sakhalin
    {   61, 1596,   85562,   85571,   85591,  9, 20, 15, }, // Samoa
    {   61, 1602,       0,   85606,       0,  0, 24,  0, }, // Seychelles
    {   61, 1624,       0,   85630,       0,  0, 15,  0, }, // Singapore
    {   61, 1634,       0,   85645,       0,  0, 24,  0, }, // Solomon
    {   61, 1642,       0,   85669,       0,  0, 21,  0, }, // South_Georgia
    {   61, 1656,       0,   85690,       0,  0, 12,  0, }, // Suriname
    {   61, 1676,       0,   85702,       0,  0, 16,  0, }, // Syowa
    {   61, 1682,       0,   85718,       0,  0,  9,  0, }, // Tahiti
    {   61, 1689,   85727,   85737,   85758, 10, 21, 16, }, // Taipei
    {   61, 1696,       0,   85774,       0,  0, 16,  0, }, // Tajikistan
    {   61, 1724,       0,   85790,       0,  0, 11,  0, }, // Tokelau
    {   61, 1732,   85801,   85810,   85830,  9, 20, 15, }, // Tonga
    {   61, 1738,       0,   85845,       0,  0,  9,  0, }, // Truk
    {   61, 1750,   85854,   85871,   85899, 17, 28, 23, }, // Turkmenistan
    {   61, 1763,       0,   85922,       0,  0, 10,  0, }, // Tuvalu
    {   61, 1777,   85932,   85946,   85971, 14, 25, 20, }, // Uruguay
    {   61, 1792,   85991,   86006,   86032, 15, 26, 21, }, // Uzbekistan
    {   61, 1803,   86053,   86064,   86086, 11, 22, 17, }, // Vanuatu
    {   61, 1811,       0,   86103,       0,  0, 16,  0, }, // Venezuela
    {   61, 1821,   86119,   86137,   86166, 18, 29, 24, }, // Vladivostok
    {   61, 1833,   86190,   86206,   86233, 16, 27, 22, }, // Volgograd
    {   61, 1843,       0,   86255,       0,  0, 18,  0, }, // Vostok
    {   61, 1850,       0,   86273,       0,  0, 17,  0, }, // Wake
    {   61, 1855,       0,   86290,       0,  0, 27,  0, }, // Wallis
    {   61, 1862,   86317,   86327,   86348, 10, 21, 16, }, // Yakutsk
    {   61, 1870,   86364,   86383,   86413, 19, 30, 25, }, // Yekaterinburg
    {   61, 1892,       0,   86438,       0,  0,  9,  0, }, // Yukon
    {   64,  799,       0,   86447,       0,  0, 17,  0, }, // GMT Bhojpuri/Devanagari/India
    {   66,    0,   86464,   86471,   86489,  7, 18, 12, }, // Acre Bodo/Devanagari/India
    {   66,    5,       0,   86501,       0,  0, 14,  0, }, // Afghanistan
    {   66,   17,       0,   86515,       0,  0, 15,  0, }, // Africa_Central
    {   66,   32,       0,   86530,       0,  0, 16,  0, }, // Africa_Eastern
    {   66,   65,       0,   86546,       0,  0, 25,  0, }, // Africa_Southern
    {   66,   81,   86571,   86589,   86615, 18, 26, 26, }, // Africa_Western
    {   66,  107,   86641,   86653,   86675, 12, 22, 19, }, // Alaska
    {   66,  128,   86694,   86705,   86726, 11, 21, 15, }, // Almaty
    {   66,  135,   86741,   86749,   86765,  8, 16, 16, }, // Amazon
    {   66,  142,   86781,   86803,   86836, 22, 33, 29, }, // America_Central
    {   66,  158,   86865,   86892,   86927, 27, 35, 34, }, // America_Eastern
    {   66,  174,   86961,   86989,   87025, 28, 36, 35, }, // America_Mountain
    {   66,  191,   87060,   87085,   87118, 25, 33, 32, }, // America_Pacific
    {   66,  207,   87150,   87159,   87180,  9, 21, 15, }, // Anadyr
    {   66,  214,   87195,   87203,   87219,  8, 16, 12, }, // Apia
    {   66,  219,   87231,   87240,   87262,  9, 22, 16, }, // Aqtau
    {   66,  225,   87278,   87287,   87310,  9, 23, 17, }, // Aqtobe
    {   66,  232,   87327,   87338,   87357, 11, 19, 18, }, // Arabian
    {   66,  240,   87375,   87389,   87411, 14, 22, 22, }, // Argentina
    {   66,  250,   87433,   87453,   87481, 20, 28, 28, }, // Argentina_Western
    {   66,  268,   87509,   87521,   87541, 12, 20, 20, }, // Armenia
    {   66,  286,   87561,   87573,   87593, 12, 20, 19, }, // Atlantic
    {   66,  295,   87612,   87632,   87662, 20, 30, 27, }, // Australia_Central
    {   66,  313,   87689,   87719,   87757, 30, 38, 37, }, // Australia_CentralWestern
    {   66,  338,   87794,   87818,   87852, 24, 34, 33, }, // Australia_Eastern
    {   66,  356,   87885,   87908,   87941, 23, 33, 32, }, // Australia_Western
    {   66,  374,   87973,   87986,   88007, 13, 21, 21, }, // Azerbaijan
    {   66,  385,   88028,   88039,   88058, 11, 19, 19, }, // Azores
    {   66,  397,   88077,   88088,   88107, 11, 19, 19, }, // Bangladesh
    {   66,  415,       0,   88126,       0,  0,  8,  0, }, // Bhutan
    {   66,  422,       0,   88134,       0,  0, 10,  0, }, // Bolivia
    {   66,  437,   88144,   88157,   88178, 13, 21, 21, }, // Brasilia
    {   66,  454,       0,   88199,       0,  0, 33,  0, }, // Brunei
    {   66,  461,   88232,   88245,   88266, 13, 21, 21, }, // Cape_Verde
    {   66,  478,       0,   88287,       0,  0, 19,  0, }, // Chamorro
    {   66,  487,   88306,   88314,   88330,  8, 16, 15, }, // Chatham
    {   66,  495,   88345,   88352,   88367,  7, 15, 15, }, // Chile
    {   66,  501,   88382,   88390,   88406,  8, 16, 15, }, // China
    {   66,  507,   88421,   88433,   88453, 12, 20, 20, }, // Choibalsan
    {   66,  518,       0,   88473,       0,  0, 19,  0, }, // Christmas
    {   66,  528,       0,   88492,       0,  0, 18,  0, }, // Cocos
    {   66,  534,   88510,   88521,   88540, 11, 19, 19, }, // Colombia
    {   66,  543,   88559,   88576,   88601, 17, 25, 31, }, // Cook
    {   66,  548,   88632,   88640,   88656,  8, 16, 15, }, // Cuba
    {   66,  559,       0,   88671,       0,  0,  8,  0, }, // Davis
    {   66,  575,       0,   88679,       0,  0, 18,  0, }, // DumontDUrville
    {   66,  612,       0,   88697,       0,  0, 25,  0, }, // East_Timor
    {   66,  623,   88722,   88740,   88766, 18, 26, 26, }, // Easter
    {   66,  630,       0,   88792,       0,  0, 10,  0, }, // Ecuador
    {   66,  638,   88802,   88817,   88840, 15, 23, 23, }, // Europe_Central
    {   66,  653,   88863,   88883,   88911, 20, 28, 28, }, // Europe_Eastern
    {   66,  668,       0,   88939,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {   66,  691,   88968,   88984,   89008, 16, 24, 24, }, // Europe_Western
    {   66,  706,   89032,   89053,   89082, 21, 29, 29, }, // Falkland
    {   66,  715,   89111,   89118,   89133,  7, 15, 15, }, // Fiji
    {   66,  720,       0,   89148,       0,  0, 18,  0, }, // French_Guiana
    {   66,  734,       0,   89166,       0,  0, 35,  0, }, // French_Southern
    {   66,  757,       0,   89201,       0,  0, 12,  0, }, // Galapagos
    {   66,  767,       0,   89213,       0,  0, 12,  0, }, // Gambier
    {   66,  775,   89225,   89235,   89253, 10, 18, 18, }, // Georgia
    {   66,  783,       0,   89271,       0,  0, 22,  0, }, // Gilbert_Islands
    {   66,  799,       0,   89293,       0,  0, 20,  0, }, // GMT
    {   66,  841,   89313,   89332,   89359, 19, 27, 28, }, // Greenland_Eastern
    {   66,  859,   89387,   89406,   89433, 19, 27, 27, }, // Greenland_Western
    {   66,  877,       0,   89460,       0,  0, 19,  0, }, // Guam
    {   66,  882,       0,   89479,       0,  0, 16,  0, }, // Gulf
    {   66,  887,       0,   89495,       0,  0,  9,  0, }, // Guyana
    {   66,  894,   89504,   89523,   89550, 19, 27, 26, }, // Hawaii_Aleutian
    {   66,  910,   89576,   89583,   89598,  7, 15, 15, }, // Hong_Kong
    {   66,  920,   89613,   89621,   89637,  8, 16, 16, }, // Hovd
    {   66,  925,       0,   89653,       0,  0, 18,  0, }, // India
    {   66,  931,       0,   89671,       0,  0, 18,  0, }, // Indian_Ocean
    {   66,  944,       0,   89689,       0,  0, 24,  0, }, // Indochina
    {   66,  954,       0,   89713,       0,  0, 25,  0, }, // Indonesia_Central
    {   66,  972,       0,   89738,       0,  0, 33,  0, }, // Indonesia_Eastern
    {   66,  990,       0,   89771,       0,  0, 34,  0, }, // Indonesia_Western
    {   66, 1008,   89805,   89812,   89827,  7, 15, 14, }, // Iran
    {   66, 1019,   89841,   89854,   89875, 13, 21, 22, }, // Irkutsk
    {   66, 1027,   89897,   89907,   89925, 10, 18, 17, }, // Israel
    {   66, 1034,   89942,   89950,   89966,  8, 16, 15, }, // Japan
    {   66, 1040,   89981,   90013,   90050, 32, 37, 31, }, // Kamchatka
    {   66, 1058,       0,   90081,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {   66, 1077,       0,   90100,       0,  0, 19,  0, }, // Kazakhstan_Western
    {   66, 1106,   90119,   90128,   90145,  9, 17, 16, }, // Korea
    {   66, 1112,       0,   90161,       0,  0,  8,  0, }, // Kosrae
    {   66, 1119,   90169,   90187,   90213, 18, 26, 27, }, // Krasnoyarsk
    {   66, 1151,       0,   90240,       0,  0, 14,  0, }, // Kyrgystan
    {   66, 1161,       0,   90254,       0,  0, 19,  0, }, // Lanka
    {   66, 1175,       0,   90273,       0,  0, 18,  0, }, // Line_Islands
    {   66, 1188,   90291,   90303,   90323, 12, 20, 19, }, // Lord_Howe
    {   66, 1198,   90342,   90350,   90370,  8, 20, 14, }, // Macau
    {   66, 1204,   90384,   90394,   90412, 10, 18, 18, }, // Magadan
    {   66, 1219,       0,   90430,       0,  0, 22,  0, }, // Malaysia
    {   66, 1228,       0,   90452,       0,  0, 11,  0, }, // Maldives
    {   66, 1237,       0,   90463,       0,  0, 12,  0, }, // Marquesas
    {   66, 1247,       0,   90475,       0,  0, 21,  0, }, // Marshall_Islands
    {   66, 1264,   90496,   90510,   90532, 14, 22, 22, }, // Mauritius
    {   66, 1274,       0,   90554,       0,  0,  7,  0, }, // Mawson
    {   66, 1281,   90561,   90582,   90611, 21, 29, 28, }, // Mexico_Pacific
    {   66, 1296,   90639,   90651,   90671, 12, 20, 20, }, // Mongolia
    {   66, 1305,   90691,   90699,   90715,  8, 16, 16, }, // Moscow
    {   66, 1312,       0,   90731,       0,  0, 23,  0, }, // Myanmar
    {   66, 1320,       0,   90754,       0,  0,  8,  0, }, // Nauru
    {   66, 1326,       0,   90762,       0,  0,  8,  0, }, // Nepal
    {   66, 1332,   90770,   90788,   90814, 18, 26, 26, }, // New_Caledonia
    {   66, 1346,   90840,   90853,   90874, 13, 21, 20, }, // New_Zealand
    {   66, 1358,   90894,   90911,   90936, 17, 25, 24, }, // Newfoundland
    {   66, 1371,       0,   90960,       0,  0,  7,  0, }, // Niue
    {   66, 1376,   90967,   90981,   91003, 14, 22, 21, }, // Norfolk
    {   66, 1384,   91024,   91047,   91078, 23, 31, 31, }, // Noronha
    {   66, 1392,       0,   91109,       0,  0, 28,  0, }, // North_Mariana
    {   66, 1406,   91137,   91152,   91175, 15, 23, 23, }, // Novosibirsk
    {   66, 1418,   91198,   91207,   91224,  9, 17, 17, }, // Omsk
    {   66, 1428,   91241,   91253,   91273, 12, 20, 20, }, // Pakistan
    {   66, 1437,       0,   91293,       0,  0,  8,  0, }, // Palau
    {   66, 1443,       0,   91301,       0,  0, 17,  0, }, // Papua_New_Guinea
    {   66, 1460,   91318,   91329,   91348, 11, 19, 19, }, // Paraguay
    {   66, 1469,   91367,   91374,   91389,  7, 15, 15, }, // Peru
    {   66, 1474,   91404,   91416,   91440, 12, 24, 18, }, // Philippines
    {   66, 1486,       0,   91458,       0,  0, 20,  0, }, // Phoenix_Islands
    {   66, 1502,   91478,   91505,   91540, 27, 35, 34, }, // Pierre_Miquelon
    {   66, 1518,       0,   91574,       0,  0, 11,  0, }, // Pitcairn
    {   66, 1527,       0,   91585,       0,  0,  8,  0, }, // Ponape
    {   66, 1534,       0,   91593,       0,  0, 10,  0, }, // Pyongyang
    {   66, 1544,   91603,   91620,   91647, 17, 27, 21, }, // Qyzylorda
    {   66, 1554,       0,   91668,       0,  0, 11,  0, }, // Reunion
    {   66, 1562,       0,   91679,       0,  0,  8,  0, }, // Rothera
    {   66, 1570,   91687,   91697,   91715, 10, 18, 18, }, // Sakhalin
    {   66, 1579,   91733,   91743,   91763, 10, 20, 14, }, // Samara
    {   66, 1596,   91777,   91786,   91803,  9, 17, 16, }, // Samoa
    {   66, 1602,       0,   91819,       0,  0, 10,  0, }, // Seychelles
    {   66, 1624,       0,   91829,       0,  0, 23,  0, }, // Singapore
    {   66, 1634,       0,   91852,       0,  0, 19,  0, }, // Solomon
    {   66, 1642,       0,   91871,       0,  0, 15,  0, }, // South_Georgia
    {   66, 1656,       0,   91886,       0,  0, 10,  0, }, // Suriname
    {   66, 1676,       0,   91896,       0,  0,  8,  0, }, // Syowa
    {   66, 1682,       0,   91904,       0,  0,  9,  0, }, // Tahiti
    {   66, 1689,   91913,   91922,   91939,  9, 17, 16, }, // Taipei
    {   66, 1696,       0,   91955,       0,  0, 14,  0, }, // Tajikistan
    {   66, 1724,       0,   91969,       0,  0,  8,  0, }, // Tokelau
    {   66, 1732,   91977,   91984,   91999,  7, 15, 15, }, // Tonga
    {   66, 1738,       0,   92014,       0,  0,  6,  0, }, // Truk
    {   66, 1750,   92020,   92037,   92062, 17, 25, 25, }, // Turkmenistan
    {   66, 1763,       0,   92087,       0,  0,  9,  0, }, // Tuvalu
    {   66, 1777,   92096,   92106,   92124, 10, 18, 18, }, // Uruguay
    {   66, 1792,   92142,   92156,   92178, 14, 22, 22, }, // Uzbekistan
    {   66, 1803,   92200,   92210,   92228, 10, 18, 18, }, // Vanuatu
    {   66, 1811,       0,   92246,       0,  0, 13,  0, }, // Venezuela
    {   66, 1821,   92259,   92274,   92297, 15, 23, 23, }, // Vladivostok
    {   66, 1833,   92320,   92333,   92354, 13, 21, 21, }, // Volgograd
    {   66, 1843,       0,   92265,       0,  0,  9,  0, }, // Vostok
    {   66, 1850,       0,   92375,       0,  0, 12,  0, }, // Wake
    {   66, 1855,       0,   92387,       0,  0, 19,  0, }, // Wallis
    {   66, 1862,   92406,   92418,   92438, 12, 20, 20, }, // Yakutsk
    {   66, 1870,   92458,   92476,   92502, 18, 26, 26, }, // Yekaterinburg
    {   66, 1892,       0,   92528,       0,  0,  7,  0, }, // Yukon
    {   67,    0,   92535,   92545,   92566, 10, 21, 29, }, // Acre Bosnian/Latin/Bosnia and Herzegovina
    {   67,    5,       0,   92595,       0,  0, 21,  0, }, // Afghanistan
    {   67,   17,       0,   92616,       0,  0, 24,  0, }, // Africa_Central
    {   67,   32,       0,   92640,       0,  0, 22,  0, }, // Africa_Eastern
    {   67,   65,       0,   92662,       0,  0, 31,  0, }, // Africa_Southern
    {   67,   81,   92693,   92715,   92748, 22, 33, 29, }, // Africa_Western
    {   67,  107,   92777,   92796,   92826, 19, 30, 26, }, // Alaska
    {   67,  128,   92852,   92864,   92887, 12, 23, 31, }, // Almaty
    {   67,  135,   92918,   92935,   92963, 17, 28, 24, }, // Amazon
    {   67,  142,   92987,   93021,   93066, 34, 45, 41, }, // America_Central
    {   67,  158,   93107,   93139,   93182, 32, 43, 39, }, // America_Eastern
    {   67,  174,   93221,   93255,   93300, 34, 45, 41, }, // America_Mountain
    {   67,  191,   93341,   93375,   93420, 34, 45, 41, }, // America_Pacific
    {   67,  207,   93461,   93473,   93496, 12, 23, 31, }, // Anadyr
    {   67,  214,   93527,   93542,   93568, 15, 26, 22, }, // Apia
    {   67,  219,   93590,   93602,   93625, 12, 23, 31, }, // Aqtau
    {   67,  225,   93656,   93669,   93693, 13, 24, 32, }, // Aqtobe
    {   67,  232,   93725,   93742,   93770, 17, 28, 24, }, // Arabian
    {   67,  240,   93794,   93813,   93843, 19, 30, 26, }, // Argentina
    {   67,  250,   93869,   93895,   93932, 26, 37, 33, }, // Argentina_Western
    {   67,  268,   93965,   93981,   94008, 16, 27, 23, }, // Armenia
    {   67,  286,   94031,   94065,   94110, 34, 45, 41, }, // Atlantic
    {   67,  295,   94151,   94180,   94220, 29, 40, 36, }, // Australia_Central
    {   67,  313,   94256,   94294,   94342, 38, 48, 44, }, // Australia_CentralWestern
    {   67,  338,   94386,   94413,   94451, 27, 38, 34, }, // Australia_Eastern
    {   67,  356,   94485,   94512,   94550, 27, 38, 34, }, // Australia_Western
    {   67,  374,   94584,   94606,   94639, 22, 33, 29, }, // Azerbaijan
    {   67,  385,   94668,   94683,   94709, 15, 26, 22, }, // Azores
    {   67,  397,   94731,   94750,   94780, 19, 30, 26, }, // Bangladesh
    {   67,  415,       0,   94806,       0,  0, 16,  0, }, // Bhutan
    {   67,  422,       0,   94822,       0,  0, 18,  0, }, // Bolivia
    {   67,  437,   94840,   94859,   94889, 19, 30, 26, }, // Brasilia
    {   67,  454,       0,   94915,       0,  0, 17,  0, }, // Brunei
    {   67,  461,   94932,   94951,   94981, 19, 30, 26, }, // Cape_Verde
    {   67,  478,       0,   95007,       0,  0, 27,  0, }, // Chamorro
    {   67,  487,   95034,   95050,   95077, 16, 27, 23, }, // Chatham
    {   67,  495,   95100,   95117,   95145, 17, 28, 24, }, // Chile
    {   67,  501,   95169,   95184,   95210, 15, 26, 22, }, // China
    {   67,  507,   95232,   95252,   95283, 20, 31, 27, }, // Choibalsan
    {   67,  518,       0,   95310,       0,  0, 26,  0, }, // Christmas
    {   67,  528,       0,   95336,       0,  0, 25,  0, }, // Cocos
    {   67,  534,   95361,   95380,   95410, 19, 30, 26, }, // Colombia
    {   67,  543,   95436,   95463,   95501, 27, 38, 38, }, // Cook
    {   67,  548,   95539,   95555,   95582, 16, 27, 23, }, // Cuba
    {   67,  559,       0,   95605,       0,  0, 21,  0, }, // Davis
    {   67,  575,       0,   95626,       0,  0, 32,  0, }, // DumontDUrville
    {   67,  612,       0,   95658,       0,  0, 23,  0, }, // East_Timor
    {   67,  623,   95681,   95705,   95740, 24, 35, 31, }, // Easter
    {   67,  630,       0,   95771,       0,  0, 18,  0, }, // Ecuador
    {   67,  638,   95789,   95814,   95850, 25, 36, 32, }, // Europe_Central
    {   67,  653,   95882,   95905,   95939, 23, 34, 30, }, // Europe_Eastern
    {   67,  668,       0,   95969,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {   67,  691,   95998,   96021,   96055, 23, 34, 30, }, // Europe_Western
    {   67,  706,   96085,   96104,   96134, 19, 30, 26, }, // Falkland
    {   67,  715,   96160,   96178,   96207, 18, 29, 24, }, // Fiji
    {   67,  720,       0,   96231,       0,  0, 26,  0, }, // French_Guiana
    {   67,  734,       0,   96257,       0,  0, 52,  0, }, // French_Southern
    {   67,  757,       0,   96309,       0,  0, 19,  0, }, // Galapagos
    {   67,  767,       0,   96328,       0,  0, 19,  0, }, // Gambier
    {   67,  775,   96347,   96364,   96392, 17, 28, 24, }, // Georgia
    {   67,  783,       0,   96416,       0,  0, 31,  0, }, // Gilbert_Islands
    {   67,  799,       0,   96447,       0,  0, 16,  0, }, // GMT
    {   67,  841,   96463,   96489,   96526, 26, 37, 33, }, // Greenland_Eastern
    {   67,  859,   96559,   96585,   96622, 26, 37, 33, }, // Greenland_Western
    {   67,  877,       0,   96655,       0,  0, 21,  0, }, // Guam
    {   67,  882,       0,   96676,       0,  0, 27,  0, }, // Gulf
    {   67,  887,       0,   96703,       0,  0, 17,  0, }, // Guyana
    {   67,  894,   96720,   96747,   96785, 27, 38, 34, }, // Hawaii_Aleutian
    {   67,  910,   96819,   96837,   96866, 18, 29, 25, }, // Hong_Kong
    {   67,  920,   96891,   96906,   96932, 15, 26, 22, }, // Hovd
    {   67,  925,       0,   96954,       0,  0, 27,  0, }, // India
    {   67,  931,       0,   96981,       0,  0, 27,  0, }, // Indian_Ocean
    {   67,  944,       0,   97008,       0,  0, 19,  0, }, // Indochina
    {   67,  954,       0,   97027,       0,  0, 29,  0, }, // Indonesia_Central
    {   67,  972,       0,   97056,       0,  0, 27,  0, }, // Indonesia_Eastern
    {   67,  990,       0,   97083,       0,  0, 27,  0, }, // Indonesia_Western
    {   67, 1008,   97110,   97125,   97151, 15, 26, 22, }, // Iran
    {   67, 1019,   97173,   97189,   97216, 16, 27, 23, }, // Irkutsk
    {   67, 1027,   97239,   97256,   97284, 17, 28, 24, }, // Israel
    {   67, 1034,   97308,   97324,   97351, 16, 27, 23, }, // Japan
    {   67, 1040,   97374,   97403,   97443, 29, 40, 48, }, // Kamchatka
    {   67, 1058,       0,   97491,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {   67, 1077,       0,   97518,       0,  0, 27,  0, }, // Kazakhstan_Western
    {   67, 1106,   97545,   97561,   97588, 16, 27, 23, }, // Korea
    {   67, 1112,       0,   97611,       0,  0, 24,  0, }, // Kosrae
    {   67, 1119,   97635,   97655,   97686, 20, 31, 27, }, // Krasnoyarsk
    {   67, 1151,       0,   97713,       0,  0, 20,  0, }, // Kyrgystan
    {   67, 1161,       0,   97733,       0,  0, 11,  0, }, // Lanka
    {   67, 1175,       0,   97744,       0,  0, 24,  0, }, // Line_Islands
    {   67, 1188,   97768,   97794,   97831, 26, 37, 33, }, // Lord_Howe
    {   67, 1198,   97864,   97875,   97897, 11, 22, 30, }, // Macau
    {   67, 1204,   97927,   97945,   97974, 18, 29, 25, }, // Magadan
    {   67, 1219,       0,   97999,       0,  0, 18,  0, }, // Malaysia
    {   67, 1228,       0,   98017,       0,  0, 17,  0, }, // Maldives
    {   67, 1237,       0,   98034,       0,  0, 26,  0, }, // Marquesas
    {   67, 1247,       0,   98060,       0,  0, 30,  0, }, // Marshall_Islands
    {   67, 1264,   98090,   98109,   98139, 19, 30, 26, }, // Mauritius
    {   67, 1274,       0,   98165,       0,  0, 22,  0, }, // Mawson
    {   67, 1281,   98187,   98213,   98250, 26, 37, 33, }, // Mexico_Pacific
    {   67, 1296,   98283,   98303,   98334, 20, 31, 27, }, // Mongolia
    {   67, 1305,   98361,   98378,   98406, 17, 28, 24, }, // Moscow
    {   67, 1312,       0,   98430,       0,  0, 19,  0, }, // Myanmar
    {   67, 1320,       0,   98449,       0,  0, 23,  0, }, // Nauru
    {   67, 1326,       0,   98472,       0,  0, 16,  0, }, // Nepal
    {   67, 1332,   98488,   98512,   98547, 24, 35, 31, }, // New_Caledonia
    {   67, 1346,   98578,   98599,   98631, 21, 32, 28, }, // New_Zealand
    {   67, 1358,   98659,   98682,   98716, 23, 34, 30, }, // Newfoundland
    {   67, 1371,       0,   98746,       0,  0, 22,  0, }, // Niue
    {   67, 1376,   98768,   98785,   98813, 17, 28, 24, }, // Norfolk
    {   67, 1384,   98837,   98874,   98922, 37, 48, 44, }, // Noronha
    {   67, 1392,       0,   98966,       0,  0, 31,  0, }, // North_Mariana
    {   67, 1406,   98997,   99017,   99048, 20, 31, 27, }, // Novosibirsk
    {   67, 1418,   99075,   99088,   99112, 13, 24, 20, }, // Omsk
    {   67, 1428,   99132,   99151,   99181, 19, 30, 26, }, // Pakistan
    {   67, 1437,       0,   99207,       0,  0, 23,  0, }, // Palau
    {   67, 1443,       0,   99230,       0,  0, 30,  0, }, // Papua_New_Guinea
    {   67, 1460,   99260,   99279,   99309, 19, 30, 26, }, // Paraguay
    {   67, 1469,   99335,   99352,   99380, 17, 28, 24, }, // Peru
    {   67, 1474,   99404,   99422,   99451, 18, 29, 25, }, // Philippines
    {   67, 1486,       0,   99476,       0,  0, 26,  0, }, // Phoenix_Islands
    {   67, 1502,   99502,   99543,   99595, 41, 52, 48, }, // Pierre_Miquelon
    {   67, 1518,       0,   99643,       0,  0, 27,  0, }, // Pitcairn
    {   67, 1527,       0,   99670,       0,  0, 24,  0, }, // Ponape
    {   67, 1534,       0,   99694,       0,  0, 19,  0, }, // Pyongyang
    {   67, 1544,   99713,   99728,   99754, 15, 26, 34, }, // Qyzylorda
    {   67, 1554,       0,   99788,       0,  0, 18,  0, }, // Reunion
    {   67, 1562,       0,   99806,       0,  0, 23,  0, }, // Rothera
    {   67, 1570,   99829,   99847,   99876, 18, 29, 25, }, // Sakhalin
    {   67, 1579,   99901,   99913,   99936, 12, 23, 31, }, // Samara
    {   67, 1596,   99967,   99984,  100012, 17, 28, 24, }, // Samoa
    {   67, 1602,       0,  100036,       0,  0, 17,  0, }, // Seychelles
    {   67, 1624,       0,  100053,       0,  0, 30,  0, }, // Singapore
    {   67, 1634,       0,  100083,       0,  0, 31,  0, }, // Solomon
    {   67, 1642,       0,  100114,       0,  0, 24,  0, }, // South_Georgia
    {   67, 1656,       0,  100138,       0,  0, 18,  0, }, // Suriname
    {   67, 1676,       0,  100156,       0,  0, 21,  0, }, // Syowa
    {   67, 1682,       0,  100177,       0,  0, 18,  0, }, // Tahiti
    {   67, 1689,  100195,  100212,  100240, 17, 28, 24, }, // Taipei
    {   67, 1696,       0,  100264,       0,  0, 22,  0, }, // Tajikistan
    {   67, 1724,       0,  100286,       0,  0, 25,  0, }, // Tokelau
    {   67, 1732,  100311,  100328,  100356, 17, 28, 24, }, // Tonga
    {   67, 1738,       0,  100380,       0,  0, 13,  0, }, // Truk
    {   67, 1750,  100393,  100416,  100450, 23, 34, 30, }, // Turkmenistan
    {   67, 1763,       0,  100480,       0,  0, 19,  0, }, // Tuvalu
    {   67, 1777,  100499,  100517,  100546, 18, 29, 25, }, // Uruguay
    {   67, 1792,  100571,  100592,  100624, 21, 32, 28, }, // Uzbekistan
    {   67, 1803,  100652,  100672,  100703, 20, 31, 27, }, // Vanuatu
    {   67, 1811,       0,  100730,       0,  0, 21,  0, }, // Venezuela
    {   67, 1821,  100751,  100772,  100804, 21, 32, 28, }, // Vladivostok
    {   67, 1833,  100832,  100852,  100883, 20, 31, 27, }, // Volgograd
    {   67, 1843,       0,  100910,       0,  0, 22,  0, }, // Vostok
    {   67, 1850,       0,  100932,       0,  0, 22,  0, }, // Wake
    {   67, 1855,       0,  100954,       0,  0, 34,  0, }, // Wallis
    {   67, 1862,  100988,  101004,  101031, 16, 27, 23, }, // Yakutsk
    {   67, 1870,  101054,  101077,  101111, 23, 34, 30, }, // Yekaterinburg
    {   67, 1892,       0,  101141,       0,  0, 16,  0, }, // Yukon
    {   68,    0,  101157,  101167,  101188, 10, 21, 27, }, // Acre Bosnian/Cyrillic/Bosnia and Herzegovina
    {   68,    5,       0,  101215,       0,  0, 18,  0, }, // Afghanistan
    {   68,   17,       0,  101233,       0,  0, 25,  0, }, // Africa_Central
    {   68,   32,       0,  101258,       0,  0, 23,  0, }, // Africa_Eastern
    {   68,   65,       0,  101281,       0,  0, 21,  0, }, // Africa_Southern
    {   68,   81,  101302,  101325,  101359, 23, 34, 38, }, // Africa_Western
    {   68,  107,  101397,  101411,  101436, 14, 25, 20, }, // Alaska
    {   68,  128,  101456,  101468,  101491, 12, 23, 29, }, // Almaty
    {   68,  135,  101520,  101534,  101559, 14, 25, 29, }, // Amazon
    {   68,  142,  101588,  101605,  101633, 17, 28, 32, }, // America_Central
    {   68,  158,  101665,  101680,  101706, 15, 26, 30, }, // America_Eastern
    {   68,  174,  101736,  101753,  101781, 17, 28, 32, }, // America_Mountain
    {   68,  191,  101813,  101830,  101858, 17, 28, 32, }, // America_Pacific
    {   68,  207,  101890,  101902,  101925, 12, 23, 29, }, // Anadyr
    {   68,  214,  101954,  101967,  101991, 13, 24, 28, }, // Apia
    {   68,  219,  102019,  102032,  102056, 13, 24, 30, }, // Aqtau
    {   68,  225,  102086,  102099,  102123, 13, 24, 30, }, // Aqtobe
    {   68,  232,  102153,  102170,  102198, 17, 28, 32, }, // Arabian
    {   68,  240,  102230,  102247,  102275, 17, 28, 32, }, // Argentina
    {   68,  250,  102307,  102332,  102368, 25, 36, 40, }, // Argentina_Western
    {   68,  268,  102408,  102424,  102451, 16, 27, 31, }, // Armenia
    {   68,  286,  102482,  102498,  102525, 16, 27, 31, }, // Atlantic
    {   68,  295,  102556,  102586,  102627, 30, 41, 45, }, // Australia_Central
    {   68,  313,  102672,  102710,  102759, 38, 49, 53, }, // Australia_CentralWestern
    {   68,  338,  102812,  102840,  102879, 28, 39, 43, }, // Australia_Eastern
    {   68,  356,  102922,  102950,  102989, 28, 39, 43, }, // Australia_Western
    {   68,  374,  103032,  103050,  103079, 18, 29, 33, }, // Azerbaijan
    {   68,  385,  103112,  103125,  103149, 13, 24, 28, }, // Azores
    {   68,  397,  103177,  103194,  103222, 17, 28, 32, }, // Bangladesh
    {   68,  415,       0,  103254,       0,  0, 13,  0, }, // Bhutan
    {   68,  422,       0,  103267,       0,  0, 16,  0, }, // Bolivia
    {   68,  437,  103283,  103300,  103328, 17, 28, 32, }, // Brasilia
    {   68,  454,       0,  103360,       0,  0, 24,  0, }, // Brunei
    {   68,  461,  103384,  103403,  103433, 19, 30, 34, }, // Cape_Verde
    {   68,  478,       0,  103467,       0,  0, 14,  0, }, // Chamorro
    {   68,  487,  103481,  103494,  103518, 13, 24, 28, }, // Chatham
    {   68,  495,  103546,  103558,  103581, 12, 23, 27, }, // Chile
    {   68,  501,  103608,  103620,  103643, 12, 23, 27, }, // China
    {   68,  507,  103670,  103687,  103715, 17, 28, 32, }, // Choibalsan
    {   68,  518,       0,  103747,       0,  0, 22,  0, }, // Christmas
    {   68,  528,       0,  103769,       0,  0, 29,  0, }, // Cocos
    {   68,  534,  103798,  103815,  103843, 17, 28, 32, }, // Colombia
    {   68,  543,  103875,  103896,  103928, 21, 32, 41, }, // Cook
    {   68,  548,  103969,  103981,  104004, 12, 23, 27, }, // Cuba
    {   68,  559,       0,  104031,       0,  0, 14,  0, }, // Davis
    {   68,  575,       0,  104045,       0,  0, 21,  0, }, // DumontDUrville
    {   68,  612,       0,  104066,       0,  0, 19,  0, }, // East_Timor
    {   68,  623,  104085,  104107,  104140, 22, 33, 37, }, // Easter
    {   68,  630,       0,  104177,       0,  0, 15,  0, }, // Ecuador
    {   68,  638,  104192,  104214,  104247, 22, 33, 37, }, // Europe_Central
    {   68,  653,  104284,  104307,  104341, 23, 34, 38, }, // Europe_Eastern
    {   68,  668,       0,  104379,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {   68,  691,  104407,  104430,  104464, 23, 34, 38, }, // Europe_Western
    {   68,  706,  104502,  104528,  104565, 26, 37, 41, }, // Falkland
    {   68,  715,  104606,  104618,  104641, 12, 23, 27, }, // Fiji
    {   68,  720,       0,  104668,       0,  0, 25,  0, }, // French_Guiana
    {   68,  734,       0,  104693,       0,  0, 37,  0, }, // French_Southern
    {   68,  757,       0,  104730,       0,  0, 17,  0, }, // Galapagos
    {   68,  767,       0,  104747,       0,  0, 16,  0, }, // Gambier
    {   68,  775,  104763,  104778,  104804, 15, 26, 30, }, // Georgia
    {   68,  783,       0,  104834,       0,  0, 22,  0, }, // Gilbert_Islands
    {   68,  799,       0,  104856,       0,  0, 23,  0, }, // GMT
    {   68,  841,  104879,  104903,  104938, 24, 35, 39, }, // Greenland_Eastern
    {   68,  859,  104977,  105001,  105036, 24, 35, 39, }, // Greenland_Western
    {   68,  877,       0,  105075,       0,  0, 21,  0, }, // Guam
    {   68,  882,       0,  105096,       0,  0, 16,  0, }, // Gulf
    {   68,  887,       0,  104678,       0,  0, 15,  0, }, // Guyana
    {   68,  894,  105112,  105137,  105173, 25, 36, 40, }, // Hawaii_Aleutian
    {   68,  910,  105213,  105230,  105258, 17, 28, 33, }, // Hong_Kong
    {   68,  920,  105291,  105303,  105326, 12, 23, 27, }, // Hovd
    {   68,  925,       0,  105353,       0,  0, 27,  0, }, // India
    {   68,  931,       0,  105380,       0,  0, 25,  0, }, // Indian_Ocean
    {   68,  944,       0,  105405,       0,  0, 16,  0, }, // Indochina
    {   68,  954,       0,  105421,       0,  0, 30,  0, }, // Indonesia_Central
    {   68,  972,       0,  105451,       0,  0, 28,  0, }, // Indonesia_Eastern
    {   68,  990,       0,  105479,       0,  0, 28,  0, }, // Indonesia_Western
    {   68, 1008,  105507,  105519,  105542, 12, 23, 27, }, // Iran
    {   68, 1019,  105569,  105583,  105608, 14, 25, 29, }, // Irkutsk
    {   68, 1027,  105637,  105654,  105682, 17, 28, 32, }, // Israel
    {   68, 1034,  105714,  105730,  105757, 16, 27, 31, }, // Japan
    {   68, 1040,  105788,  105818,  105859, 30, 41, 46, }, // Kamchatka
    {   68, 1058,       0,  105905,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {   68, 1077,       0,  105933,       0,  0, 28,  0, }, // Kazakhstan_Western
    {   68, 1106,  105961,  105977,  106004, 16, 27, 31, }, // Korea
    {   68, 1112,       0,  106035,       0,  0, 13,  0, }, // Kosrae
    {   68, 1119,  106048,  106068,  106098, 20, 30, 34, }, // Krasnoyarsk
    {   68, 1151,       0,  106132,       0,  0, 17,  0, }, // Kyrgystan
    {   68, 1161,       0,  106149,       0,  0, 15,  0, }, // Lanka
    {   68, 1175,       0,  106164,       0,  0, 19,  0, }, // Line_Islands
    {   68, 1188,  106183,  106199,  106226, 16, 27, 31, }, // Lord_Howe
    {   68, 1198,  106257,  106268,  106290, 11, 22, 27, }, // Macau
    {   68, 1204,  106317,  106332,  106358, 15, 26, 29, }, // Magadan
    {   68, 1219,       0,  106387,       0,  0, 16,  0, }, // Malaysia
    {   68, 1228,       0,  106403,       0,  0, 15,  0, }, // Maldives
    {   68, 1237,       0,  106418,       0,  0, 14,  0, }, // Marquesas
    {   68, 1247,       0,  106432,       0,  0, 24,  0, }, // Marshall_Islands
    {   68, 1264,  106456,  106474,  106503, 18, 29, 33, }, // Mauritius
    {   68, 1274,       0,  106536,       0,  0, 13,  0, }, // Mawson
    {   68, 1281,  106549,  106575,  106612, 26, 37, 41, }, // Mexico_Pacific
    {   68, 1296,  106653,  106671,  106700, 18, 29, 33, }, // Mongolia
    {   68, 1305,  106733,  106747,  106772, 14, 25, 29, }, // Moscow
    {   68, 1312,       0,  106801,       0,  0, 16,  0, }, // Myanmar
    {   68, 1320,       0,  106817,       0,  0, 13,  0, }, // Nauru
    {   68, 1326,       0,  106830,       0,  0, 13,  0, }, // Nepal
    {   68, 1332,  106843,  106866,  106900, 23, 34, 38, }, // New_Caledonia
    {   68, 1346,  106938,  106957,  106987, 19, 30, 34, }, // New_Zealand
    {   68, 1358,  107021,  107040,  107070, 19, 30, 34, }, // Newfoundland
    {   68, 1371,       0,  107104,       0,  0, 12,  0, }, // Niue
    {   68, 1376,  107116,  107138,  107171, 22, 33, 37, }, // Norfolk
    {   68, 1384,  107208,  107234,  107271, 26, 37, 41, }, // Noronha
    {   68, 1392,       0,  107312,       0,  0, 31,  0, }, // North_Mariana
    {   68, 1406,  107343,  107363,  107393, 20, 30, 34, }, // Novosibirsk
    {   68, 1418,  107427,  107439,  107462, 12, 23, 27, }, // Omsk
    {   68, 1428,  107489,  107505,  107532, 16, 27, 31, }, // Pakistan
    {   68, 1437,       0,  107563,       0,  0, 13,  0, }, // Palau
    {   68, 1443,       0,  107576,       0,  0, 26,  0, }, // Papua_New_Guinea
    {   68, 1460,  107602,  107618,  107645, 16, 27, 31, }, // Paraguay
    {   68, 1469,  107676,  107688,  107711, 12, 23, 27, }, // Peru
    {   68, 1474,  107738,  107754,  107781, 16, 27, 31, }, // Philippines
    {   68, 1486,       0,  107812,       0,  0, 21,  0, }, // Phoenix_Islands
    {   68, 1502,  107833,  107859,  107896, 26, 37, 40, }, // Pierre_Miquelon
    {   68, 1518,       0,  107936,       0,  0, 15,  0, }, // Pitcairn
    {   68, 1527,       0,  107951,       0,  0, 14,  0, }, // Ponape
    {   68, 1534,       0,  107965,       0,  0, 17,  0, }, // Pyongyang
    {   68, 1544,  107982,  107997,  108023, 15, 26, 32, }, // Qyzylorda
    {   68, 1554,       0,  108055,       0,  0, 15,  0, }, // Reunion
    {   68, 1562,       0,  108070,       0,  0, 14,  0, }, // Rothera
    {   68, 1570,  108084,  108099,  108125, 15, 26, 30, }, // Sakhalin
    {   68, 1579,  108155,  108167,  108190, 12, 23, 29, }, // Samara
    {   68, 1596,  108219,  108232,  108256, 13, 24, 28, }, // Samoa
    {   68, 1602,       0,  108284,       0,  0, 15,  0, }, // Seychelles
    {   68, 1624,       0,  108299,       0,  0, 27,  0, }, // Singapore
    {   68, 1634,       0,  108326,       0,  0, 25,  0, }, // Solomon
    {   68, 1642,       0,  108351,       0,  0, 21,  0, }, // South_Georgia
    {   68, 1656,       0,  108372,       0,  0, 15,  0, }, // Suriname
    {   68, 1676,       0,  108387,       0,  0, 12,  0, }, // Syowa
    {   68, 1682,       0,  108399,       0,  0, 14,  0, }, // Tahiti
    {   68, 1689,  108413,  108427,  108452, 14, 25, 29, }, // Taipei
    {   68, 1696,       0,  108481,       0,  0, 18,  0, }, // Tajikistan
    {   68, 1724,       0,  108499,       0,  0, 15,  0, }, // Tokelau
    {   68, 1732,  108514,  108527,  108551, 13, 24, 28, }, // Tonga
    {   68, 1738,       0,  108579,       0,  0, 12,  0, }, // Truk
    {   68, 1750,  108591,  108611,  108642, 20, 31, 35, }, // Turkmenistan
    {   68, 1763,       0,  108677,       0,  0, 14,  0, }, // Tuvalu
    {   68, 1777,  108691,  108706,  108732, 15, 26, 30, }, // Uruguay
    {   68, 1792,  108762,  108780,  108809, 18, 29, 33, }, // Uzbekistan
    {   68, 1803,  108842,  108857,  108883, 15, 26, 30, }, // Vanuatu
    {   68, 1811,       0,  108913,       0,  0, 17,  0, }, // Venezuela
    {   68, 1821,  108930,  108949,  108979, 19, 30, 34, }, // Vladivostok
    {   68, 1833,  109013,  109030,  109058, 17, 28, 32, }, // Volgograd
    {   68, 1843,       0,  109090,       0,  0, 14,  0, }, // Vostok
    {   68, 1850,       0,  109104,       0,  0, 19,  0, }, // Wake
    {   68, 1855,       0,  109123,       0,  0, 29,  0, }, // Wallis
    {   68, 1862,  109152,  109167,  109193, 15, 26, 30, }, // Yakutsk
    {   68, 1870,  109223,  109244,  109276, 21, 32, 36, }, // Yekaterinburg
    {   69,    5,       0,  109312,       0,  0, 15,  0, }, // Afghanistan Breton/Latin/France
    {   69,   17,       0,  109327,       0,  0, 15,  0, }, // Africa_Central
    {   69,   32,       0,  109342,       0,  0, 19,  0, }, // Africa_Eastern
    {   69,   65,       0,  109361,       0,  0, 20,  0, }, // Africa_Southern
    {   69,   81,  109381,  109403,  109433, 22, 30, 27, }, // Africa_Western
    {   69,  107,  109460,  109470,  109488, 10, 18, 15, }, // Alaska
    {   69,  128,  109503,  109513,  109531, 10, 18, 15, }, // Almaty
    {   69,  135,  109546,  109559,  109580, 13, 21, 18, }, // Amazon
    {   69,  142,  109598,  109612,  109634, 14, 22, 19, }, // America_Central
    {   69,  158,  109653,  109665,  109685, 12, 20, 17, }, // America_Eastern
    {   69,  174,  109702,  109717,  109740, 15, 23, 20, }, // America_Mountain
    {   69,  191,  109760,  109773,  109794, 13, 21, 18, }, // America_Pacific
    {   69,  207,  109812,  109823,  109842, 11, 19, 16, }, // Anadyr
    {   69,  214,  109858,  109866,  109882,  8, 16, 13, }, // Apia
    {   69,  232,  109895,  109905,  109923, 10, 18, 15, }, // Arabian
    {   69,  240,  109938,  109953,  109976, 15, 23, 20, }, // Argentina
    {   69,  250,  109996,  110023,  110058, 27, 35, 32, }, // Argentina_Western
    {   69,  268,  110090,  110101,  110120, 11, 19, 16, }, // Armenia
    {   69,  286,  110136,  110151,  110174, 15, 23, 20, }, // Atlantic
    {   69,  295,  110194,  110212,  110238, 18, 26, 23, }, // Australia_Central
    {   69,  313,  110261,  110291,  110329, 30, 38, 35, }, // Australia_CentralWestern
    {   69,  338,  110364,  110386,  110416, 22, 30, 27, }, // Australia_Eastern
    {   69,  356,  110443,  110468,  110501, 25, 33, 30, }, // Australia_Western
    {   69,  374,  110531,  110546,  110569, 15, 23, 20, }, // Azerbaijan
    {   69,  385,  110589,  110602,  110623, 13, 21, 18, }, // Azores
    {   69,  397,  110641,  110655,  110677, 14, 22, 19, }, // Bangladesh
    {   69,  415,       0,  110696,       0,  0, 11,  0, }, // Bhutan
    {   69,  422,       0,  110707,       0,  0, 11,  0, }, // Bolivia
    {   69,  437,  110718,  110730,  110750, 12, 20, 17, }, // Brasilia
    {   69,  454,       0,  110767,       0,  0, 21,  0, }, // Brunei
    {   69,  461,  110788,  110805,  110830, 17, 25, 22, }, // Cape_Verde
    {   69,  478,       0,  110852,       0,  0, 12,  0, }, // Chamorro
    {   69,  487,  110864,  110875,  110894, 11, 19, 16, }, // Chatham
    {   69,  495,  110910,  110919,  110936,  9, 17, 14, }, // Chile
    {   69,  501,  110950,  110958,  110974,  8, 16, 13, }, // China
    {   69,  507,  110987,  111001,  111023, 14, 22, 19, }, // Choibalsan
    {   69,  518,       0,  111042,       0,  0, 18,  0, }, // Christmas
    {   69,  528,       0,  111060,       0,  0, 15,  0, }, // Cocos
    {   69,  534,  111075,  111087,  111107, 12, 20, 17, }, // Colombia
    {   69,  543,  111124,  111138,  111160, 14, 22, 19, }, // Cook
    {   69,  548,  111179,  111187,  111203,  8, 16, 13, }, // Cuba
    {   69,  559,       0,  111216,       0,  0,  9,  0, }, // Davis
    {   69,  575,       0,  111225,       0,  0, 20,  0, }, // DumontDUrville
    {   69,  612,       0,  111245,       0,  0, 18,  0, }, // East_Timor
    {   69,  623,  111263,  111276,  111297, 13, 21, 18, }, // Easter
    {   69,  630,       0,  111315,       0,  0, 11,  0, }, // Ecuador
    {   69,  638,  111326,  111341,  111364, 15, 23, 20, }, // Europe_Central
    {   69,  653,  111384,  111403,  111430, 19, 27, 24, }, // Europe_Eastern
    {   69,  668,       0,  111454,       0,  0, 15,  0, }, // Europe_Further_Eastern
    {   69,  691,  111469,  111491,  111521, 22, 30, 27, }, // Europe_Western
    {   69,  706,  111548,  111566,  111592, 18, 26, 23, }, // Falkland
    {   69,  715,  111615,  111624,  111641,  9, 17, 14, }, // Fiji
    {   69,  720,       0,  111655,       0,  0, 17,  0, }, // French_Guiana
    {   69,  734,       0,  111672,       0,  0, 40,  0, }, // French_Southern
    {   69,  757,       0,  111712,       0,  0, 19,  0, }, // Galapagos
    {   69,  767,       0,  111731,       0,  0, 17,  0, }, // Gambier
    {   69,  775,  111748,  111758,  111776, 10, 18, 15, }, // Georgia
    {   69,  783,       0,  111791,       0,  0, 17,  0, }, // Gilbert_Islands
    {   69,  799,       0,  111808,       0,  0, 28,  0, }, // GMT
    {   69,  841,  111836,  111858,  111888, 22, 30, 27, }, // Greenland_Eastern
    {   69,  859,  111915,  111940,  111973, 25, 33, 30, }, // Greenland_Western
    {   69,  877,       0,  112003,       0,  0, 16,  0, }, // Guam
    {   69,  882,       0,  112019,       0,  0, 36,  0, }, // Gulf
    {   69,  887,       0,  112055,       0,  0, 10,  0, }, // Guyana
    {   69,  894,  112065,  112091,  112125, 26, 34, 31, }, // Hawaii_Aleutian
    {   69,  910,  112156,  112169,  112190, 13, 21, 18, }, // Hong_Kong
    {   69,  920,  112208,  112216,  112232,  8, 16, 13, }, // Hovd
    {   69,  925,       0,  112245,       0,  0, 17,  0, }, // India
    {   69,  931,       0,  112262,       0,  0, 17,  0, }, // Indian_Ocean
    {   69,  944,       0,  112279,       0,  0, 14,  0, }, // Indochina
    {   69,  954,       0,  112293,       0,  0, 19,  0, }, // Indonesia_Central
    {   69,  972,       0,  112312,       0,  0, 22,  0, }, // Indonesia_Eastern
    {   69,  990,       0,  112334,       0,  0, 25,  0, }, // Indonesia_Western
    {   69, 1008,  112359,  112367,  112383,  8, 16, 13, }, // Iran
    {   69, 1019,  112396,  112407,  112426, 11, 19, 16, }, // Irkutsk
    {   69, 1027,  112442,  112452,  112470, 10, 18, 15, }, // Israel
    {   69, 1034,  112485,  112494,  112511,  9, 17, 14, }, // Japan
    {   69, 1058,       0,  112525,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {   69, 1077,       0,  112547,       0,  0, 25,  0, }, // Kazakhstan_Western
    {   69, 1106,  112572,  112581,  112598,  9, 17, 14, }, // Korea
    {   69, 1112,       0,  112612,       0,  0, 10,  0, }, // Kosrae
    {   69, 1119,  112622,  112637,  112660, 15, 23, 20, }, // Krasnoyarsk
    {   69, 1151,       0,  112680,       0,  0, 14,  0, }, // Kyrgystan
    {   69, 1161,       0,  112694,       0,  0, 13,  0, }, // Lanka
    {   69, 1175,       0,  112707,       0,  0, 16,  0, }, // Line_Islands
    {   69, 1188,  112723,  112736,  112757, 13, 21, 18, }, // Lord_Howe
    {   69, 1198,  112775,  112784,  112801,  9, 17, 14, }, // Macau
    {   69, 1204,  112815,  112826,  112845, 11, 19, 16, }, // Magadan
    {   69, 1219,       0,  112861,       0,  0, 12,  0, }, // Malaysia
    {   69, 1228,       0,  112873,       0,  0, 15,  0, }, // Maldives
    {   69, 1237,       0,  112888,       0,  0, 16,  0, }, // Marquesas
    {   69, 1247,       0,  112904,       0,  0, 18,  0, }, // Marshall_Islands
    {   69, 1264,  112922,  112931,  112948,  9, 17, 14, }, // Mauritius
    {   69, 1274,       0,  112962,       0,  0, 10,  0, }, // Mawson
    {   69, 1281,  112972,  112995,  113026, 23, 31, 28, }, // Mexico_Pacific
    {   69, 1296,  113054,  113069,  113092, 15, 23, 20, }, // Mongolia
    {   69, 1305,  113112,  113122,  113140, 10, 18, 15, }, // Moscow
    {   69, 1312,       0,  113155,       0,  0, 11,  0, }, // Myanmar
    {   69, 1320,       0,  113166,       0,  0,  9,  0, }, // Nauru
    {   69, 1326,       0,  113175,       0,  0,  9,  0, }, // Nepal
    {   69, 1332,  113184,  113203,  113230, 19, 27, 24, }, // New_Caledonia
    {   69, 1346,  113254,  113270,  113294, 16, 24, 21, }, // New_Zealand
    {   69, 1358,  113315,  113331,  113355, 16, 24, 21, }, // Newfoundland
    {   69, 1371,       0,  113376,       0,  0,  8,  0, }, // Niue
    {   69, 1376,  113384,  113400,  113424, 16, 24, 21, }, // Norfolk
    {   69, 1384,  113445,  113468,  113499, 23, 31, 28, }, // Noronha
    {   69, 1392,       0,  113527,       0,  0, 20,  0, }, // North_Mariana
    {   69, 1406,  113547,  113562,  113585, 15, 23, 20, }, // Novosibirsk
    {   69, 1418,  113605,  113613,  113629,  8, 16, 13, }, // Omsk
    {   69, 1428,  113642,  113654,  113674, 12, 20, 17, }, // Pakistan
    {   69, 1437,       0,  113691,       0,  0,  9,  0, }, // Palau
    {   69, 1443,       0,  113700,       0,  0, 22,  0, }, // Papua_New_Guinea
    {   69, 1460,  113722,  113734,  113754, 12, 20, 17, }, // Paraguay
    {   69, 1469,  113771,  113780,  113797,  9, 17, 14, }, // Peru
    {   69, 1474,  113811,  113827,  113851, 16, 24, 21, }, // Philippines
    {   69, 1486,       0,  113872,       0,  0, 17,  0, }, // Phoenix_Islands
    {   69, 1502,  113889,  113912,  113943, 23, 31, 28, }, // Pierre_Miquelon
    {   69, 1518,       0,  113971,       0,  0, 12,  0, }, // Pitcairn
    {   69, 1527,       0,  113983,       0,  0, 11,  0, }, // Ponape
    {   69, 1534,       0,  113994,       0,  0, 14,  0, }, // Pyongyang
    {   69, 1554,       0,  114008,       0,  0, 14,  0, }, // Reunion
    {   69, 1562,       0,  114022,       0,  0, 11,  0, }, // Rothera
    {   69, 1570,  114033,  114045,  114065, 12, 20, 17, }, // Sakhalin
    {   69, 1596,  114082,  114091,  114108,  9, 17, 14, }, // Samoa
    {   69, 1602,       0,  114122,       0,  0, 12,  0, }, // Seychelles
    {   69, 1624,       0,  114134,       0,  0, 21,  0, }, // Singapore
    {   69, 1634,       0,  114155,       0,  0, 17,  0, }, // Solomon
    {   69, 1642,       0,  114172,       0,  0, 17,  0, }, // South_Georgia
    {   69, 1656,       0,  114189,       0,  0, 11,  0, }, // Suriname
    {   69, 1676,       0,  114200,       0,  0,  9,  0, }, // Syowa
    {   69, 1682,       0,  114209,       0,  0, 10,  0, }, // Tahiti
    {   69, 1689,  114219,  114229,  114247, 10, 18, 15, }, // Taipei
    {   69, 1696,       0,  114262,       0,  0, 15,  0, }, // Tajikistan
    {   69, 1724,       0,  114277,       0,  0, 11,  0, }, // Tokelau
    {   69, 1732,  114288,  114297,  114314,  9, 17, 14, }, // Tonga
    {   69, 1738,       0,  114328,       0,  0,  9,  0, }, // Truk
    {   69, 1750,  114337,  114353,  114377, 16, 24, 21, }, // Turkmenistan
    {   69, 1763,       0,  114398,       0,  0, 10,  0, }, // Tuvalu
    {   69, 1777,  114408,  114419,  114438, 11, 19, 16, }, // Uruguay
    {   69, 1792,  114454,  114469,  114492, 15, 23, 20, }, // Uzbekistan
    {   69, 1803,  114512,  114523,  114542, 11, 19, 16, }, // Vanuatu
    {   69, 1811,       0,  114558,       0,  0, 13,  0, }, // Venezuela
    {   69, 1821,  114571,  114586,  114609, 15, 23, 20, }, // Vladivostok
    {   69, 1833,  114629,  114642,  114663, 13, 21, 18, }, // Volgograd
    {   69, 1843,       0,  114681,       0,  0, 10,  0, }, // Vostok
    {   69, 1850,       0,  114691,       0,  0, 15,  0, }, // Wake
    {   69, 1855,       0,  114706,       0,  0, 20,  0, }, // Wallis
    {   69, 1862,  114726,  114737,  114756, 11, 19, 16, }, // Yakutsk
    {   69, 1870,  114772,  114790,  114816, 18, 26, 23, }, // Yekaterinburg
    {   70,    5,       0,  114839,       0,  0, 19,  0, }, // Afghanistan Bulgarian/Cyrillic/Bulgaria
    {   70,   17,       0,  114858,       0,  0, 25,  0, }, // Africa_Central
    {   70,   32,       0,  114883,       0,  0, 23,  0, }, // Africa_Eastern
    {   70,   65,       0,  114906,       0,  0, 20,  0, }, // Africa_Southern
    {   70,   81,  114926,  114949,  114983, 23, 34, 36, }, // Africa_Western
    {   70,  107,  115019,  115025,  115050,  6, 25, 27, }, // Alaska
    {   70,  135,  115077,  115092,  115118, 15, 26, 28, }, // Amazon
    {   70,  142,  115146,  115180,  115225, 34, 45, 47, }, // America_Central
    {   70,  158,  115272,  115304,  115347, 32, 43, 45, }, // America_Eastern
    {   70,  174,  115392,  115426,  115471, 34, 45, 47, }, // America_Mountain
    {   70,  191,  115518,  115555,  115603, 37, 48, 50, }, // America_Pacific
    {   70,  207,  101890,  115653,  115678, 12, 25, 27, }, // Anadyr
    {   70,  214,  115705,  115709,  115732,  4, 23, 25, }, // Apia
    {   70,  232,  115757,  115770,  115794, 13, 24, 26, }, // Arabian
    {   70,  240,  115820,  115837,  115865, 17, 28, 30, }, // Argentina
    {   70,  250,  115895,  115919,  115954, 24, 35, 37, }, // Argentina_Western
    {   70,  268,  115991,  116005,  116030, 14, 25, 27, }, // Armenia
    {   70,  286,  116057,  116094,  116142, 37, 48, 50, }, // Atlantic
    {   70,  295,  116192,  116219,  116257, 27, 38, 40, }, // Australia_Central
    {   70,  313,  116297,  116332,  116378, 35, 46, 48, }, // Australia_CentralWestern
    {   70,  338,  116426,  116451,  116487, 25, 36, 38, }, // Australia_Eastern
    {   70,  356,  116525,  116550,  116586, 25, 36, 38, }, // Australia_Western
    {   70,  374,  116624,  116644,  116675, 20, 31, 33, }, // Azerbaijan
    {   70,  385,  116708,  116723,  116757, 15, 34, 36, }, // Azores
    {   70,  397,  116793,  116810,  116838, 17, 28, 30, }, // Bangladesh
    {   70,  415,       0,  116868,       0,  0, 14,  0, }, // Bhutan
    {   70,  422,       0,  116882,       0,  0, 16,  0, }, // Bolivia
    {   70,  437,  116898,  116913,  116939, 15, 26, 28, }, // Brasilia
    {   70,  454,       0,  116967,       0,  0, 17,  0, }, // Brunei
    {   70,  461,  116984,  116994,  117023, 10, 29, 31, }, // Cape_Verde
    {   70,  478,       0,  117054,       0,  0, 14,  0, }, // Chamorro
    {   70,  487,  117068,  117082,  117107, 14, 25, 27, }, // Chatham
    {   70,  495,  117134,  117148,  117173, 14, 25, 27, }, // Chile
    {   70,  501,  117200,  117214,  117239, 14, 25, 27, }, // China
    {   70,  507,  117266,  117284,  117313, 18, 29, 31, }, // Choibalsan
    {   70,  518,       0,  117344,       0,  0, 16,  0, }, // Christmas
    {   70,  528,       0,  117360,       0,  0, 16,  0, }, // Cocos
    {   70,  534,  117376,  117393,  117421, 17, 28, 30, }, // Colombia
    {   70,  543,  117451,  117462,  117492, 11, 30, 32, }, // Cook
    {   70,  548,  117524,  117538,  117563, 14, 25, 27, }, // Cuba
    {   70,  559,       0,  117590,       0,  0,  6,  0, }, // Davis
    {   70,  575,       0,  117596,       0,  0, 12,  0, }, // DumontDUrville
    {   70,  612,       0,  117608,       0,  0, 21,  0, }, // East_Timor
    {   70,  623,  117629,  117647,  117684, 18, 37, 39, }, // Easter
    {   70,  630,       0,  117723,       0,  0, 16,  0, }, // Ecuador
    {   70,  638,  117739,  117764,  117800, 25, 36, 38, }, // Europe_Central
    {   70,  653,  117838,  117861,  117895, 23, 34, 36, }, // Europe_Eastern
    {   70,  668,       0,  117931,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {   70,  691,  117962,  117985,  118019, 23, 34, 29, }, // Europe_Western
    {   70,  706,  118048,  118067,  118105, 19, 38, 40, }, // Falkland
    {   70,  715,  118145,  118160,  118186, 15, 26, 28, }, // Fiji
    {   70,  720,       0,  118214,       0,  0, 14,  0, }, // French_Guiana
    {   70,  734,       0,  118228,       0,  0, 38,  0, }, // French_Southern
    {   70,  757,       0,  118266,       0,  0, 17,  0, }, // Galapagos
    {   70,  767,       0,  118283,       0,  0,  6,  0, }, // Gambier
    {   70,  775,  118289,  118304,  118330, 15, 26, 28, }, // Georgia
    {   70,  783,       0,  118358,       0,  0, 15,  0, }, // Gilbert_Islands
    {   70,  799,       0,  118373,       0,  0, 22,  0, }, // GMT
    {   70,  841,  118395,  118419,  118454, 24, 35, 37, }, // Greenland_Eastern
    {   70,  859,  118491,  118515,  118550, 24, 35, 37, }, // Greenland_Western
    {   70,  882,       0,  118587,       0,  0, 15,  0, }, // Gulf
    {   70,  887,       0,  118602,       0,  0,  5,  0, }, // Guyana
    {   70,  894,  118607,  118630,  118664, 23, 34, 36, }, // Hawaii_Aleutian
    {   70,  910,  118700,  118716,  118743, 16, 27, 29, }, // Hong_Kong
    {   70,  920,  118772,  118785,  118809, 13, 24, 26, }, // Hovd
    {   70,  925,       0,  118835,       0,  0, 14,  0, }, // India
    {   70,  931,       0,  118849,       0,  0, 14,  0, }, // Indian_Ocean
    {   70,  944,       0,  118863,       0,  0, 18,  0, }, // Indochina
    {   70,  954,       0,  118881,       0,  0, 27,  0, }, // Indonesia_Central
    {   70,  972,       0,  118908,       0,  0, 25,  0, }, // Indonesia_Eastern
    {   70,  990,       0,  118933,       0,  0, 25,  0, }, // Indonesia_Western
    {   70, 1008,  118958,  118971,  118995, 13, 24, 26, }, // Iran
    {   70, 1019,  119021,  119035,  119060, 14, 25, 27, }, // Irkutsk
    {   70, 1027,  119087,  119102,  119128, 15, 26, 28, }, // Israel
    {   70, 1034,  119156,  119169,  119193, 13, 24, 26, }, // Japan
    {   70, 1040,  119219,  119248,  119288, 29, 40, 44, }, // Kamchatka
    {   70, 1058,       0,  119332,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {   70, 1077,       0,  119357,       0,  0, 25,  0, }, // Kazakhstan_Western
    {   70, 1106,  119382,  119396,  119421, 14, 25, 27, }, // Korea
    {   70, 1112,       0,  119448,       0,  0,  6,  0, }, // Kosrae
    {   70, 1119,  119454,  119471,  119499, 17, 28, 30, }, // Krasnoyarsk
    {   70, 1151,       0,  119529,       0,  0, 19,  0, }, // Kyrgystan
    {   70, 1175,       0,  119548,       0,  0, 20,  0, }, // Line_Islands
    {   70, 1188,  119568,  119576,  119603,  8, 27, 29, }, // Lord_Howe
    {   70, 1204,  119632,  119648,  119675, 16, 27, 29, }, // Magadan
    {   70, 1219,       0,  119704,       0,  0, 17,  0, }, // Malaysia
    {   70, 1228,       0,  119721,       0,  0, 15,  0, }, // Maldives
    {   70, 1237,       0,  119736,       0,  0, 16,  0, }, // Marquesas
    {   70, 1247,       0,  119752,       0,  0, 17,  0, }, // Marshall_Islands
    {   70, 1264,  119769,  119777,  119804,  8, 27, 29, }, // Mauritius
    {   70, 1274,       0,  119833,       0,  0,  6,  0, }, // Mawson
    {   70, 1281,  119839,  119869,  119910, 30, 41, 43, }, // Mexico_Pacific
    {   70, 1296,  119953,  119971,  120000, 18, 29, 31, }, // Mongolia
    {   70, 1305,  120031,  120046,  120072, 15, 26, 28, }, // Moscow
    {   70, 1312,       0,  120100,       0,  0, 16,  0, }, // Myanmar
    {   70, 1320,       0,   11259,       0,  0,  5,  0, }, // Nauru
    {   70, 1326,       0,  120116,       0,  0, 14,  0, }, // Nepal
    {   70, 1332,  120130,  120150,  120181, 20, 31, 33, }, // New_Caledonia
    {   70, 1346,  120214,  120233,  120263, 19, 30, 32, }, // New_Zealand
    {   70, 1358,  120295,  120315,  120346, 20, 31, 33, }, // Newfoundland
    {   70, 1371,       0,   11482,       0,  0,  4,  0, }, // Niue
    {   70, 1376,  120379,  120395,  120422, 16, 27, 29, }, // Norfolk
    {   70, 1384,  120451,  120469,  120506, 18, 37, 39, }, // Noronha
    {   70, 1406,  120545,  120563,  120592, 18, 29, 31, }, // Novosibirsk
    {   70, 1418,  120623,  120634,  120656, 11, 22, 24, }, // Omsk
    {   70, 1428,  120680,  120697,  120725, 17, 28, 30, }, // Pakistan
    {   70, 1437,       0,   11830,       0,  0,  5,  0, }, // Palau
    {   70, 1443,       0,  120755,       0,  0, 17,  0, }, // Papua_New_Guinea
    {   70, 1460,  120772,  120789,  120817, 17, 28, 30, }, // Paraguay
    {   70, 1469,  120847,  120862,  120888, 15, 26, 28, }, // Peru
    {   70, 1474,  120916,  120932,  120959, 16, 27, 29, }, // Philippines
    {   70, 1486,       0,  120988,       0,  0, 14,  0, }, // Phoenix_Islands
    {   70, 1502,  121002,  121020,  121057, 18, 37, 39, }, // Pierre_Miquelon
    {   70, 1518,       0,   12119,       0,  0,  7,  0, }, // Pitcairn
    {   70, 1527,       0,  107951,       0,  0,  6,  0, }, // Ponape
    {   70, 1534,       0,  121096,       0,  0, 15,  0, }, // Pyongyang
    {   70, 1554,       0,  121111,       0,  0,  7,  0, }, // Reunion
    {   70, 1562,       0,   12210,       0,  0,  6,  0, }, // Rothera
    {   70, 1570,  121118,  121134,  121161, 16, 27, 29, }, // Sakhalin
    {   70, 1579,  108155,  121190,  121215, 12, 25, 27, }, // Samara
    {   70, 1596,  121242,  121257,  121283, 15, 26, 28, }, // Samoa
    {   70, 1602,       0,  121311,       0,  0,  7,  0, }, // Seychelles
    {   70, 1624,       0,  121318,       0,  0, 17,  0, }, // Singapore
    {   70, 1634,       0,  121335,       0,  0, 18,  0, }, // Solomon
    {   70, 1642,       0,  121353,       0,  0, 13,  0, }, // South_Georgia
    {   70, 1656,       0,  121366,       0,  0, 16,  0, }, // Suriname
    {   70, 1676,       0,  108387,       0,  0,  4,  0, }, // Syowa
    {   70, 1682,       0,  121382,       0,  0, 15,  0, }, // Tahiti
    {   70, 1689,  121397,  121402,  121426,  5, 24, 26, }, // Taipei
    {   70, 1696,       0,  121452,       0,  0, 20,  0, }, // Tajikistan
    {   70, 1724,       0,   12536,       0,  0,  7,  0, }, // Tokelau
    {   70, 1732,   12543,  121472,  121496,  5, 24, 26, }, // Tonga
    {   70, 1738,       0,  121522,       0,  0,  4,  0, }, // Truk
    {   70, 1750,  121526,  121547,  121579, 21, 32, 34, }, // Turkmenistan
    {   70, 1763,       0,   12666,       0,  0,  6,  0, }, // Tuvalu
    {   70, 1777,  121613,  121629,  121656, 16, 27, 29, }, // Uruguay
    {   70, 1792,  121685,  121704,  121734, 19, 30, 32, }, // Uzbekistan
    {   70, 1803,   12794,  121766,  121792,  7, 26, 28, }, // Vanuatu
    {   70, 1811,       0,  121820,       0,  0, 17,  0, }, // Venezuela
    {   70, 1821,  121837,  121857,  121888, 20, 31, 33, }, // Vladivostok
    {   70, 1833,  121921,  121939,  121968, 18, 29, 31, }, // Volgograd
    {   70, 1843,       0,  109090,       0,  0,  6,  0, }, // Vostok
    {   70, 1850,       0,  121999,       0,  0, 11,  0, }, // Wake
    {   70, 1855,       0,  122010,       0,  0, 14,  0, }, // Wallis
    {   70, 1862,  122024,  122037,  122063, 13, 26, 28, }, // Yakutsk
    {   70, 1870,  122091,  122112,  122144, 21, 32, 34, }, // Yekaterinburg
    {   70, 1892,       0,  122178,       0,  0,  4,  0, }, // Yukon
    {   71,    5,       0,  122182,       0,  0, 20,  0, }, // Afghanistan Burmese/Myanmar/Myanmar
    {   71,   17,       0,  122202,       0,  0, 17,  0, }, // Africa_Central
    {   71,   32,       0,  122219,       0,  0, 18,  0, }, // Africa_Eastern
    {   71,   65,       0,  122237,       0,  0, 18,  0, }, // Africa_Southern
    {   71,   81,  122255,  122274,  122298, 19, 24, 27, }, // Africa_Western
    {   71,  107,  122325,  122338,  122356, 13, 18, 26, }, // Alaska
    {   71,  135,  122382,  122395,  122413, 13, 18, 20, }, // Amazon
    {   71,  142,  122433,  122449,  122470, 16, 21, 29, }, // America_Central
    {   71,  158,  122499,  122516,  122538, 17, 22, 30, }, // America_Eastern
    {   71,  174,  122568,  122583,  122603, 15, 20, 28, }, // America_Mountain
    {   71,  191,  122631,  122644,  122662, 13, 18, 26, }, // America_Pacific
    {   71,  214,  122688,  122700,  122717, 12, 17, 20, }, // Apia
    {   71,  232,  122737,  122750,  122768, 13, 18, 21, }, // Arabian
    {   71,  240,  122789,  122808,  122832, 19, 24, 26, }, // Argentina
    {   71,  250,  122858,  122884,  122915, 26, 31, 34, }, // Argentina_Western
    {   71,  268,  122949,  122967,  122990, 18, 23, 26, }, // Armenia
    {   71,  286,  123016,  123033,  123055, 17, 22, 30, }, // Atlantic
    {   71,  295,  123085,  123111,  123142, 26, 31, 34, }, // Australia_Central
    {   71,  313,  123176,  123211,  123251, 35, 40, 43, }, // Australia_CentralWestern
    {   71,  338,  123294,  123314,  123339, 20, 25, 28, }, // Australia_Eastern
    {   71,  356,  123367,  123388,  123344, 21, 26, 23, }, // Australia_Western
    {   71,  374,  123414,  123433,  123457, 19, 24, 27, }, // Azerbaijan
    {   71,  385,  123484,  123502,  123525, 18, 23, 26, }, // Azores
    {   71,  397,  123551,  123572,  123598, 21, 26, 29, }, // Bangladesh
    {   71,  415,       0,  123627,       0,  0, 12,  0, }, // Bhutan
    {   71,  422,       0,  123639,       0,  0, 19,  0, }, // Bolivia
    {   71,  437,  123658,  123671,  123689, 13, 18, 21, }, // Brasilia
    {   71,  454,       0,  123710,       0,  0, 21,  0, }, // Brunei
    {   71,  461,  123731,  123746,  123766, 15, 20, 23, }, // Cape_Verde
    {   71,  478,       0,  123789,       0,  0, 16,  0, }, // Chamorro
    {   71,  487,  123805,  123818,  123836, 13, 18, 22, }, // Chatham
    {   71,  495,  123858,  123870,  123887, 12, 17, 20, }, // Chile
    {   71,  501,  123907,  123919,  123936, 12, 17, 20, }, // China
    {   71,  507,  123956,  123975,  123999, 19, 24, 27, }, // Choibalsan
    {   71,  518,       0,  124026,       0,  0, 21,  0, }, // Christmas
    {   71,  528,       0,  124047,       0,  0, 20,  0, }, // Cocos
    {   71,  534,  124067,  124083,  124104, 16, 21, 24, }, // Colombia
    {   71,  543,  124128,  124147,  124171, 19, 24, 27, }, // Cook
    {   71,  548,  124198,  124212,  124231, 14, 19, 27, }, // Cuba
    {   71,  559,       0,  124258,       0,  0, 13,  0, }, // Davis
    {   71,  575,       0,  124271,       0,  0, 26,  0, }, // DumontDUrville
    {   71,  612,       0,  124297,       0,  0, 17,  0, }, // East_Timor
    {   71,  623,  124314,  124332,  124355, 18, 23, 26, }, // Easter
    {   71,  630,       0,  124381,       0,  0, 15,  0, }, // Ecuador
    {   71,  638,  124396,  124418,  124445, 22, 27, 30, }, // Europe_Central
    {   71,  653,  124475,  124492,  124514, 17, 22, 25, }, // Europe_Eastern
    {   71,  668,       0,  124539,       0,  0, 25,  0, }, // Europe_Further_Eastern
    {   71,  691,  124564,  124582,  124605, 18, 23, 26, }, // Europe_Western
    {   71,  706,  124631,  124654,  124682, 23, 28, 31, }, // Falkland
    {   71,  715,  124713,  124725,  124742, 12, 17, 20, }, // Fiji
    {   71,  720,       0,  124762,       0,  0, 23,  0, }, // French_Guiana
    {   71,  734,       0,  124785,       0,  0, 40,  0, }, // French_Southern
    {   71,  757,       0,  124825,       0,  0, 20,  0, }, // Galapagos
    {   71,  767,       0,  124845,       0,  0, 14,  0, }, // Gambier
    {   71,  775,  124859,  124876,  124898, 17, 22, 25, }, // Georgia
    {   71,  783,       0,  124923,       0,  0, 22,  0, }, // Gilbert_Islands
    {   71,  799,       0,  124945,       0,  0, 20,  0, }, // GMT
    {   71,  841,  124965,  124986,  125012, 21, 26, 33, }, // Greenland_Eastern
    {   71,  859,  125045,  125068,  125096, 23, 28, 35, }, // Greenland_Western
    {   71,  882,       0,  125131,       0,  0, 17,  0, }, // Gulf
    {   71,  887,       0,  125148,       0,  0, 16,  0, }, // Guyana
    {   71,  894,  125164,  125192,  125225, 28, 33, 41, }, // Hawaii_Aleutian
    {   71,  910,  125266,  125283,  125305, 17, 22, 25, }, // Hong_Kong
    {   71,  920,  125330,  125343,  125361, 13, 18, 21, }, // Hovd
    {   71,  925,       0,  125382,       0,  0, 19,  0, }, // India
    {   71,  931,       0,  125401,       0,  0, 22,  0, }, // Indian_Ocean
    {   71,  944,       0,  125423,       0,  0, 23,  0, }, // Indochina
    {   71,  954,       0,  125446,       0,  0, 31,  0, }, // Indonesia_Central
    {   71,  972,       0,  125477,       0,  0, 32,  0, }, // Indonesia_Eastern
    {   71,  990,       0,  125509,       0,  0, 33,  0, }, // Indonesia_Western
    {   71, 1008,  125542,  125554,  125571, 12, 17, 20, }, // Iran
    {   71, 1019,  125591,  125606,  125626, 15, 20, 23, }, // Irkutsk
    {   71, 1027,  125649,  125663,  125682, 14, 19, 22, }, // Israel
    {   71, 1034,  125704,  125716,  125733, 12, 17, 20, }, // Japan
    {   71, 1058,       0,  125753,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {   71, 1077,       0,  125774,       0,  0, 22,  0, }, // Kazakhstan_Western
    {   71, 1106,  125796,  125812,  125833, 16, 21, 24, }, // Korea
    {   71, 1112,       0,  125857,       0,  0, 17,  0, }, // Kosrae
    {   71, 1119,  125874,  125894,  125919, 20, 25, 28, }, // Krasnoyarsk
    {   71, 1151,       0,  125947,       0,  0, 17,  0, }, // Kyrgystan
    {   71, 1161,       0,  125964,       0,  0, 17,  0, }, // Lanka
    {   71, 1175,       0,  125981,       0,  0, 21,  0, }, // Line_Islands
    {   71, 1188,  126002,  126020,  126043, 18, 23, 26, }, // Lord_Howe
    {   71, 1198,  126069,  126082,  126100, 13, 18, 21, }, // Macau
    {   71, 1204,  126121,  126136,  126156, 15, 20, 23, }, // Magadan
    {   71, 1219,       0,  126179,       0,  0, 15,  0, }, // Malaysia
    {   71, 1228,       0,  126194,       0,  0, 17,  0, }, // Maldives
    {   71, 1237,       0,  126211,       0,  0, 17,  0, }, // Marquesas
    {   71, 1247,       0,  126228,       0,  0, 21,  0, }, // Marshall_Islands
    {   71, 1264,  126249,  126264,  126284, 15, 20, 23, }, // Mauritius
    {   71, 1274,       0,  126307,       0,  0, 15,  0, }, // Mawson
    {   71, 1281,  126322,  126346,  126375, 24, 29, 37, }, // Mexico_Pacific
    {   71, 1296,  126412,  126428,  126449, 16, 21, 24, }, // Mongolia
    {   71, 1305,  126473,  126488,  126508, 15, 20, 23, }, // Moscow
    {   71, 1312,       0,  126531,       0,  0, 13,  0, }, // Myanmar
    {   71, 1320,       0,  126544,       0,  0, 13,  0, }, // Nauru
    {   71, 1326,       0,  126557,       0,  0, 12,  0, }, // Nepal
    {   71, 1332,  126569,  126595,  126625, 26, 30, 33, }, // New_Caledonia
    {   71, 1346,  126658,  126674,  126695, 16, 21, 24, }, // New_Zealand
    {   71, 1358,  126719,  126738,  126762, 19, 24, 32, }, // Newfoundland
    {   71, 1371,       0,  126794,       0,  0, 13,  0, }, // Niue
    {   71, 1376,  126807,  126829,  126856, 22, 27, 35, }, // Norfolk
    {   71, 1384,  126891,  126919,  126952, 28, 33, 36, }, // Noronha
    {   71, 1406,  126988,  127011,  127039, 23, 28, 31, }, // Novosibirsk
    {   71, 1418,  127070,  127086,  127107, 16, 21, 24, }, // Omsk
    {   71, 1428,  127131,  127147,  127168, 16, 21, 24, }, // Pakistan
    {   71, 1437,       0,  127192,       0,  0, 13,  0, }, // Palau
    {   71, 1443,       0,  127205,       0,  0, 21,  0, }, // Papua_New_Guinea
    {   71, 1460,  127226,  127241,  127261, 15, 20, 23, }, // Paraguay
    {   71, 1469,  127284,  127296,  127313, 12, 17, 20, }, // Peru
    {   71, 1474,  127333,  127350,  127372, 17, 22, 25, }, // Philippines
    {   71, 1486,       0,  127397,       0,  0, 21,  0, }, // Phoenix_Islands
    {   71, 1502,  127418,  127447,  127481, 29, 34, 44, }, // Pierre_Miquelon
    {   71, 1518,       0,  127525,       0,  0, 17,  0, }, // Pitcairn
    {   71, 1527,       0,  127542,       0,  0, 16,  0, }, // Ponape
    {   71, 1534,       0,  127558,       0,  0, 15,  0, }, // Pyongyang
    {   71, 1554,       0,  127573,       0,  0, 15,  0, }, // Reunion
    {   71, 1562,       0,  127588,       0,  0, 14,  0, }, // Rothera
    {   71, 1570,  127602,  127616,  127635, 14, 19, 22, }, // Sakhalin
    {   71, 1596,  127657,  127670,  127688, 13, 18, 23, }, // Samoa
    {   71, 1602,       0,  127711,       0,  0, 13,  0, }, // Seychelles
    {   71, 1624,       0,  127724,       0,  0, 15,  0, }, // Singapore
    {   71, 1634,       0,  127739,       0,  0, 24,  0, }, // Solomon
    {   71, 1642,       0,  127763,       0,  0, 22,  0, }, // South_Georgia
    {   71, 1656,       0,  127785,       0,  0, 15,  0, }, // Suriname
    {   71, 1676,       0,  127800,       0,  0, 13,  0, }, // Syowa
    {   71, 1682,       0,  127813,       0,  0, 12,  0, }, // Tahiti
    {   71, 1689,  127825,  127839,  127858, 14, 19, 22, }, // Taipei
    {   71, 1696,       0,  127880,       0,  0, 20,  0, }, // Tajikistan
    {   71, 1724,       0,  127900,       0,  0, 15,  0, }, // Tokelau
    {   71, 1732,  127915,  127928,  127946, 13, 18, 21, }, // Tonga
    {   71, 1738,       0,  127967,       0,  0, 12,  0, }, // Truk
    {   71, 1750,  127979,  128001,  128028, 22, 27, 30, }, // Turkmenistan
    {   71, 1763,       0,  128058,       0,  0, 14,  0, }, // Tuvalu
    {   71, 1777,  128072,  128086,  128105, 14, 19, 22, }, // Uruguay
    {   71, 1792,  128127,  128146,  128170, 19, 24, 27, }, // Uzbekistan
    {   71, 1803,  128197,  128211,  128230, 14, 19, 22, }, // Vanuatu
    {   71, 1811,       0,  128252,       0,  0, 18,  0, }, // Venezuela
    {   71, 1821,  128270,  128293,  128321, 23, 28, 31, }, // Vladivostok
    {   71, 1833,  128352,  128371,  128395, 19, 24, 25, }, // Volgograd
    {   71, 1843,       0,  128420,       0,  0, 17,  0, }, // Vostok
    {   71, 1850,       0,  128437,       0,  0, 19,  0, }, // Wake
    {   71, 1855,       0,  128456,       0,  0, 27,  0, }, // Wallis
    {   71, 1862,  128483,  128499,  128520, 16, 21, 24, }, // Yakutsk
    {   71, 1870,  128544,  128567,  128595, 23, 28, 30, }, // Yekaterinburg
    {   71, 1892,       0,  128625,       0,  0, 14,  0, }, // Yukon
    {   72,    0,  128639,  128643,  128649,  4,  6,  6, }, // Acre Cantonese/Traditional Han/Hong Kong
    {   72,    5,       0,  128655,       0,  0,  5,  0, }, // Afghanistan
    {   72,   17,       0,  128660,       0,  0,  4,  0, }, // Africa_Central
    {   72,   32,       0,  128664,       0,  0,  4,  0, }, // Africa_Eastern
    {   72,   65,       0,  128668,       0,  0,  6,  0, }, // Africa_Southern
    {   72,   81,  128674,  128678,  128684,  4,  6,  6, }, // Africa_Western
    {   72,  107,  128690,  128696,  128704,  6,  8,  8, }, // Alaska
    {   72,  128,  128712,  128718,  128726,  6,  8,  8, }, // Almaty
    {   72,  135,  128734,  128739,  128746,  5,  7,  7, }, // Amazon
    {   72,  142,  128753,  128757,  128763,  4,  6,  6, }, // America_Central
    {   72,  158,  128769,  128773,  128779,  4,  6,  6, }, // America_Eastern
    {   72,  174,  128785,  128789,  128795,  4,  6,  6, }, // America_Mountain
    {   72,  191,  128801,  128806,  128813,  5,  7,  7, }, // America_Pacific
    {   72,  207,  128820,  128826,  128834,  6,  8,  8, }, // Anadyr
    {   72,  214,  128842,  128847,  128854,  5,  7,  7, }, // Apia
    {   72,  219,  128861,  128866,  128873,  5,  7,  7, }, // Aqtau
    {   72,  225,  128880,  128886,  128894,  6,  8,  8, }, // Aqtobe
    {   72,  232,  128902,  128907,  128914,  5,  7,  7, }, // Arabian
    {   72,  240,  128921,  128926,  128933,  5,  7,  7, }, // Argentina
    {   72,  250,  128940,  128947,  128956,  7,  9,  9, }, // Argentina_Western
    {   72,  268,  128965,  128971,  128979,  6,  8,  8, }, // Armenia
    {   72,  286,  128987,  128992,  128999,  5,  7,  7, }, // Atlantic
    {   72,  295,  129006,  129012,  129020,  6,  8,  8, }, // Australia_Central
    {   72,  313,  129028,  129035,  129044,  7,  9,  9, }, // Australia_CentralWestern
    {   72,  338,  129053,  129059,  129067,  6,  8,  8, }, // Australia_Eastern
    {   72,  356,  129075,  129081,  129089,  6,  8,  8, }, // Australia_Western
    {   72,  374,  129097,  129103,  129111,  6,  8,  8, }, // Azerbaijan
    {   72,  385,  129119,  129126,  129135,  7,  9,  9, }, // Azores
    {   72,  397,  129144,  129149,  129156,  5,  7,  7, }, // Bangladesh
    {   72,  415,       0,  129163,       0,  0,  4,  0, }, // Bhutan
    {   72,  422,       0,  129167,       0,  0,  6,  0, }, // Bolivia
    {   72,  437,  129173,  129179,  129187,  6,  8,  8, }, // Brasilia
    {   72,  454,       0,  129195,       0,  0,  4,  0, }, // Brunei
    {   72,  461,  129199,  129204,  129211,  5,  7,  7, }, // Cape_Verde
    {   72,  472,       0,  129218,       0,  0,  5,  0, }, // Casey
    {   72,  478,       0,  129223,       0,  0,  5,  0, }, // Chamorro
    {   72,  487,  129228,  129234,  129242,  6,  8,  8, }, // Chatham
    {   72,  495,  129250,  129254,  129260,  4,  6,  6, }, // Chile
    {   72,  501,  129266,  129270,  129276,  4,  6,  6, }, // China
    {   72,  507,  129282,  129287,  129294,  5,  7,  7, }, // Choibalsan
    {   72,  518,       0,  129301,       0,  0,  5,  0, }, // Christmas
    {   72,  528,       0,  129306,       0,  0,  7,  0, }, // Cocos
    {   72,  534,  129313,  129319,  129327,  6,  8,  8, }, // Colombia
    {   72,  543,  129335,  129341,  129349,  6,  8,  9, }, // Cook
    {   72,  548,  129358,  129362,  129368,  4,  6,  6, }, // Cuba
    {   72,  559,       0,  129374,       0,  0,  5,  0, }, // Davis
    {   72,  575,       0,  129379,       0,  0,  7,  0, }, // DumontDUrville
    {   72,  612,       0,  129386,       0,  0,  5,  0, }, // East_Timor
    {   72,  623,  129391,  129397,  129405,  6,  8,  8, }, // Easter
    {   72,  630,       0,  129413,       0,  0,  5,  0, }, // Ecuador
    {   72,  638,  129418,  129422,  129428,  4,  6,  6, }, // Europe_Central
    {   72,  653,  129434,  129438,  129444,  4,  6,  6, }, // Europe_Eastern
    {   72,  668,       0,  129450,       0,  0,  6,  0, }, // Europe_Further_Eastern
    {   72,  691,  129456,  129460,  129466,  4,  6,  6, }, // Europe_Western
    {   72,  706,  129472,  129479,  129488,  7,  9,  9, }, // Falkland
    {   72,  715,  129497,  129501,  129507,  4,  6,  6, }, // Fiji
    {   72,  720,       0,  129513,       0,  0,  7,  0, }, // French_Guiana
    {   72,  734,       0,  129520,       0,  0,  9,  0, }, // French_Southern
    {   72,  757,       0,  129529,       0,  0,  8,  0, }, // Galapagos
    {   72,  767,       0,  129537,       0,  0,  7,  0, }, // Gambier
    {   72,  775,  129544,  129549,  129556,  5,  7,  7, }, // Georgia
    {   72,  783,       0,  129563,       0,  0,  8,  0, }, // Gilbert_Islands
    {   72,  799,       0,  129571,       0,  0,  8,  0, }, // GMT
    {   72,  841,  129579,  129586,  129595,  7,  9,  9, }, // Greenland_Eastern
    {   72,  859,  129604,  129611,  129620,  7,  9,  9, }, // Greenland_Western
    {   72,  877,       0,  129629,       0,  0,  6,  0, }, // Guam
    {   72,  882,       0,  129635,       0,  0,  9,  0, }, // Gulf
    {   72,  887,       0,  129644,       0,  0,  5,  0, }, // Guyana
    {   72,  894,  129649,  129658,  129669,  9, 11, 11, }, // Hawaii_Aleutian
    {   72,  910,  129680,  129684,  129690,  4,  6,  6, }, // Hong_Kong
    {   72,  920,  129696,  129701,  129708,  5,  7,  7, }, // Hovd
    {   72,  925,       0,  129715,       0,  0,  6,  0, }, // India
    {   72,  931,       0,  129721,       0,  0,  5,  0, }, // Indian_Ocean
    {   72,  944,       0,  129726,       0,  0,  6,  0, }, // Indochina
    {   72,  954,       0,  129732,       0,  0,  6,  0, }, // Indonesia_Central
    {   72,  972,       0,  129738,       0,  0,  6,  0, }, // Indonesia_Eastern
    {   72,  990,       0,  129744,       0,  0,  6,  0, }, // Indonesia_Western
    {   72, 1008,  129750,  129754,  129760,  4,  6,  6, }, // Iran
    {   72, 1019,  129766,  129773,  129782,  7,  9,  9, }, // Irkutsk
    {   72, 1027,  129791,  129796,  129803,  5,  7,  7, }, // Israel
    {   72, 1034,  129810,  129814,  129820,  4,  6,  6, }, // Japan
    {   72, 1040,  129826,  129837,  129850, 11, 13, 15, }, // Kamchatka
    {   72, 1058,       0,  129865,       0,  0,  6,  0, }, // Kazakhstan_Eastern
    {   72, 1077,       0,  129871,       0,  0,  6,  0, }, // Kazakhstan_Western
    {   72, 1106,  129877,  129881,  129887,  4,  6,  6, }, // Korea
    {   72, 1112,       0,  129893,       0,  0,  5,  0, }, // Kosrae
    {   72, 1119,  129898,  129908,  129920, 10, 12, 12, }, // Krasnoyarsk
    {   72, 1151,       0,  129932,       0,  0,  6,  0, }, // Kyrgystan
    {   72, 1161,       0,  129938,       0,  0,  4,  0, }, // Lanka
    {   72, 1175,       0,  129942,       0,  0,  6,  0, }, // Line_Islands
    {   72, 1188,  129948,  129954,  129962,  6,  8,  8, }, // Lord_Howe
    {   72, 1198,  129970,  129974,  129980,  4,  6,  6, }, // Macau
    {   72, 1204,  129986,  129991,  129998,  5,  7,  7, }, // Magadan
    {   72, 1219,       0,  130005,       0,  0,  6,  0, }, // Malaysia
    {   72, 1228,       0,  130011,       0,  0,  6,  0, }, // Maldives
    {   72, 1237,       0,  130017,       0,  0,  6,  0, }, // Marquesas
    {   72, 1247,       0,  130023,       0,  0,  7,  0, }, // Marshall_Islands
    {   72, 1264,  130030,  130036,  130044,  6,  8,  8, }, // Mauritius
    {   72, 1274,       0,  130052,       0,  0,  4,  0, }, // Mawson
    {   72, 1281,  130056,  130064,  130074,  8, 10, 10, }, // Mexico_Pacific
    {   72, 1296,  130084,  130090,  130098,  6,  8,  8, }, // Mongolia
    {   72, 1305,  130106,  130111,  130118,  5,  7,  7, }, // Moscow
    {   72, 1312,       0,  130125,       0,  0,  4,  0, }, // Myanmar
    {   72, 1320,       0,  130129,       0,  0,  4,  0, }, // Nauru
    {   72, 1326,       0,  130133,       0,  0,  5,  0, }, // Nepal
    {   72, 1332,  130138,  130146,  130156,  8, 10, 12, }, // New_Caledonia
    {   72, 1346,  130168,  130173,  130180,  5,  7,  7, }, // New_Zealand
    {   72, 1358,  130187,  130192,  130199,  5,  7,  7, }, // Newfoundland
    {   72, 1371,       0,  130206,       0,  0,  5,  0, }, // Niue
    {   72, 1376,  130211,  130217,  130225,  6,  8,  8, }, // Norfolk
    {   72, 1384,  130233,  130247,  130263, 14, 16, 16, }, // Noronha
    {   72, 1392,       0,  130279,       0,  0,  9,  0, }, // North_Mariana
    {   72, 1406,  130288,  130295,  130304,  7,  9,  9, }, // Novosibirsk
    {   72, 1418,  130313,  130319,  130327,  6,  8,  8, }, // Omsk
    {   72, 1428,  130335,  130341,  130349,  6,  8,  8, }, // Pakistan
    {   72, 1437,       0,  130357,       0,  0,  4,  0, }, // Palau
    {   72, 1443,       0,  130361,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   72, 1460,  130370,  130375,  130382,  5,  7,  7, }, // Paraguay
    {   72, 1469,  130389,  130393,  130399,  4,  6,  6, }, // Peru
    {   72, 1474,  130405,  130410,  130417,  5,  7,  7, }, // Philippines
    {   72, 1486,       0,  130424,       0,  0,  6,  0, }, // Phoenix_Islands
    {   72, 1502,  130430,  130442,  130456, 12, 14, 14, }, // Pierre_Miquelon
    {   72, 1518,       0,  130470,       0,  0,  5,  0, }, // Pitcairn
    {   72, 1527,       0,  130475,       0,  0,  5,  0, }, // Ponape
    {   72, 1534,       0,  130480,       0,  0,  4,  0, }, // Pyongyang
    {   72, 1544,  130484,  130492,  130502,  8, 10, 10, }, // Qyzylorda
    {   72, 1554,       0,  130512,       0,  0,  5,  0, }, // Reunion
    {   72, 1562,       0,  130517,       0,  0,  5,  0, }, // Rothera
    {   72, 1570,  130522,  130527,  130534,  5,  7,  7, }, // Sakhalin
    {   72, 1579,  130541,  130546,  130553,  5,  7,  7, }, // Samara
    {   72, 1596,  130560,  130565,  130572,  5,  7,  7, }, // Samoa
    {   72, 1602,       0,  130579,       0,  0,  5,  0, }, // Seychelles
    {   72, 1624,       0,  130584,       0,  0,  7,  0, }, // Singapore
    {   72, 1634,       0,  130591,       0,  0,  7,  0, }, // Solomon
    {   72, 1642,       0,  130598,       0,  0,  6,  0, }, // South_Georgia
    {   72, 1656,       0,  130604,       0,  0,  5,  0, }, // Suriname
    {   72, 1676,       0,  130609,       0,  0,  6,  0, }, // Syowa
    {   72, 1682,       0,  130615,       0,  0,  5,  0, }, // Tahiti
    {   72, 1689,  130620,  130624,  130630,  4,  6,  6, }, // Taipei
    {   72, 1696,       0,  130636,       0,  0,  5,  0, }, // Tajikistan
    {   72, 1724,       0,  130641,       0,  0,  7,  0, }, // Tokelau
    {   72, 1732,  130648,  130652,  130658,  4,  6,  6, }, // Tonga
    {   72, 1738,       0,  130664,       0,  0,  5,  0, }, // Truk
    {   72, 1750,  130669,  130674,  130681,  5,  7,  7, }, // Turkmenistan
    {   72, 1763,       0,  130688,       0,  0,  5,  0, }, // Tuvalu
    {   72, 1777,  130693,  130698,  130705,  5,  7,  7, }, // Uruguay
    {   72, 1792,  130712,  130718,  130726,  6,  8,  8, }, // Uzbekistan
    {   72, 1803,  130734,  130739,  130746,  5,  7,  7, }, // Vanuatu
    {   72, 1811,       0,  130753,       0,  0,  6,  0, }, // Venezuela
    {   72, 1821,  130759,  130764,  130771,  5,  7,  7, }, // Vladivostok
    {   72, 1833,  130778,  130785,  130794,  7,  9,  9, }, // Volgograd
    {   72, 1843,       0,  130803,       0,  0,  6,  0, }, // Vostok
    {   72, 1850,       0,  130809,       0,  0,  5,  0, }, // Wake
    {   72, 1855,       0,  130814,       0,  0, 11,  0, }, // Wallis
    {   72, 1862,  130825,  130831,  130839,  6,  8,  8, }, // Yakutsk
    {   72, 1870,  130847,  130854,  130863,  7,  9,  9, }, // Yekaterinburg
    {   72, 1892,       0,  130872,       0,  0,  4,  0, }, // Yukon
    {   73,    0,  130876,  130880,  130886,  4,  6,  6, }, // Acre Cantonese/Simplified Han/China
    {   73,    5,       0,  130892,       0,  0,  5,  0, }, // Afghanistan
    {   73,   17,       0,  130897,       0,  0,  4,  0, }, // Africa_Central
    {   73,   32,       0,  130901,       0,  0,  4,  0, }, // Africa_Eastern
    {   73,   65,       0,  130905,       0,  0,  6,  0, }, // Africa_Southern
    {   73,   81,  130911,  130915,  130921,  4,  6,  6, }, // Africa_Western
    {   73,  107,  130927,  130933,  130941,  6,  8,  8, }, // Alaska
    {   73,  128,  130949,  130955,  130963,  6,  8,  8, }, // Almaty
    {   73,  135,  130971,  130976,  130983,  5,  7,  7, }, // Amazon
    {   73,  142,  130990,  130994,  131000,  4,  6,  6, }, // America_Central
    {   73,  158,  131006,  131010,  131016,  4,  6,  6, }, // America_Eastern
    {   73,  174,  131022,  131026,  131032,  4,  6,  6, }, // America_Mountain
    {   73,  191,  131038,  131043,  131050,  5,  7,  7, }, // America_Pacific
    {   73,  207,  131057,  131063,  131071,  6,  8,  8, }, // Anadyr
    {   73,  214,  131079,  131084,  131091,  5,  7,  7, }, // Apia
    {   73,  219,  131098,  131103,  131110,  5,  7,  7, }, // Aqtau
    {   73,  225,  131117,  131123,  131131,  6,  8,  8, }, // Aqtobe
    {   73,  232,  131139,  131144,  131151,  5,  7,  7, }, // Arabian
    {   73,  240,  131158,  131163,  131170,  5,  7,  7, }, // Argentina
    {   73,  250,  131177,  131184,  131193,  7,  9,  9, }, // Argentina_Western
    {   73,  268,  131202,  131208,  131216,  6,  8,  8, }, // Armenia
    {   73,  286,  131224,  131229,  131236,  5,  7,  7, }, // Atlantic
    {   73,  295,  131243,  131249,  131257,  6,  8,  8, }, // Australia_Central
    {   73,  313,  131265,  131272,  131281,  7,  9,  9, }, // Australia_CentralWestern
    {   73,  338,  131290,  131296,  131304,  6,  8,  8, }, // Australia_Eastern
    {   73,  356,  131312,  131318,  131326,  6,  8,  8, }, // Australia_Western
    {   73,  374,  131334,  131340,  131348,  6,  8,  8, }, // Azerbaijan
    {   73,  385,  131356,  131363,  131372,  7,  9,  9, }, // Azores
    {   73,  397,  131381,  131386,  131393,  5,  7,  7, }, // Bangladesh
    {   73,  415,       0,  131400,       0,  0,  4,  0, }, // Bhutan
    {   73,  422,       0,  131404,       0,  0,  6,  0, }, // Bolivia
    {   73,  437,  131410,  131416,  131424,  6,  8,  8, }, // Brasilia
    {   73,  454,       0,  131432,       0,  0,  4,  0, }, // Brunei
    {   73,  461,  131436,  131441,  131448,  5,  7,  7, }, // Cape_Verde
    {   73,  472,       0,  131455,       0,  0,  5,  0, }, // Casey
    {   73,  478,       0,  131460,       0,  0,  5,  0, }, // Chamorro
    {   73,  487,  131465,  131471,  131479,  6,  8,  8, }, // Chatham
    {   73,  495,  131487,  131491,  131497,  4,  6,  6, }, // Chile
    {   73,  501,  131503,  131507,  131513,  4,  6,  6, }, // China
    {   73,  507,  131519,  131524,  131531,  5,  7,  7, }, // Choibalsan
    {   73,  518,       0,  131538,       0,  0,  5,  0, }, // Christmas
    {   73,  528,       0,  131543,       0,  0,  7,  0, }, // Cocos
    {   73,  534,  131550,  131556,  131564,  6,  8,  8, }, // Colombia
    {   73,  543,  131572,  131578,  131586,  6,  8,  9, }, // Cook
    {   73,  548,  131595,  131599,  131605,  4,  6,  6, }, // Cuba
    {   73,  559,       0,  131611,       0,  0,  5,  0, }, // Davis
    {   73,  575,       0,  131616,       0,  0,  7,  0, }, // DumontDUrville
    {   73,  612,       0,  131623,       0,  0,  5,  0, }, // East_Timor
    {   73,  623,  131628,  131634,  131642,  6,  8,  8, }, // Easter
    {   73,  630,       0,  131650,       0,  0,  5,  0, }, // Ecuador
    {   73,  638,  131655,  131659,  131665,  4,  6,  6, }, // Europe_Central
    {   73,  653,  131671,  131675,  131681,  4,  6,  6, }, // Europe_Eastern
    {   73,  668,       0,  131687,       0,  0,  6,  0, }, // Europe_Further_Eastern
    {   73,  691,  131693,  131697,  131703,  4,  6,  6, }, // Europe_Western
    {   73,  706,  131709,  131716,  131725,  7,  9,  9, }, // Falkland
    {   73,  715,  131734,  131738,  131744,  4,  6,  6, }, // Fiji
    {   73,  720,       0,  131750,       0,  0,  7,  0, }, // French_Guiana
    {   73,  734,       0,  131757,       0,  0,  9,  0, }, // French_Southern
    {   73,  757,       0,  131766,       0,  0,  8,  0, }, // Galapagos
    {   73,  767,       0,  131774,       0,  0,  7,  0, }, // Gambier
    {   73,  775,  131781,  131786,  131793,  5,  7,  7, }, // Georgia
    {   73,  783,       0,  131800,       0,  0,  8,  0, }, // Gilbert_Islands
    {   73,  799,       0,  131808,       0,  0,  8,  0, }, // GMT
    {   73,  841,  131816,  131823,  131832,  7,  9,  9, }, // Greenland_Eastern
    {   73,  859,  131841,  131848,  131857,  7,  9,  9, }, // Greenland_Western
    {   73,  877,       0,  131866,       0,  0,  6,  0, }, // Guam
    {   73,  882,       0,  131872,       0,  0,  9,  0, }, // Gulf
    {   73,  887,       0,  131881,       0,  0,  5,  0, }, // Guyana
    {   73,  894,  131886,  131895,  131906,  9, 11, 11, }, // Hawaii_Aleutian
    {   73,  910,  131917,  131921,  131927,  4,  6,  6, }, // Hong_Kong
    {   73,  920,  131933,  131938,  131945,  5,  7,  7, }, // Hovd
    {   73,  925,       0,  131952,       0,  0,  6,  0, }, // India
    {   73,  931,       0,  131958,       0,  0,  5,  0, }, // Indian_Ocean
    {   73,  944,       0,  131963,       0,  0,  6,  0, }, // Indochina
    {   73,  954,       0,  131969,       0,  0,  6,  0, }, // Indonesia_Central
    {   73,  972,       0,  131975,       0,  0,  6,  0, }, // Indonesia_Eastern
    {   73,  990,       0,  131981,       0,  0,  6,  0, }, // Indonesia_Western
    {   73, 1008,  131987,  131991,  131997,  4,  6,  6, }, // Iran
    {   73, 1019,  132003,  132010,  132019,  7,  9,  9, }, // Irkutsk
    {   73, 1027,  132028,  132033,  132040,  5,  7,  7, }, // Israel
    {   73, 1034,  132047,  132051,  132057,  4,  6,  6, }, // Japan
    {   73, 1040,  132063,  132074,  132087, 11, 13, 15, }, // Kamchatka
    {   73, 1058,       0,  132102,       0,  0,  6,  0, }, // Kazakhstan_Eastern
    {   73, 1077,       0,  132108,       0,  0,  6,  0, }, // Kazakhstan_Western
    {   73, 1106,  132114,  132118,  132124,  4,  6,  6, }, // Korea
    {   73, 1112,       0,  132130,       0,  0,  5,  0, }, // Kosrae
    {   73, 1119,  132135,  132145,  132157, 10, 12, 12, }, // Krasnoyarsk
    {   73, 1151,       0,  132169,       0,  0,  6,  0, }, // Kyrgystan
    {   73, 1161,       0,  132175,       0,  0,  4,  0, }, // Lanka
    {   73, 1175,       0,  132179,       0,  0,  6,  0, }, // Line_Islands
    {   73, 1188,  132185,  132191,  132199,  6,  8,  8, }, // Lord_Howe
    {   73, 1198,  132207,  132211,  132217,  4,  6,  6, }, // Macau
    {   73, 1204,  132223,  132228,  132235,  5,  7,  7, }, // Magadan
    {   73, 1219,       0,  132242,       0,  0,  6,  0, }, // Malaysia
    {   73, 1228,       0,  132248,       0,  0,  6,  0, }, // Maldives
    {   73, 1237,       0,  132254,       0,  0,  6,  0, }, // Marquesas
    {   73, 1247,       0,  132260,       0,  0,  7,  0, }, // Marshall_Islands
    {   73, 1264,  132267,  132273,  132281,  6,  8,  8, }, // Mauritius
    {   73, 1274,       0,  132289,       0,  0,  4,  0, }, // Mawson
    {   73, 1281,  132293,  132301,  132311,  8, 10, 10, }, // Mexico_Pacific
    {   73, 1296,  132321,  132327,  132335,  6,  8,  8, }, // Mongolia
    {   73, 1305,  132343,  132348,  132355,  5,  7,  7, }, // Moscow
    {   73, 1312,       0,  132362,       0,  0,  4,  0, }, // Myanmar
    {   73, 1320,       0,  132366,       0,  0,  4,  0, }, // Nauru
    {   73, 1326,       0,  132370,       0,  0,  5,  0, }, // Nepal
    {   73, 1332,  132375,  132383,  132393,  8, 10, 12, }, // New_Caledonia
    {   73, 1346,  132405,  132410,  132417,  5,  7,  7, }, // New_Zealand
    {   73, 1358,  132424,  132429,  132436,  5,  7,  7, }, // Newfoundland
    {   73, 1371,       0,  132443,       0,  0,  5,  0, }, // Niue
    {   73, 1376,  132448,  132454,  132462,  6,  8,  8, }, // Norfolk
    {   73, 1384,  132470,  132484,  132500, 14, 16, 16, }, // Noronha
    {   73, 1392,       0,  132516,       0,  0,  9,  0, }, // North_Mariana
    {   73, 1406,  132525,  132532,  132541,  7,  9,  9, }, // Novosibirsk
    {   73, 1418,  132550,  132556,  132564,  6,  8,  8, }, // Omsk
    {   73, 1428,  132572,  132578,  132586,  6,  8,  8, }, // Pakistan
    {   73, 1437,       0,  132594,       0,  0,  4,  0, }, // Palau
    {   73, 1443,       0,  132598,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   73, 1460,  132607,  132612,  132619,  5,  7,  7, }, // Paraguay
    {   73, 1469,  132626,  132630,  132636,  4,  6,  6, }, // Peru
    {   73, 1474,  132642,  132647,  132654,  5,  7,  7, }, // Philippines
    {   73, 1486,       0,  132661,       0,  0,  6,  0, }, // Phoenix_Islands
    {   73, 1502,  132667,  132679,  132693, 12, 14, 14, }, // Pierre_Miquelon
    {   73, 1518,       0,  132707,       0,  0,  5,  0, }, // Pitcairn
    {   73, 1527,       0,  132712,       0,  0,  5,  0, }, // Ponape
    {   73, 1534,       0,  132717,       0,  0,  4,  0, }, // Pyongyang
    {   73, 1544,  132721,  132729,  132739,  8, 10, 10, }, // Qyzylorda
    {   73, 1554,       0,  132749,       0,  0,  5,  0, }, // Reunion
    {   73, 1562,       0,  132754,       0,  0,  5,  0, }, // Rothera
    {   73, 1570,  132759,  132764,  132771,  5,  7,  7, }, // Sakhalin
    {   73, 1579,  132778,  132783,  132790,  5,  7,  7, }, // Samara
    {   73, 1596,  132797,  132802,  132809,  5,  7,  7, }, // Samoa
    {   73, 1602,       0,  132816,       0,  0,  5,  0, }, // Seychelles
    {   73, 1624,       0,  132821,       0,  0,  7,  0, }, // Singapore
    {   73, 1634,       0,  132828,       0,  0,  7,  0, }, // Solomon
    {   73, 1642,       0,  132835,       0,  0,  6,  0, }, // South_Georgia
    {   73, 1656,       0,  132841,       0,  0,  5,  0, }, // Suriname
    {   73, 1676,       0,  132846,       0,  0,  6,  0, }, // Syowa
    {   73, 1682,       0,  132852,       0,  0,  5,  0, }, // Tahiti
    {   73, 1689,  132857,  132861,  132867,  4,  6,  6, }, // Taipei
    {   73, 1696,       0,  132873,       0,  0,  5,  0, }, // Tajikistan
    {   73, 1724,       0,  132878,       0,  0,  7,  0, }, // Tokelau
    {   73, 1732,  132885,  132889,  132895,  4,  6,  6, }, // Tonga
    {   73, 1738,       0,  132901,       0,  0,  5,  0, }, // Truk
    {   73, 1750,  132906,  132911,  132918,  5,  7,  7, }, // Turkmenistan
    {   73, 1763,       0,  132925,       0,  0,  5,  0, }, // Tuvalu
    {   73, 1777,  132930,  132935,  132942,  5,  7,  7, }, // Uruguay
    {   73, 1792,  132949,  132955,  132963,  6,  8,  8, }, // Uzbekistan
    {   73, 1803,  132971,  132976,  132983,  5,  7,  7, }, // Vanuatu
    {   73, 1811,       0,  132990,       0,  0,  6,  0, }, // Venezuela
    {   73, 1821,  132996,  133001,  133008,  5,  7,  7, }, // Vladivostok
    {   73, 1833,  133015,  133022,  133031,  7,  9,  9, }, // Volgograd
    {   73, 1843,       0,  133040,       0,  0,  6,  0, }, // Vostok
    {   73, 1850,       0,  133046,       0,  0,  5,  0, }, // Wake
    {   73, 1855,       0,  133051,       0,  0, 11,  0, }, // Wallis
    {   73, 1862,  133062,  133068,  133076,  6,  8,  8, }, // Yakutsk
    {   73, 1870,  133084,  133091,  133100,  7,  9,  9, }, // Yekaterinburg
    {   73, 1892,       0,  133109,       0,  0,  4,  0, }, // Yukon
    {   74,    5,       0,  133113,       0,  0, 20,  0, }, // Afghanistan Catalan/Latin/Spain
    {   74,   17,       0,  133133,       0,  0, 24,  0, }, // Africa_Central
    {   74,   32,       0,  133157,       0,  0, 25,  0, }, // Africa_Eastern
    {   74,   65,       0,  133182,       0,  0, 34,  0, }, // Africa_Southern
    {   74,   81,  133216,  133243,  133280, 27, 37, 35, }, // Africa_Western
    {   74,  107,   52265,  133315,  133338, 13, 23, 21, }, // Alaska
    {   74,  135,  133359,  133377,  133405, 18, 28, 26, }, // Amazon
    {   74,  142,  133431,  133462,  133503, 31, 41, 39, }, // America_Central
    {   74,  158,  133542,  133574,  133616, 32, 42, 40, }, // America_Eastern
    {   74,  174,  133656,  133691,  133736, 35, 45, 43, }, // America_Mountain
    {   74,  191,  133779,  133814,  133859, 35, 45, 43, }, // America_Pacific
    {   74,  207,  133902,  133915,  133938, 13, 23, 23, }, // Anadyr
    {   74,  214,   52946,  133961,  133982, 11, 21, 19, }, // Apia
    {   74,  232,  134001,  134010,  134029,  9, 19, 17, }, // Arabian
    {   74,  240,  134046,  134065,  134094, 19, 29, 27, }, // Argentina
    {   74,  250,  134121,  134150,  134189, 29, 39, 37, }, // Argentina_Western
    {   74,  268,  134226,  134240,  134264, 14, 24, 22, }, // Armenia
    {   74,  286,  134286,  134304,  134332, 18, 28, 26, }, // Atlantic
    {   74,  295,  134358,  134382,  134416, 24, 34, 32, }, // Australia_Central
    {   74,  313,  134448,  134482,  134526, 34, 44, 42, }, // Australia_CentralWestern
    {   74,  338,  134568,  134593,  134628, 25, 35, 33, }, // Australia_Eastern
    {   74,  356,  134661,  134688,  134725, 27, 37, 35, }, // Australia_Western
    {   74,  374,  134760,  134781,  134812, 21, 31, 29, }, // Azerbaijan
    {   74,  385,  134841,  134859,  134887, 18, 28, 26, }, // Azores
    {   74,  397,  134913,  134931,  134959, 18, 28, 26, }, // Bangladesh
    {   74,  415,       0,  134985,       0,  0, 14,  0, }, // Bhutan
    {   74,  422,       0,  134999,       0,  0, 15,  0, }, // Bolivia
    {   74,  437,  135014,  135030,  135056, 16, 26, 24, }, // Brasilia
    {   74,  454,       0,  135080,       0,  0, 25,  0, }, // Brunei
    {   74,  461,  135105,  135121,  135147, 16, 26, 24, }, // Cape_Verde
    {   74,  478,       0,  135171,       0,  0, 26,  0, }, // Chamorro
    {   74,  487,   54300,  135197,  135222, 15, 25, 23, }, // Chatham
    {   74,  495,  135245,  135257,  135279, 12, 22, 20, }, // Chile
    {   74,  501,  135299,  135314,  135339, 15, 25, 23, }, // China
    {   74,  507,   54477,  135362,  135390, 18, 28, 26, }, // Choibalsan
    {   74,  518,       0,  135416,       0,  0, 18,  0, }, // Christmas
    {   74,  528,       0,  135434,       0,  0, 23,  0, }, // Cocos
    {   74,  534,  135457,  135473,  135499, 16, 26, 24, }, // Colombia
    {   74,  543,  135523,  135545,  135577, 22, 32, 35, }, // Cook
    {   74,  548,   54767,  135612,  135634, 12, 22, 20, }, // Cuba
    {   74,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {   74,  575,       0,  135654,       0,  0, 24,  0, }, // DumontDUrville
    {   74,  612,       0,   54858,       0,  0, 22,  0, }, // East_Timor
    {   74,  623,  135678,  135702,  135736, 24, 34, 32, }, // Easter
    {   74,  630,       0,  135768,       0,  0, 17,  0, }, // Ecuador
    {   74,  638,  135785,  135809,  135843, 24, 34, 32, }, // Europe_Central
    {   74,  653,  135875,  135897,  135929, 22, 32, 30, }, // Europe_Eastern
    {   74,  668,       0,  135959,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {   74,  691,  135990,  136013,  136046, 23, 33, 31, }, // Europe_Western
    {   74,  706,  136077,  136103,  136139, 26, 36, 34, }, // Falkland
    {   74,  715,  136173,  136185,  136207, 12, 22, 20, }, // Fiji
    {   74,  720,       0,  136227,       0,  0, 27,  0, }, // French_Guiana
    {   74,  734,       0,  136254,       0,  0, 33,  0, }, // French_Southern
    {   74,  757,       0,   55488,       0,  0, 17,  0, }, // Galapagos
    {   74,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {   74,  775,  136287,  136302,  136327, 15, 25, 23, }, // Georgia
    {   74,  783,       0,  136350,       0,  0, 25,  0, }, // Gilbert_Islands
    {   74,  799,       0,  136375,       0,  0, 29,  0, }, // GMT
    {   74,  841,  136404,  136432,  136470, 28, 38, 36, }, // Greenland_Eastern
    {   74,  859,  136506,  136535,  136574, 29, 39, 37, }, // Greenland_Western
    {   74,  882,       0,  136611,       0,  0, 23,  0, }, // Gulf
    {   74,  887,       0,  136634,       0,  0, 14,  0, }, // Guyana
    {   74,  894,   55883,  136648,  136683, 25, 35, 33, }, // Hawaii_Aleutian
    {   74,  910,  136716,  136733,  136760, 17, 27, 25, }, // Hong_Kong
    {   74,  920,  136785,  136798,  136821, 13, 23, 21, }, // Hovd
    {   74,  925,       0,  136842,       0,  0, 15,  0, }, // India
    {   74,  931,       0,  136857,       0,  0, 20,  0, }, // Indian_Ocean
    {   74,  944,       0,  136877,       0,  0, 18,  0, }, // Indochina
    {   74,  954,       0,  136895,       0,  0, 24,  0, }, // Indonesia_Central
    {   74,  972,       0,  136919,       0,  0, 25,  0, }, // Indonesia_Eastern
    {   74,  990,       0,  136944,       0,  0, 26,  0, }, // Indonesia_Western
    {   74, 1008,  136970,  136984,  137008, 14, 24, 22, }, // Iran
    {   74, 1019,   56288,  137030,  137054, 14, 24, 22, }, // Irkutsk
    {   74, 1027,   56348,  137076,  137099, 13, 23, 21, }, // Israel
    {   74, 1034,  137120,  137133,  137156, 13, 23, 21, }, // Japan
    {   74, 1040,  137177,  137194,  137238, 17, 44, 44, }, // Kamchatka
    {   74, 1058,       0,  137282,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {   74, 1077,       0,  137310,       0,  0, 29,  0, }, // Kazakhstan_Western
    {   74, 1106,   56632,  137339,  137362, 13, 23, 21, }, // Korea
    {   74, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {   74, 1119,  137383,  137402,  137431, 19, 29, 27, }, // Krasnoyarsk
    {   74, 1151,       0,  137458,       0,  0, 20,  0, }, // Kyrgystan
    {   74, 1175,       0,  137478,       0,  0, 22,  0, }, // Line_Islands
    {   74, 1188,   56833,  137500,  137527, 17, 27, 27, }, // Lord_Howe
    {   74, 1198,  137554,  137567,  137590, 13, 23, 21, }, // Macau
    {   74, 1204,  137611,  137626,  137651, 15, 25, 23, }, // Magadan
    {   74, 1219,       0,  137674,       0,  0, 16,  0, }, // Malaysia
    {   74, 1228,       0,  137690,       0,  0, 20,  0, }, // Maldives
    {   74, 1237,       0,   57057,       0,  0, 21,  0, }, // Marquesas
    {   74, 1247,       0,  137710,       0,  0, 26,  0, }, // Marshall_Islands
    {   74, 1264,   57105,  137736,  137761, 15, 25, 23, }, // Mauritius
    {   74, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {   74, 1281,  137784,  137809,  137844, 25, 35, 33, }, // Mexico_Pacific
    {   74, 1296,  137877,  137895,  137923, 18, 28, 26, }, // Mongolia
    {   74, 1305,  137949,  137963,  137987, 14, 24, 22, }, // Moscow
    {   74, 1312,       0,   57410,       0,  0, 15,  0, }, // Myanmar
    {   74, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {   74, 1326,       0,   57438,       0,  0, 14,  0, }, // Nepal
    {   74, 1332,  138009,  138031,  138063, 22, 32, 30, }, // New_Caledonia
    {   74, 1346,  138093,  138113,  138143, 20, 30, 28, }, // New_Zealand
    {   74, 1358,  138171,  138188,  138215, 17, 27, 25, }, // Newfoundland
    {   74, 1371,       0,   57698,       0,  0, 12,  0, }, // Niue
    {   74, 1376,  138240,  138262,  138294, 22, 32, 30, }, // Norfolk
    {   74, 1384,   57800,  138324,  138361, 27, 37, 35, }, // Noronha
    {   74, 1406,  138396,  138416,  138446, 20, 30, 28, }, // Novosibirsk
    {   74, 1418,   58011,  138474,  138495, 11, 21, 19, }, // Omsk
    {   74, 1428,  138514,  138531,  138558, 17, 27, 25, }, // Pakistan
    {   74, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {   74, 1443,       0,  138583,       0,  0, 25,  0, }, // Papua_New_Guinea
    {   74, 1460,  138608,  138625,  138652, 17, 27, 25, }, // Paraguay
    {   74, 1469,   58242,  138677,  138700, 13, 23, 21, }, // Peru
    {   74, 1474,  138721,  138742,  138773, 21, 31, 29, }, // Philippines
    {   74, 1486,       0,  138802,       0,  0, 25,  0, }, // Phoenix_Islands
    {   74, 1502,  138827,  138859,  138901, 32, 42, 40, }, // Pierre_Miquelon
    {   74, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {   74, 1527,       0,   58521,       0,  0, 14,  0, }, // Ponape
    {   74, 1534,       0,  138941,       0,  0, 17,  0, }, // Pyongyang
    {   74, 1554,       0,   58621,       0,  0, 14,  0, }, // Reunion
    {   74, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {   74, 1570,  138958,  138974,  139000, 16, 26, 24, }, // Sakhalin
    {   74, 1579,   58714,  139024,  139048, 14, 24, 22, }, // Samara
    {   74, 1596,   58774,  139070,  139093, 13, 23, 21, }, // Samoa
    {   74, 1602,       0,  139114,       0,  0, 22,  0, }, // Seychelles
    {   74, 1624,       0,  139136,       0,  0, 16,  0, }, // Singapore
    {   74, 1634,       0,  139152,       0,  0, 24,  0, }, // Solomon
    {   74, 1642,       0,  139176,       0,  0, 23,  0, }, // South_Georgia
    {   74, 1656,       0,  139199,       0,  0, 15,  0, }, // Suriname
    {   74, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {   74, 1682,       0,  139214,       0,  0, 14,  0, }, // Tahiti
    {   74, 1689,  139228,  139242,  139266, 14, 24, 22, }, // Taipei
    {   74, 1696,       0,  139288,       0,  0, 20,  0, }, // Tajikistan
    {   74, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {   74, 1732,   59062,  139308,  139331, 13, 23, 21, }, // Tonga
    {   74, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {   74, 1750,  139352,  139373,  139404, 21, 31, 29, }, // Turkmenistan
    {   74, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {   74, 1777,  139433,  139450,  139477, 17, 27, 25, }, // Uruguay
    {   74, 1792,  139502,  139522,  139552, 20, 30, 28, }, // Uzbekistan
    {   74, 1803,  139580,  139594,  139618, 14, 24, 22, }, // Vanuatu
    {   74, 1811,       0,  139640,       0,  0, 17,  0, }, // Venezuela
    {   74, 1821,   59451,  139657,  139686, 19, 29, 27, }, // Vladivostok
    {   74, 1833,  139713,  139730,  139757, 17, 27, 25, }, // Volgograd
    {   74, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {   74, 1850,       0,  139782,       0,  0, 22,  0, }, // Wake
    {   74, 1855,       0,  139804,       0,  0, 23,  0, }, // Wallis
    {   74, 1862,  139827,  139842,  139867, 15, 25, 23, }, // Yakutsk
    {   74, 1870,  139890,  139911,  139942, 21, 31, 29, }, // Yekaterinburg
    {   74, 1892,       0,  139971,       0,  0, 13,  0, }, // Yukon
    {   78,    5,       0,  139984,       0,  0, 19,  0, }, // Afghanistan Cebuano/Latin/Philippines
    {   78,   17,       0,  140003,       0,  0, 22,  0, }, // Africa_Central
    {   78,   32,       0,  140025,       0,  0, 19,  0, }, // Africa_Eastern
    {   78,   65,       0,  140044,       0,  0, 30,  0, }, // Africa_Southern
    {   78,   81,  140074,  140093,  140122, 19, 29, 31, }, // Africa_Western
    {   78,  107,  140153,  140167,  140191, 14, 24, 23, }, // Alaska
    {   78,  135,  140214,  140228,  140252, 14, 24, 26, }, // Amazon
    {   78,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {   78,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {   78,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {   78,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {   78,  214,  140278,  140290,  140312, 12, 22, 21, }, // Apia
    {   78,  232,  140333,  140347,  140371, 14, 24, 26, }, // Arabian
    {   78,  240,  140397,  140414,  140441, 17, 27, 29, }, // Argentina
    {   78,  250,  140470,  140495,  140530, 25, 35, 37, }, // Argentina_Western
    {   78,  268,  140567,  140582,  140607, 15, 25, 27, }, // Armenia
    {   78,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {   78,  295,  140634,  140659,  140694, 25, 35, 37, }, // Australia_Central
    {   78,  313,  140731,  140764,  140807, 33, 43, 45, }, // Australia_CentralWestern
    {   78,  338,  140852,  140877,  140912, 25, 35, 37, }, // Australia_Eastern
    {   78,  356,  140949,  140974,  141009, 25, 35, 37, }, // Australia_Western
    {   78,  374,  141046,  141064,  141092, 18, 28, 30, }, // Azerbaijan
    {   78,  385,  141122,  141136,  141160, 14, 24, 26, }, // Azores
    {   78,  397,  141186,  141204,  141232, 18, 28, 30, }, // Bangladesh
    {   78,  415,       0,  141262,       0,  0, 14,  0, }, // Bhutan
    {   78,  422,       0,  141276,       0,  0, 15,  0, }, // Bolivia
    {   78,  437,  141291,  141307,  141333, 16, 26, 28, }, // Brasilia
    {   78,  454,       0,  141361,       0,  0, 25,  0, }, // Brunei
    {   78,  461,  141386,  141404,  141432, 18, 28, 30, }, // Cape_Verde
    {   78,  478,       0,  141462,       0,  0, 26,  0, }, // Chamorro
    {   78,  487,  141488,  141503,  141528, 15, 25, 27, }, // Chatham
    {   78,  495,  141555,  141568,  141591, 13, 23, 25, }, // Chile
    {   78,  501,  141616,  141629,  141652, 13, 23, 25, }, // China
    {   78,  507,  141677,  141695,  141723, 18, 28, 30, }, // Choibalsan
    {   78,  518,       0,  141753,       0,  0, 24,  0, }, // Christmas
    {   78,  528,       0,  141777,       0,  0, 21,  0, }, // Cocos
    {   78,  534,  141798,  141814,  141840, 16, 26, 28, }, // Colombia
    {   78,  543,  141868,  141888,  141918, 20, 30, 42, }, // Cook
    {   78,  548,  141960,  141972,  141994, 12, 22, 24, }, // Cuba
    {   78,  559,       0,  142018,       0,  0, 13,  0, }, // Davis
    {   78,  575,       0,  142031,       0,  0, 24,  0, }, // DumontDUrville
    {   78,  612,       0,  142055,       0,  0, 18,  0, }, // East_Timor
    {   78,  623,  142073,  142094,  142125, 21, 31, 33, }, // Easter
    {   78,  630,       0,  142158,       0,  0, 15,  0, }, // Ecuador
    {   78,  638,  142173,  142195,  142227, 22, 32, 34, }, // Europe_Central
    {   78,  653,  142261,  142283,  142315, 22, 32, 34, }, // Europe_Eastern
    {   78,  668,       0,  142349,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {   78,  691,  142379,  142401,  142433, 22, 32, 34, }, // Europe_Western
    {   78,  706,  142467,  142491,  142525, 24, 34, 36, }, // Falkland
    {   78,  715,  142561,  142573,  142595, 12, 22, 24, }, // Fiji
    {   78,  720,       0,  142619,       0,  0, 21,  0, }, // French_Guiana
    {   78,  734,       0,  142640,       0,  0, 36,  0, }, // French_Southern
    {   78,  757,       0,  142676,       0,  0, 17,  0, }, // Galapagos
    {   78,  767,       0,  142693,       0,  0, 15,  0, }, // Gambier
    {   78,  775,  142708,  142723,  142748, 15, 25, 27, }, // Georgia
    {   78,  783,       0,  142775,       0,  0, 23,  0, }, // Gilbert_Islands
    {   78,  799,       0,  142798,       0,  0, 22,  0, }, // GMT
    {   78,  841,  142820,  142842,  142874, 22, 32, 34, }, // Greenland_Eastern
    {   78,  859,  142908,  142930,  142962, 22, 32, 34, }, // Greenland_Western
    {   78,  882,       0,  142996,       0,  0, 22,  0, }, // Gulf
    {   78,  887,       0,  143018,       0,  0, 14,  0, }, // Guyana
    {   78,  894,  143032,  143055,  143088, 23, 33, 35, }, // Hawaii_Aleutian
    {   78,  910,  143123,  143140,  143167, 17, 27, 29, }, // Hong_Kong
    {   78,  920,  143196,  143208,  143230, 12, 22, 24, }, // Hovd
    {   78,  925,       0,  143254,       0,  0, 23,  0, }, // India
    {   78,  931,       0,  143277,       0,  0, 20,  0, }, // Indian_Ocean
    {   78,  944,       0,  143297,       0,  0, 17,  0, }, // Indochina
    {   78,  954,       0,  143314,       0,  0, 25,  0, }, // Indonesia_Central
    {   78,  972,       0,  143339,       0,  0, 25,  0, }, // Indonesia_Eastern
    {   78,  990,       0,  143364,       0,  0, 25,  0, }, // Indonesia_Western
    {   78, 1008,  143389,  143401,  143423, 12, 22, 24, }, // Iran
    {   78, 1019,  143447,  143462,  143487, 15, 25, 27, }, // Irkutsk
    {   78, 1027,  143514,  143528,  143552, 14, 24, 26, }, // Israel
    {   78, 1034,  143578,  143591,  143614, 13, 23, 24, }, // Japan
    {   78, 1058,       0,  143638,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {   78, 1077,       0,  143661,       0,  0, 23,  0, }, // Kazakhstan_Western
    {   78, 1106,  143684,  143697,  143720, 13, 23, 25, }, // Korea
    {   78, 1112,       0,  143745,       0,  0, 14,  0, }, // Kosrae
    {   78, 1119,  143759,  143778,  143807, 19, 29, 31, }, // Krasnoyarsk
    {   78, 1151,       0,  143838,       0,  0, 18,  0, }, // Kyrgystan
    {   78, 1175,       0,  143856,       0,  0, 20,  0, }, // Line_Islands
    {   78, 1188,  143876,  143893,  143920, 17, 27, 29, }, // Lord_Howe
    {   78, 1204,  143949,  143964,  143989, 15, 25, 27, }, // Magadan
    {   78, 1219,       0,  144016,       0,  0, 16,  0, }, // Malaysia
    {   78, 1228,       0,  144032,       0,  0, 16,  0, }, // Maldives
    {   78, 1237,       0,  144048,       0,  0, 17,  0, }, // Marquesas
    {   78, 1247,       0,  144065,       0,  0, 24,  0, }, // Marshall_Islands
    {   78, 1264,  144089,  144106,  144133, 17, 27, 29, }, // Mauritius
    {   78, 1274,       0,  144162,       0,  0, 14,  0, }, // Mawson
    {   78, 1281,  144176,  144199,  144232, 23, 33, 35, }, // Mexico_Pacific
    {   78, 1296,  144267,  144286,  144315, 19, 29, 31, }, // Mongolia
    {   78, 1305,  144346,  144360,  144384, 14, 24, 26, }, // Moscow
    {   78, 1312,       0,  144410,       0,  0, 15,  0, }, // Myanmar
    {   78, 1320,       0,  144425,       0,  0, 13,  0, }, // Nauru
    {   78, 1326,       0,  144438,       0,  0, 13,  0, }, // Nepal
    {   78, 1332,  144451,  144472,  144503, 21, 31, 33, }, // New_Caledonia
    {   78, 1346,  144536,  144555,  144584, 19, 29, 31, }, // New_Zealand
    {   78, 1358,  144615,  144635,  144665, 20, 30, 32, }, // Newfoundland
    {   78, 1371,       0,  144697,       0,  0, 12,  0, }, // Niue
    {   78, 1376,  144709,  144731,  144763, 22, 32, 34, }, // Norfolk
    {   78, 1384,  144797,  144824,  144861, 27, 37, 39, }, // Noronha
    {   78, 1406,  144900,  144919,  144948, 19, 29, 31, }, // Novosibirsk
    {   78, 1418,  144979,  144991,  145013, 12, 22, 24, }, // Omsk
    {   78, 1428,  145037,  145053,  145079, 16, 26, 28, }, // Pakistan
    {   78, 1437,       0,  145107,       0,  0, 13,  0, }, // Palau
    {   78, 1443,       0,  145120,       0,  0, 24,  0, }, // Papua_New_Guinea
    {   78, 1460,  145144,  145160,  145186, 16, 26, 28, }, // Paraguay
    {   78, 1469,  145214,  145226,  145248, 12, 22, 24, }, // Peru
    {   78, 1474,  145272,  145289,  145316, 17, 27, 29, }, // Philippines
    {   78, 1486,       0,  145345,       0,  0, 23,  0, }, // Phoenix_Islands
    {   78, 1502,  145368,  145397,  145436, 29, 39, 40, }, // Pierre_Miquelon
    {   78, 1518,       0,  145476,       0,  0, 16,  0, }, // Pitcairn
    {   78, 1527,       0,  145492,       0,  0, 14,  0, }, // Ponape
    {   78, 1534,       0,  145506,       0,  0, 17,  0, }, // Pyongyang
    {   78, 1554,       0,  145523,       0,  0, 15,  0, }, // Reunion
    {   78, 1562,       0,  145538,       0,  0, 15,  0, }, // Rothera
    {   78, 1570,  145553,  145569,  145595, 16, 26, 28, }, // Sakhalin
    {   78, 1596,  145623,  145636,  145659, 13, 23, 22, }, // Samoa
    {   78, 1602,       0,  145681,       0,  0, 18,  0, }, // Seychelles
    {   78, 1624,       0,  145699,       0,  0, 27,  0, }, // Singapore
    {   78, 1634,       0,  145726,       0,  0, 23,  0, }, // Solomon
    {   78, 1642,       0,  145749,       0,  0, 21,  0, }, // South_Georgia
    {   78, 1656,       0,  145770,       0,  0, 16,  0, }, // Suriname
    {   78, 1676,       0,  145786,       0,  0, 13,  0, }, // Syowa
    {   78, 1682,       0,  145799,       0,  0, 14,  0, }, // Tahiti
    {   78, 1689,  145813,  145827,  145851, 14, 24, 26, }, // Taipei
    {   78, 1696,       0,  145877,       0,  0, 18,  0, }, // Tajikistan
    {   78, 1724,       0,  145895,       0,  0, 15,  0, }, // Tokelau
    {   78, 1732,  145910,  145923,  145946, 13, 23, 25, }, // Tonga
    {   78, 1738,       0,  145971,       0,  0, 13,  0, }, // Truk
    {   78, 1750,  145984,  146004,  146034, 20, 30, 32, }, // Turkmenistan
    {   78, 1763,       0,  146066,       0,  0, 14,  0, }, // Tuvalu
    {   78, 1777,  146080,  146095,  146120, 15, 25, 27, }, // Uruguay
    {   78, 1792,  146147,  146165,  146193, 18, 28, 30, }, // Uzbekistan
    {   78, 1803,  146223,  146238,  146263, 15, 25, 27, }, // Vanuatu
    {   78, 1811,       0,  146290,       0,  0, 17,  0, }, // Venezuela
    {   78, 1821,  146307,  146326,  146355, 19, 29, 31, }, // Vladivostok
    {   78, 1833,  146386,  146403,  146430, 17, 27, 29, }, // Volgograd
    {   78, 1843,       0,  146459,       0,  0, 14,  0, }, // Vostok
    {   78, 1850,       0,  146473,       0,  0, 19,  0, }, // Wake
    {   78, 1855,       0,  146492,       0,  0, 23,  0, }, // Wallis
    {   78, 1862,  146515,  146530,  146555, 15, 25, 27, }, // Yakutsk
    {   78, 1870,  146582,  146603,  146634, 21, 31, 33, }, // Yekaterinburg
    {   78, 1892,       0,  146667,       0,  0, 13,  0, }, // Yukon
    {   82,    0,  146680,  146705,  146741, 25, 36, 48, }, // Acre Chakma/Chakma/Bangladesh
    {   82,    5,       0,  146789,       0,  0, 35,  0, }, // Afghanistan
    {   82,   17,       0,  146824,       0,  0, 40,  0, }, // Africa_Central
    {   82,   32,       0,  146864,       0,  0, 38,  0, }, // Africa_Eastern
    {   82,   65,       0,  146902,       0,  0, 49,  0, }, // Africa_Southern
    {   82,   81,  146951,  146993,  147046, 42, 53, 65, }, // Africa_Western
    {   82,  107,  147111,  147134,  147168, 23, 34, 38, }, // Alaska
    {   82,  128,  147206,  147231,  147267, 25, 36, 44, }, // Almaty
    {   82,  135,  147311,  147340,  147380, 29, 40, 48, }, // Amazon
    {   82,  142,  147428,  147460,  147505, 32, 45, 49, }, // America_Central
    {   82,  158,  147554,  147588,  147637, 34, 49, 53, }, // America_Eastern
    {   82,  174,  147690,  147728,  147781, 38, 53, 51, }, // America_Mountain
    {   82,  191,  147832,  147897,  147973, 65, 76, 78, }, // America_Pacific
    {   82,  207,  148051,  148080,  148120, 29, 40, 52, }, // Anadyr
    {   82,  214,  148172,  148195,  148229, 23, 34, 36, }, // Apia
    {   82,  219,  148265,  148290,  148326, 25, 36, 48, }, // Aqtau
    {   82,  225,  148374,  148401,  148439, 27, 38, 50, }, // Aqtobe
    {   82,  232,  148489,  148512,  148546, 23, 34, 42, }, // Arabian
    {   82,  240,  148588,  148621,  148665, 33, 44, 56, }, // Argentina
    {   82,  250,  148721,  148771,  148836, 50, 65, 73, }, // Argentina_Western
    {   82,  268,  148909,  148938,  148978, 29, 40, 52, }, // Armenia
    {   82,  286,  149030,  149067,  149115, 37, 48, 56, }, // Atlantic
    {   82,  295,  149171,  149234,  149308, 63, 74, 82, }, // Australia_Central
    {   82,  338,  149390,  149440,  149501, 50, 61, 69, }, // Australia_Eastern
    {   82,  356,  149570,  149624,  149689, 54, 65, 73, }, // Australia_Western
    {   82,  374,  149762,  149793,  149835, 31, 42, 54, }, // Azerbaijan
    {   82,  385,  149889,  149918,  149958, 29, 40, 52, }, // Azores
    {   82,  397,  150010,  150037,  150075, 27, 38, 50, }, // Bangladesh
    {   82,  415,       0,  150125,       0,  0, 23,  0, }, // Bhutan
    {   82,  422,       0,  150148,       0,  0, 27,  0, }, // Bolivia
    {   82,  437,  150175,  150204,  150244, 29, 40, 52, }, // Brasilia
    {   82,  454,       0,  150296,       0,  0, 46,  0, }, // Brunei
    {   82,  461,  150342,  150374,  150417, 32, 43, 55, }, // Cape_Verde
    {   82,  478,       0,  150472,       0,  0, 34,  0, }, // Chamorro
    {   82,  487,  150506,  150531,  150567, 25, 36, 44, }, // Chatham
    {   82,  495,  150611,  150632,  150664, 21, 32, 44, }, // Chile
    {   82,  501,  150708,  150729,  150761, 21, 32, 40, }, // China
    {   82,  507,  150801,  150836,  150882, 35, 46, 58, }, // Choibalsan
    {   82,  518,       0,  150940,       0,  0, 50,  0, }, // Christmas
    {   82,  528,       0,  150990,       0,  0, 56,  0, }, // Cocos
    {   82,  534,  151046,  151077,  151119, 31, 42, 54, }, // Colombia
    {   82,  543,  151173,  151225,  151288, 52, 63, 92, }, // Cook
    {   82,  548,  151380,  151407,  151441, 27, 34, 42, }, // Cuba
    {   82,  559,       0,  151483,       0,  0, 25,  0, }, // Davis
    {   82,  575,       0,  151508,       0,  0, 55,  0, }, // DumontDUrville
    {   82,  612,       0,  151563,       0,  0, 38,  0, }, // East_Timor
    {   82,  623,  151601,  151647,  151704, 46, 57, 69, }, // Easter
    {   82,  630,       0,  151773,       0,  0, 33,  0, }, // Ecuador
    {   82,  638,  151806,  151854,  151913, 48, 59, 71, }, // Europe_Central
    {   82,  653,  151984,  152030,  152087, 46, 57, 69, }, // Europe_Eastern
    {   82,  668,       0,  152156,       0,  0, 71,  0, }, // Europe_Further_Eastern
    {   82,  691,  152227,  152275,  152336, 48, 61, 73, }, // Europe_Western
    {   82,  706,  152409,  152475,  152552, 66, 77, 89, }, // Falkland
    {   82,  715,  152641,  152662,  152694, 21, 32, 44, }, // Fiji
    {   82,  720,       0,  152738,       0,  0, 30,  0, }, // French_Guiana
    {   82,  734,       0,  152768,       0,  0, 66,  0, }, // French_Southern
    {   82,  757,       0,  152834,       0,  0, 27,  0, }, // Galapagos
    {   82,  767,       0,  152861,       0,  0, 33,  0, }, // Gambier
    {   82,  775,  152894,  152921,  152959, 27, 38, 50, }, // Georgia
    {   82,  783,       0,  153009,       0,  0, 54,  0, }, // Gilbert_Islands
    {   82,  799,       0,  153063,       0,  0, 34,  0, }, // GMT
    {   82,  841,  153097,  153149,  153212, 52, 63, 75, }, // Greenland_Eastern
    {   82,  859,  153287,  153343,  153410, 56, 67, 79, }, // Greenland_Western
    {   82,  877,       0,  153489,       0,  0, 30,  0, }, // Guam
    {   82,  882,       0,  153519,       0,  0, 46,  0, }, // Gulf
    {   82,  887,       0,  153565,       0,  0, 21,  0, }, // Guyana
    {   82,  894,  153586,  153634,  153685, 48, 51, 59, }, // Hawaii_Aleutian
    {   82,  910,  153744,  153770,  153807, 26, 37, 49, }, // Hong_Kong
    {   82,  920,  153856,  153881,  153917, 25, 36, 48, }, // Hovd
    {   82,  925,       0,  153965,       0,  0, 40,  0, }, // India
    {   82,  931,       0,  154005,       0,  0, 50,  0, }, // Indian_Ocean
    {   82,  944,       0,  154055,       0,  0, 33,  0, }, // Indochina
    {   82,  954,       0,  154088,       0,  0, 55,  0, }, // Indonesia_Central
    {   82,  972,       0,  154143,       0,  0, 48,  0, }, // Indonesia_Eastern
    {   82,  990,       0,  154191,       0,  0, 52,  0, }, // Indonesia_Western
    {   82, 1008,  154243,  154266,  154300, 23, 34, 42, }, // Iran
    {   82, 1019,  154342,  154379,  154427, 37, 48, 60, }, // Irkutsk
    {   82, 1027,  154487,  154516,  154558, 29, 42, 50, }, // Israel
    {   82, 1034,  154608,  154629,  154661, 21, 32, 40, }, // Japan
    {   82, 1040,  154701,  154781,  154868, 80, 87,103, }, // Kamchatka
    {   82, 1058,       0,  154971,       0,  0, 55,  0, }, // Kazakhstan_Eastern
    {   82, 1077,       0,  155026,       0,  0, 59,  0, }, // Kazakhstan_Western
    {   82, 1106,  155085,  155112,  155150, 27, 38, 46, }, // Korea
    {   82, 1112,       0,  155196,       0,  0, 29,  0, }, // Kosrae
    {   82, 1119,  155225,  155266,  155318, 41, 52, 64, }, // Krasnoyarsk
    {   82, 1151,       0,  155382,       0,  0, 35,  0, }, // Kyrgystan
    {   82, 1161,       0,  155417,       0,  0, 21,  0, }, // Lanka
    {   82, 1175,       0,  155438,       0,  0, 52,  0, }, // Line_Islands
    {   82, 1188,  155490,  155522,  155565, 32, 43, 51, }, // Lord_Howe
    {   82, 1198,  155616,  155637,  155665, 21, 28, 44, }, // Macau
    {   82, 1204,  155709,  155736,  155774, 27, 38, 50, }, // Magadan
    {   82, 1219,       0,  155824,       0,  0, 29,  0, }, // Malaysia
    {   82, 1228,       0,  155853,       0,  0, 27,  0, }, // Maldives
    {   82, 1237,       0,  155880,       0,  0, 29,  0, }, // Marquesas
    {   82, 1247,       0,  155909,       0,  0, 56,  0, }, // Marshall_Islands
    {   82, 1264,  155965,  155992,  156030, 27, 38, 50, }, // Mauritius
    {   82, 1274,       0,  156080,       0,  0, 29,  0, }, // Mawson
    {   82, 1281,  156109,  156180,  156262, 71, 82, 90, }, // Mexico_Pacific
    {   82, 1296,  156352,  156386,  156431, 34, 45, 57, }, // Mongolia
    {   82, 1305,  156488,  156513,  156549, 25, 36, 48, }, // Moscow
    {   82, 1312,       0,  156597,       0,  0, 27,  0, }, // Myanmar
    {   82, 1320,       0,  156624,       0,  0, 23,  0, }, // Nauru
    {   82, 1326,       0,  156647,       0,  0, 23,  0, }, // Nepal
    {   82, 1332,  156670,  156712,  156765, 42, 53, 65, }, // New_Caledonia
    {   82, 1346,  156830,  156869,  156919, 39, 50, 58, }, // New_Zealand
    {   82, 1358,  156977,  157022,  157078, 45, 56, 64, }, // Newfoundland
    {   82, 1371,       0,  157142,       0,  0, 25,  0, }, // Niue
    {   82, 1376,  157167,  157227,  157298, 60, 71, 79, }, // Norfolk
    {   82, 1384,  157377,  157425,  157484, 48, 59, 71, }, // Noronha
    {   82, 1392,       0,  157555,       0,  0, 73,  0, }, // North_Mariana
    {   82, 1406,  157628,  157669,  157721, 41, 52, 64, }, // Novosibirsk
    {   82, 1418,  157785,  157814,  157854, 29, 40, 52, }, // Omsk
    {   82, 1428,  157906,  157935,  157975, 29, 40, 52, }, // Pakistan
    {   82, 1437,       0,  158027,       0,  0, 21,  0, }, // Palau
    {   82, 1443,       0,  158048,       0,  0, 39,  0, }, // Papua_New_Guinea
    {   82, 1460,  158087,  158116,  158156, 29, 40, 52, }, // Paraguay
    {   82, 1469,  158208,  158229,  158261, 21, 32, 44, }, // Peru
    {   82, 1474,  158305,  158334,  158374, 29, 40, 52, }, // Philippines
    {   82, 1486,       0,  158426,       0,  0, 60,  0, }, // Phoenix_Islands
    {   82, 1502,  158486,  158546,  158617, 60, 71, 79, }, // Pierre_Miquelon
    {   82, 1518,       0,  158696,       0,  0, 35,  0, }, // Pitcairn
    {   82, 1527,       0,  158731,       0,  0, 23,  0, }, // Ponape
    {   82, 1534,       0,  158754,       0,  0, 32,  0, }, // Pyongyang
    {   82, 1544,  158786,  158817,  158855, 31, 38, 54, }, // Qyzylorda
    {   82, 1554,       0,  158909,       0,  0, 35,  0, }, // Reunion
    {   82, 1562,       0,  158944,       0,  0, 23,  0, }, // Rothera
    {   82, 1570,  158967,  158992,  159028, 25, 36, 48, }, // Sakhalin
    {   82, 1579,  159076,  159095,  159121, 19, 26, 42, }, // Samara
    {   82, 1596,  159163,  159184,  159216, 21, 32, 44, }, // Samoa
    {   82, 1602,       0,  159260,       0,  0, 29,  0, }, // Seychelles
    {   82, 1624,       0,  159289,       0,  0, 38,  0, }, // Singapore
    {   82, 1634,       0,  159327,       0,  0, 60,  0, }, // Solomon
    {   82, 1642,       0,  159387,       0,  0, 40,  0, }, // South_Georgia
    {   82, 1656,       0,  159427,       0,  0, 27,  0, }, // Suriname
    {   82, 1676,       0,  159454,       0,  0, 21,  0, }, // Syowa
    {   82, 1682,       0,  159475,       0,  0, 23,  0, }, // Tahiti
    {   82, 1689,  159498,  159523,  159559, 25, 36, 44, }, // Taipei
    {   82, 1696,       0,  159603,       0,  0, 29,  0, }, // Tajikistan
    {   82, 1724,       0,  159632,       0,  0, 27,  0, }, // Tokelau
    {   82, 1732,  159659,  159682,  159716, 23, 34, 46, }, // Tonga
    {   82, 1738,       0,  159762,       0,  0, 21,  0, }, // Truk
    {   82, 1750,  159783,  159826,  159880, 43, 54, 66, }, // Turkmenistan
    {   82, 1763,       0,  159946,       0,  0, 23,  0, }, // Tuvalu
    {   82, 1777,  159969,  159998,  160038, 29, 40, 52, }, // Uruguay
    {   82, 1792,  160090,  160125,  160171, 35, 46, 58, }, // Uzbekistan
    {   82, 1803,  160229,  160254,  160290, 25, 36, 48, }, // Vanuatu
    {   82, 1811,       0,  160338,       0,  0, 31,  0, }, // Venezuela
    {   82, 1821,  160369,  160408,  160458, 39, 50, 62, }, // Vladivostok
    {   82, 1833,  160520,  160555,  160603, 35, 48, 58, }, // Volgograd
    {   82, 1843,       0,  160661,       0,  0, 27,  0, }, // Vostok
    {   82, 1850,       0,  160688,       0,  0, 44,  0, }, // Wake
    {   82, 1855,       0,  160732,       0,  0, 41,  0, }, // Wallis
    {   82, 1862,  160773,  160804,  160846, 31, 42, 54, }, // Yakutsk
    {   82, 1870,  160900,  160947,  161005, 47, 58, 70, }, // Yekaterinburg
    {   84,    5,       0,  161075,       0,  0,  6,  0, }, // Afghanistan Chechen/Cyrillic/Russia
    {   84,   17,       0,  161081,       0,  0, 14,  0, }, // Africa_Central
    {   84,   32,       0,  161095,       0,  0, 16,  0, }, // Africa_Eastern
    {   84,   65,       0,  161111,       0,  0, 13,  0, }, // Africa_Southern
    {   84,   81,  161124,  161140,  161172, 16, 32, 30, }, // Africa_Western
    {   84,  107,  115019,  161202,  161224,  6, 22, 20, }, // Alaska
    {   84,  135,    6710,  161244,  161268,  8, 24, 22, }, // Amazon
    {   84,  142,  161290,  161305,  161336, 15, 31, 29, }, // America_Central
    {   84,  158,  161365,  161382,  161415, 17, 33, 31, }, // America_Eastern
    {   84,  174,  161446,  161464,  161493, 18, 29, 27, }, // America_Mountain
    {   84,  191,  161520,  161537,  161565, 17, 28, 26, }, // America_Pacific
    {   84,  214,  161591,  161606,  161632, 15, 26, 24, }, // Apia
    {   84,  232,  161656,  161675,  161712, 19, 37, 36, }, // Arabian
    {   84,  240,    7438,  161748,  161773,  9, 25, 23, }, // Argentina
    {   84,  250,  161796,  161815,  161850, 19, 35, 33, }, // Argentina_Western
    {   84,  268,  161883,  161893,  161919, 10, 26, 24, }, // Armenia
    {   84,  286,  161943,  161957,  161982, 14, 25, 23, }, // Atlantic
    {   84,  295,  162005,  162021,  162053, 16, 32, 30, }, // Australia_Central
    {   84,  313,  162083,  162114,  162156, 31, 42, 40, }, // Australia_CentralWestern
    {   84,  338,  162196,  162214,  162248, 18, 34, 32, }, // Australia_Eastern
    {   84,  356,  162280,  162298,  162332, 18, 34, 32, }, // Australia_Western
    {   84,  374,   81570,  162364,  162391, 11, 27, 25, }, // Azerbaijan
    {   84,  385,  162416,  162432,  162464, 16, 32, 30, }, // Azores
    {   84,  397,    8323,  162494,  162519,  9, 25, 23, }, // Bangladesh
    {   84,  415,       0,    8384,       0,  0,  5,  0, }, // Bhutan
    {   84,  422,       0,    8389,       0,  0,  6,  0, }, // Bolivia
    {   84,  437,    8396,  162542,  162565,  7, 23, 21, }, // Brasilia
    {   84,  454,       0,  162586,       0,  0, 17,  0, }, // Brunei
    {   84,  461,    8471,  162603,  162629, 10, 26, 24, }, // Cape_Verde
    {   84,  478,       0,    8540,       0,  0,  7,  0, }, // Chamorro
    {   84,  487,    8547,  162653,  162674,  5, 21, 19, }, // Chatham
    {   84,  495,    8596,  162693,  162713,  4, 20, 18, }, // Chile
    {   84,  501,  162731,  162737,  162762,  6, 25, 23, }, // China
    {   84,  507,  117266,  162785,  162810,  9, 25, 23, }, // Choibalsan
    {   84,  518,       0,  162833,       0,  0, 31,  0, }, // Christmas
    {   84,  528,       0,  162864,       0,  0, 16,  0, }, // Cocos
    {   84,  534,    8781,  162880,  162903,  7, 23, 21, }, // Colombia
    {   84,  543,  162924,  162937,  162966, 13, 29, 27, }, // Cook
    {   84,  548,    8934,  162993,  163013,  4, 20, 18, }, // Cuba
    {   84,  559,       0,  117590,       0,  0,  6,  0, }, // Davis
    {   84,  575,       0,  163031,       0,  0, 14,  0, }, // DumontDUrville
    {   84,  612,       0,  163045,       0,  0, 15,  0, }, // East_Timor
    {   84,  623,  163060,  163073,  163102, 13, 29, 27, }, // Easter
    {   84,  630,       0,   82494,       0,  0,  7,  0, }, // Ecuador
    {   84,  638,  163129,  163143,  163173, 14, 30, 28, }, // Europe_Central
    {   84,  653,  163201,  163217,  163249, 16, 32, 30, }, // Europe_Eastern
    {   84,  668,       0,  163279,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {   84,  691,  163311,  163327,  163359, 16, 32, 30, }, // Europe_Western
    {   84,  706,  163389,  163409,  163445, 20, 36, 34, }, // Falkland
    {   84,  715,  118145,  163479,  163500,  5, 21, 19, }, // Fiji
    {   84,  720,       0,  163519,       0,  0, 17,  0, }, // French_Guiana
    {   84,  734,       0,  163536,       0,  0, 38,  0, }, // French_Southern
    {   84,  757,       0,  163574,       0,  0, 21,  0, }, // Galapagos
    {   84,  767,       0,  163595,       0,  0,  6,  0, }, // Gambier
    {   84,  775,  163601,  163611,  163637, 10, 26, 24, }, // Georgia
    {   84,  783,       0,  163661,       0,  0, 17,  0, }, // Gilbert_Islands
    {   84,  799,       0,  163678,       0,  0, 21,  0, }, // GMT
    {   84,  841,  163699,  163718,  163753, 19, 35, 33, }, // Greenland_Eastern
    {   84,  859,  163786,  163805,  163840, 19, 35, 33, }, // Greenland_Western
    {   84,  882,       0,  163873,       0,  0, 14,  0, }, // Gulf
    {   84,  887,       0,  163887,       0,  0,  6,  0, }, // Guyana
    {   84,  894,  163893,  163912,  163942, 19, 30, 28, }, // Hawaii_Aleutian
    {   84,  910,  163970,  163977,  164000,  7, 23, 21, }, // Hong_Kong
    {   84,  920,   10086,  164021,  164041,  4, 20, 18, }, // Hovd
    {   84,  925,       0,  164059,       0,  0,  6,  0, }, // India
    {   84,  931,       0,  164065,       0,  0, 11,  0, }, // Indian_Ocean
    {   84,  944,       0,  118863,       0,  0,  9,  0, }, // Indochina
    {   84,  954,       0,  164076,       0,  0, 16,  0, }, // Indonesia_Central
    {   84,  972,       0,  164092,       0,  0, 18,  0, }, // Indonesia_Eastern
    {   84,  990,       0,  164110,       0,  0, 18,  0, }, // Indonesia_Western
    {   84, 1008,  164128,  164139,  164166, 11, 27, 25, }, // Iran
    {   84, 1019,   10271,  164191,  164214,  7, 23, 21, }, // Irkutsk
    {   84, 1027,   10326,  164235,  164258,  7, 23, 21, }, // Israel
    {   84, 1034,   83794,  164279,  164299,  4, 20, 18, }, // Japan
    {   84, 1058,       0,  164317,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {   84, 1077,       0,  164336,       0,  0, 19,  0, }, // Kazakhstan_Western
    {   84, 1106,  119382,  164355,  164376,  5, 21, 19, }, // Korea
    {   84, 1112,       0,  164395,       0,  0,  6,  0, }, // Kosrae
    {   84, 1119,  119454,  164401,  164427, 10, 26, 24, }, // Krasnoyarsk
    {   84, 1151,       0,   10715,       0,  0,  7,  0, }, // Kyrgystan
    {   84, 1175,       0,  164451,       0,  0, 12,  0, }, // Line_Islands
    {   84, 1188,   84044,  164463,  164487,  8, 24, 22, }, // Lord_Howe
    {   84, 1204,   10852,  164509,  164532,  7, 23, 21, }, // Magadan
    {   84, 1219,       0,  119704,       0,  0,  7,  0, }, // Malaysia
    {   84, 1228,       0,  164553,       0,  0,  9,  0, }, // Maldives
    {   84, 1237,       0,  164562,       0,  0, 16,  0, }, // Marquesas
    {   84, 1247,       0,  164578,       0,  0, 17,  0, }, // Marshall_Islands
    {   84, 1264,   10966,  164595,  164618,  7, 23, 21, }, // Mauritius
    {   84, 1274,       0,   11021,       0,  0,  6,  0, }, // Mawson
    {   84, 1281,  164639,  164665,  164702, 26, 37, 35, }, // Mexico_Pacific
    {   84, 1296,   11137,  164737,  164763, 10, 26, 24, }, // Mongolia
    {   84, 1305,   11201,  164787,  164809,  6, 22, 20, }, // Moscow
    {   84, 1312,       0,  164829,       0,  0,  6,  0, }, // Myanmar
    {   84, 1320,       0,   11259,       0,  0,  5,  0, }, // Nauru
    {   84, 1326,       0,   11264,       0,  0,  5,  0, }, // Nepal
    {   84, 1332,  164835,  164849,  164879, 14, 30, 28, }, // New_Caledonia
    {   84, 1346,  164907,  164920,  164949, 13, 29, 27, }, // New_Zealand
    {   84, 1358,  164976,  164988,  165016, 12, 28, 26, }, // Newfoundland
    {   84, 1371,       0,  165042,       0,  0,  4,  0, }, // Niue
    {   84, 1376,   11486,  165046,  165069,  7, 23, 21, }, // Norfolk
    {   84, 1384,  165090,  165109,  165144, 19, 35, 33, }, // Noronha
    {   84, 1406,   11659,  165177,  165204, 11, 27, 25, }, // Novosibirsk
    {   84, 1418,   11726,  165229,  165249,  4, 20, 18, }, // Omsk
    {   84, 1428,   11772,  165267,  165291,  8, 24, 22, }, // Pakistan
    {   84, 1437,       0,   11830,       0,  0,  5,  0, }, // Palau
    {   84, 1443,       0,  165313,       0,  0, 20,  0, }, // Papua_New_Guinea
    {   84, 1460,  120772,  165333,  165357,  8, 24, 22, }, // Paraguay
    {   84, 1469,   11912,  165379,  165399,  4, 20, 18, }, // Peru
    {   84, 1474,  165417,  165427,  165453, 10, 26, 24, }, // Philippines
    {   84, 1486,       0,  165477,       0,  0, 14,  0, }, // Phoenix_Islands
    {   84, 1502,  165491,  165512,  165549, 21, 37, 35, }, // Pierre_Miquelon
    {   84, 1518,       0,  165584,       0,  0,  7,  0, }, // Pitcairn
    {   84, 1527,       0,  165591,       0,  0, 14,  0, }, // Ponape
    {   84, 1534,       0,   85459,       0,  0,  7,  0, }, // Pyongyang
    {   84, 1554,       0,  165605,       0,  0,  7,  0, }, // Reunion
    {   84, 1562,       0,   12210,       0,  0,  6,  0, }, // Rothera
    {   84, 1570,   12216,  165612,  165635,  7, 23, 21, }, // Sakhalin
    {   84, 1596,   12333,  165656,  165677,  5, 21, 19, }, // Samoa
    {   84, 1602,       0,  165696,       0,  0, 18,  0, }, // Seychelles
    {   84, 1624,       0,   12407,       0,  0,  8,  0, }, // Singapore
    {   84, 1634,       0,  165714,       0,  0, 17,  0, }, // Solomon
    {   84, 1642,       0,  165731,       0,  0, 13,  0, }, // South_Georgia
    {   84, 1656,       0,   12457,       0,  0,  7,  0, }, // Suriname
    {   84, 1676,       0,   85714,       0,  0,  4,  0, }, // Syowa
    {   84, 1682,       0,  165744,       0,  0, 13,  0, }, // Tahiti
    {   84, 1689,  165757,  165764,  165787,  7, 23, 21, }, // Taipei
    {   84, 1696,       0,  121452,       0,  0, 11,  0, }, // Tajikistan
    {   84, 1724,       0,   12536,       0,  0,  7,  0, }, // Tokelau
    {   84, 1732,   12543,  165808,  165829,  5, 21, 19, }, // Tonga
    {   84, 1738,       0,  165848,       0,  0,  4,  0, }, // Truk
    {   84, 1750,   12596,  165852,  165876,  8, 24, 22, }, // Turkmenistan
    {   84, 1763,       0,   12666,       0,  0,  6,  0, }, // Tuvalu
    {   84, 1777,   85932,  165898,  165921,  7, 23, 21, }, // Uruguay
    {   84, 1792,  108762,  165942,  165969, 10, 27, 25, }, // Uzbekistan
    {   84, 1803,   12794,  165994,  166017,  7, 23, 21, }, // Vanuatu
    {   84, 1811,       0,  166038,       0,  0,  9,  0, }, // Venezuela
    {   84, 1821,   12858,  166047,  166074, 11, 27, 25, }, // Vladivostok
    {   84, 1833,   12925,  166099,  166124,  9, 25, 23, }, // Volgograd
    {   84, 1843,       0,  109090,       0,  0,  6,  0, }, // Vostok
    {   84, 1850,       0,  166147,       0,  0, 10,  0, }, // Wake
    {   84, 1855,       0,  166157,       0,  0, 18,  0, }, // Wallis
    {   84, 1862,  122024,  166175,  166197,  6, 22, 20, }, // Yakutsk
    {   84, 1870,   13070,  166217,  166245, 12, 28, 26, }, // Yekaterinburg
    {   85,    5,       0,  166271,       0,  0, 13,  0, }, // Afghanistan Cherokee/Cherokee/United States
    {   85,   17,       0,  166284,       0,  0, 15,  0, }, // Africa_Central
    {   85,   32,       0,  166299,       0,  0, 16,  0, }, // Africa_Eastern
    {   85,   65,       0,  166315,       0,  0, 22,  0, }, // Africa_Southern
    {   85,   81,  166337,  166353,  166375, 16, 22, 19, }, // Africa_Western
    {   85,  107,  166394,  166404,  166420, 10, 16, 16, }, // Alaska
    {   85,  135,  166436,  166446,  166462, 10, 16, 13, }, // Amazon
    {   85,  142,  166475,  166484,  166499,  9, 15, 15, }, // America_Central
    {   85,  158,  166514,  166527,  166546, 13, 19, 19, }, // America_Eastern
    {   85,  174,  166565,  166574,  166589,  9, 15, 15, }, // America_Mountain
    {   85,  191,  166604,  166614,  166630, 10, 16, 16, }, // America_Pacific
    {   85,  214,  166646,  166655,  166670,  9, 15, 15, }, // Apia
    {   85,  232,  166685,  166695,  166711, 10, 16, 16, }, // Arabian
    {   85,  240,  166727,  166739,  166757, 12, 18, 15, }, // Argentina
    {   85,  250,  166772,  166792,  166818, 20, 26, 29, }, // Argentina_Western
    {   85,  268,  166847,  166858,  166875, 11, 17, 14, }, // Armenia
    {   85,  286,  166889,  166899,  166915, 10, 16, 16, }, // Atlantic
    {   85,  295,  166931,  166945,  166965, 14, 20, 20, }, // Australia_Central
    {   85,  313,  166985,  167007,  167035, 22, 28, 28, }, // Australia_CentralWestern
    {   85,  338,  167063,  167081,  167105, 18, 24, 24, }, // Australia_Eastern
    {   85,  356,  166989,  167011,  167039, 18, 24, 24, }, // Australia_Western
    {   85,  374,  167129,  167141,  167159, 12, 18, 15, }, // Azerbaijan
    {   85,  385,  167174,  167184,  167200, 10, 16, 13, }, // Azores
    {   85,  397,  167213,  167225,  167243, 12, 18, 15, }, // Bangladesh
    {   85,  415,       0,  167258,       0,  0,  9,  0, }, // Bhutan
    {   85,  422,       0,  167267,       0,  0, 10,  0, }, // Bolivia
    {   85,  437,  167277,  167287,  167303, 10, 16, 13, }, // Brasilia
    {   85,  454,       0,  167316,       0,  0, 15,  0, }, // Brunei
    {   85,  461,  167331,  167348,  167371, 17, 23, 20, }, // Cape_Verde
    {   85,  478,       0,  167391,       0,  0, 15,  0, }, // Chamorro
    {   85,  487,  167406,  167415,  167430,  9, 15, 15, }, // Chatham
    {   85,  495,  167445,  167453,  167467,  8, 14, 11, }, // Chile
    {   85,  501,  167478,  167490,  167508, 12, 18, 18, }, // China
    {   85,  507,  167526,  167538,  167556, 12, 18, 15, }, // Choibalsan
    {   85,  518,       0,  167571,       0,  0, 19,  0, }, // Christmas
    {   85,  528,       0,  167590,       0,  0, 15,  0, }, // Cocos
    {   85,  534,  167605,  167617,  167635, 12, 18, 15, }, // Colombia
    {   85,  543,  167650,  167670,  167696, 20, 26, 27, }, // Cook
    {   85,  548,  167723,  167731,  167745,  8, 14, 14, }, // Cuba
    {   85,  559,       0,  167759,       0,  0,  9,  0, }, // Davis
    {   85,  575,       0,  167768,       0,  0, 17,  0, }, // DumontDUrville
    {   85,  612,       0,  167785,       0,  0, 14,  0, }, // East_Timor
    {   85,  623,  167799,  167818,  167843, 19, 25, 22, }, // Easter
    {   85,  630,       0,  167865,       0,  0, 10,  0, }, // Ecuador
    {   85,  638,  167875,  167888,  167907, 13, 19, 16, }, // Europe_Central
    {   85,  653,  167923,  167940,  167963, 17, 23, 20, }, // Europe_Eastern
    {   85,  668,       0,  167983,       0,  0, 14,  0, }, // Europe_Further_Eastern
    {   85,  691,  167997,  168014,  168037, 17, 23, 20, }, // Europe_Western
    {   85,  706,  168057,  168071,  168091, 14, 20, 17, }, // Falkland
    {   85,  715,  168108,  168116,  168130,  8, 14, 11, }, // Fiji
    {   85,  720,       0,  168141,       0,  0, 14,  0, }, // French_Guiana
    {   85,  734,       0,  168155,       0,  0, 24,  0, }, // French_Southern
    {   85,  757,       0,  168179,       0,  0, 18,  0, }, // Galapagos
    {   85,  767,       0,  168197,       0,  0, 10,  0, }, // Gambier
    {   85,  775,  168207,  168217,  168233, 10, 16, 13, }, // Georgia
    {   85,  783,       0,  168246,       0,  0, 16,  0, }, // Gilbert_Islands
    {   85,  799,       0,  168262,       0,  0, 13,  0, }, // GMT
    {   85,  841,  168275,  168291,  168313, 16, 22, 19, }, // Greenland_Eastern
    {   85,  859,  168332,  168348,  168370, 16, 22, 19, }, // Greenland_Western
    {   85,  882,       0,  168389,       0,  0, 17,  0, }, // Gulf
    {   85,  887,       0,  168406,       0,  0,  9,  0, }, // Guyana
    {   85,  894,  168415,  168431,  168453, 16, 22, 22, }, // Hawaii_Aleutian
    {   85,  910,  168475,  168488,  168507, 13, 19, 16, }, // Hong_Kong
    {   85,  920,  168523,  168532,  168547,  9, 15, 12, }, // Hovd
    {   85,  925,       0,  168559,       0,  0, 17,  0, }, // India
    {   85,  931,       0,  168576,       0,  0, 16,  0, }, // Indian_Ocean
    {   85,  944,       0,  168592,       0,  0, 15,  0, }, // Indochina
    {   85,  954,       0,  168607,       0,  0, 16,  0, }, // Indonesia_Central
    {   85,  972,       0,  168623,       0,  0, 20,  0, }, // Indonesia_Eastern
    {   85,  990,       0,  168643,       0,  0, 20,  0, }, // Indonesia_Western
    {   85, 1008,  168663,  168672,  168687,  9, 15, 15, }, // Iran
    {   85, 1019,  168702,  168712,  168728, 10, 16, 13, }, // Irkutsk
    {   85, 1027,  168741,  168751,  168767, 10, 16, 17, }, // Israel
    {   85, 1034,  168784,  168794,  168810, 10, 16, 16, }, // Japan
    {   85, 1058,       0,  168826,       0,  0, 17,  0, }, // Kazakhstan_Eastern
    {   85, 1077,       0,  168843,       0,  0, 17,  0, }, // Kazakhstan_Western
    {   85, 1106,  168860,  168870,  168886, 10, 16, 16, }, // Korea
    {   85, 1112,       0,  168902,       0,  0,  9,  0, }, // Kosrae
    {   85, 1119,  168911,  168923,  168941, 12, 18, 15, }, // Krasnoyarsk
    {   85, 1151,       0,  168956,       0,  0, 11,  0, }, // Kyrgystan
    {   85, 1175,       0,  168967,       0,  0, 17,  0, }, // Line_Islands
    {   85, 1188,  168984,  168998,  169018, 14, 20, 20, }, // Lord_Howe
    {   85, 1204,  169038,  169048,  169064, 10, 16, 13, }, // Magadan
    {   85, 1219,       0,  169077,       0,  0, 11,  0, }, // Malaysia
    {   85, 1228,       0,  169088,       0,  0, 11,  0, }, // Maldives
    {   85, 1237,       0,  169099,       0,  0, 11,  0, }, // Marquesas
    {   85, 1247,       0,  169110,       0,  0, 15,  0, }, // Marshall_Islands
    {   85, 1264,  169125,  169136,  169153, 11, 17, 14, }, // Mauritius
    {   85, 1274,       0,  169167,       0,  0,  9,  0, }, // Mawson
    {   85, 1281,  169176,  169192,  169214, 16, 22, 22, }, // Mexico_Pacific
    {   85, 1296,  169236,  169249,  169268, 13, 19, 16, }, // Mongolia
    {   85, 1305,  169284,  169294,  169310, 10, 16, 13, }, // Moscow
    {   85, 1312,       0,  169323,       0,  0, 10,  0, }, // Myanmar
    {   85, 1320,       0,  169333,       0,  0,  8,  0, }, // Nauru
    {   85, 1326,       0,  169341,       0,  0,  9,  0, }, // Nepal
    {   85, 1332,  169350,  169365,  169386, 15, 21, 18, }, // New_Caledonia
    {   85, 1346,  169404,  169418,  169438, 14, 20, 20, }, // New_Zealand
    {   85, 1358,  169458,  169474,  169496, 16, 22, 22, }, // Newfoundland
    {   85, 1371,       0,  169518,       0,  0,  8,  0, }, // Niue
    {   85, 1376,  169526,  169543,  169566, 17, 23, 20, }, // Norfolk
    {   85, 1384,  169586,  169603,  169626, 17, 23, 20, }, // Noronha
    {   85, 1406,  169646,  169658,  169676, 12, 18, 15, }, // Novosibirsk
    {   85, 1418,  169691,  169701,  169717, 10, 16, 13, }, // Omsk
    {   85, 1428,  169730,  169741,  169758, 11, 17, 14, }, // Pakistan
    {   85, 1437,       0,  169772,       0,  0,  8,  0, }, // Palau
    {   85, 1443,       0,  169780,       0,  0, 15,  0, }, // Papua_New_Guinea
    {   85, 1460,  169795,  169804,  169819,  9, 15, 12, }, // Paraguay
    {   85, 1469,  169831,  169839,  169853,  8, 14, 11, }, // Peru
    {   85, 1474,  169864,  169876,  169894, 12, 18, 15, }, // Philippines
    {   85, 1486,       0,  169909,       0,  0, 18,  0, }, // Phoenix_Islands
    {   85, 1502,  169927,  169947,  169973, 20, 26, 26, }, // Pierre_Miquelon
    {   85, 1518,       0,  169999,       0,  0, 10,  0, }, // Pitcairn
    {   85, 1527,       0,  170009,       0,  0,  9,  0, }, // Ponape
    {   85, 1534,       0,  170018,       0,  0, 13,  0, }, // Pyongyang
    {   85, 1554,       0,  170031,       0,  0, 11,  0, }, // Reunion
    {   85, 1562,       0,  170042,       0,  0,  9,  0, }, // Rothera
    {   85, 1570,  170051,  170061,  170077, 10, 16, 13, }, // Sakhalin
    {   85, 1596,  170090,  170099,  170114,  9, 15, 15, }, // Samoa
    {   85, 1602,       0,  170129,       0,  0, 10,  0, }, // Seychelles
    {   85, 1624,       0,  170139,       0,  0, 17,  0, }, // Singapore
    {   85, 1634,       0,  170156,       0,  0, 16,  0, }, // Solomon
    {   85, 1642,       0,  170172,       0,  0, 15,  0, }, // South_Georgia
    {   85, 1656,       0,  170187,       0,  0, 10,  0, }, // Suriname
    {   85, 1676,       0,  170197,       0,  0,  9,  0, }, // Syowa
    {   85, 1682,       0,  170206,       0,  0,  9,  0, }, // Tahiti
    {   85, 1689,  170215,  170224,  170239,  9, 15, 15, }, // Taipei
    {   85, 1696,       0,  170254,       0,  0, 12,  0, }, // Tajikistan
    {   85, 1724,       0,  170266,       0,  0, 10,  0, }, // Tokelau
    {   85, 1732,  170276,  170285,  170300,  9, 15, 12, }, // Tonga
    {   85, 1738,       0,  170312,       0,  0,  8,  0, }, // Truk
    {   85, 1750,  170320,  170334,  170354, 14, 20, 17, }, // Turkmenistan
    {   85, 1763,       0,  170371,       0,  0,  9,  0, }, // Tuvalu
    {   85, 1777,  170380,  170389,  170404,  9, 15, 12, }, // Uruguay
    {   85, 1792,  170416,  170429,  170448, 13, 19, 16, }, // Uzbekistan
    {   85, 1803,  170464,  170474,  170490, 10, 16, 13, }, // Vanuatu
    {   85, 1811,       0,  170503,       0,  0, 11,  0, }, // Venezuela
    {   85, 1821,  170514,  170527,  170546, 13, 19, 16, }, // Vladivostok
    {   85, 1833,  170562,  170573,  170590, 11, 17, 14, }, // Volgograd
    {   85, 1843,       0,  170604,       0,  0, 10,  0, }, // Vostok
    {   85, 1850,       0,  170614,       0,  0, 15,  0, }, // Wake
    {   85, 1855,       0,  170629,       0,  0, 16,  0, }, // Wallis
    {   85, 1862,  170645,  170655,  170671, 10, 16, 13, }, // Yakutsk
    {   85, 1870,  170684,  170697,  170716, 13, 19, 16, }, // Yekaterinburg
    {   85, 1892,       0,  170732,       0,  0,  9,  0, }, // Yukon
    {   88,    0,  170741,  170746,  170753,  5,  7,  7, }, // Acre Chinese/Simplified Han/China
    {   88,    5,       0,  130892,       0,  0,  5,  0, }, // Afghanistan
    {   88,   17,       0,  170760,       0,  0,  6,  0, }, // Africa_Central
    {   88,   32,       0,  170766,       0,  0,  6,  0, }, // Africa_Eastern
    {   88,   65,       0,  130905,       0,  0,  6,  0, }, // Africa_Southern
    {   88,   81,  170772,  170778,  170786,  6,  8,  8, }, // Africa_Western
    {   88,  107,  130927,  130933,  130941,  6,  8,  8, }, // Alaska
    {   88,  128,  130949,  130955,  130963,  6,  8,  8, }, // Almaty
    {   88,  135,  130971,  130976,  130983,  5,  7,  7, }, // Amazon
    {   88,  142,  170794,  170800,  170808,  6,  8,  8, }, // America_Central
    {   88,  158,  170816,  170822,  170830,  6,  8,  8, }, // America_Eastern
    {   88,  174,  170838,  170844,  170852,  6,  8,  8, }, // America_Mountain
    {   88,  191,  170860,  170867,  170876,  7,  9,  9, }, // America_Pacific
    {   88,  207,  131057,  170885,  170893,  6,  8,  8, }, // Anadyr
    {   88,  214,  131079,  131084,  131091,  5,  7,  7, }, // Apia
    {   88,  219,  170901,  170906,  170913,  5,  7,  7, }, // Aqtau
    {   88,  225,  170920,  170926,  170934,  6,  8,  8, }, // Aqtobe
    {   88,  232,  131139,  131144,  131151,  5,  7,  7, }, // Arabian
    {   88,  240,  131158,  131163,  131170,  5,  7,  7, }, // Argentina
    {   88,  250,  131177,  131184,  131193,  7,  9,  9, }, // Argentina_Western
    {   88,  268,  131202,  131208,  131216,  6,  8,  8, }, // Armenia
    {   88,  286,  131224,  131229,  131236,  5,  7,  7, }, // Atlantic
    {   88,  295,  170942,  170950,  170960,  8, 10, 10, }, // Australia_Central
    {   88,  313,  170970,  170979,  170990,  9, 11, 11, }, // Australia_CentralWestern
    {   88,  338,  171001,  171009,  171019,  8, 10, 10, }, // Australia_Eastern
    {   88,  356,  171029,  171037,  171047,  8, 10, 10, }, // Australia_Western
    {   88,  374,  171057,  171063,  171071,  6,  8,  8, }, // Azerbaijan
    {   88,  385,  131356,  131363,  131372,  7,  9,  9, }, // Azores
    {   88,  397,  131381,  131386,  131393,  5,  7,  7, }, // Bangladesh
    {   88,  415,       0,  131400,       0,  0,  4,  0, }, // Bhutan
    {   88,  422,       0,  171079,       0,  0,  8,  0, }, // Bolivia
    {   88,  437,  131410,  131416,  131424,  6,  8,  8, }, // Brasilia
    {   88,  454,       0,  171087,       0,  0,  8,  0, }, // Brunei
    {   88,  461,  171095,  171100,  171107,  5,  7,  7, }, // Cape_Verde
    {   88,  472,       0,  171114,       0,  0,  4,  0, }, // Casey
    {   88,  478,       0,  171118,       0,  0,  5,  0, }, // Chamorro
    {   88,  487,  171123,  171128,  171135,  5,  7,  7, }, // Chatham
    {   88,  495,  131487,  131491,  131497,  4,  6,  6, }, // Chile
    {   88,  501,  131503,  131507,  131513,  4,  6,  6, }, // China
    {   88,  507,  131519,  131524,  131531,  5,  7,  7, }, // Choibalsan
    {   88,  518,       0,  131538,       0,  0,  5,  0, }, // Christmas
    {   88,  528,       0,  131543,       0,  0,  7,  0, }, // Cocos
    {   88,  534,  131550,  131556,  131564,  6,  8,  8, }, // Colombia
    {   88,  543,  131572,  131578,  171142,  6,  8,  8, }, // Cook
    {   88,  548,  131595,  131599,  131605,  4,  6,  6, }, // Cuba
    {   88,  559,       0,  131611,       0,  0,  5,  0, }, // Davis
    {   88,  575,       0,  171150,       0,  0,  8,  0, }, // DumontDUrville
    {   88,  612,       0,  131623,       0,  0,  5,  0, }, // East_Timor
    {   88,  623,  131628,  131634,  131642,  6,  8,  8, }, // Easter
    {   88,  630,       0,  171158,       0,  0,  8,  0, }, // Ecuador
    {   88,  638,  131655,  131659,  131665,  4,  6,  6, }, // Europe_Central
    {   88,  653,  131671,  131675,  131681,  4,  6,  6, }, // Europe_Eastern
    {   88,  668,       0,  171166,       0,  0,  6,  0, }, // Europe_Further_Eastern
    {   88,  691,  131693,  131697,  131703,  4,  6,  6, }, // Europe_Western
    {   88,  706,  131709,  131716,  131725,  7,  9,  9, }, // Falkland
    {   88,  715,  131734,  131738,  131744,  4,  6,  6, }, // Fiji
    {   88,  720,       0,  171172,       0,  0,  9,  0, }, // French_Guiana
    {   88,  734,       0,  171181,       0,  0, 11,  0, }, // French_Southern
    {   88,  757,       0,  171192,       0,  0,  7,  0, }, // Galapagos
    {   88,  767,       0,  171199,       0,  0,  5,  0, }, // Gambier
    {   88,  775,  171204,  171210,  171218,  6,  8,  8, }, // Georgia
    {   88,  783,       0,  131800,       0,  0,  8,  0, }, // Gilbert_Islands
    {   88,  799,       0,  171226,       0,  0,  8,  0, }, // GMT
    {   88,  841,  171234,  171242,  171252,  8, 10, 10, }, // Greenland_Eastern
    {   88,  859,  171262,  171270,  171280,  8, 10, 10, }, // Greenland_Western
    {   88,  877,       0,  171290,       0,  0,  4,  0, }, // Guam
    {   88,  882,       0,  171294,       0,  0,  6,  0, }, // Gulf
    {   88,  887,       0,  131752,       0,  0,  5,  0, }, // Guyana
    {   88,  894,  131886,  131895,  131906,  9, 11, 11, }, // Hawaii_Aleutian
    {   88,  910,  131917,  131921,  131927,  4,  6,  6, }, // Hong_Kong
    {   88,  920,  131933,  131938,  131945,  5,  7,  7, }, // Hovd
    {   88,  925,       0,  171300,       0,  0,  4,  0, }, // India
    {   88,  931,       0,  131958,       0,  0,  5,  0, }, // Indian_Ocean
    {   88,  944,       0,  171304,       0,  0,  6,  0, }, // Indochina
    {   88,  954,       0,  171310,       0,  0,  9,  0, }, // Indonesia_Central
    {   88,  972,       0,  171319,       0,  0,  9,  0, }, // Indonesia_Eastern
    {   88,  990,       0,  171328,       0,  0,  9,  0, }, // Indonesia_Western
    {   88, 1008,  131987,  131991,  131997,  4,  6,  6, }, // Iran
    {   88, 1019,  171337,  171344,  171353,  7,  9,  9, }, // Irkutsk
    {   88, 1027,  132028,  132033,  132040,  5,  7,  7, }, // Israel
    {   88, 1034,  132047,  132051,  132057,  4,  6,  6, }, // Japan
    {   88, 1040,  171362,  171377,  171394, 15, 17, 17, }, // Kamchatka
    {   88, 1058,       0,  171411,       0,  0,  9,  0, }, // Kazakhstan_Eastern
    {   88, 1077,       0,  171420,       0,  0,  9,  0, }, // Kazakhstan_Western
    {   88, 1106,  132114,  132118,  132124,  4,  6,  6, }, // Korea
    {   88, 1112,       0,  171429,       0,  0,  5,  0, }, // Kosrae
    {   88, 1119,  132135,  132145,  132157, 10, 12, 12, }, // Krasnoyarsk
    {   88, 1151,       0,  171434,       0,  0,  8,  0, }, // Kyrgystan
    {   88, 1161,       0,  132175,       0,  0,  4,  0, }, // Lanka
    {   88, 1175,       0,  132179,       0,  0,  6,  0, }, // Line_Islands
    {   88, 1188,  132185,  132191,  132199,  6,  8,  8, }, // Lord_Howe
    {   88, 1198,  132207,  132211,  132217,  4,  6,  6, }, // Macau
    {   88, 1204,  132223,  132228,  132235,  5,  7,  7, }, // Magadan
    {   88, 1219,       0,  132242,       0,  0,  6,  0, }, // Malaysia
    {   88, 1228,       0,  171442,       0,  0,  6,  0, }, // Maldives
    {   88, 1237,       0,  171448,       0,  0,  8,  0, }, // Marquesas
    {   88, 1247,       0,  132260,       0,  0,  7,  0, }, // Marshall_Islands
    {   88, 1264,  171456,  171462,  171470,  6,  8,  8, }, // Mauritius
    {   88, 1274,       0,  132289,       0,  0,  4,  0, }, // Mawson
    {   88, 1281,  132293,  132301,  132311,  8, 10, 10, }, // Mexico_Pacific
    {   88, 1296,  132321,  132327,  132335,  6,  8,  8, }, // Mongolia
    {   88, 1305,  132343,  132348,  132355,  5,  7,  7, }, // Moscow
    {   88, 1312,       0,  132362,       0,  0,  4,  0, }, // Myanmar
    {   88, 1320,       0,  171478,       0,  0,  4,  0, }, // Nauru
    {   88, 1326,       0,  132370,       0,  0,  5,  0, }, // Nepal
    {   88, 1332,  132375,  132383,  171482,  8, 10, 10, }, // New_Caledonia
    {   88, 1346,  171492,  171497,  171504,  5,  7,  7, }, // New_Zealand
    {   88, 1358,  132424,  132429,  132436,  5,  7,  7, }, // Newfoundland
    {   88, 1371,       0,  171511,       0,  0,  4,  0, }, // Niue
    {   88, 1376,  132448,  132454,  132462,  6,  8,  8, }, // Norfolk
    {   88, 1384,  171515,  171528,  171543, 13, 15, 15, }, // Noronha
    {   88, 1392,       0,  132516,       0,  0,  9,  0, }, // North_Mariana
    {   88, 1406,  132525,  132532,  132541,  7,  9,  9, }, // Novosibirsk
    {   88, 1418,  132550,  132556,  132564,  6,  8,  8, }, // Omsk
    {   88, 1428,  132572,  132578,  132586,  6,  8,  8, }, // Pakistan
    {   88, 1437,       0,  171558,       0,  0,  4,  0, }, // Palau
    {   88, 1443,       0,  171562,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   88, 1460,  132607,  132612,  132619,  5,  7,  7, }, // Paraguay
    {   88, 1469,  132626,  132630,  132636,  4,  6,  6, }, // Peru
    {   88, 1474,  132642,  132647,  132654,  5,  7,  7, }, // Philippines
    {   88, 1486,       0,  171571,       0,  0,  8,  0, }, // Phoenix_Islands
    {   88, 1502,  132667,  132679,  132693, 12, 14, 14, }, // Pierre_Miquelon
    {   88, 1518,       0,  171579,       0,  0,  6,  0, }, // Pitcairn
    {   88, 1527,       0,  132712,       0,  0,  5,  0, }, // Ponape
    {   88, 1534,       0,  132717,       0,  0,  4,  0, }, // Pyongyang
    {   88, 1544,  171585,  171592,  171601,  7,  9,  9, }, // Qyzylorda
    {   88, 1554,       0,  171610,       0,  0,  5,  0, }, // Reunion
    {   88, 1562,       0,  132754,       0,  0,  5,  0, }, // Rothera
    {   88, 1570,  132759,  132764,  132771,  5,  7,  7, }, // Sakhalin
    {   88, 1579,  132778,  132783,  132790,  5,  7,  7, }, // Samara
    {   88, 1596,  132797,  132802,  132809,  5,  7,  7, }, // Samoa
    {   88, 1602,       0,  171615,       0,  0,  5,  0, }, // Seychelles
    {   88, 1624,       0,  132821,       0,  0,  7,  0, }, // Singapore
    {   88, 1634,       0,  171620,       0,  0,  7,  0, }, // Solomon
    {   88, 1642,       0,  171627,       0,  0,  7,  0, }, // South_Georgia
    {   88, 1656,       0,  171634,       0,  0,  5,  0, }, // Suriname
    {   88, 1676,       0,  171639,       0,  0,  4,  0, }, // Syowa
    {   88, 1682,       0,  171643,       0,  0,  6,  0, }, // Tahiti
    {   88, 1689,  132857,  132861,  132867,  4,  6,  6, }, // Taipei
    {   88, 1696,       0,  171649,       0,  0,  7,  0, }, // Tajikistan
    {   88, 1724,       0,  171656,       0,  0,  5,  0, }, // Tokelau
    {   88, 1732,  171661,  171665,  171671,  4,  6,  6, }, // Tonga
    {   88, 1738,       0,  171677,       0,  0,  4,  0, }, // Truk
    {   88, 1750,  171681,  171688,  171697,  7,  9,  9, }, // Turkmenistan
    {   88, 1763,       0,  171706,       0,  0,  5,  0, }, // Tuvalu
    {   88, 1777,  132930,  132935,  132942,  5,  7,  7, }, // Uruguay
    {   88, 1792,  171711,  171719,  171729,  8, 10, 10, }, // Uzbekistan
    {   88, 1803,  171739,  171745,  171753,  6,  8,  8, }, // Vanuatu
    {   88, 1811,       0,  132990,       0,  0,  6,  0, }, // Venezuela
    {   88, 1821,  132996,  133001,  133008,  5,  7,  7, }, // Vladivostok
    {   88, 1833,  133015,  133022,  133031,  7,  9,  9, }, // Volgograd
    {   88, 1843,       0,  133040,       0,  0,  6,  0, }, // Vostok
    {   88, 1850,       0,  133046,       0,  0,  5,  0, }, // Wake
    {   88, 1855,       0,  171761,       0,  0,  9,  0, }, // Wallis
    {   88, 1862,  171770,  171776,  171784,  6,  8,  8, }, // Yakutsk
    {   88, 1870,  133084,  133091,  133100,  7,  9,  9, }, // Yekaterinburg
    {   88, 1892,       0,  133109,       0,  0,  4,  0, }, // Yukon
    {   91, 1219,       0,  132242,       0,  0,  6,  0, }, // Malaysia Chinese/Simplified Han/Singapore
    {   91, 1624,       0,  132821,       0,  0,  7,  0, }, // Singapore
    {   92,   65,       0,  171792,       0,  0,  4,  0, }, // Africa_Southern Chinese/Traditional Han/Hong Kong
    {   92,  142,  171796,  171802,  171810,  6,  8,  8, }, // America_Central
    {   92,  158,  171818,  171824,  171832,  6,  8,  8, }, // America_Eastern
    {   92,  174,  171840,  171846,  171854,  6,  8,  8, }, // America_Mountain
    {   92,  191,  171862,  171869,  171878,  7,  9,  9, }, // America_Pacific
    {   92,  374,  171887,  171893,  171901,  6,  8,  8, }, // Azerbaijan
    {   92,  461,  171909,  171914,  171921,  5,  7,  7, }, // Cape_Verde
    {   92,  528,       0,  171928,       0,  0,  7,  0, }, // Cocos
    {   92,  575,       0,  171935,       0,  0,  7,  0, }, // DumontDUrville
    {   92,  630,       0,  171942,       0,  0,  6,  0, }, // Ecuador
    {   92,  757,       0,  171948,       0,  0,  8,  0, }, // Galapagos
    {   92,  775,  171956,  171962,  171970,  6,  8,  8, }, // Georgia
    {   92,  882,       0,  171978,       0,  0,  7,  0, }, // Gulf
    {   92,  887,       0,  129515,       0,  0,  5,  0, }, // Guyana
    {   92,  925,       0,  171985,       0,  0,  4,  0, }, // India
    {   92, 1019,  171989,  171996,  172005,  7,  9,  9, }, // Irkutsk
    {   92, 1112,       0,  172014,       0,  0,  5,  0, }, // Kosrae
    {   92, 1228,       0,  172019,       0,  0,  6,  0, }, // Maldives
    {   92, 1237,       0,  172025,       0,  0,  6,  0, }, // Marquesas
    {   92, 1264,  172031,  172037,  172045,  6,  8,  8, }, // Mauritius
    {   92, 1320,       0,  172053,       0,  0,  4,  0, }, // Nauru
    {   92, 1332,  130138,  130146,  172057,  8, 10, 10, }, // New_Caledonia
    {   92, 1384,  172067,  172078,  172091, 11, 13, 13, }, // Noronha
    {   92, 1443,       0,  172104,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   92, 1518,       0,  172113,       0,  0,  5,  0, }, // Pitcairn
    {   92, 1602,       0,  172118,       0,  0,  5,  0, }, // Seychelles
    {   92, 1624,       0,  172123,       0,  0,  5,  0, }, // Singapore
    {   92, 1634,       0,  172128,       0,  0,  7,  0, }, // Solomon
    {   92, 1656,       0,  172135,       0,  0,  5,  0, }, // Suriname
    {   92, 1732,  172140,  172144,  172150,  4,  6,  6, }, // Tonga
    {   92, 1763,       0,  172156,       0,  0,  5,  0, }, // Tuvalu
    {   92, 1803,  172161,  172167,  172175,  6,  8,  8, }, // Vanuatu
    {   92, 1862,  172183,  172189,  172197,  6,  8,  8, }, // Yakutsk
    {   93,   65,       0,  171792,       0,  0,  4,  0, }, // Africa_Southern Chinese/Traditional Han/Macao
    {   93,  142,  171796,  171802,  171810,  6,  8,  8, }, // America_Central
    {   93,  158,  171818,  171824,  171832,  6,  8,  8, }, // America_Eastern
    {   93,  174,  171840,  171846,  171854,  6,  8,  8, }, // America_Mountain
    {   93,  191,  171862,  171869,  171878,  7,  9,  9, }, // America_Pacific
    {   93,  374,  171887,  171893,  171901,  6,  8,  8, }, // Azerbaijan
    {   93,  461,  171909,  171914,  171921,  5,  7,  7, }, // Cape_Verde
    {   93,  528,       0,  171928,       0,  0,  7,  0, }, // Cocos
    {   93,  575,       0,  171935,       0,  0,  7,  0, }, // DumontDUrville
    {   93,  630,       0,  171942,       0,  0,  6,  0, }, // Ecuador
    {   93,  757,       0,  171948,       0,  0,  8,  0, }, // Galapagos
    {   93,  775,  171956,  171962,  171970,  6,  8,  8, }, // Georgia
    {   93,  882,       0,  171978,       0,  0,  7,  0, }, // Gulf
    {   93,  887,       0,  129515,       0,  0,  5,  0, }, // Guyana
    {   93,  925,       0,  171985,       0,  0,  4,  0, }, // India
    {   93, 1019,  171989,  171996,  172005,  7,  9,  9, }, // Irkutsk
    {   93, 1112,       0,  172014,       0,  0,  5,  0, }, // Kosrae
    {   93, 1228,       0,  172019,       0,  0,  6,  0, }, // Maldives
    {   93, 1237,       0,  172025,       0,  0,  6,  0, }, // Marquesas
    {   93, 1264,  172031,  172037,  172045,  6,  8,  8, }, // Mauritius
    {   93, 1320,       0,  172053,       0,  0,  4,  0, }, // Nauru
    {   93, 1332,  130138,  130146,  172057,  8, 10, 10, }, // New_Caledonia
    {   93, 1384,  172067,  172078,  172091, 11, 13, 13, }, // Noronha
    {   93, 1443,       0,  172104,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   93, 1518,       0,  172113,       0,  0,  5,  0, }, // Pitcairn
    {   93, 1602,       0,  172118,       0,  0,  5,  0, }, // Seychelles
    {   93, 1624,       0,  172123,       0,  0,  5,  0, }, // Singapore
    {   93, 1634,       0,  172128,       0,  0,  7,  0, }, // Solomon
    {   93, 1656,       0,  172135,       0,  0,  5,  0, }, // Suriname
    {   93, 1732,  172140,  172144,  172150,  4,  6,  6, }, // Tonga
    {   93, 1763,       0,  172156,       0,  0,  5,  0, }, // Tuvalu
    {   93, 1803,  172161,  172167,  172175,  6,  8,  8, }, // Vanuatu
    {   93, 1862,  172183,  172189,  172197,  6,  8,  8, }, // Yakutsk
    {   94,    0,  128639,  128643,  128649,  4,  6,  6, }, // Acre Chinese/Traditional Han/Taiwan
    {   94,    5,       0,  128655,       0,  0,  5,  0, }, // Afghanistan
    {   94,   17,       0,  128660,       0,  0,  4,  0, }, // Africa_Central
    {   94,   32,       0,  128664,       0,  0,  4,  0, }, // Africa_Eastern
    {   94,   65,       0,  128668,       0,  0,  6,  0, }, // Africa_Southern
    {   94,   81,  128674,  128678,  128684,  4,  6,  6, }, // Africa_Western
    {   94,  107,  128690,  128696,  128704,  6,  8,  8, }, // Alaska
    {   94,  128,  128712,  128718,  128726,  6,  8,  8, }, // Almaty
    {   94,  135,  128734,  128739,  128746,  5,  7,  7, }, // Amazon
    {   94,  142,  128753,  128757,  128763,  4,  6,  6, }, // America_Central
    {   94,  158,  128769,  128773,  128779,  4,  6,  6, }, // America_Eastern
    {   94,  174,  128785,  128789,  128795,  4,  6,  6, }, // America_Mountain
    {   94,  191,  128801,  128806,  128813,  5,  7,  7, }, // America_Pacific
    {   94,  207,  128820,  128826,  128834,  6,  8,  8, }, // Anadyr
    {   94,  214,  128842,  128847,  128854,  5,  7,  7, }, // Apia
    {   94,  219,  128861,  128866,  128873,  5,  7,  7, }, // Aqtau
    {   94,  225,  128880,  128886,  128894,  6,  8,  8, }, // Aqtobe
    {   94,  232,  128902,  128907,  128914,  5,  7,  7, }, // Arabian
    {   94,  240,  128921,  128926,  128933,  5,  7,  7, }, // Argentina
    {   94,  250,  128940,  128947,  128956,  7,  9,  9, }, // Argentina_Western
    {   94,  268,  128965,  128971,  128979,  6,  8,  8, }, // Armenia
    {   94,  286,  128987,  128992,  128999,  5,  7,  7, }, // Atlantic
    {   94,  295,  129006,  129012,  129020,  6,  8,  8, }, // Australia_Central
    {   94,  313,  129028,  129035,  129044,  7,  9,  9, }, // Australia_CentralWestern
    {   94,  338,  129053,  129059,  129067,  6,  8,  8, }, // Australia_Eastern
    {   94,  356,  129075,  129081,  129089,  6,  8,  8, }, // Australia_Western
    {   94,  374,  129097,  129103,  129111,  6,  8,  8, }, // Azerbaijan
    {   94,  385,  129119,  129126,  129135,  7,  9,  9, }, // Azores
    {   94,  397,  129144,  129149,  129156,  5,  7,  7, }, // Bangladesh
    {   94,  415,       0,  129163,       0,  0,  4,  0, }, // Bhutan
    {   94,  422,       0,  129167,       0,  0,  6,  0, }, // Bolivia
    {   94,  437,  129173,  129179,  129187,  6,  8,  8, }, // Brasilia
    {   94,  454,       0,  129195,       0,  0,  4,  0, }, // Brunei
    {   94,  461,  129199,  129204,  129211,  5,  7,  7, }, // Cape_Verde
    {   94,  472,       0,  129218,       0,  0,  5,  0, }, // Casey
    {   94,  478,       0,  129223,       0,  0,  5,  0, }, // Chamorro
    {   94,  487,  129228,  129234,  129242,  6,  8,  8, }, // Chatham
    {   94,  495,  129250,  129254,  129260,  4,  6,  6, }, // Chile
    {   94,  501,  129266,  129270,  129276,  4,  6,  6, }, // China
    {   94,  507,  129282,  129287,  129294,  5,  7,  7, }, // Choibalsan
    {   94,  518,       0,  129301,       0,  0,  5,  0, }, // Christmas
    {   94,  528,       0,  129306,       0,  0,  7,  0, }, // Cocos
    {   94,  534,  129313,  129319,  129327,  6,  8,  8, }, // Colombia
    {   94,  543,  129335,  129341,  129349,  6,  8,  9, }, // Cook
    {   94,  548,  129358,  129362,  129368,  4,  6,  6, }, // Cuba
    {   94,  559,       0,  129374,       0,  0,  5,  0, }, // Davis
    {   94,  575,       0,  129379,       0,  0,  7,  0, }, // DumontDUrville
    {   94,  612,       0,  129386,       0,  0,  5,  0, }, // East_Timor
    {   94,  623,  129391,  129397,  129405,  6,  8,  8, }, // Easter
    {   94,  630,       0,  129413,       0,  0,  5,  0, }, // Ecuador
    {   94,  638,  129418,  129422,  129428,  4,  6,  6, }, // Europe_Central
    {   94,  653,  129434,  129438,  129444,  4,  6,  6, }, // Europe_Eastern
    {   94,  668,       0,  129450,       0,  0,  6,  0, }, // Europe_Further_Eastern
    {   94,  691,  129456,  129460,  129466,  4,  6,  6, }, // Europe_Western
    {   94,  706,  129472,  129479,  129488,  7,  9,  9, }, // Falkland
    {   94,  715,  129497,  129501,  129507,  4,  6,  6, }, // Fiji
    {   94,  720,       0,  129513,       0,  0,  7,  0, }, // French_Guiana
    {   94,  734,       0,  129520,       0,  0,  9,  0, }, // French_Southern
    {   94,  757,       0,  129529,       0,  0,  8,  0, }, // Galapagos
    {   94,  767,       0,  129537,       0,  0,  7,  0, }, // Gambier
    {   94,  775,  129544,  129549,  129556,  5,  7,  7, }, // Georgia
    {   94,  783,       0,  129563,       0,  0,  8,  0, }, // Gilbert_Islands
    {   94,  799,       0,  129571,       0,  0,  8,  0, }, // GMT
    {   94,  841,  129579,  129586,  129595,  7,  9,  9, }, // Greenland_Eastern
    {   94,  859,  129604,  129611,  129620,  7,  9,  9, }, // Greenland_Western
    {   94,  877,       0,  129629,       0,  0,  6,  0, }, // Guam
    {   94,  882,       0,  129635,       0,  0,  9,  0, }, // Gulf
    {   94,  887,       0,  129644,       0,  0,  5,  0, }, // Guyana
    {   94,  894,  129649,  129658,  129669,  9, 11, 11, }, // Hawaii_Aleutian
    {   94,  910,  129680,  129684,  129690,  4,  6,  6, }, // Hong_Kong
    {   94,  920,  129696,  129701,  129708,  5,  7,  7, }, // Hovd
    {   94,  925,       0,  129715,       0,  0,  6,  0, }, // India
    {   94,  931,       0,  129721,       0,  0,  5,  0, }, // Indian_Ocean
    {   94,  944,       0,  172205,       0,  0,  6,  0, }, // Indochina
    {   94,  954,       0,  129732,       0,  0,  6,  0, }, // Indonesia_Central
    {   94,  972,       0,  129738,       0,  0,  6,  0, }, // Indonesia_Eastern
    {   94,  990,       0,  129744,       0,  0,  6,  0, }, // Indonesia_Western
    {   94, 1008,  129750,  129754,  129760,  4,  6,  6, }, // Iran
    {   94, 1019,  129766,  129773,  129782,  7,  9,  9, }, // Irkutsk
    {   94, 1027,  129791,  129796,  129803,  5,  7,  7, }, // Israel
    {   94, 1034,  129810,  129814,  129820,  4,  6,  6, }, // Japan
    {   94, 1040,  129826,  129837,  129850, 11, 13, 15, }, // Kamchatka
    {   94, 1058,       0,  129865,       0,  0,  6,  0, }, // Kazakhstan_Eastern
    {   94, 1077,       0,  129871,       0,  0,  6,  0, }, // Kazakhstan_Western
    {   94, 1106,  129877,  129881,  129887,  4,  6,  6, }, // Korea
    {   94, 1112,       0,  129893,       0,  0,  5,  0, }, // Kosrae
    {   94, 1119,  129898,  129908,  129920, 10, 12, 12, }, // Krasnoyarsk
    {   94, 1151,       0,  129932,       0,  0,  6,  0, }, // Kyrgystan
    {   94, 1161,       0,  129938,       0,  0,  4,  0, }, // Lanka
    {   94, 1175,       0,  129942,       0,  0,  6,  0, }, // Line_Islands
    {   94, 1188,  129948,  129954,  129962,  6,  8,  8, }, // Lord_Howe
    {   94, 1198,  129970,  129974,  129980,  4,  6,  6, }, // Macau
    {   94, 1204,  129986,  129991,  129998,  5,  7,  7, }, // Magadan
    {   94, 1219,       0,  130005,       0,  0,  6,  0, }, // Malaysia
    {   94, 1228,       0,  130011,       0,  0,  6,  0, }, // Maldives
    {   94, 1237,       0,  130017,       0,  0,  6,  0, }, // Marquesas
    {   94, 1247,       0,  130023,       0,  0,  7,  0, }, // Marshall_Islands
    {   94, 1264,  130030,  130036,  130044,  6,  8,  8, }, // Mauritius
    {   94, 1274,       0,  130052,       0,  0,  4,  0, }, // Mawson
    {   94, 1281,  130056,  130064,  130074,  8, 10, 10, }, // Mexico_Pacific
    {   94, 1296,  130084,  130090,  130098,  6,  8,  8, }, // Mongolia
    {   94, 1305,  130106,  130111,  130118,  5,  7,  7, }, // Moscow
    {   94, 1312,       0,  130125,       0,  0,  4,  0, }, // Myanmar
    {   94, 1320,       0,  130129,       0,  0,  4,  0, }, // Nauru
    {   94, 1326,       0,  130133,       0,  0,  5,  0, }, // Nepal
    {   94, 1332,  130138,  130146,  130156,  8, 10, 12, }, // New_Caledonia
    {   94, 1346,  130168,  130173,  130180,  5,  7,  7, }, // New_Zealand
    {   94, 1358,  130187,  130192,  130199,  5,  7,  7, }, // Newfoundland
    {   94, 1371,       0,  130206,       0,  0,  5,  0, }, // Niue
    {   94, 1376,  130211,  130217,  130225,  6,  8,  8, }, // Norfolk
    {   94, 1384,  130233,  130247,  130263, 14, 16, 16, }, // Noronha
    {   94, 1392,       0,  130279,       0,  0,  9,  0, }, // North_Mariana
    {   94, 1406,  130288,  130295,  130304,  7,  9,  9, }, // Novosibirsk
    {   94, 1418,  130313,  130319,  130327,  6,  8,  8, }, // Omsk
    {   94, 1428,  130335,  130341,  130349,  6,  8,  8, }, // Pakistan
    {   94, 1437,       0,  130357,       0,  0,  4,  0, }, // Palau
    {   94, 1443,       0,  130361,       0,  0,  9,  0, }, // Papua_New_Guinea
    {   94, 1460,  130370,  130375,  130382,  5,  7,  7, }, // Paraguay
    {   94, 1469,  130389,  130393,  130399,  4,  6,  6, }, // Peru
    {   94, 1474,  130405,  130410,  130417,  5,  7,  7, }, // Philippines
    {   94, 1486,       0,  130424,       0,  0,  6,  0, }, // Phoenix_Islands
    {   94, 1502,  172211,  172222,  172235, 11, 13, 13, }, // Pierre_Miquelon
    {   94, 1518,       0,  130470,       0,  0,  5,  0, }, // Pitcairn
    {   94, 1527,       0,  130475,       0,  0,  5,  0, }, // Ponape
    {   94, 1534,       0,  130480,       0,  0,  4,  0, }, // Pyongyang
    {   94, 1544,  130484,  130492,  130502,  8, 10, 10, }, // Qyzylorda
    {   94, 1554,       0,  130512,       0,  0,  5,  0, }, // Reunion
    {   94, 1562,       0,  130517,       0,  0,  5,  0, }, // Rothera
    {   94, 1570,  130522,  130527,  130534,  5,  7,  7, }, // Sakhalin
    {   94, 1579,  130541,  130546,  130553,  5,  7,  7, }, // Samara
    {   94, 1596,  130560,  130565,  130572,  5,  7,  7, }, // Samoa
    {   94, 1602,       0,  130579,       0,  0,  5,  0, }, // Seychelles
    {   94, 1624,       0,  130584,       0,  0,  7,  0, }, // Singapore
    {   94, 1634,       0,  130591,       0,  0,  7,  0, }, // Solomon
    {   94, 1642,       0,  130598,       0,  0,  6,  0, }, // South_Georgia
    {   94, 1656,       0,  130604,       0,  0,  5,  0, }, // Suriname
    {   94, 1676,       0,  130609,       0,  0,  6,  0, }, // Syowa
    {   94, 1682,       0,  130615,       0,  0,  5,  0, }, // Tahiti
    {   94, 1689,  130620,  130624,  130630,  4,  6,  6, }, // Taipei
    {   94, 1696,       0,  130636,       0,  0,  5,  0, }, // Tajikistan
    {   94, 1724,       0,  130641,       0,  0,  7,  0, }, // Tokelau
    {   94, 1732,  130648,  130652,  130658,  4,  6,  6, }, // Tonga
    {   94, 1738,       0,  130664,       0,  0,  5,  0, }, // Truk
    {   94, 1750,  130669,  130674,  130681,  5,  7,  7, }, // Turkmenistan
    {   94, 1763,       0,  130688,       0,  0,  5,  0, }, // Tuvalu
    {   94, 1777,  130693,  130698,  130705,  5,  7,  7, }, // Uruguay
    {   94, 1792,  130712,  130718,  130726,  6,  8,  8, }, // Uzbekistan
    {   94, 1803,  130734,  130739,  130746,  5,  7,  7, }, // Vanuatu
    {   94, 1811,       0,  130753,       0,  0,  6,  0, }, // Venezuela
    {   94, 1821,  130759,  130764,  130771,  5,  7,  7, }, // Vladivostok
    {   94, 1833,  130778,  130785,  130794,  7,  9,  9, }, // Volgograd
    {   94, 1843,       0,  130803,       0,  0,  6,  0, }, // Vostok
    {   94, 1850,       0,  130809,       0,  0,  5,  0, }, // Wake
    {   94, 1855,       0,  130814,       0,  0, 11,  0, }, // Wallis
    {   94, 1862,  130825,  130831,  130839,  6,  8,  8, }, // Yakutsk
    {   94, 1870,  130847,  130854,  130863,  7,  9,  9, }, // Yekaterinburg
    {   94, 1892,       0,  172248,       0,  0,  6,  0, }, // Yukon
    {   95,  142,  172254,  172280,  172314, 26, 34, 34, }, // America_Central Church/Cyrillic/Russia
    {   95,  158,  172348,  172376,  172412, 28, 36, 36, }, // America_Eastern
    {   95,  174,  172448,  172479,  172518, 31, 39, 39, }, // America_Mountain
    {   95,  191,  172557,  172578,  172607, 21, 29, 29, }, // America_Pacific
    {   95,  286,  172636,  172658,  172688, 22, 30, 30, }, // Atlantic
    {   95,  638,  172718,  172743,  172776, 25, 33, 33, }, // Europe_Central
    {   95,  653,  172809,  172836,  172871, 27, 35, 35, }, // Europe_Eastern
    {   95,  668,       0,  172906,       0,  0, 34,  0, }, // Europe_Further_Eastern
    {   95,  691,  172940,  172966,  173000, 26, 34, 34, }, // Europe_Western
    {   95,  799,       0,  173034,       0,  0, 28,  0, }, // GMT
    {   95, 1019,  173062,  173080,  173106, 18, 26, 26, }, // Irkutsk
    {   95, 1058,       0,  173132,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {   95, 1077,       0,  173154,       0,  0, 21,  0, }, // Kazakhstan_Western
    {   95, 1119,  173175,  173195,  173223, 20, 28, 28, }, // Krasnoyarsk
    {   95, 1151,       0,  173251,       0,  0,  9,  0, }, // Kyrgystan
    {   95, 1204,  173260,  173279,  173306, 19, 27, 27, }, // Magadan
    {   95, 1305,  173333,  173351,  173377, 18, 26, 26, }, // Moscow
    {   95, 1406,  173403,  173424,  173453, 21, 29, 29, }, // Novosibirsk
    {   95, 1418,  173482,  173497,  173520, 15, 23, 23, }, // Omsk
    {   95, 1570,  173543,  173562,  173589, 19, 27, 27, }, // Sakhalin
    {   95, 1821,  173616,  173638,  173668, 22, 30, 30, }, // Vladivostok
    {   95, 1833,  173698,  173719,  173748, 21, 29, 29, }, // Volgograd
    {   95, 1862,  173777,  173794,  173819, 17, 25, 25, }, // Yakutsk
    {   95, 1870,  173844,  173869,  173902, 25, 33, 33, }, // Yekaterinburg
    {   96,    5,       0,  173935,       0,  0, 17,  0, }, // Afghanistan Chuvash/Cyrillic/Russia
    {   96,   17,       0,  173952,       0,  0, 17,  0, }, // Africa_Central
    {   96,   32,       0,  173969,       0,  0, 24,  0, }, // Africa_Eastern
    {   96,   65,       0,  173993,       0,  0, 20,  0, }, // Africa_Southern
    {   96,   81,  174013,  174031,  174060, 18, 29, 26, }, // Africa_Western
    {   96,  107,  174086,  174099,  174123, 13, 24, 21, }, // Alaska
    {   96,  135,  174144,  174159,  174185, 15, 26, 23, }, // Amazon
    {   96,  142,  174208,  174226,  174255, 18, 29, 26, }, // America_Central
    {   96,  158,  174281,  174306,  174342, 25, 36, 33, }, // America_Eastern
    {   96,  174,  174375,  174401,  174438, 26, 37, 34, }, // America_Mountain
    {   96,  191,  174472,  174490,  174519, 18, 29, 26, }, // America_Pacific
    {   96,  214,  174545,  174556,  174578, 11, 22, 19, }, // Apia
    {   96,  232,  174597,  174608,  174630, 11, 22, 19, }, // Arabian
    {   96,  240,  174649,  174665,  174692, 16, 27, 24, }, // Argentina
    {   96,  250,  174716,  174737,  174769, 21, 32, 29, }, // Argentina_Western
    {   96,  268,  174798,  174811,  174835, 13, 24, 21, }, // Armenia
    {   96,  286,  174856,  174872,  174899, 16, 27, 24, }, // Atlantic
    {   96,  295,  174923,  174942,  174972, 19, 30, 27, }, // Australia_Central
    {   96,  313,  174999,  175023,  175058, 24, 35, 32, }, // Australia_CentralWestern
    {   96,  338,  175090,  175116,  175153, 26, 37, 34, }, // Australia_Eastern
    {   96,  356,  175187,  175207,  175238, 20, 31, 28, }, // Australia_Western
    {   96,  374,  175266,  175284,  175313, 18, 29, 26, }, // Azerbaijan
    {   96,  385,  175339,  175360,  175392, 21, 32, 29, }, // Azores
    {   96,  397,  175421,  175437,  175464, 16, 27, 24, }, // Bangladesh
    {   96,  415,       0,  175488,       0,  0, 12,  0, }, // Bhutan
    {   96,  422,       0,  175500,       0,  0, 13,  0, }, // Bolivia
    {   96,  437,  175513,  175527,  175552, 14, 25, 22, }, // Brasilia
    {   96,  454,       0,  175574,       0,  0, 24,  0, }, // Brunei
    {   96,  461,  175598,  175615,  175643, 17, 28, 25, }, // Cape_Verde
    {   96,  478,       0,  175668,       0,  0, 14,  0, }, // Chamorro
    {   96,  487,  175682,  175694,  175717, 12, 23, 20, }, // Chatham
    {   96,  495,  175737,  175748,  175770, 11, 22, 19, }, // Chile
    {   96,  501,  175789,  175801,  175824, 12, 23, 20, }, // China
    {   96,  507,  175844,  175860,  175887, 16, 27, 24, }, // Choibalsan
    {   96,  518,       0,  175911,       0,  0, 21,  0, }, // Christmas
    {   96,  528,       0,  175932,       0,  0, 22,  0, }, // Cocos
    {   96,  534,  175954,  175968,  175993, 14, 25, 22, }, // Colombia
    {   96,  543,  176015,  176037,  176070, 22, 33, 30, }, // Cook
    {   96,  548,  176100,  176111,  176133, 11, 22, 19, }, // Cuba
    {   96,  559,       0,  176152,       0,  0, 13,  0, }, // Davis
    {   96,  575,       0,  176165,       0,  0, 21,  0, }, // DumontDUrville
    {   96,  612,       0,  176186,       0,  0, 23,  0, }, // East_Timor
    {   96,  623,  176209,  176230,  176262, 21, 32, 29, }, // Easter
    {   96,  630,       0,  176291,       0,  0, 14,  0, }, // Ecuador
    {   96,  638,  176305,  176322,  176350, 17, 28, 25, }, // Europe_Central
    {   96,  653,  176375,  176399,  176434, 24, 35, 32, }, // Europe_Eastern
    {   96,  668,       0,  176466,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {   96,  691,  176496,  176514,  176543, 18, 29, 26, }, // Europe_Western
    {   96,  706,  176569,  176594,  176630, 25, 36, 33, }, // Falkland
    {   96,  715,  176663,  176675,  176698, 12, 23, 20, }, // Fiji
    {   96,  720,       0,  176718,       0,  0, 20,  0, }, // French_Guiana
    {   96,  734,       0,  176738,       0,  0, 48,  0, }, // French_Southern
    {   96,  757,       0,  176786,       0,  0, 26,  0, }, // Galapagos
    {   96,  767,       0,  176812,       0,  0, 13,  0, }, // Gambier
    {   96,  775,  176825,  176837,  176860, 12, 23, 20, }, // Georgia
    {   96,  783,       0,  176880,       0,  0, 26,  0, }, // Gilbert_Islands
    {   96,  799,       0,  176906,       0,  0, 22,  0, }, // GMT
    {   96,  841,  176928,  176955,  176993, 27, 38, 35, }, // Greenland_Eastern
    {   96,  859,  177028,  177049,  177081, 21, 32, 19, }, // Greenland_Western
    {   96,  882,       0,  177100,       0,  0, 18,  0, }, // Gulf
    {   96,  887,       0,  177118,       0,  0, 13,  0, }, // Guyana
    {   96,  894,  177131,  177150,  177180, 19, 30, 27, }, // Hawaii_Aleutian
    {   96,  910,  177207,  177221,  177246, 14, 25, 22, }, // Hong_Kong
    {   96,  920,  177268,  177279,  177301, 11, 22, 19, }, // Hovd
    {   96,  925,       0,  177320,       0,  0, 11,  0, }, // India
    {   96,  931,       0,  177331,       0,  0, 18,  0, }, // Indian_Ocean
    {   96,  944,       0,  177349,       0,  0, 16,  0, }, // Indochina
    {   96,  954,       0,  177365,       0,  0, 19,  0, }, // Indonesia_Central
    {   96,  972,       0,  177384,       0,  0, 26,  0, }, // Indonesia_Eastern
    {   96,  990,       0,  177410,       0,  0, 20,  0, }, // Indonesia_Western
    {   96, 1008,  177430,  177441,  177463, 11, 22, 19, }, // Iran
    {   96, 1019,  177482,  177496,  177521, 14, 25, 22, }, // Irkutsk
    {   96, 1027,  177543,  177557,  177582, 14, 25, 22, }, // Israel
    {   96, 1034,  177604,  177616,  177639, 12, 23, 20, }, // Japan
    {   96, 1058,       0,  177659,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {   96, 1077,       0,  177686,       0,  0, 21,  0, }, // Kazakhstan_Western
    {   96, 1106,  177707,  177719,  177742, 12, 23, 20, }, // Korea
    {   96, 1112,       0,  177762,       0,  0, 13,  0, }, // Kosrae
    {   96, 1119,  177775,  177792,  177820, 17, 28, 25, }, // Krasnoyarsk
    {   96, 1151,       0,  177845,       0,  0, 14,  0, }, // Kyrgystan
    {   96, 1175,       0,  177859,       0,  0, 21,  0, }, // Line_Islands
    {   96, 1188,  177880,  177895,  177921, 15, 26, 23, }, // Lord_Howe
    {   96, 1204,  177944,  177958,  177983, 14, 25, 22, }, // Magadan
    {   96, 1219,       0,  178005,       0,  0, 14,  0, }, // Malaysia
    {   96, 1228,       0,  178019,       0,  0, 17,  0, }, // Maldives
    {   96, 1237,       0,  178036,       0,  0, 25,  0, }, // Marquesas
    {   96, 1247,       0,  178061,       0,  0, 24,  0, }, // Marshall_Islands
    {   96, 1264,  178085,  178100,  178126, 15, 26, 23, }, // Mauritius
    {   96, 1274,       0,  178149,       0,  0, 13,  0, }, // Mawson
    {   96, 1281,  178162,  178188,  178225, 26, 37, 34, }, // Mexico_Pacific
    {   96, 1296,  178259,  178276,  178304, 17, 28, 25, }, // Mongolia
    {   96, 1305,  178329,  178342,  178366, 13, 24, 21, }, // Moscow
    {   96, 1312,       0,  178387,       0,  0, 13,  0, }, // Myanmar
    {   96, 1320,       0,  178400,       0,  0, 12,  0, }, // Nauru
    {   96, 1326,       0,  178412,       0,  0, 12,  0, }, // Nepal
    {   96, 1332,  178424,  178444,  178475, 20, 31, 28, }, // New_Caledonia
    {   96, 1346,  178503,  178522,  178552, 19, 30, 27, }, // New_Zealand
    {   96, 1358,  178579,  178598,  178628, 19, 30, 27, }, // Newfoundland
    {   96, 1371,       0,  178655,       0,  0, 11,  0, }, // Niue
    {   96, 1376,  178666,  178680,  178705, 14, 25, 22, }, // Norfolk
    {   96, 1384,  178727,  178753,  178790, 26, 37, 34, }, // Noronha
    {   96, 1406,  178824,  178842,  178871, 18, 29, 26, }, // Novosibirsk
    {   96, 1418,  178897,  178908,  178930, 11, 22, 19, }, // Omsk
    {   96, 1428,  178949,  178964,  178990, 15, 26, 23, }, // Pakistan
    {   96, 1437,       0,  179013,       0,  0, 12,  0, }, // Palau
    {   96, 1443,       0,  179025,       0,  0, 26,  0, }, // Papua_New_Guinea
    {   96, 1460,  179051,  179066,  179092, 15, 26, 23, }, // Paraguay
    {   96, 1469,  179115,  179126,  179148, 11, 22, 19, }, // Peru
    {   96, 1474,  179167,  179185,  179214, 18, 29, 26, }, // Philippines
    {   96, 1486,       0,  179240,       0,  0, 13,  0, }, // Phoenix_Islands
    {   96, 1502,  179253,  179281,  179320, 28, 39, 36, }, // Pierre_Miquelon
    {   96, 1518,       0,  179356,       0,  0, 14,  0, }, // Pitcairn
    {   96, 1527,       0,  179370,       0,  0, 13,  0, }, // Ponape
    {   96, 1534,       0,   85459,       0,  0,  7,  0, }, // Pyongyang
    {   96, 1554,       0,  179383,       0,  0, 14,  0, }, // Reunion
    {   96, 1562,       0,  179397,       0,  0, 13,  0, }, // Rothera
    {   96, 1570,  179410,  179424,  179449, 14, 25, 22, }, // Sakhalin
    {   96, 1596,  179471,  179483,  179506, 12, 23, 20, }, // Samoa
    {   96, 1602,       0,  179526,       0,  0, 23,  0, }, // Seychelles
    {   96, 1624,       0,  179549,       0,  0, 15,  0, }, // Singapore
    {   96, 1634,       0,  179564,       0,  0, 14,  0, }, // Solomon
    {   96, 1642,       0,  179578,       0,  0, 20,  0, }, // South_Georgia
    {   96, 1656,       0,  179598,       0,  0, 14,  0, }, // Suriname
    {   96, 1676,       0,  179612,       0,  0, 11,  0, }, // Syowa
    {   96, 1682,       0,  179623,       0,  0, 12,  0, }, // Tahiti
    {   96, 1689,  179635,  179648,  179672, 13, 24, 21, }, // Taipei
    {   96, 1696,       0,  179693,       0,  0, 18,  0, }, // Tajikistan
    {   96, 1724,       0,  179711,       0,  0, 14,  0, }, // Tokelau
    {   96, 1732,  179725,  179737,  179760, 12, 23, 20, }, // Tonga
    {   96, 1738,       0,  179780,       0,  0, 11,  0, }, // Truk
    {   96, 1750,  179791,  179810,  179840, 19, 30, 27, }, // Turkmenistan
    {   96, 1763,       0,  179867,       0,  0, 13,  0, }, // Tuvalu
    {   96, 1777,  179880,  179894,  179919, 14, 25, 22, }, // Uruguay
    {   96, 1792,  179941,  179958,  179986, 17, 28, 25, }, // Uzbekistan
    {   96, 1803,  180011,  180025,  180050, 14, 25, 22, }, // Vanuatu
    {   96, 1811,       0,  180072,       0,  0, 16,  0, }, // Venezuela
    {   96, 1821,  180088,  180106,  180135, 18, 29, 26, }, // Vladivostok
    {   96, 1833,  180161,  180177,  180204, 16, 27, 24, }, // Volgograd
    {   96, 1843,       0,  180228,       0,  0, 13,  0, }, // Vostok
    {   96, 1850,       0,  180241,       0,  0, 11,  0, }, // Wake
    {   96, 1855,       0,  180252,       0,  0, 25,  0, }, // Wallis
    {   96, 1862,  180277,  180290,  180314, 13, 24, 21, }, // Yakutsk
    {   96, 1870,  180335,  180354,  180384, 19, 30, 27, }, // Yekaterinburg
    {   96, 1892,       0,  180411,       0,  0, 11,  0, }, // Yukon
    {   97,   17,       0,  180422,       0,  0, 28,  0, }, // Africa_Central Colognian/Latin/Germany
    {   97,   32,       0,  180450,       0,  0, 22,  0, }, // Africa_Eastern
    {   97,   65,       0,  180472,       0,  0, 23,  0, }, // Africa_Southern
    {   97,   81,  180495,  180518,  180553, 23, 35, 29, }, // Africa_Western
    {   97,  385,  180582,  180599,  180628, 17, 29, 23, }, // Azores
    {   97,  461,  180651,  180672,  180705, 21, 33, 27, }, // Cape_Verde
    {   97,  638,  180732,  180756,  180792, 24, 36, 30, }, // Europe_Central
    {   97,  653,  180822,  180842,  180874, 20, 32, 26, }, // Europe_Eastern
    {   97,  691,  180900,  180921,  180954, 21, 33, 27, }, // Europe_Western
    {   97,  799,       0,  180981,       0,  0, 27,  0, }, // GMT
    {   97,  931,       0,  181008,       0,  0, 29,  0, }, // Indian_Ocean
    {   97, 1264,  181037,  181055,  181085, 18, 30, 24, }, // Mauritius
    {   97, 1554,       0,  181109,       0,  0, 16,  0, }, // Reunion
    {   97, 1602,       0,  181125,       0,  0, 22,  0, }, // Seychelles
    {   99,  799,       0,  181147,       0,  0, 24,  0, }, // GMT Corsican/Latin/France
    {  100,    0,  181171,  181183,  181206, 12, 23, 19, }, // Acre Croatian/Latin/Croatia
    {  100,    5,       0,  181225,       0,  0, 21,  0, }, // Afghanistan
    {  100,   17,       0,  181246,       0,  0, 22,  0, }, // Africa_Central
    {  100,   32,       0,  181268,       0,  0, 22,  0, }, // Africa_Eastern
    {  100,   65,       0,  181290,       0,  0, 20,  0, }, // Africa_Southern
    {  100,   81,  181310,  181332,  181365, 22, 33, 29, }, // Africa_Western
    {  100,  107,  181394,  181409,  181435, 15, 26, 22, }, // Alaska
    {  100,  128,  181457,  181474,  181502, 17, 28, 24, }, // Almaty
    {  100,  135,  181526,  181543,  181571, 17, 28, 24, }, // Amazon
    {  100,  142,  181595,  181612,  181640, 17, 28, 24, }, // America_Central
    {  100,  158,   93124,   93156,   93199, 15, 26, 22, }, // America_Eastern
    {  100,  174,   93238,   93272,   93317, 17, 28, 24, }, // America_Mountain
    {  100,  191,   93358,   93392,   93437, 17, 28, 24, }, // America_Pacific
    {  100,  207,  181664,  181681,  181709, 17, 28, 24, }, // Anadyr
    {  100,  214,  181733,  181746,  181770, 13, 24, 20, }, // Apia
    {  100,  219,  181790,  181809,  181839, 19, 30, 26, }, // Aqtau
    {  100,  225,  181865,  181885,  181916, 20, 31, 27, }, // Aqtobe
    {  100,  232,  181943,  181958,  181984, 15, 26, 22, }, // Arabian
    {  100,  240,   93876,   93902,   93939, 19, 30, 26, }, // Argentina
    {  100,  250,  182006,  182032,  182069, 26, 37, 33, }, // Argentina_Western
    {  100,  268,  182102,  182118,  182145, 16, 27, 23, }, // Armenia
    {  100,  286,   94048,   94082,   94127, 17, 28, 24, }, // Atlantic
    {  100,  295,  182168,  182193,  182229, 25, 36, 32, }, // Australia_Central
    {  100,  313,  182261,  182297,  182344, 36, 47, 43, }, // Australia_CentralWestern
    {  100,  338,  182387,  182412,  182448, 25, 36, 32, }, // Australia_Eastern
    {  100,  356,  182480,  182505,  182541, 25, 36, 32, }, // Australia_Western
    {  100,  374,  182573,  182595,  182628, 22, 33, 29, }, // Azerbaijan
    {  100,  385,  182657,  182672,  182698, 15, 26, 22, }, // Azores
    {  100,  397,  182720,  182739,  182769, 19, 30, 26, }, // Bangladesh
    {  100,  415,       0,  182795,       0,  0, 16,  0, }, // Bhutan
    {  100,  422,       0,  182811,       0,  0, 18,  0, }, // Bolivia
    {  100,  437,  182829,  182846,  182874, 17, 28, 24, }, // Brasilia
    {  100,  454,       0,  182898,       0,  0, 28,  0, }, // Brunei
    {  100,  461,  182926,  182953,  182991, 27, 38, 34, }, // Cape_Verde
    {  100,  472,       0,  183025,       0,  0, 15,  0, }, // Casey
    {  100,  478,       0,  183040,       0,  0, 27,  0, }, // Chamorro
    {  100,  487,  183067,  183083,  183110, 16, 27, 23, }, // Chatham
    {  100,  495,  183133,  183150,  183178, 17, 28, 24, }, // Chile
    {  100,  501,   97012,  183202,  183228, 15, 26, 22, }, // China
    {  100,  507,  183250,  183271,  183303, 21, 32, 28, }, // Choibalsan
    {  100,  518,       0,  183331,       0,  0, 22,  0, }, // Christmas
    {  100,  528,       0,  183353,       0,  0, 23,  0, }, // Cocos
    {  100,  534,  183376,  183395,  183425, 19, 30, 26, }, // Colombia
    {  100,  543,  183451,  183473,  183506, 22, 33, 46, }, // Cook
    {  100,  548,  183552,  183568,  183595, 16, 27, 23, }, // Cuba
    {  100,  559,       0,  183618,       0,  0, 14,  0, }, // Davis
    {  100,  575,       0,  183632,       0,  0, 25,  0, }, // DumontDUrville
    {  100,  612,       0,  183657,       0,  0, 23,  0, }, // East_Timor
    {  100,  623,  183680,  183703,  183737, 23, 34, 30, }, // Easter
    {  100,  630,       0,  183767,       0,  0, 18,  0, }, // Ecuador
    {  100,  638,  183785,  183808,  183842, 23, 34, 30, }, // Europe_Central
    {  100,  653,  183872,  183895,  183929, 23, 34, 30, }, // Europe_Eastern
    {  100,  668,       0,  183959,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  100,  691,  183989,  184012,  184046, 23, 34, 30, }, // Europe_Western
    {  100,  706,  184076,  184095,  184125, 19, 30, 26, }, // Falkland
    {  100,  715,  184151,  184166,  184192, 15, 26, 22, }, // Fiji
    {  100,  720,       0,  184214,       0,  0, 24,  0, }, // French_Guiana
    {  100,  734,       0,  184238,       0,  0, 51,  0, }, // French_Southern
    {  100,  757,       0,  184289,       0,  0, 18,  0, }, // Galapagos
    {  100,  767,       0,  184307,       0,  0, 16,  0, }, // Gambier
    {  100,  775,  184323,  184340,  184368, 17, 28, 24, }, // Georgia
    {  100,  783,       0,  184392,       0,  0, 25,  0, }, // Gilbert_Islands
    {  100,  799,       0,  184417,       0,  0, 19,  0, }, // GMT
    {  100,  841,  184436,  184462,  184499, 26, 37, 33, }, // Greenland_Eastern
    {  100,  859,  184532,  184558,  184595, 26, 37, 33, }, // Greenland_Western
    {  100,  877,       0,  184628,       0,  0, 26,  0, }, // Guam
    {  100,  882,       0,  184654,       0,  0, 28,  0, }, // Gulf
    {  100,  887,       0,   96240,       0,  0, 17,  0, }, // Guyana
    {  100,  894,  184682,  184707,  184743, 25, 36, 32, }, // Hawaii_Aleutian
    {  100,  910,  184775,  184793,  184822, 18, 29, 25, }, // Hong_Kong
    {  100,  920,  184847,  184862,  184888, 15, 26, 22, }, // Hovd
    {  100,  925,       0,  184910,       0,  0, 16,  0, }, // India
    {  100,  931,       0,  184926,       0,  0, 24,  0, }, // Indian_Ocean
    {  100,  944,       0,  184950,       0,  0, 19,  0, }, // Indochina
    {  100,  954,       0,  184969,       0,  0, 27,  0, }, // Indonesia_Central
    {  100,  972,       0,  184996,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  100,  990,       0,  185023,       0,  0, 27,  0, }, // Indonesia_Western
    {  100, 1008,  185050,  185065,  185091, 15, 26, 22, }, // Iran
    {  100, 1019,  185113,  185129,  185156, 16, 27, 23, }, // Irkutsk
    {  100, 1027,  185179,  185196,  185224, 17, 28, 24, }, // Israel
    {  100, 1034,  185248,  185264,  185291, 16, 27, 23, }, // Japan
    {  100, 1040,  185314,  185345,  185387, 31, 42, 38, }, // Kamchatka
    {  100, 1058,       0,  185425,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  100, 1077,       0,  185452,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  100, 1106,  185479,  185495,  185522, 16, 27, 23, }, // Korea
    {  100, 1112,       0,  185545,       0,  0, 14,  0, }, // Kosrae
    {  100, 1119,  185559,  185579,  185610, 20, 31, 27, }, // Krasnoyarsk
    {  100, 1151,       0,  185637,       0,  0, 20,  0, }, // Kyrgystan
    {  100, 1161,       0,  185657,       0,  0, 17,  0, }, // Lanka
    {  100, 1175,       0,  185674,       0,  0, 25,  0, }, // Line_Islands
    {  100, 1188,  185699,  185722,  185756, 23, 34, 30, }, // Lord_Howe
    {  100, 1198,  185786,  185802,  185829, 16, 27, 23, }, // Macau
    {  100, 1204,  185852,  185870,  185899, 18, 29, 25, }, // Magadan
    {  100, 1219,       0,  185924,       0,  0, 18,  0, }, // Malaysia
    {  100, 1228,       0,  185942,       0,  0, 17,  0, }, // Maldives
    {  100, 1237,       0,  185959,       0,  0, 19,  0, }, // Marquesas
    {  100, 1247,       0,  185978,       0,  0, 24,  0, }, // Marshall_Islands
    {  100, 1264,  186002,  186021,  186051, 19, 30, 26, }, // Mauritius
    {  100, 1274,       0,  186077,       0,  0, 17,  0, }, // Mawson
    {  100, 1281,  186094,  186120,  186157, 26, 37, 33, }, // Mexico_Pacific
    {  100, 1296,  186190,  186210,  186241, 20, 31, 27, }, // Mongolia
    {  100, 1305,  186268,  186285,  186313, 17, 28, 24, }, // Moscow
    {  100, 1312,       0,  186337,       0,  0, 18,  0, }, // Myanmar
    {  100, 1320,       0,  186355,       0,  0, 14,  0, }, // Nauru
    {  100, 1326,       0,  186369,       0,  0, 16,  0, }, // Nepal
    {  100, 1332,  186385,  186408,  186442, 23, 34, 30, }, // New_Caledonia
    {  100, 1346,  186472,  186493,  186525, 21, 32, 28, }, // New_Zealand
    {  100, 1358,  186553,  186576,  186610, 23, 34, 30, }, // Newfoundland
    {  100, 1371,       0,  186640,       0,  0, 13,  0, }, // Niue
    {  100, 1376,  186653,  186674,  186706, 21, 32, 28, }, // Norfolk
    {  100, 1384,  186734,  186767,  186811, 33, 44, 40, }, // Noronha
    {  100, 1392,       0,  186851,       0,  0, 33,  0, }, // North_Mariana
    {  100, 1406,  186884,  186904,  186935, 20, 31, 27, }, // Novosibirsk
    {  100, 1418,  186962,  186975,  186999, 13, 24, 20, }, // Omsk
    {  100, 1428,  187019,  187038,  187068, 19, 30, 26, }, // Pakistan
    {  100, 1437,       0,  187094,       0,  0, 14,  0, }, // Palau
    {  100, 1443,       0,  187108,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  100, 1460,  187134,  187153,  187183, 19, 30, 26, }, // Paraguay
    {  100, 1469,  187209,  187226,  187254, 17, 28, 24, }, // Peru
    {  100, 1474,  187278,  187296,  187325, 18, 29, 25, }, // Philippines
    {  100, 1486,       0,  187350,       0,  0, 21,  0, }, // Phoenix_Islands
    {  100, 1502,  187371,  187403,  187446, 32, 43, 39, }, // Pierre_Miquelon
    {  100, 1518,       0,  187485,       0,  0, 19,  0, }, // Pitcairn
    {  100, 1527,       0,  187504,       0,  0, 18,  0, }, // Ponape
    {  100, 1534,       0,  187522,       0,  0, 19,  0, }, // Pyongyang
    {  100, 1544,  187541,  187564,  187598, 23, 34, 30, }, // Qyzylorda
    {  100, 1554,       0,  187628,       0,  0, 16,  0, }, // Reunion
    {  100, 1562,       0,  187644,       0,  0, 15,  0, }, // Rothera
    {  100, 1570,  187659,  187677,  187706, 18, 29, 25, }, // Sakhalin
    {  100, 1579,  187731,  187747,  187774, 16, 27, 23, }, // Samara
    {  100, 1596,  187797,  187814,  187842, 17, 28, 24, }, // Samoa
    {  100, 1602,       0,  187866,       0,  0, 17,  0, }, // Seychelles
    {  100, 1624,       0,  187883,       0,  0, 19,  0, }, // Singapore
    {  100, 1634,       0,  187902,       0,  0, 25,  0, }, // Solomon
    {  100, 1642,       0,  187927,       0,  0, 22,  0, }, // South_Georgia
    {  100, 1656,       0,  187949,       0,  0, 18,  0, }, // Suriname
    {  100, 1676,       0,  187967,       0,  0, 13,  0, }, // Syowa
    {  100, 1682,       0,  187980,       0,  0, 16,  0, }, // Tahiti
    {  100, 1689,  187996,  188012,  188039, 16, 27, 23, }, // Taipei
    {  100, 1696,       0,  188062,       0,  0, 22,  0, }, // Tajikistan
    {  100, 1724,       0,  188084,       0,  0, 16,  0, }, // Tokelau
    {  100, 1732,  188100,  188113,  188137, 13, 24, 20, }, // Tonga
    {  100, 1738,       0,  188157,       0,  0, 14,  0, }, // Truk
    {  100, 1750,  188171,  188194,  188228, 23, 34, 30, }, // Turkmenistan
    {  100, 1763,       0,  188258,       0,  0, 15,  0, }, // Tuvalu
    {  100, 1777,  188273,  188291,  188320, 18, 29, 25, }, // Uruguay
    {  100, 1792,  188345,  188366,  188398, 21, 32, 28, }, // Uzbekistan
    {  100, 1803,  188426,  188442,  188469, 16, 27, 23, }, // Vanuatu
    {  100, 1811,       0,  188492,       0,  0, 19,  0, }, // Venezuela
    {  100, 1821,  188511,  188532,  188564, 21, 32, 28, }, // Vladivostok
    {  100, 1833,  188592,  188612,  188643, 20, 31, 27, }, // Volgograd
    {  100, 1843,       0,  100756,       0,  0, 16,  0, }, // Vostok
    {  100, 1850,       0,  188670,       0,  0, 18,  0, }, // Wake
    {  100, 1855,       0,  188688,       0,  0, 29,  0, }, // Wallis
    {  100, 1862,  188717,  188733,  188760, 16, 27, 23, }, // Yakutsk
    {  100, 1870,  188783,  188806,  188840, 23, 34, 30, }, // Yekaterinburg
    {  100, 1892,       0,  188870,       0,  0, 16,  0, }, // Yukon
    {  102,    0,  188886,  188898,  188921, 12, 23, 18, }, // Acre Czech/Latin/Czechia
    {  102,    5,       0,  188939,       0,  0, 13,  0, }, // Afghanistan
    {  102,   17,       0,  188952,       0,  0, 17,  0, }, // Africa_Central
    {  102,   32,       0,  188969,       0,  0, 18,  0, }, // Africa_Eastern
    {  102,   65,       0,  188987,       0,  0, 15,  0, }, // Africa_Southern
    {  102,   81,  189002,  189019,  189047, 17, 28, 23, }, // Africa_Western
    {  102,  107,  189070,  189082,  189105, 12, 23, 18, }, // Alaska
    {  102,  128,  189123,  189135,  189158, 12, 23, 18, }, // Almaty
    {  102,  135,  189176,  189189,  189213, 13, 24, 19, }, // Amazon
    {  102,  142,  189232,  189260,  189299, 28, 39, 34, }, // America_Central
    {  102,  158,  189333,  189360,  189398, 27, 38, 33, }, // America_Eastern
    {  102,  174,  189431,  189456,  189492, 25, 36, 31, }, // America_Mountain
    {  102,  191,  189523,  189551,  189590, 28, 39, 34, }, // America_Pacific
    {  102,  207,  189624,  189637,  189661, 13, 24, 19, }, // Anadyr
    {  102,  214,  189680,  189691,  189713, 11, 22, 17, }, // Apia
    {  102,  219,  189730,  189742,  189765, 12, 23, 18, }, // Aqtau
    {  102,  225,  189783,  189795,  189818, 12, 23, 18, }, // Aqtobe
    {  102,  232,  189836,  189847,  189869, 11, 22, 17, }, // Arabian
    {  102,  240,  189886,  189901,  189927, 15, 26, 21, }, // Argentina
    {  102,  250,  189948,  189969,  190001, 21, 32, 27, }, // Argentina_Western
    {  102,  268,  190028,  190040,  190063, 12, 23, 18, }, // Armenia
    {  102,  286,  190081,  190095,  190120, 14, 25, 20, }, // Atlantic
    {  102,  295,  190140,  190160,  190191, 20, 31, 26, }, // Australia_Central
    {  102,  313,  190217,  190245,  190284, 28, 39, 34, }, // Australia_CentralWestern
    {  102,  338,  190318,  190339,  190371, 21, 32, 27, }, // Australia_Eastern
    {  102,  356,  190398,  190418,  190449, 20, 31, 26, }, // Australia_Western
    {  102,  374,  190475,  190493,  190522, 18, 29, 24, }, // Azerbaijan
    {  102,  385,  190546,  190557,  190579, 11, 22, 17, }, // Azores
    {  102,  397,  190596,  190612,  190639, 16, 27, 22, }, // Bangladesh
    {  102,  415,       0,  190661,       0,  0, 13,  0, }, // Bhutan
    {  102,  422,       0,  190674,       0,  0, 14,  0, }, // Bolivia
    {  102,  437,  190688,  190703,  190729, 15, 26, 21, }, // Brasilia
    {  102,  454,       0,  190750,       0,  0, 13,  0, }, // Brunei
    {  102,  461,  190763,  190777,  190802, 14, 25, 20, }, // Cape_Verde
    {  102,  472,       0,  190822,       0,  0, 19,  0, }, // Casey
    {  102,  478,       0,  190841,       0,  0, 14,  0, }, // Chamorro
    {  102,  487,  190855,  190869,  190894, 14, 25, 20, }, // Chatham
    {  102,  495,  190914,  190925,  190947, 11, 22, 17, }, // Chile
    {  102,  501,  190964,  190974,  190995, 10, 21, 16, }, // China
    {  102,  507,  191011,  191027,  191054, 16, 27, 22, }, // Choibalsan
    {  102,  518,       0,  191076,       0,  0, 21,  0, }, // Christmas
    {  102,  528,       0,  191097,       0,  0, 22,  0, }, // Cocos
    {  102,  534,  191119,  191134,  191160, 15, 26, 21, }, // Colombia
    {  102,  543,  191181,  191202,  191234, 21, 32, 27, }, // Cook
    {  102,  548,  191261,  191273,  191296, 12, 23, 18, }, // Cuba
    {  102,  559,       0,  191314,       0,  0, 20,  0, }, // Davis
    {  102,  575,       0,  191334,       0,  0, 29,  0, }, // DumontDUrville
    {  102,  612,       0,  191363,       0,  0, 19,  0, }, // East_Timor
    {  102,  623,  191382,  191407,  191443, 25, 36, 31, }, // Easter
    {  102,  630,       0,  191474,       0,  0, 14,  0, }, // Ecuador
    {  102,  638,  191488,  191506,  191535, 18, 29, 24, }, // Europe_Central
    {  102,  653,  191559,  191578,  191608, 19, 30, 25, }, // Europe_Eastern
    {  102,  668,       0,  191633,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  102,  691,  191657,  191675,  191704, 18, 29, 24, }, // Europe_Western
    {  102,  706,  191728,  191743,  191769, 15, 26, 21, }, // Falkland
    {  102,  715,  191790,  191803,  191827, 13, 24, 19, }, // Fiji
    {  102,  720,       0,  191846,       0,  0, 23,  0, }, // French_Guiana
    {  102,  734,       0,  191869,       0,  0, 47,  0, }, // French_Southern
    {  102,  757,       0,  191916,       0,  0, 14,  0, }, // Galapagos
    {  102,  767,       0,  191930,       0,  0, 14,  0, }, // Gambier
    {  102,  775,  191944,  191957,  191981, 13, 24, 19, }, // Georgia
    {  102,  783,       0,  192000,       0,  0, 24,  0, }, // Gilbert_Islands
    {  102,  799,       0,  192024,       0,  0, 24,  0, }, // GMT
    {  102,  841,  192048,  192066,  192095, 18, 29, 24, }, // Greenland_Eastern
    {  102,  859,  192119,  192136,  192164, 17, 28, 23, }, // Greenland_Western
    {  102,  877,       0,  192187,       0,  0, 11,  0, }, // Guam
    {  102,  882,       0,  192198,       0,  0, 30,  0, }, // Gulf
    {  102,  887,       0,  191857,       0,  0, 12,  0, }, // Guyana
    {  102,  894,  192228,  192249,  192281, 21, 32, 27, }, // Hawaii_Aleutian
    {  102,  910,  192308,  192323,  192349, 15, 26, 21, }, // Hong_Kong
    {  102,  920,  192370,  192381,  192403, 11, 22, 17, }, // Hovd
    {  102,  925,       0,  192420,       0,  0, 11,  0, }, // India
    {  102,  931,       0,  192431,       0,  0, 19,  0, }, // Indian_Ocean
    {  102,  944,       0,  192450,       0,  0, 14,  0, }, // Indochina
    {  102,  954,       0,  192464,       0,  0, 19,  0, }, // Indonesia_Central
    {  102,  972,       0,  192483,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  102,  990,       0,  192503,       0,  0, 19,  0, }, // Indonesia_Western
    {  102, 1008,  192522,  192533,  192555, 11, 22, 17, }, // Iran
    {  102, 1019,  192572,  192584,  192607, 12, 23, 18, }, // Irkutsk
    {  102, 1027,  192625,  192638,  192662, 13, 24, 19, }, // Israel
    {  102, 1034,  192681,  192693,  192716, 12, 23, 18, }, // Japan
    {  102, 1040,  192734,  192762,  192801, 28, 39, 34, }, // Kamchatka
    {  102, 1058,       0,  192835,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  102, 1077,       0,  192859,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  102, 1106,  192882,  192894,  192917, 12, 23, 18, }, // Korea
    {  102, 1112,       0,  192935,       0,  0, 13,  0, }, // Kosrae
    {  102, 1119,  192948,  192964,  192991, 16, 27, 22, }, // Krasnoyarsk
    {  102, 1151,       0,  193013,       0,  0, 13,  0, }, // Kyrgystan
    {  102, 1161,       0,  193026,       0,  0, 13,  0, }, // Lanka
    {  102, 1175,       0,  193039,       0,  0, 23,  0, }, // Line_Islands
    {  102, 1188,  193062,  193084,  193117, 22, 33, 28, }, // Lord_Howe
    {  102, 1198,  193145,  193157,  193180, 12, 23, 18, }, // Macau
    {  102, 1204,  193198,  193212,  193237, 14, 25, 20, }, // Magadan
    {  102, 1219,       0,  193257,       0,  0, 12,  0, }, // Malaysia
    {  102, 1228,       0,  193269,       0,  0, 14,  0, }, // Maldives
    {  102, 1237,       0,  193283,       0,  0, 13,  0, }, // Marquesas
    {  102, 1247,       0,  193296,       0,  0, 25,  0, }, // Marshall_Islands
    {  102, 1264,  193321,  193336,  193362, 15, 26, 21, }, // Mauritius
    {  102, 1274,       0,  193383,       0,  0, 21,  0, }, // Mawson
    {  102, 1281,  193404,  193425,  193457, 21, 32, 27, }, // Mexico_Pacific
    {  102, 1296,  193484,  193500,  193527, 16, 27, 22, }, // Mongolia
    {  102, 1305,  193549,  193562,  193586, 13, 24, 19, }, // Moscow
    {  102, 1312,       0,  193605,       0,  0, 14,  0, }, // Myanmar
    {  102, 1320,       0,  193619,       0,  0, 11,  0, }, // Nauru
    {  102, 1326,       0,  193630,       0,  0, 12,  0, }, // Nepal
    {  102, 1332,  193642,  193660,  193689, 18, 29, 24, }, // New_Caledonia
    {  102, 1346,  193713,  193730,  193758, 17, 28, 23, }, // New_Zealand
    {  102, 1358,  193781,  193800,  193830, 19, 30, 25, }, // Newfoundland
    {  102, 1371,       0,  193855,       0,  0, 12,  0, }, // Niue
    {  102, 1376,  193867,  193881,  193906, 14, 25, 20, }, // Norfolk
    {  102, 1384,  193926,  193960,  194005, 34, 45, 40, }, // Noronha
    {  102, 1392,       0,  194045,       0,  0, 19,  0, }, // North_Mariana
    {  102, 1406,  194064,  194080,  194107, 16, 27, 22, }, // Novosibirsk
    {  102, 1418,  194129,  194138,  194158,  9, 20, 15, }, // Omsk
    {  102, 1428,  194173,  194188,  194214, 15, 26, 21, }, // Pakistan
    {  102, 1437,       0,  194235,       0,  0, 12,  0, }, // Palau
    {  102, 1443,       0,  194247,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  102, 1460,  194268,  194283,  194309, 15, 26, 21, }, // Paraguay
    {  102, 1469,  194330,  194343,  194367, 13, 24, 19, }, // Peru
    {  102, 1474,  194386,  194400,  194425, 14, 25, 20, }, // Philippines
    {  102, 1486,       0,  194445,       0,  0, 22,  0, }, // Phoenix_Islands
    {  102, 1502,  194467,  194489,  194522, 22, 33, 28, }, // Pierre_Miquelon
    {  102, 1518,       0,  194550,       0,  0, 25,  0, }, // Pitcairn
    {  102, 1527,       0,  194575,       0,  0, 12,  0, }, // Ponape
    {  102, 1534,       0,  194587,       0,  0, 18,  0, }, // Pyongyang
    {  102, 1544,  194605,  194620,  194646, 15, 26, 21, }, // Qyzylorda
    {  102, 1554,       0,  194667,       0,  0, 14,  0, }, // Reunion
    {  102, 1562,       0,  194681,       0,  0, 21,  0, }, // Rothera
    {  102, 1570,  194702,  194717,  194743, 15, 26, 21, }, // Sakhalin
    {  102, 1579,  194764,  194776,  194799, 12, 23, 18, }, // Samara
    {  102, 1596,  194817,  194829,  194852, 12, 23, 18, }, // Samoa
    {  102, 1602,       0,  194870,       0,  0, 14,  0, }, // Seychelles
    {  102, 1624,       0,  194884,       0,  0, 15,  0, }, // Singapore
    {  102, 1634,       0,  194899,       0,  0, 25,  0, }, // Solomon
    {  102, 1642,       0,  194924,       0,  0, 17,  0, }, // South_Georgia
    {  102, 1656,       0,  194941,       0,  0, 14,  0, }, // Suriname
    {  102, 1676,       0,  194955,       0,  0, 16,  0, }, // Syowa
    {  102, 1682,       0,  194971,       0,  0, 12,  0, }, // Tahiti
    {  102, 1689,  194983,  194998,  195024, 15, 26, 21, }, // Taipei
    {  102, 1696,       0,  195045,       0,  0, 12,  0, }, // Tajikistan
    {  102, 1724,       0,  195057,       0,  0, 14,  0, }, // Tokelau
    {  102, 1732,  195071,  195082,  195104, 11, 22, 17, }, // Tonga
    {  102, 1738,       0,  195121,       0,  0, 12,  0, }, // Truk
    {  102, 1750,  195133,  195147,  195172, 14, 25, 20, }, // Turkmenistan
    {  102, 1763,       0,  195192,       0,  0, 12,  0, }, // Tuvalu
    {  102, 1777,  195204,  195218,  195243, 14, 25, 20, }, // Uruguay
    {  102, 1792,  195263,  195274,  195296, 11, 22, 17, }, // Uzbekistan
    {  102, 1803,  195313,  195326,  195350, 13, 24, 19, }, // Vanuatu
    {  102, 1811,       0,  195369,       0,  0, 15,  0, }, // Venezuela
    {  102, 1821,  195384,  195401,  195429, 17, 28, 23, }, // Vladivostok
    {  102, 1833,  195452,  195468,  195495, 16, 27, 22, }, // Volgograd
    {  102, 1843,       0,  195517,       0,  0, 18,  0, }, // Vostok
    {  102, 1850,       0,  195535,       0,  0, 16,  0, }, // Wake
    {  102, 1855,       0,  195551,       0,  0, 27,  0, }, // Wallis
    {  102, 1862,  195578,  195590,  195613, 12, 23, 18, }, // Yakutsk
    {  102, 1870,  195631,  195650,  195680, 19, 30, 25, }, // Yekaterinburg
    {  102, 1892,       0,  195705,       0,  0, 12,  0, }, // Yukon
    {  103,    0,  195717,  195725,  195739,  8, 14, 14, }, // Acre Danish/Latin/Denmark
    {  103,    5,       0,  195753,       0,  0, 12,  0, }, // Afghanistan
    {  103,   17,       0,  195765,       0,  0, 20,  0, }, // Africa_Central
    {  103,   32,       0,  195785,       0,  0, 16,  0, }, // Africa_Eastern
    {  103,   65,       0,  195801,       0,  0, 16,  0, }, // Africa_Southern
    {  103,   81,  195817,  195834,  195857, 17, 23, 23, }, // Africa_Western
    {  103,  107,  195880,  195890,  195906, 10, 16, 16, }, // Alaska
    {  103,  128,  195922,  195932,  195948, 10, 16, 16, }, // Almaty
    {  103,  135,  195964,  195976,  195994, 12, 18, 18, }, // Amazon
    {  103,  142,  196012,  196023,  196040, 11, 17, 17, }, // America_Central
    {  103,  158,  196057,  196068,  196085, 11, 17, 17, }, // America_Eastern
    {  103,  174,  196102,  196114,  196132, 12, 18, 18, }, // America_Mountain
    {  103,  191,  196150,  196161,  196178, 11, 17, 17, }, // America_Pacific
    {  103,  207,  196195,  196205,  196221, 10, 16, 16, }, // Anadyr
    {  103,  214,  196237,  196245,  196259,  8, 14, 14, }, // Apia
    {  103,  219,  196273,  196282,  196297,  9, 15, 15, }, // Aqtau
    {  103,  225,  196312,  196322,  196338, 10, 16, 16, }, // Aqtobe
    {  103,  232,  196354,  196365,  196382, 11, 17, 17, }, // Arabian
    {  103,  240,  196399,  196412,  196432, 13, 20, 20, }, // Argentina
    {  103,  250,  196452,  196470,  196494, 18, 24, 24, }, // Argentina_Western
    {  103,  268,  196518,  196529,  196546, 11, 17, 17, }, // Armenia
    {  103,  286,  196563,  196575,  196593, 12, 18, 18, }, // Atlantic
    {  103,  295,  196611,  196631,  196657, 20, 26, 26, }, // Australia_Central
    {  103,  313,  196683,  196711,  196745, 28, 34, 34, }, // Australia_CentralWestern
    {  103,  338,  196779,  196795,  196817, 16, 22, 22, }, // Australia_Eastern
    {  103,  356,  196839,  196856,  196879, 17, 23, 23, }, // Australia_Western
    {  103,  374,  196902,  196920,  196944, 18, 24, 24, }, // Azerbaijan
    {  103,  385,  196968,  196980,  196998, 12, 18, 18, }, // Azores
    {  103,  397,  197016,  197030,  197050, 14, 20, 20, }, // Bangladesh
    {  103,  415,       0,  197070,       0,  0, 10,  0, }, // Bhutan
    {  103,  422,       0,  197080,       0,  0, 14,  0, }, // Bolivia
    {  103,  437,  197094,  197109,  197130, 15, 21, 21, }, // Brasilia
    {  103,  454,       0,  197151,       0,  0, 21,  0, }, // Brunei
    {  103,  461,  197172,  197185,  197204, 13, 19, 19, }, // Cape_Verde
    {  103,  478,       0,  197223,       0,  0, 12,  0, }, // Chamorro
    {  103,  487,  197235,  197246,  197263, 11, 17, 17, }, // Chatham
    {  103,  495,  197280,  197292,  197310, 12, 18, 18, }, // Chile
    {  103,  501,  197328,  197340,  197358, 12, 18, 18, }, // China
    {  103,  507,  197376,  197391,  197412, 15, 21, 21, }, // Choibalsan
    {  103,  518,       0,  197433,       0,  0, 17,  0, }, // Christmas
    {  103,  528,       0,  197450,       0,  0, 20,  0, }, // Cocos
    {  103,  534,  197470,  197485,  197506, 15, 21, 21, }, // Colombia
    {  103,  543,  197527,  197540,  197559, 13, 19, 19, }, // Cook
    {  103,  548,  197578,  197589,  197606, 11, 17, 17, }, // Cuba
    {  103,  559,       0,  197623,       0,  0,  9,  0, }, // Davis
    {  103,  575,       0,  197632,       0,  0, 20,  0, }, // DumontDUrville
    {  103,  612,       0,  197652,       0,  0, 12,  0, }, // East_Timor
    {  103,  623,  197664,  197676,  197694, 12, 18, 18, }, // Easter
    {  103,  630,       0,  197712,       0,  0, 16,  0, }, // Ecuador
    {  103,  638,  197728,  197748,  197774, 20, 26, 26, }, // Europe_Central
    {  103,  653,  197800,  197816,  197838, 16, 22, 22, }, // Europe_Eastern
    {  103,  668,       0,  197860,       0,  0, 21,  0, }, // Europe_Further_Eastern
    {  103,  691,  197881,  197898,  197921, 17, 23, 23, }, // Europe_Western
    {  103,  706,  197944,  197962,  197986, 18, 24, 24, }, // Falkland
    {  103,  715,  198010,  198022,  198040, 12, 18, 18, }, // Fiji
    {  103,  720,       0,  198058,       0,  0, 17,  0, }, // French_Guiana
    {  103,  734,       0,  198075,       0,  0, 46,  0, }, // French_Southern
    {  103,  757,       0,  198121,       0,  0, 13,  0, }, // Galapagos
    {  103,  767,       0,  198134,       0,  0, 11,  0, }, // Gambier
    {  103,  775,  198145,  198157,  198175, 12, 18, 18, }, // Georgia
    {  103,  783,       0,  198193,       0,  0, 16,  0, }, // Gilbert_Islands
    {  103,  799,       0,  198209,       0,  0,  3,  0, }, // GMT
    {  103,  841,  198212,  198229,  198252, 17, 23, 23, }, // Greenland_Eastern
    {  103,  859,  198275,  198293,  198317, 18, 24, 24, }, // Greenland_Western
    {  103,  877,       0,  198341,       0,  0, 14,  0, }, // Guam
    {  103,  882,       0,  198355,       0,  0, 21,  0, }, // Gulf
    {  103,  887,       0,  198065,       0,  0, 10,  0, }, // Guyana
    {  103,  894,  198376,  198395,  198420, 19, 25, 25, }, // Hawaii_Aleutian
    {  103,  910,  198445,  198457,  198475, 12, 18, 18, }, // Hong_Kong
    {  103,  920,  198493,  198501,  198515,  8, 14, 14, }, // Hovd
    {  103,  925,       0,  198529,       0,  0, 16,  0, }, // India
    {  103,  931,       0,  198545,       0,  0, 23,  0, }, // Indian_Ocean
    {  103,  944,       0,  198568,       0,  0, 12,  0, }, // Indochina
    {  103,  954,       0,  198580,       0,  0, 21,  0, }, // Indonesia_Central
    {  103,  972,       0,  198601,       0,  0, 17,  0, }, // Indonesia_Eastern
    {  103,  990,       0,  198618,       0,  0, 18,  0, }, // Indonesia_Western
    {  103, 1008,  198636,  198646,  198662, 10, 16, 16, }, // Iran
    {  103, 1019,  198678,  198689,  198706, 11, 17, 17, }, // Irkutsk
    {  103, 1027,  198723,  198735,  198753, 12, 18, 18, }, // Israel
    {  103, 1034,  198771,  198782,  198799, 11, 17, 17, }, // Japan
    {  103, 1040,  198816,  198844,  198878, 28, 34, 34, }, // Kamchatka
    {  103, 1058,       0,  198912,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  103, 1077,       0,  198931,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  103, 1106,  198951,  198963,  198981, 12, 18, 18, }, // Korea
    {  103, 1112,       0,  198999,       0,  0, 10,  0, }, // Kosrae
    {  103, 1119,  199009,  199024,  199045, 15, 21, 21, }, // Krasnoyarsk
    {  103, 1151,       0,  199066,       0,  0, 13,  0, }, // Kyrgystan
    {  103, 1161,       0,  199079,       0,  0,  9,  0, }, // Lanka
    {  103, 1175,       0,  199088,       0,  0, 14,  0, }, // Line_Islands
    {  103, 1188,  199102,  199115,  199134, 13, 19, 19, }, // Lord_Howe
    {  103, 1198,  199153,  199162,  199177,  9, 15, 15, }, // Macau
    {  103, 1204,  199192,  199203,  199220, 11, 17, 17, }, // Magadan
    {  103, 1219,       0,  199237,       0,  0, 12,  0, }, // Malaysia
    {  103, 1228,       0,  199249,       0,  0, 14,  0, }, // Maldives
    {  103, 1237,       0,  199263,       0,  0, 13,  0, }, // Marquesas
    {  103, 1247,       0,  199276,       0,  0, 17,  0, }, // Marshall_Islands
    {  103, 1264,  199293,  199306,  199325, 13, 19, 19, }, // Mauritius
    {  103, 1274,       0,  199344,       0,  0, 10,  0, }, // Mawson
    {  103, 1281,  199354,  199375,  199402, 21, 27, 27, }, // Mexico_Pacific
    {  103, 1296,  199429,  199443,  199463, 14, 20, 20, }, // Mongolia
    {  103, 1305,  199483,  199493,  199509, 10, 16, 16, }, // Moscow
    {  103, 1312,       0,  199525,       0,  0, 11,  0, }, // Myanmar
    {  103, 1320,       0,  199536,       0,  0,  9,  0, }, // Nauru
    {  103, 1326,       0,  199545,       0,  0, 14,  0, }, // Nepal
    {  103, 1332,  199559,  199576,  199599, 17, 23, 23, }, // New_Caledonia
    {  103, 1346,  199622,  199638,  199660, 16, 22, 22, }, // New_Zealand
    {  103, 1358,  199682,  199700,  199724, 18, 24, 24, }, // Newfoundland
    {  103, 1371,       0,  199748,       0,  0,  8,  0, }, // Niue
    {  103, 1376,  199756,  199774,  199798, 18, 24, 24, }, // Norfolk
    {  103, 1384,  199822,  199845,  199874, 23, 29, 29, }, // Noronha
    {  103, 1392,       0,  199903,       0,  0, 18,  0, }, // North_Mariana
    {  103, 1406,  199921,  199936,  199957, 15, 21, 21, }, // Novosibirsk
    {  103, 1418,  199978,  199986,  200000,  8, 14, 14, }, // Omsk
    {  103, 1428,  200014,  200028,  200048, 14, 20, 20, }, // Pakistan
    {  103, 1437,       0,  200068,       0,  0,  9,  0, }, // Palau
    {  103, 1443,       0,  200077,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  103, 1460,  200096,  200112,  200134, 16, 22, 22, }, // Paraguay
    {  103, 1469,  200156,  200170,  200190, 14, 20, 20, }, // Peru
    {  103, 1474,  200210,  200224,  200244, 14, 20, 20, }, // Philippines
    {  103, 1486,       0,  200264,       0,  0, 14,  0, }, // Phoenix_Islands
    {  103, 1502,  200278,  200307,  200342, 29, 35, 35, }, // Pierre_Miquelon
    {  103, 1518,       0,  200377,       0,  0, 12,  0, }, // Pitcairn
    {  103, 1527,       0,  200389,       0,  0, 10,  0, }, // Ponape
    {  103, 1534,       0,  200399,       0,  0, 13,  0, }, // Pyongyang
    {  103, 1544,  200412,  200425,  200444, 13, 19, 19, }, // Qyzylorda
    {  103, 1554,       0,  200463,       0,  0, 11,  0, }, // Reunion
    {  103, 1562,       0,  200474,       0,  0, 11,  0, }, // Rothera
    {  103, 1570,  200485,  200497,  200515, 12, 18, 18, }, // Sakhalin
    {  103, 1579,  200533,  200543,  200559, 10, 16, 16, }, // Samara
    {  103, 1596,  200575,  200587,  200605, 12, 18, 18, }, // Samoa
    {  103, 1602,       0,  200623,       0,  0, 15,  0, }, // Seychelles
    {  103, 1624,       0,  200638,       0,  0, 13,  0, }, // Singapore
    {  103, 1634,       0,  200651,       0,  0, 16,  0, }, // Solomon
    {  103, 1642,       0,  200667,       0,  0, 17,  0, }, // South_Georgia
    {  103, 1656,       0,  200684,       0,  0, 11,  0, }, // Suriname
    {  103, 1676,       0,  200695,       0,  0,  9,  0, }, // Syowa
    {  103, 1682,       0,  200704,       0,  0, 10,  0, }, // Tahiti
    {  103, 1689,  200714,  200724,  200740, 10, 16, 16, }, // Taipei
    {  103, 1696,       0,  200756,       0,  0, 14,  0, }, // Tajikistan
    {  103, 1724,       0,  200770,       0,  0, 11,  0, }, // Tokelau
    {  103, 1732,  200781,  200793,  200811, 12, 18, 18, }, // Tonga
    {  103, 1738,       0,  200829,       0,  0,  9,  0, }, // Truk
    {  103, 1750,  200838,  200851,  200870, 13, 19, 19, }, // Turkmenistan
    {  103, 1763,       0,  200889,       0,  0, 10,  0, }, // Tuvalu
    {  103, 1777,  200899,  200914,  200935, 15, 21, 21, }, // Uruguay
    {  103, 1792,  200956,  200968,  200986, 12, 18, 18, }, // Uzbekistan
    {  103, 1803,  201004,  201015,  201032, 11, 17, 17, }, // Vanuatu
    {  103, 1811,       0,  201049,       0,  0, 16,  0, }, // Venezuela
    {  103, 1821,  201065,  201080,  201101, 15, 21, 21, }, // Vladivostok
    {  103, 1833,  201122,  201135,  201154, 13, 19, 19, }, // Volgograd
    {  103, 1843,       0,  201173,       0,  0, 10,  0, }, // Vostok
    {  103, 1850,       0,  201183,       0,  0, 15,  0, }, // Wake
    {  103, 1855,       0,  201198,       0,  0, 20,  0, }, // Wallis
    {  103, 1862,  201218,  201229,  201246, 11, 17, 17, }, // Yakutsk
    {  103, 1870,  201263,  201280,  201303, 17, 23, 23, }, // Yekaterinburg
    {  103, 1892,       0,  201326,       0,  0,  9,  0, }, // Yukon
    {  106,  142,  201335,  201360,  201390, 25, 30, 32, }, // America_Central Dogri/Devanagari/India
    {  106,  158,  201422,  201447,  201477, 25, 30, 32, }, // America_Eastern
    {  106,  174,  201509,  201535,  201566, 26, 31, 33, }, // America_Mountain
    {  106,  191,  201599,  201625,  201656, 26, 31, 33, }, // America_Pacific
    {  106,  286,  201689,  201702,  201720, 13, 18, 20, }, // Atlantic
    {  106,  638,  201740,  201758,  201781, 18, 23, 30, }, // Europe_Central
    {  106,  653,  201811,  201829,  201852, 18, 23, 30, }, // Europe_Eastern
    {  106,  691,  201882,  201900,  201923, 18, 23, 30, }, // Europe_Western
    {  106,  799,       0,   86447,       0,  0, 17,  0, }, // GMT
    {  108,    0,  201953,  201962,  201980,  9, 18, 14, }, // Acre Dutch/Latin/Netherlands
    {  108,    5,       0,  201994,       0,  0, 14,  0, }, // Afghanistan
    {  108,   17,       0,  202008,       0,  0, 24,  0, }, // Africa_Central
    {  108,   32,       0,  202032,       0,  0, 20,  0, }, // Africa_Eastern
    {  108,   65,       0,  202052,       0,  0, 20,  0, }, // Africa_Southern
    {  108,   81,  202072,  202092,  202121, 20, 29, 25, }, // Africa_Western
    {  108,  107,  202146,  202157,  202177, 11, 20, 16, }, // Alaska
    {  108,  128,  202193,  202206,  202228, 13, 22, 18, }, // Almaty
    {  108,  135,  202246,  202258,  202279, 12, 21, 17, }, // Amazon
    {  108,  142,  202296,  202308,  202329, 12, 21, 17, }, // America_Central
    {  108,  158,  202346,  202358,  202379, 12, 21, 17, }, // America_Eastern
    {  108,  174,  202396,  202409,  202431, 13, 22, 18, }, // America_Mountain
    {  108,  191,  202449,  202461,  202482, 12, 21, 17, }, // America_Pacific
    {  108,  207,  202499,  202510,  202530, 11, 20, 16, }, // Anadyr
    {  108,  214,  202546,  202555,  202573,  9, 18, 14, }, // Apia
    {  108,  219,  202587,  202597,  202616, 10, 19, 15, }, // Aqtau
    {  108,  225,  202631,  202642,  202662, 11, 20, 16, }, // Aqtobe
    {  108,  232,  202678,  202692,  202715, 14, 23, 19, }, // Arabian
    {  108,  240,  202734,  202750,  202775, 16, 25, 21, }, // Argentina
    {  108,  250,  202796,  202817,  202847, 21, 30, 26, }, // Argentina_Western
    {  108,  268,  202873,  202886,  202908, 13, 22, 18, }, // Armenia
    {  108,  286,  202926,  202939,  202961, 13, 22, 18, }, // Atlantic
    {  108,  295,  202979,  203003,  203036, 24, 33, 29, }, // Australia_Central
    {  108,  313,  203065,  203100,  203144, 35, 44, 40, }, // Australia_CentralWestern
    {  108,  338,  203184,  203206,  203237, 22, 31, 27, }, // Australia_Eastern
    {  108,  356,  203264,  203286,  203317, 22, 31, 27, }, // Australia_Western
    {  108,  374,  203344,  203364,  203393, 20, 29, 25, }, // Azerbaijan
    {  108,  385,  203418,  203429,  203449, 11, 20, 16, }, // Azores
    {  108,  397,  203465,  203479,  203502, 14, 23, 19, }, // Bangladesh
    {  108,  415,       0,  203521,       0,  0, 14,  0, }, // Bhutan
    {  108,  422,       0,  203535,       0,  0, 16,  0, }, // Bolivia
    {  108,  437,  203551,  203568,  203594, 17, 26, 22, }, // Brasilia
    {  108,  454,       0,  203616,       0,  0, 13,  0, }, // Brunei
    {  108,  461,  203629,  203647,  203674, 18, 27, 23, }, // Cape_Verde
    {  108,  472,       0,  203697,       0,  0, 10,  0, }, // Casey
    {  108,  478,       0,  203707,       0,  0, 13,  0, }, // Chamorro
    {  108,  487,  203720,  203732,  203753, 12, 21, 17, }, // Chatham
    {  108,  495,  203770,  203784,  203807, 14, 23, 19, }, // Chile
    {  108,  501,  203826,  203838,  203859, 12, 21, 17, }, // China
    {  108,  507,  203876,  203892,  203917, 16, 25, 21, }, // Choibalsan
    {  108,  518,       0,  203938,       0,  0, 22,  0, }, // Christmas
    {  108,  528,       0,  203960,       0,  0, 18,  0, }, // Cocos
    {  108,  534,  203978,  203995,  204021, 17, 26, 22, }, // Colombia
    {  108,  543,  204043,  204060,  204086, 17, 26, 28, }, // Cook
    {  108,  548,  204114,  204127,  204149, 13, 22, 18, }, // Cuba
    {  108,  559,       0,  204167,       0,  0, 10,  0, }, // Davis
    {  108,  575,       0,  204177,       0,  0, 21,  0, }, // DumontDUrville
    {  108,  612,       0,  204198,       0,  0, 18,  0, }, // East_Timor
    {  108,  623,  204216,  204233,  204259, 17, 26, 22, }, // Easter
    {  108,  630,       0,  204281,       0,  0, 17,  0, }, // Ecuador
    {  108,  638,  204298,  204318,  204347, 20, 29, 25, }, // Europe_Central
    {  108,  653,  204372,  204390,  204417, 18, 27, 23, }, // Europe_Eastern
    {  108,  668,       0,  204440,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  108,  691,  204470,  204488,  204515, 18, 27, 23, }, // Europe_Western
    {  108,  706,  204538,  204559,  204589, 21, 30, 26, }, // Falkland
    {  108,  715,  204615,  204628,  204650, 13, 22, 18, }, // Fiji
    {  108,  720,       0,  204668,       0,  0, 19,  0, }, // French_Guiana
    {  108,  734,       0,  204687,       0,  0, 38,  0, }, // French_Southern
    {  108,  757,       0,  204725,       0,  0, 22,  0, }, // Galapagos
    {  108,  767,       0,  204747,       0,  0, 20,  0, }, // Gambier
    {  108,  775,  204767,  204782,  204806, 15, 24, 20, }, // Georgia
    {  108,  783,       0,  204826,       0,  0, 20,  0, }, // Gilbert_Islands
    {  108,  799,       0,    2861,       0,  0, 19,  0, }, // GMT
    {  108,  841,  204846,  204867,  204897, 21, 30, 26, }, // Greenland_Eastern
    {  108,  859,  204923,  204944,  204974, 21, 30, 26, }, // Greenland_Western
    {  108,  877,       0,  205000,       0,  0, 21,  0, }, // Guam
    {  108,  882,       0,  205021,       0,  0, 18,  0, }, // Gulf
    {  108,  887,       0,  204674,       0,  0, 13,  0, }, // Guyana
    {  108,  894,  205039,  205062,  205094, 23, 32, 28, }, // Hawaii_Aleutian
    {  108,  910,  205122,  205137,  205161, 15, 24, 20, }, // Hong_Kong
    {  108,  920,  205181,  205190,  205208,  9, 18, 14, }, // Hovd
    {  108,  925,       0,  205222,       0,  0, 12,  0, }, // India
    {  108,  931,       0,  205234,       0,  0, 20,  0, }, // Indian_Ocean
    {  108,  944,       0,  205254,       0,  0, 16,  0, }, // Indochina
    {  108,  954,       0,  205270,       0,  0, 26,  0, }, // Indonesia_Central
    {  108,  972,       0,  205296,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  108,  990,       0,  205318,       0,  0, 22,  0, }, // Indonesia_Western
    {  108, 1008,  205340,  205352,  205373, 12, 21, 17, }, // Iran
    {  108, 1019,  205390,  205403,  205425, 13, 22, 18, }, // Irkutsk
    {  108, 1027,  205443,  205459,  205484, 16, 25, 21, }, // Israel
    {  108, 1034,  205505,  205517,  205538, 12, 21, 17, }, // Japan
    {  108, 1040,  205555,  205585,  205624, 30, 39, 35, }, // Kamchatka
    {  108, 1058,       0,  205659,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  108, 1077,       0,  205677,       0,  0, 18,  0, }, // Kazakhstan_Western
    {  108, 1106,  205695,  205709,  205732, 14, 23, 19, }, // Korea
    {  108, 1112,       0,  205751,       0,  0, 13,  0, }, // Kosrae
    {  108, 1119,  205764,  205780,  205805, 16, 25, 21, }, // Krasnoyarsk
    {  108, 1151,       0,  205826,       0,  0, 16,  0, }, // Kyrgystan
    {  108, 1161,       0,  205842,       0,  0, 10,  0, }, // Lanka
    {  108, 1175,       0,  205852,       0,  0, 18,  0, }, // Line_Islands
    {  108, 1188,  205870,  205893,  205925, 23, 32, 28, }, // Lord_Howe
    {  108, 1198,  205953,  205965,  205986, 12, 21, 17, }, // Macau
    {  108, 1204,  206003,  206015,  206036, 12, 21, 17, }, // Magadan
    {  108, 1219,       0,  206053,       0,  0, 16,  0, }, // Malaysia
    {  108, 1228,       0,  206069,       0,  0, 16,  0, }, // Maldives
    {  108, 1237,       0,  206085,       0,  0, 22,  0, }, // Marquesas
    {  108, 1247,       0,  206107,       0,  0, 21,  0, }, // Marshall_Islands
    {  108, 1264,  206128,  206145,  206171, 17, 26, 22, }, // Mauritius
    {  108, 1274,       0,  206193,       0,  0, 11,  0, }, // Mawson
    {  108, 1281,  206204,  206227,  206259, 23, 32, 28, }, // Mexico_Pacific
    {  108, 1296,  206287,  206303,  206328, 16, 25, 21, }, // Mongolia
    {  108, 1305,  206349,  206360,  206380, 11, 20, 16, }, // Moscow
    {  108, 1312,       0,  206396,       0,  0, 15,  0, }, // Myanmar
    {  108, 1320,       0,  206411,       0,  0, 15,  0, }, // Nauru
    {  108, 1326,       0,  206426,       0,  0, 13,  0, }, // Nepal
    {  108, 1332,  206439,  206462,  206494, 23, 32, 28, }, // New_Caledonia
    {  108, 1346,  206522,  206542,  206571, 20, 29, 25, }, // New_Zealand
    {  108, 1358,  206596,  206613,  206639, 17, 26, 22, }, // Newfoundland
    {  108, 1371,       0,  206661,       0,  0, 11,  0, }, // Niue
    {  108, 1376,  206672,  206692,  206721, 20, 29, 25, }, // Norfolk
    {  108, 1384,  206746,  206770,  206803, 24, 33, 29, }, // Noronha
    {  108, 1392,       0,  206832,       0,  0, 25,  0, }, // North_Mariana
    {  108, 1406,  206857,  206873,  206898, 16, 25, 21, }, // Novosibirsk
    {  108, 1418,  206919,  206928,  206946,  9, 18, 14, }, // Omsk
    {  108, 1428,  206960,  206976,  207001, 16, 25, 21, }, // Pakistan
    {  108, 1437,       0,  207022,       0,  0, 12,  0, }, // Palau
    {  108, 1443,       0,  207034,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  108, 1460,  207060,  207078,  207105, 18, 27, 23, }, // Paraguay
    {  108, 1469,  207128,  207142,  207165, 14, 23, 19, }, // Peru
    {  108, 1474,  207184,  207199,  207223, 15, 24, 20, }, // Philippines
    {  108, 1486,       0,  207243,       0,  0, 20,  0, }, // Phoenix_Islands
    {  108, 1502,  207263,  207292,  207330, 29, 38, 34, }, // Pierre_Miquelon
    {  108, 1518,       0,  207364,       0,  0, 21,  0, }, // Pitcairn
    {  108, 1527,       0,  207385,       0,  0, 12,  0, }, // Ponape
    {  108, 1534,       0,  207397,       0,  0, 14,  0, }, // Pyongyang
    {  108, 1544,  207411,  207425,  207448, 14, 23, 19, }, // Qyzylorda
    {  108, 1554,       0,  207467,       0,  0, 14,  0, }, // Reunion
    {  108, 1562,       0,  207481,       0,  0, 12,  0, }, // Rothera
    {  108, 1570,  207493,  207506,  207528, 13, 22, 18, }, // Sakhalin
    {  108, 1579,  207546,  207557,  207577, 11, 20, 16, }, // Samara
    {  108, 1596,  207593,  207607,  207630, 14, 23, 19, }, // Samoa
    {  108, 1602,       0,  207649,       0,  0, 14,  0, }, // Seychelles
    {  108, 1624,       0,  207663,       0,  0, 25,  0, }, // Singapore
    {  108, 1634,       0,  207688,       0,  0, 21,  0, }, // Solomon
    {  108, 1642,       0,  207709,       0,  0, 20,  0, }, // South_Georgia
    {  108, 1656,       0,  207729,       0,  0, 15,  0, }, // Suriname
    {  108, 1676,       0,  207744,       0,  0, 10,  0, }, // Syowa
    {  108, 1682,       0,  207754,       0,  0, 16,  0, }, // Tahiti
    {  108, 1689,  207770,  207781,  207801, 11, 20, 16, }, // Taipei
    {  108, 1696,       0,  207817,       0,  0, 15,  0, }, // Tajikistan
    {  108, 1724,       0,  207832,       0,  0, 21,  0, }, // Tokelau
    {  108, 1732,  207853,  207867,  207890, 14, 23, 19, }, // Tonga
    {  108, 1738,       0,  207909,       0,  0, 12,  0, }, // Truk
    {  108, 1750,  207921,  207936,  207960, 15, 24, 20, }, // Turkmenistan
    {  108, 1763,       0,  207980,       0,  0, 16,  0, }, // Tuvalu
    {  108, 1777,  207996,  208013,  208039, 17, 26, 22, }, // Uruguay
    {  108, 1792,  208061,  208075,  208098, 14, 23, 19, }, // Uzbekistan
    {  108, 1803,  208117,  208134,  208160, 17, 26, 22, }, // Vanuatu
    {  108, 1811,       0,  208182,       0,  0, 17,  0, }, // Venezuela
    {  108, 1821,  208199,  208215,  208240, 16, 25, 21, }, // Vladivostok
    {  108, 1833,  208261,  208275,  208298, 14, 23, 19, }, // Volgograd
    {  108, 1843,       0,  208317,       0,  0, 11,  0, }, // Vostok
    {  108, 1850,       0,  208328,       0,  0, 18,  0, }, // Wake
    {  108, 1855,       0,  208346,       0,  0, 23,  0, }, // Wallis
    {  108, 1862,  208369,  208382,  208404, 13, 22, 18, }, // Yakutsk
    {  108, 1870,  208422,  208442,  208471, 20, 29, 25, }, // Yekaterinburg
    {  108, 1892,       0,  208496,       0,  0, 10,  0, }, // Yukon
    {  114, 1656,       0,  207729,       0,  0, 15,  0, }, // Suriname Dutch/Latin/Suriname
    {  115,    5,       0,  208506,       0,  0, 19,  0, }, // Afghanistan Dzongkha/Tibetan/Bhutan
    {  115,   17,       0,  208525,       0,  0, 26,  0, }, // Africa_Central
    {  115,   32,       0,  208551,       0,  0, 24,  0, }, // Africa_Eastern
    {  115,   65,       0,  208575,       0,  0, 25,  0, }, // Africa_Southern
    {  115,   81,  208600,  208625,  208657, 25, 32, 34, }, // Africa_Western
    {  115,  107,  208691,  208705,  208726, 14, 21, 23, }, // Alaska
    {  115,  135,  208749,  208765,  208788, 16, 23, 25, }, // Amazon
    {  115,  142,  208813,  208844,  208882, 31, 38, 40, }, // America_Central
    {  115,  158,  208922,  208951,  208987, 29, 36, 38, }, // America_Eastern
    {  115,  174,  209025,  209053,  209088, 28, 35, 37, }, // America_Mountain
    {  115,  191,  209125,  209155,  209192, 30, 37, 39, }, // America_Pacific
    {  115,  232,  209231,  209249,  209274, 18, 25, 23, }, // Arabian
    {  115,  240,  209297,  209315,  209340, 18, 25, 27, }, // Argentina
    {  115,  250,  209367,  209395,  209430, 28, 35, 37, }, // Argentina_Western
    {  115,  268,  209467,  209484,  209508, 17, 24, 26, }, // Armenia
    {  115,  286,  209534,  209552,  209577, 18, 25, 27, }, // Atlantic
    {  115,  295,  209604,  209636,  209675, 32, 39, 41, }, // Australia_Central
    {  115,  313,  209716,  209751,  209794, 35, 43, 45, }, // Australia_CentralWestern
    {  115,  338,  209839,  209875,  209918, 36, 43, 45, }, // Australia_Eastern
    {  115,  356,  209720,  209756,  209799, 31, 38, 40, }, // Australia_Western
    {  115,  374,  209963,  209983,  210010, 20, 27, 29, }, // Azerbaijan
    {  115,  385,  210039,  210053,  210074, 14, 21, 23, }, // Azores
    {  115,  397,  210097,  210113,  210136, 16, 23, 25, }, // Bangladesh
    {  115,  415,       0,  210161,       0,  0, 16,  0, }, // Bhutan
    {  115,  422,       0,  210177,       0,  0, 18,  0, }, // Bolivia
    {  115,  437,  210195,  210212,  210236, 17, 24, 26, }, // Brasilia
    {  115,  461,  210262,  210276,  210297, 14, 21, 23, }, // Cape_Verde
    {  115,  495,  210320,  210332,  210351, 12, 19, 21, }, // Chile
    {  115,  501,  210372,  210385,  210405, 13, 20, 22, }, // China
    {  115,  518,       0,  210427,       0,  0, 27,  0, }, // Christmas
    {  115,  534,  210454,  210473,  210499, 19, 26, 28, }, // Colombia
    {  115,  548,  210527,  210541,  210562, 14, 21, 23, }, // Cuba
    {  115,  623,  210585,  210609,  210640, 24, 31, 33, }, // Easter
    {  115,  630,       0,  210673,       0,  0, 15,  0, }, // Ecuador
    {  115,  638,  210688,  210715,  210749, 27, 34, 36, }, // Europe_Central
    {  115,  653,  210785,  210810,  210842, 25, 32, 34, }, // Europe_Eastern
    {  115,  691,  210876,  210902,  210935, 26, 33, 35, }, // Europe_Western
    {  115,  706,  210970,  210996,  211029, 26, 33, 35, }, // Falkland
    {  115,  720,       0,  211064,       0,  0, 20,  0, }, // French_Guiana
    {  115,  757,       0,  211084,       0,  0, 17,  0, }, // Galapagos
    {  115,  775,  211101,  211113,  211132, 12, 19, 21, }, // Georgia
    {  115,  799,       0,  211153,       0,  0, 27,  0, }, // GMT
    {  115,  841,  211180,  211206,  211239, 26, 33, 35, }, // Greenland_Eastern
    {  115,  859,  211274,  211301,  211335, 27, 34, 36, }, // Greenland_Western
    {  115,  882,       0,  211371,       0,  0, 12,  0, }, // Gulf
    {  115,  887,       0,  211383,       0,  0, 13,  0, }, // Guyana
    {  115,  894,  211396,  211421,  211453, 25, 32, 34, }, // Hawaii_Aleutian
    {  115,  925,       0,  211487,       0,  0, 13,  0, }, // India
    {  115,  931,       0,  211500,       0,  0, 25,  0, }, // Indian_Ocean
    {  115,  944,       0,  211525,       0,  0, 20,  0, }, // Indochina
    {  115,  954,       0,  211545,       0,  0, 32,  0, }, // Indonesia_Central
    {  115,  972,       0,  211577,       0,  0, 30,  0, }, // Indonesia_Eastern
    {  115,  990,       0,  211607,       0,  0, 31,  0, }, // Indonesia_Western
    {  115, 1008,  211638,  211651,  211671, 13, 20, 22, }, // Iran
    {  115, 1019,  211693,  211707,  211728, 14, 21, 23, }, // Irkutsk
    {  115, 1027,  211751,  211765,  211786, 14, 21, 23, }, // Israel
    {  115, 1034,  211809,  211821,  211840, 12, 19, 21, }, // Japan
    {  115, 1106,  211861,  211875,  211896, 14, 21, 23, }, // Korea
    {  115, 1119,  211919,  211939,  211966, 20, 27, 29, }, // Krasnoyarsk
    {  115, 1204,  211995,  212010,  212032, 15, 22, 24, }, // Magadan
    {  115, 1228,       0,  212056,       0,  0, 15,  0, }, // Maldives
    {  115, 1264,  212071,  212087,  212110, 16, 23, 25, }, // Mauritius
    {  115, 1305,  212135,  212148,  212168, 13, 20, 22, }, // Moscow
    {  115, 1326,       0,  212190,       0,  0, 13,  0, }, // Nepal
    {  115, 1346,  212203,  212222,  212248, 19, 26, 28, }, // New_Zealand
    {  115, 1358,  212276,  212298,  212327, 22, 29, 31, }, // Newfoundland
    {  115, 1384,  212358,  212389,  212427, 31, 38, 40, }, // Noronha
    {  115, 1406,  212467,  212489,  212518, 22, 29, 31, }, // Novosibirsk
    {  115, 1418,  212549,  212562,  212582, 13, 20, 22, }, // Omsk
    {  115, 1428,  212604,  212621,  212645, 17, 24, 26, }, // Pakistan
    {  115, 1460,  212671,  212688,  212712, 17, 24, 26, }, // Paraguay
    {  115, 1469,  212738,  212749,  212767, 11, 18, 20, }, // Peru
    {  115, 1502,  212787,  212814,  212848, 27, 34, 36, }, // Pierre_Miquelon
    {  115, 1554,       0,  212884,       0,  0, 19,  0, }, // Reunion
    {  115, 1570,  212903,  212917,  212938, 14, 21, 23, }, // Sakhalin
    {  115, 1602,       0,  212961,       0,  0, 14,  0, }, // Seychelles
    {  115, 1656,       0,  212975,       0,  0, 16,  0, }, // Suriname
    {  115, 1777,  212991,  213011,  213038, 20, 27, 29, }, // Uruguay
    {  115, 1811,       0,  213067,       0,  0, 20,  0, }, // Venezuela
    {  115, 1821,  213087,  213109,  213138, 22, 29, 31, }, // Vladivostok
    {  115, 1833,  213169,  213189,  213216, 20, 27, 29, }, // Volgograd
    {  115, 1862,  213245,  213260,  213282, 15, 22, 24, }, // Yakutsk
    {  115, 1870,  213306,  213328,  213357, 22, 29, 31, }, // Yekaterinburg
    {  117,    0,       0,       9,      27,  9, 18, 16, }, // Acre English/Latin/United States
    {  117,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan
    {  117,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central
    {  117,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  117,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  117,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  117,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  117,  128,     235,     246,     266, 11, 20, 18, }, // Almaty
    {  117,  135,     284,     295,     315, 11, 20, 18, }, // Amazon
    {  117,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  117,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  117,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  117,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  117,  207,     552,     563,     583, 11, 20, 18, }, // Anadyr
    {  117,  214,     601,     610,     628,  9, 18, 18, }, // Apia
    {  117,  219,     646,     656,     675, 10, 19, 17, }, // Aqtau
    {  117,  225,     692,     703,     723, 11, 20, 18, }, // Aqtobe
    {  117,  232,     741,     753,     774, 12, 21, 21, }, // Arabian
    {  117,  240,     795,     809,     832, 14, 23, 21, }, // Argentina
    {  117,  250,     853,     875,     906, 22, 31, 29, }, // Argentina_Western
    {  117,  268,     935,     947,     968, 12, 21, 19, }, // Armenia
    {  117,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  117,  295,    1044,    1066,    1098, 22, 32, 32, }, // Australia_Central
    {  117,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {  117,  338,    1241,    1263,    1295, 22, 32, 32, }, // Australia_Eastern
    {  117,  356,    1327,    1349,    1381, 22, 32, 32, }, // Australia_Western
    {  117,  374,    1413,    1428,    1452, 15, 24, 22, }, // Azerbaijan
    {  117,  385,    1474,    1485,    1505, 11, 20, 18, }, // Azores
    {  117,  397,    1523,    1538,    1562, 15, 24, 22, }, // Bangladesh
    {  117,  415,       0,    1584,       0,  0, 11,  0, }, // Bhutan
    {  117,  422,       0,    1595,       0,  0, 12,  0, }, // Bolivia
    {  117,  437,    1607,    1620,    1642, 13, 22, 20, }, // Brasilia
    {  117,  454,       0,    1662,       0,  0, 22,  0, }, // Brunei
    {  117,  461,    1684,    1699,    1723, 15, 24, 22, }, // Cape_Verde
    {  117,  472,       0,    1745,       0,  0, 10,  0, }, // Casey
    {  117,  478,       0,    1755,       0,  0, 22,  0, }, // Chamorro
    {  117,  487,    1777,    1789,    1810, 12, 21, 21, }, // Chatham
    {  117,  495,    1831,    1841,    1860, 10, 19, 17, }, // Chile
    {  117,  501,    1877,    1887,    1906, 10, 19, 19, }, // China
    {  117,  507,    1925,    1940,    1964, 15, 24, 22, }, // Choibalsan
    {  117,  518,       0,    1986,       0,  0, 21,  0, }, // Christmas
    {  117,  528,       0,    2007,       0,  0, 18,  0, }, // Cocos
    {  117,  534,    2025,    2038,    2060, 13, 22, 20, }, // Colombia
    {  117,  543,    2080,    2097,    2123, 17, 26, 29, }, // Cook
    {  117,  548,    2152,    2161,    2179,  9, 18, 18, }, // Cuba
    {  117,  559,       0,    2197,       0,  0, 10,  0, }, // Davis
    {  117,  575,       0,    2207,       0,  0, 21,  0, }, // DumontDUrville
    {  117,  612,       0,    2228,       0,  0, 15,  0, }, // East_Timor
    {  117,  623,    2243,    2261,    2288, 18, 27, 25, }, // Easter
    {  117,  630,       0,    2313,       0,  0, 12,  0, }, // Ecuador
    {  117,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  117,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  117,  668,       0,    2483,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  117,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  117,  706,    2591,    2612,    2642, 21, 30, 28, }, // Falkland
    {  117,  715,    2670,    2679,    2697,  9, 18, 16, }, // Fiji
    {  117,  720,       0,    2713,       0,  0, 18,  0, }, // French_Guiana
    {  117,  734,       0,    2731,       0,  0, 32,  0, }, // French_Southern
    {  117,  757,       0,    2763,       0,  0, 14,  0, }, // Galapagos
    {  117,  767,       0,    2777,       0,  0, 12,  0, }, // Gambier
    {  117,  775,    2789,    2801,    2822, 12, 21, 19, }, // Georgia
    {  117,  783,       0,    2841,       0,  0, 20,  0, }, // Gilbert_Islands
    {  117,  799,       0,    2861,       0,  0, 19,  0, }, // GMT
    {  117,  813,    2880,    2894,    2917, 14, 23, 21, }, // Greenland
    {  117,  841,    2938,    2957,    2985, 19, 28, 26, }, // Greenland_Eastern
    {  117,  859,    3011,    3030,    3058, 19, 28, 26, }, // Greenland_Western
    {  117,  877,       0,    3084,       0,  0, 18,  0, }, // Guam
    {  117,  882,       0,    3102,       0,  0, 18,  0, }, // Gulf
    {  117,  887,       0,    3120,       0,  0, 11,  0, }, // Guyana
    {  117,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  117,  910,    3209,    3223,    3246, 14, 23, 21, }, // Hong_Kong
    {  117,  920,    3267,    3276,    3294,  9, 18, 16, }, // Hovd
    {  117,  925,       0,    3310,       0,  0, 19,  0, }, // India
    {  117,  931,       0,    3329,       0,  0, 17,  0, }, // Indian_Ocean
    {  117,  944,       0,    3346,       0,  0, 14,  0, }, // Indochina
    {  117,  954,       0,    3360,       0,  0, 22,  0, }, // Indonesia_Central
    {  117,  972,       0,    3382,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  117,  990,       0,    3404,       0,  0, 22,  0, }, // Indonesia_Western
    {  117, 1008,    3426,    3435,    3453,  9, 18, 18, }, // Iran
    {  117, 1019,    3471,    3483,    3504, 12, 21, 19, }, // Irkutsk
    {  117, 1027,    3523,    3534,    3554, 11, 20, 20, }, // Israel
    {  117, 1034,    3574,    3584,    3603, 10, 19, 19, }, // Japan
    {  117, 1040,    3622,    3651,    3689, 29, 38, 36, }, // Kamchatka
    {  117, 1058,       0,    3725,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  117, 1077,       0,    3745,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  117, 1106,    3765,    3776,    3796, 11, 20, 20, }, // Korea
    {  117, 1112,       0,    3816,       0,  0, 11,  0, }, // Kosrae
    {  117, 1119,    3827,    3843,    3868, 16, 25, 23, }, // Krasnoyarsk
    {  117, 1151,       0,    3891,       0,  0, 15,  0, }, // Kyrgystan
    {  117, 1161,       0,    3906,       0,  0, 10,  0, }, // Lanka
    {  117, 1175,       0,    3916,       0,  0, 17,  0, }, // Line_Islands
    {  117, 1188,    3933,    3947,    3970, 14, 23, 23, }, // Lord_Howe
    {  117, 1198,    3993,    4003,    4022, 10, 19, 17, }, // Macau
    {  117, 1204,    4039,    4051,    4072, 12, 21, 19, }, // Magadan
    {  117, 1219,       0,    4091,       0,  0, 13,  0, }, // Malaysia
    {  117, 1228,       0,    4104,       0,  0, 13,  0, }, // Maldives
    {  117, 1237,       0,    4117,       0,  0, 14,  0, }, // Marquesas
    {  117, 1247,       0,    4131,       0,  0, 21,  0, }, // Marshall_Islands
    {  117, 1264,    4152,    4166,    4189, 14, 23, 21, }, // Mauritius
    {  117, 1274,       0,    4210,       0,  0, 11,  0, }, // Mawson
    {  117, 1281,    4221,    4241,    4270, 20, 29, 29, }, // Mexico_Pacific
    {  117, 1296,    4299,    4315,    4340, 16, 25, 23, }, // Mongolia
    {  117, 1305,    4363,    4374,    4394, 11, 20, 18, }, // Moscow
    {  117, 1312,       0,    4412,       0,  0, 12,  0, }, // Myanmar
    {  117, 1320,       0,    4424,       0,  0, 10,  0, }, // Nauru
    {  117, 1326,       0,    4434,       0,  0, 10,  0, }, // Nepal
    {  117, 1332,    4444,    4462,    4489, 18, 27, 25, }, // New_Caledonia
    {  117, 1346,    4514,    4530,    4555, 16, 25, 25, }, // New_Zealand
    {  117, 1358,    4580,    4597,    4623, 17, 26, 26, }, // Newfoundland
    {  117, 1371,       0,    4649,       0,  0,  9,  0, }, // Niue
    {  117, 1376,    4658,    4677,    4705, 19, 28, 28, }, // Norfolk
    {  117, 1384,    4733,    4757,    4790, 24, 33, 31, }, // Noronha
    {  117, 1392,       0,    4821,       0,  0, 26,  0, }, // North_Mariana
    {  117, 1406,    4847,    4863,    4888, 16, 25, 23, }, // Novosibirsk
    {  117, 1418,    4911,    4920,    4938,  9, 18, 16, }, // Omsk
    {  117, 1428,    4954,    4967,    4989, 13, 22, 20, }, // Pakistan
    {  117, 1437,       0,    5009,       0,  0, 10,  0, }, // Palau
    {  117, 1443,       0,    5019,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  117, 1460,    5040,    5053,    5075, 13, 22, 20, }, // Paraguay
    {  117, 1469,    5095,    5104,    5122,  9, 18, 16, }, // Peru
    {  117, 1474,    5138,    5153,    5177, 15, 24, 22, }, // Philippines
    {  117, 1486,       0,    5199,       0,  0, 20,  0, }, // Phoenix_Islands
    {  117, 1502,    5219,    5245,    5280, 26, 35, 35, }, // Pierre_Miquelon
    {  117, 1518,       0,    5315,       0,  0, 13,  0, }, // Pitcairn
    {  117, 1527,       0,    5328,       0,  0, 11,  0, }, // Ponape
    {  117, 1534,       0,    5339,       0,  0, 14,  0, }, // Pyongyang
    {  117, 1544,    5353,    5367,    5390, 14, 23, 21, }, // Qyzylorda
    {  117, 1554,       0,    5411,       0,  0, 12,  0, }, // Reunion
    {  117, 1562,       0,    5423,       0,  0, 12,  0, }, // Rothera
    {  117, 1570,    5435,    5448,    5470, 13, 22, 20, }, // Sakhalin
    {  117, 1579,    5490,    5501,    5521, 11, 20, 18, }, // Samara
    {  117, 1596,    5539,    5549,    5568, 10, 19, 19, }, // Samoa
    {  117, 1602,       0,    5587,       0,  0, 15,  0, }, // Seychelles
    {  117, 1624,       0,    5602,       0,  0, 23,  0, }, // Singapore
    {  117, 1634,       0,    5625,       0,  0, 20,  0, }, // Solomon
    {  117, 1642,       0,    5645,       0,  0, 18,  0, }, // South_Georgia
    {  117, 1656,       0,    5663,       0,  0, 13,  0, }, // Suriname
    {  117, 1676,       0,    5676,       0,  0, 10,  0, }, // Syowa
    {  117, 1682,       0,    5686,       0,  0, 11,  0, }, // Tahiti
    {  117, 1689,    5697,    5708,    5728, 11, 20, 20, }, // Taipei
    {  117, 1696,       0,    5748,       0,  0, 15,  0, }, // Tajikistan
    {  117, 1724,       0,    5763,       0,  0, 12,  0, }, // Tokelau
    {  117, 1732,    5775,    5785,    5804, 10, 19, 17, }, // Tonga
    {  117, 1738,       0,    5821,       0,  0, 10,  0, }, // Truk
    {  117, 1750,    5831,    5848,    5874, 17, 26, 24, }, // Turkmenistan
    {  117, 1763,       0,    5898,       0,  0, 11,  0, }, // Tuvalu
    {  117, 1777,    5909,    5921,    5942, 12, 21, 19, }, // Uruguay
    {  117, 1792,    5961,    5976,    6000, 15, 24, 22, }, // Uzbekistan
    {  117, 1803,    6022,    6034,    6055, 12, 21, 19, }, // Vanuatu
    {  117, 1811,       0,    6074,       0,  0, 14,  0, }, // Venezuela
    {  117, 1821,    6088,    6104,    6129, 16, 25, 23, }, // Vladivostok
    {  117, 1833,    6152,    6166,    6189, 14, 23, 21, }, // Volgograd
    {  117, 1843,       0,    6210,       0,  0, 11,  0, }, // Vostok
    {  117, 1850,       0,    6221,       0,  0, 16,  0, }, // Wake
    {  117, 1855,       0,    6237,       0,  0, 20,  0, }, // Wallis
    {  117, 1862,    6257,    6269,    6290, 12, 21, 19, }, // Yakutsk
    {  117, 1870,    6309,    6327,    6354, 18, 27, 25, }, // Yekaterinburg
    {  117, 1892,       0,    6379,       0,  0, 10,  0, }, // Yukon
    {  118,  107,  213388,  213407,  213443, 19, 36, 30, }, // Alaska English/Deseret/United States
    {  118,  142,  213473,  213494,  213532, 21, 38, 32, }, // America_Central
    {  118,  158,  213564,  213583,  213619, 19, 36, 30, }, // America_Eastern
    {  118,  174,  213649,  213668,  213704, 19, 36, 30, }, // America_Mountain
    {  118,  191,  213734,  213755,  213793, 21, 38, 32, }, // America_Pacific
    {  118,  286,  213825,  213848,  213888, 23, 40, 34, }, // Atlantic
    {  118,  910,  213922,  213942,  213979, 20, 37, 31, }, // Hong_Kong
    {  118, 1358,  214010,  214037,  214081, 27, 44, 38, }, // Newfoundland
    {  120,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Anguilla
    {  120,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  120,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  120,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  120,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  120,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  120,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  120, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  121,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Antigua and Barbuda
    {  121,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  121,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  121,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  121,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  121,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  121,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  121, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  122,   32,       0,  214212,       0,  0, 19,  0, }, // Africa_Eastern English/Latin/Australia
    {  122,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  122,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  122,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  122,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  122,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  122,  232,  214231,  214242,  214262, 11, 20, 20, }, // Arabian
    {  122,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  122,  295,  214282,    1066,    1098, 23, 32, 32, }, // Australia_Central
    {  122,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {  122,  338,  214305,    1263,    1295, 23, 32, 32, }, // Australia_Eastern
    {  122,  356,  214328,    1349,    1381, 23, 32, 32, }, // Australia_Western
    {  122,  501,    1877,    1887,  214351, 10, 19, 17, }, // China
    {  122,  543,  214368,  214384,  214409, 16, 25, 23, }, // Cook
    {  122,  882,       0,    3102,       0,  0, 18,  0, }, // Gulf
    {  122,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  122, 1034,    3574,    3584,  214432, 10, 19, 17, }, // Japan
    {  122, 1106,  214449,    3776,  214459, 10, 20, 18, }, // Korea
    {  122, 1188,    3933,    3947,    3970, 14, 23, 23, }, // Lord_Howe
    {  122, 1305,    4363,    4374,  214477, 11, 20, 20, }, // Moscow
    {  122, 1346,    4514,    4530,    4555, 16, 25, 25, }, // New_Zealand
    {  122, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  122, 1596,    5539,    5549,  214497, 10, 19, 17, }, // Samoa
    {  122, 1689,    5697,    5708,  214514, 11, 20, 18, }, // Taipei
    {  123,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Austria
    {  123,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  123,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  123,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  123,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  123,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  123,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  123,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  123,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  123,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  123, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  124,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Bahamas
    {  124,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  124,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  124,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  124,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  124,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  124,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  124, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  125,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Barbados
    {  125,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  125,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  125,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  125,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  125,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  125,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  125, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  126,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Belgium
    {  126,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  126,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  126,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  126,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  126,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  126,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  126,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  126,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  126,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  126, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  127,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Belize
    {  127,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  127,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  127,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  127,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  127,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  127,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  127, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  128,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Bermuda
    {  128,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  128,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  128,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  128,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  128,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  128,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  128, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  129,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Botswana
    {  129,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  129,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  129,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  129,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  129,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  129,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  129,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  129,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  129,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  129,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  129, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  130,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/British Indian Ocean Territory
    {  130,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  130,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  130,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  130,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  130,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  130,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  130, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  131,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/British Virgin Islands
    {  131,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  131,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  131,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  131,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  131,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  131,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  131, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  133,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Cameroon
    {  133,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  133,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  133,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  133,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  133,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  133,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  133,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  133,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  133,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  133,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  133, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  134,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan English/Latin/Canada
    {  134,  107,     184,     195,  214532, 11, 20, 27, }, // Alaska
    {  134,  142,     333,     345,  214559, 12, 21, 28, }, // America_Central
    {  134,  158,     387,     399,  214587, 12, 21, 28, }, // America_Eastern
    {  134,  174,     441,     454,  214615, 13, 22, 29, }, // America_Mountain
    {  134,  191,     498,     510,  214644, 12, 21, 28, }, // America_Pacific
    {  134,  214,     601,     610,  214672,  9, 18, 25, }, // Apia
    {  134,  232,     741,     753,  214697, 12, 21, 28, }, // Arabian
    {  134,  240,     795,     809,     832, 14, 23, 21, }, // Argentina
    {  134,  286,     987,    1000,  214725, 13, 22, 29, }, // Atlantic
    {  134,  295,    1044,    1066,  214754, 22, 32, 39, }, // Australia_Central
    {  134,  313,    1130,    1161,  214793, 31, 40, 47, }, // Australia_CentralWestern
    {  134,  338,    1241,    1263,  214840, 22, 32, 39, }, // Australia_Eastern
    {  134,  356,    1327,    1349,  214879, 22, 32, 39, }, // Australia_Western
    {  134,  397,    1523,    1538,    1562, 15, 24, 22, }, // Bangladesh
    {  134,  415,       0,    1584,       0,  0, 11,  0, }, // Bhutan
    {  134,  437,    1607,    1620,    1642, 13, 22, 20, }, // Brasilia
    {  134,  454,       0,    1662,       0,  0, 22,  0, }, // Brunei
    {  134,  487,    1777,    1789,  214918, 12, 21, 28, }, // Chatham
    {  134,  501,    1877,    1887,  214946, 10, 19, 26, }, // China
    {  134,  518,       0,    1986,       0,  0, 21,  0, }, // Christmas
    {  134,  528,       0,    2007,       0,  0, 18,  0, }, // Cocos
    {  134,  534,    2025,    2038,    2060, 13, 22, 20, }, // Colombia
    {  134,  548,    2152,    2161,  214972,  9, 18, 25, }, // Cuba
    {  134,  612,       0,    2228,       0,  0, 15,  0, }, // East_Timor
    {  134,  623,    2243,    2261,    2288, 18, 27, 25, }, // Easter
    {  134,  630,       0,    2313,       0,  0, 12,  0, }, // Ecuador
    {  134,  706,    2591,    2612,    2642, 21, 30, 28, }, // Falkland
    {  134,  734,       0,  214997,       0,  0, 34,  0, }, // French_Southern
    {  134,  757,       0,    2763,       0,  0, 14,  0, }, // Galapagos
    {  134,  841,    2938,    2957,    2985, 19, 28, 26, }, // Greenland_Eastern
    {  134,  887,       0,    3120,       0,  0, 11,  0, }, // Guyana
    {  134,  894,    3131,    3151,  215031, 20, 29, 36, }, // Hawaii_Aleutian
    {  134,  925,       0,    3310,       0,  0, 19,  0, }, // India
    {  134,  944,       0,    3346,       0,  0, 14,  0, }, // Indochina
    {  134,  954,       0,    3360,       0,  0, 22,  0, }, // Indonesia_Central
    {  134,  972,       0,    3382,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  134,  990,       0,    3404,       0,  0, 22,  0, }, // Indonesia_Western
    {  134, 1008,    3426,    3435,  215067,  9, 18, 25, }, // Iran
    {  134, 1027,    3523,    3534,  215092, 11, 20, 27, }, // Israel
    {  134, 1034,    3574,    3584,  215119, 10, 19, 26, }, // Japan
    {  134, 1106,    3765,    3776,  215145, 11, 20, 27, }, // Korea
    {  134, 1188,    3933,    3947,  215172, 14, 23, 30, }, // Lord_Howe
    {  134, 1219,       0,    4091,       0,  0, 13,  0, }, // Malaysia
    {  134, 1228,       0,    4104,       0,  0, 13,  0, }, // Maldives
    {  134, 1281,    4221,    4241,  215202, 20, 29, 36, }, // Mexico_Pacific
    {  134, 1326,       0,    4434,       0,  0, 10,  0, }, // Nepal
    {  134, 1346,    4514,    4530,  215238, 16, 25, 32, }, // New_Zealand
    {  134, 1358,    4580,    4597,  215270, 17, 26, 33, }, // Newfoundland
    {  134, 1376,    4658,    4677,  215303, 19, 28, 35, }, // Norfolk
    {  134, 1384,    4733,    4757,    4790, 24, 33, 31, }, // Noronha
    {  134, 1428,    4954,    4967,    4989, 13, 22, 20, }, // Pakistan
    {  134, 1460,    5040,    5053,    5075, 13, 22, 20, }, // Paraguay
    {  134, 1469,    5095,    5104,    5122,  9, 18, 16, }, // Peru
    {  134, 1502,  215338,  215367,  215405, 29, 38, 45, }, // Pierre_Miquelon
    {  134, 1596,    5539,    5549,  215450, 10, 19, 26, }, // Samoa
    {  134, 1689,    5697,    5708,  215476, 11, 20, 27, }, // Taipei
    {  134, 1777,    5909,    5921,    5942, 12, 21, 19, }, // Uruguay
    {  134, 1811,       0,    6074,       0,  0, 14,  0, }, // Venezuela
    {  134, 1855,       0,  215503,       0,  0, 22,  0, }, // Wallis
    {  135,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Cayman Islands
    {  135,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  135,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  135,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  135,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  135,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  135,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  135, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  136,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Christmas Island
    {  136,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  136,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  136,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  136,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  136,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  136,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  136, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  137,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Cocos Islands
    {  137,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  137,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  137,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  137,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  137,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  137,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  137, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  138,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Cook Islands
    {  138,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  138,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  138,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  138,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  138,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  138,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  138, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  139,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Cyprus
    {  139,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  139,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  139,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  139,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  139,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  139,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  139, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  140,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Denmark
    {  140,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  140,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  140,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  140,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  140,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  140,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  140,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  140,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  140,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  140, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  141,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Diego Garcia
    {  141,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  141,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  141,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  141,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  141,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  141,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  141, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  142,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Dominica
    {  142,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  142,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  142,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  142,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  142,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  142,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  142, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  143,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Eritrea
    {  143,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  143,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  143,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  143,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  143,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  143,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  143,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  143,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  143,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  143,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  143, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  144,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Eswatini
    {  144,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  144,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  144,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  144,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  144,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  144,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  144,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  144,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  144,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  144,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  144, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  145,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Europe
    {  145,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  145,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  145,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  145,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  145,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  145,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  145,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  145,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  145,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  145, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  146,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Falkland Islands
    {  146,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  146,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  146,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  146,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  146,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  146,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  146, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  147,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Fiji
    {  147,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  147,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  147,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  147,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  147,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  147,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  147, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  148,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Finland
    {  148,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  148,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  148,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  148,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  148,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  148,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  148,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  148,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  148,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  148, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  149,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Gambia
    {  149,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  149,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  149,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  149,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  149,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  149,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  149,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  149,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  149,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  149,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  149, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  150,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Germany
    {  150,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  150,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  150,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  150,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  150,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  150,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  150,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  150,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  150,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  150, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  151,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Ghana
    {  151,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  151,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  151,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  151,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  151,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  151,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  151,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  151,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  151,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  151,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  151, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  152,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Gibraltar
    {  152,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  152,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  152,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  152,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  152,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  152,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  152, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  153,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Grenada
    {  153,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  153,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  153,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  153,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  153,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  153,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  153, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  154,  478,       0,    1755,       0,  0, 22,  0, }, // Chamorro English/Latin/Guam
    {  155,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Guernsey
    {  155,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  155,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  155,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  155,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  155,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  155,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  155, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  156,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Guyana
    {  156,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  156,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  156,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  156,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  156,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  156,  887,       0,    3120,       0,  0, 11,  0, }, // Guyana
    {  156,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  156, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  157,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Hong Kong
    {  157,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  157,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  157,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  157,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  157,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  157,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  157,  910,    3209,    3223,    3246, 14, 23, 21, }, // Hong_Kong
    {  157, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  158,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/India
    {  158,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  158,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  158,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  158,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  158,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  158,  882,       0,    3102,       0,  0, 18,  0, }, // Gulf
    {  158,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  158,  925,       0,    3310,       0,  0, 19,  0, }, // India
    {  158, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  159,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Indonesia
    {  159,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  159,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  159,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  159,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  159,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  159,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  159,  954,       0,    3360,       0,  0, 22,  0, }, // Indonesia_Central
    {  159,  972,       0,    3382,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  159,  990,       0,    3404,       0,  0, 22,  0, }, // Indonesia_Western
    {  159, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  160,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Ireland
    {  160,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  160,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  160,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  160,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  160,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  160,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  160, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  161,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Isle of Man
    {  161,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  161,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  161,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  161,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  161,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  161,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  161, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  162,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Israel
    {  162,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  162,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  162,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  162,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  162,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  162,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  162, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  163,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Jamaica
    {  163,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  163,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  163,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  163,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  163,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  163,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  163, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  164,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Jersey
    {  164,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  164,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  164,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  164,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  164,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  164,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  164, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  165,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Kenya
    {  165,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  165,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  165,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  165,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  165,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  165,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  165,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  165,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  165,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  165,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  165, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  166,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Kiribati
    {  166,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  166,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  166,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  166,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  166,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  166,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  166, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  167,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Lesotho
    {  167,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  167,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  167,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  167,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  167,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  167,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  167,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  167,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  167,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  167,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  167, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  168,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Liberia
    {  168,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  168,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  168,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  168,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  168,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  168,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  168,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  168,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  168,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  168,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  168, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  169,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Macao
    {  169,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  169,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  169,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  169,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  169,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  169,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  169,  910,    3209,    3223,    3246, 14, 23, 21, }, // Hong_Kong
    {  169, 1198,    3993,    4003,    4022, 10, 19, 17, }, // Macau
    {  169, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  170,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Madagascar
    {  170,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  170,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  170,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  170,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  170,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  170,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  170,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  170,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  170,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  170,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  170, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  171,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Malawi
    {  171,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  171,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  171,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  171,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  171,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  171,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  171,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  171,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  171,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  171,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  171, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  172,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Malaysia
    {  172,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  172,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  172,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  172,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  172,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  172,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  172, 1219,       0,    4091,       0,  0, 13,  0, }, // Malaysia
    {  172, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  173,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Maldives
    {  173,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  173,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  173,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  173,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  173,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  173,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  173, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  174,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Malta
    {  174,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  174,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  174,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  174,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  174,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  174,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  174, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  175,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Marshall Islands
    {  175,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  175,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  175,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  175,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  175,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  175,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  176,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Mauritius
    {  176,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  176,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  176,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  176,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  176,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  176,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  176,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  176,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  176,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  176,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  176, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  177,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Micronesia
    {  177,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  177,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  177,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  177,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  177,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  177,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  177, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  178,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Montserrat
    {  178,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  178,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  178,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  178,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  178,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  178,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  178, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  179,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Namibia
    {  179,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  179,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  179,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  179,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  179,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  179,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  179,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  179,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  179,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  179,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  179, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  180,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Nauru
    {  180,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  180,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  180,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  180,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  180,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  180,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  180, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  181,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Netherlands
    {  181,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  181,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  181,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  181,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  181,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  181,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  181,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  181,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  181,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  181, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  182,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/New Zealand
    {  182,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  182,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  182,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  182,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  182,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  182,  295,    1044,    1066,    1098, 22, 32, 32, }, // Australia_Central
    {  182,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {  182,  338,    1241,    1263,    1295, 22, 32, 32, }, // Australia_Eastern
    {  182,  356,    1327,    1349,    1381, 22, 32, 32, }, // Australia_Western
    {  182,  487,    1777,    1789,    1810, 12, 21, 21, }, // Chatham
    {  182,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  182, 1188,    3933,    3947,    3970, 14, 23, 23, }, // Lord_Howe
    {  182, 1346,    4514,    4530,    4555, 16, 25, 25, }, // New_Zealand
    {  182, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  183,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Nigeria
    {  183,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  183,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  183,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  183,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  183,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  183,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  183,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  183,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  183,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  183,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  183, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  184,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Niue
    {  184,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  184,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  184,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  184,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  184,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  184,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  184, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  185,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Norfolk Island
    {  185,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  185,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  185,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  185,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  185,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  185,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  185, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  186,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Northern Mariana Islands
    {  186,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  186,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  186,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  186,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  186,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  186,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  187,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Pakistan
    {  187,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  187,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  187,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  187,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  187,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  187,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  187, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  188,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Palau
    {  188,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  188,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  188,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  188,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  188,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  188,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  188, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  189,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Papua New Guinea
    {  189,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  189,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  189,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  189,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  189,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  189,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  189, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  191,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Pitcairn
    {  191,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  191,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  191,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  191,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  191,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  191,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  191, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  193,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Rwanda
    {  193,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  193,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  193,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  193,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  193,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  193,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  193,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  193,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  193,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  193,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  193, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  194,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Saint Helena
    {  194,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  194,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  194,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  194,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  194,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  194,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  194, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  195,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Saint Kitts and Nevis
    {  195,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  195,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  195,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  195,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  195,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  195,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  195, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  196,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Saint Lucia
    {  196,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  196,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  196,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  196,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  196,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  196,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  196, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  197,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Saint Vincent and Grenadines
    {  197,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  197,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  197,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  197,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  197,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  197,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  197, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  198,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Samoa
    {  198,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  198,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  198,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  198,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  198,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  198,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  198, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  199,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Seychelles
    {  199,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  199,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  199,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  199,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  199,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  199,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  199, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  200,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Sierra Leone
    {  200,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  200,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  200,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  200,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  200,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  200,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  200,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  200,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  200,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  200,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  200, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  201,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Singapore
    {  201,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  201,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  201,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  201,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  201,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  201,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  201, 1219,       0,    4091,       0,  0, 13,  0, }, // Malaysia
    {  201, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  201, 1624,       0,    5602,       0,  0, 23,  0, }, // Singapore
    {  202,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Sint Maarten
    {  202,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  202,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  202,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  202,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  202,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  202,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  202, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  203,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Slovenia
    {  203,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  203,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  203,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  203,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  203,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  203,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  203,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  203,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  203,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  203, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  204,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Solomon Islands
    {  204,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  204,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  204,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  204,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  204,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  204,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  204, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  205,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/South Africa
    {  205,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  205,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  205,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  205,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  205,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  205,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  205,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  205,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  205,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  205,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  205, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  206,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/South Sudan
    {  206,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  206,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  206,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  206,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  206,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  206,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  206,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  206,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  206,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  206,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  206, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  207,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Sudan
    {  207,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  207,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  207,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  207,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  207,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  207,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  207,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  207,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  207,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  207,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  207, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  208,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Sweden
    {  208,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  208,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  208,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  208,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  208,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  208,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  208,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  208,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  208,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  208, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  209,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Switzerland
    {  209,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  209,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  209,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  209,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  209,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  209,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  209,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  209,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  209,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  209, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  210,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Tanzania
    {  210,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  210,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  210,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  210,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  210,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  210,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  210,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  210,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  210,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  210,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  210, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  211,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Tokelau
    {  211,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  211,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  211,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  211,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  211,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  211,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  211, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  212,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Tonga
    {  212,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  212,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  212,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  212,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  212,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  212,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  212, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  213,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Trinidad and Tobago
    {  213,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  213,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  213,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  213,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  213,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  213,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  213, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  214,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Turks and Caicos Islands
    {  214,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  214,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  214,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  214,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  214,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  214,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  214, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  215,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Tuvalu
    {  215,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  215,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  215,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  215,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  215,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  215,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  215, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  216,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Uganda
    {  216,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  216,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  216,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  216,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  216,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  216,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  216,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  216,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  216,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  216,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  216, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  217,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/United Arab Emirates
    {  217,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  217,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  217,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  217,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  217,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  217,  882,       0,    3102,       0,  0, 18,  0, }, // Gulf
    {  217,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  218,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/United Kingdom
    {  218,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  218,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  218,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  218,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  218,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  218,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  218,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  218,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  218,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  218, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  221,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/Vanuatu
    {  221,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  221,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  221,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  221,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  221,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  221,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  221, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  222,  107,     184,     195,     215, 11, 20, 20, }, // Alaska English/Latin/world
    {  222,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  222,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  222,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  222,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  222,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  222,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  222, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  223,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Zambia
    {  223,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  223,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  223,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  223,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  223,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  223,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  223,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  223,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  223,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  223,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  223, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  224,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central English/Latin/Zimbabwe
    {  224,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  224,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  224,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  224,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  224,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  224,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  224,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  224,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  224,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  224,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  224, 1502,  214119,  214144,  214178, 25, 34, 34, }, // Pierre_Miquelon
    {  226,    5,       0,  215525,       0,  0, 17,  0, }, // Afghanistan Erzya/Cyrillic/Russia
    {  226,  107,  215542,  215554,  215574, 12, 20, 19, }, // Alaska
    {  226,  135,  215593,  215606,  215627, 13, 21, 20, }, // Amazon
    {  226,  397,  215647,  215663,  215687, 16, 24, 23, }, // Bangladesh
    {  226,  437,  215710,  215723,  215744, 13, 21, 20, }, // Brasilia
    {  226,  495,  215764,  215774,  215792, 10, 18, 17, }, // Chile
    {  226,  501,  215809,  215820,  215839, 11, 19, 18, }, // China
    {  226,  548,  215857,  215867,  215885, 10, 18, 17, }, // Cuba
    {  226,  925,       0,  215902,       0,  0, 19,  0, }, // India
    {  226, 1008,  215921,  215932,  215951, 11, 19, 18, }, // Iran
    {  226, 1034,  215969,  215981,  216001, 12, 20, 19, }, // Japan
    {  226, 1058,       0,  216020,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  226, 1077,       0,  216047,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  226, 1106,  216075,  216086,  216105, 11, 19, 18, }, // Korea
    {  226, 1305,  216123,  216136,  216157, 13, 21, 20, }, // Moscow
    {  226, 1346,  216177,  216194,  216219, 17, 25, 24, }, // New_Zealand
    {  226, 1418,  216243,  216255,  216275, 12, 20, 19, }, // Omsk
    {  226, 1460,  216294,  216308,  216330, 14, 22, 21, }, // Paraguay
    {  226, 1570,  216351,  216365,  216387, 14, 22, 21, }, // Sakhalin
    {  226, 1777,  216408,  216421,  216442, 13, 21, 20, }, // Uruguay
    {  226, 1792,  216462,  216479,  216504, 17, 25, 24, }, // Uzbekistan
    {  227,   17,       0,  216528,       0,  0, 17,  0, }, // Africa_Central Esperanto/Latin/world
    {  227,   32,       0,  216545,       0,  0, 18,  0, }, // Africa_Eastern
    {  227,   65,       0,  216563,       0,  0, 15,  0, }, // Africa_Southern
    {  227,   81,  216578,  216598,  216624, 20, 26, 27, }, // Africa_Western
    {  227,  107,  216651,  216663,  216681, 12, 18, 19, }, // Alaska
    {  227,  135,  216700,  216714,  216734, 14, 20, 21, }, // Amazon
    {  227,  142,  216755,  216779,  216809, 24, 30, 31, }, // America_Central
    {  227,  158,  216840,  216865,  216896, 25, 31, 32, }, // America_Eastern
    {  227,  174,  216928,  216953,  216984, 25, 31, 32, }, // America_Mountain
    {  227,  191,  217016,  217042,  217074, 26, 32, 33, }, // America_Pacific
    {  227,  232,  217107,  217118,  217135, 11, 17, 18, }, // Arabian
    {  227,  240,  217153,  217168,  217189, 15, 21, 22, }, // Argentina
    {  227,  250,  217211,  217235,  217265, 24, 30, 31, }, // Argentina_Western
    {  227,  268,  217296,  217308,  217326, 12, 18, 19, }, // Armenia
    {  227,  286,  217345,  217372,  217405, 27, 33, 34, }, // Atlantic
    {  227,  295,  217439,  217461,  217489, 22, 28, 29, }, // Australia_Central
    {  227,  313,  217518,  217548,  217584, 30, 36, 37, }, // Australia_CentralWestern
    {  227,  338,  217621,  217644,  217673, 23, 29, 30, }, // Australia_Eastern
    {  227,  356,  217523,  217553,  217589, 25, 31, 32, }, // Australia_Western
    {  227,  374,  217703,  217720,  217743, 17, 23, 24, }, // Azerbaijan
    {  227,  385,  217767,  217781,  217801, 14, 20, 21, }, // Azores
    {  227,  422,       0,  217822,       0,  0, 13,  0, }, // Bolivia
    {  227,  437,  217835,  217849,  217869, 14, 20, 21, }, // Brasilia
    {  227,  454,       0,  217890,       0,  0, 13,  0, }, // Brunei
    {  227,  461,  217903,  217918,  217939, 15, 21, 22, }, // Cape_Verde
    {  227,  487,  217961,  217974,  217993, 13, 19, 20, }, // Chatham
    {  227,  495,  218013,  218024,  218041, 11, 17, 18, }, // Chile
    {  227,  501,  218059,  218069,  218085, 10, 16, 17, }, // China
    {  227,  507,  218102,  218119,  218142, 17, 23, 24, }, // Choibalsan
    {  227,  518,       0,  218166,       0,  0, 21,  0, }, // Christmas
    {  227,  528,       0,  218187,       0,  0, 17,  0, }, // Cocos
    {  227,  534,  218204,  218218,  218238, 14, 20, 21, }, // Colombia
    {  227,  548,  218259,  218271,  218289, 12, 18, 19, }, // Cuba
    {  227,  559,       0,  218308,       0,  0, 13,  0, }, // Davis
    {  227,  575,       0,  218321,       0,  0, 24,  0, }, // DumontDUrville
    {  227,  612,       0,  218345,       0,  0, 18,  0, }, // East_Timor
    {  227,  623,  218363,  218379,  218401, 16, 22, 23, }, // Easter
    {  227,  630,       0,  218424,       0,  0, 14,  0, }, // Ecuador
    {  227,  638,  218438,  218455,  218478, 17, 23, 24, }, // Europe_Central
    {  227,  653,  218502,  218520,  218544, 18, 24, 25, }, // Europe_Eastern
    {  227,  668,       0,  218569,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  227,  691,  218595,  218615,  218641, 20, 26, 27, }, // Europe_Western
    {  227,  706,  218668,  218683,  218704, 15, 21, 22, }, // Falkland
    {  227,  720,       0,  218726,       0,  0, 21,  0, }, // French_Guiana
    {  227,  734,       0,  218747,       0,  0, 32,  0, }, // French_Southern
    {  227,  757,       0,  218779,       0,  0, 14,  0, }, // Galapagos
    {  227,  775,  218793,  218807,  218827, 14, 20, 21, }, // Georgia
    {  227,  799,       0,  218848,       0,  0, 28,  0, }, // GMT
    {  227,  841,  218876,  218899,  218928, 23, 29, 30, }, // Greenland_Eastern
    {  227,  859,  218958,  218983,  219014, 25, 31, 32, }, // Greenland_Western
    {  227,  882,       0,  219046,       0,  0, 17,  0, }, // Gulf
    {  227,  887,       0,  219063,       0,  0, 12,  0, }, // Guyana
    {  227,  894,  219075,  219097,  219125, 22, 28, 29, }, // Hawaii_Aleutian
    {  227,  910,  219154,  219168,  219188, 14, 20, 21, }, // Hong_Kong
    {  227,  920,  219209,  219220,  219237, 11, 17, 18, }, // Hovd
    {  227,  925,       0,  219255,       0,  0, 12,  0, }, // India
    {  227,  931,       0,  219267,       0,  0, 16,  0, }, // Indian_Ocean
    {  227,  944,       0,  219283,       0,  0, 15,  0, }, // Indochina
    {  227,  954,       0,  219298,       0,  0, 22,  0, }, // Indonesia_Central
    {  227,  972,       0,  219320,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  227,  990,       0,  219343,       0,  0, 25,  0, }, // Indonesia_Western
    {  227, 1019,  219368,  219382,  219402, 14, 20, 21, }, // Irkutsk
    {  227, 1027,  219423,  219436,  219455, 13, 19, 20, }, // Israel
    {  227, 1034,  219475,  219487,  219505, 12, 18, 19, }, // Japan
    {  227, 1058,       0,  219524,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  227, 1077,       0,  219544,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  227, 1106,  219566,  219577,  219594, 11, 17, 18, }, // Korea
    {  227, 1119,  219612,  219630,  219654, 18, 24, 25, }, // Krasnoyarsk
    {  227, 1151,       0,  219679,       0,  0, 13,  0, }, // Kyrgystan
    {  227, 1188,  219692,  219709,  219732, 17, 23, 24, }, // Lord_Howe
    {  227, 1204,  219756,  219770,  219790, 14, 20, 21, }, // Magadan
    {  227, 1219,       0,  219811,       0,  0, 14,  0, }, // Malaysia
    {  227, 1264,  219825,  219839,  219859, 14, 20, 21, }, // Mauritius
    {  227, 1281,  219880,  219902,  219930, 22, 28, 29, }, // Mexico_Pacific
    {  227, 1296,  219959,  219975,  219997, 16, 22, 23, }, // Mongolia
    {  227, 1305,  220020,  220032,  220050, 12, 18, 19, }, // Moscow
    {  227, 1312,       0,  220069,       0,  0, 11,  0, }, // Myanmar
    {  227, 1346,  220080,  220096,  220118, 16, 22, 23, }, // New_Zealand
    {  227, 1358,  220141,  220157,  220179, 16, 22, 23, }, // Newfoundland
    {  227, 1376,  220202,  220221,  220246, 19, 25, 26, }, // Norfolk
    {  227, 1384,  220272,  220299,  220332, 27, 33, 34, }, // Noronha
    {  227, 1406,  220366,  220384,  220408, 18, 24, 25, }, // Novosibirsk
    {  227, 1418,  220433,  220444,  220461, 11, 17, 18, }, // Omsk
    {  227, 1460,  220479,  220494,  220515, 15, 21, 22, }, // Paraguay
    {  227, 1469,  220537,  220548,  220565, 11, 17, 18, }, // Peru
    {  227, 1474,  220583,  220597,  220617, 14, 20, 21, }, // Philippines
    {  227, 1502,  220638,  220671,  220710, 33, 39, 40, }, // Pierre_Miquelon
    {  227, 1534,       0,  220750,       0,  0, 16,  0, }, // Pyongyang
    {  227, 1554,       0,  220766,       0,  0, 14,  0, }, // Reunion
    {  227, 1570,  220780,  220794,  220814, 14, 20, 21, }, // Sakhalin
    {  227, 1602,       0,  220835,       0,  0, 13,  0, }, // Seychelles
    {  227, 1624,       0,  220848,       0,  0, 21,  0, }, // Singapore
    {  227, 1642,       0,  220869,       0,  0, 19,  0, }, // South_Georgia
    {  227, 1656,       0,  220888,       0,  0, 14,  0, }, // Suriname
    {  227, 1689,  220902,  220914,  220932, 12, 18, 19, }, // Taipei
    {  227, 1696,       0,  220951,       0,  0, 12,  0, }, // Tajikistan
    {  227, 1750,  220963,  220977,  220997, 14, 20, 21, }, // Turkmenistan
    {  227, 1777,  221018,  221032,  221052, 14, 20, 21, }, // Uruguay
    {  227, 1792,  221073,  221085,  221103, 12, 18, 19, }, // Uzbekistan
    {  227, 1811,       0,  221122,       0,  0, 15,  0, }, // Venezuela
    {  227, 1821,  221137,  221155,  221179, 18, 24, 25, }, // Vladivostok
    {  227, 1833,  221204,  221220,  221242, 16, 22, 23, }, // Volgograd
    {  227, 1862,  221265,  221279,  221299, 14, 20, 21, }, // Yakutsk
    {  227, 1870,  221320,  221340,  221366, 20, 26, 27, }, // Yekaterinburg
    {  227, 1892,       0,  221393,       0,  0, 13,  0, }, // Yukon
    {  228,    0,  221406,  221414,  221430,  8, 16, 12, }, // Acre Estonian/Latin/Estonia
    {  228,    5,       0,  221442,       0,  0, 15,  0, }, // Afghanistan
    {  228,   17,       0,  221457,       0,  0, 16,  0, }, // Africa_Central
    {  228,   32,       0,  221473,       0,  0, 15,  0, }, // Africa_Eastern
    {  228,   65,       0,  221488,       0,  0, 25,  0, }, // Africa_Southern
    {  228,   81,  221513,  221530,  221555, 17, 25, 21, }, // Africa_Western
    {  228,  107,  221576,  221586,  221604, 10, 18, 14, }, // Alaska
    {  228,  128,  221618,  221628,  221646, 10, 18, 14, }, // Almaty
    {  228,  135,  221660,  221673,  221694, 13, 21, 17, }, // Amazon
    {  228,  142,  221711,  221728,  221753, 17, 25, 21, }, // America_Central
    {  228,  158,  221774,  221788,  221810, 14, 22, 18, }, // America_Eastern
    {  228,  174,  221828,  221846,  221872, 18, 26, 22, }, // America_Mountain
    {  228,  191,  221894,  221912,  221938, 18, 26, 22, }, // America_Pacific
    {  228,  207,  221960,  221971,  221990, 11, 19, 15, }, // Anadyr
    {  228,  214,  222005,  222013,  222029,  8, 16, 12, }, // Apia
    {  228,  219,  222041,  222050,  222067,  9, 17, 13, }, // Aqtau
    {  228,  225,  222080,  222090,  222108, 10, 18, 14, }, // Aqtobe
    {  228,  232,  222122,  222133,  222152, 11, 19, 15, }, // Arabian
    {  228,  240,  222167,  222180,  222201, 13, 21, 17, }, // Argentina
    {  228,  250,  222218,  222237,  222264, 19, 27, 23, }, // Argentina_Western
    {  228,  268,  222287,  222299,  222319, 12, 20, 16, }, // Armenia
    {  228,  286,  222335,  222346,  222365, 11, 19, 15, }, // Atlantic
    {  228,  295,  222380,  222399,  222426, 19, 27, 23, }, // Australia_Central
    {  228,  313,  222449,  222474,  222507, 25, 33, 29, }, // Australia_CentralWestern
    {  228,  338,  222536,  222554,  222580, 18, 26, 22, }, // Australia_Eastern
    {  228,  356,  222602,  222622,  222650, 20, 28, 24, }, // Australia_Western
    {  228,  374,  222674,  222691,  222716, 17, 25, 21, }, // Azerbaijan
    {  228,  385,  222737,  222750,  222771, 13, 21, 17, }, // Azores
    {  228,  397,  222788,  222803,  222826, 15, 23, 19, }, // Bangladesh
    {  228,  415,       0,  222845,       0,  0, 11,  0, }, // Bhutan
    {  228,  422,       0,  222856,       0,  0, 12,  0, }, // Bolivia
    {  228,  437,  222868,  222881,  222902, 13, 21, 17, }, // Brasilia
    {  228,  454,       0,  222919,       0,  0, 10,  0, }, // Brunei
    {  228,  461,  222929,  222948,  222975, 19, 27, 23, }, // Cape_Verde
    {  228,  472,       0,  222998,       0,  0,  9,  0, }, // Casey
    {  228,  478,       0,  223007,       0,  0, 20,  0, }, // Chamorro
    {  228,  487,  223027,  223039,  223059, 12, 20, 16, }, // Chatham
    {  228,  495,  223075,  223085,  223103, 10, 18, 14, }, // Chile
    {  228,  501,  223117,  223126,  223143,  9, 17, 13, }, // China
    {  228,  507,  223156,  223171,  223194, 15, 23, 19, }, // Choibalsan
    {  228,  518,       0,  223213,       0,  0, 14,  0, }, // Christmas
    {  228,  528,       0,  223227,       0,  0, 16,  0, }, // Cocos
    {  228,  534,  223243,  223255,  223275, 12, 20, 16, }, // Colombia
    {  228,  543,  223291,  223307,  223331, 16, 24, 28, }, // Cook
    {  228,  548,  223359,  223368,  223385,  9, 17, 13, }, // Cuba
    {  228,  559,       0,  223398,       0,  0, 10,  0, }, // Davis
    {  228,  575,       0,  223408,       0,  0, 22,  0, }, // DumontDUrville
    {  228,  612,       0,  223430,       0,  0, 14,  0, }, // East_Timor
    {  228,  623,  223444,  223462,  223488, 18, 26, 22, }, // Easter
    {  228,  630,       0,  223510,       0,  0, 12,  0, }, // Ecuador
    {  228,  638,  223522,  223538,  223562, 16, 24, 20, }, // Europe_Central
    {  228,  653,  223582,  223597,  223620, 15, 23, 19, }, // Europe_Eastern
    {  228,  668,       0,  223639,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  228,  691,  223668,  223685,  223710, 17, 25, 21, }, // Europe_Western
    {  228,  706,  223731,  223751,  223779, 20, 28, 24, }, // Falkland
    {  228,  715,  223803,  223812,  223829,  9, 17, 13, }, // Fiji
    {  228,  720,       0,  223842,       0,  0, 22,  0, }, // French_Guiana
    {  228,  734,       0,  223864,       0,  0, 41,  0, }, // French_Southern
    {  228,  757,       0,  223905,       0,  0, 14,  0, }, // Galapagos
    {  228,  767,       0,  223919,       0,  0, 12,  0, }, // Gambier
    {  228,  775,  223931,  223942,  223961, 11, 19, 15, }, // Georgia
    {  228,  783,       0,  223976,       0,  0, 19,  0, }, // Gilbert_Islands
    {  228,  799,       0,  223995,       0,  0, 14,  0, }, // GMT
    {  228,  841,  224009,  224026,  224051, 17, 25, 21, }, // Greenland_Eastern
    {  228,  859,  224072,  224091,  224118, 19, 27, 23, }, // Greenland_Western
    {  228,  877,       0,  224141,       0,  0, 17,  0, }, // Guam
    {  228,  882,       0,  224158,       0,  0, 23,  0, }, // Gulf
    {  228,  887,       0,  224181,       0,  0, 10,  0, }, // Guyana
    {  228,  894,  224191,  224209,  224235, 18, 26, 22, }, // Hawaii_Aleutian
    {  228,  910,  224257,  224270,  224291, 13, 21, 17, }, // Hong_Kong
    {  228,  920,  224308,  224317,  224334,  9, 17, 13, }, // Hovd
    {  228,  925,       0,  224347,       0,  0,  9,  0, }, // India
    {  228,  931,       0,  224356,       0,  0, 17,  0, }, // Indian_Ocean
    {  228,  944,       0,  224373,       0,  0, 13,  0, }, // Indochina
    {  228,  954,       0,  224386,       0,  0, 19,  0, }, // Indonesia_Central
    {  228,  972,       0,  224405,       0,  0, 18,  0, }, // Indonesia_Eastern
    {  228,  990,       0,  224423,       0,  0, 20,  0, }, // Indonesia_Western
    {  228, 1008,  224443,  224453,  224471, 10, 18, 14, }, // Iran
    {  228, 1019,  224485,  224497,  224517, 12, 20, 16, }, // Irkutsk
    {  228, 1027,  224533,  224545,  224565, 12, 20, 16, }, // Israel
    {  228, 1034,  224581,  224592,  224611, 11, 19, 15, }, // Japan
    {  228, 1040,  224626,  224654,  224675, 28, 21, 17, }, // Kamchatka
    {  228, 1058,       0,  224692,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  228, 1077,       0,  224710,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  228, 1106,  224730,  224739,  224756,  9, 17, 13, }, // Korea
    {  228, 1112,       0,  224769,       0,  0, 10,  0, }, // Kosrae
    {  228, 1119,  224779,  224795,  224819, 16, 24, 20, }, // Krasnoyarsk
    {  228, 1151,       0,  224839,       0,  0, 15,  0, }, // Kyrgystan
    {  228, 1161,       0,  224854,       0,  0, 13,  0, }, // Lanka
    {  228, 1175,       0,  224867,       0,  0, 17,  0, }, // Line_Islands
    {  228, 1188,  224884,  224899,  224922, 15, 23, 19, }, // Lord_Howe
    {  228, 1198,  224941,  224950,  224967,  9, 17, 13, }, // Macau
    {  228, 1204,  224980,  224992,  225012, 12, 20, 16, }, // Magadan
    {  228, 1219,       0,  225028,       0,  0, 12,  0, }, // Malaysia
    {  228, 1228,       0,  225040,       0,  0, 12,  0, }, // Maldives
    {  228, 1237,       0,  225052,       0,  0, 16,  0, }, // Marquesas
    {  228, 1247,       0,  225068,       0,  0, 20,  0, }, // Marshall_Islands
    {  228, 1264,  225088,  225102,  225124, 14, 22, 18, }, // Mauritius
    {  228, 1274,       0,  225142,       0,  0, 11,  0, }, // Mawson
    {  228, 1281,  225153,  225179,  225213, 26, 34, 30, }, // Mexico_Pacific
    {  228, 1296,  225243,  225259,  225283, 16, 24, 20, }, // Mongolia
    {  228, 1305,  225303,  225313,  225331, 10, 18, 14, }, // Moscow
    {  228, 1312,       0,  225345,       0,  0,  9,  0, }, // Myanmar
    {  228, 1320,       0,  225354,       0,  0,  9,  0, }, // Nauru
    {  228, 1326,       0,  225363,       0,  0, 10,  0, }, // Nepal
    {  228, 1332,  225373,  225391,  225417, 18, 26, 22, }, // New_Caledonia
    {  228, 1346,  225439,  225454,  225477, 15, 23, 19, }, // New_Zealand
    {  228, 1358,  225496,  225513,  225538, 17, 25, 21, }, // Newfoundland
    {  228, 1371,       0,  225559,       0,  0,  8,  0, }, // Niue
    {  228, 1376,  225567,  225585,  225611, 18, 26, 22, }, // Norfolk
    {  228, 1384,  225633,  225656,  225687, 23, 31, 27, }, // Noronha
    {  228, 1392,       0,  225714,       0,  0, 18,  0, }, // North_Mariana
    {  228, 1406,  225732,  225748,  225772, 16, 24, 20, }, // Novosibirsk
    {  228, 1418,  225792,  225801,  225818,  9, 17, 13, }, // Omsk
    {  228, 1428,  225831,  225844,  225865, 13, 21, 17, }, // Pakistan
    {  228, 1437,       0,  225882,       0,  0,  9,  0, }, // Palau
    {  228, 1443,       0,  225891,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  228, 1460,  225912,  225924,  225944, 12, 20, 16, }, // Paraguay
    {  228, 1469,  225960,  225969,  225986,  9, 17, 13, }, // Peru
    {  228, 1474,  225999,  226012,  226033, 13, 21, 17, }, // Philippines
    {  228, 1486,       0,  226050,       0,  0, 18,  0, }, // Phoenix_Islands
    {  228, 1502,  226068,  226099,  226138, 31, 39, 35, }, // Pierre_Miquelon
    {  228, 1518,       0,  226173,       0,  0, 13,  0, }, // Pitcairn
    {  228, 1527,       0,  226186,       0,  0, 11,  0, }, // Ponape
    {  228, 1534,       0,  226197,       0,  0, 14,  0, }, // Pyongyang
    {  228, 1544,  226211,  226224,  226245, 13, 21, 17, }, // Qyzylorda
    {  228, 1554,       0,  226262,       0,  0, 12,  0, }, // Reunion
    {  228, 1562,       0,  226274,       0,  0, 11,  0, }, // Rothera
    {  228, 1570,  226285,  226298,  226319, 13, 21, 17, }, // Sakhalin
    {  228, 1579,  226336,  226346,  226364, 10, 18, 14, }, // Samara
    {  228, 1596,  226378,  226387,  226404,  9, 17, 13, }, // Samoa
    {  228, 1602,       0,  226417,       0,  0, 12,  0, }, // Seychelles
    {  228, 1624,       0,  226429,       0,  0, 21,  0, }, // Singapore
    {  228, 1634,       0,  226450,       0,  0, 20,  0, }, // Solomon
    {  228, 1642,       0,  226470,       0,  0, 17,  0, }, // South_Georgia
    {  228, 1656,       0,  226487,       0,  0, 12,  0, }, // Suriname
    {  228, 1676,       0,  226499,       0,  0,  9,  0, }, // Syowa
    {  228, 1682,       0,  226508,       0,  0, 10,  0, }, // Tahiti
    {  228, 1689,  226518,  226528,  226546, 10, 18, 14, }, // Taipei
    {  228, 1696,       0,  226560,       0,  0, 16,  0, }, // Tajikistan
    {  228, 1724,       0,  226576,       0,  0, 11,  0, }, // Tokelau
    {  228, 1732,  226587,  226596,  226613,  9, 17, 13, }, // Tonga
    {  228, 1738,       0,  226626,       0,  0, 10,  0, }, // Truk
    {  228, 1750,  226636,  226653,  226678, 17, 25, 21, }, // Turkmenistan
    {  228, 1763,       0,  226699,       0,  0, 10,  0, }, // Tuvalu
    {  228, 1777,  226709,  226720,  226739, 11, 19, 15, }, // Uruguay
    {  228, 1792,  226754,  226769,  226792, 15, 23, 19, }, // Uzbekistan
    {  228, 1803,  226811,  226822,  226841, 11, 19, 15, }, // Vanuatu
    {  228, 1811,       0,  226856,       0,  0, 13,  0, }, // Venezuela
    {  228, 1821,  226869,  226885,  226909, 16, 24, 20, }, // Vladivostok
    {  228, 1833,  226929,  226943,  226965, 14, 22, 18, }, // Volgograd
    {  228, 1843,       0,  226983,       0,  0, 11,  0, }, // Vostok
    {  228, 1850,       0,  226994,       0,  0, 10,  0, }, // Wake
    {  228, 1855,       0,  227004,       0,  0, 21,  0, }, // Wallis
    {  228, 1862,  227025,  227037,  227057, 12, 20, 16, }, // Yakutsk
    {  228, 1870,  227073,  227091,  227117, 18, 26, 22, }, // Yekaterinburg
    {  228, 1892,       0,  227139,       0,  0, 10,  0, }, // Yukon
    {  229,    0,  227149,  227162,  227181, 13, 19, 23, }, // Acre Ewe/Latin/Ghana
    {  229,    5,       0,  227204,       0,  0, 21,  0, }, // Afghanistan
    {  229,   17,       0,  227225,       0,  0, 24,  0, }, // Africa_Central
    {  229,   32,       0,  227249,       0,  0, 21,  0, }, // Africa_Eastern
    {  229,   65,       0,  227270,       0,  0, 29,  0, }, // Africa_Southern
    {  229,   81,  227299,  227320,  227348, 21, 28, 31, }, // Africa_Western
    {  229,  107,  227379,  227394,  227417, 15, 23, 21, }, // Alaska
    {  229,  128,  227438,  227453,  227474, 15, 21, 25, }, // Almaty
    {  229,  135,  227499,  227514,  227537, 15, 23, 26, }, // Amazon
    {  229,  142,  227563,  227586,  227617, 23, 31, 29, }, // America_Central
    {  229,  158,  227646,  227671,  227703, 25, 32, 30, }, // America_Eastern
    {  229,  174,  227733,  227751,  227776, 18, 25, 23, }, // America_Mountain
    {  229,  191,  227799,  227815,  227839, 16, 24, 22, }, // America_Pacific
    {  229,  207,  227861,  227876,  227897, 15, 21, 23, }, // Anadyr
    {  229,  214,  227920,  227934,  227955, 14, 21, 19, }, // Apia
    {  229,  219,  227974,  227988,  228008, 14, 20, 24, }, // Aqtau
    {  229,  225,  228032,  228047,  228068, 15, 21, 16, }, // Aqtobe
    {  229,  232,  228084,  228100,  228123, 16, 23, 26, }, // Arabian
    {  229,  240,  228149,  228168,  228194, 19, 26, 29, }, // Argentina
    {  229,  250,  228223,  228251,  228286, 28, 35, 38, }, // Argentina_Western
    {  229,  268,  228324,  228341,  228365, 17, 24, 27, }, // Armenia
    {  229,  286,  228392,  228409,  228433, 17, 24, 22, }, // Atlantic
    {  229,  295,  228455,  228482,  228517, 27, 35, 36, }, // Australia_Central
    {  229,  313,  228553,  228600,  228643, 47, 43, 41, }, // Australia_CentralWestern
    {  229,  338,  228684,  228711,  228746, 27, 35, 33, }, // Australia_Eastern
    {  229,  356,  228779,  228806,  228841, 27, 35, 33, }, // Australia_Western
    {  229,  374,  228874,  228894,  228921, 20, 27, 30, }, // Azerbaijan
    {  229,  385,  228951,  228967,  228990, 16, 23, 26, }, // Azores
    {  229,  397,  229016,  229036,  229063, 20, 27, 30, }, // Bangladesh
    {  229,  415,       0,  229093,       0,  0, 16,  0, }, // Bhutan
    {  229,  422,       0,  229109,       0,  0, 17,  0, }, // Bolivia
    {  229,  437,  229126,  229144,  229169, 18, 25, 28, }, // Brasilia
    {  229,  454,       0,  229197,       0,  0, 27,  0, }, // Brunei
    {  229,  461,  229224,  229244,  229271, 20, 27, 30, }, // Cape_Verde
    {  229,  478,       0,  229301,       0,  0, 18,  0, }, // Chamorro
    {  229,  487,  229319,  229336,  229360, 17, 24, 22, }, // Chatham
    {  229,  495,  229382,  229397,  229419, 15, 22, 25, }, // Chile
    {  229,  501,  229444,  229459,  229481, 15, 22, 20, }, // China
    {  229,  507,  229501,  229521,  229548, 20, 27, 30, }, // Choibalsan
    {  229,  518,       0,  229578,       0,  0, 26,  0, }, // Christmas
    {  229,  528,       0,  229604,       0,  0, 23,  0, }, // Cocos
    {  229,  534,  229627,  229645,  229670, 18, 25, 28, }, // Colombia
    {  229,  543,  229698,  229720,  229749, 22, 29, 32, }, // Cook
    {  229,  548,  229781,  229794,  229814, 13, 20, 18, }, // Cuba
    {  229,  559,       0,  229832,       0,  0, 15,  0, }, // Davis
    {  229,  575,       0,  229847,       0,  0, 26,  0, }, // DumontDUrville
    {  229,  612,       0,  229873,       0,  0, 20,  0, }, // East_Timor
    {  229,  623,  229893,  229916,  229946, 23, 30, 33, }, // Easter
    {  229,  630,       0,  229979,       0,  0, 17,  0, }, // Ecuador
    {  229,  638,  229996,  230020,  230051, 24, 31, 34, }, // Europe_Central
    {  229,  653,  230085,  230108,  230137, 23, 29, 31, }, // Europe_Eastern
    {  229,  691,  230168,  230192,  230223, 24, 31, 34, }, // Europe_Western
    {  229,  706,  230257,  230283,  230316, 26, 33, 36, }, // Falkland
    {  229,  715,  230352,  230366,  230387, 14, 21, 24, }, // Fiji
    {  229,  720,       0,  230411,       0,  0, 23,  0, }, // French_Guiana
    {  229,  734,       0,  230434,       0,  0, 37,  0, }, // French_Southern
    {  229,  757,       0,  230471,       0,  0, 19,  0, }, // Galapagos
    {  229,  767,       0,  230490,       0,  0, 17,  0, }, // Gambier
    {  229,  775,  230507,  230524,  230548, 17, 24, 27, }, // Georgia
    {  229,  783,       0,  230575,       0,  0, 25,  0, }, // Gilbert_Islands
    {  229,  799,       0,  230600,       0,  0, 19,  0, }, // GMT
    {  229,  841,  230619,  230642,  230673, 23, 31, 34, }, // Greenland_Eastern
    {  229,  859,  230707,  230731,  230762, 24, 31, 29, }, // Greenland_Western
    {  229,  882,       0,  230791,       0,  0, 21,  0, }, // Gulf
    {  229,  887,       0,  230812,       0,  0, 16,  0, }, // Guyana
    {  229,  894,  230828,  230852,  230883, 24, 31, 29, }, // Hawaii_Aleutian
    {  229,  910,  230912,  230931,  230957, 19, 26, 29, }, // Hong_Kong
    {  229,  920,  230986,  231000,  231021, 14, 21, 24, }, // Hovd
    {  229,  925,       0,  231045,       0,  0, 15,  0, }, // India
    {  229,  931,       0,  231060,       0,  0, 22,  0, }, // Indian_Ocean
    {  229,  944,       0,  231082,       0,  0, 19,  0, }, // Indochina
    {  229,  954,       0,  231101,       0,  0, 27,  0, }, // Indonesia_Central
    {  229,  972,       0,  231128,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  229,  990,       0,  231155,       0,  0, 27,  0, }, // Indonesia_Western
    {  229, 1008,  231182,  231196,  231217, 14, 21, 19, }, // Iran
    {  229, 1019,  231236,  231253,  231277, 17, 24, 27, }, // Irkutsk
    {  229, 1027,  231304,  231320,  231343, 16, 23, 21, }, // Israel
    {  229, 1034,  231364,  231379,  231401, 15, 22, 25, }, // Japan
    {  229, 1040,  231426,  231459,  231498, 33, 39, 41, }, // Kamchatka
    {  229, 1058,       0,  231539,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {  229, 1077,       0,  231564,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  229, 1106,  231589,  231604,  231626, 15, 22, 25, }, // Korea
    {  229, 1112,       0,  231651,       0,  0, 16,  0, }, // Kosrae
    {  229, 1119,  231667,  231688,  231716, 21, 28, 31, }, // Krasnoyarsk
    {  229, 1151,       0,  231747,       0,  0, 19,  0, }, // Kyrgystan
    {  229, 1175,       0,  231766,       0,  0, 22,  0, }, // Line_Islands
    {  229, 1188,  231788,  231807,  231833, 19, 26, 24, }, // Lord_Howe
    {  229, 1198,  231857,  231871,  231891, 14, 20, 22, }, // Macau
    {  229, 1204,  231913,  231930,  231954, 17, 24, 27, }, // Magadan
    {  229, 1219,       0,  231981,       0,  0, 18,  0, }, // Malaysia
    {  229, 1228,       0,  231999,       0,  0, 18,  0, }, // Maldives
    {  229, 1237,       0,  232017,       0,  0, 19,  0, }, // Marquesas
    {  229, 1247,       0,  232036,       0,  0, 26,  0, }, // Marshall_Islands
    {  229, 1264,  232062,  232081,  232107, 19, 26, 29, }, // Mauritius
    {  229, 1274,       0,  232136,       0,  0, 16,  0, }, // Mawson
    {  229, 1281,  232152,  232177,  232209, 25, 32, 29, }, // Mexico_Pacific
    {  229, 1296,  232238,  232258,  232285, 20, 27, 30, }, // Mongolia
    {  229, 1305,  232315,  232331,  232354, 16, 23, 26, }, // Moscow
    {  229, 1312,       0,  232380,       0,  0, 17,  0, }, // Myanmar
    {  229, 1320,       0,  232397,       0,  0, 15,  0, }, // Nauru
    {  229, 1326,       0,  232412,       0,  0, 15,  0, }, // Nepal
    {  229, 1332,  232427,  232450,  232480, 23, 30, 33, }, // New_Caledonia
    {  229, 1346,  232513,  232534,  232562, 21, 28, 26, }, // New_Zealand
    {  229, 1358,  232588,  232609,  232637, 21, 28, 26, }, // Newfoundland
    {  229, 1371,       0,  232663,       0,  0, 14,  0, }, // Niue
    {  229, 1376,  232677,  232701,  232732, 24, 31, 34, }, // Norfolk
    {  229, 1384,  232766,  232795,  232831, 29, 36, 39, }, // Noronha
    {  229, 1406,  232870,  232891,  232919, 21, 28, 31, }, // Novosibirsk
    {  229, 1418,  232950,  232964,  232985, 14, 21, 24, }, // Omsk
    {  229, 1428,  233009,  233027,  233052, 18, 25, 28, }, // Pakistan
    {  229, 1437,       0,  233080,       0,  0, 15,  0, }, // Palau
    {  229, 1443,       0,  233095,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  229, 1460,  233121,  233139,  233164, 18, 25, 28, }, // Paraguay
    {  229, 1469,  233192,  233206,  233227, 14, 21, 23, }, // Peru
    {  229, 1474,  233250,  233270,  233297, 20, 27, 30, }, // Philippines
    {  229, 1486,       0,  233327,       0,  0, 25,  0, }, // Phoenix_Islands
    {  229, 1502,  233352,  233382,  233419, 30, 37, 35, }, // Pierre_Miquelon
    {  229, 1518,       0,  233454,       0,  0, 18,  0, }, // Pitcairn
    {  229, 1527,       0,  233472,       0,  0, 16,  0, }, // Ponape
    {  229, 1534,       0,  233488,       0,  0, 19,  0, }, // Pyongyang
    {  229, 1544,  233507,  233525,  233549, 18, 24, 28, }, // Qyzylorda
    {  229, 1554,       0,  233577,       0,  0, 17,  0, }, // Reunion
    {  229, 1562,       0,  233594,       0,  0, 17,  0, }, // Rothera
    {  229, 1570,  233611,  233629,  233654, 18, 25, 28, }, // Sakhalin
    {  229, 1579,  233682,  233697,  233718, 15, 21, 23, }, // Samara
    {  229, 1596,  233741,  233756,  233778, 15, 22, 20, }, // Samoa
    {  229, 1602,       0,  233798,       0,  0, 20,  0, }, // Seychelles
    {  229, 1624,       0,  233818,       0,  0, 26,  0, }, // Singapore
    {  229, 1634,       0,  233844,       0,  0, 25,  0, }, // Solomon
    {  229, 1642,       0,  233869,       0,  0, 23,  0, }, // South_Georgia
    {  229, 1656,       0,  233892,       0,  0, 17,  0, }, // Suriname
    {  229, 1676,       0,  233909,       0,  0, 15,  0, }, // Syowa
    {  229, 1682,       0,  233924,       0,  0, 16,  0, }, // Tahiti
    {  229, 1689,  233940,  233956,  233979, 16, 23, 21, }, // Taipei
    {  229, 1696,       0,  234000,       0,  0, 20,  0, }, // Tajikistan
    {  229, 1724,       0,  234020,       0,  0, 17,  0, }, // Tokelau
    {  229, 1732,  234037,  234052,  234074, 15, 22, 25, }, // Tonga
    {  229, 1738,       0,  234099,       0,  0, 15,  0, }, // Truk
    {  229, 1750,  234114,  234136,  234165, 22, 29, 32, }, // Turkmenistan
    {  229, 1763,       0,  234197,       0,  0, 16,  0, }, // Tuvalu
    {  229, 1777,  234213,  234230,  234254, 17, 24, 27, }, // Uruguay
    {  229, 1792,  234281,  234301,  234328, 20, 27, 30, }, // Uzbekistan
    {  229, 1803,  234358,  234375,  234399, 17, 24, 27, }, // Vanuatu
    {  229, 1811,       0,  234426,       0,  0, 19,  0, }, // Venezuela
    {  229, 1821,  234445,  234466,  234494, 21, 28, 31, }, // Vladivostok
    {  229, 1833,  234525,  234544,  234570, 19, 26, 28, }, // Volgograd
    {  229, 1843,       0,  234598,       0,  0, 16,  0, }, // Vostok
    {  229, 1850,       0,  234614,       0,  0, 21,  0, }, // Wake
    {  229, 1855,       0,  234635,       0,  0, 25,  0, }, // Wallis
    {  229, 1862,  234660,  234677,  234701, 17, 24, 27, }, // Yakutsk
    {  229, 1870,  234728,  234751,  234781, 23, 30, 33, }, // Yekaterinburg
    {  232,    5,       0,  234814,       0,  0, 14,  0, }, // Afghanistan Faroese/Latin/Faroe Islands
    {  232,   17,       0,  234828,       0,  0, 13,  0, }, // Africa_Central
    {  232,   32,       0,  234841,       0,  0, 16,  0, }, // Africa_Eastern
    {  232,   65,       0,  234857,       0,  0, 22,  0, }, // Africa_Southern
    {  232,   81,  234879,  234895,  234918, 16, 23, 22, }, // Africa_Western
    {  232,  107,  234940,  234950,  234967, 10, 17, 16, }, // Alaska
    {  232,  135,  234983,  234994,  235012, 11, 18, 17, }, // Amazon
    {  232,  142,  235029,  235040,  235058, 11, 18, 17, }, // America_Central
    {  232,  158,  235075,  235086,  235104, 11, 18, 17, }, // America_Eastern
    {  232,  174,  235121,  235133,  235152, 12, 19, 18, }, // America_Mountain
    {  232,  191,  235170,  235181,  235199, 11, 18, 17, }, // America_Pacific
    {  232,  214,  235216,  235224,  235239,  8, 15, 14, }, // Apia
    {  232,  232,  235253,  235264,  235282, 11, 18, 17, }, // Arabian
    {  232,  240,  235299,  235312,  235332, 13, 20, 19, }, // Argentina
    {  232,  250,  235351,  235371,  235398, 20, 27, 26, }, // Argentina_Western
    {  232,  268,  235424,  235435,  235453, 11, 18, 17, }, // Armenia
    {  232,  286,  235470,  235482,  235501, 12, 19, 18, }, // Atlantic
    {  232,  295,  235519,  235536,  235560, 17, 24, 23, }, // Australia_Central
    {  232,  313,  235583,  235606,  235636, 23, 30, 29, }, // Australia_CentralWestern
    {  232,  338,  235665,  235685,  235712, 20, 27, 26, }, // Australia_Eastern
    {  232,  356,  235586,  235609,  235639, 20, 27, 26, }, // Australia_Western
    {  232,  374,  235738,  235752,  235773, 14, 21, 20, }, // Azerbaijan
    {  232,  385,  235793,  235806,  235826, 13, 20, 19, }, // Azores
    {  232,  397,  235845,  235859,  235880, 14, 21, 20, }, // Bangladesh
    {  232,  415,       0,  235900,       0,  0,  9,  0, }, // Bhutan
    {  232,  422,       0,  235909,       0,  0, 11,  0, }, // Bolivia
    {  232,  437,  235920,  235932,  235951, 12, 19, 18, }, // Brasilia
    {  232,  454,       0,  235969,       0,  0, 21,  0, }, // Brunei
    {  232,  461,  235990,  236010,  236037, 20, 27, 26, }, // Cape_Verde
    {  232,  478,       0,  236063,       0,  0, 19,  0, }, // Chamorro
    {  232,  487,  236082,  236093,  236111, 11, 18, 17, }, // Chatham
    {  232,  495,  236128,  236136,  236151,  8, 15, 14, }, // Chile
    {  232,  501,  236165,  236173,  236188,  8, 15, 14, }, // China
    {  232,  507,  236202,  236216,  236237, 14, 21, 20, }, // Choibalsan
    {  232,  518,       0,  236257,       0,  0, 13,  0, }, // Christmas
    {  232,  528,       0,  236270,       0,  0, 16,  0, }, // Cocos
    {  232,  534,  236286,  236298,  236317, 12, 19, 18, }, // Colombia
    {  232,  543,  236335,  236351,  236374, 16, 23, 22, }, // Cook
    {  232,  548,  236396,  236404,  236419,  8, 15, 14, }, // Cuba
    {  232,  559,       0,  236433,       0,  0,  9,  0, }, // Davis
    {  232,  575,       0,  236442,       0,  0, 20,  0, }, // DumontDUrville
    {  232,  612,       0,  236462,       0,  0, 15,  0, }, // East_Timor
    {  232,  623,  236477,  236492,  236514, 15, 22, 21, }, // Easter
    {  232,  630,       0,  236535,       0,  0, 11,  0, }, // Ecuador
    {  232,  638,  236546,  236559,  236579, 13, 20, 19, }, // Europe_Central
    {  232,  653,  236598,  236614,  236637, 16, 23, 22, }, // Europe_Eastern
    {  232,  668,       0,  236659,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  232,  691,  236682,  236698,  236721, 16, 23, 22, }, // Europe_Western
    {  232,  706,  236743,  236763,  236790, 20, 27, 26, }, // Falkland
    {  232,  715,  236816,  236824,  236839,  8, 15, 14, }, // Fiji
    {  232,  720,       0,  236853,       0,  0, 18,  0, }, // French_Guiana
    {  232,  734,       0,  236871,       0,  0, 41,  0, }, // French_Southern
    {  232,  757,       0,  236912,       0,  0, 13,  0, }, // Galapagos
    {  232,  767,       0,  236925,       0,  0, 11,  0, }, // Gambier
    {  232,  775,  236936,  236947,  236965, 11, 18, 17, }, // Georgia
    {  232,  783,       0,  236982,       0,  0, 18,  0, }, // Gilbert_Islands
    {  232,  799,       0,  237000,       0,  0, 18,  0, }, // GMT
    {  232,  841,  237018,  237039,  237067, 21, 28, 27, }, // Greenland_Eastern
    {  232,  859,  237094,  237115,  237143, 21, 28, 27, }, // Greenland_Western
    {  232,  882,       0,  237170,       0,  0, 15,  0, }, // Gulf
    {  232,  887,       0,  236861,       0,  0, 10,  0, }, // Guyana
    {  232,  894,  237185,  237204,  237230, 19, 26, 25, }, // Hawaii_Aleutian
    {  232,  910,  237255,  237268,  237288, 13, 20, 19, }, // Hong_Kong
    {  232,  920,  237307,  237315,  237330,  8, 15, 14, }, // Hovd
    {  232,  925,       0,  237344,       0,  0,  9,  0, }, // India
    {  232,  931,       0,  237353,       0,  0, 12,  0, }, // Indian_Ocean
    {  232,  944,       0,  237365,       0,  0, 12,  0, }, // Indochina
    {  232,  954,       0,  237377,       0,  0, 17,  0, }, // Indonesia_Central
    {  232,  972,       0,  237394,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  232,  990,       0,  237414,       0,  0, 20,  0, }, // Indonesia_Western
    {  232, 1008,  237434,  237442,  237457,  8, 15, 14, }, // Iran
    {  232, 1019,  237471,  237482,  237500, 11, 18, 17, }, // Irkutsk
    {  232, 1027,  237517,  237527,  237544, 10, 17, 16, }, // Israel
    {  232, 1034,  237560,  237569,  237585,  9, 16, 15, }, // Japan
    {  232, 1058,       0,  237600,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  232, 1077,       0,  237620,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  232, 1106,  237640,  237649,  237665,  9, 16, 15, }, // Korea
    {  232, 1112,       0,  237680,       0,  0, 10,  0, }, // Kosrae
    {  232, 1119,  237690,  237705,  237727, 15, 22, 21, }, // Krasnoyarsk
    {  232, 1151,       0,  237748,       0,  0, 12,  0, }, // Kyrgystan
    {  232, 1175,       0,  237760,       0,  0, 15,  0, }, // Line_Islands
    {  232, 1188,  237775,  237788,  237808, 13, 20, 19, }, // Lord_Howe
    {  232, 1204,  237827,  237838,  237856, 11, 18, 17, }, // Magadan
    {  232, 1219,       0,  237873,       0,  0, 12,  0, }, // Malaysia
    {  232, 1228,       0,  237885,       0,  0, 17,  0, }, // Maldives
    {  232, 1237,       0,  237902,       0,  0, 13,  0, }, // Marquesas
    {  232, 1247,       0,  237915,       0,  0, 19,  0, }, // Marshall_Islands
    {  232, 1264,  237934,  237946,  237965, 12, 19, 18, }, // Mauritius
    {  232, 1274,       0,  237983,       0,  0, 10,  0, }, // Mawson
    {  232, 1281,  237993,  238012,  238038, 19, 26, 25, }, // Mexico_Pacific
    {  232, 1296,  238063,  238077,  238098, 14, 21, 20, }, // Mongolia
    {  232, 1305,  238118,  238128,  238145, 10, 17, 16, }, // Moscow
    {  232, 1312,       0,  238161,       0,  0, 19,  0, }, // Myanmar
    {  232, 1320,       0,  238180,       0,  0,  9,  0, }, // Nauru
    {  232, 1326,       0,  238189,       0,  0,  9,  0, }, // Nepal
    {  232, 1332,  238198,  238213,  238235, 15, 22, 21, }, // New_Caledonia
    {  232, 1346,  238256,  238268,  238287, 12, 19, 18, }, // New_Zealand
    {  232, 1358,  238305,  238321,  238344, 16, 23, 22, }, // Newfoundland
    {  232, 1371,       0,  238366,       0,  0,  8,  0, }, // Niue
    {  232, 1376,  238374,  238390,  238413, 16, 23, 22, }, // Norfolk
    {  232, 1384,  238435,  238458,  238488, 23, 30, 29, }, // Noronha
    {  232, 1406,  238517,  238532,  238554, 15, 22, 21, }, // Novosibirsk
    {  232, 1418,  238575,  238583,  238598,  8, 15, 14, }, // Omsk
    {  232, 1428,  238612,  238624,  238643, 12, 19, 18, }, // Pakistan
    {  232, 1437,       0,  238661,       0,  0,  9,  0, }, // Palau
    {  232, 1443,       0,  238670,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  232, 1460,  238688,  238700,  238719, 12, 19, 18, }, // Paraguay
    {  232, 1469,  238737,  238745,  238760,  8, 15, 14, }, // Peru
    {  232, 1474,  238774,  238791,  238815, 17, 24, 23, }, // Philippines
    {  232, 1486,       0,  238838,       0,  0, 18,  0, }, // Phoenix_Islands
    {  232, 1502,  238856,  238881,  238913, 25, 32, 31, }, // Pierre_Miquelon
    {  232, 1518,       0,  238944,       0,  0, 19,  0, }, // Pitcairn
    {  232, 1527,       0,  238963,       0,  0, 10,  0, }, // Ponape
    {  232, 1534,       0,  238973,       0,  0, 13,  0, }, // Pyongyang
    {  232, 1554,       0,  238986,       0,  0, 11,  0, }, // Reunion
    {  232, 1562,       0,  238997,       0,  0, 11,  0, }, // Rothera
    {  232, 1570,  239008,  239020,  239039, 12, 19, 18, }, // Sakhalin
    {  232, 1596,  239057,  239066,  239082,  9, 16, 15, }, // Samoa
    {  232, 1602,       0,  239097,       0,  0, 19,  0, }, // Seychelles
    {  232, 1624,       0,  239116,       0,  0, 12,  0, }, // Singapore
    {  232, 1634,       0,  239128,       0,  0, 18,  0, }, // Solomon
    {  232, 1642,       0,  239146,       0,  0, 23,  0, }, // South_Georgia
    {  232, 1656,       0,  239169,       0,  0, 11,  0, }, // Suriname
    {  232, 1676,       0,  239180,       0,  0,  9,  0, }, // Syowa
    {  232, 1682,       0,  239189,       0,  0, 10,  0, }, // Tahiti
    {  232, 1689,  239199,  239209,  239226, 10, 17, 16, }, // Taipei
    {  232, 1696,       0,  239242,       0,  0, 16,  0, }, // Tajikistan
    {  232, 1724,       0,  239258,       0,  0, 11,  0, }, // Tokelau
    {  232, 1732,  239269,  239278,  239294,  9, 16, 15, }, // Tonga
    {  232, 1738,       0,  239309,       0,  0,  9,  0, }, // Truk
    {  232, 1750,  239318,  239334,  239357, 16, 23, 22, }, // Turkmenistan
    {  232, 1763,       0,  239379,       0,  0, 10,  0, }, // Tuvalu
    {  232, 1777,  239389,  239400,  239418, 11, 18, 17, }, // Uruguay
    {  232, 1792,  239435,  239449,  239470, 14, 21, 20, }, // Uzbekistan
    {  232, 1803,  239490,  239501,  239519, 11, 18, 17, }, // Vanuatu
    {  232, 1811,       0,  239536,       0,  0, 13,  0, }, // Venezuela
    {  232, 1821,  239549,  239564,  239586, 15, 22, 21, }, // Vladivostok
    {  232, 1833,  239607,  239620,  239640, 13, 20, 19, }, // Volgograd
    {  232, 1843,       0,  239659,       0,  0, 10,  0, }, // Vostok
    {  232, 1850,       0,  239669,       0,  0, 13,  0, }, // Wake
    {  232, 1855,       0,  239682,       0,  0, 28,  0, }, // Wallis
    {  232, 1862,  239710,  239721,  239739, 11, 18, 17, }, // Yakutsk
    {  232, 1870,  239756,  239773,  239797, 17, 24, 23, }, // Yekaterinburg
    {  232, 1892,       0,  239820,       0,  0,  9,  0, }, // Yukon
    {  234,    5,       0,  139984,       0,  0, 19,  0, }, // Afghanistan Filipino/Latin/Philippines
    {  234,   17,       0,  239829,       0,  0, 22,  0, }, // Africa_Central
    {  234,   32,       0,  239851,       0,  0, 24,  0, }, // Africa_Eastern
    {  234,   65,       0,  239875,       0,  0, 20,  0, }, // Africa_Southern
    {  234,   81,  239895,  239919,  239955, 24, 36, 36, }, // Africa_Western
    {  234,  107,  140153,  239991,  140191, 14, 26, 23, }, // Alaska
    {  234,  135,  140214,  240017,  240043, 14, 26, 26, }, // Amazon
    {  234,  142,  240069,  240101,  240145, 32, 44, 44, }, // America_Central
    {  234,  158,  240189,  240225,  240274, 36, 49, 48, }, // America_Eastern
    {  234,  174,  240322,  240356,  240402, 34, 46, 46, }, // America_Mountain
    {  234,  191,  240448,  240484,  240532, 36, 48, 48, }, // America_Pacific
    {  234,  207,  240580,  240594,  240617, 14, 23, 21, }, // Anadyr
    {  234,  214,  140278,  240638,  140312, 12, 24, 21, }, // Apia
    {  234,  232,  140333,  240662,  240688, 14, 26, 23, }, // Arabian
    {  234,  240,  140397,  240711,  240740, 17, 29, 29, }, // Argentina
    {  234,  250,  240769,  240796,  240835, 27, 39, 39, }, // Argentina_Western
    {  234,  268,  140567,  240874,  240901, 15, 27, 27, }, // Armenia
    {  234,  286,  240928,  240945,  240974, 17, 29, 29, }, // Atlantic
    {  234,  295,  241003,  241028,  241065, 25, 37, 34, }, // Australia_Central
    {  234,  313,  241099,  241136,  241182, 37, 46, 44, }, // Australia_CentralWestern
    {  234,  338,  241226,  241253,  241292, 27, 39, 36, }, // Australia_Eastern
    {  234,  356,  241328,  241355,  241394, 27, 39, 36, }, // Australia_Western
    {  234,  374,  141046,  241430,  241460, 18, 30, 30, }, // Azerbaijan
    {  234,  385,  141122,  241490,  241516, 14, 26, 26, }, // Azores
    {  234,  397,  141186,  241542,  241572, 18, 30, 30, }, // Bangladesh
    {  234,  415,       0,  141262,       0,  0, 14,  0, }, // Bhutan
    {  234,  422,       0,  141276,       0,  0, 15,  0, }, // Bolivia
    {  234,  437,  141291,  241602,  241630, 16, 28, 28, }, // Brasilia
    {  234,  454,       0,  141361,       0,  0, 25,  0, }, // Brunei
    {  234,  461,  141386,  241658,  241688, 18, 30, 30, }, // Cape_Verde
    {  234,  478,       0,  241718,       0,  0, 28,  0, }, // Chamorro
    {  234,  487,  141488,  241746,  241773, 15, 27, 24, }, // Chatham
    {  234,  495,  141555,  241797,  241822, 13, 25, 25, }, // Chile
    {  234,  501,  241847,  241860,  241885, 13, 25, 22, }, // China
    {  234,  507,  141677,  241907,  241937, 18, 30, 30, }, // Choibalsan
    {  234,  518,       0,  141753,       0,  0, 24,  0, }, // Christmas
    {  234,  528,       0,  141777,       0,  0, 21,  0, }, // Cocos
    {  234,  534,  141798,  241967,  241995, 16, 28, 28, }, // Colombia
    {  234,  543,  141868,  242023,  242055, 20, 32, 44, }, // Cook
    {  234,  548,  141960,  242099,  242123, 12, 24, 24, }, // Cuba
    {  234,  559,       0,  142018,       0,  0, 13,  0, }, // Davis
    {  234,  575,       0,  142031,       0,  0, 24,  0, }, // DumontDUrville
    {  234,  612,       0,  142055,       0,  0, 18,  0, }, // East_Timor
    {  234,  623,  142073,  242147,  242180, 21, 33, 33, }, // Easter
    {  234,  630,       0,  142158,       0,  0, 15,  0, }, // Ecuador
    {  234,  638,  242213,  242235,  242269, 22, 34, 34, }, // Europe_Central
    {  234,  653,  242303,  242327,  242363, 24, 36, 36, }, // Europe_Eastern
    {  234,  668,       0,  242399,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  234,  691,  242430,  242454,  242490, 24, 36, 36, }, // Europe_Western
    {  234,  706,  142467,  242526,  242562, 24, 36, 36, }, // Falkland
    {  234,  715,  142561,  242598,  242622, 12, 24, 24, }, // Fiji
    {  234,  720,       0,  142619,       0,  0, 21,  0, }, // French_Guiana
    {  234,  734,       0,  242646,       0,  0, 38,  0, }, // French_Southern
    {  234,  757,       0,  142676,       0,  0, 17,  0, }, // Galapagos
    {  234,  767,       0,  142693,       0,  0, 15,  0, }, // Gambier
    {  234,  775,  142708,  242684,  242711, 15, 27, 27, }, // Georgia
    {  234,  783,       0,  142775,       0,  0, 23,  0, }, // Gilbert_Islands
    {  234,  799,       0,    2861,       0,  0, 19,  0, }, // GMT
    {  234,  841,  242738,  242765,  242804, 27, 39, 39, }, // Greenland_Eastern
    {  234,  859,  242843,  242870,  242909, 27, 39, 39, }, // Greenland_Western
    {  234,  882,       0,  143006,       0,  0, 12,  0, }, // Gulf
    {  234,  887,       0,  143018,       0,  0, 14,  0, }, // Guyana
    {  234,  894,  143032,  242948,  242983, 23, 35, 35, }, // Hawaii_Aleutian
    {  234,  910,  143123,  243018,  243047, 17, 29, 29, }, // Hong_Kong
    {  234,  920,  143196,  243076,  243100, 12, 24, 24, }, // Hovd
    {  234,  925,       0,  243124,       0,  0, 25,  0, }, // India
    {  234,  931,       0,  143277,       0,  0, 20,  0, }, // Indian_Ocean
    {  234,  944,       0,  143297,       0,  0, 17,  0, }, // Indochina
    {  234,  954,       0,  243149,       0,  0, 25,  0, }, // Indonesia_Central
    {  234,  972,       0,  243174,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  234,  990,       0,  243201,       0,  0, 27,  0, }, // Indonesia_Western
    {  234, 1008,  143389,  243228,  243252, 12, 24, 21, }, // Iran
    {  234, 1019,  143447,  243273,  243300, 15, 27, 27, }, // Irkutsk
    {  234, 1027,  143514,  243327,  243353, 14, 26, 23, }, // Israel
    {  234, 1034,  143578,  243376,  243401, 13, 25, 22, }, // Japan
    {  234, 1040,  243423,  243455,  243496, 32, 41, 39, }, // Kamchatka
    {  234, 1058,       0,  243535,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  234, 1077,       0,  243563,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  234, 1106,  143684,  243591,  243616, 13, 25, 22, }, // Korea
    {  234, 1112,       0,  143745,       0,  0, 14,  0, }, // Kosrae
    {  234, 1119,  143759,  243638,  243669, 19, 31, 31, }, // Krasnoyarsk
    {  234, 1151,       0,  243700,       0,  0, 17,  0, }, // Kyrgystan
    {  234, 1175,       0,  143856,       0,  0, 20,  0, }, // Line_Islands
    {  234, 1188,  143876,  243717,  243746, 17, 29, 27, }, // Lord_Howe
    {  234, 1204,  143949,  243773,  243800, 15, 27, 27, }, // Magadan
    {  234, 1219,       0,  144016,       0,  0, 16,  0, }, // Malaysia
    {  234, 1228,       0,  144032,       0,  0, 16,  0, }, // Maldives
    {  234, 1237,       0,  144048,       0,  0, 17,  0, }, // Marquesas
    {  234, 1247,       0,  144065,       0,  0, 24,  0, }, // Marshall_Islands
    {  234, 1264,  144089,  243827,  243856, 17, 29, 29, }, // Mauritius
    {  234, 1274,       0,  144162,       0,  0, 14,  0, }, // Mawson
    {  234, 1281,  243885,  243911,  243949, 26, 38, 38, }, // Mexico_Pacific
    {  234, 1296,  243987,  244005,  244035, 18, 30, 30, }, // Mongolia
    {  234, 1305,  144346,  244065,  244091, 14, 26, 26, }, // Moscow
    {  234, 1312,       0,  144410,       0,  0, 15,  0, }, // Myanmar
    {  234, 1320,       0,  144425,       0,  0, 13,  0, }, // Nauru
    {  234, 1326,       0,  144438,       0,  0, 13,  0, }, // Nepal
    {  234, 1332,  144451,  244117,  244150, 21, 33, 33, }, // New_Caledonia
    {  234, 1346,  144536,  244183,  244214, 19, 31, 28, }, // New_Zealand
    {  234, 1358,  144615,  244242,  244274, 20, 32, 32, }, // Newfoundland
    {  234, 1371,       0,  144697,       0,  0, 12,  0, }, // Niue
    {  234, 1376,  144709,  244306,  244340, 22, 34, 31, }, // Norfolk
    {  234, 1384,  144797,  244371,  244410, 27, 39, 39, }, // Noronha
    {  234, 1406,  144900,  244449,  244480, 19, 31, 31, }, // Novosibirsk
    {  234, 1418,  144979,  244511,  244535, 12, 24, 24, }, // Omsk
    {  234, 1428,  145037,  244559,  244587, 16, 28, 28, }, // Pakistan
    {  234, 1437,       0,  145107,       0,  0, 13,  0, }, // Palau
    {  234, 1443,       0,  145120,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  234, 1460,  145144,  244615,  244643, 16, 28, 28, }, // Paraguay
    {  234, 1469,  145214,  244671,  244695, 12, 24, 24, }, // Peru
    {  234, 1474,  145272,  244719,  244748, 17, 29, 29, }, // Philippines
    {  234, 1486,       0,  145345,       0,  0, 23,  0, }, // Phoenix_Islands
    {  234, 1502,  244777,  244808,  244851, 31, 43, 43, }, // Pierre_Miquelon
    {  234, 1518,       0,  145476,       0,  0, 16,  0, }, // Pitcairn
    {  234, 1527,       0,  145492,       0,  0, 14,  0, }, // Ponape
    {  234, 1534,       0,  145506,       0,  0, 17,  0, }, // Pyongyang
    {  234, 1554,       0,  145523,       0,  0, 15,  0, }, // Reunion
    {  234, 1562,       0,  145538,       0,  0, 15,  0, }, // Rothera
    {  234, 1570,  145553,  244894,  244922, 16, 28, 28, }, // Sakhalin
    {  234, 1579,  244950,  244964,  244987, 14, 23, 15, }, // Samara
    {  234, 1596,  145623,  245002,  145659, 13, 25, 22, }, // Samoa
    {  234, 1602,       0,  145681,       0,  0, 18,  0, }, // Seychelles
    {  234, 1624,       0,  245027,       0,  0, 29,  0, }, // Singapore
    {  234, 1634,       0,  145726,       0,  0, 23,  0, }, // Solomon
    {  234, 1642,       0,  245056,       0,  0, 21,  0, }, // South_Georgia
    {  234, 1656,       0,  145770,       0,  0, 16,  0, }, // Suriname
    {  234, 1676,       0,  145786,       0,  0, 13,  0, }, // Syowa
    {  234, 1682,       0,  145799,       0,  0, 14,  0, }, // Tahiti
    {  234, 1689,  145813,  245077,  245103, 14, 26, 23, }, // Taipei
    {  234, 1696,       0,  145877,       0,  0, 18,  0, }, // Tajikistan
    {  234, 1724,       0,  145895,       0,  0, 15,  0, }, // Tokelau
    {  234, 1732,  145910,  245126,  245151, 13, 25, 25, }, // Tonga
    {  234, 1738,       0,  145971,       0,  0, 13,  0, }, // Truk
    {  234, 1750,  145984,  245176,  245208, 20, 32, 32, }, // Turkmenistan
    {  234, 1763,       0,  146066,       0,  0, 14,  0, }, // Tuvalu
    {  234, 1777,  146080,  245240,  245267, 15, 27, 27, }, // Uruguay
    {  234, 1792,  146147,  245294,  245324, 18, 30, 30, }, // Uzbekistan
    {  234, 1803,  146223,  245354,  245381, 15, 27, 27, }, // Vanuatu
    {  234, 1811,       0,  146290,       0,  0, 17,  0, }, // Venezuela
    {  234, 1821,  146307,  245408,  245439, 19, 31, 31, }, // Vladivostok
    {  234, 1833,  146386,  245470,  245499, 17, 29, 29, }, // Volgograd
    {  234, 1843,       0,  146459,       0,  0, 14,  0, }, // Vostok
    {  234, 1850,       0,  146473,       0,  0, 19,  0, }, // Wake
    {  234, 1855,       0,  146492,       0,  0, 23,  0, }, // Wallis
    {  234, 1862,  146515,  245528,  245555, 15, 27, 27, }, // Yakutsk
    {  234, 1870,  146582,  245582,  245615, 21, 33, 33, }, // Yekaterinburg
    {  234, 1892,       0,  146667,       0,  0, 13,  0, }, // Yukon
    {  235,    0,  245648,  245658,  245676, 10, 18, 14, }, // Acre Finnish/Latin/Finland
    {  235,    5,       0,  245690,       0,  0, 17,  0, }, // Afghanistan
    {  235,   17,       0,  245707,       0,  0, 18,  0, }, // Africa_Central
    {  235,   32,       0,  245725,       0,  0, 16,  0, }, // Africa_Eastern
    {  235,   65,       0,  245741,       0,  0, 18,  0, }, // Africa_Southern
    {  235,   81,  245759,  245777,  245803, 18, 26, 22, }, // Africa_Western
    {  235,  107,  245825,  245837,  245857, 12, 20, 16, }, // Alaska
    {  235,  128,  245873,  245885,  245905, 12, 20, 16, }, // Almaty
    {  235,  135,  245921,  245934,  245955, 13, 21, 17, }, // Amazon
    {  235,  142,  245972,  245998,  246032, 26, 34, 30, }, // America_Central
    {  235,  158,  246062,  246087,  246120, 25, 33, 29, }, // America_Eastern
    {  235,  174,  246149,  246167,  246193, 18, 26, 22, }, // America_Mountain
    {  235,  191,  246215,  246244,  246281, 29, 37, 33, }, // America_Pacific
    {  235,  207,  246314,  246327,  246348, 13, 21, 17, }, // Anadyr
    {  235,  214,  246365,  246375,  246393, 10, 18, 14, }, // Apia
    {  235,  219,  246407,  246419,  246439, 12, 20, 16, }, // Aqtau
    {  235,  225,  246455,  246467,  246487, 12, 20, 16, }, // Aqtobe
    {  235,  232,  246503,  246521,  246547, 18, 26, 22, }, // Arabian
    {  235,  240,  246569,  246585,  246609, 16, 24, 20, }, // Argentina
    {  235,  250,  246629,  246651,  246681, 22, 30, 26, }, // Argentina_Western
    {  235,  268,  246707,  246720,  246741, 13, 21, 17, }, // Armenia
    {  235,  286,  246758,  246779,  246808, 21, 29, 25, }, // Atlantic
    {  235,  295,  246833,  246854,  246883, 21, 29, 25, }, // Australia_Central
    {  235,  313,  246908,  246938,  246976, 30, 38, 34, }, // Australia_CentralWestern
    {  235,  338,  247010,  247029,  247056, 19, 27, 23, }, // Australia_Eastern
    {  235,  356,  247079,  247100,  247129, 21, 29, 25, }, // Australia_Western
    {  235,  374,  247154,  247172,  247198, 18, 26, 22, }, // Azerbaijan
    {  235,  385,  247220,  247232,  247252, 12, 20, 16, }, // Azores
    {  235,  397,  247268,  247285,  247310, 17, 25, 21, }, // Bangladesh
    {  235,  415,       0,  247331,       0,  0, 13,  0, }, // Bhutan
    {  235,  422,       0,  247344,       0,  0, 13,  0, }, // Bolivia
    {  235,  437,  247357,  247371,  247393, 14, 22, 18, }, // Brasilia
    {  235,  454,       0,  247411,       0,  0, 12,  0, }, // Brunei
    {  235,  461,  247423,  247438,  247461, 15, 23, 19, }, // Cape_Verde
    {  235,  472,       0,  247480,       0,  0, 11,  0, }, // Casey
    {  235,  478,       0,  247491,       0,  0, 14,  0, }, // Chamorro
    {  235,  487,  247505,  247519,  247541, 14, 22, 18, }, // Chatham
    {  235,  495,  247559,  247570,  247589, 11, 19, 15, }, // Chile
    {  235,  501,  247604,  247615,  247634, 11, 19, 15, }, // China
    {  235,  507,  247649,  247664,  247687, 15, 23, 19, }, // Choibalsan
    {  235,  518,       0,  247706,       0,  0, 16,  0, }, // Christmas
    {  235,  528,       0,  247722,       0,  0, 18,  0, }, // Cocos
    {  235,  534,  247740,  247754,  247776, 14, 22, 18, }, // Colombia
    {  235,  543,  247794,  247812,  247838, 18, 26, 22, }, // Cook
    {  235,  548,  247860,  247871,  247890, 11, 19, 15, }, // Cuba
    {  235,  559,       0,  247905,       0,  0, 12,  0, }, // Davis
    {  235,  575,       0,  247917,       0,  0, 22,  0, }, // DumontDUrville
    {  235,  612,       0,  247939,       0,  0, 16,  0, }, // East_Timor
    {  235,  623,  247955,  247974,  248001, 19, 27, 23, }, // Easter
    {  235,  630,       0,  248024,       0,  0, 14,  0, }, // Ecuador
    {  235,  638,  248038,  248057,  248084, 19, 27, 23, }, // Europe_Central
    {  235,  653,  248107,  248124,  248149, 17, 25, 21, }, // Europe_Eastern
    {  235,  668,       0,  248170,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  235,  691,  248194,  248213,  248240, 19, 27, 23, }, // Europe_Western
    {  235,  706,  248263,  248285,  248315, 22, 30, 26, }, // Falkland
    {  235,  715,  248341,  248352,  248371, 11, 19, 15, }, // Fiji
    {  235,  720,       0,  248386,       0,  0, 21,  0, }, // French_Guiana
    {  235,  734,       0,  248407,       0,  0, 48,  0, }, // French_Southern
    {  235,  757,       0,  248455,       0,  0, 21,  0, }, // Galapagos
    {  235,  767,       0,  248476,       0,  0, 19,  0, }, // Gambier
    {  235,  775,  248495,  248508,  248529, 13, 21, 17, }, // Georgia
    {  235,  783,       0,  248546,       0,  0, 19,  0, }, // Gilbert_Islands
    {  235,  799,       0,  248565,       0,  0, 24,  0, }, // GMT
    {  235,  841,  248589,  248608,  248635, 19, 27, 23, }, // Greenland_Eastern
    {  235,  859,  248658,  248679,  248708, 21, 29, 25, }, // Greenland_Western
    {  235,  877,       0,  248733,       0,  0, 11,  0, }, // Guam
    {  235,  882,       0,  248744,       0,  0, 31,  0, }, // Gulf
    {  235,  887,       0,  248775,       0,  0, 12,  0, }, // Guyana
    {  235,  894,  248787,  248810,  248841, 23, 31, 27, }, // Hawaii_Aleutian
    {  235,  910,  248868,  248883,  248906, 15, 23, 19, }, // Hong_Kong
    {  235,  920,  248925,  248936,  248955, 11, 19, 15, }, // Hovd
    {  235,  925,       0,  248970,       0,  0, 11,  0, }, // India
    {  235,  931,       0,  248981,       0,  0, 22,  0, }, // Indian_Ocean
    {  235,  944,       0,  249003,       0,  0, 15,  0, }, // Indochina
    {  235,  954,       0,  249018,       0,  0, 21,  0, }, // Indonesia_Central
    {  235,  972,       0,  249039,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  235,  990,       0,  249058,       0,  0, 21,  0, }, // Indonesia_Western
    {  235, 1008,  249079,  249090,  249109, 11, 19, 15, }, // Iran
    {  235, 1019,  249124,  249138,  249160, 14, 22, 18, }, // Irkutsk
    {  235, 1027,  249178,  249191,  249212, 13, 21, 17, }, // Israel
    {  235, 1034,  249229,  249241,  249261, 12, 20, 16, }, // Japan
    {  235, 1040,  249277,  249292,  249315, 15, 23, 19, }, // Kamchatka
    {  235, 1058,       0,  249334,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  235, 1077,       0,  249354,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  235, 1106,  249376,  249387,  249406, 11, 19, 15, }, // Korea
    {  235, 1112,       0,  249421,       0,  0, 12,  0, }, // Kosrae
    {  235, 1119,  249433,  249451,  249477, 18, 26, 22, }, // Krasnoyarsk
    {  235, 1151,       0,  249499,       0,  0, 14,  0, }, // Kyrgystan
    {  235, 1161,       0,  249513,       0,  0, 15,  0, }, // Lanka
    {  235, 1175,       0,  249528,       0,  0, 16,  0, }, // Line_Islands
    {  235, 1188,  249544,  249559,  249582, 15, 23, 19, }, // Lord_Howe
    {  235, 1198,  249601,  249612,  249631, 11, 19, 15, }, // Macau
    {  235, 1204,  249646,  249660,  249682, 14, 22, 18, }, // Magadan
    {  235, 1219,       0,  249700,       0,  0, 13,  0, }, // Malaysia
    {  235, 1228,       0,  249713,       0,  0, 16,  0, }, // Maldives
    {  235, 1237,       0,  249729,       0,  0, 21,  0, }, // Marquesas
    {  235, 1247,       0,  249750,       0,  0, 22,  0, }, // Marshall_Islands
    {  235, 1264,  249772,  249789,  249814, 17, 25, 21, }, // Mauritius
    {  235, 1274,       0,  249835,       0,  0, 13,  0, }, // Mawson
    {  235, 1281,  249848,  249873,  249906, 25, 33, 29, }, // Mexico_Pacific
    {  235, 1296,  249935,  249952,  249977, 17, 25, 21, }, // Mongolia
    {  235, 1305,  249998,  250011,  250032, 13, 21, 17, }, // Moscow
    {  235, 1312,       0,  250049,       0,  0, 14,  0, }, // Myanmar
    {  235, 1320,       0,  250063,       0,  0, 11,  0, }, // Nauru
    {  235, 1326,       0,  250074,       0,  0, 12,  0, }, // Nepal
    {  235, 1332,  250086,  250107,  250136, 21, 29, 25, }, // New_Caledonia
    {  235, 1346,  250161,  250181,  250209, 20, 28, 24, }, // New_Zealand
    {  235, 1358,  250233,  250252,  250279, 19, 27, 23, }, // Newfoundland
    {  235, 1371,       0,  250302,       0,  0, 10,  0, }, // Niue
    {  235, 1376,  250312,  250332,  250360, 20, 28, 24, }, // Norfolk
    {  235, 1384,  250384,  250409,  250442, 25, 33, 29, }, // Noronha
    {  235, 1392,       0,  250471,       0,  0, 23,  0, }, // North_Mariana
    {  235, 1406,  250494,  250512,  250538, 18, 26, 22, }, // Novosibirsk
    {  235, 1418,  250560,  250571,  250590, 11, 19, 15, }, // Omsk
    {  235, 1428,  250605,  250620,  250643, 15, 23, 19, }, // Pakistan
    {  235, 1437,       0,  250662,       0,  0, 11,  0, }, // Palau
    {  235, 1443,       0,  250673,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  235, 1460,  250697,  250711,  250733, 14, 22, 18, }, // Paraguay
    {  235, 1469,  250751,  250761,  250779, 10, 18, 14, }, // Peru
    {  235, 1474,  250793,  250810,  250835, 17, 25, 21, }, // Philippines
    {  235, 1486,       0,  250856,       0,  0, 19,  0, }, // Phoenix_Islands
    {  235, 1502,  250875,  250907,  250947, 32, 40, 36, }, // Pierre_Miquelon
    {  235, 1518,       0,  250983,       0,  0, 15,  0, }, // Pitcairn
    {  235, 1527,       0,  250998,       0,  0, 13,  0, }, // Ponape
    {  235, 1534,       0,  251011,       0,  0, 16,  0, }, // Pyongyang
    {  235, 1544,  251027,  251042,  251065, 15, 23, 19, }, // Qyzylorda
    {  235, 1554,       0,  251084,       0,  0, 14,  0, }, // Reunion
    {  235, 1562,       0,  251098,       0,  0, 13,  0, }, // Rothera
    {  235, 1570,  251111,  251125,  251147, 14, 22, 18, }, // Sakhalin
    {  235, 1579,  251165,  251177,  251197, 12, 20, 16, }, // Samara
    {  235, 1596,  251213,  251224,  251243, 11, 19, 15, }, // Samoa
    {  235, 1602,       0,  251258,       0,  0, 16,  0, }, // Seychelles
    {  235, 1624,       0,  251274,       0,  0, 15,  0, }, // Singapore
    {  235, 1634,       0,  251289,       0,  0, 19,  0, }, // Solomon
    {  235, 1642,       0,  251308,       0,  0, 19,  0, }, // South_Georgia
    {  235, 1656,       0,  251327,       0,  0, 14,  0, }, // Suriname
    {  235, 1676,       0,  251341,       0,  0, 11,  0, }, // Syowa
    {  235, 1682,       0,  251352,       0,  0, 12,  0, }, // Tahiti
    {  235, 1689,  251364,  251376,  251396, 12, 20, 16, }, // Taipei
    {  235, 1696,       0,  251412,       0,  0, 18,  0, }, // Tajikistan
    {  235, 1724,       0,  251430,       0,  0, 13,  0, }, // Tokelau
    {  235, 1732,  251443,  251454,  251473, 11, 19, 15, }, // Tonga
    {  235, 1738,       0,  251488,       0,  0, 12,  0, }, // Truk
    {  235, 1750,  251500,  251519,  251546, 19, 27, 23, }, // Turkmenistan
    {  235, 1763,       0,  251569,       0,  0, 12,  0, }, // Tuvalu
    {  235, 1777,  251581,  251594,  251615, 13, 21, 17, }, // Uruguay
    {  235, 1792,  251632,  251649,  251674, 17, 25, 21, }, // Uzbekistan
    {  235, 1803,  251695,  251708,  251729, 13, 21, 17, }, // Vanuatu
    {  235, 1811,       0,  251746,       0,  0, 15,  0, }, // Venezuela
    {  235, 1821,  251761,  251779,  251805, 18, 26, 22, }, // Vladivostok
    {  235, 1833,  251827,  251843,  251867, 16, 24, 20, }, // Volgograd
    {  235, 1843,       0,  251887,       0,  0, 13,  0, }, // Vostok
    {  235, 1850,       0,  251900,       0,  0, 10,  0, }, // Wake
    {  235, 1855,       0,  251910,       0,  0, 24,  0, }, // Wallis
    {  235, 1862,  251934,  251948,  251970, 14, 22, 18, }, // Yakutsk
    {  235, 1870,  251988,  252008,  252036, 20, 28, 24, }, // Yekaterinburg
    {  235, 1892,       0,  252060,       0,  0, 12,  0, }, // Yukon
    {  236,    0,  252072,  252087,  252110, 15, 23, 21, }, // Acre French/Latin/France
    {  236,    5,       0,  252131,       0,  0, 22,  0, }, // Afghanistan
    {  236,   17,       0,  252153,       0,  0, 32,  0, }, // Africa_Central
    {  236,   32,       0,  252185,       0,  0, 32,  0, }, // Africa_Eastern
    {  236,   65,       0,  252217,       0,  0, 35,  0, }, // Africa_Southern
    {  236,   81,  252252,  252278,  252312, 26, 34, 32, }, // Africa_Western
    {  236,  107,  252344,  252361,  252386, 17, 25, 23, }, // Alaska
    {  236,  128,  252409,  252425,  252449, 16, 24, 22, }, // Almaty
    {  236,  135,  252471,  252490,  252517, 19, 27, 25, }, // Amazon
    {  236,  142,  252542,  252572,  252610, 30, 38, 36, }, // America_Central
    {  236,  158,  252646,  252675,  252712, 29, 37, 35, }, // America_Eastern
    {  236,  174,  252747,  252766,  252793, 19, 27, 25, }, // America_Mountain
    {  236,  191,  252818,  252851,  252892, 33, 41, 39, }, // America_Pacific
    {  236,  207,  252931,  252945,  252967, 14, 22, 20, }, // Anadyr
    {  236,  214,  252987,  252999,  253019, 12, 20, 18, }, // Apia
    {  236,  219,  253037,  253051,  253073, 14, 22, 20, }, // Aqtau
    {  236,  225,  253093,  253107,  253129, 14, 22, 20, }, // Aqtobe
    {  236,  232,  253149,  253166,  253191, 17, 25, 23, }, // Arabian
    {  236,  240,  253214,  253234,  253259, 20, 25, 26, }, // Argentina
    {  236,  250,  253285,  253310,  253343, 25, 33, 31, }, // Argentina_Western
    {  236,  268,  253374,  253392,  253418, 18, 26, 21, }, // Armenia
    {  236,  286,  253439,  253460,  253489, 21, 29, 27, }, // Atlantic
    {  236,  295,  253516,  253546,  253584, 30, 38, 36, }, // Australia_Central
    {  236,  313,  253620,  253656,  253700, 36, 44, 42, }, // Australia_CentralWestern
    {  236,  338,  253742,  253771,  253808, 29, 37, 35, }, // Australia_Eastern
    {  236,  356,  253843,  253874,  253913, 31, 39, 37, }, // Australia_Western
    {  236,  374,  253950,  253972,  254002, 22, 30, 25, }, // Azerbaijan
    {  236,  385,  254027,  254043,  254067, 16, 24, 22, }, // Azores
    {  236,  397,  254089,  254108,  254135, 19, 27, 25, }, // Bangladesh
    {  236,  415,       0,  254160,       0,  0, 16,  0, }, // Bhutan
    {  236,  422,       0,  254176,       0,  0, 16,  0, }, // Bolivia
    {  236,  437,  254192,  254209,  254234, 17, 25, 23, }, // Brasilia
    {  236,  454,       0,  254257,       0,  0, 15,  0, }, // Brunei
    {  236,  461,  254272,  254289,  254314, 17, 25, 23, }, // Cape_Verde
    {  236,  472,       0,  254337,       0,  0, 14,  0, }, // Casey
    {  236,  478,       0,  254351,       0,  0, 18,  0, }, // Chamorro
    {  236,  487,  254369,  254391,  254421, 22, 30, 28, }, // Chatham
    {  236,  495,  254449,  254463,  254485, 14, 22, 20, }, // Chile
    {  236,  501,  254505,  254522,  254547, 17, 25, 20, }, // China
    {  236,  507,  254567,  254586,  254613, 19, 27, 25, }, // Choibalsan
    {  236,  518,       0,  254638,       0,  0, 24,  0, }, // Christmas
    {  236,  528,       0,  254662,       0,  0, 20,  0, }, // Cocos
    {  236,  534,  254682,  254699,  254724, 17, 25, 23, }, // Colombia
    {  236,  543,  254747,  254766,  254793, 19, 27, 25, }, // Cook
    {  236,  548,  254818,  254831,  254852, 13, 21, 19, }, // Cuba
    {  236,  559,       0,  254871,       0,  0, 14,  0, }, // Davis
    {  236,  575,       0,  254885,       0,  0, 25,  0, }, // DumontDUrville
    {  236,  612,       0,  254910,       0,  0, 23,  0, }, // East_Timor
    {  236,  623,  254933,  254957,  254989, 24, 32, 30, }, // Easter
    {  236,  630,       0,  255019,       0,  0, 19,  0, }, // Ecuador
    {  236,  638,  255038,  255061,  255092, 23, 31, 29, }, // Europe_Central
    {  236,  653,  255121,  255144,  255175, 23, 31, 29, }, // Europe_Eastern
    {  236,  668,       0,  255204,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  236,  691,  255224,  255249,  255282, 25, 33, 31, }, // Europe_Western
    {  236,  706,  255313,  255337,  255369, 24, 32, 30, }, // Falkland
    {  236,  715,  255399,  255419,  255447, 20, 28, 26, }, // Fiji
    {  236,  720,       0,  255473,       0,  0, 28,  0, }, // French_Guiana
    {  236,  734,       0,  255501,       0,  0, 53,  0, }, // French_Southern
    {  236,  757,       0,  255554,       0,  0, 24,  0, }, // Galapagos
    {  236,  767,       0,  255578,       0,  0, 22,  0, }, // Gambier
    {  236,  775,  255600,  255619,  255646, 19, 27, 22, }, // Georgia
    {  236,  783,       0,  255668,       0,  0, 22,  0, }, // Gilbert_Islands
    {  236,  799,       0,  255690,       0,  0, 26,  0, }, // GMT
    {  236,  841,  255716,  255743,  255778, 27, 35, 33, }, // Greenland_Eastern
    {  236,  859,  255811,  255840,  255877, 29, 37, 35, }, // Greenland_Western
    {  236,  877,       0,  255912,       0,  0, 13,  0, }, // Guam
    {  236,  882,       0,  255925,       0,  0, 14,  0, }, // Gulf
    {  236,  887,       0,  255939,       0,  0, 15,  0, }, // Guyana
    {  236,  894,  255954,  255982,  256018, 28, 36, 34, }, // Hawaii_Aleutian
    {  236,  910,  256052,  256070,  256096, 18, 26, 24, }, // Hong_Kong
    {  236,  920,  256120,  256133,  256154, 13, 21, 19, }, // Hovd
    {  236,  925,       0,  256173,       0,  0, 15,  0, }, // India
    {  236,  931,       0,  256188,       0,  0, 23,  0, }, // Indian_Ocean
    {  236,  944,       0,  256211,       0,  0, 17,  0, }, // Indochina
    {  236,  954,       0,  256228,       0,  0, 26,  0, }, // Indonesia_Central
    {  236,  972,       0,  256254,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  236,  990,       0,  256279,       0,  0, 27,  0, }, // Indonesia_Western
    {  236, 1008,  256306,  256321,  256341, 15, 20, 18, }, // Iran
    {  236, 1019,  256359,  256375,  256399, 16, 24, 22, }, // Irkutsk
    {  236, 1027,  256421,  256435,  256457, 14, 22, 20, }, // Israel
    {  236, 1034,  256477,  256491,  256513, 14, 22, 20, }, // Japan
    {  236, 1040,  256533,  256566,  256607, 33, 41, 39, }, // Kamchatka
    {  236, 1058,       0,  256646,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  236, 1077,       0,  256674,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  236, 1106,  256704,  256721,  256746, 17, 25, 20, }, // Korea
    {  236, 1112,       0,  256766,       0,  0, 15,  0, }, // Kosrae
    {  236, 1119,  256781,  256801,  256829, 20, 28, 26, }, // Krasnoyarsk
    {  236, 1151,       0,  256855,       0,  0, 21,  0, }, // Kyrgystan
    {  236, 1161,       0,  256876,       0,  0, 14,  0, }, // Lanka
    {  236, 1175,       0,  256890,       0,  0, 26,  0, }, // Line_Islands
    {  236, 1188,  256916,  256934,  256960, 18, 26, 24, }, // Lord_Howe
    {  236, 1198,  256984,  256998,  257020, 14, 22, 20, }, // Macau
    {  236, 1204,  257040,  257056,  257080, 16, 24, 22, }, // Magadan
    {  236, 1219,       0,  257102,       0,  0, 20,  0, }, // Malaysia
    {  236, 1228,       0,  257122,       0,  0, 18,  0, }, // Maldives
    {  236, 1237,       0,  257140,       0,  0, 24,  0, }, // Marquesas
    {  236, 1247,       0,  257164,       0,  0, 23,  0, }, // Marshall_Islands
    {  236, 1264,  257187,  257203,  257227, 16, 24, 22, }, // Mauritius
    {  236, 1274,       0,  257249,       0,  0, 15,  0, }, // Mawson
    {  236, 1281,  257264,  257291,  257326, 27, 35, 33, }, // Mexico_Pacific
    {  236, 1296,  257359,  257378,  257405, 19, 27, 25, }, // Mongolia
    {  236, 1305,  257430,  257445,  257468, 15, 23, 21, }, // Moscow
    {  236, 1312,       0,  257489,       0,  0, 16,  0, }, // Myanmar
    {  236, 1320,       0,  257505,       0,  0, 14,  0, }, // Nauru
    {  236, 1326,       0,  257519,       0,  0, 14,  0, }, // Nepal
    {  236, 1332,  257533,  257563,  257601, 30, 38, 33, }, // New_Caledonia
    {  236, 1346,  257634,  257662,  257698, 28, 36, 34, }, // New_Zealand
    {  236, 1358,  257732,  257752,  257780, 20, 28, 26, }, // Newfoundland
    {  236, 1371,       0,  257806,       0,  0, 13,  0, }, // Niue
    {  236, 1376,  257819,  257841,  257871, 22, 30, 28, }, // Norfolk
    {  236, 1384,  257899,  257927,  257963, 28, 36, 34, }, // Noronha
    {  236, 1392,       0,  257997,       0,  0, 32,  0, }, // North_Mariana
    {  236, 1406,  258029,  258050,  258079, 21, 29, 27, }, // Novosibirsk
    {  236, 1418,  258106,  258119,  258140, 13, 21, 19, }, // Omsk
    {  236, 1428,  258159,  258176,  258201, 17, 25, 23, }, // Pakistan
    {  236, 1437,       0,  258224,       0,  0, 16,  0, }, // Palau
    {  236, 1443,       0,  258240,       0,  0, 37,  0, }, // Papua_New_Guinea
    {  236, 1460,  258277,  258294,  258319, 17, 25, 23, }, // Paraguay
    {  236, 1469,  258342,  258356,  258378, 14, 22, 20, }, // Peru
    {  236, 1474,  258398,  258419,  258448, 21, 29, 27, }, // Philippines
    {  236, 1486,       0,  258475,       0,  0, 22,  0, }, // Phoenix_Islands
    {  236, 1502,  258497,  258530,  258571, 33, 41, 39, }, // Pierre_Miquelon
    {  236, 1518,       0,  258610,       0,  0, 23,  0, }, // Pitcairn
    {  236, 1527,       0,  258633,       0,  0, 25,  0, }, // Ponape
    {  236, 1534,       0,  258658,       0,  0, 18,  0, }, // Pyongyang
    {  236, 1544,  258676,  258694,  258720, 18, 26, 24, }, // Qyzylorda
    {  236, 1554,       0,  258744,       0,  0, 19,  0, }, // Reunion
    {  236, 1562,       0,  258763,       0,  0, 16,  0, }, // Rothera
    {  236, 1570,  258779,  258797,  258823, 18, 26, 24, }, // Sakhalin
    {  236, 1579,  258847,  258862,  258885, 15, 23, 21, }, // Samara
    {  236, 1596,  258906,  258921,  258944, 15, 23, 21, }, // Samoa
    {  236, 1602,       0,  258965,       0,  0, 20,  0, }, // Seychelles
    {  236, 1624,       0,  258985,       0,  0, 18,  0, }, // Singapore
    {  236, 1634,       0,  259003,       0,  0, 22,  0, }, // Solomon
    {  236, 1642,       0,  259025,       0,  0, 23,  0, }, // South_Georgia
    {  236, 1656,       0,  259048,       0,  0, 17,  0, }, // Suriname
    {  236, 1676,       0,  259065,       0,  0, 14,  0, }, // Syowa
    {  236, 1682,       0,  259079,       0,  0, 15,  0, }, // Tahiti
    {  236, 1689,  259094,  259109,  259132, 15, 23, 21, }, // Taipei
    {  236, 1696,       0,  259153,       0,  0, 20,  0, }, // Tajikistan
    {  236, 1724,       0,  259173,       0,  0, 16,  0, }, // Tokelau
    {  236, 1732,  259189,  259204,  259227, 15, 23, 20, }, // Tonga
    {  236, 1738,       0,  259247,       0,  0, 14,  0, }, // Truk
    {  236, 1750,  259261,  259282,  259311, 21, 29, 27, }, // Turkmenistan
    {  236, 1763,       0,  259338,       0,  0, 16,  0, }, // Tuvalu
    {  236, 1777,  259354,  259372,  259398, 18, 26, 24, }, // Uruguay
    {  236, 1792,  259422,  259444,  259474, 22, 30, 28, }, // Uzbekistan
    {  236, 1803,  259502,  259518,  259542, 16, 24, 22, }, // Vanuatu
    {  236, 1811,       0,  259564,       0,  0, 18,  0, }, // Venezuela
    {  236, 1821,  259582,  259602,  259630, 20, 28, 26, }, // Vladivostok
    {  236, 1833,  259656,  259674,  259700, 18, 26, 24, }, // Volgograd
    {  236, 1843,       0,  259724,       0,  0, 15,  0, }, // Vostok
    {  236, 1850,       0,  259739,       0,  0, 19,  0, }, // Wake
    {  236, 1855,       0,  259758,       0,  0, 25,  0, }, // Wallis
    {  236, 1862,  259783,  259800,  259825, 17, 25, 23, }, // Yakutsk
    {  236, 1870,  259848,  259869,  259898, 21, 29, 27, }, // Yekaterinburg
    {  236, 1892,       0,  259925,       0,  0, 22,  0, }, // Yukon
    {  243,    0,  252072,  252087,  259947, 15, 23, 23, }, // Acre French/Latin/Canada
    {  243,   17,       0,  259970,       0,  0, 24,  0, }, // Africa_Central
    {  243,   32,       0,  259994,       0,  0, 25,  0, }, // Africa_Eastern
    {  243,   65,       0,  260019,       0,  0, 30,  0, }, // Africa_Southern
    {  243,   81,  252252,  252278,  260049, 26, 34, 34, }, // Africa_Western
    {  243,  107,  252344,  252361,  260083, 17, 25, 25, }, // Alaska
    {  243,  128,  252409,  252425,  260108, 16, 24, 24, }, // Almaty
    {  243,  135,  252471,  252490,  260132, 19, 27, 27, }, // Amazon
    {  243,  142,  256228,  260159,  260182, 15, 23, 23, }, // America_Central
    {  243,  158,  252646,  252675,  260205, 14, 22, 22, }, // America_Eastern
    {  243,  174,  252747,  252766,  260227, 19, 27, 27, }, // America_Mountain
    {  243,  191,  252818,  252851,  260254, 18, 26, 26, }, // America_Pacific
    {  243,  207,  252931,  252945,  260280, 14, 22, 22, }, // Anadyr
    {  243,  214,  252987,  252999,  260302, 12, 20, 20, }, // Apia
    {  243,  219,  253037,  253051,  260322, 14, 22, 22, }, // Aqtau
    {  243,  225,  253093,  253107,  260344, 14, 22, 22, }, // Aqtobe
    {  243,  232,  253149,  253166,  260366, 17, 25, 25, }, // Arabian
    {  243,  240,  253214,  253234,  260391, 20, 25, 28, }, // Argentina
    {  243,  250,  253285,  253310,  260419, 25, 33, 33, }, // Argentina_Western
    {  243,  268,  253374,  253392,  260452, 18, 26, 23, }, // Armenia
    {  243,  286,  253439,  253460,  260475, 21, 29, 29, }, // Atlantic
    {  243,  295,  253516,  253546,  260504, 30, 38, 38, }, // Australia_Central
    {  243,  313,  253620,  253656,  260542, 36, 44, 44, }, // Australia_CentralWestern
    {  243,  338,  253742,  253771,  260586, 29, 37, 37, }, // Australia_Eastern
    {  243,  356,  253843,  253874,  260623, 31, 39, 39, }, // Australia_Western
    {  243,  374,  253950,  253972,  260662, 22, 30, 27, }, // Azerbaijan
    {  243,  385,  254027,  254043,  260689, 16, 24, 24, }, // Azores
    {  243,  397,  254089,  254108,  260713, 19, 27, 27, }, // Bangladesh
    {  243,  437,  254192,  254209,  260740, 17, 25, 25, }, // Brasilia
    {  243,  454,       0,  260765,       0,  0, 15,  0, }, // Brunei
    {  243,  461,  254272,  254289,  260780, 17, 25, 25, }, // Cape_Verde
    {  243,  495,  254449,  254463,  260805, 14, 22, 22, }, // Chile
    {  243,  501,  260827,  260841,  260863, 14, 22, 22, }, // China
    {  243,  507,  254567,  254586,  260885, 19, 27, 27, }, // Choibalsan
    {  243,  534,  254682,  254699,  260912, 17, 25, 25, }, // Colombia
    {  243,  543,  254747,  254766,  260937, 19, 27, 27, }, // Cook
    {  243,  548,  254818,  254831,  260964, 13, 21, 21, }, // Cuba
    {  243,  623,  254933,  254957,  260985, 24, 32, 32, }, // Easter
    {  243,  638,  261017,  261043,  261077, 26, 34, 34, }, // Europe_Central
    {  243,  653,  261111,  261137,  261171, 26, 34, 34, }, // Europe_Eastern
    {  243,  691,  261205,  261233,  261269, 28, 36, 36, }, // Europe_Western
    {  243,  706,  255313,  255337,  261305, 24, 32, 32, }, // Falkland
    {  243,  715,  255399,  255419,  261337, 20, 28, 28, }, // Fiji
    {  243,  720,       0,  261365,       0,  0, 25,  0, }, // French_Guiana
    {  243,  775,  255600,  255619,  261390, 19, 27, 24, }, // Georgia
    {  243,  841,  255716,  255743,  261414, 27, 35, 35, }, // Greenland_Eastern
    {  243,  859,  255811,  255840,  261449, 29, 37, 37, }, // Greenland_Western
    {  243,  894,  261486,  261512,  261546, 26, 34, 34, }, // Hawaii_Aleutian
    {  243,  910,  256052,  256070,  261580, 18, 26, 26, }, // Hong_Kong
    {  243,  920,  256120,  256133,  261606, 13, 21, 21, }, // Hovd
    {  243, 1008,  256306,  256321,  261627, 15, 20, 20, }, // Iran
    {  243, 1019,  256359,  256375,  261647, 16, 24, 24, }, // Irkutsk
    {  243, 1027,  256421,  256435,  261671, 14, 22, 22, }, // Israel
    {  243, 1034,  256477,  256491,  261693, 14, 22, 22, }, // Japan
    {  243, 1040,  256533,  256566,  261715, 33, 41, 41, }, // Kamchatka
    {  243, 1106,  256704,  256721,  261756, 17, 25, 22, }, // Korea
    {  243, 1119,  256781,  256801,  261778, 20, 28, 28, }, // Krasnoyarsk
    {  243, 1188,  256916,  256934,  261806, 18, 26, 26, }, // Lord_Howe
    {  243, 1198,  256984,  256998,  261832, 14, 22, 22, }, // Macau
    {  243, 1204,  257040,  257056,  261854, 16, 24, 24, }, // Magadan
    {  243, 1264,  257187,  257203,  261878, 16, 24, 24, }, // Mauritius
    {  243, 1281,  257264,  257291,  261902, 27, 35, 35, }, // Mexico_Pacific
    {  243, 1296,  257359,  257378,  261937, 19, 27, 27, }, // Mongolia
    {  243, 1305,  257430,  257445,  261964, 15, 23, 23, }, // Moscow
    {  243, 1332,  257533,  257563,  261987, 30, 38, 35, }, // New_Caledonia
    {  243, 1346,  257634,  257662,  262022, 28, 36, 36, }, // New_Zealand
    {  243, 1358,  257732,  257752,  262058, 20, 28, 28, }, // Newfoundland
    {  243, 1371,       0,  262086,       0,  0, 14,  0, }, // Niue
    {  243, 1376,  257819,  257841,  262100, 22, 30, 30, }, // Norfolk
    {  243, 1384,  257899,  257927,  262130, 28, 36, 36, }, // Noronha
    {  243, 1406,  258029,  258050,  262166, 21, 29, 29, }, // Novosibirsk
    {  243, 1418,  262195,  262207,  262227, 12, 20, 20, }, // Omsk
    {  243, 1428,  258159,  258176,  262247, 17, 25, 25, }, // Pakistan
    {  243, 1460,  258277,  258294,  262272, 17, 25, 25, }, // Paraguay
    {  243, 1469,  258342,  258356,  262297, 14, 22, 22, }, // Peru
    {  243, 1474,  258398,  258419,  262319, 21, 29, 29, }, // Philippines
    {  243, 1502,  258497,  258530,  262348, 33, 41, 41, }, // Pierre_Miquelon
    {  243, 1554,       0,  262389,       0,  0, 19,  0, }, // Reunion
    {  243, 1570,  258779,  258797,  262408, 18, 26, 26, }, // Sakhalin
    {  243, 1596,  258906,  258921,  262434, 15, 23, 23, }, // Samoa
    {  243, 1689,  259094,  259109,  262457, 15, 23, 23, }, // Taipei
    {  243, 1732,  259189,  259204,  262480, 15, 23, 22, }, // Tonga
    {  243, 1750,  259261,  259282,  262502, 21, 29, 29, }, // Turkmenistan
    {  243, 1777,  259354,  259372,  262531, 18, 26, 26, }, // Uruguay
    {  243, 1792,  259422,  259444,  262557, 22, 30, 30, }, // Uzbekistan
    {  243, 1803,  259502,  259518,  262587, 16, 24, 24, }, // Vanuatu
    {  243, 1821,  259582,  259602,  262611, 20, 28, 28, }, // Vladivostok
    {  243, 1833,  259656,  259674,  262639, 18, 26, 26, }, // Volgograd
    {  243, 1862,  259783,  259800,  262665, 17, 25, 25, }, // Yakutsk
    {  243, 1870,  259848,  259869,  262690, 21, 29, 29, }, // Yekaterinburg
    {  251,  720,       0,  255473,       0,  0, 28,  0, }, // French_Guiana French/Latin/French Guiana
    {  282,  638,  262719,  262740,  262770, 21, 30, 28, }, // Europe_Central Friulian/Latin/Italy
    {  282,  653,  262798,  262820,  262851, 22, 31, 29, }, // Europe_Eastern
    {  282,  691,  262880,  262903,  262935, 23, 32, 30, }, // Europe_Western
    {  282, 1305,  262965,  262978,  263000, 13, 22, 20, }, // Moscow
    {  289,    0,  263020,  263045,  263089, 25, 44, 36, }, // Acre Fulah/Adlam/Guinea
    {  289,    5,       0,  263125,       0,  0, 37,  0, }, // Afghanistan
    {  289,   17,       0,  263162,       0,  0, 46,  0, }, // Africa_Central
    {  289,   32,       0,  263208,       0,  0, 48,  0, }, // Africa_Eastern
    {  289,   65,       0,  263256,       0,  0, 67,  0, }, // Africa_Southern
    {  289,   81,  263323,  263373,  263442, 50, 69, 61, }, // Africa_Western
    {  289,  107,  263503,  263532,  263589, 29, 57, 49, }, // Alaska
    {  289,  128,  263638,  263665,  263711, 27, 46, 38, }, // Almaty
    {  289,  135,  263749,  263778,  263826, 29, 48, 40, }, // Amazon
    {  289,  142,  263866,  263927,  264007, 61, 80, 72, }, // America_Central
    {  289,  158,  264079,  264154,  264248, 75, 94, 86, }, // America_Eastern
    {  289,  174,  264334,  264407,  264499, 73, 92, 61, }, // America_Mountain
    {  289,  191,  264560,  264621,  264701, 61, 80, 72, }, // America_Pacific
    {  289,  207,  264773,  264802,  264850, 29, 48, 40, }, // Anadyr
    {  289,  214,  264890,  264917,  264963, 27, 46, 38, }, // Apia
    {  289,  219,  265001,  265026,  265070, 25, 44, 36, }, // Aqtau
    {  289,  225,  265106,  265135,  265183, 29, 48, 40, }, // Aqtobe
    {  289,  232,  265223,  265254,  265304, 31, 50, 42, }, // Arabian
    {  289,  240,  265346,  265381,  265435, 35, 54, 46, }, // Argentina
    {  289,  250,  265481,  265539,  265616, 58, 77, 69, }, // Argentina_Western
    {  289,  268,  265685,  265718,  265770, 33, 52, 44, }, // Armenia
    {  289,  286,  265814,  265837,  265879, 23, 42, 34, }, // Atlantic
    {  289,  295,  265913,  265965,  266036, 52, 71, 63, }, // Australia_Central
    {  289,  313,  266099,  266172,  266264, 73, 92, 84, }, // Australia_CentralWestern
    {  289,  338,  266348,  266408,  266487, 60, 79, 71, }, // Australia_Eastern
    {  289,  356,  266558,  266620,  266701, 62, 81, 73, }, // Australia_Western
    {  289,  374,  266774,  266809,  266863, 35, 54, 46, }, // Azerbaijan
    {  289,  385,  266909,  266936,  266982, 27, 46, 38, }, // Azores
    {  289,  397,  267020,  267057,  267113, 37, 56, 48, }, // Bangladesh
    {  289,  415,       0,  267161,       0,  0, 27,  0, }, // Bhutan
    {  289,  422,       0,  267188,       0,  0, 33,  0, }, // Bolivia
    {  289,  437,  267221,  267256,  267310, 35, 54, 46, }, // Brasilia
    {  289,  454,       0,  267356,       0,  0, 27,  0, }, // Brunei
    {  289,  461,  267383,  267419,  267474, 36, 55, 47, }, // Cape_Verde
    {  289,  472,       0,  267521,       0,  0, 27,  0, }, // Casey
    {  289,  478,       0,  267548,       0,  0, 48,  0, }, // Chamorro
    {  289,  487,  267596,  267623,  267669, 27, 46, 38, }, // Chatham
    {  289,  495,  267707,  267732,  267776, 25, 44, 36, }, // Chile
    {  289,  501,  267812,  267837,  267881, 25, 44, 36, }, // China
    {  289,  507,  267917,  267950,  268006, 33, 56, 44, }, // Choibalsan
    {  289,  518,       0,  268050,       0,  0, 48,  0, }, // Christmas
    {  289,  528,       0,  268098,       0,  0, 27,  0, }, // Cocos
    {  289,  534,  268125,  268160,  268214, 35, 54, 46, }, // Colombia
    {  289,  543,  268260,  268298,  268355, 38, 57, 60, }, // Cook
    {  289,  548,  268415,  268442,  268488, 27, 46, 38, }, // Cuba
    {  289,  559,       0,  268526,       0,  0, 27,  0, }, // Davis
    {  289,  575,       0,  268553,       0,  0, 42,  0, }, // DumontDUrville
    {  289,  612,       0,  268595,       0,  0, 48,  0, }, // East_Timor
    {  289,  623,  268643,  268681,  268738, 38, 57, 49, }, // Easter
    {  289,  630,       0,  268787,       0,  0, 33,  0, }, // Ecuador
    {  289,  638,  268820,  268864,  268927, 44, 63, 55, }, // Europe_Central
    {  289,  653,  268982,  269030,  269097, 48, 67, 59, }, // Europe_Eastern
    {  289,  668,       0,  269156,       0,  0, 54,  0, }, // Europe_Further_Eastern
    {  289,  691,  269210,  269260,  269329, 50, 69, 61, }, // Europe_Western
    {  289,  706,  269390,  269434,  269497, 44, 63, 55, }, // Falkland
    {  289,  715,  269552,  269577,  269621, 25, 44, 36, }, // Fiji
    {  289,  720,       0,  269657,       0,  0, 42,  0, }, // French_Guiana
    {  289,  734,       0,  269699,       0,  0, 59,  0, }, // French_Southern
    {  289,  757,       0,  269758,       0,  0, 35,  0, }, // Galapagos
    {  289,  767,       0,  269793,       0,  0, 31,  0, }, // Gambier
    {  289,  775,  269824,  269853,  269901, 29, 48, 40, }, // Georgia
    {  289,  783,       0,  269941,       0,  0, 46,  0, }, // Gilbert_Islands
    {  289,  799,       0,  269987,       0,  0, 54,  0, }, // GMT
    {  289,  841,  270041,  270089,  270156, 48, 67, 59, }, // Greenland_Eastern
    {  289,  859,  270215,  270265,  270334, 50, 69, 61, }, // Greenland_Western
    {  289,  877,       0,  270395,       0,  0, 46,  0, }, // Guam
    {  289,  882,       0,  270441,       0,  0, 25,  0, }, // Gulf
    {  289,  887,       0,  269657,       0,  0, 29,  0, }, // Guyana
    {  289,  894,  270466,  270514,  270581, 48, 67, 59, }, // Hawaii_Aleutian
    {  289,  910,  270640,  270672,  270723, 32, 51, 43, }, // Hong_Kong
    {  289,  920,  270766,  270791,  270835, 25, 44, 36, }, // Hovd
    {  289,  925,       0,  270871,       0,  0, 46,  0, }, // India
    {  289,  931,       0,  270917,       0,  0, 42,  0, }, // Indian_Ocean
    {  289,  944,       0,  270959,       0,  0, 31,  0, }, // Indochina
    {  289,  954,       0,  270990,       0,  0, 54,  0, }, // Indonesia_Central
    {  289,  972,       0,  271044,       0,  0, 58,  0, }, // Indonesia_Eastern
    {  289,  990,       0,  271102,       0,  0, 60,  0, }, // Indonesia_Western
    {  289, 1008,  271162,  271187,  271231, 25, 44, 36, }, // Iran
    {  289, 1019,  271267,  271300,  271352, 33, 52, 44, }, // Irkutsk
    {  289, 1027,  271396,  271429,  271481, 33, 52, 44, }, // Israel
    {  289, 1034,  271525,  271552,  271598, 27, 46, 38, }, // Japan
    {  289, 1058,       0,  271636,       0,  0, 37,  0, }, // Kazakhstan_Eastern
    {  289, 1077,       0,  271673,       0,  0, 58,  0, }, // Kazakhstan_Western
    {  289, 1106,  271731,  271762,  271812, 31, 50, 42, }, // Korea
    {  289, 1112,       0,  271854,       0,  0, 27,  0, }, // Kosrae
    {  289, 1119,  271881,  271922,  271980, 41, 58, 52, }, // Krasnoyarsk
    {  289, 1151,       0,  272032,       0,  0, 35,  0, }, // Kyrgystan
    {  289, 1175,       0,  272067,       0,  0, 40,  0, }, // Line_Islands
    {  289, 1188,  272107,  272141,  272194, 34, 53, 45, }, // Lord_Howe
    {  289, 1198,  272239,  272268,  272316, 29, 48, 40, }, // Macau
    {  289, 1204,  272356,  272385,  272433, 29, 48, 40, }, // Magadan
    {  289, 1219,       0,  272473,       0,  0, 33,  0, }, // Malaysia
    {  289, 1228,       0,  272506,       0,  0, 33,  0, }, // Maldives
    {  289, 1237,       0,  272539,       0,  0, 31,  0, }, // Marquesas
    {  289, 1247,       0,  272570,       0,  0, 42,  0, }, // Marshall_Islands
    {  289, 1264,  272612,  272641,  272689, 29, 48, 40, }, // Mauritius
    {  289, 1274,       0,  272729,       0,  0, 27,  0, }, // Mawson
    {  289, 1281,  272756,  272798,  272859, 42, 61, 53, }, // Mexico_Pacific
    {  289, 1296,  272912,  272945,  272997, 33, 52, 44, }, // Mongolia
    {  289, 1305,  273041,  273066,  273110, 25, 44, 36, }, // Moscow
    {  289, 1312,       0,  273146,       0,  0, 31,  0, }, // Myanmar
    {  289, 1320,       0,  273177,       0,  0, 25,  0, }, // Nauru
    {  289, 1326,       0,  273202,       0,  0, 25,  0, }, // Nepal
    {  289, 1332,  273227,  273273,  273338, 46, 65, 57, }, // New_Caledonia
    {  289, 1346,  273395,  273431,  273486, 36, 55, 47, }, // New_Zealand
    {  289, 1358,  273533,  273573,  273632, 40, 59, 51, }, // Newfoundland
    {  289, 1371,       0,  273683,       0,  0, 25,  0, }, // Niue
    {  289, 1376,  273708,  273754,  273819, 46, 65, 57, }, // Norfolk
    {  289, 1384,  273876,  273931,  274005, 55, 74, 66, }, // Noronha
    {  289, 1406,  274071,  274108,  274164, 37, 56, 48, }, // Novosibirsk
    {  289, 1418,  274212,  274239,  274285, 27, 46, 38, }, // Omsk
    {  289, 1428,  274323,  274356,  274408, 33, 52, 44, }, // Pakistan
    {  289, 1437,       0,  274452,       0,  0, 29,  0, }, // Palau
    {  289, 1443,       0,  274481,       0,  0, 45,  0, }, // Papua_New_Guinea
    {  289, 1460,  274526,  274557,  274607, 31, 50, 42, }, // Paraguay
    {  289, 1469,  274649,  274674,  274718, 25, 44, 59, }, // Peru
    {  289, 1474,  274777,  274808,  274858, 31, 50, 42, }, // Philippines
    {  289, 1486,       0,  274900,       0,  0, 44,  0, }, // Phoenix_Islands
    {  289, 1502,  274944,  274994,  275063, 50, 69, 61, }, // Pierre_Miquelon
    {  289, 1518,       0,  275124,       0,  0, 33,  0, }, // Pitcairn
    {  289, 1527,       0,  275157,       0,  0, 29,  0, }, // Ponape
    {  289, 1534,       0,  275186,       0,  0, 31,  0, }, // Pyongyang
    {  289, 1544,  275217,  275250,  275302, 33, 52, 44, }, // Qyzylorda
    {  289, 1554,       0,  275346,       0,  0, 29,  0, }, // Reunion
    {  289, 1562,       0,  275375,       0,  0, 27,  0, }, // Rothera
    {  289, 1570,  275402,  275433,  275483, 31, 50, 42, }, // Sakhalin
    {  289, 1596,  275525,  275552,  275598, 27, 46, 38, }, // Samoa
    {  289, 1602,       0,  275636,       0,  0, 29,  0, }, // Seychelles
    {  289, 1624,       0,  275665,       0,  0, 52,  0, }, // Singapore
    {  289, 1634,       0,  275717,       0,  0, 48,  0, }, // Solomon
    {  289, 1642,       0,  275765,       0,  0, 40,  0, }, // South_Georgia
    {  289, 1656,       0,  275805,       0,  0, 29,  0, }, // Suriname
    {  289, 1676,       0,  275834,       0,  0, 29,  0, }, // Syowa
    {  289, 1682,       0,  275863,       0,  0, 29,  0, }, // Tahiti
    {  289, 1689,  275892,  275919,  275965, 27, 46, 38, }, // Taipei
    {  289, 1696,       0,  276003,       0,  0, 37,  0, }, // Tajikistan
    {  289, 1724,       0,  276040,       0,  0, 35,  0, }, // Tokelau
    {  289, 1732,  276075,  276100,  276144, 25, 44, 36, }, // Tonga
    {  289, 1738,       0,  276180,       0,  0, 25,  0, }, // Truk
    {  289, 1750,  276205,  276248,  276310, 43, 62, 54, }, // Turkmenistan
    {  289, 1763,       0,  276364,       0,  0, 29,  0, }, // Tuvalu
    {  289, 1777,  276393,  276422,  276470, 29, 48, 40, }, // Uruguay
    {  289, 1792,  276510,  276547,  276603, 37, 56, 48, }, // Uzbekistan
    {  289, 1803,  276651,  276684,  276736, 33, 52, 44, }, // Vanuatu
    {  289, 1811,       0,  276780,       0,  0, 35,  0, }, // Venezuela
    {  289, 1821,  276815,  276850,  276904, 35, 54, 46, }, // Vladivostok
    {  289, 1833,  276950,  276987,  277043, 37, 56, 48, }, // Volgograd
    {  289, 1843,       0,  277091,       0,  0, 27,  0, }, // Vostok
    {  289, 1850,       0,  277118,       0,  0, 40,  0, }, // Wake
    {  289, 1855,       0,  277158,       0,  0, 44,  0, }, // Wallis
    {  289, 1862,  277202,  277235,  277287, 33, 52, 44, }, // Yakutsk
    {  289, 1870,  277331,  277374,  277436, 43, 62, 54, }, // Yekaterinburg
    {  307,    0,  277490,  277497,  277508,  7, 11, 19, }, // Acre Gaelic/Latin/United Kingdom
    {  307,    5,       0,  277527,       0,  0, 15,  0, }, // Afghanistan
    {  307,   17,       0,  277542,       0,  0, 17,  0, }, // Africa_Central
    {  307,   32,       0,  277559,       0,  0, 16,  0, }, // Africa_Eastern
    {  307,   65,       0,  277575,       0,  0, 16,  0, }, // Africa_Southern
    {  307,   81,  277591,  277607,  277627, 16, 20, 28, }, // Africa_Western
    {  307,  107,  277655,  277664,  277677,  9, 13, 21, }, // Alaska
    {  307,  128,  277698,  277707,  277720,  9, 13, 21, }, // Almaty
    {  307,  135,  277741,  277751,  277765, 10, 14, 22, }, // Amazon
    {  307,  142,  277787,  277816,  277849, 29, 33, 41, }, // America_Central
    {  307,  158,  277890,  277918,  277950, 28, 32, 40, }, // America_Eastern
    {  307,  174,  277990,  278018,  278050, 28, 32, 40, }, // America_Mountain
    {  307,  191,  278090,  278108,  278130, 18, 22, 30, }, // America_Pacific
    {  307,  207,  278160,  278169,  278182,  9, 13, 21, }, // Anadyr
    {  307,  214,  278203,  278210,  278221,  7, 11, 19, }, // Apia
    {  307,  219,  278240,  278248,  278260,  8, 12, 20, }, // Aqtau
    {  307,  225,  278280,  278289,  278302,  9, 13, 21, }, // Aqtobe
    {  307,  232,  278323,  278333,  278347, 10, 14, 22, }, // Arabian
    {  307,  240,  278369,  278387,  278409, 18, 22, 30, }, // Argentina
    {  307,  250,  278439,  278466,  278497, 27, 31, 39, }, // Argentina_Western
    {  307,  268,  278536,  278548,  278564, 12, 16, 24, }, // Armenia
    {  307,  286,  278588,  278605,  278626, 17, 21, 29, }, // Atlantic
    {  307,  295,  278655,  278675,  278699, 20, 24, 32, }, // Australia_Central
    {  307,  313,  278731,  278758,  278789, 27, 31, 39, }, // Australia_CentralWestern
    {  307,  338,  278828,  278847,  278870, 19, 23, 31, }, // Australia_Eastern
    {  307,  356,  278901,  278920,  278943, 19, 23, 31, }, // Australia_Western
    {  307,  374,  278974,  278989,  279008, 15, 19, 27, }, // Azerbaijan
    {  307,  385,  279035,  279058,  279085, 23, 27, 35, }, // Azores
    {  307,  397,  279120,  279133,  279150, 13, 17, 25, }, // Bangladesh
    {  307,  415,       0,  279175,       0,  0,  9,  0, }, // Bhutan
    {  307,  422,       0,  279184,       0,  0, 12,  0, }, // Bolivia
    {  307,  437,  279196,  279208,  279224, 12, 16, 24, }, // Brasilia
    {  307,  454,       0,  279248,       0,  0, 25,  0, }, // Brunei
    {  307,  461,  279273,  279289,  279309, 16, 20, 28, }, // Cape_Verde
    {  307,  472,       0,  279337,       0,  0,  9,  0, }, // Casey
    {  307,  478,       0,  279346,       0,  0, 11,  0, }, // Chamorro
    {  307,  487,  279357,  279367,  279381, 10, 14, 22, }, // Chatham
    {  307,  495,  279403,  279413,  279427, 10, 14, 22, }, // Chile
    {  307,  501,  279449,  279459,  279473, 10, 14, 22, }, // China
    {  307,  507,  279495,  279508,  279525, 13, 17, 25, }, // Choibalsan
    {  307,  518,       0,  279550,       0,  0, 20,  0, }, // Christmas
    {  307,  528,       0,  279570,       0,  0, 18,  0, }, // Cocos
    {  307,  534,  279588,  279600,  279616, 12, 16, 24, }, // Colombia
    {  307,  543,  279640,  279656,  279676, 16, 20, 34, }, // Cook
    {  307,  548,  279710,  279717,  279728,  7, 11, 19, }, // Cuba
    {  307,  559,       0,  279747,       0,  0,  9,  0, }, // Davis
    {  307,  575,       0,  279756,       0,  0, 19,  0, }, // DumontDUrville
    {  307,  612,       0,  279775,       0,  0, 17,  0, }, // East_Timor
    {  307,  623,  279792,  279811,  279834, 19, 23, 31, }, // Easter
    {  307,  630,       0,  279865,       0,  0, 12,  0, }, // Ecuador
    {  307,  638,  279877,  279902,  279931, 25, 29, 37, }, // Europe_Central
    {  307,  653,  279968,  279992,  280020, 24, 28, 36, }, // Europe_Eastern
    {  307,  668,       0,  280056,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  307,  691,  280088,  280112,  280140, 24, 28, 36, }, // Europe_Western
    {  307,  706,  280176,  280202,  280232, 26, 30, 38, }, // Falkland
    {  307,  715,  280270,  280277,  280288,  7, 11, 19, }, // Fiji
    {  307,  720,       0,  280307,       0,  0, 23,  0, }, // French_Guiana
    {  307,  734,       0,  280330,       0,  0, 40,  0, }, // French_Southern
    {  307,  757,       0,  280370,       0,  0, 13,  0, }, // Galapagos
    {  307,  767,       0,  280383,       0,  0, 11,  0, }, // Gambier
    {  307,  775,  280394,  280411,  280432, 17, 21, 29, }, // Georgia
    {  307,  783,       0,  280461,       0,  0, 23,  0, }, // Gilbert_Islands
    {  307,  799,       0,    2861,       0,  0, 19,  0, }, // GMT
    {  307,  841,  280484,  280507,  280534, 23, 27, 35, }, // Greenland_Eastern
    {  307,  859,  280569,  280592,  280619, 23, 27, 35, }, // Greenland_Western
    {  307,  877,       0,  280654,       0,  0,  7,  0, }, // Guam
    {  307,  882,       0,  280661,       0,  0, 13,  0, }, // Gulf
    {  307,  887,       0,  280307,       0,  0, 12,  0, }, // Guyana
    {  307,  894,  280674,  280709,  280748, 35, 39, 47, }, // Hawaii_Aleutian
    {  307,  910,  280795,  280807,  280823, 12, 16, 24, }, // Hong_Kong
    {  307,  920,  280847,  280854,  280865,  7, 11, 19, }, // Hovd
    {  307,  925,       0,  280884,       0,  0, 17,  0, }, // India
    {  307,  931,       0,  280901,       0,  0, 22,  0, }, // Indian_Ocean
    {  307,  944,       0,  280923,       0,  0, 17,  0, }, // Indochina
    {  307,  954,       0,  280940,       0,  0, 25,  0, }, // Indonesia_Central
    {  307,  972,       0,  280965,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  307,  990,       0,  280989,       0,  0, 24,  0, }, // Indonesia_Western
    {  307, 1008,  281013,  281022,  281035,  9, 13, 21, }, // Iran
    {  307, 1019,  281056,  281066,  281080, 10, 14, 22, }, // Irkutsk
    {  307, 1027,  281102,  281112,  281126, 10, 14, 22, }, // Israel
    {  307, 1034,  281148,  281162,  281180, 14, 18, 26, }, // Japan
    {  307, 1040,  281206,  281233,  281264, 27, 31, 39, }, // Kamchatka
    {  307, 1058,       0,  281303,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  307, 1077,       0,  281324,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  307, 1106,  281345,  281355,  281369, 10, 14, 22, }, // Korea
    {  307, 1112,       0,  281391,       0,  0,  9,  0, }, // Kosrae
    {  307, 1119,  281400,  281414,  281432, 14, 18, 26, }, // Krasnoyarsk
    {  307, 1151,       0,  281458,       0,  0, 14,  0, }, // Kyrgystan
    {  307, 1161,       0,  281472,       0,  0,  8,  0, }, // Lanka
    {  307, 1175,       0,  281480,       0,  0, 19,  0, }, // Line_Islands
    {  307, 1188,  281499,  281511,  281527, 12, 16, 24, }, // Lord_Howe
    {  307, 1198,  281551,  281561,  281575, 10, 14, 22, }, // Macau
    {  307, 1204,  281597,  281607,  281621, 10, 14, 22, }, // Magadan
    {  307, 1219,       0,  281643,       0,  0, 14,  0, }, // Malaysia
    {  307, 1228,       0,  281657,       0,  0, 26,  0, }, // Maldives
    {  307, 1237,       0,  281683,       0,  0, 23,  0, }, // Marquesas
    {  307, 1247,       0,  281706,       0,  0, 21,  0, }, // Marshall_Islands
    {  307, 1264,  281727,  281753,  281783, 26, 30, 38, }, // Mauritius
    {  307, 1274,       0,  281821,       0,  0, 10,  0, }, // Mawson
    {  307, 1281,  281831,  281859,  281891, 28, 32, 40, }, // Mexico_Pacific
    {  307, 1296,  281931,  281944,  281961, 13, 17, 25, }, // Mongolia
    {  307, 1305,  281986,  281995,  282008,  9, 13, 21, }, // Moscow
    {  307, 1312,       0,  282029,       0,  0, 10,  0, }, // Myanmar
    {  307, 1320,       0,  282039,       0,  0,  9,  0, }, // Nauru
    {  307, 1326,       0,  282048,       0,  0,  9,  0, }, // Nepal
    {  307, 1332,  282057,  282077,  282101, 20, 24, 32, }, // New_Caledonia
    {  307, 1346,  282133,  282152,  282175, 19, 23, 31, }, // New_Zealand
    {  307, 1358,  282206,  282223,  282244, 17, 21, 29, }, // Newfoundland
    {  307, 1371,       0,  282273,       0,  0,  7,  0, }, // Niue
    {  307, 1376,  282280,  282297,  282318, 17, 21, 29, }, // Norfolk
    {  307, 1384,  282347,  282369,  282395, 22, 26, 34, }, // Noronha
    {  307, 1392,       0,  282429,       0,  0, 34,  0, }, // North_Mariana
    {  307, 1406,  282463,  282477,  282495, 14, 18, 26, }, // Novosibirsk
    {  307, 1418,  282521,  282528,  282539,  7, 11, 19, }, // Omsk
    {  307, 1428,  282558,  282570,  282586, 12, 16, 24, }, // Pakistan
    {  307, 1437,       0,  282610,       0,  0,  9,  0, }, // Palau
    {  307, 1443,       0,  282619,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  307, 1460,  282645,  282658,  282675, 13, 17, 25, }, // Paraguay
    {  307, 1469,  282700,  282708,  282720,  8, 12, 20, }, // Peru
    {  307, 1474,  282740,  282765,  282794, 25, 29, 37, }, // Philippines
    {  307, 1486,       0,  282831,       0,  0, 19,  0, }, // Phoenix_Islands
    {  307, 1502,  282850,  282879,  282912, 29, 33, 41, }, // Pierre_Miquelon
    {  307, 1518,       0,  282953,       0,  0, 17,  0, }, // Pitcairn
    {  307, 1527,       0,  282970,       0,  0, 10,  0, }, // Ponape
    {  307, 1534,       0,  282980,       0,  0, 13,  0, }, // Pyongyang
    {  307, 1544,  282993,  283005,  283021, 12, 16, 24, }, // Qyzylorda
    {  307, 1554,       0,  283045,       0,  0, 10,  0, }, // Reunion
    {  307, 1562,       0,  283055,       0,  0, 10,  0, }, // Rothera
    {  307, 1570,  283065,  283076,  283091, 11, 15, 23, }, // Sakhalin
    {  307, 1579,  283114,  283123,  283136,  9, 13, 21, }, // Samara
    {  307, 1596,  283157,  283167,  283181, 10, 14, 22, }, // Samoa
    {  307, 1602,       0,  283203,       0,  0, 25,  0, }, // Seychelles
    {  307, 1624,       0,  283228,       0,  0, 12,  0, }, // Singapore
    {  307, 1634,       0,  283240,       0,  0, 20,  0, }, // Solomon
    {  307, 1642,       0,  283260,       0,  0, 17,  0, }, // South_Georgia
    {  307, 1656,       0,  283277,       0,  0, 11,  0, }, // Suriname
    {  307, 1676,       0,  283288,       0,  0,  9,  0, }, // Syowa
    {  307, 1682,       0,  283297,       0,  0,  9,  0, }, // Tahiti
    {  307, 1689,  283306,  283315,  283328,  9, 13, 21, }, // Taipei
    {  307, 1696,       0,  283349,       0,  0, 16,  0, }, // Tajikistan
    {  307, 1724,       0,  283365,       0,  0, 10,  0, }, // Tokelau
    {  307, 1732,  283375,  283383,  283395,  8, 12, 20, }, // Tonga
    {  307, 1738,       0,  283415,       0,  0,  8,  0, }, // Truk
    {  307, 1750,  283423,  283439,  283459, 16, 20, 28, }, // Turkmenistan
    {  307, 1763,       0,  283487,       0,  0, 10,  0, }, // Tuvalu
    {  307, 1777,  283497,  283509,  283525, 12, 16, 24, }, // Uruguay
    {  307, 1792,  283549,  283562,  283579, 13, 17, 25, }, // Uzbekistan
    {  307, 1803,  283604,  283614,  283628, 10, 14, 22, }, // Vanuatu
    {  307, 1811,       0,  283650,       0,  0, 17,  0, }, // Venezuela
    {  307, 1821,  283667,  283681,  283699, 14, 18, 26, }, // Vladivostok
    {  307, 1833,  283725,  283737,  283753, 12, 16, 24, }, // Volgograd
    {  307, 1843,       0,  283777,       0,  0,  9,  0, }, // Vostok
    {  307, 1850,       0,  283786,       0,  0, 14,  0, }, // Wake
    {  307, 1855,       0,  283800,       0,  0, 21,  0, }, // Wallis
    {  307, 1862,  283821,  283831,  283845, 10, 14, 22, }, // Yakutsk
    {  307, 1870,  283867,  283883,  283903, 16, 20, 28, }, // Yekaterinburg
    {  307, 1892,       0,  283931,       0,  0,  8,  0, }, // Yukon
    {  308,   17,       0,  283939,       0,  0, 16,  0, }, // Africa_Central Ga/Latin/Ghana
    {  308,   32,       0,  283955,       0,  0, 17,  0, }, // Africa_Eastern
    {  308,   65,       0,  283972,       0,  0, 15,  0, }, // Africa_Southern
    {  308,   81,  283987,  284004,  284034, 17, 30, 32, }, // Africa_Western
    {  308,  107,  284066,  284075,  284097,  9, 22, 24, }, // Alaska
    {  308,  142,  284121,  284142,  284176, 21, 34, 36, }, // America_Central
    {  308,  158,  284212,  284234,  284269, 22, 35, 37, }, // America_Eastern
    {  308,  174,  284306,  284328,  284363, 22, 35, 37, }, // America_Mountain
    {  308,  191,  284400,  284410,  284433, 10, 23, 25, }, // America_Pacific
    {  308,  286,  284458,  284469,  284493, 11, 24, 26, }, // Atlantic
    {  308,  385,  284519,  284528,  284550,  9, 22, 24, }, // Azores
    {  308,  461,  284574,  284587,  284613, 13, 26, 28, }, // Cape_Verde
    {  308,  548,  284641,  284648,  284668,  7, 20, 22, }, // Cuba
    {  308,  734,       0,  284690,       0,  0, 43,  0, }, // French_Southern
    {  308,  841,  284733,  284753,  284786, 20, 33, 35, }, // Greenland_Eastern
    {  308,  859,  284821,  284841,  284874, 20, 33, 35, }, // Greenland_Western
    {  308,  894,  284909,  284926,  284956, 17, 30, 32, }, // Hawaii_Aleutian
    {  308,  931,       0,  284988,       0,  0, 17,  0, }, // Indian_Ocean
    {  308, 1264,  285005,  285017,  285042, 12, 25, 27, }, // Mauritius
    {  308, 1281,  285069,  285086,  285116, 17, 30, 32, }, // Mexico_Pacific
    {  308, 1358,  285148,  285163,  285191, 15, 28, 30, }, // Newfoundland
    {  308, 1502,  285221,  285245,  285282, 24, 37, 39, }, // Pierre_Miquelon
    {  308, 1554,       0,  285321,       0,  0, 10,  0, }, // Reunion
    {  308, 1602,       0,  285331,       0,  0, 13,  0, }, // Seychelles
    {  309,    5,       0,  285344,       0,  0, 18,  0, }, // Afghanistan Galician/Latin/Spain
    {  309,   17,       0,  285362,       0,  0, 22,  0, }, // Africa_Central
    {  309,   32,       0,  285384,       0,  0, 23,  0, }, // Africa_Eastern
    {  309,   65,       0,  285407,       0,  0, 25,  0, }, // Africa_Southern
    {  309,   81,  285432,  285457,  285491, 25, 34, 34, }, // Africa_Western
    {  309,  107,  285525,  285539,  285562, 14, 23, 23, }, // Alaska
    {  309,  135,  285585,  285601,  285626, 16, 25, 25, }, // Amazon
    {  309,  142,  285651,  285677,  285712, 26, 35, 35, }, // America_Central
    {  309,  158,  285747,  285778,  285818, 31, 40, 40, }, // America_Eastern
    {  309,  174,  285858,  285891,  285933, 33, 42, 42, }, // America_Mountain
    {  309,  191,  285975,  286009,  286052, 34, 43, 43, }, // America_Pacific
    {  309,  207,  286095,  286112,  286138, 17, 26, 26, }, // Anadyr
    {  309,  214,  286164,  286176,  286197, 12, 21, 21, }, // Apia
    {  309,  232,  286218,  286228,  286247, 10, 19, 19, }, // Arabian
    {  309,  240,  286266,  286283,  286309, 17, 26, 26, }, // Argentina
    {  309,  250,  286335,  286363,  286400, 28, 37, 37, }, // Argentina_Western
    {  309,  268,  286437,  286452,  286476, 15, 24, 24, }, // Armenia
    {  309,  286,  286500,  286517,  286543, 17, 26, 26, }, // Atlantic
    {  309,  295,  286569,  286594,  286628, 25, 34, 34, }, // Australia_Central
    {  309,  313,  286662,  286698,  286743, 36, 45, 45, }, // Australia_CentralWestern
    {  309,  338,  286788,  286814,  286849, 26, 35, 35, }, // Australia_Eastern
    {  309,  356,  286662,  286698,  286743, 28, 37, 37, }, // Australia_Western
    {  309,  374,  286884,  286902,  286929, 18, 27, 27, }, // Azerbaijan
    {  309,  385,  286956,  286971,  286995, 15, 24, 24, }, // Azores
    {  309,  397,  287019,  287037,  287064, 18, 27, 27, }, // Bangladesh
    {  309,  415,       0,  287091,       0,  0, 13,  0, }, // Bhutan
    {  309,  422,       0,  287104,       0,  0, 15,  0, }, // Bolivia
    {  309,  437,  287119,  287135,  287160, 16, 25, 25, }, // Brasilia
    {  309,  454,       0,  287185,       0,  0, 25,  0, }, // Brunei
    {  309,  461,  287210,  287228,  287255, 18, 27, 27, }, // Cape_Verde
    {  309,  478,       0,  287282,       0,  0, 22,  0, }, // Chamorro
    {  309,  487,  287304,  287319,  287343, 15, 24, 24, }, // Chatham
    {  309,  495,  287367,  287380,  287402, 13, 22, 22, }, // Chile
    {  309,  501,  287424,  287437,  287459, 13, 22, 22, }, // China
    {  309,  507,  287481,  287499,  287526, 18, 27, 27, }, // Choibalsan
    {  309,  518,       0,  287553,       0,  0, 22,  0, }, // Christmas
    {  309,  528,       0,  287575,       0,  0, 20,  0, }, // Cocos
    {  309,  534,  287595,  287611,  287636, 16, 25, 25, }, // Colombia
    {  309,  543,  287661,  287680,  287708, 19, 28, 34, }, // Cook
    {  309,  548,  287742,  287754,  287775, 12, 21, 21, }, // Cuba
    {  309,  559,       0,  287796,       0,  0, 13,  0, }, // Davis
    {  309,  575,       0,  287809,       0,  0, 24,  0, }, // DumontDUrville
    {  309,  612,       0,  287833,       0,  0, 19,  0, }, // East_Timor
    {  309,  623,  287852,  287874,  287905, 22, 31, 31, }, // Easter
    {  309,  630,       0,  287936,       0,  0, 15,  0, }, // Ecuador
    {  309,  638,  287951,  287973,  288004, 22, 31, 31, }, // Europe_Central
    {  309,  653,  288035,  288058,  288090, 23, 32, 32, }, // Europe_Eastern
    {  309,  668,       0,  288122,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  309,  691,  288151,  288176,  288210, 25, 34, 34, }, // Europe_Western
    {  309,  706,  288244,  288267,  288299, 23, 32, 32, }, // Falkland
    {  309,  715,  288331,  288343,  288364, 12, 21, 21, }, // Fiji
    {  309,  720,       0,  288385,       0,  0, 23,  0, }, // French_Guiana
    {  309,  734,       0,  288408,       0,  0, 47,  0, }, // French_Southern
    {  309,  757,       0,  288455,       0,  0, 18,  0, }, // Galapagos
    {  309,  767,       0,  288473,       0,  0, 15,  0, }, // Gambier
    {  309,  775,  288488,  288503,  288527, 15, 24, 24, }, // Georgia
    {  309,  783,       0,  288551,       0,  0, 22,  0, }, // Gilbert_Islands
    {  309,  799,       0,  288573,       0,  0, 30,  0, }, // GMT
    {  309,  841,  288603,  288631,  288668, 28, 37, 37, }, // Greenland_Eastern
    {  309,  859,  288705,  288735,  288774, 30, 39, 39, }, // Greenland_Western
    {  309,  882,       0,  288813,       0,  0, 13,  0, }, // Gulf
    {  309,  887,       0,  288385,       0,  0, 14,  0, }, // Guyana
    {  309,  894,  288826,  288856,  288895, 30, 39, 39, }, // Hawaii_Aleutian
    {  309,  910,  288934,  288951,  288977, 17, 26, 26, }, // Hong_Kong
    {  309,  920,  289003,  289015,  289036, 12, 21, 21, }, // Hovd
    {  309,  925,       0,  289057,       0,  0, 13,  0, }, // India
    {  309,  931,       0,  289070,       0,  0, 21,  0, }, // Indian_Ocean
    {  309,  944,       0,  289091,       0,  0, 17,  0, }, // Indochina
    {  309,  954,       0,  289108,       0,  0, 25,  0, }, // Indonesia_Central
    {  309,  972,       0,  289133,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  309,  990,       0,  289159,       0,  0, 28,  0, }, // Indonesia_Western
    {  309, 1008,  289187,  289199,  289220, 12, 21, 21, }, // Iran
    {  309, 1019,  289241,  289256,  289280, 15, 24, 24, }, // Irkutsk
    {  309, 1027,  289304,  289318,  289341, 14, 23, 23, }, // Israel
    {  309, 1034,  289364,  289377,  289399, 13, 22, 22, }, // Japan
    {  309, 1040,  289421,  289456,  289500, 35, 44, 44, }, // Kamchatka
    {  309, 1058,       0,  289544,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  309, 1077,       0,  289571,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  309, 1106,  289600,  289613,  289635, 13, 22, 22, }, // Korea
    {  309, 1112,       0,  289657,       0,  0, 14,  0, }, // Kosrae
    {  309, 1119,  289671,  289690,  289718, 19, 28, 28, }, // Krasnoyarsk
    {  309, 1151,       0,  289746,       0,  0, 20,  0, }, // Kyrgystan
    {  309, 1175,       0,  289766,       0,  0, 22,  0, }, // Line_Islands
    {  309, 1188,  289788,  289805,  289831, 17, 26, 26, }, // Lord_Howe
    {  309, 1204,  289857,  289872,  289896, 15, 24, 24, }, // Magadan
    {  309, 1219,       0,  289920,       0,  0, 16,  0, }, // Malaysia
    {  309, 1228,       0,  289936,       0,  0, 17,  0, }, // Maldives
    {  309, 1237,       0,  289953,       0,  0, 18,  0, }, // Marquesas
    {  309, 1247,       0,  289971,       0,  0, 23,  0, }, // Marshall_Islands
    {  309, 1264,  289994,  290010,  290035, 16, 25, 25, }, // Mauritius
    {  309, 1274,       0,  290060,       0,  0, 14,  0, }, // Mawson
    {  309, 1281,  290074,  290099,  290133, 25, 34, 34, }, // Mexico_Pacific
    {  309, 1296,  290167,  290186,  290214, 19, 28, 28, }, // Mongolia
    {  309, 1305,  290242,  290257,  290281, 15, 24, 24, }, // Moscow
    {  309, 1312,       0,  290305,       0,  0, 15,  0, }, // Myanmar
    {  309, 1320,       0,  290320,       0,  0, 13,  0, }, // Nauru
    {  309, 1326,       0,  290333,       0,  0, 13,  0, }, // Nepal
    {  309, 1332,  290346,  290368,  290399, 22, 31, 31, }, // New_Caledonia
    {  309, 1346,  290430,  290451,  290481, 21, 30, 30, }, // New_Zealand
    {  309, 1358,  290511,  290529,  290556, 18, 27, 27, }, // Newfoundland
    {  309, 1371,       0,  290583,       0,  0, 12,  0, }, // Niue
    {  309, 1376,  290595,  290615,  290644, 20, 29, 29, }, // Norfolk
    {  309, 1384,  290673,  290700,  290736, 27, 36, 36, }, // Noronha
    {  309, 1406,  290772,  290791,  290819, 19, 28, 28, }, // Novosibirsk
    {  309, 1418,  290847,  290859,  290880, 12, 21, 21, }, // Omsk
    {  309, 1428,  290901,  290918,  290944, 17, 26, 26, }, // Pakistan
    {  309, 1437,       0,  290970,       0,  0, 13,  0, }, // Palau
    {  309, 1443,       0,  290983,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  309, 1460,  291008,  291024,  291049, 16, 25, 25, }, // Paraguay
    {  309, 1469,  291074,  291086,  291107, 12, 21, 21, }, // Peru
    {  309, 1474,  291128,  291145,  291171, 17, 26, 26, }, // Philippines
    {  309, 1486,       0,  291197,       0,  0, 20,  0, }, // Phoenix_Islands
    {  309, 1502,  291217,  291249,  291290, 32, 41, 41, }, // Pierre_Miquelon
    {  309, 1518,       0,  291331,       0,  0, 16,  0, }, // Pitcairn
    {  309, 1527,       0,  291347,       0,  0, 15,  0, }, // Ponape
    {  309, 1534,       0,   58535,       0,  0, 17,  0, }, // Pyongyang
    {  309, 1554,       0,  291362,       0,  0, 15,  0, }, // Reunion
    {  309, 1562,       0,  291377,       0,  0, 15,  0, }, // Rothera
    {  309, 1570,  291392,  291408,  291433, 16, 25, 25, }, // Sakhalin
    {  309, 1579,  291458,  291475,  291501, 17, 26, 26, }, // Samara
    {  309, 1596,  291527,  291540,  291562, 13, 22, 22, }, // Samoa
    {  309, 1602,       0,  291584,       0,  0, 19,  0, }, // Seychelles
    {  309, 1624,       0,  291603,       0,  0, 16,  0, }, // Singapore
    {  309, 1634,       0,  291619,       0,  0, 22,  0, }, // Solomon
    {  309, 1642,       0,  291641,       0,  0, 22,  0, }, // South_Georgia
    {  309, 1656,       0,  291663,       0,  0, 16,  0, }, // Suriname
    {  309, 1676,       0,  291679,       0,  0, 13,  0, }, // Syowa
    {  309, 1682,       0,  291692,       0,  0, 14,  0, }, // Tahiti
    {  309, 1689,  291706,  291720,  291743, 14, 23, 23, }, // Taipei
    {  309, 1696,       0,  291766,       0,  0, 19,  0, }, // Tajikistan
    {  309, 1724,       0,  291785,       0,  0, 15,  0, }, // Tokelau
    {  309, 1732,  291800,  291813,  291835, 13, 22, 22, }, // Tonga
    {  309, 1738,       0,  291857,       0,  0, 13,  0, }, // Truk
    {  309, 1750,  291870,  291890,  291919, 20, 29, 29, }, // Turkmenistan
    {  309, 1763,       0,  291948,       0,  0, 14,  0, }, // Tuvalu
    {  309, 1777,  291962,  291977,  292001, 15, 24, 24, }, // Uruguay
    {  309, 1792,  292025,  292043,  292070, 18, 27, 27, }, // Uzbekistan
    {  309, 1803,  292097,  292112,  292136, 15, 24, 24, }, // Vanuatu
    {  309, 1811,       0,  292160,       0,  0, 17,  0, }, // Venezuela
    {  309, 1821,  292177,  292196,  292224, 19, 28, 28, }, // Vladivostok
    {  309, 1833,  292252,  292270,  292297, 18, 27, 27, }, // Volgograd
    {  309, 1843,       0,  292324,       0,  0, 14,  0, }, // Vostok
    {  309, 1850,       0,  292338,       0,  0, 17,  0, }, // Wake
    {  309, 1855,       0,  292355,       0,  0, 23,  0, }, // Wallis
    {  309, 1862,  292378,  292393,  292417, 15, 24, 24, }, // Yakutsk
    {  309, 1870,  292441,  292462,  292492, 21, 30, 30, }, // Yekaterinburg
    {  309, 1892,       0,  292522,       0,  0, 13,  0, }, // Yukon
    {  313,    5,       0,  292535,       0,  0, 13,  0, }, // Afghanistan Georgian/Georgian/Georgia
    {  313,   17,       0,  292548,       0,  0, 22,  0, }, // Africa_Central
    {  313,   32,       0,  292570,       0,  0, 22,  0, }, // Africa_Eastern
    {  313,   65,       0,  292592,       0,  0, 19,  0, }, // Africa_Southern
    {  313,   81,  292611,  292631,  292663, 20, 32, 29, }, // Africa_Western
    {  313,  107,  292692,  292703,  292726, 11, 23, 20, }, // Alaska
    {  313,  135,  292746,  292759,  292784, 13, 25, 22, }, // Amazon
    {  313,  142,  292806,  292838,  292882, 32, 44, 41, }, // America_Central
    {  313,  158,  292923,  292957,  293003, 34, 46, 43, }, // America_Eastern
    {  313,  174,  293046,  293077,  293120, 31, 43, 40, }, // America_Mountain
    {  313,  191,  293160,  293196,  293244, 36, 48, 45, }, // America_Pacific
    {  313,  214,  293289,  293298,  293319,  9, 21, 18, }, // Apia
    {  313,  232,  293337,  293349,  293373, 12, 24, 21, }, // Arabian
    {  313,  240,  293394,  293408,  293434, 14, 26, 23, }, // Argentina
    {  313,  250,  293457,  293480,  293515, 23, 35, 32, }, // Argentina_Western
    {  313,  268,  293547,  293559,  293583, 12, 24, 21, }, // Armenia
    {  313,  286,  293604,  293626,  293660, 22, 34, 31, }, // Atlantic
    {  313,  295,  293691,  293716,  293753, 25, 37, 34, }, // Australia_Central
    {  313,  313,  293787,  293824,  293873, 37, 49, 46, }, // Australia_CentralWestern
    {  313,  338,  293919,  293944,  293981, 25, 37, 34, }, // Australia_Eastern
    {  313,  356,  293801,  293838,  293887, 23, 35, 32, }, // Australia_Western
    {  313,  374,  294015,  294031,  294059, 16, 28, 25, }, // Azerbaijan
    {  313,  385,  294084,  294105,  294138, 21, 33, 30, }, // Azores
    {  313,  397,  294168,  294183,  294210, 15, 27, 24, }, // Bangladesh
    {  313,  415,       0,  294234,       0,  0, 11,  0, }, // Bhutan
    {  313,  422,       0,  294245,       0,  0, 12,  0, }, // Bolivia
    {  313,  437,  294257,  294270,  294295, 13, 25, 22, }, // Brasilia
    {  313,  454,       0,  294317,       0,  0, 22,  0, }, // Brunei
    {  313,  461,  294339,  294354,  294381, 15, 27, 24, }, // Cape_Verde
    {  313,  478,       0,  294405,       0,  0, 11,  0, }, // Chamorro
    {  313,  487,  294416,  294427,  294450, 11, 23, 20, }, // Chatham
    {  313,  495,  294470,  294479,  294500,  9, 21, 18, }, // Chile
    {  313,  501,  294518,  294529,  294552, 11, 23, 22, }, // China
    {  313,  507,  294574,  294589,  294616, 15, 27, 24, }, // Choibalsan
    {  313,  518,       0,  294640,       0,  0, 18,  0, }, // Christmas
    {  313,  528,       0,  294658,       0,  0, 22,  0, }, // Cocos
    {  313,  534,  294680,  294693,  294718, 13, 25, 22, }, // Colombia
    {  313,  543,  294740,  294760,  294792, 20, 32, 38, }, // Cook
    {  313,  548,  294830,  294839,  294860,  9, 21, 18, }, // Cuba
    {  313,  559,       0,  294878,       0,  0, 11,  0, }, // Davis
    {  313,  575,       0,  294889,       0,  0, 18,  0, }, // DumontDUrville
    {  313,  612,       0,  294907,       0,  0, 22,  0, }, // East_Timor
    {  313,  623,  294929,  294950,  294983, 21, 33, 30, }, // Easter
    {  313,  630,       0,  295013,       0,  0, 13,  0, }, // Ecuador
    {  313,  638,  295026,  295048,  295082, 22, 34, 31, }, // Europe_Central
    {  313,  653,  295113,  295135,  295169, 22, 34, 31, }, // Europe_Eastern
    {  313,  668,       0,  295200,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  313,  691,  295230,  295250,  295282, 20, 32, 29, }, // Europe_Western
    {  313,  706,  295311,  295336,  295373, 25, 37, 34, }, // Falkland
    {  313,  715,  295407,  295416,  295437,  9, 21, 18, }, // Fiji
    {  313,  720,       0,  295455,       0,  0, 23,  0, }, // French_Guiana
    {  313,  734,       0,  295478,       0,  0, 37,  0, }, // French_Southern
    {  313,  757,       0,  295515,       0,  0, 15,  0, }, // Galapagos
    {  313,  767,       0,  295530,       0,  0, 13,  0, }, // Gambier
    {  313,  775,  295543,  295558,  295585, 15, 27, 24, }, // Georgia
    {  313,  783,       0,  295609,       0,  0, 24,  0, }, // Gilbert_Islands
    {  313,  799,       0,  295633,       0,  0, 21,  0, }, // GMT
    {  313,  841,  295654,  295680,  295718, 26, 38, 35, }, // Greenland_Eastern
    {  313,  859,  295753,  295777,  295813, 24, 36, 33, }, // Greenland_Western
    {  313,  882,       0,  295846,       0,  0, 31,  0, }, // Gulf
    {  313,  887,       0,  295877,       0,  0, 11,  0, }, // Guyana
    {  313,  894,  295888,  295910,  295944, 22, 34, 31, }, // Hawaii_Aleutian
    {  313,  910,  295975,  295988,  296013, 13, 25, 22, }, // Hong_Kong
    {  313,  920,  296035,  296045,  296067, 10, 22, 19, }, // Hovd
    {  313,  925,       0,  296086,       0,  0, 12,  0, }, // India
    {  313,  931,       0,  296098,       0,  0, 31,  0, }, // Indian_Ocean
    {  313,  944,       0,  296129,       0,  0, 15,  0, }, // Indochina
    {  313,  954,       0,  296144,       0,  0, 25,  0, }, // Indonesia_Central
    {  313,  972,       0,  296169,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  313,  990,       0,  296194,       0,  0, 23,  0, }, // Indonesia_Western
    {  313, 1008,  296217,  296227,  296249, 10, 22, 21, }, // Iran
    {  313, 1019,  296270,  296283,  296308, 13, 25, 22, }, // Irkutsk
    {  313, 1027,  296330,  296342,  296366, 12, 24, 21, }, // Israel
    {  313, 1034,  296387,  296399,  296423, 12, 24, 21, }, // Japan
    {  313, 1058,       0,  296444,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  313, 1077,       0,  296468,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  313, 1106,  296490,  296500,  296522, 10, 22, 19, }, // Korea
    {  313, 1112,       0,  296541,       0,  0, 10,  0, }, // Kosrae
    {  313, 1119,  296551,  296568,  296597, 17, 29, 26, }, // Krasnoyarsk
    {  313, 1151,       0,  296623,       0,  0, 14,  0, }, // Kyrgystan
    {  313, 1175,       0,  296637,       0,  0, 21,  0, }, // Line_Islands
    {  313, 1188,  296658,  296671,  296696, 13, 25, 22, }, // Lord_Howe
    {  313, 1204,  296718,  296731,  296756, 13, 25, 22, }, // Magadan
    {  313, 1219,       0,  296778,       0,  0, 13,  0, }, // Malaysia
    {  313, 1228,       0,  296791,       0,  0, 14,  0, }, // Maldives
    {  313, 1237,       0,  296805,       0,  0, 23,  0, }, // Marquesas
    {  313, 1247,       0,  296828,       0,  0, 23,  0, }, // Marshall_Islands
    {  313, 1264,  296851,  296863,  296887, 12, 24, 21, }, // Mauritius
    {  313, 1274,       0,  296908,       0,  0, 12,  0, }, // Mawson
    {  313, 1281,  296920,  296947,  296986, 27, 39, 36, }, // Mexico_Pacific
    {  313, 1296,  297022,  297038,  297066, 16, 28, 25, }, // Mongolia
    {  313, 1305,  297091,  297103,  297127, 12, 24, 21, }, // Moscow
    {  313, 1312,       0,  297148,       0,  0, 13,  0, }, // Myanmar
    {  313, 1320,       0,  297161,       0,  0, 10,  0, }, // Nauru
    {  313, 1326,       0,  297171,       0,  0, 11,  0, }, // Nepal
    {  313, 1332,  297182,  297202,  297234, 20, 32, 29, }, // New_Caledonia
    {  313, 1346,  297263,  297282,  297313, 19, 31, 28, }, // New_Zealand
    {  313, 1358,  297341,  297359,  297389, 18, 30, 27, }, // Newfoundland
    {  313, 1371,       0,  297416,       0,  0,  9,  0, }, // Niue
    {  313, 1376,  297425,  297447,  297481, 22, 34, 31, }, // Norfolk
    {  313, 1384,  297512,  297536,  297572, 24, 36, 33, }, // Noronha
    {  313, 1406,  297605,  297622,  297651, 17, 29, 26, }, // Novosibirsk
    {  313, 1418,  297677,  297687,  297709, 10, 22, 19, }, // Omsk
    {  313, 1428,  297728,  297742,  297768, 14, 26, 23, }, // Pakistan
    {  313, 1437,       0,  297791,       0,  0, 10,  0, }, // Palau
    {  313, 1443,       0,  297801,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  313, 1460,  297824,  297837,  297862, 13, 25, 22, }, // Paraguay
    {  313, 1469,  297884,  297893,  297914,  9, 21, 18, }, // Peru
    {  313, 1474,  297932,  297947,  297974, 15, 27, 24, }, // Philippines
    {  313, 1486,       0,  297998,       0,  0, 23,  0, }, // Phoenix_Islands
    {  313, 1502,  298021,  298048,  298087, 27, 39, 36, }, // Pierre_Miquelon
    {  313, 1518,       0,  298123,       0,  0, 13,  0, }, // Pitcairn
    {  313, 1527,       0,  298136,       0,  0, 11,  0, }, // Ponape
    {  313, 1534,       0,  298147,       0,  0, 13,  0, }, // Pyongyang
    {  313, 1554,       0,  298160,       0,  0, 14,  0, }, // Reunion
    {  313, 1562,       0,  298174,       0,  0, 11,  0, }, // Rothera
    {  313, 1570,  298185,  298198,  298223, 13, 25, 22, }, // Sakhalin
    {  313, 1596,  298245,  298255,  298277, 10, 22, 19, }, // Samoa
    {  313, 1602,       0,  298296,       0,  0, 23,  0, }, // Seychelles
    {  313, 1624,       0,  298319,       0,  0, 14,  0, }, // Singapore
    {  313, 1634,       0,  298333,       0,  0, 24,  0, }, // Solomon
    {  313, 1642,       0,  298357,       0,  0, 20,  0, }, // South_Georgia
    {  313, 1656,       0,  298377,       0,  0, 13,  0, }, // Suriname
    {  313, 1676,       0,  298390,       0,  0, 10,  0, }, // Syowa
    {  313, 1682,       0,  298400,       0,  0, 10,  0, }, // Tahiti
    {  313, 1689,  298410,  298421,  298444, 11, 23, 20, }, // Taipei
    {  313, 1696,       0,  298464,       0,  0, 13,  0, }, // Tajikistan
    {  313, 1724,       0,  298477,       0,  0, 12,  0, }, // Tokelau
    {  313, 1732,  298489,  298499,  298521, 10, 22, 19, }, // Tonga
    {  313, 1738,       0,  298540,       0,  0, 10,  0, }, // Truk
    {  313, 1750,  298550,  298565,  298592, 15, 27, 24, }, // Turkmenistan
    {  313, 1763,       0,  298616,       0,  0, 11,  0, }, // Tuvalu
    {  313, 1777,  298627,  298639,  298663, 12, 24, 21, }, // Uruguay
    {  313, 1792,  298684,  298697,  298722, 13, 25, 22, }, // Uzbekistan
    {  313, 1803,  298744,  298756,  298780, 12, 24, 21, }, // Vanuatu
    {  313, 1811,       0,  298801,       0,  0, 14,  0, }, // Venezuela
    {  313, 1821,  298815,  298832,  298861, 17, 29, 26, }, // Vladivostok
    {  313, 1833,  298887,  298902,  298929, 15, 27, 24, }, // Volgograd
    {  313, 1843,       0,  298820,       0,  0, 12,  0, }, // Vostok
    {  313, 1850,       0,  298953,       0,  0, 19,  0, }, // Wake
    {  313, 1855,       0,  298972,       0,  0, 21,  0, }, // Wallis
    {  313, 1862,  298993,  299006,  299031, 13, 25, 22, }, // Yakutsk
    {  313, 1870,  299053,  299071,  299101, 18, 30, 27, }, // Yekaterinburg
    {  313, 1892,       0,  299128,       0,  0, 11,  0, }, // Yukon
    {  314,    0,  299139,  299148,  299163,  9, 15, 15, }, // Acre German/Latin/Germany
    {  314,    5,       0,  299178,       0,  0, 16,  0, }, // Afghanistan
    {  314,   17,       0,  299194,       0,  0, 24,  0, }, // Africa_Central
    {  314,   32,       0,  299218,       0,  0, 20,  0, }, // Africa_Eastern
    {  314,   65,       0,  299238,       0,  0, 20,  0, }, // Africa_Southern
    {  314,   81,  299258,  299279,  299306, 21, 27, 27, }, // Africa_Western
    {  314,  107,  299333,  299344,  299361, 11, 17, 17, }, // Alaska
    {  314,  128,  299378,  299389,  299406, 11, 17, 17, }, // Almaty
    {  314,  135,  299423,  299436,  299455, 13, 19, 19, }, // Amazon
    {  314,  142,  299474,  299503,  299539, 29, 36, 36, }, // America_Central
    {  314,  158,  299575,  299606,  299644, 31, 38, 38, }, // America_Eastern
    {  314,  174,  299682,  299701,  299726, 19, 25, 25, }, // America_Mountain
    {  314,  191,  299751,  299783,  299822, 32, 39, 39, }, // America_Pacific
    {  314,  207,  299861,  299872,  299889, 11, 17, 17, }, // Anadyr
    {  314,  214,  299906,  299915,  299930,  9, 15, 15, }, // Apia
    {  314,  219,  299945,  299955,  299971, 10, 16, 16, }, // Aqtau
    {  314,  225,  299987,  299998,  300015, 11, 17, 17, }, // Aqtobe
    {  314,  232,  300032,  300046,  300066, 14, 20, 20, }, // Arabian
    {  314,  240,  300086,  300104,  300128, 18, 24, 24, }, // Argentina
    {  314,  250,  300152,  300174,  300202, 22, 28, 28, }, // Argentina_Western
    {  314,  268,  300230,  300245,  300266, 15, 21, 21, }, // Armenia
    {  314,  286,  300287,  300300,  300319, 13, 19, 19, }, // Atlantic
    {  314,  295,  300338,  300362,  300392, 24, 30, 30, }, // Australia_Central
    {  314,  313,  300422,  300452,  300488, 30, 36, 36, }, // Australia_CentralWestern
    {  314,  338,  300524,  300544,  300570, 20, 26, 26, }, // Australia_Eastern
    {  314,  356,  300431,  300461,  300497, 21, 27, 27, }, // Australia_Western
    {  314,  374,  300596,  300619,  300648, 23, 29, 29, }, // Azerbaijan
    {  314,  385,  300677,  300688,  300705, 11, 17, 17, }, // Azores
    {  314,  397,  300722,  300738,  300760, 16, 22, 22, }, // Bangladesh
    {  314,  415,       0,  300782,       0,  0, 11,  0, }, // Bhutan
    {  314,  422,       0,  300793,       0,  0, 18,  0, }, // Bolivia
    {  314,  437,  300811,  300824,  300843, 13, 19, 19, }, // Brasilia
    {  314,  454,       0,  300862,       0,  0, 22,  0, }, // Brunei
    {  314,  461,  300884,  300899,  300920, 15, 21, 21, }, // Cape_Verde
    {  314,  472,       0,  300941,       0,  0, 10,  0, }, // Casey
    {  314,  478,       0,  300951,       0,  0, 13,  0, }, // Chamorro
    {  314,  487,  300964,  300976,  300994, 12, 18, 18, }, // Chatham
    {  314,  495,  301012,  301028,  301050, 16, 22, 22, }, // Chile
    {  314,  501,  301072,  301088,  301110, 16, 22, 22, }, // China
    {  314,  507,  301132,  301149,  301172, 17, 23, 23, }, // Choibalsan
    {  314,  518,       0,  301195,       0,  0, 20,  0, }, // Christmas
    {  314,  528,       0,  301215,       0,  0, 16,  0, }, // Cocos
    {  314,  534,  301231,  301250,  301275, 19, 25, 25, }, // Colombia
    {  314,  543,  301300,  301315,  301336, 15, 21, 21, }, // Cook
    {  314,  548,  301357,  301372,  301393, 15, 21, 21, }, // Cuba
    {  314,  559,       0,  301414,       0,  0, 10,  0, }, // Davis
    {  314,  575,       0,  301424,       0,  0, 21,  0, }, // DumontDUrville
    {  314,  612,       0,  301445,       0,  0, 13,  0, }, // East_Timor
    {  314,  623,  301458,  301473,  301494, 15, 21, 21, }, // Easter
    {  314,  630,       0,  301515,       0,  0, 20,  0, }, // Ecuador
    {  314,  638,  301535,  301557,  301585, 22, 28, 28, }, // Europe_Central
    {  314,  653,  301613,  301632,  301657, 19, 25, 25, }, // Europe_Eastern
    {  314,  668,       0,  301682,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {  314,  691,  301700,  301720,  301746, 20, 26, 26, }, // Europe_Western
    {  314,  706,  301772,  301791,  301816, 19, 25, 25, }, // Falkland
    {  314,  715,  301841,  301853,  301871, 12, 18, 18, }, // Fiji
    {  314,  720,       0,  301889,       0,  0, 24,  0, }, // French_Guiana
    {  314,  734,       0,  301913,       0,  0, 42,  0, }, // French_Southern
    {  314,  757,       0,  301955,       0,  0, 14,  0, }, // Galapagos
    {  314,  767,       0,  301969,       0,  0, 12,  0, }, // Gambier
    {  314,  775,  301981,  301996,  302017, 15, 21, 21, }, // Georgia
    {  314,  783,       0,  302038,       0,  0, 19,  0, }, // Gilbert_Islands
    {  314,  799,       0,  302057,       0,  0, 23,  0, }, // GMT
    {  314,  841,  302080,  302096,  302118, 16, 22, 22, }, // Greenland_Eastern
    {  314,  859,  302140,  302157,  302180, 17, 23, 23, }, // Greenland_Western
    {  314,  877,       0,  302203,       0,  0,  9,  0, }, // Guam
    {  314,  882,       0,  302212,       0,  0,  9,  0, }, // Gulf
    {  314,  887,       0,  302221,       0,  0, 11,  0, }, // Guyana
    {  314,  894,  302232,  302251,  302276, 19, 25, 25, }, // Hawaii_Aleutian
    {  314,  910,  302301,  302314,  302333, 13, 19, 19, }, // Hong_Kong
    {  314,  920,  302352,  302362,  302378, 10, 16, 16, }, // Hovd
    {  314,  925,       0,  302394,       0,  0, 19,  0, }, // India
    {  314,  931,       0,  302413,       0,  0, 20,  0, }, // Indian_Ocean
    {  314,  944,       0,  302433,       0,  0, 14,  0, }, // Indochina
    {  314,  954,       0,  302447,       0,  0, 24,  0, }, // Indonesia_Central
    {  314,  972,       0,  302471,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  314,  990,       0,  302491,       0,  0, 21,  0, }, // Indonesia_Western
    {  314, 1008,  302512,  302526,  302546, 14, 20, 20, }, // Iran
    {  314, 1019,  302566,  302580,  302600, 14, 20, 20, }, // Irkutsk
    {  314, 1027,  302620,  302636,  302658, 16, 22, 22, }, // Israel
    {  314, 1034,  302680,  302695,  302716, 15, 21, 21, }, // Japan
    {  314, 1040,  302737,  302753,  302775, 16, 22, 22, }, // Kamchatka
    {  314, 1058,       0,  302797,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  314, 1077,       0,  302816,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  314, 1106,  302836,  302852,  302874, 16, 22, 22, }, // Korea
    {  314, 1112,       0,  302896,       0,  0, 11,  0, }, // Kosrae
    {  314, 1119,  302907,  302925,  302949, 18, 24, 24, }, // Krasnoyarsk
    {  314, 1151,       0,  302973,       0,  0, 16,  0, }, // Kyrgystan
    {  314, 1161,       0,  302989,       0,  0, 14,  0, }, // Lanka
    {  314, 1175,       0,  303003,       0,  0, 17,  0, }, // Line_Islands
    {  314, 1188,  303020,  303034,  303054, 14, 20, 20, }, // Lord_Howe
    {  314, 1198,  303074,  303084,  303100, 10, 16, 16, }, // Macau
    {  314, 1204,  303116,  303128,  303146, 12, 18, 18, }, // Magadan
    {  314, 1219,       0,  303164,       0,  0, 16,  0, }, // Malaysia
    {  314, 1228,       0,  303180,       0,  0, 14,  0, }, // Maldives
    {  314, 1237,       0,  303194,       0,  0, 14,  0, }, // Marquesas
    {  314, 1247,       0,  303208,       0,  0, 19,  0, }, // Marshall_Islands
    {  314, 1264,  303227,  303241,  303261, 14, 20, 20, }, // Mauritius
    {  314, 1274,       0,  303281,       0,  0, 11,  0, }, // Mawson
    {  314, 1281,  303292,  303316,  303347, 24, 31, 31, }, // Mexico_Pacific
    {  314, 1296,  303378,  303394,  303416, 16, 22, 22, }, // Mongolia
    {  314, 1305,  303438,  303451,  303470, 13, 19, 19, }, // Moscow
    {  314, 1312,       0,  303489,       0,  0, 12,  0, }, // Myanmar
    {  314, 1320,       0,  303501,       0,  0, 10,  0, }, // Nauru
    {  314, 1326,       0,  303511,       0,  0, 17,  0, }, // Nepal
    {  314, 1332,  303528,  303548,  303574, 20, 26, 26, }, // New_Caledonia
    {  314, 1346,  303600,  303615,  303636, 15, 21, 21, }, // New_Zealand
    {  314, 1358,  303657,  303673,  303695, 16, 22, 22, }, // Newfoundland
    {  314, 1371,       0,  303717,       0,  0,  9,  0, }, // Niue
    {  314, 1376,  303726,  303743,  303766, 17, 23, 23, }, // Norfolk
    {  314, 1384,  303789,  303813,  303843, 24, 30, 30, }, // Noronha
    {  314, 1392,       0,  303873,       0,  0, 23,  0, }, // North_Mariana
    {  314, 1406,  303896,  303914,  303938, 18, 24, 24, }, // Novosibirsk
    {  314, 1418,  303962,  303973,  303990, 11, 17, 17, }, // Omsk
    {  314, 1428,  304007,  304025,  304049, 18, 24, 24, }, // Pakistan
    {  314, 1437,       0,  304073,       0,  0, 10,  0, }, // Palau
    {  314, 1443,       0,  304083,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  314, 1460,  304103,  304121,  304145, 18, 24, 24, }, // Paraguay
    {  314, 1469,  304169,  304185,  304207, 16, 22, 22, }, // Peru
    {  314, 1474,  304229,  304248,  304273, 19, 25, 25, }, // Philippines
    {  314, 1486,       0,  304298,       0,  0, 18,  0, }, // Phoenix_Islands
    {  314, 1502,  304316,  304344,  304378, 28, 34, 34, }, // Pierre_Miquelon
    {  314, 1518,       0,  304412,       0,  0, 19,  0, }, // Pitcairn
    {  314, 1527,       0,  304431,       0,  0, 11,  0, }, // Ponape
    {  314, 1534,       0,  304442,       0,  0, 14,  0, }, // Pyongyang
    {  314, 1544,  304456,  304471,  304492, 15, 21, 20, }, // Qyzylorda
    {  314, 1554,       0,  304512,       0,  0, 12,  0, }, // Reunion
    {  314, 1562,       0,  304524,       0,  0, 12,  0, }, // Rothera
    {  314, 1570,  304536,  304549,  304568, 13, 19, 19, }, // Sakhalin
    {  314, 1579,  304587,  304598,  304615, 11, 17, 17, }, // Samara
    {  314, 1596,  304632,  304642,  304658, 10, 16, 16, }, // Samoa
    {  314, 1602,       0,  304674,       0,  0, 15,  0, }, // Seychelles
    {  314, 1624,       0,  304689,       0,  0, 24,  0, }, // Singapore
    {  314, 1634,       0,  304713,       0,  0, 14,  0, }, // Solomon
    {  314, 1642,       0,  304727,       0,  0, 18,  0, }, // South_Georgia
    {  314, 1656,       0,  304745,       0,  0, 13,  0, }, // Suriname
    {  314, 1676,       0,  304758,       0,  0, 10,  0, }, // Syowa
    {  314, 1682,       0,  304768,       0,  0, 11,  0, }, // Tahiti
    {  314, 1689,  304779,  304790,  304807, 11, 17, 17, }, // Taipei
    {  314, 1696,       0,  304824,       0,  0, 18,  0, }, // Tajikistan
    {  314, 1724,       0,  304842,       0,  0, 12,  0, }, // Tokelau
    {  314, 1732,  304854,  304869,  304890, 15, 21, 21, }, // Tonga
    {  314, 1738,       0,  304911,       0,  0, 10,  0, }, // Truk
    {  314, 1750,  304921,  304938,  304961, 17, 23, 23, }, // Turkmenistan
    {  314, 1763,       0,  304984,       0,  0, 11,  0, }, // Tuvalu
    {  314, 1777,  304995,  305012,  305035, 17, 23, 23, }, // Uruguay
    {  314, 1792,  305058,  305073,  305094, 15, 21, 21, }, // Uzbekistan
    {  314, 1803,  305115,  305127,  305145, 12, 18, 18, }, // Vanuatu
    {  314, 1811,       0,  305163,       0,  0, 14,  0, }, // Venezuela
    {  314, 1821,  305177,  305195,  305219, 18, 24, 24, }, // Vladivostok
    {  314, 1833,  305243,  305259,  305281, 16, 22, 22, }, // Volgograd
    {  314, 1843,       0,  305303,       0,  0, 11,  0, }, // Vostok
    {  314, 1850,       0,  305314,       0,  0, 15,  0, }, // Wake
    {  314, 1855,       0,  305329,       0,  0, 22,  0, }, // Wallis
    {  314, 1862,  305351,  305365,  305385, 14, 20, 20, }, // Yakutsk
    {  314, 1870,  305405,  305425,  305451, 20, 26, 26, }, // Yekaterinburg
    {  314, 1892,       0,  305477,       0,  0, 10,  0, }, // Yukon
    {  320,  454,       0,  305487,       0,  0, 11,  0, }, // Brunei German/Latin/Switzerland
    {  320, 1634,       0,  305498,       0,  0, 18,  0, }, // Solomon
    {  321,    5,       0,  305516,       0,  0, 14,  0, }, // Afghanistan Greek/Greek/Greece
    {  321,   17,       0,  305530,       0,  0, 21,  0, }, // Africa_Central
    {  321,   32,       0,  305551,       0,  0, 22,  0, }, // Africa_Eastern
    {  321,   65,       0,  305573,       0,  0, 28,  0, }, // Africa_Southern
    {  321,   81,  305601,  305620,  305649, 19, 29, 26, }, // Africa_Western
    {  321,  107,  305675,  305686,  305707, 11, 21, 18, }, // Alaska
    {  321,  135,  305725,  305738,  305761, 13, 23, 20, }, // Amazon
    {  321,  142,  305781,  305810,  305849, 29, 39, 36, }, // America_Central
    {  321,  158,  305885,  305915,  305955, 30, 40, 37, }, // America_Eastern
    {  321,  174,  305992,  306019,  306056, 27, 37, 34, }, // America_Mountain
    {  321,  191,  306090,  306103,  306126, 13, 23, 20, }, // America_Pacific
    {  321,  207,  306146,  306157,  306178, 11, 21, 18, }, // Anadyr
    {  321,  214,  306196,  306204,  306222,  8, 18, 15, }, // Apia
    {  321,  232,  306237,  306248,  306269, 11, 21, 18, }, // Arabian
    {  321,  240,  306287,  306301,  306325, 14, 24, 21, }, // Argentina
    {  321,  250,  306346,  306368,  306400, 22, 32, 29, }, // Argentina_Western
    {  321,  268,  306429,  306441,  306463, 12, 22, 19, }, // Armenia
    {  321,  286,  306482,  306496,  306520, 14, 24, 21, }, // Atlantic
    {  321,  295,  306541,  306565,  306599, 24, 34, 31, }, // Australia_Central
    {  321,  313,  306630,  306658,  306696, 28, 38, 35, }, // Australia_CentralWestern
    {  321,  338,  306731,  306756,  306791, 25, 35, 32, }, // Australia_Eastern
    {  321,  356,  306823,  306845,  306877, 22, 32, 29, }, // Australia_Western
    {  321,  374,  306906,  306922,  306948, 16, 26, 23, }, // Azerbaijan
    {  321,  385,  306971,  306981,  307001, 10, 20, 17, }, // Azores
    {  321,  397,  307018,  307034,  307060, 16, 26, 23, }, // Bangladesh
    {  321,  415,       0,  307083,       0,  0, 11,  0, }, // Bhutan
    {  321,  422,       0,  307094,       0,  0, 12,  0, }, // Bolivia
    {  321,  437,  307106,  307120,  307144, 14, 24, 21, }, // Brasilia
    {  321,  454,       0,  307165,       0,  0, 24,  0, }, // Brunei
    {  321,  461,  307189,  307212,  307245, 23, 33, 30, }, // Cape_Verde
    {  321,  478,       0,  307275,       0,  0, 11,  0, }, // Chamorro
    {  321,  487,  307286,  307296,  307316, 10, 20, 17, }, // Chatham
    {  321,  495,  307333,  307342,  307361,  9, 19, 16, }, // Chile
    {  321,  501,  307377,  307386,  307405,  9, 19, 16, }, // China
    {  321,  507,  307421,  307436,  307461, 15, 25, 22, }, // Choibalsan
    {  321,  518,       0,  307483,       0,  0, 23,  0, }, // Christmas
    {  321,  528,       0,  307506,       0,  0, 15,  0, }, // Cocos
    {  321,  534,  307521,  307534,  307557, 13, 23, 20, }, // Colombia
    {  321,  543,  307577,  307591,  307615, 14, 24, 21, }, // Cook
    {  321,  548,  307636,  307646,  307666, 10, 20, 17, }, // Cuba
    {  321,  559,       0,  307683,       0,  0, 11,  0, }, // Davis
    {  321,  575,       0,  307694,       0,  0, 21,  0, }, // DumontDUrville
    {  321,  612,       0,  307715,       0,  0, 20,  0, }, // East_Timor
    {  321,  623,  307735,  307750,  307775, 15, 25, 22, }, // Easter
    {  321,  630,       0,  307797,       0,  0, 14,  0, }, // Ecuador
    {  321,  638,  307811,  307832,  307863, 21, 31, 28, }, // Europe_Central
    {  321,  653,  307891,  307913,  307945, 22, 32, 29, }, // Europe_Eastern
    {  321,  668,       0,  307974,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  321,  691,  308006,  308025,  308054, 19, 29, 26, }, // Europe_Western
    {  321,  706,  308080,  308097,  308124, 17, 27, 24, }, // Falkland
    {  321,  715,  308148,  308157,  308176,  9, 19, 16, }, // Fiji
    {  321,  720,       0,  308192,       0,  0, 21,  0, }, // French_Guiana
    {  321,  734,       0,  308213,       0,  0, 34,  0, }, // French_Southern
    {  321,  757,       0,  308247,       0,  0, 15,  0, }, // Galapagos
    {  321,  767,       0,  308262,       0,  0, 12,  0, }, // Gambier
    {  321,  775,  308274,  308286,  308308, 12, 22, 19, }, // Georgia
    {  321,  783,       0,  308327,       0,  0, 19,  0, }, // Gilbert_Islands
    {  321,  799,       0,  308346,       0,  0, 19,  0, }, // GMT
    {  321,  841,  308365,  308391,  308427, 26, 36, 33, }, // Greenland_Eastern
    {  321,  859,  308460,  308483,  308516, 23, 33, 30, }, // Greenland_Western
    {  321,  877,       0,  308546,       0,  0, 10,  0, }, // Guam
    {  321,  882,       0,  308556,       0,  0, 10,  0, }, // Gulf
    {  321,  887,       0,  308566,       0,  0, 12,  0, }, // Guyana
    {  321,  894,  308578,  308604,  308640, 26, 36, 33, }, // Hawaii_Aleutian
    {  321,  910,  308673,  308688,  308713, 15, 25, 22, }, // Hong_Kong
    {  321,  920,  308735,  308744,  308763,  9, 19, 16, }, // Hovd
    {  321,  925,       0,  308779,       0,  0, 10,  0, }, // India
    {  321,  931,       0,  308789,       0,  0, 19,  0, }, // Indian_Ocean
    {  321,  944,       0,  308808,       0,  0, 13,  0, }, // Indochina
    {  321,  954,       0,  308821,       0,  0, 24,  0, }, // Indonesia_Central
    {  321,  972,       0,  308845,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  321,  990,       0,  308870,       0,  0, 22,  0, }, // Indonesia_Western
    {  321, 1008,  308892,  308900,  308918,  8, 18, 15, }, // Iran
    {  321, 1019,  308933,  308945,  308967, 12, 22, 19, }, // Irkutsk
    {  321, 1027,  308986,  308996,  309016, 10, 20, 17, }, // Israel
    {  321, 1034,  309033,  309045,  309067, 12, 22, 19, }, // Japan
    {  321, 1040,  309086,  309099,  309137, 13, 38, 35, }, // Kamchatka
    {  321, 1058,       0,  309172,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  321, 1077,       0,  309196,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  321, 1106,  309217,  309227,  309247, 10, 20, 17, }, // Korea
    {  321, 1112,       0,  309264,       0,  0, 10,  0, }, // Kosrae
    {  321, 1119,  309274,  309290,  309316, 16, 26, 23, }, // Krasnoyarsk
    {  321, 1151,       0,  309339,       0,  0, 13,  0, }, // Kyrgystan
    {  321, 1175,       0,  309352,       0,  0, 14,  0, }, // Line_Islands
    {  321, 1188,  309366,  309380,  309404, 14, 24, 21, }, // Lord_Howe
    {  321, 1198,  309425,  309434,  309453,  9, 19, 16, }, // Macau
    {  321, 1204,  309469,  309482,  309505, 13, 23, 20, }, // Magadan
    {  321, 1219,       0,  309525,       0,  0, 13,  0, }, // Malaysia
    {  321, 1228,       0,  309538,       0,  0, 12,  0, }, // Maldives
    {  321, 1237,       0,  309550,       0,  0, 12,  0, }, // Marquesas
    {  321, 1247,       0,  309562,       0,  0, 16,  0, }, // Marshall_Islands
    {  321, 1264,  309578,  309591,  309614, 13, 23, 20, }, // Mauritius
    {  321, 1274,       0,  309634,       0,  0,  9,  0, }, // Mawson
    {  321, 1281,  309643,  309664,  309695, 21, 31, 28, }, // Mexico_Pacific
    {  321, 1296,  309723,  309739,  309765, 16, 26, 23, }, // Mongolia
    {  321, 1305,  309788,  309798,  309818, 10, 20, 17, }, // Moscow
    {  321, 1312,       0,  309835,       0,  0, 11,  0, }, // Myanmar
    {  321, 1320,       0,  309846,       0,  0, 11,  0, }, // Nauru
    {  321, 1326,       0,  309857,       0,  0,  9,  0, }, // Nepal
    {  321, 1332,  309866,  309885,  309914, 19, 29, 26, }, // New_Caledonia
    {  321, 1346,  309940,  309958,  309986, 18, 28, 25, }, // New_Zealand
    {  321, 1358,  310011,  310023,  310045, 12, 22, 19, }, // Newfoundland
    {  321, 1371,       0,  310064,       0,  0,  9,  0, }, // Niue
    {  321, 1376,  310073,  310090,  310117, 17, 27, 24, }, // Norfolk
    {  321, 1384,  310141,  310166,  310201, 25, 35, 32, }, // Noronha
    {  321, 1392,       0,  310233,       0,  0, 27,  0, }, // North_Mariana
    {  321, 1406,  310260,  310276,  310302, 16, 26, 23, }, // Novosibirsk
    {  321, 1418,  310325,  310333,  310351,  8, 18, 15, }, // Omsk
    {  321, 1428,  310366,  310378,  310400, 12, 22, 19, }, // Pakistan
    {  321, 1437,       0,  310419,       0,  0, 10,  0, }, // Palau
    {  321, 1443,       0,  310429,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  321, 1460,  310454,  310468,  310492, 14, 24, 21, }, // Paraguay
    {  321, 1469,  310513,  310522,  310541,  9, 19, 16, }, // Peru
    {  321, 1474,  310557,  310571,  310595, 14, 24, 21, }, // Philippines
    {  321, 1486,       0,  310616,       0,  0, 16,  0, }, // Phoenix_Islands
    {  321, 1502,  310632,  310656,  310690, 24, 34, 31, }, // Pierre_Miquelon
    {  321, 1518,       0,  310721,       0,  0, 11,  0, }, // Pitcairn
    {  321, 1527,       0,  310732,       0,  0, 10,  0, }, // Ponape
    {  321, 1534,       0,  310742,       0,  0, 14,  0, }, // Pyongyang
    {  321, 1554,       0,  310756,       0,  0, 11,  0, }, // Reunion
    {  321, 1562,       0,  310767,       0,  0, 10,  0, }, // Rothera
    {  321, 1570,  310777,  310790,  310813, 13, 23, 20, }, // Sakhalin
    {  321, 1579,  310833,  310844,  310865, 11, 21, 18, }, // Samara
    {  321, 1596,  310883,  310892,  310911,  9, 19, 16, }, // Samoa
    {  321, 1602,       0,  310927,       0,  0, 13,  0, }, // Seychelles
    {  321, 1624,       0,  310940,       0,  0, 15,  0, }, // Singapore
    {  321, 1634,       0,  310955,       0,  0, 20,  0, }, // Solomon
    {  321, 1642,       0,  310975,       0,  0, 19,  0, }, // South_Georgia
    {  321, 1656,       0,  310994,       0,  0, 12,  0, }, // Suriname
    {  321, 1676,       0,  311006,       0,  0,  9,  0, }, // Syowa
    {  321, 1682,       0,  311015,       0,  0, 10,  0, }, // Tahiti
    {  321, 1689,  311025,  311035,  311055, 10, 20, 17, }, // Taipei
    {  321, 1696,       0,  311072,       0,  0, 15,  0, }, // Tajikistan
    {  321, 1724,       0,  311087,       0,  0, 12,  0, }, // Tokelau
    {  321, 1732,  311099,  311109,  311129, 10, 20, 17, }, // Tonga
    {  321, 1738,       0,  311146,       0,  0,  9,  0, }, // Truk
    {  321, 1750,  311155,  311172,  311199, 17, 27, 24, }, // Turkmenistan
    {  321, 1763,       0,  311223,       0,  0, 12,  0, }, // Tuvalu
    {  321, 1777,  311235,  311250,  311275, 15, 25, 22, }, // Uruguay
    {  321, 1792,  311297,  311313,  311339, 16, 26, 23, }, // Uzbekistan
    {  321, 1803,  311362,  311375,  311398, 13, 23, 20, }, // Vanuatu
    {  321, 1811,       0,  311418,       0,  0, 15,  0, }, // Venezuela
    {  321, 1821,  311433,  311448,  311473, 15, 25, 22, }, // Vladivostok
    {  321, 1833,  311495,  311511,  311537, 16, 26, 23, }, // Volgograd
    {  321, 1843,       0,  311560,       0,  0, 10,  0, }, // Vostok
    {  321, 1850,       0,  311570,       0,  0, 16,  0, }, // Wake
    {  321, 1855,       0,  311586,       0,  0, 23,  0, }, // Wallis
    {  321, 1862,  311609,  311622,  311645, 13, 23, 20, }, // Yakutsk
    {  321, 1870,  311665,  311685,  311715, 20, 30, 27, }, // Yekaterinburg
    {  321, 1892,       0,  311742,       0,  0, 11,  0, }, // Yukon
    {  323,  422,       0,  311753,       0,  0, 11,  0, }, // Bolivia Guarani/Latin/Paraguay
    {  323,  630,       0,  311764,       0,  0, 11,  0, }, // Ecuador
    {  323,  757,       0,  311775,       0,  0, 13,  0, }, // Galapagos
    {  323, 1811,       0,  311788,       0,  0, 13,  0, }, // Venezuela
    {  324,    0,  311801,  311808,  311825,  7, 17, 15, }, // Acre Gujarati/Gujarati/India
    {  324,    5,       0,  311840,       0,  0, 15,  0, }, // Afghanistan
    {  324,   17,       0,  311855,       0,  0, 16,  0, }, // Africa_Central
    {  324,   32,       0,  311871,       0,  0, 17,  0, }, // Africa_Eastern
    {  324,   65,       0,  311888,       0,  0, 23,  0, }, // Africa_Southern
    {  324,   81,  311911,  311929,  311952, 18, 23, 26, }, // Africa_Western
    {  324,  107,  311978,  311989,  312005, 11, 16, 16, }, // Alaska
    {  324,  128,  312021,  312032,  312053, 11, 21, 19, }, // Almaty
    {  324,  135,  312072,  312082,  312097, 10, 15, 18, }, // Amazon
    {  324,  142,  312115,  312142,  312174, 27, 32, 32, }, // America_Central
    {  324,  158,  312206,  312230,  312259, 24, 29, 29, }, // America_Eastern
    {  324,  174,  312288,  312313,  312343, 25, 30, 30, }, // America_Mountain
    {  324,  191,  312373,  312398,  312428, 25, 30, 30, }, // America_Pacific
    {  324,  207,  312458,  312468,  312488, 10, 20, 18, }, // Anadyr
    {  324,  214,  312506,  312515,  312529,  9, 14, 14, }, // Apia
    {  324,  219,  312543,  312553,  312573, 10, 20, 18, }, // Aqtau
    {  324,  225,  312591,  312601,  312621, 10, 20, 18, }, // Aqtobe
    {  324,  232,  312639,  312650,  312666, 11, 16, 16, }, // Arabian
    {  324,  240,  312682,  312697,  312717, 15, 20, 23, }, // Argentina
    {  324,  250,  312740,  312763,  312791, 23, 28, 31, }, // Argentina_Western
    {  324,  268,  312822,  312835,  312853, 13, 18, 21, }, // Armenia
    {  324,  286,  312874,  312887,  312905, 13, 18, 18, }, // Atlantic
    {  324,  295,  312923,  312948,  312978, 25, 30, 30, }, // Australia_Central
    {  324,  313,  313008,  313041,  313079, 33, 38, 38, }, // Australia_CentralWestern
    {  324,  338,  313117,  313140,  313168, 23, 28, 28, }, // Australia_Eastern
    {  324,  356,  313196,  313219,  313247, 23, 28, 28, }, // Australia_Western
    {  324,  374,  313275,  313287,  313304, 12, 17, 20, }, // Azerbaijan
    {  324,  385,  313324,  313334,  313349, 10, 15, 18, }, // Azores
    {  324,  397,  313367,  313381,  313400, 14, 19, 22, }, // Bangladesh
    {  324,  415,       0,  313422,       0,  0,  9,  0, }, // Bhutan
    {  324,  422,       0,  313431,       0,  0, 12,  0, }, // Bolivia
    {  324,  437,  313443,  313457,  313476, 14, 19, 22, }, // Brasilia
    {  324,  454,       0,  313498,       0,  0, 19,  0, }, // Brunei
    {  324,  461,  313517,  313530,  313548, 13, 18, 21, }, // Cape_Verde
    {  324,  478,       0,  313569,       0,  0, 15,  0, }, // Chamorro
    {  324,  487,  313584,  313594,  313609, 10, 15, 15, }, // Chatham
    {  324,  495,  313624,  313632,  313645,  8, 13, 16, }, // Chile
    {  324,  501,  313661,  313668,  313680,  7, 12, 12, }, // China
    {  324,  507,  313692,  313705,  313723, 13, 18, 21, }, // Choibalsan
    {  324,  518,       0,  313744,       0,  0, 19,  0, }, // Christmas
    {  324,  528,       0,  313763,       0,  0, 19,  0, }, // Cocos
    {  324,  534,  313782,  313795,  313813, 13, 18, 21, }, // Colombia
    {  324,  543,  313834,  313851,  313873, 17, 22, 30, }, // Cook
    {  324,  548,  313903,  313913,  313928, 10, 15, 15, }, // Cuba
    {  324,  559,       0,  313943,       0,  0,  9,  0, }, // Davis
    {  324,  575,       0,  313952,       0,  0, 23,  0, }, // DumontDUrville
    {  324,  612,       0,  313975,       0,  0, 15,  0, }, // East_Timor
    {  324,  623,  313990,  314007,  314029, 17, 22, 25, }, // Easter
    {  324,  630,       0,  314054,       0,  0, 12,  0, }, // Ecuador
    {  324,  638,  314066,  314083,  314105, 17, 22, 25, }, // Europe_Central
    {  324,  653,  314130,  314149,  314173, 19, 24, 27, }, // Europe_Eastern
    {  324,  668,       0,  314200,       0,  0, 25,  0, }, // Europe_Further_Eastern
    {  324,  691,  314225,  314245,  314270, 20, 25, 28, }, // Europe_Western
    {  324,  706,  314298,  314320,  314347, 22, 27, 30, }, // Falkland
    {  324,  715,  314377,  314385,  314398,  8, 13, 16, }, // Fiji
    {  324,  720,       0,  314414,       0,  0, 16,  0, }, // French_Guiana
    {  324,  734,       0,  314430,       0,  0, 32,  0, }, // French_Southern
    {  324,  757,       0,  314462,       0,  0, 13,  0, }, // Galapagos
    {  324,  767,       0,  314475,       0,  0, 12,  0, }, // Gambier
    {  324,  775,  314487,  314501,  314520, 14, 19, 22, }, // Georgia
    {  324,  783,       0,  314542,       0,  0, 21,  0, }, // Gilbert_Islands
    {  324,  799,       0,  314563,       0,  0, 18,  0, }, // GMT
    {  324,  841,  314581,  314601,  314626, 20, 25, 28, }, // Greenland_Eastern
    {  324,  859,  314654,  314675,  314701, 21, 26, 29, }, // Greenland_Western
    {  324,  877,       0,  314730,       0,  0, 19,  0, }, // Guam
    {  324,  882,       0,  314749,       0,  0, 13,  0, }, // Gulf
    {  324,  887,       0,  314421,       0,  0,  9,  0, }, // Guyana
    {  324,  894,  314762,  314780,  314803, 18, 23, 23, }, // Hawaii_Aleutian
    {  324,  910,  314826,  314839,  314857, 13, 18, 21, }, // Hong_Kong
    {  324,  920,  314878,  314887,  314901,  9, 14, 17, }, // Hovd
    {  324,  925,       0,  314918,       0,  0, 15,  0, }, // India
    {  324,  931,       0,  314933,       0,  0, 18,  0, }, // Indian_Ocean
    {  324,  944,       0,  314951,       0,  0, 14,  0, }, // Indochina
    {  324,  954,       0,  314965,       0,  0, 20,  0, }, // Indonesia_Central
    {  324,  972,       0,  314985,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  324,  990,       0,  315008,       0,  0, 23,  0, }, // Indonesia_Western
    {  324, 1008,  315031,  315039,  315052,  8, 13, 13, }, // Iran
    {  324, 1019,  315065,  315079,  315098, 14, 19, 22, }, // Irkutsk
    {  324, 1027,  315120,  315130,  315145, 10, 15, 15, }, // Israel
    {  324, 1034,  315160,  315169,  315183,  9, 14, 14, }, // Japan
    {  324, 1040,  315197,  315227,  315267, 30, 40, 38, }, // Kamchatka
    {  324, 1058,       0,  315305,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  324, 1077,       0,  315325,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  324, 1106,  315346,  315356,  315371, 10, 15, 15, }, // Korea
    {  324, 1112,       0,  315386,       0,  0,  9,  0, }, // Kosrae
    {  324, 1119,  315395,  315414,  315438, 19, 24, 27, }, // Krasnoyarsk
    {  324, 1151,       0,  315465,       0,  0, 17,  0, }, // Kyrgystan
    {  324, 1161,       0,  315482,       0,  0,  8,  0, }, // Lanka
    {  324, 1175,       0,  315490,       0,  0, 18,  0, }, // Line_Islands
    {  324, 1188,  315508,  315521,  315539, 13, 18, 18, }, // Lord_Howe
    {  324, 1198,  315557,  315565,  315583,  8, 18, 16, }, // Macau
    {  324, 1204,  315599,  315608,  315622,  9, 14, 17, }, // Magadan
    {  324, 1219,       0,  315639,       0,  0, 11,  0, }, // Malaysia
    {  324, 1228,       0,  315650,       0,  0, 10,  0, }, // Maldives
    {  324, 1237,       0,  315660,       0,  0, 14,  0, }, // Marquesas
    {  324, 1247,       0,  315674,       0,  0, 20,  0, }, // Marshall_Islands
    {  324, 1264,  315694,  315706,  315723, 12, 17, 20, }, // Mauritius
    {  324, 1274,       0,  315743,       0,  0,  8,  0, }, // Mawson
    {  324, 1281,  315751,  315771,  315796, 20, 25, 25, }, // Mexico_Pacific
    {  324, 1296,  315821,  315835,  315854, 14, 19, 21, }, // Mongolia
    {  324, 1305,  315875,  315885,  315900, 10, 15, 18, }, // Moscow
    {  324, 1312,       0,  315918,       0,  0, 12,  0, }, // Myanmar
    {  324, 1320,       0,  315930,       0,  0,  8,  0, }, // Nauru
    {  324, 1326,       0,  315938,       0,  0,  9,  0, }, // Nepal
    {  324, 1332,  315947,  315966,  315990, 19, 24, 27, }, // New_Caledonia
    {  324, 1346,  316017,  316032,  316052, 15, 20, 20, }, // New_Zealand
    {  324, 1358,  316072,  316091,  316115, 19, 24, 24, }, // Newfoundland
    {  324, 1371,       0,  316139,       0,  0,  8,  0, }, // Niue
    {  324, 1376,  316147,  316165,  316188, 18, 23, 23, }, // Norfolk
    {  324, 1384,  316211,  316236,  316266, 25, 30, 31, }, // Noronha
    {  324, 1392,       0,  316297,       0,  0, 24,  0, }, // North_Mariana
    {  324, 1406,  316321,  316337,  316358, 16, 21, 24, }, // Novosibirsk
    {  324, 1418,  316382,  316392,  316407, 10, 15, 18, }, // Omsk
    {  324, 1428,  316425,  316438,  316456, 13, 18, 21, }, // Pakistan
    {  324, 1437,       0,  316477,       0,  0,  8,  0, }, // Palau
    {  324, 1443,       0,  316485,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  324, 1460,  316504,  316516,  316533, 12, 17, 20, }, // Paraguay
    {  324, 1469,  316553,  316561,  316574,  8, 13, 16, }, // Peru
    {  324, 1474,  316590,  316602,  316619, 12, 17, 20, }, // Philippines
    {  324, 1486,       0,  316639,       0,  0, 21,  0, }, // Phoenix_Islands
    {  324, 1502,  316660,  316688,  316721, 28, 33, 33, }, // Pierre_Miquelon
    {  324, 1518,       0,  316754,       0,  0, 13,  0, }, // Pitcairn
    {  324, 1527,       0,  316767,       0,  0,  9,  0, }, // Ponape
    {  324, 1534,       0,  316776,       0,  0, 14,  0, }, // Pyongyang
    {  324, 1544,  316790,  316804,  316828, 14, 24, 22, }, // Qyzylorda
    {  324, 1554,       0,  316850,       0,  0, 12,  0, }, // Reunion
    {  324, 1562,       0,  316862,       0,  0, 10,  0, }, // Rothera
    {  324, 1570,  316872,  316882,  316897, 10, 15, 18, }, // Sakhalin
    {  324, 1579,  316915,  316924,  316943,  9, 19, 17, }, // Samara
    {  324, 1596,  316960,  316968,  316981,  8, 13, 16, }, // Samoa
    {  324, 1602,       0,  316997,       0,  0, 10,  0, }, // Seychelles
    {  324, 1624,       0,  317007,       0,  0, 17,  0, }, // Singapore
    {  324, 1634,       0,  317024,       0,  0, 20,  0, }, // Solomon
    {  324, 1642,       0,  317044,       0,  0, 21,  0, }, // South_Georgia
    {  324, 1656,       0,  317065,       0,  0, 11,  0, }, // Suriname
    {  324, 1676,       0,  317076,       0,  0, 10,  0, }, // Syowa
    {  324, 1682,       0,  317086,       0,  0, 10,  0, }, // Tahiti
    {  324, 1689,  317096,  317106,  317121, 10, 15, 15, }, // Taipei
    {  324, 1696,       0,  317136,       0,  0, 15,  0, }, // Tajikistan
    {  324, 1724,       0,  317151,       0,  0, 10,  0, }, // Tokelau
    {  324, 1732,  317161,  317170,  317184,  9, 14, 17, }, // Tonga
    {  324, 1738,       0,  317201,       0,  0,  8,  0, }, // Truk
    {  324, 1750,  317209,  317227,  317250, 18, 23, 26, }, // Turkmenistan
    {  324, 1763,       0,  317276,       0,  0,  9,  0, }, // Tuvalu
    {  324, 1777,  317285,  317296,  317312, 11, 16, 19, }, // Uruguay
    {  324, 1792,  317331,  317347,  317368, 16, 21, 24, }, // Uzbekistan
    {  324, 1803,  317392,  317401,  317415,  9, 14, 17, }, // Vanuatu
    {  324, 1811,       0,  317432,       0,  0, 13,  0, }, // Venezuela
    {  324, 1821,  317445,  317462,  317484, 17, 22, 25, }, // Vladivostok
    {  324, 1833,  317509,  317524,  317544, 15, 20, 23, }, // Volgograd
    {  324, 1843,       0,  317451,       0,  0, 11,  0, }, // Vostok
    {  324, 1850,       0,  317567,       0,  0, 15,  0, }, // Wake
    {  324, 1855,       0,  317582,       0,  0, 22,  0, }, // Wallis
    {  324, 1862,  317604,  317617,  317635, 13, 18, 21, }, // Yakutsk
    {  324, 1870,  317656,  317673,  317695, 17, 22, 25, }, // Yekaterinburg
    {  324, 1892,       0,  317720,       0,  0,  9,  0, }, // Yukon
    {  326,    5,       0,  317729,       0,  0, 19,  0, }, // Afghanistan Hausa/Latin/Nigeria
    {  326,   17,       0,  317748,       0,  0, 25,  0, }, // Africa_Central
    {  326,   32,       0,  317773,       0,  0, 23,  0, }, // Africa_Eastern
    {  326,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  326,   81,  317796,  317819,  317852, 23, 33, 33, }, // Africa_Western
    {  326,  107,  317885,  317899,  317923, 14, 24, 22, }, // Alaska
    {  326,  135,  317945,  317959,  317983, 14, 24, 24, }, // Amazon
    {  326,  142,  318007,  318044,  318093, 37, 49, 45, }, // America_Central
    {  326,  158,  318138,  318174,  318219, 36, 45, 43, }, // America_Eastern
    {  326,  174,  318262,  318295,  318339, 33, 44, 39, }, // America_Mountain
    {  326,  191,  318378,  318402,  318437, 24, 35, 31, }, // America_Pacific
    {  326,  214,  318468,  318480,  318502, 12, 22, 20, }, // Apia
    {  326,  232,  318522,     753,  318537, 15, 21, 23, }, // Arabian
    {  326,  240,  318560,  318577,  318604, 17, 27, 27, }, // Argentina
    {  326,  250,  318631,  318657,  318693, 26, 36, 36, }, // Argentina_Western
    {  326,  268,  318729,  318744,  318769, 15, 25, 25, }, // Armenia
    {  326,  286,  318794,  318839,  318894, 45, 55, 53, }, // Atlantic
    {  326,  295,    1044,  318947,  318985, 22, 38, 36, }, // Australia_Central
    {  326,  313,  319021,  319058,  319105, 37, 47, 45, }, // Australia_CentralWestern
    {  326,  338,  319150,    1263,  319178, 28, 32, 36, }, // Australia_Eastern
    {  326,  356,  319214,  319241,  319278, 27, 37, 35, }, // Australia_Western
    {  326,  374,  319313,  319331,  319359, 18, 28, 28, }, // Azerbaijan
    {  326,  385,  319387,  319401,  319427, 14, 26, 23, }, // Azores
    {  326,  397,  319450,  319468,  319496, 18, 28, 28, }, // Bangladesh
    {  326,  415,       0,    1584,       0,  0, 11,  0, }, // Bhutan
    {  326,  422,       0,  319524,       0,  0, 15,  0, }, // Bolivia
    {  326,  437,  319539,  319556,  319583, 17, 27, 27, }, // Brasilia
    {  326,  454,       0,  319610,       0,  0, 25,  0, }, // Brunei
    {  326,  461,  319635,    1699,  319653, 18, 24, 28, }, // Cape_Verde
    {  326,  478,       0,  319681,       0,  0, 26,  0, }, // Chamorro
    {  326,  487,  319707,  319722,  319747, 15, 25, 23, }, // Chatham
    {  326,  495,  319770,  319783,  319806, 13, 23, 23, }, // Chile
    {  326,  501,  319829,  319840,  319861, 11, 21, 19, }, // China
    {  326,  507,  319880,  319898,  319926, 18, 28, 28, }, // Choibalsan
    {  326,  518,       0,  319954,       0,  0, 24,  0, }, // Christmas
    {  326,  528,       0,  319978,       0,  0, 21,  0, }, // Cocos
    {  326,  534,  319999,  320015,  320041, 16, 26, 26, }, // Colombia
    {  326,  543,  320067,  320087,  320117, 20, 30, 36, }, // Cook
    {  326,  548,  320153,  320167,  320189, 14, 22, 20, }, // Cuba
    {  326,  559,       0,  320209,       0,  0, 13,  0, }, // Davis
    {  326,  575,       0,  320222,       0,  0, 24,  0, }, // DumontDUrville
    {  326,  612,       0,  320246,       0,  0, 18,  0, }, // East_Timor
    {  326,  623,  320264,  320285,  320316, 21, 31, 31, }, // Easter
    {  326,  630,       0,  320347,       0,  0, 15,  0, }, // Ecuador
    {  326,  638,  320362,  320385,  320412, 23, 27, 30, }, // Europe_Central
    {  326,  653,  320442,  320462,  320486, 20, 24, 30, }, // Europe_Eastern
    {  326,  668,       0,  320516,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  326,  691,  320547,  320571,  320599, 24, 28, 38, }, // Europe_Western
    {  326,  706,  320637,  320661,  320695, 24, 34, 34, }, // Falkland
    {  326,  715,  320729,  320741,  320763, 12, 22, 22, }, // Fiji
    {  326,  720,       0,  320785,       0,  0, 21,  0, }, // French_Guiana
    {  326,  734,       0,  320806,       0,  0, 37,  0, }, // French_Southern
    {  326,  757,       0,  320843,       0,  0, 17,  0, }, // Galapagos
    {  326,  767,       0,  320860,       0,  0, 15,  0, }, // Gambier
    {  326,  775,  320875,  320890,    2822, 15, 25, 19, }, // Georgia
    {  326,  783,       0,  320915,       0,  0, 23,  0, }, // Gilbert_Islands
    {  326,  799,       0,  320938,       0,  0, 27,  0, }, // GMT
    {  326,  841,  320965,  320991,  321027, 26, 36, 34, }, // Greenland_Eastern
    {  326,  859,  321061,  321087,  321123, 26, 36, 34, }, // Greenland_Western
    {  326,  882,       0,  321157,       0,  0, 12,  0, }, // Gulf
    {  326,  887,       0,  321169,       0,  0, 14,  0, }, // Guyana
    {  326,  894,  321183,  321208,  321241, 25, 33, 31, }, // Hawaii_Aleutian
    {  326,  910,  321272,  321289,  321316, 17, 27, 27, }, // Hong_Kong
    {  326,  920,  321343,  321355,  321377, 12, 22, 22, }, // Hovd
    {  326,  925,       0,    3310,       0,  0, 19,  0, }, // India
    {  326,  931,       0,  321399,       0,  0, 20,  0, }, // Indian_Ocean
    {  326,  944,       0,  321419,       0,  0, 17,  0, }, // Indochina
    {  326,  954,       0,  321436,       0,  0, 28,  0, }, // Indonesia_Central
    {  326,  972,       0,    3382,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  326,  990,       0,  321464,       0,  0, 26,  0, }, // Indonesia_Western
    {  326, 1008,  321490,  321502,  321524, 12, 22, 20, }, // Iran
    {  326, 1019,  321544,  321559,  321584, 15, 25, 25, }, // Irkutsk
    {  326, 1027,  321609,    3534,    3554, 16, 20, 20, }, // Israel
    {  326, 1034,  321625,    3584,    3603, 13, 19, 19, }, // Japan
    {  326, 1058,       0,  321638,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  326, 1077,       0,  321665,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  326, 1106,  321692,  321706,  321730, 14, 24, 22, }, // Korea
    {  326, 1112,       0,  321752,       0,  0, 14,  0, }, // Kosrae
    {  326, 1119,  321766,  321785,  321814, 19, 29, 29, }, // Krasnoyarsk
    {  326, 1151,       0,  321843,       0,  0, 18,  0, }, // Kyrgystan
    {  326, 1175,       0,  321861,       0,  0, 20,  0, }, // Line_Islands
    {  326, 1188,  321881,  321898,  321925, 17, 27, 30, }, // Lord_Howe
    {  326, 1204,  321955,  321970,  321995, 15, 25, 25, }, // Magadan
    {  326, 1219,       0,  322020,       0,  0, 16,  0, }, // Malaysia
    {  326, 1228,       0,  322036,       0,  0, 16,  0, }, // Maldives
    {  326, 1237,       0,  322052,       0,  0, 17,  0, }, // Marquesas
    {  326, 1247,       0,  322069,       0,  0, 24,  0, }, // Marshall_Islands
    {  326, 1264,  322093,  322110,  322137, 17, 27, 27, }, // Mauritius
    {  326, 1274,       0,  322164,       0,  0, 14,  0, }, // Mawson
    {  326, 1281,  322178,  322201,  322234, 23, 33, 31, }, // Mexico_Pacific
    {  326, 1296,  322265,  322284,  322313, 19, 29, 29, }, // Mongolia
    {  326, 1305,  322342,  322356,  322380, 14, 24, 24, }, // Moscow
    {  326, 1312,       0,  322404,       0,  0, 15,  0, }, // Myanmar
    {  326, 1320,       0,  322419,       0,  0, 13,  0, }, // Nauru
    {  326, 1326,       0,  322432,       0,  0, 13,  0, }, // Nepal
    {  326, 1332,  322445,  322466,  322497, 21, 31, 31, }, // New_Caledonia
    {  326, 1346,  322528,  322547,  322576, 19, 29, 27, }, // New_Zealand
    {  326, 1358,  322603,  322623,  322654, 20, 31, 27, }, // Newfoundland
    {  326, 1371,       0,  322681,       0,  0, 12,  0, }, // Niue
    {  326, 1376,  322693,  322715,  322747, 22, 32, 30, }, // Norfolk
    {  326, 1384,  322777,  322804,  322841, 27, 37, 37, }, // Noronha
    {  326, 1406,  322878,    4863,  322897, 19, 25, 29, }, // Novosibirsk
    {  326, 1418,  322926,  322938,  322960, 12, 22, 22, }, // Omsk
    {  326, 1428,  322982,  322998,  323024, 16, 26, 26, }, // Pakistan
    {  326, 1437,       0,  323050,       0,  0, 13,  0, }, // Palau
    {  326, 1443,       0,  323063,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  326, 1460,  323087,  323103,  323129, 16, 26, 26, }, // Paraguay
    {  326, 1469,  323155,  323167,  323189, 12, 22, 22, }, // Peru
    {  326, 1474,  323211,  323229,  323257, 18, 28, 28, }, // Philippines
    {  326, 1486,       0,  323285,       0,  0, 23,  0, }, // Phoenix_Islands
    {  326, 1502,  323308,  323338,  323378, 30, 40, 38, }, // Pierre_Miquelon
    {  326, 1518,       0,  323416,       0,  0, 16,  0, }, // Pitcairn
    {  326, 1527,       0,  323432,       0,  0, 14,  0, }, // Ponape
    {  326, 1534,       0,  323446,       0,  0, 17,  0, }, // Pyongyang
    {  326, 1554,       0,  323463,       0,  0, 15,  0, }, // Reunion
    {  326, 1562,       0,  323478,       0,  0, 15,  0, }, // Rothera
    {  326, 1570,  323493,  323509,  323535, 16, 26, 26, }, // Sakhalin
    {  326, 1596,  323561,  323574,  323597, 13, 23, 26, }, // Samoa
    {  326, 1602,       0,  323623,       0,  0, 18,  0, }, // Seychelles
    {  326, 1624,       0,  323641,       0,  0, 27,  0, }, // Singapore
    {  326, 1634,       0,  323668,       0,  0, 23,  0, }, // Solomon
    {  326, 1642,       0,  323691,       0,  0, 24,  0, }, // South_Georgia
    {  326, 1656,       0,  323715,       0,  0, 16,  0, }, // Suriname
    {  326, 1676,       0,  323731,       0,  0, 13,  0, }, // Syowa
    {  326, 1682,       0,  323744,       0,  0, 14,  0, }, // Tahiti
    {  326, 1689,  323758,  323772,  323796, 14, 24, 22, }, // Taipei
    {  326, 1696,       0,  323818,       0,  0, 18,  0, }, // Tajikistan
    {  326, 1724,       0,    5763,       0,  0, 12,  0, }, // Tokelau
    {  326, 1732,  323836,  323849,  323872, 13, 23, 23, }, // Tonga
    {  326, 1738,       0,  323895,       0,  0, 13,  0, }, // Truk
    {  326, 1750,  323908,  323928,    5874, 20, 30, 24, }, // Turkmenistan
    {  326, 1763,       0,  323958,       0,  0, 14,  0, }, // Tuvalu
    {  326, 1777,  323972,  323987,  324012, 15, 25, 25, }, // Uruguay
    {  326, 1792,  324037,  324055,  324083, 18, 28, 28, }, // Uzbekistan
    {  326, 1803,  324111,  324126,  324151, 15, 25, 25, }, // Vanuatu
    {  326, 1811,       0,  324176,       0,  0, 17,  0, }, // Venezuela
    {  326, 1821,  324193,  324212,  324241, 19, 29, 29, }, // Vladivostok
    {  326, 1833,  324270,  324287,  324314, 17, 27, 27, }, // Volgograd
    {  326, 1843,       0,  324341,       0,  0, 14,  0, }, // Vostok
    {  326, 1850,       0,  324355,       0,  0, 19,  0, }, // Wake
    {  326, 1855,       0,  324374,       0,  0, 24,  0, }, // Wallis
    {  326, 1862,  324398,  324413,  324438, 15, 25, 25, }, // Yakutsk
    {  326, 1870,  324463,  324484,  324515, 21, 31, 31, }, // Yekaterinburg
    {  326, 1892,       0,  324546,       0,  0, 13,  0, }, // Yukon
    {  332,    5,       0,  324559,       0,  0, 13,  0, }, // Afghanistan Hebrew/Hebrew/Israel
    {  332,   17,       0,  324572,       0,  0, 16,  0, }, // Africa_Central
    {  332,   32,       0,  324588,       0,  0, 16,  0, }, // Africa_Eastern
    {  332,   65,       0,  324604,       0,  0, 16,  0, }, // Africa_Southern
    {  332,   81,  324620,  324636,  324659, 16, 23, 22, }, // Africa_Western
    {  332,  107,  324681,  324691,  324708, 10, 17, 16, }, // Alaska
    {  332,  128,  324724,  324734,  324751, 10, 17, 16, }, // Almaty
    {  332,  135,  324767,  324778,  324796, 11, 18, 17, }, // Amazon
    {  332,  142,  324813,  324828,  324850, 15, 22, 21, }, // America_Central
    {  332,  158,  324871,  324887,  324910, 16, 23, 22, }, // America_Eastern
    {  332,  174,  324932,  324954,  324983, 22, 29, 28, }, // America_Mountain
    {  332,  191,  325011,  325026,  325048, 15, 22, 21, }, // America_Pacific
    {  332,  207,  325069,  325079,  325094, 10, 15, 14, }, // Anadyr
    {  332,  214,  325108,  325117,  325133,  9, 16, 15, }, // Apia
    {  332,  219,  325148,  325158,  325175, 10, 17, 16, }, // Aqtau
    {  332,  225,  325191,  325202,  325221, 11, 19, 18, }, // Aqtobe
    {  332,  232,  325239,  325255,  325278, 16, 23, 22, }, // Arabian
    {  332,  240,  325300,  325313,  325333, 13, 20, 19, }, // Argentina
    {  332,  250,  325352,  325370,  325395, 18, 25, 24, }, // Argentina_Western
    {  332,  268,  325419,  325430,  325448, 11, 18, 17, }, // Armenia
    {  332,  286,  325465,  325487,  325516, 22, 29, 28, }, // Atlantic
    {  332,  295,  325544,  325562,  325587, 18, 25, 24, }, // Australia_Central
    {  332,  313,  325611,  325634,  325664, 23, 30, 29, }, // Australia_CentralWestern
    {  332,  338,  325693,  325711,  325736, 18, 25, 24, }, // Australia_Eastern
    {  332,  356,  325760,  325778,  325803, 18, 25, 24, }, // Australia_Western
    {  332,  374,  325827,  325841,  325862, 14, 21, 20, }, // Azerbaijan
    {  332,  385,  325882,  325901,  325927, 19, 26, 25, }, // Azores
    {  332,  397,  325952,  325963,  325981, 11, 18, 17, }, // Bangladesh
    {  332,  415,       0,  325998,       0,  0, 10,  0, }, // Bhutan
    {  332,  422,       0,  326008,       0,  0, 12,  0, }, // Bolivia
    {  332,  437,  326020,  326032,  326051, 12, 19, 18, }, // Brasilia
    {  332,  454,       0,  326069,       0,  0, 20,  0, }, // Brunei
    {  332,  461,  326089,  326101,  326120, 12, 19, 18, }, // Cape_Verde
    {  332,  478,       0,  326138,       0,  0, 12,  0, }, // Chamorro
    {  332,  487,  326150,  326161,  326179, 11, 18, 17, }, // Chatham
    {  332,  495,  326196,  326206,  326223, 10, 17, 16, }, // Chile
    {  332,  501,  326239,  326247,  326262,  8, 15, 14, }, // China
    {  332,  507,  326276,  326289,  326309, 13, 20, 19, }, // Choibalsan
    {  332,  518,       0,  326328,       0,  0, 16,  0, }, // Christmas
    {  332,  528,       0,  326344,       0,  0, 14,  0, }, // Cocos
    {  332,  534,  326358,  326371,  326391, 13, 20, 19, }, // Colombia
    {  332,  543,  326344,  326410,  326429, 12, 19, 25, }, // Cook
    {  332,  548,  326454,  326463,  326479,  9, 16, 15, }, // Cuba
    {  332,  559,       0,  326494,       0,  0, 11,  0, }, // Davis
    {  332,  575,       0,  326505,       0,  0, 20,  0, }, // DumontDUrville
    {  332,  612,       0,  326525,       0,  0, 15,  0, }, // East_Timor
    {  332,  623,  326540,  326553,  326573, 13, 20, 19, }, // Easter
    {  332,  630,       0,  326592,       0,  0, 12,  0, }, // Ecuador
    {  332,  638,  326604,  326620,  326643, 16, 23, 22, }, // Europe_Central
    {  332,  653,  326665,  326681,  326704, 16, 23, 22, }, // Europe_Eastern
    {  332,  668,       0,  326726,       0,  0, 10,  0, }, // Europe_Further_Eastern
    {  332,  691,  326736,  326752,  326775, 16, 23, 22, }, // Europe_Western
    {  332,  706,  326797,  326812,  326834, 15, 22, 21, }, // Falkland
    {  332,  715,  326855,  326865,  326882, 10, 17, 16, }, // Fiji
    {  332,  720,       0,  326898,       0,  0, 18,  0, }, // French_Guiana
    {  332,  734,       0,  326916,       0,  0, 41,  0, }, // French_Southern
    {  332,  757,       0,  326957,       0,  0, 16,  0, }, // Galapagos
    {  332,  767,       0,  326973,       0,  0, 15,  0, }, // Gambier
    {  332,  775,  326988,  327000,  327019, 12, 19, 18, }, // Georgia
    {  332,  783,       0,  327037,       0,  0, 15,  0, }, // Gilbert_Islands
    {  332,  799,       0,  327052,       0,  0, 13,  0, }, // GMT
    {  332,  841,  327065,  327082,  327106, 17, 24, 23, }, // Greenland_Eastern
    {  332,  859,  327129,  327146,  327170, 17, 24, 23, }, // Greenland_Western
    {  332,  877,       0,  327193,       0,  0,  9,  0, }, // Guam
    {  332,  882,       0,  327202,       0,  0, 17,  0, }, // Gulf
    {  332,  887,       0,  326898,       0,  0, 10,  0, }, // Guyana
    {  332,  894,  327219,  327245,  327278, 26, 33, 32, }, // Hawaii_Aleutian
    {  332,  910,  327310,  327324,  327345, 14, 21, 20, }, // Hong_Kong
    {  332,  920,  327365,  327374,  327390,  9, 16, 15, }, // Hovd
    {  332,  925,       0,  327405,       0,  0,  9,  0, }, // India
    {  332,  931,       0,  327414,       0,  0, 20,  0, }, // Indian_Ocean
    {  332,  944,       0,  327434,       0,  0, 13,  0, }, // Indochina
    {  332,  954,       0,  327447,       0,  0, 19,  0, }, // Indonesia_Central
    {  332,  972,       0,  327466,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  332,  990,       0,  327485,       0,  0, 19,  0, }, // Indonesia_Western
    {  332, 1008,  327504,  327514,  327531, 10, 17, 16, }, // Iran
    {  332, 1019,  327547,  327560,  327580, 13, 20, 19, }, // Irkutsk
    {  332, 1027,  327599,  327609,  327626, 10, 17, 16, }, // Israel
    {  332, 1034,  327642,  327650,  327665,  8, 15, 14, }, // Japan
    {  332, 1040,  327679,  327704,  327734, 25, 30, 29, }, // Kamchatka
    {  332, 1058,       0,  327763,       0,  0, 16,  0, }, // Kazakhstan_Eastern
    {  332, 1077,       0,  327779,       0,  0, 16,  0, }, // Kazakhstan_Western
    {  332, 1106,  327795,  327806,  327824, 11, 18, 17, }, // Korea
    {  332, 1112,       0,  327841,       0,  0, 11,  0, }, // Kosrae
    {  332, 1119,  327852,  327867,  327889, 15, 22, 21, }, // Krasnoyarsk
    {  332, 1151,       0,  327910,       0,  0, 14,  0, }, // Kyrgystan
    {  332, 1175,       0,  327924,       0,  0, 13,  0, }, // Line_Islands
    {  332, 1188,  327937,  327954,  327978, 17, 24, 23, }, // Lord_Howe
    {  332, 1198,  328001,  328010,  328024,  9, 14, 13, }, // Macau
    {  332, 1204,  328037,  328046,  328062,  9, 16, 15, }, // Magadan
    {  332, 1219,       0,  328077,       0,  0, 10,  0, }, // Malaysia
    {  332, 1228,       0,  328087,       0,  0, 20,  0, }, // Maldives
    {  332, 1237,       0,  328107,       0,  0, 14,  0, }, // Marquesas
    {  332, 1247,       0,  328121,       0,  0, 13,  0, }, // Marshall_Islands
    {  332, 1264,  328134,  328148,  328169, 14, 21, 20, }, // Mauritius
    {  332, 1274,       0,  328189,       0,  0, 11,  0, }, // Mawson
    {  332, 1281,  328200,  328216,  328239, 16, 23, 22, }, // Mexico_Pacific
    {  332, 1296,  328261,  328277,  328300, 16, 23, 22, }, // Mongolia
    {  332, 1305,  328322,  328333,  328351, 11, 18, 17, }, // Moscow
    {  332, 1312,       0,  328368,       0,  0, 11,  0, }, // Myanmar
    {  332, 1320,       0,  328379,       0,  0, 10,  0, }, // Nauru
    {  332, 1326,       0,  328389,       0,  0,  9,  0, }, // Nepal
    {  332, 1332,  328398,  328416,  328441, 18, 25, 24, }, // New_Caledonia
    {  332, 1346,  328465,  328479,  328500, 14, 21, 20, }, // New_Zealand
    {  332, 1358,  328520,  328536,  328559, 16, 23, 22, }, // Newfoundland
    {  332, 1371,       0,  328581,       0,  0, 10,  0, }, // Niue
    {  332, 1376,  328591,  328606,  328628, 15, 22, 21, }, // Norfolk
    {  332, 1384,  328649,  328671,  328700, 22, 29, 28, }, // Noronha
    {  332, 1392,       0,  328728,       0,  0, 24,  0, }, // North_Mariana
    {  332, 1406,  328752,  328768,  328791, 16, 23, 22, }, // Novosibirsk
    {  332, 1418,  328813,  328823,  328840, 10, 17, 16, }, // Omsk
    {  332, 1428,  328856,  328867,  328885, 11, 18, 17, }, // Pakistan
    {  332, 1437,       0,  328902,       0,  0,  9,  0, }, // Palau
    {  332, 1443,       0,  328911,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  332, 1460,  328934,  328946,  328965, 12, 19, 18, }, // Paraguay
    {  332, 1469,  328983,  328991,  329006,  8, 15, 14, }, // Peru
    {  332, 1474,  329020,  329035,  329057, 15, 22, 21, }, // Philippines
    {  332, 1486,       0,  329078,       0,  0, 15,  0, }, // Phoenix_Islands
    {  332, 1502,  329093,  329114,  329142, 21, 28, 27, }, // Pierre_Miquelon
    {  332, 1518,       0,  329169,       0,  0, 11,  0, }, // Pitcairn
    {  332, 1527,       0,  329180,       0,  0, 11,  0, }, // Ponape
    {  332, 1534,       0,  329191,       0,  0, 14,  0, }, // Pyongyang
    {  332, 1544,  329205,  329219,  329240, 14, 21, 20, }, // Qyzylorda
    {  332, 1554,       0,  329260,       0,  0, 12,  0, }, // Reunion
    {  332, 1562,       0,  329272,       0,  0, 11,  0, }, // Rothera
    {  332, 1570,  329283,  329293,  329310, 10, 17, 16, }, // Sakhalin
    {  332, 1579,  329326,  329335,  329349,  9, 14, 13, }, // Samara
    {  332, 1596,  329362,  329372,  329389, 10, 17, 16, }, // Samoa
    {  332, 1602,       0,  329405,       0,  0, 14,  0, }, // Seychelles
    {  332, 1624,       0,  329419,       0,  0, 12,  0, }, // Singapore
    {  332, 1634,       0,  329431,       0,  0, 13,  0, }, // Solomon
    {  332, 1642,       0,  329444,       0,  0, 18,  0, }, // South_Georgia
    {  332, 1656,       0,  329462,       0,  0, 12,  0, }, // Suriname
    {  332, 1676,       0,  329474,       0,  0, 11,  0, }, // Syowa
    {  332, 1682,       0,  329485,       0,  0, 10,  0, }, // Tahiti
    {  332, 1689,  329495,  329506,  329524, 11, 18, 17, }, // Taipei
    {  332, 1696,       0,  329541,       0,  0, 14,  0, }, // Tajikistan
    {  332, 1724,       0,  329555,       0,  0, 11,  0, }, // Tokelau
    {  332, 1732,  329566,  329576,  329593, 10, 17, 16, }, // Tonga
    {  332, 1738,       0,  329609,       0,  0,  9,  0, }, // Truk
    {  332, 1750,  329618,  329633,  329655, 15, 22, 21, }, // Turkmenistan
    {  332, 1763,       0,  329676,       0,  0, 11,  0, }, // Tuvalu
    {  332, 1777,  329687,  329701,  329722, 14, 21, 20, }, // Uruguay
    {  332, 1792,  329742,  329756,  329777, 14, 21, 20, }, // Uzbekistan
    {  332, 1803,  329797,  329808,  329826, 11, 18, 17, }, // Vanuatu
    {  332, 1811,       0,  329843,       0,  0, 12,  0, }, // Venezuela
    {  332, 1821,  329855,  329870,  329892, 15, 22, 21, }, // Vladivostok
    {  332, 1833,  329913,  329926,  329946, 13, 20, 19, }, // Volgograd
    {  332, 1843,       0,  329965,       0,  0, 11,  0, }, // Vostok
    {  332, 1850,       0,  329976,       0,  0, 13,  0, }, // Wake
    {  332, 1855,       0,  329989,       0,  0, 19,  0, }, // Wallis
    {  332, 1862,  330008,  330019,  330037, 11, 18, 17, }, // Yakutsk
    {  332, 1870,  330054,  330069,  330091, 15, 22, 21, }, // Yekaterinburg
    {  332, 1892,       0,  330112,       0,  0, 10,  0, }, // Yukon
    {  333,    5,       0,  330122,       0,  0, 16,  0, }, // Afghanistan Hindi/Devanagari/India
    {  333,   17,       0,  330138,       0,  0, 17,  0, }, // Africa_Central
    {  333,   32,       0,  330155,       0,  0, 19,  0, }, // Africa_Eastern
    {  333,   65,       0,  330174,       0,  0, 24,  0, }, // Africa_Southern
    {  333,   81,  330198,  330217,  330241, 19, 24, 32, }, // Africa_Western
    {  333,  107,  330273,  330284,  330301, 11, 17, 19, }, // Alaska
    {  333,  135,  330320,  330330,  330345, 10, 15, 23, }, // Amazon
    {  333,  142,  330368,  330395,  330427, 27, 32, 34, }, // America_Central
    {  333,  158,  330461,  330486,  330516, 25, 30, 32, }, // America_Eastern
    {  333,  174,  330548,  330574,  330605, 26, 31, 33, }, // America_Mountain
    {  333,  191,  330638,  330664,  330695, 26, 31, 33, }, // America_Pacific
    {  333,  207,  330728,  330739,  330755, 11, 16, 24, }, // Anadyr
    {  333,  214,  330779,  330787,  330800,  8, 13, 15, }, // Apia
    {  333,  232,  330815,  330822,  330834,  7, 12, 14, }, // Arabian
    {  333,  240,  330848,  330862,  330881, 14, 19, 27, }, // Argentina
    {  333,  250,  330908,  330930,  330957, 22, 27, 35, }, // Argentina_Western
    {  333,  268,  330992,  331005,  331023, 13, 18, 26, }, // Armenia
    {  333,  286,  331049,  331061,  331078, 12, 17, 19, }, // Atlantic
    {  333,  295,  331097,  331118,  331149, 21, 31, 33, }, // Australia_Central
    {  333,  313,  331182,  331216,  331255, 34, 39, 41, }, // Australia_CentralWestern
    {  333,  338,  331296,  331318,  331347, 22, 29, 31, }, // Australia_Eastern
    {  333,  356,  331378,  331401,  331430, 23, 29, 31, }, // Australia_Western
    {  333,  374,  331461,  331474,  331492, 13, 18, 26, }, // Azerbaijan
    {  333,  385,  331518,  331529,  331545, 11, 16, 24, }, // Azores
    {  333,  397,  331569,  331583,  331602, 14, 19, 27, }, // Bangladesh
    {  333,  415,       0,  331629,       0,  0,  9,  0, }, // Bhutan
    {  333,  422,       0,  331638,       0,  0, 12,  0, }, // Bolivia
    {  333,  437,  331650,  331665,  331685, 15, 20, 28, }, // Brasilia
    {  333,  454,       0,  331713,       0,  0, 21,  0, }, // Brunei
    {  333,  461,  331734,  331746,  331763, 12, 17, 25, }, // Cape_Verde
    {  333,  478,       0,  331788,       0,  0, 15,  0, }, // Chamorro
    {  333,  487,  331803,  331811,  331824,  8, 13, 15, }, // Chatham
    {  333,  495,  331839,  331847,  331860,  8, 13, 21, }, // Chile
    {  333,  501,  331881,  331888,  331900,  7, 12, 14, }, // China
    {  333,  507,  331914,  331927,  331945, 13, 18, 26, }, // Choibalsan
    {  333,  518,       0,  331971,       0,  0, 17,  0, }, // Christmas
    {  333,  528,       0,  331988,       0,  0, 19,  0, }, // Cocos
    {  333,  534,  332007,  332019,  332036, 12, 17, 25, }, // Colombia
    {  333,  543,  332061,  332078,  332100, 17, 22, 37, }, // Cook
    {  333,  548,  332137,  332147,  332162, 10, 15, 17, }, // Cuba
    {  333,  559,       0,  332179,       0,  0,  9,  0, }, // Davis
    {  333,  575,       0,  332188,       0,  0, 23,  0, }, // DumontDUrville
    {  333,  612,       0,  332211,       0,  0, 16,  0, }, // East_Timor
    {  333,  623,  332227,  332242,  332262, 15, 20, 28, }, // Easter
    {  333,  630,       0,  332290,       0,  0, 12,  0, }, // Ecuador
    {  333,  638,  332302,  332318,  332339, 16, 21, 31, }, // Europe_Central
    {  333,  653,  332370,  332388,  332411, 18, 23, 31, }, // Europe_Eastern
    {  333,  668,       0,  332442,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  333,  691,  332465,  332484,  332508, 19, 24, 33, }, // Europe_Western
    {  333,  706,  332541,  332563,  332590, 22, 27, 35, }, // Falkland
    {  333,  715,  332625,  332634,  332648,  9, 14, 22, }, // Fiji
    {  333,  720,       0,  332670,       0,  0, 18,  0, }, // French_Guiana
    {  333,  734,       0,  332688,       0,  0, 33,  0, }, // French_Southern
    {  333,  757,       0,  332721,       0,  0, 16,  0, }, // Galapagos
    {  333,  767,       0,  332737,       0,  0, 11,  0, }, // Gambier
    {  333,  775,  332748,  332760,  332777, 12, 17, 25, }, // Georgia
    {  333,  783,       0,  332802,       0,  0, 22,  0, }, // Gilbert_Islands
    {  333,  799,       0,   86447,       0,  0, 17,  0, }, // GMT
    {  333,  841,  332824,  332844,  332869, 20, 25, 33, }, // Greenland_Eastern
    {  333,  859,  332902,  332923,  332949, 21, 26, 34, }, // Greenland_Western
    {  333,  882,       0,  332983,       0,  0, 14,  0, }, // Gulf
    {  333,  887,       0,  332678,       0,  0, 10,  0, }, // Guyana
    {  333,  894,  332997,  333013,  333034, 16, 21, 23, }, // Hawaii_Aleutian
    {  333,  910,  333057,  333070,  333088, 13, 18, 26, }, // Hong_Kong
    {  333,  920,  333114,  333123,  333137,  9, 14, 22, }, // Hovd
    {  333,  925,       0,  333159,       0,  0, 15,  0, }, // India
    {  333,  931,       0,  333174,       0,  0, 16,  0, }, // Indian_Ocean
    {  333,  944,       0,  333190,       0,  0, 13,  0, }, // Indochina
    {  333,  954,       0,  333203,       0,  0, 19,  0, }, // Indonesia_Central
    {  333,  972,       0,  333222,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  333,  990,       0,  333243,       0,  0, 22,  0, }, // Indonesia_Western
    {  333, 1008,  333265,  333273,  333286,  8, 13, 15, }, // Iran
    {  333, 1019,  333301,  333315,  333334, 14, 19, 27, }, // Irkutsk
    {  333, 1027,  333361,  333372,  333388, 11, 16, 18, }, // Israel
    {  333, 1034,  333406,  333415,  333429,  9, 14, 16, }, // Japan
    {  333, 1040,  333445,  333474,  333508, 29, 34, 42, }, // Kamchatka
    {  333, 1058,       0,  333550,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  333, 1077,       0,  333570,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  333, 1106,  333591,  333602,  333618, 11, 16, 18, }, // Korea
    {  333, 1112,       0,  333636,       0,  0, 10,  0, }, // Kosrae
    {  333, 1119,  333646,  333665,  333689, 19, 24, 32, }, // Krasnoyarsk
    {  333, 1151,       0,  333721,       0,  0, 16,  0, }, // Kyrgystan
    {  333, 1175,       0,  333737,       0,  0, 18,  0, }, // Line_Islands
    {  333, 1188,  333755,  333769,  333788, 14, 19, 21, }, // Lord_Howe
    {  333, 1204,  333809,  333820,  333836, 11, 16, 24, }, // Magadan
    {  333, 1219,       0,  333860,       0,  0, 11,  0, }, // Malaysia
    {  333, 1228,       0,  333871,       0,  0, 10,  0, }, // Maldives
    {  333, 1237,       0,  333881,       0,  0, 12,  0, }, // Marquesas
    {  333, 1247,       0,  333893,       0,  0, 20,  0, }, // Marshall_Islands
    {  333, 1264,  333913,  333923,  333938, 10, 15, 23, }, // Mauritius
    {  333, 1274,       0,  333961,       0,  0, 10,  0, }, // Mawson
    {  333, 1281,  333971,  333991,  334016, 20, 25, 27, }, // Mexico_Pacific
    {  333, 1296,  334043,  334056,  334074, 13, 18, 26, }, // Mongolia
    {  333, 1305,  334100,  334110,  334125, 10, 15, 23, }, // Moscow
    {  333, 1312,       0,  334148,       0,  0, 12,  0, }, // Myanmar
    {  333, 1320,       0,  334160,       0,  0,  8,  0, }, // Nauru
    {  333, 1326,       0,  334168,       0,  0,  9,  0, }, // Nepal
    {  333, 1332,  334177,  334196,  334220, 19, 24, 32, }, // New_Caledonia
    {  333, 1346,  334252,  334267,  334287, 15, 20, 22, }, // New_Zealand
    {  333, 1358,  334309,  334327,  334350, 18, 23, 25, }, // Newfoundland
    {  333, 1371,       0,  334375,       0,  0,  8,  0, }, // Niue
    {  333, 1376,  334383,  334400,  334422, 17, 22, 24, }, // Norfolk
    {  333, 1384,  334446,  334473,  334505, 27, 32, 40, }, // Noronha
    {  333, 1406,  334545,  334562,  334584, 17, 22, 30, }, // Novosibirsk
    {  333, 1418,  334614,  334624,  334639, 10, 15, 23, }, // Omsk
    {  333, 1428,  334662,  334675,  334693, 13, 18, 26, }, // Pakistan
    {  333, 1437,       0,  334719,       0,  0,  8,  0, }, // Palau
    {  333, 1443,       0,  334727,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  333, 1460,  334746,  334758,  334775, 12, 17, 25, }, // Paraguay
    {  333, 1469,  334800,  334808,  334821,  8, 13, 21, }, // Peru
    {  333, 1474,  334842,  334854,  334871, 12, 17, 25, }, // Philippines
    {  333, 1486,       0,  334896,       0,  0, 22,  0, }, // Phoenix_Islands
    {  333, 1502,  334918,  334945,  334977, 27, 32, 34, }, // Pierre_Miquelon
    {  333, 1518,       0,  335011,       0,  0, 12,  0, }, // Pitcairn
    {  333, 1527,       0,  335023,       0,  0, 10,  0, }, // Ponape
    {  333, 1534,       0,  335033,       0,  0, 14,  0, }, // Pyongyang
    {  333, 1554,       0,  335047,       0,  0, 12,  0, }, // Reunion
    {  333, 1562,       0,  335059,       0,  0, 10,  0, }, // Rothera
    {  333, 1570,  335069,  335079,  335094, 10, 15, 23, }, // Sakhalin
    {  333, 1579,  335117,  335126,  335140,  9, 14, 22, }, // Samara
    {  333, 1596,  335162,  335170,  335183,  8, 13, 15, }, // Samoa
    {  333, 1602,       0,  335198,       0,  0, 11,  0, }, // Seychelles
    {  333, 1624,       0,  335209,       0,  0, 12,  0, }, // Singapore
    {  333, 1634,       0,  335221,       0,  0, 20,  0, }, // Solomon
    {  333, 1642,       0,  335241,       0,  0, 20,  0, }, // South_Georgia
    {  333, 1656,       0,  335261,       0,  0, 11,  0, }, // Suriname
    {  333, 1676,       0,  335272,       0,  0, 10,  0, }, // Syowa
    {  333, 1682,       0,  335282,       0,  0, 10,  0, }, // Tahiti
    {  333, 1689,  335292,  335301,  335315,  9, 14, 16, }, // Taipei
    {  333, 1696,       0,  335331,       0,  0, 15,  0, }, // Tajikistan
    {  333, 1724,       0,  335346,       0,  0, 11,  0, }, // Tokelau
    {  333, 1732,  335357,  335366,  335380,  9, 14, 22, }, // Tonga
    {  333, 1738,       0,  335402,       0,  0,  7,  0, }, // Truk
    {  333, 1750,  335409,  335427,  335450, 18, 23, 31, }, // Turkmenistan
    {  333, 1763,       0,  335481,       0,  0, 10,  0, }, // Tuvalu
    {  333, 1777,  335491,  335502,  335518, 11, 16, 24, }, // Uruguay
    {  333, 1792,  335542,  335559,  335581, 17, 22, 30, }, // Uzbekistan
    {  333, 1803,  335611,  335621,  335636, 10, 15, 23, }, // Vanuatu
    {  333, 1811,       0,  335659,       0,  0, 14,  0, }, // Venezuela
    {  333, 1821,  335673,  335690,  335712, 17, 22, 30, }, // Vladivostok
    {  333, 1833,  335742,  335757,  335777, 15, 20, 28, }, // Volgograd
    {  333, 1843,       0,  335679,       0,  0, 11,  0, }, // Vostok
    {  333, 1850,       0,  335805,       0,  0, 13,  0, }, // Wake
    {  333, 1855,       0,  335818,       0,  0, 22,  0, }, // Wallis
    {  333, 1862,   92406,  335840,  335858, 13, 18, 26, }, // Yakutsk
    {  333, 1870,  335884,  335901,  335923, 17, 22, 30, }, // Yekaterinburg
    {  333, 1892,       0,  335953,       0,  0,  9,  0, }, // Yukon
    {  334,    0,       0,       9,      27,  9, 18, 16, }, // Acre Hindi/Latin/India
    {  334,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan
    {  334,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central
    {  334,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  334,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  334,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  334,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  334,  128,     235,     246,     266, 11, 20, 18, }, // Almaty
    {  334,  135,     284,     295,     315, 11, 20, 18, }, // Amazon
    {  334,  142,  335962,  335988,  336023, 26, 35, 35, }, // America_Central
    {  334,  158,  336058,  336084,  336119, 26, 35, 35, }, // America_Eastern
    {  334,  174,  336154,  336181,  336217, 27, 36, 36, }, // America_Mountain
    {  334,  191,  336253,  336279,  336314, 26, 35, 35, }, // America_Pacific
    {  334,  207,     552,     563,     583, 11, 20, 18, }, // Anadyr
    {  334,  214,     601,     610,     628,  9, 18, 18, }, // Apia
    {  334,  219,     646,     656,     675, 10, 19, 17, }, // Aqtau
    {  334,  225,     692,     703,     723, 11, 20, 18, }, // Aqtobe
    {  334,  232,     741,     753,     774, 12, 21, 21, }, // Arabian
    {  334,  240,     795,     809,     832, 14, 23, 21, }, // Argentina
    {  334,  250,     853,     875,     906, 22, 31, 29, }, // Argentina_Western
    {  334,  268,     935,     947,     968, 12, 21, 19, }, // Armenia
    {  334,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  334,  295,    1044,    1066,    1098, 22, 32, 32, }, // Australia_Central
    {  334,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {  334,  338,    1241,    1263,    1295, 22, 32, 32, }, // Australia_Eastern
    {  334,  356,    1327,    1349,    1381, 22, 32, 32, }, // Australia_Western
    {  334,  374,    1413,    1428,    1452, 15, 24, 22, }, // Azerbaijan
    {  334,  385,    1474,    1485,    1505, 11, 20, 18, }, // Azores
    {  334,  397,    1523,    1538,    1562, 15, 24, 22, }, // Bangladesh
    {  334,  415,       0,    1584,       0,  0, 11,  0, }, // Bhutan
    {  334,  422,       0,    1595,       0,  0, 12,  0, }, // Bolivia
    {  334,  437,    1607,    1620,    1642, 13, 22, 20, }, // Brasilia
    {  334,  454,       0,    1662,       0,  0, 22,  0, }, // Brunei
    {  334,  461,    1684,    1699,    1723, 15, 24, 22, }, // Cape_Verde
    {  334,  472,       0,    1745,       0,  0, 10,  0, }, // Casey
    {  334,  478,       0,    1755,       0,  0, 22,  0, }, // Chamorro
    {  334,  487,    1777,    1789,    1810, 12, 21, 21, }, // Chatham
    {  334,  495,    1831,    1841,    1860, 10, 19, 17, }, // Chile
    {  334,  501,    1877,    1887,    1906, 10, 19, 19, }, // China
    {  334,  507,    1925,    1940,    1964, 15, 24, 22, }, // Choibalsan
    {  334,  518,       0,    1986,       0,  0, 21,  0, }, // Christmas
    {  334,  528,       0,    2007,       0,  0, 18,  0, }, // Cocos
    {  334,  534,    2025,    2038,    2060, 13, 22, 20, }, // Colombia
    {  334,  543,    2080,    2097,    2123, 17, 26, 29, }, // Cook
    {  334,  548,    2152,    2161,    2179,  9, 18, 18, }, // Cuba
    {  334,  559,       0,    2197,       0,  0, 10,  0, }, // Davis
    {  334,  575,       0,    2207,       0,  0, 21,  0, }, // DumontDUrville
    {  334,  612,       0,    2228,       0,  0, 15,  0, }, // East_Timor
    {  334,  623,    2243,    2261,    2288, 18, 27, 25, }, // Easter
    {  334,  630,       0,    2313,       0,  0, 12,  0, }, // Ecuador
    {  334,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  334,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  334,  668,       0,    2483,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  334,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  334,  706,    2591,    2612,    2642, 21, 30, 28, }, // Falkland
    {  334,  715,    2670,    2679,    2697,  9, 18, 16, }, // Fiji
    {  334,  720,       0,    2713,       0,  0, 18,  0, }, // French_Guiana
    {  334,  734,       0,    2731,       0,  0, 32,  0, }, // French_Southern
    {  334,  757,       0,    2763,       0,  0, 14,  0, }, // Galapagos
    {  334,  767,       0,    2777,       0,  0, 12,  0, }, // Gambier
    {  334,  775,    2789,    2801,    2822, 12, 21, 19, }, // Georgia
    {  334,  783,       0,    2841,       0,  0, 20,  0, }, // Gilbert_Islands
    {  334,  799,       0,    2861,       0,  0, 19,  0, }, // GMT
    {  334,  813,    2880,    2894,    2917, 14, 23, 21, }, // Greenland
    {  334,  841,    2938,    2957,    2985, 19, 28, 26, }, // Greenland_Eastern
    {  334,  859,    3011,    3030,    3058, 19, 28, 26, }, // Greenland_Western
    {  334,  877,       0,    3084,       0,  0, 18,  0, }, // Guam
    {  334,  882,       0,    3102,       0,  0, 18,  0, }, // Gulf
    {  334,  887,       0,    3120,       0,  0, 11,  0, }, // Guyana
    {  334,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  334,  910,    3209,    3223,    3246, 14, 23, 21, }, // Hong_Kong
    {  334,  920,    3267,    3276,    3294,  9, 18, 16, }, // Hovd
    {  334,  925,       0,    3310,       0,  0, 19,  0, }, // India
    {  334,  931,       0,    3329,       0,  0, 17,  0, }, // Indian_Ocean
    {  334,  944,       0,    3346,       0,  0, 14,  0, }, // Indochina
    {  334,  954,       0,    3360,       0,  0, 22,  0, }, // Indonesia_Central
    {  334,  972,       0,    3382,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  334,  990,       0,    3404,       0,  0, 22,  0, }, // Indonesia_Western
    {  334, 1008,    3426,    3435,    3453,  9, 18, 18, }, // Iran
    {  334, 1019,    3471,    3483,    3504, 12, 21, 19, }, // Irkutsk
    {  334, 1027,    3523,    3534,    3554, 11, 20, 20, }, // Israel
    {  334, 1034,    3574,    3584,    3603, 10, 19, 19, }, // Japan
    {  334, 1040,    3622,    3651,    3689, 29, 38, 36, }, // Kamchatka
    {  334, 1058,       0,    3725,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  334, 1077,       0,    3745,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  334, 1106,    3765,    3776,    3796, 11, 20, 20, }, // Korea
    {  334, 1112,       0,    3816,       0,  0, 11,  0, }, // Kosrae
    {  334, 1119,    3827,    3843,    3868, 16, 25, 23, }, // Krasnoyarsk
    {  334, 1151,       0,    3891,       0,  0, 15,  0, }, // Kyrgystan
    {  334, 1161,       0,    3906,       0,  0, 10,  0, }, // Lanka
    {  334, 1175,       0,    3916,       0,  0, 17,  0, }, // Line_Islands
    {  334, 1188,    3933,    3947,    3970, 14, 23, 23, }, // Lord_Howe
    {  334, 1198,    3993,    4003,    4022, 10, 19, 17, }, // Macau
    {  334, 1204,    4039,    4051,    4072, 12, 21, 19, }, // Magadan
    {  334, 1219,       0,    4091,       0,  0, 13,  0, }, // Malaysia
    {  334, 1228,       0,    4104,       0,  0, 13,  0, }, // Maldives
    {  334, 1237,       0,    4117,       0,  0, 14,  0, }, // Marquesas
    {  334, 1247,       0,    4131,       0,  0, 21,  0, }, // Marshall_Islands
    {  334, 1264,    4152,    4166,    4189, 14, 23, 21, }, // Mauritius
    {  334, 1274,       0,    4210,       0,  0, 11,  0, }, // Mawson
    {  334, 1281,    4221,    4241,    4270, 20, 29, 29, }, // Mexico_Pacific
    {  334, 1296,    4299,    4315,    4340, 16, 25, 23, }, // Mongolia
    {  334, 1305,    4363,    4374,    4394, 11, 20, 18, }, // Moscow
    {  334, 1312,       0,    4412,       0,  0, 12,  0, }, // Myanmar
    {  334, 1320,       0,    4424,       0,  0, 10,  0, }, // Nauru
    {  334, 1326,       0,    4434,       0,  0, 10,  0, }, // Nepal
    {  334, 1332,    4444,    4462,    4489, 18, 27, 25, }, // New_Caledonia
    {  334, 1346,    4514,    4530,    4555, 16, 25, 25, }, // New_Zealand
    {  334, 1358,    4580,    4597,    4623, 17, 26, 26, }, // Newfoundland
    {  334, 1371,       0,    4649,       0,  0,  9,  0, }, // Niue
    {  334, 1376,    4658,    4677,    4705, 19, 28, 28, }, // Norfolk
    {  334, 1384,    4733,    4757,    4790, 24, 33, 31, }, // Noronha
    {  334, 1392,       0,    4821,       0,  0, 26,  0, }, // North_Mariana
    {  334, 1406,    4847,    4863,    4888, 16, 25, 23, }, // Novosibirsk
    {  334, 1418,    4911,    4920,    4938,  9, 18, 16, }, // Omsk
    {  334, 1428,    4954,    4967,    4989, 13, 22, 20, }, // Pakistan
    {  334, 1437,       0,    5009,       0,  0, 10,  0, }, // Palau
    {  334, 1443,       0,    5019,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  334, 1460,    5040,    5053,    5075, 13, 22, 20, }, // Paraguay
    {  334, 1469,    5095,    5104,    5122,  9, 18, 16, }, // Peru
    {  334, 1474,    5138,    5153,    5177, 15, 24, 22, }, // Philippines
    {  334, 1486,       0,    5199,       0,  0, 20,  0, }, // Phoenix_Islands
    {  334, 1502,    5219,    5245,    5280, 26, 35, 35, }, // Pierre_Miquelon
    {  334, 1518,       0,    5315,       0,  0, 13,  0, }, // Pitcairn
    {  334, 1527,       0,    5328,       0,  0, 11,  0, }, // Ponape
    {  334, 1534,       0,    5339,       0,  0, 14,  0, }, // Pyongyang
    {  334, 1544,    5353,    5367,    5390, 14, 23, 21, }, // Qyzylorda
    {  334, 1554,       0,  336349,       0,  0, 12,  0, }, // Reunion
    {  334, 1562,       0,    5423,       0,  0, 12,  0, }, // Rothera
    {  334, 1570,    5435,    5448,    5470, 13, 22, 20, }, // Sakhalin
    {  334, 1579,    5490,    5501,    5521, 11, 20, 18, }, // Samara
    {  334, 1596,    5539,    5549,    5568, 10, 19, 19, }, // Samoa
    {  334, 1602,       0,    5587,       0,  0, 15,  0, }, // Seychelles
    {  334, 1624,       0,    5602,       0,  0, 23,  0, }, // Singapore
    {  334, 1634,       0,    5625,       0,  0, 20,  0, }, // Solomon
    {  334, 1642,       0,    5645,       0,  0, 18,  0, }, // South_Georgia
    {  334, 1656,       0,    5663,       0,  0, 13,  0, }, // Suriname
    {  334, 1676,       0,    5676,       0,  0, 10,  0, }, // Syowa
    {  334, 1682,       0,    5686,       0,  0, 11,  0, }, // Tahiti
    {  334, 1689,    5697,    5708,    5728, 11, 20, 20, }, // Taipei
    {  334, 1696,       0,    5748,       0,  0, 15,  0, }, // Tajikistan
    {  334, 1724,       0,    5763,       0,  0, 12,  0, }, // Tokelau
    {  334, 1732,    5775,    5785,    5804, 10, 19, 17, }, // Tonga
    {  334, 1738,       0,    5821,       0,  0, 10,  0, }, // Truk
    {  334, 1750,    5831,    5848,    5874, 17, 26, 24, }, // Turkmenistan
    {  334, 1763,       0,    5898,       0,  0, 11,  0, }, // Tuvalu
    {  334, 1777,    5909,    5921,    5942, 12, 21, 19, }, // Uruguay
    {  334, 1792,    5961,    5976,    6000, 15, 24, 22, }, // Uzbekistan
    {  334, 1803,    6022,    6034,    6055, 12, 21, 19, }, // Vanuatu
    {  334, 1811,       0,    6074,       0,  0, 14,  0, }, // Venezuela
    {  334, 1821,    6088,    6104,    6129, 16, 25, 23, }, // Vladivostok
    {  334, 1833,    6152,    6166,    6189, 14, 23, 21, }, // Volgograd
    {  334, 1843,       0,    6210,       0,  0, 11,  0, }, // Vostok
    {  334, 1850,       0,    6221,       0,  0, 16,  0, }, // Wake
    {  334, 1855,       0,    6237,       0,  0, 20,  0, }, // Wallis
    {  334, 1862,    6257,    6269,    6290, 12, 21, 19, }, // Yakutsk
    {  334, 1870,    6309,    6327,    6354, 18, 27, 25, }, // Yekaterinburg
    {  334, 1892,       0,    6379,       0,  0, 10,  0, }, // Yukon
    {  335,    0,  336361,  336369,  336381,  8, 12, 14, }, // Acre Hungarian/Latin/Hungary
    {  335,    5,       0,  336395,       0,  0, 16,  0, }, // Afghanistan
    {  335,   17,       0,  336411,       0,  0, 22,  0, }, // Africa_Central
    {  335,   32,       0,  336433,       0,  0, 22,  0, }, // Africa_Eastern
    {  335,   65,       0,  336455,       0,  0, 20,  0, }, // Africa_Southern
    {  335,   81,  336475,  336497,  336520, 22, 23, 24, }, // Africa_Western
    {  335,  107,  336544,  336556,  336572, 12, 16, 18, }, // Alaska
    {  335,  128,  336590,  336600,  336614, 10, 14, 16, }, // Almaty
    {  335,  135,  336630,  336643,  336661, 13, 18, 19, }, // Amazon
    {  335,  142,  336680,  336703,  336730, 23, 27, 29, }, // America_Central
    {  335,  158,  336759,  336781,  336807, 22, 26, 28, }, // America_Eastern
    {  335,  174,  336835,  336849,  336867, 14, 18, 20, }, // America_Mountain
    {  335,  191,  336887,  336905,  336927, 18, 22, 24, }, // America_Pacific
    {  335,  207,  336951,  336962,  336977, 11, 15, 17, }, // Anadyr
    {  335,  214,  336994,  337003,  337017,  9, 14, 15, }, // Apia
    {  335,  219,  337032,  337042,  337056, 10, 14, 16, }, // Aqtau
    {  335,  225,  337072,  337083,  337098, 11, 15, 17, }, // Aqtobe
    {  335,  232,  337115,  337123,  337136,  8, 13, 14, }, // Arabian
    {  335,  240,  337150,  337164,  337183, 14, 19, 20, }, // Argentina
    {  335,  250,  337203,  337228,  337254, 25, 26, 27, }, // Argentina_Western
    {  335,  268,  337281,  337298,  337320, 17, 22, 23, }, // Armenia
    {  335,  286,  337343,  337361,  337383, 18, 22, 24, }, // Atlantic
    {  335,  295,  337407,  337428,  337454, 21, 26, 27, }, // Australia_Central
    {  335,  313,  337481,  337509,  337542, 28, 33, 34, }, // Australia_CentralWestern
    {  335,  338,  337576,  337597,  337623, 21, 26, 27, }, // Australia_Eastern
    {  335,  356,  337487,  337515,  337548, 22, 27, 28, }, // Australia_Western
    {  335,  374,  337650,  337667,  337689, 17, 22, 23, }, // Azerbaijan
    {  335,  385,  337712,  337725,  337739, 13, 14, 15, }, // Azores
    {  335,  397,  337754,  337768,  337787, 14, 19, 20, }, // Bangladesh
    {  335,  415,       0,  337807,       0,  0, 10,  0, }, // Bhutan
    {  335,  422,       0,  337817,       0,  0, 17,  0, }, // Bolivia
    {  335,  437,  337834,  337847,  337865, 13, 18, 19, }, // Brasilia
    {  335,  454,       0,  337884,       0,  0, 23,  0, }, // Brunei
    {  335,  461,  337907,  337934,  337962, 27, 28, 29, }, // Cape_Verde
    {  335,  472,       0,  337991,       0,  0, 11,  0, }, // Casey
    {  335,  478,       0,  338002,       0,  0, 18,  0, }, // Chamorro
    {  335,  487,  338020,  338032,  338049, 12, 17, 18, }, // Chatham
    {  335,  495,  338067,  338081,  338096, 14, 15, 16, }, // Chile
    {  335,  501,  338112,  338121,  338135,  9, 14, 15, }, // China
    {  335,  507,  338150,  338166,  338187, 16, 21, 22, }, // Choibalsan
    {  335,  518,       0,  338209,       0,  0, 21,  0, }, // Christmas
    {  335,  528,       0,  338230,       0,  0, 25,  0, }, // Cocos
    {  335,  534,  338255,  338268,  338286, 13, 18, 19, }, // Colombia
    {  335,  543,  338305,  338323,  338346, 18, 23, 28, }, // Cook
    {  335,  548,  338374,  338387,  338401, 13, 14, 15, }, // Cuba
    {  335,  559,       0,  338416,       0,  0, 10,  0, }, // Davis
    {  335,  575,       0,  338426,       0,  0, 22,  0, }, // DumontDUrville
    {  335,  612,       0,  338448,       0,  0, 21,  0, }, // East_Timor
    {  335,  623,  338469,  338491,  338514, 22, 23, 24, }, // Easter
    {  335,  630,       0,  338538,       0,  0, 17,  0, }, // Ecuador
    {  335,  638,  338555,  338576,  338598, 21, 22, 23, }, // Europe_Central
    {  335,  653,  338621,  338642,  338664, 21, 22, 23, }, // Europe_Eastern
    {  335,  668,       0,  338687,       0,  0, 11,  0, }, // Europe_Further_Eastern
    {  335,  691,  338698,  338720,  338743, 22, 23, 24, }, // Europe_Western
    {  335,  706,  338767,  338789,  338816, 22, 27, 28, }, // Falkland
    {  335,  715,  338844,  338854,  338869, 10, 15, 16, }, // Fiji
    {  335,  720,       0,  338885,       0,  0, 19,  0, }, // French_Guiana
    {  335,  734,       0,  338904,       0,  0, 31,  0, }, // French_Southern
    {  335,  757,       0,  338935,       0,  0, 19,  0, }, // Galapagos
    {  335,  767,       0,  338954,       0,  0, 12,  0, }, // Gambier
    {  335,  775,  338966,  338977,  338993, 11, 16, 17, }, // Georgia
    {  335,  783,       0,  339010,       0,  0, 21,  0, }, // Gilbert_Islands
    {  335,  799,       0,  339031,       0,  0, 29,  0, }, // GMT
    {  335,  841,  339060,  339083,  339107, 23, 24, 25, }, // Greenland_Eastern
    {  335,  859,  339132,  339156,  339181, 24, 25, 26, }, // Greenland_Western
    {  335,  877,       0,  339207,       0,  0, 13,  0, }, // Guam
    {  335,  882,       0,  339220,       0,  0, 17,  0, }, // Gulf
    {  335,  887,       0,  339237,       0,  0, 16,  0, }, // Guyana
    {  335,  894,  339253,  339274,  339296, 21, 22, 23, }, // Hawaii_Aleutian
    {  335,  910,  339319,  339336,  339354, 17, 18, 19, }, // Hong_Kong
    {  335,  920,  339373,  339382,  339396,  9, 14, 15, }, // Hovd
    {  335,  925,       0,  339411,       0,  0, 15,  0, }, // India
    {  335,  931,       0,  339426,       0,  0, 17,  0, }, // Indian_Ocean
    {  335,  944,       0,  339443,       0,  0, 13,  0, }, // Indochina
    {  335,  954,       0,  339456,       0,  0, 20,  0, }, // Indonesia_Central
    {  335,  972,       0,  339476,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  335,  990,       0,  339496,       0,  0, 26,  0, }, // Indonesia_Western
    {  335, 1008,  339522,  339531,  339545,  9, 14, 15, }, // Iran
    {  335, 1019,  339560,  339573,  339591, 13, 18, 19, }, // Irkutsk
    {  335, 1027,  339610,  339621,  339637, 11, 16, 17, }, // Israel
    {  335, 1034,  339654,  339663,  339677,  9, 14, 15, }, // Japan
    {  335, 1040,  339692,  339721,  339754, 29, 33, 35, }, // Kamchatka
    {  335, 1058,       0,  339789,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  335, 1077,       0,  339810,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  335, 1106,  339832,  339842,  339857, 10, 15, 16, }, // Korea
    {  335, 1112,       0,  339873,       0,  0, 11,  0, }, // Kosrae
    {  335, 1119,  339884,  339902,  339925, 18, 23, 24, }, // Krasnoyarsk
    {  335, 1151,       0,  339949,       0,  0, 17,  0, }, // Kyrgystan
    {  335, 1161,       0,  339966,       0,  0, 10,  0, }, // Lanka
    {  335, 1175,       0,  339976,       0,  0, 17,  0, }, // Line_Islands
    {  335, 1188,  339993,  340014,  340040, 21, 26, 27, }, // Lord_Howe
    {  335, 1198,  340067,  340077,  340091, 10, 14, 16, }, // Macau
    {  335, 1204,  340107,  340119,  340136, 12, 17, 18, }, // Magadan
    {  335, 1219,       0,  340154,       0,  0, 13,  0, }, // Malaysia
    {  335, 1228,       0,  340167,       0,  0, 20,  0, }, // Maldives
    {  335, 1237,       0,  340187,       0,  0, 23,  0, }, // Marquesas
    {  335, 1247,       0,  340210,       0,  0, 22,  0, }, // Marshall_Islands
    {  335, 1264,  340232,  340250,  340269, 18, 19, 20, }, // Mauritius
    {  335, 1274,       0,  340289,       0,  0, 11,  0, }, // Mawson
    {  335, 1281,  340300,  340326,  340356, 26, 30, 32, }, // Mexico_Pacific
    {  335, 1296,  340388,  340402,  340421, 14, 19, 20, }, // Mongolia
    {  335, 1305,  340441,  340453,  340470, 12, 17, 18, }, // Moscow
    {  335, 1312,       0,  340488,       0,  0, 12,  0, }, // Myanmar
    {  335, 1320,       0,  340500,       0,  0, 10,  0, }, // Nauru
    {  335, 1326,       0,  340510,       0,  0, 10,  0, }, // Nepal
    {  335, 1332,  340520,  340537,  340559, 17, 22, 23, }, // New_Caledonia
    {  335, 1346,  340582,  340596,  340615, 14, 19, 20, }, // New_Zealand
    {  335, 1358,  340635,  340651,  340671, 16, 20, 22, }, // Newfoundland
    {  335, 1371,       0,  340693,       0,  0,  9,  0, }, // Niue
    {  335, 1376,  340702,  340723,  340749, 21, 26, 27, }, // Norfolk
    {  335, 1384,  340776,  340801,  340831, 25, 30, 31, }, // Noronha
    {  335, 1392,       0,  340862,       0,  0, 27,  0, }, // North_Mariana
    {  335, 1406,  340889,  340907,  340930, 18, 23, 24, }, // Novosibirsk
    {  335, 1418,  340954,  340964,  340979, 10, 15, 16, }, // Omsk
    {  335, 1428,  340995,  341009,  341028, 14, 19, 20, }, // Pakistan
    {  335, 1437,       0,  341048,       0,  0, 10,  0, }, // Palau
    {  335, 1443,       0,  341058,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  335, 1460,  341078,  341091,  341109, 13, 18, 19, }, // Paraguay
    {  335, 1469,  341128,  341137,  341151,  9, 14, 15, }, // Peru
    {  335, 1474,  341166,  341185,  341209, 19, 24, 25, }, // Philippines
    {  335, 1486,       0,  341234,       0,  0, 26,  0, }, // Phoenix_Islands
    {  335, 1502,  341260,  341290,  341324, 30, 34, 36, }, // Pierre_Miquelon
    {  335, 1518,       0,  341360,       0,  0, 22,  0, }, // Pitcairn
    {  335, 1527,       0,  341382,       0,  0, 18,  0, }, // Ponape
    {  335, 1534,       0,  341400,       0,  0, 12,  0, }, // Pyongyang
    {  335, 1544,  341412,  341426,  341444, 14, 18, 20, }, // Qyzylorda
    {  335, 1554,       0,  341464,       0,  0, 12,  0, }, // Reunion
    {  335, 1562,       0,  341476,       0,  0, 12,  0, }, // Rothera
    {  335, 1570,  341488,  341501,  341519, 13, 18, 19, }, // Sakhalin
    {  335, 1579,  341538,  341550,  341566, 12, 16, 18, }, // Samara
    {  335, 1596,  341584,  341595,  341611, 11, 16, 17, }, // Samoa
    {  335, 1602,       0,  341628,       0,  0, 23,  0, }, // Seychelles
    {  335, 1624,       0,  341651,       0,  0, 19,  0, }, // Singapore
    {  335, 1634,       0,  341670,       0,  0, 21,  0, }, // Solomon
    {  335, 1642,       0,  341691,       0,  0, 17,  0, }, // South_Georgia
    {  335, 1656,       0,  341708,       0,  0, 13,  0, }, // Suriname
    {  335, 1676,       0,  341721,       0,  0, 10,  0, }, // Syowa
    {  335, 1682,       0,  341731,       0,  0, 10,  0, }, // Tahiti
    {  335, 1689,  341741,  341751,  341766, 10, 15, 16, }, // Taipei
    {  335, 1696,       0,  341782,       0,  0, 18,  0, }, // Tajikistan
    {  335, 1724,       0,  341800,       0,  0, 12,  0, }, // Tokelau
    {  335, 1732,  341812,  341822,  341837, 10, 15, 16, }, // Tonga
    {  335, 1738,       0,  341853,       0,  0,  9,  0, }, // Truk
    {  335, 1750,  341862,  341880,  341903, 18, 23, 24, }, // Turkmenistan
    {  335, 1763,       0,  341927,       0,  0, 11,  0, }, // Tuvalu
    {  335, 1777,  341938,  341950,  341967, 12, 17, 18, }, // Uruguay
    {  335, 1792,  341985,  342001,  342022, 16, 21, 22, }, // Uzbekistan
    {  335, 1803,  342044,  342056,  342073, 12, 17, 18, }, // Vanuatu
    {  335, 1811,       0,  342091,       0,  0, 14,  0, }, // Venezuela
    {  335, 1821,  342105,  342123,  342146, 18, 23, 24, }, // Vladivostok
    {  335, 1833,  342170,  342184,  342203, 14, 19, 20, }, // Volgograd
    {  335, 1843,       0,  342111,       0,  0, 12,  0, }, // Vostok
    {  335, 1850,       0,  342223,       0,  0, 16,  0, }, // Wake
    {  335, 1855,       0,  342239,       0,  0, 22,  0, }, // Wallis
    {  335, 1862,  342261,  342274,  342292, 13, 18, 19, }, // Yakutsk
    {  335, 1870,  342311,  342330,  342354, 19, 24, 25, }, // Yekaterinburg
    {  335, 1892,       0,  342379,       0,  0, 10,  0, }, // Yukon
    {  336,    5,       0,  342389,       0,  0, 14,  0, }, // Afghanistan Icelandic/Latin/Iceland
    {  336,   17,       0,  342403,       0,  0, 14,  0, }, // Africa_Central
    {  336,   32,       0,  342417,       0,  0, 17,  0, }, // Africa_Eastern
    {  336,   65,       0,  342434,       0,  0, 16,  0, }, // Africa_Southern
    {  336,   81,  342450,  342467,  342493, 17, 26, 25, }, // Africa_Western
    {  336,  107,  342518,  342531,  342550, 13, 19, 18, }, // Alaska
    {  336,  135,  342568,  342578,  342604, 10, 26, 25, }, // Amazon
    {  336,  142,  342629,  342668,  342713, 39, 45, 44, }, // America_Central
    {  336,  158,  342757,  342799,  342847, 42, 48, 47, }, // America_Eastern
    {  336,  174,  342894,  342914,  342940, 20, 26, 25, }, // America_Mountain
    {  336,  191,  342965,  342988,  343017, 23, 29, 28, }, // America_Pacific
    {  336,  207,  343045,  343058,  343077, 13, 19, 18, }, // Anadyr
    {  336,  214,  343095,  343106,  343123, 11, 17, 16, }, // Apia
    {  336,  232,  343139,  343149,  343168, 10, 19, 18, }, // Arabian
    {  336,  240,  343186,  343199,  343221, 13, 22, 21, }, // Argentina
    {  336,  250,  343242,  343262,  343291, 20, 29, 28, }, // Argentina_Western
    {  336,  268,  343319,  343330,  343350, 11, 20, 19, }, // Armenia
    {  336,  286,  343369,  343394,  343425, 25, 31, 30, }, // Atlantic
    {  336,  295,  343455,  343474,  343499, 19, 25, 24, }, // Australia_Central
    {  336,  313,  343523,  343553,  343589, 30, 36, 35, }, // Australia_CentralWestern
    {  336,  338,  343624,  343646,  343674, 22, 28, 27, }, // Australia_Eastern
    {  336,  356,  343701,  343723,  343751, 22, 28, 27, }, // Australia_Western
    {  336,  374,  343778,  343794,  343819, 16, 25, 24, }, // Azerbaijan
    {  336,  385,  343843,  343855,  343877, 12, 22, 21, }, // Azores
    {  336,  397,  343898,  343913,  343936, 15, 23, 22, }, // Bangladesh
    {  336,  415,       0,  343958,       0,  0,  9,  0, }, // Bhutan
    {  336,  422,       0,  343967,       0,  0, 11,  0, }, // Bolivia
    {  336,  437,  343978,  343990,  344011, 12, 21, 20, }, // Brasilia
    {  336,  454,       0,  344031,       0,  0, 10,  0, }, // Brunei
    {  336,  461,  344041,  344058,  344085, 17, 27, 26, }, // Cape_Verde
    {  336,  478,       0,  344111,       0,  0, 19,  0, }, // Chamorro
    {  336,  487,  344130,  344142,  344162, 12, 20, 19, }, // Chatham
    {  336,  495,  344181,  344189,  344206,  8, 17, 16, }, // Chile
    {  336,  501,  344222,  344230,  344247,  8, 17, 16, }, // China
    {  336,  507,  344263,  344280,  344303, 17, 23, 22, }, // Choibalsan
    {  336,  518,       0,  344325,       0,  0, 13,  0, }, // Christmas
    {  336,  528,       0,  344338,       0,  0, 13,  0, }, // Cocos
    {  336,  534,  344351,  344363,  344384, 12, 21, 20, }, // Colombia
    {  336,  543,  344404,  344418,  344442, 14, 24, 27, }, // Cook
    {  336,  548,  344469,  344477,  344494,  8, 17, 16, }, // Cuba
    {  336,  559,       0,  344510,       0,  0, 10,  0, }, // Davis
    {  336,  575,       0,  344520,       0,  0, 23,  0, }, // DumontDUrville
    {  336,  612,       0,  344543,       0,  0, 18,  0, }, // East_Timor
    {  336,  623,  344561,  344574,  344596, 13, 22, 21, }, // Easter
    {  336,  630,       0,  344617,       0,  0, 11,  0, }, // Ecuador
    {  336,  638,  344628,  344642,  344665, 14, 23, 22, }, // Europe_Central
    {  336,  653,  344687,  344704,  344730, 17, 26, 25, }, // Europe_Eastern
    {  336,  668,       0,  344755,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  336,  691,  344777,  344794,  344820, 17, 26, 25, }, // Europe_Western
    {  336,  706,  344845,  344862,  344889, 17, 27, 26, }, // Falkland
    {  336,  715,  344915,  344928,  344951, 13, 23, 22, }, // Fiji
    {  336,  720,       0,  344973,       0,  0, 22,  0, }, // French_Guiana
    {  336,  734,       0,  344995,       0,  0, 55,  0, }, // French_Southern
    {  336,  757,       0,  345050,       0,  0, 14,  0, }, // Galapagos
    {  336,  767,       0,  345064,       0,  0, 12,  0, }, // Gambier
    {  336,  775,  345076,  345087,  345107, 11, 20, 19, }, // Georgia
    {  336,  783,       0,  345126,       0,  0, 20,  0, }, // Gilbert_Islands
    {  336,  799,       0,  345146,       0,  0, 20,  0, }, // GMT
    {  336,  841,  345166,  345186,  345215, 20, 29, 28, }, // Greenland_Eastern
    {  336,  859,  345243,  345263,  345292, 20, 29, 28, }, // Greenland_Western
    {  336,  882,       0,  345320,       0,  0, 24,  0, }, // Gulf
    {  336,  887,       0,  345344,       0,  0, 11,  0, }, // Guyana
    {  336,  894,  345355,  345377,  345405, 22, 28, 27, }, // Hawaii_Aleutian
    {  336,  910,  345432,  345446,  345468, 14, 22, 21, }, // Hong_Kong
    {  336,  920,  345489,  345498,  345515,  9, 17, 16, }, // Hovd
    {  336,  925,       0,  345531,       0,  0, 12,  0, }, // India
    {  336,  931,       0,  345543,       0,  0, 16,  0, }, // Indian_Ocean
    {  336,  944,       0,  345559,       0,  0, 12,  0, }, // Indochina
    {  336,  954,       0,  345571,       0,  0, 17,  0, }, // Indonesia_Central
    {  336,  972,       0,  345588,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  336,  990,       0,  345608,       0,  0, 20,  0, }, // Indonesia_Western
    {  336, 1008,  345628,  345637,  345654,  9, 17, 16, }, // Iran
    {  336, 1019,  345670,  345684,  345704, 14, 20, 19, }, // Irkutsk
    {  336, 1027,  345723,  345734,  345753, 11, 19, 18, }, // Israel
    {  336, 1034,  345771,  345781,  345799, 10, 18, 17, }, // Japan
    {  336, 1040,  345816,  345847,  345884, 31, 37, 36, }, // Kamchatka
    {  336, 1058,       0,  345920,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  336, 1077,       0,  345943,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  336, 1106,  345966,  345975,  345993,  9, 18, 17, }, // Korea
    {  336, 1112,       0,  346010,       0,  0, 11,  0, }, // Kosrae
    {  336, 1119,  346021,  346039,  346063, 18, 24, 23, }, // Krasnoyarsk
    {  336, 1151,       0,  346086,       0,  0, 14,  0, }, // Kyrgystan
    {  336, 1175,       0,  346100,       0,  0, 13,  0, }, // Line_Islands
    {  336, 1188,  346113,  346134,  346161, 21, 27, 26, }, // Lord_Howe
    {  336, 1204,  346187,  346201,  346221, 14, 20, 19, }, // Magadan
    {  336, 1219,       0,  346240,       0,  0, 11,  0, }, // Malaysia
    {  336, 1228,       0,  346251,       0,  0, 15,  0, }, // Maldives
    {  336, 1237,       0,  346266,       0,  0, 26,  0, }, // Marquesas
    {  336, 1247,       0,  346292,       0,  0, 21,  0, }, // Marshall_Islands
    {  336, 1264,  346313,  346325,  346346, 12, 21, 20, }, // Mauritius
    {  336, 1274,       0,  346366,       0,  0, 11,  0, }, // Mawson
    {  336, 1281,  346377,  346399,  346437, 22, 38, 37, }, // Mexico_Pacific
    {  336, 1296,  346474,  346491,  346514, 17, 23, 22, }, // Mongolia
    {  336, 1305,  346536,  346546,  346565, 10, 19, 18, }, // Moscow
    {  336, 1312,       0,  346583,       0,  0, 12,  0, }, // Myanmar
    {  336, 1320,       0,  346595,       0,  0,  9,  0, }, // Nauru
    {  336, 1326,       0,  346604,       0,  0,  9,  0, }, // Nepal
    {  336, 1332,  346613,  346634,  346661, 21, 27, 26, }, // New_Caledonia
    {  336, 1346,  346687,  346707,  346733, 20, 26, 25, }, // New_Zealand
    {  336, 1358,  346758,  346778,  346804, 20, 26, 25, }, // Newfoundland
    {  336, 1371,       0,  346829,       0,  0,  9,  0, }, // Niue
    {  336, 1376,  346838,  346857,  346882, 19, 25, 24, }, // Norfolk
    {  336, 1384,  346906,  346932,  346964, 26, 32, 31, }, // Noronha
    {  336, 1406,  346995,  347013,  347037, 18, 24, 23, }, // Novosibirsk
    {  336, 1418,  347060,  347071,  347088, 11, 17, 16, }, // Omsk
    {  336, 1428,  347104,  347116,  347137, 12, 21, 20, }, // Pakistan
    {  336, 1437,       0,  347157,       0,  0,  8,  0, }, // Palau
    {  336, 1443,       0,  347165,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  336, 1460,  347188,  347199,  347219, 11, 20, 19, }, // Paraguay
    {  336, 1469,  347238,  347246,  347263,  8, 17, 16, }, // Peru
    {  336, 1474,  347279,  347294,  347319, 15, 25, 24, }, // Philippines
    {  336, 1486,       0,  347343,       0,  0, 13,  0, }, // Phoenix_Islands
    {  336, 1502,  347356,  347388,  347426, 32, 38, 37, }, // Pierre_Miquelon
    {  336, 1518,       0,  347463,       0,  0, 13,  0, }, // Pitcairn
    {  336, 1527,       0,  347476,       0,  0, 11,  0, }, // Ponape
    {  336, 1534,       0,  347487,       0,  0, 16,  0, }, // Pyongyang
    {  336, 1554,       0,  347503,       0,  0, 12,  0, }, // Reunion
    {  336, 1562,       0,  347515,       0,  0, 12,  0, }, // Rothera
    {  336, 1570,  347527,  347542,  347563, 15, 21, 20, }, // Sakhalin
    {  336, 1579,  347583,  347596,  347615, 13, 19, 18, }, // Samara
    {  336, 1596,  347633,  347643,  347661, 10, 18, 17, }, // Samoa
    {  336, 1602,       0,  347678,       0,  0, 19,  0, }, // Seychelles
    {  336, 1624,       0,  347697,       0,  0, 12,  0, }, // Singapore
    {  336, 1634,       0,  347709,       0,  0, 16,  0, }, // Solomon
    {  336, 1642,       0,  347725,       0,  0, 17,  0, }, // South_Georgia
    {  336, 1656,       0,  347742,       0,  0, 11,  0, }, // Suriname
    {  336, 1676,       0,  347753,       0,  0, 10,  0, }, // Syowa
    {  336, 1682,       0,  347763,       0,  0, 11,  0, }, // Tahiti
    {  336, 1689,  347774,  347785,  347804, 11, 19, 18, }, // Taipei
    {  336, 1696,       0,  347822,       0,  0, 17,  0, }, // Tajikistan
    {  336, 1724,       0,  347839,       0,  0, 11,  0, }, // Tokelau
    {  336, 1732,  347850,  347859,  347877,  9, 18, 17, }, // Tonga
    {  336, 1738,       0,  347894,       0,  0, 10,  0, }, // Truk
    {  336, 1750,  347904,  347921,  347946, 17, 25, 24, }, // Turkmenistan
    {  336, 1763,       0,  347970,       0,  0, 10,  0, }, // Tuvalu
    {  336, 1777,  347980,  347990,  348009, 10, 19, 18, }, // Uruguay
    {  336, 1792,  348027,  348042,  348065, 15, 23, 22, }, // Uzbekistan
    {  336, 1803,  348087,  348099,  348119, 12, 20, 19, }, // Vanuatu
    {  336, 1811,       0,  348138,       0,  0, 13,  0, }, // Venezuela
    {  336, 1821,  348151,  348169,  348193, 18, 24, 23, }, // Vladivostok
    {  336, 1833,  348216,  348232,  348254, 16, 22, 21, }, // Volgograd
    {  336, 1843,       0,  348275,       0,  0, 11,  0, }, // Vostok
    {  336, 1850,       0,  348286,       0,  0, 16,  0, }, // Wake
    {  336, 1855,       0,  348302,       0,  0, 29,  0, }, // Wallis
    {  336, 1862,  348331,  348345,  348365, 14, 20, 19, }, // Yakutsk
    {  336, 1870,  348384,  348404,  348430, 20, 26, 25, }, // Yekaterinburg
    {  336, 1892,       0,  348455,       0,  0, 12,  0, }, // Yukon
    {  338,    5,       0,  348467,       0,  0, 15,  0, }, // Afghanistan Igbo/Latin/Nigeria
    {  338,   17,       0,  348482,       0,  0, 16,  0, }, // Africa_Central
    {  338,   32,       0,  348498,       0,  0, 33,  0, }, // Africa_Eastern
    {  338,   65,       0,  348531,       0,  0, 38,  0, }, // Africa_Southern
    {  338,   81,  348569,  348602,  348642, 33, 40, 42, }, // Africa_Western
    {  338,  107,  348684,  348694,  348711, 10, 17, 14, }, // Alaska
    {  338,  135,  348725,  348735,  348752, 10, 17, 19, }, // Amazon
    {  338,  142,  348771,  348789,  348814, 18, 25, 22, }, // America_Central
    {  338,  158,  348498,  348836,  348869, 26, 33, 30, }, // America_Eastern
    {  338,  174,  348899,  348916,  348940, 17, 24, 21, }, // America_Mountain
    {  338,  191,  348961,  348981,  349008, 20, 27, 24, }, // America_Pacific
    {  338,  214,  349032,  349040,  349055,  8, 15, 12, }, // Apia
    {  338,  232,  349067,  349075,  349090,  8, 15, 12, }, // Arabian
    {  338,  240,  349102,  349115,  349135, 13, 20, 22, }, // Argentina
    {  338,  250,  349157,  349193,  349236, 36, 43, 45, }, // Argentina_Western
    {  338,  268,  349281,  349292,  349310, 11, 18, 20, }, // Armenia
    {  338,  286,  349330,  349351,  349379, 21, 28, 25, }, // Atlantic
    {  338,  295,  349404,  349423,  349449, 19, 26, 23, }, // Australia_Central
    {  338,  313,  349472,  349514,  349563, 42, 49, 46, }, // Australia_CentralWestern
    {  338,  338,  349609,  349645,  349688, 36, 43, 40, }, // Australia_Eastern
    {  338,  356,  349728,  349764,  349807, 36, 43, 40, }, // Australia_Western
    {  338,  374,  349847,  349861,  349882, 14, 21, 23, }, // Azerbaijan
    {  338,  385,  349905,  349915,  349932, 10, 17, 19, }, // Azores
    {  338,  397,  349951,  349965,  349986, 14, 21, 23, }, // Bangladesh
    {  338,  415,       0,  350009,       0,  0, 10,  0, }, // Bhutan
    {  338,  422,       0,  350019,       0,  0, 11,  0, }, // Bolivia
    {  338,  437,  350030,  350042,  350061, 12, 19, 21, }, // Brasilia
    {  338,  454,       0,  350082,       0,  0, 21,  0, }, // Brunei
    {  338,  461,  350103,  350117,  350138, 14, 21, 23, }, // Cape_Verde
    {  338,  478,       0,  350161,       0,  0, 19,  0, }, // Chamorro
    {  338,  487,  350180,  350191,  350209, 11, 18, 15, }, // Chatham
    {  338,  495,  350224,  350233,  350249,  9, 16, 18, }, // Chile
    {  338,  501,  350267,  350276,  350292,  9, 16, 13, }, // China
    {  338,  507,  350305,  350317,  350336, 12, 19, 21, }, // Choibalsan
    {  338,  518,       0,  350357,       0,  0, 22,  0, }, // Christmas
    {  338,  528,       0,  350379,       0,  0, 17,  0, }, // Cocos
    {  338,  534,  350396,  350408,  350427, 12, 19, 21, }, // Colombia
    {  338,  543,  350448,  350464,  350487, 16, 23, 31, }, // Cook
    {  338,  548,  350518,  350526,  350541,  8, 15, 21, }, // Cuba
    {  338,  559,       0,  350562,       0,  0,  9,  0, }, // Davis
    {  338,  575,       0,  350571,       0,  0, 20,  0, }, // DumontDUrville
    {  338,  612,       0,  350591,       0,  0, 32,  0, }, // East_Timor
    {  338,  623,  350623,  350656,  350696, 33, 40, 42, }, // Easter
    {  338,  630,       0,  350738,       0,  0, 11,  0, }, // Ecuador
    {  338,  638,  350749,  350774,  350806, 25, 32, 34, }, // Europe_Central
    {  338,  653,  350840,  350873,  350913, 33, 40, 42, }, // Europe_Eastern
    {  338,  668,       0,    2483,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  338,  691,  350955,  350988,  351028, 33, 40, 42, }, // Europe_Western
    {  338,  706,  351070,  351090,  351117, 20, 27, 29, }, // Falkland
    {  338,  715,  351146,  351154,  351169,  8, 15, 17, }, // Fiji
    {  338,  720,       0,  351186,       0,  0, 17,  0, }, // French_Guiana
    {  338,  734,       0,  351203,       0,  0, 31,  0, }, // French_Southern
    {  338,  757,       0,  351234,       0,  0, 13,  0, }, // Galapagos
    {  338,  767,       0,  351247,       0,  0, 11,  0, }, // Gambier
    {  338,  775,  351258,  351269,  351287, 11, 18, 20, }, // Georgia
    {  338,  783,       0,  351307,       0,  0, 19,  0, }, // Gilbert_Islands
    {  338,  799,       0,  351326,       0,  0, 27,  0, }, // GMT
    {  338,  841,  351353,  351389,  351432, 36, 43, 45, }, // Greenland_Eastern
    {  338,  859,  351477,  351513,  351556, 36, 43, 45, }, // Greenland_Western
    {  338,  882,       0,  351601,       0,  0, 15,  0, }, // Gulf
    {  338,  887,       0,  351616,       0,  0, 10,  0, }, // Guyana
    {  338,  894,  351626,  351645,  351671, 19, 26, 23, }, // Hawaii_Aleutian
    {  338,  910,  351694,  351707,  351727, 13, 20, 22, }, // Hong_Kong
    {  338,  920,  351749,  351757,  351772,  8, 15, 17, }, // Hovd
    {  338,  925,       0,  351789,       0,  0, 16,  0, }, // India
    {  338,  931,       0,  351805,       0,  0, 17,  0, }, // Indian_Ocean
    {  338,  944,       0,  351822,       0,  0, 13,  0, }, // Indochina
    {  338,  954,       0,  351835,       0,  0, 19,  0, }, // Indonesia_Central
    {  338,  972,       0,  351854,       0,  0, 36,  0, }, // Indonesia_Eastern
    {  338,  990,       0,  351890,       0,  0, 36,  0, }, // Indonesia_Western
    {  338, 1008,  351926,  351934,  351949,  8, 15, 12, }, // Iran
    {  338, 1019,  351961,  351972,  351990, 11, 18, 20, }, // Irkutsk
    {  338, 1027,  352010,  352020,  352037, 10, 17, 14, }, // Israel
    {  338, 1034,  352051,  352060,  352076,  9, 16, 13, }, // Japan
    {  338, 1058,       0,  352089,       0,  0, 37,  0, }, // Kazakhstan_Eastern
    {  338, 1077,       0,  352126,       0,  0, 37,  0, }, // Kazakhstan_Western
    {  338, 1106,  352163,  352172,  352188,  9, 16, 13, }, // Korea
    {  338, 1112,       0,  352201,       0,  0, 10,  0, }, // Kosrae
    {  338, 1119,  352211,  352226,  352248, 15, 22, 24, }, // Krasnoyarsk
    {  338, 1151,       0,  352272,       0,  0, 14,  0, }, // Kyrgystan
    {  338, 1175,       0,  352286,       0,  0, 16,  0, }, // Line_Islands
    {  338, 1188,  352302,  352315,  352335, 13, 20, 17, }, // Lord_Howe
    {  338, 1204,  352352,  352363,  352381, 11, 18, 20, }, // Magadan
    {  338, 1219,       0,  352401,       0,  0, 12,  0, }, // Malaysia
    {  338, 1228,       0,  352413,       0,  0, 12,  0, }, // Maldives
    {  338, 1237,       0,  352425,       0,  0, 13,  0, }, // Marquesas
    {  338, 1247,       0,  352438,       0,  0, 20,  0, }, // Marshall_Islands
    {  338, 1264,  352458,  352471,  352491, 13, 20, 22, }, // Mauritius
    {  338, 1274,       0,  352513,       0,  0, 10,  0, }, // Mawson
    {  338, 1281,  352523,  352542,  352568, 19, 26, 23, }, // Mexico_Pacific
    {  338, 1296,  352591,  352606,  352628, 15, 22, 24, }, // Mongolia
    {  338, 1305,  352652,  352662,  352679, 10, 17, 19, }, // Moscow
    {  338, 1312,       0,  352698,       0,  0, 11,  0, }, // Myanmar
    {  338, 1320,       0,  352709,       0,  0,  9,  0, }, // Nauru
    {  338, 1326,       0,  352718,       0,  0,  9,  0, }, // Nepal
    {  338, 1332,  352727,  352744,  352768, 17, 24, 26, }, // New_Caledonia
    {  338, 1346,  352794,  352809,  352831, 15, 22, 19, }, // New_Zealand
    {  338, 1358,  352850,  352866,  352889, 16, 23, 20, }, // Newfoundland
    {  338, 1371,       0,  352909,       0,  0,  8,  0, }, // Niue
    {  338, 1376,  352917,  352935,  352960, 18, 25, 27, }, // Norfolk
    {  338, 1384,  352987,  353010,  353040, 23, 30, 32, }, // Noronha
    {  338, 1406,  353072,  353087,  353109, 15, 22, 24, }, // Novosibirsk
    {  338, 1418,  353133,  353141,  353156,  8, 15, 17, }, // Omsk
    {  338, 1428,  353173,  353185,  353204, 12, 19, 21, }, // Pakistan
    {  338, 1437,       0,  353225,       0,  0,  9,  0, }, // Palau
    {  338, 1443,       0,  353234,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  338, 1460,  353254,  353266,  353285, 12, 19, 21, }, // Paraguay
    {  338, 1469,  353306,  353314,  353329,  8, 15, 17, }, // Peru
    {  338, 1474,  353346,  353360,  353381, 14, 21, 23, }, // Philippines
    {  338, 1486,       0,  353404,       0,  0, 19,  0, }, // Phoenix_Islands
    {  338, 1502,  353423,  353448,  353480, 25, 32, 29, }, // Pierre_Miquelon
    {  338, 1518,       0,  353509,       0,  0, 12,  0, }, // Pitcairn
    {  338, 1527,       0,  353521,       0,  0, 10,  0, }, // Ponape
    {  338, 1534,       0,  353531,       0,  0, 13,  0, }, // Pyongyang
    {  338, 1554,       0,  353544,       0,  0, 11,  0, }, // Reunion
    {  338, 1562,       0,  353555,       0,  0, 11,  0, }, // Rothera
    {  338, 1570,  353566,  353578,  353597, 12, 19, 21, }, // Sakhalin
    {  338, 1596,  353618,  353627,  353643,  9, 16, 13, }, // Samoa
    {  338, 1602,       0,  353656,       0,  0, 14,  0, }, // Seychelles
    {  338, 1624,       0,  353670,       0,  0, 20,  0, }, // Singapore
    {  338, 1634,       0,  353690,       0,  0, 19,  0, }, // Solomon
    {  338, 1642,       0,  353709,       0,  0, 17,  0, }, // South_Georgia
    {  338, 1656,       0,  353726,       0,  0, 12,  0, }, // Suriname
    {  338, 1676,       0,  353738,       0,  0,  9,  0, }, // Syowa
    {  338, 1682,       0,  353747,       0,  0, 10,  0, }, // Tahiti
    {  338, 1689,  353757,  353767,  353784, 10, 17, 14, }, // Taipei
    {  338, 1696,       0,  353798,       0,  0, 14,  0, }, // Tajikistan
    {  338, 1724,       0,  353812,       0,  0, 11,  0, }, // Tokelau
    {  338, 1732,  353823,  353832,  353848,  9, 16, 18, }, // Tonga
    {  338, 1738,       0,  353866,       0,  0,  9,  0, }, // Truk
    {  338, 1750,  353875,  353889,  353910, 14, 21, 23, }, // Turkmenistan
    {  338, 1763,       0,  353933,       0,  0, 10,  0, }, // Tuvalu
    {  338, 1777,  353943,  353954,  353972, 11, 18, 20, }, // Uruguay
    {  338, 1792,  353992,  354004,  354023, 12, 19, 21, }, // Uzbekistan
    {  338, 1803,  354044,  354055,  354073, 11, 18, 20, }, // Vanuatu
    {  338, 1811,       0,  354093,       0,  0, 13,  0, }, // Venezuela
    {  338, 1821,  354106,  354121,  354143, 15, 22, 24, }, // Vladivostok
    {  338, 1833,  354167,  354180,  354200, 13, 20, 22, }, // Volgograd
    {  338, 1843,       0,  354222,       0,  0, 10,  0, }, // Vostok
    {  338, 1850,       0,  354232,       0,  0, 15,  0, }, // Wake
    {  338, 1855,       0,  354247,       0,  0, 19,  0, }, // Wallis
    {  338, 1862,  354266,  354277,  354295, 11, 18, 20, }, // Yakutsk
    {  338, 1870,  354315,  354332,  354356, 17, 24, 26, }, // Yekaterinburg
    {  338, 1892,       0,  354382,       0,  0,  9,  0, }, // Yukon
    {  340,    0,  354391,  354401,  354419, 10, 18, 22, }, // Acre Indonesian/Latin/Indonesia
    {  340,    5,       0,  354441,       0,  0, 16,  0, }, // Afghanistan
    {  340,   17,       0,  354457,       0,  0, 19,  0, }, // Africa_Central
    {  340,   32,       0,  354476,       0,  0, 18,  0, }, // Africa_Eastern
    {  340,   65,       0,  354494,       0,  0, 28,  0, }, // Africa_Southern
    {  340,   81,  354522,  354540,  354566, 18, 26, 30, }, // Africa_Western
    {  340,  107,  354596,  354608,  354628, 12, 20, 24, }, // Alaska
    {  340,  128,  354652,  354664,  354684, 12, 20, 24, }, // Almaty
    {  340,  135,  354708,  354720,  354740, 12, 20, 24, }, // Amazon
    {  340,  142,  354764,  354776,  354796, 12, 20, 24, }, // America_Central
    {  340,  158,  354820,  354831,  354850, 11, 19, 23, }, // America_Eastern
    {  340,  174,  354873,  354889,  354913, 16, 24, 28, }, // America_Mountain
    {  340,  191,  354941,  354954,  354975, 13, 21, 25, }, // America_Pacific
    {  340,  207,  355000,  355012,  355032, 12, 20, 24, }, // Anadyr
    {  340,  214,  355056,  355066,  355084, 10, 18, 22, }, // Apia
    {  340,  219,  355106,  355117,  355136, 11, 19, 23, }, // Aqtau
    {  340,  225,  355159,  355171,  355191, 12, 20, 24, }, // Aqtobe
    {  340,  232,  355215,  355225,  355243, 10, 18, 22, }, // Arabian
    {  340,  240,  355265,  355280,  355303, 15, 23, 27, }, // Argentina
    {  340,  250,  355330,  355358,  355394, 28, 36, 40, }, // Argentina_Western
    {  340,  268,  355434,  355447,  355468, 13, 21, 25, }, // Armenia
    {  340,  286,  355493,  355507,  355529, 14, 22, 26, }, // Atlantic
    {  340,  295,  355555,  355577,  355607, 22, 30, 34, }, // Australia_Central
    {  340,  313,  355641,  355669,  355705, 28, 36, 40, }, // Australia_CentralWestern
    {  340,  338,  355745,  355766,  355795, 21, 29, 33, }, // Australia_Eastern
    {  340,  356,  355828,  355849,  355878, 21, 29, 33, }, // Australia_Western
    {  340,  374,  355911,  355927,  355951, 16, 24, 28, }, // Azerbaijan
    {  340,  385,  355979,  355991,  356011, 12, 20, 24, }, // Azores
    {  340,  397,  356035,  356051,  356075, 16, 24, 28, }, // Bangladesh
    {  340,  415,       0,  356103,       0,  0, 12,  0, }, // Bhutan
    {  340,  422,       0,  356115,       0,  0, 13,  0, }, // Bolivia
    {  340,  437,  356128,  356140,  356160, 12, 20, 24, }, // Brasilia
    {  340,  454,       0,  356184,       0,  0, 23,  0, }, // Brunei
    {  340,  461,  356207,  356226,  356253, 19, 27, 31, }, // Cape_Verde
    {  340,  472,       0,  356284,       0,  0, 11,  0, }, // Casey
    {  340,  478,       0,  356295,       0,  0, 22,  0, }, // Chamorro
    {  340,  487,  356317,  356330,  356351, 13, 21, 25, }, // Chatham
    {  340,  495,  356376,  356386,  356404, 10, 18, 22, }, // Chile
    {  340,  501,  356426,  356440,  356462, 14, 22, 26, }, // China
    {  340,  507,  356488,  356504,  356528, 16, 24, 28, }, // Choibalsan
    {  340,  518,       0,  356556,       0,  0, 17,  0, }, // Christmas
    {  340,  528,       0,  356573,       0,  0, 21,  0, }, // Cocos
    {  340,  534,  356594,  356608,  356630, 14, 22, 26, }, // Colombia
    {  340,  543,  356656,  356671,  356694, 15, 23, 34, }, // Cook
    {  340,  548,  356728,  356738,  356756, 10, 18, 22, }, // Cuba
    {  340,  559,       0,  356778,       0,  0, 11,  0, }, // Davis
    {  340,  575,       0,  356789,       0,  0, 22,  0, }, // DumontDUrville
    {  340,  612,       0,  356811,       0,  0, 17,  0, }, // East_Timor
    {  340,  623,  356828,  356846,  356872, 18, 26, 30, }, // Easter
    {  340,  630,       0,  356902,       0,  0, 13,  0, }, // Ecuador
    {  340,  638,  356915,  356933,  356959, 18, 26, 30, }, // Europe_Central
    {  340,  653,  356989,  357006,  357031, 17, 25, 29, }, // Europe_Eastern
    {  340,  668,       0,  357060,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  340,  691,  357082,  357099,  357124, 17, 25, 29, }, // Europe_Western
    {  340,  706,  357153,  357177,  357209, 24, 32, 36, }, // Falkland
    {  340,  715,  357245,  357255,  357273, 10, 18, 22, }, // Fiji
    {  340,  720,       0,  357295,       0,  0, 20,  0, }, // French_Guiana
    {  340,  734,       0,  357315,       0,  0, 44,  0, }, // French_Southern
    {  340,  757,       0,  357359,       0,  0, 15,  0, }, // Galapagos
    {  340,  767,       0,  357374,       0,  0, 13,  0, }, // Gambier
    {  340,  775,  357387,  357400,  357421, 13, 21, 25, }, // Georgia
    {  340,  783,       0,  357446,       0,  0, 18,  0, }, // Gilbert_Islands
    {  340,  799,       0,    2861,       0,  0, 19,  0, }, // GMT
    {  340,  841,  357464,  357485,  357514, 21, 29, 33, }, // Greenland_Eastern
    {  340,  859,  357547,  357568,  357597, 21, 29, 33, }, // Greenland_Western
    {  340,  877,       0,  357630,       0,  0, 10,  0, }, // Guam
    {  340,  882,       0,  357640,       0,  0, 19,  0, }, // Gulf
    {  340,  887,       0,  357295,       0,  0, 12,  0, }, // Guyana
    {  340,  894,  357659,  357680,  357709, 21, 29, 33, }, // Hawaii_Aleutian
    {  340,  910,  357742,  357757,  357780, 15, 23, 27, }, // Hong_Kong
    {  340,  920,  357807,  357817,  357835, 10, 18, 22, }, // Hovd
    {  340,  925,       0,  357857,       0,  0, 11,  0, }, // India
    {  340,  931,       0,  357868,       0,  0, 21,  0, }, // Indian_Ocean
    {  340,  944,       0,  357889,       0,  0, 15,  0, }, // Indochina
    {  340,  954,       0,  357904,       0,  0, 22,  0, }, // Indonesia_Central
    {  340,  972,       0,  357926,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  340,  990,       0,  357947,       0,  0, 21,  0, }, // Indonesia_Western
    {  340, 1008,  357968,  357978,  357996, 10, 18, 22, }, // Iran
    {  340, 1019,  358018,  358031,  358052, 13, 21, 25, }, // Irkutsk
    {  340, 1027,  358077,  358089,  358109, 12, 20, 24, }, // Israel
    {  340, 1034,  358133,  358145,  358165, 12, 20, 24, }, // Japan
    {  340, 1040,  358189,  358219,  358257, 30, 38, 42, }, // Kamchatka
    {  340, 1058,       0,  358299,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  340, 1077,       0,  358321,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  340, 1106,  358343,  358354,  358373, 11, 19, 23, }, // Korea
    {  340, 1112,       0,  358396,       0,  0, 12,  0, }, // Kosrae
    {  340, 1119,  358408,  358425,  358450, 17, 25, 29, }, // Krasnoyarsk
    {  340, 1151,       0,  358479,       0,  0, 16,  0, }, // Kyrgystan
    {  340, 1161,       0,  358495,       0,  0, 11,  0, }, // Lanka
    {  340, 1175,       0,  358506,       0,  0, 15,  0, }, // Line_Islands
    {  340, 1188,  358521,  358536,  358559, 15, 23, 27, }, // Lord_Howe
    {  340, 1198,  358586,  358597,  358616, 11, 19, 23, }, // Macau
    {  340, 1204,  358639,  358652,  358673, 13, 21, 25, }, // Magadan
    {  340, 1219,       0,  358698,       0,  0, 14,  0, }, // Malaysia
    {  340, 1228,       0,  358712,       0,  0, 14,  0, }, // Maldives
    {  340, 1237,       0,  358726,       0,  0, 15,  0, }, // Marquesas
    {  340, 1247,       0,  358741,       0,  0, 19,  0, }, // Marshall_Islands
    {  340, 1264,  358760,  358775,  358798, 15, 23, 27, }, // Mauritius
    {  340, 1274,       0,  358825,       0,  0, 12,  0, }, // Mawson
    {  340, 1281,  358837,  358858,  358887, 21, 29, 33, }, // Mexico_Pacific
    {  340, 1296,  358920,  358936,  358960, 16, 24, 28, }, // Mongolia
    {  340, 1305,  358988,  359000,  359020, 12, 20, 24, }, // Moscow
    {  340, 1312,       0,  359044,       0,  0, 13,  0, }, // Myanmar
    {  340, 1320,       0,  359057,       0,  0, 11,  0, }, // Nauru
    {  340, 1326,       0,  359068,       0,  0, 11,  0, }, // Nepal
    {  340, 1332,  359079,  359099,  359127, 20, 28, 32, }, // New_Caledonia
    {  340, 1346,  359159,  359178,  359205, 19, 27, 31, }, // New_Zealand
    {  340, 1358,  359236,  359254,  359280, 18, 26, 30, }, // Newfoundland
    {  340, 1371,       0,  359310,       0,  0, 10,  0, }, // Niue
    {  340, 1376,  359320,  359339,  359366, 19, 27, 31, }, // Norfolk
    {  340, 1384,  359397,  359422,  359455, 25, 33, 37, }, // Noronha
    {  340, 1392,       0,  359492,       0,  0, 24,  0, }, // North_Mariana
    {  340, 1406,  359516,  359533,  359558, 17, 25, 29, }, // Novosibirsk
    {  340, 1418,  359587,  359597,  359615, 10, 18, 22, }, // Omsk
    {  340, 1428,  359637,  359651,  359673, 14, 22, 26, }, // Pakistan
    {  340, 1437,       0,  359699,       0,  0, 11,  0, }, // Palau
    {  340, 1443,       0,  359710,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  340, 1460,  359728,  359742,  359764, 14, 22, 26, }, // Paraguay
    {  340, 1469,  359790,  359800,  359818, 10, 18, 22, }, // Peru
    {  340, 1474,  359840,  359854,  359876, 14, 22, 26, }, // Philippines
    {  340, 1486,       0,  359902,       0,  0, 23,  0, }, // Phoenix_Islands
    {  340, 1502,  359925,  359956,  359995, 31, 39, 43, }, // Pierre_Miquelon
    {  340, 1518,       0,  360038,       0,  0, 14,  0, }, // Pitcairn
    {  340, 1527,       0,  360052,       0,  0, 12,  0, }, // Ponape
    {  340, 1534,       0,  360064,       0,  0, 15,  0, }, // Pyongyang
    {  340, 1544,  360079,  360094,  360117, 15, 23, 27, }, // Qyzylorda
    {  340, 1554,       0,  360144,       0,  0, 13,  0, }, // Reunion
    {  340, 1562,       0,  360157,       0,  0, 13,  0, }, // Rothera
    {  340, 1570,  360170,  360184,  360206, 14, 22, 26, }, // Sakhalin
    {  340, 1579,  360232,  360244,  360264, 12, 20, 24, }, // Samara
    {  340, 1596,  360288,  360299,  360318, 11, 19, 23, }, // Samoa
    {  340, 1602,       0,  360341,       0,  0, 16,  0, }, // Seychelles
    {  340, 1624,       0,  360357,       0,  0, 23,  0, }, // Singapore
    {  340, 1634,       0,  360380,       0,  0, 23,  0, }, // Solomon
    {  340, 1642,       0,  360403,       0,  0, 21,  0, }, // South_Georgia
    {  340, 1656,       0,  360424,       0,  0, 14,  0, }, // Suriname
    {  340, 1676,       0,  360438,       0,  0, 11,  0, }, // Syowa
    {  340, 1682,       0,  360449,       0,  0, 12,  0, }, // Tahiti
    {  340, 1689,  360461,  360473,  360493, 12, 20, 24, }, // Taipei
    {  340, 1696,       0,  360517,       0,  0, 16,  0, }, // Tajikistan
    {  340, 1724,       0,  360533,       0,  0, 13,  0, }, // Tokelau
    {  340, 1732,  360546,  360557,  360576, 11, 19, 23, }, // Tonga
    {  340, 1738,       0,  360599,       0,  0, 11,  0, }, // Truk
    {  340, 1750,  360610,  360628,  360654, 18, 26, 30, }, // Turkmenistan
    {  340, 1763,       0,  360684,       0,  0, 12,  0, }, // Tuvalu
    {  340, 1777,  360696,  360709,  360730, 13, 21, 25, }, // Uruguay
    {  340, 1792,  360755,  360771,  360795, 16, 24, 28, }, // Uzbekistan
    {  340, 1803,  360823,  360836,  360857, 13, 21, 25, }, // Vanuatu
    {  340, 1811,       0,  360882,       0,  0, 15,  0, }, // Venezuela
    {  340, 1821,  360897,  360914,  360939, 17, 25, 29, }, // Vladivostok
    {  340, 1833,  360968,  360983,  361006, 15, 23, 27, }, // Volgograd
    {  340, 1843,       0,  361033,       0,  0, 12,  0, }, // Vostok
    {  340, 1850,       0,  361045,       0,  0, 20,  0, }, // Wake
    {  340, 1855,       0,  361065,       0,  0, 23,  0, }, // Wallis
    {  340, 1862,  361088,  361101,  361122, 13, 21, 25, }, // Yakutsk
    {  340, 1870,  361147,  361166,  361193, 19, 27, 31, }, // Yekaterinburg
    {  340, 1892,       0,  361224,       0,  0, 11,  0, }, // Yukon
    {  341,    5,       0,  361235,       0,  0, 19,  0, }, // Afghanistan Interlingua/Latin/world
    {  341,   17,       0,  361254,       0,  0, 22,  0, }, // Africa_Central
    {  341,   32,       0,  361276,       0,  0, 22,  0, }, // Africa_Eastern
    {  341,   65,       0,  361298,       0,  0, 31,  0, }, // Africa_Southern
    {  341,   81,  361329,  361352,  361384, 23, 32, 30, }, // Africa_Western
    {  341,  107,  361414,  361428,  361449, 14, 21, 21, }, // Alaska
    {  341,  135,  361470,  361486,  361511, 16, 25, 23, }, // Amazon
    {  341,  142,   37394,  361534,  361553, 12, 19, 19, }, // America_Central
    {  341,  158,  361572,  361584,  361603, 12, 19, 19, }, // America_Eastern
    {  341,  174,  361622,  361640,  361665, 18, 25, 25, }, // America_Mountain
    {  341,  191,  361690,  361702,  361721, 12, 19, 19, }, // America_Pacific
    {  341,  214,  286164,  361740,  361761, 12, 21, 19, }, // Apia
    {  341,  232,  361780,  361790,  361809, 10, 19, 17, }, // Arabian
    {  341,  240,  361826,  361843,  361869, 17, 26, 24, }, // Argentina
    {  341,  250,  361893,  361921,  361958, 28, 37, 35, }, // Argentina_Western
    {  341,  268,  286437,  361993,  362017, 15, 24, 22, }, // Armenia
    {  341,  286,  362039,  362052,  362072, 13, 20, 20, }, // Atlantic
    {  341,  295,  362092,  362117,  362151, 25, 34, 32, }, // Australia_Central
    {  341,  313,  362183,  362218,  362262, 35, 44, 42, }, // Australia_CentralWestern
    {  341,  338,  362304,  362330,  362365, 26, 35, 33, }, // Australia_Eastern
    {  341,  356,  362398,  362426,  362463, 28, 37, 35, }, // Australia_Western
    {  341,  374,  362498,  362518,  362547, 20, 29, 27, }, // Azerbaijan
    {  341,  385,  362574,  362589,  362611, 15, 22, 22, }, // Azores
    {  341,  397,  287019,  362633,  362660, 18, 27, 25, }, // Bangladesh
    {  341,  415,       0,  362685,       0,  0, 14,  0, }, // Bhutan
    {  341,  422,       0,  287104,       0,  0, 15,  0, }, // Bolivia
    {  341,  437,  287119,  362699,  362724, 16, 25, 23, }, // Brasilia
    {  341,  454,       0,  287185,       0,  0, 25,  0, }, // Brunei
    {  341,  461,  362747,  362765,  362792, 18, 27, 25, }, // Cape_Verde
    {  341,  478,       0,  362817,       0,  0, 25,  0, }, // Chamorro
    {  341,  487,  287304,  362842,  362866, 15, 24, 22, }, // Chatham
    {  341,  495,  287367,  362888,  362910, 13, 22, 20, }, // Chile
    {  341,  501,  362930,  362943,  362965, 13, 22, 20, }, // China
    {  341,  507,  287481,  362985,  363012, 18, 27, 25, }, // Choibalsan
    {  341,  518,       0,  363037,       0,  0, 24,  0, }, // Christmas
    {  341,  528,       0,  363061,       0,  0, 22,  0, }, // Cocos
    {  341,  534,  287595,  363083,  363108, 16, 25, 23, }, // Colombia
    {  341,  543,  363131,  363152,  363182, 21, 30, 28, }, // Cook
    {  341,  548,  287742,  363210,  363229, 12, 19, 19, }, // Cuba
    {  341,  559,       0,  287796,       0,  0, 13,  0, }, // Davis
    {  341,  575,       0,  287809,       0,  0, 24,  0, }, // DumontDUrville
    {  341,  612,       0,  363248,       0,  0, 21,  0, }, // East_Timor
    {  341,  623,  363269,  363294,  363328, 25, 34, 32, }, // Easter
    {  341,  630,       0,  287936,       0,  0, 15,  0, }, // Ecuador
    {  341,  638,  363360,  363382,  363411, 22, 29, 29, }, // Europe_Central
    {  341,  653,  363440,  363463,  363493, 23, 30, 30, }, // Europe_Eastern
    {  341,  668,       0,  363523,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  341,  691,  363552,  363577,  363609, 25, 32, 32, }, // Europe_Western
    {  341,  706,  363641,  363666,  363700, 25, 34, 32, }, // Falkland
    {  341,  715,  363732,  363744,  363765, 12, 21, 19, }, // Fiji
    {  341,  720,       0,  363784,       0,  0, 23,  0, }, // French_Guiana
    {  341,  734,       0,  363807,       0,  0, 36,  0, }, // French_Southern
    {  341,  757,       0,  363843,       0,  0, 18,  0, }, // Galapagos
    {  341,  767,       0,  288473,       0,  0, 15,  0, }, // Gambier
    {  341,  775,  363861,  363876,  363900, 15, 24, 22, }, // Georgia
    {  341,  783,       0,  363922,       0,  0, 24,  0, }, // Gilbert_Islands
    {  341,  799,       0,  363946,       0,  0, 23,  0, }, // GMT
    {  341,  841,  363969,  363997,  364032, 28, 35, 35, }, // Greenland_Eastern
    {  341,  859,  364067,  364097,  364134, 30, 37, 37, }, // Greenland_Western
    {  341,  882,       0,  364171,       0,  0, 23,  0, }, // Gulf
    {  341,  887,       0,  364194,       0,  0, 14,  0, }, // Guyana
    {  341,  894,  364208,  364233,  364265, 25, 32, 32, }, // Hawaii_Aleutian
    {  341,  910,  364297,  364313,  364338, 16, 25, 23, }, // Hong_Kong
    {  341,  920,  289003,  364361,  364382, 12, 21, 19, }, // Hovd
    {  341,  925,       0,  364401,       0,  0, 22,  0, }, // India
    {  341,  931,       0,  364423,       0,  0, 22,  0, }, // Indian_Ocean
    {  341,  944,       0,  289091,       0,  0, 17,  0, }, // Indochina
    {  341,  954,       0,  289108,       0,  0, 25,  0, }, // Indonesia_Central
    {  341,  972,       0,  364445,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  341,  990,       0,  364470,       0,  0, 26,  0, }, // Indonesia_Western
    {  341, 1008,  364496,  364508,  364529, 12, 21, 19, }, // Iran
    {  341, 1019,  289241,  364548,  364570, 15, 22, 22, }, // Irkutsk
    {  341, 1027,  289304,  364592,  364615, 14, 23, 21, }, // Israel
    {  341, 1034,  364636,  364649,  364671, 13, 22, 20, }, // Japan
    {  341, 1058,       0,  364691,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  341, 1077,       0,  364717,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  341, 1106,  289600,  364744,  364766, 13, 22, 20, }, // Korea
    {  341, 1112,       0,  289657,       0,  0, 14,  0, }, // Kosrae
    {  341, 1119,  364786,  364805,  364831, 19, 26, 26, }, // Krasnoyarsk
    {  341, 1151,       0,  364857,       0,  0, 20,  0, }, // Kyrgystan
    {  341, 1175,       0,  364877,       0,  0, 26,  0, }, // Line_Islands
    {  341, 1188,  289788,  364903,  364929, 17, 26, 24, }, // Lord_Howe
    {  341, 1204,  289857,  364953,  364975, 15, 22, 22, }, // Magadan
    {  341, 1219,       0,  364997,       0,  0, 16,  0, }, // Malaysia
    {  341, 1228,       0,  365013,       0,  0, 17,  0, }, // Maldives
    {  341, 1237,       0,  365030,       0,  0, 17,  0, }, // Marquesas
    {  341, 1247,       0,  365047,       0,  0, 25,  0, }, // Marshall_Islands
    {  341, 1264,  365072,  365088,  365113, 16, 25, 23, }, // Mauritius
    {  341, 1274,       0,  290060,       0,  0, 14,  0, }, // Mawson
    {  341, 1281,  365136,  365161,  365193, 25, 32, 32, }, // Mexico_Pacific
    {  341, 1296,  365225,  365243,  365270, 18, 27, 25, }, // Mongolia
    {  341, 1305,  290242,  365295,  365317, 15, 22, 22, }, // Moscow
    {  341, 1312,       0,  290305,       0,  0, 15,  0, }, // Myanmar
    {  341, 1320,       0,  290320,       0,  0, 13,  0, }, // Nauru
    {  341, 1326,       0,  290333,       0,  0, 13,  0, }, // Nepal
    {  341, 1332,  365339,  365361,  365392, 22, 31, 29, }, // New_Caledonia
    {  341, 1346,  365421,  365441,  365470, 20, 29, 27, }, // New_Zealand
    {  341, 1358,  365497,  365514,  365538, 17, 24, 24, }, // Newfoundland
    {  341, 1371,       0,  290583,       0,  0, 12,  0, }, // Niue
    {  341, 1376,  365562,  365585,  365617, 23, 32, 30, }, // Norfolk
    {  341, 1384,  290673,  365647,  365683, 27, 36, 34, }, // Noronha
    {  341, 1406,  290772,  365717,  365743, 19, 26, 26, }, // Novosibirsk
    {  341, 1418,  290847,  365769,  365788, 12, 19, 19, }, // Omsk
    {  341, 1428,  365807,  365823,  365848, 16, 25, 23, }, // Pakistan
    {  341, 1437,       0,  290970,       0,  0, 13,  0, }, // Palau
    {  341, 1443,       0,  365871,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  341, 1460,  365896,  365912,  365937, 16, 25, 23, }, // Paraguay
    {  341, 1469,  365960,  365972,  365993, 12, 21, 19, }, // Peru
    {  341, 1474,  366012,  366032,  366061, 20, 29, 27, }, // Philippines
    {  341, 1486,       0,  366088,       0,  0, 24,  0, }, // Phoenix_Islands
    {  341, 1502,  366112,  366143,  366181, 31, 38, 38, }, // Pierre_Miquelon
    {  341, 1518,       0,  291331,       0,  0, 16,  0, }, // Pitcairn
    {  341, 1527,       0,  366219,       0,  0, 14,  0, }, // Ponape
    {  341, 1534,       0,   58535,       0,  0, 17,  0, }, // Pyongyang
    {  341, 1554,       0,  366233,       0,  0, 15,  0, }, // Reunion
    {  341, 1562,       0,  291377,       0,  0, 15,  0, }, // Rothera
    {  341, 1570,  366248,  366264,  366287, 16, 23, 23, }, // Sakhalin
    {  341, 1596,  291527,  366310,  366332, 13, 22, 20, }, // Samoa
    {  341, 1602,       0,  366352,       0,  0, 19,  0, }, // Seychelles
    {  341, 1624,       0,  366371,       0,  0, 26,  0, }, // Singapore
    {  341, 1634,       0,  366397,       0,  0, 24,  0, }, // Solomon
    {  341, 1642,       0,  366421,       0,  0, 23,  0, }, // South_Georgia
    {  341, 1656,       0,  291663,       0,  0, 16,  0, }, // Suriname
    {  341, 1676,       0,  291679,       0,  0, 13,  0, }, // Syowa
    {  341, 1682,       0,  366444,       0,  0, 14,  0, }, // Tahiti
    {  341, 1689,  291706,  366458,  366481, 14, 23, 21, }, // Taipei
    {  341, 1696,       0,  366502,       0,  0, 18,  0, }, // Tajikistan
    {  341, 1724,       0,  291785,       0,  0, 15,  0, }, // Tokelau
    {  341, 1732,  291800,  366520,  366542, 13, 22, 20, }, // Tonga
    {  341, 1738,       0,  291857,       0,  0, 13,  0, }, // Truk
    {  341, 1750,  366562,  366582,  366611, 20, 29, 27, }, // Turkmenistan
    {  341, 1763,       0,  291948,       0,  0, 14,  0, }, // Tuvalu
    {  341, 1777,  366638,  366653,  366677, 15, 24, 22, }, // Uruguay
    {  341, 1792,  366699,  366717,  366744, 18, 27, 25, }, // Uzbekistan
    {  341, 1803,  292097,  366769,  366793, 15, 24, 22, }, // Vanuatu
    {  341, 1811,       0,  292160,       0,  0, 17,  0, }, // Venezuela
    {  341, 1821,  292177,  366815,  366841, 19, 26, 26, }, // Vladivostok
    {  341, 1833,  292252,  366867,  366891, 17, 24, 24, }, // Volgograd
    {  341, 1843,       0,  292324,       0,  0, 14,  0, }, // Vostok
    {  341, 1850,       0,  366915,       0,  0, 20,  0, }, // Wake
    {  341, 1855,       0,  292355,       0,  0, 23,  0, }, // Wallis
    {  341, 1862,  366935,  366950,  366972, 15, 22, 22, }, // Yakutsk
    {  341, 1870,  366994,  367014,  367041, 20, 27, 27, }, // Yekaterinburg
    {  341, 1892,       0,  292522,       0,  0, 13,  0, }, // Yukon
    {  342,  107,  367068,  367082,  367105, 14, 23, 22, }, // Alaska Interlingue/Latin/Estonia
    {  342,  142,  367127,  367150,  367182, 23, 32, 31, }, // America_Central
    {  342,  158,  367213,  367237,  367270, 24, 33, 32, }, // America_Eastern
    {  342,  174,  367302,  367324,  367355, 22, 31, 30, }, // America_Mountain
    {  342,  191,  367385,  367408,  367440, 23, 32, 31, }, // America_Pacific
    {  342,  286,  367471,  367495,  367528, 24, 33, 32, }, // Atlantic
    {  342,  385,  367560,  367577,  367603, 17, 26, 25, }, // Azores
    {  342,  518,       0,  367628,       0,  0, 26,  0, }, // Christmas
    {  342,  528,       0,  367654,       0,  0, 24,  0, }, // Cocos
    {  342,  543,  367678,  367701,  367733, 23, 32, 31, }, // Cook
    {  342,  548,  367764,  367776,  367797, 12, 21, 20, }, // Cuba
    {  342,  638,  367817,  367839,  367870, 22, 31, 30, }, // Europe_Central
    {  342,  653,  367900,  367918,  367945, 18, 27, 26, }, // Europe_Eastern
    {  342,  691,  367971,  367990,  368018, 19, 28, 27, }, // Europe_Western
    {  342,  706,  368045,  368072,  368108, 27, 36, 35, }, // Falkland
    {  342,  783,       0,  368143,       0,  0, 26,  0, }, // Gilbert_Islands
    {  342,  799,       0,  368169,       0,  0, 27,  0, }, // GMT
    {  342,  841,  368196,  368225,  368263, 29, 38, 37, }, // Greenland_Eastern
    {  342,  859,  368300,  368331,  368371, 31, 40, 39, }, // Greenland_Western
    {  342,  894,  368410,  368435,  368469, 25, 34, 33, }, // Hawaii_Aleutian
    {  342, 1175,       0,  368502,       0,  0, 23,  0, }, // Line_Islands
    {  342, 1188,  368525,  368551,  368586, 26, 35, 34, }, // Lord_Howe
    {  342, 1237,       0,  368620,       0,  0, 20,  0, }, // Marquesas
    {  342, 1247,       0,  368640,       0,  0, 27,  0, }, // Marshall_Islands
    {  342, 1281,  368667,  368689,  368720, 22, 31, 30, }, // Mexico_Pacific
    {  342, 1358,  368750,  368772,  368803, 22, 31, 30, }, // Newfoundland
    {  342, 1376,  368833,  368857,  368890, 24, 33, 32, }, // Norfolk
    {  342, 1486,       0,  368922,       0,  0, 24,  0, }, // Phoenix_Islands
    {  342, 1634,       0,  368946,       0,  0, 26,  0, }, // Solomon
    {  342, 1850,       0,  368972,       0,  0, 21,  0, }, // Wake
    {  342, 1892,       0,  368993,       0,  0, 15,  0, }, // Yukon
    {  345,    0,  369008,  369015,  369035,  7, 20, 17, }, // Acre Irish/Latin/Ireland
    {  345,    5,       0,  369052,       0,  0, 19,  0, }, // Afghanistan
    {  345,   17,       0,  369071,       0,  0, 19,  0, }, // Africa_Central
    {  345,   32,       0,  369090,       0,  0, 23,  0, }, // Africa_Eastern
    {  345,   65,       0,  369113,       0,  0, 20,  0, }, // Africa_Southern
    {  345,   81,  369133,  369155,  369190, 22, 35, 32, }, // Africa_Western
    {  345,  107,  369222,  369231,  369253,  9, 22, 19, }, // Alaska
    {  345,  128,  369272,  369281,  369303,  9, 22, 19, }, // Almaty
    {  345,  135,  369322,  369337,  369365, 15, 28, 25, }, // Amazon
    {  345,  142,  369390,  369418,  369459, 28, 41, 38, }, // America_Central
    {  345,  158,  369497,  369529,  369574, 32, 45, 42, }, // America_Eastern
    {  345,  174,  369616,  369645,  369687, 29, 42, 39, }, // America_Mountain
    {  345,  191,  369726,  369745,  369777, 19, 32, 29, }, // America_Pacific
    {  345,  207,  369806,  369815,  369837,  9, 22, 19, }, // Anadyr
    {  345,  214,  369856,  369863,  369883,  7, 20, 17, }, // Apia
    {  345,  219,  369900,  369908,  369929,  8, 21, 18, }, // Aqtau
    {  345,  225,  369947,  369956,  369978,  9, 22, 19, }, // Aqtobe
    {  345,  232,  369997,  370010,  370036, 13, 26, 23, }, // Arabian
    {  345,  240,  370059,  370076,  370106, 17, 30, 27, }, // Argentina
    {  345,  250,  370133,  370161,  370202, 28, 41, 38, }, // Argentina_Western
    {  345,  268,  370240,  370255,  370283, 15, 28, 25, }, // Armenia
    {  345,  286,  370308,  370324,  370353, 16, 29, 26, }, // Atlantic
    {  345,  295,  370379,  370398,  370430, 19, 32, 29, }, // Australia_Central
    {  345,  313,  370459,  370487,  370528, 28, 41, 38, }, // Australia_CentralWestern
    {  345,  338,  370566,  370590,  370627, 24, 37, 34, }, // Australia_Eastern
    {  345,  356,  370661,  370684,  370720, 23, 36, 33, }, // Australia_Western
    {  345,  374,  370753,  370773,  370806, 20, 33, 30, }, // Azerbaijan
    {  345,  385,  370836,  370847,  370871, 11, 24, 21, }, // Azores
    {  345,  397,  370892,  370910,  370941, 18, 31, 28, }, // Bangladesh
    {  345,  415,       0,  370969,       0,  0, 13,  0, }, // Bhutan
    {  345,  422,       0,  370982,       0,  0, 13,  0, }, // Bolivia
    {  345,  437,  370995,  371007,  371032, 12, 25, 22, }, // Brasilia
    {  345,  454,       0,  371054,       0,  0, 20,  0, }, // Brunei
    {  345,  461,  371074,  371087,  371113, 13, 26, 23, }, // Cape_Verde
    {  345,  472,       0,  371136,       0,  0, 17,  0, }, // Casey
    {  345,  478,       0,  371153,       0,  0, 25,  0, }, // Chamorro
    {  345,  487,  371178,  371188,  371211, 10, 23, 20, }, // Chatham
    {  345,  495,  371231,  371241,  371264, 10, 23, 20, }, // Chile
    {  345,  501,  371284,  371294,  371317, 10, 23, 20, }, // China
    {  345,  507,  371337,  371350,  371376, 13, 26, 23, }, // Choibalsan
    {  345,  518,       0,  371399,       0,  0, 19,  0, }, // Christmas
    {  345,  528,       0,  371418,       0,  0, 16,  0, }, // Cocos
    {  345,  534,  371434,  371447,  371473, 13, 26, 23, }, // Colombia
    {  345,  543,  371496,  371511,  371539, 15, 28, 31, }, // Cook
    {  345,  548,  371570,  371578,  371599,  8, 21, 18, }, // Cuba
    {  345,  559,       0,  371617,       0,  0,  8,  0, }, // Davis
    {  345,  575,       0,  371625,       0,  0, 19,  0, }, // DumontDUrville
    {  345,  612,       0,  371644,       0,  0, 16,  0, }, // East_Timor
    {  345,  623,  371660,  371678,  371709, 18, 31, 28, }, // Easter
    {  345,  630,       0,  371737,       0,  0, 11,  0, }, // Ecuador
    {  345,  638,  371748,  371764,  371793, 16, 29, 26, }, // Europe_Central
    {  345,  653,  371819,  371840,  371874, 21, 34, 31, }, // Europe_Eastern
    {  345,  668,       0,  371905,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  345,  691,  371931,  371951,  371984, 20, 33, 30, }, // Europe_Western
    {  345,  706,  372014,  372035,  372069, 21, 34, 31, }, // Falkland
    {  345,  715,  372100,  372109,  372131,  9, 22, 19, }, // Fiji
    {  345,  720,       0,  372150,       0,  0, 20,  0, }, // French_Guiana
    {  345,  734,       0,  372170,       0,  0, 52,  0, }, // French_Southern
    {  345,  757,       0,  372222,       0,  0, 20,  0, }, // Galapagos
    {  345,  767,       0,  372242,       0,  0, 10,  0, }, // Gambier
    {  345,  775,  372252,  372266,  372293, 14, 27, 24, }, // Georgia
    {  345,  783,       0,  372317,       0,  0, 14,  0, }, // Gilbert_Islands
    {  345,  799,       0,  372331,       0,  0, 17,  0, }, // GMT
    {  345,  841,  372348,  372374,  372413, 26, 39, 36, }, // Greenland_Eastern
    {  345,  859,  372449,  372474,  372512, 25, 38, 35, }, // Greenland_Western
    {  345,  877,       0,  372547,       0,  0, 21,  0, }, // Guam
    {  345,  882,       0,  372568,       0,  0, 30,  0, }, // Gulf
    {  345,  887,       0,  372598,       0,  0, 12,  0, }, // Guyana
    {  345,  894,  372610,  372626,  372655, 16, 29, 26, }, // Hawaii_Aleutian
    {  345,  910,  372681,  372693,  372718, 12, 25, 22, }, // Hong_Kong
    {  345,  920,  372740,  372747,  372767,  7, 20, 17, }, // Hovd
    {  345,  925,       0,  372784,       0,  0, 25,  0, }, // India
    {  345,  931,       0,  372809,       0,  0, 21,  0, }, // Indian_Ocean
    {  345,  944,       0,  372830,       0,  0, 15,  0, }, // Indochina
    {  345,  954,       0,  372845,       0,  0, 20,  0, }, // Indonesia_Central
    {  345,  972,       0,  372865,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  345,  990,       0,  372890,       0,  0, 24,  0, }, // Indonesia_Western
    {  345, 1008,  372914,  372928,  372955, 14, 27, 24, }, // Iran
    {  345, 1019,  372979,  372989,  373012, 10, 23, 20, }, // Irkutsk
    {  345, 1027,  373032,  373042,  373065, 10, 23, 20, }, // Israel
    {  345, 1034,  373085,  373099,  373126, 14, 27, 24, }, // Japan
    {  345, 1040,  373150,  373178,  373219, 28, 41, 38, }, // Kamchatka
    {  345, 1058,       0,  373257,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  345, 1077,       0,  373283,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  345, 1106,  373308,  373319,  373343, 11, 24, 21, }, // Korea
    {  345, 1112,       0,  373364,       0,  0,  9,  0, }, // Kosrae
    {  345, 1119,  373373,  373387,  373414, 14, 27, 24, }, // Krasnoyarsk
    {  345, 1151,       0,  373438,       0,  0, 18,  0, }, // Kyrgystan
    {  345, 1161,       0,  373456,       0,  0, 13,  0, }, // Lanka
    {  345, 1175,       0,  373469,       0,  0, 18,  0, }, // Line_Islands
    {  345, 1188,  373487,  373499,  373524, 12, 25, 22, }, // Lord_Howe
    {  345, 1198,  373546,  373555,  373577,  9, 22, 19, }, // Macau
    {  345, 1204,  373596,  373607,  373631, 11, 24, 21, }, // Magadan
    {  345, 1219,       0,  373652,       0,  0, 15,  0, }, // Malaysia
    {  345, 1228,       0,  373667,       0,  0, 20,  0, }, // Maldives
    {  345, 1237,       0,  373687,       0,  0, 23,  0, }, // Marquesas
    {  345, 1247,       0,  373710,       0,  0, 19,  0, }, // Marshall_Islands
    {  345, 1264,  373729,  373746,  373776, 17, 30, 27, }, // Mauritius
    {  345, 1274,       0,  373803,       0,  0,  9,  0, }, // Mawson
    {  345, 1281,  373812,  373843,  373887, 31, 44, 41, }, // Mexico_Pacific
    {  345, 1296,  373928,  373940,  373965, 12, 25, 22, }, // Mongolia
    {  345, 1305,  373987,  373996,  374018,  9, 22, 19, }, // Moscow
    {  345, 1312,       0,  374037,       0,  0, 11,  0, }, // Myanmar
    {  345, 1320,       0,  374048,       0,  0,  7,  0, }, // Nauru
    {  345, 1326,       0,  374055,       0,  0, 10,  0, }, // Nepal
    {  345, 1332,  374065,  374085,  374118, 20, 33, 30, }, // New_Caledonia
    {  345, 1346,  374148,  374168,  374201, 20, 33, 30, }, // New_Zealand
    {  345, 1358,  374231,  374249,  374280, 18, 31, 28, }, // Newfoundland
    {  345, 1371,       0,  374308,       0,  0,  7,  0, }, // Niue
    {  345, 1376,  374315,  374332,  374362, 17, 30, 27, }, // Norfolk
    {  345, 1384,  374389,  374411,  374446, 22, 35, 32, }, // Noronha
    {  345, 1392,       0,  374478,       0,  0, 32,  0, }, // North_Mariana
    {  345, 1406,  374510,  374524,  374551, 14, 27, 24, }, // Novosibirsk
    {  345, 1418,  374575,  374582,  374602,  7, 20, 17, }, // Omsk
    {  345, 1428,  374619,  374635,  374664, 16, 29, 26, }, // Pakistan
    {  345, 1437,       0,  374690,       0,  0, 16,  0, }, // Palau
    {  345, 1443,       0,  374706,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  345, 1460,  374726,  374737,  374761, 11, 24, 21, }, // Paraguay
    {  345, 1469,  374782,  374792,  374815, 10, 23, 20, }, // Peru
    {  345, 1474,  374835,  374860,  374898, 25, 38, 35, }, // Philippines
    {  345, 1486,       0,  374933,       0,  0, 22,  0, }, // Phoenix_Islands
    {  345, 1502,  374955,  374982,  375022, 27, 40, 37, }, // Pierre_Miquelon
    {  345, 1518,       0,  375059,       0,  0, 11,  0, }, // Pitcairn
    {  345, 1527,       0,  375070,       0,  0, 11,  0, }, // Ponape
    {  345, 1534,       0,  375081,       0,  0, 12,  0, }, // Pyongyang
    {  345, 1544,  375093,  375105,  375130, 12, 25, 22, }, // Qyzylorda
    {  345, 1554,       0,  375152,       0,  0, 10,  0, }, // Reunion
    {  345, 1562,       0,  375162,       0,  0, 10,  0, }, // Rothera
    {  345, 1570,  375172,  375184,  375209, 12, 25, 22, }, // Sakhalin
    {  345, 1579,  375231,  375241,  375264, 10, 23, 20, }, // Samara
    {  345, 1596,  375284,  375292,  375313,  8, 21, 18, }, // Samoa
    {  345, 1602,       0,  375331,       0,  0, 13,  0, }, // Seychelles
    {  345, 1624,       0,  375344,       0,  0, 26,  0, }, // Singapore
    {  345, 1634,       0,  375370,       0,  0, 19,  0, }, // Solomon
    {  345, 1642,       0,  375389,       0,  0, 20,  0, }, // South_Georgia
    {  345, 1656,       0,  375409,       0,  0, 11,  0, }, // Suriname
    {  345, 1676,       0,  375420,       0,  0,  8,  0, }, // Syowa
    {  345, 1682,       0,  375428,       0,  0, 11,  0, }, // Tahiti
    {  345, 1689,  375439,  375448,  375470,  9, 22, 19, }, // Taipei
    {  345, 1696,       0,  375489,       0,  0, 21,  0, }, // Tajikistan
    {  345, 1724,       0,  375510,       0,  0, 17,  0, }, // Tokelau
    {  345, 1732,  375527,  375536,  375558,  9, 22, 19, }, // Tonga
    {  345, 1738,       0,  375577,       0,  0,  8,  0, }, // Truk
    {  345, 1750,  375585,  375607,  375642, 22, 35, 32, }, // Turkmenistan
    {  345, 1763,       0,  375674,       0,  0, 10,  0, }, // Tuvalu
    {  345, 1777,  375684,  375693,  375715,  9, 22, 19, }, // Uruguay
    {  345, 1792,  375734,  375756,  375791, 22, 35, 32, }, // Uzbekistan
    {  345, 1803,  375823,  375833,  375856, 10, 23, 20, }, // Vanuatu
    {  345, 1811,       0,  375876,       0,  0, 13,  0, }, // Venezuela
    {  345, 1821,  375889,  375903,  375930, 14, 27, 24, }, // Vladivostok
    {  345, 1833,  375954,  375966,  375991, 12, 25, 22, }, // Volgograd
    {  345, 1843,       0,  376013,       0,  0,  9,  0, }, // Vostok
    {  345, 1850,       0,  376022,       0,  0, 14,  0, }, // Wake
    {  345, 1855,       0,  376036,       0,  0, 21,  0, }, // Wallis
    {  345, 1862,  376057,  376067,  376090, 10, 23, 20, }, // Yakutsk
    {  345, 1870,  376110,  376126,  376155, 16, 29, 26, }, // Yekaterinburg
    {  345, 1892,       0,  376181,       0,  0,  8,  0, }, // Yukon
    {  347,    5,       0,  376189,       0,  0, 20,  0, }, // Afghanistan Italian/Latin/Italy
    {  347,   17,       0,  376209,       0,  0, 24,  0, }, // Africa_Central
    {  347,   32,       0,  376233,       0,  0, 25,  0, }, // Africa_Eastern
    {  347,   65,       0,  376258,       0,  0, 27,  0, }, // Africa_Southern
    {  347,   81,  376285,  376312,  376348, 27, 36, 34, }, // Africa_Western
    {  347,  107,  376382,  376397,  376421, 15, 24, 22, }, // Alaska
    {  347,  135,  376443,  376461,  376488, 18, 27, 25, }, // Amazon
    {  347,  142,  376513,  376529,  376554, 16, 25, 23, }, // America_Central
    {  347,  158,  376577,  376594,  376620, 17, 26, 24, }, // America_Eastern
    {  347,  174,  376644,  376669,  376703, 25, 34, 32, }, // America_Mountain
    {  347,  191,  376735,  376755,  376784, 20, 29, 27, }, // America_Pacific
    {  347,  207,  376811,  376824,  376846, 13, 22, 20, }, // Anadyr
    {  347,  214,  376866,  376877,  376897, 11, 20, 18, }, // Apia
    {  347,  232,  376915,  376924,  376942,  9, 18, 16, }, // Arabian
    {  347,  240,  376958,  376976,  377003, 18, 27, 25, }, // Argentina
    {  347,  250,  377028,  377058,  377097, 30, 39, 37, }, // Argentina_Western
    {  347,  268,  377134,  377150,  377175, 16, 25, 23, }, // Armenia
    {  347,  286,  377198,  377216,  377243, 18, 27, 25, }, // Atlantic
    {  347,  295,  377268,  377295,  377331, 27, 36, 34, }, // Australia_Central
    {  347,  313,  377365,  377400,  377444, 35, 44, 42, }, // Australia_CentralWestern
    {  347,  338,  377486,  377514,  377551, 28, 37, 35, }, // Australia_Eastern
    {  347,  356,  377586,  377616,  377655, 30, 39, 37, }, // Australia_Western
    {  347,  374,  377692,  377712,  377741, 20, 29, 27, }, // Azerbaijan
    {  347,  385,  377768,  377785,  377811, 17, 26, 24, }, // Azores
    {  347,  397,  377835,  377853,  377880, 18, 27, 25, }, // Bangladesh
    {  347,  415,       0,  377905,       0,  0, 14,  0, }, // Bhutan
    {  347,  422,       0,  377919,       0,  0, 17,  0, }, // Bolivia
    {  347,  437,  377936,  377951,  377975, 15, 24, 22, }, // Brasilia
    {  347,  454,       0,  377997,       0,  0, 25,  0, }, // Brunei
    {  347,  461,  378022,  378039,  378065, 17, 26, 24, }, // Cape_Verde
    {  347,  478,       0,  378089,       0,  0, 15,  0, }, // Chamorro
    {  347,  487,  378104,  378121,  378147, 17, 26, 24, }, // Chatham
    {  347,  495,  378171,  378183,  378204, 12, 21, 19, }, // Chile
    {  347,  501,  378223,  378237,  378260, 14, 23, 21, }, // China
    {  347,  507,  378281,  378298,  378324, 17, 26, 24, }, // Choibalsan
    {  347,  518,       0,  378348,       0,  0, 24,  0, }, // Christmas
    {  347,  528,       0,  378372,       0,  0, 21,  0, }, // Cocos
    {  347,  534,  378393,  378411,  378438, 18, 27, 25, }, // Colombia
    {  347,  543,  378463,  378483,  378512, 20, 29, 33, }, // Cook
    {  347,  548,  378545,  378556,  378576, 11, 20, 18, }, // Cuba
    {  347,  559,       0,  378594,       0,  0, 12,  0, }, // Davis
    {  347,  575,       0,  378606,       0,  0, 23,  0, }, // DumontDUrville
    {  347,  612,       0,  378629,       0,  0, 16,  0, }, // East_Timor
    {  347,  623,  378645,  378669,  378702, 24, 33, 31, }, // Easter
    {  347,  630,       0,  378733,       0,  0, 16,  0, }, // Ecuador
    {  347,  638,  378749,  378773,  378806, 24, 33, 31, }, // Europe_Central
    {  347,  653,  378837,  378862,  378896, 25, 34, 32, }, // Europe_Eastern
    {  347,  668,       0,  378928,       0,  0, 39,  0, }, // Europe_Further_Eastern
    {  347,  691,  378967,  378994,  379030, 27, 36, 34, }, // Europe_Western
    {  347,  706,  379064,  379088,  379121, 24, 33, 31, }, // Falkland
    {  347,  715,  379152,  379166,  379189, 14, 23, 21, }, // Fiji
    {  347,  720,       0,  379210,       0,  0, 25,  0, }, // French_Guiana
    {  347,  734,       0,  379235,       0,  0, 46,  0, }, // French_Southern
    {  347,  757,       0,  379281,       0,  0, 19,  0, }, // Galapagos
    {  347,  767,       0,  379300,       0,  0, 14,  0, }, // Gambier
    {  347,  775,  379314,  379331,  379357, 17, 26, 24, }, // Georgia
    {  347,  783,       0,  379381,       0,  0, 23,  0, }, // Gilbert_Islands
    {  347,  799,       0,  379404,       0,  0, 30,  0, }, // GMT
    {  347,  841,  379434,  379465,  379505, 31, 40, 38, }, // Greenland_Eastern
    {  347,  859,  379543,  379576,  379618, 33, 42, 40, }, // Greenland_Western
    {  347,  882,       0,  379658,       0,  0, 13,  0, }, // Gulf
    {  347,  887,       0,  379671,       0,  0, 16,  0, }, // Guyana
    {  347,  894,  379687,  379718,  379758, 31, 40, 38, }, // Hawaii_Aleutian
    {  347,  910,  379796,  379812,  379837, 16, 25, 23, }, // Hong_Kong
    {  347,  920,  379860,  379871,  379891, 11, 20, 18, }, // Hovd
    {  347,  925,       0,  379909,       0,  0, 23,  0, }, // India
    {  347,  931,       0,  379932,       0,  0, 23,  0, }, // Indian_Ocean
    {  347,  944,       0,  379955,       0,  0, 17,  0, }, // Indochina
    {  347,  954,       0,  379972,       0,  0, 27,  0, }, // Indonesia_Central
    {  347,  972,       0,  379999,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  347,  990,       0,  380027,       0,  0, 30,  0, }, // Indonesia_Western
    {  347, 1008,  380057,  380070,  380092, 13, 22, 20, }, // Iran
    {  347, 1019,  380112,  380126,  380149, 14, 23, 21, }, // Irkutsk
    {  347, 1027,  380170,  380184,  380207, 14, 23, 21, }, // Israel
    {  347, 1034,  380228,  380244,  380269, 16, 25, 23, }, // Japan
    {  347, 1040,  380292,  380323,  380363, 31, 40, 38, }, // Kamchatka
    {  347, 1058,       0,  380401,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  347, 1077,       0,  380429,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  347, 1106,  380459,  380470,  380490, 11, 20, 18, }, // Korea
    {  347, 1112,       0,  380508,       0,  0, 14,  0, }, // Kosrae
    {  347, 1119,  380522,  380540,  380567, 18, 27, 25, }, // Krasnoyarsk
    {  347, 1151,       0,  380592,       0,  0, 20,  0, }, // Kyrgystan
    {  347, 1175,       0,  380612,       0,  0, 29,  0, }, // Line_Islands
    {  347, 1188,  380641,  380657,  380682, 16, 25, 23, }, // Lord_Howe
    {  347, 1198,  380705,  380717,  380738, 12, 21, 19, }, // Macau
    {  347, 1204,  380757,  380771,  380794, 14, 23, 21, }, // Magadan
    {  347, 1219,       0,  380815,       0,  0, 17,  0, }, // Malaysia
    {  347, 1228,       0,  380832,       0,  0, 17,  0, }, // Maldives
    {  347, 1237,       0,  380849,       0,  0, 18,  0, }, // Marquesas
    {  347, 1247,       0,  380867,       0,  0, 24,  0, }, // Marshall_Islands
    {  347, 1264,  380891,  380910,  380938, 19, 28, 26, }, // Mauritius
    {  347, 1274,       0,  380964,       0,  0, 13,  0, }, // Mawson
    {  347, 1281,  380977,  381003,  381038, 26, 35, 33, }, // Mexico_Pacific
    {  347, 1296,  381071,  381088,  381114, 17, 26, 24, }, // Mongolia
    {  347, 1305,  381138,  381150,  381171, 12, 21, 19, }, // Moscow
    {  347, 1312,       0,  381190,       0,  0, 18,  0, }, // Myanmar
    {  347, 1320,       0,  381208,       0,  0, 12,  0, }, // Nauru
    {  347, 1326,       0,  381220,       0,  0, 13,  0, }, // Nepal
    {  347, 1332,  381233,  381258,  381292, 25, 34, 32, }, // New_Caledonia
    {  347, 1346,  381324,  381347,  381379, 23, 32, 30, }, // New_Zealand
    {  347, 1358,  381409,  381425,  381450, 16, 25, 23, }, // Newfoundland
    {  347, 1371,       0,  381473,       0,  0, 11,  0, }, // Niue
    {  347, 1376,  381484,  381507,  381539, 23, 32, 30, }, // Norfolk
    {  347, 1384,  381569,  381595,  381630, 26, 35, 33, }, // Noronha
    {  347, 1406,  381663,  381681,  381708, 18, 27, 25, }, // Novosibirsk
    {  347, 1418,  381733,  381744,  381764, 11, 20, 18, }, // Omsk
    {  347, 1428,  381782,  381798,  381823, 16, 25, 23, }, // Pakistan
    {  347, 1437,       0,  381846,       0,  0, 12,  0, }, // Palau
    {  347, 1443,       0,  381858,       0,  0, 28,  0, }, // Papua_New_Guinea
    {  347, 1460,  381886,  381902,  381927, 16, 25, 23, }, // Paraguay
    {  347, 1469,  381950,  381962,  381983, 12, 21, 19, }, // Peru
    {  347, 1474,  382002,  382021,  382049, 19, 28, 26, }, // Philippines
    {  347, 1486,       0,  382075,       0,  0, 28,  0, }, // Phoenix_Islands
    {  347, 1502,  382103,  382133,  382172, 30, 39, 37, }, // Pierre_Miquelon
    {  347, 1518,       0,  382209,       0,  0, 18,  0, }, // Pitcairn
    {  347, 1527,       0,  382227,       0,  0, 14,  0, }, // Ponape
    {  347, 1534,       0,  382241,       0,  0, 16,  0, }, // Pyongyang
    {  347, 1554,       0,  382257,       0,  0, 15,  0, }, // Reunion
    {  347, 1562,       0,  382272,       0,  0, 14,  0, }, // Rothera
    {  347, 1570,  382286,  382301,  382325, 15, 24, 22, }, // Sakhalin
    {  347, 1579,  382347,  382360,  382382, 13, 22, 20, }, // Samara
    {  347, 1596,  382402,  382414,  382435, 12, 21, 19, }, // Samoa
    {  347, 1602,       0,  382454,       0,  0, 20,  0, }, // Seychelles
    {  347, 1624,       0,  382474,       0,  0, 16,  0, }, // Singapore
    {  347, 1634,       0,  382490,       0,  0, 24,  0, }, // Solomon
    {  347, 1642,       0,  382514,       0,  0, 25,  0, }, // South_Georgia
    {  347, 1656,       0,  382539,       0,  0, 16,  0, }, // Suriname
    {  347, 1676,       0,  382555,       0,  0, 12,  0, }, // Syowa
    {  347, 1682,       0,  382567,       0,  0, 13,  0, }, // Tahiti
    {  347, 1689,  382580,  382593,  382615, 13, 22, 20, }, // Taipei
    {  347, 1696,       0,  382635,       0,  0, 18,  0, }, // Tajikistan
    {  347, 1724,       0,  382653,       0,  0, 14,  0, }, // Tokelau
    {  347, 1732,  382667,  382679,  382700, 12, 21, 19, }, // Tonga
    {  347, 1738,       0,  382719,       0,  0, 13,  0, }, // Truk
    {  347, 1750,  382732,  382752,  382781, 20, 29, 27, }, // Turkmenistan
    {  347, 1763,       0,  382808,       0,  0, 13,  0, }, // Tuvalu
    {  347, 1777,  382821,  382837,  382862, 16, 25, 23, }, // Uruguay
    {  347, 1792,  382885,  382904,  382932, 19, 28, 26, }, // Uzbekistan
    {  347, 1803,  382958,  382973,  382997, 15, 24, 22, }, // Vanuatu
    {  347, 1811,       0,  383019,       0,  0, 17,  0, }, // Venezuela
    {  347, 1821,  383036,  383054,  383081, 18, 27, 25, }, // Vladivostok
    {  347, 1833,  383106,  383122,  383147, 16, 25, 23, }, // Volgograd
    {  347, 1843,       0,  383170,       0,  0, 13,  0, }, // Vostok
    {  347, 1850,       0,  383183,       0,  0, 22,  0, }, // Wake
    {  347, 1855,       0,  383205,       0,  0, 22,  0, }, // Wallis
    {  347, 1862,  383227,  383241,  383264, 14, 23, 21, }, // Yakutsk
    {  347, 1870,  383285,  383304,  383332, 19, 28, 26, }, // Yekaterinburg
    {  347, 1892,       0,  383358,       0,  0, 15,  0, }, // Yukon
    {  351,    0,  383373,  383378,  383384,  5,  6,  6, }, // Acre Japanese/Japanese/Japan
    {  351,    5,       0,  383390,       0,  0,  9,  0, }, // Afghanistan
    {  351,   17,       0,  383399,       0,  0,  8,  0, }, // Africa_Central
    {  351,   32,       0,  383407,       0,  0,  7,  0, }, // Africa_Eastern
    {  351,   65,       0,  383414,       0,  0,  8,  0, }, // Africa_Southern
    {  351,   81,  383422,  383429,  383437,  7,  8,  8, }, // Africa_Western
    {  351,  107,  383445,  383451,  383458,  6,  7,  7, }, // Alaska
    {  351,  128,  383465,  383472,  383480,  7,  8,  8, }, // Almaty
    {  351,  135,  383488,  383494,  383501,  6,  7,  7, }, // Amazon
    {  351,  142,  383508,  383516,  383525,  8,  9,  9, }, // America_Central
    {  351,  158,  383534,  383542,  383551,  8,  9,  9, }, // America_Eastern
    {  351,  174,  383560,  383568,  383577,  8,  9,  9, }, // America_Mountain
    {  351,  191,  383586,  383595,  383605,  9, 10, 10, }, // America_Pacific
    {  351,  207,  383615,  383622,  383630,  7,  8,  8, }, // Anadyr
    {  351,  214,  383638,  383643,  383649,  5,  6,  6, }, // Apia
    {  351,  219,  383655,  383661,  383668,  6,  7,  7, }, // Aqtau
    {  351,  225,  383675,  383681,  383688,  6,  7,  7, }, // Aqtobe
    {  351,  232,  383695,  383701,  383708,  6,  7,  7, }, // Arabian
    {  351,  240,  383715,  383723,  383732,  8,  9,  9, }, // Argentina
    {  351,  250,  383741,  383751,  383762, 10, 11, 11, }, // Argentina_Western
    {  351,  268,  383773,  383780,  383788,  7,  8,  8, }, // Armenia
    {  351,  286,  128987,  128992,  383796,  5,  6,  6, }, // Atlantic
    {  351,  295,  383802,  383813,  383825, 11, 12, 12, }, // Australia_Central
    {  351,  313,  383837,  383849,  383862, 12, 13, 13, }, // Australia_CentralWestern
    {  351,  338,  383875,  383886,  383898, 11, 12, 12, }, // Australia_Eastern
    {  351,  356,  383910,  383921,  383933, 11, 12, 12, }, // Australia_Western
    {  351,  374,  383945,  383955,  383966, 10, 11, 11, }, // Azerbaijan
    {  351,  385,  383977,  383983,  383990,  6,  7,  7, }, // Azores
    {  351,  397,  383997,  384006,  384016,  9, 10, 10, }, // Bangladesh
    {  351,  415,       0,  384026,       0,  0,  6,  0, }, // Bhutan
    {  351,  422,       0,  384032,       0,  0,  6,  0, }, // Bolivia
    {  351,  437,  384038,  384045,  384053,  7,  8,  8, }, // Brasilia
    {  351,  454,       0,  384061,       0,  0, 13,  0, }, // Brunei
    {  351,  461,  384074,  384082,  384091,  8,  9,  9, }, // Cape_Verde
    {  351,  472,       0,  384100,       0,  0,  8,  0, }, // Casey
    {  351,  478,       0,  384108,       0,  0,  6,  0, }, // Chamorro
    {  351,  487,  384114,  384120,  384127,  6,  7,  7, }, // Chatham
    {  351,  495,  384134,  384138,  384143,  4,  5,  5, }, // Chile
    {  351,  501,  384148,  384152,  384157,  4,  5,  5, }, // China
    {  351,  507,  384162,  384171,  384181,  9, 10, 10, }, // Choibalsan
    {  351,  518,       0,  384191,       0,  0,  8,  0, }, // Christmas
    {  351,  528,       0,  384199,       0,  0,  7,  0, }, // Cocos
    {  351,  534,  384206,  384213,  384221,  7,  8,  8, }, // Colombia
    {  351,  543,  384229,  384236,  384244,  7,  8,  8, }, // Cook
    {  351,  548,  384252,  384258,  384265,  6,  7,  7, }, // Cuba
    {  351,  559,       0,  384272,       0,  0,  8,  0, }, // Davis
    {  351,  575,       0,  384280,       0,  0, 15,  0, }, // DumontDUrville
    {  351,  612,       0,  384295,       0,  0,  8,  0, }, // East_Timor
    {  351,  623,  384303,  384311,  384320,  8,  9,  9, }, // Easter
    {  351,  630,       0,  384329,       0,  0,  7,  0, }, // Ecuador
    {  351,  638,  384336,  384345,  384355,  9, 10, 10, }, // Europe_Central
    {  351,  653,  384365,  384373,  384382,  8,  9,  9, }, // Europe_Eastern
    {  351,  668,       0,  384391,       0,  0,  9,  0, }, // Europe_Further_Eastern
    {  351,  691,  384400,  384408,  384417,  8,  9,  9, }, // Europe_Western
    {  351,  706,  384426,  384437,  384449, 11, 12, 12, }, // Falkland
    {  351,  715,  384461,  384467,  384474,  6,  7,  7, }, // Fiji
    {  351,  720,       0,  384481,       0,  0,  7,  0, }, // French_Guiana
    {  351,  734,       0,  384488,       0,  0,  8,  0, }, // French_Southern
    {  351,  757,       0,  384496,       0,  0,  7,  0, }, // Galapagos
    {  351,  767,       0,  384503,       0,  0,  8,  0, }, // Gambier
    {  351,  775,  384511,  384518,  384526,  7,  8,  8, }, // Georgia
    {  351,  783,       0,  384534,       0,  0,  9,  0, }, // Gilbert_Islands
    {  351,  799,       0,  384543,       0,  0,  8,  0, }, // GMT
    {  351,  841,  384551,  384562,  384574, 11, 12, 12, }, // Greenland_Eastern
    {  351,  859,  384586,  384597,  384609, 11, 12, 12, }, // Greenland_Western
    {  351,  877,       0,  384621,       0,  0,  5,  0, }, // Guam
    {  351,  882,       0,  384626,       0,  0,  5,  0, }, // Gulf
    {  351,  887,       0,  384631,       0,  0,  6,  0, }, // Guyana
    {  351,  894,  384637,  384650,  384664, 13, 14, 14, }, // Hawaii_Aleutian
    {  351,  910,  129680,  129684,  384678,  4,  5,  5, }, // Hong_Kong
    {  351,  920,  384683,  384688,  384694,  5,  6,  6, }, // Hovd
    {  351,  925,       0,  384700,       0,  0,  6,  0, }, // India
    {  351,  931,       0,  384706,       0,  0,  6,  0, }, // Indian_Ocean
    {  351,  944,       0,  384712,       0,  0,  7,  0, }, // Indochina
    {  351,  954,       0,  384719,       0,  0, 10,  0, }, // Indonesia_Central
    {  351,  972,       0,  384729,       0,  0, 10,  0, }, // Indonesia_Eastern
    {  351,  990,       0,  384739,       0,  0, 10,  0, }, // Indonesia_Western
    {  351, 1008,  384749,  384754,  384760,  5,  6,  6, }, // Iran
    {  351, 1019,  384766,  384774,  384783,  8,  9,  9, }, // Irkutsk
    {  351, 1027,  384792,  384799,  384807,  7,  8,  8, }, // Israel
    {  351, 1034,  129810,  129814,  384815,  4,  5,  5, }, // Japan
    {  351, 1040,  384820,  384839,  384859, 19, 20, 20, }, // Kamchatka
    {  351, 1058,       0,  384879,       0,  0,  9,  0, }, // Kazakhstan_Eastern
    {  351, 1077,       0,  384888,       0,  0,  9,  0, }, // Kazakhstan_Western
    {  351, 1106,  384897,  384901,  384906,  4,  5,  5, }, // Korea
    {  351, 1112,       0,  384911,       0,  0,  6,  0, }, // Kosrae
    {  351, 1119,  384917,  384927,  384938, 10, 11, 11, }, // Krasnoyarsk
    {  351, 1151,       0,  384949,       0,  0,  6,  0, }, // Kyrgystan
    {  351, 1161,       0,  384955,       0,  0,  5,  0, }, // Lanka
    {  351, 1175,       0,  384960,       0,  0,  7,  0, }, // Line_Islands
    {  351, 1188,  384967,  384974,  384982,  7,  8,  8, }, // Lord_Howe
    {  351, 1198,  384990,  384995,  385001,  5,  6,  6, }, // Macau
    {  351, 1204,  385007,  385013,  385020,  6,  7,  7, }, // Magadan
    {  351, 1219,       0,  385027,       0,  0,  7,  0, }, // Malaysia
    {  351, 1228,       0,  385034,       0,  0,  7,  0, }, // Maldives
    {  351, 1237,       0,  385041,       0,  0,  7,  0, }, // Marquesas
    {  351, 1247,       0,  385048,       0,  0,  9,  0, }, // Marshall_Islands
    {  351, 1264,  385057,  385065,  385074,  8,  9,  9, }, // Mauritius
    {  351, 1274,       0,  385083,       0,  0,  8,  0, }, // Mawson
    {  351, 1281,  385091,  385100,  385110,  9, 10, 10, }, // Mexico_Pacific
    {  351, 1296,  385120,  385129,  385139,  9, 10, 10, }, // Mongolia
    {  351, 1305,  385149,  385155,  385162,  6,  7,  7, }, // Moscow
    {  351, 1312,       0,  385169,       0,  0,  7,  0, }, // Myanmar
    {  351, 1320,       0,  385176,       0,  0,  5,  0, }, // Nauru
    {  351, 1326,       0,  385181,       0,  0,  6,  0, }, // Nepal
    {  351, 1332,  385187,  385197,  385208, 10, 11, 11, }, // New_Caledonia
    {  351, 1346,  385219,  385229,  385240, 10, 11, 11, }, // New_Zealand
    {  351, 1358,  385251,  385263,  385276, 12, 13, 13, }, // Newfoundland
    {  351, 1371,       0,  385289,       0,  0,  5,  0, }, // Niue
    {  351, 1376,  385294,  385303,  385313,  9, 10, 10, }, // Norfolk
    {  351, 1384,  385323,  385339,  385356, 16, 17, 17, }, // Noronha
    {  351, 1392,       0,  385373,       0,  0,  9,  0, }, // North_Mariana
    {  351, 1406,  385382,  385392,  385403, 10, 11, 11, }, // Novosibirsk
    {  351, 1418,  385414,  385420,  385427,  6,  7,  7, }, // Omsk
    {  351, 1428,  385434,  385441,  385449,  7,  8,  8, }, // Pakistan
    {  351, 1437,       0,  385457,       0,  0,  5,  0, }, // Palau
    {  351, 1443,       0,  385462,       0,  0, 11,  0, }, // Papua_New_Guinea
    {  351, 1460,  385473,  385480,  385488,  7,  8,  8, }, // Paraguay
    {  351, 1469,  385496,  385501,  385507,  5,  6,  6, }, // Peru
    {  351, 1474,  385513,  385520,  385528,  7,  8,  8, }, // Philippines
    {  351, 1486,       0,  385536,       0,  0, 10,  0, }, // Phoenix_Islands
    {  351, 1502,  385546,  385561,  385577, 15, 16, 16, }, // Pierre_Miquelon
    {  351, 1518,       0,  385593,       0,  0,  7,  0, }, // Pitcairn
    {  351, 1527,       0,  385600,       0,  0,  5,  0, }, // Ponape
    {  351, 1534,       0,  385605,       0,  0,  4,  0, }, // Pyongyang
    {  351, 1544,  385609,  385616,  385624,  7,  8,  8, }, // Qyzylorda
    {  351, 1554,       0,  385632,       0,  0,  7,  0, }, // Reunion
    {  351, 1562,       0,  385639,       0,  0,  7,  0, }, // Rothera
    {  351, 1570,  385646,  385652,  385659,  6,  7,  7, }, // Sakhalin
    {  351, 1579,  385666,  385671,  385677,  5,  6,  6, }, // Samara
    {  351, 1596,  385683,  385688,  385694,  5,  6,  6, }, // Samoa
    {  351, 1602,       0,  385700,       0,  0,  7,  0, }, // Seychelles
    {  351, 1624,       0,  385707,       0,  0,  9,  0, }, // Singapore
    {  351, 1634,       0,  385716,       0,  0,  8,  0, }, // Solomon
    {  351, 1642,       0,  385724,       0,  0, 10,  0, }, // South_Georgia
    {  351, 1656,       0,  385734,       0,  0,  6,  0, }, // Suriname
    {  351, 1676,       0,  130609,       0,  0,  6,  0, }, // Syowa
    {  351, 1682,       0,  385740,       0,  0,  5,  0, }, // Tahiti
    {  351, 1689,  130620,  130624,  385745,  4,  5,  5, }, // Taipei
    {  351, 1696,       0,  385750,       0,  0,  8,  0, }, // Tajikistan
    {  351, 1724,       0,  385758,       0,  0,  6,  0, }, // Tokelau
    {  351, 1732,  385764,  385769,  385775,  5,  6,  6, }, // Tonga
    {  351, 1738,       0,  385781,       0,  0,  6,  0, }, // Truk
    {  351, 1750,  385787,  385797,  385808, 10, 11, 11, }, // Turkmenistan
    {  351, 1763,       0,  385819,       0,  0,  5,  0, }, // Tuvalu
    {  351, 1777,  385824,  385831,  385839,  7,  8,  8, }, // Uruguay
    {  351, 1792,  385847,  385856,  385866,  9, 10, 10, }, // Uzbekistan
    {  351, 1803,  385876,  385882,  385889,  6,  7,  7, }, // Vanuatu
    {  351, 1811,       0,  385896,       0,  0,  7,  0, }, // Venezuela
    {  351, 1821,  385903,  385912,  385922,  9, 10, 10, }, // Vladivostok
    {  351, 1833,  385932,  385941,  385951,  9, 10, 10, }, // Volgograd
    {  351, 1843,       0,  385961,       0,  0,  9,  0, }, // Vostok
    {  351, 1850,       0,  385970,       0,  0,  7,  0, }, // Wake
    {  351, 1855,       0,  385977,       0,  0, 10,  0, }, // Wallis
    {  351, 1862,  385987,  385994,  386002,  7,  8,  8, }, // Yakutsk
    {  351, 1870,  386010,  386020,  386031, 10, 11, 11, }, // Yekaterinburg
    {  351, 1892,       0,  386042,       0,  0,  6,  0, }, // Yukon
    {  352,    5,       0,  386048,       0,  0, 17,  0, }, // Afghanistan Javanese/Latin/Indonesia
    {  352,   17,       0,  386065,       0,  0, 19,  0, }, // Africa_Central
    {  352,   32,       0,  386084,       0,  0, 18,  0, }, // Africa_Eastern
    {  352,   65,       0,  386102,       0,  0, 26,  0, }, // Africa_Southern
    {  352,   81,  386128,  386146,  386172, 18, 26, 25, }, // Africa_Western
    {  352,  107,  386197,  386209,  386229, 12, 20, 19, }, // Alaska
    {  352,  135,  386248,  386260,  386280, 12, 20, 19, }, // Amazon
    {  352,  142,  386299,  386311,  386331, 12, 20, 19, }, // America_Central
    {  352,  158,  386350,  386367,  386392, 17, 25, 24, }, // America_Eastern
    {  352,  174,  386416,  386426,  386444, 10, 18, 17, }, // America_Mountain
    {  352,  191,  386461,  386474,  386495, 13, 21, 20, }, // America_Pacific
    {  352,  214,  386515,  386525,  386543, 10, 18, 17, }, // Apia
    {  352,  232,  386560,  386570,  386588, 10, 18, 17, }, // Arabian
    {  352,  240,  386605,  386620,  386643, 15, 23, 22, }, // Argentina
    {  352,  250,  386665,  386692,  386727, 27, 35, 34, }, // Argentina_Western
    {  352,  268,  386761,  386774,  386795, 13, 21, 20, }, // Armenia
    {  352,  286,  386815,  386829,  386851, 14, 22, 21, }, // Atlantic
    {  352,  295,  386872,  386894,  386924, 22, 30, 29, }, // Australia_Central
    {  352,  313,  386953,  386987,  387029, 34, 42, 41, }, // Australia_CentralWestern
    {  352,  338,  387070,  387097,  387132, 27, 35, 34, }, // Australia_Eastern
    {  352,  356,  387166,  387193,  387228, 27, 35, 34, }, // Australia_Western
    {  352,  374,  387262,  387278,  387302, 16, 24, 23, }, // Azerbaijan
    {  352,  385,  387325,  387337,  387357, 12, 20, 19, }, // Azores
    {  352,  397,  387376,  387392,  387416, 16, 24, 23, }, // Bangladesh
    {  352,  415,       0,  387439,       0,  0, 12,  0, }, // Bhutan
    {  352,  422,       0,  387451,       0,  0, 13,  0, }, // Bolivia
    {  352,  437,  387464,  387478,  387500, 14, 22, 21, }, // Brasilia
    {  352,  454,       0,  387521,       0,  0, 23,  0, }, // Brunei
    {  352,  461,  387544,  387563,  387590, 19, 27, 26, }, // Cape_Verde
    {  352,  478,       0,  387616,       0,  0, 14,  0, }, // Chamorro
    {  352,  487,  387630,  387643,  387664, 13, 21, 20, }, // Chatham
    {  352,  495,  387684,  387695,  387714, 11, 19, 18, }, // Chile
    {  352,  501,  387732,  387742,  387760, 10, 18, 17, }, // China
    {  352,  507,  387777,  387793,  387817, 16, 24, 23, }, // Choibalsan
    {  352,  518,       0,  387840,       0,  0, 16,  0, }, // Christmas
    {  352,  528,       0,  387856,       0,  0, 20,  0, }, // Cocos
    {  352,  534,  387876,  387890,  387912, 14, 22, 21, }, // Colombia
    {  352,  543,  387933,  387952,  387979, 19, 27, 26, }, // Cook
    {  352,  548,  388005,  388015,  388033, 10, 18, 17, }, // Cuba
    {  352,  559,       0,  388050,       0,  0, 11,  0, }, // Davis
    {  352,  575,       0,  388061,       0,  0, 22,  0, }, // DumontDUrville
    {  352,  612,       0,  388083,       0,  0, 17,  0, }, // East_Timor
    {  352,  623,  388100,  388117,  388142, 17, 25, 24, }, // Easter
    {  352,  630,       0,  388166,       0,  0, 13,  0, }, // Ecuador
    {  352,  638,  388179,  388197,  388223, 18, 26, 25, }, // Europe_Central
    {  352,  653,  388248,  388271,  388302, 23, 31, 30, }, // Europe_Eastern
    {  352,  668,       0,  388332,       0,  0, 39,  0, }, // Europe_Further_Eastern
    {  352,  691,  388371,  388394,  388425, 23, 31, 30, }, // Europe_Western
    {  352,  706,  388455,  388478,  388509, 23, 31, 30, }, // Falkland
    {  352,  715,  388539,  388549,  388567, 10, 18, 17, }, // Fiji
    {  352,  720,       0,  388584,       0,  0, 20,  0, }, // French_Guiana
    {  352,  734,       0,  388604,       0,  0, 39,  0, }, // French_Southern
    {  352,  757,       0,  388643,       0,  0, 15,  0, }, // Galapagos
    {  352,  767,       0,  388658,       0,  0, 13,  0, }, // Gambier
    {  352,  775,  388671,  388684,  388705, 13, 21, 20, }, // Georgia
    {  352,  783,       0,  388725,       0,  0, 22,  0, }, // Gilbert_Islands
    {  352,  799,       0,  388747,       0,  0, 22,  0, }, // GMT
    {  352,  841,  388769,  388789,  388817, 20, 28, 27, }, // Greenland_Eastern
    {  352,  859,  388844,  388864,  388892, 20, 28, 27, }, // Greenland_Western
    {  352,  882,       0,  388919,       0,  0, 19,  0, }, // Gulf
    {  352,  887,       0,  388938,       0,  0, 12,  0, }, // Guyana
    {  352,  894,  388950,  388971,  389000, 21, 29, 28, }, // Hawaii_Aleutian
    {  352,  910,  389028,  389043,  389066, 15, 23, 22, }, // Hong_Kong
    {  352,  920,  389088,  389098,  389116, 10, 18, 17, }, // Hovd
    {  352,  925,       0,  389133,       0,  0, 19,  0, }, // India
    {  352,  931,       0,  389152,       0,  0, 19,  0, }, // Indian_Ocean
    {  352,  944,       0,  389171,       0,  0, 14,  0, }, // Indochina
    {  352,  954,       0,  389185,       0,  0, 22,  0, }, // Indonesia_Central
    {  352,  972,       0,  389207,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  352,  990,       0,  389234,       0,  0, 27,  0, }, // Indonesia_Western
    {  352, 1008,  389261,  389271,  389289, 10, 18, 17, }, // Iran
    {  352, 1019,  389306,  389319,  389340, 13, 21, 20, }, // Irkutsk
    {  352, 1027,  389360,  389372,  389392, 12, 20, 19, }, // Israel
    {  352, 1034,  389411,  389423,  389443, 12, 20, 19, }, // Japan
    {  352, 1058,       0,  389462,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  352, 1077,       0,  389484,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  352, 1106,  389506,  389517,  389536, 11, 19, 18, }, // Korea
    {  352, 1112,       0,  389554,       0,  0, 12,  0, }, // Kosrae
    {  352, 1119,  389566,  389583,  389608, 17, 25, 24, }, // Krasnoyarsk
    {  352, 1151,       0,  389632,       0,  0, 16,  0, }, // Kyrgystan
    {  352, 1175,       0,  389648,       0,  0, 19,  0, }, // Line_Islands
    {  352, 1188,  389667,  389682,  389705, 15, 23, 22, }, // Lord_Howe
    {  352, 1204,  389727,  389740,  389761, 13, 21, 20, }, // Magadan
    {  352, 1219,       0,  389781,       0,  0, 14,  0, }, // Malaysia
    {  352, 1228,       0,  389795,       0,  0, 14,  0, }, // Maldives
    {  352, 1237,       0,  389809,       0,  0, 15,  0, }, // Marquesas
    {  352, 1247,       0,  389824,       0,  0, 23,  0, }, // Marshall_Islands
    {  352, 1264,  389847,  389862,  389885, 15, 23, 22, }, // Mauritius
    {  352, 1274,       0,  389907,       0,  0, 12,  0, }, // Mawson
    {  352, 1281,  389919,  389940,  389969, 21, 29, 28, }, // Mexico_Pacific
    {  352, 1296,  389997,  390014,  390039, 17, 25, 24, }, // Mongolia
    {  352, 1305,  390063,  390075,  390095, 12, 20, 19, }, // Moscow
    {  352, 1312,       0,  390114,       0,  0, 13,  0, }, // Myanmar
    {  352, 1320,       0,  390127,       0,  0, 11,  0, }, // Nauru
    {  352, 1326,       0,  390138,       0,  0, 11,  0, }, // Nepal
    {  352, 1332,  390149,  390170,  390199, 21, 29, 28, }, // New_Caledonia
    {  352, 1346,  390227,  390247,  390275, 20, 28, 27, }, // New_Zealand
    {  352, 1358,  390302,  390320,  390346, 18, 26, 25, }, // Newfoundland
    {  352, 1371,       0,  390371,       0,  0, 10,  0, }, // Niue
    {  352, 1376,  390381,  390399,  390425, 18, 26, 25, }, // Norfolk
    {  352, 1384,  390450,  390475,  390508, 25, 33, 32, }, // Noronha
    {  352, 1406,  390540,  390557,  390582, 17, 25, 24, }, // Novosibirsk
    {  352, 1418,  390606,  390616,  390634, 10, 18, 17, }, // Omsk
    {  352, 1428,  390651,  390665,  390687, 14, 22, 21, }, // Pakistan
    {  352, 1437,       0,  390708,       0,  0, 11,  0, }, // Palau
    {  352, 1443,       0,  390719,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  352, 1460,  390737,  390751,  390773, 14, 22, 21, }, // Paraguay
    {  352, 1469,  390794,  390804,  390822, 10, 18, 17, }, // Peru
    {  352, 1474,  390839,  390853,  390875, 14, 22, 21, }, // Philippines
    {  352, 1486,       0,  390896,       0,  0, 22,  0, }, // Phoenix_Islands
    {  352, 1502,  390918,  390949,  390988, 31, 39, 38, }, // Pierre_Miquelon
    {  352, 1518,       0,  391026,       0,  0, 14,  0, }, // Pitcairn
    {  352, 1527,       0,  391040,       0,  0, 12,  0, }, // Ponape
    {  352, 1534,       0,  391052,       0,  0, 15,  0, }, // Pyongyang
    {  352, 1554,       0,  391067,       0,  0, 13,  0, }, // Reunion
    {  352, 1562,       0,  391080,       0,  0, 13,  0, }, // Rothera
    {  352, 1570,  391093,  391107,  391129, 14, 22, 21, }, // Sakhalin
    {  352, 1596,  391150,  391161,  391180, 11, 19, 18, }, // Samoa
    {  352, 1602,       0,  391198,       0,  0, 16,  0, }, // Seychelles
    {  352, 1624,       0,  391214,       0,  0, 15,  0, }, // Singapore
    {  352, 1634,       0,  391229,       0,  0, 22,  0, }, // Solomon
    {  352, 1642,       0,  391251,       0,  0, 19,  0, }, // South_Georgia
    {  352, 1656,       0,  391270,       0,  0, 14,  0, }, // Suriname
    {  352, 1676,       0,  391284,       0,  0, 11,  0, }, // Syowa
    {  352, 1682,       0,  391295,       0,  0, 12,  0, }, // Tahiti
    {  352, 1689,  391307,  391319,  391339, 12, 20, 19, }, // Taipei
    {  352, 1696,       0,  391358,       0,  0, 16,  0, }, // Tajikistan
    {  352, 1724,       0,  391374,       0,  0, 13,  0, }, // Tokelau
    {  352, 1732,  391387,  391398,  391417, 11, 19, 18, }, // Tonga
    {  352, 1738,       0,  391435,       0,  0, 11,  0, }, // Truk
    {  352, 1750,  391446,  391464,  391490, 18, 26, 25, }, // Turkmenistan
    {  352, 1763,       0,  391515,       0,  0, 12,  0, }, // Tuvalu
    {  352, 1777,  391527,  391540,  391561, 13, 21, 20, }, // Uruguay
    {  352, 1792,  391581,  391597,  391621, 16, 24, 23, }, // Uzbekistan
    {  352, 1803,  391644,  391657,  391678, 13, 21, 20, }, // Vanuatu
    {  352, 1811,       0,  391698,       0,  0, 15,  0, }, // Venezuela
    {  352, 1821,  391713,  391730,  391755, 17, 25, 24, }, // Vladivostok
    {  352, 1833,  391779,  391794,  391817, 15, 23, 22, }, // Volgograd
    {  352, 1843,       0,  391839,       0,  0, 12,  0, }, // Vostok
    {  352, 1850,       0,  391851,       0,  0, 15,  0, }, // Wake
    {  352, 1855,       0,  391866,       0,  0, 23,  0, }, // Wallis
    {  352, 1862,  391889,  391902,  391923, 13, 21, 20, }, // Yakutsk
    {  352, 1870,  391943,  391962,  391989, 19, 27, 26, }, // Yekaterinburg
    {  352, 1892,       0,  392015,       0,  0, 11,  0, }, // Yukon
    {  355,   17,       0,  392026,       0,  0, 21,  0, }, // Africa_Central Kabuverdianu/Latin/Cape Verde
    {  355,   32,       0,  392047,       0,  0, 22,  0, }, // Africa_Eastern
    {  355,   65,       0,  392069,       0,  0, 20,  0, }, // Africa_Southern
    {  355,   81,  392089,  392112,  392142, 23, 30, 32, }, // Africa_Western
    {  355,  107,  392174,  392187,  392207, 13, 20, 22, }, // Alaska
    {  355,  142,  392229,  392240,  392258, 11, 18, 20, }, // America_Central
    {  355,  158,  392278,  392290,  392309, 12, 19, 21, }, // America_Eastern
    {  355,  174,  392330,  392345,  392367, 15, 22, 24, }, // America_Mountain
    {  355,  191,  392391,  392406,  392428, 15, 22, 24, }, // America_Pacific
    {  355,  286,  392452,  392468,  392491, 16, 23, 25, }, // Atlantic
    {  355,  295,  392516,  392540,  392571, 24, 31, 33, }, // Australia_Central
    {  355,  313,  392604,  392637,  392677, 33, 40, 42, }, // Australia_CentralWestern
    {  355,  338,  392719,  392744,  392776, 25, 32, 34, }, // Australia_Eastern
    {  355,  356,  392810,  392836,  392869, 26, 33, 35, }, // Australia_Western
    {  355,  461,  392904,  392921,  392945, 17, 24, 26, }, // Cape_Verde
    {  355,  548,  392971,  392982,  393000, 11, 18, 20, }, // Cuba
    {  355,  638,  393020,  393041,  393069, 21, 28, 30, }, // Europe_Central
    {  355,  653,  393099,  393121,  393150, 22, 29, 31, }, // Europe_Eastern
    {  355,  691,  393181,  393204,  393234, 23, 30, 32, }, // Europe_Western
    {  355,  799,       0,  393266,       0,  0, 22,  0, }, // GMT
    {  355,  841,  393288,  393315,  393349, 27, 34, 36, }, // Greenland_Eastern
    {  355,  859,  393385,  393413,  393448, 28, 35, 37, }, // Greenland_Western
    {  355,  894,  393485,  393506,  393534, 21, 28, 30, }, // Hawaii_Aleutian
    {  355, 1281,  393564,  393588,  393619, 24, 31, 33, }, // Mexico_Pacific
    {  355, 1358,  393652,  393668,  393691, 16, 23, 25, }, // Newfoundland
    {  355, 1502,  393716,  393742,  393775, 26, 33, 35, }, // Pierre_Miquelon
    {  355, 1892,       0,  393810,       0,  0, 12,  0, }, // Yukon
    {  356,    5,       0,  393822,       0,  0, 17,  0, }, // Afghanistan Kabyle/Latin/Algeria
    {  356,   17,       0,  393839,       0,  0, 24,  0, }, // Africa_Central
    {  356,   32,       0,  393863,       0,  0, 24,  0, }, // Africa_Eastern
    {  356,   65,       0,  393887,       0,  0, 30,  0, }, // Africa_Southern
    {  356,   81,  393917,  393939,  393969, 22, 30, 31, }, // Africa_Western
    {  356,  107,  394000,  394013,  394034, 13, 21, 22, }, // Alaska
    {  356,  135,  394056,  394069,  394090, 13, 21, 22, }, // Amazon
    {  356,  142,  394112,  394137,  394170, 25, 33, 34, }, // America_Central
    {  356,  158,  394204,  394228,  394260, 24, 32, 33, }, // America_Eastern
    {  356,  174,  394293,  394317,  394349, 24, 32, 33, }, // America_Mountain
    {  356,  191,  394382,  394393,  394412, 11, 19, 20, }, // America_Pacific
    {  356,  214,  394432,  394443,  394462, 11, 19, 20, }, // Apia
    {  356,  232,  394482,  394492,  394510, 10, 18, 19, }, // Arabian
    {  356,  240,  394529,  394544,  394567, 15, 23, 24, }, // Argentina
    {  356,  250,  394591,  394616,  394649, 25, 33, 34, }, // Argentina_Western
    {  356,  268,  394683,  394697,  394719, 14, 22, 23, }, // Armenia
    {  356,  286,  394742,  394754,  394774, 12, 20, 21, }, // Atlantic
    {  356,  295,  394795,  394820,  394853, 25, 33, 34, }, // Australia_Central
    {  356,  313,  394887,  394921,  394963, 34, 42, 43, }, // Australia_CentralWestern
    {  356,  338,  395006,  395031,  395064, 25, 33, 34, }, // Australia_Eastern
    {  356,  356,  395098,  395121,  395152, 23, 31, 30, }, // Australia_Western
    {  356,  374,  395182,  395198,  395222, 16, 24, 25, }, // Azerbaijan
    {  356,  385,  395247,  395260,  395281, 13, 21, 22, }, // Azores
    {  356,  397,  395303,  395319,  395343, 16, 24, 25, }, // Bangladesh
    {  356,  415,       0,  395368,       0,  0, 12,  0, }, // Bhutan
    {  356,  422,       0,  395380,       0,  0, 13,  0, }, // Bolivia
    {  356,  437,  395393,  395408,  395431, 15, 23, 24, }, // Brasilia
    {  356,  454,       0,  395455,       0,  0, 23,  0, }, // Brunei
    {  356,  461,  395478,  395492,  395514, 14, 22, 23, }, // Cape_Verde
    {  356,  478,       0,  395537,       0,  0, 22,  0, }, // Chamorro
    {  356,  487,  395559,  395572,  395593, 13, 21, 20, }, // Chatham
    {  356,  495,  395613,  395624,  395643, 11, 19, 20, }, // Chile
    {  356,  501,  395663,  395673,  395691, 10, 18, 19, }, // China
    {  356,  507,  395710,  395726,  395750, 16, 24, 25, }, // Choibalsan
    {  356,  518,       0,  395775,       0,  0, 25,  0, }, // Christmas
    {  356,  528,       0,  395800,       0,  0, 22,  0, }, // Cocos
    {  356,  534,  395822,  395837,  395860, 15, 23, 24, }, // Colombia
    {  356,  543,  395800,  395884,  395913, 21, 29, 30, }, // Cook
    {  356,  548,  395943,  395954,  395973, 11, 19, 20, }, // Cuba
    {  356,  559,       0,  395993,       0,  0, 12,  0, }, // Davis
    {  356,  575,       0,  396005,       0,  0, 23,  0, }, // DumontDUrville
    {  356,  612,       0,  396028,       0,  0, 22,  0, }, // East_Timor
    {  356,  623,  396050,  396073,  396104, 23, 31, 32, }, // Easter
    {  356,  630,       0,  396136,       0,  0, 14,  0, }, // Ecuador
    {  356,  638,  396150,  396173,  396204, 23, 31, 32, }, // Europe_Central
    {  356,  653,  396236,  396259,  396290, 23, 31, 32, }, // Europe_Eastern
    {  356,  668,       0,  396322,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  356,  691,  396352,  396373,  396402, 21, 29, 28, }, // Europe_Western
    {  356,  706,  396430,  396456,  396490, 26, 34, 33, }, // Falkland
    {  356,  715,  396523,  396534,  396553, 11, 19, 20, }, // Fiji
    {  356,  720,       0,  396573,       0,  0, 24,  0, }, // French_Guiana
    {  356,  734,       0,  396597,       0,  0, 41,  0, }, // French_Southern
    {  356,  757,       0,  396638,       0,  0, 16,  0, }, // Galapagos
    {  356,  767,       0,  396654,       0,  0, 23,  0, }, // Gambier
    {  356,  775,  396677,  396691,  396713, 14, 22, 23, }, // Georgia
    {  356,  783,       0,  396736,       0,  0, 22,  0, }, // Gilbert_Islands
    {  356,  799,       0,  396758,       0,  0, 24,  0, }, // GMT
    {  356,  841,  396782,  396807,  396840, 25, 33, 34, }, // Greenland_Eastern
    {  356,  859,  396874,  396897,  396928, 23, 31, 32, }, // Greenland_Western
    {  356,  882,       0,  396960,       0,  0, 19,  0, }, // Gulf
    {  356,  887,       0,  396573,       0,  0, 13,  0, }, // Guyana
    {  356,  894,  396979,  397000,  397029, 21, 29, 30, }, // Hawaii_Aleutian
    {  356,  910,  397059,  397075,  397099, 16, 24, 25, }, // Hong_Kong
    {  356,  920,  397124,  397135,  397154, 11, 19, 20, }, // Hovd
    {  356,  925,       0,  397174,       0,  0, 20,  0, }, // India
    {  356,  931,       0,  397194,       0,  0, 20,  0, }, // Indian_Ocean
    {  356,  944,       0,  397214,       0,  0, 14,  0, }, // Indochina
    {  356,  954,       0,  397228,       0,  0, 27,  0, }, // Indonesia_Central
    {  356,  972,       0,  397255,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  356,  990,       0,  397281,       0,  0, 24,  0, }, // Indonesia_Western
    {  356, 1008,  397305,  397316,  397335, 11, 19, 18, }, // Iran
    {  356, 1019,  397353,  397367,  397389, 14, 22, 23, }, // Irkutsk
    {  356, 1027,  397412,  397426,  397448, 14, 22, 23, }, // Israel
    {  356, 1034,  397471,  397483,  397503, 12, 20, 21, }, // Japan
    {  356, 1058,       0,  397524,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  356, 1077,       0,  397551,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  356, 1106,  397578,  397590,  397610, 12, 20, 21, }, // Korea
    {  356, 1112,       0,  397631,       0,  0, 13,  0, }, // Kosrae
    {  356, 1119,  397644,  397662,  397687, 18, 25, 27, }, // Krasnoyarsk
    {  356, 1151,       0,  397714,       0,  0, 17,  0, }, // Kyrgystan
    {  356, 1175,       0,  397731,       0,  0, 19,  0, }, // Line_Islands
    {  356, 1188,  397750,  397766,  397789, 16, 23, 25, }, // Lord_Howe
    {  356, 1204,  397814,  397828,  397850, 14, 22, 23, }, // Magadan
    {  356, 1219,       0,  397873,       0,  0, 14,  0, }, // Malaysia
    {  356, 1228,       0,  397887,       0,  0, 13,  0, }, // Maldives
    {  356, 1237,       0,  397900,       0,  0, 22,  0, }, // Marquesas
    {  356, 1247,       0,  397922,       0,  0, 22,  0, }, // Marshall_Islands
    {  356, 1264,  397944,  397956,  397976, 12, 20, 21, }, // Mauritius
    {  356, 1274,       0,  397997,       0,  0, 13,  0, }, // Mawson
    {  356, 1281,  398010,  398030,  398058, 20, 28, 29, }, // Mexico_Pacific
    {  356, 1296,  398087,  398103,  398127, 16, 24, 25, }, // Mongolia
    {  356, 1305,  398152,  398164,  398184, 12, 20, 21, }, // Moscow
    {  356, 1312,       0,  398205,       0,  0, 14,  0, }, // Myanmar
    {  356, 1320,       0,  398219,       0,  0, 11,  0, }, // Nauru
    {  356, 1326,       0,  398230,       0,  0, 12,  0, }, // Nepal
    {  356, 1332,  398242,  398267,  398300, 25, 33, 34, }, // New_Caledonia
    {  356, 1346,  398334,  398356,  398386, 22, 30, 29, }, // New_Zealand
    {  356, 1358,  398415,  398435,  398463, 20, 28, 29, }, // Newfoundland
    {  356, 1371,       0,  398492,       0,  0, 11,  0, }, // Niue
    {  356, 1376,  398503,  398527,  398559, 24, 32, 33, }, // Norfolk
    {  356, 1384,  398592,  398617,  398650, 25, 33, 34, }, // Noronha
    {  356, 1406,  398684,  398702,  398727, 18, 25, 27, }, // Novosibirsk
    {  356, 1418,  398754,  398765,  398783, 11, 18, 20, }, // Omsk
    {  356, 1428,  398803,  398818,  398841, 15, 23, 24, }, // Pakistan
    {  356, 1437,       0,  398865,       0,  0, 12,  0, }, // Palau
    {  356, 1443,       0,  398877,       0,  0, 29,  0, }, // Papua_New_Guinea
    {  356, 1460,  398906,  398921,  398944, 15, 23, 24, }, // Paraguay
    {  356, 1469,  398968,  398979,  398998, 11, 19, 20, }, // Peru
    {  356, 1474,  399018,  399032,  399054, 14, 22, 23, }, // Philippines
    {  356, 1486,       0,  399077,       0,  0, 24,  0, }, // Phoenix_Islands
    {  356, 1502,  399101,  399125,  399157, 24, 32, 33, }, // Pierre_Miquelon
    {  356, 1518,       0,  399190,       0,  0, 14,  0, }, // Pitcairn
    {  356, 1527,       0,  399204,       0,  0, 13,  0, }, // Ponape
    {  356, 1534,       0,  399217,       0,  0, 16,  0, }, // Pyongyang
    {  356, 1554,       0,  399233,       0,  0, 15,  0, }, // Reunion
    {  356, 1562,       0,  399248,       0,  0, 14,  0, }, // Rothera
    {  356, 1570,  399262,  399277,  399299, 15, 22, 24, }, // Sakhalin
    {  356, 1596,  399323,  399335,  399355, 12, 20, 21, }, // Samoa
    {  356, 1602,       0,  399376,       0,  0, 13,  0, }, // Seychelles
    {  356, 1624,       0,  399389,       0,  0, 23,  0, }, // Singapore
    {  356, 1634,       0,  399412,       0,  0, 23,  0, }, // Solomon
    {  356, 1642,       0,  399435,       0,  0, 22,  0, }, // South_Georgia
    {  356, 1656,       0,  399457,       0,  0, 14,  0, }, // Suriname
    {  356, 1676,       0,  399471,       0,  0, 12,  0, }, // Syowa
    {  356, 1682,       0,  399483,       0,  0, 13,  0, }, // Tahiti
    {  356, 1689,  399496,  399509,  399530, 13, 21, 22, }, // Taipei
    {  356, 1696,       0,  399552,       0,  0, 17,  0, }, // Tajikistan
    {  356, 1724,       0,  399569,       0,  0, 14,  0, }, // Tokelau
    {  356, 1732,  399583,  399595,  399615, 12, 20, 21, }, // Tonga
    {  356, 1738,       0,  399636,       0,  0, 12,  0, }, // Truk
    {  356, 1750,  399648,  399667,  399694, 19, 27, 28, }, // Turkmenistan
    {  356, 1763,       0,  399722,       0,  0, 13,  0, }, // Tuvalu
    {  356, 1777,  399735,  399749,  399771, 14, 22, 23, }, // Uruguay
    {  356, 1792,  399794,  399811,  399836, 17, 25, 26, }, // Uzbekistan
    {  356, 1803,  399862,  399877,  399899, 15, 22, 24, }, // Vanuatu
    {  356, 1811,       0,  399923,       0,  0, 16,  0, }, // Venezuela
    {  356, 1821,  399939,  399957,  399982, 18, 25, 27, }, // Vladivostok
    {  356, 1833,  400009,  400025,  400048, 16, 23, 25, }, // Volgograd
    {  356, 1843,       0,  400073,       0,  0, 13,  0, }, // Vostok
    {  356, 1850,       0,  400086,       0,  0, 22,  0, }, // Wake
    {  356, 1855,       0,  400108,       0,  0, 26,  0, }, // Wallis
    {  356, 1862,  400134,  400148,  400170, 14, 22, 23, }, // Yakutsk
    {  356, 1870,  400193,  400213,  400240, 20, 27, 29, }, // Yekaterinburg
    {  361,    0,  400269,  400277,  400294,  8, 17, 15, }, // Acre Kannada/Kannada/India
    {  361,    5,       0,  400309,       0,  0, 15,  0, }, // Afghanistan
    {  361,   17,       0,  400324,       0,  0, 16,  0, }, // Africa_Central
    {  361,   32,       0,  400340,       0,  0, 17,  0, }, // Africa_Eastern
    {  361,   65,       0,  400357,       0,  0, 27,  0, }, // Africa_Southern
    {  361,   81,  400384,  400402,  400429, 18, 27, 25, }, // Africa_Western
    {  361,  107,  400454,  400465,  400485, 11, 20, 17, }, // Alaska
    {  361,  128,  400502,  400513,  400533, 11, 20, 18, }, // Almaty
    {  361,  135,  400551,  400562,  400582, 11, 20, 18, }, // Amazon
    {  361,  142,  400600,  400624,  400657, 24, 33, 31, }, // America_Central
    {  361,  158,  400688,  400712,  400745, 24, 33, 29, }, // America_Eastern
    {  361,  174,  400774,  400797,  400829, 23, 32, 28, }, // America_Mountain
    {  361,  191,  400857,  400883,  400918, 26, 35, 31, }, // America_Pacific
    {  361,  207,  400949,  400960,  400980, 11, 20, 16, }, // Anadyr
    {  361,  214,  400996,  401005,  401023,  9, 18, 14, }, // Apia
    {  361,  219,  401037,  401046,  401064,  9, 18, 16, }, // Aqtau
    {  361,  225,  401080,  401091,  401111, 11, 20, 18, }, // Aqtobe
    {  361,  232,  401129,  401141,  401162, 12, 21, 17, }, // Arabian
    {  361,  240,  401179,  401193,  401216, 14, 23, 21, }, // Argentina
    {  361,  250,  401237,  401258,  401288, 21, 30, 28, }, // Argentina_Western
    {  361,  268,  401316,  401328,  401349, 12, 21, 19, }, // Armenia
    {  361,  286,  401368,  401382,  401405, 14, 23, 19, }, // Atlantic
    {  361,  295,  401424,  401446,  401478, 22, 32, 28, }, // Australia_Central
    {  361,  313,  401506,  401536,  401575, 30, 39, 35, }, // Australia_CentralWestern
    {  361,  338,  401610,  401631,  401662, 21, 31, 27, }, // Australia_Eastern
    {  361,  356,  401689,  401711,  401743, 22, 32, 28, }, // Australia_Western
    {  361,  374,  401771,  401785,  401808, 14, 23, 21, }, // Azerbaijan
    {  361,  385,  401829,  401839,  401858, 10, 19, 17, }, // Azores
    {  361,  397,  401875,  401889,  401912, 14, 23, 21, }, // Bangladesh
    {  361,  415,       0,  401933,       0,  0, 10,  0, }, // Bhutan
    {  361,  422,       0,  401943,       0,  0, 12,  0, }, // Bolivia
    {  361,  437,  401955,  401970,  401993, 15, 23, 21, }, // Brasilia
    {  361,  454,       0,  402014,       0,  0, 20,  0, }, // Brunei
    {  361,  461,  402034,  402048,  402071, 14, 23, 21, }, // Cape_Verde
    {  361,  478,       0,  402092,       0,  0, 18,  0, }, // Chamorro
    {  361,  487,  402110,  402119,  402137,  9, 18, 14, }, // Chatham
    {  361,  495,  402151,  402159,  402176,  8, 17, 15, }, // Chile
    {  361,  501,  402191,  402199,  402216,  8, 17, 13, }, // China
    {  361,  507,  402229,  402245,  402272, 16, 27, 23, }, // Choibalsan
    {  361,  518,       0,  402295,       0,  0, 19,  0, }, // Christmas
    {  361,  528,       0,  402314,       0,  0, 18,  0, }, // Cocos
    {  361,  534,  402332,  402344,  402365, 12, 21, 19, }, // Colombia
    {  361,  543,  402384,  402400,  402425, 16, 25, 33, }, // Cook
    {  361,  548,  402458,  402468,  402487, 10, 19, 15, }, // Cuba
    {  361,  559,       0,  402502,       0,  0, 10,  0, }, // Davis
    {  361,  575,       0,  402512,       0,  0, 21,  0, }, // DumontDUrville
    {  361,  612,       0,  402533,       0,  0, 16,  0, }, // East_Timor
    {  361,  623,  402549,  402565,  402590, 16, 25, 23, }, // Easter
    {  361,  630,       0,  402613,       0,  0, 13,  0, }, // Ecuador
    {  361,  638,  402626,  402644,  402671, 18, 27, 25, }, // Europe_Central
    {  361,  653,  402696,  402715,  402743, 19, 28, 26, }, // Europe_Eastern
    {  361,  668,       0,  402769,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {  361,  691,  402797,  402817,  402846, 20, 29, 27, }, // Europe_Western
    {  361,  706,  402873,  402897,  402930, 24, 33, 31, }, // Falkland
    {  361,  715,  402961,  402969,  402986,  8, 17, 15, }, // Fiji
    {  361,  720,       0,  403001,       0,  0, 17,  0, }, // French_Guiana
    {  361,  734,       0,  403018,       0,  0, 35,  0, }, // French_Southern
    {  361,  757,       0,  403053,       0,  0, 14,  0, }, // Galapagos
    {  361,  767,       0,  403067,       0,  0, 14,  0, }, // Gambier
    {  361,  775,  403081,  403093,  403114, 12, 21, 19, }, // Georgia
    {  361,  783,       0,  403133,       0,  0, 21,  0, }, // Gilbert_Islands
    {  361,  799,       0,  403154,       0,  0, 25,  0, }, // GMT
    {  361,  841,  403179,  403203,  403236, 24, 33, 31, }, // Greenland_Eastern
    {  361,  859,  403267,  403292,  403326, 25, 34, 32, }, // Greenland_Western
    {  361,  877,       0,  403358,       0,  0, 19,  0, }, // Guam
    {  361,  882,       0,  403377,       0,  0, 18,  0, }, // Gulf
    {  361,  887,       0,  403009,       0,  0,  9,  0, }, // Guyana
    {  361,  894,  403395,  403415,  403444, 20, 29, 25, }, // Hawaii_Aleutian
    {  361,  910,  403469,  403484,  403508, 15, 24, 22, }, // Hong_Kong
    {  361,  920,  403530,  403539,  403557,  9, 18, 16, }, // Hovd
    {  361,  925,       0,  403573,       0,  0, 19,  0, }, // India
    {  361,  931,       0,  403592,       0,  0, 17,  0, }, // Indian_Ocean
    {  361,  944,       0,  403609,       0,  0, 12,  0, }, // Indochina
    {  361,  954,       0,  403621,       0,  0, 19,  0, }, // Indonesia_Central
    {  361,  972,       0,  403640,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  361,  990,       0,  403660,       0,  0, 20,  0, }, // Indonesia_Western
    {  361, 1008,  403680,  403689,  403707,  9, 18, 14, }, // Iran
    {  361, 1019,  403721,  403736,  403760, 15, 24, 22, }, // Irkutsk
    {  361, 1027,  403782,  403793,  403813, 11, 20, 16, }, // Israel
    {  361, 1034,  403829,  403838,  403856,  9, 18, 14, }, // Japan
    {  361, 1040,  403870,  403904,  403947, 34, 43, 41, }, // Kamchatka
    {  361, 1058,       0,  403988,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  361, 1077,       0,  404008,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  361, 1106,  404029,  404040,  404060, 11, 20, 16, }, // Korea
    {  361, 1112,       0,  404076,       0,  0,  9,  0, }, // Kosrae
    {  361, 1119,  404085,  404106,  404136, 21, 30, 28, }, // Krasnoyarsk
    {  361, 1151,       0,  404164,       0,  0, 16,  0, }, // Kyrgystan
    {  361, 1161,       0,  404180,       0,  0,  8,  0, }, // Lanka
    {  361, 1175,       0,  404188,       0,  0, 16,  0, }, // Line_Islands
    {  361, 1188,  404204,  404219,  404243, 15, 24, 22, }, // Lord_Howe
    {  361, 1198,  404265,  404274,  404292,  9, 18, 16, }, // Macau
    {  361, 1204,  404308,  404318,  404337, 10, 19, 17, }, // Magadan
    {  361, 1219,       0,  404354,       0,  0, 11,  0, }, // Malaysia
    {  361, 1228,       0,  404365,       0,  0, 14,  0, }, // Maldives
    {  361, 1237,       0,  404379,       0,  0, 15,  0, }, // Marquesas
    {  361, 1247,       0,  404394,       0,  0, 19,  0, }, // Marshall_Islands
    {  361, 1264,  404413,  404424,  404444, 11, 20, 18, }, // Mauritius
    {  361, 1274,       0,  404462,       0,  0, 12,  0, }, // Mawson
    {  361, 1281,  404474,  404496,  404527, 22, 31, 27, }, // Mexico_Pacific
    {  361, 1296,  404554,  404569,  404593, 15, 24, 22, }, // Mongolia
    {  361, 1305,  404615,  404625,  404644, 10, 19, 17, }, // Moscow
    {  361, 1312,       0,  404661,       0,  0, 14,  0, }, // Myanmar
    {  361, 1320,       0,  404675,       0,  0,  8,  0, }, // Nauru
    {  361, 1326,       0,  404683,       0,  0,  9,  0, }, // Nepal
    {  361, 1332,  404692,  404712,  404741, 20, 29, 28, }, // New_Caledonia
    {  361, 1346,  404769,  404786,  404812, 17, 26, 22, }, // New_Zealand
    {  361, 1358,  404834,  404855,  404885, 21, 30, 26, }, // Newfoundland
    {  361, 1371,       0,  404911,       0,  0,  8,  0, }, // Niue
    {  361, 1376,  404919,  404937,  404964, 18, 27, 25, }, // Norfolk
    {  361, 1384,  404989,  405014,  405048, 25, 34, 31, }, // Noronha
    {  361, 1392,       0,  405079,       0,  0, 25,  0, }, // North_Mariana
    {  361, 1406,  405104,  405123,  405151, 19, 28, 26, }, // Novosibirsk
    {  361, 1418,  405177,  405188,  405208, 11, 20, 18, }, // Omsk
    {  361, 1428,  405226,  405239,  405261, 13, 22, 20, }, // Pakistan
    {  361, 1437,       0,  405281,       0,  0, 10,  0, }, // Palau
    {  361, 1443,       0,  405291,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  361, 1460,  405312,  405323,  405343, 11, 20, 18, }, // Paraguay
    {  361, 1469,  405361,  405369,  405386,  8, 17, 15, }, // Peru
    {  361, 1474,  405401,  405413,  405434, 12, 21, 19, }, // Philippines
    {  361, 1486,       0,  405453,       0,  0, 20,  0, }, // Phoenix_Islands
    {  361, 1502,  405473,  405504,  405544, 31, 40, 36, }, // Pierre_Miquelon
    {  361, 1518,       0,  405580,       0,  0, 15,  0, }, // Pitcairn
    {  361, 1527,       0,  405595,       0,  0, 10,  0, }, // Ponape
    {  361, 1534,       0,  405605,       0,  0, 16,  0, }, // Pyongyang
    {  361, 1544,  405621,  405635,  405658, 14, 23, 21, }, // Qyzylorda
    {  361, 1554,       0,  405679,       0,  0, 13,  0, }, // Reunion
    {  361, 1562,       0,  405692,       0,  0, 10,  0, }, // Rothera
    {  361, 1570,  405702,  405718,  405743, 16, 25, 23, }, // Sakhalin
    {  361, 1579,  405766,  405773,  405789,  7, 16, 14, }, // Samara
    {  361, 1596,  405803,  405812,  405830,  9, 18, 16, }, // Samoa
    {  361, 1602,       0,  405846,       0,  0, 12,  0, }, // Seychelles
    {  361, 1624,       0,  405858,       0,  0, 21,  0, }, // Singapore
    {  361, 1634,       0,  405879,       0,  0, 18,  0, }, // Solomon
    {  361, 1642,       0,  405897,       0,  0, 19,  0, }, // South_Georgia
    {  361, 1656,       0,  405916,       0,  0, 12,  0, }, // Suriname
    {  361, 1676,       0,  405928,       0,  0, 10,  0, }, // Syowa
    {  361, 1682,       0,  405938,       0,  0, 10,  0, }, // Tahiti
    {  361, 1689,  405948,  405956,  405973,  8, 17, 13, }, // Taipei
    {  361, 1696,       0,  405986,       0,  0, 14,  0, }, // Tajikistan
    {  361, 1724,       0,  406000,       0,  0, 12,  0, }, // Tokelau
    {  361, 1732,  406012,  406021,  406039,  9, 18, 16, }, // Tonga
    {  361, 1738,       0,  406055,       0,  0,  8,  0, }, // Truk
    {  361, 1750,  406063,  406084,  406114, 21, 30, 28, }, // Turkmenistan
    {  361, 1763,       0,  406142,       0,  0, 10,  0, }, // Tuvalu
    {  361, 1777,  406152,  406163,  406183, 11, 20, 18, }, // Uruguay
    {  361, 1792,  406201,  406218,  406244, 17, 26, 24, }, // Uzbekistan
    {  361, 1803,  406268,  406277,  406295,  9, 18, 16, }, // Vanuatu
    {  361, 1811,       0,  406311,       0,  0, 14,  0, }, // Venezuela
    {  361, 1821,  406325,  406343,  406370, 18, 27, 25, }, // Vladivostok
    {  361, 1833,  406395,  406411,  406436, 16, 25, 23, }, // Volgograd
    {  361, 1843,       0,  406459,       0,  0, 12,  0, }, // Vostok
    {  361, 1850,       0,  406471,       0,  0, 14,  0, }, // Wake
    {  361, 1855,       0,  406485,       0,  0, 27,  0, }, // Wallis
    {  361, 1862,  406512,  406525,  406547, 13, 22, 20, }, // Yakutsk
    {  361, 1870,  406567,  406585,  406612, 18, 27, 26, }, // Yekaterinburg
    {  361, 1892,       0,  406638,       0,  0, 10,  0, }, // Yukon
    {  362,    0,  406648,  406659,  406678, 11, 19, 17, }, // Acre Kashmiri/Arabic/India
    {  362,    5,       0,  406695,       0,  0, 16,  0, }, // Afghanistan
    {  362,   17,       0,  406711,       0,  0, 19,  0, }, // Africa_Central
    {  362,   32,       0,  406730,       0,  0, 19,  0, }, // Africa_Eastern
    {  362,   65,       0,  406749,       0,  0, 19,  0, }, // Africa_Southern
    {  362,   81,  406768,  406787,  406814, 19, 27, 25, }, // Africa_Western
    {  362,  107,  406839,  406852,  406873, 13, 21, 22, }, // Alaska
    {  362,  128,  406895,  406908,  406929, 13, 21, 19, }, // Almaty
    {  362,  135,  406948,  406961,  406982, 13, 21, 19, }, // Amazon
    {  362,  142,  407001,  407012,  407031, 11, 19, 20, }, // America_Central
    {  362,  158,  407051,  407062,  407081, 11, 19, 20, }, // America_Eastern
    {  362,  174,  407101,  407114,  407135, 13, 21, 22, }, // America_Mountain
    {  362,  191,  407157,  407170,  407191, 13, 21, 22, }, // America_Pacific
    {  362,  207,  407213,  407227,  407249, 14, 22, 19, }, // Anadyr
    {  362,  219,  407268,  407280,  407300, 12, 20, 18, }, // Aqtau
    {  362,  225,  407318,  407330,  407350, 12, 20, 18, }, // Aqtobe
    {  362,  232,  407368,  407382,  407404, 14, 22, 23, }, // Arabian
    {  362,  240,  407427,  407443,  407467, 16, 24, 22, }, // Argentina
    {  362,  250,  407489,  407511,  407541, 22, 30, 28, }, // Argentina_Western
    {  362,  268,  407569,  407584,  407607, 15, 23, 21, }, // Armenia
    {  362,  286,  407628,  407643,  407666, 15, 23, 24, }, // Atlantic
    {  362,  295,  407690,  407712,  407742, 22, 30, 31, }, // Australia_Central
    {  362,  313,  407773,  407801,  407837, 28, 36, 37, }, // Australia_CentralWestern
    {  362,  338,  407874,  407896,  407926, 22, 30, 31, }, // Australia_Eastern
    {  362,  356,  407957,  407979,  408010, 22, 31, 32, }, // Australia_Western
    {  362,  374,  408042,  408057,  408080, 15, 23, 21, }, // Azerbaijan
    {  362,  385,  408101,  408114,  408135, 13, 21, 18, }, // Azores
    {  362,  397,  408153,  408168,  408191, 15, 23, 21, }, // Bangladesh
    {  362,  415,       0,  408212,       0,  0, 12,  0, }, // Bhutan
    {  362,  422,       0,  408224,       0,  0, 14,  0, }, // Bolivia
    {  362,  437,  408238,  408253,  408276, 15, 23, 21, }, // Brasilia
    {  362,  454,       0,  408297,       0,  0, 24,  0, }, // Brunei
    {  362,  461,  408321,  408335,  408357, 14, 22, 15, }, // Cape_Verde
    {  362,  478,       0,  408372,       0,  0, 20,  0, }, // Chamorro
    {  362,  487,  408392,  408404,  408424, 12, 20, 18, }, // Chatham
    {  362,  495,  408442,  408452,  408470, 10, 18, 16, }, // Chile
    {  362,  501,  408486,  408497,  408516, 11, 19, 20, }, // China
    {  362,  507,  408536,  408551,  408574, 15, 23, 21, }, // Choibalsan
    {  362,  518,       0,  408595,       0,  0, 12,  0, }, // Christmas
    {  362,  528,       0,  408607,       0,  0, 20,  0, }, // Cocos
    {  362,  534,  408627,  408642,  408665, 15, 23, 21, }, // Colombia
    {  362,  543,  408686,  408704,  408730, 18, 26, 28, }, // Cook
    {  362,  548,  408758,  408770,  408790, 12, 20, 21, }, // Cuba
    {  362,  559,       0,  408811,       0,  0, 11,  0, }, // Davis
    {  362,  575,       0,  408822,       0,  0, 22,  0, }, // DumontDUrville
    {  362,  612,       0,  408844,       0,  0, 18,  0, }, // East_Timor
    {  362,  623,  408862,  408881,  408908, 19, 27, 25, }, // Easter
    {  362,  630,       0,  408933,       0,  0, 14,  0, }, // Ecuador
    {  362,  638,  408947,  408965,  408991, 18, 26, 24, }, // Europe_Central
    {  362,  653,  409015,  409033,  409059, 18, 26, 24, }, // Europe_Eastern
    {  362,  668,       0,  409083,       0,  0, 21,  0, }, // Europe_Further_Eastern
    {  362,  691,  409104,  409123,  409150, 19, 27, 26, }, // Europe_Western
    {  362,  706,  409176,  409196,  409224, 20, 28, 26, }, // Falkland
    {  362,  715,  409250,  409261,  409280, 11, 19, 17, }, // Fiji
    {  362,  720,       0,  409297,       0,  0, 19,  0, }, // French_Guiana
    {  362,  734,       0,  409316,       0,  0, 30,  0, }, // French_Southern
    {  362,  757,       0,  409346,       0,  0, 15,  0, }, // Galapagos
    {  362,  767,       0,  409361,       0,  0, 14,  0, }, // Gambier
    {  362,  775,  409375,  409391,  409415, 16, 24, 22, }, // Georgia
    {  362,  783,       0,  409437,       0,  0, 20,  0, }, // Gilbert_Islands
    {  362,  799,       0,  409457,       0,  0, 20,  0, }, // GMT
    {  362,  841,  409477,  409502,  409535, 25, 33, 31, }, // Greenland_Eastern
    {  362,  859,  409566,  409591,  409624, 25, 33, 31, }, // Greenland_Western
    {  362,  877,       0,  409655,       0,  0, 11,  0, }, // Guam
    {  362,  882,       0,  409666,       0,  0, 18,  0, }, // Gulf
    {  362,  887,       0,  409684,       0,  0, 12,  0, }, // Guyana
    {  362,  894,  409696,  409719,  409750, 23, 31, 29, }, // Hawaii_Aleutian
    {  362,  910,  409779,  409793,  409815, 14, 22, 19, }, // Hong_Kong
    {  362,  920,  409834,  409844,  409862, 10, 18, 16, }, // Hovd
    {  362,  925,       0,  409878,       0,  0,  9,  0, }, // India
    {  362,  931,       0,  409887,       0,  0, 21,  0, }, // Indian_Ocean
    {  362,  944,       0,  409908,       0,  0, 16,  0, }, // Indochina
    {  362,  954,       0,  409924,       0,  0, 23,  0, }, // Indonesia_Central
    {  362,  972,       0,  409947,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  362,  990,       0,  409971,       0,  0, 24,  0, }, // Indonesia_Western
    {  362, 1008,  409995,  410009,  410031, 14, 22, 20, }, // Iran
    {  362, 1019,  410051,  410065,  410087, 14, 22, 20, }, // Irkutsk
    {  362, 1027,  410107,  410122,  410145, 15, 23, 24, }, // Israel
    {  362, 1034,  410169,  410181,  410201, 12, 20, 21, }, // Japan
    {  362, 1040,  410222,  410236,  410258, 14, 22, 20, }, // Kamchatka
    {  362, 1058,       0,  410278,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  362, 1077,       0,  410297,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  362, 1106,  410316,  410328,  410348, 12, 20, 21, }, // Korea
    {  362, 1112,       0,  410369,       0,  0, 12,  0, }, // Kosrae
    {  362, 1119,  410381,  410399,  410425, 18, 26, 24, }, // Krasnoyarsk
    {  362, 1151,       0,  410449,       0,  0, 13,  0, }, // Kyrgystan
    {  362, 1161,       0,  410462,       0,  0, 11,  0, }, // Lanka
    {  362, 1175,       0,  410473,       0,  0, 20,  0, }, // Line_Islands
    {  362, 1188,  410493,  410508,  410531, 15, 23, 18, }, // Lord_Howe
    {  362, 1198,  410549,  410562,  410583, 13, 21, 19, }, // Macau
    {  362, 1204,  410602,  410615,  410636, 13, 21, 19, }, // Magadan
    {  362, 1219,       0,  410655,       0,  0, 14,  0, }, // Malaysia
    {  362, 1228,       0,  410669,       0,  0, 15,  0, }, // Maldives
    {  362, 1237,       0,  410684,       0,  0, 16,  0, }, // Marquesas
    {  362, 1247,       0,  410700,       0,  0, 20,  0, }, // Marshall_Islands
    {  362, 1264,  410720,  410733,  410754, 13, 21, 19, }, // Mauritius
    {  362, 1274,       0,  410773,       0,  0, 11,  0, }, // Mawson
    {  362, 1281,  410784,  410803,  410830, 19, 27, 27, }, // Mexico_Pacific
    {  362, 1296,  410857,  410871,  410893, 14, 22, 21, }, // Mongolia
    {  362, 1305,  410914,  410926,  410945, 12, 19, 17, }, // Moscow
    {  362, 1312,       0,  410962,       0,  0, 14,  0, }, // Myanmar
    {  362, 1320,       0,  410976,       0,  0, 14,  0, }, // Nauru
    {  362, 1326,       0,  410990,       0,  0, 12,  0, }, // Nepal
    {  362, 1332,  411002,  411022,  411050, 20, 28, 26, }, // New_Caledonia
    {  362, 1346,  411076,  411091,  411114, 15, 23, 23, }, // New_Zealand
    {  362, 1358,  411137,  411156,  411183, 19, 27, 27, }, // Newfoundland
    {  362, 1371,       0,  411210,       0,  0, 11,  0, }, // Niue
    {  362, 1376,  411221,  411234,  411255, 13, 21, 19, }, // Norfolk
    {  362, 1384,  411274,  411297,  411328, 23, 31, 29, }, // Noronha
    {  362, 1392,       0,  411357,       0,  0, 21,  0, }, // North_Mariana
    {  362, 1406,  411378,  411395,  411420, 17, 25, 23, }, // Novosibirsk
    {  362, 1418,  411443,  411454,  411473, 11, 19, 17, }, // Omsk
    {  362, 1428,  411490,  411504,  411526, 14, 22, 20, }, // Pakistan
    {  362, 1437,       0,  411546,       0,  0, 11,  0, }, // Palau
    {  362, 1443,       0,  411557,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  362, 1460,  411579,  411592,  411613, 13, 21, 19, }, // Paraguay
    {  362, 1469,  411632,  411643,  411662, 11, 19, 17, }, // Peru
    {  362, 1474,  411679,  411695,  411719, 16, 24, 22, }, // Philippines
    {  362, 1486,       0,  411741,       0,  0, 21,  0, }, // Phoenix_Islands
    {  362, 1502,  411762,  411788,  411822, 26, 34, 35, }, // Pierre_Miquelon
    {  362, 1518,       0,  411857,       0,  0, 14,  0, }, // Pitcairn
    {  362, 1527,       0,  411871,       0,  0, 11,  0, }, // Ponape
    {  362, 1534,       0,  411882,       0,  0, 14,  0, }, // Pyongyang
    {  362, 1544,  411896,  411909,  411930, 13, 21, 19, }, // Qyzylorda
    {  362, 1554,       0,  411949,       0,  0, 16,  0, }, // Reunion
    {  362, 1562,       0,  411965,       0,  0, 13,  0, }, // Rothera
    {  362, 1570,  411978,  411992,  412014, 14, 22, 20, }, // Sakhalin
    {  362, 1579,  412034,  412045,  412064, 11, 19, 17, }, // Samara
    {  362, 1596,  412081,  412092,  412111, 11, 19, 17, }, // Samoa
    {  362, 1602,       0,  412128,       0,  0, 13,  0, }, // Seychelles
    {  362, 1624,       0,  412141,       0,  0, 15,  0, }, // Singapore
    {  362, 1634,       0,  412156,       0,  0, 26,  0, }, // Solomon
    {  362, 1642,       0,  412182,       0,  0, 20,  0, }, // South_Georgia
    {  362, 1656,       0,  412202,       0,  0, 13,  0, }, // Suriname
    {  362, 1676,       0,  412215,       0,  0, 11,  0, }, // Syowa
    {  362, 1682,       0,  412226,       0,  0, 12,  0, }, // Tahiti
    {  362, 1689,  412238,  412248,  412266, 10, 18, 18, }, // Taipei
    {  362, 1696,       0,  412284,       0,  0, 13,  0, }, // Tajikistan
    {  362, 1724,       0,  412297,       0,  0, 12,  0, }, // Tokelau
    {  362, 1732,  412309,  412321,  412341, 12, 20, 18, }, // Tonga
    {  362, 1738,       0,  412359,       0,  0, 10,  0, }, // Truk
    {  362, 1750,  412369,  412384,  412407, 15, 23, 24, }, // Turkmenistan
    {  362, 1763,       0,  412431,       0,  0, 14,  0, }, // Tuvalu
    {  362, 1777,  412445,  412460,  412483, 15, 23, 21, }, // Uruguay
    {  362, 1792,  412504,  412521,  412546, 17, 25, 25, }, // Uzbekistan
    {  362, 1803,  412571,  412587,  412611, 16, 24, 22, }, // Vanuatu
    {  362, 1811,       0,  412633,       0,  0, 16,  0, }, // Venezuela
    {  362, 1821,  412649,  412666,  412691, 17, 25, 23, }, // Vladivostok
    {  362, 1833,  412714,  412729,  412752, 15, 23, 21, }, // Volgograd
    {  362, 1843,       0,  412654,       0,  0, 12,  0, }, // Vostok
    {  362, 1850,       0,  412773,       0,  0, 17,  0, }, // Wake
    {  362, 1855,       0,  412790,       0,  0, 26,  0, }, // Wallis
    {  362, 1862,  412816,  412829,  412850, 13, 21, 20, }, // Yakutsk
    {  362, 1870,  412870,  412889,  412916, 19, 27, 24, }, // Yekaterinburg
    {  362, 1892,       0,  412940,       0,  0, 10,  0, }, // Yukon
    {  363,  142,  412950,  412960,  412981, 10, 21, 18, }, // America_Central Kashmiri/Devanagari/India
    {  363,  158,  412999,  413008,  413028,  9, 20, 17, }, // America_Eastern
    {  363,  174,  413045,  413055,  413076, 10, 21, 18, }, // America_Mountain
    {  363,  191,  413094,  413104,  413125, 10, 21, 18, }, // America_Pacific
    {  363,  286,  413143,  413154,  413176, 11, 22, 19, }, // Atlantic
    {  363,  638,  413195,  413210,  413236, 15, 26, 19, }, // Europe_Central
    {  363,  653,  413255,  413270,  413296, 15, 26, 19, }, // Europe_Eastern
    {  363,  691,  413315,  413330,  413356, 15, 26, 19, }, // Europe_Western
    {  363,  799,       0,  413375,       0,  0, 15,  0, }, // GMT
    {  364,    5,       0,  413390,       0,  0, 16,  0, }, // Afghanistan Kazakh/Cyrillic/Kazakhstan
    {  364,   17,       0,  413406,       0,  0, 21,  0, }, // Africa_Central
    {  364,   32,       0,  413427,       0,  0, 19,  0, }, // Africa_Eastern
    {  364,   65,       0,  413446,       0,  0, 33,  0, }, // Africa_Southern
    {  364,   81,  413479,  413498,  413528, 19, 30, 25, }, // Africa_Western
    {  364,  107,  413553,  413566,  413590, 13, 24, 19, }, // Alaska
    {  364,  135,  413609,  413624,  413650, 15, 26, 21, }, // Amazon
    {  364,  142,  413671,  413703,  413746, 32, 43, 38, }, // America_Central
    {  364,  158,  413784,  413814,  413855, 30, 41, 36, }, // America_Eastern
    {  364,  174,  413891,  413919,  413958, 28, 39, 34, }, // America_Mountain
    {  364,  191,  413992,  414029,  414077, 37, 48, 43, }, // America_Pacific
    {  364,  214,  414120,  414131,  414153, 11, 22, 17, }, // Apia
    {  364,  232,  414170,  414190,  414221, 20, 31, 26, }, // Arabian
    {  364,  240,  414247,  414263,  414290, 16, 27, 22, }, // Argentina
    {  364,  250,  414312,  414334,  414367, 22, 33, 28, }, // Argentina_Western
    {  364,  268,  414395,  414409,  414434, 14, 25, 20, }, // Armenia
    {  364,  286,  414454,  414470,  414497, 16, 27, 22, }, // Atlantic
    {  364,  295,  414519,  414543,  414578, 24, 35, 30, }, // Australia_Central
    {  364,  313,  414608,  414638,  414679, 30, 41, 36, }, // Australia_CentralWestern
    {  364,  338,  414715,  414737,  414770, 22, 33, 28, }, // Australia_Eastern
    {  364,  356,  414798,  414820,  414853, 22, 33, 28, }, // Australia_Western
    {  364,  374,  414881,  414898,  414926, 17, 28, 23, }, // Azerbaijan
    {  364,  385,  414949,  414969,  415000, 20, 31, 26, }, // Azores
    {  364,  397,  415026,  415042,  415069, 16, 27, 22, }, // Bangladesh
    {  364,  415,       0,  415091,       0,  0, 12,  0, }, // Bhutan
    {  364,  422,       0,  415103,       0,  0, 14,  0, }, // Bolivia
    {  364,  437,  415117,  415132,  415158, 15, 26, 21, }, // Brasilia
    {  364,  454,       0,  415179,       0,  0, 24,  0, }, // Brunei
    {  364,  461,  415203,  415220,  415248, 17, 28, 23, }, // Cape_Verde
    {  364,  478,       0,  415271,       0,  0, 25,  0, }, // Chamorro
    {  364,  487,  415296,  415308,  415331, 12, 23, 18, }, // Chatham
    {  364,  495,  415349,  415360,  415382, 11, 22, 17, }, // Chile
    {  364,  501,  415399,  415411,  415434, 12, 23, 18, }, // China
    {  364,  507,  415452,  415468,  415495, 16, 27, 22, }, // Choibalsan
    {  364,  518,       0,  415517,       0,  0, 25,  0, }, // Christmas
    {  364,  528,       0,  415542,       0,  0, 24,  0, }, // Cocos
    {  364,  534,  415566,  415581,  415607, 15, 26, 21, }, // Colombia
    {  364,  543,  415628,  415650,  415683, 22, 33, 37, }, // Cook
    {  364,  548,  415720,  415731,  415753, 11, 22, 17, }, // Cuba
    {  364,  559,       0,  415770,       0,  0, 13,  0, }, // Davis
    {  364,  575,       0,  415783,       0,  0, 21,  0, }, // DumontDUrville
    {  364,  612,       0,  415804,       0,  0, 18,  0, }, // East_Timor
    {  364,  623,  415822,  415840,  415869, 18, 29, 24, }, // Easter
    {  364,  630,       0,  415893,       0,  0, 14,  0, }, // Ecuador
    {  364,  638,  415907,  415928,  415960, 21, 32, 27, }, // Europe_Central
    {  364,  653,  415987,  416006,  416036, 19, 30, 25, }, // Europe_Eastern
    {  364,  668,       0,  416061,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  364,  691,  416085,  416104,  416134, 19, 30, 25, }, // Europe_Western
    {  364,  706,  416159,  416183,  416218, 24, 35, 30, }, // Falkland
    {  364,  715,  416248,  416260,  416283, 12, 23, 18, }, // Fiji
    {  364,  720,       0,  416301,       0,  0, 23,  0, }, // French_Guiana
    {  364,  734,       0,  416324,       0,  0, 49,  0, }, // French_Southern
    {  364,  757,       0,  416373,       0,  0, 16,  0, }, // Galapagos
    {  364,  767,       0,  416389,       0,  0, 13,  0, }, // Gambier
    {  364,  775,  416402,  416415,  416439, 13, 24, 19, }, // Georgia
    {  364,  783,       0,  416458,       0,  0, 26,  0, }, // Gilbert_Islands
    {  364,  799,       0,  416484,       0,  0, 14,  0, }, // GMT
    {  364,  841,  416498,  416521,  416555, 23, 34, 29, }, // Greenland_Eastern
    {  364,  859,  416584,  416607,  416641, 23, 34, 29, }, // Greenland_Western
    {  364,  882,       0,  416670,       0,  0, 32,  0, }, // Gulf
    {  364,  887,       0,  416702,       0,  0, 13,  0, }, // Guyana
    {  364,  894,  416715,  416747,  416790, 32, 43, 38, }, // Hawaii_Aleutian
    {  364,  910,  416828,  416842,  416867, 14, 25, 20, }, // Hong_Kong
    {  364,  920,  416887,  416898,  416920, 11, 22, 17, }, // Hovd
    {  364,  925,       0,  416937,       0,  0, 26,  0, }, // India
    {  364,  931,       0,  416963,       0,  0, 18,  0, }, // Indian_Ocean
    {  364,  944,       0,  416981,       0,  0, 16,  0, }, // Indochina
    {  364,  954,       0,  416997,       0,  0, 24,  0, }, // Indonesia_Central
    {  364,  972,       0,  417021,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  364,  990,       0,  417043,       0,  0, 22,  0, }, // Indonesia_Western
    {  364, 1008,  417065,  417076,  417098, 11, 22, 17, }, // Iran
    {  364, 1019,  417115,  417129,  417154, 14, 25, 20, }, // Irkutsk
    {  364, 1027,  417174,  417188,  417213, 14, 25, 20, }, // Israel
    {  364, 1034,  417233,  417247,  417272, 14, 25, 20, }, // Japan
    {  364, 1058,       0,  417292,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  364, 1077,       0,  417314,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  364, 1106,  417336,  417348,  417371, 12, 23, 18, }, // Korea
    {  364, 1112,       0,  417389,       0,  0, 13,  0, }, // Kosrae
    {  364, 1119,  417402,  417419,  417447, 17, 28, 23, }, // Krasnoyarsk
    {  364, 1151,       0,  417470,       0,  0, 17,  0, }, // Kyrgystan
    {  364, 1175,       0,  417487,       0,  0, 20,  0, }, // Line_Islands
    {  364, 1188,  417507,  417522,  417548, 15, 26, 21, }, // Lord_Howe
    {  364, 1204,  417569,  417583,  417608, 14, 25, 20, }, // Magadan
    {  364, 1219,       0,  417628,       0,  0, 15,  0, }, // Malaysia
    {  364, 1228,       0,  417643,       0,  0, 23,  0, }, // Maldives
    {  364, 1237,       0,  417666,       0,  0, 22,  0, }, // Marquesas
    {  364, 1247,       0,  417688,       0,  0, 23,  0, }, // Marshall_Islands
    {  364, 1264,  417711,  417726,  417752, 15, 26, 21, }, // Mauritius
    {  364, 1274,       0,  417773,       0,  0, 13,  0, }, // Mawson
    {  364, 1281,  417786,  417812,  417849, 26, 37, 32, }, // Mexico_Pacific
    {  364, 1296,  417881,  417897,  417924, 16, 27, 22, }, // Mongolia
    {  364, 1305,  417946,  417959,  417983, 13, 24, 19, }, // Moscow
    {  364, 1312,       0,  418002,       0,  0, 13,  0, }, // Myanmar
    {  364, 1320,       0,  418015,       0,  0, 12,  0, }, // Nauru
    {  364, 1326,       0,  418027,       0,  0, 12,  0, }, // Nepal
    {  364, 1332,  418039,  418060,  418092, 21, 32, 27, }, // New_Caledonia
    {  364, 1346,  418119,  418139,  418170, 20, 31, 26, }, // New_Zealand
    {  364, 1358,  418196,  418215,  418245, 19, 30, 25, }, // Newfoundland
    {  364, 1371,       0,  418270,       0,  0, 11,  0, }, // Niue
    {  364, 1376,  418281,  418301,  418332, 20, 31, 26, }, // Norfolk
    {  364, 1384,  418358,  418384,  418421, 26, 37, 32, }, // Noronha
    {  364, 1406,  418453,  418469,  418496, 16, 27, 22, }, // Novosibirsk
    {  364, 1418,  418518,  418529,  418551, 11, 22, 17, }, // Omsk
    {  364, 1428,  418568,  418583,  418609, 15, 26, 21, }, // Pakistan
    {  364, 1437,       0,  418630,       0,  0, 12,  0, }, // Palau
    {  364, 1443,       0,  418642,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  364, 1460,  418668,  418683,  418709, 15, 26, 21, }, // Paraguay
    {  364, 1469,  418730,  418741,  418763, 11, 22, 17, }, // Peru
    {  364, 1474,  418780,  418804,  418839, 24, 35, 30, }, // Philippines
    {  364, 1486,       0,  418869,       0,  0, 22,  0, }, // Phoenix_Islands
    {  364, 1502,  418891,  418919,  418958, 28, 39, 34, }, // Pierre_Miquelon
    {  364, 1518,       0,  418992,       0,  0, 14,  0, }, // Pitcairn
    {  364, 1527,       0,  419006,       0,  0, 13,  0, }, // Ponape
    {  364, 1534,       0,  419019,       0,  0, 14,  0, }, // Pyongyang
    {  364, 1554,       0,  419033,       0,  0, 14,  0, }, // Reunion
    {  364, 1562,       0,  419047,       0,  0, 13,  0, }, // Rothera
    {  364, 1570,  419060,  419074,  419099, 14, 25, 20, }, // Sakhalin
    {  364, 1596,  419119,  419131,  419154, 12, 23, 18, }, // Samoa
    {  364, 1602,       0,  419172,       0,  0, 23,  0, }, // Seychelles
    {  364, 1624,       0,  419195,       0,  0, 26,  0, }, // Singapore
    {  364, 1634,       0,  419221,       0,  0, 23,  0, }, // Solomon
    {  364, 1642,       0,  419244,       0,  0, 23,  0, }, // South_Georgia
    {  364, 1656,       0,  419267,       0,  0, 14,  0, }, // Suriname
    {  364, 1676,       0,  419281,       0,  0, 11,  0, }, // Syowa
    {  364, 1682,       0,  419292,       0,  0, 12,  0, }, // Tahiti
    {  364, 1689,  419304,  419317,  419341, 13, 24, 19, }, // Taipei
    {  364, 1696,       0,  419360,       0,  0, 16,  0, }, // Tajikistan
    {  364, 1724,       0,  419376,       0,  0, 14,  0, }, // Tokelau
    {  364, 1732,  419390,  419402,  419425, 12, 23, 18, }, // Tonga
    {  364, 1738,       0,  419443,       0,  0, 11,  0, }, // Truk
    {  364, 1750,  419454,  419473,  419503, 19, 30, 25, }, // Turkmenistan
    {  364, 1763,       0,  419528,       0,  0, 13,  0, }, // Tuvalu
    {  364, 1777,  419541,  419555,  419580, 14, 25, 20, }, // Uruguay
    {  364, 1792,  419600,  419616,  419643, 16, 27, 22, }, // Uzbekistan
    {  364, 1803,  419665,  419679,  419704, 14, 25, 20, }, // Vanuatu
    {  364, 1811,       0,  419724,       0,  0, 16,  0, }, // Venezuela
    {  364, 1821,  419740,  419758,  419787, 18, 29, 24, }, // Vladivostok
    {  364, 1833,  419811,  419827,  419854, 16, 27, 22, }, // Volgograd
    {  364, 1843,       0,  419876,       0,  0, 13,  0, }, // Vostok
    {  364, 1850,       0,  419889,       0,  0, 17,  0, }, // Wake
    {  364, 1855,       0,  419906,       0,  0, 25,  0, }, // Wallis
    {  364, 1862,  419931,  419944,  419968, 13, 24, 19, }, // Yakutsk
    {  364, 1870,  419987,  420006,  420036, 19, 30, 25, }, // Yekaterinburg
    {  364, 1892,       0,  420061,       0,  0, 11,  0, }, // Yukon
    {  366,    5,       0,  420072,       0,  0, 24,  0, }, // Afghanistan Khmer/Khmer/Cambodia
    {  366,   17,       0,  420096,       0,  0, 24,  0, }, // Africa_Central
    {  366,   32,       0,  420120,       0,  0, 25,  0, }, // Africa_Eastern
    {  366,   65,       0,  420145,       0,  0, 27,  0, }, // Africa_Southern
    {  366,   81,  420172,  420197,  420231, 25, 34, 39, }, // Africa_Western
    {  366,  107,  420270,  420286,  420311, 16, 25, 26, }, // Alaska
    {  366,  135,  420337,  420355,  420381, 18, 26, 28, }, // Amazon
    {  366,  142,  420409,  420447,  420493, 38, 46, 45, }, // America_Central
    {  366,  158,  420538,  420570,  420611, 32, 41, 40, }, // America_Eastern
    {  366,  174,  420651,  420692,  420741, 41, 49, 48, }, // America_Mountain
    {  366,  191,  420789,  420813,  420845, 24, 32, 31, }, // America_Pacific
    {  366,  214,  420876,  420890,  420912, 14, 22, 23, }, // Apia
    {  366,  232,  420935,  420950,  420974, 15, 24, 24, }, // Arabian
    {  366,  240,  420998,  421016,  421042, 18, 26, 28, }, // Argentina
    {  366,  250,  421070,  421097,  421132, 27, 35, 37, }, // Argentina_Western
    {  366,  268,  421169,  421183,  421206, 14, 23, 26, }, // Armenia
    {  366,  286,  421232,  421250,  421277, 18, 27, 27, }, // Atlantic
    {  366,  295,  421304,  421328,  421362, 24, 34, 36, }, // Australia_Central
    {  366,  313,  421398,  421440,  421489, 42, 49, 50, }, // Australia_CentralWestern
    {  366,  338,  421539,  421565,  421600, 26, 35, 35, }, // Australia_Eastern
    {  366,  356,  421635,  421663,  421699, 28, 36, 35, }, // Australia_Western
    {  366,  374,  421734,  421754,  421783, 20, 29, 32, }, // Azerbaijan
    {  366,  385,  421815,  421829,  421852, 14, 23, 26, }, // Azores
    {  366,  397,  421878,  421896,  421923, 18, 27, 30, }, // Bangladesh
    {  366,  415,       0,  421953,       0,  0, 12,  0, }, // Bhutan
    {  366,  422,       0,  421965,       0,  0, 14,  0, }, // Bolivia
    {  366,  437,  421979,  421999,  422028, 20, 29, 33, }, // Brasilia
    {  366,  454,       0,  422061,       0,  0, 22,  0, }, // Brunei
    {  366,  461,  422083,  422098,  422122, 15, 24, 27, }, // Cape_Verde
    {  366,  478,       0,  422149,       0,  0, 25,  0, }, // Chamorro
    {  366,  487,  422174,  422187,  422209, 13, 22, 22, }, // Chatham
    {  366,  495,  422231,  422242,  422261, 11, 19, 21, }, // Chile
    {  366,  501,  422282,  422293,  422313, 11, 20, 20, }, // China
    {  366,  507,  422333,  422350,  422376, 17, 26, 29, }, // Choibalsan
    {  366,  518,       0,  422405,       0,  0, 21,  0, }, // Christmas
    {  366,  528,       0,  422426,       0,  0, 23,  0, }, // Cocos
    {  366,  534,  422449,  422465,  422490, 16, 25, 28, }, // Colombia
    {  366,  543,  422518,  422539,  422569, 21, 30, 45, }, // Cook
    {  366,  548,  422614,  422627,  422649, 13, 22, 22, }, // Cuba
    {  366,  559,       0,  422671,       0,  0, 13,  0, }, // Davis
    {  366,  575,       0,  422684,       0,  0, 24,  0, }, // DumontDUrville
    {  366,  612,       0,  422708,       0,  0, 22,  0, }, // East_Timor
    {  366,  623,  422730,  422747,  422772, 17, 25, 27, }, // Easter
    {  366,  630,       0,  422799,       0,  0, 17,  0, }, // Ecuador
    {  366,  638,  422816,  422837,  422867, 21, 30, 33, }, // Europe_Central
    {  366,  653,  422900,  422924,  422958, 24, 34, 36, }, // Europe_Eastern
    {  366,  668,       0,  422994,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  366,  691,  423025,  423047,  423078, 22, 31, 34, }, // Europe_Western
    {  366,  706,  423112,  423138,  423173, 26, 35, 38, }, // Falkland
    {  366,  715,  423211,  423225,  423248, 14, 23, 28, }, // Fiji
    {  366,  720,       0,  423276,       0,  0, 23,  0, }, // French_Guiana
    {  366,  734,       0,  423299,       0,  0, 36,  0, }, // French_Southern
    {  366,  757,       0,  423335,       0,  0, 18,  0, }, // Galapagos
    {  366,  767,       0,  423353,       0,  0, 13,  0, }, // Gambier
    {  366,  775,  423366,  423383,  423409, 17, 26, 30, }, // Georgia
    {  366,  783,       0,  423439,       0,  0, 18,  0, }, // Gilbert_Islands
    {  366,  799,       0,  423457,       0,  0, 14,  0, }, // GMT
    {  366,  841,  423471,  423498,  423536, 27, 38, 37, }, // Greenland_Eastern
    {  366,  859,  423573,  423595,  423625, 22, 30, 32, }, // Greenland_Western
    {  366,  882,       0,  423657,       0,  0, 20,  0, }, // Gulf
    {  366,  887,       0,  423276,       0,  0, 16,  0, }, // Guyana
    {  366,  894,  423677,  423702,  423736, 25, 34, 33, }, // Hawaii_Aleutian
    {  366,  910,  423769,  423783,  423806, 14, 23, 26, }, // Hong_Kong
    {  366,  920,  423832,  423843,  423862, 11, 19, 23, }, // Hovd
    {  366,  925,       0,  423885,       0,  0, 21,  0, }, // India
    {  366,  931,       0,  423906,       0,  0, 23,  0, }, // Indian_Ocean
    {  366,  944,       0,  423929,       0,  0, 16,  0, }, // Indochina
    {  366,  954,       0,  423945,       0,  0, 27,  0, }, // Indonesia_Central
    {  366,  972,       0,  423972,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  366,  990,       0,  423999,       0,  0, 27,  0, }, // Indonesia_Western
    {  366, 1008,  424026,  424041,  424065, 15, 24, 24, }, // Iran
    {  366, 1019,  424089,  424106,  424131, 17, 25, 27, }, // Irkutsk
    {  366, 1027,  424158,  424176,  424203, 18, 27, 27, }, // Israel
    {  366, 1034,  424230,  424243,  424265, 13, 22, 21, }, // Japan
    {  366, 1058,       0,  424286,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  366, 1077,       0,  424313,       0,  0, 31,  0, }, // Kazakhstan_Western
    {  366, 1106,  424344,  424357,  424379, 13, 22, 22, }, // Korea
    {  366, 1112,       0,  424401,       0,  0, 14,  0, }, // Kosrae
    {  366, 1119,  424415,  424433,  424460, 18, 27, 30, }, // Krasnoyarsk
    {  366, 1151,       0,  424490,       0,  0, 21,  0, }, // Kyrgystan
    {  366, 1175,       0,  424511,       0,  0, 15,  0, }, // Line_Islands
    {  366, 1188,  424526,  424538,  424560, 12, 22, 21, }, // Lord_Howe
    {  366, 1204,  424581,  424599,  424626, 18, 27, 30, }, // Magadan
    {  366, 1219,       0,  424656,       0,  0, 16,  0, }, // Malaysia
    {  366, 1228,       0,  424672,       0,  0, 16,  0, }, // Maldives
    {  366, 1237,       0,  424688,       0,  0, 21,  0, }, // Marquesas
    {  366, 1247,       0,  424709,       0,  0, 14,  0, }, // Marshall_Islands
    {  366, 1264,  424723,  424737,  424760, 14, 23, 24, }, // Mauritius
    {  366, 1274,       0,  424784,       0,  0, 16,  0, }, // Mawson
    {  366, 1281,  424800,  424828,  424864, 28, 36, 37, }, // Mexico_Pacific
    {  366, 1296,  424901,  424920,  424948, 19, 28, 31, }, // Mongolia
    {  366, 1305,  424979,  424993,  425016, 14, 23, 26, }, // Moscow
    {  366, 1312,       0,  425042,       0,  0, 18,  0, }, // Myanmar
    {  366, 1320,       0,  425060,       0,  0, 12,  0, }, // Nauru
    {  366, 1326,       0,  425072,       0,  0, 13,  0, }, // Nepal
    {  366, 1332,  425085,  425106,  425135, 21, 29, 31, }, // New_Caledonia
    {  366, 1346,  425166,  425184,  425211, 18, 27, 27, }, // New_Zealand
    {  366, 1358,  425238,  425257,  425287, 19, 30, 27, }, // Newfoundland
    {  366, 1371,       0,  425314,       0,  0, 12,  0, }, // Niue
    {  366, 1376,  425326,  425346,  425371, 20, 25, 27, }, // Norfolk
    {  366, 1384,  425398,  425427,  425464, 29, 37, 39, }, // Noronha
    {  366, 1406,  425503,  425523,  425552, 20, 29, 32, }, // Novosibirsk
    {  366, 1418,  425584,  425595,  425615, 11, 20, 23, }, // Omsk
    {  366, 1428,  425638,  425656,  425683, 18, 27, 30, }, // Pakistan
    {  366, 1437,       0,  425713,       0,  0, 12,  0, }, // Palau
    {  366, 1443,       0,  425725,       0,  0, 29,  0, }, // Papua_New_Guinea
    {  366, 1460,  425754,  425773,  425801, 19, 28, 30, }, // Paraguay
    {  366, 1469,  425831,  425844,  425866, 13, 22, 25, }, // Peru
    {  366, 1474,  425891,  425908,  425934, 17, 26, 30, }, // Philippines
    {  366, 1486,       0,  425964,       0,  0, 17,  0, }, // Phoenix_Islands
    {  366, 1502,  425981,  426012,  426050, 31, 38, 38, }, // Pierre_Miquelon
    {  366, 1518,       0,  426088,       0,  0, 13,  0, }, // Pitcairn
    {  366, 1527,       0,  426101,       0,  0, 15,  0, }, // Ponape
    {  366, 1534,       0,  426116,       0,  0, 15,  0, }, // Pyongyang
    {  366, 1554,       0,  426131,       0,  0, 15,  0, }, // Reunion
    {  366, 1562,       0,  426146,       0,  0, 16,  0, }, // Rothera
    {  366, 1570,  426162,  426179,  426205, 17, 26, 29, }, // Sakhalin
    {  366, 1596,  426234,  426247,  426268, 13, 21, 23, }, // Samoa
    {  366, 1602,       0,  426291,       0,  0, 15,  0, }, // Seychelles
    {  366, 1624,       0,  426306,       0,  0, 17,  0, }, // Singapore
    {  366, 1634,       0,  426323,       0,  0, 20,  0, }, // Solomon
    {  366, 1642,       0,  426343,       0,  0, 21,  0, }, // South_Georgia
    {  366, 1656,       0,  426364,       0,  0, 15,  0, }, // Suriname
    {  366, 1676,       0,  426379,       0,  0, 16,  0, }, // Syowa
    {  366, 1682,       0,  426395,       0,  0, 14,  0, }, // Tahiti
    {  366, 1689,  426409,  426422,  426444, 13, 22, 22, }, // Taipei
    {  366, 1696,       0,  426466,       0,  0, 17,  0, }, // Tajikistan
    {  366, 1724,       0,  426483,       0,  0, 14,  0, }, // Tokelau
    {  366, 1732,  426497,  426512,  426537, 15, 25, 27, }, // Tonga
    {  366, 1738,       0,  426564,       0,  0, 13,  0, }, // Truk
    {  366, 1750,  426577,  426598,  426627, 21, 29, 32, }, // Turkmenistan
    {  366, 1763,       0,  426659,       0,  0, 16,  0, }, // Tuvalu
    {  366, 1777,  426675,  426694,  426722, 19, 28, 32, }, // Uruguay
    {  366, 1792,  426754,  426775,  426805, 21, 30, 33, }, // Uzbekistan
    {  366, 1803,  426838,  426853,  426878, 15, 25, 27, }, // Vanuatu
    {  366, 1811,       0,  426905,       0,  0, 21,  0, }, // Venezuela
    {  366, 1821,  426926,  426949,  426981, 23, 32, 35, }, // Vladivostok
    {  366, 1833,  427016,  427036,  427065, 20, 29, 32, }, // Volgograd
    {  366, 1843,       0,  427097,       0,  0, 16,  0, }, // Vostok
    {  366, 1850,       0,  427113,       0,  0, 14,  0, }, // Wake
    {  366, 1855,       0,  427127,       0,  0, 27,  0, }, // Wallis
    {  366, 1862,  427154,  427173,  427201, 19, 28, 31, }, // Yakutsk
    {  366, 1870,  427232,  427254,  427285, 22, 31, 35, }, // Yekaterinburg
    {  366, 1892,       0,  427320,       0,  0, 11,  0, }, // Yukon
    {  370,    5,       0,  427331,       0,  0, 15,  0, }, // Afghanistan Konkani/Devanagari/India
    {  370,   17,       0,  427346,       0,  0, 16,  0, }, // Africa_Central
    {  370,   32,       0,  427362,       0,  0, 17,  0, }, // Africa_Eastern
    {  370,   65,       0,  427379,       0,  0, 27,  0, }, // Africa_Southern
    {  370,   81,  427406,  427424,  427451, 18, 27, 26, }, // Africa_Western
    {  370,  107,  427477,  427488,  427508, 11, 20, 18, }, // Alaska
    {  370,  135,  427526,  427536,  427555, 10, 19, 18, }, // Amazon
    {  370,  142,  427573,  427581,  427598,  8, 17, 15, }, // America_Central
    {  370,  158,  427613,  427622,  427640,  9, 18, 16, }, // America_Eastern
    {  370,  174,  427656,  427667,  427687, 11, 20, 18, }, // America_Mountain
    {  370,  191,  427705,  427716,  427736, 11, 20, 18, }, // America_Pacific
    {  370,  214,  427754,  427763,  427781,  9, 18, 16, }, // Apia
    {  370,  232,  427797,  427805,  427822,  8, 17, 15, }, // Arabian
    {  370,  240,  427837,  427851,  427874, 14, 23, 22, }, // Argentina
    {  370,  250,  427896,  427917,  427947, 21, 30, 29, }, // Argentina_Western
    {  370,  268,  427976,  427989,  428011, 13, 22, 21, }, // Armenia
    {  370,  286,  428032,  428044,  428065, 12, 21, 19, }, // Atlantic
    {  370,  295,  428084,  428104,  428133, 20, 29, 27, }, // Australia_Central
    {  370,  313,  428160,  428187,  428223, 27, 36, 34, }, // Australia_CentralWestern
    {  370,  338,  428257,  428278,  428308, 21, 30, 28, }, // Australia_Eastern
    {  370,  356,  428336,  428358,  428389, 22, 31, 29, }, // Australia_Western
    {  370,  374,  428418,  428430,  428451, 12, 21, 20, }, // Azerbaijan
    {  370,  385,  428471,  428477,  428496,  6, 19, 18, }, // Azores
    {  370,  397,  428514,  428527,  428549, 13, 22, 21, }, // Bangladesh
    {  370,  415,       0,  428570,       0,  0,  5,  0, }, // Bhutan
    {  370,  422,       0,  428575,       0,  0, 14,  0, }, // Bolivia
    {  370,  437,  428589,  428603,  428626, 14, 23, 22, }, // Brasilia
    {  370,  454,       0,  428648,       0,  0, 22,  0, }, // Brunei
    {  370,  461,  428670,  428683,  428705, 13, 22, 21, }, // Cape_Verde
    {  370,  478,       0,  428726,       0,  0, 19,  0, }, // Chamorro
    {  370,  487,  428745,  428753,  428770,  8, 17, 15, }, // Chatham
    {  370,  495,  428785,  428793,  428810,  8, 17, 16, }, // Chile
    {  370,  501,  428826,  428833,  428849,  7, 16, 14, }, // China
    {  370,  507,  428863,  428876,  428898, 13, 22, 21, }, // Choibalsan
    {  370,  518,       0,  428919,       0,  0, 17,  0, }, // Christmas
    {  370,  528,       0,  428936,       0,  0, 15,  0, }, // Cocos
    {  370,  534,  428951,  428963,  428984, 12, 21, 20, }, // Colombia
    {  370,  543,  429004,  429017,  429039, 13, 22, 26, }, // Cook
    {  370,  548,  429065,  429075,  429094, 10, 19, 17, }, // Cuba
    {  370,  559,       0,  429111,       0,  0, 11,  0, }, // Davis
    {  370,  575,       0,  429122,       0,  0, 18,  0, }, // DumontDUrville
    {  370,  612,       0,  429140,       0,  0, 15,  0, }, // East_Timor
    {  370,  623,  429155,  429170,  429194, 15, 24, 23, }, // Easter
    {  370,  630,       0,  429217,       0,  0, 12,  0, }, // Ecuador
    {  370,  638,  429229,  429246,  429272, 17, 26, 25, }, // Europe_Central
    {  370,  653,  429297,  429315,  429342, 18, 27, 26, }, // Europe_Eastern
    {  370,  668,       0,  429368,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  370,  691,  429391,  429410,  429438, 19, 28, 27, }, // Europe_Western
    {  370,  706,  429465,  429483,  429510, 18, 27, 26, }, // Falkland
    {  370,  715,  429536,  429544,  429561,  8, 17, 16, }, // Fiji
    {  370,  720,       0,  429577,       0,  0, 16,  0, }, // French_Guiana
    {  370,  734,       0,  429593,       0,  0, 34,  0, }, // French_Southern
    {  370,  757,       0,  429627,       0,  0, 13,  0, }, // Galapagos
    {  370,  767,       0,  429640,       0,  0, 12,  0, }, // Gambier
    {  370,  775,  429652,  429664,  429685, 12, 21, 20, }, // Georgia
    {  370,  783,       0,  429705,       0,  0, 18,  0, }, // Gilbert_Islands
    {  370,  799,       0,  429723,       0,  0, 17,  0, }, // GMT
    {  370,  841,  429740,  429758,  429785, 18, 27, 26, }, // Greenland_Eastern
    {  370,  859,  429811,  429830,  429858, 19, 28, 27, }, // Greenland_Western
    {  370,  882,       0,  429885,       0,  0, 17,  0, }, // Gulf
    {  370,  887,       0,  429902,       0,  0, 10,  0, }, // Guyana
    {  370,  894,  429912,  429929,  429955, 17, 26, 24, }, // Hawaii_Aleutian
    {  370,  910,  429979,  429992,  430014, 13, 22, 21, }, // Hong_Kong
    {  370,  920,  430035,  430041,  430060,  6, 19, 18, }, // Hovd
    {  370,  925,       0,  430078,       0,  0, 19,  0, }, // India
    {  370,  931,       0,  430097,       0,  0, 17,  0, }, // Indian_Ocean
    {  370,  944,       0,  430114,       0,  0, 11,  0, }, // Indochina
    {  370,  954,       0,  430125,       0,  0, 19,  0, }, // Indonesia_Central
    {  370,  972,       0,  430144,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  370,  990,       0,  430164,       0,  0, 21,  0, }, // Indonesia_Western
    {  370, 1008,   89805,  430185,  430202,  4, 17, 15, }, // Iran
    {  370, 1019,  430217,  430230,  430252, 13, 22, 21, }, // Irkutsk
    {  370, 1027,  430273,  430284,  430304, 11, 20, 18, }, // Israel
    {  370, 1034,  430322,  430330,  430347,  8, 17, 15, }, // Japan
    {  370, 1058,       0,  430362,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  370, 1077,       0,  430381,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  370, 1106,  430401,  430411,  430430, 10, 19, 17, }, // Korea
    {  370, 1112,       0,  430447,       0,  0,  9,  0, }, // Kosrae
    {  370, 1119,  430456,  430475,  430503, 19, 28, 27, }, // Krasnoyarsk
    {  370, 1151,       0,  430530,       0,  0, 17,  0, }, // Kyrgystan
    {  370, 1175,       0,  430547,       0,  0, 14,  0, }, // Line_Islands
    {  370, 1188,  430561,  430575,  430598, 14, 23, 21, }, // Lord_Howe
    {  370, 1204,  430619,  430629,  430648, 10, 19, 18, }, // Magadan
    {  370, 1219,       0,  430666,       0,  0, 11,  0, }, // Malaysia
    {  370, 1228,       0,  430677,       0,  0, 10,  0, }, // Maldives
    {  370, 1237,       0,  430687,       0,  0, 12,  0, }, // Marquesas
    {  370, 1247,       0,  430699,       0,  0, 16,  0, }, // Marshall_Islands
    {  370, 1264,  430715,  430725,  430744, 10, 19, 18, }, // Mauritius
    {  370, 1274,       0,  430762,       0,  0,  8,  0, }, // Mawson
    {  370, 1281,  430770,  430790,  430819, 20, 29, 27, }, // Mexico_Pacific
    {  370, 1296,  430846,  430858,  430879, 12, 21, 20, }, // Mongolia
    {  370, 1305,  430899,  430909,  430928, 10, 19, 18, }, // Moscow
    {  370, 1312,       0,  430946,       0,  0, 12,  0, }, // Myanmar
    {  370, 1320,       0,  430958,       0,  0,  7,  0, }, // Nauru
    {  370, 1326,       0,  430965,       0,  0,  9,  0, }, // Nepal
    {  370, 1332,  430974,  430993,  431021, 19, 28, 27, }, // New_Caledonia
    {  370, 1346,  431048,  431063,  431087, 15, 24, 22, }, // New_Zealand
    {  370, 1358,  431109,  431125,  431150, 16, 25, 23, }, // Newfoundland
    {  370, 1371,       0,  431173,       0,  0,  9,  0, }, // Niue
    {  370, 1376,  431182,  431198,  431223, 16, 25, 24, }, // Norfolk
    {  370, 1384,  431247,  431271,  431304, 24, 33, 32, }, // Noronha
    {  370, 1406,  334545,  431336,  431362, 13, 26, 25, }, // Novosibirsk
    {  370, 1418,  431387,  431397,  431416, 10, 19, 18, }, // Omsk
    {  370, 1428,   91241,  431434,  431456,  9, 22, 21, }, // Pakistan
    {  370, 1437,       0,  431477,       0,  0,  8,  0, }, // Palau
    {  370, 1443,       0,  431485,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  370, 1460,  431504,  431514,  431533, 10, 19, 18, }, // Paraguay
    {  370, 1469,  431551,  431559,  431576,  8, 17, 16, }, // Peru
    {  370, 1474,  431592,  431604,  431625, 12, 21, 20, }, // Philippines
    {  370, 1486,       0,  431645,       0,  0, 17,  0, }, // Phoenix_Islands
    {  370, 1502,  431662,  431686,  431719, 24, 33, 31, }, // Pierre_Miquelon
    {  370, 1518,       0,  431750,       0,  0, 11,  0, }, // Pitcairn
    {  370, 1527,       0,  431761,       0,  0,  9,  0, }, // Ponape
    {  370, 1534,       0,  431770,       0,  0, 14,  0, }, // Pyongyang
    {  370, 1554,       0,  431784,       0,  0, 12,  0, }, // Reunion
    {  370, 1562,       0,  431796,       0,  0, 10,  0, }, // Rothera
    {  370, 1570,  431806,  431815,  431833,  9, 18, 17, }, // Sakhalin
    {  370, 1596,  431850,  431859,  431877,  9, 18, 16, }, // Samoa
    {  370, 1602,       0,  431893,       0,  0, 11,  0, }, // Seychelles
    {  370, 1624,       0,  431904,       0,  0, 21,  0, }, // Singapore
    {  370, 1634,       0,  431925,       0,  0, 18,  0, }, // Solomon
    {  370, 1642,       0,  431943,       0,  0, 19,  0, }, // South_Georgia
    {  370, 1656,       0,  431962,       0,  0, 11,  0, }, // Suriname
    {  370, 1676,       0,  431973,       0,  0, 10,  0, }, // Syowa
    {  370, 1682,       0,  431983,       0,  0, 10,  0, }, // Tahiti
    {  370, 1689,  431993,  432002,  432020,  9, 18, 16, }, // Taipei
    {  370, 1696,       0,  432036,       0,  0, 14,  0, }, // Tajikistan
    {  370, 1724,       0,  432050,       0,  0, 10,  0, }, // Tokelau
    {  370, 1732,  432060,  432069,  432087,  9, 18, 17, }, // Tonga
    {  370, 1738,       0,  432104,       0,  0,  7,  0, }, // Truk
    {  370, 1750,  335409,  432111,  432138, 14, 27, 26, }, // Turkmenistan
    {  370, 1763,       0,  432164,       0,  0, 10,  0, }, // Tuvalu
    {  370, 1777,  432174,  432185,  432205, 11, 20, 19, }, // Uruguay
    {  370, 1792,  432224,  432241,  432267, 17, 26, 25, }, // Uzbekistan
    {  370, 1803,  432292,  432301,  432319,  9, 18, 17, }, // Vanuatu
    {  370, 1811,       0,  432336,       0,  0, 13,  0, }, // Venezuela
    {  370, 1821,  432349,  432366,  432392, 17, 26, 25, }, // Vladivostok
    {  370, 1833,  432417,  432434,  432460, 17, 26, 25, }, // Volgograd
    {  370, 1843,       0,  432355,       0,  0, 11,  0, }, // Vostok
    {  370, 1850,       0,  432485,       0,  0, 13,  0, }, // Wake
    {  370, 1855,       0,  432498,       0,  0, 22,  0, }, // Wallis
    {  370, 1862,  432520,  432532,  432553, 12, 21, 20, }, // Yakutsk
    {  370, 1870,  432573,  432590,  432616, 17, 26, 25, }, // Yekaterinburg
    {  370, 1892,       0,  432641,       0,  0,  9,  0, }, // Yukon
    {  371,    0,  432650,  432656,  432663,  6,  7, 10, }, // Acre Korean/Korean/South Korea
    {  371,    5,       0,  432673,       0,  0,  9,  0, }, // Afghanistan
    {  371,   17,       0,  432682,       0,  0,  9,  0, }, // Africa_Central
    {  371,   32,       0,  432691,       0,  0,  8,  0, }, // Africa_Eastern
    {  371,   65,       0,  432699,       0,  0,  8,  0, }, // Africa_Southern
    {  371,   81,  432707,  432715,  432724,  8,  9, 12, }, // Africa_Western
    {  371,  107,  432736,  432743,  432751,  7,  8, 11, }, // Alaska
    {  371,  128,  432762,  432771,  432781,  9, 10, 10, }, // Almaty
    {  371,  135,  432791,  432797,  432804,  6,  7, 10, }, // Amazon
    {  371,  142,  432814,  432821,  432829,  7,  8, 11, }, // America_Central
    {  371,  158,  432840,  432847,  432855,  7,  8, 11, }, // America_Eastern
    {  371,  174,  432866,  432873,  432881,  7,  8, 11, }, // America_Mountain
    {  371,  191,  432892,  432900,  432909,  8,  9, 12, }, // America_Pacific
    {  371,  207,  432921,  432928,  432936,  7,  8, 11, }, // Anadyr
    {  371,  214,  432947,  432953,  432960,  6,  7, 10, }, // Apia
    {  371,  219,  432970,  432979,  432989,  9, 10, 10, }, // Aqtau
    {  371,  225,  432999,  433008,  433018,  9, 10, 10, }, // Aqtobe
    {  371,  232,  433028,  433035,  433043,  7,  8, 11, }, // Arabian
    {  371,  240,  433054,  433062,  433071,  8,  9, 12, }, // Argentina
    {  371,  250,  433083,  433094,  433106, 11, 12, 15, }, // Argentina_Western
    {  371,  268,  433121,  433129,  433138,  8,  9, 12, }, // Armenia
    {  371,  286,  433150,  433156,  433163,  6,  7, 10, }, // Atlantic
    {  371,  295,  433173,  433186,  433200, 13, 14, 17, }, // Australia_Central
    {  371,  313,  433217,  433231,  433246, 14, 15, 18, }, // Australia_CentralWestern
    {  371,  338,  433264,  433277,  433291, 13, 14, 17, }, // Australia_Eastern
    {  371,  356,  433308,  433321,  433335, 13, 14, 17, }, // Australia_Western
    {  371,  374,  433352,  433361,  433371,  9, 10, 13, }, // Azerbaijan
    {  371,  385,  433384,  433391,  433399,  7,  8, 11, }, // Azores
    {  371,  397,  433410,  433418,  433427,  8,  9, 12, }, // Bangladesh
    {  371,  415,       0,  433439,       0,  0,  5,  0, }, // Bhutan
    {  371,  422,       0,  433444,       0,  0,  7,  0, }, // Bolivia
    {  371,  437,  433451,  433459,  433468,  8,  9, 12, }, // Brasilia
    {  371,  454,       0,  433480,       0,  0,  7,  0, }, // Brunei
    {  371,  461,  433487,  433496,  433506,  9, 10, 13, }, // Cape_Verde
    {  371,  472,       0,  433519,       0,  0,  6,  0, }, // Casey
    {  371,  478,       0,  433525,       0,  0,  6,  0, }, // Chamorro
    {  371,  487,  433531,  433536,  433542,  5,  6,  9, }, // Chatham
    {  371,  495,  433551,  433556,  433562,  5,  6,  9, }, // Chile
    {  371,  501,  433571,  433576,  433582,  5,  6,  9, }, // China
    {  371,  507,  433591,  433598,  433606,  7,  8, 11, }, // Choibalsan
    {  371,  518,       0,  433617,       0,  0,  9,  0, }, // Christmas
    {  371,  528,       0,  433626,       0,  0,  9,  0, }, // Cocos
    {  371,  534,  433635,  433642,  433650,  7,  8, 11, }, // Colombia
    {  371,  543,  433661,  433668,  433676,  7,  8, 14, }, // Cook
    {  371,  548,  433690,  433695,  433701,  5,  6,  9, }, // Cuba
    {  371,  559,       0,  433710,       0,  0,  7,  0, }, // Davis
    {  371,  575,       0,  433717,       0,  0,  8,  0, }, // DumontDUrville
    {  371,  612,       0,  433725,       0,  0,  7,  0, }, // East_Timor
    {  371,  623,  433732,  433739,  433747,  7,  8, 11, }, // Easter
    {  371,  630,       0,  433758,       0,  0,  7,  0, }, // Ecuador
    {  371,  638,  433765,  433772,  433780,  7,  8, 11, }, // Europe_Central
    {  371,  653,  433791,  433797,  433804,  6,  7, 10, }, // Europe_Eastern
    {  371,  668,       0,  433814,       0,  0,  8,  0, }, // Europe_Further_Eastern
    {  371,  691,  433822,  433828,  433835,  6,  7, 10, }, // Europe_Western
    {  371,  706,  433845,  433855,  433866, 10, 11, 14, }, // Falkland
    {  371,  715,  433880,  433885,  433891,  5,  6,  9, }, // Fiji
    {  371,  720,       0,  433900,       0,  0, 12,  0, }, // French_Guiana
    {  371,  734,       0,  433912,       0,  0, 19,  0, }, // French_Southern
    {  371,  757,       0,  433931,       0,  0,  8,  0, }, // Galapagos
    {  371,  767,       0,  433939,       0,  0,  6,  0, }, // Gambier
    {  371,  775,  433945,  433951,  433958,  6,  7, 10, }, // Georgia
    {  371,  783,       0,  433968,       0,  0,  9,  0, }, // Gilbert_Islands
    {  371,  799,       0,  433977,       0,  0,  8,  0, }, // GMT
    {  371,  841,  433985,  433995,  434006, 10, 11, 14, }, // Greenland_Eastern
    {  371,  859,  434020,  434030,  434041, 10, 11, 14, }, // Greenland_Western
    {  371,  877,       0,  434055,       0,  0,  7,  0, }, // Guam
    {  371,  882,       0,  434062,       0,  0,  7,  0, }, // Gulf
    {  371,  887,       0,  433905,       0,  0,  7,  0, }, // Guyana
    {  371,  894,  434069,  434079,  434090, 10, 11, 14, }, // Hawaii_Aleutian
    {  371,  910,  434104,  434109,  434115,  5,  6,  9, }, // Hong_Kong
    {  371,  920,  434124,  434130,  434137,  6,  7, 10, }, // Hovd
    {  371,  925,       0,  434147,       0,  0,  6,  0, }, // India
    {  371,  931,       0,  434153,       0,  0,  6,  0, }, // Indian_Ocean
    {  371,  944,       0,  434159,       0,  0,  8,  0, }, // Indochina
    {  371,  954,       0,  434167,       0,  0, 11,  0, }, // Indonesia_Central
    {  371,  972,       0,  434178,       0,  0, 11,  0, }, // Indonesia_Eastern
    {  371,  990,       0,  434189,       0,  0, 11,  0, }, // Indonesia_Western
    {  371, 1008,  434200,  434205,  434211,  5,  6,  9, }, // Iran
    {  371, 1019,  434220,  434228,  434237,  8,  9, 12, }, // Irkutsk
    {  371, 1027,  434249,  434256,  434264,  7,  8, 11, }, // Israel
    {  371, 1034,  434275,  434280,  434286,  5,  6,  9, }, // Japan
    {  371, 1040,  434295,  434311,  434328, 16, 17, 20, }, // Kamchatka
    {  371, 1058,       0,  434348,       0,  0, 11,  0, }, // Kazakhstan_Eastern
    {  371, 1077,       0,  434359,       0,  0, 11,  0, }, // Kazakhstan_Western
    {  371, 1106,  434370,  434377,  434385,  7,  8, 11, }, // Korea
    {  371, 1112,       0,  434396,       0,  0,  8,  0, }, // Kosrae
    {  371, 1119,  434404,  434415,  434427, 11, 12, 15, }, // Krasnoyarsk
    {  371, 1151,       0,  434442,       0,  0,  9,  0, }, // Kyrgystan
    {  371, 1161,       0,  434451,       0,  0,  8,  0, }, // Lanka
    {  371, 1175,       0,  434459,       0,  0,  8,  0, }, // Line_Islands
    {  371, 1188,  434467,  434475,  434484,  8,  9, 12, }, // Lord_Howe
    {  371, 1198,  434496,  434502,  434511,  6,  9, 10, }, // Macau
    {  371, 1204,  434521,  434527,  434534,  6,  7, 10, }, // Magadan
    {  371, 1219,       0,  434544,       0,  0,  8,  0, }, // Malaysia
    {  371, 1228,       0,  434552,       0,  0,  6,  0, }, // Maldives
    {  371, 1237,       0,  434558,       0,  0, 10,  0, }, // Marquesas
    {  371, 1247,       0,  434568,       0,  0,  8,  0, }, // Marshall_Islands
    {  371, 1264,  434576,  434583,  434591,  7,  8, 11, }, // Mauritius
    {  371, 1274,       0,  434602,       0,  0,  5,  0, }, // Mawson
    {  371, 1281,  434607,  434617,  434628, 10, 11, 14, }, // Mexico_Pacific
    {  371, 1296,  434642,  434650,  434659,  8,  9, 12, }, // Mongolia
    {  371, 1305,  434671,  434678,  434686,  7,  8, 11, }, // Moscow
    {  371, 1312,       0,  434697,       0,  0,  6,  0, }, // Myanmar
    {  371, 1320,       0,  434703,       0,  0,  6,  0, }, // Nauru
    {  371, 1326,       0,  434709,       0,  0,  5,  0, }, // Nepal
    {  371, 1332,  434714,  434723,  434733,  9, 10, 13, }, // New_Caledonia
    {  371, 1346,  434746,  434753,  434761,  7,  8, 11, }, // New_Zealand
    {  371, 1358,  434772,  434780,  434789,  8,  9, 12, }, // Newfoundland
    {  371, 1371,       0,  434801,       0,  0,  6,  0, }, // Niue
    {  371, 1376,  434807,  434813,  434820,  6,  7, 10, }, // Norfolk
    {  371, 1384,  434830,  434843,  434857, 13, 14, 17, }, // Noronha
    {  371, 1392,       0,  434874,       0,  0, 14,  0, }, // North_Mariana
    {  371, 1406,  434888,  434898,  434909, 10, 11, 14, }, // Novosibirsk
    {  371, 1418,  434923,  434929,  434936,  6,  7, 10, }, // Omsk
    {  371, 1428,  434946,  434953,  434961,  7,  8, 11, }, // Pakistan
    {  371, 1437,       0,  434972,       0,  0,  6,  0, }, // Palau
    {  371, 1443,       0,  434978,       0,  0,  9,  0, }, // Papua_New_Guinea
    {  371, 1460,  434987,  434994,  435002,  7,  8, 11, }, // Paraguay
    {  371, 1469,  435013,  435018,  435024,  5,  6,  9, }, // Peru
    {  371, 1474,  435033,  435039,  435046,  6,  7, 10, }, // Philippines
    {  371, 1486,       0,  435056,       0,  0,  9,  0, }, // Phoenix_Islands
    {  371, 1502,  435065,  435078,  435092, 13, 14, 17, }, // Pierre_Miquelon
    {  371, 1518,       0,  435109,       0,  0,  6,  0, }, // Pitcairn
    {  371, 1527,       0,  435115,       0,  0,  6,  0, }, // Ponape
    {  371, 1534,       0,  432895,       0,  0,  5,  0, }, // Pyongyang
    {  371, 1544,  435121,  435129,  435140,  8, 11, 12, }, // Qyzylorda
    {  371, 1554,       0,  435152,       0,  0,  7,  0, }, // Reunion
    {  371, 1562,       0,  435159,       0,  0,  6,  0, }, // Rothera
    {  371, 1570,  435165,  435171,  435178,  6,  7, 10, }, // Sakhalin
    {  371, 1579,  435188,  435194,  435201,  6,  7, 10, }, // Samara
    {  371, 1596,  435211,  435217,  435224,  6,  7, 10, }, // Samoa
    {  371, 1602,       0,  435234,       0,  0,  6,  0, }, // Seychelles
    {  371, 1624,       0,  435240,       0,  0,  8,  0, }, // Singapore
    {  371, 1634,       0,  435248,       0,  0,  9,  0, }, // Solomon
    {  371, 1642,       0,  435257,       0,  0, 10,  0, }, // South_Georgia
    {  371, 1656,       0,  435267,       0,  0,  6,  0, }, // Suriname
    {  371, 1676,       0,  435273,       0,  0,  5,  0, }, // Syowa
    {  371, 1682,       0,  435278,       0,  0,  6,  0, }, // Tahiti
    {  371, 1689,  435284,  435289,  435295,  5,  6,  9, }, // Taipei
    {  371, 1696,       0,  435304,       0,  0,  8,  0, }, // Tajikistan
    {  371, 1724,       0,  435312,       0,  0,  7,  0, }, // Tokelau
    {  371, 1732,  435319,  435324,  435330,  5,  6,  9, }, // Tonga
    {  371, 1738,       0,  435339,       0,  0,  5,  0, }, // Truk
    {  371, 1750,  435344,  435354,  435365, 10, 11, 14, }, // Turkmenistan
    {  371, 1763,       0,  435379,       0,  0,  6,  0, }, // Tuvalu
    {  371, 1777,  435385,  435392,  435400,  7,  8, 11, }, // Uruguay
    {  371, 1792,  435411,  435420,  435430,  9, 10, 13, }, // Uzbekistan
    {  371, 1803,  435443,  435450,  435458,  7,  8, 11, }, // Vanuatu
    {  371, 1811,       0,  435469,       0,  0,  8,  0, }, // Venezuela
    {  371, 1821,  435477,  435487,  435498, 10, 11, 14, }, // Vladivostok
    {  371, 1833,  435512,  435520,  435529,  8,  9, 12, }, // Volgograd
    {  371, 1843,       0,  435541,       0,  0,  6,  0, }, // Vostok
    {  371, 1850,       0,  435547,       0,  0,  7,  0, }, // Wake
    {  371, 1855,       0,  435554,       0,  0, 12,  0, }, // Wallis
    {  371, 1862,  435566,  435573,  435581,  7,  8, 11, }, // Yakutsk
    {  371, 1870,  435592,  435602,  435613, 10, 11, 14, }, // Yekaterinburg
    {  371, 1892,       0,  435627,       0,  0,  5,  0, }, // Yukon
    {  373, 1106,  435632,  435637,  435643,  5,  6,  9, }, // Korea Korean/Korean/North Korea
    {  378,    5,       0,  435652,       0,  0, 17,  0, }, // Afghanistan Kurdish/Latin/Turkey
    {  378,   17,       0,  435669,       0,  0, 20,  0, }, // Africa_Central
    {  378,   32,       0,  435689,       0,  0, 23,  0, }, // Africa_Eastern
    {  378,   65,       0,  435712,       0,  0, 31,  0, }, // Africa_Southern
    {  378,   81,  435743,  435764,  435796, 21, 32, 31, }, // Africa_Western
    {  378,  107,  435827,  435841,  435866, 14, 25, 24, }, // Alaska
    {  378,  135,  435890,  435903,  435927, 13, 24, 23, }, // Amazon
    {  378,  142,  435950,  435982,  436025, 32, 43, 42, }, // America_Central
    {  378,  158,  436067,  436099,  436143, 32, 44, 43, }, // America_Eastern
    {  378,  174,  436186,  436217,  436259, 31, 42, 41, }, // America_Mountain
    {  378,  191,  436300,  436333,  436377, 33, 44, 43, }, // America_Pacific
    {  378,  214,  436420,  436432,  436455, 12, 23, 22, }, // Apia
    {  378,  232,  436477,  436493,  436520, 16, 27, 26, }, // Arabian
    {  378,  240,  436546,  436561,  436587, 15, 26, 25, }, // Argentina
    {  378,  250,  436612,  436634,  436667, 22, 33, 32, }, // Argentina_Western
    {  378,  268,  436699,  436716,  436744, 17, 28, 27, }, // Armenia
    {  378,  286,  436771,  436786,  436812, 15, 26, 25, }, // Atlantic
    {  378,  295,  436837,  436861,  436896, 24, 35, 34, }, // Australia_Central
    {  378,  313,  436930,  436965,  437011, 35, 46, 45, }, // Australia_CentralWestern
    {  378,  338,  437056,  437083,  437121, 27, 38, 37, }, // Australia_Eastern
    {  378,  356,  437158,  437183,  437219, 25, 36, 35, }, // Australia_Western
    {  378,  374,  437254,  437271,  437299, 17, 28, 27, }, // Azerbaijan
    {  378,  385,  437326,  437338,  437361, 12, 23, 22, }, // Azores
    {  378,  397,  437383,  437399,  437426, 16, 27, 26, }, // Bangladesh
    {  378,  415,       0,  437452,       0,  0, 12,  0, }, // Bhutan
    {  378,  422,       0,  437464,       0,  0, 15,  0, }, // Bolivia
    {  378,  437,  437479,  437495,  437522, 16, 27, 26, }, // Brasilia
    {  378,  454,       0,  437548,       0,  0, 24,  0, }, // Brunei
    {  378,  461,  437572,  437590,  437619, 18, 29, 28, }, // Cape_Verde
    {  378,  478,       0,  437647,       0,  0, 27,  0, }, // Chamorro
    {  378,  487,  437674,  437688,  437713, 14, 25, 24, }, // Chatham
    {  378,  495,  437737,  437749,  437772, 12, 23, 22, }, // Chile
    {  378,  501,  437794,  437804,  437825, 10, 21, 20, }, // China
    {  378,  507,  437845,  437862,  437890, 17, 28, 27, }, // Choibalsan
    {  378,  518,       0,  437917,       0,  0, 23,  0, }, // Christmas
    {  378,  528,       0,  437940,       0,  0, 20,  0, }, // Cocos
    {  378,  534,  437960,  437977,  438005, 17, 28, 27, }, // Colombia
    {  378,  543,  438032,  438051,  438081, 19, 30, 32, }, // Cook
    {  378,  548,  438113,  438125,  438148, 12, 23, 22, }, // Cuba
    {  378,  559,       0,  438170,       0,  0, 12,  0, }, // Davis
    {  378,  575,       0,  438182,       0,  0, 24,  0, }, // DumontDUrville
    {  378,  612,       0,  438206,       0,  0, 21,  0, }, // East_Timor
    {  378,  623,  438227,  438250,  438284, 23, 34, 33, }, // Easter
    {  378,  630,       0,  438317,       0,  0, 14,  0, }, // Ecuador
    {  378,  638,  438331,  438351,  438382, 20, 31, 30, }, // Europe_Central
    {  378,  653,  438412,  438435,  438469, 23, 34, 33, }, // Europe_Eastern
    {  378,  668,       0,  438502,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  378,  691,  438534,  438555,  438587, 21, 32, 31, }, // Europe_Western
    {  378,  706,  438618,  438641,  438675, 23, 34, 33, }, // Falkland
    {  378,  715,  438708,  438720,  438743, 12, 23, 22, }, // Fiji
    {  378,  720,       0,  438765,       0,  0, 22,  0, }, // French_Guiana
    {  378,  734,       0,  438787,       0,  0, 34,  0, }, // French_Southern
    {  378,  757,       0,  438821,       0,  0, 16,  0, }, // Galapagos
    {  378,  767,       0,  438837,       0,  0, 14,  0, }, // Gambier
    {  378,  775,  438851,  438867,  438894, 16, 27, 26, }, // Georgia
    {  378,  783,       0,  438920,       0,  0, 22,  0, }, // Gilbert_Islands
    {  378,  799,       0,  438942,       0,  0, 26,  0, }, // GMT
    {  378,  841,  438968,  438992,  439027, 24, 35, 34, }, // Greenland_Eastern
    {  378,  859,  439061,  439083,  439116, 22, 33, 32, }, // Greenland_Western
    {  378,  882,       0,  439148,       0,  0, 24,  0, }, // Gulf
    {  378,  887,       0,  439172,       0,  0, 14,  0, }, // Guyana
    {  378,  894,  439186,  439208,  439241, 22, 33, 32, }, // Hawaii_Aleutian
    {  378,  910,  439273,  439289,  439316, 16, 27, 26, }, // Hong_Kong
    {  378,  920,  439342,  439353,  439375, 11, 22, 21, }, // Hovd
    {  378,  925,       0,  439396,       0,  0, 27,  0, }, // India
    {  378,  931,       0,  439423,       0,  0, 20,  0, }, // Indian_Ocean
    {  378,  944,       0,  439443,       0,  0, 15,  0, }, // Indochina
    {  378,  954,       0,  439458,       0,  0, 23,  0, }, // Indonesia_Central
    {  378,  972,       0,  439481,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  378,  990,       0,  439507,       0,  0, 24,  0, }, // Indonesia_Western
    {  378, 1008,  439531,  439542,  439564, 11, 22, 21, }, // Iran
    {  378, 1019,  439585,  439599,  439624, 14, 25, 24, }, // Irkutsk
    {  378, 1027,  439648,  439661,  439685, 13, 24, 23, }, // Israel
    {  378, 1034,  439708,  439723,  439749, 15, 26, 25, }, // Japan
    {  378, 1058,       0,  439774,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  378, 1077,       0,  439800,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  378, 1106,  439824,  439836,  439859, 12, 23, 22, }, // Korea
    {  378, 1112,       0,  439881,       0,  0, 14,  0, }, // Kosrae
    {  378, 1119,  439895,  439913,  439942, 18, 29, 28, }, // Krasnoyarsk
    {  378, 1151,       0,  439970,       0,  0, 18,  0, }, // Kyrgystan
    {  378, 1175,       0,  439988,       0,  0, 20,  0, }, // Line_Islands
    {  378, 1188,  440008,  440025,  440053, 17, 28, 27, }, // Lord_Howe
    {  378, 1204,  440080,  440094,  440119, 14, 25, 24, }, // Magadan
    {  378, 1219,       0,  440143,       0,  0, 15,  0, }, // Malaysia
    {  378, 1228,       0,  440158,       0,  0, 14,  0, }, // Maldives
    {  378, 1237,       0,  440172,       0,  0, 16,  0, }, // Marquesas
    {  378, 1247,       0,  440188,       0,  0, 21,  0, }, // Marshall_Islands
    {  378, 1264,  440209,  440225,  440252, 16, 27, 26, }, // Mauritius
    {  378, 1274,       0,  440278,       0,  0, 13,  0, }, // Mawson
    {  378, 1281,  440291,  440315,  440350, 24, 35, 34, }, // Mexico_Pacific
    {  378, 1296,  440384,  440400,  440427, 16, 27, 26, }, // Mongolia
    {  378, 1305,  440453,  440468,  440494, 15, 26, 25, }, // Moscow
    {  378, 1312,       0,  440519,       0,  0, 14,  0, }, // Myanmar
    {  378, 1320,       0,  440533,       0,  0, 13,  0, }, // Nauru
    {  378, 1326,       0,  440546,       0,  0, 12,  0, }, // Nepal
    {  378, 1332,  440558,  440578,  440609, 20, 31, 30, }, // New_Caledonia
    {  378, 1346,  440639,  440657,  440686, 18, 29, 28, }, // New_Zealand
    {  378, 1358,  440714,  440733,  440763, 19, 30, 29, }, // Newfoundland
    {  378, 1371,       0,  440792,       0,  0, 12,  0, }, // Niue
    {  378, 1376,  440804,  440825,  440857, 21, 32, 31, }, // Norfolk
    {  378, 1384,  440888,  440915,  440953, 27, 38, 37, }, // Noronha
    {  378, 1406,  440990,  441008,  441037, 18, 29, 28, }, // Novosibirsk
    {  378, 1418,  441065,  441076,  441098, 11, 22, 21, }, // Omsk
    {  378, 1428,  441119,  441134,  441160, 15, 26, 25, }, // Pakistan
    {  378, 1437,       0,  441185,       0,  0, 13,  0, }, // Palau
    {  378, 1443,       0,  441198,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  378, 1460,  441224,  441239,  441265, 15, 26, 25, }, // Paraguay
    {  378, 1469,  441290,  441302,  441325, 12, 23, 22, }, // Peru
    {  378, 1474,  441347,  441361,  441386, 14, 25, 24, }, // Philippines
    {  378, 1486,       0,  441410,       0,  0, 22,  0, }, // Phoenix_Islands
    {  378, 1502,  441432,  441462,  441503, 30, 41, 40, }, // Pierre_Miquelon
    {  378, 1518,       0,  441543,       0,  0, 15,  0, }, // Pitcairn
    {  378, 1527,       0,  441558,       0,  0, 14,  0, }, // Ponape
    {  378, 1534,       0,  441572,       0,  0, 16,  0, }, // Pyongyang
    {  378, 1554,       0,  441588,       0,  0, 14,  0, }, // Reunion
    {  378, 1562,       0,  441602,       0,  0, 15,  0, }, // Rothera
    {  378, 1570,  441617,  441631,  441656, 14, 25, 24, }, // Sakhalin
    {  378, 1596,  441680,  441693,  441717, 13, 24, 23, }, // Samoa
    {  378, 1602,       0,  441740,       0,  0, 15,  0, }, // Seychelles
    {  378, 1624,       0,  441755,       0,  0, 26,  0, }, // Singapore
    {  378, 1634,       0,  441781,       0,  0, 22,  0, }, // Solomon
    {  378, 1642,       0,  441803,       0,  0, 21,  0, }, // South_Georgia
    {  378, 1656,       0,  441824,       0,  0, 14,  0, }, // Suriname
    {  378, 1676,       0,  441838,       0,  0, 13,  0, }, // Syowa
    {  378, 1682,       0,  441851,       0,  0, 14,  0, }, // Tahiti
    {  378, 1689,  441865,  441879,  441904, 14, 25, 24, }, // Taipei
    {  378, 1696,       0,  441928,       0,  0, 17,  0, }, // Tajikistan
    {  378, 1724,       0,  441945,       0,  0, 15,  0, }, // Tokelau
    {  378, 1732,  441960,  441973,  441997, 13, 24, 23, }, // Tonga
    {  378, 1738,       0,  442020,       0,  0, 12,  0, }, // Truk
    {  378, 1750,  442032,  442051,  442081, 19, 30, 29, }, // Turkmenistan
    {  378, 1763,       0,  442110,       0,  0, 14,  0, }, // Tuvalu
    {  378, 1777,  442124,  442138,  442163, 14, 25, 24, }, // Uruguay
    {  378, 1792,  442187,  442204,  442232, 17, 28, 27, }, // Uzbekistan
    {  378, 1803,  442259,  442274,  442300, 15, 26, 25, }, // Vanuatu
    {  378, 1811,       0,  442325,       0,  0, 17,  0, }, // Venezuela
    {  378, 1821,  442342,  442360,  442389, 18, 29, 28, }, // Vladivostok
    {  378, 1833,  442417,  442433,  442460, 16, 27, 26, }, // Volgograd
    {  378, 1843,       0,  442486,       0,  0, 13,  0, }, // Vostok
    {  378, 1850,       0,  442499,       0,  0, 19,  0, }, // Wake
    {  378, 1855,       0,  442518,       0,  0, 23,  0, }, // Wallis
    {  378, 1862,  442541,  442555,  442580, 14, 25, 24, }, // Yakutsk
    {  378, 1870,  442604,  442624,  442655, 20, 31, 30, }, // Yekaterinburg
    {  378, 1892,       0,  442685,       0,  0, 12,  0, }, // Yukon
    {  380,    5,       0,  442697,       0,  0, 19,  0, }, // Afghanistan Kyrgyz/Cyrillic/Kyrgyzstan
    {  380,   17,       0,  442716,       0,  0, 25,  0, }, // Africa_Central
    {  380,   32,       0,  442741,       0,  0, 21,  0, }, // Africa_Eastern
    {  380,   65,       0,  442762,       0,  0, 22,  0, }, // Africa_Southern
    {  380,   81,  442784,  442805,  442831, 21, 26, 27, }, // Africa_Western
    {  380,  107,  442858,  442873,  442894, 15, 21, 21, }, // Alaska
    {  380,  135,  442915,  442930,  442951, 15, 21, 21, }, // Amazon
    {  380,  142,  442972,  443004,  443044, 32, 40, 40, }, // America_Central
    {  380,  158,  443084,  443114,  443150, 30, 36, 36, }, // America_Eastern
    {  380,  174,  443186,  443214,  443248, 28, 34, 34, }, // America_Mountain
    {  380,  191,  443282,  443317,  443358, 35, 41, 41, }, // America_Pacific
    {  380,  214,  443399,  443412,  443431, 13, 19, 19, }, // Apia
    {  380,  232,  443450,  443465,  443485, 15, 20, 20, }, // Arabian
    {  380,  240,  443505,  443523,  443547, 18, 24, 24, }, // Argentina
    {  380,  250,  443571,  443595,  443625, 24, 30, 30, }, // Argentina_Western
    {  380,  268,  443655,  443671,  443692, 16, 21, 22, }, // Armenia
    {  380,  286,  443714,  443732,  443756, 18, 24, 24, }, // Atlantic
    {  380,  295,  443780,  443808,  443841, 28, 33, 34, }, // Australia_Central
    {  380,  313,  443875,  443909,  443948, 34, 39, 40, }, // Australia_CentralWestern
    {  380,  338,  443988,  444012,  444041, 24, 29, 30, }, // Australia_Eastern
    {  380,  356,  444071,  444095,  444124, 24, 29, 30, }, // Australia_Western
    {  380,  374,  444154,  444173,  444197, 19, 24, 25, }, // Azerbaijan
    {  380,  385,  444222,  444235,  444253, 13, 18, 20, }, // Azores
    {  380,  397,  444273,  444291,  444314, 18, 23, 24, }, // Bangladesh
    {  380,  415,       0,  444338,       0,  0, 14,  0, }, // Bhutan
    {  380,  422,       0,  444352,       0,  0, 16,  0, }, // Bolivia
    {  380,  437,  444368,  444385,  444408, 17, 23, 23, }, // Brasilia
    {  380,  454,       0,  444431,       0,  0, 26,  0, }, // Brunei
    {  380,  461,  444457,  444476,  444500, 19, 24, 25, }, // Cape_Verde
    {  380,  478,       0,  444525,       0,  0, 16,  0, }, // Chamorro
    {  380,  487,  444541,  444555,  444573, 14, 18, 20, }, // Chatham
    {  380,  495,  444593,  444606,  444625, 13, 19, 19, }, // Chile
    {  380,  501,  444644,  444658,  444677, 14, 19, 19, }, // China
    {  380,  507,  444696,  444714,  444737, 18, 23, 24, }, // Choibalsan
    {  380,  518,       0,  444761,       0,  0, 25,  0, }, // Christmas
    {  380,  528,       0,  444786,       0,  0, 26,  0, }, // Cocos
    {  380,  534,  444812,  444829,  444852, 17, 23, 23, }, // Colombia
    {  380,  543,  444875,  444899,  444929, 24, 30, 36, }, // Cook
    {  380,  548,  444965,  444978,  444997, 13, 19, 19, }, // Cuba
    {  380,  559,       0,  445016,       0,  0, 14,  0, }, // Davis
    {  380,  575,       0,  445030,       0,  0, 22,  0, }, // DumontDUrville
    {  380,  612,       0,  445052,       0,  0, 20,  0, }, // East_Timor
    {  380,  623,  445072,  445095,  445123, 23, 28, 28, }, // Easter
    {  380,  630,       0,  445151,       0,  0, 16,  0, }, // Ecuador
    {  380,  638,  445167,  445192,  445222, 25, 30, 31, }, // Europe_Central
    {  380,  653,  445253,  445274,  445300, 21, 26, 27, }, // Europe_Eastern
    {  380,  668,       0,  445327,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  380,  691,  445358,  445379,  445405, 21, 26, 27, }, // Europe_Western
    {  380,  706,  445432,  445461,  445495, 29, 34, 35, }, // Falkland
    {  380,  715,  445530,  445543,  445561, 13, 18, 19, }, // Fiji
    {  380,  720,       0,  445580,       0,  0, 23,  0, }, // French_Guiana
    {  380,  734,       0,  445603,       0,  0, 40,  0, }, // French_Southern
    {  380,  757,       0,  445643,       0,  0, 18,  0, }, // Galapagos
    {  380,  767,       0,  445661,       0,  0, 15,  0, }, // Gambier
    {  380,  775,  445676,  445691,  445711, 15, 20, 21, }, // Georgia
    {  380,  783,       0,  445732,       0,  0, 16,  0, }, // Gilbert_Islands
    {  380,  799,       0,  445748,       0,  0, 28,  0, }, // GMT
    {  380,  841,  445776,  445801,  445832, 25, 31, 31, }, // Greenland_Eastern
    {  380,  859,  445863,  445888,  445919, 25, 31, 31, }, // Greenland_Western
    {  380,  882,       0,  445950,       0,  0, 28,  0, }, // Gulf
    {  380,  887,       0,  445588,       0,  0, 15,  0, }, // Guyana
    {  380,  894,  445978,  445999,  446026, 21, 27, 27, }, // Hawaii_Aleutian
    {  380,  910,  446053,  446069,  446090, 16, 21, 22, }, // Hong_Kong
    {  380,  920,  446112,  446125,  446143, 13, 18, 19, }, // Hovd
    {  380,  925,       0,  446162,       0,  0, 14,  0, }, // India
    {  380,  931,       0,  446176,       0,  0, 19,  0, }, // Indian_Ocean
    {  380,  944,       0,  446195,       0,  0, 18,  0, }, // Indochina
    {  380,  954,       0,  446213,       0,  0, 28,  0, }, // Indonesia_Central
    {  380,  972,       0,  446241,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  380,  990,       0,  446265,       0,  0, 24,  0, }, // Indonesia_Western
    {  380, 1008,  446289,  446302,  446320, 13, 18, 22, }, // Iran
    {  380, 1019,  446342,  446358,  446379, 16, 21, 21, }, // Irkutsk
    {  380, 1027,  446400,  446415,  446436, 15, 21, 21, }, // Israel
    {  380, 1034,  446457,  446471,  446490, 14, 19, 20, }, // Japan
    {  380, 1058,       0,  446510,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  380, 1077,       0,  446534,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  380, 1106,  446558,  446572,  446591, 14, 19, 20, }, // Korea
    {  380, 1112,       0,  446611,       0,  0, 15,  0, }, // Kosrae
    {  380, 1119,  446626,  446645,  446669, 19, 24, 25, }, // Krasnoyarsk
    {  380, 1151,       0,  446694,       0,  0, 19,  0, }, // Kyrgystan
    {  380, 1175,       0,  446713,       0,  0, 25,  0, }, // Line_Islands
    {  380, 1188,  446738,  446755,  446777, 17, 22, 23, }, // Lord_Howe
    {  380, 1204,  446800,  446816,  446837, 16, 21, 22, }, // Magadan
    {  380, 1219,       0,  446859,       0,  0, 17,  0, }, // Malaysia
    {  380, 1228,       0,  446876,       0,  0, 16,  0, }, // Maldives
    {  380, 1237,       0,  446892,       0,  0, 17,  0, }, // Marquesas
    {  380, 1247,       0,  446909,       0,  0, 28,  0, }, // Marshall_Islands
    {  380, 1264,  446937,  446954,  446976, 17, 22, 23, }, // Mauritius
    {  380, 1274,       0,  446999,       0,  0, 15,  0, }, // Mawson
    {  380, 1281,  447014,  447042,  447076, 28, 34, 34, }, // Mexico_Pacific
    {  380, 1296,  447110,  447129,  447153, 19, 24, 25, }, // Mongolia
    {  380, 1305,  447178,  447193,  447213, 15, 20, 21, }, // Moscow
    {  380, 1312,       0,  447234,       0,  0, 16,  0, }, // Myanmar
    {  380, 1320,       0,  447250,       0,  0, 14,  0, }, // Nauru
    {  380, 1326,       0,  447264,       0,  0, 14,  0, }, // Nepal
    {  380, 1332,  447278,  447301,  447330, 23, 29, 29, }, // New_Caledonia
    {  380, 1346,  447359,  447381,  447408, 22, 27, 27, }, // New_Zealand
    {  380, 1358,  447435,  447455,  447481, 20, 26, 26, }, // Newfoundland
    {  380, 1371,       0,  447507,       0,  0, 13,  0, }, // Niue
    {  380, 1376,  447520,  447536,  447558, 16, 22, 22, }, // Norfolk
    {  380, 1384,  447580,  447608,  447642, 28, 34, 34, }, // Noronha
    {  380, 1406,  447676,  447696,  447721, 20, 25, 26, }, // Novosibirsk
    {  380, 1418,  447747,  447760,  447778, 13, 18, 19, }, // Omsk
    {  380, 1428,  447797,  447814,  447836, 17, 22, 23, }, // Pakistan
    {  380, 1437,       0,  447859,       0,  0, 14,  0, }, // Palau
    {  380, 1443,       0,  447873,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  380, 1460,  447898,  447915,  447938, 17, 23, 23, }, // Paraguay
    {  380, 1469,  447961,  447974,  447993, 13, 19, 19, }, // Peru
    {  380, 1474,  448012,  448041,  448076, 29, 35, 35, }, // Philippines
    {  380, 1486,       0,  448111,       0,  0, 27,  0, }, // Phoenix_Islands
    {  380, 1502,  448138,  448168,  448204, 30, 36, 36, }, // Pierre_Miquelon
    {  380, 1518,       0,  448240,       0,  0, 17,  0, }, // Pitcairn
    {  380, 1527,       0,  448257,       0,  0, 15,  0, }, // Ponape
    {  380, 1534,       0,  448272,       0,  0, 15,  0, }, // Pyongyang
    {  380, 1554,       0,  448287,       0,  0, 16,  0, }, // Reunion
    {  380, 1562,       0,  448303,       0,  0, 15,  0, }, // Rothera
    {  380, 1570,  448318,  448334,  448355, 16, 21, 22, }, // Sakhalin
    {  380, 1596,  448377,  448391,  448411, 14, 20, 20, }, // Samoa
    {  380, 1602,       0,  448431,       0,  0, 15,  0, }, // Seychelles
    {  380, 1624,       0,  448446,       0,  0, 17,  0, }, // Singapore
    {  380, 1634,       0,  448463,       0,  0, 28,  0, }, // Solomon
    {  380, 1642,       0,  448491,       0,  0, 22,  0, }, // South_Georgia
    {  380, 1656,       0,  448513,       0,  0, 17,  0, }, // Suriname
    {  380, 1676,       0,  448530,       0,  0, 13,  0, }, // Syowa
    {  380, 1682,       0,  448543,       0,  0, 14,  0, }, // Tahiti
    {  380, 1689,  448557,  448572,  448592, 15, 20, 20, }, // Taipei
    {  380, 1696,       0,  448612,       0,  0, 18,  0, }, // Tajikistan
    {  380, 1724,       0,  448630,       0,  0, 16,  0, }, // Tokelau
    {  380, 1732,  448646,  448660,  448680, 14, 20, 20, }, // Tonga
    {  380, 1738,       0,  448700,       0,  0, 13,  0, }, // Truk
    {  380, 1750,  448713,  448733,  448758, 20, 25, 26, }, // Turkmenistan
    {  380, 1763,       0,  448784,       0,  0, 15,  0, }, // Tuvalu
    {  380, 1777,  448799,  448815,  448837, 16, 22, 22, }, // Uruguay
    {  380, 1792,  448859,  448877,  448900, 18, 23, 24, }, // Uzbekistan
    {  380, 1803,  448924,  448940,  448961, 16, 21, 22, }, // Vanuatu
    {  380, 1811,       0,  448983,       0,  0, 18,  0, }, // Venezuela
    {  380, 1821,  449001,  449021,  449046, 20, 25, 26, }, // Vladivostok
    {  380, 1833,  449072,  449090,  449113, 18, 23, 24, }, // Volgograd
    {  380, 1843,       0,  449137,       0,  0, 15,  0, }, // Vostok
    {  380, 1850,       0,  449152,       0,  0, 25,  0, }, // Wake
    {  380, 1855,       0,  449177,       0,  0, 26,  0, }, // Wallis
    {  380, 1862,  449203,  449218,  449238, 15, 20, 21, }, // Yakutsk
    {  380, 1870,  449259,  449280,  449306, 21, 26, 27, }, // Yekaterinburg
    {  380, 1892,       0,  449333,       0,  0, 13,  0, }, // Yukon
    {  383,    0,  449346,  449358,  449378, 12, 20, 20, }, // Acre Lao/Lao/Laos
    {  383,    5,       0,  449398,       0,  0, 16,  0, }, // Afghanistan
    {  383,   17,       0,  449414,       0,  0, 19,  0, }, // Africa_Central
    {  383,   32,       0,  449433,       0,  0, 27,  0, }, // Africa_Eastern
    {  383,   65,       0,  449460,       0,  0, 19,  0, }, // Africa_Southern
    {  383,   81,  449479,  449506,  449544, 27, 38, 38, }, // Africa_Western
    {  383,  107,  449582,  449593,  449612, 11, 19, 18, }, // Alaska
    {  383,  128,  449630,  449640,  449658, 10, 18, 18, }, // Almaty
    {  383,  135,  449676,  449693,  449718, 17, 25, 25, }, // Amazon
    {  383,  142,  449612,  449743,  449758,  7, 15, 14, }, // America_Central
    {  383,  158,  449772,  449785,  449806, 13, 21, 20, }, // America_Eastern
    {  383,  174,  449826,  449839,  449860, 13, 21, 20, }, // America_Mountain
    {  383,  191,  449880,  449891,  449910, 11, 19, 18, }, // America_Pacific
    {  383,  214,  449928,  449937,  449954,  9, 17, 16, }, // Apia
    {  383,  219,  449970,  449981,  450000, 11, 19, 19, }, // Aqtau
    {  383,  225,  450019,  450030,  450049, 11, 19, 19, }, // Aqtobe
    {  383,  232,  450068,  450083,  450108, 15, 25, 18, }, // Arabian
    {  383,  240,  450126,  450143,  450172, 17, 29, 30, }, // Argentina
    {  383,  250,  450202,  450229,  450266, 27, 37, 37, }, // Argentina_Western
    {  383,  268,  450303,  450314,  450333, 11, 19, 19, }, // Armenia
    {  383,  286,  450352,  450367,  450390, 15, 23, 22, }, // Atlantic
    {  383,  295,  450412,  450433,  450465, 21, 32, 30, }, // Australia_Central
    {  383,  313,  450495,  450527,  450570, 32, 43, 42, }, // Australia_CentralWestern
    {  383,  338,  450612,  450640,  450680, 28, 40, 38, }, // Australia_Eastern
    {  383,  356,  450718,  450747,  450787, 29, 40, 38, }, // Australia_Western
    {  383,  374,  450825,  450840,  450863, 15, 23, 23, }, // Azerbaijan
    {  383,  385,  450886,  450902,  450928, 16, 26, 26, }, // Azores
    {  383,  397,  450954,  450969,  450992, 15, 23, 24, }, // Bangladesh
    {  383,  415,       0,  451016,       0,  0, 12,  0, }, // Bhutan
    {  383,  422,       0,  451028,       0,  0, 15,  0, }, // Bolivia
    {  383,  437,  451043,  451061,  451081, 18, 20, 32, }, // Brasilia
    {  383,  454,       0,  451113,       0,  0, 25,  0, }, // Brunei
    {  383,  461,  451138,  451152,  451178, 14, 26, 25, }, // Cape_Verde
    {  383,  472,       0,  451203,       0,  0,  8,  0, }, // Casey
    {  383,  478,       0,  451211,       0,  0, 14,  0, }, // Chamorro
    {  383,  487,  451225,  451237,  451260, 12, 23, 21, }, // Chatham
    {  383,  495,  451281,  451292,  451313, 11, 21, 21, }, // Chile
    {  383,  501,  451334,  451343,  451358,  9, 15, 19, }, // China
    {  383,  507,  451377,  451393,  451413, 16, 20, 22, }, // Choibalsan
    {  383,  518,       0,  451435,       0,  0, 20,  0, }, // Christmas
    {  383,  528,       0,  451455,       0,  0, 17,  0, }, // Cocos
    {  383,  534,  451472,  451484,  451503, 12, 19, 20, }, // Colombia
    {  383,  543,  451523,  451538,  451561, 15, 23, 35, }, // Cook
    {  383,  548,  451596,  451605,  451625,  9, 20, 16, }, // Cuba
    {  383,  559,       0,  451641,       0,  0,  9,  0, }, // Davis
    {  383,  575,       0,  451650,       0,  0, 14,  0, }, // DumontDUrville
    {  383,  612,       0,  451664,       0,  0, 17,  0, }, // East_Timor
    {  383,  623,  451681,  451700,  451729, 19, 29, 29, }, // Easter
    {  383,  630,       0,  451758,       0,  0, 15,  0, }, // Ecuador
    {  383,  638,  451773,  451789,  451815, 16, 26, 28, }, // Europe_Central
    {  383,  653,  451843,  451867,  451903, 24, 36, 34, }, // Europe_Eastern
    {  383,  668,       0,  451937,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  383,  691,  451968,  451992,  452026, 24, 34, 34, }, // Europe_Western
    {  383,  706,  452060,  452086,  452122, 26, 36, 36, }, // Falkland
    {  383,  715,  452158,  452166,  452182,  8, 16, 16, }, // Fiji
    {  383,  720,       0,  452198,       0,  0, 21,  0, }, // French_Guiana
    {  383,  734,       0,  452219,       0,  0, 29,  0, }, // French_Southern
    {  383,  757,       0,  452248,       0,  0, 19,  0, }, // Galapagos
    {  383,  767,       0,  452267,       0,  0, 10,  0, }, // Gambier
    {  383,  775,  452277,  452286,  452303,  9, 17, 17, }, // Georgia
    {  383,  783,       0,  452320,       0,  0, 19,  0, }, // Gilbert_Islands
    {  383,  799,       0,  452339,       0,  0, 14,  0, }, // GMT
    {  383,  841,  452353,  452376,  452404, 23, 28, 28, }, // Greenland_Eastern
    {  383,  859,  452432,  452452,  452480, 20, 28, 27, }, // Greenland_Western
    {  383,  877,       0,  452507,       0,  0,  7,  0, }, // Guam
    {  383,  882,       0,  452514,       0,  0, 13,  0, }, // Gulf
    {  383,  887,       0,  452527,       0,  0, 11,  0, }, // Guyana
    {  383,  894,  452538,  452555,  452580, 17, 25, 24, }, // Hawaii_Aleutian
    {  383,  910,  452604,  452617,  452641, 13, 24, 25, }, // Hong_Kong
    {  383,  920,  452666,  452678,  452702, 12, 24, 24, }, // Hovd
    {  383,  925,       0,  452726,       0,  0, 11,  0, }, // India
    {  383,  931,       0,  452737,       0,  0, 21,  0, }, // Indian_Ocean
    {  383,  944,       0,  452758,       0,  0, 12,  0, }, // Indochina
    {  383,  954,       0,  452770,       0,  0, 23,  0, }, // Indonesia_Central
    {  383,  972,       0,  452793,       0,  0, 31,  0, }, // Indonesia_Eastern
    {  383,  990,       0,  452824,       0,  0, 31,  0, }, // Indonesia_Western
    {  383, 1008,  452855,  452867,  452889, 12, 22, 22, }, // Iran
    {  383, 1019,  452911,  452928,  452954, 17, 26, 26, }, // Irkutsk
    {  383, 1027,  452980,  452997,  453024, 17, 27, 19, }, // Israel
    {  383, 1034,  453043,  453057,  453082, 14, 25, 23, }, // Japan
    {  383, 1058,       0,  453105,       0,  0, 30,  0, }, // Kazakhstan_Eastern
    {  383, 1077,       0,  453135,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  383, 1106,  453165,  453176,  453201, 11, 25, 23, }, // Korea
    {  383, 1112,       0,  453224,       0,  0,  9,  0, }, // Kosrae
    {  383, 1119,  453233,  453254,  453285, 21, 31, 31, }, // Krasnoyarsk
    {  383, 1151,       0,  453316,       0,  0, 13,  0, }, // Kyrgystan
    {  383, 1161,       0,  453329,       0,  0,  9,  0, }, // Lanka
    {  383, 1175,       0,  453338,       0,  0, 18,  0, }, // Line_Islands
    {  383, 1188,  453356,  453370,  453395, 14, 25, 28, }, // Lord_Howe
    {  383, 1198,  453423,  453433,  453451, 10, 18, 18, }, // Macau
    {  383, 1204,  453469,  453482,  453503, 13, 21, 21, }, // Magadan
    {  383, 1219,       0,  453524,       0,  0, 15,  0, }, // Malaysia
    {  383, 1228,       0,  453539,       0,  0, 10,  0, }, // Maldives
    {  383, 1237,       0,  453549,       0,  0, 12,  0, }, // Marquesas
    {  383, 1247,       0,  453561,       0,  0, 21,  0, }, // Marshall_Islands
    {  383, 1264,  453582,  453600,  453628, 18, 28, 30, }, // Mauritius
    {  383, 1274,       0,  453658,       0,  0,  9,  0, }, // Mawson
    {  383, 1281,  453667,  453687,  453715, 20, 28, 27, }, // Mexico_Pacific
    {  383, 1296,  453742,  453757,  453780, 15, 23, 22, }, // Mongolia
    {  383, 1305,  453802,  453815,  453838, 13, 23, 23, }, // Moscow
    {  383, 1312,       0,  453861,       0,  0,  9,  0, }, // Myanmar
    {  383, 1320,       0,  453870,       0,  0, 14,  0, }, // Nauru
    {  383, 1326,       0,  453884,       0,  0, 13,  0, }, // Nepal
    {  383, 1332,  453897,  453913,  453937, 16, 24, 24, }, // New_Caledonia
    {  383, 1346,  453961,  453977,  454003, 16, 26, 25, }, // New_Zealand
    {  383, 1358,  454028,  454045,  454073, 17, 28, 20, }, // Newfoundland
    {  383, 1371,       0,  454093,       0,  0, 10,  0, }, // Niue
    {  383, 1376,  454103,  454123,  454153, 20, 30, 30, }, // Norfolk
    {  383, 1384,  454183,  454203,  454231, 20, 28, 28, }, // Noronha
    {  383, 1392,       0,  454259,       0,  0, 24,  0, }, // North_Mariana
    {  383, 1406,  454283,  454305,  454337, 22, 32, 32, }, // Novosibirsk
    {  383, 1418,  454369,  454383,  454407, 14, 24, 24, }, // Omsk
    {  383, 1428,  454431,  454446,  454472, 15, 26, 28, }, // Pakistan
    {  383, 1437,       0,  454500,       0,  0, 10,  0, }, // Palau
    {  383, 1443,       0,  454510,       0,  0, 13,  0, }, // Papua_New_Guinea
    {  383, 1460,  454523,  454538,  454564, 15, 26, 26, }, // Paraguay
    {  383, 1469,  454590,  454601,  454624, 11, 23, 22, }, // Peru
    {  383, 1474,  454646,  454663,  454690, 17, 27, 27, }, // Philippines
    {  383, 1486,       0,  454717,       0,  0, 17,  0, }, // Phoenix_Islands
    {  383, 1502,  454734,  454764,  454804, 30, 40, 38, }, // Pierre_Miquelon
    {  383, 1518,       0,  454842,       0,  0, 12,  0, }, // Pitcairn
    {  383, 1527,       0,  454854,       0,  0,  9,  0, }, // Ponape
    {  383, 1534,       0,  454863,       0,  0, 10,  0, }, // Pyongyang
    {  383, 1544,  454873,  454884,  454903, 11, 19, 19, }, // Qyzylorda
    {  383, 1554,       0,  454922,       0,  0, 18,  0, }, // Reunion
    {  383, 1562,       0,  454940,       0,  0, 11,  0, }, // Rothera
    {  383, 1570,  454951,  454966,  454991, 15, 25, 25, }, // Sakhalin
    {  383, 1596,  455016,  455025,  455042,  9, 17, 17, }, // Samoa
    {  383, 1602,       0,  455059,       0,  0, 16,  0, }, // Seychelles
    {  383, 1624,       0,  455075,       0,  0, 15,  0, }, // Singapore
    {  383, 1634,       0,  455090,       0,  0, 19,  0, }, // Solomon
    {  383, 1642,       0,  455109,       0,  0, 12,  0, }, // South_Georgia
    {  383, 1656,       0,  455121,       0,  0, 15,  0, }, // Suriname
    {  383, 1676,       0,  455136,       0,  0,  9,  0, }, // Syowa
    {  383, 1682,       0,  455145,       0,  0, 10,  0, }, // Tahiti
    {  383, 1689,  455155,  455166,  455188, 11, 22, 21, }, // Taipei
    {  383, 1696,       0,  455209,       0,  0, 14,  0, }, // Tajikistan
    {  383, 1724,       0,  455223,       0,  0, 12,  0, }, // Tokelau
    {  383, 1732,  455235,  455244,  455261,  9, 17, 17, }, // Tonga
    {  383, 1738,       0,  455278,       0,  0,  7,  0, }, // Truk
    {  383, 1750,  455285,  455300,  455323, 15, 23, 23, }, // Turkmenistan
    {  383, 1763,       0,  455346,       0,  0, 10,  0, }, // Tuvalu
    {  383, 1777,  455356,  455372,  455398, 16, 26, 26, }, // Uruguay
    {  383, 1792,  455424,  455441,  455466, 17, 25, 25, }, // Uzbekistan
    {  383, 1803,  455491,  455503,  455523, 12, 20, 20, }, // Vanuatu
    {  383, 1811,       0,  455543,       0,  0, 20,  0, }, // Venezuela
    {  383, 1821,  455563,  455577,  455599, 14, 22, 22, }, // Vladivostok
    {  383, 1833,  455621,  455633,  455653, 12, 20, 20, }, // Volgograd
    {  383, 1843,       0,  455673,       0,  0, 11,  0, }, // Vostok
    {  383, 1850,       0,  455684,       0,  0, 11,  0, }, // Wake
    {  383, 1855,       0,  455695,       0,  0, 21,  0, }, // Wallis
    {  383, 1862,  455716,  455726,  455744, 10, 18, 18, }, // Yakutsk
    {  383, 1870,  455762,  455779,  455804, 17, 25, 25, }, // Yekaterinburg
    {  383, 1892,       0,  455829,       0,  0,  9,  0, }, // Yukon
    {  385,    5,       0,  455838,       0,  0, 18,  0, }, // Afghanistan Latvian/Latin/Latvia
    {  385,   17,       0,  455856,       0,  0, 20,  0, }, // Africa_Central
    {  385,   32,       0,  455876,       0,  0, 20,  0, }, // Africa_Eastern
    {  385,   65,       0,  455896,       0,  0, 27,  0, }, // Africa_Southern
    {  385,   81,  455923,  455942,  455968, 19, 26, 27, }, // Africa_Western
    {  385,  107,  455995,  456008,  456028, 13, 20, 21, }, // Alaska
    {  385,  135,  456049,  456063,  456084, 14, 21, 22, }, // Amazon
    {  385,  142,  456106,  456122,  456145, 16, 23, 24, }, // America_Central
    {  385,  158,  456169,  456183,  456204, 14, 21, 22, }, // America_Eastern
    {  385,  174,  456226,  456237,  456255, 11, 18, 19, }, // America_Mountain
    {  385,  191,  456274,  456292,  456317, 18, 25, 26, }, // America_Pacific
    {  385,  207,  456343,  456357,  456378, 14, 21, 22, }, // Anadyr
    {  385,  214,  456400,  456412,  456431, 12, 19, 20, }, // Apia
    {  385,  232,  456451,  456474,  456504, 23, 30, 31, }, // Arabian
    {  385,  240,  456535,  456551,  456574, 16, 23, 24, }, // Argentina
    {  385,  250,  456598,  456620,  456649, 22, 29, 30, }, // Argentina_Western
    {  385,  268,  456679,  456694,  456716, 15, 22, 23, }, // Armenia
    {  385,  286,  456739,  456755,  456778, 16, 23, 24, }, // Atlantic
    {  385,  295,  456802,  456830,  456865, 28, 35, 36, }, // Australia_Central
    {  385,  313,  456901,  456937,  456980, 36, 43, 44, }, // Australia_CentralWestern
    {  385,  338,  457024,  457050,  457083, 26, 33, 34, }, // Australia_Eastern
    {  385,  356,  457117,  457142,  457174, 25, 32, 33, }, // Australia_Western
    {  385,  374,  457207,  457226,  457252, 19, 26, 27, }, // Azerbaijan
    {  385,  385,  457279,  457295,  457318, 16, 23, 24, }, // Azores
    {  385,  397,  457342,  457359,  457383, 17, 24, 25, }, // Bangladesh
    {  385,  415,       0,  457408,       0,  0, 13,  0, }, // Bhutan
    {  385,  422,       0,  457421,       0,  0, 15,  0, }, // Bolivia
    {  385,  437,  457436,  457452,  457475, 16, 23, 24, }, // Brasilia
    {  385,  454,       0,  457499,       0,  0, 26,  0, }, // Brunei
    {  385,  461,  457525,  457541,  457564, 16, 23, 24, }, // Cape_Verde
    {  385,  478,       0,  457588,       0,  0, 20,  0, }, // Chamorro
    {  385,  487,  457608,  457621,  457641, 13, 20, 21, }, // Chatham
    {  385,  495,  457662,  457673,  457691, 11, 18, 19, }, // Chile
    {  385,  501,  457710,  457721,  457739, 11, 18, 19, }, // China
    {  385,  507,  457758,  457775,  457799, 17, 24, 25, }, // Choibalsan
    {  385,  518,       0,  457824,       0,  0, 22,  0, }, // Christmas
    {  385,  528,       0,  457846,       0,  0, 27,  0, }, // Cocos
    {  385,  534,  457873,  457889,  457912, 16, 23, 24, }, // Colombia
    {  385,  543,  457936,  457951,  457973, 15, 22, 23, }, // Cook
    {  385,  548,  457996,  458007,  458025, 11, 18, 19, }, // Cuba
    {  385,  559,       0,  458044,       0,  0, 14,  0, }, // Davis
    {  385,  575,       0,  458058,       0,  0, 19,  0, }, // DumontDUrville
    {  385,  612,       0,  458077,       0,  0, 20,  0, }, // East_Timor
    {  385,  623,  458097,  458118,  458146, 21, 28, 29, }, // Easter
    {  385,  630,       0,  458175,       0,  0, 15,  0, }, // Ecuador
    {  385,  638,  458190,  458210,  458237, 20, 27, 28, }, // Europe_Central
    {  385,  653,  458265,  458285,  458312, 20, 27, 28, }, // Europe_Eastern
    {  385,  668,       0,  458340,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  385,  691,  458372,  458391,  458417, 19, 26, 27, }, // Europe_Western
    {  385,  706,  458444,  458474,  458511, 30, 37, 38, }, // Falkland
    {  385,  715,  458549,  458560,  458578, 11, 18, 19, }, // Fiji
    {  385,  720,       0,  458597,       0,  0, 23,  0, }, // French_Guiana
    {  385,  734,       0,  458620,       0,  0, 54,  0, }, // French_Southern
    {  385,  757,       0,  458674,       0,  0, 14,  0, }, // Galapagos
    {  385,  767,       0,  458688,       0,  0, 17,  0, }, // Gambier
    {  385,  775,  458705,  458719,  458740, 14, 21, 22, }, // Georgia
    {  385,  783,       0,  458762,       0,  0, 19,  0, }, // Gilbert_Islands
    {  385,  799,       0,  458781,       0,  0, 14,  0, }, // GMT
    {  385,  841,  458795,  458818,  458848, 23, 30, 31, }, // Greenland_Eastern
    {  385,  859,  458879,  458901,  458930, 22, 29, 30, }, // Greenland_Western
    {  385,  882,       0,  458960,       0,  0, 19,  0, }, // Gulf
    {  385,  887,       0,  458979,       0,  0, 13,  0, }, // Guyana
    {  385,  894,  458992,  459011,  459037, 19, 26, 27, }, // Hawaii_Aleutian
    {  385,  910,  459064,  459079,  459101, 15, 22, 23, }, // Hong_Kong
    {  385,  920,  459124,  459136,  459155, 12, 19, 20, }, // Hovd
    {  385,  925,       0,  459175,       0,  0, 20,  0, }, // India
    {  385,  931,       0,  459195,       0,  0, 20,  0, }, // Indian_Ocean
    {  385,  944,       0,  459215,       0,  0, 15,  0, }, // Indochina
    {  385,  954,       0,  459230,       0,  0, 24,  0, }, // Indonesia_Central
    {  385,  972,       0,  459254,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  385,  990,       0,  459278,       0,  0, 23,  0, }, // Indonesia_Western
    {  385, 1008,  459301,  459313,  459332, 12, 19, 20, }, // Iran
    {  385, 1019,  459352,  459367,  459389, 15, 22, 23, }, // Irkutsk
    {  385, 1027,  459412,  459426,  459447, 14, 21, 22, }, // Israel
    {  385, 1034,  459469,  459482,  459502, 13, 20, 21, }, // Japan
    {  385, 1040,  459523,  459555,  459594, 32, 39, 40, }, // Kamchatka
    {  385, 1058,       0,  459634,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  385, 1077,       0,  459658,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  385, 1106,  459681,  459694,  459714, 13, 20, 21, }, // Korea
    {  385, 1112,       0,  459735,       0,  0, 12,  0, }, // Kosrae
    {  385, 1119,  459747,  459766,  459792, 19, 26, 27, }, // Krasnoyarsk
    {  385, 1151,       0,  459819,       0,  0, 18,  0, }, // Kyrgystan
    {  385, 1175,       0,  459837,       0,  0, 17,  0, }, // Line_Islands
    {  385, 1188,  459854,  459876,  459905, 22, 29, 30, }, // Lord_Howe
    {  385, 1204,  459935,  459950,  459972, 15, 22, 23, }, // Magadan
    {  385, 1219,       0,  459995,       0,  0, 16,  0, }, // Malaysia
    {  385, 1228,       0,  460011,       0,  0, 16,  0, }, // Maldives
    {  385, 1237,       0,  460027,       0,  0, 18,  0, }, // Marquesas
    {  385, 1247,       0,  460045,       0,  0, 18,  0, }, // Marshall_Islands
    {  385, 1264,  460063,  460079,  460102, 16, 23, 24, }, // Mauritius
    {  385, 1274,       0,  460126,       0,  0, 13,  0, }, // Mawson
    {  385, 1281,  460139,  460177,  460222, 38, 45, 46, }, // Mexico_Pacific
    {  385, 1296,  460268,  460285,  460309, 17, 24, 25, }, // Mongolia
    {  385, 1305,  460334,  460348,  460369, 14, 21, 22, }, // Moscow
    {  385, 1312,       0,  460391,       0,  0, 13,  0, }, // Myanmar
    {  385, 1320,       0,  460404,       0,  0, 11,  0, }, // Nauru
    {  385, 1326,       0,  460415,       0,  0, 13,  0, }, // Nepal
    {  385, 1332,  460428,  460449,  460477, 21, 28, 29, }, // New_Caledonia
    {  385, 1346,  460506,  460524,  460549, 18, 25, 26, }, // New_Zealand
    {  385, 1358,  460575,  460594,  460620, 19, 26, 27, }, // Newfoundland
    {  385, 1371,       0,  460647,       0,  0, 11,  0, }, // Niue
    {  385, 1376,  460658,  460679,  460707, 21, 28, 29, }, // Norfolk
    {  385, 1384,  460736,  460761,  460793, 25, 32, 33, }, // Noronha
    {  385, 1406,  460826,  460845,  460871, 19, 26, 27, }, // Novosibirsk
    {  385, 1418,  460898,  460910,  460929, 12, 19, 20, }, // Omsk
    {  385, 1428,  460949,  460965,  460988, 16, 23, 24, }, // Pakistan
    {  385, 1437,       0,  461012,       0,  0, 11,  0, }, // Palau
    {  385, 1443,       0,  461023,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  385, 1460,  461047,  461063,  461086, 16, 23, 24, }, // Paraguay
    {  385, 1469,  461110,  461120,  461137, 10, 17, 18, }, // Peru
    {  385, 1474,  461155,  461169,  461190, 14, 21, 22, }, // Philippines
    {  385, 1486,       0,  461212,       0,  0, 18,  0, }, // Phoenix_Islands
    {  385, 1502,  461230,  461258,  461293, 28, 35, 36, }, // Pierre_Miquelon
    {  385, 1518,       0,  461329,       0,  0, 15,  0, }, // Pitcairn
    {  385, 1527,       0,  461344,       0,  0, 12,  0, }, // Ponape
    {  385, 1534,       0,  461356,       0,  0, 15,  0, }, // Pyongyang
    {  385, 1554,       0,  461371,       0,  0, 15,  0, }, // Reunion
    {  385, 1562,       0,  461386,       0,  0, 13,  0, }, // Rothera
    {  385, 1570,  461399,  461414,  461436, 15, 22, 23, }, // Sakhalin
    {  385, 1579,  461459,  461472,  461492, 13, 20, 21, }, // Samara
    {  385, 1596,  461513,  461524,  461542, 11, 18, 19, }, // Samoa
    {  385, 1602,       0,  461561,       0,  0, 18,  0, }, // Seychelles
    {  385, 1624,       0,  461579,       0,  0, 16,  0, }, // Singapore
    {  385, 1634,       0,  461595,       0,  0, 19,  0, }, // Solomon
    {  385, 1642,       0,  461614,       0,  0, 23,  0, }, // South_Georgia
    {  385, 1656,       0,  461637,       0,  0, 15,  0, }, // Suriname
    {  385, 1676,       0,  461652,       0,  0, 11,  0, }, // Syowa
    {  385, 1682,       0,  461663,       0,  0, 11,  0, }, // Tahiti
    {  385, 1689,  461674,  461686,  461705, 12, 19, 20, }, // Taipei
    {  385, 1696,       0,  461725,       0,  0, 19,  0, }, // Tajikistan
    {  385, 1724,       0,  461744,       0,  0, 13,  0, }, // Tokelau
    {  385, 1732,  461757,  461769,  461788, 12, 19, 20, }, // Tonga
    {  385, 1738,       0,  461808,       0,  0, 11,  0, }, // Truk
    {  385, 1750,  461819,  461839,  461866, 20, 27, 28, }, // Turkmenistan
    {  385, 1763,       0,  461894,       0,  0, 12,  0, }, // Tuvalu
    {  385, 1777,  461906,  461921,  461943, 15, 22, 23, }, // Uruguay
    {  385, 1792,  461966,  461984,  462009, 18, 25, 26, }, // Uzbekistan
    {  385, 1803,  462035,  462048,  462068, 13, 20, 21, }, // Vanuatu
    {  385, 1811,       0,  462089,       0,  0, 16,  0, }, // Venezuela
    {  385, 1821,  462105,  462124,  462150, 19, 26, 27, }, // Vladivostok
    {  385, 1833,  462177,  462194,  462218, 17, 24, 25, }, // Volgograd
    {  385, 1843,       0,  462243,       0,  0, 14,  0, }, // Vostok
    {  385, 1850,       0,  462257,       0,  0, 17,  0, }, // Wake
    {  385, 1855,       0,  462274,       0,  0, 24,  0, }, // Wallis
    {  385, 1862,  462298,  462313,  462335, 15, 22, 23, }, // Yakutsk
    {  385, 1870,  462358,  462379,  462407, 21, 28, 29, }, // Yekaterinburg
    {  385, 1892,       0,  462436,       0,  0, 13,  0, }, // Yukon
    {  386,   17,       0,  462449,       0,  0, 20,  0, }, // Africa_Central Lingala/Latin/Congo - Kinshasa
    {  386,   32,       0,  462469,       0,  0, 26,  0, }, // Africa_Eastern
    {  386,   65,       0,  462495,       0,  0, 24,  0, }, // Africa_Southern
    {  386,  799,       0,  462519,       0,  0, 17,  0, }, // GMT
    {  386, 1602,       0,  462536,       0,  0, 17,  0, }, // Seychelles
    {  390,    0,  462553,  462563,  462586, 10, 23, 18, }, // Acre Lithuanian/Latin/Lithuania
    {  390,    5,       0,  462604,       0,  0, 18,  0, }, // Afghanistan
    {  390,   17,       0,  462622,       0,  0, 24,  0, }, // Africa_Central
    {  390,   32,       0,  462646,       0,  0, 19,  0, }, // Africa_Eastern
    {  390,   65,       0,  462665,       0,  0, 20,  0, }, // Africa_Southern
    {  390,   81,  462685,  462706,  462734, 21, 28, 29, }, // Africa_Western
    {  390,  107,  462763,  462778,  462800, 15, 22, 23, }, // Alaska
    {  390,  128,  462823,  462837,  462858, 14, 21, 22, }, // Almaty
    {  390,  135,  462880,  462895,  462917, 15, 22, 23, }, // Amazon
    {  390,  142,  462940,  462970,  463007, 30, 37, 38, }, // America_Central
    {  390,  158,  463045,  463073,  463108, 28, 35, 36, }, // America_Eastern
    {  390,  174,  463144,  463173,  463209, 29, 36, 37, }, // America_Mountain
    {  390,  191,  463246,  463287,  463335, 41, 48, 49, }, // America_Pacific
    {  390,  207,  463384,  463399,  463421, 15, 22, 23, }, // Anadyr
    {  390,  214,  463444,  463457,  463477, 13, 20, 21, }, // Apia
    {  390,  219,  463498,  463510,  463529, 12, 19, 20, }, // Aqtau
    {  390,  225,  463549,  463563,  463584, 14, 21, 22, }, // Aqtobe
    {  390,  232,  463606,  463621,  463643, 15, 22, 23, }, // Arabian
    {  390,  240,  463666,  463683,  463707, 17, 24, 25, }, // Argentina
    {  390,  250,  463732,  463756,  463787, 24, 31, 32, }, // Argentina_Western
    {  390,  268,  463819,  463835,  463858, 16, 23, 24, }, // Armenia
    {  390,  286,  463882,  463896,  463917, 14, 21, 22, }, // Atlantic
    {  390,  295,  463939,  463967,  464002, 28, 35, 36, }, // Australia_Central
    {  390,  313,  464038,  464073,  464115, 35, 42, 43, }, // Australia_CentralWestern
    {  390,  338,  464158,  464181,  464211, 23, 30, 31, }, // Australia_Eastern
    {  390,  356,  464242,  464267,  464299, 25, 32, 33, }, // Australia_Western
    {  390,  374,  464332,  464351,  464377, 19, 26, 27, }, // Azerbaijan
    {  390,  385,  464404,  464421,  464445, 17, 24, 25, }, // Azores
    {  390,  397,  464470,  464487,  464511, 17, 24, 25, }, // Bangladesh
    {  390,  415,       0,  464536,       0,  0, 13,  0, }, // Bhutan
    {  390,  422,       0,  464549,       0,  0, 16,  0, }, // Bolivia
    {  390,  437,  464565,  464582,  464606, 17, 24, 25, }, // Brasilia
    {  390,  454,       0,  464631,       0,  0, 27,  0, }, // Brunei
    {  390,  461,  464658,  464680,  464709, 22, 29, 30, }, // Cape_Verde
    {  390,  472,       0,  464739,       0,  0, 13,  0, }, // Casey
    {  390,  478,       0,  464752,       0,  0, 13,  0, }, // Chamorro
    {  390,  487,  464765,  464778,  464798, 13, 20, 21, }, // Chatham
    {  390,  495,  464819,  464831,  464850, 12, 19, 20, }, // Chile
    {  390,  501,  464870,  464884,  464905, 14, 21, 22, }, // China
    {  390,  507,  464927,  464944,  464968, 17, 24, 25, }, // Choibalsan
    {  390,  518,       0,  464993,       0,  0, 19,  0, }, // Christmas
    {  390,  528,       0,  465012,       0,  0, 18,  0, }, // Cocos
    {  390,  534,  465030,  465047,  465071, 17, 24, 25, }, // Colombia
    {  390,  543,  465096,  465112,  465135, 16, 23, 30, }, // Cook
    {  390,  548,  465165,  465177,  465196, 12, 19, 20, }, // Cuba
    {  390,  559,       0,  465216,       0,  0, 14,  0, }, // Davis
    {  390,  575,       0,  465230,       0,  0, 24,  0, }, // DumontDUrville
    {  390,  612,       0,  465254,       0,  0, 18,  0, }, // East_Timor
    {  390,  623,  465272,  465291,  465317, 19, 26, 27, }, // Easter
    {  390,  630,       0,  465344,       0,  0, 15,  0, }, // Ecuador
    {  390,  638,  465359,  465381,  465410, 22, 29, 30, }, // Europe_Central
    {  390,  653,  465440,  465459,  465485, 19, 26, 27, }, // Europe_Eastern
    {  390,  668,       0,  465512,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {  390,  691,  465540,  465561,  465589, 21, 28, 29, }, // Europe_Western
    {  390,  706,  465618,  465639,  465667, 21, 28, 29, }, // Falkland
    {  390,  715,  465696,  465709,  465729, 13, 20, 21, }, // Fiji
    {  390,  720,       0,  465750,       0,  0, 26,  0, }, // French_Guiana
    {  390,  734,       0,  465776,       0,  0, 39,  0, }, // French_Southern
    {  390,  757,       0,  465815,       0,  0, 15,  0, }, // Galapagos
    {  390,  767,       0,  465830,       0,  0, 14,  0, }, // Gambier
    {  390,  775,  465844,  465859,  465881, 15, 22, 23, }, // Georgia
    {  390,  783,       0,  465904,       0,  0, 20,  0, }, // Gilbert_Islands
    {  390,  799,       0,  465924,       0,  0, 15,  0, }, // GMT
    {  390,  841,  465939,  465963,  465994, 24, 31, 32, }, // Greenland_Eastern
    {  390,  859,  466026,  466052,  466085, 26, 33, 34, }, // Greenland_Western
    {  390,  877,       0,  466119,       0,  0, 12,  0, }, // Guam
    {  390,  882,       0,  466131,       0,  0, 23,  0, }, // Gulf
    {  390,  887,       0,  466154,       0,  0, 14,  0, }, // Guyana
    {  390,  894,  466168,  466188,  466215, 20, 27, 28, }, // Hawaii_Aleutian
    {  390,  910,  466243,  466258,  466280, 15, 22, 23, }, // Hong_Kong
    {  390,  920,  466303,  466315,  466334, 12, 19, 20, }, // Hovd
    {  390,  925,       0,  466354,       0,  0, 14,  0, }, // India
    {  390,  931,       0,  466368,       0,  0, 24,  0, }, // Indian_Ocean
    {  390,  944,       0,  466392,       0,  0, 18,  0, }, // Indochina
    {  390,  954,       0,  466410,       0,  0, 28,  0, }, // Indonesia_Central
    {  390,  972,       0,  466438,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  390,  990,       0,  466461,       0,  0, 25,  0, }, // Indonesia_Western
    {  390, 1008,  466486,  466498,  466517, 12, 19, 20, }, // Iran
    {  390, 1019,  466537,  466552,  466574, 15, 22, 23, }, // Irkutsk
    {  390, 1027,  466597,  466612,  466634, 15, 22, 23, }, // Israel
    {  390, 1034,  466657,  466673,  466696, 16, 23, 24, }, // Japan
    {  390, 1040,  466720,  466752,  466791, 32, 39, 40, }, // Kamchatka
    {  390, 1058,       0,  466831,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  390, 1077,       0,  466854,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  390, 1106,  466879,  466893,  466914, 14, 21, 22, }, // Korea
    {  390, 1112,       0,  466936,       0,  0, 14,  0, }, // Kosrae
    {  390, 1119,  466950,  466969,  466995, 19, 26, 27, }, // Krasnoyarsk
    {  390, 1151,       0,  467022,       0,  0, 17,  0, }, // Kyrgystan
    {  390, 1161,       0,  467039,       0,  0, 13,  0, }, // Lanka
    {  390, 1175,       0,  467052,       0,  0, 17,  0, }, // Line_Islands
    {  390, 1188,  467069,  467085,  467108, 16, 23, 24, }, // Lord_Howe
    {  390, 1198,  467132,  467144,  467163, 12, 19, 20, }, // Macau
    {  390, 1204,  467183,  467198,  467220, 15, 22, 23, }, // Magadan
    {  390, 1219,       0,  467243,       0,  0, 17,  0, }, // Malaysia
    {  390, 1228,       0,  467260,       0,  0, 14,  0, }, // Maldives
    {  390, 1237,       0,  467274,       0,  0, 19,  0, }, // Marquesas
    {  390, 1247,       0,  467293,       0,  0, 19,  0, }, // Marshall_Islands
    {  390, 1264,  467312,  467330,  467355, 18, 25, 26, }, // Mauritius
    {  390, 1274,       0,  467381,       0,  0, 13,  0, }, // Mawson
    {  390, 1281,  467394,  467427,  467467, 33, 40, 41, }, // Mexico_Pacific
    {  390, 1296,  467508,  467526,  467551, 18, 25, 26, }, // Mongolia
    {  390, 1305,  467577,  467591,  467612, 14, 21, 22, }, // Moscow
    {  390, 1312,       0,  467634,       0,  0, 15,  0, }, // Myanmar
    {  390, 1320,       0,  467649,       0,  0, 12,  0, }, // Nauru
    {  390, 1326,       0,  467661,       0,  0, 13,  0, }, // Nepal
    {  390, 1332,  467674,  467702,  467737, 28, 35, 36, }, // New_Caledonia
    {  390, 1346,  467773,  467800,  467834, 27, 34, 35, }, // New_Zealand
    {  390, 1358,  467869,  467889,  467916, 20, 27, 28, }, // Newfoundland
    {  390, 1371,       0,  467944,       0,  0, 12,  0, }, // Niue
    {  390, 1376,  467956,  467976,  468003, 20, 27, 28, }, // Norfolk
    {  390, 1384,  468031,  468058,  468092, 27, 34, 35, }, // Noronha
    {  390, 1392,       0,  468127,       0,  0, 28,  0, }, // North_Mariana
    {  390, 1406,  468155,  468174,  468200, 19, 26, 27, }, // Novosibirsk
    {  390, 1418,  468227,  468239,  468258, 12, 19, 20, }, // Omsk
    {  390, 1428,  468278,  468294,  468317, 16, 23, 24, }, // Pakistan
    {  390, 1437,       0,  468341,       0,  0, 12,  0, }, // Palau
    {  390, 1443,       0,  468353,       0,  0, 31,  0, }, // Papua_New_Guinea
    {  390, 1460,  468384,  468402,  468427, 18, 25, 26, }, // Paraguay
    {  390, 1469,  468453,  468464,  468482, 11, 18, 19, }, // Peru
    {  390, 1474,  468501,  468516,  468538, 15, 22, 23, }, // Philippines
    {  390, 1486,       0,  468561,       0,  0, 19,  0, }, // Phoenix_Islands
    {  390, 1502,  468580,  468608,  468643, 28, 35, 36, }, // Pierre_Miquelon
    {  390, 1518,       0,  468679,       0,  0, 15,  0, }, // Pitcairn
    {  390, 1527,       0,  468694,       0,  0, 14,  0, }, // Ponape
    {  390, 1534,       0,  468708,       0,  0, 16,  0, }, // Pyongyang
    {  390, 1544,  468724,  468741,  468765, 17, 24, 25, }, // Qyzylorda
    {  390, 1554,       0,  468790,       0,  0, 15,  0, }, // Reunion
    {  390, 1562,       0,  468805,       0,  0, 14,  0, }, // Rothera
    {  390, 1570,  468819,  468835,  468858, 16, 23, 24, }, // Sakhalin
    {  390, 1579,  468882,  468896,  468917, 14, 21, 22, }, // Samara
    {  390, 1596,  468939,  468951,  468970, 12, 19, 20, }, // Samoa
    {  390, 1602,       0,  468990,       0,  0, 15,  0, }, // Seychelles
    {  390, 1624,       0,  469005,       0,  0, 16,  0, }, // Singapore
    {  390, 1634,       0,  469021,       0,  0, 21,  0, }, // Solomon
    {  390, 1642,       0,  469042,       0,  0, 23,  0, }, // South_Georgia
    {  390, 1656,       0,  469065,       0,  0, 15,  0, }, // Suriname
    {  390, 1676,       0,  469080,       0,  0, 13,  0, }, // Syowa
    {  390, 1682,       0,  469093,       0,  0, 14,  0, }, // Tahiti
    {  390, 1689,  469107,  469123,  469146, 16, 23, 24, }, // Taipei
    {  390, 1696,       0,  469170,       0,  0, 19,  0, }, // Tajikistan
    {  390, 1724,       0,  469189,       0,  0, 14,  0, }, // Tokelau
    {  390, 1732,  469203,  469216,  469236, 13, 20, 21, }, // Tonga
    {  390, 1738,       0,  469257,       0,  0, 11,  0, }, // Truk
    {  390, 1750,  469268,  469288,  469315, 20, 27, 28, }, // Turkmenistan
    {  390, 1763,       0,  469343,       0,  0, 13,  0, }, // Tuvalu
    {  390, 1777,  469356,  469373,  469397, 17, 24, 25, }, // Uruguay
    {  390, 1792,  469422,  469440,  469465, 18, 25, 26, }, // Uzbekistan
    {  390, 1803,  469491,  469505,  469526, 14, 21, 22, }, // Vanuatu
    {  390, 1811,       0,  469548,       0,  0, 17,  0, }, // Venezuela
    {  390, 1821,  469565,  469584,  469610, 19, 26, 27, }, // Vladivostok
    {  390, 1833,  469637,  469654,  469678, 17, 24, 25, }, // Volgograd
    {  390, 1843,       0,  469703,       0,  0, 14,  0, }, // Vostok
    {  390, 1850,       0,  469717,       0,  0, 18,  0, }, // Wake
    {  390, 1855,       0,  469735,       0,  0, 24,  0, }, // Wallis
    {  390, 1862,  469759,  469774,  469796, 15, 22, 23, }, // Yakutsk
    {  390, 1870,  469819,  469840,  469868, 21, 28, 29, }, // Yekaterinburg
    {  390, 1892,       0,  469897,       0,  0, 13,  0, }, // Yukon
    {  391,  534,  469910,  469932,  469960, 22, 28, 25, }, // Colombia Lojban/Latin/world
    {  391,  757,       0,  469985,       0,  0, 21,  0, }, // Galapagos
    {  392,    5,       0,  470006,       0,  0, 14,  0, }, // Afghanistan Lower Sorbian/Latin/Germany
    {  392,   17,       0,  470020,       0,  0, 18,  0, }, // Africa_Central
    {  392,   32,       0,  470038,       0,  0, 21,  0, }, // Africa_Eastern
    {  392,   65,       0,  470059,       0,  0, 23,  0, }, // Africa_Southern
    {  392,   81,  470082,  470104,  470137, 22, 33, 31, }, // Africa_Western
    {  392,  107,  470168,  470182,  470207, 14, 25, 23, }, // Alaska
    {  392,  135,  470230,  470244,  470269, 14, 25, 23, }, // Amazon
    {  392,  142,  470292,  470325,  470369, 33, 44, 42, }, // America_Central
    {  392,  158,  470411,  470445,  470490, 34, 45, 43, }, // America_Eastern
    {  392,  174,  470533,  470563,  470604, 30, 41, 39, }, // America_Mountain
    {  392,  191,  470643,  470676,  470720, 33, 44, 42, }, // America_Pacific
    {  392,  214,  470762,  470773,  470795, 11, 22, 20, }, // Apia
    {  392,  232,  470815,  470827,  470850, 12, 23, 21, }, // Arabian
    {  392,  240,  470871,  470886,  470912, 15, 26, 24, }, // Argentina
    {  392,  250,  470936,  470962,  470999, 26, 37, 35, }, // Argentina_Western
    {  392,  268,  471034,  471046,  471069, 12, 23, 21, }, // Armenia
    {  392,  286,  471090,  471104,  471129, 14, 25, 23, }, // Atlantic
    {  392,  295,  471152,  471173,  471205, 21, 32, 30, }, // Australia_Central
    {  392,  313,  471235,  471268,  471312, 33, 44, 42, }, // Australia_CentralWestern
    {  392,  338,  471354,  471378,  471413, 24, 35, 33, }, // Australia_Eastern
    {  392,  356,  471446,  471471,  471507, 25, 36, 34, }, // Australia_Western
    {  392,  374,  471541,  471560,  471590, 19, 30, 28, }, // Azerbaijan
    {  392,  385,  471618,  471629,  471651, 11, 22, 20, }, // Azores
    {  392,  397,  471671,  471687,  471714, 16, 27, 25, }, // Bangladesh
    {  392,  415,       0,  471739,       0,  0, 13,  0, }, // Bhutan
    {  392,  422,       0,  471752,       0,  0, 13,  0, }, // Bolivia
    {  392,  437,  471765,  471777,  471800, 12, 23, 21, }, // Brasilia
    {  392,  454,       0,  471821,       0,  0, 13,  0, }, // Brunei
    {  392,  461,  471834,  471848,  471873, 14, 25, 23, }, // Cape_Verde
    {  392,  478,       0,  471896,       0,  0, 14,  0, }, // Chamorro
    {  392,  487,  471910,  471924,  471949, 14, 25, 23, }, // Chatham
    {  392,  495,  471972,  471983,  472005, 11, 22, 20, }, // Chile
    {  392,  501,  472025,  472036,  472058, 11, 22, 20, }, // China
    {  392,  507,  472078,  472095,  472123, 17, 28, 26, }, // Choibalsan
    {  392,  518,       0,  472149,       0,  0, 19,  0, }, // Christmas
    {  392,  528,       0,  472168,       0,  0, 20,  0, }, // Cocos
    {  392,  534,  472188,  472202,  472227, 14, 25, 23, }, // Colombia
    {  392,  543,  472250,  472269,  472299, 19, 30, 28, }, // Cook
    {  392,  548,  472327,  472339,  472362, 12, 23, 21, }, // Cuba
    {  392,  559,       0,  472383,       0,  0,  9,  0, }, // Davis
    {  392,  575,       0,  472392,       0,  0, 18,  0, }, // DumontDUrville
    {  392,  612,       0,  472410,       0,  0, 22,  0, }, // East_Timor
    {  392,  623,  472432,  472451,  472481, 19, 30, 28, }, // Easter
    {  392,  630,       0,  472509,       0,  0, 14,  0, }, // Ecuador
    {  392,  638,  472523,  472545,  472578, 22, 33, 31, }, // Europe_Central
    {  392,  653,  472609,  472633,  472668, 24, 35, 33, }, // Europe_Eastern
    {  392,  668,       0,  472701,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {  392,  691,  472719,  472744,  472780, 25, 36, 34, }, // Europe_Western
    {  392,  706,  472814,  472829,  472855, 15, 26, 24, }, // Falkland
    {  392,  715,  472879,  472891,  472914, 12, 23, 21, }, // Fiji
    {  392,  720,       0,  472935,       0,  0, 22,  0, }, // French_Guiana
    {  392,  734,       0,  472957,       0,  0, 59,  0, }, // French_Southern
    {  392,  757,       0,  473016,       0,  0, 15,  0, }, // Galapagos
    {  392,  767,       0,  473031,       0,  0, 14,  0, }, // Gambier
    {  392,  775,  473045,  473058,  473082, 13, 24, 22, }, // Georgia
    {  392,  783,       0,  473104,       0,  0, 22,  0, }, // Gilbert_Islands
    {  392,  799,       0,  473126,       0,  0, 16,  0, }, // GMT
    {  392,  841,  473142,  473167,  473203, 25, 36, 34, }, // Greenland_Eastern
    {  392,  859,  473237,  473263,  473300, 26, 37, 35, }, // Greenland_Western
    {  392,  882,       0,  473335,       0,  0, 20,  0, }, // Gulf
    {  392,  887,       0,  473355,       0,  0, 12,  0, }, // Guyana
    {  392,  894,  473367,  473389,  473422, 22, 33, 31, }, // Hawaii_Aleutian
    {  392,  910,  473453,  473468,  473494, 15, 26, 24, }, // Hong_Kong
    {  392,  920,  473518,  473530,  473553, 12, 23, 21, }, // Hovd
    {  392,  925,       0,  473574,       0,  0, 11,  0, }, // India
    {  392,  931,       0,  473585,       0,  0, 20,  0, }, // Indian_Ocean
    {  392,  944,       0,  473605,       0,  0, 15,  0, }, // Indochina
    {  392,  954,       0,  473620,       0,  0, 20,  0, }, // Indonesia_Central
    {  392,  972,       0,  473640,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  392,  990,       0,  473659,       0,  0, 24,  0, }, // Indonesia_Western
    {  392, 1008,  473683,  473694,  473716, 11, 22, 20, }, // Iran
    {  392, 1019,  473736,  473748,  473771, 12, 23, 21, }, // Irkutsk
    {  392, 1027,  473792,  473805,  473829, 13, 24, 22, }, // Israel
    {  392, 1034,  473851,  473863,  473886, 12, 23, 21, }, // Japan
    {  392, 1058,       0,  473907,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  392, 1077,       0,  473930,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  392, 1106,  473954,  473966,  473989, 12, 23, 21, }, // Korea
    {  392, 1112,       0,  474010,       0,  0, 13,  0, }, // Kosrae
    {  392, 1119,  474023,  474039,  474066, 16, 27, 25, }, // Krasnoyarsk
    {  392, 1151,       0,  474091,       0,  0, 12,  0, }, // Kyrgystan
    {  392, 1175,       0,  474103,       0,  0, 20,  0, }, // Line_Islands
    {  392, 1188,  474123,  474141,  474170, 18, 29, 27, }, // Lord_Howe
    {  392, 1204,  474197,  474211,  474236, 14, 25, 23, }, // Magadan
    {  392, 1219,       0,  474259,       0,  0, 14,  0, }, // Malaysia
    {  392, 1228,       0,  474273,       0,  0, 14,  0, }, // Maldives
    {  392, 1237,       0,  474287,       0,  0, 13,  0, }, // Marquesas
    {  392, 1247,       0,  474300,       0,  0, 23,  0, }, // Marshall_Islands
    {  392, 1264,  474323,  474337,  474362, 14, 25, 23, }, // Mauritius
    {  392, 1274,       0,  474385,       0,  0, 10,  0, }, // Mawson
    {  392, 1281,  474395,  474416,  474448, 21, 32, 30, }, // Mexico_Pacific
    {  392, 1296,  474478,  474495,  474523, 17, 28, 26, }, // Mongolia
    {  392, 1305,  474549,  474562,  474586, 13, 24, 22, }, // Moscow
    {  392, 1312,       0,  474608,       0,  0, 14,  0, }, // Myanmar
    {  392, 1320,       0,  474622,       0,  0, 12,  0, }, // Nauru
    {  392, 1326,       0,  474634,       0,  0, 12,  0, }, // Nepal
    {  392, 1332,  474646,  474665,  474695, 19, 30, 28, }, // New_Caledonia
    {  392, 1346,  474723,  474741,  474770, 18, 29, 27, }, // New_Zealand
    {  392, 1358,  474797,  474816,  474846, 19, 30, 28, }, // Newfoundland
    {  392, 1371,       0,  474874,       0,  0, 11,  0, }, // Niue
    {  392, 1376,  474885,  474901,  474928, 16, 27, 25, }, // Norfolk
    {  392, 1384,  474953,  474976,  475010, 23, 34, 32, }, // Noronha
    {  392, 1406,  475042,  475058,  475085, 16, 27, 25, }, // Novosibirsk
    {  392, 1418,  475110,  475119,  475139,  9, 20, 18, }, // Omsk
    {  392, 1428,  475157,  475172,  475198, 15, 26, 24, }, // Pakistan
    {  392, 1437,       0,  475222,       0,  0, 12,  0, }, // Palau
    {  392, 1443,       0,  475234,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  392, 1460,  475256,  475271,  475297, 15, 26, 24, }, // Paraguay
    {  392, 1469,  475321,  475332,  475354, 11, 22, 20, }, // Peru
    {  392, 1474,  475374,  475388,  475413, 14, 25, 23, }, // Philippines
    {  392, 1486,       0,  475436,       0,  0, 22,  0, }, // Phoenix_Islands
    {  392, 1502,  475458,  475486,  475525, 28, 39, 37, }, // Pierre_Miquelon
    {  392, 1518,       0,  475562,       0,  0, 23,  0, }, // Pitcairn
    {  392, 1527,       0,  475585,       0,  0, 12,  0, }, // Ponape
    {  392, 1534,       0,  475597,       0,  0, 16,  0, }, // Pyongyang
    {  392, 1554,       0,  475613,       0,  0, 14,  0, }, // Reunion
    {  392, 1562,       0,  475627,       0,  0, 11,  0, }, // Rothera
    {  392, 1570,  475638,  475653,  475679, 15, 26, 24, }, // Sakhalin
    {  392, 1596,  475703,  475715,  475738, 12, 23, 21, }, // Samoa
    {  392, 1602,       0,  475759,       0,  0, 14,  0, }, // Seychelles
    {  392, 1624,       0,  475773,       0,  0, 15,  0, }, // Singapore
    {  392, 1634,       0,  475788,       0,  0, 14,  0, }, // Solomon
    {  392, 1642,       0,  475802,       0,  0, 25,  0, }, // South_Georgia
    {  392, 1656,       0,  475827,       0,  0, 14,  0, }, // Suriname
    {  392, 1676,       0,  475841,       0,  0,  9,  0, }, // Syowa
    {  392, 1682,       0,  475850,       0,  0, 13,  0, }, // Tahiti
    {  392, 1689,  475863,  475878,  475904, 15, 26, 24, }, // Taipei
    {  392, 1696,       0,  475928,       0,  0, 14,  0, }, // Tajikistan
    {  392, 1724,       0,  475942,       0,  0, 14,  0, }, // Tokelau
    {  392, 1732,  475956,  475968,  475991, 12, 23, 21, }, // Tonga
    {  392, 1738,       0,  476012,       0,  0, 12,  0, }, // Truk
    {  392, 1750,  476024,  476039,  476065, 15, 26, 24, }, // Turkmenistan
    {  392, 1763,       0,  476089,       0,  0, 12,  0, }, // Tuvalu
    {  392, 1777,  476101,  476115,  476140, 14, 25, 23, }, // Uruguay
    {  392, 1792,  476163,  476176,  476200, 13, 24, 22, }, // Uzbekistan
    {  392, 1803,  476222,  476235,  476259, 13, 24, 22, }, // Vanuatu
    {  392, 1811,       0,  476281,       0,  0, 15,  0, }, // Venezuela
    {  392, 1821,  476296,  476314,  476343, 18, 29, 27, }, // Vladivostok
    {  392, 1833,  476370,  476386,  476413, 16, 27, 25, }, // Volgograd
    {  392, 1843,       0,  476438,       0,  0, 10,  0, }, // Vostok
    {  392, 1850,       0,  476448,       0,  0, 13,  0, }, // Wake
    {  392, 1855,       0,  476461,       0,  0, 25,  0, }, // Wallis
    {  392, 1862,  476486,  476498,  476521, 12, 23, 21, }, // Yakutsk
    {  392, 1870,  476542,  476562,  476593, 20, 31, 29, }, // Yekaterinburg
    {  392, 1892,       0,  476622,       0,  0, 12,  0, }, // Yukon
    {  393,   17,       0,  476634,       0,  0, 24,  0, }, // Africa_Central Low German/Latin/Germany
    {  393,   32,       0,  476658,       0,  0, 20,  0, }, // Africa_Eastern
    {  393,   65,       0,  476678,       0,  0, 20,  0, }, // Africa_Southern
    {  393,   81,  476698,  476718,  476746, 20, 28, 26, }, // Africa_Western
    {  393,  142,  476772,  476802,  476841, 30, 39, 37, }, // America_Central
    {  393,  158,  476878,  476907,  476944, 29, 37, 35, }, // America_Eastern
    {  393,  174,  476979,  477005,  477040, 26, 35, 33, }, // America_Mountain
    {  393,  191,  477073,  477102,  477140, 29, 38, 36, }, // America_Pacific
    {  393,  232,  477176,  477189,  477210, 13, 21, 19, }, // Arabian
    {  393,  286,  477229,  477259,  477298, 30, 39, 37, }, // Atlantic
    {  393,  295,  477335,  477359,  477391, 24, 32, 30, }, // Australia_Central
    {  393,  313,  477421,  477449,  477485, 28, 36, 34, }, // Australia_CentralWestern
    {  393,  338,  477519,  477539,  477567, 20, 28, 26, }, // Australia_Eastern
    {  393,  356,  477593,  477613,  477641, 20, 28, 26, }, // Australia_Western
    {  393,  501,  477667,  477677,  477695, 10, 18, 16, }, // China
    {  393,  638,  477711,  477734,  477765, 23, 31, 29, }, // Europe_Central
    {  393,  653,  477794,  477813,  477840, 19, 27, 25, }, // Europe_Eastern
    {  393,  691,  477865,  477884,  477911, 19, 27, 25, }, // Europe_Western
    {  393,  799,       0,  477936,       0,  0, 19,  0, }, // GMT
    {  393,  925,       0,  477955,       0,  0, 11,  0, }, // India
    {  393,  944,       0,  477966,       0,  0, 22,  0, }, // Indochina
    {  393,  954,       0,  477988,       0,  0, 23,  0, }, // Indonesia_Central
    {  393,  972,       0,  478011,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  393,  990,       0,  478030,       0,  0, 19,  0, }, // Indonesia_Western
    {  393, 1027,  478049,  478060,  478079, 11, 19, 17, }, // Israel
    {  393, 1034,  478096,  478110,  478132, 14, 22, 20, }, // Japan
    {  393, 1106,  478152,  478167,  478190, 15, 23, 21, }, // Korea
    {  393, 1305,  478211,  478222,  478241, 11, 19, 17, }, // Moscow
    {  399,    0,  478258,  478267,  478282,  9, 15, 15, }, // Acre Luxembourgish/Latin/Luxembourg
    {  399,    5,       0,  478297,       0,  0, 16,  0, }, // Afghanistan
    {  399,   17,       0,  478313,       0,  0, 23,  0, }, // Africa_Central
    {  399,   32,       0,  478336,       0,  0, 19,  0, }, // Africa_Eastern
    {  399,   65,       0,  478355,       0,  0, 19,  0, }, // Africa_Southern
    {  399,   81,  478374,  478394,  478420, 20, 26, 26, }, // Africa_Western
    {  399,  107,  478446,  478457,  478474, 11, 17, 17, }, // Alaska
    {  399,  128,  478491,  478502,  478519, 11, 17, 17, }, // Almaty
    {  399,  135,  478536,  478549,  478568, 13, 19, 19, }, // Amazon
    {  399,  142,  478587,  478614,  478648, 27, 34, 34, }, // America_Central
    {  399,  158,  478682,  478712,  478749, 30, 37, 37, }, // America_Eastern
    {  399,  174,  478786,  478805,  478830, 19, 25, 25, }, // America_Mountain
    {  399,  191,  478855,  478886,  478924, 31, 38, 38, }, // America_Pacific
    {  399,  207,  478962,  478973,  478990, 11, 17, 17, }, // Anadyr
    {  399,  232,  479007,  479020,  479039, 13, 19, 19, }, // Arabian
    {  399,  240,  479058,  479075,  479098, 17, 23, 23, }, // Argentina
    {  399,  250,  479121,  479142,  479169, 21, 27, 27, }, // Argentina_Western
    {  399,  268,  479196,  479210,  479230, 14, 20, 20, }, // Armenia
    {  399,  286,  479250,  479263,  479282, 13, 19, 19, }, // Atlantic
    {  399,  295,  479301,  479324,  479353, 23, 29, 29, }, // Australia_Central
    {  399,  313,  479382,  479411,  479446, 29, 35, 35, }, // Australia_CentralWestern
    {  399,  338,  479481,  479500,  479525, 19, 25, 25, }, // Australia_Eastern
    {  399,  356,  479391,  479420,  479455, 20, 26, 26, }, // Australia_Western
    {  399,  374,  479550,  479572,  479600, 22, 28, 28, }, // Azerbaijan
    {  399,  385,  479628,  479639,  479656, 11, 17, 17, }, // Azores
    {  399,  397,  479673,  479689,  479711, 16, 22, 22, }, // Bangladesh
    {  399,  415,       0,  479733,       0,  0, 11,  0, }, // Bhutan
    {  399,  422,       0,  479744,       0,  0, 17,  0, }, // Bolivia
    {  399,  437,  479761,  479774,  479793, 13, 19, 19, }, // Brasilia
    {  399,  454,       0,  479812,       0,  0, 11,  0, }, // Brunei
    {  399,  461,  479823,  479837,  479857, 14, 20, 20, }, // Cape_Verde
    {  399,  478,       0,  479877,       0,  0, 13,  0, }, // Chamorro
    {  399,  487,  479890,  479902,  479920, 12, 18, 18, }, // Chatham
    {  399,  495,  479938,  479953,  479974, 15, 21, 21, }, // Chile
    {  399,  501,  479995,  480010,  480031, 15, 21, 21, }, // China
    {  399,  507,  480052,  480067,  480088, 15, 21, 21, }, // Choibalsan
    {  399,  518,       0,  480109,       0,  0, 22,  0, }, // Christmas
    {  399,  528,       0,  480131,       0,  0, 17,  0, }, // Cocos
    {  399,  534,  480148,  480166,  480190, 18, 24, 24, }, // Colombia
    {  399,  543,  480214,  480230,  480252, 16, 22, 22, }, // Cook
    {  399,  548,  480274,  480288,  480308, 14, 20, 20, }, // Cuba
    {  399,  559,       0,  480328,       0,  0, 10,  0, }, // Davis
    {  399,  575,       0,  480338,       0,  0, 21,  0, }, // DumontDUrville
    {  399,  612,       0,  480359,       0,  0, 13,  0, }, // East_Timor
    {  399,  623,  480372,  480390,  480414, 18, 24, 24, }, // Easter
    {  399,  630,       0,  480438,       0,  0, 19,  0, }, // Ecuador
    {  399,  638,  480457,  480478,  480505, 21, 27, 27, }, // Europe_Central
    {  399,  653,  480532,  480550,  480574, 18, 24, 24, }, // Europe_Eastern
    {  399,  691,  480598,  480617,  480642, 19, 25, 25, }, // Europe_Western
    {  399,  706,  480667,  480687,  480713, 20, 26, 26, }, // Falkland
    {  399,  715,  480739,  480751,  480769, 12, 18, 18, }, // Fiji
    {  399,  720,       0,  480787,       0,  0, 23,  0, }, // French_Guiana
    {  399,  734,       0,  480810,       0,  0, 41,  0, }, // French_Southern
    {  399,  757,       0,  480851,       0,  0, 14,  0, }, // Galapagos
    {  399,  767,       0,  480865,       0,  0, 12,  0, }, // Gambier
    {  399,  775,  480877,  480891,  480911, 14, 20, 20, }, // Georgia
    {  399,  783,       0,  480931,       0,  0, 20,  0, }, // Gilbert_Islands
    {  399,  799,       0,  480951,       0,  0, 22,  0, }, // GMT
    {  399,  841,  480973,  480989,  481011, 16, 22, 22, }, // Greenland_Eastern
    {  399,  859,  481033,  481050,  481073, 17, 23, 23, }, // Greenland_Western
    {  399,  877,       0,  481096,       0,  0,  9,  0, }, // Guam
    {  399,  882,       0,  481105,       0,  0,  9,  0, }, // Gulf
    {  399,  887,       0,  481114,       0,  0, 11,  0, }, // Guyana
    {  399,  894,  481125,  481144,  481169, 19, 25, 25, }, // Hawaii_Aleutian
    {  399,  910,  481194,  481208,  481228, 14, 20, 20, }, // Hong_Kong
    {  399,  920,  481248,  481257,  481272,  9, 15, 15, }, // Hovd
    {  399,  925,       0,  481287,       0,  0, 12,  0, }, // India
    {  399,  931,       0,  481299,       0,  0, 20,  0, }, // Indian_Ocean
    {  399,  944,       0,  481319,       0,  0, 14,  0, }, // Indochina
    {  399,  954,       0,  481333,       0,  0, 23,  0, }, // Indonesia_Central
    {  399,  972,       0,  481356,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  399,  990,       0,  481375,       0,  0, 20,  0, }, // Indonesia_Western
    {  399, 1008,  481395,  481408,  481427, 13, 19, 19, }, // Iran
    {  399, 1019,  481446,  481458,  481476, 12, 18, 18, }, // Irkutsk
    {  399, 1027,  481494,  481509,  481530, 15, 21, 21, }, // Israel
    {  399, 1034,  481551,  481565,  481585, 14, 20, 20, }, // Japan
    {  399, 1040,  481605,  481621,  481643, 16, 22, 22, }, // Kamchatka
    {  399, 1058,       0,  481665,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  399, 1077,       0,  481683,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  399, 1106,  481702,  481717,  481738, 15, 21, 21, }, // Korea
    {  399, 1112,       0,  481759,       0,  0, 11,  0, }, // Kosrae
    {  399, 1119,  481770,  481786,  481808, 16, 22, 22, }, // Krasnoyarsk
    {  399, 1151,       0,  481830,       0,  0, 16,  0, }, // Kyrgystan
    {  399, 1175,       0,  481846,       0,  0, 18,  0, }, // Line_Islands
    {  399, 1188,  481864,  481878,  481898, 14, 20, 20, }, // Lord_Howe
    {  399, 1204,  481918,  481930,  481948, 12, 18, 18, }, // Magadan
    {  399, 1219,       0,  481966,       0,  0, 15,  0, }, // Malaysia
    {  399, 1228,       0,  481981,       0,  0, 13,  0, }, // Maldives
    {  399, 1237,       0,  481994,       0,  0, 14,  0, }, // Marquesas
    {  399, 1247,       0,  482008,       0,  0, 20,  0, }, // Marshall_Islands
    {  399, 1264,  482028,  482042,  482062, 14, 20, 20, }, // Mauritius
    {  399, 1274,       0,  482082,       0,  0, 11,  0, }, // Mawson
    {  399, 1281,  482093,  482116,  482146, 23, 30, 30, }, // Mexico_Pacific
    {  399, 1296,  482176,  482192,  482214, 16, 22, 22, }, // Mongolia
    {  399, 1305,  482236,  482249,  482268, 13, 19, 19, }, // Moscow
    {  399, 1312,       0,  482287,       0,  0, 12,  0, }, // Myanmar
    {  399, 1320,       0,  482299,       0,  0, 10,  0, }, // Nauru
    {  399, 1326,       0,  482309,       0,  0, 16,  0, }, // Nepal
    {  399, 1332,  482325,  482344,  482369, 19, 25, 25, }, // New_Caledonia
    {  399, 1346,  482394,  482409,  482430, 15, 21, 21, }, // New_Zealand
    {  399, 1358,  482451,  482467,  482489, 16, 22, 22, }, // Newfoundland
    {  399, 1371,       0,  482511,       0,  0,  9,  0, }, // Niue
    {  399, 1376,  482520,  482539,  482564, 19, 25, 25, }, // Norfolk
    {  399, 1384,  482589,  482613,  482643, 24, 30, 30, }, // Noronha
    {  399, 1406,  482673,  482689,  482711, 16, 22, 22, }, // Novosibirsk
    {  399, 1418,  482733,  482742,  482757,  9, 15, 15, }, // Omsk
    {  399, 1428,  482772,  482789,  482812, 17, 23, 23, }, // Pakistan
    {  399, 1437,       0,  482835,       0,  0, 10,  0, }, // Palau
    {  399, 1443,       0,  482845,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  399, 1460,  482865,  482884,  482909, 19, 25, 25, }, // Paraguay
    {  399, 1469,  482934,  482949,  482970, 15, 21, 21, }, // Peru
    {  399, 1474,  482991,  483010,  483035, 19, 25, 25, }, // Philippines
    {  399, 1486,       0,  483060,       0,  0, 19,  0, }, // Phoenix_Islands
    {  399, 1502,  483079,  483107,  483141, 28, 34, 34, }, // Pierre_Miquelon
    {  399, 1518,       0,  483175,       0,  0, 20,  0, }, // Pitcairn
    {  399, 1527,       0,  483195,       0,  0, 11,  0, }, // Ponape
    {  399, 1554,       0,  483206,       0,  0, 12,  0, }, // Reunion
    {  399, 1562,       0,  483218,       0,  0, 12,  0, }, // Rothera
    {  399, 1570,  483230,  483243,  483262, 13, 19, 19, }, // Sakhalin
    {  399, 1579,  483281,  483292,  483309, 11, 17, 17, }, // Samara
    {  399, 1596,  483326,  483336,  483352, 10, 16, 16, }, // Samoa
    {  399, 1602,       0,  483368,       0,  0, 15,  0, }, // Seychelles
    {  399, 1624,       0,  483383,       0,  0, 21,  0, }, // Singapore
    {  399, 1634,       0,  483404,       0,  0, 19,  0, }, // Solomon
    {  399, 1642,       0,  483423,       0,  0, 17,  0, }, // South_Georgia
    {  399, 1656,       0,  483440,       0,  0, 13,  0, }, // Suriname
    {  399, 1676,       0,  483453,       0,  0, 10,  0, }, // Syowa
    {  399, 1682,       0,  483463,       0,  0, 11,  0, }, // Tahiti
    {  399, 1689,  483474,  483485,  483502, 11, 17, 17, }, // Taipei
    {  399, 1696,       0,  483519,       0,  0, 18,  0, }, // Tajikistan
    {  399, 1724,       0,  483537,       0,  0, 12,  0, }, // Tokelau
    {  399, 1732,  483549,  483564,  483585, 15, 21, 21, }, // Tonga
    {  399, 1738,       0,  483606,       0,  0, 10,  0, }, // Truk
    {  399, 1750,  483616,  483633,  483656, 17, 23, 23, }, // Turkmenistan
    {  399, 1763,       0,  483679,       0,  0, 11,  0, }, // Tuvalu
    {  399, 1777,  483690,  483708,  483731, 18, 23, 24, }, // Uruguay
    {  399, 1792,  483755,  483770,  483791, 15, 21, 21, }, // Uzbekistan
    {  399, 1803,  483812,  483824,  483842, 12, 18, 18, }, // Vanuatu
    {  399, 1811,       0,  483860,       0,  0, 14,  0, }, // Venezuela
    {  399, 1821,  483874,  483890,  483912, 16, 22, 22, }, // Vladivostok
    {  399, 1833,  483934,  483948,  483968, 14, 20, 20, }, // Volgograd
    {  399, 1843,       0,  483988,       0,  0, 11,  0, }, // Vostok
    {  399, 1850,       0,  483999,       0,  0, 15,  0, }, // Wake
    {  399, 1855,       0,  484014,       0,  0, 20,  0, }, // Wallis
    {  399, 1862,  484034,  484046,  484064, 12, 18, 18, }, // Yakutsk
    {  399, 1870,  484082,  484101,  484126, 19, 25, 25, }, // Yekaterinburg
    {  401,    0,  101157,  101167,  484151, 10, 21, 29, }, // Acre Macedonian/Cyrillic/Macedonia
    {  401,    5,       0,  484180,       0,  0, 19,  0, }, // Afghanistan
    {  401,   17,       0,  484199,       0,  0, 22,  0, }, // Africa_Central
    {  401,   32,       0,  484221,       0,  0, 23,  0, }, // Africa_Eastern
    {  401,   65,       0,  484244,       0,  0, 34,  0, }, // Africa_Southern
    {  401,   81,  114926,  484278,  484312, 23, 34, 42, }, // Africa_Western
    {  401,  107,  484354,  484370,  484397, 16, 27, 35, }, // Alaska
    {  401,  135,  484432,  484447,  484473, 15, 26, 34, }, // Amazon
    {  401,  142,  484507,  484541,  484586, 34, 45, 53, }, // America_Central
    {  401,  158,  484639,  484671,  484714, 32, 43, 51, }, // America_Eastern
    {  401,  174,  484765,  484799,  484844, 34, 45, 53, }, // America_Mountain
    {  401,  191,  484897,  484931,  484976, 34, 45, 53, }, // America_Pacific
    {  401,  207,  485029,  485044,  485070, 15, 26, 21, }, // Anadyr
    {  401,  214,  485091,  485105,  485130, 14, 25, 20, }, // Apia
    {  401,  232,  485150,  485163,  485187, 13, 24, 32, }, // Arabian
    {  401,  240,  485219,  485237,  485266, 18, 29, 37, }, // Argentina
    {  401,  250,  485303,  485329,  485366, 26, 37, 45, }, // Argentina_Western
    {  401,  268,  485411,  485428,  485456, 17, 28, 23, }, // Armenia
    {  401,  286,  485479,  485494,  485520, 15, 26, 34, }, // Atlantic
    {  401,  295,  485554,  485583,  485623, 29, 40, 48, }, // Australia_Central
    {  401,  313,  485671,  485710,  485760, 39, 50, 58, }, // Australia_CentralWestern
    {  401,  338,  485818,  485845,  485883, 27, 38, 46, }, // Australia_Eastern
    {  401,  356,  485929,  485956,  485994, 27, 38, 46, }, // Australia_Western
    {  401,  374,  486040,  486059,  486089, 19, 30, 25, }, // Azerbaijan
    {  401,  385,  486114,  486138,  486173, 24, 35, 30, }, // Azores
    {  401,  397,  486203,  486221,  486250, 18, 29, 24, }, // Bangladesh
    {  401,  415,       0,  486274,       0,  0, 14,  0, }, // Bhutan
    {  401,  422,       0,  486288,       0,  0, 17,  0, }, // Bolivia
    {  401,  437,  486305,  486323,  486352, 18, 29, 37, }, // Brasilia
    {  401,  454,       0,  486389,       0,  0, 25,  0, }, // Brunei
    {  401,  461,  486414,  486433,  486463, 19, 30, 38, }, // Cape_Verde
    {  401,  478,       0,  486501,       0,  0, 15,  0, }, // Chamorro
    {  401,  487,  486516,  486530,  486555, 14, 25, 33, }, // Chatham
    {  401,  495,  486588,  486601,  486625, 13, 24, 32, }, // Chile
    {  401,  501,  486657,  486670,  486694, 13, 24, 32, }, // China
    {  401,  507,  486726,  486744,  486773, 18, 29, 24, }, // Choibalsan
    {  401,  518,       0,  486797,       0,  0, 23,  0, }, // Christmas
    {  401,  528,       0,  486820,       0,  0, 25,  0, }, // Cocos
    {  401,  534,  486845,  486863,  486892, 18, 29, 37, }, // Colombia
    {  401,  543,  486929,  486952,  486986, 23, 34, 29, }, // Cook
    {  401,  548,  487015,  487028,  487052, 13, 24, 32, }, // Cuba
    {  401,  559,       0,  487084,       0,  0, 15,  0, }, // Davis
    {  401,  575,       0,  487099,       0,  0, 21,  0, }, // DumontDUrville
    {  401,  612,       0,  487120,       0,  0, 22,  0, }, // East_Timor
    {  401,  623,  487142,  487169,  487207, 27, 38, 33, }, // Easter
    {  401,  630,       0,  487240,       0,  0, 16,  0, }, // Ecuador
    {  401,  638,  487256,  487276,  487307, 20, 31, 26, }, // Europe_Central
    {  401,  653,  487333,  487354,  487386, 21, 32, 27, }, // Europe_Eastern
    {  401,  668,       0,  487413,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  401,  691,  487433,  487454,  487486, 21, 32, 27, }, // Europe_Western
    {  401,  706,  487513,  487541,  487580, 28, 39, 47, }, // Falkland
    {  401,  715,  487627,  487640,  487664, 13, 24, 19, }, // Fiji
    {  401,  720,       0,  487683,       0,  0, 26,  0, }, // French_Guiana
    {  401,  734,       0,  487709,       0,  0, 35,  0, }, // French_Southern
    {  401,  757,       0,  487744,       0,  0, 18,  0, }, // Galapagos
    {  401,  767,       0,  487762,       0,  0, 14,  0, }, // Gambier
    {  401,  775,  487776,  487792,  487819, 16, 27, 22, }, // Georgia
    {  401,  783,       0,  487841,       0,  0, 27,  0, }, // Gilbert_Islands
    {  401,  799,       0,  487868,       0,  0, 22,  0, }, // GMT
    {  401,  841,  487890,  487915,  487951, 25, 36, 44, }, // Greenland_Eastern
    {  401,  859,  487995,  488020,  488056, 25, 36, 44, }, // Greenland_Western
    {  401,  882,       0,  488100,       0,  0, 23,  0, }, // Gulf
    {  401,  887,       0,  488123,       0,  0, 16,  0, }, // Guyana
    {  401,  894,  488139,  488172,  488216, 33, 44, 52, }, // Hawaii_Aleutian
    {  401,  910,  488268,  488286,  488315, 18, 29, 24, }, // Hong_Kong
    {  401,  920,  488339,  488352,  488376, 13, 24, 19, }, // Hovd
    {  401,  925,       0,  488395,       0,  0, 15,  0, }, // India
    {  401,  931,       0,  488410,       0,  0, 22,  0, }, // Indian_Ocean
    {  401,  944,       0,  488432,       0,  0, 17,  0, }, // Indochina
    {  401,  954,       0,  488449,       0,  0, 29,  0, }, // Indonesia_Central
    {  401,  972,       0,  488478,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  401,  990,       0,  488505,       0,  0, 27,  0, }, // Indonesia_Western
    {  401, 1008,  488532,  488545,  488569, 13, 24, 32, }, // Iran
    {  401, 1019,  488601,  488617,  488644, 16, 27, 22, }, // Irkutsk
    {  401, 1027,  488666,  488681,  488707, 15, 26, 34, }, // Israel
    {  401, 1034,  488741,  488758,  488786, 17, 28, 36, }, // Japan
    {  401, 1058,       0,  488822,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  401, 1077,       0,  488848,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  401, 1106,  488874,  488889,  488915, 15, 26, 34, }, // Korea
    {  401, 1112,       0,  488949,       0,  0, 14,  0, }, // Kosrae
    {  401, 1119,  488963,  488983,  489014, 20, 31, 26, }, // Krasnoyarsk
    {  401, 1151,       0,  489040,       0,  0, 18,  0, }, // Kyrgystan
    {  401, 1175,       0,  489058,       0,  0, 24,  0, }, // Line_Islands
    {  401, 1188,  489082,  489099,  489127, 17, 28, 36, }, // Lord_Howe
    {  401, 1204,  489163,  489179,  489206, 16, 27, 22, }, // Magadan
    {  401, 1219,       0,  489228,       0,  0, 17,  0, }, // Malaysia
    {  401, 1228,       0,  489245,       0,  0, 16,  0, }, // Maldives
    {  401, 1237,       0,  489261,       0,  0, 17,  0, }, // Marquesas
    {  401, 1247,       0,  489278,       0,  0, 26,  0, }, // Marshall_Islands
    {  401, 1264,  489304,  489322,  489351, 18, 29, 37, }, // Mauritius
    {  401, 1274,       0,  489388,       0,  0, 14,  0, }, // Mawson
    {  401, 1281,  489402,  489428,  489465, 26, 37, 32, }, // Mexico_Pacific
    {  401, 1296,  489497,  489516,  489546, 19, 30, 25, }, // Mongolia
    {  401, 1305,  489571,  489586,  489612, 15, 26, 34, }, // Moscow
    {  401, 1312,       0,  489646,       0,  0, 16,  0, }, // Myanmar
    {  401, 1320,       0,  489662,       0,  0, 14,  0, }, // Nauru
    {  401, 1326,       0,  489676,       0,  0, 14,  0, }, // Nepal
    {  401, 1332,  489690,  489714,  489749, 24, 35, 30, }, // New_Caledonia
    {  401, 1346,  489779,  489798,  489828, 19, 30, 38, }, // New_Zealand
    {  401, 1358,  489866,  489886,  489917, 20, 31, 39, }, // Newfoundland
    {  401, 1371,       0,  489956,       0,  0, 13,  0, }, // Niue
    {  401, 1376,  489969,  489994,  490030, 25, 36, 44, }, // Norfolk
    {  401, 1384,  490074,  490101,  490139, 27, 38, 46, }, // Noronha
    {  401, 1406,  490185,  490205,  490236, 20, 31, 26, }, // Novosibirsk
    {  401, 1418,  490262,  490275,  490299, 13, 24, 19, }, // Omsk
    {  401, 1428,  490318,  490335,  490363, 17, 28, 23, }, // Pakistan
    {  401, 1437,       0,  490386,       0,  0, 14,  0, }, // Palau
    {  401, 1443,       0,  490400,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  401, 1460,  490427,  490444,  490472, 17, 28, 36, }, // Paraguay
    {  401, 1469,  490508,  490521,  490545, 13, 24, 32, }, // Peru
    {  401, 1474,  490577,  490594,  490622, 17, 28, 23, }, // Philippines
    {  401, 1486,       0,  490645,       0,  0, 25,  0, }, // Phoenix_Islands
    {  401, 1502,  490670,  490698,  490737, 28, 39, 47, }, // Pierre_Miquelon
    {  401, 1518,       0,  490784,       0,  0, 16,  0, }, // Pitcairn
    {  401, 1527,       0,  490800,       0,  0, 15,  0, }, // Ponape
    {  401, 1534,       0,  490815,       0,  0, 18,  0, }, // Pyongyang
    {  401, 1554,       0,  490833,       0,  0, 17,  0, }, // Reunion
    {  401, 1562,       0,  490850,       0,  0, 15,  0, }, // Rothera
    {  401, 1570,  490865,  490881,  490908, 16, 27, 22, }, // Sakhalin
    {  401, 1579,  108155,  108167,  490930, 12, 23, 31, }, // Samara
    {  401, 1596,  490961,  490975,  491000, 14, 25, 20, }, // Samoa
    {  401, 1602,       0,  491020,       0,  0, 16,  0, }, // Seychelles
    {  401, 1624,       0,  491036,       0,  0, 17,  0, }, // Singapore
    {  401, 1634,       0,  491053,       0,  0, 27,  0, }, // Solomon
    {  401, 1642,       0,  491080,       0,  0, 22,  0, }, // South_Georgia
    {  401, 1656,       0,  491102,       0,  0, 16,  0, }, // Suriname
    {  401, 1676,       0,  491118,       0,  0, 15,  0, }, // Syowa
    {  401, 1682,       0,  491133,       0,  0, 15,  0, }, // Tahiti
    {  401, 1689,  491148,  491163,  491189, 15, 26, 34, }, // Taipei
    {  401, 1696,       0,  491223,       0,  0, 19,  0, }, // Tajikistan
    {  401, 1724,       0,  491242,       0,  0, 16,  0, }, // Tokelau
    {  401, 1732,  491258,  491272,  491297, 14, 25, 20, }, // Tonga
    {  401, 1738,       0,  491317,       0,  0, 13,  0, }, // Truk
    {  401, 1750,  491330,  491351,  491383, 21, 32, 27, }, // Turkmenistan
    {  401, 1763,       0,  491410,       0,  0, 15,  0, }, // Tuvalu
    {  401, 1777,  491425,  491441,  491468, 16, 27, 35, }, // Uruguay
    {  401, 1792,  491503,  491522,  491552, 19, 30, 25, }, // Uzbekistan
    {  401, 1803,  491577,  491593,  491620, 16, 27, 22, }, // Vanuatu
    {  401, 1811,       0,  491642,       0,  0, 18,  0, }, // Venezuela
    {  401, 1821,  491660,  491680,  491711, 20, 31, 26, }, // Vladivostok
    {  401, 1833,  491737,  491755,  491784, 18, 29, 37, }, // Volgograd
    {  401, 1843,       0,  491821,       0,  0, 15,  0, }, // Vostok
    {  401, 1850,       0,  491836,       0,  0, 22,  0, }, // Wake
    {  401, 1855,       0,  491858,       0,  0, 23,  0, }, // Wallis
    {  401, 1862,  491881,  491897,  491924, 16, 27, 22, }, // Yakutsk
    {  401, 1870,  491946,  491967,  491999, 21, 32, 27, }, // Yekaterinburg
    {  401, 1892,       0,  492026,       0,  0, 14,  0, }, // Yukon
    {  403,    5,       0,  492040,       0,  0, 16,  0, }, // Afghanistan Maithili/Devanagari/India
    {  403,   17,       0,  492056,       0,  0, 17,  0, }, // Africa_Central
    {  403,   32,       0,  492073,       0,  0, 17,  0, }, // Africa_Eastern
    {  403,   65,       0,  492090,       0,  0, 24,  0, }, // Africa_Southern
    {  403,   81,  492114,  492133,  492157, 19, 24, 21, }, // Africa_Western
    {  403,  107,  330273,  492178,  492194, 11, 16, 18, }, // Alaska
    {  403,  135,  492212,  492222,  492237, 10, 15, 14, }, // Amazon
    {  403,  142,  330368,  330395,  330427, 27, 32, 34, }, // America_Central
    {  403,  158,  330461,  330486,  330516, 25, 30, 32, }, // America_Eastern
    {  403,  174,  330548,  330574,  330605, 26, 31, 33, }, // America_Mountain
    {  403,  191,  330638,  330664,  330695, 26, 31, 33, }, // America_Pacific
    {  403,  232,  492251,  492263,  492280, 12, 17, 19, }, // Arabian
    {  403,  240,  492299,  492314,  492334, 15, 20, 19, }, // Argentina
    {  403,  250,  492353,  492376,  492404, 23, 28, 27, }, // Argentina_Western
    {  403,  268,  492431,  492445,  492464, 14, 19, 18, }, // Armenia
    {  403,  286,  331049,  331061,  331078, 12, 17, 19, }, // Atlantic
    {  403,  295,  492482,  492503,  492530, 21, 27, 29, }, // Australia_Central
    {  403,  313,  492559,  492589,  492624, 30, 35, 37, }, // Australia_CentralWestern
    {  403,  338,  492661,  492683,  492711, 22, 28, 30, }, // Australia_Eastern
    {  403,  356,  492741,  492765,  492795, 24, 30, 32, }, // Australia_Western
    {  403,  374,  492827,  492840,  492858, 13, 18, 17, }, // Azerbaijan
    {  403,  385,  492875,  492886,  492902, 11, 16, 15, }, // Azores
    {  403,  397,  492917,  492930,  492948, 13, 18, 17, }, // Bangladesh
    {  403,  415,       0,  492965,       0,  0, 10,  0, }, // Bhutan
    {  403,  422,       0,  492975,       0,  0, 13,  0, }, // Bolivia
    {  403,  437,  492988,  493003,  493023, 15, 20, 19, }, // Brasilia
    {  403,  454,       0,  493042,       0,  0, 21,  0, }, // Brunei
    {  403,  461,  493063,  493077,  493095, 14, 18, 17, }, // Cape_Verde
    {  403,  487,  493112,  493121,  493135,  9, 14, 16, }, // Chatham
    {  403,  495,  493151,  493160,  493174,  9, 14, 13, }, // Chile
    {  403,  501,  493187,  493196,  493210,  9, 14, 16, }, // China
    {  403,  507,  493226,  493239,  493257, 13, 18, 17, }, // Choibalsan
    {  403,  518,       0,  493274,       0,  0, 18,  0, }, // Christmas
    {  403,  528,       0,  493292,       0,  0, 16,  0, }, // Cocos
    {  403,  534,  493308,  493321,  493339, 13, 18, 17, }, // Colombia
    {  403,  548,  493356,  493367,  493383, 11, 16, 18, }, // Cuba
    {  403,  612,       0,  493401,       0,  0, 16,  0, }, // East_Timor
    {  403,  623,  493417,  493434,  493456, 17, 22, 21, }, // Easter
    {  403,  630,       0,  493477,       0,  0, 13,  0, }, // Ecuador
    {  403,  638,  332302,  332318,  332339, 16, 21, 31, }, // Europe_Central
    {  403,  653,  332370,  332388,  332411, 18, 23, 31, }, // Europe_Eastern
    {  403,  668,       0,  493490,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  403,  691,  332465,  332484,  332508, 19, 24, 33, }, // Europe_Western
    {  403,  706,  493516,  493535,  493559, 19, 24, 23, }, // Falkland
    {  403,  720,       0,  493582,       0,  0, 18,  0, }, // French_Guiana
    {  403,  734,       0,  493600,       0,  0, 44,  0, }, // French_Southern
    {  403,  757,       0,  493644,       0,  0, 14,  0, }, // Galapagos
    {  403,  775,  493658,  493671,  493689, 13, 18, 17, }, // Georgia
    {  403,  799,       0,   86447,       0,  0, 17,  0, }, // GMT
    {  403,  841,  493706,  493726,  493751, 20, 25, 24, }, // Greenland_Eastern
    {  403,  859,  493775,  493797,  493824, 22, 27, 26, }, // Greenland_Western
    {  403,  882,       0,  493850,       0,  0, 13,  0, }, // Gulf
    {  403,  887,       0,  493589,       0,  0, 11,  0, }, // Guyana
    {  403,  894,  493863,  493882,  493906, 19, 24, 26, }, // Hawaii_Aleutian
    {  403,  910,  493932,  493946,  493965, 14, 19, 18, }, // Hong_Kong
    {  403,  920,  493983,  493993,  494008, 10, 15, 14, }, // Hovd
    {  403,  925,       0,  333159,       0,  0, 15,  0, }, // India
    {  403,  931,       0,  494022,       0,  0, 17,  0, }, // Indian_Ocean
    {  403,  944,       0,  494039,       0,  0, 14,  0, }, // Indochina
    {  403,  954,       0,  494053,       0,  0, 20,  0, }, // Indonesia_Central
    {  403,  972,       0,  494073,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  403,  990,       0,  494094,       0,  0, 23,  0, }, // Indonesia_Western
    {  403, 1008,  494117,  494126,  494139,  9, 13, 16, }, // Iran
    {  403, 1019,  494155,  494169,  494188, 14, 19, 18, }, // Irkutsk
    {  403, 1027,  494206,  494217,  494233, 11, 16, 18, }, // Israel
    {  403, 1034,  494251,  494261,  494276, 10, 15, 17, }, // Japan
    {  403, 1058,       0,  494293,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  403, 1077,       0,  494312,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  403, 1106,  494333,  494344,  494360, 11, 16, 18, }, // Korea
    {  403, 1119,  494378,  494393,  494418, 15, 25, 24, }, // Krasnoyarsk
    {  403, 1151,       0,  494442,       0,  0, 11,  0, }, // Kyrgystan
    {  403, 1188,  494453,  494468,  494488, 15, 20, 22, }, // Lord_Howe
    {  403, 1204,  494510,  494521,  494537, 11, 16, 15, }, // Magadan
    {  403, 1219,       0,  494552,       0,  0, 12,  0, }, // Malaysia
    {  403, 1228,       0,  494564,       0,  0, 11,  0, }, // Maldives
    {  403, 1264,  494575,  494586,  494602, 11, 16, 15, }, // Mauritius
    {  403, 1281,  494617,  494638,  494664, 21, 26, 28, }, // Mexico_Pacific
    {  403, 1296,  494692,  494704,  494721, 12, 17, 16, }, // Mongolia
    {  403, 1305,  494737,  494748,  494764, 11, 16, 15, }, // Moscow
    {  403, 1312,       0,  494779,       0,  0, 11,  0, }, // Myanmar
    {  403, 1326,       0,  494790,       0,  0, 10,  0, }, // Nepal
    {  403, 1332,  494800,  494819,  494843, 19, 24, 26, }, // New_Caledonia
    {  403, 1346,  494869,  494884,  494904, 15, 20, 22, }, // New_Zealand
    {  403, 1358,  494926,  494944,  494967, 18, 23, 25, }, // Newfoundland
    {  403, 1376,  494992,  495010,  495033, 18, 23, 25, }, // Norfolk
    {  403, 1384,  495058,  495083,  495113, 25, 30, 29, }, // Noronha
    {  403, 1406,  495142,  495160,  495183, 18, 23, 22, }, // Novosibirsk
    {  403, 1418,  495205,  495216,  495232, 11, 16, 15, }, // Omsk
    {  403, 1428,  495247,  495261,  495280, 14, 19, 18, }, // Pakistan
    {  403, 1460,  495298,  495310,  495327, 12, 17, 16, }, // Paraguay
    {  403, 1469,  495343,  495352,  495366,  9, 14, 13, }, // Peru
    {  403, 1474,  495379,  495391,  495408, 12, 17, 16, }, // Philippines
    {  403, 1502,  495424,  495451,  495483, 27, 32, 34, }, // Pierre_Miquelon
    {  403, 1534,       0,  495517,       0,  0, 15,  0, }, // Pyongyang
    {  403, 1554,       0,  495532,       0,  0, 13,  0, }, // Reunion
    {  403, 1570,  495545,  495556,  495572, 11, 16, 15, }, // Sakhalin
    {  403, 1596,  495587,  495596,  495610,  9, 14, 13, }, // Samoa
    {  403, 1602,       0,  495623,       0,  0, 12,  0, }, // Seychelles
    {  403, 1624,       0,  495635,       0,  0, 18,  0, }, // Singapore
    {  403, 1642,       0,  495653,       0,  0, 20,  0, }, // South_Georgia
    {  403, 1656,       0,  495673,       0,  0, 12,  0, }, // Suriname
    {  403, 1689,  495685,  495696,  495712, 11, 16, 18, }, // Taipei
    {  403, 1696,       0,   91955,       0,  0, 11,  0, }, // Tajikistan
    {  403, 1750,  495730,  495749,  495773, 19, 24, 23, }, // Turkmenistan
    {  403, 1777,  495796,  495808,  495825, 12, 17, 16, }, // Uruguay
    {  403, 1792,  495841,  495857,  495878, 16, 21, 20, }, // Uzbekistan
    {  403, 1811,       0,  495898,       0,  0, 14,  0, }, // Venezuela
    {  403, 1821,  495912,  495930,  495953, 18, 23, 22, }, // Vladivostok
    {  403, 1833,  495975,  495991,  496012, 16, 21, 20, }, // Volgograd
    {  403, 1862,  496032,  496045,  496063, 13, 18, 17, }, // Yakutsk
    {  403, 1870,  496080,  496098,  496121, 18, 23, 22, }, // Yekaterinburg
    {  403, 1892,       0,  496143,       0,  0, 10,  0, }, // Yukon
    {  407,    0,  496153,  496165,  496190, 12, 25, 22, }, // Acre Malayalam/Malayalam/India
    {  407,    5,       0,  496212,       0,  0, 18,  0, }, // Afghanistan
    {  407,   17,       0,  496230,       0,  0, 18,  0, }, // Africa_Central
    {  407,   32,       0,  496248,       0,  0, 21,  0, }, // Africa_Eastern
    {  407,   65,       0,  496269,       0,  0, 32,  0, }, // Africa_Southern
    {  407,   81,  496301,  496324,  496360, 23, 36, 35, }, // Africa_Western
    {  407,  107,  496395,  496407,  496431, 12, 24, 21, }, // Alaska
    {  407,  128,  496452,  496462,  496485, 10, 23, 20, }, // Almaty
    {  407,  135,  496505,  496515,  496538, 10, 23, 22, }, // Amazon
    {  407,  142,  496560,  496589,  496631, 29, 42, 38, }, // America_Central
    {  407,  158,  496669,  496698,  496740, 29, 42, 38, }, // America_Eastern
    {  407,  174,  496778,  496806,  496847, 28, 41, 37, }, // America_Mountain
    {  407,  191,  496884,  496912,  496953, 28, 41, 37, }, // America_Pacific
    {  407,  207,  496990,  497001,  497025, 11, 24, 21, }, // Anadyr
    {  407,  214,  497046,  497055,  497077,  9, 22, 18, }, // Apia
    {  407,  219,  497095,  497105,  497128, 10, 23, 20, }, // Aqtau
    {  407,  225,  497148,  497160,  497185, 12, 25, 22, }, // Aqtobe
    {  407,  232,  497207,  497219,  497244, 12, 25, 21, }, // Arabian
    {  407,  240,  497265,  497278,  497304, 13, 26, 25, }, // Argentina
    {  407,  250,  497329,  497352,  497388, 23, 36, 35, }, // Argentina_Western
    {  407,  268,  497423,  497435,  497460, 12, 25, 24, }, // Armenia
    {  407,  286,  497484,  497503,  497535, 19, 32, 28, }, // Atlantic
    {  407,  295,  497563,  497586,  497623, 23, 37, 33, }, // Australia_Central
    {  407,  313,  497656,  497690,  497737, 34, 47, 43, }, // Australia_CentralWestern
    {  407,  338,  497780,  497804,  497842, 24, 38, 34, }, // Australia_Eastern
    {  407,  356,  497876,  497902,  497942, 26, 40, 36, }, // Australia_Western
    {  407,  374,  497978,  497991,  498017, 13, 26, 25, }, // Azerbaijan
    {  407,  385,  498042,  498053,  498077, 11, 24, 23, }, // Azores
    {  407,  397,  498100,  498115,  498143, 15, 28, 27, }, // Bangladesh
    {  407,  415,       0,  498170,       0,  0, 12,  0, }, // Bhutan
    {  407,  422,       0,  498182,       0,  0, 12,  0, }, // Bolivia
    {  407,  437,  498194,  498207,  498233, 13, 26, 25, }, // Brasilia
    {  407,  454,       0,  498258,       0,  0, 22,  0, }, // Brunei
    {  407,  461,  498280,  498295,  498323, 15, 28, 27, }, // Cape_Verde
    {  407,  478,       0,  498350,       0,  0, 23,  0, }, // Chamorro
    {  407,  487,  498373,  498384,  498408, 11, 24, 23, }, // Chatham
    {  407,  495,  498431,  498440,  498462,  9, 22, 21, }, // Chile
    {  407,  501,  498483,  498491,  498512,  8, 21, 17, }, // China
    {  407,  507,  498529,  498544,  498572, 15, 28, 26, }, // Choibalsan
    {  407,  518,       0,  498598,       0,  0, 22,  0, }, // Christmas
    {  407,  528,       0,  498620,       0,  0, 22,  0, }, // Cocos
    {  407,  534,  498642,  498654,  498679, 12, 25, 24, }, // Colombia
    {  407,  543,  498703,  498723,  498756, 20, 33, 38, }, // Cook
    {  407,  548,  498794,  498804,  498827, 10, 23, 19, }, // Cuba
    {  407,  559,       0,  498846,       0,  0, 11,  0, }, // Davis
    {  407,  575,       0,  498857,       0,  0, 25,  0, }, // DumontDUrville
    {  407,  612,       0,  498882,       0,  0, 18,  0, }, // East_Timor
    {  407,  623,  498900,  498919,  498951, 19, 32, 31, }, // Easter
    {  407,  630,       0,  498982,       0,  0, 12,  0, }, // Ecuador
    {  407,  638,  498994,  499015,  499049, 21, 34, 32, }, // Europe_Central
    {  407,  653,  499081,  499102,  499136, 21, 34, 32, }, // Europe_Eastern
    {  407,  668,       0,  499168,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  407,  691,  499197,  499220,  499256, 23, 36, 35, }, // Europe_Western
    {  407,  706,  499291,  499317,  499356, 26, 39, 38, }, // Falkland
    {  407,  715,  499394,  499403,  499425,  9, 22, 21, }, // Fiji
    {  407,  720,       0,  499446,       0,  0, 17,  0, }, // French_Guiana
    {  407,  734,       0,  499463,       0,  0, 31,  0, }, // French_Southern
    {  407,  757,       0,  499494,       0,  0, 15,  0, }, // Galapagos
    {  407,  767,       0,  499509,       0,  0, 13,  0, }, // Gambier
    {  407,  775,  499522,  499535,  499561, 13, 26, 25, }, // Georgia
    {  407,  783,       0,  499586,       0,  0, 24,  0, }, // Gilbert_Islands
    {  407,  799,       0,  499610,       0,  0, 20,  0, }, // GMT
    {  407,  841,  499630,  499653,  499689, 23, 36, 35, }, // Greenland_Eastern
    {  407,  859,  499724,  499749,  499787, 25, 38, 37, }, // Greenland_Western
    {  407,  877,       0,  499824,       0,  0, 23,  0, }, // Guam
    {  407,  882,       0,  499847,       0,  0, 22,  0, }, // Gulf
    {  407,  887,       0,  499454,       0,  0,  9,  0, }, // Guyana
    {  407,  894,  499869,  499887,  499918, 18, 31, 27, }, // Hawaii_Aleutian
    {  407,  910,  499945,  499960,  499988, 15, 28, 27, }, // Hong_Kong
    {  407,  920,  500015,  500024,  500046,  9, 22, 21, }, // Hovd
    {  407,  925,       0,  500067,       0,  0, 25,  0, }, // India
    {  407,  931,       0,  500092,       0,  0, 22,  0, }, // Indian_Ocean
    {  407,  944,       0,  500114,       0,  0, 12,  0, }, // Indochina
    {  407,  954,       0,  500126,       0,  0, 20,  0, }, // Indonesia_Central
    {  407,  972,       0,  500146,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  407,  990,       0,  500169,       0,  0, 25,  0, }, // Indonesia_Western
    {  407, 1008,  500194,  500203,  500225,  9, 22, 18, }, // Iran
    {  407, 1019,  500243,  500255,  500280, 12, 25, 25, }, // Irkutsk
    {  407, 1027,  500305,  500318,  500344, 13, 26, 22, }, // Israel
    {  407, 1034,  500366,  500377,  500401, 11, 24, 20, }, // Japan
    {  407, 1040,  500421,  500452,  500496, 31, 44, 41, }, // Kamchatka
    {  407, 1058,       0,  500537,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  407, 1077,       0,  500560,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  407, 1106,  500585,  500596,  500620, 11, 24, 20, }, // Korea
    {  407, 1112,       0,  500640,       0,  0, 10,  0, }, // Kosrae
    {  407, 1119,  500650,  500672,  500707, 22, 35, 34, }, // Krasnoyarsk
    {  407, 1151,       0,  500741,       0,  0, 15,  0, }, // Kyrgystan
    {  407, 1161,       0,  500756,       0,  0,  9,  0, }, // Lanka
    {  407, 1175,       0,  500765,       0,  0, 17,  0, }, // Line_Islands
    {  407, 1188,  500782,  500797,  500825, 15, 28, 24, }, // Lord_Howe
    {  407, 1198,  500849,  500857,  500878,  8, 21, 18, }, // Macau
    {  407, 1204,  500896,  500906,  500929, 10, 23, 22, }, // Magadan
    {  407, 1219,       0,  500951,       0,  0, 11,  0, }, // Malaysia
    {  407, 1228,       0,  500962,       0,  0, 15,  0, }, // Maldives
    {  407, 1237,       0,  500977,       0,  0, 13,  0, }, // Marquesas
    {  407, 1247,       0,  500990,       0,  0, 19,  0, }, // Marshall_Islands
    {  407, 1264,  501009,  501023,  501050, 14, 27, 26, }, // Mauritius
    {  407, 1274,       0,  501076,       0,  0,  9,  0, }, // Mawson
    {  407, 1281,  501085,  501107,  501143, 22, 36, 31, }, // Mexico_Pacific
    {  407, 1296,  501174,  501189,  501217, 15, 28, 27, }, // Mongolia
    {  407, 1305,  501244,  501255,  501279, 11, 24, 24, }, // Moscow
    {  407, 1312,       0,  501303,       0,  0, 13,  0, }, // Myanmar
    {  407, 1320,       0,  501316,       0,  0,  9,  0, }, // Nauru
    {  407, 1326,       0,  501325,       0,  0, 12,  0, }, // Nepal
    {  407, 1332,  501337,  501356,  501388, 19, 32, 31, }, // New_Caledonia
    {  407, 1346,  501419,  501435,  501464, 16, 29, 25, }, // New_Zealand
    {  407, 1358,  501489,  501511,  501546, 22, 35, 31, }, // Newfoundland
    {  407, 1371,       0,  501577,       0,  0, 11,  0, }, // Niue
    {  407, 1376,  501588,  501610,  501645, 22, 35, 31, }, // Norfolk
    {  407, 1384,  501676,  501698,  501733, 22, 35, 34, }, // Noronha
    {  407, 1392,       0,  501767,       0,  0, 29,  0, }, // North_Mariana
    {  407, 1406,  501796,  501815,  501846, 19, 31, 31, }, // Novosibirsk
    {  407, 1418,  501877,  501891,  501918, 14, 27, 26, }, // Omsk
    {  407, 1428,  501944,  501960,  501989, 16, 29, 28, }, // Pakistan
    {  407, 1437,       0,  502017,       0,  0, 10,  0, }, // Palau
    {  407, 1443,       0,  502027,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  407, 1460,  502050,  502062,  502087, 12, 25, 24, }, // Paraguay
    {  407, 1469,  502111,  502120,  502142,  9, 22, 21, }, // Peru
    {  407, 1474,  502163,  502177,  502204, 14, 27, 26, }, // Philippines
    {  407, 1486,       0,  502230,       0,  0, 21,  0, }, // Phoenix_Islands
    {  407, 1502,  502251,  502281,  502324, 30, 43, 39, }, // Pierre_Miquelon
    {  407, 1518,       0,  502363,       0,  0, 14,  0, }, // Pitcairn
    {  407, 1527,       0,  502377,       0,  0, 13,  0, }, // Ponape
    {  407, 1534,       0,  502390,       0,  0, 18,  0, }, // Pyongyang
    {  407, 1544,  502408,  502421,  502447, 13, 26, 23, }, // Qyzylorda
    {  407, 1554,       0,  502470,       0,  0, 13,  0, }, // Reunion
    {  407, 1562,       0,  502483,       0,  0, 10,  0, }, // Rothera
    {  407, 1570,  502493,  502504,  502528, 11, 24, 23, }, // Sakhalin
    {  407, 1579,  502551,  502560,  502582,  9, 22, 19, }, // Samara
    {  407, 1596,  502601,  502610,  502632,  9, 22, 22, }, // Samoa
    {  407, 1602,       0,  502654,       0,  0, 12,  0, }, // Seychelles
    {  407, 1624,       0,  502666,       0,  0, 27,  0, }, // Singapore
    {  407, 1634,       0,  502693,       0,  0, 19,  0, }, // Solomon
    {  407, 1642,       0,  502712,       0,  0, 21,  0, }, // South_Georgia
    {  407, 1656,       0,  502733,       0,  0, 14,  0, }, // Suriname
    {  407, 1676,       0,  502747,       0,  0,  9,  0, }, // Syowa
    {  407, 1682,       0,  502756,       0,  0, 11,  0, }, // Tahiti
    {  407, 1689,  502767,  502781,  502808, 14, 27, 23, }, // Taipei
    {  407, 1696,       0,  502831,       0,  0, 18,  0, }, // Tajikistan
    {  407, 1724,       0,  502849,       0,  0, 13,  0, }, // Tokelau
    {  407, 1732,  502862,  502871,  502893,  9, 22, 21, }, // Tonga
    {  407, 1738,       0,  502914,       0,  0, 11,  0, }, // Truk
    {  407, 1750,  502925,  502947,  502982, 22, 35, 34, }, // Turkmenistan
    {  407, 1763,       0,  503016,       0,  0, 11,  0, }, // Tuvalu
    {  407, 1777,  503027,  503039,  503064, 12, 25, 24, }, // Uruguay
    {  407, 1792,  503088,  503108,  503141, 20, 33, 32, }, // Uzbekistan
    {  407, 1803,  503173,  503185,  503210, 12, 25, 24, }, // Vanuatu
    {  407, 1811,       0,  503234,       0,  0, 14,  0, }, // Venezuela
    {  407, 1821,  503248,  503269,  503303, 21, 34, 33, }, // Vladivostok
    {  407, 1833,  503336,  503352,  503381, 16, 29, 28, }, // Volgograd
    {  407, 1843,       0,  503254,       0,  0, 15,  0, }, // Vostok
    {  407, 1850,       0,  503409,       0,  0, 18,  0, }, // Wake
    {  407, 1855,       0,  503427,       0,  0, 27,  0, }, // Wallis
    {  407, 1862,  503454,  503469,  503497, 15, 28, 27, }, // Yakutsk
    {  407, 1870,  503524,  503545,  503579, 21, 34, 33, }, // Yekaterinburg
    {  407, 1892,       0,  503612,       0,  0, 12,  0, }, // Yukon
    {  408,    0,  354391,  503624,  354419, 10, 17, 22, }, // Acre Malay/Latin/Malaysia
    {  408,    5,       0,  503641,       0,  0, 17,  0, }, // Afghanistan
    {  408,   17,       0,  354457,       0,  0, 19,  0, }, // Africa_Central
    {  408,   32,       0,  354476,       0,  0, 18,  0, }, // Africa_Eastern
    {  408,   65,       0,  503658,       0,  0, 27,  0, }, // Africa_Southern
    {  408,   81,  354522,  503685,  354566, 18, 25, 30, }, // Africa_Western
    {  408,  107,  354596,  503710,  503729, 12, 19, 18, }, // Alaska
    {  408,  128,  354652,  503747,  354684, 12, 19, 24, }, // Almaty
    {  408,  135,  354708,  503766,  354740, 12, 19, 24, }, // Amazon
    {  408,  142,  503785,  503796,  503814, 11, 18, 18, }, // America_Central
    {  408,  158,  354820,  503832,  503850, 11, 18, 17, }, // America_Eastern
    {  408,  174,  503867,  503884,  503908, 17, 24, 23, }, // America_Mountain
    {  408,  191,  354941,  503931,  503951, 13, 20, 19, }, // America_Pacific
    {  408,  207,  355000,  503970,  355032, 12, 19, 24, }, // Anadyr
    {  408,  214,  355056,  503989,  504006, 10, 17, 16, }, // Apia
    {  408,  219,  355106,  504022,  355136, 11, 20, 23, }, // Aqtau
    {  408,  225,  355159,  504042,  355191, 12, 19, 24, }, // Aqtobe
    {  408,  232,  355215,  504061,  504078, 10, 17, 16, }, // Arabian
    {  408,  240,  355265,  504094,  355303, 15, 22, 27, }, // Argentina
    {  408,  250,  504116,  504137,  504165, 21, 28, 33, }, // Argentina_Western
    {  408,  268,  355434,  504198,  355468, 13, 20, 25, }, // Armenia
    {  408,  286,  355493,  504218,  504239, 14, 21, 20, }, // Atlantic
    {  408,  295,  504259,  504281,  504310, 22, 29, 28, }, // Australia_Central
    {  408,  313,  355641,  504338,  504373, 28, 35, 34, }, // Australia_CentralWestern
    {  408,  338,  504407,  504428,  504456, 21, 28, 27, }, // Australia_Eastern
    {  408,  356,  504483,  504504,  504532, 21, 28, 27, }, // Australia_Western
    {  408,  374,  355911,  504559,  355951, 16, 23, 28, }, // Azerbaijan
    {  408,  385,  355979,  504582,  356011, 12, 19, 24, }, // Azores
    {  408,  397,  356035,  504601,  356075, 16, 23, 28, }, // Bangladesh
    {  408,  415,       0,  356103,       0,  0, 12,  0, }, // Bhutan
    {  408,  422,       0,  356115,       0,  0, 13,  0, }, // Bolivia
    {  408,  437,  504624,  504638,  504659, 14, 21, 26, }, // Brasilia
    {  408,  454,       0,  356184,       0,  0, 23,  0, }, // Brunei
    {  408,  461,  356207,  504685,  356253, 19, 26, 31, }, // Cape_Verde
    {  408,  478,       0,  504711,       0,  0, 21,  0, }, // Chamorro
    {  408,  487,  356317,  504732,  504752, 13, 20, 19, }, // Chatham
    {  408,  495,  504771,  504782,  504800, 11, 18, 23, }, // Chile
    {  408,  501,  504823,  504834,  504852, 11, 18, 17, }, // China
    {  408,  507,  356488,  504869,  356528, 16, 23, 28, }, // Choibalsan
    {  408,  518,       0,  504892,       0,  0, 21,  0, }, // Christmas
    {  408,  528,       0,  356573,       0,  0, 21,  0, }, // Cocos
    {  408,  534,  504913,  504927,  504948, 14, 21, 26, }, // Colombia
    {  408,  543,  504974,  504994,  505021, 20, 27, 40, }, // Cook
    {  408,  548,  505061,  505071,  505088, 10, 17, 16, }, // Cuba
    {  408,  559,       0,  356778,       0,  0, 11,  0, }, // Davis
    {  408,  575,       0,  356789,       0,  0, 22,  0, }, // DumontDUrville
    {  408,  612,       0,  505104,       0,  0, 17,  0, }, // East_Timor
    {  408,  623,  505121,  505139,  505164, 18, 25, 30, }, // Easter
    {  408,  630,       0,  505194,       0,  0, 13,  0, }, // Ecuador
    {  408,  638,  505207,  505226,  505252, 19, 26, 31, }, // Europe_Central
    {  408,  653,  505283,  505301,  505326, 18, 25, 30, }, // Europe_Eastern
    {  408,  668,       0,  505356,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  408,  691,  505380,  505398,  505423, 18, 25, 30, }, // Europe_Western
    {  408,  706,  357153,  505453,  357209, 24, 31, 36, }, // Falkland
    {  408,  715,  357245,  505484,  357273, 10, 17, 22, }, // Fiji
    {  408,  720,       0,  505501,       0,  0, 21,  0, }, // French_Guiana
    {  408,  734,       0,  505522,       0,  0, 36,  0, }, // French_Southern
    {  408,  757,       0,  357359,       0,  0, 15,  0, }, // Galapagos
    {  408,  767,       0,  357374,       0,  0, 13,  0, }, // Gambier
    {  408,  775,  357387,  505558,  357421, 13, 20, 25, }, // Georgia
    {  408,  783,       0,  505578,       0,  0, 23,  0, }, // Gilbert_Islands
    {  408,  799,       0,  505601,       0,  0, 19,  0, }, // GMT
    {  408,  841,  357464,  505620,  357514, 21, 28, 33, }, // Greenland_Eastern
    {  408,  859,  357547,  505648,  357597, 21, 28, 33, }, // Greenland_Western
    {  408,  882,       0,  505676,       0,  0, 18,  0, }, // Gulf
    {  408,  887,       0,  357295,       0,  0, 12,  0, }, // Guyana
    {  408,  894,  357659,  505694,  505722, 21, 28, 27, }, // Hawaii_Aleutian
    {  408,  910,  357742,  505749,  357780, 15, 22, 27, }, // Hong_Kong
    {  408,  920,  357807,  505771,  357835, 10, 17, 22, }, // Hovd
    {  408,  925,       0,  505788,       0,  0, 18,  0, }, // India
    {  408,  931,       0,  505806,       0,  0, 18,  0, }, // Indian_Ocean
    {  408,  944,       0,  357889,       0,  0, 15,  0, }, // Indochina
    {  408,  954,       0,  357904,       0,  0, 22,  0, }, // Indonesia_Central
    {  408,  972,       0,  357926,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  408,  990,       0,  357947,       0,  0, 21,  0, }, // Indonesia_Western
    {  408, 1008,  357968,  505824,  505841, 10, 17, 16, }, // Iran
    {  408, 1019,  358018,  505857,  358052, 13, 20, 25, }, // Irkutsk
    {  408, 1027,  358077,  505877,  505896, 12, 19, 18, }, // Israel
    {  408, 1034,  505914,  505925,  505943, 11, 18, 17, }, // Japan
    {  408, 1040,  505960,  505990,  358257, 30, 37, 42, }, // Kamchatka
    {  408, 1058,       0,  358299,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  408, 1077,       0,  358321,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  408, 1106,  358343,  506027,  506045, 11, 18, 17, }, // Korea
    {  408, 1112,       0,  358396,       0,  0, 12,  0, }, // Kosrae
    {  408, 1119,  358408,  506062,  358450, 17, 24, 29, }, // Krasnoyarsk
    {  408, 1151,       0,  506086,       0,  0, 15,  0, }, // Kyrgystan
    {  408, 1175,       0,  506101,       0,  0, 20,  0, }, // Line_Islands
    {  408, 1188,  358521,  506121,  506143, 15, 22, 21, }, // Lord_Howe
    {  408, 1198,  506164,  506175,  506193, 11, 18, 23, }, // Macau
    {  408, 1204,  358639,  506216,  358673, 13, 20, 25, }, // Magadan
    {  408, 1219,       0,  358698,       0,  0, 14,  0, }, // Malaysia
    {  408, 1228,       0,  506236,       0,  0, 14,  0, }, // Maldives
    {  408, 1237,       0,  358726,       0,  0, 15,  0, }, // Marquesas
    {  408, 1247,       0,  506250,       0,  0, 24,  0, }, // Marshall_Islands
    {  408, 1264,  358760,  506274,  358798, 15, 22, 27, }, // Mauritius
    {  408, 1274,       0,  358825,       0,  0, 12,  0, }, // Mawson
    {  408, 1281,  506296,  506316,  506343, 20, 27, 26, }, // Mexico_Pacific
    {  408, 1296,  358920,  506369,  358960, 16, 23, 28, }, // Mongolia
    {  408, 1305,  506392,  506404,  506423, 12, 19, 24, }, // Moscow
    {  408, 1312,       0,  359044,       0,  0, 13,  0, }, // Myanmar
    {  408, 1320,       0,  359057,       0,  0, 11,  0, }, // Nauru
    {  408, 1326,       0,  359068,       0,  0, 11,  0, }, // Nepal
    {  408, 1332,  506447,  506466,  506492, 19, 26, 31, }, // New_Caledonia
    {  408, 1346,  506523,  506540,  506564, 17, 24, 23, }, // New_Zealand
    {  408, 1358,  359236,  506587,  506612, 18, 25, 24, }, // Newfoundland
    {  408, 1371,       0,  359310,       0,  0, 10,  0, }, // Niue
    {  408, 1376,  506636,  506659,  506689, 23, 30, 29, }, // Norfolk
    {  408, 1384,  359397,  506718,  359455, 25, 32, 37, }, // Noronha
    {  408, 1392,       0,  506750,       0,  0, 29,  0, }, // North_Mariana
    {  408, 1406,  359516,  506779,  359558, 17, 24, 29, }, // Novosibirsk
    {  408, 1418,  359587,  506803,  359615, 10, 17, 22, }, // Omsk
    {  408, 1428,  359637,  506820,  359673, 14, 21, 26, }, // Pakistan
    {  408, 1437,       0,  359699,       0,  0, 11,  0, }, // Palau
    {  408, 1443,       0,  506841,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  408, 1460,  359728,  506863,  359764, 14, 21, 26, }, // Paraguay
    {  408, 1469,  359790,  506884,  359818, 10, 17, 22, }, // Peru
    {  408, 1474,  359840,  506901,  359876, 14, 21, 26, }, // Philippines
    {  408, 1486,       0,  359902,       0,  0, 23,  0, }, // Phoenix_Islands
    {  408, 1502,  359925,  506922,  506960, 31, 38, 37, }, // Pierre_Miquelon
    {  408, 1518,       0,  360038,       0,  0, 14,  0, }, // Pitcairn
    {  408, 1527,       0,  360052,       0,  0, 12,  0, }, // Ponape
    {  408, 1534,       0,  360064,       0,  0, 15,  0, }, // Pyongyang
    {  408, 1544,  360079,  506997,  360117, 15, 22, 27, }, // Qyzylorda
    {  408, 1554,       0,  360144,       0,  0, 13,  0, }, // Reunion
    {  408, 1562,       0,  360157,       0,  0, 13,  0, }, // Rothera
    {  408, 1570,  360170,  507019,  360206, 14, 21, 26, }, // Sakhalin
    {  408, 1579,  360232,  507040,  360264, 12, 19, 24, }, // Samara
    {  408, 1596,  360288,  507059,  360318, 11, 18, 23, }, // Samoa
    {  408, 1602,       0,  360341,       0,  0, 16,  0, }, // Seychelles
    {  408, 1624,       0,  507077,       0,  0, 22,  0, }, // Singapore
    {  408, 1634,       0,  360380,       0,  0, 23,  0, }, // Solomon
    {  408, 1642,       0,  360403,       0,  0, 21,  0, }, // South_Georgia
    {  408, 1656,       0,  360424,       0,  0, 14,  0, }, // Suriname
    {  408, 1676,       0,  360438,       0,  0, 11,  0, }, // Syowa
    {  408, 1682,       0,  360449,       0,  0, 12,  0, }, // Tahiti
    {  408, 1689,  360461,  507099,  507118, 12, 19, 18, }, // Taipei
    {  408, 1696,       0,  360517,       0,  0, 16,  0, }, // Tajikistan
    {  408, 1724,       0,  360533,       0,  0, 13,  0, }, // Tokelau
    {  408, 1732,  360546,  507136,  360576, 11, 18, 23, }, // Tonga
    {  408, 1738,       0,  360599,       0,  0, 11,  0, }, // Truk
    {  408, 1750,  360610,  507154,  360654, 18, 25, 30, }, // Turkmenistan
    {  408, 1763,       0,  360684,       0,  0, 12,  0, }, // Tuvalu
    {  408, 1777,  360696,  507179,  360730, 13, 20, 25, }, // Uruguay
    {  408, 1792,  360755,  507199,  360795, 16, 23, 28, }, // Uzbekistan
    {  408, 1803,  360823,  507222,  360857, 13, 20, 25, }, // Vanuatu
    {  408, 1811,       0,  360882,       0,  0, 15,  0, }, // Venezuela
    {  408, 1821,  360897,  507242,  360939, 17, 24, 29, }, // Vladivostok
    {  408, 1833,  360968,  507266,  361006, 15, 22, 27, }, // Volgograd
    {  408, 1843,       0,  361033,       0,  0, 12,  0, }, // Vostok
    {  408, 1850,       0,  507288,       0,  0, 16,  0, }, // Wake
    {  408, 1855,       0,  361065,       0,  0, 23,  0, }, // Wallis
    {  408, 1862,  361088,  507304,  361122, 13, 20, 25, }, // Yakutsk
    {  408, 1870,  361147,  507324,  361193, 19, 26, 31, }, // Yekaterinburg
    {  408, 1892,       0,  507350,       0,  0, 10,  0, }, // Yukon
    {  409,  454,       0,  507360,       0,  0, 20,  0, }, // Brunei Malay/Arabic/Brunei
    {  409,  925,       0,  507380,       0,  0, 18,  0, }, // India
    {  409,  931,       0,  507398,       0,  0, 17,  0, }, // Indian_Ocean
    {  409,  954,       0,  507415,       0,  0, 18,  0, }, // Indonesia_Central
    {  409,  972,       0,  507433,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  409,  990,       0,  507453,       0,  0, 20,  0, }, // Indonesia_Western
    {  409, 1219,       0,  507473,       0,  0, 11,  0, }, // Malaysia
    {  410,  454,       0,  507360,       0,  0, 20,  0, }, // Brunei Malay/Arabic/Malaysia
    {  410,  925,       0,  507380,       0,  0, 18,  0, }, // India
    {  410,  931,       0,  507398,       0,  0, 17,  0, }, // Indian_Ocean
    {  410,  954,       0,  507415,       0,  0, 18,  0, }, // Indonesia_Central
    {  410,  972,       0,  507433,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  410,  990,       0,  507453,       0,  0, 20,  0, }, // Indonesia_Western
    {  410, 1219,       0,  507473,       0,  0, 11,  0, }, // Malaysia
    {  414,  638,  507484,  507504,  507533, 20, 29, 29, }, // Europe_Central Maltese/Latin/Malta
    {  414,  799,       0,    2861,       0,  0, 19,  0, }, // GMT
    {  415,  142,  507562,  507591,  507631, 29, 40, 36, }, // America_Central Manipuri/Bangla/India
    {  415,  158,  507667,  507694,  507732, 27, 38, 34, }, // America_Eastern
    {  415,  174,  507766,  507794,  507833, 28, 39, 35, }, // America_Mountain
    {  415,  191,  507868,  507895,  507933, 27, 38, 34, }, // America_Pacific
    {  415,  286,  507967,  507981,  508006, 14, 25, 21, }, // Atlantic
    {  415,  638,  508027,  508053,  508090, 26, 37, 30, }, // Europe_Central
    {  415,  653,  508120,  508144,  508179, 24, 35, 28, }, // Europe_Eastern
    {  415,  691,  508207,  508232,  508268, 25, 36, 29, }, // Europe_Western
    {  415,  799,       0,  508297,       0,  0, 17,  0, }, // GMT
    {  418,    5,       0,  508314,       0,  0, 15,  0, }, // Afghanistan Maori/Latin/New Zealand
    {  418,   17,       0,  508329,       0,  0, 23,  0, }, // Africa_Central
    {  418,   32,       0,  508352,       0,  0, 27,  0, }, // Africa_Eastern
    {  418,   65,       0,  508379,       0,  0, 35,  0, }, // Africa_Southern
    {  418,   81,  508414,  508437,  508469, 23, 32, 31, }, // Africa_Western
    {  418,  107,  508500,  508509,  508528,  9, 19, 16, }, // Alaska
    {  418,  135,  508544,  508554,  508574, 10, 20, 18, }, // Amazon
    {  418,  142,  508592,  508601,  508620,  9, 19, 16, }, // America_Central
    {  418,  158,  508636,  508646,  508666, 10, 20, 17, }, // America_Eastern
    {  418,  174,  508683,  508692,  508711,  9, 19, 16, }, // America_Mountain
    {  418,  191,  508727,  508734,  508751,  7, 17, 14, }, // America_Pacific
    {  418,  214,  508765,  508772,  508789,  7, 17, 14, }, // Apia
    {  418,  232,  508803,  508812,  508831,  9, 19, 16, }, // Arabian
    {  418,  240,  508847,  508857,  508877, 10, 20, 18, }, // Argentina
    {  418,  250,  508895,  508915,  508945, 20, 30, 28, }, // Argentina_Western
    {  418,  268,  508973,  508982,  509001,  9, 19, 17, }, // Armenia
    {  418,  286,  509018,  509029,  509050, 11, 21, 18, }, // Atlantic
    {  418,  295,  509068,  509089,  509120, 21, 31, 28, }, // Australia_Central
    {  418,  313,  509148,  509173,  509208, 25, 35, 32, }, // Australia_CentralWestern
    {  418,  338,  509240,  509268,  509306, 28, 38, 35, }, // Australia_Eastern
    {  418,  356,  509341,  509365,  509399, 24, 34, 31, }, // Australia_Western
    {  418,  374,  509430,  509444,  509468, 14, 24, 22, }, // Azerbaijan
    {  418,  385,  509490,  509499,  509518,  9, 19, 17, }, // Azores
    {  418,  397,  509535,  509548,  509571, 13, 23, 21, }, // Bangladesh
    {  418,  415,       0,  509592,       0,  0,  9,  0, }, // Bhutan
    {  418,  422,       0,  509601,       0,  0, 10,  0, }, // Bolivia
    {  418,  437,  509611,  509621,  509641, 10, 20, 18, }, // Brasilia
    {  418,  454,       0,  509659,       0,  0, 21,  0, }, // Brunei
    {  418,  461,  509680,  509702,  509734, 22, 32, 30, }, // Cape_Verde
    {  418,  478,       0,  509764,       0,  0, 21,  0, }, // Chamorro
    {  418,  487,  509785,  509794,  509813,  9, 19, 16, }, // Chatham
    {  418,  495,  509829,  509836,  509853,  7, 17, 15, }, // Chile
    {  418,  501,  509868,  509876,  509894,  8, 18, 15, }, // China
    {  418,  507,  509909,  509922,  509945, 13, 23, 21, }, // Choibalsan
    {  418,  518,       0,  509966,       0,  0, 26,  0, }, // Christmas
    {  418,  528,       0,  509992,       0,  0, 23,  0, }, // Cocos
    {  418,  534,  510015,  510027,  510049, 12, 22, 20, }, // Colombia
    {  418,  543,  510069,  510083,  510107, 14, 24, 29, }, // Cook
    {  418,  548,  510136,  510144,  510162,  8, 18, 15, }, // Cuba
    {  418,  559,       0,  510177,       0,  0,  9,  0, }, // Davis
    {  418,  575,       0,  510186,       0,  0, 19,  0, }, // DumontDUrville
    {  418,  612,       0,  510205,       0,  0, 24,  0, }, // East_Timor
    {  418,  623,  510229,  510254,  510287, 25, 33, 31, }, // Easter
    {  418,  630,       0,  510318,       0,  0, 10,  0, }, // Ecuador
    {  418,  638,  510328,  510343,  510368, 15, 25, 23, }, // Europe_Central
    {  418,  653,  510391,  510407,  510433, 16, 26, 24, }, // Europe_Eastern
    {  418,  668,       0,  510457,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  418,  691,  510484,  510496,  510518, 12, 22, 20, }, // Europe_Western
    {  418,  706,  510538,  510560,  510592, 22, 32, 30, }, // Falkland
    {  418,  715,  510622,  510630,  510648,  8, 18, 16, }, // Fiji
    {  418,  720,       0,  510664,       0,  0, 13,  0, }, // French_Guiana
    {  418,  734,       0,  510677,       0,  0, 48,  0, }, // French_Southern
    {  418,  757,       0,  510725,       0,  0, 12,  0, }, // Galapagos
    {  418,  767,       0,  510737,       0,  0, 10,  0, }, // Gambier
    {  418,  775,  510747,  510755,  510773,  8, 18, 16, }, // Georgia
    {  418,  783,       0,  510789,       0,  0, 11,  0, }, // Gilbert_Islands
    {  418,  799,       0,  510800,       0,  0, 20,  0, }, // GMT
    {  418,  841,  510820,  510849,  510890, 29, 41, 39, }, // Greenland_Eastern
    {  418,  859,  510929,  510954,  510991, 25, 37, 35, }, // Greenland_Western
    {  418,  882,       0,  511026,       0,  0, 19,  0, }, // Gulf
    {  418,  887,       0,  511045,       0,  0,  9,  0, }, // Guyana
    {  418,  894,  511054,  511072,  511100, 18, 28, 25, }, // Hawaii_Aleutian
    {  418,  910,  511125,  511136,  511157, 11, 21, 19, }, // Hong_Kong
    {  418,  920,  511176,  511183,  511200,  7, 17, 15, }, // Hovd
    {  418,  925,       0,  511215,       0,  0,  7,  0, }, // India
    {  418,  931,       0,  511222,       0,  0, 18,  0, }, // Indian_Ocean
    {  418,  944,       0,  511240,       0,  0, 12,  0, }, // Indochina
    {  418,  954,       0,  511252,       0,  0, 20,  0, }, // Indonesia_Central
    {  418,  972,       0,  511272,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  418,  990,       0,  511299,       0,  0, 23,  0, }, // Indonesia_Western
    {  418, 1008,  511322,  511330,  511348,  8, 18, 15, }, // Iran
    {  418, 1019,  511363,  511373,  511393, 10, 20, 18, }, // Irkutsk
    {  418, 1027,  511411,  511422,  511443, 11, 21, 18, }, // Israel
    {  418, 1034,  511461,  511470,  511489,  9, 19, 16, }, // Japan
    {  418, 1058,       0,  511505,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  418, 1077,       0,  511531,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  418, 1106,  511553,  511561,  511579,  8, 18, 15, }, // Korea
    {  418, 1112,       0,  511594,       0,  0,  9,  0, }, // Kosrae
    {  418, 1119,  511603,  511617,  511641, 14, 24, 22, }, // Krasnoyarsk
    {  418, 1151,       0,  511663,       0,  0, 12,  0, }, // Kyrgystan
    {  418, 1175,       0,  511675,       0,  0, 26,  0, }, // Line_Islands
    {  418, 1188,  511701,  511713,  511735, 12, 22, 19, }, // Lord_Howe
    {  418, 1204,  511754,  511764,  511784, 10, 20, 18, }, // Magadan
    {  418, 1219,       0,  511802,       0,  0,  9,  0, }, // Malaysia
    {  418, 1228,       0,  511811,       0,  0, 11,  0, }, // Maldives
    {  418, 1237,       0,  511822,       0,  0, 12,  0, }, // Marquesas
    {  418, 1247,       0,  511834,       0,  0, 20,  0, }, // Marshall_Islands
    {  418, 1264,  511854,  511863,  511882,  9, 19, 17, }, // Mauritius
    {  418, 1274,       0,  511899,       0,  0,  9,  0, }, // Mawson
    {  418, 1281,  511908,  511922,  511946, 14, 24, 21, }, // Mexico_Pacific
    {  418, 1296,  511967,  511981,  512005, 14, 24, 22, }, // Mongolia
    {  418, 1305,  512027,  512037,  512057, 10, 20, 18, }, // Moscow
    {  418, 1312,       0,  512075,       0,  0,  7,  0, }, // Myanmar
    {  418, 1320,       0,  512082,       0,  0,  8,  0, }, // Nauru
    {  418, 1326,       0,  512090,       0,  0,  9,  0, }, // Nepal
    {  418, 1332,  512099,  512115,  512141, 16, 26, 24, }, // New_Caledonia
    {  418, 1346,  512165,  512176,  512197, 11, 21, 18, }, // New_Zealand
    {  418, 1358,  512215,  512230,  512255, 15, 25, 22, }, // Newfoundland
    {  418, 1371,       0,  512277,       0,  0,  7,  0, }, // Niue
    {  418, 1376,  512284,  512306,  512338, 22, 32, 29, }, // Norfolk
    {  418, 1384,  512367,  512389,  512421, 22, 32, 30, }, // Noronha
    {  418, 1406,  512451,  512465,  512489, 14, 24, 22, }, // Novosibirsk
    {  418, 1418,  512511,  512518,  512535,  7, 17, 15, }, // Omsk
    {  418, 1428,  512550,  512561,  512582, 11, 21, 19, }, // Pakistan
    {  418, 1437,       0,  512601,       0,  0,  8,  0, }, // Palau
    {  418, 1443,       0,  512609,       0,  0, 15,  0, }, // Papua_New_Guinea
    {  418, 1460,  512624,  512634,  512654, 10, 20, 18, }, // Paraguay
    {  418, 1469,  512672,  512679,  512696,  7, 17, 15, }, // Peru
    {  418, 1474,  512711,  512722,  512743, 11, 21, 19, }, // Philippines
    {  418, 1486,       0,  512762,       0,  0, 24,  0, }, // Phoenix_Islands
    {  418, 1502,  512786,  512811,  512848, 25, 37, 34, }, // Pierre_Miquelon
    {  418, 1518,       0,  512882,       0,  0, 11,  0, }, // Pitcairn
    {  418, 1527,       0,  512893,       0,  0,  9,  0, }, // Ponape
    {  418, 1534,       0,  512902,       0,  0, 12,  0, }, // Pyongyang
    {  418, 1554,       0,  512914,       0,  0, 10,  0, }, // Reunion
    {  418, 1562,       0,  512924,       0,  0, 10,  0, }, // Rothera
    {  418, 1570,  512934,  512945,  512966, 11, 21, 19, }, // Sakhalin
    {  418, 1596,  512985,  512993,  513011,  8, 18, 15, }, // Samoa
    {  418, 1602,       0,  513026,       0,  0, 10,  0, }, // Seychelles
    {  418, 1624,       0,  513036,       0,  0, 21,  0, }, // Singapore
    {  418, 1634,       0,  513057,       0,  0, 22,  0, }, // Solomon
    {  418, 1642,       0,  513079,       0,  0, 20,  0, }, // South_Georgia
    {  418, 1656,       0,  513099,       0,  0, 11,  0, }, // Suriname
    {  418, 1676,       0,  513110,       0,  0,  8,  0, }, // Syowa
    {  418, 1682,       0,  513118,       0,  0,  9,  0, }, // Tahiti
    {  418, 1689,  513127,  513136,  513155,  9, 19, 16, }, // Taipei
    {  418, 1696,       0,  513171,       0,  0, 14,  0, }, // Tajikistan
    {  418, 1724,       0,  513185,       0,  0, 10,  0, }, // Tokelau
    {  418, 1732,  513195,  513203,  513221,  8, 18, 16, }, // Tonga
    {  418, 1738,       0,  513237,       0,  0,  8,  0, }, // Truk
    {  418, 1750,  513245,  513261,  513287, 16, 26, 24, }, // Turkmenistan
    {  418, 1763,       0,  513311,       0,  0,  9,  0, }, // Tuvalu
    {  418, 1777,  513320,  513329,  513348,  9, 19, 17, }, // Uruguay
    {  418, 1792,  513365,  513379,  513403, 14, 24, 22, }, // Uzbekistan
    {  418, 1803,  513425,  513436,  513457, 11, 21, 19, }, // Vanuatu
    {  418, 1811,       0,  513476,       0,  0, 12,  0, }, // Venezuela
    {  418, 1821,  513488,  513502,  513526, 14, 24, 22, }, // Vladivostok
    {  418, 1833,  513548,  513560,  513582, 12, 22, 20, }, // Volgograd
    {  418, 1843,       0,  513602,       0,  0,  9,  0, }, // Vostok
    {  418, 1850,       0,  513611,       0,  0, 17,  0, }, // Wake
    {  418, 1855,       0,  513628,       0,  0, 20,  0, }, // Wallis
    {  418, 1862,  513648,  513658,  513678, 10, 20, 18, }, // Yakutsk
    {  418, 1870,  513696,  513712,  513738, 16, 26, 24, }, // Yekaterinburg
    {  418, 1892,       0,  513762,       0,  0,  8,  0, }, // Yukon
    {  420,    0,  513770,  513777,  513790,  7, 13, 15, }, // Acre Marathi/Devanagari/India
    {  420,    5,       0,  513805,       0,  0, 15,  0, }, // Afghanistan
    {  420,   17,       0,  513820,       0,  0, 17,  0, }, // Africa_Central
    {  420,   32,       0,  513837,       0,  0, 17,  0, }, // Africa_Eastern
    {  420,   65,       0,  513854,       0,  0, 25,  0, }, // Africa_Southern
    {  420,   81,  513879,  513897,  513922, 18, 25, 26, }, // Africa_Western
    {  420,  107,  427477,  513948,  513966, 11, 18, 23, }, // Alaska
    {  420,  128,  513989,  514000,  514017, 11, 17, 24, }, // Almaty
    {  420,  135,  514041,  514053,  514072, 12, 19, 20, }, // Amazon
    {  420,  142,  514092,  514104,  514123, 12, 19, 24, }, // America_Central
    {  420,  158,  514147,  514160,  514180, 13, 20, 25, }, // America_Eastern
    {  420,  174,  427656,  514205,  514223, 11, 18, 23, }, // America_Mountain
    {  420,  191,  514246,  514257,  514275, 11, 18, 23, }, // America_Pacific
    {  420,  207,  514298,  514309,  514325, 11, 16, 23, }, // Anadyr
    {  420,  214,  514348,  514357,  514373,  9, 16, 21, }, // Apia
    {  420,  219,  514394,  514406,  514424, 12, 18, 25, }, // Aqtau
    {  420,  225,  514449,  514462,  514481, 13, 19, 26, }, // Aqtobe
    {  420,  232,  514507,  514518,  514536, 11, 18, 23, }, // Arabian
    {  420,  240,  427837,  514559,  514580, 14, 21, 22, }, // Argentina
    {  420,  250,  514602,  514624,  514653, 22, 29, 30, }, // Argentina_Western
    {  420,  268,  427976,  514683,  514703, 13, 20, 21, }, // Armenia
    {  420,  286,  514724,  514736,  514755, 12, 19, 24, }, // Atlantic
    {  420,  295,  514779,  514799,  514826, 20, 27, 32, }, // Australia_Central
    {  420,  313,  514858,  514885,  514919, 27, 34, 39, }, // Australia_CentralWestern
    {  420,  338,  514958,  514979,  515007, 21, 28, 33, }, // Australia_Eastern
    {  420,  356,  515040,  515062,  515091, 22, 29, 34, }, // Australia_Western
    {  420,  374,  515125,  515137,  515156, 12, 19, 20, }, // Azerbaijan
    {  420,  385,  515176,  515188,  515207, 12, 19, 20, }, // Azores
    {  420,  397,  428514,  515227,  515247, 13, 20, 21, }, // Bangladesh
    {  420,  415,       0,  515268,       0,  0,  9,  0, }, // Bhutan
    {  420,  422,       0,  428575,       0,  0, 14,  0, }, // Bolivia
    {  420,  437,  428589,  515277,  515298, 14, 21, 22, }, // Brasilia
    {  420,  454,       0,  515320,       0,  0, 20,  0, }, // Brunei
    {  420,  461,  515340,  515355,  515377, 15, 22, 23, }, // Cape_Verde
    {  420,  478,       0,  515400,       0,  0, 17,  0, }, // Chamorro
    {  420,  487,  428745,  515417,  515432,  8, 15, 20, }, // Chatham
    {  420,  495,  428785,  515452,  515467,  8, 15, 16, }, // Chile
    {  420,  501,  515483,  515491,  515506,  8, 15, 20, }, // China
    {  420,  507,  428863,  515526,  515546, 13, 20, 21, }, // Choibalsan
    {  420,  518,       0,  515567,       0,  0, 15,  0, }, // Christmas
    {  420,  528,       0,  515582,       0,  0, 14,  0, }, // Cocos
    {  420,  534,  428951,  515596,  515615, 12, 19, 20, }, // Colombia
    {  420,  543,  515635,  515647,  515666, 12, 19, 25, }, // Cook
    {  420,  548,  515691,  515701,  515718, 10, 17, 22, }, // Cuba
    {  420,  559,       0,  515740,       0,  0, 11,  0, }, // Davis
    {  420,  575,       0,  515751,       0,  0, 23,  0, }, // DumontDUrville
    {  420,  612,       0,  515774,       0,  0, 15,  0, }, // East_Timor
    {  420,  623,  515789,  515802,  515822, 13, 20, 21, }, // Easter
    {  420,  630,       0,  429217,       0,  0, 12,  0, }, // Ecuador
    {  420,  638,  515843,  515861,  515886, 18, 25, 26, }, // Europe_Central
    {  420,  653,  515912,  515930,  515955, 18, 25, 26, }, // Europe_Eastern
    {  420,  668,       0,  515981,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  420,  691,  516008,  516027,  516053, 19, 26, 27, }, // Europe_Western
    {  420,  706,  516080,  516095,  516117, 15, 22, 23, }, // Falkland
    {  420,  715,  429536,  516140,  516155,  8, 15, 16, }, // Fiji
    {  420,  720,       0,  429577,       0,  0, 16,  0, }, // French_Guiana
    {  420,  734,       0,  516171,       0,  0, 33,  0, }, // French_Southern
    {  420,  757,       0,  516204,       0,  0, 13,  0, }, // Galapagos
    {  420,  767,       0,  516217,       0,  0, 12,  0, }, // Gambier
    {  420,  775,  429652,  516229,  516248, 12, 19, 20, }, // Georgia
    {  420,  783,       0,  516268,       0,  0, 17,  0, }, // Gilbert_Islands
    {  420,  799,       0,  516285,       0,  0, 18,  0, }, // GMT
    {  420,  841,  516303,  516321,  516346, 18, 25, 26, }, // Greenland_Eastern
    {  420,  859,  516372,  516391,  516417, 19, 26, 27, }, // Greenland_Western
    {  420,  877,       0,  516444,       0,  0, 14,  0, }, // Guam
    {  420,  882,       0,  516458,       0,  0, 15,  0, }, // Gulf
    {  420,  887,       0,  429584,       0,  0,  9,  0, }, // Guyana
    {  420,  894,  516473,  516487,  516508, 14, 21, 26, }, // Hawaii_Aleutian
    {  420,  910,  429979,  516534,  516554, 13, 20, 21, }, // Hong_Kong
    {  420,  920,  516575,  516586,  516604, 11, 18, 19, }, // Hovd
    {  420,  925,       0,  516623,       0,  0, 17,  0, }, // India
    {  420,  931,       0,  516640,       0,  0, 15,  0, }, // Indian_Ocean
    {  420,  944,       0,  516655,       0,  0, 13,  0, }, // Indochina
    {  420,  954,       0,  516668,       0,  0, 20,  0, }, // Indonesia_Central
    {  420,  972,       0,  516688,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  420,  990,       0,  516712,       0,  0, 22,  0, }, // Indonesia_Western
    {  420, 1008,  516734,  516742,  516757,  8, 15, 20, }, // Iran
    {  420, 1019,  516777,  516790,  516810, 13, 20, 21, }, // Irkutsk
    {  420, 1027,  516831,  516838,  516856,  7, 18, 23, }, // Israel
    {  420, 1034,  430322,  516879,  516894,  8, 15, 20, }, // Japan
    {  420, 1040,  516914,  516947,  516986, 33, 39, 46, }, // Kamchatka
    {  420, 1058,       0,  517032,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  420, 1077,       0,  517051,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  420, 1106,  430401,  517071,  517088, 10, 17, 22, }, // Korea
    {  420, 1112,       0,  517110,       0,  0, 11,  0, }, // Kosrae
    {  420, 1119,  430456,  517121,  517147, 19, 26, 27, }, // Krasnoyarsk
    {  420, 1151,       0,  517174,       0,  0, 14,  0, }, // Kyrgystan
    {  420, 1161,       0,  517188,       0,  0,  8,  0, }, // Lanka
    {  420, 1175,       0,  517196,       0,  0, 13,  0, }, // Line_Islands
    {  420, 1188,  430561,  517209,  517230, 14, 21, 26, }, // Lord_Howe
    {  420, 1198,  517256,  517264,  517278,  8, 14, 21, }, // Macau
    {  420, 1204,  517299,  517309,  517326, 10, 17, 18, }, // Magadan
    {  420, 1219,       0,  430666,       0,  0, 11,  0, }, // Malaysia
    {  420, 1228,       0,  517344,       0,  0, 10,  0, }, // Maldives
    {  420, 1237,       0,  517354,       0,  0, 15,  0, }, // Marquesas
    {  420, 1247,       0,  517369,       0,  0, 15,  0, }, // Marshall_Islands
    {  420, 1264,  517384,  517394,  517411, 10, 17, 18, }, // Mauritius
    {  420, 1274,       0,  430762,       0,  0,  8,  0, }, // Mawson
    {  420, 1281,  517429,  517449,  517476, 20, 27, 32, }, // Mexico_Pacific
    {  420, 1296,  517508,  517522,  517543, 14, 21, 22, }, // Mongolia
    {  420, 1305,  430899,  517565,  517582, 10, 17, 18, }, // Moscow
    {  420, 1312,       0,  430946,       0,  0, 12,  0, }, // Myanmar
    {  420, 1320,       0,  517600,       0,  0,  8,  0, }, // Nauru
    {  420, 1326,       0,  430965,       0,  0,  9,  0, }, // Nepal
    {  420, 1332,  517608,  517627,  517653, 19, 26, 27, }, // New_Caledonia
    {  420, 1346,  517680,  517693,  517713, 13, 20, 25, }, // New_Zealand
    {  420, 1358,  517738,  517755,  517779, 17, 24, 29, }, // Newfoundland
    {  420, 1371,       0,  431173,       0,  0,  9,  0, }, // Niue
    {  420, 1376,  517808,  517822,  517843, 14, 21, 26, }, // Norfolk
    {  420, 1384,  431247,  517869,  517900, 24, 31, 32, }, // Noronha
    {  420, 1392,       0,  517932,       0,  0, 20,  0, }, // North_Mariana
    {  420, 1406,  517952,  517969,  517993, 17, 24, 25, }, // Novosibirsk
    {  420, 1418,  431387,  518018,  518035, 10, 17, 18, }, // Omsk
    {  420, 1428,  518053,  518066,  518086, 13, 20, 21, }, // Pakistan
    {  420, 1437,       0,  431477,       0,  0,  8,  0, }, // Palau
    {  420, 1443,       0,  518107,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  420, 1460,  518126,  518138,  518157, 12, 19, 20, }, // Paraguay
    {  420, 1469,  518177,  518185,  518200,  8, 15, 16, }, // Peru
    {  420, 1474,  518216,  518228,  518247, 12, 19, 20, }, // Philippines
    {  420, 1486,       0,  518267,       0,  0, 17,  0, }, // Phoenix_Islands
    {  420, 1502,  518284,  518311,  518346, 27, 35, 40, }, // Pierre_Miquelon
    {  420, 1518,       0,  518386,       0,  0, 12,  0, }, // Pitcairn
    {  420, 1527,       0,  518398,       0,  0, 10,  0, }, // Ponape
    {  420, 1534,       0,  431770,       0,  0, 14,  0, }, // Pyongyang
    {  420, 1544,  518408,  518423,  518444, 15, 21, 28, }, // Qyzylorda
    {  420, 1554,       0,  431784,       0,  0, 12,  0, }, // Reunion
    {  420, 1562,       0,  431796,       0,  0, 10,  0, }, // Rothera
    {  420, 1570,  431806,  518472,  518488,  9, 16, 17, }, // Sakhalin
    {  420, 1579,  518505,  518514,  518529,  9, 15, 22, }, // Samara
    {  420, 1596,  431850,  518551,  518567,  9, 16, 21, }, // Samoa
    {  420, 1602,       0,  431893,       0,  0, 11,  0, }, // Seychelles
    {  420, 1624,       0,  518588,       0,  0, 19,  0, }, // Singapore
    {  420, 1634,       0,  518607,       0,  0, 16,  0, }, // Solomon
    {  420, 1642,       0,  431943,       0,  0, 19,  0, }, // South_Georgia
    {  420, 1656,       0,  431962,       0,  0, 11,  0, }, // Suriname
    {  420, 1676,       0,  431973,       0,  0, 10,  0, }, // Syowa
    {  420, 1682,       0,  431983,       0,  0, 10,  0, }, // Tahiti
    {  420, 1689,  431993,  518623,  518639,  9, 16, 21, }, // Taipei
    {  420, 1696,       0,  518660,       0,  0, 15,  0, }, // Tajikistan
    {  420, 1724,       0,  518675,       0,  0, 11,  0, }, // Tokelau
    {  420, 1732,  432060,  518686,  518702,  9, 16, 17, }, // Tonga
    {  420, 1738,       0,  518719,       0,  0,  7,  0, }, // Truk
    {  420, 1750,  518726,  518744,  518769, 18, 25, 26, }, // Turkmenistan
    {  420, 1763,       0,  518795,       0,  0, 10,  0, }, // Tuvalu
    {  420, 1777,  432174,  518805,  518823, 11, 18, 19, }, // Uruguay
    {  420, 1792,  518842,  518857,  518879, 15, 22, 23, }, // Uzbekistan
    {  420, 1803,  518902,  518913,  518931, 11, 18, 19, }, // Vanuatu
    {  420, 1811,       0,  518950,       0,  0, 15,  0, }, // Venezuela
    {  420, 1821,  432349,  518965,  518989, 17, 24, 25, }, // Vladivostok
    {  420, 1833,  432417,  519014,  519038, 17, 24, 25, }, // Volgograd
    {  420, 1843,       0,  519063,       0,  0, 13,  0, }, // Vostok
    {  420, 1850,       0,  519076,       0,  0, 11,  0, }, // Wake
    {  420, 1855,       0,  519087,       0,  0, 20,  0, }, // Wallis
    {  420, 1862,  519107,  519119,  519138, 12, 19, 20, }, // Yakutsk
    {  420, 1870,  519158,  519173,  519195, 15, 22, 23, }, // Yekaterinburg
    {  420, 1892,       0,  519218,       0,  0,  9,  0, }, // Yukon
    {  427,    5,       0,  519227,       0,  0, 15,  0, }, // Afghanistan Mongolian/Cyrillic/Mongolia
    {  427,   17,       0,  519242,       0,  0, 16,  0, }, // Africa_Central
    {  427,   32,       0,  519258,       0,  0, 17,  0, }, // Africa_Eastern
    {  427,   65,       0,  519275,       0,  0, 27,  0, }, // Africa_Southern
    {  427,   81,  519302,  519321,  519349, 19, 28, 24, }, // Africa_Western
    {  427,  107,  519373,  519385,  519406, 12, 21, 17, }, // Alaska
    {  427,  135,  519423,  519434,  519454, 11, 20, 16, }, // Amazon
    {  427,  142,  519470,  519477,  519493,  7, 16, 12, }, // America_Central
    {  427,  158,  519505,  519520,  519544, 15, 24, 20, }, // America_Eastern
    {  427,  174,  519564,  519573,  519591,  9, 18, 14, }, // America_Mountain
    {  427,  191,  519605,  519622,  519648, 17, 26, 22, }, // America_Pacific
    {  427,  214,  519670,  519682,  519703, 12, 21, 17, }, // Apia
    {  427,  232,  519720,  519730,  519749, 10, 19, 15, }, // Arabian
    {  427,  240,  519764,  519777,  519799, 13, 22, 18, }, // Argentina
    {  427,  250,  519817,  519837,  519866, 20, 29, 25, }, // Argentina_Western
    {  427,  268,  519891,  519903,  519924, 12, 21, 17, }, // Armenia
    {  427,  286,  519941,  519953,  519974, 12, 21, 17, }, // Atlantic
    {  427,  295,  519991,  520009,  520036, 18, 27, 23, }, // Australia_Central
    {  427,  313,  520059,  520091,  520132, 32, 41, 37, }, // Australia_CentralWestern
    {  427,  338,  520169,  520188,  520223, 19, 35, 31, }, // Australia_Eastern
    {  427,  356,  520254,  520275,  520312, 21, 37, 33, }, // Australia_Western
    {  427,  374,  520345,  520360,  520384, 15, 24, 20, }, // Azerbaijan
    {  427,  385,  520404,  520414,  520433, 10, 19, 15, }, // Azores
    {  427,  397,  520448,  520464,  520489, 16, 25, 21, }, // Bangladesh
    {  427,  415,       0,  520510,       0,  0, 10,  0, }, // Bhutan
    {  427,  422,       0,  520520,       0,  0, 12,  0, }, // Bolivia
    {  427,  437,  520532,  520545,  520567, 13, 22, 18, }, // Brasilia
    {  427,  454,       0,  520585,       0,  0, 23,  0, }, // Brunei
    {  427,  461,  520608,  520624,  520649, 16, 25, 21, }, // Cape_Verde
    {  427,  478,       0,  520670,       0,  0, 15,  0, }, // Chamorro
    {  427,  487,  520685,  520696,  520716, 11, 20, 16, }, // Chatham
    {  427,  495,  520732,  520742,  520761, 10, 19, 15, }, // Chile
    {  427,  501,  520776,  520787,  520807, 11, 20, 16, }, // China
    {  427,  507,  520823,  520840,  520866, 17, 26, 22, }, // Choibalsan
    {  427,  518,       0,  520888,       0,  0, 17,  0, }, // Christmas
    {  427,  528,       0,  520905,       0,  0, 15,  0, }, // Cocos
    {  427,  534,  520920,  520932,  520953, 12, 21, 17, }, // Colombia
    {  427,  543,  520970,  520990,  521019, 20, 29, 31, }, // Cook
    {  427,  548,  521050,  521059,  521077,  9, 18, 14, }, // Cuba
    {  427,  559,       0,  521091,       0,  0, 12,  0, }, // Davis
    {  427,  575,       0,  521103,       0,  0, 20,  0, }, // DumontDUrville
    {  427,  612,       0,  521123,       0,  0, 16,  0, }, // East_Timor
    {  427,  623,  521139,  521156,  521182, 17, 26, 22, }, // Easter
    {  427,  630,       0,  521204,       0,  0, 13,  0, }, // Ecuador
    {  427,  638,  521217,  521232,  521256, 15, 24, 20, }, // Europe_Central
    {  427,  653,  521276,  521292,  521317, 16, 25, 21, }, // Europe_Eastern
    {  427,  668,       0,  521338,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  427,  691,  521360,  521378,  521405, 18, 27, 23, }, // Europe_Western
    {  427,  706,  521428,  521452,  521485, 24, 33, 29, }, // Falkland
    {  427,  715,  521514,  521526,  521547, 12, 21, 17, }, // Fiji
    {  427,  720,       0,  521564,       0,  0, 22,  0, }, // French_Guiana
    {  427,  734,       0,  521586,       0,  0, 33,  0, }, // French_Southern
    {  427,  757,       0,  521619,       0,  0, 15,  0, }, // Galapagos
    {  427,  767,       0,  521634,       0,  0, 14,  0, }, // Gambier
    {  427,  775,  521648,  521659,  521679, 11, 20, 16, }, // Georgia
    {  427,  783,       0,  521695,       0,  0, 24,  0, }, // Gilbert_Islands
    {  427,  799,       0,  521719,       0,  0, 14,  0, }, // GMT
    {  427,  841,  521733,  521752,  521780, 19, 28, 24, }, // Greenland_Eastern
    {  427,  859,  521804,  521825,  521855, 21, 30, 26, }, // Greenland_Western
    {  427,  882,       0,  521881,       0,  0, 21,  0, }, // Gulf
    {  427,  887,       0,  521902,       0,  0, 14,  0, }, // Guyana
    {  427,  894,  521916,  521933,  521959, 17, 26, 22, }, // Hawaii_Aleutian
    {  427,  910,  521981,  521997,  522022, 16, 25, 21, }, // Hong_Kong
    {  427,  920,  522043,  522053,  522072, 10, 19, 15, }, // Hovd
    {  427,  925,       0,  522087,       0,  0, 14,  0, }, // India
    {  427,  931,       0,  522101,       0,  0, 21,  0, }, // Indian_Ocean
    {  427,  944,       0,  522122,       0,  0, 27,  0, }, // Indochina
    {  427,  954,       0,  522149,       0,  0, 18,  0, }, // Indonesia_Central
    {  427,  972,       0,  522167,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  427,  990,       0,  522186,       0,  0, 21,  0, }, // Indonesia_Western
    {  427, 1008,  522207,  522216,  522234,  9, 18, 14, }, // Iran
    {  427, 1019,  522248,  522261,  522283, 13, 22, 18, }, // Irkutsk
    {  427, 1027,  522301,  522314,  522336, 13, 22, 18, }, // Israel
    {  427, 1034,  522354,  522363,  522381,  9, 18, 14, }, // Japan
    {  427, 1058,       0,  522395,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  427, 1077,       0,  522414,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  427, 1106,  522435,  522449,  522472, 14, 23, 19, }, // Korea
    {  427, 1112,       0,  522491,       0,  0, 13,  0, }, // Kosrae
    {  427, 1119,  522504,  522521,  522547, 17, 26, 22, }, // Krasnoyarsk
    {  427, 1151,       0,  522569,       0,  0, 13,  0, }, // Kyrgystan
    {  427, 1175,       0,  522582,       0,  0, 17,  0, }, // Line_Islands
    {  427, 1188,  522599,  522615,  522640, 16, 25, 21, }, // Lord_Howe
    {  427, 1204,  522661,  522673,  522694, 12, 21, 17, }, // Magadan
    {  427, 1219,       0,  522711,       0,  0, 12,  0, }, // Malaysia
    {  427, 1228,       0,  522723,       0,  0, 14,  0, }, // Maldives
    {  427, 1237,       0,  522737,       0,  0, 21,  0, }, // Marquesas
    {  427, 1247,       0,  522758,       0,  0, 22,  0, }, // Marshall_Islands
    {  427, 1264,  522780,  522793,  522815, 13, 22, 18, }, // Mauritius
    {  427, 1274,       0,  522833,       0,  0, 11,  0, }, // Mawson
    {  427, 1281,  522844,  522868,  522901, 24, 33, 29, }, // Mexico_Pacific
    {  427, 1296,  522930,  522947,  522973, 17, 26, 22, }, // Mongolia
    {  427, 1305,  522995,  523009,  523032, 14, 23, 19, }, // Moscow
    {  427, 1312,       0,  523051,       0,  0, 13,  0, }, // Myanmar
    {  427, 1320,       0,  523064,       0,  0, 13,  0, }, // Nauru
    {  427, 1326,       0,  523077,       0,  0, 10,  0, }, // Nepal
    {  427, 1332,  523087,  523106,  523134, 19, 28, 24, }, // New_Caledonia
    {  427, 1346,  523158,  523175,  523201, 17, 26, 22, }, // New_Zealand
    {  427, 1358,  523223,  523242,  523270, 19, 28, 24, }, // Newfoundland
    {  427, 1371,       0,  523294,       0,  0, 12,  0, }, // Niue
    {  427, 1376,  523306,  523323,  523349, 17, 26, 22, }, // Norfolk
    {  427, 1384,  523371,  523397,  523432, 26, 35, 31, }, // Noronha
    {  427, 1406,  523463,  523481,  523508, 18, 27, 23, }, // Novosibirsk
    {  427, 1418,  523531,  523542,  523562, 11, 20, 16, }, // Omsk
    {  427, 1428,  523578,  523591,  523613, 13, 22, 18, }, // Pakistan
    {  427, 1437,       0,  523631,       0,  0, 13,  0, }, // Palau
    {  427, 1443,       0,  523644,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  427, 1460,  523666,  523679,  523701, 13, 22, 18, }, // Paraguay
    {  427, 1469,  523719,  523731,  523752, 12, 21, 17, }, // Peru
    {  427, 1474,  523769,  523783,  523806, 14, 23, 19, }, // Philippines
    {  427, 1486,       0,  523825,       0,  0, 16,  0, }, // Phoenix_Islands
    {  427, 1502,  523841,  523866,  523900, 25, 34, 30, }, // Pierre_Miquelon
    {  427, 1518,       0,  523930,       0,  0, 14,  0, }, // Pitcairn
    {  427, 1527,       0,  523944,       0,  0, 14,  0, }, // Ponape
    {  427, 1534,       0,  523958,       0,  0, 11,  0, }, // Pyongyang
    {  427, 1554,       0,  523969,       0,  0, 12,  0, }, // Reunion
    {  427, 1562,       0,  523981,       0,  0, 14,  0, }, // Rothera
    {  427, 1570,  523995,  524007,  524028, 12, 21, 17, }, // Sakhalin
    {  427, 1596,  524045,  524058,  524080, 13, 22, 18, }, // Samoa
    {  427, 1602,       0,  524098,       0,  0, 22,  0, }, // Seychelles
    {  427, 1624,       0,  524120,       0,  0, 14,  0, }, // Singapore
    {  427, 1634,       0,  524134,       0,  0, 21,  0, }, // Solomon
    {  427, 1642,       0,  524155,       0,  0, 20,  0, }, // South_Georgia
    {  427, 1656,       0,  524175,       0,  0, 13,  0, }, // Suriname
    {  427, 1676,       0,  524188,       0,  0, 12,  0, }, // Syowa
    {  427, 1682,       0,  524200,       0,  0, 13,  0, }, // Tahiti
    {  427, 1689,  524213,  524224,  524244, 11, 20, 16, }, // Taipei
    {  427, 1696,       0,  524260,       0,  0, 15,  0, }, // Tajikistan
    {  427, 1724,       0,  524275,       0,  0, 15,  0, }, // Tokelau
    {  427, 1732,  524290,  524303,  524325, 13, 22, 18, }, // Tonga
    {  427, 1738,       0,  524343,       0,  0, 11,  0, }, // Truk
    {  427, 1750,  524354,  524371,  524397, 17, 26, 22, }, // Turkmenistan
    {  427, 1763,       0,  524419,       0,  0, 14,  0, }, // Tuvalu
    {  427, 1777,  524433,  524445,  524466, 12, 21, 17, }, // Uruguay
    {  427, 1792,  524483,  524498,  524522, 15, 24, 20, }, // Uzbekistan
    {  427, 1803,  524542,  524557,  524581, 15, 24, 20, }, // Vanuatu
    {  427, 1811,       0,  524601,       0,  0, 15,  0, }, // Venezuela
    {  427, 1821,  524616,  524634,  524661, 18, 27, 23, }, // Vladivostok
    {  427, 1833,  524684,  524699,  524723, 15, 24, 20, }, // Volgograd
    {  427, 1843,       0,  524743,       0,  0, 13,  0, }, // Vostok
    {  427, 1850,       0,  524756,       0,  0, 19,  0, }, // Wake
    {  427, 1855,       0,  524775,       0,  0, 24,  0, }, // Wallis
    {  427, 1862,  524799,  524809,  524828, 10, 19, 15, }, // Yakutsk
    {  427, 1870,  524843,  524862,  524890, 19, 28, 24, }, // Yekaterinburg
    {  427, 1892,       0,  524914,       0,  0, 15,  0, }, // Yukon
    {  429,  142,  524929,  524936,  524955,  7, 19, 16, }, // America_Central Mongolian/Mongolian/Mongolia
    {  429,  158,  524971,  524990,  525021, 19, 31, 28, }, // America_Eastern
    {  429,  174,  525049,  525066,  525095, 17, 29, 22, }, // America_Mountain
    {  429,  191,  525117,  525139,  525174, 22, 35, 31, }, // America_Pacific
    {  429,  286,  525205,  525222,  525250, 17, 28, 23, }, // Atlantic
    {  429,  638,  525273,  525293,  525325, 20, 32, 28, }, // Europe_Central
    {  429,  653,  525353,  525375,  525409, 22, 34, 30, }, // Europe_Eastern
    {  429,  691,  525439,  525463,  525499, 24, 36, 32, }, // Europe_Western
    {  429,  799,       0,  525531,       0,  0, 17,  0, }, // GMT
    {  435,    5,       0,  525548,       0,  0, 15,  0, }, // Afghanistan Nepali/Devanagari/Nepal
    {  435,   17,       0,  525563,       0,  0, 21,  0, }, // Africa_Central
    {  435,   32,       0,  525584,       0,  0, 18,  0, }, // Africa_Eastern
    {  435,   65,       0,  525602,       0,  0, 18,  0, }, // Africa_Southern
    {  435,   81,  525620,  525638,  525661, 18, 23, 31, }, // Africa_Western
    {  435,  107,  525692,  525704,  525721, 12, 17, 17, }, // Alaska
    {  435,  135,  525738,  525748,  525763, 10, 15, 23, }, // Amazon
    {  435,  142,  525786,  525799,  525817, 13, 18, 18, }, // America_Central
    {  435,  158,  330476,  330501,  525835, 10, 15, 15, }, // America_Eastern
    {  435,  174,  525850,  525860,  525875, 10, 15, 15, }, // America_Mountain
    {  435,  191,  525890,  525903,  525921, 13, 18, 18, }, // America_Pacific
    {  435,  214,  525939,  525948,  525962,  9, 14, 14, }, // Apia
    {  435,  232,  525976,  525984,  525997,  8, 13, 13, }, // Arabian
    {  435,  240,  526010,  526024,  526043, 14, 19, 27, }, // Argentina
    {  435,  250,  526070,  526092,  526119, 22, 27, 35, }, // Argentina_Western
    {  435,  268,  526154,  526167,  526185, 13, 18, 26, }, // Armenia
    {  435,  286,  526211,  526225,  526244, 14, 19, 19, }, // Atlantic
    {  435,  295,  526263,  526288,  526318, 25, 30, 30, }, // Australia_Central
    {  435,  313,  526348,  526381,  526419, 33, 38, 38, }, // Australia_CentralWestern
    {  435,  338,  526457,  526479,  526506, 22, 27, 27, }, // Australia_Eastern
    {  435,  356,  526358,  526391,  526429, 23, 28, 28, }, // Australia_Western
    {  435,  374,  526533,  526545,  526562, 12, 17, 25, }, // Azerbaijan
    {  435,  385,  526587,  526598,  526614, 11, 16, 24, }, // Azores
    {  435,  397,  526638,  526651,  526669, 13, 18, 26, }, // Bangladesh
    {  435,  415,       0,  526695,       0,  0, 10,  0, }, // Bhutan
    {  435,  422,       0,  526705,       0,  0, 12,  0, }, // Bolivia
    {  435,  437,  526717,  526731,  526750, 14, 19, 27, }, // Brasilia
    {  435,  454,       0,  526777,       0,  0, 21,  0, }, // Brunei
    {  435,  461,  526798,  526811,  526829, 13, 18, 26, }, // Cape_Verde
    {  435,  478,       0,  526855,       0,  0, 17,  0, }, // Chamorro
    {  435,  487,  526872,  526881,  526895,  9, 14, 14, }, // Chatham
    {  435,  495,  331839,  331847,  331860,  8, 13, 21, }, // Chile
    {  435,  501,  331881,  331888,  526909,  7, 12, 12, }, // China
    {  435,  507,  526921,  526934,  526952, 13, 18, 26, }, // Choibalsan
    {  435,  518,       0,  526978,       0,  0, 17,  0, }, // Christmas
    {  435,  528,       0,  526995,       0,  0, 14,  0, }, // Cocos
    {  435,  534,  527009,  527024,  527044, 15, 20, 28, }, // Colombia
    {  435,  543,  527072,  527084,  527101, 12, 17, 29, }, // Cook
    {  435,  548,  527130,  527142,  527159, 12, 17, 17, }, // Cuba
    {  435,  559,       0,  527176,       0,  0,  9,  0, }, // Davis
    {  435,  575,       0,  527185,       0,  0, 23,  0, }, // DumontDUrville
    {  435,  612,       0,  527208,       0,  0, 16,  0, }, // East_Timor
    {  435,  623,  527224,  527238,  527257, 14, 19, 22, }, // Easter
    {  435,  630,       0,  527279,       0,  0, 12,  0, }, // Ecuador
    {  435,  638,  527291,  527313,  527340, 22, 27, 35, }, // Europe_Central
    {  435,  653,  527375,  527394,  527418, 19, 24, 32, }, // Europe_Eastern
    {  435,  668,       0,  527450,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  435,  691,  527472,  527492,  527350, 20, 25, 25, }, // Europe_Western
    {  435,  706,  527517,  527537,  527562, 20, 25, 33, }, // Falkland
    {  435,  715,  527595,  527603,  527616,  8, 13, 21, }, // Fiji
    {  435,  720,       0,  527637,       0,  0, 18,  0, }, // French_Guiana
    {  435,  734,       0,  527655,       0,  0, 32,  0, }, // French_Southern
    {  435,  757,       0,  527687,       0,  0, 14,  0, }, // Galapagos
    {  435,  767,       0,  527701,       0,  0, 14,  0, }, // Gambier
    {  435,  775,  527715,  527726,  527742, 11, 16, 24, }, // Georgia
    {  435,  783,       0,  527766,       0,  0, 17,  0, }, // Gilbert_Islands
    {  435,  799,       0,  527783,       0,  0, 16,  0, }, // GMT
    {  435,  841,  527799,  527824,  527854, 25, 30, 38, }, // Greenland_Eastern
    {  435,  859,  527892,  527918,  527949, 26, 31, 39, }, // Greenland_Western
    {  435,  882,       0,  527988,       0,  0, 13,  0, }, // Gulf
    {  435,  887,       0,  332678,       0,  0, 10,  0, }, // Guyana
    {  435,  894,  528001,  528017,  528038, 16, 21, 21, }, // Hawaii_Aleutian
    {  435,  910,  528059,  528067,  528080,  8, 13, 21, }, // Hong_Kong
    {  435,  920,  528101,  528110,  528124,  9, 14, 22, }, // Hovd
    {  435,  925,       0,  333159,       0,  0, 15,  0, }, // India
    {  435,  931,       0,  528146,       0,  0, 17,  0, }, // Indian_Ocean
    {  435,  944,       0,  528163,       0,  0, 14,  0, }, // Indochina
    {  435,  954,       0,  528177,       0,  0, 25,  0, }, // Indonesia_Central
    {  435,  972,       0,  528202,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  435,  990,       0,  528224,       0,  0, 23,  0, }, // Indonesia_Western
    {  435, 1008,  528247,  528256,  528270,  9, 14, 14, }, // Iran
    {  435, 1019,  528284,  528298,  528317, 14, 19, 27, }, // Irkutsk
    {  435, 1027,  528344,  528354,  528369, 10, 15, 15, }, // Israel
    {  435, 1034,  333406,  333415,  528384,  9, 14, 14, }, // Japan
    {  435, 1058,       0,  528398,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  435, 1077,       0,  528418,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  435, 1106,  528438,  528450,  528467, 12, 17, 17, }, // Korea
    {  435, 1112,       0,  333636,       0,  0, 10,  0, }, // Kosrae
    {  435, 1119,  528484,  528500,  528521, 16, 21, 29, }, // Krasnoyarsk
    {  435, 1151,       0,  528550,       0,  0, 15,  0, }, // Kyrgystan
    {  435, 1175,       0,  528565,       0,  0, 13,  0, }, // Line_Islands
    {  435, 1188,  528578,  528591,  528609, 13, 18, 18, }, // Lord_Howe
    {  435, 1204,  333809,  333820,  333836, 11, 16, 24, }, // Magadan
    {  435, 1219,       0,  528627,       0,  0, 11,  0, }, // Malaysia
    {  435, 1228,       0,  528638,       0,  0, 13,  0, }, // Maldives
    {  435, 1237,       0,  528651,       0,  0, 12,  0, }, // Marquesas
    {  435, 1247,       0,  528663,       0,  0, 15,  0, }, // Marshall_Islands
    {  435, 1264,  528678,  528688,  528703, 10, 15, 23, }, // Mauritius
    {  435, 1274,       0,  528726,       0,  0,  9,  0, }, // Mawson
    {  435, 1281,  528735,  528757,  528784, 22, 27, 27, }, // Mexico_Pacific
    {  435, 1296,  528811,  528825,  528844, 14, 19, 27, }, // Mongolia
    {  435, 1305,  528871,  528880,  528894,  9, 14, 22, }, // Moscow
    {  435, 1312,       0,  528916,       0,  0, 12,  0, }, // Myanmar
    {  435, 1320,       0,  528928,       0,  0,  9,  0, }, // Nauru
    {  435, 1326,       0,  528937,       0,  0, 10,  0, }, // Nepal
    {  435, 1332,  528947,  528966,  528990, 19, 24, 32, }, // New_Caledonia
    {  435, 1346,  529022,  529039,  529061, 17, 22, 22, }, // New_Zealand
    {  435, 1358,  529083,  529106,  529133, 23, 27, 26, }, // Newfoundland
    {  435, 1371,       0,  529159,       0,  0,  8,  0, }, // Niue
    {  435, 1376,  529167,  529185,  529208, 18, 23, 31, }, // Norfolk
    {  435, 1384,  529239,  529264,  529294, 25, 30, 38, }, // Noronha
    {  435, 1406,  529332,  529349,  529371, 17, 22, 30, }, // Novosibirsk
    {  435, 1418,  334614,  334624,  334639, 10, 15, 23, }, // Omsk
    {  435, 1428,  529401,  529415,  529434, 14, 19, 27, }, // Pakistan
    {  435, 1437,       0,  529461,       0,  0,  9,  0, }, // Palau
    {  435, 1443,       0,  529470,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  435, 1460,  529489,  529501,  529518, 12, 17, 25, }, // Paraguay
    {  435, 1469,  529543,  334808,  529551,  8, 13, 21, }, // Peru
    {  435, 1474,  529572,  529584,  529601, 12, 17, 25, }, // Philippines
    {  435, 1486,       0,  529626,       0,  0, 16,  0, }, // Phoenix_Islands
    {  435, 1502,  529642,  529673,  529709, 31, 36, 36, }, // Pierre_Miquelon
    {  435, 1518,       0,  529745,       0,  0, 11,  0, }, // Pitcairn
    {  435, 1527,       0,  335023,       0,  0, 10,  0, }, // Ponape
    {  435, 1534,       0,  529756,       0,  0, 12,  0, }, // Pyongyang
    {  435, 1554,       0,  529768,       0,  0, 12,  0, }, // Reunion
    {  435, 1562,       0,  335059,       0,  0, 10,  0, }, // Rothera
    {  435, 1570,  529780,  529791,  529807, 11, 16, 24, }, // Sakhalin
    {  435, 1596,  529831,  529840,  529854,  9, 14, 14, }, // Samoa
    {  435, 1602,       0,  529868,       0,  0, 14,  0, }, // Seychelles
    {  435, 1624,       0,  529882,       0,  0, 17,  0, }, // Singapore
    {  435, 1634,       0,  529899,       0,  0, 16,  0, }, // Solomon
    {  435, 1642,       0,  529915,       0,  0, 18,  0, }, // South_Georgia
    {  435, 1656,       0,  529933,       0,  0, 12,  0, }, // Suriname
    {  435, 1676,       0,  335272,       0,  0, 10,  0, }, // Syowa
    {  435, 1682,       0,  335282,       0,  0, 10,  0, }, // Tahiti
    {  435, 1689,  529945,  529955,  529970, 10, 15, 15, }, // Taipei
    {  435, 1696,       0,  529985,       0,  0, 14,  0, }, // Tajikistan
    {  435, 1724,       0,  529999,       0,  0, 11,  0, }, // Tokelau
    {  435, 1732,  335357,  335366,  335380,  9, 14, 22, }, // Tonga
    {  435, 1738,       0,  530010,       0,  0,  8,  0, }, // Truk
    {  435, 1750,  335409,  335427,  530018, 18, 23, 36, }, // Turkmenistan
    {  435, 1763,       0,  530054,       0,  0, 10,  0, }, // Tuvalu
    {  435, 1777,  335491,  530064,  335518, 11, 16, 24, }, // Uruguay
    {  435, 1792,  530080,  530096,  530117, 16, 21, 29, }, // Uzbekistan
    {  435, 1803,  530146,  530157,  530173, 11, 16, 24, }, // Vanuatu
    {  435, 1811,       0,  530197,       0,  0, 15,  0, }, // Venezuela
    {  435, 1821,  530212,  530229,  530251, 17, 22, 30, }, // Vladivostok
    {  435, 1833,  530281,  530296,  530316, 15, 20, 28, }, // Volgograd
    {  435, 1843,       0,  530218,       0,  0, 11,  0, }, // Vostok
    {  435, 1850,       0,  530344,       0,  0, 12,  0, }, // Wake
    {  435, 1855,       0,  530356,       0,  0, 19,  0, }, // Wallis
    {  435, 1862,  530375,  530386,  530402, 11, 16, 24, }, // Yakutsk
    {  435, 1870,  530426,  530443,  530465, 17, 22, 30, }, // Yekaterinburg
    {  435, 1892,       0,  530495,       0,  0, 11,  0, }, // Yukon
    {  436,  925,       0,  333159,       0,  0, 15,  0, }, // India Nepali/Devanagari/India
    {  439,    5,       0,  530506,       0,  0, 15,  0, }, // Afghanistan Nigerian Pidgin/Latin/Nigeria
    {  439,   17,       0,  530521,       0,  0, 17,  0, }, // Africa_Central
    {  439,   32,       0,  530538,       0,  0, 15,  0, }, // Africa_Eastern
    {  439,   65,       0,  530553,       0,  0, 22,  0, }, // Africa_Southern
    {  439,   81,  530575,  530592,  530615, 17, 23, 28, }, // Africa_Western
    {  439,  107,  530643,  530654,  530671, 11, 17, 18, }, // Alaska
    {  439,  135,  530689,  530700,  530717, 11, 17, 22, }, // Amazon
    {  439,  142,  530739,  530768,  530803, 29, 35, 36, }, // America_Central
    {  439,  158,  530839,  530868,  530903, 29, 35, 36, }, // America_Eastern
    {  439,  174,  530939,  530970,  531007, 31, 37, 38, }, // America_Mountain
    {  439,  191,  531045,  531076,  531113, 31, 37, 38, }, // America_Pacific
    {  439,  214,  531151,  531160,  531175,  9, 15, 16, }, // Apia
    {  439,  232,  531191,  531202,  531219, 11, 17, 18, }, // Arabian
    {  439,  240,  531237,  531250,  531269, 13, 19, 24, }, // Argentina
    {  439,  250,  531293,  531314,  531341, 21, 27, 32, }, // Argentina_Western
    {  439,  268,  531373,  531386,  531405, 13, 19, 24, }, // Armenia
    {  439,  286,  531429,  531442,  531461, 13, 19, 20, }, // Atlantic
    {  439,  295,  531481,  531500,  531525, 19, 25, 26, }, // Australia_Central
    {  439,  313,  531551,  531578,  531611, 27, 33, 34, }, // Australia_CentralWestern
    {  439,  338,  531645,  531664,  531689, 19, 25, 26, }, // Australia_Eastern
    {  439,  356,  531715,  531736,  531763, 21, 27, 28, }, // Australia_Western
    {  439,  374,  531791,  531806,  531827, 15, 21, 26, }, // Azerbaijan
    {  439,  385,  531853,  531862,  531877,  9, 15, 20, }, // Azores
    {  439,  397,  531897,  531912,  531933, 15, 21, 22, }, // Bangladesh
    {  439,  415,       0,  531955,       0,  0, 10,  0, }, // Bhutan
    {  439,  422,       0,  531965,       0,  0, 18,  0, }, // Bolivia
    {  439,  437,  531983,  531996,  532015, 13, 19, 24, }, // Brasilia
    {  439,  454,       0,  532039,       0,  0, 21,  0, }, // Brunei
    {  439,  461,  532060,  532073,  532092, 13, 19, 24, }, // Cape_Verde
    {  439,  478,       0,  532116,       0,  0, 18,  0, }, // Chamorro
    {  439,  487,  532134,  532145,  532162, 11, 17, 18, }, // Chatham
    {  439,  495,  532180,  532190,  532206, 10, 16, 21, }, // Chile
    {  439,  501,  532227,  532238,  532255, 11, 17, 18, }, // China
    {  439,  507,  532273,  532288,  532309, 15, 21, 26, }, // Choibalsan
    {  439,  518,       0,  532335,       0,  0, 19,  0, }, // Christmas
    {  439,  528,       0,  532354,       0,  0, 18,  0, }, // Cocos
    {  439,  534,  532372,  532385,  532404, 13, 19, 24, }, // Colombia
    {  439,  543,  532428,  532444,  532466, 16, 22, 31, }, // Cook
    {  439,  548,  532497,  532506,  532521,  9, 15, 16, }, // Cuba
    {  439,  559,       0,  532537,       0,  0, 10,  0, }, // Davis
    {  439,  575,       0,  532547,       0,  0, 19,  0, }, // DumontDUrville
    {  439,  612,       0,  532566,       0,  0, 13,  0, }, // East_Timor
    {  439,  623,  532579,  532588,  532603,  9, 15, 20, }, // Easter
    {  439,  630,       0,  532623,       0,  0, 13,  0, }, // Ecuador
    {  439,  638,  532636,  532652,  532674, 16, 22, 27, }, // Europe_Central
    {  439,  653,  532701,  532717,  532739, 16, 22, 27, }, // Europe_Eastern
    {  439,  668,       0,  532766,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  439,  691,  532786,  532804,  532828, 18, 24, 29, }, // Europe_Western
    {  439,  706,  532857,  532871,  532891, 14, 20, 25, }, // Falkland
    {  439,  715,  532916,  532925,  532940,  9, 15, 20, }, // Fiji
    {  439,  720,       0,  532960,       0,  0, 18,  0, }, // French_Guiana
    {  439,  734,       0,  532978,       0,  0, 30,  0, }, // French_Southern
    {  439,  757,       0,  533008,       0,  0, 14,  0, }, // Galapagos
    {  439,  767,       0,  533022,       0,  0, 12,  0, }, // Gambier
    {  439,  775,  533034,  533045,  533062, 11, 17, 22, }, // Georgia
    {  439,  783,       0,  533084,       0,  0, 19,  0, }, // Gilbert_Islands
    {  439,  799,       0,  533103,       0,  0, 17,  0, }, // GMT
    {  439,  841,  533120,  533138,  533162, 18, 24, 29, }, // Greenland_Eastern
    {  439,  859,  533191,  533210,  533235, 19, 25, 30, }, // Greenland_Western
    {  439,  882,       0,  533265,       0,  0, 16,  0, }, // Gulf
    {  439,  887,       0,  533281,       0,  0, 11,  0, }, // Guyana
    {  439,  894,  533292,  533312,  533338, 20, 26, 27, }, // Hawaii_Aleutian
    {  439,  910,  533365,  533379,  533399, 14, 20, 25, }, // Hong_Kong
    {  439,  920,  533424,  533433,  533448,  9, 15, 20, }, // Hovd
    {  439,  925,       0,  533468,       0,  0, 16,  0, }, // India
    {  439,  931,       0,  533484,       0,  0, 18,  0, }, // Indian_Ocean
    {  439,  944,       0,  533502,       0,  0, 15,  0, }, // Indochina
    {  439,  954,       0,  533517,       0,  0, 22,  0, }, // Indonesia_Central
    {  439,  972,       0,  533539,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  439,  990,       0,  533561,       0,  0, 24,  0, }, // Indonesia_Western
    {  439, 1008,  533585,  533594,  533609,  9, 15, 16, }, // Iran
    {  439, 1019,  533625,  533637,  533655, 12, 18, 23, }, // Irkutsk
    {  439, 1027,  533678,  533688,  533704, 10, 16, 17, }, // Israel
    {  439, 1034,  533721,  533731,  533747, 10, 16, 17, }, // Japan
    {  439, 1058,       0,  533764,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  439, 1077,       0,  533782,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  439, 1106,  533802,  533812,  533828, 10, 16, 17, }, // Korea
    {  439, 1112,       0,  533845,       0,  0, 12,  0, }, // Kosrae
    {  439, 1119,  533857,  533872,  533893, 15, 21, 26, }, // Krasnoyarsk
    {  439, 1151,       0,  533919,       0,  0, 13,  0, }, // Kyrgystan
    {  439, 1175,       0,  533932,       0,  0, 17,  0, }, // Line_Islands
    {  439, 1188,  533949,  533961,  533979, 12, 18, 19, }, // Lord_Howe
    {  439, 1204,  533998,  534010,  534028, 12, 18, 23, }, // Magadan
    {  439, 1219,       0,  534051,       0,  0, 13,  0, }, // Malaysia
    {  439, 1228,       0,  534064,       0,  0, 12,  0, }, // Maldives
    {  439, 1237,       0,  534076,       0,  0, 14,  0, }, // Marquesas
    {  439, 1247,       0,  534090,       0,  0, 19,  0, }, // Marshall_Islands
    {  439, 1264,  534109,  534122,  534141, 13, 19, 24, }, // Mauritius
    {  439, 1274,       0,  534165,       0,  0, 11,  0, }, // Mawson
    {  439, 1281,  534176,  534197,  534224, 21, 27, 28, }, // Mexico_Pacific
    {  439, 1296,  534252,  534265,  534284, 13, 19, 24, }, // Mongolia
    {  439, 1305,  534308,  534319,  534336, 11, 17, 22, }, // Moscow
    {  439, 1312,       0,  534358,       0,  0, 11,  0, }, // Myanmar
    {  439, 1320,       0,  534369,       0,  0, 10,  0, }, // Nauru
    {  439, 1326,       0,  534379,       0,  0, 10,  0, }, // Nepal
    {  439, 1332,  534389,  534407,  534431, 18, 24, 29, }, // New_Caledonia
    {  439, 1346,  534460,  534475,  534496, 15, 21, 22, }, // New_Zealand
    {  439, 1358,  534518,  534535,  534558, 17, 23, 24, }, // Newfoundland
    {  439, 1371,       0,  534582,       0,  0,  9,  0, }, // Niue
    {  439, 1376,  534591,  534612,  534639, 21, 27, 28, }, // Norfolk
    {  439, 1384,  534667,  534691,  534721, 24, 30, 35, }, // Noronha
    {  439, 1406,  534756,  534771,  534792, 15, 21, 26, }, // Novosibirsk
    {  439, 1418,  534818,  534827,  534842,  9, 15, 20, }, // Omsk
    {  439, 1428,  534862,  534875,  534894, 13, 19, 24, }, // Pakistan
    {  439, 1437,       0,  534918,       0,  0, 10,  0, }, // Palau
    {  439, 1443,       0,  534928,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  439, 1460,  534947,  534960,  534979, 13, 19, 24, }, // Paraguay
    {  439, 1469,  535003,  535012,  535027,  9, 15, 20, }, // Peru
    {  439, 1474,  535047,  535059,  535077, 12, 18, 22, }, // Philippines
    {  439, 1486,       0,  535099,       0,  0, 19,  0, }, // Phoenix_Islands
    {  439, 1502,  535118,  535142,  535173, 24, 31, 32, }, // Pierre_Miquelon
    {  439, 1518,       0,  535205,       0,  0, 11,  0, }, // Pitcairn
    {  439, 1527,       0,  535216,       0,  0, 11,  0, }, // Ponape
    {  439, 1534,       0,  535227,       0,  0, 14,  0, }, // Pyongyang
    {  439, 1554,       0,  535241,       0,  0, 13,  0, }, // Reunion
    {  439, 1562,       0,  535254,       0,  0, 12,  0, }, // Rothera
    {  439, 1570,  535266,  535279,  535298, 13, 19, 24, }, // Sakhalin
    {  439, 1596,  535322,  535332,  535348, 10, 16, 17, }, // Samoa
    {  439, 1602,       0,  535365,       0,  0, 13,  0, }, // Seychelles
    {  439, 1624,       0,  535378,       0,  0, 12,  0, }, // Singapore
    {  439, 1634,       0,  535390,       0,  0, 21,  0, }, // Solomon
    {  439, 1642,       0,  535411,       0,  0, 16,  0, }, // South_Georgia
    {  439, 1656,       0,  535427,       0,  0, 12,  0, }, // Suriname
    {  439, 1676,       0,  535439,       0,  0, 10,  0, }, // Syowa
    {  439, 1682,       0,  535449,       0,  0, 11,  0, }, // Tahiti
    {  439, 1689,  535460,  535471,  535488, 11, 17, 18, }, // Taipei
    {  439, 1696,       0,  535506,       0,  0, 15,  0, }, // Tajikistan
    {  439, 1724,       0,  535521,       0,  0, 12,  0, }, // Tokelau
    {  439, 1732,  535533,  535544,  535561, 11, 17, 22, }, // Tonga
    {  439, 1738,       0,  535583,       0,  0,  9,  0, }, // Truk
    {  439, 1750,  535592,  535608,  535630, 16, 22, 27, }, // Turkmenistan
    {  439, 1763,       0,  535657,       0,  0, 11,  0, }, // Tuvalu
    {  439, 1777,  535668,  535681,  535700, 13, 19, 24, }, // Uruguay
    {  439, 1792,  535724,  535739,  535760, 15, 21, 26, }, // Uzbekistan
    {  439, 1803,  535786,  535798,  535816, 12, 18, 18, }, // Vanuatu
    {  439, 1811,       0,  535834,       0,  0, 15,  0, }, // Venezuela
    {  439, 1821,  535849,  535865,  535887, 16, 22, 27, }, // Vladivostok
    {  439, 1833,  535914,  535928,  535948, 14, 20, 25, }, // Volgograd
    {  439, 1843,       0,  535973,       0,  0, 12,  0, }, // Vostok
    {  439, 1850,       0,  535985,       0,  0, 15,  0, }, // Wake
    {  439, 1855,       0,  536000,       0,  0, 20,  0, }, // Wallis
    {  439, 1862,  536020,  536032,  536050, 12, 18, 23, }, // Yakutsk
    {  439, 1870,  536073,  536090,  536113, 17, 23, 28, }, // Yekaterinburg
    {  439, 1892,       0,  536141,       0,  0, 10,  0, }, // Yukon
    {  440,   17,       0,  536151,       0,  0, 34,  0, }, // Africa_Central Nko/Nko/Guinea
    {  440,   32,       0,  536185,       0,  0, 33,  0, }, // Africa_Eastern
    {  440,   65,       0,  536218,       0,  0, 35,  0, }, // Africa_Southern
    {  440,   81,  536253,  536281,  536315, 28, 34, 34, }, // Africa_Western
    {  440,  461,  536349,  536373,  536403, 24, 30, 30, }, // Cape_Verde
    {  440,  734,       0,  536433,       0,  0, 50,  0, }, // French_Southern
    {  440,  799,       0,  536483,       0,  0, 16,  0, }, // GMT
    {  440,  931,       0,  536499,       0,  0, 25,  0, }, // Indian_Ocean
    {  440, 1264,  536524,  536539,  536560, 15, 21, 21, }, // Mauritius
    {  440, 1554,       0,  536581,       0,  0, 18,  0, }, // Reunion
    {  440, 1602,       0,  536599,       0,  0, 17,  0, }, // Seychelles
    {  441,  142,  536616,  536629,  536654, 13, 25, 21, }, // America_Central Northern Luri/Arabic/Iran
    {  443,  638,  536675,  536693,  536718, 18, 25, 24, }, // Europe_Central Northern Sami/Latin/Norway
    {  443,  653,  536742,  536761,  536787, 19, 26, 25, }, // Europe_Eastern
    {  443,  691,  536812,  536830,  536855, 18, 25, 24, }, // Europe_Western
    {  443,  799,       0,  536879,       0,  0, 21,  0, }, // GMT
    {  443, 1305,  536900,  536911,  536929, 11, 18, 17, }, // Moscow
    {  444,    5,       0,  536946,       0,  0, 17,  0, }, // Afghanistan Northern Sami/Latin/Finland
    {  444,   17,       0,  536963,       0,  0, 18,  0, }, // Africa_Central
    {  444,   32,       0,  536981,       0,  0, 19,  0, }, // Africa_Eastern
    {  444,   65,       0,  537000,       0,  0, 23,  0, }, // Africa_Southern
    {  444,   81,  537023,  537041,  537064, 18, 23, 24, }, // Africa_Western
    {  444,  107,  537088,  537099,  537115, 11, 16, 17, }, // Alaska
    {  444,  135,  537132,  537144,  537161, 12, 17, 18, }, // Amazon
    {  444,  142,  536707,  537179,  537196, 11, 17, 18, }, // America_Central
    {  444,  158,  537214,  537226,  537243, 12, 17, 18, }, // America_Eastern
    {  444,  174,  537261,  537272,  537288, 11, 16, 17, }, // America_Mountain
    {  444,  191,  537305,  537319,  537338, 14, 19, 20, }, // America_Pacific
    {  444,  214,  537358,  537367,  537381,  9, 14, 15, }, // Apia
    {  444,  232,  537396,  537407,  537423, 11, 16, 17, }, // Arabian
    {  444,  240,  537440,  537454,  537473, 14, 19, 20, }, // Argentina
    {  444,  250,  537493,  537513,  537538, 20, 25, 26, }, // Argentina_Western
    {  444,  268,  537564,  537576,  537593, 12, 17, 18, }, // Armenia
    {  444,  286,  537611,  537626,  537646, 15, 20, 21, }, // Atlantic
    {  444,  295,  537667,  537687,  537712, 20, 25, 26, }, // Australia_Central
    {  444,  313,  537738,  537770,  537807, 32, 37, 38, }, // Australia_CentralWestern
    {  444,  338,  537845,  537866,  537892, 21, 26, 27, }, // Australia_Eastern
    {  444,  356,  537919,  537939,  537964, 20, 25, 26, }, // Australia_Western
    {  444,  374,  537990,  538006,  538027, 16, 21, 22, }, // Azerbaijan
    {  444,  385,  538049,  538061,  538078, 12, 17, 18, }, // Azores
    {  444,  397,  538096,  538112,  538133, 16, 21, 22, }, // Bangladesh
    {  444,  415,       0,  538155,       0,  0, 12,  0, }, // Bhutan
    {  444,  422,       0,  538167,       0,  0, 12,  0, }, // Bolivia
    {  444,  437,  538179,  538192,  538210, 13, 18, 19, }, // Brasilia
    {  444,  454,       0,  538229,       0,  0, 23,  0, }, // Brunei
    {  444,  461,  538252,  538266,  538285, 14, 19, 20, }, // Cape_Verde
    {  444,  478,       0,  538305,       0,  0, 17,  0, }, // Chamorro
    {  444,  487,  538322,  538335,  538353, 13, 18, 19, }, // Chatham
    {  444,  495,  538372,  538382,  538397, 10, 15, 16, }, // Chile
    {  444,  501,  538413,  538424,  538440, 11, 16, 17, }, // China
    {  444,  507,  538457,  538473,  538494, 16, 21, 22, }, // Choibalsan
    {  444,  518,       0,  538516,       0,  0, 16,  0, }, // Christmas
    {  444,  528,       0,  538532,       0,  0, 17,  0, }, // Cocos
    {  444,  534,  538549,  538562,  538580, 13, 18, 19, }, // Colombia
    {  444,  543,  538599,  538615,  538636, 16, 21, 34, }, // Cook
    {  444,  548,  538670,  538679,  538693,  9, 14, 15, }, // Cuba
    {  444,  559,       0,  538708,       0,  0, 11,  0, }, // Davis
    {  444,  575,       0,  538719,       0,  0, 21,  0, }, // DumontDUrville
    {  444,  612,       0,  538740,       0,  0, 18,  0, }, // East_Timor
    {  444,  623,  538758,  538775,  538797, 17, 22, 23, }, // Easter
    {  444,  630,       0,  538820,       0,  0, 13,  0, }, // Ecuador
    {  444,  638,  538833,  538851,  538874, 18, 23, 24, }, // Europe_Central
    {  444,  653,  538898,  538917,  538941, 19, 24, 25, }, // Europe_Eastern
    {  444,  668,       0,  538966,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  444,  691,  538997,  539015,  539038, 18, 23, 24, }, // Europe_Western
    {  444,  706,  539062,  539082,  539107, 20, 25, 26, }, // Falkland
    {  444,  715,  539133,  539142,  539156,  9, 14, 15, }, // Fiji
    {  444,  720,       0,  539171,       0,  0, 23,  0, }, // French_Guiana
    {  444,  734,       0,  539194,       0,  0, 35,  0, }, // French_Southern
    {  444,  757,       0,  539229,       0,  0, 15,  0, }, // Galapagos
    {  444,  767,       0,  539244,       0,  0, 13,  0, }, // Gambier
    {  444,  775,  539257,  539269,  539286, 12, 17, 18, }, // Georgia
    {  444,  783,       0,  539304,       0,  0, 19,  0, }, // Gilbert_Islands
    {  444,  799,       0,  539323,       0,  0, 15,  0, }, // GMT
    {  444,  841,  539338,  539362,  539391, 24, 29, 30, }, // Greenland_Eastern
    {  444,  859,  539421,  539444,  539472, 23, 28, 29, }, // Greenland_Western
    {  444,  882,       0,  539501,       0,  0, 15,  0, }, // Gulf
    {  444,  887,       0,  539183,       0,  0, 11,  0, }, // Guyana
    {  444,  894,  539516,  539538,  539565, 22, 27, 28, }, // Hawaii_Aleutian
    {  444,  910,  539593,  539608,  539628, 15, 20, 21, }, // Hong_Kong
    {  444,  920,  539649,  539659,  539674, 10, 15, 16, }, // Hovd
    {  444,  925,       0,  539690,       0,  0, 15,  0, }, // India
    {  444,  931,       0,  539705,       0,  0, 13,  0, }, // Indian_Ocean
    {  444,  944,       0,  539718,       0,  0, 15,  0, }, // Indochina
    {  444,  954,       0,  539733,       0,  0, 20,  0, }, // Indonesia_Central
    {  444,  972,       0,  539753,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  444,  990,       0,  539774,       0,  0, 20,  0, }, // Indonesia_Western
    {  444, 1008,  539794,  539804,  539819, 10, 15, 16, }, // Iran
    {  444, 1019,  539835,  539847,  539864, 12, 17, 18, }, // Irkutsk
    {  444, 1027,  539882,  539894,  539911, 12, 17, 18, }, // Israel
    {  444, 1034,  539929,  539940,  539956, 11, 16, 17, }, // Japan
    {  444, 1058,       0,  539973,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  444, 1077,       0,  539995,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  444, 1106,  540016,  540026,  540041, 10, 15, 16, }, // Korea
    {  444, 1112,       0,  540057,       0,  0, 12,  0, }, // Kosrae
    {  444, 1119,  540069,  540086,  540108, 17, 22, 23, }, // Krasnoyarsk
    {  444, 1151,       0,  540131,       0,  0, 13,  0, }, // Kyrgystan
    {  444, 1175,       0,  540144,       0,  0, 16,  0, }, // Line_Islands
    {  444, 1188,  540160,  540174,  540193, 14, 19, 20, }, // Lord_Howe
    {  444, 1204,  540213,  540226,  540244, 13, 18, 19, }, // Magadan
    {  444, 1219,       0,  540263,       0,  0, 12,  0, }, // Malaysia
    {  444, 1228,       0,  540275,       0,  0, 17,  0, }, // Maldives
    {  444, 1237,       0,  540292,       0,  0, 17,  0, }, // Marquesas
    {  444, 1247,       0,  540309,       0,  0, 20,  0, }, // Marshall_Islands
    {  444, 1264,  540329,  540344,  540364, 15, 20, 21, }, // Mauritius
    {  444, 1274,       0,  540385,       0,  0, 12,  0, }, // Mawson
    {  444, 1281,  540397,  540419,  540446, 22, 27, 28, }, // Mexico_Pacific
    {  444, 1296,  540474,  540490,  540511, 16, 21, 22, }, // Mongolia
    {  444, 1305,  540533,  540544,  540560, 11, 16, 17, }, // Moscow
    {  444, 1312,       0,  540577,       0,  0, 13,  0, }, // Myanmar
    {  444, 1320,       0,  540590,       0,  0, 10,  0, }, // Nauru
    {  444, 1326,       0,  540600,       0,  0, 11,  0, }, // Nepal
    {  444, 1332,  540611,  540630,  540654, 19, 24, 25, }, // New_Caledonia
    {  444, 1346,  540679,  540696,  540718, 17, 22, 23, }, // New_Zealand
    {  444, 1358,  540741,  540759,  540782, 18, 23, 24, }, // Newfoundland
    {  444, 1371,       0,  540806,       0,  0, 10,  0, }, // Niue
    {  444, 1376,       0,  540816,       0,  0, 17,  0, }, // Norfolk
    {  444, 1384,  540833,  540857,  540886, 24, 29, 30, }, // Noronha
    {  444, 1406,  540916,  540933,  540955, 17, 22, 23, }, // Novosibirsk
    {  444, 1418,  540978,  540988,  541003, 10, 15, 16, }, // Omsk
    {  444, 1428,  541019,  541033,  541052, 14, 19, 20, }, // Pakistan
    {  444, 1437,       0,  541072,       0,  0, 11,  0, }, // Palau
    {  444, 1443,       0,  541083,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  444, 1460,  541105,  541119,  541138, 14, 19, 20, }, // Paraguay
    {  444, 1469,  541158,  541167,  541181,  9, 14, 15, }, // Peru
    {  444, 1474,  541196,  541214,  541237, 18, 23, 24, }, // Philippines
    {  444, 1486,       0,  541261,       0,  0, 19,  0, }, // Phoenix_Islands
    {  444, 1502,  541280,  541305,  541335, 25, 30, 31, }, // Pierre_Miquelon
    {  444, 1518,       0,  541366,       0,  0, 20,  0, }, // Pitcairn
    {  444, 1527,       0,  541386,       0,  0, 11,  0, }, // Ponape
    {  444, 1534,       0,  541397,       0,  0, 15,  0, }, // Pyongyang
    {  444, 1554,       0,  541412,       0,  0, 13,  0, }, // Reunion
    {  444, 1562,       0,  541425,       0,  0, 12,  0, }, // Rothera
    {  444, 1570,  541437,  541450,  541468, 13, 18, 19, }, // Sakhalin
    {  444, 1596,  541487,  541497,  541512, 10, 15, 16, }, // Samoa
    {  444, 1602,       0,  541528,       0,  0, 16,  0, }, // Seychelles
    {  444, 1624,       0,  541544,       0,  0, 19,  0, }, // Singapore
    {  444, 1634,       0,  541563,       0,  0, 19,  0, }, // Solomon
    {  444, 1642,       0,  541582,       0,  0, 18,  0, }, // South_Georgia
    {  444, 1656,       0,  541600,       0,  0, 13,  0, }, // Suriname
    {  444, 1676,       0,  541613,       0,  0, 10,  0, }, // Syowa
    {  444, 1682,       0,  541623,       0,  0, 11,  0, }, // Tahiti
    {  444, 1689,  541634,  541646,  541663, 12, 17, 18, }, // Taipei
    {  444, 1696,       0,  541681,       0,  0, 16,  0, }, // Tajikistan
    {  444, 1724,       0,  541697,       0,  0, 13,  0, }, // Tokelau
    {  444, 1732,  541710,  541720,  541735, 10, 15, 16, }, // Tonga
    {  444, 1738,       0,  541751,       0,  0, 11,  0, }, // Truk
    {  444, 1750,  541762,  541780,  541803, 18, 23, 24, }, // Turkmenistan
    {  444, 1763,       0,  541827,       0,  0, 11,  0, }, // Tuvalu
    {  444, 1777,  541838,  541851,  541869, 13, 18, 19, }, // Uruguay
    {  444, 1792,  541888,  541904,  541925, 16, 21, 22, }, // Uzbekistan
    {  444, 1803,  541947,  541959,  541976, 12, 17, 18, }, // Vanuatu
    {  444, 1811,       0,  541994,       0,  0, 14,  0, }, // Venezuela
    {  444, 1821,  542008,  542025,  542047, 17, 22, 23, }, // Vladivostok
    {  444, 1833,  542070,  542085,  542105, 15, 20, 21, }, // Volgograd
    {  444, 1843,       0,  542126,       0,  0, 12,  0, }, // Vostok
    {  444, 1850,       0,  542138,       0,  0, 14,  0, }, // Wake
    {  444, 1855,       0,  542152,       0,  0, 22,  0, }, // Wallis
    {  444, 1862,  542174,  542186,  542203, 12, 17, 18, }, // Yakutsk
    {  444, 1870,  542221,  542240,  542264, 19, 24, 25, }, // Yekaterinburg
    {  448,    0,  195717,  542289,  542303,  8, 14, 14, }, // Acre Norwegian Bokmal/Latin/Norway
    {  448,    5,       0,  542317,       0,  0, 12,  0, }, // Afghanistan
    {  448,   17,       0,  542329,       0,  0, 20,  0, }, // Africa_Central
    {  448,   32,       0,  542349,       0,  0, 16,  0, }, // Africa_Eastern
    {  448,   65,       0,  542365,       0,  0, 16,  0, }, // Africa_Southern
    {  448,   81,  542381,  542398,  542421, 17, 23, 23, }, // Africa_Western
    {  448,  107,  542444,  542456,  542474, 12, 18, 18, }, // Alaska
    {  448,  128,  195922,  542492,  542511, 10, 19, 17, }, // Almaty
    {  448,  135,  542528,  542549,  542571, 21, 22, 22, }, // Amazon
    {  448,  142,  542593,  542631,  542670, 38, 39, 39, }, // America_Central
    {  448,  158,  542709,  542751,  542794, 42, 43, 43, }, // America_Eastern
    {  448,  174,  542837,  542871,  542906, 34, 35, 35, }, // America_Mountain
    {  448,  191,  542941,  542990,  543040, 49, 50, 50, }, // America_Pacific
    {  448,  207,  543090,  543110,  543136, 20, 26, 26, }, // Anadyr
    {  448,  214,  543162,  543179,  543197, 17, 18, 18, }, // Apia
    {  448,  219,  196273,  543215,  543233,  9, 18, 16, }, // Aqtau
    {  448,  225,  196312,  543249,  543268, 10, 19, 17, }, // Aqtobe
    {  448,  232,  543285,  543296,  543315, 11, 19, 17, }, // Arabian
    {  448,  240,  196456,  196474,  196498, 14, 20, 20, }, // Argentina
    {  448,  250,  543332,  543350,  543374, 18, 24, 24, }, // Argentina_Western
    {  448,  268,  543398,  543409,  543426, 11, 17, 17, }, // Armenia
    {  448,  286,  543443,  543494,  543546, 51, 52, 52, }, // Atlantic
    {  448,  295,  543598,  543618,  543644, 20, 26, 26, }, // Australia_Central
    {  448,  313,  543670,  543695,  543726, 25, 31, 31, }, // Australia_CentralWestern
    {  448,  338,  543757,  543773,  543795, 16, 22, 22, }, // Australia_Eastern
    {  448,  356,  543817,  543834,  543857, 17, 23, 23, }, // Australia_Western
    {  448,  374,  543880,  543898,  543922, 18, 24, 24, }, // Azerbaijan
    {  448,  385,  543946,  543957,  543974, 11, 17, 17, }, // Azores
    {  448,  397,  543991,  544008,  544031, 17, 23, 23, }, // Bangladesh
    {  448,  415,       0,  544054,       0,  0, 12,  0, }, // Bhutan
    {  448,  422,       0,  544066,       0,  0, 14,  0, }, // Bolivia
    {  448,  437,  544080,  544101,  544123, 21, 22, 22, }, // Brasilia
    {  448,  454,       0,  544145,       0,  0, 30,  0, }, // Brunei
    {  448,  461,  544175,  544190,  544211, 15, 21, 21, }, // Cape_Verde
    {  448,  472,       0,  544232,       0,  0,  9,  0, }, // Casey
    {  448,  478,       0,  544241,       0,  0, 21,  0, }, // Chamorro
    {  448,  487,  544262,  544282,  544303, 20, 21, 21, }, // Chatham
    {  448,  495,  544324,  544336,  544354, 12, 18, 18, }, // Chile
    {  448,  501,  544372,  544384,  544402, 12, 18, 18, }, // China
    {  448,  507,  544420,  544444,  544469, 24, 25, 25, }, // Choibalsan
    {  448,  518,       0,  544494,       0,  0, 25,  0, }, // Christmas
    {  448,  528,       0,  544519,       0,  0, 23,  0, }, // Cocos
    {  448,  534,  544542,  544557,  544578, 15, 21, 21, }, // Colombia
    {  448,  543,  544599,  544621,  544644, 22, 23, 28, }, // Cook
    {  448,  548,  544672,  544683,  544700, 11, 17, 17, }, // Cuba
    {  448,  559,       0,  544717,       0,  0, 18,  0, }, // Davis
    {  448,  575,       0,  544735,       0,  0, 29,  0, }, // DumontDUrville
    {  448,  612,       0,  544764,       0,  0, 17,  0, }, // East_Timor
    {  448,  623,  544781,  544802,  544824, 21, 22, 22, }, // Easter
    {  448,  630,       0,  544846,       0,  0, 16,  0, }, // Ecuador
    {  448,  638,  544862,  544882,  544908, 20, 26, 26, }, // Europe_Central
    {  448,  653,  544934,  544950,  544972, 16, 22, 22, }, // Europe_Eastern
    {  448,  668,       0,  544994,       0,  0, 22,  0, }, // Europe_Further_Eastern
    {  448,  691,  545016,  545033,  545056, 17, 23, 23, }, // Europe_Western
    {  448,  706,  545079,  545106,  545134, 27, 28, 28, }, // Falkland
    {  448,  715,  545162,  545174,  545192, 12, 18, 18, }, // Fiji
    {  448,  720,       0,  545210,       0,  0, 26,  0, }, // French_Guiana
    {  448,  734,       0,  545236,       0,  0, 38,  0, }, // French_Southern
    {  448,  757,       0,  545274,       0,  0, 27,  0, }, // Galapagos
    {  448,  767,       0,  545301,       0,  0, 20,  0, }, // Gambier
    {  448,  775,  545321,  545333,  545351, 12, 18, 18, }, // Georgia
    {  448,  783,       0,  545369,       0,  0, 25,  0, }, // Gilbert_Islands
    {  448,  799,       0,  545394,       0,  0, 19,  0, }, // GMT
    {  448,  841,  545413,  545430,  545453, 17, 23, 23, }, // Greenland_Eastern
    {  448,  859,  545476,  545494,  545518, 18, 24, 24, }, // Greenland_Western
    {  448,  877,       0,  545542,       0,  0,  8,  0, }, // Guam
    {  448,  882,       0,  545550,       0,  0, 24,  0, }, // Gulf
    {  448,  887,       0,  545574,       0,  0, 11,  0, }, // Guyana
    {  448,  894,  545585,  545616,  545648, 31, 32, 32, }, // Hawaii_Aleutian
    {  448,  910,  545680,  545701,  545723, 21, 22, 22, }, // Hong_Kong
    {  448,  920,  545745,  545763,  545782, 18, 19, 19, }, // Hovd
    {  448,  925,       0,  545801,       0,  0, 10,  0, }, // India
    {  448,  931,       0,  545811,       0,  0, 23,  0, }, // Indian_Ocean
    {  448,  944,       0,  545834,       0,  0, 16,  0, }, // Indochina
    {  448,  954,       0,  545850,       0,  0, 21,  0, }, // Indonesia_Central
    {  448,  972,       0,  545871,       0,  0, 17,  0, }, // Indonesia_Eastern
    {  448,  990,       0,  545888,       0,  0, 18,  0, }, // Indonesia_Western
    {  448, 1008,  545906,  545916,  545932, 10, 16, 16, }, // Iran
    {  448, 1019,  545948,  545968,  545989, 20, 21, 21, }, // Irkutsk
    {  448, 1027,  546010,  546022,  546040, 12, 18, 18, }, // Israel
    {  448, 1034,  546058,  546069,  546086, 11, 17, 17, }, // Japan
    {  448, 1040,  546103,  546143,  546189, 40, 46, 46, }, // Kamchatka
    {  448, 1058,       0,  546235,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  448, 1077,       0,  546254,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  448, 1106,  546274,  546286,  546304, 12, 18, 18, }, // Korea
    {  448, 1112,       0,  546322,       0,  0, 19,  0, }, // Kosrae
    {  448, 1119,  546341,  546365,  546390, 24, 25, 25, }, // Krasnoyarsk
    {  448, 1151,       0,  546415,       0,  0, 13,  0, }, // Kyrgystan
    {  448, 1161,       0,  546428,       0,  0,  9,  0, }, // Lanka
    {  448, 1175,       0,  546437,       0,  0, 23,  0, }, // Line_Islands
    {  448, 1188,  546460,  546486,  546513, 26, 27, 27, }, // Lord_Howe
    {  448, 1198,  546540,  546549,  546567,  9, 18, 16, }, // Macau
    {  448, 1204,  546583,  546603,  546624, 20, 21, 21, }, // Magadan
    {  448, 1219,       0,  546645,       0,  0, 13,  0, }, // Malaysia
    {  448, 1228,       0,  546658,       0,  0, 13,  0, }, // Maldives
    {  448, 1237,       0,  546671,       0,  0, 27,  0, }, // Marquesas
    {  448, 1247,       0,  546698,       0,  0, 17,  0, }, // Marshall_Islands
    {  448, 1264,  546715,  546728,  546747, 13, 19, 19, }, // Mauritius
    {  448, 1274,       0,  546766,       0,  0, 19,  0, }, // Mawson
    {  448, 1281,  546785,  546830,  546876, 45, 46, 46, }, // Mexico_Pacific
    {  448, 1296,  546922,  546945,  546969, 23, 24, 24, }, // Mongolia
    {  448, 1305,  546993,  547012,  547032, 19, 20, 20, }, // Moscow
    {  448, 1312,       0,  547052,       0,  0, 13,  0, }, // Myanmar
    {  448, 1320,       0,  547065,       0,  0, 11,  0, }, // Nauru
    {  448, 1326,       0,  547076,       0,  0, 11,  0, }, // Nepal
    {  448, 1332,  547087,  547100,  547119, 13, 19, 19, }, // New_Caledonia
    {  448, 1346,  547138,  547154,  547176, 16, 22, 22, }, // New_Zealand
    {  448, 1358,  547198,  547223,  547249, 25, 26, 26, }, // Newfoundland
    {  448, 1371,       0,  547275,       0,  0, 17,  0, }, // Niue
    {  448, 1376,  547292,  547315,  547339, 23, 24, 24, }, // Norfolk
    {  448, 1384,  547363,  547395,  547428, 32, 33, 33, }, // Noronha
    {  448, 1392,       0,  547461,       0,  0, 18,  0, }, // North_Mariana
    {  448, 1406,  547479,  547503,  547528, 24, 25, 25, }, // Novosibirsk
    {  448, 1418,  547553,  547570,  547588, 17, 18, 18, }, // Omsk
    {  448, 1428,  547606,  547620,  547640, 14, 20, 20, }, // Pakistan
    {  448, 1437,       0,  547660,       0,  0, 12,  0, }, // Palau
    {  448, 1443,       0,  547672,       0,  0, 12,  0, }, // Papua_New_Guinea
    {  448, 1460,  547684,  547700,  547722, 16, 22, 22, }, // Paraguay
    {  448, 1469,  547744,  547756,  547774, 12, 18, 18, }, // Peru
    {  448, 1474,  547792,  547806,  547826, 14, 20, 20, }, // Philippines
    {  448, 1486,       0,  547846,       0,  0, 25,  0, }, // Phoenix_Islands
    {  448, 1502,  547871,  547908,  547946, 37, 38, 38, }, // Pierre_Miquelon
    {  448, 1518,       0,  547984,       0,  0, 21,  0, }, // Pitcairn
    {  448, 1527,       0,  548005,       0,  0, 20,  0, }, // Ponape
    {  448, 1534,       0,  548025,       0,  0, 22,  0, }, // Pyongyang
    {  448, 1544,  200412,  548047,  548069, 13, 22, 20, }, // Qyzylorda
    {  448, 1554,       0,  548089,       0,  0, 20,  0, }, // Reunion
    {  448, 1562,       0,  548109,       0,  0, 20,  0, }, // Rothera
    {  448, 1570,  548129,  548150,  548172, 21, 22, 22, }, // Sakhalin
    {  448, 1579,  548194,  548214,  548240, 20, 26, 26, }, // Samara
    {  448, 1596,  548266,  548278,  548296, 12, 18, 18, }, // Samoa
    {  448, 1602,       0,  548314,       0,  0, 15,  0, }, // Seychelles
    {  448, 1624,       0,  548329,       0,  0, 14,  0, }, // Singapore
    {  448, 1634,       0,  548343,       0,  0, 13,  0, }, // Solomon
    {  448, 1642,       0,  548356,       0,  0, 24,  0, }, // South_Georgia
    {  448, 1656,       0,  548380,       0,  0, 13,  0, }, // Suriname
    {  448, 1676,       0,  548393,       0,  0, 18,  0, }, // Syowa
    {  448, 1682,       0,  548411,       0,  0, 12,  0, }, // Tahiti
    {  448, 1689,  548423,  548442,  548462, 19, 20, 20, }, // Taipei
    {  448, 1696,       0,  548482,       0,  0, 14,  0, }, // Tajikistan
    {  448, 1724,       0,  548496,       0,  0, 20,  0, }, // Tokelau
    {  448, 1732,  548516,  548528,  548546, 12, 18, 18, }, // Tonga
    {  448, 1738,       0,  548564,       0,  0, 23,  0, }, // Truk
    {  448, 1750,  548587,  548600,  548619, 13, 19, 19, }, // Turkmenistan
    {  448, 1763,       0,  548638,       0,  0, 11,  0, }, // Tuvalu
    {  448, 1777,  548649,  548664,  548685, 15, 21, 21, }, // Uruguay
    {  448, 1792,  548706,  548718,  548736, 12, 18, 18, }, // Uzbekistan
    {  448, 1803,  548754,  548767,  548786, 13, 19, 19, }, // Vanuatu
    {  448, 1811,       0,  548805,       0,  0, 16,  0, }, // Venezuela
    {  448, 1821,  548821,  548845,  548870, 24, 25, 25, }, // Vladivostok
    {  448, 1833,  548895,  548917,  548940, 22, 23, 23, }, // Volgograd
    {  448, 1843,       0,  548963,       0,  0, 19,  0, }, // Vostok
    {  448, 1850,       0,  548982,       0,  0, 24,  0, }, // Wake
    {  448, 1855,       0,  549006,       0,  0, 35,  0, }, // Wallis
    {  448, 1862,  549041,  549061,  549082, 20, 21, 21, }, // Yakutsk
    {  448, 1870,  549103,  549129,  549156, 26, 27, 27, }, // Yekaterinburg
    {  448, 1892,       0,  549183,       0,  0, 18,  0, }, // Yukon
    {  450,    0,  195717,  542289,  542303,  8, 14, 14, }, // Acre Norwegian Nynorsk/Latin/Norway
    {  450,    5,       0,  542317,       0,  0, 12,  0, }, // Afghanistan
    {  450,   17,       0,  542329,       0,  0, 20,  0, }, // Africa_Central
    {  450,   32,       0,  549201,       0,  0, 17,  0, }, // Africa_Eastern
    {  450,   65,       0,  542365,       0,  0, 16,  0, }, // Africa_Southern
    {  450,   81,  542381,  549218,  549243, 17, 25, 23, }, // Africa_Western
    {  450,  107,  542444,  542456,  549266, 12, 18, 18, }, // Alaska
    {  450,  128,  195922,  542492,  542511, 10, 19, 17, }, // Almaty
    {  450,  135,  542528,  542549,  549284, 21, 22, 22, }, // Amazon
    {  450,  142,  549306,  549340,  549375, 34, 35, 35, }, // America_Central
    {  450,  158,  549410,  549453,  549497, 43, 44, 43, }, // America_Eastern
    {  450,  174,  542837,  542871,  549540, 34, 35, 35, }, // America_Mountain
    {  450,  191,  549575,  549624,  549674, 49, 50, 50, }, // America_Pacific
    {  450,  207,  543090,  543110,  543136, 20, 26, 26, }, // Anadyr
    {  450,  214,  543162,  543179,  549724, 17, 18, 18, }, // Apia
    {  450,  219,  196273,  543215,  543233,  9, 18, 16, }, // Aqtau
    {  450,  225,  196312,  543249,  543268, 10, 19, 17, }, // Aqtobe
    {  450,  232,  543285,  549742,  549759, 11, 17, 17, }, // Arabian
    {  450,  240,  196456,  196474,  549776, 14, 20, 20, }, // Argentina
    {  450,  250,  543332,  543350,  549796, 18, 24, 24, }, // Argentina_Western
    {  450,  268,  543398,  543409,  549820, 11, 17, 17, }, // Armenia
    {  450,  286,  543443,  543494,  549837, 51, 52, 52, }, // Atlantic
    {  450,  295,  543598,  549889,  549917, 20, 28, 26, }, // Australia_Central
    {  450,  313,  543670,  549943,  549976, 25, 33, 31, }, // Australia_CentralWestern
    {  450,  338,  550007,  550024,  550049, 17, 25, 23, }, // Australia_Eastern
    {  450,  356,  543817,  550072,  550097, 17, 25, 23, }, // Australia_Western
    {  450,  374,  543880,  543898,  550120, 18, 24, 24, }, // Azerbaijan
    {  450,  385,  543946,  543957,  550144, 11, 17, 17, }, // Azores
    {  450,  397,  543991,  544008,  550161, 17, 23, 23, }, // Bangladesh
    {  450,  415,       0,  544054,       0,  0, 12,  0, }, // Bhutan
    {  450,  422,       0,  544066,       0,  0, 14,  0, }, // Bolivia
    {  450,  437,  544080,  544101,  550184, 21, 22, 22, }, // Brasilia
    {  450,  454,       0,  544145,       0,  0, 30,  0, }, // Brunei
    {  450,  461,  544175,  544190,  550206, 15, 21, 21, }, // Cape_Verde
    {  450,  472,       0,  544232,       0,  0,  9,  0, }, // Casey
    {  450,  478,       0,  544241,       0,  0, 21,  0, }, // Chamorro
    {  450,  487,  544262,  544282,  550227, 20, 21, 21, }, // Chatham
    {  450,  495,  544324,  544336,  550248, 12, 18, 18, }, // Chile
    {  450,  501,  544372,  544384,  550266, 12, 18, 18, }, // China
    {  450,  507,  544420,  544444,  550284, 24, 25, 25, }, // Choibalsan
    {  450,  518,       0,  544494,       0,  0, 25,  0, }, // Christmas
    {  450,  528,       0,  550309,       0,  0, 23,  0, }, // Cocos
    {  450,  534,  550332,  550347,  550368, 15, 21, 21, }, // Colombia
    {  450,  543,  550389,  550411,  550434, 22, 23, 23, }, // Cook
    {  450,  548,  550457,  550468,  550485, 11, 17, 17, }, // Cuba
    {  450,  559,       0,  544717,       0,  0, 18,  0, }, // Davis
    {  450,  575,       0,  550502,       0,  0, 29,  0, }, // DumontDUrville
    {  450,  612,       0,  550531,       0,  0, 18,  0, }, // East_Timor
    {  450,  623,  544781,  544802,  550549, 21, 22, 22, }, // Easter
    {  450,  630,       0,  544846,       0,  0, 16,  0, }, // Ecuador
    {  450,  638,  544862,  550571,  550599, 20, 28, 26, }, // Europe_Central
    {  450,  653,  550625,  550642,  550667, 17, 25, 23, }, // Europe_Eastern
    {  450,  668,       0,  550690,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  450,  691,  545016,  550713,  550738, 17, 25, 23, }, // Europe_Western
    {  450,  706,  550761,  550788,  550816, 27, 28, 28, }, // Falkland
    {  450,  715,  545162,  545174,  550844, 12, 18, 18, }, // Fiji
    {  450,  720,       0,  545210,       0,  0, 26,  0, }, // French_Guiana
    {  450,  734,       0,  550862,       0,  0, 38,  0, }, // French_Southern
    {  450,  757,       0,  550900,       0,  0, 27,  0, }, // Galapagos
    {  450,  767,       0,  545301,       0,  0, 20,  0, }, // Gambier
    {  450,  775,  545321,  545333,  550927, 12, 18, 18, }, // Georgia
    {  450,  783,       0,  550945,       0,  0, 25,  0, }, // Gilbert_Islands
    {  450,  799,       0,  545394,       0,  0, 19,  0, }, // GMT
    {  450,  841,  550970,  550988,  551012, 18, 24, 24, }, // Greenland_Eastern
    {  450,  859,  545476,  545494,  551036, 18, 24, 24, }, // Greenland_Western
    {  450,  877,       0,  545542,       0,  0,  8,  0, }, // Guam
    {  450,  882,       0,  545550,       0,  0, 24,  0, }, // Gulf
    {  450,  887,       0,  545574,       0,  0, 11,  0, }, // Guyana
    {  450,  894,  545585,  545616,  551060, 31, 32, 32, }, // Hawaii_Aleutian
    {  450,  910,  551092,  551112,  551138, 20, 26, 26, }, // Hong_Kong
    {  450,  920,  545745,  545763,  551164, 18, 19, 19, }, // Hovd
    {  450,  925,       0,  545801,       0,  0, 10,  0, }, // India
    {  450,  931,       0,  545811,       0,  0, 23,  0, }, // Indian_Ocean
    {  450,  944,       0,  545834,       0,  0, 16,  0, }, // Indochina
    {  450,  954,       0,  545850,       0,  0, 21,  0, }, // Indonesia_Central
    {  450,  972,       0,  551183,       0,  0, 18,  0, }, // Indonesia_Eastern
    {  450,  990,       0,  545888,       0,  0, 18,  0, }, // Indonesia_Western
    {  450, 1008,  545906,  545916,  551201, 10, 16, 16, }, // Iran
    {  450, 1019,  545948,  545968,  551217, 20, 21, 21, }, // Irkutsk
    {  450, 1027,  546010,  546022,  551238, 12, 18, 18, }, // Israel
    {  450, 1034,  546058,  546069,  551256, 11, 17, 17, }, // Japan
    {  450, 1040,  546103,  546143,  546189, 40, 46, 46, }, // Kamchatka
    {  450, 1058,       0,  551273,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  450, 1077,       0,  546254,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  450, 1106,  546274,  546286,  551293, 12, 18, 18, }, // Korea
    {  450, 1112,       0,  546322,       0,  0, 19,  0, }, // Kosrae
    {  450, 1119,  546341,  546365,  551311, 24, 25, 25, }, // Krasnoyarsk
    {  450, 1151,       0,  546415,       0,  0, 13,  0, }, // Kyrgystan
    {  450, 1161,       0,  546428,       0,  0,  9,  0, }, // Lanka
    {  450, 1175,       0,  551336,       0,  0, 22,  0, }, // Line_Islands
    {  450, 1188,  546460,  546486,  551358, 26, 27, 27, }, // Lord_Howe
    {  450, 1198,  546540,  546549,  546567,  9, 18, 16, }, // Macau
    {  450, 1204,  546583,  546603,  551385, 20, 21, 21, }, // Magadan
    {  450, 1219,       0,  546645,       0,  0, 13,  0, }, // Malaysia
    {  450, 1228,       0,  546658,       0,  0, 13,  0, }, // Maldives
    {  450, 1237,       0,  551406,       0,  0, 27,  0, }, // Marquesas
    {  450, 1247,       0,  546698,       0,  0, 17,  0, }, // Marshall_Islands
    {  450, 1264,  546715,  546728,  551433, 13, 19, 19, }, // Mauritius
    {  450, 1274,       0,  546766,       0,  0, 19,  0, }, // Mawson
    {  450, 1281,  551452,  551497,  551543, 45, 46, 46, }, // Mexico_Pacific
    {  450, 1296,  546922,  546945,  551589, 23, 24, 24, }, // Mongolia
    {  450, 1305,  546993,  547012,  551613, 19, 20, 20, }, // Moscow
    {  450, 1312,       0,  547052,       0,  0, 13,  0, }, // Myanmar
    {  450, 1320,       0,  547065,       0,  0, 11,  0, }, // Nauru
    {  450, 1326,       0,  547076,       0,  0, 11,  0, }, // Nepal
    {  450, 1332,  547087,  547100,  551633, 13, 19, 19, }, // New_Caledonia
    {  450, 1346,  551652,  551667,  551688, 15, 21, 21, }, // New_Zealand
    {  450, 1358,  547198,  547223,  551709, 25, 26, 26, }, // Newfoundland
    {  450, 1371,       0,  547275,       0,  0, 17,  0, }, // Niue
    {  450, 1376,  547292,  547315,  551735, 23, 24, 24, }, // Norfolk
    {  450, 1384,  547363,  547395,  551759, 32, 33, 33, }, // Noronha
    {  450, 1392,       0,  547461,       0,  0, 18,  0, }, // North_Mariana
    {  450, 1406,  547479,  547503,  551792, 24, 25, 25, }, // Novosibirsk
    {  450, 1418,  547553,  547570,  551817, 17, 18, 18, }, // Omsk
    {  450, 1428,  547606,  547620,  551835, 14, 20, 20, }, // Pakistan
    {  450, 1437,       0,  547660,       0,  0, 12,  0, }, // Palau
    {  450, 1443,       0,  547672,       0,  0, 12,  0, }, // Papua_New_Guinea
    {  450, 1460,  547684,  547700,  551855, 16, 22, 22, }, // Paraguay
    {  450, 1469,  547744,  547756,  551877, 12, 18, 18, }, // Peru
    {  450, 1474,  547792,  547806,  551895, 14, 20, 20, }, // Philippines
    {  450, 1486,       0,  551915,       0,  0, 25,  0, }, // Phoenix_Islands
    {  450, 1502,  547871,  547908,  551940, 37, 38, 38, }, // Pierre_Miquelon
    {  450, 1518,       0,  547984,       0,  0, 21,  0, }, // Pitcairn
    {  450, 1527,       0,  548005,       0,  0, 20,  0, }, // Ponape
    {  450, 1534,       0,  548025,       0,  0, 22,  0, }, // Pyongyang
    {  450, 1544,  200412,  548047,  548069, 13, 22, 20, }, // Qyzylorda
    {  450, 1554,       0,  548089,       0,  0, 20,  0, }, // Reunion
    {  450, 1562,       0,  548109,       0,  0, 20,  0, }, // Rothera
    {  450, 1570,  548129,  548150,  551978, 21, 22, 22, }, // Sakhalin
    {  450, 1579,  548194,  548214,  548240, 20, 26, 26, }, // Samara
    {  450, 1596,  548266,  548278,  552000, 12, 18, 18, }, // Samoa
    {  450, 1602,       0,  548314,       0,  0, 15,  0, }, // Seychelles
    {  450, 1624,       0,  548329,       0,  0, 14,  0, }, // Singapore
    {  450, 1634,       0,  548343,       0,  0, 13,  0, }, // Solomon
    {  450, 1642,       0,  548356,       0,  0, 24,  0, }, // South_Georgia
    {  450, 1656,       0,  548380,       0,  0, 13,  0, }, // Suriname
    {  450, 1676,       0,  548393,       0,  0, 18,  0, }, // Syowa
    {  450, 1682,       0,  548411,       0,  0, 12,  0, }, // Tahiti
    {  450, 1689,  548423,  548442,  552018, 19, 20, 20, }, // Taipei
    {  450, 1696,       0,  548482,       0,  0, 14,  0, }, // Tajikistan
    {  450, 1724,       0,  548496,       0,  0, 20,  0, }, // Tokelau
    {  450, 1732,  548516,  548528,  552038, 12, 18, 18, }, // Tonga
    {  450, 1738,       0,  552056,       0,  0, 23,  0, }, // Truk
    {  450, 1750,  548587,  548600,  552079, 13, 19, 19, }, // Turkmenistan
    {  450, 1763,       0,  548638,       0,  0, 11,  0, }, // Tuvalu
    {  450, 1777,  548649,  548664,  552098, 15, 21, 21, }, // Uruguay
    {  450, 1792,  548706,  548718,  552119, 12, 18, 18, }, // Uzbekistan
    {  450, 1803,  548754,  548767,  552137, 13, 19, 19, }, // Vanuatu
    {  450, 1811,       0,  548805,       0,  0, 16,  0, }, // Venezuela
    {  450, 1821,  548821,  548845,  552156, 24, 25, 25, }, // Vladivostok
    {  450, 1833,  548895,  548917,  552181, 22, 23, 23, }, // Volgograd
    {  450, 1843,       0,  548963,       0,  0, 19,  0, }, // Vostok
    {  450, 1850,       0,  548982,       0,  0, 24,  0, }, // Wake
    {  450, 1855,       0,  552204,       0,  0, 35,  0, }, // Wallis
    {  450, 1862,  549041,  549061,  552239, 20, 21, 21, }, // Yakutsk
    {  450, 1870,  549103,  549129,  552260, 26, 27, 27, }, // Yekaterinburg
    {  450, 1892,       0,  549183,       0,  0, 18,  0, }, // Yukon
    {  454,    5,       0,  552287,       0,  0, 16,  0, }, // Afghanistan Occitan/Latin/France
    {  454,   17,       0,  552303,       0,  0, 21,  0, }, // Africa_Central
    {  454,   32,       0,  552324,       0,  0, 21,  0, }, // Africa_Eastern
    {  454,   65,       0,  552345,       0,  0, 16,  0, }, // Africa_Southern
    {  454,   81,  552361,  552385,  552420, 24, 35, 32, }, // Africa_Western
    {  454,  107,   52266,  552452,  133339, 12, 23, 20, }, // Alaska
    {  454,  135,  552475,  552492,  552520, 17, 28, 22, }, // Amazon
    {  454,  142,  552542,  552554,  552577, 12, 23, 20, }, // America_Central
    {  454,  158,  552597,  552609,  552632, 12, 23, 20, }, // America_Eastern
    {  454,  174,  552652,  552670,  552699, 18, 29, 26, }, // America_Mountain
    {  454,  191,  365137,  552725,  552751, 15, 26, 23, }, // America_Pacific
    {  454,  214,   52947,  552774,  552794, 10, 20, 17, }, // Apia
    {  454,  232,   53109,  552811,  552834, 12, 23, 20, }, // Arabian
    {  454,  240,  552854,  552869,  552895, 15, 26, 23, }, // Argentina
    {  454,  250,  552918,  552943,  552979, 25, 36, 33, }, // Argentina_Western
    {  454,  268,   53331,  553012,  553036, 13, 24, 21, }, // Armenia
    {  454,  286,   37885,  553057,  553085, 17, 28, 25, }, // Atlantic
    {  454,  295,  553110,  553134,  553168, 24, 34, 32, }, // Australia_Central
    {  454,  313,  553200,  553233,  553276, 33, 43, 52, }, // Australia_CentralWestern
    {  454,  338,  553328,  553352,  553386, 24, 34, 32, }, // Australia_Eastern
    {  454,  356,  553418,  553444,  553480, 26, 36, 34, }, // Australia_Western
    {  454,  374,  553514,  553530,  553557, 16, 27, 24, }, // Azerbaijan
    {  454,  385,  553581,  553598,  553626, 17, 28, 25, }, // Azores
    {  454,  397,  553651,  553668,  553696, 17, 28, 25, }, // Bangladesh
    {  454,  415,       0,  553721,       0,  0, 12,  0, }, // Bhutan
    {  454,  422,       0,   54085,       0,  0, 14,  0, }, // Bolivia
    {  454,  437,   54100,  553733,  553759, 15, 26, 23, }, // Brasilia
    {  454,  454,       0,  135081,       0,  0, 13,  0, }, // Brunei
    {  454,  461,  553782,  553797,  553823, 15, 26, 23, }, // Cape_Verde
    {  454,  478,       0,  553846,       0,  0, 25,  0, }, // Chamorro
    {  454,  487,   54301,  553871,  135223, 14, 24, 22, }, // Chatham
    {  454,  495,   54364,  553895,  553918, 12, 23, 20, }, // Chile
    {  454,  501,   54421,  553938,  553961, 12, 23, 20, }, // China
    {  454,  507,   54478,  553981,  135391, 17, 28, 25, }, // Choibalsan
    {  454,  518,       0,  554009,       0,  0, 22,  0, }, // Christmas
    {  454,  528,       0,  554031,       0,  0, 22,  0, }, // Cocos
    {  454,  534,  135458,  554053,  135500, 15, 26, 23, }, // Colombia
    {  454,  543,  554079,  554100,  554131, 21, 31, 35, }, // Cook
    {  454,  548,   54768,  554166,  135635, 11, 22, 19, }, // Cuba
    {  454,  559,       0,   54822,       0,  0, 12,  0, }, // Davis
    {  454,  575,       0,   54835,       0,  0, 23,  0, }, // DumontDUrville
    {  454,  612,       0,  554188,       0,  0, 21,  0, }, // East_Timor
    {  454,  623,  554209,  554233,  554268, 24, 35, 32, }, // Easter
    {  454,  630,       0,  554300,       0,  0, 13,  0, }, // Ecuador
    {  454,  638,  554313,  554334,  554366, 21, 32, 29, }, // Europe_Central
    {  454,  653,  554395,  554416,  554448, 21, 32, 29, }, // Europe_Eastern
    {  454,  668,       0,  554477,       0,  0, 34,  0, }, // Europe_Further_Eastern
    {  454,  691,  554511,  554533,  554566, 22, 33, 30, }, // Europe_Western
    {  454,  706,  554596,  554621,  554657, 25, 36, 33, }, // Falkland
    {  454,  715,  136174,  554690,  136208, 11, 21, 19, }, // Fiji
    {  454,  720,       0,  554711,       0,  0, 26,  0, }, // French_Guiana
    {  454,  734,       0,  554737,       0,  0, 51,  0, }, // French_Southern
    {  454,  757,       0,  554788,       0,  0, 16,  0, }, // Galapagos
    {  454,  767,       0,   55506,       0,  0, 14,  0, }, // Gambier
    {  454,  775,  136288,  554804,  136328, 14, 25, 22, }, // Georgia
    {  454,  783,       0,  136351,       0,  0, 24,  0, }, // Gilbert_Islands
    {  454,  799,       0,  554829,       0,  0, 28,  0, }, // GMT
    {  454,  841,  554857,  554884,  554922, 27, 38, 35, }, // Greenland_Eastern
    {  454,  859,  554957,  554985,  555024, 28, 39, 36, }, // Greenland_Western
    {  454,  882,       0,  555060,       0,  0, 23,  0, }, // Gulf
    {  454,  887,       0,  554711,       0,  0, 17,  0, }, // Guyana
    {  454,  894,  555083,  555106,  555140, 23, 34, 31, }, // Hawaii_Aleutian
    {  454,  910,  136717,  555171,  136761, 16, 27, 24, }, // Hong_Kong
    {  454,  920,   56046,  555198,  555220, 11, 22, 19, }, // Hovd
    {  454,  925,       0,  555239,       0,  0, 25,  0, }, // India
    {  454,  931,       0,  555264,       0,  0, 20,  0, }, // Indian_Ocean
    {  454,  944,       0,   56147,       0,  0, 15,  0, }, // Indochina
    {  454,  954,       0,  555284,       0,  0, 25,  0, }, // Indonesia_Central
    {  454,  972,       0,  555309,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  454,  990,       0,  555335,       0,  0, 28,  0, }, // Indonesia_Western
    {  454, 1008,  555363,  555373,  555394, 10, 21, 18, }, // Iran
    {  454, 1019,   56289,  555412,  137055, 13, 24, 21, }, // Irkutsk
    {  454, 1027,  555436,  555448,  555471, 12, 23, 20, }, // Israel
    {  454, 1034,  364637,  555491,  555514, 12, 23, 20, }, // Japan
    {  454, 1058,       0,  555534,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  454, 1077,       0,  555561,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  454, 1106,  555590,  555602,  555625, 12, 23, 20, }, // Korea
    {  454, 1112,       0,   56690,       0,  0, 13,  0, }, // Kosrae
    {  454, 1119,   56704,  555645,  555674, 18, 29, 26, }, // Krasnoyarsk
    {  454, 1151,       0,  555700,       0,  0, 17,  0, }, // Kyrgystan
    {  454, 1175,       0,  555717,       0,  0, 32,  0, }, // Line_Islands
    {  454, 1188,   56834,  555749,  555775, 16, 26, 24, }, // Lord_Howe
    {  454, 1204,  137612,  555799,  137652, 14, 25, 22, }, // Magadan
    {  454, 1219,       0,  137675,       0,  0, 15,  0, }, // Malaysia
    {  454, 1228,       0,  555824,       0,  0, 16,  0, }, // Maldives
    {  454, 1237,       0,  555840,       0,  0, 26,  0, }, // Marquesas
    {  454, 1247,       0,  555866,       0,  0, 25,  0, }, // Marshall_Islands
    {  454, 1264,   57106,  555891,  137762, 14, 25, 22, }, // Mauritius
    {  454, 1274,       0,   57172,       0,  0, 13,  0, }, // Mawson
    {  454, 1281,  555916,  555939,  555973, 23, 34, 31, }, // Mexico_Pacific
    {  454, 1296,  365226,  556004,  556032, 17, 28, 25, }, // Mongolia
    {  454, 1305,  556057,  556070,  556094, 13, 24, 21, }, // Moscow
    {  454, 1312,       0,   57411,       0,  0, 14,  0, }, // Myanmar
    {  454, 1320,       0,   57426,       0,  0, 12,  0, }, // Nauru
    {  454, 1326,       0,  290334,       0,  0, 12,  0, }, // Nepal
    {  454, 1332,  556115,  556136,  556167, 21, 31, 29, }, // New_Caledonia
    {  454, 1346,  556196,  556215,  556244, 19, 29, 27, }, // New_Zealand
    {  454, 1358,  556271,  556288,  556316, 17, 28, 25, }, // Newfoundland
    {  454, 1371,       0,   57699,       0,  0, 11,  0, }, // Niue
    {  454, 1376,  138241,  556341,  138295, 21, 31, 29, }, // Norfolk
    {  454, 1384,  556372,  556395,  556429, 23, 34, 31, }, // Noronha
    {  454, 1406,   57937,  556460,  556489, 18, 29, 26, }, // Novosibirsk
    {  454, 1418,   58012,  556515,  138496, 10, 21, 18, }, // Omsk
    {  454, 1428,  365808,  556536,  556562, 15, 26, 23, }, // Pakistan
    {  454, 1437,       0,  556585,       0,  0, 13,  0, }, // Palau
    {  454, 1443,       0,  556598,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  454, 1460,  365897,  556622,  556648, 15, 26, 23, }, // Paraguay
    {  454, 1469,  556671,  556682,  556704, 11, 22, 19, }, // Peru
    {  454, 1474,   58300,  556723,  556750, 16, 27, 24, }, // Philippines
    {  454, 1486,       0,  556774,       0,  0, 22,  0, }, // Phoenix_Islands
    {  454, 1502,  556796,  556824,  556863, 28, 39, 36, }, // Pierre_Miquelon
    {  454, 1518,       0,   58506,       0,  0, 15,  0, }, // Pitcairn
    {  454, 1527,       0,  291348,       0,  0, 14,  0, }, // Ponape
    {  454, 1534,       0,   58536,       0,  0, 16,  0, }, // Pyongyang
    {  454, 1554,       0,  556899,       0,  0, 17,  0, }, // Reunion
    {  454, 1562,       0,   58637,       0,  0, 14,  0, }, // Rothera
    {  454, 1570,  556916,  556930,  556955, 14, 25, 22, }, // Sakhalin
    {  454, 1596,   58775,  556977,  139094, 12, 22, 20, }, // Samoa
    {  454, 1602,       0,  556999,       0,  0, 17,  0, }, // Seychelles
    {  454, 1624,       0,  557016,       0,  0, 15,  0, }, // Singapore
    {  454, 1634,       0,  557031,       0,  0, 24,  0, }, // Solomon
    {  454, 1642,       0,  139177,       0,  0, 22,  0, }, // South_Georgia
    {  454, 1656,       0,  139200,       0,  0, 14,  0, }, // Suriname
    {  454, 1676,       0,   58941,       0,  0, 12,  0, }, // Syowa
    {  454, 1682,       0,   58954,       0,  0, 13,  0, }, // Tahiti
    {  454, 1689,  139229,  557055,  139267, 13, 24, 21, }, // Taipei
    {  454, 1696,       0,  366503,       0,  0, 17,  0, }, // Tajikistan
    {  454, 1724,       0,   59048,       0,  0, 14,  0, }, // Tokelau
    {  454, 1732,   59063,  557079,  139332, 12, 22, 20, }, // Tonga
    {  454, 1738,       0,   59120,       0,  0, 12,  0, }, // Truk
    {  454, 1750,  366563,  557101,  557131, 19, 30, 27, }, // Turkmenistan
    {  454, 1763,       0,   59214,       0,  0, 13,  0, }, // Tuvalu
    {  454, 1777,  557158,  557174,  557201, 16, 27, 24, }, // Uruguay
    {  454, 1792,  366700,  557225,  557253, 17, 28, 25, }, // Uzbekistan
    {  454, 1803,   59372,  557278,  557302, 14, 24, 22, }, // Vanuatu
    {  454, 1811,       0,  139641,       0,  0, 16,  0, }, // Venezuela
    {  454, 1821,   59452,  557324,  139687, 18, 29, 26, }, // Vladivostok
    {  454, 1833,  139714,  557353,  139758, 16, 27, 24, }, // Volgograd
    {  454, 1843,       0,   59596,       0,  0, 13,  0, }, // Vostok
    {  454, 1850,       0,  557380,       0,  0, 18,  0, }, // Wake
    {  454, 1855,       0,  292356,       0,  0, 22,  0, }, // Wallis
    {  454, 1862,   59654,  557398,  557423, 14, 25, 22, }, // Yakutsk
    {  454, 1870,  557445,  557463,  557492, 18, 29, 26, }, // Yekaterinburg
    {  454, 1892,       0,  139972,       0,  0, 12,  0, }, // Yukon
    {  455,   17,       0,  552303,       0,  0, 20,  0, }, // Africa_Central Occitan/Latin/Spain
    {  455,   32,       0,  557518,       0,  0, 21,  0, }, // Africa_Eastern
    {  455,   81,  557539,  557562,  557595, 23, 33, 31, }, // Africa_Western
    {  455,  107,   52266,  557626,  133339, 12, 22, 20, }, // Alaska
    {  455,  135,  557648,  557664,  557690, 16, 26, 24, }, // Amazon
    {  455,  142,  557714,  557725,  557746, 11, 21, 19, }, // America_Central
    {  455,  158,  557765,  557777,  557799, 12, 22, 20, }, // America_Eastern
    {  455,  174,  557819,  557844,  557879, 25, 35, 33, }, // America_Mountain
    {  455,  191,  557912,  557928,  557954, 16, 26, 24, }, // America_Pacific
    {  455,  232,   53109,  557978,  558000, 12, 22, 20, }, // Arabian
    {  455,  240,  552854,  558020,  552895, 15, 25, 23, }, // Argentina
    {  455,  250,  558045,  558070,  558106, 25, 36, 34, }, // Argentina_Western
    {  455,  268,   53331,  558140,  553036, 13, 23, 21, }, // Armenia
    {  455,  286,  558163,  558179,  558205, 16, 26, 24, }, // Atlantic
    {  455,  295,  558229,  558252,  558285, 23, 33, 31, }, // Australia_Central
    {  455,  313,  558316,  558348,  558391, 32, 43, 40, }, // Australia_CentralWestern
    {  455,  338,  558431,  558455,  558489, 24, 34, 32, }, // Australia_Eastern
    {  455,  356,  558521,  558547,  558583, 26, 36, 44, }, // Australia_Western
    {  455,  374,  553514,  558627,  553557, 16, 26, 24, }, // Azerbaijan
    {  455,  385,  558653,  558667,  558691, 14, 24, 22, }, // Azores
    {  455,  397,  553651,  558713,  134960, 17, 27, 25, }, // Bangladesh
    {  455,  437,   54100,  558740,  553759, 15, 25, 23, }, // Brasilia
    {  455,  461,  553782,  558765,  553823, 15, 25, 23, }, // Cape_Verde
    {  455,  495,   54364,  558790,  553918, 12, 22, 20, }, // Chile
    {  455,  501,   54421,  558812,  553961, 12, 22, 20, }, // China
    {  455,  507,   54478,  558834,  135391, 17, 27, 25, }, // Choibalsan
    {  455,  518,       0,  558861,       0,  0, 22,  0, }, // Christmas
    {  455,  528,       0,  558883,       0,  0, 22,  0, }, // Cocos
    {  455,  534,  135458,  558905,  135500, 15, 25, 23, }, // Colombia
    {  455,  543,  558930,  558948,  558976, 18, 28, 32, }, // Cook
    {  455,  548,   54768,  559008,  135635, 11, 21, 19, }, // Cuba
    {  455,  612,       0,  559029,       0,  0, 21,  0, }, // East_Timor
    {  455,  623,  559050,  559073,  559106, 23, 33, 31, }, // Easter
    {  455,  638,  559137,  559157,  559187, 20, 30, 28, }, // Europe_Central
    {  455,  653,  554395,  559215,  554448, 21, 31, 29, }, // Europe_Eastern
    {  455,  668,       0,  559246,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  455,  691,  559278,  559301,  559334, 23, 33, 31, }, // Europe_Western
    {  455,  706,  559365,  559387,  559419, 22, 32, 30, }, // Falkland
    {  455,  720,       0,  559449,       0,  0, 25,  0, }, // French_Guiana
    {  455,  734,       0,  559474,       0,  0, 47,  0, }, // French_Southern
    {  455,  757,       0,   55489,       0,  0, 16,  0, }, // Galapagos
    {  455,  775,  136288,  559521,  136328, 14, 24, 22, }, // Georgia
    {  455,  783,       0,  559545,       0,  0, 21,  0, }, // Gilbert_Islands
    {  455,  799,       0,  559566,       0,  0, 30,  0, }, // GMT
    {  455,  841,  559596,  559623,  559660, 27, 37, 35, }, // Greenland_Eastern
    {  455,  859,  559695,  559724,  559763, 29, 39, 37, }, // Greenland_Western
    {  455,  887,       0,  136635,       0,  0, 13,  0, }, // Guyana
    {  455,  894,  555083,  559800,  555140, 23, 33, 31, }, // Hawaii_Aleutian
    {  455,  910,  136717,  559833,  136761, 16, 26, 24, }, // Hong_Kong
    {  455,  920,   56046,  559859,  555220, 11, 21, 19, }, // Hovd
    {  455,  925,       0,  559880,       0,  0, 24,  0, }, // India
    {  455,  931,       0,  559904,       0,  0, 19,  0, }, // Indian_Ocean
    {  455,  954,       0,  559923,       0,  0, 24,  0, }, // Indonesia_Central
    {  455,  972,       0,  559947,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  455,  990,       0,  559972,       0,  0, 27,  0, }, // Indonesia_Western
    {  455, 1008,  555363,  559999,  555394, 10, 20, 18, }, // Iran
    {  455, 1019,   56289,  560019,  137055, 13, 23, 21, }, // Irkutsk
    {  455, 1027,  555436,  560042,  555471, 12, 22, 20, }, // Israel
    {  455, 1034,  364637,  560064,  555514, 12, 22, 20, }, // Japan
    {  455, 1058,       0,  560086,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  455, 1077,       0,  560112,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  455, 1106,  555590,  560140,  555625, 12, 22, 20, }, // Korea
    {  455, 1119,   56704,  560162,  555674, 18, 28, 26, }, // Krasnoyarsk
    {  455, 1175,       0,  560190,       0,  0, 29,  0, }, // Line_Islands
    {  455, 1204,  137612,  560219,  137652, 14, 24, 22, }, // Magadan
    {  455, 1237,       0,  560243,       0,  0, 23,  0, }, // Marquesas
    {  455, 1247,       0,  560266,       0,  0, 22,  0, }, // Marshall_Islands
    {  455, 1264,   57106,  560288,  137762, 14, 24, 22, }, // Mauritius
    {  455, 1281,  560312,  560337,  560372, 25, 35, 33, }, // Mexico_Pacific
    {  455, 1296,  365226,  560405,  556032, 17, 27, 25, }, // Mongolia
    {  455, 1305,  556057,  560432,  556094, 13, 23, 21, }, // Moscow
    {  455, 1326,       0,  560455,       0,  0, 14,  0, }, // Nepal
    {  455, 1358,  556271,  560469,  556316, 17, 27, 25, }, // Newfoundland
    {  455, 1376,  560496,  560517,  560548, 21, 31, 29, }, // Norfolk
    {  455, 1384,   57801,  560577,  138362, 26, 36, 34, }, // Noronha
    {  455, 1406,   57937,  560613,  556489, 18, 28, 26, }, // Novosibirsk
    {  455, 1418,   58012,  560641,  138496, 10, 20, 18, }, // Omsk
    {  455, 1428,  560661,  560678,  560705, 17, 27, 25, }, // Pakistan
    {  455, 1460,  365897,  560730,  556648, 15, 25, 23, }, // Paraguay
    {  455, 1469,  556671,  560755,  560776, 11, 21, 18, }, // Peru
    {  455, 1474,   58300,  560794,  556750, 16, 26, 24, }, // Philippines
    {  455, 1486,       0,  560820,       0,  0, 19,  0, }, // Phoenix_Islands
    {  455, 1502,  556796,  560839,  556863, 28, 38, 36, }, // Pierre_Miquelon
    {  455, 1554,       0,  560877,       0,  0, 14,  0, }, // Reunion
    {  455, 1570,  556916,  560891,  556955, 14, 24, 22, }, // Sakhalin
    {  455, 1634,       0,  560915,       0,  0, 21,  0, }, // Solomon
    {  455, 1642,       0,  560936,       0,  0, 23,  0, }, // South_Georgia
    {  455, 1689,  139229,  560959,  139267, 13, 23, 21, }, // Taipei
    {  455, 1750,  366563,  560982,  557131, 19, 29, 27, }, // Turkmenistan
    {  455, 1777,  561011,  561024,  561047, 13, 23, 21, }, // Uruguay
    {  455, 1792,  366700,  561068,  557253, 17, 27, 25, }, // Uzbekistan
    {  455, 1821,   59452,  561095,  139687, 18, 28, 26, }, // Vladivostok
    {  455, 1833,  139714,  561123,  139758, 16, 26, 24, }, // Volgograd
    {  455, 1850,       0,  561149,       0,  0, 18,  0, }, // Wake
    {  455, 1862,   59654,  561167,  557423, 14, 24, 22, }, // Yakutsk
    {  455, 1870,  557445,  561191,  557492, 18, 28, 26, }, // Yekaterinburg
    {  456,    0,  561219,  561226,  561238,  7, 12, 15, }, // Acre Odia/Odia/India
    {  456,    5,       0,  561253,       0,  0, 15,  0, }, // Afghanistan
    {  456,   17,       0,  561268,       0,  0, 16,  0, }, // Africa_Central
    {  456,   32,       0,  561284,       0,  0, 17,  0, }, // Africa_Eastern
    {  456,   65,       0,  561301,       0,  0, 26,  0, }, // Africa_Southern
    {  456,   81,  561327,  561345,  561371, 18, 26, 25, }, // Africa_Western
    {  456,  107,  561396,  561407,  561426, 11, 19, 19, }, // Alaska
    {  456,  128,  561445,  561455,  561470, 10, 15, 18, }, // Almaty
    {  456,  135,  561488,  561498,  561516, 10, 18, 23, }, // Amazon
    {  456,  142,  561539,  561552,  561573, 13, 21, 21, }, // America_Central
    {  456,  158,  561594,  561608,  561630, 14, 22, 22, }, // America_Eastern
    {  456,  174,  561652,  561664,  561684, 12, 20, 20, }, // America_Mountain
    {  456,  191,  561704,  561716,  561736, 12, 20, 20, }, // America_Pacific
    {  456,  207,  561756,  561767,  561783, 11, 16, 19, }, // Anadyr
    {  456,  214,  561802,  561810,  561826,  8, 16, 16, }, // Apia
    {  456,  219,  561842,  561850,  561863,  8, 13, 16, }, // Aqtau
    {  456,  225,  561879,  561889,  561904, 10, 15, 18, }, // Aqtobe
    {  456,  232,  561922,  561931,  561948,  9, 17, 17, }, // Arabian
    {  456,  240,  561965,  561980,  562003, 15, 23, 26, }, // Argentina
    {  456,  250,  562029,  562051,  562081, 22, 30, 33, }, // Argentina_Western
    {  456,  268,  562114,  562126,  562146, 12, 20, 23, }, // Armenia
    {  456,  286,  562169,  562183,  562205, 14, 22, 22, }, // Atlantic
    {  456,  295,  562227,  562246,  562273, 19, 27, 27, }, // Australia_Central
    {  456,  313,  562300,  562326,  562360, 26, 34, 34, }, // Australia_CentralWestern
    {  456,  338,  562394,  562414,  562442, 20, 28, 28, }, // Australia_Eastern
    {  456,  356,  562470,  562491,  562520, 21, 29, 29, }, // Australia_Western
    {  456,  374,  562549,  562563,  562584, 14, 21, 21, }, // Azerbaijan
    {  456,  385,  562605,  562616,  562635, 11, 19, 22, }, // Azores
    {  456,  397,  562657,  562669,  562689, 12, 20, 23, }, // Bangladesh
    {  456,  415,       0,  562712,       0,  0,  9,  0, }, // Bhutan
    {  456,  422,       0,  562721,       0,  0, 10,  0, }, // Bolivia
    {  456,  437,  562731,  562744,  562765, 13, 21, 24, }, // Brasilia
    {  456,  454,       0,  562789,       0,  0, 21,  0, }, // Brunei
    {  456,  461,  562810,  562825,  562848, 15, 23, 22, }, // Cape_Verde
    {  456,  478,       0,  562870,       0,  0, 18,  0, }, // Chamorro
    {  456,  487,  562888,  562899,  562918, 11, 19, 19, }, // Chatham
    {  456,  495,  562937,  562945,  562961,  8, 16, 19, }, // Chile
    {  456,  501,  562980,  562987,  563002,  7, 15, 15, }, // China
    {  456,  507,  563017,  563030,  563051, 13, 21, 24, }, // Choibalsan
    {  456,  518,       0,  563075,       0,  0, 20,  0, }, // Christmas
    {  456,  528,       0,  563095,       0,  0, 22,  0, }, // Cocos
    {  456,  534,  563117,  563128,  563147, 11, 19, 22, }, // Colombia
    {  456,  543,  563169,  563189,  563217, 20, 28, 38, }, // Cook
    {  456,  548,  563255,  563265,  563283, 10, 18, 18, }, // Cuba
    {  456,  559,       0,  563301,       0,  0, 11,  0, }, // Davis
    {  456,  575,       0,  563312,       0,  0, 23,  0, }, // DumontDUrville
    {  456,  612,       0,  563335,       0,  0, 17,  0, }, // East_Timor
    {  456,  623,  563352,  563373,  563402, 21, 29, 32, }, // Easter
    {  456,  630,       0,  563434,       0,  0, 11,  0, }, // Ecuador
    {  456,  638,  563445,  563466,  563495, 21, 29, 32, }, // Europe_Central
    {  456,  653,  563527,  563549,  563579, 22, 30, 33, }, // Europe_Eastern
    {  456,  668,       0,  563612,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  456,  691,  563644,  563667,  563698, 23, 31, 34, }, // Europe_Western
    {  456,  706,  563732,  563756,  563788, 24, 32, 35, }, // Falkland
    {  456,  715,  563823,  563831,  563847,  8, 16, 19, }, // Fiji
    {  456,  720,       0,  563866,       0,  0, 17,  0, }, // French_Guiana
    {  456,  734,       0,  563883,       0,  0, 37,  0, }, // French_Southern
    {  456,  757,       0,  563920,       0,  0, 14,  0, }, // Galapagos
    {  456,  767,       0,  563934,       0,  0, 13,  0, }, // Gambier
    {  456,  775,  563947,  563957,  563975, 10, 18, 17, }, // Georgia
    {  456,  783,       0,  563992,       0,  0, 24,  0, }, // Gilbert_Islands
    {  456,  799,       0,  564016,       0,  0, 18,  0, }, // GMT
    {  456,  841,  564034,  564057,  564088, 23, 31, 34, }, // Greenland_Eastern
    {  456,  859,  564122,  564146,  564178, 24, 32, 32, }, // Greenland_Western
    {  456,  882,       0,  564210,       0,  0, 16,  0, }, // Gulf
    {  456,  887,       0,  564226,       0,  0,  9,  0, }, // Guyana
    {  456,  894,  564235,  564251,  564275, 16, 24, 24, }, // Hawaii_Aleutian
    {  456,  910,  564299,  564308,  564325,  9, 17, 20, }, // Hong_Kong
    {  456,  920,  564345,  564355,  564373, 10, 18, 21, }, // Hovd
    {  456,  925,       0,  564394,       0,  0, 16,  0, }, // India
    {  456,  931,       0,  564410,       0,  0, 17,  0, }, // Indian_Ocean
    {  456,  944,       0,  564427,       0,  0, 14,  0, }, // Indochina
    {  456,  954,       0,  564441,       0,  0, 19,  0, }, // Indonesia_Central
    {  456,  972,       0,  564460,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  456,  990,       0,  564480,       0,  0, 21,  0, }, // Indonesia_Western
    {  456, 1008,  564501,  564509,  564525,  8, 16, 16, }, // Iran
    {  456, 1019,  564541,  564554,  564575, 13, 21, 27, }, // Irkutsk
    {  456, 1027,  564602,  564613,  564632, 11, 19, 19, }, // Israel
    {  456, 1034,  564651,  564660,  564677,  9, 17, 17, }, // Japan
    {  456, 1040,  564694,  564722,  564755, 28, 33, 36, }, // Kamchatka
    {  456, 1058,       0,  564791,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  456, 1077,       0,  564812,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  456, 1106,  564833,  564842,  564859,  9, 17, 17, }, // Korea
    {  456, 1112,       0,  564876,       0,  0, 10,  0, }, // Kosrae
    {  456, 1119,  564886,  564903,  564928, 17, 25, 28, }, // Krasnoyarsk
    {  456, 1151,       0,  564956,       0,  0, 15,  0, }, // Kyrgystan
    {  456, 1161,       0,  564971,       0,  0,  9,  0, }, // Lanka
    {  456, 1175,       0,  564980,       0,  0, 21,  0, }, // Line_Islands
    {  456, 1188,  565001,  565014,  565035, 13, 21, 21, }, // Lord_Howe
    {  456, 1198,  565056,  565065,  565078,  9, 13, 16, }, // Macau
    {  456, 1204,  565094,  565106,  565126, 12, 20, 23, }, // Magadan
    {  456, 1219,       0,  565149,       0,  0, 11,  0, }, // Malaysia
    {  456, 1228,       0,  565160,       0,  0, 12,  0, }, // Maldives
    {  456, 1237,       0,  565172,       0,  0, 17,  0, }, // Marquesas
    {  456, 1247,       0,  565189,       0,  0, 24,  0, }, // Marshall_Islands
    {  456, 1264,  565213,  565225,  565245, 12, 20, 19, }, // Mauritius
    {  456, 1274,       0,  565264,       0,  0, 11,  0, }, // Mawson
    {  456, 1281,  565275,  565298,  565329, 23, 31, 31, }, // Mexico_Pacific
    {  456, 1296,  565360,  565376,  565400, 16, 24, 28, }, // Mongolia
    {  456, 1305,  565428,  565437,  565454,  9, 17, 20, }, // Moscow
    {  456, 1312,       0,  565474,       0,  0, 13,  0, }, // Myanmar
    {  456, 1320,       0,  565487,       0,  0,  9,  0, }, // Nauru
    {  456, 1326,       0,  565496,       0,  0,  9,  0, }, // Nepal
    {  456, 1332,  565505,  565523,  565549, 18, 26, 29, }, // New_Caledonia
    {  456, 1346,  565578,  565593,  565616, 15, 23, 23, }, // New_Zealand
    {  456, 1358,  565639,  565661,  565691, 22, 30, 30, }, // Newfoundland
    {  456, 1371,       0,  565721,       0,  0,  8,  0, }, // Niue
    {  456, 1376,  565729,  565746,  565771, 17, 25, 28, }, // Norfolk
    {  456, 1384,  565799,  565826,  565861, 27, 35, 38, }, // Noronha
    {  456, 1392,       0,  565899,       0,  0, 27,  0, }, // North_Mariana
    {  456, 1406,  565926,  565942,  565966, 16, 24, 27, }, // Novosibirsk
    {  456, 1418,  565993,  566002,  566019,  9, 17, 20, }, // Omsk
    {  456, 1428,  566039,  566052,  566073, 13, 21, 24, }, // Pakistan
    {  456, 1437,       0,  566097,       0,  0,  9,  0, }, // Palau
    {  456, 1443,       0,  566106,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  456, 1460,  566125,  566136,  566155, 11, 19, 22, }, // Paraguay
    {  456, 1469,  566177,  566185,  566201,  8, 16, 19, }, // Peru
    {  456, 1474,  566220,  566234,  566256, 14, 22, 25, }, // Philippines
    {  456, 1486,       0,  566281,       0,  0, 22,  0, }, // Phoenix_Islands
    {  456, 1502,  566303,  566334,  566373, 31, 39, 39, }, // Pierre_Miquelon
    {  456, 1518,       0,  566412,       0,  0, 14,  0, }, // Pitcairn
    {  456, 1527,       0,  566426,       0,  0, 10,  0, }, // Ponape
    {  456, 1534,       0,  566436,       0,  0, 15,  0, }, // Pyongyang
    {  456, 1544,  566451,  566465,  566484, 14, 19, 22, }, // Qyzylorda
    {  456, 1554,       0,  566506,       0,  0, 14,  0, }, // Reunion
    {  456, 1562,       0,  566520,       0,  0, 10,  0, }, // Rothera
    {  456, 1570,  566530,  566541,  566560, 11, 19, 22, }, // Sakhalin
    {  456, 1579,  566582,  566591,  566605,  9, 14, 17, }, // Samara
    {  456, 1596,  566622,  566631,  566648,  9, 17, 17, }, // Samoa
    {  456, 1602,       0,  566665,       0,  0, 15,  0, }, // Seychelles
    {  456, 1624,       0,  566680,       0,  0, 23,  0, }, // Singapore
    {  456, 1634,       0,  566703,       0,  0, 21,  0, }, // Solomon
    {  456, 1642,       0,  566724,       0,  0, 17,  0, }, // South_Georgia
    {  456, 1656,       0,  566741,       0,  0, 13,  0, }, // Suriname
    {  456, 1676,       0,  566754,       0,  0,  8,  0, }, // Syowa
    {  456, 1682,       0,  566762,       0,  0, 10,  0, }, // Tahiti
    {  456, 1689,  566772,  566782,  566800, 10, 18, 18, }, // Taipei
    {  456, 1696,       0,  566818,       0,  0, 15,  0, }, // Tajikistan
    {  456, 1724,       0,  566833,       0,  0, 11,  0, }, // Tokelau
    {  456, 1732,  566844,  566854,  566872, 10, 18, 21, }, // Tonga
    {  456, 1738,       0,  566893,       0,  0,  9,  0, }, // Truk
    {  456, 1750,  566902,  566920,  566946, 18, 26, 25, }, // Turkmenistan
    {  456, 1763,       0,  566971,       0,  0, 10,  0, }, // Tuvalu
    {  456, 1777,  566981,  566991,  567009, 10, 18, 21, }, // Uruguay
    {  456, 1792,  567030,  567045,  567068, 15, 23, 22, }, // Uzbekistan
    {  456, 1803,  567090,  567101,  567120, 11, 19, 18, }, // Vanuatu
    {  456, 1811,       0,  567138,       0,  0, 13,  0, }, // Venezuela
    {  456, 1821,  567151,  567169,  567195, 18, 26, 29, }, // Vladivostok
    {  456, 1833,  567224,  567239,  567262, 15, 23, 26, }, // Volgograd
    {  456, 1843,       0,  567288,       0,  0, 13,  0, }, // Vostok
    {  456, 1850,       0,  567301,       0,  0, 15,  0, }, // Wake
    {  456, 1855,       0,  567316,       0,  0, 22,  0, }, // Wallis
    {  456, 1862,  567338,  567350,  567370, 12, 20, 23, }, // Yakutsk
    {  456, 1870,  567393,  567411,  567437, 18, 26, 29, }, // Yekaterinburg
    {  456, 1892,       0,  567466,       0,  0, 10,  0, }, // Yukon
    {  461,  638,  567476,  567502,  567539, 26, 37, 35, }, // Europe_Central Ossetic/Cyrillic/Russia
    {  461,  653,  567574,  567597,  567631, 23, 34, 32, }, // Europe_Eastern
    {  461,  691,  567663,  567689,  567726, 26, 37, 35, }, // Europe_Western
    {  461,  775,  567761,  567780,  567810, 19, 30, 28, }, // Georgia
    {  461,  799,       0,  567838,       0,  0, 25,  0, }, // GMT
    {  461, 1305,  567863,  567878,  567904, 15, 26, 24, }, // Moscow
    {  462,  799,       0,    2861,       0,  0, 19,  0, }, // GMT Papiamento/Latin/Curacao
    {  464,    5,       0,  567928,       0,  0, 13,  0, }, // Afghanistan Pashto/Arabic/Afghanistan
    {  464,   17,       0,  567941,       0,  0, 16,  0, }, // Africa_Central
    {  464,   32,       0,  567957,       0,  0, 15,  0, }, // Africa_Eastern
    {  464,   65,       0,  567972,       0,  0, 23,  0, }, // Africa_Southern
    {  464,   81,  567995,  568012,  568038, 17, 26, 26, }, // Africa_Western
    {  464,  107,  568064,  568074,  568091, 10, 17, 24, }, // Alaska
    {  464,  135,  568115,  568126,  568144, 11, 18, 16, }, // Amazon
    {  464,  142,  568160,  568169,  568185,  9, 16, 23, }, // America_Central
    {  464,  158,  568208,  568216,  568231,  8, 15, 19, }, // America_Eastern
    {  464,  174,  568250,  568261,  568277, 11, 16, 20, }, // America_Mountain
    {  464,  191,  568297,  568306,  568324,  9, 18, 20, }, // America_Pacific
    {  464,  214,  568344,  568352,  568367,  8, 15, 22, }, // Apia
    {  464,  232,  568389,  568397,  568412,  8, 15, 22, }, // Arabian
    {  464,  240,  568434,  568446,  568465, 12, 19, 17, }, // Argentina
    {  464,  250,  568482,  568501,  568527, 19, 26, 24, }, // Argentina_Western
    {  464,  268,  568551,  568563,  568582, 12, 19, 17, }, // Armenia
    {  464,  286,  568599,  568611,  568630, 12, 19, 23, }, // Atlantic
    {  464,  295,  568653,  568671,  568696, 18, 25, 32, }, // Australia_Central
    {  464,  313,  568728,  568753,  568785, 25, 32, 39, }, // Australia_CentralWestern
    {  464,  338,  568824,  568841,  568865, 17, 24, 31, }, // Australia_Eastern
    {  464,  356,  568896,  568915,  568941, 19, 26, 33, }, // Australia_Western
    {  464,  374,  568974,  568989,  569009, 15, 20, 19, }, // Azerbaijan
    {  464,  385,  569028,  569038,  569055, 10, 17, 15, }, // Azores
    {  464,  397,  569070,  569083,  569102, 13, 19, 18, }, // Bangladesh
    {  464,  415,       0,  569120,       0,  0, 10,  0, }, // Bhutan
    {  464,  422,       0,  569130,       0,  0, 11,  0, }, // Bolivia
    {  464,  437,  569141,  569151,  569168, 10, 17, 15, }, // Brasilia
    {  464,  454,       0,  569183,       0,  0, 21,  0, }, // Brunei
    {  464,  461,  569204,  569216,  569235, 12, 19, 16, }, // Cape_Verde
    {  464,  478,       0,  569251,       0,  0, 16,  0, }, // Chamorro
    {  464,  487,  569267,  569276,  569292,  9, 16, 23, }, // Chatham
    {  464,  495,  569315,  569322,  569336,  7, 14, 12, }, // Chile
    {  464,  501,  569348,  569355,  569369,  7, 14, 20, }, // China
    {  464,  507,  569389,  569402,  569422, 13, 20, 18, }, // Choibalsan
    {  464,  518,       0,  569440,       0,  0, 15,  0, }, // Christmas
    {  464,  528,       0,  569455,       0,  0, 18,  0, }, // Cocos
    {  464,  534,  569473,  569484,  569502, 11, 18, 16, }, // Colombia
    {  464,  543,  569518,  569534,  569557, 16, 23, 25, }, // Cook
    {  464,  548,  569582,  569591,  569607,  9, 16, 20, }, // Cuba
    {  464,  559,       0,  569627,       0,  0,  8,  0, }, // Davis
    {  464,  575,       0,  569635,       0,  0, 14,  0, }, // DumontDUrville
    {  464,  612,       0,  569649,       0,  0, 14,  0, }, // East_Timor
    {  464,  623,  569663,  569677,  569698, 14, 21, 19, }, // Easter
    {  464,  630,       0,  569717,       0,  0, 13,  0, }, // Ecuador
    {  464,  638,  569730,  569747,  569771, 17, 24, 21, }, // Europe_Central
    {  464,  653,  569792,  569808,  569831, 16, 23, 21, }, // Europe_Eastern
    {  464,  668,       0,  569852,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  464,  691,  569872,  569890,  569915, 18, 25, 23, }, // Europe_Western
    {  464,  706,  569938,  569953,  569977, 15, 24, 22, }, // Falkland
    {  464,  715,  569999,  570006,  570020,  7, 14, 14, }, // Fiji
    {  464,  720,       0,  570034,       0,  0, 18,  0, }, // French_Guiana
    {  464,  734,       0,  570052,       0,  0, 29,  0, }, // French_Southern
    {  464,  757,       0,  570081,       0,  0, 11,  0, }, // Galapagos
    {  464,  767,       0,  570092,       0,  0, 10,  0, }, // Gambier
    {  464,  775,  570102,  570112,  570129, 10, 17, 19, }, // Georgia
    {  464,  783,       0,  570148,       0,  0, 18,  0, }, // Gilbert_Islands
    {  464,  799,       0,  570166,       0,  0, 18,  0, }, // GMT
    {  464,  841,  570184,  570203,  570229, 19, 26, 24, }, // Greenland_Eastern
    {  464,  859,  570253,  570272,  570298, 19, 26, 24, }, // Greenland_Western
    {  464,  882,       0,  570322,       0,  0, 15,  0, }, // Gulf
    {  464,  887,       0,  570337,       0,  0, 13,  0, }, // Guyana
    {  464,  894,  570350,  570367,  570391, 17, 24, 26, }, // Hawaii_Aleutian
    {  464,  910,  570417,  570430,  570450, 13, 20, 18, }, // Hong_Kong
    {  464,  920,  570468,  570476,  570491,  8, 15, 13, }, // Hovd
    {  464,  925,       0,  570504,       0,  0, 14,  0, }, // India
    {  464,  931,       0,  570518,       0,  0, 15,  0, }, // Indian_Ocean
    {  464,  944,       0,  570533,       0,  0, 13,  0, }, // Indochina
    {  464,  954,       0,  570546,       0,  0, 18,  0, }, // Indonesia_Central
    {  464,  972,       0,  570564,       0,  0, 13,  0, }, // Indonesia_Eastern
    {  464,  990,       0,  570577,       0,  0, 20,  0, }, // Indonesia_Western
    {  464, 1008,  570597,  570606,  570622,  9, 16, 23, }, // Iran
    {  464, 1019,  570645,  570656,  570674, 11, 18, 16, }, // Irkutsk
    {  464, 1027,  570690,  570701,  570719, 11, 18, 25, }, // Israel
    {  464, 1034,  570744,  570753,  570769,  9, 16, 23, }, // Japan
    {  464, 1058,       0,  570792,       0,  0, 17,  0, }, // Kazakhstan_Eastern
    {  464, 1077,       0,  570809,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  464, 1106,  570828,  570839,  570857, 11, 18, 25, }, // Korea
    {  464, 1112,       0,  570882,       0,  0, 10,  0, }, // Kosrae
    {  464, 1119,  570892,  570907,  570929, 15, 22, 20, }, // Krasnoyarsk
    {  464, 1151,       0,  570949,       0,  0, 13,  0, }, // Kyrgystan
    {  464, 1175,       0,  570962,       0,  0, 17,  0, }, // Line_Islands
    {  464, 1188,  570979,  570991,  571010, 12, 19, 26, }, // Lord_Howe
    {  464, 1204,  571036,  571046,  571063, 10, 17, 15, }, // Magadan
    {  464, 1219,       0,  571078,       0,  0, 12,  0, }, // Malaysia
    {  464, 1228,       0,  571090,       0,  0, 10,  0, }, // Maldives
    {  464, 1237,       0,  571100,       0,  0, 10,  0, }, // Marquesas
    {  464, 1247,       0,  571110,       0,  0, 18,  0, }, // Marshall_Islands
    {  464, 1264,  571128,  571139,  571157, 11, 18, 18, }, // Mauritius
    {  464, 1274,       0,  571175,       0,  0,  9,  0, }, // Mawson
    {  464, 1281,  571184,  571200,  571223, 16, 23, 25, }, // Mexico_Pacific
    {  464, 1296,  571248,  571262,  571283, 14, 21, 19, }, // Mongolia
    {  464, 1305,  571302,  571311,  571327,  9, 16, 13, }, // Moscow
    {  464, 1312,       0,  571340,       0,  0, 11,  0, }, // Myanmar
    {  464, 1320,       0,  571351,       0,  0,  9,  0, }, // Nauru
    {  464, 1326,       0,  571360,       0,  0,  9,  0, }, // Nepal
    {  464, 1332,  571369,  571386,  571410, 17, 24, 21, }, // New_Caledonia
    {  464, 1346,  571431,  571445,  571466, 14, 21, 28, }, // New_Zealand
    {  464, 1358,  571494,  571511,  571536, 17, 25, 31, }, // Newfoundland
    {  464, 1371,       0,  571567,       0,  0,  8,  0, }, // Niue
    {  464, 1376,  571575,  571590,  571615, 15, 25, 23, }, // Norfolk
    {  464, 1384,  571638,  571660,  571689, 22, 29, 27, }, // Noronha
    {  464, 1406,  571716,  571729,  571749, 13, 20, 18, }, // Novosibirsk
    {  464, 1418,  571767,  571776,  571792,  9, 16, 14, }, // Omsk
    {  464, 1428,  571806,  571817,  571835, 11, 18, 16, }, // Pakistan
    {  464, 1437,       0,  571851,       0,  0,  9,  0, }, // Palau
    {  464, 1443,       0,  571860,       0,  0, 17,  0, }, // Papua_New_Guinea
    {  464, 1460,  571877,  571889,  571908, 12, 19, 17, }, // Paraguay
    {  464, 1469,  571925,  571933,  571948,  8, 15, 13, }, // Peru
    {  464, 1474,  571961,  571971,  571988, 10, 17, 15, }, // Philippines
    {  464, 1486,       0,  572003,       0,  0, 16,  0, }, // Phoenix_Islands
    {  464, 1502,  572019,  572042,  572072, 23, 30, 32, }, // Pierre_Miquelon
    {  464, 1518,       0,  572104,       0,  0, 13,  0, }, // Pitcairn
    {  464, 1527,       0,  572117,       0,  0,  9,  0, }, // Ponape
    {  464, 1534,       0,  572126,       0,  0, 14,  0, }, // Pyongyang
    {  464, 1554,       0,  572140,       0,  0, 12,  0, }, // Reunion
    {  464, 1562,       0,  572152,       0,  0,  9,  0, }, // Rothera
    {  464, 1570,  572161,  572170,  572186,  9, 16, 14, }, // Sakhalin
    {  464, 1596,  572200,  572208,  572223,  8, 15, 22, }, // Samoa
    {  464, 1602,       0,  572245,       0,  0, 10,  0, }, // Seychelles
    {  464, 1624,       0,  572255,       0,  0, 18,  0, }, // Singapore
    {  464, 1634,       0,  572273,       0,  0, 19,  0, }, // Solomon
    {  464, 1642,       0,  572292,       0,  0, 17,  0, }, // South_Georgia
    {  464, 1656,       0,  572309,       0,  0, 11,  0, }, // Suriname
    {  464, 1676,       0,  572320,       0,  0,  8,  0, }, // Syowa
    {  464, 1682,       0,  572328,       0,  0,  9,  0, }, // Tahiti
    {  464, 1689,  572337,  572346,  572362,  9, 16, 23, }, // Taipei
    {  464, 1696,       0,  572385,       0,  0, 12,  0, }, // Tajikistan
    {  464, 1724,       0,  572397,       0,  0, 11,  0, }, // Tokelau
    {  464, 1732,  572408,  572417,  572434,  9, 17, 14, }, // Tonga
    {  464, 1738,       0,  572448,       0,  0,  7,  0, }, // Truk
    {  464, 1750,  572455,  572469,  572489, 14, 20, 18, }, // Turkmenistan
    {  464, 1763,       0,  572507,       0,  0, 10,  0, }, // Tuvalu
    {  464, 1777,  572517,  572529,  572548, 12, 19, 17, }, // Uruguay
    {  464, 1792,  572565,  572577,  572596, 12, 19, 17, }, // Uzbekistan
    {  464, 1803,  572613,  572624,  572641, 11, 17, 15, }, // Vanuatu
    {  464, 1811,       0,  572656,       0,  0, 12,  0, }, // Venezuela
    {  464, 1821,  572668,  572682,  572703, 14, 21, 19, }, // Vladivostok
    {  464, 1833,  572722,  572735,  572755, 13, 20, 18, }, // Volgograd
    {  464, 1843,       0,  572773,       0,  0, 10,  0, }, // Vostok
    {  464, 1850,       0,  572783,       0,  0, 12,  0, }, // Wake
    {  464, 1855,       0,  572795,       0,  0, 18,  0, }, // Wallis
    {  464, 1862,  572813,  572824,  572842, 11, 18, 18, }, // Yakutsk
    {  464, 1870,  572860,  572875,  572898, 15, 23, 22, }, // Yekaterinburg
    {  464, 1892,       0,  572920,       0,  0, 11,  0, }, // Yukon
    {  465,  107,  568064,  568074,  572931, 10, 17, 26, }, // Alaska Pashto/Arabic/Pakistan
    {  465,  142,  568160,  568169,  572957,  9, 16, 18, }, // America_Central
    {  465,  158,  568208,  568216,  572975,  8, 15, 19, }, // America_Eastern
    {  465,  174,  568250,  568261,  572994, 11, 16, 20, }, // America_Mountain
    {  465,  191,  568297,  568306,  573014,  9, 18, 20, }, // America_Pacific
    {  465,  214,  568344,  568352,  573034,  8, 15, 17, }, // Apia
    {  465,  232,  568389,  568397,  573051,  8, 15, 18, }, // Arabian
    {  465,  286,  568599,  568611,  573069, 12, 19, 23, }, // Atlantic
    {  465,  356,  568896,  568915,  573092, 19, 26, 28, }, // Australia_Western
    {  465,  501,  569348,  569355,  573120,  7, 14, 20, }, // China
    {  465,  548,  569582,  569591,  573140,  9, 16, 20, }, // Cuba
    {  465,  691,  573160,  573177,  573202, 17, 25, 25, }, // Europe_Western
    {  465,  734,       0,  573227,       0,  0, 29,  0, }, // French_Southern
    {  465,  894,  570350,  570367,  573256, 17, 24, 26, }, // Hawaii_Aleutian
    {  465, 1008,  570597,  570606,  573282,  9, 16, 18, }, // Iran
    {  465, 1027,  570690,  570701,  573300, 11, 18, 21, }, // Israel
    {  465, 1034,  570744,  570753,  573321,  9, 16, 20, }, // Japan
    {  465, 1106,  570828,  570839,  573341, 11, 18, 25, }, // Korea
    {  465, 1175,       0,  573366,       0,  0, 17,  0, }, // Line_Islands
    {  465, 1188,  570979,  570991,  573383, 12, 19, 23, }, // Lord_Howe
    {  465, 1281,  571184,  571200,  573406, 16, 23, 25, }, // Mexico_Pacific
    {  465, 1346,  571431,  571445,  573431, 14, 21, 29, }, // New_Zealand
    {  465, 1358,  571494,  571511,  573460, 17, 25, 27, }, // Newfoundland
    {  465, 1502,  572019,  572042,  573487, 23, 30, 32, }, // Pierre_Miquelon
    {  465, 1596,  572200,  572208,  573519,  8, 15, 24, }, // Samoa
    {  466,    5,       0,  573543,       0,  0, 13,  0, }, // Afghanistan Persian/Arabic/Iran
    {  466,   17,       0,  573556,       0,  0, 15,  0, }, // Africa_Central
    {  466,   32,       0,  573571,       0,  0, 14,  0, }, // Africa_Eastern
    {  466,   65,       0,  573585,       0,  0, 20,  0, }, // Africa_Southern
    {  466,   81,  573605,  573619,  573638, 14, 19, 23, }, // Africa_Western
    {  466,  107,  573661,  573671,  573686, 10, 15, 19, }, // Alaska
    {  466,  128,  573705,  573716,  573732, 11, 16, 20, }, // Almaty
    {  466,  135,  573752,  573762,  573777, 10, 15, 19, }, // Amazon
    {  466,  142,  573796,  573811,  573831, 15, 20, 24, }, // America_Central
    {  466,  158,  573855,  573869,  573888, 14, 19, 23, }, // America_Eastern
    {  466,  174,  573911,  573930,  573954, 19, 24, 28, }, // America_Mountain
    {  466,  191,  573982,  573996,  574015, 14, 19, 23, }, // America_Pacific
    {  466,  207,  574038,  574048,  574063, 10, 15, 19, }, // Anadyr
    {  466,  214,  574082,  574090,  574103,  8, 13, 17, }, // Apia
    {  466,  232,  574120,  574131,  574147, 11, 16, 20, }, // Arabian
    {  466,  240,  574167,  574179,  574196, 12, 17, 21, }, // Argentina
    {  466,  250,  574217,  574233,  574254, 16, 21, 25, }, // Argentina_Western
    {  466,  268,  574279,  574291,  574308, 12, 17, 21, }, // Armenia
    {  466,  286,  574329,  574341,  574358, 12, 17, 21, }, // Atlantic
    {  466,  295,  574379,  574396,  574418, 17, 22, 26, }, // Australia_Central
    {  466,  313,  574444,  574465,  574491, 21, 26, 30, }, // Australia_CentralWestern
    {  466,  338,  574521,  574537,  574558, 16, 21, 25, }, // Australia_Eastern
    {  466,  356,  574583,  574599,  574620, 16, 21, 25, }, // Australia_Western
    {  466,  374,  574645,  574665,  574690, 20, 25, 29, }, // Azerbaijan
    {  466,  385,  574719,  574727,  574740,  8, 13, 17, }, // Azores
    {  466,  397,  574757,  574768,  574784, 11, 16, 20, }, // Bangladesh
    {  466,  415,       0,  574804,       0,  0,  9,  0, }, // Bhutan
    {  466,  422,       0,  574813,       0,  0, 10,  0, }, // Bolivia
    {  466,  437,  574823,  574835,  574852, 12, 17, 21, }, // Brasilia
    {  466,  454,       0,  574873,       0,  0, 20,  0, }, // Brunei
    {  466,  461,  574893,  574904,  574920, 11, 16, 20, }, // Cape_Verde
    {  466,  478,       0,  574940,       0,  0, 15,  0, }, // Chamorro
    {  466,  487,  574955,  574965,  574980, 10, 15, 19, }, // Chatham
    {  466,  495,  574999,  575007,  575020,  8, 13, 17, }, // Chile
    {  466,  501,  575037,  575044,  575056,  7, 12, 16, }, // China
    {  466,  507,  575072,  575085,  575103, 13, 18, 22, }, // Choibalsan
    {  466,  518,       0,  575125,       0,  0, 17,  0, }, // Christmas
    {  466,  528,       0,  575142,       0,  0, 15,  0, }, // Cocos
    {  466,  534,  575157,  575167,  575182, 10, 15, 19, }, // Colombia
    {  466,  543,  575142,  575201,  575219, 13, 18, 22, }, // Cook
    {  466,  548,  575241,  575249,  575262,  8, 13, 17, }, // Cuba
    {  466,  559,       0,  575279,       0,  0,  9,  0, }, // Davis
    {  466,  575,       0,  575288,       0,  0, 16,  0, }, // DumontDUrville
    {  466,  612,       0,  575304,       0,  0, 14,  0, }, // East_Timor
    {  466,  623,  575318,  575334,  575355, 16, 21, 25, }, // Easter
    {  466,  630,       0,  575380,       0,  0, 11,  0, }, // Ecuador
    {  466,  638,  575391,  575405,  575424, 14, 19, 23, }, // Europe_Central
    {  466,  653,  575447,  575460,  575478, 13, 18, 22, }, // Europe_Eastern
    {  466,  668,       0,  575500,       0,  0, 36,  0, }, // Europe_Further_Eastern
    {  466,  691,  575536,  575549,  575567, 13, 18, 22, }, // Europe_Western
    {  466,  706,  575589,  575606,  575628, 17, 22, 26, }, // Falkland
    {  466,  715,  575654,  575662,  575675,  8, 13, 17, }, // Fiji
    {  466,  720,       0,  575692,       0,  0, 16,  0, }, // French_Guiana
    {  466,  734,       0,  575708,       0,  0, 37,  0, }, // French_Southern
    {  466,  757,       0,  575745,       0,  0, 13,  0, }, // Galapagos
    {  466,  767,       0,  575758,       0,  0, 10,  0, }, // Gambier
    {  466,  775,  575768,  575779,  575795, 11, 16, 20, }, // Georgia
    {  466,  783,       0,  575815,       0,  0, 16,  0, }, // Gilbert_Islands
    {  466,  799,       0,  575831,       0,  0, 11,  0, }, // GMT
    {  466,  841,  575842,  575857,  575877, 15, 20, 24, }, // Greenland_Eastern
    {  466,  859,  575901,  575916,  575936, 15, 20, 24, }, // Greenland_Western
    {  466,  877,       0,  575960,       0,  0, 13,  0, }, // Guam
    {  466,  882,       0,  575973,       0,  0, 18,  0, }, // Gulf
    {  466,  887,       0,  575692,       0,  0,  9,  0, }, // Guyana
    {  466,  894,  575991,  576007,  576028, 16, 21, 25, }, // Hawaii_Aleutian
    {  466,  910,  576053,  576064,  576080, 11, 16, 20, }, // Hong_Kong
    {  466,  920,  576100,  576108,  576121,  8, 13, 17, }, // Hovd
    {  466,  925,       0,  576138,       0,  0,  7,  0, }, // India
    {  466,  931,       0,  576145,       0,  0, 15,  0, }, // Indian_Ocean
    {  466,  944,       0,  576160,       0,  0, 11,  0, }, // Indochina
    {  466,  954,       0,  576171,       0,  0, 16,  0, }, // Indonesia_Central
    {  466,  972,       0,  576187,       0,  0, 15,  0, }, // Indonesia_Eastern
    {  466,  990,       0,  576202,       0,  0, 15,  0, }, // Indonesia_Western
    {  466, 1008,  576217,  576226,  576240,  9, 14, 18, }, // Iran
    {  466, 1019,  576258,  576270,  576287, 12, 17, 21, }, // Irkutsk
    {  466, 1027,  576308,  576319,  576335, 11, 16, 20, }, // Israel
    {  466, 1034,  576355,  576363,  576376,  8, 13, 17, }, // Japan
    {  466, 1040,  576393,  576418,  576448, 25, 30, 34, }, // Kamchatka
    {  466, 1058,       0,  576482,       0,  0, 16,  0, }, // Kazakhstan_Eastern
    {  466, 1077,       0,  576498,       0,  0, 16,  0, }, // Kazakhstan_Western
    {  466, 1106,  576514,  576521,  576533,  7, 12, 16, }, // Korea
    {  466, 1112,       0,  576549,       0,  0, 10,  0, }, // Kosrae
    {  466, 1119,  576559,  576574,  576594, 15, 20, 24, }, // Krasnoyarsk
    {  466, 1151,       0,  576618,       0,  0, 13,  0, }, // Kyrgystan
    {  466, 1161,       0,  576631,       0,  0,  9,  0, }, // Lanka
    {  466, 1175,       0,  576640,       0,  0, 14,  0, }, // Line_Islands
    {  466, 1188,  576654,  576663,  576677,  9, 14, 18, }, // Lord_Howe
    {  466, 1198,  576695,  576705,  576720, 10, 15, 19, }, // Macau
    {  466, 1204,  576739,  576750,  576766, 11, 16, 20, }, // Magadan
    {  466, 1219,       0,  576786,       0,  0,  9,  0, }, // Malaysia
    {  466, 1228,       0,  576795,       0,  0, 10,  0, }, // Maldives
    {  466, 1237,       0,  576805,       0,  0, 11,  0, }, // Marquesas
    {  466, 1247,       0,  576816,       0,  0, 16,  0, }, // Marshall_Islands
    {  466, 1264,  576832,  576841,  576855,  9, 14, 18, }, // Mauritius
    {  466, 1274,       0,  576873,       0,  0, 10,  0, }, // Mawson
    {  466, 1281,  576883,  576896,  576914, 13, 18, 22, }, // Mexico_Pacific
    {  466, 1296,  576936,  576951,  576971, 15, 20, 24, }, // Mongolia
    {  466, 1305,  576995,  577003,  577016,  8, 13, 17, }, // Moscow
    {  466, 1312,       0,  577033,       0,  0, 11,  0, }, // Myanmar
    {  466, 1320,       0,  577044,       0,  0, 10,  0, }, // Nauru
    {  466, 1326,       0,  577054,       0,  0,  8,  0, }, // Nepal
    {  466, 1332,  577062,  577080,  577103, 18, 23, 27, }, // New_Caledonia
    {  466, 1346,  577130,  577142,  577159, 12, 17, 21, }, // New_Zealand
    {  466, 1358,  577180,  577194,  577213, 14, 19, 23, }, // Newfoundland
    {  466, 1371,       0,  577236,       0,  0,  9,  0, }, // Niue
    {  466, 1376,  577245,  577263,  577286, 18, 23, 27, }, // Norfolk
    {  466, 1384,  577313,  577335,  577362, 22, 27, 31, }, // Noronha
    {  466, 1392,       0,  577393,       0,  0, 24,  0, }, // North_Mariana
    {  466, 1406,  577417,  577431,  577450, 14, 19, 23, }, // Novosibirsk
    {  466, 1418,  577473,  577482,  577496,  9, 14, 18, }, // Omsk
    {  466, 1428,  577514,  577525,  577541, 11, 16, 20, }, // Pakistan
    {  466, 1437,       0,  577561,       0,  0, 10,  0, }, // Palau
    {  466, 1443,       0,  577571,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  466, 1460,  577589,  577601,  577618, 12, 17, 21, }, // Paraguay
    {  466, 1469,  577639,  577646,  577658,  7, 12, 16, }, // Peru
    {  466, 1474,  577674,  577685,  577701, 11, 16, 20, }, // Philippines
    {  466, 1486,       0,  577721,       0,  0, 16,  0, }, // Phoenix_Islands
    {  466, 1502,  577737,  577757,  577782, 20, 25, 29, }, // Pierre_Miquelon
    {  466, 1518,       0,  577811,       0,  0, 12,  0, }, // Pitcairn
    {  466, 1527,       0,  577823,       0,  0, 10,  0, }, // Ponape
    {  466, 1534,       0,  577833,       0,  0, 14,  0, }, // Pyongyang
    {  466, 1544,  577847,  577862,  577882, 15, 20, 24, }, // Qyzylorda
    {  466, 1554,       0,  577906,       0,  0, 11,  0, }, // Reunion
    {  466, 1562,       0,  577917,       0,  0,  9,  0, }, // Rothera
    {  466, 1570,  577926,  577937,  577953, 11, 16, 20, }, // Sakhalin
    {  466, 1579,  577973,  577983,  577998, 10, 15, 19, }, // Samara
    {  466, 1596,  578017,  578026,  578040,  9, 14, 18, }, // Samoa
    {  466, 1602,       0,  578058,       0,  0,  8,  0, }, // Seychelles
    {  466, 1624,       0,  578066,       0,  0, 11,  0, }, // Singapore
    {  466, 1634,       0,  578077,       0,  0, 16,  0, }, // Solomon
    {  466, 1642,       0,  578093,       0,  0, 17,  0, }, // South_Georgia
    {  466, 1656,       0,  578110,       0,  0, 11,  0, }, // Suriname
    {  466, 1676,       0,  578121,       0,  0,  8,  0, }, // Syowa
    {  466, 1682,       0,  578129,       0,  0, 10,  0, }, // Tahiti
    {  466, 1689,  578139,  578148,  578162,  9, 14, 18, }, // Taipei
    {  466, 1696,       0,  578180,       0,  0, 13,  0, }, // Tajikistan
    {  466, 1724,       0,  578193,       0,  0, 11,  0, }, // Tokelau
    {  466, 1732,  578204,  578213,  578227,  9, 14, 18, }, // Tonga
    {  466, 1738,       0,  578245,       0,  0,  9,  0, }, // Truk
    {  466, 1750,  578254,  578267,  578285, 13, 18, 22, }, // Turkmenistan
    {  466, 1763,       0,  578307,       0,  0, 10,  0, }, // Tuvalu
    {  466, 1777,  578317,  578328,  578344, 11, 16, 20, }, // Uruguay
    {  466, 1792,  578364,  578376,  578393, 12, 17, 21, }, // Uzbekistan
    {  466, 1803,  578414,  578424,  578439, 10, 15, 19, }, // Vanuatu
    {  466, 1811,       0,  578458,       0,  0, 11,  0, }, // Venezuela
    {  466, 1821,  578469,  578484,  578504, 15, 20, 24, }, // Vladivostok
    {  466, 1833,  578528,  578540,  578557, 12, 17, 21, }, // Volgograd
    {  466, 1843,       0,  578578,       0,  0,  9,  0, }, // Vostok
    {  466, 1850,       0,  578587,       0,  0, 14,  0, }, // Wake
    {  466, 1855,       0,  578601,       0,  0, 18,  0, }, // Wallis
    {  466, 1862,  578619,  578630,  578646, 11, 16, 20, }, // Yakutsk
    {  466, 1870,  578666,  578681,  578701, 15, 20, 24, }, // Yekaterinburg
    {  466, 1892,       0,  578725,       0,  0,  9,  0, }, // Yukon
    {  468,    5,       0,  578734,       0,  0, 15,  0, }, // Afghanistan Polish/Latin/Poland
    {  468,   17,       0,  578749,       0,  0, 23,  0, }, // Africa_Central
    {  468,   32,       0,  578772,       0,  0, 24,  0, }, // Africa_Eastern
    {  468,   65,       0,  578796,       0,  0, 25,  0, }, // Africa_Southern
    {  468,   81,  578821,  578845,  578881, 24, 36, 30, }, // Africa_Western
    {  468,  107,  578911,  578922,  578947, 11, 25, 19, }, // Alaska
    {  468,  128,  578966,  578978,  579002, 12, 24, 18, }, // Almaty
    {  468,  135,  579020,  579034,  579060, 14, 26, 20, }, // Amazon
    {  468,  142,  579080,  579104,  579140, 24, 36, 30, }, // America_Central
    {  468,  158,  579170,  579195,  579232, 25, 37, 31, }, // America_Eastern
    {  468,  174,  579263,  579274,  579297, 11, 23, 17, }, // America_Mountain
    {  468,  191,  579314,  579329,  579356, 15, 27, 21, }, // America_Pacific
    {  468,  207,  579377,  579388,  579411, 11, 23, 17, }, // Anadyr
    {  468,  214,  579428,  579437,  579460,  9, 23, 17, }, // Apia
    {  468,  219,  579477,  579491,  579517, 14, 26, 20, }, // Aqtau
    {  468,  225,  579537,  579553,  579581, 16, 28, 22, }, // Aqtobe
    {  468,  232,  579603,  579624,  579659, 21, 35, 29, }, // Arabian
    {  468,  240,  579688,  579702,  579730, 14, 28, 22, }, // Argentina
    {  468,  250,  579752,  579776,  579814, 24, 38, 32, }, // Argentina_Western
    {  468,  268,  579846,  579858,  579884, 12, 26, 20, }, // Armenia
    {  468,  286,  579904,  579919,  579946, 15, 27, 21, }, // Atlantic
    {  468,  295,  579967,  579992,  580029, 25, 37, 31, }, // Australia_Central
    {  468,  313,  580060,  580095,  580142, 35, 47, 41, }, // Australia_CentralWestern
    {  468,  338,  580183,  580209,  580247, 26, 38, 32, }, // Australia_Eastern
    {  468,  356,  580279,  580305,  580343, 26, 38, 32, }, // Australia_Western
    {  468,  374,  580375,  580391,  580421, 16, 30, 24, }, // Azerbaijan
    {  468,  385,  580445,  580455,  580479, 10, 24, 18, }, // Azores
    {  468,  397,  580497,  580512,  580541, 15, 29, 23, }, // Bangladesh
    {  468,  415,       0,  580564,       0,  0, 11,  0, }, // Bhutan
    {  468,  422,       0,  580575,       0,  0, 12,  0, }, // Bolivia
    {  468,  437,  580587,  580600,  580627, 13, 27, 21, }, // Brasilia
    {  468,  454,       0,  580648,       0,  0, 11,  0, }, // Brunei
    {  468,  461,  580659,  580689,  580733, 30, 44, 38, }, // Cape_Verde
    {  468,  478,       0,  580771,       0,  0, 13,  0, }, // Chamorro
    {  468,  487,  580784,  580796,  580822, 12, 26, 20, }, // Chatham
    {  468,  495,  580842,  580852,  580876, 10, 24, 18, }, // Chile
    {  468,  501,  580894,  580904,  580928, 10, 24, 18, }, // China
    {  468,  507,  580946,  580961,  580990, 15, 29, 23, }, // Choibalsan
    {  468,  518,       0,  581013,       0,  0, 28,  0, }, // Christmas
    {  468,  528,       0,  581041,       0,  0, 19,  0, }, // Cocos
    {  468,  534,  581060,  581073,  581100, 13, 27, 21, }, // Colombia
    {  468,  543,  581121,  581137,  581167, 16, 30, 24, }, // Cook
    {  468,  548,  581191,  581200,  581223,  9, 23, 17, }, // Cuba
    {  468,  559,       0,  581240,       0,  0, 10,  0, }, // Davis
    {  468,  575,       0,  581250,       0,  0, 21,  0, }, // DumontDUrville
    {  468,  612,       0,  581271,       0,  0, 19,  0, }, // East_Timor
    {  468,  623,  581290,  581312,  581348, 22, 36, 30, }, // Easter
    {  468,  630,       0,  581378,       0,  0, 12,  0, }, // Ecuador
    {  468,  638,  581390,  581413,  581448, 23, 35, 29, }, // Europe_Central
    {  468,  653,  581477,  581501,  581537, 24, 36, 30, }, // Europe_Eastern
    {  468,  668,       0,  581567,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  468,  691,  581598,  581622,  581658, 24, 36, 30, }, // Europe_Western
    {  468,  706,  581688,  581702,  581730, 14, 28, 22, }, // Falkland
    {  468,  715,  581752,  581762,  581786, 10, 24, 18, }, // Fiji
    {  468,  720,       0,  581804,       0,  0, 21,  0, }, // French_Guiana
    {  468,  734,       0,  581825,       0,  0, 51,  0, }, // French_Southern
    {  468,  757,       0,  581876,       0,  0, 14,  0, }, // Galapagos
    {  468,  767,       0,  581890,       0,  0, 19,  0, }, // Gambier
    {  468,  775,  581909,  581920,  581945, 11, 25, 19, }, // Georgia
    {  468,  783,       0,  581964,       0,  0, 19,  0, }, // Gilbert_Islands
    {  468,  799,       0,  581983,       0,  0, 16,  0, }, // GMT
    {  468,  841,  581999,  582024,  582063, 25, 39, 33, }, // Greenland_Eastern
    {  468,  859,  582096,  582121,  582160, 25, 39, 33, }, // Greenland_Western
    {  468,  882,       0,  582193,       0,  0, 18,  0, }, // Gulf
    {  468,  887,       0,  581804,       0,  0, 11,  0, }, // Guyana
    {  468,  894,  582211,  582229,  582261, 18, 32, 26, }, // Hawaii_Aleutian
    {  468,  910,  582287,  582300,  582327, 13, 27, 21, }, // Hong_Kong
    {  468,  920,  582348,  582358,  582382, 10, 24, 18, }, // Hovd
    {  468,  925,       0,  582400,       0,  0, 25,  0, }, // India
    {  468,  931,       0,  582425,       0,  0, 19,  0, }, // Indian_Ocean
    {  468,  944,       0,  582444,       0,  0, 16,  0, }, // Indochina
    {  468,  954,       0,  582460,       0,  0, 23,  0, }, // Indonesia_Central
    {  468,  972,       0,  582483,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  468,  990,       0,  582507,       0,  0, 24,  0, }, // Indonesia_Western
    {  468, 1008,  582531,  582540,  582563,  9, 23, 17, }, // Iran
    {  468, 1019,  582580,  582591,  582616, 11, 25, 19, }, // Irkutsk
    {  468, 1027,  582635,  582646,  582671, 11, 25, 19, }, // Israel
    {  468, 1034,  582690,  582702,  582728, 12, 26, 20, }, // Japan
    {  468, 1040,  582748,  582777,  582818, 29, 41, 35, }, // Kamchatka
    {  468, 1058,       0,  582853,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  468, 1077,       0,  582877,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  468, 1106,  582901,  582911,  582935, 10, 24, 18, }, // Korea
    {  468, 1112,       0,  582953,       0,  0, 11,  0, }, // Kosrae
    {  468, 1119,  582964,  582980,  583010, 16, 30, 24, }, // Krasnoyarsk
    {  468, 1151,       0,  583034,       0,  0, 14,  0, }, // Kyrgystan
    {  468, 1175,       0,  583048,       0,  0, 17,  0, }, // Line_Islands
    {  468, 1188,  583065,  583079,  583107, 14, 28, 22, }, // Lord_Howe
    {  468, 1204,  583129,  583141,  583167, 12, 26, 20, }, // Magadan
    {  468, 1219,       0,  583187,       0,  0, 12,  0, }, // Malaysia
    {  468, 1228,       0,  583199,       0,  0, 13,  0, }, // Maldives
    {  468, 1237,       0,  583212,       0,  0, 12,  0, }, // Marquesas
    {  468, 1247,       0,  583224,       0,  0, 20,  0, }, // Marshall_Islands
    {  468, 1264,  583244,  583258,  583286, 14, 28, 22, }, // Mauritius
    {  468, 1274,       0,  583308,       0,  0, 11,  0, }, // Mawson
    {  468, 1281,  583319,  583343,  583379, 24, 36, 30, }, // Mexico_Pacific
    {  468, 1296,  583409,  583424,  583453, 15, 29, 23, }, // Mongolia
    {  468, 1305,  583476,  583487,  583512, 11, 25, 19, }, // Moscow
    {  468, 1312,       0,  583531,       0,  0, 11,  0, }, // Myanmar
    {  468, 1320,       0,  583542,       0,  0, 10,  0, }, // Nauru
    {  468, 1326,       0,  583552,       0,  0, 10,  0, }, // Nepal
    {  468, 1332,  583562,  583581,  583614, 19, 33, 27, }, // New_Caledonia
    {  468, 1346,  583641,  583659,  583691, 18, 32, 26, }, // New_Zealand
    {  468, 1358,  583717,  583737,  583771, 20, 34, 28, }, // Newfoundland
    {  468, 1371,       0,  583799,       0,  0,  9,  0, }, // Niue
    {  468, 1376,  583808,  583820,  583846, 12, 26, 20, }, // Norfolk
    {  468, 1384,  583866,  583890,  583928, 24, 38, 32, }, // Noronha
    {  468, 1406,  583960,  583976,  584006, 16, 30, 24, }, // Novosibirsk
    {  468, 1418,  584030,  584039,  584062,  9, 23, 17, }, // Omsk
    {  468, 1428,  584079,  584092,  584119, 13, 27, 21, }, // Pakistan
    {  468, 1437,       0,  584140,       0,  0, 10,  0, }, // Palau
    {  468, 1443,       0,  584150,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  468, 1460,  584172,  584185,  584212, 13, 27, 21, }, // Paraguay
    {  468, 1469,  584233,  584242,  584265,  9, 23, 17, }, // Peru
    {  468, 1474,  584282,  584295,  584322, 13, 27, 21, }, // Philippines
    {  468, 1486,       0,  584343,       0,  0, 11,  0, }, // Phoenix_Islands
    {  468, 1502,  584354,  584382,  584424, 28, 42, 36, }, // Pierre_Miquelon
    {  468, 1518,       0,  584460,       0,  0, 13,  0, }, // Pitcairn
    {  468, 1527,       0,  584473,       0,  0, 12,  0, }, // Ponape
    {  468, 1534,       0,  584485,       0,  0, 14,  0, }, // Pyongyang
    {  468, 1544,  584499,  584515,  584543, 16, 28, 22, }, // Qyzylorda
    {  468, 1554,       0,  584565,       0,  0, 12,  0, }, // Reunion
    {  468, 1562,       0,  584577,       0,  0, 12,  0, }, // Rothera
    {  468, 1570,  584589,  584602,  584629, 13, 27, 21, }, // Sakhalin
    {  468, 1579,  584650,  584661,  584684, 11, 23, 17, }, // Samara
    {  468, 1596,  584701,  584711,  584735, 10, 24, 18, }, // Samoa
    {  468, 1602,       0,  584753,       0,  0, 12,  0, }, // Seychelles
    {  468, 1624,       0,  584765,       0,  0, 13,  0, }, // Singapore
    {  468, 1634,       0,  584778,       0,  0, 19,  0, }, // Solomon
    {  468, 1642,       0,  584797,       0,  0, 23,  0, }, // South_Georgia
    {  468, 1656,       0,  584820,       0,  0, 12,  0, }, // Suriname
    {  468, 1676,       0,  584832,       0,  0, 10,  0, }, // Syowa
    {  468, 1682,       0,  584842,       0,  0, 11,  0, }, // Tahiti
    {  468, 1689,  584853,  584864,  584889, 11, 25, 19, }, // Taipei
    {  468, 1696,       0,  584908,       0,  0, 16,  0, }, // Tajikistan
    {  468, 1724,       0,  584924,       0,  0, 12,  0, }, // Tokelau
    {  468, 1732,  584936,  584946,  584970, 10, 24, 18, }, // Tonga
    {  468, 1738,       0,  584988,       0,  0, 10,  0, }, // Truk
    {  468, 1750,  584998,  585015,  585046, 17, 31, 25, }, // Turkmenistan
    {  468, 1763,       0,  585071,       0,  0, 11,  0, }, // Tuvalu
    {  468, 1777,  585082,  585094,  585120, 12, 26, 20, }, // Uruguay
    {  468, 1792,  585140,  585155,  585184, 15, 29, 23, }, // Uzbekistan
    {  468, 1803,  585207,  585219,  585245, 12, 26, 20, }, // Vanuatu
    {  468, 1811,       0,  585265,       0,  0, 14,  0, }, // Venezuela
    {  468, 1821,  585279,  585295,  585325, 16, 30, 24, }, // Vladivostok
    {  468, 1833,  585349,  585363,  585391, 14, 28, 22, }, // Volgograd
    {  468, 1843,       0,  585413,       0,  0, 11,  0, }, // Vostok
    {  468, 1850,       0,  585424,       0,  0,  9,  0, }, // Wake
    {  468, 1855,       0,  585433,       0,  0, 20,  0, }, // Wallis
    {  468, 1862,  585453,  585464,  585489, 11, 25, 19, }, // Yakutsk
    {  468, 1870,  585508,  585526,  585558, 18, 32, 26, }, // Yekaterinburg
    {  468, 1892,       0,  585584,       0,  0, 10,  0, }, // Yukon
    {  469,    0,  585594,  585609,  585631, 15, 22, 24, }, // Acre Portuguese/Latin/Brazil
    {  469,    5,       0,  585655,       0,  0, 22,  0, }, // Afghanistan
    {  469,   17,       0,  585677,       0,  0, 25,  0, }, // Africa_Central
    {  469,   32,       0,  585702,       0,  0, 26,  0, }, // Africa_Eastern
    {  469,   65,       0,  585728,       0,  0, 24,  0, }, // Africa_Southern
    {  469,   81,  585752,  585779,  585813, 27, 34, 36, }, // Africa_Western
    {  469,  107,  585849,  585866,  585890, 17, 24, 26, }, // Alaska
    {  469,  128,  585916,  585933,  585957, 17, 24, 26, }, // Almaty
    {  469,  135,  585983,  586002,  586028, 19, 26, 28, }, // Amazon
    {  469,  142,  586056,  586071,  586093, 15, 22, 24, }, // America_Central
    {  469,  158,  586117,  586133,  586156, 16, 23, 25, }, // America_Eastern
    {  469,  174,  586181,  586202,  586230, 21, 28, 30, }, // America_Mountain
    {  469,  191,  586260,  586279,  586305, 19, 26, 28, }, // America_Pacific
    {  469,  207,  586333,  586350,  586374, 17, 24, 26, }, // Anadyr
    {  469,  214,  586400,  586415,  586437, 15, 22, 24, }, // Apia
    {  469,  219,  586461,  586477,  586500, 16, 23, 25, }, // Aqtau
    {  469,  225,  586525,  586542,  586566, 17, 24, 26, }, // Aqtobe
    {  469,  232,  586592,  586609,  586633, 17, 24, 26, }, // Arabian
    {  469,  240,  586659,  586679,  586706, 20, 27, 29, }, // Argentina
    {  469,  250,  586735,  586765,  586802, 30, 37, 39, }, // Argentina_Western
    {  469,  268,  586841,  586859,  586884, 18, 25, 27, }, // Armenia
    {  469,  286,  586911,  586931,  586958, 20, 27, 29, }, // Atlantic
    {  469,  295,  586987,  587015,  587050, 28, 35, 37, }, // Australia_Central
    {  469,  313,  587087,  587124,  587168, 37, 44, 46, }, // Australia_CentralWestern
    {  469,  338,  587214,  587243,  587279, 29, 36, 38, }, // Australia_Eastern
    {  469,  356,  587317,  587347,  587384, 30, 37, 39, }, // Australia_Western
    {  469,  374,  587423,  587445,  587474, 22, 29, 31, }, // Azerbaijan
    {  469,  385,  587505,  587523,  587548, 18, 25, 27, }, // Azores
    {  469,  397,  587575,  587596,  587624, 21, 28, 30, }, // Bangladesh
    {  469,  415,       0,  587654,       0,  0, 16,  0, }, // Bhutan
    {  469,  422,       0,  587670,       0,  0, 18,  0, }, // Bolivia
    {  469,  437,  587688,  587707,  587733, 19, 26, 28, }, // Brasilia
    {  469,  454,       0,  587761,       0,  0, 28,  0, }, // Brunei
    {  469,  461,  587789,  587810,  587838, 21, 28, 30, }, // Cape_Verde
    {  469,  478,       0,  587868,       0,  0, 19,  0, }, // Chamorro
    {  469,  487,  587887,  587905,  587930, 18, 25, 27, }, // Chatham
    {  469,  495,  587957,  587973,  587996, 16, 23, 25, }, // Chile
    {  469,  501,  588021,  588037,  588060, 16, 23, 25, }, // China
    {  469,  507,  588085,  588106,  588134, 21, 28, 30, }, // Choibalsan
    {  469,  518,       0,  588164,       0,  0, 25,  0, }, // Christmas
    {  469,  528,       0,  588189,       0,  0, 22,  0, }, // Cocos
    {  469,  534,  588211,  588230,  588256, 19, 26, 28, }, // Colombia
    {  469,  543,  588284,  588306,  588335, 22, 29, 36, }, // Cook
    {  469,  548,  588371,  588386,  588408, 15, 22, 24, }, // Cuba
    {  469,  559,       0,  588432,       0,  0, 16,  0, }, // Davis
    {  469,  575,       0,  588448,       0,  0, 27,  0, }, // DumontDUrville
    {  469,  612,       0,  588475,       0,  0, 22,  0, }, // East_Timor
    {  469,  623,  588497,  588522,  588554, 25, 32, 34, }, // Easter
    {  469,  630,       0,  588588,       0,  0, 18,  0, }, // Ecuador
    {  469,  638,  588606,  588631,  588663, 25, 32, 34, }, // Europe_Central
    {  469,  653,  588697,  588723,  588756, 26, 33, 35, }, // Europe_Eastern
    {  469,  668,       0,  588791,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  469,  691,  588823,  588850,  588884, 27, 34, 36, }, // Europe_Western
    {  469,  706,  588920,  588946,  588979, 26, 33, 35, }, // Falkland
    {  469,  715,  589014,  589029,  589051, 15, 22, 24, }, // Fiji
    {  469,  720,       0,  589075,       0,  0, 26,  0, }, // French_Guiana
    {  469,  734,       0,  589101,       0,  0, 52,  0, }, // French_Southern
    {  469,  757,       0,  589153,       0,  0, 20,  0, }, // Galapagos
    {  469,  767,       0,  589173,       0,  0, 18,  0, }, // Gambier
    {  469,  775,  589191,  589209,  589234, 18, 25, 27, }, // Georgia
    {  469,  783,       0,  589261,       0,  0, 26,  0, }, // Gilbert_Islands
    {  469,  799,       0,  589287,       0,  0, 33,  0, }, // GMT
    {  469,  841,  589320,  589350,  589387, 30, 37, 39, }, // Greenland_Eastern
    {  469,  859,  589426,  589458,  589497, 32, 39, 41, }, // Greenland_Western
    {  469,  877,       0,  589538,       0,  0, 22,  0, }, // Guam
    {  469,  882,       0,  589560,       0,  0, 16,  0, }, // Gulf
    {  469,  887,       0,  589075,       0,  0, 17,  0, }, // Guyana
    {  469,  894,  589576,  589608,  589647, 32, 39, 41, }, // Hawaii_Aleutian
    {  469,  910,  589688,  589708,  589735, 20, 27, 29, }, // Hong_Kong
    {  469,  920,  589764,  589779,  589801, 15, 22, 24, }, // Hovd
    {  469,  925,       0,  589825,       0,  0, 23,  0, }, // India
    {  469,  931,       0,  589848,       0,  0, 24,  0, }, // Indian_Ocean
    {  469,  944,       0,  589872,       0,  0, 20,  0, }, // Indochina
    {  469,  954,       0,  589892,       0,  0, 28,  0, }, // Indonesia_Central
    {  469,  972,       0,  589920,       0,  0, 29,  0, }, // Indonesia_Eastern
    {  469,  990,       0,  589949,       0,  0, 30,  0, }, // Indonesia_Western
    {  469, 1008,  589979,  589993,  590014, 14, 21, 23, }, // Iran
    {  469, 1019,  590037,  590055,  590080, 18, 25, 27, }, // Irkutsk
    {  469, 1027,  590107,  590124,  590148, 17, 24, 26, }, // Israel
    {  469, 1034,  590174,  590190,  590213, 16, 23, 25, }, // Japan
    {  469, 1040,  590238,  590273,  590315, 35, 42, 44, }, // Kamchatka
    {  469, 1058,       0,  590359,       0,  0, 31,  0, }, // Kazakhstan_Eastern
    {  469, 1077,       0,  590390,       0,  0, 32,  0, }, // Kazakhstan_Western
    {  469, 1106,  590422,  590439,  590463, 17, 24, 26, }, // Korea
    {  469, 1112,       0,  590489,       0,  0, 17,  0, }, // Kosrae
    {  469, 1119,  590506,  590528,  590557, 22, 29, 31, }, // Krasnoyarsk
    {  469, 1151,       0,  590588,       0,  0, 22,  0, }, // Kyrgystan
    {  469, 1161,       0,  590610,       0,  0, 16,  0, }, // Lanka
    {  469, 1175,       0,  590626,       0,  0, 26,  0, }, // Line_Islands
    {  469, 1188,  590652,  590672,  590699, 20, 27, 29, }, // Lord_Howe
    {  469, 1198,  590728,  590744,  590767, 16, 23, 25, }, // Macau
    {  469, 1204,  590792,  590810,  590835, 18, 25, 27, }, // Magadan
    {  469, 1219,       0,  590862,       0,  0, 18,  0, }, // Malaysia
    {  469, 1228,       0,  590880,       0,  0, 26,  0, }, // Maldives
    {  469, 1237,       0,  590906,       0,  0, 21,  0, }, // Marquesas
    {  469, 1247,       0,  590927,       0,  0, 26,  0, }, // Marshall_Islands
    {  469, 1264,  590953,  590972,  590998, 19, 26, 28, }, // Mauritius
    {  469, 1274,       0,  591026,       0,  0, 17,  0, }, // Mawson
    {  469, 1281,  591043,  591071,  591106, 28, 35, 37, }, // Mexico_Pacific
    {  469, 1296,  591143,  591164,  591192, 21, 28, 30, }, // Mongolia
    {  469, 1305,  591222,  591239,  591263, 17, 24, 26, }, // Moscow
    {  469, 1312,       0,  591289,       0,  0, 18,  0, }, // Myanmar
    {  469, 1320,       0,  591307,       0,  0, 16,  0, }, // Nauru
    {  469, 1326,       0,  591323,       0,  0, 16,  0, }, // Nepal
    {  469, 1332,  591339,  591364,  591396, 25, 32, 34, }, // New_Caledonia
    {  469, 1346,  591430,  591454,  591485, 24, 31, 33, }, // New_Zealand
    {  469, 1358,  591518,  591539,  591567, 21, 28, 30, }, // Newfoundland
    {  469, 1371,       0,  591597,       0,  0, 15,  0, }, // Niue
    {  469, 1376,  591612,  591635,  591665, 23, 30, 32, }, // Norfolk
    {  469, 1384,  591697,  591727,  591764, 30, 37, 39, }, // Noronha
    {  469, 1392,       0,  591803,       0,  0, 34,  0, }, // North_Mariana
    {  469, 1406,  591837,  591859,  591888, 22, 29, 31, }, // Novosibirsk
    {  469, 1418,  591919,  591934,  591956, 15, 22, 24, }, // Omsk
    {  469, 1428,  591980,  592000,  592027, 20, 27, 29, }, // Pakistan
    {  469, 1437,       0,  592056,       0,  0, 16,  0, }, // Palau
    {  469, 1443,       0,  592072,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  469, 1460,  592099,  592118,  592144, 19, 26, 28, }, // Paraguay
    {  469, 1469,  592172,  592187,  592209, 15, 22, 24, }, // Peru
    {  469, 1474,  592233,  592254,  592282, 21, 28, 30, }, // Philippines
    {  469, 1486,       0,  592312,       0,  0, 23,  0, }, // Phoenix_Islands
    {  469, 1502,  592335,  592366,  592404, 31, 38, 37, }, // Pierre_Miquelon
    {  469, 1518,       0,  592441,       0,  0, 19,  0, }, // Pitcairn
    {  469, 1527,       0,  592460,       0,  0, 17,  0, }, // Ponape
    {  469, 1534,       0,  592477,       0,  0, 20,  0, }, // Pyongyang
    {  469, 1544,  592497,  592517,  592544, 20, 27, 29, }, // Qyzylorda
    {  469, 1554,       0,  592573,       0,  0, 18,  0, }, // Reunion
    {  469, 1562,       0,  592591,       0,  0, 18,  0, }, // Rothera
    {  469, 1570,  592609,  592628,  592654, 19, 26, 28, }, // Sakhalin
    {  469, 1579,  592682,  592699,  592723, 17, 24, 26, }, // Samara
    {  469, 1596,  592749,  592765,  592788, 16, 23, 25, }, // Samoa
    {  469, 1602,       0,  592813,       0,  0, 20,  0, }, // Seychelles
    {  469, 1624,       0,  592833,       0,  0, 27,  0, }, // Singapore
    {  469, 1634,       0,  592860,       0,  0, 25,  0, }, // Solomon
    {  469, 1642,       0,  592885,       0,  0, 25,  0, }, // South_Georgia
    {  469, 1656,       0,  592910,       0,  0, 19,  0, }, // Suriname
    {  469, 1676,       0,  592929,       0,  0, 16,  0, }, // Syowa
    {  469, 1682,       0,  592945,       0,  0, 16,  0, }, // Tahiti
    {  469, 1689,  592961,  592978,  593002, 17, 24, 26, }, // Taipei
    {  469, 1696,       0,  593028,       0,  0, 22,  0, }, // Tajikistan
    {  469, 1724,       0,  593050,       0,  0, 18,  0, }, // Tokelau
    {  469, 1732,  593068,  593084,  593107, 16, 23, 25, }, // Tonga
    {  469, 1738,       0,  593132,       0,  0, 16,  0, }, // Truk
    {  469, 1750,  593148,  593172,  593203, 24, 31, 33, }, // Turkmenistan
    {  469, 1763,       0,  593236,       0,  0, 17,  0, }, // Tuvalu
    {  469, 1777,  593253,  593271,  593296, 18, 25, 27, }, // Uruguay
    {  469, 1792,  593323,  593345,  593374, 22, 29, 31, }, // Uzbekistan
    {  469, 1803,  593405,  593423,  593448, 18, 25, 27, }, // Vanuatu
    {  469, 1811,       0,  593475,       0,  0, 20,  0, }, // Venezuela
    {  469, 1821,  593495,  593517,  593546, 22, 29, 31, }, // Vladivostok
    {  469, 1833,  593577,  593598,  593626, 21, 28, 30, }, // Volgograd
    {  469, 1843,       0,  593656,       0,  0, 17,  0, }, // Vostok
    {  469, 1850,       0,  593673,       0,  0, 22,  0, }, // Wake
    {  469, 1855,       0,  593695,       0,  0, 26,  0, }, // Wallis
    {  469, 1862,  593721,  593739,  593764, 18, 25, 27, }, // Yakutsk
    {  469, 1870,  593791,  593815,  593846, 24, 31, 33, }, // Yekaterinburg
    {  469, 1892,       0,  593879,       0,  0, 16,  0, }, // Yukon
    {  470,    0,  593895,  593907,  593926, 12, 19, 21, }, // Acre Portuguese/Latin/Angola
    {  470,    5,       0,  593947,       0,  0, 19,  0, }, // Afghanistan
    {  470,   17,       0,  593966,       0,  0, 22,  0, }, // Africa_Central
    {  470,   32,       0,  593988,       0,  0, 23,  0, }, // Africa_Eastern
    {  470,   65,       0,  594011,       0,  0, 21,  0, }, // Africa_Southern
    {  470,   81,  594032,  594056,  594087, 24, 31, 33, }, // Africa_Western
    {  470,  107,  594120,  594134,  594155, 14, 21, 23, }, // Alaska
    {  470,  128,  594178,  594192,  594213, 14, 21, 23, }, // Almaty
    {  470,  135,  594236,  594252,  594275, 16, 23, 25, }, // Amazon
    {  470,  142,  594300,  594328,  594363, 28, 35, 37, }, // America_Central
    {  470,  158,  594400,  594429,  594465, 29, 36, 38, }, // America_Eastern
    {  470,  174,  594503,  594535,  594574, 32, 39, 41, }, // America_Mountain
    {  470,  191,  594615,  594647,  594686, 32, 39, 41, }, // America_Pacific
    {  470,  207,  594727,  594741,  594762, 14, 21, 23, }, // Anadyr
    {  470,  214,  594785,  594797,  594816, 12, 19, 21, }, // Apia
    {  470,  219,  594837,  594850,  594870, 13, 20, 22, }, // Aqtau
    {  470,  225,  594892,  594906,  594927, 14, 21, 23, }, // Aqtobe
    {  470,  232,  594950,  594964,  594985, 14, 21, 23, }, // Arabian
    {  470,  240,  595008,  595025,  595049, 17, 24, 26, }, // Argentina
    {  470,  250,  595075,  595102,  595136, 27, 34, 36, }, // Argentina_Western
    {  470,  268,  595172,  595187,  595209, 15, 22, 24, }, // Armenia
    {  470,  286,  595233,  595250,  595274, 17, 24, 26, }, // Atlantic
    {  470,  295,  595300,  595325,  595357, 25, 32, 34, }, // Australia_Central
    {  470,  313,  595391,  595426,  595468, 35, 42, 44, }, // Australia_CentralWestern
    {  470,  338,  595512,  595538,  595571, 26, 33, 35, }, // Australia_Eastern
    {  470,  356,  595606,  595633,  595667, 27, 34, 36, }, // Australia_Western
    {  470,  374,  595703,  595721,  595746, 18, 25, 27, }, // Azerbaijan
    {  470,  385,  595773,  595788,  595810, 15, 22, 24, }, // Azores
    {  470,  397,  595834,  595853,  595879, 19, 26, 28, }, // Bangladesh
    {  470,  415,       0,  595907,       0,  0, 13,  0, }, // Bhutan
    {  470,  422,       0,  595920,       0,  0, 15,  0, }, // Bolivia
    {  470,  437,  135014,  595935,  595958, 16, 23, 25, }, // Brasilia
    {  470,  454,       0,  595983,       0,  0, 25,  0, }, // Brunei
    {  470,  461,  596008,  596026,  596051, 18, 25, 27, }, // Cape_Verde
    {  470,  478,       0,  596078,       0,  0, 23,  0, }, // Chamorro
    {  470,  487,   54300,  596101,  596123, 15, 22, 24, }, // Chatham
    {  470,  495,  596147,  596160,  596180, 13, 20, 22, }, // Chile
    {  470,  501,  596202,  596215,  596235, 13, 20, 22, }, // China
    {  470,  507,   54477,  596257,  596282, 18, 25, 27, }, // Choibalsan
    {  470,  518,       0,  596309,       0,  0, 21,  0, }, // Christmas
    {  470,  528,       0,  596330,       0,  0, 20,  0, }, // Cocos
    {  470,  534,  596350,  596366,  596389, 16, 23, 25, }, // Colombia
    {  470,  543,  596414,  596433,  596459, 19, 26, 28, }, // Cook
    {  470,  548,   54767,  596487,  596506, 12, 19, 21, }, // Cuba
    {  470,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {  470,  575,       0,   54834,       0,  0, 24,  0, }, // DumontDUrville
    {  470,  612,       0,  596527,       0,  0, 19,  0, }, // East_Timor
    {  470,  623,  596546,  596568,  596597, 22, 29, 31, }, // Easter
    {  470,  630,       0,  596628,       0,  0, 15,  0, }, // Ecuador
    {  470,  638,  596643,  596665,  596694, 22, 29, 31, }, // Europe_Central
    {  470,  653,  596725,  596748,  596778, 23, 30, 32, }, // Europe_Eastern
    {  470,  668,       0,  596810,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  470,  691,  596841,  596865,  596896, 24, 31, 33, }, // Europe_Western
    {  470,  706,  596929,  596952,  596982, 23, 30, 32, }, // Falkland
    {  470,  715,  136173,  597014,  597033, 12, 19, 21, }, // Fiji
    {  470,  720,       0,  597054,       0,  0, 23,  0, }, // French_Guiana
    {  470,  734,       0,  597077,       0,  0, 48,  0, }, // French_Southern
    {  470,  757,       0,  597125,       0,  0, 18,  0, }, // Galapagos
    {  470,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {  470,  775,  597143,  597158,  597180, 15, 22, 24, }, // Georgia
    {  470,  783,       0,  597204,       0,  0, 22,  0, }, // Gilbert_Islands
    {  470,  799,       0,  597226,       0,  0, 17,  0, }, // GMT
    {  470,  841,  597243,  597271,  597306, 28, 35, 37, }, // Greenland_Eastern
    {  470,  859,  597343,  597372,  597408, 29, 36, 38, }, // Greenland_Western
    {  470,  877,       0,  597446,       0,  0, 19,  0, }, // Guam
    {  470,  882,       0,  597465,       0,  0, 20,  0, }, // Gulf
    {  470,  887,       0,  597054,       0,  0, 14,  0, }, // Guyana
    {  470,  894,  597485,  597508,  597538, 23, 30, 32, }, // Hawaii_Aleutian
    {  470,  910,  136716,  597570,  597594, 17, 24, 26, }, // Hong_Kong
    {  470,  920,   56045,  597620,  597639, 12, 19, 21, }, // Hovd
    {  470,  925,       0,  597660,       0,  0, 20,  0, }, // India
    {  470,  931,       0,  597680,       0,  0, 21,  0, }, // Indian_Ocean
    {  470,  944,       0,  597701,       0,  0, 17,  0, }, // Indochina
    {  470,  954,       0,  597718,       0,  0, 25,  0, }, // Indonesia_Central
    {  470,  972,       0,  597743,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  470,  990,       0,  597769,       0,  0, 27,  0, }, // Indonesia_Western
    {  470, 1008,  597796,  597808,  597827, 12, 19, 21, }, // Iran
    {  470, 1019,  597848,  597863,  597885, 15, 22, 24, }, // Irkutsk
    {  470, 1027,  597909,  597923,  597944, 14, 21, 23, }, // Israel
    {  470, 1034,  597967,  597980,  598000, 13, 20, 22, }, // Japan
    {  470, 1040,  598022,  598054,  598093, 32, 39, 41, }, // Kamchatka
    {  470, 1058,       0,  598134,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  470, 1077,       0,  598162,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  470, 1106,  598191,  598205,  598226, 14, 21, 23, }, // Korea
    {  470, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {  470, 1119,   56703,  598249,  598275, 19, 26, 28, }, // Krasnoyarsk
    {  470, 1151,       0,  598303,       0,  0, 19,  0, }, // Kyrgystan
    {  470, 1161,       0,  598322,       0,  0, 17,  0, }, // Lanka
    {  470, 1175,       0,  598339,       0,  0, 19,  0, }, // Line_Islands
    {  470, 1188,   56833,  598358,  598382, 17, 24, 26, }, // Lord_Howe
    {  470, 1198,  137554,  598408,  598428, 13, 20, 22, }, // Macau
    {  470, 1204,  137611,  598450,  598472, 15, 22, 24, }, // Magadan
    {  470, 1219,       0,  598496,       0,  0, 15,  0, }, // Malaysia
    {  470, 1228,       0,  598511,       0,  0, 17,  0, }, // Maldives
    {  470, 1237,       0,  598528,       0,  0, 24,  0, }, // Marquesas
    {  470, 1247,       0,  598552,       0,  0, 23,  0, }, // Marshall_Islands
    {  470, 1264,  598575,  598591,  598614, 16, 23, 25, }, // Mauritius
    {  470, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {  470, 1281,  598639,  598664,  598696, 25, 32, 34, }, // Mexico_Pacific
    {  470, 1296,  598730,  598748,  598773, 18, 25, 27, }, // Mongolia
    {  470, 1305,  598800,  598815,  598837, 15, 22, 24, }, // Moscow
    {  470, 1312,       0,  598861,       0,  0, 15,  0, }, // Myanmar
    {  470, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {  470, 1326,       0,  598876,       0,  0, 13,  0, }, // Nepal
    {  470, 1332,  598889,  598911,  598940, 22, 29, 31, }, // New_Caledonia
    {  470, 1346,  598971,  598992,  599020, 21, 28, 30, }, // New_Zealand
    {  470, 1358,  599050,  599068,  599093, 18, 25, 27, }, // Newfoundland
    {  470, 1371,       0,  599120,       0,  0, 12,  0, }, // Niue
    {  470, 1376,  599132,  599152,  599179, 20, 27, 29, }, // Norfolk
    {  470, 1384,   57800,  599208,  599242, 27, 34, 36, }, // Noronha
    {  470, 1392,       0,  599278,       0,  0, 31,  0, }, // North_Mariana
    {  470, 1406,   57936,  599309,  599335, 19, 26, 28, }, // Novosibirsk
    {  470, 1418,  599363,  599375,  599394, 12, 19, 21, }, // Omsk
    {  470, 1428,  599415,  599432,  599456, 17, 24, 26, }, // Pakistan
    {  470, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {  470, 1443,       0,  599482,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  470, 1460,  599506,  599522,  599545, 16, 23, 25, }, // Paraguay
    {  470, 1469,  599570,  599582,  599601, 12, 19, 21, }, // Peru
    {  470, 1474,  599622,  599640,  599665, 18, 25, 27, }, // Philippines
    {  470, 1486,       0,  599692,       0,  0, 20,  0, }, // Phoenix_Islands
    {  470, 1502,  599712,  599740,  599775, 28, 35, 37, }, // Pierre_Miquelon
    {  470, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {  470, 1527,       0,   58521,       0,  0, 14,  0, }, // Ponape
    {  470, 1534,       0,  138941,       0,  0, 17,  0, }, // Pyongyang
    {  470, 1544,   58552,  599812,  599836, 17, 24, 26, }, // Qyzylorda
    {  470, 1554,       0,  599862,       0,  0, 15,  0, }, // Reunion
    {  470, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {  470, 1570,  599877,  599893,  599916, 16, 23, 25, }, // Sakhalin
    {  470, 1579,   58714,  599941,  599962, 14, 21, 23, }, // Samara
    {  470, 1596,   58774,  599985,  600005, 13, 20, 22, }, // Samoa
    {  470, 1602,       0,  600027,       0,  0, 18,  0, }, // Seychelles
    {  470, 1624,       0,  600045,       0,  0, 24,  0, }, // Singapore
    {  470, 1634,       0,  600069,       0,  0, 22,  0, }, // Solomon
    {  470, 1642,       0,  600091,       0,  0, 22,  0, }, // South_Georgia
    {  470, 1656,       0,  600113,       0,  0, 16,  0, }, // Suriname
    {  470, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {  470, 1682,       0,  600129,       0,  0, 13,  0, }, // Tahiti
    {  470, 1689,   58967,  600142,  600162, 13, 20, 22, }, // Taipei
    {  470, 1696,       0,  600184,       0,  0, 19,  0, }, // Tajikistan
    {  470, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {  470, 1732,   59062,  600203,  600223, 13, 20, 22, }, // Tonga
    {  470, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {  470, 1750,  600245,  600267,  600296, 22, 29, 31, }, // Turkmenistan
    {  470, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {  470, 1777,  600327,  600342,  600364, 15, 22, 24, }, // Uruguay
    {  470, 1792,  600388,  600407,  600433, 19, 26, 28, }, // Uzbekistan
    {  470, 1803,  600461,  600476,  600498, 15, 22, 24, }, // Vanuatu
    {  470, 1811,       0,  600522,       0,  0, 17,  0, }, // Venezuela
    {  470, 1821,   59451,  600539,  600565, 19, 26, 28, }, // Vladivostok
    {  470, 1833,  600593,  600611,  600636, 18, 25, 27, }, // Volgograd
    {  470, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {  470, 1850,       0,  600663,       0,  0, 17,  0, }, // Wake
    {  470, 1855,       0,  600680,       0,  0, 23,  0, }, // Wallis
    {  470, 1862,   59653,  600703,  600725, 15, 22, 24, }, // Yakutsk
    {  470, 1870,  600749,  600770,  600798, 21, 28, 30, }, // Yekaterinburg
    {  470, 1892,       0,  600828,       0,  0, 13,  0, }, // Yukon
    {  471,    0,  593895,  593907,  593926, 12, 19, 21, }, // Acre Portuguese/Latin/Cape Verde
    {  471,    5,       0,  593947,       0,  0, 19,  0, }, // Afghanistan
    {  471,   17,       0,  593966,       0,  0, 22,  0, }, // Africa_Central
    {  471,   32,       0,  593988,       0,  0, 23,  0, }, // Africa_Eastern
    {  471,   65,       0,  594011,       0,  0, 21,  0, }, // Africa_Southern
    {  471,   81,  594032,  594056,  594087, 24, 31, 33, }, // Africa_Western
    {  471,  107,  594120,  594134,  594155, 14, 21, 23, }, // Alaska
    {  471,  128,  594178,  594192,  594213, 14, 21, 23, }, // Almaty
    {  471,  135,  594236,  594252,  594275, 16, 23, 25, }, // Amazon
    {  471,  142,  594300,  594328,  594363, 28, 35, 37, }, // America_Central
    {  471,  158,  594400,  594429,  594465, 29, 36, 38, }, // America_Eastern
    {  471,  174,  594503,  594535,  594574, 32, 39, 41, }, // America_Mountain
    {  471,  191,  594615,  594647,  594686, 32, 39, 41, }, // America_Pacific
    {  471,  207,  594727,  594741,  594762, 14, 21, 23, }, // Anadyr
    {  471,  214,  594785,  594797,  594816, 12, 19, 21, }, // Apia
    {  471,  219,  594837,  594850,  594870, 13, 20, 22, }, // Aqtau
    {  471,  225,  594892,  594906,  594927, 14, 21, 23, }, // Aqtobe
    {  471,  232,  594950,  594964,  594985, 14, 21, 23, }, // Arabian
    {  471,  240,  595008,  595025,  595049, 17, 24, 26, }, // Argentina
    {  471,  250,  595075,  595102,  595136, 27, 34, 36, }, // Argentina_Western
    {  471,  268,  595172,  595187,  595209, 15, 22, 24, }, // Armenia
    {  471,  286,  595233,  595250,  595274, 17, 24, 26, }, // Atlantic
    {  471,  295,  595300,  595325,  595357, 25, 32, 34, }, // Australia_Central
    {  471,  313,  595391,  595426,  595468, 35, 42, 44, }, // Australia_CentralWestern
    {  471,  338,  595512,  595538,  595571, 26, 33, 35, }, // Australia_Eastern
    {  471,  356,  595606,  595633,  595667, 27, 34, 36, }, // Australia_Western
    {  471,  374,  595703,  595721,  595746, 18, 25, 27, }, // Azerbaijan
    {  471,  385,  595773,  595788,  595810, 15, 22, 24, }, // Azores
    {  471,  397,  595834,  595853,  595879, 19, 26, 28, }, // Bangladesh
    {  471,  415,       0,  595907,       0,  0, 13,  0, }, // Bhutan
    {  471,  422,       0,  595920,       0,  0, 15,  0, }, // Bolivia
    {  471,  437,  135014,  595935,  595958, 16, 23, 25, }, // Brasilia
    {  471,  454,       0,  595983,       0,  0, 25,  0, }, // Brunei
    {  471,  461,  596008,  596026,  596051, 18, 25, 27, }, // Cape_Verde
    {  471,  478,       0,  596078,       0,  0, 23,  0, }, // Chamorro
    {  471,  487,   54300,  596101,  596123, 15, 22, 24, }, // Chatham
    {  471,  495,  596147,  596160,  596180, 13, 20, 22, }, // Chile
    {  471,  501,  596202,  596215,  596235, 13, 20, 22, }, // China
    {  471,  507,   54477,  596257,  596282, 18, 25, 27, }, // Choibalsan
    {  471,  518,       0,  596309,       0,  0, 21,  0, }, // Christmas
    {  471,  528,       0,  596330,       0,  0, 20,  0, }, // Cocos
    {  471,  534,  596350,  596366,  596389, 16, 23, 25, }, // Colombia
    {  471,  543,  596414,  596433,  596459, 19, 26, 28, }, // Cook
    {  471,  548,   54767,  596487,  596506, 12, 19, 21, }, // Cuba
    {  471,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {  471,  575,       0,   54834,       0,  0, 24,  0, }, // DumontDUrville
    {  471,  612,       0,  596527,       0,  0, 19,  0, }, // East_Timor
    {  471,  623,  596546,  596568,  596597, 22, 29, 31, }, // Easter
    {  471,  630,       0,  596628,       0,  0, 15,  0, }, // Ecuador
    {  471,  638,  596643,  596665,  596694, 22, 29, 31, }, // Europe_Central
    {  471,  653,  596725,  596748,  596778, 23, 30, 32, }, // Europe_Eastern
    {  471,  668,       0,  596810,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  471,  691,  596841,  596865,  596896, 24, 31, 33, }, // Europe_Western
    {  471,  706,  596929,  596952,  596982, 23, 30, 32, }, // Falkland
    {  471,  715,  136173,  597014,  597033, 12, 19, 21, }, // Fiji
    {  471,  720,       0,  597054,       0,  0, 23,  0, }, // French_Guiana
    {  471,  734,       0,  597077,       0,  0, 48,  0, }, // French_Southern
    {  471,  757,       0,  597125,       0,  0, 18,  0, }, // Galapagos
    {  471,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {  471,  775,  597143,  597158,  597180, 15, 22, 24, }, // Georgia
    {  471,  783,       0,  597204,       0,  0, 22,  0, }, // Gilbert_Islands
    {  471,  799,       0,  597226,       0,  0, 17,  0, }, // GMT
    {  471,  841,  597243,  597271,  597306, 28, 35, 37, }, // Greenland_Eastern
    {  471,  859,  597343,  597372,  597408, 29, 36, 38, }, // Greenland_Western
    {  471,  877,       0,  597446,       0,  0, 19,  0, }, // Guam
    {  471,  882,       0,  597465,       0,  0, 20,  0, }, // Gulf
    {  471,  887,       0,  597054,       0,  0, 14,  0, }, // Guyana
    {  471,  894,  597485,  597508,  597538, 23, 30, 32, }, // Hawaii_Aleutian
    {  471,  910,  136716,  597570,  597594, 17, 24, 26, }, // Hong_Kong
    {  471,  920,   56045,  597620,  597639, 12, 19, 21, }, // Hovd
    {  471,  925,       0,  597660,       0,  0, 20,  0, }, // India
    {  471,  931,       0,  597680,       0,  0, 21,  0, }, // Indian_Ocean
    {  471,  944,       0,  597701,       0,  0, 17,  0, }, // Indochina
    {  471,  954,       0,  597718,       0,  0, 25,  0, }, // Indonesia_Central
    {  471,  972,       0,  597743,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  471,  990,       0,  597769,       0,  0, 27,  0, }, // Indonesia_Western
    {  471, 1008,  597796,  597808,  597827, 12, 19, 21, }, // Iran
    {  471, 1019,  597848,  597863,  597885, 15, 22, 24, }, // Irkutsk
    {  471, 1027,  597909,  597923,  597944, 14, 21, 23, }, // Israel
    {  471, 1034,  597967,  597980,  598000, 13, 20, 22, }, // Japan
    {  471, 1040,  598022,  598054,  598093, 32, 39, 41, }, // Kamchatka
    {  471, 1058,       0,  598134,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  471, 1077,       0,  598162,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  471, 1106,  598191,  598205,  598226, 14, 21, 23, }, // Korea
    {  471, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {  471, 1119,   56703,  598249,  598275, 19, 26, 28, }, // Krasnoyarsk
    {  471, 1151,       0,  598303,       0,  0, 19,  0, }, // Kyrgystan
    {  471, 1161,       0,  598322,       0,  0, 17,  0, }, // Lanka
    {  471, 1175,       0,  598339,       0,  0, 19,  0, }, // Line_Islands
    {  471, 1188,   56833,  598358,  598382, 17, 24, 26, }, // Lord_Howe
    {  471, 1198,  137554,  598408,  598428, 13, 20, 22, }, // Macau
    {  471, 1204,  137611,  598450,  598472, 15, 22, 24, }, // Magadan
    {  471, 1219,       0,  598496,       0,  0, 15,  0, }, // Malaysia
    {  471, 1228,       0,  598511,       0,  0, 17,  0, }, // Maldives
    {  471, 1237,       0,  598528,       0,  0, 24,  0, }, // Marquesas
    {  471, 1247,       0,  598552,       0,  0, 23,  0, }, // Marshall_Islands
    {  471, 1264,  598575,  598591,  598614, 16, 23, 25, }, // Mauritius
    {  471, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {  471, 1281,  598639,  598664,  598696, 25, 32, 34, }, // Mexico_Pacific
    {  471, 1296,  598730,  598748,  598773, 18, 25, 27, }, // Mongolia
    {  471, 1305,  598800,  598815,  598837, 15, 22, 24, }, // Moscow
    {  471, 1312,       0,  598861,       0,  0, 15,  0, }, // Myanmar
    {  471, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {  471, 1326,       0,  598876,       0,  0, 13,  0, }, // Nepal
    {  471, 1332,  598889,  598911,  598940, 22, 29, 31, }, // New_Caledonia
    {  471, 1346,  598971,  598992,  599020, 21, 28, 30, }, // New_Zealand
    {  471, 1358,  599050,  599068,  599093, 18, 25, 27, }, // Newfoundland
    {  471, 1371,       0,  599120,       0,  0, 12,  0, }, // Niue
    {  471, 1376,  599132,  599152,  599179, 20, 27, 29, }, // Norfolk
    {  471, 1384,   57800,  599208,  599242, 27, 34, 36, }, // Noronha
    {  471, 1392,       0,  599278,       0,  0, 31,  0, }, // North_Mariana
    {  471, 1406,   57936,  599309,  599335, 19, 26, 28, }, // Novosibirsk
    {  471, 1418,  599363,  599375,  599394, 12, 19, 21, }, // Omsk
    {  471, 1428,  599415,  599432,  599456, 17, 24, 26, }, // Pakistan
    {  471, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {  471, 1443,       0,  599482,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  471, 1460,  599506,  599522,  599545, 16, 23, 25, }, // Paraguay
    {  471, 1469,  599570,  599582,  599601, 12, 19, 21, }, // Peru
    {  471, 1474,  599622,  599640,  599665, 18, 25, 27, }, // Philippines
    {  471, 1486,       0,  599692,       0,  0, 20,  0, }, // Phoenix_Islands
    {  471, 1502,  599712,  599740,  599775, 28, 35, 37, }, // Pierre_Miquelon
    {  471, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {  471, 1527,       0,   58521,       0,  0, 14,  0, }, // Ponape
    {  471, 1534,       0,  138941,       0,  0, 17,  0, }, // Pyongyang
    {  471, 1544,   58552,  599812,  599836, 17, 24, 26, }, // Qyzylorda
    {  471, 1554,       0,  599862,       0,  0, 15,  0, }, // Reunion
    {  471, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {  471, 1570,  599877,  599893,  599916, 16, 23, 25, }, // Sakhalin
    {  471, 1579,   58714,  599941,  599962, 14, 21, 23, }, // Samara
    {  471, 1596,   58774,  599985,  600005, 13, 20, 22, }, // Samoa
    {  471, 1602,       0,  600027,       0,  0, 18,  0, }, // Seychelles
    {  471, 1624,       0,  600045,       0,  0, 24,  0, }, // Singapore
    {  471, 1634,       0,  600069,       0,  0, 22,  0, }, // Solomon
    {  471, 1642,       0,  600091,       0,  0, 22,  0, }, // South_Georgia
    {  471, 1656,       0,  600113,       0,  0, 16,  0, }, // Suriname
    {  471, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {  471, 1682,       0,  600129,       0,  0, 13,  0, }, // Tahiti
    {  471, 1689,   58967,  600142,  600162, 13, 20, 22, }, // Taipei
    {  471, 1696,       0,  600184,       0,  0, 19,  0, }, // Tajikistan
    {  471, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {  471, 1732,   59062,  600203,  600223, 13, 20, 22, }, // Tonga
    {  471, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {  471, 1750,  600245,  600267,  600296, 22, 29, 31, }, // Turkmenistan
    {  471, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {  471, 1777,  600327,  600342,  600364, 15, 22, 24, }, // Uruguay
    {  471, 1792,  600388,  600407,  600433, 19, 26, 28, }, // Uzbekistan
    {  471, 1803,  600461,  600476,  600498, 15, 22, 24, }, // Vanuatu
    {  471, 1811,       0,  600522,       0,  0, 17,  0, }, // Venezuela
    {  471, 1821,   59451,  600539,  600565, 19, 26, 28, }, // Vladivostok
    {  471, 1833,  600593,  600611,  600636, 18, 25, 27, }, // Volgograd
    {  471, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {  471, 1850,       0,  600663,       0,  0, 17,  0, }, // Wake
    {  471, 1855,       0,  600680,       0,  0, 23,  0, }, // Wallis
    {  471, 1862,   59653,  600703,  600725, 15, 22, 24, }, // Yakutsk
    {  471, 1870,  600749,  600770,  600798, 21, 28, 30, }, // Yekaterinburg
    {  471, 1892,       0,  600828,       0,  0, 13,  0, }, // Yukon
    {  472,    0,  593895,  593907,  593926, 12, 19, 21, }, // Acre Portuguese/Latin/Equatorial Guinea
    {  472,    5,       0,  593947,       0,  0, 19,  0, }, // Afghanistan
    {  472,   17,       0,  593966,       0,  0, 22,  0, }, // Africa_Central
    {  472,   32,       0,  593988,       0,  0, 23,  0, }, // Africa_Eastern
    {  472,   65,       0,  594011,       0,  0, 21,  0, }, // Africa_Southern
    {  472,   81,  594032,  594056,  594087, 24, 31, 33, }, // Africa_Western
    {  472,  107,  594120,  594134,  594155, 14, 21, 23, }, // Alaska
    {  472,  128,  594178,  594192,  594213, 14, 21, 23, }, // Almaty
    {  472,  135,  594236,  594252,  594275, 16, 23, 25, }, // Amazon
    {  472,  142,  594300,  594328,  594363, 28, 35, 37, }, // America_Central
    {  472,  158,  594400,  594429,  594465, 29, 36, 38, }, // America_Eastern
    {  472,  174,  594503,  594535,  594574, 32, 39, 41, }, // America_Mountain
    {  472,  191,  594615,  594647,  594686, 32, 39, 41, }, // America_Pacific
    {  472,  207,  594727,  594741,  594762, 14, 21, 23, }, // Anadyr
    {  472,  214,  594785,  594797,  594816, 12, 19, 21, }, // Apia
    {  472,  219,  594837,  594850,  594870, 13, 20, 22, }, // Aqtau
    {  472,  225,  594892,  594906,  594927, 14, 21, 23, }, // Aqtobe
    {  472,  232,  594950,  594964,  594985, 14, 21, 23, }, // Arabian
    {  472,  240,  595008,  595025,  595049, 17, 24, 26, }, // Argentina
    {  472,  250,  595075,  595102,  595136, 27, 34, 36, }, // Argentina_Western
    {  472,  268,  595172,  595187,  595209, 15, 22, 24, }, // Armenia
    {  472,  286,  595233,  595250,  595274, 17, 24, 26, }, // Atlantic
    {  472,  295,  595300,  595325,  595357, 25, 32, 34, }, // Australia_Central
    {  472,  313,  595391,  595426,  595468, 35, 42, 44, }, // Australia_CentralWestern
    {  472,  338,  595512,  595538,  595571, 26, 33, 35, }, // Australia_Eastern
    {  472,  356,  595606,  595633,  595667, 27, 34, 36, }, // Australia_Western
    {  472,  374,  595703,  595721,  595746, 18, 25, 27, }, // Azerbaijan
    {  472,  385,  595773,  595788,  595810, 15, 22, 24, }, // Azores
    {  472,  397,  595834,  595853,  595879, 19, 26, 28, }, // Bangladesh
    {  472,  415,       0,  595907,       0,  0, 13,  0, }, // Bhutan
    {  472,  422,       0,  595920,       0,  0, 15,  0, }, // Bolivia
    {  472,  437,  135014,  595935,  595958, 16, 23, 25, }, // Brasilia
    {  472,  454,       0,  595983,       0,  0, 25,  0, }, // Brunei
    {  472,  461,  596008,  596026,  596051, 18, 25, 27, }, // Cape_Verde
    {  472,  478,       0,  596078,       0,  0, 23,  0, }, // Chamorro
    {  472,  487,   54300,  596101,  596123, 15, 22, 24, }, // Chatham
    {  472,  495,  596147,  596160,  596180, 13, 20, 22, }, // Chile
    {  472,  501,  596202,  596215,  596235, 13, 20, 22, }, // China
    {  472,  507,   54477,  596257,  596282, 18, 25, 27, }, // Choibalsan
    {  472,  518,       0,  596309,       0,  0, 21,  0, }, // Christmas
    {  472,  528,       0,  596330,       0,  0, 20,  0, }, // Cocos
    {  472,  534,  596350,  596366,  596389, 16, 23, 25, }, // Colombia
    {  472,  543,  596414,  596433,  596459, 19, 26, 28, }, // Cook
    {  472,  548,   54767,  596487,  596506, 12, 19, 21, }, // Cuba
    {  472,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {  472,  575,       0,   54834,       0,  0, 24,  0, }, // DumontDUrville
    {  472,  612,       0,  596527,       0,  0, 19,  0, }, // East_Timor
    {  472,  623,  596546,  596568,  596597, 22, 29, 31, }, // Easter
    {  472,  630,       0,  596628,       0,  0, 15,  0, }, // Ecuador
    {  472,  638,  596643,  596665,  596694, 22, 29, 31, }, // Europe_Central
    {  472,  653,  596725,  596748,  596778, 23, 30, 32, }, // Europe_Eastern
    {  472,  668,       0,  596810,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  472,  691,  596841,  596865,  596896, 24, 31, 33, }, // Europe_Western
    {  472,  706,  596929,  596952,  596982, 23, 30, 32, }, // Falkland
    {  472,  715,  136173,  597014,  597033, 12, 19, 21, }, // Fiji
    {  472,  720,       0,  597054,       0,  0, 23,  0, }, // French_Guiana
    {  472,  734,       0,  597077,       0,  0, 48,  0, }, // French_Southern
    {  472,  757,       0,  597125,       0,  0, 18,  0, }, // Galapagos
    {  472,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {  472,  775,  597143,  597158,  597180, 15, 22, 24, }, // Georgia
    {  472,  783,       0,  597204,       0,  0, 22,  0, }, // Gilbert_Islands
    {  472,  799,       0,  597226,       0,  0, 17,  0, }, // GMT
    {  472,  841,  597243,  597271,  597306, 28, 35, 37, }, // Greenland_Eastern
    {  472,  859,  597343,  597372,  597408, 29, 36, 38, }, // Greenland_Western
    {  472,  877,       0,  597446,       0,  0, 19,  0, }, // Guam
    {  472,  882,       0,  597465,       0,  0, 20,  0, }, // Gulf
    {  472,  887,       0,  597054,       0,  0, 14,  0, }, // Guyana
    {  472,  894,  597485,  597508,  597538, 23, 30, 32, }, // Hawaii_Aleutian
    {  472,  910,  136716,  597570,  597594, 17, 24, 26, }, // Hong_Kong
    {  472,  920,   56045,  597620,  597639, 12, 19, 21, }, // Hovd
    {  472,  925,       0,  597660,       0,  0, 20,  0, }, // India
    {  472,  931,       0,  597680,       0,  0, 21,  0, }, // Indian_Ocean
    {  472,  944,       0,  597701,       0,  0, 17,  0, }, // Indochina
    {  472,  954,       0,  597718,       0,  0, 25,  0, }, // Indonesia_Central
    {  472,  972,       0,  597743,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  472,  990,       0,  597769,       0,  0, 27,  0, }, // Indonesia_Western
    {  472, 1008,  597796,  597808,  597827, 12, 19, 21, }, // Iran
    {  472, 1019,  597848,  597863,  597885, 15, 22, 24, }, // Irkutsk
    {  472, 1027,  597909,  597923,  597944, 14, 21, 23, }, // Israel
    {  472, 1034,  597967,  597980,  598000, 13, 20, 22, }, // Japan
    {  472, 1040,  598022,  598054,  598093, 32, 39, 41, }, // Kamchatka
    {  472, 1058,       0,  598134,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  472, 1077,       0,  598162,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  472, 1106,  598191,  598205,  598226, 14, 21, 23, }, // Korea
    {  472, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {  472, 1119,   56703,  598249,  598275, 19, 26, 28, }, // Krasnoyarsk
    {  472, 1151,       0,  598303,       0,  0, 19,  0, }, // Kyrgystan
    {  472, 1161,       0,  598322,       0,  0, 17,  0, }, // Lanka
    {  472, 1175,       0,  598339,       0,  0, 19,  0, }, // Line_Islands
    {  472, 1188,   56833,  598358,  598382, 17, 24, 26, }, // Lord_Howe
    {  472, 1198,  137554,  598408,  598428, 13, 20, 22, }, // Macau
    {  472, 1204,  137611,  598450,  598472, 15, 22, 24, }, // Magadan
    {  472, 1219,       0,  598496,       0,  0, 15,  0, }, // Malaysia
    {  472, 1228,       0,  598511,       0,  0, 17,  0, }, // Maldives
    {  472, 1237,       0,  598528,       0,  0, 24,  0, }, // Marquesas
    {  472, 1247,       0,  598552,       0,  0, 23,  0, }, // Marshall_Islands
    {  472, 1264,  598575,  598591,  598614, 16, 23, 25, }, // Mauritius
    {  472, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {  472, 1281,  598639,  598664,  598696, 25, 32, 34, }, // Mexico_Pacific
    {  472, 1296,  598730,  598748,  598773, 18, 25, 27, }, // Mongolia
    {  472, 1305,  598800,  598815,  598837, 15, 22, 24, }, // Moscow
    {  472, 1312,       0,  598861,       0,  0, 15,  0, }, // Myanmar
    {  472, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {  472, 1326,       0,  598876,       0,  0, 13,  0, }, // Nepal
    {  472, 1332,  598889,  598911,  598940, 22, 29, 31, }, // New_Caledonia
    {  472, 1346,  598971,  598992,  599020, 21, 28, 30, }, // New_Zealand
    {  472, 1358,  599050,  599068,  599093, 18, 25, 27, }, // Newfoundland
    {  472, 1371,       0,  599120,       0,  0, 12,  0, }, // Niue
    {  472, 1376,  599132,  599152,  599179, 20, 27, 29, }, // Norfolk
    {  472, 1384,   57800,  599208,  599242, 27, 34, 36, }, // Noronha
    {  472, 1392,       0,  599278,       0,  0, 31,  0, }, // North_Mariana
    {  472, 1406,   57936,  599309,  599335, 19, 26, 28, }, // Novosibirsk
    {  472, 1418,  599363,  599375,  599394, 12, 19, 21, }, // Omsk
    {  472, 1428,  599415,  599432,  599456, 17, 24, 26, }, // Pakistan
    {  472, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {  472, 1443,       0,  599482,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  472, 1460,  599506,  599522,  599545, 16, 23, 25, }, // Paraguay
    {  472, 1469,  599570,  599582,  599601, 12, 19, 21, }, // Peru
    {  472, 1474,  599622,  599640,  599665, 18, 25, 27, }, // Philippines
    {  472, 1486,       0,  599692,       0,  0, 20,  0, }, // Phoenix_Islands
    {  472, 1502,  599712,  599740,  599775, 28, 35, 37, }, // Pierre_Miquelon
    {  472, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {  472, 1527,       0,   58521,       0,  0, 14,  0, }, // Ponape
    {  472, 1534,       0,  138941,       0,  0, 17,  0, }, // Pyongyang
    {  472, 1544,   58552,  599812,  599836, 17, 24, 26, }, // Qyzylorda
    {  472, 1554,       0,  599862,       0,  0, 15,  0, }, // Reunion
    {  472, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {  472, 1570,  599877,  599893,  599916, 16, 23, 25, }, // Sakhalin
    {  472, 1579,   58714,  599941,  599962, 14, 21, 23, }, // Samara
    {  472, 1596,   58774,  599985,  600005, 13, 20, 22, }, // Samoa
    {  472, 1602,       0,  600027,       0,  0, 18,  0, }, // Seychelles
    {  472, 1624,       0,  600045,       0,  0, 24,  0, }, // Singapore
    {  472, 1634,       0,  600069,       0,  0, 22,  0, }, // Solomon
    {  472, 1642,       0,  600091,       0,  0, 22,  0, }, // South_Georgia
    {  472, 1656,       0,  600113,       0,  0, 16,  0, }, // Suriname
    {  472, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {  472, 1682,       0,  600129,       0,  0, 13,  0, }, // Tahiti
    {  472, 1689,   58967,  600142,  600162, 13, 20, 22, }, // Taipei
    {  472, 1696,       0,  600184,       0,  0, 19,  0, }, // Tajikistan
    {  472, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {  472, 1732,   59062,  600203,  600223, 13, 20, 22, }, // Tonga
    {  472, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {  472, 1750,  600245,  600267,  600296, 22, 29, 31, }, // Turkmenistan
    {  472, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {  472, 1777,  600327,  600342,  600364, 15, 22, 24, }, // Uruguay
    {  472, 1792,  600388,  600407,  600433, 19, 26, 28, }, // Uzbekistan
    {  472, 1803,  600461,  600476,  600498, 15, 22, 24, }, // Vanuatu
    {  472, 1811,       0,  600522,       0,  0, 17,  0, }, // Venezuela
    {  472, 1821,   59451,  600539,  600565, 19, 26, 28, }, // Vladivostok
    {  472, 1833,  600593,  600611,  600636, 18, 25, 27, }, // Volgograd
    {  472, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {  472, 1850,       0,  600663,       0,  0, 17,  0, }, // Wake
    {  472, 1855,       0,  600680,       0,  0, 23,  0, }, // Wallis
    {  472, 1862,   59653,  600703,  600725, 15, 22, 24, }, // Yakutsk
    {  472, 1870,  600749,  600770,  600798, 21, 28, 30, }, // Yekaterinburg
    {  472, 1892,       0,  600828,       0,  0, 13,  0, }, // Yukon
    {  473,    0,  593895,  593907,  593926, 12, 19, 21, }, // Acre Portuguese/Latin/Guinea-Bissau
    {  473,    5,       0,  593947,       0,  0, 19,  0, }, // Afghanistan
    {  473,   17,       0,  593966,       0,  0, 22,  0, }, // Africa_Central
    {  473,   32,       0,  593988,       0,  0, 23,  0, }, // Africa_Eastern
    {  473,   65,       0,  594011,       0,  0, 21,  0, }, // Africa_Southern
    {  473,   81,  594032,  594056,  594087, 24, 31, 33, }, // Africa_Western
    {  473,  107,  594120,  594134,  594155, 14, 21, 23, }, // Alaska
    {  473,  128,  594178,  594192,  594213, 14, 21, 23, }, // Almaty
    {  473,  135,  594236,  594252,  594275, 16, 23, 25, }, // Amazon
    {  473,  142,  594300,  594328,  594363, 28, 35, 37, }, // America_Central
    {  473,  158,  594400,  594429,  594465, 29, 36, 38, }, // America_Eastern
    {  473,  174,  594503,  594535,  594574, 32, 39, 41, }, // America_Mountain
    {  473,  191,  594615,  594647,  594686, 32, 39, 41, }, // America_Pacific
    {  473,  207,  594727,  594741,  594762, 14, 21, 23, }, // Anadyr
    {  473,  214,  594785,  594797,  594816, 12, 19, 21, }, // Apia
    {  473,  219,  594837,  594850,  594870, 13, 20, 22, }, // Aqtau
    {  473,  225,  594892,  594906,  594927, 14, 21, 23, }, // Aqtobe
    {  473,  232,  594950,  594964,  594985, 14, 21, 23, }, // Arabian
    {  473,  240,  595008,  595025,  595049, 17, 24, 26, }, // Argentina
    {  473,  250,  595075,  595102,  595136, 27, 34, 36, }, // Argentina_Western
    {  473,  268,  595172,  595187,  595209, 15, 22, 24, }, // Armenia
    {  473,  286,  595233,  595250,  595274, 17, 24, 26, }, // Atlantic
    {  473,  295,  595300,  595325,  595357, 25, 32, 34, }, // Australia_Central
    {  473,  313,  595391,  595426,  595468, 35, 42, 44, }, // Australia_CentralWestern
    {  473,  338,  595512,  595538,  595571, 26, 33, 35, }, // Australia_Eastern
    {  473,  356,  595606,  595633,  595667, 27, 34, 36, }, // Australia_Western
    {  473,  374,  595703,  595721,  595746, 18, 25, 27, }, // Azerbaijan
    {  473,  385,  595773,  595788,  595810, 15, 22, 24, }, // Azores
    {  473,  397,  595834,  595853,  595879, 19, 26, 28, }, // Bangladesh
    {  473,  415,       0,  595907,       0,  0, 13,  0, }, // Bhutan
    {  473,  422,       0,  595920,       0,  0, 15,  0, }, // Bolivia
    {  473,  437,  135014,  595935,  595958, 16, 23, 25, }, // Brasilia
    {  473,  454,       0,  595983,       0,  0, 25,  0, }, // Brunei
    {  473,  461,  596008,  596026,  596051, 18, 25, 27, }, // Cape_Verde
    {  473,  478,       0,  596078,       0,  0, 23,  0, }, // Chamorro
    {  473,  487,   54300,  596101,  596123, 15, 22, 24, }, // Chatham
    {  473,  495,  596147,  596160,  596180, 13, 20, 22, }, // Chile
    {  473,  501,  596202,  596215,  596235, 13, 20, 22, }, // China
    {  473,  507,   54477,  596257,  596282, 18, 25, 27, }, // Choibalsan
    {  473,  518,       0,  596309,       0,  0, 21,  0, }, // Christmas
    {  473,  528,       0,  596330,       0,  0, 20,  0, }, // Cocos
    {  473,  534,  596350,  596366,  596389, 16, 23, 25, }, // Colombia
    {  473,  543,  596414,  596433,  596459, 19, 26, 28, }, // Cook
    {  473,  548,   54767,  596487,  596506, 12, 19, 21, }, // Cuba
    {  473,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {  473,  575,       0,   54834,       0,  0, 24,  0, }, // DumontDUrville
    {  473,  612,       0,  596527,       0,  0, 19,  0, }, // East_Timor
    {  473,  623,  596546,  596568,  596597, 22, 29, 31, }, // Easter
    {  473,  630,       0,  596628,       0,  0, 15,  0, }, // Ecuador
    {  473,  638,  596643,  596665,  596694, 22, 29, 31, }, // Europe_Central
    {  473,  653,  596725,  596748,  596778, 23, 30, 32, }, // Europe_Eastern
    {  473,  668,       0,  596810,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  473,  691,  596841,  596865,  596896, 24, 31, 33, }, // Europe_Western
    {  473,  706,  596929,  596952,  596982, 23, 30, 32, }, // Falkland
    {  473,  715,  136173,  597014,  597033, 12, 19, 21, }, // Fiji
    {  473,  720,       0,  597054,       0,  0, 23,  0, }, // French_Guiana
    {  473,  734,       0,  597077,       0,  0, 48,  0, }, // French_Southern
    {  473,  757,       0,  597125,       0,  0, 18,  0, }, // Galapagos
    {  473,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {  473,  775,  597143,  597158,  597180, 15, 22, 24, }, // Georgia
    {  473,  783,       0,  597204,       0,  0, 22,  0, }, // Gilbert_Islands
    {  473,  799,       0,  597226,       0,  0, 17,  0, }, // GMT
    {  473,  841,  597243,  597271,  597306, 28, 35, 37, }, // Greenland_Eastern
    {  473,  859,  597343,  597372,  597408, 29, 36, 38, }, // Greenland_Western
    {  473,  877,       0,  597446,       0,  0, 19,  0, }, // Guam
    {  473,  882,       0,  597465,       0,  0, 20,  0, }, // Gulf
    {  473,  887,       0,  597054,       0,  0, 14,  0, }, // Guyana
    {  473,  894,  597485,  597508,  597538, 23, 30, 32, }, // Hawaii_Aleutian
    {  473,  910,  136716,  597570,  597594, 17, 24, 26, }, // Hong_Kong
    {  473,  920,   56045,  597620,  597639, 12, 19, 21, }, // Hovd
    {  473,  925,       0,  597660,       0,  0, 20,  0, }, // India
    {  473,  931,       0,  597680,       0,  0, 21,  0, }, // Indian_Ocean
    {  473,  944,       0,  597701,       0,  0, 17,  0, }, // Indochina
    {  473,  954,       0,  597718,       0,  0, 25,  0, }, // Indonesia_Central
    {  473,  972,       0,  597743,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  473,  990,       0,  597769,       0,  0, 27,  0, }, // Indonesia_Western
    {  473, 1008,  597796,  597808,  597827, 12, 19, 21, }, // Iran
    {  473, 1019,  597848,  597863,  597885, 15, 22, 24, }, // Irkutsk
    {  473, 1027,  597909,  597923,  597944, 14, 21, 23, }, // Israel
    {  473, 1034,  597967,  597980,  598000, 13, 20, 22, }, // Japan
    {  473, 1040,  598022,  598054,  598093, 32, 39, 41, }, // Kamchatka
    {  473, 1058,       0,  598134,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  473, 1077,       0,  598162,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  473, 1106,  598191,  598205,  598226, 14, 21, 23, }, // Korea
    {  473, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {  473, 1119,   56703,  598249,  598275, 19, 26, 28, }, // Krasnoyarsk
    {  473, 1151,       0,  598303,       0,  0, 19,  0, }, // Kyrgystan
    {  473, 1161,       0,  598322,       0,  0, 17,  0, }, // Lanka
    {  473, 1175,       0,  598339,       0,  0, 19,  0, }, // Line_Islands
    {  473, 1188,   56833,  598358,  598382, 17, 24, 26, }, // Lord_Howe
    {  473, 1198,  137554,  598408,  598428, 13, 20, 22, }, // Macau
    {  473, 1204,  137611,  598450,  598472, 15, 22, 24, }, // Magadan
    {  473, 1219,       0,  598496,       0,  0, 15,  0, }, // Malaysia
    {  473, 1228,       0,  598511,       0,  0, 17,  0, }, // Maldives
    {  473, 1237,       0,  598528,       0,  0, 24,  0, }, // Marquesas
    {  473, 1247,       0,  598552,       0,  0, 23,  0, }, // Marshall_Islands
    {  473, 1264,  598575,  598591,  598614, 16, 23, 25, }, // Mauritius
    {  473, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {  473, 1281,  598639,  598664,  598696, 25, 32, 34, }, // Mexico_Pacific
    {  473, 1296,  598730,  598748,  598773, 18, 25, 27, }, // Mongolia
    {  473, 1305,  598800,  598815,  598837, 15, 22, 24, }, // Moscow
    {  473, 1312,       0,  598861,       0,  0, 15,  0, }, // Myanmar
    {  473, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {  473, 1326,       0,  598876,       0,  0, 13,  0, }, // Nepal
    {  473, 1332,  598889,  598911,  598940, 22, 29, 31, }, // New_Caledonia
    {  473, 1346,  598971,  598992,  599020, 21, 28, 30, }, // New_Zealand
    {  473, 1358,  599050,  599068,  599093, 18, 25, 27, }, // Newfoundland
    {  473, 1371,       0,  599120,       0,  0, 12,  0, }, // Niue
    {  473, 1376,  599132,  599152,  599179, 20, 27, 29, }, // Norfolk
    {  473, 1384,   57800,  599208,  599242, 27, 34, 36, }, // Noronha
    {  473, 1392,       0,  599278,       0,  0, 31,  0, }, // North_Mariana
    {  473, 1406,   57936,  599309,  599335, 19, 26, 28, }, // Novosibirsk
    {  473, 1418,  599363,  599375,  599394, 12, 19, 21, }, // Omsk
    {  473, 1428,  599415,  599432,  599456, 17, 24, 26, }, // Pakistan
    {  473, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {  473, 1443,       0,  599482,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  473, 1460,  599506,  599522,  599545, 16, 23, 25, }, // Paraguay
    {  473, 1469,  599570,  599582,  599601, 12, 19, 21, }, // Peru
    {  473, 1474,  599622,  599640,  599665, 18, 25, 27, }, // Philippines
    {  473, 1486,       0,  599692,       0,  0, 20,  0, }, // Phoenix_Islands
    {  473, 1502,  599712,  599740,  599775, 28, 35, 37, }, // Pierre_Miquelon
    {  473, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {  473, 1527,       0,   58521,       0,  0, 14,  0, }, // Ponape
    {  473, 1534,       0,  138941,       0,  0, 17,  0, }, // Pyongyang
    {  473, 1544,   58552,  599812,  599836, 17, 24, 26, }, // Qyzylorda
    {  473, 1554,       0,  599862,       0,  0, 15,  0, }, // Reunion
    {  473, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {  473, 1570,  599877,  599893,  599916, 16, 23, 25, }, // Sakhalin
    {  473, 1579,   58714,  599941,  599962, 14, 21, 23, }, // Samara
    {  473, 1596,   58774,  599985,  600005, 13, 20, 22, }, // Samoa
    {  473, 1602,       0,  600027,       0,  0, 18,  0, }, // Seychelles
    {  473, 1624,       0,  600045,       0,  0, 24,  0, }, // Singapore
    {  473, 1634,       0,  600069,       0,  0, 22,  0, }, // Solomon
    {  473, 1642,       0,  600091,       0,  0, 22,  0, }, // South_Georgia
    {  473, 1656,       0,  600113,       0,  0, 16,  0, }, // Suriname
    {  473, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {  473, 1682,       0,  600129,       0,  0, 13,  0, }, // Tahiti
    {  473, 1689,   58967,  600142,  600162, 13, 20, 22, }, // Taipei
    {  473, 1696,       0,  600184,       0,  0, 19,  0, }, // Tajikistan
    {  473, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {  473, 1732,   59062,  600203,  600223, 13, 20, 22, }, // Tonga
    {  473, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {  473, 1750,  600245,  600267,  600296, 22, 29, 31, }, // Turkmenistan
    {  473, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {  473, 1777,  600327,  600342,  600364, 15, 22, 24, }, // Uruguay
    {  473, 1792,  600388,  600407,  600433, 19, 26, 28, }, // Uzbekistan
    {  473, 1803,  600461,  600476,  600498, 15, 22, 24, }, // Vanuatu
    {  473, 1811,       0,  600522,       0,  0, 17,  0, }, // Venezuela
    {  473, 1821,   59451,  600539,  600565, 19, 26, 28, }, // Vladivostok
    {  473, 1833,  600593,  600611,  600636, 18, 25, 27, }, // Volgograd
    {  473, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {  473, 1850,       0,  600663,       0,  0, 17,  0, }, // Wake
    {  473, 1855,       0,  600680,       0,  0, 23,  0, }, // Wallis
    {  473, 1862,   59653,  600703,  600725, 15, 22, 24, }, // Yakutsk
    {  473, 1870,  600749,  600770,  600798, 21, 28, 30, }, // Yekaterinburg
    {  473, 1892,       0,  600828,       0,  0, 13,  0, }, // Yukon
    {  474,    0,  593895,  593907,  593926, 12, 19, 21, }, // Acre Portuguese/Latin/Luxembourg
    {  474,    5,       0,  593947,       0,  0, 19,  0, }, // Afghanistan
    {  474,   17,       0,  593966,       0,  0, 22,  0, }, // Africa_Central
    {  474,   32,       0,  593988,       0,  0, 23,  0, }, // Africa_Eastern
    {  474,   65,       0,  594011,       0,  0, 21,  0, }, // Africa_Southern
    {  474,   81,  594032,  594056,  594087, 24, 31, 33, }, // Africa_Western
    {  474,  107,  594120,  594134,  594155, 14, 21, 23, }, // Alaska
    {  474,  128,  594178,  594192,  594213, 14, 21, 23, }, // Almaty
    {  474,  135,  594236,  594252,  594275, 16, 23, 25, }, // Amazon
    {  474,  142,  594300,  594328,  594363, 28, 35, 37, }, // America_Central
    {  474,  158,  594400,  594429,  594465, 29, 36, 38, }, // America_Eastern
    {  474,  174,  594503,  594535,  594574, 32, 39, 41, }, // America_Mountain
    {  474,  191,  594615,  594647,  594686, 32, 39, 41, }, // America_Pacific
    {  474,  207,  594727,  594741,  594762, 14, 21, 23, }, // Anadyr
    {  474,  214,  594785,  594797,  594816, 12, 19, 21, }, // Apia
    {  474,  219,  594837,  594850,  594870, 13, 20, 22, }, // Aqtau
    {  474,  225,  594892,  594906,  594927, 14, 21, 23, }, // Aqtobe
    {  474,  232,  594950,  594964,  594985, 14, 21, 23, }, // Arabian
    {  474,  240,  595008,  595025,  595049, 17, 24, 26, }, // Argentina
    {  474,  250,  595075,  595102,  595136, 27, 34, 36, }, // Argentina_Western
    {  474,  268,  595172,  595187,  595209, 15, 22, 24, }, // Armenia
    {  474,  286,  595233,  595250,  595274, 17, 24, 26, }, // Atlantic
    {  474,  295,  595300,  595325,  595357, 25, 32, 34, }, // Australia_Central
    {  474,  313,  595391,  595426,  595468, 35, 42, 44, }, // Australia_CentralWestern
    {  474,  338,  595512,  595538,  595571, 26, 33, 35, }, // Australia_Eastern
    {  474,  356,  595606,  595633,  595667, 27, 34, 36, }, // Australia_Western
    {  474,  374,  595703,  595721,  595746, 18, 25, 27, }, // Azerbaijan
    {  474,  385,  595773,  595788,  595810, 15, 22, 24, }, // Azores
    {  474,  397,  595834,  595853,  595879, 19, 26, 28, }, // Bangladesh
    {  474,  415,       0,  595907,       0,  0, 13,  0, }, // Bhutan
    {  474,  422,       0,  595920,       0,  0, 15,  0, }, // Bolivia
    {  474,  437,  135014,  595935,  595958, 16, 23, 25, }, // Brasilia
    {  474,  454,       0,  595983,       0,  0, 25,  0, }, // Brunei
    {  474,  461,  596008,  596026,  596051, 18, 25, 27, }, // Cape_Verde
    {  474,  478,       0,  596078,       0,  0, 23,  0, }, // Chamorro
    {  474,  487,   54300,  596101,  596123, 15, 22, 24, }, // Chatham
    {  474,  495,  596147,  596160,  596180, 13, 20, 22, }, // Chile
    {  474,  501,  596202,  596215,  596235, 13, 20, 22, }, // China
    {  474,  507,   54477,  596257,  596282, 18, 25, 27, }, // Choibalsan
    {  474,  518,       0,  596309,       0,  0, 21,  0, }, // Christmas
    {  474,  528,       0,  596330,       0,  0, 20,  0, }, // Cocos
    {  474,  534,  596350,  596366,  596389, 16, 23, 25, }, // Colombia
    {  474,  543,  596414,  596433,  596459, 19, 26, 28, }, // Cook
    {  474,  548,   54767,  596487,  596506, 12, 19, 21, }, // Cuba
    {  474,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {  474,  575,       0,   54834,       0,  0, 24,  0, }, // DumontDUrville
    {  474,  612,       0,  596527,       0,  0, 19,  0, }, // East_Timor
    {  474,  623,  596546,  596568,  596597, 22, 29, 31, }, // Easter
    {  474,  630,       0,  596628,       0,  0, 15,  0, }, // Ecuador
    {  474,  638,  596643,  596665,  596694, 22, 29, 31, }, // Europe_Central
    {  474,  653,  596725,  596748,  596778, 23, 30, 32, }, // Europe_Eastern
    {  474,  668,       0,  596810,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  474,  691,  596841,  596865,  596896, 24, 31, 33, }, // Europe_Western
    {  474,  706,  596929,  596952,  596982, 23, 30, 32, }, // Falkland
    {  474,  715,  136173,  597014,  597033, 12, 19, 21, }, // Fiji
    {  474,  720,       0,  597054,       0,  0, 23,  0, }, // French_Guiana
    {  474,  734,       0,  597077,       0,  0, 48,  0, }, // French_Southern
    {  474,  757,       0,  597125,       0,  0, 18,  0, }, // Galapagos
    {  474,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {  474,  775,  597143,  597158,  597180, 15, 22, 24, }, // Georgia
    {  474,  783,       0,  597204,       0,  0, 22,  0, }, // Gilbert_Islands
    {  474,  799,       0,  597226,       0,  0, 17,  0, }, // GMT
    {  474,  841,  597243,  597271,  597306, 28, 35, 37, }, // Greenland_Eastern
    {  474,  859,  597343,  597372,  597408, 29, 36, 38, }, // Greenland_Western
    {  474,  877,       0,  597446,       0,  0, 19,  0, }, // Guam
    {  474,  882,       0,  597465,       0,  0, 20,  0, }, // Gulf
    {  474,  887,       0,  597054,       0,  0, 14,  0, }, // Guyana
    {  474,  894,  597485,  597508,  597538, 23, 30, 32, }, // Hawaii_Aleutian
    {  474,  910,  136716,  597570,  597594, 17, 24, 26, }, // Hong_Kong
    {  474,  920,   56045,  597620,  597639, 12, 19, 21, }, // Hovd
    {  474,  925,       0,  597660,       0,  0, 20,  0, }, // India
    {  474,  931,       0,  597680,       0,  0, 21,  0, }, // Indian_Ocean
    {  474,  944,       0,  597701,       0,  0, 17,  0, }, // Indochina
    {  474,  954,       0,  597718,       0,  0, 25,  0, }, // Indonesia_Central
    {  474,  972,       0,  597743,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  474,  990,       0,  597769,       0,  0, 27,  0, }, // Indonesia_Western
    {  474, 1008,  597796,  597808,  597827, 12, 19, 21, }, // Iran
    {  474, 1019,  597848,  597863,  597885, 15, 22, 24, }, // Irkutsk
    {  474, 1027,  597909,  597923,  597944, 14, 21, 23, }, // Israel
    {  474, 1034,  597967,  597980,  598000, 13, 20, 22, }, // Japan
    {  474, 1040,  598022,  598054,  598093, 32, 39, 41, }, // Kamchatka
    {  474, 1058,       0,  598134,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  474, 1077,       0,  598162,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  474, 1106,  598191,  598205,  598226, 14, 21, 23, }, // Korea
    {  474, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {  474, 1119,   56703,  598249,  598275, 19, 26, 28, }, // Krasnoyarsk
    {  474, 1151,       0,  598303,       0,  0, 19,  0, }, // Kyrgystan
    {  474, 1161,       0,  598322,       0,  0, 17,  0, }, // Lanka
    {  474, 1175,       0,  598339,       0,  0, 19,  0, }, // Line_Islands
    {  474, 1188,   56833,  598358,  598382, 17, 24, 26, }, // Lord_Howe
    {  474, 1198,  137554,  598408,  598428, 13, 20, 22, }, // Macau
    {  474, 1204,  137611,  598450,  598472, 15, 22, 24, }, // Magadan
    {  474, 1219,       0,  598496,       0,  0, 15,  0, }, // Malaysia
    {  474, 1228,       0,  598511,       0,  0, 17,  0, }, // Maldives
    {  474, 1237,       0,  598528,       0,  0, 24,  0, }, // Marquesas
    {  474, 1247,       0,  598552,       0,  0, 23,  0, }, // Marshall_Islands
    {  474, 1264,  598575,  598591,  598614, 16, 23, 25, }, // Mauritius
    {  474, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {  474, 1281,  598639,  598664,  598696, 25, 32, 34, }, // Mexico_Pacific
    {  474, 1296,  598730,  598748,  598773, 18, 25, 27, }, // Mongolia
    {  474, 1305,  598800,  598815,  598837, 15, 22, 24, }, // Moscow
    {  474, 1312,       0,  598861,       0,  0, 15,  0, }, // Myanmar
    {  474, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {  474, 1326,       0,  598876,       0,  0, 13,  0, }, // Nepal
    {  474, 1332,  598889,  598911,  598940, 22, 29, 31, }, // New_Caledonia
    {  474, 1346,  598971,  598992,  599020, 21, 28, 30, }, // New_Zealand
    {  474, 1358,  599050,  599068,  599093, 18, 25, 27, }, // Newfoundland
    {  474, 1371,       0,  599120,       0,  0, 12,  0, }, // Niue
    {  474, 1376,  599132,  599152,  599179, 20, 27, 29, }, // Norfolk
    {  474, 1384,   57800,  599208,  599242, 27, 34, 36, }, // Noronha
    {  474, 1392,       0,  599278,       0,  0, 31,  0, }, // North_Mariana
    {  474, 1406,   57936,  599309,  599335, 19, 26, 28, }, // Novosibirsk
    {  474, 1418,  599363,  599375,  599394, 12, 19, 21, }, // Omsk
    {  474, 1428,  599415,  599432,  599456, 17, 24, 26, }, // Pakistan
    {  474, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {  474, 1443,       0,  599482,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  474, 1460,  599506,  599522,  599545, 16, 23, 25, }, // Paraguay
    {  474, 1469,  599570,  599582,  599601, 12, 19, 21, }, // Peru
    {  474, 1474,  599622,  599640,  599665, 18, 25, 27, }, // Philippines
    {  474, 1486,       0,  599692,       0,  0, 20,  0, }, // Phoenix_Islands
    {  474, 1502,  599712,  599740,  599775, 28, 35, 37, }, // Pierre_Miquelon
    {  474, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {  474, 1527,       0,   58521,       0,  0, 14,  0, }, // Ponape
    {  474, 1534,       0,  138941,       0,  0, 17,  0, }, // Pyongyang
    {  474, 1544,   58552,  599812,  599836, 17, 24, 26, }, // Qyzylorda
    {  474, 1554,       0,  599862,       0,  0, 15,  0, }, // Reunion
    {  474, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {  474, 1570,  599877,  599893,  599916, 16, 23, 25, }, // Sakhalin
    {  474, 1579,   58714,  599941,  599962, 14, 21, 23, }, // Samara
    {  474, 1596,   58774,  599985,  600005, 13, 20, 22, }, // Samoa
    {  474, 1602,       0,  600027,       0,  0, 18,  0, }, // Seychelles
    {  474, 1624,       0,  600045,       0,  0, 24,  0, }, // Singapore
    {  474, 1634,       0,  600069,       0,  0, 22,  0, }, // Solomon
    {  474, 1642,       0,  600091,       0,  0, 22,  0, }, // South_Georgia
    {  474, 1656,       0,  600113,       0,  0, 16,  0, }, // Suriname
    {  474, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {  474, 1682,       0,  600129,       0,  0, 13,  0, }, // Tahiti
    {  474, 1689,   58967,  600142,  600162, 13, 20, 22, }, // Taipei
    {  474, 1696,       0,  600184,       0,  0, 19,  0, }, // Tajikistan
    {  474, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {  474, 1732,   59062,  600203,  600223, 13, 20, 22, }, // Tonga
    {  474, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {  474, 1750,  600245,  600267,  600296, 22, 29, 31, }, // Turkmenistan
    {  474, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {  474, 1777,  600327,  600342,  600364, 15, 22, 24, }, // Uruguay
    {  474, 1792,  600388,  600407,  600433, 19, 26, 28, }, // Uzbekistan
    {  474, 1803,  600461,  600476,  600498, 15, 22, 24, }, // Vanuatu
    {  474, 1811,       0,  600522,       0,  0, 17,  0, }, // Venezuela
    {  474, 1821,   59451,  600539,  600565, 19, 26, 28, }, // Vladivostok
    {  474, 1833,  600593,  600611,  600636, 18, 25, 27, }, // Volgograd
    {  474, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {  474, 1850,       0,  600663,       0,  0, 17,  0, }, // Wake
    {  474, 1855,       0,  600680,       0,  0, 23,  0, }, // Wallis
    {  474, 1862,   59653,  600703,  600725, 15, 22, 24, }, // Yakutsk
    {  474, 1870,  600749,  600770,  600798, 21, 28, 30, }, // Yekaterinburg
    {  474, 1892,       0,  600828,       0,  0, 13,  0, }, // Yukon
    {  475,    0,  593895,  593907,  593926, 12, 19, 21, }, // Acre Portuguese/Latin/Macao
    {  475,    5,       0,  593947,       0,  0, 19,  0, }, // Afghanistan
    {  475,   17,       0,  593966,       0,  0, 22,  0, }, // Africa_Central
    {  475,   32,       0,  593988,       0,  0, 23,  0, }, // Africa_Eastern
    {  475,   65,       0,  594011,       0,  0, 21,  0, }, // Africa_Southern
    {  475,   81,  594032,  594056,  594087, 24, 31, 33, }, // Africa_Western
    {  475,  107,  594120,  594134,  594155, 14, 21, 23, }, // Alaska
    {  475,  128,  594178,  594192,  594213, 14, 21, 23, }, // Almaty
    {  475,  135,  594236,  594252,  594275, 16, 23, 25, }, // Amazon
    {  475,  142,  594300,  594328,  594363, 28, 35, 37, }, // America_Central
    {  475,  158,  594400,  594429,  594465, 29, 36, 38, }, // America_Eastern
    {  475,  174,  594503,  594535,  594574, 32, 39, 41, }, // America_Mountain
    {  475,  191,  594615,  594647,  594686, 32, 39, 41, }, // America_Pacific
    {  475,  207,  594727,  594741,  594762, 14, 21, 23, }, // Anadyr
    {  475,  214,  594785,  594797,  594816, 12, 19, 21, }, // Apia
    {  475,  219,  594837,  594850,  594870, 13, 20, 22, }, // Aqtau
    {  475,  225,  594892,  594906,  594927, 14, 21, 23, }, // Aqtobe
    {  475,  232,  594950,  594964,  594985, 14, 21, 23, }, // Arabian
    {  475,  240,  595008,  595025,  595049, 17, 24, 26, }, // Argentina
    {  475,  250,  595075,  595102,  595136, 27, 34, 36, }, // Argentina_Western
    {  475,  268,  595172,  595187,  595209, 15, 22, 24, }, // Armenia
    {  475,  286,  595233,  595250,  595274, 17, 24, 26, }, // Atlantic
    {  475,  295,  595300,  595325,  595357, 25, 32, 34, }, // Australia_Central
    {  475,  313,  595391,  595426,  595468, 35, 42, 44, }, // Australia_CentralWestern
    {  475,  338,  595512,  595538,  595571, 26, 33, 35, }, // Australia_Eastern
    {  475,  356,  595606,  595633,  595667, 27, 34, 36, }, // Australia_Western
    {  475,  374,  595703,  595721,  595746, 18, 25, 27, }, // Azerbaijan
    {  475,  385,  595773,  595788,  595810, 15, 22, 24, }, // Azores
    {  475,  397,  595834,  595853,  595879, 19, 26, 28, }, // Bangladesh
    {  475,  415,       0,  595907,       0,  0, 13,  0, }, // Bhutan
    {  475,  422,       0,  595920,       0,  0, 15,  0, }, // Bolivia
    {  475,  437,  135014,  595935,  595958, 16, 23, 25, }, // Brasilia
    {  475,  454,       0,  595983,       0,  0, 25,  0, }, // Brunei
    {  475,  461,  596008,  596026,  596051, 18, 25, 27, }, // Cape_Verde
    {  475,  478,       0,  596078,       0,  0, 23,  0, }, // Chamorro
    {  475,  487,   54300,  596101,  596123, 15, 22, 24, }, // Chatham
    {  475,  495,  596147,  596160,  596180, 13, 20, 22, }, // Chile
    {  475,  501,  596202,  596215,  596235, 13, 20, 22, }, // China
    {  475,  507,   54477,  596257,  596282, 18, 25, 27, }, // Choibalsan
    {  475,  518,       0,  596309,       0,  0, 21,  0, }, // Christmas
    {  475,  528,       0,  596330,       0,  0, 20,  0, }, // Cocos
    {  475,  534,  596350,  596366,  596389, 16, 23, 25, }, // Colombia
    {  475,  543,  596414,  596433,  596459, 19, 26, 28, }, // Cook
    {  475,  548,   54767,  596487,  596506, 12, 19, 21, }, // Cuba
    {  475,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {  475,  575,       0,   54834,       0,  0, 24,  0, }, // DumontDUrville
    {  475,  612,       0,  596527,       0,  0, 19,  0, }, // East_Timor
    {  475,  623,  596546,  596568,  596597, 22, 29, 31, }, // Easter
    {  475,  630,       0,  596628,       0,  0, 15,  0, }, // Ecuador
    {  475,  638,  596643,  596665,  596694, 22, 29, 31, }, // Europe_Central
    {  475,  653,  596725,  596748,  596778, 23, 30, 32, }, // Europe_Eastern
    {  475,  668,       0,  596810,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  475,  691,  596841,  596865,  596896, 24, 31, 33, }, // Europe_Western
    {  475,  706,  596929,  596952,  596982, 23, 30, 32, }, // Falkland
    {  475,  715,  136173,  597014,  597033, 12, 19, 21, }, // Fiji
    {  475,  720,       0,  597054,       0,  0, 23,  0, }, // French_Guiana
    {  475,  734,       0,  597077,       0,  0, 48,  0, }, // French_Southern
    {  475,  757,       0,  597125,       0,  0, 18,  0, }, // Galapagos
    {  475,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {  475,  775,  597143,  597158,  597180, 15, 22, 24, }, // Georgia
    {  475,  783,       0,  597204,       0,  0, 22,  0, }, // Gilbert_Islands
    {  475,  799,       0,  597226,       0,  0, 17,  0, }, // GMT
    {  475,  841,  597243,  597271,  597306, 28, 35, 37, }, // Greenland_Eastern
    {  475,  859,  597343,  597372,  597408, 29, 36, 38, }, // Greenland_Western
    {  475,  877,       0,  597446,       0,  0, 19,  0, }, // Guam
    {  475,  882,       0,  597465,       0,  0, 20,  0, }, // Gulf
    {  475,  887,       0,  597054,       0,  0, 14,  0, }, // Guyana
    {  475,  894,  597485,  597508,  597538, 23, 30, 32, }, // Hawaii_Aleutian
    {  475,  910,  136716,  597570,  597594, 17, 24, 26, }, // Hong_Kong
    {  475,  920,   56045,  597620,  597639, 12, 19, 21, }, // Hovd
    {  475,  925,       0,  597660,       0,  0, 20,  0, }, // India
    {  475,  931,       0,  597680,       0,  0, 21,  0, }, // Indian_Ocean
    {  475,  944,       0,  597701,       0,  0, 17,  0, }, // Indochina
    {  475,  954,       0,  597718,       0,  0, 25,  0, }, // Indonesia_Central
    {  475,  972,       0,  597743,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  475,  990,       0,  597769,       0,  0, 27,  0, }, // Indonesia_Western
    {  475, 1008,  597796,  597808,  597827, 12, 19, 21, }, // Iran
    {  475, 1019,  597848,  597863,  597885, 15, 22, 24, }, // Irkutsk
    {  475, 1027,  597909,  597923,  597944, 14, 21, 23, }, // Israel
    {  475, 1034,  597967,  597980,  598000, 13, 20, 22, }, // Japan
    {  475, 1040,  598022,  598054,  598093, 32, 39, 41, }, // Kamchatka
    {  475, 1058,       0,  598134,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  475, 1077,       0,  598162,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  475, 1106,  598191,  598205,  598226, 14, 21, 23, }, // Korea
    {  475, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {  475, 1119,   56703,  598249,  598275, 19, 26, 28, }, // Krasnoyarsk
    {  475, 1151,       0,  598303,       0,  0, 19,  0, }, // Kyrgystan
    {  475, 1161,       0,  598322,       0,  0, 17,  0, }, // Lanka
    {  475, 1175,       0,  598339,       0,  0, 19,  0, }, // Line_Islands
    {  475, 1188,   56833,  598358,  598382, 17, 24, 26, }, // Lord_Howe
    {  475, 1198,  137554,  598408,  598428, 13, 20, 22, }, // Macau
    {  475, 1204,  137611,  598450,  598472, 15, 22, 24, }, // Magadan
    {  475, 1219,       0,  598496,       0,  0, 15,  0, }, // Malaysia
    {  475, 1228,       0,  598511,       0,  0, 17,  0, }, // Maldives
    {  475, 1237,       0,  598528,       0,  0, 24,  0, }, // Marquesas
    {  475, 1247,       0,  598552,       0,  0, 23,  0, }, // Marshall_Islands
    {  475, 1264,  598575,  598591,  598614, 16, 23, 25, }, // Mauritius
    {  475, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {  475, 1281,  598639,  598664,  598696, 25, 32, 34, }, // Mexico_Pacific
    {  475, 1296,  598730,  598748,  598773, 18, 25, 27, }, // Mongolia
    {  475, 1305,  598800,  598815,  598837, 15, 22, 24, }, // Moscow
    {  475, 1312,       0,  598861,       0,  0, 15,  0, }, // Myanmar
    {  475, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {  475, 1326,       0,  598876,       0,  0, 13,  0, }, // Nepal
    {  475, 1332,  598889,  598911,  598940, 22, 29, 31, }, // New_Caledonia
    {  475, 1346,  598971,  598992,  599020, 21, 28, 30, }, // New_Zealand
    {  475, 1358,  599050,  599068,  599093, 18, 25, 27, }, // Newfoundland
    {  475, 1371,       0,  599120,       0,  0, 12,  0, }, // Niue
    {  475, 1376,  599132,  599152,  599179, 20, 27, 29, }, // Norfolk
    {  475, 1384,   57800,  599208,  599242, 27, 34, 36, }, // Noronha
    {  475, 1392,       0,  599278,       0,  0, 31,  0, }, // North_Mariana
    {  475, 1406,   57936,  599309,  599335, 19, 26, 28, }, // Novosibirsk
    {  475, 1418,  599363,  599375,  599394, 12, 19, 21, }, // Omsk
    {  475, 1428,  599415,  599432,  599456, 17, 24, 26, }, // Pakistan
    {  475, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {  475, 1443,       0,  599482,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  475, 1460,  599506,  599522,  599545, 16, 23, 25, }, // Paraguay
    {  475, 1469,  599570,  599582,  599601, 12, 19, 21, }, // Peru
    {  475, 1474,  599622,  599640,  599665, 18, 25, 27, }, // Philippines
    {  475, 1486,       0,  599692,       0,  0, 20,  0, }, // Phoenix_Islands
    {  475, 1502,  599712,  599740,  599775, 28, 35, 37, }, // Pierre_Miquelon
    {  475, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {  475, 1527,       0,   58521,       0,  0, 14,  0, }, // Ponape
    {  475, 1534,       0,  138941,       0,  0, 17,  0, }, // Pyongyang
    {  475, 1544,   58552,  599812,  599836, 17, 24, 26, }, // Qyzylorda
    {  475, 1554,       0,  599862,       0,  0, 15,  0, }, // Reunion
    {  475, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {  475, 1570,  599877,  599893,  599916, 16, 23, 25, }, // Sakhalin
    {  475, 1579,   58714,  599941,  599962, 14, 21, 23, }, // Samara
    {  475, 1596,   58774,  599985,  600005, 13, 20, 22, }, // Samoa
    {  475, 1602,       0,  600027,       0,  0, 18,  0, }, // Seychelles
    {  475, 1624,       0,  600045,       0,  0, 24,  0, }, // Singapore
    {  475, 1634,       0,  600069,       0,  0, 22,  0, }, // Solomon
    {  475, 1642,       0,  600091,       0,  0, 22,  0, }, // South_Georgia
    {  475, 1656,       0,  600113,       0,  0, 16,  0, }, // Suriname
    {  475, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {  475, 1682,       0,  600129,       0,  0, 13,  0, }, // Tahiti
    {  475, 1689,   58967,  600142,  600162, 13, 20, 22, }, // Taipei
    {  475, 1696,       0,  600184,       0,  0, 19,  0, }, // Tajikistan
    {  475, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {  475, 1732,   59062,  600203,  600223, 13, 20, 22, }, // Tonga
    {  475, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {  475, 1750,  600245,  600267,  600296, 22, 29, 31, }, // Turkmenistan
    {  475, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {  475, 1777,  600327,  600342,  600364, 15, 22, 24, }, // Uruguay
    {  475, 1792,  600388,  600407,  600433, 19, 26, 28, }, // Uzbekistan
    {  475, 1803,  600461,  600476,  600498, 15, 22, 24, }, // Vanuatu
    {  475, 1811,       0,  600522,       0,  0, 17,  0, }, // Venezuela
    {  475, 1821,   59451,  600539,  600565, 19, 26, 28, }, // Vladivostok
    {  475, 1833,  600593,  600611,  600636, 18, 25, 27, }, // Volgograd
    {  475, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {  475, 1850,       0,  600663,       0,  0, 17,  0, }, // Wake
    {  475, 1855,       0,  600680,       0,  0, 23,  0, }, // Wallis
    {  475, 1862,   59653,  600703,  600725, 15, 22, 24, }, // Yakutsk
    {  475, 1870,  600749,  600770,  600798, 21, 28, 30, }, // Yekaterinburg
    {  475, 1892,       0,  600828,       0,  0, 13,  0, }, // Yukon
    {  476,    0,  593895,  593907,  593926, 12, 19, 21, }, // Acre Portuguese/Latin/Mozambique
    {  476,    5,       0,  593947,       0,  0, 19,  0, }, // Afghanistan
    {  476,   17,       0,  593966,       0,  0, 22,  0, }, // Africa_Central
    {  476,   32,       0,  593988,       0,  0, 23,  0, }, // Africa_Eastern
    {  476,   65,       0,  594011,       0,  0, 21,  0, }, // Africa_Southern
    {  476,   81,  594032,  594056,  594087, 24, 31, 33, }, // Africa_Western
    {  476,  107,  594120,  594134,  594155, 14, 21, 23, }, // Alaska
    {  476,  128,  594178,  594192,  594213, 14, 21, 23, }, // Almaty
    {  476,  135,  594236,  594252,  594275, 16, 23, 25, }, // Amazon
    {  476,  142,  594300,  594328,  594363, 28, 35, 37, }, // America_Central
    {  476,  158,  594400,  594429,  594465, 29, 36, 38, }, // America_Eastern
    {  476,  174,  594503,  594535,  594574, 32, 39, 41, }, // America_Mountain
    {  476,  191,  594615,  594647,  594686, 32, 39, 41, }, // America_Pacific
    {  476,  207,  594727,  594741,  594762, 14, 21, 23, }, // Anadyr
    {  476,  214,  594785,  594797,  594816, 12, 19, 21, }, // Apia
    {  476,  219,  594837,  594850,  594870, 13, 20, 22, }, // Aqtau
    {  476,  225,  594892,  594906,  594927, 14, 21, 23, }, // Aqtobe
    {  476,  232,  594950,  594964,  594985, 14, 21, 23, }, // Arabian
    {  476,  240,  595008,  595025,  595049, 17, 24, 26, }, // Argentina
    {  476,  250,  595075,  595102,  595136, 27, 34, 36, }, // Argentina_Western
    {  476,  268,  595172,  595187,  595209, 15, 22, 24, }, // Armenia
    {  476,  286,  595233,  595250,  595274, 17, 24, 26, }, // Atlantic
    {  476,  295,  595300,  595325,  595357, 25, 32, 34, }, // Australia_Central
    {  476,  313,  595391,  595426,  595468, 35, 42, 44, }, // Australia_CentralWestern
    {  476,  338,  595512,  595538,  595571, 26, 33, 35, }, // Australia_Eastern
    {  476,  356,  595606,  595633,  595667, 27, 34, 36, }, // Australia_Western
    {  476,  374,  595703,  595721,  595746, 18, 25, 27, }, // Azerbaijan
    {  476,  385,  595773,  595788,  595810, 15, 22, 24, }, // Azores
    {  476,  397,  595834,  595853,  595879, 19, 26, 28, }, // Bangladesh
    {  476,  415,       0,  595907,       0,  0, 13,  0, }, // Bhutan
    {  476,  422,       0,  595920,       0,  0, 15,  0, }, // Bolivia
    {  476,  437,  135014,  595935,  595958, 16, 23, 25, }, // Brasilia
    {  476,  454,       0,  595983,       0,  0, 25,  0, }, // Brunei
    {  476,  461,  596008,  596026,  596051, 18, 25, 27, }, // Cape_Verde
    {  476,  478,       0,  596078,       0,  0, 23,  0, }, // Chamorro
    {  476,  487,   54300,  596101,  596123, 15, 22, 24, }, // Chatham
    {  476,  495,  596147,  596160,  596180, 13, 20, 22, }, // Chile
    {  476,  501,  596202,  596215,  596235, 13, 20, 22, }, // China
    {  476,  507,   54477,  596257,  596282, 18, 25, 27, }, // Choibalsan
    {  476,  518,       0,  596309,       0,  0, 21,  0, }, // Christmas
    {  476,  528,       0,  596330,       0,  0, 20,  0, }, // Cocos
    {  476,  534,  596350,  596366,  596389, 16, 23, 25, }, // Colombia
    {  476,  543,  596414,  596433,  596459, 19, 26, 28, }, // Cook
    {  476,  548,   54767,  596487,  596506, 12, 19, 21, }, // Cuba
    {  476,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {  476,  575,       0,   54834,       0,  0, 24,  0, }, // DumontDUrville
    {  476,  612,       0,  596527,       0,  0, 19,  0, }, // East_Timor
    {  476,  623,  596546,  596568,  596597, 22, 29, 31, }, // Easter
    {  476,  630,       0,  596628,       0,  0, 15,  0, }, // Ecuador
    {  476,  638,  596643,  596665,  596694, 22, 29, 31, }, // Europe_Central
    {  476,  653,  596725,  596748,  596778, 23, 30, 32, }, // Europe_Eastern
    {  476,  668,       0,  596810,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  476,  691,  596841,  596865,  596896, 24, 31, 33, }, // Europe_Western
    {  476,  706,  596929,  596952,  596982, 23, 30, 32, }, // Falkland
    {  476,  715,  136173,  597014,  597033, 12, 19, 21, }, // Fiji
    {  476,  720,       0,  597054,       0,  0, 23,  0, }, // French_Guiana
    {  476,  734,       0,  597077,       0,  0, 48,  0, }, // French_Southern
    {  476,  757,       0,  597125,       0,  0, 18,  0, }, // Galapagos
    {  476,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {  476,  775,  597143,  597158,  597180, 15, 22, 24, }, // Georgia
    {  476,  783,       0,  597204,       0,  0, 22,  0, }, // Gilbert_Islands
    {  476,  799,       0,  597226,       0,  0, 17,  0, }, // GMT
    {  476,  841,  597243,  597271,  597306, 28, 35, 37, }, // Greenland_Eastern
    {  476,  859,  597343,  597372,  597408, 29, 36, 38, }, // Greenland_Western
    {  476,  877,       0,  597446,       0,  0, 19,  0, }, // Guam
    {  476,  882,       0,  597465,       0,  0, 20,  0, }, // Gulf
    {  476,  887,       0,  597054,       0,  0, 14,  0, }, // Guyana
    {  476,  894,  597485,  597508,  597538, 23, 30, 32, }, // Hawaii_Aleutian
    {  476,  910,  136716,  597570,  597594, 17, 24, 26, }, // Hong_Kong
    {  476,  920,   56045,  597620,  597639, 12, 19, 21, }, // Hovd
    {  476,  925,       0,  597660,       0,  0, 20,  0, }, // India
    {  476,  931,       0,  597680,       0,  0, 21,  0, }, // Indian_Ocean
    {  476,  944,       0,  597701,       0,  0, 17,  0, }, // Indochina
    {  476,  954,       0,  597718,       0,  0, 25,  0, }, // Indonesia_Central
    {  476,  972,       0,  597743,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  476,  990,       0,  597769,       0,  0, 27,  0, }, // Indonesia_Western
    {  476, 1008,  597796,  597808,  597827, 12, 19, 21, }, // Iran
    {  476, 1019,  597848,  597863,  597885, 15, 22, 24, }, // Irkutsk
    {  476, 1027,  597909,  597923,  597944, 14, 21, 23, }, // Israel
    {  476, 1034,  597967,  597980,  598000, 13, 20, 22, }, // Japan
    {  476, 1040,  598022,  598054,  598093, 32, 39, 41, }, // Kamchatka
    {  476, 1058,       0,  598134,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  476, 1077,       0,  598162,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  476, 1106,  598191,  598205,  598226, 14, 21, 23, }, // Korea
    {  476, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {  476, 1119,   56703,  598249,  598275, 19, 26, 28, }, // Krasnoyarsk
    {  476, 1151,       0,  598303,       0,  0, 19,  0, }, // Kyrgystan
    {  476, 1161,       0,  598322,       0,  0, 17,  0, }, // Lanka
    {  476, 1175,       0,  598339,       0,  0, 19,  0, }, // Line_Islands
    {  476, 1188,   56833,  598358,  598382, 17, 24, 26, }, // Lord_Howe
    {  476, 1198,  137554,  598408,  598428, 13, 20, 22, }, // Macau
    {  476, 1204,  137611,  598450,  598472, 15, 22, 24, }, // Magadan
    {  476, 1219,       0,  598496,       0,  0, 15,  0, }, // Malaysia
    {  476, 1228,       0,  598511,       0,  0, 17,  0, }, // Maldives
    {  476, 1237,       0,  598528,       0,  0, 24,  0, }, // Marquesas
    {  476, 1247,       0,  598552,       0,  0, 23,  0, }, // Marshall_Islands
    {  476, 1264,  598575,  598591,  598614, 16, 23, 25, }, // Mauritius
    {  476, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {  476, 1281,  598639,  598664,  598696, 25, 32, 34, }, // Mexico_Pacific
    {  476, 1296,  598730,  598748,  598773, 18, 25, 27, }, // Mongolia
    {  476, 1305,  598800,  598815,  598837, 15, 22, 24, }, // Moscow
    {  476, 1312,       0,  598861,       0,  0, 15,  0, }, // Myanmar
    {  476, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {  476, 1326,       0,  598876,       0,  0, 13,  0, }, // Nepal
    {  476, 1332,  598889,  598911,  598940, 22, 29, 31, }, // New_Caledonia
    {  476, 1346,  598971,  598992,  599020, 21, 28, 30, }, // New_Zealand
    {  476, 1358,  599050,  599068,  599093, 18, 25, 27, }, // Newfoundland
    {  476, 1371,       0,  599120,       0,  0, 12,  0, }, // Niue
    {  476, 1376,  599132,  599152,  599179, 20, 27, 29, }, // Norfolk
    {  476, 1384,   57800,  599208,  599242, 27, 34, 36, }, // Noronha
    {  476, 1392,       0,  599278,       0,  0, 31,  0, }, // North_Mariana
    {  476, 1406,   57936,  599309,  599335, 19, 26, 28, }, // Novosibirsk
    {  476, 1418,  599363,  599375,  599394, 12, 19, 21, }, // Omsk
    {  476, 1428,  599415,  599432,  599456, 17, 24, 26, }, // Pakistan
    {  476, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {  476, 1443,       0,  599482,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  476, 1460,  599506,  599522,  599545, 16, 23, 25, }, // Paraguay
    {  476, 1469,  599570,  599582,  599601, 12, 19, 21, }, // Peru
    {  476, 1474,  599622,  599640,  599665, 18, 25, 27, }, // Philippines
    {  476, 1486,       0,  599692,       0,  0, 20,  0, }, // Phoenix_Islands
    {  476, 1502,  599712,  599740,  599775, 28, 35, 37, }, // Pierre_Miquelon
    {  476, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {  476, 1527,       0,   58521,       0,  0, 14,  0, }, // Ponape
    {  476, 1534,       0,  138941,       0,  0, 17,  0, }, // Pyongyang
    {  476, 1544,   58552,  599812,  599836, 17, 24, 26, }, // Qyzylorda
    {  476, 1554,       0,  599862,       0,  0, 15,  0, }, // Reunion
    {  476, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {  476, 1570,  599877,  599893,  599916, 16, 23, 25, }, // Sakhalin
    {  476, 1579,   58714,  599941,  599962, 14, 21, 23, }, // Samara
    {  476, 1596,   58774,  599985,  600005, 13, 20, 22, }, // Samoa
    {  476, 1602,       0,  600027,       0,  0, 18,  0, }, // Seychelles
    {  476, 1624,       0,  600045,       0,  0, 24,  0, }, // Singapore
    {  476, 1634,       0,  600069,       0,  0, 22,  0, }, // Solomon
    {  476, 1642,       0,  600091,       0,  0, 22,  0, }, // South_Georgia
    {  476, 1656,       0,  600113,       0,  0, 16,  0, }, // Suriname
    {  476, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {  476, 1682,       0,  600129,       0,  0, 13,  0, }, // Tahiti
    {  476, 1689,   58967,  600142,  600162, 13, 20, 22, }, // Taipei
    {  476, 1696,       0,  600184,       0,  0, 19,  0, }, // Tajikistan
    {  476, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {  476, 1732,   59062,  600203,  600223, 13, 20, 22, }, // Tonga
    {  476, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {  476, 1750,  600245,  600267,  600296, 22, 29, 31, }, // Turkmenistan
    {  476, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {  476, 1777,  600327,  600342,  600364, 15, 22, 24, }, // Uruguay
    {  476, 1792,  600388,  600407,  600433, 19, 26, 28, }, // Uzbekistan
    {  476, 1803,  600461,  600476,  600498, 15, 22, 24, }, // Vanuatu
    {  476, 1811,       0,  600522,       0,  0, 17,  0, }, // Venezuela
    {  476, 1821,   59451,  600539,  600565, 19, 26, 28, }, // Vladivostok
    {  476, 1833,  600593,  600611,  600636, 18, 25, 27, }, // Volgograd
    {  476, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {  476, 1850,       0,  600663,       0,  0, 17,  0, }, // Wake
    {  476, 1855,       0,  600680,       0,  0, 23,  0, }, // Wallis
    {  476, 1862,   59653,  600703,  600725, 15, 22, 24, }, // Yakutsk
    {  476, 1870,  600749,  600770,  600798, 21, 28, 30, }, // Yekaterinburg
    {  476, 1892,       0,  600828,       0,  0, 13,  0, }, // Yukon
    {  477,    0,  593895,  593907,  593926, 12, 19, 21, }, // Acre Portuguese/Latin/Portugal
    {  477,    5,       0,  593947,       0,  0, 19,  0, }, // Afghanistan
    {  477,   17,       0,  593966,       0,  0, 22,  0, }, // Africa_Central
    {  477,   32,       0,  593988,       0,  0, 23,  0, }, // Africa_Eastern
    {  477,   65,       0,  594011,       0,  0, 21,  0, }, // Africa_Southern
    {  477,   81,  594032,  594056,  594087, 24, 31, 33, }, // Africa_Western
    {  477,  107,  594120,  594134,  594155, 14, 21, 23, }, // Alaska
    {  477,  128,  594178,  594192,  594213, 14, 21, 23, }, // Almaty
    {  477,  135,  594236,  594252,  594275, 16, 23, 25, }, // Amazon
    {  477,  142,  594300,  594328,  594363, 28, 35, 37, }, // America_Central
    {  477,  158,  594400,  594429,  594465, 29, 36, 38, }, // America_Eastern
    {  477,  174,  594503,  594535,  594574, 32, 39, 41, }, // America_Mountain
    {  477,  191,  594615,  594647,  594686, 32, 39, 41, }, // America_Pacific
    {  477,  207,  594727,  594741,  594762, 14, 21, 23, }, // Anadyr
    {  477,  214,  594785,  594797,  594816, 12, 19, 21, }, // Apia
    {  477,  219,  594837,  594850,  594870, 13, 20, 22, }, // Aqtau
    {  477,  225,  594892,  594906,  594927, 14, 21, 23, }, // Aqtobe
    {  477,  232,  594950,  594964,  594985, 14, 21, 23, }, // Arabian
    {  477,  240,  595008,  595025,  595049, 17, 24, 26, }, // Argentina
    {  477,  250,  595075,  595102,  595136, 27, 34, 36, }, // Argentina_Western
    {  477,  268,  595172,  595187,  595209, 15, 22, 24, }, // Armenia
    {  477,  286,  595233,  595250,  595274, 17, 24, 26, }, // Atlantic
    {  477,  295,  595300,  595325,  595357, 25, 32, 34, }, // Australia_Central
    {  477,  313,  595391,  595426,  595468, 35, 42, 44, }, // Australia_CentralWestern
    {  477,  338,  595512,  595538,  595571, 26, 33, 35, }, // Australia_Eastern
    {  477,  356,  595606,  595633,  595667, 27, 34, 36, }, // Australia_Western
    {  477,  374,  595703,  595721,  595746, 18, 25, 27, }, // Azerbaijan
    {  477,  385,  595773,  595788,  595810, 15, 22, 24, }, // Azores
    {  477,  397,  595834,  595853,  595879, 19, 26, 28, }, // Bangladesh
    {  477,  415,       0,  595907,       0,  0, 13,  0, }, // Bhutan
    {  477,  422,       0,  595920,       0,  0, 15,  0, }, // Bolivia
    {  477,  437,  135014,  595935,  595958, 16, 23, 25, }, // Brasilia
    {  477,  454,       0,  595983,       0,  0, 25,  0, }, // Brunei
    {  477,  461,  596008,  596026,  596051, 18, 25, 27, }, // Cape_Verde
    {  477,  478,       0,  596078,       0,  0, 23,  0, }, // Chamorro
    {  477,  487,   54300,  596101,  596123, 15, 22, 24, }, // Chatham
    {  477,  495,  596147,  596160,  596180, 13, 20, 22, }, // Chile
    {  477,  501,  596202,  596215,  596235, 13, 20, 22, }, // China
    {  477,  507,   54477,  596257,  596282, 18, 25, 27, }, // Choibalsan
    {  477,  518,       0,  596309,       0,  0, 21,  0, }, // Christmas
    {  477,  528,       0,  596330,       0,  0, 20,  0, }, // Cocos
    {  477,  534,  596350,  596366,  596389, 16, 23, 25, }, // Colombia
    {  477,  543,  596414,  596433,  596459, 19, 26, 28, }, // Cook
    {  477,  548,   54767,  596487,  596506, 12, 19, 21, }, // Cuba
    {  477,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {  477,  575,       0,   54834,       0,  0, 24,  0, }, // DumontDUrville
    {  477,  612,       0,  596527,       0,  0, 19,  0, }, // East_Timor
    {  477,  623,  596546,  596568,  596597, 22, 29, 31, }, // Easter
    {  477,  630,       0,  596628,       0,  0, 15,  0, }, // Ecuador
    {  477,  638,  596643,  596665,  596694, 22, 29, 31, }, // Europe_Central
    {  477,  653,  596725,  596748,  596778, 23, 30, 32, }, // Europe_Eastern
    {  477,  668,       0,  596810,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  477,  691,  596841,  596865,  596896, 24, 31, 33, }, // Europe_Western
    {  477,  706,  596929,  596952,  596982, 23, 30, 32, }, // Falkland
    {  477,  715,  136173,  597014,  597033, 12, 19, 21, }, // Fiji
    {  477,  720,       0,  597054,       0,  0, 23,  0, }, // French_Guiana
    {  477,  734,       0,  597077,       0,  0, 48,  0, }, // French_Southern
    {  477,  757,       0,  597125,       0,  0, 18,  0, }, // Galapagos
    {  477,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {  477,  775,  597143,  597158,  597180, 15, 22, 24, }, // Georgia
    {  477,  783,       0,  597204,       0,  0, 22,  0, }, // Gilbert_Islands
    {  477,  799,       0,  597226,       0,  0, 17,  0, }, // GMT
    {  477,  841,  597243,  597271,  597306, 28, 35, 37, }, // Greenland_Eastern
    {  477,  859,  597343,  597372,  597408, 29, 36, 38, }, // Greenland_Western
    {  477,  877,       0,  597446,       0,  0, 19,  0, }, // Guam
    {  477,  882,       0,  597465,       0,  0, 20,  0, }, // Gulf
    {  477,  887,       0,  597054,       0,  0, 14,  0, }, // Guyana
    {  477,  894,  597485,  597508,  597538, 23, 30, 32, }, // Hawaii_Aleutian
    {  477,  910,  136716,  597570,  597594, 17, 24, 26, }, // Hong_Kong
    {  477,  920,   56045,  597620,  597639, 12, 19, 21, }, // Hovd
    {  477,  925,       0,  597660,       0,  0, 20,  0, }, // India
    {  477,  931,       0,  597680,       0,  0, 21,  0, }, // Indian_Ocean
    {  477,  944,       0,  597701,       0,  0, 17,  0, }, // Indochina
    {  477,  954,       0,  597718,       0,  0, 25,  0, }, // Indonesia_Central
    {  477,  972,       0,  597743,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  477,  990,       0,  597769,       0,  0, 27,  0, }, // Indonesia_Western
    {  477, 1008,  597796,  597808,  597827, 12, 19, 21, }, // Iran
    {  477, 1019,  597848,  597863,  597885, 15, 22, 24, }, // Irkutsk
    {  477, 1027,  597909,  597923,  597944, 14, 21, 23, }, // Israel
    {  477, 1034,  597967,  597980,  598000, 13, 20, 22, }, // Japan
    {  477, 1040,  598022,  598054,  598093, 32, 39, 41, }, // Kamchatka
    {  477, 1058,       0,  598134,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  477, 1077,       0,  598162,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  477, 1106,  598191,  598205,  598226, 14, 21, 23, }, // Korea
    {  477, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {  477, 1119,   56703,  598249,  598275, 19, 26, 28, }, // Krasnoyarsk
    {  477, 1151,       0,  598303,       0,  0, 19,  0, }, // Kyrgystan
    {  477, 1161,       0,  598322,       0,  0, 17,  0, }, // Lanka
    {  477, 1175,       0,  598339,       0,  0, 19,  0, }, // Line_Islands
    {  477, 1188,   56833,  598358,  598382, 17, 24, 26, }, // Lord_Howe
    {  477, 1198,  137554,  598408,  598428, 13, 20, 22, }, // Macau
    {  477, 1204,  137611,  598450,  598472, 15, 22, 24, }, // Magadan
    {  477, 1219,       0,  598496,       0,  0, 15,  0, }, // Malaysia
    {  477, 1228,       0,  598511,       0,  0, 17,  0, }, // Maldives
    {  477, 1237,       0,  598528,       0,  0, 24,  0, }, // Marquesas
    {  477, 1247,       0,  598552,       0,  0, 23,  0, }, // Marshall_Islands
    {  477, 1264,  598575,  598591,  598614, 16, 23, 25, }, // Mauritius
    {  477, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {  477, 1281,  598639,  598664,  598696, 25, 32, 34, }, // Mexico_Pacific
    {  477, 1296,  598730,  598748,  598773, 18, 25, 27, }, // Mongolia
    {  477, 1305,  598800,  598815,  598837, 15, 22, 24, }, // Moscow
    {  477, 1312,       0,  598861,       0,  0, 15,  0, }, // Myanmar
    {  477, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {  477, 1326,       0,  598876,       0,  0, 13,  0, }, // Nepal
    {  477, 1332,  598889,  598911,  598940, 22, 29, 31, }, // New_Caledonia
    {  477, 1346,  598971,  598992,  599020, 21, 28, 30, }, // New_Zealand
    {  477, 1358,  599050,  599068,  599093, 18, 25, 27, }, // Newfoundland
    {  477, 1371,       0,  599120,       0,  0, 12,  0, }, // Niue
    {  477, 1376,  599132,  599152,  599179, 20, 27, 29, }, // Norfolk
    {  477, 1384,   57800,  599208,  599242, 27, 34, 36, }, // Noronha
    {  477, 1392,       0,  599278,       0,  0, 31,  0, }, // North_Mariana
    {  477, 1406,   57936,  599309,  599335, 19, 26, 28, }, // Novosibirsk
    {  477, 1418,  599363,  599375,  599394, 12, 19, 21, }, // Omsk
    {  477, 1428,  599415,  599432,  599456, 17, 24, 26, }, // Pakistan
    {  477, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {  477, 1443,       0,  599482,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  477, 1460,  599506,  599522,  599545, 16, 23, 25, }, // Paraguay
    {  477, 1469,  599570,  599582,  599601, 12, 19, 21, }, // Peru
    {  477, 1474,  599622,  599640,  599665, 18, 25, 27, }, // Philippines
    {  477, 1486,       0,  599692,       0,  0, 20,  0, }, // Phoenix_Islands
    {  477, 1502,  599712,  599740,  599775, 28, 35, 37, }, // Pierre_Miquelon
    {  477, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {  477, 1527,       0,   58521,       0,  0, 14,  0, }, // Ponape
    {  477, 1534,       0,  138941,       0,  0, 17,  0, }, // Pyongyang
    {  477, 1544,   58552,  599812,  599836, 17, 24, 26, }, // Qyzylorda
    {  477, 1554,       0,  599862,       0,  0, 15,  0, }, // Reunion
    {  477, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {  477, 1570,  599877,  599893,  599916, 16, 23, 25, }, // Sakhalin
    {  477, 1579,   58714,  599941,  599962, 14, 21, 23, }, // Samara
    {  477, 1596,   58774,  599985,  600005, 13, 20, 22, }, // Samoa
    {  477, 1602,       0,  600027,       0,  0, 18,  0, }, // Seychelles
    {  477, 1624,       0,  600045,       0,  0, 24,  0, }, // Singapore
    {  477, 1634,       0,  600069,       0,  0, 22,  0, }, // Solomon
    {  477, 1642,       0,  600091,       0,  0, 22,  0, }, // South_Georgia
    {  477, 1656,       0,  600113,       0,  0, 16,  0, }, // Suriname
    {  477, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {  477, 1682,       0,  600129,       0,  0, 13,  0, }, // Tahiti
    {  477, 1689,   58967,  600142,  600162, 13, 20, 22, }, // Taipei
    {  477, 1696,       0,  600184,       0,  0, 19,  0, }, // Tajikistan
    {  477, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {  477, 1732,   59062,  600203,  600223, 13, 20, 22, }, // Tonga
    {  477, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {  477, 1750,  600245,  600267,  600296, 22, 29, 31, }, // Turkmenistan
    {  477, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {  477, 1777,  600327,  600342,  600364, 15, 22, 24, }, // Uruguay
    {  477, 1792,  600388,  600407,  600433, 19, 26, 28, }, // Uzbekistan
    {  477, 1803,  600461,  600476,  600498, 15, 22, 24, }, // Vanuatu
    {  477, 1811,       0,  600522,       0,  0, 17,  0, }, // Venezuela
    {  477, 1821,   59451,  600539,  600565, 19, 26, 28, }, // Vladivostok
    {  477, 1833,  600593,  600611,  600636, 18, 25, 27, }, // Volgograd
    {  477, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {  477, 1850,       0,  600663,       0,  0, 17,  0, }, // Wake
    {  477, 1855,       0,  600680,       0,  0, 23,  0, }, // Wallis
    {  477, 1862,   59653,  600703,  600725, 15, 22, 24, }, // Yakutsk
    {  477, 1870,  600749,  600770,  600798, 21, 28, 30, }, // Yekaterinburg
    {  477, 1892,       0,  600828,       0,  0, 13,  0, }, // Yukon
    {  478,    0,  593895,  593907,  593926, 12, 19, 21, }, // Acre Portuguese/Latin/Sao Tome and Principe
    {  478,    5,       0,  593947,       0,  0, 19,  0, }, // Afghanistan
    {  478,   17,       0,  593966,       0,  0, 22,  0, }, // Africa_Central
    {  478,   32,       0,  593988,       0,  0, 23,  0, }, // Africa_Eastern
    {  478,   65,       0,  594011,       0,  0, 21,  0, }, // Africa_Southern
    {  478,   81,  594032,  594056,  594087, 24, 31, 33, }, // Africa_Western
    {  478,  107,  594120,  594134,  594155, 14, 21, 23, }, // Alaska
    {  478,  128,  594178,  594192,  594213, 14, 21, 23, }, // Almaty
    {  478,  135,  594236,  594252,  594275, 16, 23, 25, }, // Amazon
    {  478,  142,  594300,  594328,  594363, 28, 35, 37, }, // America_Central
    {  478,  158,  594400,  594429,  594465, 29, 36, 38, }, // America_Eastern
    {  478,  174,  594503,  594535,  594574, 32, 39, 41, }, // America_Mountain
    {  478,  191,  594615,  594647,  594686, 32, 39, 41, }, // America_Pacific
    {  478,  207,  594727,  594741,  594762, 14, 21, 23, }, // Anadyr
    {  478,  214,  594785,  594797,  594816, 12, 19, 21, }, // Apia
    {  478,  219,  594837,  594850,  594870, 13, 20, 22, }, // Aqtau
    {  478,  225,  594892,  594906,  594927, 14, 21, 23, }, // Aqtobe
    {  478,  232,  594950,  594964,  594985, 14, 21, 23, }, // Arabian
    {  478,  240,  595008,  595025,  595049, 17, 24, 26, }, // Argentina
    {  478,  250,  595075,  595102,  595136, 27, 34, 36, }, // Argentina_Western
    {  478,  268,  595172,  595187,  595209, 15, 22, 24, }, // Armenia
    {  478,  286,  595233,  595250,  595274, 17, 24, 26, }, // Atlantic
    {  478,  295,  595300,  595325,  595357, 25, 32, 34, }, // Australia_Central
    {  478,  313,  595391,  595426,  595468, 35, 42, 44, }, // Australia_CentralWestern
    {  478,  338,  595512,  595538,  595571, 26, 33, 35, }, // Australia_Eastern
    {  478,  356,  595606,  595633,  595667, 27, 34, 36, }, // Australia_Western
    {  478,  374,  595703,  595721,  595746, 18, 25, 27, }, // Azerbaijan
    {  478,  385,  595773,  595788,  595810, 15, 22, 24, }, // Azores
    {  478,  397,  595834,  595853,  595879, 19, 26, 28, }, // Bangladesh
    {  478,  415,       0,  595907,       0,  0, 13,  0, }, // Bhutan
    {  478,  422,       0,  595920,       0,  0, 15,  0, }, // Bolivia
    {  478,  437,  135014,  595935,  595958, 16, 23, 25, }, // Brasilia
    {  478,  454,       0,  595983,       0,  0, 25,  0, }, // Brunei
    {  478,  461,  596008,  596026,  596051, 18, 25, 27, }, // Cape_Verde
    {  478,  478,       0,  596078,       0,  0, 23,  0, }, // Chamorro
    {  478,  487,   54300,  596101,  596123, 15, 22, 24, }, // Chatham
    {  478,  495,  596147,  596160,  596180, 13, 20, 22, }, // Chile
    {  478,  501,  596202,  596215,  596235, 13, 20, 22, }, // China
    {  478,  507,   54477,  596257,  596282, 18, 25, 27, }, // Choibalsan
    {  478,  518,       0,  596309,       0,  0, 21,  0, }, // Christmas
    {  478,  528,       0,  596330,       0,  0, 20,  0, }, // Cocos
    {  478,  534,  596350,  596366,  596389, 16, 23, 25, }, // Colombia
    {  478,  543,  596414,  596433,  596459, 19, 26, 28, }, // Cook
    {  478,  548,   54767,  596487,  596506, 12, 19, 21, }, // Cuba
    {  478,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {  478,  575,       0,   54834,       0,  0, 24,  0, }, // DumontDUrville
    {  478,  612,       0,  596527,       0,  0, 19,  0, }, // East_Timor
    {  478,  623,  596546,  596568,  596597, 22, 29, 31, }, // Easter
    {  478,  630,       0,  596628,       0,  0, 15,  0, }, // Ecuador
    {  478,  638,  596643,  596665,  596694, 22, 29, 31, }, // Europe_Central
    {  478,  653,  596725,  596748,  596778, 23, 30, 32, }, // Europe_Eastern
    {  478,  668,       0,  596810,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  478,  691,  596841,  596865,  596896, 24, 31, 33, }, // Europe_Western
    {  478,  706,  596929,  596952,  596982, 23, 30, 32, }, // Falkland
    {  478,  715,  136173,  597014,  597033, 12, 19, 21, }, // Fiji
    {  478,  720,       0,  597054,       0,  0, 23,  0, }, // French_Guiana
    {  478,  734,       0,  597077,       0,  0, 48,  0, }, // French_Southern
    {  478,  757,       0,  597125,       0,  0, 18,  0, }, // Galapagos
    {  478,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {  478,  775,  597143,  597158,  597180, 15, 22, 24, }, // Georgia
    {  478,  783,       0,  597204,       0,  0, 22,  0, }, // Gilbert_Islands
    {  478,  799,       0,  597226,       0,  0, 17,  0, }, // GMT
    {  478,  841,  597243,  597271,  597306, 28, 35, 37, }, // Greenland_Eastern
    {  478,  859,  597343,  597372,  597408, 29, 36, 38, }, // Greenland_Western
    {  478,  877,       0,  597446,       0,  0, 19,  0, }, // Guam
    {  478,  882,       0,  597465,       0,  0, 20,  0, }, // Gulf
    {  478,  887,       0,  597054,       0,  0, 14,  0, }, // Guyana
    {  478,  894,  597485,  597508,  597538, 23, 30, 32, }, // Hawaii_Aleutian
    {  478,  910,  136716,  597570,  597594, 17, 24, 26, }, // Hong_Kong
    {  478,  920,   56045,  597620,  597639, 12, 19, 21, }, // Hovd
    {  478,  925,       0,  597660,       0,  0, 20,  0, }, // India
    {  478,  931,       0,  597680,       0,  0, 21,  0, }, // Indian_Ocean
    {  478,  944,       0,  597701,       0,  0, 17,  0, }, // Indochina
    {  478,  954,       0,  597718,       0,  0, 25,  0, }, // Indonesia_Central
    {  478,  972,       0,  597743,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  478,  990,       0,  597769,       0,  0, 27,  0, }, // Indonesia_Western
    {  478, 1008,  597796,  597808,  597827, 12, 19, 21, }, // Iran
    {  478, 1019,  597848,  597863,  597885, 15, 22, 24, }, // Irkutsk
    {  478, 1027,  597909,  597923,  597944, 14, 21, 23, }, // Israel
    {  478, 1034,  597967,  597980,  598000, 13, 20, 22, }, // Japan
    {  478, 1040,  598022,  598054,  598093, 32, 39, 41, }, // Kamchatka
    {  478, 1058,       0,  598134,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  478, 1077,       0,  598162,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  478, 1106,  598191,  598205,  598226, 14, 21, 23, }, // Korea
    {  478, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {  478, 1119,   56703,  598249,  598275, 19, 26, 28, }, // Krasnoyarsk
    {  478, 1151,       0,  598303,       0,  0, 19,  0, }, // Kyrgystan
    {  478, 1161,       0,  598322,       0,  0, 17,  0, }, // Lanka
    {  478, 1175,       0,  598339,       0,  0, 19,  0, }, // Line_Islands
    {  478, 1188,   56833,  598358,  598382, 17, 24, 26, }, // Lord_Howe
    {  478, 1198,  137554,  598408,  598428, 13, 20, 22, }, // Macau
    {  478, 1204,  137611,  598450,  598472, 15, 22, 24, }, // Magadan
    {  478, 1219,       0,  598496,       0,  0, 15,  0, }, // Malaysia
    {  478, 1228,       0,  598511,       0,  0, 17,  0, }, // Maldives
    {  478, 1237,       0,  598528,       0,  0, 24,  0, }, // Marquesas
    {  478, 1247,       0,  598552,       0,  0, 23,  0, }, // Marshall_Islands
    {  478, 1264,  598575,  598591,  598614, 16, 23, 25, }, // Mauritius
    {  478, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {  478, 1281,  598639,  598664,  598696, 25, 32, 34, }, // Mexico_Pacific
    {  478, 1296,  598730,  598748,  598773, 18, 25, 27, }, // Mongolia
    {  478, 1305,  598800,  598815,  598837, 15, 22, 24, }, // Moscow
    {  478, 1312,       0,  598861,       0,  0, 15,  0, }, // Myanmar
    {  478, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {  478, 1326,       0,  598876,       0,  0, 13,  0, }, // Nepal
    {  478, 1332,  598889,  598911,  598940, 22, 29, 31, }, // New_Caledonia
    {  478, 1346,  598971,  598992,  599020, 21, 28, 30, }, // New_Zealand
    {  478, 1358,  599050,  599068,  599093, 18, 25, 27, }, // Newfoundland
    {  478, 1371,       0,  599120,       0,  0, 12,  0, }, // Niue
    {  478, 1376,  599132,  599152,  599179, 20, 27, 29, }, // Norfolk
    {  478, 1384,   57800,  599208,  599242, 27, 34, 36, }, // Noronha
    {  478, 1392,       0,  599278,       0,  0, 31,  0, }, // North_Mariana
    {  478, 1406,   57936,  599309,  599335, 19, 26, 28, }, // Novosibirsk
    {  478, 1418,  599363,  599375,  599394, 12, 19, 21, }, // Omsk
    {  478, 1428,  599415,  599432,  599456, 17, 24, 26, }, // Pakistan
    {  478, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {  478, 1443,       0,  599482,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  478, 1460,  599506,  599522,  599545, 16, 23, 25, }, // Paraguay
    {  478, 1469,  599570,  599582,  599601, 12, 19, 21, }, // Peru
    {  478, 1474,  599622,  599640,  599665, 18, 25, 27, }, // Philippines
    {  478, 1486,       0,  599692,       0,  0, 20,  0, }, // Phoenix_Islands
    {  478, 1502,  599712,  599740,  599775, 28, 35, 37, }, // Pierre_Miquelon
    {  478, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {  478, 1527,       0,   58521,       0,  0, 14,  0, }, // Ponape
    {  478, 1534,       0,  138941,       0,  0, 17,  0, }, // Pyongyang
    {  478, 1544,   58552,  599812,  599836, 17, 24, 26, }, // Qyzylorda
    {  478, 1554,       0,  599862,       0,  0, 15,  0, }, // Reunion
    {  478, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {  478, 1570,  599877,  599893,  599916, 16, 23, 25, }, // Sakhalin
    {  478, 1579,   58714,  599941,  599962, 14, 21, 23, }, // Samara
    {  478, 1596,   58774,  599985,  600005, 13, 20, 22, }, // Samoa
    {  478, 1602,       0,  600027,       0,  0, 18,  0, }, // Seychelles
    {  478, 1624,       0,  600045,       0,  0, 24,  0, }, // Singapore
    {  478, 1634,       0,  600069,       0,  0, 22,  0, }, // Solomon
    {  478, 1642,       0,  600091,       0,  0, 22,  0, }, // South_Georgia
    {  478, 1656,       0,  600113,       0,  0, 16,  0, }, // Suriname
    {  478, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {  478, 1682,       0,  600129,       0,  0, 13,  0, }, // Tahiti
    {  478, 1689,   58967,  600142,  600162, 13, 20, 22, }, // Taipei
    {  478, 1696,       0,  600184,       0,  0, 19,  0, }, // Tajikistan
    {  478, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {  478, 1732,   59062,  600203,  600223, 13, 20, 22, }, // Tonga
    {  478, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {  478, 1750,  600245,  600267,  600296, 22, 29, 31, }, // Turkmenistan
    {  478, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {  478, 1777,  600327,  600342,  600364, 15, 22, 24, }, // Uruguay
    {  478, 1792,  600388,  600407,  600433, 19, 26, 28, }, // Uzbekistan
    {  478, 1803,  600461,  600476,  600498, 15, 22, 24, }, // Vanuatu
    {  478, 1811,       0,  600522,       0,  0, 17,  0, }, // Venezuela
    {  478, 1821,   59451,  600539,  600565, 19, 26, 28, }, // Vladivostok
    {  478, 1833,  600593,  600611,  600636, 18, 25, 27, }, // Volgograd
    {  478, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {  478, 1850,       0,  600663,       0,  0, 17,  0, }, // Wake
    {  478, 1855,       0,  600680,       0,  0, 23,  0, }, // Wallis
    {  478, 1862,   59653,  600703,  600725, 15, 22, 24, }, // Yakutsk
    {  478, 1870,  600749,  600770,  600798, 21, 28, 30, }, // Yekaterinburg
    {  478, 1892,       0,  600828,       0,  0, 13,  0, }, // Yukon
    {  479,    0,  593895,  593907,  593926, 12, 19, 21, }, // Acre Portuguese/Latin/Switzerland
    {  479,    5,       0,  593947,       0,  0, 19,  0, }, // Afghanistan
    {  479,   17,       0,  593966,       0,  0, 22,  0, }, // Africa_Central
    {  479,   32,       0,  593988,       0,  0, 23,  0, }, // Africa_Eastern
    {  479,   65,       0,  594011,       0,  0, 21,  0, }, // Africa_Southern
    {  479,   81,  594032,  594056,  594087, 24, 31, 33, }, // Africa_Western
    {  479,  107,  594120,  594134,  594155, 14, 21, 23, }, // Alaska
    {  479,  128,  594178,  594192,  594213, 14, 21, 23, }, // Almaty
    {  479,  135,  594236,  594252,  594275, 16, 23, 25, }, // Amazon
    {  479,  142,  594300,  594328,  594363, 28, 35, 37, }, // America_Central
    {  479,  158,  594400,  594429,  594465, 29, 36, 38, }, // America_Eastern
    {  479,  174,  594503,  594535,  594574, 32, 39, 41, }, // America_Mountain
    {  479,  191,  594615,  594647,  594686, 32, 39, 41, }, // America_Pacific
    {  479,  207,  594727,  594741,  594762, 14, 21, 23, }, // Anadyr
    {  479,  214,  594785,  594797,  594816, 12, 19, 21, }, // Apia
    {  479,  219,  594837,  594850,  594870, 13, 20, 22, }, // Aqtau
    {  479,  225,  594892,  594906,  594927, 14, 21, 23, }, // Aqtobe
    {  479,  232,  594950,  594964,  594985, 14, 21, 23, }, // Arabian
    {  479,  240,  595008,  595025,  595049, 17, 24, 26, }, // Argentina
    {  479,  250,  595075,  595102,  595136, 27, 34, 36, }, // Argentina_Western
    {  479,  268,  595172,  595187,  595209, 15, 22, 24, }, // Armenia
    {  479,  286,  595233,  595250,  595274, 17, 24, 26, }, // Atlantic
    {  479,  295,  595300,  595325,  595357, 25, 32, 34, }, // Australia_Central
    {  479,  313,  595391,  595426,  595468, 35, 42, 44, }, // Australia_CentralWestern
    {  479,  338,  595512,  595538,  595571, 26, 33, 35, }, // Australia_Eastern
    {  479,  356,  595606,  595633,  595667, 27, 34, 36, }, // Australia_Western
    {  479,  374,  595703,  595721,  595746, 18, 25, 27, }, // Azerbaijan
    {  479,  385,  595773,  595788,  595810, 15, 22, 24, }, // Azores
    {  479,  397,  595834,  595853,  595879, 19, 26, 28, }, // Bangladesh
    {  479,  415,       0,  595907,       0,  0, 13,  0, }, // Bhutan
    {  479,  422,       0,  595920,       0,  0, 15,  0, }, // Bolivia
    {  479,  437,  135014,  595935,  595958, 16, 23, 25, }, // Brasilia
    {  479,  454,       0,  595983,       0,  0, 25,  0, }, // Brunei
    {  479,  461,  596008,  596026,  596051, 18, 25, 27, }, // Cape_Verde
    {  479,  478,       0,  596078,       0,  0, 23,  0, }, // Chamorro
    {  479,  487,   54300,  596101,  596123, 15, 22, 24, }, // Chatham
    {  479,  495,  596147,  596160,  596180, 13, 20, 22, }, // Chile
    {  479,  501,  596202,  596215,  596235, 13, 20, 22, }, // China
    {  479,  507,   54477,  596257,  596282, 18, 25, 27, }, // Choibalsan
    {  479,  518,       0,  596309,       0,  0, 21,  0, }, // Christmas
    {  479,  528,       0,  596330,       0,  0, 20,  0, }, // Cocos
    {  479,  534,  596350,  596366,  596389, 16, 23, 25, }, // Colombia
    {  479,  543,  596414,  596433,  596459, 19, 26, 28, }, // Cook
    {  479,  548,   54767,  596487,  596506, 12, 19, 21, }, // Cuba
    {  479,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {  479,  575,       0,   54834,       0,  0, 24,  0, }, // DumontDUrville
    {  479,  612,       0,  596527,       0,  0, 19,  0, }, // East_Timor
    {  479,  623,  596546,  596568,  596597, 22, 29, 31, }, // Easter
    {  479,  630,       0,  596628,       0,  0, 15,  0, }, // Ecuador
    {  479,  638,  596643,  596665,  596694, 22, 29, 31, }, // Europe_Central
    {  479,  653,  596725,  596748,  596778, 23, 30, 32, }, // Europe_Eastern
    {  479,  668,       0,  596810,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  479,  691,  596841,  596865,  596896, 24, 31, 33, }, // Europe_Western
    {  479,  706,  596929,  596952,  596982, 23, 30, 32, }, // Falkland
    {  479,  715,  136173,  597014,  597033, 12, 19, 21, }, // Fiji
    {  479,  720,       0,  597054,       0,  0, 23,  0, }, // French_Guiana
    {  479,  734,       0,  597077,       0,  0, 48,  0, }, // French_Southern
    {  479,  757,       0,  597125,       0,  0, 18,  0, }, // Galapagos
    {  479,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {  479,  775,  597143,  597158,  597180, 15, 22, 24, }, // Georgia
    {  479,  783,       0,  597204,       0,  0, 22,  0, }, // Gilbert_Islands
    {  479,  799,       0,  597226,       0,  0, 17,  0, }, // GMT
    {  479,  841,  597243,  597271,  597306, 28, 35, 37, }, // Greenland_Eastern
    {  479,  859,  597343,  597372,  597408, 29, 36, 38, }, // Greenland_Western
    {  479,  877,       0,  597446,       0,  0, 19,  0, }, // Guam
    {  479,  882,       0,  597465,       0,  0, 20,  0, }, // Gulf
    {  479,  887,       0,  597054,       0,  0, 14,  0, }, // Guyana
    {  479,  894,  597485,  597508,  597538, 23, 30, 32, }, // Hawaii_Aleutian
    {  479,  910,  136716,  597570,  597594, 17, 24, 26, }, // Hong_Kong
    {  479,  920,   56045,  597620,  597639, 12, 19, 21, }, // Hovd
    {  479,  925,       0,  597660,       0,  0, 20,  0, }, // India
    {  479,  931,       0,  597680,       0,  0, 21,  0, }, // Indian_Ocean
    {  479,  944,       0,  597701,       0,  0, 17,  0, }, // Indochina
    {  479,  954,       0,  597718,       0,  0, 25,  0, }, // Indonesia_Central
    {  479,  972,       0,  597743,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  479,  990,       0,  597769,       0,  0, 27,  0, }, // Indonesia_Western
    {  479, 1008,  597796,  597808,  597827, 12, 19, 21, }, // Iran
    {  479, 1019,  597848,  597863,  597885, 15, 22, 24, }, // Irkutsk
    {  479, 1027,  597909,  597923,  597944, 14, 21, 23, }, // Israel
    {  479, 1034,  597967,  597980,  598000, 13, 20, 22, }, // Japan
    {  479, 1040,  598022,  598054,  598093, 32, 39, 41, }, // Kamchatka
    {  479, 1058,       0,  598134,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  479, 1077,       0,  598162,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  479, 1106,  598191,  598205,  598226, 14, 21, 23, }, // Korea
    {  479, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {  479, 1119,   56703,  598249,  598275, 19, 26, 28, }, // Krasnoyarsk
    {  479, 1151,       0,  598303,       0,  0, 19,  0, }, // Kyrgystan
    {  479, 1161,       0,  598322,       0,  0, 17,  0, }, // Lanka
    {  479, 1175,       0,  598339,       0,  0, 19,  0, }, // Line_Islands
    {  479, 1188,   56833,  598358,  598382, 17, 24, 26, }, // Lord_Howe
    {  479, 1198,  137554,  598408,  598428, 13, 20, 22, }, // Macau
    {  479, 1204,  137611,  598450,  598472, 15, 22, 24, }, // Magadan
    {  479, 1219,       0,  598496,       0,  0, 15,  0, }, // Malaysia
    {  479, 1228,       0,  598511,       0,  0, 17,  0, }, // Maldives
    {  479, 1237,       0,  598528,       0,  0, 24,  0, }, // Marquesas
    {  479, 1247,       0,  598552,       0,  0, 23,  0, }, // Marshall_Islands
    {  479, 1264,  598575,  598591,  598614, 16, 23, 25, }, // Mauritius
    {  479, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {  479, 1281,  598639,  598664,  598696, 25, 32, 34, }, // Mexico_Pacific
    {  479, 1296,  598730,  598748,  598773, 18, 25, 27, }, // Mongolia
    {  479, 1305,  598800,  598815,  598837, 15, 22, 24, }, // Moscow
    {  479, 1312,       0,  598861,       0,  0, 15,  0, }, // Myanmar
    {  479, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {  479, 1326,       0,  598876,       0,  0, 13,  0, }, // Nepal
    {  479, 1332,  598889,  598911,  598940, 22, 29, 31, }, // New_Caledonia
    {  479, 1346,  598971,  598992,  599020, 21, 28, 30, }, // New_Zealand
    {  479, 1358,  599050,  599068,  599093, 18, 25, 27, }, // Newfoundland
    {  479, 1371,       0,  599120,       0,  0, 12,  0, }, // Niue
    {  479, 1376,  599132,  599152,  599179, 20, 27, 29, }, // Norfolk
    {  479, 1384,   57800,  599208,  599242, 27, 34, 36, }, // Noronha
    {  479, 1392,       0,  599278,       0,  0, 31,  0, }, // North_Mariana
    {  479, 1406,   57936,  599309,  599335, 19, 26, 28, }, // Novosibirsk
    {  479, 1418,  599363,  599375,  599394, 12, 19, 21, }, // Omsk
    {  479, 1428,  599415,  599432,  599456, 17, 24, 26, }, // Pakistan
    {  479, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {  479, 1443,       0,  599482,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  479, 1460,  599506,  599522,  599545, 16, 23, 25, }, // Paraguay
    {  479, 1469,  599570,  599582,  599601, 12, 19, 21, }, // Peru
    {  479, 1474,  599622,  599640,  599665, 18, 25, 27, }, // Philippines
    {  479, 1486,       0,  599692,       0,  0, 20,  0, }, // Phoenix_Islands
    {  479, 1502,  599712,  599740,  599775, 28, 35, 37, }, // Pierre_Miquelon
    {  479, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {  479, 1527,       0,   58521,       0,  0, 14,  0, }, // Ponape
    {  479, 1534,       0,  138941,       0,  0, 17,  0, }, // Pyongyang
    {  479, 1544,   58552,  599812,  599836, 17, 24, 26, }, // Qyzylorda
    {  479, 1554,       0,  599862,       0,  0, 15,  0, }, // Reunion
    {  479, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {  479, 1570,  599877,  599893,  599916, 16, 23, 25, }, // Sakhalin
    {  479, 1579,   58714,  599941,  599962, 14, 21, 23, }, // Samara
    {  479, 1596,   58774,  599985,  600005, 13, 20, 22, }, // Samoa
    {  479, 1602,       0,  600027,       0,  0, 18,  0, }, // Seychelles
    {  479, 1624,       0,  600045,       0,  0, 24,  0, }, // Singapore
    {  479, 1634,       0,  600069,       0,  0, 22,  0, }, // Solomon
    {  479, 1642,       0,  600091,       0,  0, 22,  0, }, // South_Georgia
    {  479, 1656,       0,  600113,       0,  0, 16,  0, }, // Suriname
    {  479, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {  479, 1682,       0,  600129,       0,  0, 13,  0, }, // Tahiti
    {  479, 1689,   58967,  600142,  600162, 13, 20, 22, }, // Taipei
    {  479, 1696,       0,  600184,       0,  0, 19,  0, }, // Tajikistan
    {  479, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {  479, 1732,   59062,  600203,  600223, 13, 20, 22, }, // Tonga
    {  479, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {  479, 1750,  600245,  600267,  600296, 22, 29, 31, }, // Turkmenistan
    {  479, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {  479, 1777,  600327,  600342,  600364, 15, 22, 24, }, // Uruguay
    {  479, 1792,  600388,  600407,  600433, 19, 26, 28, }, // Uzbekistan
    {  479, 1803,  600461,  600476,  600498, 15, 22, 24, }, // Vanuatu
    {  479, 1811,       0,  600522,       0,  0, 17,  0, }, // Venezuela
    {  479, 1821,   59451,  600539,  600565, 19, 26, 28, }, // Vladivostok
    {  479, 1833,  600593,  600611,  600636, 18, 25, 27, }, // Volgograd
    {  479, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {  479, 1850,       0,  600663,       0,  0, 17,  0, }, // Wake
    {  479, 1855,       0,  600680,       0,  0, 23,  0, }, // Wallis
    {  479, 1862,   59653,  600703,  600725, 15, 22, 24, }, // Yakutsk
    {  479, 1870,  600749,  600770,  600798, 21, 28, 30, }, // Yekaterinburg
    {  479, 1892,       0,  600828,       0,  0, 13,  0, }, // Yukon
    {  480,    0,  593895,  593907,  593926, 12, 19, 21, }, // Acre Portuguese/Latin/Timor-Leste
    {  480,    5,       0,  593947,       0,  0, 19,  0, }, // Afghanistan
    {  480,   17,       0,  593966,       0,  0, 22,  0, }, // Africa_Central
    {  480,   32,       0,  593988,       0,  0, 23,  0, }, // Africa_Eastern
    {  480,   65,       0,  594011,       0,  0, 21,  0, }, // Africa_Southern
    {  480,   81,  594032,  594056,  594087, 24, 31, 33, }, // Africa_Western
    {  480,  107,  594120,  594134,  594155, 14, 21, 23, }, // Alaska
    {  480,  128,  594178,  594192,  594213, 14, 21, 23, }, // Almaty
    {  480,  135,  594236,  594252,  594275, 16, 23, 25, }, // Amazon
    {  480,  142,  594300,  594328,  594363, 28, 35, 37, }, // America_Central
    {  480,  158,  594400,  594429,  594465, 29, 36, 38, }, // America_Eastern
    {  480,  174,  594503,  594535,  594574, 32, 39, 41, }, // America_Mountain
    {  480,  191,  594615,  594647,  594686, 32, 39, 41, }, // America_Pacific
    {  480,  207,  594727,  594741,  594762, 14, 21, 23, }, // Anadyr
    {  480,  214,  594785,  594797,  594816, 12, 19, 21, }, // Apia
    {  480,  219,  594837,  594850,  594870, 13, 20, 22, }, // Aqtau
    {  480,  225,  594892,  594906,  594927, 14, 21, 23, }, // Aqtobe
    {  480,  232,  594950,  594964,  594985, 14, 21, 23, }, // Arabian
    {  480,  240,  595008,  595025,  595049, 17, 24, 26, }, // Argentina
    {  480,  250,  595075,  595102,  595136, 27, 34, 36, }, // Argentina_Western
    {  480,  268,  595172,  595187,  595209, 15, 22, 24, }, // Armenia
    {  480,  286,  595233,  595250,  595274, 17, 24, 26, }, // Atlantic
    {  480,  295,  595300,  595325,  595357, 25, 32, 34, }, // Australia_Central
    {  480,  313,  595391,  595426,  595468, 35, 42, 44, }, // Australia_CentralWestern
    {  480,  338,  595512,  595538,  595571, 26, 33, 35, }, // Australia_Eastern
    {  480,  356,  595606,  595633,  595667, 27, 34, 36, }, // Australia_Western
    {  480,  374,  595703,  595721,  595746, 18, 25, 27, }, // Azerbaijan
    {  480,  385,  595773,  595788,  595810, 15, 22, 24, }, // Azores
    {  480,  397,  595834,  595853,  595879, 19, 26, 28, }, // Bangladesh
    {  480,  415,       0,  595907,       0,  0, 13,  0, }, // Bhutan
    {  480,  422,       0,  595920,       0,  0, 15,  0, }, // Bolivia
    {  480,  437,  135014,  595935,  595958, 16, 23, 25, }, // Brasilia
    {  480,  454,       0,  595983,       0,  0, 25,  0, }, // Brunei
    {  480,  461,  596008,  596026,  596051, 18, 25, 27, }, // Cape_Verde
    {  480,  478,       0,  596078,       0,  0, 23,  0, }, // Chamorro
    {  480,  487,   54300,  596101,  596123, 15, 22, 24, }, // Chatham
    {  480,  495,  596147,  596160,  596180, 13, 20, 22, }, // Chile
    {  480,  501,  596202,  596215,  596235, 13, 20, 22, }, // China
    {  480,  507,   54477,  596257,  596282, 18, 25, 27, }, // Choibalsan
    {  480,  518,       0,  596309,       0,  0, 21,  0, }, // Christmas
    {  480,  528,       0,  596330,       0,  0, 20,  0, }, // Cocos
    {  480,  534,  596350,  596366,  596389, 16, 23, 25, }, // Colombia
    {  480,  543,  596414,  596433,  596459, 19, 26, 28, }, // Cook
    {  480,  548,   54767,  596487,  596506, 12, 19, 21, }, // Cuba
    {  480,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {  480,  575,       0,   54834,       0,  0, 24,  0, }, // DumontDUrville
    {  480,  612,       0,  596527,       0,  0, 19,  0, }, // East_Timor
    {  480,  623,  596546,  596568,  596597, 22, 29, 31, }, // Easter
    {  480,  630,       0,  596628,       0,  0, 15,  0, }, // Ecuador
    {  480,  638,  596643,  596665,  596694, 22, 29, 31, }, // Europe_Central
    {  480,  653,  596725,  596748,  596778, 23, 30, 32, }, // Europe_Eastern
    {  480,  668,       0,  596810,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  480,  691,  596841,  596865,  596896, 24, 31, 33, }, // Europe_Western
    {  480,  706,  596929,  596952,  596982, 23, 30, 32, }, // Falkland
    {  480,  715,  136173,  597014,  597033, 12, 19, 21, }, // Fiji
    {  480,  720,       0,  597054,       0,  0, 23,  0, }, // French_Guiana
    {  480,  734,       0,  597077,       0,  0, 48,  0, }, // French_Southern
    {  480,  757,       0,  597125,       0,  0, 18,  0, }, // Galapagos
    {  480,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {  480,  775,  597143,  597158,  597180, 15, 22, 24, }, // Georgia
    {  480,  783,       0,  597204,       0,  0, 22,  0, }, // Gilbert_Islands
    {  480,  799,       0,  597226,       0,  0, 17,  0, }, // GMT
    {  480,  841,  597243,  597271,  597306, 28, 35, 37, }, // Greenland_Eastern
    {  480,  859,  597343,  597372,  597408, 29, 36, 38, }, // Greenland_Western
    {  480,  877,       0,  597446,       0,  0, 19,  0, }, // Guam
    {  480,  882,       0,  597465,       0,  0, 20,  0, }, // Gulf
    {  480,  887,       0,  597054,       0,  0, 14,  0, }, // Guyana
    {  480,  894,  597485,  597508,  597538, 23, 30, 32, }, // Hawaii_Aleutian
    {  480,  910,  136716,  597570,  597594, 17, 24, 26, }, // Hong_Kong
    {  480,  920,   56045,  597620,  597639, 12, 19, 21, }, // Hovd
    {  480,  925,       0,  597660,       0,  0, 20,  0, }, // India
    {  480,  931,       0,  597680,       0,  0, 21,  0, }, // Indian_Ocean
    {  480,  944,       0,  597701,       0,  0, 17,  0, }, // Indochina
    {  480,  954,       0,  597718,       0,  0, 25,  0, }, // Indonesia_Central
    {  480,  972,       0,  597743,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  480,  990,       0,  597769,       0,  0, 27,  0, }, // Indonesia_Western
    {  480, 1008,  597796,  597808,  597827, 12, 19, 21, }, // Iran
    {  480, 1019,  597848,  597863,  597885, 15, 22, 24, }, // Irkutsk
    {  480, 1027,  597909,  597923,  597944, 14, 21, 23, }, // Israel
    {  480, 1034,  597967,  597980,  598000, 13, 20, 22, }, // Japan
    {  480, 1040,  598022,  598054,  598093, 32, 39, 41, }, // Kamchatka
    {  480, 1058,       0,  598134,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  480, 1077,       0,  598162,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  480, 1106,  598191,  598205,  598226, 14, 21, 23, }, // Korea
    {  480, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {  480, 1119,   56703,  598249,  598275, 19, 26, 28, }, // Krasnoyarsk
    {  480, 1151,       0,  598303,       0,  0, 19,  0, }, // Kyrgystan
    {  480, 1161,       0,  598322,       0,  0, 17,  0, }, // Lanka
    {  480, 1175,       0,  598339,       0,  0, 19,  0, }, // Line_Islands
    {  480, 1188,   56833,  598358,  598382, 17, 24, 26, }, // Lord_Howe
    {  480, 1198,  137554,  598408,  598428, 13, 20, 22, }, // Macau
    {  480, 1204,  137611,  598450,  598472, 15, 22, 24, }, // Magadan
    {  480, 1219,       0,  598496,       0,  0, 15,  0, }, // Malaysia
    {  480, 1228,       0,  598511,       0,  0, 17,  0, }, // Maldives
    {  480, 1237,       0,  598528,       0,  0, 24,  0, }, // Marquesas
    {  480, 1247,       0,  598552,       0,  0, 23,  0, }, // Marshall_Islands
    {  480, 1264,  598575,  598591,  598614, 16, 23, 25, }, // Mauritius
    {  480, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {  480, 1281,  598639,  598664,  598696, 25, 32, 34, }, // Mexico_Pacific
    {  480, 1296,  598730,  598748,  598773, 18, 25, 27, }, // Mongolia
    {  480, 1305,  598800,  598815,  598837, 15, 22, 24, }, // Moscow
    {  480, 1312,       0,  598861,       0,  0, 15,  0, }, // Myanmar
    {  480, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {  480, 1326,       0,  598876,       0,  0, 13,  0, }, // Nepal
    {  480, 1332,  598889,  598911,  598940, 22, 29, 31, }, // New_Caledonia
    {  480, 1346,  598971,  598992,  599020, 21, 28, 30, }, // New_Zealand
    {  480, 1358,  599050,  599068,  599093, 18, 25, 27, }, // Newfoundland
    {  480, 1371,       0,  599120,       0,  0, 12,  0, }, // Niue
    {  480, 1376,  599132,  599152,  599179, 20, 27, 29, }, // Norfolk
    {  480, 1384,   57800,  599208,  599242, 27, 34, 36, }, // Noronha
    {  480, 1392,       0,  599278,       0,  0, 31,  0, }, // North_Mariana
    {  480, 1406,   57936,  599309,  599335, 19, 26, 28, }, // Novosibirsk
    {  480, 1418,  599363,  599375,  599394, 12, 19, 21, }, // Omsk
    {  480, 1428,  599415,  599432,  599456, 17, 24, 26, }, // Pakistan
    {  480, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {  480, 1443,       0,  599482,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  480, 1460,  599506,  599522,  599545, 16, 23, 25, }, // Paraguay
    {  480, 1469,  599570,  599582,  599601, 12, 19, 21, }, // Peru
    {  480, 1474,  599622,  599640,  599665, 18, 25, 27, }, // Philippines
    {  480, 1486,       0,  599692,       0,  0, 20,  0, }, // Phoenix_Islands
    {  480, 1502,  599712,  599740,  599775, 28, 35, 37, }, // Pierre_Miquelon
    {  480, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {  480, 1527,       0,   58521,       0,  0, 14,  0, }, // Ponape
    {  480, 1534,       0,  138941,       0,  0, 17,  0, }, // Pyongyang
    {  480, 1544,   58552,  599812,  599836, 17, 24, 26, }, // Qyzylorda
    {  480, 1554,       0,  599862,       0,  0, 15,  0, }, // Reunion
    {  480, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {  480, 1570,  599877,  599893,  599916, 16, 23, 25, }, // Sakhalin
    {  480, 1579,   58714,  599941,  599962, 14, 21, 23, }, // Samara
    {  480, 1596,   58774,  599985,  600005, 13, 20, 22, }, // Samoa
    {  480, 1602,       0,  600027,       0,  0, 18,  0, }, // Seychelles
    {  480, 1624,       0,  600045,       0,  0, 24,  0, }, // Singapore
    {  480, 1634,       0,  600069,       0,  0, 22,  0, }, // Solomon
    {  480, 1642,       0,  600091,       0,  0, 22,  0, }, // South_Georgia
    {  480, 1656,       0,  600113,       0,  0, 16,  0, }, // Suriname
    {  480, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {  480, 1682,       0,  600129,       0,  0, 13,  0, }, // Tahiti
    {  480, 1689,   58967,  600142,  600162, 13, 20, 22, }, // Taipei
    {  480, 1696,       0,  600184,       0,  0, 19,  0, }, // Tajikistan
    {  480, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {  480, 1732,   59062,  600203,  600223, 13, 20, 22, }, // Tonga
    {  480, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {  480, 1750,  600245,  600267,  600296, 22, 29, 31, }, // Turkmenistan
    {  480, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {  480, 1777,  600327,  600342,  600364, 15, 22, 24, }, // Uruguay
    {  480, 1792,  600388,  600407,  600433, 19, 26, 28, }, // Uzbekistan
    {  480, 1803,  600461,  600476,  600498, 15, 22, 24, }, // Vanuatu
    {  480, 1811,       0,  600522,       0,  0, 17,  0, }, // Venezuela
    {  480, 1821,   59451,  600539,  600565, 19, 26, 28, }, // Vladivostok
    {  480, 1833,  600593,  600611,  600636, 18, 25, 27, }, // Volgograd
    {  480, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {  480, 1850,       0,  600663,       0,  0, 17,  0, }, // Wake
    {  480, 1855,       0,  600680,       0,  0, 23,  0, }, // Wallis
    {  480, 1862,   59653,  600703,  600725, 15, 22, 24, }, // Yakutsk
    {  480, 1870,  600749,  600770,  600798, 21, 28, 30, }, // Yekaterinburg
    {  480, 1892,       0,  600828,       0,  0, 13,  0, }, // Yukon
    {  481,  142,  600841,  600865,  600896, 24, 31, 31, }, // America_Central Prussian/Latin/Poland
    {  481,  158,  600927,  600951,  600982, 24, 31, 31, }, // America_Eastern
    {  481,  174,  601013,  601034,  601062, 21, 28, 28, }, // America_Mountain
    {  481,  191,  601090,  601115,  601147, 25, 32, 32, }, // America_Pacific
    {  481,  286,  601179,  601195,  601218, 16, 23, 23, }, // Atlantic
    {  481,  638,  601241,  601264,  601294, 23, 30, 30, }, // Europe_Central
    {  481,  653,  601324,  601347,  601377, 23, 30, 30, }, // Europe_Eastern
    {  481,  691,  601407,  601432,  601464, 25, 32, 32, }, // Europe_Western
    {  481,  799,       0,  601496,       0,  0, 15,  0, }, // GMT
    {  482,    5,       0,  601511,       0,  0, 16,  0, }, // Afghanistan Punjabi/Gurmukhi/India
    {  482,   17,       0,  601527,       0,  0, 18,  0, }, // Africa_Central
    {  482,   32,       0,  601545,       0,  0, 17,  0, }, // Africa_Eastern
    {  482,   65,       0,  601562,       0,  0, 24,  0, }, // Africa_Southern
    {  482,   81,  601586,  601603,  601626, 17, 23, 27, }, // Africa_Western
    {  482,  107,  601653,  601664,  601681, 11, 17, 19, }, // Alaska
    {  482,  128,  601700,  601711,  601728, 11, 17, 21, }, // Almaty
    {  482,  135,  601749,  601760,  601777, 11, 17, 21, }, // Amazon
    {  482,  142,  601798,  601822,  601852, 24, 30, 32, }, // America_Central
    {  482,  158,  601884,  601907,  601936, 23, 29, 31, }, // America_Eastern
    {  482,  174,  601967,  601992,  602023, 25, 31, 33, }, // America_Mountain
    {  482,  191,  602056,  602081,  602112, 25, 31, 33, }, // America_Pacific
    {  482,  214,  602145,  602154,  602169,  9, 15, 17, }, // Apia
    {  482,  219,  602186,  602196,  602212, 10, 16, 20, }, // Aqtau
    {  482,  225,  602232,  602242,  602258, 10, 16, 20, }, // Aqtobe
    {  482,  232,  602278,  602287,  602302,  9, 15, 17, }, // Arabian
    {  482,  240,  602319,  602332,  602351, 13, 19, 23, }, // Argentina
    {  482,  250,  602374,  602393,  602418, 19, 25, 29, }, // Argentina_Western
    {  482,  268,  602447,  602459,  602477, 12, 18, 22, }, // Armenia
    {  482,  286,  602499,  602512,  602531, 13, 19, 21, }, // Atlantic
    {  482,  295,  602552,  602574,  602602, 22, 28, 30, }, // Australia_Central
    {  482,  313,  602632,  602660,  602694, 28, 34, 36, }, // Australia_CentralWestern
    {  482,  338,  602730,  602751,  602778, 21, 27, 29, }, // Australia_Eastern
    {  482,  356,  602807,  602828,  602855, 21, 27, 29, }, // Australia_Western
    {  482,  374,  602884,  602899,  602920, 15, 21, 25, }, // Azerbaijan
    {  482,  385,  602945,  602956,  602973, 11, 17, 21, }, // Azores
    {  482,  397,  602994,  603008,  603028, 14, 20, 24, }, // Bangladesh
    {  482,  415,       0,  603052,       0,  0, 10,  0, }, // Bhutan
    {  482,  422,       0,  603062,       0,  0, 12,  0, }, // Bolivia
    {  482,  437,  603074,  603089,  603110, 15, 21, 25, }, // Brasilia
    {  482,  454,       0,  603135,       0,  0, 20,  0, }, // Brunei
    {  482,  461,  603155,  603167,  603185, 12, 18, 22, }, // Cape_Verde
    {  482,  472,       0,  603207,       0,  0,  9,  0, }, // Casey
    {  482,  478,       0,  603216,       0,  0, 17,  0, }, // Chamorro
    {  482,  487,  603233,  603242,  603257,  9, 15, 17, }, // Chatham
    {  482,  495,  603274,  603283,  603298,  9, 15, 19, }, // Chile
    {  482,  501,  603317,  603325,  603339,  8, 14, 16, }, // China
    {  482,  507,  603355,  603368,  603387, 13, 19, 23, }, // Choibalsan
    {  482,  518,       0,  603410,       0,  0, 19,  0, }, // Christmas
    {  482,  528,       0,  603429,       0,  0, 16,  0, }, // Cocos
    {  482,  534,  603445,  603457,  603475, 12, 18, 22, }, // Colombia
    {  482,  543,  603497,  603513,  603535, 16, 22, 30, }, // Cook
    {  482,  548,  603565,  603575,  603591, 10, 16, 18, }, // Cuba
    {  482,  559,       0,  603609,       0,  0, 10,  0, }, // Davis
    {  482,  575,       0,  603619,       0,  0, 21,  0, }, // DumontDUrville
    {  482,  612,       0,  603640,       0,  0, 16,  0, }, // East_Timor
    {  482,  623,  603656,  603672,  603694, 16, 22, 26, }, // Easter
    {  482,  630,       0,  603720,       0,  0, 12,  0, }, // Ecuador
    {  482,  638,  603732,  603746,  603766, 14, 20, 24, }, // Europe_Central
    {  482,  653,  603790,  603806,  603828, 16, 22, 26, }, // Europe_Eastern
    {  482,  668,       0,  603854,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  482,  691,  603874,  603890,  603912, 16, 22, 26, }, // Europe_Western
    {  482,  706,  603938,  603959,  603986, 21, 27, 31, }, // Falkland
    {  482,  715,  604017,  604028,  604045, 11, 17, 21, }, // Fiji
    {  482,  720,       0,  604066,       0,  0, 17,  0, }, // French_Guiana
    {  482,  734,       0,  604083,       0,  0, 30,  0, }, // French_Southern
    {  482,  757,       0,  604113,       0,  0, 13,  0, }, // Galapagos
    {  482,  767,       0,  604126,       0,  0, 12,  0, }, // Gambier
    {  482,  775,  604138,  604149,  604166, 11, 17, 21, }, // Georgia
    {  482,  783,       0,  604187,       0,  0, 18,  0, }, // Gilbert_Islands
    {  482,  799,       0,  604205,       0,  0, 17,  0, }, // GMT
    {  482,  841,  604222,  604242,  604268, 20, 26, 30, }, // Greenland_Eastern
    {  482,  859,  604298,  604318,  604344, 20, 26, 30, }, // Greenland_Western
    {  482,  877,       0,  604374,       0,  0,  9,  0, }, // Guam
    {  482,  882,       0,  604383,       0,  0, 15,  0, }, // Gulf
    {  482,  887,       0,  604398,       0,  0, 11,  0, }, // Guyana
    {  482,  894,  604409,  604429,  604455, 20, 26, 28, }, // Hawaii_Aleutian
    {  482,  910,  604483,  604497,  604517, 14, 20, 24, }, // Hong_Kong
    {  482,  920,  604541,  604550,  604565,  9, 15, 19, }, // Hovd
    {  482,  925,       0,  604584,       0,  0, 16,  0, }, // India
    {  482,  931,       0,  604600,       0,  0, 18,  0, }, // Indian_Ocean
    {  482,  944,       0,  604618,       0,  0, 14,  0, }, // Indochina
    {  482,  954,       0,  604632,       0,  0, 20,  0, }, // Indonesia_Central
    {  482,  972,       0,  604652,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  482,  990,       0,  604673,       0,  0, 21,  0, }, // Indonesia_Western
    {  482, 1008,  604694,  604703,  604718,  9, 15, 17, }, // Iran
    {  482, 1019,  604735,  604747,  604765, 12, 18, 22, }, // Irkutsk
    {  482, 1027,  604787,  604799,  604817, 12, 18, 20, }, // Israel
    {  482, 1034,  604837,  604846,  604861,  9, 15, 17, }, // Japan
    {  482, 1058,       0,  604878,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  482, 1077,       0,  604899,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  482, 1106,  604920,  604931,  604948, 11, 17, 19, }, // Korea
    {  482, 1112,       0,  604967,       0,  0, 10,  0, }, // Kosrae
    {  482, 1119,  604977,  604994,  605017, 17, 23, 27, }, // Krasnoyarsk
    {  482, 1151,       0,  605044,       0,  0, 14,  0, }, // Kyrgystan
    {  482, 1161,       0,  605058,       0,  0,  9,  0, }, // Lanka
    {  482, 1175,       0,  605067,       0,  0, 16,  0, }, // Line_Islands
    {  482, 1188,  605083,  605097,  605117, 14, 20, 22, }, // Lord_Howe
    {  482, 1198,  605139,  605148,  605163,  9, 15, 19, }, // Macau
    {  482, 1204,  605182,  605193,  605210, 11, 17, 21, }, // Magadan
    {  482, 1219,       0,  605231,       0,  0, 12,  0, }, // Malaysia
    {  482, 1228,       0,  605243,       0,  0, 11,  0, }, // Maldives
    {  482, 1237,       0,  605254,       0,  0, 13,  0, }, // Marquesas
    {  482, 1247,       0,  605267,       0,  0, 18,  0, }, // Marshall_Islands
    {  482, 1264,  605285,  605297,  605315, 12, 18, 22, }, // Mauritius
    {  482, 1274,       0,  605337,       0,  0,  9,  0, }, // Mawson
    {  482, 1281,  605346,  605366,  605392, 20, 26, 28, }, // Mexico_Pacific
    {  482, 1296,  605420,  605433,  605452, 13, 19, 23, }, // Mongolia
    {  482, 1305,  605475,  605485,  605501, 10, 16, 20, }, // Moscow
    {  482, 1312,       0,  605521,       0,  0, 12,  0, }, // Myanmar
    {  482, 1320,       0,  605533,       0,  0, 10,  0, }, // Nauru
    {  482, 1326,       0,  605543,       0,  0, 10,  0, }, // Nepal
    {  482, 1332,  605553,  605571,  605595, 18, 24, 28, }, // New_Caledonia
    {  482, 1346,  605623,  605638,  605659, 15, 21, 23, }, // New_Zealand
    {  482, 1358,  605682,  605699,  605722, 17, 23, 25, }, // Newfoundland
    {  482, 1371,       0,  605747,       0,  0,  8,  0, }, // Niue
    {  482, 1376,  605755,  605773,  605797, 18, 24, 26, }, // Norfolk
    {  482, 1384,  605823,  605846,  605875, 23, 29, 33, }, // Noronha
    {  482, 1392,       0,  605908,       0,  0, 24,  0, }, // North_Mariana
    {  482, 1406,  605932,  605948,  605970, 16, 22, 26, }, // Novosibirsk
    {  482, 1418,  605996,  606005,  606020,  9, 15, 19, }, // Omsk
    {  482, 1428,  606039,  606052,  606071, 13, 19, 23, }, // Pakistan
    {  482, 1437,       0,  606094,       0,  0,  9,  0, }, // Palau
    {  482, 1443,       0,  606103,       0,  0, 19,  0, }, // Papua_New_Guinea
    {  482, 1460,  606122,  606134,  606152, 12, 18, 22, }, // Paraguay
    {  482, 1469,  606174,  606183,  606198,  9, 15, 19, }, // Peru
    {  482, 1474,  606217,  606230,  606249, 13, 19, 23, }, // Philippines
    {  482, 1486,       0,  606272,       0,  0, 18,  0, }, // Phoenix_Islands
    {  482, 1502,  606290,  606316,  606348, 26, 32, 34, }, // Pierre_Miquelon
    {  482, 1518,       0,  606382,       0,  0, 12,  0, }, // Pitcairn
    {  482, 1527,       0,  606394,       0,  0, 11,  0, }, // Ponape
    {  482, 1534,       0,  606405,       0,  0, 14,  0, }, // Pyongyang
    {  482, 1544,  606419,  606433,  606453, 14, 20, 24, }, // Qyzylorda
    {  482, 1554,       0,  606477,       0,  0, 13,  0, }, // Reunion
    {  482, 1562,       0,  606490,       0,  0, 11,  0, }, // Rothera
    {  482, 1570,  606501,  606511,  606527, 10, 16, 20, }, // Sakhalin
    {  482, 1596,  606547,  606557,  606573, 10, 16, 18, }, // Samoa
    {  482, 1602,       0,  606591,       0,  0, 11,  0, }, // Seychelles
    {  482, 1624,       0,  606602,       0,  0, 19,  0, }, // Singapore
    {  482, 1634,       0,  606621,       0,  0, 19,  0, }, // Solomon
    {  482, 1642,       0,  606640,       0,  0, 17,  0, }, // South_Georgia
    {  482, 1656,       0,  606657,       0,  0, 12,  0, }, // Suriname
    {  482, 1676,       0,  606669,       0,  0, 10,  0, }, // Syowa
    {  482, 1682,       0,  606679,       0,  0, 11,  0, }, // Tahiti
    {  482, 1689,  606690,  606699,  606714,  9, 15, 17, }, // Taipei
    {  482, 1696,       0,  606731,       0,  0, 15,  0, }, // Tajikistan
    {  482, 1724,       0,  606746,       0,  0, 12,  0, }, // Tokelau
    {  482, 1732,  606758,  606768,  606784, 10, 16, 20, }, // Tonga
    {  482, 1738,       0,  606804,       0,  0,  8,  0, }, // Truk
    {  482, 1750,  606812,  606829,  606852, 17, 23, 27, }, // Turkmenistan
    {  482, 1763,       0,  606879,       0,  0, 11,  0, }, // Tuvalu
    {  482, 1777,  606890,  606901,  606918, 11, 17, 21, }, // Uruguay
    {  482, 1792,  606939,  606955,  606977, 16, 22, 26, }, // Uzbekistan
    {  482, 1803,  607003,  607015,  607033, 12, 18, 22, }, // Vanuatu
    {  482, 1811,       0,  607055,       0,  0, 15,  0, }, // Venezuela
    {  482, 1821,  607070,  607085,  607106, 15, 21, 25, }, // Vladivostok
    {  482, 1833,  607131,  607146,  607167, 15, 21, 25, }, // Volgograd
    {  482, 1843,       0,  607192,       0,  0, 11,  0, }, // Vostok
    {  482, 1850,       0,  607203,       0,  0, 15,  0, }, // Wake
    {  482, 1855,       0,  607218,       0,  0, 21,  0, }, // Wallis
    {  482, 1862,  607239,  607250,  607267, 11, 17, 21, }, // Yakutsk
    {  482, 1870,  607288,  607303,  607324, 15, 21, 25, }, // Yekaterinburg
    {  482, 1892,       0,  607349,       0,  0, 10,  0, }, // Yukon
    {  484,    5,       0,  607359,       0,  0, 18,  0, }, // Afghanistan Quechua/Latin/Peru
    {  484,   17,       0,  607377,       0,  0, 22,  0, }, // Africa_Central
    {  484,   32,       0,  607399,       0,  0, 23,  0, }, // Africa_Eastern
    {  484,   65,       0,  607422,       0,  0, 17,  0, }, // Africa_Southern
    {  484,   81,  607439,  607464,  607498, 25, 34, 44, }, // Africa_Western
    {  484,  107,  607542,  607556,  607579, 14, 23, 24, }, // Alaska
    {  484,  135,  607603,  607619,  607644, 16, 25, 26, }, // Amazon
    {  484,  142,  607670,  607682,  607704, 12, 22, 22, }, // America_Central
    {  484,  158,  607726,  607739,  607761, 13, 22, 23, }, // America_Eastern
    {  484,  174,  607784,  607802,  607829, 18, 27, 28, }, // America_Mountain
    {  484,  191,  607857,  607874,  607900, 17, 26, 27, }, // America_Pacific
    {  484,  214,  594785,  607927,  607948, 12, 21, 22, }, // Apia
    {  484,  232,  607970,  607984,  608007, 14, 23, 24, }, // Arabian
    {  484,  240,  608031,  608048,  608074, 17, 26, 27, }, // Argentina
    {  484,  250,  608101,  608128,  608164, 27, 36, 37, }, // Argentina_Western
    {  484,  268,  608201,  608216,  608240, 15, 24, 25, }, // Armenia
    {  484,  286,  608265,  608283,  608310, 18, 27, 28, }, // Atlantic
    {  484,  295,  608338,  608363,  608397, 25, 34, 35, }, // Australia_Central
    {  484,  313,  608432,  608468,  608513, 36, 45, 46, }, // Australia_CentralWestern
    {  484,  338,  608559,  608585,  608620, 26, 35, 36, }, // Australia_Eastern
    {  484,  356,  608656,  608684,  608721, 28, 37, 38, }, // Australia_Western
    {  484,  374,  608759,  608777,  608804, 18, 27, 28, }, // Azerbaijan
    {  484,  385,  608832,  608850,  608877, 18, 27, 28, }, // Azores
    {  484,  397,  134913,  608905,  608932, 18, 27, 28, }, // Bangladesh
    {  484,  415,       0,   54071,       0,  0, 13,  0, }, // Bhutan
    {  484,  422,       0,    1595,       0,  0, 12,  0, }, // Bolivia
    {  484,  437,   54099,  608960,  608985, 16, 25, 26, }, // Brasilia
    {  484,  454,       0,  135080,       0,  0, 25,  0, }, // Brunei
    {  484,  461,  596008,  609011,  609038, 18, 27, 28, }, // Cape_Verde
    {  484,  478,       0,  609066,       0,  0, 25,  0, }, // Chamorro
    {  484,  487,   54300,  609091,  609115, 15, 24, 25, }, // Chatham
    {  484,  495,   54363,  609140,  609162, 13, 22, 23, }, // Chile
    {  484,  501,   54420,  609185,  609207, 13, 22, 23, }, // China
    {  484,  507,  609230,  609248,  609275, 18, 27, 28, }, // Choibalsan
    {  484,  518,       0,  609303,       0,  0, 22,  0, }, // Christmas
    {  484,  528,       0,  609325,       0,  0, 19,  0, }, // Cocos
    {  484,  534,   54608,  609344,  609369, 16, 25, 26, }, // Colombia
    {  484,  543,  609395,  609413,  609440, 18, 27, 28, }, // Cook
    {  484,  548,   54767,  609468,  609489, 12, 21, 22, }, // Cuba
    {  484,  559,       0,   54821,       0,  0, 13,  0, }, // Davis
    {  484,  575,       0,   54834,       0,  0, 24,  0, }, // DumontDUrville
    {  484,  612,       0,   54858,       0,  0, 22,  0, }, // East_Timor
    {  484,  623,  609511,  609533,  609564, 22, 31, 32, }, // Easter
    {  484,  630,       0,  609596,       0,  0, 15,  0, }, // Ecuador
    {  484,  638,  609611,  609633,  609664, 22, 31, 32, }, // Europe_Central
    {  484,  653,  609696,  609719,  609751, 23, 32, 33, }, // Europe_Eastern
    {  484,  668,       0,  609784,       0,  0, 37,  0, }, // Europe_Further_Eastern
    {  484,  691,  609821,  609846,  609880, 25, 34, 35, }, // Europe_Western
    {  484,  706,  609915,  609937,  609968, 22, 31, 32, }, // Falkland
    {  484,  715,  136173,  610000,  610021, 12, 21, 22, }, // Fiji
    {  484,  720,       0,  610043,       0,  0, 24,  0, }, // French_Guiana
    {  484,  734,       0,  610067,       0,  0, 35,  0, }, // French_Southern
    {  484,  757,       0,  610102,       0,  0, 23,  0, }, // Galapagos
    {  484,  767,       0,   55505,       0,  0, 15,  0, }, // Gambier
    {  484,  775,  610125,  610140,  610164, 15, 24, 25, }, // Georgia
    {  484,  783,       0,  610189,       0,  0, 21,  0, }, // Gilbert_Islands
    {  484,  799,       0,  610210,       0,  0, 31,  0, }, // GMT
    {  484,  841,   55629,  610241,  610269, 19, 28, 29, }, // Greenland_Eastern
    {  484,  859,  610298,  610328,  610367, 30, 39, 40, }, // Greenland_Western
    {  484,  882,       0,  610407,       0,  0, 23,  0, }, // Gulf
    {  484,  887,       0,  136634,       0,  0, 14,  0, }, // Guyana
    {  484,  894,  610430,  610453,  610485, 23, 32, 33, }, // Hawaii_Aleutian
    {  484,  910,  136716,  610518,  610544, 17, 26, 27, }, // Hong_Kong
    {  484,  920,   56045,  610571,  610592, 12, 21, 22, }, // Hovd
    {  484,  925,       0,  610614,       0,  0, 22,  0, }, // India
    {  484,  931,       0,  610636,       0,  0, 22,  0, }, // Indian_Ocean
    {  484,  944,       0,  610658,       0,  0, 17,  0, }, // Indochina
    {  484,  954,       0,  610675,       0,  0, 25,  0, }, // Indonesia_Central
    {  484,  972,       0,  610700,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  484,  990,       0,  610726,       0,  0, 28,  0, }, // Indonesia_Western
    {  484, 1008,  610754,  610766,  610787, 12, 21, 22, }, // Iran
    {  484, 1019,  597848,  610809,  610833, 15, 24, 25, }, // Irkutsk
    {  484, 1027,  597909,  610858,  610881, 14, 23, 24, }, // Israel
    {  484, 1034,  610905,  610918,  610940, 13, 22, 23, }, // Japan
    {  484, 1058,       0,  610963,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  484, 1077,       0,  610990,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  484, 1106,   56632,  611018,  611040, 13, 22, 23, }, // Korea
    {  484, 1112,       0,   56689,       0,  0, 14,  0, }, // Kosrae
    {  484, 1119,   56703,  611063,  611091, 19, 28, 29, }, // Krasnoyarsk
    {  484, 1151,       0,  611120,       0,  0, 18,  0, }, // Kyrgystan
    {  484, 1175,       0,  611138,       0,  0, 30,  0, }, // Line_Islands
    {  484, 1188,   56833,  611168,  611194, 17, 26, 27, }, // Lord_Howe
    {  484, 1204,  137611,  611221,  611245, 15, 24, 25, }, // Magadan
    {  484, 1219,       0,   57022,       0,  0, 15,  0, }, // Malaysia
    {  484, 1228,       0,  611270,       0,  0, 16,  0, }, // Maldives
    {  484, 1237,       0,  611286,       0,  0, 17,  0, }, // Marquesas
    {  484, 1247,       0,  611303,       0,  0, 22,  0, }, // Marshall_Islands
    {  484, 1264,  611325,  611341,  611366, 16, 25, 26, }, // Mauritius
    {  484, 1274,       0,   57171,       0,  0, 14,  0, }, // Mawson
    {  484, 1281,  611392,  611418,  611453, 26, 35, 36, }, // Mexico_Pacific
    {  484, 1296,  611489,  611507,  611534, 18, 27, 28, }, // Mongolia
    {  484, 1305,   57353,  611562,  611584, 13, 22, 23, }, // Moscow
    {  484, 1312,       0,   57410,       0,  0, 15,  0, }, // Myanmar
    {  484, 1320,       0,   57425,       0,  0, 13,  0, }, // Nauru
    {  484, 1326,       0,  611607,       0,  0, 13,  0, }, // Nepal
    {  484, 1332,   57452,  611620,  611652, 23, 32, 33, }, // New_Caledonia
    {  484, 1346,   57539,  611685,  611715, 21, 30, 31, }, // New_Zealand
    {  484, 1358,  138171,  611746,  611772, 17, 26, 27, }, // Newfoundland
    {  484, 1371,       0,   57698,       0,  0, 12,  0, }, // Niue
    {  484, 1376,  611799,  611822,  611854, 23, 32, 33, }, // Norfolk
    {  484, 1384,   57800,  611887,  611923, 27, 36, 37, }, // Noronha
    {  484, 1406,   57936,  611960,  611988, 19, 28, 29, }, // Novosibirsk
    {  484, 1418,  599363,  612017,  612038, 12, 21, 22, }, // Omsk
    {  484, 1428,  612060,  612076,  612101, 16, 25, 26, }, // Pakistan
    {  484, 1437,       0,   58134,       0,  0, 13,  0, }, // Palau
    {  484, 1443,       0,  612127,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  484, 1460,  612153,  612169,  612194, 16, 25, 26, }, // Paraguay
    {  484, 1469,  612220,  612232,  612253, 12, 21, 22, }, // Peru
    {  484, 1474,  612275,  612292,  612318, 17, 26, 27, }, // Philippines
    {  484, 1486,       0,  612345,       0,  0, 21,  0, }, // Phoenix_Islands
    {  484, 1502,  612366,  612394,  612431, 28, 37, 38, }, // Pierre_Miquelon
    {  484, 1518,       0,   58505,       0,  0, 16,  0, }, // Pitcairn
    {  484, 1527,       0,  612469,       0,  0, 15,  0, }, // Ponape
    {  484, 1534,       0,  612484,       0,  0, 16,  0, }, // Pyongyang
    {  484, 1554,       0,  612500,       0,  0, 15,  0, }, // Reunion
    {  484, 1562,       0,   58636,       0,  0, 15,  0, }, // Rothera
    {  484, 1570,  612515,  612531,  612556, 16, 25, 26, }, // Sakhalin
    {  484, 1596,   58774,  612582,  612604, 13, 22, 23, }, // Samoa
    {  484, 1602,       0,  612627,       0,  0, 18,  0, }, // Seychelles
    {  484, 1624,       0,  612645,       0,  0, 25,  0, }, // Singapore
    {  484, 1634,       0,  612670,       0,  0, 21,  0, }, // Solomon
    {  484, 1642,       0,  612691,       0,  0, 23,  0, }, // South_Georgia
    {  484, 1656,       0,  139199,       0,  0, 15,  0, }, // Suriname
    {  484, 1676,       0,   58940,       0,  0, 13,  0, }, // Syowa
    {  484, 1682,       0,   58953,       0,  0, 14,  0, }, // Tahiti
    {  484, 1689,   58967,  612714,  612737, 14, 23, 24, }, // Taipei
    {  484, 1696,       0,  612761,       0,  0, 18,  0, }, // Tajikistan
    {  484, 1724,       0,   59047,       0,  0, 15,  0, }, // Tokelau
    {  484, 1732,   59062,  612779,  612801, 13, 22, 23, }, // Tonga
    {  484, 1738,       0,   59119,       0,  0, 13,  0, }, // Truk
    {  484, 1750,  612824,  612844,  612873, 20, 29, 30, }, // Turkmenistan
    {  484, 1763,       0,   59213,       0,  0, 14,  0, }, // Tuvalu
    {  484, 1777,  612903,  612918,  612942, 15, 24, 25, }, // Uruguay
    {  484, 1792,  612967,  612985,  613012, 18, 27, 28, }, // Uzbekistan
    {  484, 1803,   59371,  613040,  613064, 15, 24, 25, }, // Vanuatu
    {  484, 1811,       0,   59434,       0,  0, 17,  0, }, // Venezuela
    {  484, 1821,   59451,  613089,  613117, 19, 28, 29, }, // Vladivostok
    {  484, 1833,  600593,  613146,  613173, 18, 27, 28, }, // Volgograd
    {  484, 1843,       0,   59595,       0,  0, 14,  0, }, // Vostok
    {  484, 1850,       0,  613201,       0,  0, 17,  0, }, // Wake
    {  484, 1855,       0,   59630,       0,  0, 23,  0, }, // Wallis
    {  484, 1862,   59653,  613218,  613242, 15, 24, 25, }, // Yakutsk
    {  484, 1870,  613267,  613288,  613318, 21, 30, 31, }, // Yekaterinburg
    {  484, 1892,       0,  613349,       0,  0,  9,  0, }, // Yukon
    {  485,  422,       0,    1595,       0,  0, 12,  0, }, // Bolivia Quechua/Latin/Bolivia
    {  485, 1469,  612220,  612232,  612253, 12, 21, 22, }, // Peru
    {  486,  630,       0,  609596,       0,  0, 15,  0, }, // Ecuador Quechua/Latin/Ecuador
    {  486, 1469,  612220,  612232,  612253, 12, 21, 22, }, // Peru
    {  487,    0,  613358,  613366,  613383,  8, 17, 16, }, // Acre Romanian/Latin/Romania
    {  487,    5,       0,  613399,       0,  0, 18,  0, }, // Afghanistan
    {  487,   17,       0,  613417,       0,  0, 20,  0, }, // Africa_Central
    {  487,   32,       0,  613437,       0,  0, 21,  0, }, // Africa_Eastern
    {  487,   65,       0,  613458,       0,  0, 23,  0, }, // Africa_Southern
    {  487,   81,  613481,  613504,  613538, 23, 34, 33, }, // Africa_Western
    {  487,  107,  613571,  613585,  613608, 14, 23, 22, }, // Alaska
    {  487,  128,  613630,  613640,  613659, 10, 19, 18, }, // Almaty
    {  487,  135,  613677,  613691,  613716, 14, 25, 24, }, // Amazon
    {  487,  142,  613740,  613767,  613803, 27, 36, 35, }, // America_Central
    {  487,  158,  613838,  613866,  613903, 28, 37, 36, }, // America_Eastern
    {  487,  174,  613939,  613971,  614014, 32, 43, 42, }, // America_Mountain
    {  487,  191,  614056,  614088,  614131, 32, 43, 42, }, // America_Pacific
    {  487,  207,  614173,  614187,  614210, 14, 23, 22, }, // Anadyr
    {  487,  214,  614232,  614244,  614265, 12, 21, 20, }, // Apia
    {  487,  219,  614285,  614294,  614312,  9, 18, 25, }, // Aqtau
    {  487,  225,  614337,  614347,  614366, 10, 19, 26, }, // Aqtobe
    {  487,  232,  614392,  614401,  614419,  9, 18, 17, }, // Arabian
    {  487,  240,  614436,  614450,  614475, 14, 25, 24, }, // Argentina
    {  487,  250,  614499,  614525,  614562, 26, 37, 36, }, // Argentina_Western
    {  487,  268,  614598,  614610,  614633, 12, 23, 22, }, // Armenia
    {  487,  286,  614655,  614688,  614732, 33, 44, 43, }, // Atlantic
    {  487,  295,  614775,  614798,  614832, 23, 34, 33, }, // Australia_Central
    {  487,  313,  614865,  614899,  614944, 34, 45, 44, }, // Australia_CentralWestern
    {  487,  338,  614988,  615012,  615047, 24, 35, 34, }, // Australia_Eastern
    {  487,  356,  615081,  615107,  615144, 26, 37, 36, }, // Australia_Western
    {  487,  374,  615180,  615199,  615229, 19, 30, 29, }, // Azerbaijan
    {  487,  385,  615258,  615271,  615293, 13, 22, 21, }, // Azores
    {  487,  397,  615314,  615332,  615359, 18, 27, 26, }, // Bangladesh
    {  487,  415,       0,  615385,       0,  0, 14,  0, }, // Bhutan
    {  487,  422,       0,  615399,       0,  0, 12,  0, }, // Bolivia
    {  487,  437,  615411,  615424,  615448, 13, 24, 23, }, // Brasilia
    {  487,  454,       0,  615471,       0,  0, 25,  0, }, // Brunei
    {  487,  461,  615496,  615515,  615543, 19, 28, 27, }, // Cape_Verde
    {  487,  478,       0,  615570,       0,  0, 16,  0, }, // Chamorro
    {  487,  487,  615586,  615601,  615625, 15, 24, 23, }, // Chatham
    {  487,  495,  615648,  615661,  615683, 13, 22, 21, }, // Chile
    {  487,  501,  615704,  615714,  615735, 10, 21, 20, }, // China
    {  487,  507,  615755,  615773,  615800, 18, 27, 26, }, // Choibalsan
    {  487,  518,       0,  615826,       0,  0, 24,  0, }, // Christmas
    {  487,  528,       0,  615850,       0,  0, 19,  0, }, // Cocos
    {  487,  534,  615869,  615882,  615906, 13, 24, 23, }, // Colombia
    {  487,  543,  615929,  615947,  615976, 18, 29, 28, }, // Cook
    {  487,  548,  616004,  616013,  616033,  9, 20, 19, }, // Cuba
    {  487,  559,       0,  616052,       0,  0, 13,  0, }, // Davis
    {  487,  575,       0,  616065,       0,  0, 24,  0, }, // DumontDUrville
    {  487,  612,       0,  616089,       0,  0, 20,  0, }, // East_Timor
    {  487,  623,  616109,  616132,  616164, 23, 32, 31, }, // Easter
    {  487,  630,       0,  616195,       0,  0, 15,  0, }, // Ecuador
    {  487,  638,  616210,  616230,  616261, 20, 31, 30, }, // Europe_Central
    {  487,  653,  616291,  616309,  616338, 18, 29, 28, }, // Europe_Eastern
    {  487,  668,       0,  616366,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  487,  691,  616396,  616415,  616445, 19, 30, 29, }, // Europe_Western
    {  487,  706,  616474,  616499,  616533, 25, 34, 33, }, // Falkland
    {  487,  715,  616566,  616578,  616599, 12, 21, 20, }, // Fiji
    {  487,  720,       0,  616619,       0,  0, 23,  0, }, // French_Guiana
    {  487,  734,       0,  616642,       0,  0, 51,  0, }, // French_Southern
    {  487,  757,       0,  616693,       0,  0, 17,  0, }, // Galapagos
    {  487,  767,       0,  616710,       0,  0, 15,  0, }, // Gambier
    {  487,  775,  616725,  616737,  616760, 12, 23, 22, }, // Georgia
    {  487,  783,       0,  616782,       0,  0, 21,  0, }, // Gilbert_Islands
    {  487,  799,       0,  616803,       0,  0, 17,  0, }, // GMT
    {  487,  841,  616820,  616845,  616881, 25, 36, 35, }, // Greenland_Eastern
    {  487,  859,  616916,  616943,  616981, 27, 38, 37, }, // Greenland_Western
    {  487,  882,       0,  617018,       0,  0, 23,  0, }, // Gulf
    {  487,  887,       0,  616619,       0,  0, 14,  0, }, // Guyana
    {  487,  894,  617041,  617064,  617096, 23, 32, 31, }, // Hawaii_Aleutian
    {  487,  910,  617127,  617144,  617170, 17, 26, 25, }, // Hong_Kong
    {  487,  920,  617195,  617207,  617228, 12, 21, 20, }, // Hovd
    {  487,  925,       0,  617248,       0,  0, 10,  0, }, // India
    {  487,  931,       0,  617258,       0,  0, 20,  0, }, // Indian_Ocean
    {  487,  944,       0,  617278,       0,  0, 14,  0, }, // Indochina
    {  487,  954,       0,  617292,       0,  0, 23,  0, }, // Indonesia_Central
    {  487,  972,       0,  617315,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  487,  990,       0,  617336,       0,  0, 22,  0, }, // Indonesia_Western
    {  487, 1008,  617358,  617370,  617393, 12, 23, 22, }, // Iran
    {  487, 1019,  617415,  617429,  617452, 14, 23, 22, }, // Irkutsk
    {  487, 1027,  617474,  617488,  617513, 14, 25, 24, }, // Israel
    {  487, 1034,  617537,  617549,  617572, 12, 23, 22, }, // Japan
    {  487, 1040,  617594,  617625,  617665, 31, 40, 39, }, // Kamchatka
    {  487, 1058,       0,  617704,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  487, 1077,       0,  617730,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  487, 1106,  617757,  617767,  617788, 10, 21, 20, }, // Korea
    {  487, 1112,       0,  617808,       0,  0, 14,  0, }, // Kosrae
    {  487, 1119,  617822,  617841,  617869, 19, 28, 27, }, // Krasnoyarsk
    {  487, 1151,       0,  617896,       0,  0, 18,  0, }, // Kyrgystan
    {  487, 1175,       0,  617914,       0,  0, 21,  0, }, // Line_Islands
    {  487, 1188,  617935,  617952,  617978, 17, 26, 25, }, // Lord_Howe
    {  487, 1204,  618003,  618018,  618042, 15, 24, 23, }, // Magadan
    {  487, 1219,       0,  618065,       0,  0, 16,  0, }, // Malaysia
    {  487, 1228,       0,  618081,       0,  0, 15,  0, }, // Maldives
    {  487, 1237,       0,  618096,       0,  0, 22,  0, }, // Marquesas
    {  487, 1247,       0,  618118,       0,  0, 22,  0, }, // Marshall_Islands
    {  487, 1264,  618140,  618157,  618183, 17, 26, 25, }, // Mauritius
    {  487, 1274,       0,  618208,       0,  0, 14,  0, }, // Mawson
    {  487, 1281,  618222,  618248,  618285, 26, 37, 36, }, // Mexico_Pacific
    {  487, 1296,  618321,  618339,  618366, 18, 27, 26, }, // Mongolia
    {  487, 1305,  618392,  618404,  618427, 12, 23, 22, }, // Moscow
    {  487, 1312,       0,  618449,       0,  0, 15,  0, }, // Myanmar
    {  487, 1320,       0,  618464,       0,  0, 13,  0, }, // Nauru
    {  487, 1326,       0,  618477,       0,  0, 13,  0, }, // Nepal
    {  487, 1332,  618490,  618508,  618537, 18, 29, 28, }, // New_Caledonia
    {  487, 1346,  618565,  618582,  618610, 17, 28, 27, }, // New_Zealand
    {  487, 1358,  618637,  618657,  618686, 20, 29, 28, }, // Newfoundland
    {  487, 1371,       0,  618714,       0,  0, 12,  0, }, // Niue
    {  487, 1376,  618726,  618745,  618775, 19, 30, 29, }, // Norfolk
    {  487, 1384,  618804,  618831,  618867, 27, 36, 35, }, // Noronha
    {  487, 1406,  618902,  618921,  618949, 19, 28, 27, }, // Novosibirsk
    {  487, 1418,  618976,  618988,  619009, 12, 21, 20, }, // Omsk
    {  487, 1428,  619029,  619045,  619072, 16, 27, 26, }, // Pakistan
    {  487, 1437,       0,  619098,       0,  0, 13,  0, }, // Palau
    {  487, 1443,       0,  619111,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  487, 1460,  619136,  619152,  619177, 16, 25, 24, }, // Paraguay
    {  487, 1469,  619201,  619213,  619234, 12, 21, 20, }, // Peru
    {  487, 1474,  619254,  619270,  619295, 16, 25, 24, }, // Philippines
    {  487, 1486,       0,  619319,       0,  0, 21,  0, }, // Phoenix_Islands
    {  487, 1502,  619340,  619372,  619413, 32, 41, 40, }, // Pierre_Miquelon
    {  487, 1518,       0,  619453,       0,  0, 16,  0, }, // Pitcairn
    {  487, 1527,       0,  619469,       0,  0, 14,  0, }, // Ponape
    {  487, 1534,       0,  619483,       0,  0, 17,  0, }, // Pyongyang
    {  487, 1554,       0,  619500,       0,  0, 15,  0, }, // Reunion
    {  487, 1562,       0,  619515,       0,  0, 15,  0, }, // Rothera
    {  487, 1570,  619530,  619545,  619569, 15, 24, 23, }, // Sakhalin
    {  487, 1579,  619592,  619606,  619629, 14, 23, 22, }, // Samara
    {  487, 1596,  619651,  619664,  619686, 13, 22, 21, }, // Samoa
    {  487, 1602,       0,  619707,       0,  0, 18,  0, }, // Seychelles
    {  487, 1624,       0,  619725,       0,  0, 17,  0, }, // Singapore
    {  487, 1634,       0,  619742,       0,  0, 21,  0, }, // Solomon
    {  487, 1642,       0,  619763,       0,  0, 19,  0, }, // South_Georgia
    {  487, 1656,       0,  619782,       0,  0, 15,  0, }, // Suriname
    {  487, 1676,       0,  619797,       0,  0, 13,  0, }, // Syowa
    {  487, 1682,       0,  619810,       0,  0, 14,  0, }, // Tahiti
    {  487, 1689,  619824,  619838,  619861, 14, 23, 22, }, // Taipei
    {  487, 1696,       0,  619883,       0,  0, 19,  0, }, // Tajikistan
    {  487, 1724,       0,  619902,       0,  0, 15,  0, }, // Tokelau
    {  487, 1732,  619917,  619930,  619952, 13, 22, 21, }, // Tonga
    {  487, 1738,       0,  619973,       0,  0, 13,  0, }, // Truk
    {  487, 1750,  619986,  620006,  620035, 20, 29, 28, }, // Turkmenistan
    {  487, 1763,       0,  620063,       0,  0, 14,  0, }, // Tuvalu
    {  487, 1777,  620077,  620092,  620118, 15, 26, 25, }, // Uruguay
    {  487, 1792,  620143,  620161,  620188, 18, 27, 26, }, // Uzbekistan
    {  487, 1803,  620214,  620229,  620253, 15, 24, 23, }, // Vanuatu
    {  487, 1811,       0,  620276,       0,  0, 14,  0, }, // Venezuela
    {  487, 1821,  620290,  620309,  620337, 19, 28, 27, }, // Vladivostok
    {  487, 1833,  620364,  620381,  620407, 17, 26, 25, }, // Volgograd
    {  487, 1843,       0,  620432,       0,  0, 14,  0, }, // Vostok
    {  487, 1850,       0,  620446,       0,  0, 16,  0, }, // Wake
    {  487, 1855,       0,  620462,       0,  0, 24,  0, }, // Wallis
    {  487, 1862,  620486,  620500,  620523, 14, 23, 22, }, // Yakutsk
    {  487, 1870,  620545,  620565,  620594, 20, 29, 28, }, // Yekaterinburg
    {  487, 1892,       0,  620622,       0,  0, 13,  0, }, // Yukon
    {  489,  142,  620635,  620647,  620671, 12, 24, 20, }, // America_Central Romansh/Latin/Switzerland
    {  489,  158,  620691,  620704,  620729, 13, 25, 21, }, // America_Eastern
    {  489,  174,  620750,  620766,  620794, 16, 28, 24, }, // America_Mountain
    {  489,  191,  620818,  620830,  620854, 12, 24, 20, }, // America_Pacific
    {  489,  286,  620874,  620887,  620912, 13, 25, 21, }, // Atlantic
    {  489,  638,  620933,  620958,  620995, 25, 37, 33, }, // Europe_Central
    {  489,  653,  621028,  621054,  621092, 26, 38, 34, }, // Europe_Eastern
    {  489,  691,  621126,  621151,  621188, 25, 37, 33, }, // Europe_Western
    {  489,  799,       0,  621221,       0,  0, 14,  0, }, // GMT
    {  492,    0,  621235,  621245,  621267, 10, 22, 17, }, // Acre Russian/Cyrillic/Russia
    {  492,    5,       0,    6439,       0,  0, 10,  0, }, // Afghanistan
    {  492,   17,       0,  621284,       0,  0, 18,  0, }, // Africa_Central
    {  492,   32,       0,  621302,       0,  0, 16,  0, }, // Africa_Eastern
    {  492,   65,       0,  621318,       0,  0, 12,  0, }, // Africa_Southern
    {  492,   81,  621330,  621345,  621379, 15, 34, 29, }, // Africa_Western
    {  492,  107,  115019,  621408,  621433,  6, 25, 20, }, // Alaska
    {  492,  128,  621453,  621467,  621493, 14, 26, 21, }, // Almaty
    {  492,  135,    6710,  621514,  621541,  8, 27, 22, }, // Amazon
    {  492,  142,  621563,  621582,  621620, 19, 38, 33, }, // America_Central
    {  492,  158,  621653,  621670,  621706, 17, 36, 31, }, // America_Eastern
    {  492,  174,  621737,  621768,  621811, 31, 43, 38, }, // America_Mountain
    {  492,  191,  621849,  621868,  621899, 19, 31, 26, }, // America_Pacific
    {  492,  207,  621925,  621941,  621966, 16, 25, 20, }, // Anadyr
    {  492,  214,    7200,  621986,  622009,  4, 23, 18, }, // Apia
    {  492,  219,  622027,  622038,  622062, 11, 24, 18, }, // Aqtau
    {  492,  225,  622080,  622092,  622116, 12, 24, 19, }, // Aqtobe
    {  492,  232,  622135,  622152,  622188, 17, 36, 31, }, // Arabian
    {  492,  240,    7438,  622219,  622247,  9, 28, 23, }, // Argentina
    {  492,  250,  622270,  622288,  622325, 18, 37, 32, }, // Argentina_Western
    {  492,  268,  414395,  622357,  622383,  7, 26, 21, }, // Armenia
    {  492,  286,  622404,  622423,  622454, 19, 31, 26, }, // Atlantic
    {  492,  295,  622480,  622501,  622541, 21, 40, 35, }, // Australia_Central
    {  492,  313,  622576,  622613,  622662, 37, 49, 44, }, // Australia_CentralWestern
    {  492,  338,  622706,  622725,  622763, 19, 38, 33, }, // Australia_Eastern
    {  492,  356,  622796,  622814,  622851, 18, 37, 32, }, // Australia_Western
    {  492,  374,   81570,  622883,  622913, 11, 30, 25, }, // Azerbaijan
    {  492,  385,  622938,  622951,  622983, 13, 32, 27, }, // Azores
    {  492,  397,    8323,  623010,  623038,  9, 28, 23, }, // Bangladesh
    {  492,  415,       0,    8384,       0,  0,  5,  0, }, // Bhutan
    {  492,  422,       0,  415103,       0,  0,  7,  0, }, // Bolivia
    {  492,  437,  415117,  623061,  623088,  8, 27, 22, }, // Brasilia
    {  492,  454,       0,  162586,       0,  0, 17,  0, }, // Brunei
    {  492,  461,    8471,  623110,  623139, 10, 29, 24, }, // Cape_Verde
    {  492,  472,       0,  623163,       0,  0,  5,  0, }, // Casey
    {  492,  478,       0,    8540,       0,  0,  7,  0, }, // Chamorro
    {  492,  487,    8547,  623168,  623192,  5, 24, 19, }, // Chatham
    {  492,  495,    8596,  623211,  623234,  4, 23, 18, }, // Chile
    {  492,  501,  117200,  623252,  623276,  5, 24, 19, }, // China
    {  492,  507,  117266,  623295,  623323,  9, 28, 23, }, // Choibalsan
    {  492,  518,       0,  623346,       0,  0, 13,  0, }, // Christmas
    {  492,  528,       0,  623359,       0,  0, 14,  0, }, // Cocos
    {  492,  534,  415566,  623373,  623400,  8, 27, 22, }, // Colombia
    {  492,  543,  623422,  623434,  623465, 12, 31, 30, }, // Cook
    {  492,  548,    8934,  623495,  623518,  4, 23, 18, }, // Cuba
    {  492,  559,       0,  117590,       0,  0,  6,  0, }, // Davis
    {  492,  575,       0,  163031,       0,  0, 14,  0, }, // DumontDUrville
    {  492,  612,       0,  623536,       0,  0, 15,  0, }, // East_Timor
    {  492,  623,  623551,  623560,  623588,  9, 28, 23, }, // Easter
    {  492,  630,       0,   82494,       0,  0,  7,  0, }, // Ecuador
    {  492,  638,  623611,  623629,  623666, 18, 37, 32, }, // Europe_Central
    {  492,  653,  623698,  623714,  623749, 16, 35, 30, }, // Europe_Eastern
    {  492,  668,       0,  623779,       0,  0, 16,  0, }, // Europe_Further_Eastern
    {  492,  691,  623795,  623810,  623844, 15, 34, 29, }, // Europe_Western
    {  492,  706,  623873,  623890,  623926, 17, 36, 31, }, // Falkland
    {  492,  715,  118145,  623957,  623981,  5, 24, 19, }, // Fiji
    {  492,  720,       0,  624000,       0,  0, 18,  0, }, // French_Guiana
    {  492,  734,       0,  624018,       0,  0, 45,  0, }, // French_Southern
    {  492,  757,       0,  624063,       0,  0, 18,  0, }, // Galapagos
    {  492,  767,       0,  163595,       0,  0,  6,  0, }, // Gambier
    {  492,  775,  416402,  624081,  624106,  6, 25, 20, }, // Georgia
    {  492,  783,       0,  624126,       0,  0, 13,  0, }, // Gilbert_Islands
    {  492,  799,       0,  624139,       0,  0, 25,  0, }, // GMT
    {  492,  841,  624164,  624184,  624222, 20, 38, 34, }, // Greenland_Eastern
    {  492,  859,  624256,  624275,  624313, 19, 38, 33, }, // Greenland_Western
    {  492,  877,       0,    9922,       0,  0,  4,  0, }, // Guam
    {  492,  882,       0,  624346,       0,  0, 16,  0, }, // Gulf
    {  492,  887,       0,  163887,       0,  0,  6,  0, }, // Guyana
    {  492,  894,  624362,  624386,  624422, 24, 36, 31, }, // Hawaii_Aleutian
    {  492,  910,  163970,  624453,  624479,  7, 26, 21, }, // Hong_Kong
    {  492,  920,   10086,  624500,  624523,  4, 23, 18, }, // Hovd
    {  492,  925,       0,  215902,       0,  0,  5,  0, }, // India
    {  492,  931,       0,  624541,       0,  0, 15,  0, }, // Indian_Ocean
    {  492,  944,       0,  118863,       0,  0,  9,  0, }, // Indochina
    {  492,  954,       0,  624556,       0,  0, 21,  0, }, // Indonesia_Central
    {  492,  972,       0,  624577,       0,  0, 19,  0, }, // Indonesia_Eastern
    {  492,  990,       0,  624596,       0,  0, 18,  0, }, // Indonesia_Western
    {  492, 1008,   10225,  624614,  624637,  4, 23, 18, }, // Iran
    {  492, 1019,   10271,  624655,  624681,  7, 26, 21, }, // Irkutsk
    {  492, 1027,   10326,  624702,  624728,  7, 26, 21, }, // Israel
    {  492, 1034,  215969,  624749,  624774,  6, 25, 20, }, // Japan
    {  492, 1040,  624794,  624818,  624861, 24, 43, 38, }, // Kamchatka
    {  492, 1058,       0,  624899,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  492, 1077,       0,  624918,       0,  0, 18,  0, }, // Kazakhstan_Western
    {  492, 1106,  417336,  624936,  624960,  5, 24, 19, }, // Korea
    {  492, 1112,       0,   10642,       0,  0,  6,  0, }, // Kosrae
    {  492, 1119,  119454,  624979,  625008, 10, 29, 24, }, // Krasnoyarsk
    {  492, 1151,       0,  625032,       0,  0,  8,  0, }, // Kyrgystan
    {  492, 1161,       0,   10723,       0,  0,  9,  0, }, // Lanka
    {  492, 1175,       0,  625040,       0,  0,  9,  0, }, // Line_Islands
    {  492, 1188,   84044,  625049,  625076,  8, 27, 22, }, // Lord_Howe
    {  492, 1198,   10803,  625098,  625122,  5, 24, 19, }, // Macau
    {  492, 1204,   10852,  625141,  625167,  7, 26, 21, }, // Magadan
    {  492, 1219,       0,  417628,       0,  0,  8,  0, }, // Malaysia
    {  492, 1228,       0,  625188,       0,  0,  8,  0, }, // Maldives
    {  492, 1237,       0,  625196,       0,  0, 15,  0, }, // Marquesas
    {  492, 1247,       0,  625211,       0,  0, 18,  0, }, // Marshall_Islands
    {  492, 1264,  178085,  625229,  625256,  8, 27, 22, }, // Mauritius
    {  492, 1274,       0,   11021,       0,  0,  6,  0, }, // Mawson
    {  492, 1281,  625278,  625310,  625354, 32, 44, 39, }, // Mexico_Pacific
    {  492, 1296,   11137,  625393,  625422, 10, 29, 24, }, // Mongolia
    {  492, 1305,   11201,  625446,  625471,  6, 25, 20, }, // Moscow
    {  492, 1312,       0,  164829,       0,  0,  6,  0, }, // Myanmar
    {  492, 1320,       0,   11259,       0,  0,  5,  0, }, // Nauru
    {  492, 1326,       0,   11264,       0,  0,  5,  0, }, // Nepal
    {  492, 1332,  625491,  625506,  625540, 15, 34, 29, }, // New_Caledonia
    {  492, 1346,  625569,  625583,  625616, 14, 33, 28, }, // New_Zealand
    {  492, 1358,  164976,  625644,  625675, 12, 31, 26, }, // Newfoundland
    {  492, 1371,       0,  165042,       0,  0,  4,  0, }, // Niue
    {  492, 1376,   11486,  625701,  625727,  7, 26, 21, }, // Norfolk
    {  492, 1384,  165090,  625748,  625786, 19, 38, 33, }, // Noronha
    {  492, 1392,       0,  625819,       0,  0, 24,  0, }, // North_Mariana
    {  492, 1406,   11659,  625843,  625873, 11, 30, 25, }, // Novosibirsk
    {  492, 1418,   11726,  625898,  625921,  4, 23, 18, }, // Omsk
    {  492, 1428,   11772,  625939,  625966,  8, 27, 22, }, // Pakistan
    {  492, 1437,       0,   11830,       0,  0,  5,  0, }, // Palau
    {  492, 1443,       0,  625988,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  492, 1460,  120772,  626008,  626035,  8, 27, 22, }, // Paraguay
    {  492, 1469,   11912,  626057,  626080,  4, 23, 18, }, // Peru
    {  492, 1474,  626098,  626107,  626135,  9, 28, 23, }, // Philippines
    {  492, 1486,       0,  626158,       0,  0, 11,  0, }, // Phoenix_Islands
    {  492, 1502,  626169,  626187,  626224, 18, 37, 32, }, // Pierre_Miquelon
    {  492, 1518,       0,  165584,       0,  0,  7,  0, }, // Pitcairn
    {  492, 1527,       0,   12126,       0,  0,  6,  0, }, // Ponape
    {  492, 1534,       0,   85459,       0,  0,  7,  0, }, // Pyongyang
    {  492, 1544,   12139,  626256,  626285, 10, 29, 24, }, // Qyzylorda
    {  492, 1554,       0,  165605,       0,  0,  7,  0, }, // Reunion
    {  492, 1562,       0,   12210,       0,  0,  6,  0, }, // Rothera
    {  492, 1570,   12216,  626309,  626335,  7, 26, 21, }, // Sakhalin
    {  492, 1579,  626356,  626370,  626397, 14, 27, 22, }, // Samara
    {  492, 1596,   12333,  626419,  626443,  5, 24, 19, }, // Samoa
    {  492, 1602,       0,  626462,       0,  0, 19,  0, }, // Seychelles
    {  492, 1624,       0,   12407,       0,  0,  8,  0, }, // Singapore
    {  492, 1634,       0,  626481,       0,  0, 18,  0, }, // Solomon
    {  492, 1642,       0,  626499,       0,  0, 13,  0, }, // South_Georgia
    {  492, 1656,       0,   12457,       0,  0,  7,  0, }, // Suriname
    {  492, 1676,       0,   85714,       0,  0,  4,  0, }, // Syowa
    {  492, 1682,       0,   12469,       0,  0,  5,  0, }, // Tahiti
    {  492, 1689,  165757,  626512,  626538,  7, 26, 21, }, // Taipei
    {  492, 1696,       0,  121452,       0,  0, 11,  0, }, // Tajikistan
    {  492, 1724,       0,   12536,       0,  0,  7,  0, }, // Tokelau
    {  492, 1732,   12543,  626559,  626583,  5, 24, 19, }, // Tonga
    {  492, 1738,       0,   12592,       0,  0,  4,  0, }, // Truk
    {  492, 1750,   12596,  626602,  626633, 12, 31, 26, }, // Turkmenistan
    {  492, 1763,       0,   12666,       0,  0,  6,  0, }, // Tuvalu
    {  492, 1777,   85932,  626659,  626685,  7, 26, 21, }, // Uruguay
    {  492, 1792,  108762,  626706,  626735, 10, 29, 24, }, // Uzbekistan
    {  492, 1803,   12794,  626759,  626785,  7, 26, 21, }, // Vanuatu
    {  492, 1811,       0,  166038,       0,  0,  9,  0, }, // Venezuela
    {  492, 1821,   12858,  626806,  626836, 11, 30, 25, }, // Vladivostok
    {  492, 1833,   12925,  626861,  626889,  9, 28, 23, }, // Volgograd
    {  492, 1843,       0,  109090,       0,  0,  6,  0, }, // Vostok
    {  492, 1850,       0,   86286,       0,  0,  4,  0, }, // Wake
    {  492, 1855,       0,  626912,       0,  0, 15,  0, }, // Wallis
    {  492, 1862,  122024,  626927,  626952,  6, 25, 20, }, // Yakutsk
    {  492, 1870,   13070,  626972,  627003, 12, 31, 26, }, // Yekaterinburg
    {  492, 1892,       0,  122178,       0,  0,  4,  0, }, // Yukon
    {  500,  232,  627029,  627039,  627055, 10, 16, 18, }, // Arabian Sakha/Cyrillic/Russia
    {  500,  268,  627073,  627084,  627101, 11, 17, 19, }, // Armenia
    {  500,  295,  627120,  627139,  627164, 19, 25, 27, }, // Australia_Central
    {  500,  338,  627191,  627210,  627235, 19, 25, 27, }, // Australia_Eastern
    {  500,  356,  627262,  627282,  627308, 20, 26, 28, }, // Australia_Western
    {  500,  501,  627336,  627346,  627362, 10, 16, 18, }, // China
    {  500,  507,  627380,  627394,  627414, 14, 20, 22, }, // Choibalsan
    {  500,  775,  627436,  627449,  627468, 13, 19, 21, }, // Georgia
    {  500,  799,       0,  198209,       0,  0,  3,  0, }, // GMT
    {  500,  925,       0,  627489,       0,  0, 18,  0, }, // India
    {  500, 1008,  627507,  627517,  627533, 10, 16, 18, }, // Iran
    {  500, 1034,  627551,  627564,  627583, 13, 19, 21, }, // Japan
    {  500, 1058,       0,  627604,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  500, 1077,       0,  627624,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  500, 1106,  627645,  627656,  627673, 11, 17, 19, }, // Korea
    {  500, 1119,  627692,  627709,  627732, 17, 23, 25, }, // Krasnoyarsk
    {  500, 1151,       0,  627757,       0,  0, 15,  0, }, // Kyrgystan
    {  500, 1204,  627772,  627785,  627804, 13, 19, 21, }, // Magadan
    {  500, 1296,  627825,  627841,  627863, 16, 22, 24, }, // Mongolia
    {  500, 1305,  627887,  627899,  627917, 12, 18, 20, }, // Moscow
    {  500, 1346,  627937,  627955,  627978, 18, 23, 25, }, // New_Zealand
    {  500, 1406,  628003,  628021,  628045, 18, 24, 26, }, // Novosibirsk
    {  500, 1418,  628071,  628082,  628099, 11, 17, 19, }, // Omsk
    {  500, 1428,  628118,  628132,  628152, 14, 20, 22, }, // Pakistan
    {  500, 1570,  628174,  628187,  628206, 13, 19, 21, }, // Sakhalin
    {  500, 1821,  628227,  628243,  628268, 16, 25, 27, }, // Vladivostok
    {  500, 1833,  628295,  628309,  628329, 14, 20, 22, }, // Volgograd
    {  500, 1862,  628351,  628366,  628387, 15, 21, 23, }, // Yakutsk
    {  500, 1870,  628410,  628427,  628450, 17, 23, 25, }, // Yekaterinburg
    {  504,  142,  628475,  628499,  628529, 24, 30, 29, }, // America_Central Sanskrit/Devanagari/India
    {  504,  158,  628558,  628583,  628614, 25, 31, 30, }, // America_Eastern
    {  504,  174,  628644,  628667,  628696, 23, 29, 28, }, // America_Mountain
    {  504,  191,  628724,  628754,  628790, 30, 36, 35, }, // America_Pacific
    {  504,  286,  628825,  628839,  628859, 14, 20, 19, }, // Atlantic
    {  504,  638,  628878,  628895,  628918, 17, 23, 25, }, // Europe_Central
    {  504,  653,  628943,  628961,  628985, 18, 24, 26, }, // Europe_Eastern
    {  504,  691,  629011,  629033,  629061, 22, 28, 30, }, // Europe_Western
    {  504,  799,       0,  629091,       0,  0, 17,  0, }, // GMT
    {  505,  107,  629108,  629120,  629142, 12, 22, 18, }, // Alaska Santali/Ol Chiki/India
    {  505,  135,  629160,  629171,  629191, 11, 20, 17, }, // Amazon
    {  505,  142,  629208,  629219,  629240, 11, 21, 17, }, // America_Central
    {  505,  158,  629257,  629268,  629289, 11, 21, 18, }, // America_Eastern
    {  505,  174,  629307,  629319,  629341, 12, 22, 18, }, // America_Mountain
    {  505,  191,  629359,  629372,  629395, 13, 23, 19, }, // America_Pacific
    {  505,  240,  629414,  629429,  629453, 15, 24, 21, }, // Argentina
    {  505,  250,  629474,  629494,  629524, 20, 30, 26, }, // Argentina_Western
    {  505,  286,  629550,  629563,  629586, 13, 23, 19, }, // Atlantic
    {  505,  548,  629605,  629615,  629631, 10, 16, 16, }, // Cuba
    {  505,  638,  629647,  629668,  629699, 21, 31, 27, }, // Europe_Central
    {  505,  653,  629726,  629746,  629776, 20, 30, 26, }, // Europe_Eastern
    {  505,  691,  629802,  629823,  629854, 21, 31, 27, }, // Europe_Western
    {  505,  799,       0,  629881,       0,  0, 16,  0, }, // GMT
    {  505,  841,  629897,  629917,  629943, 20, 26, 26, }, // Greenland_Eastern
    {  505,  859,  629969,  629988,  630013, 19, 25, 25, }, // Greenland_Western
    {  505,  894,  630038,  630059,  630086, 21, 27, 27, }, // Hawaii_Aleutian
    {  505, 1281,  630113,  630134,  630161, 21, 27, 27, }, // Mexico_Pacific
    {  505, 1358,  630188,  630207,  630232, 19, 25, 25, }, // Newfoundland
    {  505, 1502,  630257,  630285,  630319, 28, 34, 34, }, // Pierre_Miquelon
    {  505, 1892,       0,  630353,       0,  0, 10,  0, }, // Yukon
    {  507,    0,  630363,  630374,  630395, 11, 21, 18, }, // Acre Sardinian/Latin/Italy
    {  507,    5,       0,  630413,       0,  0, 20,  0, }, // Afghanistan
    {  507,   17,       0,  630433,       0,  0, 25,  0, }, // Africa_Central
    {  507,   32,       0,  630458,       0,  0, 25,  0, }, // Africa_Eastern
    {  507,   65,       0,  630483,       0,  0, 37,  0, }, // Africa_Southern
    {  507,   81,  630520,  630547,  630584, 27, 37, 34, }, // Africa_Western
    {  507,  107,  630618,  630633,  630658, 15, 25, 22, }, // Alaska
    {  507,  128,  630680,  630693,  630716, 13, 23, 20, }, // Almaty
    {  507,  135,  630736,  630754,  630782, 18, 28, 25, }, // Amazon
    {  507,  142,  630807,  630824,  630851, 17, 27, 24, }, // America_Central
    {  507,  158,  376577,  630875,  376620, 17, 27, 24, }, // America_Eastern
    {  507,  174,  630902,  630925,  630958, 23, 33, 30, }, // America_Mountain
    {  507,  191,  630988,  631011,  631044, 23, 33, 30, }, // America_Pacific
    {  507,  207,  631074,  631087,  631110, 13, 23, 20, }, // Anadyr
    {  507,  214,  631130,  631141,  631162, 11, 21, 18, }, // Apia
    {  507,  219,  631180,  631192,  631214, 12, 22, 19, }, // Aqtau
    {  507,  225,  631233,  631246,  631269, 13, 23, 20, }, // Aqtobe
    {  507,  232,  631289,  631298,  631317,  9, 19, 16, }, // Arabian
    {  507,  240,  631333,  631351,  631379, 18, 28, 25, }, // Argentina
    {  507,  250,  631404,  631434,  631474, 30, 40, 37, }, // Argentina_Western
    {  507,  268,  631511,  631527,  631553, 16, 26, 23, }, // Armenia
    {  507,  286,  631576,  631594,  631622, 18, 28, 25, }, // Atlantic
    {  507,  295,  631647,  631675,  631713, 28, 38, 35, }, // Australia_Central
    {  507,  313,  631748,  631786,  631834, 38, 48, 45, }, // Australia_CentralWestern
    {  507,  338,  631879,  631907,  631945, 28, 38, 35, }, // Australia_Eastern
    {  507,  356,  631980,  632010,  632050, 30, 40, 37, }, // Australia_Western
    {  507,  374,  632087,  632107,  632137, 20, 30, 27, }, // Azerbaijan
    {  507,  385,  632164,  632182,  632210, 18, 28, 25, }, // Azores
    {  507,  397,  632235,  632255,  632285, 20, 30, 27, }, // Bangladesh
    {  507,  415,       0,  632312,       0,  0, 16,  0, }, // Bhutan
    {  507,  422,       0,  632328,       0,  0, 17,  0, }, // Bolivia
    {  507,  437,  632345,  632360,  632385, 15, 25, 22, }, // Brasilia
    {  507,  454,       0,  632407,       0,  0, 16,  0, }, // Brunei
    {  507,  461,  632423,  632443,  632473, 20, 30, 27, }, // Cape_Verde
    {  507,  472,       0,  632500,       0,  0, 12,  0, }, // Casey
    {  507,  478,       0,  632512,       0,  0, 25,  0, }, // Chamorro
    {  507,  487,  632537,  632555,  632583, 18, 28, 25, }, // Chatham
    {  507,  495,  632608,  632623,  632648, 15, 25, 22, }, // Chile
    {  507,  501,  632670,  632685,  632710, 15, 25, 22, }, // China
    {  507,  507,  632732,  632749,  632776, 17, 27, 24, }, // Choibalsan
    {  507,  518,       0,  632800,       0,  0, 31,  0, }, // Christmas
    {  507,  528,       0,  632831,       0,  0, 23,  0, }, // Cocos
    {  507,  534,  632854,  632872,  632900, 18, 28, 25, }, // Colombia
    {  507,  543,  632925,  632947,  632979, 22, 32, 29, }, // Cook
    {  507,  548,  633008,  633019,  633040, 11, 21, 18, }, // Cuba
    {  507,  559,       0,  633058,       0,  0, 12,  0, }, // Davis
    {  507,  575,       0,  633070,       0,  0, 23,  0, }, // DumontDUrville
    {  507,  612,       0,  633093,       0,  0, 19,  0, }, // East_Timor
    {  507,  623,  633112,  633135,  633168, 23, 33, 30, }, // Easter
    {  507,  630,       0,  633198,       0,  0, 16,  0, }, // Ecuador
    {  507,  638,  633214,  633239,  633274, 25, 35, 32, }, // Europe_Central
    {  507,  653,  633306,  633331,  633366, 25, 35, 32, }, // Europe_Eastern
    {  507,  668,       0,  633398,       0,  0, 46,  0, }, // Europe_Further_Eastern
    {  507,  691,  633444,  633471,  633508, 27, 37, 34, }, // Europe_Western
    {  507,  706,  633542,  633568,  633604, 26, 36, 33, }, // Falkland
    {  507,  715,  633637,  633652,  633677, 15, 25, 22, }, // Fiji
    {  507,  720,       0,  633699,       0,  0, 26,  0, }, // French_Guiana
    {  507,  734,       0,  633725,       0,  0, 50,  0, }, // French_Southern
    {  507,  757,       0,  633775,       0,  0, 20,  0, }, // Galapagos
    {  507,  767,       0,  633795,       0,  0, 14,  0, }, // Gambier
    {  507,  775,  633809,  633826,  633853, 17, 27, 24, }, // Georgia
    {  507,  783,       0,  633877,       0,  0, 25,  0, }, // Gilbert_Islands
    {  507,  799,       0,  633902,       0,  0, 32,  0, }, // GMT
    {  507,  841,  633934,  633965,  634006, 31, 41, 38, }, // Greenland_Eastern
    {  507,  859,  634044,  634077,  634120, 33, 43, 40, }, // Greenland_Western
    {  507,  877,       0,  634160,       0,  0, 21,  0, }, // Guam
    {  507,  882,       0,  634181,       0,  0, 25,  0, }, // Gulf
    {  507,  887,       0,  634206,       0,  0, 16,  0, }, // Guyana
    {  507,  894,  634222,  634256,  634300, 34, 44, 41, }, // Hawaii_Aleutian
    {  507,  910,  634341,  634357,  634383, 16, 26, 23, }, // Hong_Kong
    {  507,  920,  634406,  634417,  634438, 11, 21, 18, }, // Hovd
    {  507,  925,       0,  634456,       0,  0, 24,  0, }, // India
    {  507,  931,       0,  634480,       0,  0, 24,  0, }, // Indian_Ocean
    {  507,  944,       0,  634504,       0,  0, 18,  0, }, // Indochina
    {  507,  954,       0,  634522,       0,  0, 28,  0, }, // Indonesia_Central
    {  507,  972,       0,  634550,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  507,  990,       0,  634578,       0,  0, 30,  0, }, // Indonesia_Western
    {  507, 1008,  634608,  634621,  634644, 13, 23, 20, }, // Iran
    {  507, 1019,  634664,  634678,  634702, 14, 24, 21, }, // Irkutsk
    {  507, 1027,  634723,  634737,  634761, 14, 24, 21, }, // Israel
    {  507, 1034,  634782,  634799,  634826, 17, 27, 24, }, // Japan
    {  507, 1040,  634850,  634881,  634922, 31, 41, 38, }, // Kamchatka
    {  507, 1058,       0,  634960,       0,  0, 30,  0, }, // Kazakhstan_Eastern
    {  507, 1077,       0,  634990,       0,  0, 32,  0, }, // Kazakhstan_Western
    {  507, 1106,  380459,  635022,  380490, 11, 21, 18, }, // Korea
    {  507, 1112,       0,  635043,       0,  0, 13,  0, }, // Kosrae
    {  507, 1119,  635056,  635074,  635102, 18, 28, 25, }, // Krasnoyarsk
    {  507, 1151,       0,  635127,       0,  0, 22,  0, }, // Kyrgystan
    {  507, 1161,       0,  635149,       0,  0, 12,  0, }, // Lanka
    {  507, 1175,       0,  635161,       0,  0, 29,  0, }, // Line_Islands
    {  507, 1188,  635190,  635206,  635232, 16, 26, 23, }, // Lord_Howe
    {  507, 1198,  635255,  635267,  635289, 12, 22, 19, }, // Macau
    {  507, 1204,  635308,  635322,  635346, 14, 24, 21, }, // Magadan
    {  507, 1219,       0,  635367,       0,  0, 17,  0, }, // Malaysia
    {  507, 1228,       0,  635384,       0,  0, 19,  0, }, // Maldives
    {  507, 1237,       0,  635403,       0,  0, 20,  0, }, // Marquesas
    {  507, 1247,       0,  635423,       0,  0, 26,  0, }, // Marshall_Islands
    {  507, 1264,  635449,  635470,  635501, 21, 31, 28, }, // Mauritius
    {  507, 1274,       0,  635529,       0,  0, 13,  0, }, // Mawson
    {  507, 1281,  635542,  635571,  635610, 29, 39, 36, }, // Mexico_Pacific
    {  507, 1296,  635646,  635663,  635690, 17, 27, 24, }, // Mongolia
    {  507, 1305,  635714,  635726,  635748, 12, 22, 19, }, // Moscow
    {  507, 1312,       0,  635767,       0,  0, 17,  0, }, // Myanmar
    {  507, 1320,       0,  635784,       0,  0, 12,  0, }, // Nauru
    {  507, 1326,       0,  635796,       0,  0, 15,  0, }, // Nepal
    {  507, 1332,  635811,  635834,  635867, 23, 33, 30, }, // New_Caledonia
    {  507, 1346,  635897,  635918,  635949, 21, 31, 28, }, // New_Zealand
    {  507, 1358,  635977,  635993,  636019, 16, 26, 23, }, // Newfoundland
    {  507, 1371,       0,  636042,       0,  0, 11,  0, }, // Niue
    {  507, 1376,  636053,  636075,  636107, 22, 32, 29, }, // Norfolk
    {  507, 1384,  636136,  636165,  636204, 29, 39, 36, }, // Noronha
    {  507, 1392,       0,  636240,       0,  0, 42,  0, }, // North_Mariana
    {  507, 1406,  636282,  636300,  636328, 18, 28, 25, }, // Novosibirsk
    {  507, 1418,  636353,  636364,  636385, 11, 21, 18, }, // Omsk
    {  507, 1428,  636403,  636421,  636449, 18, 28, 25, }, // Pakistan
    {  507, 1437,       0,  636474,       0,  0, 12,  0, }, // Palau
    {  507, 1443,       0,  636486,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  507, 1460,  636512,  636530,  636558, 18, 28, 25, }, // Paraguay
    {  507, 1469,  636583,  636597,  636621, 14, 24, 21, }, // Peru
    {  507, 1474,  636642,  636662,  636692, 20, 30, 27, }, // Philippines
    {  507, 1486,       0,  636719,       0,  0, 32,  0, }, // Phoenix_Islands
    {  507, 1502,  636751,  636781,  636821, 30, 40, 37, }, // Pierre_Miquelon
    {  507, 1518,       0,  636858,       0,  0, 19,  0, }, // Pitcairn
    {  507, 1527,       0,  636877,       0,  0, 14,  0, }, // Ponape
    {  507, 1534,       0,  636891,       0,  0, 16,  0, }, // Pyongyang
    {  507, 1544,  636907,  636923,  636949, 16, 26, 23, }, // Qyzylorda
    {  507, 1554,       0,  636972,       0,  0, 18,  0, }, // Reunion
    {  507, 1562,       0,  636990,       0,  0, 14,  0, }, // Rothera
    {  507, 1570,  637004,  637019,  637044, 15, 25, 22, }, // Sakhalin
    {  507, 1579,  637066,  637079,  637102, 13, 23, 20, }, // Samara
    {  507, 1596,  637122,  637138,  637164, 16, 26, 23, }, // Samoa
    {  507, 1602,       0,  637187,       0,  0, 21,  0, }, // Seychelles
    {  507, 1624,       0,  637208,       0,  0, 16,  0, }, // Singapore
    {  507, 1634,       0,  637224,       0,  0, 26,  0, }, // Solomon
    {  507, 1642,       0,  637250,       0,  0, 27,  0, }, // South_Georgia
    {  507, 1656,       0,  637277,       0,  0, 18,  0, }, // Suriname
    {  507, 1676,       0,  637295,       0,  0, 12,  0, }, // Syowa
    {  507, 1682,       0,  637307,       0,  0, 13,  0, }, // Tahiti
    {  507, 1689,  637320,  637333,  637356, 13, 23, 20, }, // Taipei
    {  507, 1696,       0,  637376,       0,  0, 20,  0, }, // Tajikistan
    {  507, 1724,       0,  637396,       0,  0, 17,  0, }, // Tokelau
    {  507, 1732,  637413,  637428,  637453, 15, 25, 22, }, // Tonga
    {  507, 1738,       0,  637475,       0,  0, 15,  0, }, // Truk
    {  507, 1750,  637490,  637512,  637544, 22, 32, 29, }, // Turkmenistan
    {  507, 1763,       0,  637573,       0,  0, 16,  0, }, // Tuvalu
    {  507, 1777,  637589,  637605,  637631, 16, 26, 23, }, // Uruguay
    {  507, 1792,  637654,  637673,  637702, 19, 29, 26, }, // Uzbekistan
    {  507, 1803,  637728,  637745,  637772, 17, 27, 24, }, // Vanuatu
    {  507, 1811,       0,  637796,       0,  0, 20,  0, }, // Venezuela
    {  507, 1821,  637816,  637834,  637862, 18, 28, 25, }, // Vladivostok
    {  507, 1833,  637887,  637903,  637929, 16, 26, 23, }, // Volgograd
    {  507, 1843,       0,  637952,       0,  0, 13,  0, }, // Vostok
    {  507, 1850,       0,  637965,       0,  0, 22,  0, }, // Wake
    {  507, 1855,       0,  637987,       0,  0, 22,  0, }, // Wallis
    {  507, 1862,  638009,  638023,  638047, 14, 24, 21, }, // Yakutsk
    {  507, 1870,  638068,  638088,  638118, 20, 30, 27, }, // Yekaterinburg
    {  507, 1892,       0,  638145,       0,  0, 15,  0, }, // Yukon
    {  509,    0,  101157,  101167,  101188, 10, 21, 27, }, // Acre Serbian/Cyrillic/Serbia
    {  509,    5,       0,  638160,       0,  0, 16,  0, }, // Afghanistan
    {  509,   17,       0,  638176,       0,  0, 23,  0, }, // Africa_Central
    {  509,   32,       0,  638199,       0,  0, 21,  0, }, // Africa_Eastern
    {  509,   65,       0,  638220,       0,  0, 19,  0, }, // Africa_Southern
    {  509,   81,  638239,  638260,  638292, 21, 32, 27, }, // Africa_Western
    {  509,  107,  101397,  638319,  638343,  6, 24, 19, }, // Alaska
    {  509,  128,  101456,  101468,  101491, 12, 23, 29, }, // Almaty
    {  509,  135,  638362,  638374,  638398, 12, 24, 19, }, // Amazon
    {  509,  142,  638417,  638448,  638490, 31, 42, 37, }, // America_Central
    {  509,  158,  638527,  638556,  638596, 29, 40, 35, }, // America_Eastern
    {  509,  174,  638631,  638662,  638704, 31, 42, 37, }, // America_Mountain
    {  509,  191,  638741,  638772,  638814, 31, 42, 37, }, // America_Pacific
    {  509,  207,  101890,  101902,  101925, 12, 23, 29, }, // Anadyr
    {  509,  214,  638851,  638862,  638885, 11, 23, 18, }, // Apia
    {  509,  219,  102019,  102032,  102056, 13, 24, 30, }, // Aqtau
    {  509,  225,  102086,  102099,  102123, 13, 24, 30, }, // Aqtobe
    {  509,  232,  638903,  638918,  638944, 15, 26, 21, }, // Arabian
    {  509,  240,  638965,  638980,  639007, 15, 27, 22, }, // Argentina
    {  509,  250,  639029,  639052,  639087, 23, 35, 30, }, // Argentina_Western
    {  509,  268,  639117,  639132,  639159, 15, 27, 22, }, // Armenia
    {  509,  286,  485479,  485494,  639181, 15, 26, 21, }, // Atlantic
    {  509,  295,  639202,  639230,  639269, 28, 39, 34, }, // Australia_Central
    {  509,  313,  639303,  639339,  639386, 36, 47, 42, }, // Australia_CentralWestern
    {  509,  338,  639428,  639454,  639491, 26, 37, 32, }, // Australia_Eastern
    {  509,  356,  639523,  639549,  639586, 26, 37, 32, }, // Australia_Western
    {  509,  374,  639618,  639634,  639662, 16, 28, 23, }, // Azerbaijan
    {  509,  385,  639685,  639696,  639719, 11, 23, 18, }, // Azores
    {  509,  397,  639737,  639752,  639779, 15, 27, 22, }, // Bangladesh
    {  509,  415,       0,  639801,       0,  0, 11,  0, }, // Bhutan
    {  509,  422,       0,  639812,       0,  0, 14,  0, }, // Bolivia
    {  509,  437,  639826,  639841,  639868, 15, 27, 22, }, // Brasilia
    {  509,  454,       0,  639890,       0,  0, 22,  0, }, // Brunei
    {  509,  461,  639912,  639936,  639972, 24, 36, 31, }, // Cape_Verde
    {  509,  478,       0,  640003,       0,  0, 12,  0, }, // Chamorro
    {  509,  487,  640015,  640026,  640049, 11, 23, 18, }, // Chatham
    {  509,  495,  640067,  640077,  640099, 10, 22, 17, }, // Chile
    {  509,  501,  640116,  640126,  640150, 10, 24, 17, }, // China
    {  509,  507,  640167,  640182,  640209, 15, 27, 22, }, // Choibalsan
    {  509,  518,       0,  640231,       0,  0, 20,  0, }, // Christmas
    {  509,  528,       0,  640251,       0,  0, 27,  0, }, // Cocos
    {  509,  534,  640278,  640293,  640320, 15, 27, 22, }, // Colombia
    {  509,  543,  640342,  640361,  640392, 19, 31, 31, }, // Cook
    {  509,  548,    8934,  640423,  640445,  4, 22, 17, }, // Cuba
    {  509,  559,       0,  640462,       0,  0, 12,  0, }, // Davis
    {  509,  575,       0,  640474,       0,  0, 19,  0, }, // DumontDUrville
    {  509,  612,       0,  640493,       0,  0, 19,  0, }, // East_Timor
    {  509,  623,  640512,  640532,  640564, 20, 32, 27, }, // Easter
    {  509,  630,       0,  640591,       0,  0, 13,  0, }, // Ecuador
    {  509,  638,  640604,  640624,  640655, 20, 31, 26, }, // Europe_Central
    {  509,  653,  487333,  487354,  640681, 21, 32, 27, }, // Europe_Eastern
    {  509,  668,       0,  640708,       0,  0, 25,  0, }, // Europe_Further_Eastern
    {  509,  691,  487433,  487454,  640733, 21, 32, 27, }, // Europe_Western
    {  509,  706,  640760,  640784,  640820, 24, 36, 31, }, // Falkland
    {  509,  715,  640851,  640861,  640883, 10, 22, 17, }, // Fiji
    {  509,  720,       0,  640900,       0,  0, 23,  0, }, // French_Guiana
    {  509,  734,       0,  487709,       0,  0, 35,  0, }, // French_Southern
    {  509,  757,       0,  640923,       0,  0, 15,  0, }, // Galapagos
    {  509,  767,       0,  640938,       0,  0, 13,  0, }, // Gambier
    {  509,  775,  640951,  640964,  640989, 13, 25, 20, }, // Georgia
    {  509,  783,       0,  641009,       0,  0, 20,  0, }, // Gilbert_Islands
    {  509,  799,       0,  641029,       0,  0, 23,  0, }, // GMT
    {  509,  841,  104879,  641052,  641086, 16, 34, 29, }, // Greenland_Eastern
    {  509,  859,  104977,  641115,  641149, 16, 34, 29, }, // Greenland_Western
    {  509,  877,       0,  105075,       0,  0, 21,  0, }, // Guam
    {  509,  882,       0,  641178,       0,  0, 14,  0, }, // Gulf
    {  509,  887,       0,  640910,       0,  0, 13,  0, }, // Guyana
    {  509,  894,  641192,  641215,  641249, 23, 34, 29, }, // Hawaii_Aleutian
    {  509,  910,  641278,  641293,  641320, 15, 27, 22, }, // Hong_Kong
    {  509,  920,  641342,  641352,  641374, 10, 22, 17, }, // Hovd
    {  509,  925,       0,  641391,       0,  0, 25,  0, }, // India
    {  509,  931,       0,  641416,       0,  0, 23,  0, }, // Indian_Ocean
    {  509,  944,       0,  641439,       0,  0, 14,  0, }, // Indochina
    {  509,  954,       0,  641453,       0,  0, 28,  0, }, // Indonesia_Central
    {  509,  972,       0,  641481,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  509,  990,       0,  641507,       0,  0, 26,  0, }, // Indonesia_Western
    {  509, 1008,  641533,  641543,  641565, 10, 22, 17, }, // Iran
    {  509, 1019,  641582,  641594,  641618, 12, 24, 19, }, // Irkutsk
    {  509, 1027,  119087,  641637,  641663, 15, 26, 21, }, // Israel
    {  509, 1034,  641684,  641698,  641723, 14, 25, 20, }, // Japan
    {  509, 1040,  105788,  105818,  641743, 30, 41, 47, }, // Kamchatka
    {  509, 1058,       0,  641790,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  509, 1077,       0,  641816,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  509, 1106,  641842,  641856,  641881, 14, 25, 20, }, // Korea
    {  509, 1112,       0,  641901,       0,  0, 11,  0, }, // Kosrae
    {  509, 1119,  641912,  641929,  641958, 17, 29, 24, }, // Krasnoyarsk
    {  509, 1151,       0,  641982,       0,  0, 15,  0, }, // Kyrgystan
    {  509, 1161,       0,  106149,       0,  0, 15,  0, }, // Lanka
    {  509, 1175,       0,  641997,       0,  0, 17,  0, }, // Line_Islands
    {  509, 1188,  642014,  642028,  642054, 14, 26, 21, }, // Lord_Howe
    {  509, 1198,  106257,  106268,  642075, 11, 22, 28, }, // Macau
    {  509, 1204,  642103,  642116,  642141, 13, 25, 20, }, // Magadan
    {  509, 1219,       0,  642161,       0,  0, 14,  0, }, // Malaysia
    {  509, 1228,       0,  642175,       0,  0, 13,  0, }, // Maldives
    {  509, 1237,       0,  642188,       0,  0, 12,  0, }, // Marquesas
    {  509, 1247,       0,  642200,       0,  0, 22,  0, }, // Marshall_Islands
    {  509, 1264,  642222,  642238,  642266, 16, 28, 23, }, // Mauritius
    {  509, 1274,       0,  642289,       0,  0, 11,  0, }, // Mawson
    {  509, 1281,  642300,  642316,  642350, 16, 34, 29, }, // Mexico_Pacific
    {  509, 1296,  642379,  642395,  642423, 16, 28, 23, }, // Mongolia
    {  509, 1305,  642446,  642458,  642482, 12, 24, 19, }, // Moscow
    {  509, 1312,       0,  642501,       0,  0, 14,  0, }, // Myanmar
    {  509, 1320,       0,  642515,       0,  0, 11,  0, }, // Nauru
    {  509, 1326,       0,  642526,       0,  0, 11,  0, }, // Nepal
    {  509, 1332,  642537,  642558,  642591, 21, 33, 28, }, // New_Caledonia
    {  509, 1346,  642619,  642636,  642665, 17, 29, 24, }, // New_Zealand
    {  509, 1358,  107021,  642689,  642718, 11, 29, 24, }, // Newfoundland
    {  509, 1371,       0,  642742,       0,  0, 10,  0, }, // Niue
    {  509, 1376,  642752,  642772,  642804, 20, 32, 27, }, // Norfolk
    {  509, 1384,  642831,  642855,  642891, 24, 36, 31, }, // Noronha
    {  509, 1392,       0,  107312,       0,  0, 31,  0, }, // North_Mariana
    {  509, 1406,  642922,  642939,  642968, 17, 29, 24, }, // Novosibirsk
    {  509, 1418,  642992,  643002,  643024, 10, 22, 17, }, // Omsk
    {  509, 1428,  643041,  643055,  643081, 14, 26, 21, }, // Pakistan
    {  509, 1437,       0,  643102,       0,  0, 11,  0, }, // Palau
    {  509, 1443,       0,  643113,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  509, 1460,  643137,  643151,  643177, 14, 26, 21, }, // Paraguay
    {  509, 1469,  643198,  643208,  643230, 10, 22, 17, }, // Peru
    {  509, 1474,  643247,  643261,  643287, 14, 26, 21, }, // Philippines
    {  509, 1486,       0,  643308,       0,  0, 19,  0, }, // Phoenix_Islands
    {  509, 1502,  107833,  643327,  643363, 18, 36, 31, }, // Pierre_Miquelon
    {  509, 1518,       0,  643394,       0,  0, 13,  0, }, // Pitcairn
    {  509, 1527,       0,  643407,       0,  0, 12,  0, }, // Ponape
    {  509, 1534,       0,  643419,       0,  0, 17,  0, }, // Pyongyang
    {  509, 1544,  107982,  107997,  108023, 15, 26, 32, }, // Qyzylorda
    {  509, 1554,       0,  643436,       0,  0, 13,  0, }, // Reunion
    {  509, 1562,       0,  643449,       0,  0, 12,  0, }, // Rothera
    {  509, 1570,  643461,  643474,  643499, 13, 25, 20, }, // Sakhalin
    {  509, 1579,  108155,  108167,  108190, 12, 23, 29, }, // Samara
    {  509, 1596,  643519,  643530,  643553, 11, 23, 18, }, // Samoa
    {  509, 1602,       0,  643571,       0,  0, 13,  0, }, // Seychelles
    {  509, 1624,       0,  643584,       0,  0, 26,  0, }, // Singapore
    {  509, 1634,       0,  643610,       0,  0, 23,  0, }, // Solomon
    {  509, 1642,       0,  643633,       0,  0, 19,  0, }, // South_Georgia
    {  509, 1656,       0,  643652,       0,  0, 13,  0, }, // Suriname
    {  509, 1676,       0,  643665,       0,  0, 10,  0, }, // Syowa
    {  509, 1682,       0,  643675,       0,  0, 12,  0, }, // Tahiti
    {  509, 1689,  643687,  643699,  643723, 12, 24, 19, }, // Taipei
    {  509, 1696,       0,  643742,       0,  0, 16,  0, }, // Tajikistan
    {  509, 1724,       0,  643758,       0,  0, 13,  0, }, // Tokelau
    {  509, 1732,  643771,  643782,  643805, 11, 23, 18, }, // Tonga
    {  509, 1738,       0,  643823,       0,  0, 10,  0, }, // Truk
    {  509, 1750,  643833,  643851,  643881, 18, 30, 25, }, // Turkmenistan
    {  509, 1763,       0,  643906,       0,  0, 12,  0, }, // Tuvalu
    {  509, 1777,  643918,  643931,  643956, 13, 25, 20, }, // Uruguay
    {  509, 1792,  643976,  643992,  644020, 16, 28, 23, }, // Uzbekistan
    {  509, 1803,  644043,  644056,  644081, 13, 25, 20, }, // Vanuatu
    {  509, 1811,       0,  644101,       0,  0, 15,  0, }, // Venezuela
    {  509, 1821,  644116,  644133,  644162, 17, 29, 24, }, // Vladivostok
    {  509, 1833,  644186,  644201,  644228, 15, 27, 22, }, // Volgograd
    {  509, 1843,       0,  644250,       0,  0, 12,  0, }, // Vostok
    {  509, 1850,       0,  644262,       0,  0, 17,  0, }, // Wake
    {  509, 1855,       0,  644279,       0,  0, 27,  0, }, // Wallis
    {  509, 1862,  644306,  644319,  644344, 13, 25, 20, }, // Yakutsk
    {  509, 1870,  644364,  644383,  644414, 19, 31, 26, }, // Yekaterinburg
    {  509, 1892,       0,  492035,       0,  0,  5,  0, }, // Yukon
    {  510,    5,       0,  101215,       0,  0, 18,  0, }, // Afghanistan Serbian/Cyrillic/Bosnia and Herzegovina
    {  510,   17,       0,  101233,       0,  0, 25,  0, }, // Africa_Central
    {  510,   32,       0,  101258,       0,  0, 23,  0, }, // Africa_Eastern
    {  510,   65,       0,  101281,       0,  0, 21,  0, }, // Africa_Southern
    {  510,   81,  101302,  101325,  644440, 23, 34, 29, }, // Africa_Western
    {  510,  107,  101397,  644469,  644495,  6, 26, 21, }, // Alaska
    {  510,  135,  101520,  644516,  644542, 14, 26, 21, }, // Amazon
    {  510,  142,  644563,  644597,  644642, 34, 45, 40, }, // America_Central
    {  510,  158,  644682,  644714,  644757, 32, 43, 38, }, // America_Eastern
    {  510,  174,  644795,  644829,  644874, 34, 45, 40, }, // America_Mountain
    {  510,  191,  644914,  644948,  644993, 34, 45, 40, }, // America_Pacific
    {  510,  214,  101954,  645033,  645058, 13, 25, 20, }, // Apia
    {  510,  232,  102153,  102170,  645078, 17, 28, 23, }, // Arabian
    {  510,  240,  102230,  645101,  645130, 17, 29, 24, }, // Argentina
    {  510,  250,  102307,  645154,  645191, 25, 37, 32, }, // Argentina_Western
    {  510,  268,  645223,  645240,  645269, 17, 29, 24, }, // Armenia
    {  510,  286,  645293,  645310,  645338, 17, 28, 23, }, // Atlantic
    {  510,  295,  102556,  102586,  645361, 30, 41, 36, }, // Australia_Central
    {  510,  313,  102672,  102710,  645397, 38, 49, 44, }, // Australia_CentralWestern
    {  510,  338,  102812,  102840,  645441, 28, 39, 34, }, // Australia_Eastern
    {  510,  356,  102922,  102950,  645475, 28, 39, 34, }, // Australia_Western
    {  510,  374,  103032,  645509,  645539, 18, 30, 25, }, // Azerbaijan
    {  510,  385,  103112,  645564,  645589, 13, 25, 20, }, // Azores
    {  510,  397,  103177,  645609,  645638, 17, 29, 24, }, // Bangladesh
    {  510,  415,       0,  103254,       0,  0, 13,  0, }, // Bhutan
    {  510,  422,       0,  103267,       0,  0, 16,  0, }, // Bolivia
    {  510,  437,  103283,  645662,  645691, 17, 29, 24, }, // Brasilia
    {  510,  454,       0,  645715,       0,  0, 24,  0, }, // Brunei
    {  510,  461,  645739,  645765,  645803, 26, 38, 33, }, // Cape_Verde
    {  510,  478,       0,  103467,       0,  0, 14,  0, }, // Chamorro
    {  510,  487,  103481,  645836,  645861, 13, 25, 20, }, // Chatham
    {  510,  495,  103546,  645881,  645905, 12, 24, 19, }, // Chile
    {  510,  501,  103608,  645924,  645950, 12, 26, 19, }, // China
    {  510,  507,  103670,  645969,  645998, 17, 29, 24, }, // Choibalsan
    {  510,  518,       0,  646022,       0,  0, 22,  0, }, // Christmas
    {  510,  528,       0,  646044,       0,  0, 32,  0, }, // Cocos
    {  510,  534,  103798,  646076,  646105, 17, 29, 24, }, // Colombia
    {  510,  543,  646129,  646150,  646183, 21, 33, 32, }, // Cook
    {  510,  548,    8934,  646215,  646239,  4, 24, 19, }, // Cuba
    {  510,  559,       0,  104031,       0,  0, 14,  0, }, // Davis
    {  510,  575,       0,  646258,       0,  0, 21,  0, }, // DumontDUrville
    {  510,  612,       0,  646279,       0,  0, 21,  0, }, // East_Timor
    {  510,  623,  104085,  646300,  646334, 22, 34, 29, }, // Easter
    {  510,  630,       0,  104177,       0,  0, 15,  0, }, // Ecuador
    {  510,  638,  646363,  646385,  646418, 22, 33, 28, }, // Europe_Central
    {  510,  653,  104284,  104307,  646446, 23, 34, 29, }, // Europe_Eastern
    {  510,  691,  104407,  104430,  646475, 23, 34, 29, }, // Europe_Western
    {  510,  706,  104502,  646504,  646542, 26, 38, 33, }, // Falkland
    {  510,  715,  104606,  646575,  646599, 12, 24, 19, }, // Fiji
    {  510,  720,       0,  104668,       0,  0, 25,  0, }, // French_Guiana
    {  510,  734,       0,  104693,       0,  0, 37,  0, }, // French_Southern
    {  510,  757,       0,  104730,       0,  0, 17,  0, }, // Galapagos
    {  510,  767,       0,  646618,       0,  0, 15,  0, }, // Gambier
    {  510,  775,  104763,  646633,  646660, 15, 27, 22, }, // Georgia
    {  510,  783,       0,  646682,       0,  0, 25,  0, }, // Gilbert_Islands
    {  510,  799,       0,  646707,       0,  0, 25,  0, }, // GMT
    {  510,  841,  104879,  646732,  646768, 16, 36, 31, }, // Greenland_Eastern
    {  510,  859,  104977,  646799,  646835, 16, 36, 31, }, // Greenland_Western
    {  510,  882,       0,  105096,       0,  0, 16,  0, }, // Gulf
    {  510,  887,       0,  104678,       0,  0, 15,  0, }, // Guyana
    {  510,  894,  105112,  105137,  646866, 25, 36, 31, }, // Hawaii_Aleutian
    {  510,  910,  105213,  646897,  646926, 17, 29, 24, }, // Hong_Kong
    {  510,  920,  105291,  646950,  646974, 12, 24, 19, }, // Hovd
    {  510,  925,       0,  105353,       0,  0, 27,  0, }, // India
    {  510,  931,       0,  105380,       0,  0, 25,  0, }, // Indian_Ocean
    {  510,  944,       0,  105405,       0,  0, 16,  0, }, // Indochina
    {  510,  954,       0,  105421,       0,  0, 30,  0, }, // Indonesia_Central
    {  510,  972,       0,  105451,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  510,  990,       0,  105479,       0,  0, 28,  0, }, // Indonesia_Western
    {  510, 1008,  105507,  646993,  647017, 12, 24, 19, }, // Iran
    {  510, 1019,  105569,  647036,  647062, 14, 26, 21, }, // Irkutsk
    {  510, 1027,  105637,  105654,  647083, 17, 28, 23, }, // Israel
    {  510, 1034,  105714,  105730,  647106, 16, 27, 22, }, // Japan
    {  510, 1058,       0,  105905,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  510, 1077,       0,  105933,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  510, 1106,  105961,  105977,  647128, 16, 27, 22, }, // Korea
    {  510, 1112,       0,  106035,       0,  0, 13,  0, }, // Kosrae
    {  510, 1119,  647150,  647169,  647200, 19, 31, 26, }, // Krasnoyarsk
    {  510, 1151,       0,  106132,       0,  0, 17,  0, }, // Kyrgystan
    {  510, 1175,       0,  647226,       0,  0, 23,  0, }, // Line_Islands
    {  510, 1188,  106183,  647249,  647277, 16, 28, 23, }, // Lord_Howe
    {  510, 1204,  106317,  647300,  647327, 15, 27, 22, }, // Magadan
    {  510, 1219,       0,  106387,       0,  0, 16,  0, }, // Malaysia
    {  510, 1228,       0,  106403,       0,  0, 15,  0, }, // Maldives
    {  510, 1237,       0,  106418,       0,  0, 14,  0, }, // Marquesas
    {  510, 1247,       0,  106432,       0,  0, 24,  0, }, // Marshall_Islands
    {  510, 1264,  106456,  647349,  647379, 18, 30, 25, }, // Mauritius
    {  510, 1274,       0,  106536,       0,  0, 13,  0, }, // Mawson
    {  510, 1281,  642300,  647404,  647440, 16, 36, 31, }, // Mexico_Pacific
    {  510, 1296,  106653,  647471,  647501, 18, 30, 26, }, // Mongolia
    {  510, 1305,  106733,  647527,  647553, 14, 26, 21, }, // Moscow
    {  510, 1312,       0,  647574,       0,  0, 15,  0, }, // Myanmar
    {  510, 1320,       0,  106817,       0,  0, 13,  0, }, // Nauru
    {  510, 1326,       0,  106830,       0,  0, 13,  0, }, // Nepal
    {  510, 1332,  106843,  647589,  647624, 23, 35, 30, }, // New_Caledonia
    {  510, 1346,  106938,  647654,  647685, 19, 31, 26, }, // New_Zealand
    {  510, 1358,  107021,  647711,  647742, 11, 31, 26, }, // Newfoundland
    {  510, 1371,       0,  647768,       0,  0, 13,  0, }, // Niue
    {  510, 1376,  647781,  647803,  647837, 22, 34, 29, }, // Norfolk
    {  510, 1384,  107208,  647866,  647904, 26, 38, 33, }, // Noronha
    {  510, 1406,  647937,  647956,  647987, 19, 31, 26, }, // Novosibirsk
    {  510, 1418,  107427,  648013,  648037, 12, 24, 19, }, // Omsk
    {  510, 1428,  107489,  648056,  648084, 16, 28, 23, }, // Pakistan
    {  510, 1437,       0,  107563,       0,  0, 13,  0, }, // Palau
    {  510, 1443,       0,  107576,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  510, 1460,  107602,  648107,  648135, 16, 28, 23, }, // Paraguay
    {  510, 1469,  107676,  648158,  648182, 12, 24, 19, }, // Peru
    {  510, 1474,  107738,  648201,  648229, 16, 28, 23, }, // Philippines
    {  510, 1486,       0,  107812,       0,  0, 21,  0, }, // Phoenix_Islands
    {  510, 1502,  107833,  648252,  648290, 18, 38, 33, }, // Pierre_Miquelon
    {  510, 1518,       0,  107936,       0,  0, 15,  0, }, // Pitcairn
    {  510, 1527,       0,  648323,       0,  0, 14,  0, }, // Ponape
    {  510, 1534,       0,  648337,       0,  0, 19,  0, }, // Pyongyang
    {  510, 1554,       0,  648356,       0,  0, 15,  0, }, // Reunion
    {  510, 1562,       0,  108070,       0,  0, 14,  0, }, // Rothera
    {  510, 1570,  108084,  648371,  648398, 15, 27, 22, }, // Sakhalin
    {  510, 1596,  108219,  648420,  648445, 13, 25, 20, }, // Samoa
    {  510, 1602,       0,  108284,       0,  0, 15,  0, }, // Seychelles
    {  510, 1624,       0,  648465,       0,  0, 28,  0, }, // Singapore
    {  510, 1634,       0,  108326,       0,  0, 25,  0, }, // Solomon
    {  510, 1642,       0,  108351,       0,  0, 21,  0, }, // South_Georgia
    {  510, 1656,       0,  108372,       0,  0, 15,  0, }, // Suriname
    {  510, 1676,       0,  108387,       0,  0, 12,  0, }, // Syowa
    {  510, 1682,       0,  108399,       0,  0, 14,  0, }, // Tahiti
    {  510, 1689,  108413,  648493,  648519, 14, 26, 21, }, // Taipei
    {  510, 1696,       0,  108481,       0,  0, 18,  0, }, // Tajikistan
    {  510, 1724,       0,  108499,       0,  0, 15,  0, }, // Tokelau
    {  510, 1732,  108514,  648540,  648565, 13, 25, 20, }, // Tonga
    {  510, 1738,       0,  648585,       0,  0, 11,  0, }, // Truk
    {  510, 1750,  108591,  648596,  648628, 20, 32, 27, }, // Turkmenistan
    {  510, 1763,       0,  108677,       0,  0, 14,  0, }, // Tuvalu
    {  510, 1777,  108691,  648655,  648682, 15, 27, 22, }, // Uruguay
    {  510, 1792,  108762,  648704,  648734, 18, 30, 25, }, // Uzbekistan
    {  510, 1803,  108842,  648759,  648786, 15, 27, 22, }, // Vanuatu
    {  510, 1811,       0,  108913,       0,  0, 17,  0, }, // Venezuela
    {  510, 1821,  108930,  648808,  648839, 19, 31, 26, }, // Vladivostok
    {  510, 1833,  109013,  648865,  648894, 17, 29, 24, }, // Volgograd
    {  510, 1843,       0,  109090,       0,  0, 14,  0, }, // Vostok
    {  510, 1850,       0,  648918,       0,  0, 19,  0, }, // Wake
    {  510, 1855,       0,  648937,       0,  0, 29,  0, }, // Wallis
    {  510, 1862,  109152,  648966,  648993, 15, 27, 22, }, // Yakutsk
    {  510, 1870,  109223,  649015,  649048, 21, 33, 28, }, // Yekaterinburg
    {  513,    0,  649076,  649086,  649107, 10, 21, 29, }, // Acre Serbian/Latin/Bosnia and Herzegovina
    {  513,    5,       0,  649136,       0,  0, 18,  0, }, // Afghanistan
    {  513,   17,       0,  649154,       0,  0, 25,  0, }, // Africa_Central
    {  513,   32,       0,  649179,       0,  0, 23,  0, }, // Africa_Eastern
    {  513,   65,       0,  649202,       0,  0, 21,  0, }, // Africa_Southern
    {  513,   81,  649223,  649246,  649280, 23, 34, 31, }, // Africa_Western
    {  513,  107,   92777,  649311,  649338,  7, 27, 24, }, // Alaska
    {  513,  128,  649362,  649374,  649397, 12, 23, 31, }, // Almaty
    {  513,  135,  649428,  649442,  649468, 14, 26, 23, }, // Amazon
    {  513,  142,   92987,   93021,  649491, 34, 45, 42, }, // America_Central
    {  513,  158,   93107,   93139,  649533, 32, 43, 40, }, // America_Eastern
    {  513,  174,   93221,   93255,  649573, 34, 45, 42, }, // America_Mountain
    {  513,  191,   93341,   93375,  649615, 34, 45, 41, }, // America_Pacific
    {  513,  207,   93461,   93473,   93496, 12, 23, 31, }, // Anadyr
    {  513,  214,  649656,  649669,  649694, 13, 25, 22, }, // Apia
    {  513,  219,  649716,  649729,  649753, 13, 24, 32, }, // Aqtau
    {  513,  225,  649785,  649798,  649822, 13, 24, 32, }, // Aqtobe
    {  513,  232,   93725,   93742,  649854, 17, 28, 25, }, // Arabian
    {  513,  240,  649879,  649896,  649925, 17, 29, 26, }, // Argentina
    {  513,  250,  649951,  649976,  650013, 25, 37, 34, }, // Argentina_Western
    {  513,  268,  650047,  650064,  650093, 17, 29, 26, }, // Armenia
    {  513,  286,  650119,  650136,  650164, 17, 28, 25, }, // Atlantic
    {  513,  295,  650189,  650219,  650260, 30, 41, 38, }, // Australia_Central
    {  513,  313,   94256,  650298,  650347, 38, 49, 46, }, // Australia_CentralWestern
    {  513,  338,  650393,  650421,  650460, 28, 39, 36, }, // Australia_Eastern
    {  513,  356,  650496,  650524,  650563, 28, 39, 36, }, // Australia_Western
    {  513,  374,  650599,  650618,  650649, 19, 31, 28, }, // Azerbaijan
    {  513,  385,  650677,  650690,  650715, 13, 25, 22, }, // Azores
    {  513,  397,  650737,  650754,  650783, 17, 29, 26, }, // Bangladesh
    {  513,  415,       0,  650809,       0,  0, 13,  0, }, // Bhutan
    {  513,  422,       0,  650822,       0,  0, 16,  0, }, // Bolivia
    {  513,  437,  650838,  650855,  650884, 17, 29, 26, }, // Brasilia
    {  513,  454,       0,  650910,       0,  0, 24,  0, }, // Brunei
    {  513,  461,  650934,  650960,  650998, 26, 38, 35, }, // Cape_Verde
    {  513,  478,       0,  651033,       0,  0, 14,  0, }, // Chamorro
    {  513,  487,  651047,  651060,  651085, 13, 25, 22, }, // Chatham
    {  513,  495,  651107,  651119,  651143, 12, 24, 21, }, // Chile
    {  513,  501,  651164,   95184,  651176, 12, 26, 21, }, // China
    {  513,  507,  651197,  651214,  651243, 17, 29, 26, }, // Choibalsan
    {  513,  518,       0,  651269,       0,  0, 22,  0, }, // Christmas
    {  513,  528,       0,  651291,       0,  0, 32,  0, }, // Cocos
    {  513,  534,  651323,  651340,  651369, 17, 29, 26, }, // Colombia
    {  513,  543,  651395,  651416,  651449, 21, 33, 34, }, // Cook
    {  513,  548,   15132,  651483,  651507,  4, 24, 21, }, // Cuba
    {  513,  559,       0,  651528,       0,  0, 14,  0, }, // Davis
    {  513,  575,       0,  651542,       0,  0, 21,  0, }, // DumontDUrville
    {  513,  612,       0,  651563,       0,  0, 21,  0, }, // East_Timor
    {  513,  623,  651584,  651607,  651642, 23, 35, 32, }, // Easter
    {  513,  630,       0,  651674,       0,  0, 15,  0, }, // Ecuador
    {  513,  638,  651689,  651712,  651746, 23, 34, 31, }, // Europe_Central
    {  513,  653,   95882,   95905,  651777, 23, 34, 31, }, // Europe_Eastern
    {  513,  668,       0,  651808,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  513,  691,   95998,   96021,  651834, 23, 34, 31, }, // Europe_Western
    {  513,  706,  651865,  651891,  651929, 26, 38, 35, }, // Falkland
    {  513,  715,  651964,  651977,  652002, 13, 25, 22, }, // Fiji
    {  513,  720,       0,  652024,       0,  0, 25,  0, }, // French_Guiana
    {  513,  734,       0,  652049,       0,  0, 37,  0, }, // French_Southern
    {  513,  757,       0,  652086,       0,  0, 17,  0, }, // Galapagos
    {  513,  767,       0,  652103,       0,  0, 15,  0, }, // Gambier
    {  513,  775,  652118,  652133,  652160, 15, 27, 24, }, // Georgia
    {  513,  783,       0,  652184,       0,  0, 25,  0, }, // Gilbert_Islands
    {  513,  799,       0,  652209,       0,  0, 26,  0, }, // GMT
    {  513,  841,  652235,  652251,  652287, 16, 36, 33, }, // Greenland_Eastern
    {  513,  859,  652320,  652336,  652372, 16, 36, 33, }, // Greenland_Western
    {  513,  877,       0,   96655,       0,  0, 21,  0, }, // Guam
    {  513,  882,       0,  652405,       0,  0, 16,  0, }, // Gulf
    {  513,  887,       0,  652034,       0,  0, 15,  0, }, // Guyana
    {  513,  894,  652421,  652446,  652482, 25, 36, 33, }, // Hawaii_Aleutian
    {  513,  910,  652515,  652532,  652561, 17, 29, 26, }, // Hong_Kong
    {  513,  920,  652587,  652599,  652623, 12, 24, 21, }, // Hovd
    {  513,  925,       0,   96954,       0,  0, 27,  0, }, // India
    {  513,  931,       0,  652644,       0,  0, 25,  0, }, // Indian_Ocean
    {  513,  944,       0,  652669,       0,  0, 16,  0, }, // Indochina
    {  513,  954,       0,  652685,       0,  0, 30,  0, }, // Indonesia_Central
    {  513,  972,       0,  652715,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  513,  990,       0,  652743,       0,  0, 28,  0, }, // Indonesia_Western
    {  513, 1008,  652771,  652783,  652807, 12, 24, 21, }, // Iran
    {  513, 1019,  652828,  652842,  652868, 14, 26, 23, }, // Irkutsk
    {  513, 1027,   97239,   97256,  652891, 17, 28, 25, }, // Israel
    {  513, 1034,   97308,   97324,  652916, 16, 27, 24, }, // Japan
    {  513, 1040,  652940,  652970,  653011, 30, 41, 49, }, // Kamchatka
    {  513, 1058,       0,  653060,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  513, 1077,       0,  653088,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  513, 1106,   97545,   97561,  653116, 16, 27, 24, }, // Korea
    {  513, 1112,       0,  653140,       0,  0, 13,  0, }, // Kosrae
    {  513, 1119,  653153,  653172,  653203, 19, 31, 28, }, // Krasnoyarsk
    {  513, 1151,       0,  653231,       0,  0, 17,  0, }, // Kyrgystan
    {  513, 1161,       0,  653248,       0,  0, 15,  0, }, // Lanka
    {  513, 1175,       0,  653263,       0,  0, 23,  0, }, // Line_Islands
    {  513, 1188,  653286,  653302,  653330, 16, 28, 25, }, // Lord_Howe
    {  513, 1198,   97864,   97875,   97897, 11, 22, 30, }, // Macau
    {  513, 1204,  653355,  653370,  653397, 15, 27, 24, }, // Magadan
    {  513, 1219,       0,  653421,       0,  0, 16,  0, }, // Malaysia
    {  513, 1228,       0,  653437,       0,  0, 15,  0, }, // Maldives
    {  513, 1237,       0,  653452,       0,  0, 14,  0, }, // Marquesas
    {  513, 1247,       0,  653466,       0,  0, 24,  0, }, // Marshall_Islands
    {  513, 1264,  653490,  653508,  653538, 18, 30, 27, }, // Mauritius
    {  513, 1274,       0,  653565,       0,  0, 13,  0, }, // Mawson
    {  513, 1281,  653578,  653594,  653630, 16, 36, 33, }, // Mexico_Pacific
    {  513, 1296,  653663,  653681,  653711, 18, 30, 28, }, // Mongolia
    {  513, 1305,  653739,  653753,  653779, 14, 26, 23, }, // Moscow
    {  513, 1312,       0,  653802,       0,  0, 15,  0, }, // Myanmar
    {  513, 1320,       0,  653817,       0,  0, 13,  0, }, // Nauru
    {  513, 1326,       0,  653830,       0,  0, 13,  0, }, // Nepal
    {  513, 1332,  653843,  653866,  653901, 23, 35, 32, }, // New_Caledonia
    {  513, 1346,  653933,  653952,  653983, 19, 31, 28, }, // New_Zealand
    {  513, 1358,   24341,  654011,  654043, 12, 32, 29, }, // Newfoundland
    {  513, 1371,       0,  654072,       0,  0, 13,  0, }, // Niue
    {  513, 1376,  654085,  654107,  654141, 22, 34, 31, }, // Norfolk
    {  513, 1384,  654172,  654199,  654238, 27, 39, 36, }, // Noronha
    {  513, 1392,       0,   98966,       0,  0, 31,  0, }, // North_Mariana
    {  513, 1406,  654274,  654293,  654324, 19, 31, 28, }, // Novosibirsk
    {  513, 1418,  654352,  654364,  654388, 12, 24, 21, }, // Omsk
    {  513, 1428,  654409,  654425,  654453, 16, 28, 25, }, // Pakistan
    {  513, 1437,       0,  654478,       0,  0, 13,  0, }, // Palau
    {  513, 1443,       0,  654491,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  513, 1460,  654517,  654533,  654561, 16, 28, 25, }, // Paraguay
    {  513, 1469,  654586,  654598,  654622, 12, 24, 21, }, // Peru
    {  513, 1474,  654643,  654659,  654687, 16, 28, 25, }, // Philippines
    {  513, 1486,       0,  654712,       0,  0, 21,  0, }, // Phoenix_Islands
    {  513, 1502,  654733,  654751,  654789, 18, 38, 35, }, // Pierre_Miquelon
    {  513, 1518,       0,  654824,       0,  0, 15,  0, }, // Pitcairn
    {  513, 1527,       0,  654839,       0,  0, 14,  0, }, // Ponape
    {  513, 1534,       0,   99694,       0,  0, 19,  0, }, // Pyongyang
    {  513, 1544,   99713,   99728,   99754, 15, 26, 34, }, // Qyzylorda
    {  513, 1554,       0,  654853,       0,  0, 15,  0, }, // Reunion
    {  513, 1562,       0,  654868,       0,  0, 14,  0, }, // Rothera
    {  513, 1570,  654882,  654897,  654924, 15, 27, 24, }, // Sakhalin
    {  513, 1579,   99901,   99913,   99936, 12, 23, 31, }, // Samara
    {  513, 1596,  654948,  654961,  654986, 13, 25, 22, }, // Samoa
    {  513, 1602,       0,  655008,       0,  0, 15,  0, }, // Seychelles
    {  513, 1624,       0,  655023,       0,  0, 28,  0, }, // Singapore
    {  513, 1634,       0,  655051,       0,  0, 25,  0, }, // Solomon
    {  513, 1642,       0,  655076,       0,  0, 23,  0, }, // South_Georgia
    {  513, 1656,       0,  655099,       0,  0, 15,  0, }, // Suriname
    {  513, 1676,       0,  655114,       0,  0, 12,  0, }, // Syowa
    {  513, 1682,       0,  655126,       0,  0, 14,  0, }, // Tahiti
    {  513, 1689,  655140,  655154,  655180, 14, 26, 23, }, // Taipei
    {  513, 1696,       0,  655203,       0,  0, 19,  0, }, // Tajikistan
    {  513, 1724,       0,  655222,       0,  0, 15,  0, }, // Tokelau
    {  513, 1732,  655237,  655250,  655275, 13, 25, 22, }, // Tonga
    {  513, 1738,       0,  655297,       0,  0, 11,  0, }, // Truk
    {  513, 1750,  655308,  655328,  655360, 20, 32, 29, }, // Turkmenistan
    {  513, 1763,       0,  655389,       0,  0, 14,  0, }, // Tuvalu
    {  513, 1777,  655403,  655418,  655445, 15, 27, 24, }, // Uruguay
    {  513, 1792,  655469,  655487,  655517, 18, 30, 27, }, // Uzbekistan
    {  513, 1803,  655544,  655559,  655586, 15, 27, 24, }, // Vanuatu
    {  513, 1811,       0,  655610,       0,  0, 17,  0, }, // Venezuela
    {  513, 1821,  655627,  655646,  655677, 19, 31, 28, }, // Vladivostok
    {  513, 1833,  655705,  655722,  655751, 17, 29, 26, }, // Volgograd
    {  513, 1843,       0,  655777,       0,  0, 14,  0, }, // Vostok
    {  513, 1850,       0,  655791,       0,  0, 19,  0, }, // Wake
    {  513, 1855,       0,  655810,       0,  0, 29,  0, }, // Wallis
    {  513, 1862,  655839,  655854,  655881, 15, 27, 24, }, // Yakutsk
    {  513, 1870,  655905,  655926,  655959, 21, 33, 30, }, // Yekaterinburg
    {  513, 1892,       0,   26429,       0,  0,  5,  0, }, // Yukon
    {  514,    0,  649076,  649086,  649107, 10, 21, 29, }, // Acre Serbian/Latin/Kosovo
    {  514,    5,       0,  655989,       0,  0, 16,  0, }, // Afghanistan
    {  514,   17,       0,  656005,       0,  0, 23,  0, }, // Africa_Central
    {  514,   32,       0,  656028,       0,  0, 21,  0, }, // Africa_Eastern
    {  514,   65,       0,  656049,       0,  0, 19,  0, }, // Africa_Southern
    {  514,   81,  656068,  656089,  656121, 21, 32, 28, }, // Africa_Western
    {  514,  107,   92777,  656149,  656174,  7, 25, 21, }, // Alaska
    {  514,  128,  649362,  649374,  649397, 12, 23, 31, }, // Almaty
    {  514,  135,  656195,  656207,  656231, 12, 24, 20, }, // Amazon
    {  514,  142,  656251,  656282,  656324, 31, 42, 38, }, // America_Central
    {  514,  158,  656362,  656391,  656431, 29, 40, 36, }, // America_Eastern
    {  514,  174,  656467,  656498,  656540, 31, 42, 38, }, // America_Mountain
    {  514,  191,  656578,  656609,  656651, 31, 42, 38, }, // America_Pacific
    {  514,  207,   93461,   93473,   93496, 12, 23, 31, }, // Anadyr
    {  514,  214,  656689,  656700,  656723, 11, 23, 19, }, // Apia
    {  514,  219,  649716,  649729,  649753, 13, 24, 32, }, // Aqtau
    {  514,  225,  649785,  649798,  649822, 13, 24, 32, }, // Aqtobe
    {  514,  232,  656742,  656757,  656783, 15, 26, 22, }, // Arabian
    {  514,  240,  656805,  656820,  656847, 15, 27, 23, }, // Argentina
    {  514,  250,  656870,  656893,  656928, 23, 35, 31, }, // Argentina_Western
    {  514,  268,  656959,  656974,  657001, 15, 27, 23, }, // Armenia
    {  514,  286,  657024,  657039,  657065, 15, 26, 22, }, // Atlantic
    {  514,  295,  657087,  657115,  657154, 28, 39, 35, }, // Australia_Central
    {  514,  313,  657189,  657225,  657272, 36, 47, 43, }, // Australia_CentralWestern
    {  514,  338,  657315,  657341,  657378, 26, 37, 33, }, // Australia_Eastern
    {  514,  356,  657411,  657437,  657474, 26, 37, 33, }, // Australia_Western
    {  514,  374,  657507,  657524,  657553, 17, 29, 25, }, // Azerbaijan
    {  514,  385,  657578,  657589,  657612, 11, 23, 19, }, // Azores
    {  514,  397,  657631,  657646,  657673, 15, 27, 23, }, // Bangladesh
    {  514,  415,       0,  657696,       0,  0, 11,  0, }, // Bhutan
    {  514,  422,       0,  657707,       0,  0, 14,  0, }, // Bolivia
    {  514,  437,  657721,  657736,  657763, 15, 27, 23, }, // Brasilia
    {  514,  454,       0,  657786,       0,  0, 22,  0, }, // Brunei
    {  514,  461,  657808,  657832,  657868, 24, 36, 32, }, // Cape_Verde
    {  514,  478,       0,  657900,       0,  0, 12,  0, }, // Chamorro
    {  514,  487,  657912,  657923,  657946, 11, 23, 19, }, // Chatham
    {  514,  495,  657965,  657975,  657997, 10, 22, 18, }, // Chile
    {  514,  501,  658015,  658025,  658049, 10, 24, 18, }, // China
    {  514,  507,  658067,  658082,  658109, 15, 27, 23, }, // Choibalsan
    {  514,  518,       0,  658132,       0,  0, 20,  0, }, // Christmas
    {  514,  528,       0,  658152,       0,  0, 27,  0, }, // Cocos
    {  514,  534,  658179,  658194,  658221, 15, 27, 23, }, // Colombia
    {  514,  543,  658244,  658263,  658294, 19, 31, 32, }, // Cook
    {  514,  548,   15132,  658326,  658348,  4, 22, 18, }, // Cuba
    {  514,  559,       0,  658366,       0,  0, 12,  0, }, // Davis
    {  514,  575,       0,  658378,       0,  0, 19,  0, }, // DumontDUrville
    {  514,  612,       0,  658397,       0,  0, 19,  0, }, // East_Timor
    {  514,  623,  658416,  658437,  658470, 21, 33, 29, }, // Easter
    {  514,  630,       0,  658499,       0,  0, 13,  0, }, // Ecuador
    {  514,  638,  658512,  658533,  658565, 21, 32, 28, }, // Europe_Central
    {  514,  653,  658593,  658614,  658646, 21, 32, 28, }, // Europe_Eastern
    {  514,  668,       0,  651808,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  514,  691,  658674,  658695,  658727, 21, 32, 28, }, // Europe_Western
    {  514,  706,  658755,  658779,  658815, 24, 36, 32, }, // Falkland
    {  514,  715,  658847,  658858,  658881, 11, 23, 19, }, // Fiji
    {  514,  720,       0,  658900,       0,  0, 23,  0, }, // French_Guiana
    {  514,  734,       0,  658923,       0,  0, 35,  0, }, // French_Southern
    {  514,  757,       0,  658958,       0,  0, 15,  0, }, // Galapagos
    {  514,  767,       0,  658973,       0,  0, 13,  0, }, // Gambier
    {  514,  775,  658986,  658999,  659024, 13, 25, 21, }, // Georgia
    {  514,  783,       0,  659045,       0,  0, 20,  0, }, // Gilbert_Islands
    {  514,  799,       0,  659065,       0,  0, 24,  0, }, // GMT
    {  514,  841,  652235,  659089,  659123, 16, 34, 30, }, // Greenland_Eastern
    {  514,  859,  652320,  659153,  659187, 16, 34, 30, }, // Greenland_Western
    {  514,  877,       0,   96655,       0,  0, 21,  0, }, // Guam
    {  514,  882,       0,  659217,       0,  0, 14,  0, }, // Gulf
    {  514,  887,       0,  658910,       0,  0, 13,  0, }, // Guyana
    {  514,  894,  659231,  659254,  659288, 23, 34, 30, }, // Hawaii_Aleutian
    {  514,  910,  659318,  659333,  659360, 15, 27, 23, }, // Hong_Kong
    {  514,  920,  659383,  659393,  659415, 10, 22, 18, }, // Hovd
    {  514,  925,       0,  659433,       0,  0, 25,  0, }, // India
    {  514,  931,       0,  659458,       0,  0, 23,  0, }, // Indian_Ocean
    {  514,  944,       0,  659481,       0,  0, 14,  0, }, // Indochina
    {  514,  954,       0,  659495,       0,  0, 28,  0, }, // Indonesia_Central
    {  514,  972,       0,  659523,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  514,  990,       0,  659549,       0,  0, 26,  0, }, // Indonesia_Western
    {  514, 1008,  659575,  659585,  659607, 10, 22, 18, }, // Iran
    {  514, 1019,  659625,  659637,  659661, 12, 24, 20, }, // Irkutsk
    {  514, 1027,  659681,  659696,  659722, 15, 26, 22, }, // Israel
    {  514, 1034,  659744,  659758,  659783, 14, 25, 21, }, // Japan
    {  514, 1040,  652940,  652970,  653011, 30, 41, 49, }, // Kamchatka
    {  514, 1058,       0,  659804,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  514, 1077,       0,  659830,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  514, 1106,  659856,  659870,  659895, 14, 25, 21, }, // Korea
    {  514, 1112,       0,  659916,       0,  0, 11,  0, }, // Kosrae
    {  514, 1119,  659927,  659944,  659973, 17, 29, 25, }, // Krasnoyarsk
    {  514, 1151,       0,  659998,       0,  0, 15,  0, }, // Kyrgystan
    {  514, 1161,       0,  653248,       0,  0, 15,  0, }, // Lanka
    {  514, 1175,       0,  660013,       0,  0, 17,  0, }, // Line_Islands
    {  514, 1188,  660030,  660044,  660070, 14, 26, 22, }, // Lord_Howe
    {  514, 1198,   97864,   97875,   97897, 11, 22, 30, }, // Macau
    {  514, 1204,  660092,  660105,  660130, 13, 25, 21, }, // Magadan
    {  514, 1219,       0,  660151,       0,  0, 14,  0, }, // Malaysia
    {  514, 1228,       0,  660165,       0,  0, 13,  0, }, // Maldives
    {  514, 1237,       0,  660178,       0,  0, 12,  0, }, // Marquesas
    {  514, 1247,       0,  660190,       0,  0, 22,  0, }, // Marshall_Islands
    {  514, 1264,  660212,  660228,  660256, 16, 28, 24, }, // Mauritius
    {  514, 1274,       0,  660280,       0,  0, 11,  0, }, // Mawson
    {  514, 1281,  653578,  660291,  660325, 16, 34, 30, }, // Mexico_Pacific
    {  514, 1296,  660355,  660371,  660399, 16, 28, 24, }, // Mongolia
    {  514, 1305,  660423,  660435,  660459, 12, 24, 20, }, // Moscow
    {  514, 1312,       0,  660479,       0,  0, 14,  0, }, // Myanmar
    {  514, 1320,       0,  660493,       0,  0, 11,  0, }, // Nauru
    {  514, 1326,       0,  660504,       0,  0, 11,  0, }, // Nepal
    {  514, 1332,  660515,  660536,  660569, 21, 33, 29, }, // New_Caledonia
    {  514, 1346,  660598,  660615,  660644, 17, 29, 25, }, // New_Zealand
    {  514, 1358,   24341,  660669,  660699, 12, 30, 26, }, // Newfoundland
    {  514, 1371,       0,  660725,       0,  0, 10,  0, }, // Niue
    {  514, 1376,  660735,  660755,  660787, 20, 32, 28, }, // Norfolk
    {  514, 1384,  660815,  660840,  660877, 25, 37, 33, }, // Noronha
    {  514, 1392,       0,   98966,       0,  0, 31,  0, }, // North_Mariana
    {  514, 1406,  660910,  660927,  660956, 17, 29, 25, }, // Novosibirsk
    {  514, 1418,  660981,  660991,  661013, 10, 22, 18, }, // Omsk
    {  514, 1428,  661031,  661045,  661071, 14, 26, 22, }, // Pakistan
    {  514, 1437,       0,  661093,       0,  0, 11,  0, }, // Palau
    {  514, 1443,       0,  661104,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  514, 1460,  661128,  661142,  661168, 14, 26, 22, }, // Paraguay
    {  514, 1469,  661190,  661200,  661222, 10, 22, 18, }, // Peru
    {  514, 1474,  661240,  661254,  661280, 14, 26, 22, }, // Philippines
    {  514, 1486,       0,  661302,       0,  0, 19,  0, }, // Phoenix_Islands
    {  514, 1502,  654733,  661321,  661357, 18, 36, 32, }, // Pierre_Miquelon
    {  514, 1518,       0,  661389,       0,  0, 13,  0, }, // Pitcairn
    {  514, 1527,       0,  661402,       0,  0, 12,  0, }, // Ponape
    {  514, 1534,       0,  661414,       0,  0, 17,  0, }, // Pyongyang
    {  514, 1544,   99713,   99728,   99754, 15, 26, 34, }, // Qyzylorda
    {  514, 1554,       0,  661431,       0,  0, 13,  0, }, // Reunion
    {  514, 1562,       0,  661444,       0,  0, 12,  0, }, // Rothera
    {  514, 1570,  661456,  661469,  661494, 13, 25, 21, }, // Sakhalin
    {  514, 1579,   99901,   99913,   99936, 12, 23, 31, }, // Samara
    {  514, 1596,  661515,  661526,  661549, 11, 23, 19, }, // Samoa
    {  514, 1602,       0,  661568,       0,  0, 13,  0, }, // Seychelles
    {  514, 1624,       0,  661581,       0,  0, 26,  0, }, // Singapore
    {  514, 1634,       0,  661607,       0,  0, 23,  0, }, // Solomon
    {  514, 1642,       0,  661630,       0,  0, 21,  0, }, // South_Georgia
    {  514, 1656,       0,  661651,       0,  0, 13,  0, }, // Suriname
    {  514, 1676,       0,  661664,       0,  0, 10,  0, }, // Syowa
    {  514, 1682,       0,  661674,       0,  0, 12,  0, }, // Tahiti
    {  514, 1689,  661686,  661698,  661722, 12, 24, 20, }, // Taipei
    {  514, 1696,       0,  661742,       0,  0, 17,  0, }, // Tajikistan
    {  514, 1724,       0,  661759,       0,  0, 13,  0, }, // Tokelau
    {  514, 1732,  661772,  661783,  661806, 11, 23, 19, }, // Tonga
    {  514, 1738,       0,  661825,       0,  0, 10,  0, }, // Truk
    {  514, 1750,  661835,  661853,  661883, 18, 30, 26, }, // Turkmenistan
    {  514, 1763,       0,  661909,       0,  0, 12,  0, }, // Tuvalu
    {  514, 1777,  661921,  661934,  661959, 13, 25, 21, }, // Uruguay
    {  514, 1792,  661980,  661996,  662024, 16, 28, 24, }, // Uzbekistan
    {  514, 1803,  662048,  662061,  662086, 13, 25, 21, }, // Vanuatu
    {  514, 1811,       0,  662107,       0,  0, 15,  0, }, // Venezuela
    {  514, 1821,  662122,  662139,  662168, 17, 29, 25, }, // Vladivostok
    {  514, 1833,  662193,  662208,  662235, 15, 27, 23, }, // Volgograd
    {  514, 1843,       0,  662258,       0,  0, 12,  0, }, // Vostok
    {  514, 1850,       0,  662270,       0,  0, 17,  0, }, // Wake
    {  514, 1855,       0,  662287,       0,  0, 27,  0, }, // Wallis
    {  514, 1862,  662314,  662327,  662352, 13, 25, 21, }, // Yakutsk
    {  514, 1870,  662373,  662392,  662423, 19, 31, 27, }, // Yekaterinburg
    {  514, 1892,       0,   26429,       0,  0,  5,  0, }, // Yukon
    {  515,    0,  649076,  649086,  649107, 10, 21, 29, }, // Acre Serbian/Latin/Montenegro
    {  515,    5,       0,  655989,       0,  0, 16,  0, }, // Afghanistan
    {  515,   17,       0,  656005,       0,  0, 23,  0, }, // Africa_Central
    {  515,   32,       0,  656028,       0,  0, 21,  0, }, // Africa_Eastern
    {  515,   65,       0,  656049,       0,  0, 19,  0, }, // Africa_Southern
    {  515,   81,  656068,  656089,  656121, 21, 32, 28, }, // Africa_Western
    {  515,  107,   92777,  656149,  656174,  7, 25, 21, }, // Alaska
    {  515,  128,  649362,  649374,  649397, 12, 23, 31, }, // Almaty
    {  515,  135,  656195,  656207,  656231, 12, 24, 20, }, // Amazon
    {  515,  142,  656251,  656282,  656324, 31, 42, 38, }, // America_Central
    {  515,  158,  656362,  656391,  656431, 29, 40, 36, }, // America_Eastern
    {  515,  174,  656467,  656498,  656540, 31, 42, 38, }, // America_Mountain
    {  515,  191,  656578,  656609,  656651, 31, 42, 38, }, // America_Pacific
    {  515,  207,   93461,   93473,   93496, 12, 23, 31, }, // Anadyr
    {  515,  214,  656689,  656700,  656723, 11, 23, 19, }, // Apia
    {  515,  219,  649716,  649729,  649753, 13, 24, 32, }, // Aqtau
    {  515,  225,  649785,  649798,  649822, 13, 24, 32, }, // Aqtobe
    {  515,  232,  656742,  656757,  656783, 15, 26, 22, }, // Arabian
    {  515,  240,  656805,  656820,  656847, 15, 27, 23, }, // Argentina
    {  515,  250,  656870,  656893,  656928, 23, 35, 31, }, // Argentina_Western
    {  515,  268,  656959,  656974,  657001, 15, 27, 23, }, // Armenia
    {  515,  286,  657024,  657039,  657065, 15, 26, 22, }, // Atlantic
    {  515,  295,  657087,  657115,  657154, 28, 39, 35, }, // Australia_Central
    {  515,  313,  657189,  657225,  657272, 36, 47, 43, }, // Australia_CentralWestern
    {  515,  338,  657315,  657341,  657378, 26, 37, 33, }, // Australia_Eastern
    {  515,  356,  657411,  657437,  657474, 26, 37, 33, }, // Australia_Western
    {  515,  374,  657507,  657524,  657553, 17, 29, 25, }, // Azerbaijan
    {  515,  385,  657578,  657589,  657612, 11, 23, 19, }, // Azores
    {  515,  397,  657631,  657646,  657673, 15, 27, 23, }, // Bangladesh
    {  515,  415,       0,  657696,       0,  0, 11,  0, }, // Bhutan
    {  515,  422,       0,  657707,       0,  0, 14,  0, }, // Bolivia
    {  515,  437,  657721,  657736,  657763, 15, 27, 23, }, // Brasilia
    {  515,  454,       0,  657786,       0,  0, 22,  0, }, // Brunei
    {  515,  461,  657808,  657832,  657868, 24, 36, 32, }, // Cape_Verde
    {  515,  478,       0,  657900,       0,  0, 12,  0, }, // Chamorro
    {  515,  487,  657912,  657923,  657946, 11, 23, 19, }, // Chatham
    {  515,  495,  657965,  657975,  657997, 10, 22, 18, }, // Chile
    {  515,  501,  658015,  658025,  658049, 10, 24, 18, }, // China
    {  515,  507,  658067,  658082,  658109, 15, 27, 23, }, // Choibalsan
    {  515,  518,       0,  658132,       0,  0, 20,  0, }, // Christmas
    {  515,  528,       0,  658152,       0,  0, 27,  0, }, // Cocos
    {  515,  534,  658179,  658194,  658221, 15, 27, 23, }, // Colombia
    {  515,  543,  658244,  658263,  658294, 19, 31, 32, }, // Cook
    {  515,  548,   15132,  658326,  658348,  4, 22, 18, }, // Cuba
    {  515,  559,       0,  658366,       0,  0, 12,  0, }, // Davis
    {  515,  575,       0,  658378,       0,  0, 19,  0, }, // DumontDUrville
    {  515,  612,       0,  658397,       0,  0, 19,  0, }, // East_Timor
    {  515,  623,  658416,  658437,  658470, 21, 33, 29, }, // Easter
    {  515,  630,       0,  658499,       0,  0, 13,  0, }, // Ecuador
    {  515,  638,  658512,  658533,  658565, 21, 32, 28, }, // Europe_Central
    {  515,  653,  658593,  658614,  658646, 21, 32, 28, }, // Europe_Eastern
    {  515,  668,       0,  651808,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  515,  691,  658674,  658695,  658727, 21, 32, 28, }, // Europe_Western
    {  515,  706,  658755,  658779,  658815, 24, 36, 32, }, // Falkland
    {  515,  715,  658847,  658858,  658881, 11, 23, 19, }, // Fiji
    {  515,  720,       0,  658900,       0,  0, 23,  0, }, // French_Guiana
    {  515,  734,       0,  658923,       0,  0, 35,  0, }, // French_Southern
    {  515,  757,       0,  658958,       0,  0, 15,  0, }, // Galapagos
    {  515,  767,       0,  658973,       0,  0, 13,  0, }, // Gambier
    {  515,  775,  658986,  658999,  659024, 13, 25, 21, }, // Georgia
    {  515,  783,       0,  659045,       0,  0, 20,  0, }, // Gilbert_Islands
    {  515,  799,       0,  659065,       0,  0, 24,  0, }, // GMT
    {  515,  841,  652235,  659089,  659123, 16, 34, 30, }, // Greenland_Eastern
    {  515,  859,  652320,  659153,  659187, 16, 34, 30, }, // Greenland_Western
    {  515,  877,       0,   96655,       0,  0, 21,  0, }, // Guam
    {  515,  882,       0,  659217,       0,  0, 14,  0, }, // Gulf
    {  515,  887,       0,  658910,       0,  0, 13,  0, }, // Guyana
    {  515,  894,  659231,  659254,  659288, 23, 34, 30, }, // Hawaii_Aleutian
    {  515,  910,  659318,  659333,  659360, 15, 27, 23, }, // Hong_Kong
    {  515,  920,  659383,  659393,  659415, 10, 22, 18, }, // Hovd
    {  515,  925,       0,  659433,       0,  0, 25,  0, }, // India
    {  515,  931,       0,  659458,       0,  0, 23,  0, }, // Indian_Ocean
    {  515,  944,       0,  659481,       0,  0, 14,  0, }, // Indochina
    {  515,  954,       0,  659495,       0,  0, 28,  0, }, // Indonesia_Central
    {  515,  972,       0,  659523,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  515,  990,       0,  659549,       0,  0, 26,  0, }, // Indonesia_Western
    {  515, 1008,  659575,  659585,  659607, 10, 22, 18, }, // Iran
    {  515, 1019,  659625,  659637,  659661, 12, 24, 20, }, // Irkutsk
    {  515, 1027,  659681,  659696,  659722, 15, 26, 22, }, // Israel
    {  515, 1034,  659744,  659758,  659783, 14, 25, 21, }, // Japan
    {  515, 1040,  652940,  652970,  653011, 30, 41, 49, }, // Kamchatka
    {  515, 1058,       0,  659804,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  515, 1077,       0,  659830,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  515, 1106,  659856,  659870,  659895, 14, 25, 21, }, // Korea
    {  515, 1112,       0,  659916,       0,  0, 11,  0, }, // Kosrae
    {  515, 1119,  659927,  659944,  659973, 17, 29, 25, }, // Krasnoyarsk
    {  515, 1151,       0,  659998,       0,  0, 15,  0, }, // Kyrgystan
    {  515, 1161,       0,  653248,       0,  0, 15,  0, }, // Lanka
    {  515, 1175,       0,  660013,       0,  0, 17,  0, }, // Line_Islands
    {  515, 1188,  660030,  660044,  660070, 14, 26, 22, }, // Lord_Howe
    {  515, 1198,   97864,   97875,   97897, 11, 22, 30, }, // Macau
    {  515, 1204,  660092,  660105,  660130, 13, 25, 21, }, // Magadan
    {  515, 1219,       0,  660151,       0,  0, 14,  0, }, // Malaysia
    {  515, 1228,       0,  660165,       0,  0, 13,  0, }, // Maldives
    {  515, 1237,       0,  660178,       0,  0, 12,  0, }, // Marquesas
    {  515, 1247,       0,  660190,       0,  0, 22,  0, }, // Marshall_Islands
    {  515, 1264,  660212,  660228,  660256, 16, 28, 24, }, // Mauritius
    {  515, 1274,       0,  660280,       0,  0, 11,  0, }, // Mawson
    {  515, 1281,  653578,  660291,  660325, 16, 34, 30, }, // Mexico_Pacific
    {  515, 1296,  660355,  660371,  660399, 16, 28, 24, }, // Mongolia
    {  515, 1305,  660423,  660435,  660459, 12, 24, 20, }, // Moscow
    {  515, 1312,       0,  660479,       0,  0, 14,  0, }, // Myanmar
    {  515, 1320,       0,  660493,       0,  0, 11,  0, }, // Nauru
    {  515, 1326,       0,  660504,       0,  0, 11,  0, }, // Nepal
    {  515, 1332,  660515,  660536,  660569, 21, 33, 29, }, // New_Caledonia
    {  515, 1346,  660598,  660615,  660644, 17, 29, 25, }, // New_Zealand
    {  515, 1358,   24341,  660669,  660699, 12, 30, 26, }, // Newfoundland
    {  515, 1371,       0,  660725,       0,  0, 10,  0, }, // Niue
    {  515, 1376,  660735,  660755,  660787, 20, 32, 28, }, // Norfolk
    {  515, 1384,  660815,  660840,  660877, 25, 37, 33, }, // Noronha
    {  515, 1392,       0,   98966,       0,  0, 31,  0, }, // North_Mariana
    {  515, 1406,  660910,  660927,  660956, 17, 29, 25, }, // Novosibirsk
    {  515, 1418,  660981,  660991,  661013, 10, 22, 18, }, // Omsk
    {  515, 1428,  661031,  661045,  661071, 14, 26, 22, }, // Pakistan
    {  515, 1437,       0,  661093,       0,  0, 11,  0, }, // Palau
    {  515, 1443,       0,  661104,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  515, 1460,  661128,  661142,  661168, 14, 26, 22, }, // Paraguay
    {  515, 1469,  661190,  661200,  661222, 10, 22, 18, }, // Peru
    {  515, 1474,  661240,  661254,  661280, 14, 26, 22, }, // Philippines
    {  515, 1486,       0,  661302,       0,  0, 19,  0, }, // Phoenix_Islands
    {  515, 1502,  654733,  661321,  661357, 18, 36, 32, }, // Pierre_Miquelon
    {  515, 1518,       0,  661389,       0,  0, 13,  0, }, // Pitcairn
    {  515, 1527,       0,  661402,       0,  0, 12,  0, }, // Ponape
    {  515, 1534,       0,  661414,       0,  0, 17,  0, }, // Pyongyang
    {  515, 1544,   99713,   99728,   99754, 15, 26, 34, }, // Qyzylorda
    {  515, 1554,       0,  661431,       0,  0, 13,  0, }, // Reunion
    {  515, 1562,       0,  661444,       0,  0, 12,  0, }, // Rothera
    {  515, 1570,  661456,  661469,  661494, 13, 25, 21, }, // Sakhalin
    {  515, 1579,   99901,   99913,   99936, 12, 23, 31, }, // Samara
    {  515, 1596,  661515,  661526,  661549, 11, 23, 19, }, // Samoa
    {  515, 1602,       0,  661568,       0,  0, 13,  0, }, // Seychelles
    {  515, 1624,       0,  661581,       0,  0, 26,  0, }, // Singapore
    {  515, 1634,       0,  661607,       0,  0, 23,  0, }, // Solomon
    {  515, 1642,       0,  661630,       0,  0, 21,  0, }, // South_Georgia
    {  515, 1656,       0,  661651,       0,  0, 13,  0, }, // Suriname
    {  515, 1676,       0,  661664,       0,  0, 10,  0, }, // Syowa
    {  515, 1682,       0,  661674,       0,  0, 12,  0, }, // Tahiti
    {  515, 1689,  661686,  661698,  661722, 12, 24, 20, }, // Taipei
    {  515, 1696,       0,  661742,       0,  0, 17,  0, }, // Tajikistan
    {  515, 1724,       0,  661759,       0,  0, 13,  0, }, // Tokelau
    {  515, 1732,  661772,  661783,  661806, 11, 23, 19, }, // Tonga
    {  515, 1738,       0,  661825,       0,  0, 10,  0, }, // Truk
    {  515, 1750,  661835,  661853,  661883, 18, 30, 26, }, // Turkmenistan
    {  515, 1763,       0,  661909,       0,  0, 12,  0, }, // Tuvalu
    {  515, 1777,  661921,  661934,  661959, 13, 25, 21, }, // Uruguay
    {  515, 1792,  661980,  661996,  662024, 16, 28, 24, }, // Uzbekistan
    {  515, 1803,  662048,  662061,  662086, 13, 25, 21, }, // Vanuatu
    {  515, 1811,       0,  662107,       0,  0, 15,  0, }, // Venezuela
    {  515, 1821,  662122,  662139,  662168, 17, 29, 25, }, // Vladivostok
    {  515, 1833,  662193,  662208,  662235, 15, 27, 23, }, // Volgograd
    {  515, 1843,       0,  662258,       0,  0, 12,  0, }, // Vostok
    {  515, 1850,       0,  662270,       0,  0, 17,  0, }, // Wake
    {  515, 1855,       0,  662287,       0,  0, 27,  0, }, // Wallis
    {  515, 1862,  662314,  662327,  662352, 13, 25, 21, }, // Yakutsk
    {  515, 1870,  662373,  662392,  662423, 19, 31, 27, }, // Yekaterinburg
    {  515, 1892,       0,   26429,       0,  0,  5,  0, }, // Yukon
    {  516,    0,  649076,  649086,  649107, 10, 21, 29, }, // Acre Serbian/Latin/Serbia
    {  516,    5,       0,  655989,       0,  0, 16,  0, }, // Afghanistan
    {  516,   17,       0,  656005,       0,  0, 23,  0, }, // Africa_Central
    {  516,   32,       0,  656028,       0,  0, 21,  0, }, // Africa_Eastern
    {  516,   65,       0,  656049,       0,  0, 19,  0, }, // Africa_Southern
    {  516,   81,  656068,  656089,  656121, 21, 32, 28, }, // Africa_Western
    {  516,  107,   92777,  656149,  656174,  7, 25, 21, }, // Alaska
    {  516,  128,  649362,  649374,  649397, 12, 23, 31, }, // Almaty
    {  516,  135,  656195,  656207,  656231, 12, 24, 20, }, // Amazon
    {  516,  142,  656251,  656282,  656324, 31, 42, 38, }, // America_Central
    {  516,  158,  656362,  656391,  656431, 29, 40, 36, }, // America_Eastern
    {  516,  174,  656467,  656498,  656540, 31, 42, 38, }, // America_Mountain
    {  516,  191,  656578,  656609,  656651, 31, 42, 38, }, // America_Pacific
    {  516,  207,   93461,   93473,   93496, 12, 23, 31, }, // Anadyr
    {  516,  214,  656689,  656700,  656723, 11, 23, 19, }, // Apia
    {  516,  219,  649716,  649729,  649753, 13, 24, 32, }, // Aqtau
    {  516,  225,  649785,  649798,  649822, 13, 24, 32, }, // Aqtobe
    {  516,  232,  656742,  656757,  656783, 15, 26, 22, }, // Arabian
    {  516,  240,  656805,  656820,  656847, 15, 27, 23, }, // Argentina
    {  516,  250,  656870,  656893,  656928, 23, 35, 31, }, // Argentina_Western
    {  516,  268,  656959,  656974,  657001, 15, 27, 23, }, // Armenia
    {  516,  286,  657024,  657039,  657065, 15, 26, 22, }, // Atlantic
    {  516,  295,  657087,  657115,  657154, 28, 39, 35, }, // Australia_Central
    {  516,  313,  657189,  657225,  657272, 36, 47, 43, }, // Australia_CentralWestern
    {  516,  338,  657315,  657341,  657378, 26, 37, 33, }, // Australia_Eastern
    {  516,  356,  657411,  657437,  657474, 26, 37, 33, }, // Australia_Western
    {  516,  374,  657507,  657524,  657553, 17, 29, 25, }, // Azerbaijan
    {  516,  385,  657578,  657589,  657612, 11, 23, 19, }, // Azores
    {  516,  397,  657631,  657646,  657673, 15, 27, 23, }, // Bangladesh
    {  516,  415,       0,  657696,       0,  0, 11,  0, }, // Bhutan
    {  516,  422,       0,  657707,       0,  0, 14,  0, }, // Bolivia
    {  516,  437,  657721,  657736,  657763, 15, 27, 23, }, // Brasilia
    {  516,  454,       0,  657786,       0,  0, 22,  0, }, // Brunei
    {  516,  461,  657808,  657832,  657868, 24, 36, 32, }, // Cape_Verde
    {  516,  478,       0,  657900,       0,  0, 12,  0, }, // Chamorro
    {  516,  487,  657912,  657923,  657946, 11, 23, 19, }, // Chatham
    {  516,  495,  657965,  657975,  657997, 10, 22, 18, }, // Chile
    {  516,  501,  658015,  658025,  658049, 10, 24, 18, }, // China
    {  516,  507,  658067,  658082,  658109, 15, 27, 23, }, // Choibalsan
    {  516,  518,       0,  658132,       0,  0, 20,  0, }, // Christmas
    {  516,  528,       0,  658152,       0,  0, 27,  0, }, // Cocos
    {  516,  534,  658179,  658194,  658221, 15, 27, 23, }, // Colombia
    {  516,  543,  658244,  658263,  658294, 19, 31, 32, }, // Cook
    {  516,  548,   15132,  658326,  658348,  4, 22, 18, }, // Cuba
    {  516,  559,       0,  658366,       0,  0, 12,  0, }, // Davis
    {  516,  575,       0,  658378,       0,  0, 19,  0, }, // DumontDUrville
    {  516,  612,       0,  658397,       0,  0, 19,  0, }, // East_Timor
    {  516,  623,  658416,  658437,  658470, 21, 33, 29, }, // Easter
    {  516,  630,       0,  658499,       0,  0, 13,  0, }, // Ecuador
    {  516,  638,  658512,  658533,  658565, 21, 32, 28, }, // Europe_Central
    {  516,  653,  658593,  658614,  658646, 21, 32, 28, }, // Europe_Eastern
    {  516,  668,       0,  651808,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  516,  691,  658674,  658695,  658727, 21, 32, 28, }, // Europe_Western
    {  516,  706,  658755,  658779,  658815, 24, 36, 32, }, // Falkland
    {  516,  715,  658847,  658858,  658881, 11, 23, 19, }, // Fiji
    {  516,  720,       0,  658900,       0,  0, 23,  0, }, // French_Guiana
    {  516,  734,       0,  658923,       0,  0, 35,  0, }, // French_Southern
    {  516,  757,       0,  658958,       0,  0, 15,  0, }, // Galapagos
    {  516,  767,       0,  658973,       0,  0, 13,  0, }, // Gambier
    {  516,  775,  658986,  658999,  659024, 13, 25, 21, }, // Georgia
    {  516,  783,       0,  659045,       0,  0, 20,  0, }, // Gilbert_Islands
    {  516,  799,       0,  659065,       0,  0, 24,  0, }, // GMT
    {  516,  841,  652235,  659089,  659123, 16, 34, 30, }, // Greenland_Eastern
    {  516,  859,  652320,  659153,  659187, 16, 34, 30, }, // Greenland_Western
    {  516,  877,       0,   96655,       0,  0, 21,  0, }, // Guam
    {  516,  882,       0,  659217,       0,  0, 14,  0, }, // Gulf
    {  516,  887,       0,  658910,       0,  0, 13,  0, }, // Guyana
    {  516,  894,  659231,  659254,  659288, 23, 34, 30, }, // Hawaii_Aleutian
    {  516,  910,  659318,  659333,  659360, 15, 27, 23, }, // Hong_Kong
    {  516,  920,  659383,  659393,  659415, 10, 22, 18, }, // Hovd
    {  516,  925,       0,  659433,       0,  0, 25,  0, }, // India
    {  516,  931,       0,  659458,       0,  0, 23,  0, }, // Indian_Ocean
    {  516,  944,       0,  659481,       0,  0, 14,  0, }, // Indochina
    {  516,  954,       0,  659495,       0,  0, 28,  0, }, // Indonesia_Central
    {  516,  972,       0,  659523,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  516,  990,       0,  659549,       0,  0, 26,  0, }, // Indonesia_Western
    {  516, 1008,  659575,  659585,  659607, 10, 22, 18, }, // Iran
    {  516, 1019,  659625,  659637,  659661, 12, 24, 20, }, // Irkutsk
    {  516, 1027,  659681,  659696,  659722, 15, 26, 22, }, // Israel
    {  516, 1034,  659744,  659758,  659783, 14, 25, 21, }, // Japan
    {  516, 1040,  652940,  652970,  653011, 30, 41, 49, }, // Kamchatka
    {  516, 1058,       0,  659804,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  516, 1077,       0,  659830,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  516, 1106,  659856,  659870,  659895, 14, 25, 21, }, // Korea
    {  516, 1112,       0,  659916,       0,  0, 11,  0, }, // Kosrae
    {  516, 1119,  659927,  659944,  659973, 17, 29, 25, }, // Krasnoyarsk
    {  516, 1151,       0,  659998,       0,  0, 15,  0, }, // Kyrgystan
    {  516, 1161,       0,  653248,       0,  0, 15,  0, }, // Lanka
    {  516, 1175,       0,  660013,       0,  0, 17,  0, }, // Line_Islands
    {  516, 1188,  660030,  660044,  660070, 14, 26, 22, }, // Lord_Howe
    {  516, 1198,   97864,   97875,   97897, 11, 22, 30, }, // Macau
    {  516, 1204,  660092,  660105,  660130, 13, 25, 21, }, // Magadan
    {  516, 1219,       0,  660151,       0,  0, 14,  0, }, // Malaysia
    {  516, 1228,       0,  660165,       0,  0, 13,  0, }, // Maldives
    {  516, 1237,       0,  660178,       0,  0, 12,  0, }, // Marquesas
    {  516, 1247,       0,  660190,       0,  0, 22,  0, }, // Marshall_Islands
    {  516, 1264,  660212,  660228,  660256, 16, 28, 24, }, // Mauritius
    {  516, 1274,       0,  660280,       0,  0, 11,  0, }, // Mawson
    {  516, 1281,  653578,  660291,  660325, 16, 34, 30, }, // Mexico_Pacific
    {  516, 1296,  660355,  660371,  660399, 16, 28, 24, }, // Mongolia
    {  516, 1305,  660423,  660435,  660459, 12, 24, 20, }, // Moscow
    {  516, 1312,       0,  660479,       0,  0, 14,  0, }, // Myanmar
    {  516, 1320,       0,  660493,       0,  0, 11,  0, }, // Nauru
    {  516, 1326,       0,  660504,       0,  0, 11,  0, }, // Nepal
    {  516, 1332,  660515,  660536,  660569, 21, 33, 29, }, // New_Caledonia
    {  516, 1346,  660598,  660615,  660644, 17, 29, 25, }, // New_Zealand
    {  516, 1358,   24341,  660669,  660699, 12, 30, 26, }, // Newfoundland
    {  516, 1371,       0,  660725,       0,  0, 10,  0, }, // Niue
    {  516, 1376,  660735,  660755,  660787, 20, 32, 28, }, // Norfolk
    {  516, 1384,  660815,  660840,  660877, 25, 37, 33, }, // Noronha
    {  516, 1392,       0,   98966,       0,  0, 31,  0, }, // North_Mariana
    {  516, 1406,  660910,  660927,  660956, 17, 29, 25, }, // Novosibirsk
    {  516, 1418,  660981,  660991,  661013, 10, 22, 18, }, // Omsk
    {  516, 1428,  661031,  661045,  661071, 14, 26, 22, }, // Pakistan
    {  516, 1437,       0,  661093,       0,  0, 11,  0, }, // Palau
    {  516, 1443,       0,  661104,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  516, 1460,  661128,  661142,  661168, 14, 26, 22, }, // Paraguay
    {  516, 1469,  661190,  661200,  661222, 10, 22, 18, }, // Peru
    {  516, 1474,  661240,  661254,  661280, 14, 26, 22, }, // Philippines
    {  516, 1486,       0,  661302,       0,  0, 19,  0, }, // Phoenix_Islands
    {  516, 1502,  654733,  661321,  661357, 18, 36, 32, }, // Pierre_Miquelon
    {  516, 1518,       0,  661389,       0,  0, 13,  0, }, // Pitcairn
    {  516, 1527,       0,  661402,       0,  0, 12,  0, }, // Ponape
    {  516, 1534,       0,  661414,       0,  0, 17,  0, }, // Pyongyang
    {  516, 1544,   99713,   99728,   99754, 15, 26, 34, }, // Qyzylorda
    {  516, 1554,       0,  661431,       0,  0, 13,  0, }, // Reunion
    {  516, 1562,       0,  661444,       0,  0, 12,  0, }, // Rothera
    {  516, 1570,  661456,  661469,  661494, 13, 25, 21, }, // Sakhalin
    {  516, 1579,   99901,   99913,   99936, 12, 23, 31, }, // Samara
    {  516, 1596,  661515,  661526,  661549, 11, 23, 19, }, // Samoa
    {  516, 1602,       0,  661568,       0,  0, 13,  0, }, // Seychelles
    {  516, 1624,       0,  661581,       0,  0, 26,  0, }, // Singapore
    {  516, 1634,       0,  661607,       0,  0, 23,  0, }, // Solomon
    {  516, 1642,       0,  661630,       0,  0, 21,  0, }, // South_Georgia
    {  516, 1656,       0,  661651,       0,  0, 13,  0, }, // Suriname
    {  516, 1676,       0,  661664,       0,  0, 10,  0, }, // Syowa
    {  516, 1682,       0,  661674,       0,  0, 12,  0, }, // Tahiti
    {  516, 1689,  661686,  661698,  661722, 12, 24, 20, }, // Taipei
    {  516, 1696,       0,  661742,       0,  0, 17,  0, }, // Tajikistan
    {  516, 1724,       0,  661759,       0,  0, 13,  0, }, // Tokelau
    {  516, 1732,  661772,  661783,  661806, 11, 23, 19, }, // Tonga
    {  516, 1738,       0,  661825,       0,  0, 10,  0, }, // Truk
    {  516, 1750,  661835,  661853,  661883, 18, 30, 26, }, // Turkmenistan
    {  516, 1763,       0,  661909,       0,  0, 12,  0, }, // Tuvalu
    {  516, 1777,  661921,  661934,  661959, 13, 25, 21, }, // Uruguay
    {  516, 1792,  661980,  661996,  662024, 16, 28, 24, }, // Uzbekistan
    {  516, 1803,  662048,  662061,  662086, 13, 25, 21, }, // Vanuatu
    {  516, 1811,       0,  662107,       0,  0, 15,  0, }, // Venezuela
    {  516, 1821,  662122,  662139,  662168, 17, 29, 25, }, // Vladivostok
    {  516, 1833,  662193,  662208,  662235, 15, 27, 23, }, // Volgograd
    {  516, 1843,       0,  662258,       0,  0, 12,  0, }, // Vostok
    {  516, 1850,       0,  662270,       0,  0, 17,  0, }, // Wake
    {  516, 1855,       0,  662287,       0,  0, 27,  0, }, // Wallis
    {  516, 1862,  662314,  662327,  662352, 13, 25, 21, }, // Yakutsk
    {  516, 1870,  662373,  662392,  662423, 19, 31, 27, }, // Yekaterinburg
    {  516, 1892,       0,   26429,       0,  0,  5,  0, }, // Yukon
    {  522,    5,       0,   13145,       0,  0, 10,  0, }, // Afghanistan Silesian/Latin/Poland
    {  522,   17,       0,  662450,       0,  0, 27,  0, }, // Africa_Central
    {  522,   32,       0,  662477,       0,  0, 24,  0, }, // Africa_Eastern
    {  522,   65,       0,  662501,       0,  0, 25,  0, }, // Africa_Southern
    {  522,   81,  662526,  662550,  662587, 24, 37, 31, }, // Africa_Western
    {  522,  107,  662618,  662633,  662661, 15, 28, 22, }, // Alaska
    {  522,  128,  578966,  662683,  662708, 12, 25, 19, }, // Almaty
    {  522,  135,  662727,  662741,  662768, 14, 27, 21, }, // Amazon
    {  522,  142,  662789,  662817,  662858, 28, 41, 35, }, // America_Central
    {  522,  158,  579170,  662893,  662931, 25, 38, 32, }, // America_Eastern
    {  522,  174,  662963,  662974,  662998, 11, 24, 18, }, // America_Mountain
    {  522,  191,  579314,  663016,  663044, 15, 28, 22, }, // America_Pacific
    {  522,  207,  579377,  663066,  663090, 11, 24, 18, }, // Anadyr
    {  522,  219,  579477,  663108,  663135, 14, 27, 21, }, // Aqtau
    {  522,  225,  579537,  663156,  663185, 16, 29, 23, }, // Aqtobe
    {  522,  232,  663208,  663224,  663260, 16, 36, 30, }, // Arabian
    {  522,  240,  663290,  663299,  663328,  9, 29, 23, }, // Argentina
    {  522,  250,  663351,  663370,  663409, 19, 39, 33, }, // Argentina_Western
    {  522,  268,  663442,  663450,  663478,  8, 28, 22, }, // Armenia
    {  522,  286,  579904,  663500,  663528, 15, 28, 22, }, // Atlantic
    {  522,  295,  663550,  663579,  663621, 29, 42, 36, }, // Australia_Central
    {  522,  313,  663657,  663696,  663748, 39, 52, 46, }, // Australia_CentralWestern
    {  522,  338,  663794,  663820,  663859, 26, 39, 33, }, // Australia_Eastern
    {  522,  356,  663670,  663709,  663761, 26, 39, 33, }, // Australia_Western
    {  522,  374,  580380,  663892,  663923, 11, 31, 25, }, // Azerbaijan
    {  522,  385,  580450,  663948,  663973,  5, 25, 19, }, // Azores
    {  522,  397,  580502,  663992,  664022, 10, 30, 24, }, // Bangladesh
    {  522,  422,       0,  664046,       0,  0,  8,  0, }, // Bolivia
    {  522,  437,  110722,  664054,  664082,  8, 28, 22, }, // Brasilia
    {  522,  461,  664104,  664130,  664176, 26, 46, 40, }, // Cape_Verde
    {  522,  478,       0,  580776,       0,  0,  8,  0, }, // Chamorro
    {  522,  495,  580842,  664216,  664241, 10, 25, 19, }, // Chile
    {  522,  501,  580899,  664260,  664285,  5, 25, 19, }, // China
    {  522,  507,  580951,  664304,  664334, 10, 30, 24, }, // Choibalsan
    {  522,  518,       0,  664358,       0,  0, 12,  0, }, // Christmas
    {  522,  528,       0,  581046,       0,  0, 14,  0, }, // Cocos
    {  522,  534,  465030,  664370,  664399,  9, 29, 23, }, // Colombia
    {  522,  543,  581126,  664422,  664453, 11, 31, 25, }, // Cook
    {  522,  548,   15132,  664478,  664502,  4, 24, 18, }, // Cuba
    {  522,  575,       0,    2207,       0,  0, 16,  0, }, // DumontDUrville
    {  522,  612,       0,  581276,       0,  0, 14,  0, }, // East_Timor
    {  522,  623,  664520,  664537,  664574, 17, 37, 31, }, // Easter
    {  522,  630,       0,  438323,       0,  0,  7,  0, }, // Ecuador
    {  522,  638,  664605,  664632,  664672, 27, 40, 34, }, // Europe_Central
    {  522,  653,  664706,  664730,  664767, 24, 37, 31, }, // Europe_Eastern
    {  522,  668,       0,  664798,       0,  0, 31,  0, }, // Europe_Further_Eastern
    {  522,  691,  664829,  664853,  664890, 24, 37, 31, }, // Europe_Western
    {  522,  706,  581693,  664921,  664950,  9, 29, 23, }, // Falkland
    {  522,  715,  581757,  664973,  664998,  5, 25, 19, }, // Fiji
    {  522,  720,       0,  665017,       0,  0, 16,  0, }, // French_Guiana
    {  522,  734,       0,  665033,       0,  0, 45,  0, }, // French_Southern
    {  522,  757,       0,  581876,       0,  0, 14,  0, }, // Galapagos
    {  522,  767,       0,  581895,       0,  0, 14,  0, }, // Gambier
    {  522,  775,  665078,  665085,  665112,  7, 27, 21, }, // Georgia
    {  522,  783,       0,  665133,       0,  0, 16,  0, }, // Gilbert_Islands
    {  522,  799,       0,  665149,       0,  0, 19,  0, }, // GMT
    {  522,  841,  665168,  665189,  665230, 21, 41, 35, }, // Greenland_Eastern
    {  522,  859,  665265,  665286,  665327, 21, 41, 35, }, // Greenland_Western
    {  522,  882,       0,  582198,       0,  0, 13,  0, }, // Gulf
    {  522,  887,       0,  236861,       0,  0,  6,  0, }, // Guyana
    {  522,  894,  582216,  665362,  665395, 13, 33, 27, }, // Hawaii_Aleutian
    {  522,  910,  665422,  665430,  665458,  8, 28, 22, }, // Hong_Kong
    {  522,  920,  582353,  665480,  665505,  5, 25, 19, }, // Hovd
    {  522,  925,       0,  665524,       0,  0, 26,  0, }, // India
    {  522,  931,       0,  665550,       0,  0, 14,  0, }, // Indian_Ocean
    {  522,  944,       0,  665564,       0,  0, 16,  0, }, // Indochina
    {  522,  954,       0,  665580,       0,  0, 23,  0, }, // Indonesia_Central
    {  522,  972,       0,  665603,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  522,  990,       0,  665623,       0,  0, 20,  0, }, // Indonesia_Western
    {  522, 1019,  539835,  665643,  665669,  6, 26, 20, }, // Irkutsk
    {  522, 1027,   23036,  665689,  665715,  6, 26, 20, }, // Israel
    {  522, 1034,  665735,  665743,  665771,  8, 28, 22, }, // Japan
    {  522, 1040,  582748,  665793,  665835, 29, 42, 36, }, // Kamchatka
    {  522, 1058,       0,  582858,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  522, 1077,       0,  582882,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  522, 1119,   16559,  665871,  665902, 11, 31, 25, }, // Krasnoyarsk
    {  522, 1151,       0,   16618,       0,  0,  9,  0, }, // Kyrgystan
    {  522, 1175,       0,    3916,       0,  0, 12,  0, }, // Line_Islands
    {  522, 1188,    3933,  665927,  665956,  9, 29, 23, }, // Lord_Howe
    {  522, 1219,       0,  665979,       0,  0,  8,  0, }, // Malaysia
    {  522, 1228,       0,  583204,       0,  0,  8,  0, }, // Maldives
    {  522, 1237,       0,  583217,       0,  0,  7,  0, }, // Marquesas
    {  522, 1247,       0,  583229,       0,  0, 15,  0, }, // Marshall_Islands
    {  522, 1281,  583319,  665987,  666024, 24, 37, 31, }, // Mexico_Pacific
    {  522, 1296,  583414,  666055,  666085, 10, 30, 24, }, // Mongolia
    {  522, 1305,  583481,  666109,  666130,  6, 21, 15, }, // Moscow
    {  522, 1312,       0,  346583,       0,  0,  6,  0, }, // Myanmar
    {  522, 1332,  666145,  666160,  666195, 15, 35, 29, }, // New_Caledonia
    {  522, 1346,  666224,  666238,  666272, 14, 34, 28, }, // New_Zealand
    {  522, 1358,  666300,  666316,  666352, 16, 36, 30, }, // Newfoundland
    {  522, 1384,    4733,  666382,  666421, 19, 39, 33, }, // Noronha
    {  522, 1406,  583965,  666454,  666485, 11, 31, 25, }, // Novosibirsk
    {  522, 1418,  666510,  666514,  666538,  4, 24, 18, }, // Omsk
    {  522, 1443,       0,  666556,       0,  0, 17,  0, }, // Papua_New_Guinea
    {  522, 1460,  584177,  666573,  666601,  8, 28, 22, }, // Paraguay
    {  522, 1469,  584233,  666623,  666647,  9, 24, 18, }, // Peru
    {  522, 1474,  584287,  666665,  666693,  8, 28, 22, }, // Philippines
    {  522, 1486,       0,  666715,       0,  0,  6,  0, }, // Phoenix_Islands
    {  522, 1502,  584359,  666721,  666764, 23, 43, 37, }, // Pierre_Miquelon
    {  522, 1527,       0,  113987,       0,  0,  7,  0, }, // Ponape
    {  522, 1534,       0,  666801,       0,  0,  9,  0, }, // Pyongyang
    {  522, 1544,  584499,  666810,  666839, 16, 29, 23, }, // Qyzylorda
    {  522, 1570,  207493,  666862,  666890,  8, 28, 22, }, // Sakhalin
    {  522, 1579,  584650,  666912,  666936, 11, 24, 18, }, // Samara
    {  522, 1602,       0,  584758,       0,  0,  7,  0, }, // Seychelles
    {  522, 1624,       0,   58868,       0,  0,  8,  0, }, // Singapore
    {  522, 1634,       0,  666954,       0,  0, 14,  0, }, // Solomon
    {  522, 1642,       0,  666968,       0,  0, 18,  0, }, // South_Georgia
    {  522, 1656,       0,    5663,       0,  0,  7,  0, }, // Suriname
    {  522, 1689,  100195,  666986,  667012,  6, 26, 20, }, // Taipei
    {  522, 1696,       0,  584913,       0,  0, 11,  0, }, // Tajikistan
    {  522, 1732,  667032,  667037,  667062,  5, 25, 19, }, // Tonga
    {  522, 1738,       0,    5821,       0,  0,  5,  0, }, // Truk
    {  522, 1750,  667081,  667093,  667125, 12, 32, 26, }, // Turkmenistan
    {  522, 1777,  585087,  667151,  667178,  7, 27, 21, }, // Uruguay
    {  522, 1811,       0,  667199,       0,  0,  9,  0, }, // Venezuela
    {  522, 1821,  585284,  667208,  667239, 11, 31, 25, }, // Vladivostok
    {  522, 1833,  585354,  667264,  667293,  9, 29, 23, }, // Volgograd
    {  522, 1843,       0,   18641,       0,  0,  6,  0, }, // Vostok
    {  522, 1855,       0,  139812,       0,  0, 15,  0, }, // Wallis
    {  522, 1862,  542174,  667316,  667342,  6, 26, 20, }, // Yakutsk
    {  522, 1870,  585513,  667362,  667395, 13, 33, 27, }, // Yekaterinburg
    {  522, 1892,       0,  667422,       0,  0, 13,  0, }, // Yukon
    {  523,    5,       0,  667435,       0,  0, 16,  0, }, // Afghanistan Sindhi/Arabic/Pakistan
    {  523,   17,       0,  667451,       0,  0, 19,  0, }, // Africa_Central
    {  523,   32,       0,  667470,       0,  0, 18,  0, }, // Africa_Eastern
    {  523,   65,       0,  667488,       0,  0, 24,  0, }, // Africa_Southern
    {  523,   81,  667512,  667531,  667557, 19, 26, 30, }, // Africa_Western
    {  523,  107,  667587,  667600,  667620, 13, 20, 22, }, // Alaska
    {  523,  135,  667642,  667656,  667677, 14, 21, 25, }, // Amazon
    {  523,  142,  667702,  667711,  667727,  9, 16, 18, }, // America_Central
    {  523,  158,  667745,  667754,  667770,  9, 16, 18, }, // America_Eastern
    {  523,  174,  667788,  667797,  667813,  9, 16, 18, }, // America_Mountain
    {  523,  191,  667831,  667841,  667858, 10, 17, 19, }, // America_Pacific
    {  523,  214,  667877,  667888,  667906, 11, 18, 20, }, // Apia
    {  523,  232,  667926,  667938,  667957, 12, 19, 21, }, // Arabian
    {  523,  240,  667978,  667993,  668012, 15, 19, 26, }, // Argentina
    {  523,  250,  668038,  668059,  668087, 21, 28, 32, }, // Argentina_Western
    {  523,  268,  668119,  668133,  668154, 14, 21, 25, }, // Armenia
    {  523,  286,  668179,  668194,  668216, 15, 22, 24, }, // Atlantic
    {  523,  295,  668240,  668261,  668289, 21, 28, 30, }, // Australia_Central
    {  523,  313,  668319,  668346,  668380, 27, 34, 36, }, // Australia_CentralWestern
    {  523,  338,  668416,  668436,  668464, 20, 28, 30, }, // Australia_Eastern
    {  523,  356,  668494,  668515,  668543, 21, 28, 30, }, // Australia_Western
    {  523,  374,  668573,  668590,  668614, 17, 24, 28, }, // Azerbaijan
    {  523,  385,  668642,  668654,  668673, 12, 19, 23, }, // Azores
    {  523,  397,  668696,  668711,  668733, 15, 22, 26, }, // Bangladesh
    {  523,  415,       0,  668759,       0,  0, 12,  0, }, // Bhutan
    {  523,  422,       0,  668771,       0,  0, 12,  0, }, // Bolivia
    {  523,  437,  668783,  668797,  668819, 14, 22, 26, }, // Brasilia
    {  523,  454,       0,  668845,       0,  0, 24,  0, }, // Brunei
    {  523,  461,  668869,  668883,  668904, 14, 21, 25, }, // Cape_Verde
    {  523,  478,       0,  668929,       0,  0, 19,  0, }, // Chamorro
    {  523,  487,  668948,  668959,  668977, 11, 18, 20, }, // Chatham
    {  523,  495,  668997,  669007,  669024, 10, 17, 21, }, // Chile
    {  523,  501,  669045,  669055,  669074, 10, 19, 21, }, // China
    {  523,  507,  669095,  669112,  669136, 17, 24, 28, }, // Choibalsan
    {  523,  518,       0,  669164,       0,  0, 21,  0, }, // Christmas
    {  523,  528,       0,  669185,       0,  0, 21,  0, }, // Cocos
    {  523,  534,  669206,  669220,  669241, 14, 21, 25, }, // Colombia
    {  523,  543,  669266,  669285,  669311, 19, 26, 33, }, // Cook
    {  523,  548,  669344,  669356,  669375, 12, 19, 21, }, // Cuba
    {  523,  559,       0,  669396,       0,  0, 11,  0, }, // Davis
    {  523,  575,       0,  669407,       0,  0, 20,  0, }, // DumontDUrville
    {  523,  612,       0,  669427,       0,  0, 17,  0, }, // East_Timor
    {  523,  623,  669444,  669465,  669493, 21, 28, 32, }, // Easter
    {  523,  630,       0,  669525,       0,  0, 15,  0, }, // Ecuador
    {  523,  638,  669540,  669555,  669577, 15, 22, 26, }, // Europe_Central
    {  523,  653,  669603,  669618,  669640, 15, 22, 26, }, // Europe_Eastern
    {  523,  668,       0,  669666,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  523,  691,  669686,  669701,  669723, 15, 22, 24, }, // Europe_Western
    {  523,  706,  669747,  669771,  669802, 24, 31, 35, }, // Falkland
    {  523,  715,  669837,  669847,  669864, 10, 17, 21, }, // Fiji
    {  523,  720,       0,  669885,       0,  0, 21,  0, }, // French_Guiana
    {  523,  734,       0,  669906,       0,  0, 30,  0, }, // French_Southern
    {  523,  757,       0,  669936,       0,  0, 16,  0, }, // Galapagos
    {  523,  767,       0,  669952,       0,  0, 14,  0, }, // Gambier
    {  523,  775,  669966,  669979,  669999, 13, 20, 24, }, // Georgia
    {  523,  783,       0,  670023,       0,  0, 21,  0, }, // Gilbert_Islands
    {  523,  799,       0,  670044,       0,  0, 17,  0, }, // GMT
    {  523,  841,  670061,  670083,  670112, 22, 29, 33, }, // Greenland_Eastern
    {  523,  859,  670145,  670167,  670196, 22, 29, 33, }, // Greenland_Western
    {  523,  882,       0,  670229,       0,  0, 16,  0, }, // Gulf
    {  523,  887,       0,  670245,       0,  0, 11,  0, }, // Guyana
    {  523,  894,  670256,  670276,  670303, 20, 27, 29, }, // Hawaii_Aleutian
    {  523,  910,  670332,  670348,  670371, 16, 23, 27, }, // Hong_Kong
    {  523,  920,  670398,  670409,  670427, 11, 18, 22, }, // Hovd
    {  523,  925,       0,  670449,       0,  0, 18,  0, }, // India
    {  523,  931,       0,  670467,       0,  0, 15,  0, }, // Indian_Ocean
    {  523,  944,       0,  670482,       0,  0, 17,  0, }, // Indochina
    {  523,  954,       0,  670499,       0,  0, 22,  0, }, // Indonesia_Central
    {  523,  972,       0,  670521,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  523,  990,       0,  670542,       0,  0, 22,  0, }, // Indonesia_Western
    {  523, 1008,  670564,  670576,  670595, 12, 19, 21, }, // Iran
    {  523, 1019,  670616,  670629,  670649, 13, 20, 22, }, // Irkutsk
    {  523, 1027,  670671,  670685,  670706, 14, 21, 23, }, // Israel
    {  523, 1034,  670729,  670741,  670760, 12, 19, 21, }, // Japan
    {  523, 1058,       0,  670781,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  523, 1077,       0,  670801,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  523, 1106,  670822,  670834,  670853, 12, 19, 21, }, // Korea
    {  523, 1112,       0,  670874,       0,  0, 14,  0, }, // Kosrae
    {  523, 1119,  670888,  670906,  670931, 18, 25, 27, }, // Krasnoyarsk
    {  523, 1151,       0,  670958,       0,  0, 15,  0, }, // Kyrgystan
    {  523, 1175,       0,  670973,       0,  0, 20,  0, }, // Line_Islands
    {  523, 1188,  670993,  671009,  671032, 16, 23, 25, }, // Lord_Howe
    {  523, 1204,  671057,  671070,  671090, 13, 20, 22, }, // Magadan
    {  523, 1219,       0,  671112,       0,  0, 15,  0, }, // Malaysia
    {  523, 1228,       0,  671127,       0,  0, 13,  0, }, // Maldives
    {  523, 1237,       0,  671140,       0,  0, 14,  0, }, // Marquesas
    {  523, 1247,       0,  671154,       0,  0, 21,  0, }, // Marshall_Islands
    {  523, 1264,  671175,  671189,  671210, 14, 21, 25, }, // Mauritius
    {  523, 1274,       0,  671235,       0,  0, 13,  0, }, // Mawson
    {  523, 1281,  671248,  671266,  671294, 18, 28, 30, }, // Mexico_Pacific
    {  523, 1296,  671324,  671341,  671365, 17, 24, 28, }, // Mongolia
    {  523, 1305,  671393,  671405,  671424, 12, 19, 21, }, // Moscow
    {  523, 1312,       0,  671445,       0,  0, 14,  0, }, // Myanmar
    {  523, 1320,       0,  671459,       0,  0, 13,  0, }, // Nauru
    {  523, 1326,       0,  671472,       0,  0, 12,  0, }, // Nepal
    {  523, 1332,  671484,  671504,  671531, 20, 27, 31, }, // New_Caledonia
    {  523, 1346,  671562,  671578,  671601, 16, 23, 25, }, // New_Zealand
    {  523, 1358,  671626,  671648,  671677, 22, 29, 31, }, // Newfoundland
    {  523, 1371,       0,  671708,       0,  0, 12,  0, }, // Niue
    {  523, 1376,  671720,  671743,  671773, 23, 30, 32, }, // Norfolk
    {  523, 1384,  671805,  671830,  671862, 25, 32, 33, }, // Noronha
    {  523, 1406,  671895,  671913,  671938, 18, 25, 27, }, // Novosibirsk
    {  523, 1418,  671965,  671977,  671996, 12, 19, 21, }, // Omsk
    {  523, 1428,  672017,  672031,  672052, 14, 21, 25, }, // Pakistan
    {  523, 1437,       0,  672077,       0,  0, 12,  0, }, // Palau
    {  523, 1443,       0,  672089,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  523, 1460,  672109,  672124,  672146, 15, 22, 26, }, // Paraguay
    {  523, 1469,  672172,  672183,  672201, 11, 18, 22, }, // Peru
    {  523, 1474,  672223,  672236,  672256, 13, 20, 24, }, // Philippines
    {  523, 1486,       0,  672280,       0,  0, 22,  0, }, // Phoenix_Islands
    {  523, 1502,  672302,  672327,  672359, 25, 32, 34, }, // Pierre_Miquelon
    {  523, 1518,       0,  672393,       0,  0, 13,  0, }, // Pitcairn
    {  523, 1527,       0,  672406,       0,  0, 13,  0, }, // Ponape
    {  523, 1534,       0,  672419,       0,  0, 17,  0, }, // Pyongyang
    {  523, 1554,       0,  672436,       0,  0, 15,  0, }, // Reunion
    {  523, 1562,       0,  672451,       0,  0, 13,  0, }, // Rothera
    {  523, 1570,  672464,  672477,  672497, 13, 20, 22, }, // Sakhalin
    {  523, 1596,  672519,  672531,  672550, 12, 19, 21, }, // Samoa
    {  523, 1602,       0,  672571,       0,  0, 13,  0, }, // Seychelles
    {  523, 1624,       0,  672584,       0,  0, 21,  0, }, // Singapore
    {  523, 1634,       0,  672605,       0,  0, 22,  0, }, // Solomon
    {  523, 1642,       0,  672627,       0,  0, 17,  0, }, // South_Georgia
    {  523, 1656,       0,  672644,       0,  0, 15,  0, }, // Suriname
    {  523, 1676,       0,  672659,       0,  0, 13,  0, }, // Syowa
    {  523, 1682,       0,  672672,       0,  0, 13,  0, }, // Tahiti
    {  523, 1689,  672685,  672697,  672716, 12, 19, 21, }, // Taipei
    {  523, 1696,       0,  672737,       0,  0, 15,  0, }, // Tajikistan
    {  523, 1724,       0,  672752,       0,  0, 15,  0, }, // Tokelau
    {  523, 1732,  672767,  672779,  672798, 12, 19, 23, }, // Tonga
    {  523, 1738,       0,  672821,       0,  0, 11,  0, }, // Truk
    {  523, 1750,  672832,  672849,  672873, 17, 24, 28, }, // Turkmenistan
    {  523, 1763,       0,  672901,       0,  0, 13,  0, }, // Tuvalu
    {  523, 1777,  672914,  672929,  672951, 15, 22, 26, }, // Uruguay
    {  523, 1792,  672977,  672992,  673014, 15, 22, 26, }, // Uzbekistan
    {  523, 1803,  673040,  673054,  673075, 14, 21, 23, }, // Vanuatu
    {  523, 1811,       0,  673098,       0,  0, 15,  0, }, // Venezuela
    {  523, 1821,  673113,  673130,  673154, 17, 24, 27, }, // Vladivostok
    {  523, 1833,  673181,  673197,  673220, 16, 23, 25, }, // Volgograd
    {  523, 1843,       0,  673245,       0,  0, 13,  0, }, // Vostok
    {  523, 1850,       0,  673258,       0,  0, 19,  0, }, // Wake
    {  523, 1855,       0,  673277,       0,  0, 19,  0, }, // Wallis
    {  523, 1862,  673296,  673309,  673329, 13, 20, 22, }, // Yakutsk
    {  523, 1870,  673351,  673368,  673392, 17, 24, 26, }, // Yekaterinburg
    {  523, 1892,       0,  673418,       0,  0, 11,  0, }, // Yukon
    {  524,  142,  673429,  673440,  673458, 11, 18, 20, }, // America_Central Sindhi/Devanagari/India
    {  524,  158,  673478,  673487,  673503,  9, 16, 18, }, // America_Eastern
    {  524,  174,  673521,  673532,  673550, 11, 18, 20, }, // America_Mountain
    {  524,  191,  673570,  673582,  673601, 12, 19, 21, }, // America_Pacific
    {  524,  286,  673622,  673635,  673655, 13, 20, 22, }, // Atlantic
    {  524,  638,  673677,  673695,  673721, 18, 26, 29, }, // Europe_Central
    {  524,  653,  673750,  673767,  673791, 17, 24, 27, }, // Europe_Eastern
    {  524,  691,  673818,  673837,  673863, 19, 26, 29, }, // Europe_Western
    {  524,  799,       0,  673892,       0,  0, 18,  0, }, // GMT
    {  525,    5,       0,  673910,       0,  0, 17,  0, }, // Afghanistan Sinhala/Sinhala/Sri Lanka
    {  525,   17,       0,  673927,       0,  0, 23,  0, }, // Africa_Central
    {  525,   32,       0,  673950,       0,  0, 25,  0, }, // Africa_Eastern
    {  525,   65,       0,  673975,       0,  0, 22,  0, }, // Africa_Southern
    {  525,   81,  673997,  674019,  674047, 22, 28, 30, }, // Africa_Western
    {  525,  107,  674077,  674089,  674107, 12, 18, 21, }, // Alaska
    {  525,  135,  674128,  674141,  674160, 13, 19, 21, }, // Amazon
    {  525,  142,  674181,  674208,  674241, 27, 33, 36, }, // America_Central
    {  525,  158,  674277,  674306,  674341, 29, 35, 38, }, // America_Eastern
    {  525,  174,  674379,  674405,  674437, 26, 32, 35, }, // America_Mountain
    {  525,  191,  674472,  674501,  674536, 29, 35, 38, }, // America_Pacific
    {  525,  214,  674574,  674585,  674602, 11, 17, 16, }, // Apia
    {  525,  232,  674618,  674629,  674646, 11, 17, 17, }, // Arabian
    {  525,  240,  674663,  674679,  674701, 16, 22, 24, }, // Argentina
    {  525,  250,  674725,  674747,  674775, 22, 28, 30, }, // Argentina_Western
    {  525,  268,  674805,  674820,  674841, 15, 21, 24, }, // Armenia
    {  525,  286,  674865,  674882,  674905, 17, 23, 26, }, // Atlantic
    {  525,  295,  674931,  674958,  674984, 27, 26, 33, }, // Australia_Central
    {  525,  313,  675017,  675050,  675089, 33, 39, 39, }, // Australia_CentralWestern
    {  525,  338,  675128,  675157,  675192, 29, 35, 35, }, // Australia_Eastern
    {  525,  356,  675227,  675253,  675096, 26, 32, 32, }, // Australia_Western
    {  525,  374,  675285,  675302,  675325, 17, 23, 26, }, // Azerbaijan
    {  525,  385,  675351,  675364,  675383, 13, 19, 22, }, // Azores
    {  525,  397,  675405,  675419,  675439, 14, 20, 22, }, // Bangladesh
    {  525,  415,       0,  675461,       0,  0, 11,  0, }, // Bhutan
    {  525,  422,       0,  675472,       0,  0, 14,  0, }, // Bolivia
    {  525,  437,  675486,  675499,  675518, 13, 19, 21, }, // Brasilia
    {  525,  454,       0,  675539,       0,  0, 23,  0, }, // Brunei
    {  525,  461,  675562,  675576,  675596, 14, 20, 22, }, // Cape_Verde
    {  525,  478,       0,  675618,       0,  0, 11,  0, }, // Chamorro
    {  525,  487,  675629,  675640,  675657, 11, 17, 16, }, // Chatham
    {  525,  495,  675673,  675683,  675699, 10, 16, 18, }, // Chile
    {  525,  501,  675717,  675726,  675741,  9, 15, 15, }, // China
    {  525,  507,  675756,  675773,  675796, 17, 23, 26, }, // Choibalsan
    {  525,  518,       0,  675822,       0,  0, 22,  0, }, // Christmas
    {  525,  528,       0,  675844,       0,  0, 18,  0, }, // Cocos
    {  525,  534,  675862,  675878,  675900, 16, 22, 24, }, // Colombia
    {  525,  543,  675924,  675940,  675962, 16, 22, 29, }, // Cook
    {  525,  548,  675991,  676005,  676025, 14, 20, 23, }, // Cuba
    {  525,  559,       0,  676048,       0,  0, 12,  0, }, // Davis
    {  525,  575,       0,  676060,       0,  0, 24,  0, }, // DumontDUrville
    {  525,  612,       0,  676084,       0,  0, 21,  0, }, // East_Timor
    {  525,  623,  676105,  676123,  676147, 18, 24, 26, }, // Easter
    {  525,  630,       0,  676173,       0,  0, 14,  0, }, // Ecuador
    {  525,  638,  676187,  676207,  676233, 20, 26, 29, }, // Europe_Central
    {  525,  653,  676262,  676284,  676312, 22, 28, 31, }, // Europe_Eastern
    {  525,  668,       0,  676343,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  525,  691,  676370,  676389,  676414, 19, 25, 28, }, // Europe_Western
    {  525,  706,  676442,  676463,  676490, 21, 27, 29, }, // Falkland
    {  525,  715,  676519,  676529,  676545, 10, 16, 19, }, // Fiji
    {  525,  720,       0,  676564,       0,  0, 17,  0, }, // French_Guiana
    {  525,  734,       0,  676581,       0,  0, 38,  0, }, // French_Southern
    {  525,  757,       0,  676619,       0,  0, 13,  0, }, // Galapagos
    {  525,  767,       0,  676632,       0,  0, 15,  0, }, // Gambier
    {  525,  775,  676647,  676663,  676685, 16, 22, 25, }, // Georgia
    {  525,  783,       0,  676710,       0,  0, 21,  0, }, // Gilbert_Islands
    {  525,  799,       0,  676731,       0,  0, 22,  0, }, // GMT
    {  525,  841,  676753,  676779,  676811, 26, 32, 34, }, // Greenland_Eastern
    {  525,  859,  676845,  676868,  676897, 23, 29, 31, }, // Greenland_Western
    {  525,  882,       0,  676928,       0,  0, 11,  0, }, // Gulf
    {  525,  887,       0,  676571,       0,  0, 10,  0, }, // Guyana
    {  525,  894,  676939,  676962,  676991, 23, 29, 32, }, // Hawaii_Aleutian
    {  525,  910,  677023,  677035,  677053, 12, 18, 21, }, // Hong_Kong
    {  525,  920,  677074,  677086,  677104, 12, 18, 21, }, // Hovd
    {  525,  925,       0,  677125,       0,  0, 15,  0, }, // India
    {  525,  931,       0,  677140,       0,  0, 19,  0, }, // Indian_Ocean
    {  525,  944,       0,  677159,       0,  0, 14,  0, }, // Indochina
    {  525,  954,       0,  677173,       0,  0, 26,  0, }, // Indonesia_Central
    {  525,  972,       0,  677199,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  525,  990,       0,  677227,       0,  0, 25,  0, }, // Indonesia_Western
    {  525, 1008,  677252,  677262,  677278, 10, 16, 14, }, // Iran
    {  525, 1019,  677292,  677309,  677332, 17, 23, 26, }, // Irkutsk
    {  525, 1027,  677358,  677372,  677392, 14, 20, 20, }, // Israel
    {  525, 1034,  677412,  677422,  677438, 10, 16, 16, }, // Japan
    {  525, 1058,       0,  677454,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  525, 1077,       0,  677477,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  525, 1106,  677497,  677511,  677531, 14, 20, 20, }, // Korea
    {  525, 1112,       0,  677551,       0,  0, 12,  0, }, // Kosrae
    {  525, 1119,  677563,  677585,  677613, 22, 28, 31, }, // Krasnoyarsk
    {  525, 1151,       0,  677644,       0,  0, 17,  0, }, // Kyrgystan
    {  525, 1161,       0,  677661,       0,  0, 16,  0, }, // Lanka
    {  525, 1175,       0,  677677,       0,  0, 17,  0, }, // Line_Islands
    {  525, 1188,  677694,  677711,  677734, 17, 23, 22, }, // Lord_Howe
    {  525, 1204,  677756,  677768,  677786, 12, 18, 21, }, // Magadan
    {  525, 1219,       0,  677807,       0,  0, 16,  0, }, // Malaysia
    {  525, 1228,       0,  677823,       0,  0, 16,  0, }, // Maldives
    {  525, 1237,       0,  677839,       0,  0, 17,  0, }, // Marquesas
    {  525, 1247,       0,  677856,       0,  0, 19,  0, }, // Marshall_Islands
    {  525, 1264,  677875,  677887,  677905, 12, 18, 20, }, // Mauritius
    {  525, 1274,       0,  677925,       0,  0, 13,  0, }, // Mawson
    {  525, 1281,  677938,  677961,  677990, 23, 29, 32, }, // Mexico_Pacific
    {  525, 1296,  678022,  678039,  678062, 17, 23, 26, }, // Mongolia
    {  525, 1305,  678088,  678101,  678120, 13, 19, 22, }, // Moscow
    {  525, 1312,       0,  678142,       0,  0, 15,  0, }, // Myanmar
    {  525, 1320,       0,  678157,       0,  0, 12,  0, }, // Nauru
    {  525, 1326,       0,  678169,       0,  0, 11,  0, }, // Nepal
    {  525, 1332,  678180,  678200,  678226, 20, 26, 29, }, // New_Caledonia
    {  525, 1346,  678255,  678269,  678289, 14, 20, 19, }, // New_Zealand
    {  525, 1358,  678308,  678327,  678352, 19, 25, 28, }, // Newfoundland
    {  525, 1371,       0,  678380,       0,  0, 10,  0, }, // Niue
    {  525, 1376,  678390,  678410,  678436, 20, 26, 29, }, // Norfolk
    {  525, 1384,  678465,  678492,  678525, 27, 33, 35, }, // Noronha
    {  525, 1406,  678560,  678579,  678604, 19, 25, 28, }, // Novosibirsk
    {  525, 1418,  678632,  678645,  678664, 13, 19, 22, }, // Omsk
    {  525, 1428,  678686,  678701,  678722, 15, 21, 23, }, // Pakistan
    {  525, 1437,       0,  678745,       0,  0, 11,  0, }, // Palau
    {  525, 1443,       0,  678756,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  525, 1460,  678780,  678793,  678812, 13, 19, 21, }, // Paraguay
    {  525, 1469,  678833,  678843,  678859, 10, 16, 18, }, // Peru
    {  525, 1474,  678877,  678890,  678909, 13, 19, 22, }, // Philippines
    {  525, 1486,       0,  678931,       0,  0, 20,  0, }, // Phoenix_Islands
    {  525, 1502,  678951,  678979,  679013, 28, 34, 37, }, // Pierre_Miquelon
    {  525, 1518,       0,  679050,       0,  0, 16,  0, }, // Pitcairn
    {  525, 1527,       0,  679066,       0,  0, 11,  0, }, // Ponape
    {  525, 1534,       0,  679077,       0,  0, 17,  0, }, // Pyongyang
    {  525, 1554,       0,  679094,       0,  0, 15,  0, }, // Reunion
    {  525, 1562,       0,  679109,       0,  0, 12,  0, }, // Rothera
    {  525, 1570,  679121,  679133,  679151, 12, 18, 21, }, // Sakhalin
    {  525, 1596,  679172,  679184,  679202, 12, 18, 21, }, // Samoa
    {  525, 1602,       0,  679223,       0,  0, 14,  0, }, // Seychelles
    {  525, 1624,       0,  679237,       0,  0, 16,  0, }, // Singapore
    {  525, 1634,       0,  679253,       0,  0, 19,  0, }, // Solomon
    {  525, 1642,       0,  679272,       0,  0, 18,  0, }, // South_Georgia
    {  525, 1656,       0,  679290,       0,  0, 13,  0, }, // Suriname
    {  525, 1676,       0,  679303,       0,  0, 12,  0, }, // Syowa
    {  525, 1682,       0,  679315,       0,  0, 12,  0, }, // Tahiti
    {  525, 1689,  679327,  679339,  679357, 12, 18, 18, }, // Taipei
    {  525, 1696,       0,  679375,       0,  0, 16,  0, }, // Tajikistan
    {  525, 1724,       0,  679391,       0,  0, 14,  0, }, // Tokelau
    {  525, 1732,  679405,  679416,  679433, 11, 17, 20, }, // Tonga
    {  525, 1738,       0,  679453,       0,  0, 10,  0, }, // Truk
    {  525, 1750,  679463,  679483,  679509, 20, 26, 29, }, // Turkmenistan
    {  525, 1763,       0,  679538,       0,  0, 12,  0, }, // Tuvalu
    {  525, 1777,  679550,  679563,  679582, 13, 19, 21, }, // Uruguay
    {  525, 1792,  679603,  679621,  679645, 18, 24, 27, }, // Uzbekistan
    {  525, 1803,  679672,  679684,  679702, 12, 18, 20, }, // Vanuatu
    {  525, 1811,       0,  679722,       0,  0, 16,  0, }, // Venezuela
    {  525, 1821,  679738,  679757,  679782, 19, 25, 28, }, // Vladivostok
    {  525, 1833,  679810,  679829,  679854, 19, 25, 28, }, // Volgograd
    {  525, 1843,       0,  679882,       0,  0, 14,  0, }, // Vostok
    {  525, 1850,       0,  679896,       0,  0, 16,  0, }, // Wake
    {  525, 1855,       0,  679912,       0,  0, 22,  0, }, // Wallis
    {  525, 1862,  679934,  679949,  679970, 15, 21, 24, }, // Yakutsk
    {  525, 1870,  679994,  680014,  680040, 20, 26, 29, }, // Yekaterinburg
    {  525, 1892,       0,  680069,       0,  0, 12,  0, }, // Yukon
    {  527,    0,  188886,  680081,  680104, 12, 23, 18, }, // Acre Slovak/Latin/Slovakia
    {  527,    5,       0,  680122,       0,  0, 12,  0, }, // Afghanistan
    {  527,   17,       0,  680134,       0,  0, 17,  0, }, // Africa_Central
    {  527,   32,       0,  188969,       0,  0, 18,  0, }, // Africa_Eastern
    {  527,   65,       0,  680151,       0,  0, 15,  0, }, // Africa_Southern
    {  527,   81,  189002,  680166,  680194, 17, 28, 23, }, // Africa_Western
    {  527,  107,  189070,  680217,  680240, 12, 23, 18, }, // Alaska
    {  527,  128,  680258,  680271,  680295, 13, 24, 19, }, // Almaty
    {  527,  135,  189176,  680314,  680338, 13, 24, 19, }, // Amazon
    {  527,  142,  680357,  680385,  680424, 28, 39, 34, }, // America_Central
    {  527,  158,  680458,  680485,  680523, 27, 38, 33, }, // America_Eastern
    {  527,  174,  189431,  680556,  680592, 25, 36, 31, }, // America_Mountain
    {  527,  191,  680623,  680653,  680694, 30, 41, 36, }, // America_Pacific
    {  527,  207,  680730,  680743,  680767, 13, 24, 19, }, // Anadyr
    {  527,  214,  189680,  680786,  680808, 11, 22, 17, }, // Apia
    {  527,  219,  680825,  680837,  680860, 12, 23, 18, }, // Aqtau
    {  527,  225,  680878,  680890,  680913, 12, 23, 18, }, // Aqtobe
    {  527,  232,  189836,  680931,  680953, 11, 22, 17, }, // Arabian
    {  527,  240,  680970,  680985,  681011, 15, 26, 21, }, // Argentina
    {  527,  250,  681032,  681053,  681085, 21, 32, 27, }, // Argentina_Western
    {  527,  268,  681112,  681124,  681147, 12, 23, 18, }, // Armenia
    {  527,  286,  190081,  681165,  681190, 14, 25, 20, }, // Atlantic
    {  527,  295,  681210,  681230,  681261, 20, 31, 26, }, // Australia_Central
    {  527,  313,  681287,  681315,  681354, 28, 39, 34, }, // Australia_CentralWestern
    {  527,  338,  681388,  681409,  681441, 21, 32, 27, }, // Australia_Eastern
    {  527,  356,  681468,  681488,  681519, 20, 31, 26, }, // Australia_Western
    {  527,  374,  681545,  681563,  681592, 18, 29, 24, }, // Azerbaijan
    {  527,  385,  190546,  681616,  681638, 11, 22, 17, }, // Azores
    {  527,  397,  681655,  681671,  681698, 16, 27, 22, }, // Bangladesh
    {  527,  415,       0,  681720,       0,  0, 13,  0, }, // Bhutan
    {  527,  422,       0,  681733,       0,  0, 14,  0, }, // Bolivia
    {  527,  437,  681747,  681760,  681784, 13, 24, 19, }, // Brasilia
    {  527,  454,       0,  190750,       0,  0, 13,  0, }, // Brunei
    {  527,  461,  190763,  681803,  681828, 14, 25, 20, }, // Cape_Verde
    {  527,  472,       0,  190822,       0,  0, 19,  0, }, // Casey
    {  527,  478,       0,  190841,       0,  0, 14,  0, }, // Chamorro
    {  527,  487,  190855,  681848,  681873, 14, 25, 20, }, // Chatham
    {  527,  495,  681893,  681903,  681924, 10, 21, 16, }, // Chile
    {  527,  501,  681940,  681950,  681971, 10, 21, 16, }, // China
    {  527,  507,  191011,  681987,  682014, 16, 27, 22, }, // Choibalsan
    {  527,  518,       0,  682036,       0,  0, 22,  0, }, // Christmas
    {  527,  528,       0,  682058,       0,  0, 23,  0, }, // Cocos
    {  527,  534,  191119,  682081,  682107, 15, 26, 21, }, // Colombia
    {  527,  543,  682128,  682150,  682183, 22, 33, 28, }, // Cook
    {  527,  548,  682211,  682223,  682246, 12, 23, 18, }, // Cuba
    {  527,  559,       0,  682264,       0,  0, 21,  0, }, // Davis
    {  527,  575,       0,  682285,       0,  0, 30,  0, }, // DumontDUrville
    {  527,  612,       0,  191363,       0,  0, 19,  0, }, // East_Timor
    {  527,  623,  682315,  682339,  682374, 24, 35, 30, }, // Easter
    {  527,  630,       0,  191474,       0,  0, 14,  0, }, // Ecuador
    {  527,  638,  682404,  682422,  682451, 18, 29, 24, }, // Europe_Central
    {  527,  653,  682475,  682494,  682524, 19, 30, 25, }, // Europe_Eastern
    {  527,  668,       0,  682549,       0,  0, 10,  0, }, // Europe_Further_Eastern
    {  527,  691,  682559,  682577,  682606, 18, 29, 24, }, // Europe_Western
    {  527,  706,  191728,  682630,  682656, 15, 26, 21, }, // Falkland
    {  527,  715,  191790,  682677,  682701, 13, 24, 19, }, // Fiji
    {  527,  720,       0,  682720,       0,  0, 22,  0, }, // French_Guiana
    {  527,  734,       0,  682742,       0,  0, 46,  0, }, // French_Southern
    {  527,  757,       0,  682788,       0,  0, 14,  0, }, // Galapagos
    {  527,  767,       0,  191930,       0,  0, 14,  0, }, // Gambier
    {  527,  775,  682802,  682815,  682839, 13, 24, 19, }, // Georgia
    {  527,  783,       0,  682858,       0,  0, 25,  0, }, // Gilbert_Islands
    {  527,  799,       0,  682883,       0,  0, 16,  0, }, // GMT
    {  527,  841,  682899,  682917,  682946, 18, 29, 24, }, // Greenland_Eastern
    {  527,  859,  682970,  682987,  683015, 17, 28, 23, }, // Greenland_Western
    {  527,  877,       0,  683038,       0,  0, 11,  0, }, // Guam
    {  527,  882,       0,  683049,       0,  0, 31,  0, }, // Gulf
    {  527,  887,       0,  191857,       0,  0, 12,  0, }, // Guyana
    {  527,  894,  192228,  683080,  683112, 21, 32, 27, }, // Hawaii_Aleutian
    {  527,  910,  683139,  683153,  683178, 14, 25, 20, }, // Hong_Kong
    {  527,  920,  683198,  683210,  683233, 12, 23, 18, }, // Hovd
    {  527,  925,       0,  192420,       0,  0, 11,  0, }, // India
    {  527,  931,       0,  683251,       0,  0, 19,  0, }, // Indian_Ocean
    {  527,  944,       0,  683270,       0,  0, 14,  0, }, // Indochina
    {  527,  954,       0,  683284,       0,  0, 20,  0, }, // Indonesia_Central
    {  527,  972,       0,  683304,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  527,  990,       0,  683325,       0,  0, 20,  0, }, // Indonesia_Western
    {  527, 1008,  683345,  683356,  683378, 11, 22, 17, }, // Iran
    {  527, 1019,  192572,  683395,  683418, 12, 23, 18, }, // Irkutsk
    {  527, 1027,  192625,  683436,  683460, 13, 24, 19, }, // Israel
    {  527, 1034,  192681,  683479,  683502, 12, 23, 18, }, // Japan
    {  527, 1040,  683520,  683547,  683585, 27, 38, 37, }, // Kamchatka
    {  527, 1058,       0,  683622,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  527, 1077,       0,  683646,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  527, 1106,  683669,  683681,  683704, 12, 23, 18, }, // Korea
    {  527, 1112,       0,  192935,       0,  0, 13,  0, }, // Kosrae
    {  527, 1119,  192948,  683722,  683749, 16, 27, 22, }, // Krasnoyarsk
    {  527, 1151,       0,  683771,       0,  0, 13,  0, }, // Kyrgystan
    {  527, 1161,       0,  683784,       0,  0, 13,  0, }, // Lanka
    {  527, 1175,       0,  683797,       0,  0, 24,  0, }, // Line_Islands
    {  527, 1188,  193062,  683821,  683854, 22, 33, 28, }, // Lord_Howe
    {  527, 1198,  683882,  683894,  683917, 12, 23, 18, }, // Macau
    {  527, 1204,  193198,  683935,  683960, 14, 25, 20, }, // Magadan
    {  527, 1219,       0,  683980,       0,  0, 15,  0, }, // Malaysia
    {  527, 1228,       0,  683995,       0,  0, 13,  0, }, // Maldives
    {  527, 1237,       0,  684008,       0,  0, 13,  0, }, // Marquesas
    {  527, 1247,       0,  684021,       0,  0, 26,  0, }, // Marshall_Islands
    {  527, 1264,  684047,  684062,  684088, 15, 26, 21, }, // Mauritius
    {  527, 1274,       0,  684109,       0,  0, 22,  0, }, // Mawson
    {  527, 1281,  684131,  684154,  684188, 23, 34, 29, }, // Mexico_Pacific
    {  527, 1296,  684217,  684233,  684260, 16, 27, 22, }, // Mongolia
    {  527, 1305,  684282,  684295,  684319, 13, 24, 19, }, // Moscow
    {  527, 1312,       0,  684338,       0,  0, 14,  0, }, // Myanmar
    {  527, 1320,       0,  684352,       0,  0, 12,  0, }, // Nauru
    {  527, 1326,       0,  684364,       0,  0, 12,  0, }, // Nepal
    {  527, 1332,  684376,  684394,  684423, 18, 29, 24, }, // New_Caledonia
    {  527, 1346,  193713,  684447,  684475, 17, 28, 23, }, // New_Zealand
    {  527, 1358,  193781,  684498,  684528, 19, 30, 25, }, // Newfoundland
    {  527, 1371,       0,  193855,       0,  0, 12,  0, }, // Niue
    {  527, 1376,  684553,  684566,  684590, 13, 24, 19, }, // Norfolk
    {  527, 1384,  684609,  684643,  684688, 34, 45, 40, }, // Noronha
    {  527, 1392,       0,  684728,       0,  0, 19,  0, }, // North_Mariana
    {  527, 1406,  194064,  684747,  684774, 16, 27, 22, }, // Novosibirsk
    {  527, 1418,  194129,  684796,  684816,  9, 20, 15, }, // Omsk
    {  527, 1428,  684831,  684846,  684872, 15, 26, 21, }, // Pakistan
    {  527, 1437,       0,  194235,       0,  0, 12,  0, }, // Palau
    {  527, 1443,       0,  684893,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  527, 1460,  684915,  684930,  684956, 15, 26, 21, }, // Paraguay
    {  527, 1469,  684977,  684990,  685014, 13, 24, 19, }, // Peru
    {  527, 1474,  685033,  685047,  685072, 14, 25, 20, }, // Philippines
    {  527, 1486,       0,  685092,       0,  0, 23,  0, }, // Phoenix_Islands
    {  527, 1502,  194467,  685115,  685148, 22, 33, 28, }, // Pierre_Miquelon
    {  527, 1518,       0,  685176,       0,  0, 26,  0, }, // Pitcairn
    {  527, 1527,       0,  194575,       0,  0, 12,  0, }, // Ponape
    {  527, 1534,       0,  685202,       0,  0, 17,  0, }, // Pyongyang
    {  527, 1544,  685219,  685234,  685260, 15, 26, 21, }, // Qyzylorda
    {  527, 1554,       0,  194667,       0,  0, 14,  0, }, // Reunion
    {  527, 1562,       0,  685281,       0,  0, 22,  0, }, // Rothera
    {  527, 1570,  194702,  685303,  685329, 15, 26, 21, }, // Sakhalin
    {  527, 1579,  685350,  685362,  685385, 12, 23, 18, }, // Samara
    {  527, 1596,  194817,  685403,  685426, 12, 23, 18, }, // Samoa
    {  527, 1602,       0,  194870,       0,  0, 14,  0, }, // Seychelles
    {  527, 1624,       0,  685444,       0,  0, 26,  0, }, // Singapore
    {  527, 1634,       0,  685470,       0,  0, 25,  0, }, // Solomon
    {  527, 1642,       0,  685495,       0,  0, 18,  0, }, // South_Georgia
    {  527, 1656,       0,  194941,       0,  0, 14,  0, }, // Suriname
    {  527, 1676,       0,  194955,       0,  0, 16,  0, }, // Syowa
    {  527, 1682,       0,  194971,       0,  0, 12,  0, }, // Tahiti
    {  527, 1689,  194983,  685513,  685539, 15, 26, 21, }, // Taipei
    {  527, 1696,       0,  685560,       0,  0, 12,  0, }, // Tajikistan
    {  527, 1724,       0,  195057,       0,  0, 14,  0, }, // Tokelau
    {  527, 1732,  195071,  685572,  685594, 11, 22, 17, }, // Tonga
    {  527, 1738,       0,  195121,       0,  0, 12,  0, }, // Truk
    {  527, 1750,  685611,  685625,  685650, 14, 25, 20, }, // Turkmenistan
    {  527, 1763,       0,  195192,       0,  0, 12,  0, }, // Tuvalu
    {  527, 1777,  685670,  685684,  685709, 14, 25, 20, }, // Uruguay
    {  527, 1792,  195263,  685729,  685751, 11, 22, 17, }, // Uzbekistan
    {  527, 1803,  195313,  685768,  685792, 13, 24, 19, }, // Vanuatu
    {  527, 1811,       0,  195369,       0,  0, 15,  0, }, // Venezuela
    {  527, 1821,  195384,  685811,  685839, 17, 28, 23, }, // Vladivostok
    {  527, 1833,  195452,  685862,  685889, 16, 27, 22, }, // Volgograd
    {  527, 1843,       0,  195517,       0,  0, 18,  0, }, // Vostok
    {  527, 1850,       0,  195535,       0,  0, 16,  0, }, // Wake
    {  527, 1855,       0,  685911,       0,  0, 28,  0, }, // Wallis
    {  527, 1862,  195578,  685939,  685962, 12, 23, 18, }, // Yakutsk
    {  527, 1870,  685980,  685999,  686029, 19, 30, 25, }, // Yekaterinburg
    {  527, 1892,       0,  195705,       0,  0, 12,  0, }, // Yukon
    {  528,    5,       0,  686054,       0,  0, 17,  0, }, // Afghanistan Slovenian/Latin/Slovenia
    {  528,   17,       0,  686071,       0,  0, 20,  0, }, // Africa_Central
    {  528,   32,       0,  686091,       0,  0, 18,  0, }, // Africa_Eastern
    {  528,   65,       0,  686109,       0,  0, 16,  0, }, // Africa_Southern
    {  528,   81,  686125,  686143,  686172, 18, 29, 26, }, // Africa_Western
    {  528,  107,  686198,  686209,  686231, 11, 22, 19, }, // Alaska
    {  528,  135,  686250,  686263,  686287, 13, 24, 21, }, // Amazon
    {  528,  142,  686308,  686321,  686345, 13, 24, 21, }, // America_Central
    {  528,  158,  686366,  686377,  686399, 11, 22, 19, }, // America_Eastern
    {  528,  174,  686418,  686428,  686449, 10, 21, 18, }, // America_Mountain
    {  528,  191,  686467,  686480,  686504, 13, 24, 21, }, // America_Pacific
    {  528,  207,  686525,  686538,  686562, 13, 24, 21, }, // Anadyr
    {  528,  214,  686583,  686592,  686612,  9, 20, 17, }, // Apia
    {  528,  232,  686629,  686640,  686662, 11, 22, 19, }, // Arabian
    {  528,  240,  686681,  686696,  686722, 15, 26, 23, }, // Argentina
    {  528,  250,  686745,  686768,  686802, 23, 34, 31, }, // Argentina_Western
    {  528,  268,  686833,  686845,  686868, 12, 23, 20, }, // Armenia
    {  528,  286,  686888,  686901,  686925, 13, 24, 21, }, // Atlantic
    {  528,  295,  686946,  686970,  687005, 24, 35, 32, }, // Australia_Central
    {  528,  313,  687037,  687069,  687112, 32, 43, 40, }, // Australia_CentralWestern
    {  528,  338,  687152,  687174,  687207, 22, 33, 30, }, // Australia_Eastern
    {  528,  356,  687237,  687259,  687292, 22, 33, 30, }, // Australia_Western
    {  528,  374,  687322,  687340,  687369, 18, 29, 26, }, // Azerbaijan
    {  528,  385,  687395,  687406,  687428, 11, 22, 19, }, // Azores
    {  528,  397,  687447,  687462,  687488, 15, 26, 23, }, // Bangladesh
    {  528,  415,       0,  687511,       0,  0, 12,  0, }, // Bhutan
    {  528,  422,       0,  687523,       0,  0, 14,  0, }, // Bolivia
    {  528,  437,  687537,  687550,  687574, 13, 24, 21, }, // Brasilia
    {  528,  454,       0,  687595,       0,  0, 13,  0, }, // Brunei
    {  528,  461,  687608,  687622,  687647, 14, 25, 22, }, // Cape_Verde
    {  528,  478,       0,  687669,       0,  0, 23,  0, }, // Chamorro
    {  528,  487,  687692,  687704,  687727, 12, 23, 20, }, // Chatham
    {  528,  495,  687747,  687757,  687778, 10, 21, 18, }, // Chile
    {  528,  501,  687796,  687808,  687831, 12, 23, 20, }, // China
    {  528,  507,  687851,  687867,  687894, 16, 27, 24, }, // Choibalsan
    {  528,  518,       0,  687918,       0,  0, 17,  0, }, // Christmas
    {  528,  528,       0,  687935,       0,  0, 19,  0, }, // Cocos
    {  528,  534,  687954,  687969,  687995, 15, 26, 23, }, // Colombia
    {  528,  543,  688018,  688035,  688063, 17, 28, 32, }, // Cook
    {  528,  548,  688095,  688107,  688130, 12, 23, 20, }, // Cuba
    {  528,  559,       0,  688150,       0,  0, 10,  0, }, // Davis
    {  528,  575,       0,  688160,       0,  0, 21,  0, }, // DumontDUrville
    {  528,  612,       0,  688181,       0,  0, 19,  0, }, // East_Timor
    {  528,  623,  688200,  688221,  688253, 21, 32, 29, }, // Easter
    {  528,  630,       0,  688282,       0,  0, 14,  0, }, // Ecuador
    {  528,  638,  688296,  688315,  688345, 19, 30, 27, }, // Europe_Central
    {  528,  653,  688372,  688391,  688421, 19, 30, 27, }, // Europe_Eastern
    {  528,  668,       0,  688448,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  528,  691,  688475,  688494,  688524, 19, 30, 27, }, // Europe_Western
    {  528,  706,  688551,  688574,  688608, 23, 34, 31, }, // Falkland
    {  528,  715,  688639,  688652,  688676, 13, 24, 21, }, // Fiji
    {  528,  720,       0,  688697,       0,  0, 22,  0, }, // French_Guiana
    {  528,  734,       0,  688719,       0,  0, 34,  0, }, // French_Southern
    {  528,  757,       0,  688753,       0,  0, 13,  0, }, // Galapagos
    {  528,  767,       0,  688766,       0,  0, 14,  0, }, // Gambier
    {  528,  775,  688780,  688793,  688817, 13, 24, 21, }, // Georgia
    {  528,  783,       0,  688838,       0,  0, 21,  0, }, // Gilbert_Islands
    {  528,  799,       0,  688859,       0,  0, 22,  0, }, // GMT
    {  528,  841,  688881,  688903,  688936, 22, 33, 30, }, // Greenland_Eastern
    {  528,  859,  688966,  688988,  689021, 22, 33, 30, }, // Greenland_Western
    {  528,  882,       0,  689051,       0,  0, 23,  0, }, // Gulf
    {  528,  887,       0,  689074,       0,  0, 13,  0, }, // Guyana
    {  528,  894,  689087,  689108,  689140, 21, 32, 29, }, // Hawaii_Aleutian
    {  528,  910,  689169,  689183,  689208, 14, 25, 22, }, // Hong_Kong
    {  528,  920,  689230,  689241,  689263, 11, 22, 19, }, // Hovd
    {  528,  925,       0,  689282,       0,  0, 23,  0, }, // India
    {  528,  931,       0,  689305,       0,  0, 20,  0, }, // Indian_Ocean
    {  528,  944,       0,  689325,       0,  0, 16,  0, }, // Indochina
    {  528,  954,       0,  689341,       0,  0, 25,  0, }, // Indonesia_Central
    {  528,  972,       0,  689366,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  528,  990,       0,  689390,       0,  0, 24,  0, }, // Indonesia_Western
    {  528, 1008,  689414,  689425,  689447, 11, 22, 19, }, // Iran
    {  528, 1019,  689466,  689478,  689501, 12, 23, 20, }, // Irkutsk
    {  528, 1027,  689521,  689534,  689558, 13, 24, 21, }, // Israel
    {  528, 1034,  689579,  689591,  689614, 12, 23, 20, }, // Japan
    {  528, 1040,  689634,  689661,  689699, 27, 38, 35, }, // Kamchatka
    {  528, 1058,       0,  689734,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  528, 1077,       0,  689758,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  528, 1106,  689782,  689794,  689817, 12, 23, 20, }, // Korea
    {  528, 1112,       0,  689837,       0,  0, 13,  0, }, // Kosrae
    {  528, 1119,  689850,  689866,  689893, 16, 27, 24, }, // Krasnoyarsk
    {  528, 1151,       0,  689917,       0,  0, 18,  0, }, // Kyrgystan
    {  528, 1175,       0,  689935,       0,  0, 21,  0, }, // Line_Islands
    {  528, 1188,  689956,  689975,  690005, 19, 30, 27, }, // Lord_Howe
    {  528, 1204,  690032,  690046,  690071, 14, 25, 22, }, // Magadan
    {  528, 1219,       0,  690093,       0,  0, 14,  0, }, // Malaysia
    {  528, 1228,       0,  690107,       0,  0, 13,  0, }, // Maldives
    {  528, 1237,       0,  690120,       0,  0, 19,  0, }, // Marquesas
    {  528, 1247,       0,  690139,       0,  0, 22,  0, }, // Marshall_Islands
    {  528, 1264,  690161,  690176,  690202, 15, 26, 23, }, // Mauritius
    {  528, 1274,       0,  690225,       0,  0, 13,  0, }, // Mawson
    {  528, 1281,  690238,  690259,  690291, 21, 32, 29, }, // Mexico_Pacific
    {  528, 1296,  690320,  690336,  690363, 16, 27, 24, }, // Mongolia
    {  528, 1305,  690387,  690400,  690424, 13, 24, 21, }, // Moscow
    {  528, 1312,       0,  690445,       0,  0, 14,  0, }, // Myanmar
    {  528, 1320,       0,  690459,       0,  0, 13,  0, }, // Nauru
    {  528, 1326,       0,  690472,       0,  0, 12,  0, }, // Nepal
    {  528, 1332,  690484,  690504,  690535, 20, 31, 28, }, // New_Caledonia
    {  528, 1346,  690563,  690580,  690608, 17, 28, 25, }, // New_Zealand
    {  528, 1358,  690633,  690652,  690682, 19, 30, 27, }, // Newfoundland
    {  528, 1371,       0,  690709,       0,  0, 12,  0, }, // Niue
    {  528, 1376,  690721,  690741,  690772, 20, 31, 28, }, // Norfolk
    {  528, 1384,  690800,  690824,  690859, 24, 35, 32, }, // Noronha
    {  528, 1406,  690891,  690907,  690934, 16, 27, 24, }, // Novosibirsk
    {  528, 1418,  690958,  690967,  690987,  9, 20, 17, }, // Omsk
    {  528, 1428,  691004,  691019,  691045, 15, 26, 23, }, // Pakistan
    {  528, 1437,       0,  691068,       0,  0, 12,  0, }, // Palau
    {  528, 1443,       0,  691080,       0,  0, 13,  0, }, // Papua_New_Guinea
    {  528, 1460,  691093,  691108,  691134, 15, 26, 23, }, // Paraguay
    {  528, 1469,  691157,  691169,  691192, 12, 23, 20, }, // Peru
    {  528, 1474,  691212,  691226,  691251, 14, 25, 22, }, // Philippines
    {  528, 1486,       0,  691273,       0,  0, 18,  0, }, // Phoenix_Islands
    {  528, 1502,  691291,  691320,  691360, 29, 40, 37, }, // Pierre_Miquelon
    {  528, 1518,       0,  691397,       0,  0, 15,  0, }, // Pitcairn
    {  528, 1527,       0,  691412,       0,  0, 12,  0, }, // Ponape
    {  528, 1534,       0,  691424,       0,  0, 15,  0, }, // Pyongyang
    {  528, 1554,       0,  691439,       0,  0, 14,  0, }, // Reunion
    {  528, 1562,       0,  691453,       0,  0, 13,  0, }, // Rothera
    {  528, 1570,  691466,  691480,  691505, 14, 25, 22, }, // Sakhalin
    {  528, 1579,  691527,  691539,  691562, 12, 23, 20, }, // Samara
    {  528, 1596,  691582,  691595,  691619, 13, 24, 21, }, // Samoa
    {  528, 1602,       0,  691640,       0,  0, 13,  0, }, // Seychelles
    {  528, 1624,       0,  691653,       0,  0, 26,  0, }, // Singapore
    {  528, 1634,       0,  691679,       0,  0, 20,  0, }, // Solomon
    {  528, 1642,       0,  691699,       0,  0, 19,  0, }, // South_Georgia
    {  528, 1656,       0,  691718,       0,  0, 14,  0, }, // Suriname
    {  528, 1676,       0,  691732,       0,  0, 10,  0, }, // Syowa
    {  528, 1682,       0,  691742,       0,  0, 14,  0, }, // Tahiti
    {  528, 1689,  691756,  691769,  691793, 13, 24, 21, }, // Taipei
    {  528, 1696,       0,  691814,       0,  0, 18,  0, }, // Tajikistan
    {  528, 1724,       0,  691832,       0,  0, 14,  0, }, // Tokelau
    {  528, 1732,  691846,  691859,  691883, 13, 24, 21, }, // Tonga
    {  528, 1738,       0,  691904,       0,  0, 15,  0, }, // Truk
    {  528, 1750,  691919,  691938,  691968, 19, 30, 27, }, // Turkmenistan
    {  528, 1763,       0,  691995,       0,  0, 14,  0, }, // Tuvalu
    {  528, 1777,  692009,  692023,  692048, 14, 25, 22, }, // Uruguay
    {  528, 1792,  692070,  692087,  692115, 17, 28, 25, }, // Uzbekistan
    {  528, 1803,  692140,  692155,  692181, 15, 26, 23, }, // Vanuatu
    {  528, 1811,       0,  692204,       0,  0, 15,  0, }, // Venezuela
    {  528, 1821,  692219,  692236,  692264, 17, 28, 25, }, // Vladivostok
    {  528, 1833,  692289,  692305,  692332, 16, 27, 24, }, // Volgograd
    {  528, 1843,       0,  692356,       0,  0, 12,  0, }, // Vostok
    {  528, 1850,       0,  692368,       0,  0, 14,  0, }, // Wake
    {  528, 1855,       0,  692382,       0,  0, 21,  0, }, // Wallis
    {  528, 1862,  692403,  692415,  692438, 12, 23, 20, }, // Yakutsk
    {  528, 1870,  692458,  692477,  692507, 19, 30, 27, }, // Yekaterinburg
    {  528, 1892,       0,  692534,       0,  0, 12,  0, }, // Yukon
    {  530,    0,  692546,  692559,  692586, 13, 27, 22, }, // Acre Somali/Latin/Somalia
    {  530,    5,       0,  692608,       0,  0, 21,  0, }, // Afghanistan
    {  530,   17,       0,  692629,       0,  0, 24,  0, }, // Africa_Central
    {  530,   32,       0,  692653,       0,  0, 21,  0, }, // Africa_Eastern
    {  530,   65,       0,  692674,       0,  0, 38,  0, }, // Africa_Southern
    {  530,   81,  692712,  692736,  692774, 24, 38, 35, }, // Africa_Western
    {  530,  107,  692809,  692823,  692851, 14, 28, 27, }, // Alaska
    {  530,  128,  692878,  692892,  692920, 14, 28, 35, }, // Almaty
    {  530,  135,  692955,  692969,  692997, 14, 28, 25, }, // Amazon
    {  530,  142,  693022,  693058,  693108, 36, 50, 49, }, // America_Central
    {  530,  158,  693157,  693193,  693240, 36, 47, 46, }, // America_Eastern
    {  530,  174,  693286,  693326,  693377, 40, 51, 49, }, // America_Mountain
    {  530,  191,  693426,  693464,  693513, 38, 49, 48, }, // America_Pacific
    {  530,  207,  693561,  693576,  693605, 15, 29, 24, }, // Anadyr
    {  530,  214,  693629,  693642,  693669, 13, 27, 26, }, // Apia
    {  530,  219,  693695,  693708,  693735, 13, 27, 31, }, // Aqtau
    {  530,  225,  693766,  693780,  693808, 14, 28, 32, }, // Aqtobe
    {  530,  232,  693840,  693855,  693884, 15, 29, 28, }, // Arabian
    {  530,  240,  693912,  693929,  693961, 17, 32, 29, }, // Argentina
    {  530,  250,  693990,  694018,  694060, 28, 42, 39, }, // Argentina_Western
    {  530,  268,  694099,  694116,  694147, 17, 31, 28, }, // Armenia
    {  530,  286,  694175,  694214,  694264, 39, 50, 49, }, // Atlantic
    {  530,  295,  694313,  694342,  694385, 29, 43, 42, }, // Australia_Central
    {  530,  313,  694427,  694466,  694519, 39, 53, 48, }, // Australia_CentralWestern
    {  530,  338,  694567,  694593,  694635, 26, 42, 39, }, // Australia_Eastern
    {  530,  356,  694674,  694703,  694746, 29, 43, 42, }, // Australia_Western
    {  530,  374,  694788,  694806,  694838, 18, 32, 29, }, // Azerbaijan
    {  530,  385,  694867,  694881,  694909, 14, 28, 25, }, // Azores
    {  530,  397,  694934,  694953,  694986, 19, 33, 30, }, // Bangladesh
    {  530,  415,       0,  695016,       0,  0, 14,  0, }, // Bhutan
    {  530,  422,       0,  695030,       0,  0, 17,  0, }, // Bolivia
    {  530,  437,  695047,  695066,  695099, 19, 33, 30, }, // Brasilia
    {  530,  454,       0,  695129,       0,  0, 27,  0, }, // Brunei
    {  530,  461,  695156,  695175,  695208, 19, 33, 30, }, // Cape_Verde
    {  530,  478,       0,  695238,       0,  0, 14,  0, }, // Chamorro
    {  530,  487,  695252,  695266,  695294, 14, 28, 27, }, // Chatham
    {  530,  495,  695321,  695333,  695359, 12, 26, 23, }, // Chile
    {  530,  501,  695382,  695398,  695428, 16, 30, 29, }, // China
    {  530,  507,  695457,  695474,  695505, 17, 31, 28, }, // Choibalsan
    {  530,  518,       0,  695533,       0,  0, 23,  0, }, // Christmas
    {  530,  528,       0,  695556,       0,  0, 20,  0, }, // Cocos
    {  530,  534,  695576,  695593,  695624, 17, 31, 28, }, // Colombia
    {  530,  543,  695652,  695673,  695708, 21, 35, 39, }, // Cook
    {  530,  548,  695747,  695760,  695787, 13, 27, 26, }, // Cuba
    {  530,  559,       0,  695813,       0,  0, 13,  0, }, // Davis
    {  530,  575,       0,  695826,       0,  0, 26,  0, }, // DumontDUrville
    {  530,  612,       0,  695852,       0,  0, 19,  0, }, // East_Timor
    {  530,  623,  695871,  695892,  695927, 21, 35, 32, }, // Easter
    {  530,  630,       0,  695959,       0,  0, 17,  0, }, // Ecuador
    {  530,  638,  695976,  695999,  696036, 23, 37, 34, }, // Europe_Central
    {  530,  653,  696070,  696090,  696124, 20, 34, 31, }, // Europe_Eastern
    {  530,  668,       0,  696155,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  530,  691,  696182,  696205,  696242, 23, 37, 34, }, // Europe_Western
    {  530,  706,  696276,  696302,  696342, 26, 40, 37, }, // Falkland
    {  530,  715,  696379,  696391,  696417, 12, 26, 23, }, // Fiji
    {  530,  720,       0,  696440,       0,  0, 21,  0, }, // French_Guiana
    {  530,  734,       0,  696461,       0,  0, 42,  0, }, // French_Southern
    {  530,  757,       0,  696503,       0,  0, 17,  0, }, // Galapagos
    {  530,  767,       0,  696520,       0,  0, 16,  0, }, // Gambier
    {  530,  775,  696536,  696552,  696582, 16, 30, 27, }, // Georgia
    {  530,  783,       0,  696609,       0,  0, 23,  0, }, // Gilbert_Islands
    {  530,  799,       0,  696632,       0,  0, 18,  0, }, // GMT
    {  530,  841,  696650,  696678,  696717, 28, 39, 36, }, // Greenland_Eastern
    {  530,  859,  696753,  696781,  696823, 28, 42, 39, }, // Greenland_Western
    {  530,  882,       0,  696862,       0,  0, 15,  0, }, // Gulf
    {  530,  887,       0,  696877,       0,  0, 15,  0, }, // Guyana
    {  530,  894,  696892,  696916,  696954, 24, 38, 37, }, // Hawaii_Aleutian
    {  530,  910,  696991,  697010,  697043, 19, 33, 30, }, // Hong_Kong
    {  530,  920,  697073,  697086,  697113, 13, 27, 24, }, // Hovd
    {  530,  925,       0,  697137,       0,  0, 29,  0, }, // India
    {  530,  931,       0,  697166,       0,  0, 25,  0, }, // Indian_Ocean
    {  530,  944,       0,  697191,       0,  0, 18,  0, }, // Indochina
    {  530,  954,       0,  697209,       0,  0, 30,  0, }, // Indonesia_Central
    {  530,  972,       0,  697239,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  530,  990,       0,  697259,       0,  0, 30,  0, }, // Indonesia_Western
    {  530, 1008,  697289,  697303,  697331, 14, 28, 27, }, // Iran
    {  530, 1019,  697358,  697374,  697404, 16, 30, 27, }, // Irkutsk
    {  530, 1027,  697431,  697447,  697477, 16, 30, 29, }, // Israel
    {  530, 1034,  697506,  697520,  697548, 14, 28, 27, }, // Japan
    {  530, 1040,  697575,  697608,  697655, 33, 47, 42, }, // Kamchatka
    {  530, 1058,       0,  697697,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  530, 1077,       0,  697724,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  530, 1106,  697753,  697768,  697797, 15, 29, 28, }, // Korea
    {  530, 1112,       0,  697825,       0,  0, 14,  0, }, // Kosrae
    {  530, 1119,  697839,  697860,  697895, 21, 35, 32, }, // Krasnoyarsk
    {  530, 1151,       0,  697927,       0,  0, 20,  0, }, // Kyrgystan
    {  530, 1175,       0,  697947,       0,  0, 19,  0, }, // Line_Islands
    {  530, 1188,  697966,  697981,  698010, 15, 29, 28, }, // Lord_Howe
    {  530, 1204,  698038,  698052,  698081, 14, 29, 26, }, // Magadan
    {  530, 1219,       0,  698107,       0,  0, 18,  0, }, // Malaysia
    {  530, 1228,       0,  698125,       0,  0, 16,  0, }, // Maldives
    {  530, 1237,       0,  698141,       0,  0, 18,  0, }, // Marquesas
    {  530, 1247,       0,  698159,       0,  0, 26,  0, }, // Marshall_Islands
    {  530, 1264,  698185,  698204,  698237, 19, 33, 30, }, // Mauritius
    {  530, 1274,       0,  698267,       0,  0, 14,  0, }, // Mawson
    {  530, 1281,  698281,  698308,  698349, 27, 41, 40, }, // Mexico_Pacific
    {  530, 1296,  698389,  698409,  698443, 20, 34, 31, }, // Mongolia
    {  530, 1305,  698474,  698488,  698516, 14, 28, 25, }, // Moscow
    {  530, 1312,       0,  698541,       0,  0, 17,  0, }, // Myanmar
    {  530, 1320,       0,  698558,       0,  0, 14,  0, }, // Nauru
    {  530, 1326,       0,  698572,       0,  0, 15,  0, }, // Nepal
    {  530, 1332,  698587,  698610,  698648, 23, 38, 35, }, // New_Caledonia
    {  530, 1346,  698683,  698704,  698739, 21, 35, 34, }, // New_Zealand
    {  530, 1358,  698773,  698794,  698829, 21, 35, 34, }, // Newfoundland
    {  530, 1371,       0,  698587,       0,  0, 13,  0, }, // Niue
    {  530, 1376,  698863,  698885,  698921, 22, 36, 34, }, // Norfolk
    {  530, 1384,  698955,  698983,  699026, 28, 43, 40, }, // Noronha
    {  530, 1406,  699066,  699086,  699120, 20, 34, 31, }, // Novosibirsk
    {  530, 1418,  699151,  699163,  699189, 12, 26, 23, }, // Omsk
    {  530, 1428,  699212,  699229,  699260, 17, 31, 28, }, // Pakistan
    {  530, 1437,       0,  699288,       0,  0, 13,  0, }, // Palau
    {  530, 1443,       0,  699301,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  530, 1460,  699327,  699344,  699375, 17, 31, 28, }, // Paraguay
    {  530, 1469,  699403,  699416,  699443, 13, 27, 24, }, // Peru
    {  530, 1474,  699467,  699483,  699513, 16, 30, 27, }, // Philippines
    {  530, 1486,       0,  699540,       0,  0, 25,  0, }, // Phoenix_Islands
    {  530, 1502,  699565,  699594,  699634, 29, 40, 42, }, // Pierre_Miquelon
    {  530, 1518,       0,  699676,       0,  0, 15,  0, }, // Pitcairn
    {  530, 1527,       0,  699691,       0,  0, 14,  0, }, // Ponape
    {  530, 1534,       0,  699705,       0,  0, 18,  0, }, // Pyongyang
    {  530, 1544,  699723,  699740,  699771, 17, 31, 35, }, // Qyzylorda
    {  530, 1554,       0,  699806,       0,  0, 18,  0, }, // Reunion
    {  530, 1562,       0,  699824,       0,  0, 14,  0, }, // Rothera
    {  530, 1570,  699838,  699854,  699884, 16, 30, 27, }, // Sakhalin
    {  530, 1579,  699911,  699926,  699955, 15, 29, 24, }, // Samara
    {  530, 1596,  699979,  699992,  700019, 13, 27, 26, }, // Samoa
    {  530, 1602,       0,  700045,       0,  0, 17,  0, }, // Seychelles
    {  530, 1624,       0,  700062,       0,  0, 17,  0, }, // Singapore
    {  530, 1634,       0,  700079,       0,  0, 24,  0, }, // Solomon
    {  530, 1642,       0,  700103,       0,  0, 21,  0, }, // South_Georgia
    {  530, 1656,       0,  700124,       0,  0, 16,  0, }, // Suriname
    {  530, 1676,       0,  700140,       0,  0, 14,  0, }, // Syowa
    {  530, 1682,       0,  700154,       0,  0, 14,  0, }, // Tahiti
    {  530, 1689,  700168,  700182,  700210, 14, 28, 27, }, // Taipei
    {  530, 1696,       0,  700237,       0,  0, 18,  0, }, // Tajikistan
    {  530, 1724,       0,  700255,       0,  0, 15,  0, }, // Tokelau
    {  530, 1732,  700270,  700283,  700310, 13, 27, 24, }, // Tonga
    {  530, 1738,       0,  700334,       0,  0, 12,  0, }, // Truk
    {  530, 1750,  700346,  700367,  700401, 21, 34, 31, }, // Turkmenistan
    {  530, 1763,       0,  700432,       0,  0, 14,  0, }, // Tuvalu
    {  530, 1777,  700446,  700462,  700492, 16, 30, 27, }, // Uruguay
    {  530, 1792,  700519,  700537,  700569, 18, 32, 29, }, // Uzbekistan
    {  530, 1803,  700598,  700613,  700642, 15, 29, 26, }, // Vanuatu
    {  530, 1811,       0,  700668,       0,  0, 19,  0, }, // Venezuela
    {  530, 1821,  700687,  700707,  700741, 20, 34, 31, }, // Vladivostok
    {  530, 1833,  700772,  700789,  700820, 17, 31, 28, }, // Volgograd
    {  530, 1843,       0,  700848,       0,  0, 14,  0, }, // Vostok
    {  530, 1850,       0,  700862,       0,  0, 21,  0, }, // Wake
    {  530, 1855,       0,  700883,       0,  0, 22,  0, }, // Wallis
    {  530, 1862,  700905,  700920,  700947, 15, 27, 24, }, // Yakutsk
    {  530, 1870,  700971,  700992,  701027, 21, 35, 32, }, // Yekaterinburg
    {  530, 1892,       0,  701059,       0,  0, 13,  0, }, // Yukon
    {  541,  107,  361414,  701072,  701095, 14, 23, 24, }, // Alaska Spanish/Latin/Spain
    {  541,  142,   37394,  285677,   37467, 12, 21, 22, }, // America_Central
    {  541,  158,   37509,  701119,   37584, 13, 22, 23, }, // America_Eastern
    {  541,  174,  701141,  701169,  701206, 28, 37, 38, }, // America_Mountain
    {  541,  191,  701244,  701261,  701287, 17, 26, 27, }, // America_Pacific
    {  541,  214,  286164,  286176,  701314, 12, 21, 25, }, // Apia
    {  541,  286,  701339,  701357,  701384, 18, 27, 28, }, // Atlantic
    {  541,  478,       0,  701412,       0,  0, 25,  0, }, // Chamorro
    {  541,  638,  363360,  701437,  701468, 22, 31, 32, }, // Europe_Central
    {  541,  653,  363440,  701500,  701532, 23, 32, 33, }, // Europe_Eastern
    {  541,  691,  363552,  701565,  701599, 25, 34, 35, }, // Europe_Western
    {  541,  734,       0,  701634,       0,  0, 51,  0, }, // French_Southern
    {  541,  799,       0,  701685,       0,  0, 31,  0, }, // GMT
    {  541,  894,  701716,  701740,  701773, 24, 33, 34, }, // Hawaii_Aleutian
    {  541,  925,       0,  701807,       0,  0, 25,  0, }, // India
    {  541,  931,       0,  701832,       0,  0, 22,  0, }, // Indian_Ocean
    {  541, 1237,       0,  365030,       0,  0, 17,  0, }, // Marquesas
    {  541, 1312,       0,  290305,       0,  0, 15,  0, }, // Myanmar
    {  541, 1486,       0,  701854,       0,  0, 23,  0, }, // Phoenix_Islands
    {  542,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Argentina
    {  542,  240,  361826,  701950,  701976, 17, 26, 27, }, // Argentina
    {  542,  250,  361893,  702003,  702040, 28, 37, 38, }, // Argentina_Western
    {  542,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  542,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  542,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  542,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  542,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  542,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  542,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  542,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  542, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  542, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  542, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  542, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  542, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  543,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Belize
    {  543,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  543,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  543,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  543,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  543,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  543,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  543,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  543,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  543, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  543, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  543, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  543, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  543, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  544,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Bolivia
    {  544,  422,       0,  287104,       0,  0, 15,  0, }, // Bolivia
    {  544,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  544,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  544,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  544,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  544,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  544,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  544,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  544,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  544, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  544, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  544, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  544, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  544, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  545,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Brazil
    {  545,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  545,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  545,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  545,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  545,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  545,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  545,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  545,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  545, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  545, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  545, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  545, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  545, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  548,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Chile
    {  548,  495,  287367,  287380,  702773, 13, 22, 23, }, // Chile
    {  548,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  548,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  548,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  548,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  548,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  548,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  548,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  548,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  548, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  548, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  548, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  548, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  548, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  549,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Colombia
    {  549,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  549,  534,  287595,  287611,  702796, 16, 25, 26, }, // Colombia
    {  549,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  549,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  549,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  549,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  549,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  549,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  549,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  549, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  549, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  549, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  549, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  549, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  550,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Costa Rica
    {  550,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  550,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  550,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  550,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  550,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  550,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  550,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  550,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  550, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  550, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  550, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  550, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  550, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  551,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Cuba
    {  551,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  551,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  551,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  551,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  551,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  551,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  551,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  551,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  551, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  551, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  551, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  551, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  551, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  552,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Dominican Republic
    {  552,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  552,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  552,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  552,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  552,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  552,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  552,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  552,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  552, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  552, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  552, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  552, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  552, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  553,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Ecuador
    {  553,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  553,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  553,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  553,  630,       0,  287936,       0,  0, 15,  0, }, // Ecuador
    {  553,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  553,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  553,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  553,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  553,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  553, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  553, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  553, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  553, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  553, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  554,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/El Salvador
    {  554,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  554,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  554,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  554,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  554,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  554,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  554,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  554,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  554, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  554, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  554, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  554, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  554, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  556,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Guatemala
    {  556,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  556,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  556,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  556,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  556,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  556,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  556,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  556,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  556, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  556, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  556, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  556, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  556, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  557,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Honduras
    {  557,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  557,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  557,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  557,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  557,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  557,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  557,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  557,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  557, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  557, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  557, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  557, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  557, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  558,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Latin America
    {  558,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  558,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  558,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  558,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  558,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  558,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  558,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  558,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  558, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  558, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  558, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  558, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  558, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  559,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Mexico
    {  559,  518,       0,  702822,       0,  0, 26,  0, }, // Christmas
    {  559,  623,  702848,  702870,  702904, 22, 34, 35, }, // Easter
    {  559,  706,  702939,  702961,  702992, 22, 31, 32, }, // Falkland
    {  559,  783,       0,  703024,       0,  0, 25,  0, }, // Gilbert_Islands
    {  559, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  559, 1850,       0,  703049,       0,  0, 20,  0, }, // Wake
    {  560,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Nicaragua
    {  560,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  560,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  560,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  560,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  560,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  560,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  560,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  560,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  560, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  560, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  560, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  560, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  560, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  561,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Panama
    {  561,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  561,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  561,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  561,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  561,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  561,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  561,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  561,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  561, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  561, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  561, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  561, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  561, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  562,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Paraguay
    {  562,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  562,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  562,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  562,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  562,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  562,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  562,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  562,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  562, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  562, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  562, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  562, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  562, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  563,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Peru
    {  563,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  563,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  563,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  563,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  563,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  563,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  563,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  563,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  563, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  563, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  563, 1469,  703069,  703081,  703102, 12, 21, 22, }, // Peru
    {  563, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  563, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  563, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  565,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Puerto Rico
    {  565,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  565,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  565,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  565,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  565,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  565,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  565,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  565,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  565, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  565, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  565, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  565, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  565, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  566,    0,  703124,  703136,  703157, 12, 21, 22, }, // Acre Spanish/Latin/United States
    {  566,    5,       0,  285344,       0,  0, 18,  0, }, // Afghanistan
    {  566,   17,       0,  703179,       0,  0, 22,  0, }, // Africa_Central
    {  566,   32,       0,  703201,       0,  0, 23,  0, }, // Africa_Eastern
    {  566,   65,       0,  703224,       0,  0, 17,  0, }, // Africa_Southern
    {  566,   81,  703241,  703266,  703300, 25, 34, 35, }, // Africa_Western
    {  566,  107,  361414,  701072,  701095, 14, 23, 24, }, // Alaska
    {  566,  135,  703335,  703352,  703378, 17, 26, 27, }, // Amazon
    {  566,  142,   37394,  285677,   37467, 12, 21, 22, }, // America_Central
    {  566,  158,   37509,  701119,   37584, 13, 22, 23, }, // America_Eastern
    {  566,  174,  701141,  701169,  701206, 28, 37, 38, }, // America_Mountain
    {  566,  191,  701244,  701261,  701287, 17, 26, 27, }, // America_Pacific
    {  566,  207,  703405,  703419,  703442, 14, 23, 24, }, // Anadyr
    {  566,  214,  286164,  286176,  703466, 12, 21, 22, }, // Apia
    {  566,  219,  703488,  703501,  703523, 13, 22, 23, }, // Aqtau
    {  566,  225,  703546,  703560,  703583, 14, 23, 24, }, // Aqtobe
    {  566,  232,  703607,  703621,  703644, 14, 23, 24, }, // Arabian
    {  566,  240,  361826,  701950,  701976, 17, 26, 27, }, // Argentina
    {  566,  250,  361893,  702003,  702040, 28, 37, 38, }, // Argentina_Western
    {  566,  268,  286437,  286452,  703668, 15, 24, 25, }, // Armenia
    {  566,  286,  701339,  701357,  701384, 18, 27, 28, }, // Atlantic
    {  566,  295,  362092,  703693,  703727, 25, 34, 35, }, // Australia_Central
    {  566,  313,  703762,  703795,  703837, 33, 42, 43, }, // Australia_CentralWestern
    {  566,  338,  362304,  703880,  703915, 26, 35, 36, }, // Australia_Eastern
    {  566,  356,  362398,  703951,  703988, 28, 37, 38, }, // Australia_Western
    {  566,  374,  704026,  704044,  704071, 18, 27, 28, }, // Azerbaijan
    {  566,  385,  704099,  704117,  704144, 18, 27, 28, }, // Azores
    {  566,  397,  704172,  704189,  704215, 17, 26, 27, }, // Bangladesh
    {  566,  415,       0,  287091,       0,  0, 13,  0, }, // Bhutan
    {  566,  422,       0,  287104,       0,  0, 15,  0, }, // Bolivia
    {  566,  437,  287119,  287135,  704242, 16, 25, 26, }, // Brasilia
    {  566,  454,       0,  704268,       0,  0, 14,  0, }, // Brunei
    {  566,  461,  287210,  287228,  704282, 18, 27, 28, }, // Cape_Verde
    {  566,  478,       0,  704310,       0,  0, 16,  0, }, // Chamorro
    {  566,  487,  287304,  287319,  704326, 15, 24, 25, }, // Chatham
    {  566,  495,  287367,  287380,  702773, 13, 22, 23, }, // Chile
    {  566,  501,  362930,  704351,  704373, 13, 22, 23, }, // China
    {  566,  507,  287481,  287499,  704396, 18, 27, 28, }, // Choibalsan
    {  566,  518,       0,  704424,       0,  0, 26,  0, }, // Christmas
    {  566,  528,       0,  704450,       0,  0, 23,  0, }, // Cocos
    {  566,  534,  287595,  287611,  702796, 16, 25, 26, }, // Colombia
    {  566,  543,  704473,  704495,  704526, 22, 31, 38, }, // Cook
    {  566,  548,  287742,  287754,  704564, 12, 21, 22, }, // Cuba
    {  566,  559,       0,  287796,       0,  0, 13,  0, }, // Davis
    {  566,  575,       0,  287809,       0,  0, 24,  0, }, // DumontDUrville
    {  566,  612,       0,  704586,       0,  0, 22,  0, }, // East_Timor
    {  566,  623,  704608,  702870,  702904, 25, 34, 35, }, // Easter
    {  566,  630,       0,  287936,       0,  0, 15,  0, }, // Ecuador
    {  566,  638,  363360,  701437,  701468, 22, 31, 32, }, // Europe_Central
    {  566,  653,  363440,  701500,  701532, 23, 32, 33, }, // Europe_Eastern
    {  566,  668,       0,  704633,       0,  0, 35,  0, }, // Europe_Further_Eastern
    {  566,  691,  363552,  701565,  701599, 25, 34, 35, }, // Europe_Western
    {  566,  706,  704668,  704694,  704729, 26, 35, 36, }, // Falkland
    {  566,  715,  704765,  704777,  704798, 12, 21, 22, }, // Fiji
    {  566,  720,       0,  704820,       0,  0, 27,  0, }, // French_Guiana
    {  566,  734,       0,  704847,       0,  0, 52,  0, }, // French_Southern
    {  566,  757,       0,  704899,       0,  0, 17,  0, }, // Galapagos
    {  566,  767,       0,  288473,       0,  0, 15,  0, }, // Gambier
    {  566,  775,  363861,  704916,  704940, 15, 24, 25, }, // Georgia
    {  566,  783,       0,  704965,       0,  0, 25,  0, }, // Gilbert_Islands
    {  566,  799,       0,  701685,       0,  0, 31,  0, }, // GMT
    {  566,  841,  363969,  704990,  705027, 28, 37, 38, }, // Greenland_Eastern
    {  566,  859,  364067,  705065,  705104, 30, 39, 40, }, // Greenland_Western
    {  566,  877,       0,   55839,       0,  0, 21,  0, }, // Guam
    {  566,  882,       0,  705144,       0,  0, 23,  0, }, // Gulf
    {  566,  887,       0,  364194,       0,  0, 14,  0, }, // Guyana
    {  566,  894,  701716,  701740,  701773, 24, 33, 34, }, // Hawaii_Aleutian
    {  566,  910,  288934,  288951,  705167, 17, 26, 27, }, // Hong_Kong
    {  566,  920,  289003,  289015,  705194, 12, 21, 22, }, // Hovd
    {  566,  925,       0,  705216,       0,  0, 13,  0, }, // India
    {  566,  931,       0,  705229,       0,  0, 22,  0, }, // Indian_Ocean
    {  566,  944,       0,  289091,       0,  0, 17,  0, }, // Indochina
    {  566,  954,       0,  705251,       0,  0, 25,  0, }, // Indonesia_Central
    {  566,  972,       0,  705276,       0,  0, 26,  0, }, // Indonesia_Eastern
    {  566,  990,       0,  705302,       0,  0, 28,  0, }, // Indonesia_Western
    {  566, 1008,  289187,  289199,  705330, 12, 21, 22, }, // Iran
    {  566, 1019,  289241,  289256,  705352, 15, 24, 25, }, // Irkutsk
    {  566, 1027,  289304,  289318,  705377, 14, 23, 24, }, // Israel
    {  566, 1034,  705401,  705414,  705436, 13, 22, 23, }, // Japan
    {  566, 1040,  705459,  705476,  705502, 17, 26, 27, }, // Kamchatka
    {  566, 1058,       0,  705529,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  566, 1077,       0,  705556,       0,  0, 29,  0, }, // Kazakhstan_Western
    {  566, 1106,  289600,  289613,  705585, 13, 22, 23, }, // Korea
    {  566, 1112,       0,  289657,       0,  0, 14,  0, }, // Kosrae
    {  566, 1119,  364786,  705608,  705636, 19, 28, 29, }, // Krasnoyarsk
    {  566, 1151,       0,  705665,       0,  0, 18,  0, }, // Kyrgystan
    {  566, 1161,       0,  705683,       0,  0, 17,  0, }, // Lanka
    {  566, 1175,       0,  705700,       0,  0, 34,  0, }, // Line_Islands
    {  566, 1188,  289788,  289805,  705734, 17, 26, 27, }, // Lord_Howe
    {  566, 1198,  705761,  705774,  705796, 13, 22, 23, }, // Macau
    {  566, 1204,  705819,  705834,  705858, 15, 24, 25, }, // Magadan
    {  566, 1219,       0,  705883,       0,  0, 15,  0, }, // Malaysia
    {  566, 1228,       0,  705898,       0,  0, 16,  0, }, // Maldives
    {  566, 1237,       0,  705914,       0,  0, 27,  0, }, // Marquesas
    {  566, 1247,       0,  705941,       0,  0, 26,  0, }, // Marshall_Islands
    {  566, 1264,  289994,  290010,  705967, 16, 25, 26, }, // Mauritius
    {  566, 1274,       0,  290060,       0,  0, 14,  0, }, // Mawson
    {  566, 1281,  705993,  706020,  706056, 27, 36, 37, }, // Mexico_Pacific
    {  566, 1296,  706093,  706111,  706138, 18, 27, 28, }, // Mongolia
    {  566, 1305,  706166,  706179,  706201, 13, 22, 23, }, // Moscow
    {  566, 1312,       0,  706224,       0,  0, 26,  0, }, // Myanmar
    {  566, 1320,       0,  290320,       0,  0, 13,  0, }, // Nauru
    {  566, 1326,       0,  290333,       0,  0, 13,  0, }, // Nepal
    {  566, 1332,  706250,  706273,  706305, 23, 32, 33, }, // New_Caledonia
    {  566, 1346,  706338,  706359,  706389, 21, 30, 31, }, // New_Zealand
    {  566, 1358,  365497,  706420,  706446, 17, 26, 27, }, // Newfoundland
    {  566, 1371,       0,  290583,       0,  0, 12,  0, }, // Niue
    {  566, 1376,  706473,  706496,  706528, 23, 32, 33, }, // Norfolk
    {  566, 1384,  290673,  290700,  706561, 27, 36, 37, }, // Noronha
    {  566, 1392,       0,  706598,       0,  0, 36,  0, }, // North_Mariana
    {  566, 1406,  290772,  290791,  706634, 19, 28, 29, }, // Novosibirsk
    {  566, 1418,  290847,  290859,  706663, 12, 21, 22, }, // Omsk
    {  566, 1428,  706685,  706701,  706726, 16, 25, 26, }, // Pakistan
    {  566, 1437,       0,  706752,       0,  0, 14,  0, }, // Palau
    {  566, 1443,       0,  706766,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  566, 1460,  365896,  706792,  706817, 16, 25, 26, }, // Paraguay
    {  566, 1469,  703069,  703081,  703102, 12, 21, 22, }, // Peru
    {  566, 1474,  291128,  291145,  706843, 17, 26, 27, }, // Philippines
    {  566, 1486,       0,  706870,       0,  0, 23,  0, }, // Phoenix_Islands
    {  566, 1502,  706893,  706921,  706958, 28, 37, 38, }, // Pierre_Miquelon
    {  566, 1518,       0,  291331,       0,  0, 16,  0, }, // Pitcairn
    {  566, 1527,       0,  291347,       0,  0, 15,  0, }, // Ponape
    {  566, 1534,       0,   58535,       0,  0, 17,  0, }, // Pyongyang
    {  566, 1544,   58552,   58569,  706996, 17, 26, 27, }, // Qyzylorda
    {  566, 1554,       0,  291362,       0,  0, 15,  0, }, // Reunion
    {  566, 1562,       0,  291377,       0,  0, 15,  0, }, // Rothera
    {  566, 1570,  707023,  707038,  707062, 15, 24, 25, }, // Sakhalin
    {  566, 1579,  707087,  707101,  707124, 14, 23, 24, }, // Samara
    {  566, 1596,  291527,  291540,  707148, 13, 22, 23, }, // Samoa
    {  566, 1602,       0,  707171,       0,  0, 18,  0, }, // Seychelles
    {  566, 1624,       0,  291603,       0,  0, 16,  0, }, // Singapore
    {  566, 1634,       0,  707189,       0,  0, 25,  0, }, // Solomon
    {  566, 1642,       0,  707214,       0,  0, 23,  0, }, // South_Georgia
    {  566, 1656,       0,  291663,       0,  0, 15,  0, }, // Suriname
    {  566, 1676,       0,  291679,       0,  0, 13,  0, }, // Syowa
    {  566, 1682,       0,  291692,       0,  0, 14,  0, }, // Tahiti
    {  566, 1689,  707237,  707251,  707274, 14, 23, 24, }, // Taipei
    {  566, 1696,       0,  707298,       0,  0, 18,  0, }, // Tajikistan
    {  566, 1724,       0,  291785,       0,  0, 15,  0, }, // Tokelau
    {  566, 1732,  291800,  291813,  707316, 13, 22, 23, }, // Tonga
    {  566, 1738,       0,  291857,       0,  0, 13,  0, }, // Truk
    {  566, 1750,  291870,  291890,  707339, 20, 29, 30, }, // Turkmenistan
    {  566, 1763,       0,  291948,       0,  0, 14,  0, }, // Tuvalu
    {  566, 1777,  366638,  707369,  707393, 15, 24, 25, }, // Uruguay
    {  566, 1792,  292025,  292043,  707418, 18, 27, 28, }, // Uzbekistan
    {  566, 1803,  292097,  292112,  707446, 15, 24, 25, }, // Vanuatu
    {  566, 1811,       0,  292160,       0,  0, 17,  0, }, // Venezuela
    {  566, 1821,  292177,  292196,  707471, 19, 28, 29, }, // Vladivostok
    {  566, 1833,  292252,  292270,  707500, 18, 27, 28, }, // Volgograd
    {  566, 1843,       0,  292324,       0,  0, 14,  0, }, // Vostok
    {  566, 1850,       0,  707528,       0,  0, 20,  0, }, // Wake
    {  566, 1855,       0,  707548,       0,  0, 23,  0, }, // Wallis
    {  566, 1862,  366935,  707571,  707595, 15, 24, 25, }, // Yakutsk
    {  566, 1870,  292441,  292462,  707620, 21, 30, 31, }, // Yekaterinburg
    {  566, 1892,       0,  707651,       0,  0, 13,  0, }, // Yukon
    {  567,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Uruguay
    {  567,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  567,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  567,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  567,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  567,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  567,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  567,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  567,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  567, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  567, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  567, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  567, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  567, 1777,  366638,  707369,  707393, 15, 24, 25, }, // Uruguay
    {  567, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  568,  174,  701877,  701895,  701922, 18, 27, 28, }, // America_Mountain Spanish/Latin/Venezuela
    {  568,  528,       0,  702078,       0,  0, 19,  0, }, // Cocos
    {  568,  543,  702097,  702119,  702150, 22, 31, 38, }, // Cook
    {  568,  623,  702188,  702213,  702244, 25, 31, 35, }, // Easter
    {  568,  653,  702279,  702302,  702334, 23, 32, 33, }, // Europe_Eastern
    {  568,  668,       0,  702367,       0,  0, 33,  0, }, // Europe_Further_Eastern
    {  568,  691,  702400,  702424,  702457, 24, 33, 34, }, // Europe_Western
    {  568,  706,  702491,  702517,  702552, 26, 35, 36, }, // Falkland
    {  568,  783,       0,  702588,       0,  0, 21,  0, }, // Gilbert_Islands
    {  568, 1247,       0,  702609,       0,  0, 22,  0, }, // Marshall_Islands
    {  568, 1376,  702631,  702654,  702686, 23, 32, 33, }, // Norfolk
    {  568, 1534,       0,  702719,       0,  0, 16,  0, }, // Pyongyang
    {  568, 1634,       0,  702735,       0,  0, 21,  0, }, // Solomon
    {  568, 1811,       0,  292160,       0,  0, 17,  0, }, // Venezuela
    {  568, 1850,       0,  702756,       0,  0, 17,  0, }, // Wake
    {  570,  142,  354764,  354776,  707664, 12, 20, 23, }, // America_Central Sundanese/Latin/Indonesia
    {  570,  158,  707687,  707698,  707717, 11, 19, 22, }, // America_Eastern
    {  570,  174,  707739,  707755,  707779, 16, 24, 27, }, // America_Mountain
    {  570,  191,  354941,  354954,  707806, 13, 21, 24, }, // America_Pacific
    {  570,  286,  355493,  355507,  707830, 14, 22, 25, }, // Atlantic
    {  570,  534,  356594,  356608,  707855, 14, 22, 25, }, // Colombia
    {  570,  638,  707880,  707898,  707924, 18, 26, 29, }, // Europe_Central
    {  570,  653,  707953,  707970,  707995, 17, 25, 28, }, // Europe_Eastern
    {  570,  691,  708023,  708040,  708065, 17, 25, 28, }, // Europe_Western
    {  570,  757,       0,  357359,       0,  0, 15,  0, }, // Galapagos
    {  570,  799,       0,  708093,       0,  0, 15,  0, }, // GMT
    {  571,    5,       0,  708108,       0,  0, 18,  0, }, // Afghanistan Swahili/Latin/Tanzania
    {  571,   17,       0,  708126,       0,  0, 21,  0, }, // Africa_Central
    {  571,   32,       0,  708147,       0,  0, 23,  0, }, // Africa_Eastern
    {  571,   65,       0,  708170,       0,  0, 31,  0, }, // Africa_Southern
    {  571,   81,  708201,  708224,  708258, 23, 34, 41, }, // Africa_Western
    {  571,  107,  708299,  708312,  708336, 13, 24, 23, }, // Alaska
    {  571,  135,  708359,  708372,  708396, 13, 24, 31, }, // Amazon
    {  571,  142,  708427,  708438,  708460, 11, 22, 21, }, // America_Central
    {  571,  158,  708481,  708497,  708524, 16, 27, 26, }, // America_Eastern
    {  571,  174,  708550,  708565,  708591, 15, 26, 25, }, // America_Mountain
    {  571,  191,  708616,  708631,  708657, 15, 26, 25, }, // America_Pacific
    {  571,  207,  708682,  708695,  708719, 13, 24, 25, }, // Anadyr
    {  571,  214,  708744,  708755,  708777, 11, 22, 21, }, // Apia
    {  571,  232,  708798,  708813,  708839, 15, 26, 25, }, // Arabian
    {  571,  240,  708864,  708880,  708907, 16, 27, 34, }, // Argentina
    {  571,  250,  708941,  708971,  709012, 30, 41, 48, }, // Argentina_Western
    {  571,  268,  709060,  709074,  709099, 14, 25, 32, }, // Armenia
    {  571,  286,  709131,  709147,  709174, 16, 27, 26, }, // Atlantic
    {  571,  295,  709200,  709224,  709259, 24, 35, 34, }, // Australia_Central
    {  571,  313,  709293,  709330,  709378, 37, 48, 47, }, // Australia_CentralWestern
    {  571,  338,  709425,  709451,  709492, 26, 41, 40, }, // Australia_Eastern
    {  571,  356,  709532,  709558,  709595, 26, 37, 36, }, // Australia_Western
    {  571,  374,  709631,  709648,  709676, 17, 28, 35, }, // Azerbaijan
    {  571,  385,  709711,  709724,  709748, 13, 24, 31, }, // Azores
    {  571,  397,  709779,  709796,  709824, 17, 28, 35, }, // Bangladesh
    {  571,  415,       0,  709859,       0,  0, 13,  0, }, // Bhutan
    {  571,  422,       0,  709872,       0,  0, 14,  0, }, // Bolivia
    {  571,  437,  709886,  709901,  709927, 15, 26, 33, }, // Brasilia
    {  571,  454,       0,  709960,       0,  0, 24,  0, }, // Brunei
    {  571,  461,  709984,  710001,  710029, 17, 28, 35, }, // Cape_Verde
    {  571,  478,       0,  710064,       0,  0, 26,  0, }, // Chamorro
    {  571,  487,  710090,  710104,  710129, 14, 25, 24, }, // Chatham
    {  571,  495,  710153,  710165,  710188, 12, 23, 30, }, // Chile
    {  571,  501,  710218,  710231,  710255, 13, 24, 23, }, // China
    {  571,  507,  710278,  710295,  710323, 17, 28, 35, }, // Choibalsan
    {  571,  518,       0,  710358,       0,  0, 26,  0, }, // Christmas
    {  571,  528,       0,  710384,       0,  0, 23,  0, }, // Cocos
    {  571,  534,  710407,  710422,  710448, 15, 26, 33, }, // Colombia
    {  571,  543,  710481,  710503,  710536, 22, 33, 41, }, // Cook
    {  571,  548,  710577,  710588,  710610, 11, 22, 21, }, // Cuba
    {  571,  559,       0,  710631,       0,  0, 12,  0, }, // Davis
    {  571,  575,       0,  710643,       0,  0, 23,  0, }, // DumontDUrville
    {  571,  612,       0,  710666,       0,  0, 22,  0, }, // East_Timor
    {  571,  623,  710688,  710712,  710747, 24, 35, 42, }, // Easter
    {  571,  630,       0,  710789,       0,  0, 13,  0, }, // Ecuador
    {  571,  638,  710802,  710822,  710853, 20, 31, 38, }, // Europe_Central
    {  571,  653,  710891,  710917,  710954, 26, 37, 44, }, // Europe_Eastern
    {  571,  668,       0,  710998,       0,  0, 32,  0, }, // Europe_Further_Eastern
    {  571,  691,  711030,  711056,  711093, 26, 37, 44, }, // Europe_Western
    {  571,  706,  711137,  711163,  711200, 26, 37, 44, }, // Falkland
    {  571,  715,  711244,  711255,  711277, 11, 22, 29, }, // Fiji
    {  571,  720,       0,  711306,       0,  0, 25,  0, }, // French_Guiana
    {  571,  734,       0,  711331,       0,  0, 39,  0, }, // French_Southern
    {  571,  757,       0,  711370,       0,  0, 16,  0, }, // Galapagos
    {  571,  767,       0,  711386,       0,  0, 14,  0, }, // Gambier
    {  571,  775,  711400,  711412,  711435, 12, 23, 30, }, // Georgia
    {  571,  783,       0,  711465,       0,  0, 25,  0, }, // Gilbert_Islands
    {  571,  799,       0,  711490,       0,  0, 16,  0, }, // GMT
    {  571,  841,  711506,  711532,  711569, 26, 37, 44, }, // Greenland_Eastern
    {  571,  859,  711613,  711639,  711676, 26, 37, 44, }, // Greenland_Western
    {  571,  882,       0,  711720,       0,  0, 22,  0, }, // Gulf
    {  571,  887,       0,  711742,       0,  0, 13,  0, }, // Guyana
    {  571,  894,  711755,  711777,  711810, 22, 33, 32, }, // Hawaii_Aleutian
    {  571,  910,  711842,  711858,  711885, 16, 27, 34, }, // Hong_Kong
    {  571,  920,  711919,  711930,  711952, 11, 22, 29, }, // Hovd
    {  571,  925,       0,  711981,       0,  0, 23,  0, }, // India
    {  571,  931,       0,  712004,       0,  0, 19,  0, }, // Indian_Ocean
    {  571,  944,       0,  712023,       0,  0, 16,  0, }, // Indochina
    {  571,  954,       0,  712039,       0,  0, 24,  0, }, // Indonesia_Central
    {  571,  972,       0,  712063,       0,  0, 30,  0, }, // Indonesia_Eastern
    {  571,  990,       0,  712093,       0,  0, 30,  0, }, // Indonesia_Western
    {  571, 1008,  712123,  712134,  712156, 11, 22, 21, }, // Iran
    {  571, 1019,  712177,  712191,  712216, 14, 25, 32, }, // Irkutsk
    {  571, 1027,  712248,  712262,  712287, 14, 25, 24, }, // Israel
    {  571, 1034,  712311,  712323,  712347, 12, 24, 22, }, // Japan
    {  571, 1040,  712369,  712400,  712442, 31, 42, 43, }, // Kamchatka
    {  571, 1058,       0,  712485,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  571, 1077,       0,  712512,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  571, 1106,  712539,  712551,  712574, 12, 23, 22, }, // Korea
    {  571, 1112,       0,  712596,       0,  0, 13,  0, }, // Kosrae
    {  571, 1119,  712609,  712627,  712655, 18, 28, 36, }, // Krasnoyarsk
    {  571, 1151,       0,  712691,       0,  0, 16,  0, }, // Kyrgystan
    {  571, 1175,       0,  712707,       0,  0, 22,  0, }, // Line_Islands
    {  571, 1188,  712729,  712745,  712772, 16, 27, 26, }, // Lord_Howe
    {  571, 1204,  712798,  712812,  712837, 14, 25, 32, }, // Magadan
    {  571, 1219,       0,  712869,       0,  0, 15,  0, }, // Malaysia
    {  571, 1228,       0,  712884,       0,  0, 15,  0, }, // Maldives
    {  571, 1237,       0,  712899,       0,  0, 16,  0, }, // Marquesas
    {  571, 1247,       0,  712915,       0,  0, 26,  0, }, // Marshall_Islands
    {  571, 1264,  712941,  712954,  712978, 13, 24, 31, }, // Mauritius
    {  571, 1274,       0,  713009,       0,  0, 13,  0, }, // Mawson
    {  571, 1281,  713022,  713048,  713085, 26, 37, 36, }, // Mexico_Pacific
    {  571, 1296,  713121,  713138,  713166, 17, 28, 35, }, // Mongolia
    {  571, 1305,  713201,  713214,  713238, 13, 24, 31, }, // Moscow
    {  571, 1312,       0,  713269,       0,  0, 14,  0, }, // Myanmar
    {  571, 1320,       0,  713283,       0,  0, 12,  0, }, // Nauru
    {  571, 1326,       0,  713295,       0,  0, 12,  0, }, // Nepal
    {  571, 1332,  713307,  713327,  713358, 20, 31, 38, }, // New_Caledonia
    {  571, 1346,  713396,  713414,  713443, 18, 29, 28, }, // New_Zealand
    {  571, 1358,  713471,  713490,  713520, 19, 30, 29, }, // Newfoundland
    {  571, 1371,       0,  713549,       0,  0, 11,  0, }, // Niue
    {  571, 1376,  713560,  713585,  713621, 25, 36, 43, }, // Norfolk
    {  571, 1384,  713664,  713690,  713727, 26, 37, 44, }, // Noronha
    {  571, 1406,  713771,  713789,  713818, 18, 29, 36, }, // Novosibirsk
    {  571, 1418,  713854,  713865,  713887, 11, 22, 29, }, // Omsk
    {  571, 1428,  713916,  713931,  713957, 15, 26, 33, }, // Pakistan
    {  571, 1437,       0,  713990,       0,  0, 12,  0, }, // Palau
    {  571, 1443,       0,  714002,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  571, 1460,  714025,  714040,  714066, 15, 26, 33, }, // Paraguay
    {  571, 1469,  714099,  714110,  714132, 11, 22, 29, }, // Peru
    {  571, 1474,  714161,  714177,  714204, 16, 27, 34, }, // Philippines
    {  571, 1486,       0,  714238,       0,  0, 25,  0, }, // Phoenix_Islands
    {  571, 1502,  714263,  714294,  714336, 31, 42, 41, }, // Pierre_Miquelon
    {  571, 1518,       0,  714377,       0,  0, 15,  0, }, // Pitcairn
    {  571, 1527,       0,  714392,       0,  0, 13,  0, }, // Ponape
    {  571, 1534,       0,  714405,       0,  0, 16,  0, }, // Pyongyang
    {  571, 1554,       0,  714421,       0,  0, 14,  0, }, // Reunion
    {  571, 1562,       0,  714435,       0,  0, 14,  0, }, // Rothera
    {  571, 1570,  714449,  714464,  714490, 15, 26, 33, }, // Sakhalin
    {  571, 1579,  714523,  714536,  714560, 13, 24, 25, }, // Samara
    {  571, 1596,  714585,  714597,  714620, 12, 23, 30, }, // Samoa
    {  571, 1602,       0,  714650,       0,  0, 18,  0, }, // Seychelles
    {  571, 1624,       0,  714668,       0,  0, 27,  0, }, // Singapore
    {  571, 1634,       0,  714695,       0,  0, 25,  0, }, // Solomon
    {  571, 1642,       0,  714720,       0,  0, 21,  0, }, // South_Georgia
    {  571, 1656,       0,  714741,       0,  0, 15,  0, }, // Suriname
    {  571, 1676,       0,  714756,       0,  0, 12,  0, }, // Syowa
    {  571, 1682,       0,  714768,       0,  0, 13,  0, }, // Tahiti
    {  571, 1689,  714781,  714794,  714818, 13, 24, 23, }, // Taipei
    {  571, 1696,       0,  714841,       0,  0, 17,  0, }, // Tajikistan
    {  571, 1724,       0,  714858,       0,  0, 14,  0, }, // Tokelau
    {  571, 1732,  714872,  714884,  714907, 12, 23, 30, }, // Tonga
    {  571, 1738,       0,  714937,       0,  0, 12,  0, }, // Truk
    {  571, 1750,  714949,  714968,  714998, 19, 30, 37, }, // Turkmenistan
    {  571, 1763,       0,  715035,       0,  0, 13,  0, }, // Tuvalu
    {  571, 1777,  715048,  715062,  715087, 14, 25, 32, }, // Uruguay
    {  571, 1792,  715119,  715136,  715164, 17, 28, 35, }, // Uzbekistan
    {  571, 1803,  715199,  715213,  715238, 14, 25, 32, }, // Vanuatu
    {  571, 1811,       0,  715270,       0,  0, 16,  0, }, // Venezuela
    {  571, 1821,  715286,  715304,  715333, 18, 29, 36, }, // Vladivostok
    {  571, 1833,  715369,  715385,  715412, 16, 27, 34, }, // Volgograd
    {  571, 1843,       0,  715446,       0,  0, 13,  0, }, // Vostok
    {  571, 1850,       0,  715459,       0,  0, 22,  0, }, // Wake
    {  571, 1855,       0,  715481,       0,  0, 23,  0, }, // Wallis
    {  571, 1862,  715504,  715518,  715543, 14, 25, 32, }, // Yakutsk
    {  571, 1870,  715575,  715595,  715626, 20, 31, 38, }, // Yekaterinburg
    {  571, 1892,       0,  715664,       0,  0, 12,  0, }, // Yukon
    {  573,    5,       0,  715676,       0,  0, 19,  0, }, // Afghanistan Swahili/Latin/Kenya
    {  573,   81,  708201,  708224,  715695, 23, 34, 41, }, // Africa_Western
    {  573,  135,  708359,  708372,  715736, 13, 24, 31, }, // Amazon
    {  573,  240,  715767,  715782,  715808, 15, 26, 30, }, // Argentina
    {  573,  250,  715838,  715867,  715907, 29, 40, 47, }, // Argentina_Western
    {  573,  268,  709060,  709074,  715954, 14, 25, 32, }, // Armenia
    {  573,  313,  715986,  716025,  716074, 39, 49, 48, }, // Australia_CentralWestern
    {  573,  374,  716122,  716138,  716165, 16, 27, 34, }, // Azerbaijan
    {  573,  385,  709711,  709724,  716199, 13, 24, 31, }, // Azores
    {  573,  397,  716230,  716248,  716277, 18, 29, 36, }, // Bangladesh
    {  573,  415,       0,  716313,       0,  0, 13,  0, }, // Bhutan
    {  573,  437,  716326,  716340,  716365, 14, 25, 32, }, // Brasilia
    {  573,  461,  716397,  716412,  716438, 15, 26, 33, }, // Cape_Verde
    {  573,  507,  710278,  710295,  716471, 17, 28, 35, }, // Choibalsan
    {  573,  534,  710407,  710422,  716506, 15, 26, 33, }, // Colombia
    {  573,  543,  710481,  710503,  716539, 22, 33, 45, }, // Cook
    {  573,  548,  710577,  716584,  710610, 11, 22, 21, }, // Cuba
    {  573,  623,  710688,  710712,  716606, 24, 35, 42, }, // Easter
    {  573,  638,  710802,  710822,  716648, 20, 31, 38, }, // Europe_Central
    {  573,  653,  710891,  710917,  716686, 26, 37, 44, }, // Europe_Eastern
    {  573,  691,  711030,  711056,  716730, 26, 37, 44, }, // Europe_Western
    {  573,  720,       0,  711306,       0,  0, 13,  0, }, // French_Guiana
    {  573,  775,  716774,  716787,  716811, 13, 24, 31, }, // Georgia
    {  573,  841,  711506,  711532,  716842, 26, 37, 44, }, // Greenland_Eastern
    {  573,  882,       0,  716886,       0,  0, 23,  0, }, // Gulf
    {  573,  910,  711842,  711858,  716909, 16, 27, 34, }, // Hong_Kong
    {  573,  920,  711919,  711930,  716943, 11, 22, 29, }, // Hovd
    {  573, 1008,  716972,  716984,  717007, 12, 23, 22, }, // Iran
    {  573, 1019,  712177,  712191,  717029, 14, 25, 32, }, // Irkutsk
    {  573, 1034,  717061,  712323,  717074, 13, 24, 23, }, // Japan
    {  573, 1058,       0,  717097,       0,  0, 28,  0, }, // Kazakhstan_Eastern
    {  573, 1077,       0,  717125,       0,  0, 28,  0, }, // Kazakhstan_Western
    {  573, 1119,  712609,  712627,  717153, 18, 28, 36, }, // Krasnoyarsk
    {  573, 1204,  712798,  712812,  717189, 14, 25, 32, }, // Magadan
    {  573, 1219,       0,  717221,       0,  0, 14,  0, }, // Malaysia
    {  573, 1228,       0,  717235,       0,  0, 14,  0, }, // Maldives
    {  573, 1264,  712941,  712954,  717249, 13, 24, 31, }, // Mauritius
    {  573, 1296,  717280,  717298,  717327, 18, 29, 36, }, // Mongolia
    {  573, 1305,  713201,  713214,  717363, 13, 24, 31, }, // Moscow
    {  573, 1312,       0,  713269,       0,  0, 13,  0, }, // Myanmar
    {  573, 1326,       0,  717394,       0,  0, 13,  0, }, // Nepal
    {  573, 1332,  717407,  717428,  717460, 21, 32, 39, }, // New_Caledonia
    {  573, 1346,  717499,  717516,  717544, 17, 28, 27, }, // New_Zealand
    {  573, 1376,  713560,  713585,  717571, 25, 36, 43, }, // Norfolk
    {  573, 1406,  713771,  713789,  717614, 18, 29, 36, }, // Novosibirsk
    {  573, 1418,  713854,  713865,  717650, 11, 22, 29, }, // Omsk
    {  573, 1428,  717679,  717695,  717722, 16, 27, 34, }, // Pakistan
    {  573, 1443,       0,  714002,       0,  0, 12,  0, }, // Papua_New_Guinea
    {  573, 1460,  714025,  714040,  717756, 15, 26, 33, }, // Paraguay
    {  573, 1469,  714099,  714110,  717789, 11, 22, 29, }, // Peru
    {  573, 1474,  714161,  714177,  717818, 16, 27, 34, }, // Philippines
    {  573, 1486,       0,  717852,       0,  0, 25,  0, }, // Phoenix_Islands
    {  573, 1570,  714449,  714464,  717877, 15, 26, 33, }, // Sakhalin
    {  573, 1596,  714585,  714597,  717910, 12, 23, 22, }, // Samoa
    {  573, 1624,       0,  717932,       0,  0, 26,  0, }, // Singapore
    {  573, 1642,       0,  717958,       0,  0, 19,  0, }, // South_Georgia
    {  573, 1696,       0,  717977,       0,  0, 17,  0, }, // Tajikistan
    {  573, 1732,  714872,  714884,  717994, 12, 23, 30, }, // Tonga
    {  573, 1750,  718024,  718044,  718075, 20, 31, 38, }, // Turkmenistan
    {  573, 1777,  715048,  715062,  718113, 14, 25, 32, }, // Uruguay
    {  573, 1792,  718145,  718163,  718192, 18, 29, 36, }, // Uzbekistan
    {  573, 1803,  715199,  715213,  718228, 14, 25, 32, }, // Vanuatu
    {  573, 1821,  715286,  715304,  718260, 18, 29, 36, }, // Vladivostok
    {  573, 1833,  715369,  715385,  718296, 16, 27, 34, }, // Volgograd
    {  573, 1862,  715504,  715518,  718330, 14, 25, 32, }, // Yakutsk
    {  573, 1870,  715575,  715595,  718362, 20, 31, 38, }, // Yekaterinburg
    {  577,    0,  718400,  718419,  718444, 19, 25, 25, }, // Acre Swedish/Latin/Sweden
    {  577,    5,       0,  542317,       0,  0, 12,  0, }, // Afghanistan
    {  577,   17,       0,  718469,       0,  0, 20,  0, }, // Africa_Central
    {  577,   32,       0,  718489,       0,  0, 16,  0, }, // Africa_Eastern
    {  577,   65,       0,  718505,       0,  0, 16,  0, }, // Africa_Southern
    {  577,   81,  718521,  718538,  718561, 17, 23, 23, }, // Africa_Western
    {  577,  107,  718584,  718593,  718610,  9, 17, 17, }, // Alaska
    {  577,  128,  718627,  718636,  718651,  9, 15, 15, }, // Almaty
    {  577,  135,  542585,  718666,  718685, 11, 19, 19, }, // Amazon
    {  577,  142,  718704,  718729,  718760, 25, 31, 31, }, // America_Central
    {  577,  158,  718791,  718812,  718839, 21, 27, 27, }, // America_Eastern
    {  577,  174,  718866,  718884,  718910, 18, 26, 26, }, // America_Mountain
    {  577,  191,  718936,  718958,  718986, 22, 28, 28, }, // America_Pacific
    {  577,  207,  719014,  719023,  719038,  9, 15, 15, }, // Anadyr
    {  577,  214,  719053,  719060,  719075,  7, 15, 15, }, // Apia
    {  577,  219,  719090,  719098,  719112,  8, 14, 14, }, // Aqtau
    {  577,  225,  719126,  719135,  719150,  9, 15, 15, }, // Aqtobe
    {  577,  232,  719165,  719181,  719203, 16, 22, 22, }, // Arabian
    {  577,  240,  719225,  719242,  719265, 17, 23, 23, }, // Argentina
    {  577,  250,  719288,  719306,  719330, 18, 24, 24, }, // Argentina_Western
    {  577,  268,  719354,  719366,  719384, 12, 18, 18, }, // Armenia
    {  577,  286,  719402,  719426,  719456, 24, 30, 30, }, // Atlantic
    {  577,  295,  719486,  719507,  719534, 21, 27, 27, }, // Australia_Central
    {  577,  313,  719561,  719586,  719617, 25, 31, 31, }, // Australia_CentralWestern
    {  577,  338,  719648,  719665,  719688, 17, 23, 23, }, // Australia_Eastern
    {  577,  356,  719711,  719729,  719753, 18, 24, 24, }, // Australia_Western
    {  577,  374,  719777,  719795,  719819, 18, 24, 24, }, // Azerbaijan
    {  577,  385,  719843,  719854,  719871, 11, 17, 17, }, // Azores
    {  577,  397,  543991,  544008,  550161, 17, 23, 23, }, // Bangladesh
    {  577,  415,       0,  544054,       0,  0, 12,  0, }, // Bhutan
    {  577,  422,       0,  544066,       0,  0, 14,  0, }, // Bolivia
    {  577,  437,  544137,  719888,  719907, 11, 19, 19, }, // Brasilia
    {  577,  454,       0,  719926,       0,  0,  9,  0, }, // Brunei
    {  577,  461,  719935,  719947,  719967, 12, 20, 20, }, // Cape_Verde
    {  577,  472,       0,  719987,       0,  0,  8,  0, }, // Casey
    {  577,  478,       0,  544254,       0,  0, 11,  0, }, // Chamorro
    {  577,  487,  719995,  720005,  720023, 10, 18, 18, }, // Chatham
    {  577,  495,  544324,  544336,  550248, 12, 18, 18, }, // Chile
    {  577,  501,  544372,  544384,  550266, 12, 18, 18, }, // China
    {  577,  507,  720041,  720054,  720075, 13, 21, 21, }, // Choibalsan
    {  577,  518,       0,  720096,       0,  0, 10,  0, }, // Christmas
    {  577,  528,       0,  720106,       0,  0, 17,  0, }, // Cocos
    {  577,  534,  544542,  544557,  720123, 15, 21, 21, }, // Colombia
    {  577,  543,  720144,  720158,  720178, 14, 20, 20, }, // Cook
    {  577,  548,  550457,  550468,  550485, 11, 17, 17, }, // Cuba
    {  577,  559,       0,  544730,       0,  0,  8,  0, }, // Davis
    {  577,  575,       0,  720198,       0,  0, 20,  0, }, // DumontDUrville
    {  577,  612,       0,  720218,       0,  0, 15,  0, }, // East_Timor
    {  577,  623,  720233,  720241,  720258,  8, 17, 17, }, // Easter
    {  577,  630,       0,  544846,       0,  0, 16,  0, }, // Ecuador
    {  577,  638,  720275,  720295,  720321, 20, 26, 26, }, // Europe_Central
    {  577,  653,  720347,  720363,  720385, 16, 22, 22, }, // Europe_Eastern
    {  577,  668,       0,  720407,       0,  0, 14,  0, }, // Europe_Further_Eastern
    {  577,  691,  720421,  720438,  720461, 17, 23, 23, }, // Europe_Western
    {  577,  706,  720484,  720503,  720528, 19, 25, 25, }, // Falkland
    {  577,  715,  720553,  720560,  720575,  7, 15, 15, }, // Fiji
    {  577,  720,       0,  720590,       0,  0, 17,  0, }, // French_Guiana
    {  577,  734,       0,  720607,       0,  0, 29,  0, }, // French_Southern
    {  577,  757,       0,  720636,       0,  0, 12,  0, }, // Galapagos
    {  577,  767,       0,  720648,       0,  0, 10,  0, }, // Gambier
    {  577,  775,  545321,  545333,  550927, 12, 18, 18, }, // Georgia
    {  577,  783,       0,  720658,       0,  0, 11,  0, }, // Gilbert_Islands
    {  577,  799,       0,  720669,       0,  0, 12,  0, }, // GMT
    {  577,  841,  720681,  720698,  720721, 17, 23, 23, }, // Greenland_Eastern
    {  577,  859,  720744,  720762,  720786, 18, 24, 24, }, // Greenland_Western
    {  577,  877,       0,  720810,       0,  0,  7,  0, }, // Guam
    {  577,  882,       0,  720817,       0,  0, 17,  0, }, // Gulf
    {  577,  887,       0,  545230,       0,  0,  9,  0, }, // Guyana
    {  577,  894,  720834,  720845,  720864, 11, 19, 19, }, // Hawaii_Aleutian
    {  577,  910,  545737,  720883,  720902, 11, 19, 19, }, // Hong_Kong
    {  577,  920,  720921,  720929,  720945,  8, 16, 16, }, // Hovd
    {  577,  925,       0,  545801,       0,  0, 10,  0, }, // India
    {  577,  931,       0,  720961,       0,  0, 33,  0, }, // Indian_Ocean
    {  577,  944,       0,  545834,       0,  0, 16,  0, }, // Indochina
    {  577,  954,       0,  720994,       0,  0, 21,  0, }, // Indonesia_Central
    {  577,  972,       0,  721015,       0,  0, 17,  0, }, // Indonesia_Eastern
    {  577,  990,       0,  721032,       0,  0, 18,  0, }, // Indonesia_Western
    {  577, 1008,  545906,  545916,  551201, 10, 16, 16, }, // Iran
    {  577, 1019,  721050,  721060,  721078, 10, 18, 18, }, // Irkutsk
    {  577, 1027,  721096,  721109,  721128, 13, 19, 19, }, // Israel
    {  577, 1034,  546058,  546069,  551256, 11, 17, 17, }, // Japan
    {  577, 1040,  721147,  721159,  721177, 12, 18, 18, }, // Kamchatka
    {  577, 1058,       0,  721195,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  577, 1077,       0,  721213,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  577, 1106,  546274,  546286,  551293, 12, 18, 18, }, // Korea
    {  577, 1112,       0,  546335,       0,  0,  9,  0, }, // Kosrae
    {  577, 1119,  551325,  721232,  721254, 14, 22, 22, }, // Krasnoyarsk
    {  577, 1151,       0,  721276,       0,  0, 13,  0, }, // Kyrgystan
    {  577, 1161,       0,  721289,       0,  0, 12,  0, }, // Lanka
    {  577, 1175,       0,  721301,       0,  0, 14,  0, }, // Line_Islands
    {  577, 1188,  721315,  721327,  721347, 12, 20, 20, }, // Lord_Howe
    {  577, 1198,  721367,  721375,  721389,  8, 14, 14, }, // Macau
    {  577, 1204,  551399,  721403,  721421, 10, 18, 18, }, // Magadan
    {  577, 1219,       0,  546645,       0,  0, 13,  0, }, // Malaysia
    {  577, 1228,       0,  721439,       0,  0, 13,  0, }, // Maldives
    {  577, 1237,       0,  721452,       0,  0, 12,  0, }, // Marquesas
    {  577, 1247,       0,  721464,       0,  0, 18,  0, }, // Marshall_Islands
    {  577, 1264,  721482,  721494,  721514, 12, 20, 20, }, // Mauritius
    {  577, 1274,       0,  546779,       0,  0,  9,  0, }, // Mawson
    {  577, 1281,  721534,  721557,  721591, 23, 34, 34, }, // Mexico_Pacific
    {  577, 1296,  721625,  721639,  721661, 14, 22, 22, }, // Mongolia
    {  577, 1305,  721683,  721692,  721709,  9, 17, 17, }, // Moscow
    {  577, 1312,       0,  721726,       0,  0, 13,  0, }, // Myanmar
    {  577, 1320,       0,  721739,       0,  0,  8,  0, }, // Nauru
    {  577, 1326,       0,  721747,       0,  0, 14,  0, }, // Nepal
    {  577, 1332,  721761,  721778,  721803, 17, 25, 25, }, // New_Caledonia
    {  577, 1346,  721828,  721843,  721864, 15, 21, 21, }, // New_Zealand
    {  577, 1358,  547263,  721885,  721908, 15, 23, 23, }, // Newfoundland
    {  577, 1371,       0,  547288,       0,  0,  7,  0, }, // Niue
    {  577, 1376,  721931,  721945,  721965, 14, 20, 20, }, // Norfolk
    {  577, 1384,  721985,  722007,  722037, 22, 30, 30, }, // Noronha
    {  577, 1392,       0,  722067,       0,  0, 19,  0, }, // North_Mariana
    {  577, 1406,  547542,  722086,  722108, 14, 22, 22, }, // Novosibirsk
    {  577, 1418,  722130,  722137,  722152,  7, 15, 15, }, // Omsk
    {  577, 1428,  547606,  547620,  551835, 14, 20, 20, }, // Pakistan
    {  577, 1437,       0,  722167,       0,  0,  8,  0, }, // Palau
    {  577, 1443,       0,  722175,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  577, 1460,  547684,  547700,  551855, 16, 22, 22, }, // Paraguay
    {  577, 1469,  547744,  547756,  551877, 12, 18, 18, }, // Peru
    {  577, 1474,  547792,  547806,  551895, 14, 20, 20, }, // Philippines
    {  577, 1486,       0,  722196,       0,  0, 12,  0, }, // Phoenix_Islands
    {  577, 1502,  722208,  722236,  722271, 28, 35, 35, }, // Pierre_Miquelon
    {  577, 1518,       0,  547997,       0,  0, 11,  0, }, // Pitcairn
    {  577, 1527,       0,  722306,       0,  0,  9,  0, }, // Ponape
    {  577, 1534,       0,  722315,       0,  0, 12,  0, }, // Pyongyang
    {  577, 1544,  722327,  722339,  722357, 12, 18, 18, }, // Qyzylorda
    {  577, 1554,       0,  548102,       0,  0, 10,  0, }, // Reunion
    {  577, 1562,       0,  548122,       0,  0, 10,  0, }, // Rothera
    {  577, 1570,  722375,  722386,  722405, 11, 19, 19, }, // Sakhalin
    {  577, 1579,  722424,  722433,  722448,  9, 15, 15, }, // Samara
    {  577, 1596,  548266,  548278,  552000, 12, 18, 18, }, // Samoa
    {  577, 1602,       0,  722463,       0,  0, 15,  0, }, // Seychelles
    {  577, 1624,       0,  722478,       0,  0, 12,  0, }, // Singapore
    {  577, 1634,       0,  722490,       0,  0, 17,  0, }, // Solomon
    {  577, 1642,       0,  722507,       0,  0, 15,  0, }, // South_Georgia
    {  577, 1656,       0,  722522,       0,  0, 10,  0, }, // Suriname
    {  577, 1676,       0,  722532,       0,  0,  8,  0, }, // Syowa
    {  577, 1682,       0,  722540,       0,  0,  9,  0, }, // Tahiti
    {  577, 1689,  722549,  722558,  722575,  9, 17, 17, }, // Taipei
    {  577, 1696,       0,  722592,       0,  0, 15,  0, }, // Tajikistan
    {  577, 1724,       0,  722607,       0,  0, 10,  0, }, // Tokelau
    {  577, 1732,  722617,  722625,  722641,  8, 16, 16, }, // Tonga
    {  577, 1738,       0,  722657,       0,  0,  8,  0, }, // Truk
    {  577, 1750,  548587,  548600,  552079, 13, 19, 19, }, // Turkmenistan
    {  577, 1763,       0,  722665,       0,  0,  9,  0, }, // Tuvalu
    {  577, 1777,  548649,  548664,  552098, 15, 21, 21, }, // Uruguay
    {  577, 1792,  722674,  722686,  722704, 12, 18, 18, }, // Uzbekistan
    {  577, 1803,  722722,  722732,  722750, 10, 18, 18, }, // Vanuatu
    {  577, 1811,       0,  548805,       0,  0, 16,  0, }, // Venezuela
    {  577, 1821,  548884,  722768,  722790, 14, 22, 22, }, // Vladivostok
    {  577, 1833,  548954,  722812,  722832, 12, 20, 20, }, // Volgograd
    {  577, 1843,       0,  548976,       0,  0,  9,  0, }, // Vostok
    {  577, 1850,       0,  722852,       0,  0, 14,  0, }, // Wake
    {  577, 1855,       0,  722866,       0,  0, 28,  0, }, // Wallis
    {  577, 1862,  549096,  722894,  722912, 10, 18, 18, }, // Yakutsk
    {  577, 1870,  549170,  722930,  722954, 16, 24, 24, }, // Yekaterinburg
    {  577, 1892,       0,  722978,       0,  0,  8,  0, }, // Yukon
    {  580,    0,  722986,  722995,  723014,  9, 19, 15, }, // Acre Swiss German/Latin/Switzerland
    {  580,    5,       0,  723029,       0,  0, 18,  0, }, // Afghanistan
    {  580,   17,       0,  723047,       0,  0, 24,  0, }, // Africa_Central
    {  580,   32,       0,  723071,       0,  0, 22,  0, }, // Africa_Eastern
    {  580,   65,       0,  723093,       0,  0, 21,  0, }, // Africa_Southern
    {  580,   81,  723114,  723137,  723170, 23, 33, 29, }, // Africa_Western
    {  580,  107,  723199,  723210,  723231, 11, 21, 17, }, // Alaska
    {  580,  128,  723248,  723259,  723280, 11, 21, 17, }, // Almaty
    {  580,  135,  723297,  723310,  723333, 13, 23, 19, }, // Amazon
    {  580,  142,  723352,  723373,  723404, 21, 31, 27, }, // America_Central
    {  580,  638,  723431,  723453,  723485, 22, 32, 28, }, // Europe_Central
    {  580,  653,  723513,  723534,  723565, 21, 31, 27, }, // Europe_Eastern
    {  580,  691,  723592,  723614,  723646, 22, 32, 28, }, // Europe_Western
    {  580, 1305,  723674,  723687,  723710, 13, 23, 19, }, // Moscow
    {  583,    0,  723729,  723742,  723776, 13, 34, 20, }, // Acre Syriac/Syriac/Iraq
    {  583,    5,       0,  723796,       0,  0, 15,  0, }, // Afghanistan
    {  583,   17,       0,  723811,       0,  0, 19,  0, }, // Africa_Central
    {  583,   32,       0,  723830,       0,  0, 17,  0, }, // Africa_Eastern
    {  583,   65,       0,  723847,       0,  0, 27,  0, }, // Africa_Southern
    {  583,   81,  723874,  723891,  723918, 17, 27, 24, }, // Africa_Western
    {  583,  107,  723942,  723954,  723976, 12, 22, 22, }, // Alaska
    {  583,  128,  723998,  724010,  724032, 12, 22, 22, }, // Almaty
    {  583,  135,  724054,  724064,  724084, 10, 20, 20, }, // Amazon
    {  583,  142,  724104,  724131,  724168, 27, 37, 37, }, // America_Central
    {  583,  158,  724205,  724233,  724271, 28, 38, 38, }, // America_Eastern
    {  583,  174,  724309,  724336,  724373, 27, 37, 37, }, // America_Mountain
    {  583,  191,  724410,  724437,  724474, 27, 37, 37, }, // America_Pacific
    {  583,  207,  724511,  724523,  724545, 12, 22, 22, }, // Anadyr
    {  583,  214,  724567,  724577,  724597, 10, 20, 20, }, // Apia
    {  583,  219,  724617,  724627,  724648, 10, 21, 21, }, // Aqtau
    {  583,  225,  724669,  724680,  724701, 11, 21, 21, }, // Aqtobe
    {  583,  232,  724722,  724733,  724754, 11, 21, 21, }, // Arabian
    {  583,  240,  724775,  724789,  724813, 14, 24, 24, }, // Argentina
    {  583,  250,  724837,  724859,  724891, 22, 32, 32, }, // Argentina_Western
    {  583,  268,  724923,  724935,  724957, 12, 22, 22, }, // Armenia
    {  583,  286,  724979,  725008,  725047, 29, 39, 39, }, // Atlantic
    {  583,  295,  725086,  725107,  725138, 21, 31, 31, }, // Australia_Central
    {  583,  313,  725169,  725198,  725237, 29, 39, 39, }, // Australia_CentralWestern
    {  583,  338,  725276,  725298,  725330, 22, 32, 32, }, // Australia_Eastern
    {  583,  356,  725362,  725384,  725416, 22, 32, 32, }, // Australia_Western
    {  583,  374,  725448,  725462,  725486, 14, 24, 24, }, // Azerbaijan
    {  583,  385,  725510,  725520,  725540, 10, 20, 20, }, // Azores
    {  583,  397,  725560,  725574,  725598, 14, 24, 24, }, // Bangladesh
    {  583,  415,       0,  725622,       0,  0, 11,  0, }, // Bhutan
    {  583,  422,       0,  725633,       0,  0, 13,  0, }, // Bolivia
    {  583,  437,  725646,  725660,  725684, 14, 24, 24, }, // Brasilia
    {  583,  454,       0,  725708,       0,  0, 21,  0, }, // Brunei
    {  583,  461,  725729,  725744,  725769, 15, 25, 25, }, // Cape_Verde
    {  583,  472,       0,  725794,       0,  0, 11,  0, }, // Casey
    {  583,  478,       0,  725805,       0,  0, 23,  0, }, // Chamorro
    {  583,  487,  725828,  725839,  725860, 11, 21, 21, }, // Chatham
    {  583,  495,  725881,  725892,  725913, 11, 21, 21, }, // Chile
    {  583,  501,  725934,  725943,  725962,  9, 19, 19, }, // China
    {  583,  507,  725981,  725994,  726017, 13, 23, 23, }, // Choibalsan
    {  583,  518,       0,  726040,       0,  0, 20,  0, }, // Christmas
    {  583,  528,       0,  726060,       0,  0, 19,  0, }, // Cocos
    {  583,  534,  726079,  726093,  726117, 14, 24, 24, }, // Colombia
    {  583,  543,  726141,  726157,  726183, 16, 26, 26, }, // Cook
    {  583,  548,  726209,  726219,  726239, 10, 20, 20, }, // Cuba
    {  583,  559,       0,  726259,       0,  0, 10,  0, }, // Davis
    {  583,  575,       0,  726269,       0,  0, 20,  0, }, // DumontDUrville
    {  583,  612,       0,  726289,       0,  0, 16,  0, }, // East_Timor
    {  583,  623,  726305,  726321,  726347, 16, 26, 26, }, // Easter
    {  583,  630,       0,  726373,       0,  0, 13,  0, }, // Ecuador
    {  583,  638,  726386,  726405,  726434, 19, 29, 29, }, // Europe_Central
    {  583,  653,  726463,  726483,  726513, 20, 30, 30, }, // Europe_Eastern
    {  583,  668,       0,  726543,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  583,  691,  726566,  726586,  726616, 20, 30, 30, }, // Europe_Western
    {  583,  706,  726646,  726666,  726696, 20, 30, 30, }, // Falkland
    {  583,  715,  726726,  726736,  726756, 10, 20, 20, }, // Fiji
    {  583,  720,       0,  726776,       0,  0, 20,  0, }, // French_Guiana
    {  583,  734,       0,  726796,       0,  0, 29,  0, }, // French_Southern
    {  583,  757,       0,  726825,       0,  0, 14,  0, }, // Galapagos
    {  583,  767,       0,  726839,       0,  0, 11,  0, }, // Gambier
    {  583,  775,  726850,  726862,  726884, 12, 22, 22, }, // Georgia
    {  583,  783,       0,  726906,       0,  0, 20,  0, }, // Gilbert_Islands
    {  583,  799,       0,  726926,       0,  0, 11,  0, }, // GMT
    {  583,  841,  726937,  726957,  726987, 20, 30, 30, }, // Greenland_Eastern
    {  583,  859,  727017,  727039,  727071, 22, 32, 32, }, // Greenland_Western
    {  583,  877,       0,  727103,       0,  0, 20,  0, }, // Guam
    {  583,  882,       0,  727123,       0,  0, 21,  0, }, // Gulf
    {  583,  887,       0,  726776,       0,  0, 12,  0, }, // Guyana
    {  583,  894,  727144,  727163,  727192, 19, 29, 29, }, // Hawaii_Aleutian
    {  583,  910,  727221,  727236,  727261, 15, 25, 25, }, // Hong_Kong
    {  583,  920,  727286,  727291,  727311,  5, 20, 20, }, // Hovd
    {  583,  925,       0,  727331,       0,  0, 20,  0, }, // India
    {  583,  931,       0,  727351,       0,  0, 20,  0, }, // Indian_Ocean
    {  583,  944,       0,  727371,       0,  0, 13,  0, }, // Indochina
    {  583,  954,       0,  727384,       0,  0, 20,  0, }, // Indonesia_Central
    {  583,  972,       0,  727404,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  583,  990,       0,  727425,       0,  0, 21,  0, }, // Indonesia_Western
    {  583, 1008,  727446,  727457,  727478, 11, 21, 21, }, // Iran
    {  583, 1019,  727499,  727513,  727537, 14, 24, 24, }, // Irkutsk
    {  583, 1027,  727561,  727574,  727597, 13, 23, 23, }, // Israel
    {  583, 1034,  727620,  727629,  727648,  9, 19, 19, }, // Japan
    {  583, 1040,  727667,  727694,  727731, 27, 37, 37, }, // Kamchatka
    {  583, 1058,       0,  727768,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  583, 1077,       0,  727786,       0,  0, 18,  0, }, // Kazakhstan_Western
    {  583, 1106,  727804,  727816,  727838, 12, 22, 22, }, // Korea
    {  583, 1112,       0,  727860,       0,  0, 11,  0, }, // Kosrae
    {  583, 1119,  727871,  727887,  727913, 16, 26, 26, }, // Krasnoyarsk
    {  583, 1151,       0,  727939,       0,  0, 16,  0, }, // Kyrgystan
    {  583, 1161,       0,  727955,       0,  0, 11,  0, }, // Lanka
    {  583, 1175,       0,  727966,       0,  0, 18,  0, }, // Line_Islands
    {  583, 1188,  727984,  727998,  728022, 14, 24, 24, }, // Lord_Howe
    {  583, 1198,  728046,  728057,  728078, 11, 21, 21, }, // Macau
    {  583, 1204,  728099,  728112,  728134, 13, 22, 22, }, // Magadan
    {  583, 1219,       0,  728156,       0,  0, 12,  0, }, // Malaysia
    {  583, 1228,       0,  728168,       0,  0, 21,  0, }, // Maldives
    {  583, 1237,       0,  728189,       0,  0, 15,  0, }, // Marquesas
    {  583, 1247,       0,  728204,       0,  0, 20,  0, }, // Marshall_Islands
    {  583, 1264,  728224,  728238,  728262, 14, 24, 24, }, // Mauritius
    {  583, 1274,       0,  728286,       0,  0, 12,  0, }, // Mawson
    {  583, 1281,  728298,  728317,  728346, 19, 29, 29, }, // Mexico_Pacific
    {  583, 1296,  728375,  728391,  728417, 16, 26, 26, }, // Mongolia
    {  583, 1305,  728443,  728454,  728475, 11, 21, 21, }, // Moscow
    {  583, 1312,       0,  728496,       0,  0, 13,  0, }, // Myanmar
    {  583, 1320,       0,  728509,       0,  0, 11,  0, }, // Nauru
    {  583, 1326,       0,  728520,       0,  0, 11,  0, }, // Nepal
    {  583, 1332,  728531,  728549,  728577, 18, 28, 28, }, // New_Caledonia
    {  583, 1346,  728605,  728620,  728645, 15, 25, 25, }, // New_Zealand
    {  583, 1358,  728670,  728686,  728712, 16, 26, 26, }, // Newfoundland
    {  583, 1371,       0,  728738,       0,  0, 10,  0, }, // Niue
    {  583, 1376,  728748,  728767,  728796, 19, 29, 29, }, // Norfolk
    {  583, 1384,  728825,  728846,  728877, 21, 31, 31, }, // Noronha
    {  583, 1392,       0,  728908,       0,  0, 28,  0, }, // North_Mariana
    {  583, 1406,  728936,  728952,  728978, 16, 26, 26, }, // Novosibirsk
    {  583, 1418,  729004,  729014,  729034, 10, 20, 20, }, // Omsk
    {  583, 1428,  729054,  729067,  729090, 13, 23, 23, }, // Pakistan
    {  583, 1437,       0,  729113,       0,  0, 10,  0, }, // Palau
    {  583, 1443,       0,  729123,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  583, 1460,  729145,  729157,  729179, 12, 22, 22, }, // Paraguay
    {  583, 1469,  729201,  729211,  729231, 10, 20, 20, }, // Peru
    {  583, 1474,  729251,  729266,  729291, 15, 25, 25, }, // Philippines
    {  583, 1486,       0,  729316,       0,  0, 20,  0, }, // Phoenix_Islands
    {  583, 1502,  729336,  729359,  729392, 23, 33, 33, }, // Pierre_Miquelon
    {  583, 1518,       0,  729425,       0,  0, 14,  0, }, // Pitcairn
    {  583, 1527,       0,  729439,       0,  0, 12,  0, }, // Ponape
    {  583, 1534,       0,  729451,       0,  0, 14,  0, }, // Pyongyang
    {  583, 1544,  729465,  729480,  729505, 15, 25, 25, }, // Qyzylorda
    {  583, 1554,       0,  729530,       0,  0, 13,  0, }, // Reunion
    {  583, 1562,       0,  729543,       0,  0, 12,  0, }, // Rothera
    {  583, 1570,  729555,  729568,  729591, 13, 23, 23, }, // Sakhalin
    {  583, 1579,  729614,  729625,  729646, 11, 21, 21, }, // Samara
    {  583, 1596,  729667,  729677,  729697, 10, 20, 20, }, // Samoa
    {  583, 1602,       0,  729717,       0,  0, 12,  0, }, // Seychelles
    {  583, 1624,       0,  729729,       0,  0, 24,  0, }, // Singapore
    {  583, 1634,       0,  729753,       0,  0, 20,  0, }, // Solomon
    {  583, 1642,       0,  729773,       0,  0, 21,  0, }, // South_Georgia
    {  583, 1656,       0,  729794,       0,  0, 13,  0, }, // Suriname
    {  583, 1676,       0,  729807,       0,  0, 10,  0, }, // Syowa
    {  583, 1682,       0,  729817,       0,  0, 11,  0, }, // Tahiti
    {  583, 1689,  729828,  729840,  729862, 12, 22, 22, }, // Taipei
    {  583, 1696,       0,  729884,       0,  0, 15,  0, }, // Tajikistan
    {  583, 1724,       0,  729899,       0,  0, 13,  0, }, // Tokelau
    {  583, 1732,  729912,  729922,  729942, 10, 20, 20, }, // Tonga
    {  583, 1738,       0,  729962,       0,  0, 10,  0, }, // Truk
    {  583, 1750,  729972,  729988,  730014, 16, 26, 26, }, // Turkmenistan
    {  583, 1763,       0,  730040,       0,  0, 12,  0, }, // Tuvalu
    {  583, 1777,  730052,  730066,  730090, 14, 24, 24, }, // Uruguay
    {  583, 1792,  730114,  730129,  730154, 15, 25, 25, }, // Uzbekistan
    {  583, 1803,  730179,  730192,  730215, 13, 23, 23, }, // Vanuatu
    {  583, 1811,       0,  730238,       0,  0, 13,  0, }, // Venezuela
    {  583, 1821,  730251,  730268,  730295, 17, 27, 27, }, // Vladivostok
    {  583, 1833,  730322,  730337,  730362, 15, 25, 25, }, // Volgograd
    {  583, 1843,       0,  730387,       0,  0, 12,  0, }, // Vostok
    {  583, 1850,       0,  730399,       0,  0, 18,  0, }, // Wake
    {  583, 1855,       0,  730417,       0,  0, 19,  0, }, // Wallis
    {  583, 1862,  730436,  730449,  730472, 13, 23, 23, }, // Yakutsk
    {  583, 1870,  730495,  730512,  730539, 17, 27, 27, }, // Yekaterinburg
    {  583, 1892,       0,  730566,       0,  0, 11,  0, }, // Yukon
    {  589,  142,  730577,  730590,  730614, 13, 24, 21, }, // America_Central Tajik/Cyrillic/Tajikistan
    {  589,  158,  730635,  730646,  730668, 11, 22, 19, }, // America_Eastern
    {  589,  174,  730687,  730697,  730718, 10, 21, 18, }, // America_Mountain
    {  589,  191,  730736,  730754,  730783, 18, 29, 26, }, // America_Pacific
    {  589,  286,  730809,  730824,  730850, 15, 26, 23, }, // Atlantic
    {  589,  638,  730873,  730895,  730928, 22, 33, 33, }, // Europe_Central
    {  589,  653,  730961,  730981,  731012, 20, 31, 31, }, // Europe_Eastern
    {  589,  691,  731043,  731063,  731094, 20, 31, 31, }, // Europe_Western
    {  589,  799,       0,  731125,       0,  0, 20,  0, }, // GMT
    {  590,    0,  731145,  731156,  731170, 11, 14, 16, }, // Acre Tamil/Tamil/India
    {  590,    5,       0,  731186,       0,  0, 20,  0, }, // Afghanistan
    {  590,   17,       0,  731206,       0,  0, 23,  0, }, // Africa_Central
    {  590,   32,       0,  731229,       0,  0, 24,  0, }, // Africa_Eastern
    {  590,   65,       0,  731253,       0,  0, 29,  0, }, // Africa_Southern
    {  590,   81,  731282,  731305,  731336, 23, 31, 28, }, // Africa_Western
    {  590,  107,  731364,  731377,  731398, 13, 21, 20, }, // Alaska
    {  590,  128,  731418,  731431,  731447, 13, 16, 18, }, // Almaty
    {  590,  135,  731465,  731478,  731499, 13, 21, 18, }, // Amazon
    {  590,  142,  731517,  731529,  731549, 12, 20, 19, }, // America_Central
    {  590,  158,  731568,  731585,  731610, 17, 25, 24, }, // America_Eastern
    {  590,  174,  731634,  731649,  731672, 15, 23, 22, }, // America_Mountain
    {  590,  191,  731694,  731707,  731728, 13, 21, 20, }, // America_Pacific
    {  590,  207,  731748,  731760,  731777, 12, 17, 19, }, // Anadyr
    {  590,  214,  731796,  731807,  731826, 11, 19, 18, }, // Apia
    {  590,  219,  731844,  731855,  731869, 11, 14, 16, }, // Aqtau
    {  590,  225,  731885,  731898,  731914, 13, 16, 18, }, // Aqtobe
    {  590,  232,  731932,  731944,  731964, 12, 20, 19, }, // Arabian
    {  590,  240,  731983,  732000,  732025, 17, 25, 22, }, // Argentina
    {  590,  250,  732047,  732075,  732111, 28, 36, 33, }, // Argentina_Western
    {  590,  268,  732144,  732158,  732180, 14, 22, 19, }, // Armenia
    {  590,  286,  732199,  732216,  732241, 17, 25, 24, }, // Atlantic
    {  590,  295,  732265,  732288,  732321, 23, 33, 32, }, // Australia_Central
    {  590,  313,  732353,  732389,  732433, 36, 44, 43, }, // Australia_CentralWestern
    {  590,  338,  732476,  732504,  732542, 28, 38, 37, }, // Australia_Eastern
    {  590,  356,  732579,  732606,  732643, 27, 37, 36, }, // Australia_Western
    {  590,  374,  732679,  732695,  732719, 16, 24, 21, }, // Azerbaijan
    {  590,  385,  732740,  732752,  732772, 12, 20, 18, }, // Azores
    {  590,  397,  732790,  732803,  732824, 13, 21, 18, }, // Bangladesh
    {  590,  415,       0,  732842,       0,  0, 12,  0, }, // Bhutan
    {  590,  422,       0,  732854,       0,  0, 14,  0, }, // Bolivia
    {  590,  437,  732868,  732884,  732908, 16, 24, 21, }, // Brasilia
    {  590,  454,       0,  732929,       0,  0, 23,  0, }, // Brunei
    {  590,  461,  732952,  732969,  732994, 17, 25, 22, }, // Cape_Verde
    {  590,  478,       0,  733016,       0,  0, 20,  0, }, // Chamorro
    {  590,  487,  733036,  733049,  733070, 13, 21, 20, }, // Chatham
    {  590,  495,  733090,  733100,  733118, 10, 18, 15, }, // Chile
    {  590,  501,  733133,  733142,  733159,  9, 17, 16, }, // China
    {  590,  507,  733175,  733192,  733217, 17, 25, 22, }, // Choibalsan
    {  590,  518,       0,  733239,       0,  0, 22,  0, }, // Christmas
    {  590,  528,       0,  733261,       0,  0, 20,  0, }, // Cocos
    {  590,  534,  733281,  733296,  733319, 15, 23, 20, }, // Colombia
    {  590,  543,  733339,  733357,  733383, 18, 26, 27, }, // Cook
    {  590,  548,  733410,  733422,  733442, 12, 20, 19, }, // Cuba
    {  590,  559,       0,  733461,       0,  0, 12,  0, }, // Davis
    {  590,  575,       0,  733473,       0,  0, 27,  0, }, // DumontDUrville
    {  590,  612,       0,  733500,       0,  0, 20,  0, }, // East_Timor
    {  590,  623,  733520,  733537,  733562, 17, 25, 22, }, // Easter
    {  590,  630,       0,  733584,       0,  0, 14,  0, }, // Ecuador
    {  590,  638,  733598,  733619,  733648, 21, 29, 26, }, // Europe_Central
    {  590,  653,  733674,  733700,  733734, 26, 34, 31, }, // Europe_Eastern
    {  590,  668,       0,  733765,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  590,  691,  733791,  733816,  733849, 25, 33, 30, }, // Europe_Western
    {  590,  706,  733879,  733904,  733937, 25, 33, 30, }, // Falkland
    {  590,  715,  733967,  733978,  733997, 11, 19, 16, }, // Fiji
    {  590,  720,       0,  734013,       0,  0, 20,  0, }, // French_Guiana
    {  590,  734,       0,  734033,       0,  0, 39,  0, }, // French_Southern
    {  590,  757,       0,  734072,       0,  0, 13,  0, }, // Galapagos
    {  590,  767,       0,  734085,       0,  0, 15,  0, }, // Gambier
    {  590,  775,  734100,  734114,  734136, 14, 22, 19, }, // Georgia
    {  590,  783,       0,  734155,       0,  0, 23,  0, }, // Gilbert_Islands
    {  590,  799,       0,  734178,       0,  0, 23,  0, }, // GMT
    {  590,  841,  734201,  734227,  734261, 26, 34, 31, }, // Greenland_Eastern
    {  590,  859,  734292,  734317,  734350, 25, 33, 30, }, // Greenland_Western
    {  590,  877,       0,  734380,       0,  0, 12,  0, }, // Guam
    {  590,  882,       0,  734392,       0,  0, 21,  0, }, // Gulf
    {  590,  887,       0,  734022,       0,  0, 11,  0, }, // Guyana
    {  590,  894,  734413,  734433,  734461, 20, 28, 27, }, // Hawaii_Aleutian
    {  590,  910,  734488,  734502,  734524, 14, 22, 19, }, // Hong_Kong
    {  590,  920,  734543,  734555,  734575, 12, 20, 17, }, // Hovd
    {  590,  925,       0,  734592,       0,  0, 20,  0, }, // India
    {  590,  931,       0,  734612,       0,  0, 25,  0, }, // Indian_Ocean
    {  590,  944,       0,  734637,       0,  0, 14,  0, }, // Indochina
    {  590,  954,       0,  734651,       0,  0, 23,  0, }, // Indonesia_Central
    {  590,  972,       0,  734674,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  590,  990,       0,  734702,       0,  0, 27,  0, }, // Indonesia_Western
    {  590, 1008,  734729,  734740,  734759, 11, 19, 18, }, // Iran
    {  590, 1019,  734777,  734794,  734819, 17, 25, 22, }, // Irkutsk
    {  590, 1027,  734841,  734854,  734875, 13, 21, 20, }, // Israel
    {  590, 1034,  734895,  734908,  734929, 13, 21, 20, }, // Japan
    {  590, 1040,  734949,  734983,  735020, 34, 37, 39, }, // Kamchatka
    {  590, 1058,       0,  735059,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  590, 1077,       0,  735082,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  590, 1106,  735104,  735115,  735134, 11, 19, 18, }, // Korea
    {  590, 1112,       0,  735152,       0,  0, 12,  0, }, // Kosrae
    {  590, 1119,  735164,  735185,  735214, 21, 29, 26, }, // Krasnoyarsk
    {  590, 1151,       0,  735240,       0,  0, 18,  0, }, // Kyrgystan
    {  590, 1161,       0,  735258,       0,  0, 11,  0, }, // Lanka
    {  590, 1175,       0,  735269,       0,  0, 18,  0, }, // Line_Islands
    {  590, 1188,  735287,  735304,  735329, 17, 25, 24, }, // Lord_Howe
    {  590, 1198,  735353,  735366,  735382, 13, 16, 18, }, // Macau
    {  590, 1204,  735400,  735411,  735430, 11, 19, 16, }, // Magadan
    {  590, 1219,       0,  735446,       0,  0, 12,  0, }, // Malaysia
    {  590, 1228,       0,  735458,       0,  0, 18,  0, }, // Maldives
    {  590, 1237,       0,  735476,       0,  0, 18,  0, }, // Marquesas
    {  590, 1247,       0,  735494,       0,  0, 21,  0, }, // Marshall_Islands
    {  590, 1264,  735515,  735530,  735553, 15, 23, 20, }, // Mauritius
    {  590, 1274,       0,  735573,       0,  0, 11,  0, }, // Mawson
    {  590, 1281,  735584,  735607,  735638, 23, 31, 30, }, // Mexico_Pacific
    {  590, 1296,  735668,  735684,  735708, 16, 24, 21, }, // Mongolia
    {  590, 1305,  735729,  735741,  735761, 12, 20, 17, }, // Moscow
    {  590, 1312,       0,  735778,       0,  0, 15,  0, }, // Myanmar
    {  590, 1320,       0,  735793,       0,  0, 11,  0, }, // Nauru
    {  590, 1326,       0,  735804,       0,  0, 11,  0, }, // Nepal
    {  590, 1332,  735815,  735835,  735863, 20, 28, 25, }, // New_Caledonia
    {  590, 1346,  735888,  735906,  735932, 18, 26, 25, }, // New_Zealand
    {  590, 1358,  735957,  735981,  736013, 24, 32, 31, }, // Newfoundland
    {  590, 1371,       0,  736044,       0,  0, 10,  0, }, // Niue
    {  590, 1376,  736054,  736074,  736102, 20, 28, 27, }, // Norfolk
    {  590, 1384,  736129,  736156,  736192, 27, 36, 33, }, // Noronha
    {  590, 1392,       0,  736225,       0,  0, 26,  0, }, // North_Mariana
    {  590, 1406,  736251,  736271,  736299, 20, 28, 25, }, // Novosibirsk
    {  590, 1418,  736324,  736337,  736358, 13, 21, 18, }, // Omsk
    {  590, 1428,  736376,  736392,  736416, 16, 24, 21, }, // Pakistan
    {  590, 1437,       0,  736437,       0,  0, 11,  0, }, // Palau
    {  590, 1443,       0,  736448,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  590, 1460,  736471,  736484,  736505, 13, 21, 18, }, // Paraguay
    {  590, 1469,  736523,  736533,  736551, 10, 18, 15, }, // Peru
    {  590, 1474,  736566,  736582,  736606, 16, 24, 21, }, // Philippines
    {  590, 1486,       0,  736627,       0,  0, 23,  0, }, // Phoenix_Islands
    {  590, 1502,  736650,  736685,  736728, 35, 43, 42, }, // Pierre_Miquelon
    {  590, 1518,       0,  736770,       0,  0, 18,  0, }, // Pitcairn
    {  590, 1527,       0,  736788,       0,  0, 12,  0, }, // Ponape
    {  590, 1534,       0,  736800,       0,  0, 16,  0, }, // Pyongyang
    {  590, 1544,  736816,  736832,  736851, 16, 19, 21, }, // Qyzylorda
    {  590, 1554,       0,  736872,       0,  0, 15,  0, }, // Reunion
    {  590, 1562,       0,  736887,       0,  0, 12,  0, }, // Rothera
    {  590, 1570,  736899,  736911,  736931, 12, 20, 17, }, // Sakhalin
    {  590, 1579,  736948,  736958,  736971, 10, 13, 15, }, // Samara
    {  590, 1596,  736986,  736997,  737016, 11, 19, 18, }, // Samoa
    {  590, 1602,       0,  737034,       0,  0, 14,  0, }, // Seychelles
    {  590, 1624,       0,  737048,       0,  0, 25,  0, }, // Singapore
    {  590, 1634,       0,  737073,       0,  0, 20,  0, }, // Solomon
    {  590, 1642,       0,  737093,       0,  0, 21,  0, }, // South_Georgia
    {  590, 1656,       0,  737114,       0,  0, 14,  0, }, // Suriname
    {  590, 1676,       0,  737128,       0,  0, 12,  0, }, // Syowa
    {  590, 1682,       0,  737140,       0,  0, 11,  0, }, // Tahiti
    {  590, 1689,  737151,  737163,  737183, 12, 20, 19, }, // Taipei
    {  590, 1696,       0,  737202,       0,  0, 17,  0, }, // Tajikistan
    {  590, 1724,       0,  737219,       0,  0, 16,  0, }, // Tokelau
    {  590, 1732,  737235,  737247,  737267, 12, 20, 17, }, // Tonga
    {  590, 1738,       0,  737284,       0,  0, 10,  0, }, // Truk
    {  590, 1750,  737294,  737316,  737346, 22, 30, 27, }, // Turkmenistan
    {  590, 1763,       0,  737373,       0,  0, 12,  0, }, // Tuvalu
    {  590, 1777,  737385,  737398,  737419, 13, 21, 18, }, // Uruguay
    {  590, 1792,  737437,  737456,  737483, 19, 27, 24, }, // Uzbekistan
    {  590, 1803,  737507,  737522,  737545, 15, 23, 20, }, // Vanuatu
    {  590, 1811,       0,  737565,       0,  0, 14,  0, }, // Venezuela
    {  590, 1821,  737579,  737599,  737627, 20, 28, 25, }, // Vladivostok
    {  590, 1833,  737652,  737670,  737696, 18, 26, 23, }, // Volgograd
    {  590, 1843,       0,  737585,       0,  0, 14,  0, }, // Vostok
    {  590, 1850,       0,  737719,       0,  0, 15,  0, }, // Wake
    {  590, 1855,       0,  737734,       0,  0, 30,  0, }, // Wallis
    {  590, 1862,  737764,  737779,  737802, 15, 23, 20, }, // Yakutsk
    {  590, 1870,  737822,  737843,  737872, 21, 29, 26, }, // Yekaterinburg
    {  590, 1892,       0,  737898,       0,  0, 12,  0, }, // Yukon
    {  591,  925,       0,  734592,       0,  0, 20,  0, }, // India Tamil/Tamil/Malaysia
    {  591, 1219,       0,  735446,       0,  0, 12,  0, }, // Malaysia
    {  591, 1624,       0,  737048,       0,  0, 25,  0, }, // Singapore
    {  592,  925,       0,  734592,       0,  0, 20,  0, }, // India Tamil/Tamil/Singapore
    {  592, 1219,       0,  735446,       0,  0, 12,  0, }, // Malaysia
    {  592, 1624,       0,  737048,       0,  0, 25,  0, }, // Singapore
    {  594,  142,  737910,  737927,  737951, 17, 24, 32, }, // America_Central Taroko/Latin/Taiwan
    {  594,  158,  737983,  738000,  738024, 17, 24, 32, }, // America_Eastern
    {  594,  174,  738056,  738074,  738099, 18, 25, 28, }, // America_Mountain
    {  594,  191,  738127,  738142,  738172, 15, 30, 38, }, // America_Pacific
    {  594,  286,  738210,  738229,  738255, 19, 26, 34, }, // Atlantic
    {  594,  638,  738289,  738300,  738318, 11, 18, 26, }, // Europe_Central
    {  594,  653,  738344,  738355,  738373, 11, 18, 26, }, // Europe_Eastern
    {  594,  691,  738399,  738409,  738426, 10, 17, 25, }, // Europe_Western
    {  594,  799,       0,  738451,       0,  0, 20,  0, }, // GMT
    {  596,    0,  738471,       0,       0, 10,  0,  0, }, // Acre Tatar/Cyrillic/Russia
    {  596,  142,  738481,  738507,  738540, 26, 33, 32, }, // America_Central
    {  596,  158,  738572,  738602,  738639, 30, 37, 36, }, // America_Eastern
    {  596,  174,  738675,  738700,  738732, 25, 32, 31, }, // America_Mountain
    {  596,  191,  738763,  738794,  738832, 31, 38, 37, }, // America_Pacific
    {  596,  207,  738869,       0,       0, 14,  0,  0, }, // Anadyr
    {  596,  286,  738883,  738913,  738950, 30, 37, 36, }, // Atlantic
    {  596,  638,  738986,  739004,  739029, 18, 25, 24, }, // Europe_Central
    {  596,  653,  739053,  739075,  739104, 22, 29, 28, }, // Europe_Eastern
    {  596,  691,  739132,  739154,  739183, 22, 29, 28, }, // Europe_Western
    {  596,  799,       0,  739211,       0,  0, 21,  0, }, // GMT
    {  596, 1040,  739232,       0,       0, 31,  0,  0, }, // Kamchatka
    {  596, 1579,  739263,       0,       0, 13,  0,  0, }, // Samara
    {  597,    0,  739276,  739285,  739304,  9, 19, 15, }, // Acre Telugu/Telugu/India
    {  597,    5,       0,  739319,       0,  0, 17,  0, }, // Afghanistan
    {  597,   17,       0,  739336,       0,  0, 21,  0, }, // Africa_Central
    {  597,   32,       0,  739357,       0,  0, 19,  0, }, // Africa_Eastern
    {  597,   65,       0,  739376,       0,  0, 29,  0, }, // Africa_Southern
    {  597,   81,  739405,  739424,  739453, 19, 29, 25, }, // Africa_Western
    {  597,  107,  739478,  739490,  739512, 12, 22, 26, }, // Alaska
    {  597,  128,  739538,  739550,  739572, 12, 22, 18, }, // Almaty
    {  597,  135,  739590,  739602,  739624, 12, 22, 18, }, // Amazon
    {  597,  142,  739642,  739652,  739672, 10, 20, 24, }, // America_Central
    {  597,  158,  739696,  739707,  739728, 11, 21, 25, }, // America_Eastern
    {  597,  174,  739753,  739767,  739791, 14, 24, 28, }, // America_Mountain
    {  597,  191,  739819,  739831,  739853, 12, 22, 26, }, // America_Pacific
    {  597,  207,  739879,  739890,  739914, 11, 24, 20, }, // Anadyr
    {  597,  214,  739934,  739944,  739964, 10, 20, 15, }, // Apia
    {  597,  219,  739979,  739991,  740013, 12, 22, 18, }, // Aqtau
    {  597,  225,  740031,  740043,  740065, 12, 22, 18, }, // Aqtobe
    {  597,  232,  740083,  740096,  740119, 13, 23, 27, }, // Arabian
    {  597,  240,  740146,  740161,  740186, 15, 25, 21, }, // Argentina
    {  597,  250,  740207,  740229,  740261, 22, 32, 28, }, // Argentina_Western
    {  597,  268,  740289,  740303,  740327, 14, 24, 20, }, // Armenia
    {  597,  286,  740347,  740362,  740387, 15, 25, 29, }, // Atlantic
    {  597,  295,  740416,  740438,  740470, 22, 32, 36, }, // Australia_Central
    {  597,  313,  740506,  740535,  740575, 29, 40, 43, }, // Australia_CentralWestern
    {  597,  338,  740618,  740641,  740675, 23, 34, 38, }, // Australia_Eastern
    {  597,  356,  740713,  740541,  740736, 23, 34, 38, }, // Australia_Western
    {  597,  374,  740774,  740789,  740814, 15, 25, 21, }, // Azerbaijan
    {  597,  385,  740835,  740847,  740869, 12, 22, 18, }, // Azores
    {  597,  397,  740887,  740902,  740927, 15, 25, 21, }, // Bangladesh
    {  597,  415,       0,  740948,       0,  0, 11,  0, }, // Bhutan
    {  597,  422,       0,  740959,       0,  0, 13,  0, }, // Bolivia
    {  597,  437,  740972,  740987,  741012, 15, 25, 21, }, // Brasilia
    {  597,  454,       0,  741033,       0,  0, 20,  0, }, // Brunei
    {  597,  461,  741053,  741069,  741095, 16, 26, 22, }, // Cape_Verde
    {  597,  478,       0,  741117,       0,  0, 22,  0, }, // Chamorro
    {  597,  487,  741139,  741149,  741169, 10, 20, 24, }, // Chatham
    {  597,  495,  741193,  741202,  741221,  9, 19, 15, }, // Chile
    {  597,  501,  741236,  741245,  741264,  9, 19, 23, }, // China
    {  597,  507,  741287,  741303,  741329, 16, 26, 22, }, // Choibalsan
    {  597,  518,       0,  741351,       0,  0, 19,  0, }, // Christmas
    {  597,  528,       0,  741370,       0,  0, 17,  0, }, // Cocos
    {  597,  534,  741387,  741400,  741423, 13, 23, 19, }, // Colombia
    {  597,  543,  741442,  741457,  741482, 15, 25, 26, }, // Cook
    {  597,  548,  741508,  741519,  741540, 11, 21, 25, }, // Cuba
    {  597,  559,       0,  741565,       0,  0, 11,  0, }, // Davis
    {  597,  575,       0,  741576,       0,  0, 27,  0, }, // DumontDUrville
    {  597,  612,       0,  741603,       0,  0, 18,  0, }, // East_Timor
    {  597,  623,  741621,  741637,  741663, 16, 26, 22, }, // Easter
    {  597,  630,       0,  741685,       0,  0, 13,  0, }, // Ecuador
    {  597,  638,  741698,  741721,  741754, 23, 33, 29, }, // Europe_Central
    {  597,  653,  741783,  741804,  741835, 21, 31, 27, }, // Europe_Eastern
    {  597,  668,       0,  741862,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  597,  691,  741889,  741910,  741941, 21, 31, 27, }, // Europe_Western
    {  597,  706,  741968,  741991,  742024, 23, 33, 29, }, // Falkland
    {  597,  715,  742053,  742062,  742081,  9, 19, 15, }, // Fiji
    {  597,  720,       0,  742096,       0,  0, 18,  0, }, // French_Guiana
    {  597,  734,       0,  742114,       0,  0, 38,  0, }, // French_Southern
    {  597,  757,       0,  742152,       0,  0, 15,  0, }, // Galapagos
    {  597,  767,       0,  742167,       0,  0, 13,  0, }, // Gambier
    {  597,  775,  742180,  742193,  742216, 13, 23, 19, }, // Georgia
    {  597,  783,       0,  742235,       0,  0, 20,  0, }, // Gilbert_Islands
    {  597,  799,       0,  742255,       0,  0, 21,  0, }, // GMT
    {  597,  841,  742276,  742302,  742338, 26, 36, 32, }, // Greenland_Eastern
    {  597,  859,  742370,  742396,  742432, 26, 36, 32, }, // Greenland_Western
    {  597,  877,       0,  742464,       0,  0, 21,  0, }, // Guam
    {  597,  882,       0,  742485,       0,  0, 20,  0, }, // Gulf
    {  597,  887,       0,  742104,       0,  0, 10,  0, }, // Guyana
    {  597,  894,  742505,  742526,  742557, 21, 31, 35, }, // Hawaii_Aleutian
    {  597,  910,  742592,  742605,  742628, 13, 23, 19, }, // Hong_Kong
    {  597,  920,  742647,  742658,  742679, 11, 21, 17, }, // Hovd
    {  597,  925,       0,  742696,       0,  0, 22,  0, }, // India
    {  597,  931,       0,  742718,       0,  0, 21,  0, }, // Indian_Ocean
    {  597,  944,       0,  742739,       0,  0, 13,  0, }, // Indochina
    {  597,  954,       0,  742752,       0,  0, 24,  0, }, // Indonesia_Central
    {  597,  972,       0,  742776,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  597,  990,       0,  742798,       0,  0, 22,  0, }, // Indonesia_Western
    {  597, 1008,  742820,  742830,  742850, 10, 20, 24, }, // Iran
    {  597, 1019,  742874,  742890,  742916, 16, 26, 22, }, // Irkutsk
    {  597, 1027,  742938,  742952,  742976, 14, 24, 28, }, // Israel
    {  597, 1034,  743004,  743014,  743034, 10, 20, 24, }, // Japan
    {  597, 1040,  743058,  743094,  743140, 36, 46, 42, }, // Kamchatka
    {  597, 1058,       0,  743182,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  597, 1077,       0,  743205,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  597, 1106,  743227,  743239,  743261, 12, 22, 26, }, // Korea
    {  597, 1112,       0,  743287,       0,  0, 13,  0, }, // Kosrae
    {  597, 1119,  743300,  743321,  743352, 21, 31, 27, }, // Krasnoyarsk
    {  597, 1151,       0,  743379,       0,  0, 17,  0, }, // Kyrgystan
    {  597, 1161,       0,  743396,       0,  0,  9,  0, }, // Lanka
    {  597, 1175,       0,  743405,       0,  0, 15,  0, }, // Line_Islands
    {  597, 1188,  743420,  743436,  743462, 16, 26, 21, }, // Lord_Howe
    {  597, 1198,  743483,  743493,  743513, 10, 20, 16, }, // Macau
    {  597, 1204,  743529,  743540,  743561, 11, 21, 17, }, // Magadan
    {  597, 1219,       0,  743578,       0,  0, 12,  0, }, // Malaysia
    {  597, 1228,       0,  743590,       0,  0, 14,  0, }, // Maldives
    {  597, 1237,       0,  743604,       0,  0, 17,  0, }, // Marquesas
    {  597, 1247,       0,  743621,       0,  0, 18,  0, }, // Marshall_Islands
    {  597, 1264,  743639,  743651,  743673, 12, 22, 18, }, // Mauritius
    {  597, 1274,       0,  743691,       0,  0, 10,  0, }, // Mawson
    {  597, 1281,  743701,  743723,  743755, 22, 32, 36, }, // Mexico_Pacific
    {  597, 1296,  743791,  743806,  743831, 15, 25, 21, }, // Mongolia
    {  597, 1305,  743852,  743863,  743884, 11, 21, 17, }, // Moscow
    {  597, 1312,       0,  743901,       0,  0, 13,  0, }, // Myanmar
    {  597, 1320,       0,  743914,       0,  0,  9,  0, }, // Nauru
    {  597, 1326,       0,  743923,       0,  0, 11,  0, }, // Nepal
    {  597, 1332,  743934,  743954,  743984, 20, 30, 26, }, // New_Caledonia
    {  597, 1346,  744010,  744028,  744056, 18, 28, 32, }, // New_Zealand
    {  597, 1358,  744088,  744110,  744142, 22, 32, 36, }, // Newfoundland
    {  597, 1371,       0,  744178,       0,  0,  9,  0, }, // Niue
    {  597, 1376,  744187,  744205,  744233, 18, 28, 23, }, // Norfolk
    {  597, 1384,  744256,  744282,  744318, 26, 36, 32, }, // Noronha
    {  597, 1392,       0,  744350,       0,  0, 25,  0, }, // North_Mariana
    {  597, 1406,  744375,  744394,  744423, 19, 29, 25, }, // Novosibirsk
    {  597, 1418,  744448,  744460,  744482, 12, 22, 18, }, // Omsk
    {  597, 1428,  744500,  744515,  744540, 15, 25, 21, }, // Pakistan
    {  597, 1437,       0,  744561,       0,  0, 11,  0, }, // Palau
    {  597, 1443,       0,  744572,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  597, 1460,  744595,  744607,  744629, 12, 22, 18, }, // Paraguay
    {  597, 1469,  744647,  744656,  744675,  9, 19, 15, }, // Peru
    {  597, 1474,  744690,  744705,  744730, 15, 25, 21, }, // Philippines
    {  597, 1486,       0,  744751,       0,  0, 19,  0, }, // Phoenix_Islands
    {  597, 1502,  744770,  744806,  744852, 36, 46, 49, }, // Pierre_Miquelon
    {  597, 1518,       0,  744901,       0,  0, 16,  0, }, // Pitcairn
    {  597, 1527,       0,  744917,       0,  0, 11,  0, }, // Ponape
    {  597, 1534,       0,  744928,       0,  0, 15,  0, }, // Pyongyang
    {  597, 1544,  744943,  744958,  744983, 15, 25, 21, }, // Qyzylorda
    {  597, 1554,       0,  745004,       0,  0, 14,  0, }, // Reunion
    {  597, 1562,       0,  745018,       0,  0, 11,  0, }, // Rothera
    {  597, 1570,  745029,  745041,  745063, 12, 22, 18, }, // Sakhalin
    {  597, 1579,  745081,  745091,  745111, 10, 20, 16, }, // Samara
    {  597, 1596,  745127,  745137,  745157, 10, 20, 24, }, // Samoa
    {  597, 1602,       0,  745181,       0,  0, 13,  0, }, // Seychelles
    {  597, 1624,       0,  745194,       0,  0, 23,  0, }, // Singapore
    {  597, 1634,       0,  745217,       0,  0, 17,  0, }, // Solomon
    {  597, 1642,       0,  745234,       0,  0, 20,  0, }, // South_Georgia
    {  597, 1656,       0,  745254,       0,  0, 13,  0, }, // Suriname
    {  597, 1676,       0,  745267,       0,  0, 11,  0, }, // Syowa
    {  597, 1682,       0,  745278,       0,  0, 10,  0, }, // Tahiti
    {  597, 1689,  745288,  745297,  745316,  9, 19, 23, }, // Taipei
    {  597, 1696,       0,  745339,       0,  0, 16,  0, }, // Tajikistan
    {  597, 1724,       0,  745355,       0,  0, 13,  0, }, // Tokelau
    {  597, 1732,  745368,  745378,  745398, 10, 20, 16, }, // Tonga
    {  597, 1738,       0,  745414,       0,  0,  8,  0, }, // Truk
    {  597, 1750,  745422,  745444,  745476, 22, 32, 28, }, // Turkmenistan
    {  597, 1763,       0,  745504,       0,  0, 11,  0, }, // Tuvalu
    {  597, 1777,  745515,  745527,  745549, 12, 22, 18, }, // Uruguay
    {  597, 1792,  745567,  745585,  745613, 18, 28, 24, }, // Uzbekistan
    {  597, 1803,  745637,  745647,  745667, 10, 20, 16, }, // Vanuatu
    {  597, 1811,       0,  745683,       0,  0, 13,  0, }, // Venezuela
    {  597, 1821,  745696,  745715,  745744, 19, 29, 25, }, // Vladivostok
    {  597, 1833,  745769,  745786,  745813, 17, 27, 23, }, // Volgograd
    {  597, 1843,       0,  745702,       0,  0, 13,  0, }, // Vostok
    {  597, 1850,       0,  745836,       0,  0, 14,  0, }, // Wake
    {  597, 1855,       0,  745850,       0,  0, 24,  0, }, // Wallis
    {  597, 1862,  745874,  745889,  745914, 15, 25, 21, }, // Yakutsk
    {  597, 1870,  745935,  745955,  745985, 20, 30, 26, }, // Yekaterinburg
    {  597, 1892,       0,  746011,       0,  0, 11,  0, }, // Yukon
    {  600,    0,  746022,  746031,  746047,  9, 16, 16, }, // Acre Thai/Thai/Thailand
    {  600,    5,       0,  746063,       0,  0, 15,  0, }, // Afghanistan
    {  600,   17,       0,  746078,       0,  0, 15,  0, }, // Africa_Central
    {  600,   32,       0,  746093,       0,  0, 19,  0, }, // Africa_Eastern
    {  600,   65,       0,  746112,       0,  0, 14,  0, }, // Africa_Southern
    {  600,   81,  746126,  746144,  746169, 18, 25, 25, }, // Africa_Western
    {  600,  107,  746194,  746205,  746223, 11, 18, 20, }, // Alaska
    {  600,  128,  746243,  746254,  746272, 11, 18, 18, }, // Almaty
    {  600,  135,  746290,  746301,  746319, 11, 18, 18, }, // Amazon
    {  600,  142,  746337,  746362,  746394, 25, 32, 31, }, // America_Central
    {  600,  158,  746425,  746454,  746490, 29, 36, 35, }, // America_Eastern
    {  600,  174,  746525,  746551,  746584, 26, 33, 32, }, // America_Mountain
    {  600,  191,  746616,  746641,  746673, 25, 32, 31, }, // America_Pacific
    {  600,  207,  746704,  746716,  746735, 12, 19, 19, }, // Anadyr
    {  600,  214,  746754,  746764,  746781, 10, 17, 16, }, // Apia
    {  600,  219,  746797,  746808,  746826, 11, 18, 18, }, // Aqtau
    {  600,  225,  746844,  746855,  746873, 11, 18, 18, }, // Aqtobe
    {  600,  232,  746891,  746901,  746918, 10, 17, 16, }, // Arabian
    {  600,  240,  746934,  746949,  746971, 15, 22, 22, }, // Argentina
    {  600,  250,  746993,  747018,  747053, 25, 35, 35, }, // Argentina_Western
    {  600,  268,  747088,  747102,  747123, 14, 21, 21, }, // Armenia
    {  600,  286,  747144,  747157,  747177, 13, 20, 22, }, // Atlantic
    {  600,  295,  747199,  747217,  747251, 18, 34, 33, }, // Australia_Central
    {  600,  313,  747284,  747318,  747359, 34, 41, 40, }, // Australia_CentralWestern
    {  600,  338,  747399,  747421,  747456, 22, 35, 34, }, // Australia_Eastern
    {  600,  356,  747490,  747511,  747545, 21, 34, 33, }, // Australia_Western
    {  600,  374,  747578,  747594,  747617, 16, 23, 23, }, // Azerbaijan
    {  600,  385,  747640,  747651,  747669, 11, 18, 18, }, // Azores
    {  600,  397,  747687,  747700,  747720, 13, 20, 20, }, // Bangladesh
    {  600,  415,       0,  747740,       0,  0,  9,  0, }, // Bhutan
    {  600,  422,       0,  747749,       0,  0, 12,  0, }, // Bolivia
    {  600,  437,  747761,  747774,  747794, 13, 20, 20, }, // Brasilia
    {  600,  454,       0,  747814,       0,  0, 19,  0, }, // Brunei
    {  600,  461,  747833,  747846,  747866, 13, 20, 20, }, // Cape_Verde
    {  600,  472,       0,  747886,       0,  0, 10,  0, }, // Casey
    {  600,  478,       0,  747896,       0,  0, 12,  0, }, // Chamorro
    {  600,  487,  747908,  747917,  747933,  9, 16, 15, }, // Chatham
    {  600,  495,  747948,  747956,  747971,  8, 15, 15, }, // Chile
    {  600,  501,  747986,  747993,  748007,  7, 14, 13, }, // China
    {  600,  507,  748020,  748033,  748053, 13, 20, 20, }, // Choibalsan
    {  600,  518,       0,  748073,       0,  0, 17,  0, }, // Christmas
    {  600,  528,       0,  748090,       0,  0, 17,  0, }, // Cocos
    {  600,  534,  748107,  748120,  748140, 13, 20, 20, }, // Colombia
    {  600,  543,  748160,  748175,  748197, 15, 22, 27, }, // Cook
    {  600,  548,  748224,  748233,  748249,  9, 16, 18, }, // Cuba
    {  600,  559,       0,  748267,       0,  0,  9,  0, }, // Davis
    {  600,  575,       0,  748276,       0,  0, 20,  0, }, // DumontDUrville
    {  600,  612,       0,  748296,       0,  0, 18,  0, }, // East_Timor
    {  600,  623,  748314,  748330,  748353, 16, 23, 23, }, // Easter
    {  600,  630,       0,  748376,       0,  0, 13,  0, }, // Ecuador
    {  600,  638,  748389,  748402,  748422, 13, 20, 20, }, // Europe_Central
    {  600,  653,  748442,  748459,  748483, 17, 24, 24, }, // Europe_Eastern
    {  600,  668,       0,  748507,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  600,  691,  748527,  748543,  748566, 16, 23, 23, }, // Europe_Western
    {  600,  706,  748589,  748611,  748640, 22, 29, 29, }, // Falkland
    {  600,  715,  748669,  748677,  748692,  8, 15, 15, }, // Fiji
    {  600,  720,       0,  748707,       0,  0, 16,  0, }, // French_Guiana
    {  600,  734,       0,  748723,       0,  0, 33,  0, }, // French_Southern
    {  600,  757,       0,  748756,       0,  0, 13,  0, }, // Galapagos
    {  600,  767,       0,  748769,       0,  0, 13,  0, }, // Gambier
    {  600,  775,  748782,  748794,  748813, 12, 19, 19, }, // Georgia
    {  600,  783,       0,  748832,       0,  0, 21,  0, }, // Gilbert_Islands
    {  600,  799,       0,  748853,       0,  0, 17,  0, }, // GMT
    {  600,  841,  748870,  748891,  748919, 21, 28, 28, }, // Greenland_Eastern
    {  600,  859,  748947,  748967,  748994, 20, 27, 27, }, // Greenland_Western
    {  600,  877,       0,  749021,       0,  0,  7,  0, }, // Guam
    {  600,  882,       0,  749028,       0,  0,  9,  0, }, // Gulf
    {  600,  887,       0,  749037,       0,  0, 11,  0, }, // Guyana
    {  600,  894,  749048,  749067,  749093, 19, 26, 25, }, // Hawaii_Aleutian
    {  600,  910,  749118,  749128,  749145, 10, 17, 17, }, // Hong_Kong
    {  600,  920,  749162,  749171,  749187,  9, 16, 16, }, // Hovd
    {  600,  925,       0,  749203,       0,  0, 11,  0, }, // India
    {  600,  931,       0,  749214,       0,  0, 19,  0, }, // Indian_Ocean
    {  600,  944,       0,  749233,       0,  0, 12,  0, }, // Indochina
    {  600,  954,       0,  749245,       0,  0, 22,  0, }, // Indonesia_Central
    {  600,  972,       0,  749267,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  600,  990,       0,  749294,       0,  0, 26,  0, }, // Indonesia_Western
    {  600, 1008,  749320,  749331,  749349, 11, 18, 17, }, // Iran
    {  600, 1019,  749366,  749380,  749401, 14, 21, 21, }, // Irkutsk
    {  600, 1027,  749422,  749434,  749453, 12, 19, 18, }, // Israel
    {  600, 1034,  749471,  749482,  749500, 11, 18, 17, }, // Japan
    {  600, 1040,  749517,  749529,  749557, 12, 28, 35, }, // Kamchatka
    {  600, 1058,       0,  749592,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  600, 1077,       0,  749613,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  600, 1106,  749633,  749643,  749660, 10, 17, 16, }, // Korea
    {  600, 1112,       0,  749676,       0,  0,  9,  0, }, // Kosrae
    {  600, 1119,  749685,  749700,  749722, 15, 22, 22, }, // Krasnoyarsk
    {  600, 1151,       0,  749744,       0,  0, 15,  0, }, // Kyrgystan
    {  600, 1161,       0,  749759,       0,  0,  9,  0, }, // Lanka
    {  600, 1175,       0,  749768,       0,  0, 16,  0, }, // Line_Islands
    {  600, 1188,  749784,  749797,  749817, 13, 20, 19, }, // Lord_Howe
    {  600, 1198,  749836,  749846,  749863, 10, 17, 17, }, // Macau
    {  600, 1204,  749880,  749891,  749909, 11, 18, 18, }, // Magadan
    {  600, 1219,       0,  749927,       0,  0, 12,  0, }, // Malaysia
    {  600, 1228,       0,  749939,       0,  0, 12,  0, }, // Maldives
    {  600, 1237,       0,  749951,       0,  0, 13,  0, }, // Marquesas
    {  600, 1247,       0,  749964,       0,  0, 21,  0, }, // Marshall_Islands
    {  600, 1264,  749985,  749998,  750018, 13, 20, 23, }, // Mauritius
    {  600, 1274,       0,  750041,       0,  0, 11,  0, }, // Mawson
    {  600, 1281,  750052,  750071,  750097, 19, 26, 25, }, // Mexico_Pacific
    {  600, 1296,  750122,  750137,  750159, 15, 22, 22, }, // Mongolia
    {  600, 1305,  750181,  750190,  750206,  9, 16, 16, }, // Moscow
    {  600, 1312,       0,  750222,       0,  0,  8,  0, }, // Myanmar
    {  600, 1320,       0,  750230,       0,  0, 10,  0, }, // Nauru
    {  600, 1326,       0,  750240,       0,  0,  9,  0, }, // Nepal
    {  600, 1332,  750249,  750266,  750290, 17, 24, 24, }, // New_Caledonia
    {  600, 1346,  750314,  750328,  750349, 14, 21, 20, }, // New_Zealand
    {  600, 1358,  750369,  750386,  750410, 17, 24, 23, }, // Newfoundland
    {  600, 1371,       0,  750433,       0,  0, 10,  0, }, // Niue
    {  600, 1376,  750443,  750460,  750484, 17, 24, 23, }, // Norfolk
    {  600, 1384,  750507,  750529,  750558, 22, 29, 32, }, // Noronha
    {  600, 1392,       0,  750590,       0,  0, 25,  0, }, // North_Mariana
    {  600, 1406,  750615,  750630,  750652, 15, 22, 22, }, // Novosibirsk
    {  600, 1418,  750674,  750684,  750701, 10, 17, 17, }, // Omsk
    {  600, 1428,  750718,  750730,  750749, 12, 19, 19, }, // Pakistan
    {  600, 1437,       0,  750768,       0,  0,  9,  0, }, // Palau
    {  600, 1443,       0,  750777,       0,  0, 16,  0, }, // Papua_New_Guinea
    {  600, 1460,  750793,  750805,  750824, 12, 19, 19, }, // Paraguay
    {  600, 1469,  750843,  750851,  750866,  8, 15, 15, }, // Peru
    {  600, 1474,  750881,  750895,  750916, 14, 21, 21, }, // Philippines
    {  600, 1486,       0,  750937,       0,  0, 19,  0, }, // Phoenix_Islands
    {  600, 1502,  750956,  750979,  751009, 23, 30, 32, }, // Pierre_Miquelon
    {  600, 1518,       0,  751041,       0,  0, 12,  0, }, // Pitcairn
    {  600, 1527,       0,  751053,       0,  0, 10,  0, }, // Ponape
    {  600, 1534,       0,  751063,       0,  0, 12,  0, }, // Pyongyang
    {  600, 1544,  751075,  751091,  751114, 16, 23, 23, }, // Qyzylorda
    {  600, 1554,       0,  751137,       0,  0, 12,  0, }, // Reunion
    {  600, 1562,       0,  751149,       0,  0, 10,  0, }, // Rothera
    {  600, 1570,  751159,  751170,  751188, 11, 18, 18, }, // Sakhalin
    {  600, 1579,  751206,  751216,  751233, 10, 17, 17, }, // Samara
    {  600, 1596,  751250,  751259,  751275,  9, 16, 16, }, // Samoa
    {  600, 1602,       0,  751291,       0,  0, 11,  0, }, // Seychelles
    {  600, 1624,       0,  751302,       0,  0, 12,  0, }, // Singapore
    {  600, 1634,       0,  751314,       0,  0, 19,  0, }, // Solomon
    {  600, 1642,       0,  751333,       0,  0, 17,  0, }, // South_Georgia
    {  600, 1656,       0,  751350,       0,  0, 12,  0, }, // Suriname
    {  600, 1676,       0,  751362,       0,  0, 10,  0, }, // Syowa
    {  600, 1682,       0,  751372,       0,  0, 10,  0, }, // Tahiti
    {  600, 1689,  751382,  751390,  751405,  8, 15, 14, }, // Taipei
    {  600, 1696,       0,  751419,       0,  0, 14,  0, }, // Tajikistan
    {  600, 1724,       0,  751433,       0,  0, 11,  0, }, // Tokelau
    {  600, 1732,  751444,  751453,  751469,  9, 16, 16, }, // Tonga
    {  600, 1738,       0,  751485,       0,  0,  7,  0, }, // Truk
    {  600, 1750,  751492,  751510,  751535, 18, 25, 25, }, // Turkmenistan
    {  600, 1763,       0,  751560,       0,  0, 10,  0, }, // Tuvalu
    {  600, 1777,  751570,  751582,  751601, 12, 19, 19, }, // Uruguay
    {  600, 1792,  751620,  751635,  751657, 15, 22, 22, }, // Uzbekistan
    {  600, 1803,  751679,  751691,  751710, 12, 19, 19, }, // Vanuatu
    {  600, 1811,       0,  751729,       0,  0, 14,  0, }, // Venezuela
    {  600, 1821,  751743,  751758,  751780, 15, 22, 22, }, // Vladivostok
    {  600, 1833,  751802,  751815,  751835, 13, 20, 20, }, // Volgograd
    {  600, 1843,       0,  751855,       0,  0, 10,  0, }, // Vostok
    {  600, 1850,       0,  751865,       0,  0, 11,  0, }, // Wake
    {  600, 1855,       0,  751876,       0,  0, 19,  0, }, // Wallis
    {  600, 1862,  751895,  751907,  751926, 12, 19, 19, }, // Yakutsk
    {  600, 1870,  751945,  751963,  751988, 18, 25, 25, }, // Yekaterinburg
    {  600, 1892,       0,  752013,       0,  0,  9,  0, }, // Yukon
    {  604,    0,  752022,  752028,  752038,  6, 10, 11, }, // Acre Tigrinya/Ethiopic/Ethiopia
    {  604,   17,       0,  752049,       0,  0, 13,  0, }, // Africa_Central
    {  604,   32,       0,  752062,       0,  0, 12,  0, }, // Africa_Eastern
    {  604,   65,       0,  752074,       0,  0, 11,  0, }, // Africa_Southern
    {  604,   81,  752085,  752097,  752113, 12, 16, 17, }, // Africa_Western
    {  604,  107,  752130,  752137,  752148,  7, 11, 12, }, // Alaska
    {  604,  135,  752160,  752167,  752178,  7, 11, 12, }, // Amazon
    {  604,  240,  752190,  752199,  752212,  9, 13, 14, }, // Argentina
    {  604,  385,  752226,  752233,  752244,  7, 11, 12, }, // Azores
    {  604,  422,       0,  752256,       0,  0,  7,  0, }, // Bolivia
    {  604,  437,  752263,  752271,  752283,  8, 12, 13, }, // Brasilia
    {  604,  461,  752296,  752305,  752318,  9, 13, 14, }, // Cape_Verde
    {  604,  495,  752332,  752337,  752346,  5,  9, 10, }, // Chile
    {  604,  534,  752356,  752364,  752376,  8, 12, 13, }, // Colombia
    {  604,  623,  752389,  752399,  752413, 10, 14, 15, }, // Easter
    {  604,  630,       0,  752428,       0,  0,  7,  0, }, // Ecuador
    {  604,  638,  752435,  752448,  752465, 13, 17, 12, }, // Europe_Central
    {  604,  653,  752477,  752489,  752505, 12, 16, 17, }, // Europe_Eastern
    {  604,  706,  752522,  752535,  752552, 13, 17, 18, }, // Falkland
    {  604,  720,       0,  752570,       0,  0, 13,  0, }, // French_Guiana
    {  604,  734,       0,  752583,       0,  0, 28,  0, }, // French_Southern
    {  604,  757,       0,  752611,       0,  0,  8,  0, }, // Galapagos
    {  604,  799,       0,  198209,       0,  0,  3,  0, }, // GMT
    {  604,  887,       0,  752619,       0,  0,  6,  0, }, // Guyana
    {  604,  931,       0,  752625,       0,  0, 13,  0, }, // Indian_Ocean
    {  604,  954,       0,  752638,       0,  0, 15,  0, }, // Indonesia_Central
    {  604,  972,       0,  752653,       0,  0, 15,  0, }, // Indonesia_Eastern
    {  604,  990,       0,  752668,       0,  0, 15,  0, }, // Indonesia_Western
    {  604, 1219,       0,  752683,       0,  0,  7,  0, }, // Malaysia
    {  604, 1264,  752690,  752698,  752710,  8, 12, 13, }, // Mauritius
    {  604, 1384,  752723,  752738,  752757, 15, 19, 20, }, // Noronha
    {  604, 1460,  752777,  752784,  752795,  7, 11, 12, }, // Paraguay
    {  604, 1469,  752807,  752812,  752821,  5,  9, 10, }, // Peru
    {  604, 1554,       0,  752831,       0,  0,  8,  0, }, // Reunion
    {  604, 1602,       0,  752839,       0,  0,  7,  0, }, // Seychelles
    {  604, 1624,       0,  752846,       0,  0,  8,  0, }, // Singapore
    {  604, 1642,       0,  752854,       0,  0, 11,  0, }, // South_Georgia
    {  604, 1656,       0,  752865,       0,  0,  7,  0, }, // Suriname
    {  604, 1777,  752872,  752879,  752890,  7, 11, 12, }, // Uruguay
    {  604, 1811,       0,  752902,       0,  0,  8,  0, }, // Venezuela
    {  607,    0,  752910,  752925,  752953, 15, 28, 27, }, // Acre Tongan/Latin/Tonga
    {  607,    5,       0,  752980,       0,  0, 23,  0, }, // Afghanistan
    {  607,   17,       0,  753003,       0,  0, 22,  0, }, // Africa_Central
    {  607,   32,       0,  753025,       0,  0, 24,  0, }, // Africa_Eastern
    {  607,   65,       0,  753049,       0,  0, 23,  0, }, // Africa_Southern
    {  607,   81,  753072,  753096,  753133, 24, 37, 36, }, // Africa_Western
    {  607,  107,  753169,  753186,  753216, 17, 30, 29, }, // Alaska
    {  607,  128,  753245,  753262,  753292, 17, 30, 29, }, // Almaty
    {  607,  135,  753321,  753338,  753368, 17, 30, 29, }, // Amazon
    {  607,  142,  753397,  753427,  753470, 30, 43, 42, }, // America_Central
    {  607,  158,  753512,  753544,  753589, 32, 45, 44, }, // America_Eastern
    {  607,  174,  753633,  753666,  753712, 33, 46, 45, }, // America_Mountain
    {  607,  191,  753757,  753791,  753838, 34, 47, 46, }, // America_Pacific
    {  607,  207,  753884,  753907,  753943, 23, 36, 35, }, // Anadyr
    {  607,  214,  753978,  753991,  754017, 13, 26, 25, }, // Apia
    {  607,  219,  754042,  754058,  754087, 16, 29, 28, }, // Aqtau
    {  607,  225,  754115,  754132,  754162, 17, 30, 29, }, // Aqtobe
    {  607,  232,  754191,  754207,  754236, 16, 29, 28, }, // Arabian
    {  607,  240,  754264,  754283,  754315, 19, 32, 31, }, // Argentina
    {  607,  250,  754346,  754372,  754411, 26, 39, 38, }, // Argentina_Western
    {  607,  268,  754449,  754465,  754494, 16, 29, 28, }, // Armenia
    {  607,  286,  754522,  754560,  754611, 38, 51, 50, }, // Atlantic
    {  607,  295,  754661,  754687,  754726, 26, 39, 38, }, // Australia_Central
    {  607,  313,  754764,  754797,  754843, 33, 46, 45, }, // Australia_CentralWestern
    {  607,  338,  754888,  754916,  754957, 28, 41, 40, }, // Australia_Eastern
    {  607,  356,  754997,  755025,  755066, 28, 41, 40, }, // Australia_Western
    {  607,  374,  755106,  755126,  755159, 20, 33, 32, }, // Azerbaijan
    {  607,  385,  755191,  755208,  755238, 17, 30, 29, }, // Azores
    {  607,  397,  755267,  755287,  755320, 20, 33, 32, }, // Bangladesh
    {  607,  415,       0,  755352,       0,  0, 15,  0, }, // Bhutan
    {  607,  422,       0,  755367,       0,  0, 16,  0, }, // Bolivia
    {  607,  437,  755383,  755400,  755430, 17, 30, 29, }, // Brasilia
    {  607,  454,       0,  755459,       0,  0, 16,  0, }, // Brunei
    {  607,  461,  755475,  755494,  755526, 19, 32, 31, }, // Cape_Verde
    {  607,  472,       0,  755557,       0,  0, 14,  0, }, // Casey
    {  607,  478,       0,  755571,       0,  0, 15,  0, }, // Chamorro
    {  607,  487,  755586,  755603,  755633, 17, 30, 29, }, // Chatham
    {  607,  495,  755662,  755675,  755701, 13, 26, 25, }, // Chile
    {  607,  501,  755726,  755741,  755769, 15, 28, 27, }, // China
    {  607,  507,  755796,  755816,  755849, 20, 33, 32, }, // Choibalsan
    {  607,  518,       0,  755881,       0,  0, 23,  0, }, // Christmas
    {  607,  528,       0,  755904,       0,  0, 19,  0, }, // Cocos
    {  607,  534,  755923,  755941,  755972, 18, 31, 30, }, // Colombia
    {  607,  543,  756002,  756023,  756057, 21, 34, 33, }, // Cook
    {  607,  548,  756090,  756104,  756131, 14, 27, 26, }, // Cuba
    {  607,  559,       0,  756157,       0,  0, 15,  0, }, // Davis
    {  607,  575,       0,  756172,       0,  0, 22,  0, }, // DumontDUrville
    {  607,  612,       0,  756194,       0,  0, 21,  0, }, // East_Timor
    {  607,  623,  756215,  756231,  756260, 16, 29, 28, }, // Easter
    {  607,  630,       0,  756288,       0,  0, 17,  0, }, // Ecuador
    {  607,  638,  756305,  756326,  756360, 21, 34, 33, }, // Europe_Central
    {  607,  653,  756393,  756416,  756452, 23, 36, 35, }, // Europe_Eastern
    {  607,  668,       0,  756487,       0,  0, 28,  0, }, // Europe_Further_Eastern
    {  607,  691,  756515,  756538,  756574, 23, 36, 35, }, // Europe_Western
    {  607,  706,  756609,  756635,  756674, 26, 39, 38, }, // Falkland
    {  607,  715,  756712,  756725,  756751, 13, 26, 25, }, // Fiji
    {  607,  720,       0,  756776,       0,  0, 28,  0, }, // French_Guiana
    {  607,  734,       0,  756804,       0,  0, 27,  0, }, // French_Southern
    {  607,  757,       0,  756831,       0,  0, 19,  0, }, // Galapagos
    {  607,  767,       0,  756850,       0,  0, 16,  0, }, // Gambier
    {  607,  775,  756866,  756881,  756909, 15, 28, 27, }, // Georgia
    {  607,  783,       0,  756936,       0,  0, 17,  0, }, // Gilbert_Islands
    {  607,  799,       0,  756953,       0,  0, 25,  0, }, // GMT
    {  607,  841,  756978,  757003,  757041, 25, 38, 37, }, // Greenland_Eastern
    {  607,  859,  757078,  757103,  757141, 25, 38, 37, }, // Greenland_Western
    {  607,  877,       0,  757178,       0,  0, 14,  0, }, // Guam
    {  607,  882,       0,  757192,       0,  0, 15,  0, }, // Gulf
    {  607,  887,       0,  756776,       0,  0, 15,  0, }, // Guyana
    {  607,  894,  757207,  757230,  757266, 23, 36, 35, }, // Hawaii_Aleutian
    {  607,  910,  757301,  757321,  757354, 20, 33, 32, }, // Hong_Kong
    {  607,  920,  757386,  757401,  757429, 15, 28, 27, }, // Hovd
    {  607,  925,       0,  757456,       0,  0, 16,  0, }, // India
    {  607,  931,       0,  757472,       0,  0, 21,  0, }, // Indian_Ocean
    {  607,  944,       0,  757493,       0,  0, 21,  0, }, // Indochina
    {  607,  954,       0,  757514,       0,  0, 25,  0, }, // Indonesia_Central
    {  607,  972,       0,  757539,       0,  0, 27,  0, }, // Indonesia_Eastern
    {  607,  990,       0,  757566,       0,  0, 27,  0, }, // Indonesia_Western
    {  607, 1008,  757593,  757609,  757638, 16, 29, 28, }, // Iran
    {  607, 1019,  757666,  757691,  757729, 25, 38, 37, }, // Irkutsk
    {  607, 1027,  757766,  757783,  757813, 17, 30, 29, }, // Israel
    {  607, 1034,  757842,  757858,  757887, 16, 29, 28, }, // Japan
    {  607, 1040,  757915,  757948,  757994, 33, 46, 45, }, // Kamchatka
    {  607, 1058,       0,  758039,       0,  0, 26,  0, }, // Kazakhstan_Eastern
    {  607, 1077,       0,  758065,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  607, 1106,  758091,  758105,  758132, 14, 27, 26, }, // Korea
    {  607, 1112,       0,  758158,       0,  0, 16,  0, }, // Kosrae
    {  607, 1119,  758174,  758203,  758245, 29, 42, 41, }, // Krasnoyarsk
    {  607, 1151,       0,  758286,       0,  0, 19,  0, }, // Kyrgystan
    {  607, 1161,       0,  758305,       0,  0, 16,  0, }, // Lanka
    {  607, 1175,       0,  758321,       0,  0, 22,  0, }, // Line_Islands
    {  607, 1188,  758343,  758365,  758400, 22, 35, 34, }, // Lord_Howe
    {  607, 1198,  758434,  758448,  758475, 14, 27, 26, }, // Macau
    {  607, 1204,  758501,  758524,  758560, 23, 36, 35, }, // Magadan
    {  607, 1219,       0,  758595,       0,  0, 17,  0, }, // Malaysia
    {  607, 1228,       0,  758612,       0,  0, 19,  0, }, // Maldives
    {  607, 1237,       0,  758631,       0,  0, 17,  0, }, // Marquesas
    {  607, 1247,       0,  758648,       0,  0, 23,  0, }, // Marshall_Islands
    {  607, 1264,  758671,  758690,  758722, 19, 32, 31, }, // Mauritius
    {  607, 1274,       0,  758753,       0,  0, 16,  0, }, // Mawson
    {  607, 1281,  758769,  758796,  758836, 27, 40, 39, }, // Mexico_Pacific
    {  607, 1296,  758875,  758894,  758926, 19, 32, 31, }, // Mongolia
    {  607, 1305,  758957,  758979,  759014, 22, 35, 34, }, // Moscow
    {  607, 1312,       0,  759048,       0,  0, 13,  0, }, // Myanmar
    {  607, 1320,       0,  759061,       0,  0, 14,  0, }, // Nauru
    {  607, 1326,       0,  759075,       0,  0, 15,  0, }, // Nepal
    {  607, 1332,  759090,  759113,  759149, 23, 36, 35, }, // New_Caledonia
    {  607, 1346,  759184,  759201,  759231, 17, 30, 29, }, // New_Zealand
    {  607, 1358,  759260,  759283,  759319, 23, 36, 35, }, // Newfoundland
    {  607, 1371,       0,  759354,       0,  0, 13,  0, }, // Niue
    {  607, 1376,  759367,  759383,  759412, 16, 29, 28, }, // Norfolk
    {  607, 1384,  759440,  759468,  759509, 28, 41, 40, }, // Noronha
    {  607, 1392,       0,  759549,       0,  0, 23,  0, }, // North_Mariana
    {  607, 1406,  759572,  759599,  759639, 27, 40, 39, }, // Novosibirsk
    {  607, 1418,  759678,  759701,  759737, 23, 36, 35, }, // Omsk
    {  607, 1428,  759772,  759791,  759823, 19, 32, 31, }, // Pakistan
    {  607, 1437,       0,  759854,       0,  0, 14,  0, }, // Palau
    {  607, 1443,       0,  759868,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  607, 1460,  759889,  759906,  759936, 17, 30, 29, }, // Paraguay
    {  607, 1469,  759965,  759978,  760004, 13, 26, 25, }, // Peru
    {  607, 1474,  760029,  760047,  760078, 18, 31, 30, }, // Philippines
    {  607, 1486,       0,  760108,       0,  0, 26,  0, }, // Phoenix_Islands
    {  607, 1502,  760134,  760162,  760203, 28, 41, 40, }, // Pierre_Miquelon
    {  607, 1518,       0,  760243,       0,  0, 17,  0, }, // Pitcairn
    {  607, 1527,       0,  760260,       0,  0, 15,  0, }, // Ponape
    {  607, 1534,       0,  760275,       0,  0, 19,  0, }, // Pyongyang
    {  607, 1544,  760294,  760311,  760341, 17, 30, 29, }, // Qyzylorda
    {  607, 1554,       0,  760370,       0,  0, 17,  0, }, // Reunion
    {  607, 1562,       0,  760387,       0,  0, 15,  0, }, // Rothera
    {  607, 1570,  760402,  760425,  760461, 23, 36, 35, }, // Sakhalin
    {  607, 1579,  760496,  760517,  760551, 21, 34, 33, }, // Samara
    {  607, 1596,  760584,  760600,  760629, 16, 29, 28, }, // Samoa
    {  607, 1602,       0,  760657,       0,  0, 25,  0, }, // Seychelles
    {  607, 1624,       0,  760682,       0,  0, 17,  0, }, // Singapore
    {  607, 1634,       0,  760699,       0,  0, 25,  0, }, // Solomon
    {  607, 1642,       0,  760724,       0,  0, 20,  0, }, // South_Georgia
    {  607, 1656,       0,  760744,       0,  0, 17,  0, }, // Suriname
    {  607, 1676,       0,  760761,       0,  0, 14,  0, }, // Syowa
    {  607, 1682,       0,  760775,       0,  0, 15,  0, }, // Tahiti
    {  607, 1689,  760790,  760805,  760833, 15, 28, 27, }, // Taipei
    {  607, 1696,       0,  760860,       0,  0, 19,  0, }, // Tajikistan
    {  607, 1724,       0,  760879,       0,  0, 16,  0, }, // Tokelau
    {  607, 1732,  760895,  760909,  760936, 14, 27, 26, }, // Tonga
    {  607, 1738,       0,  760962,       0,  0, 13,  0, }, // Truk
    {  607, 1750,  760975,  760998,  761034, 23, 36, 35, }, // Turkmenistan
    {  607, 1763,       0,  761069,       0,  0, 15,  0, }, // Tuvalu
    {  607, 1777,  761084,  761101,  761131, 17, 30, 29, }, // Uruguay
    {  607, 1792,  761160,  761181,  761215, 21, 34, 33, }, // Uzbekistan
    {  607, 1803,  761248,  761264,  761293, 16, 29, 28, }, // Vanuatu
    {  607, 1811,       0,  761321,       0,  0, 18,  0, }, // Venezuela
    {  607, 1821,  761339,  761368,  761410, 29, 42, 41, }, // Vladivostok
    {  607, 1833,  761451,  761478,  761518, 27, 40, 39, }, // Volgograd
    {  607, 1843,       0,  761557,       0,  0, 17,  0, }, // Vostok
    {  607, 1850,       0,  761574,       0,  0, 17,  0, }, // Wake
    {  607, 1855,       0,  761591,       0,  0, 24,  0, }, // Wallis
    {  607, 1862,  761615,  761641,  761680, 26, 39, 38, }, // Yakutsk
    {  607, 1870,  761718,  761748,  761791, 30, 43, 42, }, // Yekaterinburg
    {  607, 1892,       0,  761833,       0,  0, 15,  0, }, // Yukon
    {  611,    0,  761848,  761858,  761877, 10, 19, 14, }, // Acre Turkish/Latin/Turkey
    {  611,    5,       0,  761891,       0,  0, 16,  0, }, // Afghanistan
    {  611,   17,       0,  761907,       0,  0, 17,  0, }, // Africa_Central
    {  611,   32,       0,  761924,       0,  0, 17,  0, }, // Africa_Eastern
    {  611,   65,       0,  761941,       0,  0, 27,  0, }, // Africa_Southern
    {  611,   81,  761968,  761985,  762011, 17, 26, 21, }, // Africa_Western
    {  611,  107,  762032,  762044,  762065, 12, 21, 16, }, // Alaska
    {  611,  128,  762081,  762093,  762114, 12, 21, 16, }, // Almaty
    {  611,  135,  762130,  762142,  762163, 12, 21, 16, }, // Amazon
    {  611,  142,  762179,  762206,  762242, 27, 36, 31, }, // America_Central
    {  611,  158,  762273,  762297,  762330, 24, 33, 28, }, // America_Eastern
    {  611,  174,  762358,  762381,  762413, 23, 32, 27, }, // America_Mountain
    {  611,  191,  762440,  762467,  762503, 27, 36, 31, }, // America_Pacific
    {  611,  207,  762534,  762546,  762567, 12, 21, 16, }, // Anadyr
    {  611,  214,  762583,  762593,  762612, 10, 19, 14, }, // Apia
    {  611,  219,  762626,  762637,  762657, 11, 20, 15, }, // Aqtau
    {  611,  225,  762672,  762684,  762705, 12, 21, 16, }, // Aqtobe
    {  611,  232,  762721,  762736,  762760, 15, 24, 19, }, // Arabian
    {  611,  240,  762779,  762793,  762816, 14, 23, 18, }, // Argentina
    {  611,  250,  762834,  762853,  762881, 19, 28, 23, }, // Argentina_Western
    {  611,  268,  762904,  762920,  762945, 16, 25, 20, }, // Armenia
    {  611,  286,  762965,  762979,  763002, 14, 23, 18, }, // Atlantic
    {  611,  295,  763020,  763041,  763071, 21, 30, 25, }, // Australia_Central
    {  611,  313,  763096,  763120,  763153, 24, 33, 28, }, // Australia_CentralWestern
    {  611,  338,  763181,  763202,  763232, 21, 30, 25, }, // Australia_Eastern
    {  611,  356,  763099,  763123,  763156, 21, 30, 25, }, // Australia_Western
    {  611,  374,  763257,  763273,  763298, 16, 25, 20, }, // Azerbaijan
    {  611,  385,  763318,  763331,  763353, 13, 22, 17, }, // Azores
    {  611,  397,  763370,  763385,  763409, 15, 24, 19, }, // Bangladesh
    {  611,  415,       0,  763428,       0,  0, 11,  0, }, // Bhutan
    {  611,  422,       0,  763439,       0,  0, 13,  0, }, // Bolivia
    {  611,  437,  763452,  763466,  763489, 14, 23, 18, }, // Brasilia
    {  611,  454,       0,  763507,       0,  0, 25,  0, }, // Brunei
    {  611,  461,  763532,  763548,  763573, 16, 25, 20, }, // Cape_Verde
    {  611,  472,       0,  763593,       0,  0, 11,  0, }, // Casey
    {  611,  478,       0,  763604,       0,  0, 14,  0, }, // Chamorro
    {  611,  487,  763618,  763631,  763653, 13, 22, 17, }, // Chatham
    {  611,  495,  763670,  763680,  763699, 10, 19, 14, }, // Chile
    {  611,  501,  763713,  763722,  763740,  9, 18, 13, }, // China
    {  611,  507,  763753,  763768,  763792, 15, 24, 19, }, // Choibalsan
    {  611,  518,       0,  763811,       0,  0, 21,  0, }, // Christmas
    {  611,  528,       0,  763832,       0,  0, 19,  0, }, // Cocos
    {  611,  534,  763851,  763866,  763890, 15, 24, 19, }, // Colombia
    {  611,  543,  763909,  763927,  763954, 18, 27, 27, }, // Cook
    {  611,  548,  763981,  763991,  764010, 10, 19, 14, }, // Cuba
    {  611,  559,       0,  764024,       0,  0, 11,  0, }, // Davis
    {  611,  575,       0,  764035,       0,  0, 22,  0, }, // DumontDUrville
    {  611,  612,       0,  764057,       0,  0, 16,  0, }, // East_Timor
    {  611,  623,  764073,  764093,  764122, 20, 29, 24, }, // Easter
    {  611,  630,       0,  764146,       0,  0, 13,  0, }, // Ecuador
    {  611,  638,  764159,  764176,  764202, 17, 26, 21, }, // Europe_Central
    {  611,  653,  764223,  764240,  764266, 17, 26, 21, }, // Europe_Eastern
    {  611,  668,       0,  764287,       0,  0, 23,  0, }, // Europe_Further_Eastern
    {  611,  691,  764310,  764327,  764353, 17, 26, 21, }, // Europe_Western
    {  611,  706,  764374,  764396,  764427, 22, 31, 26, }, // Falkland
    {  611,  715,  764453,  764463,  764482, 10, 19, 14, }, // Fiji
    {  611,  720,       0,  764496,       0,  0, 22,  0, }, // French_Guiana
    {  611,  734,       0,  764518,       0,  0, 33,  0, }, // French_Southern
    {  611,  757,       0,  764551,       0,  0, 15,  0, }, // Galapagos
    {  611,  767,       0,  764566,       0,  0, 13,  0, }, // Gambier
    {  611,  775,  764579,  764594,  764618, 15, 24, 19, }, // Georgia
    {  611,  783,       0,  764637,       0,  0, 21,  0, }, // Gilbert_Islands
    {  611,  799,       0,  764658,       0,  0, 24,  0, }, // GMT
    {  611,  841,  764682,  764701,  764729, 19, 28, 23, }, // Greenland_Eastern
    {  611,  859,  764752,  764771,  764799, 19, 28, 23, }, // Greenland_Western
    {  611,  877,       0,  764822,       0,  0, 19,  0, }, // Guam
    {  611,  882,       0,  764841,       0,  0, 12,  0, }, // Gulf
    {  611,  887,       0,  764853,       0,  0, 12,  0, }, // Guyana
    {  611,  894,  764865,  764883,  764910, 18, 27, 22, }, // Hawaii_Aleutian
    {  611,  910,  764932,  764947,  764971, 15, 24, 19, }, // Hong_Kong
    {  611,  920,  764990,  765000,  765019, 10, 19, 14, }, // Hovd
    {  611,  925,       0,  765033,       0,  0, 24,  0, }, // India
    {  611,  931,       0,  765057,       0,  0, 19,  0, }, // Indian_Ocean
    {  611,  944,       0,  765076,       0,  0, 14,  0, }, // Indochina
    {  611,  954,       0,  765090,       0,  0, 20,  0, }, // Indonesia_Central
    {  611,  972,       0,  765110,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  611,  990,       0,  765130,       0,  0, 20,  0, }, // Indonesia_Western
    {  611, 1008,  765150,  765160,  765179, 10, 19, 14, }, // Iran
    {  611, 1019,  765193,  765206,  765228, 13, 22, 17, }, // Irkutsk
    {  611, 1027,  765245,  765257,  765278, 12, 21, 16, }, // Israel
    {  611, 1034,  765294,  765307,  765329, 13, 22, 17, }, // Japan
    {  611, 1040,  765346,  765375,  765413, 29, 38, 33, }, // Kamchatka
    {  611, 1058,       0,  765446,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  611, 1077,       0,  765467,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  611, 1106,  765488,  765498,  765517, 10, 19, 14, }, // Korea
    {  611, 1112,       0,  765531,       0,  0, 12,  0, }, // Kosrae
    {  611, 1119,  765543,  765560,  765586, 17, 26, 21, }, // Krasnoyarsk
    {  611, 1151,       0,  765607,       0,  0, 17,  0, }, // Kyrgystan
    {  611, 1161,       0,  765624,       0,  0, 11,  0, }, // Lanka
    {  611, 1175,       0,  765635,       0,  0, 18,  0, }, // Line_Islands
    {  611, 1188,  765653,  765668,  765692, 15, 24, 19, }, // Lord_Howe
    {  611, 1198,  765711,  765722,  765742, 11, 20, 15, }, // Macau
    {  611, 1204,  765757,  765770,  765792, 13, 22, 17, }, // Magadan
    {  611, 1219,       0,  765809,       0,  0, 13,  0, }, // Malaysia
    {  611, 1228,       0,  765822,       0,  0, 15,  0, }, // Maldives
    {  611, 1237,       0,  765837,       0,  0, 20,  0, }, // Marquesas
    {  611, 1247,       0,  765857,       0,  0, 22,  0, }, // Marshall_Islands
    {  611, 1264,  765879,  765894,  765918, 15, 24, 19, }, // Mauritius
    {  611, 1274,       0,  765937,       0,  0, 12,  0, }, // Mawson
    {  611, 1281,  765949,  765977,  766014, 28, 37, 32, }, // Mexico_Pacific
    {  611, 1296,  766046,  766062,  766087, 16, 25, 20, }, // Mongolia
    {  611, 1305,  766107,  766120,  766142, 13, 22, 17, }, // Moscow
    {  611, 1312,       0,  766159,       0,  0, 13,  0, }, // Myanmar
    {  611, 1320,       0,  766172,       0,  0, 11,  0, }, // Nauru
    {  611, 1326,       0,  766183,       0,  0, 11,  0, }, // Nepal
    {  611, 1332,  766194,  766214,  766243, 20, 29, 24, }, // New_Caledonia
    {  611, 1346,  766267,  766285,  766312, 18, 27, 22, }, // New_Zealand
    {  611, 1358,  766334,  766352,  766379, 18, 27, 22, }, // Newfoundland
    {  611, 1371,       0,  766401,       0,  0, 10,  0, }, // Niue
    {  611, 1376,  766411,  766430,  766458, 19, 28, 23, }, // Norfolk
    {  611, 1384,  766481,  766506,  766540, 25, 34, 29, }, // Noronha
    {  611, 1392,       0,  766569,       0,  0, 27,  0, }, // North_Mariana
    {  611, 1406,  766596,  766613,  766639, 17, 26, 21, }, // Novosibirsk
    {  611, 1418,  766660,  766670,  766689, 10, 19, 14, }, // Omsk
    {  611, 1428,  766703,  766717,  766740, 14, 23, 18, }, // Pakistan
    {  611, 1437,       0,  766758,       0,  0, 11,  0, }, // Palau
    {  611, 1443,       0,  766769,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  611, 1460,  766790,  766804,  766827, 14, 23, 18, }, // Paraguay
    {  611, 1469,  766845,  766855,  766874, 10, 19, 14, }, // Peru
    {  611, 1474,  766888,  766904,  766929, 16, 25, 20, }, // Philippines
    {  611, 1486,       0,  766949,       0,  0, 21,  0, }, // Phoenix_Islands
    {  611, 1502,  766970,  767000,  767039, 30, 39, 34, }, // Pierre_Miquelon
    {  611, 1518,       0,  767073,       0,  0, 14,  0, }, // Pitcairn
    {  611, 1527,       0,  767087,       0,  0, 12,  0, }, // Ponape
    {  611, 1534,       0,  767099,       0,  0, 15,  0, }, // Pyongyang
    {  611, 1544,  767114,  767129,  767153, 15, 24, 19, }, // Qyzylorda
    {  611, 1554,       0,  767172,       0,  0, 13,  0, }, // Reunion
    {  611, 1562,       0,  767185,       0,  0, 13,  0, }, // Rothera
    {  611, 1570,  767198,  767211,  767233, 13, 22, 17, }, // Sakhalin
    {  611, 1579,  767250,  767262,  767283, 12, 21, 16, }, // Samara
    {  611, 1596,  767299,  767310,  767330, 11, 20, 15, }, // Samoa
    {  611, 1602,       0,  767345,       0,  0, 15,  0, }, // Seychelles
    {  611, 1624,       0,  767360,       0,  0, 23,  0, }, // Singapore
    {  611, 1634,       0,  767383,       0,  0, 21,  0, }, // Solomon
    {  611, 1642,       0,  767404,       0,  0, 19,  0, }, // South_Georgia
    {  611, 1656,       0,  767423,       0,  0, 13,  0, }, // Suriname
    {  611, 1676,       0,  767436,       0,  0, 11,  0, }, // Syowa
    {  611, 1682,       0,  767447,       0,  0, 12,  0, }, // Tahiti
    {  611, 1689,  767459,  767471,  767492, 12, 21, 16, }, // Taipei
    {  611, 1696,       0,  767508,       0,  0, 16,  0, }, // Tajikistan
    {  611, 1724,       0,  767524,       0,  0, 13,  0, }, // Tokelau
    {  611, 1732,  767537,  767548,  767568, 11, 20, 15, }, // Tonga
    {  611, 1738,       0,  767583,       0,  0, 11,  0, }, // Truk
    {  611, 1750,  767594,  767612,  767639, 18, 27, 22, }, // Turkmenistan
    {  611, 1763,       0,  767661,       0,  0, 12,  0, }, // Tuvalu
    {  611, 1777,  767673,  767686,  767708, 13, 22, 17, }, // Uruguay
    {  611, 1792,  767725,  767741,  767766, 16, 25, 20, }, // Uzbekistan
    {  611, 1803,  767786,  767799,  767821, 13, 22, 17, }, // Vanuatu
    {  611, 1811,       0,  767838,       0,  0, 15,  0, }, // Venezuela
    {  611, 1821,  767853,  767870,  767896, 17, 26, 21, }, // Vladivostok
    {  611, 1833,  767917,  767932,  767956, 15, 24, 19, }, // Volgograd
    {  611, 1843,       0,  767975,       0,  0, 12,  0, }, // Vostok
    {  611, 1850,       0,  767987,       0,  0, 16,  0, }, // Wake
    {  611, 1855,       0,  768003,       0,  0, 22,  0, }, // Wallis
    {  611, 1862,  768025,  768038,  768060, 13, 22, 17, }, // Yakutsk
    {  611, 1870,  768077,  768096,  768124, 19, 28, 23, }, // Yekaterinburg
    {  611, 1892,       0,  768147,       0,  0, 11,  0, }, // Yukon
    {  613,    5,       0,  768158,       0,  0, 16,  0, }, // Afghanistan Turkmen/Latin/Turkmenistan
    {  613,   17,       0,  768174,       0,  0, 20,  0, }, // Africa_Central
    {  613,   32,       0,  768194,       0,  0, 21,  0, }, // Africa_Eastern
    {  613,   65,       0,  768215,       0,  0, 29,  0, }, // Africa_Southern
    {  613,   81,  768244,  768265,  768295, 21, 30, 29, }, // Africa_Western
    {  613,  107,  768324,  768337,  768359, 13, 22, 21, }, // Alaska
    {  613,  135,  768380,  768392,  768413, 12, 21, 20, }, // Amazon
    {  613,  142,  768433,  768448,  768478, 15, 30, 29, }, // America_Central
    {  613,  158,  768507,  768540,  768582, 33, 42, 41, }, // America_Eastern
    {  613,  174,  768623,  768651,  768688, 28, 37, 36, }, // America_Mountain
    {  613,  191,  768724,  768760,  768805, 36, 45, 44, }, // America_Pacific
    {  613,  207,  768849,       0,       0, 12,  0,  0, }, // Anadyr
    {  613,  214,  768861,  768871,  768890, 10, 19, 18, }, // Apia
    {  613,  232,  768908,  768927,  768955, 19, 28, 27, }, // Arabian
    {  613,  240,  768982,  768997,  769021, 15, 24, 23, }, // Argentina
    {  613,  250,  769044,  769068,  769101, 24, 33, 32, }, // Argentina_Western
    {  613,  268,  769133,  769149,  769174, 16, 25, 24, }, // Armenia
    {  613,  286,  769198,  769212,  769235, 14, 23, 22, }, // Atlantic
    {  613,  295,  769257,  769281,  769314, 24, 33, 32, }, // Australia_Central
    {  613,  313,  769346,  769379,  769421, 33, 42, 41, }, // Australia_CentralWestern
    {  613,  338,  769462,  769487,  769521, 25, 34, 33, }, // Australia_Eastern
    {  613,  356,  769554,  769579,  769613, 25, 34, 33, }, // Australia_Western
    {  613,  374,  769646,  769662,  769687, 16, 25, 24, }, // Azerbaijan
    {  613,  385,  769711,  769729,  769756, 18, 27, 26, }, // Azores
    {  613,  397,  769782,  769797,  769821, 15, 24, 23, }, // Bangladesh
    {  613,  415,       0,  769844,       0,  0, 11,  0, }, // Bhutan
    {  613,  422,       0,  769855,       0,  0, 14,  0, }, // Bolivia
    {  613,  437,  769869,  769884,  769908, 15, 24, 23, }, // Brasilia
    {  613,  454,       0,  769931,       0,  0, 23,  0, }, // Brunei
    {  613,  461,  769954,  769970,  769995, 16, 25, 24, }, // Cape_Verde
    {  613,  478,       0,  770019,       0,  0, 13,  0, }, // Chamorro
    {  613,  487,  770032,  770043,  770063, 11, 20, 19, }, // Chatham
    {  613,  495,  770082,  770092,  770111, 10, 19, 18, }, // Chile
    {  613,  501,  770129,  770140,  770160, 11, 20, 19, }, // China
    {  613,  507,  770179,  770194,  770218, 15, 24, 22, }, // Choibalsan
    {  613,  518,       0,  770240,       0,  0, 21,  0, }, // Christmas
    {  613,  528,       0,  770261,       0,  0, 19,  0, }, // Cocos
    {  613,  534,  770280,  770295,  770319, 15, 24, 23, }, // Colombia
    {  613,  543,  770342,  770359,  770385, 17, 26, 25, }, // Cook
    {  613,  548,  770410,  770420,  770439, 10, 19, 18, }, // Cuba
    {  613,  559,       0,  770457,       0,  0, 12,  0, }, // Davis
    {  613,  575,       0,  770469,       0,  0, 21,  0, }, // DumontDUrville
    {  613,  612,       0,  770490,       0,  0, 20,  0, }, // East_Timor
    {  613,  623,  770510,  770527,  770553, 17, 26, 25, }, // Easter
    {  613,  630,       0,  770578,       0,  0, 13,  0, }, // Ecuador
    {  613,  638,  770591,  770612,  770642, 21, 30, 29, }, // Europe_Central
    {  613,  653,  770671,  770693,  770724, 22, 31, 30, }, // Europe_Eastern
    {  613,  668,       0,  770754,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  613,  691,  770781,  770803,  770834, 22, 31, 30, }, // Europe_Western
    {  613,  706,  770864,  770886,  770917, 22, 31, 30, }, // Falkland
    {  613,  715,  770947,  770957,  770976, 10, 19, 18, }, // Fiji
    {  613,  720,       0,  770994,       0,  0, 22,  0, }, // French_Guiana
    {  613,  734,       0,  771016,       0,  0, 43,  0, }, // French_Southern
    {  613,  757,       0,  771059,       0,  0, 23,  0, }, // Galapagos
    {  613,  767,       0,  771082,       0,  0, 12,  0, }, // Gambier
    {  613,  775,  771094,  771107,  771129, 13, 22, 21, }, // Georgia
    {  613,  783,       0,  771150,       0,  0, 21,  0, }, // Gilbert_Islands
    {  613,  799,       0,  771171,       0,  0, 20,  0, }, // GMT
    {  613,  841,  771191,  771217,  771252, 26, 35, 34, }, // Greenland_Eastern
    {  613,  859,  771286,  771312,  771347, 26, 35, 34, }, // Greenland_Western
    {  613,  882,       0,  771381,       0,  0, 26,  0, }, // Gulf
    {  613,  887,       0,  771407,       0,  0, 12,  0, }, // Guyana
    {  613,  894,  771419,  771436,  771462, 17, 26, 25, }, // Hawaii_Aleutian
    {  613,  910,  771487,  771500,  771522, 13, 22, 21, }, // Hong_Kong
    {  613,  920,  771543,  771553,  771572, 10, 19, 18, }, // Hovd
    {  613,  925,       0,  771590,       0,  0, 24,  0, }, // India
    {  613,  931,       0,  771614,       0,  0, 18,  0, }, // Indian_Ocean
    {  613,  944,       0,  771632,       0,  0, 16,  0, }, // Indochina
    {  613,  954,       0,  771648,       0,  0, 24,  0, }, // Indonesia_Central
    {  613,  972,       0,  771672,       0,  0, 25,  0, }, // Indonesia_Eastern
    {  613,  990,       0,  771697,       0,  0, 25,  0, }, // Indonesia_Western
    {  613, 1008,  771722,  771733,  771753, 11, 20, 19, }, // Iran
    {  613, 1019,  771772,  771785,  771807, 13, 22, 21, }, // Irkutsk
    {  613, 1027,  771828,  771841,  771863, 13, 22, 21, }, // Israel
    {  613, 1034,  771884,  771898,  771921, 14, 23, 22, }, // Japan
    {  613, 1040,  771943,       0,       0, 30,  0,  0, }, // Kamchatka
    {  613, 1058,       0,  771973,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {  613, 1077,       0,  771998,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  613, 1106,  772023,  772035,  772056, 12, 21, 20, }, // Korea
    {  613, 1112,       0,  772076,       0,  0, 13,  0, }, // Kosrae
    {  613, 1119,  772089,  772106,  772132, 17, 26, 25, }, // Krasnoyarsk
    {  613, 1151,       0,  772157,       0,  0, 17,  0, }, // Kyrgystan
    {  613, 1175,       0,  772174,       0,  0, 18,  0, }, // Line_Islands
    {  613, 1188,  772192,  772206,  772229, 14, 23, 22, }, // Lord_Howe
    {  613, 1204,  772251,  772264,  772286, 13, 22, 21, }, // Magadan
    {  613, 1219,       0,  772307,       0,  0, 15,  0, }, // Malaysia
    {  613, 1228,       0,  772322,       0,  0, 15,  0, }, // Maldives
    {  613, 1237,       0,  772337,       0,  0, 20,  0, }, // Marquesas
    {  613, 1247,       0,  772357,       0,  0, 21,  0, }, // Marshall_Islands
    {  613, 1264,  772378,  772392,  772415, 14, 23, 22, }, // Mauritius
    {  613, 1274,       0,  772437,       0,  0, 12,  0, }, // Mawson
    {  613, 1281,  772449,  772475,  772510, 26, 35, 34, }, // Mexico_Pacific
    {  613, 1296,  772544,  772560,  772585, 16, 25, 24, }, // Mongolia
    {  613, 1305,  772609,  772621,  772642, 12, 21, 20, }, // Moscow
    {  613, 1312,       0,  772662,       0,  0, 12,  0, }, // Myanmar
    {  613, 1320,       0,  772674,       0,  0, 11,  0, }, // Nauru
    {  613, 1326,       0,  772685,       0,  0, 11,  0, }, // Nepal
    {  613, 1332,  772696,  772717,  772747, 21, 30, 29, }, // New_Caledonia
    {  613, 1346,  772776,  772796,  772825, 20, 29, 28, }, // New_Zealand
    {  613, 1358,  772853,  772871,  772898, 18, 27, 26, }, // Newfoundland
    {  613, 1371,       0,  772924,       0,  0, 10,  0, }, // Niue
    {  613, 1376,  772934,  772953,  772981, 19, 28, 27, }, // Norfolk
    {  613, 1384,  773008,  773033,  773067, 25, 34, 33, }, // Noronha
    {  613, 1406,  773100,  773117,  773143, 17, 26, 24, }, // Novosibirsk
    {  613, 1418,  773167,  773177,  773196, 10, 19, 18, }, // Omsk
    {  613, 1428,  773214,  773228,  773251, 14, 23, 22, }, // Pakistan
    {  613, 1437,       0,  773273,       0,  0, 11,  0, }, // Palau
    {  613, 1443,       0,  773284,       0,  0, 26,  0, }, // Papua_New_Guinea
    {  613, 1460,  773310,  773324,  773347, 14, 23, 22, }, // Paraguay
    {  613, 1469,  773369,  773379,  773398, 10, 19, 18, }, // Peru
    {  613, 1474,  773416,  773433,  773459, 17, 26, 25, }, // Philippines
    {  613, 1486,       0,  773484,       0,  0, 20,  0, }, // Phoenix_Islands
    {  613, 1502,  773504,  773523,  773557, 19, 34, 33, }, // Pierre_Miquelon
    {  613, 1518,       0,  773590,       0,  0, 13,  0, }, // Pitcairn
    {  613, 1527,       0,  773603,       0,  0, 12,  0, }, // Ponape
    {  613, 1534,       0,  773615,       0,  0, 13,  0, }, // Pyongyang
    {  613, 1554,       0,  773628,       0,  0, 14,  0, }, // Reunion
    {  613, 1562,       0,  773642,       0,  0, 12,  0, }, // Rothera
    {  613, 1570,  773654,  773667,  773689, 13, 22, 21, }, // Sakhalin
    {  613, 1579,  773710,       0,       0, 12,  0,  0, }, // Samara
    {  613, 1596,  773722,  773733,  773753, 11, 20, 19, }, // Samoa
    {  613, 1602,       0,  773772,       0,  0, 20,  0, }, // Seychelles
    {  613, 1624,       0,  773792,       0,  0, 14,  0, }, // Singapore
    {  613, 1634,       0,  773806,       0,  0, 21,  0, }, // Solomon
    {  613, 1642,       0,  773827,       0,  0, 22,  0, }, // South_Georgia
    {  613, 1656,       0,  773849,       0,  0, 13,  0, }, // Suriname
    {  613, 1676,       0,  773862,       0,  0, 11,  0, }, // Syowa
    {  613, 1682,       0,  773873,       0,  0, 11,  0, }, // Tahiti
    {  613, 1689,  773884,  773896,  773917, 12, 21, 20, }, // Taipei
    {  613, 1696,       0,  773937,       0,  0, 16,  0, }, // Tajikistan
    {  613, 1724,       0,  773953,       0,  0, 13,  0, }, // Tokelau
    {  613, 1732,  773966,  773977,  773997, 11, 20, 19, }, // Tonga
    {  613, 1738,       0,  774016,       0,  0, 10,  0, }, // Truk
    {  613, 1750,  774026,  774044,  774071, 18, 27, 26, }, // Turkmenistan
    {  613, 1763,       0,  774097,       0,  0, 12,  0, }, // Tuvalu
    {  613, 1777,  774109,  774122,  774144, 13, 22, 21, }, // Uruguay
    {  613, 1792,  774165,  774181,  774206, 16, 25, 24, }, // Uzbekistan
    {  613, 1803,  774230,  774243,  774265, 13, 22, 21, }, // Vanuatu
    {  613, 1811,       0,  774286,       0,  0, 15,  0, }, // Venezuela
    {  613, 1821,  774301,  774318,  774344, 17, 26, 25, }, // Vladivostok
    {  613, 1833,  774369,  774384,  774408, 15, 24, 23, }, // Volgograd
    {  613, 1843,       0,  774431,       0,  0, 12,  0, }, // Vostok
    {  613, 1850,       0,  774443,       0,  0, 16,  0, }, // Wake
    {  613, 1855,       0,  774459,       0,  0, 22,  0, }, // Wallis
    {  613, 1862,  774481,  774494,  774516, 13, 22, 21, }, // Yakutsk
    {  613, 1870,  774537,  774556,  774584, 19, 28, 27, }, // Yekaterinburg
    {  613, 1892,       0,  774611,       0,  0, 11,  0, }, // Yukon
    {  615,    0,  774622,  774631,  774653,  9, 22, 17, }, // Acre Ukrainian/Cyrillic/Ukraine
    {  615,    5,       0,  774670,       0,  0, 22,  0, }, // Afghanistan
    {  615,   17,       0,  774692,       0,  0, 31,  0, }, // Africa_Central
    {  615,   32,       0,  774723,       0,  0, 27,  0, }, // Africa_Eastern
    {  615,   65,       0,  774750,       0,  0, 29,  0, }, // Africa_Southern
    {  615,   81,  774779,  774807,  774847, 28, 40, 35, }, // Africa_Western
    {  615,  107,  774882,  774900,  774930, 18, 30, 25, }, // Alaska
    {  615,  135,  774955,  774975,  775007, 20, 32, 27, }, // Amazon
    {  615,  142,  775034,  775076,  775130, 42, 54, 49, }, // America_Central
    {  615,  158,  775179,  775217,  775267, 38, 50, 45, }, // America_Eastern
    {  615,  174,  775312,  775351,  775402, 39, 51, 46, }, // America_Mountain
    {  615,  191,  775448,  775493,  775550, 45, 57, 52, }, // America_Pacific
    {  615,  207,  775602,  775613,  775637, 11, 24, 19, }, // Anadyr
    {  615,  214,  775656,  775671,  775698, 15, 27, 22, }, // Apia
    {  615,  232,  775720,  775738,  775768, 18, 30, 25, }, // Arabian
    {  615,  240,  775793,  775815,  775849, 22, 34, 29, }, // Argentina
    {  615,  250,  775878,  775907,  775948, 29, 41, 39, }, // Argentina_Western
    {  615,  268,  775987,  776007,  776039, 20, 32, 27, }, // Armenia
    {  615,  286,  776066,  776086,  776118, 20, 32, 27, }, // Atlantic
    {  615,  295,  776145,  776178,  776223, 33, 45, 40, }, // Australia_Central
    {  615,  313,  776263,  776305,  776359, 42, 54, 49, }, // Australia_CentralWestern
    {  615,  338,  776408,  776437,  776478, 29, 41, 36, }, // Australia_Eastern
    {  615,  356,  776514,  776544,  776586, 30, 42, 37, }, // Australia_Western
    {  615,  374,  776623,  776648,  776685, 25, 37, 32, }, // Azerbaijan
    {  615,  385,  776717,  776747,  776789, 30, 42, 37, }, // Azores
    {  615,  397,  776826,  776846,  776878, 20, 32, 27, }, // Bangladesh
    {  615,  415,       0,  776905,       0,  0, 17,  0, }, // Bhutan
    {  615,  422,       0,  776922,       0,  0, 21,  0, }, // Bolivia
    {  615,  437,  776943,  776964,  776997, 21, 33, 28, }, // Brasilia
    {  615,  454,       0,  777025,       0,  0, 17,  0, }, // Brunei
    {  615,  461,  777042,  777073,  777116, 31, 43, 38, }, // Cape_Verde
    {  615,  478,       0,  777154,       0,  0, 42,  0, }, // Chamorro
    {  615,  487,  777196,  777224,  777264, 28, 40, 35, }, // Chatham
    {  615,  495,  777299,  777318,  777349, 19, 31, 26, }, // Chile
    {  615,  501,  777375,  777394,  777425, 19, 31, 26, }, // China
    {  615,  507,  777451,  777472,  777505, 21, 33, 28, }, // Choibalsan
    {  615,  518,       0,  777533,       0,  0, 26,  0, }, // Christmas
    {  615,  528,       0,  777559,       0,  0, 30,  0, }, // Cocos
    {  615,  534,  777589,  777611,  777645, 22, 34, 29, }, // Colombia
    {  615,  543,  777674,  777699,  777736, 25, 37, 32, }, // Cook
    {  615,  548,  777768,  777784,  777812, 16, 28, 23, }, // Cuba
    {  615,  559,       0,  777835,       0,  0, 25,  0, }, // Davis
    {  615,  575,       0,  777860,       0,  0, 34,  0, }, // DumontDUrville
    {  615,  612,       0,  777894,       0,  0, 26,  0, }, // East_Timor
    {  615,  623,  777920,  777945,  777982, 25, 37, 32, }, // Easter
    {  615,  630,       0,  778014,       0,  0, 19,  0, }, // Ecuador
    {  615,  638,  778033,  778064,  778107, 31, 43, 38, }, // Europe_Central
    {  615,  653,  778145,  778172,  778211, 27, 39, 34, }, // Europe_Eastern
    {  615,  668,       0,  778245,       0,  0, 35,  0, }, // Europe_Further_Eastern
    {  615,  691,  778280,  778308,  778348, 28, 40, 35, }, // Europe_Western
    {  615,  706,  778383,  778417,  778463, 34, 46, 41, }, // Falkland
    {  615,  715,  778504,  778520,  778548, 16, 28, 23, }, // Fiji
    {  615,  720,       0,  778571,       0,  0, 27,  0, }, // French_Guiana
    {  615,  734,       0,  778598,       0,  0, 59,  0, }, // French_Southern
    {  615,  757,       0,  778657,       0,  0, 19,  0, }, // Galapagos
    {  615,  767,       0,  778676,       0,  0, 26,  0, }, // Gambier
    {  615,  775,  778702,  778722,  778754, 20, 32, 27, }, // Georgia
    {  615,  783,       0,  778781,       0,  0, 29,  0, }, // Gilbert_Islands
    {  615,  799,       0,  778810,       0,  0, 12,  0, }, // GMT
    {  615,  841,  778822,  778851,  778892, 29, 41, 36, }, // Greenland_Eastern
    {  615,  859,  778928,  778958,  779000, 30, 42, 37, }, // Greenland_Western
    {  615,  877,       0,  779037,       0,  0, 24,  0, }, // Guam
    {  615,  882,       0,  779061,       0,  0, 24,  0, }, // Gulf
    {  615,  887,       0,  779085,       0,  0, 16,  0, }, // Guyana
    {  615,  894,  779101,  779130,  779171, 29, 41, 36, }, // Hawaii_Aleutian
    {  615,  910,  779207,  779226,  779257, 19, 31, 26, }, // Hong_Kong
    {  615,  920,  779283,  779299,  779327, 16, 28, 23, }, // Hovd
    {  615,  925,       0,  779350,       0,  0, 31,  0, }, // India
    {  615,  931,       0,  779381,       0,  0, 29,  0, }, // Indian_Ocean
    {  615,  944,       0,  779410,       0,  0, 20,  0, }, // Indochina
    {  615,  954,       0,  779430,       0,  0, 33,  0, }, // Indonesia_Central
    {  615,  972,       0,  779463,       0,  0, 29,  0, }, // Indonesia_Eastern
    {  615,  990,       0,  779492,       0,  0, 30,  0, }, // Indonesia_Western
    {  615, 1008,  779522,  779540,  779570, 18, 30, 25, }, // Iran
    {  615, 1019,  779595,  779614,  779645, 19, 31, 26, }, // Irkutsk
    {  615, 1027,  779671,  779692,  779725, 21, 33, 28, }, // Israel
    {  615, 1034,  779753,  779771,  779801, 18, 30, 25, }, // Japan
    {  615, 1040,  779826,  779846,  779878, 20, 32, 27, }, // Kamchatka
    {  615, 1058,       0,  779905,       0,  0, 29,  0, }, // Kazakhstan_Eastern
    {  615, 1077,       0,  779934,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  615, 1106,  779964,  779983,  780014, 19, 31, 26, }, // Korea
    {  615, 1112,       0,  780040,       0,  0, 26,  0, }, // Kosrae
    {  615, 1119,  780066,  780088,  780122, 22, 34, 29, }, // Krasnoyarsk
    {  615, 1151,       0,  780151,       0,  0, 22,  0, }, // Kyrgystan
    {  615, 1161,       0,  780173,       0,  0, 10,  0, }, // Lanka
    {  615, 1175,       0,  780183,       0,  0, 24,  0, }, // Line_Islands
    {  615, 1188,  780207,  780235,  780275, 28, 40, 35, }, // Lord_Howe
    {  615, 1204,  780310,  780331,  780364, 21, 33, 28, }, // Magadan
    {  615, 1219,       0,  780392,       0,  0, 19,  0, }, // Malaysia
    {  615, 1228,       0,  780411,       0,  0, 21,  0, }, // Maldives
    {  615, 1237,       0,  780432,       0,  0, 31,  0, }, // Marquesas
    {  615, 1247,       0,  780463,       0,  0, 32,  0, }, // Marshall_Islands
    {  615, 1264,  780495,  780523,  780563, 28, 40, 35, }, // Mauritius
    {  615, 1274,       0,  780598,       0,  0, 26,  0, }, // Mawson
    {  615, 1281,  780624,  780657,  780702, 33, 45, 40, }, // Mexico_Pacific
    {  615, 1296,  780742,  780764,  780798, 22, 34, 29, }, // Mongolia
    {  615, 1305,  780827,  780847,  780879, 20, 32, 27, }, // Moscow
    {  615, 1312,       0,  780906,       0,  0, 17,  0, }, // Myanmar
    {  615, 1320,       0,  780923,       0,  0, 25,  0, }, // Nauru
    {  615, 1326,       0,  780948,       0,  0, 17,  0, }, // Nepal
    {  615, 1332,  780965,  781001,  781049, 36, 48, 43, }, // New_Caledonia
    {  615, 1346,  781092,  781117,  781154, 25, 37, 32, }, // New_Zealand
    {  615, 1358,  781186,  781218,  781262, 32, 44, 30, }, // Newfoundland
    {  615, 1371,       0,  781292,       0,  0, 24,  0, }, // Niue
    {  615, 1376,  781316,  781343,  781382, 27, 39, 34, }, // Norfolk
    {  615, 1384,  781416,  781458,  781512, 42, 54, 49, }, // Noronha
    {  615, 1406,  781561,  781584,  781619, 23, 35, 30, }, // Novosibirsk
    {  615, 1418,  781649,  781665,  781693, 16, 28, 23, }, // Omsk
    {  615, 1428,  781716,  781736,  781768, 20, 32, 27, }, // Pakistan
    {  615, 1437,       0,  781795,       0,  0, 25,  0, }, // Palau
    {  615, 1443,       0,  781820,       0,  0, 38,  0, }, // Papua_New_Guinea
    {  615, 1460,  781858,  781880,  781914, 22, 34, 29, }, // Paraguay
    {  615, 1469,  781943,  781958,  781985, 15, 27, 22, }, // Peru
    {  615, 1474,  782007,  782029,  782063, 22, 34, 29, }, // Philippines
    {  615, 1486,       0,  782092,       0,  0, 27,  0, }, // Phoenix_Islands
    {  615, 1502,  782119,  782158,  782209, 39, 51, 46, }, // Pierre_Miquelon
    {  615, 1518,       0,  782255,       0,  0, 28,  0, }, // Pitcairn
    {  615, 1527,       0,  782283,       0,  0, 26,  0, }, // Ponape
    {  615, 1534,       0,  782309,       0,  0, 19,  0, }, // Pyongyang
    {  615, 1544,  782328,  782342,  782369, 14, 27, 22, }, // Qyzylorda
    {  615, 1554,       0,  782391,       0,  0, 28,  0, }, // Reunion
    {  615, 1562,       0,  782419,       0,  0, 26,  0, }, // Rothera
    {  615, 1570,  782445,  782466,  782499, 21, 33, 28, }, // Sakhalin
    {  615, 1579,  782527,  782546,  782577, 19, 31, 26, }, // Samara
    {  615, 1596,  782603,  782628,  782665, 25, 37, 32, }, // Samoa
    {  615, 1602,       0,  782697,       0,  0, 33,  0, }, // Seychelles
    {  615, 1624,       0,  782730,       0,  0, 20,  0, }, // Singapore
    {  615, 1634,       0,  782750,       0,  0, 32,  0, }, // Solomon
    {  615, 1642,       0,  782782,       0,  0, 37,  0, }, // South_Georgia
    {  615, 1656,       0,  782819,       0,  0, 19,  0, }, // Suriname
    {  615, 1676,       0,  782838,       0,  0, 24,  0, }, // Syowa
    {  615, 1682,       0,  782862,       0,  0, 25,  0, }, // Tahiti
    {  615, 1689,  782887,  782904,  782933, 17, 29, 24, }, // Taipei
    {  615, 1696,       0,  782957,       0,  0, 23,  0, }, // Tajikistan
    {  615, 1724,       0,  782980,       0,  0, 28,  0, }, // Tokelau
    {  615, 1732,  783008,  783034,  783072, 26, 38, 33, }, // Tonga
    {  615, 1738,       0,  783105,       0,  0, 25,  0, }, // Truk
    {  615, 1750,  783130,  783154,  783190, 24, 36, 31, }, // Turkmenistan
    {  615, 1763,       0,  783221,       0,  0, 27,  0, }, // Tuvalu
    {  615, 1777,  783248,  783266,  783296, 18, 30, 25, }, // Uruguay
    {  615, 1792,  783321,  783343,  783377, 22, 34, 29, }, // Uzbekistan
    {  615, 1803,  783406,  783434,  783474, 28, 40, 35, }, // Vanuatu
    {  615, 1811,       0,  783509,       0,  0, 20,  0, }, // Venezuela
    {  615, 1821,  783529,  783553,  783589, 24, 36, 31, }, // Vladivostok
    {  615, 1833,  783620,  783643,  783678, 23, 35, 30, }, // Volgograd
    {  615, 1843,       0,  783708,       0,  0, 26,  0, }, // Vostok
    {  615, 1850,       0,  783734,       0,  0, 24,  0, }, // Wake
    {  615, 1855,       0,  783758,       0,  0, 36,  0, }, // Wallis
    {  615, 1862,  783794,  783812,  783842, 18, 30, 25, }, // Yakutsk
    {  615, 1870,  783867,  783892,  783929, 25, 37, 32, }, // Yekaterinburg
    {  615, 1892,       0,  783961,       0,  0, 29,  0, }, // Yukon
    {  616,    5,       0,  783990,       0,  0, 13,  0, }, // Afghanistan Upper Sorbian/Latin/Germany
    {  616,   17,       0,  784003,       0,  0, 20,  0, }, // Africa_Central
    {  616,   32,       0,  784023,       0,  0, 18,  0, }, // Africa_Eastern
    {  616,   65,       0,  784041,       0,  0, 16,  0, }, // Africa_Southern
    {  616,   81,  784057,  784074,  784102, 17, 28, 23, }, // Africa_Western
    {  616,  107,  784125,  784138,  784162, 13, 24, 19, }, // Alaska
    {  616,  135,  784181,  784195,  784220, 14, 25, 20, }, // Amazon
    {  616,  142,  784240,  784269,  784309, 29, 40, 35, }, // America_Central
    {  616,  158,  784344,  784372,  784411, 28, 39, 34, }, // America_Eastern
    {  616,  174,  784445,  784471,  784508, 26, 37, 32, }, // America_Mountain
    {  616,  191,  784540,  784569,  784609, 29, 40, 35, }, // America_Pacific
    {  616,  214,  784644,  784655,  784677, 11, 22, 17, }, // Apia
    {  616,  232,  784694,  784705,  784727, 11, 22, 17, }, // Arabian
    {  616,  240,  784744,  784759,  784785, 15, 26, 21, }, // Argentina
    {  616,  250,  784806,  784827,  784859, 21, 32, 27, }, // Argentina_Western
    {  616,  268,  784886,  784898,  784921, 12, 23, 18, }, // Armenia
    {  616,  286,  784939,  784953,  784978, 14, 25, 20, }, // Atlantic
    {  616,  295,  784998,  785019,  785051, 21, 32, 27, }, // Australia_Central
    {  616,  313,  785078,  785107,  785147, 29, 40, 34, }, // Australia_CentralWestern
    {  616,  338,  785181,  785202,  785234, 21, 32, 27, }, // Australia_Eastern
    {  616,  356,  785261,  785281,  785312, 20, 31, 26, }, // Australia_Western
    {  616,  374,  785338,  785356,  785385, 18, 29, 24, }, // Azerbaijan
    {  616,  385,  785409,  785420,  785442, 11, 22, 17, }, // Azores
    {  616,  397,  785459,  785475,  785502, 16, 27, 22, }, // Bangladesh
    {  616,  415,       0,  785524,       0,  0, 13,  0, }, // Bhutan
    {  616,  422,       0,  785537,       0,  0, 13,  0, }, // Bolivia
    {  616,  437,  785550,  785564,  785589, 14, 25, 20, }, // Brasilia
    {  616,  454,       0,  785609,       0,  0, 13,  0, }, // Brunei
    {  616,  461,  785622,  785636,  785661, 14, 25, 20, }, // Cape_Verde
    {  616,  478,       0,  785681,       0,  0, 15,  0, }, // Chamorro
    {  616,  487,  785696,  785710,  785735, 14, 25, 20, }, // Chatham
    {  616,  495,  785755,  785766,  785788, 11, 22, 17, }, // Chile
    {  616,  501,  785805,  785816,  785838, 11, 22, 17, }, // China
    {  616,  507,  687851,  785855,  785882, 16, 27, 22, }, // Choibalsan
    {  616,  518,       0,  785904,       0,  0, 18,  0, }, // Christmas
    {  616,  528,       0,  785922,       0,  0, 20,  0, }, // Cocos
    {  616,  534,  785942,  785956,  785981, 14, 25, 20, }, // Colombia
    {  616,  543,  786001,  786020,  786050, 19, 30, 25, }, // Cook
    {  616,  548,  786075,  786086,  786108, 11, 22, 17, }, // Cuba
    {  616,  559,       0,  786125,       0,  0, 11,  0, }, // Davis
    {  616,  575,       0,  786136,       0,  0, 24,  0, }, // DumontDUrville
    {  616,  612,       0,  786160,       0,  0, 20,  0, }, // East_Timor
    {  616,  623,  786180,  786199,  786229, 19, 30, 25, }, // Easter
    {  616,  630,       0,  786254,       0,  0, 14,  0, }, // Ecuador
    {  616,  638,  786268,  786287,  786317, 19, 30, 25, }, // Europe_Central
    {  616,  653,  786342,  786361,  786391, 19, 30, 25, }, // Europe_Eastern
    {  616,  668,       0,  786416,       0,  0, 18,  0, }, // Europe_Further_Eastern
    {  616,  691,  786434,  786452,  786481, 18, 29, 24, }, // Europe_Western
    {  616,  706,  786505,  786520,  786546, 15, 26, 21, }, // Falkland
    {  616,  715,  786567,  786579,  786602, 12, 23, 18, }, // Fiji
    {  616,  720,       0,  786620,       0,  0, 21,  0, }, // French_Guiana
    {  616,  734,       0,  786641,       0,  0, 50,  0, }, // French_Southern
    {  616,  757,       0,  786691,       0,  0, 15,  0, }, // Galapagos
    {  616,  767,       0,  786706,       0,  0, 14,  0, }, // Gambier
    {  616,  775,  786720,  786733,  786757, 13, 24, 19, }, // Georgia
    {  616,  783,       0,  786776,       0,  0, 22,  0, }, // Gilbert_Islands
    {  616,  799,       0,  786798,       0,  0, 16,  0, }, // GMT
    {  616,  841,  786814,  786836,  786869, 22, 33, 28, }, // Greenland_Eastern
    {  616,  859,  786897,  786918,  786950, 21, 32, 27, }, // Greenland_Western
    {  616,  882,       0,  786977,       0,  0, 20,  0, }, // Gulf
    {  616,  887,       0,  786629,       0,  0, 12,  0, }, // Guyana
    {  616,  894,  786997,  787019,  787052, 22, 33, 28, }, // Hawaii_Aleutian
    {  616,  910,  787080,  787095,  787121, 15, 26, 21, }, // Hong_Kong
    {  616,  920,  787142,  787154,  787177, 12, 23, 18, }, // Hovd
    {  616,  925,       0,  787195,       0,  0, 11,  0, }, // India
    {  616,  931,       0,  787206,       0,  0, 19,  0, }, // Indian_Ocean
    {  616,  944,       0,  787225,       0,  0, 15,  0, }, // Indochina
    {  616,  954,       0,  787240,       0,  0, 20,  0, }, // Indonesia_Central
    {  616,  972,       0,  787260,       0,  0, 16,  0, }, // Indonesia_Eastern
    {  616,  990,       0,  787276,       0,  0, 19,  0, }, // Indonesia_Western
    {  616, 1008,  787295,  787306,  787328, 11, 22, 17, }, // Iran
    {  616, 1019,  689466,  787345,  787368, 12, 23, 18, }, // Irkutsk
    {  616, 1027,  787386,  787399,  787423, 13, 24, 19, }, // Israel
    {  616, 1034,  787442,  787454,  787477, 12, 23, 18, }, // Japan
    {  616, 1058,       0,  787495,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  616, 1077,       0,  787516,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  616, 1106,  787536,  787548,  787571, 12, 23, 18, }, // Korea
    {  616, 1112,       0,  787589,       0,  0, 13,  0, }, // Kosrae
    {  616, 1119,  689850,  787602,  787629, 16, 27, 22, }, // Krasnoyarsk
    {  616, 1151,       0,  787651,       0,  0, 12,  0, }, // Kyrgystan
    {  616, 1175,       0,  787663,       0,  0, 20,  0, }, // Line_Islands
    {  616, 1188,  787683,  787701,  787730, 18, 29, 24, }, // Lord_Howe
    {  616, 1204,  690032,  787754,  787779, 14, 25, 20, }, // Magadan
    {  616, 1219,       0,  787799,       0,  0, 14,  0, }, // Malaysia
    {  616, 1228,       0,  787813,       0,  0, 14,  0, }, // Maldives
    {  616, 1237,       0,  787827,       0,  0, 15,  0, }, // Marquesas
    {  616, 1247,       0,  787842,       0,  0, 23,  0, }, // Marshall_Islands
    {  616, 1264,  787865,  787880,  787906, 15, 26, 21, }, // Mauritius
    {  616, 1274,       0,  690225,       0,  0, 13,  0, }, // Mawson
    {  616, 1281,  787927,  787948,  787980, 21, 32, 27, }, // Mexico_Pacific
    {  616, 1296,  788007,  788024,  788052, 17, 28, 23, }, // Mongolia
    {  616, 1305,  788075,  788088,  788112, 13, 24, 19, }, // Moscow
    {  616, 1312,       0,  788131,       0,  0, 14,  0, }, // Myanmar
    {  616, 1320,       0,  788145,       0,  0, 12,  0, }, // Nauru
    {  616, 1326,       0,  788157,       0,  0, 12,  0, }, // Nepal
    {  616, 1332,  788169,  788187,  788216, 18, 29, 24, }, // New_Caledonia
    {  616, 1346,  788240,  788258,  788287, 18, 29, 24, }, // New_Zealand
    {  616, 1358,  788311,  788330,  788360, 19, 30, 25, }, // Newfoundland
    {  616, 1371,       0,  788385,       0,  0, 11,  0, }, // Niue
    {  616, 1376,  788396,  788412,  788439, 16, 27, 22, }, // Norfolk
    {  616, 1384,  788461,  788490,  788530, 29, 40, 35, }, // Noronha
    {  616, 1406,  788565,  788581,  788608, 16, 27, 22, }, // Novosibirsk
    {  616, 1418,  690958,  788630,  788650,  9, 20, 15, }, // Omsk
    {  616, 1428,  788665,  788680,  788706, 15, 26, 21, }, // Pakistan
    {  616, 1437,       0,  788727,       0,  0, 12,  0, }, // Palau
    {  616, 1443,       0,  788739,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  616, 1460,  788761,  788776,  788802, 15, 26, 21, }, // Paraguay
    {  616, 1469,  788823,  788834,  788856, 11, 22, 17, }, // Peru
    {  616, 1474,  788873,  788887,  788912, 14, 25, 20, }, // Philippines
    {  616, 1486,       0,  788932,       0,  0, 22,  0, }, // Phoenix_Islands
    {  616, 1502,  788954,  788985,  789027, 31, 42, 37, }, // Pierre_Miquelon
    {  616, 1518,       0,  789064,       0,  0, 23,  0, }, // Pitcairn
    {  616, 1527,       0,  789087,       0,  0, 13,  0, }, // Ponape
    {  616, 1534,       0,  789100,       0,  0, 16,  0, }, // Pyongyang
    {  616, 1554,       0,  789116,       0,  0, 14,  0, }, // Reunion
    {  616, 1562,       0,  789130,       0,  0, 14,  0, }, // Rothera
    {  616, 1570,  789144,  789159,  789185, 15, 26, 21, }, // Sakhalin
    {  616, 1596,  789206,  789218,  789241, 12, 23, 18, }, // Samoa
    {  616, 1602,       0,  789259,       0,  0, 15,  0, }, // Seychelles
    {  616, 1624,       0,  789274,       0,  0, 15,  0, }, // Singapore
    {  616, 1634,       0,  789289,       0,  0, 22,  0, }, // Solomon
    {  616, 1642,       0,  789311,       0,  0, 18,  0, }, // South_Georgia
    {  616, 1656,       0,  789329,       0,  0, 14,  0, }, // Suriname
    {  616, 1676,       0,  789343,       0,  0, 12,  0, }, // Syowa
    {  616, 1682,       0,  789355,       0,  0, 13,  0, }, // Tahiti
    {  616, 1689,  789368,  789381,  789405, 13, 24, 19, }, // Taipei
    {  616, 1696,       0,  789424,       0,  0, 13,  0, }, // Tajikistan
    {  616, 1724,       0,  789437,       0,  0, 14,  0, }, // Tokelau
    {  616, 1732,  789451,  789463,  789486, 12, 23, 18, }, // Tonga
    {  616, 1738,       0,  789504,       0,  0, 12,  0, }, // Truk
    {  616, 1750,  789516,  789530,  789555, 14, 25, 20, }, // Turkmenistan
    {  616, 1763,       0,  789575,       0,  0, 13,  0, }, // Tuvalu
    {  616, 1777,  789588,  789602,  789627, 14, 25, 20, }, // Uruguay
    {  616, 1792,  789647,  789659,  789682, 12, 23, 18, }, // Uzbekistan
    {  616, 1803,  789700,  789714,  789739, 14, 25, 20, }, // Vanuatu
    {  616, 1811,       0,  789759,       0,  0, 15,  0, }, // Venezuela
    {  616, 1821,  789774,  789792,  789821, 18, 29, 24, }, // Vladivostok
    {  616, 1833,  789845,  789861,  789888, 16, 27, 22, }, // Volgograd
    {  616, 1843,       0,  789910,       0,  0, 13,  0, }, // Vostok
    {  616, 1850,       0,  789923,       0,  0, 13,  0, }, // Wake
    {  616, 1855,       0,  789936,       0,  0, 25,  0, }, // Wallis
    {  616, 1862,  692403,  789961,  789984, 12, 23, 18, }, // Yakutsk
    {  616, 1870,  790002,  790022,  790053, 20, 31, 26, }, // Yekaterinburg
    {  616, 1892,       0,  790079,       0,  0, 12,  0, }, // Yukon
    {  617,    5,       0,  790091,       0,  0, 16,  0, }, // Afghanistan Urdu/Arabic/Pakistan
    {  617,   17,       0,  790107,       0,  0, 16,  0, }, // Africa_Central
    {  617,   32,       0,  790123,       0,  0, 17,  0, }, // Africa_Eastern
    {  617,   65,       0,  790140,       0,  0, 25,  0, }, // Africa_Southern
    {  617,   81,  790165,  790182,  790207, 17, 25, 21, }, // Africa_Western
    {  617,  107,  790228,  790239,  790259, 11, 20, 19, }, // Alaska
    {  617,  135,  790278,  790289,  790310, 11, 21, 26, }, // Amazon
    {  617,  142,  790336,  790346,  790365, 10, 19, 18, }, // America_Central
    {  617,  158,  790383,  790394,  790414, 11, 20, 19, }, // America_Eastern
    {  617,  174,  790433,  790445,  790466, 12, 21, 20, }, // America_Mountain
    {  617,  191,  790486,  790496,  790515, 10, 19, 18, }, // America_Pacific
    {  617,  207,  790533,  790543,  790562, 10, 19, 14, }, // Anadyr
    {  617,  214,  790576,  790586,  790604, 10, 18, 18, }, // Apia
    {  617,  232,  790622,  790632,  790649, 10, 17, 16, }, // Arabian
    {  617,  240,  790665,  790680,  790702, 15, 22, 28, }, // Argentina
    {  617,  250,  790730,  790751,  790779, 21, 28, 34, }, // Argentina_Western
    {  617,  268,  790813,  790827,  790848, 14, 21, 27, }, // Armenia
    {  617,  286,  790875,  790887,  790908, 12, 21, 20, }, // Atlantic
    {  617,  295,  790928,  790947,  790975, 19, 28, 27, }, // Australia_Central
    {  617,  313,  791002,  791028,  791063, 26, 35, 34, }, // Australia_CentralWestern
    {  617,  338,  791097,  791117,  791146, 20, 29, 28, }, // Australia_Eastern
    {  617,  356,  791174,  791194,  791223, 20, 29, 28, }, // Australia_Western
    {  617,  374,  791251,  791268,  791292, 17, 24, 30, }, // Azerbaijan
    {  617,  385,  791322,  791335,  791355, 13, 20, 26, }, // Azores
    {  617,  397,  791381,  791397,  791420, 16, 23, 29, }, // Bangladesh
    {  617,  415,       0,  791449,       0,  0, 13,  0, }, // Bhutan
    {  617,  422,       0,  791462,       0,  0, 14,  0, }, // Bolivia
    {  617,  437,  791476,  791489,  791511, 13, 22, 17, }, // Brasilia
    {  617,  454,       0,  791528,       0,  0, 21,  0, }, // Brunei
    {  617,  461,  791549,  791562,  791583, 13, 21, 17, }, // Cape_Verde
    {  617,  478,       0,  791600,       0,  0, 19,  0, }, // Chamorro
    {  617,  487,  791619,  791629,  791648, 10, 19, 18, }, // Chatham
    {  617,  495,  791666,  791676,  791693, 10, 17, 23, }, // Chile
    {  617,  501,  791716,  791726,  791742, 10, 16, 17, }, // China
    {  617,  507,  791759,  791772,  791793, 13, 21, 18, }, // Choibalsan
    {  617,  518,       0,  791811,       0,  0, 17,  0, }, // Christmas
    {  617,  528,       0,  791828,       0,  0, 18,  0, }, // Cocos
    {  617,  534,  791846,  791858,  791879, 12, 21, 27, }, // Colombia
    {  617,  543,  791906,  791921,  791944, 15, 23, 23, }, // Cook
    {  617,  548,  791967,  791977,  791996, 10, 19, 18, }, // Cuba
    {  617,  559,       0,  792014,       0,  0,  9,  0, }, // Davis
    {  617,  575,       0,  792023,       0,  0, 21,  0, }, // DumontDUrville
    {  617,  612,       0,  792044,       0,  0, 16,  0, }, // East_Timor
    {  617,  623,  792060,  792079,  792105, 19, 26, 32, }, // Easter
    {  617,  630,       0,  792137,       0,  0, 15,  0, }, // Ecuador
    {  617,  638,  792152,  792167,  792190, 15, 23, 29, }, // Europe_Central
    {  617,  653,  792219,  792236,  792260, 17, 24, 30, }, // Europe_Eastern
    {  617,  668,       0,  792290,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  617,  691,  792310,  792327,  792351, 17, 24, 30, }, // Europe_Western
    {  617,  706,  792381,  792404,  792434, 23, 30, 36, }, // Falkland
    {  617,  715,  792470,  792478,  792494,  8, 16, 12, }, // Fiji
    {  617,  720,       0,  792506,       0,  0, 18,  0, }, // French_Guiana
    {  617,  734,       0,  792524,       0,  0, 29,  0, }, // French_Southern
    {  617,  757,       0,  792553,       0,  0, 16,  0, }, // Galapagos
    {  617,  767,       0,  792569,       0,  0, 12,  0, }, // Gambier
    {  617,  775,  792581,  792594,  792614, 13, 20, 26, }, // Georgia
    {  617,  783,       0,  792640,       0,  0, 18,  0, }, // Gilbert_Islands
    {  617,  799,       0,  792658,       0,  0, 18,  0, }, // GMT
    {  617,  841,  792676,  792696,  792725, 20, 29, 35, }, // Greenland_Eastern
    {  617,  859,  792760,  792780,  792809, 20, 29, 35, }, // Greenland_Western
    {  617,  882,       0,  792844,       0,  0, 18,  0, }, // Gulf
    {  617,  887,       0,  792512,       0,  0, 12,  0, }, // Guyana
    {  617,  894,  792862,  792881,  792909, 19, 28, 27, }, // Hawaii_Aleutian
    {  617,  910,  409779,  409793,  792936, 14, 22, 18, }, // Hong_Kong
    {  617,  920,  792954,  792963,  792980,  9, 17, 13, }, // Hovd
    {  617,  925,       0,  792993,       0,  0, 22,  0, }, // India
    {  617,  931,       0,  793015,       0,  0, 12,  0, }, // Indian_Ocean
    {  617,  944,       0,  793027,       0,  0, 12,  0, }, // Indochina
    {  617,  954,       0,  793039,       0,  0, 19,  0, }, // Indonesia_Central
    {  617,  972,       0,  793058,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  617,  990,       0,  793078,       0,  0, 20,  0, }, // Indonesia_Western
    {  617, 1008,  793098,  793110,  793129, 12, 19, 18, }, // Iran
    {  617, 1019,  793147,  793158,  793177, 11, 19, 15, }, // Irkutsk
    {  617, 1027,  793192,  793206,  793227, 14, 21, 20, }, // Israel
    {  617, 1034,  793247,  793257,  793275, 10, 18, 18, }, // Japan
    {  617, 1040,  793293,  793319,  793354, 26, 35, 30, }, // Kamchatka
    {  617, 1058,       0,  793384,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  617, 1077,       0,  793405,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  617, 1106,  793426,  793436,  793454, 10, 18, 18, }, // Korea
    {  617, 1112,       0,  793472,       0,  0, 10,  0, }, // Kosrae
    {  617, 1119,  793482,  793498,  793521, 16, 23, 20, }, // Krasnoyarsk
    {  617, 1151,       0,  793541,       0,  0, 14,  0, }, // Kyrgystan
    {  617, 1175,       0,  793555,       0,  0, 17,  0, }, // Line_Islands
    {  617, 1188,  793572,  793586,  793609, 14, 23, 22, }, // Lord_Howe
    {  617, 1204,  793631,  793642,  793662, 11, 20, 15, }, // Magadan
    {  617, 1219,       0,  793677,       0,  0, 11,  0, }, // Malaysia
    {  617, 1228,       0,  793688,       0,  0, 13,  0, }, // Maldives
    {  617, 1237,       0,  793701,       0,  0, 12,  0, }, // Marquesas
    {  617, 1247,       0,  793713,       0,  0, 18,  0, }, // Marshall_Islands
    {  617, 1264,  793731,  793742,  793761, 11, 19, 15, }, // Mauritius
    {  617, 1274,       0,  793776,       0,  0, 10,  0, }, // Mawson
    {  617, 1281,  793786,  793804,  793831, 18, 27, 26, }, // Mexico_Pacific
    {  617, 1296,  793857,  793873,  793897, 16, 24, 20, }, // Mongolia
    {  617, 1305,  793917,  793927,  793946, 10, 19, 14, }, // Moscow
    {  617, 1312,       0,  793960,       0,  0, 12,  0, }, // Myanmar
    {  617, 1320,       0,  793972,       0,  0, 10,  0, }, // Nauru
    {  617, 1326,       0,  793982,       0,  0, 12,  0, }, // Nepal
    {  617, 1332,  793994,  794012,  794038, 18, 26, 22, }, // New_Caledonia
    {  617, 1346,  794060,  794077,  794101, 17, 24, 23, }, // New_Zealand
    {  617, 1358,  411137,  794124,  411183, 19, 28, 27, }, // Newfoundland
    {  617, 1371,       0,  794152,       0,  0,  9,  0, }, // Niue
    {  617, 1376,  794161,  794181,  794208, 20, 27, 25, }, // Norfolk
    {  617, 1384,  794233,  794257,  794288, 24, 31, 27, }, // Noronha
    {  617, 1406,  794315,  794330,  794353, 15, 23, 19, }, // Novosibirsk
    {  617, 1418,  794372,  794382,  794400, 10, 18, 14, }, // Omsk
    {  617, 1428,  794414,  794428,  794449, 14, 21, 27, }, // Pakistan
    {  617, 1437,       0,  794476,       0,  0,  9,  0, }, // Palau
    {  617, 1443,       0,  794485,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  617, 1460,  794503,  794518,  794540, 15, 22, 28, }, // Paraguay
    {  617, 1469,  794568,  794579,  794597, 11, 18, 24, }, // Peru
    {  617, 1474,  794621,  794632,  794651, 11, 19, 15, }, // Philippines
    {  617, 1486,       0,  794666,       0,  0, 18,  0, }, // Phoenix_Islands
    {  617, 1502,  794684,  794710,  794745, 26, 35, 34, }, // Pierre_Miquelon
    {  617, 1518,       0,  794779,       0,  0, 12,  0, }, // Pitcairn
    {  617, 1527,       0,  794791,       0,  0, 11,  0, }, // Ponape
    {  617, 1534,       0,  794802,       0,  0, 14,  0, }, // Pyongyang
    {  617, 1554,       0,  794816,       0,  0, 13,  0, }, // Reunion
    {  617, 1562,       0,  794829,       0,  0, 14,  0, }, // Rothera
    {  617, 1570,  794843,  794854,  794873, 11, 19, 15, }, // Sakhalin
    {  617, 1579,  794888,  794898,  794917, 10, 19, 14, }, // Samara
    {  617, 1596,  794931,  794941,  794959, 10, 18, 18, }, // Samoa
    {  617, 1602,       0,  794977,       0,  0, 11,  0, }, // Seychelles
    {  617, 1624,       0,  794988,       0,  0, 20,  0, }, // Singapore
    {  617, 1634,       0,  795008,       0,  0, 18,  0, }, // Solomon
    {  617, 1642,       0,  795026,       0,  0, 17,  0, }, // South_Georgia
    {  617, 1656,       0,  795043,       0,  0, 14,  0, }, // Suriname
    {  617, 1676,       0,  795057,       0,  0, 10,  0, }, // Syowa
    {  617, 1682,       0,  795067,       0,  0, 11,  0, }, // Tahiti
    {  617, 1689,  795078,  795092,  795115, 14, 23, 21, }, // Taipei
    {  617, 1696,       0,  795136,       0,  0, 15,  0, }, // Tajikistan
    {  617, 1724,       0,  795151,       0,  0, 12,  0, }, // Tokelau
    {  617, 1732,  795163,  795173,  795191, 10, 18, 14, }, // Tonga
    {  617, 1738,       0,  795205,       0,  0,  8,  0, }, // Truk
    {  617, 1750,  795213,  795230,  795254, 17, 24, 30, }, // Turkmenistan
    {  617, 1763,       0,  795284,       0,  0, 10,  0, }, // Tuvalu
    {  617, 1777,  795294,  795309,  795331, 15, 22, 28, }, // Uruguay
    {  617, 1792,  795359,  795374,  795396, 15, 22, 28, }, // Uzbekistan
    {  617, 1803,  795424,  795436,  795456, 12, 20, 16, }, // Vanuatu
    {  617, 1811,       0,  795472,       0,  0, 16,  0, }, // Venezuela
    {  617, 1821,  795488,  795504,  795528, 16, 24, 20, }, // Vladivostok
    {  617, 1833,  795548,  795562,  795585, 14, 23, 18, }, // Volgograd
    {  617, 1843,       0,  795603,       0,  0, 13,  0, }, // Vostok
    {  617, 1850,       0,  795616,       0,  0, 15,  0, }, // Wake
    {  617, 1855,       0,  795631,       0,  0, 20,  0, }, // Wallis
    {  617, 1862,  795651,  795662,  795682, 11, 20, 15, }, // Yakutsk
    {  617, 1870,  795697,  795713,  795738, 16, 25, 20, }, // Yekaterinburg
    {  617, 1892,       0,  412940,       0,  0, 10,  0, }, // Yukon
    {  618,    5,       0,  795758,       0,  0, 14,  0, }, // Afghanistan Urdu/Arabic/India
    {  618,  135,  795772,  795784,  795804, 12, 20, 16, }, // Amazon
    {  618,  232,  795820,  795828,  790649,  8, 16, 16, }, // Arabian
    {  618,  250,  795844,  795863,  795890, 19, 27, 23, }, // Argentina_Western
    {  618,  268,  795913,  795925,  795945, 12, 20, 16, }, // Armenia
    {  618,  374,  795961,  795976,  795999, 15, 23, 19, }, // Azerbaijan
    {  618,  397,  796018,  796032,  796054, 14, 22, 18, }, // Bangladesh
    {  618,  415,       0,  796072,       0,  0, 11,  0, }, // Bhutan
    {  618,  422,       0,  796083,       0,  0, 12,  0, }, // Bolivia
    {  618,  437,  791476,  796095,  791511, 13, 21, 17, }, // Brasilia
    {  618,  495,  796116,  796124,  796140,  8, 16, 12, }, // Chile
    {  618,  534,  791846,  796152,  796172, 12, 20, 16, }, // Colombia
    {  618,  623,  796188,  796205,  796230, 17, 25, 21, }, // Easter
    {  618,  630,       0,  796251,       0,  0, 13,  0, }, // Ecuador
    {  618,  638,  796264,  792167,  792190, 16, 23, 29, }, // Europe_Central
    {  618,  706,  796280,  796301,  796330, 21, 29, 25, }, // Falkland
    {  618,  720,       0,  796355,       0,  0, 16,  0, }, // French_Guiana
    {  618,  757,       0,  796371,       0,  0, 14,  0, }, // Galapagos
    {  618,  775,  795032,  796385,  796404, 11, 19, 15, }, // Georgia
    {  618,  799,       0,  796419,       0,  0, 16,  0, }, // GMT
    {  618,  882,       0,  796435,       0,  0, 17,  0, }, // Gulf
    {  618,  887,       0,  796361,       0,  0, 10,  0, }, // Guyana
    {  618,  925,       0,  796452,       0,  0, 18,  0, }, // India
    {  618, 1008,  796470,  796480,  793129, 10, 18, 18, }, // Iran
    {  618, 1027,  796498,  796510,  793227, 12, 20, 20, }, // Israel
    {  618, 1058,       0,  796530,       0,  0, 19,  0, }, // Kazakhstan_Eastern
    {  618, 1077,       0,  796549,       0,  0, 19,  0, }, // Kazakhstan_Western
    {  618, 1151,       0,  796568,       0,  0, 12,  0, }, // Kyrgystan
    {  618, 1228,       0,  796580,       0,  0, 11,  0, }, // Maldives
    {  618, 1326,       0,  796591,       0,  0, 10,  0, }, // Nepal
    {  618, 1346,  796601,  796616,  794101, 15, 23, 23, }, // New_Zealand
    {  618, 1384,  796639,  796661,  794288, 22, 30, 27, }, // Noronha
    {  618, 1428,  796691,  796703,  796723, 12, 20, 16, }, // Pakistan
    {  618, 1460,  796739,  796752,  796773, 13, 21, 17, }, // Paraguay
    {  618, 1469,  796790,  796799,  796816,  9, 17, 13, }, // Peru
    {  618, 1562,       0,  796829,       0,  0, 12,  0, }, // Rothera
    {  618, 1656,       0,  796841,       0,  0, 12,  0, }, // Suriname
    {  618, 1696,       0,  412284,       0,  0, 13,  0, }, // Tajikistan
    {  618, 1750,  412369,  412384,  796853, 15, 23, 19, }, // Turkmenistan
    {  618, 1777,  796872,  796885,  796906, 13, 21, 17, }, // Uruguay
    {  618, 1792,  796923,  796936,  796957, 13, 21, 17, }, // Uzbekistan
    {  618, 1811,       0,  796974,       0,  0, 14,  0, }, // Venezuela
    {  618, 1843,       0,  796988,       0,  0, 11,  0, }, // Vostok
    {  619,    0,  796999,  797010,  797031, 11, 21, 18, }, // Acre Uyghur/Arabic/China
    {  619,    5,       0,  797049,       0,  0, 17,  0, }, // Afghanistan
    {  619,   17,       0,  797066,       0,  0, 21,  0, }, // Africa_Central
    {  619,   32,       0,  797087,       0,  0, 20,  0, }, // Africa_Eastern
    {  619,   65,       0,  797107,       0,  0, 31,  0, }, // Africa_Southern
    {  619,   81,  797138,  797158,  797188, 20, 30, 27, }, // Africa_Western
    {  619,  107,  797215,  797229,  797253, 14, 24, 21, }, // Alaska
    {  619,  128,  797274,  797287,  797310, 13, 23, 20, }, // Almaty
    {  619,  135,  797330,  797343,  797366, 13, 23, 20, }, // Amazon
    {  619,  142,  797386,  797405,  797434, 19, 29, 26, }, // America_Central
    {  619,  158,  797460,  797478,  797506, 18, 28, 25, }, // America_Eastern
    {  619,  174,  797531,  797540,  797559,  9, 19, 16, }, // America_Mountain
    {  619,  191,  797575,  797592,  797619, 17, 27, 24, }, // America_Pacific
    {  619,  207,  797643,  797656,  797679, 13, 23, 20, }, // Anadyr
    {  619,  219,  797699,  797711,  797733, 12, 22, 19, }, // Aqtau
    {  619,  225,  797752,  797765,  797788, 13, 23, 20, }, // Aqtobe
    {  619,  232,  797808,  797819,  797840, 11, 21, 18, }, // Arabian
    {  619,  240,  797858,  797874,  797900, 16, 26, 23, }, // Argentina
    {  619,  250,  797923,  797946,  797979, 23, 33, 30, }, // Argentina_Western
    {  619,  268,  798009,  798024,  798049, 15, 25, 22, }, // Armenia
    {  619,  286,  798071,  798093,  798125, 22, 32, 29, }, // Atlantic
    {  619,  295,  798154,  798185,  798226, 31, 41, 38, }, // Australia_Central
    {  619,  313,  798264,  798302,  798351, 38, 49, 45, }, // Australia_CentralWestern
    {  619,  338,  798396,  798426,  798466, 30, 40, 37, }, // Australia_Eastern
    {  619,  356,  798503,  798533,  798573, 30, 40, 37, }, // Australia_Western
    {  619,  374,  798610,  798627,  798654, 17, 27, 24, }, // Azerbaijan
    {  619,  385,  798678,  798689,  798710, 11, 21, 18, }, // Azores
    {  619,  397,  798728,  798742,  798766, 14, 24, 21, }, // Bangladesh
    {  619,  415,       0,  798787,       0,  0, 11,  0, }, // Bhutan
    {  619,  422,       0,  798798,       0,  0, 14,  0, }, // Bolivia
    {  619,  437,  798812,  798828,  798854, 16, 26, 23, }, // Brasilia
    {  619,  454,       0,  798877,       0,  0, 24,  0, }, // Brunei
    {  619,  461,  798901,  798919,  798947, 18, 28, 25, }, // Cape_Verde
    {  619,  472,       0,  798972,       0,  0, 11,  0, }, // Casey
    {  619,  478,       0,  798983,       0,  0, 23,  0, }, // Chamorro
    {  619,  487,  799006,  799017,  799038, 11, 21, 18, }, // Chatham
    {  619,  495,  799056,  799066,  799086, 10, 20, 17, }, // Chile
    {  619,  501,  799103,  799114,  799135, 11, 21, 18, }, // China
    {  619,  507,  799153,  799168,  799193, 15, 25, 22, }, // Choibalsan
    {  619,  518,       0,  799215,       0,  0, 22,  0, }, // Christmas
    {  619,  528,       0,  799237,       0,  0, 18,  0, }, // Cocos
    {  619,  534,  799255,  799270,  799295, 15, 25, 22, }, // Colombia
    {  619,  543,  799317,  799336,  799365, 19, 29, 32, }, // Cook
    {  619,  548,  799397,  799407,  799427, 10, 20, 17, }, // Cuba
    {  619,  559,       0,  799444,       0,  0, 11,  0, }, // Davis
    {  619,  575,       0,  799455,       0,  0, 19,  0, }, // DumontDUrville
    {  619,  612,       0,  799474,       0,  0, 18,  0, }, // East_Timor
    {  619,  623,  799492,  799511,  799542, 19, 31, 26, }, // Easter
    {  619,  630,       0,  799568,       0,  0, 14,  0, }, // Ecuador
    {  619,  638,  799582,  799603,  799634, 21, 31, 28, }, // Europe_Central
    {  619,  653,  799662,  799682,  799712, 20, 30, 27, }, // Europe_Eastern
    {  619,  691,  799739,  799759,  799789, 20, 30, 27, }, // Europe_Western
    {  619,  706,  799816,  799840,  799874, 24, 34, 31, }, // Falkland
    {  619,  715,  799905,  799915,  799935, 10, 20, 17, }, // Fiji
    {  619,  720,       0,  799952,       0,  0, 35,  0, }, // French_Guiana
    {  619,  734,       0,  799987,       0,  0, 49,  0, }, // French_Southern
    {  619,  757,       0,  800036,       0,  0, 15,  0, }, // Galapagos
    {  619,  767,       0,  800051,       0,  0, 14,  0, }, // Gambier
    {  619,  775,  800065,  800079,  800103, 14, 24, 21, }, // Georgia
    {  619,  783,       0,  800124,       0,  0, 23,  0, }, // Gilbert_Islands
    {  619,  799,       0,  800147,       0,  0, 14,  0, }, // GMT
    {  619,  841,  800161,  800183,  800215, 22, 32, 29, }, // Greenland_Eastern
    {  619,  859,  800244,  800266,  800298, 22, 32, 29, }, // Greenland_Western
    {  619,  877,       0,  800327,       0,  0, 21,  0, }, // Guam
    {  619,  882,       0,  800348,       0,  0, 20,  0, }, // Gulf
    {  619,  887,       0,  799973,       0,  0, 14,  0, }, // Guyana
    {  619,  894,  800368,  800387,  800416, 19, 29, 26, }, // Hawaii_Aleutian
    {  619,  910,  800442,  800455,  800478, 13, 23, 20, }, // Hong_Kong
    {  619,  920,  800498,  800508,  800528, 10, 20, 17, }, // Hovd
    {  619,  925,       0,  800545,       0,  0, 25,  0, }, // India
    {  619,  931,       0,  800570,       0,  0, 18,  0, }, // Indian_Ocean
    {  619,  944,       0,  800588,       0,  0, 16,  0, }, // Indochina
    {  619,  954,       0,  800604,       0,  0, 25,  0, }, // Indonesia_Central
    {  619,  972,       0,  800629,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  619,  990,       0,  800653,       0,  0, 24,  0, }, // Indonesia_Western
    {  619, 1008,  800677,  800688,  800709, 11, 21, 18, }, // Iran
    {  619, 1019,  800727,  800741,  800765, 14, 24, 21, }, // Irkutsk
    {  619, 1027,  800786,  800803,  800830, 17, 27, 24, }, // Israel
    {  619, 1034,  800854,  800868,  800892, 14, 24, 21, }, // Japan
    {  619, 1040,  800913,  800943,  800983, 30, 40, 37, }, // Kamchatka
    {  619, 1058,       0,  801020,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  619, 1077,       0,  801043,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  619, 1106,  801066,  801078,  801100, 12, 22, 19, }, // Korea
    {  619, 1112,       0,  801119,       0,  0, 13,  0, }, // Kosrae
    {  619, 1119,  801132,  801150,  801178, 18, 28, 25, }, // Krasnoyarsk
    {  619, 1151,       0,  801203,       0,  0, 17,  0, }, // Kyrgystan
    {  619, 1161,       0,  801220,       0,  0, 16,  0, }, // Lanka
    {  619, 1175,       0,  801236,       0,  0, 20,  0, }, // Line_Islands
    {  619, 1188,  801256,  801270,  801294, 14, 24, 21, }, // Lord_Howe
    {  619, 1198,  801315,  801327,  801349, 12, 22, 19, }, // Macau
    {  619, 1204,  801368,  801381,  801404, 13, 23, 20, }, // Magadan
    {  619, 1219,       0,  801424,       0,  0, 15,  0, }, // Malaysia
    {  619, 1228,       0,  801439,       0,  0, 13,  0, }, // Maldives
    {  619, 1237,       0,  801452,       0,  0, 12,  0, }, // Marquesas
    {  619, 1247,       0,  801464,       0,  0, 22,  0, }, // Marshall_Islands
    {  619, 1264,  801486,  801502,  801528, 16, 26, 23, }, // Mauritius
    {  619, 1274,       0,  801551,       0,  0, 12,  0, }, // Mawson
    {  619, 1281,  801563,  801588,  801623, 25, 35, 32, }, // Mexico_Pacific
    {  619, 1296,  801655,  801671,  801697, 16, 26, 23, }, // Mongolia
    {  619, 1305,  801720,  801732,  801754, 12, 22, 19, }, // Moscow
    {  619, 1312,       0,  801773,       0,  0, 11,  0, }, // Myanmar
    {  619, 1320,       0,  801784,       0,  0, 11,  0, }, // Nauru
    {  619, 1326,       0,  801795,       0,  0, 11,  0, }, // Nepal
    {  619, 1332,  801806,  801827,  801858, 21, 31, 28, }, // New_Caledonia
    {  619, 1346,  801886,  801906,  801936, 20, 30, 27, }, // New_Zealand
    {  619, 1358,  801963,  801981,  802009, 18, 28, 25, }, // Newfoundland
    {  619, 1371,       0,  802034,       0,  0, 12,  0, }, // Niue
    {  619, 1376,  802046,  802069,  802102, 23, 33, 30, }, // Norfolk
    {  619, 1384,  802132,  802154,  802186, 22, 32, 29, }, // Noronha
    {  619, 1392,       0,  802215,       0,  0, 32,  0, }, // North_Mariana
    {  619, 1406,  802247,  802264,  802291, 17, 27, 24, }, // Novosibirsk
    {  619, 1418,  802315,  802326,  802347, 11, 21, 18, }, // Omsk
    {  619, 1428,  802365,  802379,  802403, 14, 24, 21, }, // Pakistan
    {  619, 1437,       0,  802424,       0,  0, 11,  0, }, // Palau
    {  619, 1443,       0,  802435,       0,  0, 28,  0, }, // Papua_New_Guinea
    {  619, 1460,  802463,  802477,  802501, 14, 24, 21, }, // Paraguay
    {  619, 1469,  802522,  802532,  802552, 10, 20, 17, }, // Peru
    {  619, 1474,  802569,  802583,  802607, 14, 24, 21, }, // Philippines
    {  619, 1486,       0,  802628,       0,  0, 22,  0, }, // Phoenix_Islands
    {  619, 1502,  802650,  802679,  802718, 29, 39, 36, }, // Pierre_Miquelon
    {  619, 1518,       0,  802754,       0,  0, 13,  0, }, // Pitcairn
    {  619, 1527,       0,  802767,       0,  0, 12,  0, }, // Ponape
    {  619, 1544,  802779,  802795,  802821, 16, 26, 23, }, // Qyzylorda
    {  619, 1554,       0,  802844,       0,  0, 15,  0, }, // Reunion
    {  619, 1562,       0,  802859,       0,  0, 12,  0, }, // Rothera
    {  619, 1570,  802871,  802884,  802907, 13, 23, 20, }, // Sakhalin
    {  619, 1579,  802927,  802939,  802961, 12, 22, 19, }, // Samara
    {  619, 1596,  802980,  802992,  803014, 12, 22, 22, }, // Samoa
    {  619, 1602,       0,  803036,       0,  0, 12,  0, }, // Seychelles
    {  619, 1624,       0,  803048,       0,  0, 14,  0, }, // Singapore
    {  619, 1634,       0,  803062,       0,  0, 23,  0, }, // Solomon
    {  619, 1642,       0,  803085,       0,  0, 21,  0, }, // South_Georgia
    {  619, 1656,       0,  803106,       0,  0, 13,  0, }, // Suriname
    {  619, 1676,       0,  803119,       0,  0, 10,  0, }, // Syowa
    {  619, 1682,       0,  803129,       0,  0, 11,  0, }, // Tahiti
    {  619, 1689,  803140,  803152,  803174, 12, 22, 19, }, // Taipei
    {  619, 1696,       0,  803193,       0,  0, 16,  0, }, // Tajikistan
    {  619, 1724,       0,  803209,       0,  0, 13,  0, }, // Tokelau
    {  619, 1732,  803222,  803233,  803254, 11, 21, 18, }, // Tonga
    {  619, 1738,       0,  803272,       0,  0,  9,  0, }, // Truk
    {  619, 1750,  803281,  803299,  803327, 18, 28, 25, }, // Turkmenistan
    {  619, 1763,       0,  803352,       0,  0, 12,  0, }, // Tuvalu
    {  619, 1777,  803364,  803378,  803402, 14, 24, 21, }, // Uruguay
    {  619, 1792,  803423,  803440,  803467, 17, 27, 24, }, // Uzbekistan
    {  619, 1803,  803491,  803505,  803529, 14, 24, 21, }, // Vanuatu
    {  619, 1811,       0,  803550,       0,  0, 16,  0, }, // Venezuela
    {  619, 1821,  803566,  803584,  803612, 18, 28, 25, }, // Vladivostok
    {  619, 1833,  803637,  803652,  803677, 15, 25, 22, }, // Volgograd
    {  619, 1843,       0,  803572,       0,  0, 12,  0, }, // Vostok
    {  619, 1850,       0,  803699,       0,  0, 17,  0, }, // Wake
    {  619, 1855,       0,  803716,       0,  0, 22,  0, }, // Wallis
    {  619, 1862,  803738,  803751,  803774, 13, 23, 20, }, // Yakutsk
    {  619, 1870,  803794,  803813,  803842, 19, 29, 26, }, // Yekaterinburg
    {  620,    5,       0,  803868,       0,  0, 17,  0, }, // Afghanistan Uzbek/Latin/Uzbekistan
    {  620,   17,       0,  803885,       0,  0, 21,  0, }, // Africa_Central
    {  620,   32,       0,  803906,       0,  0, 20,  0, }, // Africa_Eastern
    {  620,   65,       0,  803926,       0,  0, 29,  0, }, // Africa_Southern
    {  620,   81,  803955,  803975,  804004, 20, 29, 26, }, // Africa_Western
    {  620,  107,  804030,  804043,  804065, 13, 22, 19, }, // Alaska
    {  620,  135,  804084,  804098,  804121, 14, 23, 20, }, // Amazon
    {  620,  142,  804141,  804163,  804194, 22, 31, 28, }, // America_Central
    {  620,  158,  804222,  804243,  804273, 21, 30, 27, }, // America_Eastern
    {  620,  174,  804300,  804317,  804343, 17, 26, 23, }, // America_Mountain
    {  620,  191,  804366,  804384,  804411, 18, 27, 24, }, // America_Pacific
    {  620,  214,  804435,  804445,  804464, 10, 19, 16, }, // Apia
    {  620,  232,  804480,  804504,  804537, 24, 33, 30, }, // Arabian
    {  620,  240,  804567,  804582,  804606, 15, 24, 21, }, // Argentina
    {  620,  250,  804627,  804650,  804682, 23, 32, 29, }, // Argentina_Western
    {  620,  268,  804711,  804727,  804752, 16, 25, 22, }, // Armenia
    {  620,  286,  804774,  804789,  804813, 15, 24, 21, }, // Atlantic
    {  620,  295,  804834,  804859,  804893, 25, 34, 31, }, // Australia_Central
    {  620,  313,  804924,  804957,  804999, 33, 42, 39, }, // Australia_CentralWestern
    {  620,  338,  805038,  805062,  805095, 24, 33, 30, }, // Australia_Eastern
    {  620,  356,  805125,  805149,  805182, 24, 33, 30, }, // Australia_Western
    {  620,  374,  805212,  805228,  805253, 16, 25, 22, }, // Azerbaijan
    {  620,  385,  805275,  805294,  805322, 19, 28, 25, }, // Azores
    {  620,  397,  805347,  805363,  805388, 16, 25, 22, }, // Bangladesh
    {  620,  415,       0,  805410,       0,  0, 11,  0, }, // Bhutan
    {  620,  422,       0,  805421,       0,  0, 14,  0, }, // Bolivia
    {  620,  437,  805435,  805450,  805474, 15, 24, 21, }, // Brasilia
    {  620,  454,       0,  805495,       0,  0, 23,  0, }, // Brunei
    {  620,  461,  805518,  805534,  805559, 16, 25, 22, }, // Cape_Verde
    {  620,  478,       0,  805581,       0,  0, 23,  0, }, // Chamorro
    {  620,  487,  805604,  805616,  805637, 12, 21, 18, }, // Chatham
    {  620,  495,  805655,  805666,  805686, 11, 20, 17, }, // Chile
    {  620,  501,  805703,  805714,  805734, 11, 20, 17, }, // China
    {  620,  507,  805751,  805767,  805792, 16, 25, 22, }, // Choibalsan
    {  620,  518,       0,  805814,       0,  0, 21,  0, }, // Christmas
    {  620,  528,       0,  805835,       0,  0, 20,  0, }, // Cocos
    {  620,  534,  805855,  805870,  805894, 15, 24, 21, }, // Colombia
    {  620,  543,  805915,  805933,  805960, 18, 27, 30, }, // Cook
    {  620,  548,  805990,  806000,  806019, 10, 19, 16, }, // Cuba
    {  620,  559,       0,  806035,       0,  0, 12,  0, }, // Davis
    {  620,  575,       0,  806047,       0,  0, 21,  0, }, // DumontDUrville
    {  620,  612,       0,  806068,       0,  0, 19,  0, }, // East_Timor
    {  620,  623,  806087,  806104,  806130, 17, 26, 23, }, // Easter
    {  620,  630,       0,  806153,       0,  0, 13,  0, }, // Ecuador
    {  620,  638,  806166,  806188,  806219, 22, 31, 28, }, // Europe_Central
    {  620,  653,  806247,  806268,  806298, 21, 30, 27, }, // Europe_Eastern
    {  620,  668,       0,  806325,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  620,  691,  806351,  806372,  806402, 21, 30, 27, }, // Europe_Western
    {  620,  706,  806429,  806452,  806484, 23, 32, 29, }, // Falkland
    {  620,  715,  806513,  806523,  806542, 10, 19, 16, }, // Fiji
    {  620,  720,       0,  806558,       0,  0, 22,  0, }, // French_Guiana
    {  620,  734,       0,  806580,       0,  0, 45,  0, }, // French_Southern
    {  620,  757,       0,  806625,       0,  0, 15,  0, }, // Galapagos
    {  620,  767,       0,  806640,       0,  0, 12,  0, }, // Gambier
    {  620,  775,  806652,  806665,  806687, 13, 22, 19, }, // Georgia
    {  620,  783,       0,  806706,       0,  0, 22,  0, }, // Gilbert_Islands
    {  620,  799,       0,  806728,       0,  0, 23,  0, }, // GMT
    {  620,  841,  806751,  806776,  806810, 25, 34, 31, }, // Greenland_Eastern
    {  620,  859,  806841,  806866,  806900, 25, 34, 31, }, // Greenland_Western
    {  620,  882,       0,  806931,       0,  0, 28,  0, }, // Gulf
    {  620,  887,       0,  806959,       0,  0, 12,  0, }, // Guyana
    {  620,  894,  806971,  806989,  807016, 18, 27, 24, }, // Hawaii_Aleutian
    {  620,  910,  807040,  807053,  807075, 13, 22, 19, }, // Hong_Kong
    {  620,  920,  807094,  807104,  807123, 10, 19, 16, }, // Hovd
    {  620,  925,       0,  807139,       0,  0, 24,  0, }, // India
    {  620,  931,       0,  807163,       0,  0, 17,  0, }, // Indian_Ocean
    {  620,  944,       0,  807180,       0,  0, 16,  0, }, // Indochina
    {  620,  954,       0,  807196,       0,  0, 25,  0, }, // Indonesia_Central
    {  620,  972,       0,  807221,       0,  0, 24,  0, }, // Indonesia_Eastern
    {  620,  990,       0,  807245,       0,  0, 24,  0, }, // Indonesia_Western
    {  620, 1008,  807269,  807279,  807298, 10, 19, 16, }, // Iran
    {  620, 1019,  807314,  807327,  807349, 13, 22, 19, }, // Irkutsk
    {  620, 1027,  807368,  807380,  807401, 12, 21, 18, }, // Israel
    {  620, 1034,  807419,  807433,  807456, 14, 23, 20, }, // Japan
    {  620, 1058,       0,  807476,       0,  0, 25,  0, }, // Kazakhstan_Eastern
    {  620, 1077,       0,  807501,       0,  0, 25,  0, }, // Kazakhstan_Western
    {  620, 1106,  807526,  807538,  807559, 12, 21, 18, }, // Korea
    {  620, 1112,       0,  807577,       0,  0, 12,  0, }, // Kosrae
    {  620, 1119,  807589,  807606,  807632, 17, 26, 23, }, // Krasnoyarsk
    {  620, 1151,       0,  807655,       0,  0, 18,  0, }, // Kyrgystan
    {  620, 1175,       0,  807673,       0,  0, 19,  0, }, // Line_Islands
    {  620, 1188,  807692,  807706,  807729, 14, 23, 20, }, // Lord_Howe
    {  620, 1204,  807749,  807762,  807784, 13, 22, 19, }, // Magadan
    {  620, 1219,       0,  807803,       0,  0, 15,  0, }, // Malaysia
    {  620, 1228,       0,  807818,       0,  0, 21,  0, }, // Maldives
    {  620, 1237,       0,  807839,       0,  0, 21,  0, }, // Marquesas
    {  620, 1247,       0,  807860,       0,  0, 23,  0, }, // Marshall_Islands
    {  620, 1264,  807883,  807897,  807920, 14, 23, 20, }, // Mauritius
    {  620, 1274,       0,  807940,       0,  0, 12,  0, }, // Mawson
    {  620, 1281,  807952,  807978,  808013, 26, 35, 32, }, // Mexico_Pacific
    {  620, 1296,  808045,  808061,  808086, 16, 25, 22, }, // Mongolia
    {  620, 1305,  808108,  808120,  808141, 12, 21, 18, }, // Moscow
    {  620, 1312,       0,  808159,       0,  0, 12,  0, }, // Myanmar
    {  620, 1320,       0,  808171,       0,  0, 11,  0, }, // Nauru
    {  620, 1326,       0,  808182,       0,  0, 11,  0, }, // Nepal
    {  620, 1332,  808193,  808215,  808246, 22, 31, 28, }, // New_Caledonia
    {  620, 1346,  808274,  808295,  808325, 21, 30, 27, }, // New_Zealand
    {  620, 1358,  808352,  808370,  808397, 18, 27, 24, }, // Newfoundland
    {  620, 1371,       0,  808421,       0,  0, 11,  0, }, // Niue
    {  620, 1376,  808432,  808451,  808479, 19, 28, 25, }, // Norfolk
    {  620, 1384,  808504,  808529,  808563, 25, 34, 31, }, // Noronha
    {  620, 1406,  808594,  808611,  808637, 17, 26, 23, }, // Novosibirsk
    {  620, 1418,  808660,  808670,  808689, 10, 19, 16, }, // Omsk
    {  620, 1428,  808705,  808719,  808742, 14, 23, 20, }, // Pakistan
    {  620, 1437,       0,  808762,       0,  0, 11,  0, }, // Palau
    {  620, 1443,       0,  808773,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  620, 1460,  808798,  808812,  808835, 14, 23, 20, }, // Paraguay
    {  620, 1469,  808855,  808865,  808884, 10, 19, 16, }, // Peru
    {  620, 1474,  808900,  808914,  808937, 14, 23, 20, }, // Philippines
    {  620, 1486,       0,  808957,       0,  0, 21,  0, }, // Phoenix_Islands
    {  620, 1502,  808978,  809003,  809037, 25, 34, 31, }, // Pierre_Miquelon
    {  620, 1518,       0,  809068,       0,  0, 13,  0, }, // Pitcairn
    {  620, 1527,       0,  809081,       0,  0, 12,  0, }, // Ponape
    {  620, 1534,       0,  809093,       0,  0, 13,  0, }, // Pyongyang
    {  620, 1554,       0,  809106,       0,  0, 14,  0, }, // Reunion
    {  620, 1562,       0,  809120,       0,  0, 12,  0, }, // Rothera
    {  620, 1570,  809132,  809145,  809167, 13, 22, 19, }, // Sakhalin
    {  620, 1596,  809186,  809197,  809217, 11, 20, 17, }, // Samoa
    {  620, 1602,       0,  809234,       0,  0, 22,  0, }, // Seychelles
    {  620, 1624,       0,  809256,       0,  0, 14,  0, }, // Singapore
    {  620, 1634,       0,  809270,       0,  0, 22,  0, }, // Solomon
    {  620, 1642,       0,  809292,       0,  0, 22,  0, }, // South_Georgia
    {  620, 1656,       0,  809314,       0,  0, 13,  0, }, // Suriname
    {  620, 1676,       0,  809327,       0,  0, 11,  0, }, // Syowa
    {  620, 1682,       0,  809338,       0,  0, 11,  0, }, // Tahiti
    {  620, 1689,  809349,  809361,  809382, 12, 21, 18, }, // Taipei
    {  620, 1696,       0,  809400,       0,  0, 16,  0, }, // Tajikistan
    {  620, 1724,       0,  809416,       0,  0, 13,  0, }, // Tokelau
    {  620, 1732,  809429,  809440,  809460, 11, 20, 17, }, // Tonga
    {  620, 1738,       0,  809477,       0,  0, 11,  0, }, // Truk
    {  620, 1750,  809488,  809506,  809533, 18, 27, 24, }, // Turkmenistan
    {  620, 1763,       0,  809557,       0,  0, 12,  0, }, // Tuvalu
    {  620, 1777,  809569,  809582,  809604, 13, 22, 19, }, // Uruguay
    {  620, 1792,  809623,  809640,  809666, 17, 26, 23, }, // Uzbekistan
    {  620, 1803,  809689,  809702,  809724, 13, 22, 19, }, // Vanuatu
    {  620, 1811,       0,  809743,       0,  0, 15,  0, }, // Venezuela
    {  620, 1821,  809758,  809775,  809801, 17, 26, 23, }, // Vladivostok
    {  620, 1833,  809824,  809839,  809863, 15, 24, 21, }, // Volgograd
    {  620, 1843,       0,  809884,       0,  0, 12,  0, }, // Vostok
    {  620, 1850,       0,  809896,       0,  0, 16,  0, }, // Wake
    {  620, 1855,       0,  809912,       0,  0, 22,  0, }, // Wallis
    {  620, 1862,  809934,  809947,  809969, 13, 22, 19, }, // Yakutsk
    {  620, 1870,  809988,  810007,  810035, 19, 28, 25, }, // Yekaterinburg
    {  620, 1892,       0,  810060,       0,  0, 11,  0, }, // Yukon
    {  621,    5,       0,  810071,       0,  0, 14,  0, }, // Afghanistan Uzbek/Arabic/Afghanistan
    {  622,    5,       0,  810085,       0,  0, 16,  0, }, // Afghanistan Uzbek/Cyrillic/Uzbekistan
    {  622,   17,       0,  810101,       0,  0, 21,  0, }, // Africa_Central
    {  622,   32,       0,  810122,       0,  0, 19,  0, }, // Africa_Eastern
    {  622,   65,       0,  810141,       0,  0, 20,  0, }, // Africa_Southern
    {  622,   81,  810161,  810180,  810208, 19, 28, 24, }, // Africa_Western
    {  622,  107,  810232,  810244,  810265, 12, 21, 21, }, // Alaska
    {  622,  135,  810286,  810300,  810323, 14, 23, 19, }, // Amazon
    {  622,  142,  810342,  810357,  810396, 15, 39, 39, }, // America_Central
    {  622,  158,  810435,  810463,  810500, 28, 37, 37, }, // America_Eastern
    {  622,  174,  810537,  810562,  810596, 25, 34, 34, }, // America_Mountain
    {  622,  191,  810630,  810663,  810705, 33, 42, 42, }, // America_Pacific
    {  622,  232,  810747,  810762,  810786, 15, 24, 24, }, // Arabian
    {  622,  240,  810810,  810825,  810849, 15, 24, 20, }, // Argentina
    {  622,  250,  810869,  810891,  810922, 22, 31, 27, }, // Argentina_Western
    {  622,  268,  810949,  810965,  810990, 16, 25, 21, }, // Armenia
    {  622,  286,  811011,  811026,  811050, 15, 24, 24, }, // Atlantic
    {  622,  295,  811074,  811098,  811131, 24, 33, 33, }, // Australia_Central
    {  622,  313,  811164,  811195,  811235, 31, 40, 40, }, // Australia_CentralWestern
    {  622,  338,  811275,  811297,  811328, 22, 31, 31, }, // Australia_Eastern
    {  622,  356,  811359,  811381,  811412, 22, 31, 31, }, // Australia_Western
    {  622,  374,  811443,  811459,  811484, 16, 25, 21, }, // Azerbaijan
    {  622,  385,  811505,  811515,  811534, 10, 19, 15, }, // Azores
    {  622,  397,  811549,  811564,  811588, 15, 24, 20, }, // Bangladesh
    {  622,  415,       0,  811608,       0,  0, 11,  0, }, // Bhutan
    {  622,  422,       0,  811619,       0,  0, 13,  0, }, // Bolivia
    {  622,  437,  811632,  811646,  811669, 14, 23, 19, }, // Brasilia
    {  622,  454,       0,  811688,       0,  0, 23,  0, }, // Brunei
    {  622,  461,  811711,  811727,  811752, 16, 25, 21, }, // Cape_Verde
    {  622,  478,       0,  811773,       0,  0, 13,  0, }, // Chamorro
    {  622,  487,  811786,  811798,  811819, 12, 21, 21, }, // Chatham
    {  622,  495,  811840,  811850,  811869, 10, 19, 15, }, // Chile
    {  622,  501,  811884,  811895,  811915, 11, 20, 20, }, // China
    {  622,  507,  811935,  811950,  811974, 15, 24, 20, }, // Choibalsan
    {  622,  518,       0,  811994,       0,  0, 21,  0, }, // Christmas
    {  622,  528,       0,  812015,       0,  0, 20,  0, }, // Cocos
    {  622,  534,  812035,  812049,  812072, 14, 23, 19, }, // Colombia
    {  622,  543,  812091,  812109,  812136, 18, 27, 28, }, // Cook
    {  622,  548,  812164,  812174,  812193, 10, 19, 19, }, // Cuba
    {  622,  559,       0,  812212,       0,  0, 11,  0, }, // Davis
    {  622,  575,       0,  812223,       0,  0, 20,  0, }, // DumontDUrville
    {  622,  612,       0,  812243,       0,  0, 18,  0, }, // East_Timor
    {  622,  623,  812261,  812278,  812304, 17, 26, 22, }, // Easter
    {  622,  630,       0,  812326,       0,  0, 13,  0, }, // Ecuador
    {  622,  638,  812339,  812360,  812390, 21, 30, 26, }, // Europe_Central
    {  622,  653,  812416,  812435,  812463, 19, 28, 24, }, // Europe_Eastern
    {  622,  691,  812487,  812506,  812534, 19, 28, 24, }, // Europe_Western
    {  622,  706,  812558,  812582,  812615, 24, 33, 29, }, // Falkland
    {  622,  715,  812644,  812654,  812673, 10, 19, 15, }, // Fiji
    {  622,  720,       0,  812688,       0,  0, 22,  0, }, // French_Guiana
    {  622,  734,       0,  812710,       0,  0, 35,  0, }, // French_Southern
    {  622,  757,       0,  812745,       0,  0, 15,  0, }, // Galapagos
    {  622,  767,       0,  812760,       0,  0, 13,  0, }, // Gambier
    {  622,  775,  812773,  812785,  812806, 12, 21, 17, }, // Georgia
    {  622,  783,       0,  812823,       0,  0, 22,  0, }, // Gilbert_Islands
    {  622,  799,       0,  812845,       0,  0, 13,  0, }, // GMT
    {  622,  841,  812858,  812881,  812913, 23, 32, 28, }, // Greenland_Eastern
    {  622,  859,  812941,  812964,  812996, 23, 32, 28, }, // Greenland_Western
    {  622,  882,       0,  813024,       0,  0, 12,  0, }, // Gulf
    {  622,  887,       0,  813036,       0,  0, 12,  0, }, // Guyana
    {  622,  894,  813048,  813066,  813093, 18, 27, 27, }, // Hawaii_Aleutian
    {  622,  910,  813120,  813133,  813155, 13, 22, 18, }, // Hong_Kong
    {  622,  920,  813173,  813183,  813202, 10, 19, 15, }, // Hovd
    {  622,  925,       0,  813217,       0,  0, 15,  0, }, // India
    {  622,  931,       0,  813232,       0,  0, 17,  0, }, // Indian_Ocean
    {  622,  944,       0,  813249,       0,  0, 16,  0, }, // Indochina
    {  622,  954,       0,  813265,       0,  0, 24,  0, }, // Indonesia_Central
    {  622,  972,       0,  813289,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  622,  990,       0,  813311,       0,  0, 22,  0, }, // Indonesia_Western
    {  622, 1008,  813333,  813343,  813362, 10, 19, 19, }, // Iran
    {  622, 1019,  813381,  813394,  813416, 13, 22, 18, }, // Irkutsk
    {  622, 1027,  813434,  813446,  813467, 12, 21, 21, }, // Israel
    {  622, 1034,  813488,  813500,  813521, 12, 21, 21, }, // Japan
    {  622, 1058,       0,  813542,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  622, 1077,       0,  813565,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  622, 1106,  813588,  813599,  813619, 11, 20, 20, }, // Korea
    {  622, 1112,       0,  813639,       0,  0, 12,  0, }, // Kosrae
    {  622, 1119,  813651,  813667,  813692, 16, 25, 21, }, // Krasnoyarsk
    {  622, 1151,       0,  813713,       0,  0, 17,  0, }, // Kyrgystan
    {  622, 1175,       0,  813730,       0,  0, 19,  0, }, // Line_Islands
    {  622, 1188,  813749,  813764,  813788, 15, 24, 24, }, // Lord_Howe
    {  622, 1204,  813812,  813825,  813847, 13, 22, 18, }, // Magadan
    {  622, 1219,       0,  813865,       0,  0, 14,  0, }, // Malaysia
    {  622, 1228,       0,  813879,       0,  0, 15,  0, }, // Maldives
    {  622, 1237,       0,  813894,       0,  0, 14,  0, }, // Marquesas
    {  622, 1247,       0,  813908,       0,  0, 22,  0, }, // Marshall_Islands
    {  622, 1264,  813930,  813944,  813967, 14, 23, 19, }, // Mauritius
    {  622, 1274,       0,  813986,       0,  0, 13,  0, }, // Mawson
    {  622, 1296,  813999,  814015,  814040, 16, 25, 21, }, // Mongolia
    {  622, 1305,  814061,  814073,  814094, 12, 21, 17, }, // Moscow
    {  622, 1312,       0,  814111,       0,  0, 12,  0, }, // Myanmar
    {  622, 1320,       0,  814123,       0,  0, 11,  0, }, // Nauru
    {  622, 1326,       0,  814134,       0,  0, 11,  0, }, // Nepal
    {  622, 1332,  814145,  814165,  814194, 20, 29, 25, }, // New_Caledonia
    {  622, 1346,  814219,  814238,  814266, 19, 28, 28, }, // New_Zealand
    {  622, 1358,  814294,  814312,  814339, 18, 27, 27, }, // Newfoundland
    {  622, 1371,       0,  814366,       0,  0, 10,  0, }, // Niue
    {  622, 1376,  814376,  814395,  814423, 19, 28, 24, }, // Norfolk
    {  622, 1384,  814447,  814472,  814506, 25, 34, 30, }, // Noronha
    {  622, 1406,  814536,  814553,  814579, 17, 26, 22, }, // Novosibirsk
    {  622, 1418,  814601,  814611,  814630, 10, 19, 15, }, // Omsk
    {  622, 1428,  814645,  814659,  814682, 14, 23, 19, }, // Pakistan
    {  622, 1437,       0,  814701,       0,  0, 11,  0, }, // Palau
    {  622, 1443,       0,  814712,       0,  0, 23,  0, }, // Papua_New_Guinea
    {  622, 1460,  814735,  814749,  814772, 14, 23, 19, }, // Paraguay
    {  622, 1469,  814791,  814801,  814820, 10, 19, 15, }, // Peru
    {  622, 1474,  814835,  814849,  814872, 14, 23, 19, }, // Philippines
    {  622, 1486,       0,  814891,       0,  0, 21,  0, }, // Phoenix_Islands
    {  622, 1502,  814912,  814938,  814973, 26, 35, 35, }, // Pierre_Miquelon
    {  622, 1518,       0,  815008,       0,  0, 13,  0, }, // Pitcairn
    {  622, 1527,       0,  815021,       0,  0, 12,  0, }, // Ponape
    {  622, 1554,       0,  815033,       0,  0, 13,  0, }, // Reunion
    {  622, 1562,       0,  815046,       0,  0, 12,  0, }, // Rothera
    {  622, 1570,  815058,  815071,  815093, 13, 22, 18, }, // Sakhalin
    {  622, 1596,  815111,  815122,  815142, 11, 20, 20, }, // Samoa
    {  622, 1602,       0,  815162,       0,  0, 21,  0, }, // Seychelles
    {  622, 1624,       0,  815183,       0,  0, 14,  0, }, // Singapore
    {  622, 1634,       0,  815197,       0,  0, 22,  0, }, // Solomon
    {  622, 1642,       0,  815219,       0,  0, 22,  0, }, // South_Georgia
    {  622, 1656,       0,  815241,       0,  0, 13,  0, }, // Suriname
    {  622, 1676,       0,  815254,       0,  0, 11,  0, }, // Syowa
    {  622, 1682,       0,  815265,       0,  0, 11,  0, }, // Tahiti
    {  622, 1689,  815276,  815288,  815309, 12, 21, 21, }, // Taipei
    {  622, 1696,       0,  815330,       0,  0, 16,  0, }, // Tajikistan
    {  622, 1724,       0,  815346,       0,  0, 13,  0, }, // Tokelau
    {  622, 1732,  815359,  815370,  815390, 11, 20, 16, }, // Tonga
    {  622, 1738,       0,  815406,       0,  0, 10,  0, }, // Truk
    {  622, 1750,  815416,  815434,  815461, 18, 27, 23, }, // Turkmenistan
    {  622, 1763,       0,  815484,       0,  0, 12,  0, }, // Tuvalu
    {  622, 1777,  815496,  815509,  815531, 13, 22, 18, }, // Uruguay
    {  622, 1792,  815549,  815565,  815590, 16, 25, 21, }, // Uzbekistan
    {  622, 1803,  815611,  815624,  815646, 13, 22, 18, }, // Vanuatu
    {  622, 1811,       0,  815664,       0,  0, 15,  0, }, // Venezuela
    {  622, 1821,  815679,  815696,  815722, 17, 26, 22, }, // Vladivostok
    {  622, 1833,  815744,  815759,  815783, 15, 24, 20, }, // Volgograd
    {  622, 1843,       0,  815803,       0,  0, 12,  0, }, // Vostok
    {  622, 1850,       0,  815815,       0,  0, 16,  0, }, // Wake
    {  622, 1855,       0,  815831,       0,  0, 22,  0, }, // Wallis
    {  622, 1862,  815853,  815865,  815886, 12, 21, 17, }, // Yakutsk
    {  622, 1870,  815903,  815921,  815948, 18, 27, 23, }, // Yekaterinburg
    {  626,    0,  815971,  815979,  815993,  8, 14, 15, }, // Acre Vietnamese/Latin/Vietnam
    {  626,    5,       0,  816008,       0,  0, 15,  0, }, // Afghanistan
    {  626,   17,       0,  816023,       0,  0, 13,  0, }, // Africa_Central
    {  626,   32,       0,  816036,       0,  0, 12,  0, }, // Africa_Eastern
    {  626,   65,       0,  816048,       0,  0, 17,  0, }, // Africa_Southern
    {  626,   81,  816065,  816076,  816093, 11, 17, 18, }, // Africa_Western
    {  626,  107,  816111,  816121,  816137, 10, 16, 17, }, // Alaska
    {  626,  128,  816154,  816164,  816180, 10, 16, 17, }, // Almaty
    {  626,  135,  816197,  816207,  816223, 10, 16, 17, }, // Amazon
    {  626,  142,  816240,  816254,  816274, 14, 20, 21, }, // America_Central
    {  626,  158,  816295,  816308,  816327, 13, 19, 20, }, // America_Eastern
    {  626,  174,  816347,  816359,  816377, 12, 18, 19, }, // America_Mountain
    {  626,  191,  816396,  816415,  816440, 19, 25, 26, }, // America_Pacific
    {  626,  207,  816466,  816476,  816492, 10, 16, 17, }, // Anadyr
    {  626,  214,  816509,  816517,  816531,  8, 14, 15, }, // Apia
    {  626,  219,  816546,  816555,  816570,  9, 15, 16, }, // Aqtau
    {  626,  225,  816586,  816596,  816612, 10, 16, 17, }, // Aqtobe
    {  626,  232,  816629,  816638,  816653,  9, 15, 16, }, // Arabian
    {  626,  240,  816669,  816682,  816701, 13, 19, 20, }, // Argentina
    {  626,  250,  816721,  816743,  816771, 22, 28, 29, }, // Argentina_Western
    {  626,  268,  816800,  816811,  816828, 11, 17, 18, }, // Armenia
    {  626,  286,  816846,  816863,  816886, 17, 23, 24, }, // Atlantic
    {  626,  295,  816910,  816934,  816964, 24, 30, 31, }, // Australia_Central
    {  626,  313,  816995,  817023,  817057, 28, 34, 35, }, // Australia_CentralWestern
    {  626,  338,  817092,  817115,  817144, 23, 29, 30, }, // Australia_Eastern
    {  626,  356,  817174,  817196,  817224, 22, 28, 29, }, // Australia_Western
    {  626,  374,  817253,  817267,  817287, 14, 20, 21, }, // Azerbaijan
    {  626,  385,  817308,  817318,  817334, 10, 16, 17, }, // Azores
    {  626,  397,  817351,  817365,  817385, 14, 20, 21, }, // Bangladesh
    {  626,  415,       0,  817406,       0,  0, 10,  0, }, // Bhutan
    {  626,  422,       0,  817416,       0,  0, 11,  0, }, // Bolivia
    {  626,  437,  817427,  817439,  817457, 12, 18, 19, }, // Brasilia
    {  626,  454,       0,  817476,       0,  0, 21,  0, }, // Brunei
    {  626,  461,  817497,  817511,  817531, 14, 20, 21, }, // Cape_Verde
    {  626,  472,       0,  817552,       0,  0,  9,  0, }, // Casey
    {  626,  478,       0,  817561,       0,  0, 12,  0, }, // Chamorro
    {  626,  487,  817573,  817584,  817601, 11, 17, 18, }, // Chatham
    {  626,  495,  817619,  817628,  817643,  9, 15, 16, }, // Chile
    {  626,  501,  817659,  817673,  817693, 14, 20, 21, }, // China
    {  626,  507,  817714,  817728,  817748, 14, 20, 21, }, // Choibalsan
    {  626,  518,       0,  817769,       0,  0, 17,  0, }, // Christmas
    {  626,  528,       0,  817786,       0,  0, 18,  0, }, // Cocos
    {  626,  534,  817804,  817816,  817834, 12, 18, 19, }, // Colombia
    {  626,  543,  817853,  817870,  817893, 17, 23, 28, }, // Cook
    {  626,  548,  817921,  817929,  817943,  8, 14, 15, }, // Cuba
    {  626,  559,       0,  817958,       0,  0,  9,  0, }, // Davis
    {  626,  575,       0,  817967,       0,  0, 20,  0, }, // DumontDUrville
    {  626,  612,       0,  817987,       0,  0, 14,  0, }, // East_Timor
    {  626,  623,  818001,  818018,  818041, 17, 23, 24, }, // Easter
    {  626,  630,       0,  818065,       0,  0, 11,  0, }, // Ecuador
    {  626,  638,  818076,  818088,  818106, 12, 18, 19, }, // Europe_Central
    {  626,  653,  818125,  818136,  818153, 11, 17, 18, }, // Europe_Eastern
    {  626,  668,       0,  818171,       0,  0, 21,  0, }, // Europe_Further_Eastern
    {  626,  691,  818192,  818202,  818218, 10, 16, 17, }, // Europe_Western
    {  626,  706,  818235,  818256,  818283, 21, 27, 28, }, // Falkland
    {  626,  715,  818311,  818319,  818333,  8, 14, 15, }, // Fiji
    {  626,  720,       0,  818348,       0,  0, 21,  0, }, // French_Guiana
    {  626,  734,       0,  818369,       0,  0, 28,  0, }, // French_Southern
    {  626,  757,       0,  818397,       0,  0, 13,  0, }, // Galapagos
    {  626,  767,       0,  818410,       0,  0, 11,  0, }, // Gambier
    {  626,  775,  818421,  818432,  818449, 11, 17, 18, }, // Georgia
    {  626,  783,       0,  818467,       0,  0, 20,  0, }, // Gilbert_Islands
    {  626,  799,       0,  818487,       0,  0, 24,  0, }, // GMT
    {  626,  841,  818511,  818534,  818563, 23, 29, 30, }, // Greenland_Eastern
    {  626,  859,  818593,  818615,  818643, 22, 28, 29, }, // Greenland_Western
    {  626,  877,       0,  818672,       0,  0, 14,  0, }, // Guam
    {  626,  882,       0,  818686,       0,  0, 19,  0, }, // Gulf
    {  626,  887,       0,  818705,       0,  0, 10,  0, }, // Guyana
    {  626,  894,  818715,  818731,  818753, 16, 22, 23, }, // Hawaii_Aleutian
    {  626,  910,  818776,  818789,  818808, 13, 19, 20, }, // Hong_Kong
    {  626,  920,  818828,  818836,  818850,  8, 14, 15, }, // Hovd
    {  626,  925,       0,  818865,       0,  0, 15,  0, }, // India
    {  626,  931,       0,  818880,       0,  0, 15,  0, }, // Indian_Ocean
    {  626,  944,       0,  818895,       0,  0, 14,  0, }, // Indochina
    {  626,  954,       0,  818909,       0,  0, 24,  0, }, // Indonesia_Central
    {  626,  972,       0,  818933,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  626,  990,       0,  818956,       0,  0, 22,  0, }, // Indonesia_Western
    {  626, 1008,  818978,  818986,  819000,  8, 14, 15, }, // Iran
    {  626, 1019,  819015,  819026,  819043, 11, 17, 18, }, // Irkutsk
    {  626, 1027,  819061,  819071,  819087, 10, 16, 17, }, // Israel
    {  626, 1034,  819104,  819116,  819134, 12, 18, 19, }, // Japan
    {  626, 1040,  819153,  819181,  819215, 28, 34, 35, }, // Kamchatka
    {  626, 1058,       0,  819250,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  626, 1077,       0,  819274,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  626, 1106,  819297,  819309,  819327, 12, 18, 19, }, // Korea
    {  626, 1112,       0,  819346,       0,  0, 10,  0, }, // Kosrae
    {  626, 1119,  819356,  819371,  819392, 15, 21, 22, }, // Krasnoyarsk
    {  626, 1151,       0,  819414,       0,  0, 13,  0, }, // Kyrgystan
    {  626, 1161,       0,  819427,       0,  0,  9,  0, }, // Lanka
    {  626, 1175,       0,  819436,       0,  0, 17,  0, }, // Line_Islands
    {  626, 1188,  819453,  819466,  819485, 13, 19, 20, }, // Lord_Howe
    {  626, 1198,  819505,  819515,  819531, 10, 16, 17, }, // Macau
    {  626, 1204,  819548,  819559,  819576, 11, 17, 18, }, // Magadan
    {  626, 1219,       0,  819594,       0,  0, 12,  0, }, // Malaysia
    {  626, 1228,       0,  819606,       0,  0, 12,  0, }, // Maldives
    {  626, 1237,       0,  819618,       0,  0, 13,  0, }, // Marquesas
    {  626, 1247,       0,  819631,       0,  0, 21,  0, }, // Marshall_Islands
    {  626, 1264,  819652,  819665,  819684, 13, 19, 20, }, // Mauritius
    {  626, 1274,       0,  819704,       0,  0, 10,  0, }, // Mawson
    {  626, 1281,  819714,  819740,  819772, 26, 32, 33, }, // Mexico_Pacific
    {  626, 1296,  819805,  819819,  819839, 14, 20, 21, }, // Mongolia
    {  626, 1305,  819860,  819872,  819890, 12, 18, 19, }, // Moscow
    {  626, 1312,       0,  819909,       0,  0, 11,  0, }, // Myanmar
    {  626, 1320,       0,  819920,       0,  0,  9,  0, }, // Nauru
    {  626, 1326,       0,  819929,       0,  0,  9,  0, }, // Nepal
    {  626, 1332,  819938,  819955,  819978, 17, 23, 24, }, // New_Caledonia
    {  626, 1346,  820002,  820017,  820038, 15, 21, 22, }, // New_Zealand
    {  626, 1358,  820060,  820076,  820098, 16, 22, 23, }, // Newfoundland
    {  626, 1371,       0,  820121,       0,  0,  8,  0, }, // Niue
    {  626, 1376,  820129,  820144,  820165, 15, 21, 22, }, // Norfolk
    {  626, 1384,  820187,  820210,  820239, 23, 29, 30, }, // Noronha
    {  626, 1392,       0,  820269,       0,  0, 24,  0, }, // North_Mariana
    {  626, 1406,  820293,  820308,  820329, 15, 21, 22, }, // Novosibirsk
    {  626, 1418,  820351,  820359,  820373,  8, 14, 15, }, // Omsk
    {  626, 1428,  820388,  820400,  820418, 12, 18, 19, }, // Pakistan
    {  626, 1437,       0,  820437,       0,  0,  9,  0, }, // Palau
    {  626, 1443,       0,  820446,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  626, 1460,  820466,  820478,  820496, 12, 18, 19, }, // Paraguay
    {  626, 1469,  820515,  820523,  820537,  8, 14, 15, }, // Peru
    {  626, 1474,  820552,  820565,  820584, 13, 19, 20, }, // Philippines
    {  626, 1486,       0,  820604,       0,  0, 20,  0, }, // Phoenix_Islands
    {  626, 1502,  820624,  820650,  820682, 26, 32, 35, }, // Pierre_Miquelon
    {  626, 1518,       0,  820717,       0,  0, 12,  0, }, // Pitcairn
    {  626, 1527,       0,  820729,       0,  0, 10,  0, }, // Ponape
    {  626, 1534,       0,  820739,       0,  0, 15,  0, }, // Pyongyang
    {  626, 1544,  820754,  820767,  820786, 13, 19, 20, }, // Qyzylorda
    {  626, 1554,       0,  820806,       0,  0, 11,  0, }, // Reunion
    {  626, 1562,       0,  820817,       0,  0, 11,  0, }, // Rothera
    {  626, 1570,  820828,  820840,  820858, 12, 18, 19, }, // Sakhalin
    {  626, 1579,  820877,  820887,  820903, 10, 16, 17, }, // Samara
    {  626, 1596,  820920,  820929,  820944,  9, 15, 18, }, // Samoa
    {  626, 1602,       0,  820962,       0,  0, 14,  0, }, // Seychelles
    {  626, 1624,       0,  820976,       0,  0, 13,  0, }, // Singapore
    {  626, 1634,       0,  820989,       0,  0, 20,  0, }, // Solomon
    {  626, 1642,       0,  821009,       0,  0, 15,  0, }, // South_Georgia
    {  626, 1656,       0,  821024,       0,  0, 12,  0, }, // Suriname
    {  626, 1676,       0,  821036,       0,  0,  9,  0, }, // Syowa
    {  626, 1682,       0,  821045,       0,  0, 10,  0, }, // Tahiti
    {  626, 1689,  821055,  821066,  821083, 11, 17, 18, }, // Taipei
    {  626, 1696,       0,  821101,       0,  0, 14,  0, }, // Tajikistan
    {  626, 1724,       0,  821115,       0,  0, 11,  0, }, // Tokelau
    {  626, 1732,  821126,  821135,  821150,  9, 15, 16, }, // Tonga
    {  626, 1738,       0,  821166,       0,  0,  9,  0, }, // Truk
    {  626, 1750,  821175,  821191,  821213, 16, 22, 23, }, // Turkmenistan
    {  626, 1763,       0,  821236,       0,  0, 10,  0, }, // Tuvalu
    {  626, 1777,  821246,  821257,  821274, 11, 17, 18, }, // Uruguay
    {  626, 1792,  821292,  821306,  821326, 14, 20, 21, }, // Uzbekistan
    {  626, 1803,  821347,  821358,  821375, 11, 17, 18, }, // Vanuatu
    {  626, 1811,       0,  821393,       0,  0, 13,  0, }, // Venezuela
    {  626, 1821,  821406,  821421,  821442, 15, 21, 22, }, // Vladivostok
    {  626, 1833,  821464,  821477,  821496, 13, 19, 20, }, // Volgograd
    {  626, 1843,       0,  821516,       0,  0, 10,  0, }, // Vostok
    {  626, 1850,       0,  821526,       0,  0, 12,  0, }, // Wake
    {  626, 1855,       0,  821538,       0,  0, 20,  0, }, // Wallis
    {  626, 1862,  821558,  821569,  821586, 11, 17, 18, }, // Yakutsk
    {  626, 1870,  821604,  821621,  821644, 17, 23, 24, }, // Yekaterinburg
    {  626, 1892,       0,  821668,       0,  0,  9,  0, }, // Yukon
    {  630,  286,  821677,  821690,  821711, 13, 21, 19, }, // Atlantic Walser/Latin/Switzerland
    {  630,  638,  821730,  821749,  821776, 19, 27, 25, }, // Europe_Central
    {  630,  653,  821801,  821817,  821841, 16, 24, 22, }, // Europe_Eastern
    {  630,  691,  821863,  821880,  821905, 17, 25, 23, }, // Europe_Western
    {  632,    5,       0,  821928,       0,  0, 17,  0, }, // Afghanistan Welsh/Latin/United Kingdom
    {  632,   17,       0,  821945,       0,  0, 24,  0, }, // Africa_Central
    {  632,   32,       0,  821969,       0,  0, 21,  0, }, // Africa_Eastern
    {  632,   65,       0,  821990,       0,  0, 24,  0, }, // Africa_Southern
    {  632,   81,  822014,  822037,  822068, 23, 31, 27, }, // Africa_Western
    {  632,  107,  822095,  822107,  822127, 12, 20, 16, }, // Alaska
    {  632,  135,  822143,  822157,  822179, 14, 22, 18, }, // Amazon
    {  632,  142,  822197,  822229,  822269, 32, 40, 36, }, // America_Central
    {  632,  158,  822305,  822334,  822371, 29, 37, 33, }, // America_Eastern
    {  632,  174,  822404,  822436,  822476, 32, 40, 36, }, // America_Mountain
    {  632,  191,  822512,  822547,  822590, 35, 43, 39, }, // America_Pacific
    {  632,  214,  822629,  822639,  822657, 10, 18, 14, }, // Apia
    {  632,  232,  822671,  822685,  822707, 14, 22, 18, }, // Arabian
    {  632,  240,  822725,  822742,  822764, 17, 22, 18, }, // Argentina
    {  632,  250,  822782,  822806,  822838, 24, 32, 28, }, // Argentina_Western
    {  632,  268,  822866,  822879,  822900, 13, 21, 17, }, // Armenia
    {  632,  286,  822917,  822941,  822973, 24, 32, 28, }, // Atlantic
    {  632,  295,  823001,  823027,  823061, 26, 34, 30, }, // Australia_Central
    {  632,  313,  823091,  823127,  823171, 36, 44, 40, }, // Australia_CentralWestern
    {  632,  338,  823211,  823234,  823265, 23, 31, 27, }, // Australia_Eastern
    {  632,  356,  823292,  823317,  823350, 25, 33, 29, }, // Australia_Western
    {  632,  374,  823379,  823395,  823419, 16, 24, 20, }, // Azerbaijan
    {  632,  385,  823439,  823454,  823477, 15, 23, 19, }, // Azores
    {  632,  397,  823496,  823512,  823536, 16, 24, 20, }, // Bangladesh
    {  632,  415,       0,  823556,       0,  0, 12,  0, }, // Bhutan
    {  632,  422,       0,  823568,       0,  0, 13,  0, }, // Bolivia
    {  632,  437,  823581,  823595,  823617, 14, 22, 18, }, // Brasilia
    {  632,  454,       0,  823635,       0,  0, 23,  0, }, // Brunei
    {  632,  461,  823658,  823674,  823698, 16, 24, 20, }, // Cape_Verde
    {  632,  478,       0,  823718,       0,  0, 14,  0, }, // Chamorro
    {  632,  487,  823732,  823745,  823766, 13, 21, 17, }, // Chatham
    {  632,  495,  823783,  823794,  823813, 11, 19, 15, }, // Chile
    {  632,  501,  823828,  823841,  823862, 13, 21, 17, }, // China
    {  632,  507,  823879,  823895,  823919, 16, 24, 20, }, // Choibalsan
    {  632,  518,       0,  823939,       0,  0, 20,  0, }, // Christmas
    {  632,  528,       0,  823959,       0,  0, 20,  0, }, // Cocos
    {  632,  534,  823979,  823993,  824015, 14, 22, 18, }, // Colombia
    {  632,  543,  824033,  824052,  824079, 19, 27, 30, }, // Cook
    {  632,  548,  824109,  824120,  824139, 11, 19, 14, }, // Cuba
    {  632,  559,       0,  824153,       0,  0, 11,  0, }, // Davis
    {  632,  575,       0,  824164,       0,  0, 22,  0, }, // DumontDUrville
    {  632,  612,       0,  824186,       0,  0, 19,  0, }, // East_Timor
    {  632,  623,  824205,  824222,  824247, 17, 25, 21, }, // Easter
    {  632,  630,       0,  824268,       0,  0, 13,  0, }, // Ecuador
    {  632,  638,  824281,  824303,  824333, 22, 30, 26, }, // Europe_Central
    {  632,  653,  824359,  824378,  824405, 19, 27, 23, }, // Europe_Eastern
    {  632,  668,       0,  824428,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  632,  691,  824452,  824473,  824502, 21, 29, 25, }, // Europe_Western
    {  632,  706,  824527,  824560,  824601, 33, 41, 37, }, // Falkland
    {  632,  715,  824638,  824648,  824666, 10, 18, 14, }, // Fiji
    {  632,  720,       0,  824680,       0,  0, 21,  0, }, // French_Guiana
    {  632,  734,       0,  824701,       0,  0, 43,  0, }, // French_Southern
    {  632,  757,       0,  824744,       0,  0, 15,  0, }, // Galapagos
    {  632,  767,       0,  824759,       0,  0, 13,  0, }, // Gambier
    {  632,  775,  824772,  824785,  824806, 13, 21, 17, }, // Georgia
    {  632,  783,       0,  824823,       0,  0, 22,  0, }, // Gilbert_Islands
    {  632,  799,       0,  824845,       0,  0, 23,  0, }, // GMT
    {  632,  841,  824868,  824893,  824926, 25, 33, 29, }, // Greenland_Eastern
    {  632,  859,  824955,  824982,  825017, 27, 35, 31, }, // Greenland_Western
    {  632,  882,       0,  825048,       0,  0, 21,  0, }, // Gulf
    {  632,  887,       0,  825069,       0,  0, 12,  0, }, // Guyana
    {  632,  894,  825081,  825102,  825131, 21, 29, 25, }, // Hawaii_Aleutian
    {  632,  910,  825156,  825171,  825194, 15, 23, 19, }, // Hong_Kong
    {  632,  920,  825213,  825223,  825241, 10, 18, 14, }, // Hovd
    {  632,  925,       0,  825255,       0,  0, 11,  0, }, // India
    {  632,  931,       0,  825266,       0,  0, 19,  0, }, // Indian_Ocean
    {  632,  944,       0,  825285,       0,  0, 18,  0, }, // Indochina
    {  632,  954,       0,  825303,       0,  0, 26,  0, }, // Indonesia_Central
    {  632,  972,       0,  825329,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  632,  990,       0,  825352,       0,  0, 25,  0, }, // Indonesia_Western
    {  632, 1008,  825377,  825387,  825405, 10, 18, 14, }, // Iran
    {  632, 1019,  825419,  825432,  825453, 13, 21, 17, }, // Irkutsk
    {  632, 1027,  825470,  825482,  825502, 12, 20, 16, }, // Israel
    {  632, 1034,  825518,  825529,  825548, 11, 19, 15, }, // Japan
    {  632, 1058,       0,  825563,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  632, 1077,       0,  825587,       0,  0, 26,  0, }, // Kazakhstan_Western
    {  632, 1106,  825613,  825624,  825643, 11, 19, 15, }, // Korea
    {  632, 1112,       0,  825658,       0,  0, 12,  0, }, // Kosrae
    {  632, 1119,  825670,  825687,  825712, 17, 25, 21, }, // Krasnoyarsk
    {  632, 1151,       0,  825733,       0,  0, 16,  0, }, // Kyrgystan
    {  632, 1175,       0,  825749,       0,  0, 19,  0, }, // Line_Islands
    {  632, 1188,  825768,  825790,  825820, 22, 30, 26, }, // Lord_Howe
    {  632, 1204,  825846,  825859,  825880, 13, 21, 17, }, // Magadan
    {  632, 1219,       0,  825897,       0,  0, 14,  0, }, // Malaysia
    {  632, 1228,       0,  825911,       0,  0, 16,  0, }, // Maldives
    {  632, 1237,       0,  825927,       0,  0, 15,  0, }, // Marquesas
    {  632, 1247,       0,  825942,       0,  0, 23,  0, }, // Marshall_Islands
    {  632, 1264,  825965,  825980,  826003, 15, 23, 19, }, // Mauritius
    {  632, 1274,       0,  826022,       0,  0, 12,  0, }, // Mawson
    {  632, 1281,  826034,  826056,  826086, 22, 30, 26, }, // Mexico_Pacific
    {  632, 1296,  826112,  826128,  826152, 16, 24, 20, }, // Mongolia
    {  632, 1305,  826172,  826184,  826204, 12, 20, 16, }, // Moscow
    {  632, 1312,       0,  826220,       0,  0, 13,  0, }, // Myanmar
    {  632, 1320,       0,  826233,       0,  0, 11,  0, }, // Nauru
    {  632, 1326,       0,  826244,       0,  0, 11,  0, }, // Nepal
    {  632, 1332,  826255,  826277,  826307, 22, 30, 26, }, // New_Caledonia
    {  632, 1346,  826333,  826352,  826379, 19, 27, 23, }, // New_Zealand
    {  632, 1358,  826402,  826420,  826446, 18, 26, 22, }, // Newfoundland
    {  632, 1371,       0,  826468,       0,  0, 10,  0, }, // Niue
    {  632, 1376,  826478,  826496,  826522, 18, 26, 22, }, // Norfolk
    {  632, 1384,  826544,  826569,  826602, 25, 33, 29, }, // Noronha
    {  632, 1406,  826631,  826648,  826673, 17, 25, 21, }, // Novosibirsk
    {  632, 1418,  826694,  826704,  826722, 10, 18, 14, }, // Omsk
    {  632, 1428,  826736,  826750,  826772, 14, 22, 18, }, // Pakistan
    {  632, 1437,       0,  826790,       0,  0, 11,  0, }, // Palau
    {  632, 1443,       0,  826801,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  632, 1460,  826826,  826840,  826862, 14, 22, 18, }, // Paraguay
    {  632, 1469,  826880,  826891,  826910, 11, 19, 15, }, // Peru
    {  632, 1474,  826925,  826940,  826963, 15, 23, 19, }, // Philippines
    {  632, 1486,       0,  826982,       0,  0, 22,  0, }, // Phoenix_Islands
    {  632, 1502,  827004,  827034,  827072, 30, 38, 34, }, // Pierre_Miquelon
    {  632, 1518,       0,  827106,       0,  0, 14,  0, }, // Pitcairn
    {  632, 1527,       0,  827120,       0,  0, 13,  0, }, // Ponape
    {  632, 1534,       0,  827133,       0,  0, 15,  0, }, // Pyongyang
    {  632, 1554,       0,  827148,       0,  0, 13,  0, }, // Reunion
    {  632, 1562,       0,  827161,       0,  0, 13,  0, }, // Rothera
    {  632, 1570,  827174,  827188,  827210, 14, 22, 18, }, // Sakhalin
    {  632, 1579,  827228,  827240,  827260, 12, 20, 16, }, // Samara
    {  632, 1596,  827276,  827287,  827306, 11, 19, 15, }, // Samoa
    {  632, 1602,       0,  827321,       0,  0, 16,  0, }, // Seychelles
    {  632, 1624,       0,  827337,       0,  0, 15,  0, }, // Singapore
    {  632, 1634,       0,  827352,       0,  0, 22,  0, }, // Solomon
    {  632, 1642,       0,  827374,       0,  0, 16,  0, }, // South_Georgia
    {  632, 1656,       0,  827390,       0,  0, 14,  0, }, // Suriname
    {  632, 1676,       0,  827404,       0,  0, 11,  0, }, // Syowa
    {  632, 1682,       0,  827415,       0,  0, 12,  0, }, // Tahiti
    {  632, 1689,  827427,  827439,  827459, 12, 20, 16, }, // Taipei
    {  632, 1696,       0,  827475,       0,  0, 16,  0, }, // Tajikistan
    {  632, 1724,       0,  827491,       0,  0, 13,  0, }, // Tokelau
    {  632, 1732,  827504,  827515,  827534, 11, 19, 15, }, // Tonga
    {  632, 1738,       0,  827549,       0,  0, 11,  0, }, // Truk
    {  632, 1750,  827560,  827578,  827604, 18, 26, 22, }, // Turkmenistan
    {  632, 1763,       0,  827626,       0,  0, 12,  0, }, // Tuvalu
    {  632, 1777,  827638,  827651,  827672, 13, 21, 17, }, // Uruguay
    {  632, 1792,  827689,  827705,  827729, 16, 24, 20, }, // Uzbekistan
    {  632, 1803,  827749,  827762,  827783, 13, 21, 17, }, // Vanuatu
    {  632, 1811,       0,  827800,       0,  0, 15,  0, }, // Venezuela
    {  632, 1821,  827815,  827832,  827857, 17, 25, 21, }, // Vladivostok
    {  632, 1833,  827878,  827893,  827916, 15, 23, 19, }, // Volgograd
    {  632, 1843,       0,  827935,       0,  0, 12,  0, }, // Vostok
    {  632, 1850,       0,  827947,       0,  0, 15,  0, }, // Wake
    {  632, 1855,       0,  827962,       0,  0, 21,  0, }, // Wallis
    {  632, 1862,  827983,  827996,  828017, 13, 21, 17, }, // Yakutsk
    {  632, 1870,  828034,  828053,  828080, 19, 27, 23, }, // Yekaterinburg
    {  632, 1892,       0,  828103,       0,  0, 11,  0, }, // Yukon
    {  633,    5,       0,  828114,       0,  0, 14,  0, }, // Afghanistan Western Balochi/Arabic/Pakistan
    {  633, 1058,       0,  828128,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  633, 1077,       0,  828152,       0,  0, 23,  0, }, // Kazakhstan_Western
    {  633, 1151,       0,  828175,       0,  0, 15,  0, }, // Kyrgystan
    {  633, 1696,       0,  828190,       0,  0, 14,  0, }, // Tajikistan
    {  638,    0,  828204,  828213,  828230,  9, 17, 15, }, // Acre Western Frisian/Latin/Netherlands
    {  638,    5,       0,  828245,       0,  0, 15,  0, }, // Afghanistan
    {  638,   17,       0,  828260,       0,  0, 25,  0, }, // Africa_Central
    {  638,   32,       0,  828285,       0,  0, 21,  0, }, // Africa_Eastern
    {  638,   65,       0,  828306,       0,  0, 20,  0, }, // Africa_Southern
    {  638,   81,  828326,  828347,  828376, 21, 29, 27, }, // Africa_Western
    {  638,  107,  828403,  828414,  828433, 11, 19, 17, }, // Alaska
    {  638,  128,  828450,  828463,  828484, 13, 21, 19, }, // Almaty
    {  638,  135,  828503,  828515,  828535, 12, 20, 18, }, // Amazon
    {  638,  142,  828553,  828565,  828585, 12, 20, 18, }, // America_Central
    {  638,  158,  828603,  828615,  828635, 12, 20, 18, }, // America_Eastern
    {  638,  174,  828653,  828666,  828687, 13, 21, 19, }, // America_Mountain
    {  638,  191,  828706,  828718,  828738, 12, 20, 18, }, // America_Pacific
    {  638,  207,  828756,  828767,  828786, 11, 19, 17, }, // Anadyr
    {  638,  219,  828803,  828813,  828831, 10, 18, 16, }, // Aqtau
    {  638,  225,  828847,  828858,  828877, 11, 19, 17, }, // Aqtobe
    {  638,  232,  828894,  828907,  828928, 13, 21, 19, }, // Arabian
    {  638,  240,  828947,  828963,  828987, 16, 24, 22, }, // Argentina
    {  638,  250,  829009,  829030,  829059, 21, 29, 27, }, // Argentina_Western
    {  638,  268,  829086,  829099,  829120, 13, 21, 19, }, // Armenia
    {  638,  286,  829139,  829152,  829173, 13, 21, 19, }, // Atlantic
    {  638,  295,  829192,  829215,  829246, 23, 31, 29, }, // Australia_Central
    {  638,  313,  829275,  829309,  829351, 34, 42, 40, }, // Australia_CentralWestern
    {  638,  338,  829391,  829412,  829441, 21, 29, 27, }, // Australia_Eastern
    {  638,  356,  829468,  829489,  829518, 21, 29, 27, }, // Australia_Western
    {  638,  374,  829545,  829566,  829595, 21, 29, 27, }, // Azerbaijan
    {  638,  385,  829622,  829633,  829652, 11, 19, 17, }, // Azores
    {  638,  397,  829669,  829683,  829705, 14, 22, 20, }, // Bangladesh
    {  638,  415,       0,  829725,       0,  0, 15,  0, }, // Bhutan
    {  638,  422,       0,  829740,       0,  0, 17,  0, }, // Bolivia
    {  638,  437,  829757,  829775,  829801, 18, 26, 24, }, // Brasilia
    {  638,  454,       0,  829825,       0,  0, 13,  0, }, // Brunei
    {  638,  461,  829838,  829855,  829880, 17, 25, 23, }, // Cape_Verde
    {  638,  478,       0,  829903,       0,  0, 13,  0, }, // Chamorro
    {  638,  487,  829916,  829928,  829948, 12, 20, 18, }, // Chatham
    {  638,  495,  829966,  829980,  830002, 14, 22, 20, }, // Chile
    {  638,  501,  830022,  830035,  830056, 13, 21, 19, }, // China
    {  638,  507,  830075,  830091,  830115, 16, 24, 22, }, // Choibalsan
    {  638,  518,       0,  830137,       0,  0, 18,  0, }, // Christmas
    {  638,  528,       0,  830155,       0,  0, 18,  0, }, // Cocos
    {  638,  534,  830173,  830191,  830217, 18, 26, 24, }, // Colombia
    {  638,  543,  830241,  830257,  830281, 16, 24, 28, }, // Cook
    {  638,  548,  830309,  830323,  830345, 14, 22, 20, }, // Cuba
    {  638,  559,       0,  830365,       0,  0, 10,  0, }, // Davis
    {  638,  575,       0,  830375,       0,  0, 21,  0, }, // DumontDUrville
    {  638,  612,       0,  830396,       0,  0, 18,  0, }, // East_Timor
    {  638,  623,  830414,  830433,  830460, 19, 27, 25, }, // Easter
    {  638,  630,       0,  830485,       0,  0, 18,  0, }, // Ecuador
    {  638,  638,  830503,  830525,  830555, 22, 30, 28, }, // Europe_Central
    {  638,  653,  830583,  830603,  830631, 20, 28, 26, }, // Europe_Eastern
    {  638,  691,  830657,  830677,  830705, 20, 28, 26, }, // Europe_Western
    {  638,  706,  830731,  830751,  830779, 20, 28, 26, }, // Falkland
    {  638,  715,  830805,  830817,  830837, 12, 20, 18, }, // Fiji
    {  638,  720,       0,  830855,       0,  0, 20,  0, }, // French_Guiana
    {  638,  734,       0,  830875,       0,  0, 35,  0, }, // French_Southern
    {  638,  757,       0,  830910,       0,  0, 22,  0, }, // Galapagos
    {  638,  767,       0,  830932,       0,  0, 20,  0, }, // Gambier
    {  638,  775,  830952,  830966,  830988, 14, 22, 20, }, // Georgia
    {  638,  783,       0,  831008,       0,  0, 20,  0, }, // Gilbert_Islands
    {  638,  799,       0,    2861,       0,  0, 19,  0, }, // GMT
    {  638,  841,  831028,  831049,  831078, 21, 29, 27, }, // Greenland_Eastern
    {  638,  859,  831105,  831126,  831155, 21, 29, 27, }, // Greenland_Western
    {  638,  877,       0,  831182,       0,  0, 20,  0, }, // Guam
    {  638,  882,       0,  831202,       0,  0, 17,  0, }, // Gulf
    {  638,  887,       0,  830861,       0,  0, 14,  0, }, // Guyana
    {  638,  894,  831219,  831241,  831271, 22, 30, 28, }, // Hawaii_Aleutian
    {  638,  910,  831299,  831314,  831337, 15, 23, 21, }, // Hong_Kong
    {  638,  920,  831358,  831367,  831384,  9, 17, 15, }, // Hovd
    {  638,  925,       0,  831399,       0,  0, 14,  0, }, // India
    {  638,  931,       0,  831413,       0,  0, 19,  0, }, // Indian_Ocean
    {  638,  944,       0,  831432,       0,  0, 16,  0, }, // Indochina
    {  638,  954,       0,  831448,       0,  0, 25,  0, }, // Indonesia_Central
    {  638,  972,       0,  831473,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  638,  990,       0,  831494,       0,  0, 21,  0, }, // Indonesia_Western
    {  638, 1008,  831515,  831528,  831549, 13, 21, 19, }, // Iran
    {  638, 1019,  831568,  831581,  831602, 13, 21, 19, }, // Irkutsk
    {  638, 1027,  831621,  831636,  831659, 15, 23, 21, }, // Israel
    {  638, 1034,  831680,  831693,  831714, 13, 21, 19, }, // Japan
    {  638, 1040,  831733,  831763,  831801, 30, 38, 36, }, // Kamchatka
    {  638, 1058,       0,  831837,       0,  0, 18,  0, }, // Kazakhstan_Eastern
    {  638, 1077,       0,  831855,       0,  0, 18,  0, }, // Kazakhstan_Western
    {  638, 1106,  831873,  831888,  831911, 15, 23, 21, }, // Korea
    {  638, 1112,       0,  831932,       0,  0, 13,  0, }, // Kosrae
    {  638, 1119,  831945,  831961,  831985, 16, 24, 22, }, // Krasnoyarsk
    {  638, 1151,       0,  832007,       0,  0, 15,  0, }, // Kyrgystan
    {  638, 1161,       0,  832022,       0,  0, 10,  0, }, // Lanka
    {  638, 1175,       0,  832032,       0,  0, 18,  0, }, // Line_Islands
    {  638, 1188,  832050,  832073,  832104, 23, 31, 29, }, // Lord_Howe
    {  638, 1198,  832133,  832145,  832165, 12, 20, 18, }, // Macau
    {  638, 1204,  832183,  832195,  832215, 12, 20, 18, }, // Magadan
    {  638, 1219,       0,  832233,       0,  0, 15,  0, }, // Malaysia
    {  638, 1228,       0,  832248,       0,  0, 15,  0, }, // Maldives
    {  638, 1237,       0,  832263,       0,  0, 22,  0, }, // Marquesas
    {  638, 1247,       0,  832285,       0,  0, 21,  0, }, // Marshall_Islands
    {  638, 1264,  832306,  832324,  832350, 18, 26, 24, }, // Mauritius
    {  638, 1274,       0,  832374,       0,  0, 11,  0, }, // Mawson
    {  638, 1296,  832385,  832401,  832425, 16, 24, 22, }, // Mongolia
    {  638, 1305,  832447,  832458,  832477, 11, 19, 17, }, // Moscow
    {  638, 1312,       0,  832494,       0,  0, 15,  0, }, // Myanmar
    {  638, 1320,       0,  832509,       0,  0, 16,  0, }, // Nauru
    {  638, 1326,       0,  832525,       0,  0, 13,  0, }, // Nepal
    {  638, 1332,  832538,  832558,  832586, 20, 28, 26, }, // New_Caledonia
    {  638, 1346,  832612,  832630,  832656, 18, 26, 24, }, // New_Zealand
    {  638, 1358,  832680,  832699,  832726, 19, 27, 25, }, // Newfoundland
    {  638, 1371,       0,  832751,       0,  0, 11,  0, }, // Niue
    {  638, 1376,  832762,  832782,  832810, 20, 28, 26, }, // Norfolk
    {  638, 1384,  832836,  832860,  832892, 24, 32, 30, }, // Noronha
    {  638, 1392,       0,  832922,       0,  0, 25,  0, }, // North_Mariana
    {  638, 1406,  832947,  832963,  832987, 16, 24, 22, }, // Novosibirsk
    {  638, 1418,  833009,  833018,  833035,  9, 17, 15, }, // Omsk
    {  638, 1428,  833050,  833067,  833092, 17, 25, 23, }, // Pakistan
    {  638, 1437,       0,  833115,       0,  0, 12,  0, }, // Palau
    {  638, 1443,       0,  833127,       0,  0, 25,  0, }, // Papua_New_Guinea
    {  638, 1460,  833152,  833171,  833198, 19, 27, 25, }, // Paraguay
    {  638, 1469,  833223,  833238,  833261, 15, 23, 21, }, // Peru
    {  638, 1474,  833282,  833297,  833320, 15, 23, 21, }, // Philippines
    {  638, 1486,       0,  833341,       0,  0, 20,  0, }, // Phoenix_Islands
    {  638, 1502,  833361,  833390,  833427, 29, 37, 35, }, // Pierre_Miquelon
    {  638, 1518,       0,  833462,       0,  0, 22,  0, }, // Pitcairn
    {  638, 1527,       0,  833484,       0,  0, 12,  0, }, // Ponape
    {  638, 1544,  833496,  833510,  833532, 14, 22, 20, }, // Qyzylorda
    {  638, 1554,       0,  833552,       0,  0, 14,  0, }, // Reunion
    {  638, 1562,       0,  833566,       0,  0, 12,  0, }, // Rothera
    {  638, 1570,  833578,  833591,  833612, 13, 21, 19, }, // Sakhalin
    {  638, 1579,  833631,  833642,  833661, 11, 19, 17, }, // Samara
    {  638, 1596,  833678,  833693,  833716, 15, 23, 21, }, // Samoa
    {  638, 1602,       0,  833737,       0,  0, 14,  0, }, // Seychelles
    {  638, 1624,       0,  833751,       0,  0, 24,  0, }, // Singapore
    {  638, 1634,       0,  833775,       0,  0, 21,  0, }, // Solomon
    {  638, 1642,       0,  833796,       0,  0, 18,  0, }, // South_Georgia
    {  638, 1656,       0,  833814,       0,  0, 16,  0, }, // Suriname
    {  638, 1676,       0,  833830,       0,  0, 10,  0, }, // Syowa
    {  638, 1682,       0,  833840,       0,  0, 17,  0, }, // Tahiti
    {  638, 1689,  833857,  833868,  833887, 11, 19, 17, }, // Taipei
    {  638, 1696,       0,  833904,       0,  0, 15,  0, }, // Tajikistan
    {  638, 1724,       0,  833919,       0,  0, 21,  0, }, // Tokelau
    {  638, 1732,  833940,  833955,  833978, 15, 23, 21, }, // Tonga
    {  638, 1738,       0,  833999,       0,  0, 12,  0, }, // Truk
    {  638, 1750,  834011,  834026,  834049, 15, 23, 21, }, // Turkmenistan
    {  638, 1763,       0,  834070,       0,  0, 17,  0, }, // Tuvalu
    {  638, 1777,  834087,  834105,  834131, 18, 26, 24, }, // Uruguay
    {  638, 1792,  834155,  834169,  834191, 14, 22, 20, }, // Uzbekistan
    {  638, 1803,  834211,  834229,  834255, 18, 26, 24, }, // Vanuatu
    {  638, 1811,       0,  834279,       0,  0, 18,  0, }, // Venezuela
    {  638, 1821,  834297,  834313,  834337, 16, 24, 22, }, // Vladivostok
    {  638, 1833,  834359,  834373,  834395, 14, 22, 20, }, // Volgograd
    {  638, 1843,       0,  834415,       0,  0, 11,  0, }, // Vostok
    {  638, 1850,       0,  834426,       0,  0, 18,  0, }, // Wake
    {  638, 1855,       0,  834444,       0,  0, 23,  0, }, // Wallis
    {  638, 1862,  834467,  834480,  834501, 13, 21, 19, }, // Yakutsk
    {  638, 1870,  834520,  834540,  834568, 20, 28, 26, }, // Yekaterinburg
    {  640,  142,  834594,  834613,  834645, 19, 32, 28, }, // America_Central Wolof/Latin/Senegal
    {  640,  158,  834673,  834687,  834715, 14, 28, 25, }, // America_Eastern
    {  640,  174,  834740,  834756,  834784, 16, 28, 25, }, // America_Mountain
    {  640,  191,  834809,  834827,  834857, 18, 30, 27, }, // America_Pacific
    {  640,  286,  834884,  834903,  834931, 19, 28, 28, }, // Atlantic
    {  640,  638,  834959,  834986,  835025, 27, 39, 35, }, // Europe_Central
    {  640,  653,  835060,  835085,  835122, 25, 37, 33, }, // Europe_Eastern
    {  640,  691,  835155,  835184,  835226, 29, 42, 38, }, // Europe_Western
    {  640,  799,       0,  835264,       0,  0, 21,  0, }, // GMT
    {  641,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan Xhosa/Latin/South Africa
    {  641,   17,       0,      59,       0,  0, 19,  0, }, // Africa_Central
    {  641,   32,       0,      78,       0,  0, 16,  0, }, // Africa_Eastern
    {  641,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  641,   81,     120,     136,     161, 16, 25, 23, }, // Africa_Western
    {  641,  107,     184,     195,     215, 11, 20, 20, }, // Alaska
    {  641,  135,     284,     295,     315, 11, 20, 18, }, // Amazon
    {  641,  142,     333,     345,     366, 12, 21, 21, }, // America_Central
    {  641,  158,     387,     399,     420, 12, 21, 21, }, // America_Eastern
    {  641,  174,     441,     454,     476, 13, 22, 22, }, // America_Mountain
    {  641,  191,     498,     510,     531, 12, 21, 21, }, // America_Pacific
    {  641,  214,     601,     610,     628,  9, 18, 18, }, // Apia
    {  641,  232,     741,     753,     774, 12, 21, 21, }, // Arabian
    {  641,  240,     795,     809,     832, 14, 23, 21, }, // Argentina
    {  641,  250,     853,     875,     906, 22, 31, 29, }, // Argentina_Western
    {  641,  268,     935,     947,     968, 12, 21, 19, }, // Armenia
    {  641,  286,     987,    1000,    1022, 13, 22, 22, }, // Atlantic
    {  641,  295,    1044,    1066,    1098, 22, 32, 32, }, // Australia_Central
    {  641,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {  641,  338,    1241,    1263,    1295, 22, 32, 32, }, // Australia_Eastern
    {  641,  356,    1327,    1349,    1381, 22, 32, 32, }, // Australia_Western
    {  641,  374,    1413,    1428,    1452, 15, 24, 22, }, // Azerbaijan
    {  641,  385,    1474,    1485,    1505, 11, 20, 18, }, // Azores
    {  641,  397,    1523,    1538,    1562, 15, 24, 22, }, // Bangladesh
    {  641,  415,       0,    1584,       0,  0, 11,  0, }, // Bhutan
    {  641,  422,       0,    1595,       0,  0, 12,  0, }, // Bolivia
    {  641,  437,    1607,    1620,    1642, 13, 22, 20, }, // Brasilia
    {  641,  454,       0,    1662,       0,  0, 22,  0, }, // Brunei
    {  641,  461,    1684,    1699,    1723, 15, 24, 22, }, // Cape_Verde
    {  641,  478,       0,    1755,       0,  0, 22,  0, }, // Chamorro
    {  641,  487,    1777,    1789,    1810, 12, 21, 21, }, // Chatham
    {  641,  495,    1831,    1841,    1860, 10, 19, 17, }, // Chile
    {  641,  501,    1877,    1887,    1906, 10, 19, 19, }, // China
    {  641,  507,    1925,    1940,    1964, 15, 24, 22, }, // Choibalsan
    {  641,  518,       0,    1986,       0,  0, 21,  0, }, // Christmas
    {  641,  528,       0,    2007,       0,  0, 18,  0, }, // Cocos
    {  641,  534,    2025,    2038,    2060, 13, 22, 20, }, // Colombia
    {  641,  543,    2080,    2097,    2123, 17, 26, 29, }, // Cook
    {  641,  548,    2152,    2161,    2179,  9, 18, 18, }, // Cuba
    {  641,  559,       0,    2197,       0,  0, 10,  0, }, // Davis
    {  641,  575,       0,    2207,       0,  0, 21,  0, }, // DumontDUrville
    {  641,  612,       0,    2228,       0,  0, 15,  0, }, // East_Timor
    {  641,  623,    2243,    2261,    2288, 18, 27, 25, }, // Easter
    {  641,  630,       0,    2313,       0,  0, 12,  0, }, // Ecuador
    {  641,  638,    2325,    2346,    2376, 21, 30, 28, }, // Europe_Central
    {  641,  653,    2404,    2425,    2455, 21, 30, 28, }, // Europe_Eastern
    {  641,  668,       0,    2483,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  641,  691,    2512,    2533,    2563, 21, 30, 28, }, // Europe_Western
    {  641,  706,    2591,    2612,    2642, 21, 30, 28, }, // Falkland
    {  641,  715,    2670,    2679,    2697,  9, 18, 16, }, // Fiji
    {  641,  720,       0,    2713,       0,  0, 18,  0, }, // French_Guiana
    {  641,  734,       0,    2731,       0,  0, 32,  0, }, // French_Southern
    {  641,  757,       0,    2763,       0,  0, 14,  0, }, // Galapagos
    {  641,  767,       0,    2777,       0,  0, 12,  0, }, // Gambier
    {  641,  775,    2789,    2801,    2822, 12, 21, 19, }, // Georgia
    {  641,  783,       0,    2841,       0,  0, 20,  0, }, // Gilbert_Islands
    {  641,  799,       0,    2861,       0,  0, 19,  0, }, // GMT
    {  641,  841,    2938,    2957,    2985, 19, 28, 26, }, // Greenland_Eastern
    {  641,  859,    3011,    3030,    3058, 19, 28, 26, }, // Greenland_Western
    {  641,  882,       0,    3102,       0,  0, 18,  0, }, // Gulf
    {  641,  887,       0,    3120,       0,  0, 11,  0, }, // Guyana
    {  641,  894,    3131,    3151,    3180, 20, 29, 29, }, // Hawaii_Aleutian
    {  641,  910,    3209,    3223,    3246, 14, 23, 21, }, // Hong_Kong
    {  641,  920,    3267,    3276,    3294,  9, 18, 16, }, // Hovd
    {  641,  925,       0,    3310,       0,  0, 19,  0, }, // India
    {  641,  931,       0,    3329,       0,  0, 17,  0, }, // Indian_Ocean
    {  641,  944,       0,    3346,       0,  0, 14,  0, }, // Indochina
    {  641,  954,       0,    3360,       0,  0, 22,  0, }, // Indonesia_Central
    {  641,  972,       0,    3382,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  641,  990,       0,    3404,       0,  0, 22,  0, }, // Indonesia_Western
    {  641, 1008,    3426,    3435,    3453,  9, 18, 18, }, // Iran
    {  641, 1019,    3471,    3483,    3504, 12, 21, 19, }, // Irkutsk
    {  641, 1027,    3523,    3534,    3554, 11, 20, 20, }, // Israel
    {  641, 1034,    3574,    3584,    3603, 10, 19, 19, }, // Japan
    {  641, 1058,       0,    3725,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  641, 1077,       0,    3745,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  641, 1106,    3765,    3776,    3796, 11, 20, 20, }, // Korea
    {  641, 1112,       0,    3816,       0,  0, 11,  0, }, // Kosrae
    {  641, 1119,    3827,    3843,    3868, 16, 25, 23, }, // Krasnoyarsk
    {  641, 1151,       0,    3891,       0,  0, 15,  0, }, // Kyrgystan
    {  641, 1175,       0,    3916,       0,  0, 17,  0, }, // Line_Islands
    {  641, 1188,    3933,    3947,    3970, 14, 23, 23, }, // Lord_Howe
    {  641, 1204,    4039,    4051,    4072, 12, 21, 19, }, // Magadan
    {  641, 1219,       0,    4091,       0,  0, 13,  0, }, // Malaysia
    {  641, 1228,       0,    4104,       0,  0, 13,  0, }, // Maldives
    {  641, 1237,       0,    4117,       0,  0, 14,  0, }, // Marquesas
    {  641, 1247,       0,    4131,       0,  0, 21,  0, }, // Marshall_Islands
    {  641, 1264,    4152,    4166,    4189, 14, 23, 21, }, // Mauritius
    {  641, 1274,       0,    4210,       0,  0, 11,  0, }, // Mawson
    {  641, 1281,    4221,    4241,    4270, 20, 29, 29, }, // Mexico_Pacific
    {  641, 1296,    4299,    4315,    4340, 16, 25, 23, }, // Mongolia
    {  641, 1305,    4363,    4374,    4394, 11, 20, 18, }, // Moscow
    {  641, 1312,       0,    4412,       0,  0, 12,  0, }, // Myanmar
    {  641, 1320,       0,    4424,       0,  0, 10,  0, }, // Nauru
    {  641, 1326,       0,    4434,       0,  0, 10,  0, }, // Nepal
    {  641, 1332,    4444,    4462,    4489, 18, 27, 25, }, // New_Caledonia
    {  641, 1346,    4514,    4530,    4555, 16, 25, 25, }, // New_Zealand
    {  641, 1358,    4580,    4597,    4623, 17, 26, 26, }, // Newfoundland
    {  641, 1371,       0,    4649,       0,  0,  9,  0, }, // Niue
    {  641, 1376,    4658,    4677,    4705, 19, 28, 28, }, // Norfolk
    {  641, 1384,    4733,    4757,    4790, 24, 33, 31, }, // Noronha
    {  641, 1406,    4847,    4863,    4888, 16, 25, 23, }, // Novosibirsk
    {  641, 1418,    4911,    4920,    4938,  9, 18, 16, }, // Omsk
    {  641, 1428,    4954,    4967,    4989, 13, 22, 20, }, // Pakistan
    {  641, 1437,       0,    5009,       0,  0, 10,  0, }, // Palau
    {  641, 1443,       0,    5019,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  641, 1460,    5040,    5053,    5075, 13, 22, 20, }, // Paraguay
    {  641, 1469,    5095,    5104,    5122,  9, 18, 16, }, // Peru
    {  641, 1474,    5138,    5153,    5177, 15, 24, 22, }, // Philippines
    {  641, 1486,       0,    5199,       0,  0, 20,  0, }, // Phoenix_Islands
    {  641, 1502,    5219,    5245,    5280, 26, 35, 35, }, // Pierre_Miquelon
    {  641, 1518,       0,    5315,       0,  0, 13,  0, }, // Pitcairn
    {  641, 1527,       0,    5328,       0,  0, 11,  0, }, // Ponape
    {  641, 1534,       0,    5339,       0,  0, 14,  0, }, // Pyongyang
    {  641, 1554,       0,    5411,       0,  0, 12,  0, }, // Reunion
    {  641, 1562,       0,    5423,       0,  0, 12,  0, }, // Rothera
    {  641, 1570,    5435,    5448,    5470, 13, 22, 20, }, // Sakhalin
    {  641, 1596,    5539,    5549,    5568, 10, 19, 19, }, // Samoa
    {  641, 1602,       0,    5587,       0,  0, 15,  0, }, // Seychelles
    {  641, 1624,       0,    5602,       0,  0, 23,  0, }, // Singapore
    {  641, 1634,       0,    5625,       0,  0, 20,  0, }, // Solomon
    {  641, 1642,       0,    5645,       0,  0, 18,  0, }, // South_Georgia
    {  641, 1656,       0,    5663,       0,  0, 13,  0, }, // Suriname
    {  641, 1676,       0,    5676,       0,  0, 10,  0, }, // Syowa
    {  641, 1682,       0,    5686,       0,  0, 11,  0, }, // Tahiti
    {  641, 1689,    5697,    5708,    5728, 11, 20, 20, }, // Taipei
    {  641, 1696,       0,    5748,       0,  0, 15,  0, }, // Tajikistan
    {  641, 1724,       0,    5763,       0,  0, 12,  0, }, // Tokelau
    {  641, 1732,    5775,    5785,    5804, 10, 19, 17, }, // Tonga
    {  641, 1738,       0,    5821,       0,  0, 10,  0, }, // Truk
    {  641, 1750,    5831,    5848,    5874, 17, 26, 24, }, // Turkmenistan
    {  641, 1763,       0,    5898,       0,  0, 11,  0, }, // Tuvalu
    {  641, 1777,    5909,    5921,    5942, 12, 21, 19, }, // Uruguay
    {  641, 1792,    5961,    5976,    6000, 15, 24, 22, }, // Uzbekistan
    {  641, 1803,    6022,    6034,    6055, 12, 21, 19, }, // Vanuatu
    {  641, 1811,       0,    6074,       0,  0, 14,  0, }, // Venezuela
    {  641, 1821,    6088,    6104,    6129, 16, 25, 23, }, // Vladivostok
    {  641, 1833,    6152,    6166,    6189, 14, 23, 21, }, // Volgograd
    {  641, 1843,       0,    6210,       0,  0, 11,  0, }, // Vostok
    {  641, 1850,       0,    6221,       0,  0, 16,  0, }, // Wake
    {  641, 1855,       0,    6237,       0,  0, 20,  0, }, // Wallis
    {  641, 1862,    6257,    6269,    6290, 12, 21, 19, }, // Yakutsk
    {  641, 1870,    6309,    6327,    6354, 18, 27, 25, }, // Yekaterinburg
    {  641, 1892,       0,    6379,       0,  0, 10,  0, }, // Yukon
    {  644,    5,       0,      43,       0,  0, 16,  0, }, // Afghanistan Yoruba/Latin/Nigeria
    {  644,   17,       0,  835285,       0,  0, 19,  0, }, // Africa_Central
    {  644,   32,       0,  835304,       0,  0, 23,  0, }, // Africa_Eastern
    {  644,   65,       0,      94,       0,  0, 26,  0, }, // Africa_Southern
    {  644,   81,  835327,  835351,  835383, 24, 32, 29, }, // Africa_Western
    {  644,  107,  835412,  835425,  835446, 13, 21, 22, }, // Alaska
    {  644,  135,  835468,  835482,  835504, 14, 22, 18, }, // Amazon
    {  644,  142,  835522,  835542,  835568, 20, 26, 28, }, // America_Central
    {  644,  158,  835596,  835615,  835640, 19, 25, 27, }, // America_Eastern
    {  644,  174,  835667,  835676,  835691,  9, 15, 21, }, // America_Mountain
    {  644,  191,  835712,  835727,  835748, 15, 21, 23, }, // America_Pacific
    {  644,  214,     601,     610,     628,  9, 18, 18, }, // Apia
    {  644,  232,     741,     753,     774, 12, 21, 21, }, // Arabian
    {  644,  240,  835771,  835784,  835804, 13, 20, 19, }, // Argentina
    {  644,  250,  835823,  835849,  835883, 26, 34, 30, }, // Argentina_Western
    {  644,  268,     935,     947,     968, 12, 21, 19, }, // Armenia
    {  644,  286,  835913,  835930,  835953, 17, 23, 25, }, // Atlantic
    {  644,  295,    1044,    1066,    1098, 22, 32, 32, }, // Australia_Central
    {  644,  313,    1130,    1161,    1201, 31, 40, 40, }, // Australia_CentralWestern
    {  644,  338,    1241,    1263,    1295, 22, 32, 32, }, // Australia_Eastern
    {  644,  356,    1327,    1349,    1381, 22, 32, 32, }, // Australia_Western
    {  644,  374,    1413,    1428,    1452, 15, 24, 22, }, // Azerbaijan
    {  644,  385,  835978,  835990,  836010, 12, 20, 17, }, // Azores
    {  644,  397,    1523,    1538,    1562, 15, 24, 22, }, // Bangladesh
    {  644,  415,       0,    1584,       0,  0, 11,  0, }, // Bhutan
    {  644,  422,       0,  836027,       0,  0, 12,  0, }, // Bolivia
    {  644,  437,  836039,  836053,  836073, 14, 20, 18, }, // Brasilia
    {  644,  454,       0,    1662,       0,  0, 22,  0, }, // Brunei
    {  644,  461,  836091,  836107,  836131, 16, 24, 22, }, // Cape_Verde
    {  644,  478,       0,    1755,       0,  0, 22,  0, }, // Chamorro
    {  644,  487,    1777,    1789,    1810, 12, 21, 21, }, // Chatham
    {  644,  495,  836153,  836163,  836181, 10, 18, 14, }, // Chile
    {  644,  501,  836195,  836206,  836227, 11, 21, 19, }, // China
    {  644,  507,    1925,    1940,    1964, 15, 24, 22, }, // Choibalsan
    {  644,  518,       0,    1986,       0,  0, 21,  0, }, // Christmas
    {  644,  528,       0,    2007,       0,  0, 18,  0, }, // Cocos
    {  644,  534,  836246,  836259,  836278, 13, 19, 18, }, // Colombia
    {  644,  543,    2080,    2097,    2123, 17, 26, 29, }, // Cook
    {  644,  548,  836296,  836306,  836324, 10, 18, 18, }, // Cuba
    {  644,  559,       0,    2197,       0,  0, 10,  0, }, // Davis
    {  644,  575,       0,    2207,       0,  0, 21,  0, }, // DumontDUrville
    {  644,  612,       0,  836342,       0,  0, 21,  0, }, // East_Timor
    {  644,  623,  836363,  836384,  836408, 21, 24, 23, }, // Easter
    {  644,  630,       0,  836431,       0,  0, 12,  0, }, // Ecuador
    {  644,  638,  836443,  836461,  836485, 18, 24, 23, }, // Europe_Central
    {  644,  653,  836508,  836534,  836566, 26, 32, 31, }, // Europe_Eastern
    {  644,  668,       0,  836597,       0,  0, 29,  0, }, // Europe_Further_Eastern
    {  644,  691,  836626,  836648,  836676, 22, 28, 31, }, // Europe_Western
    {  644,  706,  836707,  836722,  836752, 15, 30, 27, }, // Falkland
    {  644,  715,    2670,    2679,    2697,  9, 18, 16, }, // Fiji
    {  644,  720,       0,  836779,       0,  0, 20,  0, }, // French_Guiana
    {  644,  734,       0,  836799,       0,  0, 32,  0, }, // French_Southern
    {  644,  757,       0,  836831,       0,  0, 13,  0, }, // Galapagos
    {  644,  767,       0,    2777,       0,  0, 12,  0, }, // Gambier
    {  644,  775,    2789,    2801,    2822, 12, 21, 19, }, // Georgia
    {  644,  783,       0,    2841,       0,  0, 20,  0, }, // Gilbert_Islands
    {  644,  799,       0,    2861,       0,  0, 19,  0, }, // GMT
    {  644,  841,  836844,  836869,  836903, 25, 34, 25, }, // Greenland_Eastern
    {  644,  859,  836928,  836954,  836988, 26, 34, 33, }, // Greenland_Western
    {  644,  882,       0,    3102,       0,  0, 18,  0, }, // Gulf
    {  644,  887,       0,  837021,       0,  0, 12,  0, }, // Guyana
    {  644,  894,  837033,  837054,  837083, 21, 29, 29, }, // Hawaii_Aleutian
    {  644,  910,    3209,    3223,    3246, 14, 23, 21, }, // Hong_Kong
    {  644,  920,    3267,    3276,    3294,  9, 18, 16, }, // Hovd
    {  644,  925,       0,    3310,       0,  0, 19,  0, }, // India
    {  644,  931,       0,  837112,       0,  0, 18,  0, }, // Indian_Ocean
    {  644,  944,       0,  837130,       0,  0, 15,  0, }, // Indochina
    {  644,  954,       0,  837145,       0,  0, 30,  0, }, // Indonesia_Central
    {  644,  972,       0,    3382,       0,  0, 22,  0, }, // Indonesia_Eastern
    {  644,  990,       0,  837175,       0,  0, 26,  0, }, // Indonesia_Western
    {  644, 1008,    3426,    3435,    3453,  9, 18, 18, }, // Iran
    {  644, 1019,  837201,  837214,  837235, 13, 21, 18, }, // Irkutsk
    {  644, 1027,    3523,    3534,    3554, 11, 20, 20, }, // Israel
    {  644, 1034,    3574,    3584,    3603, 10, 19, 19, }, // Japan
    {  644, 1058,       0,    3725,       0,  0, 20,  0, }, // Kazakhstan_Eastern
    {  644, 1077,       0,    3745,       0,  0, 20,  0, }, // Kazakhstan_Western
    {  644, 1106,    3765,    3776,    3796, 11, 20, 20, }, // Korea
    {  644, 1112,       0,    3816,       0,  0, 11,  0, }, // Kosrae
    {  644, 1119,    3827,    3843,    3868, 16, 25, 23, }, // Krasnoyarsk
    {  644, 1151,       0,    3891,       0,  0, 15,  0, }, // Kyrgystan
    {  644, 1175,       0,    3916,       0,  0, 17,  0, }, // Line_Islands
    {  644, 1188,    3933,    3947,    3970, 14, 23, 23, }, // Lord_Howe
    {  644, 1204,    4039,    4051,    4072, 12, 21, 19, }, // Magadan
    {  644, 1219,       0,    4091,       0,  0, 13,  0, }, // Malaysia
    {  644, 1228,       0,    4104,       0,  0, 13,  0, }, // Maldives
    {  644, 1237,       0,    4117,       0,  0, 14,  0, }, // Marquesas
    {  644, 1247,       0,    4131,       0,  0, 21,  0, }, // Marshall_Islands
    {  644, 1264,  837253,  837267,  837289, 14, 22, 19, }, // Mauritius
    {  644, 1274,       0,    4210,       0,  0, 11,  0, }, // Mawson
    {  644, 1281,  837308,  837331,  837362, 23, 31, 31, }, // Mexico_Pacific
    {  644, 1296,    4299,    4315,    4340, 16, 25, 23, }, // Mongolia
    {  644, 1305,    4363,    4374,    4394, 11, 20, 18, }, // Moscow
    {  644, 1312,       0,    4412,       0,  0, 12,  0, }, // Myanmar
    {  644, 1320,       0,    4424,       0,  0, 10,  0, }, // Nauru
    {  644, 1326,       0,    4434,       0,  0, 10,  0, }, // Nepal
    {  644, 1332,    4444,    4462,    4489, 18, 27, 25, }, // New_Caledonia
    {  644, 1346,    4514,    4530,    4555, 16, 25, 25, }, // New_Zealand
    {  644, 1358,  837393,  837411,  837437, 18, 26, 26, }, // Newfoundland
    {  644, 1371,       0,    4649,       0,  0,  9,  0, }, // Niue
    {  644, 1376,    4658,    4677,    4705, 19, 28, 28, }, // Norfolk
    {  644, 1384,  837463,  837485,  837513, 22, 28, 29, }, // Noronha
    {  644, 1406,    4847,    4863,    4888, 16, 25, 23, }, // Novosibirsk
    {  644, 1418,    4911,    4920,    4938,  9, 18, 16, }, // Omsk
    {  644, 1428,    4954,    4967,    4989, 13, 22, 20, }, // Pakistan
    {  644, 1437,       0,    5009,       0,  0, 10,  0, }, // Palau
    {  644, 1443,       0,    5019,       0,  0, 21,  0, }, // Papua_New_Guinea
    {  644, 1460,  837542,  837556,  837578, 14, 22, 19, }, // Paraguay
    {  644, 1469,  837597,  837607,  837625, 10, 18, 15, }, // Peru
    {  644, 1474,    5138,    5153,    5177, 15, 24, 22, }, // Philippines
    {  644, 1486,       0,    5199,       0,  0, 20,  0, }, // Phoenix_Islands
    {  644, 1502,  837640,  837663,  837694, 23, 31, 31, }, // Pierre_Miquelon
    {  644, 1518,       0,    5315,       0,  0, 13,  0, }, // Pitcairn
    {  644, 1527,       0,    5328,       0,  0, 11,  0, }, // Ponape
    {  644, 1534,       0,    5339,       0,  0, 14,  0, }, // Pyongyang
    {  644, 1554,       0,  837725,       0,  0, 17,  0, }, // Reunion
    {  644, 1562,       0,    5423,       0,  0, 12,  0, }, // Rothera
    {  644, 1570,    5435,    5448,    5470, 13, 22, 20, }, // Sakhalin
    {  644, 1596,    5539,    5549,    5568, 10, 19, 19, }, // Samoa
    {  644, 1602,       0,  837742,       0,  0, 15,  0, }, // Seychelles
    {  644, 1624,       0,    5602,       0,  0, 23,  0, }, // Singapore
    {  644, 1634,       0,    5625,       0,  0, 20,  0, }, // Solomon
    {  644, 1642,       0,  837757,       0,  0, 17,  0, }, // South_Georgia
    {  644, 1656,       0,  837774,       0,  0, 14,  0, }, // Suriname
    {  644, 1676,       0,    5676,       0,  0, 10,  0, }, // Syowa
    {  644, 1682,       0,    5686,       0,  0, 11,  0, }, // Tahiti
    {  644, 1689,    5697,    5708,    5728, 11, 20, 20, }, // Taipei
    {  644, 1696,       0,    5748,       0,  0, 15,  0, }, // Tajikistan
    {  644, 1724,       0,    5763,       0,  0, 12,  0, }, // Tokelau
    {  644, 1732,    5775,    5785,    5804, 10, 19, 17, }, // Tonga
    {  644, 1738,       0,    5821,       0,  0, 10,  0, }, // Truk
    {  644, 1750,    5831,    5848,    5874, 17, 26, 24, }, // Turkmenistan
    {  644, 1763,       0,    5898,       0,  0, 11,  0, }, // Tuvalu
    {  644, 1777,  837788,  837800,  837821, 12, 21, 17, }, // Uruguay
    {  644, 1792,    5961,    5976,    6000, 15, 24, 22, }, // Uzbekistan
    {  644, 1803,    6022,    6034,    6055, 12, 21, 19, }, // Vanuatu
    {  644, 1811,       0,  837838,       0,  0, 14,  0, }, // Venezuela
    {  644, 1821,    6088,    6104,    6129, 16, 25, 23, }, // Vladivostok
    {  644, 1833,    6152,    6166,    6189, 14, 23, 21, }, // Volgograd
    {  644, 1843,       0,    6210,       0,  0, 11,  0, }, // Vostok
    {  644, 1850,       0,    6221,       0,  0, 16,  0, }, // Wake
    {  644, 1855,       0,    6237,       0,  0, 20,  0, }, // Wallis
    {  644, 1862,    6257,    6269,    6290, 12, 21, 19, }, // Yakutsk
    {  644, 1870,    6309,    6327,    6354, 18, 27, 25, }, // Yekaterinburg
    {  644, 1892,       0,  837852,       0,  0, 13,  0, }, // Yukon
    {  645,   81,  837865,  837889,  837921, 24, 32, 29, }, // Africa_Western Yoruba/Latin/Benin
    {  645,  107,  835412,  837950,  837971, 13, 21, 22, }, // Alaska
    {  645,  135,  837993,  838007,  838029, 14, 22, 18, }, // Amazon
    {  645,  142,  835522,  835542,  838047, 20, 26, 28, }, // America_Central
    {  645,  158,  835596,  835615,  838075, 19, 25, 27, }, // America_Eastern
    {  645,  174,  835667,  835676,  838102,  9, 15, 21, }, // America_Mountain
    {  645,  191,  835712,  835727,  838123, 15, 21, 23, }, // America_Pacific
    {  645,  240,  838146,  835784,  835804, 13, 20, 19, }, // Argentina
    {  645,  250,  838159,  838185,  838219, 26, 34, 30, }, // Argentina_Western
    {  645,  286,  835913,  835930,  838249, 17, 23, 25, }, // Atlantic
    {  645,  385,  838274,  838286,  838306, 12, 20, 17, }, // Azores
    {  645,  461,  836091,  838323,  838347, 16, 24, 22, }, // Cape_Verde
    {  645,  495,  838369,  838380,  838399, 11, 19, 15, }, // Chile
    {  645,  501,  838414,  838426,  838448, 12, 22, 20, }, // China
    {  645,  548,  836296,  838468,  838486, 10, 18, 18, }, // Cuba
    {  645,  638,  836443,  836461,  838504, 18, 24, 23, }, // Europe_Central
    {  645,  653,  836508,  836534,  838527, 26, 32, 31, }, // Europe_Eastern
    {  645,  691,  838558,  838580,  838608, 22, 28, 31, }, // Europe_Western
    {  645,  706,  836707,  838639,  836752, 15, 30, 27, }, // Falkland
    {  645,  841,  836844,  838669,  836903, 25, 34, 25, }, // Greenland_Eastern
    {  645,  859,  838703,  838729,  838763, 26, 34, 33, }, // Greenland_Western
    {  645,  894,  837033,  838796,  838825, 21, 29, 29, }, // Hawaii_Aleutian
    {  645,  990,       0,  838854,       0,  0, 26,  0, }, // Indonesia_Western
    {  645, 1019,  837201,  838880,  838901, 13, 21, 18, }, // Irkutsk
    {  645, 1264,  838919,  838935,  838959, 16, 24, 21, }, // Mauritius
    {  645, 1281,  838980,  839004,  839035, 24, 31, 31, }, // Mexico_Pacific
    {  645, 1358,  837393,  839066,  839092, 18, 26, 26, }, // Newfoundland
    {  645, 1460,  837542,  839118,  837578, 14, 22, 19, }, // Paraguay
    {  645, 1469,  837597,  839140,  837625, 10, 18, 15, }, // Peru
    {  645, 1502,  837640,  839158,  839189, 23, 31, 31, }, // Pierre_Miquelon
    {  645, 1554,       0,  839220,       0,  0, 17,  0, }, // Reunion
    {  645, 1602,       0,  839237,       0,  0, 16,  0, }, // Seychelles
    {  645, 1642,       0,  839253,       0,  0, 17,  0, }, // South_Georgia
    {  645, 1777,  837788,  839270,  837821, 12, 21, 17, }, // Uruguay
    {  645, 1892,       0,  839291,       0,  0, 13,  0, }, // Yukon
    {  647,  799,       0,  839304,       0,  0, 31,  0, }, // GMT Zhuang/Latin/China
    {  648,    5,       0,  839335,       0,  0, 26,  0, }, // Afghanistan Zulu/Latin/South Africa
    {  648,   17,       0,  839361,       0,  0, 29,  0, }, // Africa_Central
    {  648,   32,       0,  839390,       0,  0, 31,  0, }, // Africa_Eastern
    {  648,   65,       0,  839421,       0,  0, 40,  0, }, // Africa_Southern
    {  648,   81,  839461,  839494,  839537, 33, 43, 43, }, // Africa_Western
    {  648,  107,  839580,  839601,  839637, 21, 36, 30, }, // Alaska
    {  648,  135,  839667,  839688,  839724, 21, 36, 31, }, // Amazon
    {  648,  142,  839755,  839792,  839844, 37, 52, 46, }, // America_Central
    {  648,  158,  839890,  839924,  839973, 34, 49, 43, }, // America_Eastern
    {  648,  174,  840016,  840054,  840107, 38, 53, 47, }, // America_Mountain
    {  648,  191,  840154,  840191,  840243, 37, 52, 46, }, // America_Pacific
    {  648,  207,  840289,  840306,  840332, 17, 26, 24, }, // Anadyr
    {  648,  214,  840356,  840375,  840404, 19, 29, 28, }, // Apia
    {  648,  232,  840432,  840454,  840486, 22, 32, 29, }, // Arabian
    {  648,  240,  840515,  840539,  840578, 24, 39, 34, }, // Argentina
    {  648,  250,  840612,  840647,  840697, 35, 50, 45, }, // Argentina_Western
    {  648,  268,  840742,  840763,  840795, 21, 32, 30, }, // Armenia
    {  648,  286,  840825,  840848,  840886, 23, 38, 32, }, // Atlantic
    {  648,  295,  840918,  840950,  840993, 32, 43, 42, }, // Australia_Central
    {  648,  313,  841035,  841073,  841121, 38, 48, 47, }, // Australia_CentralWestern
    {  648,  338,  841168,  841200,  841240, 32, 40, 39, }, // Australia_Eastern
    {  648,  356,  841279,  841311,  841354, 32, 43, 42, }, // Australia_Western
    {  648,  374,  841396,  841421,  841456, 25, 35, 33, }, // Azerbaijan
    {  648,  385,  841489,  841510,  841546, 21, 36, 31, }, // Azores
    {  648,  397,  841577,  841602,  841637, 25, 35, 35, }, // Bangladesh
    {  648,  415,       0,  841672,       0,  0, 21,  0, }, // Bhutan
    {  648,  422,       0,  841693,       0,  0, 22,  0, }, // Bolivia
    {  648,  437,  841715,  841738,  841776, 23, 38, 33, }, // Brasilia
    {  648,  454,       0,  841809,       0,  0, 32,  0, }, // Brunei
    {  648,  461,  841841,  841866,  841902, 25, 36, 33, }, // Cape_Verde
    {  648,  478,       0,  841935,       0,  0, 33,  0, }, // Chamorro
    {  648,  487,  841968,  841990,  842022, 22, 32, 31, }, // Chatham
    {  648,  495,  842053,  842073,  842108, 20, 35, 30, }, // Chile
    {  648,  501,  842138,  842158,  842188, 20, 30, 27, }, // China
    {  648,  507,  842215,  842240,  842275, 25, 35, 30, }, // Choibalsan
    {  648,  518,       0,  842305,       0,  0, 31,  0, }, // Christmas
    {  648,  528,       0,  842336,       0,  0, 28,  0, }, // Cocos
    {  648,  534,  842364,  842387,  842425, 23, 38, 33, }, // Colombia
    {  648,  543,  842458,  842485,  842522, 27, 37, 50, }, // Cook
    {  648,  548,  842572,  842591,  842625, 19, 34, 28, }, // Cuba
    {  648,  559,       0,  842653,       0,  0, 20,  0, }, // Davis
    {  648,  575,       0,  842673,       0,  0, 31,  0, }, // DumontDUrville
    {  648,  612,       0,  842704,       0,  0, 25,  0, }, // East_Timor
    {  648,  623,  842729,  842757,  842800, 28, 43, 38, }, // Easter
    {  648,  630,       0,  842838,       0,  0, 22,  0, }, // Ecuador
    {  648,  638,  842860,  842889,  842933, 29, 44, 39, }, // Europe_Central
    {  648,  653,  842972,  843001,  843045, 29, 44, 39, }, // Europe_Eastern
    {  648,  668,       0,  843084,       0,  0, 37,  0, }, // Europe_Further_Eastern
    {  648,  691,  843121,  843150,  843194, 29, 44, 39, }, // Europe_Western
    {  648,  706,  843233,  843264,  843310, 31, 46, 41, }, // Falkland
    {  648,  715,  843351,  843370,  843399, 19, 29, 27, }, // Fiji
    {  648,  720,       0,  843426,       0,  0, 28,  0, }, // French_Guiana
    {  648,  734,       0,  843454,       0,  0, 45,  0, }, // French_Southern
    {  648,  757,       0,  843499,       0,  0, 24,  0, }, // Galapagos
    {  648,  767,       0,  843523,       0,  0, 22,  0, }, // Gambier
    {  648,  775,  843545,  843567,  843599, 22, 32, 30, }, // Georgia
    {  648,  783,       0,  843629,       0,  0, 30,  0, }, // Gilbert_Islands
    {  648,  799,       0,  843659,       0,  0, 29,  0, }, // GMT
    {  648,  841,  843688,  843717,  843761, 29, 44, 38, }, // Greenland_Eastern
    {  648,  859,  843799,  843828,  843872, 29, 44, 39, }, // Greenland_Western
    {  648,  882,       0,  843911,       0,  0, 29,  0, }, // Gulf
    {  648,  887,       0,  843940,       0,  0, 21,  0, }, // Guyana
    {  648,  894,  843961,  843990,  844034, 29, 44, 38, }, // Hawaii_Aleutian
    {  648,  910,  844072,  844096,  844130, 24, 34, 32, }, // Hong_Kong
    {  648,  920,  844162,  844181,  844210, 19, 29, 24, }, // Hovd
    {  648,  925,       0,  844234,       0,  0, 30,  0, }, // India
    {  648,  931,       0,  844264,       0,  0, 27,  0, }, // Indian_Ocean
    {  648,  944,       0,  844291,       0,  0, 24,  0, }, // Indochina
    {  648,  954,       0,  844315,       0,  0, 32,  0, }, // Indonesia_Central
    {  648,  972,       0,  844347,       0,  0, 32,  0, }, // Indonesia_Eastern
    {  648,  990,       0,  844379,       0,  0, 32,  0, }, // Indonesia_Western
    {  648, 1008,  844411,  844430,  844459, 19, 29, 28, }, // Iran
    {  648, 1019,  844487,  844509,  844541, 22, 32, 29, }, // Irkutsk
    {  648, 1027,  844570,  844591,  844622, 21, 31, 31, }, // Israel
    {  648, 1034,  844653,  844673,  844703, 20, 30, 27, }, // Japan
    {  648, 1040,  844730,  844765,  844809, 35, 44, 42, }, // Kamchatka
    {  648, 1058,       0,  844851,       0,  0, 39,  0, }, // Kazakhstan_Eastern
    {  648, 1077,       0,  844890,       0,  0, 40,  0, }, // Kazakhstan_Western
    {  648, 1106,  844930,  844950,  844980, 20, 30, 27, }, // Korea
    {  648, 1112,       0,  845007,       0,  0, 21,  0, }, // Kosrae
    {  648, 1119,  845028,  845054,  845090, 26, 36, 33, }, // Krasnoyarsk
    {  648, 1151,       0,  845123,       0,  0, 24,  0, }, // Kyrgystan
    {  648, 1175,       0,  845147,       0,  0, 27,  0, }, // Line_Islands
    {  648, 1188,  845174,  845198,  845232, 24, 34, 33, }, // Lord_Howe
    {  648, 1204,  845265,  845287,  845319, 22, 32, 29, }, // Magadan
    {  648, 1219,       0,  845348,       0,  0, 23,  0, }, // Malaysia
    {  648, 1228,       0,  845371,       0,  0, 23,  0, }, // Maldives
    {  648, 1237,       0,  845394,       0,  0, 24,  0, }, // Marquesas
    {  648, 1247,       0,  845418,       0,  0, 31,  0, }, // Marshall_Islands
    {  648, 1264,  845449,  845473,  845507, 24, 34, 32, }, // Mauritius
    {  648, 1274,       0,  845539,       0,  0, 21,  0, }, // Mawson
    {  648, 1281,  845560,  845590,  845635, 30, 45, 39, }, // Mexico_Pacific
    {  648, 1296,  845674,  845699,  845734, 25, 35, 30, }, // Mongolia
    {  648, 1305,  845764,  845785,  845821, 21, 36, 28, }, // Moscow
    {  648, 1312,       0,  845849,       0,  0, 22,  0, }, // Myanmar
    {  648, 1320,       0,  845871,       0,  0, 20,  0, }, // Nauru
    {  648, 1326,       0,  845891,       0,  0, 20,  0, }, // Nepal
    {  648, 1332,  845911,  845939,  845977, 28, 38, 38, }, // New_Caledonia
    {  648, 1346,  846015,  846041,  846077, 26, 36, 35, }, // New_Zealand
    {  648, 1358,  846112,  846139,  846181, 27, 42, 36, }, // Newfoundland
    {  648, 1371,       0,  846217,       0,  0, 19,  0, }, // Niue
    {  648, 1376,  846236,  846266,  846306, 30, 40, 40, }, // Norfolk
    {  648, 1384,  846346,  846380,  846424, 34, 44, 44, }, // Noronha
    {  648, 1406,  846468,  846494,  846530, 26, 36, 36, }, // Novosibirsk
    {  648, 1418,  846566,  846585,  846614, 19, 29, 29, }, // Omsk
    {  648, 1428,  846643,  846666,  846699, 23, 33, 33, }, // Pakistan
    {  648, 1437,       0,  846732,       0,  0, 20,  0, }, // Palau
    {  648, 1443,       0,  846752,       0,  0, 31,  0, }, // Papua_New_Guinea
    {  648, 1460,  846783,  846806,  846839, 23, 33, 33, }, // Paraguay
    {  648, 1469,  846872,  846891,  846920, 19, 29, 29, }, // Peru
    {  648, 1474,  846949,  846974,  847009, 25, 35, 33, }, // Philippines
    {  648, 1486,       0,  847042,       0,  0, 30,  0, }, // Phoenix_Islands
    {  648, 1502,  847072,  847113,  847168, 41, 55, 50, }, // Pierre_Miquelon
    {  648, 1518,       0,  847218,       0,  0, 23,  0, }, // Pitcairn
    {  648, 1527,       0,  847241,       0,  0, 21,  0, }, // Ponape
    {  648, 1534,       0,  847262,       0,  0, 24,  0, }, // Pyongyang
    {  648, 1554,       0,  847286,       0,  0, 22,  0, }, // Reunion
    {  648, 1562,       0,  847308,       0,  0, 22,  0, }, // Rothera
    {  648, 1570,  847330,  847353,  847386, 23, 33, 30, }, // Sakhalin
    {  648, 1579,  847416,  847433,  847459, 17, 26, 24, }, // Samara
    {  648, 1596,  847483,  847503,  847533, 20, 30, 29, }, // Samoa
    {  648, 1602,       0,  847562,       0,  0, 25,  0, }, // Seychelles
    {  648, 1624,       0,  847587,       0,  0, 34,  0, }, // Singapore
    {  648, 1634,       0,  847621,       0,  0, 30,  0, }, // Solomon
    {  648, 1642,       0,  847651,       0,  0, 28,  0, }, // South_Georgia
    {  648, 1656,       0,  847679,       0,  0, 23,  0, }, // Suriname
    {  648, 1676,       0,  847702,       0,  0, 20,  0, }, // Syowa
    {  648, 1682,       0,  847722,       0,  0, 21,  0, }, // Tahiti
    {  648, 1689,  847743,  847764,  847795, 21, 31, 28, }, // Taipei
    {  648, 1696,       0,  847823,       0,  0, 25,  0, }, // Tajikistan
    {  648, 1724,       0,  847848,       0,  0, 22,  0, }, // Tokelau
    {  648, 1732,  847870,  847890,  847920, 20, 30, 30, }, // Tonga
    {  648, 1738,       0,  847950,       0,  0, 20,  0, }, // Truk
    {  648, 1750,  847970,  847997,  848034, 27, 37, 35, }, // Turkmenistan
    {  648, 1763,       0,  848069,       0,  0, 21,  0, }, // Tuvalu
    {  648, 1777,  848090,  848112,  848149, 22, 37, 32, }, // Uruguay
    {  648, 1792,  848181,  848206,  848241, 25, 35, 33, }, // Uzbekistan
    {  648, 1803,  848274,  848296,  848333, 22, 37, 32, }, // Vanuatu
    {  648, 1811,       0,  848365,       0,  0, 24,  0, }, // Venezuela
    {  648, 1821,  848389,  848415,  848451, 26, 36, 33, }, // Vladivostok
    {  648, 1833,  848484,  848508,  848542, 24, 34, 34, }, // Volgograd
    {  648, 1843,       0,  848576,       0,  0, 21,  0, }, // Vostok
    {  648, 1850,       0,  848597,       0,  0, 26,  0, }, // Wake
    {  648, 1855,       0,  848623,       0,  0, 33,  0, }, // Wallis
    {  648, 1862,  848656,  848678,  848710, 22, 32, 29, }, // Yakutsk
    {  648, 1870,  848739,  848767,  848805, 28, 38, 35, }, // Yekaterinburg
    {  648, 1892,       0,    6379,       0,  0, 10,  0, }, // Yukon
    {  649,    0,  848840,  848851,  848865, 11, 14, 17, }, // Acre Kaingang/Latin/Brazil
    {  649,    5,       0,  848882,       0,  0, 16,  0, }, // Afghanistan
    {  649,   17,       0,  848898,       0,  0, 18,  0, }, // Africa_Central
    {  649,   32,       0,  848916,       0,  0, 19,  0, }, // Africa_Eastern
    {  649,   65,       0,  848935,       0,  0, 17,  0, }, // Africa_Southern
    {  649,   81,  848952,  848971,  848993, 19, 22, 25, }, // Africa_Western
    {  649,  107,  849018,  849032,  849049, 14, 17, 20, }, // Alaska
    {  649,  128,  849069,  849082,  849098, 13, 16, 19, }, // Almaty
    {  649,  135,  849117,  849131,  849148, 14, 17, 20, }, // Amazon
    {  649,  142,  848905,  849168,  849182, 11, 14, 17, }, // America_Central
    {  649,  158,  849199,  849211,  849226, 12, 15, 18, }, // America_Eastern
    {  649,  174,  849244,  849254,  849267, 10, 13, 16, }, // America_Mountain
    {  649,  191,  849283,  849295,  849310, 12, 15, 18, }, // America_Pacific
    {  649,  207,  849328,  849342,  849359, 14, 17, 20, }, // Anadyr
    {  649,  214,  849379,  849391,  849406, 12, 15, 18, }, // Apia
    {  649,  219,  849424,  849436,  849451, 12, 15, 18, }, // Aqtau
    {  649,  225,  849469,  849482,  849498, 13, 16, 19, }, // Aqtobe
    {  649,  232,  849517,  849531,  849548, 14, 17, 20, }, // Arabian
    {  649,  240,  849568,  849583,  849602, 15, 19, 21, }, // Argentina
    {  649,  250,  849623,  849644,  849668, 21, 24, 26, }, // Argentina_Western
    {  649,  268,  849694,  849709,  849727, 15, 18, 21, }, // Armenia
    {  649,  286,  849748,  849763,  849781, 15, 18, 21, }, // Atlantic
    {  649,  295,  849802,  849824,  849849, 22, 25, 28, }, // Australia_Central
    {  649,  313,  849877,  849904,  849935, 27, 31, 34, }, // Australia_CentralWestern
    {  649,  338,  849969,  849992,  850018, 23, 26, 29, }, // Australia_Eastern
    {  649,  356,  850047,  850070,  850096, 23, 26, 29, }, // Australia_Western
    {  649,  374,  850125,  850142,  850162, 17, 20, 23, }, // Azerbaijan
    {  649,  385,  850185,  850199,  850216, 14, 17, 19, }, // Azores
    {  649,  397,  850235,  850252,  850272, 17, 20, 23, }, // Bangladesh
    {  649,  415,       0,  850295,       0,  0, 12,  0, }, // Bhutan
    {  649,  422,       0,  850307,       0,  0, 14,  0, }, // Bolivia
    {  649,  437,  850321,  850336,  850354, 15, 18, 21, }, // Brasilia
    {  649,  454,       0,  850375,       0,  0, 22,  0, }, // Brunei
    {  649,  461,  850397,  850411,  850428, 14, 17, 20, }, // Cape_Verde
    {  649,  478,       0,  850448,       0,  0, 13,  0, }, // Chamorro
    {  649,  487,  850461,  850475,  850492, 14, 17, 20, }, // Chatham
    {  649,  495,  850512,  850523,  850537, 11, 14, 17, }, // Chile
    {  649,  501,  850554,  850565,  850579, 11, 14, 17, }, // China
    {  649,  507,  850596,  850611,  850629, 15, 18, 21, }, // Choibalsan
    {  649,  518,       0,  850650,       0,  0, 25,  0, }, // Christmas
    {  649,  528,       0,  850675,       0,  0, 22,  0, }, // Cocos
    {  649,  534,  850697,  850712,  850730, 15, 18, 21, }, // Colombia
    {  649,  543,  850751,  850771,  850794, 20, 23, 26, }, // Cook
    {  649,  548,  850820,  850831,  850845, 11, 14, 17, }, // Cuba
    {  649,  559,       0,  850862,       0,  0, 13,  0, }, // Davis
    {  649,  575,       0,  850875,       0,  0, 19,  0, }, // DumontDUrville
    {  649,  612,       0,  850894,       0,  0, 18,  0, }, // East_Timor
    {  649,  623,  850912,  850936,  850963, 24, 27, 30, }, // Easter
    {  649,  630,       0,  850993,       0,  0, 15,  0, }, // Ecuador
    {  649,  638,  851008,  851025,  851045, 17, 20, 23, }, // Europe_Central
    {  649,  653,  851068,  851086,  851107, 18, 21, 24, }, // Europe_Eastern
    {  649,  668,       0,  851131,       0,  0, 27,  0, }, // Europe_Further_Eastern
    {  649,  691,  851158,  851176,  851197, 18, 21, 24, }, // Europe_Western
    {  649,  706,  851221,  851244,  851269, 23, 25, 29, }, // Falkland
    {  649,  715,  851298,  851309,  851323, 11, 14, 17, }, // Fiji
    {  649,  720,       0,  851340,       0,  0, 19,  0, }, // French_Guiana
    {  649,  734,       0,  851359,       0,  0, 32,  0, }, // French_Southern
    {  649,  757,       0,  851391,       0,  0, 16,  0, }, // Galapagos
    {  649,  767,       0,  851407,       0,  0, 12,  0, }, // Gambier
    {  649,  775,  851419,  851432,  851448, 13, 16, 19, }, // Georgia
    {  649,  783,       0,  851467,       0,  0, 24,  0, }, // Gilbert_Islands
    {  649,  799,       0,  851491,       0,  0, 26,  0, }, // GMT
    {  649,  841,  851517,  851540,  851566, 23, 26, 29, }, // Greenland_Eastern
    {  649,  859,  851595,  851618,  851644, 23, 26, 29, }, // Greenland_Western
    {  649,  877,       0,  851673,       0,  0, 15,  0, }, // Guam
    {  649,  882,       0,  851688,       0,  0, 12,  0, }, // Gulf
    {  649,  887,       0,  851700,       0,  0, 13,  0, }, // Guyana
    {  649,  894,  851713,  851746,  851782, 33, 36, 39, }, // Hawaii_Aleutian
    {  649,  910,  851821,  851835,  851852, 14, 17, 20, }, // Hong_Kong
    {  649,  920,  851872,  851884,  851899, 12, 15, 18, }, // Hovd
    {  649,  925,       0,  851917,       0,  0, 15,  0, }, // India
    {  649,  931,       0,  851932,       0,  0, 19,  0, }, // Indian_Ocean
    {  649,  944,       0,  851951,       0,  0, 14,  0, }, // Indochina
    {  649,  954,       0,  851965,       0,  0, 19,  0, }, // Indonesia_Central
    {  649,  972,       0,  851984,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  649,  990,       0,  852004,       0,  0, 20,  0, }, // Indonesia_Western
    {  649, 1008,  852024,  852034,  852047, 10, 13, 16, }, // Iran
    {  649, 1019,  852063,  852080,  852100, 17, 20, 23, }, // Irkutsk
    {  649, 1027,  852123,  852136,  852152, 13, 16, 19, }, // Israel
    {  649, 1034,  852171,  852182,  852196, 11, 14, 17, }, // Japan
    {  649, 1040,  852213,  852246,  852282, 33, 36, 39, }, // Kamchatka
    {  649, 1058,       0,  852321,       0,  0, 24,  0, }, // Kazakhstan_Eastern
    {  649, 1077,       0,  852345,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  649, 1106,  852369,  852382,  852398, 13, 16, 19, }, // Korea
    {  649, 1112,       0,  852417,       0,  0, 18,  0, }, // Kosrae
    {  649, 1119,  852435,  852454,  852476, 19, 22, 25, }, // Krasnoyarsk
    {  649, 1151,       0,  852501,       0,  0, 17,  0, }, // Kyrgystan
    {  649, 1161,       0,  852518,       0,  0, 12,  0, }, // Lanka
    {  649, 1175,       0,  852530,       0,  0, 23,  0, }, // Line_Islands
    {  649, 1188,  852553,  852570,  852590, 17, 20, 23, }, // Lord_Howe
    {  649, 1198,  852613,  852625,  852640, 12, 15, 18, }, // Macau
    {  649, 1204,  852658,  852672,  852689, 14, 17, 20, }, // Magadan
    {  649, 1219,       0,  852709,       0,  0, 13,  0, }, // Malaysia
    {  649, 1228,       0,  852722,       0,  0, 23,  0, }, // Maldives
    {  649, 1237,       0,  852745,       0,  0, 18,  0, }, // Marquesas
    {  649, 1247,       0,  852763,       0,  0, 22,  0, }, // Marshall_Islands
    {  649, 1264,  852785,  852798,  852814, 13, 16, 19, }, // Mauritius
    {  649, 1274,       0,  852833,       0,  0, 13,  0, }, // Mawson
    {  649, 1281,  852846,  852868,  852893, 22, 25, 28, }, // Mexico_Pacific
    {  649, 1296,  852921,  852938,  852958, 17, 20, 23, }, // Mongolia
    {  649, 1305,  852981,  852995,  853012, 14, 17, 20, }, // Moscow
    {  649, 1312,       0,  853032,       0,  0, 14,  0, }, // Myanmar
    {  649, 1320,       0,  853046,       0,  0, 13,  0, }, // Nauru
    {  649, 1326,       0,  853059,       0,  0, 12,  0, }, // Nepal
    {  649, 1332,  853071,  853092,  853116, 21, 24, 27, }, // New_Caledonia
    {  649, 1346,  853143,  853162,  853184, 19, 22, 25, }, // New_Zealand
    {  649, 1358,  853209,  853222,  853238, 13, 16, 19, }, // Newfoundland
    {  649, 1371,       0,  853257,       0,  0, 11,  0, }, // Niue
    {  649, 1376,  853268,  853292,  853319, 24, 27, 30, }, // Norfolk
    {  649, 1384,  853349,  853373,  853400, 24, 27, 30, }, // Noronha
    {  649, 1392,       0,  853430,       0,  0, 30,  0, }, // North_Mariana
    {  649, 1406,  853460,  853480,  853503, 20, 23, 29, }, // Novosibirsk
    {  649, 1418,  853532,  853546,  853563, 14, 17, 20, }, // Omsk
    {  649, 1428,  853583,  853599,  853618, 16, 19, 22, }, // Pakistan
    {  649, 1437,       0,  853640,       0,  0, 13,  0, }, // Palau
    {  649, 1443,       0,  853653,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  649, 1460,  853675,  853691,  853710, 16, 19, 22, }, // Paraguay
    {  649, 1469,  853732,  853743,  853757, 11, 14, 17, }, // Peru
    {  649, 1474,  853774,  853789,  853807, 15, 18, 17, }, // Philippines
    {  649, 1486,       0,  853824,       0,  0, 21,  0, }, // Phoenix_Islands
    {  649, 1502,  853845,  853872,  853902, 27, 30, 33, }, // Pierre_Miquelon
    {  649, 1518,       0,  853935,       0,  0, 24,  0, }, // Pitcairn
    {  649, 1527,       0,  853959,       0,  0, 13,  0, }, // Ponape
    {  649, 1534,       0,  853972,       0,  0, 15,  0, }, // Pyongyang
    {  649, 1544,  853987,  854003,  854022, 16, 19, 22, }, // Qyzylorda
    {  649, 1554,       0,  854044,       0,  0, 12,  0, }, // Reunion
    {  649, 1562,       0,  854056,       0,  0, 13,  0, }, // Rothera
    {  649, 1570,  854069,  854084,  854102, 15, 18, 21, }, // Sakhalin
    {  649, 1579,  854123,  854136,  854152, 13, 16, 19, }, // Samara
    {  649, 1596,  854171,  854184,  854200, 13, 16, 19, }, // Samoa
    {  649, 1602,       0,  854219,       0,  0, 16,  0, }, // Seychelles
    {  649, 1624,       0,  854235,       0,  0, 18,  0, }, // Singapore
    {  649, 1634,       0,  854253,       0,  0, 23,  0, }, // Solomon
    {  649, 1642,       0,  854276,       0,  0, 20,  0, }, // South_Georgia
    {  649, 1656,       0,  854296,       0,  0, 15,  0, }, // Suriname
    {  649, 1676,       0,  854311,       0,  0, 12,  0, }, // Syowa
    {  649, 1682,       0,  854323,       0,  0, 12,  0, }, // Tahiti
    {  649, 1689,  854335,  854348,  854364, 13, 16, 19, }, // Taipei
    {  649, 1696,       0,  854383,       0,  0, 18,  0, }, // Tajikistan
    {  649, 1724,       0,  854401,       0,  0, 15,  0, }, // Tokelau
    {  649, 1732,  854416,  854427,  854441, 11, 14, 17, }, // Tonga
    {  649, 1738,       0,  854458,       0,  0, 12,  0, }, // Truk
    {  649, 1750,  854470,  854491,  854515, 21, 24, 27, }, // Turkmenistan
    {  649, 1763,       0,  854542,       0,  0, 13,  0, }, // Tuvalu
    {  649, 1777,  854555,  854570,  854588, 15, 18, 21, }, // Uruguay
    {  649, 1792,  854609,  854628,  854650, 19, 22, 25, }, // Uzbekistan
    {  649, 1803,  854675,  854690,  854708, 15, 18, 21, }, // Vanuatu
    {  649, 1811,       0,  854729,       0,  0, 17,  0, }, // Venezuela
    {  649, 1821,  854746,  854767,  854790, 21, 23, 26, }, // Vladivostok
    {  649, 1833,  854816,  854833,  854853, 17, 20, 23, }, // Volgograd
    {  649, 1843,       0,  854876,       0,  0, 15,  0, }, // Vostok
    {  649, 1850,       0,  854891,       0,  0, 21,  0, }, // Wake
    {  649, 1855,       0,  854912,       0,  0, 24,  0, }, // Wallis
    {  649, 1862,  854936,  854954,  854976, 18, 22, 24, }, // Yakutsk
    {  649, 1870,  855000,  855020,  855043, 20, 23, 26, }, // Yekaterinburg
    {  650,    0,  855069,  855087,  855112, 18, 25, 29, }, // Acre Nheengatu/Latin/Brazil
    {  650,    5,       0,  855141,       0,  0, 22,  0, }, // Afghanistan
    {  650,   17,       0,  855163,       0,  0, 27,  0, }, // Africa_Central
    {  650,   32,       0,  855190,       0,  0, 27,  0, }, // Africa_Eastern
    {  650,   65,       0,  855217,       0,  0, 23,  0, }, // Africa_Southern
    {  650,   81,  855240,  855269,  855304, 29, 35, 39, }, // Africa_Western
    {  650,  107,  855343,  855359,  855379, 16, 20, 27, }, // Alaska
    {  650,  128,  855406,  855421,  855440, 15, 19, 26, }, // Almaty
    {  650,  135,  855466,  855482,  855502, 16, 20, 27, }, // Amazon
    {  650,  142,  855529,  855537,  855563,  8, 26, 30, }, // America_Central
    {  650,  158,  855593,  855608,  855635, 15, 27, 26, }, // America_Eastern
    {  650,  174,  855661,  855674,  855690, 13, 16, 19, }, // America_Mountain
    {  650,  191,  855709,  855724,  855742, 15, 18, 21, }, // America_Pacific
    {  650,  207,  855763,  855777,  855798, 14, 21, 25, }, // Anadyr
    {  650,  214,  855823,  855837,  855858, 14, 21, 25, }, // Apia
    {  650,  219,  855883,  855898,  855920, 15, 22, 26, }, // Aqtau
    {  650,  225,  855946,  855962,  855985, 16, 23, 27, }, // Aqtobe
    {  650,  232,  856012,  856028,  856051, 16, 23, 27, }, // Arabian
    {  650,  240,  856078,  856095,  856119, 17, 24, 28, }, // Argentina
    {  650,  250,  856147,  856176,  856212, 29, 36, 40, }, // Argentina_Western
    {  650,  268,  856252,  856270,  856295, 18, 25, 29, }, // Armenia
    {  650,  286,  856324,  856342,  856367, 18, 25, 28, }, // Atlantic
    {  650,  295,  856395,  856427,  856466, 32, 39, 43, }, // Australia_Central
    {  650,  313,  856509,  856553,  856604, 44, 51, 55, }, // Australia_CentralWestern
    {  650,  338,  856659,  856691,  856730, 32, 39, 43, }, // Australia_Eastern
    {  650,  356,  856773,  856806,  856846, 33, 40, 44, }, // Australia_Western
    {  650,  374,  856890,  856910,  856937, 20, 27, 31, }, // Azerbaijan
    {  650,  385,  856968,  856986,  857011, 18, 25, 29, }, // Azores
    {  650,  397,  857040,  857059,  857085, 19, 26, 30, }, // Bangladesh
    {  650,  415,       0,  857115,       0,  0, 14,  0, }, // Bhutan
    {  650,  422,       0,  857129,       0,  0, 16,  0, }, // Bolivia
    {  650,  437,  857145,  857160,  857178, 15, 18, 21, }, // Brasilia
    {  650,  454,       0,  857199,       0,  0, 26,  0, }, // Brunei
    {  650,  461,  857225,  857246,  857274, 21, 28, 32, }, // Cape_Verde
    {  650,  478,       0,  857306,       0,  0, 15,  0, }, // Chamorro
    {  650,  487,  857321,  857336,  857358, 15, 22, 26, }, // Chatham
    {  650,  495,  857384,  857397,  857417, 13, 20, 24, }, // Chile
    {  650,  501,  857441,  857454,  857474, 13, 20, 24, }, // China
    {  650,  507,  857498,  857514,  857537, 16, 23, 27, }, // Choibalsan
    {  650,  518,       0,  857564,       0,  0, 24,  0, }, // Christmas
    {  650,  528,       0,  857588,       0,  0, 27,  0, }, // Cocos
    {  650,  534,  857615,  857631,  857654, 16, 23, 27, }, // Colombia
    {  650,  543,  857681,  857705,  857736, 24, 31, 42, }, // Cook
    {  650,  548,  857778,  857791,  857811, 13, 20, 24, }, // Cuba
    {  650,  559,       0,  857835,       0,  0, 13,  0, }, // Davis
    {  650,  575,       0,  857848,       0,  0, 25,  0, }, // DumontDUrville
    {  650,  612,       0,  857873,       0,  0, 22,  0, }, // East_Timor
    {  650,  623,  857895,  857920,  857952, 25, 32, 36, }, // Easter
    {  650,  630,       0,  857988,       0,  0, 15,  0, }, // Ecuador
    {  650,  638,  858003,  858029,  858062, 26, 33, 37, }, // Europe_Central
    {  650,  653,  858099,  858125,  858158, 26, 33, 37, }, // Europe_Eastern
    {  650,  668,       0,  858195,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  650,  691,  858221,  858248,  858282, 27, 34, 38, }, // Europe_Western
    {  650,  706,  858320,  858345,  858377, 25, 32, 36, }, // Falkland
    {  650,  715,  858413,  858426,  858446, 13, 20, 24, }, // Fiji
    {  650,  720,       0,  858470,       0,  0, 26,  0, }, // French_Guiana
    {  650,  734,       0,  858496,       0,  0, 47,  0, }, // French_Southern
    {  650,  757,       0,  858543,       0,  0, 21,  0, }, // Galapagos
    {  650,  767,       0,  858564,       0,  0, 16,  0, }, // Gambier
    {  650,  775,  858580,  858596,  858619, 16, 23, 27, }, // Georgia
    {  650,  783,       0,  858646,       0,  0, 25,  0, }, // Gilbert_Islands
    {  650,  799,       0,  858671,       0,  0, 34,  0, }, // GMT
    {  650,  841,  858705,  858736,  858774, 31, 38, 42, }, // Greenland_Eastern
    {  650,  859,  858816,  858848,  858887, 32, 39, 43, }, // Greenland_Western
    {  650,  877,       0,  858930,       0,  0, 20,  0, }, // Guam
    {  650,  882,       0,  858950,       0,  0, 14,  0, }, // Gulf
    {  650,  887,       0,  858964,       0,  0, 15,  0, }, // Guyana
    {  650,  894,  858979,  859019,  859066, 40, 47, 51, }, // Hawaii_Aleutian
    {  650,  910,  859117,  859133,  859156, 16, 23, 27, }, // Hong_Kong
    {  650,  920,  859183,  859198,  859220, 15, 22, 26, }, // Hovd
    {  650,  925,       0,  859246,       0,  0, 20,  0, }, // India
    {  650,  931,       0,  859266,       0,  0, 22,  0, }, // Indian_Ocean
    {  650,  944,       0,  859288,       0,  0, 16,  0, }, // Indochina
    {  650,  954,       0,  859304,       0,  0, 29,  0, }, // Indonesia_Central
    {  650,  972,       0,  859333,       0,  0, 29,  0, }, // Indonesia_Eastern
    {  650,  990,       0,  859362,       0,  0, 30,  0, }, // Indonesia_Western
    {  650, 1008,  859392,  859404,  859423, 12, 19, 23, }, // Iran
    {  650, 1019,  859446,  859465,  859491, 19, 26, 30, }, // Irkutsk
    {  650, 1027,  859521,  859538,  859562, 17, 24, 28, }, // Israel
    {  650, 1034,  859590,  859603,  859623, 13, 20, 24, }, // Japan
    {  650, 1040,  859647,  859680,  859720, 33, 40, 44, }, // Kamchatka
    {  650, 1058,       0,  859764,       0,  0, 32,  0, }, // Kazakhstan_Eastern
    {  650, 1077,       0,  859796,       0,  0, 33,  0, }, // Kazakhstan_Western
    {  650, 1106,  859829,  859844,  859866, 15, 22, 26, }, // Korea
    {  650, 1112,       0,  859892,       0,  0, 16,  0, }, // Kosrae
    {  650, 1119,  859908,  859933,  859965, 25, 32, 36, }, // Krasnoyarsk
    {  650, 1151,       0,  860001,       0,  0, 21,  0, }, // Kyrgystan
    {  650, 1161,       0,  860022,       0,  0, 13,  0, }, // Lanka
    {  650, 1175,       0,  860035,       0,  0, 24,  0, }, // Line_Islands
    {  650, 1188,  860059,  860079,  860106, 20, 27, 31, }, // Lord_Howe
    {  650, 1198,  860137,  860151,  860172, 14, 21, 25, }, // Macau
    {  650, 1204,  860197,  860212,  860234, 15, 22, 26, }, // Magadan
    {  650, 1219,       0,  860260,       0,  0, 17,  0, }, // Malaysia
    {  650, 1228,       0,  860277,       0,  0, 30,  0, }, // Maldives
    {  650, 1237,       0,  860307,       0,  0, 19,  0, }, // Marquesas
    {  650, 1247,       0,  860326,       0,  0, 27,  0, }, // Marshall_Islands
    {  650, 1264,  860353,  860371,  860396, 18, 25, 29, }, // Mauritius
    {  650, 1274,       0,  860425,       0,  0, 14,  0, }, // Mawson
    {  650, 1281,  860439,  860465,  860498, 26, 33, 37, }, // Mexico_Pacific
    {  650, 1296,  860535,  860554,  860580, 19, 26, 30, }, // Mongolia
    {  650, 1305,  860610,  860625,  860647, 15, 22, 26, }, // Moscow
    {  650, 1312,       0,  860673,       0,  0, 15,  0, }, // Myanmar
    {  650, 1320,       0,  860688,       0,  0, 14,  0, }, // Nauru
    {  650, 1326,       0,  860702,       0,  0, 14,  0, }, // Nepal
    {  650, 1332,  860716,  860742,  860775, 26, 33, 37, }, // New_Caledonia
    {  650, 1346,  860812,  860836,  860867, 24, 31, 35, }, // New_Zealand
    {  650, 1358,  860902,  860921,  860947, 19, 26, 30, }, // Newfoundland
    {  650, 1371,       0,  860977,       0,  0, 13,  0, }, // Niue
    {  650, 1376,  860990,  861014,  861045, 24, 31, 35, }, // Norfolk
    {  650, 1384,  861080,  861102,  861131, 22, 29, 33, }, // Noronha
    {  650, 1392,       0,  861164,       0,  0, 36,  0, }, // North_Mariana
    {  650, 1406,  861200,  861226,  861259, 26, 33, 37, }, // Novosibirsk
    {  650, 1418,  861296,  861312,  861335, 16, 23, 27, }, // Omsk
    {  650, 1428,  861362,  861381,  861407, 19, 26, 30, }, // Pakistan
    {  650, 1437,       0,  861437,       0,  0, 14,  0, }, // Palau
    {  650, 1443,       0,  861451,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  650, 1460,  861478,  861496,  861521, 18, 25, 28, }, // Paraguay
    {  650, 1469,  861549,  861562,  861582, 13, 20, 24, }, // Peru
    {  650, 1474,  861606,  861623,  861647, 17, 24, 28, }, // Philippines
    {  650, 1486,       0,  861675,       0,  0, 15,  0, }, // Phoenix_Islands
    {  650, 1502,  861690,  861721,  861759, 31, 38, 42, }, // Pierre_Miquelon
    {  650, 1518,       0,  861801,       0,  0, 17,  0, }, // Pitcairn
    {  650, 1527,       0,  861818,       0,  0, 15,  0, }, // Ponape
    {  650, 1534,       0,  861833,       0,  0, 17,  0, }, // Pyongyang
    {  650, 1544,  861850,  861867,  861891, 17, 24, 28, }, // Qyzylorda
    {  650, 1554,       0,  861919,       0,  0, 22,  0, }, // Reunion
    {  650, 1562,       0,  861941,       0,  0, 15,  0, }, // Rothera
    {  650, 1570,  861956,  861973,  861997, 17, 24, 28, }, // Sakhalin
    {  650, 1579,  862025,  862040,  862062, 15, 22, 26, }, // Samara
    {  650, 1596,  862088,  862103,  862125, 15, 22, 26, }, // Samoa
    {  650, 1602,       0,  862151,       0,  0, 16,  0, }, // Seychelles
    {  650, 1624,       0,  862167,       0,  0, 24,  0, }, // Singapore
    {  650, 1634,       0,  862191,       0,  0, 27,  0, }, // Solomon
    {  650, 1642,       0,  862218,       0,  0, 23,  0, }, // South_Georgia
    {  650, 1656,       0,  862241,       0,  0, 17,  0, }, // Suriname
    {  650, 1676,       0,  862258,       0,  0, 15,  0, }, // Syowa
    {  650, 1682,       0,  862273,       0,  0, 14,  0, }, // Tahiti
    {  650, 1689,  862287,  862302,  862324, 15, 22, 26, }, // Taipei
    {  650, 1696,       0,  862350,       0,  0, 21,  0, }, // Tajikistan
    {  650, 1724,       0,  862371,       0,  0, 16,  0, }, // Tokelau
    {  650, 1732,  862387,  862400,  862420, 13, 20, 24, }, // Tonga
    {  650, 1738,       0,  862444,       0,  0, 14,  0, }, // Truk
    {  650, 1750,  862458,  862482,  862513, 24, 31, 35, }, // Turkmenistan
    {  650, 1763,       0,  862548,       0,  0, 15,  0, }, // Tuvalu
    {  650, 1777,  862563,  862580,  862604, 17, 24, 28, }, // Uruguay
    {  650, 1792,  862632,  862657,  862689, 25, 32, 36, }, // Uzbekistan
    {  650, 1803,  862725,  862741,  862764, 16, 23, 27, }, // Vanuatu
    {  650, 1811,       0,  862791,       0,  0, 18,  0, }, // Venezuela
    {  650, 1821,  862809,  862832,  862862, 23, 30, 34, }, // Vladivostok
    {  650, 1833,  862896,  862916,  862943, 20, 27, 31, }, // Volgograd
    {  650, 1843,       0,  862974,       0,  0, 17,  0, }, // Vostok
    {  650, 1850,       0,  862991,       0,  0, 14,  0, }, // Wake
    {  650, 1855,       0,  863005,       0,  0, 25,  0, }, // Wallis
    {  650, 1862,  863030,  863049,  863075, 19, 26, 30, }, // Yakutsk
    {  650, 1870,  863105,  863125,  863152, 20, 27, 31, }, // Yekaterinburg
    {  651,  415,       0,  863183,       0,  0, 14,  0, }, // Bhutan Nheengatu/Latin/Colombia
    {  651, 1502,  863197,  863228,  863266, 31, 38, 42, }, // Pierre_Miquelon
    {  652,  415,       0,  863183,       0,  0, 14,  0, }, // Bhutan Nheengatu/Latin/Venezuela
    {  652, 1502,  863197,  863228,  863266, 31, 38, 42, }, // Pierre_Miquelon
    {  653,  799,       0,   86447,       0,  0, 17,  0, }, // GMT Haryanvi/Devanagari/India
    {  654,    5,       0,  863308,       0,  0, 18,  0, }, // Afghanistan Northern Frisian/Latin/Germany
    {  654,   17,       0,  863326,       0,  0, 21,  0, }, // Africa_Central
    {  654,   32,       0,  863347,       0,  0, 17,  0, }, // Africa_Eastern
    {  654,   65,       0,  863364,       0,  0, 29,  0, }, // Africa_Southern
    {  654,   81,  863393,  863411,  863438, 18, 27, 23, }, // Africa_Western
    {  654,  107,  863461,  863472,  863492, 11, 20, 16, }, // Alaska
    {  654,  135,  863508,  863521,  863543, 13, 22, 18, }, // Amazon
    {  654,  142,  863561,  863584,  863616, 23, 32, 28, }, // America_Central
    {  654,  158,  863644,  863663,  863691, 19, 28, 24, }, // America_Eastern
    {  654,  174,  863715,  863735,  863764, 20, 29, 25, }, // America_Mountain
    {  654,  191,  863789,  863811,  863843, 22, 32, 28, }, // America_Pacific
    {  654,  214,  863871,  863880,  863898,  9, 18, 14, }, // Apia
    {  654,  232,  863912,  863925,  863947, 13, 22, 18, }, // Arabian
    {  654,  240,  863965,  863982,  864008, 17, 26, 22, }, // Argentina
    {  654,  250,  864030,  864053,  864085, 23, 32, 28, }, // Argentina_Western
    {  654,  268,  864113,  864127,  864150, 14, 23, 19, }, // Armenia
    {  654,  286,  864169,  864192,  864224, 23, 32, 28, }, // Atlantic
    {  654,  295,  864252,  864277,  864311, 25, 34, 30, }, // Australia_Central
    {  654,  313,  864341,  864371,  864410, 30, 39, 35, }, // Australia_CentralWestern
    {  654,  338,  864445,  864466,  864496, 21, 30, 26, }, // Australia_Eastern
    {  654,  356,  864522,  864544,  864575, 22, 31, 27, }, // Australia_Western
    {  654,  374,  864602,  864621,  864649, 19, 28, 24, }, // Azerbaijan
    {  654,  385,  864673,  864686,  864708, 13, 22, 18, }, // Azores
    {  654,  397,  864726,  864742,  864767, 16, 25, 21, }, // Bangladesh
    {  654,  415,       0,  864788,       0,  0, 13,  0, }, // Bhutan
    {  654,  422,       0,  864801,       0,  0, 14,  0, }, // Bolivia
    {  654,  437,  864815,  864830,  864854, 15, 24, 20, }, // Brasilia
    {  654,  454,       0,  864874,       0,  0, 22,  0, }, // Brunei
    {  654,  461,  864896,  864910,  864933, 14, 23, 20, }, // Cape_Verde
    {  654,  478,       0,  864953,       0,  0, 13,  0, }, // Chamorro
    {  654,  487,  864966,  864978,  864999, 12, 21, 17, }, // Chatham
    {  654,  495,  865016,  865027,  865047, 11, 20, 16, }, // Chile
    {  654,  501,  865063,  865075,  865096, 12, 21, 17, }, // China
    {  654,  507,  865113,  865130,  865156, 17, 26, 22, }, // Choibalsan
    {  654,  518,       0,  865178,       0,  0, 14,  0, }, // Christmas
    {  654,  528,       0,  865192,       0,  0, 18,  0, }, // Cocos
    {  654,  534,  865210,  865224,  865247, 14, 23, 19, }, // Colombia
    {  654,  543,  865266,  865283,  865309, 17, 26, 29, }, // Cook
    {  654,  548,  865338,  865348,  865367, 10, 19, 15, }, // Cuba
    {  654,  559,       0,  865382,       0,  0, 10,  0, }, // Davis
    {  654,  575,       0,  865392,       0,  0, 21,  0, }, // DumontDUrville
    {  654,  612,       0,  865413,       0,  0, 16,  0, }, // East_Timor
    {  654,  623,  865429,  865445,  865470, 16, 25, 21, }, // Easter
    {  654,  630,       0,  865491,       0,  0, 13,  0, }, // Ecuador
    {  654,  638,  865504,  865523,  865551, 19, 28, 24, }, // Europe_Central
    {  654,  653,  865575,  865593,  865620, 18, 27, 23, }, // Europe_Eastern
    {  654,  668,       0,  865643,       0,  0, 26,  0, }, // Europe_Further_Eastern
    {  654,  691,  865669,  865688,  865716, 19, 28, 24, }, // Europe_Western
    {  654,  706,  865740,  865760,  865789, 20, 29, 25, }, // Falkland
    {  654,  715,  865814,  865826,  865847, 12, 21, 17, }, // Fiji
    {  654,  720,       0,  865864,       0,  0, 23,  0, }, // French_Guiana
    {  654,  734,       0,  865887,       0,  0, 32,  0, }, // French_Southern
    {  654,  757,       0,  865919,       0,  0, 14,  0, }, // Galapagos
    {  654,  767,       0,  865933,       0,  0, 12,  0, }, // Gambier
    {  654,  775,  865945,  865958,  865980, 13, 22, 18, }, // Georgia
    {  654,  783,       0,  865998,       0,  0, 20,  0, }, // Gilbert_Islands
    {  654,  799,       0,  866018,       0,  0, 20,  0, }, // GMT
    {  654,  841,  866038,  866056,  866083, 18, 27, 23, }, // Greenland_Eastern
    {  654,  859,  866106,  866125,  866153, 19, 28, 24, }, // Greenland_Western
    {  654,  882,       0,  866177,       0,  0,  9,  0, }, // Gulf
    {  654,  887,       0,  866186,       0,  0, 12,  0, }, // Guyana
    {  654,  894,  866198,  866217,  866245, 19, 28, 24, }, // Hawaii_Aleutian
    {  654,  910,  866269,  866282,  866304, 13, 22, 18, }, // Hong_Kong
    {  654,  920,  866322,  866332,  866351, 10, 19, 15, }, // Hovd
    {  654,  925,       0,  866366,       0,  0, 11,  0, }, // India
    {  654,  931,       0,  866377,       0,  0, 19,  0, }, // Indian_Ocean
    {  654,  944,       0,  866396,       0,  0, 16,  0, }, // Indochina
    {  654,  954,       0,  866412,       0,  0, 25,  0, }, // Indonesia_Central
    {  654,  972,       0,  866437,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  654,  990,       0,  866458,       0,  0, 22,  0, }, // Indonesia_Western
    {  654, 1008,  866480,  866490,  866509, 10, 19, 15, }, // Iran
    {  654, 1019,  866524,  866536,  866557, 12, 21, 17, }, // Irkutsk
    {  654, 1027,  866574,  866585,  866605, 11, 20, 16, }, // Israel
    {  654, 1034,  866621,  866632,  866652, 11, 20, 16, }, // Japan
    {  654, 1058,       0,  866668,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  654, 1077,       0,  866689,       0,  0, 22,  0, }, // Kazakhstan_Western
    {  654, 1106,  866711,  866721,  866740, 10, 19, 15, }, // Korea
    {  654, 1112,       0,  866755,       0,  0, 11,  0, }, // Kosrae
    {  654, 1119,  866766,  866782,  866807, 16, 25, 21, }, // Krasnoyarsk
    {  654, 1151,       0,  866828,       0,  0, 15,  0, }, // Kyrgystan
    {  654, 1175,       0,  866843,       0,  0, 17,  0, }, // Line_Islands
    {  654, 1188,  866860,  866874,  866897, 14, 23, 19, }, // Lord_Howe
    {  654, 1204,  866916,  866928,  866949, 12, 21, 17, }, // Magadan
    {  654, 1219,       0,  866966,       0,  0, 13,  0, }, // Malaysia
    {  654, 1228,       0,  866979,       0,  0, 15,  0, }, // Maldives
    {  654, 1237,       0,  866994,       0,  0, 14,  0, }, // Marquesas
    {  654, 1247,       0,  867008,       0,  0, 21,  0, }, // Marshall_Islands
    {  654, 1264,  867029,  867043,  867066, 14, 23, 19, }, // Mauritius
    {  654, 1274,       0,  867085,       0,  0, 11,  0, }, // Mawson
    {  654, 1281,  867096,  867117,  867147, 21, 30, 26, }, // Mexico_Pacific
    {  654, 1296,  867173,  867189,  867214, 16, 25, 21, }, // Mongolia
    {  654, 1305,  867235,  867246,  867266, 11, 20, 16, }, // Moscow
    {  654, 1312,       0,  867282,       0,  0, 13,  0, }, // Myanmar
    {  654, 1320,       0,  867295,       0,  0, 10,  0, }, // Nauru
    {  654, 1326,       0,  867305,       0,  0, 12,  0, }, // Nepal
    {  654, 1332,  867317,  867336,  867364, 19, 28, 24, }, // New_Caledonia
    {  654, 1346,  867388,  867402,  867425, 14, 23, 19, }, // New_Zealand
    {  654, 1358,  867444,  867459,  867483, 15, 24, 20, }, // Newfoundland
    {  654, 1371,       0,  867503,       0,  0,  9,  0, }, // Niue
    {  654, 1376,  867512,  867530,  867557, 18, 27, 23, }, // Norfolk
    {  654, 1384,  867580,  867604,  867637, 24, 33, 29, }, // Noronha
    {  654, 1406,  867666,  867682,  867707, 16, 25, 21, }, // Novosibirsk
    {  654, 1418,  867728,  867737,  867755,  9, 18, 14, }, // Omsk
    {  654, 1428,  867769,  867783,  867806, 14, 23, 19, }, // Pakistan
    {  654, 1437,       0,  867825,       0,  0, 10,  0, }, // Palau
    {  654, 1443,       0,  867835,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  654, 1460,  867855,  867868,  867890, 13, 22, 18, }, // Paraguay
    {  654, 1469,  867908,  867918,  867937, 10, 19, 15, }, // Peru
    {  654, 1474,  867952,  867967,  867991, 15, 24, 20, }, // Philippines
    {  654, 1486,       0,  868011,       0,  0, 20,  0, }, // Phoenix_Islands
    {  654, 1502,  868031,  868058,  868094, 27, 36, 32, }, // Pierre_Miquelon
    {  654, 1518,       0,  868126,       0,  0, 21,  0, }, // Pitcairn
    {  654, 1527,       0,  868147,       0,  0, 11,  0, }, // Ponape
    {  654, 1534,       0,  868158,       0,  0, 14,  0, }, // Pyongyang
    {  654, 1554,       0,  868172,       0,  0, 12,  0, }, // Reunion
    {  654, 1562,       0,  868184,       0,  0, 12,  0, }, // Rothera
    {  654, 1570,  868196,  868209,  868231, 13, 22, 18, }, // Sakhalin
    {  654, 1596,  868249,  868259,  868278, 10, 19, 15, }, // Samoa
    {  654, 1602,       0,  868293,       0,  0, 14,  0, }, // Seychelles
    {  654, 1624,       0,  868307,       0,  0, 14,  0, }, // Singapore
    {  654, 1634,       0,  868321,       0,  0, 15,  0, }, // Solomon
    {  654, 1642,       0,  868336,       0,  0, 17,  0, }, // South_Georgia
    {  654, 1656,       0,  868353,       0,  0, 13,  0, }, // Suriname
    {  654, 1676,       0,  868366,       0,  0, 10,  0, }, // Syowa
    {  654, 1682,       0,  868376,       0,  0, 11,  0, }, // Tahiti
    {  654, 1689,  868387,  868398,  868418, 11, 20, 16, }, // Taipei
    {  654, 1696,       0,  868434,       0,  0, 17,  0, }, // Tajikistan
    {  654, 1724,       0,  868451,       0,  0, 12,  0, }, // Tokelau
    {  654, 1732,  868463,  868473,  868492, 10, 19, 15, }, // Tonga
    {  654, 1738,       0,  868507,       0,  0, 10,  0, }, // Truk
    {  654, 1750,  868517,  868536,  868564, 19, 28, 24, }, // Turkmenistan
    {  654, 1763,       0,  868588,       0,  0, 12,  0, }, // Tuvalu
    {  654, 1777,  868600,  868612,  868633, 12, 21, 17, }, // Uruguay
    {  654, 1792,  868650,  868666,  868691, 16, 25, 21, }, // Uzbekistan
    {  654, 1803,  868712,  868724,  868745, 12, 21, 17, }, // Vanuatu
    {  654, 1811,       0,  868762,       0,  0, 16,  0, }, // Venezuela
    {  654, 1821,  868778,  868794,  868819, 16, 25, 21, }, // Vladivostok
    {  654, 1833,  868840,  868854,  868877, 14, 23, 19, }, // Volgograd
    {  654, 1843,       0,  868896,       0,  0, 11,  0, }, // Vostok
    {  654, 1850,       0,  868907,       0,  0, 15,  0, }, // Wake
    {  654, 1855,       0,  868922,       0,  0, 21,  0, }, // Wallis
    {  654, 1862,  868943,  868955,  868976, 12, 21, 17, }, // Yakutsk
    {  654, 1870,  868993,  869011,  869038, 18, 27, 23, }, // Yekaterinburg
    {  654, 1892,       0,  869061,       0,  0, 10,  0, }, // Yukon
    {  655,  799,       0,  869071,       0,  0, 18,  0, }, // GMT Rajasthani/Devanagari/India
    {  657,  799,       0,  869089,       0,  0, 16,  0, }, // GMT Toki Pona/Latin/world
    {  658,  799,       0,  869105,       0,  0, 15,  0, }, // GMT Pijin/Latin/Solomon Islands
    {  660,  107,  869120,  869134,  869157, 14, 23, 21, }, // Alaska Baluchi/Arabic/Pakistan
    {  660,  135,  869178,  869191,  869213, 13, 22, 20, }, // Amazon
    {  660,  142,  869233,  869252,  869280, 19, 28, 26, }, // America_Central
    {  660,  158,  869306,  869328,  869359, 22, 31, 29, }, // America_Eastern
    {  660,  174,  869388,  869410,  869441, 22, 31, 29, }, // America_Mountain
    {  660,  191,  869470,  869492,  869523, 22, 31, 29, }, // America_Pacific
    {  660,  240,  869552,  869567,  869591, 15, 24, 22, }, // Argentina
    {  660,  250,  869613,  869635,  869666, 22, 31, 29, }, // Argentina_Western
    {  660,  295,  869695,  869717,  869748, 22, 31, 29, }, // Australia_Central
    {  660,  313,  869777,  869805,  869842, 28, 37, 35, }, // Australia_CentralWestern
    {  660,  338,  869877,  869902,  869936, 25, 34, 32, }, // Australia_Eastern
    {  660,  356,  869782,  869810,  869847, 23, 32, 30, }, // Australia_Western
    {  660,  437,  869968,  869981,  870003, 13, 22, 20, }, // Brasilia
    {  660,  638,  870023,  870040,  870066, 17, 26, 24, }, // Europe_Central
    {  660,  653,  870090,  870110,  870139, 20, 29, 27, }, // Europe_Eastern
    {  660,  668,       0,  870166,       0,  0, 36,  0, }, // Europe_Further_Eastern
    {  660,  691,  870202,  870220,  870247, 18, 27, 25, }, // Europe_Western
    {  660,  894,  870272,  870285,  870307, 13, 22, 20, }, // Hawaii_Aleutian
    {  660,  954,       0,  870327,       0,  0, 31,  0, }, // Indonesia_Central
    {  660,  972,       0,  870358,       0,  0, 34,  0, }, // Indonesia_Eastern
    {  660,  990,       0,  870392,       0,  0, 32,  0, }, // Indonesia_Western
    {  660, 1019,  870424,  870439,  870463, 15, 24, 22, }, // Irkutsk
    {  660, 1058,       0,  870485,       0,  0, 33,  0, }, // Kazakhstan_Eastern
    {  660, 1077,       0,  870518,       0,  0, 31,  0, }, // Kazakhstan_Western
    {  660, 1119,  870549,  870568,  870596, 19, 28, 26, }, // Krasnoyarsk
    {  660, 1188,  870622,  870647,  870681, 25, 34, 32, }, // Lord_Howe
    {  660, 1204,  870713,  870727,  870750, 14, 23, 21, }, // Magadan
    {  660, 1281,  870771,  870794,  870826, 23, 32, 30, }, // Mexico_Pacific
    {  660, 1305,  870856,  870868,  870889, 12, 21, 19, }, // Moscow
    {  660, 1358,  870908,  870929,  870959, 21, 30, 28, }, // Newfoundland
    {  660, 1384,  870987,  871001,  871024, 14, 23, 21, }, // Noronha
    {  660, 1406,  871045,  871063,  871090, 18, 27, 25, }, // Novosibirsk
    {  660, 1418,  871115,  871127,  871148, 12, 21, 19, }, // Omsk
    {  660, 1821,  871167,  871185,  871212, 18, 27, 25, }, // Vladivostok
    {  660, 1862,  871237,  871251,  871274, 14, 23, 21, }, // Yakutsk
    {  660, 1870,  871295,  871314,  871342, 19, 28, 26, }, // Yekaterinburg
    {  661,  107,  871368,  871381,  871401, 13, 20, 21, }, // Alaska Baluchi/Latin/Pakistan
    {  661,  135,  871422,  871435,  871455, 13, 20, 21, }, // Amazon
    {  661,  142,  871476,  871497,  871525, 21, 28, 29, }, // America_Central
    {  661,  158,  871554,  871577,  871607, 23, 30, 31, }, // America_Eastern
    {  661,  174,  871638,  871661,  871691, 23, 30, 31, }, // America_Mountain
    {  661,  191,  871722,  871744,  871773, 22, 29, 30, }, // America_Pacific
    {  661,  240,  871803,  871819,  871842, 16, 23, 24, }, // Argentina
    {  661,  250,  871866,  871890,  871921, 24, 31, 32, }, // Argentina_Western
    {  661,  295,  871953,  871976,  872006, 23, 30, 31, }, // Australia_Central
    {  661,  313,  872037,  872067,  872104, 30, 37, 38, }, // Australia_CentralWestern
    {  661,  338,  872142,  872167,  872199, 25, 32, 33, }, // Australia_Eastern
    {  661,  356,  872232,  872255,  872285, 23, 30, 31, }, // Australia_Western
    {  661,  437,  872316,  872329,  872349, 13, 20, 21, }, // Brasilia
    {  661,  638,  872370,  872390,  872417, 20, 27, 28, }, // Europe_Central
    {  661,  653,  872445,  872467,  872496, 22, 29, 30, }, // Europe_Eastern
    {  661,  668,       0,  872526,       0,  0, 38,  0, }, // Europe_Further_Eastern
    {  661,  691,  872564,  872584,  872611, 20, 27, 28, }, // Europe_Western
    {  661,  894,  872639,  872658,  872684, 19, 26, 27, }, // Hawaii_Aleutian
    {  661,  954,       0,  872711,       0,  0, 33,  0, }, // Indonesia_Central
    {  661,  972,       0,  872744,       0,  0, 35,  0, }, // Indonesia_Eastern
    {  661,  990,       0,  872779,       0,  0, 33,  0, }, // Indonesia_Western
    {  661, 1019,  872812,  872825,  872845, 13, 20, 21, }, // Irkutsk
    {  661, 1058,       0,  872866,       0,  0, 34,  0, }, // Kazakhstan_Eastern
    {  661, 1077,       0,  872900,       0,  0, 32,  0, }, // Kazakhstan_Western
    {  661, 1119,  872932,  872949,  872973, 17, 24, 25, }, // Krasnoyarsk
    {  661, 1188,  872998,  873023,  873055, 25, 32, 33, }, // Lord_Howe
    {  661, 1204,  873088,  873102,  873123, 14, 21, 22, }, // Magadan
    {  661, 1281,  873145,  873168,  873198, 23, 30, 31, }, // Mexico_Pacific
    {  661, 1305,  873229,  873241,  873260, 12, 19, 20, }, // Moscow
    {  661, 1358,  873280,  873298,  873323, 18, 25, 26, }, // Newfoundland
    {  661, 1384,  873349,  873362,  873382, 13, 20, 21, }, // Noronha
    {  661, 1406,  873403,  873420,  873444, 17, 24, 25, }, // Novosibirsk
    {  661, 1418,  873469,  873480,  873498, 11, 18, 19, }, // Omsk
    {  661, 1821,  873517,  873536,  873562, 19, 26, 27, }, // Vladivostok
    {  661, 1862,  873589,  873602,  873622, 13, 20, 21, }, // Yakutsk
    {  661, 1870,  873643,  873662,  873688, 19, 26, 27, }, // Yekaterinburg
    {  662,    5,       0,  873715,       0,  0, 19,  0, }, // Afghanistan Ligurian/Latin/Italy
    {  662,   17,       0,  873734,       0,  0, 22,  0, }, // Africa_Central
    {  662,   32,       0,  873756,       0,  0, 25,  0, }, // Africa_Eastern
    {  662,   65,       0,  873781,       0,  0, 26,  0, }, // Africa_Southern
    {  662,   81,  873807,  873832,  873866, 25, 34, 32, }, // Africa_Western
    {  662,  107,  873898,  873912,  873935, 14, 23, 21, }, // Alaska
    {  662,  135,  873956,  873973,  873999, 17, 26, 24, }, // Amazon
    {  662,  142,  874023,  874047,  874080, 24, 33, 31, }, // America_Central
    {  662,  158,  874111,  874138,  874174, 27, 36, 34, }, // America_Eastern
    {  662,  174,  874208,  874231,  874263, 23, 32, 30, }, // America_Mountain
    {  662,  191,  874293,  874321,  874358, 28, 37, 35, }, // America_Pacific
    {  662,  214,  874393,  874403,  874422, 10, 19, 17, }, // Apia
    {  662,  232,  874439,  874453,  874476, 14, 23, 21, }, // Arabian
    {  662,  240,  874497,  874514,  874540, 17, 26, 24, }, // Argentina
    {  662,  250,  874564,  874592,  874629, 28, 37, 35, }, // Argentina_Western
    {  662,  268,  874664,  874679,  874703, 15, 24, 22, }, // Armenia
    {  662,  286,  874725,  874756,  874796, 31, 40, 38, }, // Atlantic
    {  662,  295,  874834,  874859,  874893, 25, 34, 32, }, // Australia_Central
    {  662,  313,  874925,  874958,  875000, 33, 42, 40, }, // Australia_CentralWestern
    {  662,  338,  875040,  875068,  875105, 28, 37, 35, }, // Australia_Eastern
    {  662,  356,  875140,  875168,  875205, 28, 37, 35, }, // Australia_Western
    {  662,  374,  875240,  875259,  875287, 19, 28, 26, }, // Azerbaijan
    {  662,  385,  875313,  875324,  875344, 11, 20, 18, }, // Azores
    {  662,  397,  875362,  875378,  875403, 16, 25, 23, }, // Bangladesh
    {  662,  415,       0,  875426,       0,  0, 12,  0, }, // Bhutan
    {  662,  422,       0,  875438,       0,  0, 13,  0, }, // Bolivia
    {  662,  437,  875451,  875465,  875488, 14, 23, 21, }, // Brasilia
    {  662,  454,       0,  875509,       0,  0, 23,  0, }, // Brunei
    {  662,  461,  875532,  875549,  875575, 17, 26, 24, }, // Cape_Verde
    {  662,  478,       0,  875599,       0,  0, 14,  0, }, // Chamorro
    {  662,  487,  875613,  875626,  875648, 13, 22, 20, }, // Chatham
    {  662,  495,  875668,  875678,  875697, 10, 19, 17, }, // Chile
    {  662,  501,  875714,  875724,  875743, 10, 19, 17, }, // China
    {  662,  507,  875760,  875776,  875801, 16, 25, 23, }, // Choibalsan
    {  662,  518,       0,  875824,       0,  0, 22,  0, }, // Christmas
    {  662,  528,       0,  875846,       0,  0, 16,  0, }, // Cocos
    {  662,  534,  875862,  875876,  875899, 14, 23, 21, }, // Colombia
    {  662,  543,  875920,  875935,  875959, 15, 24, 22, }, // Cook
    {  662,  548,  875981,  875992,  876012, 11, 20, 18, }, // Cuba
    {  662,  559,       0,  876030,       0,  0, 11,  0, }, // Davis
    {  662,  575,       0,  876041,       0,  0, 22,  0, }, // DumontDUrville
    {  662,  612,       0,  876063,       0,  0, 15,  0, }, // East_Timor
    {  662,  623,  876078,  876100,  876131, 22, 31, 29, }, // Easter
    {  662,  630,       0,  876160,       0,  0, 15,  0, }, // Ecuador
    {  662,  638,  876175,  876197,  876228, 22, 31, 29, }, // Europe_Central
    {  662,  653,  876257,  876282,  876316, 25, 34, 32, }, // Europe_Eastern
    {  662,  668,       0,  876348,       0,  0, 17,  0, }, // Europe_Further_Eastern
    {  662,  691,  876365,  876390,  876424, 25, 34, 32, }, // Europe_Western
    {  662,  706,  876456,  876474,  876501, 18, 27, 25, }, // Falkland
    {  662,  715,  876526,  876536,  876555, 10, 19, 17, }, // Fiji
    {  662,  720,       0,  876572,       0,  0, 22,  0, }, // French_Guiana
    {  662,  734,       0,  876594,       0,  0, 42,  0, }, // French_Southern
    {  662,  757,       0,  876636,       0,  0, 15,  0, }, // Galapagos
    {  662,  767,       0,  876651,       0,  0, 13,  0, }, // Gambier
    {  662,  775,  876664,  876677,  876699, 13, 22, 20, }, // Georgia
    {  662,  783,       0,  876719,       0,  0, 18,  0, }, // Gilbert_Islands
    {  662,  799,       0,  876737,       0,  0, 27,  0, }, // GMT
    {  662,  841,  876764,  876792,  876829, 28, 37, 35, }, // Greenland_Eastern
    {  662,  859,  876864,  876892,  876929, 28, 37, 35, }, // Greenland_Western
    {  662,  882,       0,  876964,       0,  0, 20,  0, }, // Gulf
    {  662,  887,       0,  876572,       0,  0, 12,  0, }, // Guyana
    {  662,  894,  876984,  877005,  877035, 21, 30, 28, }, // Hawaii_Aleutian
    {  662,  910,  877063,  877078,  877102, 15, 24, 22, }, // Hong_Kong
    {  662,  920,  877124,  877134,  877153, 10, 19, 17, }, // Hovd
    {  662,  925,       0,  877170,       0,  0, 13,  0, }, // India
    {  662,  931,       0,  877183,       0,  0, 21,  0, }, // Indian_Ocean
    {  662,  944,       0,  877204,       0,  0, 16,  0, }, // Indochina
    {  662,  954,       0,  877220,       0,  0, 25,  0, }, // Indonesia_Central
    {  662,  972,       0,  877245,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  662,  990,       0,  877273,       0,  0, 28,  0, }, // Indonesia_Western
    {  662, 1008,  877301,  877313,  877334, 12, 21, 19, }, // Iran
    {  662, 1019,  877353,  877366,  877388, 13, 22, 20, }, // Irkutsk
    {  662, 1027,  877408,  877420,  877441, 12, 21, 19, }, // Israel
    {  662, 1034,  877460,  877473,  877495, 13, 22, 20, }, // Japan
    {  662, 1058,       0,  877515,       0,  0, 27,  0, }, // Kazakhstan_Eastern
    {  662, 1077,       0,  877542,       0,  0, 27,  0, }, // Kazakhstan_Western
    {  662, 1106,  877569,  877580,  877600, 11, 20, 18, }, // Korea
    {  662, 1112,       0,  877618,       0,  0, 12,  0, }, // Kosrae
    {  662, 1119,  877630,  877647,  877673, 17, 26, 24, }, // Krasnoyarsk
    {  662, 1151,       0,  877697,       0,  0, 18,  0, }, // Kyrgystan
    {  662, 1175,       0,  877715,       0,  0, 19,  0, }, // Line_Islands
    {  662, 1188,  877734,  877749,  877773, 15, 24, 22, }, // Lord_Howe
    {  662, 1204,  877795,  877808,  877830, 13, 22, 20, }, // Magadan
    {  662, 1219,       0,  877850,       0,  0, 13,  0, }, // Malaysia
    {  662, 1228,       0,  877863,       0,  0, 13,  0, }, // Maldives
    {  662, 1237,       0,  877876,       0,  0, 15,  0, }, // Marquesas
    {  662, 1247,       0,  877891,       0,  0, 19,  0, }, // Marshall_Islands
    {  662, 1264,  877910,  877925,  877949, 15, 24, 22, }, // Mauritius
    {  662, 1274,       0,  877971,       0,  0, 12,  0, }, // Mawson
    {  662, 1281,  877983,  878007,  878040, 24, 33, 31, }, // Mexico_Pacific
    {  662, 1296,  878071,  878086,  878110, 15, 24, 22, }, // Mongolia
    {  662, 1305,  878132,  878143,  878163, 11, 20, 18, }, // Moscow
    {  662, 1312,       0,  878181,       0,  0, 14,  0, }, // Myanmar
    {  662, 1320,       0,  878195,       0,  0, 11,  0, }, // Nauru
    {  662, 1326,       0,  878206,       0,  0, 11,  0, }, // Nepal
    {  662, 1332,  878217,  878238,  878268, 21, 30, 28, }, // New_Caledonia
    {  662, 1346,  878296,  878315,  878343, 19, 28, 26, }, // New_Zealand
    {  662, 1358,  878369,  878384,  878408, 15, 24, 22, }, // Newfoundland
    {  662, 1371,       0,  878430,       0,  0, 10,  0, }, // Niue
    {  662, 1376,  878440,  878460,  878489, 20, 29, 27, }, // Norfolk
    {  662, 1384,  878516,  878541,  878575, 25, 34, 32, }, // Noronha
    {  662, 1406,  878607,  878624,  878650, 17, 26, 24, }, // Novosibirsk
    {  662, 1418,  878674,  878684,  878703, 10, 19, 17, }, // Omsk
    {  662, 1428,  878720,  878734,  878757, 14, 23, 21, }, // Pakistan
    {  662, 1437,       0,  878778,       0,  0, 11,  0, }, // Palau
    {  662, 1443,       0,  878789,       0,  0, 24,  0, }, // Papua_New_Guinea
    {  662, 1460,  878813,  878827,  878850, 14, 23, 21, }, // Paraguay
    {  662, 1469,  878871,  878881,  878900, 10, 19, 17, }, // Peru
    {  662, 1474,  878917,  878931,  878954, 14, 23, 21, }, // Philippines
    {  662, 1486,       0,  878975,       0,  0, 18,  0, }, // Phoenix_Islands
    {  662, 1502,  878993,  879016,  879048, 23, 32, 30, }, // Pierre_Miquelon
    {  662, 1518,       0,  879078,       0,  0, 14,  0, }, // Pitcairn
    {  662, 1527,       0,  879092,       0,  0, 13,  0, }, // Ponape
    {  662, 1534,       0,  879105,       0,  0, 15,  0, }, // Pyongyang
    {  662, 1554,       0,  879120,       0,  0, 13,  0, }, // Reunion
    {  662, 1562,       0,  879133,       0,  0, 13,  0, }, // Rothera
    {  662, 1570,  879146,  879160,  879183, 14, 23, 21, }, // Sakhalin
    {  662, 1596,  879204,  879215,  879235, 11, 20, 18, }, // Samoa
    {  662, 1602,       0,  879253,       0,  0, 16,  0, }, // Seychelles
    {  662, 1624,       0,  879269,       0,  0, 14,  0, }, // Singapore
    {  662, 1634,       0,  879283,       0,  0, 18,  0, }, // Solomon
    {  662, 1642,       0,  879301,       0,  0, 20,  0, }, // South_Georgia
    {  662, 1656,       0,  879321,       0,  0, 14,  0, }, // Suriname
    {  662, 1676,       0,  879335,       0,  0, 11,  0, }, // Syowa
    {  662, 1682,       0,  879346,       0,  0, 12,  0, }, // Tahiti
    {  662, 1689,  879358,  879370,  879391, 12, 21, 19, }, // Taipei
    {  662, 1696,       0,  879410,       0,  0, 16,  0, }, // Tajikistan
    {  662, 1724,       0,  879426,       0,  0, 13,  0, }, // Tokelau
    {  662, 1732,  879439,  879450,  879470, 11, 20, 18, }, // Tonga
    {  662, 1738,       0,  879488,       0,  0, 11,  0, }, // Truk
    {  662, 1750,  879499,  879517,  879544, 18, 27, 25, }, // Turkmenistan
    {  662, 1763,       0,  879569,       0,  0, 12,  0, }, // Tuvalu
    {  662, 1777,  879581,  879596,  879620, 15, 24, 22, }, // Uruguay
    {  662, 1792,  879642,  879660,  879687, 18, 27, 25, }, // Uzbekistan
    {  662, 1803,  879712,  879725,  879747, 13, 22, 20, }, // Vanuatu
    {  662, 1811,       0,  879767,       0,  0, 16,  0, }, // Venezuela
    {  662, 1821,  879783,  879800,  879826, 17, 26, 24, }, // Vladivostok
    {  662, 1833,  879850,  879865,  879889, 15, 24, 22, }, // Volgograd
    {  662, 1843,       0,  879911,       0,  0, 12,  0, }, // Vostok
    {  662, 1850,       0,  879923,       0,  0, 20,  0, }, // Wake
    {  662, 1855,       0,  879943,       0,  0, 21,  0, }, // Wallis
    {  662, 1862,  879964,  879977,  879999, 13, 22, 20, }, // Yakutsk
    {  662, 1870,  880019,  880036,  880062, 17, 26, 24, }, // Yekaterinburg
    {  662, 1892,       0,  880086,       0,  0, 11,  0, }, // Yukon
    {  665,    5,       0,  880097,       0,  0, 16,  0, }, // Afghanistan Torwali/Arabic/Pakistan
    {  665,   17,       0,  790107,       0,  0, 16,  0, }, // Africa_Central
    {  665,   32,       0,  790123,       0,  0, 17,  0, }, // Africa_Eastern
    {  665,   65,       0,  790140,       0,  0, 25,  0, }, // Africa_Southern
    {  665,   81,  790165,  790182,  790207, 17, 25, 21, }, // Africa_Western
    {  665,  107,  790228,  790239,  790259, 11, 20, 19, }, // Alaska
    {  665,  135,  790278,  880113,  880134, 11, 21, 26, }, // Amazon
    {  665,  142,  790336,  790346,  790365, 10, 19, 18, }, // America_Central
    {  665,  158,  790383,  790394,  790414, 11, 20, 19, }, // America_Eastern
    {  665,  174,  790433,  790445,  790466, 12, 21, 20, }, // America_Mountain
    {  665,  191,  790486,  790496,  790515, 10, 19, 18, }, // America_Pacific
    {  665,  214,  790576,  790586,  790604, 10, 18, 18, }, // Apia
    {  665,  232,  880160,  880170,  790649, 10, 17, 16, }, // Arabian
    {  665,  240,  880187,  880202,  880224, 15, 22, 28, }, // Argentina
    {  665,  250,  880252,  880273,  880301, 21, 28, 34, }, // Argentina_Western
    {  665,  268,  880335,  880349,  880370, 14, 21, 27, }, // Armenia
    {  665,  286,  790875,  790887,  790908, 12, 21, 20, }, // Atlantic
    {  665,  295,  790928,  790947,  790975, 19, 28, 27, }, // Australia_Central
    {  665,  313,  791002,  791028,  791063, 26, 35, 34, }, // Australia_CentralWestern
    {  665,  338,  791097,  791117,  791146, 20, 29, 28, }, // Australia_Eastern
    {  665,  356,  791174,  791195,  791223, 20, 28, 28, }, // Australia_Western
    {  665,  374,  880397,  880414,  880438, 17, 24, 30, }, // Azerbaijan
    {  665,  385,  880468,  880481,  880501, 13, 20, 26, }, // Azores
    {  665,  397,  880527,  880543,  880566, 16, 23, 29, }, // Bangladesh
    {  665,  415,       0,  880595,       0,  0, 13,  0, }, // Bhutan
    {  665,  422,       0,  880608,       0,  0, 14,  0, }, // Bolivia
    {  665,  437,  791476,  791489,  791511, 13, 22, 17, }, // Brasilia
    {  665,  454,       0,  791528,       0,  0, 21,  0, }, // Brunei
    {  665,  461,  791549,  791562,  791583, 13, 21, 17, }, // Cape_Verde
    {  665,  478,       0,  791600,       0,  0, 19,  0, }, // Chamorro
    {  665,  487,  791619,  791629,  791648, 10, 19, 18, }, // Chatham
    {  665,  495,  880622,  880632,  880649, 10, 17, 23, }, // Chile
    {  665,  501,  880672,  791726,  791742, 10, 16, 17, }, // China
    {  665,  507,  791759,  791772,  791793, 13, 21, 18, }, // Choibalsan
    {  665,  518,       0,  791811,       0,  0, 17,  0, }, // Christmas
    {  665,  528,       0,  791828,       0,  0, 18,  0, }, // Cocos
    {  665,  534,  791846,  880682,  880703, 12, 21, 27, }, // Colombia
    {  665,  543,  791906,  791921,  791944, 15, 23, 23, }, // Cook
    {  665,  548,  791967,  791977,  791996, 10, 19, 18, }, // Cuba
    {  665,  559,       0,  792014,       0,  0,  9,  0, }, // Davis
    {  665,  575,       0,  792023,       0,  0, 21,  0, }, // DumontDUrville
    {  665,  612,       0,  792044,       0,  0, 16,  0, }, // East_Timor
    {  665,  623,  880730,  880749,  880775, 19, 26, 32, }, // Easter
    {  665,  630,       0,  880807,       0,  0, 15,  0, }, // Ecuador
    {  665,  638,  880822,  880837,  880860, 15, 23, 29, }, // Europe_Central
    {  665,  653,  880889,  880906,  880930, 17, 24, 30, }, // Europe_Eastern
    {  665,  668,       0,  880960,       0,  0, 20,  0, }, // Europe_Further_Eastern
    {  665,  691,  880980,  880997,  881021, 17, 24, 30, }, // Europe_Western
    {  665,  706,  881051,  881074,  881104, 23, 30, 36, }, // Falkland
    {  665,  715,  792470,  792478,  792494,  8, 16, 12, }, // Fiji
    {  665,  720,       0,  881140,       0,  0, 18,  0, }, // French_Guiana
    {  665,  734,       0,  881158,       0,  0, 28,  0, }, // French_Southern
    {  665,  757,       0,  881186,       0,  0, 16,  0, }, // Galapagos
    {  665,  767,       0,  792569,       0,  0, 12,  0, }, // Gambier
    {  665,  775,  881202,  881215,  881235, 13, 20, 26, }, // Georgia
    {  665,  783,       0,  792640,       0,  0, 18,  0, }, // Gilbert_Islands
    {  665,  799,       0,  881261,       0,  0, 18,  0, }, // GMT
    {  665,  841,  792676,  792696,  881279, 20, 29, 35, }, // Greenland_Eastern
    {  665,  859,  792760,  792780,  881314, 20, 29, 35, }, // Greenland_Western
    {  665,  882,       0,  881349,       0,  0, 18,  0, }, // Gulf
    {  665,  887,       0,  881146,       0,  0, 12,  0, }, // Guyana
    {  665,  894,  792862,  792881,  792909, 19, 28, 27, }, // Hawaii_Aleutian
    {  665,  910,  881367,  881381,  881403, 14, 22, 18, }, // Hong_Kong
    {  665,  920,  792954,  792963,  792980,  9, 17, 13, }, // Hovd
    {  665,  925,       0,  881421,       0,  0, 22,  0, }, // India
    {  665,  931,       0,  793015,       0,  0, 12,  0, }, // Indian_Ocean
    {  665,  944,       0,  793027,       0,  0, 12,  0, }, // Indochina
    {  665,  954,       0,  793039,       0,  0, 19,  0, }, // Indonesia_Central
    {  665,  972,       0,  793058,       0,  0, 20,  0, }, // Indonesia_Eastern
    {  665,  990,       0,  793078,       0,  0, 20,  0, }, // Indonesia_Western
    {  665, 1008,  881443,  881455,  793129, 12, 19, 18, }, // Iran
    {  665, 1019,  793147,  793158,  793177, 11, 19, 15, }, // Irkutsk
    {  665, 1027,  881474,  881488,  793227, 14, 21, 20, }, // Israel
    {  665, 1034,  793247,  793257,  793275, 10, 18, 18, }, // Japan
    {  665, 1058,       0,  881509,       0,  0, 21,  0, }, // Kazakhstan_Eastern
    {  665, 1077,       0,  881530,       0,  0, 21,  0, }, // Kazakhstan_Western
    {  665, 1106,  793426,  793436,  793454, 10, 18, 18, }, // Korea
    {  665, 1112,       0,  793472,       0,  0, 10,  0, }, // Kosrae
    {  665, 1119,  793482,  793498,  793521, 16, 23, 20, }, // Krasnoyarsk
    {  665, 1151,       0,  881551,       0,  0, 14,  0, }, // Kyrgystan
    {  665, 1175,       0,  793555,       0,  0, 17,  0, }, // Line_Islands
    {  665, 1188,  793572,  793586,  793609, 14, 23, 22, }, // Lord_Howe
    {  665, 1204,  793631,  793642,  793662, 11, 20, 15, }, // Magadan
    {  665, 1219,       0,  793677,       0,  0, 11,  0, }, // Malaysia
    {  665, 1228,       0,  881565,       0,  0, 13,  0, }, // Maldives
    {  665, 1237,       0,  793701,       0,  0, 12,  0, }, // Marquesas
    {  665, 1247,       0,  793713,       0,  0, 18,  0, }, // Marshall_Islands
    {  665, 1264,  793731,  793742,  793761, 11, 19, 15, }, // Mauritius
    {  665, 1274,       0,  793776,       0,  0, 10,  0, }, // Mawson
    {  665, 1281,  793786,  793804,  793831, 18, 27, 26, }, // Mexico_Pacific
    {  665, 1296,  793857,  793873,  793897, 16, 24, 20, }, // Mongolia
    {  665, 1305,  793917,  793927,  793946, 10, 19, 14, }, // Moscow
    {  665, 1312,       0,  793960,       0,  0, 12,  0, }, // Myanmar
    {  665, 1320,       0,  793972,       0,  0, 10,  0, }, // Nauru
    {  665, 1326,       0,  881578,       0,  0, 12,  0, }, // Nepal
    {  665, 1332,  793994,  794012,  794038, 18, 26, 22, }, // New_Caledonia
    {  665, 1346,  881590,  881607,  794101, 17, 24, 23, }, // New_Zealand
    {  665, 1358,  411137,  794124,  411183, 19, 28, 27, }, // Newfoundland
    {  665, 1371,       0,  794152,       0,  0,  9,  0, }, // Niue
    {  665, 1376,  881631,  881651,  881678, 20, 27, 33, }, // Norfolk
    {  665, 1384,  881711,  881735,  794288, 24, 31, 27, }, // Noronha
    {  665, 1406,  794315,  794330,  794353, 15, 23, 19, }, // Novosibirsk
    {  665, 1418,  794372,  794382,  794400, 10, 18, 14, }, // Omsk
    {  665, 1428,  881766,  881780,  881801, 14, 21, 27, }, // Pakistan
    {  665, 1437,       0,  794476,       0,  0,  9,  0, }, // Palau
    {  665, 1443,       0,  794485,       0,  0, 18,  0, }, // Papua_New_Guinea
    {  665, 1460,  881828,  881843,  881865, 15, 22, 28, }, // Paraguay
    {  665, 1469,  881893,  881904,  881922, 11, 18, 24, }, // Peru
    {  665, 1474,  794621,  794632,  794651, 11, 19, 15, }, // Philippines
    {  665, 1486,       0,  794666,       0,  0, 18,  0, }, // Phoenix_Islands
    {  665, 1502,  881946,  881971,  882005, 25, 34, 33, }, // Pierre_Miquelon
    {  665, 1518,       0,  794779,       0,  0, 12,  0, }, // Pitcairn
    {  665, 1527,       0,  794791,       0,  0, 11,  0, }, // Ponape
    {  665, 1534,       0,  882038,       0,  0, 14,  0, }, // Pyongyang
    {  665, 1554,       0,  794816,       0,  0, 13,  0, }, // Reunion
    {  665, 1562,       0,  882052,       0,  0, 14,  0, }, // Rothera
    {  665, 1570,  794843,  794854,  794873, 11, 19, 15, }, // Sakhalin
    {  665, 1596,  794931,  794941,  794959, 10, 18, 18, }, // Samoa
    {  665, 1602,       0,  794977,       0,  0, 11,  0, }, // Seychelles
    {  665, 1624,       0,  794988,       0,  0, 20,  0, }, // Singapore
    {  665, 1634,       0,  795008,       0,  0, 18,  0, }, // Solomon
    {  665, 1642,       0,  795026,       0,  0, 17,  0, }, // South_Georgia
    {  665, 1656,       0,  882066,       0,  0, 14,  0, }, // Suriname
    {  665, 1676,       0,  795057,       0,  0, 10,  0, }, // Syowa
    {  665, 1682,       0,  795067,       0,  0, 11,  0, }, // Tahiti
    {  665, 1689,  795078,  795092,  795115, 14, 23, 21, }, // Taipei
    {  665, 1696,       0,  882080,       0,  0, 15,  0, }, // Tajikistan
    {  665, 1724,       0,  795151,       0,  0, 12,  0, }, // Tokelau
    {  665, 1732,  795163,  795173,  795191, 10, 18, 14, }, // Tonga
    {  665, 1738,       0,  795205,       0,  0,  8,  0, }, // Truk
    {  665, 1750,  882095,  882112,  882136, 17, 24, 30, }, // Turkmenistan
    {  665, 1763,       0,  795284,       0,  0, 10,  0, }, // Tuvalu
    {  665, 1777,  882166,  882181,  882203, 15, 22, 28, }, // Uruguay
    {  665, 1792,  882231,  882246,  882268, 15, 22, 28, }, // Uzbekistan
    {  665, 1803,  795424,  795436,  795456, 12, 20, 16, }, // Vanuatu
    {  665, 1811,       0,  882296,       0,  0, 16,  0, }, // Venezuela
    {  665, 1821,  795488,  795504,  795528, 16, 24, 20, }, // Vladivostok
    {  665, 1833,  795548,  795562,  795585, 14, 23, 18, }, // Volgograd
    {  665, 1843,       0,  882312,       0,  0, 13,  0, }, // Vostok
    {  665, 1850,       0,  795616,       0,  0, 15,  0, }, // Wake
    {  665, 1855,       0,  795631,       0,  0, 20,  0, }, // Wallis
    {  665, 1862,  795651,  795662,  795682, 11, 20, 15, }, // Yakutsk
    {  665, 1870,  795697,  795713,  795738, 16, 25, 20, }, // Yekaterinburg
    {  665, 1892,       0,  412940,       0,  0, 10,  0, }, // Yukon
    {  666,    5,       0,  882325,       0,  0, 22,  0, }, // Afghanistan Anii/Latin/Benin
    {  666,   17,       0,  882347,       0,  0, 31,  0, }, // Africa_Central
    {  666,   32,       0,  882378,       0,  0, 33,  0, }, // Africa_Eastern
    {  666,   65,       0,  882411,       0,  0, 36,  0, }, // Africa_Southern
    {  666,   81,  882447,  882484,  882528, 37, 44, 46, }, // Africa_Western
    {  666,  107,  882574,  882593,  882619, 19, 26, 28, }, // Alaska
    {  666,  135,  882647,  882665,  882690, 18, 25, 27, }, // Amazon
    {  666,  142,  882717,  882759,  882808, 42, 49, 51, }, // America_Central
    {  666,  158,  882859,  882902,  882952, 43, 50, 52, }, // America_Eastern
    {  666,  174,  883004,  883043,  883089, 39, 46, 48, }, // America_Mountain
    {  666,  191,  883137,  883167,  883204, 30, 37, 39, }, // America_Pacific
    {  666,  214,  883243,  883260,  883284, 17, 24, 26, }, // Apia
    {  666,  232,  883310,  883335,  883367, 25, 32, 34, }, // Arabian
    {  666,  240,  883401,  883421,  883448, 20, 27, 29, }, // Argentina
    {  666,  250,  883477,  883511,  883552, 34, 41, 43, }, // Argentina_Western
    {  666,  268,  883595,  883614,  883640, 19, 26, 28, }, // Armenia
    {  666,  286,  883668,  883699,  883737, 31, 38, 40, }, // Atlantic
    {  666,  295,  883777,  883808,  883846, 31, 38, 40, }, // Australia_Central
    {  666,  313,  883886,  883931,  883983, 45, 52, 54, }, // Australia_CentralWestern
    {  666,  338,  884037,  884069,  884108, 32, 39, 41, }, // Australia_Eastern
    {  666,  356,  884149,  884186,  884230, 37, 44, 46, }, // Australia_Western
    {  666,  374,  884276,  884298,  884327, 22, 29, 31, }, // Azerbaijan
    {  666,  385,  884358,  884375,  884399, 17, 24, 26, }, // Azores
    {  666,  397,  884425,  884449,  884480, 24, 31, 33, }, // Bangladesh
    {  666,  415,       0,  884513,       0,  0, 17,  0, }, // Bhutan
    {  666,  422,       0,  884530,       0,  0, 20,  0, }, // Bolivia
    {  666,  437,  884550,  884571,  884599, 21, 28, 30, }, // Brasilia
    {  666,  454,       0,  884629,       0,  0, 29,  0, }, // Brunei
    {  666,  461,  884658,  884677,  884703, 19, 26, 28, }, // Cape_Verde
    {  666,  478,       0,  884731,       0,  0, 20,  0, }, // Chamorro
    {  666,  487,  884751,  884769,  884794, 18, 25, 27, }, // Chatham
    {  666,  495,  884821,  884839,  884864, 18, 25, 27, }, // Chile
    {  666,  501,  884891,  884908,  884932, 17, 24, 26, }, // China
    {  666,  507,  884958,  884980,  885009, 22, 29, 31, }, // Choibalsan
    {  666,  518,       0,  885040,       0,  0, 30,  0, }, // Christmas
    {  666,  528,       0,  885070,       0,  0, 32,  0, }, // Cocos
    {  666,  534,  885102,  885122,  885149, 20, 27, 29, }, // Colombia
    {  666,  543,  885178,  885208,  885245, 30, 37, 39, }, // Cook
    {  666,  548,  885284,  885301,  885325, 17, 24, 26, }, // Cuba
    {  666,  559,       0,  885351,       0,  0, 17,  0, }, // Davis
    {  666,  575,       0,  885368,       0,  0, 24,  0, }, // DumontDUrville
    {  666,  612,       0,  885392,       0,  0, 27,  0, }, // East_Timor
    {  666,  623,  885419,  885450,  885487, 31, 37, 39, }, // Easter
    {  666,  630,       0,  885526,       0,  0, 21,  0, }, // Ecuador
    {  666,  638,  885547,  885578,  885616, 31, 38, 40, }, // Europe_Central
    {  666,  653,  885656,  885689,  885729, 33, 40, 42, }, // Europe_Eastern
    {  666,  668,       0,  885771,       0,  0, 44,  0, }, // Europe_Further_Eastern
    {  666,  691,  885815,  885852,  885896, 37, 44, 46, }, // Europe_Western
    {  666,  706,  885942,  885975,  886015, 33, 40, 42, }, // Falkland
    {  666,  715,  886057,  886073,  886096, 16, 23, 25, }, // Fiji
    {  666,  720,       0,  886121,       0,  0, 34,  0, }, // French_Guiana
    {  666,  734,       0,  886155,       0,  0, 49,  0, }, // French_Southern
    {  666,  757,       0,  886204,       0,  0, 21,  0, }, // Galapagos
    {  666,  767,       0,  886225,       0,  0, 20,  0, }, // Gambier
    {  666,  775,  886245,  886264,  886290, 19, 26, 28, }, // Georgia
    {  666,  783,       0,  886318,       0,  0, 33,  0, }, // Gilbert_Islands
    {  666,  799,       0,  886351,       0,  0, 21,  0, }, // GMT
    {  666,  841,  886372,  886402,  886439, 30, 37, 39, }, // Greenland_Eastern
    {  666,  859,  886478,  886512,  886553, 34, 41, 43, }, // Greenland_Western
    {  666,  877,       0,  886596,       0,  0, 17,  0, }, // Guam
    {  666,  882,       0,  886613,       0,  0, 16,  0, }, // Gulf
    {  666,  887,       0,  886629,       0,  0, 19,  0, }, // Guyana
    {  666,  894,  886648,  886677,  886713, 29, 36, 38, }, // Hawaii_Aleutian
    {  666,  910,  886751,  886770,  886796, 19, 26, 28, }, // Hong_Kong
    {  666,  920,  886824,  886841,  886865, 17, 24, 26, }, // Hovd
    {  666,  925,       0,  886891,       0,  0, 18,  0, }, // India
    {  666,  931,       0,  886909,       0,  0, 26,  0, }, // Indian_Ocean
    {  666,  944,       0,  886935,       0,  0, 21,  0, }, // Indochina
    {  666,  954,       0,  886956,       0,  0, 31,  0, }, // Indonesia_Central
    {  666,  972,       0,  886987,       0,  0, 32,  0, }, // Indonesia_Eastern
    {  666,  990,       0,  887019,       0,  0, 37,  0, }, // Indonesia_Western
    {  666, 1008,  887056,  887072,  887095, 16, 23, 25, }, // Iran
    {  666, 1019,  887120,  887137,  887161, 17, 24, 26, }, // Irkutsk
    {  666, 1027,  887187,  887207,  887234, 20, 27, 29, }, // Israel
    {  666, 1034,  887263,  887280,  887304, 17, 24, 26, }, // Japan
    {  666, 1058,       0,  887330,       0,  0, 30,  0, }, // Kazakhstan_Eastern
    {  666, 1077,       0,  887360,       0,  0, 34,  0, }, // Kazakhstan_Western
    {  666, 1106,  887394,  887411,  887435, 17, 24, 26, }, // Korea
    {  666, 1112,       0,  887461,       0,  0, 18,  0, }, // Kosrae
    {  666, 1119,  887479,  887501,  887530, 22, 29, 31, }, // Krasnoyarsk
    {  666, 1151,       0,  887561,       0,  0, 21,  0, }, // Kyrgystan
    {  666, 1161,       0,  887582,       0,  0, 23,  0, }, // Lanka
    {  666, 1175,       0,  887605,       0,  0, 30,  0, }, // Line_Islands
    {  666, 1188,  887635,  887655,  887682, 20, 27, 29, }, // Lord_Howe
    {  666, 1198,  887711,  887730,  887756, 19, 26, 27, }, // Macau
    {  666, 1204,  887783,  887802,  887828, 19, 26, 28, }, // Magadan
    {  666, 1219,       0,  887856,       0,  0, 19,  0, }, // Malaysia
    {  666, 1228,       0,  887875,       0,  0, 20,  0, }, // Maldives
    {  666, 1237,       0,  887895,       0,  0, 20,  0, }, // Marquesas
    {  666, 1247,       0,  887915,       0,  0, 33,  0, }, // Marshall_Islands
    {  666, 1264,  887948,  887966,  887991, 18, 25, 27, }, // Mauritius
    {  666, 1274,       0,  888018,       0,  0, 17,  0, }, // Mawson
    {  666, 1281,  888035,  888063,  888098, 28, 35, 37, }, // Mexico_Pacific
    {  666, 1296,  888135,  888157,  888186, 22, 29, 31, }, // Mongolia
    {  666, 1305,  888217,  888235,  888260, 18, 25, 27, }, // Moscow
    {  666, 1312,       0,  888287,       0,  0, 20,  0, }, // Myanmar
    {  666, 1320,       0,  888307,       0,  0, 18,  0, }, // Nauru
    {  666, 1326,       0,  888325,       0,  0, 18,  0, }, // Nepal
    {  666, 1332,  888343,  888370,  888404, 27, 34, 36, }, // New_Caledonia
    {  666, 1346,  888440,  888464,  888495, 24, 31, 33, }, // New_Zealand
    {  666, 1358,  888528,  888555,  888589, 27, 34, 36, }, // Newfoundland
    {  666, 1371,       0,  888625,       0,  0, 18,  0, }, // Niue
    {  666, 1376,  888643,  888675,  888714, 32, 39, 41, }, // Norfolk
    {  666, 1384,  888755,  888786,  888824, 31, 38, 40, }, // Noronha
    {  666, 1392,       0,  888864,       0,  0, 46,  0, }, // North_Mariana
    {  666, 1406,  888910,  888932,  888961, 22, 29, 31, }, // Novosibirsk
    {  666, 1418,  888992,  889010,  889035, 18, 25, 27, }, // Omsk
    {  666, 1428,  889062,  889082,  889109, 20, 27, 29, }, // Pakistan
    {  666, 1437,       0,  889138,       0,  0, 19,  0, }, // Palau
    {  666, 1443,       0,  889157,       0,  0, 32,  0, }, // Papua_New_Guinea
    {  666, 1460,  889189,  889210,  889238, 21, 28, 30, }, // Paraguay
    {  666, 1469,  889268,  889285,  889309, 17, 24, 26, }, // Peru
    {  666, 1474,  889335,  889355,  889382, 20, 27, 29, }, // Philippines
    {  666, 1486,       0,  889411,       0,  0, 33,  0, }, // Phoenix_Islands
    {  666, 1502,  889444,  889477,  889517, 33, 40, 42, }, // Pierre_Miquelon
    {  666, 1518,       0,  889559,       0,  0, 33,  0, }, // Pitcairn
    {  666, 1527,       0,  889592,       0,  0, 19,  0, }, // Ponape
    {  666, 1534,       0,  889611,       0,  0, 20,  0, }, // Pyongyang
    {  666, 1554,       0,  889631,       0,  0, 20,  0, }, // Reunion
    {  666, 1562,       0,  889651,       0,  0, 19,  0, }, // Rothera
    {  666, 1570,  889670,  889689,  889715, 19, 26, 28, }, // Sakhalin
    {  666, 1596,  889743,  889761,  889786, 18, 25, 27, }, // Samoa
    {  666, 1602,       0,  889813,       0,  0, 18,  0, }, // Seychelles
    {  666, 1624,       0,  889831,       0,  0, 21,  0, }, // Singapore
    {  666, 1634,       0,  889852,       0,  0, 34,  0, }, // Solomon
    {  666, 1642,       0,  889886,       0,  0, 29,  0, }, // South_Georgia
    {  666, 1656,       0,  889915,       0,  0, 19,  0, }, // Suriname
    {  666, 1676,       0,  889934,       0,  0, 19,  0, }, // Syowa
    {  666, 1682,       0,  889953,       0,  0, 19,  0, }, // Tahiti
    {  666, 1689,  889972,  889990,  890015, 18, 25, 27, }, // Taipei
    {  666, 1696,       0,  890042,       0,  0, 22,  0, }, // Tajikistan
    {  666, 1724,       0,  890064,       0,  0, 19,  0, }, // Tokelau
    {  666, 1732,  890083,  890100,  890124, 17, 24, 26, }, // Tonga
    {  666, 1738,       0,  890150,       0,  0, 16,  0, }, // Truk
    {  666, 1750,  890166,  890190,  890221, 24, 31, 33, }, // Turkmenistan
    {  666, 1763,       0,  890254,       0,  0, 18,  0, }, // Tuvalu
    {  666, 1777,  890272,  890292,  890319, 20, 27, 29, }, // Uruguay
    {  666, 1792,  890348,  890371,  890401, 23, 30, 32, }, // Uzbekistan
    {  666, 1803,  890433,  890453,  890480, 20, 27, 29, }, // Vanuatu
    {  666, 1811,       0,  890509,       0,  0, 23,  0, }, // Venezuela
    {  666, 1821,  890532,  890555,  890585, 23, 30, 32, }, // Vladivostok
    {  666, 1833,  890617,  890639,  890668, 22, 29, 31, }, // Volgograd
    {  666, 1843,       0,  890699,       0,  0, 18,  0, }, // Vostok
    {  666, 1850,       0,  890717,       0,  0, 30,  0, }, // Wake
    {  666, 1855,       0,  890747,       0,  0, 27,  0, }, // Wallis
    {  666, 1862,  890774,  890791,  890815, 17, 24, 26, }, // Yakutsk
    {  666, 1870,  890841,  890865,  890896, 24, 31, 33, }, // Yekaterinburg
    {  666, 1892,       0,  890929,       0,  0, 17,  0, }, // Yukon
    {  667,    5,       0,  890946,       0,  0, 20,  0, }, // Afghanistan Kangri/Devanagari/India
    {  667,   17,       0,  890966,       0,  0, 20,  0, }, // Africa_Central
    {  667,   32,       0,  890986,       0,  0, 21,  0, }, // Africa_Eastern
    {  667,   65,       0,  891007,       0,  0, 24,  0, }, // Africa_Southern
    {  667,   81,  891031,  891053,  891080, 22, 27, 33, }, // Africa_Western
    {  667,  107,  891113,  891127,  891147, 14, 20, 35, }, // Alaska
    {  667,  135,  891182,  891196,  891215, 14, 19, 25, }, // Amazon
    {  667,  142,  891240,  891269,  891303, 29, 34, 49, }, // America_Central
    {  667,  158,  891352,  891375,  891406, 23, 31, 46, }, // America_Eastern
    {  667,  174,  891452,  891479,  891512, 27, 33, 48, }, // America_Mountain
    {  667,  191,  891560,  891590,  891625, 30, 35, 50, }, // America_Pacific
    {  667,  214,  891675,  891686,  891702, 11, 16, 31, }, // Apia
    {  667,  232,  891733,  891743,  891758, 10, 15, 30, }, // Arabian
    {  667,  240,  891788,  891805,  891827, 17, 22, 28, }, // Argentina
    {  667,  250,  891855,  891880,  891910, 25, 30, 36, }, // Argentina_Western
    {  667,  268,  891946,  891962,  891983, 16, 21, 27, }, // Armenia
    {  667,  286,  892010,  892025,  892045, 15, 20, 35, }, // Atlantic
    {  667,  295,  892080,  892104,  892140, 24, 36, 51, }, // Australia_Central
    {  667,  313,  892191,  892228,  892269, 37, 41, 54, }, // Australia_CentralWestern
    {  667,  338,  892323,  892347,  892378, 24, 31, 46, }, // Australia_Eastern
    {  667,  356,  892424,  892450,  892481, 26, 31, 46, }, // Australia_Western
    {  667,  374,  892527,  892543,  892564, 16, 21, 27, }, // Azerbaijan
    {  667,  385,  892591,  892605,  892624, 14, 19, 25, }, // Azores
    {  667,  397,  892649,  892667,  892690, 18, 23, 29, }, // Bangladesh
    {  667,  415,       0,  892719,       0,  0, 13,  0, }, // Bhutan
    {  667,  422,       0,  892732,       0,  0, 15,  0, }, // Bolivia
    {  667,  437,  892747,  892765,  892788, 18, 23, 29, }, // Brasilia
    {  667,  454,       0,  892817,       0,  0, 24,  0, }, // Brunei
    {  667,  461,  892841,  892856,  892876, 15, 20, 26, }, // Cape_Verde
    {  667,  478,       0,  892902,       0,  0, 18,  0, }, // Chamorro
    {  667,  487,  892920,  892931,  892947, 11, 16, 31, }, // Chatham
    {  667,  495,  892978,  892989,  893005, 11, 16, 22, }, // Chile
    {  667,  501,  893027,  893037,  893052, 10, 15, 30, }, // China
    {  667,  507,  893082,  893098,  893119, 16, 21, 27, }, // Choibalsan
    {  667,  518,       0,  893146,       0,  0, 19,  0, }, // Christmas
    {  667,  528,       0,  893165,       0,  0, 28,  0, }, // Cocos
    {  667,  534,  893193,  893208,  893228, 15, 20, 26, }, // Colombia
    {  667,  543,  893254,  893280,  893311, 26, 31, 43, }, // Cook
    {  667,  548,  893354,  893367,  893385, 13, 18, 33, }, // Cuba
    {  667,  559,       0,  893418,       0,  0, 12,  0, }, // Davis
    {  667,  575,       0,  893430,       0,  0, 26,  0, }, // DumontDUrville
    {  667,  612,       0,  893456,       0,  0, 19,  0, }, // East_Timor
    {  667,  623,  893475,  893492,  893514, 17, 22, 28, }, // Easter
    {  667,  630,       0,  893542,       0,  0, 15,  0, }, // Ecuador
    {  667,  638,  893557,  893576,  893600, 19, 24, 30, }, // Europe_Central
    {  667,  653,  893630,  893649,  893673, 19, 24, 30, }, // Europe_Eastern
    {  667,  668,       0,  893703,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  667,  691,  893727,  893748,  893774, 21, 26, 32, }, // Europe_Western
    {  667,  706,  893806,  893837,  893873, 31, 36, 42, }, // Falkland
    {  667,  715,  893915,  893927,  893944, 12, 17, 23, }, // Fiji
    {  667,  720,       0,  893967,       0,  0, 21,  0, }, // French_Guiana
    {  667,  734,       0,  893988,       0,  0, 39,  0, }, // French_Southern
    {  667,  757,       0,  894027,       0,  0, 16,  0, }, // Galapagos
    {  667,  767,       0,  894043,       0,  0, 14,  0, }, // Gambier
    {  667,  775,  894057,  894072,  894092, 15, 20, 26, }, // Georgia
    {  667,  783,       0,  894118,       0,  0, 31,  0, }, // Gilbert_Islands
    {  667,  799,       0,  894149,       0,  0, 16,  0, }, // GMT
    {  667,  841,  894165,  894184,  894211, 19, 27, 33, }, // Greenland_Eastern
    {  667,  859,  894244,  894265,  894294, 21, 29, 35, }, // Greenland_Western
    {  667,  882,       0,  894329,       0,  0, 17,  0, }, // Gulf
    {  667,  887,       0,  893975,       0,  0, 13,  0, }, // Guyana
    {  667,  894,  894346,  894365,  894389, 19, 24, 39, }, // Hawaii_Aleutian
    {  667,  910,  894428,  894444,  894465, 16, 21, 27, }, // Hong_Kong
    {  667,  920,  894492,  894504,  894521, 12, 17, 23, }, // Hovd
    {  667,  925,       0,  894544,       0,  0, 17,  0, }, // India
    {  667,  931,       0,  894561,       0,  0, 19,  0, }, // Indian_Ocean
    {  667,  944,       0,  894580,       0,  0, 16,  0, }, // Indochina
    {  667,  954,       0,  894596,       0,  0, 23,  0, }, // Indonesia_Central
    {  667,  972,       0,  894619,       0,  0, 23,  0, }, // Indonesia_Eastern
    {  667,  990,       0,  894642,       0,  0, 25,  0, }, // Indonesia_Western
    {  667, 1008,  894667,  894678,  894694, 11, 16, 31, }, // Iran
    {  667, 1019,  894725,  894742,  894764, 17, 22, 28, }, // Irkutsk
    {  667, 1027,  894792,  894806,  894825, 14, 19, 34, }, // Israel
    {  667, 1034,  894859,  894872,  894890, 13, 18, 33, }, // Japan
    {  667, 1058,       0,  894923,       0,  0, 22,  0, }, // Kazakhstan_Eastern
    {  667, 1077,       0,  894945,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  667, 1106,  894969,  894983,  895002, 14, 19, 34, }, // Korea
    {  667, 1112,       0,  895036,       0,  0, 13,  0, }, // Kosrae
    {  667, 1119,  895049,  895071,  895098, 22, 27, 33, }, // Krasnoyarsk
    {  667, 1151,       0,  895131,       0,  0, 19,  0, }, // Kyrgystan
    {  667, 1175,       0,  895150,       0,  0, 27,  0, }, // Line_Islands
    {  667, 1188,  895177,  895194,  895216, 17, 22, 37, }, // Lord_Howe
    {  667, 1204,  895253,  895267,  895286, 14, 19, 25, }, // Magadan
    {  667, 1219,       0,  895311,       0,  0, 14,  0, }, // Malaysia
    {  667, 1228,       0,  895325,       0,  0, 13,  0, }, // Maldives
    {  667, 1237,       0,  895338,       0,  0, 15,  0, }, // Marquesas
    {  667, 1247,       0,  895353,       0,  0, 29,  0, }, // Marshall_Islands
    {  667, 1264,  895382,  895395,  895413, 13, 18, 24, }, // Mauritius
    {  667, 1274,       0,  895437,       0,  0, 13,  0, }, // Mawson
    {  667, 1281,  895450,  895473,  895501, 23, 28, 43, }, // Mexico_Pacific
    {  667, 1296,  895544,  895560,  895581, 16, 21, 27, }, // Mongolia
    {  667, 1305,  895608,  895621,  895639, 13, 18, 24, }, // Moscow
    {  667, 1312,       0,  895663,       0,  0, 15,  0, }, // Myanmar
    {  667, 1320,       0,  895678,       0,  0, 11,  0, }, // Nauru
    {  667, 1326,       0,  895689,       0,  0, 12,  0, }, // Nepal
    {  667, 1332,  895701,  895723,  895750, 22, 27, 33, }, // New_Caledonia
    {  667, 1346,  895783,  895801,  895824, 18, 23, 38, }, // New_Zealand
    {  667, 1358,  895862,  895883,  895909, 21, 26, 41, }, // Newfoundland
    {  667, 1371,       0,  895950,       0,  0, 11,  0, }, // Niue
    {  667, 1376,  895961,  895980,  896004, 19, 24, 39, }, // Norfolk
    {  667, 1384,  896043,  896073,  896108, 30, 35, 41, }, // Noronha
    {  667, 1406,  896149,  896169,  896194, 20, 25, 31, }, // Novosibirsk
    {  667, 1418,  896225,  896238,  896256, 13, 18, 24, }, // Omsk
    {  667, 1428,  896280,  896297,  896319, 17, 22, 28, }, // Pakistan
    {  667, 1437,       0,  896347,       0,  0, 11,  0, }, // Palau
    {  667, 1443,       0,  896358,       0,  0, 22,  0, }, // Papua_New_Guinea
    {  667, 1460,  896380,  896395,  896415, 15, 20, 26, }, // Paraguay
    {  667, 1469,  896441,  896452,  896468, 11, 16, 22, }, // Peru
    {  667, 1474,  896490,  896505,  896525, 15, 20, 26, }, // Philippines
    {  667, 1486,       0,  896551,       0,  0, 31,  0, }, // Phoenix_Islands
    {  667, 1502,  896582,  896613,  896649, 31, 36, 51, }, // Pierre_Miquelon
    {  667, 1518,       0,  896700,       0,  0, 15,  0, }, // Pitcairn
    {  667, 1527,       0,  896715,       0,  0, 13,  0, }, // Ponape
    {  667, 1534,       0,  896728,       0,  0, 17,  0, }, // Pyongyang
    {  667, 1554,       0,  896745,       0,  0, 15,  0, }, // Reunion
    {  667, 1562,       0,  896760,       0,  0, 13,  0, }, // Rothera
    {  667, 1570,  896773,  896786,  896804, 13, 18, 24, }, // Sakhalin
    {  667, 1596,  896828,  896839,  896855, 11, 16, 31, }, // Samoa
    {  667, 1602,       0,  896886,       0,  0, 14,  0, }, // Seychelles
    {  667, 1624,       0,  896900,       0,  0, 16,  0, }, // Singapore
    {  667, 1634,       0,  896916,       0,  0, 29,  0, }, // Solomon
    {  667, 1642,       0,  896945,       0,  0, 25,  0, }, // South_Georgia
    {  667, 1656,       0,  896970,       0,  0, 15,  0, }, // Suriname
    {  667, 1676,       0,  896985,       0,  0, 13,  0, }, // Syowa
    {  667, 1682,       0,  896998,       0,  0, 13,  0, }, // Tahiti
    {  667, 1689,  897011,  897023,  897040, 12, 17, 32, }, // Taipei
    {  667, 1696,       0,  897072,       0,  0, 19,  0, }, // Tajikistan
    {  667, 1724,       0,  897091,       0,  0, 14,  0, }, // Tokelau
    {  667, 1732,  897105,  897117,  897134, 12, 17, 23, }, // Tonga
    {  667, 1738,       0,  897157,       0,  0, 10,  0, }, // Truk
    {  667, 1750,  897167,  897188,  897215, 21, 27, 33, }, // Turkmenistan
    {  667, 1763,       0,  897248,       0,  0, 13,  0, }, // Tuvalu
    {  667, 1777,  897261,  897275,  897294, 14, 19, 25, }, // Uruguay
    {  667, 1792,  897319,  897339,  897364, 20, 25, 31, }, // Uzbekistan
    {  667, 1803,  897395,  897408,  897426, 13, 18, 24, }, // Vanuatu
    {  667, 1811,       0,  897450,       0,  0, 17,  0, }, // Venezuela
    {  667, 1821,  897467,  897487,  897512, 20, 25, 31, }, // Vladivostok
    {  667, 1833,  897543,  897561,  897584, 18, 23, 29, }, // Volgograd
    {  667, 1843,       0,  897473,       0,  0, 14,  0, }, // Vostok
    {  667, 1850,       0,  897613,       0,  0, 15,  0, }, // Wake
    {  667, 1855,       0,  897628,       0,  0, 26,  0, }, // Wallis
    {  667, 1862,  897654,  897670,  897691, 16, 21, 27, }, // Yakutsk
    {  667, 1870,  897718,  897738,  897763, 20, 25, 31, }, // Yekaterinburg
    {  668,    5,       0,  897794,       0,  0, 20,  0, }, // Afghanistan Venetian/Latin/Italy
    {  668,   17,       0,  897814,       0,  0, 24,  0, }, // Africa_Central
    {  668,   32,       0,  897838,       0,  0, 25,  0, }, // Africa_Eastern
    {  668,   65,       0,  897863,       0,  0, 27,  0, }, // Africa_Southern
    {  668,   81,  897890,  897916,  897950, 26, 34, 33, }, // Africa_Western
    {  668,  107,  897983,  897998,  898021, 15, 23, 22, }, // Alaska
    {  668,  135,  898043,  898060,  898085, 17, 25, 24, }, // Amazon
    {  668,  142,  898109,  898138,  898175, 29, 37, 36, }, // America_Central
    {  668,  158,  898211,  898241,  898279, 30, 38, 37, }, // America_Eastern
    {  668,  174,  898316,  898351,  898394, 35, 43, 42, }, // America_Mountain
    {  668,  191,  898436,  898470,  898512, 34, 42, 41, }, // America_Pacific
    {  668,  214,  631130,  898553,  898572, 11, 19, 18, }, // Apia
    {  668,  232,  898590,  898605,  898628, 15, 23, 22, }, // Arabian
    {  668,  240,  898650,  898668,  898694, 18, 26, 25, }, // Argentina
    {  668,  250,  898719,  898748,  898785, 29, 37, 36, }, // Argentina_Western
    {  668,  268,  898821,  898837,  898861, 16, 24, 23, }, // Armenia
    {  668,  286,  898884,  898919,  898962, 35, 43, 42, }, // Atlantic
    {  668,  295,  899004,  899031,  899066, 27, 35, 34, }, // Australia_Central
    {  668,  313,  899100,  899136,  899180, 36, 44, 43, }, // Australia_CentralWestern
    {  668,  338,  899223,  899251,  899287, 28, 36, 35, }, // Australia_Eastern
    {  668,  356,  899322,  899351,  899388, 29, 37, 36, }, // Australia_Western
    {  668,  374,  899424,  899443,  899470, 19, 27, 26, }, // Azerbaijan
    {  668,  385,  899496,  899511,  899534, 15, 23, 22, }, // Azores
    {  668,  397,  899556,  899575,  899602, 19, 27, 26, }, // Bangladesh
    {  668,  415,       0,  899628,       0,  0, 14,  0, }, // Bhutan
    {  668,  422,       0,  899642,       0,  0, 17,  0, }, // Bolivia
    {  668,  437,  899659,  899674,  899697, 15, 23, 22, }, // Brasilia
    {  668,  454,       0,  899719,       0,  0, 15,  0, }, // Brunei
    {  668,  461,  899734,  899750,  899774, 16, 24, 23, }, // Cape_Verde
    {  668,  478,       0,  899797,       0,  0, 14,  0, }, // Chamorro
    {  668,  487,  899811,  899833,  899863, 22, 30, 29, }, // Chatham
    {  668,  495,  899892,  899905,  899926, 13, 21, 20, }, // Chile
    {  668,  501,  899946,  899960,  899982, 14, 22, 21, }, // China
    {  668,  507,  632732,  900003,  900028, 17, 25, 24, }, // Choibalsan
    {  668,  518,       0,  900052,       0,  0, 24,  0, }, // Christmas
    {  668,  528,       0,  900076,       0,  0, 21,  0, }, // Cocos
    {  668,  534,  900097,  900115,  900141, 18, 26, 25, }, // Colombia
    {  668,  543,  900166,  900186,  900214, 20, 28, 27, }, // Cook
    {  668,  548,  633008,  900241,  900260, 11, 19, 18, }, // Cuba
    {  668,  559,       0,  633058,       0,  0, 12,  0, }, // Davis
    {  668,  575,       0,  900278,       0,  0, 23,  0, }, // DumontDUrville
    {  668,  612,       0,  900301,       0,  0, 18,  0, }, // East_Timor
    {  668,  623,  900319,  900343,  900375, 24, 32, 31, }, // Easter
    {  668,  630,       0,  900406,       0,  0, 16,  0, }, // Ecuador
    {  668,  638,  900422,  900446,  900478, 24, 32, 31, }, // Europe_Central
    {  668,  653,  900509,  900534,  900567, 25, 33, 32, }, // Europe_Eastern
    {  668,  668,       0,  900599,       0,  0, 30,  0, }, // Europe_Further_Eastern
    {  668,  691,  900629,  900655,  900689, 26, 34, 33, }, // Europe_Western
    {  668,  706,  900722,  900745,  900776, 23, 31, 30, }, // Falkland
    {  668,  715,  900806,  900826,  900854, 20, 28, 27, }, // Fiji
    {  668,  720,       0,  900881,       0,  0, 25,  0, }, // French_Guiana
    {  668,  734,       0,  900906,       0,  0, 49,  0, }, // French_Southern
    {  668,  757,       0,  900955,       0,  0, 19,  0, }, // Galapagos
    {  668,  767,       0,  900974,       0,  0, 23,  0, }, // Gambier
    {  668,  775,  900997,  901013,  901037, 16, 24, 23, }, // Georgia
    {  668,  783,       0,  901060,       0,  0, 23,  0, }, // Gilbert_Islands
    {  668,  799,       0,  901083,       0,  0, 30,  0, }, // GMT
    {  668,  841,  901113,  901143,  901181, 30, 38, 37, }, // Greenland_Eastern
    {  668,  859,  901218,  901249,  901288, 31, 39, 38, }, // Greenland_Western
    {  668,  882,       0,  901326,       0,  0, 14,  0, }, // Gulf
    {  668,  887,       0,  900881,       0,  0, 16,  0, }, // Guyana
    {  668,  894,  901340,  901363,  901394, 23, 31, 30, }, // Hawaii_Aleutian
    {  668,  910,  634341,  901424,  901448, 16, 24, 23, }, // Hong_Kong
    {  668,  920,  634406,  901471,  901490, 11, 19, 18, }, // Hovd
    {  668,  925,       0,  901508,       0,  0, 14,  0, }, // India
    {  668,  931,       0,  901522,       0,  0, 22,  0, }, // Indian_Ocean
    {  668,  944,       0,  901544,       0,  0, 17,  0, }, // Indochina
    {  668,  954,       0,  901561,       0,  0, 27,  0, }, // Indonesia_Central
    {  668,  972,       0,  901588,       0,  0, 28,  0, }, // Indonesia_Eastern
    {  668,  990,       0,  901616,       0,  0, 29,  0, }, // Indonesia_Western
    {  668, 1008,  901645,  901658,  901679, 13, 21, 20, }, // Iran
    {  668, 1019,  634664,  901699,  901721, 14, 22, 21, }, // Irkutsk
    {  668, 1027,  901742,  901756,  901778, 14, 22, 21, }, // Israel
    {  668, 1034,  901799,  901813,  901835, 14, 22, 21, }, // Japan
    {  668, 1058,       0,  901856,       0,  0, 29,  0, }, // Kazakhstan_Eastern
    {  668, 1077,       0,  901885,       0,  0, 30,  0, }, // Kazakhstan_Western
    {  668, 1106,  901915,  901930,  901953, 15, 23, 22, }, // Korea
    {  668, 1112,       0,  901975,       0,  0, 21,  0, }, // Kosrae
    {  668, 1119,  635056,  901996,  902022, 18, 26, 25, }, // Krasnoyarsk
    {  668, 1151,       0,  902047,       0,  0, 21,  0, }, // Kyrgystan
    {  668, 1175,       0,  902068,       0,  0, 27,  0, }, // Line_Islands
    {  668, 1188,  902095,  902119,  902151, 24, 32, 31, }, // Lord_Howe
    {  668, 1204,  635308,  902182,  902204, 14, 22, 21, }, // Magadan
    {  668, 1219,       0,  902225,       0,  0, 17,  0, }, // Malaysia
    {  668, 1228,       0,  902242,       0,  0, 23,  0, }, // Maldives
    {  668, 1237,       0,  902265,       0,  0, 24,  0, }, // Marquesas
    {  668, 1247,       0,  902289,       0,  0, 24,  0, }, // Marshall_Islands
    {  668, 1264,  902313,  902337,  902369, 24, 32, 31, }, // Mauritius
    {  668, 1274,       0,  635529,       0,  0, 13,  0, }, // Mawson
    {  668, 1281,  902400,  902429,  902466, 29, 37, 36, }, // Mexico_Pacific
    {  668, 1296,  902502,  902519,  902544, 17, 25, 24, }, // Mongolia
    {  668, 1305,  635714,  902568,  902588, 12, 20, 19, }, // Moscow
    {  668, 1312,       0,  902607,       0,  0, 25,  0, }, // Myanmar
    {  668, 1320,       0,  902632,       0,  0, 20,  0, }, // Nauru
    {  668, 1326,       0,  902652,       0,  0, 14,  0, }, // Nepal
    {  668, 1332,  902666,  902690,  902722, 24, 32, 31, }, // New_Caledonia
    {  668, 1346,  902753,  902775,  902805, 22, 30, 29, }, // New_Zealand
    {  668, 1358,  902834,  902849,  902872, 15, 23, 22, }, // Newfoundland
    {  668, 1371,       0,  902894,       0,  0, 19,  0, }, // Niue
    {  668, 1376,  902913,  902935,  902965, 22, 30, 29, }, // Norfolk
    {  668, 1384,  902994,  903020,  903054, 26, 34, 33, }, // Noronha
    {  668, 1406,  636282,  903087,  903113, 18, 26, 25, }, // Novosibirsk
    {  668, 1418,  636353,  903138,  903157, 11, 19, 18, }, // Omsk
    {  668, 1428,  903175,  903192,  903217, 17, 25, 24, }, // Pakistan
    {  668, 1437,       0,  903241,       0,  0, 21,  0, }, // Palau
    {  668, 1443,       0,  903262,       0,  0, 27,  0, }, // Papua_New_Guinea
    {  668, 1460,  903289,  903306,  903331, 17, 25, 24, }, // Paraguay
    {  668, 1469,  903355,  903368,  903389, 13, 21, 20, }, // Peru
    {  668, 1474,  903409,  903433,  903465, 24, 32, 31, }, // Philippines
    {  668, 1486,       0,  903496,       0,  0, 22,  0, }, // Phoenix_Islands
    {  668, 1502,  903518,  903544,  903578, 26, 34, 33, }, // Pierre_Miquelon
    {  668, 1518,       0,  903611,       0,  0, 24,  0, }, // Pitcairn
    {  668, 1527,       0,  903635,       0,  0, 21,  0, }, // Ponape
    {  668, 1534,       0,  636891,       0,  0, 16,  0, }, // Pyongyang
    {  668, 1554,       0,  903656,       0,  0, 22,  0, }, // Reunion
    {  668, 1562,       0,  636990,       0,  0, 14,  0, }, // Rothera
    {  668, 1570,  903678,  903692,  903714, 14, 22, 21, }, // Sakhalin
    {  668, 1596,  903735,  903756,  903785, 21, 29, 28, }, // Samoa
    {  668, 1602,       0,  903813,       0,  0, 29,  0, }, // Seychelles
    {  668, 1624,       0,  637208,       0,  0, 16,  0, }, // Singapore
    {  668, 1634,       0,  903842,       0,  0, 23,  0, }, // Solomon
    {  668, 1642,       0,  903865,       0,  0, 26,  0, }, // South_Georgia
    {  668, 1656,       0,  903891,       0,  0, 17,  0, }, // Suriname
    {  668, 1676,       0,  637295,       0,  0, 12,  0, }, // Syowa
    {  668, 1682,       0,  903908,       0,  0, 20,  0, }, // Tahiti
    {  668, 1689,  903928,  903941,  903962, 13, 21, 20, }, // Taipei
    {  668, 1696,       0,  903982,       0,  0, 19,  0, }, // Tajikistan
    {  668, 1724,       0,  904001,       0,  0, 23,  0, }, // Tokelau
    {  668, 1732,  904024,  904045,  904074, 21, 29, 28, }, // Tonga
    {  668, 1738,       0,  904102,       0,  0, 14,  0, }, // Truk
    {  668, 1750,  904116,  904137,  904166, 21, 29, 28, }, // Turkmenistan
    {  668, 1763,       0,  904194,       0,  0, 22,  0, }, // Tuvalu
    {  668, 1777,  904216,  904232,  904256, 16, 24, 23, }, // Uruguay
    {  668, 1792,  904279,  904298,  904325, 19, 27, 26, }, // Uzbekistan
    {  668, 1803,  904351,  904374,  904405, 23, 31, 30, }, // Vanuatu
    {  668, 1811,       0,  904435,       0,  0, 18,  0, }, // Venezuela
    {  668, 1821,  637816,  904453,  904479, 18, 26, 25, }, // Vladivostok
    {  668, 1833,  904504,  904521,  904546, 17, 25, 24, }, // Volgograd
    {  668, 1843,       0,  637952,       0,  0, 13,  0, }, // Vostok
    {  668, 1850,       0,  904570,       0,  0, 19,  0, }, // Wake
    {  668, 1855,       0,  904589,       0,  0, 31,  0, }, // Wallis
    {  668, 1862,  638009,  904620,  904642, 14, 22, 21, }, // Yakutsk
    {  668, 1870,  904663,  904683,  904711, 20, 28, 27, }, // Yekaterinburg
    {  668, 1892,       0,  904738,       0,  0, 14,  0, }, // Yukon
    {  669,    5,       0,  904752,       0,  0, 16,  0, }, // Afghanistan Kuvi/Latin/India
    {  669,   17,       0,  904768,       0,  0, 19,  0, }, // Africa_Central
    {  669,   32,       0,  904787,       0,  0, 21,  0, }, // Africa_Eastern
    {  669,   65,       0,  904808,       0,  0, 17,  0, }, // Africa_Southern
    {  669,   81,  904825,  904847,  904878, 22, 31, 28, }, // Africa_Western
    {  669,  107,  904906,  904917,  904936, 11, 19, 19, }, // Alaska
    {  669,  135,  904955,  904966,  904985, 11, 19, 21, }, // Amazon
    {  669,  142,  905006,  905017,  905036, 11, 19, 19, }, // America_Central
    {  669,  158,  905055,  905068,  905089, 13, 21, 21, }, // America_Eastern
    {  669,  174,  905110,  905120,  905138, 10, 18, 18, }, // America_Mountain
    {  669,  191,  905156,  905168,  905188, 12, 20, 20, }, // America_Pacific
    {  669,  214,  905208,  905217,  905234,  9, 17, 16, }, // Apia
    {  669,  232,  905250,  905262,  905282, 12, 20, 19, }, // Arabian
    {  669,  240,  905301,  905315,  905337, 14, 22, 25, }, // Argentina
    {  669,  250,  905362,  905387,  905420, 25, 33, 31, }, // Argentina_Western
    {  669,  268,  905451,  905463,  905483, 12, 20, 18, }, // Armenia
    {  669,  286,  905501,  905514,  905535, 13, 21, 20, }, // Atlantic
    {  669,  295,  905555,  905575,  905605, 20, 30, 29, }, // Australia_Central
    {  669,  313,  905634,  905667,  905708, 33, 41, 40, }, // Australia_CentralWestern
    {  669,  338,  905748,  905770,  905802, 22, 32, 32, }, // Australia_Eastern
    {  669,  356,  905834,  905858,  905892, 24, 34, 33, }, // Australia_Western
    {  669,  374,  905925,  905940,  905963, 15, 23, 21, }, // Azerbaijan
    {  669,  385,  905984,  905995,  906014, 11, 19, 17, }, // Azores
    {  669,  397,  906031,  906045,  906067, 14, 22, 20, }, // Bangladesh
    {  669,  415,       0,  906087,       0,  0, 10,  0, }, // Bhutan
    {  669,  422,       0,  906097,       0,  0, 11,  0, }, // Bolivia
    {  669,  437,  906108,  906121,  906142, 13, 21, 19, }, // Brasilia
    {  669,  454,       0,  906161,       0,  0, 22,  0, }, // Brunei
    {  669,  461,  906183,  906194,  906213, 11, 19, 17, }, // Cape_Verde
    {  669,  478,       0,  906230,       0,  0, 18,  0, }, // Chamorro
    {  669,  487,  906248,  906259,  906278, 11, 19, 18, }, // Chatham
    {  669,  495,  906296,  906305,  906322,  9, 17, 15, }, // Chile
    {  669,  501,  906337,  906345,  906361,  8, 16, 15, }, // China
    {  669,  507,  906376,  906389,  906410, 13, 21, 19, }, // Choibalsan
    {  669,  518,       0,  906429,       0,  0, 16,  0, }, // Christmas
    {  669,  528,       0,  906445,       0,  0, 14,  0, }, // Cocos
    {  669,  534,  906459,  906472,  906493, 13, 21, 19, }, // Colombia
    {  669,  543,  906512,  906524,  906544, 12, 20, 22, }, // Cook
    {  669,  548,  906566,  906575,  906592,  9, 17, 16, }, // Cuba
    {  669,  559,       0,  906608,       0,  0, 10,  0, }, // Davis
    {  669,  575,       0,  906618,       0,  0, 21,  0, }, // DumontDUrville
    {  669,  612,       0,  906639,       0,  0, 19,  0, }, // East_Timor
    {  669,  623,  906658,  906671,  906692, 13, 21, 19, }, // Easter
    {  669,  630,       0,  906711,       0,  0, 12,  0, }, // Ecuador
    {  669,  638,  906723,  906744,  906773, 21, 29, 27, }, // Europe_Central
    {  669,  653,  906800,  906822,  906852, 22, 30, 28, }, // Europe_Eastern
    {  669,  668,       0,  906880,       0,  0, 24,  0, }, // Europe_Further_Eastern
    {  669,  691,  906904,  906929,  906962, 25, 33, 31, }, // Europe_Western
    {  669,  706,  906993,  907013,  907041, 20, 28, 26, }, // Falkland
    {  669,  715,  907067,  907076,  907093,  9, 17, 15, }, // Fiji
    {  669,  720,       0,  907108,       0,  0, 17,  0, }, // French_Guiana
    {  669,  734,       0,  907125,       0,  0, 31,  0, }, // French_Southern
    {  669,  757,       0,  907156,       0,  0, 13,  0, }, // Galapagos
    {  669,  767,       0,  907169,       0,  0, 12,  0, }, // Gambier
    {  669,  775,  907181,  907192,  907211, 11, 19, 17, }, // Georgia
    {  669,  783,       0,  907228,       0,  0, 15,  0, }, // Gilbert_Islands
    {  669,  799,       0,  907243,       0,  0, 16,  0, }, // GMT
    {  669,  841,  907259,  907280,  907309, 21, 29, 27, }, // Greenland_Eastern
    {  669,  859,  907336,  907359,  907390, 23, 31, 29, }, // Greenland_Western
    {  669,  882,       0,  907419,       0,  0, 17,  0, }, // Gulf
    {  669,  887,       0,  907115,       0,  0, 10,  0, }, // Guyana
    {  669,  894,  907436,  907455,  907482, 19, 27, 26, }, // Hawaii_Aleutian
    {  669,  910,  907508,  907522,  907544, 14, 22, 20, }, // Hong_Kong
    {  669,  920,  907564,  907573,  907590,  9, 17, 15, }, // Hovd
    {  669,  925,       0,  907605,       0,  0, 18,  0, }, // India
    {  669,  931,       0,  907623,       0,  0, 15,  0, }, // Indian_Ocean
    {  669,  944,       0,  907638,       0,  0, 13,  0, }, // Indochina
    {  669,  954,       0,  907651,       0,  0, 20,  0, }, // Indonesia_Central
    {  669,  972,       0,  907671,       0,  0, 21,  0, }, // Indonesia_Eastern
    {  669,  990,       0,  907692,       0,  0, 23,  0, }, // Indonesia_Western
    {  669, 1008,  907715,  907724,  907741,  9, 17, 16, }, // Iran
    {  669, 1019,  907757,  907770,  907791, 13, 21, 19, }, // Irkutsk
    {  669, 1027,  907810,  907822,  907842, 12, 20, 19, }, // Israel
    {  669, 1034,  907861,  907871,  907889, 10, 18, 17, }, // Japan
    {  669, 1058,       0,  907906,       0,  0, 23,  0, }, // Kazakhstan_Eastern
    {  669, 1077,       0,  907929,       0,  0, 24,  0, }, // Kazakhstan_Western
    {  669, 1106,  907953,  907964,  907983, 11, 19, 18, }, // Korea
    {  669, 1112,       0,  908001,       0,  0, 11,  0, }, // Kosrae
    {  669, 1119,  908012,  908027,  908050, 15, 23, 21, }, // Krasnoyarsk
    {  669, 1151,       0,  908071,       0,  0, 15,  0, }, // Kyrgystan
    {  669, 1175,       0,  908086,       0,  0, 13,  0, }, // Line_Islands
    {  669, 1188,  908099,  908112,  908133, 13, 21, 20, }, // Lord_Howe
    {  669, 1204,  908153,  908165,  908185, 12, 20, 19, }, // Magadan
    {  669, 1219,       0,  908204,       0,  0, 12,  0, }, // Malaysia
    {  669, 1228,       0,  908216,       0,  0, 11,  0, }, // Maldives
    {  669, 1237,       0,  908227,       0,  0, 12,  0, }, // Marquesas
    {  669, 1247,       0,  908239,       0,  0, 16,  0, }, // Marshall_Islands
    {  669, 1264,  908255,  908267,  908286, 12, 19, 18, }, // Mauritius
    {  669, 1274,       0,  908304,       0,  0, 11,  0, }, // Mawson
    {  669, 1281,  908315,  908336,  908365, 21, 29, 28, }, // Mexico_Pacific
    {  669, 1296,  908393,  908407,  908429, 14, 22, 20, }, // Mongolia
    {  669, 1305,  908449,  908459,  908477, 10, 18, 16, }, // Moscow
    {  669, 1312,       0,  908493,       0,  0, 13,  0, }, // Myanmar
    {  669, 1320,       0,  908506,       0,  0, 10,  0, }, // Nauru
    {  669, 1326,       0,  908516,       0,  0, 10,  0, }, // Nepal
    {  669, 1332,  908526,  908544,  908570, 18, 26, 24, }, // New_Caledonia
    {  669, 1346,  908594,  908608,  908630, 14, 22, 21, }, // New_Zealand
    {  669, 1358,  908651,  908669,  908695, 18, 26, 25, }, // Newfoundland
    {  669, 1371,       0,  908720,       0,  0,  8,  0, }, // Niue
    {  669, 1376,  908728,  908743,  908766, 15, 23, 22, }, // Norfolk
    {  669, 1384,  908788,  908811,  908842, 23, 31, 29, }, // Noronha
    {  669, 1406,  908871,  908887,  908911, 16, 24, 22, }, // Novosibirsk
    {  669, 1418,  908933,  908943,  908961, 10, 18, 16, }, // Omsk
    {  669, 1428,  908977,  908990,  909011, 13, 21, 19, }, // Pakistan
    {  669, 1437,       0,  909030,       0,  0, 10,  0, }, // Palau
    {  669, 1443,       0,  909040,       0,  0, 20,  0, }, // Papua_New_Guinea
    {  669, 1460,  909060,  909073,  909094, 13, 21, 19, }, // Paraguay
    {  669, 1469,  909113,  909122,  909139,  9, 17, 15, }, // Peru
    {  669, 1474,  909154,  909166,  909186, 12, 20, 18, }, // Philippines
    {  669, 1486,       0,  909204,       0,  0, 19,  0, }, // Phoenix_Islands
    {  669, 1502,  909223,  909251,  909287, 28, 36, 35, }, // Pierre_Miquelon
    {  669, 1518,       0,  909322,       0,  0, 13,  0, }, // Pitcairn
    {  669, 1527,       0,  909335,       0,  0, 11,  0, }, // Ponape
    {  669, 1534,       0,  909346,       0,  0, 15,  0, }, // Pyongyang
    {  669, 1554,       0,  909361,       0,  0, 12,  0, }, // Reunion
    {  669, 1562,       0,  909373,       0,  0, 11,  0, }, // Rothera
    {  669, 1570,  909384,  909396,  909416, 12, 20, 18, }, // Sakhalin
    {  669, 1596,  909434,  909445,  909464, 11, 19, 13, }, // Samoa
    {  669, 1602,       0,  909477,       0,  0, 11,  0, }, // Seychelles
    {  669, 1624,       0,  909488,       0,  0, 21,  0, }, // Singapore
    {  669, 1634,       0,  909509,       0,  0, 19,  0, }, // Solomon
    {  669, 1642,       0,  909528,       0,  0, 17,  0, }, // South_Georgia
    {  669, 1656,       0,  909545,       0,  0, 12,  0, }, // Suriname
    {  669, 1676,       0,  909557,       0,  0,  9,  0, }, // Syowa
    {  669, 1682,       0,  909566,       0,  0, 11,  0, }, // Tahiti
    {  669, 1689,  909577,  909587,  909605, 10, 18, 12, }, // Taipei
    {  669, 1696,       0,  909617,       0,  0, 15,  0, }, // Tajikistan
    {  669, 1724,       0,  909632,       0,  0, 12,  0, }, // Tokelau
    {  669, 1732,  909644,  909654,  909672, 10, 18, 16, }, // Tonga
    {  669, 1738,       0,  909688,       0,  0,  8,  0, }, // Truk
    {  669, 1750,  909696,  909713,  909737, 17, 24, 23, }, // Turkmenistan
    {  669, 1763,       0,  909760,       0,  0, 11,  0, }, // Tuvalu
    {  669, 1777,  909771,  909782,  909800, 11, 18, 17, }, // Uruguay
    {  669, 1792,  909817,  909832,  909855, 15, 23, 20, }, // Uzbekistan
    {  669, 1803,  909875,  909887,  909907, 12, 20, 18, }, // Vanuatu
    {  669, 1811,       0,  909925,       0,  0, 14,  0, }, // Venezuela
    {  669, 1821,  909939,  909955,  909979, 16, 24, 22, }, // Vladivostok
    {  669, 1833,  910001,  910015,  910037, 14, 22, 20, }, // Volgograd
    {  669, 1843,       0,  909944,       0,  0, 11,  0, }, // Vostok
    {  669, 1850,       0,  910057,       0,  0, 14,  0, }, // Wake
    {  669, 1855,       0,  910071,       0,  0, 16,  0, }, // Wallis
    {  669, 1862,  910087,  910100,  910121, 13, 21, 19, }, // Yakutsk
    {  669, 1870,  910140,  910158,  910184, 18, 26, 24, }, // Yekaterinburg
    {  670,  799,       0,  910208,       0,  0, 17,  0, }, // GMT Kuvi/Devanagari/India
    {  671,  799,       0,  910225,       0,  0, 17,  0, }, // GMT Kuvi/Odia/India
    {  672,  799,       0,  742255,       0,  0, 21,  0, }, // GMT Kuvi/Telugu/India
}; // Metazone long name table

// Sorted by locale index, then meta key
static inline constexpr LocaleMetaZoneShortNames localeMetaZoneShortNameTable[] = {
    // locInd, metaKey, (generic, standard, DST){ind, sz}
    {    0,  107,    0,    3,    7,  3,  4,  4, }, // Alaska C/AnyScript/AnyTerritory
    {    0,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {    0,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {    0,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {    0,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {    0,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {    0,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {    0,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {    5,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central Afrikaans/Latin/South Africa
    {    5,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {    5,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {    5,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {   14,  882,    0,   82,    0,  0,  3,  0, }, // Gulf Arabic/Arabic/Egypt
    {   44,  925,    0,   85,    0,  0,  6,  0, }, // India Assamese/Bangla/India
    {   45,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Asturian/Latin/Spain
    {   45,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {   45,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   45,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   45,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   45,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {   45,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {   45,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {   45,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {   45,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {   45,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {   56,  925,    0,  112,    0,  0,  3,  0, }, // India Bangla/Bangla/Bangladesh
    {   60,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Basque/Latin/Spain
    {   60,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {   60,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   60,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   60,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   60,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {   60,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {   60,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {   60,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {   60,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {   60,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {   66,  925,    0,  112,    0,  0,  3,  0, }, // India Bodo/Devanagari/India
    {   67,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Bosnian/Latin/Bosnia and Herzegovina
    {   67,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {   67,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {   67,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {   74,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Catalan/Latin/Spain
    {   74,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {   74,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {   74,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {   82,  925,    0,  112,    0,  0,  3,  0, }, // India Chakma/Chakma/Bangladesh
    {   85,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Cherokee/Cherokee/United States
    {   85,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {   85,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   85,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   85,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   85,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {   85,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {   91, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia Chinese/Simplified Han/Singapore
    {   91, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {   92,  142,   11,   13,   16,  2,  3,  3, }, // America_Central Chinese/Traditional Han/Hong Kong
    {   92,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   92,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   92,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   93,  142,   11,   13,   16,  2,  3,  3, }, // America_Central Chinese/Traditional Han/Macao
    {   93,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   93,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   93,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   94,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Chinese/Traditional Han/Taiwan
    {   94,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {   94,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {   94,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {   94,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {   94,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {   94,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {   94,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {   97,  638,  121,  121,  124,  3,  3,  4, }, // Europe_Central Colognian/Latin/Germany
    {   97,  653,  128,  128,  131,  3,  3,  4, }, // Europe_Eastern
    {   97,  691,  135,  135,  138,  3,  3,  4, }, // Europe_Western
    {   98,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Cornish/Latin/United Kingdom
    {   98,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {   98,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {   98,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  100,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Croatian/Latin/Croatia
    {  100,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  100,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  100,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  102,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Czech/Latin/Czechia
    {  102,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  102,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  102,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  102,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  102,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  102,  638,  142,  142,  145,  3,  3,  4, }, // Europe_Central
    {  103,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Danish/Latin/Denmark
    {  103,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  103,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  108,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Dutch/Latin/Netherlands
    {  108,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  108,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  108,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  108,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  108,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  108,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  108,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  108,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  108,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  108,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  114, 1656,    0,  149,    0,  0,  3,  0, }, // Suriname Dutch/Latin/Suriname
    {  115,  415,    0,  152,    0,  0,  2,  0, }, // Bhutan Dzongkha/Tibetan/Bhutan
    {  117,  107,    0,    3,    7,  3,  4,  4, }, // Alaska English/Latin/United States
    {  117,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  117,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  117,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  117,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  117,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  117,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  117,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  120,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Anguilla
    {  120,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  120,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  120,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  120,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  120,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  120,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  121,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Antigua and Barbuda
    {  121,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  121,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  121,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  121,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  121,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  121,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  122,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Australia
    {  122,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  122,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  122,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  122,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  122,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  122,  295,  157,  160,  164,  3,  4,  4, }, // Australia_Central
    {  122,  313,  168,  172,  177,  4,  5,  5, }, // Australia_CentralWestern
    {  122,  338,  182,  185,  189,  3,  4,  4, }, // Australia_Eastern
    {  122,  356,  193,  196,  200,  3,  4,  4, }, // Australia_Western
    {  122,  882,    0,  204,    0,  0,  7,  0, }, // Gulf
    {  122,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  122, 1188,  211,  214,  218,  3,  4,  4, }, // Lord_Howe
    {  122, 1346,  222,  225,  229,  3,  4,  4, }, // New_Zealand
    {  123,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Austria
    {  123,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  123,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  123,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  123,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  123,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  123,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  123,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  123,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  123,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  124,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Bahamas
    {  124,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  124,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  124,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  124,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  124,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  124,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  125,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Barbados
    {  125,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  125,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  125,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  125,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  125,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  125,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  126,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Belgium
    {  126,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  126,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  126,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  126,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  126,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  126,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  126,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  126,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  126,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  127,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Belize
    {  127,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  127,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  127,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  127,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  127,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  127,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  128,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Bermuda
    {  128,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  128,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  128,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  128,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  128,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  128,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  129,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Botswana
    {  129,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  129,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  129,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  129,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  129,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  129,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  129,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  129,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  129,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  129,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  130,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/British Indian Ocean Territory
    {  130,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  130,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  130,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  130,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  130,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  130,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  131,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/British Virgin Islands
    {  131,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  131,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  131,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  131,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  131,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  131,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  133,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Cameroon
    {  133,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  133,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  133,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  133,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  133,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  133,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  133,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  133,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  133,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  133,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  134,    5,    0,  233,    0,  0,  3,  0, }, // Afghanistan English/Latin/Canada
    {  134,  107,    0,    3,    7,  3,  4,  4, }, // Alaska
    {  134,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  134,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  134,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  134,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  134,  240,  236,    0,    0,  3,  0,  0, }, // Argentina
    {  134,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  134,  313,  168,  172,  177,  4,  5,  5, }, // Australia_CentralWestern
    {  134,  338,  182,  185,  189,  3,  4,  4, }, // Australia_Eastern
    {  134,  356,    0,  196,  200,  0,  4,  4, }, // Australia_Western
    {  134,  397,    0,  239,    0,  0,  3,  0, }, // Bangladesh
    {  134,  415,    0,  242,    0,  0,  3,  0, }, // Bhutan
    {  134,  437,  245,    0,  248,  3,  0,  4, }, // Brasilia
    {  134,  454,    0,  252,    0,  0,  3,  0, }, // Brunei
    {  134,  487,    0,  255,  260,  0,  5,  5, }, // Chatham
    {  134,  518,    0,  265,    0,  0,  3,  0, }, // Christmas
    {  134,  528,    0,  268,    0,  0,  3,  0, }, // Cocos
    {  134,  534,    0,    0,  271,  0,  0,  4, }, // Colombia
    {  134,  612,    0,  275,    0,  0,  3,  0, }, // East_Timor
    {  134,  623,    0,  278,  282,  0,  4,  5, }, // Easter
    {  134,  630,    0,  287,    0,  0,  3,  0, }, // Ecuador
    {  134,  706,  290,    0,  293,  3,  0,  4, }, // Falkland
    {  134,  757,    0,  297,    0,  0,  4,  0, }, // Galapagos
    {  134,  841,  301,    0,    0,  3,  0,  0, }, // Greenland_Eastern
    {  134,  887,    0,  304,    0,  0,  3,  0, }, // Guyana
    {  134,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  134,  925,    0,  112,    0,  0,  3,  0, }, // India
    {  134,  944,    0,  307,    0,  0,  3,  0, }, // Indochina
    {  134,  954,    0,  310,    0,  0,  4,  0, }, // Indonesia_Central
    {  134,  972,    0,  310,    0,  0,  3,  0, }, // Indonesia_Eastern
    {  134,  990,    0,  314,    0,  0,  3,  0, }, // Indonesia_Western
    {  134, 1008,    0,  317,  321,  0,  4,  4, }, // Iran
    {  134, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  134, 1228,    0,  325,    0,  0,  3,  0, }, // Maldives
    {  134, 1326,    0,  328,    0,  0,  3,  0, }, // Nepal
    {  134, 1358,  253,  331,  334,  2,  3,  3, }, // Newfoundland
    {  134, 1384,  337,    0,    0,  3,  0,  0, }, // Noronha
    {  134, 1428,    0,  340,    0,  0,  3,  0, }, // Pakistan
    {  134, 1460,  343,    0,  346,  3,  0,  4, }, // Paraguay
    {  134, 1469,  350,    0,    0,  3,  0,  0, }, // Peru
    {  134, 1502,  353,  356,  360,  3,  4,  4, }, // Pierre_Miquelon
    {  134, 1777,    0,  364,  367,  0,  3,  4, }, // Uruguay
    {  134, 1811,    0,  371,    0,  0,  3,  0, }, // Venezuela
    {  135,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Cayman Islands
    {  135,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  135,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  135,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  135,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  135,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  135,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  136,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Christmas Island
    {  136,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  136,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  136,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  136,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  136,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  136,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  137,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Cocos Islands
    {  137,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  137,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  137,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  137,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  137,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  137,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  138,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Cook Islands
    {  138,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  138,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  138,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  138,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  138,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  138,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  139,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Cyprus
    {  139,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  139,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  139,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  139,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  139,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  139,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  140,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Denmark
    {  140,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  140,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  140,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  140,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  140,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  140,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  140,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  140,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  140,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  141,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Diego Garcia
    {  141,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  141,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  141,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  141,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  141,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  141,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  142,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Dominica
    {  142,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  142,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  142,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  142,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  142,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  142,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  143,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Eritrea
    {  143,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  143,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  143,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  143,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  143,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  143,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  143,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  143,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  143,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  143,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  144,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Eswatini
    {  144,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  144,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  144,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  144,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  144,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  144,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  144,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  144,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  144,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  144,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  145,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Europe
    {  145,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  145,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  145,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  145,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  145,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  145,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  145,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  145,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  145,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  146,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Falkland Islands
    {  146,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  146,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  146,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  146,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  146,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  146,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  147,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Fiji
    {  147,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  147,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  147,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  147,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  147,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  147,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  148,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Finland
    {  148,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  148,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  148,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  148,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  148,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  148,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  148,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  148,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  148,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  149,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Gambia
    {  149,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  149,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  149,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  149,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  149,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  149,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  149,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  149,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  149,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  149,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  150,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Germany
    {  150,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  150,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  150,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  150,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  150,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  150,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  150,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  150,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  150,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  151,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Ghana
    {  151,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  151,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  151,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  151,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  151,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  151,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  151,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  151,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  151,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  151,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  152,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Gibraltar
    {  152,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  152,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  152,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  152,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  152,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  152,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  153,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Grenada
    {  153,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  153,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  153,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  153,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  153,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  153,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  154,  478,    0,  374,    0,  0,  4,  0, }, // Chamorro English/Latin/Guam
    {  155,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Guernsey
    {  155,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  155,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  155,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  155,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  155,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  155,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  156,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Guyana
    {  156,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  156,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  156,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  156,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  156,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  156,  887,    0,  304,    0,  0,  3,  0, }, // Guyana
    {  156,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  157,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Hong Kong
    {  157,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  157,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  157,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  157,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  157,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  157,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  157,  910,  378,  378,  381,  3,  3,  4, }, // Hong_Kong
    {  158,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/India
    {  158,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  158,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  158,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  158,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  158,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  158,  882,    0,   82,    0,  0,  3,  0, }, // Gulf
    {  158,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  158,  925,    0,  112,    0,  0,  3,  0, }, // India
    {  159,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Indonesia
    {  159,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  159,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  159,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  159,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  159,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  159,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  159,  954,    0,  310,    0,  0,  4,  0, }, // Indonesia_Central
    {  159,  972,    0,  310,    0,  0,  3,  0, }, // Indonesia_Eastern
    {  159,  990,    0,  314,    0,  0,  3,  0, }, // Indonesia_Western
    {  160,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Ireland
    {  160,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  160,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  160,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  160,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  160,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  160,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  161,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Isle of Man
    {  161,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  161,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  161,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  161,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  161,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  161,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  162,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Israel
    {  162,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  162,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  162,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  162,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  162,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  162,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  163,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Jamaica
    {  163,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  163,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  163,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  163,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  163,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  163,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  164,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Jersey
    {  164,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  164,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  164,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  164,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  164,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  164,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  165,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Kenya
    {  165,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  165,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  165,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  165,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  165,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  165,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  165,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  165,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  165,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  165,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  166,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Kiribati
    {  166,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  166,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  166,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  166,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  166,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  166,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  167,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Lesotho
    {  167,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  167,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  167,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  167,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  167,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  167,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  167,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  167,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  167,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  167,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  168,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Liberia
    {  168,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  168,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  168,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  168,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  168,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  168,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  168,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  168,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  168,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  168,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  169,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Macao
    {  169,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  169,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  169,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  169,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  169,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  169,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  169,  910,  378,  378,  381,  3,  3,  4, }, // Hong_Kong
    {  169, 1198,   29,   29,   32,  3,  3,  3, }, // Macau
    {  170,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Madagascar
    {  170,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  170,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  170,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  170,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  170,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  170,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  170,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  170,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  170,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  170,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  171,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Malawi
    {  171,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  171,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  171,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  171,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  171,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  171,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  171,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  171,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  171,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  171,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  172,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Malaysia
    {  172,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  172,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  172,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  172,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  172,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  172,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  172, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  173,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Maldives
    {  173,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  173,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  173,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  173,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  173,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  173,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  174,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Malta
    {  174,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  174,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  174,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  174,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  174,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  174,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  175,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Marshall Islands
    {  175,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  175,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  175,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  175,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  175,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  175,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  176,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Mauritius
    {  176,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  176,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  176,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  176,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  176,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  176,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  176,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  176,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  176,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  176,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  177,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Micronesia
    {  177,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  177,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  177,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  177,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  177,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  177,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  178,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Montserrat
    {  178,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  178,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  178,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  178,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  178,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  178,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  179,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Namibia
    {  179,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  179,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  179,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  179,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  179,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  179,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  179,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  179,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  179,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  179,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  180,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Nauru
    {  180,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  180,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  180,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  180,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  180,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  180,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  181,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Netherlands
    {  181,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  181,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  181,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  181,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  181,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  181,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  181,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  181,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  181,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  182,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/New Zealand
    {  182,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  182,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  182,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  182,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  182,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  182,  295,  157,  160,  164,  3,  4,  4, }, // Australia_Central
    {  182,  313,  168,  172,  177,  4,  5,  5, }, // Australia_CentralWestern
    {  182,  338,  182,  185,  189,  3,  4,  4, }, // Australia_Eastern
    {  182,  356,  193,  196,  200,  3,  4,  4, }, // Australia_Western
    {  182,  487,  385,  255,  260,  4,  5,  5, }, // Chatham
    {  182,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  182, 1188,  211,  214,  218,  3,  4,  4, }, // Lord_Howe
    {  182, 1346,  222,  225,  229,  3,  4,  4, }, // New_Zealand
    {  183,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Nigeria
    {  183,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  183,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  183,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  183,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  183,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  183,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  183,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  183,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  183,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  183,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  184,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Niue
    {  184,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  184,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  184,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  184,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  184,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  184,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  185,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Norfolk Island
    {  185,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  185,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  185,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  185,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  185,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  185,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  186,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Northern Mariana Islands
    {  186,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  186,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  186,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  186,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  186,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  186,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  187,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Pakistan
    {  187,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  187,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  187,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  187,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  187,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  187,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  188,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Palau
    {  188,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  188,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  188,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  188,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  188,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  188,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  189,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Papua New Guinea
    {  189,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  189,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  189,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  189,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  189,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  189,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  191,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Pitcairn
    {  191,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  191,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  191,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  191,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  191,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  191,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  193,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Rwanda
    {  193,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  193,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  193,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  193,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  193,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  193,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  193,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  193,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  193,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  193,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  194,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Saint Helena
    {  194,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  194,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  194,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  194,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  194,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  194,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  195,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Saint Kitts and Nevis
    {  195,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  195,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  195,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  195,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  195,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  195,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  196,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Saint Lucia
    {  196,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  196,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  196,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  196,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  196,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  196,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  197,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Saint Vincent and Grenadines
    {  197,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  197,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  197,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  197,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  197,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  197,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  198,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Samoa
    {  198,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  198,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  198,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  198,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  198,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  198,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  199,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Seychelles
    {  199,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  199,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  199,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  199,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  199,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  199,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  200,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Sierra Leone
    {  200,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  200,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  200,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  200,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  200,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  200,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  200,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  200,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  200,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  200,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  201,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Singapore
    {  201,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  201,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  201,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  201,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  201,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  201,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  201, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  201, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  202,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Sint Maarten
    {  202,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  202,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  202,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  202,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  202,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  202,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  203,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Slovenia
    {  203,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  203,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  203,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  203,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  203,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  203,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  203,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  203,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  203,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  204,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Solomon Islands
    {  204,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  204,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  204,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  204,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  204,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  204,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  205,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/South Africa
    {  205,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  205,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  205,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  205,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  205,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  205,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  205,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  205,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  205,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  205,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  206,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/South Sudan
    {  206,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  206,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  206,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  206,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  206,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  206,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  206,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  206,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  206,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  206,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  207,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Sudan
    {  207,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  207,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  207,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  207,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  207,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  207,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  207,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  207,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  207,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  207,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  208,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Sweden
    {  208,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  208,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  208,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  208,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  208,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  208,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  208,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  208,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  208,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  209,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Switzerland
    {  209,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  209,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  209,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  209,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  209,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  209,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  209,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  209,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  209,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  210,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Tanzania
    {  210,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  210,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  210,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  210,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  210,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  210,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  210,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  210,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  210,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  210,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  211,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Tokelau
    {  211,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  211,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  211,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  211,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  211,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  211,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  212,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Tonga
    {  212,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  212,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  212,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  212,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  212,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  212,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  213,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Trinidad and Tobago
    {  213,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  213,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  213,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  213,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  213,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  213,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  214,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Turks and Caicos Islands
    {  214,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  214,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  214,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  214,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  214,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  214,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  215,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Tuvalu
    {  215,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  215,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  215,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  215,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  215,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  215,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  216,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Uganda
    {  216,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  216,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  216,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  216,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  216,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  216,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  216,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  216,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  216,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  216,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  217,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/United Arab Emirates
    {  217,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  217,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  217,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  217,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  217,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  217,  882,    0,   82,    0,  0,  3,  0, }, // Gulf
    {  217,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  218,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/United Kingdom
    {  218,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  218,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  218,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  218,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  218,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  218,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  218,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  218,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  218,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  221,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/Vanuatu
    {  221,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  221,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  221,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  221,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  221,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  221,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  222,  107,  154,  154,  154,  3,  3,  3, }, // Alaska English/Latin/world
    {  222,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  222,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  222,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  222,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  222,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  222,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  223,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Zambia
    {  223,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  223,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  223,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  223,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  223,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  223,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  223,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  223,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  223,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  223,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  224,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central English/Latin/Zimbabwe
    {  224,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  224,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  224,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  224,  107,  154,  154,  154,  3,  3,  3, }, // Alaska
    {  224,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  224,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  224,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  224,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  224,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  224,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  229,   17,    0,   65,    0,  0,  3,  0, }, // Africa_Central Ewe/Latin/Ghana
    {  229,   32,    0,   68,    0,  0,  3,  0, }, // Africa_Eastern
    {  229,   65,    0,   71,    0,  0,  4,  0, }, // Africa_Southern
    {  229,   81,   75,   75,   78,  3,  3,  4, }, // Africa_Western
    {  236,  107,  389,  392,  396,  3,  4,  4, }, // Alaska French/Latin/France
    {  236,  142,  400,  402,  405,  2,  3,  3, }, // America_Central
    {  236,  158,  396,  408,  411,  2,  3,  3, }, // America_Eastern
    {  236,  174,  414,  416,  419,  2,  3,  3, }, // America_Mountain
    {  236,  191,  422,  424,  427,  2,  3,  3, }, // America_Pacific
    {  236,  286,   54,  392,  396,  2,  3,  3, }, // Atlantic
    {  236,  548,  430,  433,  437,  3,  4,  4, }, // Cuba
    {  236,  653,   98,  101,  441,  3,  4,  4, }, // Europe_Eastern
    {  236,  691,  105,  108,  445,  3,  4,  4, }, // Europe_Western
    {  236,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  236,  841,  449,  452,  456,  3,  4,  4, }, // Greenland_Eastern
    {  236,  859,  460,  463,  467,  3,  4,  4, }, // Greenland_Western
    {  236,  894,  471,  474,  478,  3,  4,  4, }, // Hawaii_Aleutian
    {  236, 1281,  482,  486,  491,  4,  5,  5, }, // Mexico_Pacific
    {  236, 1358,  496,  499,  503,  3,  4,  4, }, // Newfoundland
    {  236, 1502,  482,  486,  491,  3,  4,  4, }, // Pierre_Miquelon
    {  243,  107,  389,  392,  396,  3,  4,  4, }, // Alaska French/Latin/Canada
    {  243,  142,  400,  402,  507,  2,  3,  3, }, // America_Central
    {  243,  158,  396,  408,  510,  2,  3,  3, }, // America_Eastern
    {  243,  174,  414,  416,  513,  2,  3,  3, }, // America_Mountain
    {  243,  191,  422,  424,  516,  2,  3,  3, }, // America_Pacific
    {  243,  286,   54,  392,  396,  2,  3,  3, }, // Atlantic
    {  243,  548,  430,  433,  437,  3,  4,  4, }, // Cuba
    {  243,  653,   98,  101,  441,  3,  4,  4, }, // Europe_Eastern
    {  243,  691,  105,  108,  445,  3,  4,  4, }, // Europe_Western
    {  243,  841,  449,  452,  456,  3,  4,  4, }, // Greenland_Eastern
    {  243,  859,  460,  463,  467,  3,  4,  4, }, // Greenland_Western
    {  243,  894,  471,  474,  478,  3,  4,  4, }, // Hawaii_Aleutian
    {  243, 1281,  482,  486,  491,  4,  5,  5, }, // Mexico_Pacific
    {  243, 1358,  212,  499,   54,  2,  3,  3, }, // Newfoundland
    {  243, 1502,  482,  486,  491,  3,  4,  4, }, // Pierre_Miquelon
    {  251,  720,    0,  519,    0,  0,  3,  0, }, // French_Guiana French/Latin/French Guiana
    {  282,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Friulian/Latin/Italy
    {  282,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  282,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  282,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  307,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Gaelic/Latin/United Kingdom
    {  307,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  307,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  307,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  307,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  307,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  307,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  307,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  307,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  307,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  307,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  309,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Galician/Latin/Spain
    {  309,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  309,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  309,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  314,  638,  121,  121,  124,  3,  3,  4, }, // Europe_Central German/Latin/Germany
    {  314,  653,  128,  128,  131,  3,  3,  4, }, // Europe_Eastern
    {  314,  691,  135,  135,  138,  3,  3,  4, }, // Europe_Western
    {  321,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Greek/Greek/Greece
    {  321,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  321,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  324,  925,    0,  112,    0,  0,  3,  0, }, // India Gujarati/Gujarati/India
    {  331,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Hawaiian/Latin/United States
    {  331,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  333,  925,    0,  112,    0,  0,  3,  0, }, // India Hindi/Devanagari/India
    {  334,  107,  154,  154,  154,  3,  3,  3, }, // Alaska Hindi/Latin/India
    {  334,  142,  154,  154,  154,  3,  3,  3, }, // America_Central
    {  334,  158,  154,  154,  154,  3,  3,  3, }, // America_Eastern
    {  334,  174,  154,  154,  154,  3,  3,  3, }, // America_Mountain
    {  334,  191,  154,  154,  154,  3,  3,  3, }, // America_Pacific
    {  334,  286,  154,  154,  154,  3,  3,  3, }, // Atlantic
    {  334,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  334,  882,    0,   82,    0,  0,  3,  0, }, // Gulf
    {  334,  894,  154,  154,  154,  3,  3,  3, }, // Hawaii_Aleutian
    {  334,  925,    0,  112,    0,  0,  3,  0, }, // India
    {  335,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Hungarian/Latin/Hungary
    {  335,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  335,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  335,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  340,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Indonesian/Latin/Indonesia
    {  340,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  340,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  340,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  340,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  340,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  340,  548,  522,  531,  541,  9, 10, 10, }, // Cuba
    {  340,  841,  301,  551,  555,  3,  4,  4, }, // Greenland_Eastern
    {  340,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  340,  954,    0,  310,    0,  0,  4,  0, }, // Indonesia_Central
    {  340,  972,    0,  310,    0,  0,  3,  0, }, // Indonesia_Eastern
    {  340,  990,    0,  314,    0,  0,  3,  0, }, // Indonesia_Western
    {  340, 1358,  253,  331,  334,  2,  3,  3, }, // Newfoundland
    {  340, 1502,  353,  356,  360,  3,  4,  4, }, // Pierre_Miquelon
    {  345,  191,  559,  562,  566,  3,  4,  4, }, // America_Pacific Irish/Latin/Ireland
    {  345,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  345,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  345,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  345,  799,    0,  570,    0,  0,  3,  0, }, // GMT
    {  347,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Italian/Latin/Italy
    {  347,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  347,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  351, 1034,  154,  573,  576,  3,  3,  3, }, // Japan Japanese/Japanese/Japan
    {  361,  925,    0,  112,    0,  0,  3,  0, }, // India Kannada/Kannada/India
    {  364,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern Kazakh/Cyrillic/Kazakhstan
    {  370,  925,    0,  112,    0,  0,  3,  0, }, // India Konkani/Devanagari/India
    {  378,  107,  579,  582,  586,  3,  4,  4, }, // Alaska Kurdish/Latin/Turkey
    {  378,  142,  590,  592,  595,  2,  3,  3, }, // America_Central
    {  378,  158,  149,  598,  601,  2,  3,  3, }, // America_Eastern
    {  378,  174,  604,  606,  609,  2,  3,  3, }, // America_Mountain
    {  378,  191,  612,  614,  617,  2,  3,  3, }, // America_Pacific
    {  378,  286,   71,  582,  586,  2,  3,  3, }, // Atlantic
    {  378,  548,  620,  622,  625,  2,  3,  3, }, // Cuba
    {  378,  841,  628,  631,  635,  3,  4,  4, }, // Greenland_Eastern
    {  378,  859,  639,  643,  648,  4,  5,  5, }, // Greenland_Western
    {  378,  894,  653,  657,  661,  4,  4,  4, }, // Hawaii_Aleutian
    {  378, 1358,  665,  668,  672,  3,  4,  4, }, // Newfoundland
    {  378, 1892,    0,  676,    0,  0,  2,  0, }, // Yukon
    {  385,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Latvian/Latin/Latvia
    {  385,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  385,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  392,  638,  121,  121,  124,  3,  3,  4, }, // Europe_Central Lower Sorbian/Latin/Germany
    {  392,  653,  128,  128,  131,  3,  3,  4, }, // Europe_Eastern
    {  392,  691,  135,  135,  138,  3,  3,  4, }, // Europe_Western
    {  401,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Macedonian/Cyrillic/Macedonia
    {  401,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  401,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  407,  925,    0,  112,    0,  0,  3,  0, }, // India Malayalam/Malayalam/India
    {  408,  799,    0,   51,    0,  0,  3,  0, }, // GMT Malay/Latin/Malaysia
    {  408, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  408, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  414,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Maltese/Latin/Malta
    {  414,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  414,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  414,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  417,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Manx/Latin/Isle of Man
    {  417,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  417,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  417,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  420,  925,    0,  112,    0,  0,  3,  0, }, // India Marathi/Devanagari/India
    {  436,  925,    0,  112,    0,  0,  3,  0, }, // India Nepali/Devanagari/India
    {  443,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Northern Sami/Latin/Norway
    {  443,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  443,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  443,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  444,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Northern Sami/Latin/Finland
    {  444,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  444,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  444,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  448,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Norwegian Bokmal/Latin/Norway
    {  448,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  448,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  448,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  448,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  448,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  448,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  448,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  448,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  448,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  448,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  450,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Norwegian Nynorsk/Latin/Norway
    {  450,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  450,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  450,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  450,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  450,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  450,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  450,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  450,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  450,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  450,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  456,  799,    0,   51,    0,  0,  3,  0, }, // GMT Odia/Odia/India
    {  456,  925,    0,  112,    0,  0,  3,  0, }, // India
    {  468,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Polish/Latin/Poland
    {  468,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  468,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  469,    0,  157,  157,  160,  3,  3,  4, }, // Acre Portuguese/Latin/Brazil
    {  469,  135,  678,  678,  681,  3,  3,  4, }, // Amazon
    {  469,  437,  245,  245,  248,  3,  3,  4, }, // Brasilia
    {  470,    0,  154,  154,  154,  3,  3,  3, }, // Acre Portuguese/Latin/Angola
    {  470,  135,  154,  154,  154,  3,  3,  3, }, // Amazon
    {  470,  385,  154,  154,  154,  3,  3,  3, }, // Azores
    {  470,  437,  154,  154,  154,  3,  3,  3, }, // Brasilia
    {  470,  638,  154,  154,  154,  3,  3,  3, }, // Europe_Central
    {  470,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern
    {  470,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  471,    0,  154,  154,  154,  3,  3,  3, }, // Acre Portuguese/Latin/Cape Verde
    {  471,  135,  154,  154,  154,  3,  3,  3, }, // Amazon
    {  471,  385,  685,  154,  154,  4,  3,  3, }, // Azores
    {  471,  437,  154,  154,  154,  3,  3,  3, }, // Brasilia
    {  471,  638,  154,  154,  154,  3,  3,  3, }, // Europe_Central
    {  471,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern
    {  471,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  472,    0,  154,  154,  154,  3,  3,  3, }, // Acre Portuguese/Latin/Equatorial Guinea
    {  472,  135,  154,  154,  154,  3,  3,  3, }, // Amazon
    {  472,  385,  685,  685,  689,  4,  4,  5, }, // Azores
    {  472,  437,  154,  154,  154,  3,  3,  3, }, // Brasilia
    {  472,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  472,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  472,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  473,    0,  154,  154,  154,  3,  3,  3, }, // Acre Portuguese/Latin/Guinea-Bissau
    {  473,  135,  154,  154,  154,  3,  3,  3, }, // Amazon
    {  473,  385,  154,  154,  154,  3,  3,  3, }, // Azores
    {  473,  437,  154,  154,  154,  3,  3,  3, }, // Brasilia
    {  473,  638,  154,  154,  154,  3,  3,  3, }, // Europe_Central
    {  473,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern
    {  473,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  474,    0,  154,  154,  154,  3,  3,  3, }, // Acre Portuguese/Latin/Luxembourg
    {  474,  135,  154,  154,  154,  3,  3,  3, }, // Amazon
    {  474,  385,  685,  685,  689,  4,  4,  5, }, // Azores
    {  474,  437,  154,  154,  154,  3,  3,  3, }, // Brasilia
    {  474,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  474,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  474,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  475,    0,  154,  154,  154,  3,  3,  3, }, // Acre Portuguese/Latin/Macao
    {  475,  135,  154,  154,  154,  3,  3,  3, }, // Amazon
    {  475,  385,  154,  154,  154,  3,  3,  3, }, // Azores
    {  475,  437,  154,  154,  154,  3,  3,  3, }, // Brasilia
    {  475,  638,  154,  154,  154,  3,  3,  3, }, // Europe_Central
    {  475,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern
    {  475,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  476,    0,  154,  154,  154,  3,  3,  3, }, // Acre Portuguese/Latin/Mozambique
    {  476,  135,  154,  154,  154,  3,  3,  3, }, // Amazon
    {  476,  385,  154,  154,  154,  3,  3,  3, }, // Azores
    {  476,  437,  154,  154,  154,  3,  3,  3, }, // Brasilia
    {  476,  638,  154,  154,  154,  3,  3,  3, }, // Europe_Central
    {  476,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern
    {  476,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  477,    0,  154,  154,  154,  3,  3,  3, }, // Acre Portuguese/Latin/Portugal
    {  477,  135,  154,  154,  154,  3,  3,  3, }, // Amazon
    {  477,  385,  685,  685,  689,  4,  4,  5, }, // Azores
    {  477,  437,  154,  154,  154,  3,  3,  3, }, // Brasilia
    {  477,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  477,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  477,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  478,    0,  154,  154,  154,  3,  3,  3, }, // Acre Portuguese/Latin/Sao Tome and Principe
    {  478,  135,  154,  154,  154,  3,  3,  3, }, // Amazon
    {  478,  385,  154,  154,  154,  3,  3,  3, }, // Azores
    {  478,  437,  154,  154,  154,  3,  3,  3, }, // Brasilia
    {  478,  638,  154,  154,  154,  3,  3,  3, }, // Europe_Central
    {  478,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern
    {  478,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  479,    0,  154,  154,  154,  3,  3,  3, }, // Acre Portuguese/Latin/Switzerland
    {  479,  135,  154,  154,  154,  3,  3,  3, }, // Amazon
    {  479,  385,  685,  685,  689,  4,  4,  5, }, // Azores
    {  479,  437,  154,  154,  154,  3,  3,  3, }, // Brasilia
    {  479,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central
    {  479,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  479,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  480,    0,  154,  154,  154,  3,  3,  3, }, // Acre Portuguese/Latin/Timor-Leste
    {  480,  135,  154,  154,  154,  3,  3,  3, }, // Amazon
    {  480,  385,  154,  154,  154,  3,  3,  3, }, // Azores
    {  480,  437,  154,  154,  154,  3,  3,  3, }, // Brasilia
    {  480,  638,  154,  154,  154,  3,  3,  3, }, // Europe_Central
    {  480,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern
    {  480,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  482,  925,    0,  112,    0,  0,  3,  0, }, // India Punjabi/Gurmukhi/India
    {  484, 1469,  350,  350,  694,  3,  3,  4, }, // Peru Quechua/Latin/Peru
    {  485,  422,    0,  698,    0,  0,  3,  0, }, // Bolivia Quechua/Latin/Bolivia
    {  485, 1469,  154,  154,  154,  3,  3,  3, }, // Peru
    {  486,  630,    0,  287,    0,  0,  3,  0, }, // Ecuador Quechua/Latin/Ecuador
    {  486, 1469,  154,  154,  154,  3,  3,  3, }, // Peru
    {  487,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Romanian/Latin/Romania
    {  487,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  487,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  487,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  507,  107,  701,  704,  708,  3,  4,  4, }, // Alaska Sardinian/Latin/Italy
    {  507,  142,  687,  712,  715,  2,  3,  3, }, // America_Central
    {  507,  158,  718,  720,  723,  2,  3,  3, }, // America_Eastern
    {  507,  174,  726,  729,  733,  3,  4,  4, }, // America_Mountain
    {  507,  191,  737,  739,  742,  2,  3,  3, }, // America_Pacific
    {  507,  286,  701,  704,  708,  2,  3,  3, }, // Atlantic
    {  507,  799,    0,  745,    0,  0,  3,  0, }, // GMT
    {  509,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Cyrillic/Serbia
    {  509,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  509,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  509,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  510,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Cyrillic/Bosnia and Herzegovina
    {  510,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  510,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  510,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  513,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Latin/Bosnia and Herzegovina
    {  513,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  513,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  513,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  514,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Latin/Kosovo
    {  514,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  514,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  514,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  515,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Latin/Montenegro
    {  515,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  515,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  515,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  516,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Serbian/Latin/Serbia
    {  516,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  516,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  516,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  522,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Silesian/Latin/Poland
    {  522,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  522,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  527,  638,  142,  142,  145,  3,  3,  4, }, // Europe_Central Slovak/Latin/Slovakia
    {  530,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian Somali/Latin/Somalia
    {  541,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Spanish/Latin/Spain
    {  541,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  541,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  541,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  542,  240,  236,  236,  748,  3,  3,  4, }, // Argentina Spanish/Latin/Argentina
    {  542,  250,  752,  752,  756,  4,  4,  5, }, // Argentina_Western
    {  542,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern
    {  542,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  543,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Belize
    {  543,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  544,  422,    0,  698,    0,  0,  3,  0, }, // Bolivia Spanish/Latin/Bolivia
    {  544,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern
    {  544,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  545,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Brazil
    {  545,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  548,  495,  761,  761,  764,  3,  3,  4, }, // Chile Spanish/Latin/Chile
    {  548,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern
    {  548,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  549,  534,  768,  768,  271,  3,  3,  4, }, // Colombia Spanish/Latin/Colombia
    {  549,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern
    {  549,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  550,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Costa Rica
    {  550,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  551,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Cuba
    {  551,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  552,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Dominican Republic
    {  552,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  553,  630,    0,  287,    0,  0,  3,  0, }, // Ecuador Spanish/Latin/Ecuador
    {  553,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern
    {  553,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  554,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/El Salvador
    {  554,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  556,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Guatemala
    {  556,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  557,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Honduras
    {  557,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  558,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Latin America
    {  558,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  560,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Nicaragua
    {  560,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  561,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Panama
    {  561,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  562,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Paraguay
    {  562,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  563,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Peru
    {  563,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  563, 1469,  350,  350,  694,  3,  3,  4, }, // Peru
    {  565,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Puerto Rico
    {  565,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  566,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Spanish/Latin/United States
    {  566,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  566,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  566,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  566,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  566,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  566,  638,  154,  154,  154,  3,  3,  3, }, // Europe_Central
    {  566,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern
    {  566,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  566,  799,    0,  154,    0,  0,  3,  0, }, // GMT
    {  566,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  567,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Uruguay
    {  567,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  567, 1777,  364,  364,  367,  3,  3,  4, }, // Uruguay
    {  568,  653,  154,  154,  154,  3,  3,  3, }, // Europe_Eastern Spanish/Latin/Venezuela
    {  568,  691,  154,  154,  154,  3,  3,  3, }, // Europe_Western
    {  568, 1811,    0,  371,    0,  0,  3,  0, }, // Venezuela
    {  577,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Swedish/Latin/Sweden
    {  577,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  577,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  577,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  580,  638,  121,  121,  124,  3,  3,  4, }, // Europe_Central Swiss German/Latin/Switzerland
    {  580,  653,  128,  128,  131,  3,  3,  4, }, // Europe_Eastern
    {  580,  691,  135,  135,  138,  3,  3,  4, }, // Europe_Western
    {  590,  925,    0,  112,    0,  0,  3,  0, }, // India Tamil/Tamil/India
    {  591,  925,    0,  154,    0,  0,  3,  0, }, // India Tamil/Tamil/Malaysia
    {  591, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  591, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  592,  925,    0,  154,    0,  0,  3,  0, }, // India Tamil/Tamil/Singapore
    {  592, 1219,    0,  115,    0,  0,  3,  0, }, // Malaysia
    {  592, 1624,    0,  118,    0,  0,  3,  0, }, // Singapore
    {  594,  142,  771,  774,  778,  3,  4,  5, }, // America_Central Taroko/Latin/Taiwan
    {  594,  158,  783,  786,  790,  3,  4,  5, }, // America_Eastern
    {  594,  174,  795,  798,  802,  3,  4,  4, }, // America_Mountain
    {  594,  191,  576,  806,  810,  2,  4,  5, }, // America_Pacific
    {  594,  286,  815,  818,  822,  3,  4,  5, }, // Atlantic
    {  594,  638,  771,  771,  827,  2,  2,  4, }, // Europe_Central
    {  594,  653,  783,  783,  831,  2,  2,  4, }, // Europe_Eastern
    {  594,  691,  573,  573,  835,  2,  2,  4, }, // Europe_Western
    {  594,  799,    0,  839,    0,  0,  3,  0, }, // GMT
    {  597,  925,    0,  112,    0,  0,  3,  0, }, // India Telugu/Telugu/India
    {  613, 1750,   26,   26,   28,  3,  3,  4, }, // Turkmenistan Turkmen/Latin/Turkmenistan
    {  616,  638,  121,  121,  124,  3,  3,  4, }, // Europe_Central Upper Sorbian/Latin/Germany
    {  616,  653,  128,  128,  131,  3,  3,  4, }, // Europe_Eastern
    {  616,  691,  135,  135,  138,  3,  3,  4, }, // Europe_Western
    {  626,  107,    0,    3,    7,  3,  4,  4, }, // Alaska Vietnamese/Latin/Vietnam
    {  626,  142,   11,   13,   16,  2,  3,  3, }, // America_Central
    {  626,  158,   19,   21,   24,  2,  3,  3, }, // America_Eastern
    {  626,  174,   27,   29,   32,  2,  3,  3, }, // America_Mountain
    {  626,  191,   35,   37,   40,  2,  3,  3, }, // America_Pacific
    {  626,  286,   43,   45,   48,  2,  3,  3, }, // Atlantic
    {  626,  894,   54,   57,   61,  3,  4,  4, }, // Hawaii_Aleutian
    {  630,  638,  121,  121,  124,  3,  3,  4, }, // Europe_Central Walser/Latin/Switzerland
    {  630,  653,  128,  128,  131,  3,  3,  4, }, // Europe_Eastern
    {  630,  691,  135,  135,  138,  3,  3,  4, }, // Europe_Western
    {  632,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Welsh/Latin/United Kingdom
    {  632,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  632,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  632,  799,    0,   51,    0,  0,  3,  0, }, // GMT
    {  638,  638,   91,   91,   94,  3,  3,  4, }, // Europe_Central Western Frisian/Latin/Netherlands
    {  638,  653,   98,   98,  101,  3,  3,  4, }, // Europe_Eastern
    {  638,  691,  105,  105,  108,  3,  3,  4, }, // Europe_Western
    {  649,    0,  157,  157,  160,  3,  3,  4, }, // Acre Kaingang/Latin/Brazil
    {  649,  135,  678,  678,  681,  3,  3,  4, }, // Amazon
    {  649,  437,  245,  245,  248,  3,  3,  4, }, // Brasilia
    {  650,    0,  157,  157,  160,  3,  3,  4, }, // Acre Nheengatu/Latin/Brazil
    {  650,  135,  678,  678,  681,  3,  3,  4, }, // Amazon
    {  650,  437,  245,  245,  248,  3,  3,  4, }, // Brasilia
    {  666,  799,    0,  842,    0,  0,  2,  0, }, // GMT Anii/Latin/Benin
}; // Metazone short name table

// Indexing matches that of locale_data in qlocale_data_p.h
static inline constexpr LocaleZoneData localeZoneData[] = {
    // LOCALE_TAGS(lng,scp,ter) xct1st, zn1st, ml1st, ms1st, (+hr, -hr, gmt, flbk, rgen, rstd, rdst){ind,sz}
    { LOCALE_TAGS(  1,  0,  0)     0,     0,     0,     0,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // C/AnyScript/AnyTerritory
    { LOCALE_TAGS(  2, 27, 90)    42,   158,     8,     4,    0,    6,    0,    0,   39,   47,   66,  6,  6,  5,  7,  8, 19, 17, }, // Abkhazian/Cyrillic/Georgia
    { LOCALE_TAGS(  3, 66, 77)   459,   315,     8,     7,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Afar/Latin/Ethiopia
    { LOCALE_TAGS(  3, 66, 67)   500,   315,     8,     8,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Afar/Latin/Djibouti
    { LOCALE_TAGS(  3, 66, 74)   500,   315,     8,     8,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Afar/Latin/Eritrea
    { LOCALE_TAGS(  4, 66,216)   500,   315,     8,     8,    0,    6,    0,    0,   97,  103,  118,  6,  6,  5,  7,  6, 15, 12, }, // Afrikaans/Latin/South Africa
    { LOCALE_TAGS(  4, 66,162)   613,   462,    12,    11,    0,    6,    0,    0,   97,  103,  118,  6,  6,  5,  7,  6, 15, 12, }, // Afrikaans/Latin/Namibia
    { LOCALE_TAGS(  5, 66, 40)   613,   462,    12,    11,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Aghem/Latin/Cameroon
    { LOCALE_TAGS(  6, 66, 92)   654,   462,    12,    12,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Akan/Latin/Ghana
    { LOCALE_TAGS(  8, 66, 40)   695,   462,    12,    13,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Akoose/Latin/Cameroon
    { LOCALE_TAGS(  9, 66,  3)   736,   462,    12,    14,    0,    6,    0,    0,  130,  137,  154,  6,  6,  5,  7,  7, 17, 14, }, // Albanian/Latin/Albania
    { LOCALE_TAGS(  9, 66,126)  1027,   619,    12,    17,    0,    6,    0,    0,  130,  137,  154,  6,  6,  5,  7,  7, 17, 14, }, // Albanian/Latin/Kosovo
    { LOCALE_TAGS(  9, 66,140)  1027,   619,    12,    17,    0,    6,    0,    0,  130,  137,  154,  6,  6,  5,  7,  7, 17, 14, }, // Albanian/Latin/Macedonia
    { LOCALE_TAGS( 11, 33, 77)  1027,   619,    12,    17,   12,   17,    5,    0,  168,  173,  184,  5,  5,  8,  7,  5, 11, 15, }, // Amharic/Ethiopic/Ethiopia
    { LOCALE_TAGS( 14,  4, 71)  1445,   766,    12,    20,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Egypt
    { LOCALE_TAGS( 14,  4,  4)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Algeria
    { LOCALE_TAGS( 14,  4, 19)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Bahrain
    { LOCALE_TAGS( 14,  4, 48)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Chad
    { LOCALE_TAGS( 14,  4, 55)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Comoros
    { LOCALE_TAGS( 14,  4, 67)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Djibouti
    { LOCALE_TAGS( 14,  4, 74)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Eritrea
    { LOCALE_TAGS( 14,  4,113)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Iraq
    { LOCALE_TAGS( 14,  4,116)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Israel
    { LOCALE_TAGS( 14,  4,122)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Jordan
    { LOCALE_TAGS( 14,  4,127)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Kuwait
    { LOCALE_TAGS( 14,  4,132)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Lebanon
    { LOCALE_TAGS( 14,  4,135)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Libya
    { LOCALE_TAGS( 14,  4,149)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Mauritania
    { LOCALE_TAGS( 14,  4,159)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Morocco
    { LOCALE_TAGS( 14,  4,176)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Oman
    { LOCALE_TAGS( 14,  4,180)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Palestinian Territories
    { LOCALE_TAGS( 14,  4,190)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Qatar
    { LOCALE_TAGS( 14,  4,205)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Saudi Arabia
    { LOCALE_TAGS( 14,  4,215)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Somalia
    { LOCALE_TAGS( 14,  4,219)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/South Sudan
    { LOCALE_TAGS( 14,  4,222)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Sudan
    { LOCALE_TAGS( 14,  4,227)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Syria
    { LOCALE_TAGS( 14,  4,238)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Tunisia
    { LOCALE_TAGS( 14,  4,245)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/United Arab Emirates
    { LOCALE_TAGS( 14,  4,257)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Western Sahara
    { LOCALE_TAGS( 14,  4,258)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/world
    { LOCALE_TAGS( 14,  4,259)  1863,   915,    13,    23,    0,    6,   13,    0,  199,  207,  222,  6,  6,  8,  7,  8, 15, 15, }, // Arabic/Arabic/Yemen
    { LOCALE_TAGS( 15, 66,220)  1863,   915,    13,    23,    0,    6,    0,    0,  237,  247,  266,  6,  6,  5,  7, 10, 19, 20, }, // Aragonese/Latin/Spain
    { LOCALE_TAGS( 17,  5, 12)  1904,   924,    13,    24,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Armenian/Armenian/Armenia
    { LOCALE_TAGS( 18,  9,110)  2322,  1068,    13,    27,    0,    6,    0,    0,  286,  293,  309,  6,  6,  5,  7,  7, 16, 19, }, // Assamese/Bangla/India
    { LOCALE_TAGS( 19, 66,220)  2739,  1212,    14,    30,    0,    6,    0,    0,  237,  328,  347,  6,  6,  5,  7, 10, 19, 19, }, // Asturian/Latin/Spain
    { LOCALE_TAGS( 20, 66,230)  2879,  1368,    25,    34,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Asu/Latin/Tanzania
    { LOCALE_TAGS( 21, 66,169)  2920,  1368,    25,    35,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Atsam/Latin/Nigeria
    { LOCALE_TAGS( 25, 66, 17)  2961,  1368,    25,    36,    0,    6,    0,    0,  366,  374,  391,  6,  6,  5,  7,  8, 17, 12, }, // Azerbaijani/Latin/Azerbaijan
    { LOCALE_TAGS( 25,  4,112)  3268,  1513,    25,    39,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Azerbaijani/Arabic/Iran
    { LOCALE_TAGS( 25,  4,113)  3304,  1513,    25,    40,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Azerbaijani/Arabic/Iraq
    { LOCALE_TAGS( 25,  4,239)  3304,  1513,    25,    40,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Azerbaijani/Arabic/Turkey
    { LOCALE_TAGS( 25, 27, 17)  3304,  1513,    25,    40,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Azerbaijani/Cyrillic/Azerbaijan
    { LOCALE_TAGS( 26, 66, 40)  3340,  1513,    25,    41,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bafia/Latin/Cameroon
    { LOCALE_TAGS( 28, 66,145)  3381,  1513,    25,    42,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bambara/Latin/Mali
    { LOCALE_TAGS( 28, 90,145)  3422,  1513,    25,    43,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bambara/Nko/Mali
    { LOCALE_TAGS( 30,  9, 20)  3422,  1513,    25,    43,    0,    6,   21,    0,  286,  403,  415,  6,  6,  6,  7,  7, 12, 15, }, // Bangla/Bangla/Bangladesh
    { LOCALE_TAGS( 30,  9,110)  3840,  1669,    26,    46,    0,    6,   21,    0,  286,  403,  415,  6,  6,  6,  7,  7, 12, 15, }, // Bangla/Bangla/India
    { LOCALE_TAGS( 31, 66, 40)  3840,  1669,    26,    46,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Basaa/Latin/Cameroon
    { LOCALE_TAGS( 32, 27,193)  3881,  1669,    26,    47,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bashkir/Cyrillic/Russia
    { LOCALE_TAGS( 33, 66,220)  3922,  1669,    26,    48,    0,   22,    0,    0,  430,  445,  470,  6,  6,  5,  7, 15, 25, 16, }, // Basque/Latin/Spain
    { LOCALE_TAGS( 35, 27, 22)  4069,  1826,    37,    52,    0,    6,    0,    0,  486,  493,  511,  6,  6,  5,  7,  7, 18, 13, }, // Belarusian/Cyrillic/Belarus
    { LOCALE_TAGS( 36, 66,260)  4487,  1970,    37,    55,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bemba/Latin/Zambia
    { LOCALE_TAGS( 37, 66,230)  4528,  1970,    37,    56,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Bena/Latin/Tanzania
    { LOCALE_TAGS( 38, 29,110)  4569,  1970,    37,    57,    0,    6,    0,    0,  524,  531,  542,  6,  6,  5,  7,  7, 11, 13, }, // Bhojpuri/Devanagari/India
    { LOCALE_TAGS( 40, 33, 74)  4610,  1971,    37,    58,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Blin/Ethiopic/Eritrea
    { LOCALE_TAGS( 41, 29,110)  4651,  1971,    37,    59,    0,    6,   27,    0,  555,  560,  573,  6,  6, 11,  7,  5, 13, 17, }, // Bodo/Devanagari/India
    { LOCALE_TAGS( 42, 66, 29)  5069,  2127,    38,    62,    0,   28,   21,    0,    0,  590,  612,  6,  7,  6,  7,  2, 22, 18, }, // Bosnian/Latin/Bosnia and Herzegovina
    { LOCALE_TAGS( 42, 27, 29)  5222,  2283,    42,    65,    0,    6,    0,    0,  630,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Bosnian/Cyrillic/Bosnia and Herzegovina
    { LOCALE_TAGS( 43, 66, 84)  5639,  2438,    42,    68,    0,    6,    0,    0,  639,  645,  659,  6,  6,  5,  7,  6, 14, 11, }, // Breton/Latin/France
    { LOCALE_TAGS( 45, 27, 36)  5760,  2587,    42,    71,    0,    6,   38,    0,    0,  670,  691,  6,  6,  9,  7,  2, 21, 23, }, // Bulgarian/Cyrillic/Bulgaria
    { LOCALE_TAGS( 46, 86,161)  6178,  2734,    42,    74,    0,    6,    0,    0,  714,  723,  737,  6,  6,  5,  7,  9, 14, 22, }, // Burmese/Myanmar/Myanmar
    { LOCALE_TAGS( 47,137,107)  6596,  2880,    42,    77,    0,    6,    0,    0,  759,  763,  769,  6,  6,  5,  7,  4,  6,  6, }, // Cantonese/Traditional Han/Hong Kong
    { LOCALE_TAGS( 47,118, 50)  7014,  3037,    42,    80,    0,    6,    0,    0,  775,   83,   90,  6,  6,  5,  7,  4,  7,  7, }, // Cantonese/Simplified Han/China
    { LOCALE_TAGS( 48, 66,220)  7277,  3194,    42,    83,    0,    6,    0,    0,  779,  790,  808,  6,  6,  5,  7, 11, 18, 16, }, // Catalan/Latin/Spain
    { LOCALE_TAGS( 48, 66,  6)  7421,  3342,    46,    86,    0,    6,    0,    0,  779,  790,  808,  6,  6,  5,  7, 11, 18, 16, }, // Catalan/Latin/Andorra
    { LOCALE_TAGS( 48, 66, 84)  7421,  3342,    46,    86,    0,    6,    0,    0,  779,  790,  808,  6,  6,  5,  7, 11, 18, 16, }, // Catalan/Latin/France
    { LOCALE_TAGS( 48, 66,117)  7421,  3342,    46,    86,    0,    6,    0,    0,  779,  790,  808,  6,  6,  5,  7, 11, 18, 16, }, // Catalan/Latin/Italy
    { LOCALE_TAGS( 49, 66,185)  7421,  3342,    46,    86,    0,    6,   21,    7,  824,  834,  854,  6,  6,  6,  5, 10, 20, 19, }, // Cebuano/Latin/Philippines
    { LOCALE_TAGS( 50, 66,159)  7462,  3486,    46,    89,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Central Atlas Tamazight/Latin/Morocco
    { LOCALE_TAGS( 51,  4,113)  7503,  3486,    46,    90,    0,    6,   47,    0,  873,  882,  895,  6,  6,  9,  7,  9, 13, 14, }, // Central Kurdish/Arabic/Iraq
    { LOCALE_TAGS( 51,  4,112)  7544,  3486,    46,    91,    0,    6,   47,    0,  873,  882,  895,  6,  6,  9,  7,  9, 13, 14, }, // Central Kurdish/Arabic/Iran
    { LOCALE_TAGS( 52, 21, 20)  7544,  3486,    46,    91,    0,    6,   21,    0,  909,  924,  954,  6,  6,  6,  7, 15, 30, 38, }, // Chakma/Chakma/Bangladesh
    { LOCALE_TAGS( 52, 21,110)  7957,  3640,    47,    94,    0,    6,   21,    0,  909,  924,  954,  6,  6,  6,  7, 15, 30, 38, }, // Chakma/Chakma/India
    { LOCALE_TAGS( 54, 27,193)  7957,  3640,    47,    94,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Chechen/Cyrillic/Russia
    { LOCALE_TAGS( 55, 23,248)  8374,  3783,    47,    97,    0,    6,    0,    0,  992, 1000, 1014,  6,  6,  5,  7,  8, 14, 14, }, // Cherokee/Cherokee/United States
    { LOCALE_TAGS( 56, 66,248)  8792,  3927,    54,   101,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Chickasaw/Latin/United States
    { LOCALE_TAGS( 57, 66,243)  8833,  3927,    54,   102,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Chiga/Latin/Uganda
    { LOCALE_TAGS( 58,118, 50)  8874,  3927,    54,   103,    0,    6,    0,   12,  775, 1028, 1034,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/China
    { LOCALE_TAGS( 58,118,107)  9292,  4084,    54,   106,    0,    6,    0,   12,  775, 1028, 1034,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/Hong Kong
    { LOCALE_TAGS( 58,118,139)  9292,  4084,    54,   106,    0,    6,    0,   12,  775, 1028, 1034,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/Macao
    { LOCALE_TAGS( 58,118,210)  9292,  4084,    54,   106,    0,    6,    0,   12,  775, 1028, 1034,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Simplified Han/Singapore
    { LOCALE_TAGS( 58,137,107)  9293,  4086,    56,   106,    0,    6,    0,   12,  759,  763,  769,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Traditional Han/Hong Kong
    { LOCALE_TAGS( 58,137,139)  9423,  4119,    60,   106,    0,    6,    0,   12,  759,  763,  769,  6,  6,  5,  6,  4,  6,  6, }, // Chinese/Traditional Han/Macao
    { LOCALE_TAGS( 58,137,228)  9553,  4152,    64,   106,    0,    6,    0,   12,  759,   83,   90,  6,  6,  5,  6,  4,  7,  7, }, // Chinese/Traditional Han/Taiwan
    { LOCALE_TAGS( 59, 27,193)  9888,  4309,    72,   109,    0,    6,    0,    0, 1040, 1051, 1070,  6,  6,  5,  7, 11, 19, 19, }, // Church/Cyrillic/Russia
    { LOCALE_TAGS( 60, 27,193)  9957,  4333,    72,   110,    0,    6,    0,    0,    0, 1089, 1109,  6,  6,  5,  7,  2, 20, 17, }, // Chuvash/Cyrillic/Russia
    { LOCALE_TAGS( 61, 66, 91) 10374,  4477,    72,   113,    0,    6,    0,    0, 1126, 1137, 1158,  6,  6,  5,  7, 11, 21, 17, }, // Colognian/Latin/Germany
    { LOCALE_TAGS( 63, 66,246) 10480,  4491,    75,   116,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Cornish/Latin/United Kingdom
    { LOCALE_TAGS( 64, 66, 84) 10521,  4491,    79,   117,    0,    6,   56,    0, 1175, 1183, 1198,  6,  6,  5,  7,  8, 15, 17, }, // Corsican/Latin/France
    { LOCALE_TAGS( 66, 66, 60) 10562,  4492,    79,   118,    0,   28,    0,    0,    0,  590,  612,  6,  7,  5,  7,  2, 22, 18, }, // Croatian/Latin/Croatia
    { LOCALE_TAGS( 66, 66, 29) 10689,  4649,    83,   121,    0,   28,    0,    0,    0,  590,  612,  6,  7,  5,  7,  2, 22, 18, }, // Croatian/Latin/Bosnia and Herzegovina
    { LOCALE_TAGS( 67, 66, 64) 10689,  4649,    83,   121,   35,   40,    0,    0, 1215,   83,   90,  5,  5,  5,  7, 15,  7,  7, }, // Czech/Latin/Czechia
    { LOCALE_TAGS( 68, 66, 65) 10858,  4806,    90,   125,   45,   51,    0,    0, 1230, 1236, 1248,  6,  6,  5,  7,  6, 12, 12, }, // Danish/Latin/Denmark
    { LOCALE_TAGS( 68, 66, 95) 10948,  4962,    93,   128,   45,   51,    0,    0, 1230, 1236, 1248,  6,  6,  5,  7,  6, 12, 12, }, // Danish/Latin/Greenland
    { LOCALE_TAGS( 69,132,144) 10948,  4962,    93,   128,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Divehi/Thaana/Maldives
    { LOCALE_TAGS( 70, 29,110) 10989,  4962,    93,   129,    0,    6,    0,    0, 1260, 1267, 1279,  6,  6,  5,  7,  7, 12, 14, }, // Dogri/Devanagari/India
    { LOCALE_TAGS( 71, 66, 40) 11030,  4971,    93,   132,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Duala/Latin/Cameroon
    { LOCALE_TAGS( 72, 66,165) 11071,  4971,    93,   133,    0,    6,    0,    0, 1293, 1303, 1322,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Netherlands
    { LOCALE_TAGS( 72, 66, 13) 11187,  5128,   104,   137,    0,    6,    0,    0, 1293, 1303, 1322,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Aruba
    { LOCALE_TAGS( 72, 66, 23) 11187,  5128,   104,   137,    0,    6,    0,    0, 1293, 1303, 1322,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Belgium
    { LOCALE_TAGS( 72, 66, 44) 11187,  5128,   104,   137,    0,    6,    0,    0, 1293, 1303, 1322,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Caribbean Netherlands
    { LOCALE_TAGS( 72, 66, 62) 11187,  5128,   104,   137,    0,    6,    0,    0, 1293, 1303, 1322,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Curacao
    { LOCALE_TAGS( 72, 66,211) 11187,  5128,   104,   137,    0,    6,    0,    0, 1293, 1303, 1322,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Sint Maarten
    { LOCALE_TAGS( 72, 66,223) 11187,  5128,   104,   137,    0,    6,    0,    0, 1293, 1303, 1322,  6,  6,  5,  7, 10, 19, 15, }, // Dutch/Latin/Suriname
    { LOCALE_TAGS( 73,134, 27) 11187,  5129,   105,   137,    0,    6,   61,   18, 1337,   83,   90,  6,  6, 12,  9, 10,  7,  7, }, // Dzongkha/Tibetan/Bhutan
    { LOCALE_TAGS( 74, 66,124) 11375,  5212,   106,   140,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Embu/Latin/Kenya
    { LOCALE_TAGS( 75, 66,248) 11416,  5212,   106,   141,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United States
    { LOCALE_TAGS( 75, 28,248) 11458,  5370,   114,   145,    0,    6,   73,    0, 1347,   83,   90,  6,  6,  9,  7,  9,  7,  7, }, // English/Deseret/United States
    { LOCALE_TAGS( 75, 66,  5) 11487,  5378,   114,   146,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/American Samoa
    { LOCALE_TAGS( 75, 66,  8) 11487,  5378,   114,   146,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Anguilla
    { LOCALE_TAGS( 75, 66, 10) 11496,  5386,   121,   147,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Antigua and Barbuda
    { LOCALE_TAGS( 75, 66, 15) 11505,  5394,   128,   148,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Australia
    { LOCALE_TAGS( 75, 66, 16) 11514,  5418,   142,   149,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Austria
    { LOCALE_TAGS( 75, 66, 18) 11523,  5429,   152,   150,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Bahamas
    { LOCALE_TAGS( 75, 66, 21) 11532,  5437,   159,   151,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Barbados
    { LOCALE_TAGS( 75, 66, 23) 11541,  5445,   166,   152,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Belgium
    { LOCALE_TAGS( 75, 66, 24) 11550,  5456,   176,   153,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Belize
    { LOCALE_TAGS( 75, 66, 26) 11559,  5464,   183,   154,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Bermuda
    { LOCALE_TAGS( 75, 66, 30) 11568,  5472,   190,   155,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Botswana
    { LOCALE_TAGS( 75, 66, 33) 11577,  5484,   201,   156,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/British Indian Ocean Territory
    { LOCALE_TAGS( 75, 66, 34) 11586,  5492,   208,   157,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/British Virgin Islands
    { LOCALE_TAGS( 75, 66, 38) 11595,  5500,   215,   158,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Burundi
    { LOCALE_TAGS( 75, 66, 40) 11595,  5500,   215,   158,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cameroon
    { LOCALE_TAGS( 75, 66, 41) 11604,  5512,   226,   159,    0,    6,    0,    0,    0,    7, 1356,  6,  6,  5,  7,  7, 16, 23, }, // English/Latin/Canada
    { LOCALE_TAGS( 75, 66, 45) 11613,  5572,   270,   159,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cayman Islands
    { LOCALE_TAGS( 75, 66, 51) 11622,  5580,   277,   160,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Christmas Island
    { LOCALE_TAGS( 75, 66, 53) 11631,  5588,   284,   161,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cocos Islands
    { LOCALE_TAGS( 75, 66, 58) 11640,  5596,   291,   162,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cook Islands
    { LOCALE_TAGS( 75, 66, 63) 11649,  5604,   298,   163,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Cyprus
    { LOCALE_TAGS( 75, 66, 65) 11658,  5612,   305,   164,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Denmark
    { LOCALE_TAGS( 75, 66, 66) 11667,  5623,   315,   165,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Diego Garcia
    { LOCALE_TAGS( 75, 66, 68) 11676,  5631,   322,   166,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Dominica
    { LOCALE_TAGS( 75, 66, 74) 11685,  5639,   329,   167,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Eritrea
    { LOCALE_TAGS( 75, 66, 76) 11694,  5651,   340,   168,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Eswatini
    { LOCALE_TAGS( 75, 66, 78) 11703,  5663,   351,   169,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Europe
    { LOCALE_TAGS( 75, 66, 80) 11712,  5674,   361,   170,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Falkland Islands
    { LOCALE_TAGS( 75, 66, 82) 11721,  5682,   368,   171,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Fiji
    { LOCALE_TAGS( 75, 66, 83) 11730,  5690,   375,   172,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Finland
    { LOCALE_TAGS( 75, 66, 89) 11739,  5701,   385,   173,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Gambia
    { LOCALE_TAGS( 75, 66, 91) 11748,  5713,   396,   174,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Germany
    { LOCALE_TAGS( 75, 66, 92) 11757,  5724,   406,   175,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Ghana
    { LOCALE_TAGS( 75, 66, 93) 11766,  5736,   417,   176,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Gibraltar
    { LOCALE_TAGS( 75, 66, 96) 11775,  5744,   424,   177,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Grenada
    { LOCALE_TAGS( 75, 66, 98) 11784,  5752,   431,   178,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Guam
    { LOCALE_TAGS( 75, 66,100) 11784,  5753,   432,   178,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Guernsey
    { LOCALE_TAGS( 75, 66,103) 11793,  5761,   439,   179,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Guyana
    { LOCALE_TAGS( 75, 66,107) 11802,  5770,   447,   180,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Hong Kong
    { LOCALE_TAGS( 75, 66,110) 11811,  5779,   455,   181,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/India
    { LOCALE_TAGS( 75, 66,111) 11821,  5789,   464,   182,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Indonesia
    { LOCALE_TAGS( 75, 66,114) 11830,  5800,   474,   183,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Ireland
    { LOCALE_TAGS( 75, 66,115) 11839,  5808,   481,   185,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Isle of Man
    { LOCALE_TAGS( 75, 66,116) 11848,  5816,   488,   186,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Israel
    { LOCALE_TAGS( 75, 66,119) 11857,  5824,   495,   187,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Jamaica
    { LOCALE_TAGS( 75, 66,121) 11866,  5832,   502,   188,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Jersey
    { LOCALE_TAGS( 75, 66,124) 11875,  5840,   509,   189,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Kenya
    { LOCALE_TAGS( 75, 66,125) 11884,  5852,   520,   190,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Kiribati
    { LOCALE_TAGS( 75, 66,133) 11893,  5860,   527,   191,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Lesotho
    { LOCALE_TAGS( 75, 66,134) 11902,  5872,   538,   192,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Liberia
    { LOCALE_TAGS( 75, 66,139) 11911,  5884,   549,   193,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Macao
    { LOCALE_TAGS( 75, 66,141) 11920,  5894,   558,   194,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Madagascar
    { LOCALE_TAGS( 75, 66,142) 11929,  5906,   569,   195,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Malawi
    { LOCALE_TAGS( 75, 66,143) 11938,  5918,   580,   196,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Malaysia
    { LOCALE_TAGS( 75, 66,144) 11947,  5927,   588,   197,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Maldives
    { LOCALE_TAGS( 75, 66,146) 11956,  5935,   595,   198,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Malta
    { LOCALE_TAGS( 75, 66,147) 11965,  5943,   602,   199,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Marshall Islands
    { LOCALE_TAGS( 75, 66,150) 11965,  5950,   609,   200,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Mauritius
    { LOCALE_TAGS( 75, 66,153) 11974,  5962,   620,   201,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Micronesia
    { LOCALE_TAGS( 75, 66,158) 11983,  5970,   627,   202,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Montserrat
    { LOCALE_TAGS( 75, 66,162) 11992,  5978,   634,   203,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Namibia
    { LOCALE_TAGS( 75, 66,163) 12001,  5990,   645,   204,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Nauru
    { LOCALE_TAGS( 75, 66,165) 12010,  5998,   652,   205,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Netherlands
    { LOCALE_TAGS( 75, 66,167) 12019,  6009,   662,   206,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/New Zealand
    { LOCALE_TAGS( 75, 66,169) 12028,  6024,   676,   207,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Nigeria
    { LOCALE_TAGS( 75, 66,171) 12037,  6036,   687,   208,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Niue
    { LOCALE_TAGS( 75, 66,172) 12046,  6044,   694,   209,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Norfolk Island
    { LOCALE_TAGS( 75, 66,173) 12055,  6052,   701,   210,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Northern Mariana Islands
    { LOCALE_TAGS( 75, 66,178) 12055,  6059,   708,   211,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Pakistan
    { LOCALE_TAGS( 75, 66,179) 12064,  6067,   715,   212,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Palau
    { LOCALE_TAGS( 75, 66,182) 12073,  6075,   722,   213,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Papua New Guinea
    { LOCALE_TAGS( 75, 66,185) 12082,  6083,   729,   214,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Philippines
    { LOCALE_TAGS( 75, 66,186) 12082,  6083,   729,   214,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Pitcairn
    { LOCALE_TAGS( 75, 66,189) 12091,  6091,   736,   215,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Puerto Rico
    { LOCALE_TAGS( 75, 66,194) 12091,  6091,   736,   215,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Rwanda
    { LOCALE_TAGS( 75, 66,196) 12100,  6103,   747,   216,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Saint Helena
    { LOCALE_TAGS( 75, 66,197) 12109,  6111,   754,   217,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Saint Kitts and Nevis
    { LOCALE_TAGS( 75, 66,198) 12118,  6119,   761,   218,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Saint Lucia
    { LOCALE_TAGS( 75, 66,201) 12127,  6127,   768,   219,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Saint Vincent and Grenadines
    { LOCALE_TAGS( 75, 66,202) 12136,  6135,   775,   220,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Samoa
    { LOCALE_TAGS( 75, 66,208) 12145,  6143,   782,   221,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Seychelles
    { LOCALE_TAGS( 75, 66,209) 12154,  6151,   789,   222,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Sierra Leone
    { LOCALE_TAGS( 75, 66,210) 12163,  6163,   800,   223,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Singapore
    { LOCALE_TAGS( 75, 66,211) 12172,  6173,   809,   224,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Sint Maarten
    { LOCALE_TAGS( 75, 66,213) 12181,  6181,   816,   225,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Slovenia
    { LOCALE_TAGS( 75, 66,214) 12190,  6192,   826,   226,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Solomon Islands
    { LOCALE_TAGS( 75, 66,216) 12199,  6200,   833,   227,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/South Africa
    { LOCALE_TAGS( 75, 66,219) 12208,  6212,   844,   228,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/South Sudan
    { LOCALE_TAGS( 75, 66,222) 12217,  6224,   855,   229,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Sudan
    { LOCALE_TAGS( 75, 66,225) 12226,  6236,   866,   230,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Sweden
    { LOCALE_TAGS( 75, 66,226) 12235,  6247,   876,   231,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Switzerland
    { LOCALE_TAGS( 75, 66,230) 12244,  6258,   886,   232,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Tanzania
    { LOCALE_TAGS( 75, 66,234) 12253,  6270,   897,   233,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Tokelau
    { LOCALE_TAGS( 75, 66,235) 12262,  6278,   904,   234,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Tonga
    { LOCALE_TAGS( 75, 66,236) 12271,  6286,   911,   235,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Trinidad and Tobago
    { LOCALE_TAGS( 75, 66,241) 12280,  6294,   918,   236,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Turks and Caicos Islands
    { LOCALE_TAGS( 75, 66,242) 12289,  6302,   925,   237,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Tuvalu
    { LOCALE_TAGS( 75, 66,243) 12298,  6310,   932,   238,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Uganda
    { LOCALE_TAGS( 75, 66,245) 12307,  6322,   943,   239,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United Arab Emirates
    { LOCALE_TAGS( 75, 66,246) 12307,  6330,   951,   240,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United Kingdom
    { LOCALE_TAGS( 75, 66,247) 12319,  6341,   961,   242,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United States Outlying Islands
    { LOCALE_TAGS( 75, 66,249) 12319,  6341,   961,   242,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/United States Virgin Islands
    { LOCALE_TAGS( 75, 66,252) 12319,  6341,   961,   242,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Vanuatu
    { LOCALE_TAGS( 75, 66,258) 12328,  6349,   968,   243,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/world
    { LOCALE_TAGS( 75, 66,260) 12337,  6357,   975,   244,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Zambia
    { LOCALE_TAGS( 75, 66,261) 12346,  6369,   986,   245,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // English/Latin/Zimbabwe
    { LOCALE_TAGS( 75,115,246) 12355,  6381,   997,   246,    0,    6,   82,    0, 1379,   83,   90,  6,  6,  9,  7,  9,  7,  7, }, // English/Shavian/United Kingdom
    { LOCALE_TAGS( 76, 27,193) 12356,  6381,   997,   247,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Erzya/Cyrillic/Russia
    { LOCALE_TAGS( 77, 66,258) 12464,  6402,   997,   248,    0,    6,   56,    0, 1388, 1399, 1416,  6,  6,  5,  7, 11, 17, 18, }, // Esperanto/Latin/world
    { LOCALE_TAGS( 78, 66, 75) 12760,  6507,   997,   249,    0,   22,   21,    0, 1434,   83,   90,  6,  6,  6,  7,  4,  7,  7, }, // Estonian/Latin/Estonia
    { LOCALE_TAGS( 79, 66, 92) 12898,  6664,   997,   252,    0,    6,   91,    0, 1438, 1450, 1469,  6,  6,  6,  7, 12, 19, 17, }, // Ewe/Latin/Ghana
    { LOCALE_TAGS( 79, 66,233) 12941,  6815,  1001,   255,    0,    6,   91,    0, 1438, 1450, 1469,  6,  6,  6,  7, 12, 19, 17, }, // Ewe/Latin/Togo
    { LOCALE_TAGS( 80, 66, 40) 12941,  6815,  1001,   255,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ewondo/Latin/Cameroon
    { LOCALE_TAGS( 81, 66, 81) 12982,  6815,  1001,   256,    0,    6,    0,    0, 1486, 1492, 1505,  6,  6,  5,  7,  6, 13, 12, }, // Faroese/Latin/Faroe Islands
    { LOCALE_TAGS( 81, 66, 65) 13058,  6959,  1001,   259,    0,    6,    0,    0, 1486, 1492, 1505,  6,  6,  5,  7,  6, 13, 12, }, // Faroese/Latin/Denmark
    { LOCALE_TAGS( 83, 66,185) 13058,  6959,  1001,   259,    0,    6,    0,    0,  824, 1517, 1539,  6,  6,  5,  7, 10, 22, 19, }, // Filipino/Latin/Philippines
    { LOCALE_TAGS( 84, 66, 83) 13105,  7106,  1001,   262,   57,   62,   56,    0, 1558, 1573, 1590,  5,  5,  5,  7, 15, 17, 13, }, // Finnish/Latin/Finland
    { LOCALE_TAGS( 85, 66, 84) 13248,  7263,  1001,   265,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/France
    { LOCALE_TAGS( 85, 66,  4) 13404,  7420,  1017,   269,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Algeria
    { LOCALE_TAGS( 85, 66, 23) 13404,  7420,  1017,   269,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Belgium
    { LOCALE_TAGS( 85, 66, 25) 13404,  7420,  1017,   269,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Benin
    { LOCALE_TAGS( 85, 66, 37) 13404,  7420,  1017,   269,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Burkina Faso
    { LOCALE_TAGS( 85, 66, 38) 13404,  7420,  1017,   269,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Burundi
    { LOCALE_TAGS( 85, 66, 40) 13404,  7420,  1017,   269,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Cameroon
    { LOCALE_TAGS( 85, 66, 41) 13404,  7420,  1017,   269,    0,   22,   56,    0, 1603, 1648, 1666,  6,  6,  5,  7, 10, 18, 18, }, // French/Latin/Canada
    { LOCALE_TAGS( 85, 66, 46) 13424,  7510,  1032,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Central African Republic
    { LOCALE_TAGS( 85, 66, 48) 13424,  7510,  1032,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Chad
    { LOCALE_TAGS( 85, 66, 55) 13424,  7510,  1032,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Comoros
    { LOCALE_TAGS( 85, 66, 56) 13424,  7510,  1032,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Congo - Brazzaville
    { LOCALE_TAGS( 85, 66, 57) 13424,  7510,  1032,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Congo - Kinshasa
    { LOCALE_TAGS( 85, 66, 67) 13424,  7510,  1032,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Djibouti
    { LOCALE_TAGS( 85, 66, 73) 13424,  7510,  1032,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Equatorial Guinea
    { LOCALE_TAGS( 85, 66, 85) 13424,  7510,  1032,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/French Guiana
    { LOCALE_TAGS( 85, 66, 86) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/French Polynesia
    { LOCALE_TAGS( 85, 66, 88) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Gabon
    { LOCALE_TAGS( 85, 66, 97) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Guadeloupe
    { LOCALE_TAGS( 85, 66,102) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Guinea
    { LOCALE_TAGS( 85, 66,104) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Haiti
    { LOCALE_TAGS( 85, 66,118) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Ivory Coast
    { LOCALE_TAGS( 85, 66,138) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Luxembourg
    { LOCALE_TAGS( 85, 66,141) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Madagascar
    { LOCALE_TAGS( 85, 66,145) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Mali
    { LOCALE_TAGS( 85, 66,148) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Martinique
    { LOCALE_TAGS( 85, 66,149) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Mauritania
    { LOCALE_TAGS( 85, 66,150) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Mauritius
    { LOCALE_TAGS( 85, 66,151) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Mayotte
    { LOCALE_TAGS( 85, 66,155) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Monaco
    { LOCALE_TAGS( 85, 66,159) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Morocco
    { LOCALE_TAGS( 85, 66,166) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/New Caledonia
    { LOCALE_TAGS( 85, 66,170) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Niger
    { LOCALE_TAGS( 85, 66,191) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Reunion
    { LOCALE_TAGS( 85, 66,194) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Rwanda
    { LOCALE_TAGS( 85, 66,195) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Saint Barthelemy
    { LOCALE_TAGS( 85, 66,199) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Saint Martin
    { LOCALE_TAGS( 85, 66,200) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Saint Pierre and Miquelon
    { LOCALE_TAGS( 85, 66,206) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Senegal
    { LOCALE_TAGS( 85, 66,208) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Seychelles
    { LOCALE_TAGS( 85, 66,226) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Switzerland
    { LOCALE_TAGS( 85, 66,227) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Syria
    { LOCALE_TAGS( 85, 66,233) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Togo
    { LOCALE_TAGS( 85, 66,238) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Tunisia
    { LOCALE_TAGS( 85, 66,252) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Vanuatu
    { LOCALE_TAGS( 85, 66,256) 13424,  7511,  1033,   271,    0,   22,   56,    0, 1603, 1613, 1632,  6,  6,  5,  7, 10, 19, 16, }, // French/Latin/Wallis and Futuna
    { LOCALE_TAGS( 86, 66,117) 13424,  7511,  1033,   271,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Friulian/Latin/Italy
    { LOCALE_TAGS( 87, 66,206) 13478,  7515,  1037,   272,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Senegal
    { LOCALE_TAGS( 87,  1, 37) 13519,  7515,  1037,   273,    0,    6,   97,    0, 1684, 1701, 1737,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Burkina Faso
    { LOCALE_TAGS( 87,  1, 40) 13519,  7515,  1037,   273,    0,    6,   97,    0, 1684, 1701, 1737,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Cameroon
    { LOCALE_TAGS( 87,  1, 89) 13519,  7515,  1037,   273,    0,    6,   97,    0, 1684, 1701, 1737,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Gambia
    { LOCALE_TAGS( 87,  1, 92) 13519,  7515,  1037,   273,    0,    6,   97,    0, 1684, 1701, 1737,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Ghana
    { LOCALE_TAGS( 87,  1,101) 13519,  7515,  1037,   273,    0,    6,   97,    0, 1684, 1701, 1737,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Guinea-Bissau
    { LOCALE_TAGS( 87,  1,102) 13519,  7515,  1037,   273,    0,    6,   97,    0, 1684, 1701, 1737,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Guinea
    { LOCALE_TAGS( 87,  1,134) 13935,  7667,  1037,   277,    0,    6,   97,    0, 1684, 1701, 1737,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Liberia
    { LOCALE_TAGS( 87,  1,149) 13935,  7667,  1037,   277,    0,    6,   97,    0, 1684, 1701, 1737,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Mauritania
    { LOCALE_TAGS( 87,  1,169) 13935,  7667,  1037,   277,    0,    6,   97,    0, 1684, 1701, 1737,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Nigeria
    { LOCALE_TAGS( 87,  1,170) 13935,  7667,  1037,   277,    0,    6,   97,    0, 1684, 1701, 1737,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Niger
    { LOCALE_TAGS( 87,  1,206) 13935,  7667,  1037,   277,    0,    6,   97,    0, 1684, 1701, 1737,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Senegal
    { LOCALE_TAGS( 87,  1,209) 13935,  7667,  1037,   277,    0,    6,   97,    0, 1684, 1701, 1737,  6,  6,  8,  7, 17, 36, 28, }, // Fulah/Adlam/Sierra Leone
    { LOCALE_TAGS( 87, 66, 37) 13935,  7667,  1037,   277,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Burkina Faso
    { LOCALE_TAGS( 87, 66, 40) 13935,  7667,  1037,   277,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Cameroon
    { LOCALE_TAGS( 87, 66, 89) 13935,  7667,  1037,   277,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Gambia
    { LOCALE_TAGS( 87, 66, 92) 13935,  7667,  1037,   277,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Ghana
    { LOCALE_TAGS( 87, 66,101) 13935,  7667,  1037,   277,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Guinea-Bissau
    { LOCALE_TAGS( 87, 66,102) 13935,  7667,  1037,   277,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Guinea
    { LOCALE_TAGS( 87, 66,134) 13935,  7667,  1037,   277,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Liberia
    { LOCALE_TAGS( 87, 66,149) 13935,  7667,  1037,   277,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Mauritania
    { LOCALE_TAGS( 87, 66,169) 13935,  7667,  1037,   277,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Nigeria
    { LOCALE_TAGS( 87, 66,170) 13935,  7667,  1037,   277,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Niger
    { LOCALE_TAGS( 87, 66,209) 13935,  7667,  1037,   277,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Fulah/Latin/Sierra Leone
    { LOCALE_TAGS( 88, 66,246) 13935,  7667,  1037,   277,    0,    6,    0,    0,    0, 1765, 1775,  6,  6,  5,  7,  2, 10, 18, }, // Gaelic/Latin/United Kingdom
    { LOCALE_TAGS( 89, 66, 92) 14088,  7824,  1048,   281,    0,    6,    0,    0, 1793, 1798, 1816,  6,  6,  5,  7,  5, 18, 20, }, // Ga/Latin/Ghana
    { LOCALE_TAGS( 90, 66,220) 14193,  7848,  1048,   284,    0,    6,    0,    0, 1836, 1847, 1867,  6,  6,  5,  7, 11, 20, 20, }, // Galician/Latin/Spain
    { LOCALE_TAGS( 91, 66,243) 14365,  7995,  1052,   287,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ganda/Latin/Uganda
    { LOCALE_TAGS( 92, 33, 77) 14406,  7995,  1052,   288,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Geez/Ethiopic/Ethiopia
    { LOCALE_TAGS( 92, 33, 74) 14447,  7995,  1052,   289,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Geez/Ethiopic/Eritrea
    { LOCALE_TAGS( 93, 35, 90) 14447,  7995,  1052,   289,    0,    6,    0,    0, 1887, 1894, 1912,  6,  6,  5,  7,  7, 18, 15, }, // Georgian/Georgian/Georgia
    { LOCALE_TAGS( 94, 66, 91) 14865,  8139,  1052,   292,    0,    6,    0,    0, 1927, 1940, 1955,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Germany
    { LOCALE_TAGS( 94, 66, 16) 14995,  8296,  1055,   295,    0,    6,    0,    0, 1927, 1940, 1955,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Austria
    { LOCALE_TAGS( 94, 66, 23) 14995,  8296,  1055,   295,    0,    6,    0,    0, 1927, 1940, 1955,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Belgium
    { LOCALE_TAGS( 94, 66,117) 14995,  8296,  1055,   295,    0,    6,    0,    0, 1927, 1940, 1955,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Italy
    { LOCALE_TAGS( 94, 66,136) 14995,  8296,  1055,   295,    0,    6,    0,    0, 1927, 1940, 1955,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Liechtenstein
    { LOCALE_TAGS( 94, 66,138) 14995,  8296,  1055,   295,    0,    6,    0,    0, 1927, 1940, 1955,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Luxembourg
    { LOCALE_TAGS( 94, 66,226) 14995,  8296,  1055,   295,    0,    6,    0,    0, 1927, 1940, 1955,  6,  6,  5,  7, 13, 15, 15, }, // German/Latin/Switzerland
    { LOCALE_TAGS( 96, 39, 94) 14998,  8298,  1055,   295,    0,    6,    0,   27, 1970, 1978, 1996,  6,  6,  5,  9,  8, 18, 15, }, // Greek/Greek/Greece
    { LOCALE_TAGS( 96, 39, 63) 15416,  8448,  1058,   298,    0,    6,    0,   27, 1970, 1978, 1996,  6,  6,  5,  9,  8, 18, 15, }, // Greek/Greek/Cyprus
    { LOCALE_TAGS( 97, 66,183) 15416,  8448,  1058,   298,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Guarani/Latin/Paraguay
    { LOCALE_TAGS( 98, 40,110) 15457,  8452,  1058,   299,    0,    6,    0,    0, 2011, 2017, 2028,  6,  6,  5,  7,  6, 11, 11, }, // Gujarati/Gujarati/India
    { LOCALE_TAGS( 99, 66,124) 15875,  8608,  1059,   302,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Gusii/Latin/Kenya
    { LOCALE_TAGS(101, 66,169) 15916,  8608,  1059,   303,    0,    6,    0,    0, 2039, 2048, 2070,  6,  6,  5,  7,  9, 22, 15, }, // Hausa/Latin/Nigeria
    { LOCALE_TAGS(101,  4,169) 15957,  8752,  1059,   306,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Hausa/Arabic/Nigeria
    { LOCALE_TAGS(101,  4,222) 15960,  8752,  1059,   307,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Hausa/Arabic/Sudan
    { LOCALE_TAGS(101, 66, 92) 15963,  8752,  1059,   308,    0,    6,    0,    0, 2039, 2048, 2070,  6,  6,  5,  7,  9, 22, 15, }, // Hausa/Latin/Ghana
    { LOCALE_TAGS(101, 66,170) 15963,  8752,  1059,   308,    0,    6,    0,    0, 2039, 2048, 2070,  6,  6,  5,  7,  9, 22, 15, }, // Hausa/Latin/Niger
    { LOCALE_TAGS(102, 66,248) 15963,  8752,  1059,   308,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Hawaiian/Latin/United States
    { LOCALE_TAGS(103, 47,116) 16004,  8752,  1061,   310,   67,   74,  105,    0, 2085, 2092, 2106,  7,  7,  6,  7,  7, 14, 13, }, // Hebrew/Hebrew/Israel
    { LOCALE_TAGS(105, 29,110) 16422,  8906,  1061,   313,    0,    6,    0,    0, 2119,  531,  542,  6,  6,  5,  7,  6, 11, 13, }, // Hindi/Devanagari/India
    { LOCALE_TAGS(105, 66,110) 16840,  9053,  1062,   317,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // Hindi/Latin/India
    { LOCALE_TAGS(107, 66,108) 16887,  9211,  1072,   321,    0,    6,    0,    0, 2125, 2131, 2141,  6,  6,  5,  7,  6, 10, 12, }, // Hungarian/Latin/Hungary
    { LOCALE_TAGS(108, 66,109) 17075,  9368,  1076,   324,    0,    6,    0,    0,    0, 2153, 2168,  6,  6,  5,  7,  2, 15, 14, }, // Icelandic/Latin/Iceland
    { LOCALE_TAGS(109, 66,258) 17212,  9515,  1076,   327,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ido/Latin/world
    { LOCALE_TAGS(110, 66,169) 17253,  9515,  1076,   328,    0,    6,    0,    0, 2182, 2188, 2201,  6,  6,  5,  7,  6, 13, 10, }, // Igbo/Latin/Nigeria
    { LOCALE_TAGS(111, 66, 83) 17294,  9659,  1076,   331,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Inari Sami/Latin/Finland
    { LOCALE_TAGS(112, 66,111) 17335,  9659,  1076,   332,   45,   51,    0,    0, 2211, 2219, 2235,  6,  6,  5,  7,  8, 16, 20, }, // Indonesian/Latin/Indonesia
    { LOCALE_TAGS(114, 66,258) 17412,  9816,  1090,   335,    0,    6,    0,    0, 2255, 2265, 2282,  6,  6,  5,  7, 10, 17, 17, }, // Interlingua/Latin/world
    { LOCALE_TAGS(115, 66, 75) 17488,  9960,  1090,   338,    0,    6,  111,    0, 2299, 2311, 2332,  6,  6,  5,  7, 12, 21, 20, }, // Interlingue/Latin/Estonia
    { LOCALE_TAGS(116, 18, 41) 17606,  9991,  1090,   341,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Inuktitut/Canadian Aboriginal/Canada
    { LOCALE_TAGS(116, 66, 41) 17647,  9991,  1090,   342,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Inuktitut/Latin/Canada
    { LOCALE_TAGS(118, 66,114) 17647,  9991,  1090,   342,    0,    6,  116,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Irish/Latin/Ireland
    { LOCALE_TAGS(118, 66,246) 17817, 10148,  1095,   345,    0,    6,  116,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Irish/Latin/United Kingdom
    { LOCALE_TAGS(119, 66,117) 17817, 10148,  1095,   345,    0,    6,    0,    0, 2352, 2358, 2374,  6,  6,  5,  7,  6, 16, 14, }, // Italian/Latin/Italy
    { LOCALE_TAGS(119, 66,203) 17939, 10296,  1098,   348,    0,    6,    0,    0, 2352, 2358, 2374,  6,  6,  5,  7,  6, 16, 14, }, // Italian/Latin/San Marino
    { LOCALE_TAGS(119, 66,226) 17939, 10296,  1098,   348,    0,    6,    0,    0, 2352, 2358, 2374,  6,  6,  5,  7,  6, 16, 14, }, // Italian/Latin/Switzerland
    { LOCALE_TAGS(119, 66,253) 17939, 10296,  1098,   348,    0,    6,    0,    0, 2352, 2358, 2374,  6,  6,  5,  7,  6, 16, 14, }, // Italian/Latin/Vatican City
    { LOCALE_TAGS(120, 53,120) 17939, 10296,  1098,   348,    0,    6,    0,   12,  759,  763, 2388,  6,  6,  5,  6,  4,  5,  5, }, // Japanese/Japanese/Japan
    { LOCALE_TAGS(121, 66,111) 18357, 10453,  1099,   351,    0,    6,    0,    0, 2393, 2401, 2417,  6,  6,  5,  7,  8, 16, 15, }, // Javanese/Latin/Indonesia
    { LOCALE_TAGS(122, 66,169) 18446, 10597,  1099,   354,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Jju/Latin/Nigeria
    { LOCALE_TAGS(123, 66,206) 18487, 10597,  1099,   355,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Jola-Fonyi/Latin/Senegal
    { LOCALE_TAGS(124, 66, 43) 18528, 10597,  1099,   356,    0,    6,    0,    0, 2432, 2441, 2457,  6,  6,  5,  7,  9, 16, 18, }, // Kabuverdianu/Latin/Cape Verde
    { LOCALE_TAGS(125, 66,  4) 18600, 10624,  1099,   359,    0,    6,  121,    0, 2475, 2483, 2498,  6,  6,  6,  7,  8, 15, 16, }, // Kabyle/Latin/Algeria
    { LOCALE_TAGS(126, 66, 40) 18834, 10767,  1099,   362,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kako/Latin/Cameroon
    { LOCALE_TAGS(127, 66, 95) 18875, 10767,  1099,   363,    0,   22,    0,   36,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kalaallisut/Latin/Greenland
    { LOCALE_TAGS(128, 66,124) 18917, 10767,  1099,   364,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kalenjin/Latin/Kenya
    { LOCALE_TAGS(129, 66,124) 18958, 10767,  1099,   365,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kamba/Latin/Kenya
    { LOCALE_TAGS(130, 56,110) 18999, 10767,  1099,   366,    0,    6,    0,    0, 2514, 2520, 2535,  6,  6,  5,  7,  6, 15, 11, }, // Kannada/Kannada/India
    { LOCALE_TAGS(132,  4,110) 19417, 10923,  1100,   369,    0,    6,    0,    0, 2546, 2553, 2567,  6,  6,  5,  7,  7, 14, 15, }, // Kashmiri/Arabic/India
    { LOCALE_TAGS(132, 29,110) 19832, 11078,  1100,   372,    0,    6,  127,    0, 2582, 2587, 2598,  6,  6, 11,  7,  5, 11, 13, }, // Kashmiri/Devanagari/India
    { LOCALE_TAGS(133, 27,123) 19872, 11087,  1100,   373,    0,    6,    0,    0, 2611, 2620, 2640,  6,  6,  5,  7,  9, 20, 15, }, // Kazakh/Cyrillic/Kazakhstan
    { LOCALE_TAGS(134, 66, 40) 20290, 11231,  1101,   376,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kenyang/Latin/Cameroon
    { LOCALE_TAGS(135, 60, 39) 20331, 11231,  1101,   377,    0,    6,  138,    0, 2655, 2666, 2686,  6,  6, 11,  7, 11, 20, 20, }, // Khmer/Khmer/Cambodia
    { LOCALE_TAGS(136, 66, 99) 20749, 11375,  1101,   380,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kiche/Latin/Guatemala
    { LOCALE_TAGS(137, 66,124) 20790, 11375,  1101,   381,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kikuyu/Latin/Kenya
    { LOCALE_TAGS(138, 66,194) 20831, 11375,  1101,   382,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kinyarwanda/Latin/Rwanda
    { LOCALE_TAGS(141, 29,110) 20872, 11375,  1101,   383,    0,    6,    0,    0, 2706, 2712, 2727,  6,  6,  5,  7,  6, 15, 13, }, // Konkani/Devanagari/India
    { LOCALE_TAGS(142, 63,218) 21289, 11519,  1102,   386,    0,    6,    0,   43, 2740, 2745, 2751,  6,  6,  5,  6,  5,  6,  9, }, // Korean/Korean/South Korea
    { LOCALE_TAGS(142, 63, 50) 21707, 11676,  1102,   389,    0,    6,    0,   43, 2740, 2745, 2751,  6,  6,  5,  6,  5,  6,  9, }, // Korean/Korean/China
    { LOCALE_TAGS(142, 63,174) 21707, 11676,  1102,   389,    0,    6,    0,   43, 2740, 2745, 2751,  6,  6,  5,  6,  5,  6,  9, }, // Korean/Korean/North Korea
    { LOCALE_TAGS(144, 66,145) 21707, 11677,  1102,   389,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Koyraboro Senni/Latin/Mali
    { LOCALE_TAGS(145, 66,145) 21748, 11677,  1102,   390,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Koyra Chiini/Latin/Mali
    { LOCALE_TAGS(146, 66,134) 21789, 11677,  1102,   391,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kpelle/Latin/Liberia
    { LOCALE_TAGS(146, 66,102) 21830, 11677,  1102,   392,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kpelle/Latin/Guinea
    { LOCALE_TAGS(148, 66,239) 21830, 11677,  1102,   392,    0,    6,    0,    0, 2760, 2772, 2796,  6,  6,  5,  7, 12, 24, 22, }, // Kurdish/Latin/Turkey
    { LOCALE_TAGS(149, 66, 40) 21999, 11821,  1114,   395,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Kwasio/Latin/Cameroon
    { LOCALE_TAGS(150, 27,128) 22040, 11821,  1114,   396,    0,    6,    0,    0, 2818,   83,   90,  6,  6,  5,  7, 11,  7,  7, }, // Kyrgyz/Cyrillic/Kyrgyzstan
    { LOCALE_TAGS(151, 66,248) 22458, 11965,  1114,   399,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Lakota/Latin/United States
    { LOCALE_TAGS(152, 66,230) 22499, 11965,  1114,   400,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Langi/Latin/Tanzania
    { LOCALE_TAGS(153, 65,129) 22540, 11965,  1114,   401,    0,    6,    0,    0, 2829, 2836, 2851,  6,  6,  5,  7,  7, 15, 14, }, // Lao/Lao/Laos
    { LOCALE_TAGS(154, 66,253) 22958, 12119,  1114,   404,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Latin/Latin/Vatican City
    { LOCALE_TAGS(155, 66,131) 22999, 12119,  1114,   405,    0,    6,    0,   36, 2865, 2880, 2899,  6,  6,  5,  7, 15, 19, 17, }, // Latvian/Latin/Latvia
    { LOCALE_TAGS(158, 66, 57) 23367, 12266,  1117,   408,    0,    6,    0,    0, 2916,   83,   90,  6,  6,  5,  7, 12,  7,  7, }, // Lingala/Latin/Congo - Kinshasa
    { LOCALE_TAGS(158, 66,  7) 23408, 12271,  1117,   409,    0,    6,    0,    0, 2916,   83,   90,  6,  6,  5,  7, 12,  7,  7, }, // Lingala/Latin/Angola
    { LOCALE_TAGS(158, 66, 46) 23408, 12271,  1117,   409,    0,    6,    0,    0, 2916,   83,   90,  6,  6,  5,  7, 12,  7,  7, }, // Lingala/Latin/Central African Republic
    { LOCALE_TAGS(158, 66, 56) 23408, 12271,  1117,   409,    0,    6,    0,    0, 2916,   83,   90,  6,  6,  5,  7, 12,  7,  7, }, // Lingala/Latin/Congo - Brazzaville
    { LOCALE_TAGS(160, 66,137) 23408, 12271,  1117,   409,    0,   22,    0,    0, 2928, 2938, 2955,  6,  6,  5,  7, 10, 17, 18, }, // Lithuanian/Latin/Lithuania
    { LOCALE_TAGS(161, 66,258) 23796, 12428,  1117,   412,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Lojban/Latin/world
    { LOCALE_TAGS(162, 66, 91) 23837, 12430,  1117,   413,    0,    6,    0,    0, 2973, 2988, 3001,  6,  6,  5,  7, 15, 13, 15, }, // Lower Sorbian/Latin/Germany
    { LOCALE_TAGS(163, 66, 91) 23956, 12574,  1120,   416,   45,   51,   56,    0, 3016, 3023, 3038,  6,  6,  5,  7,  7, 15, 13, }, // Low German/Latin/Germany
    { LOCALE_TAGS(163, 66,165) 24011, 12602,  1120,   417,   45,   51,   56,    0, 3016, 3023, 3038,  6,  6,  5,  7,  7, 15, 13, }, // Low German/Latin/Netherlands
    { LOCALE_TAGS(164, 66, 57) 24011, 12602,  1120,   417,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Luba-Katanga/Latin/Congo - Kinshasa
    { LOCALE_TAGS(165, 66,225) 24052, 12602,  1120,   418,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Lule Sami/Latin/Sweden
    { LOCALE_TAGS(165, 66,175) 24093, 12602,  1120,   419,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Lule Sami/Latin/Norway
    { LOCALE_TAGS(166, 66,124) 24093, 12602,  1120,   419,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Luo/Latin/Kenya
    { LOCALE_TAGS(167, 66,138) 24134, 12602,  1120,   420,    0,    6,    0,    0, 3051, 3058, 3071,  6,  6,  5,  7,  7, 13, 13, }, // Luxembourgish/Latin/Luxembourg
    { LOCALE_TAGS(168, 66,124) 24243, 12748,  1120,   423,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Luyia/Latin/Kenya
    { LOCALE_TAGS(169, 27,140) 24284, 12748,  1120,   424,    0,    6,    0,    0, 3084,   83,   90,  6,  6,  5,  7, 11,  7,  7, }, // Macedonian/Cyrillic/Macedonia
    { LOCALE_TAGS(170, 66,230) 24702, 12895,  1123,   427,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Machame/Latin/Tanzania
    { LOCALE_TAGS(171, 29,110) 24743, 12895,  1123,   428,    0,    6,  149,    0, 2119,  531,  542,  6,  6,  8,  7,  6, 11, 13, }, // Maithili/Devanagari/India
    { LOCALE_TAGS(172, 66,160) 25122, 13007,  1123,   429,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Makhuwa-Meetto/Latin/Mozambique
    { LOCALE_TAGS(173, 66,230) 25163, 13007,  1123,   430,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Makonde/Latin/Tanzania
    { LOCALE_TAGS(174, 66,141) 25204, 13007,  1123,   431,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Malagasy/Latin/Madagascar
    { LOCALE_TAGS(175, 74,110) 25245, 13007,  1123,   432,    0,    6,  157,    0, 3095, 3102, 3122,  6,  6,  9,  7,  7, 20, 16, }, // Malayalam/Malayalam/India
    { LOCALE_TAGS(176, 66,143) 25663, 13163,  1124,   435,    0,    6,    0,    0, 2211, 3138, 3153,  6,  6,  5,  7,  8, 15, 14, }, // Malay/Latin/Malaysia
    { LOCALE_TAGS(176,  4, 35) 25714, 13317,  1127,   438,    0,    6,    0,    0, 3167,   83,   90,  6,  6,  5,  7,  7,  7,  7, }, // Malay/Arabic/Brunei
    { LOCALE_TAGS(176,  4,143) 25721, 13324,  1127,   439,    0,    6,    0,    0, 3167,   83,   90,  6,  6,  5,  7,  7,  7,  7, }, // Malay/Arabic/Malaysia
    { LOCALE_TAGS(176, 66, 35) 25728, 13331,  1127,   440,    0,    6,    0,    0, 2211, 3138, 3153,  6,  6,  5,  7,  8, 15, 14, }, // Malay/Latin/Brunei
    { LOCALE_TAGS(176, 66,111) 25728, 13331,  1127,   440,   45,   51,    0,    0, 2211, 3138, 3153,  6,  6,  5,  7,  8, 15, 14, }, // Malay/Latin/Indonesia
    { LOCALE_TAGS(176, 66,210) 25728, 13331,  1127,   440,    0,    6,    0,    0, 2211, 3138, 3153,  6,  6,  5,  7,  8, 15, 14, }, // Malay/Latin/Singapore
    { LOCALE_TAGS(177, 66,146) 25728, 13331,  1127,   440,    0,    6,    0,    0, 3174, 3184,   90,  6,  6,  5,  7, 10, 15,  7, }, // Maltese/Latin/Malta
    { LOCALE_TAGS(179,  9,110) 25799, 13333,  1131,   441,    0,    6,  166,    0, 3199, 3206, 3229,  6,  6, 11,  7,  7, 23, 19, }, // Manipuri/Bangla/India
    { LOCALE_TAGS(179, 78,110) 25840, 13342,  1131,   442,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Manipuri/Meitei Mayek/India
    { LOCALE_TAGS(180, 66,115) 25840, 13342,  1131,   443,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Manx/Latin/Isle of Man
    { LOCALE_TAGS(181, 66,167) 25881, 13342,  1135,   444,    0,    6,    0,    0, 3248, 3253, 3268,  6,  6,  5,  7,  5, 15, 12, }, // Maori/Latin/New Zealand
    { LOCALE_TAGS(182, 66, 49) 26023, 13486,  1135,   447,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mapuche/Latin/Chile
    { LOCALE_TAGS(183, 29,110) 26064, 13486,  1135,   448,    0,    6,  177,    0, 2706, 3280, 3293,  6,  6,  7,  7,  6, 13, 18, }, // Marathi/Devanagari/India
    { LOCALE_TAGS(185, 66,124) 26482, 13642,  1136,   451,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Masai/Latin/Kenya
    { LOCALE_TAGS(185, 66,230) 26523, 13642,  1136,   452,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Masai/Latin/Tanzania
    { LOCALE_TAGS(186,  4,112) 26523, 13642,  1136,   452,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mazanderani/Arabic/Iran
    { LOCALE_TAGS(188, 66,124) 26564, 13642,  1136,   453,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Meru/Latin/Kenya
    { LOCALE_TAGS(189, 66, 40) 26605, 13642,  1136,   454,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Meta/Latin/Cameroon
    { LOCALE_TAGS(190, 66, 41) 26646, 13642,  1136,   455,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mohawk/Latin/Canada
    { LOCALE_TAGS(191, 27,156) 26687, 13642,  1136,   456,    0,    6,    0,    0, 3311, 3319, 3336,  6,  6,  5,  7,  8, 17, 13, }, // Mongolian/Cyrillic/Mongolia
    { LOCALE_TAGS(191, 83, 50) 27105, 13786,  1136,   459,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mongolian/Mongolian/China
    { LOCALE_TAGS(191, 83,156) 27146, 13786,  1136,   460,    0,    6,   21,    0, 3349, 3355, 3373,  6,  6,  6,  7,  6, 18, 15, }, // Mongolian/Mongolian/Mongolia
    { LOCALE_TAGS(192, 66,150) 27149, 13795,  1136,   461,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Morisyen/Latin/Mauritius
    { LOCALE_TAGS(193, 66, 40) 27190, 13795,  1136,   462,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Mundang/Latin/Cameroon
    { LOCALE_TAGS(194, 66,248) 27231, 13795,  1136,   463,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Muscogee/Latin/United States
    { LOCALE_TAGS(195, 66,162) 27272, 13795,  1136,   464,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Nama/Latin/Namibia
    { LOCALE_TAGS(197, 66,248) 27313, 13795,  1136,   465,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Navajo/Latin/United States
    { LOCALE_TAGS(199, 29,164) 27354, 13795,  1136,   466,    0,    6,    0,    0, 2119, 3388, 3395,  6,  6,  5,  7,  6,  7,  7, }, // Nepali/Devanagari/Nepal
    { LOCALE_TAGS(199, 29,110) 27772, 13939,  1136,   469,    0,    6,    0,    0, 2119, 3388, 3395,  6,  6,  5,  7,  6,  7,  7, }, // Nepali/Devanagari/India
    { LOCALE_TAGS(201, 66, 40) 27772, 13940,  1137,   469,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ngiemboon/Latin/Cameroon
    { LOCALE_TAGS(202, 66, 40) 27813, 13940,  1137,   470,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Ngomba/Latin/Cameroon
    { LOCALE_TAGS(203, 66,169) 27854, 13940,  1137,   471,    0,    6,    0,    0, 3402, 3409, 3422,  6,  6,  5,  7,  7, 13, 14, }, // Nigerian Pidgin/Latin/Nigeria
    { LOCALE_TAGS(204, 90,102) 28255, 14084,  1137,   474,    0,    6,  184,    0, 3436, 3444, 3444,  6,  6,  6,  7,  8, 20, 16, }, // Nko/Nko/Guinea
    { LOCALE_TAGS(205,  4,112) 28353, 14095,  1137,   475,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Northern Luri/Arabic/Iran
    { LOCALE_TAGS(205,  4,113) 28394, 14096,  1137,   476,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Northern Luri/Arabic/Iraq
    { LOCALE_TAGS(206, 66,175) 28394, 14096,  1137,   476,    0,   22,   56,   36, 3464, 3471, 3485,  6,  6,  5,  7,  7, 14, 13, }, // Northern Sami/Latin/Norway
    { LOCALE_TAGS(206, 66, 83) 28436, 14101,  1141,   477,    0,    6,   91,   36, 3464, 3498, 3510,  6,  6,  6,  7,  7, 12, 13, }, // Northern Sami/Latin/Finland
    { LOCALE_TAGS(206, 66,225) 28478, 14244,  1145,   480,    0,   22,   56,   36, 3464, 3471, 3485,  6,  6,  5,  7,  7, 14, 13, }, // Northern Sami/Latin/Sweden
    { LOCALE_TAGS(207, 66,216) 28478, 14244,  1145,   480,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Northern Sotho/Latin/South Africa
    { LOCALE_TAGS(208, 66,261) 28519, 14244,  1145,   481,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // North Ndebele/Latin/Zimbabwe
    { LOCALE_TAGS(209, 66,175) 28560, 14244,  1145,   482,    0,    6,    0,    0, 3523, 3538, 3552,  6,  6,  5,  7, 15, 14, 14, }, // Norwegian Bokmal/Latin/Norway
    { LOCALE_TAGS(209, 66,224) 28679, 14401,  1156,   486,    0,    6,    0,    0, 3523, 3538, 3552,  6,  6,  5,  7, 15, 14, 14, }, // Norwegian Bokmal/Latin/Svalbard and Jan Mayen
    { LOCALE_TAGS(210, 66,175) 28679, 14401,  1156,   486,    0,    6,    0,    0, 3523, 3538, 3566,  6,  6,  5,  7, 15, 14, 14, }, // Norwegian Nynorsk/Latin/Norway
    { LOCALE_TAGS(211, 66,219) 28799, 14558,  1167,   490,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Nuer/Latin/South Sudan
    { LOCALE_TAGS(212, 66,142) 28840, 14558,  1167,   491,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Nyanja/Latin/Malawi
    { LOCALE_TAGS(213, 66,243) 28881, 14558,  1167,   492,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Nyankole/Latin/Uganda
    { LOCALE_TAGS(214, 66, 84) 28922, 14558,  1167,   493,    0,    6,   56,    0,  238, 3580, 3598,  6,  6,  5,  7,  9, 18, 16, }, // Occitan/Latin/France
    { LOCALE_TAGS(214, 66,220) 28992, 14702,  1167,   496,    0,    6,   56,    0,  238, 3614, 3635,  6,  6,  5,  7,  9, 21, 19, }, // Occitan/Latin/Spain
    { LOCALE_TAGS(215, 91,110) 29117, 14798,  1167,   499,    0,    6,    0,    0, 3654, 3660, 3674,  6,  6,  5,  7,  6, 14, 14, }, // Odia/Odia/India
    { LOCALE_TAGS(220, 66, 77) 29535, 14953,  1169,   502,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Oromo/Latin/Ethiopia
    { LOCALE_TAGS(220, 66,124) 29576, 14953,  1169,   503,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Oromo/Latin/Kenya
    { LOCALE_TAGS(221,101,248) 29576, 14953,  1169,   503,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Osage/Osage/United States
    { LOCALE_TAGS(222, 27, 90) 29617, 14953,  1169,   504,    0,    6,    0,    0, 3688,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Ossetic/Cyrillic/Georgia
    { LOCALE_TAGS(222, 27,193) 29617, 14953,  1169,   504,    0,    6,    0,    0, 3688,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Ossetic/Cyrillic/Russia
    { LOCALE_TAGS(226, 66, 62) 29661, 14959,  1169,   505,    0,    6,    0,    0, 3697,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Papiamento/Latin/Curacao
    { LOCALE_TAGS(226, 66, 13) 29706, 14960,  1169,   506,    0,    6,    0,    0, 3697,   83,   90,  6,  6,  5,  7,  9,  7,  7, }, // Papiamento/Latin/Aruba
    { LOCALE_TAGS(227,  4,  1) 29706, 14960,  1169,   506,    0,    6,    0,    0, 3706, 3717, 3730,  6,  6,  5,  7, 11, 13, 14, }, // Pashto/Arabic/Afghanistan
    { LOCALE_TAGS(227,  4,178) 30124, 15104,  1169,   509,    0,    6,    0,    0, 3706, 3717, 3730,  6,  6,  5,  7, 11, 13, 14, }, // Pashto/Arabic/Pakistan
    { LOCALE_TAGS(228,  4,112) 30126, 15129,  1169,   509,   67,   81,  190,    0, 3744, 3750, 3761,  7,  7, 10,  7,  6, 11, 15, }, // Persian/Arabic/Iran
    { LOCALE_TAGS(228,  4,  1) 30544, 15282,  1169,   512,   67,   81,  190,    0, 3744, 3750, 3761,  7,  7, 10,  7,  6, 11, 15, }, // Persian/Arabic/Afghanistan
    { LOCALE_TAGS(230, 66,187) 30544, 15282,  1169,   512,    0,    6,    0,    0, 3776, 3784, 3805,  6,  6,  5,  7,  8, 21, 15, }, // Polish/Latin/Poland
    { LOCALE_TAGS(231, 66, 32) 30747, 15433,  1172,   515,    0,    6,    0,    0, 3820, 3830, 3848,  6,  6,  5,  7, 10, 18, 20, }, // Portuguese/Latin/Brazil
    { LOCALE_TAGS(231, 66,  7) 30909, 15589,  1175,   518,    0,    6,    0,    0,  237, 3868, 3868,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Angola
    { LOCALE_TAGS(231, 66, 43) 30977, 15745,  1182,   521,    0,    6,    0,    0,  237, 3868, 3868,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Cape Verde
    { LOCALE_TAGS(231, 66, 73) 31045, 15901,  1189,   524,    0,    6,    0,    0,  237, 3868, 3868,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Equatorial Guinea
    { LOCALE_TAGS(231, 66,101) 31113, 16057,  1196,   527,    0,    6,    0,    0,  237, 3868, 3868,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Guinea-Bissau
    { LOCALE_TAGS(231, 66,138) 31181, 16213,  1203,   530,    0,    6,    0,    0,  237, 3868, 3868,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Luxembourg
    { LOCALE_TAGS(231, 66,139) 31249, 16369,  1210,   533,    0,    6,    0,    0,  237, 3868, 3868,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Macao
    { LOCALE_TAGS(231, 66,160) 31317, 16525,  1217,   536,    0,    6,    0,    0,  237, 3868, 3868,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Mozambique
    { LOCALE_TAGS(231, 66,188) 31385, 16681,  1224,   539,    0,    6,    0,    0,  237, 3868, 3868,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Portugal
    { LOCALE_TAGS(231, 66,204) 31453, 16837,  1231,   542,    0,    6,    0,    0,  237, 3868, 3868,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Sao Tome and Principe
    { LOCALE_TAGS(231, 66,226) 31521, 16993,  1238,   545,    0,    6,    0,    0,  237, 3868, 3868,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Switzerland
    { LOCALE_TAGS(231, 66,232) 31589, 17149,  1245,   548,    0,    6,    0,    0,  237, 3868, 3868,  6,  6,  5,  7, 10, 17, 17, }, // Portuguese/Latin/Timor-Leste
    { LOCALE_TAGS(232, 66,187) 31657, 17305,  1252,   551,    0,    6,    0,    0, 3885, 3894, 3910,  6,  6,  5,  7,  9, 16, 16, }, // Prussian/Latin/Poland
    { LOCALE_TAGS(233, 41,110) 31698, 17314,  1252,   552,    0,    6,    0,    0, 3926, 3933, 3946,  6,  6,  5,  7,  7, 13, 15, }, // Punjabi/Gurmukhi/India
    { LOCALE_TAGS(233,  4,178) 32116, 17467,  1253,   555,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Punjabi/Arabic/Pakistan
    { LOCALE_TAGS(234, 66,184) 32157, 17467,  1253,   556,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Quechua/Latin/Peru
    { LOCALE_TAGS(234, 66, 28) 32272, 17611,  1254,   559,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Quechua/Latin/Bolivia
    { LOCALE_TAGS(234, 66, 70) 32272, 17613,  1256,   559,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Quechua/Latin/Ecuador
    { LOCALE_TAGS(235, 66,192) 32272, 17615,  1258,   559,    0,    6,    0,    0, 3961, 3971, 3990,  6,  6,  5,  7, 10, 19, 18, }, // Romanian/Latin/Romania
    { LOCALE_TAGS(235, 66,154) 32384, 17766,  1262,   562,    0,    6,    0,    0, 3961, 3971, 3990,  6,  6,  5,  7, 10, 19, 18, }, // Romanian/Latin/Moldova
    { LOCALE_TAGS(236, 66,226) 32384, 17766,  1262,   562,    0,    6,    0,    0, 4008, 4016, 4031,  6,  6,  5,  7,  8, 15, 16, }, // Romansh/Latin/Switzerland
    { LOCALE_TAGS(237, 66,230) 32525, 17775,  1262,   564,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rombo/Latin/Tanzania
    { LOCALE_TAGS(238, 66, 38) 32566, 17775,  1262,   565,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rundi/Latin/Burundi
    { LOCALE_TAGS(239, 27,193) 32607, 17775,  1262,   566,    0,    6,    0,    0,    0, 4047, 4068,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Russia
    { LOCALE_TAGS(239, 27, 22) 33025, 17932,  1262,   569,    0,    6,    0,    0,    0, 4047, 4068,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Belarus
    { LOCALE_TAGS(239, 27,123) 33025, 17932,  1262,   569,    0,    6,    0,    0,    0, 4047, 4068,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Kazakhstan
    { LOCALE_TAGS(239, 27,128) 33025, 17932,  1262,   569,    0,    6,    0,    0,    0, 4047, 4068,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Kyrgyzstan
    { LOCALE_TAGS(239, 27,154) 33025, 17932,  1262,   569,    0,    6,    0,    0,    0, 4047, 4068,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Moldova
    { LOCALE_TAGS(239, 27,244) 33025, 17932,  1262,   569,    0,    6,    0,    0,    0, 4047, 4068,  6,  6,  5,  7,  2, 21, 16, }, // Russian/Cyrillic/Ukraine
    { LOCALE_TAGS(240, 66,230) 33025, 17932,  1262,   569,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rwa/Latin/Tanzania
    { LOCALE_TAGS(241, 66, 74) 33066, 17932,  1262,   570,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Saho/Latin/Eritrea
    { LOCALE_TAGS(242, 27,193) 33107, 17932,  1262,   571,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sakha/Cyrillic/Russia
    { LOCALE_TAGS(243, 66,124) 33182, 17961,  1262,   572,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Samburu/Latin/Kenya
    { LOCALE_TAGS(245, 66, 46) 33223, 17961,  1262,   573,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sango/Latin/Central African Republic
    { LOCALE_TAGS(246, 66,230) 33264, 17961,  1262,   574,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sangu/Latin/Tanzania
    { LOCALE_TAGS(247, 29,110) 33305, 17961,  1262,   575,    0,    6,  200,    0, 4084, 4091, 4104,  6,  6, 12,  7,  7, 13, 11, }, // Sanskrit/Devanagari/India
    { LOCALE_TAGS(248, 93,110) 33346, 17970,  1262,   576,    0,    6,  212,    0, 4115, 4122, 4135,  6,  6,  8,  7,  7, 13, 15, }, // Santali/Ol Chiki/India
    { LOCALE_TAGS(248, 29,110) 33466, 17991,  1262,   577,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Santali/Devanagari/India
    { LOCALE_TAGS(249, 66,117) 33491, 17991,  1262,   578,    0,    6,    0,    0, 2352, 4150, 2374,  6,  6,  5,  7,  6, 17, 14, }, // Sardinian/Latin/Italy
    { LOCALE_TAGS(251, 66,160) 33601, 18148,  1269,   582,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sena/Latin/Mozambique
    { LOCALE_TAGS(252, 27,207) 33642, 18148,  1269,   583,    0,    6,    0,    0,    0, 4167, 4187,  6,  6,  5,  7,  2, 20, 15, }, // Serbian/Cyrillic/Serbia
    { LOCALE_TAGS(252, 27, 29) 34060, 18304,  1273,   586,    0,    6,    0,    0,    0, 4202, 4224,  6,  6,  5,  7,  2, 22, 17, }, // Serbian/Cyrillic/Bosnia and Herzegovina
    { LOCALE_TAGS(252, 27,126) 34083, 18446,  1277,   589,    0,    6,    0,    0,    0, 4167, 4187,  6,  6,  5,  7,  2, 20, 15, }, // Serbian/Cyrillic/Kosovo
    { LOCALE_TAGS(252, 27,157) 34083, 18446,  1277,   589,    0,    6,    0,    0,    0, 4167, 4187,  6,  6,  5,  7,  2, 20, 15, }, // Serbian/Cyrillic/Montenegro
    { LOCALE_TAGS(252, 66, 29) 34083, 18446,  1277,   589,    0,    6,    0,    0,    0,  590, 4241,  6,  6,  5,  7,  2, 22, 19, }, // Serbian/Latin/Bosnia and Herzegovina
    { LOCALE_TAGS(252, 66,126) 34501, 18602,  1281,   592,    0,    6,    0,    0,    0, 4260, 4280,  6,  6,  5,  7,  2, 20, 16, }, // Serbian/Latin/Kosovo
    { LOCALE_TAGS(252, 66,157) 34919, 18758,  1285,   595,    0,    6,    0,    0,    0, 4260, 4280,  6,  6,  5,  7,  2, 20, 16, }, // Serbian/Latin/Montenegro
    { LOCALE_TAGS(252, 66,207) 35337, 18914,  1289,   598,    0,    6,    0,    0,    0, 4260, 4280,  6,  6,  5,  7,  2, 20, 16, }, // Serbian/Latin/Serbia
    { LOCALE_TAGS(253, 66,230) 35755, 19070,  1293,   601,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Shambala/Latin/Tanzania
    { LOCALE_TAGS(254, 66,261) 35796, 19070,  1293,   602,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Shona/Latin/Zimbabwe
    { LOCALE_TAGS(255,141, 50) 35837, 19070,  1293,   603,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sichuan Yi/Yi/China
    { LOCALE_TAGS(256, 66,117) 35878, 19070,  1293,   604,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sicilian/Latin/Italy
    { LOCALE_TAGS(257, 66, 77) 35919, 19070,  1293,   605,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sidamo/Latin/Ethiopia
    { LOCALE_TAGS(258, 66,187) 35960, 19070,  1293,   606,    0,    6,    0,    0, 3776, 4296, 4318,  6,  6,  5,  7,  8, 22, 16, }, // Silesian/Latin/Poland
    { LOCALE_TAGS(259,  4,178) 36171, 19193,  1296,   609,    0,    6,    0,    0, 4334, 4340, 4353,  6,  6,  5,  7,  6, 13, 15, }, // Sindhi/Arabic/Pakistan
    { LOCALE_TAGS(259, 29,110) 36589, 19337,  1296,   612,    0,    6,  149,    0, 4368, 4375, 4389,  6,  6,  8,  7,  7, 14, 16, }, // Sindhi/Devanagari/India
    { LOCALE_TAGS(260,119,221) 36630, 19346,  1296,   613,   45,   51,  220,    0, 4405, 4413, 4427,  6,  6, 10,  7,  8, 14, 17, }, // Sinhala/Sinhala/Sri Lanka
    { LOCALE_TAGS(261, 66, 83) 37048, 19491,  1296,   616,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Skolt Sami/Latin/Finland
    { LOCALE_TAGS(262, 66,212) 37089, 19491,  1296,   617,    0,    6,    0,    0, 1215,   83,   90,  6,  6,  5,  7, 15,  7,  7, }, // Slovak/Latin/Slovakia
    { LOCALE_TAGS(263, 66,213) 37259, 19648,  1297,   621,   45,   51,   21,    0, 4444, 4450, 4467,  6,  6,  6,  7,  6, 17, 14, }, // Slovenian/Latin/Slovenia
    { LOCALE_TAGS(264, 66,243) 37410, 19795,  1297,   624,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Soga/Latin/Uganda
    { LOCALE_TAGS(265, 66,215) 37451, 19795,  1297,   625,    0,    6,    0,    0, 4481, 4491, 4515,  6,  6,  5,  7, 10, 24, 23, }, // Somali/Latin/Somalia
    { LOCALE_TAGS(265, 66, 67) 37812, 19947,  1298,   628,    0,    6,    0,    0, 4481, 4491, 4515,  6,  6,  5,  7, 10, 24, 23, }, // Somali/Latin/Djibouti
    { LOCALE_TAGS(265, 66, 77) 37812, 19947,  1298,   628,    0,    6,    0,    0, 4481, 4491, 4515,  6,  6,  5,  7, 10, 24, 23, }, // Somali/Latin/Ethiopia
    { LOCALE_TAGS(265, 66,124) 37812, 19947,  1298,   628,    0,    6,    0,    0, 4481, 4491, 4515,  6,  6,  5,  7, 10, 24, 23, }, // Somali/Latin/Kenya
    { LOCALE_TAGS(266,  4,112) 37812, 19947,  1298,   628,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Kurdish/Arabic/Iran
    { LOCALE_TAGS(266,  4,113) 37853, 19947,  1298,   629,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Kurdish/Arabic/Iraq
    { LOCALE_TAGS(267, 66,225) 37853, 19947,  1298,   629,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Sami/Latin/Sweden
    { LOCALE_TAGS(267, 66,175) 37894, 19947,  1298,   630,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Sami/Latin/Norway
    { LOCALE_TAGS(268, 66,216) 37894, 19947,  1298,   630,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Sotho/Latin/South Africa
    { LOCALE_TAGS(268, 66,133) 37935, 19947,  1298,   631,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Southern Sotho/Latin/Lesotho
    { LOCALE_TAGS(269, 66,216) 37935, 19947,  1298,   631,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // South Ndebele/Latin/South Africa
    { LOCALE_TAGS(270, 66,220) 37976, 19947,  1298,   632,    0,    6,    0,    0, 2255, 4538, 4560,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Spain
    { LOCALE_TAGS(270, 66, 11) 37987, 19966,  1302,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Argentina
    { LOCALE_TAGS(270, 66, 24) 37991, 19982,  1306,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Belize
    { LOCALE_TAGS(270, 66, 28) 37995, 19996,  1308,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Bolivia
    { LOCALE_TAGS(270, 66, 32) 37999, 20011,  1311,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Brazil
    { LOCALE_TAGS(270, 66, 42) 38003, 20025,  1313,   634,    0,    6,    0,    0, 2255, 4538, 4560,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Canary Islands
    { LOCALE_TAGS(270, 66, 47) 38003, 20025,  1313,   634,    0,    6,    0,    0, 2255, 4538, 4560,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Ceuta and Melilla
    { LOCALE_TAGS(270, 66, 49) 38003, 20025,  1313,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Chile
    { LOCALE_TAGS(270, 66, 54) 38007, 20040,  1316,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Colombia
    { LOCALE_TAGS(270, 66, 59) 38011, 20055,  1319,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Costa Rica
    { LOCALE_TAGS(270, 66, 61) 38015, 20069,  1321,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Cuba
    { LOCALE_TAGS(270, 66, 69) 38019, 20083,  1323,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Dominican Republic
    { LOCALE_TAGS(270, 66, 70) 38023, 20097,  1325,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Ecuador
    { LOCALE_TAGS(270, 66, 72) 38027, 20112,  1328,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/El Salvador
    { LOCALE_TAGS(270, 66, 73) 38031, 20126,  1330,   634,    0,    6,    0,    0, 2255, 4538, 4560,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Equatorial Guinea
    { LOCALE_TAGS(270, 66, 99) 38031, 20126,  1330,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Guatemala
    { LOCALE_TAGS(270, 66,106) 38035, 20140,  1332,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Honduras
    { LOCALE_TAGS(270, 66,130) 38039, 20154,  1334,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Latin America
    { LOCALE_TAGS(270, 66,152) 38043, 20168,  1336,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Mexico
    { LOCALE_TAGS(270, 66,168) 38051, 20175,  1336,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Nicaragua
    { LOCALE_TAGS(270, 66,181) 38055, 20189,  1338,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Panama
    { LOCALE_TAGS(270, 66,183) 38059, 20203,  1340,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Paraguay
    { LOCALE_TAGS(270, 66,184) 38063, 20217,  1342,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Peru
    { LOCALE_TAGS(270, 66,185) 38067, 20232,  1345,   634,    0,    6,    0,    0, 2255, 4538, 4560,  6,  6,  5,  7, 10, 22, 23, }, // Spanish/Latin/Philippines
    { LOCALE_TAGS(270, 66,189) 38067, 20232,  1345,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Puerto Rico
    { LOCALE_TAGS(270, 66,248) 38071, 20246,  1347,   634,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/United States
    { LOCALE_TAGS(270, 66,250) 38275, 20401,  1358,   638,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Uruguay
    { LOCALE_TAGS(270, 66,254) 38279, 20416,  1361,   638,    0,    6,    0,    0, 2255, 4583, 4602,  6,  6,  5,  7, 10, 19, 20, }, // Spanish/Latin/Venezuela
    { LOCALE_TAGS(271,135,159) 38283, 20431,  1364,   638,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Standard Moroccan Tamazight/Tifinagh/Morocco
    { LOCALE_TAGS(272, 66,111) 38324, 20431,  1364,   639,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Sundanese/Latin/Indonesia
    { LOCALE_TAGS(273, 66,230) 38365, 20442,  1364,   640,    0,    6,   21,    0, 4622, 4631, 4651,  6,  6,  6,  7,  9, 20, 19, }, // Swahili/Latin/Tanzania
    { LOCALE_TAGS(273, 66, 57) 38411, 20589,  1364,   643,    0,    6,   21,    0, 4622, 4631, 4651,  6,  6,  6,  7,  9, 20, 19, }, // Swahili/Latin/Congo - Kinshasa
    { LOCALE_TAGS(273, 66,124) 38411, 20589,  1364,   643,    0,    6,   21,    0, 4622, 4631, 4651,  6,  6,  6,  7,  9, 20, 19, }, // Swahili/Latin/Kenya
    { LOCALE_TAGS(273, 66,243) 38431, 20655,  1364,   644,    0,    6,   21,    0, 4622, 4631, 4651,  6,  6,  6,  7,  9, 20, 19, }, // Swahili/Latin/Uganda
    { LOCALE_TAGS(274, 66,216) 38431, 20655,  1364,   644,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swati/Latin/South Africa
    { LOCALE_TAGS(274, 66, 76) 38472, 20655,  1364,   645,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swati/Latin/Eswatini
    { LOCALE_TAGS(275, 66,225) 38472, 20655,  1364,   645,    0,   22,    0,    0, 4670, 4675, 4689,  6,  6,  5,  7,  5, 14, 14, }, // Swedish/Latin/Sweden
    { LOCALE_TAGS(275, 66,  2) 38600, 20812,  1368,   649,    0,   22,    0,    0, 4670, 4675, 4689,  6,  6,  5,  7,  5, 14, 14, }, // Swedish/Latin/Aland Islands
    { LOCALE_TAGS(275, 66, 83) 38600, 20812,  1368,   649,    0,   22,    0,    0, 4670, 4675, 4689,  6,  6,  5,  7,  5, 14, 14, }, // Swedish/Latin/Finland
    { LOCALE_TAGS(276, 66,226) 38600, 20812,  1368,   649,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swiss German/Latin/Switzerland
    { LOCALE_TAGS(276, 66, 84) 38708, 20826,  1371,   650,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swiss German/Latin/France
    { LOCALE_TAGS(276, 66,136) 38708, 20826,  1371,   650,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Swiss German/Latin/Liechtenstein
    { LOCALE_TAGS(277,123,113) 38708, 20826,  1371,   650,    0,    6,    0,    0, 4703, 4710, 4727,  6,  6,  5,  7,  7, 17, 17, }, // Syriac/Syriac/Iraq
    { LOCALE_TAGS(277,123,227) 39125, 20983,  1371,   653,    0,    6,    0,    0, 4703, 4710, 4727,  6,  6,  5,  7,  7, 17, 17, }, // Syriac/Syriac/Syria
    { LOCALE_TAGS(278,135,159) 39125, 20983,  1371,   653,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tachelhit/Tifinagh/Morocco
    { LOCALE_TAGS(278, 66,159) 39166, 20983,  1371,   654,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tachelhit/Latin/Morocco
    { LOCALE_TAGS(280,127,255) 39166, 20983,  1371,   654,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tai Dam/Tai Viet/Vietnam
    { LOCALE_TAGS(281, 66,124) 39207, 20983,  1371,   655,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Taita/Latin/Kenya
    { LOCALE_TAGS(282, 27,229) 39248, 20983,  1371,   656,    0,    6,    0,    0, 4744, 4752, 4771,  6,  6,  5,  7,  8, 19, 16, }, // Tajik/Cyrillic/Tajikistan
    { LOCALE_TAGS(283,129,110) 39290, 20992,  1371,   657,    0,    6,    0,    0, 4787, 4795, 4811,  6,  6,  5,  7,  8, 16, 15, }, // Tamil/Tamil/India
    { LOCALE_TAGS(283,129,143) 39708, 21148,  1372,   660,    0,    6,    0,    0, 4787, 4795, 4811,  6,  6,  5,  7,  8, 16, 15, }, // Tamil/Tamil/Malaysia
    { LOCALE_TAGS(283,129,210) 39708, 21151,  1375,   660,    0,    6,    0,    0, 4787, 4795, 4811,  6,  6,  5,  7,  8, 16, 15, }, // Tamil/Tamil/Singapore
    { LOCALE_TAGS(283,129,221) 39708, 21154,  1378,   660,    0,    6,    0,    0, 4787, 4795, 4811,  6,  6,  5,  7,  8, 16, 15, }, // Tamil/Tamil/Sri Lanka
    { LOCALE_TAGS(284, 66,228) 39708, 21154,  1378,   660,    0,    6,  230,    0, 4826,   83,   90,  6,  6,  5,  7,  8,  7,  7, }, // Taroko/Latin/Taiwan
    { LOCALE_TAGS(285, 66,170) 39760, 21163,  1387,   661,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tasawaq/Latin/Niger
    { LOCALE_TAGS(286, 27,193) 39801, 21163,  1387,   662,    0,    6,    0,    0, 4834, 4843, 4859,  6,  6,  5,  7,  9, 16, 15, }, // Tatar/Cyrillic/Russia
    { LOCALE_TAGS(287,131,110) 39842, 21176,  1387,   663,    0,    6,    0,    0, 4874, 4881, 4898,  6,  6,  5,  7,  7, 17, 21, }, // Telugu/Telugu/India
    { LOCALE_TAGS(288, 66,243) 40260, 21332,  1388,   666,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Teso/Latin/Uganda
    { LOCALE_TAGS(288, 66,124) 40301, 21332,  1388,   667,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Teso/Latin/Kenya
    { LOCALE_TAGS(289,133,231) 40301, 21332,  1388,   667,    0,    6,    0,    0, 4919, 4925, 4938,  6,  6,  5,  7,  6, 13, 12, }, // Thai/Thai/Thailand
    { LOCALE_TAGS(290,134, 50) 40719, 21489,  1388,   670,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tibetan/Tibetan/China
    { LOCALE_TAGS(290,134,110) 40760, 21489,  1388,   671,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tibetan/Tibetan/India
    { LOCALE_TAGS(291, 33, 74) 40760, 21489,  1388,   671,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tigre/Ethiopic/Eritrea
    { LOCALE_TAGS(292, 33, 77) 40801, 21489,  1388,   672,    0,    6,    0,    0, 4950, 4955, 4964,  6,  6,  5,  7,  5,  9, 10, }, // Tigrinya/Ethiopic/Ethiopia
    { LOCALE_TAGS(292, 33, 74) 41219, 21529,  1388,   675,    0,    6,    0,    0, 4950, 4955, 4964,  6,  6,  5,  7,  5,  9, 10, }, // Tigrinya/Ethiopic/Eritrea
    { LOCALE_TAGS(294, 66,182) 41219, 21529,  1388,   675,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tok Pisin/Latin/Papua New Guinea
    { LOCALE_TAGS(295, 66,235) 41260, 21529,  1388,   676,    0,    6,    0,    0, 4974, 4982, 4997,  6,  6,  5,  7,  8, 15, 14, }, // Tongan/Latin/Tonga
    { LOCALE_TAGS(296, 66,216) 41341, 21686,  1388,   680,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tsonga/Latin/South Africa
    { LOCALE_TAGS(297, 66,216) 41382, 21686,  1388,   681,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tswana/Latin/South Africa
    { LOCALE_TAGS(297, 66, 30) 41423, 21686,  1388,   682,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tswana/Latin/Botswana
    { LOCALE_TAGS(298, 66,239) 41423, 21686,  1388,   682,    0,    6,    0,    0, 5011, 5019, 5036,  6,  6,  5,  7,  8, 17, 12, }, // Turkish/Latin/Turkey
    { LOCALE_TAGS(298, 66, 63) 41571, 21843,  1388,   685,    0,    6,    0,    0, 5011, 5019, 5036,  6,  6,  5,  7,  8, 17, 12, }, // Turkish/Latin/Cyprus
    { LOCALE_TAGS(299, 66,240) 41571, 21843,  1388,   685,    0,    6,    0,    0, 5048, 5056, 5073,  6,  6,  5,  7,  8, 17, 16, }, // Turkmen/Latin/Turkmenistan
    { LOCALE_TAGS(301, 66,169) 41885, 21990,  1389,   688,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Tyap/Latin/Nigeria
    { LOCALE_TAGS(303, 27,244) 41926, 21990,  1389,   689,    0,    6,    0,    0,  504, 5089, 5109,  6,  6,  5,  7,  7, 20, 15, }, // Ukrainian/Cyrillic/Ukraine
    { LOCALE_TAGS(304, 66, 91) 42344, 22141,  1389,   692,    0,    6,    0,    0, 5124, 5139, 5152,  6,  6,  5,  7, 15, 13, 12, }, // Upper Sorbian/Latin/Germany
    { LOCALE_TAGS(305,  4,178) 42462, 22285,  1392,   695,    0,    6,   21,    0, 4334, 5164, 5177,  6,  6,  6,  7,  6, 13, 15, }, // Urdu/Arabic/Pakistan
    { LOCALE_TAGS(305,  4,110) 42880, 22432,  1392,   698,    0,    6,    0,    0, 4334, 5164, 5192,  6,  6,  5,  7,  6, 13, 12, }, // Urdu/Arabic/India
    { LOCALE_TAGS(306,  4, 50) 42888, 22475,  1392,   698,    0,    6,    0,    0, 5204, 5212, 5230,  6,  6,  5,  7,  8, 18, 15, }, // Uyghur/Arabic/China
    { LOCALE_TAGS(307, 66,251) 42929, 22628,  1392,   701,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Uzbek/Latin/Uzbekistan
    { LOCALE_TAGS(307,  4,  1) 43219, 22772,  1392,   704,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Uzbek/Arabic/Afghanistan
    { LOCALE_TAGS(307, 27,251) 43259, 22773,  1392,   705,    0,    6,    0,    0, 5245, 5253, 5270,  6,  6,  5,  7,  8, 17, 17, }, // Uzbek/Cyrillic/Uzbekistan
    { LOCALE_TAGS(308,139,134) 43298, 22912,  1392,   708,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Vai/Vai/Liberia
    { LOCALE_TAGS(308, 66,134) 43339, 22912,  1392,   709,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Vai/Latin/Liberia
    { LOCALE_TAGS(309, 66,216) 43339, 22912,  1392,   709,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Venda/Latin/South Africa
    { LOCALE_TAGS(310, 66,255) 43380, 22912,  1392,   710,    0,    6,    0,    0, 5287, 5293, 5305,  6,  6,  5,  7,  6, 12, 13, }, // Vietnamese/Latin/Vietnam
    { LOCALE_TAGS(311, 66,258) 43434, 23069,  1399,   714,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Volapuk/Latin/world
    { LOCALE_TAGS(312, 66,230) 43475, 23069,  1399,   715,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Vunjo/Latin/Tanzania
    { LOCALE_TAGS(313, 66, 23) 43516, 23069,  1399,   716,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Walloon/Latin/Belgium
    { LOCALE_TAGS(314, 66,226) 43557, 23069,  1399,   717,    0,    6,    0,    0, 5318,   83,   90,  6,  6,  5,  7,  6,  7,  7, }, // Walser/Latin/Switzerland
    { LOCALE_TAGS(315, 66, 15) 43637, 23073,  1402,   718,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Warlpiri/Latin/Australia
    { LOCALE_TAGS(316, 66,246) 43678, 23073,  1402,   719,    0,    6,    0,    0, 5324, 5332, 5348,  6,  6,  5,  7,  8, 16, 12, }, // Welsh/Latin/United Kingdom
    { LOCALE_TAGS(317,  4,178) 43752, 23218,  1406,   722,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/Pakistan
    { LOCALE_TAGS(317,  4,  1) 43832, 23223,  1406,   723,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/Afghanistan
    { LOCALE_TAGS(317,  4,112) 43832, 23223,  1406,   723,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/Iran
    { LOCALE_TAGS(317,  4,176) 43832, 23223,  1406,   723,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/Oman
    { LOCALE_TAGS(317,  4,245) 43832, 23223,  1406,   723,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Western Balochi/Arabic/United Arab Emirates
    { LOCALE_TAGS(318, 66,165) 43832, 23223,  1406,   723,    0,    6,    0,    0, 5360, 5367, 5383,  6,  6,  5,  7,  7, 16, 12, }, // Western Frisian/Latin/Netherlands
    { LOCALE_TAGS(319, 33, 77) 43952, 23374,  1409,   726,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Wolaytta/Ethiopic/Ethiopia
    { LOCALE_TAGS(320, 66,206) 43993, 23374,  1409,   727,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Wolof/Latin/Senegal
    { LOCALE_TAGS(321, 66,216) 44034, 23383,  1409,   728,    0,    6,    0,    0,    0,    7,   23,  6,  6,  5,  7,  7, 16, 16, }, // Xhosa/Latin/South Africa
    { LOCALE_TAGS(322, 66, 40) 44076, 23527,  1409,   731,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Yangben/Latin/Cameroon
    { LOCALE_TAGS(323, 47,244) 44117, 23527,  1409,   732,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Yiddish/Hebrew/Ukraine
    { LOCALE_TAGS(324, 66,169) 44161, 23527,  1409,   733,    0,    6,  235,    0, 5395, 5402, 5416,  6,  6,  5,  7,  7, 14, 16, }, // Yoruba/Latin/Nigeria
    { LOCALE_TAGS(324, 66, 25) 44280, 23671,  1409,   736,    0,    6,  235,    0, 5395, 5402, 5432,  6,  6,  5,  7,  7, 14, 16, }, // Yoruba/Latin/Benin
    { LOCALE_TAGS(325, 66,170) 44301, 23706,  1409,   737,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Zarma/Latin/Niger
    { LOCALE_TAGS(326, 66, 50) 44342, 23706,  1409,   738,    0,    6,    0,    0, 5448, 5458, 5478,  6,  6,  5,  7, 10, 20, 17, }, // Zhuang/Latin/China
    { LOCALE_TAGS(327, 66,216) 44383, 23707,  1409,   739,    0,    6,    0,    0, 5495, 5512, 5534,  6,  6,  5,  7, 17, 22, 21, }, // Zulu/Latin/South Africa
    { LOCALE_TAGS(328, 66, 32) 44799, 23854,  1409,   742,    0,    6,    0,    0, 5555, 5565, 5574,  6,  6,  5,  7, 10,  9, 18, }, // Kaingang/Latin/Brazil
    { LOCALE_TAGS(329, 66, 32) 45211, 24009,  1412,   745,    0,    6,    0,    0, 5592, 5603, 5622,  6,  6,  5,  7, 11, 19, 23, }, // Nheengatu/Latin/Brazil
    { LOCALE_TAGS(329, 66, 54) 45605, 24164,  1415,   748,    0,    6,    0,    0, 5592, 5603, 5622,  6,  6,  5,  7, 11, 19, 23, }, // Nheengatu/Latin/Colombia
    { LOCALE_TAGS(329, 66,254) 45606, 24166,  1415,   748,    0,    6,    0,    0, 5592, 5603, 5622,  6,  6,  5,  7, 11, 19, 23, }, // Nheengatu/Latin/Venezuela
    { LOCALE_TAGS(330, 29,110) 45607, 24168,  1415,   748,    0,    6,    0,    0, 2119, 5645,   90,  6,  6,  5,  7,  6, 22,  7, }, // Haryanvi/Devanagari/India
    { LOCALE_TAGS(331, 66, 91) 45648, 24169,  1415,   749,    0,    6,    0,    0, 5667, 5674, 5690,  6,  6,  5,  7,  7, 16, 12, }, // Northern Frisian/Latin/Germany
    { LOCALE_TAGS(332, 29,110) 45772, 24313,  1415,   752,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rajasthani/Devanagari/India
    { LOCALE_TAGS(333, 27,193) 45813, 24314,  1415,   753,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Moksha/Cyrillic/Russia
    { LOCALE_TAGS(334, 66,258) 45854, 24314,  1415,   754,    0,    6,  240,    0, 5702, 5702, 5713,  6,  6, 11,  7, 11, 11, 21, }, // Toki Pona/Latin/world
    { LOCALE_TAGS(335, 66,214) 45895, 24315,  1415,   755,    0,    6,   21,    0, 5734, 5741, 5756,  6,  6,  6,  7,  7, 15, 14, }, // Pijin/Latin/Solomon Islands
    { LOCALE_TAGS(336, 66,169) 45936, 24316,  1415,   756,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Obolo/Latin/Nigeria
    { LOCALE_TAGS(337,  4,178) 45977, 24316,  1415,   757,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Baluchi/Arabic/Pakistan
    { LOCALE_TAGS(337, 66,178) 46018, 24352,  1415,   758,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Baluchi/Latin/Pakistan
    { LOCALE_TAGS(338, 66,117) 46018, 24388,  1415,   758,    0,   22,   56,    0, 5770, 5776, 5791,  6,  6,  5,  7,  6, 15, 13, }, // Ligurian/Latin/Italy
    { LOCALE_TAGS(339,142,161) 46119, 24532,  1415,   761,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rohingya/Hanifi/Myanmar
    { LOCALE_TAGS(339,142, 20) 46160, 24532,  1415,   762,    0,    6,    0,    0,    0,   83,   90,  6,  6,  5,  7,  2,  7,  7, }, // Rohingya/Hanifi/Bangladesh
    { LOCALE_TAGS(340,  4,178) 46160, 24532,  1415,   762,    0,    6,    0,    0, 5804, 5810, 5823,  6,  6,  5,  7,  6, 13,  7, }, // Torwali/Arabic/Pakistan
    { LOCALE_TAGS(341, 66, 25) 46577, 24676,  1415,   765,    0,    6,  251,    0, 5830, 5844, 5865,  6,  6,  5,  7, 14, 21, 23, }, // Anii/Latin/Benin
    { LOCALE_TAGS(342, 29,110) 46982, 24824,  1416,   768,    0,    6,  256,    0, 5888, 5894, 5905,  6,  6, 15,  7,  6, 11, 26, }, // Kangri/Devanagari/India
    { LOCALE_TAGS(343, 66,117) 47023, 24967,  1416,   771,    0,    6,   56,    0, 2352, 5931, 5945,  6,  6,  5,  7,  6, 14, 13, }, // Venetian/Latin/Italy
    { LOCALE_TAGS(344, 66,110) 47233, 25111,  1416,   774,    0,    6,    0,    0, 5958, 5965, 5980,  6,  6,  5,  7,  7, 15, 14, }, // Kuvi/Latin/India
    { LOCALE_TAGS(344, 29,110) 47650, 25254,  1416,   777,    0,    6,    0,    0, 5994, 6001, 6015,  6,  6,  5,  7,  7, 14, 15, }, // Kuvi/Devanagari/India
    { LOCALE_TAGS(344, 91,110) 47690, 25255,  1416,   778,    0,    6,    0,    0, 6030, 6037, 6052,  6,  6,  5,  7,  7, 15, 14, }, // Kuvi/Odia/India
    { LOCALE_TAGS(344,131,110) 47730, 25256,  1416,   779,    0,    6,    0,    0, 4874, 4881, 4898,  6,  6,  5,  7,  7, 17, 21, }, // Kuvi/Telugu/India
    { LOCALE_TAGS(  0,  0,  0) 47770, 25257,  1416,   780,   88,   88,  271,   49, 6066, 6066, 6066,  0,  0,  0,  0,  0,  0,  0, } // Terminal row
}; // Locale/zone data

static inline constexpr char16_t hourFormatTable[] = {
0x2b, 0x48, 0x48, 0x3a, 0x6d, 0x6d, 0x2d, 0x48, 0x48, 0x3a, 0x6d, 0x6d,
0x2b, 0x48, 0x48, 0x6d, 0x6d, 0x2d, 0x48, 0x48, 0x6d, 0x6d, 0x2212, 0x48,
0x48, 0x3a, 0x6d, 0x6d, 0x20, 0x2d, 0x48, 0x48, 0x3a, 0x6d, 0x6d, 0x2b,
0x48, 0x3a, 0x6d, 0x6d, 0x2d, 0x48, 0x3a, 0x6d, 0x6d, 0x2b, 0x48, 0x48,
0x2e, 0x6d, 0x6d, 0x2d, 0x48, 0x48, 0x2e, 0x6d, 0x6d, 0x2b, 0x48, 0x2e,
0x6d, 0x6d, 0x2d, 0x48, 0x2e, 0x6d, 0x6d, 0x200e, 0x2b, 0x48, 0x48, 0x3a,
0x6d, 0x6d, 0x2d, 0x48, 0x48, 0x3a, 0x6d, 0x6d, 0x200e, 0x200e, 0x2212, 0x48,
0x48, 0x3a, 0x6d, 0x6d
};

static inline constexpr char16_t gmtFormatTable[] = {
0x47, 0x4d, 0x54, 0x25, 0x30, 0x1302, 0x20, 0x12a4, 0x121d, 0x20, 0x1272, 0x25,
0x30, 0x63a, 0x631, 0x64a, 0x646, 0x62a, 0x634, 0x25, 0x30, 0x47, 0x4d, 0x54,
0x20, 0x25, 0x30, 0x91c, 0x93f, 0x2e, 0x90f, 0x92e, 0x2e, 0x924, 0x93f, 0x20,
0x25, 0x30, 0x413, 0x440, 0x438, 0x43d, 0x443, 0x438, 0x447, 0x25, 0x30, 0x6af,
0x631, 0x6cc, 0x646, 0x6cc, 0x686, 0x20, 0x25, 0x30, 0x55, 0x54, 0x43, 0x25,
0x30, 0xf47, 0xf72, 0xf0b, 0xf68, 0xf7a, 0xf58, 0xf0b, 0xf4f, 0xf72, 0xf0b, 0x25,
0x30, 0xd801, 0xdc18, 0xd801, 0xdc23, 0xd801, 0xdc13, 0x20, 0x25, 0x30, 0xb7, 0xd801,
0xdc5c, 0xd801, 0xdc65, 0xd801, 0xdc51, 0x25, 0x30, 0x25, 0x30, 0x20, 0x47, 0x4d,
0x54, 0xd83a, 0xdd11, 0xd83a, 0xdd16, 0xd83a, 0xdd18, 0x25, 0x30, 0x47, 0x4d, 0x54,
0x25, 0x30, 0x200e, 0x54, 0x4d, 0x47, 0x25, 0x30, 0x4d, 0x41, 0x47, 0x25,
0x30, 0x4b, 0x4c, 0x47, 0x20, 0x25, 0x30, 0x91c, 0x940, 0x20, 0x90f, 0x92e,
0x20, 0x91f, 0x940, 0x20, 0x25, 0x30, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f,
0x1780, 0x179b, 0x20, 0x25, 0x30, 0x91c, 0x940, 0x90f, 0x92e, 0x91f, 0x940, 0x25,
0x30, 0xd1c, 0xd3f, 0xd0e, 0xd02, 0xd1f, 0xd3f, 0x20, 0x25, 0x30, 0x99c, 0x9bf,
0x20, 0x98f, 0x9ae, 0x20, 0x99f, 0x9bf, 0x20, 0x25, 0x30, 0x5b, 0x47, 0x4d,
0x54, 0x5d, 0x25, 0x30, 0x7dc, 0x7ed, 0x7d5, 0x7d6, 0x25, 0x30, 0x25, 0x30,
0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x648, 0x6cc, 0x686, 0x91c, 0x940, 0x2e, 0x90f,
0x92e, 0x2e, 0x91f, 0x940, 0x2e, 0x20, 0x25, 0x30, 0x1c61, 0x1c64, 0x1c6e, 0x1c62,
0x1c74, 0x1c64, 0x25, 0x30, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdb8, 0xdc0, 0xdda,
0x25, 0x30, 0x4a, 0x51, 0x47, 0x25, 0x30, 0x57, 0x41, 0x54, 0x25, 0x30,
0x74, 0x65, 0x6e, 0x70, 0x6f, 0x20, 0x55, 0x54, 0x43, 0x25, 0x30, 0x25,
0x30, 0x20, 0x47, 0x6b, 0x917, 0x94d, 0x930, 0x940, 0x970, 0x20, 0x92e, 0x948,
0x970, 0x20, 0x91f, 0x948, 0x970, 0x25, 0x30
};

static inline constexpr char16_t regionFormatTable[] = {
0x25, 0x30, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x25, 0x30, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x25,
0x30, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x25, 0x30, 0x20, 0x410, 0x430, 0x43c, 0x4ad, 0x430, 0x25,
0x30, 0x20, 0x410, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x410, 0x430, 0x43c, 0x4ad, 0x430, 0x25, 0x30, 0x20, 0x410, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x430, 0x43c, 0x4ad, 0x430, 0x25,
0x30, 0x20, 0x28, 0x2b, 0x30, 0x29, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x31,
0x29, 0x25, 0x30, 0x2d, 0x74, 0x79, 0x64, 0x25, 0x30, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x25, 0x30,
0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4f, 0x72,
0x61, 0x3a, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x3a, 0x20, 0x25, 0x30,
0x25, 0x30, 0x20, 0x130a, 0x12dc, 0x25, 0x30, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x25, 0x30, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265,
0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x25, 0x30, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x25, 0x30, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x25, 0x30, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x25, 0x30, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x25, 0x30,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x30, 0x29,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x25, 0x30, 0x20,
0x28, 0x2b, 0x31, 0x29, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x25, 0x30, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x25, 0x30, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x25, 0x30, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x25, 0x30, 0x20, 0x9a6, 0x9bf,
0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x25, 0x30,
0x20, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x25, 0x30, 0x20, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x25, 0x30, 0x20, 0x28, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x29, 0x427, 0x430, 0x441, 0x3a, 0x20, 0x25,
0x30, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x3a, 0x20, 0x25, 0x30, 0x41b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x25, 0x30, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x25, 0x30, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x938, 0x92e, 0x92f, 0x25, 0x30, 0x20, 0x938, 0x92e, 0x25, 0x30, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x25, 0x30, 0x20,
0x938, 0x93e, 0x928, 0x928, 0x93f, 0x20, 0x938, 0x94b, 0x930, 0x93e, 0x902, 0x20,
0x938, 0x92e, 0x25, 0x30, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x25, 0x30, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x3a,
0x20, 0x25, 0x30, 0x65, 0x75, 0x72, 0x20, 0x25, 0x30, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x25, 0x30, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x25, 0x30, 0x25, 0x30,
0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x25, 0x30, 0x20, 0x2013, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x25, 0x30, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x25, 0x30, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x25, 0x30, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x25, 0x30, 0x6642, 0x9593, 0x25, 0x30, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x25, 0x30, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x25, 0x30, 0x65f6, 0x95f4, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x2c, 0x20, 0x25, 0x30, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x2c, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x25, 0x30, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e,
0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x25, 0x30, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x25, 0x30, 0x628, 0x6d5, 0x6a9,
0x627, 0x62a, 0x6cc, 0x20, 0x25, 0x30, 0x6a9, 0x627, 0x62a, 0x6cc, 0x20, 0x25,
0x30, 0x20, 0x641, 0x6d5, 0x631, 0x645, 0x6cc, 0x6a9, 0x627, 0x62a, 0x6cc, 0x20,
0x25, 0x30, 0x20, 0x647, 0x627, 0x648, 0x6cc, 0x646, 0x6d5, 0x25, 0x30, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0x25, 0x30, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0x25, 0x30, 0x20, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804,
0xdd33, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0x25, 0x30, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x25, 0x30, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x25, 0x30, 0x20, 0x13aa, 0x13af, 0x20,
0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x25, 0x30, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x25, 0x30, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x25, 0x30, 0x20, 0x28,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x29, 0x25, 0x30, 0x20, 0x28, 0x437,
0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x29, 0x25, 0x30, 0x20, 0x28, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x29, 0x25, 0x30, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x25, 0x30, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x5a, 0x69,
0x63, 0x6b, 0x20, 0x76, 0x75, 0x6e, 0x20, 0x25, 0x30, 0x53, 0x63, 0x68,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x74, 0x74, 0x7a, 0x69, 0x63, 0x6b, 0x20,
0x76, 0x75, 0x6e, 0x20, 0x25, 0x30, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x69, 0x63, 0x6b, 0x20, 0x76, 0x75, 0x6e, 0x20, 0x25, 0x30, 0x6f,
0x72, 0x61, 0x20, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x28, 0x6f,
0x72, 0x61, 0x20, 0x75, 0x73, 0x75, 0x61, 0x6c, 0x65, 0x29, 0x25, 0x30,
0x20, 0x28, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x61,
0x74, 0x65, 0x29, 0x10d, 0x61, 0x73, 0x6f, 0x76, 0xe9, 0x20, 0x70, 0xe1,
0x73, 0x6d, 0x6f, 0x20, 0x25, 0x30, 0x25, 0x30, 0x2d, 0x74, 0x69, 0x64,
0x25, 0x30, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x25, 0x30, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x25, 0x30, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x25, 0x30, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x25, 0x30, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x74, 0x69, 0x6a,
0x64, 0x20, 0x69, 0x6e, 0x20, 0x25, 0x30, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x20, 0x69, 0x6e, 0x20,
0x25, 0x30, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x20,
0x69, 0x6e, 0x20, 0x25, 0x30, 0x25, 0x30, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf0d, 0x25, 0x30, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b,
0x25, 0x30, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x25,
0x30, 0x20, 0xd801, 0xdc51, 0xd801, 0xdc72, 0xd801, 0xdc65, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x28, 0x25, 0x30, 0x29, 0x25, 0x30,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x25, 0x30,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x25, 0x30, 0x20, 0x6b, 0x65, 0x6c, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x25, 0x30,
0x20, 0x74, 0xed, 0xf0, 0x25, 0x30, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x25, 0x30, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x25, 0x30, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x20, 0x6e, 0x67, 0x20, 0x25, 0x30, 0x61, 0x69,
0x6b, 0x61, 0x76, 0x79, 0xf6, 0x68, 0x79, 0x6b, 0x65, 0x3a, 0x20, 0x25,
0x30, 0x25, 0x30, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x29, 0x25, 0x30, 0x20, 0x28, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x29, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x28, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x29,
0x25, 0x30, 0x20, 0x28, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x29, 0x25, 0x30, 0x20, 0x28, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x29, 0x25, 0x30,
0x20, 0x28, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x29, 0x25, 0x30, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x25, 0x30, 0x20,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x25, 0x30, 0x20,
0xd83a, 0xdd10, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x3a, 0x20, 0x25, 0x30, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x42, 0x65, 0x25, 0x30,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42,
0x65, 0x69, 0x61, 0x14b, 0x25, 0x30, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b,
0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x3a, 0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x3a,
0x20, 0x25, 0x30, 0x10d3, 0x10e0, 0x10dd, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x25, 0x30, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x25, 0x30, 0x20, 0x28, 0x4f,
0x72, 0x74, 0x73, 0x7a, 0x65, 0x69, 0x74, 0x29, 0x25, 0x30, 0x20, 0x28,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x29, 0x25,
0x30, 0x20, 0x28, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x29, 0x38f, 0x3c1, 0x3b1, 0x20, 0x28, 0x25, 0x30, 0x29, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x28, 0x25, 0x30, 0x29, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x28, 0x25, 0x30, 0x29, 0x25, 0x30, 0x20, 0xab8, 0xaae,
0xaaf, 0x25, 0x30, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0x25, 0x30, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0x25,
0x30, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x25, 0x30, 0x20, 0x44,
0x61, 0x69, 0x64, 0x61, 0x69, 0x74, 0x61, 0x63, 0x63, 0x65, 0x6e, 0x20,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x25, 0x30, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x25, 0x30, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x25, 0x30, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x25,
0x30, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x25, 0x30, 0x20, 0x938, 0x92e,
0x92f, 0x25, 0x30, 0x20, 0x69, 0x64, 0x151, 0x25, 0x30, 0x20, 0x7a, 0xf3,
0x6e, 0x61, 0x69, 0x64, 0x151, 0x25, 0x30, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x25, 0x30, 0x20, 0x28, 0x73, 0x74, 0x61,
0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x29, 0x25, 0x30, 0x20, 0x28,
0x73, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x29, 0x4f, 0x67,
0x65, 0x20, 0x25, 0x30, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x25, 0x30, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65,
0x20, 0x25, 0x30, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x25, 0x30, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x25, 0x30, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x25, 0x30, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20,
0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x25, 0x30, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30,
0x4f, 0x72, 0x61, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x3a, 0x20, 0x25, 0x30, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x3a, 0x20, 0x25, 0x30,
0x25, 0x30, 0x590f, 0x6642, 0x9593, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x25,
0x30, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x25, 0x30, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69,
0x20, 0x25, 0x30, 0x41, 0x6b, 0x75, 0x64, 0x3a, 0x20, 0x25, 0x30, 0x25,
0x30, 0x20, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x25, 0x30, 0x20, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x25, 0x30, 0x20, 0xcb8, 0xcae, 0xcaf,
0x25, 0x30, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0x25, 0x30, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8,
0xcae, 0xcaf, 0x25, 0x30, 0x20, 0x648, 0x64e, 0x6a9, 0x6be, 0x25, 0x30, 0x20,
0x645, 0x639, 0x6cc, 0x672, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x6a9, 0x6be, 0x25,
0x30, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x648, 0x64e,
0x6a9, 0x6be, 0x25, 0x30, 0x20, 0x935, 0x916, 0x25, 0x30, 0x20, 0x92e, 0x92f,
0x949, 0x930, 0x940, 0x20, 0x935, 0x916, 0x25, 0x30, 0x20, 0x921, 0x947, 0x20,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x935, 0x916, 0x25, 0x30, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x25, 0x30, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x25, 0x30, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x20,
0x25, 0x30, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x20, 0x200b, 0x25, 0x30, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b,
0x1793, 0x17c5, 0x200b, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x935, 0x947, 0x933,
0x25, 0x30, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x25, 0x30, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f,
0x20, 0x935, 0x947, 0x933, 0x25, 0x30, 0x20, 0xc2dc, 0xac04, 0x25, 0x30, 0x20,
0xd45c, 0xc900, 0xc2dc, 0x25, 0x30, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x25, 0x30, 0x28, 0x79, 0x29, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x79, 0x61, 0x20, 0x25, 0x30, 0x28, 0x79, 0x29, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x25, 0x30, 0x28, 0x79, 0x29, 0xea, 0x25, 0x30,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0xec0, 0xea7, 0xea5,
0xeb2, 0x20, 0x25, 0x30, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95,
0xeb0, 0xe96, 0xeb2, 0xe99, 0x20, 0x25, 0x30, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81,
0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0x20, 0x25, 0x30, 0x4c, 0x61, 0x69,
0x6b, 0x61, 0x20, 0x6a, 0x6f, 0x73, 0x6c, 0x61, 0x3a, 0x20, 0x25, 0x30,
0x25, 0x30, 0x3a, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x61, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x25, 0x30, 0x3a, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x4e, 0x67, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x79, 0x61, 0x20, 0x25, 0x30,
0x4c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x3a, 0x20, 0x25, 0x30, 0x17d, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x3a,
0x20, 0x25, 0x30, 0x56, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x3a, 0x20, 0x25, 0x30, 0x43, 0x61, 0x73,
0x6f, 0x77, 0x65, 0x20, 0x70, 0x61, 0x73, 0x6d, 0x6f, 0x20, 0x25, 0x30,
0x25, 0x30, 0x20, 0x7a, 0x79, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x25, 0x30, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x25, 0x30, 0x2d, 0x54, 0x69, 0x65, 0x74, 0x25,
0x30, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69,
0x65, 0x74, 0x25, 0x30, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x65, 0x74, 0x25, 0x30, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x25, 0x30,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x25,
0x30, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x25, 0x30, 0x25,
0x30, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0x25, 0x30, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0x25, 0x30, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x25, 0x30, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x25, 0x30, 0x648,
0x642, 0x62a, 0x648, 0x20, 0x25, 0x30, 0x126, 0x69, 0x6e, 0x20, 0x74, 0x61,
0x2019, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x126, 0x69, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x25, 0x30, 0x20, 0x99f, 0x9be,
0x987, 0x9ae, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x30, 0x29, 0x20, 0x9b7, 0x9cd,
0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987,
0x9ae, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x31, 0x29, 0x20, 0x9a6, 0x9c7, 0x9b2,
0x9be, 0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x25, 0x30, 0x20, 0x57,
0x101, 0x25, 0x30, 0x20, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x25, 0x30, 0x20, 0x57, 0x101, 0x20, 0x41, 0x77,
0x61, 0x74, 0x65, 0x61, 0x25, 0x30, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x25, 0x30, 0x20, 0x938, 0x942, 0x930, 0x94d,
0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x25,
0x30, 0x2d, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x25, 0x30, 0x2d, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x25, 0x30, 0x2d, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x25, 0x30, 0x20, 0x1834, 0x1820, 0x182d, 0x25, 0x30, 0x20, 0x1830, 0x1832,
0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820,
0x182d, 0x25, 0x30, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x180e, 0x1824,
0x20, 0x1834, 0x1820, 0x182d, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x966, 0x29, 0x25,
0x30, 0x20, 0x28, 0x2b, 0x967, 0x29, 0x25, 0x30, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x25, 0x30, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x25, 0x30, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x25, 0x30, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x25, 0x30, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x2d, 0x7e6, 0x7df,
0x7cd, 0x7e1, 0x7ca, 0x7f2, 0x7d3, 0x7ca, 0x7df, 0x7cc, 0x25, 0x30, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x25, 0x30, 0x20, 0x64, 0xe1, 0x62, 0xe1, 0x6c, 0x61,
0x161, 0xe1, 0x69, 0x67, 0x69, 0x25, 0x30, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x69, 0xe1, 0x69, 0x67, 0x69, 0x25, 0x30, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x25, 0x30, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x25, 0x30, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x2013, 0x20, 0x25, 0x30,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x2013, 0x20,
0x25, 0x30, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20,
0x2013, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x28, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x29, 0x25, 0x30,
0x20, 0x28, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x29, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x6f,
0x72, 0x61, 0x72, 0x69, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0xb38, 0xb2e, 0xb5f,
0x25, 0x30, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0x25, 0x30, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0x25, 0x30, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5,
0x433, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x25, 0x30, 0x62f, 0x20,
0x25, 0x30, 0x20, 0x67e, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x25, 0x30, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x25, 0x30,
0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x20, 0x648, 0x62e, 0x62a,
0x648, 0x642, 0x62a, 0x20, 0x25, 0x30, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x25, 0x30, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x25, 0x30, 0x63, 0x7a, 0x61, 0x73,
0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x25, 0x30, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x25, 0x30, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x3a, 0x20, 0x25, 0x30, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x3a, 0x20, 0x25, 0x30, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x4b, 0x65, 0x72,
0x64, 0x101, 0x3a, 0x20, 0x25, 0x30, 0x5a, 0x113, 0x69, 0x6d, 0x61, 0x73,
0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x3a, 0x20, 0x25, 0x30, 0x44, 0x61,
0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x3a, 0x20,
0x25, 0x30, 0x25, 0x30, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0x25, 0x30, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0x25, 0x30,
0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x25, 0x30, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x25, 0x30,
0x74, 0x65, 0x6d, 0x70, 0x3a, 0x20, 0x25, 0x30, 0x74, 0x65, 0x6d, 0x70,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x3a, 0x20, 0x25, 0x30, 0x74,
0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x64, 0x3a,
0x20, 0x25, 0x30, 0x25, 0x30, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x25, 0x30, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x25, 0x30, 0x20, 0x938, 0x92e, 0x92f, 0x3a, 0x25,
0x30, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x938, 0x92e, 0x92f, 0x903,
0x25, 0x30, 0x20, 0x905, 0x92f, 0x93e, 0x92e, 0x938, 0x92e, 0x92f, 0x903, 0x25,
0x30, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x25, 0x30, 0x20, 0x1c62, 0x1c5f, 0x1c71,
0x1c5a, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x25, 0x30, 0x20, 0x1c6b, 0x1c64,
0x1c71, 0x1c75, 0x1c6e, 0x1c72, 0x1c5f, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x3a,
0x20, 0x25, 0x30, 0x25, 0x30, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x25,
0x30, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x25, 0x30, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x25, 0x30, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x25, 0x30, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x25, 0x30, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x25, 0x30, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x25, 0x30, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x25, 0x30,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x25, 0x30, 0x20, 0x648, 0x642, 0x62a, 0x25, 0x30, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x25, 0x30, 0x20,
0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x25, 0x30, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x25, 0x30, 0x20, 0x92e, 0x905,
0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x25, 0x30, 0x20,
0x926, 0x940, 0x902, 0x952, 0x939, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d,
0x924, 0x25, 0x30, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0x25, 0x30, 0x20,
0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0x25,
0x30, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0x25, 0x30, 0x20, 0x10d, 0x61, 0x73, 0x25, 0x30,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x25, 0x30, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x25, 0x30, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x25, 0x30, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44,
0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20,
0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30,
0x68, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x25, 0x30, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x25, 0x30, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x25, 0x30, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x77, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x25, 0x30, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x25, 0x30, 0x25, 0x30, 0x74, 0x69, 0x64, 0x25, 0x30, 0x20, 0x28, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x29, 0x25, 0x30, 0x20,
0x28, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x29, 0x725,
0x715, 0x722, 0x710, 0x20, 0x25, 0x30, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x25, 0x30, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x25, 0x30, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x25, 0x30,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x25, 0x30, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x25, 0x30, 0x25,
0x30, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0x25, 0x30, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0x25,
0x30, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x25, 0x30, 0x25, 0x30,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x25, 0x30, 0x20, 0x433, 0x430,
0x434, 0x4d9, 0x442, 0x438, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x25,
0x30, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x25, 0x30, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0x25, 0x30, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0x25, 0x30, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46,
0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xe40,
0xe27, 0xe25, 0xe32, 0x25, 0x30, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0x25, 0x30, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d,
0xe21, 0xe41, 0xe2a, 0xe07, 0x25, 0x30, 0x130d, 0x12dc, 0x20, 0x25, 0x30, 0x121d,
0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x25, 0x30, 0x130d, 0x12dc, 0x20, 0x12ad,
0x1228, 0x121d, 0x1272, 0x20, 0x25, 0x30, 0x54, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x25, 0x30, 0x25, 0x30, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x25, 0x30, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x25, 0x30, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x25, 0x30, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x25, 0x30, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x25, 0x30, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x25, 0x30, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x25, 0x30, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x25, 0x30, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x439, 0x447, 0x430, 0x441,
0x3a, 0x20, 0x25, 0x30, 0x2c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x439,
0x10d, 0x61, 0x73, 0x6f, 0x77, 0x65, 0x20, 0x70, 0x61, 0x73, 0x6d, 0x6f,
0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x7a, 0x79, 0x6d, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x25, 0x30, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x25, 0x30, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x25, 0x30, 0x20, 0x688, 0x6d2, 0x20, 0x644,
0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x25, 0x30, 0x20, 0x62f,
0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x25, 0x30, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x25, 0x30, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x25, 0x30,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x25, 0x30, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x25, 0x30, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x25, 0x30, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x47, 0x69, 0x1edd, 0x20, 0x25,
0x30, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x25,
0x30, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20,
0x25, 0x30, 0x25, 0x30, 0x20, 0x7a, 0x69, 0x74, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x25, 0x30, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x25, 0x30, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x25, 0x30, 0x25, 0x30, 0x2d, 0x74,
0x69, 0x69, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x69, 0x64, 0x20, 0x25, 0x30, 0x5a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x20, 0x25, 0x30, 0xcc, 0x67, 0x62, 0xe0, 0x20,
0x25, 0x30, 0x25, 0x30, 0x20, 0xcc, 0x6c, 0xe0, 0x6e, 0xe0, 0x20, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x25, 0x30, 0x20, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x25, 0x30, 0x20, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254,
0x25, 0x30, 0x20, 0x53, 0x69, 0x7a, 0x67, 0x65, 0x6e, 0x68, 0x25, 0x30,
0x20, 0x42, 0x79, 0x61, 0x75, 0x68, 0x63, 0x75, 0x6e, 0x6a, 0x20, 0x53,
0x69, 0x7a, 0x67, 0x65, 0x6e, 0x68, 0x25, 0x30, 0x20, 0x59, 0x61, 0x6c,
0x69, 0x6e, 0x67, 0x20, 0x53, 0x69, 0x7a, 0x67, 0x65, 0x6e, 0x68, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x25, 0x30, 0x25, 0x30, 0x20, 0x69, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69,
0x6c, 0x65, 0x25, 0x30, 0x20, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0xd3,
0x72, 0x61, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x25, 0x30, 0xd3, 0x72, 0x61,
0x20, 0x70, 0x1ebd, 0x20, 0x25, 0x30, 0x50, 0x72, 0x1ef9, 0x67, 0x20, 0x6b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x25, 0x30,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x25, 0x30, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x3a, 0x20, 0x25, 0x30, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x28, 0x2b, 0x30, 0x29,
0x20, 0x938, 0x94d, 0x91f, 0x948, 0x923, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20,
0x938, 0x92e, 0x92f, 0x25, 0x30, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x25, 0x30,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x25, 0x30, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x74, 0x65, 0x6e, 0x70, 0x6f, 0x20, 0x70, 0x69, 0x20, 0x25,
0x30, 0x74, 0x65, 0x6e, 0x70, 0x6f, 0x20, 0x73, 0x65, 0x6c, 0x69, 0x20,
0x73, 0x75, 0x6e, 0x6f, 0x20, 0x70, 0x69, 0x20, 0x25, 0x30, 0x25, 0x30,
0x20, 0x54, 0x61, 0x65, 0x6d, 0x25, 0x30, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x64, 0x20, 0x54, 0x61, 0x65, 0x6d, 0x25, 0x30, 0x20, 0x44,
0x65, 0x6c, 0x61, 0x65, 0x74, 0x20, 0x54, 0x61, 0x65, 0x6d, 0x6f, 0x61,
0x3a, 0x20, 0x25, 0x30, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x3a, 0x20, 0x25, 0x30, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x3a, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x648,
0x64e, 0x62e, 0x25, 0x30, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x25, 0x30, 0x20, 0x62f, 0x6be, 0x627, 0x62a, 0x25, 0x30,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x25, 0x30, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x25, 0x30, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x25, 0x30, 0x20, 0x91f,
0x948, 0x92e, 0x25, 0x30, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x25, 0x30, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20,
0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x25, 0x30, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x25, 0x30, 0x25, 0x30, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x25, 0x30, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x25, 0x30, 0x20, 0x1e0d, 0x61, 0x79, 0x6c, 0x69,
0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x25, 0x30, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0x25, 0x30, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92c,
0x947, 0x932, 0x93e, 0x25, 0x30, 0x20, 0x92e, 0x93e, 0x928, 0x93e, 0x902, 0x915,
0x949, 0x20, 0x92c, 0x947, 0x932, 0x93e, 0x25, 0x30, 0x20, 0xb2c, 0xb47, 0xb32,
0xb3e, 0x25, 0x30, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb2c, 0xb47, 0xb32, 0xb3e, 0x25, 0x30, 0x20, 0xb21, 0xb47, 0xb32, 0xb3e, 0xb07,
0xb1f, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e
};

static inline constexpr char16_t fallbackFormatTable[] = {
0x25, 0x31, 0x20, 0x28, 0x25, 0x30, 0x29, 0x25, 0x31, 0x20, 0x25, 0x30,
0x25, 0x31, 0xff08, 0x25, 0x30, 0xff09, 0x25, 0x31, 0xf0d, 0x20, 0x28, 0x25,
0x30, 0xf0b, 0x29, 0x5b, 0x25, 0x31, 0x20, 0x28, 0x25, 0x30, 0x29, 0x5d,
0x25, 0x30, 0x20, 0x28, 0x25, 0x31, 0x29, 0x25, 0x31, 0x28, 0x25, 0x30,
0x29
};

static inline constexpr char16_t exemplarCityTable[] = {
0x41, 0x73, 0x6d, 0x61, 0x72, 0x61, 0x53, 0xe3, 0x6f, 0x20, 0x54, 0x6f,
0x6d, 0xe9, 0x41, 0x73, 0x75, 0x6e, 0x63, 0x69, 0xf3, 0x6e, 0x41, 0x74,
0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x6e, 0x42, 0x61, 0x68, 0xed, 0x61, 0x20,
0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x43,
0x61, 0x6e, 0x63, 0xfa, 0x6e, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20,
0x4a, 0x75, 0xe1, 0x72, 0x65, 0x7a, 0x43, 0x75, 0x72, 0x61, 0xe7, 0x61,
0x6f, 0x4b, 0x6e, 0x6f, 0x78, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73,
0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43, 0x69, 0x74, 0x79, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x65, 0x76, 0x61, 0x79,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e,
0x63, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x63, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x69,
0x63, 0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75,
0x63, 0x6b, 0x79, 0x4c, 0x6f, 0x77, 0x65, 0x72, 0x20, 0x50, 0x72, 0x69,
0x6e, 0x63, 0x65, 0x2019, 0x73, 0x20, 0x51, 0x75, 0x61, 0x72, 0x74, 0x65,
0x72, 0x4d, 0xe9, 0x72, 0x69, 0x64, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e,
0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x75,
0x75, 0x6b, 0x49, 0x74, 0x74, 0x6f, 0x71, 0x71, 0x6f, 0x72, 0x74, 0x6f,
0x6f, 0x72, 0x6d, 0x69, 0x69, 0x74, 0x53, 0x74, 0x2e, 0x20, 0x42, 0x61,
0x72, 0x74, 0x68, 0xe9, 0x6c, 0x65, 0x6d, 0x79, 0x53, 0x74, 0x2e, 0x20,
0x4a, 0x6f, 0x68, 0x6e, 0x2019, 0x73, 0x53, 0x74, 0x2e, 0x20, 0x4b, 0x69,
0x74, 0x74, 0x73, 0x53, 0x74, 0x2e, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61,
0x53, 0x74, 0x2e, 0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x74,
0x2e, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68,
0x20, 0x43, 0x69, 0x74, 0x79, 0x4b, 0x61, 0x74, 0x68, 0x6d, 0x61, 0x6e,
0x64, 0x75, 0x4b, 0x6f, 0x6c, 0x6b, 0x61, 0x74, 0x61, 0x4d, 0x61, 0x63,
0x61, 0x6f, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x79, 0x59, 0x61,
0x6e, 0x67, 0x6f, 0x6e, 0x46, 0x61, 0x72, 0x6f, 0x65, 0x53, 0x74, 0x2e,
0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x4b, 0x79, 0x69, 0x76, 0x52,
0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x50,
0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x410, 0x431, 0x438, 0x45f, 0x430, 0x43d,
0x410, 0x43a, 0x43a, 0x440, 0x430, 0x410, 0x434, 0x434, 0x438, 0x441, 0x2d, 0x410,
0x431, 0x435, 0x431, 0x430, 0x410, 0x43b, 0x436, 0x438, 0x440, 0x410, 0x441, 0x43c,
0x435, 0x440, 0x430, 0x411, 0x430, 0x43c, 0x430, 0x43a, 0x43e, 0x411, 0x430, 0x43d,
0x433, 0x438, 0x411, 0x430, 0x43d, 0x436, 0x443, 0x43b, 0x411, 0x438, 0x441, 0x430,
0x443, 0x411, 0x43b, 0x430, 0x43d, 0x442, 0x430, 0x438, 0x440, 0x411, 0x440, 0x430,
0x437, 0x437, 0x430, 0x432, 0x438, 0x43b, 0x411, 0x443, 0x436, 0x443, 0x43c, 0x431,
0x443, 0x440, 0x430, 0x41a, 0x430, 0x438, 0x440, 0x41a, 0x430, 0x441, 0x430, 0x431,
0x43b, 0x430, 0x43d, 0x43a, 0x430, 0x421, 0x435, 0x443, 0x442, 0x430, 0x41a, 0x43e,
0x43d, 0x430, 0x43a, 0x440, 0x438, 0x414, 0x430, 0x43a, 0x430, 0x440, 0x414, 0x430,
0x440, 0x2d, 0x435, 0x441, 0x2d, 0x421, 0x430, 0x43b, 0x430, 0x43c, 0x40f, 0x438,
0x431, 0x443, 0x442, 0x438, 0x414, 0x443, 0x430, 0x43b, 0x430, 0x415, 0x43b, 0x2d,
0x410, 0x438, 0x443, 0x43d, 0x424, 0x440, 0x438, 0x442, 0x430, 0x443, 0x43d, 0x413,
0x430, 0x431, 0x43e, 0x440, 0x43e, 0x43d, 0x435, 0x4b2, 0x430, 0x440, 0x430, 0x440,
0x435, 0x418, 0x43e, 0x4b3, 0x430, 0x43d, 0x43d, 0x435, 0x441, 0x431, 0x443, 0x440,
0x433, 0x40f, 0x443, 0x431, 0x430, 0x41a, 0x430, 0x43c, 0x43f, 0x430, 0x43b, 0x430,
0x425, 0x430, 0x440, 0x442, 0x443, 0x43c, 0x41a, 0x438, 0x433, 0x430, 0x43b, 0x438,
0x41a, 0x438, 0x43d, 0x448, 0x430, 0x441, 0x430, 0x41b, 0x430, 0x433, 0x43e, 0x441,
0x41b, 0x438, 0x431, 0x440, 0x435, 0x432, 0x438, 0x43b, 0x41b, 0x43e, 0x43c, 0x435,
0x41b, 0x443, 0x430, 0x43d, 0x434, 0x430, 0x41b, 0x443, 0x431, 0x443, 0x43c, 0x431,
0x430, 0x448, 0x438, 0x41b, 0x443, 0x441, 0x430, 0x43a, 0x430, 0x41c, 0x430, 0x43b,
0x430, 0x431, 0x43e, 0x41c, 0x430, 0x43f, 0x443, 0x442, 0x443, 0x41c, 0x430, 0x441,
0x435, 0x440, 0x443, 0x41c, 0x431, 0x430, 0x431, 0x430, 0x43d, 0x435, 0x41c, 0x43e,
0x433, 0x430, 0x434, 0x438, 0x448, 0x43e, 0x41c, 0x43e, 0x43d, 0x440, 0x43e, 0x432,
0x438, 0x430, 0x41d, 0x430, 0x438, 0x440, 0x43e, 0x431, 0x438, 0x41d, 0x45f, 0x430,
0x43c, 0x435, 0x43d, 0x430, 0x41d, 0x438, 0x430, 0x43c, 0x435, 0x438, 0x41d, 0x443,
0x430, 0x43a, 0x448, 0x43e, 0x442, 0x423, 0x430, 0x433, 0x430, 0x434, 0x443, 0x433,
0x443, 0x41f, 0x43e, 0x440, 0x442, 0x43e, 0x2d, 0x41d, 0x43e, 0x432, 0x43e, 0x421,
0x430, 0x43d, 0x2d, 0x422, 0x43e, 0x43c, 0x435, 0x422, 0x440, 0x438, 0x43f, 0x43e,
0x43b, 0x438, 0x422, 0x443, 0x43d, 0x438, 0x441, 0x412, 0x438, 0x43d, 0x434, 0x445,
0x443, 0x43a, 0x410, 0x434, 0x430, 0x43a, 0x410, 0x43d, 0x43a, 0x43e, 0x440, 0x438,
0x45f, 0x410, 0x43d, 0x433, 0x438, 0x43b, 0x438, 0x430, 0x410, 0x43d, 0x442, 0x438,
0x433, 0x443, 0x430, 0x410, 0x440, 0x430, 0x433, 0x443, 0x430, 0x438, 0x43d, 0x430,
0x411, 0x443, 0x435, 0x43d, 0x43e, 0x441, 0x2d, 0x410, 0x438, 0x440, 0x435, 0x441,
0x41a, 0x430, 0x442, 0x430, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x41a, 0x43e, 0x440,
0x434, 0x43e, 0x432, 0x430, 0x416, 0x443, 0x436, 0x443, 0x438, 0x41b, 0x430, 0x2d,
0x420, 0x438, 0x43e, 0x445, 0x430, 0x41c, 0x435, 0x43d, 0x434, 0x43e, 0x441, 0x430,
0x420, 0x438, 0x43e, 0x2d, 0x413, 0x430, 0x43b, 0x435, 0x433, 0x43e, 0x441, 0x421,
0x430, 0x43b, 0x442, 0x430, 0x421, 0x430, 0x43d, 0x2d, 0x425, 0x443, 0x430, 0x43d,
0x421, 0x430, 0x43d, 0x2d, 0x41b, 0x443, 0x438, 0x441, 0x422, 0x443, 0x43a, 0x443,
0x43c, 0x430, 0x43d, 0x423, 0x448, 0x443, 0x430, 0x438, 0x430, 0x410, 0x440, 0x443,
0x431, 0x430, 0x410, 0x441, 0x443, 0x43d, 0x441, 0x43e, 0x43d, 0x41a, 0x43e, 0x440,
0x430, 0x43b, 0x2d, 0x425, 0x430, 0x440, 0x431, 0x43e, 0x440, 0x411, 0x430, 0x438,
0x430, 0x411, 0x430, 0x438, 0x430, 0x2d, 0x434, 0x435, 0x2d, 0x411, 0x430, 0x43d,
0x434, 0x435, 0x440, 0x430, 0x441, 0x411, 0x430, 0x440, 0x431, 0x430, 0x434, 0x43e,
0x441, 0x411, 0x435, 0x43b, 0x435, 0x43d, 0x411, 0x435, 0x43b, 0x438, 0x437, 0x411,
0x43b, 0x430, 0x43d, 0x43a, 0x2d, 0x421, 0x430, 0x431, 0x43b, 0x43e, 0x43d, 0x411,
0x43e, 0x430, 0x2d, 0x412, 0x438, 0x441, 0x442, 0x430, 0x411, 0x43e, 0x433, 0x43e,
0x442, 0x430, 0x411, 0x43e, 0x438, 0x441, 0x435, 0x41a, 0x435, 0x438, 0x43c, 0x431,
0x440, 0x438, 0x45f, 0x2d, 0x411, 0x435, 0x438, 0x41a, 0x430, 0x43c, 0x43f, 0x443,
0x2d, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x438, 0x41a, 0x430, 0x43d, 0x43a, 0x443,
0x43d, 0x41a, 0x430, 0x440, 0x430, 0x43a, 0x430, 0x441, 0x41a, 0x430, 0x438, 0x435,
0x43d, 0x43d, 0x430, 0x410, 0x434, 0x433, 0x44c, 0x44b, 0x43b, 0x431, 0x436, 0x44c,
0x430, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x20, 0x41a, 0x430, 0x438, 0x43c, 0x430,
0x43d, 0x427, 0x438, 0x43a, 0x430, 0x433, 0x43e, 0x427, 0x438, 0x443, 0x430, 0x443,
0x430, 0x41a, 0x43e, 0x441, 0x442, 0x430, 0x2d, 0x420, 0x438, 0x43a, 0x430, 0x41a,
0x440, 0x435, 0x441, 0x442, 0x43e, 0x43d, 0x41a, 0x443, 0x438, 0x430, 0x431, 0x430,
0x41a, 0x438, 0x443, 0x440, 0x430, 0x441, 0x430, 0x43e, 0x414, 0x435, 0x43d, 0x43c,
0x430, 0x440, 0x43a, 0x441, 0x445, 0x430, 0x432, 0x43d, 0x414, 0x43e, 0x443, 0x441,
0x43e, 0x43d, 0x414, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x2d, 0x41a, 0x440, 0x438,
0x43a, 0x414, 0x435, 0x43d, 0x432, 0x435, 0x440, 0x414, 0x435, 0x442, 0x440, 0x43e,
0x438, 0x442, 0x414, 0x43e, 0x43c, 0x438, 0x43d, 0x438, 0x43a, 0x430, 0x415, 0x434,
0x43c, 0x43e, 0x43d, 0x442, 0x43e, 0x43d, 0x415, 0x438, 0x440, 0x443, 0x43d, 0x435,
0x43f, 0x435, 0x421, 0x430, 0x43b, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x424, 0x43e,
0x440, 0x442, 0x20, 0x41d, 0x435, 0x43b, 0x441, 0x43e, 0x43d, 0x424, 0x43e, 0x440,
0x442, 0x430, 0x43b, 0x435, 0x437, 0x430, 0x413, 0x43b, 0x435, 0x438, 0x441, 0x2d,
0x411, 0x435, 0x438, 0x413, 0x443, 0x441, 0x2d, 0x411, 0x435, 0x438, 0x413, 0x440,
0x430, 0x43d, 0x434, 0x2d, 0x422, 0x435, 0x440, 0x43a, 0x413, 0x440, 0x435, 0x43d,
0x430, 0x434, 0x430, 0x413, 0x432, 0x430, 0x434, 0x435, 0x43b, 0x443, 0x43f, 0x430,
0x413, 0x432, 0x430, 0x442, 0x435, 0x43c, 0x430, 0x43b, 0x430, 0x413, 0x443, 0x430,
0x438, 0x430, 0x43a, 0x438, 0x43b, 0x413, 0x430, 0x438, 0x430, 0x43d, 0x430, 0x4b2,
0x430, 0x43b, 0x438, 0x444, 0x430, 0x43a, 0x441, 0x413, 0x430, 0x432, 0x430, 0x43d,
0x430, 0x415, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x43b, 0x43e, 0x418, 0x43d, 0x434,
0x438, 0x430, 0x43d, 0x430, 0x43f, 0x43e, 0x43b, 0x438, 0x441, 0x41d, 0x43e, 0x43a,
0x441, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x41c, 0x430,
0x440, 0x435, 0x43d, 0x433, 0x43e, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430,
0x43d, 0x430, 0x41f, 0x438, 0x442, 0x435, 0x440, 0x441, 0x431, 0x435, 0x440, 0x433,
0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x422, 0x435, 0x43b,
0x43b, 0x2d, 0x421, 0x438, 0x442, 0x438, 0x412, 0x435, 0x432, 0x435, 0x438, 0x2c,
0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d, 0x441,
0x435, 0x43d, 0x43d, 0x435, 0x441, 0x423, 0x438, 0x43d, 0x430, 0x43c, 0x430, 0x43a,
0x418, 0x43d, 0x443, 0x432, 0x438, 0x43a, 0x418, 0x43a, 0x430, 0x43b, 0x443, 0x438,
0x442, 0x418, 0x430, 0x43c, 0x430, 0x438, 0x43a, 0x430, 0x40f, 0x443, 0x43d, 0x43e,
0x41b, 0x443, 0x438, 0x441, 0x432, 0x438, 0x43b, 0x43b, 0x41c, 0x43e, 0x43d, 0x442,
0x438, 0x441, 0x435, 0x43b, 0x43b, 0x43e, 0x2c, 0x20, 0x41a, 0x435, 0x43d, 0x442,
0x443, 0x43a, 0x43a, 0x438, 0x41a, 0x440, 0x430, 0x43b, 0x435, 0x43d, 0x434, 0x435,
0x438, 0x43a, 0x41b, 0x430, 0x2d, 0x41f, 0x430, 0x441, 0x41b, 0x438, 0x43c, 0x430,
0x41b, 0x43e, 0x441, 0x2d, 0x410, 0x43d, 0x45f, 0x435, 0x43b, 0x435, 0x441, 0x41b,
0x43e, 0x443, 0x435, 0x440, 0x2d, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x2d, 0x41a,
0x443, 0x43e, 0x442, 0x435, 0x440, 0x41c, 0x430, 0x441, 0x435, 0x438, 0x43e, 0x41c,
0x430, 0x43d, 0x430, 0x433, 0x443, 0x430, 0x41c, 0x430, 0x43d, 0x430, 0x443, 0x441,
0x41c, 0x430, 0x440, 0x438, 0x433, 0x43e, 0x41c, 0x430, 0x440, 0x442, 0x438, 0x43d,
0x438, 0x43a, 0x430, 0x41c, 0x430, 0x442, 0x430, 0x43c, 0x43e, 0x440, 0x43e, 0x441,
0x41c, 0x430, 0x441, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x41c, 0x435, 0x43d, 0x43e,
0x43c, 0x438, 0x43d, 0x438, 0x41c, 0x435, 0x440, 0x438, 0x434, 0x430, 0x41c, 0x435,
0x442, 0x43b, 0x430, 0x43a, 0x430, 0x442, 0x43b, 0x430, 0x41c, 0x435, 0x445, 0x438,
0x43a, 0x43e, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x41c, 0x43e, 0x43d,
0x43a, 0x442, 0x43e, 0x43d, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x440, 0x440, 0x435,
0x438, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x432, 0x438, 0x434, 0x435, 0x43e, 0x41c,
0x43e, 0x43d, 0x442, 0x441, 0x435, 0x440, 0x440, 0x430, 0x442, 0x41d, 0x430, 0x441,
0x441, 0x430, 0x443, 0x41d, 0x438, 0x443, 0x2d, 0x418, 0x43e, 0x440, 0x43a, 0x41d,
0x43e, 0x43c, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x438, 0x430, 0x411, 0x43e, 0x438,
0x43b, 0x430, 0x2c, 0x20, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438,
0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430,
0x2c, 0x20, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x438, 0x443, 0x2d, 0x421, 0x435, 0x438,
0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x443, 0x443, 0x43a,
0x41e, 0x445, 0x438, 0x43d, 0x430, 0x433, 0x430, 0x41f, 0x430, 0x43d, 0x430, 0x43c,
0x430, 0x41f, 0x430, 0x440, 0x430, 0x43c, 0x430, 0x440, 0x438, 0x431, 0x43e, 0x424,
0x438, 0x43d, 0x438, 0x43a, 0x441, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x43e, 0x2d,
0x41f, 0x440, 0x435, 0x43d, 0x441, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x43e, 0x444,
0x2d, 0x421, 0x43f, 0x435, 0x438, 0x43d, 0x41f, 0x43e, 0x440, 0x442, 0x443, 0x2d,
0x412, 0x435, 0x43b, 0x438, 0x443, 0x41f, 0x443, 0x435, 0x440, 0x442, 0x43e, 0x2d,
0x420, 0x438, 0x43a, 0x43e, 0x41f, 0x443, 0x43d, 0x442, 0x430, 0x2d, 0x410, 0x440,
0x435, 0x43d, 0x430, 0x441, 0x420, 0x430, 0x43d, 0x43a, 0x438, 0x43d, 0x2d, 0x418,
0x43d, 0x43b, 0x435, 0x442, 0x420, 0x435, 0x441, 0x438, 0x444, 0x438, 0x420, 0x435,
0x45f, 0x430, 0x438, 0x43d, 0x430, 0x420, 0x435, 0x437, 0x43e, 0x43b, 0x438, 0x443,
0x442, 0x420, 0x438, 0x443, 0x2d, 0x411, 0x440, 0x430, 0x43d, 0x43a, 0x443, 0x421,
0x430, 0x43d, 0x442, 0x430, 0x440, 0x435, 0x43d, 0x421, 0x430, 0x43d, 0x442, 0x438,
0x430, 0x433, 0x43e, 0x421, 0x430, 0x43d, 0x442, 0x43e, 0x2d, 0x414, 0x43e, 0x43c,
0x438, 0x43d, 0x433, 0x43e, 0x421, 0x430, 0x43d, 0x2d, 0x41f, 0x430, 0x443, 0x43b,
0x443, 0x421, 0x43a, 0x43e, 0x440, 0x441, 0x431, 0x438, 0x441, 0x443, 0x43d, 0x43d,
0x421, 0x438, 0x442, 0x43a, 0x430, 0x421, 0x435, 0x43d, 0x2d, 0x411, 0x430, 0x440,
0x442, 0x435, 0x43b, 0x435, 0x43c, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x40f,
0x43e, 0x43d, 0x441, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41a, 0x438, 0x442, 0x441,
0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41b, 0x438, 0x443, 0x441, 0x438, 0x430, 0x421,
0x435, 0x43d, 0x442, 0x2d, 0x422, 0x43e, 0x43c, 0x430, 0x441, 0x421, 0x435, 0x43d,
0x442, 0x2d, 0x412, 0x438, 0x43d, 0x441, 0x435, 0x43d, 0x442, 0x421, 0x432, 0x438,
0x444, 0x442, 0x2d, 0x41a, 0x435, 0x440, 0x440, 0x435, 0x43d, 0x442, 0x422, 0x435,
0x433, 0x443, 0x441, 0x438, 0x433, 0x430, 0x43b, 0x43f, 0x430, 0x422, 0x443, 0x43b,
0x435, 0x422, 0x438, 0x445, 0x443, 0x430, 0x43d, 0x430, 0x422, 0x43e, 0x440, 0x43e,
0x43d, 0x442, 0x43e, 0x422, 0x43e, 0x440, 0x442, 0x43e, 0x43b, 0x430, 0x412, 0x430,
0x43d, 0x43a, 0x443, 0x432, 0x435, 0x440, 0x423, 0x430, 0x438, 0x442, 0x445, 0x43e,
0x440, 0x441, 0x412, 0x438, 0x43d, 0x43d, 0x438, 0x43f, 0x435, 0x433, 0x418, 0x430,
0x43a, 0x443, 0x442, 0x430, 0x442, 0x41a, 0x435, 0x438, 0x441, 0x438, 0x414, 0x435,
0x438, 0x432, 0x438, 0x441, 0x414, 0x438, 0x443, 0x43c, 0x43e, 0x43d, 0x2d, 0x434,
0x2019, 0x418, 0x443, 0x440, 0x432, 0x438, 0x43b, 0x41c, 0x430, 0x43a, 0x43a, 0x443,
0x43e, 0x440, 0x438, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x41c, 0x430, 0x43a,
0x2d, 0x41c, 0x435, 0x440, 0x434, 0x43e, 0x41f, 0x430, 0x43b, 0x43c, 0x435, 0x440,
0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x421, 0x438, 0x43e, 0x432, 0x430, 0x422,
0x440, 0x43e, 0x43b, 0x43b, 0x410, 0x43c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430,
0x440, 0x430, 0x41b, 0x43e, 0x43d, 0x433, 0x438, 0x440, 0x410, 0x434, 0x435, 0x43d,
0x410, 0x43b, 0x43c, 0x430, 0x442, 0x44b, 0x410, 0x43c, 0x43c, 0x430, 0x43d, 0x410,
0x43d, 0x430, 0x434, 0x44b, 0x440, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x410, 0x43a,
0x442, 0x43e, 0x431, 0x435, 0x410, 0x448, 0x445, 0x430, 0x431, 0x430, 0x434, 0x410,
0x442, 0x44b, 0x440, 0x430, 0x443, 0x411, 0x430, 0x433, 0x434, 0x430, 0x434, 0x411,
0x430, 0x445, 0x440, 0x435, 0x438, 0x43d, 0x411, 0x430, 0x43a, 0x443, 0x411, 0x430,
0x43d, 0x433, 0x43a, 0x43e, 0x43a, 0x411, 0x430, 0x440, 0x43d, 0x430, 0x443, 0x43b,
0x411, 0x435, 0x438, 0x440, 0x443, 0x442, 0x411, 0x438, 0x448, 0x43a, 0x435, 0x43a,
0x411, 0x440, 0x443, 0x43d, 0x435, 0x438, 0x427, 0x438, 0x442, 0x430, 0x427, 0x43e,
0x438, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x41a, 0x43e, 0x43b, 0x43e, 0x43c,
0x431, 0x43e, 0x414, 0x430, 0x43c, 0x430, 0x441, 0x43a, 0x414, 0x430, 0x43a, 0x43a,
0x430, 0x414, 0x438, 0x43b, 0x438, 0x414, 0x443, 0x431, 0x430, 0x438, 0x414, 0x443,
0x448, 0x430, 0x43d, 0x431, 0x435, 0x424, 0x430, 0x43c, 0x430, 0x433, 0x443, 0x441,
0x442, 0x430, 0x413, 0x430, 0x437, 0x430, 0x425, 0x435, 0x432, 0x440, 0x43e, 0x43d,
0x425, 0x43e, 0x448, 0x438, 0x43c, 0x438, 0x43d, 0x4b2, 0x43e, 0x43d, 0x43a, 0x43e,
0x43d, 0x433, 0x425, 0x43e, 0x432, 0x434, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x40f, 0x430, 0x43a, 0x430, 0x440, 0x442, 0x430, 0x40f, 0x430, 0x438, 0x430,
0x43f, 0x443, 0x440, 0x430, 0x418, 0x435, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x438,
0x43c, 0x41a, 0x430, 0x431, 0x443, 0x43b, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f,
0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447,
0x430, 0x442, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x41a, 0x430, 0x440, 0x430, 0x447,
0x438, 0x41a, 0x430, 0x442, 0x43c, 0x430, 0x43d, 0x434, 0x443, 0x425, 0x430, 0x43d,
0x434, 0x44b, 0x433, 0x430, 0x41a, 0x430, 0x43b, 0x43a, 0x443, 0x442, 0x442, 0x430,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x438, 0x430, 0x440, 0x441, 0x43a, 0x41a,
0x443, 0x430, 0x43b, 0x430, 0x2d, 0x41b, 0x443, 0x43c, 0x43f, 0x443, 0x440, 0x41a,
0x443, 0x447, 0x438, 0x43d, 0x433, 0x41a, 0x443, 0x432, 0x435, 0x438, 0x442, 0x41c,
0x430, 0x43a, 0x430, 0x43e, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x41c,
0x430, 0x43a, 0x430, 0x441, 0x430, 0x440, 0x41c, 0x430, 0x43d, 0x438, 0x43b, 0x430,
0x41c, 0x430, 0x441, 0x43a, 0x430, 0x442, 0x41d, 0x438, 0x43a, 0x43e, 0x441, 0x438,
0x430, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x443, 0x437, 0x43d, 0x435, 0x446, 0x43a,
0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x41e,
0x43c, 0x441, 0x43a, 0x423, 0x440, 0x430, 0x43b, 0x441, 0x43a, 0x41f, 0x43d, 0x43e,
0x43c, 0x43f, 0x435, 0x43d, 0x41f, 0x43e, 0x43d, 0x442, 0x438, 0x430, 0x43d, 0x430,
0x43a, 0x41f, 0x445, 0x435, 0x43d, 0x438, 0x430, 0x43d, 0x41a, 0x430, 0x442, 0x430,
0x440, 0x41a, 0x43e, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x438, 0x41a, 0x44b, 0x437,
0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x415, 0x440, 0x2d, 0x420, 0x438, 0x430,
0x434, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x43d, 0x434, 0x421,
0x435, 0x443, 0x43b, 0x428, 0x430, 0x43d, 0x445, 0x430, 0x438, 0x421, 0x438, 0x43d,
0x433, 0x430, 0x43f, 0x443, 0x440, 0x421, 0x440, 0x435, 0x434, 0x43d, 0x435, 0x43a,
0x43e, 0x43b, 0x44b, 0x43c, 0x441, 0x43a, 0x422, 0x430, 0x438, 0x431, 0x435, 0x438,
0x422, 0x430, 0x448, 0x43a, 0x435, 0x43d, 0x442, 0x49a, 0x430, 0x440, 0x4ad, 0x422,
0x435, 0x433, 0x435, 0x440, 0x430, 0x43d, 0x422, 0x445, 0x438, 0x43c, 0x43f, 0x445,
0x443, 0x422, 0x43e, 0x43a, 0x438, 0x43e, 0x422, 0x43e, 0x43c, 0x441, 0x43a, 0x423,
0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x423, 0x440, 0x443,
0x43c, 0x447, 0x438, 0x423, 0x441, 0x442, 0x2d, 0x41d, 0x435, 0x440, 0x430, 0x412,
0x435, 0x43d, 0x442, 0x438, 0x430, 0x43d, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x418, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x418, 0x430, 0x43d, 0x433, 0x43e, 0x43d, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x415, 0x440, 0x435, 0x432, 0x430, 0x43d,
0x410, 0x437, 0x43e, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445,
0x430, 0x49b, 0x4d9, 0x430, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41a, 0x430,
0x43d, 0x430, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430,
0x49b, 0x4d9, 0x430, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434,
0x435, 0x424, 0x430, 0x440, 0x435, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434,
0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41c, 0x430, 0x434, 0x435, 0x438, 0x440,
0x430, 0x420, 0x435, 0x438, 0x43a, 0x438, 0x430, 0x432, 0x438, 0x43a, 0x410, 0x43b,
0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433,
0x438, 0x430, 0x418, 0x525, 0x448, 0x44c, 0x43e, 0x443, 0x20, 0x415, 0x43b, 0x435,
0x43d, 0x430, 0x20, 0x43b, 0x430, 0x434, 0x433, 0x44c, 0x44b, 0x43b, 0x431, 0x436,
0x44c, 0x430, 0x445, 0x430, 0x421, 0x442, 0x435, 0x43d, 0x43b, 0x438, 0x410, 0x434,
0x435, 0x43b, 0x430, 0x438, 0x434, 0x430, 0x411, 0x440, 0x438, 0x441, 0x431, 0x435,
0x43d, 0x411, 0x440, 0x43e, 0x43a, 0x435, 0x43d, 0x2d, 0x425, 0x438, 0x43b, 0x43b,
0x414, 0x430, 0x440, 0x432, 0x438, 0x43d, 0x418, 0x443, 0x43a, 0x43b, 0x430, 0x425,
0x43e, 0x431, 0x430, 0x440, 0x442, 0x41b, 0x438, 0x43d, 0x434, 0x435, 0x43c, 0x430,
0x43d, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x41c, 0x435, 0x43b,
0x431, 0x443, 0x440, 0x43d, 0x41f, 0x435, 0x440, 0x442, 0x421, 0x438, 0x434, 0x43d,
0x435, 0x438, 0x410, 0x43c, 0x441, 0x442, 0x435, 0x440, 0x434, 0x430, 0x43c, 0x410,
0x43d, 0x434, 0x43e, 0x440, 0x440, 0x430, 0x410, 0x441, 0x442, 0x440, 0x430, 0x445,
0x430, 0x43d, 0x410, 0x444, 0x438, 0x43d, 0x411, 0x435, 0x43b, 0x433, 0x440, 0x430,
0x434, 0x411, 0x435, 0x440, 0x43b, 0x438, 0x43d, 0x411, 0x440, 0x430, 0x442, 0x438,
0x441, 0x43b, 0x430, 0x432, 0x430, 0x411, 0x440, 0x438, 0x443, 0x441, 0x441, 0x435,
0x43b, 0x411, 0x443, 0x445, 0x430, 0x440, 0x435, 0x441, 0x442, 0x411, 0x443, 0x434,
0x430, 0x43f, 0x435, 0x448, 0x442, 0x411, 0x438, 0x443, 0x437, 0x438, 0x43d, 0x433,
0x435, 0x43d, 0x2d, 0x425, 0x44b, 0x445, 0x44c, 0x442, 0x4d9, 0x438, 0x2d, 0x420,
0x435, 0x438, 0x43d, 0x435, 0x41a, 0x438, 0x448, 0x438, 0x43d, 0x435, 0x432, 0x41a,
0x43e, 0x43f, 0x435, 0x43d, 0x4b3, 0x430, 0x433, 0x435, 0x43d, 0x414, 0x443, 0x431,
0x43b, 0x438, 0x43d, 0x413, 0x438, 0x431, 0x440, 0x430, 0x43b, 0x442, 0x430, 0x440,
0x413, 0x435, 0x440, 0x43d, 0x441, 0x438, 0x425, 0x435, 0x43b, 0x441, 0x438, 0x43d,
0x43a, 0x438, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20, 0x41c, 0x435, 0x43d, 0x421,
0x442, 0x430, 0x43c, 0x431, 0x443, 0x43b, 0x40f, 0x435, 0x440, 0x441, 0x438, 0x41a,
0x430, 0x43b, 0x438, 0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x434, 0x41a, 0x438,
0x440, 0x43e, 0x432, 0x41a, 0x438, 0x435, 0x432, 0x41b, 0x438, 0x441, 0x441, 0x430,
0x431, 0x43e, 0x43d, 0x41b, 0x438, 0x443, 0x431, 0x43b, 0x438, 0x430, 0x43d, 0x430,
0x41b, 0x43e, 0x43d, 0x434, 0x43e, 0x43d, 0x41b, 0x438, 0x443, 0x43a, 0x441, 0x435,
0x43c, 0x431, 0x443, 0x440, 0x433, 0x41c, 0x430, 0x434, 0x440, 0x438, 0x434, 0x41c,
0x430, 0x43b, 0x442, 0x430, 0x41c, 0x430, 0x440, 0x438, 0x435, 0x445, 0x430, 0x43c,
0x43d, 0x41c, 0x438, 0x43d, 0x441, 0x43a, 0x41c, 0x43e, 0x43d, 0x430, 0x43a, 0x43e,
0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x41e, 0x441, 0x43b, 0x43e, 0x41f, 0x430,
0x440, 0x438, 0x436, 0x41f, 0x43e, 0x434, 0x433, 0x43e, 0x440, 0x438, 0x446, 0x430,
0x41f, 0x440, 0x430, 0x433, 0x430, 0x420, 0x438, 0x433, 0x430, 0x420, 0x438, 0x43c,
0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x421, 0x430, 0x43d, 0x2d, 0x41c, 0x430,
0x440, 0x438, 0x43d, 0x43e, 0x421, 0x430, 0x440, 0x430, 0x435, 0x432, 0x43e, 0x421,
0x430, 0x440, 0x430, 0x442, 0x43e, 0x432, 0x421, 0x438, 0x43c, 0x444, 0x435, 0x440,
0x43e, 0x43f, 0x43e, 0x43b, 0x421, 0x43a, 0x43e, 0x43f, 0x435, 0x421, 0x43e, 0x444,
0x438, 0x430, 0x421, 0x442, 0x43e, 0x43a, 0x4b3, 0x43e, 0x43b, 0x43c, 0x422, 0x430,
0x43b, 0x43b, 0x438, 0x43d, 0x422, 0x438, 0x440, 0x430, 0x43d, 0x430, 0x423, 0x43b,
0x438, 0x430, 0x43d, 0x43e, 0x432, 0x441, 0x43a, 0x412, 0x430, 0x434, 0x443, 0x446,
0x412, 0x430, 0x442, 0x438, 0x43a, 0x430, 0x43d, 0x412, 0x435, 0x43d, 0x430, 0x412,
0x438, 0x43b, 0x43d, 0x438, 0x443, 0x441, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433,
0x440, 0x430, 0x434, 0x412, 0x430, 0x440, 0x448, 0x430, 0x432, 0x430, 0x417, 0x430,
0x433, 0x440, 0x435, 0x431, 0x426, 0x438, 0x443, 0x440, 0x438, 0x445, 0x410, 0x43d,
0x442, 0x430, 0x43d, 0x430, 0x43d, 0x430, 0x440, 0x438, 0x432, 0x443, 0x427, 0x430,
0x433, 0x43e, 0x441, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20, 0x49a, 0x44c, 0x44b,
0x440, 0x441, 0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41a, 0x43e, 0x43c, 0x43e,
0x440, 0x41a, 0x435, 0x440, 0x433, 0x435, 0x43b, 0x435, 0x43d, 0x41c, 0x430, 0x435,
0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a,
0x438, 0x41c, 0x430, 0x438, 0x43e, 0x442, 0x442, 0x430, 0x420, 0x435, 0x438, 0x443,
0x43d, 0x43e, 0x43d, 0x410, 0x43f, 0x438, 0x430, 0x41e, 0x43a, 0x43b, 0x435, 0x43d,
0x434, 0x411, 0x443, 0x433, 0x435, 0x43d, 0x432, 0x438, 0x43b, 0x427, 0x430, 0x442,
0x435, 0x43c, 0x422, 0x440, 0x443, 0x43a, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20,
0x41f, 0x430, 0x441, 0x445, 0x438, 0x415, 0x444, 0x430, 0x442, 0x435, 0x424, 0x430,
0x43a, 0x430, 0x43e, 0x444, 0x43e, 0x424, 0x438, 0x45f, 0x438, 0x424, 0x443, 0x43d,
0x430, 0x444, 0x443, 0x442, 0x438, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433,
0x43e, 0x441, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b,
0x4d9, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x20, 0x413,
0x430, 0x43c, 0x431, 0x435, 0x413, 0x443, 0x430, 0x434, 0x430, 0x43b, 0x43a, 0x430,
0x43d, 0x430, 0x43b, 0x413, 0x443, 0x430, 0x43c, 0x41a, 0x430, 0x43d, 0x442, 0x43e,
0x43d, 0x41a, 0x438, 0x440, 0x438, 0x442, 0x438, 0x43c, 0x430, 0x442, 0x438, 0x41a,
0x43e, 0x441, 0x440, 0x430, 0x435, 0x41a, 0x432, 0x430, 0x45f, 0x430, 0x43b, 0x435,
0x438, 0x43d, 0x41c, 0x430, 0x45f, 0x443, 0x440, 0x43e, 0x41c, 0x430, 0x440, 0x43a,
0x438, 0x437, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b,
0x4d9, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x20, 0x41c,
0x438, 0x434, 0x443, 0x435, 0x438, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x41d, 0x438,
0x443, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x41d, 0x443, 0x43c,
0x435, 0x430, 0x41f, 0x430, 0x433, 0x43e, 0x2d, 0x41f, 0x430, 0x433, 0x43e, 0x41f,
0x430, 0x43b, 0x430, 0x443, 0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x41f,
0x43e, 0x43d, 0x43f, 0x435, 0x438, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x41c, 0x43e,
0x440, 0x441, 0x431, 0x438, 0x420, 0x430, 0x440, 0x43e, 0x442, 0x43e, 0x43d, 0x433,
0x430, 0x421, 0x430, 0x438, 0x43f, 0x430, 0x43d, 0x422, 0x430, 0x438, 0x442, 0x438,
0x422, 0x430, 0x440, 0x430, 0x432, 0x430, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x442,
0x430, 0x43f, 0x443, 0x423, 0x435, 0x438, 0x43a, 0x423, 0x43e, 0x43b, 0x43b, 0x438,
0x441, 0x41, 0x64, 0x64, 0x69, 0x73, 0x20, 0x41, 0x62, 0x65, 0x62, 0x61,
0x4b, 0x61, 0xef, 0x72, 0x6f, 0x44, 0x6a, 0x69, 0x62, 0x6f, 0x65, 0x74,
0x69, 0x4b, 0x61, 0x72, 0x74, 0x6f, 0x65, 0x6d, 0x4d, 0x6f, 0x67, 0x61,
0x64, 0x69, 0x73, 0x6a, 0x6f, 0x65, 0x42, 0x61, 0x68, 0x69, 0x61, 0x20,
0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x43, 0x61, 0x6d, 0x62,
0x72, 0x69, 0x64, 0x67, 0x65, 0x62, 0x61, 0x61, 0x69, 0x43, 0x61, 0x6e,
0x63, 0x75, 0x6e, 0x4b, 0x61, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x47, 0x6c,
0x61, 0x63, 0x65, 0x62, 0x61, 0x61, 0x69, 0x47, 0x6f, 0x6f, 0x73, 0x65,
0x62, 0x61, 0x61, 0x69, 0x4a, 0x61, 0x6d, 0x61, 0x69, 0x6b, 0x61, 0x4d,
0x65, 0x72, 0x69, 0x64, 0x61, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f,
0x73, 0x74, 0x61, 0x64, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20,
0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e, 0x6f, 0x6f, 0x72,
0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20,
0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f, 0x6f, 0x72, 0x64,
0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x69, 0x6e, 0x74, 0x20,
0x42, 0x61, 0x72, 0x74, 0x68, 0xe9, 0x6c, 0x65, 0x6d, 0x79, 0x53, 0x69,
0x6e, 0x74, 0x20, 0x4a, 0x6f, 0x68, 0x6e, 0x2019, 0x73, 0x53, 0x69, 0x6e,
0x74, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73, 0x53, 0x69, 0x6e, 0x74, 0x20,
0x4c, 0x75, 0x63, 0x69, 0x61, 0x53, 0x69, 0x6e, 0x74, 0x20, 0x54, 0x68,
0x6f, 0x6d, 0x61, 0x73, 0x53, 0x69, 0x6e, 0x74, 0x20, 0x56, 0x69, 0x6e,
0x63, 0x65, 0x6e, 0x74, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x73,
0x6a, 0x63, 0x68, 0x61, 0x62, 0x61, 0x64, 0x42, 0x61, 0x67, 0x64, 0x61,
0x64, 0x42, 0x61, 0x68, 0x72, 0x65, 0x69, 0x6e, 0x42, 0x61, 0x6b, 0x6f,
0x65, 0x42, 0x65, 0x69, 0x72, 0x6f, 0x65, 0x74, 0x42, 0x69, 0x73, 0x6a,
0x6b, 0x65, 0x6b, 0x42, 0x72, 0x6f, 0x65, 0x6e, 0x65, 0x69, 0x44, 0x61,
0x6d, 0x61, 0x73, 0x6b, 0x75, 0x73, 0x44, 0x6f, 0x65, 0x62, 0x61, 0x69,
0x48, 0x6f, 0x20, 0x54, 0x73, 0x6a, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68,
0x73, 0x74, 0x61, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67,
0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x44, 0x6a, 0x61, 0x6b,
0x61, 0x72, 0x74, 0x61, 0x4b, 0x61, 0x62, 0x6f, 0x65, 0x6c, 0x4b, 0x61,
0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61,
0x74, 0x73, 0x6a, 0x69, 0x4b, 0x61, 0x74, 0x6d, 0x61, 0x6e, 0x64, 0x6f,
0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x4b, 0x6f, 0x65, 0x61, 0x6c, 0x61, 0x2d, 0x4c, 0x6f, 0x65, 0x6d, 0x70,
0x6f, 0x65, 0x72, 0x4b, 0x6f, 0x65, 0x77, 0x65, 0x69, 0x74, 0x4d, 0x61,
0x63, 0x61, 0x75, 0x4d, 0x75, 0x73, 0x6b, 0x61, 0x74, 0x4b, 0x61, 0x74,
0x61, 0x72, 0x52, 0x69, 0x61, 0x61, 0x64, 0x53, 0x65, 0x6f, 0x65, 0x6c,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x65, 0x72, 0x54, 0x6f, 0x6b,
0x69, 0x6f, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x73,
0x6f, 0x72, 0x65, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x65, 0x4b, 0x61,
0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x53, 0x75, 0x69, 0x64,
0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0xeb, 0x53, 0x69, 0x6e, 0x74,
0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x74, 0x68, 0x65, 0x6e,
0x65, 0x42, 0x65, 0x72, 0x6c, 0x79, 0x6e, 0x42, 0x72, 0x75, 0x73, 0x73,
0x65, 0x6c, 0x42, 0x6f, 0x65, 0x6b, 0x61, 0x72, 0x65, 0x73, 0x74, 0x42,
0x6f, 0x65, 0x64, 0x61, 0x70, 0x65, 0x73, 0x74, 0x4b, 0x6f, 0x70, 0x65,
0x6e, 0x68, 0x61, 0x67, 0x65, 0x6e, 0x45, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x4d, 0x61, 0x6e, 0x4b, 0x69, 0xeb, 0x66, 0x4c, 0x69, 0x73, 0x73,
0x61, 0x62, 0x6f, 0x6e, 0x4c, 0x6f, 0x6e, 0x64, 0x65, 0x6e, 0x4c, 0x75,
0x78, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x4d, 0x6f, 0x73, 0x6b, 0x6f,
0x75, 0x50, 0x61, 0x72, 0x79, 0x73, 0x50, 0x72, 0x61, 0x61, 0x67, 0x53,
0x61, 0x72, 0x61, 0x74, 0x6f, 0x66, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x61,
0x61, 0x6e, 0x73, 0x74, 0x61, 0x64, 0x57, 0x65, 0x6e, 0x65, 0x57, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x61, 0x72, 0x73, 0x6b,
0x6f, 0x75, 0x5a, 0xfc, 0x72, 0x69, 0x63, 0x68, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0x43, 0x6f, 0x6d, 0x6f, 0x72, 0x65, 0x4d, 0x61, 0x6c, 0x65, 0x64,
0x69, 0x76, 0x65, 0x50, 0x61, 0x61, 0x73, 0x46, 0x69, 0x64, 0x6a, 0x69,
0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x45, 0x6e, 0x64, 0x65,
0x72, 0x62, 0x75, 0x72, 0x79, 0x4e, 0x6f, 0x75, 0x6d, 0xe9, 0x61, 0x4d,
0x61, 0x74, 0x61, 0x2d, 0x55, 0x74, 0x75, 0x41, 0x62, 0x69, 0x78, 0x68,
0x61, 0x6e, 0x41, 0x6b, 0x72, 0x61, 0x41, 0x64, 0x69, 0x73, 0x2d, 0x41,
0x62, 0x61, 0x62, 0xeb, 0x41, 0x6c, 0x67, 0x6a, 0x65, 0x72, 0x41, 0x73,
0x6d, 0x61, 0x72, 0xeb, 0x42, 0x61, 0x6e, 0x78, 0x68, 0x75, 0x6c, 0x42,
0x69, 0x73, 0x61, 0x75, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x72, 0xeb,
0x42, 0x72, 0x61, 0x7a, 0x61, 0x76, 0x69, 0x6c, 0x6c, 0xeb, 0x42, 0x75,
0x78, 0x68, 0x75, 0x6d, 0x62, 0x75, 0x72, 0xeb, 0x4b, 0x61, 0x6a, 0x72,
0x6f, 0x4b, 0x61, 0x7a, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0xeb, 0x54,
0x68, 0x65, 0x75, 0x74, 0x61, 0x4b, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x69,
0x44, 0x61, 0x72, 0x2d, 0x65, 0x73, 0x2d, 0x53, 0x61, 0x6c, 0x61, 0x6d,
0x58, 0x68, 0x69, 0x62, 0x75, 0x74, 0x69, 0x45, 0x6c, 0x20, 0x41, 0x6a,
0x75, 0x6e, 0x46, 0x72, 0x69, 0x74, 0x61, 0x75, 0x6e, 0x4a, 0x6f, 0x68,
0x61, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x58, 0x68, 0x75, 0x62,
0x61, 0x4b, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x4c, 0x69, 0x62, 0x72, 0x65,
0x76, 0x69, 0x6c, 0xeb, 0x4e, 0x69, 0x61, 0x6d, 0x65, 0x6a, 0x4e, 0x6f,
0x75, 0x61, 0x6b, 0x6f, 0x74, 0x55, 0x61, 0x67, 0xeb, 0x64, 0x75, 0x67,
0x75, 0x53, 0x61, 0x6f, 0x2d, 0x54, 0x6f, 0x6d, 0x65, 0x54, 0x75, 0x6e,
0x69, 0x7a, 0x56, 0x69, 0x6e, 0x74, 0x2019, 0x68, 0x75, 0x6b, 0x41, 0x6e,
0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x78, 0x68, 0x41, 0x6e, 0x67, 0x75, 0x69,
0x6c, 0xeb, 0x41, 0x72, 0x61, 0x67, 0x75, 0x61, 0x6a, 0x61, 0x6e, 0x61,
0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x2d, 0x41, 0x6a, 0x72, 0x65, 0x73,
0x4b, 0x61, 0x74, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x4b, 0x6f, 0x72,
0x64, 0x6f, 0x62, 0x61, 0x48, 0x75, 0x68, 0x75, 0x69, 0x4c, 0x61, 0x20,
0x52, 0x69, 0x6f, 0x68, 0x61, 0x52, 0x69, 0x6f, 0x2d, 0x47, 0x61, 0x6c,
0x65, 0x67, 0x6f, 0x73, 0x53, 0x61, 0x6c, 0x74, 0xeb, 0x53, 0x61, 0x6e,
0x2d, 0x48, 0x75, 0x61, 0x6e, 0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x4c, 0x75,
0x69, 0x73, 0x54, 0x75, 0x6b, 0x75, 0x6d, 0x61, 0x6e, 0x55, 0x73, 0x68,
0x75, 0x61, 0x6a, 0x61, 0x41, 0x72, 0x75, 0x62, 0xeb, 0x41, 0x73, 0x75,
0x6e, 0x73, 0x69, 0x6f, 0x6e, 0x42, 0x61, 0x68, 0x69, 0x61, 0x2d, 0x42,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x42, 0x65, 0x6c, 0x69, 0x7a,
0xeb, 0x42, 0x6c, 0x61, 0x6e, 0x6b, 0x2d, 0x53, 0x61, 0x62, 0x6c, 0x6f,
0x6e, 0x42, 0x6f, 0x61, 0x2d, 0x56, 0x69, 0x73, 0x74, 0x61, 0x42, 0x6f,
0x67, 0x6f, 0x74, 0xeb, 0x42, 0x6f, 0x69, 0x7a, 0xeb, 0x47, 0x6a, 0x69,
0x72, 0x69, 0x20, 0x69, 0x20, 0x4b, 0x65, 0x6d, 0x62, 0x72, 0x69, 0x78,
0x68, 0x69, 0x74, 0x4b, 0x61, 0x6d, 0x70, 0x6f, 0x2d, 0x47, 0x72, 0x61,
0x6e, 0x64, 0x65, 0x4b, 0x61, 0x6e, 0x6b, 0x75, 0x6e, 0x4b, 0x61, 0x72,
0x61, 0x6b, 0x61, 0x73, 0x4b, 0x61, 0x6a, 0x65, 0x6e, 0xeb, 0x4b, 0x61,
0x6a, 0x6d, 0x61, 0x6e, 0xc7, 0x69, 0x6b, 0x61, 0x67, 0x6f, 0xc7, 0x69,
0x68, 0x61, 0x68, 0x75, 0x61, 0x53, 0x69, 0x75, 0x64, 0x61, 0x64, 0x2d,
0x48, 0x75, 0x61, 0x72, 0x65, 0x7a, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x2d,
0x52, 0x69, 0x6b, 0xeb, 0x4b, 0x72, 0x65, 0x73, 0x74, 0x6f, 0x6e, 0x4b,
0x75, 0x6a, 0x61, 0x62, 0x61, 0x4b, 0x75, 0x72, 0x61, 0x73, 0x61, 0x6f,
0x44, 0x65, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x68, 0x61, 0x76, 0xeb,
0x6e, 0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x47, 0x6a, 0x69, 0x72, 0x69,
0x20, 0x69, 0x20, 0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x69, 0x74, 0x44,
0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0xeb, 0x45, 0x6a, 0x72, 0x75, 0x6e,
0x65, 0x70, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x46, 0x6f,
0x72, 0x74, 0x2d, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x47, 0x6a, 0x69,
0x72, 0x69, 0x20, 0x69, 0x20, 0x4e, 0x67, 0x72, 0x69, 0x72, 0xeb, 0x47,
0x6a, 0x69, 0x72, 0x69, 0x20, 0x69, 0x20, 0x50, 0x61, 0x74, 0xeb, 0x73,
0x54, 0x75, 0x72, 0x6b, 0x75, 0x20, 0x69, 0x20, 0x4d, 0x61, 0x64, 0x68,
0x47, 0x72, 0x61, 0x6e, 0x61, 0x64, 0xeb, 0x47, 0x75, 0x61, 0x64, 0x65,
0x6c, 0x75, 0x70, 0xeb, 0x47, 0x75, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c,
0xeb, 0x47, 0x75, 0x61, 0x6a, 0x61, 0x6b, 0x75, 0x69, 0x6c, 0x47, 0x75,
0x61, 0x6a, 0x61, 0x6e, 0xeb, 0x48, 0x61, 0x6c, 0x69, 0x66, 0x61, 0x6b,
0x73, 0x48, 0x61, 0x76, 0x61, 0x6e, 0xeb, 0x48, 0x65, 0x72, 0x6d, 0x6f,
0x73, 0x69, 0x6a, 0x6f, 0x4b, 0x6e, 0x6f, 0x6b, 0x73, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x50, 0x65, 0x74, 0xeb, 0x72, 0x73,
0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x54, 0x65, 0x6c, 0x6c, 0x2d, 0x53, 0x69, 0x74, 0x69, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x65, 0x76, 0xeb, 0x6a,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e,
0x63, 0x65, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x55, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x49, 0x6b, 0x61, 0x6c, 0x75, 0x69,
0x74, 0x58, 0x68, 0x61, 0x6d, 0x61, 0x6a, 0x6b, 0xeb, 0x58, 0x68, 0x75,
0x6e, 0x6f, 0x75, 0x4c, 0x75, 0x69, 0x7a, 0x76, 0x69, 0x6c, 0xeb, 0x4d,
0x6f, 0x6e, 0x74, 0x69, 0xe7, 0x65, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65,
0x6e, 0x74, 0x61, 0x6b, 0x69, 0x4c, 0x69, 0x6d, 0xeb, 0x4c, 0x6f, 0x73,
0x20, 0x41, 0x6e, 0x78, 0x68, 0x65, 0x6c, 0x6f, 0x73, 0x4c, 0x6f, 0x75,
0x65, 0x72, 0x2d, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x69, 0x73, 0x2d, 0x4b,
0x75, 0x6f, 0x72, 0x74, 0xeb, 0x72, 0x4d, 0x61, 0x6b, 0x65, 0x6a, 0x6f,
0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x6b, 0xeb, 0x4d, 0x65, 0x6e,
0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x51, 0x79, 0x74, 0x65, 0x74, 0x69, 0x20,
0x69, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0xeb, 0x73, 0x4d, 0x69,
0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4d, 0x6f, 0x6e, 0x6b, 0x74, 0x6f, 0x6e,
0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x72, 0x72, 0x65, 0x6a, 0x4d, 0x6f, 0x6e,
0x74, 0x73, 0x65, 0x72, 0x61, 0x74, 0x4e, 0x61, 0x73, 0x61, 0x6f, 0x4e,
0x6a, 0x75, 0x2d, 0x4a, 0x6f, 0x72, 0x6b, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x6a, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x65, 0x20, 0x56, 0x65, 0x72, 0x69, 0x75, 0x74,
0x51, 0x65, 0x6e, 0x64, 0xeb, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x65, 0x20, 0x56, 0x65, 0x72, 0x69, 0x75, 0x74, 0x4e,
0x6a, 0x75, 0x2d, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x20, 0x65, 0x20, 0x56, 0x65, 0x72, 0x69, 0x75,
0x74, 0x4e, 0x6a, 0x75, 0x6b, 0x4f, 0x68, 0x69, 0x6e, 0x61, 0x67, 0x61,
0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x6f,
0x2d, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x2d,
0x56, 0x65, 0x6c, 0x68, 0x6f, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x2d, 0x52,
0x69, 0x6b, 0x6f, 0x50, 0x75, 0x6e, 0x74, 0x61, 0x2d, 0x41, 0x72, 0x65,
0x6e, 0x61, 0x73, 0x47, 0x72, 0x79, 0x6b, 0x61, 0x20, 0x49, 0x6e, 0x6c,
0x65, 0x74, 0x52, 0x65, 0x78, 0x68, 0x69, 0x6e, 0x61, 0x52, 0x69, 0x6f,
0x2d, 0x42, 0x72, 0x61, 0x6e, 0x6b, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x6f,
0x2d, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x53, 0x61, 0x6f, 0x2d,
0x50, 0x61, 0x75, 0x6c, 0x6f, 0x49, 0x74, 0x6f, 0x6b, 0x6f, 0x72, 0x74,
0x75, 0x72, 0x6d, 0x69, 0x74, 0x53, 0x65, 0x6e, 0x2d, 0x42, 0x61, 0x72,
0x74, 0x65, 0x6c, 0x65, 0x6d, 0x69, 0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x47,
0x6a, 0x6f, 0x6e, 0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x4b, 0x69, 0x74, 0x73,
0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x4c, 0x75, 0xe7, 0x69, 0x61, 0x53, 0x68,
0xeb, 0x6e, 0x2d, 0x54, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x68, 0xeb, 0x6e,
0x2d, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x52, 0x72, 0x79, 0x6d,
0x61, 0x20, 0x65, 0x20, 0x53, 0x68, 0x70, 0x65, 0x6a, 0x74, 0xeb, 0x54,
0x65, 0x67, 0x75, 0x73, 0x69, 0x67, 0x61, 0x6c, 0x70, 0x61, 0x44, 0x68,
0x75, 0x6c, 0x65, 0x54, 0x69, 0x68, 0x75, 0x61, 0x6e, 0x61, 0x54, 0x6f,
0x72, 0x74, 0x6f, 0x6c, 0xeb, 0x56, 0x61, 0x6e, 0x6b, 0x75, 0x76, 0x65,
0x72, 0x55, 0x61, 0x6a, 0x74, 0x2019, 0x68, 0x6f, 0x72, 0x73, 0x55, 0x69,
0x6e, 0x69, 0x70, 0x65, 0x67, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x61, 0x74,
0x4b, 0x65, 0x6a, 0x73, 0x69, 0x44, 0x65, 0x6a, 0x76, 0x69, 0x73, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0xeb, 0x4d, 0x65, 0x6b, 0x75, 0x61, 0x72, 0x69, 0x4d, 0x61, 0x75,
0x73, 0x6f, 0x6e, 0x4d, 0x65, 0x6b, 0x6d, 0x75, 0x72, 0x64, 0x6f, 0x52,
0x6f, 0x64, 0x68, 0x65, 0x72, 0xeb, 0x53, 0x6a, 0x6f, 0x75, 0x61, 0x4c,
0x6f, 0x6e, 0x67, 0x2019, 0x6a, 0xeb, 0x62, 0x6a, 0x65, 0x6e, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x69, 0x41, 0x6d, 0x61, 0x6e, 0x41, 0x6e, 0x61, 0x64,
0x69, 0x72, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x41, 0x6b, 0x74, 0x6f, 0x62,
0x65, 0x41, 0x74, 0x69, 0x72, 0x61, 0x75, 0x42, 0x61, 0x68, 0x72, 0x65,
0x6a, 0x6e, 0x42, 0x65, 0x6a, 0x72, 0x75, 0x74, 0xc7, 0x69, 0x74, 0x61,
0xc7, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x4b, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x6f, 0x44, 0x61, 0x6b, 0x61, 0x46, 0x61, 0x6d, 0x61,
0x67, 0x75, 0x73, 0x74, 0xeb, 0x48, 0x6f, 0x2d, 0xc7, 0x69, 0x2d, 0x4d,
0x69, 0x6e, 0x48, 0x6f, 0x6e, 0x67, 0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x58,
0x68, 0x61, 0x6b, 0x61, 0x72, 0x74, 0xeb, 0x58, 0x68, 0x61, 0x6a, 0x61,
0x70, 0x75, 0x72, 0xeb, 0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x6b, 0xeb,
0x4b, 0x61, 0x72, 0x61, 0xe7, 0x69, 0x4b, 0x61, 0x74, 0x6d, 0x61, 0x6e,
0x64, 0x75, 0x4b, 0x61, 0x6e, 0x64, 0x69, 0x67, 0xeb, 0x4b, 0x61, 0x6c,
0x6b, 0x75, 0x74, 0xeb, 0x4b, 0x75, 0x61, 0x6c, 0x61, 0x2d, 0x4c, 0x75,
0x6d, 0x70, 0x75, 0x72, 0x4b, 0x75, 0xe7, 0x69, 0x6e, 0x67, 0x4b, 0x75,
0x76, 0x61, 0x6a, 0x74, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x4d, 0x61, 0x6b,
0x61, 0x73, 0x61, 0x72, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0xeb, 0x4e, 0x69,
0x6b, 0x6f, 0x7a, 0x69, 0x61, 0x50, 0x6e, 0x6f, 0x6d, 0x2d, 0x50, 0x65,
0x6e, 0x50, 0x65, 0x6e, 0x69, 0x61, 0x6e, 0x4b, 0x6f, 0x73, 0x74, 0x61,
0x6e, 0x61, 0x6a, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x52, 0x69, 0x61, 0x64, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x53,
0x65, 0x75, 0x6c, 0x53, 0x68, 0x61, 0x6e, 0x67, 0x61, 0x69, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x65,
0x6b, 0x6f, 0x6c, 0x69, 0x6d, 0x73, 0x6b, 0x54, 0x61, 0x6a, 0x70, 0x65,
0x6a, 0x54, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x54, 0x65, 0x68, 0x65, 0x72,
0x61, 0x6e, 0x54, 0x68, 0x69, 0x6d, 0x70, 0x75, 0x55, 0x6c, 0x61, 0x6e,
0x62, 0x61, 0x74, 0x61, 0x72, 0x56, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x61,
0x6e, 0xeb, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x52, 0x61, 0x6e,
0x67, 0x75, 0x6e, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x4a, 0x65, 0x72, 0x65, 0x76, 0x61, 0x6e, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x65, 0x4b, 0x65,
0x70, 0x69, 0x20, 0x69, 0x20, 0x47, 0x6a, 0x65, 0x6c, 0x62, 0xeb, 0x72,
0x52, 0x65, 0x69, 0x6b, 0x6a, 0x61, 0x76, 0x69, 0x6b, 0x58, 0x68, 0x6f,
0x72, 0x78, 0x68, 0x61, 0x20, 0x65, 0x20, 0x4a, 0x75, 0x67, 0x75, 0x74,
0x53, 0x68, 0xeb, 0x6e, 0x2d, 0x45, 0x6c, 0x65, 0x6e, 0xeb, 0x53, 0x74,
0x65, 0x6e, 0x6c, 0x69, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x6a, 0x64, 0x65,
0x42, 0x72, 0x69, 0x73, 0x62, 0x65, 0x6a, 0x6e, 0x42, 0x72, 0x6f, 0x6b,
0xeb, 0x6e, 0x2d, 0x48, 0x69, 0x6c, 0x6c, 0x44, 0x61, 0x72, 0x76, 0x69,
0x6e, 0x45, 0x75, 0x6b, 0x6c, 0x61, 0x4c, 0x69, 0x6e, 0x64, 0xeb, 0x6d,
0x65, 0x6e, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x75, 0x69, 0x4d,
0x65, 0x6c, 0x62, 0x75, 0x72, 0x6e, 0x50, 0xeb, 0x72, 0x74, 0x68, 0x53,
0x69, 0x64, 0x6e, 0x65, 0x6a, 0x41, 0x6e, 0x64, 0x6f, 0x72, 0x72, 0xeb,
0x41, 0x73, 0x74, 0x72, 0x61, 0x6b, 0x61, 0x6e, 0x41, 0x74, 0x68, 0x69,
0x6e, 0xeb, 0x42, 0x65, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x42, 0x72, 0x61,
0x74, 0x69, 0x73, 0x6c, 0x61, 0x76, 0xeb, 0x42, 0x72, 0x75, 0x6b, 0x73,
0x65, 0x6c, 0x42, 0x75, 0x6b, 0x75, 0x72, 0x65, 0x73, 0x68, 0x74, 0x4b,
0x69, 0x73, 0x68, 0x69, 0x6e, 0x65, 0x75, 0x47, 0x6a, 0x69, 0x62, 0x72,
0x61, 0x6c, 0x74, 0x61, 0x72, 0x47, 0x65, 0x72, 0x6e, 0x73, 0x65, 0x6a,
0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x20, 0x69, 0x20, 0x4d, 0x61,
0x6e, 0x69, 0x74, 0x53, 0x74, 0x61, 0x6d, 0x62, 0x6f, 0x6c, 0x6c, 0x58,
0x68, 0x65, 0x72, 0x73, 0x65, 0x6a, 0x4b, 0x69, 0x65, 0x76, 0x4c, 0x69,
0x73, 0x62, 0x6f, 0x6e, 0xeb, 0x4c, 0x75, 0x62, 0x6a, 0x61, 0x6e, 0xeb,
0x4c, 0x6f, 0x6e, 0x64, 0xeb, 0x72, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d,
0x62, 0x75, 0x72, 0x67, 0x4d, 0x61, 0x6c, 0x74, 0xeb, 0x4d, 0x6f, 0x6e,
0x61, 0x6b, 0x6f, 0x4d, 0x6f, 0x73, 0x6b, 0xeb, 0x50, 0x6f, 0x64, 0x67,
0x6f, 0x72, 0x69, 0x63, 0xeb, 0x50, 0x72, 0x61, 0x67, 0xeb, 0x52, 0x69,
0x67, 0xeb, 0x52, 0x6f, 0x6d, 0xeb, 0x53, 0x61, 0x6e, 0x2d, 0x4d, 0x61,
0x72, 0x69, 0x6e, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0xeb,
0x53, 0x68, 0x6b, 0x75, 0x70, 0x53, 0x6f, 0x66, 0x6a, 0x65, 0x53, 0x74,
0x6f, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x54, 0x61, 0x6c, 0x69, 0x6e, 0x54,
0x69, 0x72, 0x61, 0x6e, 0xeb, 0x55, 0x6c, 0x6a, 0x61, 0x6e, 0x6f, 0x76,
0x73, 0x6b, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e, 0x56, 0x6a, 0x65,
0x6e, 0xeb, 0x56, 0x61, 0x72, 0x73, 0x68, 0x61, 0x76, 0xeb, 0x5a, 0x79,
0x72, 0x69, 0x68, 0xc7, 0x61, 0x67, 0x6f, 0x73, 0x4b, 0x72, 0x69, 0x73,
0x68, 0x74, 0x6c, 0x69, 0x6e, 0x64, 0x6a, 0x65, 0x4b, 0x6f, 0x6d, 0x6f,
0x72, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x4d, 0x61, 0x6a,
0x6f, 0x74, 0xeb, 0x4f, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x42, 0x75, 0x6e,
0x67, 0x61, 0x6e, 0x76, 0x69, 0x6c, 0xeb, 0xc7, 0x61, 0x74, 0x6d, 0x61,
0x6e, 0xc7, 0x75, 0x6b, 0x50, 0x61, 0x73, 0x68, 0x6b, 0xeb, 0x46, 0x69,
0x78, 0x68, 0x69, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e,
0x61, 0x6c, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x74, 0x4b,
0x6f, 0x73, 0x72, 0x65, 0x4b, 0x75, 0x61, 0x78, 0x68, 0x61, 0x6c, 0x65,
0x69, 0x6e, 0x4d, 0x61, 0x68, 0x75, 0x72, 0x6f, 0x4d, 0x61, 0x72, 0x6b,
0x65, 0x7a, 0x4d, 0x69, 0x64, 0x75, 0x65, 0x6a, 0x50, 0x61, 0x67, 0x6f,
0x2d, 0x50, 0x61, 0x67, 0x6f, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x69, 0x50, 0x6f, 0x72, 0x74, 0x2d,
0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x69, 0x52, 0x61, 0x72, 0x6f, 0x74,
0x6f, 0x6e, 0x67, 0xeb, 0x54, 0x61, 0x72, 0x61, 0x76, 0xeb, 0x55, 0x65,
0x6a, 0x6b, 0x55, 0x6f, 0x6c, 0x6c, 0x69, 0x73, 0x12a0, 0x1262, 0x1303, 0x1295,
0x12a0, 0x12ad, 0x122b, 0x12a0, 0x12f2, 0x1235, 0x20, 0x12a0, 0x1260, 0x1263, 0x12a0, 0x120d,
0x1300, 0x122d, 0x1235, 0x12a0, 0x1235, 0x1218, 0x122b, 0x1263, 0x121b, 0x12ae, 0x1263, 0x1295,
0x1309, 0x12a2, 0x1263, 0x1295, 0x1301, 0x120d, 0x1262, 0x1233, 0x12a6, 0x1265, 0x120b, 0x1295,
0x1273, 0x12e8, 0x122d, 0x1265, 0x122b, 0x12db, 0x126a, 0x120d, 0x1261, 0x1301, 0x121d, 0x1265,
0x122b, 0x12ab, 0x12ed, 0x122e, 0x12ab, 0x12db, 0x1265, 0x120b, 0x1295, 0x12ab, 0x1232, 0x12a1,
0x1273, 0x12ae, 0x1293, 0x12ad, 0x122c, 0x12f3, 0x12ab, 0x122d, 0x12f3, 0x122c, 0x20, 0x1230,
0x120b, 0x121d, 0x1305, 0x1261, 0x1272, 0x12f1, 0x12a0, 0x120b, 0x12a4, 0x120d, 0x20, 0x12a0,
0x12e9, 0x12a0, 0x1295, 0x134d, 0x122a, 0x1273, 0x12cd, 0x1295, 0x130b, 0x1266, 0x122e, 0x1295,
0x1203, 0x122b, 0x122c, 0x1306, 0x1203, 0x1295, 0x1235, 0x1260, 0x122d, 0x130d, 0x1301, 0x1263,
0x12ab, 0x121d, 0x1353, 0x120b, 0x12ab, 0x122d, 0x1271, 0x121d, 0x12aa, 0x130b, 0x120a, 0x12aa,
0x1295, 0x123b, 0x1233, 0x120c, 0x130e, 0x1235, 0x120a, 0x1260, 0x122d, 0x126a, 0x120d, 0x120e,
0x121c, 0x1209, 0x12a0, 0x1295, 0x12f3, 0x1209, 0x1261, 0x121d, 0x1263, 0x123a, 0x1209, 0x1233,
0x12ab, 0x121b, 0x120b, 0x1261, 0x121b, 0x1351, 0x1271, 0x121b, 0x1234, 0x1229, 0x121d, 0x1263,
0x1263, 0x1294, 0x121e, 0x1243, 0x12f2, 0x1239, 0x121e, 0x1295, 0x122e, 0x126a, 0x12eb, 0x1293,
0x12ed, 0x122e, 0x1262, 0x1295, 0x1303, 0x121c, 0x1293, 0x1292, 0x12eb, 0x121c, 0x12ed, 0x1291,
0x12a0, 0x12ad, 0x127e, 0x1275, 0x12a1, 0x130b, 0x12f1, 0x1309, 0x1356, 0x122d, 0x1276, 0x20,
0x2d, 0x20, 0x1296, 0x126e, 0x1233, 0x12a6, 0x20, 0x1276, 0x121c, 0x1275, 0x122a, 0x1356,
0x120a, 0x1271, 0x1292, 0x12dd, 0x12ca, 0x1295, 0x12f5, 0x1206, 0x12ad, 0x12a0, 0x12f3, 0x12ad,
0x12a0, 0x1295, 0x12ae, 0x122b, 0x1305, 0x12a0, 0x1295, 0x1309, 0x12ed, 0x120b, 0x12a0, 0x1295,
0x1272, 0x1313, 0x12a0, 0x122b, 0x1309, 0x12e8, 0x1293, 0x1266, 0x1290, 0x1235, 0x20, 0x12a0,
0x12ed, 0x1228, 0x1235, 0x12ab, 0x1273, 0x121b, 0x122d, 0x12ab, 0x12ae, 0x122d, 0x12f6, 0x1263,
0x1301, 0x1301, 0x12ed, 0x120b, 0x20, 0x122a, 0x12a6, 0x1303, 0x121c, 0x1295, 0x12f6, 0x12db,
0x122a, 0x12ee, 0x20, 0x130b, 0x120c, 0x130e, 0x1235, 0x1233, 0x120d, 0x1273, 0x1233, 0x1295,
0x20, 0x1301, 0x12a0, 0x1295, 0x1233, 0x1295, 0x20, 0x120a, 0x12cd, 0x1235, 0x1271, 0x12a9,
0x121b, 0x1295, 0x12a1, 0x1239, 0x12a0, 0x12a2, 0x12a0, 0x1229, 0x1263, 0x12a0, 0x1231, 0x1295,
0x1232, 0x12ee, 0x1295, 0x12a0, 0x1272, 0x12ae, 0x12ab, 0x1295, 0x1263, 0x1202, 0x12a0, 0x1263,
0x1202, 0x12eb, 0x20, 0x1263, 0x1295, 0x12f0, 0x122b, 0x1235, 0x1263, 0x122d, 0x1264, 0x12f6,
0x1235, 0x1264, 0x1208, 0x121d, 0x1264, 0x120a, 0x12dd, 0x1265, 0x120b, 0x1295, 0x12ad, 0x2d,
0x20, 0x1233, 0x1265, 0x120e, 0x1295, 0x1266, 0x12a0, 0x20, 0x126a, 0x1235, 0x1273, 0x1266,
0x130e, 0x1273, 0x1266, 0x12ed, 0x12dd, 0x12ab, 0x121d, 0x1265, 0x122a, 0x1305, 0x20, 0x1264,
0x12ed, 0x12ab, 0x121d, 0x1356, 0x20, 0x130d, 0x122b, 0x1295, 0x12f4, 0x12ab, 0x1295, 0x12a9,
0x1295, 0x12ab, 0x122b, 0x12ab, 0x1235, 0x12ab, 0x12ed, 0x1295, 0x12a4, 0x12ab, 0x12ed, 0x121b,
0x1295, 0x127a, 0x12ab, 0x130e, 0x127a, 0x1201, 0x12cb, 0x12cd, 0x12a0, 0x1233, 0x12ee, 0x12f3,
0x12f5, 0x20, 0x1201, 0x12cb, 0x122c, 0x12dd, 0x12ae, 0x1235, 0x1273, 0x122a, 0x12ab, 0x12ad,
0x1228, 0x1235, 0x1270, 0x1295, 0x12a9, 0x12e8, 0x12a0, 0x1263, 0x12a9, 0x122b, 0x1233, 0x12ce,
0x12f3, 0x1295, 0x121b, 0x122d, 0x12ad, 0x123b, 0x126d, 0x1295, 0x12f3, 0x12cd, 0x1230, 0x1295,
0x12f3, 0x12cd, 0x1230, 0x1295, 0x20, 0x12ad, 0x122c, 0x12ad, 0x12f4, 0x1295, 0x1268, 0x122d,
0x12f2, 0x1275, 0x122e, 0x12ed, 0x1275, 0x12f6, 0x121c, 0x1292, 0x12ab, 0x12a4, 0x12f5, 0x1218,
0x1295, 0x1270, 0x1295, 0x12a2, 0x1229, 0x1294, 0x1355, 0x12a4, 0x120d, 0x1233, 0x120d, 0x126b,
0x12f6, 0x122d, 0x134e, 0x122d, 0x1275, 0x20, 0x1294, 0x120d, 0x1230, 0x1295, 0x134e, 0x122d,
0x1273, 0x120c, 0x12db, 0x130d, 0x120c, 0x1235, 0x20, 0x1264, 0x12ed, 0x1309, 0x12dd, 0x20,
0x1264, 0x12ed, 0x130d, 0x122b, 0x1295, 0x12f5, 0x20, 0x1270, 0x122d, 0x12ad, 0x130d, 0x122c,
0x1293, 0x12f3, 0x1315, 0x12f3, 0x1209, 0x1355, 0x1313, 0x1272, 0x121b, 0x120b, 0x1309, 0x12eb,
0x12a9, 0x12ed, 0x120d, 0x1309, 0x12eb, 0x1293, 0x1203, 0x120a, 0x134b, 0x12ad, 0x1235, 0x1203,
0x126b, 0x1293, 0x12b8, 0x122d, 0x121e, 0x12da, 0x120e, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293,
0x1356, 0x120a, 0x1235, 0x1296, 0x12ad, 0x1235, 0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb,
0x1293, 0x121b, 0x122c, 0x1295, 0x130e, 0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293,
0x1352, 0x1270, 0x122d, 0x1235, 0x1260, 0x122d, 0x130d, 0x2c, 0x20, 0x12a2, 0x1295, 0x12f2,
0x12eb, 0x1293, 0x1274, 0x120d, 0x20, 0x12a8, 0x1270, 0x121b, 0x2c, 0x20, 0x12a2, 0x1295,
0x12f2, 0x12eb, 0x1293, 0x126a, 0x126b, 0x12ed, 0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb,
0x1293, 0x126a, 0x1295, 0x127c, 0x1295, 0x1235, 0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb,
0x1293, 0x12ca, 0x1293, 0x121b, 0x12ad, 0x2c, 0x20, 0x12a2, 0x1295, 0x12f2, 0x12eb, 0x1293,
0x12a2, 0x1291, 0x126a, 0x12ad, 0x12a2, 0x12b3, 0x120a, 0x12cd, 0x1275, 0x1303, 0x121b, 0x12ed,
0x12ab, 0x1301, 0x1292, 0x12e9, 0x120a, 0x12cd, 0x1235, 0x126a, 0x120d, 0x121e, 0x1295, 0x1272,
0x1234, 0x120e, 0x2c, 0x20, 0x12aa, 0x1295, 0x1273, 0x12aa, 0x12ad, 0x122b, 0x1208, 0x1295,
0x12f2, 0x12ed, 0x12ad, 0x120b, 0x20, 0x1353, 0x12dd, 0x120a, 0x121b, 0x120e, 0x1235, 0x20,
0x12a0, 0x1295, 0x1300, 0x1208, 0x1235, 0x12e8, 0x1273, 0x127d, 0x129b, 0x12cd, 0x20, 0x120d,
0x12d1, 0x120d, 0x20, 0x1229, 0x1265, 0x121c, 0x1232, 0x12a6, 0x121b, 0x1293, 0x1309, 0x12a0,
0x121b, 0x1293, 0x12a1, 0x1235, 0x121b, 0x122d, 0x130e, 0x1275, 0x121b, 0x122d, 0x1272, 0x1292,
0x12ad, 0x121b, 0x1273, 0x121e, 0x122e, 0x1235, 0x121b, 0x12db, 0x1275, 0x120b, 0x1295, 0x121c,
0x1296, 0x121a, 0x1292, 0x121c, 0x122a, 0x12f3, 0x1218, 0x1275, 0x120b, 0x12ab, 0x1275, 0x120b,
0x121c, 0x12ad, 0x1232, 0x12ae, 0x20, 0x12a8, 0x1270, 0x121b, 0x121a, 0x12ae, 0x12cd, 0x120e,
0x1295, 0x121e, 0x1295, 0x12ad, 0x1276, 0x1295, 0x121e, 0x1295, 0x1270, 0x122d, 0x122c, 0x12ed,
0x121e, 0x1295, 0x1274, 0x126a, 0x12f5, 0x12ee, 0x121e, 0x1295, 0x1275, 0x1234, 0x122b, 0x1275,
0x1293, 0x1233, 0x12cd, 0x1292, 0x12cd, 0x12ee, 0x122d, 0x12ad, 0x1296, 0x121c, 0x1296, 0x122e,
0x129b, 0x1264, 0x12a1, 0x120b, 0x1205, 0x2c, 0x20, 0x1230, 0x121c, 0x1295, 0x20, 0x12f3,
0x12ae, 0x1273, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x2c, 0x20, 0x1230, 0x121c, 0x1295,
0x20, 0x12f3, 0x12ae, 0x1273, 0x12a0, 0x12f2, 0x1235, 0x20, 0x1233, 0x120c, 0x121d, 0x2c,
0x20, 0x1230, 0x121c, 0x1295, 0x20, 0x12f3, 0x12ae, 0x1273, 0x130b, 0x12f5, 0x1273, 0x1265,
0x12a6, 0x12ea, 0x1293, 0x130b, 0x1353, 0x1293, 0x121b, 0x1353, 0x122b, 0x121b, 0x122a, 0x1266,
0x134a, 0x1292, 0x12ad, 0x1235, 0x1356, 0x122d, 0x1275, 0x20, 0x12a6, 0x20, 0x1355, 0x122a,
0x1295, 0x1235, 0x12e8, 0x12a5, 0x1235, 0x1354, 0x1295, 0x20, 0x12c8, 0x12f0, 0x1265, 0x1354,
0x1275, 0x122e, 0x20, 0x126c, 0x120d, 0x1206, 0x1356, 0x122d, 0x1276, 0x122a, 0x12ae, 0x1351,
0x1295, 0x1273, 0x20, 0x12a0, 0x122c, 0x1293, 0x1235, 0x122b, 0x1295, 0x12aa, 0x1295, 0x20,
0x12a2, 0x1295, 0x120c, 0x1275, 0x1228, 0x1232, 0x134d, 0x1228, 0x1302, 0x1293, 0x122a, 0x12de,
0x120a, 0x12e9, 0x1275, 0x122a, 0x12ee, 0x20, 0x1265, 0x122b, 0x1295, 0x12ae, 0x1233, 0x1295,
0x1273, 0x122c, 0x121d, 0x1233, 0x1295, 0x1272, 0x12eb, 0x130e, 0x1233, 0x1295, 0x1276, 0x20,
0x12f6, 0x121a, 0x1295, 0x130e, 0x1233, 0x12a6, 0x20, 0x1356, 0x120e, 0x1235, 0x12ae, 0x122d,
0x1235, 0x1263, 0x12ed, 0x1230, 0x1295, 0x12f5, 0x1232, 0x1275, 0x12ab, 0x1245, 0x12f5, 0x1235,
0x1275, 0x20, 0x1264, 0x122d, 0x1270, 0x120e, 0x121c, 0x1245, 0x12f1, 0x1235, 0x20, 0x12ee,
0x1210, 0x1295, 0x1235, 0x1245, 0x12f1, 0x1235, 0x20, 0x12aa, 0x1275, 0x1235, 0x1245, 0x12f5,
0x1235, 0x1275, 0x20, 0x1209, 0x1232, 0x12eb, 0x1245, 0x12f1, 0x1235, 0x20, 0x1276, 0x121b,
0x1235, 0x1245, 0x12f1, 0x1235, 0x20, 0x126a, 0x1295, 0x1230, 0x1295, 0x1275, 0x12e8, 0x1210,
0x12cb, 0x120b, 0x20, 0x1308, 0x1295, 0x12d8, 0x1265, 0x1274, 0x1309, 0x1232, 0x130b, 0x120d,
0x1353, 0x1271, 0x120c, 0x1272, 0x1301, 0x12a0, 0x1293, 0x1276, 0x122e, 0x1295, 0x1276, 0x1276,
0x122d, 0x1276, 0x120b, 0x126b, 0x1295, 0x12ae, 0x1268, 0x122d, 0x128b, 0x12ed, 0x1275, 0x1206,
0x122d, 0x1235, 0x12ca, 0x1292, 0x1354, 0x130d, 0x12eb, 0x12a9, 0x1273, 0x1275, 0x12ab, 0x12da,
0x12ed, 0x12f3, 0x126a, 0x1235, 0x12f0, 0x121e, 0x1295, 0x1275, 0x20, 0x12f2, 0x12a1, 0x122d,
0x126a, 0x120d, 0x121b, 0x12a8, 0x122a, 0x1293, 0x12cd, 0x1230, 0x1295, 0x121b, 0x12ad, 0x1218,
0x122d, 0x12f6, 0x1353, 0x120d, 0x1218, 0x122d, 0x122e, 0x1274, 0x122b, 0x1235, 0x12ee, 0x12cb,
0x1275, 0x122e, 0x120d, 0x126d, 0x1235, 0x1276, 0x12ad, 0x120e, 0x1295, 0x130d, 0x12ed, 0x122d,
0x1263, 0x12e8, 0x1295, 0x12a4, 0x12f0, 0x1295, 0x12a0, 0x120d, 0x121b, 0x1275, 0x12ed, 0x12a0,
0x121b, 0x1295, 0x12a0, 0x1293, 0x12f5, 0x12ed, 0x122d, 0x12a0, 0x12a9, 0x1273, 0x12a1, 0x12a0,
0x12a9, 0x1276, 0x1264, 0x12a0, 0x123d, 0x130b, 0x1263, 0x1275, 0x12a0, 0x1275, 0x12ed, 0x122b,
0x12a1, 0x1263, 0x130d, 0x12f3, 0x12f5, 0x1263, 0x1205, 0x122c, 0x1295, 0x1263, 0x1295, 0x12ae,
0x12ad, 0x1263, 0x122d, 0x1293, 0x12a1, 0x120d, 0x1264, 0x1229, 0x1275, 0x1262, 0x123d, 0x12ac,
0x12ad, 0x1265, 0x1229, 0x1293, 0x12ed, 0x127a, 0x1273, 0x127e, 0x12ed, 0x1263, 0x120d, 0x1233,
0x1295, 0x12ae, 0x120e, 0x121d, 0x1266, 0x12f0, 0x121b, 0x1235, 0x1246, 0x12f2, 0x120a, 0x12f1,
0x1263, 0x12ed, 0x12f0, 0x123b, 0x1295, 0x1264, 0x134b, 0x121b, 0x1309, 0x1235, 0x1273, 0x130b,
0x12db, 0x12ac, 0x1265, 0x122e, 0x1295, 0x1206, 0x20, 0x127a, 0x20, 0x121a, 0x1295, 0x1205,
0x20, 0x12a8, 0x1270, 0x121b, 0x1206, 0x1295, 0x130d, 0x20, 0x12ae, 0x1295, 0x130d, 0x1206,
0x126d, 0x12f5, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x1303, 0x12ab, 0x122d, 0x1273,
0x1303, 0x12eb, 0x1351, 0x122b, 0x12a5, 0x12e8, 0x1229, 0x1233, 0x120c, 0x121d, 0x12ab, 0x1261,
0x120d, 0x12ab, 0x121d, 0x127b, 0x1275, 0x12ab, 0x12ab, 0x122b, 0x127a, 0x12ab, 0x1275, 0x121b,
0x1295, 0x12f1, 0x12ab, 0x1295, 0x12f2, 0x130b, 0x12ae, 0x120d, 0x12ab, 0x1273, 0x12ad, 0x122b,
0x1235, 0x1296, 0x12eb, 0x122d, 0x1235, 0x12ad, 0x12a9, 0x12cb, 0x120b, 0x20, 0x120b, 0x121d,
0x1351, 0x122d, 0x12a9, 0x127a, 0x1295, 0x130d, 0x12a9, 0x12cc, 0x1275, 0x121b, 0x12ab, 0x12a1,
0x121b, 0x130b, 0x12f3, 0x1295, 0x121b, 0x12ab, 0x1233, 0x122d, 0x121b, 0x1292, 0x120b, 0x1219,
0x1235, 0x12ab, 0x1275, 0x1292, 0x12ae, 0x1232, 0x12a0, 0x1296, 0x126e, 0x12a9, 0x1275, 0x12dd,
0x1294, 0x12ad, 0x1296, 0x126e, 0x1232, 0x1262, 0x122a, 0x1235, 0x12ad, 0x12a6, 0x121d, 0x1235,
0x12ad, 0x12a6, 0x122b, 0x120d, 0x134d, 0x1296, 0x121d, 0x20, 0x1354, 0x1295, 0x1205, 0x1356,
0x1295, 0x1272, 0x12a0, 0x1293, 0x12ad, 0x1355, 0x12ee, 0x1295, 0x130d, 0x12eb, 0x1295, 0x130d,
0x12b3, 0x1273, 0x122d, 0x12ae, 0x1235, 0x1273, 0x1293, 0x12ed, 0x12a9, 0x12ed, 0x12da, 0x120e,
0x122d, 0x12f3, 0x122a, 0x12eb, 0x12f5, 0x1233, 0x12ad, 0x1203, 0x120a, 0x1295, 0x1233, 0x121b,
0x122d, 0x12ab, 0x1295, 0x12f5, 0x1234, 0x12a6, 0x120d, 0x123b, 0x1295, 0x130b, 0x12ed, 0x1232,
0x1295, 0x130b, 0x1356, 0x122d, 0x1235, 0x122c, 0x12f5, 0x1294, 0x1235, 0x12ae, 0x120d, 0x121d,
0x1235, 0x12ad, 0x1273, 0x12ed, 0x1353, 0x12ed, 0x1273, 0x123d, 0x12ac, 0x1295, 0x1275, 0x1275,
0x1265, 0x120a, 0x1232, 0x1274, 0x1205, 0x122b, 0x1295, 0x1272, 0x121d, 0x1349, 0x1276, 0x12aa,
0x12ee, 0x1276, 0x121d, 0x1235, 0x12ad, 0x12a1, 0x120b, 0x12a0, 0x1295, 0x1263, 0x12a0, 0x1273,
0x122d, 0x12a1, 0x1229, 0x121d, 0x12aa, 0x12a1, 0x1235, 0x1275, 0x2d, 0x1294, 0x122b, 0x126c,
0x1295, 0x1272, 0x12a0, 0x1295, 0x126d, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276, 0x12ad, 0x12eb,
0x12a9, 0x1275, 0x1235, 0x12ad, 0x12eb, 0x1295, 0x130e, 0x1295, 0x12e8, 0x12ab, 0x1270, 0x122a,
0x1295, 0x1260, 0x122d, 0x130d, 0x12ed, 0x122c, 0x126b, 0x1295, 0x12a0, 0x12de, 0x1228, 0x1235,
0x1264, 0x122d, 0x1219, 0x12f3, 0x12ab, 0x1293, 0x122a, 0x12ac, 0x1355, 0x20, 0x126c, 0x122d,
0x12f0, 0x134b, 0x122e, 0x12a5, 0x121b, 0x12f4, 0x122b, 0x122c, 0x12ed, 0x12ad, 0x1303, 0x126a,
0x12ad, 0x12f0, 0x1261, 0x1265, 0x20, 0x1306, 0x122d, 0x1302, 0x12eb, 0x1245, 0x12f5, 0x1235,
0x1275, 0x20, 0x1204, 0x1208, 0x1293, 0x1235, 0x1273, 0x1295, 0x120c, 0x12ed, 0x12a0, 0x12f4,
0x120c, 0x12a5, 0x12f5, 0x1265, 0x122a, 0x1235, 0x1264, 0x1295, 0x1265, 0x122e, 0x12ad, 0x1295,
0x20, 0x1202, 0x120d, 0x12f3, 0x122d, 0x12ca, 0x1295, 0x12a1, 0x12ad, 0x120b, 0x1206, 0x1263,
0x122d, 0x1275, 0x120a, 0x1295, 0x12f5, 0x121b, 0x1295, 0x120e, 0x122d, 0x12f5, 0x20, 0x1206,
0x12ca, 0x121c, 0x120d, 0x1266, 0x122d, 0x1295, 0x1350, 0x122d, 0x12dd, 0x1232, 0x12f5, 0x1292,
0x12a0, 0x121d, 0x1235, 0x1270, 0x122d, 0x12f3, 0x121d, 0x12a0, 0x1295, 0x12f6, 0x122b, 0x12a0,
0x1235, 0x1275, 0x122b, 0x12ad, 0x1203, 0x1295, 0x12a0, 0x1274, 0x1295, 0x1235, 0x1264, 0x120d,
0x130d, 0x122c, 0x12f5, 0x1260, 0x122d, 0x120a, 0x1295, 0x1265, 0x122b, 0x1272, 0x1235, 0x120b,
0x126b, 0x1265, 0x122b, 0x1230, 0x120d, 0x1235, 0x1261, 0x12ab, 0x122c, 0x1235, 0x1275, 0x1261,
0x12f3, 0x1354, 0x1235, 0x1275, 0x1261, 0x1232, 0x1295, 0x1308, 0x1295, 0x127a, 0x1235, 0x1293,
0x12a1, 0x12ae, 0x1350, 0x1295, 0x1203, 0x1308, 0x1295, 0x12f0, 0x1265, 0x120a, 0x1295, 0x1302,
0x1265, 0x122b, 0x120d, 0x1273, 0x122d, 0x1309, 0x122d, 0x1290, 0x1232, 0x1204, 0x120d, 0x1232,
0x1295, 0x12aa, 0x12a0, 0x12ed, 0x1235, 0x120d, 0x20, 0x12a6, 0x134d, 0x20, 0x121b, 0x1295,
0x12a2, 0x1235, 0x1273, 0x1295, 0x1261, 0x120d, 0x1300, 0x122d, 0x1232, 0x12ab, 0x120a, 0x1292,
0x1295, 0x130d, 0x122b, 0x12f5, 0x12aa, 0x122e, 0x126d, 0x12aa, 0x12e8, 0x126d, 0x120a, 0x12dd,
0x1260, 0x1295, 0x120d, 0x1301, 0x1265, 0x120d, 0x1303, 0x1293, 0x1208, 0x1295, 0x12f0, 0x1295,
0x1209, 0x12ad, 0x1230, 0x121d, 0x1260, 0x122d, 0x130d, 0x121b, 0x12f5, 0x122a, 0x12f5, 0x121b,
0x120d, 0x1273, 0x121c, 0x122a, 0x1203, 0x121d, 0x1295, 0x121a, 0x1295, 0x1235, 0x12ad, 0x121e,
0x1293, 0x12ae, 0x121e, 0x1235, 0x12ae, 0x12a6, 0x1235, 0x120e, 0x1353, 0x122a, 0x1235, 0x1356,
0x12f5, 0x130e, 0x122a, 0x12ab, 0x1355, 0x122b, 0x130d, 0x122a, 0x130b, 0x122e, 0x121d, 0x1233,
0x121b, 0x122b, 0x1233, 0x1295, 0x20, 0x121b, 0x122a, 0x1296, 0x1233, 0x122a, 0x12e8, 0x126e,
0x1233, 0x122b, 0x1276, 0x126d, 0x1232, 0x121d, 0x1348, 0x122e, 0x1356, 0x120d, 0x1235, 0x12ae,
0x1355, 0x12e8, 0x1236, 0x134a, 0x12eb, 0x1235, 0x1276, 0x12ad, 0x1206, 0x120d, 0x121d, 0x1273,
0x120a, 0x1295, 0x1274, 0x122b, 0x1295, 0x12a1, 0x120d, 0x12eb, 0x1296, 0x126d, 0x1235, 0x12ad,
0x126b, 0x12f1, 0x12dd, 0x126b, 0x1272, 0x12ab, 0x1295, 0x126a, 0x12e8, 0x1293, 0x126a, 0x120a,
0x1290, 0x1235, 0x126e, 0x120d, 0x130e, 0x122b, 0x12f5, 0x12cb, 0x122d, 0x1236, 0x12cd, 0x12db,
0x130d, 0x122c, 0x1265, 0x12d9, 0x122a, 0x12ad, 0x12a0, 0x1295, 0x1273, 0x1293, 0x1293, 0x122a,
0x126e, 0x127b, 0x130e, 0x1235, 0x1308, 0x1293, 0x12ae, 0x12ae, 0x1235, 0x12ae, 0x121e, 0x122e,
0x12ac, 0x122d, 0x1309, 0x1208, 0x1295, 0x121b, 0x1204, 0x121b, 0x120d, 0x12f2, 0x1268, 0x121e,
0x122a, 0x123d, 0x12e8, 0x1235, 0x121b, 0x12ee, 0x1274, 0x122c, 0x12e9, 0x1292, 0x12e8, 0x1295,
0x12a0, 0x1352, 0x12a0, 0x12a6, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x1266, 0x130c, 0x12ed, 0x1295,
0x126a, 0x120d, 0x127b, 0x1273, 0x121d, 0x1279, 0x12ad, 0x134b, 0x1232, 0x12ab, 0x12a2, 0x134b,
0x1274, 0x134b, 0x12ab, 0x12a6, 0x134e, 0x134a, 0x1302, 0x1348, 0x1293, 0x1349, 0x1272, 0x130b,
0x120b, 0x1353, 0x130e, 0x1235, 0x130b, 0x121d, 0x1262, 0x12e8, 0x122d, 0x1309, 0x12cb, 0x12f3,
0x120d, 0x12ab, 0x1293, 0x120d, 0x1309, 0x12a0, 0x121d, 0x1206, 0x1296, 0x1209, 0x1209, 0x12a2,
0x1295, 0x12f0, 0x122d, 0x1260, 0x122a, 0x12aa, 0x122a, 0x1272, 0x121b, 0x1272, 0x12ae, 0x1235,
0x122c, 0x12ad, 0x12cb, 0x1303, 0x120a, 0x1295, 0x121b, 0x1301, 0x1229, 0x121b, 0x122d, 0x12ad,
0x12cc, 0x1233, 0x1235, 0x121a, 0x12f5, 0x12c8, 0x12ed, 0x1293, 0x12cd, 0x1229, 0x1292, 0x12cc,
0x1296, 0x122d, 0x134e, 0x120d, 0x12ad, 0x1293, 0x12a6, 0x121a, 0x12a0, 0x1353, 0x130e, 0x20,
0x1353, 0x130e, 0x1353, 0x120b, 0x12cd, 0x1352, 0x1275, 0x12a8, 0x12ed, 0x122d, 0x1295, 0x1356,
0x1205, 0x1295, 0x1354, 0x1356, 0x122d, 0x1275, 0x20, 0x121e, 0x122c, 0x1235, 0x1263, 0x12ed,
0x122b, 0x122e, 0x1276, 0x1295, 0x130b, 0x1234, 0x12ed, 0x1353, 0x1295, 0x1273, 0x1202, 0x1272,
0x1273, 0x122b, 0x12cb, 0x1276, 0x1295, 0x130b, 0x1273, 0x1351, 0x12cb, 0x12ac, 0x12cb, 0x120a,
0x1235, 0x623, 0x628, 0x64a, 0x62f, 0x62c, 0x627, 0x646, 0x623, 0x643, 0x631, 0x627,
0x623, 0x62f, 0x64a, 0x633, 0x20, 0x623, 0x628, 0x627, 0x628, 0x627, 0x627, 0x644,
0x62c, 0x632, 0x627, 0x626, 0x631, 0x623, 0x633, 0x645, 0x631, 0x629, 0x628, 0x627,
0x645, 0x627, 0x643, 0x648, 0x628, 0x627, 0x646, 0x63a, 0x648, 0x64a, 0x628, 0x627,
0x646, 0x62c, 0x648, 0x644, 0x628, 0x64a, 0x633, 0x627, 0x648, 0x628, 0x644, 0x627,
0x646, 0x62a, 0x627, 0x64a, 0x631, 0x628, 0x631, 0x627, 0x632, 0x627, 0x641, 0x64a,
0x644, 0x628, 0x648, 0x62c, 0x648, 0x645, 0x628, 0x648, 0x631, 0x627, 0x627, 0x644,
0x642, 0x627, 0x647, 0x631, 0x629, 0x627, 0x644, 0x62f, 0x627, 0x631, 0x20, 0x627,
0x644, 0x628, 0x64a, 0x636, 0x627, 0x621, 0x633, 0x64a, 0x62a, 0x627, 0x643, 0x648,
0x646, 0x627, 0x643, 0x631, 0x64a, 0x62f, 0x627, 0x643, 0x627, 0x631, 0x62f, 0x627,
0x631, 0x20, 0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x62c, 0x64a, 0x628, 0x648,
0x62a, 0x64a, 0x62f, 0x648, 0x627, 0x644, 0x627, 0x627, 0x644, 0x639, 0x64a, 0x648,
0x646, 0x641, 0x631, 0x64a, 0x20, 0x62a, 0x627, 0x648, 0x646, 0x63a, 0x627, 0x628,
0x648, 0x631, 0x648, 0x646, 0x647, 0x631, 0x627, 0x631, 0x64a, 0x62c, 0x648, 0x647,
0x627, 0x646, 0x633, 0x628, 0x631, 0x63a, 0x62c, 0x648, 0x628, 0x627, 0x643, 0x627,
0x645, 0x628, 0x627, 0x644, 0x627, 0x627, 0x644, 0x62e, 0x631, 0x637, 0x648, 0x645,
0x643, 0x64a, 0x63a, 0x627, 0x644, 0x64a, 0x643, 0x64a, 0x646, 0x634, 0x627, 0x633,
0x627, 0x644, 0x627, 0x63a, 0x648, 0x633, 0x644, 0x64a, 0x628, 0x631, 0x641, 0x64a,
0x644, 0x644, 0x648, 0x645, 0x64a, 0x644, 0x648, 0x627, 0x646, 0x62f, 0x627, 0x644,
0x648, 0x645, 0x628, 0x628, 0x627, 0x634, 0x627, 0x644, 0x648, 0x633, 0x627, 0x643,
0x627, 0x645, 0x627, 0x644, 0x627, 0x628, 0x648, 0x645, 0x627, 0x628, 0x648, 0x62a,
0x648, 0x645, 0x627, 0x633, 0x64a, 0x631, 0x648, 0x645, 0x628, 0x627, 0x628, 0x627,
0x646, 0x64a, 0x645, 0x642, 0x62f, 0x64a, 0x634, 0x64a, 0x648, 0x645, 0x648, 0x646,
0x631, 0x648, 0x641, 0x64a, 0x627, 0x646, 0x64a, 0x631, 0x648, 0x628, 0x64a, 0x646,
0x62c, 0x627, 0x645, 0x64a, 0x646, 0x627, 0x646, 0x64a, 0x627, 0x645, 0x64a, 0x646,
0x648, 0x627, 0x643, 0x634, 0x648, 0x637, 0x648, 0x627, 0x63a, 0x627, 0x62f, 0x648,
0x63a, 0x648, 0x628, 0x648, 0x631, 0x62a, 0x648, 0x20, 0x646, 0x648, 0x641, 0x648,
0x633, 0x627, 0x648, 0x20, 0x62a, 0x648, 0x645, 0x64a, 0x637, 0x631, 0x627, 0x628,
0x644, 0x633, 0x62a, 0x648, 0x646, 0x633, 0x648, 0x64a, 0x646, 0x62f, 0x647, 0x648,
0x643, 0x623, 0x62f, 0x627, 0x643, 0x623, 0x646, 0x634, 0x648, 0x631, 0x627, 0x62c,
0x623, 0x646, 0x63a, 0x648, 0x64a, 0x644, 0x627, 0x623, 0x646, 0x62a, 0x64a, 0x63a,
0x648, 0x627, 0x623, 0x631, 0x648, 0x62c, 0x648, 0x627, 0x646, 0x64a, 0x627, 0x628,
0x648, 0x64a, 0x646, 0x648, 0x633, 0x20, 0x623, 0x64a, 0x631, 0x633, 0x643, 0x627,
0x62a, 0x627, 0x645, 0x627, 0x631, 0x643, 0x627, 0x643, 0x648, 0x631, 0x62f, 0x648,
0x628, 0x627, 0x62c, 0x648, 0x62c, 0x648, 0x644, 0x627, 0x20, 0x631, 0x64a, 0x648,
0x62c, 0x627, 0x645, 0x64a, 0x646, 0x62f, 0x648, 0x632, 0x627, 0x631, 0x64a, 0x648,
0x20, 0x62c, 0x627, 0x644, 0x64a, 0x64a, 0x648, 0x633, 0x633, 0x627, 0x644, 0x637,
0x627, 0x633, 0x627, 0x646, 0x20, 0x62e, 0x648, 0x627, 0x646, 0x633, 0x627, 0x646,
0x20, 0x644, 0x648, 0x64a, 0x633, 0x62a, 0x627, 0x643, 0x645, 0x627, 0x646, 0x623,
0x634, 0x648, 0x627, 0x623, 0x631, 0x648, 0x628, 0x627, 0x623, 0x633, 0x648, 0x646,
0x633, 0x64a, 0x648, 0x646, 0x643, 0x648, 0x631, 0x627, 0x644, 0x20, 0x647, 0x627,
0x631, 0x628, 0x631, 0x628, 0x627, 0x647, 0x64a, 0x627, 0x628, 0x627, 0x647, 0x64a,
0x627, 0x20, 0x628, 0x627, 0x646, 0x62f, 0x64a, 0x631, 0x627, 0x633, 0x628, 0x631,
0x628, 0x627, 0x62f, 0x648, 0x633, 0x628, 0x644, 0x645, 0x628, 0x644, 0x64a, 0x632,
0x628, 0x644, 0x627, 0x646, 0x643, 0x2d, 0x633, 0x627, 0x628, 0x644, 0x648, 0x646,
0x628, 0x627, 0x648, 0x20, 0x641, 0x64a, 0x633, 0x62a, 0x627, 0x628, 0x648, 0x63a,
0x648, 0x62a, 0x627, 0x628, 0x648, 0x64a, 0x633, 0x643, 0x627, 0x645, 0x628, 0x631,
0x62f, 0x64a, 0x62c, 0x20, 0x628, 0x627, 0x64a, 0x643, 0x648, 0x645, 0x628, 0x648,
0x20, 0x62c, 0x631, 0x627, 0x646, 0x62f, 0x643, 0x627, 0x646, 0x643, 0x648, 0x646,
0x643, 0x627, 0x631, 0x627, 0x643, 0x627, 0x633, 0x643, 0x627, 0x64a, 0x64a, 0x646,
0x643, 0x627, 0x64a, 0x645, 0x627, 0x646, 0x634, 0x64a, 0x643, 0x627, 0x63a, 0x648,
0x62a, 0x634, 0x64a, 0x648, 0x627, 0x648, 0x627, 0x633, 0x64a, 0x648, 0x62f, 0x627,
0x62f, 0x20, 0x62e, 0x648, 0x627, 0x631, 0x64a, 0x632, 0x643, 0x648, 0x633, 0x62a,
0x627, 0x631, 0x64a, 0x643, 0x627, 0x643, 0x631, 0x64a, 0x633, 0x62a, 0x648, 0x646,
0x643, 0x64a, 0x627, 0x628, 0x627, 0x643, 0x648, 0x631, 0x627, 0x633, 0x627, 0x648,
0x62f, 0x627, 0x646, 0x645, 0x631, 0x643, 0x20, 0x634, 0x627, 0x641, 0x646, 0x62f,
0x627, 0x648, 0x633, 0x627, 0x646, 0x62f, 0x627, 0x648, 0x633, 0x646, 0x20, 0x643,
0x631, 0x64a, 0x643, 0x62f, 0x646, 0x641, 0x631, 0x62f, 0x64a, 0x62a, 0x631, 0x648,
0x64a, 0x62a, 0x62f, 0x648, 0x645, 0x64a, 0x646, 0x64a, 0x643, 0x627, 0x627, 0x64a,
0x62f, 0x645, 0x648, 0x646, 0x62a, 0x648, 0x646, 0x627, 0x64a, 0x631, 0x648, 0x646,
0x628, 0x64a, 0x627, 0x644, 0x633, 0x644, 0x641, 0x627, 0x62f, 0x648, 0x631, 0x641,
0x648, 0x631, 0x62a, 0x20, 0x646, 0x64a, 0x644, 0x633, 0x648, 0x646, 0x641, 0x648,
0x631, 0x62a, 0x627, 0x644, 0x64a, 0x632, 0x627, 0x62c, 0x644, 0x627, 0x633, 0x20,
0x628, 0x627, 0x64a, 0x62c, 0x648, 0x633, 0x20, 0x628, 0x627, 0x64a, 0x63a, 0x631,
0x627, 0x646, 0x62f, 0x20, 0x62a, 0x631, 0x643, 0x63a, 0x631, 0x64a, 0x646, 0x627,
0x62f, 0x627, 0x63a, 0x648, 0x627, 0x62f, 0x644, 0x648, 0x628, 0x63a, 0x648, 0x627,
0x62a, 0x64a, 0x645, 0x627, 0x644, 0x627, 0x63a, 0x648, 0x627, 0x64a, 0x627, 0x643,
0x648, 0x64a, 0x644, 0x63a, 0x64a, 0x627, 0x646, 0x627, 0x647, 0x627, 0x644, 0x64a,
0x641, 0x627, 0x643, 0x633, 0x647, 0x627, 0x641, 0x627, 0x646, 0x627, 0x647, 0x64a,
0x631, 0x645, 0x648, 0x633, 0x64a, 0x644, 0x648, 0x625, 0x646, 0x62f, 0x64a, 0x627,
0x646, 0x627, 0x628, 0x648, 0x644, 0x64a, 0x633, 0x643, 0x648, 0x646, 0x643, 0x633,
0x645, 0x627, 0x631, 0x646, 0x62c, 0x648, 0x628, 0x64a, 0x62a, 0x631, 0x633, 0x628,
0x631, 0x63a, 0x645, 0x62f, 0x64a, 0x646, 0x629, 0x20, 0x62a, 0x644, 0x60c, 0x20,
0x625, 0x646, 0x62f, 0x64a, 0x627, 0x646, 0x627, 0x641, 0x64a, 0x641, 0x627, 0x64a,
0x641, 0x64a, 0x646, 0x633, 0x64a, 0x646, 0x633, 0x648, 0x64a, 0x646, 0x627, 0x645,
0x627, 0x643, 0x627, 0x64a, 0x646, 0x648, 0x641, 0x64a, 0x643, 0x627, 0x643, 0x648,
0x64a, 0x644, 0x62a, 0x62c, 0x627, 0x645, 0x627, 0x64a, 0x643, 0x627, 0x62c, 0x648,
0x646, 0x64a, 0x644, 0x648, 0x64a, 0x633, 0x20, 0x641, 0x64a, 0x644, 0x645, 0x648,
0x646, 0x62a, 0x64a, 0x633, 0x64a, 0x644, 0x648, 0x643, 0x631, 0x627, 0x644, 0x646,
0x62f, 0x64a, 0x643, 0x644, 0x627, 0x20, 0x628, 0x627, 0x632, 0x644, 0x64a, 0x645,
0x627, 0x644, 0x648, 0x633, 0x20, 0x627, 0x646, 0x62c, 0x644, 0x648, 0x633, 0x62d,
0x64a, 0x20, 0x627, 0x644, 0x623, 0x645, 0x64a, 0x631, 0x20, 0x627, 0x644, 0x633,
0x641, 0x644, 0x64a, 0x645, 0x627, 0x634, 0x64a, 0x648, 0x645, 0x627, 0x646, 0x627,
0x63a, 0x648, 0x627, 0x645, 0x627, 0x646, 0x627, 0x648, 0x633, 0x645, 0x627, 0x631,
0x64a, 0x63a, 0x648, 0x62a, 0x627, 0x644, 0x645, 0x627, 0x631, 0x62a, 0x64a, 0x646,
0x64a, 0x643, 0x645, 0x627, 0x62a, 0x627, 0x645, 0x648, 0x631, 0x648, 0x633, 0x645,
0x627, 0x632, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x645, 0x64a, 0x646, 0x648, 0x645,
0x64a, 0x646, 0x64a, 0x645, 0x64a, 0x631, 0x64a, 0x62f, 0x627, 0x645, 0x64a, 0x62a,
0x644, 0x627, 0x643, 0x627, 0x62a, 0x644, 0x627, 0x645, 0x643, 0x633, 0x64a, 0x643,
0x648, 0x20, 0x633, 0x64a, 0x62a, 0x64a, 0x645, 0x643, 0x648, 0x64a, 0x644, 0x648,
0x646, 0x648, 0x64a, 0x646, 0x643, 0x62a, 0x648, 0x646, 0x645, 0x648, 0x646, 0x62a,
0x64a, 0x631, 0x64a, 0x645, 0x648, 0x646, 0x62a, 0x641, 0x64a, 0x62f, 0x64a, 0x648,
0x645, 0x648, 0x646, 0x62a, 0x633, 0x64a, 0x631, 0x627, 0x62a, 0x646, 0x627, 0x633,
0x648, 0x646, 0x64a, 0x648, 0x64a, 0x648, 0x631, 0x643, 0x646, 0x648, 0x631, 0x648,
0x646, 0x627, 0x647, 0x628, 0x64a, 0x648, 0x644, 0x627, 0x60c, 0x20, 0x62f, 0x627,
0x643, 0x648, 0x62a, 0x627, 0x20, 0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a,
0x629, 0x633, 0x646, 0x62a, 0x631, 0x646, 0x64a, 0x648, 0x20, 0x633, 0x627, 0x644,
0x64a, 0x645, 0x63a, 0x648, 0x62f, 0x62b, 0x627, 0x628, 0x623, 0x648, 0x62c, 0x64a,
0x646, 0x627, 0x62c, 0x627, 0x628, 0x646, 0x645, 0x627, 0x628, 0x627, 0x631, 0x627,
0x645, 0x627, 0x631, 0x64a, 0x628, 0x648, 0x641, 0x64a, 0x646, 0x643, 0x633, 0x628,
0x648, 0x631, 0x62a, 0x20, 0x623, 0x648, 0x20, 0x628, 0x631, 0x646, 0x633, 0x628,
0x648, 0x631, 0x62a, 0x20, 0x623, 0x648, 0x641, 0x20, 0x633, 0x628, 0x64a, 0x646,
0x628, 0x648, 0x631, 0x62a, 0x648, 0x20, 0x641, 0x64a, 0x644, 0x648, 0x628, 0x648,
0x631, 0x62a, 0x648, 0x631, 0x64a, 0x643, 0x648, 0x628, 0x648, 0x646, 0x62a, 0x627,
0x20, 0x623, 0x631, 0x64a, 0x646, 0x627, 0x632, 0x631, 0x627, 0x646, 0x643, 0x646,
0x20, 0x627, 0x646, 0x644, 0x62a, 0x631, 0x64a, 0x633, 0x64a, 0x641, 0x631, 0x64a,
0x62c, 0x64a, 0x646, 0x627, 0x631, 0x64a, 0x632, 0x648, 0x644, 0x648, 0x62a, 0x631,
0x64a, 0x648, 0x628, 0x631, 0x627, 0x646, 0x643, 0x648, 0x633, 0x627, 0x646, 0x62a,
0x627, 0x631, 0x64a, 0x645, 0x633, 0x627, 0x646, 0x62a, 0x64a, 0x627, 0x63a, 0x648,
0x633, 0x627, 0x646, 0x62a, 0x648, 0x20, 0x62f, 0x648, 0x645, 0x64a, 0x646, 0x63a,
0x648, 0x633, 0x627, 0x648, 0x20, 0x628, 0x627, 0x648, 0x644, 0x648, 0x633, 0x643,
0x648, 0x631, 0x633, 0x628, 0x64a, 0x633, 0x646, 0x62f, 0x633, 0x64a, 0x62a, 0x643,
0x627, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x628, 0x627, 0x631, 0x62a, 0x64a, 0x644,
0x64a, 0x645, 0x64a, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x62c, 0x648, 0x646, 0x633,
0x633, 0x627, 0x646, 0x62a, 0x20, 0x643, 0x64a, 0x62a, 0x633, 0x633, 0x627, 0x646,
0x62a, 0x20, 0x644, 0x648, 0x634, 0x64a, 0x627, 0x633, 0x627, 0x646, 0x62a, 0x20,
0x62a, 0x648, 0x645, 0x627, 0x633, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x641, 0x646,
0x633, 0x646, 0x62a, 0x633, 0x648, 0x641, 0x62a, 0x20, 0x643, 0x627, 0x631, 0x646,
0x62a, 0x62a, 0x64a, 0x63a, 0x648, 0x633, 0x64a, 0x63a, 0x627, 0x644, 0x628, 0x627,
0x62b, 0x64a, 0x644, 0x62a, 0x64a, 0x62e, 0x648, 0x627, 0x646, 0x627, 0x62a, 0x648,
0x631, 0x648, 0x646, 0x62a, 0x648, 0x62a, 0x648, 0x631, 0x62a, 0x648, 0x644, 0x627,
0x641, 0x627, 0x646, 0x643, 0x648, 0x641, 0x631, 0x648, 0x627, 0x64a, 0x62a, 0x20,
0x647, 0x648, 0x631, 0x633, 0x648, 0x64a, 0x646, 0x64a, 0x628, 0x64a, 0x62c, 0x64a,
0x627, 0x643, 0x648, 0x62a, 0x627, 0x62a, 0x643, 0x627, 0x633, 0x627, 0x64a, 0x62f,
0x627, 0x641, 0x64a, 0x632, 0x62f, 0x64a, 0x20, 0x645, 0x648, 0x646, 0x62a, 0x20,
0x62f, 0x648, 0x20, 0x631, 0x648, 0x641, 0x64a, 0x644, 0x645, 0x627, 0x643, 0x648,
0x627, 0x631, 0x64a, 0x645, 0x627, 0x648, 0x633, 0x648, 0x646, 0x645, 0x627, 0x643,
0x20, 0x645, 0x648, 0x631, 0x62f, 0x648, 0x628, 0x627, 0x644, 0x645, 0x64a, 0x631,
0x631, 0x648, 0x62b, 0x64a, 0x631, 0x627, 0x633, 0x627, 0x64a, 0x648, 0x648, 0x627,
0x62a, 0x631, 0x648, 0x644, 0x641, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x644, 0x648,
0x646, 0x62c, 0x64a, 0x631, 0x628, 0x64a, 0x646, 0x639, 0x62f, 0x646, 0x623, 0x644,
0x645, 0x627, 0x62a, 0x64a, 0x639, 0x645, 0x651, 0x627, 0x646, 0x623, 0x646, 0x62f,
0x64a, 0x631, 0x623, 0x643, 0x62a, 0x627, 0x648, 0x623, 0x643, 0x62a, 0x648, 0x628,
0x639, 0x634, 0x642, 0x20, 0x622, 0x628, 0x627, 0x62f, 0x623, 0x62a, 0x64a, 0x631,
0x627, 0x648, 0x628, 0x63a, 0x62f, 0x627, 0x62f, 0x627, 0x644, 0x628, 0x62d, 0x631,
0x64a, 0x646, 0x628, 0x627, 0x646, 0x643, 0x648, 0x643, 0x628, 0x627, 0x631, 0x646,
0x627, 0x648, 0x644, 0x628, 0x64a, 0x631, 0x648, 0x62a, 0x628, 0x634, 0x643, 0x64a,
0x643, 0x628, 0x631, 0x648, 0x646, 0x627, 0x64a, 0x62a, 0x634, 0x64a, 0x62a, 0x627,
0x62a, 0x634, 0x648, 0x628, 0x627, 0x644, 0x633, 0x627, 0x646, 0x643, 0x648, 0x644,
0x648, 0x645, 0x628, 0x648, 0x62f, 0x645, 0x634, 0x642, 0x62f, 0x64a, 0x644, 0x64a,
0x62f, 0x628, 0x64a, 0x62f, 0x648, 0x634, 0x627, 0x646, 0x628, 0x64a, 0x641, 0x627,
0x645, 0x627, 0x63a, 0x648, 0x633, 0x62a, 0x627, 0x63a, 0x632, 0x629, 0x647, 0x64a,
0x628, 0x631, 0x648, 0x646, 0x20, 0x28, 0x645, 0x62f, 0x64a, 0x646, 0x629, 0x20,
0x627, 0x644, 0x62e, 0x644, 0x64a, 0x644, 0x29, 0x645, 0x62f, 0x64a, 0x646, 0x629,
0x20, 0x647, 0x648, 0x20, 0x62a, 0x634, 0x64a, 0x20, 0x645, 0x646, 0x629, 0x647,
0x648, 0x646, 0x63a, 0x20, 0x643, 0x648, 0x646, 0x63a, 0x647, 0x648, 0x641, 0x62f,
0x627, 0x64a, 0x631, 0x643, 0x64a, 0x62a, 0x633, 0x643, 0x62c, 0x627, 0x643, 0x631,
0x62a, 0x627, 0x62c, 0x627, 0x64a, 0x627, 0x628, 0x64a, 0x648, 0x631, 0x627, 0x627,
0x644, 0x642, 0x62f, 0x633, 0x643, 0x627, 0x628, 0x648, 0x644, 0x643, 0x627, 0x645,
0x62a, 0x634, 0x627, 0x62a, 0x643, 0x627, 0x643, 0x631, 0x627, 0x62a, 0x634, 0x64a,
0x643, 0x627, 0x62a, 0x645, 0x627, 0x646, 0x62f, 0x648, 0x62e, 0x627, 0x646, 0x62f,
0x64a, 0x62c, 0x627, 0x643, 0x627, 0x644, 0x643, 0x62a, 0x627, 0x643, 0x631, 0x627,
0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x643, 0x643, 0x648, 0x627, 0x644,
0x627, 0x20, 0x644, 0x627, 0x645, 0x628, 0x648, 0x631, 0x643, 0x64a, 0x634, 0x64a,
0x646, 0x62c, 0x627, 0x644, 0x643, 0x648, 0x64a, 0x62a, 0x645, 0x627, 0x643, 0x627,
0x648, 0x645, 0x62c, 0x627, 0x62f, 0x646, 0x645, 0x627, 0x643, 0x633, 0x627, 0x631,
0x645, 0x627, 0x646, 0x64a, 0x644, 0x627, 0x645, 0x633, 0x642, 0x637, 0x646, 0x64a,
0x642, 0x648, 0x633, 0x64a, 0x627, 0x646, 0x648, 0x641, 0x648, 0x643, 0x648, 0x632,
0x646, 0x62a, 0x633, 0x643, 0x646, 0x648, 0x641, 0x648, 0x633, 0x628, 0x64a, 0x631,
0x633, 0x643, 0x623, 0x648, 0x645, 0x633, 0x643, 0x623, 0x648, 0x631, 0x627, 0x644,
0x628, 0x646, 0x648, 0x645, 0x20, 0x628, 0x646, 0x647, 0x628, 0x648, 0x646, 0x62a,
0x64a, 0x627, 0x646, 0x643, 0x628, 0x64a, 0x648, 0x646, 0x63a, 0x20, 0x64a, 0x627,
0x646, 0x63a, 0x642, 0x637, 0x631, 0x642, 0x648, 0x633, 0x62a, 0x627, 0x646, 0x627,
0x64a, 0x643, 0x64a, 0x632, 0x64a, 0x644, 0x648, 0x631, 0x62f, 0x627, 0x627, 0x644,
0x631, 0x64a, 0x627, 0x636, 0x633, 0x643, 0x627, 0x644, 0x64a, 0x646, 0x633, 0x645,
0x631, 0x642, 0x646, 0x62f, 0x633, 0x648, 0x644, 0x634, 0x646, 0x63a, 0x647, 0x627,
0x64a, 0x633, 0x646, 0x63a, 0x627, 0x641, 0x648, 0x631, 0x629, 0x633, 0x631, 0x64a,
0x62f, 0x646, 0x643, 0x648, 0x644, 0x64a, 0x645, 0x633, 0x643, 0x62a, 0x627, 0x64a,
0x628, 0x64a, 0x647, 0x637, 0x634, 0x642, 0x646, 0x62f, 0x62a, 0x628, 0x644, 0x64a,
0x633, 0x64a, 0x637, 0x647, 0x631, 0x627, 0x646, 0x62a, 0x64a, 0x645, 0x641, 0x648,
0x637, 0x648, 0x643, 0x64a, 0x648, 0x62a, 0x648, 0x645, 0x633, 0x643, 0x622, 0x644,
0x627, 0x646, 0x628, 0x627, 0x62a, 0x627, 0x631, 0x623, 0x631, 0x648, 0x645, 0x643,
0x64a, 0x623, 0x648, 0x633, 0x62a, 0x20, 0x646, 0x64a, 0x631, 0x627, 0x641, 0x64a,
0x627, 0x646, 0x62a, 0x64a, 0x627, 0x646, 0x641, 0x644, 0x627, 0x62f, 0x64a, 0x641,
0x648, 0x633, 0x62a, 0x643, 0x64a, 0x627, 0x643, 0x62a, 0x633, 0x643, 0x631, 0x627,
0x646, 0x63a, 0x648, 0x646, 0x64a, 0x643, 0x627, 0x62a, 0x631, 0x646, 0x628, 0x64a,
0x631, 0x62c, 0x64a, 0x631, 0x64a, 0x641, 0x627, 0x646, 0x623, 0x632, 0x648, 0x631,
0x633, 0x628, 0x631, 0x645, 0x648, 0x62f, 0x627, 0x643, 0x646, 0x627, 0x631, 0x64a,
0x627, 0x644, 0x631, 0x623, 0x633, 0x20, 0x627, 0x644, 0x623, 0x62e, 0x636, 0x631,
0x641, 0x627, 0x631, 0x648, 0x645, 0x627, 0x62f, 0x64a, 0x631, 0x627, 0x631, 0x64a,
0x643, 0x64a, 0x627, 0x641, 0x64a, 0x643, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627,
0x20, 0x627, 0x644, 0x62c, 0x646, 0x648, 0x628, 0x64a, 0x629, 0x633, 0x627, 0x646,
0x62a, 0x20, 0x647, 0x64a, 0x644, 0x64a, 0x646, 0x627, 0x627, 0x633, 0x62a, 0x627,
0x646, 0x644, 0x64a, 0x623, 0x62f, 0x64a, 0x644, 0x64a, 0x62f, 0x628, 0x631, 0x633,
0x64a, 0x628, 0x627, 0x646, 0x628, 0x631, 0x648, 0x643, 0x646, 0x20, 0x647, 0x64a,
0x644, 0x62f, 0x627, 0x631, 0x648, 0x646, 0x623, 0x648, 0x643, 0x644, 0x627, 0x647,
0x648, 0x628, 0x627, 0x631, 0x62a, 0x644, 0x64a, 0x646, 0x62f, 0x645, 0x627, 0x646,
0x644, 0x648, 0x631, 0x62f, 0x20, 0x647, 0x627, 0x648, 0x645, 0x64a, 0x644, 0x628,
0x648, 0x631, 0x646, 0x628, 0x631, 0x62b, 0x627, 0x633, 0x64a, 0x62f, 0x646, 0x64a,
0x623, 0x645, 0x633, 0x62a, 0x631, 0x62f, 0x627, 0x645, 0x623, 0x646, 0x62f, 0x648,
0x631, 0x627, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x62e, 0x627, 0x646, 0x623, 0x62b,
0x64a, 0x646, 0x627, 0x628, 0x644, 0x63a, 0x631, 0x627, 0x62f, 0x628, 0x631, 0x644,
0x64a, 0x646, 0x628, 0x631, 0x627, 0x62a, 0x64a, 0x633, 0x644, 0x627, 0x641, 0x627,
0x628, 0x631, 0x648, 0x643, 0x633, 0x644, 0x628, 0x648, 0x62e, 0x627, 0x631, 0x633,
0x62a, 0x628, 0x648, 0x62f, 0x627, 0x628, 0x633, 0x62a, 0x628, 0x648, 0x633, 0x646,
0x63a, 0x646, 0x62a, 0x634, 0x64a, 0x633, 0x64a, 0x646, 0x627, 0x648, 0x643, 0x648,
0x628, 0x646, 0x647, 0x627, 0x63a, 0x646, 0x62f, 0x628, 0x644, 0x646, 0x62c, 0x628,
0x644, 0x20, 0x637, 0x627, 0x631, 0x642, 0x63a, 0x64a, 0x631, 0x646, 0x632, 0x64a,
0x647, 0x644, 0x633, 0x646, 0x643, 0x64a, 0x62c, 0x632, 0x64a, 0x631, 0x629, 0x20,
0x645, 0x627, 0x646, 0x625, 0x633, 0x637, 0x646, 0x628, 0x648, 0x644, 0x62c, 0x64a,
0x631, 0x633, 0x64a, 0x643, 0x627, 0x644, 0x64a, 0x646, 0x62c, 0x631, 0x627, 0x62f,
0x643, 0x64a, 0x631, 0x648, 0x641, 0x643, 0x64a, 0x64a, 0x641, 0x644, 0x634, 0x628,
0x648, 0x646, 0x629, 0x644, 0x64a, 0x648, 0x628, 0x644, 0x64a, 0x627, 0x646, 0x627,
0x644, 0x646, 0x62f, 0x646, 0x644, 0x648, 0x643, 0x633, 0x645, 0x628, 0x648, 0x631,
0x63a, 0x645, 0x62f, 0x631, 0x64a, 0x62f, 0x645, 0x627, 0x644, 0x637, 0x629, 0x645,
0x627, 0x631, 0x64a, 0x647, 0x627, 0x645, 0x646, 0x645, 0x64a, 0x646, 0x633, 0x643,
0x645, 0x648, 0x646, 0x627, 0x643, 0x648, 0x645, 0x648, 0x633, 0x643, 0x648, 0x623,
0x648, 0x633, 0x644, 0x648, 0x628, 0x627, 0x631, 0x64a, 0x633, 0x628, 0x648, 0x62f,
0x63a, 0x648, 0x631, 0x64a, 0x643, 0x627, 0x628, 0x631, 0x627, 0x63a, 0x631, 0x64a,
0x63a, 0x627, 0x633, 0x645, 0x631, 0x627, 0x621, 0x633, 0x627, 0x646, 0x20, 0x645,
0x627, 0x631, 0x64a, 0x646, 0x648, 0x633, 0x631, 0x627, 0x64a, 0x64a, 0x641, 0x648,
0x633, 0x627, 0x631, 0x627, 0x62a, 0x648, 0x641, 0x633, 0x64a, 0x645, 0x641, 0x631,
0x648, 0x628, 0x648, 0x644, 0x633, 0x643, 0x648, 0x628, 0x64a, 0x635, 0x648, 0x641,
0x64a, 0x627, 0x633, 0x62a, 0x648, 0x643, 0x647, 0x648, 0x644, 0x645, 0x62a, 0x627,
0x644, 0x64a, 0x646, 0x62a, 0x64a, 0x631, 0x627, 0x646, 0x627, 0x623, 0x648, 0x644,
0x64a, 0x627, 0x646, 0x648, 0x641, 0x633, 0x643, 0x641, 0x627, 0x62f, 0x648, 0x632,
0x627, 0x644, 0x641, 0x627, 0x62a, 0x64a, 0x643, 0x627, 0x646, 0x641, 0x64a, 0x64a,
0x646, 0x627, 0x641, 0x64a, 0x644, 0x646, 0x64a, 0x648, 0x633, 0x641, 0x648, 0x644,
0x648, 0x62c, 0x631, 0x627, 0x62f, 0x648, 0x627, 0x631, 0x633, 0x648, 0x632, 0x63a,
0x631, 0x628, 0x632, 0x64a, 0x648, 0x631, 0x62e, 0x623, 0x646, 0x62a, 0x627, 0x646,
0x627, 0x646, 0x627, 0x631, 0x64a, 0x641, 0x648, 0x62a, 0x634, 0x627, 0x63a, 0x648,
0x633, 0x643, 0x631, 0x64a, 0x633, 0x645, 0x627, 0x633, 0x643, 0x648, 0x643, 0x648,
0x633, 0x62c, 0x632, 0x631, 0x20, 0x627, 0x644, 0x642, 0x645, 0x631, 0x643, 0x64a,
0x631, 0x63a, 0x648, 0x64a, 0x644, 0x64a, 0x646, 0x645, 0x627, 0x647, 0x64a, 0x627,
0x644, 0x645, 0x627, 0x644, 0x62f, 0x64a, 0x641, 0x645, 0x648, 0x631, 0x64a, 0x634,
0x64a, 0x648, 0x633, 0x645, 0x627, 0x64a, 0x648, 0x62a, 0x631, 0x64a, 0x648, 0x646,
0x64a, 0x648, 0x646, 0x623, 0x628, 0x64a, 0x627, 0x623, 0x648, 0x643, 0x644, 0x627,
0x646, 0x62f, 0x628, 0x648, 0x63a, 0x627, 0x646, 0x641, 0x64a, 0x644, 0x62a, 0x634,
0x627, 0x62b, 0x627, 0x645, 0x627, 0x633, 0x62a, 0x631, 0x625, 0x64a, 0x641, 0x627,
0x62a, 0x641, 0x627, 0x643, 0x627, 0x648, 0x641, 0x648, 0x641, 0x64a, 0x62c, 0x64a,
0x641, 0x648, 0x646, 0x627, 0x641, 0x648, 0x62a, 0x64a, 0x62c, 0x644, 0x627, 0x628,
0x627, 0x62c, 0x648, 0x633, 0x62c, 0x627, 0x645, 0x628, 0x64a, 0x631, 0x63a, 0x648,
0x627, 0x62f, 0x627, 0x644, 0x643, 0x627, 0x646, 0x627, 0x644, 0x647, 0x648, 0x646,
0x648, 0x644, 0x648, 0x644, 0x648, 0x627, 0x646, 0x62f, 0x631, 0x628, 0x64a, 0x631,
0x62c, 0x643, 0x64a, 0x631, 0x64a, 0x62a, 0x64a, 0x20, 0x645, 0x627, 0x62a, 0x64a,
0x643, 0x648, 0x633, 0x631, 0x627, 0x643, 0x648, 0x627, 0x62c, 0x627, 0x644, 0x64a,
0x646, 0x645, 0x627, 0x62c, 0x648, 0x631, 0x648, 0x645, 0x627, 0x631, 0x643, 0x64a,
0x633, 0x627, 0x633, 0x645, 0x64a, 0x62f, 0x648, 0x627, 0x64a, 0x646, 0x627, 0x648,
0x631, 0x648, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x646, 0x648, 0x645,
0x64a, 0x627, 0x628, 0x627, 0x63a, 0x648, 0x20, 0x628, 0x627, 0x63a, 0x648, 0x628,
0x627, 0x644, 0x627, 0x648, 0x628, 0x64a, 0x62a, 0x643, 0x64a, 0x631, 0x646, 0x628,
0x627, 0x646, 0x627, 0x628, 0x628, 0x648, 0x631, 0x20, 0x645, 0x648, 0x631, 0x633,
0x628, 0x64a, 0x631, 0x627, 0x631, 0x648, 0x62a, 0x648, 0x646, 0x63a, 0x627, 0x633,
0x627, 0x64a, 0x628, 0x627, 0x646, 0x62a, 0x627, 0x647, 0x64a, 0x62a, 0x64a, 0x62a,
0x627, 0x631, 0x627, 0x648, 0x627, 0x62a, 0x648, 0x646, 0x63a, 0x627, 0x62a, 0x627,
0x628, 0x648, 0x648, 0x627, 0x644, 0x64a, 0x633, 0x531, 0x562, 0x56b, 0x57b, 0x561,
0x576, 0x531, 0x584, 0x580, 0x561, 0x531, 0x564, 0x56b, 0x57d, 0x20, 0x531, 0x562,
0x565, 0x562, 0x561, 0x531, 0x56c, 0x56a, 0x56b, 0x580, 0x531, 0x57d, 0x574, 0x565,
0x580, 0x561, 0x532, 0x561, 0x574, 0x561, 0x56f, 0x578, 0x532, 0x561, 0x576, 0x563,
0x56b, 0x532, 0x561, 0x576, 0x56a, 0x578, 0x582, 0x56c, 0x532, 0x56b, 0x57d, 0x561,
0x578, 0x582, 0x532, 0x56c, 0x561, 0x576, 0x569, 0x561, 0x575, 0x580, 0x532, 0x580,
0x561, 0x566, 0x561, 0x57e, 0x56b, 0x56c, 0x532, 0x578, 0x582, 0x56a, 0x578, 0x582,
0x574, 0x562, 0x578, 0x582, 0x580, 0x561, 0x53f, 0x561, 0x570, 0x56b, 0x580, 0x565,
0x53f, 0x561, 0x57d, 0x561, 0x562, 0x56c, 0x561, 0x576, 0x56f, 0x561, 0x54d, 0x565,
0x578, 0x582, 0x57f, 0x561, 0x53f, 0x578, 0x576, 0x561, 0x56f, 0x580, 0x56b, 0x534,
0x561, 0x584, 0x561, 0x580, 0x534, 0x561, 0x580, 0x2d, 0x537, 0x57d, 0x2d, 0x54d,
0x561, 0x56c, 0x561, 0x574, 0x54b, 0x56b, 0x562, 0x578, 0x582, 0x569, 0x56b, 0x534,
0x578, 0x582, 0x561, 0x56c, 0x561, 0x537, 0x56c, 0x20, 0x531, 0x575, 0x578, 0x582,
0x576, 0x556, 0x580, 0x56b, 0x569, 0x561, 0x578, 0x582, 0x576, 0x533, 0x561, 0x562,
0x578, 0x580, 0x578, 0x576, 0x565, 0x540, 0x561, 0x580, 0x561, 0x580, 0x565, 0x545,
0x578, 0x570, 0x561, 0x576, 0x565, 0x57d, 0x562, 0x578, 0x582, 0x580, 0x563, 0x54b,
0x578, 0x582, 0x562, 0x561, 0x53f, 0x561, 0x574, 0x57a, 0x561, 0x56c, 0x561, 0x53d,
0x561, 0x580, 0x569, 0x578, 0x582, 0x574, 0x53f, 0x56b, 0x563, 0x561, 0x56c, 0x56b,
0x53f, 0x56b, 0x576, 0x577, 0x561, 0x57d, 0x561, 0x53c, 0x561, 0x563, 0x578, 0x57d,
0x53c, 0x56b, 0x562, 0x580, 0x587, 0x56b, 0x56c, 0x53c, 0x578, 0x574, 0x565, 0x53c,
0x578, 0x582, 0x561, 0x576, 0x564, 0x561, 0x53c, 0x578, 0x582, 0x562, 0x578, 0x582,
0x574, 0x562, 0x561, 0x577, 0x56b, 0x53c, 0x578, 0x582, 0x57d, 0x561, 0x56f, 0x561,
0x544, 0x561, 0x56c, 0x561, 0x562, 0x578, 0x544, 0x561, 0x57a, 0x578, 0x582, 0x57f,
0x578, 0x582, 0x544, 0x561, 0x57d, 0x565, 0x580, 0x578, 0x582, 0x544, 0x562, 0x561,
0x562, 0x561, 0x576, 0x565, 0x544, 0x578, 0x563, 0x561, 0x564, 0x56b, 0x577, 0x578,
0x544, 0x578, 0x576, 0x580, 0x578, 0x57e, 0x56b, 0x561, 0x546, 0x561, 0x575, 0x580,
0x578, 0x562, 0x56b, 0x546, 0x57b, 0x561, 0x574, 0x565, 0x576, 0x561, 0x546, 0x56b,
0x561, 0x574, 0x565, 0x575, 0x546, 0x578, 0x582, 0x561, 0x56f, 0x577, 0x578, 0x57f,
0x548, 0x582, 0x561, 0x563, 0x561, 0x564, 0x578, 0x582, 0x563, 0x578, 0x582, 0x54a,
0x578, 0x580, 0x57f, 0x578, 0x20, 0x546, 0x578, 0x57e, 0x578, 0x54d, 0x561, 0x576,
0x20, 0x54f, 0x578, 0x574, 0x565, 0x54f, 0x580, 0x56b, 0x57a, 0x578, 0x56c, 0x56b,
0x539, 0x578, 0x582, 0x576, 0x56b, 0x57d, 0x54e, 0x56b, 0x576, 0x564, 0x570, 0x578,
0x582, 0x56f, 0x531, 0x564, 0x561, 0x56f, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x531,
0x576, 0x584, 0x578, 0x580, 0x56b, 0x57b, 0x531, 0x576, 0x563, 0x578, 0x582, 0x56b,
0x56c, 0x561, 0x531, 0x576, 0x57f, 0x56b, 0x563, 0x578, 0x582, 0x561, 0x531, 0x580,
0x561, 0x563, 0x578, 0x582, 0x561, 0x56b, 0x576, 0x561, 0x532, 0x578, 0x582, 0x565,
0x576, 0x578, 0x57d, 0x20, 0x531, 0x575, 0x580, 0x565, 0x57d, 0x53f, 0x561, 0x57f,
0x561, 0x574, 0x561, 0x580, 0x56f, 0x561, 0x53f, 0x578, 0x580, 0x564, 0x578, 0x57e,
0x561, 0x53a, 0x578, 0x582, 0x56a, 0x578, 0x582, 0x575, 0x53c, 0x561, 0x20, 0x54c,
0x56b, 0x578, 0x56d, 0x561, 0x544, 0x565, 0x576, 0x564, 0x578, 0x57d, 0x561, 0x54c,
0x56b, 0x578, 0x20, 0x533, 0x561, 0x56c, 0x575, 0x565, 0x563, 0x578, 0x57d, 0x54d,
0x561, 0x56c, 0x57f, 0x561, 0x54d, 0x561, 0x576, 0x20, 0x53d, 0x578, 0x582, 0x561,
0x576, 0x54d, 0x561, 0x576, 0x20, 0x53c, 0x578, 0x582, 0x56b, 0x57d, 0x54f, 0x578,
0x582, 0x56f, 0x578, 0x582, 0x574, 0x561, 0x576, 0x548, 0x582, 0x577, 0x578, 0x582,
0x561, 0x575, 0x561, 0x531, 0x580, 0x578, 0x582, 0x562, 0x561, 0x531, 0x57d, 0x578,
0x582, 0x576, 0x57d, 0x575, 0x578, 0x576, 0x531, 0x57f, 0x56b, 0x56f, 0x578, 0x56f,
0x561, 0x576, 0x532, 0x561, 0x56b, 0x575, 0x561, 0x532, 0x561, 0x56b, 0x575, 0x561,
0x20, 0x532, 0x561, 0x576, 0x564, 0x565, 0x580, 0x561, 0x57d, 0x532, 0x561, 0x580,
0x562, 0x561, 0x564, 0x578, 0x57d, 0x532, 0x565, 0x56c, 0x565, 0x574, 0x532, 0x565,
0x56c, 0x56b, 0x566, 0x532, 0x56c, 0x561, 0x576, 0x56f, 0x2d, 0x54d, 0x561, 0x562,
0x56c, 0x578, 0x576, 0x532, 0x578, 0x561, 0x20, 0x54e, 0x56b, 0x57d, 0x57f, 0x561,
0x532, 0x578, 0x563, 0x578, 0x57f, 0x561, 0x532, 0x578, 0x575, 0x57d, 0x565, 0x554,
0x565, 0x574, 0x562, 0x580, 0x56b, 0x57b, 0x20, 0x532, 0x565, 0x575, 0x53f, 0x561,
0x574, 0x57a, 0x578, 0x582, 0x20, 0x533, 0x580, 0x561, 0x576, 0x564, 0x56b, 0x53f,
0x561, 0x576, 0x56f, 0x578, 0x582, 0x576, 0x53f, 0x561, 0x580, 0x561, 0x56f, 0x561,
0x57d, 0x53f, 0x561, 0x575, 0x565, 0x576, 0x53f, 0x561, 0x575, 0x574, 0x561, 0x576,
0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x549, 0x56b, 0x56f, 0x561,
0x563, 0x578, 0x549, 0x56b, 0x578, 0x582, 0x561, 0x578, 0x582, 0x561, 0x54d, 0x575,
0x578, 0x582, 0x564, 0x561, 0x564, 0x20, 0x53d, 0x578, 0x582, 0x561, 0x580, 0x565,
0x57d, 0x53f, 0x578, 0x57d, 0x57f, 0x561, 0x20, 0x54c, 0x56b, 0x56f, 0x561, 0x554,
0x580, 0x565, 0x57d, 0x569, 0x578, 0x576, 0x53f, 0x578, 0x582, 0x575, 0x561, 0x562,
0x561, 0x53f, 0x575, 0x578, 0x582, 0x580, 0x561, 0x57d, 0x561, 0x578, 0x534, 0x565,
0x576, 0x574, 0x561, 0x580, 0x584, 0x57d, 0x570, 0x561, 0x57e, 0x576, 0x534, 0x578,
0x578, 0x582, 0x57d, 0x578, 0x576, 0x534, 0x578, 0x578, 0x582, 0x57d, 0x578, 0x576,
0x20, 0x554, 0x580, 0x56b, 0x584, 0x534, 0x565, 0x576, 0x57e, 0x565, 0x580, 0x534,
0x565, 0x57f, 0x580, 0x578, 0x575, 0x569, 0x534, 0x578, 0x574, 0x56b, 0x576, 0x56b,
0x56f, 0x561, 0x537, 0x564, 0x574, 0x578, 0x576, 0x57f, 0x578, 0x576, 0x537, 0x575,
0x580, 0x578, 0x582, 0x576, 0x565, 0x57a, 0x565, 0x54d, 0x561, 0x56c, 0x57e, 0x561,
0x564, 0x578, 0x580, 0x556, 0x578, 0x580, 0x57f, 0x20, 0x546, 0x565, 0x56c, 0x57d,
0x578, 0x576, 0x556, 0x578, 0x580, 0x57f, 0x561, 0x56c, 0x565, 0x566, 0x561, 0x533,
0x56c, 0x565, 0x575, 0x57d, 0x20, 0x532, 0x565, 0x575, 0x533, 0x578, 0x582, 0x57d,
0x20, 0x532, 0x565, 0x575, 0x533, 0x580, 0x561, 0x576, 0x564, 0x20, 0x54f, 0x575,
0x578, 0x580, 0x584, 0x533, 0x580, 0x565, 0x576, 0x561, 0x564, 0x561, 0x533, 0x57e,
0x561, 0x564, 0x565, 0x56c, 0x578, 0x582, 0x57a, 0x561, 0x533, 0x57e, 0x561, 0x57f,
0x565, 0x574, 0x561, 0x56c, 0x561, 0x533, 0x578, 0x582, 0x561, 0x575, 0x561, 0x56f,
0x56b, 0x56c, 0x533, 0x561, 0x575, 0x561, 0x576, 0x561, 0x540, 0x561, 0x56c, 0x56b,
0x586, 0x561, 0x584, 0x57d, 0x540, 0x561, 0x57e, 0x561, 0x576, 0x561, 0x537, 0x580,
0x574, 0x578, 0x57d, 0x56b, 0x56c, 0x575, 0x578, 0x53b, 0x576, 0x564, 0x56b, 0x561,
0x576, 0x561, 0x57a, 0x578, 0x56c, 0x56b, 0x57d, 0x546, 0x578, 0x584, 0x57d, 0x2c,
0x20, 0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576, 0x561, 0x544, 0x561, 0x580, 0x565,
0x576, 0x563, 0x578, 0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576, 0x561,
0x54a, 0x565, 0x57f, 0x565, 0x580, 0x57d, 0x562, 0x578, 0x582, 0x580, 0x563, 0x2c,
0x20, 0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576, 0x561, 0x539, 0x565, 0x56c, 0x20,
0x54d, 0x56b, 0x569, 0x56b, 0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576,
0x561, 0x54e, 0x56b, 0x57e, 0x565, 0x575, 0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b,
0x561, 0x576, 0x561, 0x54e, 0x56b, 0x576, 0x57d, 0x565, 0x576, 0x57d, 0x2c, 0x20,
0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576, 0x561, 0x54e, 0x56b, 0x576, 0x561, 0x574,
0x561, 0x56f, 0x2c, 0x20, 0x53b, 0x576, 0x564, 0x56b, 0x561, 0x576, 0x561, 0x53b,
0x576, 0x578, 0x582, 0x57e, 0x56b, 0x56f, 0x53b, 0x56f, 0x561, 0x56c, 0x578, 0x582,
0x56b, 0x57f, 0x543, 0x561, 0x574, 0x561, 0x575, 0x56f, 0x561, 0x54b, 0x578, 0x582,
0x576, 0x578, 0x53c, 0x578, 0x582, 0x56b, 0x57d, 0x57e, 0x56b, 0x56c, 0x544, 0x578,
0x576, 0x57f, 0x56b, 0x57d, 0x565, 0x56c, 0x578, 0x2c, 0x20, 0x53f, 0x565, 0x576,
0x57f, 0x578, 0x582, 0x56f, 0x56b, 0x53f, 0x580, 0x561, 0x56c, 0x565, 0x576, 0x564,
0x565, 0x575, 0x56f, 0x53c, 0x561, 0x20, 0x54a, 0x561, 0x57d, 0x53c, 0x56b, 0x574,
0x561, 0x53c, 0x578, 0x57d, 0x20, 0x531, 0x576, 0x57b, 0x565, 0x56c, 0x565, 0x57d,
0x53c, 0x578, 0x578, 0x582, 0x565, 0x580, 0x20, 0x54a, 0x580, 0x56b, 0x576, 0x57d,
0x565, 0x57d, 0x20, 0x554, 0x57e, 0x578, 0x580, 0x569, 0x565, 0x580, 0x544, 0x561,
0x57d, 0x565, 0x575, 0x578, 0x544, 0x561, 0x576, 0x561, 0x563, 0x578, 0x582, 0x561,
0x544, 0x561, 0x576, 0x561, 0x578, 0x582, 0x57d, 0x544, 0x561, 0x580, 0x56b, 0x563,
0x578, 0x544, 0x561, 0x580, 0x57f, 0x56b, 0x576, 0x56b, 0x56f, 0x561, 0x544, 0x561,
0x57f, 0x561, 0x574, 0x578, 0x580, 0x578, 0x57d, 0x544, 0x561, 0x566, 0x561, 0x569,
0x56c, 0x561, 0x576, 0x544, 0x565, 0x576, 0x578, 0x574, 0x56b, 0x576, 0x56b, 0x544,
0x565, 0x580, 0x56b, 0x564, 0x561, 0x544, 0x565, 0x57f, 0x56c, 0x561, 0x56f, 0x561,
0x57f, 0x56c, 0x561, 0x544, 0x565, 0x56d, 0x56b, 0x56f, 0x578, 0x544, 0x56b, 0x584,
0x565, 0x56c, 0x578, 0x576, 0x544, 0x578, 0x576, 0x56f, 0x57f, 0x578, 0x576, 0x544,
0x578, 0x576, 0x57f, 0x565, 0x57c, 0x565, 0x575, 0x544, 0x578, 0x576, 0x57f, 0x565,
0x57e, 0x56b, 0x564, 0x565, 0x578, 0x544, 0x578, 0x576, 0x57d, 0x565, 0x57c, 0x561,
0x57f, 0x546, 0x561, 0x57d, 0x561, 0x578, 0x582, 0x546, 0x575, 0x578, 0x582, 0x20,
0x545, 0x578, 0x580, 0x584, 0x546, 0x578, 0x574, 0x546, 0x578, 0x580, 0x578, 0x576,
0x575, 0x561, 0x532, 0x578, 0x575, 0x56c, 0x561, 0x2c, 0x20, 0x540, 0x575, 0x578,
0x582, 0x57d, 0x56b, 0x57d, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x534, 0x561, 0x56f,
0x578, 0x57f, 0x561, 0x540, 0x575, 0x578, 0x582, 0x57d, 0x56b, 0x57d, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x534, 0x561, 0x56f, 0x578, 0x57f, 0x561, 0x20, 0x2d, 0x20,
0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x546, 0x575, 0x578, 0x582, 0x20,
0x54d, 0x561, 0x56c, 0x565, 0x574, 0x2c, 0x20, 0x540, 0x575, 0x578, 0x582, 0x57d,
0x56b, 0x57d, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x534, 0x561, 0x56f, 0x578, 0x57f,
0x561, 0x546, 0x578, 0x582, 0x578, 0x582, 0x56f, 0x555, 0x56d, 0x56b, 0x576, 0x561,
0x563, 0x561, 0x54a, 0x561, 0x576, 0x561, 0x574, 0x561, 0x54a, 0x561, 0x580, 0x561,
0x574, 0x561, 0x580, 0x56b, 0x562, 0x578, 0x556, 0x56b, 0x576, 0x56b, 0x584, 0x57d,
0x54a, 0x578, 0x580, 0x57f, 0x2d, 0x585, 0x2d, 0x54a, 0x580, 0x565, 0x576, 0x57d,
0x54a, 0x578, 0x580, 0x57f, 0x2d, 0x585, 0x586, 0x2d, 0x54d, 0x583, 0x565, 0x575,
0x576, 0x54a, 0x578, 0x580, 0x57f, 0x578, 0x582, 0x20, 0x54e, 0x565, 0x56c, 0x575,
0x578, 0x582, 0x54a, 0x578, 0x582, 0x565, 0x580, 0x57f, 0x578, 0x20, 0x54c, 0x56b,
0x56f, 0x578, 0x54a, 0x578, 0x582, 0x576, 0x57f, 0x561, 0x20, 0x531, 0x580, 0x565,
0x576, 0x561, 0x57d, 0x54c, 0x561, 0x576, 0x56f, 0x56b, 0x576, 0x20, 0x53b, 0x576,
0x56c, 0x565, 0x569, 0x54c, 0x565, 0x57d, 0x56b, 0x586, 0x56b, 0x54c, 0x565, 0x57b,
0x561, 0x575, 0x576, 0x561, 0x54c, 0x565, 0x566, 0x578, 0x56c, 0x575, 0x578, 0x582,
0x57f, 0x54c, 0x56b, 0x578, 0x582, 0x20, 0x532, 0x580, 0x561, 0x576, 0x56f, 0x578,
0x582, 0x54d, 0x561, 0x576, 0x57f, 0x561, 0x580, 0x565, 0x574, 0x54d, 0x561, 0x576,
0x57f, 0x575, 0x561, 0x563, 0x578, 0x54d, 0x561, 0x576, 0x57f, 0x578, 0x20, 0x534,
0x578, 0x574, 0x56b, 0x576, 0x563, 0x578, 0x54d, 0x561, 0x576, 0x20, 0x54a, 0x561,
0x578, 0x582, 0x56c, 0x578, 0x582, 0x54d, 0x56f, 0x578, 0x580, 0x57d, 0x562, 0x56b,
0x57d, 0x578, 0x582, 0x576, 0x54d, 0x56b, 0x57f, 0x56f, 0x561, 0x54d, 0x565, 0x576,
0x20, 0x532, 0x561, 0x580, 0x57f, 0x565, 0x56c, 0x574, 0x56b, 0x54d, 0x565, 0x576,
0x569, 0x20, 0x54b, 0x578, 0x576, 0x57d, 0x54d, 0x565, 0x576, 0x569, 0x20, 0x554,
0x56b, 0x57f, 0x57d, 0x54d, 0x565, 0x576, 0x569, 0x20, 0x53c, 0x575, 0x578, 0x582,
0x57d, 0x56b, 0x561, 0x54d, 0x565, 0x576, 0x569, 0x20, 0x539, 0x578, 0x574, 0x561,
0x57d, 0x54d, 0x565, 0x576, 0x569, 0x20, 0x54e, 0x56b, 0x576, 0x57d, 0x565, 0x576,
0x569, 0x54d, 0x57e, 0x56b, 0x586, 0x569, 0x20, 0x554, 0x561, 0x580, 0x565, 0x576,
0x569, 0x54f, 0x565, 0x563, 0x578, 0x582, 0x57d, 0x56b, 0x563, 0x561, 0x56c, 0x57a,
0x561, 0x54f, 0x578, 0x582, 0x56c, 0x565, 0x54f, 0x56b, 0x56d, 0x578, 0x582, 0x561,
0x576, 0x561, 0x54f, 0x578, 0x580, 0x578, 0x576, 0x57f, 0x578, 0x539, 0x578, 0x580,
0x569, 0x578, 0x56c, 0x561, 0x54e, 0x561, 0x576, 0x56f, 0x578, 0x582, 0x57e, 0x565,
0x580, 0x548, 0x582, 0x561, 0x575, 0x569, 0x570, 0x578, 0x580, 0x57d, 0x54e, 0x56b,
0x576, 0x56b, 0x57a, 0x565, 0x563, 0x545, 0x561, 0x56f, 0x578, 0x582, 0x57f, 0x561,
0x57f, 0x554, 0x565, 0x575, 0x57d, 0x56b, 0x534, 0x565, 0x575, 0x57e, 0x56b, 0x57d,
0x534, 0x575, 0x578, 0x582, 0x574, 0x578, 0x576, 0x20, 0x564, 0x2019, 0x545, 0x578,
0x582, 0x580, 0x57e, 0x56b, 0x56c, 0x544, 0x561, 0x56f, 0x56f, 0x578, 0x582, 0x578,
0x580, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x544, 0x578, 0x578, 0x582, 0x57d,
0x578, 0x576, 0x544, 0x561, 0x56f, 0x2d, 0x544, 0x565, 0x580, 0x564, 0x578, 0x54a,
0x561, 0x56c, 0x574, 0x565, 0x580, 0x54c, 0x578, 0x57f, 0x565, 0x580, 0x561, 0x54d,
0x575, 0x578, 0x57e, 0x561, 0x54f, 0x580, 0x578, 0x56c, 0x56c, 0x54e, 0x578, 0x57d,
0x57f, 0x578, 0x56f, 0x53c, 0x578, 0x576, 0x563, 0x575, 0x56b, 0x580, 0x531, 0x564,
0x565, 0x576, 0x531, 0x56c, 0x574, 0x561, 0x569, 0x56b, 0x531, 0x574, 0x574, 0x561,
0x576, 0x531, 0x576, 0x561, 0x564, 0x56b, 0x580, 0x531, 0x56f, 0x57f, 0x561, 0x578,
0x582, 0x531, 0x56f, 0x57f, 0x578, 0x562, 0x565, 0x531, 0x577, 0x56d, 0x561, 0x562,
0x561, 0x564, 0x531, 0x57f, 0x56b, 0x580, 0x561, 0x578, 0x582, 0x532, 0x561, 0x572,
0x564, 0x561, 0x564, 0x532, 0x561, 0x570, 0x580, 0x565, 0x575, 0x576, 0x532, 0x561,
0x584, 0x578, 0x582, 0x532, 0x561, 0x576, 0x563, 0x56f, 0x578, 0x56f, 0x532, 0x561,
0x57c, 0x576, 0x561, 0x578, 0x582, 0x56c, 0x532, 0x565, 0x575, 0x580, 0x578, 0x582,
0x569, 0x532, 0x56b, 0x577, 0x584, 0x565, 0x56f, 0x532, 0x580, 0x578, 0x582, 0x576,
0x565, 0x575, 0x549, 0x56b, 0x57f, 0x561, 0x549, 0x578, 0x575, 0x562, 0x561, 0x56c,
0x57d, 0x561, 0x576, 0x53f, 0x578, 0x56c, 0x578, 0x574, 0x562, 0x578, 0x534, 0x561,
0x574, 0x561, 0x57d, 0x56f, 0x578, 0x57d, 0x534, 0x561, 0x584, 0x584, 0x561, 0x534,
0x56b, 0x56c, 0x56b, 0x534, 0x578, 0x582, 0x562, 0x561, 0x575, 0x534, 0x578, 0x582,
0x577, 0x561, 0x576, 0x562, 0x565, 0x556, 0x561, 0x574, 0x561, 0x563, 0x578, 0x582,
0x57d, 0x57f, 0x561, 0x533, 0x561, 0x566, 0x561, 0x540, 0x565, 0x562, 0x580, 0x578,
0x576, 0x540, 0x578, 0x577, 0x56b, 0x574, 0x56b, 0x576, 0x540, 0x578, 0x576, 0x56f,
0x578, 0x576, 0x563, 0x540, 0x578, 0x57e, 0x564, 0x53b, 0x580, 0x56f, 0x578, 0x582,
0x57f, 0x57d, 0x56f, 0x54b, 0x561, 0x56f, 0x561, 0x580, 0x57f, 0x561, 0x54b, 0x561,
0x575, 0x57a, 0x578, 0x582, 0x580, 0x561, 0x535, 0x580, 0x578, 0x582, 0x57d, 0x561,
0x572, 0x565, 0x574, 0x554, 0x561, 0x562, 0x578, 0x582, 0x56c, 0x53f, 0x561, 0x574,
0x579, 0x561, 0x57f, 0x56f, 0x561, 0x53f, 0x561, 0x580, 0x561, 0x579, 0x56b, 0x53f,
0x561, 0x57f, 0x574, 0x561, 0x576, 0x564, 0x578, 0x582, 0x53d, 0x561, 0x576, 0x564,
0x56b, 0x563, 0x561, 0x53f, 0x561, 0x56c, 0x56f, 0x578, 0x582, 0x57f, 0x561, 0x53f,
0x580, 0x561, 0x57d, 0x576, 0x578, 0x575, 0x561, 0x580, 0x57d, 0x56f, 0x53f, 0x578,
0x582, 0x561, 0x56c, 0x561, 0x20, 0x53c, 0x578, 0x582, 0x574, 0x57a, 0x578, 0x582,
0x580, 0x53f, 0x578, 0x582, 0x579, 0x56b, 0x576, 0x563, 0x554, 0x578, 0x582, 0x57e,
0x565, 0x575, 0x569, 0x544, 0x561, 0x56f, 0x561, 0x578, 0x544, 0x561, 0x563, 0x561,
0x564, 0x561, 0x576, 0x544, 0x561, 0x56f, 0x561, 0x57d, 0x561, 0x580, 0x544, 0x561,
0x576, 0x56b, 0x56c, 0x561, 0x544, 0x561, 0x57d, 0x56f, 0x561, 0x57f, 0x546, 0x56b,
0x56f, 0x578, 0x57d, 0x56b, 0x561, 0x546, 0x578, 0x57e, 0x578, 0x56f, 0x578, 0x582,
0x566, 0x576, 0x565, 0x581, 0x56f, 0x546, 0x578, 0x57e, 0x578, 0x57d, 0x56b, 0x562,
0x56b, 0x580, 0x57d, 0x56f, 0x555, 0x574, 0x57d, 0x56f, 0x548, 0x582, 0x580, 0x561,
0x56c, 0x57d, 0x56f, 0x54a, 0x576, 0x578, 0x574, 0x57a, 0x565, 0x576, 0x54a, 0x578,
0x576, 0x57f, 0x56b, 0x561, 0x576, 0x561, 0x56f, 0x553, 0x56d, 0x565, 0x576, 0x575,
0x561, 0x576, 0x53f, 0x561, 0x57f, 0x561, 0x580, 0x53f, 0x578, 0x57d, 0x57f, 0x561,
0x576, 0x561, 0x575, 0x53f, 0x56b, 0x566, 0x56b, 0x56c, 0x578, 0x580, 0x564, 0x561,
0x537, 0x580, 0x20, 0x54c, 0x56b, 0x561, 0x564, 0x54d, 0x561, 0x56d, 0x561, 0x56c,
0x56b, 0x576, 0x54d, 0x561, 0x574, 0x561, 0x580, 0x572, 0x561, 0x576, 0x564, 0x54d,
0x565, 0x578, 0x582, 0x56c, 0x547, 0x561, 0x576, 0x570, 0x561, 0x575, 0x54d, 0x56b,
0x576, 0x563, 0x561, 0x57a, 0x578, 0x582, 0x580, 0x54d, 0x580, 0x565, 0x564, 0x576,
0x565, 0x56f, 0x578, 0x56c, 0x56b, 0x574, 0x57d, 0x56f, 0x539, 0x561, 0x575, 0x57a,
0x565, 0x575, 0x54f, 0x561, 0x577, 0x584, 0x565, 0x576, 0x564, 0x539, 0x562, 0x56b,
0x56c, 0x56b, 0x57d, 0x56b, 0x539, 0x565, 0x570, 0x580, 0x561, 0x576, 0x54f, 0x570,
0x56b, 0x574, 0x57a, 0x570, 0x578, 0x582, 0x54f, 0x578, 0x56f, 0x56b, 0x578, 0x54f,
0x578, 0x574, 0x57d, 0x56f, 0x548, 0x582, 0x56c, 0x561, 0x576, 0x20, 0x532, 0x561,
0x57f, 0x578, 0x580, 0x548, 0x582, 0x580, 0x578, 0x582, 0x574, 0x579, 0x56b, 0x548,
0x582, 0x57d, 0x57f, 0x2d, 0x546, 0x565, 0x580, 0x561, 0x54e, 0x575, 0x565, 0x576,
0x57f, 0x575, 0x561, 0x576, 0x54e, 0x56c, 0x561, 0x564, 0x56b, 0x57e, 0x578, 0x57d,
0x57f, 0x578, 0x56f, 0x545, 0x561, 0x56f, 0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x54c,
0x561, 0x576, 0x563, 0x578, 0x582, 0x576, 0x535, 0x56f, 0x561, 0x57f, 0x565, 0x580,
0x56b, 0x576, 0x562, 0x578, 0x582, 0x580, 0x563, 0x535, 0x580, 0x587, 0x561, 0x576,
0x531, 0x566, 0x578, 0x580, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b,
0x576, 0x565, 0x580, 0x532, 0x565, 0x580, 0x574, 0x578, 0x582, 0x564, 0x576, 0x565,
0x580, 0x53f, 0x561, 0x576, 0x561, 0x580, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572,
0x566, 0x56b, 0x576, 0x565, 0x580, 0x53f, 0x561, 0x562, 0x578, 0x20, 0x54e, 0x565,
0x580, 0x564, 0x565, 0x556, 0x561, 0x580, 0x565, 0x580, 0x575, 0x561, 0x576, 0x20,
0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x544, 0x561, 0x564, 0x565, 0x575,
0x580, 0x561, 0x54c, 0x565, 0x575, 0x56f, 0x575, 0x561, 0x57e, 0x56b, 0x56f, 0x540,
0x561, 0x580, 0x561, 0x57e, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x54b, 0x578, 0x580,
0x57b, 0x56b, 0x561, 0x54d, 0x578, 0x582, 0x580, 0x562, 0x20, 0x540, 0x565, 0x572,
0x56b, 0x576, 0x565, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x54d, 0x569, 0x565,
0x576, 0x56c, 0x56b, 0x531, 0x564, 0x565, 0x56c, 0x561, 0x56b, 0x564, 0x561, 0x532,
0x580, 0x56b, 0x57d, 0x562, 0x565, 0x576, 0x532, 0x580, 0x578, 0x584, 0x565, 0x576,
0x20, 0x540, 0x56b, 0x56c, 0x534, 0x561, 0x580, 0x57e, 0x56b, 0x576, 0x545, 0x578,
0x582, 0x56f, 0x56c, 0x561, 0x540, 0x578, 0x562, 0x561, 0x580, 0x57f, 0x53c, 0x56b,
0x576, 0x564, 0x565, 0x574, 0x561, 0x576, 0x53c, 0x578, 0x580, 0x564, 0x20, 0x540,
0x561, 0x578, 0x582, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x544, 0x565, 0x56c, 0x562,
0x578, 0x582, 0x580, 0x576, 0x54a, 0x565, 0x580, 0x569, 0x54d, 0x56b, 0x564, 0x576,
0x565, 0x575, 0x531, 0x574, 0x57d, 0x57f, 0x565, 0x580, 0x564, 0x561, 0x574, 0x531,
0x576, 0x564, 0x578, 0x580, 0x580, 0x561, 0x531, 0x57d, 0x57f, 0x580, 0x561, 0x56d,
0x561, 0x576, 0x531, 0x569, 0x565, 0x576, 0x584, 0x532, 0x565, 0x56c, 0x563, 0x580,
0x561, 0x564, 0x532, 0x565, 0x57c, 0x56c, 0x56b, 0x576, 0x532, 0x580, 0x561, 0x57f,
0x56b, 0x57d, 0x56c, 0x561, 0x57e, 0x561, 0x532, 0x580, 0x575, 0x578, 0x582, 0x57d,
0x565, 0x56c, 0x532, 0x578, 0x582, 0x56d, 0x561, 0x580, 0x565, 0x57d, 0x57f, 0x532,
0x578, 0x582, 0x564, 0x561, 0x57a, 0x565, 0x577, 0x57f, 0x532, 0x575, 0x578, 0x582,
0x566, 0x56b, 0x576, 0x563, 0x565, 0x576, 0x554, 0x56b, 0x577, 0x576, 0x587, 0x53f,
0x578, 0x57a, 0x565, 0x576, 0x570, 0x561, 0x563, 0x565, 0x576, 0x534, 0x578, 0x582,
0x562, 0x56c, 0x56b, 0x576, 0x54b, 0x56b, 0x562, 0x580, 0x561, 0x56c, 0x569, 0x561,
0x580, 0x533, 0x565, 0x580, 0x576, 0x57d, 0x56b, 0x540, 0x565, 0x56c, 0x57d, 0x56b,
0x576, 0x56f, 0x56b, 0x544, 0x565, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x54d,
0x57f, 0x561, 0x574, 0x562, 0x578, 0x582, 0x56c, 0x54b, 0x565, 0x580, 0x57d, 0x56b,
0x53f, 0x561, 0x56c, 0x56b, 0x576, 0x56b, 0x576, 0x563, 0x580, 0x561, 0x564, 0x53f,
0x56b, 0x580, 0x578, 0x57e, 0x53f, 0x56b, 0x587, 0x53c, 0x56b, 0x57d, 0x561, 0x562,
0x578, 0x576, 0x53c, 0x575, 0x578, 0x582, 0x562, 0x56c, 0x575, 0x561, 0x576, 0x561,
0x53c, 0x578, 0x576, 0x564, 0x578, 0x576, 0x53c, 0x575, 0x578, 0x582, 0x584, 0x57d,
0x565, 0x574, 0x562, 0x578, 0x582, 0x580, 0x563, 0x544, 0x561, 0x564, 0x580, 0x56b,
0x564, 0x544, 0x561, 0x56c, 0x569, 0x561, 0x544, 0x561, 0x580, 0x56b, 0x565, 0x570,
0x561, 0x574, 0x576, 0x544, 0x56b, 0x576, 0x57d, 0x56f, 0x544, 0x578, 0x576, 0x561,
0x56f, 0x578, 0x544, 0x578, 0x57d, 0x56f, 0x57e, 0x561, 0x555, 0x57d, 0x56c, 0x578,
0x553, 0x561, 0x580, 0x56b, 0x566, 0x54a, 0x578, 0x564, 0x563, 0x578, 0x580, 0x56b,
0x581, 0x561, 0x54a, 0x580, 0x561, 0x570, 0x561, 0x54c, 0x56b, 0x563, 0x561, 0x540,
0x57c, 0x578, 0x574, 0x54d, 0x561, 0x574, 0x561, 0x580, 0x561, 0x54d, 0x561, 0x576,
0x20, 0x544, 0x561, 0x580, 0x56b, 0x576, 0x578, 0x54d, 0x561, 0x580, 0x561, 0x587,
0x578, 0x54d, 0x561, 0x580, 0x561, 0x57f, 0x578, 0x57e, 0x54d, 0x56b, 0x574, 0x586,
0x565, 0x580, 0x578, 0x57a, 0x578, 0x56c, 0x54d, 0x56f, 0x578, 0x57a, 0x575, 0x565,
0x54d, 0x578, 0x586, 0x56b, 0x561, 0x54d, 0x57f, 0x578, 0x56f, 0x570, 0x578, 0x56c,
0x574, 0x54f, 0x561, 0x56c, 0x56c, 0x56b, 0x576, 0x54f, 0x56b, 0x580, 0x561, 0x576,
0x561, 0x548, 0x582, 0x56c, 0x575, 0x561, 0x576, 0x578, 0x57e, 0x57d, 0x56f, 0x54e,
0x561, 0x564, 0x578, 0x582, 0x581, 0x54e, 0x561, 0x57f, 0x56b, 0x56f, 0x561, 0x576,
0x54e, 0x56b, 0x565, 0x576, 0x576, 0x561, 0x54e, 0x56b, 0x56c, 0x576, 0x575, 0x578,
0x582, 0x57d, 0x54e, 0x578, 0x56c, 0x563, 0x578, 0x563, 0x580, 0x561, 0x564, 0x54e,
0x561, 0x580, 0x577, 0x561, 0x57e, 0x561, 0x536, 0x561, 0x563, 0x580, 0x565, 0x562,
0x551, 0x575, 0x578, 0x582, 0x580, 0x56b, 0x56d, 0x531, 0x576, 0x57f, 0x561, 0x576,
0x561, 0x576, 0x561, 0x580, 0x56b, 0x57e, 0x578, 0x582, 0x549, 0x561, 0x563, 0x578,
0x57d, 0x54d, 0x578, 0x582, 0x580, 0x562, 0x20, 0x53e, 0x576, 0x576, 0x564, 0x575,
0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x53f, 0x578, 0x56f, 0x578, 0x57d,
0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x53f,
0x578, 0x574, 0x578, 0x580, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b,
0x576, 0x565, 0x580, 0x53f, 0x565, 0x580, 0x563, 0x565, 0x56c, 0x565, 0x576, 0x544,
0x561, 0x567, 0x544, 0x561, 0x56c, 0x564, 0x56b, 0x57e, 0x576, 0x565, 0x580, 0x544,
0x561, 0x57e, 0x580, 0x56b, 0x56f, 0x56b, 0x578, 0x57d, 0x544, 0x561, 0x575, 0x578,
0x569, 0x54c, 0x565, 0x575, 0x578, 0x582, 0x576, 0x56b, 0x578, 0x576, 0x531, 0x57a,
0x56b, 0x561, 0x555, 0x584, 0x56c, 0x565, 0x576, 0x564, 0x532, 0x578, 0x582, 0x563,
0x565, 0x576, 0x57e, 0x56b, 0x56c, 0x549, 0x561, 0x569, 0x565, 0x574, 0x20, 0x56f,
0x572, 0x566, 0x56b, 0x549, 0x578, 0x582, 0x578, 0x582, 0x56f, 0x536, 0x561, 0x57f,
0x56f, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x537, 0x586, 0x561, 0x57f, 0x565,
0x556, 0x561, 0x56f, 0x561, 0x578, 0x586, 0x578, 0x556, 0x56b, 0x57b, 0x56b, 0x556,
0x578, 0x582, 0x576, 0x561, 0x586, 0x578, 0x582, 0x57f, 0x56b, 0x533, 0x561, 0x56c,
0x561, 0x57a, 0x561, 0x563, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572,
0x566, 0x56b, 0x576, 0x565, 0x580, 0x533, 0x561, 0x574, 0x562, 0x575, 0x565, 0x20,
0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x533, 0x578, 0x582, 0x561, 0x564,
0x561, 0x56c, 0x56f, 0x561, 0x576, 0x561, 0x56c, 0x533, 0x578, 0x582, 0x561, 0x574,
0x540, 0x578, 0x576, 0x578, 0x56c, 0x578, 0x582, 0x56c, 0x578, 0x582, 0x537, 0x576,
0x564, 0x565, 0x580, 0x562, 0x565, 0x580, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b,
0x53f, 0x56b, 0x580, 0x56b, 0x57f, 0x56b, 0x574, 0x561, 0x57f, 0x56b, 0x53f, 0x578,
0x57d, 0x580, 0x561, 0x565, 0x554, 0x57e, 0x561, 0x57b, 0x561, 0x56c, 0x565, 0x575,
0x576, 0x544, 0x561, 0x57b, 0x578, 0x582, 0x580, 0x578, 0x544, 0x561, 0x580, 0x56f,
0x56b, 0x566, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565,
0x580, 0x544, 0x56b, 0x564, 0x578, 0x582, 0x565, 0x575, 0x20, 0x56f, 0x572, 0x566,
0x56b, 0x546, 0x561, 0x578, 0x582, 0x580, 0x578, 0x582, 0x546, 0x56b, 0x578, 0x582,
0x565, 0x546, 0x578, 0x580, 0x586, 0x578, 0x56c, 0x56f, 0x546, 0x578, 0x582, 0x574,
0x565, 0x561, 0x54a, 0x561, 0x563, 0x578, 0x20, 0x54a, 0x561, 0x563, 0x578, 0x54a,
0x561, 0x56c, 0x561, 0x578, 0x582, 0x54a, 0x56b, 0x57f, 0x56f, 0x565, 0x57c, 0x576,
0x54a, 0x578, 0x576, 0x57a, 0x565, 0x56b, 0x54a, 0x578, 0x580, 0x57f, 0x20, 0x544,
0x578, 0x580, 0x57d, 0x562, 0x56b, 0x54c, 0x561, 0x580, 0x578, 0x57f, 0x578, 0x576,
0x563, 0x561, 0x54d, 0x561, 0x575, 0x57a, 0x561, 0x576, 0x539, 0x561, 0x56b, 0x569,
0x56b, 0x54f, 0x561, 0x580, 0x561, 0x57e, 0x561, 0x54f, 0x578, 0x576, 0x563, 0x561,
0x57f, 0x561, 0x57a, 0x578, 0x582, 0x548, 0x582, 0x565, 0x575, 0x584, 0x20, 0x56f,
0x572, 0x566, 0x56b, 0x548, 0x582, 0x578, 0x56c, 0x56b, 0x57d, 0x986, 0x9ac, 0x9bf,
0x9a1, 0x99c, 0x9be, 0x9a8, 0x98f, 0x995, 0x9cd, 0x9f0, 0x9be, 0x986, 0x9a6, 0x9cd,
0x9a6, 0x9bf, 0x99b, 0x20, 0x986, 0x9ac, 0x9be, 0x9ac, 0x9be, 0x986, 0x9b2, 0x99c,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x9cd, 0x99b, 0x986, 0x9b8, 0x9cd, 0x9ae, 0x9be,
0x9f0, 0x9be, 0x9ac, 0x9be, 0x9ae, 0x9be, 0x995, 0x9cb, 0x9ac, 0x9be, 0x982, 0x997,
0x9c1, 0x987, 0x9ac, 0x9c7, 0x99e, 0x9cd, 0x99c, 0x9c1, 0x9b2, 0x9ac, 0x9bf, 0x99b,
0x9be, 0x989, 0x9ac, 0x9cd, 0x9b2, 0x9be, 0x9a3, 0x9cd, 0x99f, 0x9be, 0x9af, 0x9bc,
0x9be, 0x9f0, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x99c, 0x9be, 0x9ad, 0x9bf, 0x9b2, 0x9c7,
0x9ac, 0x9c1, 0x99c, 0x9c1, 0x9ae, 0x9cd, 0x9ac, 0x9c1, 0x9f0, 0x9be, 0x995, 0x9be,
0x987, 0x9f0, 0x9cb, 0x995, 0x9be, 0x99b, 0x9be, 0x9ac, 0x9cd, 0x9b2, 0x9be, 0x982,
0x995, 0x9be, 0x99a, 0x9c7, 0x989, 0x99f, 0x9be, 0x995, 0x9cb, 0x9a8, 0x9be, 0x995,
0x9cd, 0x9f0, 0x9c0, 0x9a1, 0x9be, 0x995, 0x9be, 0x9f0, 0x9a1, 0x9be, 0x9f0, 0x20,
0x98f, 0x99b, 0x20, 0x99b, 0x9be, 0x9b2, 0x9be, 0x9ae, 0x99c, 0x9bf, 0x9ac, 0x9c1,
0x99f, 0x9bf, 0x9a1, 0x9c1, 0x9f1, 0x9be, 0x9b2, 0x9be, 0x98f, 0x9b2, 0x20, 0x986,
0x987, 0x9af, 0x9bc, 0x9c1, 0x9a8, 0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x99f, 0x9be, 0x989,
0x9a8, 0x997, 0x9be, 0x9ac, 0x9cb, 0x9f0, 0x9cb, 0x9a3, 0x9b9, 0x9be, 0x9f0, 0x9be,
0x9f0, 0x9c7, 0x99c, 0x9cb, 0x9b9, 0x9be, 0x9a8, 0x9cd, 0x9b8, 0x9ac, 0x9be, 0x9f0,
0x9cd, 0x997, 0x99c, 0x9c1, 0x9ac, 0x9be, 0x995, 0x9be, 0x9ae, 0x9cd, 0x9aa, 0x9be,
0x9b2, 0x9be, 0x996, 0x9be, 0x9f0, 0x9cd, 0x99f, 0x9c1, 0x9ae, 0x995, 0x9bf, 0x997,
0x9be, 0x9b2, 0x9c0, 0x995, 0x9bf, 0x982, 0x99a, 0x9be, 0x99b, 0x9be, 0x9b2, 0x9be,
0x997, 0x9cb, 0x99b, 0x9b2, 0x9bf, 0x9ac, 0x9cd, 0x9f0, 0x9bf, 0x9ad, 0x9bf, 0x9b2,
0x9c7, 0x9b2, 0x9cb, 0x9ae, 0x9b2, 0x9c1, 0x9f1, 0x9be, 0x9a3, 0x9cd, 0x9a1, 0x9be,
0x9b2, 0x9c1, 0x9ac, 0x9c1, 0x9ae, 0x9cd, 0x9ac, 0x9be, 0x99a, 0x9bf, 0x9b2, 0x9c1,
0x99b, 0x9be, 0x995, 0x9be, 0x9ae, 0x9be, 0x9b2, 0x9be, 0x9ac, 0x9cb, 0x9ae, 0x9be,
0x9aa, 0x9c1, 0x99f, 0x9cb, 0x9ae, 0x9be, 0x99b, 0x9c7, 0x9f0, 0x9c1, 0x98f, 0x9ae,
0x9cd, 0x9ac, 0x9be, 0x9ac, 0x9c7, 0x9a8, 0x9ae, 0x9cb, 0x997, 0x9be, 0x9a6, 0x9bf,
0x99a, 0x9c1, 0x9ae, 0x9cb, 0x9a8, 0x9f0, 0x9cb, 0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a8, 0x9be, 0x987, 0x9f0, 0x9cb, 0x9ac, 0x9bf, 0x9a8, 0x9c7, 0x9a6, 0x99c, 0x9be,
0x9ae, 0x9c7, 0x9a8, 0x9be, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9ae, 0x9c0, 0x9a8,
0x9cc, 0x986, 0x995, 0x99a, 0x9cb, 0x99f, 0x994, 0x997, 0x9be, 0x9a1, 0x9cb, 0x997,
0x9cb, 0x9aa, 0x9cb, 0x9f0, 0x9cd, 0x99f, 0x9cb, 0x2d, 0x9a8, 0x9cb, 0x9ad, 0x9cb,
0x99a, 0x9be, 0x993, 0x20, 0x99f, 0x9cb, 0x9ae, 0x9c7, 0x9a4, 0x9cd, 0x9f0, 0x9bf,
0x9aa, 0x9cb, 0x9b2, 0x9bf, 0x99f, 0x9c1, 0x9a8, 0x9bf, 0x99b, 0x9f1, 0x9bf, 0x9a3,
0x9cd, 0x9a1, 0x9b9, 0x9cb, 0x995, 0x986, 0x9a1, 0x9be, 0x995, 0x98f, 0x982, 0x995,
0x9cb, 0x9f0, 0x9c7, 0x99c, 0x98f, 0x9a8, 0x997, 0x9c1, 0x987, 0x9b2, 0x9be, 0x98f,
0x9a3, 0x9cd, 0x99f, 0x9bf, 0x997, 0x9c1, 0x9f1, 0x9be, 0x986, 0x9f0, 0x9be, 0x997,
0x9be, 0x9af, 0x9bc, 0x9c7, 0x9a8, 0x9be, 0x9ac, 0x9c1, 0x9a8, 0x99b, 0x20, 0x986,
0x9af, 0x9bc, 0x9be, 0x9f0, 0x9cd, 0x99b, 0x995, 0x9be, 0x99f, 0x9be, 0x9ae, 0x9be,
0x9f0, 0x9cd, 0x995, 0x9be, 0x995, 0x9cb, 0x9f0, 0x9cd, 0x9a1, 0x9cb, 0x9ac, 0x9be,
0x99c, 0x9c1, 0x99c, 0x9c1, 0x987, 0x9b2, 0x9be, 0x20, 0x9f0, 0x9bf, 0x993, 0x99c,
0x9be, 0x9ae, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x99c, 0x9be, 0x9f0, 0x9bf, 0x985,
0x2019, 0x20, 0x997, 0x9be, 0x9b2, 0x9c7, 0x997, 0x9cb, 0x99b, 0x99b, 0x9be, 0x9b2,
0x9cd, 0x99f, 0x9be, 0x99b, 0x9c7, 0x9a8, 0x20, 0x99c, 0x9c1, 0x9f1, 0x9be, 0x9a8,
0x99b, 0x9c7, 0x9a8, 0x20, 0x9b2, 0x9c1, 0x987, 0x99a, 0x99f, 0x9c1, 0x995, 0x9c1,
0x9ae, 0x9a8, 0x989, 0x99a, 0x9c1, 0x986, 0x987, 0x9af, 0x9bc, 0x9be, 0x986, 0x9f0,
0x9c1, 0x9ac, 0x9be, 0x986, 0x99b, 0x9c1, 0x9a8, 0x99a, 0x9bf, 0x9af, 0x9bc, 0x9a8,
0x986, 0x99f, 0x9bf, 0x995, 0x9cb, 0x995, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x9b9, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9ac, 0x9be, 0x9b9, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ac,
0x9be, 0x9a8, 0x9cd, 0x9a6, 0x9c7, 0x9f0, 0x9be, 0x99b, 0x9ac, 0x9be, 0x9f0, 0x9cd,
0x9ac, 0x9be, 0x9a1, 0x9cb, 0x99a, 0x9ac, 0x9c7, 0x9b2, 0x9c7, 0x9ae, 0x9ac, 0x9c7,
0x9b2, 0x9bf, 0x99c, 0x9ac, 0x9cd, 0x9b2, 0x9be, 0x982, 0x995, 0x2d, 0x99b, 0x9c7,
0x9ac, 0x9b2, 0x9cb, 0x9a8, 0x9ac, 0x9cb, 0x9f1, 0x9be, 0x20, 0x9ad, 0x9bf, 0x9b7,
0x9cd, 0x99f, 0x9be, 0x9ac, 0x9cb, 0x997, 0x9cb, 0x99f, 0x9be, 0x9ac, 0x2019, 0x987,
0x99c, 0x9c7, 0x995, 0x9c7, 0x9ae, 0x9ac, 0x9cd, 0x9f0, 0x9bf, 0x99c, 0x20, 0x989,
0x9aa, 0x9b8, 0x9be, 0x997, 0x9f0, 0x995, 0x9c7, 0x9ae, 0x9cd, 0x9aa, 0x9cb, 0x20,
0x997, 0x9cd, 0x9f0, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9c7, 0x995, 0x9c7, 0x9a8, 0x995,
0x9c1, 0x9a8, 0x995, 0x9be, 0x9f0, 0x9be, 0x995, 0x9be, 0x99b, 0x995, 0x9c7, 0x9af,
0x9bc, 0x9c7, 0x9a8, 0x995, 0x9c7, 0x987, 0x9ae, 0x9c7, 0x9a8, 0x99a, 0x9bf, 0x995,
0x9be, 0x997, 0x9cb, 0x99a, 0x9bf, 0x9b9, 0x9c1, 0x986, 0x9b9, 0x9c1, 0x9f1, 0x9be,
0x99a, 0x9bf, 0x989, 0x9a1, 0x9c7, 0x2018, 0x9a1, 0x20, 0x99c, 0x9f0, 0x9c7, 0x99c,
0x995, 0x9cb, 0x9b7, 0x9cd, 0x99f, 0x9be, 0x20, 0x9f0, 0x9bf, 0x995, 0x9be, 0x995,
0x9cd, 0x9f0, 0x9c7, 0x9b7, 0x9cd, 0x99f, 0x9a8, 0x995, 0x9c1, 0x987, 0x986, 0x9ac,
0x9be, 0x995, 0x9c1, 0x9f0, 0x9be, 0x995, 0x9be, 0x993, 0x9a1, 0x9c7, 0x9a8, 0x9ae,
0x9be, 0x9f0, 0x9cd, 0x995, 0x9b6, 0x9cd, 0x9ac, 0x9a8, 0x9a1, 0x9be, 0x993, 0x99b,
0x9a8, 0x9a1, 0x9be, 0x993, 0x99b, 0x9a8, 0x20, 0x995, 0x9cd, 0x9f0, 0x9c0, 0x995,
0x9a1, 0x9c7, 0x9a8, 0x9ad, 0x9be, 0x9f0, 0x9a1, 0x9c7, 0x99f, 0x9cd, 0x9f0, 0x9cb,
0x987, 0x99f, 0x9a1, 0x9ae, 0x9bf, 0x9a8, 0x9bf, 0x995, 0x9be, 0x98f, 0x9a1, 0x9ae,
0x9a3, 0x9cd, 0x99f, 0x9a8, 0x987, 0x9f0, 0x9c1, 0x9a8, 0x9c7, 0x9aa, 0x9c7, 0x98f,
0x9b2, 0x20, 0x99b, 0x9c7, 0x9b2, 0x9ad, 0x9c7, 0x9a1, 0x2019, 0x9f0, 0x9ab, 0x2019,
0x9f0, 0x9cd, 0x99f, 0x20, 0x9a8, 0x9c7, 0x9b2, 0x99b, 0x9a8, 0x9ab, 0x9cb, 0x9f0,
0x9cd, 0x99f, 0x9be, 0x9b2, 0x9c7, 0x99c, 0x9be, 0x997, 0x9cd, 0x9b2, 0x9c7, 0x99a,
0x20, 0x989, 0x9aa, 0x9b8, 0x9be, 0x997, 0x9f0, 0x997, 0x9c1, 0x99b, 0x20, 0x989,
0x9aa, 0x9b8, 0x9be, 0x997, 0x9f0, 0x997, 0x9cd, 0x9f0, 0x9c7, 0x9a3, 0x9cd, 0x9a1,
0x20, 0x99f, 0x9be, 0x9f0, 0x9cd, 0x995, 0x997, 0x9cd, 0x9f0, 0x9c7, 0x9a8, 0x9be,
0x9a1, 0x9be, 0x997, 0x9c1, 0x9f1, 0x9be, 0x9a1, 0x9c7, 0x9b2, 0x9c1, 0x9aa, 0x997,
0x9c1, 0x9f1, 0x9be, 0x99f, 0x9c7, 0x9ae, 0x9be, 0x9b2, 0x9be, 0x997, 0x9be, 0x9af,
0x9bc, 0x9be, 0x995, 0x9c1, 0x987, 0x9b2, 0x997, 0x9be, 0x9af, 0x9bc, 0x9be, 0x9a8,
0x9be, 0x9b9, 0x9c7, 0x9b2, 0x9bf, 0x9ab, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9b9, 0x9be,
0x9ad, 0x9be, 0x9a8, 0x9be, 0x9b9, 0x9be, 0x9f0, 0x9cd, 0x9ae, 0x9cb, 0x99b, 0x9bf,
0x9b2, 0x9cd, 0x9b2, 0x9cb, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a8, 0x9be, 0x9aa, 0x2019, 0x9b2, 0x9bf, 0x99b, 0x9a8, 0x995, 0x9cd, 0x9b8, 0x2c,
0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9ae,
0x9be, 0x9f0, 0x9c7, 0x982, 0x997, 0x9cb, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9aa, 0x9bf, 0x99f, 0x9be, 0x9f0, 0x9cd,
0x99b, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x99f, 0x9c7, 0x9b2, 0x20, 0x99a, 0x9bf,
0x99f, 0x9c0, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a8, 0x9be, 0x9ad, 0x9c7, 0x9ad, 0x9c7, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9ad, 0x9bf, 0x9a8, 0x99a, 0x9c7, 0x9a8,
0x9c7, 0x99b, 0x2c, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a8, 0x9be, 0x989, 0x987, 0x9a8, 0x9be, 0x9ae, 0x9c7, 0x995, 0x2c, 0x20, 0x987,
0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x987, 0x9a8, 0x9c1,
0x9ad, 0x9bf, 0x995, 0x987, 0x995, 0x9be, 0x9b2, 0x9c1, 0x987, 0x99f, 0x99c, 0x9be,
0x9ae, 0x9be, 0x987, 0x995, 0x9be, 0x99c, 0x9c1, 0x9a8, 0x9c7, 0x989, 0x9b2, 0x9c1,
0x987, 0x99b, 0x9ad, 0x9bf, 0x9b2, 0x9c7, 0x9ae, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x99a,
0x9c7, 0x9b2, 0x9cb, 0x2c, 0x20, 0x995, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9c1, 0x995,
0x9c0, 0x995, 0x9cd, 0x9f0, 0x9be, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9bf, 0x99c,
0x9bf, 0x995, 0x9b2, 0x9be, 0x20, 0x9aa, 0x9be, 0x99c, 0x9b2, 0x9bf, 0x9ae, 0x9be,
0x9b2, 0x99b, 0x20, 0x98f, 0x99e, 0x9cd, 0x99c, 0x9c7, 0x9b2, 0x99b, 0x9cd, 0x9b2,
0x2019, 0x9f1, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9bf, 0x9a8, 0x9cd, 0x9b8,
0x9c7, 0x99b, 0x20, 0x995, 0x9cb, 0x9f1, 0x9be, 0x9f0, 0x9cd, 0x99f, 0x9be, 0x9f0,
0x9ae, 0x9c7, 0x99a, 0x9bf, 0x993, 0x9ae, 0x9be, 0x9a8, 0x9be, 0x997, 0x9c1, 0x9f1,
0x9be, 0x9ae, 0x9be, 0x9a8, 0x9be, 0x99b, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x997, 0x9cb,
0x99f, 0x9ae, 0x9be, 0x9f0, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9bf, 0x995, 0x9ae, 0x9be,
0x99f, 0x9be, 0x9ae, 0x9cb, 0x9f0, 0x9cb, 0x99b, 0x9ae, 0x9be, 0x99c, 0x9be, 0x99f,
0x9cd, 0x200c, 0x9b2, 0x9be, 0x9a8, 0x9ae, 0x9c7, 0x9a8, 0x9cb, 0x9ae, 0x9bf, 0x9a8,
0x9c0, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x9a1, 0x9be, 0x9ae, 0x9c7, 0x99f, 0x9b2, 0x9be,
0x995, 0x9be, 0x99f, 0x9cd, 0x200c, 0x9b2, 0x9be, 0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8,
0x9bf, 0x995, 0x9cb, 0x20, 0x99a, 0x9bf, 0x99f, 0x9c0, 0x9ae, 0x9bf, 0x995, 0x9bf,
0x989, 0x9f1, 0x9c7, 0x9b2, 0x9a8, 0x9ae, 0x9a8, 0x9cd, 0x200c, 0x995, 0x99f, 0x9a8,
0x9ae, 0x9a3, 0x9cd, 0x99f, 0x9c7, 0x9f0, 0x9c7, 0x9f0, 0x9c0, 0x9ae, 0x9a3, 0x9cd,
0x99f, 0x9c7, 0x9ad, 0x9bf, 0x9a1, 0x9bf, 0x985, 0x2019, 0x9ae, 0x9a3, 0x9cd, 0x99f,
0x99b, 0x9c7, 0x9f0, 0x9be, 0x99f, 0x9a8, 0x9be, 0x99b, 0x9be, 0x989, 0x9a8, 0x9bf,
0x989, 0x20, 0x987, 0x9af, 0x9bc, 0x9f0, 0x9cd, 0x995, 0x9a8, 0x9cb, 0x9f0, 0x9cb,
0x9a8, 0x9cd, 0x200c, 0x9b9, 0x9be, 0x9ac, 0x9c7, 0x989, 0x9b2, 0x9be, 0x9b9, 0x2c,
0x20, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x9a1, 0x9be, 0x995, 0x9cb, 0x99f,
0x9be, 0x99a, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x2c, 0x20, 0x989, 0x9a4,
0x9cd, 0x9a4, 0x9f0, 0x20, 0x9a1, 0x9be, 0x995, 0x9cb, 0x99f, 0x9be, 0x9a8, 0x9bf,
0x989, 0x20, 0x99b, 0x9be, 0x9b2, 0x9c7, 0x9ae, 0x2c, 0x20, 0x989, 0x9a4, 0x9cd,
0x9a4, 0x9f0, 0x20, 0x9a1, 0x9be, 0x995, 0x9cb, 0x99f, 0x9be, 0x9a8, 0x9c1, 0x995,
0x985, 0x2019, 0x99c, 0x9bf, 0x9a8, 0x9be, 0x997, 0x9be, 0x9aa, 0x9be, 0x9a8, 0x9be,
0x9ae, 0x9be, 0x9aa, 0x9be, 0x9f0, 0x9be, 0x9ae, 0x9be, 0x9f0, 0x9bf, 0x9ac, 0x9cb,
0x9ab, 0x9bf, 0x9a8, 0x9bf, 0x995, 0x9cd, 0x9b8, 0x9aa, 0x2019, 0x9f0, 0x9cd, 0x99f,
0x2d, 0x994, 0x2d, 0x9aa, 0x9cd, 0x9f0, 0x9bf, 0x9a8, 0x9cd, 0x9b8, 0x9aa, 0x2019,
0x9f0, 0x9cd, 0x99f, 0x20, 0x985, 0x9f1, 0x20, 0x9b8, 0x9cd, 0x9aa, 0x9c7, 0x987,
0x9a8, 0x9aa, 0x9cb, 0x9f0, 0x9cd, 0x99f, 0x9cb, 0x20, 0x9ad, 0x9c7, 0x9b2, 0x9cd,
0x200c, 0x9b9, 0x9cb, 0x9aa, 0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x9cd, 0x99f, 0x9cb, 0x20,
0x9f0, 0x9bf, 0x995, 0x9cb, 0x9aa, 0x9c1, 0x9a3, 0x9cd, 0x99f, 0x9be, 0x20, 0x98f,
0x9f0, 0x9bf, 0x9a8, 0x9be, 0x99b, 0x9f0, 0x9c7, 0x982, 0x995, 0x9bf, 0x9a8, 0x20,
0x987, 0x9a8, 0x9b2, 0x9c7, 0x99f, 0x9f0, 0x9c7, 0x99a, 0x9be, 0x987, 0x9ab, 0x9f0,
0x9c7, 0x99c, 0x9bf, 0x9a8, 0x9be, 0x9f0, 0x9bf, 0x99c, 0x2019, 0x9b2, 0x9bf, 0x989,
0x99f, 0x9f0, 0x9bf, 0x985, 0x2019, 0x20, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x982, 0x995,
0x9cb, 0x99b, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x9c7, 0x9ae, 0x99b, 0x9c7,
0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x997, 0x9cb, 0x99b, 0x9be, 0x9a3,
0x9cd, 0x99f, 0x9cb, 0x20, 0x9a1, 0x9cb, 0x9ae, 0x9bf, 0x982, 0x997, 0x9cb, 0x99b,
0x9be, 0x993, 0x20, 0x9aa, 0x9be, 0x989, 0x9b2, 0x9cb, 0x987, 0x99f, 0x9cb, 0x995,
0x9cb, 0x9f0, 0x9cd, 0x99f, 0x9cb, 0x9f0, 0x9ae, 0x9bf, 0x99f, 0x99b, 0x9bf, 0x99f,
0x9cd, 0x200c, 0x200c, 0x995, 0x9be, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20,
0x9ac, 0x9be, 0x9f0, 0x9cd, 0x9a5, 0x9b2, 0x9c7, 0x9ae, 0x9c7, 0x99b, 0x9c7, 0x987,
0x9a3, 0x9cd, 0x99f, 0x20, 0x99c, 0x9a8, 0x99a, 0x9cd, 0x99b, 0x9c7, 0x987, 0x9a3,
0x9cd, 0x99f, 0x20, 0x995, 0x9bf, 0x99f, 0x99b, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd,
0x99f, 0x20, 0x9b2, 0x9c1, 0x99a, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x99b, 0x9c7, 0x987,
0x9a3, 0x9cd, 0x99f, 0x20, 0x9a5, 0x9ae, 0x9be, 0x99b, 0x99b, 0x9c7, 0x987, 0x9a3,
0x9cd, 0x99f, 0x20, 0x9ad, 0x9bf, 0x9a8, 0x99a, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x99b,
0x9c1, 0x987, 0x9ab, 0x9cd, 0x99f, 0x20, 0x995, 0x9be, 0x9f0, 0x9c7, 0x9a3, 0x9cd,
0x99f, 0x99f, 0x9c7, 0x997, 0x9c1, 0x99a, 0x9bf, 0x997, 0x9be, 0x9b2, 0x9cd, 0x9aa,
0x9be, 0x9a5, 0x9cd, 0x9af, 0x9c1, 0x9b2, 0x9c7, 0x9a4, 0x9bf, 0x99c, 0x9c1, 0x9f1,
0x9be, 0x9a8, 0x9be, 0x99f, 0x2019, 0x9f0, 0x2019, 0x9a3, 0x9cd, 0x99f, 0x9cb, 0x99f,
0x9cb, 0x9f0, 0x9cd, 0x99f, 0x9cb, 0x9b2, 0x9be, 0x9ad, 0x9c7, 0x9a8, 0x995, 0x9c1,
0x9ad, 0x9c7, 0x9f0, 0x9b9, 0x9cb, 0x9f1, 0x9be, 0x987, 0x99f, 0x9b9, 0x9f0, 0x9cd,
0x99a, 0x9f1, 0x9bf, 0x9a8, 0x9bf, 0x9aa, 0x9c7, 0x997, 0x9af, 0x9bc, 0x9be, 0x995,
0x9c1, 0x99f, 0x9be, 0x99f, 0x995, 0x9c7, 0x99b, 0x9c0, 0x9a1, 0x9c7, 0x9ad, 0x9bf,
0x99b, 0x9a1, 0x9c1, 0x9ae, 0x9cb, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9a1, 0x9bf, 0x20,
0x986, 0x9f0, 0x9cd, 0x9ad, 0x9bf, 0x9b2, 0x9f0, 0x9ae, 0x9c7, 0x995, 0x9cd, 0x200c,
0x995, 0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x9c0, 0x9ae, 0x9be, 0x989, 0x99b, 0x9a8, 0x9ae,
0x9c7, 0x995, 0x9ae, 0x9c1, 0x9f0, 0x9cd, 0x9a1, 0x9c1, 0x9aa, 0x9be, 0x9ae, 0x9f0,
0x9f0, 0x9cb, 0x9a5, 0x9c7, 0x9f0, 0x9be, 0x9b8, 0x9cd, 0x9ac, 0x9cb, 0x9f1, 0x9be,
0x99f, 0x9cd, 0x9f0, 0x9cb, 0x9b2, 0x9ad, 0x9cb, 0x9b7, 0x9cd, 0x99f, 0x995, 0x9b2,
0x982, 0x997, 0x9c7, 0x987, 0x9f0, 0x9ac, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9a8, 0x986,
0x9a6, 0x9c7, 0x9a8, 0x986, 0x9b2, 0x9ae, 0x9be, 0x99f, 0x9bf, 0x986, 0x9ae, 0x9cd,
0x9ae, 0x9be, 0x9a8, 0x986, 0x9a8, 0x9be, 0x9a1, 0x9bf, 0x9f0, 0x98f, 0x995, 0x9cd,
0x9af, 0x9cb, 0x99f, 0x9cd, 0x9af, 0x9be, 0x993, 0x98f, 0x995, 0x9cd, 0x9af, 0x9cb,
0x99f, 0x9ac, 0x986, 0x9b6, 0x9cd, 0x9ac, 0x9cd, 0x997, 0x9be, 0x200c, 0x9ac, 0x9be,
0x99f, 0x98f, 0x99f, 0x9f0, 0x9be, 0x989, 0x9ac, 0x9be, 0x997, 0x9a6, 0x9be, 0x9a6,
0x9ac, 0x9be, 0x9b9, 0x9f0, 0x9c7, 0x987, 0x9a8, 0x9ac, 0x9c7, 0x982, 0x995, 0x995,
0x9ac, 0x9be, 0x9f0, 0x9cd, 0x9a8, 0x9c1, 0x9b2, 0x9ac, 0x9c7, 0x987, 0x9f0, 0x9c1,
0x99f, 0x9ac, 0x9bf, 0x9b6, 0x9cd, 0x9ac, 0x9cd, 0x200c, 0x995, 0x9c7, 0x995, 0x9ac,
0x9cd, 0x9f0, 0x9c1, 0x9a8, 0x9c7, 0x987, 0x99a, 0x9bf, 0x99f, 0x9be, 0x995, 0x9cb,
0x987, 0x9ac, 0x9be, 0x9b2, 0x9cd, 0x99b, 0x9a8, 0x995, 0x9b2, 0x9ae, 0x9cd, 0x9ac,
0x9cb, 0x9a1, 0x9be, 0x9ae, 0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x99b, 0x9a2, 0x9be,
0x995, 0x9be, 0x9a1, 0x9bf, 0x9b2, 0x9bf, 0x9a1, 0x9c1, 0x9ac, 0x9be, 0x987, 0x9a1,
0x9c1, 0x99b, 0x9be, 0x9a8, 0x9ac, 0x9c7, 0x9ab, 0x9be, 0x9ae, 0x9be, 0x997, 0x9c1,
0x9b8, 0x9cd, 0x9a4, 0x9be, 0x997, 0x9be, 0x99c, 0x9be, 0x9b9, 0x9c7, 0x9ac, 0x9cd,
0x9f0, 0x9cb, 0x9a8, 0x9b9, 0x9cb, 0x20, 0x99a, 0x9bf, 0x20, 0x9ae, 0x9bf, 0x9a8,
0x20, 0x99a, 0x9bf, 0x99f, 0x9c0, 0x9b9, 0x982, 0x20, 0x995, 0x982, 0x9b9, 0x9cb,
0x9ad, 0x9cd, 0x200c, 0x9a1, 0x987, 0x9f0, 0x9cd, 0x995, 0x9c1, 0x9a4, 0x9cd, 0x9b8,
0x9cd, 0x995, 0x99c, 0x9be, 0x995, 0x9be, 0x9f0, 0x9cd, 0x99f, 0x9be, 0x99c, 0x9be,
0x9af, 0x9bc, 0x9be, 0x9aa, 0x9c1, 0x9f0, 0x9be, 0x99c, 0x9c7, 0x9f0, 0x9c1, 0x99c,
0x9be, 0x9b2, 0x9c7, 0x9ae, 0x995, 0x9be, 0x9ac, 0x9c1, 0x9b2, 0x995, 0x9be, 0x9ae,
0x99a, 0x9a4, 0x9cd, 0x995, 0x9be, 0x995, 0x9f0, 0x9be, 0x99a, 0x9c0, 0x995, 0x9be,
0x9a0, 0x9ae, 0x9be, 0x9a3, 0x9cd, 0x9a1, 0x9c1, 0x996, 0x9be, 0x9a3, 0x9cd, 0x9a1,
0x9bf, 0x997, 0x9be, 0x995, 0x9b2, 0x995, 0x9be, 0x9a4, 0x9be, 0x995, 0x9cd, 0x9f0,
0x9be, 0x99b, 0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995,
0x995, 0x9c1, 0x9f1, 0x9be, 0x9b2, 0x9be, 0x9b2, 0x9c1, 0x9ae, 0x9aa, 0x9c1, 0x9f0,
0x995, 0x9c1, 0x99a, 0x9bf, 0x982, 0x995, 0x9c1, 0x9f1, 0x9c7, 0x987, 0x99f, 0x9ae,
0x9be, 0x995, 0x9be, 0x993, 0x9ae, 0x9be, 0x997, 0x9be, 0x9a6, 0x9be, 0x9a8, 0x9ae,
0x9be, 0x995, 0x9be, 0x99b, 0x9be, 0x9f0, 0x9ae, 0x9be, 0x9a8, 0x9bf, 0x9b2, 0x9be,
0x9ae, 0x9c1, 0x9b8, 0x9cd, 0x995, 0x9c7, 0x99f, 0x9a8, 0x9bf, 0x995, 0x9cb, 0x99b,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x995, 0x9c1, 0x99c, 0x9a8,
0x9c7, 0x9a4, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x99b, 0x9bf,
0x9ac, 0x9bf, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995,
0x985, 0x2019, 0x9f0, 0x9c7, 0x9b2, 0x9a8, 0x9cb, 0x9ae, 0x20, 0x9aa, 0x9c7, 0x9a8,
0x9cd, 0x200c, 0x9b9, 0x9aa, 0x9cb, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x982, 0x995, 0x9aa, 0x9cd, 0x9af, 0x982, 0x9af, 0x9bc, 0x9be, 0x982, 0x995, 0x9be,
0x99f, 0x9be, 0x9f0, 0x995, 0x2019, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9c0, 0x995,
0x9c7, 0x99c, 0x9bf, 0x9b2, 0x9cb, 0x9f0, 0x9cd, 0x9a1, 0x9be, 0x9f0, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9a7, 0x99b, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x99b, 0x9be,
0x9ae, 0x9be, 0x9f0, 0x995, 0x9be, 0x9a3, 0x9cd, 0x9a1, 0x99b, 0x9bf, 0x989, 0x9b2,
0x99b, 0x9be, 0x982, 0x9b9, 0x9be, 0x987, 0x99b, 0x9bf, 0x982, 0x997, 0x9be, 0x9aa,
0x9c1, 0x9f0, 0x9b8, 0x9cd, 0x9f0, 0x9c7, 0x9a1, 0x9a8, 0x9bf, 0x995, 0x9cb, 0x9b2,
0x9bf, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x99f, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987, 0x9a4,
0x9be, 0x99b, 0x995, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x99f, 0x9bf, 0x9ac, 0x9b2, 0x9bf,
0x99a, 0x9bf, 0x9a4, 0x9c7, 0x9b9, 0x9f0, 0x9be, 0x9a8, 0x9a5, 0x9bf, 0x9ae, 0x9cd,
0x9ab, 0x9c1, 0x99f, 0x995, 0x9bf, 0x985, 0x2019, 0x9a4, 0x9cb, 0x9ae, 0x9cd, 0x200c,
0x9b8, 0x9cd, 0x995, 0x989, 0x9b2, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x99f, 0x9be, 0x9f0,
0x989, 0x9f0, 0x9c1, 0x9ae, 0x9cd, 0x200c, 0x995, 0x9bf, 0x989, 0x9b7, 0x9cd, 0x99f,
0x2d, 0x9a8, 0x9c7, 0x9f0, 0x9be, 0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9a3, 0x9cd,
0x99f, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a1, 0x9bf,
0x9ad, 0x9cb, 0x9b7, 0x9cd, 0x99f, 0x995, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x9a4,
0x9cd, 0x9b8, 0x9cd, 0x995, 0x9af, 0x9bc, 0x9be, 0x999, 0x9cb, 0x9a8, 0x9af, 0x9bc,
0x9c7, 0x995, 0x9be, 0x99f, 0x9c7, 0x9f0, 0x9bf, 0x9a3, 0x9ac, 0x9be, 0x9f0, 0x9cd,
0x997, 0x9af, 0x9bc, 0x9c7, 0x9f0, 0x9c7, 0x9ad, 0x9be, 0x9a8, 0x98f, 0x99c, 0x9cb,
0x9f0, 0x9c7, 0x99b, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x9ae, 0x9c1, 0x9a1, 0x9be, 0x995,
0x9c7, 0x9a8, 0x9c7, 0x9f0, 0x9c0, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9f0,
0x9cd, 0x9a6, 0x9c7, 0x9ab, 0x9be, 0x9f0, 0x9cb, 0x9ae, 0x9be, 0x9a1, 0x9c7, 0x9f0,
0x9be, 0x9f0, 0x9c7, 0x987, 0x995, 0x99c, 0x9be, 0x9ad, 0x9bf, 0x995, 0x9a6, 0x995,
0x9cd, 0x9b7, 0x9bf, 0x9a3, 0x20, 0x99c, 0x9f0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd, 0x99f, 0x20, 0x9b9, 0x9c7, 0x9b2, 0x9c7,
0x9a8, 0x9be, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x200c, 0x9b2, 0x9c0, 0x98f,
0x9a1, 0x9bf, 0x9b2, 0x9c7, 0x987, 0x9a1, 0x9ac, 0x9cd, 0x9f0, 0x9bf, 0x99a, 0x9ac,
0x9c7, 0x9a8, 0x9ac, 0x9cd, 0x9f0, 0x9cb, 0x995, 0x9c7, 0x9a8, 0x20, 0x9b9, 0x9bf,
0x9b2, 0x9a1, 0x9be, 0x9f0, 0x989, 0x987, 0x9a8, 0x987, 0x989, 0x995, 0x9cd, 0x9b2,
0x9be, 0x9b9, 0x9cb, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x99f, 0x9b2, 0x9bf, 0x9a3, 0x9cd,
0x9a1, 0x9ae, 0x9be, 0x9a8, 0x9b2, 0x9f0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993,
0x9ae, 0x9c7, 0x9b2, 0x9ac, 0x2019, 0x9f0, 0x9cd, 0x9a3, 0x9aa, 0x9be, 0x9f0, 0x9cd,
0x9a5, 0x99a, 0x9bf, 0x9a1, 0x9a8, 0x9c0, 0x986, 0x9ae, 0x9b7, 0x9cd, 0x99f, 0x9be,
0x9f0, 0x9a1, 0x9be, 0x9ae, 0x986, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x9f0, 0x9be, 0x986,
0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9be, 0x996, 0x9be, 0x9a8, 0x98f, 0x9a5, 0x9c7,
0x9a8, 0x9cd, 0x9b8, 0x9ac, 0x9c7, 0x9b2, 0x997, 0x9cd, 0x9f0, 0x9c7, 0x9a1, 0x9ac,
0x9be, 0x9f0, 0x9cd, 0x9b2, 0x9bf, 0x9a8, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x99f, 0x9bf,
0x99b, 0x9b2, 0x9be, 0x9ad, 0x9be, 0x9ac, 0x9cd, 0x9f0, 0x9c1, 0x99b, 0x9b2, 0x9c7,
0x99b, 0x9ac, 0x9c1, 0x99a, 0x9be, 0x9f0, 0x9c7, 0x9b7, 0x9cd, 0x99f, 0x9ac, 0x9c1,
0x9a1, 0x9be, 0x9aa, 0x9c7, 0x9b7, 0x9cd, 0x99f, 0x9ac, 0x9c1, 0x99b, 0x9bf, 0x9a8,
0x99c, 0x9c7, 0x9a8, 0x99a, 0x9bf, 0x99b, 0x9bf, 0x9a8, 0x9be, 0x989, 0x995, 0x9cb,
0x9aa, 0x9c7, 0x9a8, 0x9b9, 0x9be, 0x997, 0x9c7, 0x9a8, 0x9a1, 0x9be, 0x9ac, 0x9b2,
0x9bf, 0x9a8, 0x99c, 0x9bf, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x9b2, 0x9cd, 0x99f, 0x9f0,
0x997, 0x9cb, 0x9f0, 0x9c7, 0x9a8, 0x99a, 0x9bf, 0x9b9, 0x9c7, 0x9b2, 0x99b, 0x9bf,
0x982, 0x995, 0x9bf, 0x986, 0x987, 0x9b2, 0x20, 0x985, 0x9ab, 0x20, 0x9ae, 0x9c7,
0x9a8, 0x987, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9ac, 0x9c1, 0x9b2, 0x99c, 0x9be,
0x9f0, 0x9cd, 0x99a, 0x9bf, 0x995, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9bf, 0x9a8, 0x997,
0x9cd, 0x9f0, 0x9be, 0x9a1, 0x995, 0x9bf, 0x9f0, 0x9cb, 0x9ad, 0x995, 0x9bf, 0x9ad,
0x9b2, 0x9bf, 0x99b, 0x9ac, 0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9c1, 0x9ac, 0x9c7, 0x9b2,
0x99c, 0x9be, 0x9a8, 0x9be, 0x9b2, 0x9a3, 0x9cd, 0x9a1, 0x9a8, 0x9b2, 0x9be, 0x995,
0x9cd, 0x9b8, 0x9c7, 0x9ae, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x9ae, 0x9be, 0x9a6,
0x9cd, 0x9f0, 0x9bf, 0x9a6, 0x9ae, 0x9be, 0x9b2, 0x9cd, 0x99f, 0x9be, 0x9ae, 0x9be,
0x9f0, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9b9, 0x9c7, 0x9ae, 0x9ae, 0x9bf, 0x982, 0x9b8,
0x9cd, 0x995, 0x9ae, 0x9cb, 0x9a8, 0x9be, 0x995, 0x9cb, 0x9ae, 0x9b8, 0x9cd, 0x995,
0x9cb, 0x993, 0x9b8, 0x9cd, 0x9b2, 0x9cb, 0x9aa, 0x9c7, 0x9f0, 0x9bf, 0x99b, 0x9aa,
0x9cb, 0x9a1, 0x997, 0x9cb, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9aa, 0x9cd, 0x9f0, 0x9be,
0x997, 0x9f0, 0x9bf, 0x997, 0x9be, 0x99b, 0x9be, 0x9ae, 0x9be, 0x9f0, 0x9be, 0x99a,
0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9f0, 0x9bf, 0x9a8, 0x9cb, 0x99b, 0x9be, 0x9f0,
0x9be, 0x99c, 0x9c7, 0x9ad, 0x9cb, 0x99b, 0x9be, 0x9f0, 0x9be, 0x99f, 0x9cb, 0x9ad,
0x99b, 0x9bf, 0x9ae, 0x9cd, 0x9ab, 0x9c7, 0x9f0, 0x9cb, 0x9aa, 0x9cb, 0x9b2, 0x9b8,
0x9cd, 0x995, 0x9cb, 0x9aa, 0x9cd, 0x99c, 0x9c7, 0x99b, 0x9cb, 0x9ab, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9b7, 0x9cd, 0x99f, 0x995, 0x9b9, 0x9cb, 0x9ae, 0x9a4, 0x9c7, 0x9b2,
0x9bf, 0x9a8, 0x99f, 0x9be, 0x987, 0x9f0, 0x9c7, 0x9a8, 0x989, 0x9b2, 0x9cd, 0x9af,
0x9be, 0x9a8, 0x9cb, 0x9ad, 0x9cd, 0x200c, 0x9b8, 0x9cd, 0x995, 0x9ad, 0x9be, 0x9a6,
0x9c1, 0x99c, 0x9ad, 0x9c7, 0x99f, 0x9bf, 0x995, 0x9be, 0x9a8, 0x9ad, 0x9bf, 0x9af,
0x9bc, 0x9c7, 0x9a8, 0x9be, 0x9ad, 0x9bf, 0x9b2, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x99b, 0x9ad, 0x9cb, 0x9b2, 0x9cd, 0x997, 0x9cb, 0x997, 0x9cd, 0x9f0, 0x9be, 0x9a1,
0x9f1, 0x9be, 0x9f0, 0x99b, 0x9be, 0x993, 0x99c, 0x9be, 0x997, 0x9cd, 0x9f0, 0x9c7,
0x9ac, 0x99c, 0x9c1, 0x9f0, 0x9bf, 0x996, 0x98f, 0x9a3, 0x9cd, 0x99f, 0x9be, 0x9a8,
0x9be, 0x9a8, 0x9be, 0x9f0, 0x9bf, 0x9ad, 0x9cb, 0x99a, 0x9be, 0x997, 0x9cb, 0x99b,
0x996, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x99f, 0x9ae, 0x9be, 0x99b, 0x995, 0x9cb,
0x995, 0x9cb, 0x99b, 0x995, 0x9cb, 0x9ae, 0x9cb, 0x9f0, 0x9cb, 0x995, 0x9c7, 0x9f0,
0x997, 0x9c1, 0x9b2, 0x9c7, 0x9a8, 0x9ae, 0x9be, 0x9b9, 0x9c7, 0x9ae, 0x9be, 0x9b2,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9ae, 0x9f0, 0x9bf, 0x99b, 0x9be, 0x99b, 0x9ae,
0x9be, 0x9af, 0x9bc, 0x9cb, 0x99f, 0x9cd, 0x99f, 0x9c7, 0x9f0, 0x9bf, 0x987, 0x989,
0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9a8, 0x986, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x985,
0x995, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9ac, 0x9cb, 0x997, 0x9c7, 0x987, 0x9a8,
0x9ad, 0x9bf, 0x9b2, 0x9c7, 0x99a, 0x9be, 0x9a5, 0x9be, 0x9ae, 0x99a, 0x9cd, 0x99a,
0x9c1, 0x995, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x98f, 0x9ab, 0x9c7, 0x99f,
0x9ab, 0x9be, 0x995, 0x9be, 0x993, 0x9ab, 0x9cb, 0x9ab, 0x9bf, 0x99c, 0x9bf, 0x9ab,
0x9c1, 0x9a8, 0x9be, 0x9ab, 0x9c1, 0x99f, 0x9bf, 0x997, 0x9be, 0x9b2, 0x9be, 0x9aa,
0x9be, 0x997, 0x9cb, 0x99b, 0x997, 0x9c7, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9f0, 0x997, 0x9c1, 0x9f1, 0x9be, 0x9a1, 0x9b2, 0x995, 0x9be, 0x9a8, 0x9be,
0x9b2, 0x98f, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x9ac, 0x9be, 0x9f0, 0x9c0, 0x995, 0x9bf,
0x9f0, 0x9bf, 0x99f, 0x9bf, 0x9ae, 0x9be, 0x99f, 0x9bf, 0x995, 0x9cb, 0x99b, 0x9f0,
0x9be, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9cb, 0x9f1, 0x9be, 0x99c, 0x9be, 0x9b2, 0x9bf,
0x9a8, 0x9ae, 0x9be, 0x99c, 0x9c1, 0x9f0, 0x9cb, 0x9ae, 0x9be, 0x9f0, 0x9cd, 0x995,
0x99b, 0x9be, 0x99b, 0x9ae, 0x9bf, 0x9a1, 0x9f1, 0x9c7, 0x9a8, 0x9be, 0x9f0, 0x9c1,
0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9cb, 0x9a8, 0x2019, 0x9f0, 0x9ab, 0x2019, 0x995, 0x9a8,
0x989, 0x9ae, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9aa, 0x9be, 0x997, 0x9cb, 0x20, 0x9aa,
0x9be, 0x997, 0x9cb, 0x9aa, 0x9be, 0x9b2, 0x9be, 0x989, 0x9aa, 0x9bf, 0x99f, 0x995,
0x9c7, 0x987, 0x9f0, 0x9cd, 0x9a3, 0x9aa, 0x9cb, 0x9a8, 0x9aa, 0x9c7, 0x987, 0x9aa,
0x2019, 0x9f0, 0x9cd, 0x99f, 0x20, 0x9ae, 0x9cb, 0x9f0, 0x9c7, 0x99b, 0x9cd, 0x200c,
0x9ac, 0x9bf, 0x9f0, 0x9be, 0x9f0, 0x9cb, 0x99f, 0x9cb, 0x999, 0x9cd, 0x997, 0x9be,
0x99b, 0x9be, 0x987, 0x9aa, 0x9be, 0x9a8, 0x9a4, 0x9be, 0x9b9, 0x9bf, 0x9a4, 0x9bf,
0x9a4, 0x9be, 0x9f0, 0x9be, 0x9f1, 0x9be, 0x99f, 0x982, 0x997, 0x9be, 0x99f, 0x9be,
0x9aa, 0x9c1, 0x9f1, 0x9c7, 0x995, 0x9f1, 0x9be, 0x9b2, 0x9bf, 0x99b, 0x41, 0x64,
0xed, 0x73, 0x20, 0x41, 0x62, 0x65, 0x62, 0x61, 0x41, 0x72, 0x78, 0x65,
0x6c, 0x45, 0x6c, 0x20, 0x43, 0x61, 0x69, 0x72, 0x75, 0x58, 0x69, 0x62,
0x75, 0x74, 0x69, 0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62,
0x75, 0x72, 0x67, 0x75, 0x1e24, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x4c, 0x6f,
0x6d, 0xe9, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x78, 0x75, 0x58, 0x61,
0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x69, 0x61, 0x6d, 0xe9, 0x69, 0x4e, 0x75,
0x61, 0x6b, 0x78, 0x6f, 0x74, 0x55, 0x61, 0x67, 0x61, 0x64, 0x75, 0x67,
0xfa, 0x53, 0x61, 0x6e, 0x74, 0x75, 0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x54,
0x72, 0xed, 0x70, 0x6f, 0x6c, 0x69, 0x54, 0xfa, 0x6e, 0x65, 0x7a, 0x41,
0x6e, 0x67, 0x75, 0x69, 0x6c, 0x61, 0x43, 0xf3, 0x72, 0x64, 0x6f, 0x62,
0x61, 0x54, 0x75, 0x63, 0x75, 0x6d, 0xe1, 0x6e, 0x42, 0x61, 0x68, 0xed,
0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x42, 0x6f,
0x67, 0x6f, 0x74, 0xe1, 0x43, 0x61, 0x69, 0x6d, 0xe1, 0x6e, 0x47, 0x72,
0x61, 0x6e, 0x61, 0x64, 0x61, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x75,
0x70, 0x65, 0x4c, 0x61, 0x20, 0x48, 0x61, 0x62, 0x61, 0x6e, 0x61, 0x58,
0x61, 0x6d, 0x61, 0x69, 0x63, 0x61, 0x4c, 0x61, 0x20, 0x4d, 0x61, 0x72,
0x74, 0x69, 0x6e, 0x69, 0x63, 0x61, 0x43, 0x69, 0x75, 0x64, 0xe1, 0x20,
0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x75, 0x4e, 0x75, 0x65,
0x76, 0x61, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x65,
0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x4e, 0x75, 0x65, 0x76, 0x61,
0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65,
0x50, 0x61, 0x6e, 0x61, 0x6d, 0xe1, 0x50, 0x75, 0x65, 0x72, 0x74, 0x75,
0x20, 0x50, 0x72, 0xed, 0x6e, 0x63, 0x69, 0x70, 0x65, 0x50, 0x75, 0x65,
0x72, 0x74, 0x75, 0x20, 0x45, 0x73, 0x70, 0x61, 0xf1, 0x61, 0x50, 0x75,
0x65, 0x72, 0x74, 0x75, 0x20, 0x52, 0x69, 0x63, 0x75, 0x53, 0x61, 0x6e,
0x74, 0x75, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x75, 0x54, 0xf3,
0x72, 0x74, 0x6f, 0x6c, 0x61, 0x41, 0x6d, 0xe1, 0x6e, 0x42, 0x61, 0x67,
0x64, 0x61, 0x67, 0x42, 0x61, 0x1e25, 0x72, 0xe9, 0x69, 0x6e, 0x42, 0x61,
0x6b, 0xfa, 0x42, 0x72, 0x75, 0x6e, 0xe9, 0x69, 0x44, 0x61, 0x6d, 0x61,
0x73, 0x63, 0x75, 0x44, 0x75, 0x78, 0x61, 0x6e, 0x62, 0xe9, 0x48, 0x65,
0x62, 0x72, 0xf3, 0x6e, 0x43, 0x69, 0x75, 0x64, 0xe1, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68,
0x1e24, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x58, 0x61, 0x6b,
0x61, 0x72, 0x74, 0x61, 0x58, 0x61, 0x79, 0x61, 0x70, 0x75, 0x72, 0x61,
0x58, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0xe9, 0x6e, 0x4b, 0x61, 0x74,
0x6d, 0x61, 0x6e, 0x64, 0xfa, 0x43, 0x61, 0x6c, 0x63, 0x75, 0x74, 0x61,
0x4d, 0x61, 0x63, 0xe1, 0x75, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e,
0x4d, 0x61, 0x73, 0x63, 0x61, 0x74, 0x65, 0x50, 0x79, 0x6f, 0x6e, 0x67,
0x20, 0x59, 0x61, 0x6e, 0x67, 0x52, 0x69, 0x79, 0x61, 0x64, 0x53, 0x61,
0x78, 0x61, 0x6c, 0xed, 0x6e, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x63, 0x61,
0x6e, 0x64, 0x61, 0x53, 0x65, 0xfa, 0x6c, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x75, 0x72, 0x54, 0x65, 0x1e25, 0x72, 0xe1, 0x6e, 0x54, 0x6f, 0x6b,
0x69, 0x75, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0xe1, 0x74, 0x6f, 0x72,
0x56, 0x69, 0x65, 0x6e, 0x74, 0x69, 0xe1, 0x6e, 0x52, 0x61, 0x6e, 0x67,
0xfa, 0x6e, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62,
0x75, 0x72, 0x67, 0x75, 0x4c, 0x65, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x65, 0x73, 0x43, 0x61,
0x62, 0x75, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x49, 0x73, 0x6c, 0x6c,
0x65, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6f, 0x65, 0x52, 0x65, 0x69, 0x6b,
0x69, 0x61, 0x76, 0x69, 0x6b, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x72, 0x53, 0x61, 0x6e, 0x74, 0x61,
0x20, 0x4c, 0x65, 0x6e, 0x61, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x69, 0x64,
0x61, 0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0xe1, 0x6e, 0x41, 0x74, 0x65,
0x6e, 0x65, 0x73, 0x42, 0x65, 0x6c, 0x67, 0x72, 0xe1, 0x75, 0x42, 0x65,
0x72, 0x6c, 0xed, 0x6e, 0x42, 0x72, 0x75, 0x78, 0x65, 0x6c, 0x65, 0x73,
0x42, 0x75, 0x63, 0x61, 0x72, 0x65, 0x73, 0x74, 0x43, 0x6f, 0x70, 0x65,
0x6e, 0x68, 0x61, 0x67, 0x75, 0x65, 0x44, 0x75, 0x62, 0x6c, 0xed, 0x6e,
0x58, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x1e24, 0xe9, 0x6c,
0x73, 0x69, 0x6e, 0x6b, 0x69, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e,
0x67, 0x72, 0xe1, 0x75, 0x4c, 0x69, 0x73, 0x62, 0x6f, 0x61, 0x4c, 0x69,
0x75, 0x62, 0x6c, 0x69, 0x61, 0x6e, 0x61, 0x4c, 0x6f, 0x6e, 0x64, 0x72,
0x65, 0x73, 0x4c, 0x75, 0x78, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x75,
0x4d, 0xf3, 0x6e, 0x61, 0x63, 0x75, 0x4d, 0x6f, 0x73, 0x63, 0xfa, 0x50,
0x61, 0x72, 0xed, 0x73, 0x50, 0x72, 0x61, 0x67, 0x61, 0x52, 0x6f, 0x6d,
0x61, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0xed, 0x6e, 0x53, 0x6f,
0x66, 0xed, 0x61, 0x45, 0x73, 0x74, 0x6f, 0x63, 0x6f, 0x6c, 0x6d, 0x75,
0x54, 0x69, 0x72, 0x61, 0x6e, 0x61, 0x45, 0x6c, 0x20, 0x56, 0x61, 0x74,
0x69, 0x63, 0x61, 0x6e, 0x75, 0x56, 0x69, 0x65, 0x6e, 0x61, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1, 0x75, 0x56, 0x61, 0x72, 0x73, 0x6f,
0x76, 0x69, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x75, 0x52,
0x65, 0x75, 0x6e, 0x69, 0xf3, 0x6e, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61,
0x46, 0x69, 0x78, 0x69, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f,
0x73, 0x41, 0x62, 0x69, 0x63, 0x61, 0x6e, 0x41, 0x6b, 0x6b, 0x72, 0x61,
0x18f, 0x64, 0x64, 0x69, 0x73, 0x20, 0x18f, 0x62, 0x259, 0x62, 0x259, 0x18f,
0x6c, 0x63, 0x259, 0x7a, 0x61, 0x69, 0x72, 0x18f, 0x73, 0x6d, 0x259, 0x72,
0x259, 0x42, 0x61, 0x6e, 0x71, 0x75, 0x69, 0x42, 0x61, 0x6e, 0x63, 0x75,
0x6c, 0x42, 0x72, 0x61, 0x7a, 0x7a, 0x61, 0x76, 0x69, 0x6c, 0x51, 0x61,
0x68, 0x69, 0x72, 0x259, 0x4b, 0x61, 0x73, 0x61, 0x62, 0x6c, 0x61, 0x6e,
0x6b, 0x61, 0x53, 0x65, 0x75, 0x74, 0x61, 0x44, 0x61, 0x72, 0x20, 0x65,
0x73, 0x20, 0x53, 0x61, 0x6c, 0x61, 0x6d, 0x43, 0x69, 0x62, 0x75, 0x74,
0x69, 0x44, 0x75, 0x61, 0x6c, 0x61, 0x18f, 0x6c, 0x20, 0x18f, 0x79, 0x75,
0x6e, 0x51, 0x61, 0x62, 0x6f, 0x72, 0x6f, 0x6e, 0x59, 0x6f, 0x68, 0x61,
0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x71, 0x58, 0x61, 0x72, 0x74, 0x75,
0x6d, 0x4b, 0x69, 0x71, 0x61, 0x6c, 0x69, 0x4b, 0x69, 0x6e, 0x15f, 0x61,
0x73, 0x61, 0x4c, 0x61, 0x71, 0x6f, 0x73, 0x4c, 0x75, 0x62, 0x75, 0x6d,
0x62, 0x61, 0x15f, 0x69, 0x4d, 0x6f, 0x71, 0x61, 0x64, 0x69, 0x15f, 0x75,
0x4e, 0x63, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x6f, 0x75, 0x61, 0x6b,
0x15f, 0x6f, 0x74, 0x55, 0x71, 0x61, 0x64, 0x75, 0x71, 0x75, 0x53, 0x61,
0x6e, 0x20, 0x54, 0x6f, 0x6d, 0x56, 0x69, 0x6e, 0x64, 0x68, 0x75, 0x6b,
0x41, 0x6e, 0x6b, 0x6f, 0x72, 0x61, 0x63, 0x41, 0x6e, 0x67, 0x69, 0x6c,
0x79, 0x61, 0x41, 0x6e, 0x74, 0x69, 0x71, 0x75, 0x61, 0x41, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x6e, 0x61, 0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73,
0x20, 0x41, 0x79, 0x72, 0x65, 0x73, 0x52, 0x69, 0x6f, 0x78, 0x61, 0x52,
0x69, 0x6f, 0x20, 0x51, 0x61, 0x6c, 0x79, 0x65, 0x71, 0x6f, 0x73, 0x53,
0x61, 0x6e, 0x20, 0x58, 0x75, 0x61, 0x6e, 0x55, 0x15f, 0x75, 0x61, 0x79,
0x61, 0x42, 0x61, 0x68, 0x69, 0x79, 0x61, 0x42, 0x6f, 0x71, 0x6f, 0x74,
0x61, 0x42, 0x6f, 0x79, 0x73, 0x65, 0x4b, 0x65, 0x6d, 0x62, 0x72, 0x69,
0x63, 0x20, 0x4b, 0xf6, 0x72, 0x66, 0x259, 0x7a, 0x69, 0x4b, 0x61, 0x6d,
0x70, 0x6f, 0x20, 0x51, 0x72, 0x61, 0x6e, 0x64, 0x65, 0x4b, 0x61, 0x79,
0x65, 0x6e, 0x4b, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0xc7, 0x69, 0x6b, 0x61,
0x71, 0x6f, 0xc7, 0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0x4b, 0x6f,
0x73, 0x74, 0x61, 0x20, 0x52, 0x69, 0x6b, 0x61, 0x4b, 0x75, 0x79, 0x61,
0x62, 0x61, 0x44, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x15f, 0x61, 0x76,
0x6e, 0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x20, 0x4b, 0x72, 0x69, 0x6b,
0x44, 0x65, 0x74, 0x72, 0x6f, 0x79, 0x74, 0x44, 0x6f, 0x6d, 0x69, 0x6e,
0x69, 0x6b, 0x61, 0x45, 0x64, 0x6d, 0x6f, 0x6e, 0x64, 0x74, 0x6f, 0x6e,
0x130, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x65, 0x51, 0x6c, 0x65, 0x79, 0x73,
0x20, 0x4b, 0xf6, 0x72, 0x66, 0x259, 0x7a, 0x69, 0x51, 0x75, 0x7a, 0x20,
0x4b, 0xf6, 0x72, 0x66, 0x259, 0x7a, 0x69, 0x51, 0x72, 0x61, 0x6e, 0x64,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x51, 0x72, 0x65, 0x6e, 0x61, 0x64, 0x61,
0x51, 0x76, 0x61, 0x64, 0x65, 0x6c, 0x75, 0x70, 0x61, 0x51, 0x76, 0x61,
0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61, 0x51, 0x75, 0x61, 0x79, 0x61, 0x6b,
0x69, 0x6c, 0x51, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x48, 0x65, 0x72, 0x6d,
0x6f, 0x73, 0x69, 0x6c, 0x6f, 0x130, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x70, 0x6f, 0x6c, 0x69, 0x73, 0x4e, 0x6f, 0x6b, 0x73, 0x4d, 0x61, 0x72,
0x65, 0x6e, 0x71, 0x6f, 0x50, 0x69, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75,
0x72, 0x71, 0x56, 0x69, 0x76, 0x65, 0x79, 0x56, 0x69, 0x6e, 0x73, 0x65,
0x6e, 0x56, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x130, 0x6e, 0x75, 0x76,
0x69, 0x6b, 0x130, 0x71, 0x61, 0x6c, 0x75, 0x69, 0x74, 0x59, 0x61, 0x6d,
0x61, 0x79, 0x6b, 0x61, 0x43, 0x75, 0x6e, 0x6f, 0x4c, 0x75, 0x69, 0x73,
0x76, 0x69, 0x6c, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0xe7, 0x65, 0x6c, 0x6c,
0x6f, 0x4b, 0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x6b, 0x4c, 0x61,
0x20, 0x50, 0x61, 0x73, 0x4c, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x63, 0x65,
0x6c, 0x65, 0x73, 0x41, 0x15f, 0x61, 0x11f, 0x131, 0x20, 0x50, 0x72, 0x69,
0x6e, 0x73, 0x20, 0x4b, 0x76, 0x61, 0x72, 0x74, 0x61, 0x6c, 0x131, 0x4d,
0x61, 0x73, 0x65, 0x79, 0x6f, 0x4d, 0x61, 0x6e, 0x61, 0x71, 0x75, 0x61,
0x4d, 0x61, 0x72, 0x69, 0x71, 0x6f, 0x74, 0x4d, 0x61, 0x7a, 0x61, 0x6c,
0x74, 0x61, 0x6e, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x4d, 0x6f, 0x6e,
0x74, 0x65, 0x72, 0x65, 0x79, 0x4d, 0x6f, 0x6e, 0x73, 0x65, 0x72, 0x61,
0x74, 0x4e, 0x79, 0x75, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x4e, 0x6f, 0x6d,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x2c, 0x20,
0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x4e, 0x79, 0x75, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x4f, 0x63, 0x69,
0x6e, 0x61, 0x71, 0x61, 0x130, 0x73, 0x70, 0x61, 0x6e, 0x20, 0x4c, 0x69,
0x6d, 0x61, 0x6e, 0x131, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x56, 0x65,
0x6c, 0x79, 0x6f, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69,
0x6b, 0x6f, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x47, 0x69, 0x72,
0x69, 0x15f, 0x69, 0x52, 0x65, 0x73, 0x69, 0x66, 0x52, 0x65, 0x63, 0x69,
0x6e, 0x61, 0x52, 0x65, 0x7a, 0x6f, 0x6c, 0x79, 0x75, 0x74, 0x52, 0x69,
0x6f, 0x20, 0x42, 0x72, 0x61, 0x6e, 0x6b, 0x6f, 0x53, 0x61, 0x6e, 0x74,
0x79, 0x61, 0x71, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x44, 0x6f,
0x6d, 0x69, 0x6e, 0x71, 0x6f, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x75,
0x6c, 0x6f, 0x53, 0x6b, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x69, 0x73, 0x75,
0x6e, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x2d, 0x42, 0x61, 0x72, 0x74, 0x65,
0x6c, 0x65, 0x6d, 0x69, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x43, 0x6f, 0x6e,
0x73, 0x53, 0x61, 0x6e, 0x20, 0x4b, 0x69, 0x74, 0x73, 0x53, 0x61, 0x6e,
0x20, 0x4c, 0xfc, 0x73, 0x69, 0x79, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x54,
0x6f, 0x6d, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x56, 0x69, 0x6e, 0x73,
0x65, 0x6e, 0x74, 0x53, 0x76, 0x69, 0x66, 0x74, 0x20, 0x4b, 0x75, 0x72,
0x65, 0x6e, 0x74, 0x54, 0x65, 0x71, 0x75, 0x73, 0x69, 0x71, 0x61, 0x6c,
0x70, 0x61, 0x54, 0x75, 0x6c, 0x54, 0x69, 0x78, 0x75, 0x61, 0x6e, 0x61,
0x55, 0x61, 0x79, 0x74, 0x68, 0x6f, 0x72, 0x73, 0x56, 0x69, 0x6e, 0x6e,
0x69, 0x70, 0x65, 0x71, 0x4b, 0x65, 0x79, 0x73, 0x69, 0x44, 0x65, 0x79,
0x76, 0x69, 0x73, 0x4d, 0x61, 0x6b, 0x75, 0x61, 0x72, 0x69, 0x4d, 0x6f,
0x75, 0x73, 0x6f, 0x6e, 0x4d, 0x61, 0x6b, 0x20, 0x4d, 0x75, 0x72, 0x64,
0x6f, 0x53, 0x79, 0x6f, 0x76, 0x61, 0x4c, 0x6f, 0x6e, 0x71, 0x79, 0x69,
0x72, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x131, 0x41, 0x6e, 0x61, 0x64, 0x131,
0x72, 0x41, 0x15f, 0x71, 0x61, 0x62, 0x61, 0x74, 0x41, 0x74, 0x131, 0x72,
0x61, 0x75, 0x42, 0x61, 0x11f, 0x64, 0x61, 0x64, 0x42, 0x259, 0x68, 0x72,
0x65, 0x79, 0x6e, 0x42, 0x61, 0x6b, 0x131, 0x42, 0x61, 0x6e, 0x71, 0x6b,
0x6f, 0x6b, 0x42, 0x65, 0x79, 0x72, 0x75, 0x74, 0x42, 0x69, 0x15f, 0x6b,
0x65, 0x6b, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x79, 0xc7, 0x6f, 0x79, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x44, 0x259, 0x6d, 0x259, 0x15f, 0x71, 0x44,
0x259, 0x6b, 0x6b, 0x259, 0x44, 0x75, 0x62, 0x61, 0x79, 0x44, 0xfc, 0x15f,
0x259, 0x6e, 0x62, 0x259, 0x46, 0x61, 0x6d, 0x61, 0x71, 0x75, 0x73, 0x74,
0x61, 0x51, 0x259, 0x7a, 0x61, 0x48, 0x6f, 0x20, 0x15e, 0x69, 0x20, 0x4d,
0x69, 0x6e, 0x48, 0x6f, 0x6e, 0x71, 0x20, 0x4b, 0x6f, 0x6e, 0x71, 0x130,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x43, 0x61, 0x6b, 0x61, 0x72, 0x74,
0x61, 0x43, 0x61, 0x79, 0x61, 0x70, 0x75, 0x72, 0x61, 0x59, 0x65, 0x72,
0x75, 0x73, 0x259, 0x6c, 0x69, 0x6d, 0x4b, 0x61, 0x62, 0x69, 0x6c, 0x4b,
0x61, 0x6d, 0xe7, 0x61, 0x74, 0x6b, 0x61, 0x58, 0x61, 0x6e, 0x64, 0x131,
0x71, 0x61, 0x4b, 0x259, 0x6c, 0x6b, 0x259, 0x74, 0x259, 0x4b, 0x75, 0xe7,
0x69, 0x6e, 0x71, 0x4b, 0xfc, 0x76, 0x65, 0x79, 0x74, 0x4d, 0x61, 0x71,
0x61, 0x64, 0x61, 0x6e, 0x4e, 0x69, 0x6b, 0x6f, 0x73, 0x69, 0x79, 0x61,
0x50, 0x6e, 0x6f, 0x6d, 0x20, 0x50, 0x65, 0x6e, 0x50, 0x78, 0x65, 0x6e,
0x79, 0x61, 0x6e, 0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x53, 0x61, 0x78, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x259, 0x6d, 0x259, 0x72,
0x71, 0x259, 0x6e, 0x64, 0x15e, 0x61, 0x6e, 0x78, 0x61, 0x79, 0x53, 0x69,
0x6e, 0x71, 0x61, 0x70, 0x75, 0x72, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x65,
0x6b, 0x6f, 0x6c, 0x131, 0x6d, 0x73, 0x6b, 0x54, 0x61, 0x79, 0x62, 0x65,
0x79, 0x44, 0x61, 0x15f, 0x6b, 0x259, 0x6e, 0x64, 0x55, 0x6c, 0x61, 0x6e,
0x62, 0x61, 0x74, 0x6f, 0x72, 0x55, 0x72, 0x75, 0x6d, 0xe7, 0x69, 0x56,
0x79, 0x65, 0x6e, 0x74, 0x79, 0x61, 0x6e, 0x52, 0x61, 0x6e, 0x71, 0x75,
0x6e, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x71, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x20, 0x61, 0x64, 0x61,
0x6c, 0x61, 0x72, 0x131, 0x4b, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x46, 0x61, 0x72, 0x65, 0x72, 0x4d, 0x61, 0x64, 0x65, 0x79,
0x72, 0x61, 0x52, 0x65, 0x79, 0x6b, 0x79, 0x61, 0x76, 0x69, 0x6b, 0x43,
0x259, 0x6e, 0x75, 0x62, 0x69, 0x20, 0x43, 0x6f, 0x72, 0x63, 0x69, 0x79,
0x61, 0x4d, 0xfc, 0x71, 0x259, 0x64, 0x64, 0x259, 0x73, 0x20, 0x59, 0x65,
0x6c, 0x65, 0x6e, 0x61, 0x42, 0x72, 0x69, 0x73, 0x62, 0x65, 0x79, 0x6e,
0x59, 0x75, 0x6b, 0x6c, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x61,
0x75, 0x50, 0x65, 0x72, 0x74, 0x53, 0x69, 0x64, 0x6e, 0x65, 0x79, 0x48,
0x259, 0x15f, 0x74, 0x259, 0x72, 0x78, 0x61, 0x6e, 0x41, 0x66, 0x69, 0x6e,
0x61, 0x42, 0x65, 0x6c, 0x71, 0x72, 0x61, 0x64, 0x42, 0x72, 0xfc, 0x73,
0x73, 0x65, 0x6c, 0x42, 0x75, 0x78, 0x61, 0x72, 0x65, 0x73, 0x74, 0x42,
0x75, 0x64, 0x61, 0x70, 0x65, 0x15f, 0x74, 0x4b, 0x69, 0x15f, 0x69, 0x6e,
0x79, 0x6f, 0x76, 0x43, 0x259, 0x62, 0x259, 0x6c, 0x69, 0x2d, 0x54, 0x61,
0x72, 0x69, 0x71, 0x47, 0x65, 0x72, 0x6e, 0x7a, 0x65, 0x79, 0x4d, 0x65,
0x6e, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x130, 0x73, 0x74, 0x61, 0x6e,
0x62, 0x75, 0x6c, 0x43, 0x65, 0x72, 0x73, 0x69, 0x4b, 0x61, 0x6c, 0x69,
0x6e, 0x69, 0x6e, 0x71, 0x72, 0x61, 0x64, 0x4b, 0x69, 0x79, 0x65, 0x76,
0x4c, 0x79, 0x75, 0x62, 0x6c, 0x79, 0x61, 0x6e, 0x61, 0x4c, 0xfc, 0x6b,
0x73, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x71, 0x4d, 0x61, 0x72, 0x69, 0x68,
0x61, 0x6d, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x50, 0x6f, 0x64, 0x71,
0x6f, 0x72, 0x69, 0x74, 0x73, 0x61, 0x50, 0x72, 0x61, 0x71, 0x61, 0x52,
0x69, 0x71, 0x61, 0x53, 0x61, 0x72, 0x61, 0x79, 0x65, 0x76, 0x6f, 0x53,
0x6b, 0x6f, 0x70, 0x79, 0x65, 0x53, 0x6f, 0x66, 0x69, 0x79, 0x61, 0x54,
0x61, 0x6c, 0x6c, 0x69, 0x6e, 0x56, 0x61, 0x64, 0x75, 0x74, 0x73, 0x56,
0x79, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6c, 0x6e, 0xfc, 0x73, 0x56, 0x6f,
0x6c, 0x71, 0x6f, 0x71, 0x72, 0x61, 0x64, 0x56, 0x61, 0x72, 0x15f, 0x61,
0x76, 0x61, 0x5a, 0x61, 0x71, 0x72, 0x65, 0x62, 0x53, 0xfc, 0x72, 0x69,
0x78, 0xc7, 0x61, 0x71, 0x6f, 0x73, 0x4d, 0x69, 0x6c, 0x61, 0x64, 0x4b,
0x6f, 0x6d, 0x6f, 0x72, 0x6f, 0x4b, 0x65, 0x72, 0x67, 0x65, 0x6c, 0x65,
0x6e, 0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x4d, 0x61, 0x79, 0x6f,
0x74, 0x4f, 0x6b, 0x6c, 0x259, 0x6e, 0x64, 0x42, 0x75, 0x71, 0x61, 0x6e,
0x76, 0x69, 0x6c, 0xc7, 0x61, 0x74, 0x61, 0x6d, 0xc7, 0x75, 0x75, 0x6b,
0x50, 0x61, 0x73, 0x78, 0x61, 0x46, 0x69, 0x63, 0x69, 0x51, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x71, 0x6f, 0x73, 0x51, 0x61, 0x6d, 0x62, 0x69, 0x79,
0x65, 0x72, 0x51, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61,
0x6c, 0x51, 0x75, 0x61, 0x6d, 0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0xf6,
0x72, 0x69, 0x4b, 0x69, 0x72, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x79, 0x65, 0x4b, 0x76, 0x61, 0x6a, 0x61, 0x6c, 0x65,
0x79, 0x6e, 0x4d, 0x61, 0x63, 0x75, 0x72, 0x6f, 0x4d, 0x61, 0x72, 0x6b,
0x65, 0x73, 0x61, 0x73, 0x4d, 0x69, 0x64, 0x76, 0x65, 0x79, 0x50, 0x61,
0x71, 0x6f, 0x20, 0x50, 0x61, 0x71, 0x6f, 0x50, 0x6f, 0x72, 0x74, 0x20,
0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x69, 0x52, 0x61, 0x72, 0x6f, 0x74,
0x6f, 0x6e, 0x71, 0x61, 0x53, 0x61, 0x79, 0x70, 0x61, 0x6e, 0x54, 0x61,
0x72, 0x61, 0x76, 0x61, 0x54, 0x6f, 0x6e, 0x71, 0x61, 0x70, 0x61, 0x74,
0x75, 0x56, 0x65, 0x79, 0x6b, 0x986, 0x9ac, 0x9bf, 0x9a6, 0x99c, 0x9be, 0x9a8,
0x986, 0x995, 0x9cd, 0x9b0, 0x9be, 0x986, 0x9a6, 0x9cd, 0x9a6, 0x9bf, 0x9b8, 0x20,
0x986, 0x9ac, 0x9be, 0x9ac, 0x9be, 0x986, 0x9b2, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9b0, 0x9cd, 0x9b8, 0x985, 0x9cd, 0x9af, 0x9be, 0x9b8, 0x9ae, 0x9be, 0x9b0, 0x9be,
0x9b9, 0x9c1, 0x9ac, 0x9be, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x995, 0x9cb, 0x9ac, 0x9be,
0x999, 0x9cd, 0x997, 0x9c1, 0x987, 0x9ac, 0x9be, 0x99e, 0x9cd, 0x99c, 0x9c1, 0x9b2,
0x9ac, 0x9bf, 0x9b8, 0x9cb, 0x989, 0x9ac, 0x9cd, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8,
0x99f, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x99c, 0x9be,
0x9ad, 0x9bf, 0x9b2, 0x9bf, 0x9ac, 0x9c1, 0x99c, 0x9c1, 0x9ae, 0x9ac, 0x9c1, 0x9b0,
0x9b9, 0x9c1, 0x995, 0x9be, 0x9af, 0x9bc, 0x9b0, 0x9cb, 0x995, 0x9be, 0x9b8, 0x9be,
0x9ac, 0x9cd, 0x9b2, 0x9be, 0x999, 0x9cd, 0x995, 0x9be, 0x9b8, 0x9c7, 0x989, 0x99f,
0x9be, 0x995, 0x9a8, 0x9be, 0x995, 0x9cd, 0x9b0, 0x9bf, 0x9a1, 0x9be, 0x995, 0x9be,
0x9b0, 0x9a6, 0x9be, 0x9b0, 0x20, 0x98f, 0x9b8, 0x20, 0x9b8, 0x9be, 0x9b2, 0x9be,
0x9ae, 0x9a1, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9b2, 0x9be, 0x98f, 0x9b2, 0x20, 0x986,
0x9b9, 0x987, 0x989, 0x9a8, 0x9ab, 0x9cd, 0x9b0, 0x9c0, 0x99f, 0x9be, 0x989, 0x9a8,
0x997, 0x9cd, 0x9af, 0x9be, 0x9ac, 0x9c1, 0x9b0, 0x9a8, 0x9bf, 0x9b9, 0x9be, 0x9b0,
0x9be, 0x9b0, 0x9c7, 0x99c, 0x9cb, 0x9b9, 0x9be, 0x9a8, 0x9c7, 0x9b8, 0x9ac, 0x9be,
0x9b0, 0x9cd, 0x997, 0x995, 0x9be, 0x9ae, 0x9aa, 0x9be, 0x9b2, 0x9be, 0x996, 0x9be,
0x9b0, 0x9cd, 0x9a4, 0x9c1, 0x9ae, 0x995, 0x9bf, 0x997, 0x9be, 0x9b2, 0x9bf, 0x995,
0x9bf, 0x9a8, 0x9b6, 0x9be, 0x9b8, 0x9be, 0x9b2, 0x9be, 0x997, 0x9cb, 0x9b8, 0x9b2,
0x9bf, 0x9ac, 0x9cd, 0x9b0, 0x9c1, 0x9ad, 0x9bf, 0x9b2, 0x9b2, 0x9cb, 0x9ae, 0x9c7,
0x9b2, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x9be, 0x9b2, 0x9c1, 0x9ac,
0x9c1, 0x9ae, 0x9cd, 0x9ac, 0x9be, 0x9b6, 0x9bf, 0x9b2, 0x9c1, 0x9b8, 0x9be, 0x995,
0x9be, 0x9ae, 0x9be, 0x9aa, 0x9c1, 0x9a4, 0x9cb, 0x9ae, 0x9be, 0x9b9, 0x9b8, 0x9c1,
0x9b0, 0x9c1, 0x985, 0x9ae, 0x9ac, 0x9be, 0x9ac, 0x9be, 0x9a8, 0x9ae, 0x9be, 0x993,
0x997, 0x9be, 0x9a6, 0x9bf, 0x9b8, 0x9c1, 0x9ae, 0x9a8, 0x9b0, 0x9cb, 0x9ad, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x987, 0x9b0, 0x9cb, 0x9ac, 0x9bf, 0x98f, 0x9a8,
0x99c, 0x9ae, 0x9c7, 0x9a8, 0x9be, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9ae, 0x9c7,
0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x995, 0x9b6, 0x99f, 0x993, 0x9af, 0x9bc, 0x9be,
0x9b9, 0x997, 0x9c1, 0x9a1, 0x9cb, 0x997, 0x9c1, 0x9aa, 0x9cb, 0x9b0, 0x9cd, 0x99f,
0x9cb, 0x2d, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x9b8, 0x9be, 0x993, 0x20, 0x99f, 0x9cb,
0x9ae, 0x9a4, 0x9cd, 0x9b0, 0x9bf, 0x9aa, 0x9cb, 0x9b2, 0x9bf, 0x99f, 0x9bf, 0x989,
0x9a8, 0x9bf, 0x9b8, 0x989, 0x987, 0x9a8, 0x9a7, 0x9cb, 0x995, 0x986, 0x9a1, 0x995,
0x98f, 0x9a8, 0x995, 0x9cb, 0x9b0, 0x9c7, 0x99c, 0x985, 0x9cd, 0x9af, 0x9be, 0x999,
0x9cd, 0x997, 0x9c1, 0x987, 0x9b2, 0x9be, 0x985, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd,
0x99f, 0x9bf, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x986, 0x9b0, 0x9be, 0x997, 0x9c1,
0x9af, 0x9bc, 0x9be, 0x987, 0x9a8, 0x9be, 0x9ac, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x9a8,
0x9cb, 0x9b8, 0x20, 0x986, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x9c7, 0x9b8, 0x995, 0x9cd,
0x9af, 0x9be, 0x99f, 0x9be, 0x9ae, 0x9be, 0x9b0, 0x9cd, 0x995, 0x9be, 0x995, 0x9b0,
0x9cd, 0x9a1, 0x9cb, 0x9ac, 0x9be, 0x9b2, 0x9be, 0x20, 0x9b0, 0x9bf, 0x993, 0x9b9,
0x9be, 0x9ae, 0x9c7, 0x9a8, 0x9cd, 0x9a1, 0x9cb, 0x99c, 0x9be, 0x9b0, 0x9bf, 0x993,
0x20, 0x997, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x997, 0x9cb, 0x9b8, 0x9b8, 0x9cd, 0x9af,
0x9be, 0x9b2, 0x99f, 0x9be, 0x9b8, 0x9be, 0x9a8, 0x20, 0x99c, 0x9c1, 0x9af, 0x9bc,
0x9be, 0x9a8, 0x9b8, 0x9be, 0x9a8, 0x20, 0x9b2, 0x9c1, 0x987, 0x9b8, 0x99f, 0x9c1,
0x995, 0x9c1, 0x9ae, 0x9be, 0x9a8, 0x989, 0x9b6, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x987,
0x9af, 0x9bc, 0x9be, 0x98f, 0x9b0, 0x9c1, 0x9ac, 0x9be, 0x986, 0x9b8, 0x9c1, 0x9a8,
0x9b8, 0x9bf, 0x9af, 0x9bc, 0x9a8, 0x9ac, 0x9be, 0x9b9, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x20, 0x9ac, 0x9be, 0x9a8, 0x9cd, 0x9a6, 0x9c7, 0x9b0, 0x9be, 0x9b8, 0x9ac, 0x9be,
0x9b0, 0x9cd, 0x9ac, 0x9be, 0x9a1, 0x9cb, 0x99c, 0x9ac, 0x9cd, 0x9b2, 0x9be, 0x999,
0x9cd, 0x995, 0x2d, 0x9b8, 0x9be, 0x9ac, 0x9cd, 0x9b2, 0x9cb, 0x9a8, 0x9ac, 0x9cb,
0x9af, 0x9bc, 0x9be, 0x20, 0x9ad, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9ac, 0x9af,
0x9bc, 0x99c, 0x9bf, 0x995, 0x9c7, 0x9ae, 0x9ac, 0x9cd, 0x9b0, 0x9bf, 0x99c, 0x20,
0x9ac, 0x9c7, 0x995, 0x9be, 0x9ae, 0x9cd, 0x9aa, 0x9cb, 0x20, 0x997, 0x9cd, 0x9b0,
0x9be, 0x9a8, 0x9cd, 0x9a1, 0x9c7, 0x995, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x995, 0x9c1,
0x9a8, 0x995, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9be, 0x995, 0x9be, 0x9b8, 0x995, 0x9be,
0x9b9, 0x9c7, 0x9a8, 0x995, 0x9be, 0x9ae, 0x9c7, 0x9a8, 0x9b6, 0x9bf, 0x995, 0x9be,
0x997, 0x9cb, 0x99a, 0x9bf, 0x9b9, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9b9, 0x9c1, 0x9af,
0x9bc, 0x9be, 0x9b8, 0x9bf, 0x989, 0x9a6, 0x9be, 0x9a6, 0x20, 0x99c, 0x9c1, 0x9af,
0x9bc, 0x9be, 0x9b0, 0x9c7, 0x99c, 0x995, 0x9cb, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9b0,
0x9bf, 0x995, 0x9be, 0x995, 0x9cd, 0x9b0, 0x9bf, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9a8,
0x995, 0x9c1, 0x987, 0x9af, 0x9bc, 0x9be, 0x9ac, 0x9be, 0x995, 0x9c1, 0x9b0, 0x9be,
0x9b8, 0x9be, 0x993, 0x9a1, 0x9be, 0x9a8, 0x9ae, 0x9be, 0x9b0, 0x9cd, 0x995, 0x9b6,
0x9cd, 0x9af, 0x9be, 0x9ad, 0x9a8, 0x9a1, 0x9b8, 0x9cb, 0x9a8, 0x9a1, 0x9b8, 0x9cb,
0x9a8, 0x20, 0x995, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9a1, 0x9c7, 0x9a8, 0x9ad, 0x9be,
0x9b0, 0x9a1, 0x9c7, 0x99f, 0x9cd, 0x9b0, 0x9cb, 0x987, 0x99f, 0x9a1, 0x9cb, 0x9ae,
0x9bf, 0x9a8, 0x9bf, 0x995, 0x9be, 0x98f, 0x9a1, 0x9ae, 0x9a8, 0x9cd, 0x99f, 0x9cb,
0x9a8, 0x986, 0x987, 0x9b0, 0x9c1, 0x9a8, 0x9c7, 0x9aa, 0x9c7, 0x98f, 0x9b2, 0x20,
0x9b8, 0x9be, 0x9b2, 0x9ad, 0x9be, 0x9a6, 0x9cb, 0x9b0, 0x9ab, 0x9cb, 0x9b0, 0x9cd,
0x99f, 0x20, 0x9a8, 0x9c7, 0x9b2, 0x9b8, 0x9a8, 0x9ab, 0x9cb, 0x9b0, 0x9cd, 0x99f,
0x9be, 0x9b2, 0x9c7, 0x99c, 0x9be, 0x997, 0x9cd, 0x9b2, 0x9be, 0x9b8, 0x20, 0x9ac,
0x9c7, 0x997, 0x9c1, 0x9b8, 0x20, 0x9ac, 0x9c7, 0x997, 0x9cd, 0x9b0, 0x9cd, 0x9af,
0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9a4, 0x9c1, 0x9b0, 0x9cd, 0x995, 0x997, 0x9cd,
0x9b0, 0x9c7, 0x9a8, 0x9be, 0x9a1, 0x9be, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a6,
0x9c7, 0x9b2, 0x9cb, 0x9aa, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a4, 0x9c7, 0x9ae,
0x9be, 0x9b2, 0x9be, 0x997, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x987, 0x9af, 0x9bc, 0x9be,
0x995, 0x9bf, 0x9b2, 0x9b9, 0x9cd, 0x9af, 0x9be, 0x9b2, 0x9bf, 0x9ab, 0x9cd, 0x9af,
0x9be, 0x995, 0x9cd, 0x9b8, 0x9b9, 0x9be, 0x9b0, 0x9ae, 0x9cb, 0x9b8, 0x9bf, 0x9b2,
0x9cd, 0x9b2, 0x9cb, 0x987, 0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8,
0x9be, 0x9aa, 0x9cb, 0x9b2, 0x9bf, 0x9b8, 0x9a8, 0x995, 0x9cd, 0x9b8, 0x2c, 0x20,
0x987, 0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9ae, 0x9be,
0x9b0, 0x9c7, 0x999, 0x9cd, 0x997, 0x9cb, 0x2c, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a1,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9aa, 0x9bf, 0x99f, 0x9be, 0x9b0, 0x9b0,
0x9cd, 0x9b8, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x997, 0x2c, 0x20, 0x987, 0x9a8, 0x9cd,
0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x99f, 0x9c7, 0x9b2, 0x9b8, 0x9bf,
0x99f, 0x9bf, 0x2c, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a8, 0x9be, 0x9ad, 0x9c7, 0x9ad, 0x9c7, 0x9af, 0x9bc, 0x2c, 0x20, 0x987, 0x9a8,
0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9ad, 0x9bf, 0x9a8, 0x9b8,
0x9c7, 0x9a8, 0x9cd, 0x9a8, 0x9c7, 0x9b8, 0x2c, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a1,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x989, 0x987, 0x9a8, 0x9be, 0x9ae, 0x9cd,
0x9af, 0x9be, 0x995, 0x2c, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a1, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9a8, 0x9be, 0x987, 0x9a8, 0x9c1, 0x9ad, 0x9cd, 0x9af, 0x9be, 0x995, 0x987,
0x995, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9b2, 0x9c1, 0x987, 0x99f, 0x99c, 0x9c1, 0x9a8,
0x9cb, 0x9b2, 0x9c1, 0x987, 0x9b8, 0x9ad, 0x9bf, 0x9b2, 0x9ae, 0x9a8, 0x9cd, 0x99f,
0x9bf, 0x99a, 0x9c7, 0x9b2, 0x9cb, 0x2c, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x99f,
0x9be, 0x995, 0x9bf, 0x995, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c7, 0x9a8, 0x9cd, 0x9a1,
0x9be, 0x99c, 0x9bf, 0x995, 0x9b2, 0x9b8, 0x20, 0x985, 0x9cd, 0x9af, 0x9be, 0x99e,
0x9cd, 0x99c, 0x9c7, 0x9b2, 0x9c7, 0x9b8, 0x9b2, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9b0,
0x20, 0x9aa, 0x9cd, 0x9b0, 0x9bf, 0x9a8, 0x9cd, 0x9b8, 0x9c7, 0x9b8, 0x20, 0x995,
0x9cb, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x9ae, 0x9be, 0x9b8,
0x9c7, 0x9af, 0x9bc, 0x9cb, 0x9ae, 0x9be, 0x9a8, 0x9be, 0x997, 0x9c1, 0x9af, 0x9bc,
0x9be, 0x9ae, 0x9be, 0x9a8, 0x9be, 0x989, 0x9b8, 0x9ae, 0x9be, 0x9b0, 0x9bf, 0x997,
0x9cb, 0x9ae, 0x9be, 0x9b0, 0x99f, 0x9bf, 0x9a8, 0x9bf, 0x995, 0x9ae, 0x9be, 0x9a4,
0x9be, 0x9ae, 0x9cb, 0x9b0, 0x9b8, 0x9ae, 0x9be, 0x99c, 0x9be, 0x99f, 0x9b2, 0x9be,
0x9a8, 0x9ae, 0x9c7, 0x9a8, 0x9cb, 0x9ae, 0x9bf, 0x9a8, 0x9bf, 0x9ae, 0x9c7, 0x9b0,
0x9bf, 0x9a1, 0x9be, 0x9ae, 0x9c7, 0x99f, 0x9b2, 0x9be, 0x995, 0x9be, 0x99f, 0x9b2,
0x9be, 0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9cb, 0x20, 0x9b8, 0x9bf,
0x99f, 0x9bf, 0x9ae, 0x9bf, 0x995, 0x9c1, 0x9b2, 0x9a8, 0x9ae, 0x999, 0x9cd, 0x995,
0x99f, 0x9cb, 0x9a8, 0x9ae, 0x9a8, 0x9cd, 0x99f, 0x9c7, 0x9b0, 0x9bf, 0x9ae, 0x9a8,
0x9cd, 0x99f, 0x9c7, 0x9ad, 0x9bf, 0x9a1, 0x9bf, 0x993, 0x9ae, 0x9a8, 0x9cd, 0x9a4,
0x9b8, 0x9c7, 0x9b0, 0x9be, 0x9a4, 0x9a8, 0x9be, 0x9b8, 0x9be, 0x989, 0x9a8, 0x9bf,
0x989, 0x987, 0x9af, 0x9bc, 0x9b0, 0x9cd, 0x995, 0x9a8, 0x9b0, 0x9cb, 0x9a8, 0x9cd,
0x200c, 0x9b9, 0x9be, 0x9ac, 0x9c7, 0x989, 0x9b2, 0x9be, 0x2c, 0x20, 0x989, 0x9a4,
0x9cd, 0x9a4, 0x9b0, 0x20, 0x9a1, 0x9be, 0x995, 0x9cb, 0x99f, 0x9be, 0x9ae, 0x9a7,
0x9cd, 0x9af, 0x2c, 0x20, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9b0, 0x20, 0x9a1, 0x9be,
0x995, 0x9cb, 0x99f, 0x9be, 0x9a8, 0x9bf, 0x989, 0x20, 0x9b8, 0x9be, 0x9b2, 0x9c7,
0x9ae, 0x2c, 0x20, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9b0, 0x20, 0x9a1, 0x9be, 0x995,
0x9cb, 0x99f, 0x9be, 0x993, 0x99c, 0x9bf, 0x9a8, 0x9be, 0x997, 0x9be, 0x9aa, 0x9cd,
0x9af, 0x9be, 0x9b0, 0x9be, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x9ac, 0x9cb, 0x9aa, 0x9cb,
0x9b0, 0x9cd, 0x99f, 0x2d, 0x985, 0x9b9, 0x2d, 0x9aa, 0x9cd, 0x9b0, 0x9bf, 0x9a8,
0x9cd, 0x9b8, 0x9aa, 0x9cb, 0x9b0, 0x9cd, 0x99f, 0x20, 0x985, 0x9ab, 0x20, 0x9b8,
0x9cd, 0x9aa, 0x9c7, 0x9a8, 0x9aa, 0x9cb, 0x9b0, 0x9cd, 0x9a4, 0x9cb, 0x20, 0x9ad,
0x9c7, 0x9b2, 0x9cd, 0x200c, 0x9b9, 0x9cb, 0x9aa, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x9b0,
0x9cd, 0x9a4, 0x9cb, 0x20, 0x9b0, 0x9bf, 0x995, 0x9cb, 0x9aa, 0x9c1, 0x9a8, 0x9cd,
0x99f, 0x9be, 0x20, 0x986, 0x9b0, 0x9c7, 0x9a8, 0x9be, 0x9b8, 0x9b0, 0x200d, 0x9cd,
0x9af, 0x9be, 0x999, 0x9cd, 0x995, 0x9bf, 0x9a8, 0x20, 0x987, 0x9a8, 0x9b2, 0x9c7,
0x99f, 0x9b0, 0x9c7, 0x9b8, 0x9bf, 0x9ab, 0x9c7, 0x9b0, 0x9c7, 0x99c, 0x9bf, 0x9a8,
0x9be, 0x9b0, 0x9c7, 0x9b8, 0x9cb, 0x9b2, 0x9c1, 0x99f, 0x9b0, 0x9bf, 0x993, 0x20,
0x9ac, 0x9cd, 0x9b0, 0x9be, 0x999, 0x9cd, 0x995, 0x9cb, 0x9b8, 0x9c7, 0x9a8, 0x99f,
0x9c1, 0x9b0, 0x9c7, 0x9ae, 0x9b8, 0x9be, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x997, 0x9cb, 0x9b8, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x99f, 0x9cb, 0x20,
0x9a1, 0x9cb, 0x9ae, 0x9bf, 0x982, 0x997, 0x9cb, 0x9b8, 0x9be, 0x993, 0x20, 0x9aa,
0x9be, 0x989, 0x9b2, 0x9cb, 0x987, 0x99f, 0x9cd, 0x99f, 0x995, 0x9c1, 0x9af, 0x9bc,
0x9cb, 0x9b0, 0x99f, 0x9c1, 0x9b0, 0x9ae, 0x9bf, 0x99f, 0x9b6, 0x9bf, 0x99f, 0x995,
0x9be, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x9a5,
0x9c7, 0x9b2, 0x9c7, 0x9ae, 0x9bf, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x20, 0x99c,
0x9a8, 0x9cd, 0x9b8, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20, 0x995, 0x9bf,
0x99f, 0x9b8, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20, 0x9b2, 0x9c1, 0x9b8,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20, 0x9a5,
0x9ae, 0x9be, 0x9b8, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20, 0x9ad, 0x9bf,
0x9a8, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9b8, 0x9c1, 0x987, 0x9ab, 0x9cd, 0x99f,
0x20, 0x995, 0x9be, 0x9b0, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9a4, 0x9c7, 0x997, 0x9c1,
0x9b8, 0x9bf, 0x997, 0x9be, 0x9b2, 0x9aa, 0x9be, 0x9a5, 0x9c1, 0x9b2, 0x9bf, 0x9a4,
0x9bf, 0x99c, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x99f, 0x9cb, 0x9b0, 0x9a8,
0x9cd, 0x99f, 0x9cb, 0x99f, 0x9b0, 0x99f, 0x9cb, 0x9b2, 0x9be, 0x9ad, 0x9cd, 0x9af,
0x9be, 0x999, 0x9cd, 0x995, 0x9c1, 0x9ad, 0x9be, 0x9b0, 0x9b9, 0x9cb, 0x9af, 0x9bc,
0x9be, 0x987, 0x99f, 0x9b9, 0x9b0, 0x9cd, 0x9b8, 0x989, 0x987, 0x9a8, 0x9bf, 0x9aa,
0x9c7, 0x997, 0x987, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x99f, 0x9be, 0x99f, 0x995,
0x9c7, 0x987, 0x9b8, 0x9bf, 0x9a1, 0x9c7, 0x9ad, 0x9bf, 0x9b8, 0x9a1, 0x9c1, 0x9ae,
0x9a8, 0x9cd, 0x99f, 0x20, 0x9a1, 0x9bf, 0x2019, 0x989, 0x9b0, 0x9ad, 0x9bf, 0x9b2,
0x9ae, 0x9cd, 0x9af, 0x9be, 0x995, 0x995, 0x9c1, 0x9af, 0x9bc, 0x9cd, 0x9af, 0x9be,
0x9b0, 0x9bf, 0x9ae, 0x9b8, 0x9cb, 0x9a8, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x995, 0x9ae,
0x9c1, 0x9b0, 0x9cd, 0x9a1, 0x9cb, 0x9aa, 0x9be, 0x9b2, 0x9ae, 0x9be, 0x9b0, 0x9b0,
0x9a5, 0x9c7, 0x9b0, 0x9be, 0x9b8, 0x9bf, 0x993, 0x9af, 0x9bc, 0x9be, 0x99f, 0x9cd,
0x9b0, 0x9b2, 0x9ad, 0x9b8, 0x9cd, 0x99f, 0x9cb, 0x995, 0x9b2, 0x99e, 0x9cd, 0x99c,
0x9bf, 0x9af, 0x9bc, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x981, 0x9a8, 0x986, 0x9b9,
0x9a6, 0x9c7, 0x9a8, 0x985, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9be, 0x9a1, 0x9bf, 0x9b0,
0x986, 0x995, 0x99f, 0x9be, 0x989, 0x986, 0x995, 0x99f, 0x9cb, 0x9ac, 0x9c7, 0x986,
0x9b6, 0x997, 0x9be, 0x9ac, 0x9be, 0x9a4, 0x985, 0x9a4, 0x9bf, 0x9b0, 0x9be, 0x989,
0x9ac, 0x9be, 0x9b9, 0x9be, 0x9b0, 0x9bf, 0x9a8, 0x9ac, 0x9cd, 0x9af, 0x9be, 0x982,
0x995, 0x995, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x9a8, 0x9cc, 0x9b2, 0x9ac, 0x9c7, 0x987,
0x9b0, 0x9c1, 0x99f, 0x9ac, 0x9bf, 0x9b6, 0x995, 0x9c7, 0x995, 0x9ac, 0x9cd, 0x9b0,
0x9c1, 0x9a8, 0x9c7, 0x987, 0x99a, 0x9bf, 0x9a4, 0x9be, 0x99a, 0x9cb, 0x987, 0x9ac,
0x9be, 0x9b2, 0x9cd, 0x200c, 0x9b8, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9a6, 0x9be, 0x9ae,
0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x9b8, 0x9a6, 0x9bf, 0x9b2, 0x9bf, 0x9a6, 0x9c1,
0x9ac, 0x9be, 0x987, 0x9a6, 0x9c1, 0x9b6, 0x9be, 0x9a8, 0x9ac, 0x9c7, 0x9ab, 0x9be,
0x9ae, 0x9be, 0x997, 0x9be, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9b9, 0x9c7, 0x9ac, 0x9cd,
0x9b0, 0x9cb, 0x9a8, 0x9b9, 0x9cb, 0x20, 0x99a, 0x9bf, 0x20, 0x9ae, 0x9bf, 0x9a8,
0x20, 0x9b6, 0x9b9, 0x9b0, 0x9b9, 0x9cb, 0x9ad, 0x9cd, 0x9a1, 0x987, 0x9b0, 0x995,
0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995, 0x99c, 0x9be, 0x995, 0x9be, 0x9b0, 0x9cd, 0x9a4,
0x9be, 0x99c, 0x9af, 0x9bc, 0x9be, 0x9aa, 0x9c1, 0x9b0, 0x9be, 0x99c, 0x9c7, 0x9b0,
0x9c1, 0x99c, 0x9be, 0x9b2, 0x9c7, 0x9ae, 0x995, 0x9be, 0x9ae, 0x99a, 0x9be, 0x99f,
0x995, 0x9be, 0x995, 0x9b0, 0x9be, 0x99a, 0x9bf, 0x995, 0x9be, 0x9a0, 0x9ae, 0x9be,
0x9a8, 0x9cd, 0x9a1, 0x9c1, 0x996, 0x9be, 0x9a8, 0x9a1, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x997, 0x9be, 0x995, 0x9cb, 0x9b2, 0x995, 0x9be, 0x9a4, 0x9be, 0x995, 0x9cd, 0x9b0,
0x9be, 0x9b8, 0x9a8, 0x9cb, 0x987, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd, 0x9b8, 0x9cd,
0x995, 0x995, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9b2, 0x9be, 0x9b2, 0x9be, 0x9ae, 0x9aa,
0x9c1, 0x9b0, 0x995, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x9a4, 0x9ae, 0x9cd, 0x9af, 0x9be,
0x995, 0x9be, 0x993, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x997, 0x9be, 0x9a1, 0x9be, 0x9a8,
0x9ae, 0x9be, 0x995, 0x9be, 0x9b8, 0x9b8, 0x9be, 0x9b0, 0x9ae, 0x9cd, 0x9af, 0x9be,
0x9a8, 0x9bf, 0x9b2, 0x9be, 0x9ae, 0x9be, 0x9b8, 0x995, 0x99f, 0x9a8, 0x9bf, 0x995,
0x9cb, 0x9b8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9ad, 0x995, 0x9c1, 0x9af, 0x9bc,
0x9c7, 0x9a4, 0x9b8, 0x9cd, 0x9a8, 0x995, 0x9a8, 0x9ad, 0x9cb, 0x9b8, 0x9bf, 0x9ac,
0x9bf, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x993, 0x9ae, 0x9cd, 0x9b8, 0x9cd, 0x995,
0x993, 0x9b0, 0x9be, 0x9b2, 0x9a8, 0x9ae, 0x20, 0x9aa, 0x9c7, 0x9a8, 0x9aa, 0x9a8,
0x9cd, 0x99f, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9cd, 0x9af, 0x9be, 0x995, 0x9aa,
0x9bf, 0x9af, 0x9bc, 0x982, 0x987, 0x9af, 0x9bc, 0x982, 0x995, 0x9be, 0x9a4, 0x9be,
0x9b0, 0x995, 0x9cb, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9a8, 0x9af, 0x9bc, 0x995, 0x9bf,
0x99c, 0x9bf, 0x9b2, 0x9b0, 0x9cd, 0x9a1, 0x9be, 0x9b0, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9a7, 0x9b8, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9b8, 0x9ae, 0x9b0, 0x996,
0x9a8, 0x9cd, 0x9a6, 0x9b8, 0x9bf, 0x993, 0x9b2, 0x9b8, 0x9be, 0x982, 0x9b9, 0x9be,
0x987, 0x9b8, 0x9bf, 0x999, 0x9cd, 0x997, 0x9be, 0x9aa, 0x9c1, 0x9b0, 0x9b8, 0x9cd,
0x9b0, 0x9c7, 0x9a6, 0x9a8, 0x9c7, 0x995, 0x9cb, 0x9b2, 0x9af, 0x9bc, 0x9ae, 0x9b8,
0x9cd, 0x995, 0x9a4, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987, 0x9a4, 0x9be, 0x9b8, 0x996,
0x9a8, 0x9cd, 0x9a6, 0x9b8, 0x9bf, 0x9ac, 0x9bf, 0x9b2, 0x9bf, 0x9b6, 0x9bf, 0x9a4,
0x9c7, 0x9b9, 0x9c7, 0x9b0, 0x9be, 0x9a8, 0x99f, 0x9cb, 0x995, 0x9bf, 0x993, 0x9a4,
0x9cb, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x989, 0x9b2, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x9a4,
0x9be, 0x9b0, 0x989, 0x9b0, 0x9c1, 0x9ae, 0x995, 0x9bf, 0x989, 0x9b8, 0x9cd, 0x9a4,
0x2d, 0x20, 0x9a8, 0x9c7, 0x9b0, 0x9be, 0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9a8,
0x9a4, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9a8, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a6, 0x9bf,
0x9ad, 0x9b8, 0x9cd, 0x9a4, 0x9cb, 0x995, 0x987, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1,
0x99f, 0x9b8, 0x9cd, 0x995, 0x9b0, 0x9c7, 0x999, 0x9cd, 0x997, 0x9c1, 0x9a8, 0x987,
0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x99f, 0x9c7, 0x9b0, 0x9bf, 0x9a8, 0x9ac, 0x9be,
0x9b0, 0x9cd, 0x997, 0x987, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x9be, 0x9ad, 0x9be, 0x9a8,
0x986, 0x99c, 0x9cb, 0x9b0, 0x9c7, 0x9b8, 0x9ac, 0x9be, 0x9b0, 0x9ae, 0x9c1, 0x9a1,
0x9be, 0x995, 0x9be, 0x9a8, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad,
0x9be, 0x9b0, 0x9cd, 0x9a6, 0x9ab, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9cb, 0x9ae, 0x9be,
0x9a1, 0x9c7, 0x987, 0x9b0, 0x9be, 0x9b0, 0x9bf, 0x995, 0x99c, 0x9be, 0x9ad, 0x9bf,
0x995, 0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3, 0x20, 0x99c, 0x9b0, 0x9cd, 0x99c,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x2e, 0x20, 0x9b9,
0x9c7, 0x9b2, 0x9c7, 0x9a8, 0x9be, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9a8, 0x9b2, 0x9c0,
0x98f, 0x9cd, 0x9af, 0x9be, 0x9a1, 0x9c7, 0x9b2, 0x9c7, 0x987, 0x9a1, 0x9ac, 0x9cd,
0x9b0, 0x9bf, 0x9b8, 0x9ac, 0x9c7, 0x9a8, 0x9ac, 0x9cd, 0x9b0, 0x9cb, 0x995, 0x9c7,
0x9a8, 0x20, 0x9b9, 0x9bf, 0x9b2, 0x9a1, 0x9be, 0x9b0, 0x989, 0x987, 0x9a8, 0x9b9,
0x9cb, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x99f, 0x9b2, 0x9bf, 0x9a8, 0x9a1, 0x9c7, 0x9ae,
0x9cd, 0x9af, 0x9be, 0x9a8, 0x9b2, 0x9b0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993,
0x9ae, 0x9c7, 0x9b2, 0x9ac, 0x9cb, 0x9b0, 0x9cd, 0x9a8, 0x9aa, 0x9be, 0x9b0, 0x9cd,
0x9a5, 0x9b8, 0x9bf, 0x9a1, 0x9a8, 0x9bf, 0x986, 0x9ae, 0x9b8, 0x9cd, 0x99f, 0x9be,
0x9b0, 0x9a1, 0x9be, 0x9ae, 0x985, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x9cb,
0x9b0, 0x9be, 0x986, 0x9b8, 0x99f, 0x9cd, 0x9b0, 0x9be, 0x996, 0x9be, 0x9a8, 0x9ac,
0x9c7, 0x9b2, 0x997, 0x9cd, 0x9b0, 0x9c7, 0x9a1, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x9b2,
0x9bf, 0x9a8, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x9a4, 0x9bf, 0x9b8, 0x9cd, 0x9b2, 0x9be,
0x9ad, 0x9be, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9c7, 0x9b2, 0x9b8, 0x9ac, 0x9c1,
0x996, 0x9be, 0x9b0, 0x9c7, 0x9b8, 0x9cd, 0x99f, 0x9ac, 0x9c1, 0x9a1, 0x9be, 0x9aa,
0x9c7, 0x9b8, 0x9cd, 0x99f, 0x9ac, 0x9c1, 0x9b8, 0x9bf, 0x9a8, 0x997, 0x9c7, 0x9a8,
0x995, 0x9bf, 0x9b8, 0x9bf, 0x9a8, 0x9be, 0x9b9, 0x9c1, 0x995, 0x9cb, 0x9aa, 0x9c7,
0x9a8, 0x9b9, 0x9c7, 0x997, 0x9c7, 0x9a8, 0x99c, 0x9bf, 0x9ac, 0x9cd, 0x9b0, 0x9be,
0x9b2, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd,
0x9a8, 0x9b8, 0x9bf, 0x9b9, 0x9c7, 0x9b2, 0x9b8, 0x9bf, 0x999, 0x9cd, 0x995, 0x9bf,
0x986, 0x987, 0x9b2, 0x20, 0x985, 0x9ab, 0x20, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x9a8,
0x99c, 0x9be, 0x9b0, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9bf,
0x999, 0x997, 0x9cd, 0x9b0, 0x9be, 0x9a1, 0x995, 0x9bf, 0x9b0, 0x9cb, 0x9ad, 0x995,
0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9ad, 0x9b2, 0x9bf, 0x9b8, 0x9ac, 0x9a8, 0x9b2, 0x9c1,
0x9ac, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x9b2, 0x9a8, 0x9cd, 0x9a1,
0x9a8, 0x9b2, 0x9c1, 0x995, 0x9cd, 0x9b8, 0x9c7, 0x9ae, 0x9ac, 0x9be, 0x9b0, 0x9cd,
0x997, 0x9ae, 0x9be, 0x9a6, 0x9cd, 0x9b0, 0x9bf, 0x9a6, 0x9ae, 0x9b0, 0x9bf, 0x9af,
0x9bc, 0x9c7, 0x9b9, 0x9be, 0x9ae, 0x9c7, 0x9a8, 0x9ae, 0x9bf, 0x9a8, 0x9cd, 0x9b8,
0x995, 0x985, 0x9b8, 0x9b2, 0x9cb, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9bf, 0x9b8,
0x9aa, 0x9a1, 0x997, 0x9b0, 0x9bf, 0x9a4, 0x9cd, 0x9b8, 0x9be, 0x9aa, 0x9cd, 0x9b0,
0x9be, 0x997, 0x9b0, 0x9bf, 0x997, 0x9be, 0x9b8, 0x9be, 0x9ae, 0x9be, 0x9b0, 0x9be,
0x9b8, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x9a8, 0x9cb, 0x9b8, 0x9be,
0x9b0, 0x9be, 0x99c, 0x9bf, 0x9ad, 0x9cb, 0x9b8, 0x9be, 0x9b0, 0x9be, 0x99f, 0x9cb,
0x9ad, 0x9b8, 0x9bf, 0x9ae, 0x9ab, 0x9c7, 0x9b0, 0x9cb, 0x9aa, 0x9cb, 0x9b2, 0x9b8,
0x9cd, 0x995, 0x9aa, 0x9af, 0x9bc, 0x9c7, 0x9b8, 0x9cb, 0x9ab, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9b8, 0x9cd, 0x99f, 0x995, 0x9b9, 0x9cb, 0x9ae, 0x9a4, 0x9be, 0x9b9, 0x9b2,
0x9bf, 0x9a8, 0x9a4, 0x9bf, 0x9b0, 0x9be, 0x9a8, 0x9be, 0x989, 0x9b2, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x9cb, 0x9ad, 0x9b8, 0x9cd, 0x995, 0x9ad, 0x9be, 0x99f, 0x9bf,
0x995, 0x9be, 0x9a8, 0x9ad, 0x9bf, 0x9b2, 0x9a8, 0x9bf, 0x993, 0x9b8, 0x9ad, 0x9cb,
0x9b2, 0x9cd, 0x997, 0x9cb, 0x997, 0x9cd, 0x9b0, 0x9be, 0x9a6, 0x993, 0x9af, 0x9bc,
0x9be, 0x9b0, 0x9b6, 0x99c, 0x9be, 0x997, 0x9cd, 0x9b0, 0x9c7, 0x9ac, 0x99c, 0x9c1,
0x9b0, 0x9bf, 0x996, 0x986, 0x9a8, 0x9cd, 0x9a4, 0x9c1, 0x9a8, 0x9be, 0x9a8, 0x9be,
0x9b0, 0x9bf, 0x9ad, 0x9cb, 0x99b, 0x9be, 0x997, 0x9cb, 0x9b8, 0x995, 0x9cd, 0x9b0,
0x9bf, 0x9b8, 0x9ae, 0x9be, 0x9b8, 0x995, 0x9cb, 0x995, 0x9cb, 0x9b8, 0x995, 0x9cb,
0x9ae, 0x9cb, 0x9b0, 0x9cb, 0x995, 0x9be, 0x9b0, 0x9cd, 0x997, 0x9c1, 0x9b2, 0x9c7,
0x9a8, 0x9ae, 0x9b0, 0x9bf, 0x9b6, 0x9be, 0x9b8, 0x9ae, 0x9be, 0x9af, 0x9bc, 0x9cb,
0x9a4, 0x9cb, 0x9b0, 0x9bf, 0x987, 0x989, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9a8, 0x985,
0x995, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x9ac, 0x9c1, 0x997, 0x9c7,
0x9a8, 0x9ad, 0x9bf, 0x9b2, 0x9c7, 0x99a, 0x9cd, 0x9af, 0x9be, 0x9a0, 0x9be, 0x9ae,
0x987, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x987, 0x9ab, 0x9be, 0x9a4, 0x9c7, 0x9ab,
0x9cd, 0x9af, 0x9be, 0x995, 0x9be, 0x993, 0x9ab, 0x9cb, 0x997, 0x9cd, 0x9af, 0x9be,
0x9b2, 0x9be, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x997, 0x9cb, 0x9b8, 0x997, 0x9be, 0x9ae,
0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x997, 0x9cb, 0x9af, 0x9bc, 0x9be,
0x9a6, 0x9be, 0x9b2, 0x995, 0x9c1, 0x9a8, 0x9be, 0x9b2, 0x9b9, 0x9a8, 0x9cb, 0x9b2,
0x9c1, 0x9b2, 0x9c1, 0x98f, 0x9a8, 0x9cd, 0x9a1, 0x9be, 0x9b0, 0x9ac, 0x9be, 0x9b0,
0x9bf, 0x995, 0x9bf, 0x9b0, 0x9bf, 0x9a4, 0x9bf, 0x9ae, 0x9be, 0x9a4, 0x9bf, 0x995,
0x9cb, 0x9b8, 0x9b0, 0x9be, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x99c,
0x9be, 0x9b2, 0x9c7, 0x987, 0x9a8, 0x9ae, 0x9be, 0x99c, 0x9c1, 0x9b0, 0x9cb, 0x9ae,
0x9be, 0x9b0, 0x9cd, 0x995, 0x9c7, 0x9b8, 0x9be, 0x9b8, 0x9ae, 0x9bf, 0x9a1, 0x993,
0x9af, 0x9bc, 0x9c7, 0x9a8, 0x9be, 0x989, 0x9b0, 0x9c1, 0x9a8, 0x9bf, 0x989, 0x9af,
0x9bc, 0x9bf, 0x9a8, 0x9b0, 0x9ab, 0x995, 0x9a8, 0x9c1, 0x9ae, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x997, 0x9cb, 0x20, 0x9aa, 0x9cd, 0x9af, 0x9be,
0x997, 0x9cb, 0x9aa, 0x9bf, 0x99f, 0x995, 0x9c7, 0x9af, 0x9bc, 0x9be, 0x9b0, 0x9cd,
0x9a8, 0x9aa, 0x9cb, 0x9a8, 0x9be, 0x9aa, 0x9c7, 0x9aa, 0x9cb, 0x9b0, 0x9cd, 0x99f,
0x20, 0x9ae, 0x9cc, 0x9b0, 0x99c, 0x9ac, 0x9bf, 0x9b0, 0x9be, 0x9b0, 0x989, 0x9b9,
0x9a4, 0x9c1, 0x999, 0x9cd, 0x997, 0x9be, 0x9b8, 0x9be, 0x987, 0x9aa, 0x9be, 0x9a8,
0x99f, 0x9be, 0x9b0, 0x9be, 0x993, 0x9af, 0x9bc, 0x9be, 0x99f, 0x9cb, 0x999, 0x9cd,
0x997, 0x9be, 0x99f, 0x9be, 0x9aa, 0x9c1, 0x993, 0x9af, 0x9bc, 0x9c7, 0x995, 0x993,
0x9b2, 0x9bf, 0x9b8, 0x41, 0x6c, 0x6a, 0x65, 0x72, 0x42, 0x61, 0x6e, 0x67,
0x69, 0x4b, 0x61, 0x69, 0x72, 0x6f, 0x4b, 0x6f, 0x6e, 0x61, 0x6b, 0x72,
0x79, 0x44, 0x6a, 0x69, 0x62, 0x75, 0x74, 0x69, 0x41, 0x61, 0x69, 0x75,
0x6e, 0x4b, 0x68, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x4d, 0x75, 0x71, 0x64,
0x69, 0x73, 0x68, 0x6f, 0x4e, 0x2019, 0x64, 0x6a, 0x61, 0x6d, 0x65, 0x6e,
0x61, 0x4e, 0x69, 0x61, 0x6d, 0x65, 0x69, 0x55, 0x61, 0x67, 0x61, 0x64,
0x75, 0x67, 0x75, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x41, 0x69, 0x6e, 0x67, 0x69, 0x72, 0x61, 0x41, 0x72, 0x61, 0x67,
0x75, 0x61, 0xed, 0x6e, 0x61, 0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x43,
0x75, 0x69, 0x61, 0x62, 0xe1, 0x45, 0x69, 0x72, 0x75, 0x6e, 0x65, 0x70,
0xe9, 0x4d, 0x61, 0x63, 0x65, 0x69, 0xf3, 0x4d, 0x61, 0x72, 0x74, 0x69,
0x6e, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x7a, 0x61, 0x74, 0x6c, 0xe1, 0x6e,
0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x20, 0x48, 0x69, 0x72, 0x69, 0x61,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x75, 0x6e, 0x65, 0x42, 0x65, 0x75, 0x6c,
0x61, 0x68, 0x2c, 0x20, 0x49, 0x70, 0x61, 0x72, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x49,
0x70, 0x61, 0x72, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65,
0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x49, 0x70, 0x61,
0x72, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x50, 0x6f, 0x72, 0x74,
0x2d, 0x6f, 0x66, 0x2d, 0x53, 0x70, 0x61, 0x69, 0x6e, 0x53, 0x61, 0x6e,
0x74, 0x61, 0x72, 0xe9, 0x6d, 0x53, 0xe3, 0x6f, 0x20, 0x50, 0x61, 0x75,
0x6c, 0x6f, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x42, 0x61, 0x72, 0x74,
0x68, 0xe9, 0x6c, 0x65, 0x6d, 0x79, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20,
0x4a, 0x6f, 0x68, 0x6e, 0x2019, 0x73, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20,
0x4b, 0x69, 0x74, 0x74, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c,
0x75, 0x7a, 0x69, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x54, 0x68,
0x6f, 0x6d, 0x61, 0x73, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x56, 0x69,
0x6e, 0x63, 0x65, 0x6e, 0x74, 0x51, 0x61, 0x61, 0x6e, 0x61, 0x61, 0x71,
0x41, 0x73, 0x67, 0x61, 0x62, 0x61, 0x74, 0x42, 0x69, 0x78, 0x6b, 0x65,
0x6b, 0x54, 0x78, 0x69, 0x74, 0x61, 0x54, 0x78, 0x6f, 0x69, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x6f, 0x44,
0x75, 0x78, 0x61, 0x6e, 0x62, 0x65, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x4b,
0x61, 0x6d, 0x74, 0x78, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61,
0x74, 0x78, 0x69, 0x4b, 0x68, 0x61, 0x6e, 0x64, 0x69, 0x67, 0x61, 0x4b,
0x61, 0x6c, 0x6b, 0x75, 0x74, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x69, 0x61, 0x72, 0x73, 0x6b, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x74, 0x4e,
0x69, 0x6b, 0x6f, 0x73, 0x69, 0x61, 0x50, 0x69, 0x6f, 0x6e, 0x67, 0x69,
0x61, 0x6e, 0x67, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x64, 0x61, 0x54, 0x61,
0x78, 0x6b, 0x65, 0x6e, 0x74, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0xdc, 0x72, 0xfc, 0x6d, 0x71, 0x69, 0x56, 0x69, 0x65,
0x6e, 0x74, 0x69, 0x61, 0x6e, 0x45, 0x72, 0x65, 0x76, 0x61, 0x6e, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x61, 0x6b, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69,
0x61, 0x6b, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x48, 0x65, 0x67, 0x6f, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6b, 0x53, 0x61, 0x6e, 0x74, 0x61,
0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x74, 0x65, 0x6e, 0x61,
0x73, 0x42, 0x65, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x42, 0x72, 0x75, 0x73,
0x65, 0x6c, 0x61, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x65, 0x73, 0x74, 0x42,
0xfc, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x47, 0x75, 0x65, 0x72, 0x6e,
0x65, 0x73, 0x65, 0x79, 0x4d, 0x61, 0x6e, 0x20, 0x75, 0x68, 0x61, 0x72,
0x74, 0x65, 0x61, 0x4c, 0x75, 0x78, 0x65, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x6f, 0x4d, 0x61, 0x64, 0x72, 0x69, 0x6c, 0x4d, 0x6f, 0x73, 0x6b, 0x75,
0x45, 0x72, 0x72, 0x6f, 0x6d, 0x61, 0x55, 0x6c, 0x69, 0x61, 0x6e, 0x6f,
0x76, 0x73, 0x6b, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e, 0x6f, 0x20,
0x48, 0x69, 0x72, 0x69, 0x61, 0x4d, 0x61, 0x68, 0xe9, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x61, 0x6b, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x7a, 0x69,
0x6f, 0x50, 0x61, 0x7a, 0x6b, 0x6f, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74,
0x65, 0x61, 0xc9, 0x66, 0x61, 0x74, 0xe9, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x61, 0x6b, 0x4d, 0x61, 0x72, 0x6b, 0x65, 0x73, 0x61,
0x6b, 0x410, 0x431, 0x456, 0x434, 0x436, 0x430, 0x43d, 0x410, 0x43a, 0x440, 0x430,
0x410, 0x434, 0x44b, 0x441, 0x2d, 0x410, 0x431, 0x435, 0x431, 0x430, 0x410, 0x43b,
0x436, 0x44b, 0x440, 0x410, 0x441, 0x43c, 0x430, 0x440, 0x430, 0x411, 0x430, 0x43d,
0x433, 0x456, 0x411, 0x430, 0x43d, 0x434, 0x436, 0x443, 0x43b, 0x411, 0x456, 0x441,
0x430, 0x443, 0x411, 0x43b, 0x430, 0x43d, 0x442, 0x430, 0x439, 0x440, 0x411, 0x440,
0x430, 0x437, 0x430, 0x432, 0x456, 0x43b, 0x44c, 0x41a, 0x430, 0x456, 0x440, 0x41a,
0x43e, 0x43d, 0x430, 0x43a, 0x440, 0x44b, 0x414, 0x430, 0x440, 0x2d, 0x44d, 0x441,
0x2d, 0x421, 0x430, 0x43b, 0x430, 0x43c, 0x414, 0x436, 0x44b, 0x431, 0x443, 0x446,
0x456, 0x42d, 0x43b, 0x44c, 0x2d, 0x410, 0x44e, 0x43d, 0x424, 0x440, 0x44b, 0x442,
0x430, 0x45e, 0x43d, 0x413, 0x430, 0x431, 0x430, 0x440, 0x43e, 0x43d, 0x44d, 0x425,
0x430, 0x440, 0x430, 0x440, 0x44d, 0x42f, 0x433, 0x430, 0x43d, 0x44d, 0x441, 0x431,
0x443, 0x440, 0x433, 0x414, 0x436, 0x443, 0x431, 0x430, 0x41a, 0x456, 0x433, 0x430,
0x43b, 0x456, 0x41a, 0x456, 0x43d, 0x448, 0x430, 0x441, 0x430, 0x41b, 0x430, 0x433,
0x430, 0x441, 0x41b, 0x456, 0x431, 0x440, 0x44d, 0x432, 0x456, 0x43b, 0x44c, 0x41b,
0x430, 0x43c, 0x44d, 0x41b, 0x443, 0x431, 0x443, 0x43c, 0x431, 0x430, 0x448, 0x44b,
0x41c, 0x430, 0x43b, 0x430, 0x431, 0x430, 0x41c, 0x431, 0x430, 0x431, 0x430, 0x43d,
0x44d, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x44b, 0x448, 0x430, 0x41c, 0x430, 0x43d,
0x440, 0x43e, 0x432, 0x456, 0x44f, 0x41d, 0x430, 0x439, 0x440, 0x43e, 0x431, 0x456,
0x41d, 0x434, 0x436, 0x430, 0x43c, 0x435, 0x43d, 0x430, 0x41d, 0x456, 0x44f, 0x43c,
0x435, 0x439, 0x41f, 0x43e, 0x440, 0x442, 0x430, 0x2d, 0x41d, 0x43e, 0x432, 0x430,
0x421, 0x430, 0x43d, 0x2d, 0x422, 0x430, 0x43c, 0x44d, 0x422, 0x440, 0x44b, 0x43f,
0x430, 0x43b, 0x456, 0x422, 0x443, 0x43d, 0x456, 0x441, 0x412, 0x456, 0x43d, 0x434,
0x445, 0x443, 0x43a, 0x410, 0x43d, 0x43a, 0x430, 0x440, 0x44b, 0x434, 0x436, 0x410,
0x43d, 0x433, 0x456, 0x43b, 0x44c, 0x44f, 0x410, 0x43d, 0x442, 0x44b, 0x433, 0x443,
0x430, 0x410, 0x440, 0x430, 0x433, 0x443, 0x430, 0x456, 0x43d, 0x430, 0x411, 0x443,
0x44d, 0x43d, 0x430, 0x441, 0x2d, 0x410, 0x439, 0x440, 0x44d, 0x441, 0x41a, 0x43e,
0x440, 0x434, 0x430, 0x432, 0x430, 0x416, 0x443, 0x436, 0x443, 0x439, 0x41b, 0x430,
0x2d, 0x420, 0x44b, 0x451, 0x445, 0x430, 0x420, 0x44b, 0x430, 0x2d, 0x413, 0x430,
0x43b, 0x44c, 0x435, 0x433, 0x430, 0x441, 0x421, 0x430, 0x43b, 0x44c, 0x442, 0x430,
0x421, 0x430, 0x43d, 0x2d, 0x41b, 0x443, 0x456, 0x441, 0x423, 0x448, 0x443, 0x430,
0x44f, 0x410, 0x441, 0x443, 0x43d, 0x441, 0x44c, 0x451, 0x43d, 0x410, 0x442, 0x44b,
0x43a, 0x43e, 0x43a, 0x430, 0x43d, 0x411, 0x430, 0x456, 0x44f, 0x411, 0x430, 0x456,
0x44f, 0x2d, 0x434, 0x44d, 0x2d, 0x411, 0x430, 0x43d, 0x434, 0x44d, 0x440, 0x430,
0x441, 0x411, 0x430, 0x440, 0x431, 0x430, 0x434, 0x430, 0x441, 0x411, 0x435, 0x43b,
0x456, 0x437, 0x411, 0x43e, 0x430, 0x2d, 0x412, 0x456, 0x441, 0x442, 0x430, 0x411,
0x430, 0x433, 0x430, 0x442, 0x430, 0x411, 0x43e, 0x439, 0x441, 0x44d, 0x41a, 0x435,
0x43c, 0x431, 0x440, 0x44b, 0x434, 0x436, 0x2d, 0x411, 0x435, 0x439, 0x41a, 0x430,
0x43c, 0x43f, 0x443, 0x2d, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x44b, 0x41a, 0x430,
0x435, 0x43d, 0x430, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d, 0x430, 0x432, 0x44b,
0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x427, 0x44b, 0x43a, 0x430,
0x433, 0x430, 0x427, 0x44b, 0x45e, 0x430, 0x45e, 0x430, 0x421, 0x44c, 0x44e, 0x434,
0x430, 0x434, 0x2d, 0x425, 0x443, 0x430, 0x440, 0x44d, 0x441, 0x41a, 0x43e, 0x441,
0x442, 0x430, 0x2d, 0x420, 0x44b, 0x43a, 0x430, 0x41a, 0x440, 0x44d, 0x441, 0x442,
0x430, 0x43d, 0x41a, 0x443, 0x44f, 0x431, 0x430, 0x41a, 0x44e, 0x440, 0x430, 0x441,
0x430, 0x430, 0x414, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x43a, 0x441, 0x445, 0x430,
0x45e, 0x43d, 0x414, 0x43e, 0x45e, 0x441, 0x430, 0x43d, 0x414, 0x43e, 0x441, 0x430,
0x43d, 0x2d, 0x41a, 0x440, 0x44b, 0x43a, 0x414, 0x44d, 0x43d, 0x432, 0x435, 0x440,
0x414, 0x44d, 0x442, 0x440, 0x43e, 0x439, 0x442, 0x414, 0x430, 0x43c, 0x456, 0x43d,
0x456, 0x43a, 0x430, 0x42d, 0x434, 0x43c, 0x430, 0x43d, 0x442, 0x430, 0x43d, 0x42d,
0x439, 0x440, 0x443, 0x43d, 0x44d, 0x43f, 0x435, 0x421, 0x430, 0x43b, 0x44c, 0x432,
0x430, 0x434, 0x43e, 0x440, 0x424, 0x43e, 0x440, 0x442, 0x2d, 0x41d, 0x435, 0x43b,
0x44c, 0x441, 0x430, 0x43d, 0x424, 0x430, 0x440, 0x442, 0x430, 0x43b, 0x435, 0x437,
0x430, 0x413, 0x43b, 0x44d, 0x439, 0x441, 0x2d, 0x411, 0x44d, 0x439, 0x413, 0x443,
0x441, 0x2d, 0x411, 0x44d, 0x439, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x2d, 0x426,
0x451, 0x440, 0x43a, 0x413, 0x440, 0x44d, 0x43d, 0x430, 0x434, 0x430, 0x413, 0x432,
0x430, 0x434, 0x44d, 0x43b, 0x443, 0x43f, 0x430, 0x413, 0x432, 0x430, 0x442, 0x44d,
0x43c, 0x430, 0x43b, 0x430, 0x413, 0x443, 0x430, 0x44f, 0x43a, 0x456, 0x43b, 0x44c,
0x413, 0x430, 0x44f, 0x43d, 0x430, 0x413, 0x430, 0x43b, 0x456, 0x444, 0x430, 0x43a,
0x441, 0x42d, 0x440, 0x43c, 0x430, 0x441, 0x456, 0x43b, 0x44c, 0x44f, 0x406, 0x43d,
0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x43f, 0x430, 0x43b, 0x456, 0x441, 0x41d, 0x43e,
0x43a, 0x441, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x41c,
0x430, 0x440, 0x44d, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x44b,
0x44f, 0x43d, 0x430, 0x41f, 0x456, 0x442, 0x44d, 0x440, 0x441, 0x431, 0x435, 0x440,
0x433, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x422, 0x44d,
0x43b, 0x20, 0x421, 0x456, 0x446, 0x456, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x44b,
0x44f, 0x43d, 0x430, 0x412, 0x456, 0x432, 0x456, 0x2c, 0x20, 0x406, 0x43d, 0x434,
0x44b, 0x44f, 0x43d, 0x430, 0x412, 0x456, 0x43d, 0x441, 0x435, 0x43d, 0x441, 0x2c,
0x20, 0x406, 0x43d, 0x434, 0x44b, 0x44f, 0x43d, 0x430, 0x423, 0x456, 0x43d, 0x430,
0x43c, 0x430, 0x43a, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x44f, 0x43d, 0x430,
0x406, 0x43d, 0x443, 0x432, 0x456, 0x43a, 0x406, 0x43a, 0x430, 0x43b, 0x443, 0x456,
0x442, 0x42f, 0x43c, 0x430, 0x439, 0x43a, 0x430, 0x414, 0x436, 0x443, 0x43d, 0x430,
0x41b, 0x443, 0x456, 0x441, 0x432, 0x456, 0x43b, 0x41c, 0x430, 0x43d, 0x442, 0x44b,
0x441, 0x435, 0x43b, 0x430, 0x2c, 0x20, 0x41a, 0x435, 0x43d, 0x442, 0x443, 0x43a,
0x456, 0x41a, 0x440, 0x430, 0x43b, 0x435, 0x43d, 0x434, 0x44d, 0x439, 0x43a, 0x41b,
0x456, 0x43c, 0x430, 0x41b, 0x43e, 0x441, 0x2d, 0x410, 0x43d, 0x434, 0x436, 0x44d,
0x43b, 0x435, 0x441, 0x41b, 0x43e, 0x45e, 0x44d, 0x440, 0x20, 0x41f, 0x440, 0x44b,
0x43d, 0x441, 0x456, 0x437, 0x20, 0x41a, 0x432, 0x43e, 0x442, 0x44d, 0x440, 0x41c,
0x430, 0x441, 0x435, 0x451, 0x41c, 0x430, 0x43d, 0x430, 0x45e, 0x441, 0x41c, 0x430,
0x440, 0x44b, 0x433, 0x43e, 0x41c, 0x430, 0x440, 0x446, 0x456, 0x43d, 0x456, 0x43a,
0x430, 0x41c, 0x430, 0x442, 0x430, 0x43c, 0x43e, 0x440, 0x430, 0x441, 0x41c, 0x435,
0x43d, 0x43e, 0x43c, 0x456, 0x43d, 0x456, 0x41c, 0x435, 0x440, 0x44b, 0x434, 0x430,
0x41c, 0x435, 0x445, 0x456, 0x43a, 0x430, 0x41c, 0x456, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x41c, 0x43e, 0x43d, 0x43a, 0x442, 0x430, 0x43d, 0x41c, 0x430, 0x43d, 0x442,
0x44d, 0x440, 0x44d, 0x439, 0x41c, 0x430, 0x43d, 0x442, 0x44d, 0x432, 0x456, 0x434,
0x44d, 0x430, 0x41c, 0x430, 0x43d, 0x442, 0x441, 0x435, 0x440, 0x430, 0x442, 0x41d,
0x430, 0x441, 0x430, 0x45e, 0x41d, 0x44c, 0x44e, 0x2d, 0x401, 0x440, 0x43a, 0x41d,
0x430, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x411, 0x2019, 0x44e, 0x43b, 0x430, 0x2c,
0x20, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x421, 0x435, 0x43d, 0x442, 0x44d, 0x440, 0x2c,
0x20, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c, 0x44e, 0x2d, 0x421, 0x435, 0x439,
0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d,
0x430, 0x44f, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x443, 0x43a,
0x410, 0x445, 0x456, 0x43d, 0x430, 0x433, 0x430, 0x41f, 0x430, 0x440, 0x430, 0x43c,
0x430, 0x440, 0x44b, 0x431, 0x430, 0x424, 0x456, 0x43d, 0x456, 0x43a, 0x441, 0x41f,
0x43e, 0x440, 0x442, 0x2d, 0x43e, 0x2d, 0x41f, 0x440, 0x44d, 0x43d, 0x441, 0x41f,
0x43e, 0x440, 0x442, 0x2d, 0x43e, 0x444, 0x2d, 0x421, 0x43f, 0x435, 0x439, 0x43d,
0x41f, 0x43e, 0x440, 0x442, 0x443, 0x2d, 0x412, 0x435, 0x43b, 0x44c, 0x44e, 0x41f,
0x443, 0x44d, 0x440, 0x442, 0x430, 0x2d, 0x420, 0x44b, 0x43a, 0x430, 0x41f, 0x443,
0x43d, 0x442, 0x430, 0x2d, 0x410, 0x440, 0x44d, 0x43d, 0x430, 0x441, 0x420, 0x430,
0x43d, 0x43a, 0x456, 0x43d, 0x2d, 0x406, 0x43d, 0x43b, 0x435, 0x442, 0x420, 0x44d,
0x441, 0x456, 0x444, 0x456, 0x420, 0x44d, 0x434, 0x436, 0x430, 0x439, 0x43d, 0x430,
0x420, 0x44d, 0x437, 0x430, 0x43b, 0x44c, 0x44e, 0x442, 0x420, 0x44b, 0x443, 0x2d,
0x411, 0x440, 0x430, 0x43d, 0x43a, 0x443, 0x421, 0x430, 0x43d, 0x442, 0x430, 0x440,
0x44d, 0x43d, 0x421, 0x430, 0x43d, 0x442, 0x2019, 0x44f, 0x433, 0x430, 0x421, 0x430,
0x43d, 0x442, 0x430, 0x2d, 0x414, 0x430, 0x43c, 0x456, 0x43d, 0x433, 0x430, 0x421,
0x430, 0x43d, 0x2d, 0x41f, 0x430, 0x45e, 0x43b, 0x443, 0x406, 0x442, 0x430, 0x43a,
0x430, 0x440, 0x442, 0x430, 0x430, 0x440, 0x43c, 0x456, 0x442, 0x421, 0x456, 0x442,
0x43a, 0x430, 0x421, 0x435, 0x43d, 0x2d, 0x411, 0x430, 0x440, 0x442, 0x44d, 0x43b,
0x44c, 0x43c, 0x456, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x414, 0x436, 0x43e, 0x43d,
0x441, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41a, 0x456, 0x442, 0x441, 0x421, 0x435,
0x43d, 0x442, 0x2d, 0x41b, 0x44e, 0x441, 0x456, 0x44f, 0x421, 0x435, 0x43d, 0x442,
0x2d, 0x412, 0x456, 0x43d, 0x441, 0x435, 0x43d, 0x442, 0x421, 0x432, 0x456, 0x444,
0x442, 0x2d, 0x41a, 0x435, 0x440, 0x44d, 0x43d, 0x442, 0x422, 0x44d, 0x433, 0x443,
0x441, 0x456, 0x433, 0x430, 0x43b, 0x44c, 0x43f, 0x430, 0x422, 0x44b, 0x445, 0x443,
0x430, 0x43d, 0x430, 0x422, 0x430, 0x440, 0x43e, 0x43d, 0x442, 0x430, 0x422, 0x430,
0x440, 0x442, 0x43e, 0x43b, 0x430, 0x423, 0x430, 0x439, 0x442, 0x445, 0x43e, 0x440,
0x441, 0x412, 0x456, 0x43d, 0x456, 0x43f, 0x435, 0x433, 0x42f, 0x43a, 0x443, 0x442,
0x430, 0x442, 0x41a, 0x44d, 0x439, 0x441, 0x456, 0x414, 0x44d, 0x439, 0x432, 0x456,
0x441, 0x414, 0x437, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x414, 0x437, 0x44e, 0x440,
0x432, 0x456, 0x43b, 0x44c, 0x41c, 0x430, 0x43a, 0x443, 0x43e, 0x440, 0x44b, 0x41c,
0x43e, 0x45e, 0x441, 0x430, 0x43d, 0x41c, 0x430, 0x43a, 0x2d, 0x41c, 0x435, 0x440,
0x434, 0x430, 0x420, 0x43e, 0x442, 0x44d, 0x440, 0x430, 0x421, 0x451, 0x432, 0x430,
0x412, 0x430, 0x441, 0x442, 0x43e, 0x43a, 0x41b, 0x43e, 0x43d, 0x433, 0x439, 0x456,
0x440, 0x410, 0x434, 0x44d, 0x43d, 0x410, 0x43c, 0x430, 0x43d, 0x20, 0x28, 0x433,
0x43e, 0x440, 0x430, 0x434, 0x29, 0x410, 0x43a, 0x442, 0x430, 0x431, 0x44d, 0x410,
0x448, 0x433, 0x430, 0x431, 0x430, 0x442, 0x411, 0x430, 0x445, 0x440, 0x44d, 0x439,
0x43d, 0x411, 0x435, 0x439, 0x440, 0x443, 0x442, 0x411, 0x456, 0x448, 0x43a, 0x435,
0x43a, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x427, 0x44b, 0x442, 0x430, 0x427,
0x430, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x41a, 0x430, 0x43b, 0x43e,
0x43c, 0x431, 0x430, 0x414, 0x44b, 0x43b, 0x456, 0x414, 0x443, 0x431, 0x430, 0x439,
0x414, 0x443, 0x448, 0x430, 0x43d, 0x431, 0x44d, 0x425, 0x435, 0x45e, 0x440, 0x43e,
0x43d, 0x425, 0x430, 0x448, 0x44b, 0x43c, 0x456, 0x43d, 0x413, 0x430, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x425, 0x43e, 0x45e, 0x434, 0x406, 0x440, 0x43a, 0x443, 0x446,
0x43a, 0x414, 0x436, 0x430, 0x43a, 0x430, 0x440, 0x442, 0x430, 0x414, 0x436, 0x430,
0x44f, 0x43f, 0x443, 0x440, 0x430, 0x406, 0x435, 0x440, 0x443, 0x441, 0x430, 0x43b,
0x456, 0x43c, 0x41a, 0x430, 0x440, 0x430, 0x447, 0x44b, 0x41a, 0x430, 0x43b, 0x44c,
0x43a, 0x443, 0x442, 0x430, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x430, 0x44f, 0x440,
0x441, 0x43a, 0x41a, 0x443, 0x447, 0x44b, 0x43d, 0x433, 0x41a, 0x443, 0x432, 0x435,
0x439, 0x442, 0x41c, 0x430, 0x43a, 0x430, 0x430, 0x41c, 0x430, 0x43d, 0x456, 0x43b,
0x430, 0x41d, 0x456, 0x43a, 0x430, 0x441, 0x456, 0x44f, 0x41d, 0x43e, 0x432, 0x430,
0x43a, 0x443, 0x437, 0x43d, 0x435, 0x446, 0x43a, 0x41d, 0x43e, 0x432, 0x430, 0x441,
0x456, 0x431, 0x456, 0x440, 0x441, 0x43a, 0x423, 0x440, 0x430, 0x43b, 0x44c, 0x441,
0x43a, 0x41f, 0x43d, 0x430, 0x43c, 0x43f, 0x435, 0x43d, 0x44c, 0x41f, 0x430, 0x43d,
0x442, 0x44b, 0x44f, 0x43d, 0x430, 0x43a, 0x41f, 0x445, 0x435, 0x43d, 0x44c, 0x44f,
0x43d, 0x41a, 0x443, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x439, 0x41a, 0x437, 0x44b,
0x43b, 0x2d, 0x410, 0x440, 0x434, 0x430, 0x42d, 0x440, 0x2d, 0x420, 0x44b, 0x44f,
0x434, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x456, 0x43d, 0x428, 0x430, 0x43d, 0x445,
0x430, 0x439, 0x421, 0x456, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x421, 0x44f,
0x440, 0x44d, 0x434, 0x43d, 0x435, 0x43a, 0x430, 0x43b, 0x44b, 0x43c, 0x441, 0x43a,
0x422, 0x430, 0x439, 0x431, 0x44d, 0x439, 0x422, 0x431, 0x456, 0x43b, 0x456, 0x441,
0x456, 0x422, 0x44d, 0x433, 0x435, 0x440, 0x430, 0x43d, 0x422, 0x445, 0x456, 0x43c,
0x43f, 0x445, 0x443, 0x422, 0x43e, 0x43a, 0x456, 0x430, 0x423, 0x43b, 0x430, 0x43d,
0x2d, 0x411, 0x430, 0x442, 0x430, 0x440, 0x423, 0x440, 0x443, 0x43c, 0x447, 0x44b,
0x423, 0x441, 0x446, 0x44c, 0x2d, 0x41d, 0x435, 0x440, 0x430, 0x412, 0x2019, 0x435,
0x43d, 0x446, 0x44c, 0x44f, 0x43d, 0x423, 0x43b, 0x430, 0x434, 0x437, 0x456, 0x432,
0x430, 0x441, 0x442, 0x43e, 0x43a, 0x42f, 0x43a, 0x443, 0x446, 0x43a, 0x420, 0x430,
0x43d, 0x433, 0x443, 0x43d, 0x415, 0x43a, 0x430, 0x446, 0x44f, 0x440, 0x44b, 0x43d,
0x431, 0x443, 0x440, 0x433, 0x415, 0x440, 0x44d, 0x432, 0x430, 0x43d, 0x410, 0x437,
0x43e, 0x440, 0x441, 0x43a, 0x456, 0x44f, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430,
0x432, 0x44b, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x441, 0x43a, 0x456, 0x44f,
0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x41a, 0x430, 0x43d, 0x430,
0x440, 0x441, 0x43a, 0x456, 0x44f, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432,
0x44b, 0x41a, 0x430, 0x431, 0x430, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x44d, 0x424,
0x430, 0x440, 0x44d, 0x440, 0x441, 0x43a, 0x456, 0x44f, 0x20, 0x430, 0x441, 0x442,
0x440, 0x430, 0x432, 0x44b, 0x41c, 0x430, 0x434, 0x44d, 0x439, 0x440, 0x430, 0x420,
0x44d, 0x439, 0x43a, 0x2019, 0x44f, 0x432, 0x456, 0x43a, 0x41f, 0x430, 0x45e, 0x434,
0x43d, 0x451, 0x432, 0x430, 0x44f, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x456,
0x44f, 0x412, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x45e, 0x20, 0x421, 0x432, 0x44f,
0x442, 0x43e, 0x439, 0x20, 0x410, 0x43b, 0x435, 0x43d, 0x44b, 0x41f, 0x43e, 0x440,
0x442, 0x2d, 0x421, 0x442, 0x44d, 0x43d, 0x43b, 0x456, 0x410, 0x434, 0x44d, 0x43b,
0x430, 0x456, 0x434, 0x430, 0x411, 0x440, 0x44b, 0x441, 0x431, 0x435, 0x43d, 0x411,
0x440, 0x43e, 0x43a, 0x435, 0x43d, 0x2d, 0x425, 0x456, 0x43b, 0x414, 0x430, 0x440,
0x432, 0x456, 0x43d, 0x42e, 0x43a, 0x43b, 0x430, 0x41b, 0x456, 0x43d, 0x434, 0x44d,
0x43c, 0x430, 0x43d, 0x41c, 0x435, 0x43b, 0x44c, 0x431, 0x443, 0x440, 0x43d, 0x421,
0x456, 0x434, 0x43d, 0x435, 0x439, 0x410, 0x43c, 0x441, 0x442, 0x44d, 0x440, 0x434,
0x430, 0x43c, 0x410, 0x43d, 0x434, 0x43e, 0x440, 0x430, 0x410, 0x441, 0x442, 0x440,
0x430, 0x445, 0x430, 0x43d, 0x44c, 0x410, 0x444, 0x456, 0x43d, 0x44b, 0x411, 0x44f,
0x43b, 0x433, 0x440, 0x430, 0x434, 0x411, 0x435, 0x440, 0x43b, 0x456, 0x43d, 0x411,
0x440, 0x430, 0x446, 0x456, 0x441, 0x43b, 0x430, 0x432, 0x430, 0x411, 0x440, 0x443,
0x441, 0x435, 0x43b, 0x44c, 0x411, 0x443, 0x445, 0x430, 0x440, 0x44d, 0x441, 0x442,
0x411, 0x44e, 0x437, 0x456, 0x43d, 0x433, 0x435, 0x43d, 0x41a, 0x456, 0x448, 0x44b,
0x43d, 0x451, 0x45e, 0x41a, 0x430, 0x43f, 0x435, 0x43d, 0x433, 0x430, 0x433, 0x435,
0x43d, 0x414, 0x443, 0x431, 0x43b, 0x456, 0x43d, 0x413, 0x456, 0x431, 0x440, 0x430,
0x43b, 0x442, 0x430, 0x440, 0x413, 0x435, 0x440, 0x43d, 0x441, 0x456, 0x425, 0x435,
0x43b, 0x44c, 0x441, 0x456, 0x43d, 0x43a, 0x456, 0x412, 0x43e, 0x441, 0x442, 0x440,
0x430, 0x45e, 0x20, 0x41c, 0x44d, 0x43d, 0x414, 0x436, 0x44d, 0x440, 0x441, 0x456,
0x41a, 0x430, 0x43b, 0x456, 0x43d, 0x456, 0x43d, 0x433, 0x440, 0x430, 0x434, 0x41a,
0x456, 0x440, 0x430, 0x45e, 0x41a, 0x456, 0x435, 0x45e, 0x41b, 0x456, 0x441, 0x430,
0x431, 0x43e, 0x43d, 0x41b, 0x44e, 0x431, 0x43b, 0x44f, 0x43d, 0x430, 0x41b, 0x43e,
0x43d, 0x434, 0x430, 0x43d, 0x41b, 0x44e, 0x43a, 0x441, 0x435, 0x43c, 0x431, 0x443,
0x440, 0x433, 0x41c, 0x430, 0x434, 0x440, 0x44b, 0x434, 0x41c, 0x430, 0x43b, 0x44c,
0x442, 0x430, 0x41c, 0x430, 0x440, 0x44b, 0x44f, 0x43d, 0x445, 0x430, 0x43c, 0x456,
0x43d, 0x430, 0x41c, 0x456, 0x43d, 0x441, 0x43a, 0x41c, 0x430, 0x43d, 0x430, 0x43a,
0x430, 0x41c, 0x430, 0x441, 0x43a, 0x432, 0x430, 0x41e, 0x441, 0x43b, 0x430, 0x41f,
0x430, 0x440, 0x44b, 0x436, 0x41f, 0x430, 0x434, 0x433, 0x43e, 0x440, 0x44b, 0x446,
0x430, 0x420, 0x44b, 0x433, 0x430, 0x420, 0x44b, 0x43c, 0x421, 0x430, 0x43d, 0x2d,
0x41c, 0x430, 0x440, 0x44b, 0x43d, 0x430, 0x421, 0x430, 0x440, 0x430, 0x435, 0x432,
0x430, 0x421, 0x430, 0x440, 0x430, 0x442, 0x430, 0x45e, 0x421, 0x456, 0x43c, 0x444,
0x435, 0x440, 0x43e, 0x43f, 0x430, 0x43b, 0x44c, 0x421, 0x43a, 0x43e, 0x43f, 0x2019,
0x435, 0x421, 0x430, 0x444, 0x456, 0x44f, 0x421, 0x442, 0x430, 0x43a, 0x433, 0x43e,
0x43b, 0x44c, 0x43c, 0x422, 0x430, 0x43b, 0x456, 0x43d, 0x422, 0x44b, 0x440, 0x430,
0x43d, 0x430, 0x423, 0x43b, 0x44c, 0x44f, 0x43d, 0x430, 0x45e, 0x441, 0x43a, 0x412,
0x430, 0x442, 0x44b, 0x43a, 0x430, 0x43d, 0x412, 0x456, 0x43b, 0x44c, 0x43d, 0x44e,
0x441, 0x412, 0x430, 0x43b, 0x433, 0x430, 0x433, 0x440, 0x430, 0x434, 0x417, 0x430,
0x433, 0x440, 0x430, 0x431, 0x426, 0x44e, 0x440, 0x44b, 0x445, 0x410, 0x43d, 0x442,
0x430, 0x43d, 0x430, 0x43d, 0x430, 0x440, 0x44b, 0x432, 0x443, 0x427, 0x430, 0x433,
0x430, 0x441, 0x412, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x45e, 0x20, 0x41a, 0x430,
0x43b, 0x44f, 0x434, 0x41a, 0x430, 0x43a, 0x43e, 0x441, 0x430, 0x432, 0x44b, 0x44f,
0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x44b, 0x41a, 0x430, 0x43c, 0x43e,
0x440, 0x441, 0x43a, 0x456, 0x44f, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432,
0x44b, 0x41c, 0x430, 0x44d, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x44b, 0x432, 0x44b,
0x41c, 0x430, 0x45e, 0x440, 0x44b, 0x43a, 0x456, 0x439, 0x41c, 0x430, 0x451, 0x442,
0x430, 0x420, 0x44d, 0x44e, 0x43d, 0x44c, 0x451, 0x43d, 0x410, 0x43f, 0x456, 0x44f,
0x411, 0x443, 0x433, 0x435, 0x43d, 0x432, 0x456, 0x43b, 0x44c, 0x427, 0x430, 0x442,
0x44d, 0x43c, 0x412, 0x44f, 0x43b, 0x456, 0x43a, 0x430, 0x434, 0x43d, 0x44f, 0x20,
0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x45e, 0x42d, 0x444, 0x430, 0x442, 0x44d,
0x424, 0x430, 0x43a, 0x430, 0x43e, 0x444, 0x430, 0x424, 0x456, 0x434, 0x436, 0x44b,
0x424, 0x443, 0x43d, 0x430, 0x444, 0x443, 0x446, 0x456, 0x413, 0x430, 0x43b, 0x430,
0x43f, 0x430, 0x433, 0x430, 0x441, 0x410, 0x441, 0x442, 0x440, 0x430, 0x432, 0x44b,
0x20, 0x413, 0x430, 0x43c, 0x431, 0x2019, 0x435, 0x413, 0x430, 0x43d, 0x430, 0x43b,
0x443, 0x43b, 0x443, 0x42d, 0x43d, 0x434, 0x44d, 0x440, 0x431, 0x435, 0x440, 0x44b,
0x41a, 0x456, 0x440, 0x44b, 0x446, 0x456, 0x43c, 0x430, 0x446, 0x456, 0x41a, 0x443,
0x441, 0x430, 0x456, 0x435, 0x41a, 0x432, 0x430, 0x434, 0x436, 0x430, 0x43b, 0x435,
0x439, 0x43d, 0x41c, 0x430, 0x434, 0x436, 0x443, 0x440, 0x430, 0x41c, 0x430, 0x440,
0x43a, 0x456, 0x437, 0x441, 0x43a, 0x456, 0x44f, 0x20, 0x430, 0x441, 0x442, 0x440,
0x430, 0x432, 0x44b, 0x41c, 0x456, 0x434, 0x443, 0x44d, 0x439, 0x41d, 0x456, 0x443,
0x44d, 0x41d, 0x43e, 0x440, 0x444, 0x430, 0x43b, 0x43a, 0x41f, 0x430, 0x433, 0x430,
0x2d, 0x41f, 0x430, 0x433, 0x430, 0x41f, 0x456, 0x442, 0x43a, 0x44d, 0x440, 0x43d,
0x41f, 0x430, 0x43d, 0x43f, 0x435, 0x456, 0x41f, 0x43e, 0x440, 0x442, 0x2d, 0x41c,
0x43e, 0x440, 0x441, 0x431, 0x456, 0x420, 0x430, 0x440, 0x430, 0x442, 0x43e, 0x43d,
0x433, 0x430, 0x421, 0x430, 0x439, 0x43f, 0x430, 0x43d, 0x422, 0x430, 0x456, 0x446,
0x456, 0x422, 0x430, 0x43d, 0x433, 0x430, 0x442, 0x430, 0x43f, 0x443, 0x423, 0x44d,
0x439, 0x43a, 0x423, 0x43e, 0x43b, 0x456, 0x441, 0x905, 0x92c, 0x93f, 0x926, 0x91c,
0x93e, 0x928, 0x906, 0x915, 0x94d, 0x930, 0x93e, 0x90f, 0x926, 0x94d, 0x926, 0x93f,
0x938, 0x20, 0x906, 0x92c, 0x93e, 0x92c, 0x93e, 0x906, 0x932, 0x94d, 0x91c, 0x940,
0x90f, 0x930, 0x94d, 0x938, 0x906, 0x938, 0x94d, 0x92e, 0x93e, 0x930, 0x93e, 0x92c,
0x93e, 0x92e, 0x93e, 0x915, 0x2019, 0x92c, 0x93e, 0x902, 0x917, 0x942, 0x908, 0x92c,
0x928, 0x94d, 0x91c, 0x941, 0x932, 0x92c, 0x93f, 0x938, 0x94d, 0x938, 0x93e, 0x909,
0x92c, 0x94d, 0x932, 0x93e, 0x928, 0x94d, 0x91f, 0x93e, 0x908, 0x930, 0x92c, 0x94d,
0x930, 0x93e, 0x91c, 0x94d, 0x91c, 0x93e, 0x92d, 0x940, 0x932, 0x92c, 0x941, 0x91c,
0x941, 0x92e, 0x94d, 0x92c, 0x941, 0x930, 0x93e, 0x915, 0x93e, 0x92f, 0x930, 0x2019,
0x915, 0x93e, 0x938, 0x93e, 0x92c, 0x94d, 0x932, 0x93e, 0x902, 0x915, 0x93e, 0x938,
0x947, 0x909, 0x924, 0x93e, 0x915, 0x928, 0x93e, 0x915, 0x94d, 0x930, 0x940, 0x921,
0x93e, 0x915, 0x93e, 0x930, 0x926, 0x93e, 0x930, 0x20, 0x90f, 0x938, 0x20, 0x938,
0x932, 0x93e, 0x92e, 0x91c, 0x93f, 0x92c, 0x94c, 0x91f, 0x940, 0x926, 0x941, 0x906,
0x932, 0x93e, 0x90f, 0x932, 0x20, 0x906, 0x907, 0x90a, 0x928, 0x92b, 0x94d, 0x930,
0x940, 0x91f, 0x93e, 0x909, 0x928, 0x917, 0x948, 0x92c, 0x94b, 0x930, 0x94b, 0x928,
0x939, 0x93e, 0x930, 0x93e, 0x930, 0x947, 0x91c, 0x939, 0x93e, 0x928, 0x94d, 0x938,
0x92c, 0x93e, 0x930, 0x94d, 0x917, 0x91c, 0x941, 0x92c, 0x93e, 0x915, 0x93e, 0x92e,
0x94d, 0x92a, 0x93e, 0x932, 0x93e, 0x916, 0x93e, 0x930, 0x94d, 0x924, 0x942, 0x92e,
0x915, 0x93f, 0x917, 0x93e, 0x932, 0x93f, 0x915, 0x93f, 0x928, 0x94d, 0x936, 0x93e,
0x938, 0x93e, 0x932, 0x93e, 0x917, 0x2019, 0x938, 0x932, 0x93f, 0x92c, 0x94d, 0x930,
0x947, 0x92d, 0x93f, 0x932, 0x932, 0x92e, 0x947, 0x932, 0x941, 0x906, 0x928, 0x94d,
0x921, 0x93e, 0x932, 0x941, 0x92c, 0x941, 0x92e, 0x94d, 0x92c, 0x93e, 0x936, 0x940,
0x932, 0x941, 0x938, 0x93e, 0x915, 0x93e, 0x92e, 0x93e, 0x932, 0x93e, 0x92c, 0x2019,
0x92e, 0x93e, 0x92a, 0x941, 0x91f, 0x2019, 0x92e, 0x93e, 0x938, 0x947, 0x930, 0x942,
0x92e, 0x94d, 0x92c, 0x93e, 0x92c, 0x93e, 0x928, 0x947, 0x92e, 0x917, 0x93e, 0x921,
0x93f, 0x936, 0x941, 0x92e, 0x928, 0x94d, 0x930, 0x2019, 0x92d, 0x93f, 0x92f, 0x93e,
0x928, 0x93e, 0x908, 0x930, 0x94c, 0x92c, 0x940, 0x928, 0x94d, 0x91c, 0x93e, 0x92e,
0x947, 0x928, 0x93e, 0x928, 0x940, 0x906, 0x92e, 0x947, 0x928, 0x94d, 0x935, 0x93e,
0x915, 0x936, 0x2019, 0x924, 0x90a, 0x917, 0x93e, 0x921, 0x941, 0x917, 0x941, 0x92a,
0x930, 0x94d, 0x91f, 0x2019, 0x2d, 0x928, 0x92d, 0x2019, 0x938, 0x93e, 0x935, 0x20,
0x91f, 0x92e, 0x947, 0x924, 0x94d, 0x930, 0x93f, 0x92a, 0x2019, 0x932, 0x940, 0x91f,
0x94d, 0x92f, 0x942, 0x928, 0x93f, 0x938, 0x935, 0x93f, 0x928, 0x94d, 0x926, 0x939,
0x2019, 0x915, 0x90f, 0x921, 0x947, 0x915, 0x90f, 0x902, 0x915, 0x930, 0x947, 0x91c,
0x90f, 0x902, 0x917, 0x94d, 0x935, 0x940, 0x932, 0x93e, 0x90f, 0x928, 0x94d, 0x91f,
0x93f, 0x917, 0x941, 0x935, 0x93e, 0x906, 0x930, 0x93e, 0x917, 0x941, 0x906, 0x908,
0x928, 0x93e, 0x92c, 0x941, 0x90f, 0x928, 0x2019, 0x938, 0x20, 0x90f, 0x930, 0x947,
0x938, 0x915, 0x93e, 0x91f, 0x93e, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x93e, 0x915,
0x930, 0x94d, 0x921, 0x92c, 0x93e, 0x91c, 0x941, 0x91c, 0x941, 0x908, 0x932, 0x93e,
0x20, 0x930, 0x93f, 0x905, 0x91c, 0x93e, 0x92e, 0x947, 0x928, 0x94d, 0x921, 0x2019,
0x91c, 0x93e, 0x930, 0x93f, 0x905, 0x20, 0x917, 0x93e, 0x932, 0x947, 0x917, 0x938,
0x938, 0x93e, 0x932, 0x94d, 0x91f, 0x93e, 0x938, 0x93e, 0x928, 0x20, 0x939, 0x941,
0x935, 0x93e, 0x928, 0x938, 0x93e, 0x928, 0x20, 0x932, 0x941, 0x908, 0x938, 0x91f,
0x941, 0x915, 0x941, 0x92e, 0x93e, 0x928, 0x909, 0x936, 0x942, 0x906, 0x92f, 0x93f,
0x92f, 0x93e, 0x906, 0x930, 0x942, 0x92c, 0x93e, 0x906, 0x938, 0x941, 0x928, 0x938,
0x940, 0x905, 0x928, 0x906, 0x924, 0x93f, 0x915, 0x915, 0x93e, 0x928, 0x92c, 0x93e,
0x939, 0x93f, 0x92f, 0x93e, 0x92c, 0x93e, 0x939, 0x93f, 0x92f, 0x93e, 0x20, 0x92c,
0x93e, 0x928, 0x94d, 0x926, 0x947, 0x930, 0x93e, 0x938, 0x92c, 0x93e, 0x930, 0x94d,
0x92c, 0x93e, 0x921, 0x2019, 0x938, 0x92c, 0x947, 0x932, 0x947, 0x92e, 0x92c, 0x947,
0x932, 0x93f, 0x91c, 0x92c, 0x94d, 0x932, 0x93e, 0x902, 0x915, 0x2d, 0x20, 0x938,
0x93e, 0x92c, 0x94d, 0x932, 0x928, 0x92c, 0x906, 0x20, 0x92d, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x92c, 0x917, 0x2019, 0x91f, 0x93e, 0x92c, 0x92f, 0x938, 0x93f, 0x915,
0x947, 0x92e, 0x94d, 0x92c, 0x94d, 0x930, 0x93f, 0x91c, 0x20, 0x917, 0x93e, 0x925,
0x94b, 0x928, 0x915, 0x947, 0x92e, 0x94d, 0x92a, 0x20, 0x917, 0x94d, 0x930, 0x93e,
0x928, 0x94d, 0x921, 0x915, 0x93e, 0x902, 0x915, 0x941, 0x928, 0x915, 0x93e, 0x930,
0x93e, 0x915, 0x93e, 0x938, 0x915, 0x93e, 0x92f, 0x947, 0x928, 0x915, 0x948, 0x92e,
0x947, 0x928, 0x936, 0x93f, 0x915, 0x93e, 0x917, 0x2019, 0x91a, 0x941, 0x939, 0x92d,
0x93e, 0x92d, 0x93e, 0x915, 0x938, 0x94d, 0x91f, 0x93e, 0x20, 0x930, 0x93f, 0x915,
0x93e, 0x915, 0x94d, 0x930, 0x947, 0x938, 0x94d, 0x91f, 0x928, 0x915, 0x941, 0x908,
0x906, 0x92c, 0x93e, 0x915, 0x942, 0x930, 0x93e, 0x938, 0x93e, 0x935, 0x921, 0x93e,
0x928, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x938, 0x94d, 0x92d, 0x93e, 0x928, 0x921,
0x93e, 0x935, 0x938, 0x2019, 0x928, 0x921, 0x93e, 0x935, 0x938, 0x2019, 0x928, 0x20,
0x915, 0x94d, 0x930, 0x940, 0x915, 0x921, 0x947, 0x928, 0x92d, 0x94b, 0x930, 0x921,
0x947, 0x91f, 0x94d, 0x930, 0x2019, 0x908, 0x924, 0x921, 0x92e, 0x93f, 0x928, 0x93f,
0x915, 0x93e, 0x90f, 0x921, 0x92e, 0x928, 0x94d, 0x91f, 0x928, 0x90f, 0x908, 0x930,
0x941, 0x928, 0x947, 0x92a, 0x947, 0x90f, 0x932, 0x20, 0x938, 0x93e, 0x932, 0x94d,
0x92d, 0x93e, 0x921, 0x930, 0x92b, 0x930, 0x94d, 0x924, 0x20, 0x928, 0x947, 0x932,
0x938, 0x928, 0x92b, 0x930, 0x94d, 0x91f, 0x93e, 0x932, 0x947, 0x91c, 0x93e, 0x917,
0x94d, 0x932, 0x947, 0x938, 0x20, 0x92c, 0x947, 0x907, 0x917, 0x942, 0x91c, 0x20,
0x92c, 0x947, 0x907, 0x917, 0x94d, 0x930, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x924,
0x93e, 0x930, 0x94d, 0x915, 0x917, 0x94d, 0x930, 0x947, 0x928, 0x93e, 0x921, 0x93e,
0x917, 0x94d, 0x935, 0x93e, 0x926, 0x947, 0x932, 0x942, 0x92a, 0x917, 0x94d, 0x935,
0x93e, 0x91f, 0x947, 0x92e, 0x93e, 0x932, 0x93e, 0x917, 0x941, 0x906, 0x92f, 0x93e,
0x915, 0x93f, 0x932, 0x917, 0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x939, 0x947, 0x932,
0x93f, 0x92b, 0x947, 0x915, 0x94d, 0x938, 0x939, 0x93e, 0x92d, 0x93e, 0x928, 0x93e,
0x939, 0x93e, 0x930, 0x94d, 0x92e, 0x2019, 0x938, 0x93f, 0x932, 0x94d, 0x932, 0x2019,
0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92a, 0x932, 0x93f,
0x938, 0x928, 0x915, 0x94d, 0x938, 0x2c, 0x20, 0x907, 0x923, 0x94d, 0x921, 0x93f,
0x92f, 0x93e, 0x928, 0x93e, 0x92e, 0x93e, 0x930, 0x947, 0x919, 0x2019, 0x2c, 0x20,
0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92a, 0x93f, 0x924,
0x93e, 0x930, 0x94d, 0x938, 0x92c, 0x93e, 0x930, 0x94d, 0x917, 0x2c, 0x20, 0x907,
0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x924, 0x947, 0x932, 0x20,
0x928, 0x94b, 0x917, 0x94b, 0x930, 0x2c, 0x20, 0x907, 0x923, 0x94d, 0x921, 0x93f,
0x92f, 0x93e, 0x928, 0x93e, 0x92d, 0x947, 0x92d, 0x948, 0x2c, 0x20, 0x907, 0x923,
0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92d, 0x93e, 0x928, 0x938, 0x947,
0x928, 0x94d, 0x928, 0x947, 0x938, 0x2c, 0x20, 0x907, 0x923, 0x94d, 0x921, 0x93f,
0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x928, 0x93e, 0x92e, 0x947, 0x915, 0x2c,
0x20, 0x907, 0x923, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x907, 0x928,
0x942, 0x935, 0x93f, 0x915, 0x907, 0x915, 0x93e, 0x932, 0x941, 0x908, 0x924, 0x91c,
0x93e, 0x92e, 0x93e, 0x908, 0x915, 0x93e, 0x91c, 0x941, 0x928, 0x94b, 0x932, 0x941,
0x908, 0x935, 0x940, 0x932, 0x92e, 0x928, 0x94d, 0x91f, 0x93f, 0x91b, 0x947, 0x932,
0x2019, 0x2c, 0x20, 0x915, 0x947, 0x928, 0x94d, 0x91f, 0x93e, 0x915, 0x940, 0x915,
0x94d, 0x930, 0x93e, 0x932, 0x947, 0x923, 0x91c, 0x940, 0x915, 0x932, 0x93e, 0x20,
0x92a, 0x93e, 0x91c, 0x932, 0x940, 0x92e, 0x93e, 0x932, 0x938, 0x20, 0x90f, 0x928,
0x94d, 0x91c, 0x947, 0x932, 0x94b, 0x938, 0x932, 0x935, 0x93e, 0x930, 0x20, 0x92a,
0x94d, 0x930, 0x93f, 0x928, 0x94d, 0x938, 0x94b, 0x938, 0x20, 0x915, 0x94d, 0x935,
0x93e, 0x930, 0x94d, 0x924, 0x93e, 0x930, 0x92e, 0x93e, 0x91a, 0x947, 0x908, 0x913,
0x92e, 0x928, 0x93e, 0x917, 0x941, 0x906, 0x92e, 0x928, 0x93e, 0x909, 0x938, 0x92e,
0x947, 0x930, 0x93f, 0x917, 0x924, 0x92e, 0x93e, 0x930, 0x94d, 0x91f, 0x940, 0x928,
0x940, 0x915, 0x92e, 0x947, 0x924, 0x93e, 0x92e, 0x930, 0x938, 0x92e, 0x93e, 0x91c,
0x93e, 0x91f, 0x932, 0x93e, 0x928, 0x92e, 0x947, 0x928, 0x2019, 0x92e, 0x940, 0x928,
0x940, 0x92e, 0x947, 0x930, 0x93f, 0x921, 0x93e, 0x92e, 0x947, 0x924, 0x94d, 0x932,
0x93e, 0x915, 0x93e, 0x924, 0x94d, 0x932, 0x93e, 0x92e, 0x947, 0x915, 0x94d, 0x938,
0x93f, 0x915, 0x2019, 0x20, 0x938, 0x93f, 0x924, 0x940, 0x92e, 0x93f, 0x915, 0x94d,
0x935, 0x947, 0x932, 0x2019, 0x928, 0x92e, 0x915, 0x94d, 0x91f, 0x2019, 0x928, 0x92e,
0x928, 0x94d, 0x91f, 0x947, 0x930, 0x94d, 0x930, 0x947, 0x92e, 0x928, 0x94d, 0x91f,
0x947, 0x92d, 0x93f, 0x921, 0x93f, 0x905, 0x92e, 0x928, 0x938, 0x947, 0x930, 0x94d,
0x930, 0x93e, 0x924, 0x928, 0x93e, 0x938, 0x94d, 0x938, 0x93e, 0x909, 0x928, 0x93f,
0x909, 0x20, 0x92f, 0x930, 0x94d, 0x915, 0x928, 0x930, 0x2019, 0x928, 0x94d, 0x939,
0x93e, 0x92c, 0x947, 0x90a, 0x932, 0x93e, 0x939, 0x2c, 0x20, 0x938, 0x93e, 0x939,
0x93e, 0x20, 0x921, 0x93e, 0x915, 0x2019, 0x924, 0x93e, 0x938, 0x947, 0x928, 0x94d,
0x91f, 0x94b, 0x930, 0x2c, 0x20, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x921, 0x93e,
0x915, 0x2019, 0x924, 0x93e, 0x928, 0x93f, 0x909, 0x20, 0x938, 0x93e, 0x932, 0x947,
0x92e, 0x2c, 0x20, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x921, 0x93e, 0x915, 0x2019,
0x924, 0x93e, 0x928, 0x942, 0x915, 0x906, 0x91c, 0x93f, 0x928, 0x93e, 0x917, 0x93e,
0x92a, 0x93e, 0x928, 0x93e, 0x92e, 0x93e, 0x92a, 0x93e, 0x930, 0x93e, 0x92e, 0x93e,
0x930, 0x93f, 0x92c, 0x2019, 0x92b, 0x940, 0x928, 0x93f, 0x915, 0x94d, 0x938, 0x92a,
0x930, 0x94d, 0x91f, 0x2d, 0x906, 0x90a, 0x2d, 0x92a, 0x94d, 0x930, 0x93f, 0x928,
0x94d, 0x938, 0x92a, 0x930, 0x94d, 0x91f, 0x20, 0x905, 0x92b, 0x20, 0x938, 0x94d,
0x92a, 0x948, 0x928, 0x92a, 0x930, 0x94d, 0x91f, 0x2019, 0x20, 0x92d, 0x947, 0x932,
0x94d, 0x939, 0x2019, 0x92a, 0x941, 0x90f, 0x930, 0x94d, 0x91f, 0x2019, 0x20, 0x930,
0x93f, 0x915, 0x2019, 0x92a, 0x941, 0x928, 0x94d, 0x91f, 0x93e, 0x20, 0x906, 0x930,
0x947, 0x928, 0x93e, 0x938, 0x930, 0x947, 0x928, 0x915, 0x93f, 0x928, 0x20, 0x907,
0x928, 0x932, 0x947, 0x91f, 0x930, 0x947, 0x938, 0x93f, 0x92b, 0x947, 0x930, 0x947,
0x91c, 0x93f, 0x928, 0x93e, 0x930, 0x947, 0x91c, 0x932, 0x94d, 0x92f, 0x941, 0x924,
0x930, 0x93f, 0x905, 0x20, 0x92c, 0x94d, 0x930, 0x93e, 0x928, 0x94d, 0x915, 0x2019,
0x938, 0x93e, 0x928, 0x94d, 0x924, 0x93e, 0x930, 0x947, 0x92e, 0x938, 0x93e, 0x928,
0x94d, 0x924, 0x93f, 0x906, 0x917, 0x2019, 0x938, 0x93e, 0x928, 0x94d, 0x924, 0x94b,
0x20, 0x921, 0x92e, 0x93f, 0x917, 0x2019, 0x938, 0x93e, 0x935, 0x20, 0x92a, 0x93e,
0x935, 0x932, 0x2019, 0x908, 0x924, 0x94d, 0x924, 0x2019, 0x915, 0x94d, 0x935, 0x930,
0x94d, 0x924, 0x942, 0x92e, 0x940, 0x938, 0x93f, 0x924, 0x94d, 0x915, 0x93e, 0x938,
0x948, 0x928, 0x94d, 0x91f, 0x20, 0x92c, 0x93e, 0x930, 0x94d, 0x925, 0x947, 0x932,
0x947, 0x92e, 0x940, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x91c, 0x928, 0x94d,
0x938, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x915, 0x940, 0x924, 0x94d, 0x938,
0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x932, 0x941, 0x938, 0x93f, 0x92f, 0x93e,
0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x925, 0x92e, 0x93e, 0x938, 0x938, 0x948,
0x928, 0x94d, 0x91f, 0x20, 0x92d, 0x93f, 0x928, 0x94d, 0x938, 0x947, 0x928, 0x94d,
0x91f, 0x938, 0x941, 0x908, 0x92b, 0x924, 0x20, 0x915, 0x93e, 0x930, 0x94d, 0x930,
0x947, 0x928, 0x94d, 0x924, 0x924, 0x947, 0x917, 0x941, 0x938, 0x93f, 0x917, 0x93e,
0x932, 0x94d, 0x92a, 0x93e, 0x925, 0x94d, 0x92f, 0x942, 0x932, 0x91f, 0x93f, 0x939,
0x941, 0x906, 0x928, 0x93e, 0x91f, 0x930, 0x928, 0x94d, 0x91f, 0x2019, 0x91f, 0x930,
0x94d, 0x924, 0x2019, 0x932, 0x93e, 0x92d, 0x947, 0x928, 0x915, 0x942, 0x92d, 0x930,
0x935, 0x94d, 0x939, 0x93e, 0x908, 0x924, 0x939, 0x930, 0x94d, 0x938, 0x935, 0x940,
0x928, 0x940, 0x92a, 0x947, 0x917, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x93e, 0x924,
0x915, 0x947, 0x938, 0x940, 0x921, 0x947, 0x92d, 0x93f, 0x938, 0x921, 0x941, 0x92e,
0x928, 0x94d, 0x924, 0x2d, 0x921, 0x940, 0x906, 0x930, 0x94d, 0x92d, 0x940, 0x932,
0x92e, 0x93e, 0x915, 0x94d, 0x935, 0x93e, 0x930, 0x940, 0x92e, 0x94c, 0x938, 0x928,
0x92e, 0x947, 0x915, 0x92e, 0x93e, 0x930, 0x94d, 0x921, 0x2019, 0x92a, 0x93e, 0x932,
0x92e, 0x93e, 0x930, 0x930, 0x925, 0x947, 0x930, 0x93e, 0x938, 0x940, 0x905, 0x935,
0x93e, 0x924, 0x94d, 0x930, 0x2019, 0x932, 0x92d, 0x938, 0x94d, 0x924, 0x2019, 0x915,
0x932, 0x902, 0x92f, 0x93e, 0x930, 0x92c, 0x94d, 0x92f, 0x947, 0x928, 0x90f, 0x921,
0x947, 0x928, 0x906, 0x932, 0x94d, 0x92e, 0x93e, 0x91f, 0x940, 0x906, 0x92e, 0x94d,
0x92e, 0x93e, 0x928, 0x906, 0x928, 0x93e, 0x921, 0x940, 0x930, 0x905, 0x915, 0x94d,
0x924, 0x93e, 0x909, 0x906, 0x915, 0x94d, 0x924, 0x2019, 0x92c, 0x947, 0x905, 0x936,
0x94d, 0x917, 0x93e, 0x92c, 0x93e, 0x924, 0x906, 0x924, 0x940, 0x930, 0x93e, 0x909,
0x92c, 0x93e, 0x918, 0x926, 0x93e, 0x926, 0x92c, 0x93e, 0x939, 0x930, 0x948, 0x928,
0x92c, 0x948, 0x902, 0x917, 0x915, 0x949, 0x915, 0x92c, 0x93e, 0x930, 0x94d, 0x928,
0x94c, 0x932, 0x92c, 0x948, 0x930, 0x941, 0x924, 0x92c, 0x93f, 0x936, 0x94d, 0x915,
0x947, 0x915, 0x92c, 0x94d, 0x930, 0x941, 0x928, 0x907, 0x91a, 0x940, 0x924, 0x93e,
0x91a, 0x2019, 0x907, 0x92c, 0x93e, 0x932, 0x938, 0x93e, 0x928, 0x915, 0x932, 0x92e,
0x94d, 0x92c, 0x2019, 0x926, 0x93e, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x938,
0x927, 0x93e, 0x915, 0x93e, 0x921, 0x93f, 0x932, 0x940, 0x926, 0x941, 0x92c, 0x93e,
0x908, 0x926, 0x941, 0x936, 0x93e, 0x928, 0x94d, 0x92c, 0x947, 0x92b, 0x93e, 0x92e,
0x93e, 0x917, 0x941, 0x938, 0x94d, 0x924, 0x93e, 0x917, 0x93e, 0x91c, 0x93e, 0x939,
0x947, 0x92c, 0x94d, 0x930, 0x2019, 0x928, 0x938, 0x93e, 0x907, 0x917, 0x949, 0x928,
0x939, 0x902, 0x915, 0x902, 0x939, 0x2019, 0x92d, 0x94d, 0x926, 0x908, 0x930, 0x94d,
0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x91c, 0x93e, 0x915, 0x93e, 0x930,
0x94d, 0x924, 0x93e, 0x91c, 0x92f, 0x93e, 0x92a, 0x941, 0x930, 0x93e, 0x91c, 0x947,
0x930, 0x941, 0x91c, 0x93e, 0x932, 0x947, 0x92e, 0x915, 0x93e, 0x92c, 0x942, 0x932,
0x915, 0x93e, 0x92e, 0x91a, 0x93e, 0x91f, 0x915, 0x93e, 0x915, 0x930, 0x93e, 0x91a,
0x940, 0x915, 0x93e, 0x920, 0x92e, 0x93e, 0x928, 0x94d, 0x921, 0x941, 0x916, 0x93e,
0x928, 0x94d, 0x926, 0x940, 0x917, 0x93e, 0x915, 0x932, 0x915, 0x93e, 0x924, 0x93e,
0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x2019, 0x92f, 0x93e, 0x930, 0x94d,
0x938, 0x94d, 0x915, 0x915, 0x94d, 0x935, 0x93e, 0x932, 0x93e, 0x20, 0x932, 0x92e,
0x94d, 0x92a, 0x941, 0x930, 0x915, 0x942, 0x91a, 0x93f, 0x902, 0x917, 0x915, 0x941,
0x935, 0x948, 0x924, 0x92e, 0x93e, 0x915, 0x93e, 0x90a, 0x92e, 0x93e, 0x917, 0x93e,
0x926, 0x93e, 0x928, 0x92e, 0x93e, 0x915, 0x93e, 0x938, 0x94d, 0x938, 0x93e, 0x930,
0x92e, 0x928, 0x940, 0x932, 0x93e, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x947, 0x91f,
0x928, 0x93f, 0x915, 0x2019, 0x938, 0x93f, 0x92f, 0x93e, 0x928, 0x92d, 0x2019, 0x915,
0x941, 0x91c, 0x94d, 0x928, 0x947, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x928, 0x92d,
0x2019, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x905, 0x92e,
0x94d, 0x938, 0x94d, 0x915, 0x905, 0x930, 0x93e, 0x932, 0x928, 0x949, 0x92e, 0x20,
0x92a, 0x947, 0x928, 0x92a, 0x94b, 0x902, 0x924, 0x93f, 0x906, 0x928, 0x93e, 0x915,
0x92a, 0x94d, 0x92f, 0x902, 0x92f, 0x93e, 0x902, 0x915, 0x93e, 0x924, 0x93e, 0x930,
0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x948, 0x915, 0x940, 0x91c, 0x940, 0x932,
0x930, 0x94d, 0x921, 0x93e, 0x930, 0x93f, 0x92f, 0x93e, 0x927, 0x938, 0x93e, 0x916,
0x93e, 0x932, 0x93f, 0x928, 0x938, 0x92e, 0x930, 0x915, 0x93e, 0x928, 0x94d, 0x926,
0x938, 0x2019, 0x909, 0x932, 0x936, 0x93e, 0x902, 0x918, 0x93e, 0x908, 0x938, 0x93f,
0x902, 0x917, 0x93e, 0x92a, 0x941, 0x930, 0x938, 0x94d, 0x930, 0x947, 0x926, 0x928,
0x947, 0x915, 0x2019, 0x932, 0x93f, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x924, 0x93e,
0x908, 0x92a, 0x947, 0x907, 0x924, 0x93e, 0x936, 0x915, 0x947, 0x928, 0x94d, 0x924,
0x924, 0x93f, 0x92c, 0x93f, 0x932, 0x93f, 0x938, 0x940, 0x924, 0x947, 0x939, 0x930,
0x93e, 0x928, 0x925, 0x93f, 0x92e, 0x94d, 0x92b, 0x941, 0x91f, 0x2019, 0x915, 0x94d,
0x92f, 0x2019, 0x924, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x909, 0x932, 0x93e, 0x928,
0x92c, 0x93e, 0x924, 0x93e, 0x930, 0x909, 0x930, 0x941, 0x92e, 0x915, 0x940, 0x906,
0x938, 0x94d, 0x924, 0x2d, 0x928, 0x947, 0x930, 0x93e, 0x935, 0x93f, 0x92f, 0x902,
0x938, 0x93f, 0x92f, 0x93e, 0x928, 0x92d, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92d,
0x938, 0x94d, 0x924, 0x2019, 0x915, 0x92f, 0x93e, 0x915, 0x942, 0x924, 0x94d, 0x938,
0x94d, 0x915, 0x930, 0x902, 0x917, 0x942, 0x928, 0x92f, 0x947, 0x915, 0x93e, 0x924,
0x947, 0x930, 0x940, 0x928, 0x93e, 0x92c, 0x941, 0x930, 0x94d, 0x917, 0x92f, 0x947,
0x930, 0x947, 0x92d, 0x93e, 0x928, 0x906, 0x91c, 0x2019, 0x930, 0x94d, 0x938, 0x92c,
0x93e, 0x930, 0x94d, 0x92e, 0x941, 0x921, 0x93e, 0x915, 0x947, 0x928, 0x947, 0x930,
0x940, 0x915, 0x947, 0x92a, 0x20, 0x92d, 0x947, 0x930, 0x94d, 0x926, 0x947, 0x92b,
0x947, 0x930, 0x94c, 0x92e, 0x93e, 0x921, 0x948, 0x930, 0x93e, 0x930, 0x940, 0x915,
0x92f, 0x93e, 0x92d, 0x93f, 0x915, 0x916, 0x94b, 0x932, 0x93e, 0x20, 0x91c, 0x930,
0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x939,
0x947, 0x932, 0x947, 0x928, 0x93e, 0x938, 0x94d, 0x91f, 0x947, 0x928, 0x932, 0x940,
0x90f, 0x921, 0x947, 0x932, 0x948, 0x921, 0x92c, 0x94d, 0x930, 0x940, 0x938, 0x92c,
0x947, 0x928, 0x92c, 0x94d, 0x930, 0x2019, 0x915, 0x947, 0x928, 0x20, 0x939, 0x93f,
0x932, 0x921, 0x93e, 0x930, 0x94d, 0x935, 0x940, 0x928, 0x92f, 0x942, 0x915, 0x94d,
0x932, 0x93e, 0x939, 0x92c, 0x93e, 0x930, 0x94d, 0x91f, 0x932, 0x93f, 0x923, 0x94d,
0x921, 0x92e, 0x947, 0x928, 0x932, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935,
0x93f, 0x92e, 0x947, 0x932, 0x92c, 0x94c, 0x930, 0x94d, 0x928, 0x92a, 0x94b, 0x930,
0x94d, 0x925, 0x938, 0x93f, 0x921, 0x928, 0x940, 0x910, 0x92e, 0x94d, 0x938, 0x94d,
0x91f, 0x930, 0x921, 0x93e, 0x92e, 0x906, 0x928, 0x94d, 0x921, 0x2019, 0x930, 0x93e,
0x906, 0x938, 0x94d, 0x924, 0x94d, 0x930, 0x93e, 0x916, 0x93e, 0x928, 0x90f, 0x925,
0x947, 0x928, 0x94d, 0x938, 0x92c, 0x947, 0x932, 0x917, 0x94d, 0x930, 0x947, 0x921,
0x92c, 0x93e, 0x930, 0x94d, 0x932, 0x93f, 0x928, 0x92c, 0x94d, 0x930, 0x93e, 0x924,
0x93f, 0x938, 0x94d, 0x932, 0x93e, 0x92d, 0x93e, 0x92c, 0x94d, 0x930, 0x941, 0x938,
0x94d, 0x938, 0x947, 0x932, 0x94d, 0x938, 0x92c, 0x941, 0x915, 0x93e, 0x930, 0x947,
0x938, 0x94d, 0x91f, 0x92c, 0x941, 0x926, 0x93e, 0x92a, 0x947, 0x938, 0x94d, 0x91f,
0x92c, 0x941, 0x938, 0x93f, 0x902, 0x917, 0x947, 0x928, 0x91a, 0x93f, 0x938, 0x940,
0x928, 0x94c, 0x915, 0x92a, 0x947, 0x928, 0x939, 0x947, 0x917, 0x947, 0x928, 0x921,
0x93e, 0x92c, 0x932, 0x93f, 0x928, 0x91c, 0x93f, 0x92c, 0x94d, 0x930, 0x93e, 0x932,
0x94d, 0x91f, 0x93e, 0x930, 0x917, 0x93e, 0x930, 0x94d, 0x928, 0x938, 0x940, 0x939,
0x947, 0x932, 0x94d, 0x938, 0x93f, 0x902, 0x915, 0x940, 0x906, 0x92f, 0x94b, 0x932,
0x20, 0x905, 0x92b, 0x20, 0x92e, 0x947, 0x928, 0x907, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x92c, 0x941, 0x932, 0x91c, 0x93e, 0x930, 0x94d, 0x938, 0x940, 0x915, 0x93e,
0x932, 0x93f, 0x928, 0x940, 0x902, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x915, 0x93f,
0x930, 0x2019, 0x92d, 0x915, 0x940, 0x92d, 0x932, 0x93f, 0x938, 0x94d, 0x92c, 0x928,
0x91c, 0x941, 0x92c, 0x94d, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x93e, 0x932, 0x928,
0x94d, 0x926, 0x928, 0x932, 0x93e, 0x915, 0x94d, 0x91c, 0x947, 0x92e, 0x94d, 0x92c,
0x93e, 0x930, 0x94d, 0x917, 0x92e, 0x93e, 0x926, 0x94d, 0x930, 0x93f, 0x926, 0x92e,
0x93e, 0x932, 0x94d, 0x91f, 0x93e, 0x92e, 0x93e, 0x930, 0x948, 0x939, 0x93e, 0x92e,
0x92e, 0x93f, 0x928, 0x94d, 0x938, 0x94d, 0x915, 0x92e, 0x928, 0x93e, 0x915, 0x2019,
0x92e, 0x938, 0x94d, 0x915, 0x94c, 0x905, 0x938, 0x94d, 0x932, 0x2019, 0x92a, 0x947,
0x930, 0x93f, 0x938, 0x92a, 0x926, 0x917, 0x2019, 0x930, 0x93f, 0x915, 0x93e, 0x92a,
0x94d, 0x930, 0x947, 0x917, 0x930, 0x93f, 0x917, 0x93e, 0x930, 0x2019, 0x92e, 0x938,
0x93e, 0x92e, 0x93e, 0x930, 0x93e, 0x938, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x930,
0x93f, 0x928, 0x2019, 0x938, 0x93e, 0x930, 0x93e, 0x91c, 0x947, 0x92d, 0x2019, 0x938,
0x93e, 0x930, 0x93e, 0x924, 0x2019, 0x92d, 0x938, 0x93f, 0x92e, 0x94d, 0x92b, 0x947,
0x930, 0x2019, 0x92a, 0x2019, 0x932, 0x938, 0x94d, 0x915, 0x2019, 0x92a, 0x94d, 0x91c,
0x947, 0x938, 0x92b, 0x93f, 0x92f, 0x93e, 0x938, 0x94d, 0x91f, 0x2019, 0x915, 0x939,
0x2019, 0x92e, 0x91f, 0x947, 0x932, 0x93f, 0x928, 0x91f, 0x93f, 0x930, 0x947, 0x928,
0x90a, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x2019, 0x92d, 0x94d, 0x938, 0x94d, 0x915,
0x92d, 0x93e, 0x921, 0x941, 0x91c, 0x92d, 0x947, 0x91f, 0x93f, 0x915, 0x93e, 0x928,
0x92d, 0x93f, 0x92f, 0x947, 0x928, 0x94d, 0x928, 0x93e, 0x935, 0x93f, 0x932, 0x94d,
0x928, 0x93f, 0x909, 0x938, 0x92d, 0x932, 0x94d, 0x917, 0x2019, 0x917, 0x94d, 0x930,
0x947, 0x926, 0x935, 0x93e, 0x930, 0x938, 0x94c, 0x91c, 0x93e, 0x917, 0x94d, 0x930,
0x947, 0x92c, 0x91c, 0x941, 0x930, 0x93f, 0x916, 0x90f, 0x928, 0x94d, 0x91f, 0x93e,
0x928, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x92d, 0x2019, 0x91a, 0x93e, 0x917, 0x2019,
0x938, 0x916, 0x94d, 0x930, 0x940, 0x938, 0x94d, 0x91f, 0x92e, 0x93e, 0x938, 0x20,
0x926, 0x94d, 0x935, 0x940, 0x92a, 0x915, 0x915, 0x2019, 0x938, 0x915, 0x92e, 0x2019,
0x930, 0x2019, 0x915, 0x947, 0x930, 0x94d, 0x917, 0x947, 0x932, 0x947, 0x928, 0x92e,
0x93e, 0x939, 0x947, 0x92e, 0x93e, 0x932, 0x926, 0x940, 0x92d, 0x94d, 0x938, 0x92e,
0x930, 0x93f, 0x936, 0x93f, 0x92f, 0x93e, 0x938, 0x92e, 0x93e, 0x92f, 0x2019, 0x924,
0x94d, 0x924, 0x947, 0x930, 0x947, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x2019, 0x928,
0x906, 0x92a, 0x93f, 0x92f, 0x93e, 0x914, 0x915, 0x932, 0x948, 0x923, 0x94d, 0x921,
0x92c, 0x941, 0x917, 0x948, 0x928, 0x92d, 0x940, 0x932, 0x91a, 0x93e, 0x925, 0x93e,
0x92e, 0x91a, 0x942, 0x915, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930,
0x93f, 0x90f, 0x92b, 0x93e, 0x91f, 0x947, 0x92b, 0x93e, 0x915, 0x93e, 0x905, 0x92b,
0x2019, 0x92b, 0x93f, 0x91c, 0x940, 0x92b, 0x941, 0x928, 0x93e, 0x92b, 0x942, 0x91f,
0x940, 0x917, 0x93e, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x2019, 0x938, 0x917, 0x93e,
0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x947, 0x930, 0x917, 0x94d, 0x935, 0x93e, 0x921,
0x93e, 0x932, 0x915, 0x947, 0x928, 0x947, 0x932, 0x917, 0x941, 0x935, 0x93e, 0x92e,
0x939, 0x928, 0x932, 0x941, 0x932, 0x941, 0x90f, 0x928, 0x94d, 0x921, 0x93e, 0x930,
0x92c, 0x93e, 0x930, 0x940, 0x915, 0x93f, 0x930, 0x940, 0x924, 0x93f, 0x92e, 0x93e,
0x924, 0x93f, 0x915, 0x938, 0x94d, 0x930, 0x947, 0x915, 0x94d, 0x935, 0x93e, 0x91c,
0x93e, 0x932, 0x948, 0x928, 0x92e, 0x93e, 0x91c, 0x942, 0x930, 0x2019, 0x92e, 0x93e,
0x930, 0x94d, 0x915, 0x947, 0x938, 0x93e, 0x938, 0x92e, 0x940, 0x921, 0x935, 0x947,
0x928, 0x93e, 0x90a, 0x930, 0x941, 0x928, 0x940, 0x90a, 0x90f, 0x928, 0x930, 0x92b,
0x2019, 0x915, 0x928, 0x942, 0x92e, 0x93f, 0x92f, 0x93e, 0x92a, 0x93e, 0x917, 0x2019,
0x20, 0x92a, 0x93e, 0x917, 0x2019, 0x92a, 0x93e, 0x932, 0x93e, 0x909, 0x92a, 0x93f,
0x91f, 0x915, 0x948, 0x930, 0x94d, 0x928, 0x92a, 0x928, 0x93e, 0x92a, 0x947, 0x92a,
0x930, 0x94d, 0x91f, 0x20, 0x92e, 0x930, 0x94d, 0x938, 0x92c, 0x940, 0x930, 0x93e,
0x930, 0x2019, 0x91f, 0x902, 0x917, 0x93e, 0x938, 0x93e, 0x907, 0x92a, 0x93e, 0x928,
0x91f, 0x93e, 0x939, 0x93f, 0x91f, 0x940, 0x924, 0x93e, 0x930, 0x93e, 0x935, 0x93e,
0x91f, 0x902, 0x917, 0x93e, 0x91f, 0x93e, 0x92a, 0x941, 0x935, 0x947, 0x915, 0x935,
0x93e, 0x932, 0x93f, 0x938, 0x4b, 0x61, 0x7a, 0x61, 0x62, 0x6c, 0x61, 0x6e,
0x6b, 0x61, 0x44, 0x17e, 0x69, 0x62, 0x75, 0x74, 0x69, 0x4d, 0x6f, 0x67,
0x61, 0x64, 0x69, 0x161, 0x41, 0x6e, 0x67, 0x76, 0x69, 0x6c, 0x61, 0x41,
0x6e, 0x74, 0x69, 0x67, 0x76, 0x61, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x72,
0x69, 0x6b, 0x61, 0x47, 0x76, 0x61, 0x64, 0x61, 0x6c, 0x75, 0x70, 0x65,
0x47, 0x76, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61, 0x4a, 0x61, 0x6d,
0x61, 0x6a, 0x6b, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20,
0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x53,
0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c,
0x20, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x72, 0x69, 0x6b,
0x6f, 0x41, 0x6b, 0x75, 0x74, 0x6f, 0x62, 0x65, 0x41, 0x161, 0x68, 0x61,
0x62, 0x61, 0x64, 0x41, 0x74, 0x69, 0x72, 0x61, 0x6a, 0x42, 0x69, 0x161,
0x6b, 0x65, 0x6b, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x69, 0x10c, 0x6f,
0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x44, 0x75, 0x161, 0x61, 0x6e,
0x62, 0x65, 0x48, 0x6f, 0x20, 0x160, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x44,
0x17e, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61, 0x44, 0x17e, 0x61, 0x6a, 0x61,
0x70, 0x75, 0x72, 0x61, 0x4a, 0x65, 0x72, 0x75, 0x7a, 0x61, 0x6c, 0x65,
0x6d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72,
0x61, 0x10d, 0x69, 0x48, 0x61, 0x6e, 0x64, 0x69, 0x67, 0x61, 0x4b, 0x75,
0x10d, 0x69, 0x6e, 0x67, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x4e, 0x69, 0x6b,
0x6f, 0x7a, 0x69, 0x6a, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a,
0x6e, 0x6a, 0x65, 0x63, 0x6b, 0x50, 0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61,
0x6e, 0x67, 0x52, 0x69, 0x6a, 0x61, 0x64, 0x53, 0x61, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x160, 0x61, 0x6e, 0x67, 0x61, 0x6a, 0x54, 0x61, 0x161, 0x6b,
0x65, 0x6e, 0x74, 0x55, 0x72, 0x75, 0x6d, 0x10d, 0x69, 0x56, 0x69, 0x6a,
0x65, 0x6e, 0x74, 0x69, 0x6a, 0x61, 0x6e, 0x41, 0x7a, 0x6f, 0x72, 0x69,
0x52, 0x65, 0x6a, 0x6b, 0x6a, 0x61, 0x76, 0x69, 0x6b, 0x53, 0x76, 0x65,
0x74, 0x61, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x6e, 0x64,
0x6f, 0x72, 0x61, 0x41, 0x73, 0x74, 0x72, 0x61, 0x68, 0x61, 0x6e, 0x41,
0x74, 0x69, 0x6e, 0x61, 0x42, 0x72, 0x69, 0x73, 0x65, 0x6c, 0x42, 0x75,
0x6b, 0x75, 0x72, 0x65, 0x161, 0x74, 0x42, 0x75, 0x64, 0x69, 0x6d, 0x70,
0x65, 0x161, 0x74, 0x61, 0x4b, 0x69, 0x161, 0x69, 0x6e, 0x6a, 0x65, 0x76,
0x44, 0x61, 0x62, 0x6c, 0x69, 0x6e, 0x47, 0x65, 0x72, 0x6e, 0x7a, 0x69,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x4d, 0x61, 0x6e, 0x4b, 0x61,
0x6c, 0x69, 0x6e, 0x6a, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x4b, 0x69,
0x6a, 0x65, 0x76, 0x4c, 0x69, 0x73, 0x61, 0x62, 0x6f, 0x6e, 0x50, 0x61,
0x72, 0x69, 0x7a, 0x52, 0x69, 0x6d, 0x53, 0x69, 0x6d, 0x66, 0x65, 0x72,
0x6f, 0x70, 0x6f, 0x6c, 0x6a, 0x53, 0x6b, 0x6f, 0x70, 0x6c, 0x6a, 0x65,
0x53, 0x6f, 0x66, 0x69, 0x6a, 0x61, 0x160, 0x74, 0x6f, 0x6b, 0x68, 0x6f,
0x6c, 0x6d, 0x42, 0x65, 0x10d, 0x56, 0x61, 0x72, 0x161, 0x61, 0x76, 0x61,
0x43, 0x69, 0x72, 0x69, 0x68, 0x42, 0x6f, 0x17e, 0x69, 0x107, 0x6e, 0x6f,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x4b, 0x6f, 0x6b, 0x6f, 0x73,
0x6f, 0x76, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x69, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69,
0x6a, 0x75, 0x73, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x56, 0x61, 0x6c, 0x69,
0x73, 0x410, 0x434, 0x438, 0x441, 0x20, 0x410, 0x431, 0x435, 0x431, 0x430, 0x411,
0x430, 0x43d, 0x433, 0x443, 0x438, 0x411, 0x438, 0x441, 0x430, 0x43e, 0x411, 0x43b,
0x430, 0x43d, 0x442, 0x438, 0x440, 0x411, 0x440, 0x430, 0x437, 0x430, 0x432, 0x438,
0x43b, 0x411, 0x443, 0x45f, 0x443, 0x43c, 0x431, 0x443, 0x440, 0x430, 0x41a, 0x430,
0x438, 0x440, 0x43e, 0x41a, 0x430, 0x437, 0x430, 0x431, 0x43b, 0x430, 0x43d, 0x43a,
0x430, 0x415, 0x43b, 0x20, 0x410, 0x458, 0x443, 0x43d, 0x425, 0x430, 0x440, 0x430,
0x440, 0x435, 0x408, 0x43e, 0x445, 0x430, 0x43d, 0x435, 0x441, 0x431, 0x443, 0x440,
0x433, 0x41a, 0x430, 0x440, 0x442, 0x443, 0x43c, 0x41b, 0x443, 0x43c, 0x443, 0x43c,
0x431, 0x430, 0x448, 0x438, 0x41c, 0x430, 0x43f, 0x443, 0x442, 0x43e, 0x41c, 0x43e,
0x43d, 0x440, 0x43e, 0x432, 0x438, 0x458, 0x430, 0x41d, 0x430, 0x458, 0x440, 0x43e,
0x431, 0x438, 0x41d, 0x438, 0x458, 0x430, 0x43c, 0x435, 0x458, 0x41f, 0x43e, 0x440,
0x442, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x421, 0x430, 0x43e, 0x20, 0x422,
0x43e, 0x43c, 0x435, 0x415, 0x43d, 0x43a, 0x43e, 0x440, 0x438, 0x45f, 0x410, 0x43d,
0x433, 0x432, 0x438, 0x43b, 0x430, 0x410, 0x43d, 0x442, 0x438, 0x433, 0x432, 0x430,
0x410, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x430, 0x43d, 0x430, 0x411, 0x443,
0x435, 0x43d, 0x43e, 0x441, 0x20, 0x410, 0x438, 0x440, 0x435, 0x441, 0x41a, 0x43e,
0x440, 0x434, 0x43e, 0x431, 0x430, 0x425, 0x443, 0x445, 0x443, 0x458, 0x41b, 0x430,
0x20, 0x420, 0x438, 0x43e, 0x445, 0x430, 0x41c, 0x435, 0x43d, 0x434, 0x43e, 0x437,
0x430, 0x420, 0x438, 0x43e, 0x20, 0x413, 0x430, 0x459, 0x435, 0x433, 0x43e, 0x441,
0x421, 0x430, 0x43d, 0x20, 0x425, 0x443, 0x430, 0x43d, 0x421, 0x430, 0x43d, 0x20,
0x41b, 0x443, 0x438, 0x423, 0x448, 0x443, 0x430, 0x438, 0x458, 0x430, 0x410, 0x441,
0x443, 0x43d, 0x441, 0x438, 0x43e, 0x43d, 0x410, 0x442, 0x438, 0x43a, 0x43e, 0x43a,
0x430, 0x43d, 0x411, 0x430, 0x438, 0x458, 0x430, 0x411, 0x430, 0x438, 0x458, 0x430,
0x20, 0x411, 0x430, 0x43d, 0x434, 0x435, 0x440, 0x430, 0x441, 0x411, 0x435, 0x43b,
0x435, 0x43c, 0x411, 0x435, 0x43b, 0x438, 0x437, 0x435, 0x411, 0x43b, 0x430, 0x43d,
0x43a, 0x2d, 0x421, 0x435, 0x458, 0x431, 0x43b, 0x43e, 0x43d, 0x411, 0x43e, 0x430,
0x20, 0x412, 0x438, 0x441, 0x442, 0x430, 0x411, 0x43e, 0x458, 0x437, 0x438, 0x41a,
0x435, 0x43c, 0x431, 0x440, 0x438, 0x45f, 0x20, 0x411, 0x435, 0x458, 0x41a, 0x430,
0x43c, 0x43f, 0x43e, 0x20, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x435, 0x41a, 0x430,
0x458, 0x435, 0x43d, 0x41a, 0x430, 0x458, 0x43c, 0x430, 0x43d, 0x441, 0x43a, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x427, 0x438, 0x432, 0x430, 0x432,
0x430, 0x41a, 0x43e, 0x441, 0x442, 0x430, 0x440, 0x438, 0x43a, 0x430, 0x41a, 0x443,
0x440, 0x430, 0x441, 0x430, 0x43e, 0x414, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x43a,
0x441, 0x445, 0x430, 0x433, 0x435, 0x43d, 0x414, 0x43e, 0x441, 0x43e, 0x43d, 0x414,
0x43e, 0x441, 0x43e, 0x43d, 0x20, 0x41a, 0x440, 0x438, 0x43a, 0x413, 0x43b, 0x435,
0x458, 0x441, 0x20, 0x411, 0x435, 0x458, 0x413, 0x443, 0x441, 0x20, 0x411, 0x435,
0x458, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x413,
0x432, 0x430, 0x434, 0x430, 0x43b, 0x443, 0x43f, 0x435, 0x413, 0x432, 0x430, 0x458,
0x430, 0x43a, 0x438, 0x43b, 0x413, 0x443, 0x430, 0x43d, 0x430, 0x425, 0x430, 0x43b,
0x438, 0x444, 0x430, 0x43a, 0x441, 0x425, 0x430, 0x432, 0x430, 0x43d, 0x430, 0x425,
0x435, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x459, 0x43e, 0x41a, 0x43d, 0x43e, 0x43a,
0x441, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x41c,
0x430, 0x440, 0x435, 0x43d, 0x433, 0x43e, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438,
0x458, 0x430, 0x43d, 0x430, 0x41f, 0x435, 0x442, 0x435, 0x440, 0x441, 0x431, 0x443,
0x440, 0x433, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430,
0x422, 0x435, 0x43b, 0x20, 0x421, 0x438, 0x442, 0x438, 0x2c, 0x20, 0x418, 0x43d,
0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x412, 0x435, 0x432, 0x430, 0x458, 0x2c,
0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d,
0x446, 0x435, 0x43d, 0x435, 0x441, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458,
0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d, 0x430, 0x43c, 0x430, 0x43a, 0x2c, 0x20,
0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x418, 0x43a, 0x432, 0x430,
0x43b, 0x443, 0x438, 0x442, 0x408, 0x430, 0x43c, 0x430, 0x458, 0x43a, 0x430, 0x416,
0x443, 0x43d, 0x43e, 0x41c, 0x43e, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x43b, 0x43e,
0x2c, 0x20, 0x41a, 0x435, 0x43d, 0x442, 0x430, 0x43a, 0x438, 0x41a, 0x440, 0x435,
0x43b, 0x435, 0x43d, 0x434, 0x438, 0x458, 0x43a, 0x41b, 0x430, 0x20, 0x41f, 0x430,
0x437, 0x41b, 0x43e, 0x441, 0x20, 0x410, 0x43d, 0x452, 0x435, 0x43b, 0x435, 0x441,
0x41b, 0x43e, 0x443, 0x435, 0x440, 0x20, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x20,
0x41a, 0x432, 0x43e, 0x440, 0x442, 0x435, 0x440, 0x41c, 0x430, 0x441, 0x435, 0x458,
0x43e, 0x41c, 0x430, 0x43d, 0x430, 0x433, 0x432, 0x430, 0x41c, 0x430, 0x437, 0x430,
0x442, 0x43b, 0x430, 0x43d, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x43e, 0x20,
0x421, 0x438, 0x442, 0x438, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x440, 0x435, 0x458,
0x41c, 0x43e, 0x43d, 0x442, 0x441, 0x435, 0x440, 0x430, 0x442, 0x41d, 0x430, 0x441,
0x430, 0x443, 0x40a, 0x443, 0x458, 0x43e, 0x440, 0x43a, 0x41d, 0x43e, 0x440, 0x43e,
0x45a, 0x430, 0x411, 0x438, 0x458, 0x443, 0x43b, 0x430, 0x2c, 0x20, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430,
0x426, 0x435, 0x43d, 0x442, 0x430, 0x440, 0x2c, 0x20, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x43e,
0x432, 0x438, 0x20, 0x421, 0x430, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430,
0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x41f, 0x43e, 0x440, 0x442, 0x20, 0x43e,
0x444, 0x20, 0x421, 0x43f, 0x435, 0x458, 0x43d, 0x41f, 0x43e, 0x440, 0x442, 0x43e,
0x20, 0x412, 0x435, 0x459, 0x43e, 0x41f, 0x43e, 0x440, 0x442, 0x43e, 0x440, 0x438,
0x43a, 0x43e, 0x41f, 0x443, 0x43d, 0x442, 0x430, 0x20, 0x410, 0x440, 0x435, 0x43d,
0x430, 0x441, 0x420, 0x430, 0x43d, 0x43a, 0x438, 0x43d, 0x20, 0x418, 0x43d, 0x43b,
0x435, 0x442, 0x420, 0x435, 0x441, 0x438, 0x444, 0x435, 0x420, 0x435, 0x433, 0x438,
0x43d, 0x430, 0x420, 0x435, 0x441, 0x43e, 0x43b, 0x443, 0x442, 0x420, 0x438, 0x43e,
0x20, 0x411, 0x440, 0x430, 0x43d, 0x43a, 0x43e, 0x421, 0x430, 0x43d, 0x442, 0x430,
0x440, 0x435, 0x43c, 0x421, 0x430, 0x43d, 0x442, 0x438, 0x458, 0x430, 0x433, 0x43e,
0x421, 0x430, 0x43d, 0x442, 0x43e, 0x20, 0x414, 0x43e, 0x43c, 0x438, 0x43d, 0x433,
0x43e, 0x421, 0x430, 0x43e, 0x20, 0x41f, 0x430, 0x43e, 0x43b, 0x43e, 0x418, 0x442,
0x43e, 0x43a, 0x43e, 0x440, 0x442, 0x43e, 0x440, 0x43c, 0x438, 0x442, 0x421, 0x432,
0x2e, 0x20, 0x411, 0x430, 0x440, 0x442, 0x43e, 0x43b, 0x43e, 0x43c, 0x65, 0x458,
0x421, 0x432, 0x2e, 0x20, 0x40f, 0x43e, 0x43d, 0x421, 0x435, 0x43d, 0x442, 0x20,
0x41a, 0x438, 0x442, 0x441, 0x421, 0x432, 0x2e, 0x20, 0x41b, 0x443, 0x446, 0x438,
0x458, 0x430, 0x421, 0x432, 0x2e, 0x20, 0x422, 0x43e, 0x43c, 0x430, 0x421, 0x435,
0x43d, 0x442, 0x20, 0x412, 0x438, 0x43d, 0x441, 0x435, 0x43d, 0x442, 0x421, 0x432,
0x438, 0x444, 0x442, 0x20, 0x41a, 0x443, 0x440, 0x435, 0x43d, 0x442, 0x412, 0x430,
0x458, 0x442, 0x445, 0x43e, 0x440, 0x441, 0x412, 0x438, 0x43d, 0x438, 0x43f, 0x435,
0x433, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x430, 0x442, 0x41a, 0x430, 0x441, 0x435,
0x458, 0x414, 0x435, 0x458, 0x432, 0x438, 0x441, 0x414, 0x438, 0x43c, 0x43e, 0x43d,
0x20, 0x434, 0x2019, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x41c, 0x435, 0x43a, 0x432,
0x43e, 0x440, 0x438, 0x41c, 0x43e, 0x441, 0x43e, 0x43d, 0x41c, 0x430, 0x43a, 0x43c,
0x443, 0x440, 0x434, 0x43e, 0x428, 0x43e, 0x432, 0x430, 0x412, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x41b, 0x43e, 0x43d, 0x433, 0x458, 0x435, 0x440, 0x431, 0x458, 0x435,
0x43d, 0x410, 0x43b, 0x43c, 0x430, 0x442, 0x438, 0x410, 0x43d, 0x430, 0x434, 0x438,
0x440, 0x410, 0x43a, 0x443, 0x442, 0x43e, 0x431, 0x435, 0x410, 0x442, 0x438, 0x440,
0x430, 0x458, 0x411, 0x430, 0x43d, 0x43a, 0x43e, 0x43a, 0x411, 0x435, 0x458, 0x440,
0x443, 0x442, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x458, 0x438, 0x427, 0x43e, 0x458,
0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x425, 0x435, 0x431, 0x440, 0x43e, 0x43d,
0x425, 0x43e, 0x20, 0x428, 0x438, 0x20, 0x41c, 0x438, 0x43d, 0x425, 0x43e, 0x43d,
0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a,
0x40f, 0x430, 0x458, 0x430, 0x43f, 0x443, 0x440, 0x430, 0x408, 0x435, 0x440, 0x443,
0x441, 0x430, 0x43b, 0x438, 0x43c, 0x425, 0x430, 0x43d, 0x434, 0x438, 0x433, 0x430,
0x41a, 0x430, 0x43b, 0x43a, 0x443, 0x442, 0x430, 0x41a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x41a, 0x443, 0x430, 0x43b, 0x430, 0x20,
0x41b, 0x443, 0x43c, 0x43f, 0x443, 0x440, 0x41a, 0x443, 0x432, 0x430, 0x458, 0x442,
0x41c, 0x430, 0x43a, 0x430, 0x443, 0x41c, 0x443, 0x441, 0x43a, 0x430, 0x442, 0x41d,
0x438, 0x43a, 0x43e, 0x437, 0x438, 0x458, 0x430, 0x41d, 0x43e, 0x432, 0x43e, 0x43a,
0x443, 0x437, 0x45a, 0x435, 0x446, 0x43a, 0x41e, 0x440, 0x430, 0x43b, 0x41f, 0x43d,
0x43e, 0x43c, 0x20, 0x41f, 0x435, 0x43d, 0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458,
0x430, 0x43d, 0x433, 0x41a, 0x43e, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x458, 0x41a,
0x438, 0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x420, 0x438, 0x458, 0x430,
0x434, 0x428, 0x430, 0x43d, 0x433, 0x430, 0x458, 0x421, 0x440, 0x435, 0x434, 0x45a,
0x435, 0x43a, 0x43e, 0x43b, 0x438, 0x43c, 0x441, 0x43a, 0x422, 0x430, 0x458, 0x43f,
0x435, 0x458, 0x422, 0x431, 0x438, 0x43b, 0x438, 0x441, 0x438, 0x422, 0x435, 0x445,
0x435, 0x440, 0x430, 0x43d, 0x422, 0x438, 0x43c, 0x43f, 0x443, 0x423, 0x43b, 0x430,
0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x423, 0x440, 0x443, 0x43c, 0x43a,
0x432, 0x438, 0x412, 0x438, 0x458, 0x435, 0x43d, 0x442, 0x438, 0x458, 0x430, 0x43d,
0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x408, 0x435, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x408, 0x435, 0x440, 0x435,
0x432, 0x430, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x438, 0x411, 0x435, 0x440, 0x43c,
0x443, 0x434, 0x430, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x441, 0x43a, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e,
0x440, 0x442, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x424, 0x430, 0x440, 0x441, 0x43a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432,
0x430, 0x420, 0x435, 0x458, 0x43a, 0x458, 0x430, 0x432, 0x438, 0x43a, 0x408, 0x443,
0x436, 0x43d, 0x430, 0x20, 0x40f, 0x43e, 0x440, 0x45f, 0x438, 0x458, 0x430, 0x421,
0x432, 0x2e, 0x20, 0x408, 0x435, 0x43b, 0x435, 0x43d, 0x430, 0x411, 0x440, 0x438,
0x437, 0x431, 0x435, 0x458, 0x43d, 0x411, 0x440, 0x43e, 0x43a, 0x435, 0x43d, 0x20,
0x425, 0x438, 0x43b, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x421,
0x438, 0x434, 0x43d, 0x435, 0x458, 0x410, 0x442, 0x438, 0x43d, 0x430, 0x411, 0x435,
0x43e, 0x433, 0x440, 0x430, 0x434, 0x411, 0x440, 0x438, 0x441, 0x435, 0x43b, 0x411,
0x443, 0x43a, 0x443, 0x440, 0x435, 0x448, 0x442, 0x411, 0x443, 0x434, 0x438, 0x43c,
0x43f, 0x435, 0x448, 0x442, 0x430, 0x411, 0x443, 0x441, 0x438, 0x43d, 0x433, 0x435,
0x43d, 0x41a, 0x438, 0x448, 0x438, 0x45a, 0x435, 0x432, 0x41a, 0x43e, 0x43f, 0x435,
0x43d, 0x445, 0x430, 0x433, 0x435, 0x43d, 0x414, 0x430, 0x431, 0x43b, 0x438, 0x43d,
0x413, 0x435, 0x440, 0x43d, 0x437, 0x438, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e,
0x20, 0x41c, 0x435, 0x43d, 0x418, 0x441, 0x442, 0x430, 0x43d, 0x431, 0x443, 0x43b,
0x41a, 0x430, 0x43b, 0x438, 0x45a, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x434, 0x41a,
0x438, 0x458, 0x435, 0x432, 0x41b, 0x438, 0x441, 0x430, 0x431, 0x43e, 0x43d, 0x409,
0x443, 0x431, 0x459, 0x430, 0x43d, 0x430, 0x41b, 0x443, 0x43a, 0x441, 0x435, 0x43c,
0x431, 0x443, 0x440, 0x433, 0x41c, 0x430, 0x440, 0x438, 0x445, 0x430, 0x43c, 0x43d,
0x41f, 0x430, 0x440, 0x438, 0x437, 0x421, 0x430, 0x43d, 0x20, 0x41c, 0x430, 0x440,
0x438, 0x43d, 0x43e, 0x421, 0x430, 0x440, 0x430, 0x458, 0x435, 0x432, 0x43e, 0x421,
0x438, 0x43c, 0x444, 0x435, 0x440, 0x43e, 0x43f, 0x43e, 0x459, 0x421, 0x43a, 0x43e,
0x43f, 0x459, 0x435, 0x421, 0x43e, 0x444, 0x438, 0x458, 0x430, 0x421, 0x442, 0x43e,
0x43a, 0x445, 0x43e, 0x43b, 0x43c, 0x422, 0x430, 0x43b, 0x438, 0x43d, 0x423, 0x459,
0x430, 0x43d, 0x43e, 0x432, 0x441, 0x43a, 0x412, 0x430, 0x434, 0x443, 0x437, 0x411,
0x435, 0x447, 0x412, 0x438, 0x459, 0x43d, 0x443, 0x441, 0x426, 0x438, 0x440, 0x438,
0x445, 0x410, 0x43d, 0x442, 0x430, 0x43d, 0x430, 0x43d, 0x430, 0x440, 0x438, 0x432,
0x43e, 0x411, 0x43e, 0x436, 0x438, 0x45b, 0x41a, 0x43e, 0x43c, 0x43e, 0x440, 0x43e,
0x41c, 0x430, 0x445, 0x435, 0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432, 0x438, 0x41c,
0x430, 0x443, 0x440, 0x438, 0x446, 0x438, 0x458, 0x443, 0x441, 0x41c, 0x430, 0x458,
0x43e, 0x442, 0x420, 0x435, 0x443, 0x43d, 0x438, 0x43e, 0x43d, 0x410, 0x43f, 0x438,
0x458, 0x430, 0x41e, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x41a, 0x430, 0x442, 0x445,
0x430, 0x43c, 0x423, 0x441, 0x43a, 0x440, 0x448, 0x45a, 0x435, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x432, 0x43e, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x458, 0x435, 0x413,
0x432, 0x430, 0x434, 0x430, 0x43b, 0x43a, 0x430, 0x43d, 0x430, 0x43b, 0x425, 0x43e,
0x43d, 0x43e, 0x43b, 0x443, 0x43b, 0x443, 0x415, 0x43d, 0x434, 0x435, 0x440, 0x431,
0x435, 0x440, 0x438, 0x41a, 0x43e, 0x448, 0x440, 0x435, 0x41a, 0x432, 0x430, 0x45f,
0x430, 0x43b, 0x435, 0x458, 0x438, 0x43d, 0x41c, 0x430, 0x458, 0x443, 0x440, 0x43e,
0x41c, 0x438, 0x434, 0x432, 0x435, 0x458, 0x41f, 0x430, 0x433, 0x43e, 0x20, 0x41f,
0x430, 0x433, 0x43e, 0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x41f, 0x43e, 0x440,
0x442, 0x20, 0x41c, 0x43e, 0x440, 0x437, 0x431, 0x438, 0x421, 0x430, 0x458, 0x43f,
0x430, 0x43d, 0x422, 0x430, 0x445, 0x438, 0x442, 0x438, 0x412, 0x435, 0x458, 0x43a,
0x412, 0x430, 0x43b, 0x438, 0x441, 0x41, 0x64, 0x69, 0x73, 0x20, 0x41, 0x62,
0x65, 0x62, 0x61, 0x4b, 0x61, 0x65, 0x72, 0x6f, 0x44, 0x61, 0x72, 0x20,
0x65, 0x6c, 0x20, 0x42, 0x65, 0x69, 0x64, 0x61, 0x20, 0x28, 0x43, 0x61,
0x73, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x63, 0x61, 0x29, 0x4c, 0x61, 0x2bc,
0x59, 0x6f, 0x75, 0x6e, 0x4e, 0x2bc, 0x44, 0x6a, 0x61, 0x6d, 0x65, 0x6e,
0x61, 0x54, 0x61, 0x72, 0x61, 0x62, 0x75, 0x6c, 0x75, 0x73, 0x20, 0x28,
0x54, 0x72, 0x69, 0x70, 0x6f, 0x6c, 0x69, 0x29, 0x42, 0x65, 0x6c, 0xe9,
0x6d, 0x47, 0x77, 0x61, 0x64, 0x65, 0x6c, 0x6f, 0x75, 0x70, 0x4b, 0xea,
0x72, 0x20, 0x56, 0x65, 0x63, 0x2bc, 0x68, 0x69, 0x6b, 0x6f, 0x4e, 0x75,
0x75, 0x6b, 0x20, 0x28, 0x47, 0x6f, 0x64, 0x74, 0x68, 0xe5, 0x62, 0x29,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9,
0x6c, 0x65, 0x6d, 0x79, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x4a, 0x6f,
0x68, 0x6e, 0x2bc, 0x73, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x4c, 0x75,
0x63, 0x69, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x56, 0x69, 0x73, 0x61,
0x6e, 0x74, 0x51, 0xe2, 0x6e, 0xe2, 0x71, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x2bc, 0x42, 0x61, 0x6b, 0x6f, 0x75, 0x42, 0x61, 0x79, 0x72, 0x75,
0x74, 0x54, 0x63, 0x68, 0x69, 0x74, 0x61, 0x4b, 0x6f, 0x6c, 0x61, 0x6d,
0x62, 0x61, 0x4b, 0xea, 0x72, 0x20, 0x48, 0xf4, 0x2d, 0x43, 0x68, 0x69,
0x2d, 0x4d, 0x69, 0x6e, 0x68, 0x4b, 0x61, 0x62, 0x6f, 0x75, 0x6c, 0x4b,
0x61, 0x6d, 0x74, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x43, 0x61, 0x6c,
0x63, 0x75, 0x74, 0x74, 0x61, 0x4b, 0x6f, 0x77, 0x65, 0x69, 0x74, 0x4d,
0x61, 0x73, 0x71, 0x61, 0x74, 0x4c, 0x65, 0x76, 0x6b, 0x6f, 0x73, 0xed,
0x61, 0x50, 0x68, 0x6e, 0x75, 0x6d, 0x20, 0x50, 0xe9, 0x6e, 0x68, 0x50,
0x2bc, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x6f, 0x75, 0x72, 0x54, 0x6f, 0x73, 0x68, 0x6b, 0x65,
0x6e, 0x74, 0x56, 0x69, 0x61, 0x6e, 0x67, 0x63, 0x68, 0x61, 0x6e, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72,
0x67, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x7a, 0x42, 0x65, 0x72, 0x6d, 0x75,
0x64, 0x65, 0x7a, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x65, 0x7a, 0x4b,
0x61, 0x62, 0x20, 0x47, 0x6c, 0x61, 0x73, 0x46, 0x61, 0x65, 0x72, 0x6f,
0x52, 0x65, 0x79, 0x6b, 0x6a, 0x61, 0x76, 0xed, 0x6b, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x53, 0x75, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x44, 0x75,
0x6c, 0x65, 0x6e, 0x6e, 0x4a, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61,
0x72, 0x47, 0x77, 0x65, 0x72, 0x6e, 0x65, 0x6e, 0x65, 0x7a, 0x4d, 0x61,
0x6e, 0x61, 0x76, 0x4a, 0x65, 0x72, 0x7a, 0x65, 0x6e, 0x65, 0x7a, 0x4c,
0x6f, 0x6e, 0x64, 0x72, 0x65, 0x7a, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d,
0x62, 0x6f, 0x75, 0x72, 0x67, 0x4d, 0x61, 0x72, 0x6a, 0x65, 0x68, 0x61,
0x6d, 0x6e, 0x4d, 0x65, 0x6e, 0x73, 0x6b, 0x4d, 0x6f, 0x73, 0x6b, 0x6f,
0x76, 0x50, 0x72, 0x61, 0x68, 0x61, 0x4b, 0x6f, 0x6b, 0x6f, 0x7a, 0x4b,
0x6f, 0x6d, 0x6f, 0x72, 0x65, 0x7a, 0x4b, 0x65, 0x72, 0x67, 0x65, 0x6c,
0x65, 0x6e, 0x6e, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x7a, 0x4d,
0x6f, 0x72, 0x69, 0x73, 0x52, 0x65, 0xfc, 0x6e, 0x69, 0x6f, 0x6e, 0x45,
0x6e, 0x65, 0x7a, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x4d, 0x61, 0x72, 0x6b,
0x69, 0x7a, 0x410, 0x431, 0x438, 0x434, 0x436, 0x430, 0x43d, 0x411, 0x43b, 0x430,
0x43d, 0x442, 0x430, 0x439, 0x44a, 0x440, 0x41a, 0x430, 0x439, 0x440, 0x43e, 0x414,
0x430, 0x440, 0x20, 0x435, 0x441, 0x20, 0x421, 0x430, 0x43b, 0x430, 0x43c, 0x414,
0x436, 0x438, 0x431, 0x443, 0x442, 0x438, 0x415, 0x43b, 0x20, 0x410, 0x44e, 0x43d,
0x424, 0x440, 0x438, 0x439, 0x442, 0x430, 0x443, 0x43d, 0x419, 0x43e, 0x445, 0x430,
0x43d, 0x435, 0x441, 0x431, 0x443, 0x440, 0x433, 0x41c, 0x43e, 0x433, 0x430, 0x434,
0x438, 0x448, 0x443, 0x41c, 0x43e, 0x43d, 0x440, 0x43e, 0x432, 0x438, 0x44f, 0x41d,
0x430, 0x439, 0x440, 0x43e, 0x431, 0x438, 0x41d, 0x438, 0x430, 0x43c, 0x435, 0x439,
0x410, 0x43d, 0x43a, 0x44a, 0x440, 0x438, 0x434, 0x436, 0x410, 0x43d, 0x433, 0x443,
0x438, 0x43b, 0x430, 0x410, 0x440, 0x430, 0x433, 0x443, 0x430, 0x439, 0x43d, 0x430,
0x411, 0x443, 0x435, 0x43d, 0x43e, 0x441, 0x20, 0x410, 0x439, 0x440, 0x435, 0x441,
0x425, 0x443, 0x445, 0x443, 0x439, 0x420, 0x438, 0x43e, 0x20, 0x413, 0x430, 0x43b,
0x435, 0x433, 0x43e, 0x441, 0x421, 0x430, 0x43d, 0x20, 0x41b, 0x443, 0x438, 0x441,
0x411, 0x430, 0x438, 0x44f, 0x411, 0x430, 0x438, 0x44f, 0x20, 0x434, 0x435, 0x20,
0x411, 0x430, 0x43d, 0x434, 0x435, 0x440, 0x430, 0x441, 0x411, 0x43b, 0x430, 0x43d,
0x2d, 0x421, 0x430, 0x431, 0x43b, 0x43e, 0x43d, 0x411, 0x43e, 0x439, 0x441, 0x438,
0x41a, 0x435, 0x439, 0x43c, 0x431, 0x440, 0x438, 0x434, 0x436, 0x20, 0x411, 0x435,
0x439, 0x41a, 0x430, 0x439, 0x435, 0x43d, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d,
0x43e, 0x432, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x421,
0x438, 0x443, 0x434, 0x430, 0x434, 0x20, 0x425, 0x443, 0x430, 0x440, 0x435, 0x441,
0x41a, 0x43e, 0x441, 0x442, 0x430, 0x20, 0x420, 0x438, 0x43a, 0x430, 0x41a, 0x440,
0x435, 0x441, 0x442, 0x44a, 0x43d, 0x427, 0x443, 0x44f, 0x431, 0x430, 0x41a, 0x44e,
0x440, 0x430, 0x441, 0x430, 0x43e, 0x414, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x43a,
0x441, 0x445, 0x430, 0x432, 0x43d, 0x414, 0x43e, 0x443, 0x441, 0x44a, 0x43d, 0x414,
0x43e, 0x443, 0x441, 0x44a, 0x43d, 0x20, 0x41a, 0x440, 0x438, 0x439, 0x43a, 0x414,
0x435, 0x43d, 0x432, 0x44a, 0x440, 0x414, 0x435, 0x442, 0x440, 0x43e, 0x439, 0x442,
0x415, 0x434, 0x43c, 0x44a, 0x43d, 0x442, 0x44a, 0x43d, 0x415, 0x439, 0x440, 0x443,
0x43d, 0x435, 0x43f, 0x435, 0x424, 0x43e, 0x440, 0x442, 0x20, 0x41d, 0x435, 0x43b,
0x441, 0x44a, 0x43d, 0x413, 0x43b, 0x435, 0x439, 0x441, 0x20, 0x411, 0x435, 0x439,
0x413, 0x443, 0x441, 0x20, 0x411, 0x435, 0x439, 0x413, 0x440, 0x430, 0x43d, 0x434,
0x20, 0x422, 0x44a, 0x440, 0x43a, 0x413, 0x443, 0x430, 0x44f, 0x43a, 0x438, 0x43b,
0x415, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x43b, 0x44c, 0x43e, 0x41f, 0x438, 0x442,
0x44a, 0x440, 0x441, 0x431, 0x44a, 0x440, 0x433, 0x412, 0x438, 0x432, 0x438, 0x412,
0x438, 0x43d, 0x441, 0x435, 0x43d, 0x441, 0x414, 0x436, 0x443, 0x43d, 0x43e, 0x41a,
0x440, 0x430, 0x43b, 0x435, 0x43d, 0x434, 0x435, 0x439, 0x43a, 0x41b, 0x430, 0x20,
0x41f, 0x430, 0x441, 0x41b, 0x43e, 0x441, 0x20, 0x410, 0x43d, 0x434, 0x436, 0x435,
0x43b, 0x438, 0x441, 0x41b, 0x43e, 0x443, 0x44a, 0x440, 0x20, 0x43f, 0x440, 0x438,
0x43d, 0x441, 0x435, 0x441, 0x20, 0x43a, 0x443, 0x43e, 0x442, 0x44a, 0x440, 0x41c,
0x430, 0x441, 0x435, 0x439, 0x43e, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x440, 0x435,
0x439, 0x41d, 0x44e, 0x20, 0x419, 0x43e, 0x440, 0x43a, 0x41d, 0x43e, 0x443, 0x43c,
0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44f, 0x411, 0x44e, 0x43b, 0x430, 0x421, 0x435,
0x43d, 0x442, 0x44a, 0x440, 0x41d, 0x44e, 0x20, 0x421, 0x435, 0x439, 0x43b, 0x44a,
0x43c, 0x41f, 0x43e, 0x440, 0x442, 0x20, 0x44a, 0x444, 0x20, 0x421, 0x43f, 0x435,
0x439, 0x43d, 0x41f, 0x43e, 0x440, 0x442, 0x43e, 0x20, 0x412, 0x435, 0x43b, 0x44c,
0x43e, 0x41f, 0x443, 0x435, 0x440, 0x442, 0x43e, 0x20, 0x420, 0x438, 0x43a, 0x43e,
0x420, 0x438, 0x434, 0x436, 0x430, 0x439, 0x43d, 0x430, 0x420, 0x435, 0x437, 0x43e,
0x43b, 0x44e, 0x442, 0x421, 0x430, 0x43d, 0x442, 0x44f, 0x433, 0x43e, 0x421, 0x430,
0x43e, 0x20, 0x41f, 0x430, 0x443, 0x43b, 0x43e, 0x421, 0x433, 0x43e, 0x440, 0x441,
0x431, 0x438, 0x441, 0x43e, 0x43d, 0x421, 0x435, 0x43d, 0x20, 0x411, 0x430, 0x440,
0x442, 0x435, 0x43b, 0x435, 0x43c, 0x438, 0x421, 0x435, 0x439, 0x43d, 0x442, 0x20,
0x414, 0x436, 0x43e, 0x43d, 0x441, 0x421, 0x435, 0x439, 0x43d, 0x442, 0x20, 0x41a,
0x438, 0x442, 0x441, 0x421, 0x435, 0x439, 0x43d, 0x442, 0x20, 0x41b, 0x443, 0x441,
0x438, 0x44f, 0x421, 0x435, 0x439, 0x43d, 0x442, 0x20, 0x422, 0x43e, 0x43c, 0x430,
0x441, 0x421, 0x435, 0x439, 0x43d, 0x442, 0x20, 0x412, 0x438, 0x43d, 0x441, 0x44a,
0x43d, 0x442, 0x421, 0x443, 0x438, 0x444, 0x442, 0x20, 0x41a, 0x44a, 0x440, 0x435,
0x43d, 0x442, 0x412, 0x430, 0x43d, 0x43a, 0x443, 0x432, 0x44a, 0x440, 0x423, 0x438,
0x43d, 0x438, 0x43f, 0x435, 0x433, 0x41a, 0x435, 0x439, 0x441, 0x438, 0x414, 0x435,
0x439, 0x432, 0x438, 0x441, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x414, 0x44e,
0x440, 0x432, 0x438, 0x43b, 0x41c, 0x43e, 0x443, 0x441, 0x44a, 0x43d, 0x41c, 0x430,
0x43a, 0x43c, 0x44a, 0x440, 0x434, 0x43e, 0x41f, 0x430, 0x43b, 0x43c, 0x44a, 0x440,
0x41b, 0x43e, 0x43d, 0x433, 0x438, 0x440, 0x431, 0x44e, 0x435, 0x43d, 0x410, 0x442,
0x44a, 0x440, 0x430, 0x443, 0x411, 0x430, 0x445, 0x440, 0x435, 0x439, 0x43d, 0x427,
0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x425, 0x43e, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x419, 0x435, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x438, 0x43c,
0x41a, 0x43e, 0x43b, 0x43a, 0x430, 0x442, 0x430, 0x41a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x41d, 0x438, 0x43a, 0x43e, 0x437, 0x438, 0x44f,
0x410, 0x440, 0x430, 0x43b, 0x41f, 0x445, 0x435, 0x43d, 0x44f, 0x43d, 0x41a, 0x43e,
0x441, 0x442, 0x430, 0x43d, 0x430, 0x439, 0x41a, 0x44a, 0x437, 0x44a, 0x43b, 0x43e,
0x440, 0x434, 0x430, 0x420, 0x438, 0x44f, 0x434, 0x421, 0x440, 0x435, 0x434, 0x43d,
0x435, 0x43a, 0x43e, 0x43b, 0x438, 0x43c, 0x441, 0x43a, 0x422, 0x430, 0x439, 0x43f,
0x435, 0x412, 0x438, 0x435, 0x43d, 0x442, 0x44f, 0x43d, 0x42f, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x441,
0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41a, 0x430,
0x43d, 0x430, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x438, 0x41a, 0x430, 0x431, 0x43e, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435,
0x424, 0x430, 0x440, 0x44c, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x41c, 0x430, 0x434, 0x435, 0x439, 0x440, 0x430,
0x420, 0x435, 0x439, 0x43a, 0x44f, 0x432, 0x438, 0x43a, 0x42e, 0x436, 0x43d, 0x430,
0x20, 0x414, 0x436, 0x43e, 0x440, 0x434, 0x436, 0x438, 0x44f, 0x421, 0x432, 0x435,
0x442, 0x430, 0x20, 0x415, 0x43b, 0x435, 0x43d, 0x430, 0x411, 0x440, 0x438, 0x437,
0x431, 0x435, 0x439, 0x43d, 0x411, 0x440, 0x43e, 0x443, 0x43a, 0x44a, 0x43d, 0x20,
0x425, 0x438, 0x43b, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x41c,
0x435, 0x43b, 0x431, 0x44a, 0x440, 0x43d, 0x41f, 0x44a, 0x440, 0x442, 0x421, 0x438,
0x434, 0x43d, 0x438, 0x411, 0x440, 0x44e, 0x43a, 0x441, 0x435, 0x43b, 0x411, 0x443,
0x43a, 0x443, 0x440, 0x435, 0x449, 0x411, 0x443, 0x434, 0x430, 0x43f, 0x435, 0x449,
0x430, 0x411, 0x44e, 0x437, 0x438, 0x43d, 0x433, 0x435, 0x43d, 0x414, 0x44a, 0x431,
0x43b, 0x438, 0x43d, 0x413, 0x44a, 0x440, 0x43d, 0x437, 0x438, 0x425, 0x435, 0x43b,
0x437, 0x438, 0x43d, 0x43a, 0x438, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x20,
0x41c, 0x430, 0x43d, 0x414, 0x436, 0x44a, 0x440, 0x441, 0x438, 0x421, 0x43a, 0x43e,
0x43f, 0x438, 0x435, 0x421, 0x43e, 0x444, 0x438, 0x44f, 0x423, 0x43b, 0x44f, 0x43d,
0x43e, 0x432, 0x441, 0x43a, 0x412, 0x438, 0x435, 0x43d, 0x430, 0x412, 0x438, 0x43b,
0x43d, 0x44e, 0x441, 0x426, 0x44e, 0x440, 0x438, 0x445, 0x420, 0x43e, 0x436, 0x434,
0x435, 0x441, 0x442, 0x432, 0x43e, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432,
0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41a, 0x43e, 0x43c,
0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x438, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x439, 0x41c, 0x430, 0x439,
0x43e, 0x442, 0x420, 0x435, 0x44e, 0x43d, 0x438, 0x43e, 0x43d, 0x410, 0x43f, 0x438,
0x44f, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x427, 0x443, 0x44e, 0x43a, 0x412, 0x435,
0x43b, 0x438, 0x43a, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x424, 0x438, 0x434, 0x436, 0x438, 0x413, 0x430, 0x43c,
0x431, 0x438, 0x435, 0x415, 0x43d, 0x434, 0x44a, 0x440, 0x431, 0x44a, 0x440, 0x438,
0x41a, 0x43e, 0x448, 0x440, 0x430, 0x439, 0x41a, 0x443, 0x430, 0x434, 0x436, 0x430,
0x43b, 0x438, 0x43d, 0x41c, 0x430, 0x434, 0x436, 0x443, 0x440, 0x43e, 0x41c, 0x430,
0x440, 0x43a, 0x438, 0x437, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x438, 0x41c, 0x438, 0x434, 0x443, 0x435, 0x439, 0x41f, 0x43e, 0x43d, 0x43f,
0x435, 0x439, 0x41f, 0x43e, 0x440, 0x442, 0x20, 0x41c, 0x43e, 0x440, 0x441, 0x431,
0x438, 0x422, 0x430, 0x440, 0x430, 0x443, 0x430, 0x423, 0x435, 0x439, 0x43a, 0x423,
0x43e, 0x43b, 0x438, 0x441, 0x1021, 0x102c, 0x1018, 0x102e, 0x1002, 0x103b, 0x1014, 0x103a,
0x1021, 0x1000, 0x103a, 0x1000, 0x101b, 0x102c, 0x1021, 0x102c, 0x1038, 0x1012, 0x1005, 0x103a,
0x20, 0x1021, 0x1018, 0x102c, 0x1018, 0x102c, 0x1021, 0x101a, 0x103a, 0x101c, 0x103a, 0x1002,
0x103b, 0x102e, 0x1038, 0x101b, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x1021, 0x102c, 0x1038,
0x1005, 0x103a, 0x1019, 0x102c, 0x1038, 0x101b, 0x102c, 0x1018, 0x102c, 0x1019, 0x102c, 0x1000,
0x102d, 0x102f, 0x1018, 0x102c, 0x1014, 0x103a, 0x1002, 0x102e, 0x1038, 0x1018, 0x1014, 0x103a,
0x1002, 0x103b, 0x102f, 0x1038, 0x101c, 0x103a, 0x1018, 0x102e, 0x1005, 0x102c, 0x1021, 0x102d,
0x102f, 0x1018, 0x101c, 0x1014, 0x103a, 0x1010, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x101b,
0x1032, 0x101c, 0x103a, 0x1018, 0x101b, 0x102c, 0x1038, 0x1007, 0x102c, 0x1017, 0x102e, 0x1038,
0x101c, 0x103a, 0x1018, 0x1030, 0x1002, 0x103b, 0x103d, 0x1014, 0x103a, 0x1018, 0x1030, 0x1038,
0x101b, 0x102c, 0x1000, 0x102d, 0x102f, 0x1004, 0x103a, 0x101b, 0x102d, 0x102f, 0x1000, 0x102c,
0x1006, 0x102c, 0x1018, 0x101c, 0x1014, 0x103a, 0x1000, 0x102c, 0x1006, 0x102e, 0x101b, 0x1030,
0x1038, 0x1010, 0x102c, 0x1000, 0x102d, 0x102f, 0x1014, 0x102c, 0x1000, 0x101b, 0x102e, 0x1038,
0x1012, 0x1000, 0x103b, 0x1000, 0x102c, 0x1038, 0x1012, 0x102b, 0x101b, 0x1000, 0x103a, 0x1005,
0x103a, 0x1006, 0x102c, 0x101c, 0x1019, 0x103a, 0x1002, 0x103b, 0x102e, 0x1018, 0x1030, 0x1010,
0x102e, 0x1012, 0x102d, 0x102f, 0x1021, 0x1030, 0x1021, 0x102c, 0x101c, 0x102c, 0x1021, 0x101a,
0x103a, 0x101c, 0x103a, 0x1021, 0x102c, 0x101a, 0x103d, 0x1014, 0x103a, 0x1038, 0x1016, 0x101b,
0x102e, 0x1038, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1038, 0x1014, 0x103a, 0x1002, 0x102b,
0x1018, 0x102c, 0x101b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1014, 0x102e, 0x101f, 0x102c, 0x101b,
0x102c, 0x1038, 0x101b, 0x1032, 0x101a, 0x103a, 0x1002, 0x103b, 0x102d, 0x102f, 0x101f, 0x1014,
0x103a, 0x1038, 0x1014, 0x1000, 0x103a, 0x1005, 0x1018, 0x1010, 0x103a, 0x1002, 0x103b, 0x102f,
0x1018, 0x102c, 0x1000, 0x1019, 0x103a, 0x1015, 0x102b, 0x101c, 0x102c, 0x1001, 0x102b, 0x1010,
0x102d, 0x102f, 0x1021, 0x1014, 0x103a, 0x1000, 0x102e, 0x1002, 0x102b, 0x1038, 0x101c, 0x102e,
0x1000, 0x1004, 0x103a, 0x101b, 0x103e, 0x102c, 0x1005, 0x102c, 0x101c, 0x102c, 0x1002, 0x102d,
0x102f, 0x1005, 0x103a, 0x101c, 0x102e, 0x1017, 0x101b, 0x102c, 0x1017, 0x102e, 0x1038, 0x101c,
0x103a, 0x101c, 0x102d, 0x102f, 0x1019, 0x102e, 0x101c, 0x1030, 0x101d, 0x1019, 0x103a, 0x1012,
0x102b, 0x101c, 0x1030, 0x1018, 0x103d, 0x1014, 0x103a, 0x1038, 0x1018, 0x102c, 0x101b, 0x103e,
0x102e, 0x101c, 0x1030, 0x1005, 0x102c, 0x1000, 0x102c, 0x1019, 0x102c, 0x101c, 0x102c, 0x1018,
0x102d, 0x102f, 0x1019, 0x102c, 0x1015, 0x1030, 0x1038, 0x1010, 0x102d, 0x102f, 0x1019, 0x102c,
0x1006, 0x1030, 0x101b, 0x1030, 0x1038, 0x1018, 0x102c, 0x1038, 0x1018, 0x102c, 0x1014, 0x103a,
0x1038, 0x1019, 0x1031, 0x102c, 0x103a, 0x1002, 0x102b, 0x1012, 0x102e, 0x1038, 0x101e, 0x103b,
0x102f, 0x1038, 0x1019, 0x103d, 0x1014, 0x103a, 0x101b, 0x102d, 0x102f, 0x1038, 0x1017, 0x102e,
0x1038, 0x101a, 0x102c, 0x1038, 0x1014, 0x102d, 0x102f, 0x1004, 0x103a, 0x101b, 0x102d, 0x102f,
0x1018, 0x102e, 0x1021, 0x1004, 0x103a, 0x1002, 0x103b, 0x102c, 0x1019, 0x102e, 0x1014, 0x102c,
0x100a, 0x102c, 0x1019, 0x1032, 0x101a, 0x103a, 0x1014, 0x102d, 0x102f, 0x1021, 0x102c, 0x1037,
0x1001, 0x103a, 0x101b, 0x103e, 0x1031, 0x102c, 0x1037, 0x1010, 0x103a, 0x101d, 0x102b, 0x1002,
0x102b, 0x1012, 0x1030, 0x1038, 0x1002, 0x1030, 0x1015, 0x102d, 0x102f, 0x1010, 0x102d, 0x102f,
0x2d, 0x1014, 0x102d, 0x102f, 0x1017, 0x102d, 0x102f, 0x1006, 0x1031, 0x102c, 0x1004, 0x103a,
0x1010, 0x1030, 0x1019, 0x1031, 0x1038, 0x1011, 0x101b, 0x102e, 0x1015, 0x102d, 0x102f, 0x101c,
0x102e, 0x1010, 0x1030, 0x1014, 0x102e, 0x1005, 0x103a, 0x1017, 0x1004, 0x103a, 0x1038, 0x101f,
0x1030, 0x1038, 0x1001, 0x103a, 0x1021, 0x102c, 0x1012, 0x1001, 0x103b, 0x1021, 0x1014, 0x103a,
0x1000, 0x102c, 0x101b, 0x1031, 0x1037, 0x1002, 0x103b, 0x103a, 0x1021, 0x1014, 0x103a, 0x1002,
0x102e, 0x101c, 0x102c, 0x1021, 0x1014, 0x103a, 0x1010, 0x102e, 0x1002, 0x103d, 0x102b, 0x1021,
0x102c, 0x101b, 0x102c, 0x1002, 0x103d, 0x102b, 0x1021, 0x102e, 0x1014, 0x102c, 0x1017, 0x103b,
0x1030, 0x1014, 0x102d, 0x102f, 0x1021, 0x1031, 0x1038, 0x101b, 0x102d, 0x1005, 0x103a, 0x1000,
0x102c, 0x1010, 0x102c, 0x1019, 0x102c, 0x101b, 0x1000, 0x102c, 0x1000, 0x102d, 0x102f, 0x1012,
0x102d, 0x102f, 0x1018, 0x102c, 0x1002, 0x103b, 0x102f, 0x1002, 0x103b, 0x1031, 0x101c, 0x102c,
0x20, 0x101b, 0x102e, 0x101a, 0x102d, 0x102f, 0x101f, 0x102c, 0x1019, 0x1014, 0x103a, 0x1012,
0x102d, 0x102f, 0x1007, 0x102c, 0x101b, 0x102e, 0x101a, 0x102d, 0x102f, 0x20, 0x1002, 0x102b,
0x101c, 0x102e, 0x1002, 0x102d, 0x102f, 0x1005, 0x103a, 0x1006, 0x102c, 0x101c, 0x103a, 0x1010,
0x102c, 0x1006, 0x1014, 0x103a, 0x20, 0x1002, 0x103d, 0x1019, 0x103a, 0x1006, 0x1014, 0x103a,
0x20, 0x101c, 0x1030, 0x101d, 0x102e, 0x1005, 0x103a, 0x1010, 0x1030, 0x1000, 0x1030, 0x1019,
0x1014, 0x103a, 0x1025, 0x1006, 0x103d, 0x102c, 0x1021, 0x102e, 0x1021, 0x102c, 0x1021, 0x102c,
0x101b, 0x1030, 0x1038, 0x1017, 0x102c, 0x1021, 0x102c, 0x1006, 0x1030, 0x1005, 0x102e, 0x1021,
0x103d, 0x1014, 0x103a, 0x1038, 0x1021, 0x102c, 0x1010, 0x102e, 0x1000, 0x102d, 0x102f, 0x1000,
0x1014, 0x103a, 0x1018, 0x102c, 0x101f, 0x102e, 0x1021, 0x102c, 0x1018, 0x102c, 0x101f, 0x102e,
0x1021, 0x102c, 0x20, 0x1018, 0x1014, 0x103a, 0x1012, 0x101b, 0x1000, 0x103a, 0x1005, 0x103a,
0x1018, 0x102c, 0x1018, 0x1031, 0x1038, 0x1012, 0x102d, 0x102f, 0x1038, 0x1005, 0x103a, 0x1018,
0x102e, 0x101c, 0x1004, 0x103a, 0x1019, 0x103a, 0x1018, 0x101c, 0x102d, 0x1007, 0x103a, 0x1018,
0x101c, 0x103d, 0x1014, 0x103a, 0x1001, 0x103a, 0x2d, 0x1005, 0x102c, 0x1018, 0x101c, 0x103d,
0x1014, 0x103a, 0x1018, 0x102d, 0x102f, 0x1021, 0x102c, 0x1017, 0x102e, 0x1005, 0x103a, 0x1010,
0x102c, 0x1018, 0x102d, 0x102f, 0x1002, 0x102d, 0x102f, 0x1010, 0x102c, 0x1017, 0x103d, 0x102d,
0x102f, 0x1000, 0x103a, 0x1005, 0x102e, 0x1000, 0x102d, 0x1014, 0x103a, 0x1038, 0x1018, 0x101b,
0x1005, 0x103a, 0x1001, 0x103b, 0x103a, 0x20, 0x1018, 0x1031, 0x1038, 0x1000, 0x102d, 0x1019,
0x103a, 0x1015, 0x102d, 0x102f, 0x20, 0x1002, 0x101b, 0x1014, 0x103a, 0x1012, 0x102e, 0x1000,
0x1014, 0x103a, 0x1001, 0x1014, 0x103a, 0x1000, 0x102c, 0x101b, 0x102c, 0x1000, 0x102c, 0x1005,
0x103a, 0x1000, 0x1031, 0x100a, 0x1004, 0x103a, 0x1014, 0x103a, 0x1000, 0x1031, 0x1019, 0x1014,
0x103a, 0x1001, 0x103b, 0x102e, 0x1000, 0x102c, 0x1000, 0x102d, 0x102f, 0x1001, 0x103b, 0x102e,
0x101f, 0x1030, 0x1021, 0x102c, 0x101f, 0x1030, 0x1021, 0x102c, 0x1005, 0x102e, 0x101a, 0x1030,
0x1012, 0x1010, 0x103a, 0x1005, 0x103a, 0x20, 0x101f, 0x103d, 0x102c, 0x101b, 0x1000, 0x103a,
0x1005, 0x103a, 0x1000, 0x102d, 0x102f, 0x1037, 0x1005, 0x1010, 0x102c, 0x101b, 0x102e, 0x1000,
0x102c, 0x1000, 0x101b, 0x1005, 0x103a, 0x1005, 0x1010, 0x103d, 0x1014, 0x103a, 0x1000, 0x103d,
0x102e, 0x101b, 0x102c, 0x1018, 0x102c, 0x1000, 0x103b, 0x1030, 0x101b, 0x1031, 0x1038, 0x1000,
0x102d, 0x102f, 0x1038, 0x1012, 0x1014, 0x103a, 0x1019, 0x1010, 0x103a, 0x101b, 0x103e, 0x103a,
0x1017, 0x102c, 0x1014, 0x103a, 0x1012, 0x1031, 0x102b, 0x103a, 0x1006, 0x1014, 0x103a, 0x1012,
0x1031, 0x102b, 0x103a, 0x1006, 0x1014, 0x103a, 0x20, 0x1001, 0x101b, 0x102d, 0x1001, 0x103a,
0x1012, 0x1004, 0x103a, 0x1014, 0x103a, 0x1017, 0x102c, 0x1012, 0x1000, 0x103a, 0x1011, 0x101b,
0x103d, 0x102d, 0x102f, 0x1000, 0x103a, 0x1012, 0x102d, 0x102f, 0x1019, 0x102e, 0x1014, 0x102e,
0x1000, 0x102c, 0x1021, 0x1000, 0x103a, 0x1012, 0x103a, 0x1019, 0x103d, 0x1014, 0x103a, 0x1010,
0x1014, 0x103a, 0x1021, 0x102e, 0x101b, 0x1030, 0x1014, 0x102e, 0x1015, 0x102e, 0x1021, 0x101a,
0x103a, 0x101c, 0x103a, 0x1006, 0x102c, 0x1017, 0x1031, 0x1012, 0x102d, 0x102f, 0x1016, 0x102d,
0x102f, 0x1037, 0x1010, 0x103a, 0x20, 0x1014, 0x101a, 0x103a, 0x101c, 0x103a, 0x1006, 0x1004,
0x103a, 0x1016, 0x102d, 0x102f, 0x1037, 0x1010, 0x103a, 0x1010, 0x102c, 0x101c, 0x102e, 0x1007,
0x102c, 0x1002, 0x101c, 0x1032, 0x1005, 0x103a, 0x1018, 0x1031, 0x1038, 0x1002, 0x1030, 0x1038,
0x1005, 0x103a, 0x1018, 0x1031, 0x1038, 0x1002, 0x101b, 0x1014, 0x103a, 0x1012, 0x103a, 0x20,
0x1010, 0x1001, 0x103a, 0x1002, 0x101b, 0x102e, 0x1014, 0x1031, 0x1012, 0x102b, 0x1002, 0x103d,
0x102b, 0x1012, 0x102e, 0x101c, 0x102f, 0x1015, 0x103a, 0x1002, 0x103d, 0x102b, 0x1010, 0x102e,
0x1019, 0x102c, 0x101c, 0x102c, 0x1002, 0x103d, 0x102b, 0x101b, 0x102c, 0x1000, 0x103d, 0x102e,
0x1038, 0x101c, 0x103a, 0x1002, 0x102d, 0x102f, 0x1004, 0x103a, 0x101a, 0x102c, 0x1014, 0x102c,
0x101f, 0x101c, 0x102e, 0x1016, 0x1000, 0x103a, 0x1005, 0x103a, 0x101f, 0x102c, 0x1017, 0x102c,
0x1014, 0x102c, 0x101f, 0x102c, 0x1019, 0x102d, 0x102f, 0x1005, 0x103a, 0x1005, 0x102e, 0x101c,
0x102d, 0x102f, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c, 0x1038, 0x1014, 0x102c,
0x1038, 0x20, 0x1015, 0x102d, 0x102f, 0x101c, 0x102d, 0x1005, 0x103a, 0x1014, 0x1031, 0x102c,
0x1000, 0x103a, 0x1001, 0x103a, 0x1005, 0x103a, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012,
0x102e, 0x101a, 0x102c, 0x1038, 0x1014, 0x102c, 0x1038, 0x1019, 0x102c, 0x101b, 0x1014, 0x103a,
0x1002, 0x102d, 0x102f, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c,
0x1038, 0x1014, 0x102c, 0x1038, 0x1015, 0x102e, 0x1010, 0x102c, 0x1005, 0x103a, 0x1018, 0x1010,
0x103a, 0x1001, 0x103a, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c,
0x1038, 0x1014, 0x102c, 0x1038, 0x1010, 0x1032, 0x101c, 0x103a, 0x1005, 0x102e, 0x1038, 0x1010,
0x102e, 0x1038, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c, 0x1038,
0x1014, 0x102c, 0x1038, 0x1017, 0x102e, 0x1017, 0x1032, 0x104a, 0x20, 0x1021, 0x1004, 0x103a,
0x1012, 0x102e, 0x101a, 0x102c, 0x1038, 0x1014, 0x102c, 0x1038, 0x1017, 0x1004, 0x103a, 0x1006,
0x1004, 0x1037, 0x103a, 0x1005, 0x103a, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e,
0x101a, 0x102c, 0x1038, 0x1014, 0x102c, 0x1038, 0x101d, 0x102e, 0x1014, 0x102c, 0x1019, 0x1000,
0x103a, 0x1001, 0x103a, 0x104a, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102e, 0x101a, 0x102c,
0x1038, 0x1014, 0x102c, 0x1038, 0x1021, 0x102e, 0x1014, 0x102f, 0x1017, 0x102e, 0x1001, 0x103a,
0x1021, 0x102e, 0x1000, 0x102c, 0x101c, 0x1030, 0x1021, 0x102e, 0x1010, 0x103a, 0x1002, 0x103b,
0x1019, 0x1031, 0x1000, 0x102c, 0x1002, 0x103b, 0x102f, 0x1014, 0x102d, 0x102f, 0x1021, 0x1031,
0x102c, 0x101c, 0x1030, 0x101d, 0x102e, 0x1017, 0x102e, 0x1038, 0x101c, 0x103a, 0x1019, 0x103d,
0x1014, 0x103a, 0x1010, 0x102e, 0x1001, 0x103b, 0x101a, 0x103a, 0x101c, 0x102d, 0x102f, 0x104a,
0x20, 0x1000, 0x1004, 0x103a, 0x1010, 0x1015, 0x103a, 0x1000, 0x102e, 0x1001, 0x101b, 0x102c,
0x101c, 0x1032, 0x1014, 0x103a, 0x1038, 0x1012, 0x1005, 0x103a, 0x1001, 0x103b, 0x103a, 0x101c,
0x102c, 0x1015, 0x102b, 0x1007, 0x103a, 0x101c, 0x102e, 0x1019, 0x102c, 0x101c, 0x1031, 0x102c,
0x1037, 0x1005, 0x103a, 0x1021, 0x102d, 0x1014, 0x103a, 0x1002, 0x103b, 0x101a, 0x103a, 0x101c,
0x102d, 0x1005, 0x103a, 0x101c, 0x102d, 0x102f, 0x1021, 0x102c, 0x1015, 0x101b, 0x1004, 0x1037,
0x103a, 0x1005, 0x103a, 0x20, 0x1000, 0x103d, 0x102c, 0x1010, 0x102c, 0x1019, 0x102c, 0x1006,
0x1032, 0x101e, 0x103d, 0x102c, 0x1019, 0x102c, 0x1014, 0x102c, 0x1002, 0x103d, 0x102b, 0x1019,
0x102c, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1005, 0x103a, 0x1019, 0x102c, 0x101b, 0x102e,
0x1002, 0x1031, 0x102b, 0x1037, 0x1019, 0x102c, 0x1010, 0x102e, 0x1014, 0x102e, 0x1002, 0x103a,
0x1019, 0x102c, 0x1010, 0x102c, 0x1019, 0x102d, 0x102f, 0x101b, 0x102d, 0x102f, 0x1005, 0x103a,
0x1019, 0x102c, 0x1007, 0x1010, 0x103a, 0x101c, 0x1014, 0x103a, 0x1019, 0x102e, 0x1014, 0x102d,
0x102f, 0x1019, 0x102e, 0x1014, 0x102e, 0x1038, 0x1019, 0x102e, 0x101b, 0x102e, 0x1012, 0x102b,
0x1019, 0x1000, 0x103a, 0x1010, 0x103a, 0x101c, 0x102c, 0x1000, 0x1000, 0x103a, 0x101c, 0x102c,
0x1019, 0x1000, 0x103a, 0x1000, 0x1006, 0x102e, 0x1000, 0x102d, 0x102f, 0x20, 0x1005, 0x102e,
0x1038, 0x1010, 0x102e, 0x1038, 0x1019, 0x102e, 0x1000, 0x103d, 0x102e, 0x101c, 0x103d, 0x1014,
0x103a, 0x1019, 0x103d, 0x1014, 0x103a, 0x1001, 0x103a, 0x1010, 0x103d, 0x1014, 0x103a, 0x1019,
0x103d, 0x1014, 0x103a, 0x1010, 0x1032, 0x101b, 0x1031, 0x1038, 0x1019, 0x103d, 0x1014, 0x103a,
0x1010, 0x102e, 0x1017, 0x102e, 0x1012, 0x102e, 0x1021, 0x102d, 0x102f, 0x1019, 0x103d, 0x1014,
0x1037, 0x103a, 0x28, 0x1010, 0x103a, 0x29, 0x1006, 0x1031, 0x1038, 0x101b, 0x1010, 0x103a,
0x1014, 0x102c, 0x1037, 0x1006, 0x1031, 0x102c, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x101a,
0x1031, 0x102c, 0x1000, 0x103a, 0x1014, 0x102d, 0x102f, 0x101b, 0x102d, 0x102f, 0x100a, 0x102c,
0x1017, 0x103c, 0x1030, 0x101c, 0x102c, 0x104a, 0x20, 0x1019, 0x103c, 0x1031, 0x102c, 0x1000,
0x103a, 0x1012, 0x102b, 0x1000, 0x102d, 0x102f, 0x1010, 0x102c, 0x1005, 0x1004, 0x103a, 0x1010,
0x102c, 0x104a, 0x20, 0x1019, 0x103c, 0x1031, 0x102c, 0x1000, 0x103a, 0x1012, 0x102b, 0x1000,
0x102d, 0x102f, 0x1010, 0x102c, 0x1014, 0x101a, 0x1030, 0x1038, 0x1006, 0x1031, 0x1038, 0x101c,
0x1019, 0x103a, 0x104a, 0x20, 0x1019, 0x103c, 0x1031, 0x102c, 0x1000, 0x103a, 0x1012, 0x102b,
0x1000, 0x102d, 0x102f, 0x1010, 0x102c, 0x1014, 0x102f, 0x1001, 0x103a, 0x1021, 0x102d, 0x102f,
0x1001, 0x102e, 0x1014, 0x1002, 0x102b, 0x1015, 0x1014, 0x102c, 0x1038, 0x1019, 0x102c, 0x1038,
0x1015, 0x102b, 0x101b, 0x102c, 0x1019, 0x102c, 0x101b, 0x102e, 0x1018, 0x102d, 0x102f, 0x1016,
0x102e, 0x1038, 0x1014, 0x1005, 0x103a, 0x1015, 0x102d, 0x102f, 0x1037, 0x1010, 0x103a, 0x2d,
0x1021, 0x102d, 0x102f, 0x2d, 0x1015, 0x101b, 0x1004, 0x1037, 0x103a, 0x1005, 0x103a, 0x1015,
0x102d, 0x102f, 0x1037, 0x1010, 0x103a, 0x20, 0x1021, 0x1031, 0x102c, 0x1037, 0x1016, 0x103a,
0x20, 0x1005, 0x1015, 0x102d, 0x1014, 0x103a, 0x1015, 0x102d, 0x102f, 0x1037, 0x1010, 0x103a,
0x1010, 0x102d, 0x102f, 0x20, 0x1017, 0x1032, 0x101c, 0x102e, 0x101a, 0x102d, 0x102f, 0x1015,
0x1031, 0x102b, 0x103a, 0x1010, 0x102d, 0x102f, 0x101b, 0x102e, 0x1000, 0x102d, 0x102f, 0x1015,
0x103d, 0x1014, 0x103a, 0x1010, 0x102c, 0x20, 0x1021, 0x101b, 0x102e, 0x1014, 0x102c, 0x1005,
0x103a, 0x101b, 0x1014, 0x103a, 0x1000, 0x1004, 0x103a, 0x20, 0x1021, 0x1004, 0x103a, 0x1038,
0x101c, 0x1000, 0x103a, 0x101f, 0x1031, 0x1005, 0x102e, 0x1016, 0x102e, 0x101c, 0x103a, 0x101b,
0x101a, 0x103a, 0x1002, 0x103b, 0x102e, 0x1014, 0x102c, 0x101b, 0x102e, 0x1006, 0x102d, 0x102f,
0x101c, 0x102f, 0x1015, 0x103a, 0x28, 0x1010, 0x103a, 0x29, 0x101b, 0x102e, 0x101a, 0x102d,
0x102f, 0x20, 0x1018, 0x101b, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x1006, 0x1014, 0x103a,
0x1010, 0x102c, 0x101b, 0x1019, 0x103a, 0x1006, 0x1014, 0x103a, 0x1010, 0x102e, 0x1021, 0x102c,
0x1002, 0x102d, 0x102f, 0x1006, 0x1014, 0x103a, 0x1010, 0x102d, 0x102f, 0x20, 0x1012, 0x102d,
0x102f, 0x1019, 0x1004, 0x103a, 0x1038, 0x1002, 0x102d, 0x102f, 0x1006, 0x1031, 0x102c, 0x103a,
0x1015, 0x102d, 0x102f, 0x101c, 0x102d, 0x102f, 0x1021, 0x1005, 0x103a, 0x1010, 0x102c, 0x1000,
0x102c, 0x20, 0x1010, 0x102d, 0x102f, 0x1038, 0x1019, 0x102d, 0x101b, 0x1000, 0x103a, 0x1010,
0x103a, 0x1005, 0x1005, 0x103a, 0x1000, 0x102c, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1018,
0x102c, 0x101e, 0x101a, 0x103a, 0x101c, 0x1019, 0x102e, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a,
0x1002, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1000,
0x102d, 0x1005, 0x103a, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x101c, 0x1030, 0x1005, 0x102e,
0x101a, 0x102c, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x101e, 0x1031, 0x102c, 0x1019, 0x1010,
0x103a, 0x1005, 0x103a, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1017, 0x1004, 0x103a, 0x1038,
0x1006, 0x1004, 0x1037, 0x103a, 0x1005, 0x103d, 0x1010, 0x103a, 0x1016, 0x103a, 0x1010, 0x103a,
0x20, 0x1000, 0x102c, 0x1038, 0x101b, 0x1004, 0x1037, 0x103a, 0x1010, 0x102e, 0x1002, 0x1030,
0x1005, 0x102e, 0x1002, 0x101c, 0x103a, 0x1015, 0x102b, 0x101e, 0x102f, 0x101c, 0x102e, 0x1010,
0x102e, 0x1002, 0x103d, 0x102b, 0x1014, 0x102c, 0x1010, 0x102d, 0x102f, 0x101b, 0x1014, 0x103a,
0x1010, 0x102d, 0x102f, 0x1010, 0x1031, 0x102c, 0x1010, 0x102d, 0x102f, 0x101c, 0x102c, 0x1017,
0x1014, 0x103a, 0x1000, 0x1030, 0x1038, 0x1017, 0x102c, 0x1038, 0x101d, 0x102d, 0x102f, 0x1000,
0x103a, 0x28, 0x1010, 0x103a, 0x29, 0x101f, 0x102d, 0x102f, 0x1005, 0x103a, 0x101d, 0x102e,
0x1014, 0x102e, 0x1017, 0x1000, 0x103a, 0x1002, 0x103a, 0x101b, 0x102c, 0x1000, 0x102f, 0x1010,
0x1010, 0x103a, 0x1000, 0x1031, 0x1005, 0x102e, 0x1012, 0x1031, 0x1038, 0x1017, 0x1005, 0x103a,
0x1012, 0x1030, 0x1038, 0x1019, 0x1031, 0x102c, 0x1037, 0x1010, 0x103a, 0x20, 0x1012, 0x102b,
0x101b, 0x103a, 0x1017, 0x102e, 0x1038, 0x101c, 0x103a, 0x1019, 0x1000, 0x103a, 0x1000, 0x103d,
0x101a, 0x103a, 0x101b, 0x102e, 0x1019, 0x1031, 0x102c, 0x103a, 0x1005, 0x103d, 0x1014, 0x103a,
0x1019, 0x1000, 0x103a, 0x1019, 0x1030, 0x1012, 0x102d, 0x102f, 0x1038, 0x1015, 0x102b, 0x101c,
0x103a, 0x1019, 0x102c, 0x101b, 0x102d, 0x102f, 0x101e, 0x102e, 0x101b, 0x102c, 0x101b, 0x103e,
0x102d, 0x102f, 0x1038, 0x101d, 0x102b, 0x1011, 0x101b, 0x102d, 0x102f, 0x101c, 0x103a, 0x1017,
0x102d, 0x102f, 0x1005, 0x103a, 0x1010, 0x102d, 0x102f, 0x1001, 0x103a, 0x101c, 0x1031, 0x102c,
0x1004, 0x103a, 0x101b, 0x102e, 0x101a, 0x102c, 0x1038, 0x1018, 0x101b, 0x1036, 0x1021, 0x102c,
0x1012, 0x1004, 0x103a, 0x1021, 0x1031, 0x102c, 0x103a, 0x1019, 0x102c, 0x1010, 0x102e, 0x1021,
0x102c, 0x1019, 0x102c, 0x1014, 0x103a, 0x1038, 0x1021, 0x1014, 0x103a, 0x1021, 0x102c, 0x1012,
0x102e, 0x1021, 0x102c, 0x1021, 0x1000, 0x103a, 0x1010, 0x102c, 0x1025, 0x102f, 0x1021, 0x102c,
0x1001, 0x103b, 0x1010, 0x1030, 0x1038, 0x1018, 0x102e, 0x1021, 0x102c, 0x101b, 0x103e, 0x103a,
0x1002, 0x102b, 0x1018, 0x102c, 0x1012, 0x103a, 0x1021, 0x102c, 0x1010, 0x102e, 0x101b, 0x1021,
0x1030, 0x1018, 0x1002, 0x1039, 0x1002, 0x1012, 0x1000, 0x103a, 0x1018, 0x102c, 0x101b, 0x102d,
0x1014, 0x103a, 0x1038, 0x1018, 0x102c, 0x1000, 0x1030, 0x1018, 0x1014, 0x103a, 0x1000, 0x1031,
0x102c, 0x1000, 0x103a, 0x1018, 0x101b, 0x103a, 0x1014, 0x102c, 0x1021, 0x1030, 0x1018, 0x1031,
0x101b, 0x103d, 0x1010, 0x103a, 0x1018, 0x102e, 0x101b, 0x103e, 0x103a, 0x1001, 0x1000, 0x103a,
0x1018, 0x101b, 0x1030, 0x1014, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1001, 0x103b, 0x102e,
0x1010, 0x102c, 0x1001, 0x103b, 0x103d, 0x1032, 0x1018, 0x1031, 0x102c, 0x1006, 0x1014, 0x103a,
0x1000, 0x102d, 0x102f, 0x101c, 0x1036, 0x1018, 0x102d, 0x102f, 0x1012, 0x1019, 0x102c, 0x1038,
0x1005, 0x1000, 0x1015, 0x103a, 0x1012, 0x1000, 0x103a, 0x1000, 0x102c, 0x1012, 0x1005, 0x103a,
0x101c, 0x102e, 0x1012, 0x1030, 0x1018, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1012, 0x1030,
0x101b, 0x103e, 0x1014, 0x103a, 0x1038, 0x1018, 0x1032, 0x1016, 0x102c, 0x1019, 0x102c, 0x1002,
0x1030, 0x1005, 0x1010, 0x102c, 0x1002, 0x102b, 0x1007, 0x102c, 0x101f, 0x102e, 0x1018, 0x101b,
0x103d, 0x1014, 0x103a, 0x101f, 0x102d, 0x102f, 0x1001, 0x103b, 0x102e, 0x1019, 0x1004, 0x103a,
0x1038, 0x1005, 0x102e, 0x1038, 0x1010, 0x102e, 0x1038, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a,
0x1000, 0x1031, 0x102c, 0x1004, 0x103a, 0x101f, 0x102d, 0x102f, 0x1038, 0x1017, 0x103b, 0x1021,
0x102e, 0x101b, 0x1030, 0x1001, 0x1030, 0x1010, 0x103a, 0x1002, 0x103b, 0x1000, 0x102c, 0x1010,
0x102c, 0x1002, 0x103b, 0x102c, 0x101b, 0x102c, 0x1015, 0x1030, 0x101b, 0x102c, 0x1002, 0x103b,
0x1031, 0x101b, 0x102f, 0x1006, 0x101c, 0x1004, 0x103a, 0x1000, 0x1018, 0x1030, 0x1038, 0x101c,
0x103b, 0x1001, 0x1019, 0x103a, 0x1001, 0x103b, 0x102c, 0x1037, 0x1001, 0x102b, 0x1000, 0x101b,
0x102c, 0x1001, 0x103b, 0x102d, 0x1001, 0x1010, 0x103a, 0x1010, 0x1019, 0x1014, 0x103a, 0x1012,
0x1030, 0x1001, 0x1014, 0x103a, 0x1012, 0x102d, 0x102f, 0x1004, 0x103a, 0x1002, 0x102b, 0x1000,
0x102d, 0x102f, 0x101c, 0x103b, 0x1000, 0x1010, 0x103a, 0x1010, 0x102c, 0x1038, 0x1001, 0x101b,
0x102c, 0x1037, 0x1005, 0x103a, 0x1014, 0x102d, 0x102f, 0x101b, 0x102c, 0x1005, 0x103a, 0x1000,
0x103d, 0x102c, 0x101c, 0x102c, 0x101c, 0x1019, 0x103a, 0x1015, 0x1030, 0x1000, 0x1030, 0x1001,
0x103b, 0x1004, 0x103a, 0x1000, 0x1030, 0x101d, 0x102d, 0x1010, 0x103a, 0x1019, 0x1000, 0x102c,
0x1021, 0x102d, 0x102f, 0x1019, 0x102c, 0x1002, 0x102b, 0x1012, 0x1014, 0x103a, 0x1038, 0x1019,
0x1001, 0x1000, 0x103a, 0x1005, 0x103a, 0x1006, 0x102c, 0x1019, 0x1014, 0x102e, 0x101c, 0x102c,
0x1019, 0x1010, 0x103a, 0x1005, 0x103a, 0x1000, 0x1000, 0x103a, 0x1010, 0x103a, 0x1014, 0x102e,
0x1000, 0x102d, 0x102f, 0x101b, 0x103e, 0x102c, 0x1038, 0x1014, 0x102d, 0x102f, 0x1017, 0x102d,
0x102f, 0x1001, 0x1030, 0x1007, 0x103a, 0x1014, 0x1000, 0x103a, 0x1005, 0x103a, 0x1014, 0x102d,
0x102f, 0x1017, 0x102d, 0x102f, 0x1005, 0x1032, 0x1018, 0x102e, 0x1021, 0x1032, 0x101a, 0x103a,
0x1005, 0x103a, 0x1021, 0x103d, 0x1019, 0x103a, 0x1038, 0x1005, 0x103a, 0x1001, 0x103a, 0x1021,
0x1031, 0x102c, 0x103a, 0x101b, 0x1032, 0x101c, 0x103a, 0x1016, 0x1014, 0x103d, 0x1019, 0x103a,
0x1015, 0x1004, 0x103a, 0x1015, 0x103d, 0x1014, 0x103a, 0x1010, 0x102e, 0x1021, 0x102c, 0x1038,
0x1014, 0x102c, 0x1037, 0x1001, 0x103a, 0x1015, 0x103c, 0x102f, 0x1036, 0x101a, 0x1014, 0x103a,
0x1038, 0x1000, 0x1031, 0x102c, 0x1037, 0x1005, 0x103a, 0x1010, 0x1014, 0x1031, 0x1000, 0x102e,
0x1007, 0x101c, 0x1031, 0x102c, 0x103a, 0x1012, 0x102b, 0x101b, 0x102e, 0x101a, 0x102c, 0x1038,
0x1012, 0x103a, 0x1006, 0x102c, 0x1001, 0x102b, 0x101c, 0x1004, 0x103a, 0x1006, 0x1019, 0x103a,
0x1038, 0x1019, 0x102c, 0x1001, 0x1014, 0x103a, 0x1038, 0x1012, 0x103a, 0x1006, 0x102d, 0x102f,
0x1038, 0x101c, 0x103a, 0x101b, 0x103e, 0x1014, 0x103a, 0x101f, 0x102d, 0x102f, 0x1004, 0x103a,
0x1038, 0x1005, 0x1004, 0x103a, 0x1039, 0x1000, 0x102c, 0x1015, 0x1030, 0x1006, 0x101b, 0x1005,
0x103a, 0x1014, 0x1005, 0x103a, 0x1000, 0x102c, 0x101c, 0x1004, 0x103a, 0x1019, 0x103a, 0x1005,
0x103a, 0x1010, 0x102d, 0x102f, 0x1004, 0x103a, 0x1015, 0x1031, 0x1010, 0x102c, 0x101b, 0x103e,
0x103a, 0x1000, 0x1032, 0x1014, 0x1037, 0x103a, 0x1010, 0x1018, 0x102e, 0x1038, 0x101c, 0x102e,
0x1005, 0x102e, 0x1010, 0x102e, 0x101f, 0x102e, 0x101b, 0x1014, 0x103a, 0x1010, 0x1004, 0x103a,
0x1016, 0x1030, 0x1038, 0x1010, 0x102d, 0x102f, 0x1000, 0x103b, 0x102d, 0x102f, 0x1010, 0x103d,
0x1014, 0x103a, 0x1019, 0x103a, 0x1005, 0x103a, 0x1001, 0x103a, 0x1025, 0x101c, 0x1014, 0x103a,
0x1018, 0x102c, 0x1010, 0x1031, 0x102c, 0x1021, 0x1030, 0x101b, 0x102f, 0x1019, 0x103a, 0x1001,
0x103b, 0x102e, 0x1021, 0x1030, 0x1038, 0x1005, 0x103a, 0x20, 0x1014, 0x102e, 0x101b, 0x102c,
0x1038, 0x1017, 0x102e, 0x101a, 0x1004, 0x103a, 0x1000, 0x103b, 0x1014, 0x103a, 0x1038, 0x1017,
0x101c, 0x102c, 0x1012, 0x102e, 0x1017, 0x1031, 0x102c, 0x1037, 0x1005, 0x1010, 0x1031, 0x102c,
0x1037, 0x1001, 0x103a, 0x101a, 0x1030, 0x1001, 0x1030, 0x1038, 0x1010, 0x103a, 0x1005, 0x103a,
0x101b, 0x1014, 0x103a, 0x1000, 0x102f, 0x1014, 0x103a, 0x101b, 0x101a, 0x103a, 0x1001, 0x102b,
0x1038, 0x1010, 0x102e, 0x101b, 0x1004, 0x103a, 0x1018, 0x102c, 0x1038, 0x1001, 0x103a, 0x101b,
0x1032, 0x101a, 0x103a, 0x101b, 0x1031, 0x1017, 0x1014, 0x103a, 0x1038, 0x1021, 0x1031, 0x1007,
0x102d, 0x102f, 0x1038, 0x101b, 0x102e, 0x1038, 0x1005, 0x103a, 0x1018, 0x102c, 0x1019, 0x103c,
0x1030, 0x1012, 0x102b, 0x1000, 0x1014, 0x1031, 0x101b, 0x102e, 0x1000, 0x102d, 0x1010, 0x103a,
0x1015, 0x103a, 0x20, 0x1017, 0x102c, 0x1012, 0x102e, 0x1016, 0x102b, 0x101b, 0x102d, 0x102f,
0x1019, 0x1012, 0x102e, 0x1038, 0x101b, 0x102c, 0x101b, 0x1031, 0x1038, 0x1000, 0x102e, 0x1017,
0x1005, 0x103a, 0x1001, 0x103a, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1002, 0x103b, 0x1031,
0x102c, 0x103a, 0x1002, 0x103b, 0x102e, 0x101a, 0x102c, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a,
0x101f, 0x101a, 0x103a, 0x101c, 0x101a, 0x103a, 0x1014, 0x102c, 0x1005, 0x1010, 0x1014, 0x103a,
0x101c, 0x1031, 0x1021, 0x1014, 0x1039, 0x1012, 0x102e, 0x101c, 0x102d, 0x1010, 0x103a, 0x1012,
0x103a, 0x1018, 0x101b, 0x1005, 0x1039, 0x1005, 0x1018, 0x102d, 0x1014, 0x103a, 0x1038, 0x1018,
0x101b, 0x102d, 0x102f, 0x1000, 0x1004, 0x103a, 0x101f, 0x102e, 0x1038, 0x101c, 0x103a, 0x1012,
0x102b, 0x101d, 0x1004, 0x103a, 0x101a, 0x1030, 0x1038, 0x1001, 0x101c, 0x102c, 0x101f, 0x102d,
0x102f, 0x1038, 0x1018, 0x102c, 0x1038, 0x1010, 0x103a, 0x101c, 0x1004, 0x103a, 0x1038, 0x1012,
0x102e, 0x1019, 0x1014, 0x103a, 0x1038, 0x101c, 0x1031, 0x102c, 0x1037, 0x1012, 0x103a, 0x20,
0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x1019, 0x1032, 0x101c, 0x103a, 0x1018, 0x102f, 0x1014,
0x103a, 0x1038, 0x1015, 0x102b, 0x1038, 0x101e, 0x103a, 0x1006, 0x1005, 0x103a, 0x1012, 0x1014,
0x102e, 0x1021, 0x1019, 0x103a, 0x1005, 0x1010, 0x102c, 0x1012, 0x1019, 0x103a, 0x1021, 0x1014,
0x103a, 0x1012, 0x102d, 0x102f, 0x101b, 0x102c, 0x1021, 0x102c, 0x1038, 0x1005, 0x103a, 0x1010,
0x101b, 0x1001, 0x1014, 0x103a, 0x1038, 0x1021, 0x1031, 0x101e, 0x1004, 0x103a, 0x1018, 0x1032,
0x101c, 0x103a, 0x1002, 0x101b, 0x102d, 0x1010, 0x103a, 0x1018, 0x102c, 0x101c, 0x1004, 0x103a,
0x1018, 0x101b, 0x102c, 0x1010, 0x102e, 0x1038, 0x1005, 0x103a, 0x101c, 0x102c, 0x1038, 0x1017,
0x102c, 0x1018, 0x101b, 0x1015, 0x103a, 0x1006, 0x1032, 0x101c, 0x103a, 0x1018, 0x1030, 0x1001,
0x102b, 0x101b, 0x1000, 0x103a, 0x1005, 0x103a, 0x1018, 0x1030, 0x1012, 0x102b, 0x1015, 0x1000,
0x103a, 0x1005, 0x103a, 0x1018, 0x1030, 0x101b, 0x103e, 0x1004, 0x103a, 0x1002, 0x103b, 0x1004,
0x103a, 0x1001, 0x103b, 0x102e, 0x101b, 0x103e, 0x102e, 0x1014, 0x102c, 0x1038, 0x1021, 0x1030,
0x1000, 0x102d, 0x102f, 0x1015, 0x1004, 0x103a, 0x101f, 0x1031, 0x1002, 0x1004, 0x103a, 0x1012,
0x1010, 0x103a, 0x1018, 0x101c, 0x1004, 0x103a, 0x1002, 0x103b, 0x102e, 0x1018, 0x101b, 0x1031,
0x102c, 0x103a, 0x101c, 0x103a, 0x1010, 0x102c, 0x1002, 0x103d, 0x1014, 0x103a, 0x1038, 0x1007,
0x102e, 0x101f, 0x1032, 0x101c, 0x103a, 0x1005, 0x1004, 0x103a, 0x1000, 0x102e, 0x1019, 0x1014,
0x103a, 0x1038, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1021, 0x1005, 0x1039, 0x1005,
0x1010, 0x1014, 0x103a, 0x1018, 0x1030, 0x101c, 0x103a, 0x1002, 0x103b, 0x102c, 0x1005, 0x102e,
0x1001, 0x102b, 0x101c, 0x102e, 0x1014, 0x1004, 0x103a, 0x1002, 0x101b, 0x1000, 0x103a, 0x1001,
0x102e, 0x101b, 0x102d, 0x102f, 0x1017, 0x103a, 0x1001, 0x102e, 0x1038, 0x1021, 0x1000, 0x103a,
0x1016, 0x103a, 0x101c, 0x1005, 0x1039, 0x1005, 0x1018, 0x103d, 0x1014, 0x103a, 0x1038, 0x101c,
0x1030, 0x1018, 0x101c, 0x102e, 0x1021, 0x102c, 0x1038, 0x1014, 0x102c, 0x1038, 0x101c, 0x1014,
0x103a, 0x1012, 0x1014, 0x103a, 0x101c, 0x1030, 0x1007, 0x1004, 0x103a, 0x1018, 0x1010, 0x103a,
0x1019, 0x1012, 0x101b, 0x1005, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a, 0x101c, 0x103a, 0x1010,
0x102c, 0x1019, 0x101b, 0x102e, 0x1021, 0x102c, 0x20, 0x101f, 0x102c, 0x1038, 0x1019, 0x1014,
0x103a, 0x1019, 0x1004, 0x103a, 0x1038, 0x1005, 0x1001, 0x103a, 0x1019, 0x102d, 0x102f, 0x1014,
0x102c, 0x1000, 0x102d, 0x102f, 0x1019, 0x1031, 0x102c, 0x103a, 0x1005, 0x1000, 0x102d, 0x102f,
0x1021, 0x1031, 0x102c, 0x103a, 0x1005, 0x101c, 0x102d, 0x102f, 0x1015, 0x1032, 0x101b, 0x1005,
0x103a, 0x1015, 0x1031, 0x102b, 0x1037, 0x1002, 0x102d, 0x102f, 0x101b, 0x102e, 0x1000, 0x102c,
0x1015, 0x101b, 0x1000, 0x103a, 0x1002, 0x103a, 0x101b, 0x102e, 0x1002, 0x102b, 0x101b, 0x1031,
0x102c, 0x1019, 0x1006, 0x102c, 0x1019, 0x102c, 0x1038, 0x101b, 0x102c, 0x1006, 0x1014, 0x103a,
0x1019, 0x101b, 0x102e, 0x1038, 0x1014, 0x102d, 0x102f, 0x1006, 0x102c, 0x101b, 0x102c, 0x101a,
0x1031, 0x1017, 0x102d, 0x102f, 0x1006, 0x102c, 0x101b, 0x102c, 0x1010, 0x102d, 0x102f, 0x1037,
0x1016, 0x103a, 0x1005, 0x1004, 0x103a, 0x1016, 0x102c, 0x101b, 0x102d, 0x102f, 0x1038, 0x1016,
0x102d, 0x102f, 0x1005, 0x1000, 0x1031, 0x102c, 0x1037, 0x1015, 0x103a, 0x101b, 0x102c, 0x1006,
0x102d, 0x102f, 0x1016, 0x102e, 0x1021, 0x102c, 0x1005, 0x1010, 0x1031, 0x102c, 0x1037, 0x101f,
0x102f, 0x1019, 0x103a, 0x1038, 0x1011, 0x102c, 0x1038, 0x101c, 0x1004, 0x103a, 0x1010, 0x102e,
0x101b, 0x102c, 0x1014, 0x102e, 0x1021, 0x1030, 0x101c, 0x102e, 0x101a, 0x102c, 0x1014, 0x102d,
0x102f, 0x1005, 0x1000, 0x1005, 0x103a, 0x1016, 0x103a, 0x1017, 0x102c, 0x1012, 0x1030, 0x1038,
0x1005, 0x103a, 0x1017, 0x102c, 0x1010, 0x102e, 0x1000, 0x1014, 0x103a, 0x1017, 0x102e, 0x101a,
0x1004, 0x103a, 0x1014, 0x102c, 0x1017, 0x102e, 0x1038, 0x101c, 0x103a, 0x1014, 0x102e, 0x1021,
0x102d, 0x102f, 0x1038, 0x1005, 0x103a, 0x1017, 0x102d, 0x102f, 0x101c, 0x103a, 0x1002, 0x102d,
0x102f, 0x1002, 0x101b, 0x1000, 0x103a, 0x101d, 0x102b, 0x1006, 0x1031, 0x102c, 0x1007, 0x102c,
0x1002, 0x103a, 0x1002, 0x101b, 0x1000, 0x103a, 0x1018, 0x103a, 0x1007, 0x1030, 0x1038, 0x101b,
0x1005, 0x103a, 0x1001, 0x103b, 0x103a, 0x1021, 0x1014, 0x103a, 0x1010, 0x102c, 0x1014, 0x102c,
0x1014, 0x102c, 0x101b, 0x102e, 0x1038, 0x1018, 0x102d, 0x102f, 0x1001, 0x103b, 0x102c, 0x1002,
0x102d, 0x102f, 0x1005, 0x103a, 0x1001, 0x101b, 0x1005, 0x103a, 0x1005, 0x1019, 0x1010, 0x103a,
0x1000, 0x102d, 0x102f, 0x1000, 0x102d, 0x102f, 0x1038, 0x1005, 0x103a, 0x1000, 0x102d, 0x102f,
0x1019, 0x102d, 0x102f, 0x101b, 0x102d, 0x102f, 0x1001, 0x102b, 0x1002, 0x102b, 0x101c, 0x1014,
0x103a, 0x1019, 0x102c, 0x101f, 0x102e, 0x1019, 0x1031, 0x102c, 0x103a, 0x101c, 0x1012, 0x102d,
0x102f, 0x1000, 0x103a, 0x1019, 0x1031, 0x102c, 0x101b, 0x1005, 0x103a, 0x101b, 0x103e, 0x1019,
0x102c, 0x101a, 0x102d, 0x102f, 0x1010, 0x1032, 0x101b, 0x102e, 0x101a, 0x1030, 0x1014, 0x102e,
0x101a, 0x1014, 0x103a, 0x1021, 0x102c, 0x1038, 0x1015, 0x102e, 0x1021, 0x102c, 0x1038, 0x1021,
0x1031, 0x102c, 0x1037, 0x1000, 0x101c, 0x1014, 0x103a, 0x1018, 0x1030, 0x1002, 0x1014, 0x103a,
0x1017, 0x102e, 0x1038, 0x101c, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x1001, 0x103b, 0x102c,
0x1038, 0x101e, 0x1019, 0x103a, 0x1001, 0x103b, 0x102f, 0x1001, 0x103a, 0x1021, 0x102e, 0x1005,
0x1010, 0x102c, 0x1021, 0x102e, 0x1016, 0x102c, 0x1010, 0x102e, 0x1016, 0x102c, 0x1001, 0x102b,
0x1021, 0x102d, 0x102f, 0x1016, 0x102d, 0x102f, 0x1016, 0x102e, 0x1002, 0x103b, 0x102e, 0x1016,
0x1030, 0x1014, 0x102c, 0x1016, 0x1030, 0x1010, 0x102e, 0x1002, 0x102b, 0x101c, 0x102c, 0x1015,
0x102b, 0x1000, 0x1015, 0x103a, 0x1005, 0x103a, 0x1002, 0x1019, 0x103a, 0x1018, 0x102e, 0x101a,
0x102c, 0x1002, 0x103d, 0x102b, 0x1012, 0x102b, 0x1000, 0x1014, 0x1032, 0x101c, 0x103a, 0x1002,
0x1030, 0x1021, 0x1019, 0x103a, 0x101f, 0x102d, 0x102f, 0x1014, 0x102d, 0x102f, 0x101c, 0x1030,
0x101c, 0x1030, 0x1021, 0x1014, 0x103a, 0x1012, 0x102b, 0x1018, 0x1030, 0x101b, 0x102e, 0x1001,
0x101b, 0x102d, 0x1012, 0x102e, 0x1019, 0x1010, 0x102e, 0x1001, 0x102d, 0x102f, 0x1005, 0x103a,
0x101b, 0x102d, 0x102f, 0x1004, 0x103a, 0x1001, 0x103d, 0x102c, 0x1002, 0x103b, 0x102c, 0x101c,
0x102d, 0x1014, 0x103a, 0x1019, 0x102c, 0x1002, 0x103b, 0x1030, 0x1038, 0x101b, 0x102d, 0x102f,
0x1019, 0x102c, 0x1001, 0x1031, 0x1038, 0x1021, 0x1015, 0x103a, 0x1005, 0x103a, 0x1019, 0x1005,
0x103a, 0x1012, 0x103a, 0x101d, 0x1031, 0x1038, 0x1014, 0x102c, 0x1025, 0x1030, 0x101b, 0x1030,
0x1014, 0x102e, 0x1026, 0x1038, 0x1021, 0x1031, 0x1014, 0x1031, 0x102c, 0x103a, 0x1016, 0x1031,
0x102c, 0x1037, 0x1001, 0x103a, 0x1014, 0x1030, 0x1019, 0x101a, 0x103a, 0x1021, 0x102c, 0x1015,
0x102b, 0x1002, 0x102d, 0x102f, 0x1015, 0x102b, 0x1002, 0x102d, 0x102f, 0x1015, 0x101c, 0x102c,
0x1021, 0x102d, 0x102f, 0x1015, 0x102b, 0x1010, 0x103a, 0x1000, 0x101a, 0x103a, 0x101b, 0x1004,
0x103a, 0x1016, 0x102d, 0x102f, 0x1014, 0x102c, 0x1016, 0x1032, 0x1021, 0x103a, 0x1016, 0x102d,
0x102f, 0x1037, 0x1010, 0x103a, 0x20, 0x1019, 0x102d, 0x102f, 0x101b, 0x1000, 0x103a, 0x1005,
0x103a, 0x1018, 0x102e, 0x101b, 0x102c, 0x101b, 0x102d, 0x102f, 0x1010, 0x103d, 0x1014, 0x103a,
0x1038, 0x1002, 0x102b, 0x1006, 0x102d, 0x102f, 0x1004, 0x103a, 0x1015, 0x1014, 0x103a, 0x1010,
0x101f, 0x102e, 0x1010, 0x102e, 0x1010, 0x102c, 0x101b, 0x102c, 0x101d, 0x102b, 0x1010, 0x103d,
0x1014, 0x103a, 0x1002, 0x102b, 0x1010, 0x102c, 0x1015, 0x102f, 0x101d, 0x102d, 0x1010, 0x103a,
0x1001, 0x103a, 0x101d, 0x1031, 0x102b, 0x101c, 0x102e, 0x1005, 0x103a, 0x963f, 0x6bd4, 0x8b93,
0x963f, 0x514b, 0x62c9, 0x963f, 0x8fea, 0x65af, 0x963f, 0x8c9d, 0x5df4, 0x963f, 0x723e, 0x53ca,
0x723e, 0x963f, 0x65af, 0x746a, 0x62c9, 0x5df4, 0x99ac, 0x79d1, 0x73ed, 0x5409, 0x73ed, 0x7af9,
0x6bd4, 0x7d39, 0x5e03, 0x862d, 0x592a, 0x723e, 0x5e03, 0x62c9, 0x67f4, 0x7dad, 0x723e, 0x5e03,
0x677e, 0x5e03, 0x62c9, 0x958b, 0x7f85, 0x5361, 0x85a9, 0x5e03, 0x862d, 0x5361, 0x4f11, 0x9054,
0x67ef, 0x90a3, 0x514b, 0x91cc, 0x9054, 0x5580, 0x723e, 0x6c99, 0x862d, 0x6e2f, 0x5409, 0x5e03,
0x5730, 0x675c, 0x963f, 0x62c9, 0x963f, 0x5c24, 0x6069, 0x81ea, 0x7531, 0x57ce, 0x5609, 0x67cf,
0x9686, 0x91cc, 0x54c8, 0x62c9, 0x96f7, 0x7d04, 0x7ff0, 0x5c3c, 0x65af, 0x5821, 0x6731, 0x5df4,
0x574e, 0x5e15, 0x62c9, 0x5580, 0x571f, 0x7a46, 0x57fa, 0x52a0, 0x5229, 0x91d1, 0x590f, 0x6c99,
0x62c9, 0x54e5, 0x65af, 0x81ea, 0x7531, 0x5e02, 0x6d1b, 0x7f8e, 0x7f85, 0x5b89, 0x9054, 0x76e7,
0x672c, 0x5df4, 0x5e0c, 0x8def, 0x6c99, 0x5361, 0x99ac, 0x62c9, 0x535a, 0x99ac, 0x666e, 0x6258,
0x99ac, 0x8cfd, 0x9b6f, 0x58a8, 0x5df4, 0x672c, 0x6469, 0x52a0, 0x8fea, 0x4f11, 0x8499, 0x7f85,
0x7dad, 0x4e9e, 0x5948, 0x6d1b, 0x6bd4, 0x6069, 0x8cc8, 0x6885, 0x7d0d, 0x5c3c, 0x4e9e, 0x7f8e,
0x8afe, 0x514b, 0x5c11, 0x74e6, 0x52a0, 0x675c, 0x53e4, 0x6ce2, 0x591a, 0x8afe, 0x4f5b, 0x8056,
0x591a, 0x7f8e, 0x7684, 0x9ece, 0x6ce2, 0x91cc, 0x7a81, 0x5c3c, 0x65af, 0x6eab, 0x5f97, 0x548c,
0x514b, 0x827e, 0x9054, 0x514b, 0x5b89, 0x514b, 0x62c9, 0x6cbb, 0x5b89, 0x5409, 0x62c9, 0x5b89,
0x5730, 0x5361, 0x963f, 0x62c9, 0x572d, 0x90a3, 0x5e03, 0x5b9c, 0x8afe, 0x65af, 0x827e, 0x5229,
0x65af, 0x5361, 0x5854, 0x99ac, 0x5361, 0x54e5, 0x591a, 0x83ef, 0x80e1, 0x80e1, 0x4f0a, 0x62c9,
0x7565, 0x54c8, 0x9580, 0x591a, 0x85a9, 0x91cc, 0x5967, 0x52a0, 0x8036, 0x6208, 0x65af, 0x85a9,
0x723e, 0x5854, 0x8056, 0x80e1, 0x5b89, 0x8056, 0x8def, 0x6613, 0x5410, 0x5eab, 0x66fc, 0x70cf,
0x65af, 0x61f7, 0x4e9e, 0x963f, 0x8def, 0x5df4, 0x4e9e, 0x677e, 0x68ee, 0x963f, 0x8482, 0x79d1,
0x80af, 0x5df4, 0x4f0a, 0x963f, 0x5df4, 0x4f0a, 0x4e9e, 0x73ed, 0x5fb7, 0x62c9, 0x65af, 0x5df4,
0x8c9d, 0x591a, 0x8c9d, 0x502b, 0x8c9d, 0x91cc, 0x65af, 0x767d, 0x6717, 0x85a9, 0x5e03, 0x9686,
0x4fdd, 0x7dad, 0x65af, 0x5854, 0x6ce2, 0x54e5, 0x5927, 0x6ce2, 0x5915, 0x528d, 0x6a4b, 0x7063,
0x683c, 0x862d, 0x5834, 0x574e, 0x6606, 0x5361, 0x62c9, 0x5361, 0x65af, 0x958b, 0x96f2, 0x958b,
0x66fc, 0x7fa4, 0x5cf6, 0x829d, 0x52a0, 0x54e5, 0x5947, 0x83ef, 0x83ef, 0x83ef, 0x96f7, 0x65af,
0x57ce, 0x54e5, 0x65af, 0x5927, 0x9ece, 0x52a0, 0x514b, 0x96f7, 0x65af, 0x9813, 0x53e4, 0x96c5,
0x5df4, 0x5eab, 0x62c9, 0x7d22, 0x4e39, 0x99ac, 0x6c99, 0x6587, 0x9053, 0x68ee, 0x9053, 0x68ee,
0x514b, 0x91cc, 0x514b, 0x4e39, 0x4f5b, 0x5e95, 0x7279, 0x5f8b, 0x591a, 0x660e, 0x5c3c, 0x52a0,
0x827e, 0x5fb7, 0x8499, 0x541e, 0x827e, 0x9b6f, 0x5167, 0x4f69, 0x85a9, 0x723e, 0x74e6, 0x591a,
0x7d0d, 0x723e, 0x905c, 0x5821, 0x798f, 0x5854, 0x529b, 0x838e, 0x683c, 0x96f7, 0x65af, 0x8c9d,
0x9d5d, 0x7063, 0x5927, 0x7279, 0x514b, 0x5cf6, 0x683c, 0x745e, 0x7d0d, 0x9054, 0x74dc, 0x5730,
0x6d1b, 0x666e, 0x74dc, 0x5730, 0x99ac, 0x62c9, 0x74dc, 0x4e9e, 0x57fa, 0x723e, 0x84cb, 0x4e9e,
0x90a3, 0x54c8, 0x91cc, 0x6cd5, 0x514b, 0x65af, 0x54c8, 0x74e6, 0x90a3, 0x57c3, 0x83ab, 0x897f,
0x7d04, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x6ce2, 0x91cc, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x90a3,
0x5dde, 0x8afe, 0x514b, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x99ac, 0x502b, 0x54e5,
0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x5f7c, 0x5f97, 0x5821, 0x5370, 0x7b2c, 0x5b89, 0x90a3,
0x5dde, 0x6cf0, 0x723e, 0x57ce, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x7dad, 0x5a01, 0x5370,
0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x6eab, 0x68ee, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde,
0x5a01, 0x7d0d, 0x99ac, 0x514b, 0x4f0a, 0x5974, 0x7dad, 0x514b, 0x4f0a, 0x9b41, 0x7279, 0x7259,
0x8cb7, 0x52a0, 0x6731, 0x8afe, 0x8def, 0x6613, 0x65af, 0x7dad, 0x723e, 0x80af, 0x5854, 0x57fa,
0x5dde, 0x8499, 0x5730, 0x537b, 0x7f85, 0x514b, 0x62c9, 0x502b, 0x4ee3, 0x514b, 0x62c9, 0x5df4,
0x65af, 0x5229, 0x99ac, 0x6d1b, 0x6749, 0x78ef, 0x4e0b, 0x592a, 0x5b50, 0x5340, 0x99ac, 0x745f,
0x6b50, 0x99ac, 0x62ff, 0x74dc, 0x746a, 0x7459, 0x65af, 0x99ac, 0x91cc, 0x6208, 0x7279, 0x99ac,
0x4e01, 0x5c3c, 0x514b, 0x99ac, 0x5854, 0x83ab, 0x7f85, 0x65af, 0x99ac, 0x85a9, 0x7279, 0x862d,
0x7f8e, 0x8afe, 0x7c73, 0x5c3c, 0x6885, 0x91cc, 0x9054, 0x6885, 0x7279, 0x62c9, 0x5361, 0x7279,
0x62c9, 0x58a8, 0x897f, 0x54e5, 0x5e02, 0x5bc6, 0x555f, 0x5d19, 0x8499, 0x514b, 0x9813, 0x8499,
0x7279, 0x745e, 0x8499, 0x7279, 0x7dad, 0x591a, 0x8499, 0x54f2, 0x81d8, 0x62ff, 0x7d22, 0x7d10,
0x7d04, 0x8afe, 0x59c6, 0x8afe, 0x502b, 0x54c8, 0x5317, 0x9054, 0x79d1, 0x4ed6, 0x5dde, 0x5e03,
0x7531, 0x62c9, 0x5317, 0x9054, 0x79d1, 0x4ed6, 0x5dde, 0x4e2d, 0x5fc3, 0x5317, 0x9054, 0x79d1,
0x4ed6, 0x5dde, 0x7d10, 0x6c99, 0x502b, 0x52aa, 0x514b, 0x5967, 0x5e0c, 0x7d0d, 0x52a0, 0x5df4,
0x62ff, 0x99ac, 0x5df4, 0x62c9, 0x99ac, 0x5229, 0x6ce2, 0x9cf3, 0x51f0, 0x57ce, 0x592a, 0x5b50,
0x6e2f, 0x897f, 0x73ed, 0x7259, 0x6e2f, 0x7dad, 0x7559, 0x6e2f, 0x6ce2, 0x591a, 0x9ece, 0x5404,
0x84ec, 0x5854, 0x963f, 0x96f7, 0x7d0d, 0x65af, 0x862d, 0x4eca, 0x7063, 0x96f7, 0x897f, 0x975e,
0x91cc, 0x8cc8, 0x7d0d, 0x7f85, 0x65af, 0x9b6f, 0x7279, 0x91cc, 0x7d04, 0x5e03, 0x862d, 0x8056,
0x5854, 0x502b, 0x8056, 0x5730, 0x7259, 0x54e5, 0x8056, 0x591a, 0x660e, 0x54e5, 0x8056, 0x4fdd,
0x7f85, 0x4f0a, 0x6258, 0x79d1, 0x723e, 0x6258, 0x7c73, 0x7279, 0x932b, 0x7279, 0x5361, 0x8056,
0x5df4, 0x6258, 0x6d1b, 0x7e46, 0x5cf6, 0x8056, 0x7d04, 0x7ff0, 0x8056, 0x57fa, 0x8328, 0x8056,
0x9732, 0x897f, 0x4e9e, 0x8056, 0x6258, 0x99ac, 0x65af, 0x8056, 0x6587, 0x68ee, 0x65af, 0x5a01,
0x592b, 0x7279, 0x5361, 0x502b, 0x7279, 0x5fb7, 0x53e4, 0x65af, 0x52a0, 0x5df4, 0x675c, 0x91cc,
0x63d0, 0x83ef, 0x7d0d, 0x591a, 0x502b, 0x591a, 0x6258, 0x723e, 0x6258, 0x62c9, 0x6eab, 0x54e5,
0x83ef, 0x61f7, 0x7279, 0x970d, 0x65af, 0x6eab, 0x5c3c, 0x4f2f, 0x96c5, 0x5eab, 0x5854, 0x51f1,
0x897f, 0x6234, 0x7dad, 0x65af, 0x675c, 0x8499, 0x675c, 0x6bd4, 0x723e, 0x9ea5, 0x89ba, 0x7406,
0x83ab, 0x68ee, 0x9ea5, 0x514b, 0x9ed8, 0x591a, 0x5e15, 0x9ea5, 0x7f85, 0x745f, 0x62c9, 0x662d,
0x548c, 0x57fa, 0x5730, 0x7dbd, 0x723e, 0x6c83, 0x65af, 0x6258, 0x514b, 0x9686, 0x610f, 0x8033,
0x62dc, 0x6069, 0x4e9e, 0x4e01, 0x963f, 0x62c9, 0x6728, 0x5716, 0x5b89, 0x66fc, 0x963f, 0x90a3,
0x5e95, 0x963f, 0x514b, 0x5957, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x963f, 0x4ec0, 0x54c8, 0x5df4,
0x7279, 0x963f, 0x7279, 0x52de, 0x5df4, 0x683c, 0x9054, 0x5df4, 0x6797, 0x66fc, 0x8c37, 0x5df4,
0x723e, 0x7459, 0x723e, 0x8c9d, 0x9b6f, 0x7279, 0x6bd4, 0x4ec0, 0x51f1, 0x514b, 0x6c76, 0x840a,
0x8d64, 0x5854, 0x55ac, 0x5df4, 0x5c71, 0x53ef, 0x502b, 0x5761, 0x5927, 0x99ac, 0x58eb, 0x9769,
0x9054, 0x5361, 0x5e1d, 0x529b, 0x675c, 0x62dc, 0x675c, 0x6851, 0x8c9d, 0x6cd5, 0x99ac, 0x53e4,
0x65af, 0x5854, 0x52a0, 0x85a9, 0x8d6b, 0x5e03, 0x9686, 0x80e1, 0x5fd7, 0x660e, 0x5e02, 0x4e2d,
0x83ef, 0x4eba, 0x6c11, 0x5171, 0x548c, 0x570b, 0x9999, 0x6e2f, 0x7279, 0x5225, 0x884c, 0x653f,
0x5340, 0x79d1, 0x5e03, 0x591a, 0x4f0a, 0x723e, 0x5eab, 0x6b21, 0x514b, 0x96c5, 0x52a0, 0x9054,
0x52a0, 0x4e9e, 0x5e03, 0x62c9, 0x8036, 0x8def, 0x6492, 0x51b7, 0x5580, 0x5e03, 0x723e, 0x582a,
0x5bdf, 0x52a0, 0x5580, 0x62c9, 0x86a9, 0x52a0, 0x5fb7, 0x6eff, 0x90fd, 0x582a, 0x5730, 0x52a0,
0x52a0, 0x723e, 0x5404, 0x7b54, 0x514b, 0x62c9, 0x65af, 0x8afe, 0x4e9e, 0x723e, 0x65af, 0x514b,
0x5409, 0x9686, 0x5761, 0x53e4, 0x6649, 0x79d1, 0x5a01, 0x7279, 0x4e2d, 0x83ef, 0x4eba, 0x6c11,
0x5171, 0x548c, 0x570b, 0x6fb3, 0x9580, 0x7279, 0x5225, 0x884c, 0x653f, 0x5340, 0x99ac, 0x52a0,
0x4e39, 0x99ac, 0x5361, 0x6c99, 0x723e, 0x99ac, 0x5c3c, 0x62c9, 0x99ac, 0x65af, 0x958b, 0x7279,
0x5c3c, 0x53e4, 0x897f, 0x4e9e, 0x65b0, 0x5eab, 0x8332, 0x6d85, 0x8328, 0x514b, 0x65b0, 0x897f,
0x4f2f, 0x5229, 0x4e9e, 0x9102, 0x6728, 0x65af, 0x514b, 0x70cf, 0x62c9, 0x723e, 0x91d1, 0x908a,
0x5764, 0x7538, 0x5e73, 0x58e4, 0x5361, 0x9054, 0x79d1, 0x65af, 0x5854, 0x5948, 0x514b, 0x5b5c,
0x52d2, 0x5967, 0x723e, 0x9054, 0x5229, 0x96c5, 0x5fb7, 0x5eab, 0x9801, 0x5cf6, 0x6492, 0x99ac,
0x723e, 0x7f55, 0x9996, 0x723e, 0x4e0a, 0x6d77, 0x65b0, 0x52a0, 0x5761, 0x4e2d, 0x79d1, 0x96f7,
0x59c6, 0x65af, 0x514b, 0x53f0, 0x5317, 0x5854, 0x4ec0, 0x5e72, 0x7b2c, 0x6bd4, 0x5229, 0x65af,
0x5fb7, 0x9ed1, 0x862d, 0x5ef7, 0x5e03, 0x6771, 0x4eac, 0x6258, 0x6728, 0x65af, 0x514b, 0x70cf,
0x862d, 0x5df4, 0x6258, 0x70cf, 0x9b6f, 0x6728, 0x9f4a, 0x70cf, 0x65af, 0x5167, 0x62c9, 0x6c38,
0x73cd, 0x6d77, 0x53c3, 0x5d34, 0x96c5, 0x5eab, 0x6b21, 0x514b, 0x4ef0, 0x5149, 0x8449, 0x5361,
0x6377, 0x6797, 0x5821, 0x8449, 0x91cc, 0x6eab, 0x4e9e, 0x901f, 0x723e, 0x7fa4, 0x5cf6, 0x767e,
0x6155, 0x9054, 0x52a0, 0x7d0d, 0x5229, 0x7dad, 0x5fb7, 0x89d2, 0x6cd5, 0x7f85, 0x7fa4, 0x5cf6,
0x99ac, 0x5f97, 0x62c9, 0x7fa4, 0x5cf6, 0x96f7, 0x514b, 0x96c5, 0x7dad, 0x514b, 0x5357, 0x55ac,
0x6cbb, 0x4e9e, 0x8056, 0x8d6b, 0x52d2, 0x62ff, 0x5cf6, 0x53f2, 0x5766, 0x5229, 0x963f, 0x5f97,
0x96f7, 0x5fb7, 0x5e03, 0x5229, 0x65af, 0x73ed, 0x5e03, 0x7f85, 0x80af, 0x5e0c, 0x723e, 0x9054,
0x723e, 0x6587, 0x5c24, 0x514b, 0x62c9, 0x8377, 0x5df4, 0x7279, 0x6797, 0x5fb7, 0x66fc, 0x8c6a,
0x52f3, 0x7235, 0x5cf6, 0x58a8, 0x723e, 0x672c, 0x4f2f, 0x65af, 0x96ea, 0x68a8, 0x963f, 0x59c6,
0x65af, 0x7279, 0x4e39, 0x5b89, 0x9053, 0x723e, 0x963f, 0x65af, 0x7279, 0x62c9, 0x7f55, 0x96c5,
0x5178, 0x8c9d, 0x723e, 0x683c, 0x52d2, 0x67cf, 0x6797, 0x5e03, 0x62c9, 0x63d0, 0x65af, 0x62c9,
0x74e6, 0x5e03, 0x9b6f, 0x585e, 0x723e, 0x5e03, 0x52a0, 0x52d2, 0x65af, 0x7279, 0x5e03, 0x9054,
0x4f69, 0x65af, 0x5e03, 0x8f9b, 0x6839, 0x5947, 0x897f, 0x7459, 0x54e5, 0x672c, 0x54c8, 0x6839,
0x90fd, 0x67cf, 0x6797, 0x76f4, 0x5e03, 0x7f85, 0x9640, 0x6839, 0x606f, 0x5cf6, 0x8d6b, 0x723e,
0x8f9b, 0x57fa, 0x66fc, 0x5cf6, 0x4f0a, 0x65af, 0x5766, 0x5821, 0x6fa4, 0x897f, 0x5cf6, 0x52a0,
0x91cc, 0x5be7, 0x683c, 0x52d2, 0x57fa, 0x6d1b, 0x592b, 0x57fa, 0x8f14, 0x91cc, 0x65af, 0x672c,
0x76e7, 0x6bd4, 0x5b89, 0x7d0d, 0x502b, 0x6566, 0x76e7, 0x68ee, 0x5821, 0x99ac, 0x5fb7, 0x91cc,
0x99ac, 0x723e, 0x4ed6, 0x746a, 0x9e97, 0x6e2f, 0x660e, 0x65af, 0x514b, 0x6469, 0x7d0d, 0x54e5,
0x83ab, 0x65af, 0x79d1, 0x5967, 0x65af, 0x9678, 0x5df4, 0x9ece, 0x6ce2, 0x591a, 0x91cc, 0x5bdf,
0x5e03, 0x62c9, 0x683c, 0x91cc, 0x52a0, 0x7f85, 0x99ac, 0x6c99, 0x99ac, 0x62c9, 0x8056, 0x99ac,
0x5229, 0x8afe, 0x585e, 0x62c9, 0x8036, 0x4f5b, 0x85a9, 0x62c9, 0x6258, 0x592b, 0x8f9b, 0x975e,
0x6d1b, 0x6d66, 0x53f2, 0x9ad8, 0x6bd4, 0x8036, 0x7d22, 0x83f2, 0x4e9e, 0x65af, 0x5fb7, 0x54e5,
0x723e, 0x6469, 0x5854, 0x6797, 0x5730, 0x62c9, 0x90a3, 0x70cf, 0x91cc, 0x63da, 0x8afe, 0x592b,
0x65af, 0x514b, 0x74e6, 0x90fd, 0x8332, 0x68b5, 0x8482, 0x5ca1, 0x7dad, 0x4e5f, 0x7d0d, 0x7dad,
0x723e, 0x7d10, 0x65af, 0x4f0f, 0x723e, 0x52a0, 0x683c, 0x52d2, 0x83ef, 0x6c99, 0x672d, 0x683c,
0x745e, 0x5e03, 0x8607, 0x9ece, 0x4e16, 0x5b89, 0x5854, 0x90a3, 0x90a3, 0x5229, 0x4f5b, 0x67e5,
0x6208, 0x65af, 0x8056, 0x8a95, 0x5cf6, 0x79d1, 0x79d1, 0x65af, 0x7fa4, 0x5cf6, 0x79d1, 0x6469,
0x7f85, 0x7fa4, 0x5cf6, 0x51f1, 0x723e, 0x84cb, 0x6717, 0x5cf6, 0x99ac, 0x57c3, 0x5cf6, 0x99ac,
0x723e, 0x5730, 0x592b, 0x6a21, 0x91cc, 0x897f, 0x65af, 0x99ac, 0x7d04, 0x7279, 0x5cf6, 0x7559,
0x5c3c, 0x65fa, 0x5cf6, 0x963f, 0x76ae, 0x4e9e, 0x5967, 0x514b, 0x862d, 0x5e03, 0x5e72, 0x7dad,
0x723e, 0x67e5, 0x5766, 0x695a, 0x514b, 0x5fa9, 0x6d3b, 0x5cf6, 0x57c3, 0x6cd5, 0x7279, 0x6cd5,
0x8003, 0x798f, 0x6590, 0x6fdf, 0x5bcc, 0x90a3, 0x5bcc, 0x63d0, 0x52a0, 0x62c9, 0x5df4, 0x54e5,
0x7fa4, 0x5cf6, 0x7518, 0x6bd4, 0x723e, 0x7fa4, 0x5cf6, 0x74dc, 0x9054, 0x5eb7, 0x7d0d, 0x723e,
0x5cf6, 0x95dc, 0x5cf6, 0x6a80, 0x9999, 0x5c71, 0x6069, 0x5f97, 0x4f2f, 0x7406, 0x5cf6, 0x57fa,
0x91cc, 0x5730, 0x99ac, 0x5730, 0x5cf6, 0x79d1, 0x65af, 0x745e, 0x74dc, 0x52a0, 0x6797, 0x5cf6,
0x99ac, 0x6731, 0x8afe, 0x99ac, 0x53ef, 0x85a9, 0x65af, 0x5cf6, 0x4e2d, 0x9014, 0x5cf6, 0x8afe,
0x9b6f, 0x7d10, 0x57c3, 0x5cf6, 0x8afe, 0x798f, 0x514b, 0x8afe, 0x7f8e, 0x4e9e, 0x5df4, 0x54e5,
0x5df4, 0x54e5, 0x5e1b, 0x7409, 0x76ae, 0x7279, 0x80af, 0x7fa4, 0x5cf6, 0x6ce2, 0x7d0d, 0x4f69,
0x83ab, 0x58eb, 0x6bd4, 0x6e2f, 0x62c9, 0x7f85, 0x6e6f, 0x52a0, 0x585e, 0x73ed, 0x5927, 0x6eaa,
0x5730, 0x5854, 0x62c9, 0x74e6, 0x6771, 0x52a0, 0x5854, 0x5e03, 0x5cf6, 0x5a01, 0x514b, 0x74e6,
0x5229, 0x65af, 0x963f, 0x6bd4, 0x8ba9, 0x963f, 0x8fea, 0x65af, 0x963f, 0x8d1d, 0x5df4, 0x963f,
0x5c14, 0x53ca, 0x5c14, 0x963f, 0x65af, 0x739b, 0x62c9, 0x5df4, 0x9a6c, 0x79d1, 0x6bd4, 0x7ecd,
0x5e03, 0x5170, 0x592a, 0x5c14, 0x5e03, 0x62c9, 0x67f4, 0x7ef4, 0x5c14, 0x5f00, 0x7f57, 0x5361,
0x8428, 0x5e03, 0x5170, 0x5361, 0x4f11, 0x8fbe, 0x8fbe, 0x5580, 0x5c14, 0x6c99, 0x5170, 0x6e2f,
0x7ea6, 0x7ff0, 0x5c3c, 0x65af, 0x5821, 0x7f57, 0x5b89, 0x8fbe, 0x5362, 0x672c, 0x5df4, 0x5e0c,
0x9a6c, 0x62c9, 0x535a, 0x9a6c, 0x666e, 0x6258, 0x9a6c, 0x8d5b, 0x9c81, 0x8499, 0x7f57, 0x7ef4,
0x4e9a, 0x6069, 0x8d3e, 0x6885, 0x7eb3, 0x5c3c, 0x4e9a, 0x7f8e, 0x8bfa, 0x514b, 0x5c11, 0x6ce2,
0x591a, 0x8bfa, 0x4f5b, 0x5723, 0x591a, 0x7f8e, 0x6e29, 0x5f97, 0x548c, 0x514b, 0x827e, 0x8fbe,
0x514b, 0x5e03, 0x5b9c, 0x8bfa, 0x65af, 0x827e, 0x5229, 0x65af, 0x5361, 0x5854, 0x9a6c, 0x5361,
0x54e5, 0x591a, 0x534e, 0x95e8, 0x591a, 0x8428, 0x91cc, 0x5965, 0x52a0, 0x8036, 0x6208, 0x65af,
0x8428, 0x5c14, 0x5854, 0x5723, 0x80e1, 0x5b89, 0x5723, 0x8def, 0x6613, 0x5410, 0x5e93, 0x66fc,
0x4e4c, 0x65af, 0x6000, 0x4e9a, 0x4e9a, 0x677e, 0x68ee, 0x5df4, 0x4f0a, 0x4e9a, 0x73ed, 0x5fb7,
0x62c9, 0x65af, 0x5df4, 0x8d1d, 0x591a, 0x8d1d, 0x4f26, 0x8d1d, 0x91cc, 0x65af, 0x767d, 0x6717,
0x8428, 0x5e03, 0x9686, 0x4fdd, 0x7ef4, 0x65af, 0x5854, 0x5251, 0x6865, 0x6e7e, 0x683c, 0x5170,
0x573a, 0x5f00, 0x4e91, 0x5f00, 0x66fc, 0x7fa4, 0x5c9b, 0x5947, 0x534e, 0x534e, 0x534e, 0x96f7,
0x65af, 0x57ce, 0x514b, 0x96f7, 0x65af, 0x987f, 0x5e93, 0x62c9, 0x7d22, 0x4e39, 0x9a6c, 0x6c99,
0x6587, 0x827e, 0x9c81, 0x5185, 0x4f69, 0x8428, 0x5c14, 0x74e6, 0x591a, 0x7eb3, 0x5c14, 0x900a,
0x5821, 0x683c, 0x96f7, 0x65af, 0x8d1d, 0x9e45, 0x6e7e, 0x5927, 0x7279, 0x514b, 0x5c9b, 0x683c,
0x745e, 0x7eb3, 0x8fbe, 0x74dc, 0x5730, 0x9a6c, 0x62c9, 0x74dc, 0x4e9a, 0x57fa, 0x5c14, 0x76d6,
0x4e9a, 0x90a3, 0x57c3, 0x83ab, 0x897f, 0x7ea6, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x8bfa,
0x514b, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x9a6c, 0x4f26, 0x54e5, 0x5370, 0x7b2c,
0x5b89, 0x90a3, 0x5dde, 0x6cf0, 0x5c14, 0x57ce, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x7ef4,
0x5a01, 0x5370, 0x7b2c, 0x5b89, 0x90a3, 0x5dde, 0x6e29, 0x68ee, 0x65af, 0x5370, 0x7b2c, 0x5b89,
0x90a3, 0x5dde, 0x5a01, 0x7eb3, 0x9a6c, 0x514b, 0x4f0a, 0x5974, 0x7ef4, 0x514b, 0x7259, 0x4e70,
0x52a0, 0x6731, 0x8bfa, 0x8def, 0x6613, 0x65af, 0x7ef4, 0x5c14, 0x80af, 0x5854, 0x57fa, 0x5dde,
0x8499, 0x5730, 0x5374, 0x7f57, 0x514b, 0x62c9, 0x4f26, 0x4ee3, 0x514b, 0x5229, 0x9a6c, 0x6d1b,
0x6749, 0x77f6, 0x4e0b, 0x592a, 0x5b50, 0x533a, 0x9a6c, 0x745f, 0x6b27, 0x9a6c, 0x62ff, 0x74dc,
0x739b, 0x7459, 0x65af, 0x9a6c, 0x91cc, 0x6208, 0x7279, 0x9a6c, 0x4e01, 0x5c3c, 0x514b, 0x9a6c,
0x5854, 0x83ab, 0x7f57, 0x65af, 0x9a6c, 0x8428, 0x7279, 0x5170, 0x7f8e, 0x8bfa, 0x7c73, 0x5c3c,
0x6885, 0x91cc, 0x8fbe, 0x5bc6, 0x542f, 0x4ed1, 0x8499, 0x514b, 0x987f, 0x8499, 0x7279, 0x7ef4,
0x591a, 0x8499, 0x54f2, 0x814a, 0x7ebd, 0x7ea6, 0x8bfa, 0x59c6, 0x8bfa, 0x4f26, 0x54c8, 0x5317,
0x8fbe, 0x79d1, 0x4ed6, 0x5dde, 0x5e03, 0x7531, 0x62c9, 0x5317, 0x8fbe, 0x79d1, 0x4ed6, 0x5dde,
0x4e2d, 0x5fc3, 0x5317, 0x8fbe, 0x79d1, 0x4ed6, 0x5dde, 0x7ebd, 0x6c99, 0x4f26, 0x5965, 0x5e0c,
0x7eb3, 0x52a0, 0x5df4, 0x62ff, 0x9a6c, 0x5df4, 0x62c9, 0x9a6c, 0x5229, 0x6ce2, 0x51e4, 0x51f0,
0x57ce, 0x7ef4, 0x7559, 0x6e2f, 0x84ec, 0x5854, 0x963f, 0x96f7, 0x7eb3, 0x65af, 0x5170, 0x4eca,
0x6e7e, 0x91cc, 0x8d3e, 0x7eb3, 0x7f57, 0x65af, 0x9c81, 0x7279, 0x91cc, 0x7ea6, 0x5e03, 0x5170,
0x5723, 0x5854, 0x4f26, 0x5723, 0x5730, 0x7259, 0x54e5, 0x5723, 0x591a, 0x660e, 0x54e5, 0x5723,
0x4fdd, 0x7f57, 0x4f0a, 0x6258, 0x79d1, 0x5c14, 0x6258, 0x7c73, 0x7279, 0x9521, 0x7279, 0x5361,
0x5723, 0x5df4, 0x6258, 0x6d1b, 0x7f2a, 0x5c9b, 0x5723, 0x7ea6, 0x7ff0, 0x5723, 0x57fa, 0x8328,
0x5723, 0x9732, 0x897f, 0x4e9a, 0x5723, 0x6258, 0x9a6c, 0x65af, 0x5723, 0x6587, 0x68ee, 0x65af,
0x5a01, 0x592b, 0x7279, 0x5361, 0x4f26, 0x7279, 0x63d0, 0x534e, 0x7eb3, 0x591a, 0x4f26, 0x591a,
0x6258, 0x5c14, 0x6258, 0x62c9, 0x6e29, 0x54e5, 0x534e, 0x6000, 0x7279, 0x970d, 0x65af, 0x6e29,
0x5c3c, 0x4f2f, 0x96c5, 0x5e93, 0x5854, 0x51ef, 0x897f, 0x6234, 0x7ef4, 0x65af, 0x675c, 0x8499,
0x675c, 0x6bd4, 0x5c14, 0x9ea6, 0x89c9, 0x7406, 0x9ea6, 0x514b, 0x9ed8, 0x591a, 0x5e15, 0x9ea6,
0x7f57, 0x745f, 0x62c9, 0x7ef0, 0x5c14, 0x4e9a, 0x4e01, 0x963f, 0x62c9, 0x6728, 0x56fe, 0x963f,
0x7279, 0x52b3, 0x5df4, 0x683c, 0x8fbe, 0x5df4, 0x5e93, 0x5df4, 0x5c14, 0x7459, 0x5c14, 0x8d1d,
0x9c81, 0x7279, 0x6bd4, 0x4ec0, 0x51ef, 0x514b, 0x6c76, 0x83b1, 0x4e54, 0x5df4, 0x5c71, 0x53ef,
0x4f26, 0x5761, 0x5927, 0x9a6c, 0x58eb, 0x9769, 0x8fbe, 0x5361, 0x675c, 0x6851, 0x8d1d, 0x6cd5,
0x9a6c, 0x53e4, 0x65af, 0x5854, 0x52a0, 0x8428, 0x4e2d, 0x534e, 0x4eba, 0x6c11, 0x5171, 0x548c,
0x56fd, 0x9999, 0x6e2f, 0x7279, 0x522b, 0x884c, 0x653f, 0x533a, 0x4f0a, 0x5c14, 0x5e93, 0x6b21,
0x514b, 0x96c5, 0x52a0, 0x8fbe, 0x52a0, 0x4e9a, 0x5e03, 0x62c9, 0x5580, 0x5e03, 0x5c14, 0x52a0,
0x5fb7, 0x6ee1, 0x90fd, 0x52a0, 0x5c14, 0x5404, 0x7b54, 0x514b, 0x62c9, 0x65af, 0x8bfa, 0x4e9a,
0x5c14, 0x65af, 0x514b, 0x53e4, 0x664b, 0x4e2d, 0x534e, 0x4eba, 0x6c11, 0x5171, 0x548c, 0x56fd,
0x6fb3, 0x95e8, 0x7279, 0x522b, 0x884c, 0x653f, 0x533a, 0x9a6c, 0x52a0, 0x4e39, 0x9a6c, 0x5361,
0x6c99, 0x5c14, 0x9a6c, 0x5c3c, 0x62c9, 0x9a6c, 0x65af, 0x5f00, 0x7279, 0x5c3c, 0x53e4, 0x897f,
0x4e9a, 0x65b0, 0x5e93, 0x5179, 0x6d85, 0x8328, 0x514b, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9a,
0x4e4c, 0x62c9, 0x5c14, 0x91d1, 0x8fb9, 0x5361, 0x8fbe, 0x514b, 0x5b5c, 0x52d2, 0x5965, 0x5c14,
0x8fbe, 0x5e93, 0x9875, 0x5c9b, 0x6492, 0x9a6c, 0x5c14, 0x7f55, 0x9996, 0x5c14, 0x5fb7, 0x9ed1,
0x5170, 0x4e1c, 0x4eac, 0x4e4c, 0x5170, 0x5df4, 0x6258, 0x4e4c, 0x9c81, 0x6728, 0x9f50, 0x4e4c,
0x65af, 0x5185, 0x62c9, 0x6d77, 0x53c2, 0x5d34, 0x96c5, 0x5e93, 0x6b21, 0x514b, 0x53f6, 0x5361,
0x6377, 0x6797, 0x5821, 0x53f6, 0x91cc, 0x6e29, 0x4e9a, 0x901f, 0x5c14, 0x7fa4, 0x5c9b, 0x767e,
0x6155, 0x8fbe, 0x52a0, 0x7eb3, 0x5229, 0x7ef4, 0x5fb7, 0x89d2, 0x6cd5, 0x7f57, 0x7fa4, 0x5c9b,
0x9a6c, 0x5f97, 0x62c9, 0x7fa4, 0x5c9b, 0x96f7, 0x514b, 0x96c5, 0x7ef4, 0x514b, 0x5357, 0x4e54,
0x6cbb, 0x4e9a, 0x5723, 0x8d6b, 0x52d2, 0x62ff, 0x5c9b, 0x5e03, 0x7f57, 0x80af, 0x5e0c, 0x5c14,
0x8fbe, 0x5c14, 0x6587, 0x8c6a, 0x52cb, 0x7235, 0x5c9b, 0x58a8, 0x5c14, 0x672c, 0x5b89, 0x9053,
0x5c14, 0x8d1d, 0x5c14, 0x683c, 0x52d2, 0x5e03, 0x9c81, 0x585e, 0x5c14, 0x5e03, 0x8fbe, 0x4f69,
0x65af, 0x76f4, 0x5e03, 0x7f57, 0x9640, 0x6839, 0x606f, 0x5c9b, 0x8d6b, 0x5c14, 0x8f9b, 0x57fa,
0x66fc, 0x5c9b, 0x6cfd, 0x897f, 0x5c9b, 0x52a0, 0x91cc, 0x5b81, 0x683c, 0x52d2, 0x57fa, 0x8f85,
0x5362, 0x6bd4, 0x5b89, 0x7eb3, 0x4f26, 0x6566, 0x5362, 0x68ee, 0x5821, 0x9a6c, 0x5fb7, 0x91cc,
0x9a6c, 0x5c14, 0x4ed6, 0x739b, 0x4e3d, 0x6e2f, 0x6469, 0x7eb3, 0x54e5, 0x5965, 0x65af, 0x9646,
0x7f57, 0x9a6c, 0x6c99, 0x9a6c, 0x62c9, 0x5723, 0x9a6c, 0x5229, 0x8bfa, 0x8428, 0x62c9, 0x6258,
0x592b, 0x7d22, 0x83f2, 0x4e9a, 0x65af, 0x5fb7, 0x54e5, 0x5c14, 0x6469, 0x4e4c, 0x91cc, 0x626c,
0x8bfa, 0x592b, 0x65af, 0x514b, 0x74e6, 0x90fd, 0x5179, 0x68b5, 0x8482, 0x5188, 0x7ef4, 0x4e5f,
0x7eb3, 0x7ef4, 0x5c14, 0x7ebd, 0x65af, 0x4f0f, 0x5c14, 0x52a0, 0x683c, 0x52d2, 0x534e, 0x6c99,
0x82cf, 0x9ece, 0x4e16, 0x5723, 0x8bde, 0x5c9b, 0x79d1, 0x79d1, 0x65af, 0x7fa4, 0x5c9b, 0x79d1,
0x6469, 0x7f57, 0x7fa4, 0x5c9b, 0x51ef, 0x5c14, 0x76d6, 0x6717, 0x5c9b, 0x9a6c, 0x57c3, 0x5c9b,
0x9a6c, 0x5c14, 0x5730, 0x592b, 0x9a6c, 0x7ea6, 0x7279, 0x5c9b, 0x7559, 0x5c3c, 0x65fa, 0x5c9b,
0x963f, 0x76ae, 0x4e9a, 0x5965, 0x514b, 0x5170, 0x5e03, 0x5e72, 0x7ef4, 0x5c14, 0x590d, 0x6d3b,
0x5c9b, 0x6590, 0x6d4e, 0x52a0, 0x62c9, 0x5df4, 0x54e5, 0x7fa4, 0x5c9b, 0x7518, 0x6bd4, 0x5c14,
0x7fa4, 0x5c9b, 0x74dc, 0x8fbe, 0x5eb7, 0x7eb3, 0x5c14, 0x5c9b, 0x5173, 0x5c9b, 0x6069, 0x5f97,
0x4f2f, 0x7406, 0x5c9b, 0x57fa, 0x91cc, 0x5730, 0x9a6c, 0x5730, 0x5c9b, 0x74dc, 0x52a0, 0x6797,
0x5c9b, 0x9a6c, 0x6731, 0x8bfa, 0x9a6c, 0x53ef, 0x8428, 0x65af, 0x5c9b, 0x4e2d, 0x9014, 0x5c9b,
0x8bfa, 0x9c81, 0x7ebd, 0x57c3, 0x5c9b, 0x8bfa, 0x798f, 0x514b, 0x8bfa, 0x7f8e, 0x4e9a, 0x76ae,
0x7279, 0x80af, 0x7fa4, 0x5c9b, 0x6ce2, 0x7eb3, 0x4f69, 0x62c9, 0x7f57, 0x6c64, 0x52a0, 0x4e1c,
0x52a0, 0x5854, 0x5e03, 0x5c9b, 0x41, 0x6c, 0x67, 0x65, 0x72, 0x43, 0x61, 0x69,
0x72, 0x65, 0x2c, 0x20, 0x65, 0x6c, 0x61, 0x6c, 0x2d, 0x41, 0x61, 0x69,
0x75, 0x6e, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x73, 0x63, 0x69, 0x6f,
0x4d, 0x6f, 0x6e, 0x72, 0xf2, 0x76, 0x69, 0x61, 0x4e, 0x2019, 0x44, 0x6a,
0x61, 0x6d, 0x65, 0x6e, 0x61, 0x52, 0xed, 0x6f, 0x20, 0x47, 0x61, 0x6c,
0x6c, 0x65, 0x67, 0x6f, 0x73, 0x42, 0x6c, 0x61, 0x6e, 0x63, 0x20, 0x53,
0x61, 0x62, 0x6c, 0x6f, 0x6e, 0x42, 0x6f, 0x67, 0x6f, 0x74, 0xe0, 0x43,
0x61, 0x69, 0x65, 0x6e, 0x61, 0x43, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x43,
0x69, 0x75, 0x74, 0x61, 0x74, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe8, 0x78,
0x69, 0x63, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x42,
0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x4e, 0x65, 0x77,
0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x50,
0x61, 0x6e, 0x61, 0x6d, 0xe0, 0x53, 0x63, 0x6f, 0x72, 0x65, 0x73, 0x62,
0x79, 0x73, 0x75, 0x6e, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x54,
0x68, 0x6f, 0x6d, 0x61, 0x73, 0x41, 0x6e, 0xe0, 0x64, 0x69, 0x72, 0x41,
0x71, 0x74, 0x61, 0xfd, 0x41, 0x71, 0x74, 0xf3, 0x62, 0x65, 0x41, 0x15f,
0x67, 0x61, 0x62, 0x61, 0x74, 0x41, 0x74, 0x79, 0x72, 0x61, 0xfd, 0x42,
0x61, 0x72, 0x6e, 0x61, 0xfc, 0x6c, 0x54, 0x78, 0x69, 0x74, 0xe0, 0x44,
0x75, 0x69, 0x78, 0x61, 0x6e, 0x62, 0xe9, 0x48, 0x1ed3, 0x20, 0x43, 0x68,
0xed, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x4a, 0x61, 0x69, 0x70, 0x75, 0x72,
0x4e, 0x69, 0x63, 0xf2, 0x73, 0x69, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x51, 0x6f, 0x73, 0x74, 0x61,
0x6e, 0x61, 0x69, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0xed, 0x6e, 0x53,
0x65, 0xfc, 0x6c, 0x54, 0x61, 0x69, 0x78, 0x6b, 0x65, 0x6e, 0x74, 0x54,
0xf2, 0x71, 0x75, 0x69, 0x6f, 0xdc, 0x72, 0xfc, 0x6d, 0x63, 0x68, 0x69,
0x55, 0x73, 0x74, 0x2019, 0x2d, 0x4e, 0x65, 0x72, 0x61, 0x49, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x49, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x42,
0x65, 0x72, 0x6d, 0x75, 0x64, 0x65, 0x73, 0x49, 0x6c, 0x6c, 0x65, 0x73,
0x20, 0x43, 0x61, 0x6e, 0xe0, 0x72, 0x69, 0x65, 0x73, 0x43, 0x61, 0x70,
0x20, 0x56, 0x65, 0x72, 0x64, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46,
0xe8, 0x72, 0x6f, 0x65, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x64, 0x41, 0x73, 0x74, 0x72, 0x61,
0x63, 0x61, 0x6e, 0x42, 0x72, 0x75, 0x73, 0x73, 0x65, 0x6c, 0xb7, 0x6c,
0x65, 0x73, 0x43, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x67, 0x75, 0x65,
0x6e, 0x48, 0xe8, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x4b, 0xed, 0x72,
0x6f, 0x76, 0x4b, 0xed, 0x69, 0x76, 0x4d, 0xf2, 0x6e, 0x61, 0x63, 0x6f,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x53, 0x61, 0x72, 0xe0, 0x74, 0x6f,
0x76, 0x53, 0x69, 0x6d, 0x66, 0x65, 0x72, 0xf2, 0x70, 0x6f, 0x6c, 0x55,
0x6c, 0x69, 0xe0, 0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x56, 0x61, 0x74, 0x69,
0x63, 0xe0, 0x56, 0xed, 0x6c, 0x6e, 0x69, 0x75, 0x73, 0x56, 0x61, 0x72,
0x73, 0xf2, 0x76, 0x69, 0x61, 0x5a, 0xfa, 0x72, 0x69, 0x63, 0x49, 0x6c,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x65, 0x73, 0x54, 0x61, 0x68,
0x69, 0x74, 0xed, 0x53, 0x69, 0x79, 0x75, 0x64, 0x61, 0x64, 0x20, 0x73,
0x61, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e,
0x68, 0xd804, 0xdd03, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd34, 0xd804,
0xdd0e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd03, 0xd804, 0xdd13, 0xd804, 0xdd33, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd1d, 0xd804, 0xdd1d, 0xd804, 0xdd03,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd1d,
0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd1d,
0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0xd804, 0xdd2a, 0xd804, 0xdd1d,
0xd804, 0xdd1a, 0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd25, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd20,
0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd2a,
0xd804, 0xdd07, 0xd804, 0xdd20, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd07,
0xd804, 0xdd25, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd01, 0xd804, 0xdd07,
0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd07,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd25, 0xd804, 0xdd23, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28,
0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd2e,
0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd26, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd26, 0xd804, 0xdd22, 0xd804,
0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd26, 0xd804, 0xdd1a, 0xd804,
0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd09, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd07, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd23, 0xd804, 0xdd08, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd23, 0xd804, 0xdd09, 0xd804,
0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804,
0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804,
0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd23, 0xd804,
0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804,
0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd07, 0xd804,
0xdd1f, 0xd804, 0xdd23, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd1b, 0xd804,
0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd26, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd1d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804,
0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804,
0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd2d, 0xd804,
0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd24, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804,
0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804,
0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0x2d,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd03,
0xd804, 0xdd2e, 0x20, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804,
0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804,
0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd2e, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804,
0xdd03, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd03, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804,
0xdd09, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804,
0xdd2d, 0xd804, 0xdd1a, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd20,
0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd11, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13,
0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd26, 0xd804, 0xdd2a,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd23, 0x20, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804,
0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd26, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804,
0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd09,
0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd23, 0xd804, 0xdd2d, 0xd804, 0xdd2a,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd2a,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd05, 0xd804, 0xdd25, 0xd804, 0xdd2a,
0xd804, 0xdd20, 0xd804, 0xdd2d, 0xd804, 0xdd20, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd03, 0xd804, 0xdd25, 0xd804, 0xdd2a, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd23, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd26, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1d, 0xd804,
0xdd26, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1d, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd0e,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd0e,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd01, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x2d, 0xd804, 0xdd25, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804,
0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd31, 0x20, 0xd804, 0xdd1e,
0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1d, 0xd804, 0xdd2e,
0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd11, 0xd804, 0xdd1d, 0xd804, 0xdd27, 0xd804, 0xdd20,
0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd0e,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd07,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd26, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c,
0xd804, 0xdd28, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd26, 0xd804, 0xdd2a,
0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd2d, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd1d, 0xd804, 0xdd07,
0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd13,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1e, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2e,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2e,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804,
0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804,
0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2d, 0xd804, 0xdd22, 0xd804,
0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804,
0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd25, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd1e, 0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1c,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd27, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd13, 0xd804, 0xdd09, 0xd804, 0xdd31, 0xd804, 0xdd18, 0xd804, 0xdd2c,
0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd31,
0xd804, 0xdd16, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd23, 0xd804, 0xdd09, 0xd804, 0xdd2e,
0xd804, 0xdd20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd28,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd26,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1c, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd26,
0xd804, 0xdd1e, 0xd804, 0xdd1a, 0xd804, 0xdd26, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd23,
0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd23,
0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09,
0xd804, 0xdd2e, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd1b, 0xd804, 0xdd28,
0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd11, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd28, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd1e, 0xd804, 0xdd2c, 0xd804, 0xdd1e, 0xd804, 0xdd2c, 0xd804, 0xdd20, 0xd804, 0xdd34,
0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1a,
0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd1f,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x2c, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2a, 0xd804, 0xdd1e,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd07, 0xd804, 0xdd31, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd1f, 0xd804, 0xdd2d, 0xd804, 0xdd07,
0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd2d,
0xd804, 0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0x2c, 0x20,
0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd07,
0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd0e, 0xd804, 0xdd28,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd23, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd0e, 0xd804,
0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd07,
0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd20, 0xd804, 0xdd2e,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd09, 0xd804, 0xdd31, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd16, 0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2e,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd0e, 0xd804, 0xdd16, 0xd804, 0xdd34,
0xd804, 0xdd23, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd1f,
0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd24, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd25, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804,
0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804,
0xdd27, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2c, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804,
0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd25, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804,
0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd1d, 0xd804,
0xdd2c, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0x2c, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd11, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd20,
0x2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804,
0xdd2e, 0xd804, 0xdd11, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0x20,
0xd804, 0xdd25, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0x2c, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804,
0xdd11, 0xd804, 0xdd1a, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804,
0xdd2e, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804,
0xdd1b, 0xd804, 0xdd1a, 0xd804, 0xdd1f, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804,
0xdd22, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804,
0xdd2e, 0xd804, 0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x2d, 0xd804, 0xdd26, 0xd804, 0xdd27, 0x2d, 0xd804,
0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd2e, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd26,
0xd804, 0xdd2e, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804,
0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd0b, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd11,
0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1c,
0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd2a,
0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2e,
0x20, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1f,
0xd804, 0xdd28, 0xd804, 0xdd01, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd03,
0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804,
0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804,
0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x2e,
0x20, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd17, 0xd804, 0xdd2c, 0xd804,
0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd0e, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x2e, 0x20, 0xd804, 0xdd07,
0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x2e, 0x20,
0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd25,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd17, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd2d, 0xd804, 0xdd2a,
0xd804, 0xdd1c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd07, 0xd804,
0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd33, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804,
0xdd09, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd17, 0xd804, 0xdd2a, 0xd804,
0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804,
0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd11, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd1e, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804,
0xdd1e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd20, 0xd804, 0xdd2d, 0xd804,
0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd34, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804,
0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c, 0xd804,
0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd13, 0xd804, 0xdd28,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd28,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd05, 0xd804, 0xdd20, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2e, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2e,
0xd804, 0xdd1b, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd17, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd24, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd01, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd03, 0xd804, 0xdd26, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd11,
0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd26, 0xd804, 0xdd34,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd13, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd03,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd16,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd16,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd01,
0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2f, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2c, 0xd804, 0xdd2d, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2c,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd2d, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd16,
0xd804, 0xdd0c, 0xd804, 0xdd30, 0xd804, 0xdd31, 0xd804, 0xdd1d, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd1d,
0xd804, 0xdd27, 0xd804, 0xdd27, 0xd804, 0xdd31, 0xd804, 0xdd31, 0xd804, 0xdd18, 0xd804, 0xdd1f,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd14,
0xd804, 0xdd07, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd18,
0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd2d, 0xd804, 0xdd18, 0xd804, 0xdd2a, 0xd804, 0xdd25,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd0e,
0xd804, 0xdd26, 0xd804, 0xdd2c, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2e,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd0c, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd25, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd01,
0xd804, 0xdd26, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2a,
0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd0e, 0xd804, 0xdd07, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd0e,
0xd804, 0xdd27, 0xd804, 0xdd20, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd0e,
0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd23, 0xd804, 0xdd2c,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd23,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd0c, 0xd804, 0xdd11,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd0c,
0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2a, 0xd804, 0xdd08, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd07,
0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd18, 0xd804, 0xdd07,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd23,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd01, 0xd804, 0xdd07,
0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd13, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd07, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd28,
0xd804, 0xdd23, 0xd804, 0xdd1f, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd16,
0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd2e,
0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2c,
0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27,
0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd01, 0xd804, 0xdd07, 0xd804, 0xdd16,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd19, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd08,
0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd08, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd01,
0xd804, 0xdd0a, 0xd804, 0xdd2d, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd0b, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1a,
0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd20,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2d, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd06,
0xd804, 0xdd2c, 0xd804, 0xdd16, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd08, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2a,
0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd16, 0xd804, 0xdd2c, 0xd804, 0xdd26, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd17, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1c,
0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd03,
0xd804, 0xdd2e, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd16, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd1e, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1e, 0xd804,
0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804,
0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd11, 0xd804,
0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd1e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd03, 0xd804, 0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804,
0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2a, 0xd804,
0xdd13, 0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804,
0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd2d,
0xd804, 0xdd22, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd0e,
0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd27,
0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd0e, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd26, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd25,
0xd804, 0xdd33, 0xd804, 0xdd11, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28,
0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd23,
0xd804, 0xdd2c, 0xd804, 0xdd2d, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2e,
0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26, 0xd804,
0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804,
0xdd23, 0xd804, 0xdd26, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd13, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd26, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd2c,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd17, 0xd804, 0xdd27,
0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd28,
0xd804, 0xdd03, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd03,
0xd804, 0xdd33, 0xd804, 0xdd03, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd03, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd08, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2c,
0xd804, 0xdd17, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd33,
0xd804, 0xdd23, 0xd804, 0xdd1e, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2a, 0xd804, 0xdd08, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd13, 0xd804, 0xdd1b,
0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd09,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd26, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd2e,
0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd2c,
0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd1b,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd0e,
0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd23, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd26,
0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd01,
0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2d, 0xd804, 0xdd23, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd28, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd2c,
0xd804, 0xdd1e, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c,
0xd804, 0xdd1e, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd1d, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd2a,
0xd804, 0xdd1d, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd13, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd27,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd26, 0xd804, 0xdd1f, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2e,
0xd804, 0xdd1a, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd27,
0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd07, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd09, 0xd804, 0xdd34,
0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd1f,
0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804,
0xdd22, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804,
0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1c, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804,
0xdd2e, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd33, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0xd804, 0xdd21, 0xd804,
0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd2e, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd11, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd26, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd26, 0xd804, 0xdd23, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804,
0xdd1a, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd18, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804,
0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804,
0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd24, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804,
0xdd28, 0xd804, 0xdd08, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd1a, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1e, 0xd804,
0xdd2e, 0xd804, 0xdd0d, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804,
0xdd21, 0xd804, 0xdd2a, 0xd804, 0xdd25, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e,
0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd26, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd23,
0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd20, 0xd804, 0xdd2e, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd1a,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd03,
0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd2c,
0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd17, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1c, 0xd804, 0xdd16, 0xd804, 0xdd2c,
0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2e,
0xd804, 0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28,
0xd804, 0xdd1c, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd1c, 0xd804, 0xdd2a, 0xd804, 0xdd11,
0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd1b,
0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd09, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd20,
0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd20,
0xd804, 0xdd18, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd23,
0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd31, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd26,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0xd804, 0xdd23,
0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07,
0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd1f,
0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd22, 0xd804, 0xdd20, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd31, 0xd804, 0xdd0e,
0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd2d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd13, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22,
0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd20,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1c,
0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2a, 0xd804, 0xdd1f,
0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09,
0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804,
0xdd2e, 0xd804, 0xdd1b, 0xd804, 0xdd23, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804, 0xdd1b, 0xd804,
0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd20, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804,
0xdd1a, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd2f, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd22,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd01, 0xd804, 0xdd09,
0xd804, 0xdd25, 0xd804, 0xdd2d, 0xd804, 0xdd1b, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd22,
0xd804, 0xdd24, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd01, 0xd804, 0xdd09, 0xd804, 0xdd11,
0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd2c,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd28,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0x411, 0x440, 0x430, 0x437, 0x437, 0x430, 0x432, 0x438,
0x43b, 0x44c, 0x41a, 0x44a, 0x430, 0x445, 0x4c0, 0x438, 0x440, 0x430, 0x419, 0x43e,
0x445, 0x430, 0x43d, 0x43d, 0x435, 0x441, 0x431, 0x443, 0x440, 0x433, 0x41b, 0x438,
0x431, 0x440, 0x435, 0x432, 0x438, 0x43b, 0x44c, 0x410, 0x43d, 0x43a, 0x43e, 0x440,
0x438, 0x434, 0x436, 0x410, 0x43d, 0x433, 0x438, 0x43b, 0x44c, 0x44f, 0x411, 0x443,
0x44d, 0x43d, 0x43e, 0x441, 0x2d, 0x410, 0x439, 0x440, 0x435, 0x441, 0x420, 0x438,
0x43e, 0x2d, 0x413, 0x430, 0x43b, 0x44c, 0x435, 0x433, 0x43e, 0x441, 0x410, 0x441,
0x443, 0x43d, 0x441, 0x44c, 0x43e, 0x43d, 0x411, 0x430, 0x438, 0x44f, 0x2d, 0x434,
0x435, 0x2d, 0x411, 0x430, 0x43d, 0x434, 0x435, 0x440, 0x430, 0x441, 0x411, 0x43e,
0x439, 0x441, 0x435, 0x41a, 0x435, 0x439, 0x43c, 0x431, 0x440, 0x438, 0x434, 0x436,
0x2d, 0x411, 0x435, 0x439, 0x41a, 0x430, 0x439, 0x435, 0x43d, 0x43d, 0x430, 0x42d,
0x434, 0x43c, 0x43e, 0x43d, 0x442, 0x43e, 0x43d, 0x42d, 0x439, 0x440, 0x443, 0x43d,
0x435, 0x43f, 0x435, 0x424, 0x43e, 0x440, 0x442, 0x20, 0x41d, 0x435, 0x43b, 0x44c,
0x441, 0x43e, 0x43d, 0x413, 0x43b, 0x435, 0x439, 0x441, 0x2d, 0x411, 0x435, 0x439,
0x413, 0x443, 0x441, 0x2d, 0x411, 0x435, 0x439, 0x413, 0x443, 0x430, 0x44f, 0x43a,
0x438, 0x43b, 0x44c, 0x413, 0x430, 0x439, 0x430, 0x43d, 0x430, 0x413, 0x430, 0x43b,
0x438, 0x444, 0x430, 0x43a, 0x441, 0x42d, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x43b,
0x44c, 0x43e, 0x412, 0x438, 0x432, 0x44d, 0x439, 0x41b, 0x43e, 0x441, 0x2d, 0x410,
0x43d, 0x434, 0x436, 0x435, 0x43b, 0x435, 0x441, 0x41b, 0x43e, 0x443, 0x44d, 0x440,
0x2d, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x435, 0x441, 0x2d, 0x41a, 0x443, 0x43e,
0x440, 0x442, 0x435, 0x440, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x440, 0x440, 0x435,
0x439, 0x41c, 0x43e, 0x43d, 0x441, 0x435, 0x440, 0x440, 0x430, 0x442, 0x41d, 0x44c,
0x44e, 0x2d, 0x419, 0x43e, 0x440, 0x43a, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x445,
0x430, 0x411, 0x43e, 0x439, 0x43b, 0x430, 0x2c, 0x20, 0x41a, 0x44a, 0x438, 0x43b,
0x431, 0x430, 0x441, 0x435, 0x434, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442,
0x430, 0x42e, 0x43a, 0x44a, 0x2c, 0x20, 0x41a, 0x44a, 0x438, 0x43b, 0x431, 0x430,
0x441, 0x435, 0x434, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d,
0x44c, 0x44e, 0x2d, 0x421, 0x430, 0x43b, 0x435, 0x43c, 0x413, 0x43e, 0x442, 0x445,
0x43e, 0x431, 0x41f, 0x443, 0x44d, 0x440, 0x442, 0x43e, 0x2d, 0x420, 0x438, 0x43a,
0x43e, 0x420, 0x435, 0x434, 0x436, 0x430, 0x439, 0x43d, 0x430, 0x421, 0x430, 0x43d,
0x442, 0x44c, 0x44f, 0x433, 0x43e, 0x421, 0x435, 0x43d, 0x2d, 0x411, 0x430, 0x440,
0x442, 0x435, 0x43b, 0x44c, 0x43c, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41b,
0x44e, 0x441, 0x438, 0x421, 0x432, 0x438, 0x444, 0x442, 0x2d, 0x41a, 0x430, 0x440,
0x435, 0x43d, 0x442, 0x422, 0x435, 0x433, 0x443, 0x441, 0x438, 0x433, 0x430, 0x43b,
0x44c, 0x43f, 0x430, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x42e,
0x440, 0x432, 0x438, 0x43b, 0x44c, 0x422, 0x440, 0x43e, 0x43b, 0x43b, 0x44c, 0x41b,
0x43e, 0x43d, 0x433, 0x439, 0x438, 0x440, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410,
0x442, 0x430, 0x4c0, 0x430, 0x43c, 0x43c, 0x430, 0x43d, 0x410, 0x43d, 0x430, 0x434,
0x44b, 0x440, 0x44c, 0x410, 0x442, 0x438, 0x440, 0x430, 0x443, 0x411, 0x430, 0x433,
0x4c0, 0x434, 0x430, 0x434, 0x411, 0x430, 0x43a, 0x43e, 0x445, 0x414, 0x438, 0x43c,
0x430, 0x448, 0x43a, 0x44a, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x414,
0x436, 0x430, 0x439, 0x43f, 0x443, 0x440, 0x41a, 0x44a, 0x443, 0x434, 0x441, 0x2d,
0x413, 0x4c0, 0x430, 0x43b, 0x430, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430,
0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430,
0x442, 0x441, 0x43a, 0x438, 0x41a, 0x430, 0x43b, 0x44c, 0x43a, 0x443, 0x442, 0x442,
0x430, 0x41f, 0x43d, 0x43e, 0x43c, 0x43f, 0x435, 0x43d, 0x44c, 0x42d, 0x440, 0x2d,
0x420, 0x438, 0x44f, 0x434, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20,
0x433, 0x4c0, 0x2d, 0x435, 0x422, 0x435, 0x445, 0x4c0, 0x440, 0x430, 0x43d, 0x423,
0x441, 0x442, 0x44c, 0x2d, 0x41d, 0x435, 0x440, 0x430, 0x412, 0x44c, 0x435, 0x43d,
0x442, 0x44c, 0x44f, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x430, 0x43d, 0x20, 0x433,
0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x411, 0x435, 0x440, 0x43c,
0x443, 0x434, 0x430, 0x448, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x430, 0x43d, 0x20,
0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x424, 0x430, 0x440,
0x435, 0x440, 0x438, 0x439, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435,
0x43d, 0x430, 0x448, 0x41c, 0x430, 0x434, 0x435, 0x439, 0x440, 0x430, 0x2c, 0x20,
0x433, 0x4c0, 0x2d, 0x435, 0x420, 0x435, 0x439, 0x43a, 0x44c, 0x44f, 0x432, 0x438,
0x43a, 0x41a, 0x44a, 0x438, 0x43b, 0x431, 0x430, 0x20, 0x414, 0x436, 0x43e, 0x440,
0x434, 0x436, 0x438, 0x415, 0x437, 0x430, 0x447, 0x443, 0x20, 0x415, 0x43b, 0x435,
0x43d, 0x438, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x421, 0x442,
0x44d, 0x43d, 0x43b, 0x438, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443,
0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x435, 0x421, 0x438, 0x434, 0x43d, 0x435, 0x439,
0x410, 0x448, 0x442, 0x430, 0x440, 0x43a, 0x445, 0x43d, 0x435, 0x410, 0x444, 0x438,
0x43d, 0x430, 0x411, 0x440, 0x44e, 0x441, 0x441, 0x435, 0x43b, 0x44c, 0x411, 0x44e,
0x437, 0x438, 0x43d, 0x433, 0x435, 0x43d, 0x2d, 0x430, 0x43c, 0x2d, 0x425, 0x43e,
0x445, 0x440, 0x430, 0x439, 0x43d, 0x41a, 0x43e, 0x43f, 0x435, 0x43d, 0x433, 0x430,
0x433, 0x435, 0x43d, 0x425, 0x435, 0x43b, 0x44c, 0x441, 0x438, 0x43d, 0x43a, 0x438,
0x41c, 0x44d, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x435, 0x414, 0x436, 0x435,
0x440, 0x441, 0x438, 0x421, 0x438, 0x43c, 0x444, 0x435, 0x440, 0x43e, 0x43f, 0x43e,
0x43b, 0x44c, 0x421, 0x43a, 0x43e, 0x43f, 0x44c, 0x435, 0x421, 0x442, 0x43e, 0x43a,
0x433, 0x43e, 0x43b, 0x44c, 0x43c, 0x423, 0x43b, 0x44c, 0x44f, 0x43d, 0x43e, 0x432,
0x441, 0x43a, 0x412, 0x438, 0x43b, 0x44c, 0x43d, 0x44e, 0x441, 0x4c0, 0x438, 0x439,
0x441, 0x430, 0x20, 0x43f, 0x430, 0x439, 0x445, 0x430, 0x43c, 0x430, 0x440, 0x20,
0x432, 0x438, 0x43d, 0x447, 0x443, 0x20, 0x434, 0x435, 0x43d, 0x20, 0x433, 0x4c0,
0x430, 0x439, 0x440, 0x435, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x438, 0x439, 0x43d,
0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41a, 0x43e, 0x43c, 0x43e,
0x440, 0x438, 0x439, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d,
0x430, 0x448, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x430, 0x448, 0x41c,
0x430, 0x439, 0x43e, 0x440, 0x43a, 0x430, 0x420, 0x435, 0x44e, 0x43d, 0x44c, 0x43e,
0x43d, 0x411, 0x443, 0x433, 0x435, 0x43d, 0x432, 0x438, 0x43b, 0x44c, 0x2c, 0x20,
0x433, 0x4c0, 0x2d, 0x435, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x2c, 0x20, 0x433,
0x4c0, 0x2d, 0x435, 0x422, 0x440, 0x443, 0x43a, 0x2c, 0x20, 0x433, 0x4c0, 0x2d,
0x43d, 0x430, 0x448, 0x41c, 0x430, 0x440, 0x445, 0x438, 0x43d, 0x20, 0x433, 0x4c0,
0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x42d, 0x444, 0x430, 0x442, 0x435,
0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x430, 0x43d, 0x20,
0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x413, 0x430, 0x43c,
0x431, 0x44c, 0x435, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448,
0x413, 0x43e, 0x43d, 0x43e, 0x43b, 0x443, 0x43b, 0x443, 0x42d, 0x43d, 0x434, 0x435,
0x440, 0x431, 0x435, 0x440, 0x438, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x435,
0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x430, 0x43d, 0x2c, 0x20, 0x433, 0x4c0,
0x2d, 0x43d, 0x430, 0x448, 0x41c, 0x438, 0x434, 0x443, 0x44d, 0x439, 0x2c, 0x20,
0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41d, 0x438, 0x443, 0x44d, 0x41f, 0x43e,
0x43d, 0x430, 0x43f, 0x435, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448,
0x422, 0x430, 0x438, 0x442, 0x438, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430,
0x448, 0x423, 0x44d, 0x439, 0x43a, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x435, 0x13a0,
0x13c8, 0x13e3, 0x13c2, 0x13a0, 0x13c6, 0x13a1, 0x13d7, 0x13cd, 0x20, 0x13a0, 0x13c6, 0x13c6,
0x13a0, 0x13b5, 0x13e5, 0x13b5, 0x13a0, 0x13cf, 0x13b9, 0x13b3, 0x13c6, 0x13b9, 0x13aa, 0x13c7,
0x13c2, 0x13ab, 0x13eb, 0x13c6, 0x13c2, 0x13e7, 0x13b5, 0x13c7, 0x13cc, 0x13eb, 0x13c6, 0x13d8,
0x13b4, 0x13c6, 0x13cc, 0x13e9, 0x13b5, 0x13ca, 0x13e7, 0x13bb, 0x13ca, 0x13b3, 0x13a7, 0x13ef,
0x13b6, 0x13a4, 0x13c1, 0x13a6, 0x20, 0x13a6, 0x13b5, 0x13e6, 0x13d5, 0x13d1, 0x13d4, 0x13aa,
0x13be, 0x13c8, 0x13d3, 0x13a7, 0x13e9, 0x13d3, 0x20, 0x13a1, 0x13cf, 0x20, 0x13cc, 0x13b3,
0x13bb, 0x13e5, 0x13ca, 0x13d7, 0x13d9, 0x13a4, 0x13e9, 0x13b3, 0x13a1, 0x13b5, 0x20, 0x13a0,
0x13f1, 0x13f3, 0x13c2, 0x13a0, 0x13ce, 0x13c7, 0x20, 0x13a6, 0x13da, 0x13b2, 0x13a6, 0x13c9,
0x13b6, 0x13c1, 0x13ad, 0x13b3, 0x13b4, 0x13e6, 0x13ad, 0x13c1, 0x13cd, 0x13ca, 0x13a6, 0x13e7,
0x13c6, 0x13a7, 0x13bb, 0x13c6, 0x13b3, 0x13a7, 0x13da, 0x13bb, 0x13a9, 0x13a6, 0x13b5, 0x13a8,
0x13c2, 0x13dd, 0x13cc, 0x13b3, 0x13aa, 0x13cf, 0x13b5, 0x13c7, 0x13eb, 0x13b5, 0x13b6, 0x13ba,
0x13b7, 0x13e9, 0x13c2, 0x13d3, 0x13b7, 0x13ca, 0x13c6, 0x13cf, 0x13b5, 0x13cc, 0x13a7, 0x13b9,
0x13b3, 0x13c9, 0x13b9, 0x13ca, 0x13d9, 0x13b9, 0x13ce, 0x13b5, 0x13c6, 0x13c7, 0x13c1, 0x13bc,
0x13a6, 0x13d7, 0x13e1, 0x13bc, 0x13c2, 0x13b6, 0x13eb, 0x13ef, 0x13be, 0x13f1, 0x13b6, 0x13c8,
0x13be, 0x13e3, 0x13ba, 0x13be, 0x13c2, 0x13ef, 0x13ba, 0x13be, 0x13ec, 0x13e3, 0x13d8, 0x13a3,
0x13e9, 0x13a6, 0x13da, 0x13ab, 0x13e5, 0x13f3, 0x13d7, 0x13d4, 0x13b3, 0x13d7, 0x13cd, 0x13d7,
0x2d, 0x13c3, 0x13ec, 0x13cc, 0x13a3, 0x13d9, 0x13ba, 0x13df, 0x13c9, 0x13b5, 0x13da, 0x13c2,
0x13cf, 0x13ea, 0x13c4, 0x13d7, 0x13b0, 0x13a9, 0x13a0, 0x13d3, 0x13a7, 0x13a0, 0x13c2, 0x13aa,
0x13b4, 0x13e5, 0x13a0, 0x13c2, 0x13a9, 0x13b3, 0x13a4, 0x13ea, 0x13d8, 0x13a0, 0x13b3, 0x13ab,
0x13e9, 0x13f1, 0x13be, 0x13a3, 0x13cd, 0x13d3, 0x20, 0x13a4, 0x13c3, 0x13b4, 0x13a7, 0x13d4,
0x13b9, 0x13a7, 0x13aa, 0x13d9, 0x13c6, 0x13e7, 0x13e7, 0x13eb, 0x13b3, 0x20, 0x13b5, 0x13f2,
0x13ad, 0x13ba, 0x13be, 0x13d9, 0x13cc, 0x13a6, 0x13f0, 0x13aa, 0x13cf, 0x20, 0x13a4, 0x13ea,
0x13f4, 0x13cc, 0x13b3, 0x13d4, 0x13cc, 0x13c2, 0x20, 0x13e9, 0x13c2, 0x13a4, 0x13d3, 0x13c5,
0x13d7, 0x20, 0x13b7, 0x13eb, 0x13cf, 0x13da, 0x13ab, 0x13b9, 0x13c2, 0x13a4, 0x13d1, 0x13e9,
0x13ef, 0x13a0, 0x13b7, 0x13c6, 0x13a0, 0x13d1, 0x13cf, 0x13f2, 0x13c5, 0x13a0, 0x13d8, 0x13aa,
0x13a6, 0x13c2, 0x13c6, 0x13af, 0x13ef, 0x13c6, 0x13af, 0x13ef, 0x20, 0x13c6, 0x13c2, 0x13d5,
0x13b3, 0x13cf, 0x13c6, 0x13c7, 0x13d9, 0x13cd, 0x13c7, 0x13b4, 0x13bb, 0x13c7, 0x13b5, 0x13cd,
0x13dd, 0x13c2, 0x2d, 0x13cc, 0x13e0, 0x13c2, 0x13a3, 0x13cd, 0x13d3, 0x20, 0x13a0, 0x13aa,
0x13b5, 0x13f0, 0x13d7, 0x13c9, 0x13aa, 0x13d4, 0x13c9, 0x13f1, 0x13cf, 0x13a8, 0x13bb, 0x13c8,
0x13e5, 0x20, 0x13a1, 0x13c9, 0x13c4, 0x13b8, 0x13d7, 0x13a4, 0x13d4, 0x13be, 0x20, 0x13e0,
0x13a8, 0x13cf, 0x13a8, 0x13c2, 0x13ab, 0x13c2, 0x13a7, 0x13b3, 0x13a7, 0x13cf, 0x13a7, 0x13f0,
0x13c2, 0x13a8, 0x13a2, 0x13b9, 0x13c2, 0x13e5, 0x13a7, 0x13a9, 0x13e5, 0x13e9, 0x13e9, 0x13cf,
0x13f3, 0x13d3, 0x20, 0x13c6, 0x13a1, 0x13cf, 0x13aa, 0x13cd, 0x13d3, 0x13b5, 0x13a7, 0x13de,
0x13cd, 0x13d4, 0x13c2, 0x13e7, 0x13eb, 0x13c6, 0x13ab, 0x13b3, 0x13a8, 0x13a3, 0x13d5, 0x13be,
0x13b9, 0x13a7, 0x13cc, 0x13e9, 0x13c2, 0x13d3, 0x13cc, 0x13c2, 0x13d3, 0x13cc, 0x13c2, 0x20,
0x13a4, 0x13ea, 0x13f4, 0x13a6, 0x13b8, 0x13b3, 0x13d7, 0x20, 0x13a6, 0x13da, 0x13b2, 0x13d7,
0x13e0, 0x13d8, 0x13d9, 0x13bb, 0x13c2, 0x13a7, 0x13a1, 0x13d7, 0x13b9, 0x13c2, 0x13d4, 0x13c2,
0x13a1, 0x13b7, 0x13c1, 0x13c7, 0x13a1, 0x13b5, 0x20, 0x13cc, 0x13b5, 0x13c6, 0x13d9, 0x13b5,
0x13d7, 0x13d0, 0x13f4, 0x20, 0x13c1, 0x13b5, 0x13cc, 0x13c2, 0x13ec, 0x13d4, 0x13b4, 0x13ce,
0x13de, 0x13cf, 0x20, 0x13a1, 0x13c9, 0x13c4, 0x13b8, 0x13d7, 0x13cc, 0x13cc, 0x20, 0x13a1,
0x13c9, 0x13c4, 0x13b8, 0x13d7, 0x13c4, 0x13ac, 0x13eb, 0x13f3, 0x13d2, 0x20, 0x13ac, 0x13be,
0x13cb, 0x13be, 0x13d3, 0x13e9, 0x13d3, 0x13b7, 0x13c7, 0x13e9, 0x13d4, 0x13b9, 0x13b3, 0x13c6,
0x13ef, 0x13a9, 0x13b5, 0x13a6, 0x13ef, 0x13be, 0x13ad, 0x13b5, 0x13e9, 0x13a7, 0x13cf, 0x13ad,
0x13e9, 0x13be, 0x13ae, 0x13bc, 0x13cf, 0x13b6, 0x13a2, 0x13c2, 0x13d7, 0x13a0, 0x13c2, 0x13a0,
0x13c9, 0x13b5, 0x13cf, 0x13c3, 0x13c8, 0x13cf, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0,
0x13be, 0x13b9, 0x13b4, 0x13c2, 0x13aa, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be,
0x13c8, 0x13d3, 0x13c8, 0x13a6, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13d6,
0x13b5, 0x20, 0x13a6, 0x13da, 0x13b2, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be,
0x13ea, 0x13ea, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13eb, 0x13c2, 0x13ce,
0x13c1, 0x13cf, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13eb, 0x13be, 0x13b9,
0x13a9, 0x2c, 0x20, 0x13a2, 0x13c2, 0x13d7, 0x13f0, 0x13be, 0x13a2, 0x13c4, 0x13eb, 0x13a9,
0x13a2, 0x13c6, 0x13b7, 0x13f1, 0x13d8, 0x13e3, 0x13ba, 0x13a2, 0x13a7, 0x13e7, 0x13c3, 0x13b7,
0x13eb, 0x13eb, 0x13b5, 0x13b9, 0x13c2, 0x13d4, 0x13ce, 0x13b6, 0x2c, 0x20, 0x13ac, 0x13c5,
0x13d3, 0x13a9, 0x13c6, 0x13b4, 0x13c2, 0x13d7, 0x13d9, 0x13af, 0x13be, 0x13cd, 0x13a9, 0x20,
0x13d7, 0x13c2, 0x13a7, 0x13bf, 0x13e9, 0x13d7, 0x13d9, 0x13af, 0x13a1, 0x13b3, 0x13d7, 0x20,
0x13d7, 0x13dc, 0x20, 0x13a4, 0x13ac, 0x13eb, 0x13f3, 0x13af, 0x20, 0x13a9, 0x13c4, 0x13d9,
0x13d7, 0x13b9, 0x13ce, 0x13f2, 0x13b9, 0x13be, 0x13c6, 0x13b9, 0x13be, 0x13a4, 0x13cf, 0x13b9,
0x13b5, 0x13a6, 0x13d8, 0x13b9, 0x13d8, 0x13c2, 0x13c7, 0x13b9, 0x13d4, 0x13bc, 0x13b6, 0x13cf,
0x13b9, 0x13cc, 0x13dd, 0x13c2, 0x13ba, 0x13c3, 0x13bb, 0x13c2, 0x13ba, 0x13b5, 0x13d3, 0x13ba,
0x13dd, 0x13a7, 0x13dd, 0x13a0, 0x13c2, 0x13cd, 0x13c6, 0x13c2, 0x20, 0x13a6, 0x13da, 0x13b2,
0x13bb, 0x13a8, 0x13b6, 0x13c2, 0x13b9, 0x13be, 0x13d4, 0x13c2, 0x13bc, 0x13d6, 0x13b5, 0x13bc,
0x13c2, 0x13d6, 0x13eb, 0x13d5, 0x13f2, 0x13b9, 0x13c2, 0x13d8, 0x13cc, 0x13b3, 0x13d7, 0x13be,
0x13cc, 0x13eb, 0x13c4, 0x13ef, 0x13a9, 0x13c3, 0x13ba, 0x13c3, 0x13b6, 0x13be, 0x13ad, 0x13c7,
0x13f3, 0x13b3, 0x2c, 0x20, 0x13e7, 0x13f4, 0x13e2, 0x20, 0x13d3, 0x13aa, 0x13d4, 0x13a0,
0x13f0, 0x13df, 0x2c, 0x20, 0x13e7, 0x13f4, 0x13e2, 0x20, 0x13d3, 0x13aa, 0x13d4, 0x13a2,
0x13e4, 0x20, 0x13ce, 0x13b4, 0x13bb, 0x2c, 0x20, 0x13e7, 0x13f4, 0x13e2, 0x20, 0x13d3,
0x13aa, 0x13d4, 0x13c4, 0x13a9, 0x13a3, 0x13e5, 0x13be, 0x13a6, 0x13c6, 0x13be, 0x13b9, 0x13c6,
0x13b3, 0x13b9, 0x13b4, 0x13c9, 0x13e7, 0x13b4, 0x13af, 0x13cc, 0x13c5, 0x13af, 0x13e5, 0x13f3,
0x13d7, 0x13d4, 0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x2d, 0x13be, 0x13bf, 0x2d, 0x13a4, 0x13ac,
0x13eb, 0x13f3, 0x13af, 0x13e5, 0x13f3, 0x13d7, 0x13d4, 0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x20,
0x13cd, 0x13c6, 0x13c2, 0x13a8, 0x13cd, 0x13db, 0x13a4, 0x13ea, 0x13d8, 0x20, 0x13e5, 0x13f3,
0x13d7, 0x13d4, 0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x13c7, 0x13a1, 0x13d9, 0x20, 0x13b5, 0x13a2,
0x13aa, 0x13ca, 0x13d4, 0x20, 0x13a1, 0x13eb, 0x13be, 0x13cd, 0x13b4, 0x13c2, 0x13a9, 0x13c2,
0x20, 0x13a2, 0x13c2, 0x13b4, 0x13d8, 0x13b4, 0x13cf, 0x13ea, 0x13b4, 0x13a9, 0x13be, 0x13b4,
0x13d0, 0x13b7, 0x13d8, 0x13a4, 0x13c1, 0x13a6, 0x20, 0x13a4, 0x13ea, 0x13f4, 0x13cc, 0x13c2,
0x13d4, 0x13b4, 0x13bb, 0x13cc, 0x13c2, 0x13d8, 0x13ef, 0x13aa, 0x13cc, 0x13c2, 0x13d9, 0x20,
0x13d9, 0x13bb, 0x13aa, 0x13cc, 0x13a3, 0x20, 0x13c6, 0x13b6, 0x13a2, 0x13d9, 0x13aa, 0x13d9,
0x13bb, 0x13d8, 0x13cf, 0x13d8, 0x13a7, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13c6, 0x13b5,
0x13de, 0x13b4, 0x13bb, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13e3, 0x13c2, 0x20, 0x13a4,
0x13e4, 0x13b5, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13a9, 0x13d8, 0x13cf, 0x13a4, 0x13d3,
0x13c5, 0x13d8, 0x20, 0x13b7, 0x13cf, 0x13ef, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13d9,
0x13b9, 0x13cf, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13eb, 0x13c2, 0x13ce, 0x13d8, 0x13a0,
0x13ef, 0x13c4, 0x13b5, 0x20, 0x13a4, 0x13c3, 0x13b4, 0x13d6, 0x13ab, 0x13cf, 0x13a6, 0x13b5,
0x13c6, 0x13e1, 0x13b5, 0x13d8, 0x13f3, 0x13e9, 0x13be, 0x13d9, 0x13b3, 0x13be, 0x13d9, 0x13d9,
0x13d9, 0x13b3, 0x13ea, 0x13c2, 0x13ab, 0x13ea, 0x13b5, 0x13a4, 0x13c1, 0x13a6, 0x20, 0x13d0,
0x13c8, 0x13b5, 0x13eb, 0x13c2, 0x13c7, 0x13a9, 0x13ef, 0x13ab, 0x13d4, 0x13d8, 0x13d5, 0x13eb,
0x13cf, 0x13da, 0x13bc, 0x13be, 0x13d8, 0x2d, 0x13d7, 0x2019, 0x13a4, 0x13b5, 0x13eb, 0x13b5,
0x13b9, 0x13c7, 0x13b5, 0x13b9, 0x13cc, 0x13c2, 0x13bb, 0x13a6, 0x13bd, 0x13d9, 0x13c6, 0x13b5,
0x13ba, 0x13b5, 0x13b3, 0x13de, 0x13b3, 0x13cf, 0x13f2, 0x13e9, 0x13e0, 0x13b5, 0x13ec, 0x13cd,
0x13d9, 0x13a7, 0x13a6, 0x13c5, 0x13af, 0x13d3, 0x20, 0x13a4, 0x13d5, 0x13d8, 0x13f4, 0x13cc,
0x13d7, 0x13d2, 0x13a2, 0x20, 0x13a6, 0x13da, 0x13b2, 0x13a1, 0x13d5, 0x13c2, 0x13a0, 0x13b5,
0x13b9, 0x13d8, 0x13a0, 0x13b9, 0x13c2, 0x13a0, 0x13be, 0x13d7, 0x13b5, 0x13a0, 0x13a6, 0x13d4,
0x13eb, 0x13a0, 0x13a6, 0x13d9, 0x13c7, 0x13a0, 0x13cd, 0x13a6, 0x13c6, 0x13d8, 0x13a0, 0x13d8,
0x13c6, 0x13a4, 0x13c6, 0x13a9, 0x13d3, 0x13d7, 0x13c6, 0x13ad, 0x13b4, 0x13a2, 0x13c2, 0x13c7,
0x13c2, 0x13a9, 0x13aa, 0x13a9, 0x13c6, 0x13be, 0x13a3, 0x13b5, 0x13c7, 0x13b7, 0x13d8, 0x13c7,
0x13cd, 0x13a8, 0x13a9, 0x13ca, 0x13be, 0x13a2, 0x13e5, 0x13d4, 0x13e6, 0x13f1, 0x13c6, 0x13b5,
0x13cc, 0x13c2, 0x13aa, 0x13b6, 0x13bb, 0x13c9, 0x13d3, 0x13b9, 0x13cd, 0x13ac, 0x13cf, 0x13da,
0x13c6, 0x13f1, 0x13da, 0x13dd, 0x13be, 0x13c7, 0x13c6, 0x13b9, 0x13ab, 0x13cd, 0x13d4, 0x13a6,
0x13cc, 0x13ae, 0x13c9, 0x13c2, 0x13b0, 0x20, 0x13e5, 0x20, 0x13bb, 0x13c2, 0x20, 0x13a6,
0x13da, 0x13b2, 0x13b0, 0x13c2, 0x13a9, 0x20, 0x13aa, 0x13c2, 0x13a9, 0x13b0, 0x13e9, 0x13d7,
0x13a2, 0x13ab, 0x13e5, 0x13a7, 0x13e3, 0x13a7, 0x13d4, 0x13e3, 0x13ef, 0x13cb, 0x13b3, 0x13e4,
0x13b7, 0x13cc, 0x13b4, 0x13bb, 0x13a7, 0x13ca, 0x13b5, 0x13a7, 0x13bb, 0x13e3, 0x13a7, 0x13a7,
0x13b3, 0x13e5, 0x13a7, 0x13d8, 0x13b9, 0x13c2, 0x13da, 0x13a7, 0x13c2, 0x13d7, 0x13a6, 0x13aa,
0x13b5, 0x13a7, 0x13d4, 0x13dd, 0x13cd, 0x13c3, 0x13ef, 0x13cd, 0x13a7, 0x13ab, 0x13e9, 0x13b3,
0x20, 0x13b8, 0x13bb, 0x13cb, 0x13ab, 0x13e5, 0x13c2, 0x13a6, 0x13ab, 0x13ea, 0x13d8, 0x13b9,
0x13a7, 0x13a4, 0x13b9, 0x13a6, 0x13d3, 0x13c2, 0x13b9, 0x13a7, 0x13cc, 0x13b5, 0x13b9, 0x13c2,
0x13b3, 0x13bd, 0x13cd, 0x13a6, 0x13d8, 0x13c2, 0x13aa, 0x13cf, 0x13ef, 0x13c3, 0x13ec, 0x13ab,
0x13c1, 0x13d6, 0x13cd, 0x13a7, 0x13c3, 0x13ec, 0x13cf, 0x13c8, 0x13cd, 0x13a7, 0x13a3, 0x13bb,
0x13cd, 0x13a7, 0x13a3, 0x13b3, 0x13b5, 0x13bf, 0x13bb, 0x20, 0x13c7, 0x13c2, 0x13c6, 0x13c2,
0x13d8, 0x13a0, 0x13be, 0x13a9, 0x13c8, 0x13ef, 0x13c2, 0x13a9, 0x13f0, 0x13c2, 0x13a9, 0x13a7,
0x13d4, 0x13b5, 0x13a7, 0x13cd, 0x13d4, 0x13c1, 0x13a9, 0x13cf, 0x13b6, 0x13b3, 0x13d3, 0x13b5,
0x13ef, 0x13d7, 0x13cc, 0x13a7, 0x13b5, 0x13c2, 0x13cc, 0x13b9, 0x13a7, 0x13c2, 0x13d7, 0x13d0,
0x13b5, 0x13ce, 0x13c2, 0x13a6, 0x13ad, 0x13f1, 0x13cf, 0x13c2, 0x13a6, 0x13c9, 0x13b5, 0x13cd,
0x13b4, 0x13d7, 0x13c1, 0x13aa, 0x13b5, 0x13bb, 0x13cd, 0x13a7, 0x13d4, 0x13f1, 0x13c7, 0x13d4,
0x13cf, 0x13a8, 0x13c2, 0x13d8, 0x13d8, 0x13c8, 0x13b5, 0x13cf, 0x13d6, 0x13b3, 0x13c2, 0x13de,
0x13bb, 0x13e1, 0x13d9, 0x13a9, 0x13f2, 0x13d9, 0x13bb, 0x13cd, 0x13a7, 0x13a4, 0x13b3, 0x13c2,
0x13c6, 0x13d4, 0x13a4, 0x13b7, 0x13bb, 0x13a9, 0x13a4, 0x13cd, 0x13d4, 0x2d, 0x13c1, 0x13b3,
0x13eb, 0x13f0, 0x13c2, 0x13d8, 0x13ef, 0x13c1, 0x13ed, 0x13b3, 0x13d7, 0x13ec, 0x13cd, 0x13d9,
0x13a9, 0x13ef, 0x13ab, 0x13e5, 0x13a7, 0x13b3, 0x13c2, 0x13ab, 0x13c2, 0x13f0, 0x13a7, 0x13d6,
0x13b5, 0x13c2, 0x13ca, 0x13a6, 0x13f0, 0x13b4, 0x13ea, 0x13c2, 0x13a0, 0x13d0, 0x13b4, 0x13cf,
0x13c6, 0x13ca, 0x13d3, 0x13e5, 0x13cd, 0x13c6, 0x13a2, 0x13ac, 0x13be, 0x13d5, 0x13be, 0x20,
0x13a2, 0x13e4, 0x13f3, 0x13cd, 0x13d7, 0x13ea, 0x13b6, 0x13b9, 0x13d5, 0x13b3, 0x13b4, 0x13e3,
0x13eb, 0x13a9, 0x13e7, 0x13a6, 0x13be, 0x13ee, 0x20, 0x13e3, 0x13a0, 0x13e5, 0x13a2, 0x13a4,
0x13d3, 0x13c5, 0x13d8, 0x20, 0x13ae, 0x13b5, 0x13be, 0x13cd, 0x13d5, 0x13c2, 0x13b5, 0x13a1,
0x13d5, 0x13b4, 0x13d7, 0x13c7, 0x13cd, 0x13c7, 0x13c2, 0x13a4, 0x13f2, 0x13e8, 0x13af, 0x20,
0x13a6, 0x13da, 0x13cf, 0x13d3, 0x13e9, 0x13c2, 0x13f3, 0x13dd, 0x13b0, 0x13c6, 0x13d8, 0x13b4,
0x13be, 0x13d5, 0x13b9, 0x13c2, 0x13a4, 0x13ac, 0x13eb, 0x13f3, 0x13af, 0x20, 0x13ad, 0x13eb,
0x13ba, 0x13b5, 0x13c9, 0x13c1, 0x13c7, 0x13b5, 0x13dd, 0x13cf, 0x13d7, 0x13c2, 0x13a0, 0x13bb,
0x13cd, 0x13d5, 0x13d3, 0x13bb, 0x13a0, 0x13c2, 0x13d9, 0x13e9, 0x13a0, 0x13dd, 0x13a7, 0x13c2,
0x13a0, 0x13d6, 0x13c2, 0x13cf, 0x13c7, 0x13b5, 0x13c7, 0x13d7, 0x13c7, 0x13b5, 0x13c2, 0x13c6,
0x13d8, 0x13cd, 0x13b3, 0x13e9, 0x13cb, 0x13ce, 0x13b5, 0x13cf, 0x13c7, 0x13e3, 0x13b4, 0x13cd,
0x13d7, 0x13ca, 0x13d3, 0x13c7, 0x13cd, 0x13d7, 0x13ca, 0x13cf, 0x13c2, 0x13a8, 0x13c2, 0x13e5,
0x13cf, 0x13be, 0x13eb, 0x13aa, 0x13c7, 0x13c2, 0x13ae, 0x13a8, 0x13c2, 0x13db, 0x13b5, 0x13c2,
0x13e5, 0x13c6, 0x13b5, 0x13d3, 0x13ac, 0x13c2, 0x13cf, 0x13ae, 0x13b5, 0x13cf, 0x13c2, 0x13a9,
0x13a4, 0x13cd, 0x13d7, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x20, 0x13be, 0x13bf, 0x20, 0x13a0,
0x13cd, 0x13a6, 0x13ef, 0x13a2, 0x13cd, 0x13d4, 0x13c2, 0x13ca, 0x13b5, 0x13e8, 0x13b5, 0x13cf,
0x13a7, 0x13b5, 0x13c2, 0x13c6, 0x13d7, 0x13a9, 0x13b6, 0x13e9, 0x13a9, 0x13eb, 0x13b5, 0x13cf,
0x13c9, 0x13c2, 0x13e7, 0x13e3, 0x13be, 0x13b8, 0x13c2, 0x13d3, 0x13c2, 0x13b8, 0x13a7, 0x13ce,
0x13cb, 0x13a9, 0x13b9, 0x13df, 0x13d7, 0x13b9, 0x13b5, 0x13d4, 0x13ba, 0x13b5, 0x13ad, 0x13bb,
0x13ba, 0x13c2, 0x13cd, 0x13a9, 0x13bc, 0x13be, 0x13aa, 0x13b9, 0x13cd, 0x13a6, 0x13eb, 0x13a3,
0x13cf, 0x13b6, 0x13c7, 0x13eb, 0x13cf, 0x13c9, 0x13aa, 0x13b5, 0x13a7, 0x13cc, 0x13b9, 0x13b3,
0x13b9, 0x13b5, 0x13c3, 0x13cc, 0x13b3, 0x13e4, 0x13ec, 0x13cc, 0x13c6, 0x13d9, 0x13a5, 0x13cf,
0x13bb, 0x13ea, 0x13b6, 0x13c9, 0x13b5, 0x13cd, 0x13aa, 0x13e4, 0x13d0, 0x13df, 0x13a0, 0x13cd,
0x13d3, 0x13a9, 0x13b0, 0x13bb, 0x13d4, 0x13b5, 0x13c2, 0x13d8, 0x13b3, 0x13be, 0x13a4, 0x13b5,
0x13ef, 0x13c3, 0x13ec, 0x13cd, 0x13a7, 0x13e9, 0x13da, 0x13cf, 0x13a0, 0x13e5, 0x13b3, 0x13c1,
0x13e0, 0x13eb, 0x13f0, 0x13be, 0x13eb, 0x13b5, 0x13c2, 0x13f4, 0x13cf, 0x13ec, 0x13b6, 0x13aa,
0x13dd, 0x13d7, 0x13d3, 0x13bf, 0x13e9, 0x20, 0x13a4, 0x13aa, 0x13b2, 0x13a9, 0x13cc, 0x13c7,
0x13c8, 0x13d1, 0x13b5, 0x13e5, 0x13a0, 0x13c2, 0x13d4, 0x13be, 0x13be, 0x13b5, 0x13ec, 0x13e3,
0x13aa, 0x13cf, 0x13d3, 0x13c2, 0x13cd, 0x13d3, 0x13f2, 0x13af, 0x13b2, 0x13aa, 0x13aa, 0x13cd,
0x13aa, 0x13bc, 0x13b3, 0x13ac, 0x13b5, 0x13ab, 0x13f0, 0x13b4, 0x13c2, 0x13b9, 0x13ae, 0x13b9,
0x13b5, 0x13d7, 0x13eb, 0x13cd, 0x13bc, 0x13b5, 0x13cf, 0x13a5, 0x13cd, 0x13ba, 0x13ef, 0x13d6,
0x13b4, 0x13f3, 0x13c2, 0x13a0, 0x13c2, 0x13a0, 0x13c8, 0x13a0, 0x13a0, 0x13a6, 0x13b3, 0x13be,
0x13d7, 0x13ca, 0x13a8, 0x13c2, 0x13eb, 0x13b5, 0x13e3, 0x13dd, 0x13bb, 0x13e7, 0x13a9, 0x13e5,
0x13cc, 0x13d5, 0x13b4, 0x13af, 0x13cc, 0x13c5, 0x13a1, 0x13e9, 0x13d6, 0x13e9, 0x13a7, 0x13a3,
0x13ec, 0x13eb, 0x13e5, 0x13e1, 0x13be, 0x13e1, 0x13d8, 0x13a1, 0x13c6, 0x20, 0x13d3, 0x13a6,
0x13cf, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x13a6, 0x13bb, 0x13c7, 0x13b5, 0x13c6,
0x13d3, 0x13b5, 0x13a7, 0x13be, 0x13b5, 0x13c6, 0x13bb, 0x13ad, 0x13c3, 0x13b7, 0x13b7, 0x13a1,
0x13c2, 0x13c7, 0x13b5, 0x13a9, 0x13b5, 0x13d8, 0x13b9, 0x13d8, 0x13aa, 0x13cd, 0x13b4, 0x13c6,
0x13e3, 0x13b4, 0x13a2, 0x13c2, 0x13b9, 0x13e7, 0x13b6, 0x13b9, 0x13b5, 0x13a8, 0x13cc, 0x13cf,
0x13a0, 0x13f0, 0x13df, 0x13f4, 0x13da, 0x13c3, 0x13a4, 0x13b7, 0x13c3, 0x13b5, 0x13ec, 0x13b5,
0x13a9, 0x13c3, 0x13a4, 0x13ba, 0x13a0, 0x13c6, 0x13aa, 0x20, 0x13c6, 0x13aa, 0x13c6, 0x13b4,
0x13a0, 0x13eb, 0x13c8, 0x13a7, 0x13b5, 0x13c2, 0x13a4, 0x13c9, 0x13c2, 0x13c7, 0x13e5, 0x13f3,
0x13d7, 0x13d4, 0x13b3, 0x13d7, 0x13cd, 0x13d7, 0x20, 0x13bc, 0x13b5, 0x13cd, 0x13c8, 0x13b3,
0x13b6, 0x13d9, 0x13c2, 0x13a6, 0x13cc, 0x13f1, 0x13c6, 0x13c2, 0x13d4, 0x13af, 0x13d8, 0x13d4,
0x13b3, 0x13e9, 0x13d9, 0x13be, 0x13a6, 0x13d4, 0x13ca, 0x13a4, 0x13f0, 0x13e8, 0x13e9, 0x13b5,
0x13cd, 0x4e9a, 0x7684, 0x65af, 0x4e9a, 0x8d1d, 0x5df4, 0x963f, 0x65af, 0x9a6c, 0x62c9, 0x73ed,
0x73e0, 0x5c14, 0x5e03, 0x743c, 0x5e03, 0x62c9, 0x79d1, 0x7eb3, 0x514b, 0x91cc, 0x8fbe, 0x7d2f,
0x65af, 0x8428, 0x62c9, 0x59c6, 0x5409, 0x5e03, 0x63d0, 0x5f17, 0x91cc, 0x6566, 0x54c8, 0x535a,
0x7f57, 0x5185, 0x7ea6, 0x7ff0, 0x5185, 0x65af, 0x5821, 0x91d1, 0x6c99, 0x8428, 0x62c9, 0x5404,
0x65af, 0x5229, 0x4f2f, 0x7ef4, 0x5c14, 0x5362, 0x8428, 0x5361, 0x9a6c, 0x585e, 0x5362, 0x59c6,
0x5df4, 0x5df4, 0x7eb3, 0x6469, 0x52a0, 0x8fea, 0x6c99, 0x5185, 0x7f57, 0x6bd5, 0x52aa, 0x74e6,
0x514b, 0x8096, 0x7279, 0x6ce2, 0x591a, 0x8bfa, 0x4f0f, 0x57c3, 0x8fbe, 0x514b, 0x5b89, 0x514b,
0x96f7, 0x5947, 0x5b89, 0x572d, 0x62c9, 0x5b89, 0x63d0, 0x74dc, 0x963f, 0x62c9, 0x74dc, 0x4f0a,
0x7eb3, 0x79d1, 0x5c14, 0x591a, 0x74e6, 0x62c9, 0x91cc, 0x5965, 0x54c8, 0x5723, 0x8def, 0x6613,
0x65af, 0x56fe, 0x5e93, 0x66fc, 0x963f, 0x9c81, 0x5df4, 0x5df4, 0x5df4, 0x591a, 0x65af, 0x4f2f,
0x5229, 0x5179, 0x5e03, 0x5170, 0x514b, 0x8428, 0x5e03, 0x9686, 0x535a, 0x963f, 0x7ef4, 0x65af,
0x5854, 0x535a, 0x4f0a, 0x897f, 0x5927, 0x574e, 0x666e, 0x52a0, 0x62c9, 0x52a0, 0x65af, 0x5361,
0x5bb4, 0x5947, 0x74e6, 0x74e6, 0x54e5, 0x65af, 0x8fbe, 0x9ece, 0x52a0, 0x5e93, 0x4e9a, 0x5df4,
0x591a, 0x7c73, 0x5c3c, 0x52a0, 0x57c3, 0x5fb7, 0x8499, 0x987f, 0x4f9d, 0x4f26, 0x5c3c, 0x8d1d,
0x798f, 0x5854, 0x96f7, 0x8428, 0x683c, 0x83b1, 0x65af, 0x8d1d, 0x53e4, 0x65af, 0x6e7e, 0x683c,
0x6797, 0x7eb3, 0x8fbe, 0x74dc, 0x5fb7, 0x7f57, 0x666e, 0x5371, 0x5730, 0x9a6c, 0x62c9, 0x572d,
0x4e9a, 0x90a3, 0x54c8, 0x5229, 0x6cd5, 0x514b, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x6ce2,
0x5229, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde, 0x8bfa, 0x514b, 0x65af, 0x5370, 0x7b2c,
0x5b89, 0x7eb3, 0x5dde, 0x9a6c, 0x4f26, 0x6208, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde, 0x5f7c,
0x5f97, 0x65af, 0x5821, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde, 0x7279, 0x5c14, 0x57ce, 0x5370,
0x7b2c, 0x5b89, 0x7eb3, 0x5dde, 0x7ef4, 0x7ef4, 0x5e02, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde,
0x6e29, 0x68ee, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7eb3, 0x5dde, 0x5a01, 0x7eb3, 0x9a6c, 0x514b,
0x4f0a, 0x52aa, 0x7ef4, 0x514b, 0x80af, 0x5854, 0x57fa, 0x5dde, 0x8499, 0x8482, 0x585e, 0x6d1b,
0x9a6c, 0x585e, 0x7ea6, 0x9a6c, 0x90a3, 0x74dc, 0x9a6c, 0x7459, 0x65af, 0x9a6c, 0x63d0, 0x5c3c,
0x514b, 0x6885, 0x8bfa, 0x7c73, 0x5c3c, 0x58a8, 0x897f, 0x54e5, 0x57ce, 0x5bc6, 0x514b, 0x9686,
0x8499, 0x7279, 0x96f7, 0x8499, 0x5f97, 0x7ef4, 0x7684, 0x4e9a, 0x8499, 0x7279, 0x585e, 0x62c9,
0x7279, 0x62ff, 0x9a9a, 0x6d1b, 0x7f57, 0x5c3c, 0x4e9a, 0x5317, 0x8fbe, 0x79d1, 0x4ed6, 0x5dde,
0x6bd4, 0x5c24, 0x62c9, 0x5317, 0x8fbe, 0x79d1, 0x4ed6, 0x5dde, 0x7533, 0x7279, 0x5317, 0x8fbe,
0x79d1, 0x4ed6, 0x5dde, 0x65b0, 0x585e, 0x52d2, 0x59c6, 0x5e15, 0x62c9, 0x9a6c, 0x91cc, 0x535a,
0x6ce2, 0x591a, 0x97e6, 0x67f3, 0x7d2f, 0x897f, 0x8153, 0x96f7, 0x7d22, 0x5362, 0x7279, 0x91cc,
0x5965, 0x5e03, 0x90ce, 0x5e93, 0x5723, 0x5730, 0x4e9a, 0x54e5, 0x5723, 0x591a, 0x660e, 0x5404,
0x65af, 0x79d1, 0x5217, 0x65af, 0x6bd4, 0x6851, 0x5fb7, 0x5723, 0x5df4, 0x6cf0, 0x52d2, 0x7c73,
0x5c9b, 0x5723, 0x7ea6, 0x7ff0, 0x65af, 0x5723, 0x5362, 0x897f, 0x4e9a, 0x5723, 0x6587, 0x68ee,
0x7279, 0x7279, 0x53e4, 0x897f, 0x52a0, 0x5c14, 0x5df4, 0x56fe, 0x52d2, 0x8482, 0x534e, 0x7eb3,
0x4e9a, 0x5e93, 0x5854, 0x7279, 0x5361, 0x585e, 0x8fea, 0x8499, 0xb7, 0x8fea, 0x7ef4, 0x5c14,
0x9ea6, 0x683c, 0x7406, 0x5e15, 0x5c14, 0x9ed8, 0x7279, 0x7f57, 0x5c14, 0x6717, 0x4f0a, 0x5c14,
0x57ce, 0x963f, 0x7eb3, 0x5fb7, 0x5c14, 0x963f, 0x514b, 0x6258, 0x522b, 0x963f, 0x4ec0, 0x54c8,
0x5df4, 0x5fb7, 0x6587, 0x83b1, 0x79d1, 0x4f26, 0x5761, 0x8fea, 0x62dc, 0x675c, 0x5c1a, 0x522b,
0x52a0, 0x6c99, 0x5e0c, 0x4f2f, 0x4f26, 0x4f0a, 0x5c14, 0x5e93, 0x8328, 0x514b, 0x67e5, 0x4e9a,
0x666e, 0x62c9, 0x5361, 0x62c9, 0x5947, 0x6c49, 0x5fb7, 0x52a0, 0x671b, 0x52a0, 0x9521, 0x9a6c,
0x65af, 0x5580, 0x7279, 0x5c3c, 0x79d1, 0x897f, 0x4e9a, 0x5361, 0x5854, 0x5c14, 0x5e93, 0x65af,
0x5854, 0x5948, 0x514b, 0x5b5c, 0x6d1b, 0x5c14, 0x8fbe, 0x5229, 0x96c5, 0x5f97, 0x8428, 0x54c8,
0x6797, 0x4e07, 0x8c61, 0x7b26, 0x62c9, 0x8fea, 0x6c83, 0x65af, 0x6258, 0x514b, 0x96c5, 0x5e93,
0x8328, 0x514b, 0x53f6, 0x5361, 0x6377, 0x7433, 0x5821, 0x57c3, 0x91cc, 0x6e29, 0x767e, 0x6155,
0x5927, 0x52a0, 0x90a3, 0x5229, 0x4f5b, 0x5f97, 0x89d2, 0x9a6c, 0x5fb7, 0x62c9, 0x96f7, 0x514b,
0x96c5, 0x672a, 0x514b, 0x65af, 0x5766, 0x5229, 0x963f, 0x5fb7, 0x83b1, 0x5fb7, 0x5e03, 0x91cc,
0x65af, 0x73ed, 0x970d, 0x5df4, 0x7279, 0x73c0, 0x65af, 0x6089, 0x5c3c, 0x8d1d, 0x5c14, 0x683c,
0x83b1, 0x5fb7, 0x5e03, 0x62c9, 0x8fea, 0x65af, 0x62c9, 0x53d1, 0x57fa, 0x5e0c, 0x8bb7, 0x4e4c,
0x6839, 0x897f, 0x5c9b, 0x9a6c, 0x6069, 0x5c9b, 0x4f0a, 0x65af, 0x5766, 0x5e03, 0x5c14, 0x5362,
0x5e03, 0x5c14, 0x96c5, 0x90a3, 0x9a6c, 0x8033, 0x4ed6, 0x6ce2, 0x5fb7, 0x6208, 0x91cc, 0x5bdf,
0x8428, 0x9a6c, 0x62c9, 0x5723, 0x9a6c, 0x529b, 0x8bfa, 0x8428, 0x62c9, 0x70ed, 0x7a9d, 0x8f9b,
0x83f2, 0x7f57, 0x6ce2, 0x5c14, 0x65af, 0x79d1, 0x666e, 0x91cc, 0x7d22, 0x975e, 0x4e9a, 0x74e6,
0x675c, 0x5179, 0x8428, 0x683c, 0x52d2, 0x5e03, 0x53ef, 0x53ef, 0x65af, 0x9a6c, 0x5c14, 0x4ee3,
0x592b, 0x6bdb, 0x91cc, 0x6c42, 0x65af, 0x7559, 0x5c3c, 0x6c6a, 0x67e5, 0x5854, 0x59c6, 0x7279,
0x9c81, 0x514b, 0x7fa4, 0x5c9b, 0x590d, 0x6d3b, 0x8282, 0x5c9b, 0x5bcc, 0x7eb3, 0x5bcc, 0x63d0,
0x52a0, 0x62c9, 0x5e15, 0x6208, 0x65af, 0x74dc, 0x8fbe, 0x5c14, 0x5361, 0x7eb3, 0x5c14, 0x6069,
0x5fb7, 0x4f2f, 0x91cc, 0x5e93, 0x8d5b, 0x57c3, 0x5938, 0x8d3e, 0x6797, 0x9a6c, 0x6731, 0x7f57,
0x9a6c, 0x514b, 0x8428, 0x65af, 0x7459, 0x9c81, 0x52aa, 0x7f8e, 0x963f, 0x5e15, 0x679c, 0x5e15,
0x679c, 0x5e15, 0x52b3, 0x76ae, 0x7279, 0x51ef, 0x6069, 0x6ce2, 0x7eb3, 0x4f69, 0x5c9b, 0x83ab,
0x5c14, 0x5179, 0x6bd4, 0x6e2f, 0x5854, 0x5e0c, 0x63d0, 0x65af, 0x8003, 0x65af, 0x4f2f, 0x677e,
0x5fb7, 0x963f, 0x6bd4, 0x8d0a, 0x963f, 0x65af, 0x99ac, 0x62c9, 0x73ed, 0x57fa, 0x79d1, 0x7d0d,
0x514b, 0x91cc, 0x9054, 0x7d2f, 0x65af, 0x85a9, 0x62c9, 0x59c6, 0x7d04, 0x7ff0, 0x5167, 0x65af,
0x5821, 0x91d1, 0x6c99, 0x85a9, 0x5229, 0x5e03, 0x7dad, 0x76e7, 0x5b89, 0x9054, 0x76e7, 0x85a9,
0x5361, 0x99ac, 0x585e, 0x9b6f, 0x5167, 0x7f85, 0x7562, 0x6eab, 0x7279, 0x548c, 0x514b, 0x57c3,
0x9054, 0x514b, 0x963f, 0x62c9, 0x74dc, 0x4f0a, 0x7d0d, 0x79d1, 0x723e, 0x591a, 0x74e6, 0x62c9,
0x91cc, 0x5967, 0x54c8, 0x963f, 0x9b6f, 0x5df4, 0x963f, 0x677e, 0x68ee, 0x5df4, 0x5e0c, 0x96c5,
0x5df4, 0x4f0a, 0x4e9e, 0x5fb7, 0x73ed, 0x5fb7, 0x62c9, 0x65af, 0x4f2f, 0x5229, 0x8332, 0x535a,
0x963f, 0x7dad, 0x65af, 0x5854, 0x829d, 0x5a03, 0x5a03, 0x54e5, 0x65af, 0x9054, 0x9ece, 0x52a0,
0x5eab, 0x4e9e, 0x5df4, 0x9053, 0x68ee, 0x7063, 0x611b, 0x6c11, 0x9813, 0x798f, 0x5854, 0x840a,
0x85a9, 0x683c, 0x840a, 0x65af, 0x7063, 0x683c, 0x6797, 0x7d0d, 0x9054, 0x74dc, 0x5fb7, 0x7f85,
0x666e, 0x5371, 0x5730, 0x99ac, 0x62c9, 0x572d, 0x4e9e, 0x90a3, 0x590f, 0x7063, 0x62ff, 0x5370,
0x7b2c, 0x5b89, 0x7d0d, 0x6ce2, 0x5229, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x5dde, 0x8afe,
0x514b, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x5dde, 0x99ac, 0x502b, 0x54e5, 0x5370, 0x7b2c,
0x5b89, 0x7d0d, 0x5dde, 0x5f7c, 0x5f97, 0x5821, 0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x5dde, 0x7279,
0x723e, 0x57ce, 0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x5dde, 0x97cb, 0x97cb, 0x5370, 0x7b2c, 0x5b89,
0x7d0d, 0x5dde, 0x6eab, 0x68ee, 0x65af, 0x5370, 0x7b2c, 0x5b89, 0x7d0d, 0x5dde, 0x5a01, 0x7d0d,
0x99ac, 0x514b, 0x4f0a, 0x52aa, 0x7dad, 0x514b, 0x8def, 0x6613, 0x7dad, 0x723e, 0x99ac, 0x585e,
0x7d04, 0x99ac, 0x90a3, 0x74dc, 0x99ac, 0x63d0, 0x5c3c, 0x514b, 0x6885, 0x8afe, 0x7c73, 0x5c3c,
0x8499, 0x5f97, 0x7dad, 0x7684, 0x4e9e, 0x8499, 0x585e, 0x62c9, 0x7279, 0x5cf6, 0x62ff, 0x9a37,
0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x5317, 0x9054, 0x79d1, 0x4ed6, 0x5dde, 0x6bd4, 0x5c24, 0x62c9,
0x5317, 0x9054, 0x79d1, 0x4ed6, 0x5dde, 0x4e2d, 0x5fc3, 0x5e02, 0x5317, 0x9054, 0x79d1, 0x4ed6,
0x5dde, 0x65b0, 0x85a9, 0x52d2, 0x59c6, 0x97cb, 0x67f3, 0x6e2f, 0x91cc, 0x7d04, 0x5e03, 0x862d,
0x79d1, 0x8056, 0x5730, 0x4e9e, 0x54e5, 0x8056, 0x591a, 0x660e, 0x5404, 0x8056, 0x5df4, 0x6cf0,
0x52d2, 0x7c73, 0x8056, 0x7d04, 0x7ff0, 0x65af, 0x8056, 0x76e7, 0x897f, 0x4e9e, 0x8056, 0x6587,
0x68ee, 0x7279, 0x7279, 0x53e4, 0x897f, 0x52a0, 0x723e, 0x5df4, 0x5716, 0x52d2, 0x8482, 0x83ef,
0x7d0d, 0x767d, 0x99ac, 0x5e02, 0x4e9e, 0x5eab, 0x5854, 0x7279, 0x51f1, 0x897f, 0x7ad9, 0x6234,
0x7dad, 0x65af, 0x7ad9, 0x675c, 0x8499, 0x8fea, 0x7dad, 0x723e, 0x7ad9, 0x9ea5, 0x5938, 0x91cc,
0x83ab, 0x68ee, 0x7ad9, 0x9ea5, 0x514b, 0x9ed8, 0x591a, 0x7ad9, 0x5e15, 0x723e, 0x9ed8, 0x7f85,
0x745f, 0x62c9, 0x7ad9, 0x662d, 0x548c, 0x7ad9, 0x7279, 0x7f85, 0x723e, 0x7ad9, 0x6717, 0x4f0a,
0x723e, 0x57ce, 0x963f, 0x7d0d, 0x5fb7, 0x723e, 0x6bd4, 0x65af, 0x51f1, 0x514b, 0x79d1, 0x502b,
0x5761, 0x675c, 0x5c1a, 0x5225, 0x5e0c, 0x4f2f, 0x4f96, 0x4f0a, 0x723e, 0x5eab, 0x8328, 0x514b,
0x67e5, 0x96c5, 0x666e, 0x62c9, 0x932b, 0x6c5f, 0x99ac, 0x65af, 0x5580, 0x7279, 0x5c3c, 0x79d1,
0x897f, 0x4e9e, 0x5361, 0x5854, 0x723e, 0x96c5, 0x5eab, 0x8328, 0x514b, 0x8449, 0x5361, 0x6377,
0x7433, 0x5821, 0x99ac, 0x5fb7, 0x62c9, 0x5cf6, 0x53f2, 0x4e39, 0x5229, 0x963f, 0x5fb7, 0x840a,
0x5fb7, 0x5e03, 0x91cc, 0x65af, 0x672c, 0x8377, 0x4f2f, 0x7279, 0x8c9d, 0x723e, 0x683c, 0x840a,
0x5fb7, 0x4f2f, 0x62c9, 0x7b2c, 0x65af, 0x62c9, 0x74e6, 0x57fa, 0x5e0c, 0x8a25, 0x70cf, 0x6839,
0x897f, 0x5cf6, 0x99ac, 0x6069, 0x5cf6, 0x76e7, 0x5e03, 0x723e, 0x96c5, 0x90a3, 0x99ac, 0x8033,
0x4ed6, 0x85a9, 0x62c9, 0x71b1, 0x7aa9, 0x83ef, 0x675c, 0x8332, 0x85a9, 0x683c, 0x52d2, 0x5e03,
0x67e5, 0x6208, 0x65af, 0x7fa4, 0x5cf6, 0x53ef, 0x53ef, 0x65af, 0x7fa4, 0x5cf6, 0x51f1, 0x723e,
0x84cb, 0x6717, 0x7fa4, 0x5cf6, 0x99ac, 0x723e, 0x4ee3, 0x592b, 0x6bdb, 0x91cc, 0x88d8, 0x65af,
0x5e03, 0x5e72, 0x7dad, 0x723e, 0x5cf6, 0x6cd5, 0x8003, 0x798f, 0x74b0, 0x7901, 0x74dc, 0x9054,
0x723e, 0x5361, 0x7d0d, 0x723e, 0x5cf6, 0x6069, 0x5fb7, 0x4f2f, 0x91cc, 0x5cf6, 0x79d1, 0x65af,
0x96f7, 0x74dc, 0x52a0, 0x6797, 0x74b0, 0x7901, 0x99ac, 0x4e45, 0x7f85, 0x99ac, 0x514b, 0x85a9,
0x65af, 0x7fa4, 0x5cf6, 0x7459, 0x9b6f, 0x76ae, 0x7279, 0x5eb7, 0x7fa4, 0x5cf6, 0x6e6f, 0x52a0,
0x5854, 0x5e03, 0x5cf6, 0x5a01, 0x514b, 0x5cf6, 0x5b89, 0x594e, 0x62c9, 0x8377, 0x5c6c, 0x963f,
0x9b6f, 0x5df4, 0x591a, 0x7c73, 0x5c3c, 0x514b, 0x5eab, 0x65af, 0x5854, 0x5948, 0x57fa, 0x897f,
0x7d10, 0x5b89, 0x5854, 0x90a3, 0x90a3, 0x5229, 0x5f17, 0x430, 0x486, 0x43b, 0x43c, 0x430,
0x442, 0x44b, 0x300, 0x430, 0x486, 0x43d, 0x430, 0x301, 0x434, 0x44b, 0x440, 0x44c,
0x430, 0x486, 0x43a, 0x442, 0x430, 0x301, 0xa64b, 0x430, 0x486, 0x43a, 0x442, 0x44e,
0x301, 0x431, 0x438, 0x43d, 0x441, 0x43a, 0x44a, 0x431, 0x438, 0x448, 0x43a, 0x435,
0x301, 0x43a, 0x44a, 0x447, 0x438, 0x442, 0x430, 0x300, 0x438, 0x486, 0x440, 0x43a,
0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x44a, 0x43f, 0x435, 0x442, 0x440, 0x43e, 0x43f,
0x430, 0x301, 0x475, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x44a, 0x5f, 0x43a, 0x430,
0x43c, 0x447, 0x430, 0x301, 0x442, 0x441, 0x43a, 0x457, 0x439, 0x445, 0x430, 0x301,
0x43d, 0x434, 0x44b, 0x433, 0x430, 0x43a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x467,
0x301, 0x440, 0x441, 0x43a, 0x44a, 0x43c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x301,
0x43d, 0x44a, 0x43d, 0x43e, 0x432, 0x43e, 0x43a, 0xa64b, 0x437, 0x43d, 0x435, 0x301,
0x446, 0x43a, 0x44a, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x301,
0x440, 0x441, 0x43a, 0x44a, 0x47b, 0x486, 0x301, 0x43c, 0x441, 0x43a, 0x44a, 0x1c82,
0x443, 0x486, 0x440, 0x430, 0x301, 0x43b, 0x44c, 0x441, 0x43a, 0x44a, 0x43a, 0x44b,
0x437, 0x44b, 0x43b, 0x47b, 0x440, 0x434, 0x430, 0x300, 0x441, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x301, 0x43d, 0x44a, 0x441, 0x440, 0x435, 0x434, 0x43d, 0x435, 0x43a,
0x43e, 0x43b, 0x44b, 0x301, 0x43c, 0x441, 0x43a, 0x44a, 0x1c82, 0x443, 0x486, 0x301,
0x441, 0x442, 0x44c, 0x5f, 0x43d, 0x435, 0x301, 0x440, 0x430, 0x432, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x301, 0x43a, 0x44a, 0xa657, 0x486,
0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x44a, 0x454, 0x486, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x457, 0x43d, 0x431, 0xa64b, 0x301, 0x440, 0x433, 0x44a, 0x43a, 0x430,
0x43b, 0x438, 0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x301, 0x434, 0x44a, 0x43a,
0x456, 0x301, 0x435, 0x432, 0x44a, 0x43c, 0x438, 0x301, 0x43d, 0x441, 0x43a, 0x44a,
0x43c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x300, 0x441, 0x430, 0x43c, 0x430, 0x301,
0x440, 0x430, 0x441, 0x477, 0x43c, 0x444, 0x435, 0x440, 0xa64b, 0x301, 0x43f, 0x43e,
0x43b, 0x44c, 0x432, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x301, 0x434,
0x44a, 0x410, 0x441, 0x43c, 0x44d, 0x440, 0x430, 0x421, 0x430, 0x43d, 0x2d, 0x422,
0x43e, 0x43c, 0x435, 0x20, 0x421, 0x430, 0x43d, 0x2d, 0x422, 0x43e, 0x43c, 0x435,
0x421, 0x430, 0x43d, 0x2d, 0x41b, 0x443, 0x438, 0x441, 0x20, 0x421, 0x430, 0x43d,
0x2d, 0x41b, 0x443, 0x438, 0x441, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x422, 0x435, 0x43b,
0x43b, 0x2d, 0x421, 0x438, 0x442, 0x438, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438,
0x430, 0x43d, 0x430, 0x412, 0x435, 0x432, 0x435, 0x439, 0x2c, 0x20, 0x418, 0x43d,
0x434, 0x438, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d, 0x441, 0x435, 0x43d, 0x43d,
0x435, 0x441, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x423,
0x438, 0x43d, 0x430, 0x43c, 0x430, 0x43a, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438,
0x430, 0x43d, 0x430, 0x41b, 0x43e, 0x443, 0x44d, 0x440, 0x2d, 0x41f, 0x440, 0x438,
0x43d, 0x441, 0x2d, 0x41a, 0x443, 0x43e, 0x442, 0x435, 0x440, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x44c, 0x44f, 0x411, 0x43e, 0x439, 0x43b, 0x430, 0x2c, 0x20, 0x4aa,
0x443, 0x440, 0x4ab, 0x4d7, 0x440, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430,
0x426, 0x435, 0x43d, 0x442, 0x440, 0x2c, 0x20, 0x4aa, 0x443, 0x440, 0x4ab, 0x4d7,
0x440, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c, 0x44e, 0x2d,
0x421, 0x435, 0x439, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x4aa, 0x443, 0x440, 0x4ab,
0x4d7, 0x440, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x420, 0x435, 0x437,
0x43e, 0x43b, 0x44c, 0x44e, 0x442, 0x421, 0x430, 0x43d, 0x2d, 0x41f, 0x430, 0x443,
0x43b, 0x443, 0x20, 0x421, 0x430, 0x43d, 0x2d, 0x41f, 0x430, 0x443, 0x43b, 0x443,
0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41b, 0x44e, 0x441, 0x438, 0x44f, 0x414, 0x44e,
0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b, 0x44c,
0x20, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x42e, 0x440, 0x432,
0x438, 0x43b, 0x44c, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x443,
0x442, 0x440, 0x430, 0x432, 0x4d7, 0x42f, 0x43d, 0x433, 0x43e, 0x43d, 0x410, 0x437,
0x43e, 0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c,
0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432,
0x4d7, 0x441, 0x435, 0x43c, 0x41a, 0x430, 0x43d, 0x430, 0x440, 0x20, 0x443, 0x442,
0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x424, 0x430, 0x440, 0x435, 0x440,
0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x41a, 0x4d1,
0x43d, 0x442, 0x4d1, 0x440, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x438, 0x421,
0x4d1, 0x432, 0x430, 0x43f, 0x43b, 0x4d1, 0x20, 0x415, 0x43b, 0x435, 0x43d, 0x430,
0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x410, 0x444, 0x438, 0x43d, 0x441,
0x435, 0x43c, 0x41c, 0x44d, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7,
0x41c, 0x443, 0x441, 0x43a, 0x430, 0x432, 0x420, 0x430, 0x448, 0x442, 0x430, 0x432,
0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x41a, 0x43e, 0x43a, 0x43e, 0x441,
0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x41a, 0x43e,
0x43c, 0x43e, 0x440, 0x441, 0x435, 0x43c, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438,
0x432, 0x441, 0x435, 0x43c, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439,
0x41c, 0x430, 0x439, 0x43e, 0x442, 0x442, 0x430, 0x41c, 0x4d1, 0x43d, 0x43a, 0x443,
0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x413, 0x430, 0x43b, 0x430,
0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7,
0x441, 0x435, 0x43c, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435, 0x20, 0x443, 0x442,
0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43c, 0x41c, 0x430, 0x440, 0x43a, 0x438,
0x437, 0x430, 0x441, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435,
0x43c, 0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d, 0x4b, 0x61, 0x69, 0x6a,
0x72, 0x6f, 0x4c, 0x61, 0x61, 0x6a, 0x6f, 0x73, 0x57, 0x69, 0x6e, 0x64,
0x68, 0x75, 0x6b, 0x4b, 0x6f, 0xdf, 0x74, 0x61, 0x72, 0x69, 0x63, 0x6b,
0x61, 0x4a, 0x75, 0x61, 0x79, 0x61, 0x71, 0x75, 0x69, 0x6c, 0x4b, 0x6e,
0x6f, 0x78, 0x20, 0x65, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x20, 0x65, 0x6e, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x50, 0x65, 0x74, 0x65, 0x72,
0x73, 0x62, 0x75, 0x72, 0x67, 0x20, 0x65, 0x6e, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43, 0x69, 0x74,
0x79, 0x20, 0x65, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x56, 0x65, 0x76, 0x61, 0x79, 0x20, 0x65, 0x6e, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x6e, 0x65,
0x73, 0x20, 0x65, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x63, 0x20, 0x65, 0x6e, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x63,
0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x65, 0x6e, 0x20, 0x4b, 0x65, 0x6e, 0x74,
0x75, 0x63, 0x6b, 0x79, 0x53, 0x63, 0x68, 0x74, 0x61, 0x64, 0x74, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x20, 0x65, 0x6e, 0x20, 0x4e, 0x6f, 0x6f, 0x64, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x5a, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x20, 0x65, 0x6e,
0x20, 0x4e, 0x6f, 0x6f, 0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x4e, 0x65, 0x75, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x20, 0x65, 0x6e,
0x20, 0x4e, 0x6f, 0x6f, 0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x5a, 0x69, 0x6e, 0x74, 0x20, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9, 0x6c,
0x65, 0x6d, 0x79, 0x5a, 0x69, 0x6e, 0x74, 0x20, 0x4a, 0x6f, 0x68, 0x6e,
0x2019, 0x73, 0x5a, 0x69, 0x6e, 0x74, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73,
0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75, 0x74, 0x73, 0x63, 0x68,
0x69, 0x69, 0x6a, 0x61, 0x5a, 0x69, 0x6e, 0x74, 0x20, 0x54, 0x68, 0x6f,
0x6d, 0x61, 0x73, 0x5a, 0x69, 0x6e, 0x74, 0x20, 0x56, 0x69, 0x6e, 0x74,
0x73, 0x63, 0x68, 0xe4, 0x6e, 0x74, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74,
0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x53,
0x63, 0x68, 0x74, 0x61, 0x7a, 0x6a, 0x6f, 0x68, 0x6e, 0x53, 0x63, 0x68,
0x6f, 0x6f, 0x77, 0x61, 0x2d, 0x53, 0x63, 0x68, 0x74, 0x61, 0x7a, 0x6a,
0x6f, 0x68, 0x6e, 0x20, 0x6f, 0x70, 0x20, 0x64, 0xe4, 0x20, 0x45, 0x6e,
0x73, 0x65, 0x6c, 0x20, 0x4f, 0x6e, 0x6a, 0x75, 0x6c, 0x41, 0x71, 0x74,
0xf6, 0x62, 0x65, 0x41, 0x73, 0x73, 0x68, 0x67, 0x61, 0x62, 0x61, 0x74,
0x42, 0x69, 0x73, 0x63, 0x68, 0x6b, 0x65, 0x6b, 0x44, 0x75, 0x73, 0x63,
0x68, 0x61, 0x6e, 0x62, 0x65, 0x4a, 0x61, 0x61, 0x73, 0x61, 0x53, 0x69,
0x67, 0x6f, 0x6e, 0x67, 0x20, 0x28, 0x48, 0x6f, 0x2d, 0x54, 0x73, 0x63,
0x68, 0x69, 0x2d, 0x4d, 0x69, 0x6e, 0x68, 0x2d, 0x53, 0x63, 0x68, 0x74,
0x61, 0x64, 0x74, 0x29, 0x4b, 0x61, 0x6d, 0x73, 0x63, 0x68, 0x61, 0x74,
0x74, 0x6b, 0x61, 0x4e, 0x69, 0x6b, 0x6f, 0x73, 0x69, 0x6a, 0x61, 0x50,
0x6e, 0x6f, 0x6d, 0x20, 0x50, 0x65, 0x6e, 0x68, 0x51, 0x79, 0x73, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x53, 0x63, 0x68, 0x61, 0x6e, 0x67, 0x68,
0x61, 0x69, 0x53, 0x69, 0x6e, 0x67, 0x6a, 0x61, 0x70, 0x75, 0x75, 0x72,
0x54, 0x61, 0x73, 0x63, 0x68, 0x6b, 0x65, 0x6e, 0x74, 0x54, 0x69, 0x62,
0x6c, 0x69, 0x73, 0x55, 0x72, 0x72, 0x75, 0x6d, 0x70, 0x74, 0x73, 0x63,
0x68, 0x69, 0x52, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x45, 0x72, 0x69, 0x77,
0x61, 0x6e, 0x41, 0x7a, 0x6f, 0x6f, 0x72, 0x65, 0x4b, 0x61, 0x6e, 0x6e,
0x61, 0x61, 0x72, 0x65, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x46, 0xe4, 0x72, 0xf6, 0x72, 0x5a, 0x69, 0x6e, 0x74, 0x20, 0x48,
0x65, 0x6c, 0x65, 0x6e, 0x61, 0x42, 0x65, 0x6c, 0x6a, 0x72, 0x61, 0x64,
0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x72, 0x65, 0x6e, 0x4a, 0x69,
0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x61, 0x4a, 0xf6, 0xf6, 0x6e, 0x73,
0x65, 0x69, 0x45, 0x6e, 0x73, 0x65, 0x6c, 0x20, 0x4d, 0xe4, 0x6e, 0x49,
0x73, 0x74, 0x61, 0x6d, 0x62, 0x75, 0x6c, 0x4a, 0xf6, 0xf6, 0x73, 0x65,
0x69, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x6e, 0x69, 0x6e, 0x6a, 0x72, 0x61,
0x61, 0x64, 0x4c, 0x75, 0x78, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x73, 0x63,
0x68, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x52, 0x69, 0x69, 0x6a, 0x61,
0x54, 0x69, 0x72, 0x61, 0x61, 0x6e, 0x61, 0x64, 0x65, 0x72, 0x20, 0x56,
0x61, 0x74, 0x69, 0x6b, 0x61, 0x68, 0x6e, 0x57, 0x69, 0x65, 0x6e, 0x57,
0x6f, 0x6c, 0x6a, 0x6f, 0x6a, 0x72, 0x61, 0x64, 0x57, 0x61, 0x72, 0x73,
0x63, 0x68, 0x61, 0x75, 0x53, 0x61, 0x67, 0x72, 0x65, 0x62, 0x5a, 0xfc,
0x72, 0x65, 0x73, 0x63, 0x68, 0x4d, 0x61, 0x6c, 0x6c, 0x64, 0x69, 0x76,
0x65, 0x4a, 0x61, 0x6c, 0x61, 0x70, 0x70, 0x61, 0x6a, 0x6f, 0x73, 0x4a,
0x75, 0x61, 0x6d, 0x48, 0x6f, 0x6e, 0x75, 0x6c, 0x75, 0x6c, 0x75, 0x41,
0x6c, 0x17e, 0x69, 0x72, 0x47, 0x76, 0x61, 0x6a, 0x61, 0x6e, 0x61, 0x43,
0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78,
0x69, 0x63, 0x6f, 0x41, 0x6c, 0x6d, 0x61, 0x20, 0x41, 0x74, 0x61, 0x41,
0x161, 0x67, 0x61, 0x62, 0x61, 0x74, 0x10c, 0x69, 0x74, 0x61, 0x52, 0x61,
0x6e, 0x67, 0x6f, 0x6f, 0x6e, 0x41, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x6f, 0x74, 0x6f, 0x63, 0x69, 0x46, 0x65, 0x72, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x63, 0x69, 0x4a, 0x75, 0x17e, 0x6e,
0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x6a, 0x61, 0x42, 0x72,
0x75, 0x78, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x4f, 0x74, 0x6f, 0x6b, 0x20,
0x4d, 0x61, 0x6e, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x17e, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x6f, 0x74, 0x6f, 0x10d, 0x6a, 0x65, 0x41, 0x62, 0x69,
0x64, 0x17e, 0x61, 0x6e, 0x41, 0x6c, 0x17e, 0xed, 0x72, 0x4b, 0xe1, 0x68,
0x69, 0x72, 0x61, 0x43, 0x68, 0x61, 0x72, 0x74, 0xfa, 0x6d, 0x4d, 0x6f,
0x67, 0x61, 0x64, 0x69, 0x161, 0x75, 0x4e, 0x64, 0x17e, 0x61, 0x6d, 0x65,
0x6e, 0x61, 0x4e, 0x75, 0x61, 0x6b, 0x161, 0x6f, 0x74, 0x74, 0x53, 0x76,
0x61, 0x74, 0xfd, 0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x161, 0x54, 0x72, 0x69,
0x70, 0x6f, 0x6c, 0x69, 0x73, 0x4b, 0x61, 0x6a, 0x6d, 0x61, 0x6e, 0x73,
0x6b, 0xe9, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x42, 0x65,
0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e,
0xed, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0xed, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0xed,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x76, 0x61, 0x74, 0xfd,
0x20, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x11b, 0x6a, 0x53,
0x76, 0x61, 0x74, 0xfd, 0x20, 0x4b, 0x72, 0x79, 0x161, 0x74, 0x6f, 0x66,
0x53, 0x76, 0x61, 0x74, 0xe1, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x65, 0x53,
0x76, 0x61, 0x74, 0xfd, 0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x161, 0x20, 0x28,
0x4b, 0x61, 0x72, 0x69, 0x62, 0x69, 0x6b, 0x29, 0x53, 0x76, 0x61, 0x74,
0xfd, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x63, 0x41, 0x6d, 0x6d,
0xe1, 0x6e, 0x41, 0x161, 0x63, 0x68, 0x61, 0x62, 0x61, 0x64, 0x42, 0x61,
0x67, 0x64, 0xe1, 0x64, 0x42, 0x61, 0x68, 0x72, 0x61, 0x6a, 0x6e, 0x42,
0x65, 0x6a, 0x72, 0xfa, 0x74, 0x44, 0x61, 0x6d, 0x61, 0x161, 0x65, 0x6b,
0x44, 0x68, 0xe1, 0x6b, 0x61, 0x44, 0x75, 0x62, 0x61, 0x6a, 0x48, 0x6f,
0x20, 0x10c, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x6f, 0x76, 0x6f, 0x20, 0x6d,
0x11b, 0x73, 0x74, 0x6f, 0x4a, 0x65, 0x72, 0x75, 0x7a, 0x61, 0x6c, 0xe9,
0x6d, 0x4b, 0xe1, 0x62, 0x75, 0x6c, 0x4b, 0x61, 0x72, 0xe1, 0x10d, 0xed,
0x4b, 0xe1, 0x74, 0x68, 0x6d, 0xe1, 0x6e, 0x64, 0xfa, 0x43, 0x68, 0x61,
0x6e, 0x64, 0x79, 0x67, 0x61, 0x4b, 0x61, 0x6c, 0x6b, 0x61, 0x74, 0x61,
0x4e, 0x69, 0x6b, 0xf3, 0x73, 0x69, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x6b,
0x75, 0x7a, 0x6e, 0x11b, 0x63, 0x6b, 0x55, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x50, 0x68, 0x6e, 0x6f, 0x6d, 0x70, 0x65, 0x6e, 0x68, 0x50, 0x63, 0x68,
0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x52, 0x69, 0x6a, 0xe1,
0x64, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x6f, 0x75,
0x6c, 0x160, 0x61, 0x6e, 0x67, 0x68, 0x61, 0x6a, 0x53, 0x72, 0x65, 0x64,
0x6e, 0x11b, 0x6b, 0x6f, 0x6c, 0x79, 0x6d, 0x73, 0x6b, 0x54, 0x63, 0x68,
0x61, 0x6a, 0x2d, 0x70, 0x65, 0x6a, 0x54, 0x65, 0x68, 0x65, 0x72, 0xe1,
0x6e, 0x54, 0x68, 0x69, 0x6d, 0x62, 0xfa, 0x55, 0x6c, 0xe1, 0x6e, 0x62,
0xe1, 0x74, 0x61, 0x72, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x11b, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0xe9,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x42, 0x65, 0x72, 0x6d,
0x75, 0x64, 0x79, 0x4b, 0x61, 0x6e, 0xe1, 0x72, 0x73, 0x6b, 0xe9, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x4b, 0x61, 0x70, 0x76, 0x65,
0x72, 0x64, 0x79, 0x46, 0x61, 0x65, 0x72, 0x73, 0x6b, 0xe9, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x4a, 0x69, 0x17e, 0x6e, 0xed, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x53, 0x76, 0x61, 0x74, 0xe1,
0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x73, 0x74, 0x72, 0x61,
0x63, 0x68, 0x61, 0x148, 0x41, 0x74, 0x68, 0xe9, 0x6e, 0x79, 0x42, 0x11b,
0x6c, 0x65, 0x68, 0x72, 0x61, 0x64, 0x42, 0x75, 0x6b, 0x75, 0x72, 0x65,
0x161, 0x165, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x161, 0x165, 0x4b, 0x69,
0x161, 0x69, 0x6e, 0x11b, 0x76, 0x4b, 0x6f, 0x64, 0x61, 0x148, 0x48, 0x65,
0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x79, 0x4f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x20, 0x4d, 0x61, 0x6e, 0x4b, 0x79, 0x6a, 0x65, 0x76, 0x4c, 0x75, 0x62,
0x6c, 0x61, 0x148, 0x4c, 0x6f, 0x6e, 0x64, 0xfd, 0x6e, 0x4c, 0x75, 0x63,
0x65, 0x6d, 0x62, 0x75, 0x72, 0x6b, 0x50, 0x61, 0x159, 0xed, 0x17e, 0x158,
0xed, 0x6d, 0x53, 0x6f, 0x66, 0x69, 0x65, 0x56, 0x61, 0x74, 0x69, 0x6b,
0xe1, 0x6e, 0x56, 0xed, 0x64, 0x65, 0x148, 0x5a, 0xe1, 0x68, 0x159, 0x65,
0x62, 0x43, 0x75, 0x72, 0x79, 0x63, 0x68, 0x56, 0xe1, 0x6e, 0x6f, 0x10d,
0x6e, 0xed, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x4b, 0x6f, 0x6b,
0x6f, 0x73, 0x6f, 0x76, 0xe9, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x79, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x79, 0x4b, 0x65, 0x72, 0x67, 0x75,
0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x76, 0x79, 0x20, 0x6f, 0x73, 0x74, 0x72,
0x6f, 0x76, 0x79, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x76, 0x79, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x75, 0x73, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x73, 0x6b, 0xe9, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x79, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x6b, 0xe9, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f,
0x6e, 0x6f, 0x10d, 0x6e, 0xed, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x47, 0x61, 0x6c, 0x61, 0x70, 0xe1, 0x67, 0x79, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x6f, 0x76, 0x79, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x79, 0x4d, 0x61, 0x72, 0x6b, 0xe9, 0x7a, 0x79, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x76, 0x79, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x79, 0x41, 0x6c, 0x67, 0x69, 0x65, 0x72, 0x43, 0x61,
0x79, 0x6d, 0x61, 0x6e, 0xf8, 0x65, 0x72, 0x6e, 0x65, 0x41, 0x73, 0x6a,
0x6b, 0x68, 0x61, 0x62, 0x61, 0x64, 0x54, 0x73, 0x6a, 0x6f, 0x6a, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x44, 0x75, 0x73, 0x6a, 0x61, 0x6e, 0x62,
0x65, 0x4b, 0x61, 0x6d, 0x74, 0x6a, 0x61, 0x74, 0x6b, 0x61, 0x4e, 0x6f,
0x76, 0x6f, 0x6b, 0x75, 0x73, 0x6e, 0x65, 0x74, 0x73, 0x6b, 0x54, 0x61,
0x73, 0x6a, 0x6b, 0x65, 0x6e, 0x74, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x72,
0x6e, 0x65, 0x44, 0x65, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x73,
0x6b, 0x65, 0x20, 0xd8, 0x65, 0x72, 0x46, 0xe6, 0x72, 0xf8, 0x65, 0x72,
0x6e, 0x65, 0x4b, 0xf8, 0x62, 0x65, 0x6e, 0x68, 0x61, 0x76, 0x6e, 0x56,
0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x74, 0x57, 0x61, 0x72, 0x73,
0x7a, 0x61, 0x77, 0x61, 0x4a, 0x75, 0x6c, 0x65, 0xf8, 0x65, 0x6e, 0x43,
0x6f, 0x6d, 0x6f, 0x72, 0x65, 0x72, 0x6e, 0x65, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x65, 0x72, 0x6e, 0x65, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8,
0x65, 0x6e, 0x43, 0x61, 0xef, 0x72, 0x6f, 0x4b, 0x68, 0x61, 0x72, 0x74,
0x6f, 0x65, 0x6d, 0x53, 0x61, 0x6f, 0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x42,
0x65, 0x6e, 0x65, 0x64, 0x65, 0x6e, 0x20, 0x50, 0x72, 0x69, 0x6e, 0x73,
0x65, 0x6e, 0x20, 0x4b, 0x77, 0x61, 0x72, 0x74, 0x69, 0x65, 0x72, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x6f, 0x2d, 0x53, 0x74, 0x61, 0x64, 0x41, 0x6c,
0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61, 0x41, 0x74, 0x131, 0x72, 0x61, 0x77,
0x44, 0x6f, 0x65, 0x73, 0x6a, 0x61, 0x6e, 0x62, 0x65, 0x48, 0x6f, 0x20,
0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x73, 0x74, 0x61, 0x64,
0x4d, 0x61, 0x6e, 0x69, 0x6c, 0x6c, 0x61, 0x53, 0x6a, 0x61, 0x6e, 0x67,
0x68, 0x61, 0x69, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e,
0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x45,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x4b, 0x61, 0x61, 0x70, 0x76,
0x65, 0x72, 0x64, 0x69, 0xeb, 0x46, 0x61, 0x65, 0x72, 0xf6, 0x65, 0x72,
0x5a, 0x75, 0x69, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x53, 0x69, 0x6e, 0x74, 0x2d, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x42,
0x65, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x42, 0x65, 0x72, 0x6c, 0x69,
0x6a, 0x6e, 0x49, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x6f, 0x65, 0x6c, 0x50,
0x61, 0x72, 0x69, 0x6a, 0x73, 0x56, 0x61, 0x74, 0x69, 0x63, 0x61, 0x61,
0x6e, 0x73, 0x74, 0x61, 0x64, 0x57, 0x65, 0x6e, 0x65, 0x6e, 0x43, 0x68,
0x61, 0x67, 0x6f, 0x73, 0x61, 0x72, 0x63, 0x68, 0x69, 0x70, 0x65, 0x6c,
0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x6e, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x6e,
0x50, 0x61, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0xce, 0x6c,
0x65, 0x73, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4d, 0x61,
0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x6e, 0xf68, 0xf7a, 0xf40, 0xf0b, 0xf40, 0xfb2, 0xf68, 0xf0b, 0xf4c,
0xf72, 0xf66, 0xf0b, 0x20, 0xf68, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf56, 0xfb7, 0xf68,
0xf63, 0xf0b, 0xf47, 0xf72, 0xf0b, 0xf61, 0xf66, 0xf68, 0xf71, 0xf66, 0xf0b, 0xf58,
0xf0b, 0xf62, 0xf56, 0xfb7, 0xf0b, 0xf58, 0xf0b, 0xf40, 0xf7c, 0xf56, 0xfb7, 0xf44,
0xf0b, 0xf42, 0xf72, 0xf56, 0xfb7, 0xf71, 0xf53, 0xf0b, 0xf47, 0xf74, 0xf63, 0xf56,
0xfb7, 0xf72, 0xf0b, 0xf66, 0xf71, 0xf60, 0xf74, 0xf56, 0xfb7, 0xfb2, 0xf0b, 0xf5b,
0xf0b, 0xf56, 0xf72, 0xf63, 0xf56, 0xfb7, 0xf74, 0xf0b, 0xf47, 0xf74, 0xf58, 0xf0b,
0xf56, 0xfb7, 0xf74, 0xf0b, 0xf62, 0xf40, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf62, 0xf7c,
0xf40, 0xf7c, 0xf0b, 0xf53, 0xf40, 0xf0b, 0xf62, 0xf72, 0xf4c, 0xf71, 0xf0b, 0xf40,
0xf62, 0xf47, 0xf72, 0xf0b, 0xf56, 0xfb7, 0xf74, 0xf0b, 0xf4a, 0xf72, 0xf42, 0xfb7,
0xf0b, 0xf56, 0xfb7, 0xf7c, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf53, 0xf7a, 0xf67, 0xf0b,
0xf62, 0xf71, 0xf0b, 0xf62, 0xf7a, 0xf40, 0xf7a, 0xf58, 0xf0b, 0xf54, 0xf0b, 0xf63,
0xf41, 0xf71, 0xf62, 0xf0b, 0xf4a, 0xf74, 0xf58, 0xf40, 0xf72, 0xf0b, 0xf42, 0xfb7,
0xf0b, 0xf63, 0xf72, 0xf40, 0xf72, 0xf53, 0xf0b, 0xf64, 0xf0b, 0xf66, 0xf63, 0xf72,
0xf0b, 0xf56, 0xfb7, 0xfb2, 0xf0b, 0xf56, 0xf72, 0xf63, 0xf63, 0xf7c, 0xf0b, 0xf58,
0xf7a, 0xf63, 0xf74, 0xf0b, 0xf5d, 0xf7a, 0xf53, 0xf0b, 0xf4c, 0xf63, 0xf74, 0xf0b,
0xf66, 0xf71, 0xf0b, 0xf40, 0xf58, 0xf0b, 0xf63, 0xf0b, 0xf56, 0xfb7, 0xf7c, 0xf58,
0xf0b, 0xf54, 0xf74, 0xf0b, 0xf4f, 0xf7c, 0xf58, 0xf0b, 0xf66, 0xf0b, 0xf62, 0xf74,
0xf60, 0xf74, 0xf58, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf53, 0xf72,
0xf58, 0xf7c, 0xf0b, 0xf42, 0xf71, 0xf0b, 0xf4c, 0xf72, 0xf0b, 0xf64, 0xf74, 0xf58,
0xf71, 0xf53, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf56, 0xf72, 0xf0b, 0xf61, 0xf53, 0xf71,
0xf60, 0xf72, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf56, 0xf72, 0xf53, 0xf72, 0xf0b, 0xf61,
0xf0b, 0xf58, 0xf7a, 0xf53, 0xf74, 0xf0b, 0xf68, 0xf40, 0xf0b, 0xf64, 0xf60, 0xf7c,
0xf4a, 0xf5d, 0xf0b, 0xf42, 0xf71, 0xf0b, 0xf4c, 0xf74, 0xf0b, 0xf42, 0xf74, 0xf66,
0xf5d, 0xf0b, 0x20, 0xf4a, 0xf7c, 0xf0b, 0xf58, 0xf7a, 0xf4f, 0xfb2, 0xf72, 0xf0b,
0xf54, 0xf7c, 0xf0b, 0xf63, 0xf72, 0xf4a, 0xf74, 0xf0b, 0xf53, 0xf72, 0xf66, 0xf84,
0xf5d, 0xf72, 0xf53, 0xf4c, 0xf0b, 0xf67, 0xf74, 0xf40, 0xf68, 0xf53, 0xf0b, 0xf4a,
0xf72, 0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d, 0xf0b, 0xf56, 0xfb7, 0xf74, 0xf60, 0xf72,
0xf0b, 0xf53, 0xf7c, 0xf66, 0xf0b, 0x20, 0xf68, 0xf60, 0xf72, 0xf0b, 0xf62, 0xf72,
0xf66, 0xf68, 0xf0b, 0xf66, 0xf74, 0xf53, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf60, 0xf7c,
0xf53, 0xf68, 0xf0b, 0xf4f, 0xf72, 0xf0b, 0xf40, 0xf7c, 0xf0b, 0xf40, 0xf71, 0xf53,
0xf56, 0xfb7, 0xf62, 0xf0b, 0xf56, 0xf7a, 0xf0b, 0xf4c, 0xf7c, 0xf66, 0xf56, 0xfb7,
0xf7a, 0xf0b, 0xf63, 0xf72, 0xf5b, 0xf56, 0xfb7, 0xf7c, 0xf0b, 0xf42, 0xf0b, 0xf4a,
0xf40, 0xf7a, 0xf58, 0xf0b, 0xf56, 0xf72, 0xf62, 0xf72, 0xf51, 0xf46, 0xf0b, 0x20,
0xf56, 0xf60, 0xf7a, 0xf40, 0xf0b, 0xf62, 0xf0b, 0xf40, 0xf71, 0xf66, 0xf45, 0xf72,
0xf0b, 0xf40, 0xf71, 0xf0b, 0xf42, 0xf7c, 0xf40, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xf0b,
0xf62, 0xf72, 0xf0b, 0xf40, 0xf51, 0xf60, 0xf74, 0xf0b, 0xf66, 0xf71, 0xf7c, 0xf53,
0xf51, 0xf60, 0xf74, 0xf0b, 0xf66, 0xf71, 0xf7c, 0xf53, 0xf0b, 0x20, 0xf40, 0xf72,
0xf62, 0xf72, 0xf40, 0xf4c, 0xf7c, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf53, 0xf72, 0xf0b,
0xf40, 0xf68, 0xf7a, 0xf4c, 0xf0b, 0xf58, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf4a, 0xf7c,
0xf53, 0xf68, 0xf7a, 0xf63, 0xf0b, 0x20, 0xf66, 0xf71, 0xf63, 0xf0b, 0xf56, 0xf0b,
0xf4c, 0xf7c, 0xf42, 0xf71, 0xf74, 0xf66, 0xf72, 0xf0b, 0x20, 0xf56, 0xf60, 0xf7a,
0xf42, 0xfb7, 0xf7c, 0xf0b, 0xf4a, 0xf0b, 0xf58, 0xf0b, 0xf63, 0xf42, 0xfb7, 0xf0b,
0xf61, 0xf0b, 0xf53, 0xf67, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf55, 0xf7a, 0xf42, 0xf66,
0xf72, 0xf68, 0xf72, 0xf0b, 0xf53, 0xf74, 0xf0b, 0xf5d, 0xf72, 0xf40, 0xf47, 0xf71,
0xf0b, 0xf58, 0xf60, 0xf7a, 0xf0b, 0xf40, 0xf63, 0xf0b, 0xf54, 0xf71, 0xf5b, 0xf84,
0xf63, 0xf72, 0xf0b, 0xf58, 0xf58, 0xf7a, 0xf40, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf40,
0xf7c, 0xf0b, 0x20, 0xf66, 0xf72, 0xf0b, 0xf4a, 0xf72, 0xf58, 0xf71, 0xf7c, 0xf44,
0xf40, 0xf0b, 0xf4a, 0xf7c, 0xf53, 0xf58, 0xf7c, 0xf53, 0xf0b, 0xf4a, 0xf7a, 0xf0b,
0xf56, 0xfb7, 0xf72, 0xf0b, 0xf4c, 0xf72, 0xf60, 0xf7c, 0xf53, 0xf0b, 0xf66, 0xf60,
0xf7c, 0xf54, 0xf0b, 0xf53, 0xf0b, 0xf58, 0xf54, 0xf0b, 0xf62, 0xf0b, 0xf58, 0xf0b,
0xf62, 0xf72, 0xf0b, 0xf56, 0xfb7, 0xf7c, 0xf62, 0xf7a, 0xf53, 0xf0b, 0xf40, 0xf72,
0xf53, 0xf0b, 0x20, 0xf68, 0xf72, 0xf53, 0xf0b, 0xf63, 0xf7a, 0xf4a, 0xf62, 0xf72,
0xf0b, 0xf47, 0xf72, 0xf0b, 0xf53, 0xf62, 0xf72, 0xf0b, 0xf66, 0xf7c, 0xf0b, 0xf63,
0xf72, 0xf60, 0xf74, 0xf4a, 0xf66, 0xf71, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf61,
0xf0b, 0xf42, 0xfb7, 0xf7c, 0xf66, 0xf71, 0xf53, 0xf0b, 0xf4a, 0xf7c, 0xf0b, 0x20,
0xf4c, 0xf7c, 0xf0b, 0xf58, 0xf72, 0xf44, 0xf0b, 0xf42, 0xfb7, 0xf7c, 0xf68, 0xf72,
0xf66, 0xf72, 0xf0b, 0xf4a, 0xf7a, 0xf4a, 0xf0b, 0x20, 0xf47, 0xf71, 0xf7c, 0xf53,
0xf66, 0xf72, 0xf0b, 0xf66, 0xf74, 0xf0b, 0xf68, 0xf72, 0xf55, 0xf4a, 0xf0b, 0xf40,
0xf0b, 0xf62, 0xf7a, 0xf53, 0xf4a, 0xf4a, 0xf7a, 0xf0b, 0xf42, 0xf74, 0xf0b, 0xf66,
0xf72, 0xf0b, 0xf42, 0xf71, 0xf63, 0xf0b, 0xf54, 0xf4f, 0xf72, 0xf60, 0xf74, 0xf0b,
0xf5d, 0xf0b, 0xf53, 0xf4a, 0xf7c, 0xf0b, 0xf62, 0xf7c, 0xf53, 0xf0b, 0xf4a, 0xf7c,
0xf61, 0xf0b, 0xf40, 0xf74, 0xf0b, 0xf4f, 0xf4f, 0xf4c, 0xf7a, 0xf0b, 0xf5d, 0xf72,
0xf66, 0xf0d, 0xf58, 0xf60, 0xf74, 0xf0b, 0xf66, 0xf71, 0xf7c, 0xf53, 0xf55, 0xf71,
0xf63, 0xf0b, 0xf58, 0xf62, 0xf62, 0xf71, 0xf7c, 0xf0b, 0xf50, 0xf7a, 0xf0b, 0xf62,
0xf5d, 0xf7c, 0xf66, 0xf72, 0xf0b, 0xf4a, 0xf7c, 0xf40, 0xf56, 0xfb7, 0xf0b, 0xf62,
0xf7a, 0xf53, 0xf0b, 0xf56, 0xfb7, 0xfb2, 0xf74, 0xf0b, 0xf53, 0xf60, 0xf72, 0xf40,
0xf7c, 0xf0b, 0xf63, 0xf7c, 0xf58, 0xf0b, 0xf56, 0xfb7, 0xf7c, 0xf47, 0xf71, 0xf0b,
0xf40, 0xf62, 0xf0b, 0xf4a, 0xf47, 0xf0b, 0xf61, 0xf0b, 0xf54, 0xf74, 0xf0b, 0xf62,
0xf40, 0xf0b, 0xf56, 0xfb7, 0xf74, 0xf63, 0xf40, 0xf0b, 0xf62, 0xf0b, 0xf45, 0xf72,
0xf40, 0xf71, 0xf4a, 0xf0b, 0xf58, 0xf71, 0xf53, 0xf0b, 0xf4c, 0xf74, 0xf40, 0xf71,
0xf7c, 0xf63, 0xf0b, 0xf40, 0xf0b, 0xf4a, 0xf40, 0xf7c, 0xf0b, 0xf63, 0xf0b, 0x20,
0xf63, 0xf58, 0xf0b, 0xf54, 0xf74, 0xf62, 0xf54, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf4a,
0xf60, 0xf7a, 0xf0b, 0xf53, 0xf40, 0xf4a, 0xf7a, 0xf0b, 0xf62, 0xf71, 0xf53, 0xf50,
0xf72, 0xf58, 0xf0b, 0xf55, 0xf74, 0xf42, 0xf56, 0xf72, 0xf0b, 0xf61, 0xf71, 0xf53,
0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf61, 0xf71, 0xf53, 0xf62, 0xf44, 0xf0b, 0xf42, 0xfb7,
0xf74, 0xf53, 0xf56, 0xf62, 0xf0b, 0xf58, 0xf74, 0xf0b, 0xf51, 0xf40, 0xf7a, 0xf54,
0xf0b, 0xf56, 0xf71, 0xf4c, 0xf55, 0xf71, 0xf60, 0xf7a, 0xf0b, 0xf62, 0xf7c, 0xf0b,
0xf62, 0xf7a, 0xf0b, 0xf41, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf56, 0xf72, 0xf40, 0xf66,
0xf71, 0xf60, 0xf74, 0xf50, 0xf0b, 0xf47, 0xf7c, 0xf0b, 0xf47, 0xf71, 0xf68, 0xf58,
0xf66, 0xf0b, 0xf4a, 0xf62, 0xf0b, 0xf4c, 0xf58, 0xf68, 0xf71, 0xf53, 0xf0b, 0xf51,
0xf7c, 0xf0b, 0xf62, 0xf68, 0xf7a, 0xf0b, 0xf50, 0xf7a, 0xf53, 0xf66, 0xf0b, 0xf56,
0xfb7, 0xf62, 0xf0b, 0xf63, 0xf72, 0xf53, 0xf56, 0xfb2, 0xf0b, 0xf66, 0xf7a, 0xf63,
0xf66, 0xf56, 0xfb7, 0xf74, 0xf0b, 0xf40, 0xf0b, 0xf62, 0xf7a, 0xf66, 0xf4a, 0xf56,
0xfb7, 0xf74, 0xf0b, 0xf4c, 0xf0b, 0xf54, 0xf7a, 0xf66, 0xf4a, 0xf40, 0xf72, 0xf0b,
0xf64, 0xf72, 0xf0b, 0xf53, 0xf71, 0xf5d, 0xf40, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53,
0xf0b, 0xf67, 0xf7a, 0xf0b, 0xf42, 0xf7a, 0xf53, 0xf4c, 0xf56, 0xf0b, 0xf63, 0xf72,
0xf53, 0xf67, 0xf7a, 0xf63, 0xf0b, 0xf66, 0xf72, 0xf44, 0xf0b, 0xf40, 0xf72, 0xf63,
0xf71, 0xf53, 0xf0b, 0xf4c, 0xf71, 0xf53, 0xf63, 0xf42, 0xf0b, 0xf66, 0xf58, 0xf0b,
0xf56, 0xfb7, 0xf40, 0xf58, 0xf0b, 0xf4c, 0xf7a, 0xf0b, 0xf62, 0xf7a, 0xf4c, 0xf58,
0xf72, 0xf53, 0xf66, 0xf40, 0xf58, 0xf7c, 0xf0b, 0xf53, 0xf0b, 0xf40, 0xf7c, 0xf58,
0xf71, 0xf7c, 0xf66, 0xf0b, 0xf40, 0xf7c, 0xf68, 0xf7c, 0xf66, 0xf0b, 0xf63, 0xf7c,
0xf54, 0xf7a, 0xf0b, 0xf62, 0xf72, 0xf66, 0xf54, 0xfb2, 0xf42, 0xf62, 0xf72, 0xf0b,
0xf42, 0xfb7, 0xf66, 0xf0b, 0xf62, 0xf0b, 0xf61, 0xf7a, 0xf0b, 0xf56, 0xf7c, 0xf66,
0xf7c, 0xf0b, 0xf55, 0xf72, 0xf0b, 0xf61, 0xf68, 0xf72, 0xf66, 0xf0b, 0xf4a, 0xf7c,
0xf40, 0xf0b, 0xf67, 0xf7c, 0xf63, 0xf58, 0xf4a, 0xf71, 0xf0b, 0xf63, 0xf71, 0xf72,
0xf53, 0xf4a, 0xf0b, 0xf62, 0xf0b, 0xf53, 0xf72, 0xf56, 0xf0b, 0xf51, 0xf74, 0xf51,
0xf66, 0xf5d, 0xf72, 0xf0b, 0xf61, 0xf7a, 0xf0b, 0xf53, 0xf56, 0xf72, 0xf63, 0xf0b,
0xf53, 0xf72, 0xf0b, 0xf61, 0xf71, 0xf66, 0xf5d, 0xf62, 0xf0b, 0xf66, 0xf7c, 0xf5b,
0xf74, 0xf0b, 0xf62, 0xf72, 0xf46, 0xf68, 0xf71, 0xf53, 0xf0b, 0xf4f, 0xf0b, 0xf53,
0xf0b, 0xf53, 0xf0b, 0xf51, 0xf72, 0xf0b, 0xf56, 0xf7c, 0xf40, 0xf72, 0xf62, 0xf72,
0xf66, 0xf4a, 0xf0b, 0xf58, 0xf66, 0xf72, 0xf40, 0xf7c, 0xf0b, 0xf40, 0xf7c, 0xf66,
0xf72, 0xf40, 0xf7c, 0xf0b, 0xf58, 0xf7c, 0xf0b, 0xf62, 0xf7c, 0xf40, 0xf72, 0xf62,
0xf0b, 0xf42, 0xf74, 0xf0b, 0xf63, 0xf72, 0xf53, 0xf58, 0xf71, 0xf63, 0xf0b, 0xf51,
0xf72, 0xf56, 0xf66, 0xf58, 0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf64, 0xf71, 0xf66,
0xf68, 0xf0b, 0xf54, 0xf72, 0xf0b, 0xf61, 0xf55, 0xf0b, 0xf40, 0xf60, 0xf7c, 0xf0b,
0xf55, 0xf7c, 0xf55, 0xf74, 0xf0b, 0xf53, 0xf0b, 0xf55, 0xf74, 0xf0b, 0xf4a, 0xf72,
0xf42, 0xf7a, 0xf58, 0xf0b, 0xf56, 0xf72, 0xf61, 0xf62, 0xf58, 0xf7a, 0xf0b, 0xf47,
0xf74, 0xf0b, 0xf62, 0xf7c, 0xf58, 0xf72, 0xf4c, 0xf0b, 0xf5d, 0xf7a, 0xf53, 0xf71,
0xf5d, 0xf0b, 0xf62, 0xf74, 0xf54, 0xf0b, 0xf42, 0xf7c, 0xf0b, 0x20, 0xf54, 0xf0b,
0xf42, 0xf7c, 0xf54, 0xf0b, 0xf63, 0xf71, 0xf5d, 0xf62, 0xf71, 0xf0b, 0xf62, 0xf7c,
0xf0b, 0xf0b, 0xf4f, 0xf7c, 0xf44, 0xf0b, 0xf42, 0xf66, 0xf71, 0xf60, 0xf7a, 0xf0b,
0xf55, 0xf71, 0xf53, 0xf4f, 0xf7c, 0xf44, 0xf0b, 0xf42, 0xf0b, 0xf4f, 0xf0b, 0xf54,
0xf74, 0xf5d, 0xf7a, 0xf40, 0xd801, 0xdc08, 0xd801, 0xdc3c, 0xd801, 0xdc30, 0xd801, 0xdc3f,
0xd801, 0xdc01, 0xd801, 0xdc4d, 0xd801, 0xdc3f, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc2e,
0xd801, 0xdc3e, 0xd801, 0xdc12, 0xd801, 0xdc31, 0xd801, 0xdc2e, 0xd801, 0xdc46, 0xd801, 0xdc28,
0xd801, 0xdc1f, 0xd801, 0xdc2e, 0xd801, 0xdc3f, 0xd801, 0xdc2a, 0xd801, 0xdc40, 0xd801, 0xdc2c,
0xd801, 0xdc14, 0xd801, 0xdc2f, 0xd801, 0xdc4c, 0xd801, 0xdc42, 0xd801, 0xdc32, 0xd801, 0xdc49,
0xd801, 0xdc14, 0xd801, 0xdc32, 0xd801, 0xdc3b, 0xd801, 0xdc49, 0xd801, 0xdc31, 0xd801, 0xdc2e,
0xd801, 0xdc3b, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc32,
0xd801, 0xdc4c, 0xd801, 0xdc30, 0xd801, 0xdc39, 0xd801, 0xdc2c, 0xd801, 0xdc4a, 0xd801, 0xdc32,
0xd801, 0xdc45, 0xd801, 0xdc24, 0xd801, 0xdc2a, 0xd801, 0xdc3f, 0xd801, 0xdc45, 0x2c, 0x20,
0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c,
0xd801, 0xdc32, 0xd801, 0xdc23, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc2f, 0xd801, 0xdc4d,
0xd801, 0xdc40, 0xd801, 0xdc2c, 0x2c, 0x20, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c,
0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc11, 0xd801, 0xdc28,
0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc46, 0xd801, 0xdc3a, 0xd801, 0xdc32,
0xd801, 0xdc49, 0xd801, 0xdc40, 0x2c, 0x20, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c,
0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc13, 0xd801, 0xdc2f,
0xd801, 0xdc4a, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc2e, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0x2c,
0x20, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801,
0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc1a, 0xd801, 0xdc2f, 0xd801, 0xdc42, 0xd801, 0xdc29, 0x2c,
0x20, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801,
0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc1a, 0xd801, 0xdc2e, 0xd801, 0xdc4c, 0xd801, 0xdc45, 0xd801,
0xdc2f, 0xd801, 0xdc4c, 0xd801, 0xdc46, 0x2c, 0x20, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801,
0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc0e, 0xd801,
0xdc2e, 0xd801, 0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc4b, 0xd801, 0xdc30, 0xd801, 0xdc3f, 0x2c,
0x20, 0xd801, 0xdc06, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc28, 0xd801, 0xdc30, 0xd801,
0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc16, 0xd801, 0xdc2d, 0xd801, 0xdc4c, 0xd801, 0xdc2c, 0xd801,
0xdc22, 0xd801, 0xdc2d, 0xd801, 0xdc36, 0xd801, 0xdc28, 0xd801, 0xdc42, 0xd801, 0xdc2e, 0xd801,
0xdc4a, 0xd801, 0xdc23, 0xd801, 0xdc2a, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801,
0xdc45, 0xd801, 0xdc2f, 0xd801, 0xdc4a, 0xd801, 0xdc2c, 0x2c, 0x20, 0xd801, 0xdc17, 0xd801,
0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc3f, 0xd801, 0xdc28, 0xd801,
0xdc22, 0xd801, 0xdc31, 0xd801, 0xdc45, 0x20, 0xd801, 0xdc08, 0xd801, 0xdc4c, 0xd801, 0xdc3e,
0xd801, 0xdc32, 0xd801, 0xdc4a, 0xd801, 0xdc45, 0xd801, 0xdc23, 0xd801, 0xdc32, 0xd801, 0xdc4c,
0xd801, 0xdc2a, 0xd801, 0xdc4b, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc28, 0xd801, 0xdc24,
0xd801, 0xdc2d, 0x20, 0xd801, 0xdc0f, 0xd801, 0xdc31, 0xd801, 0xdc49, 0xd801, 0xdc3f, 0xd801,
0xdc24, 0xd801, 0xdc2c, 0xd801, 0xdc4b, 0xd801, 0xdc1d, 0xd801, 0xdc2f, 0xd801, 0xdc4c, 0xd801,
0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc49, 0x2c, 0x20, 0xd801, 0xdc24, 0xd801, 0xdc31, 0xd801,
0xdc49, 0xd801, 0xdc43, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc32, 0xd801, 0xdc3f, 0xd801, 0xdc2c,
0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc24, 0xd801, 0xdc2d, 0x20, 0xd801, 0xdc1d, 0xd801,
0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc32, 0xd801, 0xdc4b, 0x2c, 0x20, 0xd801, 0xdc24, 0xd801,
0xdc31, 0xd801, 0xdc49, 0xd801, 0xdc43, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc32, 0xd801, 0xdc3f,
0xd801, 0xdc2c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc19, 0xd801, 0xdc28, 0xd801, 0xdc4c,
0xd801, 0xdc2e, 0xd801, 0xdc3f, 0xd801, 0xdc45, 0xd801, 0xdc0f, 0xd801, 0xdc30, 0xd801, 0xdc3f,
0xd801, 0xdc2d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc3b, 0xd801, 0xdc10, 0xd801, 0xdc2a,
0xd801, 0xdc4c, 0xd801, 0xdc32, 0xd801, 0xdc4a, 0xd801, 0xdc2d, 0xd801, 0xdc4a, 0xd801, 0xdc2d,
0xd801, 0xdc23, 0xd801, 0xdc2e, 0xd801, 0xdc3c, 0xd801, 0xdc36, 0xd801, 0xdc29, 0xd801, 0xdc0e,
0xd801, 0xdc29, 0xd801, 0xdc3f, 0x53, 0x74, 0x20, 0x42, 0x61, 0x72, 0x74, 0x68,
0xe9, 0x6c, 0x65, 0x6d, 0x79, 0x53, 0x74, 0x20, 0x4a, 0x6f, 0x68, 0x6e,
0x2019, 0x73, 0x53, 0x74, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73, 0x53, 0x74,
0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x53, 0x74, 0x20, 0x54, 0x68, 0x6f,
0x6d, 0x61, 0x73, 0x53, 0x74, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e,
0x74, 0x53, 0x74, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x428, 0x438,
0x43a, 0x430, 0x433, 0x43e, 0x41b, 0x43e, 0x43d, 0x433, 0x439, 0x438, 0x440, 0x431,
0x44e, 0x435, 0x43d, 0x41e, 0x43c, 0x441, 0x43a, 0x43e, 0x439, 0x422, 0x430, 0x439,
0x43f, 0x435, 0x439, 0x422, 0x43e, 0x43c, 0x441, 0x43a, 0x43e, 0x439, 0x423, 0x43b,
0x430, 0x43d, 0x2d, 0x431, 0x430, 0x442, 0x43e, 0x440, 0x410, 0x444, 0x438, 0x43d,
0x44b, 0x41c, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x439, 0x41c, 0x43e, 0x441, 0x43a,
0x43e, 0x432, 0x41, 0x62, 0x69, 0x11d, 0x61, 0x6e, 0x6f, 0x41, 0x6b, 0x72,
0x61, 0x6f, 0x41, 0x64, 0x69, 0x73, 0x2d, 0x41, 0x62, 0x65, 0x62, 0x6f,
0x41, 0x6c, 0x11d, 0x65, 0x72, 0x6f, 0x41, 0x73, 0x6d, 0x65, 0x72, 0x6f,
0x42, 0x61, 0x6e, 0x67, 0x6f, 0x42, 0x61, 0x6e, 0x6a, 0x75, 0x6c, 0x6f,
0x42, 0x69, 0x73, 0x61, 0x16d, 0x6f, 0x42, 0x72, 0x61, 0x7a, 0x61, 0x76,
0x69, 0x6c, 0x6f, 0x42, 0x75, 0x135, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x6f,
0x4b, 0x61, 0x7a, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0x6f, 0x43, 0x65,
0x16d, 0x74, 0x6f, 0x4b, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x69, 0x6f, 0x44,
0x61, 0x6b, 0x61, 0x72, 0x6f, 0x44, 0x61, 0x72, 0x65, 0x73, 0x61, 0x6c,
0x61, 0x6d, 0x6f, 0x11c, 0x69, 0x62, 0x75, 0x74, 0x69, 0x6f, 0x44, 0x75,
0x61, 0x6c, 0x61, 0x6f, 0x41, 0x6a, 0x75, 0x6e, 0x6f, 0x46, 0x72, 0x69,
0x74, 0x61, 0x16d, 0x6e, 0x6f, 0x47, 0x61, 0x62, 0x6f, 0x72, 0x6f, 0x6e,
0x6f, 0x48, 0x61, 0x72, 0x61, 0x72, 0x65, 0x6f, 0x4a, 0x6f, 0x68, 0x61,
0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x11c, 0x75, 0x62, 0x61,
0x6f, 0x4b, 0x61, 0x6d, 0x70, 0x61, 0x6c, 0x6f, 0x124, 0x61, 0x72, 0x74,
0x75, 0x6d, 0x6f, 0x4b, 0x69, 0x67, 0x61, 0x6c, 0x6f, 0x4b, 0x69, 0x6e,
0x15d, 0x61, 0x73, 0x6f, 0x4c, 0x61, 0x67, 0x6f, 0x73, 0x6f, 0x4c, 0x69,
0x62, 0x72, 0x65, 0x76, 0x69, 0x6c, 0x6f, 0x4c, 0x6f, 0x6d, 0x65, 0x6f,
0x4c, 0x75, 0x61, 0x6e, 0x64, 0x6f, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62,
0x61, 0x15d, 0x6f, 0x4c, 0x75, 0x73, 0x61, 0x6b, 0x6f, 0x4d, 0x61, 0x73,
0x65, 0x72, 0x75, 0x6f, 0x4d, 0x62, 0x61, 0x62, 0x61, 0x6e, 0x6f, 0x4d,
0x6f, 0x67, 0x61, 0x64, 0x69, 0x15d, 0x6f, 0x4d, 0x6f, 0x6e, 0x72, 0x6f,
0x76, 0x69, 0x6f, 0x4e, 0x61, 0x6a, 0x72, 0x6f, 0x62, 0x69, 0x6f, 0x4e,
0x69, 0x11d, 0x61, 0x6d, 0x65, 0x6e, 0x6f, 0x4e, 0x69, 0x61, 0x6d, 0x65,
0x6a, 0x6f, 0x4e, 0x75, 0x61, 0x6b, 0x15d, 0x6f, 0x74, 0x6f, 0x56, 0x61,
0x67, 0x61, 0x64, 0x75, 0x67, 0x75, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x6f,
0x6d, 0x65, 0x6f, 0x54, 0x72, 0x69, 0x70, 0x6f, 0x6c, 0x6f, 0x54, 0x75,
0x6e, 0x69, 0x7a, 0x6f, 0x56, 0x69, 0x6e, 0x64, 0x68, 0x75, 0x6b, 0x6f,
0x41, 0x6e, 0x67, 0x76, 0x69, 0x6c, 0x6f, 0x41, 0x6e, 0x74, 0x69, 0x67,
0x76, 0x6f, 0x42, 0x6f, 0x6e, 0x61, 0x65, 0x72, 0x6f, 0x4b, 0x6f, 0x72,
0x64, 0x6f, 0x62, 0x6f, 0x53, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x6c, 0x76,
0x61, 0x64, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x75, 0x6a, 0x75,
0x79, 0x53, 0x61, 0x6c, 0x74, 0x61, 0x75, 0x72, 0x62, 0x6f, 0x53, 0x61,
0x6e, 0x20, 0x4d, 0x69, 0x67, 0x75, 0x65, 0x6c, 0x20, 0x64, 0x65, 0x20,
0x54, 0x75, 0x63, 0x75, 0x6d, 0xe1, 0x6e, 0x41, 0x72, 0x75, 0x62, 0x6f,
0x41, 0x73, 0x75, 0x6e, 0x63, 0x69, 0x6f, 0x6e, 0x6f, 0x42, 0x61, 0x72,
0x62, 0x61, 0x64, 0x6f, 0x42, 0x65, 0x6c, 0x65, 0x6d, 0x6f, 0x42, 0x65,
0x6c, 0x69, 0x7a, 0x6f, 0x42, 0x6f, 0x67, 0x6f, 0x74, 0x6f, 0x4b, 0x61,
0x6e, 0x6b, 0x75, 0x6e, 0x6f, 0x4b, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x73,
0x6f, 0x4b, 0x61, 0x6a, 0x65, 0x6e, 0x6f, 0x4b, 0x61, 0x6a, 0x6d, 0x61,
0x6e, 0x6f, 0x6a, 0x108, 0x69, 0x6b, 0x61, 0x67, 0x6f, 0x4b, 0x6f, 0x73,
0x74, 0x61, 0x72, 0x69, 0x6b, 0x6f, 0x4b, 0x75, 0x72, 0x61, 0x63, 0x61,
0x6f, 0x44, 0x65, 0x6e, 0x76, 0x65, 0x72, 0x6f, 0x44, 0x65, 0x74, 0x72,
0x6f, 0x6a, 0x74, 0x6f, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0x6f,
0x45, 0x64, 0x6d, 0x6f, 0x6e, 0x74, 0x6f, 0x6e, 0x6f, 0x53, 0x61, 0x6c,
0x76, 0x61, 0x64, 0x6f, 0x72, 0x6f, 0x46, 0x6f, 0x72, 0x74, 0x61, 0x6c,
0x65, 0x7a, 0x6f, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6f, 0x47, 0x72, 0x65, 0x6e, 0x61, 0x64, 0x6f, 0x47, 0x76,
0x61, 0x64, 0x65, 0x6c, 0x75, 0x70, 0x6f, 0x47, 0x76, 0x61, 0x74, 0x65,
0x6d, 0x61, 0x6c, 0x6f, 0x47, 0x75, 0x6a, 0x61, 0x6e, 0x6f, 0x48, 0x61,
0x6c, 0x69, 0x66, 0x61, 0x6b, 0x73, 0x6f, 0x48, 0x61, 0x76, 0x61, 0x6e,
0x6f, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x6f,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x4b, 0x6e,
0x6f, 0x78, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f,
0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x61, 0x6f, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73, 0x62,
0x75, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x6f, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43, 0x69, 0x74, 0x79, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x56, 0x65, 0x76, 0x61,
0x79, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x56,
0x69, 0x6e, 0x63, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61,
0x63, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x6f, 0x49,
0x6b, 0x61, 0x6c, 0x75, 0x69, 0x74, 0x6f, 0x4a, 0x61, 0x6d, 0x61, 0x6a,
0x6b, 0x6f, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x63, 0x65, 0x6c, 0x6c, 0x6f,
0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x6b, 0x69, 0x6f, 0x4c, 0x61,
0x2d, 0x50, 0x61, 0x7a, 0x6f, 0x4c, 0x69, 0x6d, 0x6f, 0x4c, 0x6f, 0x73,
0x61, 0x6e, 0x11d, 0x65, 0x6c, 0x65, 0x73, 0x6f, 0x4d, 0x61, 0x6e, 0x61,
0x67, 0x76, 0x6f, 0x4d, 0x61, 0x6e, 0x61, 0x16d, 0x73, 0x6f, 0x4d, 0x61,
0x72, 0x69, 0x67, 0x6f, 0x74, 0x6f, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e,
0x69, 0x6b, 0x6f, 0x4d, 0x65, 0x72, 0x69, 0x64, 0x6f, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x75, 0x72, 0x62, 0x6f, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x6f, 0x6e, 0x6f, 0x4d, 0x6f, 0x6e, 0x6b, 0x74, 0x6f, 0x6e, 0x6f, 0x4d,
0x6f, 0x6e, 0x74, 0x65, 0x72, 0x65, 0x6a, 0x6f, 0x4d, 0x6f, 0x6e, 0x63,
0x65, 0x72, 0x61, 0x74, 0x6f, 0x4e, 0x61, 0x73, 0x61, 0x16d, 0x6f, 0x4e,
0x6f, 0x76, 0x6a, 0x6f, 0x72, 0x6b, 0x6f, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x4e, 0x6f,
0x72, 0x64, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x6f, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x6f, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x6f, 0x4e, 0x75, 0x6b, 0x6f, 0x50, 0x61, 0x6e,
0x61, 0x6d, 0x6f, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x6f, 0x50, 0x6f,
0x72, 0x74, 0x6f, 0x70, 0x72, 0x69, 0x6e, 0x63, 0x6f, 0x50, 0x6f, 0x72,
0x74, 0x6f, 0x73, 0x70, 0x65, 0x6e, 0x6f, 0x50, 0x75, 0x65, 0x72, 0x74,
0x6f, 0x72, 0x69, 0x6b, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20,
0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x53, 0x61, 0x6e, 0x2d, 0x50,
0x61, 0x16d, 0x6c, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x42,
0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x6f, 0x53, 0x61, 0x6e,
0x6b, 0x74, 0x61, 0x20, 0x4b, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x66, 0x6f,
0x72, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x4c, 0x75, 0x63,
0x69, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x54, 0x6f, 0x6d,
0x61, 0x73, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x56, 0x69,
0x6e, 0x63, 0x65, 0x6e, 0x74, 0x6f, 0x54, 0x65, 0x67, 0x75, 0x63, 0x69,
0x67, 0x61, 0x6c, 0x70, 0x6f, 0x56, 0x61, 0x6e, 0x6b, 0x75, 0x76, 0x65,
0x72, 0x6f, 0x56, 0x69, 0x6e, 0x69, 0x70, 0x65, 0x67, 0x6f, 0x4d, 0x61,
0x6b, 0x76, 0x6f, 0x72, 0x61, 0x20, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f,
0x4c, 0x6f, 0x6e, 0x67, 0x6a, 0x65, 0x72, 0x75, 0x72, 0x62, 0x6f, 0x41,
0x64, 0x65, 0x6e, 0x6f, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x6f, 0x41, 0x6d,
0x61, 0x6e, 0x6f, 0x41, 0x15d, 0x125, 0x61, 0x62, 0x61, 0x64, 0x6f, 0x42,
0x61, 0x67, 0x64, 0x61, 0x64, 0x6f, 0x42, 0x61, 0x72, 0x65, 0x6a, 0x6e,
0x6f, 0x42, 0x61, 0x6b, 0x75, 0x6f, 0x42, 0x61, 0x6e, 0x6b, 0x6f, 0x6b,
0x6f, 0x42, 0x65, 0x6a, 0x72, 0x75, 0x74, 0x6f, 0x42, 0x69, 0x15d, 0x6b,
0x65, 0x6b, 0x6f, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x6f, 0x108, 0x69,
0x74, 0x61, 0x108, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x44,
0x69, 0x6c, 0x6f, 0x44, 0x75, 0x62, 0x61, 0x6a, 0x75, 0x72, 0x62, 0x6f,
0x44, 0x75, 0x15d, 0x61, 0x6e, 0x62, 0x65, 0x6f, 0x47, 0x61, 0x7a, 0x61,
0x2d, 0x75, 0x72, 0x62, 0x6f, 0x48, 0x65, 0x62, 0x72, 0x6f, 0x6e, 0x6f,
0x48, 0x6f, 0x109, 0x69, 0x6d, 0x69, 0x6e, 0x6f, 0x48, 0x6f, 0x6e, 0x6b,
0x6f, 0x6e, 0x67, 0x6f, 0x124, 0x6f, 0x76, 0x64, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x6f, 0x11c, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x6f, 0x4a,
0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x6f, 0x4b, 0x61, 0x6d,
0x109, 0x61, 0x74, 0x6b, 0x6f, 0x124, 0x61, 0x6e, 0x64, 0x69, 0x67, 0x61,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f,
0x4b, 0x75, 0x61, 0x6c, 0x61, 0x2d, 0x4c, 0x75, 0x6d, 0x70, 0x75, 0x72,
0x6f, 0x4b, 0x75, 0x76, 0x61, 0x6a, 0x74, 0x6f, 0x4d, 0x61, 0x6e, 0x69,
0x6c, 0x6f, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x74, 0x6f, 0x4e, 0x69, 0x6b,
0x6f, 0x7a, 0x69, 0x6f, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e,
0x65, 0x63, 0x6b, 0x6f, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x6f, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x4f, 0x72, 0x61,
0x6c, 0x6f, 0x50, 0x6e, 0x6f, 0x6d, 0x70, 0x65, 0x6e, 0x6f, 0x50, 0x6a,
0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x6f, 0x4b, 0x61, 0x74, 0x61,
0x72, 0x6f, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x6f, 0x52,
0x69, 0x61, 0x64, 0x6f, 0x53, 0x61, 0x125, 0x61, 0x6c, 0x65, 0x6e, 0x6f,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x64, 0x6f, 0x53, 0x65,
0x75, 0x6c, 0x6f, 0x15c, 0x61, 0x6e, 0x68, 0x61, 0x6a, 0x6f, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x6f, 0x54, 0x61, 0x6a, 0x70, 0x65,
0x6f, 0x54, 0x61, 0x15d, 0x6b, 0x65, 0x6e, 0x74, 0x6f, 0x54, 0x62, 0x69,
0x6c, 0x69, 0x73, 0x6f, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f,
0x72, 0x6f, 0x55, 0x72, 0x75, 0x6d, 0x109, 0x69, 0x6f, 0x55, 0x73, 0x74,
0x6e, 0x65, 0x72, 0x6f, 0x56, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x61, 0x6e,
0x6f, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x6f, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x52, 0x61, 0x6e,
0x67, 0x75, 0x6e, 0x6f, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x45, 0x72, 0x65, 0x76, 0x61, 0x6e,
0x6f, 0x41, 0x63, 0x6f, 0x72, 0x6f, 0x6a, 0x42, 0x65, 0x72, 0x6d, 0x75,
0x64, 0x6f, 0x6a, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x6f, 0x6a, 0x4b,
0x61, 0x62, 0x6f, 0x76, 0x65, 0x72, 0x64, 0x6f, 0x46, 0x65, 0x72, 0x6f,
0x6f, 0x6a, 0x4d, 0x61, 0x64, 0x65, 0x6a, 0x72, 0x6f, 0x52, 0x65, 0x6a,
0x6b, 0x6a, 0x61, 0x76, 0x69, 0x6b, 0x6f, 0x53, 0x75, 0x64, 0x2d, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61,
0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x53, 0x74, 0x61, 0x6e, 0x6c,
0x65, 0x6a, 0x6f, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x6a, 0x64, 0x6f, 0x42,
0x72, 0x69, 0x73, 0x62, 0x61, 0x6e, 0x6f, 0x44, 0x61, 0x72, 0x76, 0x69,
0x6e, 0x6f, 0x48, 0x6f, 0x62, 0x61, 0x72, 0x74, 0x6f, 0x4d, 0x65, 0x6c,
0x62, 0x75, 0x72, 0x6e, 0x6f, 0x50, 0x65, 0x72, 0x74, 0x6f, 0x53, 0x69,
0x64, 0x6e, 0x65, 0x6a, 0x6f, 0x41, 0x6d, 0x73, 0x74, 0x65, 0x72, 0x64,
0x61, 0x6d, 0x6f, 0x41, 0x6e, 0x64, 0x6f, 0x72, 0x6f, 0x41, 0x73, 0x74,
0x72, 0x61, 0x125, 0x61, 0x6e, 0x6f, 0x41, 0x74, 0x65, 0x6e, 0x6f, 0x42,
0x65, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x42, 0x65, 0x72, 0x6c, 0x69,
0x6e, 0x6f, 0x42, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c, 0x61, 0x76, 0x6f,
0x42, 0x72, 0x75, 0x73, 0x65, 0x6c, 0x6f, 0x42, 0x75, 0x6b, 0x61, 0x72,
0x65, 0x15d, 0x74, 0x6f, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x15d, 0x74,
0x6f, 0x42, 0xfc, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x20, 0x61, 0x6d,
0x20, 0x48, 0x6f, 0x63, 0x68, 0x72, 0x68, 0x65, 0x69, 0x6e, 0x4b, 0x69,
0x15d, 0x69, 0x6e, 0x65, 0x76, 0x6f, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68,
0x61, 0x67, 0x6f, 0x44, 0x75, 0x62, 0x6c, 0x69, 0x6e, 0x6f, 0x11c, 0x69,
0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x6f, 0x47, 0x65, 0x72, 0x6e,
0x65, 0x7a, 0x65, 0x6a, 0x6f, 0x48, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x6b,
0x6f, 0x4d, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x49,
0x73, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x6f, 0x134, 0x65, 0x72, 0x7a,
0x65, 0x6a, 0x6f, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72,
0x61, 0x64, 0x6f, 0x4b, 0x69, 0x65, 0x76, 0x6f, 0x4c, 0x69, 0x73, 0x62,
0x6f, 0x6e, 0x6f, 0x4c, 0x6a, 0x75, 0x62, 0x6c, 0x6a, 0x61, 0x6e, 0x6f,
0x4c, 0x6f, 0x6e, 0x64, 0x6f, 0x6e, 0x6f, 0x4c, 0x75, 0x6b, 0x73, 0x65,
0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4d, 0x61, 0x64, 0x72, 0x69, 0x64,
0x6f, 0x4d, 0x61, 0x6c, 0x74, 0x6f, 0x4d, 0x69, 0x6e, 0x73, 0x6b, 0x6f,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x6f, 0x50, 0x61, 0x72, 0x69, 0x7a, 0x6f,
0x50, 0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x63, 0x6f, 0x50, 0x72, 0x61,
0x67, 0x6f, 0x52, 0x69, 0x67, 0x6f, 0x52, 0x6f, 0x6d, 0x6f, 0x53, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x69, 0x6e, 0x6f, 0x53, 0x69, 0x6d, 0x66, 0x65,
0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x6f, 0x53, 0x6b, 0x6f, 0x70, 0x6a, 0x6f,
0x53, 0x6f, 0x66, 0x69, 0x6f, 0x53, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x6c,
0x6d, 0x6f, 0x54, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x54, 0x69, 0x72, 0x61,
0x6e, 0x6f, 0x55, 0x6c, 0x6a, 0x61, 0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x6f,
0x56, 0x61, 0x64, 0x75, 0x7a, 0x6f, 0x56, 0x69, 0x65, 0x6e, 0x6f, 0x56,
0x69, 0x6c, 0x6e, 0x6f, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x6f, 0x56, 0x61, 0x72, 0x73, 0x6f, 0x76, 0x69, 0x6f, 0x5a, 0x61,
0x67, 0x72, 0x65, 0x62, 0x6f, 0x5a, 0x75, 0x72, 0x69, 0x6b, 0x6f, 0x108,
0x61, 0x67, 0x6f, 0x73, 0x6f, 0x6a, 0x4b, 0x72, 0x69, 0x73, 0x74, 0x6e,
0x61, 0x73, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x4b, 0x6f, 0x6b,
0x6f, 0x73, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x6f, 0x6a, 0x4b, 0x6f, 0x6d,
0x6f, 0x72, 0x6f, 0x6a, 0x4b, 0x65, 0x72, 0x67, 0x65, 0x6c, 0x65, 0x6e,
0x6f, 0x6a, 0x4d, 0x61, 0x16d, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x4d, 0x61,
0x6a, 0x6f, 0x74, 0x6f, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x108, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x6f, 0x50, 0x61, 0x73, 0x6b, 0x69, 0x6e, 0x73,
0x75, 0x6c, 0x6f, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x6a,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c,
0x6f, 0x41, 0x6c, 0x17e, 0x69, 0x69, 0x72, 0x48, 0x61, 0x72, 0x74, 0x75,
0x6d, 0x48, 0x61, 0x76, 0x61, 0x6e, 0x6e, 0x61, 0x42, 0x65, 0x75, 0x6c,
0x61, 0x68, 0x2c, 0x20, 0x50, 0xf5, 0x68, 0x6a, 0x61, 0x2d, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20,
0x50, 0xf5, 0x68, 0x6a, 0x61, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x50,
0xf5, 0x68, 0x6a, 0x61, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0xf5, 0x41, 0x6e, 0x61, 0x64, 0xf5, 0x72, 0x41,
0x6b, 0x74, 0xf6, 0x62, 0x65, 0x41, 0x74, 0xf5, 0x72, 0x61, 0x75, 0x42,
0x61, 0x6b, 0x75, 0x75, 0x54, 0x161, 0x69, 0x74, 0x61, 0x54, 0x161, 0x6f,
0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x4a, 0x65, 0x72, 0x75, 0x75,
0x73, 0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x4b, 0x61, 0x6d, 0x74, 0x161, 0x61,
0x74, 0x6b, 0x61, 0x48, 0x61, 0x6e, 0x64, 0xf5, 0x67, 0x61, 0x4b, 0x75,
0x76, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x73, 0x71, 0x61, 0x163, 0x4b, 0xf5,
0x7a, 0xf5, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x41, 0x72, 0x2d, 0x52, 0x69,
0x79, 0x101, 0x1e11, 0x53, 0x61, 0x68, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53,
0x72, 0x65, 0x64, 0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0xf5, 0x6d, 0x73, 0x6b,
0x54, 0x68, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x54, 0x14d, 0x6b, 0x79,
0x14d, 0x41, 0x73, 0x73, 0x6f, 0x6f, 0x72, 0x69, 0x64, 0x4b, 0x61, 0x6e,
0x61, 0x61, 0x72, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x65, 0x64, 0x52,
0x6f, 0x68, 0x65, 0x6e, 0x65, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x61, 0x72,
0x65, 0x64, 0x46, 0xe4, 0xe4, 0x72, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72,
0x65, 0x64, 0x4c, 0xf5, 0x75, 0x6e, 0x61, 0x2d, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x41, 0x74, 0x65, 0x65, 0x6e, 0x61, 0x42, 0x65, 0x72,
0x6c, 0x69, 0x69, 0x6e, 0x43, 0x68, 0x69, 0x219, 0x69, 0x6e, 0x103, 0x75,
0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x61, 0x67, 0x65, 0x6e, 0x48,
0x65, 0x6c, 0x73, 0x69, 0x6e, 0x67, 0x69, 0x4d, 0x61, 0x6e, 0x69, 0x20,
0x73, 0x61, 0x61, 0x72, 0x4b, 0x69, 0x69, 0x65, 0x76, 0x4d, 0x61, 0x61,
0x72, 0x69, 0x61, 0x6e, 0x68, 0x61, 0x6d, 0x69, 0x6e, 0x61, 0x50, 0x61,
0x72, 0x69, 0x69, 0x73, 0x52, 0x69, 0x69, 0x61, 0x52, 0x6f, 0x6f, 0x6d,
0x61, 0x56, 0x69, 0x69, 0x6e, 0x56, 0x61, 0x72, 0x73, 0x73, 0x61, 0x76,
0x69, 0x4a, 0xf5, 0x75, 0x6c, 0x75, 0x73, 0x61, 0x61, 0x72, 0x4b, 0x6f,
0x6f, 0x6b, 0x6f, 0x73, 0x73, 0x61, 0x61, 0x72, 0x65, 0x64, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x69, 0x76, 0x69, 0x64, 0x4c, 0x69, 0x68, 0x61, 0x76,
0xf5, 0x74, 0x74, 0x65, 0x73, 0x61, 0x61, 0x72, 0x4d, 0x61, 0x72, 0x6b,
0x69, 0x69, 0x73, 0x61, 0x61, 0x72, 0x65, 0x64, 0x42, 0x65, 0x6c, 0x61,
0x75, 0x4b, 0x61, 0x74, 0x68, 0x6d, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d,
0x65, 0x42, 0x65, 0x6c, 0x69, 0x73, 0x42, 0x61, 0x72, 0x65, 0x69, 0x6e,
0x47, 0x61, 0x73, 0x61, 0x4b, 0x75, 0x76, 0x61, 0x69, 0x74, 0x41, 0x7a,
0x6f, 0x72, 0x75, 0x72, 0x6e, 0x61, 0x72, 0x47, 0x72, 0xf8, 0x6e, 0x68,
0xf8, 0x76, 0x64, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x46,
0xf8, 0x72, 0x6f, 0x79, 0x61, 0x72, 0x53, 0x75, 0xf0, 0x75, 0x72, 0x67,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61,
0x72, 0x4b, 0x65, 0x79, 0x70, 0x6d, 0x61, 0x6e, 0x6e, 0x61, 0x68, 0x61,
0x76, 0x6e, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x6f, 0x72, 0x67,
0x53, 0x74, 0x6f, 0x6b, 0x6b, 0x68, 0xf3, 0x6c, 0x6d, 0x56, 0x61, 0x74,
0x69, 0x6b, 0x61, 0x6e, 0x69, 0xf0, 0x56, 0x61, 0x72, 0x73, 0x6a, 0x61,
0x76, 0x61, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x6f, 0x79, 0x67, 0x67,
0x6a, 0x61, 0x72, 0x4d, 0xf3, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4c,
0x75, 0x6e, 0x67, 0x73, 0x6f, 0x64, 0x20, 0x6e, 0x67, 0x20, 0x4a, 0x75,
0xe1, 0x72, 0x65, 0x7a, 0x4c, 0x75, 0x6e, 0x67, 0x73, 0x6f, 0x64, 0x20,
0x6e, 0x67, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x50, 0x75, 0x77,
0x65, 0x72, 0x74, 0x6f, 0x20, 0x6e, 0x67, 0x20, 0x45, 0x73, 0x70, 0x61,
0x6e, 0x79, 0x61, 0x4d, 0x61, 0x6b, 0x69, 0x70, 0x6f, 0x74, 0x20, 0x6e,
0x61, 0x20, 0x4c, 0x6f, 0x6f, 0x6b, 0x20, 0x6e, 0x67, 0x20, 0x52, 0x61,
0x6e, 0x6b, 0x69, 0x6e, 0x4c, 0x75, 0x6e, 0x67, 0x73, 0x6f, 0x64, 0x20,
0x6e, 0x67, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69,
0x6e, 0x68, 0x45, 0x6c, 0x20, 0x41, 0x61, 0x69, 0xfa, 0x6e, 0x42, 0x65,
0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x50, 0x6f, 0x68, 0x6a, 0x6f, 0x69,
0x73, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x2c, 0x20, 0x50, 0x6f, 0x68, 0x6a, 0x6f, 0x69, 0x73, 0x2d,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x50, 0x6f, 0x68, 0x6a, 0x6f, 0x69, 0x73,
0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x69,
0x61, 0x67, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x4d, 0x61, 0x63, 0x71, 0x75, 0x61, 0x72, 0x69, 0x65, 0x6e, 0x73, 0x61,
0x61, 0x72, 0x69, 0x41, 0x71, 0x74, 0x61, 0x77, 0x54, 0x161, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x6f,
0x73, 0x48, 0x1ed3, 0x20, 0x43, 0x68, 0xed, 0x20, 0x4d, 0x69, 0x6e, 0x68,
0x69, 0x6e, 0x20, 0x6b, 0x61, 0x75, 0x70, 0x75, 0x6e, 0x6b, 0x69, 0x48,
0x61, 0x6e, 0x64, 0x79, 0x67, 0x61, 0x4b, 0x61, 0x6c, 0x6b, 0x75, 0x74,
0x74, 0x61, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x69, 0x41, 0x7a,
0x6f, 0x72, 0x69, 0x74, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x61, 0x6e,
0x73, 0x61, 0x61, 0x72, 0x65, 0x74, 0x46, 0xe4, 0x72, 0x73, 0x61, 0x61,
0x72, 0x65, 0x74, 0x45, 0x74, 0x65, 0x6c, 0xe4, 0x2d, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x42, 0x65, 0x72, 0x6c, 0x69, 0x69, 0x6e, 0x69,
0x42, 0x72, 0x79, 0x73, 0x73, 0x65, 0x6c, 0x43, 0x68, 0x69, 0x15f, 0x69,
0x6e, 0x103, 0x75, 0x4b, 0xf6, 0xf6, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x6d,
0x69, 0x6e, 0x61, 0x4d, 0x61, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x69, 0x4b,
0x69, 0x6f, 0x76, 0x61, 0x4c, 0x6f, 0x6e, 0x74, 0x6f, 0x6f, 0x4d, 0x6f,
0x73, 0x6b, 0x6f, 0x76, 0x61, 0x50, 0x61, 0x72, 0x69, 0x69, 0x73, 0x69,
0x52, 0x69, 0x69, 0x6b, 0x61, 0x54, 0x75, 0x6b, 0x68, 0x6f, 0x6c, 0x6d,
0x61, 0x54, 0x61, 0x6c, 0x6c, 0x69, 0x6e, 0x6e, 0x61, 0x56, 0x61, 0x74,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x69, 0x56, 0x69, 0x6c, 0x6e, 0x61, 0x56,
0x61, 0x72, 0x73, 0x6f, 0x76, 0x61, 0x4a, 0x6f, 0x75, 0x6c, 0x75, 0x73,
0x61, 0x61, 0x72, 0x69, 0x4b, 0x6f, 0x6f, 0x6b, 0x6f, 0x73, 0x73, 0x61,
0x61, 0x72, 0x65, 0x74, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x69, 0x74, 0x4b,
0x65, 0x72, 0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e, 0x73, 0x61, 0x61, 0x72,
0x65, 0x74, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x69, 0x76, 0x69, 0x74,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x61, 0x61, 0x72, 0x65,
0x74, 0x50, 0xe4, 0xe4, 0x73, 0x69, 0xe4, 0x69, 0x73, 0x73, 0x61, 0x61,
0x72, 0x69, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x73, 0x61, 0x61,
0x72, 0x65, 0x74, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73,
0x73, 0x61, 0x61, 0x72, 0x65, 0x74, 0x4d, 0x69, 0x64, 0x77, 0x61, 0x79,
0x73, 0x61, 0x61, 0x72, 0x65, 0x74, 0x41, 0x64, 0x64, 0x69, 0x73, 0x2d,
0x41, 0x62, 0x65, 0x62, 0x61, 0x4c, 0x65, 0x20, 0x43, 0x61, 0x69, 0x72,
0x65, 0x4c, 0x61, 0xe2, 0x79, 0x6f, 0x75, 0x6e, 0x65, 0x54, 0x72, 0x69,
0x70, 0x6f, 0x6c, 0x69, 0x20, 0x28, 0x4c, 0x69, 0x62, 0x79, 0x65, 0x29,
0x55, 0x73, 0x68, 0x75, 0x61, 0xef, 0x61, 0x42, 0x61, 0x68, 0x69, 0x61,
0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73,
0x4c, 0x61, 0x20, 0x42, 0x61, 0x72, 0x62, 0x61, 0x64, 0x65, 0x43, 0x61,
0xef, 0x6d, 0x61, 0x6e, 0x73, 0x44, 0xe9, 0x74, 0x72, 0x6f, 0x69, 0x74,
0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x71, 0x75, 0x65, 0x47, 0x72, 0x65,
0x6e, 0x61, 0x64, 0x65, 0x4c, 0x61, 0x20, 0x48, 0x61, 0x76, 0x61, 0x6e,
0x65, 0x4b, 0x6e, 0x6f, 0x78, 0x20, 0x5b, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x5d, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x20, 0x5b,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x5d, 0x50, 0x65, 0x74, 0x65,
0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0x20, 0x5b, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x5d, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43, 0x69, 0x74,
0x79, 0x20, 0x5b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x5d, 0x56,
0x65, 0x76, 0x61, 0x79, 0x20, 0x5b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x5d, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x20,
0x5b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x5d, 0x57, 0x69, 0x6e,
0x61, 0x6d, 0x61, 0x63, 0x20, 0x5b, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x5d, 0x4a, 0x61, 0x6d, 0x61, 0xef, 0x71, 0x75, 0x65, 0x4d, 0x6f,
0x6e, 0x74, 0x69, 0x63, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x5b, 0x4b, 0x65,
0x6e, 0x74, 0x75, 0x63, 0x6b, 0x79, 0x5d, 0x4d, 0x61, 0x6e, 0x61, 0x6f,
0x73, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x20, 0x28, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x64, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x29,
0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x20, 0x28, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x64, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x29, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x20, 0x28, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64,
0x29, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x64, 0x2019, 0x45, 0x73, 0x70, 0x61,
0x67, 0x6e, 0x65, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x63,
0x6f, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x44, 0x6f, 0x6d, 0x69, 0x6e,
0x67, 0x75, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x4a, 0x65, 0x61,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x4e,
0x65, 0x75, 0x76, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x43, 0x68,
0x72, 0x69, 0x73, 0x74, 0x6f, 0x70, 0x68, 0x65, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x65, 0x2d, 0x4c, 0x75, 0x63, 0x69, 0x65, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x54, 0x68, 0x75,
0x6c, 0xe9, 0x53, 0x68, 0x6f, 0x77, 0x61, 0x41, 0x6b, 0x74, 0x61, 0x6f,
0x75, 0x41, 0x6b, 0x74, 0x69, 0x6f, 0x75, 0x62, 0x69, 0x6e, 0x73, 0x6b,
0x41, 0x63, 0x68, 0x67, 0x61, 0x62, 0x61, 0x74, 0x41, 0x74, 0x79, 0x72,
0x61, 0x6f, 0x75, 0x42, 0x61, 0x68, 0x72, 0x65, 0xef, 0x6e, 0x42, 0x65,
0x79, 0x72, 0x6f, 0x75, 0x74, 0x68, 0x42, 0x69, 0x63, 0x68, 0x6b, 0x65,
0x6b, 0x54, 0x63, 0x68, 0x6f, 0xef, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x44, 0x75, 0x62, 0x61, 0xef, 0x44, 0x6f, 0x75, 0x63, 0x68, 0x61, 0x6e,
0x62, 0xe9, 0x46, 0x61, 0x6d, 0x61, 0x67, 0x6f, 0x75, 0x73, 0x74, 0x65,
0x48, 0xe9, 0x62, 0x72, 0x6f, 0x6e, 0x48, 0xf4, 0x2d, 0x43, 0x68, 0x69,
0x2d, 0x4d, 0x69, 0x6e, 0x68, 0x2d, 0x56, 0x69, 0x6c, 0x6c, 0x65, 0x49,
0x72, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x4a, 0xe9, 0x72, 0x75, 0x73,
0x61, 0x6c, 0x65, 0x6d, 0x4b, 0x61, 0x74, 0x6d, 0x61, 0x6e, 0x64, 0x6f,
0x75, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0xef, 0x61, 0x72, 0x73, 0x6b,
0x4b, 0x6f, 0x77, 0x65, 0xef, 0x74, 0x4d, 0x61, 0x63, 0x61, 0x73, 0x73,
0x61, 0x72, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0x6c, 0x65, 0x4e, 0x69, 0x63,
0x6f, 0x73, 0x69, 0x65, 0x4f, 0x75, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x4b,
0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0xef, 0x4b, 0x7a, 0x79, 0x6c, 0x20,
0x4f, 0x72, 0x64, 0x61, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x65, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x63, 0x61, 0x6e, 0x64, 0x65, 0x53,
0xe9, 0x6f, 0x75, 0x6c, 0x54, 0x61, 0x63, 0x68, 0x6b, 0x65, 0x6e, 0x74,
0x54, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x73, 0x69, 0x54, 0xe9, 0x68, 0xe9,
0x72, 0x61, 0x6e, 0x4f, 0x75, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x49, 0x61, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x52, 0x61,
0x6e, 0x67, 0x6f, 0x75, 0x6e, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0xce, 0x6c, 0x65, 0x73, 0x20, 0x43,
0x61, 0x6e, 0x61, 0x72, 0x69, 0x65, 0x73, 0x43, 0x61, 0x70, 0x2d, 0x56,
0x65, 0x72, 0x74, 0xce, 0x6c, 0x65, 0x73, 0x20, 0x46, 0xe9, 0x72, 0x6f,
0xe9, 0x4d, 0x61, 0x64, 0xe8, 0x72, 0x65, 0x47, 0xe9, 0x6f, 0x72, 0x67,
0x69, 0x65, 0x20, 0x64, 0x75, 0x20, 0x53, 0x75, 0x64, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x65, 0x2d, 0x48, 0xe9, 0x6c, 0xe8, 0x6e, 0x65, 0x41, 0x64,
0xe9, 0x6c, 0x61, 0xef, 0x64, 0x65, 0x41, 0x6e, 0x64, 0x6f, 0x72, 0x72,
0x65, 0x41, 0x74, 0x68, 0xe8, 0x6e, 0x65, 0x73, 0xce, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x69, 0x73, 0x62, 0x6f, 0x6e,
0x6e, 0x65, 0x4d, 0x61, 0x6c, 0x74, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x4f, 0x75, 0x6c, 0x69, 0x61, 0x6e,
0x6f, 0x76, 0x73, 0x6b, 0x4c, 0x65, 0x20, 0x56, 0x61, 0x74, 0x69, 0x63,
0x61, 0x6e, 0x56, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x56, 0x61, 0x72, 0x73,
0x6f, 0x76, 0x69, 0x65, 0x43, 0x6f, 0x6d, 0x6f, 0x72, 0x65, 0x73, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x63, 0x65, 0x4c, 0x61, 0x20, 0x52, 0xe9, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0xce, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50,
0xe2, 0x71, 0x75, 0x65, 0x73, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x69, 0x73,
0x65, 0x73, 0x50, 0x61, 0x6c, 0x61, 0x6f, 0x73, 0x4e, 0x64, 0x6a, 0x61,
0x6d, 0x65, 0x6e, 0x61, 0x54, 0x72, 0x69, 0x70, 0x6f, 0x6c, 0x69, 0x20,
0x5b, 0x4c, 0x69, 0x62, 0x79, 0x65, 0x5d, 0x42, 0x61, 0x72, 0x62, 0x61,
0x64, 0x65, 0x20, 0x28, 0x4c, 0x61, 0x29, 0xee, 0x6c, 0x65, 0x73, 0x20,
0x43, 0x61, 0xef, 0x6d, 0x61, 0x6e, 0x73, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x20, 0x5b, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x75,
0x20, 0x4e, 0x6f, 0x72, 0x64, 0x5d, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72,
0x20, 0x5b, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x75, 0x20,
0x4e, 0x6f, 0x72, 0x64, 0x5d, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64,
0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x70, 0x68, 0x65, 0x2d, 0x65,
0x74, 0x2d, 0x4e, 0x69, 0xe9, 0x76, 0xe8, 0x73, 0x54, 0xe9, 0x67, 0x75,
0x63, 0x69, 0x67, 0x61, 0x6c, 0x70, 0x61, 0x44, 0x61, 0x63, 0x63, 0x61,
0x54, 0x68, 0x69, 0x6d, 0x70, 0x68, 0x6f, 0x75, 0xee, 0x6c, 0x65, 0x73,
0x20, 0x46, 0xe9, 0x72, 0x6f, 0xe9, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x6e, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50,
0xe2, 0x71, 0x75, 0x65, 0x73, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x47, 0x6e,
0x6f, 0x76, 0x65, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x53, 0x61, 0x6e, 0x20,
0x50, 0x61, 0x75, 0x6c, 0x69, 0x20, 0x64, 0x61, 0x6c, 0x20, 0x42, 0x72,
0x61, 0x73, 0xee, 0x6c, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x73, 0x43, 0x61,
0x6e, 0x61, 0x72, 0x69, 0x69, 0x73, 0x42, 0x65, 0x6c, 0x67, 0x72, 0xe2,
0x74, 0x4c, 0x69, 0x73, 0x62, 0x6f, 0x6e, 0x65, 0x4c, 0x75, 0x62, 0x69,
0x61, 0x6e, 0x65, 0x4c, 0x75, 0x73, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72,
0x63, 0x4d, 0x6f, 0x73, 0x63, 0x68, 0x65, 0x50, 0x72, 0x61, 0x67, 0x68,
0x65, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x56, 0x69,
0x65, 0x6e, 0x65, 0xd83a, 0xdd00, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a,
0xdd22, 0xd83a, 0xdd00, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a,
0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a,
0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd00, 0xd83a,
0xdd27, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd04, 0xd83a,
0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd04, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd04, 0xd83a,
0xdd2d, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd04, 0xd83a,
0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd41, 0xd83a,
0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a,
0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a,
0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a,
0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd33, 0xd83a,
0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x2d, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd14, 0xd83a, 0xdd2d, 0xd83a, 0xdd26,
0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd01, 0xd83a, 0xdd35,
0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd02, 0xd83a, 0xdd22,
0xd83a, 0xdd34, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd0a,
0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd18,
0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd32, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd38, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd14, 0xd83a, 0xdd35, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd28, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd1d,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd25,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd27,
0xd83a, 0xdd22, 0xd83a, 0xdd02, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd27, 0xd83a, 0xdd02, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd02,
0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd02, 0xd83a, 0xdd35, 0xd83a, 0xdd31,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd02,
0xd83a, 0xdd35, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd22,
0xd83a, 0xdd43, 0xd83a, 0xdd2d, 0xd83a, 0xdd02, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd22,
0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd28,
0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd27,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd10, 0xd83a, 0xdd4b, 0xd83a, 0xdd04,
0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd2b,
0xd83a, 0xdd03, 0xd83a, 0xdd35, 0xd83a, 0xdd39, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd27,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd10, 0xd83a, 0xdd22,
0xd83a, 0xdd34, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd10,
0xd83a, 0xdd4b, 0xd83a, 0xdd14, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd10, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd25,
0xd83a, 0xdd2b, 0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd33, 0xd83a, 0xdd43, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd0f,
0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd34, 0xd83a, 0xdd3a,
0xd83a, 0xdd35, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e,
0x2d, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e,
0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd1a, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0xd83a, 0xdd28, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd1a, 0xd83a, 0xdd35,
0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd0f, 0xd83a, 0xdd2d,
0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd00,
0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd27,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd36, 0xd83a, 0xdd35,
0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a,
0xd83a, 0xdd22, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd35,
0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd04, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0x2d, 0xd83a, 0xdd09, 0xd83a,
0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a,
0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd14, 0xd83a, 0xdd35, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd34, 0xd83a,
0xdd02, 0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e,
0xd83a, 0xdd38, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd08, 0xd83a, 0xdd2d,
0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a,
0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0x2d, 0xd83a, 0xdd1d,
0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd05,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0x2d, 0xd83a, 0xdd02, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a,
0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd13, 0xd83a,
0xdd27, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a,
0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a,
0xdd00, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a,
0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd04, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd2e,
0xd83a, 0xdd27, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd25,
0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd41,
0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0x20, 0xd83a,
0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd04, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd1c,
0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd04, 0xd83a, 0xdd2e,
0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd04, 0xd83a, 0xdd2e,
0xd83a, 0xdd34, 0xd83a, 0xdd41, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd36, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a,
0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd28, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd18, 0xd83a, 0xdd2a,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd27,
0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd15, 0xd83a, 0xdd0b, 0xd83a, 0xdd31,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0x20, 0xd83a, 0xdd08, 0xd83a,
0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd35, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd43, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3e, 0xd83a, 0xdd32, 0xd83a, 0xdd01, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd01, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0x2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3e,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd01, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2a,
0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd3c, 0xd83a, 0xdd01, 0xd83a, 0xdd2e, 0xd83a, 0xdd25,
0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd09, 0xd83a, 0xdd23, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd09, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd2b, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd09, 0xd83a, 0xdd24, 0x2d, 0xd83a,
0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd3e, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd3c, 0x2d, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd27, 0xd83a, 0xdd2e,
0xd83a, 0xdd32, 0x3b, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd41, 0xd83a, 0xdd22, 0xd83a,
0xdd18, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0x2d, 0xd83a, 0xdd04, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd18, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0x2d, 0xd83a,
0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd18, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0x2d, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd33,
0xd83a, 0xdd18, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd28, 0xd83a, 0xdd46,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22,
0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2d,
0xd83a, 0xdd24, 0xd83a, 0xdd18, 0xd83a, 0xdd35, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d,
0xd83a, 0xdd2c, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd28, 0xd83a, 0xdd2e, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd27,
0xd83a, 0xdd35, 0x2c, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd03, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0x2c, 0x20,
0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd3a, 0x2c, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd1a,
0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0x2d, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0x2c, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0x2c,
0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd1c, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0x2c, 0x20, 0xd83a,
0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd0f, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0x2c, 0x20, 0xd83a, 0xdd0b, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd3e, 0xd83a,
0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd0b, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a,
0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd14, 0xd83a, 0xdd22, 0xd83a,
0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a,
0xdd14, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd02, 0xd83a, 0xdd35, 0xd83a, 0xdd2d, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a,
0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd37, 0xd83a,
0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x2c, 0x20, 0xd83a, 0xdd11, 0xd83a,
0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a,
0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a,
0xdd02, 0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd41,
0xd83a, 0xdd02, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd02,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a,
0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a,
0xdd31, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0x20, 0xd83a, 0xdd11, 0xd83a,
0xdd2e, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a,
0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd03, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd03, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a,
0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a,
0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a,
0xdd25, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a,
0xdd3c, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a,
0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd2d,
0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd33,
0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd2e,
0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd32,
0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd34, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd3c, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd34,
0xd83a, 0xdd3c, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd12, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd25, 0xd83a,
0xdd35, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0x2c, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0x2d,
0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0x2c, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0x2d, 0xd83a,
0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd05, 0xd83a, 0xdd2b,
0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd25, 0x2c, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e,
0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0x2d, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a,
0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a,
0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd0c, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd06, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a,
0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd3c, 0x2d, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a,
0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0x20,
0xd83a, 0xdd2e, 0xd83a, 0xdd2c, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd3c, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd38,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c,
0x2d, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd06, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd00,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd08,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0x20, 0xd83a,
0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd3c, 0xd83a, 0xdd08, 0xd83a,
0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd2c, 0xd83a, 0xdd2d, 0xd83a, 0xdd08, 0xd83a,
0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a,
0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0x2d,
0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2e,
0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0x2d, 0xd83a, 0xdd01, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a,
0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd31, 0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd0b, 0xd83a, 0xdd3c, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd2e,
0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd25,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c,
0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c,
0x2d, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a,
0xdd2b, 0xd83a, 0xdd25, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a,
0xdd32, 0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd32,
0xd83a, 0xdd27, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0x2d, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd46, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd02, 0xd83a, 0xdd35,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd3c, 0x2d, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0x2d,
0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd3c, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd2c, 0xd83a, 0xdd3c, 0x2d, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd3c, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd28, 0xd83a, 0xdd22, 0xd83a,
0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd1a, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd1c, 0xd83a,
0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a,
0xdd2e, 0xd83a, 0xdd0f, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd3c, 0xd83a, 0xdd38, 0xd83a,
0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd0f, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd3a, 0xd83a,
0xdd12, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd3c, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a,
0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a,
0xdd01, 0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a,
0xdd35, 0x2d, 0xd83a, 0xdd01, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d,
0xd83a, 0xdd24, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd27,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd25,
0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd06,
0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd08, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd35, 0xd83a, 0xdd31,
0xd83a, 0xdd22, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd24, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e,
0xd83a, 0xdd33, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd36, 0xd83a, 0xdd2d,
0xd83a, 0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd00,
0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd35,
0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd31, 0xd83a, 0xdd35, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd00, 0xd83a, 0xdd27, 0xd83a, 0xdd3a,
0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd00,
0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd35,
0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2a,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd04,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd24,
0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd2d, 0xd83a, 0xdd27,
0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd04,
0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd15,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd15, 0xd83a, 0xdd2e,
0xd83a, 0xdd34, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd27, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd25,
0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22,
0xd83a, 0xdd27, 0xd83a, 0xdd39, 0xd83a, 0xdd22, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd01, 0xd83a, 0xdd35, 0xd83a, 0xdd26, 0xd83a, 0xdd22,
0xd83a, 0xdd34, 0xd83a, 0xdd01, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd0a, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22,
0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd18,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd1d, 0xd83a, 0xdd2d,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd05,
0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0x2d, 0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd03, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd3a, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd16,
0xd83a, 0xdd2e, 0xd83a, 0xdd2c, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a,
0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33,
0xd83a, 0xdd35, 0xd83a, 0xdd14, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd2a,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd14, 0xd83a, 0xdd22, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd17,
0xd83a, 0xdd35, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd11,
0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd11, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd37, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd33,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd37,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd1d,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd22,
0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27,
0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27,
0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd31,
0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a,
0xdd25, 0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a,
0xdd35, 0xd83a, 0xdd37, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a,
0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd03, 0xd83a,
0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd03, 0xd83a,
0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a,
0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd10, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a,
0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd10, 0xd83a,
0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd41, 0xd83a,
0xdd32, 0xd83a, 0xdd2b, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a,
0xdd35, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a,
0xdd33, 0xd83a, 0xdd0c, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a,
0xdd35, 0xd83a, 0xdd13, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd06, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd06,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd17, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a,
0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd41, 0xd83a, 0xdd2d,
0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd08,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd05,
0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd32, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a,
0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd05,
0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd05, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd38, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd05, 0xd83a, 0xdd2d,
0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd23,
0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd27,
0xd83a, 0xdd35, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd28, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd1a, 0xd83a, 0xdd2d, 0xd83a, 0xdd26,
0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd1a, 0xd83a, 0xdd2b, 0xd83a, 0xdd38, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd1a, 0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd28,
0xd83a, 0xdd35, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd2e, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd35,
0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd13, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd13,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd13,
0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0x2d, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd1c, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a,
0xdd32, 0xd83a, 0xdd37, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd1c, 0xd83a,
0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd3e, 0xd83a,
0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a,
0xdd35, 0xd83a, 0xdd12, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a,
0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd08, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd12, 0xd83a,
0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a,
0xdd35, 0xd83a, 0xdd12, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd3e, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a,
0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd1c,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd0a, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd34,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd08, 0xd83a, 0xdd2b,
0xd83a, 0xdd34, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd33,
0xd83a, 0xdd2d, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2c, 0x2d, 0xd83a,
0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0x2d,
0xd83a, 0xdd16, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd00, 0xd83a, 0xdd23, 0xd83a, 0xdd22,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd04,
0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd26, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0x2d, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd01, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a,
0xdd31, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd13, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a,
0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd16, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a,
0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd02, 0xd83a,
0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd23, 0xd83a, 0xdd35, 0x2d, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd2b,
0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd32,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd25, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd00, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd00,
0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd04,
0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd04, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd24,
0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd3e,
0xd83a, 0xdd22, 0xd83a, 0xdd04, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33,
0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd33,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd2b,
0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd32, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd15, 0xd83a, 0xdd2d,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd31,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a, 0xdd32,
0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3a, 0xd83a, 0xdd01, 0xd83a, 0xdd35,
0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd14,
0xd83a, 0xdd2d, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd14, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a,
0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd16,
0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd33,
0xd83a, 0xdd2d, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x2d, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a,
0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd14, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a,
0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a,
0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a,
0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd3e, 0xd83a, 0xdd02, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd0b, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd26, 0xd83a,
0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd02, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd03, 0xd83a,
0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a,
0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd03, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a,
0xdd2d, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a,
0xdd2e, 0xd83a, 0xdd38, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a,
0xdd2e, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd38, 0xd83a,
0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a,
0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd08, 0xd83a, 0xdd2d, 0xd83a, 0xdd3a, 0xd83a,
0xdd22, 0xd83a, 0xdd08, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a,
0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0x2d, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd2a,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd2c,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd28, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd24,
0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd28,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd2e, 0xd83a, 0xdd2c,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd35,
0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32,
0xd83a, 0xdd35, 0xd83a, 0xdd1a, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd13, 0xd83a, 0xdd24, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0xd83a, 0xdd1c, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd36,
0xd83a, 0xdd35, 0xd83a, 0xdd1c, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd33,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd1c, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd24,
0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e,
0xd83a, 0xdd24, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a,
0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd0f, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27,
0xd83a, 0xdd2e, 0xd83a, 0xdd1f, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd1f, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd33, 0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd3a,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd11,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd11,
0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd11, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd24,
0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd23, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd03, 0xd83a, 0xdd2e,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd03,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd3c, 0xd83a, 0xdd35,
0xd83a, 0xdd08, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd28, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd0c, 0xd83a, 0xdd33, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd24,
0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd43, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25,
0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd0b,
0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd09, 0xd83a, 0xdd2c,
0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd0a, 0xd83a, 0xdd22, 0xd83a, 0xdd33,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2c, 0xd83a, 0xdd2e, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd2c, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd28, 0xd83a, 0xdd22,
0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd25,
0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd18,
0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd18,
0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd09,
0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd35,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd3c,
0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd36,
0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd03,
0xd83a, 0xdd22, 0xd83a, 0xdd36, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd27,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd23,
0xd83a, 0xdd31, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd31,
0xd83a, 0xdd2d, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0xd83a, 0xdd2e,
0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd25,
0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd3a,
0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a,
0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd06, 0xd83a,
0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd28, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a,
0xdd35, 0x2d, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd27,
0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd08, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e,
0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd05,
0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd28, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd1a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd28, 0xd83a, 0xdd35,
0xd83a, 0xdd0f, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd0f,
0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd46, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27,
0x41, 0x64, 0x64, 0x69, 0x73, 0x20, 0x41, 0x62, 0xe4, 0x62, 0x61, 0x41,
0x73, 0x6d, 0x61, 0x72, 0xe0, 0x44, 0xe0, 0x72, 0x20, 0x61, 0x73, 0x2d,
0x53, 0x61, 0x6c, 0xe0, 0x6d, 0x44, 0x69, 0x6f, 0x62, 0xf9, 0x74, 0x61,
0x69, 0x64, 0x68, 0x48, 0x61, 0x6e, 0x6e, 0x73, 0x61, 0x62, 0x6f, 0x72,
0x67, 0x68, 0x4e, 0x2019, 0x44, 0x6a, 0x61, 0x6d, 0xe9, 0x6e, 0x61, 0x41,
0x6e, 0x67, 0x75, 0x69, 0x6c, 0x6c, 0x69, 0x61, 0x41, 0x69, 0x6e, 0x74,
0xec, 0x6f, 0x67, 0x61, 0x41, 0x72, 0xf9, 0x62, 0x61, 0x41, 0x2019, 0x20,
0x42, 0x68, 0x65, 0x69, 0x6c, 0xec, 0x73, 0x43, 0x61, 0x69, 0x6d, 0x65,
0x61, 0x6e, 0x43, 0x6f, 0x73, 0x74, 0x61, 0x20, 0x52, 0xec, 0x63, 0x65,
0x61, 0x44, 0x6f, 0x69, 0x6d, 0x69, 0x6e, 0x69, 0x63, 0x65, 0x61, 0x41,
0x6e, 0x20, 0x53, 0x61, 0x6c, 0x62, 0x68, 0x61, 0x64, 0x6f, 0x72, 0x47,
0x6c, 0x61, 0x73, 0x62, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6e, 0x20, 0x54,
0x75, 0x72, 0x63, 0x20, 0x4d, 0x68, 0xf2, 0x72, 0x47, 0x72, 0x65, 0x61,
0x6e, 0xe0, 0x64, 0x61, 0x47, 0x75, 0x61, 0x74, 0x61, 0x6d, 0x61, 0x6c,
0x61, 0x47, 0x75, 0x69, 0x64, 0x68, 0x65, 0xe0, 0x6e, 0x61, 0x49, 0x6e,
0x75, 0x75, 0x76, 0x69, 0x6b, 0x44, 0x69, 0x61, 0x6d, 0x65, 0x75, 0x67,
0x61, 0x4d, 0x61, 0x69, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x63, 0x43, 0x61,
0x74, 0x68, 0x61, 0x69, 0x72, 0x20, 0x4d, 0x68, 0x65, 0x61, 0x67, 0x73,
0x61, 0x67, 0x6f, 0x4d, 0x6f, 0x6e, 0x74, 0x73, 0x61, 0x72, 0x61, 0x74,
0x4e, 0x75, 0x61, 0x64, 0x68, 0x20, 0x45, 0x61, 0x62, 0x68, 0x72, 0x61,
0x63, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x70, 0xe0,
0x69, 0x6e, 0x6e, 0x65, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x52, 0xec,
0x63, 0x65, 0x6f, 0x4b, 0x61, 0x6e, 0x67, 0x69, 0x71, 0x6c, 0x69, 0x6e,
0x69, 0x71, 0x51, 0x61, 0x75, 0x73, 0x75, 0x69, 0x74, 0x74, 0x75, 0x71,
0x4e, 0x61, 0x6f, 0x6d, 0x68, 0x20, 0x43, 0x72, 0xec, 0x73, 0x74, 0x65,
0x61, 0x6e, 0x4e, 0x61, 0x6f, 0x6d, 0x68, 0x20, 0x4c, 0xf9, 0x69, 0x73,
0x65, 0x61, 0x4e, 0x61, 0x6f, 0x6d, 0x68, 0x20, 0x42, 0x68, 0x69, 0x6f,
0x6e, 0x73, 0x61, 0x6e, 0x74, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20,
0x4d, 0x68, 0x69, 0x63, 0x47, 0x75, 0x61, 0x69, 0x72, 0x65, 0x41, 0x6d,
0x6d, 0x101, 0x6e, 0x42, 0x61, 0x67, 0x68, 0x64, 0x101, 0x64, 0x42, 0x61,
0x63, 0x68, 0x72, 0x61, 0x69, 0x6e, 0x42, 0x72, 0xf9, 0x6e, 0x61, 0x69,
0x67, 0x68, 0x47, 0xe0, 0x73, 0x61, 0x43, 0x61, 0x74, 0x68, 0x61, 0x69,
0x72, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e,
0x68, 0x49, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x4b, 0x61,
0x72, 0x101, 0x63, 0x12b, 0x43, 0x75, 0x69, 0x62, 0x68, 0xe8, 0x69, 0x74,
0x4d, 0x61, 0x63, 0xe0, 0x74, 0x68, 0x75, 0x50, 0x79, 0x65, 0x6f, 0x6e,
0x67, 0x79, 0x61, 0x6e, 0x67, 0x43, 0x61, 0x74, 0x61, 0x72, 0x51, 0x6f,
0x73, 0x74, 0x61, 0x6e, 0x61, 0x131, 0x53, 0x69, 0x6e, 0x67, 0x65, 0x61,
0x70, 0xf2, 0x72, 0x54, 0x6f, 0x161, 0x6b, 0x65, 0x6e, 0x74, 0x54, 0x2019,
0x62, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x54, 0x65, 0x68, 0x72, 0x101, 0x6e,
0x56, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x43, 0x68, 0x61, 0x6e, 0x50, 0x6f,
0x6e, 0x74, 0x61, 0x20, 0x44, 0x65, 0x6c, 0x67, 0x61, 0x64, 0x61, 0x42,
0x65, 0x61, 0x72, 0x6d, 0xf9, 0x64, 0x61, 0x4e, 0x61, 0x20, 0x68, 0x2d,
0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x43, 0x61, 0x6e,
0xe0, 0x72, 0x61, 0x63, 0x68, 0x41, 0x6e, 0x20, 0x43, 0x65, 0x61, 0x70,
0x20, 0x55, 0x61, 0x69, 0x6e, 0x65, 0x46, 0xe0, 0x72, 0x6f, 0x53, 0x65,
0xf2, 0x72, 0x73, 0x65, 0x61, 0x20, 0x61, 0x20, 0x44, 0x65, 0x61, 0x73,
0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x4e, 0x61, 0x6f, 0x6d, 0x68,
0x20, 0x45, 0x69, 0x6c, 0x69, 0x64, 0x68, 0x53, 0x69, 0x64, 0x6e, 0x69,
0x41, 0x6e, 0x20, 0xc0, 0x69, 0x74, 0x68, 0x6e, 0x65, 0x41, 0x2019, 0x20,
0x42, 0x68, 0x72, 0x75, 0x69, 0x73, 0x65, 0x61, 0x6c, 0x42, 0x65, 0x69,
0x72, 0x62, 0x68, 0x42, 0x61, 0x69, 0x6c, 0x65, 0x20, 0xc0, 0x74, 0x68,
0x61, 0x20, 0x43, 0x6c, 0x69, 0x61, 0x74, 0x68, 0x44, 0x69, 0x6f, 0x62,
0x72, 0x61, 0x6c, 0x74, 0x61, 0x72, 0x47, 0x65, 0xe0, 0x72, 0x6e, 0x73,
0x61, 0x69, 0x64, 0x68, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x4d,
0x68, 0x61, 0x6e, 0x61, 0x69, 0x6e, 0x6e, 0x44, 0x65, 0xe0, 0x72, 0x73,
0x61, 0x69, 0x64, 0x68, 0x44, 0xf9, 0x6e, 0x20, 0xc8, 0x69, 0x64, 0x65,
0x61, 0x6e, 0x6e, 0x2f, 0x4c, 0x75, 0x6e, 0x6e, 0x61, 0x69, 0x6e, 0x6e,
0x4c, 0x75, 0x67, 0x73, 0x61, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x4d, 0x6f,
0x73, 0x67, 0x6f, 0x50, 0x61, 0x72, 0x61, 0x73, 0x50, 0x72, 0xe0, 0x67,
0x52, 0x12b, 0x67, 0x61, 0x41, 0x6e, 0x20, 0x52, 0xf2, 0x69, 0x6d, 0x68,
0x41, 0x2019, 0x20, 0x42, 0x68, 0x61, 0x74, 0x61, 0x63, 0x61, 0x6e, 0x4e,
0x6f, 0x6c, 0x6c, 0x61, 0x69, 0x67, 0x4e, 0x61, 0x20, 0x68, 0x2d, 0x45,
0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x61, 0x6c,
0x61, 0x64, 0x61, 0x69, 0x62, 0x68, 0x4e, 0x61, 0x20, 0x68, 0x2d, 0x45,
0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x6f, 0x69,
0x72, 0x69, 0x73, 0x65, 0x61, 0x73, 0x52, 0x61, 0x70, 0x61, 0x20, 0x4e,
0x75, 0x69, 0x46, 0xec, 0x64, 0x69, 0x4d, 0x61, 0x6e, 0x67, 0x61, 0x72,
0x65, 0x76, 0x61, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20,
0x61, 0x2019, 0x20, 0x4d, 0x68, 0x61, 0x72, 0x63, 0x61, 0x69, 0x73, 0x4e,
0x61, 0x62, 0x68, 0x72, 0x75, 0x50, 0x61, 0x6c, 0x61, 0x62, 0x68, 0x50,
0x65, 0x69, 0x74, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0xe0, 0x69, 0x72,
0x6e, 0x55, 0x61, 0x6c, 0x6c, 0x61, 0x73, 0x4e, 0x6f, 0x75, 0x61, 0x6b,
0x73, 0x68, 0x6f, 0x74, 0x74, 0x41, 0x6e, 0x6b, 0x6f, 0x72, 0x61, 0x67,
0x65, 0x4b, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x67, 0x65, 0x20, 0x14a,
0x73, 0x68, 0x254, 0x6e, 0x69, 0x6e, 0x65, 0x20, 0x42, 0x69, 0x62, 0x69,
0x6f, 0x6f, 0x54, 0x73, 0x69, 0x6b, 0x61, 0x67, 0x6f, 0x54, 0x73, 0x69,
0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x20, 0x4b, 0x70, 0x61, 0x61, 0x6b, 0x70, 0x6f, 0x20, 0x42, 0x69, 0x62,
0x69, 0x6f, 0x6f, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x20, 0x4d, 0x254,
0x254, 0x47, 0x6c, 0x61, 0x73, 0x65, 0x20, 0x14a, 0x73, 0x68, 0x254, 0x6e,
0x69, 0x6e, 0x65, 0x20, 0x42, 0x69, 0x62, 0x69, 0x6f, 0x6f, 0x47, 0x6f,
0x6f, 0x73, 0x65, 0x20, 0x14a, 0x73, 0x68, 0x254, 0x6e, 0x69, 0x6e, 0x65,
0x20, 0x42, 0x69, 0x62, 0x69, 0x6f, 0x6f, 0x54, 0x75, 0x72, 0x6b, 0x65,
0x20, 0x57, 0x75, 0x6c, 0x75, 0x48, 0x61, 0x6c, 0x69, 0x66, 0x61, 0x73,
0x4b, 0x6e, 0x6f, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x4f, 0x73, 0x68, 0x65, 0x6b, 0x75, 0x20, 0x4d, 0x61, 0x14b, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x73,
0x65, 0x6e, 0x6e, 0x65, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73,
0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x6b,
0x79, 0x41, 0x62, 0x6c, 0x61, 0x64, 0x65, 0x20, 0x53, 0x68, 0x129, 0x61,
0x20, 0x4e, 0x69, 0x20, 0x59, 0x254, 0x254, 0x20, 0x4a, 0x254, 0x254, 0x20,
0x4d, 0x6c, 0x69, 0x4d, 0x65, 0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x4d, 0x61,
0x14b, 0x4d, 0x61, 0x14b, 0x74, 0x65, 0x14b, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x62, 0x25b,
0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x79, 0x69, 0x67,
0x62, 0x25b, 0x41, 0x62, 0x6c, 0x61, 0x64, 0x65, 0x69, 0x20, 0x41, 0x6c,
0x25b, 0x6a, 0x69, 0x61, 0x64, 0x61, 0x61, 0x6d, 0x254, 0x68, 0x65, 0x53,
0x70, 0x61, 0x69, 0x6e, 0x20, 0x4c, 0x25b, 0x6a, 0x69, 0x61, 0x64, 0x61,
0x61, 0x6d, 0x254, 0x68, 0x65, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20,
0x14a, 0x73, 0x68, 0x254, 0x6e, 0x69, 0x6e, 0x65, 0x49, 0x74, 0x74, 0x6f,
0x6b, 0x6f, 0x72, 0x74, 0x6f, 0x6f, 0x72, 0x6d, 0x69, 0x69, 0x74, 0x53,
0x74, 0x2e, 0x20, 0x4c, 0x75, 0x73, 0x69, 0x61, 0x53, 0x74, 0x2e, 0x20,
0x56, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x53, 0x77, 0x69, 0x66, 0x74,
0x20, 0x4b, 0x61, 0x72, 0x25b, 0x6e, 0x74, 0x56, 0x61, 0x6e, 0x6b, 0x6f,
0x75, 0x76, 0x65, 0x72, 0x4f, 0x6b, 0x70, 0x254, 0x14b, 0x254, 0x20, 0x59,
0x25b, 0x14b, 0x4b, 0x61, 0x73, 0x65, 0x79, 0x4d, 0x61, 0x6b, 0x77, 0x61,
0x72, 0x69, 0x65, 0x4d, 0x61, 0x6b, 0x4d, 0x75, 0x72, 0x64, 0x6f, 0x54,
0x73, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x48, 0x6f, 0x20,
0x54, 0x73, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x20, 0x4d, 0x61, 0x14b,
0x74, 0x69, 0x61, 0x73, 0x65, 0x59, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c,
0x65, 0x6d, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x73, 0x69, 0x4b, 0x75, 0x74,
0x73, 0x69, 0x6e, 0x67, 0x55, 0x72, 0x75, 0x6d, 0x6b, 0x69, 0x14a, 0x6d,
0x65, 0x14b, 0x6d, 0x65, 0x20, 0x4e, 0x69, 0x20, 0x45, 0x6b, 0x75, 0x6d,
0x254, 0x4e, 0x75, 0x14b, 0x74, 0x73, 0x254, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x54, 0x73, 0x61, 0x67, 0x6f, 0x73, 0x4b, 0x72, 0x69, 0x73, 0x6d, 0x61,
0x73, 0x186, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x54, 0x73, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x54, 0x73, 0x75, 0x75, 0x6b, 0x4d, 0x61, 0x72, 0x6b, 0x77,
0x65, 0x73, 0x61, 0x73, 0x50, 0x69, 0x74, 0x6b, 0x61, 0x69, 0x72, 0x6e,
0x41, 0x63, 0x72, 0x61, 0x41, 0x6c, 0x78, 0x65, 0x72, 0x42, 0x61, 0x6d,
0x61, 0x63, 0x6f, 0x4f, 0x20, 0x43, 0x61, 0x69, 0x72, 0x6f, 0x4f, 0x20,
0x41, 0x69, 0xfa, 0x6e, 0x58, 0x6f, 0x68, 0x61, 0x6e, 0x65, 0x73, 0x62,
0x75, 0x72, 0x67, 0x6f, 0x4b, 0x68, 0x61, 0x72, 0x74, 0xfa, 0x6e, 0x4c,
0x75, 0x73, 0x61, 0x63, 0x61, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x78,
0x6f, 0x53, 0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x54, 0x75, 0x6e,
0x65, 0x73, 0x42, 0x61, 0xed, 0x61, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20,
0x43, 0x61, 0x69, 0x6d, 0xe1, 0x6e, 0x4f, 0x20, 0x53, 0x61, 0x6c, 0x76,
0x61, 0x64, 0x6f, 0x72, 0x47, 0xfc, 0x69, 0x61, 0x6e, 0x61, 0x41, 0x20,
0x48, 0x61, 0x62, 0x61, 0x6e, 0x61, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0xe1, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x41, 0x20, 0x50, 0x61, 0x7a, 0x4f,
0x73, 0x20, 0xc1, 0x6e, 0x78, 0x65, 0x6c, 0x65, 0x73, 0x43, 0x69, 0x64,
0x61, 0x64, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63,
0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x63,
0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65,
0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x63, 0x6f,
0x74, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61,
0x63, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74,
0x65, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x50, 0x72, 0xed, 0x6e, 0x63,
0x69, 0x70, 0x65, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x45, 0x73, 0x70,
0x61, 0xf1, 0x61, 0x52, 0xed, 0x6f, 0x20, 0x42, 0x72, 0x61, 0x6e, 0x63,
0x6f, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75, 0x63, 0xed, 0x61,
0x53, 0x61, 0x6e, 0x20, 0x56, 0x69, 0x63, 0x65, 0x6e, 0x74, 0x65, 0x41,
0x64, 0xe9, 0x6e, 0x41, 0x63, 0x68, 0x6b, 0x68, 0x61, 0x62, 0x61, 0x64,
0x42, 0x61, 0x63, 0xfa, 0x43, 0x68, 0x69, 0x74, 0xe1, 0x44, 0x61, 0x6d,
0x61, 0x73, 0x63, 0x6f, 0x49, 0x61, 0x63, 0x61, 0x72, 0x74, 0x61, 0x43,
0x61, 0x62, 0x75, 0x6c, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x6f, 0x49, 0x65, 0x72, 0x65, 0x76, 0xe1, 0x6e,
0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64,
0x61, 0x73, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x61, 0x6e, 0x61,
0x72, 0x69, 0x61, 0x73, 0x52, 0x65, 0x69, 0x71, 0x75, 0x69, 0x61, 0x76,
0x69, 0x6b, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x69, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x53, 0x75, 0x72, 0xc1, 0x6d, 0x73, 0x74, 0x65, 0x72, 0x64, 0x61,
0x6d, 0x41, 0x73, 0x74, 0x72, 0x61, 0x6b, 0xe1, 0x6e, 0x42, 0x72, 0x75,
0x78, 0x65, 0x6c, 0x61, 0x73, 0x48, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x71,
0x75, 0x69, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x6e, 0x4c, 0x75, 0x78, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4d,
0xf3, 0x6e, 0x61, 0x63, 0x6f, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76, 0x61,
0x53, 0x61, 0x72, 0x61, 0x69, 0x65, 0x76, 0x6f, 0x53, 0x69, 0x6d, 0x66,
0x65, 0x72, 0xf3, 0x70, 0x6f, 0x6c, 0x45, 0x73, 0x74, 0x6f, 0x63, 0x6f,
0x6c, 0x6d, 0x6f, 0x56, 0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x49,
0x6c, 0x6c, 0x61, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61,
0x73, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x61, 0x73, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x63, 0x69, 0x6f, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x49, 0x6c, 0x6c, 0x61, 0x73,
0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x48, 0x6f,
0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0xfa, 0x10d0, 0x10d1, 0x10d8, 0x10ef, 0x10d0, 0x10dc,
0x10d8, 0x10d0, 0x10d9, 0x10e0, 0x10d0, 0x10d0, 0x10d3, 0x10d8, 0x10e1, 0x2d, 0x10d0, 0x10d1,
0x10d4, 0x10d1, 0x10d0, 0x10d0, 0x10da, 0x10df, 0x10d8, 0x10e0, 0x10d8, 0x10d0, 0x10e1, 0x10db,
0x10d0, 0x10e0, 0x10d0, 0x10d1, 0x10d0, 0x10db, 0x10d0, 0x10d9, 0x10dd, 0x10d1, 0x10d0, 0x10dc,
0x10d2, 0x10d8, 0x10d1, 0x10d0, 0x10dc, 0x10ef, 0x10e3, 0x10da, 0x10d8, 0x10d1, 0x10d8, 0x10e1,
0x10d0, 0x10e3, 0x10d1, 0x10da, 0x10d0, 0x10dc, 0x10e2, 0x10d8, 0x10e0, 0x10d8, 0x10d1, 0x10e0,
0x10d0, 0x10d6, 0x10d0, 0x10d5, 0x10d8, 0x10da, 0x10d8, 0x10d1, 0x10e3, 0x10ef, 0x10e3, 0x10db,
0x10d1, 0x10e3, 0x10e0, 0x10d0, 0x10d9, 0x10d0, 0x10d8, 0x10e0, 0x10dd, 0x10d9, 0x10d0, 0x10e1,
0x10d0, 0x10d1, 0x10da, 0x10d0, 0x10dc, 0x10d9, 0x10d0, 0x10e1, 0x10d4, 0x10e3, 0x10e2, 0x10d0,
0x10d9, 0x10dd, 0x10dc, 0x10d0, 0x10d9, 0x10e0, 0x10d8, 0x10d3, 0x10d0, 0x10d9, 0x10d0, 0x10e0,
0x10d8, 0x10d3, 0x10d0, 0x10e0, 0x2d, 0x10d4, 0x10e1, 0x2d, 0x10e1, 0x10d0, 0x10da, 0x10d0,
0x10db, 0x10d8, 0x10ef, 0x10d8, 0x10d1, 0x10e3, 0x10e2, 0x10d8, 0x10d3, 0x10e3, 0x10d0, 0x10da,
0x10d0, 0x10d4, 0x10da, 0x2d, 0x10d0, 0x10d0, 0x10d8, 0x10e3, 0x10dc, 0x10d8, 0x10e4, 0x10e0,
0x10d8, 0x10e2, 0x10d0, 0x10e3, 0x10dc, 0x10d8, 0x10d2, 0x10d0, 0x10d1, 0x10dd, 0x10e0, 0x10dd,
0x10dc, 0x10d4, 0x10f0, 0x10d0, 0x10e0, 0x10d0, 0x10e0, 0x10d4, 0x10d8, 0x10dd, 0x10f0, 0x10d0,
0x10dc, 0x10d4, 0x10e1, 0x10d1, 0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x10ef, 0x10e3, 0x10d1, 0x10d0,
0x10d9, 0x10d0, 0x10db, 0x10de, 0x10d0, 0x10da, 0x10d0, 0x10ee, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10db, 0x10d8, 0x10d9, 0x10d8, 0x10d2, 0x10d0, 0x10da, 0x10d8, 0x10d9, 0x10d8, 0x10dc, 0x10e8,
0x10d0, 0x10e1, 0x10d0, 0x10da, 0x10d0, 0x10d2, 0x10dd, 0x10e1, 0x10d8, 0x10da, 0x10d8, 0x10d1,
0x10e0, 0x10d4, 0x10d5, 0x10d8, 0x10da, 0x10d8, 0x10da, 0x10dd, 0x10db, 0x10d4, 0x10da, 0x10e3,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10da, 0x10e3, 0x10d1, 0x10e3, 0x10db, 0x10d1, 0x10d0, 0x10e8,
0x10d8, 0x10da, 0x10e3, 0x10e1, 0x10d0, 0x10d9, 0x10d0, 0x10db, 0x10d0, 0x10da, 0x10d0, 0x10d1,
0x10dd, 0x10db, 0x10d0, 0x10de, 0x10e3, 0x10e2, 0x10dd, 0x10db, 0x10d0, 0x10e1, 0x10d4, 0x10e0,
0x10e3, 0x10db, 0x10d1, 0x10d0, 0x10d1, 0x10d0, 0x10dc, 0x10d4, 0x10db, 0x10dd, 0x10d2, 0x10d0,
0x10d3, 0x10d8, 0x10e8, 0x10e3, 0x10db, 0x10dd, 0x10dc, 0x10e0, 0x10dd, 0x10d5, 0x10d8, 0x10d0,
0x10dc, 0x10d0, 0x10d8, 0x10e0, 0x10dd, 0x10d1, 0x10d8, 0x10dc, 0x10ef, 0x10d0, 0x10db, 0x10d4,
0x10dc, 0x10d0, 0x10dc, 0x10d8, 0x10d0, 0x10db, 0x10d4, 0x10d8, 0x10dc, 0x10e3, 0x10d0, 0x10d9,
0x10e8, 0x10dd, 0x10e2, 0x10d8, 0x10e3, 0x10d0, 0x10d2, 0x10d0, 0x10d3, 0x10e3, 0x10d2, 0x10e3,
0x10de, 0x10dd, 0x10e0, 0x10e2, 0x10dd, 0x2d, 0x10dc, 0x10dd, 0x10d5, 0x10dd, 0x10e1, 0x10d4,
0x10dc, 0x2d, 0x10e2, 0x10dd, 0x10db, 0x10d4, 0x10e2, 0x10e0, 0x10d8, 0x10de, 0x10dd, 0x10da,
0x10d8, 0x10e2, 0x10e3, 0x10dc, 0x10d8, 0x10e1, 0x10d8, 0x10d5, 0x10d8, 0x10dc, 0x10d3, 0x10ee,
0x10e3, 0x10d9, 0x10d8, 0x10d0, 0x10d3, 0x10d0, 0x10d9, 0x10d8, 0x10d4, 0x10dc, 0x10e5, 0x10dd,
0x10e0, 0x10d0, 0x10ef, 0x10d8, 0x10d0, 0x10dc, 0x10d2, 0x10d8, 0x10da, 0x10d8, 0x10d0, 0x10d0,
0x10dc, 0x10e2, 0x10d8, 0x10d2, 0x10e3, 0x10d0, 0x10d0, 0x10e0, 0x10d0, 0x10d2, 0x10e3, 0x10d0,
0x10d8, 0x10dc, 0x10d0, 0x10d1, 0x10e3, 0x10d4, 0x10dc, 0x10dd, 0x10e1, 0x2d, 0x10d0, 0x10d8,
0x10e0, 0x10d4, 0x10e1, 0x10d8, 0x10d9, 0x10d0, 0x10e2, 0x10d0, 0x10db, 0x10d0, 0x10e0, 0x10d9,
0x10d0, 0x10d9, 0x10dd, 0x10e0, 0x10d3, 0x10dd, 0x10d1, 0x10d0, 0x10ef, 0x10e3, 0x10ef, 0x10e3,
0x10d8, 0x10da, 0x10d0, 0x20, 0x10e0, 0x10d8, 0x10dd, 0x10ee, 0x10d0, 0x10db, 0x10d4, 0x10dc,
0x10d3, 0x10dd, 0x10d6, 0x10d0, 0x10e0, 0x10d8, 0x10dd, 0x20, 0x10d2, 0x10d0, 0x10da, 0x10d4,
0x10d2, 0x10dd, 0x10e1, 0x10d8, 0x10e1, 0x10d0, 0x10da, 0x10e2, 0x10d0, 0x10e1, 0x10d0, 0x10dc,
0x20, 0x10ee, 0x10e3, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x10d0, 0x10dc, 0x2d, 0x10da, 0x10e3,
0x10d8, 0x10e1, 0x10d8, 0x10e2, 0x10e3, 0x10d9, 0x10e3, 0x10db, 0x10d0, 0x10dc, 0x10d8, 0x10e3,
0x10e8, 0x10e3, 0x10d0, 0x10d8, 0x10d0, 0x10d0, 0x10e0, 0x10e3, 0x10d1, 0x10d0, 0x10d0, 0x10e1,
0x10e3, 0x10dc, 0x10e1, 0x10d8, 0x10dd, 0x10dc, 0x10d8, 0x10d0, 0x10e2, 0x10d8, 0x10d9, 0x10dd,
0x10d9, 0x10d0, 0x10dc, 0x10d8, 0x10d1, 0x10d0, 0x10d8, 0x10d0, 0x10d1, 0x10d0, 0x10f0, 0x10d8,
0x10d0, 0x20, 0x10d1, 0x10d0, 0x10dc, 0x10d3, 0x10d4, 0x10e0, 0x10d0, 0x10e1, 0x10d8, 0x10d1,
0x10d0, 0x10e0, 0x10d1, 0x10d0, 0x10d3, 0x10dd, 0x10e1, 0x10d8, 0x10d1, 0x10d4, 0x10da, 0x10d4,
0x10db, 0x10d8, 0x10d1, 0x10d4, 0x10da, 0x10d8, 0x10d6, 0x10d8, 0x10d1, 0x10da, 0x10d0, 0x10dc,
0x2d, 0x10e1, 0x10d0, 0x10d1, 0x10da, 0x10dd, 0x10dc, 0x10d8, 0x10d1, 0x10dd, 0x10d0, 0x20,
0x10d5, 0x10d8, 0x10e1, 0x10e2, 0x10d0, 0x10d1, 0x10dd, 0x10d2, 0x10dd, 0x10e2, 0x10d0, 0x10d1,
0x10e3, 0x10d0, 0x10e1, 0x10d8, 0x10d9, 0x10d4, 0x10db, 0x10d1, 0x10e0, 0x10d8, 0x10ef, 0x20,
0x10d1, 0x10d4, 0x10d8, 0x10d9, 0x10d0, 0x10db, 0x10de, 0x10dd, 0x20, 0x10d2, 0x10e0, 0x10d0,
0x10dc, 0x10d3, 0x10d4, 0x10d9, 0x10d0, 0x10dc, 0x10d9, 0x10e3, 0x10dc, 0x10d8, 0x10d9, 0x10d0,
0x10e0, 0x10d0, 0x10d9, 0x10d0, 0x10e1, 0x10d8, 0x10d9, 0x10d0, 0x10d8, 0x10d4, 0x10dc, 0x10d0,
0x10d9, 0x10d0, 0x10d8, 0x10db, 0x10d0, 0x10dc, 0x10d8, 0x10e9, 0x10d8, 0x10d9, 0x10d0, 0x10d2,
0x10dd, 0x10e9, 0x10d8, 0x10f0, 0x10e3, 0x10d0, 0x10f0, 0x10e3, 0x10d0, 0x10e1, 0x10d8, 0x10e3,
0x10d3, 0x10d0, 0x10d3, 0x2d, 0x10ee, 0x10e3, 0x10d0, 0x10e0, 0x10d4, 0x10e1, 0x10d8, 0x10d9,
0x10dd, 0x10e1, 0x10e2, 0x10d0, 0x2d, 0x10e0, 0x10d8, 0x10d9, 0x10d0, 0x10d9, 0x10e0, 0x10d4,
0x10e1, 0x10e2, 0x10dd, 0x10dc, 0x10d8, 0x10d9, 0x10e3, 0x10d8, 0x10d0, 0x10d1, 0x10d0, 0x10d9,
0x10d8, 0x10e3, 0x10e0, 0x10d0, 0x10e1, 0x10d0, 0x10dd, 0x10d3, 0x10d4, 0x10dc, 0x10db, 0x10d0,
0x10e0, 0x10d9, 0x10e8, 0x10d0, 0x10d5, 0x10dc, 0x10d8, 0x10d3, 0x10dd, 0x10e3, 0x10e1, 0x10dd,
0x10dc, 0x10d8, 0x10d3, 0x10dd, 0x10e3, 0x10e1, 0x10dd, 0x10dc, 0x20, 0x10e5, 0x10e0, 0x10d8,
0x10d9, 0x10d8, 0x10d3, 0x10d4, 0x10dc, 0x10d5, 0x10d4, 0x10e0, 0x10d8, 0x10d3, 0x10d4, 0x10e2,
0x10e0, 0x10dd, 0x10d8, 0x10e2, 0x10d8, 0x10d3, 0x10dd, 0x10db, 0x10d8, 0x10dc, 0x10d8, 0x10d9,
0x10d0, 0x10d4, 0x10d3, 0x10db, 0x10dd, 0x10dc, 0x10e2, 0x10dd, 0x10dc, 0x10d8, 0x10d4, 0x10d8,
0x10e0, 0x10e3, 0x10dc, 0x10d4, 0x10de, 0x10d4, 0x10e1, 0x10d0, 0x10da, 0x10d5, 0x10d0, 0x10d3,
0x10dd, 0x10e0, 0x10d8, 0x10e4, 0x10dd, 0x10e0, 0x10e2, 0x2d, 0x10dc, 0x10d4, 0x10da, 0x10e1,
0x10dd, 0x10dc, 0x10d8, 0x10e4, 0x10dd, 0x10e0, 0x10e2, 0x10d0, 0x10da, 0x10d4, 0x10d6, 0x10d0,
0x10d2, 0x10da, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10d1, 0x10d4, 0x10d8, 0x10d2, 0x10e3, 0x10d6,
0x20, 0x10d1, 0x10d4, 0x10d8, 0x10d2, 0x10e0, 0x10d0, 0x10dc, 0x10d3, 0x2d, 0x10e2, 0x10e3,
0x10e0, 0x10d9, 0x10d8, 0x10d2, 0x10e0, 0x10d4, 0x10dc, 0x10d0, 0x10d3, 0x10d0, 0x10d2, 0x10d5,
0x10d0, 0x10d3, 0x10d4, 0x10da, 0x10e3, 0x10de, 0x10d0, 0x10d2, 0x10d5, 0x10d0, 0x10e2, 0x10d4,
0x10db, 0x10d0, 0x10da, 0x10d0, 0x10d2, 0x10e3, 0x10d0, 0x10d8, 0x10d0, 0x10d9, 0x10d8, 0x10da,
0x10d8, 0x10d2, 0x10d0, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10f0, 0x10d0, 0x10da, 0x10d8, 0x10e4,
0x10d0, 0x10e5, 0x10e1, 0x10d8, 0x10f0, 0x10d0, 0x10d5, 0x10d0, 0x10dc, 0x10d0, 0x10f0, 0x10d4,
0x10e0, 0x10db, 0x10dd, 0x10e1, 0x10d8, 0x10da, 0x10dd, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0,
0x10dc, 0x10d0, 0x10de, 0x10dd, 0x10da, 0x10d8, 0x10e1, 0x10d8, 0x10dc, 0x10dd, 0x10e5, 0x10e1,
0x10d8, 0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10db, 0x10d0,
0x10e0, 0x10d4, 0x10dc, 0x10d2, 0x10dd, 0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0,
0x10dc, 0x10d0, 0x10de, 0x10d8, 0x10e2, 0x10d4, 0x10e0, 0x10e1, 0x10d1, 0x10e3, 0x10e0, 0x10d2,
0x10d8, 0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d7, 0x10d4,
0x10da, 0x20, 0x10e1, 0x10d8, 0x10d7, 0x10d8, 0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8,
0x10d0, 0x10dc, 0x10d0, 0x10d5, 0x10d8, 0x10d5, 0x10d4, 0x10d8, 0x2c, 0x20, 0x10d8, 0x10dc,
0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d5, 0x10d8, 0x10dc, 0x10e1, 0x10d4, 0x10dc, 0x10d8,
0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10e3, 0x10d8, 0x10dc,
0x10d4, 0x10db, 0x10d0, 0x10d9, 0x10d8, 0x2c, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10d8, 0x10d0,
0x10dc, 0x10d0, 0x10d8, 0x10dc, 0x10e3, 0x10d5, 0x10d8, 0x10d9, 0x10d8, 0x10d8, 0x10e5, 0x10d0,
0x10da, 0x10e3, 0x10d8, 0x10e2, 0x10d8, 0x10d8, 0x10d0, 0x10db, 0x10d0, 0x10d8, 0x10d9, 0x10d0,
0x10ef, 0x10e3, 0x10dc, 0x10dd, 0x10da, 0x10e3, 0x10d8, 0x10e1, 0x10d5, 0x10d8, 0x10da, 0x10d8,
0x10db, 0x10dd, 0x10dc, 0x10e2, 0x10d8, 0x10e9, 0x10d4, 0x10da, 0x10dd, 0x2c, 0x20, 0x10d9,
0x10d4, 0x10dc, 0x10e2, 0x10e3, 0x10d9, 0x10d8, 0x10d9, 0x10e0, 0x10d0, 0x10da, 0x10d4, 0x10dc,
0x10d3, 0x10d8, 0x10d8, 0x10d9, 0x10d8, 0x10da, 0x10d0, 0x2d, 0x10de, 0x10d0, 0x10d6, 0x10d8,
0x10da, 0x10d8, 0x10db, 0x10d0, 0x10da, 0x10dd, 0x10e1, 0x2d, 0x10d0, 0x10dc, 0x10ef, 0x10d4,
0x10da, 0x10d4, 0x10e1, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10e0, 0x20, 0x10de, 0x10e0, 0x10d8,
0x10dc, 0x10ea, 0x20, 0x10d9, 0x10d5, 0x10dd, 0x10e0, 0x10e2, 0x10d4, 0x10e0, 0x10d8, 0x10db,
0x10d0, 0x10e1, 0x10d4, 0x10d8, 0x10dd, 0x10db, 0x10d0, 0x10dc, 0x10d0, 0x10d2, 0x10e3, 0x10d0,
0x10db, 0x10d0, 0x10dc, 0x10d0, 0x10e3, 0x10e1, 0x10d8, 0x10db, 0x10d0, 0x10e0, 0x10d8, 0x10d2,
0x10dd, 0x10db, 0x10d0, 0x10e0, 0x10e2, 0x10d8, 0x10dc, 0x10d8, 0x10d9, 0x10d8, 0x10db, 0x10d0,
0x10e2, 0x10d0, 0x10db, 0x10dd, 0x10e0, 0x10dd, 0x10e1, 0x10d8, 0x10db, 0x10d0, 0x10d6, 0x10d0,
0x10e2, 0x10da, 0x10d0, 0x10dc, 0x10d8, 0x10db, 0x10d4, 0x10dc, 0x10dd, 0x10db, 0x10d8, 0x10dc,
0x10d8, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d3, 0x10d0, 0x10db, 0x10d4, 0x10e2, 0x10da, 0x10d0,
0x10d9, 0x10d0, 0x10e2, 0x10da, 0x10d0, 0x10db, 0x10d4, 0x10ee, 0x10d8, 0x10d9, 0x10dd, 0x10db,
0x10d8, 0x10d9, 0x10d4, 0x10da, 0x10dd, 0x10dc, 0x10d8, 0x10db, 0x10dd, 0x10dc, 0x10e5, 0x10e2,
0x10dd, 0x10dc, 0x10d8, 0x10db, 0x10dd, 0x10dc, 0x10e2, 0x10d4, 0x10e0, 0x10d4, 0x10d8, 0x10db,
0x10dd, 0x10dc, 0x10e2, 0x10d4, 0x10d5, 0x10d8, 0x10d3, 0x10d4, 0x10dd, 0x10db, 0x10dd, 0x10dc,
0x10e1, 0x10d4, 0x10e0, 0x10d0, 0x10e2, 0x10d8, 0x10dc, 0x10d0, 0x10e1, 0x10d0, 0x10e3, 0x10dc,
0x10d8, 0x10e3, 0x2d, 0x10d8, 0x10dd, 0x10e0, 0x10d9, 0x10d8, 0x10dc, 0x10dd, 0x10e0, 0x10dd,
0x10dc, 0x10d8, 0x10d0, 0x10d1, 0x10d8, 0x10e3, 0x10da, 0x10d0, 0x2c, 0x20, 0x10e9, 0x10e0,
0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x10d9, 0x10dd,
0x10e2, 0x10d0, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d8, 0x2c, 0x20, 0x10e9, 0x10e0,
0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x10d9, 0x10dd,
0x10e2, 0x10d0, 0x10dc, 0x10d8, 0x10e3, 0x2d, 0x10e1, 0x10d0, 0x10da, 0x10d4, 0x10db, 0x10d8,
0x2c, 0x20, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x10d8, 0x20,
0x10d3, 0x10d0, 0x10d9, 0x10dd, 0x10e2, 0x10d0, 0x10d2, 0x10dd, 0x10d3, 0x10d7, 0x10d4, 0x10d1,
0x10d8, 0x10dd, 0x10ee, 0x10d8, 0x10dc, 0x10d0, 0x10d2, 0x10d0, 0x10de, 0x10d0, 0x10dc, 0x10d0,
0x10db, 0x10d0, 0x10de, 0x10d0, 0x10e0, 0x10d0, 0x10db, 0x10d0, 0x10e0, 0x10d8, 0x10d1, 0x10dd,
0x10e4, 0x10d4, 0x10dc, 0x10d8, 0x10e5, 0x10e1, 0x10d8, 0x10de, 0x10dd, 0x10e0, 0x10e2, 0x2d,
0x10d0, 0x2d, 0x10de, 0x10e0, 0x10d8, 0x10dc, 0x10e1, 0x10d8, 0x10de, 0x10dd, 0x10e0, 0x10e2,
0x2d, 0x10dd, 0x10d5, 0x2d, 0x10e1, 0x10de, 0x10d4, 0x10d8, 0x10dc, 0x10d8, 0x10de, 0x10dd,
0x10e0, 0x10e2, 0x10e3, 0x2d, 0x10d5, 0x10d4, 0x10da, 0x10d8, 0x10dd, 0x10de, 0x10e3, 0x10d4,
0x10e0, 0x10e2, 0x10dd, 0x2d, 0x10e0, 0x10d8, 0x10d9, 0x10dd, 0x10de, 0x10e3, 0x10dc, 0x10e2,
0x10d0, 0x2d, 0x10d0, 0x10e0, 0x10d4, 0x10dc, 0x10d0, 0x10e1, 0x10d8, 0x10e0, 0x10d4, 0x10dc,
0x10d9, 0x10d8, 0x10dc, 0x20, 0x10d8, 0x10dc, 0x10da, 0x10d4, 0x10e2, 0x10d8, 0x10e0, 0x10d4,
0x10ea, 0x10d8, 0x10e4, 0x10d4, 0x10e0, 0x10d4, 0x10ef, 0x10d8, 0x10dc, 0x10d0, 0x10e0, 0x10d4,
0x10d6, 0x10dd, 0x10da, 0x10e3, 0x10e2, 0x10d4, 0x10e0, 0x10d8, 0x10dd, 0x20, 0x10d1, 0x10e0,
0x10d0, 0x10dc, 0x10d9, 0x10dd, 0x10e1, 0x10d0, 0x10dc, 0x10e2, 0x10d0, 0x10e0, 0x10d4, 0x10db,
0x10d8, 0x10e1, 0x10d0, 0x10dc, 0x10e2, 0x10d8, 0x10d0, 0x10d2, 0x10dd, 0x10e1, 0x10d0, 0x10dc,
0x10e2, 0x10dd, 0x2d, 0x10d3, 0x10dd, 0x10db, 0x10d8, 0x10dc, 0x10d2, 0x10dd, 0x10e1, 0x10d0,
0x10dc, 0x2d, 0x10de, 0x10d0, 0x10e3, 0x10da, 0x10e3, 0x10e1, 0x10d9, 0x10dd, 0x10e0, 0x10e1,
0x10d1, 0x10d8, 0x10e1, 0x10e3, 0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x10d8, 0x10e2, 0x10d9, 0x10d0,
0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10d1, 0x10d0, 0x10e0, 0x10d7, 0x10d4, 0x10da, 0x10d4, 0x10db,
0x10d8, 0x10e1, 0x10d4, 0x10dc, 0x10e2, 0x2d, 0x10ef, 0x10dd, 0x10dc, 0x10e1, 0x10d8, 0x10e1,
0x10d4, 0x10dc, 0x10e2, 0x2d, 0x10d9, 0x10d8, 0x10e2, 0x10e1, 0x10d8, 0x10e1, 0x10d4, 0x10dc,
0x10e2, 0x2d, 0x10da, 0x10e3, 0x10e1, 0x10d8, 0x10d0, 0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10e2,
0x10dd, 0x10db, 0x10d0, 0x10e1, 0x10d8, 0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10d5, 0x10d8, 0x10dc,
0x10e1, 0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10e1, 0x10d5, 0x10d8, 0x10e4, 0x10e2, 0x20, 0x10d9,
0x10d0, 0x10e0, 0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10e2, 0x10d4, 0x10d2, 0x10e3, 0x10e9, 0x10d8,
0x10d2, 0x10d0, 0x10da, 0x10de, 0x10d0, 0x10d7, 0x10e3, 0x10da, 0x10d4, 0x10e2, 0x10d8, 0x10ee,
0x10e3, 0x10d0, 0x10dc, 0x10d0, 0x10e2, 0x10dd, 0x10e0, 0x10dd, 0x10dc, 0x10e2, 0x10dd, 0x10e2,
0x10dd, 0x10e0, 0x10e2, 0x10dd, 0x10da, 0x10d0, 0x10d5, 0x10d0, 0x10dc, 0x10d9, 0x10e3, 0x10d5,
0x10d4, 0x10e0, 0x10d8, 0x10e3, 0x10d0, 0x10d8, 0x10d7, 0x10f0, 0x10dd, 0x10e0, 0x10e1, 0x10d8,
0x10e3, 0x10d8, 0x10dc, 0x10d8, 0x10de, 0x10d4, 0x10d2, 0x10d8, 0x10d8, 0x10d0, 0x10d9, 0x10e3,
0x10e2, 0x10d0, 0x10e2, 0x10d8, 0x10d9, 0x10d4, 0x10d8, 0x10e1, 0x10d8, 0x10d3, 0x10d4, 0x10d5,
0x10d8, 0x10e1, 0x10d8, 0x10d3, 0x10d8, 0x10e3, 0x10db, 0x10dd, 0x10dc, 0x20, 0x10d3, 0x2019,
0x10e3, 0x10e0, 0x10d5, 0x10d8, 0x10da, 0x10d8, 0x10db, 0x10d4, 0x10e5, 0x10d5, 0x10dd, 0x10e0,
0x10d8, 0x10db, 0x10dd, 0x10e3, 0x10e1, 0x10dd, 0x10dc, 0x10d8, 0x10db, 0x10d0, 0x10d9, 0x10db,
0x10e3, 0x10e0, 0x10d3, 0x10dd, 0x10de, 0x10d0, 0x10da, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10e0,
0x10dd, 0x10d7, 0x10d4, 0x10e0, 0x10d0, 0x10e1, 0x10d8, 0x10dd, 0x10e3, 0x10d0, 0x10e2, 0x10e0,
0x10dd, 0x10da, 0x10d8, 0x10d5, 0x10dd, 0x10e1, 0x10e2, 0x10dd, 0x10d9, 0x10d8, 0x10da, 0x10dd,
0x10dc, 0x10d2, 0x10d8, 0x10e0, 0x10d1, 0x10d8, 0x10d4, 0x10dc, 0x10d8, 0x10d0, 0x10d3, 0x10d4,
0x10dc, 0x10d8, 0x10d0, 0x10da, 0x10db, 0x10d0, 0x10e2, 0x10d8, 0x10d0, 0x10db, 0x10d0, 0x10dc,
0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d3, 0x10d8, 0x10e0, 0x10d8, 0x10d0, 0x10e5, 0x10e2, 0x10d0,
0x10e3, 0x10d0, 0x10e5, 0x10e2, 0x10dd, 0x10d1, 0x10d4, 0x10d0, 0x10e8, 0x10ee, 0x10d0, 0x10d1,
0x10d0, 0x10d3, 0x10d8, 0x10d0, 0x10e2, 0x10d8, 0x10e0, 0x10d0, 0x10e3, 0x10d1, 0x10d0, 0x10e6,
0x10d3, 0x10d0, 0x10d3, 0x10d8, 0x10d1, 0x10d0, 0x10f0, 0x10e0, 0x10d4, 0x10d8, 0x10dc, 0x10d8,
0x10d1, 0x10d0, 0x10e5, 0x10dd, 0x10d1, 0x10d0, 0x10dc, 0x10d2, 0x10d9, 0x10dd, 0x10d9, 0x10d8,
0x10d1, 0x10d0, 0x10e0, 0x10dc, 0x10d0, 0x10e3, 0x10da, 0x10d8, 0x10d1, 0x10d4, 0x10d8, 0x10e0,
0x10e3, 0x10d7, 0x10d8, 0x10d1, 0x10d8, 0x10e8, 0x10d9, 0x10d4, 0x10d9, 0x10d8, 0x10d1, 0x10e0,
0x10e3, 0x10dc, 0x10d4, 0x10d8, 0x10e9, 0x10d8, 0x10e2, 0x10d0, 0x10e9, 0x10dd, 0x10d8, 0x10d1,
0x10d0, 0x10da, 0x10e1, 0x10d0, 0x10dc, 0x10d8, 0x10d9, 0x10dd, 0x10da, 0x10dd, 0x10db, 0x10d1,
0x10dd, 0x10d3, 0x10d0, 0x10db, 0x10d0, 0x10e1, 0x10d9, 0x10d8, 0x10d3, 0x10d8, 0x10da, 0x10d8,
0x10d3, 0x10e3, 0x10d1, 0x10d0, 0x10d8, 0x10d3, 0x10e3, 0x10e8, 0x10d0, 0x10dc, 0x10d1, 0x10d4,
0x10e4, 0x10d0, 0x10db, 0x10d0, 0x10d2, 0x10e3, 0x10e1, 0x10e2, 0x10d0, 0x10d2, 0x10d0, 0x10d6,
0x10d0, 0x10f0, 0x10d4, 0x10d1, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10f0, 0x10dd, 0x10e9, 0x10d8,
0x10db, 0x10d8, 0x10dc, 0x10d8, 0x10f0, 0x10dd, 0x10dc, 0x10d9, 0x10dd, 0x10dc, 0x10d2, 0x10d8,
0x10f0, 0x10dd, 0x10d5, 0x10d3, 0x10d8, 0x10d8, 0x10e0, 0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9,
0x10d8, 0x10ef, 0x10d0, 0x10d9, 0x10d0, 0x10e0, 0x10e2, 0x10d0, 0x10ef, 0x10d0, 0x10d8, 0x10d0,
0x10de, 0x10e3, 0x10e0, 0x10d0, 0x10d8, 0x10d4, 0x10e0, 0x10e3, 0x10e1, 0x10d0, 0x10da, 0x10d8,
0x10db, 0x10d8, 0x10e5, 0x10d0, 0x10d1, 0x10e3, 0x10da, 0x10d8, 0x10d9, 0x10d0, 0x10db, 0x10e9,
0x10d0, 0x10e2, 0x10d9, 0x10d0, 0x10d9, 0x10d0, 0x10e0, 0x10d0, 0x10e9, 0x10d8, 0x10d9, 0x10d0,
0x10e2, 0x10db, 0x10d0, 0x10dc, 0x10d3, 0x10e3, 0x10ee, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d2,
0x10d0, 0x10d9, 0x10d0, 0x10da, 0x10d9, 0x10e3, 0x10e2, 0x10d0, 0x10d9, 0x10e0, 0x10d0, 0x10e1,
0x10dc, 0x10dd, 0x10d8, 0x10d0, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10d9, 0x10e3, 0x10d0, 0x10da,
0x10d0, 0x2d, 0x10da, 0x10e3, 0x10db, 0x10de, 0x10e3, 0x10e0, 0x10d8, 0x10d9, 0x10e3, 0x10e9,
0x10d8, 0x10dc, 0x10d2, 0x10d8, 0x10e5, 0x10e3, 0x10d5, 0x10d4, 0x10d8, 0x10d7, 0x10d8, 0x10db,
0x10d0, 0x10d9, 0x10d0, 0x10dd, 0x10db, 0x10d0, 0x10d2, 0x10d0, 0x10d3, 0x10d0, 0x10dc, 0x10d8,
0x10db, 0x10d0, 0x10d9, 0x10d0, 0x10e1, 0x10d0, 0x10e0, 0x10d8, 0x10db, 0x10d0, 0x10dc, 0x10d8,
0x10da, 0x10d0, 0x10db, 0x10e3, 0x10e1, 0x10d9, 0x10d0, 0x10e2, 0x10d8, 0x10dc, 0x10d8, 0x10d9,
0x10dd, 0x10e1, 0x10d8, 0x10d0, 0x10dc, 0x10dd, 0x10d5, 0x10dd, 0x10d9, 0x10e3, 0x10d6, 0x10dc,
0x10d4, 0x10ea, 0x10d9, 0x10d8, 0x10dc, 0x10dd, 0x10d5, 0x10dd, 0x10e1, 0x10d8, 0x10d1, 0x10d8,
0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10dd, 0x10db, 0x10e1, 0x10d9, 0x10d8, 0x10dd, 0x10e0, 0x10d0,
0x10da, 0x10d8, 0x10de, 0x10dc, 0x10dd, 0x10db, 0x10de, 0x10d4, 0x10dc, 0x10d8, 0x10de, 0x10dd,
0x10dc, 0x10e2, 0x10d8, 0x10d0, 0x10dc, 0x10d0, 0x10d9, 0x10d8, 0x10e4, 0x10ee, 0x10d4, 0x10dc,
0x10d8, 0x10d0, 0x10dc, 0x10d8, 0x10d9, 0x10d0, 0x10e2, 0x10d0, 0x10e0, 0x10d8, 0x10d9, 0x10dd,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d0, 0x10d8, 0x10e7, 0x10d8, 0x10d6, 0x10d8, 0x10da, 0x10dd,
0x10e0, 0x10d3, 0x10d0, 0x10d4, 0x10e0, 0x2d, 0x10e0, 0x10d8, 0x10d0, 0x10d3, 0x10d8, 0x10e1,
0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x10d0, 0x10db, 0x10d0, 0x10e0,
0x10e7, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x10d4, 0x10e3, 0x10da, 0x10d8, 0x10e8, 0x10d0,
0x10dc, 0x10ee, 0x10d0, 0x10d8, 0x10e1, 0x10d8, 0x10dc, 0x10d2, 0x10d0, 0x10de, 0x10e3, 0x10e0,
0x10d8, 0x10e1, 0x10e0, 0x10d4, 0x10d3, 0x10dc, 0x10d4, 0x10d9, 0x10dd, 0x10da, 0x10d8, 0x10db,
0x10e1, 0x10d9, 0x10d8, 0x10e2, 0x10d0, 0x10d8, 0x10de, 0x10d4, 0x10d8, 0x10e2, 0x10d0, 0x10e8,
0x10d9, 0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10d7, 0x10d1, 0x10d8, 0x10da, 0x10d8, 0x10e1, 0x10d8,
0x10d7, 0x10d4, 0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d8, 0x10d7, 0x10ee, 0x10d8, 0x10db, 0x10e4,
0x10ee, 0x10e3, 0x10e2, 0x10dd, 0x10d9, 0x10d8, 0x10dd, 0x10e2, 0x10dd, 0x10db, 0x10e1, 0x10d9,
0x10d8, 0x10e3, 0x10da, 0x10d0, 0x10dc, 0x10d1, 0x10d0, 0x10e2, 0x10d0, 0x10e0, 0x10d8, 0x10e3,
0x10e0, 0x10e3, 0x10db, 0x10e5, 0x10d8, 0x10e3, 0x10e1, 0x10e2, 0x2d, 0x10dc, 0x10d4, 0x10e0,
0x10d0, 0x10d5, 0x10d8, 0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10d0, 0x10dc, 0x10d8, 0x10d5, 0x10da,
0x10d0, 0x10d3, 0x10d8, 0x10d5, 0x10dd, 0x10e1, 0x10e2, 0x10dd, 0x10d9, 0x10d8, 0x10d8, 0x10d0,
0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d2, 0x10e3, 0x10dc,
0x10d8, 0x10d4, 0x10d9, 0x10d0, 0x10e2, 0x10d4, 0x10e0, 0x10d8, 0x10dc, 0x10d1, 0x10e3, 0x10e0,
0x10d2, 0x10d8, 0x10d4, 0x10e0, 0x10d4, 0x10d5, 0x10d0, 0x10dc, 0x10d8, 0x10d0, 0x10d6, 0x10dd,
0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1,
0x10d8, 0x10d1, 0x10d4, 0x10e0, 0x10db, 0x10e3, 0x10d3, 0x10d0, 0x10d9, 0x10d0, 0x10dc, 0x10d0,
0x10e0, 0x10d8, 0x10d9, 0x10d0, 0x10d1, 0x10dd, 0x2d, 0x10d5, 0x10d4, 0x10e0, 0x10d3, 0x10d4,
0x10e4, 0x10d0, 0x10e0, 0x10d4, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb,
0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10db, 0x10d0, 0x10d3, 0x10d4, 0x10d8, 0x10e0, 0x10d0,
0x10e0, 0x10d4, 0x10d8, 0x10d9, 0x10d8, 0x10d0, 0x10d5, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x10d0,
0x10db, 0x10ee, 0x10e0, 0x10d4, 0x10d7, 0x20, 0x10ef, 0x10dd, 0x10e0, 0x10ef, 0x10d8, 0x10d0,
0x10ec, 0x10db, 0x2e, 0x20, 0x10d4, 0x10da, 0x10d4, 0x10dc, 0x10d4, 0x10e1, 0x20, 0x10d9,
0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x10e2, 0x10d4, 0x10dc, 0x10da, 0x10d8,
0x10d0, 0x10d3, 0x10d4, 0x10da, 0x10d0, 0x10d8, 0x10d3, 0x10d0, 0x10d1, 0x10e0, 0x10d8, 0x10e1,
0x10d1, 0x10d4, 0x10d8, 0x10dc, 0x10d8, 0x10d1, 0x10e0, 0x10dd, 0x10e3, 0x10d9, 0x10d4, 0x10dc,
0x20, 0x10f0, 0x10d8, 0x10da, 0x10d8, 0x10d3, 0x10d0, 0x10e0, 0x10d5, 0x10d8, 0x10dc, 0x10d8,
0x10d4, 0x10e3, 0x10d9, 0x10da, 0x10d0, 0x10f0, 0x10dd, 0x10d1, 0x10d0, 0x10e0, 0x10e2, 0x10d8,
0x10da, 0x10d8, 0x10dc, 0x10d3, 0x10db, 0x10d0, 0x10dc, 0x10d8, 0x10da, 0x10dd, 0x10e0, 0x10d3,
0x20, 0x10f0, 0x10dd, 0x10e3, 0x10d8, 0x10db, 0x10d4, 0x10da, 0x10d1, 0x10e3, 0x10e0, 0x10dc,
0x10d8, 0x10de, 0x10d4, 0x10e0, 0x10d7, 0x10d8, 0x10e1, 0x10d8, 0x10d3, 0x10dc, 0x10d4, 0x10d8,
0x10d0, 0x10db, 0x10e1, 0x10e2, 0x10d4, 0x10e0, 0x10d3, 0x10d0, 0x10db, 0x10d8, 0x10d0, 0x10dc,
0x10d3, 0x10dd, 0x10e0, 0x10d0, 0x10d0, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10ee, 0x10d0, 0x10dc,
0x10d8, 0x10d0, 0x10d7, 0x10d4, 0x10dc, 0x10d8, 0x10d1, 0x10d4, 0x10da, 0x10d2, 0x10e0, 0x10d0,
0x10d3, 0x10d8, 0x10d1, 0x10d4, 0x10e0, 0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10d1, 0x10e0, 0x10d0,
0x10e2, 0x10d8, 0x10e1, 0x10da, 0x10d0, 0x10d5, 0x10d0, 0x10d1, 0x10e0, 0x10d8, 0x10e3, 0x10e1,
0x10d4, 0x10da, 0x10d8, 0x10d1, 0x10e3, 0x10e5, 0x10d0, 0x10e0, 0x10d4, 0x10e1, 0x10e2, 0x10d8,
0x10d1, 0x10e3, 0x10d3, 0x10d0, 0x10de, 0x10d4, 0x10e8, 0x10e2, 0x10d8, 0x10d1, 0x10e3, 0x10e1,
0x10d8, 0x10dc, 0x10d2, 0x10d4, 0x10dc, 0x10d8, 0x10d9, 0x10d8, 0x10e8, 0x10d8, 0x10dc, 0x10d8,
0x10dd, 0x10d5, 0x10d8, 0x10d9, 0x10dd, 0x10de, 0x10d4, 0x10dc, 0x10f0, 0x10d0, 0x10d2, 0x10d4,
0x10dc, 0x10d8, 0x10d3, 0x10e3, 0x10d1, 0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10d2, 0x10d8, 0x10d1,
0x10e0, 0x10d0, 0x10da, 0x10e2, 0x10d0, 0x10e0, 0x10d8, 0x10d2, 0x10d4, 0x10e0, 0x10dc, 0x10e1,
0x10d8, 0x10f0, 0x10d4, 0x10da, 0x10e1, 0x10d8, 0x10dc, 0x10d9, 0x10d8, 0x10db, 0x10d4, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x10e2,
0x10d0, 0x10db, 0x10d1, 0x10dd, 0x10da, 0x10d8, 0x10ef, 0x10d4, 0x10e0, 0x10e1, 0x10d8, 0x10d9,
0x10d0, 0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10dc, 0x10d2, 0x10e0, 0x10d0, 0x10d3, 0x10d8, 0x10d9,
0x10d8, 0x10e0, 0x10dd, 0x10d5, 0x10d8, 0x10d9, 0x10d8, 0x10d4, 0x10d5, 0x10d8, 0x10da, 0x10d8,
0x10e1, 0x10d0, 0x10d1, 0x10dd, 0x10dc, 0x10d8, 0x10da, 0x10d8, 0x10e3, 0x10d1, 0x10da, 0x10d8,
0x10d0, 0x10dc, 0x10d0, 0x10da, 0x10dd, 0x10dc, 0x10d3, 0x10dd, 0x10dc, 0x10d8, 0x10da, 0x10e3,
0x10e5, 0x10e1, 0x10d4, 0x10db, 0x10d1, 0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x10db, 0x10d0, 0x10d3,
0x10e0, 0x10d8, 0x10d3, 0x10d8, 0x10db, 0x10d0, 0x10da, 0x10e2, 0x10d0, 0x10db, 0x10d0, 0x10e0,
0x10d8, 0x10f0, 0x10d0, 0x10db, 0x10dc, 0x10d8, 0x10db, 0x10d8, 0x10dc, 0x10e1, 0x10d9, 0x10d8,
0x10db, 0x10dd, 0x10dc, 0x10d0, 0x10d9, 0x10dd, 0x10db, 0x10dd, 0x10e1, 0x10d9, 0x10dd, 0x10d5,
0x10d8, 0x10dd, 0x10e1, 0x10da, 0x10dd, 0x10de, 0x10d0, 0x10e0, 0x10d8, 0x10d6, 0x10d8, 0x10de,
0x10dd, 0x10d3, 0x10d2, 0x10dd, 0x10e0, 0x10d8, 0x10ea, 0x10d0, 0x10de, 0x10e0, 0x10d0, 0x10e6,
0x10d0, 0x10e0, 0x10d8, 0x10d2, 0x10d0, 0x10e0, 0x10dd, 0x10db, 0x10d8, 0x10e1, 0x10d0, 0x10db,
0x10d0, 0x10e0, 0x10d0, 0x10e1, 0x10d0, 0x10dc, 0x2d, 0x10db, 0x10d0, 0x10e0, 0x10d8, 0x10dc,
0x10dd, 0x10e1, 0x10d0, 0x10e0, 0x10d0, 0x10d4, 0x10d5, 0x10dd, 0x10e1, 0x10d0, 0x10e0, 0x10d0,
0x10e2, 0x10dd, 0x10d5, 0x10d8, 0x10e1, 0x10d8, 0x10db, 0x10e4, 0x10d4, 0x10e0, 0x10dd, 0x10de,
0x10dd, 0x10da, 0x10d8, 0x10e1, 0x10d9, 0x10dd, 0x10de, 0x10d8, 0x10d4, 0x10e1, 0x10dd, 0x10e4,
0x10d8, 0x10d0, 0x10e1, 0x10e2, 0x10dd, 0x10d9, 0x10f0, 0x10dd, 0x10da, 0x10db, 0x10d8, 0x10e2,
0x10d0, 0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10e2, 0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d0, 0x10e3,
0x10da, 0x10d8, 0x10d0, 0x10dc, 0x10dd, 0x10d5, 0x10e1, 0x10d9, 0x10d8, 0x10d5, 0x10d0, 0x10d3,
0x10e3, 0x10ea, 0x10d8, 0x10d5, 0x10d0, 0x10e2, 0x10d8, 0x10d9, 0x10d0, 0x10dc, 0x10d8, 0x10d5,
0x10d4, 0x10dc, 0x10d0, 0x10d5, 0x10d8, 0x10da, 0x10dc, 0x10d8, 0x10e3, 0x10e1, 0x10d8, 0x10d5,
0x10dd, 0x10da, 0x10d2, 0x10dd, 0x10d2, 0x10e0, 0x10d0, 0x10d3, 0x10d8, 0x10d5, 0x10d0, 0x10e0,
0x10e8, 0x10d0, 0x10d5, 0x10d0, 0x10d6, 0x10d0, 0x10d2, 0x10e0, 0x10d4, 0x10d1, 0x10d8, 0x10ea,
0x10d8, 0x10e3, 0x10e0, 0x10d8, 0x10ee, 0x10d8, 0x10d0, 0x10dc, 0x10e2, 0x10d0, 0x10dc, 0x10d0,
0x10dc, 0x10d0, 0x10e0, 0x10d8, 0x10d5, 0x10e3, 0x10e9, 0x10d0, 0x10d2, 0x10dd, 0x10e1, 0x10d8,
0x10e8, 0x10dd, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da,
0x10d8, 0x10e5, 0x10dd, 0x10e5, 0x10dd, 0x10e1, 0x10d8, 0x10d9, 0x10dd, 0x10db, 0x10dd, 0x10e0,
0x10dd, 0x10d9, 0x10d4, 0x10e0, 0x10d2, 0x10d4, 0x10da, 0x10d4, 0x10dc, 0x10d8, 0x10db, 0x10d0,
0x10f0, 0x10d4, 0x10db, 0x10d0, 0x10da, 0x10d3, 0x10d8, 0x10d5, 0x10d4, 0x10d1, 0x10d8, 0x10db,
0x10d0, 0x10d5, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10db, 0x10d0, 0x10d8, 0x10dd, 0x10e2, 0x10d8,
0x10e0, 0x10d4, 0x10d8, 0x10e3, 0x10dc, 0x10d8, 0x10dd, 0x10dc, 0x10d8, 0x10d0, 0x10de, 0x10d8,
0x10d0, 0x10dd, 0x10d9, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10d1, 0x10e3, 0x10d2, 0x10d4,
0x10dc, 0x10d5, 0x10d8, 0x10da, 0x10d8, 0x10e9, 0x10d0, 0x10d7, 0x10d0, 0x10db, 0x10d8, 0x10e9,
0x10e3, 0x10e3, 0x10d9, 0x10d8, 0x10d8, 0x10e1, 0x10d7, 0x10d4, 0x10e0, 0x10d8, 0x10d4, 0x10e4,
0x10d0, 0x10e2, 0x10d4, 0x10e4, 0x10d0, 0x10d9, 0x10d0, 0x10dd, 0x10e4, 0x10dd, 0x10e4, 0x10d8,
0x10ef, 0x10d8, 0x10e4, 0x10e3, 0x10dc, 0x10d0, 0x10e4, 0x10e3, 0x10e2, 0x10d8, 0x10d2, 0x10d0,
0x10da, 0x10d0, 0x10de, 0x10d0, 0x10d2, 0x10dd, 0x10e1, 0x10d8, 0x10d2, 0x10d0, 0x10db, 0x10d1,
0x10d8, 0x10d4, 0x10e0, 0x10d8, 0x10d2, 0x10d5, 0x10d0, 0x10d3, 0x10d0, 0x10da, 0x10d9, 0x10d0,
0x10dc, 0x10d0, 0x10da, 0x10d8, 0x10d2, 0x10e3, 0x10d0, 0x10db, 0x10d8, 0x10f0, 0x10dd, 0x10dc,
0x10dd, 0x10da, 0x10e3, 0x10da, 0x10e3, 0x10d4, 0x10dc, 0x10d3, 0x10d4, 0x10e0, 0x10d1, 0x10e3,
0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e0, 0x10d8, 0x10e2, 0x10d8, 0x10db, 0x10d0, 0x10e2, 0x10d8,
0x10d9, 0x10dd, 0x10e1, 0x10e0, 0x10d0, 0x10d4, 0x10d9, 0x10d5, 0x10d0, 0x10ef, 0x10d0, 0x10da,
0x10d4, 0x10d8, 0x10dc, 0x10d8, 0x10db, 0x10d0, 0x10ee, 0x10e3, 0x10e0, 0x10dd, 0x10db, 0x10d0,
0x10e0, 0x10d9, 0x10d4, 0x10d6, 0x10d0, 0x10e1, 0x10d8, 0x10db, 0x10d8, 0x10d3, 0x10e3, 0x10d4,
0x10d8, 0x10dc, 0x10d0, 0x10e3, 0x10e0, 0x10e3, 0x10dc, 0x10d8, 0x10e3, 0x10d4, 0x10dc, 0x10dd,
0x10e0, 0x10e4, 0x10dd, 0x10da, 0x10d9, 0x10d8, 0x10dc, 0x10e3, 0x10db, 0x10d4, 0x10d0, 0x10de,
0x10d0, 0x10d2, 0x10dd, 0x2d, 0x10de, 0x10d0, 0x10d2, 0x10dd, 0x10de, 0x10d0, 0x10da, 0x10d0,
0x10e3, 0x10de, 0x10d8, 0x10e2, 0x10d9, 0x10d4, 0x10e0, 0x10dc, 0x10d8, 0x10de, 0x10dd, 0x10dc,
0x10de, 0x10d4, 0x10d8, 0x10de, 0x10dd, 0x10e0, 0x10e2, 0x20, 0x10db, 0x10dd, 0x10e0, 0x10e1,
0x10d1, 0x10d8, 0x10e0, 0x10d0, 0x10e0, 0x10dd, 0x10e2, 0x10dd, 0x10dc, 0x10d2, 0x10d0, 0x10e1,
0x10d0, 0x10d8, 0x10de, 0x10d0, 0x10dc, 0x10d8, 0x10e2, 0x10d0, 0x10d8, 0x10e2, 0x10d8, 0x10e2,
0x10d0, 0x10e0, 0x10d0, 0x10e3, 0x10d0, 0x10e2, 0x10dd, 0x10dc, 0x10d2, 0x10d0, 0x10e2, 0x10d0,
0x10de, 0x10e3, 0x10e3, 0x10d4, 0x10d8, 0x10d9, 0x10d8, 0x10d5, 0x10d0, 0x10da, 0x10d8, 0x10e1,
0x10d8, 0x44, 0x61, 0x72, 0x65, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x44,
0x73, 0x63, 0x68, 0x69, 0x62, 0x75, 0x74, 0x69, 0x4d, 0x6f, 0x67, 0x61,
0x64, 0x69, 0x73, 0x63, 0x68, 0x75, 0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e,
0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f,
0x2d, 0x53, 0x74, 0x61, 0x64, 0x74, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x54,
0x73, 0x63, 0x68, 0x69, 0x74, 0x61, 0x54, 0x73, 0x63, 0x68, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x48, 0x6f, 0x2d, 0x43, 0x68, 0x69,
0x2d, 0x4d, 0x69, 0x6e, 0x68, 0x2d, 0x53, 0x74, 0x61, 0x64, 0x74, 0x43,
0x68, 0x6f, 0x77, 0x64, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x61,
0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x73, 0x63, 0x68, 0x69,
0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x65, 0x74, 0x73, 0x6b,
0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x50,
0x6a, 0xf6, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x54, 0x61, 0x69, 0x70,
0x65, 0x68, 0x54, 0x69, 0x66, 0x6c, 0x69, 0x73, 0x4b, 0x61, 0x6e, 0x61,
0x72, 0x65, 0x6e, 0x46, 0xe4, 0x72, 0xf6, 0x65, 0x72, 0x52, 0x65, 0x79,
0x6b, 0xad, 0x6a, 0x61, 0xad, 0x76, 0xed, 0x6b, 0x53, 0xfc, 0x64, 0x67,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x6e, 0x41, 0x73, 0x74, 0x72, 0x61,
0x63, 0x68, 0x61, 0x6e, 0x4b, 0x69, 0x72, 0x6f, 0x77, 0x4b, 0x69, 0x65,
0x77, 0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x77, 0x55, 0x6c, 0x6a, 0x61,
0x6e, 0x6f, 0x77, 0x73, 0x6b, 0x57, 0x65, 0x69, 0x68, 0x6e, 0x61, 0x63,
0x68, 0x74, 0x73, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x4b, 0x6f, 0x6d, 0x6f,
0x72, 0x65, 0x6e, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x76, 0x65, 0x6e,
0x4f, 0x73, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x46, 0x69,
0x64, 0x73, 0x63, 0x68, 0x69, 0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x76,
0x391, 0x3bc, 0x3c0, 0x3b9, 0x3c4, 0x3b6, 0x3ac, 0x3bd, 0x386, 0x3ba, 0x3c1, 0x3b1,
0x391, 0x3bd, 0x3c4, 0x3af, 0x3c2, 0x20, 0x391, 0x3bc, 0x3c0, 0x3ad, 0x3bc, 0x3c0,
0x3b1, 0x391, 0x3bb, 0x3b3, 0x3ad, 0x3c1, 0x3b9, 0x391, 0x3c3, 0x3bc, 0x3ac, 0x3c1,
0x3b1, 0x39c, 0x3c0, 0x3b1, 0x3bc, 0x3ac, 0x3ba, 0x3bf, 0x39c, 0x3c0, 0x3b1, 0x3bd,
0x3b3, 0x3ba, 0x3af, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b6, 0x3bf, 0x3cd, 0x3bb, 0x39c,
0x3c0, 0x3b9, 0x3c3, 0x3ac, 0x3bf, 0x3c5, 0x39c, 0x3c0, 0x3bb, 0x3b1, 0x3bd, 0x3c4,
0x3ac, 0x3b9, 0x3c1, 0x39c, 0x3c0, 0x3c1, 0x3b1, 0x3b6, 0x3b1, 0x3b2, 0x3af, 0x3bb,
0x39c, 0x3c0, 0x3bf, 0x3c5, 0x3b6, 0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3bf, 0x3cd, 0x3c1,
0x3b1, 0x39a, 0x3ac, 0x3b9, 0x3c1, 0x3bf, 0x39a, 0x3b1, 0x3b6, 0x3b1, 0x3bc, 0x3c0,
0x3bb, 0x3ac, 0x3bd, 0x3ba, 0x3b1, 0x398, 0x3ad, 0x3bf, 0x3c5, 0x3c4, 0x3b1, 0x39a,
0x3cc, 0x3bd, 0x3b1, 0x3ba, 0x3c1, 0x3b9, 0x39d, 0x3c4, 0x3b1, 0x3ba, 0x3ac, 0x3c1,
0x39d, 0x3c4, 0x3b1, 0x3c1, 0x20, 0x3b5, 0x3c2, 0x20, 0x3a3, 0x3b1, 0x3bb, 0x3ac,
0x3bc, 0x3a4, 0x3b6, 0x3b9, 0x3bc, 0x3c0, 0x3bf, 0x3c5, 0x3c4, 0x3af, 0x39d, 0x3c4,
0x3bf, 0x3c5, 0x3ac, 0x3bb, 0x3b1, 0x395, 0x3bb, 0x20, 0x391, 0x3b3, 0x3b9, 0x3bf,
0x3cd, 0x3bd, 0x3a6, 0x3c1, 0x3af, 0x3c4, 0x3b1, 0x3bf, 0x3c5, 0x3bd, 0x393, 0x3ba,
0x3b1, 0x3bc, 0x3c0, 0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3b5, 0x3a7, 0x3b1, 0x3c1, 0x3ac,
0x3c1, 0x3b5, 0x393, 0x3b9, 0x3bf, 0x3c7, 0x3ac, 0x3bd, 0x3b5, 0x3c3, 0x3bc, 0x3c0,
0x3bf, 0x3c5, 0x3c1, 0x3b3, 0x3ba, 0x3a4, 0x3b6, 0x3bf, 0x3cd, 0x3bc, 0x3c0, 0x3b1,
0x39a, 0x3b1, 0x3bc, 0x3c0, 0x3ac, 0x3bb, 0x3b1, 0x3a7, 0x3b1, 0x3c1, 0x3c4, 0x3bf,
0x3cd, 0x3bc, 0x39a, 0x3b9, 0x3b3, 0x3ba, 0x3ac, 0x3bb, 0x3b9, 0x39a, 0x3b9, 0x3bd,
0x3c3, 0x3ac, 0x3c3, 0x3b1, 0x39b, 0x3ac, 0x3b3, 0x3ba, 0x3bf, 0x3c2, 0x39b, 0x3b9,
0x3bc, 0x3c0, 0x3c1, 0x3b5, 0x3b2, 0x3af, 0x3bb, 0x39b, 0x3bf, 0x3bc, 0x3ad, 0x39b,
0x3bf, 0x3c5, 0x3ac, 0x3bd, 0x3c4, 0x3b1, 0x39b, 0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3bf,
0x3c5, 0x3bc, 0x3c0, 0x3ac, 0x3c3, 0x3b9, 0x39b, 0x3bf, 0x3c5, 0x3b6, 0x3ac, 0x3ba,
0x3b1, 0x39c, 0x3b1, 0x3bb, 0x3ac, 0x3bc, 0x3c0, 0x3bf, 0x39c, 0x3b1, 0x3c0, 0x3bf,
0x3cd, 0x3c4, 0x3bf, 0x39c, 0x3b1, 0x3c3, 0x3ad, 0x3c1, 0x3bf, 0x3c5, 0x39c, 0x3c0,
0x3b1, 0x3bc, 0x3c0, 0x3ac, 0x3bd, 0x3b5, 0x39c, 0x3bf, 0x3b3, 0x3ba, 0x3b1, 0x3bd,
0x3c4, 0x3af, 0x3c3, 0x3bf, 0x3c5, 0x39c, 0x3bf, 0x3bd, 0x3c1, 0x3cc, 0x3b2, 0x3b9,
0x3b1, 0x39d, 0x3b1, 0x3ca, 0x3c1, 0x3cc, 0x3bc, 0x3c0, 0x3b9, 0x39d, 0x3c4, 0x3b6,
0x3b1, 0x3bc, 0x3ad, 0x3bd, 0x3b1, 0x39d, 0x3b9, 0x3b1, 0x3bc, 0x3ad, 0x3b9, 0x39d,
0x3bf, 0x3c5, 0x3b1, 0x3ba, 0x3c3, 0x3cc, 0x3c4, 0x39f, 0x3c5, 0x3b1, 0x3b3, 0x3ba,
0x3b1, 0x3bd, 0x3c4, 0x3bf, 0x3cd, 0x3b3, 0x3ba, 0x3bf, 0x3c5, 0x3a0, 0x3cc, 0x3c1,
0x3c4, 0x3bf, 0x2d, 0x39d, 0x3cc, 0x3b2, 0x3bf, 0x3a3, 0x3ac, 0x3bf, 0x20, 0x3a4,
0x3bf, 0x3bc, 0x3ad, 0x3a4, 0x3c1, 0x3af, 0x3c0, 0x3bf, 0x3bb, 0x3b7, 0x3a4, 0x3cd,
0x3bd, 0x3b9, 0x3b4, 0x3b1, 0x392, 0x3af, 0x3bd, 0x3c4, 0x3c7, 0x3bf, 0x3c5, 0x3ba,
0x386, 0x3bd, 0x3c4, 0x3b1, 0x3ba, 0x386, 0x3bd, 0x3ba, 0x3bf, 0x3c1, 0x3b1, 0x3c4,
0x3b6, 0x391, 0x3bd, 0x3b3, 0x3ba, 0x3bf, 0x3c5, 0x3af, 0x3bb, 0x3b1, 0x391, 0x3bd,
0x3c4, 0x3af, 0x3b3, 0x3ba, 0x3bf, 0x3c5, 0x3b1, 0x391, 0x3c1, 0x3b1, 0x3b3, 0x3ba,
0x3bf, 0x3c5, 0x3ac, 0x3b9, 0x3bd, 0x3b1, 0x39c, 0x3c0, 0x3bf, 0x3c5, 0x3ad, 0x3bd,
0x3bf, 0x3c2, 0x20, 0x386, 0x3b9, 0x3c1, 0x3b5, 0x3c2, 0x39a, 0x3b1, 0x3c4, 0x3b1,
0x3bc, 0x3ac, 0x3c1, 0x3ba, 0x3b1, 0x39a, 0x3cc, 0x3c1, 0x3b4, 0x3bf, 0x3b2, 0x3b1,
0x3a7, 0x3bf, 0x3c5, 0x3c7, 0x3bf, 0x3cd, 0x3b9, 0x39b, 0x3b1, 0x20, 0x3a1, 0x3b9,
0x3cc, 0x3c7, 0x3b1, 0x39c, 0x3b5, 0x3bd, 0x3c4, 0x3cc, 0x3b6, 0x3b1, 0x3a1, 0x3af,
0x3bf, 0x20, 0x393, 0x3ba, 0x3b1, 0x3b3, 0x3b9, 0x3ad, 0x3b3, 0x3ba, 0x3bf, 0x3c2,
0x3a3, 0x3ac, 0x3bb, 0x3c4, 0x3b1, 0x3a3, 0x3b1, 0x3bd, 0x20, 0x3a7, 0x3bf, 0x3c5,
0x3ac, 0x3bd, 0x3a3, 0x3b1, 0x3bd, 0x20, 0x39b, 0x3bf, 0x3cd, 0x3b9, 0x3c2, 0x3a4,
0x3bf, 0x3c5, 0x3ba, 0x3bf, 0x3c5, 0x3bc, 0x3ac, 0x3bd, 0x39f, 0x3c5, 0x3c3, 0x3bf,
0x3c5, 0x3ac, 0x3b9, 0x3b1, 0x391, 0x3c1, 0x3bf, 0x3cd, 0x3bc, 0x3c0, 0x3b1, 0x391,
0x3c3, 0x3bf, 0x3c5, 0x3bd, 0x3c3, 0x3b9, 0x3cc, 0x3bd, 0x391, 0x3c4, 0x3b9, 0x3ba,
0x3cc, 0x3ba, 0x3b1, 0x3bd, 0x39c, 0x3c0, 0x3b1, 0x390, 0x3b1, 0x39c, 0x3c0, 0x3b1,
0x390, 0x3b1, 0x20, 0x3bd, 0x3c4, 0x3b5, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3c4,
0x3ad, 0x3c1, 0x3b1, 0x3c2, 0x39c, 0x3c0, 0x3b1, 0x3c1, 0x3bc, 0x3c0, 0x3ad, 0x3b9,
0x3bd, 0x3c4, 0x3bf, 0x3c2, 0x39c, 0x3c0, 0x3b5, 0x3bb, 0x3ad, 0x3bc, 0x39c, 0x3c0,
0x3b5, 0x3bb, 0x3af, 0x3b6, 0x39c, 0x3c0, 0x3bb, 0x3b1, 0x3bd, 0x20, 0x3a3, 0x3b1,
0x3bc, 0x3c0, 0x3bb, 0x3cc, 0x3bd, 0x39c, 0x3c0, 0x3cc, 0x3b1, 0x20, 0x392, 0x3af,
0x3c3, 0x3c4, 0x3b1, 0x39c, 0x3c0, 0x3bf, 0x3b3, 0x3ba, 0x3bf, 0x3c4, 0x3ac, 0x39c,
0x3c0, 0x3cc, 0x3b9, 0x3b6, 0x3b9, 0x39a, 0x3ad, 0x3bc, 0x3c0, 0x3c1, 0x3b9, 0x3c4,
0x3b6, 0x20, 0x39c, 0x3c0, 0x3ad, 0x3b9, 0x39a, 0x3ac, 0x3bc, 0x3c0, 0x3bf, 0x20,
0x393, 0x3ba, 0x3c1, 0x3ac, 0x3bd, 0x3c4, 0x3b5, 0x39a, 0x3b1, 0x3bd, 0x3ba, 0x3bf,
0x3cd, 0x3bd, 0x39a, 0x3b1, 0x3c1, 0x3ac, 0x3ba, 0x3b1, 0x3c2, 0x39a, 0x3b1, 0x3b3,
0x3b9, 0x3ad, 0x3bd, 0x39a, 0x3ad, 0x3b9, 0x3bc, 0x3b1, 0x3bd, 0x3a3, 0x3b9, 0x3ba,
0x3ac, 0x3b3, 0x3bf, 0x3a4, 0x3c3, 0x3b9, 0x3bf, 0x3c5, 0x3ac, 0x3bf, 0x3c5, 0x3b1,
0x3a3, 0x3b9, 0x3bf, 0x3c5, 0x3b4, 0x3ac, 0x3b4, 0x20, 0x3a7, 0x3bf, 0x3c5, 0x3ac,
0x3c1, 0x3b5, 0x3c2, 0x39a, 0x3cc, 0x3c3, 0x3c4, 0x3b1, 0x20, 0x3a1, 0x3af, 0x3ba,
0x3b1, 0x39a, 0x3c1, 0x3ad, 0x3c3, 0x3c4, 0x3bf, 0x3bd, 0x39a, 0x3bf, 0x3c5, 0x3b9,
0x3b1, 0x3bc, 0x3c0, 0x3ac, 0x39a, 0x3bf, 0x3c5, 0x3c1, 0x3b1, 0x3c3, 0x3ac, 0x3bf,
0x39d, 0x3c4, 0x3b1, 0x3bd, 0x3bc, 0x3b1, 0x3c1, 0x3ba, 0x3c3, 0x3ac, 0x3b2, 0x3bd,
0x39d, 0x3c4, 0x3cc, 0x3c3, 0x3bf, 0x3bd, 0x39d, 0x3c4, 0x3cc, 0x3c3, 0x3bf, 0x3bd,
0x20, 0x39a, 0x3c1, 0x3b9, 0x3ba, 0x39d, 0x3c4, 0x3ad, 0x3bd, 0x3b2, 0x3b5, 0x3c1,
0x39d, 0x3c4, 0x3b9, 0x3c4, 0x3c1, 0x3cc, 0x3b9, 0x3c4, 0x39d, 0x3c4, 0x3bf, 0x3bc,
0x3af, 0x3bd, 0x3b9, 0x3ba, 0x3b1, 0x388, 0x3bd, 0x3c4, 0x3bc, 0x3bf, 0x3bd, 0x3c4,
0x3bf, 0x3bd, 0x395, 0x3ca, 0x3c1, 0x3bf, 0x3c5, 0x3bd, 0x3b5, 0x3c0, 0x3ad, 0x395,
0x3bb, 0x20, 0x3a3, 0x3b1, 0x3bb, 0x3b2, 0x3b1, 0x3b4, 0x3cc, 0x3c1, 0x3a6, 0x3bf,
0x3c1, 0x3c4, 0x20, 0x39d, 0x3ad, 0x3bb, 0x3c3, 0x3bf, 0x3bd, 0x3a6, 0x3bf, 0x3c1,
0x3c4, 0x3b1, 0x3bb, 0x3ad, 0x3b6, 0x3b1, 0x393, 0x3ba, 0x3bb, 0x3ad, 0x3b9, 0x3c2,
0x20, 0x39c, 0x3c0, 0x3ad, 0x3b9, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3c2, 0x20, 0x39c,
0x3c0, 0x3ad, 0x3b9, 0x393, 0x3ba, 0x3c1, 0x3b1, 0x3bd, 0x3c4, 0x20, 0x3a4, 0x3bf,
0x3c5, 0x3c1, 0x3ba, 0x393, 0x3c1, 0x3b5, 0x3bd, 0x3ac, 0x3b4, 0x3b1, 0x393, 0x3bf,
0x3c5, 0x3b1, 0x3b4, 0x3b5, 0x3bb, 0x3bf, 0x3cd, 0x3c0, 0x3b7, 0x393, 0x3bf, 0x3c5,
0x3b1, 0x3c4, 0x3b5, 0x3bc, 0x3ac, 0x3bb, 0x3b1, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3b1,
0x3b3, 0x3b9, 0x3b1, 0x3ba, 0x3af, 0x3bb, 0x393, 0x3bf, 0x3c5, 0x3b9, 0x3ac, 0x3bd,
0x3b1, 0x3a7, 0x3ac, 0x3bb, 0x3b9, 0x3c6, 0x3b1, 0x3be, 0x391, 0x3b2, 0x3ac, 0x3bd,
0x3b1, 0x395, 0x3c1, 0x3bc, 0x3bf, 0x3c3, 0x3af, 0x3b3, 0x3b9, 0x3bf, 0x399, 0x3bd,
0x3c4, 0x3b9, 0x3b1, 0x3bd, 0x3ac, 0x3c0, 0x3bf, 0x3bb, 0x3b9, 0x3c2, 0x39d, 0x3bf,
0x3be, 0x2c, 0x20, 0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x39c, 0x3b1,
0x3c1, 0x3ad, 0x3bd, 0x3b3, 0x3ba, 0x3bf, 0x2c, 0x20, 0x399, 0x3bd, 0x3c4, 0x3b9,
0x3ac, 0x3bd, 0x3b1, 0x3a0, 0x3af, 0x3c4, 0x3b5, 0x3c1, 0x3c3, 0x3bc, 0x3c0, 0x3b5,
0x3c1, 0x3b3, 0x3ba, 0x2c, 0x20, 0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1,
0x3a4, 0x3b5, 0x3bb, 0x20, 0x3a3, 0x3af, 0x3c4, 0x3b9, 0x2c, 0x20, 0x399, 0x3bd,
0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x392, 0x3b9, 0x3b2, 0x3ad, 0x3b9, 0x2c, 0x20,
0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x392, 0x3b1, 0x3bd, 0x3c3, 0x3ad,
0x3bd, 0x2c, 0x20, 0x399, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x393, 0x3bf,
0x3c5, 0x3af, 0x3bd, 0x3b1, 0x3bc, 0x3b1, 0x3ba, 0x2c, 0x20, 0x399, 0x3bd, 0x3c4,
0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x399, 0x3bd, 0x3bf, 0x3cd, 0x3b2, 0x3b9, 0x3ba, 0x399,
0x3ba, 0x3b1, 0x3bb, 0x3bf, 0x3cd, 0x3b9, 0x3c4, 0x3a4, 0x3b6, 0x3b1, 0x3bc, 0x3ac,
0x3b9, 0x3ba, 0x3b1, 0x3a4, 0x3b6, 0x3bf, 0x3cd, 0x3bd, 0x3bf, 0x39b, 0x3bf, 0x3cd,
0x3b9, 0x3b2, 0x3b9, 0x3bb, 0x39c, 0x3bf, 0x3bd, 0x3c4, 0x3b9, 0x3c4, 0x3c3, 0x3ad,
0x3bb, 0x3bf, 0x2c, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3ac, 0x3ba, 0x3b9, 0x39a,
0x3c1, 0x3ac, 0x3bb, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3ba, 0x39b, 0x3b1, 0x20, 0x3a0,
0x3b1, 0x3b6, 0x39b, 0x3af, 0x3bc, 0x3b1, 0x39b, 0x3bf, 0x3c2, 0x20, 0x386, 0x3bd,
0x3c4, 0x3b6, 0x3b5, 0x3bb, 0x3b5, 0x3c2, 0x39c, 0x3b1, 0x3c3, 0x3b5, 0x3ca, 0x3cc,
0x39c, 0x3b1, 0x3bd, 0x3ac, 0x3b3, 0x3ba, 0x3bf, 0x3c5, 0x3b1, 0x39c, 0x3b1, 0x3bd,
0x3ac, 0x3bf, 0x3c5, 0x3c2, 0x39c, 0x3b1, 0x3c1, 0x3b9, 0x3b3, 0x3ba, 0x3cc, 0x39c,
0x3b1, 0x3c1, 0x3c4, 0x3b9, 0x3bd, 0x3af, 0x3ba, 0x3b1, 0x39c, 0x3b1, 0x3c4, 0x3b1,
0x3bc, 0x3cc, 0x3c1, 0x3bf, 0x3c2, 0x39c, 0x3b1, 0x3b6, 0x3b1, 0x3c4, 0x3bb, 0x3ac,
0x3bd, 0x39c, 0x3b5, 0x3bd, 0x3bf, 0x3bc, 0x3af, 0x3bd, 0x3b5, 0x39c, 0x3ad, 0x3c1,
0x3b9, 0x3b4, 0x3b1, 0x39c, 0x3b5, 0x3c4, 0x3bb, 0x3b1, 0x3ba, 0x3ac, 0x3c4, 0x3bb,
0x3b1, 0x3a0, 0x3cc, 0x3bb, 0x3b7, 0x20, 0x3c4, 0x3bf, 0x3c5, 0x20, 0x39c, 0x3b5,
0x3be, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x39c, 0x3b9, 0x3ba, 0x3b5, 0x3bb, 0x3cc, 0x3bd,
0x39c, 0x3cc, 0x3bd, 0x3ba, 0x3c4, 0x3bf, 0x3bd, 0x39c, 0x3bf, 0x3bd, 0x3c4, 0x3b5,
0x3c1, 0x3ad, 0x3b9, 0x39c, 0x3bf, 0x3bd, 0x3c4, 0x3b5, 0x3b2, 0x3b9, 0x3b4, 0x3ad,
0x3bf, 0x39c, 0x3bf, 0x3bd, 0x3c3, 0x3b5, 0x3c1, 0x3ac, 0x3c4, 0x39d, 0x3b1, 0x3c3,
0x3ac, 0x3bf, 0x3c5, 0x39d, 0x3ad, 0x3b1, 0x20, 0x3a5, 0x3cc, 0x3c1, 0x3ba, 0x3b7,
0x39d, 0x3cc, 0x3bc, 0x3b5, 0x39d, 0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3b9, 0x3b1, 0x39c,
0x3c0, 0x3ad, 0x3bf, 0x3c5, 0x3bb, 0x3b1, 0x2c, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5,
0x3b9, 0x3b1, 0x20, 0x39d, 0x3c4, 0x3b1, 0x3ba, 0x3cc, 0x3c4, 0x3b1, 0x3a3, 0x3ad,
0x3bd, 0x3c4, 0x3b5, 0x3c1, 0x2c, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1,
0x20, 0x39d, 0x3c4, 0x3b1, 0x3ba, 0x3cc, 0x3c4, 0x3b1, 0x39d, 0x3b9, 0x3bf, 0x3c5,
0x20, 0x3a3, 0x3ad, 0x3b9, 0x3bb, 0x3b5, 0x3bc, 0x2c, 0x20, 0x392, 0x3cc, 0x3c1,
0x3b5, 0x3b9, 0x3b1, 0x20, 0x39d, 0x3c4, 0x3b1, 0x3ba, 0x3cc, 0x3c4, 0x3b1, 0x39d,
0x3bf, 0x3c5, 0x3bf, 0x3cd, 0x3ba, 0x39f, 0x3c7, 0x3b9, 0x3bd, 0x3ac, 0x3b3, 0x3ba,
0x3b1, 0x3a0, 0x3b1, 0x3bd, 0x3b1, 0x3bc, 0x3ac, 0x3c2, 0x3a0, 0x3b1, 0x3c1, 0x3b1,
0x3bc, 0x3b1, 0x3c1, 0x3af, 0x3bc, 0x3c0, 0x3bf, 0x3a6, 0x3bf, 0x3af, 0x3bd, 0x3b9,
0x3be, 0x3a0, 0x3bf, 0x3c1, 0x3c4, 0x2d, 0x3bf, 0x2d, 0x3a0, 0x3c1, 0x3b5, 0x3bd,
0x3c2, 0x3a0, 0x3bf, 0x3c1, 0x3c4, 0x20, 0x3bf, 0x3c6, 0x20, 0x3a3, 0x3c0, 0x3ad,
0x3b9, 0x3bd, 0x3a0, 0x3cc, 0x3c1, 0x3c4, 0x3bf, 0x20, 0x392, 0x3ad, 0x3bb, 0x3b9,
0x3bf, 0x3a0, 0x3bf, 0x3c5, 0x3ad, 0x3c1, 0x3c4, 0x3bf, 0x20, 0x3a1, 0x3af, 0x3ba,
0x3bf, 0x3a0, 0x3bf, 0x3cd, 0x3bd, 0x3c4, 0x3b1, 0x20, 0x391, 0x3c1, 0x3ad, 0x3bd,
0x3b1, 0x3c2, 0x3a1, 0x3ac, 0x3bd, 0x3ba, 0x3b9, 0x3bd, 0x20, 0x38a, 0x3bd, 0x3bb,
0x3b5, 0x3c4, 0x3a1, 0x3b5, 0x3c3, 0x3af, 0x3c6, 0x3b5, 0x3a1, 0x3b5, 0x3c4, 0x3b6,
0x3ac, 0x3b9, 0x3bd, 0x3b1, 0x3a1, 0x3ad, 0x3b6, 0x3bf, 0x3bb, 0x3bf, 0x3c5, 0x3c4,
0x3a1, 0x3af, 0x3bf, 0x20, 0x39c, 0x3c0, 0x3c1, 0x3ac, 0x3bd, 0x3ba, 0x3bf, 0x3a3,
0x3b1, 0x3bd, 0x3c4, 0x3b1, 0x3c1, 0x3ad, 0x3bc, 0x3a3, 0x3b1, 0x3bd, 0x3c4, 0x3b9,
0x3ac, 0x3b3, 0x3ba, 0x3bf, 0x386, 0x3b3, 0x3b9, 0x3bf, 0x3c2, 0x20, 0x394, 0x3bf,
0x3bc, 0x3af, 0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3c2, 0x3a3, 0x3ac, 0x3bf, 0x20, 0x3a0,
0x3ac, 0x3bf, 0x3bb, 0x3bf, 0x3a3, 0x3ba, 0x3bf, 0x3c1, 0x3b5, 0x3c3, 0x3bc, 0x3c0,
0x3af, 0x3c3, 0x3bf, 0x3c5, 0x3bd, 0x3c4, 0x3a3, 0x3af, 0x3c4, 0x3ba, 0x3b1, 0x386,
0x3b3, 0x3b9, 0x3bf, 0x3c2, 0x20, 0x392, 0x3b1, 0x3c1, 0x3b8, 0x3bf, 0x3bb, 0x3bf,
0x3bc, 0x3b1, 0x3af, 0x3bf, 0x3c2, 0x3a3, 0x3b5, 0x3bd, 0x20, 0x3a4, 0x3b6, 0x3bf,
0x3bd, 0x3c2, 0x3a3, 0x3b5, 0x3bd, 0x20, 0x39a, 0x3b9, 0x3c4, 0x3c2, 0x391, 0x3b3,
0x3af, 0x3b1, 0x20, 0x39b, 0x3bf, 0x3c5, 0x3ba, 0x3af, 0x3b1, 0x386, 0x3b3, 0x3b9,
0x3bf, 0x3c2, 0x20, 0x398, 0x3c9, 0x3bc, 0x3ac, 0x3c2, 0x386, 0x3b3, 0x3b9, 0x3bf,
0x3c2, 0x20, 0x392, 0x3b9, 0x3ba, 0x3ad, 0x3bd, 0x3c4, 0x3b9, 0x3bf, 0x3c2, 0x3a3,
0x3bf, 0x3c5, 0x3af, 0x3c6, 0x3c4, 0x20, 0x39a, 0x3ac, 0x3c1, 0x3b5, 0x3bd, 0x3c4,
0x3a4, 0x3b5, 0x3b3, 0x3ba, 0x3bf, 0x3c5, 0x3c3, 0x3b9, 0x3b3, 0x3ba, 0x3ac, 0x3bb,
0x3c0, 0x3b1, 0x398, 0x3bf, 0x3cd, 0x3bb, 0x3b7, 0x3a4, 0x3b9, 0x3c7, 0x3bf, 0x3c5,
0x3ac, 0x3bd, 0x3b1, 0x3a4, 0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3c4, 0x3bf, 0x3a4, 0x3bf,
0x3c1, 0x3c4, 0x3cc, 0x3bb, 0x3b1, 0x392, 0x3b1, 0x3bd, 0x3ba, 0x3bf, 0x3cd, 0x3b2,
0x3b5, 0x3c1, 0x393, 0x3bf, 0x3c5, 0x3ac, 0x3b9, 0x3c4, 0x3c7, 0x3bf, 0x3c1, 0x3c2,
0x393, 0x3bf, 0x3c5, 0x3af, 0x3bd, 0x3b9, 0x3c0, 0x3b5, 0x3b3, 0x3ba, 0x393, 0x3b9,
0x3ac, 0x3ba, 0x3bf, 0x3c5, 0x3c4, 0x3b1, 0x3c4, 0x39a, 0x3ac, 0x3c3, 0x3b5, 0x3ca,
0x39d, 0x3c4, 0x3ad, 0x3b9, 0x3b2, 0x3b9, 0x3c2, 0x39d, 0x3c4, 0x3b9, 0x3bc, 0x3cc,
0x3bd, 0x20, 0x3bd, 0x3c4, 0x2019, 0x20, 0x39f, 0x3c5, 0x3c1, 0x3b2, 0x3af, 0x3bb,
0x39c, 0x3b1, 0x3ba, 0x3bf, 0x3c5, 0x3ac, 0x3c1, 0x3b9, 0x39c, 0x3cc, 0x3c3, 0x3bf,
0x3bd, 0x39c, 0x3b1, 0x3ba, 0x3bc, 0x3ad, 0x3c1, 0x3bd, 0x3c4, 0x3bf, 0x3a0, 0x3ac,
0x3bb, 0x3bc, 0x3b5, 0x3c1, 0x3a1, 0x3cc, 0x3b8, 0x3b5, 0x3c1, 0x3b1, 0x3a3, 0x3af,
0x3bf, 0x3b2, 0x3b1, 0x3a4, 0x3c1, 0x3bf, 0x3bb, 0x392, 0x3cc, 0x3c3, 0x3c4, 0x3bf,
0x3ba, 0x39b, 0x3cc, 0x3bd, 0x3b3, 0x3ba, 0x3b9, 0x3b5, 0x3b1, 0x3c1, 0x3bc, 0x3c0,
0x3b9, 0x3b5, 0x3bd, 0x386, 0x3bd, 0x3c4, 0x3b5, 0x3bd, 0x391, 0x3bb, 0x3bc, 0x3ac,
0x3c4, 0x3b9, 0x391, 0x3bc, 0x3bc, 0x3ac, 0x3bd, 0x391, 0x3bd, 0x3b1, 0x3bd, 0x3c4,
0x3af, 0x3c1, 0x391, 0x3ba, 0x3c4, 0x3ac, 0x3bf, 0x3c5, 0x391, 0x3ba, 0x3c4, 0x3cc,
0x3bc, 0x3c0, 0x3b5, 0x391, 0x3c3, 0x3c7, 0x3b1, 0x3bc, 0x3c0, 0x3ac, 0x3c4, 0x391,
0x3c4, 0x3b9, 0x3c1, 0x3ac, 0x3bf, 0x3c5, 0x392, 0x3b1, 0x3b3, 0x3b4, 0x3ac, 0x3c4,
0x3b7, 0x39c, 0x3c0, 0x3b1, 0x3c7, 0x3c1, 0x3ad, 0x3b9, 0x3bd, 0x39c, 0x3c0, 0x3b1,
0x3ba, 0x3bf, 0x3cd, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3cc, 0x3ba, 0x39c,
0x3c0, 0x3b1, 0x3c1, 0x3bd, 0x3b1, 0x3bf, 0x3cd, 0x3bb, 0x392, 0x3c5, 0x3c1, 0x3b7,
0x3c4, 0x3cc, 0x3c2, 0x39c, 0x3c0, 0x3b9, 0x3c3, 0x3ba, 0x3ad, 0x3ba, 0x39c, 0x3c0,
0x3c1, 0x3bf, 0x3c5, 0x3bd, 0x3ad, 0x3b9, 0x3a4, 0x3c3, 0x3b9, 0x3c4, 0x3ac, 0x3a4,
0x3c3, 0x3bf, 0x3ca, 0x3bc, 0x3c0, 0x3b1, 0x3bb, 0x3c3, 0x3ac, 0x3bd, 0x39a, 0x3bf,
0x3bb, 0x3cc, 0x3bc, 0x3c0, 0x3bf, 0x394, 0x3b1, 0x3bc, 0x3b1, 0x3c3, 0x3ba, 0x3cc,
0x3c2, 0x39d, 0x3c4, 0x3ac, 0x3ba, 0x3b1, 0x39d, 0x3c4, 0x3af, 0x3bb, 0x3b9, 0x39d,
0x3c4, 0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3ac, 0x3b9, 0x39d, 0x3c4, 0x3bf, 0x3c5, 0x3c3,
0x3ac, 0x3bd, 0x3bc, 0x3c0, 0x3b5, 0x391, 0x3bc, 0x3bc, 0x3cc, 0x3c7, 0x3c9, 0x3c3,
0x3c4, 0x3bf, 0x3c2, 0x393, 0x3ac, 0x3b6, 0x3b1, 0x3a7, 0x3b5, 0x3b2, 0x3c1, 0x3ce,
0x3bd, 0x3b1, 0x3a0, 0x3cc, 0x3bb, 0x3b7, 0x20, 0x3a7, 0x3bf, 0x20, 0x3a4, 0x3c3,
0x3b9, 0x20, 0x39c, 0x3b9, 0x3bd, 0x3c7, 0x3a7, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x20,
0x39a, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x3a7, 0x3bf, 0x3b2, 0x3bd, 0x3c4, 0x399, 0x3c1,
0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x3a4, 0x3b6, 0x3b1, 0x3ba, 0x3ac, 0x3c1,
0x3c4, 0x3b1, 0x3a4, 0x3b6, 0x3b1, 0x3b3, 0x3b9, 0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3c1,
0x3b1, 0x399, 0x3b5, 0x3c1, 0x3bf, 0x3c5, 0x3c3, 0x3b1, 0x3bb, 0x3ae, 0x3bc, 0x39a,
0x3b1, 0x3bc, 0x3c0, 0x3bf, 0x3cd, 0x3bb, 0x39a, 0x3b1, 0x3bc, 0x3c4, 0x3c3, 0x3ac,
0x3c4, 0x3ba, 0x3b1, 0x39a, 0x3b1, 0x3c1, 0x3ac, 0x3c4, 0x3c3, 0x3b9, 0x39a, 0x3b1,
0x3c4, 0x3bc, 0x3b1, 0x3bd, 0x3c4, 0x3bf, 0x3cd, 0x3a7, 0x3b1, 0x3bd, 0x3c4, 0x3af,
0x3b3, 0x3ba, 0x3b1, 0x39a, 0x3b1, 0x3bb, 0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3b1, 0x39a,
0x3c1, 0x3b1, 0x3c3, 0x3bd, 0x3bf, 0x3b3, 0x3b9, 0x3ac, 0x3c1, 0x3c3, 0x3ba, 0x39a,
0x3bf, 0x3c5, 0x3ac, 0x3bb, 0x3b1, 0x20, 0x39b, 0x3bf, 0x3c5, 0x3bc, 0x3c0, 0x3bf,
0x3cd, 0x3c1, 0x39a, 0x3bf, 0x3c5, 0x3c4, 0x3c3, 0x3af, 0x3bd, 0x3b3, 0x3ba, 0x39a,
0x3bf, 0x3c5, 0x3b2, 0x3ad, 0x3b9, 0x3c4, 0x39c, 0x3b1, 0x3ba, 0x3ac, 0x3bf, 0x39c,
0x3b1, 0x3b3, 0x3ba, 0x3b1, 0x3bd, 0x3c4, 0x3ac, 0x3bd, 0x39c, 0x3b1, 0x3ba, 0x3b1,
0x3c3, 0x3ac, 0x3c1, 0x39c, 0x3b1, 0x3bd, 0x3af, 0x3bb, 0x3b1, 0x39c, 0x3b1, 0x3c3,
0x3ba, 0x3ac, 0x3c4, 0x39b, 0x3b5, 0x3c5, 0x3ba, 0x3c9, 0x3c3, 0x3af, 0x3b1, 0x39d,
0x3bf, 0x3b2, 0x3bf, 0x3ba, 0x3bf, 0x3c5, 0x3b6, 0x3bd, 0x3ad, 0x3c4, 0x3c3, 0x3ba,
0x39d, 0x3bf, 0x3b2, 0x3bf, 0x3c3, 0x3b9, 0x3bc, 0x3c0, 0x3af, 0x3c1, 0x3c3, 0x3ba,
0x39f, 0x3bc, 0x3c3, 0x3ba, 0x39f, 0x3c1, 0x3ac, 0x3bb, 0x3a0, 0x3bd, 0x3bf, 0x3bc,
0x20, 0x3a0, 0x3b5, 0x3bd, 0x3c7, 0x3a0, 0x3cc, 0x3bd, 0x3c4, 0x3b9, 0x3b1, 0x3bd,
0x3b1, 0x3ba, 0x3a0, 0x3b9, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x3b3, 0x3b9, 0x3ac, 0x3bd,
0x3b3, 0x3ba, 0x39a, 0x3b1, 0x3c4, 0x3ac, 0x3c1, 0x39a, 0x3bf, 0x3c3, 0x3c4, 0x3b1,
0x3bd, 0x3ac, 0x3b9, 0x39a, 0x3b9, 0x3b6, 0x3b9, 0x3bb, 0x3bf, 0x3c1, 0x3bd, 0x3c4,
0x3ac, 0x3a1, 0x3b9, 0x3ac, 0x3bd, 0x3c4, 0x3a3, 0x3b1, 0x3c7, 0x3b1, 0x3bb, 0x3af,
0x3bd, 0x3b7, 0x3a3, 0x3b1, 0x3bc, 0x3b1, 0x3c1, 0x3ba, 0x3ac, 0x3bd, 0x3b4, 0x3b7,
0x3a3, 0x3b5, 0x3bf, 0x3cd, 0x3bb, 0x3a3, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3ac, 0x3b7,
0x3a3, 0x3b9, 0x3b3, 0x3ba, 0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3c1, 0x3b7, 0x3a3, 0x3c1,
0x3b5, 0x3bd, 0x3c4, 0x3bd, 0x3b5, 0x3ba, 0x3bf, 0x3bb, 0x3af, 0x3bc, 0x3c3, 0x3ba,
0x3a4, 0x3b1, 0x3ca, 0x3c0, 0x3ad, 0x3b9, 0x3a4, 0x3b1, 0x3c3, 0x3ba, 0x3ad, 0x3bd,
0x3b4, 0x3b7, 0x3a4, 0x3b9, 0x3c6, 0x3bb, 0x3af, 0x3b4, 0x3b1, 0x3a4, 0x3b5, 0x3c7,
0x3b5, 0x3c1, 0x3ac, 0x3bd, 0x3b7, 0x398, 0x3af, 0x3bc, 0x3c6, 0x3bf, 0x3c5, 0x3a4,
0x3cc, 0x3ba, 0x3b9, 0x3bf, 0x3a4, 0x3bf, 0x3bc, 0x3c3, 0x3ba, 0x39f, 0x3c5, 0x3bb,
0x3ac, 0x3bd, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3c4, 0x3cc, 0x3c1, 0x39f, 0x3c5, 0x3c1,
0x3bf, 0x3cd, 0x3bc, 0x3c4, 0x3c3, 0x3b9, 0x39f, 0x3c5, 0x3c3, 0x3c4, 0x2d, 0x39d,
0x3b5, 0x3c1, 0x3ac, 0x392, 0x3b9, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3ac, 0x3bd, 0x392,
0x3bb, 0x3b1, 0x3b4, 0x3b9, 0x3b2, 0x3bf, 0x3c3, 0x3c4, 0x3cc, 0x3ba, 0x393, 0x3b9,
0x3b1, 0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x3a1, 0x3b1, 0x3bd, 0x3b3, 0x3ba,
0x3bf, 0x3cd, 0x3bd, 0x391, 0x3b9, 0x3ba, 0x3b1, 0x3c4, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3bf, 0x3cd, 0x3c0, 0x3bf, 0x3bb, 0x3b7, 0x395, 0x3c1, 0x3b5, 0x3b2, 0x3ac, 0x3bd,
0x391, 0x3b6, 0x3cc, 0x3c1, 0x3b5, 0x3c2, 0x392, 0x3b5, 0x3c1, 0x3bc, 0x3bf, 0x3cd,
0x3b4, 0x3b5, 0x3c2, 0x39a, 0x3b1, 0x3bd, 0x3ac, 0x3c1, 0x3b9, 0x3b1, 0x3a0, 0x3c1,
0x3ac, 0x3c3, 0x3b9, 0x3bd, 0x3bf, 0x20, 0x391, 0x3ba, 0x3c1, 0x3c9, 0x3c4, 0x3ae,
0x3c1, 0x3b9, 0x3bf, 0x3a6, 0x3b5, 0x3c1, 0x3cc, 0x3b5, 0x3c2, 0x39c, 0x3b1, 0x3b4,
0x3ad, 0x3c1, 0x3b1, 0x3a1, 0x3ad, 0x3c5, 0x3ba, 0x3b9, 0x3b1, 0x3b2, 0x3b9, 0x3ba,
0x39d, 0x3cc, 0x3c4, 0x3b9, 0x3b1, 0x20, 0x393, 0x3b5, 0x3c9, 0x3c1, 0x3b3, 0x3af,
0x3b1, 0x391, 0x3b3, 0x2e, 0x20, 0x395, 0x3bb, 0x3ad, 0x3bd, 0x3b7, 0x3a3, 0x3c4,
0x3ac, 0x3bd, 0x3bb, 0x3b5, 0x3ca, 0x391, 0x3b4, 0x3b5, 0x3bb, 0x3b1, 0x390, 0x3b4,
0x3b1, 0x39c, 0x3c0, 0x3c1, 0x3b9, 0x3c3, 0x3bc, 0x3c0, 0x3ad, 0x3b9, 0x3bd, 0x39c,
0x3c0, 0x3c1, 0x3cc, 0x3ba, 0x3b5, 0x3bd, 0x20, 0x3a7, 0x3b9, 0x3bb, 0x39d, 0x3c4,
0x3ac, 0x3c1, 0x3b3, 0x3bf, 0x3c5, 0x3b9, 0x3bd, 0x393, 0x3b9, 0x3bf, 0x3cd, 0x3ba,
0x3bb, 0x3b1, 0x3a7, 0x3cc, 0x3bc, 0x3c0, 0x3b1, 0x3c1, 0x3c4, 0x39b, 0x3af, 0x3bd,
0x3c4, 0x3b5, 0x3bc, 0x3b1, 0x3bd, 0x39b, 0x3bf, 0x3c1, 0x3bd, 0x3c4, 0x20, 0x3a7,
0x3ac, 0x3bf, 0x3c5, 0x39c, 0x3b5, 0x3bb, 0x3b2, 0x3bf, 0x3cd, 0x3c1, 0x3bd, 0x3b7,
0x3a0, 0x3b5, 0x3c1, 0x3b8, 0x3a3, 0x3af, 0x3b4, 0x3bd, 0x3b5, 0x3ca, 0x386, 0x3bc,
0x3c3, 0x3c4, 0x3b5, 0x3c1, 0x3bd, 0x3c4, 0x3b1, 0x3bc, 0x391, 0x3bd, 0x3b4, 0x3cc,
0x3c1, 0x3b1, 0x391, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3c7, 0x3ac, 0x3bd, 0x391, 0x3b8,
0x3ae, 0x3bd, 0x3b1, 0x392, 0x3b5, 0x3bb, 0x3b9, 0x3b3, 0x3c1, 0x3ac, 0x3b4, 0x3b9,
0x392, 0x3b5, 0x3c1, 0x3bf, 0x3bb, 0x3af, 0x3bd, 0x3bf, 0x39c, 0x3c0, 0x3c1, 0x3b1,
0x3c4, 0x3b9, 0x3c3, 0x3bb, 0x3ac, 0x3b2, 0x3b1, 0x392, 0x3c1, 0x3c5, 0x3be, 0x3ad,
0x3bb, 0x3bb, 0x3b5, 0x3c2, 0x392, 0x3bf, 0x3c5, 0x3ba, 0x3bf, 0x3c5, 0x3c1, 0x3ad,
0x3c3, 0x3c4, 0x3b9, 0x392, 0x3bf, 0x3c5, 0x3b4, 0x3b1, 0x3c0, 0x3ad, 0x3c3, 0x3c4,
0x3b7, 0x39c, 0x3c0, 0x3af, 0x3c3, 0x3b9, 0x3bd, 0x3b3, 0x3ba, 0x3b5, 0x3bd, 0x39a,
0x3b9, 0x3c3, 0x3b9, 0x3bd, 0x3ac, 0x3bf, 0x3c5, 0x39a, 0x3bf, 0x3c0, 0x3b5, 0x3b3,
0x3c7, 0x3ac, 0x3b3, 0x3b7, 0x394, 0x3bf, 0x3c5, 0x3b2, 0x3bb, 0x3af, 0x3bd, 0x3bf,
0x393, 0x3b9, 0x3b2, 0x3c1, 0x3b1, 0x3bb, 0x3c4, 0x3ac, 0x3c1, 0x393, 0x3ba, 0x3ad,
0x3c1, 0x3bd, 0x3b6, 0x3b9, 0x395, 0x3bb, 0x3c3, 0x3af, 0x3bd, 0x3ba, 0x3b9, 0x39d,
0x3ae, 0x3c3, 0x3bf, 0x3c2, 0x20, 0x3c4, 0x3bf, 0x3c5, 0x20, 0x39c, 0x3b1, 0x3bd,
0x39a, 0x3c9, 0x3bd, 0x3c3, 0x3c4, 0x3b1, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3bf, 0x3cd,
0x3c0, 0x3bf, 0x3bb, 0x3b7, 0x3a4, 0x3b6, 0x3ad, 0x3c1, 0x3c3, 0x3b5, 0x3ca, 0x39a,
0x3b1, 0x3bb, 0x3af, 0x3bd, 0x3b9, 0x3bd, 0x3b3, 0x3ba, 0x3c1, 0x3b1, 0x3bd, 0x3c4,
0x39a, 0x3af, 0x3c1, 0x3bf, 0x3c6, 0x39a, 0x3af, 0x3b5, 0x3b2, 0x3bf, 0x39b, 0x3b9,
0x3c3, 0x3b1, 0x3b2, 0x3cc, 0x3bd, 0x3b1, 0x39b, 0x3b9, 0x3bf, 0x3c5, 0x3bc, 0x3c0,
0x3bb, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x39b, 0x3bf, 0x3bd, 0x3b4, 0x3af, 0x3bd, 0x3bf,
0x39b, 0x3bf, 0x3c5, 0x3be, 0x3b5, 0x3bc, 0x3b2, 0x3bf, 0x3cd, 0x3c1, 0x3b3, 0x3bf,
0x39c, 0x3b1, 0x3b4, 0x3c1, 0x3af, 0x3c4, 0x3b7, 0x39c, 0x3ac, 0x3bb, 0x3c4, 0x3b1,
0x39c, 0x3ac, 0x3c1, 0x3b9, 0x3b5, 0x3c7, 0x3b1, 0x3bc, 0x3bd, 0x39c, 0x3b9, 0x3bd,
0x3c3, 0x3ba, 0x39c, 0x3bf, 0x3bd, 0x3b1, 0x3ba, 0x3cc, 0x39c, 0x3cc, 0x3c3, 0x3c7,
0x3b1, 0x38c, 0x3c3, 0x3bb, 0x3bf, 0x3a0, 0x3b1, 0x3c1, 0x3af, 0x3c3, 0x3b9, 0x3a0,
0x3bf, 0x3bd, 0x3c4, 0x3b3, 0x3ba, 0x3cc, 0x3c1, 0x3b9, 0x3c4, 0x3c3, 0x3b1, 0x3a0,
0x3c1, 0x3ac, 0x3b3, 0x3b1, 0x3a1, 0x3af, 0x3b3, 0x3b1, 0x3a1, 0x3ce, 0x3bc, 0x3b7,
0x3a3, 0x3b1, 0x3bc, 0x3ac, 0x3c1, 0x3b1, 0x386, 0x3b3, 0x3b9, 0x3bf, 0x3c2, 0x20,
0x39c, 0x3b1, 0x3c1, 0x3af, 0x3bd, 0x3bf, 0x3c2, 0x3a3, 0x3b1, 0x3c1, 0x3ac, 0x3b3,
0x3b5, 0x3b2, 0x3bf, 0x3a3, 0x3b1, 0x3c1, 0x3ac, 0x3c4, 0x3bf, 0x3c6, 0x3a3, 0x3c5,
0x3bc, 0x3c6, 0x3b5, 0x3c1, 0x3cc, 0x3c0, 0x3bf, 0x3bb, 0x3b7, 0x3a3, 0x3ba, 0x3cc,
0x3c0, 0x3b9, 0x3b1, 0x3a3, 0x3cc, 0x3c6, 0x3b9, 0x3b1, 0x3a3, 0x3c4, 0x3bf, 0x3ba,
0x3c7, 0x3cc, 0x3bb, 0x3bc, 0x3b7, 0x3a4, 0x3b1, 0x3bb, 0x3af, 0x3bd, 0x3a4, 0x3af,
0x3c1, 0x3b1, 0x3bd, 0x3b1, 0x39f, 0x3c5, 0x3bb, 0x3b9, 0x3ac, 0x3bd, 0x3bf, 0x3c6,
0x3c3, 0x3ba, 0x392, 0x3b1, 0x3bd, 0x3c4, 0x3bf, 0x3cd, 0x3b6, 0x392, 0x3b1, 0x3c4,
0x3b9, 0x3ba, 0x3b1, 0x3bd, 0x3cc, 0x392, 0x3b9, 0x3ad, 0x3bd, 0x3bd, 0x3b7, 0x392,
0x3af, 0x3bb, 0x3bd, 0x3b9, 0x3bf, 0x3c5, 0x3c2, 0x392, 0x3cc, 0x3bb, 0x3b3, 0x3ba,
0x3bf, 0x3ba, 0x3c1, 0x3b1, 0x3bd, 0x3c4, 0x392, 0x3b1, 0x3c1, 0x3c3, 0x3bf, 0x3b2,
0x3af, 0x3b1, 0x396, 0x3ac, 0x3b3, 0x3ba, 0x3c1, 0x3b5, 0x3bc, 0x3c0, 0x396, 0x3c5,
0x3c1, 0x3af, 0x3c7, 0x3b7, 0x391, 0x3bd, 0x3c4, 0x3b1, 0x3bd, 0x3b1, 0x3bd, 0x3b1,
0x3c1, 0x3af, 0x3b2, 0x3bf, 0x3a4, 0x3c3, 0x3ac, 0x3b3, 0x3ba, 0x3bf, 0x3c2, 0x39d,
0x3ae, 0x3c3, 0x3bf, 0x3c2, 0x20, 0x3a7, 0x3c1, 0x3b9, 0x3c3, 0x3c4, 0x3bf, 0x3c5,
0x3b3, 0x3ad, 0x3bd, 0x3bd, 0x3c9, 0x3bd, 0x39a, 0x3cc, 0x3ba, 0x3bf, 0x3c2, 0x39a,
0x3bf, 0x3bc, 0x3cc, 0x3c1, 0x3bf, 0x39a, 0x3b5, 0x3c1, 0x3b3, 0x3ba, 0x3b5, 0x3bb,
0x3ad, 0x3bd, 0x39c, 0x3ac, 0x3c7, 0x3b5, 0x39c, 0x3b1, 0x3bb, 0x3b4, 0x3af, 0x3b2,
0x3b5, 0x3c2, 0x39c, 0x3b1, 0x3c5, 0x3c1, 0x3af, 0x3ba, 0x3b9, 0x3bf, 0x3c2, 0x39c,
0x3b1, 0x3b3, 0x3b9, 0x3cc, 0x3c4, 0x3a1, 0x3b5, 0x3ca, 0x3bd, 0x3b9, 0x3cc, 0x3bd,
0x391, 0x3c0, 0x3af, 0x3b1, 0x38c, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x39c, 0x3c0,
0x3bf, 0x3c5, 0x3b3, 0x3ba, 0x3b5, 0x3bd, 0x3b2, 0x3af, 0x3bb, 0x3a4, 0x3c3, 0x3ac,
0x3c4, 0x3b1, 0x3bc, 0x3a4, 0x3c3, 0x3bf, 0x3c5, 0x3ba, 0x39d, 0x3ae, 0x3c3, 0x3bf,
0x3c2, 0x20, 0x3a0, 0x3ac, 0x3c3, 0x3c7, 0x3b1, 0x395, 0x3c6, 0x3ac, 0x3c4, 0x3b5,
0x3a6, 0x3b1, 0x3ba, 0x3b1, 0x3cc, 0x3c6, 0x3bf, 0x3a6, 0x3af, 0x3c4, 0x3b6, 0x3b9,
0x3a6, 0x3bf, 0x3c5, 0x3bd, 0x3b1, 0x3c6, 0x3bf, 0x3cd, 0x3c4, 0x3b9, 0x393, 0x3ba,
0x3b1, 0x3bb, 0x3ac, 0x3c0, 0x3b1, 0x3b3, 0x3ba, 0x3bf, 0x3c2, 0x393, 0x3ba, 0x3ac,
0x3bc, 0x3c0, 0x3b9, 0x3b5, 0x3c1, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3b1, 0x3bd, 0x3c4,
0x3b1, 0x3bb, 0x3ba, 0x3b1, 0x3bd, 0x3ac, 0x3bb, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3ac,
0x3bc, 0x3a7, 0x3bf, 0x3bd, 0x3bf, 0x3bb, 0x3bf, 0x3c5, 0x3bb, 0x3bf, 0x3cd, 0x388,
0x3bd, 0x3c4, 0x3b5, 0x3c1, 0x3bc, 0x3c0, 0x3b5, 0x3c1, 0x3b9, 0x39a, 0x3b9, 0x3c1,
0x3b9, 0x3c4, 0x3b9, 0x3bc, 0x3ac, 0x3c4, 0x3b9, 0x39a, 0x3cc, 0x3c3, 0x3c1, 0x3b1,
0x3ca, 0x39a, 0x3bf, 0x3c5, 0x3b1, 0x3c4, 0x3b6, 0x3b1, 0x3bb, 0x3ad, 0x3b9, 0x3bd,
0x39c, 0x3b1, 0x3c4, 0x3b6, 0x3bf, 0x3cd, 0x3c1, 0x3bf, 0x39c, 0x3b1, 0x3c1, 0x3ba,
0x3ad, 0x3b6, 0x3b1, 0x3c2, 0x39c, 0x3af, 0x3bd, 0x3c4, 0x3b3, 0x3bf, 0x3c5, 0x3b5,
0x3ca, 0x39d, 0x3b1, 0x3bf, 0x3cd, 0x3c1, 0x3bf, 0x3c5, 0x39d, 0x3b9, 0x3bf, 0x3cd,
0x3b5, 0x39d, 0x3cc, 0x3c1, 0x3c6, 0x3bf, 0x3bb, 0x3ba, 0x39d, 0x3bf, 0x3c5, 0x3bc,
0x3ad, 0x3b1, 0x3a0, 0x3ac, 0x3b3, 0x3ba, 0x3bf, 0x20, 0x3a0, 0x3ac, 0x3b3, 0x3ba,
0x3bf, 0x3a0, 0x3b1, 0x3bb, 0x3ac, 0x3bf, 0x3c5, 0x3a0, 0x3af, 0x3c4, 0x3ba, 0x3b5,
0x3c1, 0x3bd, 0x3a0, 0x3bf, 0x3bd, 0x3ac, 0x3c0, 0x3b5, 0x3a0, 0x3bf, 0x3c1, 0x3c4,
0x20, 0x39c, 0x3cc, 0x3c1, 0x3b5, 0x3c3, 0x3bc, 0x3c0, 0x3b9, 0x3a1, 0x3b1, 0x3c1,
0x3bf, 0x3c4, 0x3cc, 0x3bd, 0x3b3, 0x3ba, 0x3b1, 0x3a3, 0x3b1, 0x3ca, 0x3c0, 0x3ac,
0x3bd, 0x3a4, 0x3b1, 0x3ca, 0x3c4, 0x3ae, 0x3a4, 0x3b1, 0x3c1, 0x3ac, 0x3bf, 0x3c5,
0x3b1, 0x3a4, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x3b1, 0x3c4, 0x3ac, 0x3c0, 0x3bf, 0x3c5,
0x393, 0x3bf, 0x3c5, 0x3ad, 0x3b9, 0x3ba, 0x393, 0x3bf, 0x3c5, 0x3ac, 0x3bb, 0x3b9,
0x3c2, 0xa86, 0xaac, 0xabf, 0xaa6, 0xacd, 0xa9c, 0xabe, 0xaa8, 0xa8d, 0xa95, 0xab0,
0xabe, 0xa8f, 0xaa1, 0xabf, 0xab8, 0x20, 0xa85, 0xaac, 0xabe, 0xaac, 0xabe, 0xa85,
0xab2, 0xacd, 0xa9c, 0xac0, 0xaaf, 0xab0, 0xacd, 0xab8, 0xa85, 0xab8, 0xacd, 0xaae,
0xabe, 0xab0, 0xabe, 0xaac, 0xaae, 0xac7, 0xa95, 0xacb, 0xaac, 0xabe, 0xa82, 0xa97,
0xac1, 0xa88, 0xaac, 0xabe, 0xa82, 0xa9c, 0xac1, 0xab2, 0xaac, 0xabf, 0xab8, 0xabe,
0xa89, 0xaac, 0xacd, 0xab2, 0xac7, 0xa82, 0xa9f, 0xac0, 0xab0, 0xaac, 0xacd, 0xab0,
0xabe, 0xa9d, 0xabe, 0xab5, 0xabf, 0xab2, 0xac7, 0xaac, 0xac1, 0xa9c, 0xaae, 0xacd,
0xaac, 0xac1, 0xab0, 0xabe, 0xa95, 0xac8, 0xab0, 0xacb, 0xa95, 0xabe, 0xab8, 0xabe,
0xaac, 0xacd, 0xab2, 0xabe, 0xa82, 0xa95, 0xabe, 0xa95, 0xacd, 0xab5, 0xac7, 0xa9f,
0xabe, 0xa95, 0xacb, 0xaa8, 0xac5, 0xa95, 0xacd, 0xab0, 0xac0, 0xaa1, 0xa95, 0xabe,
0xab0, 0xaa6, 0xabe, 0xab0, 0x20, 0xa8f, 0xab8, 0x20, 0xab8, 0xab2, 0xabe, 0xaae,
0xa9c, 0xac0, 0xaac, 0xac1, 0xa9f, 0xac0, 0xaa1, 0xacc, 0xa86, 0xab2, 0xabe, 0xa8f,
0xab2, 0x20, 0xa90, 0xa89, 0xaa8, 0xaab, 0xacd, 0xab0, 0xac0, 0xa9f, 0xabe, 0xa89,
0xaa8, 0xa97, 0xac5, 0xaac, 0xacb, 0xab0, 0xacb, 0xaa8, 0xab9, 0xab0, 0xabe, 0xab0,
0xac7, 0xa9c, 0xacb, 0xab9, 0xabe, 0xaa8, 0xacd, 0xab8, 0xaac, 0xab0, 0xacd, 0xa97,
0xa9c, 0xac1, 0xaac, 0xabe, 0xa95, 0xaae, 0xacd, 0xaaa, 0xabe, 0xab2, 0xabe, 0xa96,
0xabe, 0xab0, 0xacd, 0xa9f, 0xac2, 0xaae, 0xa95, 0xabf, 0xa97, 0xabe, 0xab2, 0xac0,
0xa95, 0xabf, 0xa82, 0xab6, 0xabe, 0xab8, 0xabe, 0xab2, 0xabe, 0xa97, 0xacb, 0xab8,
0xab2, 0xabf, 0xaac, 0xacd, 0xab0, 0xac7, 0xab5, 0xabf, 0xab2, 0xac7, 0xab2, 0xacb,
0xaae, 0xab2, 0xacd, 0xaaf, 0xac1, 0xa8f, 0xaa8, 0xacd, 0xaa1, 0xabe, 0xab2, 0xac1,
0xaac, 0xac1, 0xaae, 0xacd, 0xaac, 0xabe, 0xab6, 0xac0, 0xab2, 0xac1, 0xab8, 0xabe,
0xa95, 0xabe, 0xaae, 0xab2, 0xabe, 0xaac, 0xacb, 0xaae, 0xabe, 0xaaa, 0xac1, 0xa9f,
0xacb, 0xaae, 0xab8, 0xac7, 0xab0, 0xac1, 0xa85, 0xaae, 0xacd, 0xaac, 0xaac, 0xabe,
0xaa8, 0xaae, 0xacb, 0xa97, 0xabe, 0xaa1, 0xac0, 0xab6, 0xac1, 0xaae, 0xacb, 0xa82,
0xab0, 0xacb, 0xab5, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xac8, 0xab0, 0xacb, 0xaac, 0xac0,
0xa85, 0xaa8, 0xacd, 0xa9c, 0xaae, 0xac7, 0xaa8, 0xaa8, 0xabf, 0xaaf, 0xabe, 0xaae,
0xac0, 0xaa8, 0xacc, 0xa95, 0xa9a, 0xacb, 0xa9f, 0xa94, 0xa86, 0xa97, 0xabe, 0xaa6,
0xacc, 0xa97, 0xacc, 0xaaa, 0xacb, 0xab0, 0xacd, 0xa9f, 0xacb, 0x2d, 0xaa8, 0xacb,
0xab5, 0xacb, 0xab8, 0xabe, 0xa93, 0x20, 0xa9f, 0xac9, 0xaae, 0xa9f, 0xacd, 0xab0,
0xac0, 0xaaa, 0xacb, 0xab2, 0xac0, 0xa9f, 0xacd, 0xaaf, 0xac1, 0xaa8, 0xabf, 0xab8,
0xab5, 0xabf, 0xa82, 0xaa1, 0xab9, 0xac9, 0xa95, 0xa85, 0xaa6, 0xa95, 0xa8f, 0xaa8,
0xacd, 0xa95, 0xacb, 0xab0, 0xac7, 0xa9c, 0xa8d, 0xa82, 0xa97, 0xacd, 0xab5, 0xabf,
0xab2, 0xabe, 0xa8d, 0xaa8, 0xacd, 0xa9f, 0xabf, 0xa97, 0xac1, 0xa86, 0xa85, 0xab0,
0xabe, 0xa97, 0xac1, 0xa87, 0xaa8, 0xabe, 0xaac, 0xacd, 0xaaf, 0xac1, 0xaa8, 0xab8,
0x20, 0xa8f, 0xaaf, 0xab0, 0xacd, 0xab8, 0xa95, 0xabe, 0xaa4, 0xabe, 0xaae, 0xabe,
0xab0, 0xacd, 0xa95, 0xabe, 0xa95, 0xacb, 0xab0, 0xacd, 0xaa1, 0xacb, 0xaac, 0xabe,
0xab9, 0xac1, 0xab9, 0xac1, 0xa87, 0xab2, 0xabe, 0x20, 0xab0, 0xabf, 0xa93, 0xab9,
0xabe, 0xaae, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacb, 0xa9d, 0xabe, 0xab0, 0xabf, 0xa93,
0x20, 0xa97, 0xac7, 0xab2, 0xac7, 0xa97, 0xacb, 0xab8, 0xab8, 0xabe, 0xab2, 0xacd,
0xa9f, 0xabe, 0xab8, 0xac7, 0xaa8, 0x20, 0xa9c, 0xac1, 0xa8f, 0xaa8, 0xab8, 0xac7,
0xaa8, 0x20, 0xab2, 0xac1, 0xa87, 0xab8, 0xaa4, 0xac1, 0xa95, 0xac1, 0xaae, 0xabe,
0xaa8, 0xa89, 0xab6, 0xabe, 0xaaf, 0xabe, 0xa85, 0xab0, 0xac1, 0xaac, 0xabe, 0xa8f,
0xab8, 0xaa8, 0xacd, 0xab6, 0xabf, 0xaaf, 0xaa8, 0xa8f, 0xa9f, 0xabf, 0xa95, 0xacb,
0xa95, 0xac7, 0xaa8, 0xaac, 0xabe, 0xab9, 0xabf, 0xaaf, 0xabe, 0xaac, 0xab9, 0xabf,
0xaaf, 0xabe, 0x20, 0xaac, 0xabe, 0xa82, 0xaa6, 0xacd, 0xab0, 0xabe, 0xab8, 0xaac,
0xabe, 0xab0, 0xaac, 0xabe, 0xaa1, 0xacb, 0xab8, 0xaac, 0xac7, 0xab2, 0xac7, 0xaae,
0xaac, 0xac7, 0xab2, 0xac0, 0xa9d, 0xaac, 0xacd, 0xab2, 0xabe, 0xa82, 0xa95, 0x2d,
0xab8, 0xac7, 0xaac, 0xab2, 0xacb, 0xaa8, 0xaac, 0xacb, 0xa86, 0x20, 0xab5, 0xabf,
0xab8, 0xacd, 0xa9f, 0xabe, 0xaac, 0xacb, 0xa97, 0xacb, 0xa9f, 0xabe, 0xaac, 0xacb,
0xa87, 0xa9d, 0xa95, 0xac7, 0xaae, 0xacd, 0xaac, 0xacd, 0xab0, 0xabf, 0xa9c, 0x20,
0xaac, 0xac7, 0xa95, 0xabe, 0xaae, 0xacd, 0xaaa, 0xacb, 0x20, 0xa97, 0xacd, 0xab0,
0xabe, 0xa82, 0xaa1, 0xa95, 0xabe, 0xaa8, 0xacd, 0xa95, 0xac1, 0xaa8, 0xa95, 0xac5,
0xab0, 0xabe, 0xa95, 0xab8, 0xa95, 0xabe, 0xaaf, 0xac7, 0xaa8, 0xac7, 0xa95, 0xac7,
0xaae, 0xac7, 0xaa8, 0xab6, 0xabf, 0xa95, 0xabe, 0xa97, 0xacb, 0xa9a, 0xabf, 0xab9,
0xac1, 0xa86, 0xab9, 0xac1, 0xa86, 0xab8, 0xabf, 0xaaf, 0xac1, 0xaa6, 0xabe, 0xaa6,
0x20, 0xab9, 0xac1, 0xa86, 0xab0, 0xac7, 0xa9d, 0xa95, 0xacb, 0xab8, 0xacd, 0xa9f,
0xabe, 0x20, 0xab0, 0xabf, 0xa95, 0xabe, 0xa95, 0xacd, 0xab0, 0xac7, 0xab8, 0xacd,
0xa9f, 0xaa8, 0xa95, 0xacd, 0xaaf, 0xac1, 0xa8f, 0xaac, 0xabe, 0xa95, 0xac1, 0xab0,
0xabe, 0xa95, 0xabe, 0xa93, 0xaa1, 0xac7, 0xaa8, 0xaae, 0xabe, 0xab0, 0xacd, 0xa95,
0xab6, 0xac9, 0xaa8, 0xaa1, 0xacc, 0xab8, 0xaa8, 0xaa1, 0xacc, 0xab8, 0xaa8, 0x20,
0xa95, 0xacd, 0xab0, 0xabf, 0xa95, 0xaa6, 0xac7, 0xaa8, 0xacd, 0xab5, 0xab0, 0xaa1,
0xac7, 0xa9f, 0xacd, 0xab0, 0xacb, 0xa87, 0xa9f, 0xaa1, 0xacb, 0xaae, 0xabf, 0xaa8,
0xabf, 0xa95, 0xabe, 0xa8f, 0xaa1, 0xaae, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xaa8, 0xa87,
0xab0, 0xac1, 0xaa8, 0xac7, 0xaaa, 0xa8f, 0xab2, 0x20, 0xab8, 0xac7, 0xab2, 0xacd,
0xab5, 0xabe, 0xaa1, 0xacb, 0xab0, 0xaab, 0xacb, 0xab0, 0xacd, 0xa9f, 0x20, 0xaa8,
0xac7, 0xab2, 0xacd, 0xab8, 0xaa8, 0xaab, 0xacb, 0xab0, 0xacd, 0xa9f, 0xabe, 0xab2,
0xac7, 0xa9d, 0xabe, 0xa97, 0xacd, 0xab2, 0xac7, 0xab8, 0x20, 0xaac, 0xac7, 0xa97,
0xac2, 0xab8, 0x20, 0xaac, 0xac7, 0xa97, 0xacd, 0xab0, 0xabe, 0xaa8, 0xacd, 0xaa1,
0x20, 0xa9f, 0xab0, 0xacd, 0xa95, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa8, 0xac7, 0xaa1,
0xabe, 0xa97, 0xacd, 0xab5, 0xabe, 0xaa1, 0xac7, 0xab2, 0xacb, 0xaaa, 0xa97, 0xacd,
0xab5, 0xabe, 0xa9f, 0xac7, 0xaae, 0xabe, 0xab2, 0xabe, 0xa97, 0xac1, 0xaaf, 0xabe,
0xa95, 0xacd, 0xab5, 0xabf, 0xab2, 0xa97, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xab9, 0xac7,
0xab2, 0xabf, 0xaab, 0xac7, 0xa95, 0xacd, 0xab8, 0xab9, 0xab5, 0xabe, 0xaa8, 0xabe,
0xa8f, 0xab0, 0xaae, 0xacb, 0xab8, 0xabf, 0xaaf, 0xacb, 0xa87, 0xaa8, 0xacd, 0xaa1,
0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xaaa, 0xacb, 0xab2, 0xabf, 0xab8, 0xaa8, 0xacb,
0xa95, 0xacd, 0xab8, 0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe,
0xaa8, 0xabe, 0xaae, 0xac7, 0xab0, 0xac7, 0xa82, 0xa97, 0xacb, 0x2c, 0x20, 0xa87,
0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xaaa, 0xabf, 0xa9f, 0xab0,
0xacd, 0xab8, 0xaac, 0xab0, 0xacd, 0xa97, 0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1,
0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xa9f, 0xac7, 0xab2, 0x20, 0xab8, 0xabf, 0xa9f,
0xac0, 0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe,
0xab5, 0xac7, 0xab5, 0xac8, 0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf,
0xabe, 0xaa8, 0xabe, 0xab5, 0xabf, 0xaa8, 0xacd, 0xab8, 0xac7, 0xaa8, 0xacd, 0xab8,
0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xab5,
0xabf, 0xaa8, 0xabe, 0xaae, 0xac7, 0xa95, 0x2c, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1,
0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xa87, 0xaa8, 0xac1, 0xab5, 0xabf, 0xa95, 0xa87,
0xa95, 0xabe, 0xab2, 0xac1, 0xa87, 0xaa4, 0xa9c, 0xaae, 0xac8, 0xa95, 0xabe, 0xa9c,
0xac1, 0xaa8, 0xac7, 0xa87, 0xab2, 0xac2, 0xa87, 0xab5, 0xabf, 0xab2, 0xaae, 0xacb,
0xaa8, 0xacd, 0xa9f, 0xac0, 0xab8, 0xac7, 0xab2, 0xacb, 0x2c, 0x20, 0xa95, 0xac7,
0xaa8, 0xacd, 0xa9f, 0xac1, 0xa95, 0xac0, 0xa95, 0xacd, 0xab0, 0xabe, 0xab2, 0xac7,
0xaa8, 0xacd, 0xaa1, 0xabf, 0xa95, 0xab2, 0xabe, 0x20, 0xaaa, 0xac7, 0xa9d, 0xab2,
0xabf, 0xaae, 0xabe, 0xab2, 0xacb, 0xab8, 0x20, 0xa8f, 0xa82, 0xa9c, 0xac7, 0xab2,
0xac7, 0xab8, 0xab2, 0xacb, 0xa85, 0xab0, 0x20, 0xaaa, 0xacd, 0xab0, 0xabf, 0xaa8,
0xacd, 0xab8, 0xaa8, 0xac1, 0xa82, 0x20, 0xa95, 0xacd, 0xab5, 0xabe, 0xab0, 0xacd,
0xa9f, 0xab0, 0xaae, 0xac7, 0xab8, 0xabf, 0xa93, 0xaae, 0xabe, 0xaa8, 0xabe, 0xa97,
0xac1, 0xa86, 0xaae, 0xaa8, 0xacc, 0xab8, 0xaae, 0xac7, 0xab0, 0xac0, 0xa97, 0xacb,
0xa9f, 0xaae, 0xabe, 0xab0, 0xacd, 0xa9f, 0xac0, 0xaa8, 0xac0, 0xa95, 0xaae, 0xabe,
0xaa4, 0xabe, 0xaae, 0xacb, 0xab0, 0xacb, 0xab8, 0xaae, 0xa9d, 0xaa4, 0xacd, 0xab2,
0xabe, 0xaa8, 0xaae, 0xac7, 0xaa8, 0xacb, 0xaae, 0xabf, 0xaa8, 0xac0, 0xaae, 0xac7,
0xab0, 0xabf, 0xaa6, 0xabe, 0xaae, 0xac7, 0xa9f, 0xacd, 0xab2, 0xabe, 0xa95, 0xabe,
0xa9f, 0xacd, 0xab2, 0xabe, 0xaae, 0xac7, 0xa95, 0xacd, 0xab8, 0xabf, 0xa95, 0xacb,
0x20, 0xab8, 0xabf, 0xa9f, 0xac0, 0xaae, 0xabf, 0xa95, 0xacd, 0xaaf, 0xac1, 0xab2,
0xacb, 0xaa8, 0xaae, 0xacb, 0xaa8, 0xacd, 0xa95, 0xa9f, 0xaa8, 0xaae, 0xacb, 0xaa8,
0xacd, 0xaa4, 0xacb, 0xab0, 0xac7, 0xaae, 0xac9, 0xa82, 0xa9f, 0xac7, 0xab5, 0xabf,
0xaa1, 0xabf, 0xa93, 0xaae, 0xacb, 0xa82, 0xa9f, 0xab8, 0xac7, 0xab0, 0xabe, 0xaa4,
0xaa8, 0xabe, 0xab8, 0xabe, 0xa89, 0xaa8, 0xacd, 0xaaf, 0xac2, 0xaaf, 0xacb, 0xab0,
0xacd, 0xa95, 0xaa8, 0xacb, 0xab0, 0xacb, 0xaa8, 0xacd, 0xab9, 0xabe, 0xaac, 0xabf,
0xaaf, 0xac1, 0xab2, 0xabe, 0xab9, 0x2c, 0x20, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0,
0x20, 0xaa1, 0xac7, 0xa95, 0xacb, 0xa9f, 0xabe, 0xab8, 0xac7, 0xaa8, 0xacd, 0xa9f,
0xab0, 0x2c, 0x20, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xaa1, 0xac7, 0xa95,
0xacb, 0xa9f, 0xabe, 0xaa8, 0xacd, 0xaaf, 0xac1, 0x20, 0xab8, 0xac7, 0xab2, 0xaae,
0x2c, 0x20, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xaa1, 0xac7, 0xa95, 0xacb,
0xa9f, 0xabe, 0xaa8, 0xac2, 0xa95, 0xa93, 0xa9c, 0xabf, 0xaa8, 0xabe, 0xa97, 0xabe,
0xaaa, 0xaa8, 0xabe, 0xaae, 0xabe, 0xaaa, 0xac7, 0xab0, 0xabe, 0xaae, 0xac7, 0xab0,
0xabf, 0xaac, 0xacb, 0xaab, 0xacb, 0xaa8, 0xabf, 0xa95, 0xacd, 0xab8, 0xaaa, 0xacb,
0xab0, 0xacd, 0xa9f, 0x2d, 0xa93, 0x2d, 0xaaa, 0xacd, 0xab0, 0xabf, 0xaa8, 0xacd,
0xab8, 0xaaa, 0xacb, 0xab0, 0xacd, 0xa9f, 0x20, 0xa91, 0xaab, 0x20, 0xab8, 0xacd,
0xaaa, 0xac7, 0xaa8, 0xaaa, 0xacb, 0xab0, 0xacd, 0xa9f, 0xacb, 0x20, 0xab5, 0xac7,
0xab2, 0xacd, 0xab9, 0xacb, 0xaaa, 0xacd, 0xaaf, 0xac1, 0xa85, 0xab0, 0xacd, 0xa9f,
0xacb, 0x20, 0xab0, 0xabf, 0xa95, 0xacb, 0xaaa, 0xac1, 0xaa8, 0xacd, 0xa9f, 0xabe,
0x20, 0xa8f, 0xab0, 0xac0, 0xaa8, 0xabe, 0xa9d, 0xab0, 0xac7, 0xa82, 0xa95, 0xabf,
0xaa8, 0x20, 0xa87, 0xaa8, 0xacd, 0xab2, 0xac7, 0xa9f, 0xab0, 0xac7, 0xab8, 0xac0,
0xaab, 0xab0, 0xac7, 0xa9c, 0xac0, 0xaa8, 0xabe, 0xab0, 0xac7, 0xa9d, 0xacb, 0xab2,
0xac2, 0xa9f, 0xab0, 0xabf, 0xaaf, 0xacb, 0x20, 0xaac, 0xacd, 0xab0, 0xabe, 0xa82,
0xa95, 0xacb, 0xab8, 0xac7, 0xaa8, 0xacd, 0xaa4, 0xabe, 0xab0, 0xac7, 0xaae, 0xab8,
0xabe, 0xa82, 0xa9f, 0xabf, 0xaaf, 0xabe, 0xa97, 0xacb, 0xab8, 0xac7, 0xa82, 0xa9f,
0xacb, 0x20, 0xaa1, 0xacb, 0xaae, 0xabf, 0xa82, 0xa97, 0xacb, 0xab8, 0xabe, 0xa93,
0x20, 0xaaa, 0xabe, 0xa89, 0xab2, 0xacb, 0xa87, 0xaa4, 0xacb, 0xa95, 0xacb, 0xab0,
0xacd, 0xa9f, 0xacb, 0xab0, 0xacb, 0xaae, 0xabf, 0xa9f, 0xab8, 0xabf, 0xa9f, 0xacd,
0xa95, 0xabe, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xaac, 0xabe, 0xab0, 0xacd, 0xaa5,
0xac7, 0xab2, 0xac7, 0xaae, 0xac0, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xa9c, 0xacd,
0xab9, 0xacb, 0xaa8, 0xacd, 0xab8, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xa95, 0xac0,
0xa9f, 0xacd, 0xab8, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xab2, 0xac1, 0xab8, 0xabf,
0xaaf, 0xabe, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xaa5, 0xacb, 0xaae, 0xab8, 0xab8,
0xac7, 0xa82, 0xa9f, 0x20, 0xab5, 0xabf, 0xaa8, 0xacd, 0xab8, 0xac7, 0xaa8, 0xacd,
0xa9f, 0xab8, 0xacd, 0xab5, 0xabf, 0xaab, 0xacd, 0xa9f, 0x20, 0xa95, 0xab0, 0xa82,
0xa9f, 0xaa4, 0xac7, 0xa97, 0xac1, 0xab8, 0xabf, 0xa97, 0xab2, 0xacd, 0xaaa, 0xabe,
0xaa5, 0xac1, 0xab2, 0xac7, 0xaa4, 0xabf, 0xa9c, 0xac1, 0xa86, 0xaa8, 0xabe, 0xa9f,
0xacb, 0xab0, 0xaa8, 0xacd, 0xa9f, 0xacb, 0xa9f, 0xacb, 0xab0, 0xacd, 0xa9f, 0xacb,
0xab2, 0xabe, 0xab5, 0xac7, 0xaa8, 0xacd, 0xa95, 0xac1, 0xab5, 0xab0, 0xab5, 0xacd,
0xab9, 0xabe, 0xa87, 0xa9f, 0xab9, 0xacb, 0xab0, 0xacd, 0xab8, 0xab5, 0xabf, 0xaa8,
0xacd, 0xaa8, 0xabf, 0xaaa, 0xac7, 0xa97, 0xaaf, 0xa95, 0xac1, 0xaa4, 0xaa4, 0xa95,
0xac5, 0xab8, 0xac0, 0xaa1, 0xac7, 0xab5, 0xabf, 0xab8, 0xaa6, 0xac1, 0xaae, 0xacb,
0xaa8, 0xacd, 0xaa4, 0x20, 0xaa6, 0xac0, 0x2018, 0xa89, 0xab0, 0xacd, 0xab5, 0xabf,
0xab2, 0xaae, 0xac7, 0xa95, 0xacd, 0xab5, 0xac7, 0xab0, 0xac0, 0xaae, 0xacb, 0xab8,
0xaa8, 0xaae, 0xac5, 0xa95, 0xaae, 0xab0, 0xacd, 0xaa1, 0xacb, 0xaaa, 0xabe, 0xab2,
0xacd, 0xaae, 0xab0, 0xab0, 0xacb, 0xaa5, 0xac7, 0xab0, 0xabe, 0xab8, 0xacd, 0xaaf,
0xacb, 0xab5, 0xabe, 0xa9f, 0xacd, 0xab0, 0xacb, 0xab2, 0xab5, 0xacb, 0xab8, 0xacd,
0xa9f, 0xac9, 0xa95, 0xab2, 0xacb, 0xa82, 0xa97, 0xa87, 0xaaf, 0xab0, 0xaac, 0xabf,
0xaaf, 0xac7, 0xaa8, 0xa8f, 0xaa6, 0xac7, 0xaa8, 0xa85, 0xab2, 0xacd, 0xaae, 0xabe,
0xa9f, 0xac0, 0xa85, 0xaae, 0xacd, 0xaae, 0xabe, 0xaa8, 0xa85, 0xaa8, 0xaa6, 0xabf,
0xab0, 0xa85, 0xa95, 0xa9f, 0xabe, 0xa89, 0xa8d, 0xa95, 0xacd, 0xa9f, 0xacb, 0xaac,
0xa85, 0xab6, 0xa97, 0xabe, 0xaac, 0xa9f, 0xa85, 0xaa4, 0xacd, 0xaaf, 0xabe, 0xab0,
0xac1, 0xaac, 0xa97, 0xaa6, 0xabe, 0xaa6, 0xaac, 0xac7, 0xab9, 0xab0, 0xac0, 0xaa8,
0xaac, 0xac7, 0xa82, 0xa97, 0xa95, 0xacb, 0xa95, 0xaac, 0xabe, 0xab0, 0xaa8, 0xacc,
0xab2, 0xaac, 0xac8, 0xab0, 0xac1, 0xaa4, 0xaac, 0xabf, 0xab6, 0xa95, 0xac7, 0xa95,
0xaac, 0xacd, 0xab0, 0xac1, 0xaa8, 0xac7, 0xa87, 0xa9a, 0xabf, 0xaa4, 0xabe, 0xa9a,
0xacb, 0xa87, 0xaac, 0xabe, 0xab2, 0xacd, 0xab8, 0xaa8, 0xa95, 0xacb, 0xab2, 0xa82,
0xaac, 0xacb, 0xaa6, 0xaae, 0xabe, 0xab8, 0xacd, 0xa95, 0xab8, 0xaa2, 0xabe, 0xa95,
0xabe, 0xaa6, 0xabf, 0xab2, 0xabf, 0xaa6, 0xac1, 0xaac, 0xa88, 0xaa6, 0xac1, 0xab6,
0xabe, 0xaae, 0xacd, 0xaac, 0xac7, 0xaab, 0xabe, 0xaae, 0xabe, 0xa97, 0xac1, 0xab8,
0xacd, 0xaa4, 0xabe, 0xa97, 0xabe, 0xa9d, 0xabe, 0xab9, 0xac7, 0xaac, 0xacd, 0xab0,
0xacb, 0xaa8, 0xab9, 0xacb, 0x20, 0xa9a, 0xac0, 0x20, 0xaae, 0xac0, 0xaa8, 0x20,
0xab8, 0xabf, 0xa9f, 0xac0, 0xab9, 0xacb, 0xa82, 0xa97, 0xa95, 0xacb, 0xa82, 0xa97,
0xab9, 0xacb, 0xab5, 0xacd, 0xaa1, 0xa87, 0xab0, 0xacd, 0xa95, 0xac1, 0xaa4, 0xacd,
0xab8, 0xacd, 0xa95, 0xa9c, 0xa95, 0xabe, 0xab0, 0xacd, 0xaa4, 0xabe, 0xa9c, 0xaaf,
0xabe, 0xaaa, 0xac1, 0xab0, 0xabe, 0xa9c, 0xac7, 0xab0, 0xac1, 0xab8, 0xab2, 0xac7,
0xaae, 0xa95, 0xabe, 0xaac, 0xac1, 0xab2, 0xa95, 0xabe, 0xaae, 0xa9a, 0xa9f, 0xacd,
0xa95, 0xabe, 0xa95, 0xab0, 0xabe, 0xa9a, 0xac0, 0xa95, 0xabe, 0xaa0, 0xaae, 0xa82,
0xaa1, 0xac1, 0xa96, 0xa82, 0xaa1, 0xabf, 0xa97, 0xabe, 0xa95, 0xacb, 0xab2, 0xa95,
0xabe, 0xaa4, 0xabe, 0xa95, 0xacd, 0xab0, 0xabf, 0xab8, 0xacd, 0xaa8, 0xacb, 0xaaf,
0xabe, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0xa95, 0xac1, 0xa86, 0xab2, 0xabe, 0x20,
0xab2, 0xac1, 0xaae, 0xacd, 0xaaa, 0xac1, 0xab0, 0xa95, 0xac1, 0xa9a, 0xabf, 0xa82,
0xa97, 0xa95, 0xac1, 0xab5, 0xac8, 0xaa4, 0xaae, 0xa95, 0xabe, 0xa89, 0xaae, 0xac7,
0xa97, 0xabe, 0xaa1, 0xaa8, 0xaae, 0xabe, 0xa95, 0xab8, 0xacd, 0xab8, 0xab0, 0xaae,
0xaa8, 0xabf, 0xab2, 0xabe, 0xaae, 0xab8, 0xacd, 0xa95, 0xaa4, 0xaa8, 0xabf, 0xa95,
0xacb, 0xab8, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xacb, 0xab5, 0xacb, 0xa95, 0xac1, 0xa9d,
0xacd, 0xaa8, 0xac7, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0xaa8, 0xacb, 0xab5, 0xacb,
0xab8, 0xac0, 0xab0, 0xacd, 0xaac, 0xabf, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0xa93,
0xaae, 0xacd, 0xab8, 0xacd, 0xa95, 0xa93, 0xab0, 0xab2, 0xaab, 0xacd, 0xaa8, 0xacb,
0xaae, 0x20, 0xaaa, 0xac7, 0xaa8, 0xacd, 0xab9, 0xaaa, 0xacb, 0xaa8, 0xacd, 0xa9f,
0xabf, 0xaaf, 0xaa8, 0xac7, 0xa95, 0xaaa, 0xacd, 0xaaf, 0xacb, 0xa82, 0xa97, 0xaaf,
0xabe, 0xa82, 0xa97, 0xa95, 0xaa4, 0xabe, 0xab0, 0xa95, 0xacb, 0xab8, 0xacd, 0xa9f,
0xabe, 0xaa8, 0xac7, 0xa95, 0xabf, 0xa9d, 0xabf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1,
0xabe, 0xab0, 0xabf, 0xaaf, 0xabe, 0xaa7, 0xab8, 0xa96, 0xabe, 0xab2, 0xabf, 0xaa8,
0xab8, 0xaae, 0xab0, 0xa95, 0xaa8, 0xacd, 0xaa1, 0xab8, 0xac0, 0xa93, 0xab2, 0xab6,
0xabe, 0xa82, 0xa98, 0xabe, 0xa88, 0xab8, 0xabf, 0xa82, 0xa97, 0xabe, 0xaaa, 0xac1,
0xab0, 0xab8, 0xacd, 0xab0, 0xac7, 0xaa1, 0xacd, 0xaa8, 0xac7, 0xa95, 0xacb, 0xab2,
0xac0, 0xaae, 0xacd, 0xab8, 0xacd, 0xa95, 0xaa4, 0xabe, 0xa87, 0xaaa, 0xac7, 0xa87,
0xaa4, 0xabe, 0xab6, 0xa95, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xaac, 0xabf, 0xab2, 0xabf,
0xab8, 0xabf, 0xaa4, 0xac7, 0xab9, 0xab0, 0xabe, 0xaa8, 0xaa5, 0xac0, 0xaae, 0xacd,
0xaab, 0xac1, 0xa9f, 0xacb, 0xa95, 0xacd, 0xaaf, 0xacb, 0xaa4, 0xacb, 0xaae, 0xab8,
0xacd, 0xa95, 0xa89, 0xab2, 0xabe, 0xa82, 0xaac, 0xabe, 0xaa4, 0xab0, 0xa89, 0xab0,
0xac1, 0xaae, 0xacd, 0xa95, 0xac0, 0xa89, 0xab8, 0xacd, 0xaa4, 0x2d, 0xaa8, 0xac7,
0xab0, 0xabe, 0xab5, 0xabf, 0xaaf, 0xac7, 0xaa8, 0xa9f, 0xabf, 0xaaf, 0xac7, 0xaa8,
0xab5, 0xacd, 0xab2, 0xac7, 0xaa1, 0xabf, 0xab5, 0xacb, 0xab8, 0xacd, 0xa9f, 0xacb,
0xa95, 0xaaf, 0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xa95, 0xab0, 0xa82, 0xa97, 0xac2,
0xaa8, 0xaaf, 0xac7, 0xa95, 0xabe, 0xaa4, 0xac7, 0xab0, 0xabf, 0xaa8, 0xaac, 0xab0,
0xacd, 0xa97, 0xaaf, 0xac7, 0xab0, 0xac7, 0xab5, 0xabe, 0xaa8, 0xa8f, 0xa9d, 0xacb,
0xab0, 0xac7, 0xa9d, 0xaac, 0xab0, 0xacd, 0xaae, 0xac1, 0xaa1, 0xabe, 0xa95, 0xac5,
0xaa8, 0xac7, 0xab0, 0xac0, 0xa95, 0xac5, 0xaaa, 0x20, 0xab5, 0xab0, 0xacd, 0xaa1,
0xac7, 0xaab, 0xac7, 0xab0, 0xacb, 0xaae, 0xaa1, 0xabf, 0xab0, 0xabe, 0xab0, 0xac5,
0xa95, 0xaaf, 0xabe, 0xab5, 0xabf, 0xa95, 0xaa6, 0xa95, 0xacd, 0xab7, 0xabf, 0xaa3,
0x20, 0xa9c, 0xacd, 0xaaf, 0xacb, 0xab0, 0xacd, 0xa9c, 0xabf, 0xaaf, 0xabe, 0xab8,
0xac7, 0xa82, 0xa9f, 0x20, 0xab9, 0xac7, 0xab2, 0xac7, 0xaa8, 0xabe, 0xab8, 0xacd,
0xa9f, 0xac7, 0xaa8, 0xab2, 0xac0, 0xa8f, 0xaa1, 0xabf, 0xab2, 0xac7, 0xa87, 0xaa1,
0xaac, 0xacd, 0xab0, 0xabf, 0xab8, 0xaac, 0xac7, 0xaa8, 0xaac, 0xacd, 0xab0, 0xacb,
0xa95, 0xaa8, 0x20, 0xab9, 0xabf, 0xab2, 0xaa1, 0xabe, 0xab0, 0xacd, 0xab5, 0xabf,
0xaa8, 0xa89, 0xa95, 0xacd, 0xab2, 0xabe, 0xab9, 0xacb, 0xaac, 0xabe, 0xab0, 0xacd,
0xa9f, 0xab2, 0xabf, 0xaa8, 0xacd, 0xaa1, 0xac7, 0xaae, 0xac7, 0xaa8, 0xab2, 0xacb,
0xab0, 0xacd, 0xaa1, 0x20, 0xab9, 0xacb, 0xab5, 0xaae, 0xac7, 0xab2, 0xaac, 0xacb,
0xab0, 0xacd, 0xaa8, 0xaaa, 0xab0, 0xacd, 0xaa5, 0xab8, 0xabf, 0xaa1, 0xaa8, 0xac0,
0xa8d, 0xaae, 0xacd, 0xab8, 0xacd, 0xa9f, 0xab0, 0xaa1, 0xac5, 0xaae, 0xa8d, 0xa82,
0xaa1, 0xacb, 0xab0, 0xabe, 0xa86, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xa96, 0xaa8,
0xa8f, 0xaa5, 0xac7, 0xa82, 0xab8, 0xaac, 0xac7, 0xab2, 0xa97, 0xacd, 0xab0, 0xac7,
0xaa1, 0xaac, 0xab0, 0xacd, 0xab2, 0xabf, 0xaa8, 0xaac, 0xacd, 0xab0, 0xac7, 0xa9f,
0xabf, 0xab8, 0xab2, 0xabe, 0xab5, 0xabe, 0xaac, 0xacd, 0xab0, 0xab8, 0xac7, 0xab2,
0xacd, 0xab8, 0xaac, 0xac1, 0xa95, 0xab0, 0xac7, 0xab8, 0xacd, 0xa9f, 0xaac, 0xac1,
0xaa1, 0xabe, 0xaaa, 0xac7, 0xab8, 0xacd, 0xa9f, 0xaac, 0xabf, 0xa9d, 0xac7, 0xaa8,
0xacd, 0xa97, 0xaa8, 0xa9a, 0xabf, 0xab8, 0xac0, 0xaa8, 0xabe, 0xa89, 0xa95, 0xacb,
0xaaa, 0xaa8, 0xab9, 0xac7, 0xa97, 0xaa8, 0xaa1, 0xaac, 0xab2, 0xabf, 0xaa8, 0xa9c,
0xabf, 0xaac, 0xacd, 0xab0, 0xabe, 0xab2, 0xacd, 0xa9f, 0xab0, 0xa97, 0xab0, 0xacd,
0xaa8, 0xa9d, 0xac0, 0xab9, 0xac7, 0xab2, 0xab8, 0xabf, 0xa82, 0xa95, 0xac0, 0xa86,
0xa87, 0xab2, 0x20, 0xa91, 0xaab, 0x20, 0xaae, 0xac7, 0xaa8, 0xa87, 0xab8, 0xacd,
0xaa4, 0xabe, 0xa82, 0xaac, 0xac1, 0xab2, 0xa9c, 0xab0, 0xacd, 0xab8, 0xac0, 0xa95,
0xab2, 0xabf, 0xaa8, 0xabf, 0xaa8, 0xacd, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa1, 0xa95,
0xac0, 0xab0, 0xacb, 0xab5, 0xa95, 0xac8, 0xab5, 0xab2, 0xabf, 0xab8, 0xacd, 0xaac,
0xaa8, 0xab2, 0xac1, 0xaac, 0xacd, 0xab2, 0xabf, 0xaaf, 0xabe, 0xaa8, 0xabe, 0xab2,
0xa82, 0xaa1, 0xaa8, 0xab2, 0xa95, 0xacd, 0xa9d, 0xaae, 0xaac, 0xab0, 0xacd, 0xa97,
0xaae, 0xac7, 0xaa1, 0xacd, 0xab0, 0xabf, 0xaa1, 0xaae, 0xabe, 0xab2, 0xacd, 0xa9f,
0xabe, 0xaae, 0xac7, 0xab0, 0xabf, 0xab9, 0xac7, 0xaae, 0xaae, 0xabf, 0xa82, 0xab8,
0xacd, 0xa95, 0xaae, 0xacb, 0xaa8, 0xabe, 0xa95, 0xacb, 0xaae, 0xacb, 0xab8, 0xacd,
0xa95, 0xacb, 0xa91, 0xab8, 0xacd, 0xab2, 0xacb, 0xaaa, 0xac7, 0xab0, 0xabf, 0xab8,
0xaaa, 0xacb, 0xaa1, 0xa97, 0xacb, 0xab0, 0xabf, 0xa95, 0xabe, 0xaaa, 0xacd, 0xab0,
0xabe, 0xa97, 0xab0, 0xabf, 0xa97, 0xabe, 0xab8, 0xaae, 0xabe, 0xab0, 0xabe, 0xab8,
0xac5, 0xaa8, 0x20, 0xaae, 0xac7, 0xab0, 0xabf, 0xaa8, 0xacb, 0xab8, 0xabe, 0xab0,
0xabe, 0xa9c, 0xac7, 0xab5, 0xacb, 0xab8, 0xabe, 0xab0, 0xabe, 0xa9f, 0xacb, 0xab5,
0xab8, 0xabf, 0xaae, 0xacd, 0xaab, 0xac7, 0xab0, 0xacb, 0xaaa, 0xacb, 0xab2, 0xab8,
0xacd, 0xa95, 0xacb, 0xaaa, 0xaaf, 0xac7, 0xab8, 0xacb, 0xaab, 0xabf, 0xaaf, 0xabe,
0xab8, 0xacd, 0xa9f, 0xacb, 0xa95, 0xab9, 0xac9, 0xaae, 0xaa4, 0xab2, 0xacd, 0xab2,
0xac0, 0xaa8, 0xaa4, 0xabf, 0xab0, 0xabe, 0xaa8, 0xac7, 0xa89, 0xab2, 0xac7, 0xaa8,
0xacb, 0xab5, 0xacd, 0xab8, 0xacd, 0xa95, 0xab5, 0xac5, 0xaa6, 0xac1, 0xa9d, 0xab5,
0xac7, 0xa9f, 0xabf, 0xa95, 0xaa8, 0xab5, 0xabf, 0xaaf, 0xac7, 0xaa8, 0xabe, 0xab5,
0xabf, 0xab2, 0xacd, 0xaa8, 0xabf, 0xa85, 0xab8, 0xab5, 0xacb, 0xab2, 0xacd, 0xa97,
0xacb, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa1, 0xab5, 0xacb, 0xab0, 0xab8, 0xac9, 0xa9d,
0xac7, 0xa97, 0xab0, 0xac7, 0xaac, 0xa9d, 0xac1, 0xab0, 0xac0, 0xa95, 0xa85, 0xa82,
0xaa4, 0xabe, 0xaa8, 0xabe, 0xaa8, 0xabe, 0xab0, 0xabf, 0xab5, 0xacb, 0xa9a, 0xabe,
0xa97, 0xacb, 0xab8, 0xa95, 0xacd, 0xab0, 0xabf, 0xab8, 0xaae, 0xab8, 0xa95, 0xacb,
0xa95, 0xacb, 0xa9d, 0xa95, 0xacb, 0xaae, 0xacb, 0xab0, 0xacb, 0xa95, 0xac7, 0xab0,
0xa97, 0xacd, 0xab5, 0xac7, 0xab2, 0xabf, 0xaa8, 0xaae, 0xabe, 0xab9, 0xac7, 0xaae,
0xabe, 0xab2, 0xaa6, 0xabf, 0xab5, 0xacd, 0xab8, 0xaae, 0xacb, 0xab0, 0xabf, 0xab6,
0xabf, 0xaaf, 0xab8, 0xaae, 0xac7, 0xaaf, 0xacb, 0xa9f, 0xab0, 0xac0, 0xaaf, 0xac1,
0xaa8, 0xabf, 0xaaf, 0xaa8, 0xa8f, 0xaaa, 0xabf, 0xaaf, 0xabe, 0xa91, 0xa95, 0xab2,
0xac7, 0xaa8, 0xacd, 0xaa1, 0xaac, 0xacc, 0xa97, 0xac7, 0xa88, 0xaa8, 0xab5, 0xabf,
0xab2, 0xac7, 0xa9a, 0xaa5, 0xaae, 0xa9a, 0xac1, 0xa95, 0xa87, 0xab8, 0xacd, 0xa9f,
0xab0, 0xa8f, 0xaab, 0xac7, 0xa9f, 0xaab, 0xabe, 0xa95, 0xabe, 0xa93, 0xaab, 0xacb,
0xaab, 0xac0, 0xa9c, 0xac0, 0xaab, 0xac1, 0xaa8, 0xabe, 0xaab, 0xac1, 0xa9f, 0xac0,
0xa97, 0xabe, 0xab2, 0xabe, 0xaaa, 0xac7, 0xa97, 0xacb, 0xab8, 0xa97, 0xac7, 0xaae,
0xacd, 0xaac, 0xabf, 0xaaf, 0xab0, 0xa97, 0xacc, 0xaa1, 0xab2, 0xa95, 0xac5, 0xaa8,
0xab2, 0xa97, 0xacd, 0xab5, 0xabe, 0xaae, 0xab9, 0xacb, 0xaa8, 0xacb, 0xab2, 0xac1,
0xab2, 0xac1, 0xa8f, 0xa82, 0xaa1, 0xab0, 0xacd, 0xaac, 0xab0, 0xac0, 0xa95, 0xabf,
0xab0, 0xabf, 0xaa4, 0xabf, 0xaae, 0xabe, 0xaa4, 0xac0, 0xa95, 0xacb, 0xab8, 0xab0,
0xac7, 0xa95, 0xacd, 0xab5, 0xabe, 0xa9c, 0xabe, 0xab2, 0xac0, 0xaa8, 0xaae, 0xabe,
0xa9c, 0xac1, 0xab0, 0xacb, 0xaae, 0xabe, 0xab0, 0xacd, 0xa95, 0xac0, 0xab8, 0xabe,
0xab8, 0xaae, 0xac0, 0xaa1, 0xab5, 0xac7, 0xaa8, 0xacc, 0xab0, 0xac1, 0xaa8, 0xac0,
0xaaf, 0xac1, 0xaa8, 0xac9, 0xab0, 0xaab, 0xac9, 0xa95, 0xaa8, 0xacc, 0xaae, 0xac0,
0xa86, 0xaaa, 0xac7, 0xa97, 0xacb, 0x20, 0xaaa, 0xac7, 0xa97, 0xacb, 0xaaa, 0xab2,
0xabe, 0xa89, 0xaaa, 0xac0, 0xa9f, 0xa95, 0xac8, 0xab0, 0xacd, 0xaa8, 0xaaa, 0xacb,
0xaa8, 0xacd, 0xaaa, 0xac7, 0xaaa, 0xacb, 0xab0, 0xacd, 0xa9f, 0x20, 0xaae, 0xacb,
0xab0, 0xacd, 0xab8, 0xacd, 0xaac, 0xac0, 0xab0, 0xac7, 0xab0, 0xacb, 0xa9f, 0xacb,
0xa82, 0xa97, 0xabe, 0xab8, 0xabe, 0xa87, 0xaaa, 0xaa8, 0xaa4, 0xabe, 0xab9, 0xac0,
0xaa4, 0xac0, 0xaa4, 0xabe, 0xab0, 0xabe, 0xab5, 0xabe, 0xa9f, 0xac9, 0xa82, 0xa97,
0xabe, 0xa9f, 0xabe, 0xaaa, 0xac1, 0xab5, 0xac7, 0xa95, 0xab5, 0xabe, 0xab2, 0xabf,
0xab8, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x41, 0x72, 0x65,
0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x41, 0x72, 0x65, 0x77,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x41, 0x72,
0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x5d0, 0x5d1, 0x5d9, 0x5d2, 0x5f3, 0x5d0, 0x5df, 0x5d0, 0x5e7, 0x5e8, 0x5d4,
0x5d0, 0x5d3, 0x5d9, 0x5e1, 0x20, 0x5d0, 0x5d1, 0x5d1, 0x5d4, 0x5d0, 0x5dc, 0x5d2,
0x5f3, 0x5d9, 0x5e8, 0x5d0, 0x5e1, 0x5de, 0x5e8, 0x5d4, 0x5d1, 0x5de, 0x5d0, 0x5e7,
0x5d5, 0x5d1, 0x5e0, 0x5d2, 0x5d5, 0x5d0, 0x5d9, 0x5d1, 0x5e0, 0x5d2, 0x5f3, 0x5d5,
0x5dc, 0x5d1, 0x5d9, 0x5e1, 0x5d0, 0x5d5, 0x5d1, 0x5dc, 0x5e0, 0x5d8, 0x5d9, 0x5e8,
0x5d1, 0x5e8, 0x5d6, 0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5d1, 0x5d5, 0x5d2, 0x5f3, 0x5d5,
0x5de, 0x5d1, 0x5d5, 0x5e8, 0x5d4, 0x5e7, 0x5d4, 0x5d9, 0x5e8, 0x5e7, 0x5d6, 0x5d1,
0x5dc, 0x5e0, 0x5e7, 0x5d4, 0x5e1, 0x5d0, 0x5d5, 0x5d8, 0x5d4, 0x5e7, 0x5d5, 0x5e0,
0x5d0, 0x5e7, 0x5e8, 0x5d9, 0x5d3, 0x5e7, 0x5d0, 0x5e8, 0x5d3, 0x5d0, 0x5e8, 0x20,
0x5d0, 0x2d, 0x5e1, 0x5dc, 0x5d0, 0x5dd, 0x5d2, 0x5f3, 0x5d9, 0x5d1, 0x5d5, 0x5d8,
0x5d9, 0x5d3, 0x5d5, 0x5d0, 0x5dc, 0x5d4, 0x5d0, 0x5dc, 0x20, 0x5e2, 0x5d9, 0x5d5,
0x5df, 0x5e4, 0x5e8, 0x5d9, 0x5d8, 0x5d0, 0x5d5, 0x5df, 0x5d2, 0x5d1, 0x5d5, 0x5e8,
0x5d5, 0x5e0, 0x5d4, 0x5d4, 0x5e8, 0x5d0, 0x5e8, 0x5d4, 0x5d9, 0x5d5, 0x5d4, 0x5e0,
0x5e1, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x5d2, 0x5f3, 0x5d5, 0x5d1, 0x5d4, 0x5e7, 0x5de,
0x5e4, 0x5d0, 0x5dc, 0x5d4, 0x5d7, 0x5e8, 0x5d8, 0x5d5, 0x5dd, 0x5e7, 0x5d9, 0x5d2,
0x5dc, 0x5d9, 0x5e7, 0x5d9, 0x5e0, 0x5e9, 0x5e1, 0x5d4, 0x5dc, 0x5d0, 0x5d2, 0x5d5,
0x5e1, 0x5dc, 0x5d9, 0x5d1, 0x5e8, 0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5dc, 0x5d5, 0x5de,
0x5d4, 0x5dc, 0x5d5, 0x5d0, 0x5e0, 0x5d3, 0x5d4, 0x5dc, 0x5d5, 0x5d1, 0x5d5, 0x5de,
0x5d1, 0x5d0, 0x5e9, 0x5d9, 0x5dc, 0x5d5, 0x5e1, 0x5e7, 0x5d4, 0x5de, 0x5dc, 0x5d1,
0x5d5, 0x5de, 0x5d0, 0x5e4, 0x5d5, 0x5d8, 0x5d5, 0x5de, 0x5e1, 0x5e8, 0x5d5, 0x5de,
0x5d1, 0x5d1, 0x5e0, 0x5d4, 0x5de, 0x5d5, 0x5d2, 0x5d3, 0x5d9, 0x5e9, 0x5d5, 0x5de,
0x5d5, 0x5e0, 0x5e8, 0x5d5, 0x5d1, 0x5d9, 0x5d4, 0x5e0, 0x5d9, 0x5d9, 0x5e8, 0x5d5,
0x5d1, 0x5d9, 0x5e0, 0x5d2, 0x5f3, 0x5de, 0x5e0, 0x5d4, 0x5e0, 0x5d9, 0x5d0, 0x5de,
0x5d9, 0x5d9, 0x5e0, 0x5d5, 0x5d0, 0x5e7, 0x5e6, 0x5f3, 0x5d5, 0x5d8, 0x5d5, 0x5d5,
0x5d0, 0x5d2, 0x5d0, 0x5d3, 0x5d5, 0x5d2, 0x5d5, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x5d5,
0x20, 0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e1, 0x5d0, 0x5d5, 0x20, 0x5d8, 0x5d5, 0x5de,
0x5d4, 0x5d8, 0x5e8, 0x5d9, 0x5e4, 0x5d5, 0x5dc, 0x5d9, 0x5ea, 0x5d5, 0x5e0, 0x5d9,
0x5e1, 0x5d5, 0x5d9, 0x5e0, 0x5d3, 0x5d4, 0x5d5, 0x5e7, 0x5d0, 0x5d3, 0x5d0, 0x5e7,
0x5d0, 0x5e0, 0x5e7, 0x5d5, 0x5e8, 0x5d2, 0x5f3, 0x5d0, 0x5e0, 0x5d2, 0x5d5, 0x5d5,
0x5d9, 0x5dc, 0x5d4, 0x5d0, 0x5e0, 0x5d8, 0x5d9, 0x5d2, 0x5d5, 0x5d0, 0x5d4, 0x5d0,
0x5e8, 0x5d0, 0x5d2, 0x5d5, 0x5d0, 0x5d9, 0x5e0, 0x5d4, 0x5d1, 0x5d5, 0x5d0, 0x5e0,
0x5d5, 0x5e1, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x5e8, 0x5e1, 0x5e7, 0x5d8, 0x5de, 0x5e8,
0x5e7, 0x5d4, 0x5e7, 0x5d5, 0x5e8, 0x5d3, 0x5d5, 0x5d1, 0x5d4, 0x5d7, 0x5d5, 0x5d7,
0x5d5, 0x5d9, 0x5dc, 0x5d4, 0x20, 0x5e8, 0x5d9, 0x5d5, 0x5d7, 0x5d4, 0x5de, 0x5e0,
0x5d3, 0x5d5, 0x5d6, 0x5d4, 0x5e8, 0x5d9, 0x5d5, 0x20, 0x5d2, 0x5d0, 0x5d9, 0x5d9,
0x5d2, 0x5d5, 0x5e1, 0x5e1, 0x5dc, 0x5d8, 0x5d4, 0x5e1, 0x5df, 0x20, 0x5d7, 0x5d5,
0x5d0, 0x5df, 0x5e1, 0x5df, 0x20, 0x5dc, 0x5d5, 0x5d0, 0x5d9, 0x5e1, 0x5d8, 0x5d5,
0x5e7, 0x5d5, 0x5de, 0x5df, 0x5d0, 0x5d5, 0x5e9, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x5d4,
0x5d0, 0x5e8, 0x5d5, 0x5d1, 0x5d4, 0x5d0, 0x5e1, 0x5d5, 0x5e0, 0x5e1, 0x5d9, 0x5d5,
0x5df, 0x5d0, 0x5d8, 0x5d9, 0x5e7, 0x5d5, 0x5e7, 0x5df, 0x5d1, 0x5d0, 0x5d4, 0x5d9,
0x5d4, 0x5d1, 0x5d0, 0x5d4, 0x5d9, 0x5d4, 0x20, 0x5d1, 0x5e0, 0x5d3, 0x5e8, 0x5e1,
0x5d1, 0x5e8, 0x5d1, 0x5d3, 0x5d5, 0x5e1, 0x5d1, 0x5dc, 0x5dd, 0x5d1, 0x5dc, 0x5d9,
0x5d6, 0x5d1, 0x5dc, 0x5d0, 0x5df, 0x2d, 0x5e1, 0x5d1, 0x5dc, 0x5d5, 0x5df, 0x5d1,
0x5d5, 0x5d0, 0x5d4, 0x20, 0x5d5, 0x5d9, 0x5e1, 0x5d8, 0x5d4, 0x5d1, 0x5d5, 0x5d2,
0x5d5, 0x5d8, 0x5d4, 0x5d1, 0x5d5, 0x5d9, 0x5e1, 0x5d9, 0x5e7, 0x5d9, 0x5d9, 0x5de,
0x5d1, 0x5e8, 0x5d9, 0x5d3, 0x5d2, 0x5f3, 0x20, 0x5d1, 0x5d9, 0x5d9, 0x5e7, 0x5de,
0x5e4, 0x5d5, 0x20, 0x5d2, 0x5e8, 0x5e0, 0x5d3, 0x5d4, 0x5e7, 0x5e0, 0x5e7, 0x5d5,
0x5df, 0x5e7, 0x5e8, 0x5d0, 0x5e7, 0x5e1, 0x5e7, 0x5d0, 0x5d9, 0x5d9, 0x5df, 0x5e7,
0x5d9, 0x5d9, 0x5de, 0x5df, 0x5e9, 0x5d9, 0x5e7, 0x5d2, 0x5d5, 0x5e6, 0x5f3, 0x5d9,
0x5d5, 0x5d5, 0x5d0, 0x5d5, 0x5d5, 0x5d4, 0x5e1, 0x5d9, 0x5d5, 0x5d3, 0x5d3, 0x20,
0x5d7, 0x5d5, 0x5d0, 0x5e8, 0x5e1, 0x5e7, 0x5d5, 0x5e1, 0x5d8, 0x5d4, 0x20, 0x5e8,
0x5d9, 0x5e7, 0x5d4, 0x5e7, 0x5e8, 0x5e1, 0x5d8, 0x5d5, 0x5df, 0x5e7, 0x5d5, 0x5d9,
0x5d0, 0x5d1, 0x5d4, 0x5e7, 0x5d5, 0x5e8, 0x5d0, 0x5e1, 0x5d0, 0x5d5, 0x5d3, 0x5e0,
0x5de, 0x5e8, 0x5e7, 0x5e1, 0x5d4, 0x5d5, 0x5d5, 0x5df, 0x5d3, 0x5d5, 0x5e1, 0x5d5,
0x5df, 0x5d3, 0x5d5, 0x5e1, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5e8, 0x5d9, 0x5e7, 0x5d3,
0x5e0, 0x5d5, 0x5d5, 0x5e8, 0x5d3, 0x5d8, 0x5e8, 0x5d5, 0x5d9, 0x5d8, 0x5d3, 0x5d5,
0x5de, 0x5d9, 0x5e0, 0x5d9, 0x5e7, 0x5d4, 0x5d0, 0x5d3, 0x5de, 0x5d5, 0x5e0, 0x5d8,
0x5d5, 0x5df, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e0, 0x5e4, 0x5d9, 0x5d0, 0x5dc, 0x20,
0x5e1, 0x5dc, 0x5d1, 0x5d3, 0x5d5, 0x5e8, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x20, 0x5e0,
0x5dc, 0x5e1, 0x5d5, 0x5df, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x5d0, 0x5dc, 0x5d6, 0x5d4,
0x5d2, 0x5dc, 0x5d9, 0x5d9, 0x5e1, 0x20, 0x5d1, 0x5d9, 0x5d9, 0x5d2, 0x5d5, 0x5e1,
0x20, 0x5d1, 0x5d9, 0x5d9, 0x5d2, 0x5e8, 0x5e0, 0x5d3, 0x20, 0x5d8, 0x5d5, 0x5e8,
0x5e7, 0x5d2, 0x5d5, 0x5d0, 0x5d3, 0x5dc, 0x5d5, 0x5e4, 0x5d2, 0x5d5, 0x5d0, 0x5d8,
0x5de, 0x5dc, 0x5d4, 0x5d2, 0x5d5, 0x5d0, 0x5d9, 0x5d0, 0x5e7, 0x5d9, 0x5dc, 0x5d2,
0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5d4, 0x5dc, 0x5d9, 0x5e4, 0x5e7, 0x5e1, 0x5d4, 0x5d5,
0x5d5, 0x5d0, 0x5e0, 0x5d4, 0x5d4, 0x5e8, 0x5de, 0x5d5, 0x5e1, 0x5d9, 0x5d5, 0x5d0,
0x5d9, 0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5e4, 0x5d5, 0x5dc, 0x5d9, 0x5e1, 0x5e0,
0x5d5, 0x5e7, 0x5e1, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0,
0x5d4, 0x5de, 0x5e8, 0x5e0, 0x5d2, 0x5d5, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3,
0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5e4, 0x5d9, 0x5d8, 0x5e8, 0x5e1, 0x5d1, 0x5d5, 0x5e8,
0x5d2, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5d8,
0x5dc, 0x20, 0x5e1, 0x5d9, 0x5d8, 0x5d9, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3,
0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5d5, 0x5d9, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x5d9, 0x2c,
0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5d5, 0x5d9, 0x5e0,
0x5e1, 0x5e0, 0x5e1, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d9, 0x5d0, 0x5e0,
0x5d4, 0x5d5, 0x5d9, 0x5e0, 0x5de, 0x5d0, 0x5e7, 0x2c, 0x20, 0x5d0, 0x5d9, 0x5e0,
0x5d3, 0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x5d0, 0x5d9, 0x5e0, 0x5d5, 0x5d5, 0x5d9, 0x5e7,
0x5d0, 0x5d9, 0x5e7, 0x5dc, 0x5d5, 0x5d0, 0x5d9, 0x5d8, 0x5d2, 0x5f3, 0x5de, 0x5d9,
0x5d9, 0x5e7, 0x5d4, 0x5d2, 0x5f3, 0x5d5, 0x5e0, 0x5d5, 0x5dc, 0x5d5, 0x5d0, 0x5d9,
0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5de, 0x5d5, 0x5e0, 0x5d8, 0x5d9, 0x5e6, 0x5f3, 0x5dc,
0x5d5, 0x2c, 0x20, 0x5e7, 0x5e0, 0x5d8, 0x5d0, 0x5e7, 0x5d9, 0x5e7, 0x5e8, 0x5dc,
0x5e0, 0x5d3, 0x5d9, 0x5d9, 0x5e7, 0x5dc, 0x5d4, 0x20, 0x5e4, 0x5d0, 0x5e1, 0x5dc,
0x5d9, 0x5de, 0x5d4, 0x5dc, 0x5d5, 0x5e1, 0x20, 0x5d0, 0x5e0, 0x5d2, 0x5f3, 0x5dc,
0x5e1, 0x5dc, 0x5d5, 0x5d0, 0x5d5, 0x5d5, 0x5e8, 0x20, 0x5e4, 0x5e8, 0x5d9, 0x5e0,
0x5e1, 0x5e1, 0x20, 0x5e7, 0x5d5, 0x5d5, 0x5e8, 0x5d8, 0x5e8, 0x5de, 0x5e1, 0x5d9,
0x5d9, 0x5d0, 0x5d5, 0x5de, 0x5e0, 0x5d2, 0x5d5, 0x5d0, 0x5d4, 0x5de, 0x5e0, 0x5d0,
0x5d5, 0x5e1, 0x5de, 0x5e8, 0x5d9, 0x5d2, 0x5d5, 0x5de, 0x5e8, 0x5d8, 0x5d9, 0x5e0,
0x5d9, 0x5e7, 0x5de, 0x5d8, 0x5de, 0x5d5, 0x5e8, 0x5d5, 0x5e1, 0x5de, 0x5d6, 0x5d8,
0x5dc, 0x5df, 0x5de, 0x5e0, 0x5d5, 0x5de, 0x5d9, 0x5e0, 0x5d9, 0x5de, 0x5e8, 0x5d9,
0x5d3, 0x5d4, 0x5de, 0x5d8, 0x5dc, 0x5e7, 0x5d8, 0x5dc, 0x5d4, 0x5de, 0x5e7, 0x5e1,
0x5d9, 0x5e7, 0x5d5, 0x20, 0x5e1, 0x5d9, 0x5d8, 0x5d9, 0x5de, 0x5d9, 0x5e7, 0x5dc,
0x5d5, 0x5df, 0x5de, 0x5d5, 0x5e0, 0x5e7, 0x5d8, 0x5d5, 0x5df, 0x5de, 0x5d5, 0x5e0,
0x5d8, 0x5e8, 0x5d9, 0x5d9, 0x5de, 0x5d5, 0x5e0, 0x5d8, 0x5d5, 0x5d5, 0x5d9, 0x5d3,
0x5d0, 0x5d5, 0x5de, 0x5d5, 0x5e0, 0x5e1, 0x5e8, 0x5d0, 0x5d8, 0x5e0, 0x5e1, 0x5d0,
0x5d5, 0x5e0, 0x5d9, 0x5d5, 0x20, 0x5d9, 0x5d5, 0x5e8, 0x5e7, 0x5e0, 0x5d5, 0x5dd,
0x5e0, 0x5d5, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d4, 0x5d1, 0x5d9, 0x5d5, 0x5dc, 0x5d4,
0x2c, 0x20, 0x5d3, 0x5e7, 0x5d5, 0x5d8, 0x5d4, 0x20, 0x5d4, 0x5e6, 0x5e4, 0x5d5,
0x5e0, 0x5d9, 0x5ea, 0x5e1, 0x5e0, 0x5d8, 0x5e8, 0x2c, 0x20, 0x5d3, 0x5e7, 0x5d5,
0x5d8, 0x5d4, 0x20, 0x5d4, 0x5e6, 0x5e4, 0x5d5, 0x5e0, 0x5d9, 0x5ea, 0x5e0, 0x5d9,
0x5d5, 0x20, 0x5e1, 0x5d9, 0x5d9, 0x5dc, 0x5dd, 0x2c, 0x20, 0x5d3, 0x5e7, 0x5d5,
0x5d8, 0x5d4, 0x20, 0x5d4, 0x5e6, 0x5e4, 0x5d5, 0x5e0, 0x5d9, 0x5ea, 0x5e0, 0x5d5,
0x5d0, 0x5d5, 0x5e7, 0x5d0, 0x5d5, 0x5d2, 0x5f3, 0x5d9, 0x5e0, 0x5d0, 0x5d2, 0x5d4,
0x5e4, 0x5e0, 0x5de, 0x5d4, 0x5e4, 0x5e8, 0x5de, 0x5e8, 0x5d9, 0x5d1, 0x5d5, 0x5e4,
0x5d9, 0x5e0, 0x5d9, 0x5e7, 0x5e1, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x20, 0x5d0, 0x5d5,
0x20, 0x5e4, 0x5e8, 0x5d0, 0x5e0, 0x5e1, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x20, 0x5d0,
0x5d5, 0x5e3, 0x20, 0x5e1, 0x5e4, 0x5d9, 0x5d9, 0x5df, 0x5e4, 0x5d5, 0x5e8, 0x5d8,
0x5d5, 0x20, 0x5d5, 0x5dc, 0x5d9, 0x5d5, 0x5e4, 0x5d5, 0x5d0, 0x5e8, 0x5d8, 0x5d5,
0x20, 0x5e8, 0x5d9, 0x5e7, 0x5d5, 0x5e4, 0x5d5, 0x5e0, 0x5d8, 0x5d4, 0x20, 0x5d0,
0x5e8, 0x5e0, 0x5e1, 0x5e8, 0x5e0, 0x5e7, 0x5d9, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5e0,
0x5dc, 0x5d8, 0x5e8, 0x5e1, 0x5d9, 0x5e4, 0x5d4, 0x5e8, 0x5d2, 0x5f3, 0x5d9, 0x5e0,
0x5d4, 0x5e8, 0x5d6, 0x5d5, 0x5dc, 0x5d5, 0x5d8, 0x5e8, 0x5d9, 0x5d5, 0x20, 0x5d1,
0x5e8, 0x5e0, 0x5e7, 0x5d5, 0x5e1, 0x5e0, 0x5d8, 0x5e8, 0x5dd, 0x5e1, 0x5e0, 0x5d8,
0x5d9, 0x5d0, 0x5d2, 0x5d5, 0x5e1, 0x5e0, 0x5d8, 0x5d5, 0x20, 0x5d3, 0x5d5, 0x5de,
0x5d9, 0x5e0, 0x5d2, 0x5d5, 0x5e1, 0x5d0, 0x5d5, 0x20, 0x5e4, 0x5d0, 0x5d5, 0x5dc,
0x5d5, 0x5e1, 0x5e7, 0x5d5, 0x5e8, 0x5e1, 0x5d1, 0x5d9, 0x5e1, 0x5d5, 0x5e0, 0x5d3,
0x5e1, 0x5d9, 0x5d8, 0x5e7, 0x5d4, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5d1, 0x5e8, 0x5ea,
0x5dc, 0x5de, 0x5d9, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5d2, 0x5f3, 0x5d5, 0x5e0, 0x5e1,
0x5e1, 0x5e0, 0x5d8, 0x20, 0x5e7, 0x5d9, 0x5d8, 0x5e1, 0x5e1, 0x5e0, 0x5d8, 0x20,
0x5dc, 0x5d5, 0x5e1, 0x5d9, 0x5d4, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5ea, 0x5d5, 0x5de,
0x5d0, 0x5e1, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5d5, 0x5d9, 0x5e0, 0x5e1, 0x5e0, 0x5d8,
0x5e1, 0x5d5, 0x5d5, 0x5d9, 0x5e4, 0x5d8, 0x20, 0x5e7, 0x5e8, 0x5e0, 0x5d8, 0x5d8,
0x5d2, 0x5d5, 0x5e1, 0x5d9, 0x5d2, 0x5dc, 0x5e4, 0x5d4, 0x5ea, 0x5d5, 0x5dc, 0x5d4,
0x5d8, 0x5d9, 0x5d7, 0x5d5, 0x5d0, 0x5e0, 0x5d4, 0x5d8, 0x5d5, 0x5e8, 0x5d5, 0x5e0,
0x5d8, 0x5d5, 0x5d8, 0x5d5, 0x5e8, 0x5d8, 0x5d5, 0x5dc, 0x5d4, 0x5d5, 0x5e0, 0x5e7,
0x5d5, 0x5d1, 0x5e8, 0x5d5, 0x5d5, 0x5d9, 0x5d9, 0x5d8, 0x5d4, 0x5d5, 0x5e8, 0x5e1,
0x5d5, 0x5d5, 0x5d9, 0x5e0, 0x5d9, 0x5e4, 0x5d2, 0x5d9, 0x5e7, 0x5d5, 0x5d8, 0x5d0,
0x5d8, 0x5e7, 0x5d9, 0x5d9, 0x5e1, 0x5d9, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5d9, 0x5e1,
0x5d3, 0x5d5, 0x5de, 0x5d5, 0x5df, 0x20, 0x5d3, 0x5f3, 0x5d0, 0x5d5, 0x5e8, 0x5d5,
0x5d5, 0x5d9, 0x5dc, 0x5de, 0x5e7, 0x5d5, 0x5d5, 0x5d0, 0x5e8, 0x5d9, 0x5de, 0x5d5,
0x5e1, 0x5d5, 0x5df, 0x5de, 0x5e7, 0x2d, 0x5de, 0x5e8, 0x5d3, 0x5d5, 0x5e4, 0x5d0,
0x5dc, 0x5de, 0x5e8, 0x5e8, 0x5d5, 0x5ea, 0x5f3, 0x5e8, 0x5d4, 0x5e1, 0x5d9, 0x5d9,
0x5d5, 0x5d5, 0x5d4, 0x5d8, 0x5e8, 0x5d5, 0x5dc, 0x5d5, 0x5d5, 0x5e1, 0x5d8, 0x5d5,
0x5e7, 0x5dc, 0x5d5, 0x5e0, 0x5d2, 0x5d9, 0x5d9, 0x5e8, 0x5d1, 0x5d9, 0x5df, 0x5e2,
0x5d3, 0x5df, 0x5d0, 0x5dc, 0x5de, 0x5d0, 0x5d8, 0x5d9, 0x5e2, 0x5de, 0x5d0, 0x5df,
0x5d0, 0x5e0, 0x5d3, 0x5d9, 0x5e8, 0x5d0, 0x5e7, 0x5d8, 0x5d0, 0x5d5, 0x5d0, 0x5e7,
0x5d8, 0x5d5, 0x5d1, 0x5d4, 0x5d0, 0x5e9, 0x5d2, 0x5d1, 0x5d8, 0x5d0, 0x5d8, 0x5d9,
0x5e8, 0x5d0, 0x5d5, 0x5d1, 0x5d2, 0x5d3, 0x5d3, 0x5d1, 0x5d7, 0x5e8, 0x5d9, 0x5d9,
0x5df, 0x5d1, 0x5d0, 0x5e7, 0x5d5, 0x5d1, 0x5e0, 0x5d2, 0x5e7, 0x5d5, 0x5e7, 0x5d1,
0x5e8, 0x5e0, 0x5d0, 0x5d5, 0x5dc, 0x5d1, 0x5d9, 0x5d9, 0x5e8, 0x5d5, 0x5ea, 0x5d1,
0x5d9, 0x5e9, 0x5e7, 0x5e7, 0x5d1, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d9, 0x5e6, 0x5f3,
0x5d9, 0x5d8, 0x5d4, 0x5e6, 0x5f3, 0x5d5, 0x5d9, 0x5d1, 0x5dc, 0x5e1, 0x5df, 0x5e7,
0x5d5, 0x5dc, 0x5d5, 0x5de, 0x5d1, 0x5d5, 0x5d3, 0x5de, 0x5e9, 0x5e7, 0x5d3, 0x5d0,
0x5e7, 0x5d4, 0x5d3, 0x5d9, 0x5dc, 0x5d9, 0x5d3, 0x5d5, 0x5d1, 0x5d0, 0x5d9, 0x5d3,
0x5d5, 0x5e9, 0x5e0, 0x5d1, 0x5d4, 0x5e4, 0x5de, 0x5d2, 0x5d5, 0x5e1, 0x5d8, 0x5d4,
0x5e2, 0x5d6, 0x5d4, 0x5d7, 0x5d1, 0x5e8, 0x5d5, 0x5df, 0x5d4, 0x5d5, 0x20, 0x5e6,
0x5f3, 0x5d9, 0x20, 0x5de, 0x5d9, 0x5df, 0x20, 0x5e1, 0x5d9, 0x5d8, 0x5d9, 0x5d4,
0x5d5, 0x5e0, 0x5d2, 0x20, 0x5e7, 0x5d5, 0x5e0, 0x5d2, 0x5d7, 0x5d5, 0x5d1, 0x5d3,
0x5d0, 0x5d9, 0x5e8, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x5d2, 0x5f3, 0x5e7, 0x5e8,
0x5d8, 0x5d4, 0x5d2, 0x5f3, 0x5d0, 0x5d9, 0x5d0, 0x5e4, 0x5d5, 0x5e8, 0x5d4, 0x5d9,
0x5e8, 0x5d5, 0x5e9, 0x5dc, 0x5d9, 0x5dd, 0x5e7, 0x5d0, 0x5d1, 0x5d5, 0x5dc, 0x5e7,
0x5de, 0x5e6, 0x5f3, 0x5d8, 0x5e7, 0x5d4, 0x5e7, 0x5e8, 0x5d0, 0x5e6, 0x5f3, 0x5d9,
0x5e7, 0x5d8, 0x5de, 0x5e0, 0x5d3, 0x5d5, 0x5d7, 0x5e0, 0x5d3, 0x5d9, 0x5d2, 0x5d4,
0x5e7, 0x5d5, 0x5dc, 0x5e7, 0x5d8, 0x5d4, 0x5e7, 0x5e8, 0x5e1, 0x5e0, 0x5d5, 0x5d9,
0x5d0, 0x5e8, 0x5e1, 0x5e7, 0x5e7, 0x5d5, 0x5d0, 0x5dc, 0x5d4, 0x20, 0x5dc, 0x5d5,
0x5de, 0x5e4, 0x5d5, 0x5e8, 0x5e7, 0x5d5, 0x5e6, 0x5f3, 0x5d9, 0x5e0, 0x5d2, 0x5db,
0x5d5, 0x5d5, 0x5d9, 0x5ea, 0x5de, 0x5e7, 0x5d0, 0x5d5, 0x5de, 0x5d2, 0x5d3, 0x5df,
0x5de, 0x5d0, 0x5e7, 0x5d0, 0x5e1, 0x5d0, 0x5e8, 0x5de, 0x5e0, 0x5d9, 0x5dc, 0x5d4,
0x5de, 0x5d5, 0x5e1, 0x5e7, 0x5d8, 0x5e0, 0x5d9, 0x5e7, 0x5d5, 0x5e1, 0x5d9, 0x5d4,
0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e7, 0x5d5, 0x5d6, 0x5e0, 0x5d8, 0x5e1, 0x5e7, 0x5e0,
0x5d5, 0x5d1, 0x5d5, 0x5e1, 0x5d9, 0x5d1, 0x5d9, 0x5e8, 0x5e1, 0x5e7, 0x5d0, 0x5d5,
0x5de, 0x5e1, 0x5e7, 0x5d0, 0x5d5, 0x5e8, 0x5dc, 0x5e4, 0x5e0, 0x5d5, 0x5dd, 0x20,
0x5e4, 0x5df, 0x5e4, 0x5d5, 0x5e0, 0x5d8, 0x5d9, 0x5d0, 0x5e0, 0x5e7, 0x5e4, 0x5d9,
0x5d5, 0x5e0, 0x5d2, 0x5d9, 0x5d0, 0x5e0, 0x5d2, 0x5e7, 0x5d8, 0x5d0, 0x5e8, 0x5e7,
0x5d5, 0x5e1, 0x5d8, 0x5e0, 0x5d0, 0x5d9, 0x5e7, 0x5d9, 0x5d6, 0x5d9, 0x5dc, 0x5d5,
0x5e8, 0x5d3, 0x5d4, 0x5e8, 0x5d9, 0x5d0, 0x5d3, 0x5e1, 0x5d7, 0x5dc, 0x5d9, 0x5df,
0x5e1, 0x5de, 0x5e8, 0x5e7, 0x5e0, 0x5d3, 0x5e1, 0x5d9, 0x5d0, 0x5d5, 0x5dc, 0x5e9,
0x5e0, 0x5d7, 0x5d0, 0x5d9, 0x5e1, 0x5d9, 0x5e0, 0x5d2, 0x5e4, 0x5d5, 0x5e8, 0x5e1,
0x5e8, 0x5d3, 0x5e0, 0x5d9, 0x5d9, 0x5e7, 0x5d5, 0x5dc, 0x5d9, 0x5de, 0x5e1, 0x5e7,
0x5d8, 0x5d0, 0x5d9, 0x5e4, 0x5d9, 0x5d9, 0x5d8, 0x5e9, 0x5e7, 0x5e0, 0x5d8, 0x5d8,
0x5d1, 0x5d9, 0x5dc, 0x5d9, 0x5e1, 0x5d9, 0x5d8, 0x5d4, 0x5e8, 0x5df, 0x5d8, 0x5d4,
0x5d9, 0x5de, 0x5e4, 0x5d4, 0x5d5, 0x5d8, 0x5d5, 0x5e7, 0x5d9, 0x5d5, 0x5d8, 0x5d5,
0x5de, 0x5e1, 0x5e7, 0x5d0, 0x5d5, 0x5dc, 0x5d0, 0x5df, 0x20, 0x5d1, 0x5d0, 0x5d8,
0x5d5, 0x5e8, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5de, 0x5e7, 0x5d9, 0x5d0, 0x5d5, 0x5e1,
0x5d8, 0x2d, 0x5e0, 0x5e8, 0x5d4, 0x5d5, 0x5d9, 0x5d9, 0x5e0, 0x5d8, 0x5d9, 0x5d0,
0x5df, 0x5d5, 0x5dc, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5e1, 0x5d8, 0x5d5, 0x5e7, 0x5d9,
0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x5e8, 0x5e0, 0x5d2, 0x5d5, 0x5df, 0x5d9, 0x5e7,
0x5d8, 0x5e8, 0x5d9, 0x5e0, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x5d9, 0x5e8, 0x5d5, 0x5d5,
0x5d0, 0x5df, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5d0, 0x5d6, 0x5d5,
0x5e8, 0x5d9, 0x5d9, 0x5dd, 0x5d1, 0x5e8, 0x5de, 0x5d5, 0x5d3, 0x5d4, 0x5d4, 0x5d0,
0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5e7, 0x5e0, 0x5e8, 0x5d9, 0x5d9, 0x5dd, 0x5db,
0x5e3, 0x20, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x5e4, 0x5d0, 0x5e8, 0x5d5, 0x5de, 0x5d3,
0x5d9, 0x5d9, 0x5e8, 0x5d4, 0x5e8, 0x5d9, 0x5d9, 0x5e7, 0x5d9, 0x5d0, 0x5d5, 0x5d5,
0x5d9, 0x5e7, 0x5d3, 0x5e8, 0x5d5, 0x5dd, 0x20, 0x5d2, 0x5f3, 0x5d5, 0x5e8, 0x5d2,
0x5f3, 0x5d9, 0x5d4, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5d4, 0x5dc, 0x5e0, 0x5d4, 0x5e1,
0x5d8, 0x5e0, 0x5dc, 0x5d9, 0x5d0, 0x5d3, 0x5dc, 0x5d9, 0x5d9, 0x5d3, 0x5d1, 0x5e8,
0x5d9, 0x5e1, 0x5d1, 0x5d9, 0x5d9, 0x5df, 0x5d1, 0x5e8, 0x5d5, 0x5e7, 0x5df, 0x20,
0x5d4, 0x5d9, 0x5dc, 0x5d3, 0x5e8, 0x5d5, 0x5d5, 0x5d9, 0x5df, 0x5d9, 0x5d5, 0x5e7,
0x5dc, 0x5d4, 0x5d4, 0x5d5, 0x5d1, 0x5e8, 0x5d8, 0x5dc, 0x5d9, 0x5e0, 0x5d3, 0x5de,
0x5df, 0x5d0, 0x5d9, 0x20, 0x5d4, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x20, 0x5d4, 0x5d0,
0x5d5, 0x5de, 0x5dc, 0x5d1, 0x5d5, 0x5e8, 0x5df, 0x5e4, 0x5e8, 0x5ea, 0x5f3, 0x5e1,
0x5d9, 0x5d3, 0x5e0, 0x5d9, 0x5d0, 0x5de, 0x5e1, 0x5d8, 0x5e8, 0x5d3, 0x5dd, 0x5d0,
0x5e0, 0x5d3, 0x5d5, 0x5e8, 0x5d4, 0x5d0, 0x5e1, 0x5d8, 0x5e8, 0x5d7, 0x5df, 0x5d0,
0x5ea, 0x5d5, 0x5e0, 0x5d4, 0x5d1, 0x5dc, 0x5d2, 0x5e8, 0x5d3, 0x5d1, 0x5e8, 0x5dc,
0x5d9, 0x5df, 0x5d1, 0x5e8, 0x5d8, 0x5d9, 0x5e1, 0x5dc, 0x5d1, 0x5d4, 0x5d1, 0x5e8,
0x5d9, 0x5e1, 0x5dc, 0x5d1, 0x5d5, 0x5e7, 0x5e8, 0x5e9, 0x5d8, 0x5d1, 0x5d5, 0x5d3,
0x5e4, 0x5e9, 0x5d8, 0x5d1, 0x5d9, 0x5d6, 0x5d9, 0x5e0, 0x5d2, 0x5df, 0x5e7, 0x5d9,
0x5e9, 0x5d9, 0x5e0, 0x5d1, 0x5e7, 0x5d5, 0x5e4, 0x5e0, 0x5d4, 0x5d2, 0x5df, 0x5d3,
0x5d1, 0x5dc, 0x5d9, 0x5df, 0x5d2, 0x5d9, 0x5d1, 0x5e8, 0x5dc, 0x5d8, 0x5e8, 0x5d2,
0x5e8, 0x5e0, 0x5d6, 0x5d9, 0x5d4, 0x5dc, 0x5e1, 0x5d9, 0x5e0, 0x5e7, 0x5d9, 0x5d4,
0x5d0, 0x5d9, 0x20, 0x5de, 0x5d0, 0x5df, 0x5d0, 0x5d9, 0x5e1, 0x5d8, 0x5e0, 0x5d1,
0x5d5, 0x5dc, 0x5d2, 0x5f3, 0x5e8, 0x5d6, 0x5d9, 0x5e7, 0x5dc, 0x5d9, 0x5e0, 0x5d9,
0x5e0, 0x5d2, 0x5e8, 0x5d3, 0x5e7, 0x5d9, 0x5e8, 0x5d5, 0x5d1, 0x5e7, 0x5d9, 0x5d9,
0x5d1, 0x5dc, 0x5d9, 0x5e1, 0x5d1, 0x5d5, 0x5df, 0x5dc, 0x5d5, 0x5d1, 0x5dc, 0x5d9,
0x5d0, 0x5e0, 0x5d4, 0x5dc, 0x5d5, 0x5e0, 0x5d3, 0x5d5, 0x5df, 0x5dc, 0x5d5, 0x5e7,
0x5e1, 0x5de, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x5de, 0x5d3, 0x5e8, 0x5d9, 0x5d3, 0x5de,
0x5dc, 0x5d8, 0x5d4, 0x5de, 0x5e8, 0x5d9, 0x5d9, 0x5d4, 0x5d0, 0x5de, 0x5df, 0x5de,
0x5d9, 0x5e0, 0x5e1, 0x5e7, 0x5de, 0x5d5, 0x5e0, 0x5e7, 0x5d5, 0x5de, 0x5d5, 0x5e1,
0x5e7, 0x5d1, 0x5d4, 0x5d0, 0x5d5, 0x5e1, 0x5dc, 0x5d5, 0x5e4, 0x5e8, 0x5d9, 0x5d6,
0x5e4, 0x5d5, 0x5d3, 0x5d2, 0x5d5, 0x5e8, 0x5d9, 0x5e6, 0x5d4, 0x5e4, 0x5e8, 0x5d0,
0x5d2, 0x5e8, 0x5d9, 0x5d2, 0x5d4, 0x5e8, 0x5d5, 0x5de, 0x5d0, 0x5e1, 0x5df, 0x20,
0x5de, 0x5e8, 0x5d9, 0x5e0, 0x5d5, 0x5e1, 0x5e8, 0x5d9, 0x5d9, 0x5d1, 0x5d5, 0x5e1,
0x5e8, 0x5d0, 0x5d8, 0x5d5, 0x5d1, 0x5e1, 0x5d9, 0x5de, 0x5e4, 0x5e8, 0x5d5, 0x5e4,
0x5d5, 0x5dc, 0x5e1, 0x5e7, 0x5d5, 0x5e4, 0x5d9, 0x5d4, 0x5e1, 0x5d5, 0x5e4, 0x5d9,
0x5d4, 0x5e9, 0x5d8, 0x5d5, 0x5e7, 0x5d4, 0x5d5, 0x5dc, 0x5dd, 0x5d8, 0x5d0, 0x5dc,
0x5d9, 0x5df, 0x5d8, 0x5d9, 0x5e8, 0x5e0, 0x5d4, 0x5d0, 0x5d5, 0x5dc, 0x5d9, 0x5d0,
0x5e0, 0x5d5, 0x5d1, 0x5e1, 0x5e7, 0x5d5, 0x5d0, 0x5d3, 0x5d5, 0x5e5, 0x5d4, 0x5d5,
0x5d5, 0x5ea, 0x5d9, 0x5e7, 0x5df, 0x5d5, 0x5d9, 0x5e0, 0x5d4, 0x5d5, 0x5d9, 0x5dc,
0x5e0, 0x5d4, 0x5d5, 0x5d5, 0x5dc, 0x5d2, 0x5d5, 0x5d2, 0x5e8, 0x5d3, 0x5d5, 0x5e8,
0x5e9, 0x5d4, 0x5d6, 0x5d0, 0x5d2, 0x5e8, 0x5d1, 0x5e6, 0x5d9, 0x5e8, 0x5d9, 0x5da,
0x5d0, 0x5e0, 0x5d8, 0x5e0, 0x5e0, 0x5e8, 0x5d9, 0x5d1, 0x5d5, 0x5e6, 0x5f3, 0x5d0,
0x5d2, 0x5d5, 0x5e1, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5db, 0x5e8, 0x5d9, 0x5e1, 0x5d8,
0x5de, 0x5e1, 0x5e7, 0x5d5, 0x5e7, 0x5d5, 0x5e1, 0x5e7, 0x5d5, 0x5de, 0x5d5, 0x5e8,
0x5d5, 0x5e7, 0x5e8, 0x5d2, 0x5d5, 0x5d5, 0x5dc, 0x5df, 0x5de, 0x5d4, 0x5d0, 0x5d4,
0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5de, 0x5dc, 0x5d3, 0x5d9, 0x5d1, 0x5d9,
0x5d9, 0x5dd, 0x5de, 0x5d0, 0x5d5, 0x5e8, 0x5d9, 0x5e6, 0x5d9, 0x5d5, 0x5e1, 0x5de,
0x5d0, 0x5d9, 0x5d5, 0x5d8, 0x5e8, 0x5d0, 0x5d5, 0x5e0, 0x5d9, 0x5d5, 0x5df, 0x5d0,
0x5e4, 0x5d9, 0x5d4, 0x5d0, 0x5d5, 0x5e7, 0x5dc, 0x5e0, 0x5d3, 0x5d1, 0x5d5, 0x5d2,
0x5e0, 0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5e6, 0x5f3, 0x5d0, 0x5d8, 0x5d4, 0x5d0, 0x5dd,
0x5e6, 0x5f3, 0x5d5, 0x5e7, 0x5d0, 0x5d9, 0x20, 0x5d4, 0x5e4, 0x5e1, 0x5d7, 0x5d0,
0x5d0, 0x5e4, 0x5d8, 0x5d4, 0x5e4, 0x5e7, 0x5d0, 0x5d5, 0x5e4, 0x5d5, 0x5e4, 0x5d9,
0x5d2, 0x5f3, 0x5d9, 0x5e4, 0x5d5, 0x5e0, 0x5e4, 0x5d5, 0x5d8, 0x5d9, 0x5d2, 0x5dc,
0x5e4, 0x5d0, 0x5d2, 0x5d5, 0x5e1, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5d2, 0x5de, 0x5d1,
0x5d9, 0x5d9, 0x5d4, 0x5d2, 0x5d5, 0x5d5, 0x5d3, 0x5dc, 0x5e7, 0x5e0, 0x5d0, 0x5dc,
0x5d2, 0x5d5, 0x5d0, 0x5dd, 0x5d4, 0x5d5, 0x5e0, 0x5d5, 0x5dc, 0x5d5, 0x5dc, 0x5d5,
0x5d0, 0x5e0, 0x5d3, 0x5e8, 0x5d1, 0x5d5, 0x5e8, 0x5d9, 0x5e7, 0x5d9, 0x5e8, 0x5d9,
0x5d8, 0x5d9, 0x5de, 0x5d0, 0x5d8, 0x5d9, 0x5e7, 0x5d5, 0x5e1, 0x5e8, 0x5d4, 0x5e7,
0x5d5, 0x5d5, 0x5d0, 0x5d2, 0x5f3, 0x5dc, 0x5d9, 0x5d9, 0x5df, 0x5de, 0x5d0, 0x5d2,
0x5f3, 0x5d5, 0x5e8, 0x5d5, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5de, 0x5e8, 0x5e7, 0x5d9,
0x5d6, 0x5de, 0x5d9, 0x5d3, 0x5d5, 0x5d5, 0x5d9, 0x5d9, 0x5e0, 0x5d0, 0x5d5, 0x5e8,
0x5d5, 0x5e0, 0x5d9, 0x5d5, 0x5d0, 0x5d4, 0x5e0, 0x5d5, 0x5e8, 0x5e4, 0x5d5, 0x5e7,
0x5e0, 0x5d5, 0x5de, 0x5d0, 0x5d4, 0x5e4, 0x5d0, 0x5d2, 0x5d5, 0x20, 0x5e4, 0x5d0,
0x5d2, 0x5d5, 0x5e4, 0x5dc, 0x5d0, 0x5d5, 0x5e4, 0x5d9, 0x5d8, 0x5e7, 0x5e8, 0x5df,
0x5e4, 0x5d5, 0x5e0, 0x5e4, 0x5d9, 0x5d9, 0x5e4, 0x5d5, 0x5e8, 0x5d8, 0x20, 0x5de,
0x5d5, 0x5e8, 0x5e1, 0x5d1, 0x5d9, 0x5e8, 0x5d0, 0x5e8, 0x5d5, 0x5d8, 0x5d5, 0x5e0,
0x5d2, 0x5d4, 0x5e1, 0x5d0, 0x5d9, 0x5e4, 0x5d0, 0x5df, 0x5d8, 0x5d4, 0x5d9, 0x5d8,
0x5d9, 0x5d8, 0x5d0, 0x5e8, 0x5d0, 0x5d5, 0x5d5, 0x5d4, 0x5d8, 0x5d5, 0x5e0, 0x5d2,
0x5d8, 0x5d0, 0x5e4, 0x5d5, 0x5d5, 0x5d9, 0x5d9, 0x5e7, 0x5d5, 0x5d5, 0x5dc, 0x5d9,
0x5e1, 0x90f, 0x915, 0x94d, 0x930, 0x93e, 0x905, 0x926, 0x940, 0x938, 0x20, 0x905,
0x92c, 0x93e, 0x92c, 0x93e, 0x905, 0x932, 0x94d, 0x91c, 0x940, 0x92f, 0x930, 0x94d,
0x938, 0x905, 0x938, 0x94d, 0x92e, 0x93e, 0x930, 0x93e, 0x92c, 0x93e, 0x92e, 0x93e,
0x915, 0x94b, 0x92c, 0x93e, 0x902, 0x917, 0x941, 0x907, 0x92c, 0x948, 0x902, 0x91c,
0x941, 0x932, 0x92c, 0x93f, 0x938, 0x93e, 0x90a, 0x92c, 0x94d, 0x932, 0x93e, 0x902,
0x91f, 0x93e, 0x92f, 0x930, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c, 0x93e, 0x935,
0x93f, 0x932, 0x947, 0x92c, 0x941, 0x91c, 0x941, 0x902, 0x92c, 0x942, 0x930, 0x93e,
0x915, 0x93e, 0x92f, 0x930, 0x94b, 0x938, 0x947, 0x909, 0x91f, 0x93e, 0x915, 0x94b,
0x928, 0x93e, 0x915, 0x94d, 0x930, 0x940, 0x921, 0x915, 0x93e, 0x930, 0x926, 0x93e,
0x930, 0x20, 0x905, 0x938, 0x20, 0x938, 0x932, 0x93e, 0x92e, 0x91c, 0x93f, 0x92c,
0x942, 0x924, 0x940, 0x921, 0x942, 0x906, 0x932, 0x93e, 0x905, 0x932, 0x20, 0x906,
0x907, 0x92f, 0x942, 0x928, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x91f, 0x93e, 0x909,
0x928, 0x917, 0x93e, 0x92c, 0x94b, 0x930, 0x94b, 0x928, 0x939, 0x930, 0x93e, 0x930,
0x947, 0x91c, 0x94b, 0x939, 0x93e, 0x902, 0x938, 0x92c, 0x930, 0x94d, 0x917, 0x915,
0x902, 0x92a, 0x93e, 0x932, 0x93e, 0x915, 0x93f, 0x917, 0x93e, 0x932, 0x940, 0x915,
0x93f, 0x902, 0x936, 0x93e, 0x938, 0x93e, 0x932, 0x93e, 0x917, 0x94b, 0x938, 0x932,
0x93f, 0x92c, 0x94d, 0x930, 0x947, 0x935, 0x93f, 0x932, 0x947, 0x932, 0x94b, 0x92e,
0x932, 0x941, 0x906, 0x902, 0x921, 0x93e, 0x932, 0x941, 0x92c, 0x941, 0x92e, 0x92c,
0x93e, 0x936, 0x940, 0x92e, 0x932, 0x93e, 0x92c, 0x94b, 0x92e, 0x93e, 0x92a, 0x941,
0x91f, 0x94b, 0x90f, 0x92e, 0x92c, 0x93e, 0x92c, 0x947, 0x928, 0x92e, 0x94b, 0x917,
0x93e, 0x926, 0x93f, 0x936, 0x941, 0x92e, 0x94b, 0x928, 0x930, 0x94b, 0x935, 0x93f,
0x92f, 0x93e, 0x928, 0x948, 0x930, 0x94b, 0x92c, 0x940, 0x928, 0x947, 0x926, 0x94d,
0x91c, 0x93e, 0x92e, 0x940, 0x928, 0x93e, 0x928, 0x93f, 0x92f, 0x93e, 0x92e, 0x940,
0x928, 0x94c, 0x906, 0x915, 0x936, 0x949, 0x91f, 0x914, 0x917, 0x93e, 0x921, 0x94b,
0x917, 0x942, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x2d, 0x928, 0x94b, 0x935,
0x94b, 0x938, 0x93e, 0x913, 0x20, 0x91f, 0x94b, 0x92e, 0x924, 0x94d, 0x930, 0x93f,
0x92a, 0x94b, 0x932, 0x940, 0x935, 0x93f, 0x902, 0x921, 0x939, 0x94b, 0x915, 0x905,
0x921, 0x915, 0x90f, 0x902, 0x917, 0x94d, 0x935, 0x93f, 0x932, 0x93e, 0x90f, 0x902,
0x91f, 0x940, 0x917, 0x941, 0x906, 0x906, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947,
0x928, 0x93e, 0x92c, 0x94d, 0x92f, 0x942, 0x928, 0x938, 0x20, 0x906, 0x92f, 0x930,
0x938, 0x915, 0x94b, 0x930, 0x94d, 0x921, 0x94b, 0x92c, 0x93e, 0x91c, 0x941, 0x91c,
0x94b, 0x90f, 0x932, 0x93e, 0x20, 0x930, 0x93f, 0x913, 0x91c, 0x93e, 0x92e, 0x947,
0x902, 0x921, 0x94b, 0x91c, 0x93c, 0x93e, 0x930, 0x93f, 0x92f, 0x94b, 0x20, 0x917,
0x93e, 0x932, 0x947, 0x917, 0x94b, 0x938, 0x938, 0x948, 0x928, 0x20, 0x939, 0x94d,
0x935, 0x93e, 0x928, 0x938, 0x948, 0x928, 0x20, 0x932, 0x942, 0x908, 0x91f, 0x94b,
0x915, 0x942, 0x92e, 0x928, 0x909, 0x936, 0x941, 0x906, 0x907, 0x92f, 0x93e, 0x905,
0x930, 0x942, 0x92c, 0x93e, 0x90f, 0x938, 0x928, 0x936, 0x93f, 0x92f, 0x949, 0x928,
0x905, 0x91f, 0x93f, 0x915, 0x94b, 0x915, 0x93e, 0x928, 0x92c, 0x939, 0x93f, 0x92f,
0x93e, 0x92c, 0x947, 0x939, 0x93f, 0x92f, 0x93e, 0x20, 0x92c, 0x93e, 0x902, 0x921,
0x947, 0x930, 0x93e, 0x938, 0x92c, 0x93e, 0x930, 0x92c, 0x93e, 0x921, 0x94b, 0x938,
0x92c, 0x947, 0x932, 0x940, 0x91c, 0x93c, 0x92c, 0x94d, 0x932, 0x93e, 0x902, 0x915,
0x2d, 0x938, 0x947, 0x92c, 0x932, 0x94b, 0x928, 0x92c, 0x94b, 0x906, 0x20, 0x935,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x92c, 0x94b, 0x917, 0x94b, 0x91f, 0x93e, 0x92c,
0x949, 0x907, 0x938, 0x940, 0x915, 0x948, 0x92e, 0x94d, 0x92c, 0x94d, 0x930, 0x93f,
0x91c, 0x20, 0x916, 0x93e, 0x921, 0x93c, 0x940, 0x915, 0x948, 0x902, 0x92a, 0x94b,
0x20, 0x917, 0x94d, 0x930, 0x93e, 0x902, 0x921, 0x947, 0x915, 0x948, 0x928, 0x915,
0x941, 0x928, 0x915, 0x93e, 0x930, 0x93e, 0x915, 0x938, 0x936, 0x93f, 0x915, 0x93e,
0x917, 0x94b, 0x91a, 0x93f, 0x939, 0x941, 0x906, 0x939, 0x941, 0x906, 0x938, 0x94d,
0x92f, 0x941, 0x926, 0x93e, 0x926, 0x20, 0x939, 0x94d, 0x935, 0x93e, 0x930, 0x947,
0x91c, 0x93c, 0x915, 0x94b, 0x938, 0x94d, 0x91f, 0x93e, 0x20, 0x930, 0x93f, 0x915,
0x93e, 0x915, 0x94d, 0x92f, 0x942, 0x906, 0x92c, 0x93e, 0x915, 0x941, 0x930, 0x93e,
0x915, 0x93e, 0x913, 0x921, 0x947, 0x928, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x936,
0x949, 0x928, 0x921, 0x949, 0x938, 0x928, 0x921, 0x949, 0x938, 0x928, 0x20, 0x915,
0x94d, 0x930, 0x940, 0x915, 0x921, 0x947, 0x928, 0x935, 0x930, 0x921, 0x947, 0x91f,
0x94d, 0x930, 0x949, 0x92f, 0x91f, 0x921, 0x94b, 0x92e, 0x93f, 0x928, 0x93f, 0x915,
0x93e, 0x90f, 0x921, 0x92e, 0x902, 0x91f, 0x928, 0x905, 0x932, 0x20, 0x938, 0x932,
0x94d, 0x935, 0x93e, 0x921, 0x94b, 0x930, 0x92b, 0x93c, 0x94b, 0x930, 0x94d, 0x91f,
0x20, 0x928, 0x947, 0x932, 0x94d, 0x938, 0x928, 0x92b, 0x93c, 0x94b, 0x930, 0x94d,
0x91f, 0x93e, 0x932, 0x947, 0x91c, 0x93c, 0x93e, 0x917, 0x94d, 0x932, 0x947, 0x938,
0x20, 0x916, 0x93e, 0x921, 0x93c, 0x940, 0x917, 0x942, 0x938, 0x20, 0x916, 0x93e,
0x921, 0x93c, 0x940, 0x917, 0x94d, 0x930, 0x93e, 0x902, 0x921, 0x20, 0x91f, 0x930,
0x94d, 0x915, 0x917, 0x94d, 0x935, 0x93e, 0x921, 0x947, 0x932, 0x94b, 0x92a, 0x917,
0x94d, 0x935, 0x93e, 0x92f, 0x93e, 0x915, 0x940, 0x932, 0x917, 0x92f, 0x93e, 0x928,
0x93e, 0x939, 0x947, 0x932, 0x93f, 0x92b, 0x93c, 0x948, 0x915, 0x94d, 0x938, 0x939,
0x935, 0x93e, 0x928, 0x93e, 0x939, 0x930, 0x94d, 0x92e, 0x94b, 0x938, 0x93f, 0x932,
0x94d, 0x932, 0x94b, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92a,
0x94b, 0x932, 0x93f, 0x938, 0x928, 0x94c, 0x915, 0x94d, 0x938, 0x2c, 0x20, 0x907,
0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92e, 0x93e, 0x930, 0x947, 0x902,
0x917, 0x94b, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e,
0x92a, 0x940, 0x91f, 0x930, 0x94d, 0x938, 0x92c, 0x930, 0x94d, 0x917, 0x2c, 0x20,
0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x91f, 0x947, 0x932, 0x20,
0x938, 0x93f, 0x91f, 0x940, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e,
0x928, 0x93e, 0x935, 0x947, 0x935, 0x947, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f,
0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x902, 0x938, 0x947, 0x928, 0x947, 0x938,
0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f,
0x928, 0x93e, 0x92e, 0x947, 0x915, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f,
0x93e, 0x928, 0x93e, 0x907, 0x915, 0x93e, 0x932, 0x941, 0x908, 0x91f, 0x91c, 0x92e,
0x948, 0x915, 0x93e, 0x91c, 0x94d, 0x92f, 0x942, 0x928, 0x93e, 0x909, 0x932, 0x941,
0x907, 0x938, 0x935, 0x93f, 0x932, 0x947, 0x92e, 0x94b, 0x902, 0x91f, 0x940, 0x938,
0x947, 0x932, 0x94b, 0x2c, 0x20, 0x915, 0x947, 0x902, 0x91f, 0x915, 0x940, 0x915,
0x94d, 0x930, 0x93e, 0x932, 0x948, 0x902, 0x921, 0x93f, 0x91c, 0x93f, 0x915, 0x932,
0x93e, 0x20, 0x92a, 0x93e, 0x91c, 0x93c, 0x932, 0x949, 0x938, 0x20, 0x90f, 0x902,
0x91c, 0x93f, 0x932, 0x94d, 0x938, 0x932, 0x94b, 0x905, 0x930, 0x20, 0x92a, 0x94d,
0x930, 0x93f, 0x902, 0x938, 0x947, 0x938, 0x20, 0x915, 0x94d, 0x935, 0x93e, 0x930,
0x94d, 0x91f, 0x930, 0x92e, 0x947, 0x938, 0x940, 0x913, 0x92e, 0x93e, 0x928, 0x93e,
0x917, 0x941, 0x906, 0x92e, 0x928, 0x94c, 0x938, 0x92e, 0x948, 0x930, 0x940, 0x917,
0x94b, 0x91f, 0x92e, 0x93e, 0x930, 0x94d, 0x91f, 0x93f, 0x928, 0x93f, 0x915, 0x92e,
0x93e, 0x91f, 0x93e, 0x92e, 0x94b, 0x930, 0x94b, 0x938, 0x92e, 0x93e, 0x91c, 0x93c,
0x93e, 0x91f, 0x932, 0x93e, 0x928, 0x92e, 0x947, 0x928, 0x94b, 0x92e, 0x93f, 0x928,
0x940, 0x92e, 0x947, 0x91f, 0x94d, 0x932, 0x947, 0x915, 0x93e, 0x91f, 0x94d, 0x932,
0x93e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x20, 0x938, 0x93f,
0x91f, 0x940, 0x92e, 0x93f, 0x915, 0x947, 0x932, 0x949, 0x928, 0x92e, 0x94b, 0x902,
0x915, 0x91f, 0x928, 0x92e, 0x94b, 0x902, 0x91f, 0x947, 0x930, 0x947, 0x930, 0x940,
0x92e, 0x94b, 0x902, 0x91f, 0x947, 0x935, 0x940, 0x921, 0x93f, 0x92f, 0x94b, 0x92e,
0x94b, 0x902, 0x91f, 0x938, 0x947, 0x930, 0x93e, 0x924, 0x928, 0x93e, 0x938, 0x93e,
0x909, 0x928, 0x94d, 0x92f, 0x942, 0x92f, 0x949, 0x930, 0x94d, 0x915, 0x928, 0x94b,
0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x92c, 0x94d, 0x92f, 0x942, 0x932, 0x93e,
0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x921, 0x915, 0x94b,
0x91f, 0x93e, 0x92e, 0x927, 0x94d, 0x92f, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x926, 0x93e, 0x915, 0x94b, 0x924, 0x93e, 0x928, 0x94d, 0x92f,
0x942, 0x20, 0x938, 0x93e, 0x932, 0x947, 0x92e, 0x2c, 0x20, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x940, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x928, 0x941, 0x915,
0x913, 0x916, 0x93e, 0x91c, 0x940, 0x928, 0x93e, 0x917, 0x93e, 0x92a, 0x928, 0x93e,
0x92e, 0x93e, 0x92a, 0x93e, 0x930, 0x93e, 0x92e, 0x93e, 0x930, 0x93f, 0x92c, 0x94b,
0x92b, 0x93c, 0x940, 0x928, 0x93f, 0x915, 0x94d, 0x938, 0x92a, 0x94b, 0x930, 0x94d,
0x91f, 0x2d, 0x911, 0x2d, 0x92a, 0x94d, 0x930, 0x93f, 0x902, 0x938, 0x92a, 0x94b,
0x930, 0x94d, 0x91f, 0x20, 0x911, 0x92b, 0x93c, 0x20, 0x938, 0x94d, 0x92a, 0x947,
0x928, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x20, 0x935, 0x947, 0x932, 0x94d,
0x939, 0x94b, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x20, 0x930, 0x93f, 0x915,
0x94b, 0x92a, 0x941, 0x902, 0x91f, 0x93e, 0x20, 0x90f, 0x930, 0x93f, 0x928, 0x93e,
0x938, 0x930, 0x947, 0x902, 0x915, 0x93f, 0x928, 0x20, 0x907, 0x928, 0x932, 0x947,
0x91f, 0x930, 0x947, 0x938, 0x93e, 0x907, 0x92b, 0x93c, 0x930, 0x93f, 0x938, 0x94b,
0x932, 0x94d, 0x92f, 0x942, 0x91f, 0x930, 0x93f, 0x92f, 0x94b, 0x20, 0x92c, 0x94d,
0x930, 0x93e, 0x902, 0x915, 0x94b, 0x938, 0x948, 0x902, 0x91f, 0x93e, 0x930, 0x947,
0x92e, 0x938, 0x948, 0x902, 0x91f, 0x93f, 0x92f, 0x93e, 0x917, 0x94b, 0x938, 0x947,
0x902, 0x91f, 0x94b, 0x20, 0x921, 0x94b, 0x92e, 0x93f, 0x902, 0x917, 0x94b, 0x938,
0x93e, 0x913, 0x20, 0x92a, 0x93e, 0x909, 0x932, 0x94b, 0x907, 0x91f, 0x94b, 0x915,
0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x930, 0x92e, 0x93f, 0x91f, 0x938, 0x93f, 0x91f,
0x94d, 0x915, 0x93e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92c, 0x93e, 0x930, 0x94d,
0x925, 0x947, 0x932, 0x947, 0x92e, 0x940, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x91c,
0x94b, 0x902, 0x938, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x915, 0x93f, 0x91f, 0x94d,
0x938, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x932, 0x942, 0x938, 0x93f, 0x92f, 0x93e,
0x938, 0x947, 0x902, 0x91f, 0x20, 0x925, 0x949, 0x92e, 0x938, 0x938, 0x947, 0x902,
0x91f, 0x20, 0x935, 0x93f, 0x902, 0x938, 0x947, 0x902, 0x91f, 0x938, 0x94d, 0x935,
0x93f, 0x92b, 0x93c, 0x94d, 0x91f, 0x20, 0x915, 0x930, 0x902, 0x91f, 0x91f, 0x947,
0x917, 0x941, 0x938, 0x93f, 0x917, 0x932, 0x94d, 0x92a, 0x93e, 0x925, 0x94d, 0x92f,
0x942, 0x932, 0x947, 0x924, 0x93f, 0x91c, 0x941, 0x906, 0x928, 0x93e, 0x91f, 0x94b,
0x930, 0x902, 0x91f, 0x94b, 0x91f, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x932, 0x93e,
0x935, 0x948, 0x902, 0x915, 0x942, 0x935, 0x930, 0x935, 0x94d, 0x939, 0x93e, 0x907,
0x91f, 0x939, 0x94b, 0x930, 0x94d, 0x938, 0x935, 0x93f, 0x928, 0x940, 0x92a, 0x947,
0x917, 0x92f, 0x93e, 0x915, 0x942, 0x91f, 0x93e, 0x91f, 0x921, 0x947, 0x935, 0x93f,
0x938, 0x921, 0x94d, 0x92f, 0x942, 0x92e, 0x94b, 0x902, 0x91f, 0x20, 0x921, 0x940,
0x20, 0x905, 0x930, 0x94d, 0x935, 0x93f, 0x932, 0x947, 0x92e, 0x915, 0x94d, 0x935,
0x93e, 0x930, 0x940, 0x92e, 0x949, 0x938, 0x928, 0x92e, 0x948, 0x915, 0x92e, 0x941,
0x930, 0x94d, 0x921, 0x94b, 0x92a, 0x949, 0x92e, 0x930, 0x930, 0x94b, 0x925, 0x947,
0x930, 0x93e, 0x938, 0x94d, 0x92f, 0x94b, 0x935, 0x93e, 0x91f, 0x94d, 0x930, 0x94b,
0x932, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x932, 0x949, 0x928, 0x94d,
0x917, 0x908, 0x92f, 0x930, 0x92c, 0x93e, 0x92f, 0x947, 0x928, 0x906, 0x926, 0x947,
0x928, 0x905, 0x932, 0x94d, 0x92e, 0x93e, 0x91f, 0x940, 0x905, 0x92e, 0x94d, 0x92e,
0x93e, 0x928, 0x905, 0x928, 0x93e, 0x921, 0x93f, 0x930, 0x905, 0x915, 0x94d, 0x924,
0x94b, 0x92c, 0x90f, 0x924, 0x930, 0x93e, 0x909, 0x92c, 0x917, 0x926, 0x93e, 0x926,
0x92c, 0x939, 0x930, 0x940, 0x928, 0x92c, 0x948, 0x902, 0x915, 0x949, 0x915, 0x92c,
0x930, 0x94d, 0x928, 0x94b, 0x932, 0x92c, 0x947, 0x930, 0x941, 0x924, 0x92c, 0x94d,
0x930, 0x942, 0x928, 0x947, 0x908, 0x924, 0x94d, 0x936, 0x93f, 0x924, 0x93e, 0x91a,
0x94b, 0x907, 0x92c, 0x93e, 0x932, 0x938, 0x928, 0x915, 0x94b, 0x932, 0x902, 0x92c,
0x94b, 0x926, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x938, 0x922, 0x93e, 0x915, 0x93e,
0x921, 0x93f, 0x932, 0x93f, 0x926, 0x941, 0x92c, 0x908, 0x926, 0x941, 0x936, 0x93e,
0x902, 0x92c, 0x947, 0x92b, 0x93c, 0x93e, 0x92e, 0x93e, 0x917, 0x941, 0x938, 0x94d,
0x924, 0x93e, 0x917, 0x93e, 0x91c, 0x93c, 0x93e, 0x939, 0x947, 0x92c, 0x94d, 0x930,
0x94b, 0x928, 0x939, 0x94b, 0x20, 0x91a, 0x940, 0x20, 0x92e, 0x93f, 0x928, 0x94d,
0x939, 0x20, 0x938, 0x93f, 0x91f, 0x940, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915,
0x93e, 0x901, 0x917, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x907, 0x930, 0x94d, 0x915,
0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x91c, 0x915, 0x93e, 0x930, 0x94d, 0x924,
0x93e, 0x92f, 0x930, 0x942, 0x936, 0x932, 0x92e, 0x915, 0x93e, 0x92c, 0x941, 0x932,
0x915, 0x92e, 0x91a, 0x924, 0x94d, 0x915, 0x93e, 0x915, 0x93e, 0x920, 0x92e, 0x93e,
0x902, 0x921, 0x942, 0x916, 0x93e, 0x902, 0x921, 0x93f, 0x917, 0x93e, 0x915, 0x94b,
0x932, 0x915, 0x93e, 0x924, 0x93e, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928,
0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x915, 0x941, 0x906, 0x932,
0x93e, 0x932, 0x902, 0x92a, 0x941, 0x930, 0x92e, 0x915, 0x93e, 0x90a, 0x92e, 0x915,
0x938, 0x94d, 0x938, 0x930, 0x92e, 0x938, 0x94d, 0x915, 0x91f, 0x928, 0x93f, 0x915,
0x94b, 0x938, 0x93f, 0x92f, 0x93e, 0x928, 0x94b, 0x935, 0x94b, 0x915, 0x941, 0x91c,
0x93c, 0x94d, 0x928, 0x947, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x928, 0x94b, 0x935,
0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x913, 0x92e,
0x94d, 0x938, 0x94d, 0x915, 0x913, 0x930, 0x932, 0x928, 0x949, 0x92e, 0x20, 0x92a,
0x947, 0x928, 0x94d, 0x939, 0x92a, 0x94b, 0x902, 0x91f, 0x940, 0x92f, 0x93e, 0x902,
0x915, 0x92a, 0x94d, 0x92f, 0x94b, 0x902, 0x917, 0x92f, 0x93e, 0x902, 0x917, 0x915,
0x924, 0x930, 0x915, 0x94b, 0x938, 0x94d, 0x91f, 0x93e, 0x928, 0x947, 0x915, 0x947,
0x91c, 0x93c, 0x947, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x930, 0x93f, 0x92f,
0x93e, 0x926, 0x938, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x938, 0x92e, 0x930, 0x915,
0x902, 0x926, 0x938, 0x93f, 0x913, 0x932, 0x936, 0x902, 0x918, 0x93e, 0x908, 0x938,
0x94d, 0x930, 0x947, 0x926, 0x94d, 0x928, 0x93f, 0x915, 0x94b, 0x932, 0x93f, 0x92e,
0x938, 0x94d, 0x915, 0x924, 0x93e, 0x908, 0x92a, 0x947, 0x908, 0x924, 0x93e, 0x936,
0x915, 0x902, 0x926, 0x91f, 0x92c, 0x93f, 0x932, 0x93f, 0x938, 0x940, 0x925, 0x93f,
0x902, 0x92a, 0x942, 0x91f, 0x94b, 0x915, 0x94d, 0x92f, 0x94b, 0x924, 0x94b, 0x92e,
0x94d, 0x938, 0x94d, 0x915, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x93e, 0x924, 0x930,
0x909, 0x930, 0x942, 0x92e, 0x94d, 0x915, 0x940, 0x92f, 0x942, 0x938, 0x94d, 0x91f,
0x2013, 0x928, 0x947, 0x930, 0x93e, 0x935, 0x93f, 0x90f, 0x928, 0x924, 0x93f, 0x92f,
0x93e, 0x928, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d,
0x924, 0x94b, 0x915, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928,
0x92c, 0x930, 0x94d, 0x917, 0x92f, 0x947, 0x930, 0x947, 0x935, 0x93e, 0x928, 0x905,
0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938, 0x92c, 0x930, 0x92e, 0x942, 0x921, 0x93e,
0x915, 0x948, 0x928, 0x947, 0x930, 0x940, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930,
0x94d, 0x921, 0x92b, 0x93c, 0x948, 0x930, 0x94b, 0x92e, 0x921, 0x947, 0x930, 0x93e,
0x930, 0x947, 0x915, 0x94d, 0x92f, 0x93e, 0x935, 0x93f, 0x915, 0x926, 0x915, 0x94d,
0x937, 0x93f, 0x923, 0x20, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e,
0x938, 0x947, 0x902, 0x91f, 0x20, 0x939, 0x947, 0x932, 0x947, 0x928, 0x93e, 0x938,
0x94d, 0x91f, 0x948, 0x928, 0x932, 0x940, 0x90f, 0x921, 0x93f, 0x932, 0x947, 0x921,
0x92c, 0x94d, 0x930, 0x93f, 0x938, 0x94d, 0x92c, 0x928, 0x92c, 0x94d, 0x930, 0x94b,
0x915, 0x928, 0x20, 0x939, 0x93f, 0x932, 0x921, 0x93e, 0x930, 0x94d, 0x935, 0x93f,
0x928, 0x939, 0x94b, 0x92c, 0x93e, 0x930, 0x94d, 0x91f, 0x932, 0x93f, 0x902, 0x921,
0x947, 0x92e, 0x93e, 0x928, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b,
0x935, 0x947, 0x92e, 0x947, 0x932, 0x92c, 0x94b, 0x930, 0x94d, 0x928, 0x92a, 0x930,
0x94d, 0x925, 0x90f, 0x92e, 0x94d, 0x938, 0x94d, 0x91f, 0x930, 0x94d, 0x921, 0x92e,
0x905, 0x902, 0x921, 0x94b, 0x930, 0x93e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x93e, 0x916, 0x93e, 0x928, 0x90f, 0x925, 0x947, 0x902, 0x938, 0x92c, 0x930, 0x94d,
0x932, 0x93f, 0x928, 0x92c, 0x94d, 0x930, 0x93e, 0x924, 0x93f, 0x938, 0x94d, 0x932,
0x93e, 0x935, 0x93e, 0x92c, 0x94d, 0x930, 0x942, 0x938, 0x947, 0x932, 0x94d, 0x938,
0x92c, 0x941, 0x916, 0x93c, 0x93e, 0x930, 0x947, 0x938, 0x94d, 0x91f, 0x92c, 0x941,
0x921, 0x93e, 0x92a, 0x947, 0x938, 0x94d, 0x91f, 0x92c, 0x94d, 0x92f, 0x942, 0x938,
0x93f, 0x928, 0x91c, 0x947, 0x928, 0x91a, 0x93f, 0x938, 0x940, 0x928, 0x93e, 0x909,
0x915, 0x94b, 0x92a, 0x947, 0x928, 0x939, 0x947, 0x917, 0x928, 0x921, 0x92c, 0x932,
0x93f, 0x928, 0x91c, 0x93f, 0x92c, 0x94d, 0x930, 0x93e, 0x932, 0x94d, 0x91f, 0x930,
0x917, 0x930, 0x94d, 0x928, 0x938, 0x940, 0x939, 0x947, 0x932, 0x938, 0x93f, 0x902,
0x915, 0x940, 0x906, 0x907, 0x932, 0x20, 0x911, 0x92b, 0x93c, 0x20, 0x92e, 0x948,
0x928, 0x907, 0x938, 0x94d, 0x924, 0x93e, 0x902, 0x92c, 0x941, 0x932, 0x91c, 0x930,
0x94d, 0x938, 0x940, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x93f, 0x928, 0x917, 0x94d,
0x930, 0x93e, 0x921, 0x915, 0x93f, 0x930, 0x94b, 0x935, 0x915, 0x940, 0x935, 0x932,
0x94d, 0x92f, 0x942, 0x92c, 0x947, 0x932, 0x91c, 0x93e, 0x928, 0x93e, 0x932, 0x902,
0x926, 0x928, 0x932, 0x915, 0x94d, 0x91c, 0x93c, 0x92e, 0x92c, 0x930, 0x94d, 0x917,
0x92e, 0x948, 0x921, 0x94d, 0x930, 0x93f, 0x921, 0x92e, 0x93e, 0x930, 0x93f, 0x92f,
0x93e, 0x939, 0x948, 0x92e, 0x92e, 0x93f, 0x902, 0x938, 0x94d, 0x915, 0x92e, 0x94b,
0x928, 0x93e, 0x915, 0x94b, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x913, 0x938,
0x94d, 0x932, 0x94b, 0x92a, 0x94b, 0x921, 0x94d, 0x917, 0x94b, 0x930, 0x93f, 0x915,
0x93e, 0x92a, 0x94d, 0x930, 0x93e, 0x917, 0x930, 0x940, 0x917, 0x93e, 0x938, 0x92e,
0x93e, 0x930, 0x93e, 0x938, 0x948, 0x928, 0x20, 0x92e, 0x93e, 0x930, 0x940, 0x928,
0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x91c, 0x947, 0x935, 0x94b, 0x938, 0x93e, 0x930,
0x93e, 0x924, 0x94b, 0x935, 0x938, 0x93f, 0x92e, 0x94d, 0x92b, 0x93c, 0x947, 0x930,
0x94b, 0x92a, 0x94b, 0x932, 0x938, 0x94d, 0x915, 0x94b, 0x92a, 0x94d, 0x91c, 0x947,
0x938, 0x94b, 0x92b, 0x93c, 0x93f, 0x92f, 0x93e, 0x938, 0x94d, 0x91f, 0x949, 0x915,
0x939, 0x94b, 0x92e, 0x924, 0x947, 0x932, 0x93f, 0x928, 0x91f, 0x93e, 0x907, 0x930,
0x947, 0x928, 0x909, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94b, 0x935, 0x94d, 0x938,
0x94d, 0x915, 0x935, 0x93e, 0x926, 0x941, 0x91c, 0x93c, 0x935, 0x947, 0x91f, 0x93f,
0x915, 0x928, 0x935, 0x93f, 0x90f, 0x928, 0x93e, 0x935, 0x93f, 0x932, 0x94d, 0x928,
0x93f, 0x92f, 0x938, 0x935, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930,
0x93e, 0x921, 0x935, 0x949, 0x930, 0x938, 0x949, 0x91c, 0x93c, 0x93e, 0x917, 0x94d,
0x930, 0x947, 0x92c, 0x91c, 0x93c, 0x94d, 0x92f, 0x942, 0x930, 0x93f, 0x916, 0x93c,
0x90f, 0x902, 0x91f, 0x93e, 0x928, 0x93e, 0x928, 0x930, 0x940, 0x935, 0x94b, 0x936,
0x93e, 0x917, 0x94b, 0x938, 0x915, 0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938, 0x915,
0x94b, 0x915, 0x94b, 0x938, 0x915, 0x94b, 0x92e, 0x94b, 0x930, 0x94b, 0x915, 0x930,
0x917, 0x941, 0x932, 0x947, 0x928, 0x92e, 0x93e, 0x932, 0x926, 0x940, 0x935, 0x92e,
0x949, 0x930, 0x940, 0x936, 0x938, 0x92e, 0x93e, 0x92f, 0x94b, 0x924, 0x94d, 0x924,
0x947, 0x930, 0x940, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x928, 0x90f, 0x92a, 0x93f,
0x92f, 0x93e, 0x911, 0x915, 0x932, 0x948, 0x902, 0x921, 0x92c, 0x94b, 0x917, 0x928,
0x935, 0x93f, 0x932, 0x947, 0x91a, 0x948, 0x925, 0x92e, 0x91a, 0x915, 0x908, 0x938,
0x94d, 0x91f, 0x930, 0x90f, 0x92b, 0x93c, 0x947, 0x91f, 0x92b, 0x93c, 0x93e, 0x915,
0x93e, 0x913, 0x92b, 0x93c, 0x94b, 0x92b, 0x93c, 0x93f, 0x91c, 0x940, 0x92b, 0x93c,
0x94d, 0x92f, 0x942, 0x928, 0x93e, 0x92b, 0x93c, 0x941, 0x91f, 0x940, 0x917, 0x947,
0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x917, 0x948, 0x902, 0x92c, 0x93f,
0x92f, 0x930, 0x917, 0x94d, 0x935, 0x93e, 0x921, 0x932, 0x915, 0x928, 0x93e, 0x932,
0x939, 0x94b, 0x928, 0x94b, 0x932, 0x941, 0x932, 0x941, 0x90f, 0x902, 0x921, 0x930,
0x92c, 0x930, 0x940, 0x915, 0x94b, 0x938, 0x930, 0x93e, 0x90f, 0x915, 0x94d, 0x935,
0x93e, 0x91c, 0x93c, 0x93e, 0x932, 0x940, 0x928, 0x92e, 0x93e, 0x91c, 0x941, 0x930,
0x94b, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x947, 0x938, 0x938, 0x92e, 0x93f, 0x921,
0x935, 0x947, 0x928, 0x94c, 0x930, 0x941, 0x928, 0x940, 0x92f, 0x942, 0x928, 0x949,
0x930, 0x92b, 0x93c, 0x949, 0x915, 0x928, 0x94c, 0x92e, 0x93f, 0x92f, 0x93e, 0x92a,
0x93e, 0x917, 0x94b, 0x20, 0x92a, 0x93e, 0x917, 0x94b, 0x92a, 0x932, 0x93e, 0x90a,
0x92a, 0x94b, 0x928, 0x92a, 0x947, 0x908, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20,
0x92e, 0x94b, 0x930, 0x947, 0x938, 0x94d, 0x92c, 0x940, 0x930, 0x93e, 0x930, 0x94b,
0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x938, 0x93e, 0x92f, 0x92a, 0x93e, 0x928, 0x924,
0x93e, 0x939, 0x93f, 0x924, 0x940, 0x91f, 0x93e, 0x930, 0x93e, 0x935, 0x93e, 0x91f,
0x94b, 0x902, 0x917, 0x93e, 0x91f, 0x93e, 0x92a, 0x942, 0x41, 0x73, 0x6d, 0x65,
0x72, 0x61, 0x53, 0x61, 0x6f, 0x20, 0x54, 0x6f, 0x6d, 0x65, 0x43, 0x6f,
0x72, 0x61, 0x6c, 0x20, 0x48, 0x61, 0x72, 0x62, 0x6f, 0x75, 0x72, 0x43,
0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x4a, 0x75, 0x61, 0x72, 0x65, 0x7a,
0x43, 0x75, 0x72, 0x61, 0x63, 0x61, 0x6f, 0x53, 0x74, 0x20, 0x42, 0x61,
0x72, 0x74, 0x68, 0x65, 0x6c, 0x65, 0x6d, 0x79, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x44, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x41, 0x71,
0x74, 0x61, 0x75, 0x53, 0x61, 0x69, 0x67, 0x6f, 0x6e, 0x51, 0x6f, 0x73,
0x74, 0x61, 0x6e, 0x61, 0x79, 0x46, 0x61, 0x65, 0x72, 0x6f, 0x65, 0x52,
0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x54, 0x72, 0x75, 0x6b, 0x41, 0x64,
0x64, 0x69, 0x73, 0x7a, 0x2d, 0x41, 0x62, 0x65, 0x62, 0x61, 0x41, 0x6c,
0x67, 0xed, 0x72, 0x4b, 0x61, 0x69, 0x72, 0xf3, 0x44, 0x61, 0x72, 0x20,
0x65, 0x73, 0x2d, 0x53, 0x61, 0x6c, 0x61, 0x61, 0x6d, 0x44, 0x7a, 0x73,
0x69, 0x62, 0x75, 0x74, 0x69, 0x45, 0x6c, 0x2d, 0x41, 0x6a, 0xfa, 0x6e,
0x4b, 0x61, 0x72, 0x74, 0xfa, 0x6d, 0x4d, 0x61, 0x6c, 0x61, 0x62, 0xf3,
0x54, 0x75, 0x6e, 0x69, 0x73, 0x7a, 0x4b, 0x61, 0x6a, 0x6d, 0xe1, 0x6e,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x4d, 0x65, 0x78,
0x69, 0x6b, 0xf3, 0x76, 0xe1, 0x72, 0x6f, 0x73, 0x42, 0x65, 0x75, 0x6c,
0x61, 0x68, 0x2c, 0x20, 0xc9, 0x73, 0x7a, 0x61, 0x6b, 0x2d, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20,
0xc9, 0x73, 0x7a, 0x61, 0x6b, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0xc9,
0x73, 0x7a, 0x61, 0x6b, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x56,
0x6f, 0x73, 0x7a, 0x74, 0x6f, 0x6b, 0xc1, 0x64, 0x65, 0x6e, 0x42, 0x69,
0x73, 0x6b, 0x65, 0x6b, 0x43, 0x73, 0x69, 0x74, 0x61, 0x43, 0x73, 0x6f,
0x6a, 0x62, 0x61, 0x6c, 0x73, 0x7a, 0x61, 0x6e, 0x44, 0x61, 0x6d, 0x61,
0x73, 0x7a, 0x6b, 0x75, 0x73, 0x7a, 0x44, 0x61, 0x6b, 0x6b, 0x61, 0x44,
0x75, 0x73, 0x61, 0x6e, 0x62, 0x65, 0x47, 0xe1, 0x7a, 0x61, 0x48, 0x6f,
0x20, 0x53, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x2d, 0x76, 0xe1, 0x72,
0x6f, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b, 0x4a, 0x65,
0x72, 0x75, 0x7a, 0x73, 0xe1, 0x6c, 0x65, 0x6d, 0x4b, 0x61, 0x6d, 0x63,
0x73, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x63, 0x73, 0x69,
0x4b, 0x72, 0x61, 0x73, 0x7a, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x7a,
0x6b, 0x4b, 0x75, 0x63, 0x73, 0x65, 0x6e, 0x67, 0x4d, 0x61, 0x6b, 0x61,
0xf3, 0x4d, 0x61, 0x73, 0x7a, 0x6b, 0x61, 0x74, 0x4e, 0x6f, 0x76, 0x6f,
0x6b, 0x75, 0x7a, 0x6e, 0x79, 0x65, 0x63, 0x6b, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x7a, 0x69, 0x62, 0x69, 0x72, 0x73, 0x7a, 0x6b, 0x4f, 0x6d, 0x73,
0x7a, 0x6b, 0x50, 0x68, 0x65, 0x6e, 0x6a, 0x61, 0x6e, 0x4b, 0x6f, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x61, 0x6a, 0x53, 0x7a, 0x61, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x53, 0x7a, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x64,
0x53, 0x7a, 0xf6, 0x75, 0x6c, 0x53, 0x61, 0x6e, 0x67, 0x68, 0x61, 0x6a,
0x53, 0x7a, 0x69, 0x6e, 0x67, 0x61, 0x70, 0xfa, 0x72, 0x53, 0x7a, 0x72,
0x65, 0x64, 0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0x69, 0x6d, 0x73, 0x7a, 0x6b,
0x54, 0x61, 0x73, 0x6b, 0x65, 0x6e, 0x74, 0x54, 0x62, 0x69, 0x6c, 0x69,
0x73, 0x7a, 0x69, 0x54, 0x69, 0x6d, 0x70, 0x75, 0x54, 0x6f, 0x6b, 0x69,
0xf3, 0x54, 0x6f, 0x6d, 0x73, 0x7a, 0x6b, 0x55, 0x6c, 0xe1, 0x6e, 0x62,
0xe1, 0x74, 0x6f, 0x72, 0xdc, 0x72, 0xfc, 0x6d, 0x63, 0x73, 0x69, 0x55,
0x73, 0x7a, 0x74, 0x79, 0x2d, 0x4e, 0x79, 0x65, 0x72, 0x61, 0x56, 0x6c,
0x61, 0x67, 0x79, 0x69, 0x76, 0x6f, 0x73, 0x7a, 0x74, 0x6f, 0x6b, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x79, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4a, 0x65, 0x72,
0x65, 0x76, 0xe1, 0x6e, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x2d, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x4b, 0x61, 0x6e, 0xe1, 0x72, 0x69,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x5a, 0xf6, 0x6c,
0x64, 0x2d, 0x66, 0x6f, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x65, 0x6b, 0x46, 0x65, 0x72, 0xf6, 0x65, 0x72, 0x44, 0xe9, 0x6c,
0x69, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x53, 0x7a, 0x65,
0x6e, 0x74, 0x20, 0x49, 0x6c, 0x6f, 0x6e, 0x61, 0x41, 0x6d, 0x73, 0x7a,
0x74, 0x65, 0x72, 0x64, 0x61, 0x6d, 0x41, 0x73, 0x7a, 0x74, 0x72, 0x61,
0x68, 0xe1, 0x6e, 0x42, 0x65, 0x6c, 0x67, 0x72, 0xe1, 0x64, 0x50, 0x6f,
0x7a, 0x73, 0x6f, 0x6e, 0x79, 0x42, 0x72, 0xfc, 0x73, 0x73, 0x7a, 0x65,
0x6c, 0x4b, 0x6f, 0x70, 0x70, 0x65, 0x6e, 0x68, 0xe1, 0x67, 0x61, 0x47,
0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0xe1, 0x72, 0x4d, 0x61, 0x6e, 0x2d,
0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x49, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x62, 0x75, 0x6c, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x79, 0x69, 0x6e, 0x67,
0x72, 0xe1, 0x64, 0x4c, 0x69, 0x73, 0x73, 0x7a, 0x61, 0x62, 0x6f, 0x6e,
0x4d, 0xe1, 0x6c, 0x74, 0x61, 0x4d, 0x69, 0x6e, 0x73, 0x7a, 0x6b, 0x4d,
0x6f, 0x73, 0x7a, 0x6b, 0x76, 0x61, 0x50, 0xe1, 0x72, 0x69, 0x7a, 0x73,
0x50, 0x72, 0xe1, 0x67, 0x61, 0x52, 0xf3, 0x6d, 0x61, 0x53, 0x7a, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x53, 0x7a, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76,
0xf3, 0x53, 0x7a, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x76, 0x53, 0x7a, 0x69,
0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x53, 0x7a, 0x6b, 0x6f,
0x70, 0x6a, 0x65, 0x53, 0x7a, 0xf3, 0x66, 0x69, 0x61, 0x55, 0x6c, 0x6a,
0x61, 0x6e, 0x6f, 0x76, 0x73, 0x7a, 0x6b, 0x42, 0xe9, 0x63, 0x73, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1, 0x64, 0x56, 0x61, 0x72, 0x73,
0xf3, 0x5a, 0xe1, 0x67, 0x72, 0xe1, 0x62, 0x4b, 0x61, 0x72, 0xe1, 0x63,
0x73, 0x6f, 0x6e, 0x79, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x4b,
0xf3, 0x6b, 0x75, 0x73, 0x7a, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0xf3, 0x4d, 0x61, 0x6c, 0x64, 0xed, 0x76,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65,
0x6b, 0x48, 0xfa, 0x73, 0x76, 0xe9, 0x74, 0x2d, 0x73, 0x7a, 0x69, 0x67,
0x65, 0x74, 0x65, 0x6b, 0x46, 0x69, 0x64, 0x7a, 0x73, 0x69, 0x47, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x2d, 0x73, 0x7a, 0x69, 0x67,
0x65, 0x74, 0x65, 0x6b, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x2d,
0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x4b, 0x69, 0x72, 0x69,
0x74, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67,
0x65, 0x74, 0x65, 0x6b, 0x4b, 0x77, 0x61, 0x6a, 0x61, 0x6c, 0x65, 0x69,
0x6e, 0x2d, 0x7a, 0xe1, 0x74, 0x6f, 0x6e, 0x79, 0x4d, 0x61, 0x6a, 0x75,
0x72, 0x6f, 0x2d, 0x7a, 0xe1, 0x74, 0x6f, 0x6e, 0x79, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x65, 0x6b, 0x4d, 0x69, 0x64, 0x77, 0x61, 0x79, 0x2d, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69,
0x72, 0x6e, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x50,
0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x65, 0x6b, 0x57, 0x61, 0x6b, 0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x41, 0x6c, 0x67, 0x65, 0x69, 0x72, 0x73, 0x62, 0x6f, 0x72, 0x67,
0x42, 0x69, 0x73, 0x73, 0xe1, 0x4b, 0x61, 0xed, 0x72, 0xf3, 0x44, 0x6a,
0x69, 0x62, 0xfa, 0x74, 0xed, 0x4a, 0xf3, 0x68, 0x61, 0x6e, 0x6e, 0x65,
0x73, 0x61, 0x72, 0x62, 0x6f, 0x72, 0x67, 0x53, 0x61, 0xf3, 0x20, 0x54,
0xf3, 0x6d, 0x65, 0x54, 0x72, 0xed, 0x70, 0xf3, 0x6c, 0xed, 0x54, 0xfa,
0x6e, 0x69, 0x73, 0x62, 0x6f, 0x72, 0x67, 0x41, 0x6e, 0x67, 0x76, 0x69,
0x6c, 0x6c, 0x61, 0x41, 0x6e, 0x74, 0xed, 0x67, 0x76, 0x61, 0x41, 0x72,
0xfa, 0x62, 0x61, 0x42, 0x65, 0x6c, 0xed, 0x73, 0x4b, 0x61, 0x6e, 0x6b,
0xfa, 0x6e, 0x43, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0x2d, 0x65, 0x79, 0x6a,
0x61, 0x72, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x72, 0xed, 0x6b, 0x61, 0x44,
0xf3, 0x6d, 0x69, 0x6e, 0xed, 0x6b, 0x61, 0x47, 0x76, 0x61, 0x64, 0x65,
0x6c, 0xfa, 0x70, 0x47, 0x76, 0xe6, 0x6a, 0x61, 0x6e, 0x61, 0x4a, 0x61,
0x6d, 0x61, 0xed, 0x6b, 0x61, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0xed,
0x6b, 0x4d, 0x65, 0x78, 0xed, 0x6b, 0xf3, 0x62, 0x6f, 0x72, 0x67, 0x50,
0xfa, 0x65, 0x72, 0x74, 0xf3, 0x20, 0x52, 0xed, 0x6b, 0xf3, 0x53, 0x61,
0x6e, 0x6b, 0x74, 0x69, 0x20, 0x42, 0x61, 0x72, 0x74, 0xf3, 0x6c, 0xf3,
0x6d, 0x65, 0x75, 0x73, 0x61, 0x72, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x53,
0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73, 0x53,
0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x4c, 0xfa, 0x73, 0xed, 0x61, 0x53,
0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73,
0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x56, 0x69, 0x6e, 0x73, 0x65,
0x6e, 0x74, 0x54, 0x6f, 0x72, 0x74, 0xf3, 0x6c, 0x61, 0x42, 0x65, 0x69,
0x72, 0xfa, 0x74, 0x42, 0x72, 0xfa, 0x6e, 0x65, 0x69, 0x4b, 0xf3, 0x6c,
0x6f, 0x6d, 0x62, 0xf3, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d,
0x69, 0x6e, 0x68, 0x2d, 0x62, 0x6f, 0x72, 0x67, 0x4a, 0x65, 0x72, 0xfa,
0x73, 0x61, 0x6c, 0x65, 0x6d, 0x4b, 0x61, 0x62, 0xfa, 0x6c, 0x4b, 0x61,
0x6c, 0x6b, 0xfa, 0x74, 0x74, 0x61, 0x4b, 0xfa, 0x61, 0x6c, 0x61, 0x20,
0x4c, 0xfa, 0x6d, 0x70, 0xfa, 0x72, 0x4b, 0xfa, 0x76, 0x65, 0x69, 0x74,
0x4e, 0xed, 0x6b, 0xf3, 0x73, 0xed, 0x61, 0x52, 0xed, 0x6a, 0x61, 0x64,
0x53, 0x6a, 0x61, 0x6e, 0x67, 0x68, 0xe6, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0xfa, 0x72, 0x54, 0xf3, 0x6b, 0xfd, 0xf3, 0xda, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x79,
0x6a, 0x61, 0x72, 0x42, 0x65, 0x72, 0x6d, 0xfa, 0x64, 0x61, 0x4b, 0x61,
0x6e, 0x61, 0x72, 0xed, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x47, 0x72, 0xe6,
0x6e, 0x68, 0xf6, 0x66, 0xf0, 0x61, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x46,
0xe6, 0x72, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x53, 0x75, 0xf0, 0x75, 0x72,
0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0xed, 0x61, 0x53, 0x61, 0x6e, 0x6b,
0x74, 0x69, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0xfe, 0x65,
0x6e, 0x61, 0x42, 0xfa, 0x6b, 0x61, 0x72, 0x65, 0x73, 0x74, 0x42, 0xfa,
0x64, 0x61, 0x70, 0x65, 0x73, 0x74, 0x4b, 0x61, 0x75, 0x70, 0x6d, 0x61,
0x6e, 0x6e, 0x61, 0x68, 0xf6, 0x66, 0x6e, 0x47, 0xed, 0x62, 0x72, 0x61,
0x6c, 0x74, 0x61, 0x72, 0x4d, 0xf6, 0x6e, 0x49, 0x73, 0x74, 0x61, 0x6e,
0x62, 0xfa, 0x6c, 0x4b, 0xe6, 0x6e, 0x75, 0x67, 0x61, 0x72, 0xf0, 0x75,
0x72, 0x4c, 0x75, 0x6e, 0x64, 0xfa, 0x6e, 0x69, 0x72, 0x4c, 0xfa, 0x78,
0x65, 0x6d, 0x62, 0x6f, 0x72, 0x67, 0x4d, 0x61, 0x64, 0x72, 0xed, 0x64,
0x4d, 0x61, 0x72, 0xed, 0x75, 0x68, 0xf6, 0x66, 0x6e, 0x4d, 0xf3, 0x6e,
0x61, 0x6b, 0xf3, 0x4f, 0x73, 0x6c, 0xf3, 0x53, 0x61, 0x6e, 0x20, 0x4d,
0x61, 0x72, 0xed, 0x6e, 0xf3, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76,
0xf3, 0x53, 0xf3, 0x66, 0xed, 0x61, 0x53, 0x74, 0x6f, 0x6b, 0x6b, 0x68,
0xf3, 0x6c, 0x6d, 0x75, 0x72, 0x54, 0xed, 0x72, 0x61, 0x6e, 0x61, 0x56,
0x61, 0x74, 0xed, 0x6b, 0x61, 0x6e, 0x69, 0xf0, 0x56, 0xed, 0x6e, 0x56,
0x69, 0x6c, 0x6e, 0xed, 0x75, 0x73, 0x56, 0x61, 0x72, 0x73, 0x6a, 0xe1,
0x4a, 0xf3, 0x6c, 0x61, 0x65, 0x79, 0x4b, 0xf3, 0x6b, 0x6f, 0x73, 0x65,
0x79, 0x6a, 0x61, 0x72, 0x4d, 0x61, 0x6c, 0x64, 0xed, 0x76, 0x65, 0x79,
0x6a, 0x61, 0x72, 0x4d, 0xe1, 0x72, 0x69, 0x74, 0xed, 0x75, 0x73, 0x50,
0xe1, 0x73, 0x6b, 0x61, 0x65, 0x79, 0x6a, 0x61, 0x46, 0xed, 0x64, 0x6a,
0xed, 0x47, 0x76, 0x61, 0x6d, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73,
0x61, 0x73, 0x2d, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x4e, 0xe1, 0x72, 0xfa,
0x50, 0x61, 0x6c, 0xe1, 0x54, 0x61, 0x68, 0xed, 0x74, 0xed, 0x41, 0x6c,
0x6a, 0x69, 0x72, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x55, 0x74, 0x61, 0x72, 0x61, 0x43,
0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x20, 0x55, 0x74, 0x61, 0x72, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53,
0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x55, 0x74, 0x61, 0x72, 0x61, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x75, 0x72, 0x61, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20,
0x53, 0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e, 0x41, 0x74, 0x68, 0x65, 0x6e,
0x61, 0x4b, 0x69, 0x73, 0x68, 0x69, 0x6e, 0x65, 0x76, 0x50, 0x75, 0x6c,
0x61, 0x75, 0x20, 0x4d, 0x61, 0x6e, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61,
0x57, 0x61, 0x72, 0x73, 0x61, 0x77, 0x61, 0x4d, 0x61, 0x6c, 0x61, 0x64,
0x65, 0x77, 0x61, 0x43, 0x69, 0x74, 0x61, 0x74, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x4e, 0x6f, 0x76, 0x65, 0x20,
0x59, 0x6f, 0x72, 0x6b, 0x53, 0x61, 0x6e, 0x63, 0x74, 0x65, 0x20, 0x42,
0x61, 0x72, 0x74, 0x68, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x6f, 0x53, 0x61,
0x6e, 0x63, 0x74, 0x65, 0x20, 0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65,
0x73, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f,
0x76, 0x61, 0x53, 0x61, 0x6e, 0x63, 0x74, 0x65, 0x20, 0x43, 0x68, 0x72,
0x69, 0x73, 0x74, 0x6f, 0x70, 0x68, 0x6f, 0x72, 0x6f, 0x53, 0x61, 0x6e,
0x63, 0x74, 0x65, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61, 0x53, 0x61, 0x6e,
0x63, 0x74, 0x65, 0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x61,
0x6e, 0x63, 0x74, 0x65, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74,
0x65, 0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x41,
0x74, 0x68, 0x65, 0x6e, 0x61, 0x73, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x69, 0x73, 0x62, 0x6f,
0x6e, 0x61, 0x4e, 0x61, 0x74, 0x61, 0x6c, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x6f, 0x4d, 0x61, 0x79, 0x6f, 0x74, 0x74, 0x61, 0x50, 0x61,
0x73, 0x63, 0x68, 0x61, 0x44, 0x61, 0x72, 0x2d, 0x65, 0x73, 0x2d, 0x53,
0x61, 0x6c, 0x61, 0x61, 0x6d, 0x45, 0x6c, 0x2d, 0x41, 0x61, 0x69, 0x75,
0x6e, 0x53, 0xe3, 0x6f, 0x2d, 0x54, 0x6f, 0x6d, 0xe9, 0x42, 0x75, 0x65,
0x6e, 0x6f, 0x73, 0x2d, 0x41, 0x69, 0x72, 0x65, 0x73, 0x4c, 0x61, 0x2d,
0x52, 0x69, 0x6f, 0x6a, 0x61, 0x52, 0x69, 0x6f, 0x2d, 0x47, 0x61, 0x6c,
0x6c, 0x65, 0x67, 0x6f, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x2d, 0x4a, 0x75,
0x61, 0x6e, 0x53, 0x61, 0x6e, 0x74, 0x2d, 0x4c, 0x75, 0x69, 0x73, 0x62,
0x61, 0x79, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x61, 0x73, 0x43, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x67, 0x65, 0x2d,
0x42, 0x61, 0x79, 0x43, 0x61, 0x6d, 0x70, 0x6f, 0x2d, 0x47, 0x72, 0x61,
0x6e, 0x64, 0x65, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x2d, 0x4a, 0x75,
0x61, 0x72, 0x65, 0x7a, 0x43, 0x6f, 0x73, 0x74, 0x61, 0x2d, 0x52, 0x69,
0x63, 0x61, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x2d, 0x43, 0x72, 0x65,
0x65, 0x6b, 0x45, 0x6c, 0x2d, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f,
0x72, 0x47, 0x6c, 0x61, 0x63, 0x65, 0x2d, 0x42, 0x61, 0x79, 0x47, 0x6f,
0x6f, 0x73, 0x65, 0x2d, 0x42, 0x61, 0x79, 0x47, 0x72, 0x61, 0x6e, 0x64,
0x2d, 0x54, 0x75, 0x72, 0x6b, 0x47, 0x75, 0x61, 0x64, 0x65, 0x6c, 0x75,
0x70, 0x61, 0x54, 0x65, 0x6c, 0x6c, 0x2d, 0x43, 0x69, 0x74, 0x79, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4c, 0x6f, 0x73, 0x2d,
0x41, 0x6e, 0x67, 0x65, 0x6c, 0x65, 0x73, 0x4c, 0x6f, 0x77, 0x65, 0x72,
0x2d, 0x50, 0x72, 0x69, 0x6e, 0x63, 0x65, 0x2019, 0x73, 0x2d, 0x51, 0x75,
0x61, 0x72, 0x74, 0x65, 0x72, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x43, 0x69, 0x74, 0xe9, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c,
0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x64,
0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53,
0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x2d, 0x52,
0x69, 0x63, 0x6f, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x2d, 0x49, 0x6e,
0x6c, 0x65, 0x74, 0x52, 0x69, 0x6f, 0x2d, 0x42, 0x72, 0x61, 0x6e, 0x63,
0x6f, 0x53, 0x61, 0x6e, 0x2d, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f,
0x53, 0xe3, 0x6f, 0x2d, 0x50, 0x61, 0x75, 0x6c, 0x6f, 0x53, 0x61, 0x6e,
0x74, 0x2d, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x6f,
0x53, 0x74, 0x2e, 0x2d, 0x4a, 0x6f, 0x68, 0x6e, 0x2019, 0x73, 0x53, 0x74,
0x2e, 0x2d, 0x4b, 0x69, 0x74, 0x74, 0x73, 0x53, 0x74, 0x2e, 0x2d, 0x4c,
0x75, 0x63, 0x69, 0x61, 0x53, 0x74, 0x2e, 0x2d, 0x54, 0x68, 0x6f, 0x6d,
0x61, 0x73, 0x53, 0x74, 0x2e, 0x2d, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e,
0x74, 0x53, 0x77, 0x69, 0x66, 0x74, 0x2d, 0x43, 0x75, 0x72, 0x72, 0x65,
0x6e, 0x74, 0x50, 0x68, 0x6e, 0x6f, 0x6d, 0x2d, 0x50, 0x65, 0x6e, 0x68,
0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x69, 0x53, 0x75, 0x64,
0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x53, 0x61, 0x6e, 0x74,
0x2d, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x42, 0x72, 0x6f, 0x6b, 0x65,
0x6e, 0x2d, 0x48, 0x69, 0x6c, 0x6c, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x42, 0x75, 0x63,
0x75, 0x72, 0x65, 0x73, 0x74, 0x69, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4d, 0x6f, 0x73, 0x63, 0x76, 0x61,
0x50, 0x61, 0x73, 0x63, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x4d, 0x6f,
0x72, 0x65, 0x73, 0x62, 0x79, 0x43, 0x61, 0x74, 0x68, 0x61, 0x69, 0x72,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x6c, 0x67, 0xe9, 0x69, 0x72,
0x65, 0x43, 0x61, 0x69, 0x72, 0x65, 0x6f, 0x43, 0x6f, 0x6e, 0x61, 0x63,
0x72, 0x61, 0xed, 0x44, 0x61, 0x63, 0xe1, 0x72, 0x44, 0xe1, 0x72, 0x61,
0x73, 0x61, 0x6c, 0xe1, 0x6d, 0x4c, 0xe1, 0xfa, 0x69, 0x6e, 0x65, 0x43,
0x61, 0x72, 0x74, 0xfa, 0x6d, 0x4d, 0x61, 0x70, 0xfa, 0x74, 0xf3, 0x4d,
0x6f, 0x67, 0x61, 0x69, 0x64, 0x69, 0x73, 0x69, 0xfa, 0x4e, 0x75, 0x61,
0x63, 0x73, 0x61, 0x74, 0x54, 0x72, 0x69, 0x70, 0x69, 0x6c, 0xed, 0x54,
0xfa, 0x69, 0x6e, 0x69, 0x73, 0x41, 0x6e, 0x67, 0x61, 0xed, 0x6c, 0x65,
0x42, 0x61, 0x72, 0x62, 0x61, 0x64, 0xf3, 0x73, 0x61, 0x6e, 0x20, 0x42,
0x68, 0x65, 0x69, 0x6c, 0xed, 0x73, 0x42, 0x6f, 0x67, 0x61, 0x74, 0xe1,
0x43, 0xf3, 0x73, 0x74, 0x61, 0x20, 0x52, 0xed, 0x63, 0x65, 0x43, 0xfa,
0x72, 0x61, 0x73, 0xf3, 0x61, 0x6e, 0x20, 0x74, 0x53, 0x61, 0x6c, 0x76,
0x61, 0x64, 0xf3, 0x69, 0x72, 0x47, 0x72, 0x65, 0x61, 0x6e, 0xe1, 0x64,
0x61, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c, 0xfa, 0x69, 0x70, 0x61, 0x6e,
0x20, 0x47, 0x68, 0x75, 0xe1, 0x69, 0x6e, 0x49, 0x61, 0x6d, 0xe1, 0x69,
0x63, 0x65, 0x4c, 0xed, 0x6f, 0x6d, 0x61, 0x43, 0x61, 0x74, 0x68, 0x61,
0x69, 0x72, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x63, 0x73, 0x69, 0x63, 0x65,
0x6f, 0x4e, 0x75, 0x61, 0x2d, 0x45, 0x61, 0x62, 0x68, 0x72, 0x61, 0x63,
0x50, 0xf3, 0x72, 0x74, 0xf3, 0x20, 0x52, 0xed, 0x63, 0x65, 0x53, 0x61,
0x6e, 0x20, 0x43, 0x72, 0xed, 0x6f, 0x73, 0x74, 0xf3, 0x69, 0x72, 0x53,
0x61, 0x6e, 0x20, 0x55, 0x69, 0x6e, 0x73, 0x65, 0x61, 0x6e, 0x6e, 0x49,
0x6e, 0x69, 0x73, 0x20, 0x54, 0x75, 0x69, 0x6c, 0x65, 0x4d, 0x61, 0x63,
0x20, 0x47, 0x75, 0x61, 0x69, 0x72, 0x65, 0xc1, 0x69, 0x64, 0x69, 0x6e,
0x42, 0x61, 0x69, 0x72, 0xe9, 0x69, 0x6e, 0x42, 0x61, 0x6b, 0x69, 0x42,
0xe9, 0x69, 0x72, 0x69, 0xfa, 0x74, 0x42, 0x72, 0xfa, 0x69, 0x6e, 0xe9,
0x61, 0x6e, 0x20, 0x44, 0x61, 0x6d, 0x61, 0x69, 0x73, 0x63, 0x48, 0x65,
0x61, 0x62, 0x72, 0xf3, 0x6e, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x43, 0x6f,
0x6e, 0x67, 0x49, 0x61, 0x63, 0xe1, 0x72, 0x74, 0x61, 0x49, 0x61, 0x72,
0xfa, 0x73, 0x61, 0x69, 0x6c, 0xe9, 0x69, 0x6d, 0x43, 0x61, 0x62, 0xfa,
0x6c, 0x43, 0x61, 0x6c, 0x63, 0xfa, 0x74, 0x61, 0x43, 0x75, 0xe1, 0x69,
0x74, 0x4d, 0x61, 0x63, 0x61, 0x73, 0x61, 0x72, 0x4d, 0x61, 0x69, 0x6e,
0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x4e, 0x69, 0x6f, 0x63, 0xf3, 0x69,
0x73, 0x53, 0xfa, 0x6c, 0x53, 0x68, 0x61, 0x6e, 0x67, 0x2d, 0x68, 0x61,
0x69, 0x53, 0x69, 0x6e, 0x67, 0x65, 0x61, 0x70, 0xf3, 0x72, 0x54, 0xf3,
0x69, 0x63, 0x65, 0x6f, 0x49, 0x61, 0x63, 0xfa, 0x74, 0x73, 0x63, 0x45,
0x69, 0x72, 0x65, 0x61, 0x76, 0xe1, 0x6e, 0x6e, 0x61, 0x20, 0x68, 0x41,
0x73, 0xf3, 0x69, 0x72, 0x42, 0x65, 0x69, 0x72, 0x6d, 0x69, 0xfa, 0x64,
0x61, 0x6e, 0x61, 0x20, 0x68, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e,
0x20, 0x43, 0x68, 0x61, 0x6e, 0xe1, 0x72, 0x61, 0x63, 0x68, 0x61, 0x52,
0x69, 0x6e, 0x6e, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x69, 0x6c,
0x65, 0xe1, 0x69, 0x6e, 0x20, 0x46, 0x68, 0x61, 0x72, 0xf3, 0x4d, 0x61,
0x69, 0x64, 0xe9, 0x61, 0x72, 0x61, 0x52, 0xe9, 0x69, 0x63, 0x69, 0x76,
0xed, 0x63, 0x61, 0x6e, 0x20, 0x74, 0x53, 0x65, 0x6f, 0x69, 0x72, 0x73,
0x69, 0x61, 0x20, 0x54, 0x68, 0x65, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x20,
0x48, 0xe9, 0x69, 0x6c, 0x69, 0x6e, 0x41, 0x6d, 0x73, 0x74, 0x61, 0x72,
0x64, 0x61, 0x6d, 0x41, 0x6e, 0x64, 0xf3, 0x72, 0x61, 0x61, 0x6e, 0x20,
0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0xe1, 0x69, 0x6e, 0x61, 0x6e, 0x20,
0x41, 0x69, 0x74, 0x68, 0x69, 0x6e, 0x42, 0xe9, 0x61, 0x6c, 0x67, 0x72,
0xe1, 0x64, 0x42, 0x65, 0x69, 0x72, 0x6c, 0xed, 0x6e, 0x61, 0x6e, 0x20,
0x42, 0x68, 0x72, 0x61, 0x74, 0x61, 0x73, 0x6c, 0xe1, 0x69, 0x76, 0x61,
0x6e, 0x20, 0x42, 0x68, 0x72, 0x75, 0x69, 0x73, 0xe9, 0x69, 0x6c, 0x42,
0xfa, 0x63, 0x61, 0x69, 0x72, 0x69, 0x73, 0x74, 0x42, 0xfa, 0x64, 0x61,
0x69, 0x70, 0x65, 0x69, 0x73, 0x74, 0x43, 0xed, 0x73, 0x69, 0x6e, 0x65,
0xe1, 0x43, 0xf3, 0x62, 0x61, 0x6e, 0x68, 0xe1, 0x76, 0x61, 0x6e, 0x42,
0x61, 0x69, 0x6c, 0x65, 0x20, 0xc1, 0x74, 0x68, 0x61, 0x20, 0x43, 0x6c,
0x69, 0x61, 0x74, 0x68, 0x47, 0x69, 0x6f, 0x62, 0x72, 0xe1, 0x6c, 0x74,
0x61, 0x72, 0x47, 0x65, 0x61, 0x6e, 0x73, 0x61, 0xed, 0x48, 0x65, 0x69,
0x6c, 0x73, 0x69, 0x6e, 0x63, 0xed, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e,
0x20, 0x4d, 0x68, 0x61, 0x6e, 0x61, 0x6e, 0x6e, 0x49, 0x6f, 0x73, 0x74,
0x61, 0x6e, 0x62, 0xfa, 0x6c, 0x47, 0x65, 0x69, 0x72, 0x73, 0xed, 0x43,
0xed, 0x76, 0x4c, 0x69, 0x6f, 0x73, 0x70, 0xf3, 0x69, 0x6e, 0x4c, 0x69,
0xfa, 0x69, 0x62, 0x6c, 0x65, 0xe1, 0x6e, 0x61, 0x4c, 0x6f, 0x6e, 0x64,
0x61, 0x69, 0x6e, 0x4c, 0x75, 0x63, 0x73, 0x61, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x4d, 0x61, 0x69, 0x64, 0x72, 0x69, 0x64, 0x4d, 0x69, 0x6f, 0x6e,
0x73, 0x63, 0x4d, 0x6f, 0x6e, 0x61, 0x63, 0xf3, 0x4d, 0x6f, 0x73, 0x63,
0xf3, 0x50, 0xe1, 0x72, 0x61, 0x73, 0x50, 0x6f, 0x64, 0x67, 0x61, 0x69,
0x72, 0xed, 0x74, 0x73, 0x65, 0x52, 0xed, 0x67, 0x65, 0x61, 0x6e, 0x20,
0x52, 0xf3, 0x69, 0x6d, 0x68, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x69,
0x72, 0xed, 0x6e, 0x65, 0x53, 0x61, 0x69, 0x72, 0xe9, 0x61, 0x76, 0xf3,
0x53, 0x63, 0xf3, 0x69, 0x70, 0xe9, 0x53, 0xf3, 0x69, 0x66, 0x69, 0x61,
0x53, 0x74, 0xf3, 0x63, 0xf3, 0x6c, 0x6d, 0x54, 0x61, 0x69, 0x6c, 0x6c,
0x69, 0x6e, 0x6e, 0x54, 0x69, 0x6f, 0x72, 0xe1, 0x6e, 0x61, 0x56, 0x61,
0x64, 0xfa, 0x73, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x74, 0x61, 0x63, 0xe1,
0x69, 0x6e, 0x56, 0x69, 0x6c, 0x6e, 0x69, 0x61, 0x73, 0x56, 0xe1, 0x72,
0x73, 0xe1, 0x53, 0xe1, 0x67, 0x72, 0x61, 0x62, 0x41, 0x6e, 0x74, 0x61,
0x6e, 0x61, 0x6e, 0x61, 0x69, 0x72, 0xed, 0x76, 0x65, 0x6f, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x6f, 0x6c, 0x6c,
0x61, 0x67, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x43, 0x6f, 0x63,
0x6f, 0x73, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x43, 0x68,
0x6f, 0x6d, 0xf3, 0x72, 0x61, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e,
0x20, 0x4d, 0x68, 0x61, 0x69, 0x6c, 0x64, 0xed, 0x76, 0x65, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d, 0x68, 0x75, 0x69, 0x72, 0xed, 0x73,
0x46, 0x69, 0x64, 0x73, 0xed, 0x6e, 0x61, 0x20, 0x68, 0x4f, 0x69, 0x6c,
0x65, 0xe1, 0x69, 0x6e, 0x20, 0x4d, 0x68, 0x61, 0x72, 0x63, 0x61, 0x73,
0x61, 0x63, 0x68, 0x61, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20,
0x4d, 0x69, 0x64, 0x77, 0x61, 0x79, 0x54, 0x61, 0x69, 0x68, 0xed, 0x74,
0xed, 0x56, 0x61, 0x69, 0x6c, 0xed, 0x73, 0x41, 0x6c, 0x67, 0x65, 0x72,
0x69, 0x49, 0x6c, 0x20, 0x43, 0x61, 0x69, 0x72, 0x6f, 0x47, 0x69, 0x62,
0x75, 0x74, 0x69, 0x45, 0x6c, 0x20, 0x41, 0x79, 0x75, 0x6e, 0x47, 0x69,
0x75, 0x62, 0x61, 0x54, 0x75, 0x6e, 0x69, 0x73, 0x69, 0x43, 0x61, 0x69,
0x65, 0x6e, 0x6e, 0x61, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x75, 0x70,
0x61, 0x4c, 0x2019, 0x41, 0x76, 0x61, 0x6e, 0x61, 0x47, 0x69, 0x61, 0x6d,
0x61, 0x69, 0x63, 0x61, 0x43, 0x69, 0x74, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4d, 0x65, 0x73, 0x73, 0x69, 0x63, 0x6f, 0x42, 0x65, 0x75,
0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x43, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x4e, 0x65, 0x77, 0x20, 0x53,
0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x50, 0x6f, 0x72,
0x74, 0x6f, 0x72, 0x69, 0x63, 0x6f, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x61,
0x6f, 0x6c, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75, 0x63,
0x69, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2019, 0x46, 0x61, 0x6d,
0x61, 0x67, 0x6f, 0x73, 0x74, 0x61, 0x47, 0x69, 0x61, 0x63, 0x61, 0x72,
0x74, 0x61, 0x47, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x6d,
0x65, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72, 0x65, 0x49, 0x73, 0x6f, 0x6c,
0x65, 0x20, 0x46, 0xe6, 0x72, 0x20, 0xd8, 0x65, 0x72, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x64,
0x53, 0x61, 0x6e, 0x74, 0x2019, 0x45, 0x6c, 0x65, 0x6e, 0x61, 0x43, 0x6f,
0x70, 0x65, 0x6e, 0x61, 0x67, 0x68, 0x65, 0x6e, 0x47, 0x69, 0x62, 0x69,
0x6c, 0x74, 0x65, 0x72, 0x72, 0x61, 0x49, 0x73, 0x6f, 0x6c, 0x61, 0x20,
0x64, 0x69, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x75, 0x62, 0x69, 0x61, 0x6e,
0x61, 0x4c, 0x6f, 0x6e, 0x64, 0x72, 0x61, 0x4c, 0x75, 0x73, 0x73, 0x65,
0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x50,
0x61, 0x72, 0x69, 0x67, 0x69, 0x53, 0x69, 0x6e, 0x66, 0x65, 0x72, 0x6f,
0x70, 0x6f, 0x6c, 0x69, 0x53, 0x74, 0x6f, 0x63, 0x63, 0x6f, 0x6c, 0x6d,
0x61, 0x43, 0x69, 0x74, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x56,
0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x56, 0x61, 0x72, 0x73, 0x61,
0x76, 0x69, 0x61, 0x5a, 0x61, 0x67, 0x61, 0x62, 0x72, 0x69, 0x61, 0x5a,
0x75, 0x72, 0x69, 0x67, 0x6f, 0x4c, 0x61, 0x20, 0x52, 0x69, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x65, 0x46, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x72, 0x63,
0x68, 0x65, 0x73, 0x69, 0x30a2, 0x30d3, 0x30b8, 0x30e3, 0x30f3, 0x30a2, 0x30af, 0x30e9,
0x30a2, 0x30b8, 0x30b9, 0x30a2, 0x30d9, 0x30d0, 0x30a2, 0x30eb, 0x30b8, 0x30a7, 0x30a2, 0x30b9,
0x30de, 0x30e9, 0x30d0, 0x30de, 0x30b3, 0x30d0, 0x30f3, 0x30ae, 0x30d0, 0x30f3, 0x30b8, 0x30e5,
0x30fc, 0x30eb, 0x30d3, 0x30b5, 0x30a6, 0x30d6, 0x30e9, 0x30f3, 0x30bf, 0x30a4, 0x30e4, 0x30d6,
0x30e9, 0x30b6, 0x30d3, 0x30eb, 0x30d6, 0x30b8, 0x30e5, 0x30f3, 0x30d6, 0x30e9, 0x30ab, 0x30a4,
0x30ed, 0x30ab, 0x30b5, 0x30d6, 0x30e9, 0x30f3, 0x30ab, 0x30bb, 0x30a6, 0x30bf, 0x30b3, 0x30ca,
0x30af, 0x30ea, 0x30c0, 0x30ab, 0x30fc, 0x30eb, 0x30c0, 0x30eb, 0x30a8, 0x30b9, 0x30b5, 0x30e9,
0x30fc, 0x30e0, 0x30b8, 0x30d6, 0x30c1, 0x30c9, 0x30a5, 0x30a2, 0x30e9, 0x30a2, 0x30a4, 0x30a6,
0x30f3, 0x30d5, 0x30ea, 0x30fc, 0x30bf, 0x30a6, 0x30f3, 0x30cf, 0x30dc, 0x30ed, 0x30fc, 0x30cd,
0x30cf, 0x30e9, 0x30ec, 0x30e8, 0x30cf, 0x30cd, 0x30b9, 0x30d6, 0x30eb, 0x30b0, 0x30b8, 0x30e5,
0x30d0, 0x30ab, 0x30f3, 0x30d1, 0x30e9, 0x30cf, 0x30eb, 0x30c4, 0x30fc, 0x30e0, 0x30ad, 0x30ac,
0x30ea, 0x30ad, 0x30f3, 0x30b7, 0x30e3, 0x30b5, 0x30e9, 0x30b4, 0x30b9, 0x30ea, 0x30fc, 0x30d6,
0x30eb, 0x30f4, 0x30a3, 0x30eb, 0x30ed, 0x30e1, 0x30eb, 0x30a2, 0x30f3, 0x30c0, 0x30eb, 0x30d6,
0x30f3, 0x30d0, 0x30b7, 0x30eb, 0x30b5, 0x30ab, 0x30de, 0x30e9, 0x30dc, 0x30de, 0x30d7, 0x30c8,
0x30de, 0x30bb, 0x30eb, 0x30e0, 0x30d0, 0x30d0, 0x30fc, 0x30cd, 0x30e2, 0x30ac, 0x30c7, 0x30a3,
0x30b7, 0x30aa, 0x30e2, 0x30f3, 0x30ed, 0x30d3, 0x30a2, 0x30ca, 0x30a4, 0x30ed, 0x30d3, 0x30f3,
0x30b8, 0x30e3, 0x30e1, 0x30ca, 0x30cb, 0x30a2, 0x30e1, 0x30cc, 0x30a2, 0x30af, 0x30b7, 0x30e7,
0x30c3, 0x30c8, 0x30ef, 0x30ac, 0x30c9, 0x30a5, 0x30b0, 0x30fc, 0x30dd, 0x30eb, 0x30c8, 0x30ce,
0x30dc, 0x30b5, 0x30f3, 0x30c8, 0x30e1, 0x30c8, 0x30ea, 0x30dd, 0x30ea, 0x30c1, 0x30e5, 0x30cb,
0x30b9, 0x30a6, 0x30a3, 0x30f3, 0x30c8, 0x30d5, 0x30c3, 0x30af, 0x30a2, 0x30c0, 0x30c3, 0x30af,
0x30a2, 0x30f3, 0x30ab, 0x30ec, 0x30c3, 0x30b8, 0x30a2, 0x30f3, 0x30ae, 0x30e9, 0x30a2, 0x30f3,
0x30c6, 0x30a3, 0x30b0, 0x30a2, 0x30a2, 0x30e9, 0x30b0, 0x30a1, 0x30a4, 0x30ca, 0x30d6, 0x30a8,
0x30ce, 0x30b9, 0x30a2, 0x30a4, 0x30ec, 0x30b9, 0x30ab, 0x30bf, 0x30de, 0x30eb, 0x30ab, 0x30b3,
0x30eb, 0x30c9, 0x30d0, 0x30d5, 0x30d5, 0x30a4, 0x30e9, 0x30ea, 0x30aa, 0x30cf, 0x30e1, 0x30f3,
0x30c9, 0x30fc, 0x30b5, 0x30ea, 0x30aa, 0x30ac, 0x30b8, 0x30a7, 0x30b4, 0x30b9, 0x30b5, 0x30eb,
0x30bf, 0x30b5, 0x30f3, 0x30d5, 0x30a1, 0x30f3, 0x30b5, 0x30f3, 0x30eb, 0x30a4, 0x30b9, 0x30c8,
0x30a5, 0x30af, 0x30de, 0x30f3, 0x30a6, 0x30b7, 0x30e5, 0x30a2, 0x30a4, 0x30a2, 0x30a2, 0x30eb,
0x30d0, 0x30a2, 0x30b9, 0x30f3, 0x30b7, 0x30aa, 0x30f3, 0x30a2, 0x30c6, 0x30a3, 0x30b3, 0x30ab,
0x30f3, 0x30d0, 0x30a4, 0x30fc, 0x30a2, 0x30d0, 0x30a4, 0x30a2, 0x30d0, 0x30f3, 0x30c7, 0x30e9,
0x30d0, 0x30eb, 0x30d0, 0x30c9, 0x30b9, 0x30d9, 0x30ec, 0x30f3, 0x30d9, 0x30ea, 0x30fc, 0x30ba,
0x30d6, 0x30e9, 0x30f3, 0x30fb, 0x30b5, 0x30d6, 0x30ed, 0x30f3, 0x30dc, 0x30a2, 0x30d3, 0x30b9,
0x30bf, 0x30dc, 0x30b4, 0x30bf, 0x30dc, 0x30a4, 0x30b7, 0x30b1, 0x30f3, 0x30d6, 0x30ea, 0x30c3,
0x30b8, 0x30d9, 0x30a4, 0x30ab, 0x30f3, 0x30dd, 0x30b0, 0x30e9, 0x30f3, 0x30c7, 0x30ab, 0x30f3,
0x30af, 0x30f3, 0x30ab, 0x30e9, 0x30ab, 0x30b9, 0x30ab, 0x30a4, 0x30a8, 0x30f3, 0x30cc, 0x30b1,
0x30a4, 0x30de, 0x30f3, 0x30b7, 0x30ab, 0x30b4, 0x30c1, 0x30ef, 0x30ef, 0x30b7, 0x30a6, 0x30c0,
0x30fc, 0x30fb, 0x30d5, 0x30a2, 0x30ec, 0x30b9, 0x30b3, 0x30b9, 0x30bf, 0x30ea, 0x30ab, 0x30af,
0x30ec, 0x30b9, 0x30c8, 0x30f3, 0x30af, 0x30a4, 0x30a2, 0x30d0, 0x30ad, 0x30e5, 0x30e9, 0x30bd,
0x30fc, 0x30c7, 0x30f3, 0x30de, 0x30fc, 0x30af, 0x30b7, 0x30e3, 0x30a6, 0x30f3, 0x30c9, 0x30fc,
0x30bd, 0x30f3, 0x30c9, 0x30fc, 0x30bd, 0x30f3, 0x30af, 0x30ea, 0x30fc, 0x30af, 0x30c7, 0x30f3,
0x30d0, 0x30fc, 0x30c7, 0x30c8, 0x30ed, 0x30a4, 0x30c8, 0x30c9, 0x30df, 0x30cb, 0x30ab, 0x30a8,
0x30c9, 0x30e2, 0x30f3, 0x30c8, 0x30f3, 0x30a8, 0x30a4, 0x30eb, 0x30cd, 0x30da, 0x30a8, 0x30eb,
0x30b5, 0x30eb, 0x30d0, 0x30c9, 0x30eb, 0x30d5, 0x30a9, 0x30fc, 0x30c8, 0x30cd, 0x30eb, 0x30bd,
0x30f3, 0x30d5, 0x30a9, 0x30eb, 0x30bf, 0x30ec, 0x30b6, 0x30b0, 0x30ec, 0x30fc, 0x30b9, 0x30d9,
0x30a4, 0x30b0, 0x30fc, 0x30b9, 0x30d9, 0x30a4, 0x30b0, 0x30e9, 0x30f3, 0x30c9, 0x30bf, 0x30fc,
0x30af, 0x30b0, 0x30ec, 0x30ca, 0x30c0, 0x30b0, 0x30a2, 0x30c9, 0x30eb, 0x30fc, 0x30d7, 0x30b0,
0x30a2, 0x30c6, 0x30de, 0x30e9, 0x30b0, 0x30a2, 0x30e4, 0x30ad, 0x30eb, 0x30ac, 0x30a4, 0x30a2,
0x30ca, 0x30cf, 0x30ea, 0x30d5, 0x30a1, 0x30af, 0x30b9, 0x30cf, 0x30d0, 0x30ca, 0x30a8, 0x30eb,
0x30e2, 0x30b7, 0x30e8, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x30dd, 0x30ea, 0x30b9,
0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x5dde, 0x30ce, 0x30c3, 0x30af, 0x30b9, 0x30a4,
0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x5dde, 0x30de, 0x30ec, 0x30f3, 0x30b4, 0x30a4, 0x30f3,
0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x5dde, 0x30d4, 0x30fc, 0x30bf, 0x30fc, 0x30ba, 0x30d0, 0x30fc,
0x30b0, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x5dde, 0x30c6, 0x30eb, 0x30b7, 0x30c6,
0x30a3, 0x30a4, 0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x5dde, 0x30d3, 0x30d9, 0x30fc, 0x30a4,
0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x5dde, 0x30d3, 0x30f3, 0x30bb, 0x30f3, 0x30b9, 0x30a4,
0x30f3, 0x30c7, 0x30a3, 0x30a2, 0x30ca, 0x5dde, 0x30a6, 0x30a3, 0x30ca, 0x30de, 0x30c3, 0x30af,
0x30a4, 0x30cc, 0x30f4, 0x30a3, 0x30af, 0x30a4, 0x30ab, 0x30eb, 0x30a4, 0x30c3, 0x30c8, 0x30b8,
0x30e3, 0x30de, 0x30a4, 0x30ab, 0x30b8, 0x30e5, 0x30ce, 0x30fc, 0x30eb, 0x30a4, 0x30d3, 0x30eb,
0x30b1, 0x30f3, 0x30bf, 0x30c3, 0x30ad, 0x30fc, 0x5dde, 0x30e2, 0x30f3, 0x30c6, 0x30a3, 0x30c1,
0x30a7, 0x30ed, 0x30af, 0x30e9, 0x30ec, 0x30f3, 0x30c0, 0x30a4, 0x30af, 0x30e9, 0x30d1, 0x30b9,
0x30ea, 0x30de, 0x30ed, 0x30b5, 0x30f3, 0x30bc, 0x30eb, 0x30b9, 0x30ed, 0x30fc, 0x30ef, 0x30fc,
0x30fb, 0x30d7, 0x30ea, 0x30f3, 0x30bb, 0x30ba, 0x30fb, 0x30af, 0x30a6, 0x30a9, 0x30fc, 0x30bf,
0x30fc, 0x30de, 0x30bb, 0x30a4, 0x30aa, 0x30de, 0x30ca, 0x30b0, 0x30a2, 0x30de, 0x30ca, 0x30a6,
0x30b9, 0x30de, 0x30ea, 0x30b4, 0x30de, 0x30eb, 0x30c6, 0x30a3, 0x30cb, 0x30fc, 0x30af, 0x30de,
0x30bf, 0x30e2, 0x30ed, 0x30b9, 0x30de, 0x30b5, 0x30c8, 0x30e9, 0x30f3, 0x30e1, 0x30ce, 0x30df,
0x30cb, 0x30fc, 0x30e1, 0x30ea, 0x30c0, 0x30e1, 0x30c8, 0x30e9, 0x30ab, 0x30c8, 0x30e9, 0x30e1,
0x30ad, 0x30b7, 0x30b3, 0x30b7, 0x30c6, 0x30a3, 0x30fc, 0x30df, 0x30af, 0x30ed, 0x30f3, 0x5cf6,
0x30e2, 0x30f3, 0x30af, 0x30c8, 0x30f3, 0x30e2, 0x30f3, 0x30c6, 0x30ec, 0x30a4, 0x30e2, 0x30f3,
0x30c6, 0x30d3, 0x30c7, 0x30aa, 0x30e2, 0x30f3, 0x30c8, 0x30bb, 0x30e9, 0x30c8, 0x30ca, 0x30c3,
0x30bd, 0x30fc, 0x30cb, 0x30e5, 0x30fc, 0x30e8, 0x30fc, 0x30af, 0x30ce, 0x30fc, 0x30e0, 0x30ce,
0x30ed, 0x30fc, 0x30cb, 0x30e3, 0x30ce, 0x30fc, 0x30b9, 0x30c0, 0x30b3, 0x30bf, 0x5dde, 0x30d3,
0x30e5, 0x30fc, 0x30e9, 0x30fc, 0x30ce, 0x30fc, 0x30b9, 0x30c0, 0x30b3, 0x30bf, 0x5dde, 0x30bb,
0x30f3, 0x30bf, 0x30fc, 0x30ce, 0x30fc, 0x30b9, 0x30c0, 0x30b3, 0x30bf, 0x5dde, 0x30cb, 0x30e5,
0x30fc, 0x30bb, 0x30fc, 0x30e9, 0x30e0, 0x30cc, 0x30fc, 0x30af, 0x30aa, 0x30d2, 0x30ca, 0x30ac,
0x30d1, 0x30ca, 0x30de, 0x30d1, 0x30e9, 0x30de, 0x30ea, 0x30dc, 0x30d5, 0x30a7, 0x30cb, 0x30c3,
0x30af, 0x30b9, 0x30dd, 0x30eb, 0x30c8, 0x30fc, 0x30d7, 0x30e9, 0x30f3, 0x30b9, 0x30dd, 0x30fc,
0x30c8, 0x30aa, 0x30d6, 0x30b9, 0x30da, 0x30a4, 0x30f3, 0x30dd, 0x30eb, 0x30c8, 0x30d9, 0x30fc,
0x30ea, 0x30e7, 0x30d7, 0x30a8, 0x30eb, 0x30c8, 0x30ea, 0x30b3, 0x30d7, 0x30f3, 0x30bf, 0x30a2,
0x30ec, 0x30ca, 0x30b9, 0x30e9, 0x30f3, 0x30ad, 0x30f3, 0x30a4, 0x30f3, 0x30ec, 0x30c3, 0x30c8,
0x30ec, 0x30b7, 0x30d5, 0x30a7, 0x30ec, 0x30b8, 0x30e3, 0x30a4, 0x30ca, 0x30ec, 0x30be, 0x30ea,
0x30e5, 0x30fc, 0x30c8, 0x30ea, 0x30aa, 0x30d6, 0x30e9, 0x30f3, 0x30b3, 0x30b5, 0x30f3, 0x30bf,
0x30ec, 0x30e0, 0x30b5, 0x30f3, 0x30c1, 0x30a2, 0x30b4, 0x30b5, 0x30f3, 0x30c8, 0x30c9, 0x30df,
0x30f3, 0x30b4, 0x30b5, 0x30f3, 0x30d1, 0x30a6, 0x30ed, 0x30a4, 0x30c8, 0x30b3, 0x30eb, 0x30c8,
0x30eb, 0x30df, 0x30c3, 0x30c8, 0x30b7, 0x30c8, 0x30ab, 0x30b5, 0x30f3, 0x30fb, 0x30d0, 0x30eb,
0x30c6, 0x30eb, 0x30df, 0x30fc, 0x30bb, 0x30f3, 0x30c8, 0x30b8, 0x30e7, 0x30f3, 0x30ba, 0x30bb,
0x30f3, 0x30c8, 0x30af, 0x30ea, 0x30b9, 0x30c8, 0x30d5, 0x30a1, 0x30fc, 0x30bb, 0x30f3, 0x30c8,
0x30eb, 0x30b7, 0x30a2, 0x30bb, 0x30f3, 0x30c8, 0x30c8, 0x30fc, 0x30de, 0x30b9, 0x30bb, 0x30f3,
0x30c8, 0x30d3, 0x30f3, 0x30bb, 0x30f3, 0x30c8, 0x30b9, 0x30a6, 0x30a3, 0x30d5, 0x30c8, 0x30ab,
0x30ec, 0x30f3, 0x30c8, 0x30c6, 0x30b0, 0x30b7, 0x30ac, 0x30eb, 0x30d1, 0x30c1, 0x30e5, 0x30fc,
0x30ec, 0x30c6, 0x30a3, 0x30d5, 0x30a2, 0x30ca, 0x30c8, 0x30ed, 0x30f3, 0x30c8, 0x30c8, 0x30eb,
0x30c8, 0x30fc, 0x30e9, 0x30d0, 0x30f3, 0x30af, 0x30fc, 0x30d0, 0x30fc, 0x30db, 0x30ef, 0x30a4,
0x30c8, 0x30db, 0x30fc, 0x30b9, 0x30a6, 0x30a3, 0x30cb, 0x30da, 0x30b0, 0x30e4, 0x30af, 0x30bf,
0x30c3, 0x30c8, 0x30b1, 0x30fc, 0x30b7, 0x30fc, 0x57fa, 0x5730, 0x30c7, 0x30fc, 0x30d3, 0x30b9,
0x57fa, 0x5730, 0x30c7, 0x30e5, 0x30e2, 0x30f3, 0x30fb, 0x30c7, 0x30e5, 0x30eb, 0x30f4, 0x30a3,
0x30eb, 0x57fa, 0x5730, 0x30de, 0x30c3, 0x30b3, 0x30ea, 0x30fc, 0x30e2, 0x30fc, 0x30bd, 0x30f3,
0x57fa, 0x5730, 0x30de, 0x30af, 0x30de, 0x30fc, 0x30c9, 0x57fa, 0x5730, 0x30d1, 0x30fc, 0x30de,
0x30fc, 0x57fa, 0x5730, 0x30ed, 0x30bc, 0x30e9, 0x57fa, 0x5730, 0x30c8, 0x30ed, 0x30eb, 0x57fa,
0x5730, 0x30dc, 0x30b9, 0x30c8, 0x30fc, 0x30af, 0x57fa, 0x5730, 0x30ed, 0x30f3, 0x30b0, 0x30a4,
0x30a7, 0x30fc, 0x30eb, 0x30d3, 0x30fc, 0x30f3, 0x30a2, 0x30c7, 0x30f3, 0x30a2, 0x30eb, 0x30de,
0x30c8, 0x30a4, 0x30a2, 0x30f3, 0x30de, 0x30f3, 0x30a2, 0x30ca, 0x30c7, 0x30a3, 0x30ea, 0x30a2,
0x30af, 0x30bf, 0x30a6, 0x30a2, 0x30af, 0x30c8, 0x30d9, 0x30a2, 0x30b7, 0x30ac, 0x30d0, 0x30fc,
0x30c9, 0x30a2, 0x30c6, 0x30a3, 0x30e9, 0x30a6, 0x30d0, 0x30b0, 0x30c0, 0x30c3, 0x30c9, 0x30d0,
0x30fc, 0x30ec, 0x30fc, 0x30f3, 0x30d0, 0x30af, 0x30fc, 0x30d0, 0x30f3, 0x30b3, 0x30af, 0x30d0,
0x30eb, 0x30ca, 0x30a6, 0x30eb, 0x30d9, 0x30a4, 0x30eb, 0x30fc, 0x30c8, 0x30d3, 0x30b7, 0x30e5,
0x30b1, 0x30af, 0x30d6, 0x30eb, 0x30cd, 0x30a4, 0x30c1, 0x30bf, 0x30c1, 0x30e7, 0x30a4, 0x30d0,
0x30eb, 0x30b5, 0x30f3, 0x30b3, 0x30ed, 0x30f3, 0x30dc, 0x30c0, 0x30de, 0x30b9, 0x30ab, 0x30b9,
0x30c0, 0x30c3, 0x30ab, 0x30c9, 0x30d0, 0x30a4, 0x30c9, 0x30a5, 0x30b7, 0x30e3, 0x30f3, 0x30d9,
0x30d5, 0x30a1, 0x30de, 0x30b0, 0x30b9, 0x30bf, 0x30ac, 0x30b6, 0x30d8, 0x30d6, 0x30ed, 0x30f3,
0x30db, 0x30fc, 0x30c1, 0x30df, 0x30f3, 0x30db, 0x30d6, 0x30c9, 0x30a4, 0x30eb, 0x30af, 0x30fc,
0x30c4, 0x30af, 0x30b8, 0x30e3, 0x30ab, 0x30eb, 0x30bf, 0x30b8, 0x30e3, 0x30e4, 0x30d7, 0x30e9,
0x30a8, 0x30eb, 0x30b5, 0x30ec, 0x30e0, 0x30ab, 0x30d6, 0x30fc, 0x30eb, 0x30ab, 0x30e0, 0x30c1,
0x30e3, 0x30c3, 0x30ab, 0x30ab, 0x30e9, 0x30c1, 0x30ab, 0x30c8, 0x30de, 0x30f3, 0x30ba, 0x30cf,
0x30f3, 0x30c9, 0x30a5, 0x30a4, 0x30ac, 0x30b3, 0x30eb, 0x30ab, 0x30bf, 0x30af, 0x30e9, 0x30b9,
0x30ce, 0x30e4, 0x30eb, 0x30b9, 0x30af, 0x30af, 0x30a2, 0x30e9, 0x30eb, 0x30f3, 0x30d7, 0x30fc,
0x30eb, 0x30af, 0x30c1, 0x30f3, 0x30af, 0x30a6, 0x30a7, 0x30fc, 0x30c8, 0x30de, 0x30ab, 0x30aa,
0x30de, 0x30ac, 0x30c0, 0x30f3, 0x30de, 0x30ab, 0x30c3, 0x30b5, 0x30eb, 0x30de, 0x30cb, 0x30e9,
0x30de, 0x30b9, 0x30ab, 0x30c3, 0x30c8, 0x30cb, 0x30b3, 0x30b7, 0x30a2, 0x30ce, 0x30f4, 0x30a9,
0x30af, 0x30ba, 0x30cd, 0x30c4, 0x30af, 0x30ce, 0x30f4, 0x30a9, 0x30b7, 0x30d3, 0x30eb, 0x30b9,
0x30af, 0x30aa, 0x30e0, 0x30b9, 0x30af, 0x30aa, 0x30e9, 0x30eb, 0x30d7, 0x30ce, 0x30f3, 0x30da,
0x30f3, 0x30dd, 0x30f3, 0x30c6, 0x30a3, 0x30a2, 0x30ca, 0x30c3, 0x30af, 0x5e73, 0x58cc, 0x30ab,
0x30bf, 0x30fc, 0x30eb, 0x30b3, 0x30b9, 0x30bf, 0x30ca, 0x30a4, 0x30af, 0x30ba, 0x30ed, 0x30eb,
0x30c0, 0x30ea, 0x30e4, 0x30c9, 0x30b5, 0x30cf, 0x30ea, 0x30f3, 0x30b5, 0x30de, 0x30eb, 0x30ab,
0x30f3, 0x30c9, 0x30bd, 0x30a6, 0x30eb, 0x30b7, 0x30f3, 0x30ac, 0x30dd, 0x30fc, 0x30eb, 0x30b9,
0x30ec, 0x30c9, 0x30cd, 0x30b3, 0x30ea, 0x30e0, 0x30b9, 0x30af, 0x30bf, 0x30b7, 0x30b1, 0x30f3,
0x30c8, 0x30c8, 0x30d3, 0x30ea, 0x30b7, 0x30c6, 0x30d8, 0x30e9, 0x30f3, 0x30c6, 0x30a3, 0x30f3,
0x30d7, 0x30fc, 0x30c8, 0x30e0, 0x30b9, 0x30af, 0x30a6, 0x30e9, 0x30f3, 0x30d0, 0x30fc, 0x30c8,
0x30eb, 0x30a6, 0x30eb, 0x30e0, 0x30c1, 0x30a6, 0x30b9, 0x30c1, 0x30cd, 0x30e9, 0x30d3, 0x30a8,
0x30f3, 0x30c1, 0x30e3, 0x30f3, 0x30a6, 0x30e9, 0x30b8, 0x30aa, 0x30b9, 0x30c8, 0x30af, 0x30e4,
0x30af, 0x30fc, 0x30c4, 0x30af, 0x30e4, 0x30f3, 0x30b4, 0x30f3, 0x30a8, 0x30ab, 0x30c6, 0x30ea,
0x30f3, 0x30d6, 0x30eb, 0x30b0, 0x30a8, 0x30ec, 0x30d0, 0x30f3, 0x30a2, 0x30be, 0x30ec, 0x30b9,
0x30d0, 0x30df, 0x30e5, 0x30fc, 0x30c0, 0x30ab, 0x30ca, 0x30ea, 0x30a2, 0x30ab, 0x30fc, 0x30dc,
0x30d9, 0x30eb, 0x30c7, 0x30d5, 0x30a7, 0x30ed, 0x30fc, 0x30de, 0x30c7, 0x30a4, 0x30e9, 0x30ec,
0x30a4, 0x30ad, 0x30e3, 0x30d3, 0x30af, 0x30b5, 0x30a6, 0x30b9, 0x30b8, 0x30e7, 0x30fc, 0x30b8,
0x30a2, 0x30bb, 0x30f3, 0x30c8, 0x30d8, 0x30ec, 0x30ca, 0x30b9, 0x30bf, 0x30f3, 0x30ec, 0x30fc,
0x30a2, 0x30c7, 0x30ec, 0x30fc, 0x30c9, 0x30d6, 0x30ea, 0x30b9, 0x30d9, 0x30f3, 0x30d6, 0x30ed,
0x30fc, 0x30af, 0x30f3, 0x30d2, 0x30eb, 0x30c0, 0x30fc, 0x30a6, 0x30a3, 0x30f3, 0x30e6, 0x30fc,
0x30af, 0x30e9, 0x30db, 0x30d0, 0x30fc, 0x30c8, 0x30ea, 0x30f3, 0x30c7, 0x30de, 0x30f3, 0x30ed,
0x30fc, 0x30c9, 0x30cf, 0x30a6, 0x30e1, 0x30eb, 0x30dc, 0x30eb, 0x30f3, 0x30d1, 0x30fc, 0x30b9,
0x30b7, 0x30c9, 0x30cb, 0x30fc, 0x30a2, 0x30e0, 0x30b9, 0x30c6, 0x30eb, 0x30c0, 0x30e0, 0x30a2,
0x30f3, 0x30c9, 0x30e9, 0x30a2, 0x30b9, 0x30c8, 0x30e9, 0x30cf, 0x30f3, 0x30a2, 0x30c6, 0x30cd,
0x30d9, 0x30aa, 0x30b0, 0x30e9, 0x30fc, 0x30c9, 0x30d9, 0x30eb, 0x30ea, 0x30f3, 0x30d6, 0x30e9,
0x30c1, 0x30b9, 0x30e9, 0x30d0, 0x30d6, 0x30ea, 0x30e5, 0x30c3, 0x30bb, 0x30eb, 0x30d6, 0x30ab,
0x30ec, 0x30b9, 0x30c8, 0x30d6, 0x30c0, 0x30da, 0x30b9, 0x30c8, 0x30d3, 0x30e5, 0x30fc, 0x30b8,
0x30f3, 0x30b2, 0x30f3, 0x30ad, 0x30b7, 0x30ca, 0x30a6, 0x30b3, 0x30da, 0x30f3, 0x30cf, 0x30fc,
0x30b2, 0x30f3, 0x30c0, 0x30d6, 0x30ea, 0x30f3, 0x30b8, 0x30d6, 0x30e9, 0x30eb, 0x30bf, 0x30eb,
0x30ac, 0x30fc, 0x30f3, 0x30b8, 0x30fc, 0x30d8, 0x30eb, 0x30b7, 0x30f3, 0x30ad, 0x30de, 0x30f3,
0x5cf6, 0x30a4, 0x30b9, 0x30bf, 0x30f3, 0x30d6, 0x30fc, 0x30eb, 0x30b8, 0x30e3, 0x30fc, 0x30b8,
0x30fc, 0x30ab, 0x30ea, 0x30fc, 0x30cb, 0x30f3, 0x30b0, 0x30e9, 0x30fc, 0x30c9, 0x30ad, 0x30fc,
0x30ed, 0x30d5, 0x30ad, 0x30fc, 0x30a6, 0x30ea, 0x30b9, 0x30dc, 0x30f3, 0x30ea, 0x30e5, 0x30d6,
0x30ea, 0x30e3, 0x30ca, 0x30ed, 0x30f3, 0x30c9, 0x30f3, 0x30eb, 0x30af, 0x30bb, 0x30f3, 0x30d6,
0x30eb, 0x30af, 0x30de, 0x30c9, 0x30ea, 0x30fc, 0x30c9, 0x30de, 0x30eb, 0x30bf, 0x30de, 0x30ea,
0x30a8, 0x30cf, 0x30e0, 0x30f3, 0x30df, 0x30f3, 0x30b9, 0x30af, 0x30e2, 0x30ca, 0x30b3, 0x30e2,
0x30b9, 0x30af, 0x30ef, 0x30aa, 0x30b9, 0x30ed, 0x30d1, 0x30ea, 0x30dd, 0x30c9, 0x30b4, 0x30ea,
0x30c4, 0x30a1, 0x30d7, 0x30e9, 0x30cf, 0x30ea, 0x30ac, 0x30b5, 0x30de, 0x30e9, 0x30b5, 0x30f3,
0x30de, 0x30ea, 0x30ce, 0x30b5, 0x30e9, 0x30a8, 0x30dc, 0x30b5, 0x30e9, 0x30c8, 0x30d5, 0x30b7,
0x30f3, 0x30d5, 0x30a7, 0x30ed, 0x30dd, 0x30ea, 0x30b9, 0x30b3, 0x30d4, 0x30a8, 0x30bd, 0x30d5,
0x30a3, 0x30a2, 0x30b9, 0x30c8, 0x30c3, 0x30af, 0x30db, 0x30eb, 0x30e0, 0x30bf, 0x30ea, 0x30f3,
0x30c6, 0x30a3, 0x30e9, 0x30ca, 0x30a6, 0x30ea, 0x30e4, 0x30ce, 0x30d5, 0x30b9, 0x30af, 0x30d5,
0x30a1, 0x30c9, 0x30a5, 0x30fc, 0x30c4, 0x30d0, 0x30c1, 0x30ab, 0x30f3, 0x30a6, 0x30a3, 0x30fc,
0x30f3, 0x30f4, 0x30a3, 0x30ea, 0x30cb, 0x30e5, 0x30b9, 0x30dc, 0x30eb, 0x30b4, 0x30b0, 0x30e9,
0x30fc, 0x30c9, 0x30ef, 0x30eb, 0x30b7, 0x30e3, 0x30ef, 0x30b6, 0x30b0, 0x30ec, 0x30d6, 0x30c1,
0x30e5, 0x30fc, 0x30ea, 0x30c3, 0x30d2, 0x30a2, 0x30f3, 0x30bf, 0x30ca, 0x30ca, 0x30ea, 0x30dc,
0x30c1, 0x30e3, 0x30b4, 0x30b9, 0x30af, 0x30ea, 0x30b9, 0x30de, 0x30b9, 0x5cf6, 0x30b3, 0x30b3,
0x30b9, 0x8af8, 0x5cf6, 0x30b3, 0x30e2, 0x30ed, 0x30b1, 0x30eb, 0x30b2, 0x30ec, 0x30f3, 0x8af8,
0x5cf6, 0x30de, 0x30d8, 0x30e2, 0x30eb, 0x30c7, 0x30a3, 0x30d6, 0x30e2, 0x30fc, 0x30ea, 0x30b7,
0x30e3, 0x30b9, 0x30de, 0x30e8, 0x30c3, 0x30c8, 0x30ec, 0x30e6, 0x30cb, 0x30aa, 0x30f3, 0x30a2,
0x30d4, 0x30a2, 0x30aa, 0x30fc, 0x30af, 0x30e9, 0x30f3, 0x30c9, 0x30d6, 0x30fc, 0x30b2, 0x30f3,
0x30d3, 0x30eb, 0x30c1, 0x30e3, 0x30bf, 0x30e0, 0x30c1, 0x30e5, 0x30fc, 0x30af, 0x30a4, 0x30fc,
0x30b9, 0x30bf, 0x30fc, 0x5cf6, 0x30a8, 0x30d5, 0x30a7, 0x30c6, 0x5cf6, 0x30d5, 0x30a1, 0x30ab,
0x30aa, 0x30d5, 0x30a9, 0x30d5, 0x30a3, 0x30b8, 0x30fc, 0x30d5, 0x30ca, 0x30d5, 0x30c6, 0x30a3,
0x30ac, 0x30e9, 0x30d1, 0x30b4, 0x30b9, 0x30ac, 0x30f3, 0x30d3, 0x30a8, 0x8af8, 0x5cf6, 0x30ac,
0x30c0, 0x30eb, 0x30ab, 0x30ca, 0x30eb, 0x30b0, 0x30a2, 0x30e0, 0x30db, 0x30ce, 0x30eb, 0x30eb,
0x30a8, 0x30f3, 0x30c0, 0x30fc, 0x30d9, 0x30ea, 0x30fc, 0x5cf6, 0x30ad, 0x30ea, 0x30b9, 0x30a3,
0x30de, 0x30b9, 0x30a3, 0x5cf6, 0x30b3, 0x30b9, 0x30e9, 0x30a8, 0x30af, 0x30a7, 0x30bc, 0x30ea,
0x30f3, 0x30de, 0x30b8, 0x30e5, 0x30ed, 0x30de, 0x30eb, 0x30ad, 0x30fc, 0x30ba, 0x30df, 0x30c3,
0x30c9, 0x30a6, 0x30a7, 0x30fc, 0x5cf6, 0x30cb, 0x30a6, 0x30a8, 0x30ce, 0x30fc, 0x30d5, 0x30a9,
0x30fc, 0x30af, 0x5cf6, 0x30cc, 0x30e1, 0x30a2, 0x30d1, 0x30b4, 0x30d1, 0x30b4, 0x30d1, 0x30e9,
0x30aa, 0x30d4, 0x30c8, 0x30b1, 0x30a2, 0x30f3, 0x8af8, 0x5cf6, 0x30dd, 0x30f3, 0x30da, 0x30a4,
0x5cf6, 0x30dd, 0x30fc, 0x30c8, 0x30e2, 0x30ec, 0x30b9, 0x30d3, 0x30fc, 0x30e9, 0x30ed, 0x30c8,
0x30f3, 0x30ac, 0x30b5, 0x30a4, 0x30d1, 0x30f3, 0x30bf, 0x30d2, 0x30c1, 0x30bf, 0x30e9, 0x30ef,
0x30c8, 0x30f3, 0x30ac, 0x30bf, 0x30d7, 0x30a6, 0x30a7, 0x30fc, 0x30af, 0x5cf6, 0x30a6, 0x30a9,
0x30ea, 0x30b9, 0x8af8, 0x5cf6, 0x4d, 0x65, 0x6e, 0x64, 0x6f, 0x73, 0x61, 0x42,
0x65, 0x6c, 0x69, 0x73, 0x65, 0x54, 0x65, 0x6c, 0x75, 0x6b, 0x20, 0x43,
0x61, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x67, 0x65, 0x4b, 0x61, 0x6d, 0x70,
0x6f, 0x20, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x65, 0x4b, 0x61, 0x79, 0x65,
0x6e, 0x6e, 0x65, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x4a, 0x75,
0xe1, 0x72, 0x65, 0x73, 0x4b, 0x75, 0x69, 0x61, 0x62, 0x61, 0x42, 0x65,
0x6e, 0x74, 0x65, 0x6e, 0x67, 0x20, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e,
0x54, 0x65, 0x6c, 0x75, 0x6b, 0x20, 0x47, 0x6c, 0x61, 0x63, 0x65, 0x54,
0x65, 0x6c, 0x75, 0x6b, 0x20, 0x47, 0x6f, 0x6f, 0x73, 0x65, 0x4b, 0x75,
0x74, 0x68, 0x6f, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x42,
0x65, 0x75, 0x6c, 0x61, 0x68, 0x20, 0x5b, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x20, 0x4c, 0x6f, 0x72, 0x5d, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68,
0x20, 0x5b, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4c, 0x6f, 0x72,
0x5d, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72,
0x20, 0x5b, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4c, 0x6f, 0x72,
0x5d, 0x50, 0x61, 0x6c, 0x61, 0x62, 0x75, 0x68, 0x61, 0x6e, 0x20, 0x53,
0x70, 0x61, 0x6e, 0x79, 0x6f, 0x6c, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20,
0x42, 0x61, 0x72, 0x74, 0x68, 0x65, 0x6c, 0x65, 0x6d, 0x79, 0x53, 0x61,
0x6e, 0x74, 0x61, 0x20, 0x4a, 0x6f, 0x68, 0x6e, 0x53, 0x61, 0x6e, 0x74,
0x61, 0x20, 0x4b, 0x69, 0x74, 0x74, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x61,
0x20, 0x54, 0x68, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x61,
0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x41, 0x72, 0x75, 0x73,
0x20, 0x42, 0x61, 0x6e, 0x74, 0x65, 0x72, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x20, 0x4b, 0x69, 0x64, 0x75, 0x6c, 0x50, 0x75, 0x6c, 0x6f,
0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x75,
0x72, 0x6b, 0x4b, 0x68, 0x61, 0x67, 0x6f, 0x73, 0x50, 0x61, 0x73, 0x6b,
0x61, 0x68, 0x50, 0x65, 0x6c, 0x61, 0x62, 0x75, 0x68, 0x61, 0x6e, 0x20,
0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x79, 0x41, 0x64, 0x61, 0x6b, 0x69,
0x41, 0x6e, 0x6b, 0x6f, 0x72, 0x61, 0x6a, 0x69, 0x41, 0x6e, 0x67, 0x69,
0x6c, 0x61, 0x42, 0x61, 0xed, 0x61, 0x20, 0x64, 0x69, 0x20, 0x42, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x42, 0x61, 0x72, 0x62, 0x61, 0x64,
0x75, 0x73, 0x42, 0x65, 0x6c, 0x69, 0x7a, 0x69, 0x58, 0x69, 0x6b, 0x61,
0x67, 0x75, 0x58, 0x69, 0x75, 0xe1, 0x75, 0x61, 0x4b, 0x72, 0xe9, 0x73,
0x74, 0x6f, 0x6e, 0x4b, 0x75, 0x72, 0x61, 0x73, 0x61, 0x75, 0x47, 0x75,
0x61, 0x64, 0x61, 0x6c, 0x75, 0x70, 0x69, 0x47, 0x75, 0x61, 0x74, 0x69,
0x6d, 0x61, 0x6c, 0x61, 0x4d, 0x61, 0x6e, 0xe1, 0x67, 0x75, 0x61, 0x53,
0x69, 0x64, 0x61, 0x64, 0x69, 0x20, 0x64, 0x69, 0x20, 0x4d, 0xe9, 0x78,
0x69, 0x6b, 0x75, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x49, 0x6f, 0x72, 0x6b,
0x69, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x69,
0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x69, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x74,
0x69, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x20, 0x64, 0x69, 0x20, 0x53, 0x70,
0x61, 0x6e, 0x68, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x20, 0x52, 0x69,
0x6b, 0x75, 0x53, 0x61, 0x6e, 0x74, 0x75, 0x20, 0x44, 0x75, 0x6d, 0x69,
0x6e, 0x67, 0x75, 0x53, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x72, 0x74, 0x6f,
0x6c, 0x6f, 0x6d, 0x65, 0x75, 0x53, 0x61, 0x6e, 0x20, 0x4b, 0x72, 0x69,
0x73, 0x74, 0x6f, 0x76, 0x61, 0x6e, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20,
0x4c, 0xfa, 0x73, 0x69, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x42, 0x69, 0x73,
0x65, 0x6e, 0x74, 0x69, 0x54, 0x6f, 0x72, 0x6f, 0x6e, 0x74, 0x75, 0x55,
0xed, 0x6e, 0x69, 0x70, 0x65, 0x67, 0x41, 0x62, 0x69, 0x1e7, 0x61, 0x6e,
0x41, 0x6b, 0x1e5b, 0x61, 0x41, 0x64, 0x69, 0x73, 0x20, 0x41, 0x62, 0x61,
0x62, 0x61, 0x4c, 0x65, 0x7a, 0x7a, 0x61, 0x79, 0x65, 0x72, 0x42, 0x61,
0x6d, 0x61, 0x6b, 0x75, 0x42, 0x69, 0x73, 0x61, 0x77, 0x42, 0x72, 0x61,
0x1e93, 0x61, 0x76, 0x69, 0x6c, 0x4c, 0x71, 0x61, 0x68, 0x69, 0x72, 0x61,
0x4b, 0x61, 0x1e93, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0x61, 0x53, 0x65,
0x62, 0x74, 0x61, 0x4b, 0x75, 0x6e, 0x61, 0x6b, 0x72, 0x69, 0x44, 0x61,
0x72, 0x20, 0x53, 0x61, 0x6c, 0x61, 0x6d, 0x1e6, 0x69, 0x62, 0x75, 0x74,
0x69, 0x44, 0x77, 0x61, 0x6c, 0x61, 0x4c, 0x65, 0x25b, 0x79, 0x75, 0x6e,
0x46, 0x72, 0x69, 0x1e6d, 0x61, 0x77, 0x6e, 0x47, 0x61, 0x62, 0x75, 0x72,
0x75, 0x6e, 0x48, 0x61, 0x72, 0x61, 0x72, 0x69, 0x4a, 0x75, 0x68, 0x61,
0x6e, 0x69, 0x73, 0x62, 0x75, 0x72, 0x67, 0x58, 0x61, 0x1e5b, 0x74, 0x75,
0x6d, 0x4b, 0x69, 0x6e, 0x63, 0x61, 0x1e63, 0x61, 0x4c, 0x61, 0x67, 0x75,
0x73, 0x4c, 0x69, 0x62, 0x65, 0x72, 0x76, 0x69, 0x6c, 0x4c, 0x75, 0x6d,
0x69, 0x4c, 0x77, 0x61, 0x6e, 0x64, 0x61, 0x4c, 0x75, 0x62, 0x75, 0x6d,
0x62, 0x61, 0x63, 0x69, 0x4c, 0x75, 0x7a, 0x61, 0x6b, 0x61, 0x4d, 0x61,
0x6c, 0x61, 0x62, 0x75, 0x4d, 0x61, 0x70, 0x75, 0x1e6d, 0x75, 0x4d, 0x61,
0x73, 0x69, 0x72, 0x75, 0x4d, 0x75, 0x71, 0x61, 0x64, 0x69, 0x63, 0x75,
0x4d, 0x75, 0x6e, 0x1e5b, 0x75, 0x76, 0x79, 0x61, 0x4e, 0x61, 0x79, 0x72,
0x75, 0x62, 0x69, 0x4e, 0x1e7, 0x61, 0x6d, 0x69, 0x6e, 0x61, 0x4e, 0x79,
0x61, 0x6d, 0x65, 0x79, 0x4e, 0x77, 0x61, 0x71, 0x63, 0x75, 0x1e6d, 0x57,
0x61, 0x67, 0x61, 0x64, 0x75, 0x67, 0x75, 0x50, 0x75, 0x1e5b, 0x74, 0x75,
0x20, 0x4e, 0x75, 0x76, 0x75, 0x53, 0x61, 0x77, 0x20, 0x1e6c, 0x75, 0x6d,
0x69, 0x54, 0x72, 0x61, 0x62, 0x6c, 0x75, 0x73, 0x57, 0x69, 0x6e, 0x64,
0x68, 0x65, 0x6b, 0x41, 0x6e, 0x6b, 0x75, 0x72, 0x61, 0x6a, 0x41, 0x6e,
0x74, 0x69, 0x67, 0x61, 0x4b, 0x75, 0x72, 0x64, 0x75, 0x62, 0x61, 0x4d,
0x69, 0x6e, 0x64, 0x75, 0x1e93, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x58, 0x77,
0x61, 0x6e, 0x53, 0x61, 0x6e, 0x20, 0x4c, 0x77, 0x69, 0x73, 0x55, 0x63,
0x77, 0x61, 0x79, 0x61, 0x41, 0x73, 0x75, 0x6e, 0x73, 0x79, 0x75, 0x6e,
0x41, 0x74, 0x69, 0x6b, 0x75, 0x6b, 0x61, 0x6e, 0x42, 0x61, 0x68, 0x79,
0x61, 0x42, 0x61, 0x68, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x69,
0x72, 0x61, 0x73, 0x42, 0x69, 0x6c, 0x69, 0x7a, 0x42, 0x6c, 0x75, 0x6e,
0x2d, 0x53, 0x61, 0x62, 0x6c, 0x75, 0x6e, 0x42, 0x75, 0x67, 0x75, 0x1e6d,
0x61, 0x42, 0x77, 0x61, 0x1e93, 0x4b, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x1e7,
0x20, 0x42, 0x61, 0x79, 0x4b, 0x75, 0x6d, 0x70, 0x75, 0x20, 0x47, 0x72,
0x61, 0x6e, 0x64, 0x69, 0x4b, 0x61, 0x79, 0x61, 0x6e, 0x43, 0x69, 0x6b,
0x61, 0x67, 0x75, 0x43, 0x69, 0x77, 0x61, 0x77, 0x61, 0x4b, 0x75, 0x73,
0x74, 0x61, 0x20, 0x52, 0x69, 0x6b, 0x61, 0x4b, 0x72, 0x69, 0x73, 0x74,
0x75, 0x6e, 0x4b, 0x75, 0x72, 0x61, 0x10d, 0x61, 0x77, 0x44, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x6b, 0x63, 0x61, 0x76, 0x65, 0x6e, 0x44, 0x61, 0x77,
0x73, 0x75, 0x6e, 0x44, 0x61, 0x77, 0x73, 0x75, 0x6e, 0x20, 0x4b, 0x72,
0x69, 0x6b, 0x44, 0x69, 0x74, 0x72, 0x77, 0x61, 0x44, 0x75, 0x6d, 0x69,
0x6e, 0x69, 0x6b, 0x49, 0x64, 0x6d, 0x75, 0x6e, 0x74, 0x75, 0x6e, 0x53,
0x61, 0x6c, 0x76, 0x61, 0x64, 0x75, 0x72, 0x46, 0x75, 0x72, 0x1e6d, 0x61,
0x6c, 0x69, 0x7a, 0x61, 0x47, 0x72, 0x75, 0x6e, 0x61, 0x64, 0x61, 0x47,
0x77, 0x61, 0x64, 0x61, 0x6c, 0x75, 0x70, 0x69, 0x47, 0x77, 0x61, 0x74,
0x69, 0x6d, 0x61, 0x6c, 0x61, 0x47, 0x61, 0x79, 0x61, 0x6b, 0x69, 0x6c,
0x47, 0x75, 0x79, 0x61, 0x6e, 0x4a, 0x61, 0x6d, 0x61, 0x79, 0x6b, 0x61,
0x4d, 0x61, 0x6e, 0x61, 0x67, 0x61, 0x4d, 0x61, 0x72, 0x69, 0x67, 0x75,
0x1e6d, 0x4e, 0x75, 0x1e5b, 0x75, 0x6e, 0x68, 0x61, 0x50, 0x61, 0x72, 0x61,
0x6d, 0x61, 0x72, 0x69, 0x62, 0x75, 0x50, 0x75, 0x72, 0x74, 0x75, 0x20,
0x56, 0x69, 0x6c, 0x68, 0x75, 0x52, 0x69, 0x73, 0x69, 0x66, 0x52, 0x79,
0x75, 0x20, 0x42, 0x72, 0x61, 0x6e, 0x6b, 0x75, 0x53, 0x61, 0x6e, 0x74,
0x79, 0x61, 0x67, 0x75, 0x53, 0x61, 0x77, 0x20, 0x50, 0x61, 0x77, 0x6c,
0x75, 0x4d, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x56, 0x75, 0x73, 0x1e6d, 0x75,
0x6b, 0x4c, 0x75, 0x6e, 0x67, 0x79, 0x65, 0x72, 0x62, 0x79, 0x65, 0x6e,
0x190, 0x61, 0x64, 0x65, 0x6e, 0x190, 0x65, 0x6d, 0x6d, 0x61, 0x6e, 0x42,
0x65, 0x263, 0x64, 0x61, 0x64, 0x42, 0x61, 0x1e25, 0x72, 0x69, 0x6e, 0x42,
0x61, 0x6e, 0x67, 0x6b, 0x75, 0x6b, 0x42, 0x72, 0x75, 0x6e, 0x61, 0x79,
0x4b, 0x77, 0x61, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x4b, 0x75, 0x6c,
0x75, 0x6d, 0x62, 0x75, 0x194, 0x65, 0x7a, 0x7a, 0x61, 0x48, 0x65, 0x62,
0x72, 0x75, 0x6e, 0x48, 0x75, 0x6e, 0x67, 0x20, 0x4b, 0x75, 0x6e, 0x67,
0x48, 0x75, 0x76, 0x64, 0x4a, 0x69, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x65,
0x6d, 0x4b, 0x61, 0x72, 0x61, 0x63, 0x69, 0x4b, 0x75, 0x6c, 0x6b, 0x61,
0x74, 0x61, 0x4b, 0x77, 0x61, 0x6c, 0x61, 0x20, 0x4c, 0x75, 0x6d, 0x70,
0x75, 0x72, 0x4b, 0x77, 0x69, 0x74, 0x4d, 0x61, 0x6b, 0x61, 0x77, 0x4d,
0x75, 0x73, 0x71, 0x61, 0x1e6d, 0x4e, 0x69, 0x6b, 0x75, 0x73, 0x79, 0x61,
0x50, 0x79, 0x75, 0x6e, 0x67, 0x79, 0x75, 0x6e, 0x67, 0x51, 0x61, 0x1e6d,
0x61, 0x72, 0x4b, 0x75, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x79, 0x52, 0x69,
0x79, 0x61, 0x1e0d, 0x53, 0x69, 0x79, 0x75, 0x6c, 0x43, 0x61, 0x6e, 0x67,
0x61, 0x79, 0x53, 0x61, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x54, 0x61,
0x79, 0x70, 0x61, 0x79, 0x54, 0x75, 0x6b, 0x79, 0x75, 0x59, 0x65, 0x72,
0x76, 0x61, 0x6e, 0x41, 0x1e93, 0x75, 0x72, 0x69, 0x73, 0x42, 0x69, 0x72,
0x6d, 0x75, 0x64, 0x54, 0x69, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20,
0x6e, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x4b, 0x61, 0x70, 0x20,
0x56, 0x69, 0x72, 0x46, 0x61, 0x72, 0x77, 0x69, 0x4d, 0x61, 0x64, 0x69,
0x72, 0x61, 0x4a, 0x79, 0x75, 0x72, 0x6a, 0x79, 0x61, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x1e93, 0x75, 0x6c, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x49, 0x6c,
0x69, 0x6e, 0x61, 0x41, 0x64, 0x69, 0x6c, 0x61, 0x79, 0x69, 0x64, 0x4d,
0x61, 0x6c, 0x62, 0x75, 0x72, 0x6e, 0x41, 0x6d, 0x65, 0x73, 0x74, 0x69,
0x72, 0x64, 0x61, 0x6d, 0x41, 0x6e, 0x64, 0x75, 0x72, 0x61, 0x42, 0x69,
0x6c, 0x67, 0x72, 0x61, 0x64, 0x4b, 0x75, 0x70, 0x65, 0x6e, 0x68, 0x61,
0x67, 0x65, 0x6e, 0x44, 0x75, 0x62, 0x6c, 0x61, 0x6e, 0x4a, 0x69, 0x62,
0x72, 0x61, 0x6c, 0x1e6d, 0x61, 0x72, 0x47, 0x69, 0x72, 0x6e, 0x69, 0x7a,
0x65, 0x79, 0x48, 0x69, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x54, 0x69,
0x67, 0x7a, 0x69, 0x72, 0x74, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x6e, 0x49,
0x73, 0x1e6d, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x4a, 0x69, 0x72, 0x7a, 0x65,
0x79, 0x4b, 0x79, 0x65, 0x76, 0x4c, 0x69, 0x7a, 0x62, 0x75, 0x6e, 0x4c,
0x75, 0x6e, 0x64, 0x75, 0x6e, 0x4d, 0x61, 0x72, 0x69, 0x68, 0x61, 0x6d,
0x6e, 0x4d, 0x75, 0x6e, 0x61, 0x6b, 0x75, 0x4d, 0x75, 0x73, 0x6b, 0x75,
0x55, 0x1e63, 0x6c, 0x75, 0x52, 0x75, 0x6d, 0x53, 0x61, 0x6e, 0x20, 0x4d,
0x61, 0x72, 0x69, 0x6e, 0x75, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x69, 0x76,
0x75, 0x53, 0x6f, 0x66, 0x79, 0x61, 0x53, 0x1e6d, 0x75, 0x6b, 0x68, 0x75,
0x6c, 0x6d, 0x56, 0x79, 0x65, 0x6e, 0x6e, 0x61, 0x56, 0x69, 0x6c, 0x6e,
0x79, 0x75, 0x73, 0x5a, 0x75, 0x72, 0x69, 0x78, 0x41, 0x6e, 0x74, 0x61,
0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x76, 0x75, 0x43, 0x61, 0x67, 0x75,
0x73, 0x4b, 0x75, 0x6b, 0x75, 0x4b, 0x75, 0x6d, 0x75, 0x1e5b, 0x4b, 0x65,
0x72, 0x67, 0x6c, 0x65, 0x6e, 0x4d, 0x61, 0x68, 0x69, 0x4d, 0x75, 0x72,
0x69, 0x73, 0x4d, 0x61, 0x79, 0x75, 0x1e6d, 0x52, 0x69, 0x79, 0x75, 0x6e,
0x79, 0x75, 0x6e, 0x41, 0x70, 0x79, 0x61, 0x55, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x43, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x75, 0x73, 0x47, 0x61, 0x6d, 0x62, 0x79, 0x65, 0x72, 0x4e,
0x69, 0x79, 0x75, 0x4e, 0x75, 0x72, 0x66, 0x75, 0x6c, 0x6b, 0xc85, 0xcac,
0xcbf, 0xca6, 0xccd, 0x200c, 0xc9c, 0xca8, 0xccd, 0xc85, 0xc95, 0xccd, 0xcb0, 0xcbe,
0xc86, 0xca1, 0xcbf, 0xcb8, 0xccd, 0x20, 0xc85, 0xcac, 0xcbe, 0xcac, 0xcbe, 0xc85,
0xcb2, 0xccd, 0xc9c, 0xcc0, 0xcb0, 0xccd, 0xcb8, 0xccd, 0xc85, 0xcb8, 0xccd, 0xcae,
0xcbe, 0xcb0, 0xcbe, 0xcac, 0xcae, 0xcbe, 0xc95, 0xcca, 0xcac, 0xcbe, 0xc82, 0xc97,
0xcbf, 0xcac, 0xc82, 0xc9c, 0xcc1, 0xcb2, 0xccd, 0xcac, 0xcbf, 0xcb8, 0xccd, 0xcb8,
0xcbe, 0xcb5, 0xccd, 0xcac, 0xccd, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc9f, 0xcc8,
0xcb0, 0xccd, 0xcac, 0xccd, 0xcb0, 0xcc6, 0xc9c, 0xcbe, 0xcb5, 0xcc0, 0xcb2, 0xccd,
0xcac, 0xcc1, 0xc9c, 0xcc1, 0xc82, 0xcac, 0xcc1, 0xcb0, 0xcbe, 0xc95, 0xcc8, 0xcb0,
0xccb, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xcb8, 0xcbe, 0xcac, 0xccd, 0xcb2, 0xcbe, 0xc82,
0xc95, 0xcbe, 0xcb8, 0xcc6, 0xcaf, 0xcc1, 0xc9f, 0xcbe, 0xc95, 0xcca, 0xca8, 0xcbe,
0xc95, 0xccd, 0xcb0, 0xcbf, 0xca1, 0xcbe, 0xc95, 0xcb0, 0xccd, 0xca1, 0xcbe, 0xcb0,
0xccd, 0x20, 0xc8e, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcb2, 0xcbe, 0xcae, 0xccd, 0xc9c,
0xcbf, 0xcac, 0xcc2, 0xc9f, 0xcbf, 0xca6, 0xccc, 0xcb2, 0xcbe, 0xc8e, 0xcb2, 0xccd,
0x20, 0xc94, 0xca8, 0xccd, 0xcab, 0xccd, 0xcb0, 0xcc0, 0xc9f, 0xccc, 0xca8, 0xccd,
0xc97, 0xccd, 0xcaf, 0xcbe, 0xcac, 0xcb0, 0xccb, 0xca8, 0xcc6, 0xcb9, 0xcb0, 0xcbe,
0xcb0, 0xcc6, 0xc9c, 0xccb, 0xcb9, 0xcbe, 0xca8, 0xccd, 0xcb8, 0xccd, 0x200c, 0xcac,
0xcb0, 0xccd, 0xc97, 0xccd, 0xc9c, 0xcc1, 0xcac, 0xcbe, 0xc95, 0xc82, 0xcaa, 0xcbe,
0xcb2, 0xcbe, 0xc96, 0xcb0, 0xccd, 0xc9f, 0xccb, 0xcae, 0xccd, 0xc95, 0xcbf, 0xc97,
0xcbe, 0xcb2, 0xcbf, 0xc95, 0xcbf, 0xca8, 0xccd, 0xcb7, 0xcbe, 0xcb8, 0xcbe, 0xcb2,
0xcbe, 0xc97, 0xccb, 0xcb8, 0xccd, 0xcb2, 0xcbf, 0xcac, 0xccd, 0xcb0, 0xcc6, 0xcb5,
0xcc0, 0xcb2, 0xccd, 0xcb2, 0xccb, 0xcae, 0xccd, 0xcb2, 0xcc1, 0xcb5, 0xcbe, 0xc82,
0xca1, 0xcbe, 0xcb2, 0xcc1, 0xcac, 0xcc1, 0xc82, 0xcac, 0xcbe, 0xcb6, 0xcbf, 0xcb2,
0xcc1, 0xcb8, 0xcbe, 0xc95, 0xcbe, 0xcae, 0xcb2, 0xcbe, 0xcac, 0xccb, 0xcae, 0xccd,
0xcaf, 0xcbe, 0xcaa, 0xcc1, 0xc9f, 0xcca, 0xcae, 0xcb8, 0xcc7, 0xcb0, 0xcc1, 0xc85,
0xc82, 0xcac, 0xcac, 0xcbe, 0xca8, 0xcc6, 0xcae, 0xcca, 0xc97, 0xcbe, 0xca6, 0xcbf,
0xcb6, 0xcc1, 0xcae, 0xcca, 0xca8, 0xccd, 0xcb0, 0xccb, 0xcb5, 0xcbf, 0xcaf, 0xcbe,
0xca8, 0xcc8, 0xcb0, 0xccb, 0xcac, 0xcbf, 0xca1, 0xccd, 0xc9c, 0xcae, 0xcc7, 0xca8,
0xcbe, 0xca8, 0xcbf, 0xcaf, 0xcbe, 0xcae, 0xcc6, 0xca8, 0xccc, 0xc95, 0xccd, 0x200c,
0xc9a, 0xccb, 0xc9f, 0xccd, 0xc9f, 0xccd, 0xc94, 0xc97, 0xcbe, 0xca1, 0xccb, 0xc97,
0xcc1, 0xcaa, 0xccb, 0xcb0, 0xccd, 0xc9f, 0xccb, 0x2d, 0xca8, 0xccb, 0xcb5, 0xccb,
0xcb8, 0xcbe, 0xcb5, 0xccd, 0x20, 0xc9f, 0xcca, 0xcae, 0xcbf, 0xc9f, 0xccd, 0xcb0,
0xcc8, 0xcaa, 0xccb, 0xcb2, 0xcbf, 0xc9f, 0xcc1, 0xca8, 0xcc0, 0xcb8, 0xccd, 0xcb5,
0xcbf, 0xc82, 0xca1, 0xccd, 0xcb9, 0xccb, 0xc95, 0xccd, 0xc85, 0xca1, 0xc95, 0xccd,
0x200c, 0x200c, 0xc86, 0xc82, 0xc95, 0xcb0, 0xccb, 0xc9c, 0xccd, 0xc86, 0xc82, 0xc97,
0xccd, 0xcb5, 0xcbf, 0xcb2, 0xccd, 0xcb2, 0xcbe, 0xc86, 0xc82, 0xc9f, 0xcbf, 0xc97,
0xcc1, 0xcb5, 0xcbe, 0xc85, 0xcb0, 0xcbe, 0xc97, 0xcc1, 0xcaf, 0xcbe, 0xca8, 0xcbe,
0xcac, 0xccd, 0xcaf, 0xcc2, 0xca8, 0xcb8, 0xccd, 0x20, 0xc90, 0xcb0, 0xcbf, 0xcb8,
0xccd, 0xc95, 0xc9f, 0xcae, 0xcbe, 0xcb0, 0xccd, 0xc95, 0xc95, 0xcca, 0xcb0, 0xccd,
0xca1, 0xcca, 0xcac, 0xcbe, 0xc9c, 0xcc1, 0xc9c, 0xcc8, 0xcb2, 0xcbe, 0x20, 0xcb0,
0xcbf, 0xcaf, 0xccb, 0xc9c, 0xcbe, 0xcae, 0xcc6, 0xc82, 0xca1, 0xcca, 0xc9c, 0xcbe,
0xcb0, 0xcbf, 0xcaf, 0xccb, 0x20, 0xc97, 0xcb2, 0xccd, 0xcb2, 0xcc6, 0xc97, 0xcca,
0xcb8, 0xccd, 0xcb8, 0xcbe, 0xcb2, 0xccd, 0xc9f, 0xcbe, 0xcb8, 0xccd, 0xcaf, 0xcbe,
0xca8, 0xccd, 0x20, 0xc9c, 0xcc1, 0xcb5, 0xcbe, 0xca8, 0xccd, 0xcb8, 0xccd, 0xcaf,
0xcbe, 0xca8, 0xccd, 0x20, 0xcb2, 0xcc2, 0xcaf, 0xcc0, 0xcb8, 0xccd, 0xc9f, 0xcc1,
0xc95, 0xcc1, 0xcae, 0xcbe, 0xca8, 0xccd, 0xc89, 0xcb6, 0xccd, 0xcb5, 0xcbe, 0xcaf,
0xc85, 0xcb0, 0xcc1, 0xcac, 0xcbe, 0xc85, 0xcb8, 0xcc1, 0xca8, 0xccd, 0xcb8, 0xcbf,
0xcaf, 0xcbe, 0xca8, 0xccd, 0xc85, 0xc9f, 0xcbf, 0xc95, 0xcca, 0xc95, 0xcbe, 0xca8,
0xccd, 0xcac, 0xcb9, 0xcc0, 0xcaf, 0xcbe, 0xcac, 0xcbe, 0xcb9, 0xcbf, 0xcaf, 0xcbe,
0x20, 0xcac, 0xc82, 0xca1, 0xcc7, 0xcb0, 0xcbe, 0xcb8, 0xccd, 0xcac, 0xcbe, 0xcb0,
0xccd, 0xcac, 0xca1, 0xccb, 0xcb8, 0xccd, 0xcac, 0xcc7, 0xcb2, 0xcc6, 0xc82, 0xcac,
0xcc6, 0xcb2, 0xcc0, 0xc9c, 0xccd, 0xcac, 0xccd, 0xcb2, 0xcbe, 0xc82, 0xc95, 0xccd,
0x2d, 0xcb8, 0xcbe, 0xcac, 0xccd, 0xcb2, 0xca8, 0xccd, 0xcac, 0xccb, 0xcb5, 0xcbe,
0x20, 0xcb5, 0xcbf, 0xcb8, 0xccd, 0xc9f, 0xcac, 0xcca, 0xc97, 0xccb, 0xc9f, 0xcac,
0xccd, 0xcb5, 0xcbe, 0xcb8, 0xcbf, 0xc95, 0xcc7, 0xcae, 0xccd, 0x200c, 0xcac, 0xccd,
0xcb0, 0xcbf, 0xca1, 0xccd, 0xc9c, 0xccd, 0x20, 0xcac, 0xcc7, 0xc95, 0xcbe, 0xc82,
0xcaa, 0xcca, 0x20, 0xc97, 0xccd, 0xcb0, 0xcbe, 0xc82, 0xca1, 0xcc6, 0xc95, 0xccd,
0xcaf, 0xcbe, 0xc82, 0xc95, 0xca8, 0xccd, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xcb0, 0xc95,
0xcbe, 0xcb8, 0xccd, 0xc95, 0xcc6, 0xcaf, 0xcc6, 0xca8, 0xcbf, 0xc95, 0xcc7, 0xcae,
0xca8, 0xccd, 0xc9a, 0xcbf, 0xc95, 0xcbe, 0xc97, 0xccb, 0xc9a, 0xcbf, 0xcb5, 0xcbe,
0xcb5, 0xcb8, 0xcbf, 0xcaf, 0xcc1, 0xca1, 0xcbe, 0xca1, 0xccd, 0x20, 0xcb5, 0xcbe,
0xcb0, 0xcc6, 0xc9d, 0xccd, 0xc95, 0xccb, 0xcb8, 0xccd, 0xc9f, 0xcbe, 0x20, 0xcb0,
0xcbf, 0xc95, 0xcbe, 0xc95, 0xccd, 0xcb0, 0xcc6, 0xcb8, 0xccd, 0xc9f, 0xca8, 0xccd,
0xc95, 0xccd, 0xcaf, 0xcc2, 0xc87, 0xcac, 0xcbe, 0xc95, 0xcc1, 0xcb0, 0xcbe, 0xc95,
0xcbe, 0xcb5, 0xccd, 0xca1, 0xca8, 0xccd, 0x200c, 0xcae, 0xcbe, 0xcb0, 0xccd, 0xc95,
0xccd, 0x200c, 0xcb6, 0xccd, 0xcb5, 0xcbe, 0xca8, 0xccd, 0xca1, 0xcbe, 0xcb8, 0xca8,
0xccd, 0xca1, 0xcbe, 0xcb8, 0xca8, 0xccd, 0x20, 0xc95, 0xccd, 0xcb0, 0xcc0, 0xc95,
0xccd, 0xca1, 0xcc6, 0xca8, 0xccd, 0xcb5, 0xcb0, 0xccd, 0xca1, 0xcc6, 0xc9f, 0xccd,
0xcb0, 0xcbe, 0xcaf, 0xccd, 0xc9f, 0xccd, 0xca1, 0xcca, 0xcae, 0xcbf, 0xca8, 0xcbf,
0xc95, 0xcbe, 0xc8e, 0xca1, 0xccd, 0xcae, 0xc82, 0xc9f, 0xca8, 0xccd, 0x200c, 0xc87,
0xcb0, 0xcc1, 0xca8, 0xcbf, 0xcaa, 0xcc6, 0xc8e, 0xcb2, 0xccd, 0x20, 0xcb8, 0xcbe,
0xcb2, 0xccd, 0xcb5, 0xca1, 0xccb, 0xcb0, 0xccd, 0xcab, 0xccb, 0xcb0, 0xccd, 0xc9f,
0xccd, 0x20, 0xca8, 0xcc6, 0xcb2, 0xccd, 0xcb8, 0xca8, 0xccd, 0xcab, 0xcca, 0xcb0,
0xccd, 0xc9f, 0xcb2, 0xcc6, 0xc9c, 0xcbe, 0xc97, 0xccd, 0xcb2, 0xcc7, 0xcb8, 0xccd,
0x20, 0xcac, 0xcc7, 0xc97, 0xcc2, 0xcb8, 0xccd, 0x20, 0xcac, 0xcc7, 0xc97, 0xccd,
0xcb0, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xc9f, 0xcb0, 0xccd, 0xc95,
0xccd, 0xc97, 0xccd, 0xcb0, 0xcc6, 0xca8, 0xcbe, 0xca1, 0xc97, 0xccd, 0xcb5, 0xcbe,
0xca1, 0xcc6, 0xcb2, 0xccb, 0xcaa, 0xccd, 0xc97, 0xccd, 0xcb5, 0xcbe, 0xc9f, 0xcc6,
0xcae, 0xcbe, 0xcb2, 0xcbe, 0xc97, 0xcc1, 0xcaf, 0xcbe, 0xc95, 0xccd, 0xcb5, 0xcbf,
0xcb2, 0xccd, 0xc97, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcb9, 0xccd, 0xcaf, 0xcbe, 0xcb2,
0xcbf, 0xcab, 0xcc6, 0xc95, 0xccd, 0xcb8, 0xccd, 0xcb9, 0xcb5, 0xcbe, 0xca8, 0xcbe,
0xcb9, 0xcb0, 0xccd, 0xcae, 0xcca, 0xcb8, 0xcbf, 0xcb2, 0xccd, 0xcb2, 0xccb, 0xc87,
0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcaa, 0xcca, 0xcb2, 0xcbf, 0xcb8,
0xccd, 0xca8, 0xcbe, 0xc95, 0xccd, 0xcb8, 0xccd, 0x2c, 0x20, 0xc87, 0xc82, 0xca1,
0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcae, 0xcb0, 0xcc6, 0xc82, 0xc97, 0xcca, 0x2c,
0x20, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcaa, 0xcc0, 0xc9f,
0xcb0, 0xccd, 0xcb8, 0xccd, 0x200c, 0xcac, 0xcb0, 0xccd, 0xc97, 0xccd, 0x2c, 0x20,
0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xc9f, 0xcc6, 0xcb2, 0xccd,
0x20, 0xcb8, 0xcbf, 0xc9f, 0xcbf, 0x2c, 0x20, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf,
0xcbe, 0xca8, 0xcbe, 0xcb5, 0xcc7, 0xcb5, 0xcbe, 0xcaf, 0xccd, 0x2c, 0x20, 0xc87,
0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcb5, 0xcbf, 0xca8, 0xccd, 0x200c,
0xcb8, 0xcc6, 0xca8, 0xccd, 0xca8, 0xcc0, 0xcb8, 0xccd, 0x2c, 0x20, 0xc87, 0xc82,
0xca1, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcb5, 0xcbf, 0xca8, 0xcbe, 0xcae, 0xccd,
0xcaf, 0xcbe, 0xc95, 0xccd, 0x2c, 0x20, 0xc87, 0xc82, 0xca1, 0xcbf, 0xcaf, 0xcbe,
0xca8, 0xcbe, 0xc87, 0xca8, 0xcc1, 0xcb5, 0xcbf, 0xc95, 0xccd, 0xc88, 0xc95, 0xccd,
0xcb5, 0xcbe, 0xcb2, 0xcbf, 0xc9f, 0xccd, 0xc9c, 0xcae, 0xcc8, 0xc95, 0xcbe, 0xc9c,
0xcc1, 0xca8, 0xcc7, 0xcb5, 0xccd, 0xcb2, 0xcc2, 0xcaf, 0xcc0, 0xcb8, 0xccd, 0x200c,
0xcb5, 0xcbf, 0xcb2, 0xccd, 0xcb2, 0xcc6, 0xcae, 0xcca, 0xc82, 0xc9f, 0xcbf, 0xcb8,
0xcc6, 0xcb2, 0xccd, 0xcb2, 0xccb, 0x2c, 0x20, 0xc95, 0xcc6, 0xc82, 0xc9f, 0xcc1,
0xc95, 0xcbf, 0xc95, 0xccd, 0xcb0, 0xcc6, 0xcb2, 0xcc6, 0xc82, 0xca1, 0xcbf, 0xc9c,
0xccd, 0xc95, 0xccd, 0xcb2, 0xcbe, 0x20, 0xcaa, 0xcc6, 0xc9c, 0xccd, 0xcb2, 0xcbf,
0xcae, 0xcbe, 0xcb2, 0xcbe, 0xcb8, 0xccd, 0x20, 0xc8f, 0xc82, 0xc9c, 0xcb2, 0xcc0,
0xcb8, 0xccd, 0xcb2, 0xccb, 0xcb5, 0xcb0, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcbf,
0xca8, 0xccd, 0xcb8, 0xcb8, 0xccd, 0x20, 0xc95, 0xccd, 0xcb5, 0xcbe, 0xcb0, 0xccd,
0xc9f, 0xcb0, 0xccd, 0xcae, 0xcc6, 0xcb8, 0xcc0, 0xcaf, 0xcca, 0xcae, 0xca8, 0xcbe,
0xc97, 0xcc1, 0xcb5, 0xcbe, 0xcae, 0xca8, 0xcbe, 0xcb5, 0xccd, 0xcb8, 0xccd, 0xcae,
0xcb0, 0xcbf, 0xc97, 0xcca, 0xcae, 0xcbe, 0xcb0, 0xccd, 0xc9f, 0xcbf, 0xca8, 0xcbf,
0xc95, 0xccd, 0xcae, 0xc9f, 0xcbe, 0xcae, 0xccb, 0xcb0, 0xcb8, 0xccd, 0xcae, 0xc9c,
0xc9f, 0xccd, 0xcb2, 0xcbe, 0xca8, 0xccd, 0xcae, 0xcc6, 0xca8, 0xcca, 0xcae, 0xcbf,
0xca8, 0xcc0, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xca6, 0xcbe, 0xcae, 0xcc6, 0xc9f, 0xccd,
0xcb2, 0xcbe, 0xc95, 0xcbe, 0xc9f, 0xccd, 0xcb2, 0xcbe, 0xcae, 0xcc6, 0xc95, 0xccd,
0xcb8, 0xcbf, 0xc95, 0xccb, 0x20, 0xca8, 0xc97, 0xcb0, 0xcae, 0xcbf, 0xc95, 0xcc6,
0xcb2, 0xcbe, 0xca8, 0xccd, 0xcae, 0xcbe, 0xc82, 0xc95, 0xccd, 0xc9f, 0xca8, 0xccd,
0xcae, 0xcbe, 0xc82, 0xc9f, 0xcc6, 0xcb0, 0xccd, 0xcb0, 0xcbf, 0xcae, 0xcbe, 0xc82,
0xc9f, 0xcbf, 0xcb5, 0xcc0, 0xca1, 0xcbf, 0xcaf, 0xcca, 0xcae, 0xcbe, 0xc82, 0xc9f,
0xccd, 0x200c, 0xcb8, 0xcc6, 0xcb0, 0xcc7, 0xc9f, 0xccd, 0xca8, 0xcb8, 0xccd, 0xcb8,
0xccc, 0xcb5, 0xccd, 0xca8, 0xccd, 0xcaf, 0xcc2, 0xcaf, 0xcbe, 0xcb0, 0xccd, 0xc95,
0xccd, 0xca8, 0xccb, 0xcae, 0xccd, 0xca8, 0xcca, 0xcb0, 0xcca, 0xca8, 0xccd, 0xcb9,
0xcbe, 0xcac, 0xccd, 0xcaf, 0xcc2, 0xcb2, 0xcbe, 0x2c, 0x20, 0xc89, 0xca4, 0xccd,
0xca4, 0xcb0, 0x20, 0xca1, 0xc95, 0xcca, 0xc9f, 0xcb8, 0xcc6, 0xc82, 0xc9f, 0xcb0,
0xccd, 0x2c, 0x20, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xca1, 0xc95, 0xcca,
0xc9f, 0xca8, 0xccd, 0xcaf, 0xcc2, 0x20, 0xcb8, 0xcb2, 0xcc7, 0xc82, 0x2c, 0x20,
0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xca1, 0xc95, 0xcca, 0xc9f, 0xca8, 0xcc2,
0xc95, 0xccd, 0xc93, 0xc9c, 0xcbf, 0xca8, 0xcbe, 0xc97, 0xcbe, 0xcaa, 0xca8, 0xcbe,
0xcae, 0xcbe, 0xcaa, 0xcb0, 0xcae, 0xcbe, 0xcb0, 0xcbf, 0xcac, 0xccb, 0xcab, 0xcbf,
0xca8, 0xcbf, 0xc95, 0xccd, 0xcb8, 0xccd, 0xcaa, 0xccb, 0xcb0, 0xccd, 0xc9f, 0xccd,
0x2d, 0xc92, 0x2d, 0xcaa, 0xccd, 0xcb0, 0xcbf, 0xca8, 0xccd, 0xcb8, 0xccd, 0xcaa,
0xccb, 0xcb0, 0xccd, 0xc9f, 0xccd, 0x20, 0xc86, 0xcab, 0xccd, 0x20, 0xcb8, 0xccd,
0xcaa, 0xcc7, 0xca8, 0xccd, 0xcaa, 0xccb, 0xcb0, 0xccd, 0xc9f, 0xcca, 0x20, 0xcb5,
0xcc6, 0xcb2, 0xccd, 0xcb9, 0xccb, 0xcaa, 0xccd, 0xcaf, 0xcc1, 0xcb0, 0xccd, 0xc9f,
0xccb, 0x20, 0xcb0, 0xcbf, 0xc95, 0xcca, 0xcaa, 0xcc1, 0xc82, 0xca4, 0xcbe, 0x20,
0xc85, 0xcb0, 0xcc7, 0xca8, 0xcbe, 0xcb8, 0xccd, 0xcb0, 0xcbe, 0xc82, 0xc95, 0xcbf,
0xca8, 0xccd, 0x20, 0xc87, 0xca8, 0xccd, 0x200c, 0xcb2, 0xcc6, 0xc9f, 0xccd, 0xcb0,
0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf, 0xcb0, 0xcc6, 0xc9c, 0xcbf, 0xca8, 0xcbe, 0xcb0,
0xcc6, 0xcb8, 0xcca, 0xcb2, 0xccd, 0xcaf, 0xcc2, 0xc9f, 0xccd, 0xcb0, 0xcbf, 0xcaf,
0xccb, 0x20, 0xcac, 0xccd, 0xcb0, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc95, 0xcca, 0xcb8,
0xcbe, 0xc82, 0xc9f, 0xcb0, 0xcc6, 0xcae, 0xccd, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xc82,
0xc9f, 0xcbf, 0xcaf, 0xcbe, 0xc97, 0xcca, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc9f,
0xccb, 0x20, 0xca1, 0xcca, 0xcae, 0xcbf, 0xc82, 0xc97, 0xcca, 0xcb8, 0xcbe, 0xcb5,
0xccd, 0x20, 0xcaa, 0xcbe, 0xcb2, 0xcca, 0xc87, 0xc9f, 0xccd, 0xc9f, 0xcca, 0xc95,
0xccd, 0xc95, 0xcca, 0xcb0, 0xccd, 0xc9f, 0xcc2, 0xcae, 0xcbf, 0xcaf, 0xcc8, 0xc9f,
0xccd, 0xcb8, 0xcbf, 0xca4, 0xccd, 0xc95, 0xcbe, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd,
0x20, 0xcac, 0xcbe, 0xcb0, 0xccd, 0xca4, 0xcc6, 0xcb2, 0xcc6, 0xcae, 0xcbf, 0xcb8,
0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xc9c, 0xcbe, 0xca8, 0xccd, 0xcb8, 0xccd, 0xcb8,
0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xc95, 0xcbf, 0xc9f, 0xccd, 0xcb8, 0xccd, 0xcb8,
0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcb2, 0xcc2, 0xcb8, 0xcbf, 0xcaf, 0xcbe, 0xcb8,
0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xca5, 0xcbe, 0xcae, 0xcb8, 0xccd, 0xcb8, 0xcc7,
0xc82, 0xc9f, 0xccd, 0x20, 0xcb5, 0xcbf, 0xca8, 0xccd, 0xcb8, 0xcc6, 0xc82, 0xc9f,
0xccd, 0xcb8, 0xccd, 0xcb5, 0xcbf, 0xcab, 0xccd, 0xc9f, 0xccd, 0x20, 0xc95, 0xcb0,
0xcc6, 0xc82, 0xc9f, 0xccd, 0xca4, 0xcc6, 0xc97, 0xcc1, 0xcb8, 0xcbf, 0xc97, 0xcb2,
0xccd, 0xcaa, 0xcbe, 0xca5, 0xcc2, 0xcb2, 0xcc6, 0xca4, 0xcbf, 0xc9c, 0xccd, 0xcb5,
0xcbe, 0xca8, 0xcbe, 0xc9f, 0xcca, 0xcb0, 0xcca, 0xc82, 0xc9f, 0xccb, 0xc9f, 0xcbe,
0xcb0, 0xccd, 0xc9f, 0xcca, 0xcb2, 0xcbe, 0xcb5, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc95,
0xcca, 0xcb5, 0xcb0, 0xccd, 0x200c, 0xcb5, 0xcc8, 0xc9f, 0xccd, 0x200c, 0xcb9, 0xcbe,
0xcb0, 0xccd, 0xcb8, 0xccd, 0xcb5, 0xcbf, 0xca8, 0xccd, 0xca8, 0xcbf, 0xcaa, 0xcc6,
0xc97, 0xccd, 0xcaf, 0xcbe, 0xc95, 0xcc1, 0xc9f, 0xcbe, 0xc9f, 0xccd, 0xc95, 0xcc7,
0xcb8, 0xcbf, 0xca1, 0xcc7, 0xcb5, 0xcc0, 0xcb8, 0xccd, 0xca1, 0xcc1, 0xcae, 0xc82,
0xc9f, 0xccd, 0x20, 0xca1, 0xcbf, 0x20, 0xc89, 0xcb0, 0xccd, 0xcb5, 0xcbf, 0xcb2,
0xcc6, 0xcae, 0xccd, 0xcaf, 0xcbe, 0xc95, 0xccd, 0xcb5, 0xcc6, 0xcb0, 0xcbf, 0xcae,
0xcbe, 0xcb5, 0xccd, 0xcb8, 0xca8, 0xccd, 0xcae, 0xccd, 0xcaf, 0xcbe, 0xc95, 0xccd,
0x200d, 0xcae, 0xcc1, 0xcb0, 0xccd, 0xca1, 0xcca, 0xcaa, 0xcbe, 0xcb2, 0xccd, 0xcae,
0xcb0, 0xccd, 0xcb0, 0xcca, 0xca5, 0xcc7, 0xcb0, 0xcbe, 0xcb8, 0xccd, 0xcaf, 0xccb,
0xcb5, 0xcbe, 0xc9f, 0xccd, 0xcb0, 0xcbe, 0xcb2, 0xccd, 0xcb5, 0xccb, 0xcb8, 0xccd,
0xc9f, 0xccb, 0xc95, 0xccd, 0xcb2, 0xcbe, 0xc82, 0xc97, 0xccd, 0xcaf, 0xcc0, 0xcb0,
0xccd, 0xcac, 0xcc6, 0xca8, 0xccd, 0xc8f, 0xca1, 0xca8, 0xccd, 0xc85, 0xcb2, 0xccd,
0xcae, 0xcbe, 0xc9f, 0xcbf, 0xc85, 0xcae, 0xccd, 0xcae, 0xcbe, 0xca8, 0xccd, 0xc85,
0xca8, 0xca6, 0xccd, 0xcaf, 0xcb0, 0xccd, 0xc85, 0xc95, 0xccd, 0xca4, 0xcbe, 0xcb5,
0xccd, 0xc85, 0xc95, 0xccd, 0xc9f, 0xccb, 0xcac, 0xcc6, 0xc85, 0xcb6, 0xccd, 0xc97,
0xcac, 0xcbe, 0xca4, 0xccd, 0xc85, 0xc9f, 0xccd, 0xcb0, 0xcbe, 0xcb5, 0xccd, 0xcac,
0xcbe, 0xc97, 0xccd, 0xca6, 0xcbe, 0xca6, 0xccd, 0xcac, 0xcb9, 0xccd, 0xcb0, 0xcc7,
0xca8, 0xccd, 0xcac, 0xc95, 0xcc1, 0xcac, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc95, 0xcbe,
0xc95, 0xccd, 0xcac, 0xcbe, 0xcb0, 0xccd, 0xca8, 0xccc, 0xcb2, 0xccd, 0xcac, 0xcc0,
0xcb0, 0xc9f, 0xccd, 0xcac, 0xcbf, 0xcb6, 0xccd, 0xc95, 0xcc6, 0xc95, 0xccd, 0xcac,
0xccd, 0xcb0, 0xcc2, 0xca8, 0xcbf, 0xc9a, 0xcbf, 0xc9f, 0xc9a, 0xcca, 0xcaf, 0xccd,
0x200d, 0xcac, 0xcbe, 0xcb2, 0xccd, 0xcb8, 0xca8, 0xccd, 0xc95, 0xcca, 0xcb2, 0xc82,
0xcac, 0xcca, 0xca1, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xcb8, 0xccd, 0xca2, 0xcbe,
0xc95, 0xcbe, 0xca6, 0xcbf, 0xcb2, 0xcbf, 0xca6, 0xcc1, 0xcac, 0xcc8, 0xca6, 0xcc1,
0xcb6, 0xcbe, 0xc82, 0xcac, 0xcc6, 0xcab, 0xcae, 0xcbe, 0xc97, 0xcc1, 0xcb8, 0xccd,
0xca4, 0xcbe, 0xc97, 0xcbe, 0xc9c, 0xcbe, 0xcb9, 0xcc6, 0xcac, 0xccd, 0xcb0, 0xcbe,
0xca8, 0xccd, 0xcb9, 0xcca, 0x20, 0xc9a, 0xcbf, 0x20, 0xcae, 0xcbf, 0xca8, 0xccd,
0x20, 0xcb8, 0xcbf, 0xc9f, 0xcbf, 0xcb9, 0xcbe, 0xc82, 0xc97, 0xccd, 0x20, 0xc95,
0xcbe, 0xc82, 0xc97, 0xccd, 0xcb9, 0xccb, 0xcb5, 0xccd, 0xca1, 0xccd, 0xc87, 0xcb0,
0xc95, 0xcc1, 0xc9f, 0xccd, 0xcb8, 0xccd, 0xc95, 0xccd, 0xc9c, 0xc95, 0xcbe, 0xcb0,
0xccd, 0xca4, 0xcbe, 0xc9c, 0xcaf, 0xcaa, 0xcc1, 0xcb0, 0xc9c, 0xcc6, 0xcb0, 0xcc1,
0xcb8, 0xcb2, 0xcc6, 0xcae, 0xccd, 0xc95, 0xcbe, 0xcac, 0xcc2, 0xcb2, 0xccd, 0xc95,
0xc82, 0xc9a, 0xc9f, 0xccd, 0xc95, 0xccd, 0xc95, 0xcb0, 0xcbe, 0xc9a, 0xcbf, 0xc95,
0xca0, 0xccd, 0xcae, 0xc82, 0xca1, 0xcc1, 0xc96, 0xc82, 0xca1, 0xccd, 0xcaf, 0xcbe,
0xc97, 0xcbe, 0xc95, 0xcca, 0xcb2, 0xccd, 0xc95, 0xca4, 0xccd, 0xca4, 0xcbe, 0xc95,
0xccd, 0xcb0, 0xcbe, 0xcb8, 0xccd, 0xca8, 0xcca, 0xcaf, 0xcbe, 0xcb0, 0xccd, 0xcb8,
0xccd, 0xc95, 0xccd, 0xc95, 0xccc, 0xcb2, 0xcbe, 0x20, 0xcb2, 0xc82, 0xcaa, 0xcc1,
0xcb0, 0xc95, 0xcc1, 0xc9a, 0xcbf, 0xc82, 0xc97, 0xccd, 0xc95, 0xcc1, 0xcb5, 0xcc8,
0xca4, 0xccd, 0xcae, 0xc95, 0xcbe, 0xcb5, 0xcc1, 0xcae, 0xc97, 0xca6, 0xca8, 0xccd,
0x200c, 0xcae, 0xcc6, 0xc95, 0xcbe, 0xcb8, 0xccd, 0xcb8, 0xcb0, 0xccd, 0xcae, 0xca8,
0xcbf, 0xcb2, 0xcbe, 0xcae, 0xcb8, 0xccd, 0xc95, 0xc9f, 0xccd, 0xca8, 0xcbf, 0xc95,
0xccb, 0xcb8, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xccb, 0xcb5, 0xccb, 0xc95, 0xcc1, 0xc9c,
0xcc6, 0xca8, 0xcca, 0xcb5, 0xccb, 0xcb8, 0xcbf, 0xcac, 0xcbf, 0xcb8, 0xccd, 0xc95,
0xccd, 0xc92, 0xcae, 0xccd, 0xcb8, 0xccd, 0xc95, 0xccd, 0xc92, 0xcb0, 0xcb2, 0xccd,
0xca8, 0xccb, 0xcae, 0xccd, 0x20, 0xcaa, 0xcc6, 0xca8, 0xccd, 0xcaa, 0xcca, 0xc82,
0xc9f, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xc95, 0xccd, 0xcaa, 0xccd, 0xcaf, 0xcca, 0xc82,
0xc97, 0xccd, 0x200d, 0xcaf, 0xcbe, 0xc82, 0xc97, 0xccd, 0xc96, 0xca4, 0xcbe, 0xcb0,
0xccd, 0xc95, 0xcca, 0xcb8, 0xccd, 0xc9f, 0xca8, 0xcaf, 0xccd, 0xc95, 0xcbf, 0xc9c,
0xcc8, 0xcb2, 0xccb, 0xcb0, 0xccd, 0xca6, 0xcb0, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0xccd,
0xcb8, 0xc96, 0xcbe, 0xcb2, 0xcbf, 0xca8, 0xccd, 0xcb8, 0xcae, 0xcb0, 0xc96, 0xc82,
0xca1, 0xcb8, 0xcbf, 0xcaf, 0xccb, 0xcb2, 0xccd, 0xcb6, 0xcbe, 0xc82, 0xc98, 0xcc8,
0xcb8, 0xcbf, 0xc82, 0xc97, 0xcaa, 0xcc1, 0xcb0, 0xccd, 0xcb8, 0xccd, 0xcb0, 0xcc7,
0xca6, 0xccd, 0xca8, 0xcc7, 0xc95, 0xccb, 0xcb2, 0xcc0, 0xcae, 0xcb8, 0xccd, 0xc95,
0xca4, 0xcc8, 0xcaa, 0xcc6, 0xca4, 0xcbe, 0xcb6, 0xccd, 0xc95, 0xcc6, 0xc82, 0xc9f,
0xccd, 0xc9f, 0xcbf, 0xcac, 0xcbf, 0xcb2, 0xcbf, 0xcb8, 0xcbf, 0xca4, 0xcc6, 0xcb9,
0xccd, 0xcb0, 0xcbe, 0xca8, 0xccd, 0xca5, 0xcbf, 0xc82, 0xcaa, 0xcc1, 0xc9f, 0xccb,
0xc95, 0xcbf, 0xcaf, 0xcca, 0xc9f, 0xccb, 0xcae, 0xccd, 0x200c, 0xcb8, 0xccd, 0xc95,
0xccd, 0xc89, 0xcb2, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xca4, 0xcca, 0xcb0, 0xccd,
0xc89, 0xcb0, 0xcc1, 0xcae, 0xccd, 0x200d, 0xc95, 0xcbf, 0xc89, 0xcb8, 0xccd, 0xca4,
0xccd, 0x2d, 0xca8, 0xcc6, 0xcb0, 0xcbe, 0xcb5, 0xcbf, 0xcaf, 0xcc6, 0xc82, 0xc9f,
0xcbf, 0xcaf, 0xcbe, 0xca8, 0xccd, 0xcb5, 0xccd, 0xcb2, 0xcbe, 0xca6, 0xcbf, 0xcb5,
0xcca, 0xcb8, 0xccd, 0xc9f, 0xcbe, 0xc95, 0xccd, 0x200c, 0xcaf, 0xc95, 0xcc1, 0xc9f,
0xccd, 0xcb8, 0xccd, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc97, 0xcca, 0xca8, 0xccd,
0xc8e, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xc9f, 0xcc6, 0xcb0, 0xcbf, 0xca8, 0xccd, 0xcac,
0xcb0, 0xccd, 0xc97, 0xccd, 0xcaf, 0xcc6, 0xcb0, 0xcc6, 0xcb5, 0xcbe, 0xca8, 0xccd,
0xc85, 0xc9d, 0xccb, 0xcb0, 0xcbf, 0xcb8, 0xccd, 0xcac, 0xcb0, 0xccd, 0xcae, 0xcc1,
0xca1, 0xcbe, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xcb0, 0xcbf, 0xc95, 0xcc7, 0xcaa,
0xccd, 0x20, 0xcb5, 0xcc6, 0xcb0, 0xccd, 0xca1, 0xccd, 0xcab, 0xccd, 0xcaf, 0xcbe,
0xcb0, 0xcca, 0xcae, 0xcc6, 0xca1, 0xcbf, 0xcb0, 0xcbe, 0xcb0, 0xcc8, 0xc95, 0xccd,
0xc9c, 0xcbe, 0xcb5, 0xcbf, 0xc95, 0xccd, 0xca6, 0xc95, 0xccd, 0xcb7, 0xcbf, 0xca3,
0x20, 0xc9c, 0xcbe, 0xcb0, 0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe, 0xcb8, 0xcc7, 0xc82,
0xc9f, 0xccd, 0x20, 0xcb9, 0xcc6, 0xcb2, 0xcc6, 0xca8, 0xcbe, 0xcb8, 0xccd, 0xc9f,
0xccd, 0xcaf, 0xcbe, 0xca8, 0xccd, 0xcb2, 0xcbf, 0xc85, 0xca1, 0xcbf, 0xcb2, 0xcc7,
0xca1, 0xccd, 0xcac, 0xccd, 0xcb0, 0xcbf, 0xcb8, 0xccd, 0xcac, 0xcc7, 0xca8, 0xccd,
0x200c, 0xcac, 0xccd, 0xcb0, 0xcca, 0xc95, 0xca8, 0xccd, 0x20, 0xcb9, 0xcbf, 0xcb2,
0xccd, 0xca1, 0xcbe, 0xcb0, 0xccd, 0xcb5, 0xcbf, 0xca8, 0xccd, 0xcaf, 0xcc1, 0xc95,
0xccd, 0xcb2, 0xcbe, 0xcb9, 0xccb, 0xcac, 0xcbe, 0xcb0, 0xccd, 0xc9f, 0xccd, 0x200c,
0xcb2, 0xcbf, 0xc82, 0xca1, 0xcbf, 0xcae, 0xcbe, 0xca8, 0xccd, 0xcb2, 0xcbe, 0xcb0,
0xccd, 0xca1, 0xccd, 0x20, 0xcb9, 0xccb, 0xcb5, 0xccd, 0xcae, 0xcc6, 0xcb2, 0xccd,
0xcac, 0xcb0, 0xccd, 0xca8, 0xccd, 0xcaa, 0xcb0, 0xccd, 0xca5, 0xccd, 0xcb8, 0xcbf,
0xca1, 0xccd, 0xca8, 0xcbf, 0xc86, 0xcae, 0xccd, 0x200c, 0xcb8, 0xccd, 0xc9f, 0xcc6,
0xcb0, 0xccd, 0x200c, 0xca1, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xc85, 0xc82, 0xca1, 0xccb,
0xcb0, 0xcbe, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcbe, 0xc96, 0xcbe, 0xca8,
0xccd, 0xc85, 0xca5, 0xcc6, 0xca8, 0xccd, 0xcb8, 0xccd, 0xcac, 0xcc6, 0xcb2, 0xccd,
0x200c, 0xc97, 0xccd, 0xcb0, 0xcc7, 0xca1, 0xccd, 0xcac, 0xcb0, 0xccd, 0xcb2, 0xcbf,
0xca8, 0xccd, 0xcac, 0xccd, 0xcb0, 0xcbe, 0xc9f, 0xcbf, 0xcb8, 0xccd, 0xcb2, 0xcbe,
0xcb5, 0xcac, 0xccd, 0xcb0, 0xcb8, 0xccd, 0xcb8, 0xcc6, 0xcb2, 0xccd, 0xcb8, 0xccd,
0xcac, 0xcc1, 0xc95, 0xcbe, 0xcb0, 0xcc6, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcac, 0xcc1,
0xca1, 0xcbe, 0xcaa, 0xcc6, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcac, 0xcc1, 0xcb8, 0xcbf,
0xc82, 0xc9c, 0xcc6, 0xca8, 0xccd, 0xc9a, 0xcbf, 0xcb8, 0xcbf, 0xca8, 0xcbe, 0xcb5,
0xccd, 0xc95, 0xccb, 0xcaa, 0xcc6, 0xca8, 0xccd, 0x200c, 0xcb9, 0xcc7, 0xc97, 0xca8,
0xccd, 0xca1, 0xcc1, 0xcac, 0xccd, 0xcb2, 0xcbf, 0xca8, 0xccd, 0xc9c, 0xcbf, 0xcac,
0xccd, 0xcb0, 0xcbe, 0xcb2, 0xccd, 0xc9f, 0xcb0, 0xccd, 0xc97, 0xccd, 0xcaf, 0xcc1,
0xcb0, 0xccd, 0xca8, 0xccd, 0x200d, 0xcb8, 0xcc6, 0xcb9, 0xcc6, 0xcb2, 0xccd, 0xcb8,
0xcbf, 0xc82, 0xc95, 0xcbf, 0xc90, 0xcb2, 0xccd, 0x20, 0xc86, 0xcab, 0xccd, 0x20,
0xcae, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xccd, 0xc87, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8,
0xccd, 0x200c, 0xcac, 0xcc1, 0xcb2, 0xccd, 0xc9c, 0xcc6, 0xcb0, 0xccd, 0xcb8, 0xcbf,
0xc95, 0xcb2, 0xcbf, 0xca8, 0xcbf, 0xca8, 0xccd, 0x200c, 0xc97, 0xccd, 0xcb0, 0xcbe,
0xca1, 0xccd, 0xc95, 0xcbf, 0xcb0, 0xcbe, 0xcb5, 0xccd, 0xc95, 0xcbf, 0xcb5, 0xccd,
0xcb2, 0xcbf, 0xcb8, 0xccd, 0xcac, 0xca8, 0xccd, 0xcb2, 0xccd, 0xcaf, 0xcc2, 0xcac,
0xccd, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0xcb2, 0xc82, 0xca1, 0xca8, 0xccd,
0xcb2, 0xc95, 0xccd, 0xcb8, 0xcc6, 0xc82, 0xcac, 0xcb0, 0xccd, 0xc97, 0xccd, 0xcae,
0xccd, 0xcaf, 0xcbe, 0xca1, 0xccd, 0xcb0, 0xcbf, 0xca1, 0xccd, 0xcae, 0xcbe, 0xcb2,
0xccd, 0xc9f, 0xcbe, 0xcae, 0xcb0, 0xcbf, 0xcb9, 0xcae, 0xccd, 0xca8, 0xccd, 0xcae,
0xcbf, 0xca8, 0xccd, 0x200c, 0xcb8, 0xccd, 0xc95, 0xccd, 0xcae, 0xcca, 0xca8, 0xcbe,
0xc95, 0xcca, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xccb, 0xc93, 0xcb8, 0xccd, 0xcb2,
0xccb, 0xcaa, 0xccd, 0xcaf, 0xcbe, 0xcb0, 0xcbf, 0xcb8, 0xccd, 0xcaa, 0xcca, 0xca1,
0xccd, 0xc97, 0xcca, 0xcb0, 0xcbf, 0xc95, 0xcaa, 0xccd, 0xcb0, 0xcbe, 0xc97, 0xccd,
0xcb5, 0xcc6, 0xcb0, 0xcbf, 0xc97, 0xcbe, 0xcb0, 0xccb, 0xcae, 0xccd, 0xcb8, 0xccd,
0xcaf, 0xcbe, 0xca8, 0xccd, 0x20, 0xcae, 0xcb0, 0xcc0, 0xca8, 0xcca, 0xcb8, 0xcb0,
0xcaf, 0xcc7, 0xcb5, 0xcca, 0xcb8, 0xcb0, 0xcbe, 0xc9f, 0xccb, 0xcb5, 0xccd, 0xcb8,
0xcbf, 0xcae, 0xccd, 0xcab, 0xcc6, 0xcb0, 0xccb, 0xcaa, 0xccb, 0xcb2, 0xccd, 0xcb8,
0xccd, 0xc95, 0xcbe, 0xcaa, 0xcbf, 0xcaf, 0xcbe, 0xcb8, 0xccb, 0xcab, 0xcbf, 0xcaf,
0xcbe, 0xcb8, 0xccd, 0xc9f, 0xcbe, 0xc95, 0xccd, 0x200d, 0xcb9, 0xccb, 0xcae, 0xccd,
0xc9f, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcbf, 0xca8, 0xccd, 0xc9f, 0xcbf, 0xcb0, 0xcbe,
0xca8, 0xcc6, 0xc89, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xca8, 0xcca, 0xcb5, 0xcb8, 0xccd,
0xc95, 0xccd, 0xcb5, 0xca1, 0xcc2, 0xc9c, 0xccd, 0xcb5, 0xccd, 0xcaf, 0xcbe, 0xc9f,
0xcbf, 0xc95, 0xca8, 0xccd, 0xcb5, 0xcbf, 0xcaf, 0xcc6, 0xca8, 0xccd, 0xca8, 0xcbe,
0xcb5, 0xcbf, 0xcb2, 0xccd, 0xca8, 0xcbf, 0xcaf, 0xcb8, 0xccd, 0xcb5, 0xcca, 0xcb2,
0xccd, 0xc97, 0xcca, 0xc97, 0xccd, 0xcb0, 0xcbe, 0xca1, 0xccd, 0xcb5, 0xcbe, 0xcb0,
0xccd, 0xcb8, 0xcbe, 0xc9c, 0xcbe, 0xc97, 0xccd, 0xcb0, 0xcc6, 0xcac, 0xccd, 0x200c,
0xc9c, 0xccd, 0xcaf, 0xcc2, 0xcb0, 0xcbf, 0xc95, 0xccd, 0xc85, 0xc82, 0xc9f, 0xcbe,
0xca8, 0xca8, 0xcbe, 0xcb0, 0xcbf, 0xcb5, 0xcca, 0xc9a, 0xcbe, 0xc97, 0xcca, 0xcb8,
0xccd, 0xc95, 0xccd, 0xcb0, 0xcbf, 0xcb8, 0xccd, 0xcae, 0xcb8, 0xccd, 0xc95, 0xcca,
0xc95, 0xccb, 0xcb8, 0xccd, 0xc95, 0xcca, 0xcae, 0xccb, 0xcb0, 0xcca, 0xc95, 0xcc6,
0xcb0, 0xccd, 0xc97, 0xccd, 0xcaf, 0xcc1, 0xcb2, 0xcc6, 0xca8, 0xccd, 0xcae, 0xcbe,
0xcb9, 0xcc6, 0xcae, 0xcbe, 0xcb2, 0xccd, 0xca1, 0xcc0, 0xcb5, 0xccd, 0xcb8, 0xccd,
0xcae, 0xcbe, 0xcb0, 0xcbf, 0xcb7, 0xcb8, 0xccd, 0xcae, 0xcaf, 0xcca, 0xc9f, 0xccd,
0xc9f, 0xcc6, 0xcb0, 0xcc0, 0xcaf, 0xcc2, 0xca8, 0xcbf, 0xcaf, 0xca8, 0xccd, 0xc85,
0xcaa, 0xcbf, 0xcaf, 0xcbe, 0xc86, 0xc95, 0xccd, 0x20, 0xcb2, 0xccd, 0xcaf, 0xcbe,
0xc82, 0xca1, 0xccd, 0xcac, 0xccc, 0xc97, 0xcc6, 0xca8, 0xccd, 0x200d, 0xcb5, 0xcbf,
0xcb2, 0xccd, 0xcb2, 0xcc6, 0xc9a, 0xca5, 0xcbe, 0xcae, 0xccd, 0xc9a, 0xcc1, 0xc95,
0xccd, 0xc88, 0xcb8, 0xccd, 0xc9f, 0xcb0, 0xccd, 0xc87, 0xcab, 0xcc7, 0xc9f, 0xccd,
0xcab, 0xc95, 0xccb, 0xcab, 0xccb, 0xcab, 0xcbf, 0xc9c, 0xcbf, 0xcab, 0xcc1, 0xca8,
0xcbe, 0xcab, 0xcc1, 0xc9f, 0xcbf, 0xc97, 0xcb2, 0xcbe, 0xcaa, 0xc97, 0xccb, 0xcb8,
0xccd, 0xc97, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xcac, 0xcbf, 0xcaf, 0xcb0, 0xccd, 0xc97,
0xccd, 0xcb5, 0xcbe, 0xca1, 0xcb2, 0xccd, 0xc95, 0xcc6, 0xca8, 0xcbe, 0xcb2, 0xccd,
0xc97, 0xccd, 0xcb5, 0xcbe, 0xcae, 0xccd, 0xcb9, 0xcca, 0xca8, 0xcb2, 0xcc1, 0xcb2,
0xcc1, 0xc8e, 0xc82, 0xca1, 0xcb0, 0xccd, 0xcac, 0xcb0, 0xcbf, 0xc95, 0xcbf, 0xcb0,
0xcbf, 0xca4, 0xcbf, 0xcae, 0xca4, 0xcbf, 0xc95, 0xccb, 0xcb8, 0xccd, 0xcb0, 0xcc6,
0xc95, 0xccd, 0xcb5, 0xcbe, 0xc9c, 0xcb2, 0xcc7, 0xca8, 0xccd, 0xcae, 0xc9c, 0xcc1,
0xcb0, 0xcca, 0xcae, 0xcbe, 0xcb0, 0xccd, 0xc95, 0xccd, 0xcaf, 0xcc2, 0xcb8, 0xcb8,
0xccd, 0xcae, 0xcbf, 0xca1, 0xccd, 0x200c, 0xcb5, 0xcc7, 0xca8, 0xccc, 0xcb0, 0xcc1,
0xca8, 0xcbf, 0xcaf, 0xcc1, 0xca8, 0xcbe, 0xcb0, 0xccd, 0xcab, 0xccb, 0xc95, 0xccd,
0xca8, 0xccd, 0xcaf, 0xcc2, 0xcae, 0xcbf, 0xcaf, 0xcbe, 0xcaa, 0xcbe, 0xc97, 0xcca,
0x20, 0xcaa, 0xcbe, 0xc97, 0xcca, 0xcaa, 0xcbe, 0xcb2, 0xcbe, 0xcb5, 0xccd, 0xcaa,
0xcbf, 0xc9f, 0xccd, 0x200c, 0xc95, 0xcc8, 0xcb0, 0xccd, 0xca8, 0xccd, 0xcaa, 0xccb,
0xca8, 0xccd, 0x200c, 0xcaa, 0xcc6, 0xcaa, 0xccb, 0xcb0, 0xccd, 0xc9f, 0xccd, 0x20,
0xcae, 0xccb, 0xcb0, 0xcbf, 0xcb8, 0xccd, 0x200c, 0xcac, 0xcc8, 0xcb0, 0xcb0, 0xccb,
0xc9f, 0xccb, 0xc82, 0xc97, 0xcbe, 0xcb8, 0xcc8, 0xcaa, 0xca8, 0xccd, 0xca4, 0xcb9,
0xcbf, 0xca4, 0xcbf, 0xca4, 0xcb0, 0xcbe, 0xcb5, 0xcbe, 0xc9f, 0xcca, 0xc82, 0xc97,
0xcbe, 0xc9f, 0xcbe, 0xcaa, 0xcc1, 0xcb5, 0xcc7, 0xc95, 0xccd, 0xcb5, 0xccd, 0xcaf,
0xcbe, 0xcb2, 0xcbf, 0xcb8, 0xccd, 0x639, 0x627, 0x628, 0x650, 0x62f, 0x62c, 0x627,
0x646, 0x627, 0x620, 0x6a9, 0x631, 0x627, 0x639, 0x62f, 0x6cc, 0x633, 0x20, 0x627,
0x628, 0x627, 0x628, 0x627, 0x627, 0x64e, 0x644, 0x62c, 0x6cc, 0x656, 0x631, 0x650,
0x6cc, 0x627, 0x627, 0x64e, 0x633, 0x645, 0x6cc, 0x631, 0x627, 0x628, 0x645, 0x627,
0x6a9, 0x648, 0x628, 0x620, 0x646, 0x6af, 0x648, 0x657, 0x6cc, 0x6cc, 0x628, 0x64e,
0x646, 0x62c, 0x648, 0x657, 0x644, 0x628, 0x650, 0x633, 0x627, 0x624, 0x6ba, 0x628,
0x644, 0x627, 0x646, 0x62a, 0x64e, 0x6cc, 0x631, 0x628, 0x631, 0x627, 0x632, 0x627,
0x648, 0x650, 0x644, 0x628, 0x64f, 0x62c, 0x64f, 0x645, 0x628, 0x64f, 0x631, 0x627,
0x6a9, 0x64e, 0x6cc, 0x631, 0x648, 0x6a9, 0x620, 0x633, 0x627, 0x628, 0x644, 0x620,
0x646, 0x6a9, 0x627, 0x6a9, 0x6cc, 0x648, 0x657, 0x679, 0x627, 0x6a9, 0x648, 0x646,
0x627, 0x6a9, 0x631, 0x6cc, 0x62f, 0x64e, 0x6a9, 0x627, 0x631, 0x62f, 0x627, 0x631,
0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x688, 0x650, 0x62c, 0x6cc, 0x628, 0x648,
0x679, 0x6cc, 0x688, 0x648, 0x648, 0x644, 0x627, 0x627, 0x644, 0x20, 0x639, 0x6cc,
0x648, 0x646, 0x641, 0x631, 0x6cc, 0x20, 0x679, 0x627, 0x648, 0x64f, 0x646, 0x6af,
0x620, 0x628, 0x648, 0x631, 0x648, 0x646, 0x6c1, 0x64e, 0x631, 0x627, 0x631, 0x6cc,
0x6d2, 0x62c, 0x648, 0x6c1, 0x627, 0x646, 0x633, 0x628, 0x631, 0x6af, 0x6a9, 0x64e,
0x645, 0x67e, 0x627, 0x644, 0x627, 0x6a9, 0x6be, 0x627, 0x631, 0x62a, 0x648, 0x645,
0x6a9, 0x650, 0x6af, 0x627, 0x644, 0x6cc, 0x6a9, 0x650, 0x646, 0x634, 0x627, 0x633,
0x627, 0x644, 0x6cc, 0x6af, 0x648, 0x633, 0x644, 0x650, 0x628, 0x64e, 0x631, 0x648,
0x650, 0x644, 0x644, 0x6cc, 0x648, 0x627, 0x646, 0x688, 0x627, 0x644, 0x648, 0x628,
0x645, 0x20, 0x628, 0x627, 0x634, 0x6cc, 0x644, 0x648, 0x633, 0x627, 0x6a9, 0x627,
0x645, 0x64e, 0x67e, 0x648, 0x657, 0x679, 0x648, 0x645, 0x64e, 0x633, 0x6cc, 0x631,
0x648, 0x657, 0x645, 0x627, 0x628, 0x6cc, 0x646, 0x645, 0x648, 0x6af, 0x627, 0x62f,
0x650, 0x634, 0x648, 0x657, 0x645, 0x648, 0x646, 0x631, 0x648, 0x648, 0x650, 0x6cc,
0x627, 0x646, 0x6cc, 0x631, 0x648, 0x628, 0x6cc, 0x62c, 0x64e, 0x645, 0x6cc, 0x656,
0x646, 0x627, 0x646, 0x64e, 0x6cc, 0x645, 0x6cc, 0x6d2, 0x646, 0x648, 0x648, 0x627,
0x6a9, 0x686, 0x6be, 0x648, 0x62a, 0x627, 0x648, 0x622, 0x6af, 0x62f, 0x648, 0x6af,
0x648, 0x67e, 0x648, 0x631, 0x679, 0x648, 0x20, 0x646, 0x648, 0x648, 0x648, 0x633,
0x627, 0x648, 0x20, 0x679, 0x648, 0x645, 0x62a, 0x631, 0x67e, 0x648, 0x644, 0x6cc,
0x679, 0x648, 0x657, 0x646, 0x650, 0x633, 0x648, 0x650, 0x646, 0x688, 0x6c1, 0x648,
0x6a9, 0x627, 0x62f, 0x6a9, 0x627, 0x64e, 0x646, 0x6a9, 0x648, 0x631, 0x627, 0x62c,
0x627, 0x620, 0x646, 0x6af, 0x650, 0x648, 0x644, 0x627, 0x627, 0x620, 0x646, 0x679,
0x650, 0x6af, 0x64f, 0x648, 0x627, 0x627, 0x620, 0x631, 0x6cc, 0x6af, 0x64f, 0x648,
0x6cc, 0x646, 0x627, 0x628, 0x6cc, 0x648, 0x657, 0x646, 0x64e, 0x633, 0x20, 0x622,
0x6cc, 0x631, 0x64e, 0x633, 0x6a9, 0x6cc, 0x679, 0x627, 0x645, 0x627, 0x631, 0x6a9,
0x627, 0x6a9, 0x648, 0x631, 0x691, 0x648, 0x628, 0x627, 0x62c, 0x64f, 0x62c, 0x648,
0x6cc, 0x6d2, 0x644, 0x627, 0x20, 0x631, 0x650, 0x6cc, 0x648, 0x62c, 0x627, 0x645,
0x620, 0x646, 0x691, 0x648, 0x632, 0x627, 0x631, 0x650, 0x6cc, 0x648, 0x20, 0x6af,
0x627, 0x644, 0x6cc, 0x6af, 0x648, 0x633, 0x633, 0x627, 0x644, 0x679, 0x627, 0x633,
0x6cc, 0x646, 0x20, 0x62c, 0x64f, 0x648, 0x627, 0x6ba, 0x633, 0x6cc, 0x646, 0x20,
0x644, 0x648, 0x657, 0x6cc, 0x650, 0x633, 0x679, 0x648, 0x6a9, 0x648, 0x657, 0x645,
0x64e, 0x646, 0x627, 0x64f, 0x634, 0x64f, 0x648, 0x657, 0x627, 0x6cc, 0x6cc, 0x627,
0x627, 0x64e, 0x631, 0x648, 0x657, 0x628, 0x627, 0x627, 0x64e, 0x633, 0x64f, 0x646,
0x686, 0x650, 0x6cc, 0x64e, 0x646, 0x627, 0x679, 0x6cc, 0x20, 0x6a9, 0x648, 0x6a9,
0x646, 0x64d, 0x628, 0x64e, 0x6c1, 0x650, 0x6cc, 0x627, 0x628, 0x627, 0x6c1, 0x6cc,
0x627, 0x20, 0x628, 0x646, 0x62f, 0x627, 0x631, 0x633, 0x628, 0x627, 0x631, 0x628,
0x627, 0x688, 0x648, 0x633, 0x628, 0x620, 0x644, 0x620, 0x645, 0x628, 0x620, 0x644,
0x6cc, 0x656, 0x632, 0x628, 0x644, 0x627, 0x646, 0x6a9, 0x20, 0x633, 0x620, 0x628,
0x644, 0x64e, 0x646, 0x628, 0x648, 0x622, 0x20, 0x648, 0x650, 0x633, 0x679, 0x627,
0x628, 0x648, 0x6af, 0x648, 0x679, 0x627, 0x628, 0x648, 0x6cc, 0x650, 0x633, 0x6a9,
0x6cc, 0x645, 0x628, 0x631, 0x650, 0x62c, 0x20, 0x62e, 0x644, 0x6cc, 0x656, 0x62c,
0x6a9, 0x620, 0x645, 0x67e, 0x648, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x691, 0x6a9,
0x6cc, 0x646, 0x6a9, 0x64e, 0x646, 0x6a9, 0x6cc, 0x631, 0x64e, 0x6a9, 0x64e, 0x633,
0x6a9, 0x64e, 0x6cc, 0x6cc, 0x646, 0x6a9, 0x6cc, 0x645, 0x64e, 0x646, 0x634, 0x650,
0x6a9, 0x627, 0x6af, 0x648, 0x686, 0x650, 0x6c1, 0x64f, 0x648, 0x627, 0x20, 0x6c1,
0x64f, 0x648, 0x627, 0x6a9, 0x648, 0x633, 0x679, 0x627, 0x20, 0x631, 0x6cc, 0x656,
0x6a9, 0x627, 0x6a9, 0x631, 0x6cc, 0x633, 0x679, 0x646, 0x6a9, 0x648, 0x657, 0x6cc,
0x627, 0x628, 0x627, 0x6a9, 0x6cc, 0x648, 0x657, 0x631, 0x627, 0x6a9, 0x627, 0x627,
0x648, 0x688, 0x646, 0x645, 0x627, 0x631, 0x6a9, 0x20, 0x634, 0x627, 0x648, 0x646,
0x688, 0x627, 0x648, 0x633, 0x646, 0x688, 0x648, 0x627, 0x633, 0x64e, 0x646, 0x20,
0x6a9, 0x631, 0x6cc, 0x656, 0x6a9, 0x688, 0x6cc, 0x646, 0x648, 0x64e, 0x631, 0x688,
0x6cc, 0x679, 0x631, 0x627, 0x6cc, 0x650, 0x679, 0x688, 0x648, 0x645, 0x650, 0x646,
0x650, 0x6a9, 0x627, 0x627, 0x620, 0x688, 0x645, 0x64e, 0x646, 0x679, 0x64e, 0x646,
0x627, 0x6cc, 0x656, 0x631, 0x648, 0x657, 0x646, 0x6cc, 0x67e, 0x627, 0x6cc, 0x644,
0x20, 0x633, 0x64e, 0x644, 0x648, 0x6cc, 0x62f, 0x64e, 0x631, 0x641, 0x648, 0x631,
0x679, 0x20, 0x646, 0x6cc, 0x644, 0x633, 0x646, 0x641, 0x648, 0x631, 0x679, 0x6cc,
0x644, 0x6cc, 0x632, 0x627, 0x6af, 0x644, 0x6cc, 0x633, 0x20, 0x62e, 0x644, 0x6cc,
0x656, 0x62c, 0x6af, 0x648, 0x657, 0x633, 0x20, 0x62e, 0x644, 0x6cc, 0x656, 0x62c,
0x6af, 0x631, 0x6cc, 0x646, 0x688, 0x20, 0x62a, 0x654, 0x631, 0x6a9, 0x6af, 0x631,
0x6cc, 0x646, 0x627, 0x688, 0x627, 0x6af, 0x648, 0x691, 0x644, 0x648, 0x67e, 0x6af,
0x648, 0x627, 0x679, 0x6cc, 0x645, 0x627, 0x644, 0x627, 0x6af, 0x648, 0x627, 0x6cc,
0x627, 0x6a9, 0x650, 0x648, 0x64e, 0x644, 0x6af, 0x64f, 0x6cc, 0x627, 0x646, 0x627,
0x62d, 0x6cc, 0x644, 0x650, 0x641, 0x620, 0x6a9, 0x633, 0x62d, 0x648, 0x627, 0x646,
0x627, 0x6c2, 0x631, 0x645, 0x648, 0x633, 0x650, 0x644, 0x648, 0x627, 0x650, 0x646,
0x688, 0x6cc, 0x64e, 0x646, 0x20, 0x67e, 0x648, 0x644, 0x650, 0x633, 0x646, 0x648,
0x6a9, 0x633, 0x645, 0x6cc, 0x631, 0x646, 0x6af, 0x648, 0x60c, 0x20, 0x627, 0x646,
0x688, 0x6cc, 0x627, 0x646, 0x627, 0x67e, 0x650, 0x679, 0x633, 0x20, 0x628, 0x654,
0x631, 0x6af, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x679,
0x6cc, 0x644, 0x20, 0x633, 0x650, 0x679, 0x6cc, 0x60c, 0x20, 0x627, 0x646, 0x688,
0x6cc, 0x627, 0x646, 0x627, 0x648, 0x6cc, 0x648, 0x6cc, 0x6d2, 0x60c, 0x20, 0x627,
0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x650, 0x646, 0x633, 0x6cc, 0x646,
0x6cc, 0x633, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x648,
0x650, 0x646, 0x6cc, 0x645, 0x6cc, 0x6a9, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc,
0x627, 0x646, 0x627, 0x627, 0x650, 0x646, 0x648, 0x657, 0x648, 0x650, 0x6a9, 0x627,
0x650, 0x642, 0x627, 0x644, 0x6cc, 0x648, 0x657, 0x6cc, 0x650, 0x62a, 0x62c, 0x64e,
0x645, 0x64e, 0x6cc, 0x6a9, 0x627, 0x62c, 0x648, 0x657, 0x646, 0x6cc, 0x644, 0x648,
0x657, 0x6cc, 0x650, 0x633, 0x20, 0x648, 0x650, 0x644, 0x645, 0x648, 0x646, 0x679,
0x650, 0x633, 0x6cc, 0x644, 0x648, 0x60c, 0x20, 0x6a9, 0x6cc, 0x646, 0x679, 0x648,
0x6a9, 0x6cc, 0x6a9, 0x631, 0x627, 0x644, 0x6cc, 0x646, 0x688, 0x650, 0x6a9, 0x644,
0x627, 0x20, 0x67e, 0x627, 0x632, 0x644, 0x650, 0x645, 0x627, 0x644, 0x627, 0x633,
0x20, 0x627, 0x6cc, 0x646, 0x62c, 0x655, 0x644, 0x632, 0x644, 0x648, 0x648, 0x631,
0x20, 0x67e, 0x631, 0x646, 0x633, 0x20, 0x6a9, 0x648, 0x627, 0x631, 0x679, 0x631,
0x645, 0x6cc, 0x633, 0x650, 0x6cc, 0x648, 0x648, 0x645, 0x64e, 0x646, 0x627, 0x6af,
0x648, 0x627, 0x645, 0x64e, 0x646, 0x648, 0x633, 0x645, 0x6cc, 0x631, 0x6cc, 0x6af,
0x648, 0x679, 0x645, 0x627, 0x631, 0x679, 0x650, 0x646, 0x650, 0x6a9, 0x645, 0x6cc,
0x679, 0x627, 0x645, 0x648, 0x631, 0x648, 0x633, 0x645, 0x64e, 0x632, 0x64e, 0x679,
0x644, 0x627, 0x646, 0x645, 0x6cc, 0x646, 0x648, 0x645, 0x650, 0x646, 0x6cc, 0x645,
0x6cc, 0x631, 0x650, 0x688, 0x627, 0x645, 0x6cc, 0x679, 0x644, 0x627, 0x20, 0x6a9,
0x627, 0x679, 0x644, 0x627, 0x645, 0x6cc, 0x6a9, 0x633, 0x650, 0x6a9, 0x648, 0x20,
0x633, 0x650, 0x679, 0x6cc, 0x645, 0x650, 0x6a9, 0x6cc, 0x656, 0x644, 0x64e, 0x646,
0x645, 0x648, 0x646, 0x6a9, 0x679, 0x655, 0x646, 0x645, 0x648, 0x679, 0x6cc, 0x631,
0x6cc, 0x645, 0x648, 0x646, 0x679, 0x6cc, 0x648, 0x650, 0x688, 0x6cc, 0x648, 0x645,
0x648, 0x646, 0x698, 0x6cc, 0x631, 0x627, 0x62a, 0x646, 0x633, 0x627, 0x624, 0x6ba,
0x646, 0x650, 0x648, 0x20, 0x6cc, 0x627, 0x631, 0x6a9, 0x646, 0x648, 0x631, 0x648,
0x646, 0x6c1, 0x627, 0x628, 0x6cc, 0x648, 0x644, 0x627, 0x6c1, 0x60c, 0x20, 0x634,
0x64f, 0x645, 0x627, 0x644, 0x6cc, 0x20, 0x688, 0x6cc, 0x6a9, 0x648, 0x679, 0x627,
0x645, 0x64e, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x62c, 0x646, 0x648, 0x657, 0x628,
0x6cc, 0x20, 0x688, 0x6a9, 0x648, 0x679, 0x627, 0x646, 0x648, 0x648, 0x20, 0x633,
0x6cc, 0x644, 0x655, 0x645, 0x60c, 0x20, 0x634, 0x64f, 0x645, 0x627, 0x644, 0x6cc,
0x20, 0x688, 0x6cc, 0x6a9, 0x648, 0x679, 0x627, 0x646, 0x648, 0x657, 0x6a9, 0x627,
0x648, 0x62c, 0x6cc, 0x20, 0x646, 0x627, 0x6af, 0x627, 0x67e, 0x64e, 0x646, 0x627,
0x645, 0x627, 0x67e, 0x64e, 0x631, 0x627, 0x645, 0x627, 0x631, 0x650, 0x628, 0x648,
0x67e, 0x6be, 0x650, 0x646, 0x650, 0x6a9, 0x633, 0x67e, 0x648, 0x679, 0x20, 0x622,
0x624, 0x6ba, 0x20, 0x67e, 0x631, 0x650, 0x646, 0x633, 0x67e, 0x648, 0x679, 0x20,
0x622, 0x641, 0x20, 0x633, 0x67e, 0x6cc, 0x646, 0x67e, 0x648, 0x679, 0x648, 0x20,
0x648, 0x620, 0x644, 0x6c1, 0x648, 0x67e, 0x6cc, 0x656, 0x679, 0x648, 0x20, 0x631,
0x650, 0x6a9, 0x648, 0x67e, 0x648, 0x646, 0x679, 0x627, 0x20, 0x627, 0x631, 0x6cc,
0x646, 0x627, 0x633, 0x631, 0x6cc, 0x646, 0x6a9, 0x650, 0x646, 0x20, 0x627, 0x650,
0x646, 0x644, 0x620, 0x679, 0x631, 0x6cc, 0x633, 0x6cc, 0x641, 0x631, 0x620, 0x62c,
0x6cc, 0x656, 0x646, 0x627, 0x631, 0x650, 0x633, 0x648, 0x644, 0x6cc, 0x648, 0x657,
0x679, 0x631, 0x650, 0x6cc, 0x648, 0x20, 0x628, 0x631, 0x627, 0x646, 0x6a9, 0x648,
0x633, 0x627, 0x646, 0x62a, 0x627, 0x631, 0x6cc, 0x645, 0x633, 0x6cc, 0x646, 0x679,
0x6cc, 0x627, 0x6af, 0x648, 0x633, 0x620, 0x646, 0x679, 0x648, 0x20, 0x691, 0x648,
0x645, 0x650, 0x646, 0x6af, 0x648, 0x633, 0x627, 0x624, 0x20, 0x67e, 0x627, 0x644,
0x648, 0x633, 0x6a9, 0x648, 0x631, 0x655, 0x633, 0x628, 0x627, 0x6d2, 0x633, 0x64e,
0x646, 0x691, 0x633, 0x650, 0x679, 0x6a9, 0x627, 0x633, 0x6cc, 0x646, 0x679, 0x20,
0x628, 0x627, 0x631, 0x62a, 0x6be, 0x6cc, 0x644, 0x645, 0x6cc, 0x633, 0x620, 0x646,
0x679, 0x20, 0x62c, 0x648, 0x646, 0x633, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x6a9,
0x650, 0x679, 0x633, 0x633, 0x620, 0x646, 0x679, 0x20, 0x644, 0x648, 0x657, 0x633,
0x650, 0x6cc, 0x627, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x62a, 0x6be, 0x627, 0x645,
0x64e, 0x633, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x648, 0x650, 0x646, 0x633, 0x6cc,
0x646, 0x679, 0x633, 0x655, 0x648, 0x650, 0x641, 0x679, 0x20, 0x6a9, 0x64e, 0x631,
0x64e, 0x646, 0x679, 0x679, 0x6cc, 0x6af, 0x648, 0x633, 0x6cc, 0x20, 0x6af, 0x627,
0x644, 0x67e, 0x627, 0x62a, 0x6be, 0x6cc, 0x648, 0x657, 0x644, 0x6d2, 0x62a, 0x650,
0x62c, 0x64f, 0x648, 0x627, 0x646, 0x627, 0x679, 0x648, 0x631, 0x648, 0x646, 0x679,
0x648, 0x679, 0x627, 0x631, 0x679, 0x648, 0x644, 0x627, 0x648, 0x620, 0x646, 0x6a9,
0x648, 0x648, 0x64e, 0x631, 0x648, 0x627, 0x6cc, 0x650, 0x679, 0x20, 0x6c1, 0x627,
0x631, 0x655, 0x633, 0x648, 0x650, 0x646, 0x650, 0x67e, 0x620, 0x6af, 0x6cc, 0x6a9,
0x648, 0x657, 0x62a, 0x627, 0x62a, 0x6a9, 0x6cc, 0x633, 0x6cc, 0x688, 0x6cc, 0x648,
0x650, 0x633, 0x688, 0x64f, 0x645, 0x648, 0x646, 0x679, 0x20, 0x688, 0x20, 0x627,
0x64e, 0x631, 0x648, 0x650, 0x644, 0x645, 0x627, 0x633, 0x64e, 0x646, 0x645, 0x620,
0x6a9, 0x20, 0x645, 0x672, 0x691, 0x648, 0x67e, 0x627, 0x645, 0x64e, 0x631, 0x631,
0x648, 0x62a, 0x6be, 0x6cc, 0x631, 0x627, 0x633, 0x6cc, 0x648, 0x648, 0x627, 0x648,
0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x644, 0x648, 0x646, 0x6af, 0x20, 0x6cc, 0x6cc,
0x626, 0x631, 0x20, 0x628, 0x626, 0x6cc, 0x646, 0x627, 0x6cc, 0x688, 0x655, 0x646,
0x627, 0x64e, 0x644, 0x645, 0x627, 0x679, 0x6cc, 0x627, 0x64e, 0x645, 0x627, 0x646,
0x627, 0x64e, 0x646, 0x64e, 0x62f, 0x6cc, 0x631, 0x627, 0x6a9, 0x679, 0x648, 0x627,
0x64e, 0x642, 0x679, 0x648, 0x628, 0x627, 0x64e, 0x634, 0x6af, 0x64e, 0x628, 0x64e,
0x62a, 0x627, 0x679, 0x6cc, 0x631, 0x648, 0x628, 0x620, 0x6c1, 0x631, 0x6cc, 0x656,
0x646, 0x628, 0x627, 0x642, 0x648, 0x657, 0x628, 0x6cc, 0x646, 0x6af, 0x20, 0x6a9,
0x627, 0x6a9, 0x628, 0x631, 0x646, 0x648, 0x644, 0x628, 0x6cc, 0x631, 0x679, 0x628,
0x650, 0x634, 0x6a9, 0x6cc, 0x6a9, 0x628, 0x631, 0x648, 0x657, 0x646, 0x64e, 0x6d2,
0x686, 0x6cc, 0x679, 0x627, 0x686, 0x648, 0x6cc, 0x628, 0x627, 0x644, 0x633, 0x64e,
0x646, 0x6a9, 0x648, 0x644, 0x64e, 0x645, 0x628, 0x648, 0x62f, 0x64e, 0x645, 0x64e,
0x633, 0x6a9, 0x64e, 0x633, 0x688, 0x6be, 0x627, 0x6a9, 0x627, 0x62f, 0x650, 0x644,
0x6cc, 0x62f, 0x64f, 0x628, 0x626, 0x6cc, 0x62f, 0x64f, 0x634, 0x627, 0x646, 0x628,
0x6cc, 0x6d2, 0x641, 0x627, 0x645, 0x627, 0x20, 0x6af, 0x648, 0x633, 0x679, 0x627,
0x63a, 0x632, 0x6c1, 0x6c1, 0x6cc, 0x628, 0x631, 0x648, 0x646, 0x633, 0x6cc, 0x6af,
0x64e, 0x646, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af, 0x62d,
0x648, 0x648, 0x691, 0x627, 0x650, 0x631, 0x6a9, 0x64f, 0x633, 0x6a9, 0x62c, 0x6a9,
0x627, 0x631, 0x62a, 0x627, 0x62c, 0x64e, 0x6cc, 0x627, 0x67e, 0x648, 0x657, 0x631,
0x627, 0x6cc, 0x631, 0x648, 0x634, 0x644, 0x645, 0x642, 0x627, 0x628, 0x64f, 0x644,
0x6a9, 0x64e, 0x645, 0x686, 0x6be, 0x679, 0x6a9, 0x627, 0x6a9, 0x64e, 0x631, 0x627,
0x686, 0x6cc, 0x6a9, 0x627, 0x679, 0x6be, 0x645, 0x64e, 0x646, 0x688, 0x648, 0x657,
0x6a9, 0x6be, 0x627, 0x646, 0x688, 0x6cc, 0x6af, 0x627, 0x6a9, 0x64e, 0x644, 0x655,
0x6a9, 0x64e, 0x62a, 0x627, 0x6a9, 0x631, 0x646, 0x633, 0x646, 0x648, 0x6cc, 0x627,
0x631, 0x633, 0x6a9, 0x6a9, 0x648, 0x644, 0x627, 0x644, 0x64e, 0x645, 0x67e, 0x648,
0x657, 0x631, 0x6a9, 0x64f, 0x686, 0x650, 0x646, 0x6af, 0x6a9, 0x64f, 0x648, 0x6cc,
0x62a, 0x645, 0x6a9, 0x648, 0x645, 0x64e, 0x6af, 0x627, 0x62f, 0x64e, 0x646, 0x645,
0x64e, 0x6a9, 0x64e, 0x633, 0x64e, 0x631, 0x645, 0x64e, 0x646, 0x6cc, 0x656, 0x644,
0x627, 0x646, 0x650, 0x6a9, 0x648, 0x633, 0x650, 0x6cc, 0x627, 0x646, 0x648, 0x648,
0x648, 0x6a9, 0x64f, 0x632, 0x646, 0x6cc, 0x679, 0x633, 0x6a9, 0x646, 0x648, 0x648,
0x648, 0x633, 0x650, 0x628, 0x650, 0x631, 0x633, 0x6a9, 0x627, 0x648, 0x645, 0x633,
0x6a9, 0x627, 0x648, 0x631, 0x64e, 0x644, 0x646, 0x648, 0x645, 0x20, 0x67e, 0x620,
0x646, 0x6c1, 0x6c1, 0x67e, 0x648, 0x646, 0x62a, 0x650, 0x639, 0x627, 0x646, 0x6a9,
0x67e, 0x6cc, 0x648, 0x646, 0x6af, 0x6cc, 0x627, 0x646, 0x6af, 0x6a9, 0x648, 0x633,
0x679, 0x627, 0x646, 0x6d2, 0x642, 0x620, 0x632, 0x620, 0x644, 0x648, 0x691, 0x627,
0x631, 0x6cc, 0x627, 0x636, 0x633, 0x64e, 0x6a9, 0x6be, 0x627, 0x644, 0x650, 0x646,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x6a9, 0x64e, 0x646, 0x62f, 0x633, 0x6cc, 0x648,
0x644, 0x634, 0x627, 0x646, 0x6af, 0x6d2, 0x633, 0x650, 0x646, 0x6af, 0x627, 0x67e,
0x648, 0x631, 0x633, 0x631, 0x6cc, 0x688, 0x646, 0x6cc, 0x6a9, 0x648, 0x644, 0x6cc,
0x645, 0x633, 0x6a9, 0x62a, 0x64e, 0x6cc, 0x67e, 0x6cc, 0x6d2, 0x62a, 0x627, 0x634,
0x6a9, 0x6cc, 0x646, 0x679, 0x628, 0x650, 0x644, 0x650, 0x633, 0x6cc, 0x62a, 0x620,
0x6c1, 0x631, 0x627, 0x646, 0x62a, 0x6be, 0x650, 0x645, 0x67e, 0x648, 0x657, 0x679,
0x648, 0x6a9, 0x6cc, 0x648, 0x679, 0x648, 0x645, 0x633, 0x6a9, 0x627, 0x648, 0x644,
0x646, 0x20, 0x628, 0x627, 0x679, 0x631, 0x627, 0x64f, 0x631, 0x648, 0x645, 0x642,
0x6cc, 0x627, 0x648, 0x633, 0x679, 0x2d, 0x646, 0x6cc, 0x631, 0x627, 0x648, 0x650,
0x6cc, 0x64e, 0x646, 0x62a, 0x650, 0x6cc, 0x6cc, 0x646, 0x644, 0x627, 0x62f, 0x650,
0x648, 0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x6cc, 0x6a9, 0x648, 0x633, 0x6a9, 0x631,
0x646, 0x6af, 0x648, 0x657, 0x646, 0x6cc, 0x64e, 0x6a9, 0x627, 0x62a, 0x650, 0x631,
0x650, 0x646, 0x628, 0x654, 0x631, 0x6af, 0x6cc, 0x6d2, 0x631, 0x6cc, 0x648, 0x64e,
0x646, 0x627, 0x64e, 0x632, 0x648, 0x631, 0x64e, 0x633, 0x628, 0x631, 0x645, 0x648,
0x657, 0x691, 0x627, 0x6a9, 0x620, 0x646, 0x64e, 0x631, 0x6cc, 0x6a9, 0x6cc, 0x67e,
0x20, 0x624, 0x631, 0x691, 0x6d2, 0x641, 0x620, 0x631, 0x648, 0x645, 0x64e, 0x688,
0x6cc, 0x656, 0x631, 0x627, 0x631, 0x620, 0x6a9, 0x6cc, 0x627, 0x648, 0x650, 0x6a9,
0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627,
0x633, 0x6cc, 0x646, 0x679, 0x20, 0x6c1, 0x6cc, 0x644, 0x650, 0x646, 0x627, 0x633,
0x679, 0x6cc, 0x646, 0x644, 0x6cc, 0x627, 0x620, 0x691, 0x650, 0x644, 0x6cc, 0x691,
0x628, 0x631, 0x633, 0x628, 0x6cc, 0x646, 0x628, 0x631, 0x648, 0x6a9, 0x655, 0x646,
0x20, 0x6c1, 0x650, 0x644, 0x688, 0x627, 0x631, 0x648, 0x650, 0x646, 0x6cc, 0x648,
0x657, 0x6a9, 0x644, 0x627, 0x62d, 0x6c4, 0x628, 0x672, 0x679, 0x644, 0x650, 0x646,
0x691, 0x6cc, 0x645, 0x627, 0x646, 0x644, 0x639, 0x627, 0x691, 0x20, 0x6c1, 0x648,
0x648, 0x6d2, 0x645, 0x620, 0x644, 0x628, 0x639, 0x627, 0x631, 0x646, 0x67e, 0x654,
0x631, 0x62a, 0x6be, 0x633, 0x650, 0x691, 0x646, 0x6cc, 0x627, 0x6cc, 0x645, 0x633,
0x679, 0x64e, 0x631, 0x688, 0x6cc, 0x645, 0x627, 0x64e, 0x646, 0x688, 0x648, 0x631,
0x627, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x627, 0x20, 0x6a9, 0x6be, 0x627, 0x646,
0x627, 0x620, 0x62a, 0x6be, 0x655, 0x646, 0x633, 0x628, 0x6cc, 0x644, 0x6af, 0x631,
0x6cc, 0x688, 0x628, 0x654, 0x631, 0x644, 0x650, 0x646, 0x628, 0x631, 0x6cc, 0x679,
0x6cc, 0x633, 0x644, 0x627, 0x648, 0x627, 0x628, 0x631, 0x633, 0x655, 0x644, 0x633,
0x628, 0x64e, 0x686, 0x627, 0x631, 0x6cc, 0x633, 0x679, 0x628, 0x64f, 0x688, 0x627,
0x67e, 0x6cc, 0x633, 0x679, 0x628, 0x648, 0x633, 0x650, 0x646, 0x6af, 0x6cc, 0x646,
0x686, 0x650, 0x633, 0x6cc, 0x656, 0x646, 0x648, 0x6a9, 0x648, 0x67e, 0x646, 0x20,
0x6c1, 0x6cc, 0x6af, 0x646, 0x688, 0x64e, 0x628, 0x644, 0x650, 0x646, 0x6af, 0x650,
0x628, 0x631, 0x627, 0x644, 0x679, 0x64e, 0x631, 0x6af, 0x648, 0x6cc, 0x631, 0x646,
0x633, 0x6d2, 0x62d, 0x620, 0x644, 0x633, 0x650, 0x646, 0x6a9, 0x6cc, 0x622, 0x626,
0x644, 0x20, 0x622, 0x641, 0x20, 0x645, 0x6cc, 0x646, 0x627, 0x650, 0x633, 0x62a,
0x627, 0x646, 0x628, 0x64f, 0x644, 0x62c, 0x631, 0x633, 0x6cc, 0x6a9, 0x64e, 0x644,
0x650, 0x646, 0x627, 0x650, 0x646, 0x6af, 0x631, 0x64e, 0x62f, 0x6a9, 0x6cc, 0x631,
0x648, 0x6a9, 0x6cc, 0x656, 0x648, 0x644, 0x650, 0x633, 0x628, 0x64e, 0x646, 0x644,
0x650, 0x6cc, 0x648, 0x657, 0x628, 0x20, 0x644, 0x650, 0x6cc, 0x627, 0x646, 0x627,
0x644, 0x64e, 0x646, 0x62f, 0x64e, 0x646, 0x644, 0x64e, 0x6a9, 0x632, 0x655, 0x645,
0x628, 0x654, 0x631, 0x6af, 0x645, 0x6cc, 0x688, 0x631, 0x6cc, 0x688, 0x645, 0x627,
0x644, 0x679, 0x627, 0x645, 0x6cc, 0x631, 0x6cc, 0x20, 0x6c1, 0x6cc, 0x645, 0x645,
0x650, 0x646, 0x633, 0x6a9, 0x645, 0x648, 0x646, 0x627, 0x6a9, 0x648, 0x645, 0x627,
0x633, 0x6a9, 0x648, 0x627, 0x648, 0x633, 0x644, 0x648, 0x67e, 0x6cc, 0x631, 0x650,
0x633, 0x67e, 0x648, 0x688, 0x6af, 0x648, 0x631, 0x6cc, 0x6a9, 0x627, 0x67e, 0x631,
0x627, 0x6af, 0x631, 0x650, 0x6af, 0x627, 0x633, 0x64e, 0x645, 0x627, 0x631, 0x627,
0x633, 0x6cc, 0x646, 0x20, 0x645, 0x631, 0x6cc, 0x646, 0x648, 0x633, 0x627, 0x631,
0x627, 0x20, 0x62c, 0x6cc, 0x648, 0x648, 0x633, 0x627, 0x631, 0x627, 0x679, 0x648,
0x633, 0x650, 0x645, 0x641, 0x6cc, 0x631, 0x648, 0x67e, 0x648, 0x644, 0x633, 0x650,
0x6a9, 0x648, 0x67e, 0x6cc, 0x6d2, 0x633, 0x648, 0x641, 0x650, 0x6cc, 0x627, 0x633,
0x679, 0x627, 0x6a9, 0x20, 0x6c1, 0x648, 0x644, 0x645, 0x679, 0x620, 0x644, 0x650,
0x646, 0x679, 0x650, 0x631, 0x6cc, 0x646, 0x627, 0x648, 0x644, 0x6cc, 0x627, 0x646,
0x648, 0x648, 0x633, 0x6a9, 0x648, 0x627, 0x62f, 0x64f, 0x632, 0x648, 0x6cc, 0x679,
0x6cc, 0x6a9, 0x646, 0x648, 0x650, 0x6cc, 0x64e, 0x646, 0x646, 0x627, 0x648, 0x650,
0x644, 0x650, 0x646, 0x650, 0x6cc, 0x64e, 0x633, 0x648, 0x648, 0x644, 0x6af, 0x648,
0x6af, 0x631, 0x64e, 0x62f, 0x648, 0x627, 0x631, 0x633, 0x627, 0x632, 0x6af, 0x631,
0x6cc, 0x628, 0x632, 0x6cc, 0x648, 0x657, 0x631, 0x650, 0x6a9, 0x627, 0x620, 0x646,
0x679, 0x646, 0x627, 0x646, 0x631, 0x650, 0x648, 0x648, 0x686, 0x627, 0x6af, 0x648,
0x633, 0x6a9, 0x631, 0x650, 0x633, 0x645, 0x64e, 0x633, 0x6a9, 0x648, 0x6a9, 0x648,
0x633, 0x6a9, 0x648, 0x645, 0x648, 0x631, 0x648, 0x6a9, 0x6cc, 0x631, 0x6af, 0x648,
0x644, 0x650, 0x646, 0x645, 0x627, 0x6c1, 0x6cc, 0x6d2, 0x645, 0x627, 0x644, 0x62f,
0x6cc, 0x656, 0x648, 0x632, 0x645, 0x648, 0x631, 0x650, 0x634, 0x6cc, 0x633, 0x645,
0x6cc, 0x648, 0x679, 0x631, 0x650, 0x6cc, 0x648, 0x657, 0x646, 0x6cc, 0x64e, 0x646,
0x622, 0x67e, 0x650, 0x6cc, 0x627, 0x622, 0x6a9, 0x644, 0x6cc, 0x646, 0x688, 0x686,
0x64e, 0x62a, 0x6be, 0x64e, 0x645, 0x679, 0x631, 0x6a9, 0x627, 0x6cc, 0x656, 0x633,
0x679, 0x64e, 0x631, 0x627, 0x6cc, 0x641, 0x627, 0x62a, 0x6cc, 0x6d2, 0x641, 0x64e,
0x6a9, 0x64e, 0x648, 0x641, 0x648, 0x641, 0x650, 0x62c, 0x6cc, 0x641, 0x64f, 0x646,
0x627, 0x641, 0x648, 0x657, 0x62a, 0x6cc, 0x6af, 0x620, 0x644, 0x64e, 0x67e, 0x6af,
0x648, 0x633, 0x6af, 0x6cc, 0x645, 0x628, 0x650, 0x6cc, 0x64e, 0x631, 0x6af, 0x64f,
0x648, 0x627, 0x62f, 0x64e, 0x644, 0x686, 0x64e, 0x646, 0x64e, 0x644, 0x6c1, 0x648,
0x646, 0x648, 0x644, 0x648, 0x20, 0x644, 0x648, 0x627, 0x620, 0x646, 0x691, 0x631,
0x628, 0x6cc, 0x631, 0x6cc, 0x6a9, 0x650, 0x631, 0x650, 0x62a, 0x650, 0x645, 0x627,
0x62a, 0x6cc, 0x6a9, 0x648, 0x633, 0x631, 0x627, 0x6d2, 0x6a9, 0x64f, 0x648, 0x62c,
0x644, 0x6cc, 0x646, 0x645, 0x64e, 0x62c, 0x648, 0x657, 0x631, 0x648, 0x645, 0x627,
0x631, 0x6a9, 0x6cc, 0x633, 0x64e, 0x633, 0x645, 0x650, 0x688, 0x648, 0x6cc, 0x6d2,
0x646, 0x627, 0x648, 0x631, 0x648, 0x657, 0x646, 0x648, 0x631, 0x641, 0x648, 0x6a9,
0x646, 0x648, 0x645, 0x6cc, 0x6cc, 0x67e, 0x6cc, 0x6af, 0x648, 0x20, 0x67e, 0x6cc,
0x6af, 0x648, 0x67e, 0x64e, 0x644, 0x627, 0x648, 0x67e, 0x650, 0x679, 0x6a9, 0x6cc,
0x631, 0x64e, 0x646, 0x67e, 0x648, 0x646, 0x627, 0x67e, 0x6d2, 0x67e, 0x648, 0x679,
0x20, 0x645, 0x648, 0x631, 0x633, 0x628, 0x6cc, 0x631, 0x627, 0x631, 0x648, 0x679,
0x648, 0x646, 0x6af, 0x627, 0x633, 0x64e, 0x6cc, 0x67e, 0x627, 0x646, 0x62a, 0x64e,
0x6c1, 0x650, 0x62a, 0x6cc, 0x679, 0x64e, 0x631, 0x627, 0x648, 0x627, 0x62a, 0x648,
0x646, 0x6af, 0x627, 0x62a, 0x64e, 0x67e, 0x648, 0x657, 0x648, 0x6cc, 0x6a9, 0x648,
0x6cc, 0x644, 0x650, 0x633, 0x423, 0x448, 0x443, 0x430, 0x439, 0x44f, 0x41a, 0x435,
0x43c, 0x431, 0x440, 0x438, 0x434, 0x436, 0x2d, 0x411, 0x435, 0x439, 0x41a, 0x430,
0x439, 0x43c, 0x430, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440,
0x44b, 0x421, 0x44c, 0x44e, 0x434, 0x430, 0x434, 0x2d, 0x425, 0x443, 0x430, 0x440,
0x435, 0x441, 0x414, 0x43e, 0x441, 0x43e, 0x43d, 0x2d, 0x41a, 0x440, 0x438, 0x43a,
0x424, 0x43e, 0x440, 0x442, 0x2d, 0x41d, 0x435, 0x43b, 0x44c, 0x441, 0x43e, 0x43d,
0x412, 0x438, 0x432, 0x435, 0x439, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430,
0x43d, 0x430, 0x411, 0x43e, 0x439, 0x43b, 0x430, 0x2c, 0x20, 0x421, 0x43e, 0x43b,
0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442,
0x430, 0x421, 0x435, 0x43d, 0x442, 0x435, 0x440, 0x2c, 0x20, 0x421, 0x43e, 0x43b,
0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442,
0x430, 0x41d, 0x44c, 0x44e, 0x20, 0x421, 0x435, 0x439, 0x43b, 0x435, 0x43c, 0x2c,
0x20, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x414,
0x430, 0x43a, 0x43e, 0x442, 0x430, 0x418, 0x43b, 0x43b, 0x43e, 0x43a, 0x43a, 0x43e,
0x440, 0x442, 0x43e, 0x43e, 0x440, 0x43c, 0x438, 0x443, 0x442, 0x421, 0x443, 0x438,
0x444, 0x442, 0x2d, 0x41a, 0x430, 0x440, 0x440, 0x435, 0x43d, 0x442, 0x414, 0x44d,
0x439, 0x432, 0x438, 0x441, 0x410, 0x49b, 0x442, 0x430, 0x443, 0x410, 0x49b, 0x442,
0x4e9, 0x431, 0x435, 0x411, 0x430, 0x493, 0x434, 0x430, 0x442, 0x411, 0x430, 0x440,
0x43d, 0x430, 0x443, 0x44b, 0x43b, 0x41d, 0x438, 0x43a, 0x43e, 0x441, 0x438, 0x44f,
0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x456, 0x431, 0x456, 0x440, 0x41e, 0x43c, 0x431,
0x44b, 0x49a, 0x43e, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x439, 0x49a, 0x44b, 0x437,
0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x49b,
0x430, 0x43d, 0x434, 0x4b0, 0x43b, 0x430, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x440,
0x4ae, 0x440, 0x456, 0x43c, 0x448, 0x456, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x430,
0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x411, 0x435, 0x440, 0x43c, 0x443,
0x434, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x41a, 0x430,
0x43d, 0x430, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b,
0x424, 0x430, 0x440, 0x435, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x41e, 0x4a3, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x413,
0x435, 0x43e, 0x440, 0x433, 0x438, 0x44f, 0x4d8, 0x443, 0x43b, 0x438, 0x435, 0x20,
0x415, 0x43b, 0x435, 0x43d, 0x430, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x41c,
0x44d, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x421, 0x442, 0x430, 0x43c,
0x431, 0x4b1, 0x43b, 0x41c, 0x4d9, 0x441, 0x43a, 0x435, 0x443, 0x420, 0x43e, 0x436,
0x434, 0x435, 0x441, 0x442, 0x432, 0x43e, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b,
0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x41a, 0x43e, 0x43c, 0x43e, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x434, 0x430, 0x440, 0x44b, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x41f, 0x430, 0x441, 0x445,
0x430, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x41a, 0x443, 0x441, 0x430, 0x438,
0x435, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x434, 0x430, 0x440, 0x44b, 0x17a2, 0x17b6, 0x1794, 0x17ca, 0x17b8, 0x178a, 0x17d2, 0x1799,
0x17b6, 0x1793, 0x17a2, 0x17b6, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x17a2, 0x17b6, 0x178c, 0x17b8,
0x179f, 0x17a2, 0x17b6, 0x1794, 0x17c1, 0x1794, 0x17b6, 0x17a2, 0x17b6, 0x179b, 0x17cb, 0x17a0,
0x17d2, 0x179f, 0x17c1, 0x17a2, 0x17b6, 0x179f, 0x17d2, 0x1798, 0x17b6, 0x179a, 0x17c9, 0x17b6,
0x1794, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x17bc, 0x1794, 0x1784, 0x17cb, 0x17a0, 0x17d2,
0x1782, 0x17b8, 0x1794, 0x1784, 0x17cb, 0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17bb, 0x179b, 0x1794,
0x17ca, 0x17b8, 0x179f, 0x17d2, 0x179f, 0x17bc, 0x1794, 0x17d2, 0x179b, 0x17b6, 0x17c6, 0x1784,
0x1791, 0x17b8, 0x179a, 0x17c9, 0x17c1, 0x1794, 0x17d2, 0x179a, 0x17b6, 0x179f, 0x17d2, 0x17a0,
0x17b6, 0x179c, 0x17b8, 0x179b, 0x1794, 0x17ca, 0x17bc, 0x1799, 0x17c9, 0x17b6, 0x17c6, 0x1794,
0x17ca, 0x17bc, 0x179a, 0x17c9, 0x17b6, 0x1782, 0x17c2, 0x179a, 0x1780, 0x17b6, 0x179f, 0x17b6,
0x1794, 0x17d2, 0x179b, 0x17b6, 0x17c6, 0x1784, 0x1780, 0x17b6, 0x1788, 0x17c0, 0x178f, 0x17b6,
0x1780, 0x17bc, 0x178e, 0x17b6, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x178a, 0x17b6, 0x1780, 0x17b6,
0x178a, 0x17b6, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17b6, 0x17a1, 0x17b6, 0x17c6, 0x1787, 0x17b8,
0x1794, 0x17ca, 0x17bc, 0x1791, 0x17b8, 0x178c, 0x17bc, 0x17a2, 0x17ca, 0x17bc, 0x17a1, 0x17b6,
0x17a2, 0x17c2, 0x179b, 0x200b, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x1789, 0x17c0, 0x1793, 0x17a0,
0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b8, 0x1790, 0x17c4, 0x1793, 0x17a0, 0x17d2, 0x1782, 0x17b6,
0x1794, 0x17bc, 0x179a, 0x17c9, 0x17bc, 0x1793, 0x17a0, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x179a,
0x17c9, 0x17c1, 0x1787, 0x17bc, 0x17a0, 0x17b6, 0x1793, 0x178e, 0x17c1, 0x1794, 0x17ca, 0x17ba,
0x17a0, 0x17d2, 0x1782, 0x1787, 0x17bc, 0x1794, 0x17b6, 0x1780, 0x17c6, 0x1794, 0x17c9, 0x17b6,
0x17a1, 0x17b6, 0x1781, 0x17b6, 0x1791, 0x17bb, 0x17c6, 0x1782, 0x17b8, 0x17a0, 0x17d2, 0x1782,
0x17b6, 0x179b, 0x17b8, 0x1782, 0x17b8, 0x1793, 0x179f, 0x17d2, 0x17a0, 0x17b6, 0x179f, 0x17b6,
0x17a1, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17bc, 0x179f, 0x179b, 0x17b8, 0x1794, 0x17d2, 0x179a,
0x17b8, 0x179c, 0x17b8, 0x179b, 0x17a1, 0x17bc, 0x1798, 0x17c9, 0x17c1, 0x179b, 0x17bc, 0x17a2,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179b, 0x17bc, 0x1794, 0x17ca, 0x17bb, 0x1798, 0x1794, 0x17b6,
0x179f, 0x17ca, 0x17b8, 0x179b, 0x17bc, 0x179f, 0x17b6, 0x1780, 0x17b6, 0x1798, 0x17c9, 0x17b6,
0x17a1, 0x17b6, 0x1794, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x1796, 0x17bb, 0x1799, 0x178f, 0x17bc,
0x1798, 0x17c9, 0x17b6, 0x179f, 0x17c1, 0x179a, 0x17bb, 0x1799, 0x1794, 0x17b6, 0x1794, 0x17b6,
0x1793, 0x1798, 0x17c9, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x178c, 0x17b8, 0x179f, 0x17d2,
0x1799, 0x17bc, 0x1798, 0x17c9, 0x17bb, 0x1784, 0x179a, 0x17c9, 0x17bc, 0x179c, 0x17b8, 0x1799,
0x17c9, 0x17b6, 0x178e, 0x17c3, 0x179a, 0x17c9, 0x17bc, 0x1794, 0x17ca, 0x17b8, 0x178a, 0x17d2,
0x1799, 0x17b6, 0x1798, 0x17c1, 0x178e, 0x17b6, 0x1793, 0x17b8, 0x1789, 0x17c9, 0x17b6, 0x1798,
0x17c9, 0x17c1, 0x178e, 0x17bd, 0x1780, 0x179f, 0x17bb, 0x178f, 0x17a2, 0x17c5, 0x17a0, 0x17d2,
0x1782, 0x17b6, 0x178c, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17bc, 0x1796, 0x17d0, 0x179a, 0x178f,
0x17bc, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17c5, 0x178f, 0x17bc, 0x1798, 0x17c9,
0x17c1, 0x1791, 0x17d2, 0x179a, 0x17b8, 0x1794, 0x17c9, 0x17bc, 0x179b, 0x17b8, 0x1791, 0x17bb,
0x1799, 0x1793, 0x17b8, 0x179f, 0x179c, 0x17b8, 0x1793, 0x178a, 0x17bb, 0x1780, 0x17a2, 0x17b6,
0x178a, 0x17b6, 0x1780, 0x17a2, 0x1784, 0x17cb, 0x1785, 0x17bc, 0x179a, 0x17c9, 0x17b6, 0x1780,
0x17a2, 0x1784, 0x17cb, 0x17a0, 0x17d2, 0x1782, 0x17b8, 0x17a1, 0x17b6, 0x17a2, 0x1784, 0x17cb,
0x1791, 0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x17a0,
0x17d2, 0x1782, 0x17bb, 0x1799, 0x178e, 0x17b6, 0x1794, 0x17ca, 0x17bb, 0x1799, 0x178e, 0x17bc,
0x179f, 0x17ca, 0x17c2, 0x1780, 0x17b6, 0x178f, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x17b6,
0x1780, 0x17bc, 0x178a, 0x17bc, 0x1794, 0x17b6, 0x179f, 0x17bc, 0x179f, 0x17bb, 0x1799, 0x17a1,
0x17b6, 0x179a, 0x17b8, 0x1799, 0x17c9, 0x17bc, 0x17a0, 0x17b6, 0x1798, 0x17c9, 0x17c2, 0x1793,
0x178c, 0x17bc, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x179a, 0x17b8, 0x1799, 0x17c9, 0x17bc, 0x17a0,
0x17d2, 0x1782, 0x17b6, 0x17a1, 0x17c1, 0x17a0, 0x17d2, 0x1782, 0x17bc, 0x179f, 0x179f, 0x17b6,
0x179b, 0x17cb, 0x178f, 0x17b6, 0x179f, 0x17b6, 0x17c6, 0x1784, 0x17a0, 0x17d2, 0x179f, 0x1784,
0x17cb, 0x179f, 0x17b6, 0x1793, 0x17cb, 0x179b, 0x17bc, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x1791,
0x17bc, 0x1782, 0x17bc, 0x1798, 0x17c9, 0x1784, 0x17cb, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x179f,
0x17bc, 0x17a2, 0x17c3, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17bc, 0x1794, 0x17b6, 0x17a2, 0x17b6,
0x179f, 0x1784, 0x17cb, 0x179f, 0x17d2, 0x1799, 0x17bb, 0x1784, 0x17a2, 0x17b6, 0x1791, 0x17b8,
0x1780, 0x17bc, 0x1780, 0x17b6, 0x1793, 0x1794, 0x17b6, 0x17a0, 0x17c0, 0x1794, 0x17b6, 0x17a0,
0x17c0, 0x1794, 0x1784, 0x17cb, 0x178a, 0x17c1, 0x179a, 0x17c9, 0x17b6, 0x179f, 0x1794, 0x17b6,
0x1794, 0x17b6, 0x178a, 0x17bb, 0x179f, 0x1794, 0x17c1, 0x17a1, 0x17b9, 0x1798, 0x1794, 0x17c1,
0x179b, 0x17b8, 0x1794, 0x17d2, 0x179b, 0x1784, 0x17cb, 0x200b, 0x179f, 0x17b6, 0x1794, 0x17cb,
0x17a1, 0x17bb, 0x1784, 0x1794, 0x17c5, 0x200b, 0x179c, 0x17b8, 0x179f, 0x17d2, 0x178f, 0x17b6,
0x1794, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x178f, 0x17b6, 0x1794, 0x17ca, 0x17bc, 0x179f,
0x17ca, 0x17b8, 0x1781, 0x17c1, 0x1798, 0x1794, 0x17d2, 0x179a, 0x17b8, 0x178a, 0x1794, 0x17c1,
0x1781, 0x17c1, 0x1798, 0x1794, 0x17c9, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x1784,
0x17cb, 0x200b, 0x1781, 0x17b6, 0x1793, 0x1781, 0x17bb, 0x1793, 0x1780, 0x17b6, 0x179a, 0x17c9,
0x17b6, 0x1780, 0x17b6, 0x179f, 0x1780, 0x17b6, 0x1799, 0x17c9, 0x17c1, 0x1793, 0x1780, 0x17c3,
0x1798, 0x17c2, 0x1793, 0x179f, 0x17ca, 0x17b8, 0x1780, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17c4,
0x1787, 0x17b8, 0x17a0, 0x17bd, 0x17a0, 0x17bd, 0x179f, 0x17ca, 0x17b8, 0x17a2, 0x17ca, 0x17bc,
0x178a, 0x17b6, 0x178a, 0x20, 0x17a0, 0x17ca, 0x17bc, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17c1,
0x179f, 0x1780, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17b6, 0x179a, 0x17b8, 0x1780, 0x17b6, 0x1780,
0x17d2, 0x179a, 0x17c1, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1793, 0x1782, 0x17bb, 0x1799, 0x17a2,
0x17b6, 0x1794, 0x17b6, 0x1780, 0x17bc, 0x179a, 0x17c9, 0x17b6, 0x1780, 0x17c5, 0x178a, 0x17b6,
0x1793, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x179f, 0x17d2, 0x17a0, 0x17b6, 0x179c, 0x17b7, 0x1793,
0x178a, 0x17b6, 0x179c, 0x179f, 0x17bb, 0x1793, 0x178a, 0x17b6, 0x179c, 0x179f, 0x17bb, 0x1793,
0x200b, 0x1780, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x178a, 0x17c2, 0x1793, 0x179c, 0x17be, 0x178a,
0x17c1, 0x178f, 0x17d2, 0x179a, 0x17bc, 0x17a2, 0x17ca, 0x17b8, 0x178f, 0x178a, 0x17bc, 0x1798,
0x17b8, 0x1793, 0x17b8, 0x1780, 0x17a2, 0x17c2, 0x178f, 0x1798, 0x17c9, 0x17bb, 0x1784, 0x178f,
0x17bb, 0x1784, 0x17a2, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x17bb, 0x1799, 0x178e, 0x17c1, 0x1797,
0x17b8, 0x17a2, 0x17c2, 0x179b, 0x179f, 0x17b6, 0x179b, 0x17cb, 0x179c, 0x17c9, 0x17b6, 0x178c,
0x17d0, 0x179a, 0x17a0, 0x17d2, 0x179c, 0x178f, 0x178e, 0x17c1, 0x179b, 0x179f, 0x17b6, 0x1793,
0x17cb, 0x17a0, 0x17d2, 0x179c, 0x17d0, 0x179a, 0x178f, 0x17b6, 0x17a1, 0x17c1, 0x17a0, 0x17d2,
0x179f, 0x17b6, 0x179a, 0x1780, 0x17d2, 0x179b, 0x17b6, 0x179f, 0x200b, 0x1794, 0x17c1, 0x1780,
0x17bc, 0x179f, 0x17c1, 0x1794, 0x17c1, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17c1, 0x1793,
0x200b, 0x1791, 0x17bc, 0x1780, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17be, 0x178e, 0x17b6,
0x178a, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x178a, 0x17c1, 0x179b, 0x17bc, 0x1794, 0x17c9, 0x17c1,
0x1780, 0x17d2, 0x179c, 0x17b6, 0x178f, 0x17c1, 0x1798, 0x17c9, 0x17b6, 0x17a1, 0x17b6, 0x17a0,
0x17d2, 0x1782, 0x17bb, 0x1799, 0x17c9, 0x17b6, 0x1782, 0x17b8, 0x179b, 0x17a0, 0x17d2, 0x1782,
0x17b8, 0x1799, 0x17c9, 0x17b6, 0x1793, 0x17a0, 0x17b6, 0x179b, 0x17b8, 0x17a0, 0x17d2, 0x179c,
0x17b6, 0x1780, 0x17a1, 0x17b6, 0x17a0, 0x17b6, 0x179c, 0x17c9, 0x17b6, 0x1793, 0x17a0, 0x17ca,
0x17c2, 0x1798, 0x17c9, 0x17bc, 0x179f, 0x17ca, 0x17b8, 0x17a1, 0x17bc, 0x17a5, 0x178e, 0x17d2,
0x178c, 0x17b6, 0x178e, 0x17b6, 0x1794, 0x17c9, 0x17bc, 0x179b, 0x17b8, 0x179f, 0x178e, 0x17bb,
0x1780, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e, 0x17b6, 0x1798, 0x17c9, 0x17b6,
0x179a, 0x17c1, 0x1793, 0x1780, 0x17bc, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e,
0x17b6, 0x1797, 0x17b8, 0x1792, 0x17be, 0x200b, 0x1794, 0x17ca, 0x17b9, 0x1780, 0x20, 0x17a5,
0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e, 0x17b6, 0x1791, 0x17b8, 0x1780, 0x17d2, 0x179a, 0x17bb,
0x1784, 0x1790, 0x17c2, 0x179b, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e, 0x17b6,
0x179c, 0x17c1, 0x179c, 0x17c9, 0x17c3, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x1799,
0x17c9, 0x17b6, 0x178e, 0x17b6, 0x179c, 0x17b7, 0x1793, 0x179f, 0x17c1, 0x178e, 0x17c1, 0x179f,
0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e, 0x17b6, 0x179c, 0x17b8, 0x178e, 0x17b6,
0x1798, 0x17c9, 0x17b6, 0x1780, 0x17cb, 0x20, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x178e,
0x17b6, 0x17a2, 0x17ca, 0x17b8, 0x1793, 0x17bc, 0x179c, 0x17b8, 0x1780, 0x17a2, 0x17ca, 0x17b8,
0x1780, 0x17b6, 0x17a1, 0x17bc, 0x17a2, 0x17ca, 0x17b8, 0x178f, 0x17a0, 0x17d2, 0x179f, 0x17b6,
0x1798, 0x17c9, 0x17b6, 0x17a2, 0x17ca, 0x17b8, 0x1780, 0x17a0, 0x17d2, 0x1799, 0x17bc, 0x1793,
0x17c0, 0x179b, 0x17bc, 0x17a2, 0x17ca, 0x17b8, 0x179c, 0x17b8, 0x179b, 0x1798, 0x17c9, 0x1784,
0x1791, 0x17b8, 0x179f, 0x17c1, 0x17a1, 0x17bc, 0x20, 0x1780, 0x17c1, 0x1793, 0x1791, 0x17bc,
0x1782, 0x17b8, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17c2, 0x1793, 0x178c, 0x17b8, 0x1780,
0x17a1, 0x17b6, 0x1794, 0x17c9, 0x17b6, 0x179f, 0x179b, 0x17b8, 0x1798, 0x17c9, 0x17b6, 0x17a1,
0x17bc, 0x17a2, 0x1784, 0x17cb, 0x17a0, 0x17d2, 0x1782, 0x17b7, 0x17a1, 0x17c1, 0x179f, 0x17a1,
0x17bc, 0x179c, 0x17be, 0x200b, 0x1796, 0x17d2, 0x179a, 0x17b8, 0x1793, 0x200b, 0x17a0, 0x17d2,
0x1782, 0x17d0, 0x179a, 0x1791, 0x17d0, 0x179a, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x179f, 0x17c1,
0x17a2, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x178e, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1798,
0x17c9, 0x17b6, 0x178e, 0x17b6, 0x17a2, 0x17ca, 0x17bc, 0x179f, 0x1798, 0x17c9, 0x17b6, 0x179a,
0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17bb, 0x178f, 0x1798, 0x17c9, 0x17b6, 0x1791, 0x17b8, 0x1793,
0x17b8, 0x1780, 0x1798, 0x17c9, 0x17b6, 0x178f, 0x17b6, 0x1798, 0x17c9, 0x17bc, 0x179a, 0x17c9,
0x17bc, 0x179f, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x178f, 0x17a1, 0x1784,
0x17cb, 0x1798, 0x17c9, 0x17c1, 0x178e, 0x17bc, 0x1798, 0x17b8, 0x1793, 0x17b8, 0x1798, 0x17c9,
0x17c1, 0x179a, 0x17b8, 0x178a, 0x17b6, 0x1798, 0x17c1, 0x178f, 0x17d2, 0x179b, 0x17b6, 0x20,
0x1780, 0x17b6, 0x178f, 0x17d2, 0x179b, 0x17b6, 0x1791, 0x17b8, 0x1780, 0x17d2, 0x179a, 0x17bb,
0x1784, 0x1798, 0x17c9, 0x17b7, 0x1785, 0x179f, 0x17ca, 0x17b8, 0x1780, 0x17bc, 0x1798, 0x17b8,
0x17a0, 0x17d2, 0x1782, 0x17b8, 0x17a1, 0x17bb, 0x1784, 0x1798, 0x17c9, 0x17bb, 0x1784, 0x178f,
0x17bb, 0x1793, 0x1798, 0x17c9, 0x17bb, 0x1793, 0x178f, 0x17c1, 0x179a, 0x17c9, 0x17c1, 0x1798,
0x17c9, 0x17bb, 0x1784, 0x178f, 0x17c1, 0x179c, 0x17b8, 0x178a, 0x17c1, 0x17a2, 0x17bc, 0x1798,
0x17c9, 0x17bb, 0x1784, 0x179f, 0x17ca, 0x17c2, 0x179a, 0x17c9, 0x17b6, 0x178f, 0x200b, 0x178e,
0x17b6, 0x179f, 0x17d2, 0x179f, 0x17bc, 0x1789, 0x17bc, 0x179c, 0x1799, 0x17c9, 0x1780, 0x178e,
0x17bc, 0x179a, 0x17c9, 0x17bb, 0x1793, 0x1789, 0x17c9, 0x17b6, 0x1794, 0x17ca, 0x17ba, 0x17a1,
0x17b6, 0x20, 0x178a, 0x17b6, 0x1780, 0x17bc, 0x178f, 0x17b6, 0x200b, 0x1781, 0x17b6, 0x1784,
0x1787, 0x17be, 0x1784, 0x179f, 0x17b7, 0x1793, 0x1792, 0x17ba, 0x20, 0x178a, 0x17b6, 0x1780,
0x17bc, 0x178f, 0x17b6, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1789,
0x17bc, 0x179c, 0x200b, 0x179f, 0x17b6, 0x17a1, 0x17b9, 0x1798, 0x20, 0x178a, 0x17b6, 0x1780,
0x17bc, 0x178f, 0x17b6, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1793,
0x17bc, 0x1780, 0x17a2, 0x17bc, 0x1787, 0x17b8, 0x178e, 0x17b6, 0x17a0, 0x17d2, 0x1780, 0x17b6,
0x1794, 0x17c9, 0x17b6, 0x178e, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x1794, 0x17c9, 0x17b6, 0x179a,
0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x179a, 0x17b8, 0x1794, 0x17bc, 0x1795, 0x17bc, 0x1793,
0x17b8, 0x1780, 0x1796, 0x17d0, 0x179a, 0x17a2, 0x17bc, 0x1794, 0x17d2, 0x179a, 0x17c9, 0x17b6,
0x17c6, 0x1784, 0x1796, 0x17d0, 0x179a, 0x17a2, 0x17c1, 0x179f, 0x17d2, 0x1794, 0x17c9, 0x17b6,
0x1789, 0x1796, 0x17d0, 0x179a, 0x178f, 0x17bc, 0x200b, 0x179c, 0x17b7, 0x179b, 0x17a0, 0x17bc,
0x1796, 0x17d0, 0x179a, 0x178f, 0x17bc, 0x179a, 0x17b8, 0x1780, 0x17bc, 0x1796, 0x17bb, 0x1793,
0x178f, 0x17b6, 0x20, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17c1, 0x178e, 0x17b6, 0x179a, 0x17c9,
0x17b6, 0x1793, 0x17cb, 0x1783, 0x17b8, 0x1793, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x1793, 0x17a1,
0x17b7, 0x178f, 0x179a, 0x17c9, 0x17c1, 0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b8,
0x179a, 0x17c9, 0x17c1, 0x17a0, 0x17d2, 0x1782, 0x17b8, 0x178e, 0x17b6, 0x179a, 0x17c9, 0x17c1,
0x200b, 0x179f, 0x17ca, 0x17bc, 0x17a1, 0x17bc, 0x178f, 0x179a, 0x17b8, 0x1799, 0x17c9, 0x17bc,
0x1794, 0x17d2, 0x179a, 0x17b6, 0x1793, 0x1780, 0x17bc, 0x179f, 0x17b6, 0x1793, 0x17cb, 0x178f,
0x17b6, 0x179a, 0x17b9, 0x1798, 0x179f, 0x17b6, 0x1793, 0x17cb, 0x1791, 0x17b8, 0x17a2, 0x17c1,
0x17a0, 0x17d2, 0x1782, 0x17c4, 0x179f, 0x17b6, 0x1793, 0x17cb, 0x178a, 0x17bc, 0x1798, 0x17c9,
0x17b6, 0x17c6, 0x1784, 0x179f, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17bc, 0x17a1, 0x17bc, 0x17a2,
0x17ca, 0x17b8, 0x178f, 0x17bc, 0x1782, 0x17d0, 0x179a, 0x178f, 0x17bc, 0x1798, 0x17b8, 0x178f,
0x179f, 0x17ca, 0x17b8, 0x178f, 0x1780, 0x17b6, 0x179f, 0x17b6, 0x17c6, 0x1784, 0x200b, 0x1794,
0x17b6, 0x1792, 0x17b8, 0x179b, 0x17b8, 0x1798, 0x17c9, 0x17b8, 0x179f, 0x17b6, 0x17c6, 0x1784,
0x200b, 0x1785, 0x1793, 0x179f, 0x17b6, 0x17c6, 0x1784, 0x200b, 0x1783, 0x17b8, 0x178f, 0x179f,
0x17b6, 0x17c6, 0x1784, 0x200b, 0x179b, 0x17bc, 0x179f, 0x17c0, 0x179f, 0x17b6, 0x17c6, 0x1784,
0x200b, 0x178f, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x179f, 0x17cb, 0x179f, 0x17b6, 0x17c6, 0x1784,
0x200b, 0x179c, 0x17b8, 0x1793, 0x179f, 0x17c1, 0x1793, 0x179f, 0x17d2, 0x179c, 0x17b8, 0x178f,
0x200b, 0x1781, 0x17ba, 0x179a, 0x17b7, 0x1793, 0x178f, 0x17c1, 0x17a0, 0x17d2, 0x1782, 0x17bc,
0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x179b, 0x17cb, 0x1794, 0x17c9, 0x17b6,
0x1792, 0x17bb, 0x17a1, 0x17c1, 0x1791, 0x17b8, 0x1799, 0x17d2, 0x1799, 0x17bc, 0x17a2, 0x17b6,
0x178e, 0x17b6, 0x178f, 0x17bc, 0x179a, 0x17c9, 0x17bb, 0x1793, 0x178f, 0x17bc, 0x178f, 0x17bc,
0x178f, 0x17bc, 0x17a1, 0x17b6, 0x179c, 0x17c9, 0x17b6, 0x1793, 0x17cb, 0x1780, 0x17bc, 0x179c,
0x17be, 0x179c, 0x17c9, 0x17b6, 0x1799, 0x17a0, 0x179f, 0x179c, 0x17b8, 0x1793, 0x17b8, 0x1797,
0x17b7, 0x1780, 0x1799, 0x17c9, 0x17b6, 0x1782, 0x17bc, 0x178f, 0x17b6, 0x178f, 0x1780, 0x17b6,
0x179f, 0x17b8, 0x178a, 0x17b6, 0x179c, 0x17b8, 0x179f, 0x178c, 0x17bb, 0x1799, 0x1798, 0x17c9,
0x17bb, 0x1784, 0x178a, 0x17ba, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x179c, 0x17b8, 0x179b, 0x1798,
0x17c9, 0x17b6, 0x1780, 0x1781, 0x17d2, 0x179c, 0x17b6, 0x179a, 0x17b8, 0x1798, 0x17c9, 0x17c5,
0x179f, 0x17b6, 0x1793, 0x17cb, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x17cb, 0x1798, 0x17bb, 0x1799,
0x178a, 0x17bc, 0x1795, 0x17b6, 0x179b, 0x17cb, 0x1798, 0x17d0, 0x179a, 0x179a, 0x17c9, 0x17bc,
0x1792, 0x17b8, 0x179a, 0x17c9, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x1799, 0x17c9, 0x17bc, 0x179c,
0x17c9, 0x17b6, 0x178f, 0x17d2, 0x179a, 0x17bc, 0x179b, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17d2,
0x178f, 0x17bb, 0x1780, 0x17a1, 0x17bb, 0x1784, 0x200b, 0x1799, 0x17c9, 0x17b6, 0x200b, 0x1794,
0x17ca, 0x17b8, 0x1799, 0x17c1, 0x1793, 0x17a2, 0x17b6, 0x178a, 0x17c2, 0x1793, 0x17a2, 0x17b6,
0x179b, 0x17cb, 0x1798, 0x17c9, 0x17b6, 0x1791, 0x17b8, 0x17a2, 0x17b6, 0x1798, 0x17c9, 0x17b6,
0x1793, 0x17cb, 0x17a2, 0x17b6, 0x178e, 0x17b6, 0x178c, 0x17b8, 0x17a2, 0x17b6, 0x1780, 0x1791,
0x17bc, 0x17a2, 0x17b6, 0x1780, 0x1791, 0x17bc, 0x1794, 0x17ca, 0x17b8, 0x17a2, 0x17b6, 0x179f,
0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1794, 0x17b6, 0x178f, 0x17a2, 0x17b6, 0x1791, 0x17b8, 0x179a,
0x17c9, 0x17bc, 0x1794, 0x17b6, 0x1780, 0x178a, 0x17b6, 0x178a, 0x1794, 0x17b6, 0x179a, 0x17c9,
0x17c2, 0x1793, 0x1794, 0x17b6, 0x1782, 0x17bc, 0x1794, 0x17b6, 0x1784, 0x1780, 0x1780, 0x1794,
0x17b6, 0x1793, 0x17bc, 0x179b, 0x1794, 0x17c1, 0x179a, 0x17bc, 0x178f, 0x1794, 0x17ca, 0x17b8,
0x179f, 0x17d2, 0x1780, 0x17c1, 0x1780, 0x1796, 0x17d2, 0x179a, 0x17bb, 0x1799, 0x178e, 0x17c1,
0x1788, 0x17b8, 0x178f, 0x17b6, 0x1788, 0x17bc, 0x1794, 0x17b6, 0x179b, 0x17cb, 0x179f, 0x17b6,
0x1793, 0x1780, 0x17bc, 0x17a1, 0x17bb, 0x17c6, 0x1794, 0x17bc, 0x178a, 0x17b6, 0x1798, 0x17c9,
0x17b6, 0x179f, 0x178a, 0x17b6, 0x1780, 0x17d2, 0x1780, 0x17b6, 0x178a, 0x17c1, 0x179b, 0x17b8,
0x178c, 0x17bb, 0x1799, 0x1794, 0x17c3, 0x178c, 0x17bc, 0x179f, 0x17d2, 0x17a0, 0x17b6, 0x1793,
0x17cb, 0x1794, 0x17c1, 0x17a0, 0x17d2, 0x179c, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x17a0,
0x17d2, 0x1782, 0x17bb, 0x179f, 0x17d2, 0x178f, 0x17b6, 0x17a0, 0x17c1, 0x1794, 0x179a, 0x17bb,
0x1793, 0x17a0, 0x17bc, 0x1787, 0x17b8, 0x1798, 0x17b8, 0x1789, 0x17a0, 0x17bb, 0x1784, 0x1780,
0x17bb, 0x1784, 0x17a0, 0x17bc, 0x179c, 0x17a2, 0x17c0, 0x179a, 0x1782, 0x17bb, 0x178f, 0x179f,
0x17d2, 0x1780, 0x17b7, 0x17cd, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x1780, 0x17b6, 0x179a, 0x178f,
0x17b6, 0x1785, 0x17b6, 0x1799, 0x17c9, 0x17b6, 0x1797, 0x17bc, 0x179a, 0x17c9, 0x17b6, 0x17a0,
0x17d2, 0x179f, 0x17ca, 0x17c1, 0x179a, 0x17bb, 0x1799, 0x179f, 0x17b6, 0x17a1, 0x17b7, 0x1798,
0x1780, 0x17b6, 0x1794, 0x17ca, 0x17bb, 0x179b, 0x1780, 0x17b6, 0x17c6, 0x1786, 0x17b6, 0x178f,
0x1780, 0x17b6, 0x1780, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x1787, 0x17b8, 0x1780, 0x17b6, 0x178f,
0x17cb, 0x1798, 0x17c9, 0x17b6, 0x1793, 0x17cb, 0x178c, 0x17bc, 0x1781, 0x17b6, 0x1793, 0x17cb,
0x178c, 0x17b8, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1780, 0x17bb, 0x179b, 0x1780, 0x17b6, 0x178f,
0x17b6, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x179f, 0x1793, 0x17bb, 0x1799, 0x17c9, 0x17b6, 0x1780,
0x1780, 0x17bc, 0x17a1, 0x17b6, 0x17a1, 0x17b6, 0x17c6, 0x1796, 0x17bd, 0x179a, 0x1783, 0x17ba,
0x178f, 0x1785, 0x17b8, 0x1784, 0x1780, 0x17bc, 0x179c, 0x17c9, 0x17c2, 0x178f, 0x1798, 0x17c9,
0x17b6, 0x1780, 0x17b6, 0x179c, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x178a,
0x17b6, 0x1793, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x17b6, 0x179f, 0x179f, 0x17b6, 0x179a, 0x1798,
0x17c9, 0x17b6, 0x1793, 0x17b8, 0x179b, 0x1798, 0x17c9, 0x17b6, 0x179f, 0x17d2, 0x1780, 0x17b6,
0x178f, 0x1793, 0x17b8, 0x1780, 0x17bc, 0x179f, 0x17ca, 0x17b8, 0x178e, 0x17bc, 0x179c, 0x17c9,
0x17bc, 0x1783, 0x17bc, 0x179f, 0x178e, 0x17c1, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd,
0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17ca, 0x17b8, 0x1794, 0x17ca, 0x17ba, 0x1780,
0x17a2, 0x17bc, 0x1798, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x17a2, 0x17bc, 0x179a, 0x17c9,
0x17b6, 0x179b, 0x17cb, 0x1797, 0x17d2, 0x1793, 0x17c6, 0x1796, 0x17c1, 0x1789, 0x1794, 0x17c9,
0x17bb, 0x1793, 0x1791, 0x17b8, 0x17a2, 0x17b6, 0x178e, 0x17b6, 0x1780, 0x17cb, 0x1796, 0x17d2,
0x1799, 0x17bb, 0x1784, 0x1799, 0x17c9, 0x17b6, 0x1784, 0x1780, 0x17bc, 0x179f, 0x17d2, 0x178a,
0x17c1, 0x178e, 0x17c1, 0x1782, 0x17b8, 0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17b8, 0x17a1, 0x17bc,
0x178a, 0x17b6, 0x179a, 0x17b8, 0x1799, 0x17c9, 0x17b6, 0x178a, 0x179f, 0x17b6, 0x1781, 0x17b6,
0x179b, 0x17b8, 0x1793, 0x179f, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x1780, 0x17b6, 0x1793, 0x17cb,
0x179f, 0x17c1, 0x17a2, 0x17ca, 0x17bc, 0x179b, 0x179f, 0x17c0, 0x1784, 0x17a0, 0x17c3, 0x179f,
0x17b7, 0x1784, 0x17d2, 0x17a0, 0x1794, 0x17bc, 0x179a, 0x17b8, 0x179f, 0x17d2, 0x179a, 0x17b8,
0x178f, 0x1793, 0x17b8, 0x1780, 0x17bc, 0x179b, 0x17b8, 0x1798, 0x178f, 0x17c3, 0x1794, 0x17c9,
0x17b7, 0x178f, 0x17b6, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x1793, 0x1794, 0x17ca, 0x17b8, 0x179b,
0x17b8, 0x179f, 0x17ca, 0x17b8, 0x178f, 0x17c1, 0x17a0, 0x17c1, 0x179a, 0x17c9, 0x1784, 0x17cb,
0x1792, 0x17b8, 0x1798, 0x1797, 0x17bc, 0x178f, 0x17bc, 0x1780, 0x17d2, 0x1799, 0x17bc, 0x1790,
0x1798, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x17a2, 0x17ca, 0x17bc, 0x17a1, 0x1784, 0x17cb,
0x1794, 0x17b6, 0x1791, 0x17d0, 0x179a, 0x17a2, 0x17ca, 0x17bc, 0x179a, 0x17b6, 0x17c6, 0x17a0,
0x17d2, 0x1782, 0x17b8, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x178e, 0x17c1, 0x179a, 0x17c9,
0x17b6, 0x179c, 0x17c0, 0x1784, 0x1785, 0x1793, 0x17d2, 0x1791, 0x1793, 0x17cd, 0x179c, 0x17c9,
0x17d2, 0x179b, 0x17b6, 0x178a, 0x17b8, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17bb,
0x1780, 0x1799, 0x17c9, 0x17b6, 0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd,
0x179a, 0x17c9, 0x1784, 0x17cb, 0x17a0, 0x17d2, 0x1782, 0x17bc, 0x1793, 0x1799, 0x17c9, 0x17c1,
0x1780, 0x17b6, 0x1792, 0x17ba, 0x179a, 0x17b7, 0x1793, 0x1794, 0x17ca, 0x17be, 0x1780, 0x1799,
0x17c9, 0x17c1, 0x179a, 0x17c9, 0x17c1, 0x179c, 0x17c9, 0x17b6, 0x1793, 0x17cb, 0x17a2, 0x17b6,
0x179f, 0x17bc, 0x179a, 0x17c9, 0x17c1, 0x179f, 0x1794, 0x17c9, 0x17ba, 0x1798, 0x17bb, 0x1799,
0x178a, 0x17b6, 0x1780, 0x17b6, 0x178e, 0x17b6, 0x179a, 0x17b8, 0x1780, 0x17b6, 0x1794, 0x17cb,
0x179c, 0x17c2, 0x179a, 0x17a0, 0x17d2, 0x179c, 0x17b6, 0x179a, 0x17c9, 0x17bc, 0x1798, 0x17c9,
0x17b6, 0x178a, 0x17c0, 0x179a, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17c3, 0x1780, 0x17d2, 0x1799,
0x17b6, 0x179c, 0x17b7, 0x1780, 0x17a0, 0x17d2, 0x179f, 0x1780, 0x17a0, 0x17d2, 0x179f, 0x17ca,
0x17b8, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x178f, 0x17d2, 0x1794, 0x17bc, 0x1784, 0x179f,
0x17b6, 0x17c6, 0x1784, 0x17a0, 0x17c1, 0x17a1, 0x17c1, 0x178e, 0x17b6, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x179b, 0x17b8, 0x17a2, 0x17b6, 0x178a, 0x17c1, 0x17a1, 0x17c2, 0x178f, 0x1794,
0x17d2, 0x179a, 0x17b8, 0x179f, 0x1794, 0x17ca, 0x17c2, 0x1793, 0x1794, 0x17d2, 0x179a, 0x17bc,
0x1780, 0x1781, 0x17b7, 0x1793, 0x17a0, 0x17b8, 0x179b, 0x178a, 0x17b6, 0x179c, 0x17b8, 0x1793,
0x17a2, 0x17ca, 0x17bb, 0x1799, 0x1780, 0x17d2, 0x179b, 0x17b6, 0x17a0, 0x17bc, 0x1794, 0x17b6,
0x178f, 0x179b, 0x17b8, 0x1793, 0x178c, 0x17b8, 0x1798, 0x17c9, 0x17c2, 0x1793, 0x17a1, 0x178f,
0x17a0, 0x17bc, 0x179c, 0x17c9, 0x17c1, 0x1798, 0x17c9, 0x17c1, 0x179b, 0x1794, 0x17ca, 0x1793,
0x1797, 0x17ba, 0x1792, 0x179f, 0x17ca, 0x17b8, 0x178a, 0x1793, 0x17b8, 0x17a2, 0x17b6, 0x17c6,
0x179f, 0x17d2, 0x1791, 0x17c2, 0x178a, 0x17b6, 0x17c6, 0x17a2, 0x1784, 0x17cb, 0x178a, 0x17bc,
0x179a, 0x17c9, 0x17b6, 0x17a2, 0x17b6, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x1781,
0x17b6, 0x1793, 0x17cb, 0x17a2, 0x17b6, 0x1790, 0x17c2, 0x1793, 0x1794, 0x17c2, 0x179b, 0x1780,
0x17d2, 0x179a, 0x17b6, 0x178a, 0x1794, 0x17c2, 0x179a, 0x17a1, 0x17b6, 0x17c6, 0x1784, 0x1794,
0x17d2, 0x179a, 0x17b6, 0x1791, 0x17b8, 0x179f, 0x17d2, 0x179b, 0x17b6, 0x179c, 0x17c9, 0x17b6,
0x1796, 0x17d2, 0x179a, 0x17bb, 0x1785, 0x179f, 0x17c2, 0x179b, 0x1794, 0x17ca, 0x17bb, 0x1799,
0x1780, 0x17b6, 0x179a, 0x17c9, 0x17c1, 0x179f, 0x1794, 0x17ca, 0x17bb, 0x1799, 0x178a, 0x17b6,
0x1794, 0x17c9, 0x17c2, 0x179f, 0x1794, 0x17ca, 0x17bc, 0x179f, 0x17ca, 0x17b8, 0x1784, 0x17a0,
0x17d2, 0x1782, 0x17c2, 0x1793, 0x1787, 0x17b8, 0x179f, 0x17ca, 0x17b8, 0x178e, 0x17bc, 0x1780,
0x17bc, 0x1794, 0x17c9, 0x17b7, 0x1793, 0x17a0, 0x17b6, 0x1780, 0x178c, 0x17bb, 0x1799, 0x1794,
0x17d2, 0x179b, 0x17b6, 0x17c6, 0x1784, 0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17b8, 0x1794, 0x17d2,
0x179a, 0x17b6, 0x179b, 0x17cb, 0x178f, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17c1, 0x1793, 0x179f,
0x17ca, 0x17b8, 0x17a0, 0x17c2, 0x179b, 0x179f, 0x17ca, 0x17b7, 0x1793, 0x1782, 0x17b8, 0x17a2,
0x17c2, 0x179b, 0x17a2, 0x17bb, 0x17a0, 0x17d2, 0x179c, 0x1798, 0x17c9, 0x17c2, 0x1793, 0x17a2,
0x17ca, 0x17b8, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x1794, 0x17ca, 0x17bb, 0x179b, 0x1787,
0x17c1, 0x179f, 0x17ca, 0x17b8, 0x1780, 0x17b6, 0x179b, 0x17b8, 0x1793, 0x17b8, 0x1784, 0x17a0,
0x17d2, 0x179a, 0x17d2, 0x1782, 0x17b6, 0x178a, 0x1782, 0x17b8, 0x179a, 0x17c9, 0x17bc, 0x179c,
0x1780, 0x17c0, 0x179c, 0x179b, 0x17b8, 0x179f, 0x1794, 0x17c4, 0x1793, 0x1787, 0x17bc, 0x1794,
0x17a0, 0x17d2, 0x179f, 0x17b6, 0x178e, 0x17b6, 0x17a1, 0x17bb, 0x1784, 0x178a, 0x17cd, 0x179b,
0x17bb, 0x1785, 0x179f, 0x17c6, 0x1794, 0x17bd, 0x1798, 0x17c9, 0x17b6, 0x178c, 0x17d2, 0x179a,
0x17b8, 0x178a, 0x1798, 0x17c9, 0x17b6, 0x179b, 0x17cb, 0x178f, 0x17cd, 0x1798, 0x17c9, 0x17b6,
0x179a, 0x17c0, 0x17a0, 0x17b6, 0x17c6, 0x1798, 0x17b8, 0x1793, 0x179f, 0x17d2, 0x1780, 0x17b7,
0x17cd, 0x1798, 0x17c9, 0x17bc, 0x178e, 0x17b6, 0x1780, 0x17bc, 0x1798, 0x17bc, 0x179f, 0x17d2,
0x1782, 0x17bc, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x179b, 0x17bc, 0x1794, 0x17c9, 0x17b6, 0x179a,
0x17b8, 0x179f, 0x1794, 0x17c9, 0x17bc, 0x178a, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x179a, 0x17b8,
0x1780, 0x17b6, 0x1794, 0x17d2, 0x179a, 0x17b6, 0x1780, 0x179a, 0x17b8, 0x17a0, 0x17d2, 0x1780,
0x17b6, 0x179f, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x179f, 0x17b6, 0x17c6,
0x1784, 0x1798, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x17c6, 0x1784, 0x179f, 0x17b6, 0x179a,
0x17c9, 0x17b6, 0x1799, 0x17c9, 0x17c1, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17b6, 0x179a, 0x17c9,
0x17b6, 0x178f, 0x17b6, 0x179c, 0x179f, 0x17ca, 0x17b8, 0x1798, 0x17a0, 0x17d2, 0x179c, 0x17be,
0x179a, 0x17c9, 0x17bb, 0x1794, 0x17c9, 0x17bc, 0x179b, 0x179f, 0x17d2, 0x1780, 0x17bb, 0x1794,
0x1787, 0x17c1, 0x179f, 0x17bc, 0x17a0, 0x17ca, 0x17d2, 0x179c, 0x17b8, 0x1799, 0x17c9, 0x17b6,
0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1781, 0x17bb, 0x179b, 0x178f, 0x17b6, 0x179b, 0x17b7,
0x1793, 0x1791, 0x17b8, 0x179a, 0x17c9, 0x17b6, 0x178e, 0x17c1, 0x17a2, 0x17bb, 0x179b, 0x1799,
0x17c9, 0x17b6, 0x178e, 0x17bc, 0x179c, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x179c, 0x17c9,
0x17b6, 0x178c, 0x17bc, 0x179f, 0x179c, 0x17c9, 0x17b6, 0x1791, 0x17b8, 0x1780, 0x1784, 0x17cb,
0x179c, 0x17b8, 0x1799, 0x17c2, 0x1793, 0x179c, 0x17b8, 0x179b, 0x1793, 0x17b8, 0x1789, 0x17bc,
0x179f, 0x179c, 0x17c9, 0x17bb, 0x179b, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x17a0, 0x17d2, 0x1782,
0x17d2, 0x179a, 0x17b6, 0x178a, 0x179c, 0x17c9, 0x17b6, 0x179f, 0x17bc, 0x179c, 0x17b8, 0x179f,
0x17b6, 0x1782, 0x17d2, 0x179a, 0x17c2, 0x1794, 0x17a0, 0x17ca, 0x17d2, 0x179f, 0x17bc, 0x179a,
0x17b8, 0x1785, 0x17a2, 0x1784, 0x17cb, 0x178f, 0x17b6, 0x178e, 0x17b6, 0x178e, 0x17b6, 0x179a,
0x17b8, 0x179c, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x179f, 0x17d2, 0x1798, 0x17b6, 0x179f, 0x1780,
0x17bc, 0x1780, 0x17bc, 0x179f, 0x1780, 0x17bc, 0x1798, 0x17d0, 0x179a, 0x1783, 0x17ba, 0x17a0,
0x17d2, 0x1782, 0x17bb, 0x1799, 0x17a1, 0x17c2, 0x1793, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17c1,
0x1798, 0x17c9, 0x17b6, 0x179b, 0x17cb, 0x178c, 0x17b8, 0x179c, 0x1798, 0x17c9, 0x17bc, 0x179a,
0x17b8, 0x179f, 0x1798, 0x17c9, 0x17b6, 0x1799, 0x17bb, 0x178f, 0x179a, 0x17c1, 0x17a2, 0x17ca,
0x17bb, 0x1799, 0x17c9, 0x17bb, 0x1784, 0x17a2, 0x17b6, 0x1796, 0x17b8, 0x17a2, 0x1780, 0x17a1,
0x17c2, 0x1793, 0x1794, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17c2, 0x1793, 0x179c, 0x17b8, 0x179b,
0x1785, 0x17b6, 0x1790, 0x17b6, 0x17c6, 0x1788, 0x17bc, 0x17a2, 0x17bb, 0x1780, 0x17a2, 0x17ca,
0x17b8, 0x179f, 0x17d2, 0x1791, 0x17d0, 0x179a, 0x17a2, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c,
0x17b6, 0x178f, 0x17c1, 0x17a0, 0x17d2, 0x179c, 0x17b6, 0x1780, 0x17c5, 0x17a0, 0x17d2, 0x179c,
0x17bc, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x1787, 0x17b8, 0x17a0, 0x17ca, 0x17d2, 0x179c, 0x17bc,
0x178e, 0x17b6, 0x17a0, 0x17ca, 0x17d2, 0x179c, 0x17bc, 0x1791, 0x17b8, 0x17a0, 0x17d2, 0x1782,
0x17b6, 0x17a1, 0x17b6, 0x1794, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x179f, 0x17a0,
0x17d2, 0x1782, 0x17b6, 0x17c6, 0x1794, 0x17c0, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x178a, 0x17b6,
0x179b, 0x17cb, 0x1781, 0x17b6, 0x178e, 0x17b6, 0x179b, 0x17cb, 0x17a0, 0x17bc, 0x178e, 0x17bc,
0x179b, 0x17bc, 0x179b, 0x17bc, 0x17a2, 0x17ca, 0x17b8, 0x1793, 0x178a, 0x17ba, 0x1794, 0x17ca,
0x17bc, 0x179a, 0x17b8, 0x1782, 0x17b7, 0x179a, 0x17b8, 0x1791, 0x17b8, 0x1798, 0x17c9, 0x17b6,
0x1791, 0x17b7, 0x1780, 0x17bc, 0x179f, 0x17d2, 0x179a, 0x17c2, 0x1780, 0x17d2, 0x179c, 0x17b6,
0x17a0, 0x17d2, 0x179f, 0x17b6, 0x179b, 0x17c0, 0x1793, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2,
0x179f, 0x17bc, 0x179a, 0x17c9, 0x17bc, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17b8,
0x179f, 0x17b6, 0x179f, 0x1798, 0x17b8, 0x178f, 0x179c, 0x17c9, 0x17c1, 0x178e, 0x17bc, 0x179a,
0x17bc, 0x1793, 0x17b8, 0x179c, 0x17c9, 0x17c1, 0x178e, 0x17d0, 0x179a, 0x17a0, 0x17d2, 0x179c,
0x17bb, 0x1780, 0x178e, 0x17bc, 0x1798, 0x17c0, 0x1794, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782,
0x17c4, 0x20, 0x1794, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17c4, 0x1794, 0x17c9, 0x17b6,
0x17a1, 0x17c5, 0x1797, 0x17b8, 0x178f, 0x1781, 0x17c3, 0x179a, 0x17c9, 0x17b7, 0x1793, 0x1795,
0x17bb, 0x1793, 0x1794, 0x17c9, 0x17b8, 0x1796, 0x17d0, 0x179a, 0x1798, 0x17c9, 0x17bc, 0x179a,
0x17c1, 0x179f, 0x1794, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17bc, 0x178f,
0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x179f, 0x17c3, 0x1794, 0x17c9, 0x17b6, 0x1793,
0x178f, 0x17b6, 0x17a0, 0x17b8, 0x1791, 0x17b8, 0x178f, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x179c,
0x17c9, 0x17b6, 0x178f, 0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x178f, 0x17b6, 0x1796,
0x17bc, 0x179c, 0x17c9, 0x17b6, 0x1782, 0x17b8, 0x179c, 0x17c9, 0x17b6, 0x179b, 0x17b8, 0x179f,
0x906, 0x915, 0x930, 0x93e, 0x906, 0x926, 0x940, 0x938, 0x20, 0x905, 0x92c, 0x93e,
0x92c, 0x93e, 0x906, 0x932, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x930, 0x94d, 0x938,
0x92c, 0x93e, 0x902, 0x917, 0x940, 0x92c, 0x93e, 0x928, 0x91c, 0x941, 0x932, 0x92c,
0x93f, 0x938, 0x93e, 0x909, 0x92c, 0x94d, 0x932, 0x93e, 0x928, 0x91f, 0x93e, 0x92f,
0x930, 0x92c, 0x94d, 0x930, 0x93e, 0x91d, 0x93e, 0x935, 0x93f, 0x932, 0x947, 0x92c,
0x941, 0x91c, 0x941, 0x92e, 0x92c, 0x941, 0x930, 0x93e, 0x915, 0x94b, 0x928, 0x93e,
0x915, 0x94d, 0x930, 0x93e, 0x926, 0x93e, 0x930, 0x20, 0x907, 0x20, 0x938, 0x932,
0x93e, 0x92e, 0x921, 0x94c, 0x906, 0x932, 0x93e, 0x905, 0x932, 0x20, 0x906, 0x92f,
0x941, 0x928, 0x92b, 0x94d, 0x930, 0x93f, 0x91f, 0x93e, 0x935, 0x928, 0x91c, 0x94b,
0x939, 0x93e, 0x928, 0x94d, 0x938, 0x92c, 0x930, 0x94d, 0x917, 0x916, 0x93e, 0x930,
0x94d, 0x91f, 0x94b, 0x92e, 0x915, 0x93f, 0x928, 0x936, 0x93e, 0x938, 0x93e, 0x92e,
0x93e, 0x938, 0x947, 0x930, 0x941, 0x92e, 0x94b, 0x917, 0x93e, 0x926, 0x93f, 0x936,
0x942, 0x928, 0x93e, 0x92f, 0x930, 0x94b, 0x92c, 0x940, 0x90f, 0x928, 0x91c, 0x93e,
0x92e, 0x947, 0x928, 0x93e, 0x928, 0x935, 0x93e, 0x915, 0x938, 0x941, 0x924, 0x913,
0x917, 0x93e, 0x926, 0x94c, 0x917, 0x94b, 0x91f, 0x941, 0x928, 0x93f, 0x938, 0x905,
0x921, 0x93e, 0x915, 0x910, 0x902, 0x915, 0x930, 0x91c, 0x905, 0x902, 0x917, 0x941,
0x932, 0x93e, 0x90f, 0x901, 0x91f, 0x93f, 0x917, 0x941, 0x906, 0x905, 0x930, 0x917,
0x941, 0x92f, 0x947, 0x928, 0x93e, 0x92c, 0x94d, 0x92f, 0x942, 0x928, 0x938, 0x20,
0x906, 0x92f, 0x930, 0x94d, 0x938, 0x915, 0x93e, 0x91f, 0x93e, 0x92e, 0x93e, 0x930,
0x915, 0x93e, 0x915, 0x94b, 0x930, 0x921, 0x94b, 0x92c, 0x93e, 0x91c, 0x941, 0x91c,
0x947, 0x92f, 0x932, 0x93e, 0x20, 0x930, 0x93f, 0x92f, 0x94b, 0x91c, 0x93e, 0x92e,
0x947, 0x928, 0x94d, 0x921, 0x94b, 0x91d, 0x93e, 0x930, 0x93f, 0x92f, 0x94b, 0x20,
0x917, 0x932, 0x947, 0x917, 0x949, 0x938, 0x938, 0x93e, 0x928, 0x20, 0x91c, 0x941,
0x906, 0x928, 0x91f, 0x941, 0x915, 0x941, 0x92e, 0x945, 0x928, 0x905, 0x930, 0x941,
0x92c, 0x93e, 0x905, 0x938, 0x902, 0x938, 0x93f, 0x92f, 0x928, 0x905, 0x924, 0x93f,
0x915, 0x94b, 0x915, 0x949, 0x928, 0x92c, 0x939, 0x93f, 0x92f, 0x93e, 0x20, 0x92c,
0x902, 0x926, 0x947, 0x930, 0x938, 0x92c, 0x947, 0x932, 0x93f, 0x91d, 0x92c, 0x94d,
0x932, 0x93e, 0x902, 0x915, 0x2d, 0x938, 0x93e, 0x92c, 0x94d, 0x932, 0x94b, 0x928,
0x92c, 0x93e, 0x913, 0x20, 0x935, 0x93f, 0x938, 0x94d, 0x91f, 0x93e, 0x92c, 0x94b,
0x92f, 0x938, 0x915, 0x947, 0x902, 0x92c, 0x94d, 0x930, 0x93f, 0x91c, 0x20, 0x92c,
0x947, 0x915, 0x945, 0x92e, 0x94d, 0x92a, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x93e,
0x902, 0x921, 0x915, 0x93e, 0x928, 0x915, 0x941, 0x928, 0x915, 0x930, 0x93e, 0x915,
0x938, 0x915, 0x947, 0x907, 0x928, 0x938, 0x93f, 0x909, 0x926, 0x93e, 0x926, 0x20,
0x91c, 0x941, 0x906, 0x930, 0x947, 0x91c, 0x915, 0x94b, 0x938, 0x94d, 0x924, 0x93e,
0x20, 0x930, 0x93f, 0x915, 0x93e, 0x915, 0x94d, 0x930, 0x947, 0x938, 0x94d, 0x91f,
0x94b, 0x928, 0x915, 0x941, 0x92f, 0x92f, 0x92c, 0x93e, 0x921, 0x947, 0x928, 0x94d,
0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x936, 0x949, 0x928, 0x921, 0x94c, 0x938, 0x928,
0x921, 0x94c, 0x938, 0x928, 0x20, 0x915, 0x94d, 0x930, 0x93f, 0x915, 0x921, 0x947,
0x928, 0x94d, 0x935, 0x947, 0x930, 0x921, 0x947, 0x91f, 0x94d, 0x930, 0x949, 0x908,
0x91f, 0x921, 0x94b, 0x92e, 0x93f, 0x928, 0x940, 0x915, 0x93e, 0x90d, 0x921, 0x92e,
0x94b, 0x902, 0x91f, 0x928, 0x90f, 0x932, 0x20, 0x938, 0x93e, 0x932, 0x94d, 0x935,
0x93e, 0x921, 0x94b, 0x930, 0x92b, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x928, 0x947,
0x932, 0x94d, 0x938, 0x928, 0x92b, 0x94b, 0x930, 0x94d, 0x91f, 0x93e, 0x932, 0x947,
0x91d, 0x93e, 0x917, 0x942, 0x938, 0x20, 0x92c, 0x947, 0x917, 0x94d, 0x930, 0x901,
0x921, 0x20, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x917, 0x94d, 0x930, 0x947, 0x928,
0x945, 0x921, 0x93e, 0x917, 0x94d, 0x935, 0x93e, 0x92f, 0x947, 0x915, 0x93f, 0x932,
0x939, 0x947, 0x932, 0x93f, 0x92b, 0x945, 0x915, 0x94d, 0x938, 0x939, 0x930, 0x94d,
0x92e, 0x94b, 0x938, 0x93f, 0x932, 0x94b, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e,
0x928, 0x93e, 0x92a, 0x94b, 0x932, 0x940, 0x938, 0x928, 0x949, 0x915, 0x94d, 0x938,
0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92a, 0x93f,
0x91f, 0x930, 0x94d, 0x938, 0x92c, 0x930, 0x94d, 0x917, 0x2c, 0x20, 0x907, 0x902,
0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x947, 0x935, 0x92f, 0x2c, 0x20,
0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x902, 0x938,
0x947, 0x928, 0x938, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x935, 0x93f, 0x928, 0x93e, 0x92e, 0x945, 0x915, 0x2c, 0x20, 0x907, 0x902,
0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x907, 0x928, 0x941, 0x935, 0x93f, 0x915,
0x907, 0x915, 0x93e, 0x932, 0x941, 0x92f, 0x940, 0x91f, 0x91c, 0x942, 0x928, 0x94b,
0x932, 0x941, 0x908, 0x91c, 0x935, 0x93f, 0x932, 0x947, 0x92e, 0x94b, 0x902, 0x91f,
0x940, 0x938, 0x945, 0x932, 0x94b, 0x2c, 0x20, 0x915, 0x947, 0x902, 0x91f, 0x941,
0x915, 0x93e, 0x92f, 0x915, 0x94d, 0x930, 0x93e, 0x932, 0x947, 0x928, 0x921, 0x940,
0x915, 0x932, 0x93e, 0x20, 0x92a, 0x93e, 0x91d, 0x932, 0x93f, 0x92e, 0x93e, 0x932,
0x949, 0x938, 0x20, 0x90f, 0x902, 0x91c, 0x932, 0x940, 0x938, 0x932, 0x94b, 0x935,
0x930, 0x20, 0x92a, 0x94d, 0x930, 0x93f, 0x902, 0x938, 0x947, 0x938, 0x20, 0x915,
0x94d, 0x935, 0x93e, 0x930, 0x94d, 0x91f, 0x930, 0x92e, 0x93e, 0x938, 0x93f, 0x92f,
0x94b, 0x92e, 0x945, 0x928, 0x93e, 0x917, 0x94d, 0x935, 0x93e, 0x92e, 0x947, 0x930,
0x940, 0x917, 0x949, 0x91f, 0x92e, 0x93e, 0x91d, 0x93e, 0x91f, 0x94d, 0x932, 0x93e,
0x928, 0x92e, 0x947, 0x91f, 0x94d, 0x932, 0x93e, 0x915, 0x93e, 0x91f, 0x94d, 0x932,
0x93e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x20, 0x936, 0x93e,
0x930, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x94b, 0x928, 0x92e, 0x94b,
0x902, 0x915, 0x94d, 0x91f, 0x949, 0x928, 0x92e, 0x94b, 0x902, 0x91f, 0x947, 0x930,
0x940, 0x92e, 0x94b, 0x902, 0x91f, 0x947, 0x935, 0x94d, 0x939, 0x93f, 0x921, 0x93f,
0x92f, 0x94b, 0x92e, 0x949, 0x928, 0x94d, 0x91f, 0x938, 0x947, 0x930, 0x93e, 0x91f,
0x928, 0x938, 0x93e, 0x90a, 0x928, 0x94d, 0x92f, 0x941, 0x20, 0x92f, 0x949, 0x930,
0x94d, 0x915, 0x928, 0x94b, 0x92e, 0x947, 0x92c, 0x94d, 0x92f, 0x942, 0x932, 0x94d,
0x932, 0x93e, 0x939, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921,
0x93e, 0x915, 0x94b, 0x91f, 0x93e, 0x92e, 0x927, 0x94d, 0x92f, 0x2c, 0x20, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x93e, 0x915, 0x94b, 0x91f, 0x93e, 0x928,
0x94d, 0x92f, 0x941, 0x20, 0x938, 0x93e, 0x932, 0x947, 0x92e, 0x2c, 0x20, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x93e, 0x915, 0x94b, 0x91f, 0x93e, 0x913,
0x91c, 0x93f, 0x928, 0x93e, 0x917, 0x93e, 0x92a, 0x930, 0x93e, 0x92e, 0x93e, 0x930,
0x93f, 0x92c, 0x94b, 0x92b, 0x93f, 0x928, 0x93f, 0x915, 0x94d, 0x938, 0x92a, 0x94b,
0x930, 0x94d, 0x91f, 0x2d, 0x914, 0x2d, 0x92a, 0x94d, 0x930, 0x93f, 0x928, 0x94d,
0x938, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x911, 0x92b, 0x20, 0x938, 0x94d,
0x92a, 0x947, 0x928, 0x92a, 0x93f, 0x930, 0x94d, 0x91f, 0x94b, 0x20, 0x930, 0x93f,
0x915, 0x94b, 0x92a, 0x941, 0x902, 0x91f, 0x93e, 0x20, 0x905, 0x930, 0x947, 0x928,
0x93e, 0x938, 0x930, 0x901, 0x915, 0x93f, 0x928, 0x20, 0x907, 0x928, 0x932, 0x945,
0x91f, 0x930, 0x947, 0x938, 0x940, 0x92b, 0x930, 0x947, 0x91c, 0x940, 0x928, 0x93e,
0x930, 0x93f, 0x938, 0x94b, 0x932, 0x94d, 0x92f, 0x941, 0x91f, 0x938, 0x93e, 0x902,
0x924, 0x93e, 0x930, 0x947, 0x92e, 0x938, 0x901, 0x91f, 0x93f, 0x92f, 0x93e, 0x917,
0x94b, 0x938, 0x93e, 0x902, 0x924, 0x94b, 0x20, 0x921, 0x94b, 0x92e, 0x93f, 0x902,
0x917, 0x94b, 0x938, 0x93e, 0x913, 0x20, 0x92a, 0x93e, 0x935, 0x932, 0x94b, 0x907,
0x91f, 0x94b, 0x915, 0x949, 0x930, 0x91f, 0x949, 0x930, 0x94d, 0x92e, 0x940, 0x91f,
0x938, 0x947, 0x902, 0x91f, 0x20, 0x91c, 0x949, 0x928, 0x94d, 0x938, 0x938, 0x947,
0x902, 0x91f, 0x20, 0x932, 0x941, 0x938, 0x940, 0x92f, 0x93e, 0x938, 0x947, 0x902,
0x91f, 0x20, 0x935, 0x93f, 0x928, 0x94d, 0x938, 0x947, 0x902, 0x91f, 0x938, 0x94d,
0x935, 0x93f, 0x92b, 0x94d, 0x91f, 0x20, 0x915, 0x930, 0x902, 0x91f, 0x924, 0x947,
0x917, 0x941, 0x938, 0x93f, 0x917, 0x932, 0x94d, 0x92a, 0x93e, 0x925, 0x941, 0x932,
0x947, 0x91f, 0x949, 0x930, 0x902, 0x91f, 0x94b, 0x91f, 0x94b, 0x930, 0x91f, 0x94b,
0x932, 0x93e, 0x935, 0x94d, 0x939, 0x901, 0x915, 0x941, 0x935, 0x94d, 0x939, 0x930,
0x935, 0x94d, 0x939, 0x93e, 0x92f, 0x91f, 0x939, 0x949, 0x930, 0x94d, 0x938, 0x92f,
0x93e, 0x915, 0x941, 0x91f, 0x93e, 0x91f, 0x921, 0x947, 0x935, 0x94d, 0x939, 0x940,
0x938, 0x926, 0x94d, 0x92f, 0x941, 0x92e, 0x93e, 0x901, 0x20, 0x926, 0x942, 0x930,
0x935, 0x940, 0x92f, 0x92e, 0x945, 0x915, 0x94d, 0x935, 0x947, 0x930, 0x940, 0x92e,
0x945, 0x915, 0x92e, 0x941, 0x930, 0x94d, 0x926, 0x94b, 0x92a, 0x93e, 0x932, 0x94d,
0x92e, 0x947, 0x930, 0x932, 0x94b, 0x902, 0x917, 0x92f, 0x947, 0x905, 0x930, 0x92c,
0x94d, 0x92f, 0x947, 0x928, 0x90d, 0x921, 0x928, 0x905, 0x928, 0x93e, 0x926, 0x93f,
0x930, 0x90d, 0x915, 0x94d, 0x91f, 0x94b, 0x905, 0x915, 0x94d, 0x91f, 0x94b, 0x92c,
0x947, 0x905, 0x936, 0x917, 0x93e, 0x92c, 0x93e, 0x91f, 0x905, 0x924, 0x94d, 0x92f,
0x93e, 0x930, 0x94b, 0x92c, 0x947, 0x939, 0x930, 0x947, 0x928, 0x92c, 0x901, 0x915,
0x949, 0x915, 0x92c, 0x930, 0x928, 0x94c, 0x932, 0x92c, 0x947, 0x930, 0x942, 0x924,
0x91a, 0x93f, 0x91f, 0x93e, 0x91a, 0x94b, 0x908, 0x92c, 0x93e, 0x932, 0x94d, 0x938,
0x928, 0x926, 0x92e, 0x938, 0x94d, 0x915, 0x938, 0x926, 0x93f, 0x932, 0x940, 0x926,
0x941, 0x92c, 0x92f, 0x92b, 0x92e, 0x93e, 0x917, 0x941, 0x938, 0x94d, 0x924, 0x93e,
0x917, 0x93e, 0x91d, 0x93e, 0x939, 0x947, 0x92c, 0x94d, 0x930, 0x949, 0x928, 0x939,
0x94b, 0x935, 0x94d, 0x939, 0x921, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x91f, 0x938,
0x94d, 0x915, 0x91c, 0x93e, 0x92f, 0x93e, 0x92a, 0x941, 0x930, 0x93e, 0x91c, 0x947,
0x930, 0x941, 0x938, 0x93e, 0x932, 0x947, 0x92e, 0x915, 0x93e, 0x92e, 0x91a, 0x93e,
0x91f, 0x94d, 0x915, 0x93e, 0x916, 0x93e, 0x93c, 0x902, 0x921, 0x93f, 0x917, 0x93e,
0x915, 0x941, 0x906, 0x932, 0x93e, 0x932, 0x92e, 0x94d, 0x92a, 0x942, 0x930, 0x915,
0x941, 0x91a, 0x940, 0x902, 0x917, 0x915, 0x941, 0x935, 0x947, 0x924, 0x92e, 0x915,
0x93e, 0x935, 0x92e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x92e, 0x915, 0x93e, 0x938,
0x94d, 0x938, 0x93e, 0x930, 0x92e, 0x928, 0x93f, 0x932, 0x93e, 0x928, 0x94b, 0x935,
0x94b, 0x915, 0x941, 0x91d, 0x928, 0x947, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x92b,
0x94d, 0x928, 0x94b, 0x92e, 0x20, 0x92a, 0x947, 0x928, 0x94d, 0x939, 0x92a, 0x94b,
0x923, 0x94d, 0x91f, 0x940, 0x905, 0x928, 0x93e, 0x915, 0x915, 0x924, 0x93e, 0x930,
0x915, 0x94b, 0x938, 0x94d, 0x91f, 0x93e, 0x928, 0x93e, 0x92f, 0x915, 0x94d, 0x92f,
0x94d, 0x91c, 0x94d, 0x92f, 0x932, 0x949, 0x930, 0x921, 0x93e, 0x938, 0x916, 0x932,
0x93f, 0x928, 0x936, 0x93e, 0x902, 0x918, 0x93e, 0x92f, 0x938, 0x93f, 0x902, 0x917,
0x93e, 0x92a, 0x942, 0x930, 0x938, 0x94d, 0x930, 0x947, 0x926, 0x928, 0x947, 0x915,
0x949, 0x932, 0x92f, 0x94d, 0x92e, 0x938, 0x94d, 0x915, 0x924, 0x948, 0x92a, 0x947,
0x908, 0x924, 0x93e, 0x936, 0x94d, 0x915, 0x902, 0x926, 0x924, 0x941, 0x92c, 0x932,
0x940, 0x938, 0x940, 0x925, 0x93f, 0x92e, 0x94d, 0x92b, 0x942, 0x924, 0x94b, 0x915,
0x94d, 0x92f, 0x94b, 0x91f, 0x949, 0x92e, 0x938, 0x94d, 0x915, 0x909, 0x932, 0x93e,
0x928, 0x92c, 0x924, 0x93e, 0x930, 0x909, 0x930, 0x941, 0x92e, 0x94d, 0x915, 0x940,
0x909, 0x938, 0x94d, 0x924, 0x20, 0x928, 0x947, 0x930, 0x93e, 0x935, 0x94d, 0x939,
0x93f, 0x90f, 0x928, 0x94d, 0x91f, 0x93f, 0x92f, 0x928, 0x92f, 0x915, 0x941, 0x924,
0x94d, 0x938, 0x94d, 0x915, 0x92f, 0x93e, 0x928, 0x917, 0x949, 0x928, 0x905, 0x91d,
0x94b, 0x930, 0x947, 0x938, 0x92c, 0x930, 0x94d, 0x92e, 0x941, 0x921, 0x93e, 0x915,
0x945, 0x928, 0x930, 0x940, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x926,
0x947, 0x92b, 0x948, 0x930, 0x94b, 0x92e, 0x926, 0x93f, 0x930, 0x93e, 0x930, 0x93f,
0x915, 0x94d, 0x91c, 0x947, 0x935, 0x93f, 0x915, 0x938, 0x947, 0x902, 0x2e, 0x20,
0x939, 0x947, 0x932, 0x947, 0x928, 0x93e, 0x938, 0x94d, 0x91f, 0x947, 0x928, 0x94d,
0x932, 0x947, 0x92c, 0x94d, 0x930, 0x93f, 0x938, 0x94d, 0x92c, 0x947, 0x928, 0x909,
0x915, 0x94d, 0x932, 0x93e, 0x932, 0x93f, 0x921, 0x947, 0x92e, 0x945, 0x928, 0x92e,
0x947, 0x932, 0x92c, 0x930, 0x94d, 0x928, 0x90d, 0x92e, 0x94d, 0x938, 0x94d, 0x91f,
0x930, 0x921, 0x945, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x916, 0x928,
0x905, 0x925, 0x947, 0x928, 0x94d, 0x938, 0x92c, 0x94d, 0x930, 0x93e, 0x91f, 0x93f,
0x938, 0x94d, 0x932, 0x93e, 0x935, 0x94d, 0x939, 0x93e, 0x92c, 0x94d, 0x930, 0x941,
0x938, 0x947, 0x932, 0x94d, 0x938, 0x92c, 0x941, 0x916, 0x93e, 0x930, 0x947, 0x938,
0x94d, 0x91f, 0x92c, 0x941, 0x938, 0x93f, 0x902, 0x91c, 0x947, 0x928, 0x91a, 0x93f,
0x938, 0x940, 0x928, 0x93e, 0x90a, 0x915, 0x94b, 0x92a, 0x928, 0x939, 0x947, 0x917,
0x928, 0x921, 0x92c, 0x94d, 0x932, 0x93f, 0x928, 0x907, 0x938, 0x932, 0x947, 0x20,
0x911, 0x92b, 0x20, 0x92e, 0x945, 0x928, 0x907, 0x938, 0x94d, 0x924, 0x902, 0x92c,
0x942, 0x932, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x93f, 0x928, 0x917, 0x94d, 0x930,
0x93e, 0x921, 0x91c, 0x941, 0x92c, 0x94d, 0x932, 0x91c, 0x93e, 0x928, 0x93e, 0x932,
0x902, 0x921, 0x928, 0x932, 0x915, 0x94d, 0x938, 0x947, 0x92e, 0x92c, 0x930, 0x94d,
0x917, 0x92e, 0x93e, 0x926, 0x94d, 0x930, 0x940, 0x926, 0x92e, 0x945, 0x930, 0x940,
0x939, 0x93e, 0x92e, 0x94d, 0x928, 0x92e, 0x94b, 0x928, 0x945, 0x915, 0x94b, 0x92a,
0x945, 0x930, 0x93f, 0x938, 0x92a, 0x949, 0x921, 0x917, 0x94b, 0x930, 0x93f, 0x915,
0x93e, 0x938, 0x945, 0x928, 0x20, 0x92e, 0x93e, 0x930, 0x940, 0x928, 0x94b, 0x938,
0x93e, 0x930, 0x93e, 0x91c, 0x93f, 0x935, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x91f,
0x94b, 0x935, 0x938, 0x93f, 0x92e, 0x92b, 0x930, 0x94b, 0x92a, 0x94b, 0x932, 0x938,
0x94b, 0x92b, 0x93f, 0x92f, 0x93e, 0x91f, 0x945, 0x932, 0x93f, 0x928, 0x924, 0x93f,
0x930, 0x93e, 0x928, 0x93e, 0x935, 0x921, 0x941, 0x91c, 0x935, 0x945, 0x91f, 0x93f,
0x915, 0x928, 0x935, 0x93f, 0x92f, 0x947, 0x928, 0x93e, 0x935, 0x94d, 0x939, 0x94b,
0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x935, 0x93e, 0x930,
0x938, 0x949, 0x91d, 0x93e, 0x917, 0x94d, 0x930, 0x947, 0x92c, 0x91d, 0x941, 0x930,
0x93f, 0x91a, 0x905, 0x902, 0x91f, 0x93e, 0x928, 0x93e, 0x928, 0x93e, 0x930, 0x93f,
0x935, 0x94b, 0x91a, 0x93e, 0x917, 0x94b, 0x938, 0x915, 0x947, 0x930, 0x917, 0x941,
0x932, 0x947, 0x928, 0x92e, 0x949, 0x930, 0x93f, 0x936, 0x938, 0x92e, 0x947, 0x92f,
0x94b, 0x91f, 0x930, 0x93f, 0x92f, 0x941, 0x928, 0x93f, 0x92f, 0x928, 0x905, 0x92a,
0x93f, 0x92f, 0x93e, 0x911, 0x915, 0x932, 0x901, 0x921, 0x92c, 0x94b, 0x917, 0x947,
0x928, 0x935, 0x93f, 0x932, 0x947, 0x91a, 0x945, 0x925, 0x92e, 0x91a, 0x941, 0x915,
0x907, 0x92b, 0x947, 0x91f, 0x92b, 0x93e, 0x915, 0x93e, 0x913, 0x92b, 0x94b, 0x92b,
0x941, 0x928, 0x93e, 0x92b, 0x941, 0x91f, 0x940, 0x917, 0x93e, 0x932, 0x93e, 0x92a,
0x93e, 0x917, 0x94b, 0x938, 0x917, 0x93e, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930,
0x917, 0x94d, 0x935, 0x93e, 0x926, 0x93e, 0x932, 0x915, 0x945, 0x928, 0x93e, 0x932,
0x907, 0x902, 0x921, 0x930, 0x92c, 0x930, 0x940, 0x915, 0x93f, 0x930, 0x93f, 0x924,
0x93f, 0x92e, 0x93e, 0x924, 0x940, 0x915, 0x94b, 0x938, 0x930, 0x947, 0x915, 0x94d,
0x935, 0x93e, 0x91c, 0x932, 0x947, 0x928, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x93f,
0x938, 0x938, 0x928, 0x93e, 0x935, 0x930, 0x942, 0x928, 0x949, 0x930, 0x92b, 0x94b,
0x915, 0x928, 0x942, 0x92e, 0x947, 0x906, 0x92a, 0x93f, 0x91f, 0x915, 0x945, 0x930,
0x928, 0x92a, 0x94b, 0x939, 0x94d, 0x928, 0x92a, 0x947, 0x908, 0x92a, 0x94b, 0x930,
0x94d, 0x91f, 0x20, 0x92e, 0x949, 0x930, 0x947, 0x938, 0x94d, 0x92c, 0x940, 0x924,
0x93e, 0x930, 0x935, 0x93e, 0x935, 0x948, 0x915, 0xc544, 0xbe44, 0xc7a5, 0xc544, 0xd06c,
0xb77c, 0xc544, 0xb514, 0xc2a4, 0xc544, 0xbc14, 0xbc14, 0xc54c, 0xc81c, 0xc544, 0xc2a4, 0xba54,
0xb77c, 0xbc14, 0xb9c8, 0xcf54, 0xbc29, 0xae30, 0xbc18, 0xc904, 0xbe44, 0xc0ac, 0xc6b0, 0xbe14,
0xb79c, 0xd0c0, 0xc774, 0xc5b4, 0xbe0c, 0xb77c, 0xc790, 0xbe4c, 0xbd80, 0xc90c, 0xbd80, 0xb77c,
0xce74, 0xc774, 0xb85c, 0xce74, 0xc0ac, 0xbe14, 0xb791, 0xce74, 0xc138, 0xc6b0, 0xd0c0, 0xcf54,
0xb098, 0xd06c, 0xb9ac, 0xb2e4, 0xce74, 0xb974, 0xb2e4, 0xb974, 0xc5d0, 0xc2a4, 0xc0b4, 0xb78c,
0xc9c0, 0xbd80, 0xd2f0, 0xb450, 0xc54c, 0xb77c, 0xc5d8, 0xc544, 0xc774, 0xc6b4, 0xd504, 0xb9ac,
0xd0c0, 0xc6b4, 0xac00, 0xbcf4, 0xb85c, 0xb124, 0xd558, 0xb77c, 0xb808, 0xc694, 0xd558, 0xb124,
0xc2a4, 0xbc84, 0xadf8, 0xc8fc, 0xbc14, 0xce84, 0xd314, 0xb77c, 0xce74, 0xb974, 0xd23c, 0xd0a4,
0xac08, 0xb9ac, 0xd0a8, 0xc0e4, 0xc0ac, 0xb77c, 0xace0, 0xc2a4, 0xb9ac, 0xbe0c, 0xb974, 0xbe4c,
0xb85c, 0xba54, 0xb8e8, 0xc548, 0xb2e4, 0xb8e8, 0xbd90, 0xbc14, 0xc2dc, 0xb8e8, 0xc0ac, 0xce74,
0xb9d0, 0xb77c, 0xbcf4, 0xb9c8, 0xd478, 0xd1a0, 0xb9c8, 0xc138, 0xb8e8, 0xc74c, 0xbc14, 0xbc14,
0xb124, 0xbaa8, 0xac00, 0xb514, 0xc288, 0xbaac, 0xb85c, 0xbe44, 0xc544, 0xb098, 0xc774, 0xb85c,
0xbe44, 0xc5d4, 0xc790, 0xba54, 0xb098, 0xb2c8, 0xc544, 0xba54, 0xb204, 0xc545, 0xc1fc, 0xd2b8,
0xc640, 0xac00, 0xb450, 0xad6c, 0xd3ec, 0xb974, 0xd1a0, 0xb178, 0xbcf4, 0xc0c1, 0xd22c, 0xba54,
0xd2b8, 0xb9ac, 0xd3f4, 0xb9ac, 0xd280, 0xb2c8, 0xc2a4, 0xbe48, 0xd2b8, 0xd6c4, 0xd06c, 0xc5d0,
0xc774, 0xb2e5, 0xc575, 0xcee4, 0xb9ac, 0xc9c0, 0xc559, 0xadc8, 0xb77c, 0xc548, 0xd2f0, 0xacfc,
0xc544, 0xb77c, 0xacfc, 0xc774, 0xb098, 0xbd80, 0xc5d0, 0xb178, 0xc2a4, 0x20, 0xc544, 0xc774,
0xb808, 0xc2a4, 0xce74, 0xd0c0, 0xb9c8, 0xb974, 0xce74, 0xcf54, 0xb974, 0xb3c4, 0xbc14, 0xd6c4,
0xd6c4, 0xc774, 0xb77c, 0x20, 0xb9ac, 0xc624, 0xd558, 0xba58, 0xb3c4, 0xc0ac, 0xb9ac, 0xc624,
0x20, 0xac00, 0xc608, 0xace0, 0xc2a4, 0xc0b4, 0xd0c0, 0xc0b0, 0xd6c4, 0xc548, 0xc0b0, 0xb8e8,
0xc774, 0xc2a4, 0xd22c, 0xcfe0, 0xb9cc, 0xc6b0, 0xc218, 0xc544, 0xc774, 0xc544, 0xc544, 0xb8e8,
0xbc14, 0xc544, 0xc21c, 0xc2dc, 0xc628, 0xcf54, 0xb784, 0xd558, 0xbc84, 0xbc14, 0xd788, 0xc544,
0xbc14, 0xc774, 0xc544, 0x20, 0xbc18, 0xb370, 0xb77c, 0xc2a4, 0xbc14, 0xbca0, 0xc774, 0xb3c4,
0xc2a4, 0xbca8, 0xb818, 0xbca8, 0xb9ac, 0xc988, 0xbe14, 0xb791, 0x20, 0xc0ac, 0xbe14, 0xb871,
0xbcf4, 0xc544, 0x20, 0xbe44, 0xc2a4, 0xd0c0, 0xbcf4, 0xace0, 0xd0c0, 0xbcf4, 0xc774, 0xc2dc,
0xcf00, 0xc784, 0xbe0c, 0xb9ac, 0xc9c0, 0x20, 0xbca0, 0xc774, 0xce84, 0xd3ec, 0x20, 0xadf8,
0xb780, 0xb370, 0xce78, 0xcfe4, 0xce74, 0xb77c, 0xce74, 0xc2a4, 0xce74, 0xc60c, 0xcf00, 0xc774,
0xb9e8, 0xc2dc, 0xce74, 0xace0, 0xce58, 0xc640, 0xc640, 0xc2dc, 0xc6b0, 0xb2e4, 0xb4dc, 0xd6c4,
0xc544, 0xb808, 0xc2a4, 0xcf54, 0xc2a4, 0xd0c0, 0xb9ac, 0xce74, 0xd06c, 0xb808, 0xc2a4, 0xd1a4,
0xcfe0, 0xc774, 0xc544, 0xbc14, 0xd034, 0xb77c, 0xc18c, 0xb374, 0xb9c8, 0xd06c, 0xc0e4, 0xbe10,
0xb3c4, 0xc2a8, 0xb3c4, 0xc2a8, 0xd06c, 0xb9ad, 0xb374, 0xbc84, 0xb514, 0xd2b8, 0xb85c, 0xc774,
0xd2b8, 0xb3c4, 0xbbf8, 0xb2c8, 0xce74, 0xc5d0, 0xb4dc, 0xba3c, 0xd134, 0xc544, 0xc774, 0xb8e8,
0xb124, 0xd398, 0xc5d8, 0xc0b4, 0xbc14, 0xb3c4, 0xb974, 0xd3ec, 0xd2b8, 0x20, 0xb12c, 0xc2a8,
0xd3ec, 0xb974, 0xd0c8, 0xb808, 0xc790, 0xae00, 0xb77c, 0xc2a4, 0xbca0, 0xc774, 0xad6c, 0xc988,
0xbca0, 0xc774, 0xadf8, 0xb79c, 0xb4dc, 0x20, 0xd130, 0xd06c, 0xadf8, 0xb808, 0xb098, 0xb2e4,
0xacfc, 0xb4e4, 0xb8e8, 0xd504, 0xacfc, 0xd14c, 0xb9d0, 0xb77c, 0xacfc, 0xc57c, 0xd0ac, 0xac00,
0xc774, 0xc544, 0xb098, 0xd57c, 0xb9ac, 0xd329, 0xc2a4, 0xd558, 0xbc14, 0xb098, 0xc5d0, 0xb974,
0xbaa8, 0xc2dc, 0xc694, 0xc778, 0xb514, 0xc560, 0xb098, 0xd3f4, 0xb9ac, 0xc2a4, 0xc778, 0xb514,
0xc560, 0xb098, 0xc8fc, 0x2c, 0x20, 0xb179, 0xc2a4, 0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc,
0x2c, 0x20, 0xba38, 0xb81d, 0xace0, 0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c, 0x20,
0xd53c, 0xce20, 0xbc84, 0xadf8, 0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c, 0x20, 0xd154,
0xc2dc, 0xd2f0, 0xc778, 0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c, 0x20, 0xbe44, 0xbe44, 0xc778,
0xb514, 0xc560, 0xb098, 0xc8fc, 0x2c, 0x20, 0xbe48, 0xc13c, 0xc2a4, 0xc778, 0xb514, 0xc560,
0xb098, 0xc8fc, 0x2c, 0x20, 0xc704, 0xb108, 0xb9e5, 0xc774, 0xb204, 0xbe45, 0xc774, 0xce7c,
0xb8e8, 0xc774, 0xd2b8, 0xc790, 0xba54, 0xc774, 0xce74, 0xc8fc, 0xb178, 0xb8e8, 0xc774, 0xbe4c,
0xcf04, 0xd130, 0xd0a4, 0xc8fc, 0x2c, 0x20, 0xbaac, 0xd2f0, 0xccbc, 0xb85c, 0xd06c, 0xb77c,
0xb80c, 0xb514, 0xc988, 0xd06c, 0xb77c, 0xd30c, 0xc2a4, 0xb9ac, 0xb9c8, 0xb85c, 0xc2a4, 0xc564,
0xc824, 0xb808, 0xc2a4, 0xb85c, 0xc6cc, 0x20, 0xd504, 0xb9b0, 0xc2a4, 0x20, 0xcffc, 0xd130,
0xb9c8, 0xc138, 0xc774, 0xc624, 0xb9c8, 0xb098, 0xacfc, 0xb9c8, 0xb098, 0xc6b0, 0xc2a4, 0xb9c8,
0xb9ac, 0xacf3, 0xb9c8, 0xd2f0, 0xb2c8, 0xd06c, 0xb9c8, 0xd0c0, 0xbaa8, 0xb85c, 0xc2a4, 0xb9c8,
0xc0ac, 0xd2c0, 0xb780, 0xba54, 0xb178, 0xbbf8, 0xb2c8, 0xba54, 0xb9ac, 0xb2e4, 0xba54, 0xd2c0,
0xb77c, 0xce74, 0xd2c0, 0xb77c, 0xba55, 0xc2dc, 0xcf54, 0x20, 0xc2dc, 0xd2f0, 0xbbf8, 0xd074,
0xb871, 0xbabd, 0xd134, 0xbaac, 0xd14c, 0xb808, 0xc774, 0xbaac, 0xd14c, 0xbe44, 0xb370, 0xc624,
0xbaac, 0xc138, 0xb77c, 0xd2b8, 0xb098, 0xc18c, 0xb274, 0xc695, 0xb188, 0xb178, 0xb871, 0xc57c,
0xb178, 0xc2a4, 0xb2e4, 0xcf54, 0xd0c0, 0xc8fc, 0x2c, 0x20, 0xbca0, 0xb77c, 0xc911, 0xbd80,
0x2c, 0x20, 0xb178, 0xc2a4, 0xb2e4, 0xcf54, 0xd0c0, 0xb178, 0xc2a4, 0xb2e4, 0xcf54, 0xd0c0,
0xc8fc, 0x2c, 0x20, 0xb274, 0xc0b4, 0xb818, 0xace0, 0xb4dc, 0xd638, 0xd504, 0xc624, 0xd788,
0xb098, 0xac00, 0xd30c, 0xb098, 0xb9c8, 0xd30c, 0xb77c, 0xb9c8, 0xb9ac, 0xbcf4, 0xd53c, 0xb2c9,
0xc2a4, 0xd3ec, 0xb974, 0xd1a0, 0xd504, 0xb7ad, 0xc2a4, 0xd3ec, 0xd2b8, 0xc624, 0xbe0c, 0xc2a4,
0xd398, 0xc778, 0xd3ec, 0xb974, 0xd1a0, 0xbca8, 0xb8cc, 0xd478, 0xc5d0, 0xb974, 0xd1a0, 0xb9ac,
0xcf54, 0xd47c, 0xd0c0, 0xc544, 0xb808, 0xb098, 0xc2a4, 0xb7ad, 0xd0b9, 0x20, 0xc778, 0xb81b,
0xb808, 0xc2dc, 0xd398, 0xb9ac, 0xc790, 0xc774, 0xb098, 0xb9ac, 0xc878, 0xb8e8, 0xd2b8, 0xd788,
0xc6b0, 0x20, 0xbe0c, 0xb791, 0xcfe0, 0xc0b0, 0xd0c0, 0xb818, 0xc0b0, 0xd2f0, 0xc544, 0xace0,
0xc0b0, 0xd1a0, 0xb3c4, 0xbc0d, 0xace0, 0xc0c1, 0xd30c, 0xc6b8, 0xb8e8, 0xc2a4, 0xcf54, 0xb808,
0xc2a4, 0xbc14, 0xc774, 0xc120, 0xb4dc, 0xc2ef, 0xce74, 0xc0dd, 0xbc14, 0xb974, 0xd154, 0xb808,
0xbbf8, 0xc138, 0xc778, 0xd2b8, 0xc874, 0xc2a4, 0xc138, 0xc778, 0xd2b8, 0xd0a4, 0xce20, 0xc138,
0xc778, 0xd2b8, 0xb8e8, 0xc2dc, 0xc544, 0xc138, 0xc778, 0xd2b8, 0xd1a0, 0xb9c8, 0xc2a4, 0xc138,
0xc778, 0xd2b8, 0xbe48, 0xc13c, 0xd2b8, 0xc2a4, 0xc704, 0xd504, 0xd2b8, 0xcee4, 0xb7f0, 0xd2b8,
0xd14c, 0xad6c, 0xc2dc, 0xac08, 0xd30c, 0xd234, 0xb808, 0xd2f0, 0xd6c4, 0xc544, 0xb098, 0xd1a0,
0xb860, 0xd1a0, 0xd1a0, 0xb974, 0xd1a8, 0xb77c, 0xbca4, 0xcfe0, 0xbc84, 0xd654, 0xc774, 0xd2b8,
0xd638, 0xc2a4, 0xc704, 0xb2c8, 0xd399, 0xc57c, 0xcfe0, 0xd0c0, 0xd2b8, 0xcf00, 0xc774, 0xc2dc,
0xb370, 0xc774, 0xbe44, 0xc2a4, 0xb4a4, 0xbabd, 0x20, 0xb4a4, 0xb974, 0xbe4c, 0xb9e5, 0xcffc,
0xb9ac, 0xbaa8, 0xc2a8, 0xb9e5, 0xba38, 0xb3c4, 0xd30c, 0xba38, 0xb85c, 0xb370, 0xb77c, 0xc1fc,
0xc640, 0xd2b8, 0xb864, 0xbcf4, 0xc2a4, 0xd1a0, 0xd06c, 0xb871, 0xc774, 0xc5b4, 0xbe44, 0xc5d4,
0xc544, 0xb374, 0xc54c, 0xb9c8, 0xd2f0, 0xc554, 0xb9cc, 0xc544, 0xb098, 0xb514, 0xb9ac, 0xc544,
0xd06c, 0xd0c0, 0xc6b0, 0xc545, 0xd1a0, 0xbe0c, 0xc544, 0xc288, 0xd558, 0xbc14, 0xd2b8, 0xc544,
0xd2f0, 0xb77c, 0xc6b0, 0xbc14, 0xadf8, 0xb2e4, 0xb4dc, 0xbc14, 0xb808, 0xc778, 0xbc14, 0xcfe0,
0xbc29, 0xcf55, 0xbc14, 0xb974, 0xb098, 0xc6b8, 0xbca0, 0xc774, 0xb8e8, 0xd2b8, 0xbe44, 0xc288,
0xcf00, 0xd06c, 0xbe0c, 0xb8e8, 0xb098, 0xc774, 0xce58, 0xd0c0, 0xcd08, 0xc774, 0xbc1c, 0xc0b0,
0xcf5c, 0xb86c, 0xbcf4, 0xb2e4, 0xb9c8, 0xc2a4, 0xcfe0, 0xc2a4, 0xb51c, 0xb9ac, 0xb450, 0xbc14,
0xc774, 0xb450, 0xc0e8, 0xbca0, 0xd30c, 0xb9c8, 0xad6c, 0xc2a4, 0xd0c0, 0xac00, 0xc790, 0xd5e4,
0xbe0c, 0xb860, 0xc0ac, 0xc774, 0xacf5, 0xd64d, 0xcf69, 0xd638, 0xbe0c, 0xb4dc, 0xc774, 0xb974,
0xcfe0, 0xce20, 0xd06c, 0xc790, 0xce74, 0xb974, 0xd0c0, 0xc790, 0xc57c, 0xd478, 0xb77c, 0xc608,
0xb8e8, 0xc0b4, 0xb818, 0xce74, 0xbd88, 0xce84, 0xcc28, 0xce74, 0xce74, 0xb77c, 0xce58, 0xce74,
0xd2b8, 0xb9cc, 0xb450, 0xd55c, 0xb514, 0xac00, 0xcf5c, 0xce74, 0xd0c0, 0xd06c, 0xb77c, 0xc2a4,
0xb178, 0xc57c, 0xb974, 0xc2a4, 0xd06c, 0xcfe0, 0xc54c, 0xb77c, 0xb8f8, 0xd478, 0xb974, 0xcfe0,
0xce6d, 0xcfe0, 0xc6e8, 0xc774, 0xd2b8, 0xb9c8, 0xce74, 0xc624, 0xb9c8, 0xac00, 0xb2e8, 0xb9c8,
0xce74, 0xc0ac, 0xb974, 0xb9c8, 0xb2d0, 0xb77c, 0xbb34, 0xc2a4, 0xce74, 0xd2b8, 0xb2c8, 0xcf54,
0xc2dc, 0xc544, 0xb178, 0xbcf4, 0xcfe0, 0xc988, 0xb124, 0xce20, 0xd06c, 0xb178, 0xbcf4, 0xc2dc,
0xbe44, 0xb974, 0xc2a4, 0xd06c, 0xc634, 0xc2a4, 0xd06c, 0xc624, 0xb784, 0xd504, 0xb188, 0xd39c,
0xd3f0, 0xd2f0, 0xc544, 0xb098, 0xd06c, 0xd3c9, 0xc591, 0xce74, 0xd0c0, 0xb974, 0xcf54, 0xc2a4,
0xd0c0, 0xb098, 0xc774, 0xd0a4, 0xc9c8, 0xb85c, 0xb974, 0xb2e4, 0xb9ac, 0xc57c, 0xb4dc, 0xc0ac,
0xd560, 0xb9b0, 0xc0ac, 0xb9c8, 0xb974, 0xce78, 0xd2b8, 0xc11c, 0xc6b8, 0xc0c1, 0xd558, 0xc774,
0xc2f1, 0xac00, 0xd3ec, 0xb974, 0xc2a4, 0xb808, 0xb4dc, 0xb124, 0xcf5c, 0xb9bc, 0xc2a4, 0xd06c,
0xd0c0, 0xc774, 0xbca0, 0xc774, 0xd0c0, 0xc288, 0xcf04, 0xd2b8, 0xd2b8, 0xbe4c, 0xb9ac, 0xc2dc,
0xd14c, 0xd5e4, 0xb780, 0xd300, 0xbd80, 0xb3c4, 0xcfc4, 0xd1b0, 0xc2a4, 0xd06c, 0xc6b8, 0xb780,
0xbc14, 0xd1a0, 0xb974, 0xc6b0, 0xb8e8, 0xbb34, 0xce58, 0xc6b0, 0xc2a4, 0xd2f0, 0xb124, 0xb77c,
0xbe44, 0xc5d4, 0xd2f0, 0xc548, 0xbe14, 0xb77c, 0xb514, 0xbcf4, 0xc2a4, 0xd1a0, 0xd06c, 0xc57c,
0xcfe0, 0xce20, 0xd06c, 0xb791, 0xad70, 0xc608, 0xce74, 0xd14c, 0xb9b0, 0xbd80, 0xb974, 0xd06c,
0xc608, 0xb808, 0xbc18, 0xc544, 0xc870, 0xb808, 0xc2a4, 0xbc84, 0xbba4, 0xb2e4, 0xce74, 0xb098,
0xb9ac, 0xc544, 0x20, 0xc81c, 0xb3c4, 0xce74, 0xbcf4, 0x20, 0xbca0, 0xb974, 0xb370, 0xd398,
0xb85c, 0x20, 0xc81c, 0xb3c4, 0xb9c8, 0xb370, 0xc774, 0xb77c, 0xb808, 0xc774, 0xceac, 0xbe44,
0xd06c, 0xc0ac, 0xc6b0, 0xc2a4, 0xc870, 0xc9c0, 0xc544, 0xc138, 0xc778, 0xd2b8, 0x20, 0xd5ec,
0xb808, 0xb098, 0xc2a4, 0xd0e0, 0xb9ac, 0xc560, 0xb4e4, 0xb808, 0xc774, 0xb4dc, 0xbe0c, 0xb9ac,
0xc2a4, 0xbca0, 0xc778, 0xbe0c, 0xb85c, 0xcee8, 0xd790, 0xb2e4, 0xc708, 0xc720, 0xd074, 0xb77c,
0xd638, 0xbc14, 0xd2b8, 0xb9b0, 0xb370, 0xb9cc, 0xb85c, 0xb4dc, 0x20, 0xd558, 0xc6b0, 0xba5c,
0xbc84, 0xb978, 0xd37c, 0xc2a4, 0xc2dc, 0xb4dc, 0xb2c8, 0xc554, 0xc2a4, 0xd14c, 0xb974, 0xb2f4,
0xc548, 0xb3c4, 0xb77c, 0xc544, 0xc2a4, 0xd2b8, 0xb77c, 0xd55c, 0xc544, 0xd14c, 0xb124, 0xbca0,
0xc624, 0xadf8, 0xb77c, 0xb4dc, 0xbca0, 0xb97c, 0xb9b0, 0xbe0c, 0xb77c, 0xd2f0, 0xc2ac, 0xb77c,
0xbc14, 0xbe0c, 0xb93c, 0xc140, 0xbd80, 0xcfe0, 0xb808, 0xc288, 0xd2f0, 0xbd80, 0xb2e4, 0xd398,
0xc2a4, 0xd2b8, 0xbdd4, 0xc9c0, 0xac90, 0xd0a4, 0xc2dc, 0xb098, 0xc6b0, 0xcf54, 0xd39c, 0xd558,
0xac90, 0xb354, 0xbe14, 0xb9b0, 0xc9c0, 0xbe0c, 0xb864, 0xd130, 0xac74, 0xc9c0, 0xd5ec, 0xc2f1,
0xd0a4, 0xb9e8, 0xc12c, 0xc774, 0xc2a4, 0xd0c4, 0xbd88, 0xc800, 0xc9c0, 0xce7c, 0xb9ac, 0xb2cc,
0xadf8, 0xb77c, 0xb4dc, 0xd0a4, 0xb85c, 0xd504, 0xd0a4, 0xc608, 0xd504, 0xb9ac, 0xc2a4, 0xbcf8,
0xb958, 0xbe14, 0xb7b4, 0xb098, 0xb7f0, 0xb358, 0xb8e9, 0xc148, 0xbd80, 0xb974, 0xd06c, 0xb9c8,
0xb4dc, 0xb9ac, 0xb4dc, 0xbab0, 0xd0c0, 0xb9c8, 0xb9ac, 0xc5d0, 0xd568, 0xbbfc, 0xc2a4, 0xd06c,
0xbaa8, 0xb098, 0xcf54, 0xbaa8, 0xc2a4, 0xd06c, 0xbc14, 0xc624, 0xc2ac, 0xb85c, 0xd30c, 0xb9ac,
0xd3ec, 0xb4dc, 0xace0, 0xb9ac, 0xcc28, 0xd504, 0xb77c, 0xd558, 0xb9ac, 0xac00, 0xb85c, 0xb9c8,
0xc0ac, 0xb9c8, 0xb77c, 0xc0b0, 0xb9c8, 0xb9ac, 0xb178, 0xc0ac, 0xb77c, 0xc608, 0xbcf4, 0xc0ac,
0xb77c, 0xd1a0, 0xd504, 0xc2ec, 0xd398, 0xb85c, 0xd3f4, 0xc2a4, 0xcf54, 0xd398, 0xc18c, 0xd53c,
0xc544, 0xc2a4, 0xd1a1, 0xd640, 0xb984, 0xd0c8, 0xb9b0, 0xd2f0, 0xb77c, 0xb098, 0xc6b8, 0xb9ac,
0xc57c, 0xb178, 0xd504, 0xc2a4, 0xd06c, 0xd30c, 0xb450, 0xce20, 0xbc14, 0xd2f0, 0xce78, 0xbe44,
0xc5d4, 0xb098, 0xbe4c, 0xb2c8, 0xc6b0, 0xc2a4, 0xbcfc, 0xace0, 0xadf8, 0xb77c, 0xd2b8, 0xbc14,
0xb974, 0xc0e4, 0xbc14, 0xc790, 0xadf8, 0xb808, 0xbe0c, 0xcde8, 0xb9ac, 0xd788, 0xc548, 0xd0c0,
0xb098, 0xb098, 0xb9ac, 0xbcf4, 0xcc28, 0xace0, 0xc2a4, 0xd06c, 0xb9ac, 0xc2a4, 0xb9c8, 0xc2a4,
0xcf54, 0xcf54, 0xc2a4, 0xcf54, 0xbaa8, 0xb85c, 0xcf00, 0xb974, 0xcf08, 0xb80c, 0xb9c8, 0xd5e4,
0xbab0, 0xb514, 0xbe0c, 0xbaa8, 0xb9ac, 0xc154, 0xc2a4, 0xba54, 0xc694, 0xd2b8, 0xb808, 0xc704,
0xb2c8, 0xc639, 0xc544, 0xd53c, 0xc544, 0xc624, 0xd074, 0xb79c, 0xb4dc, 0xbd80, 0xac31, 0xbe4c,
0xcc44, 0xd140, 0xd2b8, 0xb8e8, 0xd06c, 0xc774, 0xc2a4, 0xd130, 0x20, 0xc12c, 0xc5d0, 0xd30c,
0xd14c, 0xd30c, 0xce74, 0xc624, 0xd478, 0xd53c, 0xc9c0, 0xd478, 0xb098, 0xd478, 0xd2f0, 0xac08,
0xb77c, 0xd30c, 0xace0, 0xc2a4, 0xac10, 0xbe44, 0xc5b4, 0xacfc, 0xb2ec, 0xce74, 0xb0a0, 0xad0c,
0xd638, 0xb180, 0xb8f0, 0xb8e8, 0xc5d4, 0xb354, 0xbca0, 0xb9ac, 0xd0a4, 0xb9ac, 0xd2f0, 0xb9c8,
0xd2f0, 0xcf54, 0xc2a4, 0xb808, 0xcf70, 0xc798, 0xb80c, 0xb9c8, 0xc8fc, 0xb85c, 0xb9c8, 0xd018,
0xc0ac, 0xc2a4, 0xbbf8, 0xb4dc, 0xc6e8, 0xc774, 0xb098, 0xc6b0, 0xb8e8, 0xb2c8, 0xc6b0, 0xc5d0,
0xb178, 0xd37d, 0xb204, 0xba54, 0xc544, 0xd30c, 0xace0, 0xd30c, 0xace0, 0xd314, 0xb77c, 0xc6b0,
0xd54f, 0xcf00, 0xc5b8, 0xd3ec, 0xb098, 0xd398, 0xd3ec, 0xd2b8, 0xbaa8, 0xb974, 0xc988, 0xbe44,
0xb77c, 0xb85c, 0xd1b5, 0xac00, 0xc0ac, 0xc774, 0xd310, 0xd0c0, 0xd788, 0xd2f0, 0xd0c0, 0xb77c,
0xc640, 0xd1b5, 0xac00, 0xd0c0, 0xd478, 0xc6e8, 0xc774, 0xd06c, 0xc6d4, 0xb9ac, 0xc2a4, 0x43,
0x65, 0x7a, 0x61, 0x79, 0xee, 0x72, 0x51, 0x61, 0x68, 0xee, 0x72, 0x65,
0x53, 0x65, 0x70, 0x74, 0x65, 0x4b, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0xee,
0x44, 0x61, 0x72, 0x75, 0x73, 0x65, 0x6c, 0x61, 0x6d, 0x43, 0x69, 0x62,
0xfb, 0x74, 0xee, 0x43, 0x75, 0x62, 0x61, 0x58, 0x61, 0x72, 0x74, 0xfb,
0x6d, 0x4b, 0xee, 0x67, 0x61, 0x6c, 0xee, 0x4b, 0xee, 0x6e, 0x15f, 0x61,
0x73, 0x61, 0x4c, 0xee, 0x62, 0x72, 0x65, 0x76, 0xee, 0x6c, 0x4d, 0x61,
0x73, 0x65, 0x72, 0xfb, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0xee, 0x15f, 0xfb,
0x4e, 0x61, 0xee, 0x72, 0x6f, 0x62, 0xee, 0x54, 0x72, 0x61, 0x62, 0x6c,
0xfb, 0x73, 0x54, 0xfb, 0x6e, 0x69, 0x73, 0x41, 0x72, 0xfb, 0x62, 0x61,
0x42, 0x61, 0x68, 0xee, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x61, 0x73, 0x42, 0x65, 0x6c, 0xee, 0x7a, 0x65, 0x43, 0x61, 0x6e, 0x63,
0xfb, 0x6e, 0x43, 0x69, 0xfb, 0x64, 0x61, 0x64, 0x20, 0x4a, 0x75, 0x61,
0x72, 0x65, 0x7a, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x20, 0x52, 0xee, 0x6b,
0x61, 0x44, 0x6f, 0x6d, 0xee, 0x6e, 0xee, 0x6b, 0x61, 0x4a, 0x61, 0x6d,
0x61, 0xee, 0x6b, 0x61, 0x4d, 0x65, 0x72, 0xee, 0x64, 0x61, 0x42, 0x65,
0xfb, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x43, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x79, 0x61,
0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x79,
0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x50, 0x6f, 0x72, 0x74, 0x6f,
0x20, 0x52, 0xee, 0x6b, 0x6f, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x42,
0x61, 0x72, 0x74, 0x68, 0x65, 0x6c, 0x65, 0x6d, 0x79, 0x44, 0x61, 0x76,
0xee, 0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0xee, 0x45, 0x6d, 0x61, 0x6e,
0x45, 0x15f, 0x71, 0x61, 0x62, 0x61, 0x74, 0x41, 0x74, 0xee, 0x72, 0x61,
0x76, 0x42, 0x65, 0x78, 0x64, 0x61, 0x42, 0x65, 0x68, 0x72, 0x65, 0x79,
0x6e, 0x42, 0x61, 0x6b, 0xfb, 0x42, 0x65, 0x79, 0x72, 0xfb, 0x74, 0x42,
0xee, 0x15f, 0x6b, 0x65, 0x6b, 0x42, 0x72, 0xfb, 0x6e, 0x65, 0x79, 0xc7,
0xee, 0x74, 0x61, 0x15e, 0x61, 0x6d, 0x44, 0xee, 0x6c, 0xee, 0x44, 0xfb,
0x62, 0x61, 0xee, 0x44, 0x75, 0x15f, 0x65, 0x6e, 0x62, 0x65, 0x58, 0x65,
0x7a, 0x65, 0x42, 0x61, 0x6a, 0x61, 0x72, 0xea, 0x20, 0x48, 0x6f, 0x20,
0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0xea, 0x43, 0x61, 0x79,
0x61, 0x70, 0xfb, 0x72, 0x61, 0x51, 0x75, 0x64, 0x73, 0x4b, 0x61, 0x62,
0xfb, 0x6c, 0x4b, 0x61, 0x72, 0x61, 0xe7, 0xee, 0x4b, 0x61, 0x74, 0x6d,
0x61, 0x6e, 0x64, 0xfb, 0x58, 0x61, 0x6e, 0x64, 0xee, 0x67, 0x61, 0x4b,
0xfb, 0x61, 0x6c, 0x61, 0x20, 0x4c, 0xfb, 0x6d, 0x70, 0xfb, 0x72, 0x4b,
0xfb, 0xe7, 0xee, 0x6e, 0x67, 0x4b, 0x75, 0x77, 0x65, 0x79, 0x74, 0x4d,
0x61, 0x6e, 0xee, 0x6c, 0x61, 0x4c, 0x65, 0x66, 0x6b, 0x6f, 0x15f, 0x65,
0x51, 0x65, 0x74, 0x65, 0x72, 0x51, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xee, 0x6e, 0x53, 0x65, 0x6d,
0x65, 0x72, 0x6b, 0x61, 0x6e, 0x64, 0x53, 0x65, 0xfb, 0x6c, 0x15e, 0x61,
0x6e, 0x67, 0x68, 0x61, 0x79, 0x53, 0xee, 0x6e, 0x67, 0x61, 0x70, 0xfb,
0x72, 0x54, 0x61, 0xee, 0x70, 0x65, 0xee, 0x54, 0x61, 0x15f, 0x6b, 0x65,
0x6e, 0x74, 0x54, 0x69, 0x66, 0x6c, 0xee, 0x73, 0xdb, 0x6c, 0x61, 0x6e,
0x62, 0x61, 0x74, 0x61, 0x72, 0xdb, 0x72, 0xfb, 0x6d, 0xe7, 0xee, 0xdb,
0x73, 0x74, 0x2d, 0x4e, 0x65, 0x72, 0x61, 0x52, 0x65, 0x77, 0x61, 0x6e,
0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0xea, 0x42, 0x65, 0x72, 0x6d, 0xfb, 0x64, 0x61, 0x47, 0x69, 0x72, 0x61,
0x76, 0xea, 0x6e, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x79, 0x61, 0x79,
0xea, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x79, 0x61, 0x20, 0x42,
0x61, 0x15f, 0xfb, 0x72, 0x53, 0xee, 0x64, 0x6e, 0x65, 0x79, 0x41, 0x73,
0x74, 0x72, 0x61, 0x78, 0x61, 0x6e, 0x41, 0x74, 0xee, 0x6e, 0x61, 0x42,
0x65, 0x72, 0x6c, 0xee, 0x6e, 0x42, 0x75, 0x6b, 0x72, 0x65, 0x15f, 0x42,
0xfb, 0x64, 0x61, 0x70, 0x65, 0x15f, 0x74, 0x42, 0xfb, 0x73, 0xee, 0x6e,
0x67, 0x65, 0x6e, 0x4b, 0x69, 0x15f, 0xee, 0x6e, 0x65, 0x77, 0x44, 0xfb,
0x62, 0x6c, 0xee, 0x6e, 0x47, 0xee, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61,
0x72, 0x48, 0x65, 0x6c, 0x73, 0xee, 0x6e, 0x6b, 0xee, 0x47, 0x69, 0x72,
0x61, 0x76, 0x61, 0x20, 0x4d, 0x61, 0x6e, 0xea, 0x53, 0x74, 0x65, 0x6e,
0x62, 0x6f, 0x6c, 0x4b, 0x61, 0x6c, 0xee, 0x6e, 0xee, 0x6e, 0x67, 0x72,
0x61, 0x64, 0x4b, 0xee, 0x72, 0x6f, 0x76, 0x4b, 0xee, 0x65, 0x76, 0x4c,
0xee, 0x7a, 0x62, 0x6f, 0x6e, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62,
0xfb, 0x72, 0x67, 0x4d, 0x61, 0x64, 0x72, 0xee, 0x64, 0x4d, 0xee, 0x6e,
0x73, 0x6b, 0x50, 0x61, 0x72, 0xee, 0x73, 0x50, 0x6f, 0x64, 0x67, 0x6f,
0x72, 0xee, 0x6b, 0x61, 0x52, 0xee, 0x67, 0x61, 0x53, 0x61, 0x6e, 0x20,
0x4d, 0x61, 0x72, 0xee, 0x6e, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x79, 0x62,
0x6f, 0x73, 0x6e, 0x61, 0x55, 0x73, 0x6b, 0x75, 0x70, 0x54, 0x61, 0x6c,
0xee, 0x6e, 0x54, 0xee, 0x72, 0x61, 0x6e, 0x56, 0x61, 0x74, 0xee, 0x6b,
0x61, 0x6e, 0x56, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x56, 0xee, 0x6c, 0x6e,
0xfb, 0x73, 0x57, 0x61, 0x72, 0x15f, 0x6f, 0x76, 0x61, 0x5a, 0xfb, 0x72,
0xee, 0x68, 0x4d, 0x61, 0x6c, 0x64, 0xee, 0x76, 0x47, 0x69, 0x72, 0x61,
0x76, 0x61, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x6c, 0x79, 0x61, 0x79,
0xea, 0x46, 0xee, 0x6a, 0xee, 0x54, 0x61, 0x68, 0xee, 0x74, 0xee, 0x411,
0x430, 0x433, 0x438, 0x411, 0x440, 0x430, 0x437, 0x437, 0x430, 0x432, 0x438, 0x43b,
0x43b, 0x414, 0x430, 0x440, 0x20, 0x44d, 0x441, 0x20, 0x421, 0x430, 0x43b, 0x430,
0x430, 0x43c, 0x416, 0x438, 0x431, 0x443, 0x442, 0x438, 0x42d, 0x43b, 0x20, 0x410,
0x439, 0x443, 0x43d, 0x416, 0x443, 0x431, 0x430, 0x41b, 0x438, 0x431, 0x440, 0x435,
0x432, 0x438, 0x43b, 0x43b, 0x41d, 0x436, 0x430, 0x43c, 0x435, 0x43d, 0x430, 0x41d,
0x439, 0x430, 0x43c, 0x435, 0x423, 0x438, 0x43d, 0x434, 0x445, 0x443, 0x43a, 0x410,
0x43d, 0x43a, 0x43e, 0x440, 0x438, 0x436, 0x420, 0x438, 0x43e, 0x20, 0x413, 0x430,
0x43b, 0x43b, 0x435, 0x433, 0x43e, 0x441, 0x423, 0x448, 0x443, 0x430, 0x439, 0x430,
0x411, 0x430, 0x439, 0x438, 0x430, 0x411, 0x430, 0x438, 0x439, 0x430, 0x20, 0x411,
0x430, 0x43d, 0x434, 0x435, 0x440, 0x430, 0x441, 0x41a, 0x44d, 0x43c, 0x431, 0x440,
0x438, 0x436, 0x20, 0x411, 0x435, 0x439, 0x427, 0x438, 0x445, 0x443, 0x430, 0x445,
0x443, 0x430, 0x41a, 0x443, 0x439, 0x430, 0x431, 0x430, 0x414, 0x43e, 0x443, 0x441,
0x43e, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43a, 0x42d, 0x43b, 0x20, 0x421, 0x430,
0x43b, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x413, 0x440, 0x430, 0x43d, 0x434, 0x20,
0x422, 0x4af, 0x440, 0x43a, 0x413, 0x432, 0x430, 0x434, 0x435, 0x43b, 0x443, 0x43f,
0x435, 0x413, 0x443, 0x430, 0x439, 0x430, 0x43a, 0x438, 0x43b, 0x413, 0x443, 0x439,
0x430, 0x43d, 0x430, 0x42d, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x43b, 0x43b, 0x43e,
0x41f, 0x438, 0x442, 0x435, 0x440, 0x441, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20,
0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x422, 0x435, 0x43b, 0x43b, 0x20,
0x421, 0x438, 0x442, 0x438, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d,
0x430, 0x412, 0x438, 0x43d, 0x441, 0x435, 0x43d, 0x2c, 0x20, 0x418, 0x43d, 0x434,
0x438, 0x430, 0x43d, 0x430, 0x41b, 0x443, 0x438, 0x437, 0x432, 0x438, 0x43b, 0x43b,
0x41c, 0x43e, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x43b, 0x43b, 0x43e, 0x2c, 0x20,
0x41a, 0x435, 0x43d, 0x442, 0x443, 0x43a, 0x43a, 0x438, 0x41b, 0x43e, 0x441, 0x2d,
0x410, 0x43d, 0x436, 0x435, 0x43b, 0x435, 0x441, 0x41b, 0x43e, 0x443, 0x44d, 0x440,
0x20, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x435, 0x441, 0x20, 0x41a, 0x443, 0x430,
0x442, 0x435, 0x41c, 0x435, 0x445, 0x438, 0x43a, 0x43e, 0x20, 0x448, 0x430, 0x430,
0x440, 0x44b, 0x41c, 0x43e, 0x43d, 0x441, 0x435, 0x440, 0x430, 0x442, 0x411, 0x435,
0x443, 0x43b, 0x430, 0x2c, 0x20, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20,
0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a,
0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x43d, 0x44b, 0x43d, 0x20, 0x431,
0x43e, 0x440, 0x431, 0x43e, 0x440, 0x443, 0x41d, 0x44c, 0x44e, 0x2d, 0x421, 0x430,
0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20,
0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41f, 0x43e, 0x440, 0x442, 0x20, 0x43e,
0x444, 0x20, 0x421, 0x43f, 0x435, 0x439, 0x43d, 0x41f, 0x43e, 0x440, 0x442, 0x43e,
0x20, 0x412, 0x435, 0x43b, 0x438, 0x43e, 0x420, 0x44d, 0x43d, 0x43a, 0x438, 0x43d,
0x20, 0x418, 0x43d, 0x43b, 0x435, 0x442, 0x420, 0x435, 0x437, 0x43e, 0x43b, 0x443,
0x442, 0x418, 0x442, 0x442, 0x43e, 0x43a, 0x43a, 0x43e, 0x440, 0x442, 0x43e, 0x43e,
0x440, 0x43c, 0x438, 0x438, 0x442, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x411, 0x430,
0x440, 0x442, 0x435, 0x43b, 0x435, 0x43c, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x20,
0x416, 0x43e, 0x43d, 0x441, 0x421, 0x430, 0x43d, 0x442, 0x430, 0x20, 0x41b, 0x44e,
0x447, 0x438, 0x44f, 0x421, 0x432, 0x438, 0x444, 0x442, 0x20, 0x41a, 0x430, 0x440,
0x440, 0x435, 0x43d, 0x442, 0x423, 0x438, 0x43d, 0x43d, 0x438, 0x43f, 0x435, 0x433,
0x414, 0x44d, 0x432, 0x438, 0x441, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x434,
0x2d, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x41c, 0x430, 0x43a, 0x41c, 0x451, 0x440,
0x434, 0x43e, 0x421, 0x430, 0x43e, 0x430, 0x41b, 0x43e, 0x43d, 0x433, 0x439, 0x435,
0x440, 0x431, 0x438, 0x435, 0x43d, 0x416, 0x430, 0x43a, 0x430, 0x440, 0x442, 0x430,
0x416, 0x430, 0x439, 0x430, 0x43f, 0x443, 0x440, 0x430, 0x41a, 0x430, 0x43d, 0x434,
0x44b, 0x433, 0x430, 0x41a, 0x443, 0x447, 0x438, 0x4a3, 0x41c, 0x430, 0x43a, 0x430,
0x441, 0x441, 0x430, 0x440, 0x420, 0x438, 0x439, 0x430, 0x434, 0x4ae, 0x440, 0x4af,
0x43c, 0x447, 0x4af, 0x412, 0x438, 0x435, 0x43d, 0x442, 0x438, 0x430, 0x43d, 0x41a,
0x430, 0x43d, 0x430, 0x440, 0x438, 0x41a, 0x430, 0x43f, 0x435, 0x20, 0x412, 0x435,
0x440, 0x434, 0x435, 0x422, 0x4af, 0x448, 0x442, 0x4af, 0x43a, 0x20, 0x416, 0x43e,
0x440, 0x436, 0x438, 0x44f, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x415, 0x43b, 0x435,
0x43d, 0x430, 0x411, 0x440, 0x43e, 0x443, 0x43a, 0x435, 0x43d, 0x20, 0x425, 0x438,
0x43b, 0x411, 0x443, 0x437, 0x438, 0x43d, 0x433, 0x435, 0x43d, 0x416, 0x435, 0x440,
0x441, 0x438, 0x421, 0x43a, 0x43e, 0x43f, 0x439, 0x435, 0x424, 0x430, 0x434, 0x443,
0x446, 0x41a, 0x440, 0x438, 0x441, 0x43c, 0x430, 0x441, 0x41e, 0x43a, 0x43b, 0x44d,
0x43d, 0x434, 0x427, 0x443, 0x443, 0x43a, 0x418, 0x441, 0x442, 0x435, 0x440, 0x424,
0x438, 0x436, 0x438, 0x41a, 0x43e, 0x437, 0x440, 0x430, 0x435, 0x41a, 0x443, 0x430,
0x436, 0x430, 0x43b, 0x435, 0x439, 0x43d, 0x41c, 0x430, 0x436, 0x443, 0x440, 0x43e,
0x41c, 0x430, 0x440, 0x43a, 0x435, 0x437, 0x430, 0x441, 0x41f, 0x43e, 0x440, 0x442,
0x20, 0x41c, 0x43e, 0x44d, 0x440, 0x441, 0x431, 0x438, 0x420, 0x430, 0x433, 0x43e,
0x442, 0x43e, 0x43d, 0x433, 0x430, 0xead, 0xeb2, 0xe9a, 0xeb4, 0xe94, 0xec1, 0xe88,
0xe99, 0xead, 0xeb1, 0xe81, 0xe84, 0xea3, 0xeb2, 0xec1, 0xead, 0xe94, 0xe94, 0xeb4,
0xeaa, 0x20, 0xead, 0xeb0, 0xe9a, 0xeb2, 0xe9a, 0xeb2, 0xec1, 0xead, 0xea7, 0xec0,
0xe88, 0xe8d, 0xead, 0xeb1, 0xeaa, 0xea1, 0xeb2, 0xea3, 0xeb2, 0xe9a, 0xeb2, 0xea1,
0xeb2, 0xec2, 0xe81, 0xe9a, 0xeb1, 0xe87, 0xe81, 0xeb8, 0xe8d, 0xe9a, 0xeb2, 0xe99,
0xe88, 0xeb9, 0xe9a, 0xeb4, 0xeaa, 0xec0, 0xe8a, 0xebb, 0xeb2, 0xec1, 0xe9a, 0xea5,
0xe99, 0xec4, 0xe97, 0xea3, 0xecc, 0xe9a, 0xea3, 0xeb2, 0xe8a, 0xeb2, 0xea7, 0xeb4,
0xea7, 0xe9a, 0xeb9, 0xe88, 0xeb9, 0xea1, 0xe9a, 0xeb9, 0xea3, 0xeb2, 0xec4, 0xe84,
0xec2, 0xea3, 0xe84, 0xeb2, 0xe8a, 0xeb2, 0xe9a, 0xea5, 0xeb1, 0xe87, 0xe81, 0xeb2,
0xe8a, 0xeb5, 0xea7, 0xe95, 0xeb2, 0xec2, 0xe84, 0xe99, 0xeb2, 0xe84, 0xea3, 0xeb5,
0xe94, 0xeb2, 0xe81, 0xeb2, 0xe94, 0xeb2, 0xec0, 0xead, 0xeaa, 0xeaa, 0xeb0, 0xea5,
0xeb2, 0xea1, 0xe88, 0xeb5, 0xe9a, 0xeb9, 0xe95, 0xeb4, 0xe94, 0xeb9, 0xead, 0xeb2,
0xea5, 0xeb2, 0xec0, 0xead, 0xea7, 0x20, 0xead, 0xeb2, 0xea2, 0xeb9, 0xe99, 0xe9f,
0xea3, 0xeb5, 0xe97, 0xeb2, 0xea7, 0xe81, 0xeb2, 0xe9a, 0xecd, 0xec2, 0xea3, 0xe99,
0xeae, 0xeb2, 0xea3, 0xeb2, 0xec0, 0xea3, 0xec2, 0xe88, 0xeae, 0xeb1, 0xe99, 0xec0,
0xe99, 0xe94, 0xeaa, 0xec0, 0xe9a, 0xeb5, 0xe81, 0xe88, 0xeb9, 0xe9a, 0xeb2, 0xe84,
0xeb3, 0xe9b, 0xeb2, 0xea5, 0xeb2, 0xe84, 0xeb2, 0xe97, 0xeb9, 0xea1, 0xe84, 0xeb5,
0xe81, 0xeb2, 0xea5, 0xeb5, 0xe81, 0xeb4, 0xe99, 0xe8a, 0xeb2, 0xe8a, 0xeb2, 0xea5,
0xeb2, 0xec2, 0xe81, 0xeaa, 0xea5, 0xeb4, 0xec0, 0xe9a, 0xebc, 0xeb5, 0xea7, 0xeb4,
0xea7, 0xec2, 0xea5, 0xec0, 0xea1, 0xea5, 0xea7, 0xe99, 0xe94, 0xeb2, 0xea5, 0xeb9,
0xe9a, 0xeb3, 0xe9a, 0xeb2, 0xe8a, 0xeb4, 0xea5, 0xeb9, 0xe8a, 0xeb2, 0xe81, 0xeb2,
0xea1, 0xeb2, 0xea5, 0xeb2, 0xec2, 0xe9a, 0xea1, 0xeb2, 0xe9b, 0xeb9, 0xec2, 0xe95,
0xea1, 0xeb2, 0xec0, 0xe8a, 0xea3, 0xeb9, 0xead, 0xeb3, 0xe9a, 0xeb2, 0xe9a, 0xeb2,
0xec0, 0xe99, 0xec2, 0xea1, 0xe81, 0xeb2, 0xe94, 0xeb5, 0xe8a, 0xeb9, 0xea1, 0xead,
0xe99, 0xec2, 0xea3, 0xec0, 0xea7, 0xe8d, 0xec4, 0xe99, 0xec2, 0xea3, 0xe9a, 0xeb5,
0xec0, 0xead, 0xe99, 0xe88, 0xeb2, 0xec0, 0xea1, 0xe99, 0xec8, 0xeb2, 0xe99, 0xeb5,
0xead, 0xeb2, 0xec0, 0xea1, 0xe99, 0xeb9, 0xec1, 0xead, 0xe81, 0xe8a, 0xead, 0xe94,
0xead, 0xeb9, 0xe81, 0xeb2, 0xe94, 0xeb9, 0xe81, 0xeb9, 0xe9b, 0xecd, 0xec2, 0xe95,
0x2d, 0xec2, 0xe99, 0xec2, 0xea7, 0xe8a, 0xeb2, 0xea7, 0xec2, 0xe95, 0xec0, 0xea1,
0xe97, 0xea3, 0xeb4, 0xec2, 0xe9e, 0xea5, 0xeb4, 0xe95, 0xeb9, 0xe99, 0xeb4, 0xeaa,
0xea7, 0xeb5, 0xe99, 0xeae, 0xeb9, 0xe94, 0xead, 0xeb2, 0xec1, 0xe94, 0xe81, 0xec1,
0xead, 0xe99, 0xec0, 0xe84, 0xeb5, 0xec0, 0xea3, 0xe81, 0xec1, 0xead, 0xe99, 0xe81,
0xeb4, 0xe99, 0xea5, 0xeb2, 0xec1, 0xead, 0xe99, 0xe97, 0xeb4, 0xe81, 0xebb, 0xea7,
0xead, 0xeb2, 0xea3, 0xeb2, 0xe81, 0xebb, 0xea7, 0xe99, 0xeb2, 0xe9a, 0xebb, 0xea7,
0xec2, 0xe99, 0xeaa, 0x20, 0xec4, 0xead, 0xec0, 0xea3, 0xeaa, 0xe84, 0xeb2, 0xe95,
0xeb2, 0xea1, 0xeb2, 0xe81, 0xeb2, 0xe84, 0xecd, 0xec2, 0xe94, 0xe9a, 0xeb2, 0xe88,
0xeb9, 0xe88, 0xeb8, 0xe8d, 0xea5, 0xeb2, 0x20, 0xea3, 0xeb4, 0xec2, 0xead, 0xe88,
0xeb2, 0xec0, 0xea1, 0xeb1, 0xe99, 0xec2, 0xe94, 0xe8a, 0xec8, 0xeb2, 0xea3, 0xeb4,
0xec2, 0xe81, 0x20, 0xec1, 0xe81, 0xea7, 0xec0, 0xea5, 0xe81, 0xead, 0xeaa, 0xe8a,
0xeb2, 0xe99, 0xe95, 0xeb2, 0xec1, 0xe8a, 0xe99, 0xeae, 0xea7, 0xe99, 0xec1, 0xe8a,
0xe99, 0xea5, 0xeb8, 0xe8d, 0xeaa, 0xecc, 0xe95, 0xeb9, 0xe84, 0xeb9, 0xec1, 0xea1,
0xe99, 0xead, 0xeb9, 0xe8a, 0xeb9, 0xec0, 0xead, 0xe8d, 0xead, 0xeb2, 0xea3, 0xeb9,
0xe9a, 0xeb2, 0xead, 0xeb0, 0xe8a, 0xeb8, 0xe99, 0xe8a, 0xeb4, 0xead, 0xead, 0xe87,
0xead, 0xeb2, 0xe97, 0xeb4, 0xec2, 0xe84, 0xe84, 0xeb2, 0xe99, 0xe9a, 0xeb2, 0xec0,
0xea2, 0xe8d, 0xe9a, 0xeb2, 0xec0, 0xeae, 0xe8d, 0x20, 0xec1, 0xe9a, 0xe99, 0xec0,
0xe94, 0xeb5, 0xea3, 0xeb2, 0xeaa, 0xe9a, 0xeb2, 0xec0, 0xe9a, 0xe94, 0xead, 0xeaa,
0xe9a, 0xeb5, 0xec0, 0xea5, 0xea1, 0xec0, 0xe9a, 0xea5, 0xeb5, 0xe8a, 0xe9a, 0xea5,
0xeb2, 0xe99, 0x2d, 0xe8a, 0xeb2, 0xe9a, 0xea5, 0xead, 0xe99, 0xe9a, 0xebb, 0xea7,
0x20, 0xea7, 0xeb4, 0xeaa, 0xe95, 0xeb2, 0xec2, 0xe9a, 0xec2, 0xe81, 0xe95, 0xeb2,
0xe9a, 0xead, 0xe8d, 0xe8a, 0xecc, 0xec1, 0xe84, 0xea1, 0xe9a, 0xea3, 0xeb4, 0xe94,
0x20, 0xec0, 0xe9a, 0xe81, 0xeb1, 0xe87, 0xe9b, 0xeb9, 0xea3, 0xeb1, 0xe87, 0xe88,
0xeb5, 0xec1, 0xe84, 0xe99, 0xe81, 0xeb8, 0xe99, 0xe84, 0xeb2, 0xea3, 0xeb2, 0xe81,
0xeb1, 0xeaa, 0xe84, 0xeb2, 0xec0, 0xea2, 0xe99, 0xec0, 0xe84, 0xec1, 0xea1, 0xe99,
0xe8a, 0xeb4, 0xe84, 0xeb2, 0xec2, 0xe81, 0xe8a, 0xeb4, 0xea7, 0xeb2, 0xea7, 0xeb2,
0xe8a, 0xeb5, 0xead, 0xeb9, 0xe94, 0xeb1, 0xe94, 0x20, 0xe84, 0xebb, 0xea7, 0xec0,
0xea3, 0xe8a, 0xe84, 0xead, 0xeaa, 0xe95, 0xeb2, 0xea3, 0xeb4, 0xe81, 0xeb2, 0xec0,
0xe84, 0xea3, 0xeaa, 0xe95, 0xeb1, 0xe99, 0xe81, 0xeb8, 0xea2, 0xeb2, 0xe9a, 0xeb2,
0xe81, 0xeb7, 0xea3, 0xeb2, 0xec0, 0xe8a, 0xebb, 0xeb2, 0xec0, 0xe94, 0xe99, 0xea1,
0xeb2, 0xe81, 0xec1, 0xe8a, 0xe99, 0xe94, 0xecd, 0xeaa, 0xeb1, 0xe99, 0xe94, 0xecd,
0xeaa, 0xeb1, 0xe99, 0x20, 0xe84, 0xea3, 0xeb5, 0xe81, 0xec0, 0xe94, 0xeb1, 0xe99,
0xec0, 0xea7, 0xeb5, 0xe94, 0xeb5, 0xe97, 0xea3, 0xead, 0xe8d, 0xec2, 0xe94, 0xea1,
0xeb4, 0xe99, 0xeb4, 0xe81, 0xeb2, 0xec0, 0xead, 0xe94, 0xea1, 0xead, 0xe99, 0xe95,
0xead, 0xe99, 0xec0, 0xead, 0xea3, 0xeb8, 0xec0, 0xe99, 0xe9b, 0xeb5, 0xec0, 0xead,
0xea7, 0x20, 0xe8a, 0xeb2, 0xea7, 0xeb2, 0xe94, 0xecd, 0xe9f, 0xead, 0xe94, 0x20,
0xec0, 0xe99, 0xea7, 0xeaa, 0xeb1, 0xe99, 0xe9f, 0xecd, 0xe95, 0xeb2, 0xec0, 0xea5,
0xe8a, 0xeb2, 0xec0, 0xe81, 0xea5, 0xe94, 0xec0, 0xe9a, 0xe81, 0xeb9, 0xeaa, 0xec0,
0xe9a, 0xec1, 0xe81, 0xea3, 0xe99, 0x20, 0xec0, 0xe97, 0xeb5, 0xe81, 0xec0, 0xe81,
0xea3, 0xe99, 0xeb2, 0xe94, 0xeb2, 0xe81, 0xeb2, 0xea7, 0xec0, 0xe94, 0xea5, 0xeb9,
0xe9a, 0xe81, 0xebb, 0xea7, 0xec0, 0xe95, 0xea1, 0xeb2, 0xea5, 0xeb2, 0xe81, 0xebb,
0xea7, 0xea2, 0xeb2, 0xe81, 0xeb4, 0xea5, 0xe81, 0xeb9, 0xea2, 0xeb2, 0xe99, 0xeb2,
0xeae, 0xeb2, 0xea5, 0xeb4, 0xec1, 0xe9f, 0xeb1, 0xe81, 0xeae, 0xeb2, 0xea7, 0xeb2,
0xe99, 0xeb2, 0xec0, 0xead, 0xec2, 0xea1, 0xe8a, 0xeb4, 0xec2, 0xea2, 0xead, 0xeb4,
0xe99, 0xe94, 0xeb5, 0xec0, 0xead, 0xe99, 0xeb2, 0xec2, 0xe9e, 0xea5, 0xeb4, 0xeaa,
0xe99, 0xecb, 0xead, 0xe81, 0x2c, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d,
0xe99, 0xeb2, 0xea1, 0xeb2, 0xec0, 0xea3, 0xe99, 0xec2, 0xe81, 0x2c, 0x20, 0xead,
0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xe9b, 0xeb5, 0xec0, 0xe95, 0xeb5,
0xeaa, 0xec0, 0xe9b, 0xeb5, 0xe81, 0x2c, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94,
0xe8d, 0xe99, 0xeb2, 0xec0, 0xe97, 0xea7, 0x20, 0xe8a, 0xeb4, 0xe95, 0xeb5, 0x2c,
0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xea7, 0xeb5, 0xec0,
0xea7, 0x2c, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xea7,
0xeb4, 0xe99, 0xec0, 0xe8a, 0xe99, 0xec0, 0xe99, 0xeaa, 0x2c, 0x20, 0xead, 0xeb4,
0xe99, 0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2, 0xea7, 0xeb4, 0xe99, 0xeb2, 0xec1, 0xea1,
0xeb1, 0xe81, 0x2c, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xe99, 0xeb2,
0xead, 0xeb5, 0xe99, 0xeb9, 0xea7, 0xeb4, 0xe81, 0xead, 0xeb5, 0xe81, 0xebb, 0xea7,
0xea5, 0xeb4, 0xe94, 0xe88, 0xeb2, 0xec4, 0xea1, 0xe81, 0xeb2, 0xe88, 0xeb9, 0xec2,
0xe99, 0xeab, 0xea5, 0xeb8, 0xe8d, 0xea7, 0xeb4, 0xea7, 0xea1, 0xead, 0xe99, 0xe95,
0xeb5, 0xec0, 0xe8a, 0xea7, 0xec2, 0xea5, 0x2c, 0x20, 0xec0, 0xe84, 0xe99, 0xe97,
0xeb1, 0xe81, 0xe81, 0xeb5, 0xe84, 0xea3, 0xeb2, 0xec0, 0xea5, 0xe99, 0xe94, 0xeb4,
0xe81, 0xea5, 0xeb2, 0xe9b, 0xeb2, 0xeaa, 0xea5, 0xeb5, 0xea1, 0xeb2, 0xea5, 0xead,
0xeaa, 0xec1, 0xead, 0xe99, 0xec0, 0xe88, 0xea5, 0xeb5, 0xeaa, 0xec2, 0xea5, 0xec0,
0xea7, 0xeb5, 0x20, 0xe9e, 0xea3, 0xeb4, 0xe99, 0xe8a, 0xecc, 0x20, 0xe84, 0xea7,
0xe94, 0xec0, 0xe95, 0xeb5, 0xea1, 0xeb2, 0xec0, 0xe8a, 0xec2, 0xead, 0xea1, 0xeb2,
0xe99, 0xeb2, 0xe81, 0xebb, 0xea7, 0xea1, 0xeb2, 0xec0, 0xe99, 0xebb, 0xeb2, 0xeaa,
0xecc, 0xea1, 0xeb2, 0xea3, 0xeb5, 0xec2, 0xe81, 0xe94, 0xea1, 0xeb2, 0xe97, 0xeb5,
0xe99, 0xeb4, 0xe81, 0xea1, 0xeb2, 0xe95, 0xeb2, 0xec2, 0xea1, 0xea3, 0xead, 0xeaa,
0xea1, 0xeb2, 0xe8a, 0xeb2, 0xe94, 0xea5, 0xeb2, 0xe99, 0xec0, 0xea1, 0xec2, 0xe99,
0xea1, 0xeb4, 0xe99, 0xeb5, 0xec0, 0xea1, 0xeb5, 0xea3, 0xeb4, 0xe94, 0xeb2, 0xec0,
0xea1, 0xeb1, 0xe94, 0xea5, 0xeb2, 0xe81, 0xeb1, 0xe94, 0xea5, 0xeb2, 0xec0, 0xea1,
0xeb1, 0xe81, 0xe8a, 0xeb4, 0xec2, 0xe81, 0x20, 0xe8a, 0xeb4, 0xe95, 0xeb5, 0xea1,
0xeb4, 0xe81, 0xebb, 0xea7, 0xea5, 0xead, 0xe99, 0xea1, 0xead, 0xe99, 0xe95, 0xeb1,
0xe99, 0xea1, 0xead, 0xe99, 0xec0, 0xe95, 0xeb5, 0xec0, 0xea3, 0xe8d, 0xea1, 0xead,
0xe99, 0xec0, 0xe95, 0xea7, 0xeb4, 0xec0, 0xe94, 0xec2, 0xead, 0xea1, 0xead, 0xe99,
0xec0, 0xe8a, 0xeb5, 0xea3, 0xeb1, 0xe94, 0xec1, 0xe99, 0xeaa, 0xe8a, 0xecd, 0xe99,
0xeb4, 0xea7, 0xea2, 0xead, 0xe81, 0xe99, 0xead, 0xea1, 0xe99, 0xecd, 0xea3, 0xead,
0xe99, 0xeae, 0xeb2, 0xec0, 0xe9a, 0xeb5, 0xea5, 0xeb2, 0x2c, 0x20, 0xe99, 0xead,
0xe94, 0x20, 0xe94, 0xeb2, 0xec2, 0xe81, 0xe95, 0xeb2, 0xec0, 0xe8a, 0xeb1, 0xe99,
0xec0, 0xe88, 0xeb5, 0x2c, 0x20, 0xe99, 0xead, 0xe94, 0x20, 0xe94, 0xeb2, 0xec2,
0xe81, 0xe95, 0xeb2, 0xe99, 0xeb4, 0xea7, 0x20, 0xe8a, 0xeb2, 0xec0, 0xea5, 0xea1,
0x2c, 0x20, 0xe99, 0xead, 0xe94, 0x20, 0xe94, 0xeb2, 0xec2, 0xe81, 0xe95, 0xeb2,
0xe99, 0xeb9, 0xe81, 0xec2, 0xead, 0xe88, 0xeb4, 0xe99, 0xeb2, 0xe81, 0xeb2, 0xe9e,
0xeb2, 0xe99, 0xeb2, 0xea1, 0xeb2, 0xe9e, 0xeb2, 0xea3, 0xeb2, 0xea1, 0xeb2, 0xea3,
0xeb4, 0xec2, 0xe9a, 0xe9f, 0xeb4, 0xe99, 0xeb4, 0xe81, 0xe9b, 0xecd, 0xec2, 0xe95,
0xec1, 0xe9b, 0xea3, 0xe87, 0xe9e, 0xead, 0xe94, 0xead, 0xead, 0xe9a, 0xeaa, 0xeb0,
0xec0, 0xe9b, 0xe99, 0xe9b, 0xecd, 0xe95, 0xeb9, 0x20, 0xec0, 0xea7, 0xea7, 0xea2,
0xeb9, 0xec0, 0xe9b, 0xeb5, 0xec2, 0xe97, 0xea3, 0xeb4, 0xec2, 0xe81, 0xe9e, 0xeb8,
0xe99, 0xe97, 0xeb2, 0x20, 0xead, 0xeb0, 0xea3, 0xeb5, 0xe99, 0xeb2, 0xeaa, 0xecc,
0xec1, 0xea3, 0xe99, 0xe81, 0xeb4, 0xe99, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xea5,
0xeb1, 0xe94, 0xec0, 0xea3, 0xe8a, 0xeb4, 0xec0, 0xe9f, 0xec0, 0xea3, 0xe88, 0xeb4,
0xe99, 0xeb2, 0xec0, 0xea3, 0xec2, 0xe8a, 0xea5, 0xeb8, 0xe94, 0xea3, 0xeb4, 0xec2,
0xead, 0x20, 0xe9a, 0xea3, 0xeb1, 0xe99, 0xec2, 0xe81, 0xe8a, 0xeb1, 0xe99, 0xe95,
0xeb2, 0xec0, 0xea3, 0xea1, 0xe8a, 0xeb1, 0xe99, 0xe95, 0xeb5, 0xead, 0xeb2, 0xec2,
0xe81, 0xe8a, 0xeb2, 0xe99, 0xec2, 0xe95, 0xec2, 0xe94, 0xea1, 0xeb4, 0xe99, 0xec2,
0xe81, 0xec0, 0xe8a, 0xebb, 0xeb2, 0x20, 0xec0, 0xe9b, 0xebb, 0xeb2, 0xec2, 0xea5,
0xead, 0xeb4, 0xec2, 0xe95, 0xe84, 0xecd, 0xe97, 0xebb, 0xea7, 0xea1, 0xeb4, 0xe94,
0xe8a, 0xeb4, 0xe94, 0xe81, 0xeb2, 0xec0, 0xe8a, 0xe99, 0xe9a, 0xeb2, 0xec0, 0xe97,
0xec0, 0xea5, 0xea1, 0xeb5, 0xec0, 0xe8a, 0xe99, 0x20, 0xe88, 0xead, 0xe99, 0xec0,
0xe8a, 0xe99, 0x20, 0xe84, 0xeb4, 0xe94, 0xeaa, 0xecc, 0xec0, 0xe8a, 0xe99, 0x20,
0xea5, 0xeb9, 0xec0, 0xe8a, 0xe8d, 0xec0, 0xe8a, 0xe99, 0x20, 0xec2, 0xe97, 0xea1,
0xeb1, 0xeaa, 0xec0, 0xe8a, 0xe99, 0x20, 0xea7, 0xeb4, 0xe99, 0xec0, 0xe8a, 0xe99,
0xeaa, 0xeb0, 0xea7, 0xeb4, 0xe9f, 0x20, 0xec0, 0xe84, 0xeb5, 0xec0, 0xea3, 0xe99,
0xec0, 0xe95, 0xe81, 0xeb9, 0xe8a, 0xeb5, 0xe81, 0xeb2, 0xe99, 0xe9b, 0xeb2, 0xe97,
0xeb9, 0xec0, 0xea5, 0xe97, 0xeb5, 0xe88, 0xebb, 0xea7, 0xe99, 0xeb2, 0xec2, 0xe97,
0xea3, 0xead, 0xe99, 0xec2, 0xe95, 0xe97, 0xecd, 0xec2, 0xe95, 0xea5, 0xeb2, 0xec1,
0xea7, 0xe99, 0xe84, 0xeb9, 0xec0, 0xea7, 0xeb5, 0xec4, 0xea7, 0xe97, 0xecc, 0xeae,
0xead, 0xeaa, 0xea7, 0xeb4, 0xe99, 0xe99, 0xeb5, 0xec0, 0xe9e, 0xe81, 0xea2, 0xeb2,
0xe84, 0xeb9, 0xe95, 0xeb1, 0xe94, 0xec0, 0xe84, 0xe8a, 0xeb5, 0xe94, 0xeb2, 0xea7,
0xeb5, 0xeaa, 0xe94, 0xeb9, 0xea1, 0xead, 0xe99, 0xe94, 0xeb5, 0xe8d, 0xeb9, 0xea7,
0xeb5, 0xea7, 0xeaa, 0xecc, 0xec1, 0xea1, 0xeb1, 0xe81, 0xe84, 0xebb, 0xea7, 0xea3,
0xeb5, 0xec0, 0xea1, 0xebb, 0xeb2, 0xe8a, 0xeb1, 0xe99, 0xec1, 0xea1, 0xeb1, 0xe81,
0xea1, 0xebb, 0xea7, 0xec2, 0xe94, 0xe9e, 0xeb2, 0xea1, 0xec0, 0xea1, 0xeb5, 0xec2,
0xea3, 0xec0, 0xe97, 0xea3, 0xeb2, 0xec0, 0xe8a, 0xe8d, 0xea7, 0xeb2, 0xec2, 0xe97,
0xea3, 0xea5, 0xecc, 0xea7, 0xecd, 0xeaa, 0xeb0, 0xe95, 0xead, 0xe81, 0xea5, 0xead,
0xe87, 0xec0, 0xea2, 0xe8d, 0xe9a, 0xebd, 0xe99, 0xec0, 0xead, 0xec0, 0xe94, 0xeb1,
0xe99, 0xead, 0xecd, 0xea1, 0xeb2, 0xe95, 0xeb5, 0xead, 0xeb3, 0xea1, 0xeb2, 0xe99,
0xead, 0xeb2, 0xe99, 0xeb2, 0xe94, 0xeb5, 0xead, 0xeb1, 0xe94, 0xe95, 0xeb2, 0xead,
0xeb9, 0xead, 0xeb1, 0xe94, 0xec2, 0xe97, 0xe9a, 0xeb5, 0xead, 0xeb2, 0xe94, 0xe8a,
0xecc, 0xe81, 0xeb2, 0xe9a, 0xeb1, 0xe94, 0xead, 0xeb1, 0xe94, 0xe97, 0xeb5, 0xec0,
0xea3, 0xebb, 0xeb2, 0xec1, 0xe9a, 0xe81, 0xec1, 0xe94, 0xe94, 0xe9a, 0xeb2, 0xec0,
0xea3, 0xe99, 0xe9a, 0xeb2, 0xe81, 0xeb9, 0xe9a, 0xeb2, 0xe87, 0xe81, 0xead, 0xe81,
0xe9a, 0xeb2, 0xe99, 0xebb, 0xea7, 0xec0, 0xe9a, 0xea3, 0xeb8, 0xe94, 0xe9a, 0xeb4,
0xe94, 0xe8a, 0xecc, 0xec0, 0xe84, 0xe81, 0xe9a, 0xea3, 0xeb9, 0xec4, 0xe99, 0xe8a,
0xeb5, 0xe95, 0xec8, 0xeb2, 0xe8a, 0xead, 0xe8d, 0xe9a, 0xead, 0xea5, 0xe8a, 0xeb2,
0xe99, 0xec2, 0xe84, 0xea5, 0xeb3, 0xec2, 0xe9a, 0xe94, 0xeb2, 0xea1, 0xeb2, 0xeaa,
0xe84, 0xeb1, 0xeaa, 0xe94, 0xeb2, 0xeab, 0xecc, 0xe81, 0xeb2, 0xe94, 0xeb5, 0xea5,
0xeb5, 0xe94, 0xeb9, 0xec4, 0xe9a, 0xe94, 0xeb9, 0xe8a, 0xeb2, 0xe99, 0xec0, 0xe9a,
0xe9f, 0xeb2, 0xea1, 0xeb2, 0xe81, 0xeb1, 0xeaa, 0xe97, 0xeb2, 0xe81, 0xeb2, 0xe8a,
0xeb2, 0xec0, 0xeae, 0xe9a, 0xea3, 0xead, 0xe99, 0xec2, 0xeae, 0xe88, 0xeb4, 0xea1,
0xeb4, 0xe99, 0xeae, 0xec8, 0xead, 0xe87, 0xe81, 0xebb, 0xe87, 0xeae, 0xead, 0xe9a,
0xead, 0xeb5, 0xe84, 0xeb8, 0xeaa, 0xe84, 0xecc, 0xe88, 0xeb2, 0xe81, 0xeb2, 0xe95,
0xeb2, 0xe88, 0xeb2, 0xea2, 0xeb2, 0xe9b, 0xeb9, 0xea3, 0xeb0, 0xec0, 0xea2, 0xea3,
0xeb9, 0xe8a, 0xeb2, 0xec0, 0xea5, 0xea1, 0xe84, 0xeb2, 0xe9a, 0xeb9, 0xe84, 0xeb2,
0xea1, 0xe8a, 0xeb2, 0xe94, 0xe81, 0xeb2, 0xe81, 0xeb2, 0xea3, 0xeb2, 0xe88, 0xeb5,
0xe84, 0xeb1, 0xe94, 0xea1, 0xeb1, 0xe99, 0xe94, 0xeb9, 0xec1, 0xe84, 0xe99, 0xe94,
0xeb5, 0xe81, 0xeb2, 0xec2, 0xe81, 0xea5, 0xe81, 0xeb2, 0xe95, 0xeb2, 0xe84, 0xea3,
0xeb1, 0xeaa, 0xec2, 0xe99, 0xea2, 0xeb2, 0xeaa, 0xe84, 0xecc, 0xe81, 0xebb, 0xea7,
0xea5, 0xeb2, 0xea5, 0xeb3, 0xec0, 0xe9b, 0xeb5, 0xe81, 0xeb9, 0xe8a, 0xeb5, 0xe87,
0xe84, 0xeb9, 0xec0, 0xea7, 0xe94, 0xea1, 0xeb2, 0xec0, 0xe81, 0xebb, 0xeb2, 0xea1,
0xeb2, 0xe81, 0xeb2, 0xec1, 0xe94, 0xe99, 0xea1, 0xeb2, 0xe81, 0xeb2, 0xeaa, 0xe8a,
0xeb2, 0xea1, 0xeb2, 0xe99, 0xeb5, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xeaa, 0xe84, 0xeb1,
0xe94, 0xe99, 0xeb4, 0xec2, 0xe84, 0xec0, 0xe8a, 0xe8d, 0xec2, 0xe99, 0xec2, 0xea7,
0xe84, 0xeb8, 0xeaa, 0xec0, 0xe99, 0xeb1, 0xe94, 0xec2, 0xe99, 0xec2, 0xea7, 0xe8a,
0xeb5, 0xe9a, 0xeb5, 0xeaa, 0xe84, 0xecc, 0xead, 0xead, 0xea1, 0xeaa, 0xe84, 0xecc,
0xead, 0xecd, 0xea3, 0xecd, 0xe9e, 0xeb0, 0xe99, 0xebb, 0xea1, 0xec0, 0xe9b, 0xeb1,
0xe99, 0xe9e, 0xead, 0xe99, 0xec0, 0xe97, 0xe8d, 0xe99, 0xeb1, 0xe81, 0xe9b, 0xebd,
0xe87, 0xea2, 0xeb2, 0xe87, 0xe81, 0xeb2, 0xe95, 0xeb2, 0xea3, 0xecc, 0xe84, 0xead,
0xeaa, 0xe95, 0xeb2, 0xec0, 0xe99, 0xec4, 0xe84, 0xe8a, 0xeb5, 0xea5, 0xecd, 0xe94,
0xeb2, 0xea3, 0xeb5, 0xea2, 0xeb2, 0xe94, 0xe8a, 0xeb2, 0xe84, 0xeb2, 0xea5, 0xeb4,
0xe99, 0xe8a, 0xeb2, 0xea1, 0xeb2, 0xec1, 0xe84, 0xe99, 0xec2, 0xe8a, 0xea5, 0xecc,
0xe8a, 0xec8, 0xebd, 0xe87, 0xec4, 0xeae, 0xec9, 0xeaa, 0xeb4, 0xe87, 0xe81, 0xeb0,
0xec2, 0xe9b, 0xeaa, 0xea3, 0xeb4, 0xea5, 0xec0, 0xe99, 0xeb1, 0xe81, 0xec2, 0xe81,
0xea5, 0xeb5, 0xe94, 0xec4, 0xe97, 0xec0, 0xe9b, 0xe97, 0xeb2, 0xe94, 0xeaa, 0xecc,
0xec0, 0xe84, 0xe99, 0xe97, 0xeb4, 0xe9a, 0xeb5, 0xea5, 0xeb5, 0xe8a, 0xeb5, 0xec0,
0xe95, 0xea3, 0xeb2, 0xe99, 0xe97, 0xeb4, 0xea1, 0xe9e, 0xeb9, 0xec2, 0xe95, 0xe81,
0xebd, 0xea7, 0xe97, 0xead, 0xea1, 0xeaa, 0xe81, 0xecc, 0xead, 0xeb9, 0xea5, 0xeb2,
0xe99, 0xe9a, 0xeb2, 0xe95, 0xeb2, 0xea3, 0xecc, 0xead, 0xeb9, 0xea3, 0xeb8, 0xea1,
0xe8a, 0xeb5, 0xead, 0xeb8, 0xeaa, 0x20, 0xec0, 0xe99, 0xea3, 0xeb2, 0xea7, 0xebd,
0xe87, 0xe88, 0xeb1, 0xe99, 0xea7, 0xeb5, 0xea5, 0xeb2, 0xe94, 0xeb4, 0xec2, 0xea7,
0xeaa, 0xe95, 0xead, 0xe81, 0xea2, 0xeb2, 0xe84, 0xeb8, 0xe97, 0xe8a, 0xecc, 0xea2,
0xeb2, 0xe87, 0xe81, 0xead, 0xe99, 0xec0, 0xea2, 0xe84, 0xeb2, 0xec0, 0xe95, 0xeb5,
0xea3, 0xeb4, 0xe99, 0xec0, 0xe9a, 0xeb5, 0xe81, 0xec0, 0xe8d, 0xec0, 0xea3, 0xea7,
0xeb2, 0xe99, 0xead, 0xeb2, 0xe8a, 0xecd, 0xec0, 0xea3, 0xeaa, 0xec0, 0xe9a, 0xeb5,
0xea1, 0xeb4, 0xea7, 0xe94, 0xeb2, 0xe84, 0xeb2, 0xe99, 0xeb2, 0xea3, 0xeb5, 0xec0,
0xe84, 0xe9a, 0xec0, 0xea7, 0xeb5, 0xe94, 0xec1, 0xe9f, 0xec2, 0xea3, 0xea1, 0xeb2,
0xec0, 0xe94, 0xea3, 0xeb2, 0xec0, 0xea3, 0xe81, 0xea2, 0xeb2, 0xea7, 0xeb4, 0xe81,
0xec0, 0xe8a, 0xebb, 0xeb2, 0xe88, 0xecd, 0xec0, 0xe88, 0xe8d, 0xec0, 0xe8a, 0xe99,
0x20, 0xec0, 0xeae, 0xec0, 0xea5, 0xe99, 0xeb2, 0xeaa, 0xeb0, 0xec1, 0xe95, 0xe99,
0xec0, 0xea5, 0xec0, 0xead, 0xe94, 0xeb5, 0xec1, 0xea5, 0xe94, 0xe9a, 0xea3, 0xeb4,
0xeaa, 0xec0, 0xe9a, 0xe99, 0xec2, 0xe9a, 0xea3, 0xe81, 0xec0, 0xe84, 0xe99, 0x20,
0xeae, 0xeb4, 0xea7, 0xe94, 0xeb2, 0xea7, 0xeb4, 0xe99, 0xea2, 0xeb9, 0xe84, 0xea5,
0xeb2, 0xec2, 0xeae, 0xe9a, 0xeb2, 0xe94, 0xea5, 0xeb4, 0xe99, 0xe94, 0xeb5, 0xec1,
0xea1, 0xe99, 0xec2, 0xea5, 0xe94, 0xec2, 0xeae, 0xea7, 0xeb5, 0xec0, 0xea1, 0xea7,
0xec0, 0xe9a, 0xeb4, 0xe99, 0xec0, 0xe9e, 0xeb5, 0xe94, 0xe8a, 0xeb4, 0xe94, 0xe99,
0xeb5, 0xead, 0xeb2, 0xea1, 0xeaa, 0xec0, 0xe95, 0xeb5, 0xe94, 0xeb3, 0xead, 0xeb4,
0xe99, 0xe94, 0xecd, 0xea3, 0xeb2, 0xead, 0xeb2, 0xeaa, 0xe95, 0xea3, 0xeb2, 0xe84,
0xeb2, 0xe99, 0xec0, 0xead, 0xec0, 0xe97, 0xe99, 0xeaa, 0xecc, 0xec0, 0xe9a, 0xea7,
0xec0, 0xe81, 0xea3, 0xe94, 0xec0, 0xe9a, 0xeb5, 0xea5, 0xeb4, 0xe99, 0xe9a, 0xea3,
0xeb2, 0xe97, 0xeb4, 0xeaa, 0xea5, 0xeb2, 0xea7, 0xeb2, 0xe9a, 0xea3, 0xeb1, 0xeaa,
0xec0, 0xe8a, 0xea7, 0xe9a, 0xeb9, 0xe84, 0xeb2, 0xec0, 0xea3, 0xeaa, 0xe95, 0xecc,
0xe9a, 0xeb9, 0xe94, 0xeb2, 0xec0, 0xe9b, 0xeaa, 0xe95, 0xecc, 0xe9a, 0xeb1, 0xe94,
0xe8a, 0xeb4, 0xe99, 0xec0, 0xe81, 0xe99, 0xe84, 0xeb5, 0xe8a, 0xeb5, 0xec0, 0xe99,
0xebb, 0xeb2, 0xec2, 0xe84, 0xec0, 0xe9b, 0xe99, 0xec0, 0xeae, 0xec0, 0xe81, 0xe99,
0xe94, 0xeb1, 0xe9a, 0xe9a, 0xea5, 0xeb4, 0xe99, 0xe81, 0xeb4, 0xe9a, 0xea3, 0xead,
0xea5, 0xe97, 0xeb2, 0xec0, 0xe81, 0xeb5, 0xe99, 0xe8a, 0xeb5, 0xec0, 0xeae, 0xea7,
0xe8a, 0xeb4, 0xe99, 0xe81, 0xeb4, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xec1, 0xea1, 0xe99,
0xead, 0xeb4, 0xeaa, 0xe95, 0xeb1, 0xe99, 0xe9a, 0xeb9, 0xea5, 0xecc, 0xec0, 0xe88,
0xeb5, 0xe8a, 0xeb5, 0xe84, 0xeb2, 0xea5, 0xeb4, 0xe99, 0xeb4, 0xe99, 0xe81, 0xea3,
0xeb2, 0xe94, 0xe81, 0xeb4, 0xec2, 0xea3, 0xe9a, 0xe82, 0xebd, 0xe9a, 0xea5, 0xeb4,
0xeaa, 0xe9a, 0xead, 0xe99, 0xea5, 0xeb9, 0xe9a, 0xea5, 0xeb5, 0xe8d, 0xeb2, 0xe99,
0xeb2, 0xea5, 0xead, 0xe99, 0xe94, 0xead, 0xe99, 0xea5, 0xeb1, 0xe81, 0xec0, 0xe8a,
0xeb1, 0xea1, 0xec0, 0xe9a, 0xeb5, 0xe81, 0xea1, 0xeb2, 0xe94, 0xea3, 0xeb4, 0xe94,
0xea1, 0xead, 0xea5, 0xe95, 0xeb2, 0xea1, 0xeb2, 0xea3, 0xeb5, 0xec1, 0xeae, 0xea1,
0xe99, 0xecc, 0xea1, 0xeb4, 0xe99, 0xeaa, 0xe81, 0xecc, 0xec2, 0xea1, 0xe99, 0xeb2,
0xec2, 0xe84, 0xea1, 0xead, 0xeaa, 0xec2, 0xe84, 0xead, 0xead, 0xeaa, 0xec2, 0xea5,
0xe9b, 0xeb2, 0xea3, 0xeb5, 0xeaa, 0xe9e, 0xead, 0xe94, 0xe81, 0xecd, 0xea3, 0xeb5,
0xe81, 0xeb2, 0xe9b, 0xea3, 0xeb2, 0xe81, 0xe8a, 0xeb2, 0xea1, 0xeb2, 0xea3, 0xeb2,
0xe8a, 0xeb2, 0xe99, 0xea1, 0xeb2, 0xea3, 0xeb5, 0xec2, 0xe99, 0xe8a, 0xeb2, 0xea3,
0xeb2, 0xec0, 0xea2, 0xec2, 0xea7, 0xe8a, 0xeb2, 0xea3, 0xeb2, 0xe97, 0xead, 0xe9a,
0xe8a, 0xeb5, 0xea1, 0xec0, 0xe9f, 0xec2, 0xea5, 0xec2, 0xe9b, 0xeaa, 0xeb0, 0xec2,
0xe81, 0xec0, 0xe9b, 0xe8d, 0xec2, 0xe8a, 0xec0, 0xe9f, 0xe8d, 0xeaa, 0xeb0, 0xe95,
0xead, 0xe81, 0xec2, 0xeae, 0xea1, 0xe97, 0xeb2, 0xea5, 0xea5, 0xeb4, 0xe99, 0xe99,
0xecc, 0xe97, 0xeb4, 0xea3, 0xeb2, 0xe99, 0xead, 0xeb9, 0xea5, 0xeb5, 0xead, 0xeb2,
0xe99, 0xead, 0xe9a, 0xeaa, 0xe84, 0xecc, 0xea7, 0xeb2, 0xe94, 0xeb2, 0xe8a, 0xea7,
0xeb2, 0xe95, 0xeb4, 0xe81, 0xeb1, 0xe99, 0xea7, 0xebd, 0xe99, 0xe99, 0xeb2, 0xea7,
0xeb4, 0xea5, 0xe99, 0xeb4, 0xead, 0xeb8, 0xeaa, 0xea7, 0xead, 0xea5, 0xec2, 0xe81,
0xe81, 0xea3, 0xeb2, 0xe94, 0xea7, 0xecd, 0xe8a, 0xecd, 0xe8a, 0xeb2, 0xec0, 0xe81,
0xea3, 0xe9a, 0xe8a, 0xeb9, 0xea3, 0xeb4, 0xe81, 0xead, 0xeb1, 0xe99, 0xe95, 0xeb2,
0xe99, 0xeb2, 0xe99, 0xeb2, 0xea3, 0xeb4, 0xec2, 0xea7, 0xe8a, 0xeb2, 0xec2, 0xe81,
0xeaa, 0xe84, 0xea3, 0xeb4, 0xe94, 0xeaa, 0xeb0, 0xea1, 0xeb2, 0xe94, 0xec2, 0xe84,
0xec2, 0xe84, 0xeaa, 0xec2, 0xe84, 0xec2, 0xea1, 0xec2, 0xea3, 0xec1, 0xe81, 0xec0,
0xe81, 0xeb5, 0xec0, 0xea5, 0xe99, 0xea1, 0xeb2, 0xec0, 0xeae, 0xea1, 0xeb1, 0xea5,
0xe94, 0xeb5, 0xe9f, 0xec0, 0xea1, 0xebb, 0xeb2, 0xea3, 0xeb4, 0xe97, 0xebd, 0xeaa,
0xea1, 0xeb2, 0xea2, 0xeb1, 0xead, 0xe94, 0xec0, 0xe95, 0xec0, 0xea3, 0xead, 0xeb9,
0xe99, 0xeb4, 0xea2, 0xebb, 0xe87, 0xec0, 0xead, 0xe9b, 0xeb5, 0xead, 0xeb2, 0xead,
0xeb1, 0xe81, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2, 0xea1,
0xec0, 0xe82, 0xe94, 0xe9a, 0xeb9, 0xe99, 0xe81, 0xebd, 0xe99, 0xea7, 0xeb4, 0xea7,
0xec1, 0xe8a, 0xec1, 0xe97, 0xea1, 0xe88, 0xeb1, 0xe81, 0xec0, 0xead, 0xeaa, 0xec0,
0xe95, 0xeb5, 0xec0, 0xead, 0xe9f, 0xeb2, 0xec0, 0xe95, 0xe9f, 0xeb2, 0xe81, 0xeb2,
0xea7, 0xec2, 0xe9f, 0xe9f, 0xeb9, 0xe88, 0xeb4, 0xe9f, 0xeb9, 0xe99, 0xeb0, 0xe9f,
0xeb9, 0xe95, 0xeb4, 0xe81, 0xeb2, 0xea5, 0xeb2, 0xe9b, 0xeb2, 0xe81, 0xead, 0xeaa,
0xec1, 0xe81, 0xea1, 0xec0, 0xe9a, 0xe8d, 0xe81, 0xebb, 0xea7, 0xe94, 0xeb1, 0xea5,
0xe84, 0xeb0, 0xec1, 0xe99, 0xea5, 0xe81, 0xea7, 0xea1, 0xec2, 0xeae, 0xec2, 0xe99,
0xea5, 0xeb9, 0xea5, 0xeb9, 0xec0, 0xead, 0xeb1, 0xe99, 0xec0, 0xe94, 0xeb5, 0xec0,
0xe9a, 0xeb5, 0xea5, 0xeb5, 0xe84, 0xeb5, 0xea3, 0xeb4, 0xe97, 0xeb5, 0xea1, 0xeb2,
0xe95, 0xeb5, 0xe84, 0xecd, 0xeaa, 0xec1, 0xea3, 0xe84, 0xea7, 0xeb2, 0xe88, 0xeb2,
0xec0, 0xea5, 0xe99, 0xea1, 0xeb2, 0xe88, 0xeb9, 0xec2, 0xea3, 0xea1, 0xeb2, 0xe84,
0xeb4, 0xea7, 0xe8a, 0xeb2, 0xeaa, 0xea1, 0xeb4, 0xe94, 0xec0, 0xea7, 0xe99, 0xeb2,
0xead, 0xeb9, 0xea3, 0xeb9, 0xe99, 0xeb5, 0xead, 0xeb9, 0xec0, 0xead, 0xe99, 0xecd,
0xe9f, 0xead, 0xea5, 0xecc, 0xe81, 0xe99, 0xeb9, 0xec0, 0xea1, 0xead, 0xeb2, 0xe9b,
0xeb2, 0xec2, 0xe81, 0xe9b, 0xeb2, 0xec2, 0xe81, 0xe9b, 0xeb2, 0xec0, 0xea5, 0xebb,
0xeb2, 0xe9e, 0xeb4, 0xe94, 0xec1, 0xe84, 0xe99, 0xe9b, 0xebb, 0xe99, 0xec0, 0xe9b,
0xeb5, 0xe8d, 0xe9e, 0xead, 0xe94, 0x20, 0xea1, 0xecd, 0xec0, 0xea3, 0xeaa, 0xe9a,
0xeb5, 0xea3, 0xeb2, 0xec2, 0xea3, 0xe95, 0xead, 0xe87, 0xe81, 0xeb2, 0xec4, 0xe8a,
0xe9b, 0xeb2, 0xe99, 0xe97, 0xeb2, 0xeae, 0xeb5, 0xe95, 0xeb4, 0xe95, 0xeb2, 0xea3,
0xeb2, 0xea7, 0xeb2, 0xe95, 0xead, 0xe87, 0xe81, 0xeb2, 0xe95, 0xeb2, 0xe9b, 0xeb9,
0xec0, 0xea7, 0xe81, 0xea7, 0xeb2, 0xea5, 0xeb4, 0xe94, 0x41, 0x62, 0x69, 0x64,
0x17e, 0x61, 0x6e, 0x61, 0x41, 0x64, 0x69, 0x73, 0x61, 0x62, 0x65, 0x62,
0x61, 0x41, 0x6c, 0x17e, 0x12b, 0x72, 0x61, 0x42, 0x61, 0x6e, 0x64, 0x17e,
0x75, 0x6c, 0x61, 0x42, 0x69, 0x73, 0x61, 0x76, 0x61, 0x42, 0x6c, 0x61,
0x6e, 0x74, 0x61, 0x69, 0x72, 0x61, 0x42, 0x72, 0x61, 0x7a, 0x61, 0x76,
0x69, 0x6c, 0x61, 0x42, 0x75, 0x17e, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x61,
0x4b, 0x61, 0x69, 0x72, 0x61, 0x44, 0x61, 0x6b, 0x61, 0x72, 0x61, 0x44,
0x101, 0x72, 0x65, 0x73, 0x61, 0x6c, 0x101, 0x6d, 0x61, 0x44, 0x17e, 0x69,
0x62, 0x75, 0x74, 0x69, 0x6a, 0x61, 0x41, 0x6a, 0x16b, 0x6e, 0x61, 0x46,
0x72, 0x12b, 0x74, 0x61, 0x75, 0x6e, 0x61, 0x4a, 0x6f, 0x68, 0x61, 0x6e,
0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x61, 0x44, 0x17e, 0x16b, 0x62,
0x61, 0x48, 0x61, 0x72, 0x74, 0x16b, 0x6d, 0x61, 0x4b, 0x69, 0x6e, 0x161,
0x61, 0x73, 0x61, 0x4c, 0x61, 0x67, 0x6f, 0x73, 0x61, 0x4c, 0x69, 0x62,
0x72, 0x65, 0x76, 0x69, 0x6c, 0x61, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62,
0x61, 0x161, 0x69, 0x4d, 0x61, 0x70, 0x75, 0x74, 0x75, 0x4d, 0x6f, 0x67,
0x61, 0x64, 0x12b, 0x161, 0x6f, 0x4d, 0x6f, 0x6e, 0x72, 0x6f, 0x76, 0x69,
0x6a, 0x61, 0x4e, 0x69, 0x61, 0x6d, 0x65, 0x6a, 0x61, 0x4e, 0x75, 0x61,
0x6b, 0x161, 0x6f, 0x74, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x6e, 0x6f,
0x76, 0x6f, 0x54, 0x72, 0x69, 0x70, 0x6f, 0x6c, 0x65, 0x54, 0x75, 0x6e,
0x69, 0x73, 0x61, 0x56, 0x69, 0x6e, 0x64, 0x68, 0x75, 0x6b, 0x61, 0x41,
0x64, 0x61, 0x6b, 0x61, 0x41, 0x6e, 0x6b, 0x75, 0x72, 0x101, 0x17e, 0x61,
0x41, 0x6e, 0x67, 0x69, 0x6c, 0x6a, 0x61, 0x41, 0x72, 0x61, 0x67, 0x76,
0x61, 0x69, 0x6e, 0x61, 0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x61, 0x69,
0x72, 0x65, 0x73, 0x61, 0x4b, 0x6f, 0x72, 0x64, 0x6f, 0x76, 0x61, 0x48,
0x75, 0x68, 0x75, 0x6a, 0x61, 0x4c, 0x61, 0x72, 0x6a, 0x6f, 0x68, 0x61,
0x52, 0x69, 0x6f, 0x67, 0x61, 0x6c, 0x6a, 0x65, 0x67, 0x6f, 0x73, 0x61,
0x53, 0x61, 0x6e, 0x68, 0x75, 0x61, 0x6e, 0x61, 0x53, 0x61, 0x6e, 0x6c,
0x75, 0x69, 0x73, 0x61, 0x54, 0x75, 0x6b, 0x75, 0x6d, 0x61, 0x6e, 0x61,
0x55, 0x161, 0x75, 0x61, 0x6a, 0x61, 0x41, 0x73, 0x75, 0x6e, 0x73, 0x6a,
0x6f, 0x6e, 0x61, 0x41, 0x74, 0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x6e, 0x61,
0x42, 0x61, 0x69, 0x6a, 0x61, 0x42, 0x61, 0x6a, 0x61, 0x62, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x61, 0x73, 0x61, 0x42, 0x61, 0x72, 0x62, 0x61, 0x64,
0x6f, 0x73, 0x61, 0x42, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x42, 0x65, 0x6c,
0x69, 0x7a, 0x61, 0x42, 0x6c, 0x61, 0x6e, 0x73, 0x61, 0x62, 0x6c, 0x6f,
0x6e, 0x61, 0x42, 0x6f, 0x61, 0x76, 0x69, 0x73, 0x74, 0x61, 0x42, 0x6f,
0x69, 0x73, 0x69, 0x73, 0x69, 0x74, 0x69, 0x6a, 0x61, 0x4b, 0x65, 0x6d,
0x62, 0x72, 0x69, 0x64, 0x17e, 0x62, 0x65, 0x6a, 0x61, 0x4b, 0x61, 0x6d,
0x70, 0x75, 0x67, 0x72, 0x61, 0x6e, 0x64, 0x69, 0x4b, 0x61, 0x6e, 0x6b,
0x75, 0x6e, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x4b,
0x61, 0x6a, 0x65, 0x6e, 0x6e, 0x61, 0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e,
0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x10c, 0x69, 0x6b, 0x101, 0x67,
0x61, 0x10c, 0x69, 0x76, 0x61, 0x76, 0x61, 0x48, 0x75, 0x61, 0x72, 0x65,
0x73, 0x61, 0x4b, 0x72, 0x65, 0x73, 0x74, 0x6f, 0x6e, 0x61, 0x4b, 0x69,
0x72, 0x61, 0x73, 0x61, 0x6f, 0x44, 0x65, 0x6e, 0x6d, 0x101, 0x72, 0x6b,
0x161, 0x61, 0x76, 0x6e, 0x61, 0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x61,
0x44, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x6b, 0x72, 0x12b, 0x6b, 0x61, 0x44,
0x65, 0x6e, 0x76, 0x65, 0x72, 0x61, 0x44, 0x65, 0x74, 0x72, 0x6f, 0x69,
0x74, 0x61, 0x45, 0x64, 0x6d, 0x6f, 0x6e, 0x74, 0x6f, 0x6e, 0x61, 0x53,
0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x61, 0x46, 0x6f, 0x72, 0x74,
0x6e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x61, 0x47, 0x6c, 0x65, 0x69, 0x73,
0x62, 0x65, 0x6a, 0x61, 0x47, 0x16b, 0x73, 0x62, 0x65, 0x6a, 0x61, 0x47,
0x72, 0x61, 0x6e, 0x64, 0x74, 0x6b, 0x113, 0x72, 0x6b, 0x61, 0x47, 0x72,
0x65, 0x6e, 0x101, 0x64, 0x61, 0x47, 0x76, 0x61, 0x64, 0x65, 0x6c, 0x75,
0x70, 0x61, 0x47, 0x76, 0x61, 0x6a, 0x61, 0x6b, 0x69, 0x6c, 0x61, 0x47,
0x61, 0x6a, 0x101, 0x6e, 0x61, 0x48, 0x65, 0x6c, 0x69, 0x66, 0x65, 0x6b,
0x73, 0x61, 0x45, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6a, 0x6f, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x61,
0x4e, 0x6f, 0x6b, 0x73, 0x61, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x101,
0x6e, 0x61, 0x4d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x101, 0x6e, 0x61, 0x50, 0x12b, 0x74, 0x65, 0x72, 0x73,
0x62, 0x113, 0x72, 0x67, 0x61, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x101,
0x6e, 0x61, 0x54, 0x65, 0x6c, 0x73, 0x69, 0x74, 0x69, 0x6a, 0x61, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x101, 0x6e, 0x61, 0x56, 0x12b, 0x76, 0x65,
0x6a, 0x61, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x101, 0x6e, 0x61, 0x56,
0x69, 0x6e, 0x73, 0x65, 0x6e, 0x73, 0x61, 0x2c, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x101, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x61,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x101, 0x6e, 0x61, 0x49, 0x6e, 0x75,
0x76, 0x69, 0x6b, 0x61, 0x49, 0x6b, 0x61, 0x6c, 0x75, 0x69, 0x74, 0x61,
0x44, 0x17e, 0x75, 0x6e, 0x6f, 0x4c, 0x16b, 0x69, 0x76, 0x69, 0x6c, 0x61,
0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x10d, 0x65, 0x6c, 0x6f, 0x2c, 0x20, 0x4b,
0x65, 0x6e, 0x74, 0x75, 0x6b, 0x69, 0x4b, 0x72, 0x101, 0x6c, 0x65, 0x6e,
0x64, 0x65, 0x69, 0x6b, 0x61, 0x4c, 0x61, 0x70, 0x61, 0x73, 0x61, 0x4c,
0x6f, 0x73, 0x61, 0x6e, 0x64, 0x17e, 0x65, 0x6c, 0x6f, 0x73, 0x61, 0x4c,
0x6f, 0x75, 0x65, 0x72, 0x70, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x73, 0x6b,
0x76, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x4d, 0x61, 0x73, 0x65, 0x6a, 0x6f,
0x4d, 0x61, 0x6e, 0x61, 0x67, 0x76, 0x61, 0x4d, 0x61, 0x6e, 0x61, 0x75,
0x73, 0x61, 0x4d, 0x65, 0x72, 0x69, 0x67, 0x6f, 0x74, 0x61, 0x4d, 0x61,
0x74, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x73, 0x61, 0x4d, 0x61, 0x73, 0x61,
0x74, 0x6c, 0x61, 0x6e, 0x61, 0x4d, 0x65, 0x6e, 0x6f, 0x6d, 0x69, 0x6e,
0x12b, 0x4d, 0x65, 0x68, 0x69, 0x6b, 0x6f, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x6f, 0x6e, 0x61, 0x4d, 0x6f, 0x6e, 0x6b, 0x74, 0x6f, 0x6e, 0x61, 0x4d,
0x6f, 0x6e, 0x74, 0x65, 0x72, 0x72, 0x65, 0x6a, 0x61, 0x4d, 0x6f, 0x6e,
0x74, 0x73, 0x65, 0x72, 0x72, 0x61, 0x74, 0x61, 0x4e, 0x61, 0x73, 0x6f,
0x145, 0x75, 0x6a, 0x6f, 0x72, 0x6b, 0x61, 0x4e, 0x6f, 0x6d, 0x61, 0x4e,
0x6f, 0x72, 0x6f, 0x146, 0x61, 0x42, 0x6a, 0x75, 0x6c, 0x61, 0x2c, 0x20,
0x5a, 0x69, 0x65, 0x6d, 0x65, 0x13c, 0x64, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x53, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x61, 0x2c, 0x20, 0x5a, 0x69, 0x65,
0x6d, 0x65, 0x13c, 0x64, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x145, 0x16b, 0x73,
0x65, 0x69, 0x6c, 0x65, 0x6d, 0x61, 0x2c, 0x20, 0x5a, 0x69, 0x65, 0x6d,
0x65, 0x13c, 0x64, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x16b, 0x6b, 0x61,
0x46, 0x12b, 0x6e, 0x69, 0x6b, 0x73, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f,
0x70, 0x72, 0x65, 0x6e, 0x73, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x66,
0x73, 0x70, 0x65, 0x69, 0x6e, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x76,
0x65, 0x13c, 0x75, 0x50, 0x75, 0x6e, 0x74, 0x61, 0x61, 0x72, 0x65, 0x6e,
0x61, 0x73, 0x61, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x69, 0x6e, 0x6c,
0x65, 0x74, 0x61, 0x52, 0x65, 0x73, 0x69, 0x66, 0x69, 0x52, 0x69, 0x64,
0x17e, 0x61, 0x69, 0x6e, 0x61, 0x52, 0x65, 0x7a, 0x6f, 0x6c, 0x16b, 0x74,
0x61, 0x52, 0x69, 0x6f, 0x62, 0x72, 0x61, 0x6e, 0x6b, 0x6f, 0x53, 0x61,
0x6e, 0x74, 0x61, 0x72, 0x65, 0x6e, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x6a,
0x61, 0x67, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x64, 0x6f, 0x6d, 0x69,
0x6e, 0x67, 0x6f, 0x53, 0x61, 0x6e, 0x70, 0x61, 0x75, 0x6c, 0x75, 0x49,
0x74, 0x6f, 0x6b, 0x6f, 0x72, 0x74, 0x6f, 0x72, 0x6d, 0x69, 0x74, 0x61,
0x53, 0x65, 0x6e, 0x62, 0x61, 0x72, 0x74, 0x65, 0x6c, 0x6d, 0x12b, 0x53,
0x65, 0x6e, 0x74, 0x64, 0x17e, 0x6f, 0x6e, 0x73, 0x61, 0x53, 0x65, 0x6e,
0x74, 0x6b, 0x69, 0x74, 0x73, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x6c, 0x16b,
0x73, 0x69, 0x6a, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x6f, 0x6d, 0x61, 0x73,
0x61, 0x53, 0x65, 0x6e, 0x74, 0x76, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74,
0x61, 0x53, 0x76, 0x69, 0x66, 0x74, 0x6b, 0x61, 0x72, 0x65, 0x6e, 0x74,
0x61, 0x54, 0x75, 0x6c, 0x65, 0x56, 0x61, 0x6e, 0x6b, 0x16b, 0x76, 0x65,
0x72, 0x61, 0x56, 0x61, 0x69, 0x74, 0x68, 0x6f, 0x72, 0x73, 0x61, 0x56,
0x69, 0x6e, 0x69, 0x70, 0x65, 0x67, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x61, 0x74, 0x61, 0x4b, 0x65, 0x69, 0x73, 0x69, 0x44, 0x65, 0x69, 0x76,
0x69, 0x73, 0x61, 0x44, 0x69, 0x6d, 0x6f, 0x6e, 0x64, 0x69, 0x72, 0x76,
0x69, 0x6c, 0x61, 0x4d, 0x61, 0x6b, 0x76, 0x6f, 0x72, 0x69, 0x4d, 0x6f,
0x73, 0x6f, 0x6e, 0x61, 0x4d, 0x61, 0x6b, 0x6d, 0x65, 0x72, 0x64, 0x6f,
0x50, 0x101, 0x6c, 0x6d, 0x65, 0x72, 0x61, 0x52, 0x6f, 0x74, 0x65, 0x72,
0x61, 0x160, 0x6f, 0x76, 0x61, 0x54, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x4c, 0x6f, 0x6e, 0x67, 0x6a, 0x113,
0x72, 0x62, 0x12b, 0x65, 0x6e, 0x65, 0x41, 0x64, 0x65, 0x6e, 0x61, 0x41,
0x6d, 0x6d, 0x101, 0x6e, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x61,
0x41, 0x6b, 0x74, 0x65, 0x62, 0x65, 0x41, 0x161, 0x67, 0x61, 0x62, 0x61,
0x74, 0x61, 0x42, 0x61, 0x67, 0x64, 0x101, 0x64, 0x65, 0x42, 0x61, 0x68,
0x72, 0x65, 0x69, 0x6e, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6b, 0x6f, 0x6b,
0x61, 0x42, 0x61, 0x72, 0x6e, 0x61, 0x75, 0x6c, 0x61, 0x42, 0x65, 0x69,
0x72, 0x16b, 0x74, 0x61, 0x42, 0x69, 0x161, 0x6b, 0x65, 0x6b, 0x61, 0x42,
0x72, 0x75, 0x6e, 0x65, 0x6a, 0x61, 0x10c, 0x6f, 0x69, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x61, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x61, 0x44,
0x75, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x48, 0x65, 0x62, 0x72, 0x6f, 0x6e,
0x61, 0x48, 0x6f, 0x161, 0x69, 0x6d, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x6e,
0x6b, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x4a, 0x65, 0x72, 0x75, 0x7a, 0x61,
0x6c, 0x65, 0x6d, 0x65, 0x4b, 0x61, 0x62, 0x75, 0x6c, 0x61, 0x4b, 0x61,
0x72, 0x101, 0x10d, 0x69, 0x4b, 0x61, 0x6c, 0x6b, 0x101, 0x74, 0x61, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x4b,
0x75, 0x61, 0x6c, 0x61, 0x6c, 0x75, 0x6d, 0x70, 0x75, 0x72, 0x61, 0x4b,
0x75, 0x10d, 0x69, 0x6e, 0x61, 0x4b, 0x75, 0x76, 0x65, 0x69, 0x74, 0x61,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x6b, 0x61,
0x73, 0x61, 0x72, 0x61, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x74, 0x61, 0x4e,
0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x146, 0x65, 0x63, 0x6b, 0x61, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x4f,
0x6d, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x6c, 0x61, 0x50, 0x6e, 0x6f,
0x6d, 0x70, 0x65, 0x146, 0x61, 0x50, 0x6f, 0x6e, 0x74, 0x69, 0x61, 0x6e,
0x61, 0x6b, 0x61, 0x50, 0x68, 0x65, 0x6e, 0x6a, 0x61, 0x6e, 0x61, 0x4b,
0x61, 0x74, 0x61, 0x72, 0x61, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61,
0x6a, 0x61, 0x52, 0x69, 0x6a, 0x101, 0x64, 0x61, 0x53, 0x61, 0x68, 0x61,
0x6c, 0x12b, 0x6e, 0x61, 0x53, 0x65, 0x75, 0x6c, 0x61, 0x160, 0x61, 0x6e,
0x68, 0x61, 0x6a, 0x61, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x16b, 0x72,
0x61, 0x53, 0x72, 0x65, 0x64, 0x146, 0x65, 0x6b, 0x6f, 0x6c, 0x69, 0x6d,
0x73, 0x6b, 0x61, 0x54, 0x61, 0x69, 0x62, 0x65, 0x69, 0x54, 0x61, 0x161,
0x6b, 0x65, 0x6e, 0x74, 0x61, 0x54, 0x65, 0x68, 0x65, 0x72, 0x101, 0x6e,
0x61, 0x54, 0x6f, 0x6b, 0x69, 0x6a, 0x61, 0x54, 0x6f, 0x6d, 0x73, 0x6b,
0x61, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x55,
0x73, 0x74, 0x6a, 0x146, 0x65, 0x72, 0x61, 0x56, 0x6a, 0x65, 0x6e, 0x10d,
0x61, 0x6e, 0x61, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x52,
0x61, 0x6e, 0x67, 0x75, 0x6e, 0x61, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x45, 0x72, 0x65, 0x76,
0x101, 0x6e, 0x61, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x20, 0x73, 0x61, 0x6c,
0x61, 0x73, 0x4b, 0x61, 0x6e, 0x101, 0x72, 0x69, 0x6a, 0x75, 0x20, 0x73,
0x61, 0x6c, 0x61, 0x73, 0x4b, 0x61, 0x62, 0x6f, 0x76, 0x65, 0x72, 0x64,
0x65, 0x46, 0x113, 0x72, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x52,
0x65, 0x69, 0x6b, 0x6a, 0x61, 0x76, 0x69, 0x6b, 0x61, 0x44, 0x69, 0x65,
0x6e, 0x76, 0x69, 0x64, 0x64, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a,
0x61, 0x53, 0x76, 0x2e, 0x48, 0x65, 0x6c, 0x113, 0x6e, 0x61, 0x73, 0x20,
0x73, 0x61, 0x6c, 0x61, 0x42, 0x72, 0x69, 0x73, 0x62, 0x65, 0x6e, 0x61,
0x42, 0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x68, 0x69, 0x6c, 0x61, 0x44, 0x101,
0x72, 0x76, 0x69, 0x6e, 0x61, 0x4a, 0x75, 0x6b, 0x6c, 0x61, 0x48, 0x6f,
0x62, 0x101, 0x72, 0x74, 0x61, 0x4c, 0x69, 0x6e, 0x64, 0x65, 0x6d, 0x61,
0x6e, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x68, 0x61, 0x76, 0x61, 0x4d, 0x65,
0x6c, 0x62, 0x75, 0x72, 0x6e, 0x61, 0x50, 0x113, 0x72, 0x74, 0x61, 0x53,
0x69, 0x64, 0x6e, 0x65, 0x6a, 0x61, 0x41, 0x6d, 0x73, 0x74, 0x65, 0x72,
0x64, 0x61, 0x6d, 0x61, 0x41, 0x73, 0x74, 0x72, 0x61, 0x68, 0x61, 0x146,
0x61, 0x41, 0x74, 0x113, 0x6e, 0x61, 0x73, 0x42, 0x65, 0x6c, 0x67, 0x72,
0x61, 0x64, 0x61, 0x42, 0x65, 0x72, 0x6c, 0x12b, 0x6e, 0x65, 0x42, 0x72,
0x69, 0x73, 0x65, 0x6c, 0x65, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x65, 0x73,
0x74, 0x65, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x161, 0x74, 0x61, 0x42,
0x12b, 0x7a, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x65, 0x4b, 0x69, 0x161, 0x69,
0x146, 0x65, 0x76, 0x61, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x101, 0x67,
0x65, 0x6e, 0x61, 0x44, 0x75, 0x62, 0x6c, 0x69, 0x6e, 0x61, 0x47, 0x69,
0x62, 0x72, 0x61, 0x6c, 0x74, 0x101, 0x72, 0x73, 0x47, 0x113, 0x72, 0x6e,
0x73, 0x69, 0x6a, 0x61, 0x4d, 0x65, 0x6e, 0x61, 0x73, 0x20, 0x73, 0x61,
0x6c, 0x61, 0x53, 0x74, 0x61, 0x6d, 0x62, 0x75, 0x6c, 0x61, 0x44, 0x17e,
0x113, 0x72, 0x73, 0x69, 0x6a, 0x61, 0x4b, 0x61, 0x13c, 0x69, 0x146, 0x69,
0x6e, 0x67, 0x72, 0x61, 0x64, 0x61, 0x4b, 0x69, 0x72, 0x6f, 0x76, 0x61,
0x4b, 0x69, 0x6a, 0x65, 0x76, 0x61, 0x4c, 0x69, 0x73, 0x61, 0x62, 0x6f,
0x6e, 0x61, 0x13b, 0x75, 0x62, 0x13c, 0x61, 0x6e, 0x61, 0x4c, 0x6f, 0x6e,
0x64, 0x6f, 0x6e, 0x61, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x75,
0x72, 0x67, 0x61, 0x4d, 0x61, 0x64, 0x72, 0x69, 0x64, 0x65, 0x4d, 0x61,
0x72, 0x69, 0x65, 0x68, 0x61, 0x6d, 0x6e, 0x61, 0x4d, 0x69, 0x6e, 0x73,
0x6b, 0x61, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x76, 0x61, 0x50, 0x61, 0x72,
0x12b, 0x7a, 0x65, 0x50, 0x72, 0x101, 0x67, 0x61, 0x53, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x12b, 0x6e, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76,
0x61, 0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x76, 0x61, 0x53, 0x69, 0x6d,
0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x65, 0x53, 0x74, 0x6f, 0x6b,
0x68, 0x6f, 0x6c, 0x6d, 0x61, 0x54, 0x61, 0x6c, 0x6c, 0x69, 0x6e, 0x61,
0x54, 0x69, 0x72, 0x101, 0x6e, 0x61, 0x55, 0x13c, 0x6a, 0x61, 0x6e, 0x6f,
0x76, 0x73, 0x6b, 0x61, 0x56, 0x61, 0x64, 0x75, 0x63, 0x61, 0x56, 0x61,
0x74, 0x69, 0x6b, 0x101, 0x6e, 0x73, 0x56, 0x12b, 0x6e, 0x65, 0x56, 0x69,
0x13c, 0x146, 0x61, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x61, 0x5a, 0x61, 0x67, 0x72, 0x65, 0x62, 0x61, 0x43, 0x12b, 0x72, 0x69,
0x68, 0x65, 0x10c, 0x61, 0x67, 0x6f, 0x73, 0x61, 0x5a, 0x69, 0x65, 0x6d,
0x73, 0x76, 0x113, 0x74, 0x6b, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x4b,
0x6f, 0x6b, 0x6f, 0x73, 0x75, 0x20, 0x28, 0x4b, 0x12b, 0x6c, 0x69, 0x6e,
0x67, 0x61, 0x29, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x4b, 0x6f, 0x6d, 0x6f,
0x72, 0x61, 0x73, 0x4b, 0x65, 0x72, 0x67, 0x65, 0x6c, 0x113, 0x6e, 0x61,
0x20, 0x73, 0x61, 0x6c, 0x61, 0x4d, 0x61, 0x65, 0x4d, 0x61, 0x6c, 0x64,
0x12b, 0x76, 0x69, 0x6a, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x12b, 0x63, 0x69,
0x6a, 0x61, 0x4d, 0x61, 0x6a, 0x6f, 0x74, 0x61, 0x52, 0x65, 0x69, 0x6e,
0x6a, 0x6f, 0x6e, 0x61, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x4f, 0x6b, 0x6c,
0x65, 0x6e, 0x64, 0x61, 0x42, 0x75, 0x67, 0x65, 0x6e, 0x76, 0x69, 0x6c,
0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x10c, 0x65, 0x74, 0x65, 0x6d, 0x61,
0x10c, 0x16b, 0x6b, 0x61, 0x4c, 0x69, 0x65, 0x6c, 0x64, 0x69, 0x65, 0x6e,
0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x47, 0x61, 0x6d, 0x62,
0x6a, 0x113, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x47, 0x76, 0x61, 0x64,
0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x61, 0x47, 0x75, 0x61, 0x6d,
0x61, 0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0x65, 0x72, 0x69, 0x6a, 0x61,
0x4b, 0x69, 0x72, 0x69, 0x73, 0x69, 0x6d, 0x61, 0x73, 0x69, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x6a, 0x61, 0x4b, 0x76, 0x61, 0x64, 0x17e, 0x61, 0x6c,
0x65, 0x69, 0x6e, 0x61, 0x4d, 0x61, 0x64, 0x17e, 0x75, 0x72, 0x6f, 0x4d,
0x61, 0x72, 0x137, 0x12b, 0x7a, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73,
0x4d, 0x69, 0x64, 0x76, 0x65, 0x6a, 0x73, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x61, 0x4e, 0x75, 0x6d, 0x65, 0x61, 0x50, 0x61, 0x67, 0x6f,
0x70, 0x61, 0x67, 0x6f, 0x50, 0x69, 0x74, 0x6b, 0x113, 0x72, 0x6e, 0x61,
0x50, 0x6f, 0x6e, 0x70, 0x65, 0x6a, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6d,
0x6f, 0x72, 0x73, 0x62, 0x69, 0x53, 0x61, 0x69, 0x70, 0x61, 0x6e, 0x61,
0x54, 0x61, 0x69, 0x74, 0x69, 0x56, 0x65, 0x69, 0x6b, 0x61, 0x20, 0x73,
0x61, 0x6c, 0x61, 0x56, 0x6f, 0x6c, 0x69, 0x73, 0x61, 0x41, 0x62, 0x69,
0x64, 0x17e, 0x61, 0x6e, 0x61, 0x73, 0x41, 0x6c, 0x17e, 0x79, 0x72, 0x61,
0x73, 0x42, 0x61, 0x6d, 0x61, 0x6b, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67,
0x69, 0x73, 0x42, 0x61, 0x6e, 0x64, 0x17e, 0x75, 0x6c, 0x69, 0x73, 0x42,
0x6c, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x72, 0x61, 0x73, 0x42, 0x72, 0x61,
0x7a, 0x61, 0x76, 0x69, 0x6c, 0x69, 0x73, 0x42, 0x75, 0x17e, 0x75, 0x6d,
0x62, 0x16b, 0x72, 0x61, 0x4b, 0x61, 0x69, 0x72, 0x61, 0x73, 0x4b, 0x6f,
0x6e, 0x61, 0x6b, 0x72, 0x69, 0x73, 0x44, 0x61, 0x6b, 0x61, 0x72, 0x61,
0x73, 0x44, 0x61, 0x72, 0x20, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x61,
0x6d, 0x61, 0x73, 0x44, 0x17e, 0x69, 0x62, 0x75, 0x74, 0x69, 0x73, 0x61,
0x6c, 0x20, 0x41, 0x6a, 0x16b, 0x6e, 0x61, 0x73, 0x46, 0x72, 0x69, 0x74,
0x61, 0x75, 0x6e, 0x61, 0x73, 0x47, 0x61, 0x62, 0x6f, 0x72, 0x6f, 0x6e,
0x61, 0x73, 0x48, 0x61, 0x72, 0x61, 0x72, 0x117, 0x4a, 0x6f, 0x68, 0x61,
0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x61, 0x73, 0x44, 0x17e, 0x75,
0x62, 0x61, 0x43, 0x68, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x61, 0x73, 0x4b,
0x69, 0x67, 0x61, 0x6c, 0x69, 0x73, 0x4c, 0x61, 0x67, 0x6f, 0x73, 0x61,
0x73, 0x4c, 0x69, 0x62, 0x72, 0x65, 0x76, 0x69, 0x6c, 0x69, 0x73, 0x4c,
0x6f, 0x6d, 0x117, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62, 0x61, 0x161, 0x69,
0x73, 0x4d, 0x61, 0x6c, 0x61, 0x62, 0x61, 0x73, 0x4d, 0x61, 0x70, 0x75,
0x74, 0x61, 0x73, 0x4d, 0x62, 0x61, 0x62, 0x61, 0x6e, 0x117, 0x4d, 0x6f,
0x67, 0x61, 0x64, 0x69, 0x161, 0x61, 0x73, 0x4e, 0x61, 0x69, 0x72, 0x6f,
0x62, 0x69, 0x73, 0x4e, 0x69, 0x61, 0x6d, 0x117, 0x6a, 0x75, 0x73, 0x4e,
0x75, 0x61, 0x6b, 0x161, 0x6f, 0x74, 0x61, 0x73, 0x50, 0x6f, 0x72, 0x74,
0x6f, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x54,
0x6f, 0x6d, 0x117, 0x54, 0x75, 0x6e, 0x69, 0x73, 0x61, 0x73, 0x56, 0x69,
0x6e, 0x64, 0x68, 0x75, 0x6b, 0x61, 0x73, 0x45, 0x69, 0x64, 0x61, 0x6b,
0x61, 0x73, 0x41, 0x6e, 0x6b, 0x6f, 0x72, 0x69, 0x64, 0x17e, 0x61, 0x73,
0x41, 0x6e, 0x67, 0x69, 0x6c, 0x69, 0x6a, 0x61, 0x41, 0x72, 0x61, 0x67,
0x76, 0x61, 0x6a, 0x61, 0x6e, 0x61, 0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73,
0x20, 0x41, 0x69, 0x72, 0x117, 0x73, 0x43, 0x68, 0x75, 0x63, 0x68, 0x75,
0x6a, 0x75, 0x73, 0x4c, 0x61, 0x20, 0x52, 0x69, 0x6f, 0x63, 0x68, 0x61,
0x52, 0x69, 0x6f, 0x20, 0x47, 0x61, 0x6c, 0x6a, 0x65, 0x67, 0x6f, 0x73,
0x61, 0x73, 0x53, 0x61, 0x6c, 0x74, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x20,
0x43, 0x68, 0x75, 0x61, 0x6e, 0x61, 0x73, 0x53, 0x65, 0x6e, 0x74, 0x20,
0x4c, 0x75, 0x69, 0x73, 0x61, 0x73, 0x54, 0x75, 0x6b, 0x75, 0x6d, 0x61,
0x6e, 0x61, 0x73, 0x41, 0x73, 0x75, 0x6e, 0x73, 0x6a, 0x6f, 0x6e, 0x61,
0x73, 0x41, 0x74, 0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x6e, 0x61, 0x73, 0x42,
0x61, 0x68, 0x69, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61,
0x73, 0x61, 0x73, 0x42, 0x61, 0x72, 0x62, 0x61, 0x64, 0x6f, 0x73, 0x61,
0x73, 0x42, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x73, 0x42, 0x65, 0x6c, 0x69,
0x7a, 0x61, 0x73, 0x42, 0x6c, 0x61, 0x6e, 0x10d, 0x20, 0x53, 0x61, 0x62,
0x6c, 0x6f, 0x6e, 0x61, 0x73, 0x42, 0x75, 0x61, 0x20, 0x56, 0x69, 0x73,
0x74, 0x61, 0x4b, 0x65, 0x69, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x17e, 0x20,
0x42, 0x117, 0x6a, 0x75, 0x73, 0x4b, 0x61, 0x6d, 0x70, 0x6f, 0x20, 0x47,
0x72, 0x61, 0x6e, 0x64, 0x117, 0x4b, 0x61, 0x6e, 0x6b, 0x16b, 0x6e, 0x61,
0x73, 0x4b, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x73, 0x4b, 0x61,
0x6a, 0x65, 0x6e, 0x61, 0x73, 0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x61,
0x73, 0x10c, 0x69, 0x6b, 0x61, 0x67, 0x61, 0x10c, 0x69, 0x68, 0x75, 0x61,
0x68, 0x75, 0x61, 0x53, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x43, 0x68,
0x75, 0x61, 0x72, 0x65, 0x73, 0x61, 0x73, 0x4b, 0x72, 0x65, 0x73, 0x74,
0x6f, 0x6e, 0x61, 0x73, 0x4b, 0x69, 0x75, 0x72, 0x61, 0x73, 0x61, 0x6f,
0x44, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x68, 0x61, 0x75, 0x6e,
0x61, 0x73, 0x44, 0x6f, 0x73, 0x6f, 0x6e, 0x61, 0x73, 0x44, 0x6f, 0x73,
0x6f, 0x6e, 0x20, 0x4b, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x44, 0x65, 0x6e,
0x76, 0x65, 0x72, 0x69, 0x73, 0x44, 0x65, 0x74, 0x72, 0x6f, 0x69, 0x74,
0x61, 0x73, 0x45, 0x64, 0x6d, 0x6f, 0x6e, 0x74, 0x6f, 0x6e, 0x61, 0x73,
0x45, 0x69, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x117, 0x53, 0x61, 0x6c, 0x76,
0x61, 0x64, 0x6f, 0x72, 0x61, 0x73, 0x46, 0x6f, 0x72, 0x74, 0x20, 0x4e,
0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x61, 0x73, 0x47, 0x6c, 0x65, 0x69, 0x73,
0x20, 0x42, 0x117, 0x6a, 0x75, 0x73, 0x47, 0x75, 0x73, 0x20, 0x42, 0x117,
0x6a, 0x75, 0x73, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x65, 0x72,
0x6b, 0x61, 0x73, 0x47, 0x76, 0x61, 0x64, 0x61, 0x6c, 0x75, 0x70, 0x117,
0x47, 0x76, 0x61, 0x6a, 0x61, 0x6b, 0x69, 0x6c, 0x69, 0x73, 0x48, 0x61,
0x6c, 0x69, 0x66, 0x61, 0x6b, 0x73, 0x61, 0x73, 0x48, 0x65, 0x72, 0x6d,
0x6f, 0x73, 0x69, 0x6c, 0x6a, 0x61, 0x73, 0x4e, 0x6f, 0x6b, 0x73, 0x61,
0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x61,
0x72, 0x65, 0x6e, 0x67, 0x61, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x50, 0x69, 0x74, 0x65, 0x72, 0x73, 0x62, 0x65, 0x72,
0x67, 0x61, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x54, 0x65, 0x6c, 0x20, 0x53, 0x69, 0x74, 0x69, 0x73, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x76, 0x69, 0x73, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x73,
0x65, 0x6e, 0x61, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x56, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x61, 0x73, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x49, 0x6e, 0x75, 0x76, 0x69,
0x6b, 0x61, 0x73, 0x49, 0x6b, 0x61, 0x6c, 0x75, 0x69, 0x74, 0x61, 0x73,
0x44, 0x17e, 0x75, 0x6e, 0x61, 0x73, 0x4c, 0x75, 0x69, 0x73, 0x76, 0x69,
0x6c, 0x69, 0x73, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73, 0x65, 0x6c, 0x61,
0x73, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x6b, 0x69, 0x73, 0x4b,
0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x65, 0x69, 0x6b, 0x61, 0x73, 0x4c,
0x61, 0x20, 0x50, 0x61, 0x73, 0x61, 0x73, 0x4c, 0x6f, 0x73, 0x20, 0x41,
0x6e, 0x64, 0x17e, 0x65, 0x6c, 0x61, 0x73, 0x17d, 0x65, 0x6d, 0x75, 0x74,
0x69, 0x6e, 0x69, 0x73, 0x20, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x73,
0x20, 0x4b, 0x76, 0x6f, 0x72, 0x74, 0x65, 0x72, 0x69, 0x73, 0x4d, 0x61,
0x6e, 0x61, 0x75, 0x73, 0x61, 0x73, 0x4d, 0x61, 0x74, 0x61, 0x6d, 0x6f,
0x72, 0x6f, 0x73, 0x61, 0x73, 0x4d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61,
0x6e, 0x61, 0x73, 0x4d, 0x65, 0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x73,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x61, 0x73, 0x4d, 0x6f, 0x6e, 0x6b, 0x74, 0x6f, 0x6e,
0x61, 0x73, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x72, 0x117, 0x6a, 0x75, 0x73,
0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x76, 0x69, 0x64, 0x117, 0x6a, 0x61, 0x73,
0x4d, 0x6f, 0x6e, 0x74, 0x73, 0x65, 0x72, 0x61, 0x74, 0x61, 0x73, 0x4e,
0x61, 0x73, 0x61, 0x75, 0x4e, 0x69, 0x75, 0x6a, 0x6f, 0x72, 0x6b, 0x61,
0x73, 0x4e, 0x6f, 0x6d, 0x61, 0x73, 0x42, 0x6a, 0x75, 0x6c, 0x61, 0x2c,
0x20, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x69, 0x73, 0x2c,
0x20, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x4e, 0x69, 0x75, 0x20, 0x53, 0x65, 0x69, 0x6c, 0x65,
0x6d, 0x61, 0x73, 0x2c, 0x20, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x16b, 0x6b, 0x61, 0x73,
0x4f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x67, 0x61, 0x50, 0x61, 0x72, 0x61,
0x6d, 0x61, 0x72, 0x69, 0x62, 0x61, 0x73, 0x46, 0x69, 0x6e, 0x69, 0x6b,
0x73, 0x61, 0x73, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x6f, 0x20, 0x50, 0x72,
0x65, 0x6e, 0x73, 0x61, 0x73, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x6f, 0x66,
0x20, 0x53, 0x70, 0x65, 0x69, 0x6e, 0x61, 0x73, 0x50, 0x6f, 0x72, 0x74,
0x6f, 0x20, 0x56, 0x65, 0x6c, 0x6a, 0x61, 0x73, 0x50, 0x75, 0x65, 0x72,
0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x75, 0x6e, 0x74,
0x61, 0x20, 0x41, 0x72, 0x65, 0x6e, 0x61, 0x73, 0x61, 0x73, 0x52, 0x61,
0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x6c, 0x65, 0x74, 0x61, 0x73,
0x52, 0x65, 0x73, 0x69, 0x66, 0x117, 0x52, 0x65, 0x64, 0x17e, 0x69, 0x6e,
0x61, 0x52, 0x65, 0x73, 0x6f, 0x6c, 0x75, 0x74, 0x61, 0x73, 0x52, 0x69,
0x6f, 0x20, 0x42, 0x72, 0x61, 0x6e, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x6e,
0x74, 0x61, 0x72, 0x65, 0x6e, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x6a,
0x61, 0x67, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x44, 0x6f,
0x6d, 0x69, 0x6e, 0x67, 0x61, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x61,
0x75, 0x6c, 0x61, 0x73, 0x49, 0x74, 0x6f, 0x6b, 0x6f, 0x72, 0x74, 0x6f,
0x72, 0x6d, 0x69, 0x74, 0x61, 0x73, 0x53, 0x65, 0x6e, 0x20, 0x42, 0x61,
0x72, 0x74, 0x65, 0x6c, 0x65, 0x6d, 0x69, 0x53, 0x65, 0x6e, 0x74, 0x20,
0x44, 0x17e, 0x6f, 0x6e, 0x73, 0x61, 0x73, 0x53, 0x65, 0x6e, 0x74, 0x20,
0x4b, 0x69, 0x74, 0x73, 0x61, 0x73, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x4c,
0x75, 0x73, 0x69, 0x6a, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x54, 0x6f,
0x6d, 0x61, 0x73, 0x61, 0x73, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x56, 0x69,
0x6e, 0x63, 0x65, 0x6e, 0x74, 0x61, 0x73, 0x53, 0x76, 0x69, 0x66, 0x74,
0x20, 0x4b, 0x61, 0x72, 0x65, 0x6e, 0x74, 0x61, 0x73, 0x4b, 0x61, 0x6e,
0x61, 0x6b, 0x61, 0x73, 0x54, 0x69, 0x63, 0x68, 0x75, 0x61, 0x6e, 0x61,
0x54, 0x6f, 0x72, 0x6f, 0x6e, 0x74, 0x61, 0x73, 0x56, 0x61, 0x6e, 0x6b,
0x75, 0x76, 0x65, 0x72, 0x69, 0x73, 0x56, 0x61, 0x69, 0x74, 0x68, 0x6f,
0x72, 0x73, 0x61, 0x73, 0x56, 0x69, 0x6e, 0x69, 0x70, 0x65, 0x67, 0x61,
0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x61, 0x74, 0x61, 0x73, 0x4b, 0x65,
0x69, 0x73, 0x69, 0x73, 0x44, 0x65, 0x69, 0x76, 0x69, 0x73, 0x61, 0x73,
0x44, 0x69, 0x75, 0x6d, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x69, 0x73, 0x4d, 0x61, 0x6b, 0x76, 0x6f, 0x72,
0x69, 0x73, 0x4d, 0x6f, 0x73, 0x6f, 0x6e, 0x61, 0x73, 0x4d, 0x61, 0x6b,
0x6d, 0x65, 0x72, 0x64, 0x61, 0x73, 0x50, 0x61, 0x6c, 0x6d, 0x65, 0x72,
0x69, 0x73, 0x53, 0x69, 0x6f, 0x76, 0x61, 0x54, 0x72, 0x6f, 0x6c, 0x69,
0x73, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x4c, 0x6f, 0x6e,
0x67, 0x6a, 0x79, 0x72, 0x62, 0x69, 0x65, 0x6e, 0x61, 0x73, 0x41, 0x64,
0x65, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x69, 0x73, 0x41, 0x6b, 0x74, 0x69, 0x75, 0x62,
0x69, 0x6e, 0x73, 0x6b, 0x61, 0x73, 0x41, 0x161, 0x63, 0x68, 0x61, 0x62,
0x61, 0x64, 0x61, 0x73, 0x42, 0x61, 0x67, 0x64, 0x61, 0x64, 0x61, 0x73,
0x42, 0x61, 0x68, 0x72, 0x65, 0x69, 0x6e, 0x61, 0x73, 0x42, 0x61, 0x6e,
0x6b, 0x6f, 0x6b, 0x61, 0x73, 0x42, 0x61, 0x72, 0x6e, 0x61, 0x75, 0x6c,
0x61, 0x73, 0x42, 0x65, 0x69, 0x72, 0x75, 0x74, 0x61, 0x73, 0x42, 0x69,
0x161, 0x6b, 0x65, 0x6b, 0x61, 0x73, 0x42, 0x72, 0x75, 0x6e, 0x117, 0x6a,
0x75, 0x73, 0x10c, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x61,
0x73, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x61, 0x73, 0x44, 0x61, 0x6d,
0x61, 0x73, 0x6b, 0x61, 0x73, 0x44, 0x69, 0x6c, 0x69, 0x73, 0x44, 0x75,
0x62, 0x61, 0x6a, 0x75, 0x73, 0x44, 0x75, 0x161, 0x61, 0x6e, 0x62, 0x117,
0x47, 0x61, 0x7a, 0x6f, 0x73, 0x20, 0x72, 0x75, 0x6f, 0x17e, 0x61, 0x73,
0x48, 0x65, 0x62, 0x72, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x161, 0x69,
0x6d, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67,
0x61, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x61, 0x73, 0x4a, 0x65, 0x72, 0x75, 0x7a, 0x61, 0x6c,
0x117, 0x4b, 0x61, 0x62, 0x75, 0x6c, 0x61, 0x73, 0x4b, 0x61, 0x6d, 0x10d,
0x69, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x10d, 0x69, 0x73,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61,
0x73, 0x4b, 0x76, 0x61, 0x6c, 0x61, 0x20, 0x4c, 0x75, 0x6d, 0x70, 0x16b,
0x72, 0x61, 0x73, 0x4b, 0x75, 0x10d, 0x69, 0x6e, 0x67, 0x61, 0x73, 0x4b,
0x75, 0x76, 0x65, 0x69, 0x74, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x61, 0x73, 0x4d, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x74, 0x61, 0x73, 0x4e, 0x6f, 0x76,
0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x65, 0x63, 0x6b, 0x61, 0x73, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x73, 0x4f,
0x6d, 0x73, 0x6b, 0x61, 0x73, 0x55, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x61,
0x73, 0x50, 0x6e, 0x6f, 0x6d, 0x70, 0x65, 0x6e, 0x69, 0x73, 0x50, 0x6f,
0x6e, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x6b, 0x61, 0x73, 0x50, 0x63, 0x68,
0x65, 0x6e, 0x6a, 0x61, 0x6e, 0x61, 0x73, 0x4b, 0x61, 0x74, 0x61, 0x72,
0x61, 0x73, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x6a, 0x75, 0x73,
0x4b, 0x7a, 0x79, 0x6c, 0x2d, 0x4f, 0x72, 0x64, 0x61, 0x52, 0x69, 0x6a,
0x61, 0x64, 0x61, 0x73, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x64, 0x61,
0x73, 0x53, 0x65, 0x75, 0x6c, 0x61, 0x73, 0x160, 0x61, 0x6e, 0x63, 0x68,
0x61, 0x6a, 0x75, 0x73, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x16b, 0x72,
0x61, 0x73, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0x79,
0x6d, 0x73, 0x6b, 0x61, 0x73, 0x54, 0x61, 0x69, 0x70, 0x117, 0x6a, 0x75,
0x73, 0x54, 0x61, 0x161, 0x6b, 0x65, 0x6e, 0x74, 0x61, 0x73, 0x54, 0x62,
0x69, 0x6c, 0x69, 0x73, 0x69, 0x73, 0x54, 0x65, 0x68, 0x65, 0x72, 0x61,
0x6e, 0x61, 0x73, 0x54, 0x6f, 0x6b, 0x69, 0x6a, 0x61, 0x73, 0x54, 0x6f,
0x6d, 0x73, 0x6b, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x61, 0x73, 0x55, 0x73, 0x74, 0x20, 0x4e, 0x65, 0x72,
0x61, 0x56, 0x69, 0x65, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x73,
0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x52, 0x61, 0x6e,
0x67, 0x16b, 0x6e, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x73, 0x4a, 0x65, 0x72, 0x65,
0x76, 0x61, 0x6e, 0x61, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x61, 0x69, 0x4b,
0x61, 0x6e, 0x61, 0x72, 0x173, 0x20, 0x73, 0x61, 0x6c, 0x6f, 0x73, 0x17d,
0x61, 0x6c, 0x69, 0x61, 0x73, 0x69, 0x73, 0x20, 0x4b, 0x79, 0x161, 0x75,
0x6c, 0x79, 0x73, 0x46, 0x61, 0x72, 0x65, 0x72, 0x61, 0x69, 0x52, 0x65,
0x69, 0x6b, 0x6a, 0x61, 0x76, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x69, 0x65,
0x74, 0x173, 0x20, 0x44, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x61,
0x53, 0x65, 0x6e, 0x74, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x53,
0x74, 0x65, 0x6e, 0x6c, 0x69, 0x73, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x69,
0x64, 0x117, 0x42, 0x72, 0x69, 0x73, 0x62, 0x61, 0x6e, 0x61, 0x73, 0x42,
0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x20, 0x48, 0x69, 0x6c, 0x69, 0x73, 0x44,
0x61, 0x72, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x62, 0x61, 0x72,
0x74, 0x61, 0x73, 0x4c, 0x69, 0x6e, 0x64, 0x65, 0x6d, 0x61, 0x6e, 0x61,
0x73, 0x4c, 0x6f, 0x72, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x75, 0x20, 0x73,
0x61, 0x6c, 0x61, 0x4d, 0x65, 0x6c, 0x62, 0x75, 0x72, 0x6e, 0x61, 0x73,
0x50, 0x65, 0x72, 0x74, 0x61, 0x73, 0x53, 0x69, 0x64, 0x6e, 0x117, 0x6a,
0x75, 0x73, 0x41, 0x6d, 0x73, 0x74, 0x65, 0x72, 0x64, 0x61, 0x6d, 0x61,
0x73, 0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0x68, 0x61, 0x6e, 0x117, 0x41,
0x74, 0x117, 0x6e, 0x61, 0x69, 0x42, 0x65, 0x6c, 0x67, 0x72, 0x61, 0x64,
0x61, 0x73, 0x42, 0x65, 0x72, 0x6c, 0x79, 0x6e, 0x61, 0x73, 0x42, 0x72,
0x69, 0x75, 0x73, 0x65, 0x6c, 0x69, 0x73, 0x42, 0x75, 0x6b, 0x61, 0x72,
0x65, 0x161, 0x74, 0x61, 0x73, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x161,
0x74, 0x61, 0x73, 0x42, 0x69, 0x75, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e,
0x61, 0x73, 0x4b, 0x69, 0x161, 0x69, 0x6e, 0x69, 0x6f, 0x76, 0x61, 0x73,
0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x67, 0x61, 0x44, 0x75, 0x62,
0x6c, 0x69, 0x6e, 0x61, 0x73, 0x47, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74,
0x61, 0x72, 0x61, 0x73, 0x47, 0x65, 0x72, 0x6e, 0x73, 0x69, 0x73, 0x48,
0x65, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x73, 0x4d, 0x65, 0x6e, 0x6f,
0x20, 0x73, 0x61, 0x6c, 0x61, 0x53, 0x74, 0x61, 0x6d, 0x62, 0x75, 0x6c,
0x61, 0x73, 0x44, 0x17e, 0x65, 0x72, 0x73, 0x69, 0x73, 0x4b, 0x61, 0x6c,
0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x61, 0x73, 0x4b, 0x69,
0x72, 0x6f, 0x76, 0x61, 0x73, 0x4b, 0x69, 0x6a, 0x65, 0x76, 0x61, 0x73,
0x4c, 0x6f, 0x6e, 0x64, 0x6f, 0x6e, 0x61, 0x73, 0x4c, 0x69, 0x75, 0x6b,
0x73, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x61, 0x73, 0x4d, 0x61, 0x64,
0x72, 0x69, 0x64, 0x61, 0x73, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x68,
0x61, 0x6d, 0x69, 0x6e, 0x61, 0x4d, 0x69, 0x6e, 0x73, 0x6b, 0x61, 0x73,
0x4d, 0x6f, 0x6e, 0x61, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x73, 0x6b, 0x76,
0x61, 0x4f, 0x73, 0x6c, 0x61, 0x73, 0x50, 0x61, 0x72, 0x79, 0x17e, 0x69,
0x75, 0x73, 0x52, 0x79, 0x67, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61,
0x72, 0x69, 0x6e, 0x61, 0x73, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76,
0x61, 0x73, 0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x76, 0x61, 0x73, 0x53,
0x69, 0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x53,
0x6b, 0x6f, 0x70, 0x6a, 0x117, 0x53, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x6c,
0x6d, 0x61, 0x73, 0x54, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x73, 0x55, 0x6c,
0x6a, 0x61, 0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x64,
0x75, 0x63, 0x61, 0x73, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e, 0x61,
0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x73,
0x56, 0x61, 0x72, 0x161, 0x75, 0x76, 0x61, 0x5a, 0x61, 0x67, 0x72, 0x65,
0x62, 0x61, 0x73, 0x43, 0x69, 0x75, 0x72, 0x69, 0x63, 0x68, 0x61, 0x73,
0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x79, 0x76, 0x61,
0x73, 0x10c, 0x61, 0x67, 0x6f, 0x73, 0x61, 0x73, 0x4b, 0x61, 0x6c, 0x117,
0x64, 0x173, 0x20, 0x53, 0x61, 0x6c, 0x61, 0x4b, 0x6f, 0x6b, 0x6f, 0x73,
0x173, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x4b, 0x65, 0x72, 0x67, 0x65, 0x6c,
0x65, 0x6e, 0x61, 0x73, 0x4d, 0x61, 0x68, 0x117, 0x4d, 0x61, 0x6c, 0x64,
0x79, 0x76, 0x61, 0x69, 0x4d, 0x61, 0x6a, 0x6f, 0x74, 0x61, 0x73, 0x52,
0x65, 0x75, 0x6e, 0x6a, 0x6f, 0x6e, 0x61, 0x73, 0x4f, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x73, 0x42, 0x75, 0x67, 0x65, 0x6e, 0x76, 0x69, 0x6c,
0x69, 0x73, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x61, 0x73, 0x10c, 0x75, 0x6b,
0x61, 0x73, 0x56, 0x65, 0x6c, 0x79, 0x6b, 0x173, 0x20, 0x73, 0x61, 0x6c,
0x61, 0x45, 0x66, 0x61, 0x74, 0x117, 0x46, 0x61, 0x6b, 0x61, 0x6f, 0x66,
0x61, 0x73, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x73, 0x46, 0x75, 0x6e, 0x61,
0x66, 0x75, 0x74, 0x69, 0x73, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x61, 0x69, 0x47, 0x61, 0x6d, 0x62, 0x79, 0x72, 0x61, 0x73, 0x47, 0x76,
0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x69, 0x73, 0x47,
0x75, 0x61, 0x6d, 0x61, 0x73, 0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0x75,
0x72, 0x69, 0x73, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x74,
0x69, 0x73, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x6a, 0x117, 0x4b, 0x76, 0x61,
0x64, 0x17e, 0x61, 0x6c, 0x65, 0x69, 0x6e, 0x61, 0x73, 0x4d, 0x61, 0x64,
0x17e, 0x16b, 0x72, 0x61, 0x73, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x6f,
0x20, 0x73, 0x61, 0x6c, 0x6f, 0x73, 0x4d, 0x69, 0x64, 0x76, 0x117, 0x6a,
0x75, 0x73, 0x4e, 0x69, 0x75, 0x6a, 0x117, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x61, 0x73, 0x4e, 0x75, 0x6d, 0x117, 0x6a, 0x61, 0x50, 0x61,
0x67, 0x6f, 0x20, 0x50, 0x61, 0x67, 0x61, 0x73, 0x50, 0x69, 0x74, 0x6b,
0x65, 0x72, 0x6e, 0x6f, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x50, 0x6f, 0x6e,
0x61, 0x70, 0x117, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x4d, 0x6f, 0x72, 0x73,
0x62, 0x69, 0x73, 0x53, 0x61, 0x69, 0x70, 0x61, 0x6e, 0x61, 0x73, 0x54,
0x61, 0x69, 0x74, 0x69, 0x73, 0x56, 0x65, 0x69, 0x6b, 0x6f, 0x20, 0x73,
0x61, 0x6c, 0x61, 0x56, 0x6f, 0x6c, 0x69, 0x73, 0x61, 0x73, 0x4b, 0x61,
0x69, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x6b, 0x65, 0x20, 0x6b, 0x75, 0x70,
0x79, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x2d, 0x6d, 0x11b, 0x73, 0x74,
0x6f, 0x53, 0x74, 0x2e, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9, 0x6c, 0x65,
0x6d, 0x79, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x44, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x48, 0x6f, 0x2d, 0x43, 0x68, 0x69,
0x2d, 0x4d, 0x69, 0x6e, 0x68, 0x2d, 0x6d, 0x11b, 0x73, 0x74, 0x6f, 0x50,
0x6f, 0x72, 0x74, 0x20, 0x4e, 0x75, 0x6d, 0x62, 0x61, 0x79, 0x4e, 0x6f,
0x77, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x6a, 0x65, 0x74, 0x73, 0x6b, 0x4a,
0x65, 0x72, 0x65, 0x77, 0x61, 0x6e, 0x41, 0x63, 0x6f, 0x72, 0x79, 0x46,
0xe4, 0x72, 0xf6, 0x6a, 0x65, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x64,
0x6e, 0x6a, 0x6f, 0x77, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x73, 0x6b, 0x61, 0x42, 0x11b, 0x142, 0x6f, 0x67, 0x72, 0x6f, 0x64, 0x4b,
0x69, 0x161, 0x69, 0x6e, 0x61, 0x75, 0x57, 0x69, 0x6c, 0x6e, 0x61, 0x57,
0x61, 0x72, 0x161, 0x61, 0x77, 0x61, 0x47, 0xf3, 0x64, 0x6f, 0x77, 0x6e,
0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69,
0x77, 0x79, 0x4a, 0x61, 0x74, 0x161, 0x6f, 0x77, 0x6e, 0x65, 0x20, 0x6b,
0x75, 0x70, 0x79, 0x4b, 0x61, 0x6d, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b,
0x61, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f,
0x72, 0x67, 0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x65,
0x6c, 0x65, 0x6e, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x65, 0x72, 0x67, 0x45, 0x72, 0x69, 0x76, 0x61, 0x6e, 0x4b,
0x61, 0x6e, 0x61, 0x72, 0x65, 0x73, 0x63, 0x68, 0x20, 0x49, 0x6e, 0x73,
0x65, 0x6c, 0x65, 0x6e, 0x42, 0x72, 0xe9, 0x69, 0x73, 0x73, 0x65, 0x6c,
0x4b, 0x69, 0x73, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x75, 0x4c, 0xeb, 0x74,
0x7a, 0x65, 0x62, 0x75, 0x65, 0x72, 0x67, 0x52, 0x6f, 0x75, 0x6d, 0x5a,
0xfc, 0x72, 0x65, 0x63, 0x68, 0x43, 0x68, 0x72, 0xeb, 0x73, 0x63, 0x68,
0x74, 0x64, 0x61, 0x67, 0x73, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x4f, 0x75,
0x73, 0x63, 0x68, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x411,
0x430, 0x43d, 0x45f, 0x443, 0x43b, 0x411, 0x43b, 0x430, 0x43d, 0x442, 0x430, 0x458,
0x440, 0x41b, 0x438, 0x431, 0x440, 0x432, 0x438, 0x43b, 0x421, 0x430, 0x443, 0x43d,
0x20, 0x422, 0x43e, 0x43c, 0x435, 0x410, 0x43d, 0x442, 0x438, 0x433, 0x430, 0x423,
0x448, 0x443, 0x430, 0x458, 0x430, 0x41a, 0x43e, 0x440, 0x430, 0x43b, 0x20, 0x425,
0x430, 0x440, 0x431, 0x43e, 0x440, 0x411, 0x430, 0x445, 0x438, 0x430, 0x411, 0x430,
0x445, 0x438, 0x458, 0x430, 0x20, 0x434, 0x435, 0x20, 0x411, 0x430, 0x43d, 0x434,
0x435, 0x440, 0x430, 0x441, 0x41a, 0x430, 0x458, 0x435, 0x43d, 0x430, 0x41a, 0x430,
0x458, 0x43c, 0x430, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x438, 0x421, 0x438, 0x443, 0x434, 0x430, 0x434, 0x20, 0x425, 0x443,
0x430, 0x440, 0x435, 0x437, 0x41a, 0x443, 0x458, 0x430, 0x431, 0x430, 0x414, 0x430,
0x43d, 0x43c, 0x430, 0x440, 0x43a, 0x448, 0x430, 0x43d, 0x415, 0x43b, 0x20, 0x421,
0x430, 0x43b, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x413, 0x432, 0x430, 0x458, 0x430,
0x43d, 0x430, 0x415, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x459, 0x43e, 0x418, 0x43d,
0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x43f, 0x43e, 0x43b, 0x438, 0x441, 0x41d,
0x43e, 0x43a, 0x441, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d,
0x430, 0x41f, 0x438, 0x442, 0x435, 0x440, 0x441, 0x431, 0x443, 0x440, 0x433, 0x2c,
0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x432,
0x435, 0x458, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430,
0x412, 0x435, 0x43d, 0x441, 0x435, 0x43d, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438,
0x458, 0x430, 0x43d, 0x430, 0x41a, 0x440, 0x430, 0x43b, 0x435, 0x43d, 0x434, 0x430,
0x458, 0x43a, 0x41b, 0x43e, 0x441, 0x20, 0x410, 0x43d, 0x45f, 0x435, 0x43b, 0x435,
0x441, 0x414, 0x43e, 0x43b, 0x435, 0x43d, 0x20, 0x41f, 0x440, 0x438, 0x43d, 0x441,
0x20, 0x43a, 0x432, 0x43e, 0x440, 0x442, 0x440, 0x41c, 0x438, 0x43a, 0x435, 0x43b,
0x430, 0x43d, 0x411, 0x458, 0x443, 0x43b, 0x430, 0x2c, 0x20, 0x421, 0x435, 0x432,
0x435, 0x440, 0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x40a,
0x443, 0x20, 0x421, 0x430, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x421, 0x435, 0x432,
0x435, 0x440, 0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41f,
0x43e, 0x440, 0x442, 0x20, 0x43e, 0x20, 0x41f, 0x440, 0x435, 0x43d, 0x441, 0x41f,
0x43e, 0x440, 0x442, 0x20, 0x43e, 0x432, 0x20, 0x421, 0x43f, 0x435, 0x458, 0x43d,
0x420, 0x435, 0x43d, 0x43a, 0x438, 0x43d, 0x20, 0x418, 0x43d, 0x43b, 0x435, 0x442,
0x420, 0x435, 0x45f, 0x430, 0x458, 0x43d, 0x430, 0x421, 0x435, 0x43d, 0x442, 0x20,
0x411, 0x430, 0x440, 0x442, 0x43e, 0x43b, 0x43e, 0x43c, 0x435, 0x458, 0x421, 0x435,
0x43d, 0x442, 0x20, 0x40f, 0x43e, 0x43d, 0x441, 0x421, 0x432, 0x435, 0x442, 0x438,
0x20, 0x41a, 0x438, 0x442, 0x441, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x41b, 0x443,
0x441, 0x438, 0x458, 0x430, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x422, 0x43e, 0x43c,
0x430, 0x441, 0x41a, 0x435, 0x458, 0x441, 0x438, 0x414, 0x438, 0x43c, 0x43e, 0x43d,
0x20, 0x414, 0x438, 0x440, 0x432, 0x438, 0x43b, 0x41c, 0x430, 0x43a, 0x432, 0x43e,
0x440, 0x438, 0x421, 0x430, 0x458, 0x43e, 0x432, 0x430, 0x41b, 0x43e, 0x43d, 0x433,
0x458, 0x435, 0x440, 0x431, 0x438, 0x458, 0x435, 0x43d, 0x410, 0x448, 0x43a, 0x430,
0x431, 0x430, 0x434, 0x415, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x438, 0x43c, 0x41f,
0x43e, 0x43d, 0x442, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x43a, 0x421, 0x430, 0x43a,
0x430, 0x43b, 0x438, 0x43d, 0x412, 0x438, 0x435, 0x43d, 0x442, 0x438, 0x458, 0x430,
0x43d, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x438, 0x41a,
0x430, 0x43d, 0x430, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x438, 0x424, 0x430, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x408, 0x443, 0x43a, 0x43b, 0x430, 0x411, 0x438,
0x437, 0x438, 0x43d, 0x433, 0x435, 0x43d, 0x41a, 0x438, 0x448, 0x438, 0x43d, 0x430,
0x443, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x20, 0x41c, 0x430, 0x43d, 0x40f,
0x435, 0x440, 0x437, 0x438, 0x41c, 0x430, 0x440, 0x438, 0x458, 0x435, 0x445, 0x430,
0x43c, 0x435, 0x43d, 0x421, 0x43a, 0x43e, 0x43f, 0x458, 0x435, 0x423, 0x43b, 0x458,
0x430, 0x43d, 0x43e, 0x432, 0x441, 0x43a, 0x412, 0x438, 0x43b, 0x43d, 0x443, 0x441,
0x411, 0x43e, 0x436, 0x438, 0x45c, 0x435, 0x43d, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x438, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x446,
0x438, 0x443, 0x441, 0x420, 0x435, 0x458, 0x443, 0x43d, 0x438, 0x43e, 0x43d, 0x411,
0x443, 0x433, 0x430, 0x43d, 0x432, 0x438, 0x43b, 0x427, 0x443, 0x43a, 0x412, 0x435,
0x43b, 0x438, 0x433, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x413, 0x432, 0x430, 0x43c, 0x41a, 0x432, 0x430, 0x45f,
0x430, 0x43b, 0x435, 0x458, 0x43d, 0x41c, 0x430, 0x440, 0x43a, 0x435, 0x441, 0x43a,
0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41f, 0x438, 0x442,
0x43a, 0x435, 0x440, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x438, 0x41f, 0x43e, 0x43d, 0x43f, 0x435, 0x458, 0x905, 0x92c, 0x93f,
0x91c, 0x93e, 0x928, 0x905, 0x915, 0x930, 0x93e, 0x905, 0x926, 0x93f, 0x938, 0x20,
0x905, 0x92c, 0x93e, 0x92c, 0x93e, 0x905, 0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x92c,
0x92e, 0x93e, 0x915, 0x94b, 0x92c, 0x902, 0x917, 0x941, 0x908, 0x92c, 0x902, 0x91c,
0x941, 0x932, 0x92c, 0x94d, 0x932, 0x947, 0x902, 0x91f, 0x93e, 0x92f, 0x930, 0x92c,
0x94d, 0x930, 0x947, 0x91c, 0x93e, 0x935, 0x93f, 0x932, 0x915, 0x93e, 0x939, 0x93f,
0x930, 0x93e, 0x938, 0x93f, 0x909, 0x924, 0x93e, 0x921, 0x93e, 0x930, 0x947, 0x20,
0x938, 0x932, 0x93e, 0x92e, 0x921, 0x93e, 0x909, 0x932, 0x93e, 0x90f, 0x932, 0x20,
0x910, 0x92f, 0x942, 0x928, 0x917, 0x947, 0x92c, 0x94b, 0x930, 0x94b, 0x928, 0x91c,
0x94b, 0x939, 0x93e, 0x928, 0x93f, 0x938, 0x92c, 0x930, 0x94d, 0x917, 0x915, 0x92e,
0x92a, 0x93e, 0x932, 0x93e, 0x916, 0x93e, 0x930, 0x924, 0x94c, 0x92e, 0x92e, 0x92a,
0x942, 0x924, 0x94b, 0x92e, 0x938, 0x948, 0x930, 0x942, 0x92e, 0x947, 0x902, 0x92c,
0x93e, 0x92c, 0x93e, 0x928, 0x947, 0x928, 0x947, 0x91c, 0x947, 0x92e, 0x940, 0x928,
0x93e, 0x928, 0x93f, 0x92f, 0x93e, 0x92e, 0x947, 0x92f, 0x928, 0x94c, 0x915, 0x91a,
0x94b, 0x91f, 0x914, 0x917, 0x93e, 0x921, 0x941, 0x917, 0x942, 0x92c, 0x93f, 0x902,
0x921, 0x94b, 0x90f, 0x915, 0x905, 0x926, 0x915, 0x90f, 0x902, 0x915, 0x94b, 0x930,
0x947, 0x91c, 0x90f, 0x902, 0x91f, 0x93f, 0x917, 0x941, 0x906, 0x905, 0x930, 0x93e,
0x917, 0x941, 0x90f, 0x928, 0x93e, 0x91c, 0x941, 0x91c, 0x941, 0x92f, 0x92e, 0x947,
0x902, 0x921, 0x94b, 0x91c, 0x93e, 0x930, 0x93f, 0x92f, 0x94b, 0x20, 0x917, 0x948,
0x932, 0x917, 0x938, 0x938, 0x93e, 0x932, 0x91f, 0x93e, 0x938, 0x928, 0x20, 0x91c,
0x941, 0x906, 0x928, 0x938, 0x928, 0x20, 0x932, 0x942, 0x908, 0x938, 0x909, 0x936,
0x941, 0x92f, 0x93e, 0x92f, 0x93e, 0x905, 0x938, 0x941, 0x928, 0x936, 0x93f, 0x92f,
0x928, 0x905, 0x91f, 0x93f, 0x915, 0x94b, 0x915, 0x928, 0x92c, 0x939, 0x93f, 0x92f,
0x93e, 0x20, 0x92c, 0x902, 0x921, 0x947, 0x930, 0x93e, 0x938, 0x92c, 0x93e, 0x930,
0x92c, 0x921, 0x94b, 0x938, 0x92c, 0x948, 0x932, 0x948, 0x92e, 0x92c, 0x94d, 0x932,
0x948, 0x902, 0x915, 0x2d, 0x938, 0x92c, 0x932, 0x94b, 0x928, 0x92c, 0x94b, 0x906,
0x20, 0x92c, 0x93f, 0x938, 0x94d, 0x91f, 0x93e, 0x92c, 0x94b, 0x92f, 0x91c, 0x940,
0x915, 0x948, 0x92e, 0x94d, 0x92c, 0x94d, 0x930, 0x940, 0x91c, 0x20, 0x92c, 0x947,
0x915, 0x947, 0x92e, 0x94d, 0x92a, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x947, 0x928,
0x921, 0x93e, 0x915, 0x941, 0x92f, 0x93e, 0x92c, 0x93e, 0x915, 0x94d, 0x92f, 0x942,
0x930, 0x93e, 0x938, 0x93e, 0x913, 0x926, 0x92e, 0x930, 0x94d, 0x915, 0x936, 0x928,
0x921, 0x93e, 0x909, 0x938, 0x928, 0x921, 0x93e, 0x909, 0x938, 0x928, 0x20, 0x915,
0x94d, 0x930, 0x940, 0x915, 0x921, 0x947, 0x928, 0x92c, 0x930, 0x921, 0x947, 0x91f,
0x94d, 0x930, 0x93e, 0x92f, 0x91f, 0x90f, 0x921, 0x92e, 0x94b, 0x902, 0x91f, 0x94b,
0x928, 0x908, 0x930, 0x942, 0x928, 0x947, 0x92a, 0x947, 0x92b, 0x94b, 0x930, 0x94d,
0x91f, 0x20, 0x928, 0x947, 0x932, 0x938, 0x928, 0x92b, 0x94b, 0x930, 0x94d, 0x91f,
0x93e, 0x932, 0x947, 0x91c, 0x93e, 0x917, 0x94d, 0x935, 0x93e, 0x921, 0x947, 0x932,
0x942, 0x92a, 0x917, 0x941, 0x906, 0x92f, 0x93e, 0x915, 0x94d, 0x935, 0x93f, 0x932,
0x939, 0x930, 0x92e, 0x94b, 0x938, 0x93f, 0x932, 0x94b, 0x928, 0x94b, 0x915, 0x94d,
0x938, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92e,
0x947, 0x930, 0x948, 0x902, 0x917, 0x94b, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f,
0x92f, 0x93e, 0x928, 0x93e, 0x92c, 0x940, 0x92c, 0x947, 0x2c, 0x20, 0x907, 0x902,
0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x928, 0x938, 0x947, 0x902,
0x938, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92c,
0x93f, 0x928, 0x93e, 0x92e, 0x948, 0x915, 0x2c, 0x20, 0x907, 0x902, 0x921, 0x93f,
0x92f, 0x93e, 0x928, 0x93e, 0x907, 0x928, 0x941, 0x92c, 0x93f, 0x915, 0x907, 0x915,
0x93e, 0x932, 0x935, 0x93f, 0x91f, 0x91c, 0x941, 0x928, 0x947, 0x909, 0x92e, 0x94b,
0x902, 0x91f, 0x940, 0x91a, 0x947, 0x932, 0x94b, 0x2c, 0x20, 0x915, 0x947, 0x902,
0x91f, 0x941, 0x915, 0x940, 0x915, 0x947, 0x932, 0x947, 0x902, 0x921, 0x93f, 0x91c,
0x94d, 0x915, 0x932, 0x93e, 0x938, 0x20, 0x90f, 0x902, 0x91c, 0x93f, 0x932, 0x93f,
0x938, 0x932, 0x94b, 0x905, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x93f, 0x902, 0x938,
0x947, 0x938, 0x20, 0x915, 0x94d, 0x935, 0x93e, 0x91f, 0x930, 0x92e, 0x938, 0x940,
0x92f, 0x94b, 0x92e, 0x93e, 0x930, 0x940, 0x917, 0x94c, 0x91f, 0x92e, 0x91c, 0x93e,
0x91f, 0x932, 0x93e, 0x928, 0x92e, 0x948, 0x928, 0x94b, 0x92e, 0x93f, 0x928, 0x940,
0x92e, 0x948, 0x930, 0x93f, 0x921, 0x93e, 0x92e, 0x948, 0x91f, 0x932, 0x93e, 0x915,
0x93e, 0x91f, 0x932, 0x93e, 0x92e, 0x948, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b,
0x20, 0x938, 0x93f, 0x91f, 0x940, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932,
0x93e, 0x928, 0x92e, 0x93e, 0x928, 0x94d, 0x915, 0x91f, 0x93e, 0x928, 0x92e, 0x93e,
0x928, 0x91f, 0x947, 0x948, 0x930, 0x940, 0x92e, 0x94b, 0x902, 0x91f, 0x948, 0x935,
0x93f, 0x921, 0x93f, 0x92f, 0x94b, 0x928, 0x938, 0x93e, 0x909, 0x928, 0x94d, 0x92f,
0x942, 0x92f, 0x93e, 0x930, 0x94d, 0x915, 0x92c, 0x93f, 0x92f, 0x942, 0x932, 0x93e,
0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x915, 0x94b, 0x91f,
0x93e, 0x938, 0x947, 0x902, 0x91f, 0x930, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x928, 0x94d, 0x92f, 0x942, 0x20,
0x938, 0x932, 0x947, 0x92e, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20,
0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x906,
0x909, 0x20, 0x92a, 0x94d, 0x930, 0x93f, 0x902, 0x938, 0x92a, 0x94b, 0x930, 0x94d,
0x91f, 0x20, 0x906, 0x92b, 0x20, 0x938, 0x94d, 0x92a, 0x947, 0x928, 0x92a, 0x94b,
0x930, 0x94d, 0x91f, 0x94b, 0x20, 0x92c, 0x947, 0x932, 0x94d, 0x939, 0x94b, 0x92a,
0x941, 0x902, 0x91f, 0x93e, 0x20, 0x905, 0x930, 0x947, 0x928, 0x93e, 0x91c, 0x930,
0x93f, 0x938, 0x940, 0x92b, 0x930, 0x948, 0x917, 0x93f, 0x928, 0x93e, 0x930, 0x947,
0x938, 0x93e, 0x932, 0x942, 0x924, 0x930, 0x93f, 0x92f, 0x94b, 0x20, 0x92c, 0x94d,
0x930, 0x947, 0x902, 0x915, 0x94b, 0x938, 0x947, 0x902, 0x91f, 0x94b, 0x20, 0x921,
0x94b, 0x92e, 0x93f, 0x928, 0x917, 0x94b, 0x907, 0x91f, 0x94b, 0x915, 0x94b, 0x930,
0x94d, 0x91f, 0x942, 0x930, 0x92e, 0x93f, 0x91f, 0x938, 0x93f, 0x924, 0x915, 0x93e,
0x938, 0x947, 0x902, 0x91f, 0x20, 0x92c, 0x930, 0x94d, 0x925, 0x947, 0x932, 0x92e,
0x940, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x91c, 0x93e, 0x928, 0x94d, 0x938, 0x938,
0x947, 0x902, 0x91f, 0x20, 0x932, 0x942, 0x936, 0x93f, 0x92f, 0x93e, 0x938, 0x947,
0x902, 0x91f, 0x20, 0x925, 0x94b, 0x92e, 0x938, 0x938, 0x94d, 0x935, 0x93f, 0x92b,
0x94d, 0x91f, 0x20, 0x915, 0x930, 0x947, 0x902, 0x91f, 0x924, 0x941, 0x932, 0x947,
0x92c, 0x948, 0x902, 0x915, 0x941, 0x92c, 0x930, 0x935, 0x93e, 0x907, 0x91f, 0x939,
0x93e, 0x930, 0x94d, 0x938, 0x92c, 0x93f, 0x928, 0x93f, 0x92a, 0x948, 0x917, 0x92f,
0x915, 0x941, 0x91f, 0x93e, 0x91f, 0x92e, 0x948, 0x915, 0x94d, 0x935, 0x948, 0x930,
0x940, 0x932, 0x93e, 0x928, 0x94d, 0x917, 0x93f, 0x92f, 0x930, 0x92c, 0x947, 0x928,
0x905, 0x926, 0x928, 0x905, 0x932, 0x94d, 0x92e, 0x93e, 0x924, 0x940, 0x90f, 0x928,
0x93e, 0x921, 0x93f, 0x930, 0x905, 0x924, 0x93f, 0x930, 0x93e, 0x909, 0x92c, 0x948,
0x902, 0x915, 0x93e, 0x915, 0x91a, 0x93f, 0x924, 0x93e, 0x91a, 0x94b, 0x907, 0x92c,
0x932, 0x938, 0x947, 0x928, 0x915, 0x94b, 0x932, 0x92e, 0x94d, 0x92c, 0x94b, 0x926,
0x92e, 0x93f, 0x936, 0x94d, 0x915, 0x939, 0x948, 0x92c, 0x94d, 0x930, 0x93e, 0x928,
0x939, 0x93e, 0x902, 0x917, 0x20, 0x915, 0x93e, 0x902, 0x917, 0x939, 0x94b, 0x92c,
0x94d, 0x926, 0x907, 0x930, 0x915, 0x941, 0x938, 0x94d, 0x915, 0x91c, 0x947, 0x930,
0x942, 0x938, 0x947, 0x932, 0x92e, 0x915, 0x92e, 0x91a, 0x91f, 0x915, 0x93e, 0x916,
0x93e, 0x902, 0x927, 0x94d, 0x92f, 0x917, 0x93e, 0x915, 0x94d, 0x930, 0x947, 0x938,
0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x915, 0x941,
0x906, 0x932, 0x93e, 0x932, 0x92e, 0x94d, 0x92a, 0x941, 0x930, 0x915, 0x941, 0x91a,
0x93f, 0x902, 0x917, 0x92e, 0x915, 0x93e, 0x938, 0x93e, 0x930, 0x928, 0x93f, 0x915,
0x94b, 0x936, 0x93f, 0x92f, 0x93e, 0x915, 0x94b, 0x92c, 0x94b, 0x915, 0x941, 0x91c,
0x928, 0x947, 0x91f, 0x94d, 0x938, 0x928, 0x94b, 0x92c, 0x94b, 0x938, 0x93f, 0x92c,
0x93f, 0x930, 0x938, 0x94d, 0x915, 0x913, 0x92e, 0x938, 0x94d, 0x915, 0x913, 0x930,
0x93e, 0x932, 0x92a, 0x94b, 0x902, 0x91f, 0x93f, 0x92f, 0x93e, 0x902, 0x915, 0x915,
0x94b, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947, 0x915, 0x93f, 0x91c, 0x93f, 0x932,
0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x938, 0x93e, 0x915, 0x932, 0x93f, 0x928, 0x938,
0x93f, 0x92f, 0x94b, 0x932, 0x936, 0x94d, 0x930, 0x947, 0x921, 0x928, 0x947, 0x915,
0x94b, 0x932, 0x93f, 0x92e, 0x93f, 0x938, 0x94d, 0x915, 0x924, 0x93e, 0x907, 0x92a,
0x947, 0x908, 0x91f, 0x92c, 0x93f, 0x932, 0x93f, 0x938, 0x93f, 0x925, 0x93f, 0x92e,
0x94d, 0x92a, 0x942, 0x91f, 0x94b, 0x92e, 0x94d, 0x938, 0x909, 0x932, 0x928, 0x92c,
0x91f, 0x94b, 0x930, 0x909, 0x930, 0x942, 0x92e, 0x915, 0x940, 0x909, 0x938, 0x94d,
0x924, 0x2d, 0x928, 0x947, 0x930, 0x93e, 0x92c, 0x93f, 0x92f, 0x902, 0x91f, 0x93f,
0x92f, 0x93e, 0x928, 0x92c, 0x94d, 0x932, 0x93e, 0x921, 0x93f, 0x92c, 0x94b, 0x938,
0x94d, 0x924, 0x94b, 0x915, 0x92f, 0x941, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d,
0x915, 0x92f, 0x93e, 0x902, 0x917, 0x941, 0x928, 0x92f, 0x948, 0x915, 0x947, 0x91f,
0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x92f, 0x948, 0x930, 0x947,
0x92c, 0x928, 0x90f, 0x91c, 0x94b, 0x930, 0x94d, 0x938, 0x915, 0x947, 0x92a, 0x20,
0x92c, 0x930, 0x94d, 0x921, 0x92b, 0x930, 0x93e, 0x913, 0x92e, 0x926, 0x948, 0x930,
0x93e, 0x930, 0x93f, 0x915, 0x91c, 0x947, 0x935, 0x93f, 0x915, 0x926, 0x915, 0x94d,
0x937, 0x93f, 0x923, 0x20, 0x91c, 0x93e, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e,
0x92c, 0x94d, 0x930, 0x93f, 0x938, 0x92c, 0x947, 0x928, 0x92c, 0x94d, 0x930, 0x94b,
0x915, 0x947, 0x928, 0x20, 0x939, 0x93f, 0x932, 0x921, 0x93e, 0x930, 0x94d, 0x92c,
0x93f, 0x928, 0x907, 0x92f, 0x942, 0x915, 0x932, 0x93e, 0x939, 0x94b, 0x92c, 0x930,
0x94d, 0x91f, 0x932, 0x93f, 0x902, 0x921, 0x92e, 0x948, 0x928, 0x932, 0x93e, 0x930,
0x94d, 0x921, 0x20, 0x939, 0x94b, 0x92c, 0x947, 0x90f, 0x92e, 0x94d, 0x938, 0x91f,
0x930, 0x94d, 0x921, 0x92e, 0x90f, 0x902, 0x921, 0x94b, 0x930, 0x93e, 0x905, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x93e, 0x915, 0x93e, 0x928, 0x92c, 0x93e, 0x930, 0x93f,
0x924, 0x938, 0x94d, 0x932, 0x93e, 0x935, 0x93e, 0x92c, 0x94d, 0x930, 0x938, 0x947,
0x932, 0x94d, 0x938, 0x92c, 0x938, 0x93f, 0x902, 0x917, 0x947, 0x928, 0x91a, 0x93f,
0x938, 0x93f, 0x928, 0x93e, 0x90a, 0x915, 0x94b, 0x92a, 0x947, 0x928, 0x939, 0x947,
0x917, 0x947, 0x928, 0x917, 0x930, 0x94d, 0x928, 0x91c, 0x940, 0x939, 0x948, 0x932,
0x938, 0x93f, 0x902, 0x915, 0x940, 0x906, 0x907, 0x91c, 0x932, 0x20, 0x906, 0x92b,
0x20, 0x92e, 0x948, 0x928, 0x907, 0x938, 0x94d, 0x924, 0x93e, 0x92e, 0x94d, 0x92c,
0x941, 0x932, 0x915, 0x932, 0x93f, 0x928, 0x93f, 0x928, 0x917, 0x94d, 0x930, 0x947,
0x921, 0x932, 0x93f, 0x938, 0x92c, 0x928, 0x932, 0x941, 0x92c, 0x932, 0x93f, 0x92f,
0x93e, 0x928, 0x93e, 0x932, 0x917, 0x94d, 0x91c, 0x92e, 0x92c, 0x930, 0x94d, 0x917,
0x92e, 0x947, 0x921, 0x94d, 0x930, 0x93f, 0x921, 0x92e, 0x93e, 0x932, 0x91f, 0x93e,
0x92e, 0x947, 0x930, 0x93f, 0x92f, 0x947, 0x939, 0x92e, 0x92e, 0x93f, 0x928, 0x94d,
0x938, 0x915, 0x92e, 0x94b, 0x928, 0x947, 0x915, 0x94b, 0x92e, 0x93e, 0x938, 0x94d,
0x915, 0x94b, 0x92a, 0x94b, 0x921, 0x917, 0x94b, 0x930, 0x93f, 0x936, 0x93e, 0x938,
0x948, 0x928, 0x20, 0x92e, 0x930, 0x940, 0x928, 0x94b, 0x938, 0x930, 0x93e, 0x91c,
0x947, 0x92c, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x924, 0x94b, 0x92c, 0x938, 0x93f,
0x92e, 0x94d, 0x92b, 0x947, 0x930, 0x94b, 0x92a, 0x94b, 0x932, 0x938, 0x94d, 0x915,
0x94b, 0x92a, 0x92f, 0x947, 0x938, 0x94d, 0x91f, 0x93e, 0x915, 0x939, 0x94b, 0x92e,
0x91f, 0x93e, 0x932, 0x93f, 0x928, 0x91f, 0x93f, 0x930, 0x93e, 0x928, 0x947, 0x909,
0x932, 0x94d, 0x92f, 0x947, 0x928, 0x94b, 0x92c, 0x94b, 0x938, 0x94d, 0x915, 0x92c,
0x947, 0x926, 0x941, 0x91c, 0x92c, 0x947, 0x91f, 0x93f, 0x915, 0x928, 0x92c, 0x93f,
0x90f, 0x928, 0x93e, 0x92c, 0x93f, 0x932, 0x928, 0x93f, 0x909, 0x938, 0x92c, 0x94b,
0x932, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x92c, 0x93e, 0x930, 0x938,
0x93e, 0x91c, 0x917, 0x930, 0x948, 0x92c, 0x91c, 0x94d, 0x92f, 0x942, 0x930, 0x93f,
0x916, 0x905, 0x902, 0x924, 0x93e, 0x928, 0x93e, 0x928, 0x947, 0x930, 0x93f, 0x935,
0x94b, 0x91a, 0x917, 0x94b, 0x938, 0x915, 0x948, 0x930, 0x917, 0x941, 0x90f, 0x932,
0x947, 0x928, 0x92e, 0x93e, 0x930, 0x940, 0x936, 0x938, 0x92e, 0x92f, 0x94b, 0x924,
0x947, 0x930, 0x93f, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x928, 0x906, 0x915, 0x932,
0x948, 0x902, 0x921, 0x91a, 0x947, 0x925, 0x92e, 0x917, 0x948, 0x932, 0x93e, 0x92a,
0x948, 0x917, 0x94b, 0x938, 0x928, 0x93e, 0x930, 0x92b, 0x93e, 0x915, 0xd05, 0xd2c,
0xd3f, 0xd26, 0xd4d, 0x200c, 0xd1c, 0xd3e, 0xd7b, 0x200c, 0xd06, 0xd15, 0xd4d, 0xd30,
0xd05, 0xd21, 0xd3f, 0xd38, 0xd4d, 0x20, 0xd05, 0xd2c, 0xd3e, 0xd2c, 0xd05, 0xd7e,
0xd1c, 0xd3f, 0xd2f, 0xd47, 0xd34, 0xd4d, 0x200c, 0xd38, 0xd4d, 0xd05, 0xd38, 0xd4d,
0x200d, 0xd2e, 0xd3e, 0xd30, 0xd2c, 0xd2e, 0xd3e, 0xd15, 0xd4b, 0xd2c, 0xd02, 0xd17,
0xd41, 0xd2f, 0xd3f, 0xd2c, 0xd1e, 0xd4d, 0xd1a, 0xd41, 0xd7d, 0xd2c, 0xd3f, 0xd38,
0xd4d, 0xd38, 0xd3e, 0xd35, 0xd41, 0xd2c, 0xd4d, 0xd32, 0xd3e, 0xd23, 0xd4d, 0xd1f,
0xd46, 0xd2f, 0xd7c, 0x200c, 0xd2c, 0xd4d, 0xd30, 0xd3e, 0xd38, 0xd35, 0xd3f, 0xd7d,
0xd2c, 0xd41, 0xd1c, 0xd41, 0xd02, 0x200c, 0xd2c, 0xd41, 0xd30, 0xd15, 0xd46, 0xd2f,
0xd4d, 0x200c, 0xd31, 0xd4b, 0xd15, 0xd3e, 0xd38, 0xd2c, 0xd4d, 0xd32, 0xd3e, 0xd19,
0xd4d, 0xd15, 0xd15, 0xd4d, 0xd2f, 0xd42, 0xd1f, 0xd4d, 0xd1f, 0xd15, 0xd4a, 0xd23,
0xd3e, 0xd15, 0xd4d, 0xd30, 0xd3f, 0xd21, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd7c, 0x200c,
0xd26, 0xd3e, 0xd7c, 0x20, 0xd0e, 0xd38, 0xd4d, 0x20, 0xd38, 0xd32, 0xd3e, 0xd02,
0xd26, 0xd3f, 0xd1c, 0xd3f, 0xd2c, 0xd57, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd21, 0xd57,
0xd32, 0xd0e, 0xd7d, 0x200c, 0x20, 0xd10, 0xd2f, 0xd41, 0xd7b, 0x200c, 0xd2b, 0xd4d,
0xd30, 0xd40, 0xd1f, 0xd57, 0xd7a, 0xd17, 0xd3e, 0xd2c, 0xd31, 0xd4b, 0xd7a, 0xd39,
0xd30, 0xd3e, 0xd30, 0xd46, 0xd1c, 0xd4b, 0xd39, 0xd28, 0xd4d, 0xd28, 0xd3e, 0xd38,
0xd4d, 0xd2c, 0xd7c, 0x200c, 0xd17, 0xd4d, 0xd1c, 0xd41, 0xd2c, 0xd15, 0xd2e, 0xd4d,
0xd2a, 0xd3e, 0xd32, 0xd16, 0xd3e, 0xd7c, 0x200c, 0xd24, 0xd4c, 0xd02, 0xd15, 0xd3f,
0xd17, 0xd3e, 0xd32, 0xd3f, 0xd15, 0xd3f, 0xd7b, 0xd37, 0xd3e, 0xd38, 0xd32, 0xd3e,
0xd17, 0xd4b, 0xd38, 0xd4d, 0xd32, 0xd3f, 0xd2c, 0xd4d, 0xd30, 0xd46, 0xd35, 0xd3f,
0xd32, 0xd4d, 0xd32, 0xd32, 0xd4b, 0xd02, 0xd32, 0xd41, 0xd35, 0xd3e, 0xd7b, 0xd21,
0xd32, 0xd42, 0xd2c, 0xd41, 0xd02, 0xd2c, 0xd3e, 0xd37, 0xd3f, 0xd32, 0xd41, 0xd38,
0xd3e, 0xd15, 0xd2e, 0xd32, 0xd3e, 0xd2c, 0xd4b, 0xd2e, 0xd3e, 0xd2a, 0xd4d, 0xd2f,
0xd41, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0xd2e, 0xd38, 0xd47, 0xd31, 0xd41, 0xd2e, 0xd2c,
0xd3e, 0xd2c, 0xd46, 0xd2f, 0xd4d, 0x200c, 0xd7b, 0x200c, 0xd2e, 0xd4a, 0xd17, 0xd3e,
0xd26, 0xd3f, 0xd37, 0xd41, 0xd2e, 0xd7a, 0x200c, 0xd31, 0xd4b, 0xd35, 0xd3f, 0xd2f,
0xd28, 0xd2f, 0xd4d, 0x200c, 0xd31, 0xd4b, 0xd2c, 0xd3f, 0xd1c, 0xd2e, 0xd46, 0xd28,
0xd28, 0xd3f, 0xd2f, 0xd3e, 0xd2e, 0xd3f, 0xd28, 0xd57, 0xd35, 0xd3e, 0xd15, 0xd4d,
0x200c, 0xd37, 0xd4b, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0xd14, 0xd17, 0xd3e, 0xd26, 0xd57,
0xd17, 0xd57, 0xd2a, 0xd4b, 0xd7c, 0x200c, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0x2d, 0xd28,
0xd4b, 0xd35, 0xd4b, 0xd38, 0xd3e, 0xd35, 0xd4b, 0x20, 0xd1f, 0xd4b, 0xd02, 0x200c,
0xd1f, 0xd4d, 0xd30, 0xd3f, 0xd2a, 0xd4a, 0xd33, 0xd3f, 0xd1f, 0xd4d, 0xd2f, 0xd42,
0xd23, 0xd3f, 0xd38, 0xd4d, 0xd35, 0xd3f, 0xd7b, 0xd21, 0xd4d, 0x200c, 0xd39, 0xd4b,
0xd15, 0xd4d, 0xd05, 0xd21, 0xd3e, 0xd15, 0xd4d, 0xd06, 0xd19, 0xd4d, 0xd15, 0xd31,
0xd47, 0xd1c, 0xd4d, 0xd06, 0xd7b, 0xd17, 0xd4d, 0xd35, 0xd3f, 0xd32, 0xd4d, 0xd32,
0xd06, 0xd7b, 0xd31, 0xd3f, 0xd17, 0xd4d, 0xd35, 0xd05, 0xd31, 0xd17, 0xd4d, 0xd35,
0xd48, 0xd28, 0xd2c, 0xd4d, 0xd2f, 0xd42, 0xd23, 0xd38, 0xd4d, 0x20, 0xd10, 0xd31,
0xd3f, 0xd38, 0xd4d, 0x200d, 0xd15, 0xd4d, 0xd2f, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd2e,
0xd3e, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd15, 0xd4b, 0xd7c, 0xd21, 0xd4b, 0xd2c, 0xd1c,
0xd41, 0xd1c, 0xd41, 0xd2f, 0xd4d, 0xd32, 0xd3e, 0x20, 0xd31, 0xd3f, 0xd2f, 0xd4b,
0xd1c, 0xd2e, 0xd46, 0xd7b, 0xd21, 0xd4b, 0xd38, 0xd31, 0xd3f, 0xd2f, 0xd4b, 0x20,
0xd17, 0xd4d, 0xd2f, 0xd3e, 0xd32, 0xd17, 0xd4b, 0xd38, 0xd4d, 0xd38, 0xd3e, 0xd7d,
0xd1f, 0xd4d, 0xd1f, 0xd38, 0xd3e, 0xd7b, 0x20, 0xd1c, 0xd41, 0xd35, 0xd3e, 0xd7b,
0xd38, 0xd3e, 0xd7b, 0x20, 0xd32, 0xd42, 0xd2f, 0xd3f, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd41, 0xd15, 0xd41, 0xd2e, 0xd3e, 0xd7b, 0xd09, 0xd37, 0xd3f, 0xd2f, 0xd05,
0xd31, 0xd42, 0xd2c, 0xd05, 0xd38, 0xd7b, 0x200c, 0xd37, 0xd4d, 0xd2f, 0xd7b, 0x200c,
0xd0f, 0xd31, 0xd4d, 0xd31, 0xd3f, 0xd15, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd7a, 0xd2c,
0xd39, 0xd3f, 0xd2f, 0xd2c, 0xd39, 0xd3f, 0xd2f, 0x20, 0xd2c, 0xd7b, 0xd21, 0xd3e,
0xd30, 0xd38, 0xd4d, 0xd2c, 0xd3e, 0xd7c, 0xd2c, 0xd21, 0xd4b, 0xd38, 0xd4d, 0xd2c,
0xd46, 0xd32, 0xd02, 0xd2c, 0xd46, 0xd32, 0xd40, 0xd38, 0xd4d, 0xd2c, 0xd4d, 0xd32,
0xd3e, 0xd19, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd3e, 0xd2c, 0xd4d, 0xd32, 0xd4b,
0xd7a, 0xd2c, 0xd4b, 0xd35, 0x20, 0xd35, 0xd3f, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd2c, 0xd4a, 0xd17, 0xd4b, 0xd1f, 0xd4d, 0xd1f, 0xd2c, 0xd4a, 0xd2f, 0xd4d, 0xd38,
0xd3f, 0xd15, 0xd47, 0xd02, 0xd2c, 0xd4d, 0xd30, 0xd3f, 0xd21, 0xd4d, 0x200c, 0xd1c,
0xd4d, 0x20, 0xd2c, 0xd47, 0xd15, 0xd4d, 0xd2f, 0xd3e, 0xd2e, 0xd4d, 0xd2a, 0xd4b,
0x20, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd7b, 0xd21, 0xd46, 0xd15, 0xd3e, 0xd7b, 0xd15,
0xd42, 0xd7a, 0xd15, 0xd30, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd38, 0xd4d, 0xd15, 0xd2f,
0xd40, 0xd7b, 0x200c, 0xd15, 0xd47, 0xd2e, 0xd3e, 0xd7b, 0xd1a, 0xd3f, 0xd15, 0xd4d,
0xd15, 0xd3e, 0xd17, 0xd4b, 0xd1a, 0xd3f, 0xd39, 0xd4d, 0xd35, 0xd3e, 0xd39, 0xd38,
0xd3f, 0xd2f, 0xd41, 0xd21, 0xd3e, 0xd21, 0xd4d, 0x20, 0xd39, 0xd41, 0xd35, 0xd3e,
0xd30, 0xd38, 0xd4d, 0xd15, 0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0x20, 0xd31,
0xd3f, 0xd15, 0xd4d, 0xd15, 0xd15, 0xd4d, 0xd30, 0xd46, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd7a, 0xd15, 0xd41, 0xd2f, 0xd4d, 0x200c, 0xd2c, 0xd15, 0xd41, 0xd31, 0xd3e,
0xd15, 0xd4d, 0xd15, 0xd3e, 0xd35, 0xd4b, 0xd21, 0xd3e, 0xd7b, 0xd2e, 0xd3e, 0xd7c,
0xd15, 0xd4d, 0xd15, 0xd4d, 0xd37, 0xd3e, 0xd35, 0xd4d, 0xd7b, 0xd21, 0xd4b, 0xd35,
0xd4d, 0xd38, 0xd7a, 0xd21, 0xd4b, 0xd35, 0xd4d, 0xd38, 0xd7a, 0x20, 0xd15, 0xd4d,
0xd30, 0xd40, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd21, 0xd46, 0xd7b, 0x200c, 0xd35, 0xd7c,
0xd21, 0xd46, 0xd1f, 0xd4d, 0xd30, 0xd4b, 0xd2f, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd4d,
0xd21, 0xd4a, 0xd2e, 0xd3f, 0xd28, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd0e, 0xd21, 0xd4d,
0xd2e, 0xd4b, 0xd7a, 0xd1f, 0xd7a, 0xd2f, 0xd46, 0xd31, 0xd41, 0xd28, 0xd40, 0xd2a,
0xd4d, 0xd2a, 0xd46, 0xd0e, 0xd7d, 0x20, 0xd38, 0xd3e, 0xd7d, 0x200c, 0xd35, 0xd26,
0xd4b, 0xd7c, 0xd2b, 0xd4b, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0x20, 0xd28, 0xd46,
0xd7d, 0xd38, 0xd7a, 0xd2b, 0xd4b, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd32, 0xd40, 0xd38,
0xd17, 0xd4d, 0xd32, 0xd46, 0xd2f, 0xd4d, 0xd38, 0xd4d, 0x20, 0xd2c, 0xd47, 0xd17,
0xd42, 0xd38, 0xd4d, 0x20, 0xd2c, 0xd47, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd7b, 0xd21,
0xd4d, 0x20, 0xd1f, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd17, 0xd4d, 0xd30, 0xd28,
0xd47, 0xd21, 0xd17, 0xd4d, 0xd35, 0xd3e, 0xd21, 0xd32, 0xd42, 0xd2a, 0xd4d, 0xd2a,
0xd4d, 0xd17, 0xd4d, 0xd35, 0xd3e, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd2e, 0xd3e, 0xd32,
0xd17, 0xd41, 0xd35, 0xd3e, 0xd2f, 0xd15, 0xd4d, 0xd35, 0xd3f, 0xd7d, 0xd17, 0xd2f,
0xd3e, 0xd28, 0xd39, 0xd3e, 0xd32, 0xd3f, 0xd2b, 0xd3e, 0xd15, 0xd4d, 0xd38, 0xd4d,
0xd39, 0xd35, 0xd3e, 0xd28, 0xd39, 0xd46, 0xd7c, 0xd2e, 0xd4b, 0xd38, 0xd3f, 0xd32,
0xd4d, 0xd32, 0xd4b, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd3e, 0xd2a,
0xd4b, 0xd32, 0xd40, 0xd38, 0xd4d, 0xd28, 0xd4b, 0xd15, 0xd4d, 0xd38, 0xd4d, 0x2c,
0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd2e, 0xd30, 0xd46, 0xd19,
0xd4d, 0xd15, 0xd4b, 0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28,
0xd2a, 0xd40, 0xd31, 0xd4d, 0xd31, 0xd47, 0xd34, 0xd4d, 0xd38, 0xd4d, 0xd2c, 0xd7c,
0xd17, 0xd4d, 0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd31,
0xd4d, 0xd31, 0xd46, 0xd7d, 0x20, 0xd38, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd3f, 0x2c,
0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd35, 0xd3f, 0xd35, 0xd46,
0xd2f, 0xd4d, 0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd35,
0xd3f, 0xd7b, 0xd38, 0xd46, 0xd7b, 0xd38, 0xd4d, 0x2c, 0x20, 0xd07, 0xd7b, 0xd21,
0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd35, 0xd3f, 0xd28, 0xd3e, 0xd2e, 0xd3e, 0xd15, 0xd4d,
0x2c, 0x20, 0xd07, 0xd7b, 0xd21, 0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd07, 0xd28, 0xd41,
0xd35, 0xd3f, 0xd15, 0xd4d, 0xd07, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd24, 0xd4d, 0xd1c,
0xd2e, 0xd48, 0xd15, 0xd4d, 0xd15, 0xd1c, 0xd42, 0xd28, 0xd4b, 0xd32, 0xd42, 0xd2f,
0xd3f, 0xd38, 0xd4d, 0x200c, 0xd35, 0xd3f, 0xd32, 0xd4d, 0xd32, 0xd46, 0xd2e, 0xd4b,
0xd23, 0xd4d, 0xd1f, 0xd3f, 0xd38, 0xd46, 0xd32, 0xd4d, 0xd32, 0xd4b, 0x2c, 0x20,
0xd15, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd15, 0xd3e, 0xd7c,
0xd32, 0xd7b, 0xd21, 0xd3f, 0xd1c, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd32, 0xd3e,
0x20, 0xd2a, 0xd3e, 0xd38, 0xd4d, 0xd32, 0xd3f, 0xd2e, 0xd32, 0xd4b, 0xd38, 0xd4d,
0x20, 0xd0e, 0xd2f, 0xd4d, 0xd1e, 0xd4d, 0xd1a, 0xd32, 0xd38, 0xd4d, 0xd32, 0xd4b,
0xd35, 0xd7c, 0x20, 0xd2a, 0xd4d, 0xd30, 0xd3f, 0xd7b, 0xd38, 0xd38, 0xd4d, 0x20,
0xd15, 0xd4d, 0xd35, 0xd3e, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd7c, 0xd2e, 0xd3e, 0xd38,
0xd3f, 0xd2f, 0xd4b, 0xd2e, 0xd28, 0xd3e, 0xd17, 0xd4d, 0xd35, 0xd2e, 0xd28, 0xd57,
0xd38, 0xd4d, 0xd2e, 0xd3e, 0xd30, 0xd3f, 0xd17, 0xd4b, 0xd2e, 0xd3e, 0xd7c, 0xd1f,
0xd4d, 0xd1f, 0xd3f, 0xd28, 0xd3f, 0xd15, 0xd4d, 0xd2e, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd2e, 0xd4a, 0xd31, 0xd4b, 0xd38, 0xd4d, 0xd2e, 0xd38, 0xd31, 0xd4d, 0xd31, 0xd4d,
0x200c, 0xd32, 0xd3e, 0xd7b, 0xd2e, 0xd46, 0xd28, 0xd4b, 0xd2e, 0xd3f, 0xd28, 0xd40,
0xd2e, 0xd46, 0xd30, 0xd3f, 0xd21, 0xd2e, 0xd46, 0xd21, 0xd4d, 0x200c, 0xd32, 0xd15,
0xd1f, 0xd4d, 0x200c, 0xd32, 0xd46, 0xd2e, 0xd46, 0xd15, 0xd4d, 0xd38, 0xd3f, 0xd15,
0xd4d, 0xd15, 0xd4b, 0x20, 0xd38, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd3f, 0xd2e, 0xd3f,
0xd15, 0xd4d, 0xd35, 0xd32, 0xd7b, 0xd2e, 0xd4b, 0xd02, 0xd17, 0xd4d, 0x200c, 0xd1f,
0xd7b, 0xd2e, 0xd4b, 0xd23, 0xd4d, 0xd1f, 0xd46, 0xd31, 0xd3f, 0xd2e, 0xd4a, 0xd23,
0xd4d, 0xd1f, 0xd46, 0x200c, 0xd35, 0xd40, 0xd21, 0xd3f, 0xd2f, 0xd4b, 0xd2e, 0xd4a,
0xd23, 0xd4d, 0xd1f, 0xd46, 0xd38, 0xd30, 0xd24, 0xd4d, 0xd24, 0xd4d, 0xd28, 0xd3e,
0xd38, 0xd57, 0xd28, 0xd4d, 0xd2f, 0xd42, 0xd2f, 0xd4b, 0xd7c, 0xd15, 0xd4d, 0xd15,
0xd4d, 0xd28, 0xd4b, 0xd02, 0xd28, 0xd4a, 0xd31, 0xd4b, 0xd28, 0xd2c, 0xd4d, 0xd2f,
0xd42, 0xd32, 0x2c, 0x20, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd21,
0xd46, 0xd15, 0xd4d, 0xd15, 0xd4b, 0xd1f, 0xd4d, 0xd1f, 0xd38, 0xd46, 0xd28, 0xd4d,
0xd31, 0xd7c, 0x2c, 0x20, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd21,
0xd46, 0xd15, 0xd4d, 0xd15, 0xd4b, 0xd1f, 0xd4d, 0xd1f, 0xd28, 0xd4d, 0xd2f, 0xd42,
0x20, 0xd38, 0xd46, 0xd2f, 0xd4d, 0x200c, 0xd32, 0xd02, 0x2c, 0x20, 0xd35, 0xd1f,
0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd21, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd4b, 0xd1f,
0xd4d, 0xd1f, 0xd28, 0xd42, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd12, 0xd1c, 0xd3f, 0xd7b,
0xd17, 0xd2a, 0xd28, 0xd3e, 0xd2e, 0xd2a, 0xd30, 0xd3e, 0xd2e, 0xd3e, 0xd30, 0xd3f,
0xd2c, 0xd4b, 0xd2b, 0xd40, 0xd28, 0xd3f, 0xd15, 0xd4d, 0xd38, 0xd4d, 0xd2a, 0xd4b,
0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0xd2a, 0xd4d, 0xd30, 0xd3f, 0xd7b, 0xd38, 0xd4d,
0xd2a, 0xd4b, 0xd7c, 0x200c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0x20, 0xd13, 0xd2b, 0xd4d,
0x20, 0xd38, 0xd4d, 0x200c, 0xd2a, 0xd46, 0xd2f, 0xd3f, 0xd7b, 0x200c, 0xd2a, 0xd4b,
0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0x20, 0xd35, 0xd46, 0xd32, 0xd4d, 0xd32, 0xd4b,
0xd2a, 0xd4d, 0xd2f, 0xd42, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0x20, 0xd31, 0xd3f,
0xd15, 0xd4d, 0xd15, 0xd4b, 0xd2a, 0xd41, 0xd28, 0xd4d, 0xd31, 0x20, 0xd05, 0xd30,
0xd40, 0xd28, 0xd38, 0xd4d, 0xd31, 0xd3e, 0xd19, 0xd4d, 0xd15, 0xd3f, 0xd7b, 0x20,
0xd07, 0xd7b, 0xd32, 0xd46, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd46, 0xd38, 0xd40,
0xd2b, 0xd46, 0xd31, 0xd3f, 0xd1c, 0xd40, 0xd28, 0xd31, 0xd46, 0xd38, 0xd32, 0xd4d,
0xd2f, 0xd42, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0xd31, 0xd3f, 0xd2f, 0xd4b, 0x20, 0xd2c,
0xd4d, 0xd30, 0xd3e, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd38, 0xd3e, 0xd28, 0xd4d, 0xd31,
0xd31, 0xd46, 0xd02, 0xd38, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd3f, 0xd2f, 0xd3e, 0xd17,
0xd4b, 0xd38, 0xd3e, 0xd28, 0xd4d, 0xd24, 0xd4b, 0x20, 0xd21, 0xd4b, 0xd2e, 0xd3f,
0xd02, 0xd17, 0xd4b, 0xd38, 0xd3e, 0xd35, 0xd4b, 0xd2a, 0xd4b, 0xd33, 0xd4b, 0xd07,
0xd31, 0xd4d, 0xd31, 0xd4d, 0xd35, 0xd3e, 0xd16, 0xd4d, 0xd35, 0xd3e, 0xd7c, 0xd1f,
0xd42, 0xd7c, 0xd2e, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd38, 0xd3f, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x200c, 0xd15, 0xd3e, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20,
0xd2c, 0xd7c, 0xd24, 0xd4d, 0xd24, 0xd32, 0xd46, 0xd2e, 0xd3f, 0xd38, 0xd46, 0xd28,
0xd4d, 0xd31, 0xd4d, 0x20, 0xd1c, 0xd4b, 0xd7a, 0xd38, 0xd4d, 0xd38, 0xd46, 0xd28,
0xd4d, 0xd31, 0xd4d, 0x20, 0xd15, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd38, 0xd4d,
0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd32, 0xd42, 0xd38, 0xd3f, 0xd2f,
0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd24, 0xd4b, 0xd2e, 0xd38, 0xd4d,
0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd35, 0xd3f, 0xd7b, 0xd38, 0xd46,
0xd28, 0xd4d, 0xd31, 0xd4d, 0xd38, 0xd4d, 0xd35, 0xd3f, 0xd2b, 0xd4d, 0x200c, 0xd31,
0xd4d, 0xd31, 0xd4d, 0x20, 0xd15, 0xd31, 0xd28, 0xd4d, 0xd31, 0xd4d, 0xd1f, 0xd46,
0xd17, 0xd42, 0xd38, 0xd3f, 0xd17, 0xd3e, 0xd7d, 0xd2a, 0xd24, 0xd42, 0xd33, 0xd3f,
0xd24, 0xd3f, 0xd2f, 0xd41, 0xd35, 0xd3e, 0xd28, 0xd1f, 0xd4a, 0xd31, 0xd28, 0xd4d,
0xd31, 0xd4b, 0xd1f, 0xd4b, 0xd7c, 0x200c, 0xd1f, 0xd4d, 0xd1f, 0xd4b, 0xd33, 0xd35,
0xd3e, 0xd7b, 0x200c, 0xd15, 0xd42, 0xd35, 0xd7c, 0xd35, 0xd48, 0xd31, 0xd4d, 0xd31,
0xd4d, 0xd39, 0xd4b, 0xd34, 0xd4d, 0xd38, 0xd4d, 0xd35, 0xd3f, 0xd28, 0xd4d, 0xd28,
0xd3f, 0xd2a, 0xd46, 0xd17, 0xd4d, 0xd2f, 0xd3e, 0xd15, 0xd41, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd21, 0xd46, 0xd2f, 0xd4d, 0x200c, 0xd35, 0xd3f,
0xd38, 0xd4d, 0xd21, 0xd4d, 0xd2f, 0xd42, 0xd2e, 0xd23, 0xd4d, 0xd1f, 0xd4d, 0x20,
0xd21, 0xd3f, 0x20, 0xd2f, 0xd41, 0xd7c, 0xd35, 0xd3f, 0xd7d, 0xd2e, 0xd15, 0xd4d,
0xd35, 0xd2f, 0xd31, 0xd3f, 0xd2e, 0xd3e, 0xd35, 0xd4d, 0xd38, 0xd7a, 0xd2e, 0xd3e,
0xd15, 0xd4d, 0xd2e, 0xd7c, 0xd21, 0xd4b, 0xd2a, 0xd3e, 0xd2e, 0xd7c, 0xd31, 0xd4a,
0xd24, 0xd40, 0xd31, 0xd38, 0xd4d, 0xd2f, 0xd4b, 0xd35, 0xd1f, 0xd4d, 0xd30, 0xd4b,
0xd7e, 0xd35, 0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd4b, 0xd15, 0xd4d, 0xd32,
0xd02, 0xd17, 0xd4d, 0x200d, 0xd2f, 0xd46, 0xd7c, 0xd2c, 0xd3f, 0xd7b, 0xd0f, 0xd26,
0xd46, 0xd7b, 0xd05, 0xd7d, 0xd2e, 0xd3e, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd05, 0xd2e,
0xd4d, 0xd2e, 0xd3e, 0xd7b, 0x200c, 0xd05, 0xd28, 0xd3e, 0xd21, 0xd3f, 0xd7c, 0xd05,
0xd15, 0xd4d, 0xd24, 0xd57, 0xd05, 0xd16, 0xd4d, 0x200c, 0xd24, 0xd4b, 0xd2c, 0xd4d,
0xd06, 0xd37, 0xd4d, 0x200c, 0xd17, 0xd3e, 0xd2c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0xd05,
0xd31, 0xd4d, 0xd31, 0xd3f, 0xd31, 0xd4b, 0xd2c, 0xd3e, 0xd17, 0xd4d, 0x200c, 0xd26,
0xd3e, 0xd26, 0xd4d, 0xd2c, 0xd39, 0xd4d, 0xd31, 0xd3f, 0xd7b, 0xd2c, 0xd3e, 0xd15,
0xd4d, 0xd15, 0xd41, 0xd2c, 0xd3e, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd15, 0xd4d, 0xd15,
0xd4d, 0xd2c, 0xd7c, 0xd23, 0xd4b, 0xd7d, 0xd2c, 0xd46, 0xd2f, 0xd4d, 0x200c, 0xd31,
0xd42, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0xd2c, 0xd3f, 0xd37, 0xd4d, 0x200c, 0xd15, 0xd47,
0xd15, 0xd4d, 0xd2c, 0xd4d, 0xd30, 0xd42, 0xd23, 0xd48, 0xd1a, 0xd40, 0xd31, 0xd4d,
0xd31, 0xd1a, 0xd4a, 0xd2f, 0xd4d, 0xd2c, 0xd7d, 0xd38, 0xd7b, 0xd15, 0xd4a, 0xd33,
0xd02, 0x200c, 0xd2c, 0xd4b, 0xd26, 0xd2e, 0xd3e, 0xd38, 0xd4d, 0xd15, 0xd38, 0xd4d,
0xd27, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd26, 0xd3f, 0xd32, 0xd3f, 0xd26, 0xd41, 0xd2c,
0xd3e, 0xd2f, 0xd4d, 0xd26, 0xd41, 0xd37, 0xd7b, 0x200c, 0xd2c, 0xd46, 0xd2b, 0xd3e,
0xd2e, 0xd17, 0xd41, 0xd38, 0xd4d, 0x200c, 0xd31, 0xd4d, 0xd31, 0xd17, 0xd3e, 0xd38,
0xd39, 0xd46, 0xd2c, 0xd4d, 0x200c, 0xd31, 0xd4b, 0xd7a, 0xd39, 0xd4b, 0x20, 0xd1a,
0xd3f, 0x20, 0xd2e, 0xd3f, 0xd7b, 0x20, 0xd38, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd3f,
0xd39, 0xd4b, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd02, 0xd17, 0xd4d, 0xd39, 0xd4b, 0xd21,
0xd4d, 0xd07, 0xd7c, 0xd15, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd1c, 0xd15, 0xd4d, 0xd15,
0xd3e, 0xd7c, 0xd24, 0xd4d, 0xd24, 0xd1c, 0xd2f, 0xd2a, 0xd41, 0xd30, 0xd1c, 0xd46,
0xd31, 0xd41, 0xd38, 0xd32, 0xd47, 0xd02, 0xd15, 0xd3e, 0xd2c, 0xd42, 0xd7e, 0xd15,
0xd3e, 0xd02, 0xd1a, 0xd1f, 0xd4d, 0xd15, 0xd15, 0xd31, 0xd3e, 0xd1a, 0xd4d, 0xd1a,
0xd3f, 0xd15, 0xd3e, 0xd20, 0xd4d, 0x200c, 0xd2e, 0xd23, 0xd4d, 0xd21, 0xd41, 0xd15,
0xd3e, 0xd7b, 0xd21, 0xd3f, 0xd17, 0xd15, 0xd4a, 0xd7d, 0x200c, 0xd15, 0xd4d, 0xd15,
0xd24, 0xd4d, 0xd24, 0xd15, 0xd4d, 0xd30, 0xd3e, 0xd38, 0xd4d, 0xd28, 0xd4b, 0xd2f,
0xd3e, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd35, 0xd3e, 0xd32, 0xd32, 0xd02,
0x200c, 0xd2a, 0xd42, 0xd7c, 0x200c, 0x200c, 0xd15, 0xd41, 0xd1a, 0xd3f, 0xd19, 0xd4d,
0xd15, 0xd41, 0xd35, 0xd48, 0xd24, 0xd4d, 0xd24, 0xd4d, 0xd2e, 0xd15, 0xd4d, 0xd15,
0xd3e, 0xd35, 0xd41, 0xd2e, 0xd17, 0xd21, 0xd3e, 0xd7b, 0xd2e, 0xd15, 0xd38, 0xd4d,
0xd38, 0xd7c, 0xd2e, 0xd28, 0xd3f, 0xd32, 0xd2e, 0xd38, 0xd4d, 0xd15, 0xd31, 0xd4d,
0xd31, 0xd4d, 0xd28, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd4b, 0xd37, 0xd4d, 0xd2f, 0xd28,
0xd4b, 0xd35, 0xd4b, 0xd15, 0xd41, 0xd38, 0xd46, 0xd7b, 0xd38, 0xd4d, 0x200c, 0xd15,
0xd4d, 0xd28, 0xd4a, 0xd35, 0xd4b, 0xd38, 0xd3f, 0xd2c, 0xd3f, 0xd7c, 0xd38, 0xd4d,
0xd15, 0xd4d, 0xd12, 0xd02, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd13, 0xd31, 0xd7d, 0xd2b,
0xd46, 0xd28, 0xd4b, 0xd02, 0x20, 0xd2a, 0xd46, 0xd7b, 0xd2a, 0xd4a, 0xd28, 0xd4d,
0xd31, 0xd3f, 0xd2f, 0xd3e, 0xd28, 0xd15, 0xd4d, 0xd2a, 0xd4d, 0xd2f, 0xd4b, 0xd02,
0xd17, 0xd4d, 0x200c, 0xd2f, 0xd3e, 0xd02, 0xd17, 0xd4d, 0xd16, 0xd24, 0xd4d, 0xd24,
0xd7c, 0xd15, 0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd28, 0xd47, 0xd16, 0xd3f,
0xd38, 0xd3f, 0xd32, 0xd4b, 0xd7c, 0xd21, 0xd31, 0xd3f, 0xd2f, 0xd3e, 0xd26, 0xd4d,
0xd38, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd7b, 0xd38, 0xd2e, 0xd7c, 0xd15, 0xd4d, 0xd15,
0xd28, 0xd4d, 0xd26, 0xd4d, 0xd38, 0xd4b, 0xd7e, 0xd37, 0xd3e, 0xd19, 0xd4d, 0x200c,
0xd39, 0xd3e, 0xd2f, 0xd3f, 0xd38, 0xd3f, 0xd02, 0xd17, 0xd2a, 0xd4d, 0xd2a, 0xd42,
0xd7c, 0xd38, 0xd4d, 0xd30, 0xd3f, 0xd21, 0xd4d, 0x200c, 0xd28, 0xd3f, 0xd15, 0xd4b,
0xd33, 0xd3f, 0xd02, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd24, 0xd3e, 0xd2f, 0xd4d, 0x200c,
0xd2a, 0xd46, 0xd2f, 0xd4d, 0xd24, 0xd3e, 0xd37, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15,
0xd28, 0xd4d, 0xd31, 0xd4d, 0xd24, 0xd3f, 0xd2c, 0xd3f, 0xd32, 0xd3f, 0xd38, 0xd3f,
0xd1f, 0xd46, 0xd39, 0xd4d, 0x200c, 0xd31, 0xd3e, 0xd7b, 0x200c, 0xd24, 0xd3f, 0xd02,
0xd2b, 0xd41, 0xd1f, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd2f, 0xd4b, 0xd1f, 0xd4b,
0xd02, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd09, 0xd32, 0xd3e, 0xd7b, 0xd2c, 0xd3e, 0xd24,
0xd4d, 0xd24, 0xd7c, 0xd09, 0xd31, 0xd41, 0xd02, 0xd16, 0xd3f, 0xd2f, 0xd41, 0xd38,
0xd4d, 0x2d, 0xd28, 0xd47, 0xd30, 0xd35, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd3f, 0xd2f,
0xd3e, 0xd7b, 0xd35, 0xd4d, 0xd33, 0xd3e, 0xd21, 0xd3f, 0xd35, 0xd4b, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd4b, 0xd15, 0xd4d, 0xd2f, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd31,
0xd4d, 0xd31, 0xd4d, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd31, 0xd19, 0xd4d, 0xd15,
0xd42, 0xd7a, 0x200c, 0xd2f, 0xd3e, 0xd15, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd46, 0xd31,
0xd3f, 0xd7b, 0xd2c, 0xd7c, 0xd17, 0xd4d, 0xd2f, 0xd47, 0xd30, 0xd35, 0xd7b, 0x200c,
0xd05, 0xd38, 0xd4b, 0xd31, 0xd38, 0xd4d, 0xd2c, 0xd7c, 0xd2e, 0xd41, 0xd21, 0xd15,
0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd31, 0xd3f, 0xd15, 0xd47, 0xd2a, 0xd4d, 0x20, 0xd35,
0xd46, 0xd7c, 0xd26, 0xd46, 0xd2b, 0xd46, 0xd31, 0xd4b, 0xd2e, 0xd21, 0xd46, 0xd2f,
0xd4d, 0xd31, 0xd31, 0xd47, 0xd2f, 0xd4d, 0x200c, 0xd1c, 0xd3e, 0xd35, 0xd3f, 0xd15,
0xd4d, 0xd26, 0xd15, 0xd4d, 0xd37, 0xd3f, 0xd23, 0x20, 0xd1c, 0xd4b, 0xd7c, 0xd1c,
0xd3f, 0xd2f, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd39, 0xd46, 0xd32,
0xd46, 0xd28, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0x200c, 0xd32, 0xd3f,
0xd05, 0xd21, 0xd3f, 0xd32, 0xd46, 0xd2f, 0xd4d, 0x200c, 0xd21, 0xd4d, 0xd2c, 0xd4d,
0xd30, 0xd3f, 0xd38, 0xd4d, 0x200c, 0xd2c, 0xd46, 0xd2f, 0xd3f, 0xd7b, 0xd2c, 0xd4d,
0xd30, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd7a, 0x20, 0xd39, 0xd3f, 0xd7d, 0xd21, 0xd3e,
0xd7c, 0xd35, 0xd3f, 0xd7b, 0xd2f, 0xd42, 0xd15, 0xd4d, 0xd32, 0xd39, 0xd4a, 0xd2c,
0xd3e, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0xd32, 0xd3f, 0xd7b, 0xd21, 0xd46, 0xd2e,
0xd3e, 0xd7b, 0xd32, 0xd4b, 0xd21, 0xd4d, 0x20, 0xd39, 0xd4b, 0xd35, 0xd4d, 0xd2e,
0xd46, 0xd7d, 0xd2c, 0xd7a, 0xd2a, 0xd46, 0xd7c, 0xd24, 0xd4d, 0xd24, 0xd4d, 0xd38,
0xd3f, 0xd21, 0xd4d, 0xd28, 0xd3f, 0xd06, 0xd02, 0x200c, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd7c, 0x200c, 0xd21, 0xd3e, 0xd02, 0xd05, 0xd23, 0xd4d, 0xd1f, 0xd4b, 0xd31,
0xd05, 0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30, 0xd16, 0xd3e, 0xd7b, 0xd0f, 0xd24,
0xd7b, 0x200c, 0xd38, 0xd4d, 0xd2c, 0xd46, 0xd7d, 0xd17, 0xd4d, 0xd30, 0xd47, 0xd21,
0xd4d, 0xd2c, 0xd46, 0xd7c, 0x200c, 0xd32, 0xd3f, 0xd7b, 0x200c, 0xd2c, 0xd4d, 0xd30,
0xd3e, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd38, 0xd4d, 0x200d, 0xd32, 0xd3e, 0xd35, 0xd2c,
0xd4d, 0xd30, 0xd38, 0xd4d, 0xd38, 0xd7d, 0x200c, 0xd38, 0xd4d, 0xd2c, 0xd41, 0xd1a,
0xd4d, 0xd1a, 0xd3e, 0xd31, 0xd46, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd2c,
0xd41, 0xd21, 0xd3e, 0xd2a, 0xd46, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd4d, 0xd2c,
0xd41, 0xd38, 0xd3f, 0xd7b, 0xd1c, 0xd7b, 0xd1a, 0xd3f, 0xd38, 0xd3f, 0xd28, 0xd3e,
0xd35, 0xd41, 0xd15, 0xd4b, 0xd2a, 0xd4d, 0xd2a, 0xd7b, 0x200c, 0xd39, 0xd47, 0xd17,
0xd7b, 0x200c, 0xd21, 0xd2c, 0xd4d, 0xd32, 0xd3f, 0xd7b, 0xd1c, 0xd3f, 0xd2c, 0xd4d,
0xd30, 0xd3e, 0xd7e, 0xd1f, 0xd4d, 0xd1f, 0xd7c, 0xd17, 0xd47, 0xd7a, 0xd38, 0xd47,
0xd39, 0xd46, 0xd7d, 0x200c, 0xd38, 0xd3f, 0xd19, 0xd4d, 0xd15, 0xd3f, 0xd10, 0xd7d,
0x200c, 0x20, 0xd13, 0xd2b, 0xd4d, 0x20, 0xd2e, 0xd3e, 0xd7b, 0x200c, 0xd07, 0xd38,
0xd4d, 0xd24, 0xd3e, 0xd02, 0x200c, 0xd2c, 0xd41, 0xd7e, 0x200c, 0xd1c, 0xd47, 0xd34,
0xd4d, 0x200c, 0xd38, 0xd3f, 0xd15, 0xd32, 0xd3f, 0xd28, 0xd3f, 0xd7b, 0xd17, 0xd4d,
0xd30, 0xd3e, 0xd21, 0xd4d, 0xd15, 0xd3f, 0xd31, 0xd4b, 0xd15, 0xd40, 0xd35, 0xd4d,
0xd32, 0xd3f, 0xd38, 0xd4d, 0x200c, 0xd2c, 0xd7a, 0x200c, 0xd32, 0xd41, 0xd2c, 0xd4d,
0x200d, 0xd32, 0xd3f, 0xd2f, 0xd3e, 0xd28, 0xd32, 0xd23, 0xd4d, 0xd1f, 0xd7b, 0x200c,
0xd32, 0xd15, 0xd4d, 0x200c, 0xd38, 0xd02, 0xd2c, 0xd7c, 0xd17, 0xd4d, 0xd2e, 0xd3e,
0xd21, 0xd4d, 0xd30, 0xd3f, 0xd21, 0xd4d, 0xd2e, 0xd3e, 0xd7e, 0xd1f, 0xd4d, 0xd1f,
0xd2e, 0xd30, 0xd3f, 0xd2f, 0xd3e, 0xd39, 0xd3e, 0xd2e, 0xd7b, 0xd2e, 0xd3f, 0xd7b,
0x200c, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd2e, 0xd4a, 0xd23, 0xd3e, 0xd15, 0xd4d, 0xd15,
0xd4b, 0xd2e, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4b, 0xd13, 0xd38, 0xd4d, 0xd32, 0xd4b,
0xd2a, 0xd3e, 0xd30, 0xd40, 0xd38, 0xd4d, 0xd2a, 0xd4a, 0xd21, 0xd4d, 0x200d, 0xd17,
0xd4a, 0xd31, 0xd3f, 0xd38, 0xd2a, 0xd4d, 0xd30, 0xd3e, 0xd17, 0xd4d, 0xd31, 0xd4b,
0xd02, 0xd38, 0xd2e, 0xd3e, 0xd31, 0xd38, 0xd3e, 0xd7b, 0x20, 0xd2e, 0xd3e, 0xd30,
0xd3f, 0xd28, 0xd4b, 0xd38, 0xd30, 0xd2f, 0xd47, 0xd35, 0xd4b, 0xd38, 0xd30, 0xd3e,
0xd31, 0xd4d, 0xd31, 0xd4b, 0xd35, 0xd4d, 0xd38, 0xd3f, 0xd02, 0xd2b, 0xd46, 0xd31,
0xd4b, 0xd2a, 0xd4b, 0xd7e, 0xd38, 0xd4d, 0xd15, 0xd4b, 0xd2a, 0xd4d, 0xd2a, 0xd3f,
0xd2f, 0xd46, 0xd38, 0xd4b, 0xd2b, 0xd3f, 0xd2f, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0xd39, 0xd4b, 0xd02, 0xd1f, 0xd3e, 0xd32, 0xd3f,
0xd7b, 0x200c, 0xd1f, 0xd3f, 0xd30, 0xd3e, 0xd28, 0xd46, 0xd09, 0xd32, 0xd4d, 0xd32,
0xd4d, 0xd2f, 0xd3e, 0xd28, 0xd4b, 0xd35, 0xd4d, 0x200c, 0xd38, 0xd4d, 0x200c, 0xd15,
0xd4d, 0xd35, 0xd3e, 0xd26, 0xd41, 0xd38, 0xd4d, 0xd35, 0xd24, 0xd4d, 0xd24, 0xd3f,
0xd15, 0xd4d, 0xd15, 0xd3e, 0xd7b, 0xd35, 0xd3f, 0xd2f, 0xd28, 0xd4d, 0xd28, 0xd35,
0xd3f, 0xd7d, 0x200c, 0xd28, 0xd3f, 0xd2f, 0xd38, 0xd4d, 0xd35, 0xd4b, 0xd7e, 0xd17,
0xd4b, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd21, 0xd4d, 0xd35, 0xd3e, 0xd7c, 0xd38, 0xd4b,
0xd38, 0xd3e, 0xd15, 0xd4d, 0xd30, 0xd46, 0xd2c, 0xd4d, 0xd38, 0xd42, 0xd31, 0xd3f,
0xd1a, 0xd4d, 0xd1a, 0xd4d, 0xd05, 0xd7b, 0xd31, 0xd3e, 0xd28, 0xd28, 0xd31, 0xd3f,
0xd35, 0xd4b, 0xd1a, 0xd3e, 0xd17, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4d, 0xd30, 0xd3f,
0xd38, 0xd4d, 0xd24, 0xd41, 0xd2e, 0xd38, 0xd4d, 0xd15, 0xd4b, 0xd15, 0xd4d, 0xd15,
0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4a, 0xd2e, 0xd4b, 0xd31, 0xd4b, 0xd15, 0xd46, 0xd7c,
0xd17, 0xd41, 0xd32, 0xd46, 0xd7b, 0xd2e, 0xd3e, 0xd39, 0xd3f, 0xd2e, 0xd3e, 0xd32,
0xd3f, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0xd2e, 0xd57, 0xd31, 0xd40, 0xd37,
0xd4d, 0xd2f, 0xd38, 0xd4d, 0xd2e, 0xd2f, 0xd4b, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd31,
0xd40, 0xd2f, 0xd42, 0xd23, 0xd3f, 0xd2f, 0xd7b, 0xd06, 0xd2a, 0xd3f, 0xd2f, 0xd13,
0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd4d, 0xd2c,
0xd4b, 0xd17, 0xd7a, 0xd35, 0xd3f, 0xd32, 0xd4d, 0xd32, 0xd46, 0xd1a, 0xd3e, 0xd24,
0xd4d, 0xd24, 0xd02, 0xd1a, 0xd15, 0xd4d, 0xd08, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd7c, 0xd07, 0xd2b, 0xd3e, 0xd24, 0xd46, 0xd2b, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd35,
0xd4b, 0xd2b, 0xd4b, 0xd2b, 0xd3f, 0xd1c, 0xd3f, 0xd2b, 0xd41, 0xd23, 0xd3e, 0xd2b,
0xd41, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd17, 0xd3e, 0xd32, 0xd2a, 0xd4d, 0xd2a, 0xd3e,
0xd17, 0xd4b, 0xd38, 0xd4d, 0xd17, 0xd3e, 0xd2e, 0xd4d, 0xd2a, 0xd3f, 0xd2f, 0xd7c,
0xd17, 0xd4d, 0xd35, 0xd3e, 0xd21, 0xd7d, 0xd15, 0xd28, 0xd3e, 0xd7d, 0xd17, 0xd4d,
0xd35, 0xd3e, 0xd02, 0xd39, 0xd4b, 0xd23, 0xd32, 0xd42, 0xd32, 0xd41, 0xd0e, 0xd7b,
0xd21, 0xd2c, 0xd31, 0xd3f, 0xd15, 0xd3f, 0xd30, 0xd3f, 0xd31, 0xd4d, 0xd31, 0xd3f,
0xd2e, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd3f, 0xd15, 0xd4a, 0xd38, 0xd4d, 0xd30, 0xd47,
0xd15, 0xd4d, 0xd35, 0xd3e, 0xd1c, 0xd32, 0xd46, 0xd2f, 0xd4d, 0xd7b, 0xd2e, 0xd1c,
0xd42, 0xd31, 0xd4b, 0xd2e, 0xd3e, 0xd7c, 0xd15, 0xd4d, 0xd2f, 0xd41, 0xd38, 0xd3e,
0xd38, 0xd4d, 0xd2e, 0xd3f, 0xd21, 0xd4d, 0x200c, 0xd35, 0xd47, 0xd28, 0xd57, 0xd31,
0xd41, 0xd28, 0xd3f, 0xd2f, 0xd41, 0xd28, 0xd4b, 0xd7c, 0x200c, 0xd2b, 0xd4b, 0xd15,
0xd4d, 0xd15, 0xd4d, 0xd28, 0xd4b, 0xd2e, 0xd3f, 0xd2f, 0xd2a, 0xd3e, 0xd17, 0xd4b,
0x20, 0xd2a, 0xd3e, 0xd17, 0xd4b, 0xd2a, 0xd32, 0xd3e, 0xd35, 0xd41, 0xd2a, 0xd3f,
0xd31, 0xd4d, 0xd31, 0xd4d, 0xd15, 0xd2f, 0xd3f, 0xd7b, 0x200c, 0xd2a, 0xd4b, 0xd7a,
0xd2a, 0xd46, 0xd2a, 0xd4b, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4d, 0x20, 0xd2e, 0xd4b,
0xd34, 0xd4d, 0x200c, 0xd38, 0xd4d, 0xd2c, 0xd48, 0xd31, 0xd3e, 0xd30, 0xd4b, 0xd1f,
0xd4b, 0xd02, 0xd17, 0xd38, 0xd46, 0xd2f, 0xd4d, 0x200c, 0xd2a, 0xd4d, 0xd2a, 0xd3e,
0xd7b, 0x200c, 0xd24, 0xd3e, 0xd39, 0xd3f, 0xd24, 0xd3f, 0xd24, 0xd30, 0xd3e, 0xd35,
0xd1f, 0xd4b, 0xd02, 0xd17, 0xd3e, 0xd1f, 0xd3e, 0xd2a, 0xd41, 0xd35, 0xd46, 0xd2f,
0xd4d, 0xd15, 0xd4d, 0xd35, 0xd3e, 0xd32, 0xd4d, 0xd32, 0xd3f, 0xd38, 0xd4d, 0x4b,
0x61, 0x68, 0x65, 0x72, 0x61, 0x68, 0x44, 0x61, 0x6d, 0x73, 0x79, 0x69,
0x6b, 0x42, 0x61, 0x69, 0x74, 0x75, 0x6c, 0x6d, 0x75, 0x71, 0x61, 0x64,
0x64, 0x69, 0x73, 0x633, 0x64a, 0x6a0, 0x627, 0x6a4, 0x648, 0x631, 0x627, 0x6c,
0x2d, 0x41, 0x6c, 0x121, 0x69, 0x65, 0x72, 0x69, 0x6c, 0x2d, 0x47, 0x77,
0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61, 0x69, 0x6c, 0x2d, 0x120, 0x61,
0x6d, 0x61, 0x6a, 0x6b, 0x61, 0x120, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c,
0x65, 0x6d, 0x6d, 0x69, 0x6c, 0x2d, 0x42, 0x65, 0x6c, 0x74, 0x20, 0x74,
0x61, 0x6c, 0x2d, 0x4b, 0x75, 0x77, 0x61, 0x6a, 0x74, 0x69, 0x6c, 0x2d,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x74, 0x61, 0x6e, 0x2d,
0x4e, 0x6f, 0x66, 0x73, 0x69, 0x6e, 0x68, 0x61, 0x72, 0x41, 0x74, 0x65,
0x6e, 0x69, 0x42, 0x72, 0x75, 0x73, 0x73, 0x65, 0x6c, 0x6c, 0x120, 0x69,
0x62, 0x69, 0x6c, 0x74, 0xe0, 0x4c, 0x69, 0x17c, 0x62, 0x6f, 0x6e, 0x61,
0x69, 0x6c, 0x2d, 0x4c, 0x75, 0x73, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x75, 0x56, 0x61, 0x6c, 0x6c, 0x65, 0x74, 0x74, 0x61, 0x50, 0x61,
0x72, 0x69, 0x121, 0x69, 0x52, 0x75, 0x6d, 0x61, 0x53, 0x74, 0x6f, 0x6b,
0x6b, 0x6f, 0x6c, 0x6d, 0x61, 0x69, 0x6c, 0x2d, 0x42, 0x65, 0x6c, 0x74,
0x20, 0x74, 0x61, 0x6c, 0x2d, 0x56, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x6e,
0x56, 0x6a, 0x65, 0x6e, 0x6e, 0x61, 0x56, 0x61, 0x72, 0x73, 0x61, 0x76,
0x6a, 0x61, 0x69, 0x6c, 0x2d, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x69,
0x4e, 0x67, 0x101, 0x69, 0x72, 0x6f, 0x70, 0x69, 0x54, 0x69, 0x72, 0x69,
0x70, 0x6f, 0x72, 0x12b, 0x54, 0x16b, 0x6e, 0x12b, 0x68, 0x69, 0x54, 0x65,
0x20, 0x4d, 0x6f, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x68, 0x65, 0x50, 0x61,
0x70, 0x61, 0x74, 0x6f, 0x68, 0x65, 0x50, 0x113, 0x72, 0x69, 0x68, 0x69,
0x4b, 0x65, 0x6d, 0x75, 0x72, 0x65, 0x74, 0x69, 0x20, 0x50, 0x65, 0x69,
0x4b, 0x101, 0x6d, 0x61, 0x6e, 0x61, 0x48, 0x69, 0x6b, 0x101, 0x6b, 0x6f,
0x4b, 0x6f, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b, 0x61, 0x54, 0x6f, 0x6d,
0x69, 0x6e, 0x69, 0x6b, 0x61, 0x57, 0x68, 0x61, 0x6b, 0x61, 0x6f, 0x72,
0x61, 0x4b, 0x75, 0x69, 0x68, 0x69, 0x20, 0x50, 0x65, 0x69, 0x54, 0x101,
0x6b, 0x6f, 0x72, 0x75, 0x20, 0x4e, 0x75, 0x69, 0x4b, 0x65, 0x72, 0x65,
0x6e, 0x61, 0x74, 0x61, 0x4b, 0x75, 0x61, 0x74, 0x61, 0x6d, 0x101, 0x72,
0x61, 0x48, 0x61, 0x77, 0x68, 0x101, 0x6e, 0x61, 0x48, 0x65, 0x6d, 0x65,
0x69, 0x6b, 0x61, 0x4e, 0x67, 0x101, 0x20, 0x41, 0x6e, 0x61, 0x68, 0x65,
0x72, 0x61, 0x4d, 0x101, 0x74, 0x69, 0x6e, 0x69, 0x6b, 0x69, 0x4d, 0x113,
0x68, 0x69, 0x6b, 0x6f, 0x20, 0x54, 0x101, 0x6f, 0x6e, 0x65, 0x6e, 0x75,
0x69, 0x54, 0x65, 0x20, 0x100, 0x70, 0x6f, 0x72, 0x6f, 0x20, 0x4e, 0x75,
0x69, 0x50, 0x65, 0x74, 0x61, 0x20, 0x52, 0x69, 0x6b, 0x6f, 0x48, 0x61,
0x74, 0x6f, 0x20, 0x48, 0x6f, 0x6e, 0x65, 0x48, 0x61, 0x74, 0x6f, 0x20,
0x52, 0x75, 0x69, 0x68, 0x69, 0x61, 0x48, 0x61, 0x74, 0x6f, 0x20, 0x54,
0x61, 0x6d, 0x61, 0x74, 0x69, 0x48, 0x61, 0x74, 0x6f, 0x20, 0x57, 0x113,
0x6e, 0x65, 0x74, 0x69, 0x54, 0x101, 0x72, 0x6f, 0x74, 0x6f, 0x54, 0x65,
0x20, 0x57, 0x68, 0x61, 0x6e, 0x67, 0x61, 0x2d, 0x61, 0x2d, 0x4b, 0x69,
0x77, 0x61, 0x52, 0x113, 0x77, 0x65, 0x74, 0x69, 0x4d, 0x61, 0x6b, 0x6f,
0x61, 0x72, 0x65, 0x50, 0x101, 0x6b, 0x61, 0x74, 0x61, 0x74, 0x61, 0x50,
0x101, 0x72, 0x65, 0x69, 0x6e, 0x61, 0x50, 0x61, 0x6e, 0x67, 0x61, 0x6b,
0x6f, 0x6b, 0x6f, 0x50, 0x6f, 0x72, 0x6f, 0x6e, 0x61, 0x69, 0x54, 0x75,
0x70, 0x61, 0x65, 0x4b, 0x101, 0x68, 0x61, 0x48, 0x6f, 0x6e, 0x67, 0x69,
0x70, 0x75, 0x61, 0x54, 0x69, 0x61, 0x6b, 0x101, 0x74, 0x61, 0x48, 0x69,
0x72, 0x75, 0x68, 0x101, 0x72, 0x61, 0x6d, 0x61, 0x4b, 0x61, 0x74, 0x61,
0x6d, 0x61, 0x72, 0x16b, 0x4b, 0x75, 0x61, 0x72, 0x61, 0x20, 0x52, 0x16b,
0x70, 0x61, 0x4b, 0x16b, 0x77, 0x65, 0x69, 0x74, 0x69, 0x4d, 0x61, 0x6e,
0x69, 0x72, 0x61, 0x50, 0x65, 0x6e, 0x6f, 0x6d, 0x61, 0x20, 0x50, 0x65,
0x6e, 0x61, 0x4b, 0x61, 0x74, 0x101, 0x52, 0x69, 0x61, 0x74, 0x61, 0x48,
0x6f, 0x75, 0x72, 0x61, 0x48, 0x61, 0x6e, 0x67, 0x61, 0x68, 0x61, 0x69,
0x48, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x61, 0x54, 0x65, 0x72, 0x101,
0x6e, 0x61, 0x54, 0x14d, 0x6b, 0x69, 0x6f, 0x50, 0x101, 0x6d, 0x75, 0x72,
0x61, 0x54, 0x65, 0x20, 0x4b, 0x16b, 0x72, 0x61, 0x65, 0x20, 0x4d, 0x61,
0x74, 0x6f, 0x6d, 0x61, 0x74, 0x6f, 0x48, 0x14d, 0x72, 0x69, 0x61, 0x20,
0x6b, 0x69, 0x20, 0x54, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48,
0x61, 0x74, 0x6f, 0x20, 0x48, 0x113, 0x72, 0x65, 0x6e, 0x61, 0x41, 0x74,
0x69, 0x72, 0x65, 0x69, 0x72, 0x61, 0x50, 0x69, 0x72, 0x69, 0x70, 0x61,
0x6e, 0x65, 0x54, 0x101, 0x77, 0x69, 0x6e, 0x69, 0x48, 0x6f, 0x70, 0x61,
0x74, 0x101, 0x6f, 0x6e, 0x65, 0x50, 0x6f, 0x69, 0x70, 0x69, 0x72, 0x69,
0x70, 0x69, 0x50, 0x113, 0x74, 0x69, 0x61, 0x50, 0x6f, 0x69, 0x68, 0x101,
0x6b, 0x65, 0x6e, 0x61, 0x50, 0x101, 0x70, 0x75, 0x6e, 0x69, 0x101, 0x6d,
0x69, 0x74, 0x61, 0x41, 0x6e, 0x61, 0x74, 0x14d, 0x72, 0x61, 0x100, 0x74,
0x65, 0x6e, 0x65, 0x50, 0x65, 0x61, 0x72, 0x12b, 0x6e, 0x69, 0x50, 0x61,
0x72, 0x75, 0x68, 0x69, 0x50, 0x75, 0x74, 0x61, 0x70, 0x113, 0x68, 0x69,
0x4b, 0x6f, 0x70, 0x65, 0x68, 0x65, 0x69, 0x6b, 0x61, 0x6e, 0x61, 0x54,
0x61, 0x70, 0x75, 0x72, 0x69, 0x6e, 0x69, 0x48, 0x113, 0x72, 0x69, 0x6b,
0x69, 0x54, 0x65, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20,
0x4d, 0x61, 0x6e, 0x61, 0x49, 0x74, 0x61, 0x70, 0x16b, 0x72, 0x75, 0x52,
0x12b, 0x70, 0x65, 0x6e, 0x65, 0x52, 0x101, 0x6e, 0x61, 0x6e, 0x61, 0x52,
0x61, 0x6b, 0x61, 0x70, 0x75, 0x14d, 0x4d, 0x101, 0x74, 0x69, 0x72, 0x69,
0x4d, 0x101, 0x72, 0x61, 0x74, 0x61, 0x4d, 0x6f, 0x6e, 0x101, 0x6b, 0x6f,
0x4d, 0x6f, 0x68, 0x69, 0x6b, 0x61, 0x75, 0x14c, 0x68, 0x6f, 0x72, 0x6f,
0x50, 0x61, 0x72, 0x101, 0x6b, 0x61, 0x52, 0x14d, 0x6d, 0x61, 0x48, 0x61,
0x6e, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x6f, 0x54, 0x6f, 0x6b,
0x6f, 0x6f, 0x6d, 0x6f, 0x54, 0x65, 0x20, 0x50, 0x6f, 0x68, 0x6f, 0x2d,
0x6f, 0x2d, 0x50, 0x69, 0x74, 0x61, 0x57, 0x68, 0x69, 0x65, 0x6e, 0x61,
0x48, 0x16b, 0x72, 0x69, 0x6b, 0x61, 0x4b, 0x69, 0x72, 0x69, 0x68, 0x69,
0x6d, 0x65, 0x74, 0x65, 0x4b, 0x6f, 0x6b, 0x6f, 0x68, 0x69, 0x4d, 0x101,
0x72, 0x61, 0x74, 0x69, 0x72, 0x69, 0x4d, 0x61, 0x72, 0x69, 0x68, 0x69,
0x100, 0x70, 0x69, 0x61, 0x54, 0x101, 0x6d, 0x61, 0x6b, 0x69, 0x20, 0x4d,
0x61, 0x6b, 0x61, 0x75, 0x72, 0x61, 0x75, 0x52, 0x113, 0x6b, 0x6f, 0x68,
0x75, 0x57, 0x68, 0x12b, 0x74, 0x12b, 0x4b, 0x75, 0x61, 0x6d, 0x61, 0x4e,
0x14d, 0x70, 0x6f, 0x6b, 0x65, 0x4e, 0x16b, 0x6d, 0x65, 0x61, 0x50, 0x61,
0x6e, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x6e, 0x67, 0x6f, 0x50, 0x101, 0x72,
0x61, 0x75, 0x50, 0x6f, 0x74, 0x61, 0x20, 0x4d, 0x6f, 0x61, 0x68, 0x69,
0x70, 0x69, 0x57, 0x101, 0x72, 0x69, 0x68, 0x69, 0x905, 0x92c, 0x93f, 0x926,
0x94d, 0x91c, 0x93e, 0x928, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x930, 0x93e, 0x906,
0x926, 0x93f, 0x938, 0x20, 0x905, 0x92c, 0x93e, 0x92c, 0x93e, 0x905, 0x932, 0x94d,
0x91c, 0x93f, 0x905, 0x930, 0x94d, 0x938, 0x90f, 0x938, 0x94d, 0x92e, 0x93e, 0x930,
0x93e, 0x92c, 0x93e, 0x92e, 0x915, 0x94b, 0x92c, 0x93e, 0x902, 0x917, 0x941, 0x908,
0x92c, 0x94d, 0x932, 0x901, 0x91f, 0x93e, 0x92f, 0x930, 0x92c, 0x94d, 0x930, 0x93e,
0x91d, 0x93e, 0x935, 0x94d, 0x939, 0x93f, 0x932, 0x947, 0x92c, 0x941, 0x91c, 0x941,
0x902, 0x92c, 0x941, 0x930, 0x93e, 0x915, 0x948, 0x930, 0x94b, 0x915, 0x945, 0x938,
0x93e, 0x92c, 0x94d, 0x932, 0x93e, 0x928, 0x94d, 0x915, 0x93e, 0x915, 0x94b, 0x928,
0x93e, 0x915, 0x94d, 0x930, 0x93f, 0x921, 0x915, 0x930, 0x926, 0x93e, 0x930, 0x20,
0x90f, 0x20, 0x938, 0x932, 0x93e, 0x92e, 0x926, 0x94b, 0x909, 0x906, 0x932, 0x93e,
0x90f, 0x932, 0x20, 0x910, 0x909, 0x928, 0x916, 0x93e, 0x930, 0x94d, 0x91f, 0x941,
0x92e, 0x915, 0x940, 0x917, 0x93e, 0x932, 0x940, 0x932, 0x93f, 0x92c, 0x930, 0x935,
0x94d, 0x939, 0x93f, 0x932, 0x932, 0x941, 0x92c, 0x902, 0x92c, 0x93e, 0x936, 0x940,
0x92e, 0x938, 0x947, 0x930, 0x941, 0x905, 0x902, 0x92c, 0x93e, 0x92c, 0x93e, 0x928,
0x947, 0x92e, 0x94b, 0x928, 0x930, 0x94b, 0x935, 0x94d, 0x939, 0x93f, 0x92f, 0x93e,
0x907, 0x902, 0x91c, 0x93e, 0x92e, 0x947, 0x928, 0x93e, 0x928, 0x941, 0x935, 0x93e,
0x915, 0x938, 0x941, 0x924, 0x935, 0x93e, 0x917, 0x921, 0x942, 0x917, 0x942, 0x92a,
0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x2d, 0x928, 0x94b, 0x935, 0x94d, 0x939, 0x94b,
0x938, 0x93e, 0x913, 0x20, 0x924, 0x94b, 0x92e, 0x947, 0x91f, 0x92f, 0x942, 0x928,
0x93f, 0x938, 0x935, 0x93f, 0x902, 0x921, 0x939, 0x94b, 0x90f, 0x915, 0x905, 0x921,
0x945, 0x915, 0x905, 0x901, 0x915, 0x930, 0x947, 0x91c, 0x905, 0x901, 0x917, 0x94d,
0x935, 0x93f, 0x932, 0x93e, 0x905, 0x901, 0x91f, 0x93f, 0x917, 0x94d, 0x935, 0x93e,
0x905, 0x930, 0x93e, 0x917, 0x93e, 0x92f, 0x928, 0x93e, 0x92c, 0x94d, 0x92f, 0x941,
0x928, 0x94b, 0x938, 0x20, 0x906, 0x92f, 0x930, 0x94d, 0x938, 0x915, 0x945, 0x91f,
0x93e, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x93e, 0x915, 0x949, 0x930, 0x94d, 0x921,
0x94b, 0x92c, 0x93e, 0x92e, 0x947, 0x902, 0x926, 0x94b, 0x91d, 0x93e, 0x930, 0x93f,
0x92f, 0x94b, 0x20, 0x917, 0x945, 0x932, 0x947, 0x917, 0x949, 0x938, 0x938, 0x949,
0x932, 0x94d, 0x924, 0x93e, 0x938, 0x93e, 0x928, 0x20, 0x932, 0x94d, 0x92f, 0x941,
0x907, 0x938, 0x909, 0x938, 0x94d, 0x935, 0x93e, 0x907, 0x92f, 0x93e, 0x906, 0x938,
0x941, 0x928, 0x94d, 0x938, 0x93f, 0x92f, 0x94b, 0x928, 0x905, 0x945, 0x91f, 0x93f,
0x915, 0x94b, 0x915, 0x928, 0x92c, 0x93e, 0x939, 0x93f, 0x92f, 0x93e, 0x20, 0x92c,
0x93e, 0x902, 0x926, 0x947, 0x930, 0x93e, 0x938, 0x92c, 0x93e, 0x930, 0x94d, 0x92c,
0x93e, 0x921, 0x94b, 0x938, 0x92c, 0x947, 0x932, 0x93f, 0x91d, 0x947, 0x92c, 0x94d,
0x932, 0x93e, 0x902, 0x915, 0x20, 0x938, 0x945, 0x92c, 0x932, 0x94b, 0x928, 0x92c,
0x94b, 0x906, 0x20, 0x935, 0x94d, 0x939, 0x93f, 0x938, 0x94d, 0x91f, 0x93e, 0x92c,
0x94b, 0x907, 0x938, 0x940, 0x915, 0x947, 0x902, 0x92c, 0x94d, 0x930, 0x93f, 0x91c,
0x20, 0x909, 0x92a, 0x938, 0x93e, 0x917, 0x930, 0x915, 0x945, 0x92e, 0x94d, 0x92a,
0x94b, 0x20, 0x917, 0x94d, 0x930, 0x901, 0x921, 0x947, 0x915, 0x930, 0x93e, 0x915,
0x93e, 0x938, 0x915, 0x93e, 0x92f, 0x947, 0x928, 0x947, 0x915, 0x947, 0x92e, 0x928,
0x938, 0x93f, 0x909, 0x926, 0x93e, 0x926, 0x20, 0x939, 0x941, 0x906, 0x930, 0x947,
0x91d, 0x915, 0x94d, 0x92f, 0x941, 0x930, 0x93e, 0x938, 0x93e, 0x913, 0x921, 0x947,
0x928, 0x94d, 0x935, 0x94d, 0x939, 0x930, 0x921, 0x947, 0x91f, 0x94d, 0x930, 0x949,
0x907, 0x91f, 0x90f, 0x921, 0x92e, 0x93e, 0x901, 0x91f, 0x928, 0x92f, 0x942, 0x930,
0x941, 0x928, 0x940, 0x92a, 0x947, 0x917, 0x94d, 0x932, 0x947, 0x938, 0x20, 0x909,
0x92a, 0x938, 0x93e, 0x917, 0x930, 0x917, 0x942, 0x938, 0x20, 0x909, 0x92a, 0x938,
0x93e, 0x917, 0x930, 0x917, 0x94d, 0x930, 0x901, 0x921, 0x20, 0x91f, 0x930, 0x94d,
0x915, 0x917, 0x94d, 0x930, 0x947, 0x928, 0x947, 0x921, 0x93e, 0x917, 0x94d, 0x935,
0x93e, 0x921, 0x947, 0x932, 0x94b, 0x909, 0x92a, 0x947, 0x917, 0x92f, 0x93e, 0x915,
0x94d, 0x935, 0x93f, 0x932, 0x939, 0x945, 0x932, 0x93f, 0x92b, 0x945, 0x915, 0x94d,
0x938, 0x935, 0x947, 0x935, 0x93e, 0x92f, 0x2d, 0x907, 0x902, 0x921, 0x93f, 0x92f,
0x93e, 0x928, 0x93e, 0x935, 0x93f, 0x928, 0x92e, 0x945, 0x915, 0x2c, 0x20, 0x907,
0x902, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x907, 0x915, 0x93e, 0x932, 0x941,
0x907, 0x924, 0x91c, 0x94d, 0x92f, 0x942, 0x928, 0x94c, 0x932, 0x94d, 0x92f, 0x941,
0x907, 0x938, 0x935, 0x94d, 0x939, 0x93f, 0x932, 0x92e, 0x93e, 0x901, 0x91f, 0x93f,
0x938, 0x947, 0x932, 0x94b, 0x2c, 0x20, 0x915, 0x947, 0x902, 0x91f, 0x941, 0x915,
0x940, 0x915, 0x94d, 0x930, 0x93e, 0x932, 0x947, 0x902, 0x926, 0x93f, 0x91c, 0x93f,
0x915, 0x932, 0x949, 0x938, 0x20, 0x90f, 0x902, 0x91c, 0x947, 0x932, 0x93f, 0x938,
0x932, 0x94b, 0x905, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x93f, 0x928, 0x94d, 0x938,
0x938, 0x20, 0x915, 0x94d, 0x935, 0x93e, 0x930, 0x94d, 0x91f, 0x930, 0x92e, 0x947,
0x938, 0x947, 0x907, 0x913, 0x92e, 0x947, 0x930, 0x940, 0x917, 0x94b, 0x91f, 0x92e,
0x93e, 0x91d, 0x93e, 0x924, 0x932, 0x93e, 0x928, 0x92e, 0x947, 0x924, 0x932, 0x93e,
0x915, 0x924, 0x932, 0x93e, 0x92e, 0x93e, 0x901, 0x915, 0x91f, 0x928, 0x92e, 0x949,
0x928, 0x94d, 0x91f, 0x947, 0x930, 0x947, 0x92e, 0x94b, 0x928, 0x94d, 0x91f, 0x947,
0x935, 0x94d, 0x939, 0x93f, 0x921, 0x93f, 0x92f, 0x94b, 0x92e, 0x93e, 0x901, 0x928,
0x94d, 0x91f, 0x938, 0x947, 0x930, 0x93e, 0x924, 0x92c, 0x94d, 0x92f, 0x941, 0x932,
0x93e, 0x939, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x915,
0x94b, 0x91f, 0x93e, 0x92e, 0x927, 0x94d, 0x92f, 0x2c, 0x20, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x928, 0x94d, 0x92f, 0x942,
0x20, 0x938, 0x93e, 0x932, 0x947, 0x92e, 0x2c, 0x20, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x20, 0x921, 0x915, 0x94b, 0x91f, 0x93e, 0x92a, 0x93e, 0x930, 0x92e, 0x930,
0x940, 0x92c, 0x94b, 0x92b, 0x949, 0x928, 0x93f, 0x915, 0x94d, 0x938, 0x92a, 0x94b,
0x930, 0x94d, 0x91f, 0x2d, 0x914, 0x2d, 0x92a, 0x94d, 0x930, 0x93f, 0x902, 0x938,
0x92a, 0x94d, 0x92f, 0x941, 0x90f, 0x930, 0x94d, 0x924, 0x94b, 0x20, 0x930, 0x93f,
0x915, 0x94b, 0x92a, 0x941, 0x902, 0x924, 0x93e, 0x20, 0x905, 0x930, 0x940, 0x928,
0x93e, 0x938, 0x930, 0x945, 0x928, 0x915, 0x93f, 0x928, 0x20, 0x907, 0x928, 0x932,
0x947, 0x91f, 0x930, 0x947, 0x91c, 0x94b, 0x932, 0x94d, 0x92f, 0x942, 0x91f, 0x938,
0x901, 0x91f, 0x93e, 0x930, 0x947, 0x92e, 0x938, 0x945, 0x928, 0x94d, 0x91f, 0x93f,
0x90f, 0x917, 0x94b, 0x938, 0x945, 0x928, 0x94d, 0x91f, 0x94b, 0x20, 0x921, 0x94b,
0x92e, 0x93f, 0x902, 0x917, 0x94b, 0x938, 0x93f, 0x91f, 0x915, 0x93e, 0x938, 0x947,
0x902, 0x91f, 0x20, 0x932, 0x941, 0x938, 0x93f, 0x92f, 0x93e, 0x91f, 0x947, 0x917,
0x941, 0x938, 0x93f, 0x917, 0x93e, 0x932, 0x94d, 0x92a, 0x93e, 0x91f, 0x94b, 0x930,
0x94b, 0x902, 0x91f, 0x94b, 0x935, 0x94d, 0x939, 0x945, 0x928, 0x915, 0x941, 0x935,
0x94d, 0x939, 0x930, 0x935, 0x94d, 0x939, 0x93e, 0x907, 0x91f, 0x939, 0x949, 0x930,
0x94d, 0x938, 0x92f, 0x915, 0x941, 0x924, 0x93e, 0x924, 0x915, 0x945, 0x938, 0x947,
0x921, 0x947, 0x935, 0x94d, 0x939, 0x93f, 0x938, 0x921, 0x94d, 0x92f, 0x941, 0x92e,
0x949, 0x928, 0x94d, 0x91f, 0x20, 0x921, 0x94d, 0x92f, 0x941, 0x930, 0x94d, 0x935,
0x93f, 0x932, 0x92e, 0x945, 0x915, 0x94d, 0x935, 0x93e, 0x92f, 0x930, 0x92e, 0x945,
0x915, 0x94d, 0x92e, 0x941, 0x930, 0x921, 0x94b, 0x92a, 0x93e, 0x92e, 0x947, 0x930,
0x935, 0x94b, 0x938, 0x94d, 0x91f, 0x94b, 0x915, 0x932, 0x93e, 0x901, 0x917, 0x907,
0x92f, 0x930, 0x92c, 0x940, 0x92f, 0x947, 0x928, 0x90f, 0x928, 0x93e, 0x921, 0x940,
0x92f, 0x930, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x91f, 0x94c, 0x905, 0x200d, 0x945,
0x915, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x905, 0x924, 0x93f, 0x930, 0x949, 0x92c,
0x947, 0x939, 0x930, 0x940, 0x928, 0x92c, 0x930, 0x94d, 0x928, 0x94c, 0x932, 0x92c,
0x94d, 0x930, 0x941, 0x928, 0x947, 0x908, 0x92b, 0x945, 0x92e, 0x917, 0x941, 0x938,
0x94d, 0x924, 0x93e, 0x939, 0x94b, 0x20, 0x91a, 0x93f, 0x20, 0x92e, 0x93f, 0x928,
0x94d, 0x939, 0x20, 0x936, 0x939, 0x930, 0x939, 0x93e, 0x901, 0x917, 0x915, 0x93e,
0x901, 0x917, 0x91c, 0x947, 0x930, 0x941, 0x938, 0x932, 0x947, 0x92e, 0x915, 0x945,
0x92e, 0x91a, 0x93e, 0x91f, 0x915, 0x93e, 0x916, 0x902, 0x926, 0x93f, 0x917, 0x93e,
0x915, 0x94d, 0x935, 0x93e, 0x932, 0x93e, 0x932, 0x902, 0x92a, 0x942, 0x930, 0x92e,
0x947, 0x917, 0x93e, 0x921, 0x928, 0x92e, 0x915, 0x938, 0x94d, 0x938, 0x93e, 0x930,
0x928, 0x94b, 0x935, 0x94b, 0x915, 0x941, 0x91d, 0x94d, 0x928, 0x947, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x92a, 0x94d, 0x928, 0x94b, 0x92e, 0x20, 0x92a, 0x947, 0x928,
0x94d, 0x939, 0x92a, 0x93e, 0x901, 0x91f, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x915,
0x915, 0x93f, 0x91d, 0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x938, 0x947,
0x909, 0x932, 0x938, 0x94d, 0x930, 0x947, 0x926, 0x928, 0x947, 0x915, 0x94b, 0x932,
0x940, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x925, 0x93f, 0x902, 0x92b, 0x942, 0x91f,
0x94b, 0x915, 0x93f, 0x92f, 0x94b, 0x924, 0x94b, 0x92e, 0x938, 0x915, 0x935, 0x94d,
0x939, 0x93f, 0x90f, 0x928, 0x94d, 0x91f, 0x93e, 0x907, 0x928, 0x935, 0x94d, 0x932,
0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x91f, 0x94b, 0x915, 0x92f, 0x947,
0x915, 0x94d, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x92f,
0x947, 0x930, 0x947, 0x935, 0x928, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x94d, 0x939,
0x930, 0x94d, 0x921, 0x947, 0x92b, 0x945, 0x930, 0x94b, 0x92e, 0x921, 0x940, 0x92f,
0x930, 0x93e, 0x930, 0x947, 0x92f, 0x915, 0x94d, 0x91c, 0x93e, 0x935, 0x93f, 0x915,
0x938, 0x947, 0x902, 0x91f, 0x2e, 0x20, 0x939, 0x947, 0x932, 0x947, 0x928, 0x93e,
0x938, 0x94d, 0x91f, 0x945, 0x928, 0x932, 0x947, 0x90f, 0x921, 0x932, 0x947, 0x921,
0x932, 0x93f, 0x902, 0x921, 0x947, 0x92e, 0x928, 0x905, 0x200d, 0x945, 0x92e, 0x938,
0x94d, 0x91f, 0x930, 0x921, 0x945, 0x92e, 0x905, 0x901, 0x921, 0x94b, 0x930, 0x93e,
0x92c, 0x94d, 0x930, 0x93e, 0x924, 0x93f, 0x938, 0x94d, 0x932, 0x93e, 0x935, 0x94d,
0x939, 0x93e, 0x91a, 0x93f, 0x938, 0x93f, 0x928, 0x94c, 0x917, 0x94d, 0x935, 0x947,
0x930, 0x94d, 0x928, 0x938, 0x947, 0x906, 0x92f, 0x932, 0x20, 0x911, 0x92b, 0x20,
0x92e, 0x945, 0x928, 0x915, 0x932, 0x93f, 0x928, 0x93f, 0x928, 0x917, 0x94d, 0x930,
0x93e, 0x921, 0x932, 0x941, 0x92c, 0x94d, 0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93e,
0x932, 0x915, 0x94d, 0x91d, 0x947, 0x902, 0x92c, 0x930, 0x94d, 0x917, 0x92e, 0x930,
0x93f, 0x92f, 0x947, 0x939, 0x93e, 0x92e, 0x947, 0x928, 0x911, 0x938, 0x94d, 0x932,
0x94b, 0x938, 0x945, 0x928, 0x20, 0x92e, 0x930, 0x93f, 0x928, 0x94b, 0x938, 0x93e,
0x930, 0x93e, 0x91c, 0x947, 0x935, 0x94d, 0x939, 0x94b, 0x938, 0x93f, 0x92e, 0x94d,
0x92b, 0x930, 0x94b, 0x92a, 0x94b, 0x932, 0x938, 0x94d, 0x915, 0x949, 0x92a, 0x94d,
0x91c, 0x947, 0x924, 0x93e, 0x932, 0x93f, 0x928, 0x935, 0x921, 0x942, 0x91d, 0x935,
0x94d, 0x939, 0x945, 0x91f, 0x93f, 0x915, 0x928, 0x935, 0x94d, 0x939, 0x93f, 0x90f,
0x928, 0x94d, 0x928, 0x93e, 0x935, 0x93f, 0x932, 0x928, 0x93f, 0x92f, 0x938, 0x935,
0x949, 0x930, 0x94d, 0x938, 0x93e, 0x91d, 0x945, 0x917, 0x94d, 0x930, 0x947, 0x92c,
0x91d, 0x941, 0x930, 0x93f, 0x915, 0x916, 0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938,
0x915, 0x930, 0x917, 0x94d, 0x92f, 0x942, 0x932, 0x947, 0x928, 0x92e, 0x93e, 0x92f,
0x94b, 0x91f, 0x94d, 0x91f, 0x947, 0x911, 0x915, 0x932, 0x902, 0x921, 0x92c, 0x949,
0x917, 0x945, 0x928, 0x935, 0x94d, 0x939, 0x93f, 0x932, 0x91a, 0x945, 0x91f, 0x939,
0x945, 0x92e, 0x92b, 0x941, 0x928, 0x93e, 0x92b, 0x941, 0x924, 0x940, 0x917, 0x945,
0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x917, 0x945, 0x92e, 0x94d, 0x92c,
0x93f, 0x92f, 0x930, 0x939, 0x94b, 0x928, 0x94b, 0x932, 0x941, 0x932, 0x942, 0x915,
0x94b, 0x936, 0x93e, 0x92f, 0x915, 0x94d, 0x935, 0x93e, 0x91c, 0x93e, 0x932, 0x947,
0x908, 0x928, 0x92e, 0x91c, 0x941, 0x930, 0x94b, 0x92e, 0x93e, 0x930, 0x915, 0x94d,
0x935, 0x93f, 0x938, 0x93e, 0x938, 0x928, 0x909, 0x930, 0x941, 0x928, 0x94d, 0x92f,
0x941, 0x90f, 0x928, 0x94c, 0x92e, 0x93f, 0x906, 0x92a, 0x945, 0x917, 0x94b, 0x20,
0x92a, 0x945, 0x917, 0x94b, 0x92a, 0x93f, 0x91f, 0x915, 0x947, 0x930, 0x94d, 0x928,
0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x92e, 0x94b, 0x930, 0x947, 0x938, 0x94d,
0x92c, 0x947, 0x938, 0x948, 0x92a, 0x93e, 0x928, 0x410, 0x431, 0x438, 0x436, 0x430,
0x43d, 0x410, 0x434, 0x434, 0x438, 0x441, 0x2d, 0x41, 0x431, 0x435, 0x431, 0x430,
0x411, 0x430, 0x43d, 0x433, 0x443, 0x439, 0x411, 0x438, 0x441, 0x441, 0x430, 0x443,
0x410, 0x43d, 0x43a, 0x43e, 0x440, 0x430, 0x436, 0x410, 0x440, 0x430, 0x433, 0x443,
0x430, 0x44f, 0x43d, 0x430, 0x411, 0x430, 0x439, 0x44f, 0x411, 0x430, 0x445, 0x44c,
0x44f, 0x20, 0x411, 0x430, 0x43d, 0x434, 0x435, 0x440, 0x430, 0x441, 0x41a, 0x44d,
0x43c, 0x431, 0x440, 0x438, 0x436, 0x438, 0x439, 0x43d, 0x20, 0x431, 0x443, 0x43b,
0x430, 0x43d, 0x41a, 0x430, 0x43c, 0x43f, 0x43e, 0x2d, 0x413, 0x440, 0x430, 0x43d,
0x434, 0x435, 0x421, 0x44c, 0x44e, 0x434, 0x430, 0x434, 0x20, 0x425, 0x443, 0x430,
0x440, 0x435, 0x441, 0x414, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x43a, 0x448, 0x430,
0x432, 0x43d, 0x414, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x20, 0x41a, 0x440, 0x438,
0x43a, 0x42d, 0x43b, 0x44c, 0x20, 0x421, 0x430, 0x43b, 0x44c, 0x432, 0x430, 0x434,
0x43e, 0x440, 0x424, 0x43e, 0x440, 0x442, 0x20, 0x41d, 0x44d, 0x43b, 0x44c, 0x441,
0x43e, 0x43d, 0x413, 0x43b, 0x435, 0x439, 0x441, 0x20, 0x431, 0x443, 0x43b, 0x430,
0x43d, 0x413, 0x443, 0x443, 0x441, 0x20, 0x431, 0x443, 0x43b, 0x430, 0x43d, 0x425,
0x435, 0x440, 0x43c, 0x43e, 0x441, 0x438, 0x43b, 0x43b, 0x43e, 0x422, 0x435, 0x43b,
0x20, 0x421, 0x438, 0x442, 0x438, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430,
0x43d, 0x430, 0x412, 0x438, 0x43d, 0x441, 0x435, 0x43d, 0x435, 0x441, 0x2c, 0x20,
0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d, 0x430, 0x43c,
0x430, 0x43a, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x43d, 0x430, 0x41c,
0x43e, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x43b, 0x43b, 0x43e, 0x2c, 0x20, 0x41a,
0x435, 0x43d, 0x442, 0x430, 0x43a, 0x438, 0x41a, 0x440, 0x430, 0x43b, 0x435, 0x43d,
0x434, 0x438, 0x43a, 0x41b, 0x43e, 0x432, 0x435, 0x440, 0x20, 0x41f, 0x440, 0x438,
0x43d, 0x441, 0x435, 0x441, 0x20, 0x41a, 0x432, 0x430, 0x440, 0x442, 0x435, 0x440,
0x41c, 0x430, 0x440, 0x438, 0x433, 0x43e, 0x442, 0x41d, 0x43e, 0x440, 0x43e, 0x43d,
0x430, 0x411, 0x438, 0x43b, 0x430, 0x2c, 0x20, 0x425, 0x43e, 0x439, 0x434, 0x20,
0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x422, 0x4e9, 0x432, 0x2c, 0x20, 0x425,
0x43e, 0x439, 0x434, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c,
0x44e, 0x2d, 0x421, 0x430, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x425, 0x43e, 0x439,
0x434, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x4af, 0x4af, 0x43a,
0x41e, 0x436, 0x438, 0x43d, 0x430, 0x433, 0x430, 0x41f, 0x43e, 0x440, 0x442, 0x2d,
0x43e, 0x2d, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x418, 0x441, 0x43f, 0x430, 0x43d,
0x438, 0x439, 0x20, 0x431, 0x43e, 0x43e, 0x43c, 0x442, 0x41f, 0x43e, 0x440, 0x442,
0x43e, 0x2d, 0x412, 0x435, 0x43b, 0x44c, 0x44e, 0x420, 0x438, 0x43e, 0x2d, 0x411,
0x440, 0x430, 0x43d, 0x43a, 0x43e, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x411, 0x430,
0x440, 0x442, 0x435, 0x43b, 0x44c, 0x43c, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x2d,
0x416, 0x43e, 0x43d, 0x441, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41a, 0x438, 0x442,
0x442, 0x441, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x41b, 0x44e, 0x441, 0x438, 0x430,
0x423, 0x430, 0x439, 0x442, 0x445, 0x43e, 0x440, 0x437, 0x41a, 0x44d, 0x441, 0x435,
0x439, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x42e, 0x440, 0x432,
0x438, 0x43b, 0x44c, 0x41c, 0x430, 0x43a, 0x2d, 0x41c, 0x4e9, 0x440, 0x434, 0x43e,
0x41b, 0x43e, 0x43d, 0x433, 0x438, 0x440, 0x431, 0x430, 0x439, 0x435, 0x43d, 0x410,
0x43a, 0x442, 0x4e9, 0x431, 0x435, 0x425, 0x43e, 0x20, 0x428, 0x438, 0x20, 0x41c,
0x438, 0x43d, 0x20, 0x445, 0x43e, 0x442, 0x42d, 0x440, 0x445, 0x4af, 0x4af, 0x416,
0x430, 0x439, 0x43f, 0x443, 0x440, 0x415, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x435,
0x43c, 0x41f, 0x451, 0x43d, 0x44c, 0x44f, 0x43d, 0x421, 0x4e9, 0x4af, 0x43b, 0x423,
0x43b, 0x430, 0x430, 0x43d, 0x431, 0x430, 0x430, 0x442, 0x430, 0x440, 0x4ae, 0x440,
0x4af, 0x43c, 0x447, 0x438, 0x4e8, 0x43c, 0x43d, 0x4e9, 0x434, 0x20, 0x416, 0x43e,
0x440, 0x436, 0x438, 0x430, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x425, 0x435, 0x43b,
0x435, 0x43d, 0x430, 0x411, 0x440, 0x43e, 0x43a, 0x435, 0x43d, 0x20, 0x425, 0x438,
0x43b, 0x43b, 0x415, 0x443, 0x43a, 0x43b, 0x430, 0x41b, 0x438, 0x43d, 0x434, 0x435,
0x43c, 0x44d, 0x43d, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x443, 0x41f,
0x435, 0x440, 0x441, 0x41a, 0x438, 0x448, 0x438, 0x43d, 0x451, 0x432, 0x41c, 0x44d,
0x43d, 0x20, 0x410, 0x440, 0x430, 0x43b, 0x41f, 0x430, 0x440, 0x438, 0x441, 0x420,
0x43e, 0x43c, 0x421, 0x442, 0x43e, 0x43a, 0x43e, 0x43b, 0x44c, 0x43c, 0x41e, 0x443,
0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x427, 0x4af, 0x4af, 0x43a, 0x413, 0x430, 0x43c,
0x431, 0x44c, 0x435, 0x440, 0x41a, 0x43e, 0x441, 0x440, 0x44d, 0x41a, 0x432, 0x430,
0x436, 0x430, 0x43b, 0x435, 0x439, 0x43d, 0x423, 0x44d, 0x43a, 0x1834, 0x1825, 0x1822,
0x182a, 0x1820, 0x182f, 0x1830, 0x1828, 0x1829, 0x182c, 0x1823, 0x182a, 0x1833, 0x1823, 0x1823,
0x182f, 0x1820, 0x182d, 0x1820, 0x1828, 0x182a, 0x1820, 0x182d, 0x1820, 0x1833, 0x1823, 0x1837,
0x905, 0x935, 0x93f, 0x921, 0x94d, 0x91c, 0x93e, 0x928, 0x905, 0x915, 0x94d, 0x930,
0x93e, 0x90f, 0x921, 0x94d, 0x921, 0x93f, 0x938, 0x20, 0x906, 0x92c, 0x93e, 0x92c,
0x93e, 0x905, 0x932, 0x94d, 0x91c, 0x93f, 0x92f, 0x930, 0x94d, 0x938, 0x92c, 0x94b,
0x92e, 0x93e, 0x915, 0x94b, 0x92c, 0x93e, 0x919, 0x94d, 0x917, 0x941, 0x935, 0x940,
0x92c, 0x94d, 0x932, 0x93e, 0x928, 0x94d, 0x91f, 0x93e, 0x92f, 0x930, 0x92c, 0x94d,
0x930, 0x93e, 0x91c, 0x93e, 0x92d, 0x93f, 0x932, 0x94d, 0x932, 0x947, 0x915, 0x93e,
0x907, 0x930, 0x94b, 0x915, 0x93e, 0x938, 0x93e, 0x92c, 0x94d, 0x932, 0x93e, 0x928,
0x94d, 0x915, 0x93e, 0x921, 0x93e, 0x930, 0x20, 0x90f, 0x938, 0x94d, 0x20, 0x938,
0x932, 0x93e, 0x92e, 0x91c, 0x93f, 0x92c, 0x94c, 0x902, 0x91f, 0x940, 0x921, 0x94b,
0x909, 0x906, 0x932, 0x93e, 0x90f, 0x932, 0x94d, 0x20, 0x906, 0x907, 0x92f, 0x941,
0x928, 0x92b, 0x94d, 0x930, 0x93f, 0x91f, 0x93e, 0x909, 0x928, 0x917, 0x93e, 0x935,
0x94b, 0x930, 0x94b, 0x928, 0x91c, 0x94b, 0x939, 0x93e, 0x928, 0x947, 0x938, 0x935,
0x930, 0x94d, 0x917, 0x916, 0x93e, 0x930, 0x94d, 0x91f, 0x909, 0x92e, 0x932, 0x93f,
0x92c, 0x94d, 0x930, 0x947, 0x92d, 0x93f, 0x932, 0x94d, 0x932, 0x947, 0x932, 0x941,
0x935, 0x93e, 0x928, 0x94d, 0x921, 0x93e, 0x932, 0x941, 0x92c, 0x941, 0x92e, 0x94d,
0x92c, 0x93e, 0x938, 0x940, 0x92e, 0x93e, 0x932, 0x93e, 0x92c, 0x94b, 0x92e, 0x93e,
0x92a, 0x941, 0x924, 0x94b, 0x92e, 0x94b, 0x917, 0x93e, 0x921, 0x93f, 0x936, 0x941,
0x92e, 0x94b, 0x928, 0x94d, 0x930, 0x94b, 0x92d, 0x93f, 0x92f, 0x93e, 0x928, 0x93e,
0x907, 0x930, 0x94b, 0x92c, 0x940, 0x90f, 0x928, 0x94d, 0x200c, 0x91c, 0x93e, 0x92e,
0x947, 0x928, 0x93e, 0x928, 0x93e, 0x92f, 0x92e, 0x947, 0x928, 0x94b, 0x909, 0x906,
0x915, 0x91a, 0x94b, 0x91f, 0x914, 0x906, 0x917, 0x93e, 0x921, 0x94c, 0x917, 0x94c,
0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x2d, 0x928, 0x94b, 0x92d, 0x94b, 0x938,
0x93e, 0x913, 0x20, 0x91f, 0x94b, 0x92e, 0x947, 0x935, 0x93f, 0x928, 0x94d, 0x921,
0x939, 0x94b, 0x90f, 0x915, 0x906, 0x921, 0x93e, 0x915, 0x90f, 0x919, 0x94d, 0x915,
0x94b, 0x930, 0x947, 0x91c, 0x90f, 0x919, 0x94d, 0x917, 0x94d, 0x935, 0x93f, 0x932,
0x94d, 0x932, 0x93e, 0x906, 0x930, 0x93e, 0x917, 0x941, 0x935, 0x93e, 0x928, 0x93e,
0x92c, 0x94d, 0x92f, 0x928, 0x947, 0x936, 0x20, 0x906, 0x92f, 0x930, 0x94d, 0x938,
0x915, 0x93e, 0x924, 0x93e, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x93e, 0x915, 0x94b,
0x930, 0x921, 0x94b, 0x935, 0x93e, 0x92e, 0x947, 0x928, 0x94d, 0x921, 0x94b, 0x91c,
0x93e, 0x930, 0x93f, 0x92f, 0x94b, 0x20, 0x917, 0x94d, 0x92f, 0x93e, 0x932, 0x947,
0x917, 0x94b, 0x938, 0x938, 0x93e, 0x928, 0x20, 0x91c, 0x941, 0x935, 0x93e, 0x928,
0x938, 0x93e, 0x928, 0x20, 0x932, 0x941, 0x907, 0x938, 0x909, 0x936, 0x941, 0x906,
0x907, 0x906, 0x905, 0x938, 0x928, 0x94d, 0x938, 0x93f, 0x92f, 0x928, 0x90f, 0x91f,
0x93f, 0x915, 0x94b, 0x915, 0x93e, 0x928, 0x92c, 0x93e, 0x939, 0x93f, 0x92f, 0x93e,
0x20, 0x92c, 0x93e, 0x928, 0x94d, 0x921, 0x947, 0x930, 0x93e, 0x936, 0x92c, 0x94d,
0x932, 0x93e, 0x928, 0x94d, 0x915, 0x2d, 0x938, 0x93e, 0x92c, 0x94d, 0x932, 0x94b,
0x928, 0x92c, 0x94b, 0x906, 0x20, 0x92d, 0x93f, 0x937, 0x94d, 0x91f, 0x93e, 0x92c,
0x94b, 0x907, 0x91c, 0x915, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92c, 0x94d, 0x930,
0x93f, 0x91c, 0x20, 0x92c, 0x947, 0x915, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92a,
0x94b, 0x20, 0x917, 0x94d, 0x930, 0x93e, 0x928, 0x94d, 0x921, 0x947, 0x915, 0x93e,
0x92f, 0x947, 0x928, 0x94d, 0x928, 0x947, 0x915, 0x947, 0x92e, 0x94d, 0x92f, 0x93e,
0x928, 0x91a, 0x93f, 0x939, 0x941, 0x935, 0x93e, 0x939, 0x941, 0x935, 0x93e, 0x91c,
0x941, 0x906, 0x930, 0x947, 0x91c, 0x20, 0x938, 0x939, 0x930, 0x915, 0x94b, 0x937,
0x94d, 0x91f, 0x93e, 0x20, 0x930, 0x93f, 0x915, 0x93e, 0x915, 0x94d, 0x92f, 0x941,
0x907, 0x906, 0x92c, 0x93e, 0x921, 0x93e, 0x928, 0x94d, 0x92e, 0x93e, 0x930, 0x94d,
0x915, 0x936, 0x93e, 0x92d, 0x928, 0x921, 0x938, 0x928, 0x921, 0x938, 0x928, 0x20,
0x915, 0x94d, 0x930, 0x93f, 0x915, 0x921, 0x947, 0x928, 0x94d, 0x92d, 0x930, 0x921,
0x93f, 0x91f, 0x94d, 0x930, 0x94b, 0x907, 0x91f, 0x90f, 0x921, 0x94d, 0x92e, 0x94b,
0x928, 0x94d, 0x91f, 0x928, 0x906, 0x907, 0x930, 0x928, 0x947, 0x92a, 0x947, 0x90f,
0x932, 0x94d, 0x20, 0x938, 0x93e, 0x932, 0x94d, 0x92d, 0x93e, 0x921, 0x94b, 0x930,
0x917, 0x941, 0x91c, 0x20, 0x92c, 0x947, 0x917, 0x94d, 0x930, 0x93e, 0x928, 0x94d,
0x921, 0x20, 0x91f, 0x930, 0x94d, 0x915, 0x917, 0x94d, 0x935, 0x93e, 0x921, 0x947,
0x932, 0x941, 0x92a, 0x917, 0x941, 0x92f, 0x93e, 0x915, 0x94d, 0x935, 0x93f, 0x932,
0x939, 0x94d, 0x92f, 0x93e, 0x932, 0x93f, 0x92b, 0x94d, 0x92f, 0x93e, 0x915, 0x94d,
0x938, 0x939, 0x92d, 0x93e, 0x928, 0x93e, 0x939, 0x947, 0x930, 0x94d, 0x92e, 0x94b,
0x938, 0x93f, 0x932, 0x94d, 0x932, 0x94b, 0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f,
0x93e, 0x928, 0x93e, 0x92a, 0x94b, 0x932, 0x93f, 0x938, 0x928, 0x94b, 0x915, 0x94d,
0x938, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92e,
0x93e, 0x930, 0x947, 0x928, 0x94d, 0x917, 0x94b, 0x2c, 0x20, 0x907, 0x928, 0x94d,
0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92a, 0x93f, 0x91f, 0x930, 0x94d, 0x938,
0x92c, 0x930, 0x94d, 0x917, 0x2c, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f,
0x93e, 0x928, 0x93e, 0x91f, 0x947, 0x932, 0x20, 0x938, 0x93f, 0x91f, 0x940, 0x2c,
0x20, 0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928, 0x93e, 0x92d, 0x947,
0x92d, 0x947, 0x2c, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x93f, 0x92f, 0x93e, 0x928,
0x93e, 0x92d, 0x93f, 0x928, 0x94d, 0x938, 0x947, 0x928, 0x94d, 0x938, 0x935, 0x93f,
0x928, 0x93e, 0x92e, 0x93e, 0x915, 0x2c, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x93f,
0x92f, 0x93e, 0x928, 0x93e, 0x907, 0x928, 0x941, 0x92d, 0x93f, 0x915, 0x907, 0x915,
0x94d, 0x935, 0x93e, 0x932, 0x941, 0x907, 0x91f, 0x91c, 0x92e, 0x93e, 0x907, 0x915,
0x93e, 0x91c, 0x941, 0x928, 0x93f, 0x909, 0x932, 0x941, 0x907, 0x938, 0x92d, 0x93f,
0x932, 0x94d, 0x932, 0x947, 0x92e, 0x94b, 0x928, 0x94d, 0x91f, 0x93f, 0x938, 0x947,
0x932, 0x94d, 0x932, 0x94b, 0x2c, 0x915, 0x947, 0x928, 0x94d, 0x91f, 0x915, 0x940,
0x915, 0x93e, 0x932, 0x947, 0x928, 0x94d, 0x926, 0x947, 0x91c, 0x93f, 0x915, 0x932,
0x938, 0x94d, 0x20, 0x90f, 0x928, 0x94d, 0x91c, 0x947, 0x932, 0x938, 0x932, 0x94b,
0x905, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x93f, 0x928, 0x94d, 0x938, 0x94d, 0x938,
0x20, 0x915, 0x94d, 0x935, 0x93e, 0x930, 0x94d, 0x91f, 0x930, 0x92e, 0x93e, 0x938,
0x947, 0x907, 0x913, 0x92e, 0x93e, 0x928, 0x93e, 0x917, 0x941, 0x935, 0x93e, 0x92e,
0x93e, 0x928, 0x93e, 0x909, 0x938, 0x92e, 0x94d, 0x92f, 0x93e, 0x930, 0x93f, 0x917,
0x91f, 0x92e, 0x93e, 0x91f, 0x94d, 0x924, 0x93e, 0x92e, 0x94b, 0x930, 0x94b, 0x938,
0x94d, 0x92e, 0x93e, 0x91c, 0x93e, 0x91f, 0x94d, 0x932, 0x93e, 0x928, 0x92e, 0x947,
0x91f, 0x94d, 0x932, 0x93e, 0x915, 0x94d, 0x91f, 0x932, 0x93e, 0x92e, 0x93f, 0x915,
0x94d, 0x935, 0x93f, 0x932, 0x928, 0x92e, 0x94b, 0x928, 0x94d, 0x915, 0x91f, 0x928,
0x92e, 0x94b, 0x928, 0x94d, 0x91f, 0x947, 0x930, 0x94d, 0x930, 0x947, 0x92e, 0x94b,
0x928, 0x94d, 0x91f, 0x947, 0x92d, 0x93f, 0x921, 0x93f, 0x92f, 0x94b, 0x92e, 0x928,
0x94d, 0x91f, 0x938, 0x947, 0x930, 0x94d, 0x930, 0x93e, 0x91f, 0x928, 0x94d, 0x92f,
0x941, 0x92f, 0x94b, 0x930, 0x94d, 0x915, 0x92c, 0x947, 0x909, 0x932, 0x93e, 0x2c,
0x20, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x93e, 0x915, 0x94b, 0x91f,
0x93e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x921, 0x93e, 0x915, 0x94b, 0x91f,
0x93e, 0x2c, 0x20, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x928, 0x92f,
0x93e, 0x901, 0x20, 0x938, 0x93e, 0x932, 0x947, 0x92e, 0x2c, 0x20, 0x909, 0x924,
0x94d, 0x924, 0x930, 0x20, 0x921, 0x93e, 0x915, 0x94b, 0x91f, 0x93e, 0x92a, 0x93e,
0x930, 0x93e, 0x92e, 0x93e, 0x930, 0x93f, 0x935, 0x94b, 0x92a, 0x94b, 0x930, 0x94d,
0x91f, 0x2d, 0x905, 0x909, 0x2d, 0x92a, 0x94d, 0x930, 0x93f, 0x928, 0x94d, 0x938,
0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x20, 0x905, 0x92b, 0x94d, 0x20, 0x938, 0x94d,
0x92a, 0x947, 0x928, 0x92a, 0x94b, 0x930, 0x94d, 0x91f, 0x94b, 0x20, 0x92d, 0x947,
0x932, 0x94d, 0x939, 0x94b, 0x92a, 0x94d, 0x92f, 0x941, 0x930, 0x94d, 0x91f, 0x94b,
0x20, 0x930, 0x93f, 0x915, 0x94b, 0x92a, 0x941, 0x928, 0x94d, 0x91f, 0x93e, 0x20,
0x905, 0x930, 0x93f, 0x928, 0x93e, 0x91c, 0x930, 0x93e, 0x928, 0x94d, 0x915, 0x93f,
0x928, 0x20, 0x907, 0x928, 0x94d, 0x932, 0x947, 0x91f, 0x930, 0x93f, 0x938, 0x93e,
0x907, 0x92b, 0x930, 0x93f, 0x91c, 0x94b, 0x932, 0x941, 0x91f, 0x930, 0x93f, 0x92f,
0x94b, 0x20, 0x92c, 0x94d, 0x930, 0x93e, 0x928, 0x94d, 0x915, 0x94b, 0x938, 0x93e,
0x928, 0x94d, 0x91f, 0x93e, 0x930, 0x947, 0x92e, 0x938, 0x93e, 0x928, 0x94d, 0x91f,
0x93f, 0x906, 0x917, 0x94b, 0x938, 0x93e, 0x928, 0x94d, 0x91f, 0x94b, 0x20, 0x921,
0x94b, 0x92e, 0x93f, 0x919, 0x94d, 0x917, 0x94b, 0x908, 0x91f, 0x94d, 0x91f, 0x94b,
0x915, 0x94d, 0x915, 0x94b, 0x930, 0x91f, 0x942, 0x930, 0x94d, 0x92e, 0x93f, 0x91f,
0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x92c, 0x93e, 0x930, 0x94d, 0x925, 0x947,
0x932, 0x947, 0x92e, 0x940, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x91c, 0x94b,
0x928, 0x94d, 0x938, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x94d, 0x20, 0x915, 0x93f,
0x91f, 0x94d, 0x938, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x932, 0x941, 0x938,
0x93f, 0x92f, 0x93e, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x925, 0x94b, 0x92e,
0x938, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x92d, 0x93f, 0x928, 0x94d, 0x938,
0x947, 0x928, 0x94d, 0x91f, 0x938, 0x94d, 0x935, 0x93f, 0x92b, 0x94d, 0x91f, 0x20,
0x915, 0x930, 0x947, 0x928, 0x94d, 0x91f, 0x91f, 0x94b, 0x930, 0x94b, 0x928, 0x94d,
0x91f, 0x94b, 0x91f, 0x93e, 0x930, 0x94d, 0x91f, 0x94b, 0x932, 0x93e, 0x92d, 0x94d,
0x92f, 0x93e, 0x928, 0x915, 0x94b, 0x92d, 0x930, 0x939, 0x94d, 0x935, 0x93e, 0x907,
0x91f, 0x939, 0x930, 0x94d, 0x938, 0x935, 0x93f, 0x928, 0x94d, 0x928, 0x93f, 0x92a,
0x947, 0x917, 0x915, 0x947, 0x91c, 0x947, 0x926, 0x941, 0x92e, 0x94b, 0x928, 0x94d,
0x91f, 0x20, 0x921, 0x93f, 0x20, 0x909, 0x930, 0x94d, 0x92d, 0x947, 0x932, 0x94d,
0x932, 0x947, 0x92e, 0x915, 0x94d, 0x935, 0x93e, 0x930, 0x93f, 0x908, 0x92e, 0x93e,
0x909, 0x938, 0x928, 0x92e, 0x93e, 0x915, 0x92e, 0x941, 0x930, 0x94d, 0x921, 0x94b,
0x938, 0x93f, 0x913, 0x906, 0x92d, 0x93e, 0x938, 0x94d, 0x91f, 0x94b, 0x915, 0x932,
0x919, 0x92f, 0x93f, 0x905, 0x930, 0x92c, 0x93e, 0x907, 0x90f, 0x928, 0x906, 0x928,
0x93e, 0x921, 0x93f, 0x92f, 0x930, 0x906, 0x915, 0x94d, 0x91f, 0x93e, 0x909, 0x906,
0x915, 0x94d, 0x91f, 0x94b, 0x92c, 0x905, 0x938, 0x94d, 0x915, 0x93e, 0x92c, 0x93e,
0x91f, 0x905, 0x91f, 0x93f, 0x930, 0x93e, 0x909, 0x92c, 0x939, 0x930, 0x93e, 0x908,
0x928, 0x92c, 0x948, 0x902, 0x915, 0x915, 0x92c, 0x947, 0x908, 0x930, 0x941, 0x91f,
0x92c, 0x93f, 0x938, 0x915, 0x947, 0x915, 0x94d, 0x92c, 0x94d, 0x930, 0x941, 0x928,
0x93e, 0x907, 0x91a, 0x94b, 0x907, 0x92c, 0x93e, 0x932, 0x94d, 0x938, 0x93e, 0x928,
0x926, 0x93e, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x938, 0x94d, 0x926, 0x93f, 0x932,
0x94d, 0x932, 0x940, 0x926, 0x941, 0x92c, 0x939, 0x940, 0x926, 0x938, 0x94d, 0x938,
0x93e, 0x928, 0x94d, 0x92c, 0x947, 0x939, 0x93f, 0x92c, 0x94d, 0x930, 0x94b, 0x928,
0x939, 0x94b, 0x20, 0x91a, 0x940, 0x20, 0x92e, 0x93f, 0x928, 0x94d, 0x939, 0x20,
0x936, 0x939, 0x930, 0x939, 0x919, 0x915, 0x919, 0x939, 0x94b, 0x92d, 0x94d, 0x921,
0x915, 0x93e, 0x92e, 0x91a, 0x91f, 0x94d, 0x915, 0x93e, 0x915, 0x93e, 0x920, 0x92e,
0x93e, 0x923, 0x94d, 0x921, 0x94c, 0x902, 0x916, 0x93e, 0x928, 0x94d, 0x926, 0x93f,
0x917, 0x93e, 0x915, 0x941, 0x906, 0x20, 0x932, 0x93e, 0x92e, 0x94d, 0x92a, 0x941,
0x930, 0x915, 0x941, 0x91a, 0x93f, 0x919, 0x92e, 0x915, 0x93e, 0x909, 0x92e, 0x93e,
0x917, 0x93e, 0x921, 0x93e, 0x928, 0x92e, 0x938, 0x94d, 0x915, 0x94d, 0x92f, 0x93e,
0x91f, 0x928, 0x947, 0x92d, 0x94b, 0x915, 0x941, 0x91c, 0x928, 0x947, 0x938, 0x94d,
0x915, 0x928, 0x94b, 0x92c, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938,
0x94d, 0x915, 0x92b, 0x947, 0x928, 0x94b, 0x92e, 0x20, 0x92b, 0x947, 0x928, 0x92a,
0x94b, 0x928, 0x94d, 0x91f, 0x93f, 0x906, 0x928, 0x93e, 0x915, 0x92a, 0x94d, 0x92f,
0x94b, 0x919, 0x92f, 0x93e, 0x919, 0x915, 0x938, 0x94d, 0x91f, 0x93e, 0x928, 0x947,
0x938, 0x92e, 0x93e, 0x930, 0x915, 0x923, 0x94d, 0x921, 0x938, 0x93e, 0x928, 0x94d,
0x918, 0x93e, 0x908, 0x938, 0x94d, 0x930, 0x947, 0x926, 0x94d, 0x928, 0x93f, 0x915,
0x94b, 0x932, 0x93f, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x924, 0x93e, 0x938, 0x94d,
0x915, 0x947, 0x928, 0x94d, 0x91f, 0x924, 0x947, 0x939, 0x947, 0x930, 0x93e, 0x928,
0x925, 0x93f, 0x92e, 0x94d, 0x92a, 0x941, 0x91f, 0x94b, 0x92e, 0x94d, 0x938, 0x94d,
0x915, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x91f, 0x93e, 0x930, 0x909, 0x938, 0x94d,
0x91f, 0x2d, 0x928, 0x947, 0x930, 0x93e, 0x92d, 0x93f, 0x92f, 0x928, 0x94d, 0x924,
0x93f, 0x928, 0x92d, 0x94d, 0x932, 0x93e, 0x921, 0x93f, 0x92d, 0x93e, 0x938, 0x94d,
0x91f, 0x94b, 0x915, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915,
0x930, 0x93e, 0x928, 0x94d, 0x917, 0x941, 0x928, 0x90f, 0x915, 0x91f, 0x930, 0x93f,
0x928, 0x92c, 0x941, 0x930, 0x94d, 0x917, 0x906, 0x91c, 0x94b, 0x930, 0x94d, 0x938,
0x915, 0x94d, 0x92f, 0x93e, 0x928, 0x93e, 0x930, 0x940, 0x915, 0x947, 0x92a, 0x20,
0x92d, 0x930, 0x94d, 0x921, 0x947, 0x92b, 0x93e, 0x930, 0x94b, 0x907, 0x92e, 0x93e,
0x921, 0x947, 0x907, 0x930, 0x93e, 0x930, 0x947, 0x915, 0x94d, 0x91c, 0x93e, 0x92d,
0x93f, 0x915, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x91c, 0x930, 0x94d,
0x91c, 0x93f, 0x92f, 0x93e, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x939, 0x947,
0x932, 0x947, 0x928, 0x93e, 0x938, 0x94d, 0x91f, 0x94d, 0x92f, 0x93e, 0x928, 0x94d,
0x932, 0x940, 0x90f, 0x921, 0x947, 0x932, 0x947, 0x921, 0x907, 0x92f, 0x941, 0x915,
0x94d, 0x932, 0x93e, 0x939, 0x94b, 0x92d, 0x93e, 0x930, 0x94d, 0x91f, 0x932, 0x93f,
0x928, 0x94d, 0x921, 0x947, 0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x932, 0x930, 0x94d,
0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x92e, 0x947, 0x932, 0x94d, 0x92c, 0x930,
0x94d, 0x928, 0x938, 0x93f, 0x921, 0x94d, 0x928, 0x940, 0x90f, 0x92e, 0x94d, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x94d, 0x921, 0x92e, 0x906, 0x928, 0x94d, 0x921, 0x94b,
0x930, 0x94d, 0x930, 0x93e, 0x905, 0x938, 0x94d, 0x924, 0x94d, 0x930, 0x916, 0x93e,
0x928, 0x92c, 0x94d, 0x930, 0x93e, 0x91f, 0x93f, 0x938, 0x94d, 0x932, 0x93e, 0x92d,
0x93e, 0x935, 0x941, 0x91a, 0x93e, 0x930, 0x947, 0x938, 0x94d, 0x91f, 0x92c, 0x941,
0x938, 0x93f, 0x928, 0x94d, 0x928, 0x917, 0x928, 0x91a, 0x93f, 0x938, 0x93f, 0x928,
0x93e, 0x909, 0x915, 0x94b, 0x92a, 0x947, 0x928, 0x939, 0x94d, 0x92f, 0x93e, 0x917,
0x928, 0x917, 0x941, 0x90f, 0x930, 0x94d, 0x928, 0x938, 0x947, 0x939, 0x947, 0x932,
0x94d, 0x938, 0x93f, 0x928, 0x94d, 0x915, 0x940, 0x907, 0x91c, 0x932, 0x20, 0x905,
0x92b, 0x94d, 0x20, 0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x908, 0x938, 0x94d, 0x91f,
0x93e, 0x928, 0x92c, 0x941, 0x932, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x93f, 0x928,
0x917, 0x94d, 0x930, 0x93e, 0x926, 0x915, 0x93f, 0x930, 0x94b, 0x92d, 0x915, 0x93f,
0x92d, 0x932, 0x93f, 0x938, 0x94d, 0x92c, 0x94b, 0x928, 0x932, 0x91c, 0x941, 0x92c,
0x93f, 0x91c, 0x93e, 0x928, 0x93e, 0x932, 0x923, 0x94d, 0x921, 0x928, 0x932, 0x915,
0x94d, 0x91c, 0x947, 0x92e, 0x94d, 0x935, 0x930, 0x94d, 0x917, 0x92e, 0x94d, 0x92f,
0x93e, 0x921, 0x94d, 0x930, 0x93f, 0x921, 0x92e, 0x94d, 0x92f, 0x93e, 0x930, 0x93f,
0x939, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x928, 0x92e, 0x938, 0x94d, 0x915, 0x94b,
0x92a, 0x921, 0x94d, 0x917, 0x94b, 0x930, 0x93f, 0x915, 0x93e, 0x938, 0x93e, 0x928,
0x20, 0x92e, 0x93e, 0x930, 0x93f, 0x928, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x91c,
0x947, 0x92d, 0x94b, 0x938, 0x93e, 0x930, 0x93e, 0x91f, 0x94b, 0x92d, 0x938, 0x94d,
0x915, 0x94b, 0x92a, 0x91c, 0x947, 0x938, 0x94d, 0x91f, 0x915, 0x939, 0x94b, 0x932,
0x94d, 0x92e, 0x924, 0x93e, 0x932, 0x94d, 0x932, 0x93f, 0x928, 0x909, 0x932, 0x94d,
0x92f, 0x93e, 0x928, 0x94b, 0x92d, 0x94d, 0x938, 0x94d, 0x915, 0x92d, 0x94d, 0x92f,
0x93e, 0x91f, 0x93f, 0x915, 0x928, 0x92d, 0x93f, 0x92f, 0x928, 0x93e, 0x92d, 0x93f,
0x932, 0x94d, 0x928, 0x93f, 0x905, 0x938, 0x92d, 0x94b, 0x932, 0x94d, 0x917, 0x94b,
0x917, 0x94d, 0x930, 0x93e, 0x926, 0x935, 0x93e, 0x930, 0x938, 0x905, 0x91c, 0x941,
0x930, 0x93f, 0x915, 0x905, 0x928, 0x94d, 0x91f, 0x93e, 0x928, 0x93e, 0x930, 0x93f,
0x92d, 0x94b, 0x916, 0x94d, 0x930, 0x93f, 0x938, 0x94d, 0x91f, 0x92e, 0x938, 0x915,
0x947, 0x930, 0x94d, 0x917, 0x941, 0x90f, 0x932, 0x947, 0x928, 0x92e, 0x93e, 0x932,
0x94d, 0x926, 0x93f, 0x92d, 0x94d, 0x938, 0x92e, 0x909, 0x930, 0x93f, 0x91f, 0x93f,
0x905, 0x938, 0x905, 0x915, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x92c,
0x941, 0x917, 0x947, 0x928, 0x92d, 0x93f, 0x932, 0x94d, 0x932, 0x947, 0x907, 0x938,
0x94d, 0x91f, 0x930, 0x908, 0x92b, 0x93e, 0x924, 0x947, 0x92b, 0x941, 0x928, 0x93e,
0x92b, 0x941, 0x924, 0x93f, 0x917, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938,
0x917, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930, 0x917, 0x941,
0x905, 0x921, 0x93e, 0x932, 0x915, 0x928, 0x93e, 0x932, 0x90f, 0x928, 0x94d, 0x921,
0x930, 0x92c, 0x930, 0x940, 0x915, 0x93f, 0x930, 0x93f, 0x924, 0x93f, 0x92e, 0x93e,
0x91f, 0x940, 0x915, 0x94b, 0x938, 0x94d, 0x930, 0x93e, 0x90f, 0x915, 0x94d, 0x935,
0x93e, 0x91c, 0x93e, 0x932, 0x947, 0x907, 0x928, 0x928, 0x93e, 0x909, 0x930, 0x941,
0x928, 0x93f, 0x909, 0x908, 0x928, 0x930, 0x92b, 0x94b, 0x932, 0x94d, 0x915, 0x928,
0x94b, 0x909, 0x92e, 0x947, 0x905, 0x92a, 0x932, 0x93e, 0x909, 0x92a, 0x93f, 0x924,
0x915, 0x93e, 0x908, 0x930, 0x928, 0x92a, 0x94b, 0x928, 0x93e, 0x92a, 0x947, 0x930,
0x93e, 0x913, 0x924, 0x94b, 0x902, 0x917, 0x93e, 0x938, 0x93e, 0x908, 0x92a, 0x928,
0x924, 0x930, 0x935, 0x93e, 0x91f, 0x902, 0x917, 0x93e, 0x924, 0x92a, 0x941, 0xc1,
0x62, 0x69, 0x6a, 0x61, 0x6e, 0x41, 0x6b, 0x72, 0xe1, 0x41, 0x64, 0xed,
0x20, 0x41, 0x62, 0xe1, 0x62, 0x61, 0x41, 0x6c, 0x6a, 0xed, 0x1eb9, 0x7a,
0x41, 0x73, 0x6d, 0xe1, 0x72, 0x61, 0x42, 0x61, 0x6d, 0xe1, 0x6b, 0x6f,
0x42, 0x61, 0x6e, 0x67, 0xfa, 0x69, 0x42, 0x6c, 0x61, 0x6e, 0x74, 0xe1,
0x79, 0x61, 0x42, 0x72, 0xe1, 0x7a, 0x61, 0x76, 0x69, 0x6c, 0x42, 0x75,
0x6a, 0x75, 0x6d, 0x62, 0xfa, 0x72, 0x61, 0x4b, 0x61, 0xed, 0x72, 0x6f,
0x4b, 0x61, 0x73, 0x61, 0x62, 0x6c, 0xe1, 0x6e, 0x6b, 0x61, 0x53, 0x1eb9,
0xfa, 0x74, 0x61, 0x4b, 0x1ecd, 0x6e, 0xe1, 0x6b, 0x72, 0x69, 0x44, 0x61,
0x6b, 0xe1, 0x72, 0x44, 0x61, 0x72, 0x20, 0x1eb9, 0x301, 0x73, 0x20, 0x53,
0x61, 0x6c, 0x61, 0x6d, 0x4a, 0x69, 0x62, 0xfa, 0x74, 0x69, 0x44, 0x75,
0xe1, 0x6c, 0x61, 0x1eb8, 0x6c, 0x20, 0x41, 0x69, 0x75, 0x6e, 0x46, 0x72,
0xed, 0x74, 0x61, 0x75, 0x6e, 0x48, 0xe1, 0x62, 0x6f, 0x72, 0xf3, 0x6e,
0x1eb9, 0x48, 0x61, 0x72, 0xe1, 0x72, 0x1eb9, 0x4a, 0x6f, 0x68, 0xe1, 0x6e,
0xed, 0x73, 0x62, 0x1ecd, 0x67, 0x4a, 0xfa, 0x62, 0x61, 0x4b, 0x61, 0x6d,
0x70, 0xe1, 0x6c, 0x61, 0x4b, 0x69, 0x67, 0xe1, 0x6c, 0x69, 0x4b, 0x69,
0x6e, 0x73, 0x68, 0xe1, 0x73, 0x68, 0x61, 0x4c, 0xe9, 0x67, 0x6f, 0x73,
0x4c, 0xed, 0x62, 0x72, 0x1eb9, 0x76, 0x69, 0x6c, 0x4c, 0xf3, 0x6d, 0x1eb9,
0x4c, 0x75, 0xe1, 0x6e, 0x64, 0x61, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62,
0xe1, 0x73, 0x68, 0x69, 0x4c, 0x75, 0x73, 0xe1, 0x6b, 0x61, 0x4d, 0x61,
0x6c, 0xe1, 0x62, 0x6f, 0x4d, 0x61, 0x70, 0xfa, 0x74, 0x6f, 0x4d, 0x61,
0x73, 0x1eb9, 0x301, 0x72, 0x75, 0x4d, 0x62, 0x61, 0x62, 0xe1, 0x6e, 0x1eb9,
0x4d, 0x1ecd, 0x67, 0xe1, 0x64, 0xed, 0x73, 0x68, 0x75, 0x4d, 0x6f, 0x6e,
0x72, 0xf3, 0x76, 0x69, 0x61, 0x4e, 0x61, 0xed, 0x72, 0xf3, 0x62, 0x69,
0x4e, 0x6a, 0x61, 0x6d, 0x1eb9, 0x301, 0x6e, 0x61, 0x4e, 0x69, 0xe1, 0x6d,
0x65, 0x4e, 0x6f, 0x75, 0xe1, 0x6b, 0x73, 0x68, 0x1ecd, 0x74, 0x4f, 0x75,
0x61, 0x67, 0x61, 0x64, 0xfa, 0x67, 0x75, 0x50, 0x1ecd, 0x74, 0x6f, 0x2d,
0x4e, 0xf3, 0x76, 0x6f, 0x53, 0x61, 0x6f, 0x20, 0x54, 0xf3, 0x6d, 0x1eb9,
0x54, 0x72, 0xed, 0x70, 0x1ecd, 0x301, 0x6c, 0x69, 0x57, 0xed, 0x6e, 0x64,
0x68, 0x6f, 0x6b, 0xc1, 0x64, 0x61, 0x6b, 0xc1, 0x6e, 0x6b, 0x1ecd, 0x301,
0x72, 0x65, 0x6a, 0x41, 0x6e, 0x67, 0x77, 0xed, 0x6c, 0x61, 0x41, 0x6e,
0x74, 0xed, 0x67, 0x77, 0x75, 0x61, 0x41, 0x72, 0x61, 0x67, 0x77, 0x75,
0x61, 0xed, 0x6e, 0x61, 0x42, 0x75, 0x1eb9, 0x6e, 0x6f, 0x73, 0x20, 0x41,
0xed, 0x72, 0x1eb9, 0x73, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0xe1, 0x6b, 0x61,
0x4b, 0xf3, 0x72, 0x64, 0x6f, 0x62, 0x61, 0x4c, 0x61, 0x20, 0x52, 0x69,
0xf3, 0x6b, 0x68, 0x61, 0x4d, 0x1eb9, 0x6e, 0x64, 0xf3, 0x7a, 0x61, 0x52,
0x69, 0xf3, 0x20, 0x47, 0x61, 0x6c, 0x1eb9, 0x301, 0x67, 0x1ecd, 0x73, 0x53,
0xe1, 0x6c, 0x74, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x48, 0x77, 0x61, 0x6e,
0x54, 0xfa, 0x6b, 0xfa, 0x6d, 0x61, 0x6e, 0x55, 0x73, 0x75, 0xe1, 0x79,
0x61, 0x41, 0x73, 0x75, 0x6e, 0x73, 0x69, 0x1ecd, 0x6e, 0x41, 0x74, 0xed,
0x6b, 0xf3, 0x6b, 0x61, 0x6e, 0x42, 0x61, 0x68, 0xed, 0x61, 0x20, 0x42,
0x61, 0x6e, 0x64, 0x1eb9, 0x301, 0x72, 0x61, 0x73, 0x42, 0x61, 0x72, 0x62,
0xe9, 0x64, 0x1ecd, 0x73, 0x42, 0x1eb9, 0x6c, 0x1eb9, 0x6d, 0x42, 0x1eb9, 0x6c,
0x69, 0x7a, 0x42, 0x6c, 0x61, 0x6e, 0x6b, 0x2d, 0x53, 0xe1, 0x62, 0x6c,
0x1ecd, 0x6e, 0x42, 0xf3, 0x61, 0x20, 0x56, 0xed, 0x73, 0x74, 0x61, 0x42,
0x1ecd, 0xed, 0x73, 0x69, 0x4b, 0xe9, 0x6d, 0x62, 0x72, 0xed, 0x6a, 0x20,
0x42, 0xe8, 0x4b, 0x61, 0x6d, 0x70, 0xf3, 0x20, 0x47, 0x72, 0xe1, 0x6e,
0x64, 0x1eb9, 0x4b, 0x61, 0x72, 0xe1, 0x6b, 0x61, 0x73, 0x4b, 0x61, 0x79,
0x1eb9, 0x6e, 0x4b, 0xe9, 0x6d, 0x61, 0x6e, 0x43, 0x68, 0x69, 0x6b, 0xe1,
0x67, 0x6f, 0x43, 0x68, 0x69, 0x77, 0x75, 0xe1, 0x77, 0x75, 0x61, 0x4b,
0xf3, 0x73, 0x74, 0x61, 0x20, 0x52, 0xed, 0x6b, 0x61, 0x4b, 0x72, 0x1eb9,
0x301, 0x73, 0x74, 0x1ecd, 0x6e, 0x4b, 0xfa, 0x79, 0xe1, 0x62, 0x61, 0x61,
0x4b, 0x69, 0x75, 0x72, 0xe1, 0x73, 0x61, 0x6f, 0x44, 0x61, 0x6e, 0x6d,
0xe1, 0x6b, 0x73, 0x68, 0xe1, 0x76, 0x75, 0x6e, 0x44, 0x1ecd, 0x301, 0x73,
0x1ecd, 0x6e, 0x44, 0x1ecd, 0x301, 0x73, 0x1ecd, 0x6e, 0x20, 0x4b, 0x72, 0x69,
0x6b, 0x44, 0x1eb9, 0x301, 0x6e, 0x76, 0x61, 0x44, 0x69, 0x74, 0x72, 0x1ecd,
0x69, 0x74, 0x44, 0x1ecd, 0x6d, 0xed, 0x6e, 0xed, 0x6b, 0x61, 0x1eb8, 0x301,
0x64, 0x6d, 0x1ecd, 0x301, 0x6e, 0x74, 0x1ecd, 0x6e, 0x1eb8, 0x69, 0x72, 0x75,
0x6e, 0x1eb9, 0x70, 0x1eb9, 0x45, 0x6c, 0x20, 0x53, 0xe1, 0x6c, 0x76, 0xe1,
0x64, 0x1ecd, 0x46, 0x1ecd, 0x74, 0x20, 0x4e, 0x1eb9, 0x301, 0x6c, 0x73, 0x6f,
0x6e, 0x46, 0x1ecd, 0x74, 0x61, 0x6c, 0x1eb9, 0x301, 0x7a, 0x61, 0x47, 0x6c,
0xe1, 0x73, 0x20, 0x42, 0xe8, 0x47, 0xfa, 0x7a, 0x20, 0x42, 0xe8, 0x47,
0x72, 0xe1, 0x6e, 0x64, 0x20, 0x54, 0x1ecd, 0x6b, 0x47, 0x72, 0x1eb9, 0x6e,
0xe9, 0x64, 0x61, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c, 0xfa, 0x70, 0x1eb9,
0x47, 0x75, 0xe1, 0x74, 0x1eb9, 0x6d, 0xe1, 0x6c, 0x61, 0x47, 0x75, 0xe1,
0x79, 0x61, 0x6b, 0x69, 0x6c, 0x47, 0x61, 0x79, 0xe1, 0x6e, 0x61, 0x48,
0xe1, 0x6c, 0xed, 0x66, 0x61, 0x6b, 0x73, 0x48, 0x61, 0x76, 0xe1, 0x6e,
0x61, 0x48, 0x1eb9, 0x301, 0x6d, 0xf3, 0x73, 0xed, 0x6c, 0x6f, 0x49, 0x6e,
0x64, 0x69, 0xe1, 0x6e, 0xe1, 0x70, 0x1ecd, 0x301, 0x6c, 0x69, 0x73, 0x4e,
0x1ecd, 0x6b, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x61,
0x4d, 0x61, 0x72, 0x1eb9, 0x301, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0xe1, 0x6e, 0x61, 0x50, 0xed, 0x74, 0xe1, 0x73, 0x62, 0x1ecd,
0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x61, 0x54, 0x1eb9,
0x301, 0x6c, 0x20, 0x53, 0xed, 0x74, 0x69, 0x2c, 0x20, 0x49, 0x6e, 0x64,
0x69, 0xe1, 0x6e, 0x61, 0x56, 0x1eb9, 0x301, 0x76, 0x65, 0x2c, 0x20, 0x49,
0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x61, 0x56, 0xed, 0x6e, 0x73, 0x1eb9, 0x6e,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x61, 0x57, 0xed, 0x6e,
0xe1, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0xe1, 0x6e,
0x61, 0x49, 0x6e, 0xfa, 0x76, 0x69, 0x6b, 0x49, 0x6b, 0xe1, 0x6c, 0x75,
0x69, 0x74, 0x4a, 0xfa, 0x6e, 0x6f, 0x4c, 0x75, 0xed, 0x76, 0x69, 0x6c,
0x4d, 0x1ecd, 0x6e, 0x74, 0x1eb9, 0x63, 0x68, 0x1eb9, 0x301, 0x6c, 0x6f, 0x2c,
0x20, 0x4b, 0x1eb9, 0x301, 0x6e, 0x74, 0x1ecd, 0x301, 0x6b, 0x69, 0x4b, 0x72,
0xe1, 0x6c, 0x1eb9, 0x301, 0x6e, 0x64, 0x69, 0x6a, 0x6b, 0x4c, 0xed, 0x6d,
0x61, 0x4c, 0x1ecd, 0x73, 0x20, 0xc1, 0x6e, 0x6a, 0x1eb9, 0x301, 0x6c, 0x69,
0x73, 0x4c, 0xf3, 0x77, 0xe1, 0x20, 0x50, 0x72, 0xed, 0x6e, 0x73, 0x20,
0x49, 0x6d, 0x20, 0x4b, 0x77, 0x1ecd, 0x301, 0x74, 0x61, 0x4d, 0x61, 0x73,
0x1eb9, 0x301, 0x69, 0x6f, 0x4d, 0x61, 0x6e, 0xe1, 0x75, 0x73, 0x4d, 0xe1,
0x72, 0xed, 0x67, 0x1ecd, 0x74, 0x4d, 0x61, 0x74, 0xed, 0x6e, 0x69, 0x6b,
0x4d, 0xe1, 0x74, 0x61, 0x6d, 0xf3, 0x72, 0x6f, 0x73, 0x4d, 0x61, 0x7a,
0xe1, 0x74, 0x6c, 0x61, 0x6e, 0x4d, 0x1eb9, 0x6e, 0x1ecd, 0x301, 0x6d, 0x69,
0x6e, 0x69, 0x69, 0x4d, 0x1eb9, 0x301, 0x72, 0xed, 0x64, 0x61, 0x4d, 0x1eb9,
0x74, 0x6c, 0x61, 0x6b, 0xe1, 0x74, 0x6c, 0x61, 0x4d, 0x1eb9, 0x301, 0x6b,
0x73, 0xed, 0x6b, 0xf3, 0x20, 0x53, 0xed, 0x74, 0x69, 0x4d, 0xed, 0x6b,
0x1eb9, 0x6c, 0x1ecd, 0x6e, 0x4d, 0x1ecd, 0x301, 0x6e, 0x6b, 0x74, 0x1ecd, 0x6e,
0x4d, 0x1ecd, 0x6e, 0x74, 0x1eb9, 0x72, 0x1eb9, 0x1eb9, 0x4d, 0x1ecd, 0x6e, 0x74,
0x1eb9, 0x76, 0x69, 0x64, 0x1eb9, 0x6f, 0x4d, 0x1ecd, 0x6e, 0x74, 0x73, 0x1eb9,
0x72, 0x61, 0x74, 0x4e, 0xe1, 0x73, 0x73, 0x1ecd, 0x75, 0x4e, 0x69, 0xfa,
0x20, 0x59, 0x1ecd, 0x6b, 0x4e, 0x6f, 0x6f, 0x6d, 0x4e, 0x1ecd, 0x72, 0xf3,
0x6e, 0x69, 0x61, 0x42, 0x69, 0xfa, 0x6c, 0x61, 0x2c, 0x20, 0x4e, 0x1ecd,
0x301, 0x74, 0x20, 0x44, 0x61, 0x6b, 0xf3, 0x74, 0x61, 0x53, 0x1eb9, 0x301,
0x6e, 0x74, 0x61, 0x2c, 0x20, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x44, 0x61,
0x6b, 0xf3, 0x74, 0x61, 0x4e, 0x69, 0xfa, 0x20, 0x53, 0xe9, 0x6c, 0x1eb9,
0x6d, 0x2c, 0x20, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x44, 0x61, 0x6b, 0xf3,
0x74, 0x61, 0x4f, 0x6b, 0x68, 0x69, 0x6e, 0xe1, 0x67, 0x61, 0x50, 0xe1,
0x6e, 0xe1, 0x6d, 0x61, 0x50, 0xe1, 0x72, 0xe1, 0x6d, 0x61, 0x72, 0x69,
0x62, 0x6f, 0x46, 0xed, 0x6e, 0x69, 0x6b, 0x73, 0x50, 0x1ecd, 0x74, 0x2d,
0x6f, 0x2d, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x50, 0x1ecd, 0x301, 0x74, 0x20,
0x1ecd, 0x66, 0x20, 0x53, 0x70, 0x65, 0x6e, 0x50, 0x1ecd, 0x74, 0x6f, 0x20,
0x56, 0x1eb9, 0x301, 0x6c, 0x68, 0x6f, 0x50, 0x75, 0x1eb9, 0x301, 0x72, 0x74,
0x6f, 0x20, 0x52, 0xed, 0x6b, 0x1ecd, 0x50, 0xfa, 0x6e, 0x74, 0xe1, 0x20,
0x41, 0x72, 0x1eb9, 0x301, 0x6e, 0x61, 0x73, 0x52, 0xe1, 0x6e, 0x6b, 0xed,
0x6e, 0x20, 0xcd, 0x6e, 0x6c, 0x1eb9, 0x74, 0x52, 0x1eb9, 0x73, 0xed, 0x66,
0x1eb9, 0x52, 0x1eb9, 0x6a, 0xed, 0x6e, 0x61, 0x52, 0x1eb9, 0x301, 0x7a, 0xf3,
0x6c, 0x75, 0x74, 0x52, 0x69, 0xf3, 0x20, 0x42, 0x72, 0xe1, 0x6e, 0x6b,
0x6f, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x1eb9, 0x6d, 0x53, 0x61, 0x6e,
0x74, 0x69, 0xe1, 0x67, 0x6f, 0x53, 0xe1, 0x6e, 0x74, 0xf3, 0x20, 0x44,
0x6f, 0x6d, 0xed, 0x6e, 0x67, 0x6f, 0x53, 0x61, 0x6f, 0x20, 0x50, 0x61,
0xfa, 0x6c, 0x6f, 0x49, 0x74, 0xf3, 0x6b, 0x6f, 0x74, 0xfa, 0x72, 0x6d,
0x69, 0x74, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x42, 0x61, 0x74, 0x1eb9, 0x6c,
0x1eb9, 0x301, 0x6d, 0x69, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x4a, 0x1ecd, 0x6e,
0x53, 0x65, 0x6e, 0x74, 0x20, 0x4c, 0xfa, 0x73, 0x68, 0x69, 0x61, 0x53,
0x65, 0x6e, 0x74, 0x20, 0x54, 0x1ecd, 0x6d, 0x1ecd, 0x73, 0x53, 0x65, 0x6e,
0x74, 0x20, 0x56, 0xed, 0x6e, 0x73, 0x1eb9, 0x6e, 0x74, 0x53, 0x77, 0xed,
0x66, 0x74, 0x20, 0x4b, 0x1ecd, 0x301, 0x72, 0x1eb9, 0x6e, 0x74, 0x54, 0x1eb9,
0x67, 0xfa, 0x73, 0x69, 0x67, 0xe1, 0x6c, 0x70, 0x61, 0x54, 0xfa, 0x6c,
0x69, 0x54, 0x69, 0x6b, 0x68, 0x75, 0xe1, 0x6e, 0x61, 0x54, 0x6f, 0x72,
0xf3, 0x6e, 0x74, 0x6f, 0x54, 0x1ecd, 0x74, 0xf3, 0x6c, 0x61, 0x56, 0x61,
0x6e, 0x6b, 0xfa, 0x76, 0x61, 0x57, 0x61, 0xed, 0x74, 0x68, 0x1ecd, 0x73,
0x57, 0xed, 0x6e, 0xed, 0x70, 0x1eb9, 0x67, 0x59, 0x61, 0x6b, 0xfa, 0x74,
0x61, 0x74, 0x4b, 0xe9, 0x73, 0x69, 0x44, 0xe9, 0x76, 0x69, 0x73, 0x44,
0x69, 0xfa, 0x6d, 0x1ecd, 0x301, 0x6e, 0x2d, 0x64, 0x2019, 0x55, 0x76, 0x69,
0x6c, 0x4d, 0x61, 0x6b, 0x77, 0x75, 0xe9, 0xed, 0x4d, 0x1ecd, 0x301, 0x73,
0x1ecd, 0x6e, 0x4d, 0x63, 0x4d, 0x1ecd, 0x301, 0x64, 0x6f, 0x50, 0xe1, 0x6d,
0x61, 0x52, 0x6f, 0x74, 0x1eb9, 0x301, 0x72, 0x61, 0x53, 0x69, 0xf3, 0x77,
0x61, 0x56, 0x1ecd, 0x301, 0x73, 0x74, 0x1ecd, 0x6b, 0x4c, 0x1ecd, 0x6e, 0x67,
0x79, 0x1eb9, 0x301, 0x61, 0x62, 0x69, 0x1eb9, 0x6e, 0xc9, 0x64, 0x1eb9, 0x6e,
0xc1, 0x6c, 0x6d, 0xe1, 0x74, 0x69, 0xc1, 0x6e, 0x61, 0x64, 0x69, 0x61,
0x72, 0x41, 0x6b, 0x74, 0xe1, 0x75, 0x41, 0x6b, 0x74, 0xf3, 0x62, 0x1eb9,
0xc1, 0x73, 0x68, 0x67, 0x61, 0x62, 0x61, 0x74, 0xc1, 0x74, 0xed, 0x72,
0x61, 0x75, 0x42, 0xe1, 0x67, 0x64, 0x61, 0x64, 0x42, 0x61, 0x68, 0x72,
0xe9, 0x6e, 0x42, 0xe1, 0x6e, 0x6b, 0x1ecd, 0x6b, 0x42, 0xe1, 0x72, 0x6e,
0xe1, 0x75, 0x6c, 0x42, 0x1eb9, 0x72, 0x75, 0x74, 0x42, 0x69, 0x73, 0x68,
0x6b, 0x1eb9, 0x6b, 0x42, 0x72, 0x75, 0x6e, 0x1eb9, 0x69, 0x43, 0x68, 0xed,
0x74, 0xe1, 0x43, 0x68, 0x6f, 0x69, 0x62, 0xe1, 0x6c, 0x73, 0x61, 0x6e,
0x4b, 0x6f, 0x6c, 0xf3, 0x6d, 0x62, 0x6f, 0x44, 0x61, 0x6d, 0xe1, 0x73,
0x6b, 0x1ecd, 0x73, 0x44, 0xe1, 0x6b, 0x61, 0x44, 0xed, 0x6c, 0x69, 0x44,
0x75, 0x73, 0x68, 0xe1, 0x6e, 0x62, 0x1eb9, 0x46, 0x61, 0x67, 0xfa, 0x73,
0x74, 0x61, 0x48, 0x1eb9, 0x301, 0x62, 0x72, 0x1ecd, 0x6e, 0x48, 0xf3, 0x20,
0x43, 0x68, 0x69, 0x20, 0x4d, 0xed, 0x6e, 0x20, 0x53, 0xed, 0x74, 0x69,
0x48, 0x1ecd, 0x6e, 0x67, 0x20, 0x4b, 0x1ecd, 0x6e, 0x67, 0x49, 0x72, 0x6b,
0xfa, 0x74, 0x73, 0x6b, 0x4a, 0x61, 0x6b, 0xe1, 0x74, 0x61, 0x4a, 0x61,
0x79, 0x61, 0x70, 0xfa, 0x72, 0x61, 0x4a, 0x1eb9, 0x72, 0xfa, 0x73, 0xe1,
0x6c, 0x1eb9, 0x6d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0xe1, 0x74, 0x6b, 0xe9,
0x4b, 0x61, 0x72, 0xe1, 0x63, 0x68, 0x69, 0x4b, 0x61, 0x74, 0x6d, 0xe1,
0x6e, 0x64, 0x75, 0x4b, 0x61, 0x6e, 0x64, 0xed, 0x67, 0x61, 0x4b, 0x6f,
0x6c, 0x6b, 0xe1, 0x74, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x73, 0x6b, 0x4b, 0x75, 0xe1, 0x6c, 0x61, 0x20, 0x4c, 0xfa, 0x6d,
0x70, 0x1ecd, 0x4b, 0x75, 0x77, 0x65, 0x74, 0x4d, 0x61, 0x6b, 0xe1, 0x6f,
0x4d, 0xe1, 0x67, 0xe1, 0x64, 0x61, 0x6e, 0x4d, 0x61, 0x6b, 0xe1, 0x73,
0x61, 0x4d, 0x61, 0x6e, 0xed, 0x6c, 0x61, 0x4d, 0xfa, 0x73, 0x6b, 0x61,
0x74, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x1eb9, 0x301, 0x73,
0x6b, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73, 0x6b, 0x1ecc,
0x6d, 0x73, 0x6b, 0x1ecc, 0x301, 0x72, 0x61, 0x6c, 0x46, 0x6e, 0x1ecd, 0x301,
0x6d, 0x20, 0x50, 0x1eb9, 0x6e, 0x50, 0x1ecd, 0x6e, 0x74, 0x69, 0xe1, 0x6e,
0x61, 0x6b, 0x50, 0x69, 0x1ecd, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x4b,
0xe1, 0x74, 0x61, 0x4b, 0x6f, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x69, 0x4b,
0xed, 0x7a, 0x69, 0x6c, 0x1ecd, 0x301, 0x64, 0x61, 0x52, 0x69, 0x79, 0xe1,
0x64, 0x53, 0xe1, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0xe1, 0x6d,
0xe1, 0x6b, 0x61, 0x6e, 0x64, 0x53, 0x6f, 0x6c, 0x53, 0x68, 0xe1, 0x6e,
0x67, 0x68, 0x61, 0x69, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x1ecd, 0x53,
0x72, 0x1eb9, 0x301, 0x64, 0x6e, 0x1eb9, 0x6b, 0x6f, 0x6c, 0x69, 0x6d, 0x73,
0x6b, 0x54, 0x61, 0x69, 0x70, 0x1eb9, 0x69, 0x54, 0xe1, 0x73, 0x68, 0x6b,
0x1eb9, 0x6e, 0x74, 0x54, 0x69, 0x62, 0x6c, 0xed, 0x73, 0x69, 0x54, 0x1eb9,
0x72, 0x61, 0x6e, 0x54, 0xed, 0x6d, 0x66, 0x75, 0x54, 0xf3, 0x6b, 0x79,
0x6f, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x59, 0x75, 0x72,
0x1ecd, 0x301, 0x6d, 0x6b, 0x69, 0x55, 0x73, 0x74, 0x2d, 0x4e, 0x1eb9, 0x301,
0x72, 0x61, 0x56, 0x69, 0x1eb9, 0x301, 0x6e, 0x74, 0x69, 0x1eb9, 0x6e, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x1ecd, 0x73, 0x74, 0x1ecd, 0x6b, 0x59, 0xe9,
0x6b, 0xfa, 0x74, 0x73, 0x6b, 0x59, 0x61, 0x6e, 0x67, 0x1ecd, 0x6e, 0x59,
0x1eb9, 0x6b, 0x65, 0x74, 0x1eb9, 0x72, 0xed, 0x6e, 0x62, 0x75, 0x67, 0x59,
0x1eb9, 0x72, 0x1eb9, 0x301, 0x76, 0x61, 0x6e, 0x41, 0x7a, 0x1ecd, 0x7a, 0x42,
0x1eb9, 0x6d, 0x69, 0xfa, 0x64, 0x61, 0x4b, 0x65, 0x6e, 0x65, 0x72, 0xed,
0x4b, 0xe9, 0x70, 0x20, 0x56, 0x1eb9, 0x64, 0x46, 0xe1, 0x72, 0xf3, 0x69,
0x73, 0x4d, 0x61, 0x64, 0xed, 0x72, 0x61, 0x52, 0x1eb9, 0x301, 0x6b, 0x6a,
0xe1, 0x76, 0x69, 0x6b, 0x53, 0x61, 0xfa, 0x74, 0x20, 0x4a, 0x1ecd, 0x301,
0x6a, 0x69, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x48, 0x1eb9, 0x6c, 0x1eb9,
0x301, 0x6e, 0x61, 0x53, 0x74, 0xe1, 0x6e, 0x6c, 0x69, 0x41, 0x64, 0x6c,
0x65, 0x69, 0x64, 0x42, 0x72, 0xed, 0x73, 0x62, 0x65, 0x6e, 0x42, 0x72,
0xf3, 0x6b, 0xfa, 0x6e, 0x20, 0x48, 0x69, 0x6c, 0x44, 0xe1, 0x72, 0x77,
0x69, 0x6e, 0x59, 0xfa, 0x6b, 0x6c, 0x61, 0x48, 0xf3, 0x62, 0x61, 0x74,
0x4c, 0xed, 0x6e, 0x64, 0x1eb9, 0x6d, 0x61, 0x6e, 0x4c, 0x1ecd, 0x64, 0x20,
0x48, 0x61, 0xfa, 0x4d, 0x1eb9, 0x301, 0x6c, 0x62, 0x1ecd, 0x6e, 0x50, 0x1eb9,
0x72, 0x74, 0x53, 0xed, 0x64, 0x6e, 0x69, 0xc1, 0x6d, 0x73, 0x74, 0xe1,
0x64, 0x61, 0x6d, 0x41, 0x6e, 0x64, 0x1ecd, 0x301, 0x72, 0x61, 0xc1, 0x73,
0x74, 0x72, 0x61, 0x68, 0xe1, 0x6e, 0xc1, 0x74, 0x1eb9, 0x6e, 0x73, 0x42,
0x1eb9, 0x301, 0x6c, 0x67, 0x72, 0xe9, 0x64, 0x42, 0x1eb9, 0x6c, 0x69, 0x6e,
0x42, 0x72, 0x61, 0x74, 0xed, 0x73, 0x6c, 0xe1, 0x76, 0x61, 0x42, 0x72,
0xfa, 0x73, 0x75, 0x6c, 0x73, 0x42, 0xfa, 0x6b, 0xe1, 0x72, 0x1eb9, 0x73,
0x74, 0x42, 0xfa, 0x64, 0xe1, 0x70, 0x1eb9, 0x73, 0x74, 0x42, 0x75, 0x73,
0xed, 0x6e, 0x67, 0x1eb9, 0x6e, 0x43, 0x68, 0x69, 0x73, 0x69, 0x6e, 0xe1,
0x75, 0x4b, 0x1ecd, 0x70, 0x1eb9, 0x6e, 0x68, 0xe1, 0x67, 0x1eb9, 0x6e, 0x44,
0x1ecd, 0x301, 0x62, 0x6c, 0x69, 0x6e, 0x4a, 0x69, 0x62, 0x72, 0x1ecd, 0x301,
0x6c, 0x74, 0x61, 0x47, 0x75, 0x1eb9, 0x6e, 0x7a, 0x69, 0x48, 0x1eb9, 0x6c,
0x73, 0xed, 0x6e, 0x6b, 0x69, 0x41, 0xed, 0x6c, 0x20, 0x1ecd, 0x66, 0x20,
0x4d, 0x61, 0x6e, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x62, 0x75, 0x6c, 0x4a,
0x1eb9, 0x301, 0x73, 0x69, 0x4b, 0x61, 0x6c, 0xed, 0x6e, 0xed, 0x6e, 0x67,
0x72, 0x61, 0x64, 0x4b, 0x69, 0x72, 0x1ecd, 0x76, 0x4b, 0x69, 0x1eb9, 0x76,
0x4c, 0xed, 0x73, 0x62, 0x1ecd, 0x6e, 0x4c, 0x75, 0x62, 0x6c, 0x69, 0xe1,
0x6e, 0x61, 0x4c, 0x1ecd, 0x301, 0x6e, 0x64, 0x1ecd, 0x6e, 0x4c, 0x1ecd, 0x301,
0x6b, 0x73, 0x1eb9, 0x301, 0x6d, 0x62, 0x1ecd, 0x67, 0x4d, 0x1ecd, 0x301, 0x6c,
0x74, 0x61, 0x4d, 0x61, 0x72, 0xed, 0x61, 0x68, 0xe1, 0x6d, 0x6e, 0x4d,
0x1ecd, 0x301, 0x6e, 0xe1, 0x6b, 0x6f, 0x4d, 0x1ecd, 0x301, 0x73, 0x6b, 0x6f,
0x1ecc, 0x301, 0x73, 0x6c, 0x6f, 0x50, 0xe1, 0x72, 0x69, 0x73, 0x50, 0x1ecd,
0x301, 0x6a, 0xf3, 0x72, 0xed, 0x6b, 0x61, 0x52, 0xed, 0x67, 0x61, 0x53,
0x61, 0x6d, 0xe1, 0x72, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72,
0xed, 0x6e, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x79, 0x1eb9, 0x301, 0x76, 0x6f,
0x53, 0xe1, 0x72, 0xe1, 0x74, 0x6f, 0x76, 0x53, 0xed, 0x6d, 0x66, 0x1eb9,
0x72, 0xf3, 0x70, 0x6f, 0x6c, 0x53, 0x6b, 0x1ecd, 0x301, 0x70, 0x79, 0x1eb9,
0x53, 0x74, 0x1ecd, 0x301, 0x6b, 0x68, 0x1ecd, 0x6d, 0x54, 0xe1, 0x6c, 0x69,
0x6e, 0x54, 0x69, 0x72, 0xe1, 0x6e, 0x1eb9, 0x55, 0x6c, 0x69, 0xe1, 0x6e,
0x1ecd, 0x76, 0x73, 0x6b, 0x56, 0xe1, 0x74, 0xed, 0x6b, 0x61, 0x6e, 0x56,
0x69, 0x1eb9, 0x301, 0x6e, 0x61, 0x56, 0x6f, 0x6c, 0x76, 0xf3, 0x67, 0x72,
0x61, 0x64, 0x57, 0x1ecd, 0x301, 0x73, 0x1ecd, 0x5a, 0xe1, 0x67, 0x72, 0x1eb9,
0x62, 0x5a, 0xfa, 0x72, 0x69, 0x6b, 0x41, 0x6e, 0x74, 0xe1, 0x6e, 0xe1,
0x6e, 0x61, 0x72, 0xed, 0x76, 0x6f, 0x43, 0x68, 0xe1, 0x67, 0x1ecd, 0x73,
0x4b, 0x72, 0xed, 0x73, 0x6d, 0x61, 0x73, 0x4b, 0x1ecd, 0x301, 0x6d, 0x1ecd,
0x301, 0x72, 0x6f, 0x73, 0x4b, 0x1eb9, 0x301, 0x72, 0x67, 0xfa, 0x6c, 0x1eb9,
0x6e, 0x4d, 0x61, 0x68, 0x1eb9, 0x301, 0x4d, 0x1ecd, 0x301, 0x6c, 0x64, 0x69,
0x76, 0x73, 0x4d, 0x1ecd, 0x72, 0xed, 0x73, 0x68, 0x1ecd, 0x73, 0x4d, 0x65,
0x79, 0x1ecd, 0x74, 0x52, 0x69, 0x79, 0xfa, 0x6e, 0x69, 0x1ecd, 0x6e, 0xc1,
0x70, 0x69, 0x61, 0x1ecc, 0x301, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x42, 0x75,
0x67, 0x65, 0x6e, 0x76, 0xed, 0x6c, 0x69, 0x61, 0x43, 0x68, 0xe1, 0x74,
0x61, 0x6d, 0x43, 0x68, 0x75, 0x6b, 0xcd, 0x73, 0x74, 0x61, 0x1eb8, 0x66,
0xe1, 0x74, 0x1eb9, 0x46, 0x61, 0x6b, 0xe1, 0xf3, 0x66, 0x6f, 0x46, 0xed,
0x6a, 0x69, 0x46, 0x75, 0x6e, 0x61, 0x66, 0xfa, 0x74, 0x69, 0x47, 0x61,
0x6c, 0xe1, 0x70, 0xe1, 0x67, 0x1ecd, 0x73, 0x47, 0xe1, 0x6d, 0x62, 0x69,
0x1eb9, 0x72, 0x47, 0x75, 0xe1, 0x64, 0xe1, 0x6c, 0x6b, 0x61, 0x6e, 0x61,
0x6c, 0x1eb8, 0x301, 0x6e, 0x64, 0xe1, 0x62, 0x1eb9, 0x301, 0x72, 0x69, 0x4b,
0x72, 0x69, 0x74, 0xed, 0x6d, 0xe1, 0x74, 0x69, 0x4b, 0x1ecd, 0x301, 0x73,
0x72, 0x61, 0x1eb9, 0x4b, 0x77, 0xe1, 0x6a, 0x61, 0x6c, 0x1eb9, 0x6e, 0x4d,
0x61, 0x6a, 0xfa, 0x72, 0x6f, 0x4d, 0x61, 0x6b, 0x77, 0x1eb9, 0x301, 0x73,
0x61, 0x73, 0x4d, 0xed, 0x64, 0x77, 0xe8, 0x4e, 0x61, 0xfa, 0x72, 0x75,
0x4e, 0x1ecd, 0x301, 0x72, 0x66, 0x1ecd, 0x301, 0x6c, 0x6b, 0x4e, 0xfa, 0x6d,
0x1eb9, 0x301, 0x61, 0x50, 0xe1, 0x67, 0x6f, 0x20, 0x50, 0xe1, 0x67, 0x6f,
0x50, 0x61, 0x6c, 0xe1, 0x75, 0x50, 0xed, 0x74, 0x6b, 0x61, 0x6e, 0x50,
0x1ecd, 0x6e, 0x70, 0x1eb9, 0x301, 0x69, 0x50, 0x1ecd, 0x74, 0x20, 0x4d, 0x1ecd,
0x72, 0x1eb9, 0x301, 0x73, 0x62, 0x69, 0x52, 0x61, 0x72, 0x61, 0x74, 0xf3,
0x6e, 0x67, 0x61, 0x54, 0x61, 0x68, 0xed, 0x74, 0x69, 0x54, 0x61, 0x72,
0xe1, 0x77, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0xe1, 0x70, 0x75,
0x57, 0x65, 0x6b, 0x57, 0xe1, 0x6c, 0x69, 0x7ca, 0x7d3, 0x7cc, 0x7d6, 0x7ca,
0x7f2, 0x7ec, 0x7ca, 0x7de, 0x7d9, 0x7ca, 0x7eb, 0x7ca, 0x7ec, 0x7d8, 0x7cc, 0x7eb,
0x20, 0x7db, 0x7ca, 0x7ec, 0x7d3, 0x7cb, 0x7d3, 0x7ca, 0x7eb, 0x7ca, 0x7df, 0x7d1,
0x7d6, 0x7ed, 0x7cb, 0x7ca, 0x7db, 0x7d1, 0x7e1, 0x7d9, 0x7ca, 0x7eb, 0x7d3, 0x7e1,
0x7ca, 0x7ec, 0x7de, 0x7d0, 0x7eb, 0x7d3, 0x7ca, 0x7f2, 0x7dc, 0x7ed, 0x7cc, 0x7eb,
0x7d3, 0x7ca, 0x7f2, 0x7d6, 0x7ce, 0x7df, 0x7d3, 0x7cc, 0x7db, 0x7ca, 0x7e5, 0x7cf,
0x7eb, 0x7d3, 0x7d1, 0x7df, 0x7ca, 0x7f2, 0x7d5, 0x7cc, 0x7ef, 0x7d9, 0x7d3, 0x7d9,
0x7d6, 0x7ed, 0x7ca, 0x7dd, 0x7ed, 0x7cc, 0x7df, 0x7d3, 0x7ce, 0x7d6, 0x7ed, 0x7ce,
0x7f2, 0x7d3, 0x7ce, 0x7d9, 0x7ca, 0x7eb, 0x7de, 0x7cd, 0x7ef, 0x7d9, 0x7de, 0x7db,
0x7ca, 0x7d3, 0x7ce, 0x7df, 0x7ca, 0x7f2, 0x7de, 0x7ca, 0x7eb, 0x7de, 0x7d0, 0x7e3,
0x7ca, 0x7de, 0x7d9, 0x7cc, 0x7eb, 0x7d8, 0x7ca, 0x7ec, 0x7de, 0x7ca, 0x7ef, 0x7d9,
0x7ce, 0x7eb, 0x7d8, 0x7ca, 0x7ef, 0x7d9, 0x2d, 0x7db, 0x7ca, 0x7ec, 0x7df, 0x7ca,
0x7ef, 0x7e1, 0x7d6, 0x7cc, 0x7d3, 0x7ce, 0x7d5, 0x7cc, 0x7eb, 0x7d8, 0x7ce, 0x7e5,
0x7df, 0x7ca, 0x7eb, 0x7df, 0x7ca, 0x7ef, 0x7e6, 0x7ce, 0x7e3, 0x7dd, 0x7d9, 0x7cc,
0x7d5, 0x7d0, 0x7f2, 0x7ec, 0x7dc, 0x7ed, 0x7ca, 0x7d3, 0x7cf, 0x7d9, 0x7d0, 0x7e3,
0x7e4, 0x7d9, 0x7ca, 0x7d9, 0x7cb, 0x7eb, 0x7d6, 0x7ed, 0x7ce, 0x7e5, 0x7ca, 0x7e3,
0x7cd, 0x7db, 0x7d1, 0x7d3, 0x7ce, 0x7ef, 0x7d9, 0x7d6, 0x7ce, 0x7d3, 0x7ca, 0x7eb,
0x7de, 0x7ca, 0x7f2, 0x7d4, 0x7df, 0x7ca, 0x7eb, 0x7de, 0x7ca, 0x7d9, 0x7d1, 0x7d5,
0x7ce, 0x7e1, 0x7de, 0x7cc, 0x7dc, 0x7ed, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x7de, 0x7cc,
0x7f2, 0x7db, 0x7ca, 0x7db, 0x7ca, 0x7eb, 0x7df, 0x7cb, 0x7dc, 0x7ed, 0x7d0, 0x7db,
0x7df, 0x7cc, 0x7d3, 0x7d9, 0x7cb, 0x7dd, 0x7ed, 0x7cc, 0x7df, 0x7df, 0x7cf, 0x7e1,
0x7cb, 0x7eb, 0x7df, 0x7ce, 0x7e5, 0x7ca, 0x7f2, 0x7d8, 0x7ca, 0x7eb, 0x7df, 0x7ce,
0x7d3, 0x7ce, 0x7f2, 0x7d3, 0x7ca, 0x7db, 0x7cc, 0x7eb, 0x7df, 0x7ce, 0x7db, 0x7de,
0x7ca, 0x7eb, 0x7e1, 0x7df, 0x7ca, 0x7d3, 0x7cf, 0x7eb, 0x7e1, 0x7ca, 0x7d4, 0x7ce,
0x7d5, 0x7cf, 0x7eb, 0x7e1, 0x7ca, 0x7db, 0x7cb, 0x7d9, 0x7ce, 0x7eb, 0x7d2, 0x7d3,
0x7ca, 0x7d3, 0x7ca, 0x7e3, 0x7e1, 0x7cf, 0x7dc, 0x7ed, 0x7ca, 0x7d8, 0x7cc, 0x7db,
0x7cf, 0x7eb, 0x7e1, 0x7cf, 0x7ec, 0x7d9, 0x7cf, 0x7dd, 0x7ed, 0x7cc, 0x7e6, 0x7ca,
0x7eb, 0x7e3, 0x7ca, 0x7e6, 0x7d9, 0x7cf, 0x7d3, 0x7cc, 0x7eb, 0x7d2, 0x7d6, 0x7ca,
0x7e1, 0x7cb, 0x7e3, 0x7ca, 0x7eb, 0x7e2, 0x7ca, 0x7e1, 0x7cb, 0x7eb, 0x7e3, 0x7ce,
0x7e5, 0x7ca, 0x7de, 0x7d1, 0x7db, 0x7d0, 0x7d5, 0x7e5, 0x7dc, 0x7ed, 0x7ca, 0x7ec,
0x7d8, 0x7dc, 0x7ed, 0x7ce, 0x7eb, 0x7d4, 0x7d0, 0x7d9, 0x7d1, 0x7d5, 0x7cf, 0x7eb,
0x2d, 0x7e3, 0x7dd, 0x7ed, 0x7cf, 0x7eb, 0x7db, 0x7ca, 0x7cf, 0x2d, 0x7d5, 0x7cf,
0x7e1, 0x7cb, 0x7eb, 0x7d5, 0x7d9, 0x7cc, 0x7d4, 0x7df, 0x7cc, 0x7eb, 0x20, 0x28,
0x20, 0x7df, 0x7d3, 0x7cc, 0x7eb, 0x20, 0x29, 0x7d5, 0x7ce, 0x7e3, 0x7cc, 0x7db,
0x7e5, 0x7cc, 0x7f2, 0x7d8, 0x7d0, 0x7de, 0x7dc, 0x7e6, 0x7cb, 0x7de, 0x7ce, 0x7f2,
0x7eb, 0x2d, 0x7dd, 0x7d9, 0x7cc, 0x7db, 0x7cc, 0x7eb, 0x7db, 0x7cd, 0x7f2, 0x7d5,
0x20, 0x7e4, 0x7cb, 0x7df, 0x7cd, 0x7e3, 0x7ca, 0x7f2, 0x7d5, 0x7e3, 0x7ca, 0x7e3,
0x7ca, 0x7d9, 0x7cc, 0x7dd, 0x7ed, 0x7cf, 0x7eb, 0x7db, 0x7ca, 0x7dc, 0x7ed, 0x7d0,
0x7db, 0x7de, 0x7d0, 0x7e1, 0x7d0, 0x7ef, 0x7d9, 0x7de, 0x7cd, 0x7d9, 0x7d1, 0x7dc,
0x7ed, 0x7cb, 0x7df, 0x7cd, 0x7f2, 0x7eb, 0x7e1, 0x7ca, 0x7e4, 0x7cb, 0x7eb, 0x7e1,
0x7cf, 0x7d9, 0x7cc, 0x7db, 0x7e1, 0x7ca, 0x7e6, 0x7d0, 0x7d5, 0x7d9, 0x7cb, 0x7e3,
0x7cc, 0x7e6, 0x7d0, 0x7f2, 0x7eb, 0x44, 0xe1, 0x6e, 0x6d, 0xe1, 0x72, 0x6b,
0x6b, 0x75, 0x68, 0xe1, 0x6d, 0x6d, 0x61, 0x6e, 0x42, 0x65, 0x75, 0x6c,
0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x76, 0x76, 0x69, 0x2d, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x47, 0x75, 0x6f, 0x76, 0x64, 0x64, 0xe1, 0x161,
0x2c, 0x20, 0x44, 0x61, 0x76, 0x76, 0x69, 0x2d, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c,
0x20, 0x44, 0x61, 0x76, 0x76, 0x69, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x4e, 0x6f, 0x76, 0x6f, 0x6b,
0x75, 0x73, 0x6e, 0x65, 0x63, 0x6b, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x41, 0x7a,
0x6f, 0x72, 0x61, 0x74, 0x4b, 0x61, 0x6e, 0xe1, 0x72, 0x69, 0x61, 0x4c,
0x75, 0x6c, 0x6c, 0x69, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x48, 0x65, 0x6c, 0x73, 0x73, 0x65, 0x74, 0x4d, 0x61, 0x6e, 0x73, 0x75,
0x6f, 0x6c, 0x75, 0x4a, 0x75, 0x6f, 0x76, 0x6c, 0x61, 0x73, 0x75, 0x6f,
0x6c, 0x75, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x69, 0x76, 0x76, 0x61,
0x74, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x61, 0x74,
0x43, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x6f, 0x20, 0x62, 0x79, 0x54, 0x73, 0x6a, 0x69,
0x74, 0x61, 0x43, 0x68, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x2d,
0x62, 0x79, 0x65, 0x6e, 0x4a, 0x61, 0x6a, 0x61, 0x70, 0x75, 0x72, 0x61,
0x41, 0x73, 0x6f, 0x72, 0x65, 0x6e, 0x65, 0x4b, 0x61, 0x6e, 0x61, 0x72,
0x69, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x4b, 0x61, 0x70, 0x70, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x46, 0xe6, 0x72, 0xf8, 0x79, 0x65, 0x6e, 0x65,
0x53, 0xf8, 0x72, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x42,
0x75, 0x63, 0x75, 0x72, 0x65, 0x219, 0x74, 0x69, 0x48, 0x65, 0x6c, 0x73,
0x69, 0x6e, 0x67, 0x66, 0x6f, 0x72, 0x73, 0x56, 0x61, 0x74, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x74, 0x61, 0x74, 0x65, 0x6e, 0x43, 0x68, 0x72, 0x69,
0x73, 0x74, 0x6d, 0x61, 0x73, 0xf8, 0x79, 0x61, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x65,
0x6e, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x6e, 0x65, 0x50,
0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x79, 0x61, 0x47, 0x61, 0x6c, 0xe1, 0x70,
0x61, 0x67, 0x6f, 0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0xf8, 0x79, 0x61, 0x43, 0x61, 0x79, 0x6d, 0x61,
0x6e, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x43, 0x75, 0x69, 0x61, 0x62, 0x61,
0x41, 0x73, 0x6a, 0x67, 0x61, 0x62, 0x61, 0x74, 0x59, 0x61, 0x6e, 0x67,
0xf4, 0x6e, 0x41, 0x73, 0x6f, 0x72, 0x61, 0x6e, 0x65, 0x4b, 0x61, 0x6e,
0x61, 0x72, 0x69, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x46, 0xe6, 0x72, 0xf8,
0x79, 0x61, 0x6e, 0x65, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0xf8, 0x79, 0x61,
0x6e, 0x65, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x61, 0x6e, 0x65, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x61, 0x6e, 0x65, 0x47, 0x61, 0x6c, 0xe1, 0x70,
0x61, 0x67, 0x6f, 0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x41, 0x72, 0x67,
0x69, 0xe8, 0x72, 0x73, 0x4a, 0x69, 0x62, 0x6f, 0x74, 0x69, 0x44, 0x6f,
0x61, 0x6c, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6b, 0xf2, 0x6b, 0x4b, 0x61,
0x62, 0x6f, 0x6c, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0x69, 0x6e, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x41, 0xe7,
0xf2, 0x72, 0x65, 0x73, 0x41, 0x64, 0x65, 0x6c, 0x61, 0xef, 0x64, 0x61,
0x41, 0x6e, 0x64, 0xf2, 0x72, 0x72, 0x61, 0x42, 0x72, 0x75, 0x73, 0x73,
0xe8, 0x6c, 0x61, 0x73, 0x42, 0x75, 0x63, 0x61, 0x72, 0xe8, 0x73, 0x74,
0x42, 0x75, 0x64, 0x61, 0x70, 0xe8, 0x73, 0x74, 0x4c, 0x75, 0x78, 0x65,
0x6d, 0x62, 0x6f, 0x72, 0x67, 0x4d, 0xf3, 0x6e, 0x65, 0x67, 0x75, 0x65,
0x4d, 0x6f, 0x73, 0x63, 0xf2, 0x75, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e,
0x65, 0x73, 0x41, 0x62, 0x69, 0x79, 0x61, 0x6e, 0x41, 0x72, 0x67, 0x65,
0x6c, 0x45, 0x74, 0x68, 0x20, 0x43, 0x61, 0x69, 0x72, 0x6f, 0x59, 0x69,
0x62, 0x75, 0x74, 0x69, 0x4a, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x59, 0x61,
0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x75, 0x61, 0x6b, 0x63, 0x68, 0x6f, 0x74,
0x53, 0x61, 0x6e, 0x74, 0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x43, 0xf2, 0x64,
0x6f, 0x62, 0x61, 0x42, 0x65, 0x6c, 0xe9, 0x6e, 0x42, 0x65, 0x6c, 0x69,
0x63, 0x65, 0x43, 0x61, 0x79, 0x65, 0x6e, 0x61, 0x43, 0xf2, 0x73, 0x74,
0x61, 0x20, 0x52, 0x69, 0x63, 0x61, 0x47, 0x72, 0x61, 0x6e, 0x20, 0x54,
0x75, 0x72, 0x63, 0x61, 0x45, 0x72, 0x61, 0x20, 0x48, 0x61, 0x76, 0x61,
0x6e, 0x61, 0x4d, 0x6d, 0x4d, 0x61, 0x6e, 0x61, 0x6f, 0x73, 0x43, 0x69,
0x75, 0x74, 0x61, 0x74, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69,
0x63, 0x4e, 0xf2, 0x76, 0x61, 0x20, 0x59, 0x6f, 0x72, 0x6b, 0x42, 0x65,
0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x4e, 0xf2, 0x72, 0x64, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x4e, 0xf2, 0x72, 0x64, 0x4e, 0x65,
0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x4e, 0xf2, 0x72,
0x64, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x50, 0x72, 0xed, 0x6e,
0x63, 0x69, 0x70, 0x65, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x45,
0x73, 0x70, 0x61, 0x6e, 0x68, 0x61, 0x53, 0x61, 0x6f, 0x20, 0x70, 0x61,
0x75, 0x6c, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x42, 0x61, 0x72, 0x74,
0x6f, 0x6c, 0x6f, 0x6d, 0xe9, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x4a, 0x6f,
0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e,
0xf2, 0x76, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x43, 0x72, 0x69, 0x73,
0x74, 0xf2, 0x62, 0x61, 0x6c, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x56, 0x69,
0x63, 0x65, 0x6e, 0x74, 0x54, 0xf2, 0x72, 0x74, 0x6f, 0x6c, 0x61, 0x41,
0x6e, 0xe1, 0x64, 0x79, 0x72, 0x41, 0x73, 0x6a, 0x61, 0x62, 0x61, 0x64,
0x42, 0x61, 0x72, 0xe9, 0x69, 0x6e, 0x42, 0x61, 0x72, 0x6e, 0x61, 0xfa,
0x6c, 0x44, 0x75, 0x73, 0x61, 0x6d, 0x62, 0xe9, 0x43, 0x69, 0x75, 0x74,
0x61, 0x74, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69,
0x6e, 0x68, 0x4a, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6e, 0x53,
0x61, 0x6a, 0x61, 0x6c, 0xed, 0x6e, 0x53, 0x65, 0x6f, 0x6c, 0x54, 0x69,
0x66, 0x6c, 0x69, 0x65, 0x73, 0x54, 0x69, 0x6d, 0x62, 0x75, 0x45, 0x72,
0x65, 0x76, 0xe1, 0x6e, 0x49, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x65,
0x72, 0x6f, 0x65, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64,
0x65, 0x74, 0x68, 0x20, 0x53, 0x75, 0x64, 0x53, 0x61, 0x6e, 0x74, 0x61,
0x20, 0x45, 0x6c, 0x65, 0x6e, 0x61, 0x53, 0xed, 0x64, 0x6e, 0x65, 0x79,
0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x45,
0x73, 0x74, 0x61, 0x6d, 0x62, 0x75, 0x6c, 0x4c, 0xf2, 0x6e, 0x64, 0x72,
0x65, 0x73, 0x53, 0x61, 0x6e, 0x74, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e,
0x6f, 0x53, 0x61, 0x72, 0xe1, 0x74, 0x6f, 0x76, 0x53, 0x6b, 0x6f, 0x70,
0x69, 0x65, 0x55, 0x6c, 0x69, 0xe1, 0x6e, 0x6f, 0x76, 0x73, 0x6b, 0x45,
0x74, 0x68, 0x20, 0x56, 0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x5a, 0xfa,
0x72, 0x69, 0x63, 0x68, 0x4e, 0x61, 0x64, 0x61, 0x75, 0x49, 0x73, 0x6c,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x46,
0x69, 0x79, 0x69, 0xb06, 0xb2c, 0xb3f, 0xb26, 0xb1c, 0xb3e, 0xb28, 0xb06, 0xb15,
0xb3e, 0xb30, 0xb3e, 0xb06, 0xb26, 0xb3f, 0xb38, 0xb4d, 0x200c, 0x20, 0xb06, 0xb2c,
0xb3e, 0xb2c, 0xb3e, 0xb05, 0xb32, 0xb1c, 0xb3f, 0xb5f, 0xb30, 0xb4d, 0xb38, 0xb05,
0xb38, 0xb2e, 0xb30, 0xb3e, 0xb2c, 0xb3e, 0xb2e, 0xb3e, 0xb15, 0xb4b, 0xb2c, 0xb3e,
0xb19, 0xb4d, 0xb17, 0xb41, 0xb07, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1c, 0xb41, 0xb32,
0xb2c, 0xb3f, 0xb38, 0xb3e, 0xb09, 0xb2c, 0xb4d, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f,
0xb3e, 0xb5f, 0xb3e, 0xb30, 0xb4d, 0x200c, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb1c, 0xb3e,
0xb2d, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb2c, 0xb41, 0xb1c, 0xb41, 0xb2e, 0xb4d,
0xb2c, 0xb41, 0xb30, 0xb3e, 0xb15, 0xb3e, 0xb07, 0xb30, 0xb4b, 0xb15, 0xb3e, 0xb38,
0xb3e, 0xb2c, 0xb4d, 0xb32, 0xb3e, 0xb19, 0xb4d, 0xb15, 0xb3e, 0xb38, 0xb47, 0xb09,
0xb1f, 0xb3e, 0xb15, 0xb4b, 0xb28, 0xb3e, 0xb15, 0xb4d, 0xb30, 0xb3f, 0xb21, 0xb15,
0xb3e, 0xb30, 0xb21, 0xb30, 0xb4d, 0x200c, 0x20, 0xb07, 0xb38, 0xb4d, 0x200c, 0x20,
0xb38, 0xb3e, 0xb32, 0xb3e, 0xb2e, 0xb1c, 0xb3f, 0xb2c, 0xb4b, 0xb1f, 0xb3f, 0xb21,
0xb09, 0xb32, 0xb3e, 0xb0f, 0xb32, 0xb4d, 0x200c, 0x20, 0xb0f, 0xb5f, 0xb28, 0xb4d,
0xb2b, 0xb4d, 0xb30, 0xb3f, 0xb1f, 0xb3e, 0xb09, 0xb28, 0xb4d, 0x200c, 0xb17, 0xb3e,
0xb2c, 0xb4b, 0xb30, 0xb4d, 0xb23, 0xb4d, 0xb23, 0xb39, 0xb30, 0xb3e, 0xb30, 0xb47,
0xb1c, 0xb4b, 0xb39, 0xb3e, 0xb28, 0xb4d, 0xb38, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb1c,
0xb41, 0xb2c, 0xb3e, 0xb15, 0xb3e, 0xb2e, 0xb4d, 0xb2a, 0xb3e, 0xb32, 0xb3e, 0xb16,
0xb30, 0xb1f, 0xb4c, 0xb2e, 0xb4d, 0x200c, 0xb15, 0xb3f, 0xb17, 0xb3e, 0xb32, 0xb3f,
0xb15, 0xb3f, 0xb28, 0xb38, 0xb3e, 0xb38, 0xb4d, 0x200c, 0xb32, 0xb3e, 0xb17, 0xb4b,
0xb38, 0xb4d, 0x200c, 0xb32, 0xb3f, 0xb2c, 0xb4d, 0xb30, 0xb47, 0xb2d, 0xb3f, 0xb32,
0xb4d, 0xb32, 0xb47, 0xb32, 0xb4b, 0xb2e, 0xb4d, 0x200c, 0xb32, 0xb41, 0xb06, 0xb23,
0xb4d, 0xb21, 0xb3e, 0xb32, 0xb41, 0xb2c, 0xb41, 0xb2e, 0xb4d, 0xb2c, 0xb3e, 0xb36,
0xb3f, 0xb32, 0xb41, 0xb38, 0xb3e, 0xb15, 0xb3e, 0xb2e, 0xb3e, 0xb32, 0xb3e, 0xb2c,
0xb4b, 0xb2e, 0xb3e, 0xb2a, 0xb41, 0xb24, 0xb4b, 0xb2e, 0xb47, 0xb38, 0xb47, 0xb30,
0xb41, 0xb2c, 0xb3e, 0xb2c, 0xb47, 0xb28, 0xb4d, 0x200c, 0xb2e, 0xb4b, 0xb17, 0xb3e,
0xb21, 0xb3f, 0xb36, 0xb41, 0xb2e, 0xb28, 0xb30, 0xb4b, 0xb2d, 0xb3f, 0xb06, 0xb28,
0xb3e, 0xb07, 0xb30, 0xb4b, 0xb2c, 0xb3f, 0xb1c, 0xb3e, 0xb2e, 0xb47, 0xb28, 0xb3e,
0xb28, 0xb3f, 0xb06, 0xb2e, 0xb3f, 0xb28, 0xb4c, 0xb15, 0xb3e, 0xb1a, 0xb4b, 0xb1f,
0xb05, 0xb09, 0xb17, 0xb3e, 0xb21, 0xb09, 0xb17, 0xb09, 0xb2a, 0xb4b, 0xb1f, 0xb4b,
0x2d, 0xb28, 0xb4b, 0xb2d, 0xb4b, 0xb38, 0xb3e, 0xb13, 0x20, 0xb1f, 0xb4b, 0xb2e,
0xb47, 0xb24, 0xb4d, 0xb30, 0xb3f, 0xb2a, 0xb4b, 0xb32, 0xb3f, 0xb1f, 0xb41, 0xb28,
0xb3f, 0xb38, 0xb4d, 0x200c, 0xb71, 0xb3f, 0xb23, 0xb4d, 0xb21, 0xb39, 0xb4b, 0xb0f,
0xb15, 0xb4d, 0xb06, 0xb21, 0xb3e, 0xb15, 0xb4d, 0xb06, 0xb19, 0xb4d, 0xb15, 0xb30,
0xb47, 0xb1c, 0xb4d, 0xb06, 0xb19, 0xb4d, 0xb17, 0xb41, 0xb07, 0xb32, 0xb3e, 0xb06,
0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb17, 0xb41, 0xb06, 0xb06, 0xb30, 0xb3e, 0xb17, 0xb41,
0xb06, 0xb28, 0xb3e, 0xb2c, 0xb41, 0xb0f, 0xb28, 0xb38, 0xb4d, 0x200c, 0x20, 0xb0f,
0xb30, 0xb3f, 0xb38, 0xb4d, 0x200c, 0xb15, 0xb3e, 0xb1f, 0xb3e, 0xb2e, 0xb3e, 0xb30,
0xb4d, 0xb15, 0xb3e, 0xb15, 0xb4b, 0xb30, 0xb4d, 0xb21, 0xb4b, 0xb2c, 0xb3e, 0xb1c,
0xb41, 0xb1c, 0xb41, 0xb07, 0xb32, 0xb3e, 0x20, 0xb30, 0xb3f, 0xb13, 0xb1c, 0xb3e,
0xb2e, 0xb47, 0xb23, 0xb4d, 0xb21, 0xb4b, 0xb1c, 0xb3e, 0xb30, 0xb3f, 0xb13, 0x20,
0xb17, 0xb3e, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb17, 0xb38, 0xb4d, 0x200c, 0xb38, 0xb32,
0xb4d, 0xb1f, 0xb3e, 0xb38, 0xb3e, 0xb28, 0xb4d, 0x200c, 0x20, 0xb1c, 0xb41, 0xb06,
0xb28, 0xb38, 0xb3e, 0xb28, 0xb4d, 0x200c, 0x20, 0xb32, 0xb41, 0xb07, 0xb38, 0xb4d,
0x200c, 0xb1f, 0xb41, 0xb15, 0xb41, 0xb2e, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb09, 0xb36,
0xb41, 0xb5f, 0xb3e, 0xb06, 0xb30, 0xb41, 0xb2c, 0xb3e, 0xb06, 0xb38, 0xb28, 0xb38,
0xb3f, 0xb05, 0xb28, 0xb4d, 0x200c, 0xb06, 0xb1f, 0xb3f, 0xb15, 0xb4b, 0xb15, 0xb3e,
0xb28, 0xb4d, 0xb2c, 0xb3e, 0xb39, 0xb3f, 0xb06, 0xb2c, 0xb3e, 0xb39, 0xb3f, 0xb06,
0x20, 0xb2c, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb47, 0xb30, 0xb3e, 0xb38,
0xb4d, 0xb2c, 0xb3e, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb21, 0xb4b, 0xb38, 0xb4d, 0xb2c,
0xb47, 0xb32, 0xb47, 0xb2e, 0xb2c, 0xb47, 0xb32, 0xb3f, 0xb1c, 0xb47, 0xb2c, 0xb4d,
0xb32, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x2d, 0xb38, 0xb3e, 0xb2c, 0xb32, 0xb4b, 0xb28,
0xb4d, 0xb2c, 0xb4b, 0xb06, 0x20, 0xb2d, 0xb3f, 0xb37, 0xb4d, 0xb1f, 0xb3e, 0xb2c,
0xb4b, 0xb17, 0xb4b, 0xb1f, 0xb3e, 0xb2c, 0xb4b, 0xb07, 0xb38, 0xb47, 0xb15, 0xb47,
0xb2e, 0xb4d, 0xb35, 0xb4d, 0xb30, 0xb3f, 0xb1c, 0xb4d, 0x20, 0xb2c, 0xb47, 0xb15,
0xb3e, 0xb2e, 0xb4d, 0xb2a, 0xb4b, 0x20, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb23, 0xb4d,
0xb21, 0xb47, 0xb15, 0xb3e, 0xb28, 0xb15, 0xb41, 0xb28, 0xb4d, 0xb15, 0xb3e, 0xb30,
0xb15, 0xb3e, 0xb38, 0xb4d, 0x200c, 0xb15, 0xb47, 0xb5f, 0xb47, 0xb28, 0xb4d, 0xb28,
0xb3f, 0xb15, 0xb3e, 0xb5f, 0xb2e, 0xb4d, 0xb5f, 0xb3e, 0xb28, 0xb4d, 0xb1a, 0xb3f,
0xb15, 0xb3e, 0xb17, 0xb4b, 0xb1a, 0xb3f, 0xb39, 0xb41, 0xb06, 0xb39, 0xb41, 0xb06,
0xb38, 0xb3f, 0xb09, 0xb21, 0xb3e, 0xb21, 0xb4d, 0x20, 0xb1c, 0xb41, 0xb06, 0xb30,
0xb47, 0xb1c, 0xb4d, 0xb15, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb3e, 0x20, 0xb30, 0xb3f,
0xb15, 0xb3e, 0xb15, 0xb4d, 0xb30, 0xb47, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb28, 0xb4d,
0xb15, 0xb41, 0xb07, 0xb2c, 0xb3e, 0xb15, 0xb41, 0xb30, 0xb3e, 0xb15, 0xb4b, 0xb21,
0xb3e, 0xb28, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb15, 0xb38, 0xb3e, 0xb2d, 0xb28, 0xb4d,
0xb21, 0xb38, 0xb28, 0xb4d, 0x200c, 0xb21, 0xb71, 0xb38, 0xb28, 0xb4d, 0x20, 0xb15,
0xb4d, 0xb30, 0xb3f, 0xb15, 0xb4d, 0xb21, 0xb47, 0xb28, 0xb2d, 0xb3f, 0xb30, 0xb4d,
0xb21, 0xb47, 0xb1f, 0xb4d, 0xb30, 0xb4b, 0xb07, 0xb1f, 0xb4d, 0xb21, 0xb4b, 0xb2e,
0xb3f, 0xb28, 0xb3f, 0xb15, 0xb3e, 0xb0f, 0xb21, 0xb2e, 0xb4b, 0xb28, 0xb1f, 0xb4b,
0xb28, 0xb4d, 0xb07, 0xb30, 0xb41, 0xb28, 0xb47, 0xb2a, 0xb47, 0xb0f, 0xb32, 0xb4d,
0x20, 0xb38, 0xb3e, 0xb32, 0xb2d, 0xb3e, 0xb21, 0xb4b, 0xb30, 0xb4d, 0xb2b, 0xb4b,
0xb30, 0xb4d, 0xb1f, 0xb4d, 0x20, 0xb28, 0xb47, 0xb32, 0xb38, 0xb28, 0xb4d, 0xb2b,
0xb4b, 0xb30, 0xb4d, 0xb1f, 0xb47, 0xb32, 0xb47, 0xb1c, 0xb3e, 0xb17, 0xb4d, 0xb32,
0xb3e, 0xb38, 0xb47, 0x20, 0xb2c, 0xb47, 0xb17, 0xb41, 0xb38, 0xb4d, 0x20, 0xb2c,
0xb47, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb1f, 0xb30,
0xb4d, 0xb15, 0xb4d, 0xb17, 0xb4d, 0xb30, 0xb47, 0xb28, 0xb3e, 0xb21, 0xb3e, 0xb17,
0xb41, 0xb06, 0xb21, 0xb47, 0xb32, 0xb4b, 0xb09, 0xb2a, 0xb47, 0xb17, 0xb41, 0xb06,
0xb24, 0xb47, 0xb2e, 0xb3e, 0xb32, 0xb3e, 0xb17, 0xb41, 0xb5f, 0xb3e, 0xb15, 0xb4d,
0xb71, 0xb3f, 0xb32, 0xb17, 0xb41, 0xb0f, 0xb28, 0xb3e, 0xb39, 0xb3e, 0xb32, 0xb3f,
0xb2b, 0xb4d, 0xb5f, 0xb3e, 0xb15, 0xb4d, 0xb38, 0xb4d, 0xb39, 0xb3e, 0xb2d, 0xb3e,
0xb28, 0xb3e, 0xb39, 0xb47, 0xb30, 0xb2e, 0xb4b, 0xb38, 0xb3f, 0xb32, 0xb4b, 0xb07,
0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2a, 0xb4b, 0xb32, 0xb3f, 0xb38,
0xb4d, 0xb15, 0xb4d, 0xb28, 0xb4b, 0xb15, 0xb4d, 0xb38, 0x2c, 0x20, 0xb07, 0xb23,
0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2e, 0xb3e, 0xb30, 0xb47, 0xb28, 0xb17,
0xb4b, 0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2a,
0xb47, 0xb1f, 0xb47, 0xb30, 0xb4d, 0xb38, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb4d, 0x2c,
0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb1f, 0xb47, 0xb32,
0xb4d, 0x20, 0xb38, 0xb3f, 0xb1f, 0xb3f, 0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21,
0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2d, 0xb47, 0xb2d, 0xb3e, 0xb5f, 0x2c, 0x20, 0xb07,
0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb2d, 0xb3f, 0xb28, 0xb38, 0xb47,
0xb28, 0xb4d, 0xb28, 0xb47, 0xb38, 0xb4d, 0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21,
0xb3f, 0xb06, 0xb28, 0xb3e, 0xb71, 0xb3f, 0xb28, 0xb3e, 0xb2e, 0xb3e, 0xb15, 0xb4d,
0x2c, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb07, 0xb28,
0xb41, 0xb2d, 0xb3f, 0xb15, 0xb4d, 0xb07, 0xb15, 0xb4d, 0xb35, 0xb3e, 0xb32, 0xb41,
0xb07, 0xb1f, 0xb4d, 0xb1c, 0xb3e, 0xb2e, 0xb3e, 0xb07, 0xb15, 0xb3e, 0xb1c, 0xb41,
0xb28, 0xb47, 0xb06, 0xb09, 0xb32, 0xb4c, 0xb07, 0xb38, 0xb2d, 0xb3f, 0xb32, 0xb4d,
0xb32, 0xb47, 0xb2e, 0xb4b, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb38, 0xb47, 0xb32, 0xb4b,
0x2c, 0x20, 0xb15, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb09, 0xb15, 0xb3f, 0xb15, 0xb47,
0xb15, 0xb4d, 0xb30, 0xb3e, 0xb32, 0xb47, 0xb23, 0xb4d, 0xb21, 0xb1c, 0xb3f, 0xb15,
0xb32, 0xb3e, 0x20, 0xb2a, 0xb3e, 0xb1c, 0xb4d, 0x200c, 0xb32, 0xb3f, 0xb2e, 0xb3e,
0xb32, 0xb38, 0xb4d, 0x20, 0xb0f, 0xb1e, 0xb4d, 0xb1c, 0xb47, 0xb32, 0xb47, 0xb38,
0xb4d, 0xb28, 0xb3f, 0xb2e, 0xb4d, 0xb28, 0x20, 0xb2a, 0xb4d, 0xb30, 0xb3f, 0xb28,
0xb4d, 0xb38, 0x2019, 0xb30, 0x20, 0xb15, 0xb4d, 0xb35, 0xb3e, 0xb1f, 0xb30, 0xb4d,
0xb2e, 0xb3e, 0xb38, 0xb3f, 0xb13, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb17, 0xb41, 0xb06,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb09, 0xb38, 0xb4d, 0x200c, 0xb2e, 0xb3e, 0xb30, 0xb3f,
0xb17, 0xb4b, 0xb1f, 0xb4d, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3f,
0xb15, 0xb4d, 0xb5f, 0xb41, 0xb2e, 0xb3e, 0xb1f, 0xb3e, 0xb2e, 0xb4b, 0xb30, 0xb4b,
0xb38, 0xb4d, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb1f, 0xb32, 0xb3e, 0xb28, 0xb4d, 0xb2e,
0xb47, 0xb28, 0xb4b, 0xb2e, 0xb3f, 0xb28, 0xb3f, 0xb2e, 0xb47, 0xb30, 0xb3f, 0xb21,
0xb3e, 0xb2e, 0xb3e, 0xb1f, 0xb32, 0xb3e, 0xb15, 0xb3e, 0xb1f, 0xb32, 0xb3e, 0xb2e,
0xb47, 0xb15, 0xb4d, 0xb38, 0xb3f, 0xb15, 0xb4b, 0x20, 0xb38, 0xb3f, 0xb1f, 0xb3f,
0xb2e, 0xb3f, 0xb15, 0xb4d, 0xb35, 0xb47, 0xb32, 0xb4b, 0xb28, 0xb4d, 0xb2e, 0xb3e,
0xb28, 0xb15, 0xb1f, 0xb4b, 0xb28, 0xb4d, 0xb2e, 0xb4b, 0xb28, 0xb1f, 0xb47, 0xb30,
0xb3f, 0xb0f, 0xb2e, 0xb23, 0xb4d, 0xb1f, 0xb47, 0xb2d, 0xb3f, 0xb21, 0xb3f, 0xb13,
0xb2e, 0xb4b, 0xb28, 0xb1f, 0xb38, 0xb47, 0xb30, 0xb30, 0xb3e, 0xb1f, 0xb4d, 0xb28,
0xb3e, 0xb38, 0xb3e, 0xb09, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb5f, 0xb4b, 0xb30,
0xb4d, 0xb15, 0xb4d, 0xb28, 0xb4b, 0xb2e, 0xb47, 0xb28, 0xb4b, 0xb30, 0xb4b, 0xb28,
0xb4d, 0xb28, 0xb3e, 0xb2c, 0xb47, 0xb09, 0xb32, 0xb3e, 0xb39, 0x2c, 0x20, 0xb09,
0xb24, 0xb4d, 0xb24, 0xb30, 0x20, 0xb21, 0xb3e, 0xb15, 0xb4b, 0xb1f, 0xb3e, 0xb15,
0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0x2c, 0x20, 0xb09, 0xb24, 0xb4d, 0xb24,
0xb30, 0x20, 0xb21, 0xb3e, 0xb15, 0xb4b, 0xb1f, 0xb3e, 0xb28, 0xb4d, 0xb5f, 0xb41,
0x20, 0xb38, 0xb3e, 0xb32, 0xb47, 0xb2e, 0xb4d, 0x2c, 0x20, 0xb09, 0xb24, 0xb4d,
0xb24, 0xb30, 0x20, 0xb21, 0xb3e, 0xb15, 0xb4b, 0xb1f, 0xb3e, 0xb28, 0xb41, 0xb09,
0xb15, 0xb4d, 0xb13, 0xb1c, 0xb3f, 0xb28, 0xb3e, 0xb17, 0xb3e, 0xb2a, 0xb3e, 0xb28,
0xb3e, 0xb2e, 0xb3e, 0xb2a, 0xb3e, 0xb30, 0xb3e, 0xb2e, 0xb3e, 0xb30, 0xb3f, 0xb2c,
0xb4b, 0xb2b, 0xb4b, 0xb07, 0xb28, 0xb3f, 0xb15, 0xb4d, 0xb38, 0xb2a, 0xb4b, 0xb30,
0xb4d, 0xb1f, 0xb4d, 0x2d, 0xb0f, 0xb5f, 0xb41, 0x2d, 0xb2a, 0xb4d, 0xb30, 0xb3f,
0xb28, 0xb4d, 0xb38, 0xb2a, 0xb4b, 0xb30, 0xb4d, 0xb1f, 0xb4d, 0x20, 0xb05, 0xb2b,
0xb4d, 0x20, 0xb38, 0xb4d, 0xb2a, 0xb47, 0xb28, 0xb4d, 0xb2a, 0xb4b, 0xb30, 0xb4d,
0xb1f, 0xb4b, 0x20, 0xb2d, 0xb47, 0xb32, 0xb4b, 0xb2a, 0xb41, 0xb0f, 0xb30, 0xb4d,
0xb24, 0xb4b, 0x20, 0xb30, 0xb3f, 0xb15, 0xb4b, 0xb2a, 0xb41, 0xb23, 0xb4d, 0xb1f,
0xb3e, 0x20, 0xb0f, 0xb30, 0xb3f, 0xb28, 0xb3e, 0xb38, 0xb4d, 0x200c, 0xb30, 0xb3e,
0xb28, 0xb15, 0xb3f, 0xb28, 0xb4d, 0x20, 0xb07, 0xb28, 0xb32, 0xb47, 0xb1f, 0xb4d,
0xb30, 0xb47, 0xb38, 0xb3f, 0xb2b, 0xb3f, 0xb30, 0xb47, 0xb17, 0xb3f, 0xb28, 0xb3e,
0xb30, 0xb3f, 0xb38, 0xb4b, 0xb32, 0xb41, 0xb1f, 0xb47, 0xb30, 0xb3f, 0xb13, 0x20,
0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb19, 0xb4d, 0xb15, 0xb4b, 0xb38, 0xb3e, 0xb23, 0xb4d,
0xb1f, 0xb3e, 0xb30, 0xb47, 0xb2e, 0xb4d, 0x200c, 0xb38, 0xb3e, 0xb23, 0xb4d, 0xb1f,
0xb3f, 0xb06, 0xb17, 0xb4b, 0xb38, 0xb3e, 0xb23, 0xb4d, 0xb1f, 0xb4b, 0x20, 0xb21,
0xb4b, 0xb2e, 0xb3f, 0xb19, 0xb4d, 0xb17, 0xb4b, 0xb38, 0xb3e, 0xb13, 0x20, 0xb2a,
0xb3e, 0xb13, 0xb32, 0xb4b, 0xb07, 0xb1f, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0xb35,
0xb4b, 0xb30, 0xb1f, 0xb4b, 0xb30, 0xb2e, 0xb3f, 0xb1f, 0xb4d, 0xb38, 0xb3f, 0xb1f,
0xb15, 0xb3e, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x2e, 0x20, 0xb2c, 0xb3e,
0xb30, 0xb4d, 0xb25, 0xb47, 0xb32, 0xb47, 0xb2e, 0xb3f, 0xb38, 0xb47, 0xb23, 0xb4d,
0xb1f, 0xb4d, 0x2e, 0x20, 0xb1c, 0xb28, 0xb38, 0xb4d, 0xb38, 0xb47, 0xb23, 0xb4d,
0xb1f, 0xb4d, 0x20, 0xb15, 0xb3f, 0xb1f, 0xb4d, 0xb38, 0xb4d, 0xb38, 0xb47, 0xb23,
0xb4d, 0xb1f, 0xb4d, 0x2e, 0x20, 0xb32, 0xb41, 0xb38, 0xb3f, 0xb06, 0xb38, 0xb47,
0xb23, 0xb4d, 0xb1f, 0xb4d, 0x2e, 0x20, 0xb25, 0xb4b, 0xb2e, 0xb3e, 0xb38, 0xb4d,
0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x2e, 0x20, 0xb2d, 0xb3f, 0xb28, 0xb38,
0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0xb38, 0xb41, 0xb07, 0xb2b, 0xb4d, 0xb1f, 0xb4d,
0x20, 0xb15, 0xb30, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0xb1f, 0xb47, 0xb17, 0xb41,
0xb38, 0xb3f, 0xb17, 0xb3e, 0xb32, 0xb2a, 0xb3e, 0xb25, 0xb41, 0xb32, 0xb47, 0xb24,
0xb3f, 0xb1c, 0xb41, 0xb06, 0xb28, 0xb3e, 0xb1f, 0xb4b, 0xb30, 0xb4b, 0xb23, 0xb4d,
0xb1f, 0xb4b, 0xb1f, 0xb4b, 0xb30, 0xb1f, 0xb4b, 0xb32, 0xb3e, 0xb2d, 0xb3e, 0xb19,
0xb4d, 0xb15, 0xb41, 0xb2d, 0xb30, 0xb4d, 0xb39, 0xb4d, 0xb35, 0xb3e, 0xb07, 0xb1f,
0xb39, 0xb30, 0xb4d, 0xb38, 0xb4d, 0xb71, 0xb3f, 0xb28, 0xb3f, 0xb2a, 0xb47, 0xb17,
0xb4d, 0xb5f, 0xb3e, 0xb15, 0xb41, 0xb1f, 0xb3e, 0xb1f, 0xb4d, 0xb21, 0xb47, 0xb2d,
0xb3f, 0xb38, 0xb4d, 0x200c, 0xb21, 0xb41, 0xb2e, 0xb4b, 0xb23, 0xb4d, 0xb1f, 0x20,
0xb21, 0xb3f, 0x2018, 0xb09, 0xb30, 0xb2d, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb2e,
0xb3e, 0xb15, 0xb4d, 0xb71, 0xb47, 0xb30, 0xb40, 0xb2e, 0xb3e, 0xb71, 0xb38, 0xb28,
0xb4d, 0xb2e, 0xb4d, 0xb5f, 0xb3e, 0xb15, 0xb4d, 0x200c, 0xb2e, 0xb41, 0xb30, 0xb4d,
0xb21, 0xb4b, 0xb2a, 0xb3e, 0xb32, 0xb2e, 0xb30, 0xb4d, 0x200c, 0xb30, 0xb4b, 0xb25,
0xb47, 0xb30, 0xb3e, 0xb38, 0xb4b, 0xb71, 0xb3e, 0xb1f, 0xb4d, 0xb30, 0xb4b, 0xb32,
0xb4d, 0xb2d, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0x200c, 0xb32, 0xb19,
0xb4d, 0xb17, 0xb5f, 0xb47, 0xb06, 0xb30, 0xb2c, 0xb5f, 0xb47, 0xb28, 0xb4d, 0xb0f,
0xb21, 0xb47, 0xb28, 0xb4d, 0x200c, 0xb05, 0xb32, 0xb2e, 0xb3e, 0xb1f, 0xb3f, 0xb05,
0xb2e, 0xb4d, 0xb2e, 0xb3e, 0xb28, 0xb06, 0xb28, 0xb3e, 0xb21, 0xb5f, 0xb3e, 0xb30,
0xb4d, 0xb06, 0xb15, 0xb1f, 0xb3e, 0xb09, 0xb06, 0xb15, 0xb1f, 0xb4b, 0xb2c, 0xb4d,
0x200c, 0xb06, 0xb36, 0xb4d, 0x200c, 0xb17, 0xb3e, 0xb2c, 0xb4b, 0xb1f, 0xb4d, 0x200c,
0xb05, 0xb24, 0xb3f, 0xb30, 0xb3e, 0xb09, 0xb2c, 0xb3e, 0xb17, 0xb26, 0xb3e, 0xb26,
0xb4d, 0x200c, 0xb2c, 0xb3e, 0xb39, 0xb3e, 0xb30, 0xb3f, 0xb28, 0xb4d, 0xb2c, 0xb4d,
0xb5f, 0xb3e, 0xb19, 0xb4d, 0xb17, 0xb15, 0xb15, 0xb4d, 0x200c, 0xb2c, 0xb3e, 0xb30,
0xb28, 0xb3e, 0xb09, 0xb32, 0xb4d, 0xb2c, 0xb40, 0xb30, 0xb41, 0xb1f, 0xb4d, 0x200c,
0xb2c, 0xb3f, 0xb36, 0xb15, 0xb47, 0xb15, 0xb4d, 0x200c, 0xb2c, 0xb4d, 0xb30, 0xb41,
0xb28, 0xb47, 0xb07, 0xb1a, 0xb3f, 0xb1f, 0xb3e, 0xb1a, 0xb4b, 0xb07, 0xb2c, 0xb3e,
0xb32, 0xb38, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb15, 0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb4b,
0xb21, 0xb3e, 0xb2e, 0xb3e, 0xb38, 0xb15, 0xb38, 0xb4d, 0x200c, 0xb22, 0xb3e, 0xb15,
0xb3e, 0xb26, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb40, 0xb26, 0xb41, 0xb2c, 0xb3e, 0xb07,
0xb26, 0xb41, 0xb36, 0xb3e, 0xb28, 0xb2c, 0xb47, 0xb2b, 0xb3e, 0xb2e, 0xb3e, 0xb17,
0xb41, 0xb37, 0xb4d, 0xb1f, 0xb3e, 0xb17, 0xb3e, 0xb1c, 0xb3e, 0xb39, 0xb47, 0xb2c,
0xb4d, 0xb30, 0xb28, 0xb4d, 0x200c, 0xb39, 0xb4b, 0x20, 0xb1a, 0xb3f, 0x20, 0xb2e,
0xb3f, 0xb28, 0xb4d, 0x200c, 0x20, 0xb38, 0xb3f, 0xb1f, 0xb3f, 0xb39, 0xb02, 0x20,
0xb15, 0xb02, 0xb39, 0xb4b, 0xb2d, 0xb21, 0xb4d, 0x200c, 0xb07, 0xb30, 0xb15, 0xb41,
0xb1f, 0xb38, 0xb4d, 0xb15, 0xb1c, 0xb3e, 0xb15, 0xb30, 0xb4d, 0xb24, 0xb4d, 0xb24,
0xb3e, 0xb1c, 0xb5f, 0xb2a, 0xb41, 0xb30, 0xb3e, 0xb1c, 0xb47, 0xb30, 0xb41, 0xb1c,
0xb47, 0xb32, 0xb2e, 0xb15, 0xb3e, 0xb2c, 0xb41, 0xb32, 0xb15, 0xb3e, 0xb2e, 0xb1a,
0xb3e, 0xb1f, 0xb15, 0xb3e, 0xb15, 0xb30, 0xb3e, 0xb1a, 0xb3f, 0xb15, 0xb3e, 0xb20,
0xb2e, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb41, 0xb16, 0xb3e, 0xb28, 0xb21, 0xb4d, 0xb5f,
0xb3e, 0xb17, 0xb3e, 0xb15, 0xb4b, 0xb32, 0xb15, 0xb3e, 0xb24, 0xb3e, 0xb15, 0xb4d,
0xb30, 0xb3e, 0xb38, 0xb28, 0xb4b, 0xb5f, 0xb3e, 0xb30, 0xb38, 0xb4d, 0xb15, 0xb15,
0xb4d, 0xb71, 0xb3e, 0xb32, 0xb3e, 0xb32, 0xb2e, 0xb4d, 0xb2a, 0xb41, 0xb30, 0xb15,
0xb41, 0xb1a, 0xb3f, 0xb02, 0xb15, 0xb41, 0xb0f, 0xb24, 0xb2e, 0xb3e, 0xb15, 0xb3e,
0xb09, 0xb2e, 0xb3e, 0xb17, 0xb3e, 0xb21, 0xb3e, 0xb28, 0xb4d, 0xb2e, 0xb3e, 0xb15,
0xb3e, 0xb38, 0xb3e, 0xb30, 0xb4d, 0x200c, 0xb2e, 0xb3e, 0xb28, 0xb3f, 0xb32, 0xb3e,
0xb2e, 0xb38, 0xb4d, 0xb15, 0xb3e, 0xb1f, 0xb4d, 0x200c, 0xb28, 0xb3f, 0xb15, 0xb4b,
0xb38, 0xb3f, 0xb06, 0xb28, 0xb4b, 0xb2d, 0xb4b, 0xb15, 0xb41, 0xb1c, 0xb28, 0xb47,
0xb1f, 0xb38, 0xb4d, 0xb15, 0xb28, 0xb4b, 0xb2d, 0xb4b, 0xb38, 0xb3f, 0xb2c, 0xb3f,
0xb30, 0xb38, 0xb4d, 0xb15, 0xb13, 0xb2e, 0xb38, 0xb4d, 0xb15, 0xb13, 0xb30, 0xb3e,
0xb32, 0xb4d, 0x200c, 0xb2b, 0xb28, 0xb4b, 0xb2e, 0xb4d, 0x200c, 0x20, 0xb2a, 0xb47,
0xb28, 0xb39, 0xb2a, 0xb4b, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb06, 0xb28, 0xb3e, 0xb15,
0xb4d, 0x200c, 0xb2a, 0xb4b, 0xb5f, 0xb19, 0xb4d, 0xb17, 0xb5f, 0xb3e, 0xb19, 0xb4d,
0xb17, 0xb15, 0xb24, 0xb3e, 0xb30, 0xb4d, 0xb15, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb28,
0xb47, 0xb15, 0xb40, 0xb1c, 0xb3f, 0xb32, 0xb4b, 0xb30, 0xb4d, 0xb21, 0xb3e, 0xb30,
0xb3f, 0xb06, 0xb26, 0xb38, 0xb16, 0xb3e, 0xb32, 0xb3f, 0xb28, 0xb4d, 0xb38, 0xb2e,
0xb30, 0xb15, 0xb28, 0xb4d, 0xb26, 0xb38, 0xb3f, 0xb13, 0xb32, 0xb38, 0xb02, 0xb18,
0xb3e, 0xb07, 0xb38, 0xb3f, 0xb19, 0xb4d, 0xb17, 0xb3e, 0xb2a, 0xb41, 0xb30, 0xb4d,
0x200c, 0xb38, 0xb4d, 0xb30, 0xb47, 0xb21, 0xb28, 0xb47, 0xb15, 0xb4b, 0xb32, 0xb5f,
0xb2e, 0xb38, 0xb4d, 0xb15, 0xb24, 0xb3e, 0xb07, 0xb2a, 0xb47, 0xb07, 0xb24, 0xb3e,
0xb36, 0xb15, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb1f, 0xb3f, 0xb2c, 0xb3f, 0xb32, 0xb3f,
0xb38, 0xb3f, 0xb24, 0xb47, 0xb39, 0xb47, 0xb30, 0xb3e, 0xb28, 0xb4d, 0xb25, 0xb3f,
0xb2e, 0xb4d, 0xb2a, 0xb41, 0xb1f, 0xb4b, 0xb15, 0xb3f, 0xb13, 0xb1f, 0xb4b, 0xb2e,
0xb38, 0xb4d, 0xb15, 0xb09, 0xb32, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb2c, 0xb3e, 0xb1f,
0xb30, 0xb4d, 0x200c, 0xb09, 0xb30, 0xb41, 0xb2e, 0xb15, 0xb3f, 0xb5f, 0xb41, 0xb37,
0xb4d, 0xb1f, 0x2d, 0xb28, 0xb47, 0xb30, 0xb3e, 0xb2d, 0xb3f, 0xb0f, 0xb23, 0xb4d,
0xb1f, 0xb3f, 0xb0f, 0xb28, 0xb4d, 0x200c, 0xb2d, 0xb4d, 0xb32, 0xb3e, 0xb21, 0xb3f,
0xb2d, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0xb5f, 0xb3e, 0xb15, 0xb41,
0xb1f, 0xb38, 0xb4d, 0xb15, 0xb5f, 0xb3e, 0xb19, 0xb4d, 0xb17, 0xb41, 0xb28, 0xb4d,
0x200c, 0xb5f, 0xb47, 0xb15, 0xb3e, 0xb1f, 0xb47, 0xb30, 0xb3f, 0xb28, 0xb2c, 0xb30,
0xb4d, 0xb17, 0xb4d, 0xb5f, 0xb47, 0xb30, 0xb47, 0xb2d, 0xb3e, 0xb28, 0xb4d, 0xb06,
0xb1c, 0xb4b, 0xb30, 0xb47, 0xb38, 0xb4d, 0xb2c, 0xb30, 0xb4d, 0xb2e, 0xb41, 0xb21,
0xb3e, 0xb15, 0xb3e, 0xb28, 0xb3e, 0xb30, 0xb47, 0xb15, 0xb47, 0xb2a, 0xb4d, 0x200c,
0x20, 0xb2d, 0xb30, 0xb4d, 0xb26, 0xb47, 0xb2b, 0xb3e, 0xb30, 0xb4b, 0xb0f, 0xb2e,
0xb3e, 0xb21, 0xb47, 0xb07, 0xb30, 0xb3e, 0xb30, 0xb47, 0xb5f, 0xb15, 0xb1c, 0xb3e,
0xb2d, 0xb3f, 0xb15, 0xb4d, 0xb26, 0xb15, 0xb4d, 0xb37, 0xb3f, 0xb23, 0x20, 0xb1c,
0xb30, 0xb4d, 0xb1c, 0xb3f, 0xb06, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb4d, 0x200c,
0x20, 0xb39, 0xb47, 0xb32, 0xb3f, 0xb28, 0xb3e, 0xb37, 0xb4d, 0xb1f, 0xb3e, 0xb28,
0xb32, 0xb3f, 0xb06, 0xb21, 0xb3f, 0xb32, 0xb47, 0xb21, 0xb4d, 0x200c, 0xb2c, 0xb4d,
0xb30, 0xb3f, 0xb38, 0xb2c, 0xb47, 0xb28, 0xb4d, 0x200c, 0xb2c, 0xb4d, 0xb30, 0xb4b,
0xb15, 0xb47, 0xb28, 0xb4d, 0x200c, 0x20, 0xb39, 0xb3f, 0xb32, 0xb4d, 0x200c, 0xb21,
0xb3e, 0xb30, 0xb71, 0xb3f, 0xb28, 0xb4d, 0x200c, 0xb5f, 0xb41, 0xb15, 0xb32, 0xb3e,
0xb39, 0xb4b, 0xb2c, 0xb3e, 0xb30, 0xb4d, 0xb1f, 0xb4d, 0x200c, 0xb32, 0xb3f, 0xb23,
0xb4d, 0xb21, 0xb47, 0xb2e, 0xb4d, 0xb5f, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb32, 0xb30,
0xb4d, 0xb21, 0x20, 0xb39, 0xb4b, 0xb71, 0xb47, 0xb2e, 0xb47, 0xb32, 0xb2c, 0xb4b,
0xb30, 0xb4d, 0xb23, 0xb4d, 0xb23, 0xb2a, 0xb30, 0xb4d, 0xb25, 0xb4d, 0x200c, 0xb38,
0xb3f, 0xb21, 0xb28, 0xb40, 0xb06, 0xb2e, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47,
0xb21, 0xb3e, 0xb2e, 0xb4d, 0xb06, 0xb28, 0xb21, 0xb4b, 0xb30, 0xb3e, 0xb06, 0xb37,
0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb3e, 0xb16, 0xb3e, 0xb28, 0xb4d, 0xb0f, 0xb25, 0xb47,
0xb28, 0xb4d, 0xb38, 0xb2c, 0xb47, 0xb32, 0xb17, 0xb4d, 0xb30, 0xb47, 0xb21, 0xb47,
0xb2c, 0xb30, 0xb4d, 0xb32, 0xb3f, 0xb28, 0xb4d, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb1f,
0xb3f, 0xb38, 0xb32, 0xb3e, 0xb2d, 0xb3e, 0xb2c, 0xb4d, 0xb30, 0xb38, 0xb32, 0xb4d,
0xb38, 0xb4d, 0xb2c, 0xb41, 0xb1a, 0xb3e, 0xb30, 0xb47, 0xb37, 0xb4d, 0xb1f, 0xb4d,
0xb2c, 0xb41, 0xb21, 0xb3e, 0xb2a, 0xb47, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb2c, 0xb41,
0xb38, 0xb3f, 0xb28, 0xb17, 0xb47, 0xb28, 0xb4d, 0xb1a, 0xb3f, 0xb38, 0xb3f, 0xb28,
0xb4c, 0xb15, 0xb4b, 0xb2a, 0xb47, 0xb28, 0xb39, 0xb3e, 0xb17, 0xb47, 0xb28, 0xb4d,
0xb21, 0xb2c, 0xb32, 0xb3f, 0xb28, 0xb4d, 0xb1c, 0xb3f, 0xb2c, 0xb4d, 0xb30, 0xb3e,
0xb32, 0xb1f, 0xb30, 0xb4d, 0x200c, 0xb17, 0xb41, 0xb0f, 0xb30, 0xb28, 0xb38, 0xb47,
0xb39, 0xb47, 0xb32, 0xb38, 0xb3f, 0xb28, 0xb15, 0xb3f, 0xb07, 0xb38, 0xb32, 0xb47,
0x20, 0xb05, 0xb2b, 0xb4d, 0x20, 0xb2e, 0xb47, 0xb28, 0xb4d, 0xb07, 0xb38, 0xb4d,
0xb24, 0xb3e, 0xb28, 0xb2c, 0xb41, 0xb32, 0xb1c, 0xb30, 0xb4d, 0xb38, 0xb3f, 0xb15,
0xb3e, 0xb32, 0xb3f, 0xb28, 0xb3f, 0xb28, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d,
0xb15, 0xb3f, 0xb30, 0xb4b, 0xb2d, 0xb15, 0xb3f, 0xb0f, 0xb2d, 0xb4d, 0xb32, 0xb3f,
0xb38, 0xb2c, 0xb4b, 0xb28, 0xb4d, 0xb32, 0xb1c, 0xb41, 0xb2c, 0xb4d, 0xb32, 0xb1c,
0xb3e, 0xb28, 0xb4d, 0xb32, 0xb23, 0xb4d, 0xb21, 0xb28, 0xb4d, 0xb32, 0xb15, 0xb4d,
0xb38, 0xb2e, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb2e, 0xb3e, 0xb21, 0xb4d, 0xb30, 0xb3f,
0xb21, 0xb4d, 0xb2e, 0xb3e, 0xb32, 0xb1f, 0xb3e, 0xb2e, 0xb3e, 0xb30, 0xb3f, 0xb0f,
0xb39, 0xb3e, 0xb2e, 0xb28, 0xb4d, 0xb2e, 0xb3f, 0xb28, 0xb38, 0xb4d, 0xb15, 0xb2e,
0xb4b, 0xb28, 0xb3e, 0xb15, 0xb4b, 0xb2e, 0xb38, 0xb4d, 0xb15, 0xb4b, 0xb13, 0xb38,
0xb32, 0xb4b, 0xb2a, 0xb47, 0xb30, 0xb3f, 0xb38, 0xb4d, 0xb2a, 0xb4b, 0xb21, 0xb17,
0xb4b, 0xb30, 0xb3f, 0xb15, 0xb3e, 0xb2a, 0xb4d, 0xb30, 0xb3e, 0xb17, 0xb4d, 0xb30,
0xb3f, 0xb17, 0xb3e, 0xb30, 0xb4b, 0xb2e, 0xb4d, 0xb38, 0xb3e, 0xb2e, 0xb3e, 0xb30,
0xb3e, 0xb38, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb30, 0xb3f, 0xb28, 0xb4b,
0xb38, 0xb3e, 0xb30, 0xb3e, 0xb1c, 0xb47, 0xb2d, 0xb4b, 0xb38, 0xb3e, 0xb30, 0xb3e,
0xb1f, 0xb4b, 0xb2d, 0xb4d, 0xb38, 0xb3f, 0xb2e, 0xb2b, 0xb47, 0xb30, 0xb4b, 0xb2a,
0xb4b, 0xb32, 0xb4d, 0xb38, 0xb4d, 0xb15, 0xb4b, 0xb2a, 0xb1c, 0xb47, 0xb38, 0xb4b,
0xb2b, 0xb3f, 0xb06, 0xb37, 0xb4d, 0xb1f, 0xb15, 0xb39, 0xb4b, 0xb2e, 0xb4d, 0x200c,
0xb1f, 0xb3e, 0xb32, 0xb3f, 0xb28, 0xb4d, 0xb28, 0xb1f, 0xb3f, 0xb30, 0xb3e, 0xb28,
0xb47, 0xb5f, 0xb41, 0xb32, 0xb5f, 0xb3e, 0xb28, 0xb4b, 0xb2d, 0xb38, 0xb4d, 0xb15,
0xb2d, 0xb3e, 0xb21, 0xb41, 0xb1c, 0xb2d, 0xb3e, 0xb1f, 0xb3f, 0xb15, 0xb3e, 0xb28,
0xb4d, 0xb2d, 0xb3f, 0xb0f, 0xb28, 0xb3e, 0xb2d, 0xb3f, 0xb32, 0xb28, 0xb3f, 0xb09,
0xb38, 0xb4d, 0xb2d, 0xb4b, 0xb32, 0xb17, 0xb4b, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21,
0xb4d, 0xb71, 0xb3e, 0xb30, 0xb38, 0xb71, 0xb3e, 0xb1c, 0xb3e, 0xb17, 0xb4d, 0xb30,
0xb47, 0xb2c, 0xb4d, 0xb1c, 0xb41, 0xb30, 0xb3f, 0xb15, 0xb4d, 0xb06, 0xb23, 0xb4d,
0xb1f, 0xb3e, 0xb28, 0xb3e, 0xb28, 0xb3e, 0xb30, 0xb3f, 0xb2d, 0xb4b, 0xb1a, 0xb3e,
0xb17, 0xb4b, 0xb38, 0xb4d, 0x200c, 0xb16, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb1f,
0xb2e, 0xb3e, 0xb38, 0xb15, 0xb4b, 0xb15, 0xb4b, 0xb38, 0xb4d, 0x200c, 0xb15, 0xb4b,
0xb2e, 0xb4b, 0xb30, 0xb4b, 0xb15, 0xb47, 0xb30, 0xb41, 0xb17, 0xb47, 0xb32, 0xb47,
0xb28, 0xb2e, 0xb3e, 0xb39, 0xb47, 0xb2e, 0xb3e, 0xb33, 0xb26, 0xb4d, 0xb71, 0xb40,
0xb2a, 0xb2e, 0xb4c, 0xb30, 0xb3f, 0xb38, 0xb38, 0xb4d, 0xb2e, 0xb3e, 0xb5f, 0xb4b,
0xb1f, 0xb47, 0xb30, 0xb3f, 0xb5f, 0xb41, 0xb28, 0xb3f, 0xb05, 0xb28, 0xb4d, 0x200c,
0xb06, 0xb2a, 0xb3f, 0xb06, 0xb05, 0xb15, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb2c,
0xb4c, 0xb17, 0xb47, 0xb28, 0xb4d, 0x200c, 0xb2d, 0xb3f, 0xb32, 0xb4d, 0xb32, 0xb47,
0xb1a, 0xb3e, 0xb25, 0xb3e, 0xb2e, 0xb4d, 0x200c, 0xb1a, 0xb42, 0xb15, 0xb4d, 0x200c,
0xb07, 0xb37, 0xb4d, 0xb1f, 0xb30, 0xb4d, 0x200c, 0xb07, 0xb2b, 0xb47, 0xb1f, 0xb4d,
0x200c, 0xb2b, 0xb3e, 0xb15, 0xb3e, 0xb13, 0xb2b, 0xb4b, 0xb2b, 0xb3f, 0xb1c, 0xb3f,
0xb2b, 0xb41, 0xb28, 0xb3e, 0xb2b, 0xb41, 0xb1f, 0xb3f, 0xb17, 0xb3e, 0xb32, 0xb3e,
0xb2a, 0xb3e, 0xb17, 0xb4b, 0xb38, 0xb17, 0xb3e, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb5f,
0xb30, 0xb4d, 0x200c, 0xb17, 0xb41, 0xb06, 0xb21, 0xb3e, 0xb32, 0xb15, 0xb3e, 0xb28,
0xb3e, 0xb32, 0xb39, 0xb4b, 0xb28, 0xb4b, 0xb32, 0xb41, 0xb32, 0xb41, 0xb0f, 0xb23,
0xb4d, 0xb21, 0xb47, 0xb30, 0xb2c, 0xb41, 0xb30, 0xb3f, 0xb15, 0xb3f, 0xb30, 0xb3f,
0xb24, 0xb3f, 0xb2e, 0xb3e, 0xb1f, 0xb3f, 0xb15, 0xb4b, 0xb38, 0xb30, 0xb47, 0xb07,
0xb15, 0xb4d, 0xb71, 0xb3e, 0xb1c, 0xb3e, 0xb32, 0xb47, 0xb07, 0xb28, 0xb4d, 0xb2e,
0xb3e, 0xb1c, 0xb41, 0xb30, 0xb4b, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb15, 0xb4d, 0xb71,
0xb47, 0xb38, 0xb3e, 0xb38, 0xb4d, 0x200c, 0xb2e, 0xb3f, 0xb21, 0xb4d, 0x200c, 0xb71,
0xb47, 0xb28, 0xb3e, 0xb09, 0xb30, 0xb41, 0xb28, 0xb3f, 0xb5f, 0xb42, 0xb28, 0xb30,
0xb2b, 0xb4b, 0xb15, 0xb4d, 0x200c, 0xb28, 0xb4c, 0xb2e, 0xb3f, 0xb5f, 0xb2a, 0xb3e,
0xb17, 0xb4b, 0x20, 0xb2a, 0xb3e, 0xb17, 0xb4b, 0xb2a, 0xb3e, 0xb32, 0xb3e, 0xb09,
0xb2a, 0xb3f, 0xb1f, 0xb15, 0xb3e, 0xb30, 0xb3f, 0xb28, 0xb4d, 0x200c, 0xb2a, 0xb4b,
0xb39, 0xb28, 0xb2a, 0xb47, 0xb07, 0xb2a, 0xb4b, 0xb30, 0xb4d, 0xb1f, 0xb4d, 0x200c,
0x20, 0xb2e, 0xb4b, 0xb30, 0xb47, 0xb38, 0xb2c, 0xb3f, 0xb30, 0xb3e, 0xb30, 0xb4b,
0xb1f, 0xb4b, 0xb19, 0xb4d, 0xb17, 0xb3e, 0xb38, 0xb3e, 0xb07, 0xb2a, 0xb3e, 0xb28,
0xb4d, 0xb24, 0xb3e, 0xb39, 0xb3f, 0xb24, 0xb3f, 0xb24, 0xb3e, 0xb30, 0xb71, 0xb3e,
0xb1f, 0xb4b, 0xb19, 0xb4d, 0xb17, 0xb3e, 0xb1f, 0xb3e, 0xb2a, 0xb41, 0xb71, 0xb47,
0xb15, 0xb4d, 0xb71, 0xb3e, 0xb32, 0xb3f, 0xb38, 0xb4d, 0x200c, 0x41c, 0x4d5, 0x441,
0x43a, 0x443, 0x44b, 0x4b, 0xf2, 0x72, 0x73, 0x6f, 0x75, 0x52, 0x65, 0x70,
0xfa, 0x62, 0x6c, 0x6b, 0x61, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69,
0x63, 0x61, 0x6e, 0x61, 0x48, 0x65, 0x72, 0xfa, 0x73, 0x61, 0x6c, 0xe8,
0x6d, 0x627, 0x628, 0x64a, 0x62c, 0x627, 0x646, 0x627, 0x6a9, 0x631, 0x627, 0x627,
0x62f, 0x64a, 0x633, 0x20, 0x627, 0x628, 0x627, 0x628, 0x627, 0x627, 0x644, 0x62c,
0x64a, 0x64a, 0x631, 0x632, 0x627, 0x633, 0x645, 0x627, 0x631, 0x647, 0x628, 0x627,
0x645, 0x6cc, 0x6a9, 0x648, 0x628, 0x627, 0x646, 0x6ab, 0x648, 0x64a, 0x628, 0x633,
0x627, 0x624, 0x628, 0x644, 0x646, 0x67c, 0x627, 0x64a, 0x631, 0x628, 0x631, 0x627,
0x632, 0x627, 0x648, 0x64a, 0x644, 0x628, 0x62c, 0x648, 0x6ab, 0x648, 0x631, 0x627,
0x642, 0x627, 0x647, 0x631, 0x647, 0x6a9, 0x627, 0x633, 0x627, 0x628, 0x644, 0x627,
0x646, 0x6a9, 0x627, 0x633, 0x64a, 0x648, 0x67c, 0x627, 0x6a9, 0x648, 0x646, 0x6a9,
0x631, 0x64a, 0x689, 0x627, 0x6a9, 0x627, 0x631, 0x62c, 0x628, 0x648, 0x62a, 0x64a,
0x627, 0x644, 0x6cc, 0x648, 0x646, 0x641, 0x631, 0x6cc, 0x67c, 0x648, 0x646, 0x6ab,
0x627, 0x628, 0x631, 0x648, 0x646, 0x647, 0x631, 0x627, 0x631, 0x6d0, 0x62c, 0x648,
0x647, 0x627, 0x646, 0x633, 0x628, 0x631, 0x6ab, 0x6a9, 0x645, 0x67e, 0x627, 0x644,
0x647, 0x62e, 0x631, 0x62a, 0x648, 0x645, 0x6a9, 0x64a, 0x6af, 0x627, 0x644, 0x64a,
0x6a9, 0x64a, 0x646, 0x634, 0x627, 0x633, 0x627, 0x644, 0x627, 0x6af, 0x648, 0x633,
0x644, 0x6cc, 0x628, 0x631, 0x6cc, 0x644, 0x644, 0x648, 0x648, 0x645, 0x644, 0x648,
0x646, 0x62f, 0x647, 0x644, 0x628, 0x648, 0x628, 0x627, 0x634, 0x64a, 0x645, 0x627,
0x67e, 0x648, 0x67c, 0x648, 0x645, 0x633, 0x64a, 0x631, 0x648, 0x645, 0x628, 0x627,
0x628, 0x627, 0x646, 0x6d0, 0x645, 0x648, 0x6af, 0x62f, 0x64a, 0x634, 0x648, 0x645,
0x648, 0x646, 0x631, 0x648, 0x641, 0x6cc, 0x627, 0x646, 0x627, 0x64a, 0x631, 0x648,
0x628, 0x64a, 0x646, 0x62c, 0x627, 0x645, 0x6cc, 0x646, 0x627, 0x646, 0x6cc, 0x645,
0x64a, 0x646, 0x648, 0x6a9, 0x648, 0x686, 0x67c, 0x627, 0x648, 0x6ab, 0x648, 0x689,
0x648, 0x6ab, 0x648, 0x67e, 0x648, 0x631, 0x67c, 0x648, 0x20, 0x2d, 0x20, 0x646,
0x648, 0x648, 0x633, 0x627, 0x648, 0x20, 0x67c, 0x648, 0x645, 0x62a, 0x631, 0x64a,
0x67e, 0x648, 0x644, 0x64a, 0x648, 0x6cc, 0x646, 0x647, 0x648, 0x6a9, 0x627, 0x62f,
0x627, 0x6a9, 0x627, 0x6cc, 0x646, 0x6a9, 0x631, 0x6cc, 0x62c, 0x627, 0x646, 0x6ab,
0x6cc, 0x644, 0x627, 0x627, 0x646, 0x67c, 0x64a, 0x6ab, 0x627, 0x631, 0x6ab, 0x6cc,
0x646, 0x6cc, 0x627, 0x628, 0x64a, 0x646, 0x648, 0x633, 0x20, 0x627, 0x64a, 0x64a,
0x631, 0x632, 0x6a9, 0x67c, 0x627, 0x645, 0x627, 0x631, 0x6a9, 0x627, 0x6a9, 0x648,
0x631, 0x689, 0x648, 0x628, 0x627, 0x62c, 0x648, 0x62c, 0x648, 0x64a, 0x644, 0x627,
0x631, 0x64a, 0x648, 0x62c, 0x627, 0x645, 0x6cc, 0x646, 0x689, 0x648, 0x632, 0x627,
0x631, 0x64a, 0x648, 0x20, 0x6ab, 0x64a, 0x644, 0x64a, 0x6ab, 0x648, 0x633, 0x633,
0x627, 0x644, 0x67c, 0x627, 0x633, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x627, 0x646,
0x633, 0x627, 0x646, 0x20, 0x644, 0x648, 0x6cc, 0x633, 0x67c, 0x64a, 0x6a9, 0x648,
0x648, 0x645, 0x627, 0x648, 0x634, 0x648, 0x627, 0x6cc, 0x627, 0x622, 0x631, 0x648,
0x628, 0x627, 0x627, 0x633, 0x648, 0x646, 0x633, 0x64a, 0x648, 0x646, 0x627, 0x62a,
0x6cc, 0x6a9, 0x648, 0x6a9, 0x646, 0x628, 0x647, 0x6cc, 0x627, 0x628, 0x647, 0x6cc,
0x627, 0x20, 0x628, 0x6cc, 0x646, 0x689, 0x631, 0x627, 0x633, 0x628, 0x627, 0x631,
0x628, 0x627, 0x62f, 0x648, 0x633, 0x628, 0x644, 0x6cc, 0x632, 0x628, 0x644, 0x627,
0x646, 0x6a9, 0x2d, 0x633, 0x627, 0x628, 0x644, 0x648, 0x646, 0x628, 0x648, 0x627,
0x20, 0x648, 0x6cc, 0x633, 0x67c, 0x627, 0x628, 0x648, 0x6ab, 0x67c, 0x627, 0x628,
0x648, 0x632, 0x6a9, 0x6cc, 0x645, 0x628, 0x631, 0x62c, 0x20, 0x628, 0x64a, 0x6a9,
0x645, 0x67e, 0x648, 0x20, 0x6ab, 0x631, 0x689, 0x6cc, 0x6a9, 0x64a, 0x646, 0x6a9,
0x646, 0x6a9, 0x627, 0x631, 0x627, 0x6a9, 0x627, 0x633, 0x6a9, 0x6cc, 0x6cc, 0x646,
0x6a9, 0x6cc, 0x645, 0x646, 0x634, 0x6cc, 0x6a9, 0x627, 0x6ab, 0x648, 0x686, 0x6be,
0x648, 0x627, 0x6be, 0x648, 0x627, 0x633, 0x6cc, 0x648, 0x62f, 0x627, 0x62f, 0x20,
0x62c, 0x6cc, 0x648, 0x631, 0x6cc, 0x632, 0x6a9, 0x648, 0x633, 0x62a, 0x627, 0x631,
0x6cc, 0x6a9, 0x627, 0x6a9, 0x631, 0x633, 0x67c, 0x648, 0x646, 0x6a9, 0x648, 0x6cc,
0x627, 0x628, 0x627, 0x6a9, 0x648, 0x631, 0x627, 0x6a9, 0x627, 0x624, 0x689, 0x646,
0x645, 0x627, 0x631, 0x6a9, 0x634, 0x627, 0x646, 0x62f, 0x627, 0x633, 0x646, 0x20,
0x6a9, 0x631, 0x6cc, 0x6a9, 0x689, 0x646, 0x648, 0x631, 0x689, 0x627, 0x6cc, 0x67c,
0x631, 0x648, 0x67c, 0x62f, 0x648, 0x645, 0x6cc, 0x646, 0x6cc, 0x6a9, 0x627, 0x627,
0x6cc, 0x689, 0x645, 0x648, 0x646, 0x67c, 0x646, 0x627, 0x6cc, 0x6cc, 0x631, 0x648,
0x646, 0x67e, 0x627, 0x64a, 0x644, 0x20, 0x633, 0x644, 0x648, 0x627, 0x62f, 0x648,
0x631, 0x641, 0x648, 0x631, 0x67c, 0x20, 0x646, 0x6cc, 0x644, 0x633, 0x646, 0x641,
0x648, 0x631, 0x62a, 0x6cc, 0x644, 0x632, 0x627, 0x6ab, 0x6cc, 0x633, 0x633, 0x20,
0x628, 0x64a, 0x6cc, 0x6af, 0x648, 0x632, 0x20, 0x628, 0x64a, 0x644, 0x648, 0x6cc,
0x20, 0x62a, 0x631, 0x6a9, 0x6ab, 0x631, 0x646, 0x627, 0x62f, 0x627, 0x6ab, 0x627,
0x644, 0x689, 0x6cc, 0x67e, 0x6ab, 0x648, 0x627, 0x62a, 0x645, 0x627, 0x644, 0x627,
0x6ab, 0x648, 0x64a, 0x627, 0x6a9, 0x644, 0x6ab, 0x6cc, 0x627, 0x646, 0x627, 0x647,
0x6cc, 0x644, 0x641, 0x6a9, 0x633, 0x647, 0x648, 0x627, 0x646, 0x627, 0x647, 0x631,
0x645, 0x648, 0x633, 0x6cc, 0x644, 0x648, 0x627, 0x646, 0x689, 0x64a, 0x627, 0x646,
0x627, 0x20, 0x67e, 0x648, 0x644, 0x64a, 0x633, 0x646, 0x648, 0x6a9, 0x633, 0x20,
0x627, 0x646, 0x689, 0x64a, 0x627, 0x646, 0x627, 0x645, 0x648, 0x631, 0x64a, 0x646,
0x6ab, 0x648, 0x20, 0x627, 0x646, 0x689, 0x64a, 0x627, 0x646, 0x627, 0x67e, 0x64a,
0x67c, 0x633, 0x628, 0x631, 0x6ab, 0x60c, 0x20, 0x627, 0x646, 0x689, 0x64a, 0x627,
0x646, 0x627, 0x67c, 0x644, 0x20, 0x633, 0x67c, 0x64a, 0x60c, 0x20, 0x627, 0x646,
0x689, 0x64a, 0x627, 0x646, 0x627, 0x648, 0x6cc, 0x648, 0x6cc, 0x60c, 0x20, 0x627,
0x646, 0x689, 0x64a, 0x627, 0x646, 0x627, 0x648, 0x64a, 0x646, 0x633, 0x64a, 0x646,
0x633, 0x60c, 0x20, 0x627, 0x646, 0x689, 0x64a, 0x627, 0x646, 0x627, 0x648, 0x64a,
0x646, 0x627, 0x645, 0x6a9, 0x20, 0x627, 0x646, 0x689, 0x64a, 0x627, 0x646, 0x627,
0x627, 0x646, 0x648, 0x6a9, 0x627, 0x642, 0x644, 0x6cc, 0x67c, 0x62c, 0x645, 0x627,
0x6cc, 0x6a9, 0x647, 0x62c, 0x648, 0x646, 0x648, 0x644, 0x648, 0x626, 0x633, 0x20,
0x648, 0x6cc, 0x644, 0x645, 0x648, 0x646, 0x67c, 0x64a, 0x633, 0x64a, 0x644, 0x648,
0x60c, 0x20, 0x6a9, 0x648, 0x646, 0x67c, 0x6a9, 0x6cc, 0x6a9, 0x644, 0x6cc, 0x646,
0x689, 0x6cc, 0x632, 0x62c, 0x644, 0x6cc, 0x645, 0x627, 0x644, 0x627, 0x633, 0x20,
0x627, 0x6cc, 0x646, 0x62c, 0x644, 0x633, 0x644, 0x648, 0x648, 0x631, 0x20, 0x67e,
0x631, 0x646, 0x633, 0x20, 0x6a9, 0x648, 0x627, 0x631, 0x67c, 0x631, 0x645, 0x627,
0x633, 0x64a, 0x648, 0x645, 0x646, 0x6ab, 0x648, 0x627, 0x645, 0x646, 0x627, 0x648,
0x633, 0x645, 0x64a, 0x631, 0x64a, 0x6ab, 0x627, 0x67c, 0x645, 0x627, 0x631, 0x67c,
0x6cc, 0x646, 0x6cc, 0x6a9, 0x645, 0x64a, 0x67c, 0x627, 0x645, 0x648, 0x631, 0x648,
0x633, 0x645, 0x632, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x645, 0x6cc, 0x646, 0x648,
0x645, 0x6cc, 0x646, 0x645, 0x64a, 0x631, 0x64a, 0x689, 0x627, 0x645, 0x6cc, 0x62a,
0x644, 0x627, 0x6a9, 0x627, 0x67c, 0x644, 0x627, 0x645, 0x6a9, 0x633, 0x64a, 0x6a9,
0x648, 0x20, 0x69a, 0x627, 0x631, 0x645, 0x64a, 0x6a9, 0x648, 0x64a, 0x644, 0x627,
0x646, 0x645, 0x648, 0x646, 0x6a9, 0x67c, 0x648, 0x646, 0x645, 0x646, 0x67c, 0x631,
0x631, 0x64a, 0x645, 0x648, 0x646, 0x67c, 0x20, 0x648, 0x689, 0x6cc, 0x648, 0x645,
0x627, 0x646, 0x67c, 0x6cc, 0x633, 0x6cc, 0x631, 0x62a, 0x646, 0x6cc, 0x633, 0x627,
0x648, 0x646, 0x6cc, 0x648, 0x6cc, 0x627, 0x631, 0x6a9, 0x646, 0x648, 0x631, 0x648,
0x646, 0x647, 0x627, 0x628, 0x64a, 0x648, 0x644, 0x627, 0x60c, 0x20, 0x634, 0x645,
0x627, 0x644, 0x64a, 0x20, 0x689, 0x627, 0x6a9, 0x648, 0x67c, 0x627, 0x645, 0x631,
0x6a9, 0x632, 0x60c, 0x20, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x20, 0x689, 0x627,
0x6a9, 0x648, 0x67c, 0x627, 0x646, 0x648, 0x6cc, 0x20, 0x633, 0x644, 0x6cc, 0x645,
0x60c, 0x20, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x20, 0x62f, 0x627, 0x6a9, 0x648,
0x62a, 0x627, 0x646, 0x648, 0x648, 0x6a9, 0x627, 0x648, 0x62c, 0x646, 0x6ab, 0x627,
0x67e, 0x627, 0x646, 0x627, 0x645, 0x627, 0x67e, 0x627, 0x631, 0x627, 0x645, 0x627,
0x631, 0x628, 0x648, 0x641, 0x6cc, 0x646, 0x6a9, 0x633, 0x67e, 0x648, 0x631, 0x67c,
0x20, 0x627, 0x6cc, 0x648, 0x20, 0x2d, 0x20, 0x67e, 0x631, 0x646, 0x633, 0x62f,
0x20, 0x627, 0x633, 0x67e, 0x627, 0x646, 0x6cc, 0x627, 0x20, 0x628, 0x646, 0x62f,
0x631, 0x67e, 0x648, 0x631, 0x67c, 0x648, 0x20, 0x648, 0x6cc, 0x644, 0x647, 0x648,
0x67e, 0x648, 0x631, 0x62a, 0x648, 0x20, 0x631, 0x6cc, 0x6a9, 0x648, 0x67e, 0x646,
0x67c, 0x627, 0x20, 0x622, 0x631, 0x6cc, 0x646, 0x627, 0x633, 0x631, 0x64a, 0x646,
0x6a9, 0x646, 0x20, 0x627, 0x646, 0x644, 0x67c, 0x631, 0x6cc, 0x633, 0x6cc, 0x641,
0x64a, 0x631, 0x6cc, 0x6ab, 0x6cc, 0x646, 0x627, 0x631, 0x64a, 0x633, 0x627, 0x644,
0x648, 0x67c, 0x631, 0x6cc, 0x648, 0x20, 0x628, 0x631, 0x627, 0x646, 0x6a9, 0x648,
0x633, 0x646, 0x627, 0x62a, 0x631, 0x645, 0x633, 0x646, 0x62a, 0x64a, 0x627, 0x6ab,
0x648, 0x633, 0x646, 0x62a, 0x648, 0x20, 0x689, 0x648, 0x645, 0x6cc, 0x646, 0x6ab,
0x648, 0x633, 0x627, 0x648, 0x20, 0x67e, 0x627, 0x648, 0x644, 0x648, 0x627, 0x67c,
0x648, 0x6a9, 0x648, 0x67c, 0x648, 0x631, 0x645, 0x6cc, 0x67c, 0x633, 0x6cc, 0x67c,
0x6a9, 0x627, 0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x628, 0x627, 0x631, 0x67c, 0x6cc,
0x644, 0x6cc, 0x645, 0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x62c, 0x627, 0x646, 0x632,
0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x6a9, 0x67c, 0x633, 0x633, 0x6cc, 0x646, 0x67c,
0x20, 0x644, 0x648, 0x633, 0x6cc, 0x627, 0x633, 0x64a, 0x646, 0x67c, 0x20, 0x62a,
0x647, 0x627, 0x645, 0x633, 0x633, 0x6d0, 0x646, 0x67c, 0x20, 0x648, 0x6cc, 0x633,
0x646, 0x67c, 0x633, 0x648, 0x64a, 0x641, 0x67c, 0x20, 0x6a9, 0x631, 0x646, 0x67c,
0x67c, 0x6ab, 0x633, 0x6cc, 0x6ab, 0x627, 0x644, 0x67e, 0x627, 0x62a, 0x62c, 0x648,
0x627, 0x646, 0x627, 0x67c, 0x648, 0x631, 0x646, 0x67c, 0x648, 0x67c, 0x648, 0x631,
0x62a, 0x648, 0x644, 0x627, 0x648, 0x6cc, 0x646, 0x6a9, 0x648, 0x648, 0x631, 0x648,
0x627, 0x64a, 0x67c, 0x20, 0x647, 0x627, 0x631, 0x633, 0x648, 0x6cc, 0x646, 0x67e,
0x6cc, 0x6ab, 0x64a, 0x627, 0x6a9, 0x648, 0x67c, 0x67c, 0x6a9, 0x6cc, 0x633, 0x64a,
0x689, 0x64a, 0x648, 0x633, 0x689, 0x648, 0x645, 0x648, 0x646, 0x67c, 0x20, 0x689,
0x64a, 0x20, 0x627, 0x631, 0x648, 0x644, 0x645, 0x6a9, 0x648, 0x627, 0x631, 0x64a,
0x645, 0x627, 0x648, 0x633, 0x646, 0x645, 0x6a9, 0x645, 0x631, 0x689, 0x648, 0x67e,
0x627, 0x644, 0x645, 0x631, 0x631, 0x648, 0x62f, 0x631, 0x627, 0x633, 0x6cc, 0x648,
0x627, 0x67c, 0x631, 0x648, 0x644, 0x648, 0x627, 0x633, 0x62a, 0x648, 0x6a9, 0x644,
0x627, 0x646, 0x6ab, 0x64a, 0x631, 0x628, 0x64a, 0x646, 0x627, 0x689, 0x646, 0x627,
0x644, 0x645, 0x627, 0x62a, 0x6cc, 0x627, 0x645, 0x645, 0x627, 0x646, 0x627, 0x646,
0x627, 0x689, 0x64a, 0x631, 0x627, 0x6a9, 0x67c, 0x627, 0x648, 0x627, 0x6a9, 0x62a,
0x648, 0x628, 0x627, 0x634, 0x63a, 0x20, 0x622, 0x628, 0x627, 0x62f, 0x627, 0x6d0,
0x67c, 0x631, 0x627, 0x648, 0x628, 0x62d, 0x631, 0x6cc, 0x646, 0x628, 0x646, 0x6a9,
0x627, 0x6a9, 0x628, 0x6cc, 0x631, 0x648, 0x62a, 0x628, 0x634, 0x6a9, 0x6cc, 0x6a9,
0x628, 0x631, 0x648, 0x6cc, 0x646, 0x64a, 0x686, 0x64a, 0x62a, 0x627, 0x686, 0x648,
0x64a, 0x628, 0x644, 0x633, 0x627, 0x646, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x648,
0x689, 0x647, 0x627, 0x6a9, 0x647, 0x62f, 0x644, 0x64a, 0x62f, 0x648, 0x628, 0x6cc,
0x62f, 0x648, 0x634, 0x646, 0x628, 0x64a, 0x641, 0x627, 0x645, 0x627, 0x6ab, 0x633,
0x62a, 0x627, 0x63a, 0x632, 0x647, 0x647, 0x628, 0x631, 0x648, 0x646, 0x647, 0x648,
0x20, 0x686, 0x64a, 0x20, 0x645, 0x646, 0x20, 0x69a, 0x627, 0x631, 0x647, 0x627,
0x646, 0x6ab, 0x20, 0x6a9, 0x627, 0x646, 0x6ab, 0x647, 0x627, 0x648, 0x689, 0x627,
0x631, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x62c, 0x627, 0x64a, 0x627, 0x67e, 0x648,
0x631, 0x627, 0x64a, 0x631, 0x648, 0x634, 0x644, 0x645, 0x6a9, 0x627, 0x628, 0x644,
0x6a9, 0x627, 0x645, 0x686, 0x627, 0x62a, 0x6a9, 0x627, 0x6a9, 0x631, 0x627, 0x686,
0x64a, 0x6a9, 0x67c, 0x645, 0x646, 0x689, 0x648, 0x62e, 0x646, 0x62f, 0x64a, 0x6ab,
0x627, 0x6a9, 0x648, 0x644, 0x6a9, 0x62a, 0x647, 0x6a9, 0x631, 0x64a, 0x633, 0x646,
0x648, 0x64a, 0x627, 0x631, 0x633, 0x6a9, 0x6a9, 0x648, 0x644, 0x627, 0x644, 0x645,
0x67e, 0x648, 0x631, 0x6a9, 0x648, 0x686, 0x646, 0x6ab, 0x6a9, 0x648, 0x6d0, 0x62a,
0x645, 0x6a9, 0x627, 0x648, 0x645, 0x6ab, 0x62f, 0x627, 0x646, 0x645, 0x6a9, 0x627,
0x633, 0x627, 0x631, 0x645, 0x646, 0x6cc, 0x644, 0x627, 0x646, 0x6cc, 0x6a9, 0x648,
0x633, 0x6cc, 0x627, 0x646, 0x648, 0x648, 0x6a9, 0x648, 0x632, 0x646, 0x6cc, 0x67c,
0x6a9, 0x646, 0x648, 0x648, 0x633, 0x64a, 0x628, 0x631, 0x633, 0x6a9, 0x627, 0x648,
0x631, 0x644, 0x67e, 0x646, 0x648, 0x645, 0x20, 0x67e, 0x646, 0x67e, 0x648, 0x646,
0x67c, 0x6cc, 0x646, 0x6cc, 0x6a9, 0x67e, 0x64a, 0x627, 0x646, 0x6ab, 0x20, 0x64a,
0x627, 0x646, 0x6ab, 0x6a9, 0x648, 0x633, 0x62a, 0x627, 0x646, 0x6d0, 0x642, 0x64a,
0x632, 0x64a, 0x20, 0x644, 0x648, 0x631, 0x689, 0x627, 0x633, 0x62e, 0x627, 0x644,
0x64a, 0x646, 0x634, 0x646, 0x6af, 0x6be, 0x627, 0x626, 0x6cc, 0x633, 0x6cc, 0x646,
0x6ab, 0x627, 0x67e, 0x648, 0x631, 0x633, 0x631, 0x64a, 0x689, 0x646, 0x64a, 0x6a9,
0x648, 0x644, 0x64a, 0x645, 0x633, 0x6a9, 0x62a, 0x627, 0x64a, 0x67e, 0x64a, 0x62a,
0x627, 0x634, 0x642, 0x646, 0x62f, 0x62a, 0x628, 0x644, 0x6cc, 0x633, 0x64a, 0x62a,
0x647, 0x631, 0x627, 0x646, 0x62a, 0x647, 0x64a, 0x645, 0x641, 0x648, 0x67c, 0x648,
0x6a9, 0x6cc, 0x648, 0x627, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x67c,
0x631, 0x627, 0x648, 0x631, 0x648, 0x645, 0x642, 0x64a, 0x627, 0x648, 0x633, 0x62a,
0x646, 0x64a, 0x631, 0x627, 0x648, 0x64a, 0x646, 0x67c, 0x64a, 0x646, 0x648, 0x644,
0x627, 0x62f, 0x6cc, 0x648, 0x633, 0x62a, 0x627, 0x6a9, 0x64a, 0x627, 0x6a9, 0x648,
0x67c, 0x633, 0x6a9, 0x6cc, 0x627, 0x646, 0x6ab, 0x648, 0x646, 0x64a, 0x6a9, 0x627,
0x67c, 0x64a, 0x631, 0x646, 0x628, 0x631, 0x6ab, 0x64a, 0x631, 0x64a, 0x648, 0x627,
0x646, 0x627, 0x64a, 0x632, 0x648, 0x631, 0x633, 0x6a9, 0x646, 0x627, 0x631, 0x64a,
0x6a9, 0x64a, 0x67e, 0x20, 0x648, 0x631, 0x689, 0x645, 0x62f, 0x64a, 0x631, 0x627,
0x631, 0x64a, 0x6a9, 0x62c, 0x627, 0x648, 0x64a, 0x6a9, 0x633, 0x648, 0x64a, 0x644,
0x64a, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627, 0x633, 0x6cc, 0x646, 0x67c,
0x20, 0x647, 0x6cc, 0x644, 0x6cc, 0x646, 0x627, 0x633, 0x67c, 0x646, 0x644, 0x64a,
0x627, 0x689, 0x6cc, 0x644, 0x627, 0x6cc, 0x689, 0x628, 0x631, 0x6cc, 0x633, 0x628,
0x646, 0x628, 0x631, 0x648, 0x6a9, 0x646, 0x20, 0x647, 0x644, 0x689, 0x627, 0x631,
0x648, 0x646, 0x627, 0x64a, 0x648, 0x6a9, 0x644, 0x627, 0x647, 0x648, 0x628, 0x627,
0x631, 0x67c, 0x644, 0x6cc, 0x646, 0x689, 0x627, 0x645, 0x6cc, 0x646, 0x644, 0x627,
0x631, 0x689, 0x20, 0x647, 0x648, 0x64a, 0x645, 0x6cc, 0x644, 0x628, 0x648, 0x631,
0x646, 0x67e, 0x631, 0x62a, 0x633, 0x689, 0x646, 0x64a, 0x627, 0x645, 0x633, 0x62a,
0x631, 0x62f, 0x627, 0x645, 0x627, 0x646, 0x62f, 0x648, 0x631, 0x627, 0x627, 0x633,
0x62a, 0x631, 0x627, 0x20, 0x62e, 0x627, 0x646, 0x627, 0x64a, 0x62a, 0x646, 0x632,
0x628, 0x644, 0x63a, 0x627, 0x62f, 0x628, 0x631, 0x644, 0x646, 0x628, 0x631, 0x627,
0x67c, 0x633, 0x644, 0x627, 0x648, 0x627, 0x628, 0x631, 0x648, 0x633, 0x644, 0x632,
0x628, 0x62e, 0x627, 0x631, 0x633, 0x62a, 0x628, 0x62f, 0x627, 0x67e, 0x633, 0x67c,
0x628, 0x648, 0x633, 0x64a, 0x646, 0x62c, 0x646, 0x686, 0x6cc, 0x633, 0x6cc, 0x646,
0x648, 0x6a9, 0x648, 0x67e, 0x646, 0x20, 0x647, 0x64a, 0x6ab, 0x646, 0x689, 0x628,
0x644, 0x646, 0x62c, 0x628, 0x644, 0x20, 0x627, 0x644, 0x637, 0x627, 0x631, 0x642,
0x6ab, 0x631, 0x646, 0x633, 0x64a, 0x647, 0x6cc, 0x644, 0x633, 0x646, 0x6a9, 0x6cc,
0x62f, 0x20, 0x622, 0x626, 0x644, 0x20, 0x622, 0x641, 0x20, 0x645, 0x6cc, 0x646,
0x627, 0x633, 0x62a, 0x646, 0x628, 0x648, 0x644, 0x6a9, 0x64a, 0x644, 0x646, 0x64a,
0x646, 0x6ab, 0x631, 0x627, 0x689, 0x6a9, 0x6cc, 0x631, 0x648, 0x641, 0x6a9, 0x6cc,
0x641, 0x644, 0x6cc, 0x633, 0x628, 0x648, 0x646, 0x644, 0x648, 0x628, 0x62c, 0x627,
0x646, 0x627, 0x644, 0x648, 0x6ab, 0x632, 0x627, 0x645, 0x628, 0x648, 0x631, 0x6ab,
0x645, 0x6cc, 0x689, 0x631, 0x6cc, 0x689, 0x645, 0x627, 0x644, 0x67c, 0x627, 0x645,
0x64a, 0x631, 0x64a, 0x647, 0x627, 0x645, 0x645, 0x646, 0x633, 0x6a9, 0x67e, 0x627,
0x631, 0x6cc, 0x633, 0x67e, 0x648, 0x689, 0x6ab, 0x648, 0x631, 0x64a, 0x6a9, 0x627,
0x67e, 0x631, 0x627, 0x6ab, 0x633, 0x645, 0x627, 0x631, 0x627, 0x633, 0x627, 0x646,
0x20, 0x645, 0x627, 0x631, 0x6cc, 0x646, 0x648, 0x633, 0x64a, 0x631, 0x627, 0x62c,
0x64a, 0x648, 0x627, 0x633, 0x631, 0x627, 0x62a, 0x641, 0x633, 0x6cc, 0x645, 0x641,
0x631, 0x648, 0x67e, 0x648, 0x644, 0x633, 0x6a9, 0x67e, 0x648, 0x6ab, 0x635, 0x648,
0x641, 0x6cc, 0x627, 0x627, 0x633, 0x62a, 0x648, 0x644, 0x648, 0x645, 0x627, 0x644,
0x64a, 0x627, 0x646, 0x648, 0x633, 0x6a9, 0x648, 0x627, 0x689, 0x648, 0x632, 0x648,
0x6cc, 0x67c, 0x6cc, 0x6a9, 0x627, 0x646, 0x648, 0x6cc, 0x627, 0x646, 0x627, 0x648,
0x64a, 0x644, 0x646, 0x64a, 0x648, 0x633, 0x648, 0x627, 0x644, 0x6ab, 0x648, 0x6ab,
0x631, 0x627, 0x689, 0x632, 0x6ab, 0x631, 0x628, 0x632, 0x631, 0x6cc, 0x686, 0x627,
0x646, 0x62a, 0x627, 0x646, 0x646, 0x627, 0x631, 0x64a, 0x648, 0x648, 0x686, 0x627,
0x6ab, 0x648, 0x633, 0x6a9, 0x631, 0x64a, 0x633, 0x645, 0x633, 0x6a9, 0x648, 0x6a9,
0x648, 0x632, 0x6a9, 0x631, 0x6ab, 0x648, 0x644, 0x64a, 0x646, 0x645, 0x627, 0x644,
0x62f, 0x64a, 0x67e, 0x645, 0x627, 0x631, 0x64a, 0x634, 0x64a, 0x633, 0x645, 0x6cc,
0x67c, 0x648, 0x62a, 0x631, 0x64a, 0x20, 0x64a, 0x648, 0x646, 0x64a, 0x646, 0x627,
0x67e, 0x6cc, 0x627, 0x627, 0x6a9, 0x644, 0x646, 0x62f, 0x628, 0x648, 0x6ab, 0x646,
0x20, 0x648, 0x6cc, 0x644, 0x686, 0x627, 0x62a, 0x627, 0x645, 0x686, 0x648, 0x6a9,
0x627, 0x6cc, 0x633, 0x67c, 0x631, 0x639, 0x641, 0x627, 0x62a, 0x641, 0x648, 0x6a9,
0x627, 0x641, 0x648, 0x641, 0x62c, 0x64a, 0x6ab, 0x627, 0x644, 0x67e, 0x6ab, 0x648,
0x633, 0x6ab, 0x64a, 0x645, 0x628, 0x64a, 0x631, 0x6ab, 0x648, 0x627, 0x689, 0x644,
0x6a9, 0x64a, 0x646, 0x627, 0x644, 0x647, 0x6cc, 0x646, 0x648, 0x644, 0x648, 0x644,
0x648, 0x627, 0x646, 0x689, 0x631, 0x628, 0x631, 0x64a, 0x6a9, 0x64a, 0x631, 0x64a,
0x67c, 0x645, 0x627, 0x67c, 0x64a, 0x6a9, 0x648, 0x633, 0x631, 0x627, 0x64a, 0x6a9,
0x648, 0x627, 0x62c, 0x644, 0x64a, 0x646, 0x645, 0x62c, 0x648, 0x631, 0x648, 0x645,
0x627, 0x631, 0x6a9, 0x64a, 0x633, 0x627, 0x633, 0x645, 0x64a, 0x689, 0x648, 0x6cc,
0x646, 0x627, 0x6cc, 0x631, 0x648, 0x646, 0x6cc, 0x648, 0x648, 0x646, 0x648, 0x631,
0x641, 0x6a9, 0x67e, 0x64a, 0x6ab, 0x648, 0x20, 0x67e, 0x64a, 0x6ab, 0x648, 0x67e,
0x644, 0x627, 0x648, 0x67e, 0x64a, 0x67c, 0x64a, 0x6a9, 0x64a, 0x631, 0x646, 0x67e,
0x648, 0x646, 0x67e, 0x64a, 0x67e, 0x648, 0x631, 0x67c, 0x20, 0x645, 0x648, 0x631,
0x633, 0x628, 0x6cc, 0x631, 0x627, 0x631, 0x648, 0x67c, 0x648, 0x646, 0x6ab, 0x627,
0x633, 0x64a, 0x67e, 0x627, 0x646, 0x67c, 0x647, 0x64a, 0x67c, 0x64a, 0x67c, 0x648,
0x646, 0x6ab, 0x627, 0x62a, 0x627, 0x67e, 0x648, 0x648, 0x627, 0x644, 0x633, 0x647,
0x631, 0x627, 0x631, 0x6d2, 0x62f, 0x20, 0x6a9, 0x645, 0x62a, 0x631, 0x20, 0x634,
0x647, 0x632, 0x627, 0x62f, 0x647, 0x20, 0x62f, 0x631, 0x6d2, 0x20, 0x645, 0x6cc,
0x627, 0x634, 0x62a, 0x646, 0x6cd, 0x622, 0x628, 0x6cc, 0x62c, 0x627, 0x646, 0x622,
0x62f, 0x6cc, 0x633, 0x20, 0x622, 0x628, 0x627, 0x628, 0x627, 0x627, 0x644, 0x62c,
0x632, 0x6cc, 0x631, 0x647, 0x627, 0x633, 0x645, 0x631, 0x647, 0x628, 0x627, 0x645,
0x627, 0x6a9, 0x648, 0x628, 0x627, 0x646, 0x6af, 0x6cc, 0x628, 0x6cc, 0x633, 0x627,
0x626, 0x648, 0x628, 0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x631, 0x647, 0x628, 0x631,
0x627, 0x632, 0x648, 0x6cc, 0x644, 0x6a9, 0x627, 0x632, 0x627, 0x628, 0x644, 0x627,
0x646, 0x6a9, 0x627, 0x633, 0x628, 0x62a, 0x647, 0x62f, 0x627, 0x6a9, 0x627, 0x631,
0x62c, 0x6cc, 0x628, 0x648, 0x62a, 0x6cc, 0x62f, 0x648, 0x622, 0x644, 0x627, 0x627,
0x644, 0x639, 0x6cc, 0x648, 0x646, 0x641, 0x631, 0x6cc, 0x200c, 0x62a, 0x627, 0x648,
0x646, 0x6af, 0x627, 0x628, 0x648, 0x631, 0x648, 0x646, 0x647, 0x631, 0x627, 0x631,
0x647, 0x698, 0x648, 0x647, 0x627, 0x646, 0x633, 0x628, 0x648, 0x631, 0x6af, 0x6a9,
0x627, 0x645, 0x67e, 0x627, 0x644, 0x627, 0x62e, 0x627, 0x631, 0x637, 0x648, 0x645,
0x6a9, 0x6cc, 0x6af, 0x627, 0x644, 0x6cc, 0x6a9, 0x6cc, 0x646, 0x634, 0x627, 0x633,
0x627, 0x644, 0x6cc, 0x628, 0x631, 0x648, 0x6cc, 0x644, 0x644, 0x648, 0x645, 0x647,
0x644, 0x648, 0x628, 0x648, 0x645, 0x628, 0x627, 0x634, 0x6cc, 0x644, 0x648, 0x632,
0x627, 0x6a9, 0x627, 0x645, 0x627, 0x67e, 0x648, 0x62a, 0x648, 0x645, 0x627, 0x633,
0x631, 0x648, 0x645, 0x628, 0x627, 0x628, 0x627, 0x646, 0x647, 0x645, 0x648, 0x6af,
0x627, 0x62f, 0x6cc, 0x634, 0x648, 0x645, 0x648, 0x646, 0x631, 0x648, 0x648, 0x6cc,
0x627, 0x646, 0x627, 0x6cc, 0x631, 0x648, 0x628, 0x6cc, 0x627, 0x646, 0x62c, 0x627,
0x645, 0x646, 0x627, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x646, 0x648, 0x622, 0x6a9,
0x634, 0x648, 0x62a, 0x627, 0x648, 0x622, 0x6af, 0x627, 0x62f, 0x648, 0x6af, 0x648,
0x67e, 0x648, 0x631, 0x62a, 0x648, 0x20, 0x646, 0x648, 0x648, 0x648, 0x633, 0x627,
0x626, 0x648, 0x62a, 0x648, 0x645, 0x647, 0x648, 0x6cc, 0x646, 0x62f, 0x647, 0x648,
0x6a9, 0x627, 0x6cc, 0x62f, 0x6a9, 0x627, 0x646, 0x6a9, 0x648, 0x631, 0x6cc, 0x62c,
0x622, 0x646, 0x6af, 0x648, 0x626, 0x6cc, 0x644, 0x627, 0x622, 0x646, 0x62a, 0x6cc,
0x6af, 0x648, 0x627, 0x622, 0x631, 0x627, 0x6af, 0x648, 0x627, 0x6cc, 0x646, 0x627,
0x628, 0x648, 0x626, 0x646, 0x648, 0x633, 0x200c, 0x622, 0x6cc, 0x631, 0x633, 0x6a9,
0x627, 0x62a, 0x627, 0x645, 0x627, 0x631, 0x6a9, 0x627, 0x6a9, 0x648, 0x631, 0x62f,
0x648, 0x648, 0x627, 0x62e, 0x648, 0x62e, 0x648, 0x6cc, 0x644, 0x627, 0x631, 0x6cc,
0x648, 0x62e, 0x627, 0x645, 0x646, 0x62f, 0x648, 0x633, 0x627, 0x631, 0x6cc, 0x648,
0x6af, 0x627, 0x644, 0x6af, 0x648, 0x633, 0x633, 0x627, 0x644, 0x62a, 0x627, 0x633,
0x646, 0x200c, 0x62e, 0x648, 0x627, 0x646, 0x633, 0x646, 0x200c, 0x644, 0x648, 0x626,
0x6cc, 0x633, 0x62a, 0x648, 0x6a9, 0x648, 0x645, 0x646, 0x627, 0x631, 0x648, 0x628,
0x627, 0x622, 0x633, 0x648, 0x646, 0x633, 0x6cc, 0x648, 0x646, 0x627, 0x62a, 0x6a9,
0x648, 0x6a9, 0x627, 0x646, 0x628, 0x627, 0x6cc, 0x627, 0x628, 0x627, 0x647, 0x6cc,
0x627, 0x20, 0x628, 0x627, 0x646, 0x62f, 0x631, 0x627, 0x633, 0x628, 0x644, 0x627,
0x646, 0x2010, 0x633, 0x627, 0x628, 0x644, 0x648, 0x646, 0x628, 0x648, 0x626, 0x627,
0x648, 0x6cc, 0x633, 0x62a, 0x627, 0x628, 0x648, 0x6af, 0x648, 0x62a, 0x627, 0x628,
0x648, 0x6cc, 0x633, 0x6cc, 0x6a9, 0x645, 0x628, 0x631, 0x6cc, 0x62c, 0x200c, 0x628,
0x6cc, 0x6a9, 0x645, 0x67e, 0x648, 0x20, 0x6af, 0x631, 0x627, 0x646, 0x62f, 0x647,
0x6a9, 0x627, 0x646, 0x6a9, 0x648, 0x646, 0x634, 0x6cc, 0x6a9, 0x627, 0x6af, 0x648,
0x686, 0x6cc, 0x648, 0x627, 0x648, 0x627, 0x633, 0x6cc, 0x648, 0x62f, 0x627, 0x62f,
0x20, 0x62e, 0x648, 0x627, 0x631, 0x632, 0x6a9, 0x627, 0x633, 0x62a, 0x627, 0x631,
0x6cc, 0x6a9, 0x627, 0x6a9, 0x631, 0x633, 0x62a, 0x648, 0x646, 0x6a9, 0x648, 0x6cc,
0x627, 0x648, 0x627, 0x6a9, 0x648, 0x631, 0x627, 0x633, 0x627, 0x626, 0x648, 0x62f,
0x627, 0x646, 0x645, 0x627, 0x631, 0x6a9, 0x633, 0x200c, 0x647, 0x627, 0x648, 0x646,
0x62f, 0x627, 0x648, 0x633, 0x646, 0x20, 0x6a9, 0x631, 0x6cc, 0x6a9, 0x62f, 0x646,
0x648, 0x631, 0x62f, 0x6cc, 0x62a, 0x631, 0x648, 0x6cc, 0x62a, 0x627, 0x62f, 0x645,
0x648, 0x646, 0x62a, 0x648, 0x646, 0x627, 0x6cc, 0x631, 0x648, 0x646, 0x67e, 0x647,
0x627, 0x644, 0x633, 0x627, 0x644, 0x648, 0x627, 0x62f, 0x648, 0x631, 0x641, 0x648,
0x631, 0x62a, 0x20, 0x646, 0x644, 0x633, 0x648, 0x646, 0x641, 0x648, 0x631, 0x62a,
0x627, 0x644, 0x632, 0x627, 0x6af, 0x644, 0x6cc, 0x633, 0x200c, 0x628, 0x6cc, 0x6af,
0x648, 0x633, 0x200c, 0x628, 0x6cc, 0x6af, 0x631, 0x627, 0x646, 0x62f, 0x20, 0x62a,
0x648, 0x631, 0x6a9, 0x6af, 0x631, 0x646, 0x627, 0x62f, 0x627, 0x6af, 0x648, 0x627,
0x62f, 0x644, 0x648, 0x67e, 0x6af, 0x648, 0x627, 0x62a, 0x645, 0x627, 0x644, 0x627,
0x6af, 0x648, 0x627, 0x6cc, 0x627, 0x6a9, 0x6cc, 0x644, 0x6af, 0x648, 0x6cc, 0x627,
0x646, 0x647, 0x644, 0x6cc, 0x641, 0x6a9, 0x633, 0x647, 0x627, 0x648, 0x627, 0x646,
0x627, 0x627, 0x631, 0x645, 0x648, 0x633, 0x6cc, 0x648, 0x627, 0x6cc, 0x646, 0x62f,
0x6cc, 0x627, 0x646, 0x627, 0x67e, 0x648, 0x644, 0x6cc, 0x633, 0x646, 0x627, 0x6a9,
0x633, 0x60c, 0x20, 0x627, 0x6cc, 0x646, 0x62f, 0x6cc, 0x627, 0x646, 0x627, 0x645,
0x627, 0x631, 0x646, 0x6af, 0x648, 0x60c, 0x20, 0x627, 0x6cc, 0x646, 0x62f, 0x6cc,
0x627, 0x646, 0x627, 0x67e, 0x6cc, 0x62a, 0x631, 0x632, 0x628, 0x631, 0x6af, 0x60c,
0x20, 0x627, 0x6cc, 0x646, 0x62f, 0x6cc, 0x627, 0x646, 0x627, 0x62a, 0x644, 0x200c,
0x633, 0x6cc, 0x62a, 0x6cc, 0x60c, 0x20, 0x627, 0x6cc, 0x646, 0x62f, 0x6cc, 0x627,
0x646, 0x627, 0x648, 0x6cc, 0x648, 0x6cc, 0x60c, 0x20, 0x627, 0x6cc, 0x646, 0x62f,
0x6cc, 0x627, 0x646, 0x627, 0x648, 0x6cc, 0x646, 0x633, 0x646, 0x633, 0x60c, 0x20,
0x627, 0x646, 0x62f, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x6cc, 0x646, 0x627, 0x645,
0x627, 0x6a9, 0x60c, 0x20, 0x627, 0x6cc, 0x646, 0x62f, 0x6cc, 0x627, 0x646, 0x627,
0x627, 0x6cc, 0x646, 0x648, 0x648, 0x6cc, 0x6a9, 0x627, 0x6cc, 0x6a9, 0x644, 0x648,
0x626, 0x62a, 0x62c, 0x627, 0x645, 0x627, 0x626, 0x6cc, 0x6a9, 0x627, 0x644, 0x648,
0x626, 0x6cc, 0x632, 0x648, 0x6cc, 0x644, 0x645, 0x627, 0x646, 0x62a, 0x6cc, 0x633,
0x644, 0x648, 0x60c, 0x20, 0x6a9, 0x646, 0x62a, 0x627, 0x6a9, 0x6cc, 0x6a9, 0x631,
0x627, 0x644, 0x646, 0x62f, 0x6cc, 0x6a9, 0x644, 0x627, 0x67e, 0x627, 0x632, 0x644,
0x648, 0x633, 0x200c, 0x622, 0x646, 0x62c, 0x644, 0x633, 0x628, 0x62e, 0x634, 0x20,
0x634, 0x627, 0x647, 0x632, 0x627, 0x62f, 0x647, 0x200c, 0x200c, 0x646, 0x634, 0x6cc,
0x646, 0x20, 0x67e, 0x627, 0x6cc, 0x6cc, 0x646, 0x645, 0x627, 0x633, 0x6cc, 0x648,
0x645, 0x627, 0x646, 0x627, 0x6af, 0x648, 0x627, 0x645, 0x627, 0x631, 0x6cc, 0x6af,
0x627, 0x62a, 0x645, 0x627, 0x631, 0x62a, 0x6cc, 0x646, 0x6cc, 0x6a9, 0x645, 0x627,
0x633, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x645, 0x646, 0x627, 0x645, 0x6cc, 0x646,
0x6cc, 0x645, 0x631, 0x6cc, 0x62f, 0x627, 0x645, 0x62a, 0x627, 0x644, 0x627, 0x6a9,
0x627, 0x62a, 0x644, 0x627, 0x645, 0x6a9, 0x632, 0x6cc, 0x6a9, 0x648, 0x633, 0x6cc,
0x62a, 0x6cc, 0x645, 0x6cc, 0x6a9, 0x644, 0x648, 0x646, 0x645, 0x627, 0x646, 0x6a9,
0x62a, 0x648, 0x646, 0x645, 0x648, 0x646, 0x62a, 0x631, 0x6cc, 0x645, 0x648, 0x646,
0x62a, 0x647, 0x200c, 0x648, 0x6cc, 0x62f, 0x626, 0x648, 0x645, 0x648, 0x646, 0x62a,
0x633, 0x631, 0x627, 0x62a, 0x646, 0x627, 0x633, 0x627, 0x626, 0x648, 0x646, 0x6cc,
0x648, 0x6cc, 0x648, 0x631, 0x6a9, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6cc, 0x627,
0x628, 0x6cc, 0x648, 0x644, 0x627, 0x60c, 0x20, 0x62f, 0x627, 0x6a9, 0x648, 0x62a,
0x627, 0x6cc, 0x20, 0x634, 0x645, 0x627, 0x644, 0x6cc, 0x633, 0x646, 0x62a, 0x631,
0x60c, 0x20, 0x62f, 0x627, 0x6a9, 0x648, 0x62a, 0x627, 0x6cc, 0x20, 0x634, 0x645,
0x627, 0x644, 0x6cc, 0x646, 0x6cc, 0x648, 0x633, 0x627, 0x644, 0x645, 0x60c, 0x20,
0x62f, 0x627, 0x6a9, 0x648, 0x62a, 0x627, 0x6cc, 0x20, 0x634, 0x645, 0x627, 0x644,
0x6cc, 0x627, 0x62e, 0x6cc, 0x646, 0x627, 0x6af, 0x627, 0x67e, 0x627, 0x631, 0x627,
0x645, 0x627, 0x631, 0x6cc, 0x628, 0x648, 0x67e, 0x648, 0x631, 0x62a, 0x648, 0x67e,
0x631, 0x646, 0x633, 0x67e, 0x648, 0x631, 0x62a, 0x200c, 0x622, 0x648, 0x627, 0x633,
0x67e, 0x6cc, 0x646, 0x67e, 0x648, 0x631, 0x62a, 0x648, 0x648, 0x644, 0x6cc, 0x648,
0x67e, 0x648, 0x631, 0x62a, 0x648, 0x631, 0x6cc, 0x6a9, 0x648, 0x67e, 0x648, 0x646,
0x62a, 0x627, 0x20, 0x622, 0x631, 0x646, 0x627, 0x633, 0x62e, 0x644, 0x6cc, 0x62c,
0x6a9, 0x20, 0x631, 0x646, 0x6a9, 0x6cc, 0x646, 0x631, 0x6cc, 0x633, 0x6cc, 0x641,
0x6cc, 0x631, 0x62c, 0x627, 0x6cc, 0x646, 0x627, 0x631, 0x632, 0x648, 0x644, 0x648,
0x62a, 0x631, 0x6cc, 0x648, 0x628, 0x631, 0x627, 0x646, 0x6a9, 0x648, 0x633, 0x646,
0x62a, 0x627, 0x631, 0x645, 0x633, 0x627, 0x646, 0x62a, 0x6cc, 0x627, 0x6af, 0x648,
0x633, 0x627, 0x646, 0x62a, 0x648, 0x20, 0x62f, 0x648, 0x645, 0x6cc, 0x646, 0x6af,
0x648, 0x633, 0x627, 0x626, 0x648, 0x67e, 0x627, 0x626, 0x648, 0x644, 0x648, 0x627,
0x633, 0x6a9, 0x648, 0x631, 0x633, 0x628, 0x6cc, 0x633, 0x648, 0x646, 0x62f, 0x633,
0x6cc, 0x62a, 0x6a9, 0x627, 0x633, 0x646, 0x62a, 0x20, 0x628, 0x627, 0x631, 0x62a,
0x644, 0x645, 0x6cc, 0x633, 0x646, 0x62a, 0x20, 0x62c, 0x627, 0x646, 0x633, 0x646,
0x62a, 0x20, 0x6a9, 0x6cc, 0x62a, 0x633, 0x633, 0x646, 0x62a, 0x20, 0x644, 0x648,
0x633, 0x6cc, 0x627, 0x633, 0x646, 0x62a, 0x20, 0x62a, 0x648, 0x645, 0x627, 0x633,
0x633, 0x646, 0x62a, 0x20, 0x648, 0x6cc, 0x646, 0x633, 0x646, 0x62a, 0x633, 0x648,
0x6cc, 0x641, 0x62a, 0x200c, 0x6a9, 0x627, 0x631, 0x646, 0x62a, 0x62a, 0x6af, 0x648,
0x633, 0x6cc, 0x6af, 0x627, 0x644, 0x67e, 0x627, 0x62a, 0x6cc, 0x62e, 0x648, 0x627,
0x646, 0x627, 0x62a, 0x648, 0x631, 0x646, 0x62a, 0x648, 0x648, 0x646, 0x6a9, 0x648,
0x648, 0x631, 0x648, 0x627, 0x6cc, 0x62a, 0x200c, 0x647, 0x648, 0x631, 0x633, 0x648,
0x6cc, 0x646, 0x6cc, 0x67e, 0x6af, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a, 0x627, 0x62a,
0x62f, 0x6cc, 0x648, 0x6cc, 0x633, 0x62f, 0x648, 0x645, 0x648, 0x646, 0x20, 0x62f,
0x648, 0x631, 0x648, 0x6cc, 0x644, 0x645, 0x6a9, 0x648, 0x627, 0x631, 0x6cc, 0x645,
0x6a9, 0x200c, 0x645, 0x648, 0x631, 0x62f, 0x648, 0x631, 0x648, 0x62a, 0x631, 0x627,
0x634, 0x648, 0x648, 0x627, 0x644, 0x627, 0x646, 0x6af, 0x200c, 0x6cc, 0x631, 0x628,
0x6cc, 0x646, 0x622, 0x644, 0x645, 0x627, 0x62a, 0x6cc, 0x639, 0x64e, 0x645, 0x627,
0x646, 0x622, 0x646, 0x627, 0x62f, 0x6cc, 0x631, 0x622, 0x642, 0x62a, 0x627, 0x648,
0x622, 0x642, 0x62a, 0x648, 0x628, 0x647, 0x639, 0x634, 0x642, 0x200c, 0x622, 0x628,
0x627, 0x62f, 0x622, 0x62a, 0x6cc, 0x631, 0x627, 0x648, 0x628, 0x627, 0x646, 0x6a9,
0x648, 0x6a9, 0x628, 0x627, 0x631, 0x646, 0x627, 0x626, 0x648, 0x644, 0x628, 0x6cc,
0x634, 0x6a9, 0x6a9, 0x628, 0x631, 0x648, 0x646, 0x626, 0x6cc, 0x686, 0x6cc, 0x62a,
0x627, 0x686, 0x648, 0x6cc, 0x628, 0x627, 0x644, 0x633, 0x627, 0x646, 0x6a9, 0x644,
0x645, 0x628, 0x648, 0x62f, 0x6cc, 0x644, 0x6cc, 0x62f, 0x628, 0x6cc, 0x62f, 0x648,
0x634, 0x646, 0x628, 0x647, 0x641, 0x627, 0x645, 0x627, 0x6af, 0x648, 0x633, 0x62a,
0x627, 0x627, 0x644, 0x62e, 0x644, 0x6cc, 0x644, 0x647, 0x648, 0x634, 0x6cc, 0x200c,
0x645, 0x6cc, 0x646, 0x200c, 0x633, 0x6cc, 0x62a, 0x6cc, 0x647, 0x646, 0x6af, 0x200c,
0x6a9, 0x646, 0x6af, 0x62e, 0x648, 0x648, 0x62f, 0x627, 0x6cc, 0x631, 0x6a9, 0x648,
0x62a, 0x633, 0x6a9, 0x62c, 0x627, 0x6a9, 0x627, 0x631, 0x62a, 0x627, 0x62c, 0x627,
0x6cc, 0x627, 0x67e, 0x648, 0x631, 0x627, 0x627, 0x648, 0x631, 0x634, 0x644, 0x6cc,
0x645, 0x6a9, 0x631, 0x627, 0x686, 0x6cc, 0x6a9, 0x627, 0x62a, 0x645, 0x627, 0x646,
0x62f, 0x648, 0x62e, 0x627, 0x646, 0x62f, 0x6cc, 0x6af, 0x627, 0x6a9, 0x644, 0x6a9,
0x62a, 0x647, 0x6a9, 0x631, 0x627, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633,
0x6a9, 0x6a9, 0x648, 0x627, 0x644, 0x627, 0x644, 0x627, 0x645, 0x67e, 0x648, 0x631,
0x6a9, 0x648, 0x686, 0x6cc, 0x646, 0x6af, 0x6a9, 0x648, 0x6cc, 0x62a, 0x645, 0x627,
0x6a9, 0x627, 0x626, 0x648, 0x645, 0x627, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x645,
0x627, 0x6a9, 0x627, 0x633, 0x627, 0x631, 0x645, 0x627, 0x646, 0x6cc, 0x644, 0x646,
0x6cc, 0x6a9, 0x648, 0x632, 0x6cc, 0x627, 0x646, 0x648, 0x648, 0x648, 0x6a9, 0x648,
0x632, 0x646, 0x62a, 0x633, 0x6a9, 0x646, 0x648, 0x648, 0x633, 0x6cc, 0x628, 0x6cc,
0x631, 0x6cc, 0x633, 0x6a9, 0x627, 0x648, 0x631, 0x627, 0x644, 0x67e, 0x646, 0x648,
0x645, 0x200c, 0x67e, 0x646, 0x67e, 0x648, 0x646, 0x62a, 0x6cc, 0x627, 0x646, 0x627,
0x6a9, 0x67e, 0x6cc, 0x648, 0x646, 0x6af, 0x200c, 0x6cc, 0x627, 0x646, 0x6af, 0x642,
0x648, 0x633, 0x62a, 0x627, 0x646, 0x627, 0x6cc, 0x642, 0x6cc, 0x632, 0x6cc, 0x644,
0x200c, 0x627, 0x648, 0x631, 0x62f, 0x627, 0x633, 0x627, 0x62e, 0x627, 0x644, 0x6cc,
0x646, 0x633, 0x626, 0x648, 0x644, 0x634, 0x627, 0x646, 0x6af, 0x647, 0x627, 0x6cc,
0x633, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x631, 0x627, 0x633, 0x631, 0x62f, 0x646,
0x6a9, 0x648, 0x644, 0x6cc, 0x645, 0x633, 0x6a9, 0x62a, 0x627, 0x6cc, 0x67e, 0x647,
0x62a, 0x627, 0x634, 0x6a9, 0x646, 0x62f, 0x62a, 0x641, 0x644, 0x6cc, 0x633, 0x62a,
0x6cc, 0x645, 0x641, 0x648, 0x62a, 0x648, 0x6a9, 0x6cc, 0x648, 0x62a, 0x648, 0x645,
0x633, 0x6a9, 0x627, 0x648, 0x644, 0x627, 0x646, 0x200c, 0x628, 0x627, 0x62a, 0x648,
0x631, 0x627, 0x631, 0x648, 0x645, 0x686, 0x6cc, 0x627, 0x648, 0x633, 0x62a, 0x20,
0x646, 0x631, 0x627, 0x648, 0x6cc, 0x646, 0x62a, 0x6cc, 0x627, 0x646, 0x648, 0x644,
0x627, 0x62f, 0x6cc, 0x200c, 0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x6cc, 0x627, 0x6a9,
0x648, 0x62a, 0x633, 0x6a9, 0x6cc, 0x627, 0x646, 0x6af, 0x648, 0x646, 0x6cc, 0x6a9,
0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x631, 0x6af, 0x627, 0x6cc, 0x631, 0x648,
0x627, 0x646, 0x622, 0x632, 0x648, 0x631, 0x642, 0x646, 0x627, 0x631, 0x6cc, 0x6a9,
0x6cc, 0x67e, 0x200c, 0x648, 0x631, 0x62f, 0x645, 0x627, 0x62f, 0x6cc, 0x631, 0x627,
0x631, 0x6cc, 0x6a9, 0x6cc, 0x627, 0x648, 0x6cc, 0x6a9, 0x62c, 0x648, 0x631, 0x62c,
0x6cc, 0x627, 0x6cc, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x633, 0x646, 0x62a,
0x20, 0x647, 0x644, 0x646, 0x627, 0x627, 0x633, 0x62a, 0x627, 0x646, 0x644, 0x6cc,
0x622, 0x62f, 0x644, 0x627, 0x6cc, 0x62f, 0x628, 0x631, 0x6cc, 0x633, 0x628, 0x6cc,
0x646, 0x628, 0x631, 0x648, 0x6a9, 0x646, 0x200c, 0x647, 0x6cc, 0x644, 0x62f, 0x627,
0x631, 0x648, 0x6cc, 0x646, 0x627, 0x648, 0x6a9, 0x644, 0x627, 0x644, 0x6cc, 0x646,
0x62f, 0x645, 0x646, 0x644, 0x631, 0x62f, 0x647, 0x627, 0x648, 0x645, 0x644, 0x628,
0x648, 0x631, 0x646, 0x633, 0x6cc, 0x62f, 0x646, 0x6cc, 0x622, 0x645, 0x633, 0x62a,
0x631, 0x62f, 0x627, 0x645, 0x622, 0x646, 0x62f, 0x648, 0x631, 0x627, 0x622, 0x633,
0x62a, 0x631, 0x627, 0x62e, 0x627, 0x646, 0x622, 0x62a, 0x646, 0x628, 0x644, 0x6af,
0x631, 0x627, 0x62f, 0x628, 0x631, 0x644, 0x6cc, 0x646, 0x628, 0x631, 0x627, 0x62a,
0x6cc, 0x633, 0x644, 0x627, 0x648, 0x627, 0x628, 0x631, 0x648, 0x6a9, 0x633, 0x644,
0x628, 0x648, 0x62f, 0x627, 0x67e, 0x633, 0x62a, 0x628, 0x627, 0x632, 0x646, 0x6af,
0x646, 0x6a9, 0x6cc, 0x634, 0x6cc, 0x646, 0x627, 0x648, 0x6a9, 0x67e, 0x646, 0x647,
0x627, 0x6af, 0x62f, 0x648, 0x628, 0x644, 0x6cc, 0x646, 0x62c, 0x628, 0x644, 0x200c,
0x627, 0x644, 0x637, 0x627, 0x631, 0x642, 0x6af, 0x631, 0x646, 0x632, 0x6cc, 0x647,
0x644, 0x633, 0x6cc, 0x646, 0x6a9, 0x6cc, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654,
0x20, 0x645, 0x646, 0x627, 0x633, 0x62a, 0x627, 0x646, 0x628, 0x648, 0x644, 0x62c,
0x631, 0x632, 0x6cc, 0x6a9, 0x627, 0x644, 0x6cc, 0x646, 0x6cc, 0x646, 0x6af, 0x631,
0x627, 0x62f, 0x644, 0x6cc, 0x648, 0x628, 0x644, 0x6cc, 0x627, 0x646, 0x627, 0x644,
0x648, 0x6a9, 0x632, 0x627, 0x645, 0x628, 0x648, 0x631, 0x6af, 0x645, 0x627, 0x62f,
0x631, 0x6cc, 0x62f, 0x645, 0x627, 0x644, 0x62a, 0x645, 0x627, 0x631, 0x6cc, 0x647,
0x200c, 0x647, 0x627, 0x645, 0x646, 0x645, 0x6cc, 0x646, 0x633, 0x6a9, 0x627, 0x633,
0x644, 0x648, 0x67e, 0x627, 0x62f, 0x6af, 0x627, 0x631, 0x6cc, 0x62a, 0x633, 0x627,
0x633, 0x627, 0x645, 0x627, 0x631, 0x627, 0x633, 0x627, 0x646, 0x200c, 0x645, 0x627,
0x631, 0x6cc, 0x646, 0x648, 0x633, 0x627, 0x631, 0x627, 0x6cc, 0x648, 0x648, 0x633,
0x6cc, 0x645, 0x641, 0x631, 0x648, 0x67e, 0x644, 0x627, 0x633, 0x6a9, 0x648, 0x67e,
0x6cc, 0x647, 0x635, 0x648, 0x641, 0x6cc, 0x647, 0x627, 0x633, 0x62a, 0x6a9, 0x647,
0x644, 0x645, 0x62a, 0x627, 0x644, 0x6cc, 0x646, 0x62a, 0x6cc, 0x631, 0x627, 0x646,
0x627, 0x627, 0x648, 0x644, 0x6cc, 0x627, 0x646, 0x648, 0x641, 0x633, 0x6a9, 0x641,
0x627, 0x62f, 0x648, 0x62a, 0x633, 0x648, 0x627, 0x62a, 0x6cc, 0x6a9, 0x627, 0x646,
0x648, 0x6cc, 0x644, 0x646, 0x6cc, 0x648, 0x633, 0x648, 0x644, 0x6af, 0x627, 0x6af,
0x631, 0x627, 0x62f, 0x648, 0x631, 0x634, 0x648, 0x632, 0x627, 0x6af, 0x631, 0x628,
0x632, 0x648, 0x631, 0x6cc, 0x62e, 0x622, 0x646, 0x62a, 0x627, 0x646, 0x627, 0x646,
0x627, 0x631, 0x6cc, 0x648, 0x648, 0x634, 0x627, 0x6af, 0x648, 0x633, 0x6a9, 0x631,
0x6cc, 0x633, 0x645, 0x633, 0x6a9, 0x631, 0x6af, 0x648, 0x644, 0x646, 0x645, 0x627,
0x647, 0x647, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x648, 0x645, 0x648, 0x631, 0x6cc,
0x633, 0x645, 0x627, 0x6cc, 0x648, 0x62a, 0x631, 0x626, 0x648, 0x646, 0x6cc, 0x648,
0x646, 0x622, 0x67e, 0x6cc, 0x627, 0x627, 0x648, 0x6a9, 0x644, 0x646, 0x62f, 0x628,
0x648, 0x6af, 0x646, 0x648, 0x6cc, 0x644, 0x686, 0x62a, 0x645, 0x627, 0x6cc, 0x633,
0x62a, 0x631, 0x627, 0x641, 0x627, 0x62a, 0x647, 0x641, 0x627, 0x6a9, 0x627, 0x626,
0x648, 0x641, 0x648, 0x641, 0x6cc, 0x62c, 0x6cc, 0x641, 0x648, 0x646, 0x627, 0x641,
0x648, 0x62a, 0x6cc, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x633,
0x6af, 0x627, 0x645, 0x628, 0x6cc, 0x631, 0x6af, 0x648, 0x627, 0x62f, 0x627, 0x644,
0x200c, 0x6a9, 0x627, 0x646, 0x627, 0x644, 0x627, 0x646, 0x62f, 0x631, 0x628, 0x631,
0x6cc, 0x6a9, 0x631, 0x6cc, 0x62a, 0x6cc, 0x645, 0x627, 0x62a, 0x6cc, 0x6a9, 0x648,
0x633, 0x631, 0x627, 0x6cc, 0x6a9, 0x648, 0x627, 0x62c, 0x6cc, 0x644, 0x6cc, 0x646,
0x645, 0x627, 0x631, 0x6a9, 0x648, 0x632, 0x647, 0x645, 0x6cc, 0x62f, 0x648, 0x6cc,
0x646, 0x627, 0x626, 0x648, 0x631, 0x648, 0x646, 0x6cc, 0x648, 0x626, 0x647, 0x646,
0x648, 0x631, 0x641, 0x648, 0x644, 0x6a9, 0x646, 0x648, 0x645, 0x626, 0x627, 0x67e,
0x627, 0x6af, 0x648, 0x67e, 0x627, 0x6af, 0x648, 0x67e, 0x627, 0x644, 0x627, 0x626,
0x648, 0x67e, 0x6cc, 0x62a, 0x200c, 0x6a9, 0x631, 0x646, 0x67e, 0x627, 0x646, 0x67e,
0x6cc, 0x67e, 0x648, 0x631, 0x62a, 0x200c, 0x645, 0x648, 0x631, 0x632, 0x628, 0x6cc,
0x631, 0x627, 0x631, 0x648, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x633, 0x627, 0x6cc,
0x67e, 0x627, 0x646, 0x62a, 0x627, 0x647, 0x6cc, 0x62a, 0x6cc, 0x62a, 0x648, 0x646,
0x6af, 0x627, 0x62a, 0x627, 0x67e, 0x648, 0x648, 0x627, 0x644, 0x6cc, 0x633, 0x41,
0x62, 0x69, 0x64, 0x17c, 0x61, 0x6e, 0x42, 0x61, 0x6e, 0x64, 0x17c, 0x75,
0x6c, 0x42, 0x75, 0x17c, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x61, 0x44, 0x17c,
0x69, 0x62, 0x75, 0x74, 0x69, 0x41, 0x6c, 0x2d, 0x55, 0x6a, 0x75, 0x6e,
0x44, 0x17c, 0x75, 0x62, 0x61, 0x4b, 0x69, 0x6e, 0x73, 0x7a, 0x61, 0x73,
0x61, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x73, 0x7a, 0x75, 0x4e, 0x64,
0x17c, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x61, 0x77, 0x61, 0x6b, 0x73,
0x7a, 0x75, 0x74, 0x54, 0x72, 0x79, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x4b,
0x61, 0x6a, 0x6d, 0x61, 0x6e, 0x79, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x72,
0x79, 0x6b, 0x61, 0x53, 0x61, 0x6c, 0x77, 0x61, 0x64, 0x6f, 0x72, 0x47,
0x77, 0x61, 0x64, 0x65, 0x6c, 0x75, 0x70, 0x61, 0x47, 0x75, 0x6a, 0x61,
0x6e, 0x61, 0x48, 0x61, 0x77, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x74,
0x79, 0x6e, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20,
0x28, 0x6d, 0x69, 0x61, 0x73, 0x74, 0x6f, 0x29, 0x4e, 0x6f, 0x77, 0x79,
0x20, 0x4a, 0x6f, 0x72, 0x6b, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x50, 0xf3, 0x142, 0x6e,
0x6f, 0x63, 0x6e, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x50, 0xf3, 0x142, 0x6e, 0x6f,
0x63, 0x6e, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x50, 0xf3, 0x142,
0x6e, 0x6f, 0x63, 0x6e, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x72, 0x79,
0x6b, 0x6f, 0x41, 0x142, 0x6d, 0x61, 0x74, 0x79, 0x41, 0x6b, 0x74, 0x69,
0x75, 0x62, 0x69, 0x144, 0x73, 0x6b, 0x41, 0x73, 0x7a, 0x63, 0x68, 0x61,
0x62, 0x61, 0x64, 0x42, 0x61, 0x72, 0x6e, 0x61, 0x75, 0x142, 0x42, 0x69,
0x73, 0x7a, 0x6b, 0x65, 0x6b, 0x43, 0x7a, 0x79, 0x74, 0x61, 0x43, 0x7a,
0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x44, 0x61, 0x6d, 0x61,
0x73, 0x7a, 0x65, 0x6b, 0x44, 0x75, 0x73, 0x7a, 0x61, 0x6e, 0x62, 0x65,
0x4b, 0x6f, 0x62, 0x64, 0x6f, 0x44, 0x17c, 0x61, 0x6b, 0x61, 0x72, 0x74,
0x61, 0x4a, 0x65, 0x72, 0x6f, 0x7a, 0x6f, 0x6c, 0x69, 0x6d, 0x61, 0x4b,
0x61, 0x6d, 0x63, 0x7a, 0x61, 0x74, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61,
0x63, 0x7a, 0x69, 0x4b, 0x75, 0x77, 0x65, 0x6a, 0x74, 0x4e, 0x69, 0x6b,
0x6f, 0x7a, 0x6a, 0x61, 0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x75, 0x17a, 0x6e,
0x69, 0x65, 0x63, 0x6b, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x79, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x4b, 0x75, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x6a, 0x4b,
0x79, 0x7a, 0x79, 0x142, 0x6f, 0x72, 0x64, 0x61, 0x53, 0x7a, 0x61, 0x6e,
0x67, 0x68, 0x61, 0x6a, 0x53, 0x72, 0x69, 0x65, 0x64, 0x6e, 0x69, 0x65,
0x6b, 0x6f, 0x142, 0x79, 0x6d, 0x73, 0x6b, 0x54, 0x61, 0x73, 0x7a, 0x6b,
0x69, 0x65, 0x6e, 0x74, 0x55, 0x142, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x55, 0x72, 0x75, 0x6d, 0x63, 0x7a, 0x69, 0x55, 0x73, 0x74,
0x2d, 0x4e, 0x69, 0x65, 0x72, 0x61, 0x57, 0x69, 0x65, 0x6e, 0x74, 0x69,
0x61, 0x6e, 0x57, 0x142, 0x61, 0x64, 0x79, 0x77, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x79, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x45, 0x72, 0x79, 0x77, 0x61, 0x144, 0x41, 0x7a, 0x6f, 0x72,
0x79, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x72,
0x79, 0x6a, 0x73, 0x6b, 0x69, 0x65, 0x52, 0x65, 0x70, 0x75, 0x62, 0x6c,
0x69, 0x6b, 0x61, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x67,
0x6f, 0x20, 0x50, 0x72, 0x7a, 0x79, 0x6c, 0x105, 0x64, 0x6b, 0x61, 0x57,
0x79, 0x73, 0x70, 0x79, 0x20, 0x4f, 0x77, 0x63, 0x7a, 0x65, 0x4d, 0x61,
0x64, 0x65, 0x72, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20,
0x50, 0x6f, 0x142, 0x75, 0x64, 0x6e, 0x69, 0x6f, 0x77, 0x61, 0x15a, 0x77,
0x69, 0x119, 0x74, 0x61, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41,
0x73, 0x74, 0x72, 0x61, 0x63, 0x68, 0x61, 0x144, 0x41, 0x74, 0x65, 0x6e,
0x79, 0x42, 0x72, 0x61, 0x74, 0x79, 0x73, 0x142, 0x61, 0x77, 0x61, 0x42,
0x72, 0x75, 0x6b, 0x73, 0x65, 0x6c, 0x61, 0x42, 0x75, 0x6b, 0x61, 0x72,
0x65, 0x73, 0x7a, 0x74, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x73, 0x7a,
0x74, 0x4b, 0x69, 0x73, 0x7a, 0x79, 0x6e, 0x69, 0xf3, 0x77, 0x57, 0x79,
0x73, 0x70, 0x61, 0x20, 0x4d, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6d, 0x62,
0x75, 0x142, 0x4b, 0x69, 0x6a, 0xf3, 0x77, 0x4c, 0x69, 0x7a, 0x62, 0x6f,
0x6e, 0x61, 0x4c, 0x75, 0x62, 0x6c, 0x61, 0x6e, 0x61, 0x4c, 0x6f, 0x6e,
0x64, 0x79, 0x6e, 0x4d, 0x61, 0x64, 0x72, 0x79, 0x74, 0x4d, 0x69, 0x144,
0x73, 0x6b, 0x50, 0x61, 0x72, 0x79, 0x17c, 0x52, 0x7a, 0x79, 0x6d, 0x53,
0x61, 0x72, 0x61, 0x6a, 0x65, 0x77, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x74,
0xf3, 0x77, 0x53, 0x79, 0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c,
0x53, 0x7a, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x57, 0x61, 0x74,
0x79, 0x6b, 0x61, 0x6e, 0x57, 0x69, 0x65, 0x64, 0x65, 0x144, 0x57, 0x69,
0x6c, 0x6e, 0x6f, 0x57, 0x6f, 0x142, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x5a, 0x61, 0x67, 0x72, 0x7a, 0x65, 0x62, 0x5a, 0x75, 0x72, 0x79, 0x63,
0x68, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x79, 0x77,
0x61, 0x43, 0x7a, 0x61, 0x67, 0x6f, 0x73, 0x57, 0x79, 0x73, 0x70, 0x61,
0x20, 0x42, 0x6f, 0x17c, 0x65, 0x67, 0x6f, 0x20, 0x4e, 0x61, 0x72, 0x6f,
0x64, 0x7a, 0x65, 0x6e, 0x69, 0x61, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20,
0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x77, 0x65, 0x57, 0x79, 0x73, 0x70,
0x79, 0x20, 0x4b, 0x65, 0x72, 0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e, 0x61,
0x4d, 0x61, 0x6a, 0x6f, 0x74, 0x74, 0x61, 0x57, 0x79, 0x73, 0x70, 0x61,
0x20, 0x42, 0x6f, 0x75, 0x67, 0x61, 0x69, 0x6e, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x2019, 0x61, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x57, 0x69, 0x65,
0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x61, 0x46, 0x69, 0x64, 0x17c,
0x69, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x61, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x79, 0x43, 0x6f,
0x6e, 0x61, 0x63, 0x72, 0x69, 0x4a, 0x6f, 0x61, 0x6e, 0x65, 0x73, 0x62,
0x75, 0x72, 0x67, 0x6f, 0x43, 0x61, 0x72, 0x74, 0x75, 0x6d, 0x4d, 0x6f,
0x67, 0x61, 0x64, 0xed, 0x73, 0x63, 0x69, 0x6f, 0x4e, 0x61, 0x69, 0x72,
0xf3, 0x62, 0x69, 0x41, 0x6e, 0x74, 0xed, 0x67, 0x75, 0x61, 0x54, 0x75,
0x63, 0x75, 0x6d, 0xe3, 0x41, 0x73, 0x73, 0x75, 0x6e, 0xe7, 0xe3, 0x6f,
0x47, 0x75, 0x61, 0x69, 0x61, 0x71, 0x75, 0x69, 0x6c, 0x47, 0x75, 0x69,
0x61, 0x6e, 0x61, 0x43, 0x69, 0x64, 0x61, 0x64, 0x65, 0x20, 0x64, 0x6f,
0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f, 0x4d, 0x6f, 0x6e, 0x74, 0x65,
0x76, 0x69, 0x64, 0xe9, 0x75, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e,
0x6f, 0x72, 0x74, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6e,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x65, 0x53, 0xe3, 0x6f, 0x20, 0x42, 0x61, 0x72,
0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x75, 0x53, 0xe3, 0x6f, 0x20, 0x43,
0x72, 0x69, 0x73, 0x74, 0xf3, 0x76, 0xe3, 0x6f, 0x53, 0x61, 0x6e, 0x74,
0x61, 0x20, 0x4c, 0xfa, 0x63, 0x69, 0x61, 0x53, 0xe3, 0x6f, 0x20, 0x56,
0x69, 0x63, 0x65, 0x6e, 0x74, 0x65, 0x41, 0x6d, 0xe3, 0x41, 0x73, 0x67,
0x61, 0x62, 0x61, 0x74, 0x65, 0x42, 0x65, 0x69, 0x72, 0x75, 0x74, 0x65,
0x44, 0x75, 0x63, 0x68, 0x61, 0x6d, 0x62, 0x65, 0x43, 0x69, 0x64, 0x61,
0x64, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69,
0x20, 0x4d, 0x69, 0x6e, 0x68, 0x4a, 0x61, 0x63, 0x61, 0x72, 0x74, 0x61,
0x4a, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0xe9, 0x6d, 0x43, 0x61, 0x6c,
0x63, 0x75, 0x74, 0xe1, 0x4e, 0x69, 0x63, 0xf3, 0x73, 0x69, 0x61, 0x52,
0x69, 0x61, 0x64, 0x65, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e, 0x61,
0x58, 0x61, 0x6e, 0x67, 0x61, 0x69, 0x54, 0x65, 0x65, 0x72, 0xe3, 0x54,
0xf3, 0x71, 0x75, 0x69, 0x6f, 0x52, 0x61, 0x6e, 0x67, 0x75, 0x6d, 0x45,
0x63, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f,
0x43, 0x61, 0x6e, 0xe1, 0x72, 0x69, 0x61, 0x73, 0x49, 0x6c, 0x68, 0x61,
0x73, 0x20, 0x46, 0x61, 0x72, 0x6f, 0xe9, 0x47, 0x65, 0xf3, 0x72, 0x67,
0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75, 0x6c, 0x41, 0x6d, 0x73,
0x74, 0x65, 0x72, 0x64, 0xe3, 0x41, 0x73, 0x74, 0x72, 0x61, 0x63, 0xe3,
0x42, 0x65, 0x72, 0x6c, 0x69, 0x6d, 0x42, 0x75, 0x63, 0x61, 0x72, 0x65,
0x73, 0x74, 0x65, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x73, 0x74, 0x65,
0x48, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x71, 0x75, 0x65, 0x49, 0x6c, 0x68,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4d, 0xf4, 0x6e, 0x61,
0x63, 0x6f, 0x53, 0xf3, 0x66, 0x69, 0x61, 0x56, 0x61, 0x72, 0x73, 0xf3,
0x76, 0x69, 0x61, 0x5a, 0x75, 0x72, 0x69, 0x71, 0x75, 0x65, 0x4d, 0x61,
0x75, 0x72, 0xed, 0x63, 0x69, 0x6f, 0x52, 0x65, 0x75, 0x6e, 0x69, 0xe3,
0x6f, 0x43, 0x68, 0x61, 0x74, 0x6e, 0x61, 0x6d, 0x49, 0x6c, 0x68, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f, 0x61, 0x54, 0x61,
0x72, 0x61, 0x75, 0x61, 0x41, 0x64, 0x69, 0x73, 0x2d, 0x41, 0x62, 0x65,
0x62, 0x61, 0x44, 0x61, 0x63, 0x61, 0x72, 0x4a, 0x69, 0x62, 0x75, 0x74,
0x69, 0x43, 0x61, 0x6d, 0x70, 0x61, 0x6c, 0x61, 0x50, 0x6f, 0x72, 0x74,
0x6f, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x41, 0x72, 0x61, 0x67, 0x75, 0x61,
0x69, 0x6e, 0x61, 0x43, 0x61, 0x69, 0x6d, 0xe3, 0x6f, 0x43, 0x75, 0x72,
0x61, 0xe7, 0x61, 0x75, 0x44, 0x6f, 0x6d, 0xed, 0x6e, 0x69, 0x63, 0x61,
0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x76, 0x69, 0x64, 0x65, 0x75, 0x4d, 0x6f,
0x6e, 0x73, 0x65, 0x72, 0x72, 0x61, 0x74, 0x65, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x49, 0x6f, 0x72, 0x71, 0x75, 0x65, 0x50, 0x6f, 0x72, 0x74, 0x2d,
0x61, 0x75, 0x2d, 0x50, 0x72, 0x69, 0x6e, 0x63, 0x65, 0x50, 0x6f, 0x72,
0x74, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73, 0x70, 0x61, 0x6e, 0x68,
0x61, 0x41, 0x64, 0x65, 0x6d, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x42,
0x61, 0x67, 0x64, 0x61, 0x64, 0x65, 0x42, 0x61, 0x72, 0xe9, 0x6d, 0x42,
0x61, 0x6e, 0x67, 0x75, 0x65, 0x63, 0x6f, 0x71, 0x75, 0x65, 0x43, 0x61,
0x72, 0x61, 0x63, 0x68, 0x69, 0x43, 0x61, 0x74, 0x6d, 0x61, 0x6e, 0x64,
0x75, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x54, 0x65, 0x65, 0x72, 0xe3, 0x6f,
0x54, 0x69, 0x6d, 0x70, 0x68, 0x75, 0x52, 0x65, 0x69, 0x71, 0x75, 0x69,
0x61, 0x76, 0x69, 0x71, 0x75, 0x65, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41,
0x6d, 0x65, 0x73, 0x74, 0x65, 0x72, 0x64, 0xe3, 0x6f, 0x42, 0x75, 0x73,
0x69, 0x6e, 0x67, 0x65, 0x6e, 0x43, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61,
0x67, 0x61, 0x48, 0x65, 0x6c, 0x73, 0xed, 0x6e, 0x71, 0x75, 0x69, 0x61,
0x43, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x6f,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76, 0x6f, 0x53, 0xe3, 0x6f, 0x20, 0x4d,
0x61, 0x72, 0x69, 0x6e, 0x68, 0x6f, 0x54, 0x61, 0x6c, 0x69, 0x6d, 0x49,
0x6c, 0x68, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x61, 0x74, 0x61, 0x6c,
0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x4d,
0x61, 0x68, 0x65, 0x4d, 0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x61, 0x49,
0x6c, 0x68, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f,
0x61, 0x45, 0x66, 0x61, 0x74, 0x65, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x54, 0x61, 0x72, 0x61,
0x77, 0x61, 0xa05, 0xa2c, 0xa40, 0xa26, 0xa1c, 0xa3e, 0xa28, 0xa05, 0xa71, 0xa15,
0xa30, 0xa3e, 0xa10, 0xa21, 0xa3f, 0xa38, 0x20, 0xa05, 0xa2c, 0xa3e, 0xa2c, 0xa3e,
0xa05, 0xa32, 0xa1c, 0xa40, 0xa05, 0xa30, 0xa38, 0xa05, 0xa38, 0xa2e, 0xa3e, 0xa30,
0xa3e, 0xa2c, 0xa2e, 0xa47, 0xa15, 0xa4b, 0xa2c, 0xa3e, 0xa02, 0xa17, 0xa41, 0xa08,
0xa2c, 0xa3e, 0xa02, 0xa1c, 0xa41, 0xa32, 0xa2c, 0xa3f, 0xa38, 0xa3e, 0xa09, 0xa2c,
0xa32, 0xa70, 0xa1f, 0xa3e, 0xa07, 0xa30, 0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa1c, 0xa3c,
0xa3e, 0xa35, 0xa3f, 0xa32, 0xa47, 0xa2c, 0xa41, 0xa1c, 0xa41, 0xa70, 0xa2c, 0xa41,
0xa30, 0xa3e, 0xa15, 0xa48, 0xa30, 0xa4b, 0xa15, 0xa3e, 0xa38, 0xa3e, 0xa2c, 0xa32,
0xa3e, 0xa02, 0xa15, 0xa3e, 0xa38, 0xa40, 0xa0a, 0xa1f, 0xa3e, 0xa15, 0xa4b, 0xa28,
0xa47, 0xa15, 0xa30, 0xa40, 0xa21, 0xa15, 0xa3e, 0xa30, 0xa26, 0xa3e, 0xa30, 0x20,
0xa0f, 0xa38, 0x20, 0xa38, 0xa32, 0xa3e, 0xa2e, 0xa1c, 0xa3c, 0xa40, 0xa2c, 0xa42,
0xa24, 0xa40, 0xa21, 0xa4c, 0xa06, 0xa32, 0xa3e, 0xa05, 0xa32, 0x20, 0xa05, 0xa2f,
0xa42, 0xa28, 0xa2b, 0xa30, 0xa40, 0xa1f, 0xa3e, 0xa09, 0xa28, 0xa17, 0xa3e, 0xa2c,
0xa4b, 0xa30, 0xa4b, 0xa28, 0xa39, 0xa30, 0xa3e, 0xa30, 0xa47, 0xa1c, 0xa4b, 0xa39,
0xa28, 0xa38, 0xa2c, 0xa30, 0xa17, 0xa1c, 0xa42, 0xa2c, 0xa3e, 0xa15, 0xa70, 0xa2a,
0xa3e, 0xa32, 0xa3e, 0xa16, 0xa3e, 0xa30, 0xa1f, 0xa4c, 0xa2e, 0xa15, 0xa3f, 0xa17,
0xa3e, 0xa32, 0xa40, 0xa15, 0xa3f, 0xa70, 0xa38, 0xa3c, 0xa3e, 0xa38, 0xa3e, 0xa32,
0xa3e, 0xa17, 0xa4b, 0xa38, 0xa32, 0xa3f, 0xa2c, 0xa30, 0xa35, 0xa3f, 0xa32, 0xa47,
0xa32, 0xa4b, 0xa2e, 0xa32, 0xa41, 0xa06, 0xa02, 0xa21, 0xa3e, 0xa32, 0xa41, 0xa2c,
0xa41, 0xa2e, 0xa2c, 0xa3e, 0xa38, 0xa3c, 0xa40, 0xa32, 0xa41, 0xa38, 0xa3e, 0xa15,
0xa3e, 0xa2e, 0xa3e, 0xa32, 0xa3e, 0xa2c, 0xa4b, 0xa2e, 0xa3e, 0xa2a, 0xa41, 0xa1f,
0xa4b, 0xa2e, 0xa38, 0xa47, 0xa30, 0xa42, 0xa0f, 0xa2e, 0xa2c, 0xa3e, 0xa2c, 0xa3e,
0xa28, 0xa47, 0xa2e, 0xa4b, 0xa17, 0xa3e, 0xa26, 0xa3f, 0xa38, 0xa3c, 0xa42, 0xa2e,
0xa4b, 0xa28, 0xa30, 0xa4b, 0xa35, 0xa40, 0xa06, 0xa28, 0xa48, 0xa30, 0xa4b, 0xa2c,
0xa40, 0xa10, 0xa02, 0xa1c, 0xa3e, 0xa2e, 0xa47, 0xa28, 0xa3e, 0xa28, 0xa3f, 0xa06,
0xa2e, 0xa47, 0xa28, 0xa4c, 0xa06, 0xa15, 0xa38, 0xa3c, 0xa3e, 0xa1f, 0xa09, 0xa06,
0xa17, 0xa3e, 0xa21, 0xa42, 0xa17, 0xa42, 0xa2a, 0xa4b, 0xa30, 0xa1f, 0xa4b, 0x2d,
0xa28, 0xa4b, 0xa35, 0xa4b, 0xa38, 0xa3e, 0xa13, 0x20, 0xa1f, 0xa4b, 0xa2e, 0xa24,
0xa4d, 0xa30, 0xa3f, 0xa2a, 0xa4b, 0xa32, 0xa40, 0xa1f, 0xa41, 0xa28, 0xa3f, 0xa38,
0xa35, 0xa3f, 0xa70, 0xa21, 0xa39, 0xa4b, 0xa07, 0xa15, 0xa0f, 0xa21, 0xa15, 0xa10,
0xa02, 0xa15, 0xa30, 0xa47, 0xa1c, 0xa05, 0xa70, 0xa17, 0xa41, 0xa07, 0xa32, 0xa3e,
0xa10, 0xa02, 0xa1f, 0xa40, 0xa17, 0xa41, 0xa06, 0xa06, 0xa30, 0xa17, 0xa41, 0xa06,
0xa07, 0xa28, 0xa3e, 0xa2c, 0xa42, 0xa08, 0xa28, 0xa38, 0x20, 0xa06, 0xa07, 0xa30,
0xa38, 0xa15, 0xa48, 0xa1f, 0xa3e, 0xa2e, 0xa3e, 0xa30, 0xa15, 0xa3e, 0xa15, 0xa4b,
0xa30, 0xa21, 0xa4b, 0xa2c, 0xa3e, 0xa1c, 0xa42, 0xa1c, 0xa41, 0xa0f, 0xa32, 0xa3e,
0x20, 0xa30, 0xa3f, 0xa09, 0xa1c, 0xa3e, 0xa2e, 0xa48, 0xa02, 0xa21, 0xa4b, 0xa1c,
0xa3c, 0xa3e, 0xa30, 0xa3f, 0xa13, 0x20, 0xa17, 0xa48, 0xa32, 0xa47, 0xa17, 0xa4b,
0xa38, 0xa38, 0xa3e, 0xa32, 0xa1f, 0xa3e, 0xa38, 0xa47, 0xa28, 0x20, 0xa1c, 0xa41,
0xa06, 0xa28, 0xa38, 0xa47, 0xa28, 0x20, 0xa32, 0xa42, 0xa08, 0xa38, 0xa1f, 0xa41,
0xa15, 0xa41, 0xa2e, 0xa28, 0xa09, 0xa38, 0xa3c, 0xa35, 0xa3e, 0xa07, 0xa06, 0xa05,
0xa30, 0xa42, 0xa2c, 0xa3e, 0xa10, 0xa38, 0xa41, 0xa70, 0xa15, 0xa40, 0xa05, 0xa28,
0xa10, 0xa1f, 0xa40, 0xa15, 0xa4b, 0xa15, 0xa28, 0xa2c, 0xa3e, 0xa39, 0xa40, 0xa06,
0xa2c, 0xa3e, 0xa39, 0xa40, 0xa06, 0x20, 0xa2c, 0xa3e, 0xa02, 0xa26, 0xa47, 0xa30,
0xa38, 0xa2c, 0xa3e, 0xa30, 0xa2c, 0xa3e, 0xa21, 0xa4b, 0xa38, 0xa2c, 0xa47, 0xa32,
0xa47, 0xa2e, 0xa2c, 0xa47, 0xa32, 0xa40, 0xa1c, 0xa3c, 0xa2c, 0xa32, 0xa48, 0xa02,
0xa15, 0x2d, 0xa38, 0xa48, 0xa2c, 0xa32, 0xa3e, 0xa28, 0xa2c, 0xa4b, 0xa06, 0x20,
0xa35, 0xa3f, 0xa38, 0xa1f, 0xa3e, 0xa2c, 0xa4b, 0xa17, 0xa4b, 0xa1f, 0xa3e, 0xa2c,
0xa4b, 0xa07, 0xa38, 0xa15, 0xa48, 0xa2e, 0xa2c, 0xa4d, 0xa30, 0xa3f, 0xa1c, 0x20,
0xa2c, 0xa47, 0xa15, 0xa48, 0xa02, 0xa2a, 0xa4b, 0x20, 0xa17, 0xa4d, 0xa30, 0xa3e,
0xa02, 0xa21, 0xa47, 0xa15, 0xa48, 0xa28, 0xa15, 0xa41, 0xa28, 0xa15, 0xa48, 0xa30,
0xa3e, 0xa15, 0xa3e, 0xa38, 0xa15, 0xa47, 0xa2f, 0xa47, 0xa28, 0xa47, 0xa15, 0xa47,
0xa2e, 0xa48, 0xa28, 0xa38, 0xa3c, 0xa3f, 0xa15, 0xa3e, 0xa17, 0xa4b, 0xa1a, 0xa3f,
0xa39, 0xa41, 0xa06, 0xa39, 0xa41, 0xa06, 0xa38, 0xa3f, 0xa09, 0xa21, 0xa3e, 0xa21,
0x20, 0xa39, 0xa41, 0xa06, 0xa30, 0xa47, 0xa1c, 0xa3c, 0xa15, 0xa4b, 0xa38, 0xa1f,
0xa3e, 0x20, 0xa30, 0xa40, 0xa15, 0xa3e, 0xa15, 0xa4d, 0xa30, 0xa48, 0xa38, 0xa1f,
0xa28, 0xa15, 0xa2f, 0xa41, 0xa0f, 0xa2c, 0xa3e, 0xa15, 0xa41, 0xa30, 0xa3e, 0xa15,
0xa3e, 0xa13, 0xa21, 0xa48, 0xa28, 0xa2e, 0xa3e, 0xa30, 0xa15, 0xa38, 0xa3c, 0xa4c,
0xa28, 0xa21, 0xa4c, 0xa05, 0xa38, 0xa28, 0xa21, 0xa4c, 0xa05, 0xa38, 0xa28, 0x20,
0xa15, 0xa4d, 0xa30, 0xa40, 0xa15, 0xa21, 0xa47, 0xa28, 0xa35, 0xa30, 0xa21, 0xa3f,
0xa1f, 0xa30, 0xa4b, 0xa07, 0xa1f, 0xa21, 0xa4b, 0xa2e, 0xa40, 0xa28, 0xa3f, 0xa15,
0xa3e, 0xa10, 0xa21, 0xa2e, 0xa4b, 0xa02, 0xa1f, 0xa28, 0xa2f, 0xa41, 0xa30, 0xa42,
0xa28, 0xa40, 0xa2a, 0xa47, 0xa05, 0xa32, 0x20, 0xa38, 0xa32, 0xa35, 0xa3e, 0xa21,
0xa4b, 0xa30, 0xa2b, 0xa4b, 0xa30, 0xa1f, 0x20, 0xa28, 0xa48, 0xa32, 0xa38, 0xa28,
0xa2b, 0xa4b, 0xa30, 0xa1f, 0xa3e, 0xa32, 0xa47, 0xa1c, 0xa3c, 0xa3e, 0xa17, 0xa32,
0xa47, 0xa38, 0x20, 0xa2c, 0xa47, 0xa17, 0xa42, 0xa1c, 0xa3c, 0x20, 0xa2c, 0xa47,
0xa17, 0xa30, 0xa3e, 0xa02, 0xa21, 0x20, 0xa24, 0xa41, 0xa30, 0xa15, 0xa17, 0xa4d,
0xa30, 0xa47, 0xa28, 0xa3e, 0xa21, 0xa3e, 0xa17, 0xa41, 0xa06, 0xa21, 0xa47, 0xa32,
0xa4b, 0xa2a, 0xa17, 0xa41, 0xa06, 0xa1f, 0xa47, 0xa2e, 0xa3e, 0xa32, 0xa3e, 0xa17,
0xa41, 0xa06, 0xa07, 0xa15, 0xa35, 0xa3f, 0xa32, 0xa17, 0xa41, 0xa06, 0xa28, 0xa3e,
0xa39, 0xa48, 0xa32, 0xa40, 0xa2b, 0xa48, 0xa15, 0xa38, 0xa39, 0xa35, 0xa3e, 0xa28,
0xa3e, 0xa39, 0xa30, 0xa2e, 0xa4b, 0xa38, 0xa3f, 0xa71, 0xa32, 0xa4b, 0xa07, 0xa70,
0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa2a, 0xa4b, 0xa32, 0xa3f, 0xa38, 0xa28, 0xa4b,
0xa15, 0xa38, 0x2c, 0x20, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa2e,
0xa3e, 0xa30, 0xa47, 0xa02, 0xa17, 0xa4b, 0x2c, 0x20, 0xa07, 0xa70, 0xa21, 0xa40,
0xa06, 0xa28, 0xa3e, 0xa2a, 0xa40, 0xa1f, 0xa30, 0xa38, 0xa2c, 0xa30, 0xa17, 0x2c,
0x20, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa1f, 0xa48, 0xa32, 0x20,
0xa38, 0xa3f, 0xa1f, 0xa40, 0x2c, 0x20, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28,
0xa3e, 0xa35, 0xa47, 0xa35, 0xa47, 0x2c, 0x20, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06,
0xa28, 0xa3e, 0xa35, 0xa3f, 0xa70, 0xa38, 0xa47, 0xa28, 0xa47, 0xa38, 0x2c, 0x20,
0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa35, 0xa3f, 0xa28, 0xa2e, 0xa48,
0xa15, 0x2c, 0x20, 0xa07, 0xa70, 0xa21, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa07, 0xa28,
0xa41, 0xa35, 0xa3f, 0xa15, 0xa07, 0xa15, 0xa3e, 0xa32, 0xa41, 0xa08, 0xa1f, 0xa1c,
0xa2e, 0xa3e, 0xa07, 0xa15, 0xa3e, 0xa1c, 0xa2f, 0xa42, 0xa28, 0xa4b, 0xa32, 0xa41,
0xa08, 0xa38, 0xa35, 0xa3f, 0xa32, 0xa47, 0xa2e, 0xa4b, 0xa02, 0xa1f, 0xa40, 0xa38,
0xa48, 0xa32, 0xa4b, 0x2c, 0x20, 0xa15, 0xa48, 0xa02, 0xa1f, 0xa15, 0xa40, 0xa15,
0xa4d, 0xa30, 0xa3e, 0xa32, 0xa47, 0xa02, 0xa26, 0xa3f, 0xa1c, 0xa15, 0xa32, 0xa3e,
0x20, 0xa2a, 0xa3e, 0xa1c, 0xa3c, 0xa32, 0xa40, 0xa2e, 0xa3e, 0xa32, 0xa3e, 0xa38,
0x20, 0xa10, 0xa02, 0xa1c, 0xa32, 0xa38, 0xa32, 0xa4b, 0xa05, 0xa30, 0x20, 0xa2a,
0xa4d, 0xa30, 0xa3f, 0xa70, 0xa38, 0x2019, 0xa38, 0x20, 0xa15, 0xa35, 0xa3e, 0xa30,
0xa1f, 0xa30, 0xa2e, 0xa48, 0xa38, 0xa40, 0xa13, 0xa2e, 0xa28, 0xa3e, 0xa17, 0xa41,
0xa06, 0xa2e, 0xa28, 0xa4c, 0xa38, 0xa2e, 0xa48, 0xa30, 0xa40, 0xa17, 0xa4b, 0xa1f,
0xa2e, 0xa3e, 0xa30, 0xa1f, 0xa40, 0xa28, 0xa3f, 0xa15, 0xa2e, 0xa3e, 0xa1f, 0xa3e,
0xa2e, 0xa4b, 0xa30, 0xa38, 0xa2e, 0xa1c, 0xa3c, 0xa47, 0xa24, 0xa32, 0xa3e, 0xa28,
0xa2e, 0xa48, 0xa28, 0xa4b, 0xa2e, 0xa3f, 0xa28, 0xa40, 0xa2e, 0xa47, 0xa30, 0xa3f,
0xa21, 0xa3e, 0xa2e, 0xa47, 0xa1f, 0xa32, 0xa3e, 0xa15, 0xa3e, 0xa1f, 0xa32, 0xa3e,
0xa2e, 0xa48, 0xa15, 0xa38, 0xa40, 0xa15, 0xa4b, 0x20, 0xa38, 0xa3f, 0xa1f, 0xa40,
0xa2e, 0xa3f, 0xa15, 0xa47, 0xa32, 0xa28, 0xa2e, 0xa4b, 0xa02, 0xa15, 0xa1f, 0xa28,
0xa2e, 0xa4b, 0xa28, 0xa1f, 0xa47, 0xa30, 0xa40, 0xa2e, 0xa4b, 0xa02, 0xa1f, 0xa47,
0xa35, 0xa40, 0xa21, 0xa40, 0xa13, 0xa2e, 0xa4b, 0xa02, 0xa1f, 0xa38, 0xa47, 0xa30,
0xa3e, 0xa24, 0xa28, 0xa3e, 0xa38, 0xa3e, 0xa13, 0xa28, 0xa3f, 0xa0a, 0x20, 0xa2f,
0xa3e, 0xa30, 0xa15, 0xa28, 0xa4c, 0xa30, 0xa4b, 0xa28, 0xa39, 0xa3e, 0xa2c, 0xa3f,
0xa09, 0xa32, 0xa3e, 0x2c, 0x20, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa21,
0xa15, 0xa4b, 0xa1f, 0xa3e, 0xa38, 0xa47, 0xa02, 0xa1f, 0xa30, 0x2c, 0x20, 0xa09,
0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa21, 0xa15, 0xa4b, 0xa1f, 0xa3e, 0xa28, 0xa3f,
0xa0a, 0x20, 0xa38, 0xa32, 0xa47, 0xa2e, 0x2c, 0x20, 0xa09, 0xa71, 0xa24, 0xa30,
0xa40, 0x20, 0xa21, 0xa15, 0xa4b, 0xa1f, 0xa3e, 0xa28, 0xa42, 0xa15, 0xa13, 0xa1c,
0xa40, 0xa28, 0xa3e, 0xa17, 0xa3e, 0xa2a, 0xa28, 0xa3e, 0xa2e, 0xa3e, 0xa2a, 0xa48,
0xa30, 0xa3e, 0xa2e, 0xa30, 0xa40, 0xa2c, 0xa4b, 0xa2b, 0xa3f, 0xa28, 0xa3f, 0xa15,
0xa38, 0xa2a, 0xa4b, 0xa30, 0xa1f, 0x2d, 0xa14, 0x2d, 0xa2a, 0xa4d, 0xa30, 0xa3f,
0xa70, 0xa38, 0xa2a, 0xa4b, 0xa30, 0xa1f, 0x20, 0xa14, 0xa2b, 0x20, 0xa38, 0xa2a,
0xa47, 0xa28, 0xa2a, 0xa4b, 0xa30, 0xa1f, 0xa4b, 0x20, 0xa35, 0xa47, 0xa32, 0xa4d,
0xa39, 0xa4b, 0xa2a, 0xa3f, 0xa0a, 0xa30, 0xa1f, 0xa4b, 0x20, 0xa30, 0xa40, 0xa15,
0xa4b, 0xa2a, 0xa70, 0xa1f, 0xa3e, 0x20, 0xa05, 0xa30, 0xa47, 0xa28, 0xa38, 0xa30,
0xa48, 0xa02, 0xa15, 0xa3f, 0xa28, 0x20, 0xa07, 0xa28, 0xa32, 0xa48, 0xa1f, 0xa30,
0xa47, 0xa38, 0xa40, 0xa2b, 0xa47, 0xa30, 0xa48, 0xa1c, 0xa40, 0xa28, 0xa3e, 0xa30,
0xa48, 0xa1c, 0xa3c, 0xa4b, 0xa32, 0xa3f, 0xa0a, 0xa1f, 0xa30, 0xa40, 0xa13, 0x20,
0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa02, 0xa15, 0xa4b, 0xa38, 0xa47, 0xa02, 0xa1f, 0xa3e,
0xa30, 0xa2e, 0xa38, 0xa47, 0xa02, 0xa1f, 0xa40, 0xa06, 0xa17, 0xa4b, 0xa38, 0xa47,
0xa02, 0xa1f, 0xa4b, 0x20, 0xa21, 0xa4b, 0xa2e, 0xa3f, 0xa70, 0xa17, 0xa4b, 0xa38,
0xa3e, 0xa13, 0x20, 0xa2a, 0xa4c, 0xa32, 0xa4b, 0xa07, 0xa71, 0xa1f, 0xa4b, 0xa15,
0xa4b, 0xa30, 0xa1f, 0xa42, 0xa30, 0xa2e, 0xa40, 0xa1f, 0xa38, 0xa3f, 0xa1f, 0xa15,
0xa3e, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20, 0xa2c, 0xa3e, 0xa30, 0xa25, 0xa47, 0xa32,
0xa47, 0xa2e, 0xa40, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20, 0xa1c, 0xa4c, 0xa39, 0xa28,
0xa38, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20, 0xa15, 0xa3f, 0xa1f, 0xa38, 0xa38, 0xa47,
0xa02, 0xa1f, 0x20, 0xa32, 0xa41, 0xa38, 0xa40, 0xa06, 0xa38, 0xa47, 0xa02, 0xa1f,
0x20, 0xa25, 0xa4b, 0xa2e, 0xa38, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20, 0xa35, 0xa3f,
0xa28, 0xa38, 0xa47, 0xa02, 0xa1f, 0xa38, 0xa35, 0xa3f, 0xa2b, 0xa1f, 0x20, 0xa15,
0xa30, 0xa70, 0xa1f, 0xa1f, 0xa47, 0xa17, 0xa41, 0xa38, 0xa40, 0xa17, 0xa32, 0xa2a,
0xa3e, 0xa25, 0xa41, 0xa32, 0xa47, 0xa1f, 0xa3f, 0xa1c, 0xa42, 0xa06, 0xa28, 0xa3e,
0xa1f, 0xa4b, 0xa30, 0xa3e, 0xa02, 0xa1f, 0xa4b, 0xa1f, 0xa4b, 0xa30, 0xa1f, 0xa4b,
0xa32, 0xa3e, 0xa35, 0xa48, 0xa28, 0xa15, 0xa42, 0xa35, 0xa30, 0xa35, 0xa3e, 0xa08,
0xa1f, 0xa39, 0xa4c, 0xa30, 0xa38, 0xa35, 0xa3f, 0xa28, 0xa40, 0xa2a, 0xa48, 0xa17,
0xa2f, 0xa15, 0xa41, 0xa24, 0xa24, 0xa21, 0xa47, 0xa35, 0xa3f, 0xa38, 0xa21, 0xa3f,
0xa09, 0xa2e, 0xa4b, 0xa02, 0xa1f, 0x20, 0xa21, 0xa3f, 0xa09, 0xa30, 0xa35, 0xa3f,
0xa32, 0xa47, 0xa2e, 0xa48, 0xa15, 0xa15, 0xa35, 0xa48, 0xa30, 0xa40, 0xa2e, 0xa4c,
0xa38, 0xa28, 0xa2e, 0xa48, 0xa15, 0xa2e, 0xa41, 0xa30, 0xa21, 0xa4b, 0xa2a, 0xa3e,
0xa2e, 0xa30, 0xa30, 0xa4b, 0xa25, 0xa47, 0xa30, 0xa3e, 0xa38, 0xa35, 0xa4b, 0xa2f,
0xa3e, 0xa1f, 0xa30, 0xa4b, 0xa32, 0xa35, 0xa4b, 0xa38, 0xa1f, 0xa4b, 0xa15, 0xa32,
0xa4b, 0xa02, 0xa17, 0xa08, 0xa05, 0xa30, 0xa2c, 0xa3e, 0xa07, 0xa28, 0xa05, 0xa21,
0xa47, 0xa28, 0xa05, 0xa32, 0xa2e, 0xa47, 0xa1f, 0xa40, 0xa05, 0xa2e, 0xa3e, 0xa28,
0xa10, 0xa28, 0xa3e, 0xa21, 0xa3e, 0xa07, 0xa30, 0xa05, 0xa15, 0xa24, 0xa4c, 0xa05,
0xa15, 0xa24, 0xa4b, 0xa2c, 0xa47, 0xa05, 0xa38, 0xa3c, 0xa17, 0xa3e, 0xa2c, 0xa3e,
0xa1f, 0xa0f, 0xa24, 0xa47, 0xa30, 0xa3e, 0xa13, 0xa2c, 0xa17, 0xa26, 0xa3e, 0xa26,
0xa2c, 0xa39, 0xa3f, 0xa30, 0xa40, 0xa28, 0xa2c, 0xa3e, 0xa15, 0xa42, 0xa2c, 0xa48,
0xa02, 0xa15, 0xa3e, 0xa15, 0xa2c, 0xa30, 0xa28, 0xa4c, 0xa32, 0xa2c, 0xa48, 0xa30,
0xa42, 0xa24, 0xa2c, 0xa3f, 0xa38, 0xa3c, 0xa15, 0xa47, 0xa15, 0xa2c, 0xa30, 0xa42,
0xa28, 0xa47, 0xa08, 0xa1a, 0xa3f, 0xa24, 0xa3e, 0xa1a, 0xa4b, 0xa07, 0xa32, 0xa2c,
0xa3e, 0xa32, 0xa38, 0xa28, 0xa15, 0xa4b, 0xa32, 0xa70, 0xa2c, 0xa4b, 0xa21, 0xa48,
0xa2e, 0xa38, 0xa15, 0xa38, 0xa22, 0xa3e, 0xa15, 0xa3e, 0xa21, 0xa3f, 0xa32, 0xa40,
0xa26, 0xa41, 0xa2c, 0xa08, 0xa26, 0xa41, 0xa38, 0xa3c, 0xa3e, 0xa02, 0xa2c, 0xa47,
0xa2b, 0xa3e, 0xa2e, 0xa3e, 0xa17, 0xa41, 0xa38, 0xa1f, 0xa3e, 0xa17, 0xa3e, 0xa1c,
0xa3c, 0xa3e, 0xa39, 0xa47, 0xa2c, 0xa30, 0xa3e, 0xa28, 0xa39, 0xa4b, 0x20, 0xa1a,
0xa40, 0x20, 0xa2e, 0xa3f, 0xa28, 0xa4d, 0xa39, 0x20, 0xa38, 0xa3f, 0xa1f, 0xa40,
0xa39, 0xa3e, 0xa02, 0xa17, 0x20, 0xa15, 0xa3e, 0xa02, 0xa17, 0xa39, 0xa4b, 0xa35,
0xa21, 0xa07, 0xa30, 0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0xa1c, 0xa15, 0xa3e, 0xa30,
0xa24, 0xa3e, 0xa1c, 0xa2f, 0xa3e, 0xa2a, 0xa41, 0xa30, 0xa3e, 0xa1c, 0xa47, 0xa30,
0xa42, 0xa38, 0xa32, 0xa2e, 0xa15, 0xa3e, 0xa2c, 0xa41, 0xa32, 0xa15, 0xa2e, 0xa1a,
0xa1f, 0xa15, 0xa3e, 0xa15, 0xa30, 0xa3e, 0xa1a, 0xa40, 0xa15, 0xa3e, 0xa20, 0xa2e,
0xa3e, 0xa02, 0xa21, 0xa42, 0xa16, 0xa3e, 0xa28, 0xa21, 0xa3f, 0xa17, 0xa3e, 0xa15,
0xa4b, 0xa32, 0xa15, 0xa3e, 0xa24, 0xa3e, 0xa15, 0xa30, 0xa48, 0xa38, 0xa28, 0xa47,
0xa1c, 0xa30, 0xa38, 0xa15, 0xa41, 0xa06, 0xa32, 0xa3e, 0xa32, 0xa70, 0xa2a, 0xa41,
0xa30, 0xa15, 0xa41, 0xa1a, 0xa3f, 0xa70, 0xa17, 0xa15, 0xa41, 0xa35, 0xa48, 0xa24,
0xa2e, 0xa15, 0xa3e, 0xa09, 0xa2e, 0xa48, 0xa17, 0xa47, 0xa21, 0xa28, 0xa2e, 0xa15,
0xa38, 0xa3e, 0xa30, 0xa2e, 0xa28, 0xa40, 0xa32, 0xa3e, 0xa2e, 0xa38, 0xa15, 0xa1f,
0xa28, 0xa3f, 0xa15, 0xa4b, 0xa38, 0xa40, 0xa06, 0xa28, 0xa4b, 0xa35, 0xa4b, 0xa15,
0xa41, 0xa1c, 0xa3c, 0xa28, 0xa47, 0xa1f, 0xa38, 0xa15, 0xa28, 0xa4b, 0xa35, 0xa4b,
0xa38, 0xa3f, 0xa2c, 0xa40, 0xa30, 0xa38, 0xa15, 0xa13, 0xa2e, 0xa38, 0xa15, 0xa13,
0xa30, 0xa32, 0xa2b, 0xa28, 0xa4b, 0xa2e, 0x20, 0xa2a, 0xa47, 0xa28, 0xa39, 0xa2a,
0xa4c, 0xa02, 0xa1f, 0xa40, 0xa06, 0xa28, 0xa3e, 0xa15, 0xa2a, 0xa2f, 0xa4b, 0xa02,
0xa17, 0xa2f, 0xa3e, 0xa02, 0xa17, 0xa15, 0xa24, 0xa30, 0xa15, 0xa4b, 0xa38, 0xa24,
0xa3e, 0xa28, 0xa3e, 0xa0f, 0xa15, 0xa3f, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa4b, 0xa30,
0xa21, 0xa3e, 0xa30, 0xa3f, 0xa06, 0xa27, 0xa38, 0xa16, 0xa32, 0xa40, 0xa28, 0xa38,
0xa2e, 0xa30, 0xa15, 0xa70, 0xa26, 0xa38, 0xa3f, 0xa09, 0xa32, 0xa38, 0xa3c, 0xa70,
0xa18, 0xa3e, 0xa08, 0xa38, 0xa3f, 0xa70, 0xa17, 0xa3e, 0xa2a, 0xa41, 0xa30, 0xa38,
0xa30, 0xa3f, 0xa21, 0xa28, 0xa40, 0xa15, 0xa4b, 0xa32, 0xa3f, 0xa38, 0xa15, 0xa24,
0xa48, 0xa2a, 0xa08, 0xa24, 0xa3e, 0xa38, 0xa3c, 0xa15, 0xa70, 0xa26, 0xa1f, 0xa2c,
0xa3f, 0xa32, 0xa3f, 0xa38, 0xa40, 0xa24, 0xa47, 0xa39, 0xa30, 0xa3e, 0xa28, 0xa25,
0xa3f, 0xa70, 0xa2b, 0xa42, 0xa1f, 0xa4b, 0xa15, 0xa40, 0xa13, 0xa1f, 0xa4b, 0xa2e,
0xa38, 0xa15, 0xa09, 0xa32, 0xa3e, 0xa28, 0xa2c, 0xa3e, 0xa24, 0xa30, 0xa0a, 0xa30,
0xa42, 0xa2e, 0xa15, 0xa40, 0xa09, 0xa38, 0xa24, 0x2d, 0xa28, 0xa47, 0xa30, 0xa3e,
0xa35, 0xa3e, 0xa0f, 0xa28, 0xa1f, 0xa3f, 0xa06, 0xa28, 0xa47, 0xa35, 0xa32, 0xa3e,
0xa26, 0xa40, 0xa35, 0xa4b, 0xa38, 0xa24, 0xa15, 0xa2f, 0xa15, 0xa41, 0xa24, 0xa38,
0xa15, 0xa30, 0xa70, 0xa17, 0xa42, 0xa28, 0xa2f, 0xa15, 0xa47, 0xa24, 0xa30, 0xa3f,
0xa28, 0xa2c, 0xa30, 0xa17, 0xa2f, 0xa47, 0xa30, 0xa47, 0xa35, 0xa28, 0xa05, 0xa1c,
0xa4b, 0xa30, 0xa47, 0xa38, 0xa2c, 0xa30, 0xa2e, 0xa42, 0xa21, 0xa3e, 0xa15, 0xa47,
0xa28, 0xa47, 0xa30, 0xa40, 0xa15, 0xa47, 0xa2a, 0x20, 0xa35, 0xa30, 0xa21, 0xa2b,
0xa48, 0xa30, 0xa4b, 0xa2e, 0xa21, 0xa40, 0xa05, 0xa30, 0xa3e, 0xa30, 0xa47, 0xa15,
0xa1c, 0xa3e, 0xa35, 0xa3f, 0xa15, 0xa26, 0xa71, 0xa16, 0xa23, 0xa40, 0x20, 0xa1c,
0xa3e, 0xa30, 0xa1c, 0xa40, 0xa06, 0xa38, 0xa47, 0xa02, 0xa1f, 0x20, 0xa39, 0xa47,
0xa32, 0xa47, 0xa28, 0xa3e, 0xa38, 0xa1f, 0xa47, 0xa28, 0xa32, 0xa40, 0xa10, 0xa21,
0xa40, 0xa32, 0xa47, 0xa21, 0xa2c, 0xa4d, 0xa30, 0xa3f, 0xa38, 0xa2c, 0xa47, 0xa28,
0xa2c, 0xa4d, 0xa30, 0xa4b, 0xa15, 0xa28, 0x20, 0xa39, 0xa3f, 0xa32, 0xa21, 0xa3e,
0xa30, 0xa35, 0xa3f, 0xa28, 0xa2f, 0xa42, 0xa15, 0xa32, 0xa3e, 0xa39, 0xa4b, 0xa2c,
0xa3e, 0xa30, 0xa1f, 0xa32, 0xa3f, 0xa70, 0xa21, 0xa47, 0xa2e, 0xa28, 0xa32, 0xa4c,
0xa30, 0xa21, 0x20, 0xa39, 0xa4b, 0xa35, 0xa47, 0xa2e, 0xa48, 0xa32, 0xa2c, 0xa4b,
0xa30, 0xa28, 0xa2a, 0xa30, 0xa25, 0xa38, 0xa3f, 0xa21, 0xa28, 0xa40, 0xa10, 0xa2e,
0xa38, 0xa1f, 0xa30, 0xa21, 0xa2e, 0xa05, 0xa70, 0xa21, 0xa4b, 0xa30, 0xa3e, 0xa06,
0xa38, 0xa1f, 0xa4d, 0xa30, 0xa3e, 0xa16, 0xa3e, 0xa28, 0xa0f, 0xa25, 0xa28, 0xa38,
0xa2c, 0xa48, 0xa32, 0xa17, 0xa4d, 0xa30, 0xa47, 0xa21, 0xa2c, 0xa30, 0xa32, 0xa3f,
0xa28, 0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa1f, 0xa3f, 0xa38, 0xa32, 0xa3e, 0xa35, 0xa3e,
0xa2c, 0xa30, 0xa71, 0xa38, 0xa32, 0xa1c, 0xa2c, 0xa42, 0xa15, 0xa3e, 0xa30, 0xa48,
0xa38, 0xa1f, 0xa2c, 0xa41, 0xa21, 0xa3e, 0xa2a, 0xa48, 0xa38, 0xa1f, 0xa2c, 0xa41,
0xa38, 0xa3f, 0xa70, 0xa1c, 0xa47, 0xa28, 0xa1a, 0xa3f, 0xa38, 0xa3f, 0xa28, 0xa4c,
0xa15, 0xa4b, 0xa2a, 0xa28, 0xa39, 0xa47, 0xa17, 0xa28, 0xa21, 0xa2c, 0xa32, 0xa3f,
0xa28, 0xa1c, 0xa3f, 0xa2c, 0xa30, 0xa3e, 0xa32, 0xa1f, 0xa30, 0xa17, 0xa30, 0xa28,
0xa1c, 0xa40, 0xa39, 0xa47, 0xa32, 0xa38, 0xa3f, 0xa70, 0xa15, 0xa40, 0xa06, 0xa07,
0xa32, 0x20, 0xa06, 0xa2b, 0x20, 0xa2e, 0xa48, 0xa28, 0xa07, 0xa38, 0xa24, 0xa3e,
0xa02, 0xa2c, 0xa41, 0xa32, 0xa1c, 0xa30, 0xa38, 0xa40, 0xa15, 0xa32, 0xa40, 0xa28,
0xa3f, 0xa70, 0xa17, 0xa4d, 0xa30, 0xa47, 0xa21, 0xa15, 0xa40, 0xa30, 0xa4b, 0xa35,
0xa15, 0xa40, 0xa35, 0xa32, 0xa3f, 0xa38, 0xa2c, 0xa28, 0xa32, 0xa1c, 0xa41, 0xa2c,
0xa32, 0xa1c, 0xa3e, 0xa28, 0xa3e, 0xa32, 0xa70, 0xa21, 0xa28, 0xa32, 0xa15, 0xa1c,
0xa3c, 0xa2e, 0xa2c, 0xa30, 0xa17, 0xa2e, 0xa48, 0xa21, 0xa4d, 0xa30, 0xa3f, 0xa21,
0xa2e, 0xa3e, 0xa32, 0xa1f, 0xa3e, 0xa2e, 0xa3e, 0xa30, 0xa40, 0xa0f, 0xa39, 0xa3e,
0xa2e, 0xa47, 0xa28, 0xa2e, 0xa3f, 0xa70, 0xa38, 0xa15, 0xa2e, 0xa4b, 0xa28, 0xa3e,
0xa15, 0xa4b, 0xa2e, 0xa3e, 0xa38, 0xa15, 0xa4b, 0xa13, 0xa38, 0xa32, 0xa4b, 0xa2a,
0xa48, 0xa30, 0xa3f, 0xa38, 0xa2a, 0xa4b, 0xa21, 0xa17, 0xa4b, 0xa30, 0xa40, 0xa15,
0xa3e, 0xa2a, 0xa4d, 0xa30, 0xa3e, 0xa17, 0xa30, 0xa3f, 0xa17, 0xa3e, 0xa38, 0xa48,
0xa28, 0x20, 0xa2e, 0xa30, 0xa40, 0xa28, 0xa4b, 0xa38, 0xa3e, 0xa30, 0xa3e, 0xa1c,
0xa47, 0xa35, 0xa4b, 0xa38, 0xa48, 0xa30, 0xa3e, 0xa1f, 0xa4b, 0xa35, 0xa38, 0xa3f,
0xa2e, 0xa2b, 0xa30, 0xa4b, 0xa2a, 0xa4b, 0xa32, 0xa38, 0xa15, 0xa4b, 0xa2a, 0xa1c,
0xa47, 0xa38, 0xa4b, 0xa2b, 0xa40, 0xa06, 0xa38, 0xa1f, 0xa3e, 0xa15, 0xa39, 0xa4b,
0xa2e, 0xa1f, 0xa71, 0xa32, 0xa3f, 0xa28, 0xa24, 0xa3f, 0xa30, 0xa3e, 0xa28, 0xa47,
0xa2f, 0xa41, 0xa32, 0xa3f, 0xa06, 0xa28, 0xa4b, 0xa38, 0xa15, 0xa35, 0xa3e, 0xa21,
0xa41, 0xa1c, 0xa3c, 0xa35, 0xa48, 0xa1f, 0xa3f, 0xa15, 0xa28, 0xa35, 0xa3f, 0xa06,
0xa28, 0xa3e, 0xa35, 0xa3f, 0xa32, 0xa28, 0xa3f, 0xa05, 0xa38, 0xa35, 0xa4b, 0xa32,
0xa17, 0xa4b, 0xa17, 0xa4d, 0xa30, 0xa47, 0xa21, 0xa35, 0xa3e, 0xa30, 0xa38, 0xa3e,
0xa05, 0xa1c, 0xa3c, 0xa17, 0xa30, 0xa47, 0xa2c, 0xa1c, 0xa3f, 0xa0a, 0xa30, 0xa3f,
0xa16, 0xa05, 0xa70, 0xa1f, 0xa3e, 0xa28, 0xa28, 0xa47, 0xa30, 0xa3f, 0xa35, 0xa4b,
0xa1a, 0xa3e, 0xa17, 0xa4b, 0xa38, 0xa15, 0xa4d, 0xa30, 0xa3f, 0xa38, 0xa2e, 0xa38,
0xa15, 0xa4b, 0xa15, 0xa4b, 0xa1c, 0xa3c, 0xa15, 0xa4b, 0xa2e, 0xa4b, 0xa30, 0xa4b,
0xa15, 0xa30, 0xa17, 0xa2f, 0xa42, 0xa32, 0xa47, 0xa28, 0xa2e, 0xa3e, 0xa39, 0xa47,
0xa2e, 0xa3e, 0xa32, 0xa26, 0xa40, 0xa35, 0xa2e, 0xa4c, 0xa30, 0xa3f, 0xa38, 0xa3c,
0xa38, 0xa2e, 0xa3e, 0xa2f, 0xa4b, 0xa1f, 0xa40, 0xa30, 0xa3f, 0xa2f, 0xa42, 0xa28,
0xa40, 0xa05, 0xa28, 0xa10, 0xa2a, 0xa40, 0xa06, 0xa06, 0xa15, 0xa32, 0xa48, 0xa02,
0xa21, 0xa2c, 0xa4b, 0xa17, 0xa28, 0xa35, 0xa3f, 0xa32, 0xa47, 0xa1a, 0xa48, 0xa25,
0xa2e, 0xa1a, 0xa42, 0xa15, 0xa08, 0xa38, 0xa1f, 0xa30, 0xa07, 0xa2b, 0xa47, 0xa1f,
0xa2b, 0xa15, 0xa3e, 0xa09, 0xa2b, 0xa4b, 0xa2b, 0xa3c, 0xa3f, 0xa1c, 0xa40, 0xa2b,
0xa41, 0xa28, 0xa3e, 0xa2b, 0xa41, 0xa1f, 0xa40, 0xa17, 0xa32, 0xa2a, 0xa47, 0xa17,
0xa4b, 0xa38, 0xa17, 0xa48, 0xa02, 0xa2c, 0xa40, 0xa05, 0xa30, 0xa17, 0xa41, 0xa06,
0xa21, 0xa3e, 0xa15, 0xa47, 0xa28, 0xa32, 0xa39, 0xa4b, 0xa28, 0xa4b, 0xa32, 0xa41,
0xa32, 0xa42, 0xa0f, 0xa02, 0xa21, 0xa30, 0xa2c, 0xa30, 0xa40, 0xa15, 0xa3f, 0xa30,
0xa3f, 0xa24, 0xa3f, 0xa2e, 0xa24, 0xa40, 0xa15, 0xa4b, 0xa38, 0xa4d, 0xa30, 0xa3e,
0xa0f, 0xa15, 0xa35, 0xa3e, 0xa1c, 0xa3e, 0xa32, 0xa40, 0xa28, 0xa2e, 0xa47, 0xa1c,
0xa41, 0xa30, 0xa4b, 0xa2e, 0xa3e, 0xa30, 0xa15, 0xa3f, 0xa38, 0xa3e, 0xa38, 0xa2e,
0xa3f, 0xa21, 0xa35, 0xa47, 0xa28, 0xa3e, 0xa09, 0xa30, 0xa42, 0xa28, 0xa3f, 0xa2f,
0xa42, 0xa28, 0xa4b, 0xa30, 0xa2b, 0xa4c, 0xa15, 0xa28, 0xa4c, 0xa2e, 0xa3f, 0xa06,
0xa2a, 0xa3e, 0xa17, 0xa4b, 0x20, 0xa2a, 0xa3e, 0xa17, 0xa4b, 0xa2a, 0xa32, 0xa3e,
0xa09, 0xa2a, 0xa3f, 0xa1f, 0xa15, 0xa48, 0xa30, 0xa28, 0xa2a, 0xa4b, 0xa39, 0xa28,
0xa2a, 0xa47, 0xa08, 0xa2a, 0xa4b, 0xa30, 0xa1f, 0x20, 0xa2e, 0xa4b, 0xa30, 0xa47,
0xa38, 0xa2c, 0xa40, 0xa30, 0xa3e, 0xa30, 0xa4b, 0xa1f, 0xa4b, 0xa02, 0xa17, 0xa3e,
0xa38, 0xa48, 0xa2a, 0xa3e, 0xa28, 0xa24, 0xa39, 0xa3f, 0xa24, 0xa40, 0xa1f, 0xa3e,
0xa30, 0xa3e, 0xa35, 0xa3e, 0xa1f, 0xa4b, 0xa02, 0xa17, 0xa3e, 0xa1f, 0xa3e, 0xa2a,
0xa42, 0xa35, 0xa47, 0xa15, 0xa35, 0xa3e, 0xa32, 0xa3f, 0xa38, 0x41, 0x62, 0x69,
0x79, 0xe1, 0x6e, 0x45, 0x6c, 0x20, 0x43, 0x61, 0x69, 0x72, 0x6f, 0x4a,
0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x6f,
0x54, 0x75, 0x6e, 0x65, 0x7a, 0x43, 0x75, 0x72, 0x61, 0x7a, 0x61, 0x6f,
0x43, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x6f, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x45,
0x73, 0x70, 0x61, 0xf1, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x72,
0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0xe9, 0x53, 0x61, 0x6e, 0x20, 0x4a, 0x75,
0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e,
0x6f, 0x76, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x43, 0x72, 0x69, 0x73, 0x74,
0x6f, 0x62, 0x61, 0x6c, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x54, 0x6f,
0x6d, 0x61, 0x73, 0x59, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61, 0x50, 0x69,
0x6f, 0x6e, 0x79, 0x61, 0x6e, 0x67, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e,
0xe1, 0x69, 0x53, 0x68, 0x61, 0x6e, 0x67, 0xe1, 0x69, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x72,
0x42, 0x72, 0x75, 0x73, 0x65, 0x6c, 0x61, 0x73, 0x45, 0x6c, 0x20, 0x56,
0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x64, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x64, 0x41, 0x219, 0x67, 0x61, 0x62, 0x61, 0x74, 0x41,
0x74, 0xee, 0x72, 0x61, 0x75, 0x42, 0x69, 0x219, 0x6b, 0x65, 0x6b, 0x44,
0x75, 0x219, 0x61, 0x6e, 0x62, 0x65, 0x48, 0x6f, 0x20, 0x218, 0x69, 0x20,
0x4d, 0x69, 0x6e, 0x49, 0x72, 0x6b, 0x75, 0x21b, 0x6b, 0x49, 0x65, 0x72,
0x75, 0x73, 0x61, 0x6c, 0x69, 0x6d, 0x4b, 0x61, 0x6d, 0x63, 0x69, 0x61,
0x74, 0x6b, 0x61, 0x4b, 0x75, 0x77, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x76,
0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x65, 0x21b, 0x6b, 0x50, 0x68, 0x65, 0x6e,
0x69, 0x61, 0x6e, 0x54, 0x61, 0x219, 0x6b, 0x65, 0x6e, 0x74, 0x49, 0x61,
0x6b, 0x75, 0x21b, 0x6b, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x65, 0x43, 0x61,
0x70, 0x75, 0x6c, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x75, 0x64, 0x53,
0x66, 0x2e, 0x20, 0x45, 0x6c, 0x65, 0x6e, 0x61, 0x42, 0x75, 0x64, 0x61,
0x70, 0x65, 0x73, 0x74, 0x61, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20,
0x4d, 0x61, 0x6e, 0x50, 0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x21b, 0x61,
0x56, 0x61, 0x72, 0x219, 0x6f, 0x76, 0x69, 0x61, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x20, 0x50, 0x61, 0x219, 0x74, 0x65, 0x6c, 0x75, 0x69, 0x4d,
0x61, 0x72, 0x63, 0x68, 0x69, 0x7a, 0x65, 0x49, 0x6e, 0x73, 0x75, 0x6c,
0x61, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x41, 0x6c,
0x61, 0x73, 0x63, 0x61, 0x54, 0x68, 0x65, 0x20, 0x56, 0x61, 0x6c, 0x6c,
0x65, 0x79, 0x53, 0x61, 0x6e, 0x20, 0x4a, 0x75, 0x61, 0x6e, 0x2c, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x6e, 0x6a, 0x65, 0x73, 0x74, 0x61, 0x64, 0x42, 0x65, 0x6c, 0x6d,
0x6f, 0x70, 0x61, 0x6e, 0x49, 0x6e, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43,
0x61, 0x79, 0x6d, 0x61, 0x6e, 0x43, 0x6f, 0x63, 0x6b, 0x62, 0x75, 0x72,
0x6e, 0x20, 0x54, 0x6f, 0x77, 0x6e, 0x42, 0x61, 0x73, 0x73, 0x65, 0x2d,
0x54, 0x65, 0x72, 0x72, 0x65, 0x43, 0x69, 0x74, 0x61, 0x64, 0x20, 0x64,
0x61, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x42, 0x72, 0x61, 0x64,
0x65, 0x73, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x28, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x29,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x20, 0x28, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x29,
0x52, 0x6f, 0x61, 0x64, 0x20, 0x54, 0x6f, 0x77, 0x6e, 0x4d, 0x61, 0x63,
0x20, 0x4d, 0x75, 0x72, 0x64, 0x6f, 0x42, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x53, 0x65, 0x72, 0x69, 0x20, 0x42, 0x65, 0x67, 0x61, 0x77, 0x61,
0x6e, 0x43, 0x69, 0x74, 0x61, 0x64, 0x20, 0x64, 0x61, 0x20, 0x48, 0x6f,
0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x41, 0x7a, 0x6f,
0x72, 0x61, 0x73, 0x49, 0x6e, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x61,
0x6e, 0x61, 0x72, 0x69, 0x61, 0x73, 0x49, 0x6e, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x46, 0x65, 0x72, 0x6f, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x20, 0x64, 0x61, 0x6c, 0x20, 0x53, 0x69, 0x64, 0x53, 0x6f, 0x6e,
0x74, 0x67, 0x2019, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x50, 0x6f, 0x72, 0x74,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x6c, 0x65, 0x79, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x20, 0x50, 0x65, 0x74, 0x65, 0x72, 0x20, 0x50, 0x6f, 0x72, 0x74,
0x44, 0x6f, 0x75, 0x67, 0x6c, 0x61, 0x73, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x48, 0x65, 0x6c, 0x69, 0x65, 0x72, 0x4d, 0x6f, 0x73, 0x63, 0x61,
0x75, 0x54, 0x75, 0x72, 0x69, 0x74, 0x67, 0x46, 0x6c, 0x79, 0x69, 0x6e,
0x67, 0x20, 0x46, 0x69, 0x73, 0x68, 0x20, 0x43, 0x6f, 0x76, 0x65, 0x57,
0x65, 0x73, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x43, 0x6f,
0x6d, 0x6f, 0x72, 0x61, 0x73, 0x57, 0x65, 0x6e, 0x6f, 0x49, 0x6e, 0x73,
0x6c, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50, 0x61, 0x73, 0x63, 0x61, 0x52,
0x69, 0x6b, 0x69, 0x74, 0x65, 0x61, 0x48, 0x6f, 0x6e, 0x69, 0x61, 0x72,
0x61, 0x48, 0x61, 0x67, 0xe5, 0x74, 0xf1, 0x61, 0x54, 0x6f, 0x66, 0x6f,
0x6c, 0x59, 0x61, 0x72, 0x65, 0x6e, 0x41, 0x6c, 0x6f, 0x66, 0x69, 0x4b,
0x69, 0x6e, 0x67, 0x73, 0x74, 0x6f, 0x6e, 0x4d, 0x65, 0x6c, 0x65, 0x6b,
0x6f, 0x6b, 0x50, 0x61, 0x6c, 0x69, 0x6b, 0x69, 0x72, 0x53, 0x6f, 0x75,
0x74, 0x68, 0x20, 0x54, 0x61, 0x72, 0x61, 0x77, 0x61, 0x4e, 0x75, 0x6b,
0x75, 0x2bb, 0x61, 0x6c, 0x6f, 0x66, 0x61, 0x4d, 0x61, 0x74, 0x101, 0x2bb,
0x75, 0x74, 0x75, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x20, 0x41a,
0x430, 0x439, 0x43c, 0x430, 0x43d, 0x411, 0x43e, 0x439, 0x43b, 0x430, 0x2c, 0x20,
0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x44f, 0x20, 0x414, 0x430, 0x43a,
0x43e, 0x442, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x2c, 0x20, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x430, 0x44f, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442,
0x430, 0x41d, 0x44c, 0x44e, 0x2d, 0x421, 0x435, 0x439, 0x43b, 0x435, 0x43c, 0x2c,
0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x44f, 0x20, 0x414, 0x430,
0x43a, 0x43e, 0x442, 0x430, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432,
0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442,
0x441, 0x43a, 0x438, 0x439, 0x43e, 0x2d, 0x432, 0x20, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x435, 0x20,
0x43e, 0x2d, 0x432, 0x430, 0x411, 0x435, 0x440, 0x43c, 0x443, 0x434, 0x441, 0x43a,
0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x41a, 0x430, 0x43d, 0x430, 0x440,
0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x424, 0x430, 0x440,
0x435, 0x440, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x42e,
0x436, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x438, 0x44f,
0x43e, 0x2d, 0x432, 0x20, 0x421, 0x432, 0x44f, 0x442, 0x43e, 0x439, 0x20, 0x415,
0x43b, 0x435, 0x43d, 0x44b, 0x411, 0x44e, 0x437, 0x438, 0x43d, 0x433, 0x435, 0x43d,
0x2d, 0x43d, 0x430, 0x2d, 0x412, 0x435, 0x440, 0x445, 0x43d, 0x435, 0x43c, 0x2d,
0x420, 0x435, 0x439, 0x43d, 0x435, 0x43e, 0x2d, 0x432, 0x20, 0x41c, 0x44d, 0x43d,
0x43e, 0x2d, 0x432, 0x20, 0x420, 0x43e, 0x436, 0x434, 0x435, 0x441, 0x442, 0x432,
0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x44b, 0x435, 0x20, 0x43e,
0x2d, 0x432, 0x430, 0x41a, 0x43e, 0x43c, 0x43e, 0x440, 0x44b, 0x41c, 0x430, 0x43b,
0x44c, 0x434, 0x438, 0x432, 0x44b, 0x43e, 0x2d, 0x432, 0x20, 0x41f, 0x430, 0x441,
0x445, 0x438, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x441,
0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x43e, 0x2d, 0x432, 0x430,
0x20, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435, 0x43e, 0x2d, 0x432, 0x20, 0x42d,
0x43d, 0x434, 0x435, 0x440, 0x431, 0x435, 0x440, 0x438, 0x41c, 0x430, 0x440, 0x43a,
0x438, 0x437, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x43e,
0x2d, 0x432, 0x430, 0x20, 0x41c, 0x438, 0x434, 0x443, 0x44d, 0x439, 0x410, 0x43d,
0x430, 0x430, 0x434, 0x44b, 0x440, 0x410, 0x441, 0x445, 0x430, 0x431, 0x430, 0x430,
0x442, 0x411, 0x430, 0x433, 0x434, 0x430, 0x430, 0x434, 0x411, 0x430, 0x43a, 0x443,
0x443, 0x427, 0x438, 0x442, 0x430, 0x430, 0x414, 0x430, 0x43c, 0x430, 0x430, 0x441,
0x414, 0x443, 0x431, 0x430, 0x430, 0x439, 0x425, 0x43e, 0x20, 0x428, 0x438, 0x20,
0x41c, 0x438, 0x43d, 0x44c, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x430,
0x439, 0x41a, 0x430, 0x431, 0x443, 0x443, 0x43b, 0x41a, 0x430, 0x43c, 0x447, 0x430,
0x430, 0x442, 0x43a, 0x430, 0x425, 0x430, 0x430, 0x43d, 0x434, 0x44b, 0x433, 0x430,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x430, 0x439,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x430, 0x43d, 0x41d, 0x43e, 0x432, 0x43e,
0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x430, 0x439, 0x41e, 0x43c, 0x441,
0x43a, 0x430, 0x439, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x438, 0x43d, 0x421,
0x430, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x430, 0x43d, 0x41e, 0x440, 0x442, 0x43e,
0x20, 0x425, 0x430, 0x43b, 0x44b, 0x43c, 0x430, 0x423, 0x43b, 0x430, 0x43d, 0x20,
0x411, 0x430, 0x430, 0x442, 0x430, 0x440, 0x423, 0x440, 0x443, 0x43c, 0x447, 0x443,
0x423, 0x443, 0x441, 0x20, 0x41d, 0x44c, 0x430, 0x440, 0x430, 0x414, 0x44c, 0x43e,
0x43a, 0x443, 0x443, 0x441, 0x43a, 0x430, 0x439, 0x415, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x443, 0x440, 0x410, 0x430, 0x441, 0x442, 0x440,
0x430, 0x445, 0x430, 0x43d, 0x421, 0x442, 0x430, 0x43c, 0x431, 0x443, 0x443, 0x43b,
0x41c, 0x43e, 0x441, 0x43a, 0x443, 0x431, 0x430, 0x421, 0x430, 0x43c, 0x430, 0x430,
0x440, 0x430, 0x423, 0x43b, 0x44c, 0x44f, 0x43d, 0x43e, 0x432, 0x441, 0x43a, 0x430,
0x439, 0x1c5f, 0x2e, 0x1c75, 0x1c64, 0x1c70, 0x1c61, 0x1c5f, 0x1c71, 0x1c5f, 0x1c70, 0x1c5f,
0x1c60, 0x1c5f, 0x1c71, 0x1c60, 0x1c77, 0x1c5a, 0x1c68, 0x1c6e, 0x1c61, 0x1c7d, 0x1c5f, 0x1c71,
0x1c5c, 0x1c69, 0x1c6d, 0x1c5e, 0x1c5f, 0x1c5f, 0x1c71, 0x1c74, 0x1c64, 0x1c5c, 0x1c69, 0x1c63,
0x1c5f, 0x1c5f, 0x1c68, 0x1c69, 0x1c75, 0x1c5f, 0x1c5f, 0x1c74, 0x1c64, 0x1c60, 0x1c5a, 0x1c60,
0x1c5f, 0x1c71, 0x1c75, 0x1c5f, 0x1c66, 0x1c64, 0x1c6d, 0x1c5f, 0x20, 0x1c75, 0x1c5f, 0x1c71,
0x1c70, 0x1c6e, 0x1c68, 0x1c5f, 0x1c65, 0x1c75, 0x1c5f, 0x1c68, 0x1c75, 0x1c5f, 0x1c70, 0x1c5a,
0x1c65, 0x1c75, 0x1c6e, 0x1c5e, 0x1c64, 0x1c61, 0x1c6e, 0x1c75, 0x1c5e, 0x1c5f, 0x1c71, 0x1c60,
0x1c7c, 0x1c65, 0x1c5f, 0x1c75, 0x1c5e, 0x1c5a, 0x1c71, 0x1c75, 0x1c5a, 0x1c6d, 0x1c64, 0x1c65,
0x1c60, 0x1c6e, 0x1c62, 0x1c75, 0x1c7d, 0x1c68, 0x1c6e, 0x1c61, 0x1c7d, 0x20, 0x1c75, 0x1c6e,
0x1c60, 0x1c5f, 0x1c71, 0x1c60, 0x1c69, 0x1c71, 0x1c60, 0x1c5a, 0x1c62, 0x1c5f, 0x1c71, 0x1c6a,
0x1c64, 0x1c60, 0x1c5f, 0x1c5c, 0x1c5a, 0x1c6a, 0x1c77, 0x1c64, 0x1c66, 0x1c69, 0x1c63, 0x1c5f,
0x1c66, 0x1c69, 0x1c63, 0x1c5f, 0x1c60, 0x1c5a, 0x1c65, 0x1c74, 0x1c5f, 0x20, 0x1c68, 0x1c64,
0x1c60, 0x1c5f, 0x1c60, 0x1c68, 0x1c6e, 0x1c65, 0x1c74, 0x1c5a, 0x1c71, 0x1c60, 0x1c6d, 0x1c69,
0x1c68, 0x1c5f, 0x1c60, 0x1c5f, 0x1c63, 0x1c70, 0x1c5f, 0x1c71, 0x1c62, 0x1c5f, 0x1c68, 0x1c60,
0x1c65, 0x1c5f, 0x1c63, 0x1c71, 0x1c70, 0x1c5f, 0x1c63, 0x1c65, 0x1c5a, 0x1c71, 0x1c70, 0x1c5f,
0x1c63, 0x1c65, 0x1c5a, 0x1c71, 0x20, 0x1c60, 0x1c68, 0x1c64, 0x1c60, 0x1c70, 0x1c6e, 0x1c71,
0x1c63, 0x1c6e, 0x1c68, 0x1c70, 0x1c6e, 0x1c74, 0x1c68, 0x1c5a, 0x1c6d, 0x1c6e, 0x1c74, 0x1c70,
0x1c5a, 0x1c62, 0x1c64, 0x1c71, 0x1c64, 0x1c60, 0x1c5f, 0x1c6e, 0x1c70, 0x1c62, 0x1c5a, 0x1c71,
0x1c74, 0x1c5a, 0x1c71, 0x1c6e, 0x1c5e, 0x20, 0x1c65, 0x1c5f, 0x1c5e, 0x1c63, 0x1c6e, 0x1c70,
0x1c5a, 0x1c68, 0x1c6f, 0x1c77, 0x1c5a, 0x1c74, 0x20, 0x1c71, 0x1c6e, 0x1c5e, 0x1c65, 0x1c5a,
0x1c71, 0x1c5c, 0x1c5e, 0x1c6e, 0x1c65, 0x20, 0x1c75, 0x1c6e, 0x1c5c, 0x1c69, 0x1c65, 0x20,
0x1c75, 0x1c6e, 0x1c5c, 0x1c7d, 0x1c68, 0x1c5f, 0x1c71, 0x1c70, 0x20, 0x1c74, 0x1c69, 0x1c68,
0x1c60, 0x1c5c, 0x1c7d, 0x1c68, 0x1c6e, 0x1c71, 0x1c5f, 0x1c70, 0x1c5f, 0x1c5c, 0x1c69, 0x1c63,
0x1c5f, 0x1c70, 0x1c6e, 0x1c5e, 0x1c5a, 0x1c6f, 0x1c5c, 0x1c69, 0x1c63, 0x1c5f, 0x1c74, 0x1c6e,
0x1c62, 0x1c5f, 0x1c5e, 0x1c5f, 0x1c66, 0x1c5f, 0x1c5e, 0x1c64, 0x1c6f, 0x1c77, 0x1c5f, 0x1c60,
0x1c65, 0x1c66, 0x1c5f, 0x1c63, 0x1c5f, 0x1c71, 0x1c5f, 0x1c66, 0x1c5f, 0x1c79, 0x1c68, 0x1c62,
0x1c5a, 0x1c65, 0x1c64, 0x1c5e, 0x1c5a, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71,
0x1c5f, 0x1c6f, 0x1c5a, 0x1c5e, 0x1c64, 0x1c65, 0x1c71, 0x1c5a, 0x1c60, 0x1c65, 0x2c, 0x20,
0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c62, 0x1c6e, 0x1c68, 0x1c6e,
0x1c5d, 0x1c5c, 0x1c5a, 0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71,
0x1c5f, 0x1c6f, 0x1c64, 0x1c74, 0x1c5a, 0x1c68, 0x1c65, 0x1c75, 0x1c5a, 0x1c68, 0x1c5c, 0x1c7d,
0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c74, 0x1c5a,
0x1c5e, 0x20, 0x1c65, 0x1c64, 0x1c74, 0x1c6d, 0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64,
0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c75, 0x1c77, 0x1c6e, 0x1c75, 0x1c77, 0x1c5f, 0x1c6d, 0x2c,
0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c75, 0x1c77, 0x1c64,
0x1c71, 0x1c65, 0x1c6e, 0x1c71, 0x1c65, 0x2c, 0x20, 0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d,
0x1c5f, 0x1c71, 0x1c5f, 0x1c63, 0x1c64, 0x1c71, 0x1c5f, 0x1c62, 0x1c5f, 0x1c60, 0x2c, 0x20,
0x1c64, 0x1c71, 0x1c70, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c64, 0x1c71, 0x1c69, 0x1c63,
0x1c64, 0x1c60, 0x1c64, 0x1c60, 0x1c5f, 0x1c5e, 0x1c69, 0x1c63, 0x1c64, 0x1c5b, 0x1c61, 0x1c5f,
0x1c62, 0x1c5f, 0x1c6d, 0x1c60, 0x1c5f, 0x1c61, 0x1c69, 0x1c71, 0x1c6e, 0x1c5f, 0x1c79, 0x1c63,
0x1c5e, 0x1c69, 0x1c63, 0x1c64, 0x1c65, 0x1c63, 0x1c64, 0x1c5e, 0x1c6e, 0x1c62, 0x1c5a, 0x1c71,
0x1c74, 0x1c64, 0x1c65, 0x1c6e, 0x1c5e, 0x1c5a, 0x2c, 0x20, 0x1c60, 0x1c6e, 0x1c71, 0x1c74,
0x1c69, 0x1c60, 0x1c6d, 0x1c60, 0x1c68, 0x1c5f, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x1c64, 0x1c61,
0x1c7d, 0x1c60, 0x1c5a, 0x1c5e, 0x1c6e, 0x1c65, 0x20, 0x1c6e, 0x1c67, 0x1c61, 0x1c6e, 0x1c5e,
0x1c6e, 0x1c65, 0x1c5e, 0x1c5a, 0x1c63, 0x1c5f, 0x1c79, 0x1c68, 0x20, 0x1c6f, 0x1c68, 0x1c64,
0x1c71, 0x1c65, 0x20, 0x1c60, 0x1c63, 0x1c5f, 0x1c74, 0x1c5f, 0x1c79, 0x1c68, 0x1c62, 0x1c5f,
0x1c71, 0x1c5f, 0x1c5c, 0x1c69, 0x1c63, 0x1c5f, 0x1c62, 0x1c5f, 0x1c68, 0x1c64, 0x1c5c, 0x1c5a,
0x1c74, 0x1c62, 0x1c5f, 0x1c68, 0x1c74, 0x1c64, 0x1c71, 0x1c60, 0x1c62, 0x1c5f, 0x1c74, 0x1c5f,
0x1c62, 0x1c5a, 0x1c68, 0x1c5a, 0x1c65, 0x1c62, 0x1c5f, 0x1c61, 0x1c5f, 0x1c74, 0x1c5e, 0x1c5f,
0x1c71, 0x1c62, 0x1c6e, 0x1c71, 0x1c5a, 0x1c62, 0x1c64, 0x1c71, 0x1c6e, 0x1c62, 0x1c6e, 0x1c68,
0x1c64, 0x1c70, 0x1c5f, 0x1c62, 0x1c6e, 0x1c74, 0x1c5e, 0x1c5f, 0x1c60, 0x1c5f, 0x1c74, 0x1c5e,
0x1c5f, 0x1c62, 0x1c6e, 0x1c60, 0x1c65, 0x1c64, 0x1c60, 0x1c5a, 0x20, 0x1c65, 0x1c5a, 0x1c66,
0x1c5a, 0x1c68, 0x1c62, 0x1c64, 0x1c60, 0x1c63, 0x1c6e, 0x1c5e, 0x1c5a, 0x1c71, 0x1c62, 0x1c5a,
0x1c71, 0x1c60, 0x1c74, 0x1c5a, 0x1c71, 0x1c62, 0x1c5a, 0x1c71, 0x1c74, 0x1c6e, 0x1c68, 0x1c6e,
0x1c6d, 0x1c62, 0x1c5a, 0x1c71, 0x1c74, 0x1c65, 0x1c6e, 0x1c68, 0x1c5f, 0x1c74, 0x1c71, 0x1c5f,
0x1c65, 0x1c5f, 0x1c63, 0x1c71, 0x1c64, 0x1c63, 0x20, 0x1c6d, 0x1c5a, 0x1c68, 0x1c60, 0x1c71,
0x1c5a, 0x1c62, 0x1c6e, 0x1c75, 0x1c6e, 0x1c63, 0x1c5e, 0x1c5f, 0x1c66, 0x2c, 0x20, 0x1c6e,
0x1c5b, 0x1c5a, 0x1c62, 0x20, 0x1c70, 0x1c5f, 0x1c60, 0x1c5a, 0x1c74, 0x1c5f, 0x1c65, 0x1c6e,
0x1c71, 0x1c74, 0x1c5a, 0x1c68, 0x2c, 0x20, 0x1c6e, 0x1c5b, 0x1c5a, 0x1c62, 0x20, 0x1c70,
0x1c5f, 0x1c60, 0x1c5a, 0x1c74, 0x1c5f, 0x1c71, 0x1c64, 0x1c63, 0x20, 0x1c65, 0x1c6e, 0x1c5e,
0x1c6e, 0x1c62, 0x2c, 0x20, 0x1c6e, 0x1c5b, 0x1c5a, 0x1c62, 0x20, 0x1c70, 0x1c5f, 0x1c60,
0x1c5a, 0x1c74, 0x1c5f, 0x1c71, 0x1c69, 0x1c60, 0x1c5a, 0x1c61, 0x1c64, 0x1c71, 0x1c5f, 0x1c5c,
0x1c5f, 0x1c6f, 0x1c5f, 0x1c71, 0x1c5f, 0x1c62, 0x1c5f, 0x1c6f, 0x1c77, 0x1c64, 0x1c71, 0x1c64,
0x1c60, 0x1c65, 0x1c6f, 0x1c5a, 0x1c68, 0x1c74, 0x20, 0x1c5f, 0x1c63, 0x20, 0x1c6f, 0x1c68,
0x1c64, 0x1c71, 0x1c65, 0x1c6f, 0x1c5a, 0x1c68, 0x1c74, 0x20, 0x1c5a, 0x1c6f, 0x1c77, 0x20,
0x1c65, 0x1c6f, 0x1c6e, 0x1c71, 0x1c6f, 0x1c6d, 0x1c69, 0x1c68, 0x1c74, 0x1c5a, 0x20, 0x1c68,
0x1c64, 0x1c60, 0x1c5a, 0x1c68, 0x1c5f, 0x1c71, 0x1c60, 0x1c64, 0x1c71, 0x20, 0x1c64, 0x1c71,
0x1c5e, 0x1c6e, 0x1c74, 0x1c68, 0x1c6e, 0x1c5c, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x1c5f, 0x1c68,
0x1c64, 0x1c65, 0x1c5a, 0x1c5e, 0x1c6d, 0x1c69, 0x1c74, 0x1c65, 0x1c5f, 0x1c71, 0x1c74, 0x1c5a,
0x20, 0x1c70, 0x1c5a, 0x1c62, 0x1c64, 0x1c71, 0x1c5c, 0x1c5a, 0x1c64, 0x1c5b, 0x1c5a, 0x1c60,
0x1c5a, 0x1c5b, 0x1c5a, 0x1c68, 0x1c62, 0x1c64, 0x1c5b, 0x1c65, 0x1c64, 0x1c5b, 0x1c60, 0x1c5f,
0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c75, 0x1c5f, 0x1c68, 0x1c5b, 0x1c77, 0x1c6e,
0x1c5e, 0x1c62, 0x1c64, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c61, 0x1c5a, 0x1c71,
0x1c65, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c60, 0x1c64, 0x1c74, 0x1c65, 0x1c6e,
0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c5e, 0x1c69, 0x1c65, 0x1c64, 0x1c6d, 0x1c5f, 0x1c6e,
0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c5b, 0x1c77, 0x1c5a, 0x1c62, 0x1c5f, 0x1c65, 0x1c6e,
0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c63, 0x1c64, 0x1c71, 0x1c65, 0x1c6e, 0x1c74, 0x1c65,
0x1c63, 0x1c64, 0x1c6f, 0x1c77, 0x1c74, 0x20, 0x1c60, 0x1c5a, 0x1c68, 0x1c6e, 0x1c71, 0x1c74,
0x1c5b, 0x1c6e, 0x1c5c, 0x1c69, 0x1c6a, 0x1c64, 0x1c5c, 0x1c5f, 0x1c5e, 0x1c6f, 0x1c5f, 0x1c5b,
0x1c77, 0x1c69, 0x1c5e, 0x1c6e, 0x1c74, 0x1c64, 0x1c61, 0x1c69, 0x1c63, 0x1c5f, 0x1c71, 0x1c5f,
0x1c74, 0x1c5a, 0x1c68, 0x1c5a, 0x1c71, 0x1c74, 0x1c5a, 0x1c74, 0x1c5a, 0x1c68, 0x1c74, 0x1c5a,
0x1c5e, 0x1c5f, 0x1c63, 0x1c5f, 0x1c71, 0x1c60, 0x1c5a, 0x1c63, 0x1c5f, 0x1c79, 0x1c68, 0x1c63,
0x1c5f, 0x1c6d, 0x1c64, 0x1c74, 0x1c66, 0x1c5a, 0x1c68, 0x1c65, 0x1c63, 0x1c64, 0x1c71, 0x1c64,
0x1c6f, 0x1c6e, 0x1c5c, 0x1c7d, 0x1c6d, 0x1c5f, 0x1c60, 0x1c69, 0x1c5b, 0x1c5f, 0x1c5b, 0x1c75,
0x1c5f, 0x1c79, 0x1c68, 0x1c62, 0x1c69, 0x1c71, 0x1c70, 0x1c5f, 0x53, 0x75, 0x20, 0x43,
0xe0, 0x69, 0x72, 0x75, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0xec, 0x73, 0x63,
0x69, 0x75, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x2d, 0x4e, 0x6f, 0x75, 0x54,
0x72, 0xec, 0x70, 0x6f, 0x6c, 0x69, 0x54, 0xf9, 0x6e, 0x69, 0x73, 0x69,
0x53, 0x2019, 0x41, 0x76, 0x61, 0x6e, 0x61, 0x54, 0x7a, 0x69, 0x74, 0x61,
0x64, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x4d, 0xe8, 0x73,
0x73, 0x69, 0x63, 0x75, 0x4e, 0x6f, 0x61, 0x20, 0x59, 0x6f, 0x72, 0x6b,
0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x4e, 0x6f, 0x72,
0x64, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x4e, 0x6f,
0x72, 0x64, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x50, 0xe0, 0x6e, 0x61, 0x6d, 0x61,
0x50, 0x6f, 0x72, 0x74, 0x75, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x70,
0x61, 0x67, 0x6e, 0x61, 0x53, 0x61, 0x6e, 0x74, 0x75, 0x20, 0x42, 0x61,
0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x75, 0x53, 0x61, 0x6e, 0x74,
0x75, 0x20, 0x47, 0x69, 0x75, 0x61, 0x6e, 0x6e, 0x65, 0x53, 0x61, 0x6e,
0x74, 0x75, 0x20, 0x43, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x6c, 0x75, 0x53,
0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75, 0x67, 0x68, 0x69, 0x61, 0x53,
0x61, 0x6e, 0x74, 0x75, 0x20, 0x54, 0x6f, 0x6d, 0x6d, 0x61, 0x73, 0x75,
0x53, 0x61, 0x6e, 0x74, 0x75, 0x20, 0x56, 0x69, 0x73, 0x73, 0x65, 0x6e,
0x74, 0x65, 0x54, 0x7a, 0x69, 0x74, 0x61, 0x64, 0x65, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68,
0x43, 0x61, 0x74, 0xe0, 0x72, 0x52, 0x69, 0x79, 0xe0, 0x64, 0x53, 0x65,
0xf9, 0x6c, 0x54, 0x65, 0x68, 0x65, 0x72, 0xe0, 0x6e, 0x55, 0x6c, 0xe0,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x41, 0x7a, 0x6f, 0x72, 0x72,
0x61, 0x73, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x61, 0x6e,
0xe0, 0x72, 0x69, 0x61, 0x73, 0x43, 0x61, 0x62, 0x75, 0x20, 0x42, 0x69,
0x72, 0x64, 0x65, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x46, 0xf8,
0x72, 0x6f, 0x79, 0x61, 0x72, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x53, 0x75, 0x64, 0x53, 0x61,
0x6e, 0x74, 0x2019, 0x45, 0x6c, 0x65, 0x6e, 0x65, 0x42, 0x65, 0x6c, 0x67,
0x72, 0x61, 0x64, 0x75, 0x42, 0x65, 0x72, 0x6c, 0x69, 0x6e, 0x75, 0x42,
0xf9, 0x63, 0x61, 0x72, 0x65, 0x73, 0x74, 0x42, 0xf9, 0x64, 0x61, 0x70,
0x65, 0x73, 0x74, 0x44, 0x75, 0x62, 0x6c, 0x69, 0x6e, 0x75, 0xcc, 0x73,
0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0xcc, 0x73,
0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x4d, 0xf2, 0x6e, 0x61, 0x63, 0x75,
0x53, 0x61, 0x6e, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x75,
0x49, 0x73, 0x74, 0x6f, 0x63, 0x6f, 0x6c, 0x6d, 0x61, 0x54, 0x7a, 0x69,
0x74, 0x61, 0x64, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x56,
0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x75, 0x56, 0x61, 0x72, 0x73, 0xe0,
0x76, 0x69, 0x61, 0x5a, 0x61, 0x67, 0xe0, 0x62, 0x72, 0x69, 0x61, 0x5a,
0x75, 0x72, 0x69, 0x67, 0x75, 0x4e, 0x61, 0x74, 0x69, 0x76, 0x69, 0x64,
0x61, 0x64, 0x65, 0x4d, 0x61, 0x75, 0x72, 0xec, 0x74, 0x7a, 0x69, 0x75,
0x73, 0x4d, 0x61, 0x69, 0x6f, 0x74, 0x61, 0x52, 0x65, 0x75, 0x6e, 0x69,
0x6f, 0x6e, 0x65, 0x47, 0x75, 0xe0, 0x6d, 0x4d, 0x61, 0x72, 0x63, 0x68,
0x65, 0x73, 0x61, 0x73, 0x411, 0x443, 0x435, 0x43d, 0x43e, 0x441, 0x20, 0x410,
0x458, 0x440, 0x435, 0x441, 0x41a, 0x43e, 0x440, 0x430, 0x43b, 0x20, 0x425, 0x430,
0x440, 0x431, 0x443, 0x440, 0x41a, 0x430, 0x458, 0x43c, 0x430, 0x43d, 0x441, 0x43a,
0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x41b, 0x443, 0x438, 0x432,
0x438, 0x43b, 0x435, 0x41b, 0x43e, 0x443, 0x435, 0x440, 0x20, 0x41f, 0x440, 0x438,
0x43d, 0x441, 0x438, 0x437, 0x20, 0x41a, 0x432, 0x43e, 0x440, 0x442, 0x435, 0x440,
0x413, 0x43e, 0x442, 0x445, 0x430, 0x431, 0x41f, 0x43e, 0x440, 0x442, 0x43e, 0x20,
0x420, 0x438, 0x43a, 0x43e, 0x421, 0x430, 0x43d, 0x442, 0x458, 0x430, 0x433, 0x43e,
0x421, 0x43a, 0x43e, 0x440, 0x435, 0x437, 0x431, 0x438, 0x441, 0x443, 0x43d, 0x434,
0x425, 0x43e, 0x43d, 0x433, 0x43a, 0x43e, 0x43d, 0x433, 0x423, 0x440, 0x443, 0x43c,
0x45b, 0x438, 0x424, 0x430, 0x440, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x432, 0x430, 0x421, 0x432, 0x435, 0x442, 0x430, 0x20, 0x408, 0x435, 0x43b,
0x435, 0x43d, 0x430, 0x410, 0x434, 0x435, 0x43b, 0x435, 0x458, 0x434, 0x410, 0x441,
0x442, 0x440, 0x430, 0x43a, 0x430, 0x43d, 0x411, 0x438, 0x441, 0x438, 0x43d, 0x433,
0x435, 0x43d, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x43d,
0x412, 0x438, 0x43b, 0x45a, 0x443, 0x441, 0x412, 0x438, 0x432, 0x438, 0x2c, 0x20,
0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d, 0x430, 0x412, 0x438, 0x43d, 0x441,
0x435, 0x43d, 0x441, 0x2c, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x43d,
0x430, 0x411, 0x458, 0x443, 0x43b, 0x430, 0x2c, 0x20, 0x421, 0x458, 0x435, 0x432,
0x435, 0x440, 0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x426,
0x435, 0x43d, 0x442, 0x430, 0x440, 0x2c, 0x20, 0x421, 0x458, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x43e,
0x432, 0x438, 0x20, 0x421, 0x430, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x421, 0x458,
0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442,
0x430, 0x421, 0x432, 0x438, 0x444, 0x442, 0x20, 0x41a, 0x430, 0x440, 0x435, 0x43d,
0x442, 0x414, 0x438, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x418, 0x440, 0x432,
0x438, 0x43b, 0x41b, 0x43e, 0x43d, 0x433, 0x458, 0x438, 0x440, 0x41d, 0x438, 0x458,
0x443, 0x435, 0x42, 0x61, 0x6d, 0x61, 0x6b, 0x6f, 0x42, 0x61, 0x6e, 0x67,
0x75, 0x69, 0x42, 0x61, 0x6e, 0x17e, 0x75, 0x6c, 0x42, 0x69, 0x73, 0x61,
0x6f, 0x42, 0x75, 0x64, 0x17e, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x61, 0x4b,
0x61, 0x6d, 0x70, 0x61, 0x6c, 0x61, 0x4c, 0x75, 0x61, 0x6e, 0x64, 0x61,
0x4c, 0x75, 0x73, 0x61, 0x6b, 0x61, 0x4d, 0x61, 0x6c, 0x61, 0x62, 0x6f,
0x4d, 0x61, 0x70, 0x75, 0x74, 0x6f, 0x4d, 0x62, 0x61, 0x62, 0x61, 0x6e,
0x65, 0x4e, 0x69, 0x6a, 0x61, 0x6d, 0x65, 0x6a, 0x45, 0x6e, 0x6b, 0x6f,
0x72, 0x69, 0x64, 0x17e, 0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x20, 0x41,
0x6a, 0x72, 0x65, 0x73, 0x17d, 0x75, 0x17e, 0x75, 0x69, 0x53, 0x61, 0x6e,
0x20, 0x48, 0x75, 0x61, 0x6e, 0x53, 0x61, 0x6e, 0x20, 0x4c, 0x75, 0x69,
0x55, 0x161, 0x75, 0x61, 0x69, 0x6a, 0x61, 0x41, 0x72, 0x75, 0x62, 0x61,
0x4b, 0x6f, 0x72, 0x61, 0x6c, 0x20, 0x48, 0x61, 0x72, 0x62, 0x75, 0x72,
0x42, 0x61, 0x69, 0x6a, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x61, 0x73, 0x42, 0x65, 0x6c, 0x69, 0x7a, 0x65, 0x42, 0x6c, 0x61, 0x6e,
0x6b, 0x2d, 0x53, 0x65, 0x6a, 0x62, 0x6c, 0x6f, 0x6e, 0x42, 0x6f, 0x61,
0x20, 0x56, 0x69, 0x73, 0x74, 0x61, 0x42, 0x6f, 0x67, 0x6f, 0x74, 0x61,
0x42, 0x6f, 0x6a, 0x7a, 0x69, 0x4b, 0x65, 0x6d, 0x62, 0x72, 0x69, 0x64,
0x17e, 0x20, 0x42, 0x65, 0x6a, 0x4b, 0x61, 0x6a, 0x6d, 0x61, 0x6e, 0x73,
0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x10c, 0x69, 0x6b,
0x61, 0x67, 0x6f, 0x53, 0x69, 0x75, 0x64, 0x61, 0x64, 0x20, 0x48, 0x75,
0x61, 0x72, 0x65, 0x7a, 0x44, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73,
0x68, 0x61, 0x67, 0x65, 0x6e, 0x45, 0x69, 0x72, 0x75, 0x6e, 0x65, 0x70,
0x65, 0x46, 0x6f, 0x72, 0x74, 0x61, 0x6c, 0x65, 0x7a, 0x61, 0x47, 0x6c,
0x65, 0x6a, 0x73, 0x20, 0x42, 0x65, 0x6a, 0x47, 0x75, 0x73, 0x20, 0x42,
0x65, 0x6a, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x47, 0x72, 0x65, 0x6e, 0x61, 0x64, 0x61, 0x48, 0x65, 0x72, 0x6d, 0x6f,
0x73, 0x69, 0x6c, 0x6a, 0x6f, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e,
0x61, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x4e, 0x6f, 0x6b, 0x73, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x72, 0x65,
0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e,
0x61, 0x50, 0x69, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x54, 0x65, 0x6c,
0x20, 0x53, 0x69, 0x74, 0x69, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a,
0x61, 0x6e, 0x61, 0x56, 0x69, 0x76, 0x69, 0x2c, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x6a, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x73,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x56, 0x69,
0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a,
0x61, 0x6e, 0x61, 0x49, 0x6b, 0x76, 0x61, 0x6c, 0x75, 0x69, 0x74, 0x17d,
0x75, 0x6e, 0x6f, 0x4c, 0x75, 0x69, 0x76, 0x69, 0x6c, 0x4d, 0x6f, 0x6e,
0x74, 0x69, 0x10d, 0x65, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74,
0x61, 0x6b, 0x69, 0x4b, 0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x61, 0x6a,
0x6b, 0x4c, 0x61, 0x20, 0x50, 0x61, 0x7a, 0x4c, 0x6f, 0x73, 0x20, 0x41,
0x6e, 0x111, 0x65, 0x6c, 0x65, 0x73, 0x4c, 0x6f, 0x75, 0x65, 0x72, 0x20,
0x50, 0x72, 0x69, 0x6e, 0x73, 0x69, 0x7a, 0x20, 0x4b, 0x76, 0x6f, 0x72,
0x74, 0x65, 0x72, 0x4d, 0x61, 0x7a, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x4d,
0x65, 0x74, 0x6c, 0x61, 0x6b, 0x61, 0x74, 0x6c, 0x61, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x6f, 0x20, 0x53, 0x69, 0x74, 0x69, 0x4d, 0x6f, 0x6e,
0x74, 0x65, 0x76, 0x69, 0x64, 0x65, 0x6f, 0x4e, 0x6a, 0x75, 0x6a, 0x6f,
0x72, 0x6b, 0x42, 0x6a, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x53, 0x6a, 0x65,
0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x43, 0x65, 0x6e, 0x74, 0x61, 0x72, 0x2c, 0x20, 0x53, 0x6a, 0x65, 0x76,
0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e,
0x6f, 0x76, 0x69, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53,
0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x47, 0x6f, 0x74, 0x68, 0x61, 0x62, 0x50, 0x61, 0x6e, 0x61,
0x6d, 0x61, 0x50, 0x61, 0x72, 0x61, 0x6d, 0x61, 0x72, 0x69, 0x62, 0x6f,
0x50, 0x6f, 0x72, 0x74, 0x2d, 0x6f, 0x2d, 0x50, 0x72, 0x65, 0x6e, 0x73,
0x50, 0x6f, 0x72, 0x74, 0x20, 0x6f, 0x76, 0x20, 0x53, 0x70, 0x65, 0x6a,
0x6e, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x56, 0x65, 0x6c, 0x6a, 0x6f,
0x52, 0x65, 0x73, 0x69, 0x66, 0x65, 0x52, 0x65, 0x64, 0x17e, 0x61, 0x6a,
0x6e, 0x61, 0x52, 0x65, 0x7a, 0x6f, 0x6c, 0x75, 0x74, 0x53, 0x61, 0x6e,
0x74, 0x61, 0x72, 0x65, 0x6d, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x44,
0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x53, 0x61, 0x6f, 0x20, 0x50, 0x61,
0x6f, 0x6c, 0x6f, 0x53, 0x69, 0x74, 0x6b, 0x61, 0x53, 0x76, 0x2e, 0x20,
0x4c, 0x75, 0x63, 0x69, 0x6a, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x56,
0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x54, 0x6f, 0x72, 0x6f, 0x6e, 0x74,
0x6f, 0x54, 0x6f, 0x72, 0x74, 0x6f, 0x6c, 0x61, 0x56, 0x61, 0x6a, 0x74,
0x68, 0x6f, 0x72, 0x73, 0x44, 0x69, 0x6d, 0x6f, 0x6e, 0x20, 0x64, 0x2019,
0x49, 0x72, 0x76, 0x69, 0x6c, 0x4d, 0x61, 0x6b, 0x6d, 0x75, 0x72, 0x64,
0x6f, 0x4c, 0x6f, 0x6e, 0x67, 0x6a, 0x69, 0x72, 0x46, 0x61, 0x6d, 0x61,
0x67, 0x75, 0x73, 0x74, 0x61, 0x4a, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c,
0x69, 0x6d, 0x4b, 0x75, 0x61, 0x6c, 0x61, 0x20, 0x4c, 0x75, 0x6d, 0x70,
0x75, 0x72, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0x61, 0x50, 0x6f, 0x6e, 0x74,
0x69, 0x6a, 0x61, 0x6e, 0x61, 0x6b, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a,
0x65, 0x6b, 0x6f, 0x6c, 0x69, 0x6d, 0x73, 0x6b, 0x55, 0x72, 0x75, 0x6d,
0x107, 0x69, 0x55, 0x73, 0x74, 0x2d, 0x4e, 0x65, 0x72, 0x61, 0x4b, 0x61,
0x6e, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76,
0x61, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x61,
0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x46, 0x61, 0x72, 0x73, 0x6b,
0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x4d, 0x61, 0x64, 0x65,
0x69, 0x72, 0x61, 0x4a, 0x75, 0x17e, 0x6e, 0x61, 0x20, 0x44, 0x17e, 0x6f,
0x72, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x53, 0x76, 0x65, 0x74, 0x61, 0x20,
0x4a, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x64, 0x65, 0x6c, 0x65, 0x6a,
0x64, 0x42, 0x72, 0x69, 0x7a, 0x62, 0x65, 0x6a, 0x6e, 0x49, 0x75, 0x6b,
0x6c, 0x61, 0x42, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c, 0x61, 0x76, 0x61,
0x42, 0x69, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x44, 0x17e, 0x65, 0x72,
0x7a, 0x69, 0x4c, 0x6a, 0x75, 0x62, 0x6c, 0x6a, 0x61, 0x6e, 0x61, 0x4d,
0x61, 0x6c, 0x74, 0x61, 0x4f, 0x73, 0x6c, 0x6f, 0x50, 0x6f, 0x64, 0x67,
0x6f, 0x72, 0x69, 0x63, 0x61, 0x52, 0x69, 0x67, 0x61, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e,
0x6f, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0x6f, 0x56, 0x69, 0x6c,
0x6e, 0x6a, 0x75, 0x73, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61,
0x72, 0x69, 0x76, 0x6f, 0x42, 0x75, 0x67, 0x61, 0x6e, 0x76, 0x69, 0x6c,
0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x65, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x6f, 0x46, 0x61, 0x6b, 0x61, 0x6f, 0x66, 0x6f, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x6a, 0x65, 0x4b, 0x6f, 0x161, 0x72, 0x65, 0x4b, 0x76,
0x61, 0x64, 0x17e, 0x61, 0x6c, 0x65, 0x6a, 0x69, 0x6e, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x4e, 0x69, 0x6a, 0x75, 0x65, 0x50, 0x61, 0x67, 0x6f, 0x20,
0x50, 0x61, 0x67, 0x6f, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x4d, 0x6f, 0x72,
0x7a, 0x62, 0x69, 0x52, 0x61, 0x72, 0x6f, 0x74, 0x6f, 0x6e, 0x67, 0x61,
0x53, 0x61, 0x6a, 0x70, 0x61, 0x6e, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0x61, 0x70, 0x75, 0x56, 0x65, 0x6a,
0x6b, 0x56, 0x65, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x6a, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x65, 0x73,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x6e, 0x61, 0x4c, 0x75,
0x69, 0x76, 0x69, 0x6c, 0x65, 0x42, 0x69, 0x6a, 0x75, 0x6c, 0x61, 0x2c,
0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x61, 0x72, 0x2c, 0x20, 0x53,
0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x4e, 0x6f, 0x76, 0x69, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c,
0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x6f, 0x66, 0x20, 0x53,
0x70, 0x65, 0x6a, 0x6e, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69,
0x6b, 0x6f, 0x52, 0x65, 0x67, 0x69, 0x6e, 0x61, 0x53, 0x6b, 0x6f, 0x72,
0x65, 0x7a, 0x62, 0x69, 0x73, 0x75, 0x6e, 0x64, 0x53, 0x76, 0x2e, 0x20,
0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65, 0x6a, 0x53, 0x76,
0x2e, 0x20, 0x44, 0x17e, 0x6f, 0x6e, 0x53, 0x76, 0x2e, 0x20, 0x54, 0x6f,
0x6d, 0x61, 0x44, 0x69, 0x6d, 0x6f, 0x6e, 0x20, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x4d, 0x65, 0x6b, 0x76, 0x6f, 0x72, 0x69, 0x4c, 0x6f,
0x6e, 0x67, 0x6a, 0x65, 0x72, 0x62, 0x6a, 0x65, 0x6e, 0x4e, 0x69, 0x75,
0x65, 0x4b, 0x14d, 0x6e, 0x61, 0x6b, 0x72, 0x79, 0x4d, 0x79, 0x6e, 0x64,
0x6f, 0x7a, 0x61, 0x41, 0x73, 0x75, 0x144, 0x63, 0x69, 0x6f, 0x6e, 0x4b,
0x61, 0x6a, 0x79, 0x6e, 0x6e, 0x61, 0x44, 0x14d, 0x6d, 0x69, 0x6e, 0x69,
0x6b, 0x61, 0x47, 0x72, 0x79, 0x6e, 0x61, 0x64, 0x61, 0x47, 0x77, 0x61,
0x74, 0x79, 0x6d, 0x61, 0x6c, 0x61, 0x4d, 0x61, 0x63, 0x65, 0x69, 0x14d,
0x4d, 0x79, 0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x65, 0x65, 0x4d, 0x14d, 0x6e,
0x74, 0x73, 0x65, 0x72, 0x72, 0x61, 0x74, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x50, 0x14d, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72,
0x2c, 0x20, 0x50, 0x14d, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x2c, 0x20, 0x50, 0x14d, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x42, 0x61, 0x72, 0x74, 0x68, 0xe9, 0x6c, 0x79, 0x6d, 0x79, 0x4c,
0x6f, 0x6e, 0x67, 0x79, 0x65, 0x61, 0x72, 0x62, 0x79, 0x79, 0x6e, 0x41,
0x64, 0x79, 0x6e, 0x4b, 0x6f, 0x6c, 0x14d, 0x6d, 0x62, 0x6f, 0x48, 0x65,
0x62, 0x72, 0x14d, 0x6e, 0x48, 0x14d, 0x6e, 0x67, 0x6b, 0x14d, 0x6e, 0x67,
0xd4, 0x6d, 0x73, 0x6b, 0xd4, 0x72, 0x61, 0x6c, 0x50, 0x6a, 0x14d, 0x6e,
0x67, 0x6a, 0x61, 0x6e, 0x67, 0x54, 0x61, 0x73, 0x7a, 0x6b, 0x79, 0x6e,
0x74, 0x54, 0x14d, 0x6d, 0x73, 0x6b, 0x57, 0x69, 0x79, 0x6e, 0x74, 0x69,
0x61, 0x6e, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x4b, 0x61, 0x6e, 0x61,
0x72, 0x79, 0x6a, 0x73, 0x6b, 0x65, 0x52, 0x65, 0x70, 0x75, 0x62, 0x6c,
0x69, 0x6b, 0x61, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x14d, 0x6e, 0x65, 0x67,
0x6f, 0x20, 0x50, 0x72, 0x7a, 0x69, 0x6c, 0x14d, 0x6e, 0x64, 0x6b, 0x61,
0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0xd4, 0x77, 0x63, 0x7a, 0x65, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x50, 0x6f, 0x142, 0x65, 0x64,
0x6e, 0x69, 0x6f, 0x77, 0x6f, 0x15a, 0x77, 0x69, 0x79, 0x6e, 0x74, 0x6f,
0x20, 0x48, 0x65, 0x6c, 0x79, 0x6e, 0x61, 0x41, 0x74, 0x79, 0x6e, 0x79,
0x4b, 0x69, 0x73, 0x7a, 0x79, 0x6e, 0x69, 0x14d, 0x77, 0x4b, 0x6f, 0x70,
0x79, 0x6e, 0x68, 0x61, 0x67, 0x61, 0x4b, 0x69, 0x6a, 0x14d, 0x77, 0x4c,
0x69, 0x7a, 0x62, 0x14d, 0x6e, 0x61, 0x4c, 0x14d, 0x6e, 0x64, 0x79, 0x6e,
0x4c, 0x75, 0x6b, 0x73, 0x79, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x4d, 0x14d,
0x6e, 0x61, 0x6b, 0x6f, 0xd4, 0x73, 0x6c, 0x6f, 0x53, 0x61, 0x72, 0x61,
0x74, 0x14d, 0x77, 0x57, 0x69, 0x65, 0x64, 0x79, 0x144, 0x47, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x20, 0x57, 0x79, 0x73, 0x70, 0x61, 0x4b, 0x14d, 0x6d,
0x6f, 0x72, 0x79, 0x4b, 0x65, 0x72, 0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e,
0x6f, 0x77, 0x65, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x42, 0x6f, 0x75,
0x67, 0x61, 0x69, 0x6e, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2019, 0x6f, 0x77,
0x61, 0x20, 0x57, 0x79, 0x73, 0x70, 0x61, 0x57, 0x79, 0x73, 0x70, 0x61,
0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x6f,
0x627, 0x628, 0x64a, 0x20, 0x62c, 0x627, 0x646, 0x627, 0x64a, 0x6aa, 0x631, 0x627,
0x627, 0x64a, 0x68a, 0x64a, 0x633, 0x20, 0x627, 0x628, 0x628, 0x627, 0x627, 0x644,
0x62c, 0x64a, 0x631, 0x632, 0x627, 0x632, 0x645, 0x627, 0x631, 0x627, 0x628, 0x627,
0x645, 0x627, 0x6aa, 0x648, 0x628, 0x646, 0x6af, 0x64a, 0x628, 0x64a, 0x646, 0x62c,
0x627, 0x644, 0x628, 0x633, 0x627, 0x626, 0x648, 0x628, 0x644, 0x646, 0x67d, 0x627,
0x626, 0x631, 0x628, 0x648, 0x62c, 0x645, 0x628, 0x648, 0x631, 0x627, 0x642, 0x627,
0x626, 0x631, 0x627, 0x6aa, 0x627, 0x633, 0x627, 0x628, 0x644, 0x627, 0x646, 0x6aa,
0x627, 0x633, 0x64a, 0x648, 0x67d, 0x627, 0x6aa, 0x648, 0x646, 0x627, 0x6aa, 0x631,
0x64a, 0x68a, 0x627, 0x6aa, 0x627, 0x631, 0x627, 0x644, 0x20, 0x627, 0x64a, 0x648,
0x646, 0x641, 0x631, 0x64a, 0x20, 0x67d, 0x627, 0x626, 0x648, 0x646, 0x6af, 0x64a,
0x628, 0x648, 0x631, 0x648, 0x646, 0x62c, 0x648, 0x647, 0x627, 0x646, 0x633, 0x628,
0x631, 0x6af, 0x6aa, 0x645, 0x67e, 0x627, 0x644, 0x627, 0x6aa, 0x64a, 0x6af, 0x627,
0x644, 0x64a, 0x6aa, 0x646, 0x634, 0x627, 0x633, 0x627, 0x644, 0x6cc, 0x628, 0x631,
0x648, 0x6cc, 0x644, 0x6cc, 0x644, 0x648, 0x627, 0x646, 0x68a, 0x627, 0x644, 0x648,
0x628, 0x645, 0x628, 0x627, 0x634, 0x6cc, 0x644, 0x648, 0x633, 0x627, 0x6aa, 0x627,
0x645, 0x644, 0x627, 0x628, 0x648, 0x645, 0x628, 0x627, 0x628, 0x64a, 0x646, 0x645,
0x648, 0x63a, 0x627, 0x62f, 0x6cc, 0x634, 0x648, 0x646, 0x648, 0x627, 0x6aa, 0x634,
0x648, 0x637, 0x622, 0x626, 0x648, 0x6af, 0x648, 0x20, 0x68a, 0x627, 0x626, 0x648,
0x6af, 0x648, 0x67e, 0x648, 0x631, 0x67d, 0x648, 0x20, 0x646, 0x648, 0x648, 0x633,
0x627, 0x626, 0x648, 0x20, 0x67d, 0x648, 0x645, 0x67d, 0x631, 0x67e, 0x648, 0x644,
0x64a, 0x62a, 0x64a, 0x648, 0x646, 0x633, 0x648, 0x646, 0x68a, 0x647, 0x648, 0x6aa,
0x627, 0x62f, 0x6aa, 0x627, 0x64a, 0x646, 0x6aa, 0x631, 0x64a, 0x62c, 0x627, 0x646,
0x6af, 0x648, 0x64a, 0x644, 0x627, 0x627, 0x64a, 0x646, 0x67d, 0x64a, 0x6af, 0x648,
0x627, 0x627, 0x631, 0x627, 0x6af, 0x648, 0x6cc, 0x627, 0x646, 0x627, 0x628, 0x64a,
0x648, 0x646, 0x648, 0x633, 0x20, 0x627, 0x64a, 0x626, 0x631, 0x633, 0x6aa, 0x64a,
0x67d, 0x627, 0x20, 0x645, 0x627, 0x631, 0x6aa, 0x627, 0x6aa, 0x627, 0x631, 0x68a,
0x648, 0x628, 0x627, 0x6aa, 0x648, 0x6aa, 0x648, 0x626, 0x64a, 0x644, 0x627, 0x20,
0x631, 0x6cc, 0x626, 0x648, 0x62c, 0x627, 0x645, 0x64a, 0x646, 0x68a, 0x648, 0x632,
0x627, 0x631, 0x6cc, 0x648, 0x20, 0x6af, 0x627, 0x644, 0x6cc, 0x6af, 0x648, 0x633,
0x633, 0x627, 0x644, 0x67d, 0x627, 0x633, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x622,
0x646, 0x633, 0x627, 0x646, 0x20, 0x644, 0x648, 0x626, 0x6cc, 0x633, 0x67d, 0x648,
0x6aa, 0x648, 0x645, 0x64a, 0x646, 0x627, 0x648, 0x634, 0x648, 0x622, 0x626, 0x6cc,
0x627, 0x627, 0x633, 0x627, 0x646, 0x633, 0x6cc, 0x648, 0x646, 0x627, 0x67d, 0x64a,
0x6aa, 0x648, 0x6aa, 0x646, 0x628, 0x627, 0x62d, 0x64a, 0x647, 0x628, 0x627, 0x647,
0x6cc, 0x627, 0x20, 0x628, 0x6cc, 0x646, 0x62f, 0x631, 0x627, 0x633, 0x628, 0x627,
0x631, 0x628, 0x627, 0x68a, 0x648, 0x633, 0x628, 0x6cc, 0x644, 0x645, 0x628, 0x6cc,
0x644, 0x6cc, 0x632, 0x628, 0x644, 0x627, 0x646, 0x6aa, 0x20, 0x633, 0x628, 0x644,
0x648, 0x646, 0x628, 0x627, 0x626, 0x648, 0x20, 0x648, 0x633, 0x62a, 0x627, 0x628,
0x6af, 0x648, 0x67d, 0x627, 0x628, 0x648, 0x626, 0x633, 0x6aa, 0x64a, 0x645, 0x628,
0x631, 0x62c, 0x20, 0x628, 0x64a, 0x6aa, 0x64a, 0x645, 0x67e, 0x648, 0x20, 0x6af,
0x631, 0x627, 0x646, 0x68a, 0x64a, 0x6aa, 0x627, 0x646, 0x6aa, 0x648, 0x646, 0x6aa,
0x631, 0x627, 0x6aa, 0x633, 0x6aa, 0x627, 0x626, 0x64a, 0x646, 0x633, 0x64a, 0x627,
0x645, 0x646, 0x634, 0x6aa, 0x627, 0x6af, 0x648, 0x686, 0x64a, 0x20, 0x647, 0x648,
0x627, 0x20, 0x647, 0x648, 0x627, 0x633, 0x648, 0x68a, 0x64a, 0x68a, 0x20, 0x62c,
0x648, 0x631, 0x64a, 0x632, 0x6aa, 0x648, 0x633, 0x67d, 0x627, 0x20, 0x631, 0x64a,
0x6aa, 0x627, 0x6aa, 0x631, 0x64a, 0x633, 0x67d, 0x646, 0x6aa, 0x648, 0x626, 0x64a,
0x627, 0x628, 0x627, 0x6aa, 0x64a, 0x648, 0x631, 0x627, 0x633, 0x627, 0x626, 0x648,
0x68a, 0x64a, 0x646, 0x645, 0x627, 0x631, 0x6aa, 0x20, 0x634, 0x648, 0x646, 0x68a,
0x627, 0x648, 0x633, 0x646, 0x68a, 0x627, 0x648, 0x633, 0x646, 0x20, 0x6aa, 0x631,
0x64a, 0x6aa, 0x68a, 0x64a, 0x646, 0x648, 0x631, 0x68a, 0x64a, 0x67d, 0x631, 0x627,
0x626, 0x64a, 0x67d, 0x68a, 0x648, 0x645, 0x64a, 0x646, 0x64a, 0x6aa, 0x627, 0x627,
0x64a, 0x68a, 0x645, 0x648, 0x646, 0x67d, 0x646, 0x627, 0x64a, 0x631, 0x648, 0x646,
0x64a, 0x67e, 0x627, 0x64a, 0x644, 0x20, 0x633, 0x644, 0x648, 0x627, 0x68a, 0x648,
0x631, 0x641, 0x648, 0x631, 0x67d, 0x20, 0x646, 0x64a, 0x644, 0x633, 0x646, 0x641,
0x648, 0x631, 0x67d, 0x627, 0x644, 0x64a, 0x632, 0x627, 0x6af, 0x644, 0x64a, 0x633,
0x20, 0x628, 0x64a, 0x6af, 0x631, 0x627, 0x646, 0x68a, 0x20, 0x62a, 0x631, 0x6aa,
0x6af, 0x631, 0x64a, 0x646, 0x627, 0x68a, 0x627, 0x6af, 0x648, 0x627, 0x68a, 0x64a,
0x644, 0x648, 0x67e, 0x6af, 0x648, 0x626, 0x67d, 0x64a, 0x20, 0x645, 0x627, 0x644,
0x627, 0x6af, 0x648, 0x627, 0x64a, 0x627, 0x6aa, 0x64a, 0x644, 0x6af, 0x64a, 0x627,
0x646, 0x627, 0x647, 0x64a, 0x644, 0x64a, 0x20, 0x641, 0x64a, 0x6aa, 0x633, 0x647,
0x631, 0x645, 0x648, 0x633, 0x644, 0x648, 0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646,
0x627, 0x20, 0x67e, 0x648, 0x644, 0x633, 0x646, 0x627, 0x6aa, 0x633, 0x60c, 0x20,
0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646, 0x627, 0x645, 0x631, 0x64a, 0x646, 0x6af,
0x648, 0x60c, 0x20, 0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646, 0x627, 0x67e, 0x64a,
0x67d, 0x631, 0x633, 0x628, 0x631, 0x6af, 0x60c, 0x20, 0x627, 0x646, 0x68a, 0x64a,
0x627, 0x646, 0x627, 0x67d, 0x64a, 0x644, 0x20, 0x633, 0x67d, 0x64a, 0x60c, 0x20,
0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646, 0x627, 0x648, 0x64a, 0x648, 0x64a, 0x60c,
0x20, 0x627, 0x646, 0x68a, 0x64a, 0x627, 0x646, 0x627, 0x648, 0x646, 0x633, 0x6cc,
0x646, 0x6cc, 0x632, 0x60c, 0x20, 0x627, 0x646, 0x68a, 0x6cc, 0x627, 0x646, 0x627,
0x648, 0x64a, 0x646, 0x627, 0x645, 0x64a, 0x6aa, 0x60c, 0x20, 0x627, 0x646, 0x68a,
0x64a, 0x627, 0x646, 0x627, 0x627, 0x646, 0x648, 0x648, 0x6aa, 0x627, 0x6aa, 0x627,
0x644, 0x648, 0x626, 0x67d, 0x62c, 0x645, 0x627, 0x626, 0x6aa, 0x627, 0x62c, 0x648,
0x646, 0x626, 0x648, 0x644, 0x648, 0x626, 0x64a, 0x20, 0x648, 0x64a, 0x644, 0x645,
0x648, 0x646, 0x67d, 0x64a, 0x633, 0x64a, 0x644, 0x648, 0x60c, 0x20, 0x6aa, 0x64a,
0x646, 0x67d, 0x6aa, 0x64a, 0x6aa, 0x631, 0x627, 0x644, 0x64a, 0x646, 0x68a, 0x6aa,
0x644, 0x627, 0x20, 0x67e, 0x632, 0x644, 0x627, 0x633, 0x20, 0x627, 0x64a, 0x646,
0x62c, 0x644, 0x633, 0x644, 0x648, 0x626, 0x631, 0x20, 0x67e, 0x631, 0x646, 0x633,
0x632, 0x20, 0x6aa, 0x648, 0x627, 0x631, 0x67d, 0x631, 0x645, 0x6cc, 0x633, 0x6cc,
0x626, 0x648, 0x645, 0x646, 0x627, 0x626, 0x648, 0x633, 0x645, 0x64a, 0x631, 0x64a,
0x6af, 0x648, 0x67d, 0x645, 0x627, 0x631, 0x67d, 0x64a, 0x646, 0x6aa, 0x645, 0x62a,
0x627, 0x645, 0x648, 0x631, 0x648, 0x633, 0x645, 0x632, 0x627, 0x67d, 0x644, 0x627,
0x646, 0x645, 0x6cc, 0x646, 0x648, 0x645, 0x6cc, 0x646, 0x64a, 0x645, 0x64a, 0x631,
0x64a, 0x68a, 0x627, 0x645, 0x64a, 0x67d, 0x644, 0x627, 0x6aa, 0x627, 0x67d, 0x627,
0x644, 0x627, 0x645, 0x64a, 0x6aa, 0x633, 0x64a, 0x6aa, 0x648, 0x20, 0x634, 0x647,
0x631, 0x645, 0x6cc, 0x6aa, 0x648, 0x626, 0x6cc, 0x644, 0x648, 0x646, 0x645, 0x627,
0x646, 0x6aa, 0x67d, 0x646, 0x645, 0x627, 0x646, 0x67d, 0x64a, 0x631, 0x64a, 0x645,
0x648, 0x646, 0x67d, 0x64a, 0x20, 0x648, 0x64a, 0x68a, 0x64a, 0x648, 0x645, 0x627,
0x646, 0x67d, 0x633, 0x631, 0x64a, 0x67d, 0x646, 0x64a, 0x648, 0x64a, 0x627, 0x631,
0x6aa, 0x646, 0x648, 0x631, 0x627, 0x646, 0x647, 0x64a, 0x627, 0x628, 0x64a, 0x648,
0x644, 0x627, 0x647, 0x60c, 0x20, 0x627, 0x62a, 0x631, 0x20, 0x68a, 0x6aa, 0x648,
0x67d, 0x627, 0x633, 0x64a, 0x646, 0x67d, 0x631, 0x60c, 0x20, 0x627, 0x62a, 0x631,
0x20, 0x68a, 0x6aa, 0x648, 0x67d, 0x627, 0x646, 0x64a, 0x648, 0x20, 0x633, 0x64a,
0x644, 0x645, 0x60c, 0x20, 0x627, 0x62a, 0x631, 0x20, 0x68a, 0x6aa, 0x648, 0x67d,
0x627, 0x646, 0x64a, 0x648, 0x6aa, 0x627, 0x648, 0x6aa, 0x64a, 0x646, 0x627, 0x6af,
0x627, 0x67e, 0x646, 0x627, 0x645, 0x627, 0x67e, 0x64a, 0x631, 0x627, 0x645, 0x64a,
0x631, 0x64a, 0x628, 0x648, 0x641, 0x648, 0x646, 0x64a, 0x6aa, 0x633, 0x67e, 0x648,
0x631, 0x67d, 0x20, 0x627, 0x648, 0x20, 0x67e, 0x631, 0x646, 0x633, 0x627, 0x633,
0x67e, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x67b, 0x64a, 0x67d, 0x67e, 0x648,
0x631, 0x67d, 0x648, 0x20, 0x648, 0x64a, 0x644, 0x647, 0x648, 0x67e, 0x648, 0x631,
0x67d, 0x648, 0x20, 0x631, 0x64a, 0x6aa, 0x648, 0x67e, 0x646, 0x67d, 0x627, 0x20,
0x627, 0x631, 0x64a, 0x646, 0x627, 0x633, 0x631, 0x64a, 0x646, 0x6aa, 0x646, 0x20,
0x627, 0x646, 0x644, 0x64a, 0x67d, 0x647, 0x64a, 0x633, 0x64a, 0x641, 0x64a, 0x631,
0x6cc, 0x62c, 0x6cc, 0x646, 0x627, 0x631, 0x64a, 0x632, 0x648, 0x644, 0x64a, 0x648,
0x67d, 0x631, 0x64a, 0x648, 0x20, 0x628, 0x631, 0x627, 0x646, 0x6aa, 0x648, 0x633,
0x646, 0x67d, 0x627, 0x631, 0x64a, 0x645, 0x633, 0x64a, 0x646, 0x67d, 0x64a, 0x627,
0x6af, 0x648, 0x633, 0x64a, 0x646, 0x67d, 0x648, 0x20, 0x68a, 0x648, 0x645, 0x64a,
0x646, 0x6af, 0x648, 0x633, 0x627, 0x626, 0x648, 0x20, 0x67e, 0x648, 0x644, 0x648,
0x627, 0x67d, 0x648, 0x6aa, 0x648, 0x631, 0x67d, 0x648, 0x645, 0x627, 0x626, 0x67d,
0x633, 0x67d, 0x6aa, 0x627, 0x633, 0x64a, 0x646, 0x67d, 0x20, 0x628, 0x631, 0x67f,
0x644, 0x64a, 0x645, 0x64a, 0x633, 0x64a, 0x646, 0x67d, 0x20, 0x62c, 0x627, 0x646,
0x632, 0x633, 0x64a, 0x646, 0x67d, 0x20, 0x6aa, 0x67d, 0x633, 0x633, 0x64a, 0x646,
0x67d, 0x20, 0x644, 0x648, 0x633, 0x64a, 0x627, 0x633, 0x64a, 0x646, 0x67d, 0x20,
0x67f, 0x627, 0x645, 0x633, 0x633, 0x64a, 0x646, 0x67d, 0x20, 0x648, 0x646, 0x633,
0x64a, 0x646, 0x67d, 0x633, 0x648, 0x626, 0x641, 0x67d, 0x20, 0x6aa, 0x631, 0x646,
0x67d, 0x67d, 0x64a, 0x6af, 0x648, 0x633, 0x64a, 0x6af, 0x644, 0x67e, 0x627, 0x67f,
0x648, 0x644, 0x64a, 0x62a, 0x64a, 0x62c, 0x648, 0x627, 0x646, 0x627, 0x67d, 0x648,
0x631, 0x646, 0x67d, 0x648, 0x67d, 0x648, 0x631, 0x67d, 0x648, 0x644, 0x627, 0x648,
0x64a, 0x646, 0x6aa, 0x648, 0x648, 0x631, 0x648, 0x627, 0x626, 0x64a, 0x67d, 0x20,
0x647, 0x627, 0x626, 0x648, 0x633, 0x648, 0x646, 0x64a, 0x20, 0x67e, 0x64a, 0x6af,
0x64a, 0x627, 0x6aa, 0x648, 0x62a, 0x627, 0x62a, 0x6aa, 0x64a, 0x633, 0x64a, 0x68a,
0x64a, 0x648, 0x633, 0x68a, 0x648, 0x645, 0x627, 0x646, 0x67d, 0x20, 0x62f, 0x631,
0x648, 0x64a, 0x626, 0x644, 0x645, 0x6aa, 0x648, 0x627, 0x626, 0x631, 0x64a, 0x645,
0x648, 0x633, 0x646, 0x645, 0x64a, 0x6aa, 0x20, 0x645, 0x631, 0x68a, 0x648, 0x67e,
0x627, 0x645, 0x631, 0x631, 0x648, 0x68c, 0x631, 0x627, 0x633, 0x64a, 0x648, 0x648,
0x627, 0x67d, 0x631, 0x648, 0x644, 0x648, 0x648, 0x633, 0x62a, 0x648, 0x6aa, 0x644,
0x627, 0x646, 0x6af, 0x20, 0x627, 0x626, 0x64a, 0x631, 0x628, 0x646, 0x627, 0x644,
0x645, 0x627, 0x62a, 0x64a, 0x627, 0x646, 0x64a, 0x62f, 0x631, 0x627, 0x6aa, 0x67d,
0x627, 0x626, 0x648, 0x627, 0x64a, 0x6aa, 0x67d, 0x648, 0x628, 0x622, 0x634, 0x6af,
0x627, 0x628, 0x627, 0x62f, 0x622, 0x62a, 0x64a, 0x631, 0x627, 0x626, 0x648, 0x628,
0x626, 0x646, 0x6aa, 0x627, 0x6aa, 0x628, 0x631, 0x646, 0x644, 0x628, 0x634, 0x6aa,
0x64a, 0x6aa, 0x628, 0x631, 0x648, 0x646, 0x627, 0x626, 0x64a, 0x686, 0x648, 0x626,
0x64a, 0x20, 0x628, 0x64a, 0x644, 0x633, 0x646, 0x6aa, 0x648, 0x644, 0x645, 0x628,
0x648, 0x68d, 0x627, 0x6aa, 0x627, 0x62f, 0x628, 0x626, 0x64a, 0x647, 0x648, 0x686,
0x64a, 0x20, 0x645, 0x646, 0x647, 0x627, 0x646, 0x6af, 0x20, 0x6aa, 0x627, 0x646,
0x6af, 0x647, 0x648, 0x648, 0x68a, 0x627, 0x631, 0x6aa, 0x62a, 0x633, 0x6aa, 0x62c,
0x6aa, 0x627, 0x631, 0x62a, 0x627, 0x62c, 0x64a, 0x627, 0x67e, 0x648, 0x631, 0x627,
0x64a, 0x648, 0x631, 0x648, 0x634, 0x644, 0x645, 0x6aa, 0x627, 0x628, 0x644, 0x6aa,
0x645, 0x686, 0x627, 0x633, 0x6aa, 0x64a, 0x6aa, 0x631, 0x627, 0x686, 0x64a, 0x6a9,
0x67d, 0x645, 0x646, 0x68a, 0x648, 0x6a9, 0x646, 0x62f, 0x64a, 0x627, 0x6af, 0x627,
0x6aa, 0x644, 0x6aa, 0x62a, 0x627, 0x6aa, 0x631, 0x64a, 0x633, 0x646, 0x648, 0x64a,
0x627, 0x631, 0x633, 0x6aa, 0x6aa, 0x648, 0x627, 0x644, 0x627, 0x644, 0x645, 0x67e,
0x648, 0x631, 0x6a9, 0x648, 0x686, 0x646, 0x6af, 0x6aa, 0x648, 0x64a, 0x62a, 0x645,
0x6aa, 0x627, 0x626, 0x648, 0x645, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x645, 0x64a,
0x6aa, 0x627, 0x633, 0x631, 0x645, 0x646, 0x64a, 0x644, 0x627, 0x646, 0x64a, 0x6aa,
0x648, 0x633, 0x64a, 0x627, 0x646, 0x648, 0x648, 0x6aa, 0x632, 0x646, 0x64a, 0x62a,
0x633, 0x6aa, 0x646, 0x648, 0x648, 0x627, 0x633, 0x628, 0x626, 0x64a, 0x631, 0x633,
0x6aa, 0x627, 0x648, 0x645, 0x633, 0x6aa, 0x632, 0x628, 0x627, 0x646, 0x64a, 0x641,
0x646, 0x627, 0x645, 0x20, 0x67e, 0x64a, 0x646, 0x647, 0x67e, 0x648, 0x646, 0x67d,
0x64a, 0x627, 0x646, 0x6aa, 0x634, 0x64a, 0x627, 0x646, 0x6af, 0x20, 0x64a, 0x627,
0x646, 0x6af, 0x6aa, 0x648, 0x67d, 0x627, 0x646, 0x633, 0x64a, 0x6aa, 0x64a, 0x632,
0x644, 0x648, 0x631, 0x68a, 0x627, 0x634, 0x62e, 0x627, 0x644, 0x64a, 0x646, 0x633,
0x64a, 0x648, 0x644, 0x634, 0x646, 0x6af, 0x647, 0x627, 0x626, 0x64a, 0x633, 0x631,
0x64a, 0x62f, 0x64a, 0x6aa, 0x648, 0x644, 0x64a, 0x645, 0x633, 0x6aa, 0x62a, 0x627,
0x626, 0x67e, 0x64a, 0x62a, 0x627, 0x634, 0x642, 0x646, 0x62a, 0x67f, 0x645, 0x641,
0x648, 0x67d, 0x648, 0x6aa, 0x64a, 0x648, 0x62a, 0x645, 0x633, 0x6aa, 0x627, 0x648,
0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x62a, 0x631, 0x64a, 0x648, 0x631, 0x645,
0x6aa, 0x64a, 0x627, 0x633, 0x67d, 0x20, 0x646, 0x64a, 0x631, 0x627, 0x648, 0x64a,
0x646, 0x67d, 0x64a, 0x627, 0x646, 0x648, 0x644, 0x627, 0x62f, 0x648, 0x648, 0x633,
0x62a, 0x648, 0x6aa, 0x64a, 0x627, 0x6aa, 0x62a, 0x633, 0x6aa, 0x631, 0x646, 0x6af,
0x648, 0x646, 0x6cc, 0x6aa, 0x627, 0x67d, 0x631, 0x646, 0x628, 0x631, 0x6af, 0x627,
0x632, 0x648, 0x631, 0x632, 0x6aa, 0x64a, 0x646, 0x631, 0x64a, 0x6aa, 0x64a, 0x67e,
0x20, 0x648, 0x631, 0x68a, 0x64a, 0x641, 0x64a, 0x631, 0x648, 0x645, 0x627, 0x68a,
0x64a, 0x631, 0x627, 0x631, 0x64a, 0x6aa, 0x64a, 0x648, 0x6aa, 0x68f, 0x6a9, 0x6bb,
0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627, 0x633, 0x64a, 0x646, 0x67d, 0x20,
0x647, 0x64a, 0x644, 0x64a, 0x646, 0x627, 0x627, 0x633, 0x67d, 0x64a, 0x646, 0x644,
0x64a, 0x627, 0x64a, 0x68a, 0x64a, 0x644, 0x64a, 0x68a, 0x628, 0x631, 0x633, 0x628,
0x64a, 0x646, 0x628, 0x631, 0x648, 0x6aa, 0x646, 0x20, 0x647, 0x644, 0x68a, 0x627,
0x631, 0x648, 0x646, 0x64a, 0x648, 0x6aa, 0x644, 0x627, 0x647, 0x648, 0x628, 0x627,
0x631, 0x67d, 0x644, 0x646, 0x68a, 0x645, 0x64a, 0x646, 0x644, 0x627, 0x631, 0x68a,
0x20, 0x647, 0x627, 0x626, 0x648, 0x67e, 0x631, 0x67f, 0x633, 0x68a, 0x646, 0x64a,
0x627, 0x64a, 0x645, 0x633, 0x67d, 0x631, 0x68a, 0x64a, 0x645, 0x627, 0x64a, 0x67f,
0x646, 0x632, 0x628, 0x631, 0x627, 0x62a, 0x64a, 0x633, 0x644, 0x648, 0x627, 0x628,
0x631, 0x633, 0x644, 0x632, 0x628, 0x62e, 0x627, 0x631, 0x64a, 0x633, 0x67d, 0x628,
0x62f, 0x627, 0x67e, 0x64a, 0x633, 0x67d, 0x628, 0x632, 0x64a, 0x62c, 0x64a, 0x646,
0x686, 0x633, 0x64a, 0x646, 0x627, 0x626, 0x648, 0x6aa, 0x648, 0x67e, 0x646, 0x20,
0x647, 0x64a, 0x6af, 0x646, 0x68a, 0x628, 0x644, 0x646, 0x62c, 0x628, 0x631, 0x627,
0x644, 0x67d, 0x631, 0x6af, 0x631, 0x646, 0x632, 0x64a, 0x647, 0x64a, 0x644, 0x633,
0x646, 0x6aa, 0x64a, 0x622, 0x626, 0x64a, 0x632, 0x644, 0x20, 0x622, 0x641, 0x20,
0x645, 0x64a, 0x646, 0x62c, 0x631, 0x633, 0x64a, 0x6aa, 0x644, 0x64a, 0x646, 0x646,
0x20, 0x6af, 0x631, 0x627, 0x68a, 0x6aa, 0x64a, 0x631, 0x648, 0x641, 0x6aa, 0x650,
0x64a, 0x648, 0x644, 0x633, 0x628, 0x646, 0x644, 0x628, 0x644, 0x64a, 0x627, 0x646,
0x627, 0x644, 0x646, 0x68a, 0x646, 0x644, 0x6af, 0x632, 0x645, 0x628, 0x631, 0x6af,
0x645, 0x64a, 0x68a, 0x631, 0x68a, 0x645, 0x627, 0x644, 0x67d, 0x627, 0x645, 0x646,
0x633, 0x6aa, 0x645, 0x648, 0x646, 0x627, 0x6aa, 0x648, 0x645, 0x627, 0x633, 0x6aa,
0x648, 0x67e, 0x626, 0x631, 0x633, 0x67e, 0x648, 0x68a, 0x6af, 0x648, 0x631, 0x633,
0x64a, 0x627, 0x633, 0x64a, 0x646, 0x20, 0x645, 0x631, 0x64a, 0x646, 0x648, 0x633,
0x631, 0x627, 0x62c, 0x6cc, 0x648, 0x648, 0x633, 0x631, 0x627, 0x62a, 0x648, 0x633,
0x645, 0x641, 0x631, 0x648, 0x67e, 0x648, 0x644, 0x627, 0x633, 0x6aa, 0x648, 0x67e,
0x64a, 0x633, 0x648, 0x641, 0x64a, 0x627, 0x627, 0x633, 0x67d, 0x627, 0x6aa, 0x20,
0x647, 0x648, 0x645, 0x67d, 0x627, 0x644, 0x646, 0x62a, 0x631, 0x627, 0x646, 0x64a,
0x627, 0x644, 0x64a, 0x627, 0x646, 0x648, 0x633, 0x6aa, 0x648, 0x68a, 0x648, 0x632,
0x648, 0x626, 0x67d, 0x64a, 0x6aa, 0x646, 0x648, 0x64a, 0x644, 0x646, 0x64a, 0x633,
0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x627, 0x646, 0x62a,
0x627, 0x646, 0x627, 0x646, 0x627, 0x631, 0x64a, 0x648, 0x648, 0x6aa, 0x631, 0x633,
0x645, 0x633, 0x6aa, 0x648, 0x6aa, 0x648, 0x633, 0x6aa, 0x648, 0x645, 0x648, 0x631,
0x648, 0x6aa, 0x631, 0x6af, 0x64a, 0x648, 0x644, 0x64a, 0x646, 0x645, 0x648, 0x631,
0x64a, 0x634, 0x633, 0x645, 0x64a, 0x627, 0x67d, 0x64a, 0x627, 0x67e, 0x64a, 0x627,
0x622, 0x6aa, 0x644, 0x64a, 0x646, 0x68a, 0x628, 0x648, 0x6af, 0x64a, 0x646, 0x20,
0x648, 0x64a, 0x644, 0x64a, 0x627, 0x686, 0x626, 0x67f, 0x645, 0x686, 0x64a, 0x648,
0x6aa, 0x627, 0x64a, 0x633, 0x67d, 0x631, 0x627, 0x641, 0x627, 0x62a, 0x64a, 0x641,
0x6aa, 0x627, 0x626, 0x648, 0x641, 0x648, 0x641, 0x646, 0x627, 0x641, 0x648, 0x67d,
0x64a, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x632, 0x6af, 0x64a,
0x645, 0x628, 0x64a, 0x626, 0x631, 0x6af, 0x648, 0x627, 0x68a, 0x644, 0x20, 0x6aa,
0x64a, 0x646, 0x627, 0x644, 0x647, 0x648, 0x646, 0x648, 0x20, 0x644, 0x648, 0x644,
0x648, 0x627, 0x64a, 0x646, 0x68a, 0x631, 0x628, 0x631, 0x64a, 0x6aa, 0x631, 0x64a,
0x67d, 0x645, 0x67d, 0x64a, 0x6aa, 0x648, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x6aa,
0x648, 0x627, 0x62c, 0x644, 0x64a, 0x646, 0x645, 0x631, 0x6aa, 0x64a, 0x648, 0x633,
0x633, 0x645, 0x68a, 0x648, 0x64a, 0x646, 0x627, 0x626, 0x631, 0x648, 0x646, 0x64a,
0x648, 0x648, 0x64a, 0x646, 0x627, 0x631, 0x20, 0x641, 0x648, 0x6aa, 0x646, 0x627,
0x626, 0x648, 0x645, 0x64a, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x20, 0x67e, 0x627,
0x6af, 0x648, 0x67e, 0x644, 0x627, 0x626, 0x648, 0x67e, 0x67d, 0x6aa, 0x64a, 0x631,
0x646, 0x67e, 0x648, 0x646, 0x67e, 0x6cc, 0x626, 0x64a, 0x67e, 0x648, 0x631, 0x67d,
0x20, 0x645, 0x648, 0x631, 0x633, 0x628, 0x64a, 0x631, 0x627, 0x631, 0x648, 0x67d,
0x648, 0x646, 0x6af, 0x627, 0x633, 0x64a, 0x67e, 0x646, 0x62a, 0x627, 0x647, 0x67d,
0x64a, 0x62a, 0x631, 0x627, 0x648, 0x627, 0x67d, 0x648, 0x646, 0x6af, 0x627, 0x67d,
0x627, 0x67e, 0x648, 0x648, 0x64a, 0x6aa, 0x648, 0x64a, 0x644, 0x633, 0xd85, 0xdb6,
0xdd2, 0xda2, 0xdcf, 0xdb1, 0xdca, 0xd85, 0xd9a, 0xdca, 0xdbb, 0xdcf, 0xd85, 0xda9,
0xdd2, 0xdc3, 0xdca, 0x20, 0xd85, 0xdb6, 0xdcf, 0xdb6, 0xdcf, 0xd87, 0xdbd, 0xdca,
0xda2, 0xdd2, 0xdba, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd85, 0xdc3, 0xdca, 0xdb8, 0xdcf,
0xdbb, 0xdcf, 0xdb6, 0xdb8, 0xd9a, 0xddc, 0xdb6, 0xdb1, 0xdca, 0xd9c, 0xdd4, 0xdba,
0xdd2, 0xdb6, 0xdb1, 0xdca, 0xda2, 0xdd4, 0xdbd, 0xdca, 0xdb6, 0xdd2, 0xdc3, 0xddd,
0xdb6, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xda7, 0xdba, 0xdbb, 0xdca, 0xdb6, 0xdca, 0x200d,
0xdbb, 0xdc3, 0xdcf, 0xdc0, 0xdd2, 0xdbd, 0xdca, 0xdb6, 0xdd4, 0xda2, 0xdd4, 0xdb8,
0xdca, 0xdb6, 0xdd4, 0xdbb, 0xdcf, 0xd9a, 0xdba, 0xdd2, 0xdbb, 0xddd, 0xd9a, 0xdd0,
0xdc3, 0xdb6, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xd9a, 0xdcf, 0xdc3, 0xdd9, 0xdba, 0xdd4,
0xda7, 0xdcf, 0xd9a, 0xddc, 0xdb1, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xda9, 0xd9a,
0xdbb, 0xdca, 0xdaf, 0xdcf, 0xdbb, 0xdca, 0x20, 0xd91, 0xdc3, 0xdca, 0x20, 0xdc3,
0xdbd, 0xdcf, 0xdb8, 0xdca, 0xda2, 0xdd2, 0xdb6, 0xdd4, 0xda7, 0xdd2, 0xdaf, 0xdde,
0xd85, 0xdbd, 0xdcf, 0xd91, 0xdbd, 0xdca, 0x20, 0xd86, 0xdba, 0xdd4, 0xdb1, 0xdca,
0xdc6, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xda7, 0xdc0, 0xdd4, 0xdb1, 0xdca, 0xd9c, 0xdd0,
0xdb6, 0xdbb, 0xddd, 0xdb1, 0xdca, 0xdc4, 0xdbb, 0xdcf, 0xdbb, 0xdda, 0xda2, 0xddc,
0xdc4, 0xdb1, 0xdca, 0xdb1, 0xdc3, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca, 0xda2,
0xdd4, 0xdb6, 0xdcf, 0xd9a, 0xdb8, 0xdca, 0xdb4, 0xdcf, 0xdbd, 0xdcf, 0xd9a, 0xdcf,
0xdbb, 0xdca, 0xda7, 0xddc, 0xdb8, 0xdca, 0xd9a, 0xdd2, 0xd9c, 0xdcf, 0xdbd, 0xdd2,
0xd9a, 0xdd2, 0xdb1, 0xdca, 0xdc1, 0xdcf, 0xdc3, 0xdcf, 0xdbd, 0xdcf, 0xd9c, 0xddd,
0xdc3, 0xdca, 0xdbd, 0xdd2, 0xdb6, 0xdca, 0xdbb, 0xdc0, 0xdd2, 0xdbd, 0xdca, 0xdbd,
0xddd, 0xdb8, 0xdca, 0xdbd, 0xdd4, 0xdc0, 0xdb1, 0xdca, 0xda9, 0xdcf, 0xdbd, 0xdd4,
0xdb6, 0xdd4, 0xdb8, 0xdca, 0xdb6, 0xdcf, 0xdc1, 0xdd2, 0xdbd, 0xdd4, 0xdc3, 0xdcf,
0xd9a, 0xdcf, 0xdb8, 0xdbd, 0xdb6, 0xddd, 0xdb8, 0xdb4, 0xdd4, 0xda7, 0xddd, 0xdb8,
0xdc3, 0xdda, 0xdbb, 0xdd4, 0xdb8, 0xdca, 0xdb6, 0xdb6, 0xdcf, 0xdb1, 0xdda, 0xdb8,
0xddc, 0xd9c, 0xda9, 0xdd2, 0xdc2, 0xdd4, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xdbb, 0xddc,
0xdc0, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdba, 0xdd2, 0xdbb, 0xddd, 0xdb6, 0xdd2, 0xdaf,
0xdca, 0xda2, 0xdb8, 0xdd9, 0xdb1, 0xdcf, 0xdb1, 0xdd2, 0xdba, 0xdcf, 0xdb8, 0xdda,
0xdb1, 0xdd4, 0xd86, 0xd9a, 0xdca, 0xda0, 0xddc, 0xda7, 0xdca, 0xd94, 0xd86, 0xd9c,
0xda9, 0xddc, 0xd9c, 0xddc, 0xdb4, 0xddc, 0xdbb, 0xdca, 0xdad, 0xddd, 0x2d, 0xdb1,
0xddc, 0xdc0, 0xddd, 0xdc3, 0xdcf, 0xd95, 0x20, 0xdad, 0xddd, 0xdb8, 0xdca, 0xda7,
0xdca, 0x200d, 0xdbb, 0xdd2, 0xdb4, 0xddc, 0xdbd, 0xdd2, 0xda7, 0xdd2, 0xdba, 0xdd4,
0xdb1, 0xdd2, 0xdc3, 0xdca, 0xdc0, 0xdd2, 0xdb1, 0xdca, 0xda9, 0xdca, 0xdc4, 0xddc,
0xd91, 0xd9a, 0xdca, 0xd87, 0xda9, 0xd9a, 0xdca, 0xd87, 0xdb1, 0xd9a, 0xddc, 0xdbb,
0xdcf, 0xda2, 0xdca, 0xd87, 0xdb1, 0xdca, 0xd9c, 0xdd4, 0xdba, 0xdd2, 0xdbd, 0xdcf,
0xdc0, 0xd87, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xd9c, 0xdd4, 0xdc0, 0xdcf, 0xd87, 0xdbb,
0xd9c, 0xdd4, 0xd85, 0xdba, 0xdd2, 0xdb1, 0xdcf, 0xdb6, 0xdd4, 0xdc0, 0xdb1, 0xddd,
0xdc3, 0xdca, 0x20, 0xd85, 0xdba, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xda7, 0xdb8,
0xdbb, 0xdca, 0xd9a, 0xdcf, 0xd9a, 0xddc, 0xdbb, 0xdca, 0xdaf, 0xddc, 0xdb6, 0xdcf,
0xda2, 0xdd4, 0xda2, 0xdd4, 0xdba, 0xdd2, 0xdbd, 0xdcf, 0x20, 0xdbb, 0xdd2, 0xda2,
0xddd, 0xdbd, 0xdcf, 0xdb8, 0xdd9, 0xdb1, 0xdca, 0xda9, 0xddd, 0xdc3, 0xdcf, 0xdbb,
0xdd2, 0xdba, 0xddd, 0x20, 0xd9c, 0xdbd, 0xdd9, 0xd9c, 0xddc, 0xdc3, 0xdca, 0xdc3,
0xddc, 0xdbd, 0xdca, 0xda7, 0xdcf, 0xdc3, 0xdd0, 0xdb1, 0xdca, 0x20, 0xda2, 0xdd4,
0xdc0, 0xdcf, 0xdb1, 0xdca, 0xdc3, 0xdd0, 0xdb1, 0xdca, 0x20, 0xdbd, 0xdd4, 0xdc0,
0xdd2, 0xdc3, 0xdca, 0xda7, 0xdd4, 0xd9a, 0xdd4, 0xdb8, 0xdb1, 0xdca, 0xd8b, 0xdc2,
0xdd4, 0xd85, 0xdba, 0xdd2, 0xdba, 0xdcf, 0xd85, 0xdbb, 0xdd6, 0xdb6, 0xdcf, 0xd85,
0xdc3, 0xdd4, 0xdb1, 0xdca, 0xdc3, 0xdd2, 0xdba, 0xddc, 0xdb1, 0xdca, 0xd87, 0xda7,
0xdd2, 0xd9a, 0xddc, 0xd9a, 0xdb1, 0xdca, 0xdb6, 0xdc4, 0xdd2, 0xdba, 0xdcf, 0xdb6,
0xdc4, 0xdd2, 0xdba, 0xdcf, 0x20, 0xdb6, 0xdb1, 0xdca, 0xdaf, 0xdd9, 0xdbb, 0xdcf,
0xdc3, 0xdca, 0xdb6, 0xdcf, 0xdb6, 0xda9, 0xddd, 0xdc3, 0xdca, 0xdb6, 0xdd9, 0xdbd,
0xdd9, 0xdb8, 0xdca, 0xdb6, 0xdd9, 0xdbd, 0xdd3, 0xdc3, 0xdca, 0xdb6, 0xdca, 0xdbd,
0xdd0, 0xdb1, 0xdca, 0xd9a, 0xdca, 0x2d, 0xdc3, 0xdd0, 0xdb6, 0xdca, 0xdbd, 0xdb1,
0xdca, 0xdb6, 0xddc, 0xd86, 0x20, 0xdc0, 0xdd2, 0xdc3, 0xdca, 0xda7, 0xdcf, 0xdb6,
0xddc, 0xd9c, 0xddc, 0xda7, 0xdcf, 0xdb6, 0xddc, 0xdba, 0xdd2, 0xdc3, 0xdd2, 0xd9a,
0xdda, 0xdb8, 0xdca, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xda2, 0xdca, 0x20, 0xdb6,
0xddc, 0xd9a, 0xdca, 0xd9a, 0xd9a, 0xdb8, 0xdca, 0xdb4, 0xddd, 0x20, 0xd9c, 0xdca,
0x200d, 0xdbb, 0xdd1, 0xdb1, 0xdca, 0xda9, 0xdca, 0xd9a, 0xdd0, 0xdb1, 0xdca, 0xd9a,
0xdd4, 0xdb1, 0xdca, 0xd9a, 0xdbb, 0xd9a, 0xdc3, 0xdca, 0xd9a, 0xdda, 0xdba, 0xdd9,
0xdb1, 0xdca, 0xd9a, 0xdda, 0xdb8, 0xdb1, 0xdca, 0xda0, 0xdd2, 0xd9a, 0xdcf, 0xd9c,
0xddd, 0xda0, 0xdd2, 0xdc4, 0xdd4, 0xd86, 0xdc4, 0xdd4, 0xd86, 0xdc3, 0xdd2, 0xdba,
0xdd4, 0xda9, 0xdcf, 0xda9, 0xdca, 0x20, 0xda2, 0xdd4, 0xdc0, 0xdcf, 0xdbb, 0xdd9,
0xdc3, 0xdca, 0xd9a, 0xddc, 0xdc3, 0xdca, 0xda7, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xd9a,
0xdca, 0x200d, 0xdbb, 0xdd9, 0xdc3, 0xdca, 0xda7, 0xdb1, 0xdca, 0xd9a, 0xdd4, 0xdba,
0xdd2, 0xd85, 0xdb6, 0xdcf, 0xd9a, 0xdd4, 0xdbb, 0xd9a, 0xdcf, 0xdc0, 0xddd, 0xda9,
0xdd9, 0xdb1, 0xdca, 0xdb8, 0xdcf, 0xdbb, 0xdca, 0xd9a, 0xdca, 0xdc1, 0xdc0, 0xdca,
0xdb1, 0xdca, 0xda9, 0xddd, 0xdc3, 0xdb1, 0xdca, 0xda9, 0xddd, 0xdc3, 0xdb1, 0xdca,
0x20, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xd9a, 0xdca, 0xda9, 0xdd9, 0xdb1, 0xdca,
0xdc0, 0xdbb, 0xdca, 0xda9, 0xdd9, 0xdad, 0xdca, 0xdbb, 0xdd4, 0xdc0, 0xdcf, 0xda9,
0xddc, 0xdb8, 0xdd2, 0xdb1, 0xdd2, 0xd9a, 0xdcf, 0xd91, 0xda9, 0xdca, 0xdb8, 0xdb1,
0xdca, 0xda7, 0xdb1, 0xdca, 0xd91, 0xdba, 0xdd2, 0xdbb, 0xdd4, 0xdb1, 0xdda, 0xdb4,
0xdca, 0xd91, 0xdbd, 0xdca, 0xdc3, 0xdd0, 0xdbd, 0xdca, 0xdc0, 0xdaf, 0xddd, 0xdbb,
0xdca, 0xdc6, 0xddd, 0xda7, 0xdca, 0x20, 0xdb1, 0xdd9, 0xdbd, 0xdca, 0xdc3, 0xdb1,
0xdca, 0xdc6, 0xddc, 0xdbb, 0xdca, 0xda7, 0xdbd, 0xdda, 0xdc3, 0xdcf, 0xd9c, 0xdca,
0xdbd, 0xdda, 0xdc3, 0xdca, 0x20, 0xdb6, 0xddc, 0xd9a, 0xdca, 0xd9a, 0xd9c, 0xdd6,
0xdc3, 0xdca, 0x20, 0xdb6, 0xddc, 0xd9a, 0xdca, 0xd9a, 0xd9c, 0xdca, 0xdbb, 0xdd1,
0xdb1, 0xdca, 0xda9, 0xdca, 0x20, 0xda7, 0xdbb, 0xdca, 0xd9a, 0xdca, 0xd9c, 0xdca,
0x200d, 0xdbb, 0xdd0, 0xdb1, 0xda9, 0xdcf, 0xd9c, 0xdca, 0xdc0, 0xdcf, 0xda9, 0xdca,
0xdbd, 0xddd, 0xdb4, 0xdca, 0xd9c, 0xddd, 0xdad, 0xdb8, 0xdcf, 0xdbd, 0xdcf, 0xdc0,
0xd9c, 0xdd4, 0xd85, 0xdba, 0xdcf, 0xd9a, 0xdd4, 0xdba, 0xdd2, 0xdbd, 0xdca, 0xd9c,
0xdba, 0xdb1, 0xdcf, 0xdc0, 0xdc4, 0xdbd, 0xdd2, 0xdc6, 0xdd0, 0xd9a, 0xdca, 0xdc3,
0xdca, 0xdc4, 0xdc0, 0xdcf, 0xdb1, 0xdcf, 0xdc4, 0xdd9, 0xdb8, 0xddc, 0xdc3, 0xdd2,
0xdbd, 0xddd, 0xd89, 0xdb1, 0xdca, 0xda9, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdb4, 0xddc,
0xdbd, 0xdd2, 0xdc3, 0xdca, 0xdb1, 0xddc, 0xd9a, 0xdca, 0xdc3, 0xdca, 0x2c, 0x20,
0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdcf, 0xdb8, 0xdbb, 0xdd9,
0xdb1, 0xdca, 0xd9c, 0xddd, 0x2c, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba,
0xdcf, 0xdb1, 0xdcf, 0xdb4, 0xdd3, 0xda7, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xdb6, 0xdbb,
0xdca, 0xd9c, 0xdca, 0x2c, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdcf, 0xda7, 0xdd9, 0xdbd, 0xdca, 0x20, 0xdb1, 0xd9c, 0xdbb, 0xdba, 0x2c,
0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdcf, 0xdc0, 0xdd9,
0xdc0, 0xdda, 0x2c, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1,
0xdcf, 0xdc0, 0xdd2, 0xdb1, 0xdca, 0xdc3, 0xdb1, 0xdc3, 0xdca, 0x2c, 0x20, 0xd89,
0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdcf, 0xdc0, 0xdd2, 0xdb1, 0xdb8,
0xdd0, 0xd9a, 0xdca, 0x2c, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdcf, 0xd89, 0xdb1, 0xdd4, 0xdc0, 0xdd2, 0xd9a, 0xdca, 0xd89, 0xd9a, 0xdcf,
0xdbd, 0xdd4, 0xdba, 0xdd2, 0xda7, 0xdca, 0xda2, 0xdd0, 0xdb8, 0xdd9, 0xdba, 0xdd2,
0xd9a, 0xdcf, 0xda2, 0xdd6, 0xdb1, 0xddd, 0xdbd, 0xdd4, 0xdc0, 0xdd2, 0xdc3, 0xdca,
0xdc0, 0xdd2, 0xdbd, 0xdca, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdc3, 0xdd9,
0xdbd, 0xddd, 0x2c, 0x20, 0xd9a, 0xdd9, 0xdb1, 0xdca, 0xda7, 0xd9a, 0xdd2, 0xd9a,
0xdca, 0x200d, 0xdbb, 0xdbd, 0xdd9, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xda2, 0xdca, 0xd9a,
0xdca, 0xdbd, 0xdcf, 0x20, 0xdb4, 0xdc3, 0xdca, 0xdbd, 0xdd3, 0xdb8, 0xdcf, 0xdbd,
0xddc, 0xdc3, 0xdca, 0x20, 0xd87, 0xdb1, 0xdca, 0xda2, 0xdbd, 0xdd3, 0xdc3, 0xdca,
0xdbd, 0xddd, 0xdc0, 0xdbb, 0xdca, 0x20, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdb1,
0xdca, 0xdc3, 0xdc3, 0xdca, 0x20, 0xd9a, 0xdca, 0xdc0, 0xdcf, 0xda7, 0xdbb, 0xdca,
0xdb8, 0xdc3, 0xdda, 0xdba, 0xdd2, 0xdba, 0xddd, 0xdb8, 0xdb1, 0xd9c, 0xdd4, 0xd86,
0xdb8, 0xdd0, 0xdb1, 0xddd, 0xdc3, 0xdca, 0xdb8, 0xdd0, 0xdbb, 0xdd2, 0xd9c, 0xddc,
0xda7, 0xdca, 0xdb8, 0xdcf, 0xda7, 0xdd2, 0xdb1, 0xdd2, 0xd9a, 0xdca, 0xdb8, 0xda7,
0xdb8, 0xddc, 0xdbb, 0xddc, 0xdc3, 0xdca, 0xdb8, 0xdc3, 0xda7, 0xdca, 0xdbd, 0xdcf,
0xdb1, 0xdca, 0xdb8, 0xdd9, 0xdb1, 0xddc, 0xdb8, 0xdd2, 0xdb1, 0xdd3, 0xdb8, 0xdd9,
0xdbb, 0xdd2, 0xda9, 0xdcf, 0xdb8, 0xdd9, 0xda7, 0xdca, 0xdbd, 0xd9a, 0xda7, 0xdca,
0xdbd, 0xdcf, 0xdb8, 0xdd9, 0xd9a, 0xdca, 0xdc3, 0xdd2, 0xd9a, 0xddd, 0x20, 0xdb1,
0xd9c, 0xdbb, 0xdba, 0xdb8, 0xdd2, 0xd9a, 0xdd2, 0xdba, 0xdd4, 0xdbd, 0xdb1, 0xdca,
0xdb8, 0xddc, 0xdb1, 0xdca, 0xd9a, 0xdca, 0xda7, 0xdb1, 0xdca, 0xdb8, 0xddc, 0xdb1,
0xdca, 0xda7, 0xdd9, 0xdbb, 0xdda, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xda7, 0xdd9, 0xdc0,
0xdd2, 0xda9, 0xdd2, 0xdba, 0xddd, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xda7, 0xdca, 0xdc3,
0xdda, 0xdbb, 0xdcf, 0xda7, 0xdca, 0xdb1, 0xdc3, 0xdca, 0xdc3, 0xddd, 0xdb1, 0xdd2,
0xdc0, 0xdca, 0xdba, 0xddd, 0xd9a, 0xdca, 0xdb1, 0xddd, 0xdb8, 0xdca, 0xdb1, 0xddc,
0xdbb, 0xddc, 0xdb1, 0xdca, 0xdc4, 0xdcf, 0xdb6, 0xdd2, 0xdba, 0xdd4, 0xdbd, 0xdcf,
0x2c, 0x20, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xda9, 0xd9a, 0xddd, 0xda7,
0xdcf, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x2c, 0x20, 0xd8b, 0xdad, 0xdd4,
0xdbb, 0xdd4, 0x20, 0xda9, 0xd9a, 0xddd, 0xda7, 0xdcf, 0xdb1, 0xdc0, 0x20, 0xdc3,
0xdbd, 0xdd9, 0xdb8, 0xdca, 0xdb8, 0x2c, 0x20, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4,
0x20, 0xda9, 0xd9a, 0xddd, 0xda7, 0xdcf, 0xdb1, 0xdd6, 0xd9a, 0xdca, 0xd94, 0xda2,
0xdd2, 0xdb1, 0xd9c, 0xdcf, 0xdb4, 0xdd0, 0xdb1, 0xdb8, 0xdcf, 0xdb4, 0xdd0, 0xdbb,
0xdcf, 0xdb8, 0xdbb, 0xdd2, 0xdb6, 0xddd, 0xdc6, 0xdd3, 0xdb1, 0xdd2, 0xd9a, 0xdca,
0xdc3, 0xdca, 0xdb4, 0xddc, 0xdbb, 0xdca, 0xda7, 0xdca, 0x2d, 0xd95, 0x2d, 0xdb4,
0xdca, 0x200d, 0xdbb, 0xdd2, 0xdb1, 0xdca, 0xdc3, 0xdca, 0xdb4, 0xddc, 0xdbb, 0xdca,
0xda7, 0xdca, 0x20, 0xd94, 0xdc6, 0xdca, 0x20, 0xdc3, 0xdca, 0xdb4, 0xdd9, 0xdba,
0xdd2, 0xdb1, 0xdca, 0xdb4, 0xddc, 0xdbb, 0xdca, 0xdad, 0xddd, 0x20, 0xdc0, 0xdd9,
0xdbd, 0xdca, 0xdc4, 0xddd, 0xdb4, 0xdd4, 0xdc0, 0xda7, 0xddc, 0xdbb, 0xdd2, 0xd9a,
0xddd, 0xdc0, 0xdb4, 0xdd4, 0xdb1, 0xdca, 0xda7, 0xdcf, 0x20, 0xd87, 0xdbb, 0xdd2,
0xdb1, 0xdcf, 0xdc3, 0xdca, 0xdbb, 0xdd0, 0xdb1, 0xdca, 0xd9a, 0xdd2, 0xdb1, 0xdca,
0x20, 0xdb4, 0xdd2, 0xdc0, 0xdd2, 0xdc3, 0xdd4, 0xdb8, 0xdbb, 0xdd9, 0xdc3, 0xdd2,
0xdc6, 0xdca, 0xdbb, 0xdd9, 0xda2, 0xdd2, 0xdb1, 0xdcf, 0xdbb, 0xdd9, 0xdc3, 0xddc,
0xdbd, 0xdd4, 0xda7, 0xdca, 0xdbb, 0xdd2, 0xdba, 0xddd, 0x20, 0xdb6, 0xdca, 0x200d,
0xdbb, 0xdb1, 0xdca, 0xd9a, 0xddd, 0xdc3, 0xdb1, 0xdca, 0xda7, 0xdbb, 0xdda, 0xdb8,
0xdca, 0xdc3, 0xdb1, 0xdca, 0xdad, 0xdd2, 0xdba, 0xdcf, 0xd9c, 0xddd, 0xdc3, 0xdd0,
0xdb1, 0xdca, 0xda7, 0xddd, 0x20, 0xda9, 0xddc, 0xdb8, 0xdd2, 0xdb1, 0xdca, 0xd9c,
0xddd, 0xdc3, 0xdcf, 0xdc0, 0xddd, 0x20, 0xdb4, 0xddd, 0xdbd, 0xddc, 0xd89, 0xda7,
0xddc, 0xd9a, 0xddc, 0xdbb, 0xdca, 0xda7, 0xddd, 0xdb8, 0xdd2, 0xda7, 0xdca, 0xdc3,
0xdd2, 0xda7, 0xdca, 0xd9a, 0xdcf, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xdb6,
0xdbb, 0xdca, 0xdad, 0xdbd, 0xdd9, 0xdb8, 0xdd2, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad,
0x20, 0xda2, 0xddd, 0xdb1, 0xdca, 0xd9c, 0xdda, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad,
0x20, 0xd9a, 0xdd2, 0xda7, 0xdca, 0xdc3, 0xdca, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad,
0x20, 0xdbd, 0xdd4, 0xdc3, 0xdd2, 0xdba, 0xdcf, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad,
0x20, 0xdad, 0xddd, 0xdb8, 0xdc3, 0xdca, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20,
0xdc0, 0xdd2, 0xdb1, 0xdca, 0xda7, 0xdca, 0xdc3, 0xdb1, 0xdca, 0xdc3, 0xdca, 0xdc0,
0xdd2, 0xdc6, 0xdca, 0xda7, 0xdca, 0x20, 0xd9a, 0xdbb, 0xdb1, 0xdca, 0xda7, 0xdca,
0xda7, 0xdd9, 0xd9c, 0xdd4, 0xdc3, 0xdd2, 0xd9c, 0xdbd, 0xdca, 0xdb4, 0xdcf, 0xdad,
0xdd4, 0xdbd, 0xdda, 0xda7, 0xdd2, 0xda2, 0xdd4, 0xd86, 0xdb1, 0xdcf, 0xda7, 0xddc,
0xdbb, 0xdb1, 0xdca, 0xda7, 0xddd, 0xda7, 0xddc, 0xdbb, 0xdca, 0xda7, 0xddd, 0xdbd,
0xdcf, 0xdc0, 0xdd0, 0xdb1, 0xdca, 0xd9a, 0xdd4, 0xdc0, 0xdbb, 0xdca, 0xdc0, 0xdba,
0xdd2, 0xda7, 0xdca, 0xdc4, 0xddd, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xdc0, 0xdd2, 0xdb1,
0xdd2, 0xdb4, 0xdd9, 0xd9c, 0xdca, 0xdba, 0xd9a, 0xdd4, 0xda7, 0xda7, 0xdca, 0xd9a,
0xdd0, 0xdc3, 0xdda, 0xda9, 0xdda, 0xdc0, 0xdd2, 0xdc3, 0xdca, 0xda9, 0xdd4, 0xdb8,
0xddc, 0xdb1, 0xdca, 0xda7, 0xdca, 0x20, 0xda9, 0xdd4, 0x2018, 0xdbb, 0xdca, 0xdc0,
0xdd2, 0xdbd, 0xdca, 0xdb8, 0xd9a, 0xdca, 0xd9a, 0xdd4, 0xdc0, 0xdbb, 0xdd2, 0xdb8,
0xdc0, 0xdca, 0xdc3, 0xdb1, 0xdca, 0xdb8, 0xdd0, 0xd9a, 0xdca, 0xdb8, 0xdd4, 0xdbb,
0xdca, 0xda9, 0xddd, 0xdb4, 0xdd0, 0xdbd, 0xdca, 0xdb8, 0xdbb, 0xdca, 0xdbb, 0xddc,
0xdad, 0xdd9, 0xdbb, 0xdcf, 0xdc3, 0xdd2, 0xdba, 0xddd, 0xdc0, 0xdcf, 0xda7, 0xdca,
0x200d, 0xdbb, 0xddd, 0xdbd, 0xdca, 0xdc0, 0xddc, 0xdc3, 0xdca, 0xda7, 0xddc, 0xd9a,
0xdca, 0xdbd, 0xddd, 0xdb1, 0xdca, 0xd9c, 0xdca, 0xd89, 0xdba, 0xdbb, 0xdca, 0xdb6,
0xdd2, 0xdba, 0xdd9, 0xdb1, 0xdca, 0xd92, 0xda9, 0xdca, 0xdb1, 0xdca, 0xd85, 0xdbd,
0xdca, 0xdb8, 0xda7, 0xdd2, 0xd85, 0xdb8, 0xdca, 0xdb8, 0xdcf, 0xdb1, 0xdca, 0xd87,
0xdb1, 0xdcf, 0xdaf, 0xdd2, 0xdba, 0xdca, 0xdbb, 0xdca, 0xd85, 0xd9a, 0xdca, 0xda7,
0xddd, 0xd85, 0xd9a, 0xdca, 0xda7, 0xddd, 0xdb6, 0xdca, 0xd85, 0xdc1, 0xdca, 0xd9c,
0xdb6, 0xdd0, 0xda7, 0xdca, 0xd87, 0xda7, 0xdd2, 0xdbb, 0xdc0, 0xdd4, 0xdb6, 0xdd0,
0xd9c, 0xdca, 0xda9, 0xdd1, 0xda9, 0xdca, 0xdb6, 0xdc4, 0xdbb, 0xdda, 0xdb1, 0xdca,
0xdb6, 0xdd0, 0xd82, 0xd9a, 0xddc, 0xd9a, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xdb1, 0xdde,
0xdbd, 0xdca, 0xdb6, 0xdd3, 0xdbb, 0xdd4, 0xda7, 0xdca, 0xdb6, 0xdd2, 0xdc1, 0xdca,
0xd9a, 0xdd9, 0xd9a, 0xdca, 0xdb6, 0xdf2, 0xdb1, 0xdcf, 0xdba, 0xdd2, 0xda0, 0xdd2,
0xda7, 0xdcf, 0xda0, 0xddc, 0xdba, 0xdd2, 0xdb6, 0xdbd, 0xdca, 0xdc3, 0xdcf, 0xdb1,
0xdca, 0xd9a, 0xddc, 0xdc5, 0xdb9, 0xda9, 0xdd0, 0xdb8, 0xdc3, 0xdca, 0xd9a, 0xdc3,
0xdca, 0xda9, 0xd9a, 0xdcf, 0xda9, 0xdd2, 0xdbd, 0xdd2, 0xda9, 0xdd4, 0xdb6, 0xdcf,
0xdba, 0xdd2, 0xda9, 0xdd4, 0xdc1, 0xdb1, 0xdca, 0xdb6, 0xdda, 0xdc6, 0xdd0, 0xdb8,
0xd9c, 0xdd4, 0xdc3, 0xdca, 0xda7, 0xdcf, 0xd9c, 0xdcf, 0xdc3, 0xdcf, 0xdc4, 0xdd9,
0xdb6, 0xdca, 0xdbb, 0xddd, 0xdb1, 0xdca, 0xdc4, 0xddd, 0xda0, 0xdd2, 0x20, 0xdb8,
0xdd2, 0xd82, 0x20, 0xdb1, 0xd9c, 0xdbb, 0xdba, 0xdc4, 0xddc, 0xd82, 0xd9a, 0xddc,
0xd82, 0xdc4, 0xddc, 0xdc0, 0xdca, 0xda9, 0xdca, 0xd89, 0xdbb, 0xd9a, 0xdd4, 0xda7,
0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xda2, 0xd9a, 0xdbb, 0xdca, 0xdad, 0xdcf, 0xda2,
0xdba, 0xdb4, 0xdd4, 0xdbb, 0xda2, 0xdd9, 0xdbb, 0xdd4, 0xdc3, 0xdbd, 0xdb8, 0xd9a,
0xdcf, 0xdb6, 0xdd4, 0xdbd, 0xdca, 0xd9a, 0xdb8, 0xdca, 0xda0, 0xda7, 0xdca, 0xd9a,
0xdcf, 0xd9a, 0xdbb, 0xda0, 0xdca, 0xda0, 0xdd2, 0xda0, 0xdd2, 0xd9a, 0xdad, 0xdca,
0xdb8, 0xdb1, 0xdca, 0xda9, 0xdd4, 0xd9b, 0xdb1, 0xdca, 0xda9, 0xdd2, 0xd9c, 0xdcf,
0xd9a, 0xdbd, 0xdca, 0xd9a, 0xda7, 0xdcf, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdca,
0xdb1, 0xddd, 0xdba, 0xdcf, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xd9a, 0xdca,
0xdc0, 0xdcf, 0xdbd, 0xdcf, 0xdbd, 0xdb8, 0xdca, 0xdb4, 0xdd6, 0xdbb, 0xdca, 0xd9a,
0xdd4, 0xda0, 0xdd2, 0xd82, 0xd9a, 0xdd4, 0xdc0, 0xdda, 0xda7, 0xdca, 0xdb8, 0xdd0,
0xd9a, 0xdcf, 0xdc0, 0xdd4, 0xdb8, 0xd9c, 0xda9, 0xdb1, 0xdca, 0xdb8, 0xd9a, 0xdca,
0xdc3, 0xdca, 0xdc3, 0xdcf, 0xdbb, 0xdca, 0xdb8, 0xdd0, 0xdb1, 0xdd2, 0xdbd, 0xdcf,
0xdb8, 0xdc3, 0xdca, 0xd9a, 0xda7, 0xdca, 0xdb1, 0xdd2, 0xd9a, 0xddd, 0xdc3, 0xdd2,
0xdba, 0xdcf, 0xdb1, 0xddc, 0xdc0, 0xddc, 0xd9a, 0xdd4, 0xdc3, 0xdca, 0xdb1, 0xdda,
0xda7, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xdb1, 0xddc, 0xdc0, 0xddc, 0xdc3, 0xdd2,
0xdb6, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xd94, 0xdb8, 0xdca, 0xdc3, 0xdca,
0xd9a, 0xdca, 0xd94, 0xdbb, 0xdcf, 0xdbd, 0xdca, 0xdb1, 0xddc, 0xdb8, 0xdca, 0x20,
0xdb4, 0xdd9, 0xdb1, 0xdca, 0xdb4, 0xddc, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdba, 0xdb1,
0xd9a, 0xdca, 0xdb4, 0xdca, 0xdba, 0xddd, 0xdb1, 0xdca, 0xd9c, 0xdca, 0xdba, 0xdd0,
0xdb1, 0xdca, 0xd9c, 0xdca, 0xd9a, 0xda7, 0xdcf, 0xdbb, 0xdca, 0xd9a, 0xddc, 0xdc3,
0xdca, 0xdad, 0xdcf, 0xdb1, 0xdda, 0xd9a, 0xdca, 0xdba, 0xdd2, 0xdc3, 0xdd2, 0xdbd,
0xddd, 0xdbb, 0xdca, 0xda9, 0xdcf, 0xdbb, 0xdd2, 0xdba, 0xdcf, 0xdaf, 0xdca, 0xdc3,
0xd9a, 0xdca, 0xdc4, 0xdbd, 0xdd2, 0xdb1, 0xdca, 0xdc3, 0xdcf, 0xdb8, 0xdcf, 0xdbb,
0xdca, 0xd9a, 0xdb1, 0xdca, 0xda9, 0xdca, 0xdc3, 0xddd, 0xdbd, 0xdca, 0xdc2, 0xdd0,
0xdb1, 0xdca, 0xdc4, 0xdba, 0xdd2, 0xdc3, 0xdd2, 0xd82, 0xd9c, 0xdb4, 0xdca, 0xdb4,
0xdd6, 0xdbb, 0xdd4, 0xdc0, 0xdc1, 0xdca, 0x200d, 0xdbb, 0xdd9, 0xda9, 0xdca, 0xdb1,
0xdd9, 0xd9a, 0xddd, 0xdbd, 0xdd2, 0xdb8, 0xdca, 0xdc3, 0xd9a, 0xdca, 0xdad, 0xdcf,
0xdba, 0xdd2, 0xdb4, 0xdd9, 0xda7, 0xdc2, 0xdca, 0xd9a, 0xdd9, 0xdb1, 0xdca, 0xda7,
0xdca, 0xda7, 0xdca, 0xdb6, 0xdd2, 0xdbd, 0xdd2, 0xdc3, 0xdd2, 0xda7, 0xdd9, 0xdc4,
0xdd9, 0xdbb, 0xdcf, 0xdb1, 0xdca, 0xdad, 0xdd2, 0xdb8, 0xdca, 0xdb4, 0xdd4, 0xda7,
0xddd, 0xd9a, 0xdd2, 0xdba, 0xddd, 0xda7, 0xddc, 0xdb8, 0xdca, 0xdc3, 0xdca, 0xd9a,
0xdca, 0xd8b, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0xdb6, 0xdcf, 0xda7, 0xdbb, 0xdca, 0xd8b,
0xdbb, 0xdd4, 0xdb8, 0xdca, 0xd9a, 0xdd2, 0xd8b, 0xdc3, 0xdca, 0xda7, 0xdca, 0x2d,
0xdb1, 0xdd9, 0xdbb, 0xdcf, 0xdc0, 0xdd2, 0xdba, 0xdd0, 0xdb1, 0xdca, 0xda7, 0xdd2,
0xdba, 0xdb1, 0xdca, 0xdc0, 0xdca, 0xdbd, 0xda9, 0xdd2, 0xdc0, 0xddc, 0xdc3, 0xdca,
0xda7, 0xddc, 0xd9a, 0xdca, 0xdba, 0xd9a, 0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca, 0xd9a,
0xdca, 0xdbb, 0xdd0, 0xd82, 0xd9c, 0xdd4, 0xdb1, 0xdca, 0xdba, 0xdd9, 0xd9a, 0xda7,
0xdbb, 0xdd2, 0xdb1, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca, 0xdba, 0xdd9, 0xdbb,
0xdd9, 0xdc0, 0xdb1, 0xdca, 0xd87, 0xdc3, 0xddc, 0xdbb, 0xdda, 0xdb6, 0xdbb, 0xdca,
0xdb8, 0xdd2, 0xdba, 0xdd4, 0xda9, 0xdcf, 0xd9a, 0xdd0, 0xdb1, 0xdbb, 0xdd2, 0xd9a,
0xdda, 0xdb4, 0xdca, 0x20, 0xdc0, 0xdbb, 0xdca, 0xda9, 0xdca, 0xdc6, 0xdcf, 0xdbb,
0xddd, 0xdb8, 0xda9, 0xdd9, 0xdba, 0xdd2, 0xdbb, 0xdcf, 0xdbb, 0xdd9, 0xdba, 0xdd2,
0xd9a, 0xdca, 0xda2, 0xdcf, 0xdc0, 0xdd2, 0xd9a, 0xdca, 0xdaf, 0xd9a, 0xdd4, 0xdab,
0xdd4, 0x20, 0xda2, 0xddd, 0xdbb, 0xdca, 0xda2, 0xdd2, 0xdba, 0xdcf, 0xdc0, 0xdc1,
0xdcf, 0x2e, 0x20, 0xdc4, 0xdd9, 0xdbd, 0xdda, 0xdb1, 0xdcf, 0xdc3, 0xdca, 0xda7,
0xdd0, 0xdb1, 0xdca, 0xdbd, 0xdd2, 0xd87, 0xda9, 0xdd2, 0xdbd, 0xdda, 0xda9, 0xdca,
0xdb6, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdc3, 0xdca, 0xdb6, 0xdda, 0xdb1, 0xdca, 0xdb6,
0xdca, 0x200d, 0xdbb, 0xddd, 0xd9a, 0xdb1, 0xdca, 0x20, 0xdc4, 0xdd2, 0xdbd, 0xdca,
0xda9, 0xdcf, 0xdc0, 0xdd2, 0xdb1, 0xdca, 0xd89, 0xdba, 0xdd4, 0xd9a, 0xdca, 0xdbd,
0xdcf, 0xdc4, 0xddd, 0xdb6, 0xdcf, 0xdbb, 0xdca, 0xda7, 0xdca, 0xdbd, 0xdd2, 0xdb1,
0xdca, 0xda9, 0xdda, 0xdb8, 0xdb1, 0xdca, 0xdbd, 0xddd, 0xdbb, 0xdca, 0xda9, 0xdca,
0x20, 0xdc4, 0xddd, 0xdc0, 0xdca, 0xdb8, 0xdd9, 0xdbd, 0xdca, 0xdb6, 0xddd, 0xdbb,
0xdca, 0xdb1, 0xdca, 0xdb4, 0xdbb, 0xdca, 0xdad, 0xdca, 0xdc3, 0xdd2, 0xda9, 0xdca,
0xdb1, 0xdd2, 0xd87, 0xdb8, 0xdca, 0xdc3, 0xdca, 0xda7, 0xdbb, 0xdca, 0xda9, 0xdd1,
0xdb8, 0xdca, 0xd87, 0xdb1, 0xdca, 0xda9, 0xddd, 0xdbb, 0xdcf, 0xdc0, 0xd85, 0xdc3,
0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xd9b, 0xdcf, 0xdb1, 0xdca, 0xd87, 0xdad, 0xdd0,
0xdb1, 0xdca, 0xdc3, 0xdca, 0xdb6, 0xdd9, 0xdbd, 0xdca, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdda, 0xda9, 0xdca, 0xda9, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xdbd, 0xdd2, 0xdb1, 0xdca,
0xdb6, 0xdca, 0x200d, 0xdbb, 0xda7, 0xdd2, 0xdc3, 0xdca, 0xdbd, 0xdc0, 0xdcf, 0xdb6,
0xdca, 0x200d, 0xdbb, 0xdc3, 0xdbd, 0xdca, 0xdc3, 0xdca, 0xdb6, 0xdd4, 0xd9a, 0xdcf,
0xdbb, 0xdd9, 0xdc3, 0xdca, 0xda7, 0xdca, 0xdb6, 0xdd4, 0xda9, 0xdcf, 0xdb4, 0xdd9,
0xdc3, 0xdca, 0xda7, 0xdca, 0xdb6, 0xdd4, 0xdc3, 0xdd2, 0xdb1, 0xdca, 0xda2, 0xdd9,
0xdb1, 0xdca, 0xda0, 0xdd2, 0xdc3, 0xdd2, 0xdb1, 0xddd, 0xd9a, 0xddd, 0xdb4, 0xdb1,
0xdca, 0xdc4, 0xdda, 0xd9c, 0xdb1, 0xdca, 0xda9, 0xdb6, 0xdca, 0xdbd, 0xdd2, 0xdb1,
0xdca, 0xda2, 0xdd2, 0xdb6, 0xdca, 0xdbb, 0xddd, 0xdbd, 0xdca, 0xda7, 0xdcf, 0xd9c,
0xdd4, 0xdc0, 0xdb1, 0xdca, 0xdc3, 0xdda, 0xdc4, 0xdd9, 0xdbd, 0xdca, 0xdc3, 0xdd2,
0xdb1, 0xdca, 0xd9a, 0xdd2, 0xd85, 0xdba, 0xdd2, 0xdbd, 0xdca, 0x20, 0xd94, 0xdc6,
0xdca, 0x20, 0xdb8, 0xdd1, 0xdb1, 0xdca, 0xd89, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1,
0xdca, 0xdb6, 0xdd4, 0xdbd, 0xdca, 0xda2, 0xdbb, 0xdca, 0xdc3, 0xdd2, 0xd9a, 0xdbd,
0xdd2, 0xdb1, 0xdd2, 0xdb1, 0xdca, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd1, 0xda9, 0xdca,
0xd9a, 0xdd2, 0xdbb, 0xddc, 0xdc0, 0xdca, 0xd9a, 0xdd2, 0xdc0, 0xdca, 0xdbd, 0xdd2,
0xdc3, 0xdca, 0xdb6, 0xdb1, 0xdca, 0xdbd, 0xdca, 0xda2, 0xdd4, 0xdb6, 0xdd4, 0xdbd,
0xdca, 0xda2, 0xdb1, 0xdcf, 0xdbd, 0xdb1, 0xdca, 0xda9, 0xdb1, 0xdca, 0xdbd, 0xd9a,
0xdca, 0xdc3, 0xdd0, 0xdb8, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca, 0xdb8, 0xdd0,
0xda9, 0xdca, 0xdbb, 0xdd2, 0xda9, 0xdca, 0xdb8, 0xddd, 0xdbd, 0xdca, 0xda7, 0xdcf,
0xdb8, 0xdbb, 0xdd2, 0xdba, 0xdd9, 0xdc4, 0xdb8, 0xdca, 0xdb1, 0xdca, 0xdb8, 0xdd2,
0xdb1, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xdb8, 0xddc, 0xdb1, 0xdd0, 0xd9a, 0xddd,
0xdb8, 0xddc, 0xdc3, 0xdca, 0xd9a, 0xdc0, 0xdca, 0xd94, 0xdc3, 0xdca, 0xdbd, 0xddd,
0xdb4, 0xdd0, 0xdbb, 0xdd2, 0xdc3, 0xdca, 0xdb4, 0xddc, 0xda9, 0xdca, 0xd9c, 0xddc,
0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd9, 0xd9c, 0xdca, 0xdbb,
0xdd2, 0xd9c, 0xdcf, 0xdbb, 0xddd, 0xdb8, 0xdba, 0xdc3, 0xdb8, 0xdbb, 0xdcf, 0xdc3,
0xdd0, 0xdb1, 0xdca, 0x20, 0xdb8, 0xdd0, 0xdbb, 0xdd2, 0xdb1, 0xddd, 0xdc3, 0xdbb,
0xdcf, 0xda2, 0xdd9, 0xdc0, 0xddd, 0xdc3, 0xdbb, 0xda7, 0xddc, 0xdc0, 0xdca, 0xdc3,
0xdd2, 0xdb8, 0xdca, 0xdc6, 0xdd9, 0xdbb, 0xddd, 0xdb4, 0xddc, 0xdbd, 0xdca, 0xdc3,
0xdca, 0xd9a, 0xddc, 0xdb4, 0xdca, 0xda2, 0xdda, 0xdc3, 0xddc, 0xdc6, 0xdd2, 0xdba,
0xdcf, 0xdc3, 0xdca, 0xda7, 0xddc, 0xd9a, 0xdca, 0xdc4, 0xddd, 0xdb8, 0xdca, 0xda7,
0xdbd, 0xdd2, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdbb, 0xdcf, 0xdb1, 0xdda, 0xd8b, 0xdbd,
0xdca, 0xdba, 0xdb1, 0xddc, 0xdc0, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0xdc0, 0xda9,
0xdd4, 0xdc3, 0xdca, 0xdc0, 0xdad, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0xdc0, 0xdc0,
0xdd2, 0xdba, 0xdb1, 0xdcf, 0xdc0, 0xdd2, 0xdbd, 0xdca, 0xdb1, 0xdd2, 0xdba, 0xdd4,
0xdc3, 0xdca, 0xdc0, 0xddc, 0xdbd, 0xdca, 0xd9c, 0xddc, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd1, 0xda9, 0xdca, 0xdc0, 0xdbb, 0xdca, 0xdc3, 0xddd, 0xdc3, 0xd9c, 0xdca, 0xdbb,
0xdd9, 0xdb6, 0xdca, 0xdc3, 0xdd6, 0xdbb, 0xdd2, 0xda0, 0xdca, 0xd87, 0xdb1, 0xdca,
0xda7, 0xdb1, 0xdcf, 0xdb1, 0xdbb, 0xdd2, 0xdc0, 0xddd, 0xda0, 0xdcf, 0xd9c, 0xddc,
0xdc3, 0xdca, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdc3, 0xdca, 0xdb8, 0xdc3, 0xdca,
0xd9a, 0xddc, 0xd9a, 0xddc, 0xdc3, 0xdca, 0xd9a, 0xddc, 0xdb8, 0xddc, 0xdbb, 0xddd,
0xd9a, 0xdd9, 0xdbb, 0xdca, 0xd9c, 0xdd4, 0xd91, 0xdbd, 0xdd9, 0xdb1, 0xdca, 0xdb8,
0xdcf, 0xdc4, 0xdda, 0xdb8, 0xdcf, 0xdbd, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdba, 0xdd2,
0xdb1, 0xdb8, 0xdd4, 0xdbb, 0xdd4, 0xdc3, 0xdd2, 0xdba, 0xdb8, 0xdba, 0xddd, 0xda7,
0xdd2, 0xdbb, 0xdd2, 0xdba, 0xdd4, 0xdb1, 0xdd2, 0xdba, 0xdb1, 0xdca, 0xd87, 0xdb4,
0xdd2, 0xdba, 0xdcf, 0xd95, 0xd9a, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xda9, 0xdca, 0xdb6,
0xddd, 0xd9c, 0xdb1, 0xdca, 0xdc0, 0xdd2, 0xdbd, 0xdcf, 0xda0, 0xdd0, 0xdad, 0xdb8,
0xdca, 0xda0, 0xdd6, 0xd9a, 0xdca, 0xd8a, 0xdc3, 0xdca, 0xda7, 0xdbb, 0xdca, 0xd91,
0xdc6, 0xdcf, 0xdad, 0xdca, 0xdc6, 0xdcf, 0xd9a, 0xdcf, 0xd95, 0xdc6, 0xddd, 0xdc6,
0xdd3, 0xda2, 0xdd2, 0xdc6, 0xdd4, 0xdb1, 0xdc6, 0xdd4, 0xda7, 0xdd2, 0xd9c, 0xdbd,
0xdb4, 0xd9c, 0xddd, 0xdc3, 0xdca, 0xd9c, 0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdd9,
0xdbb, 0xdca, 0xd9c, 0xdd4, 0xdc0, 0xdcf, 0xda9, 0xdbd, 0xdca, 0xd9a, 0xdd0, 0xdb1,
0xdbd, 0xdca, 0xd9c, 0xdd4, 0xdc0, 0xdcf, 0xdb8, 0xdca, 0xdc4, 0xddc, 0xdb1, 0xddc,
0xdbd, 0xdd4, 0xdbd, 0xdd4, 0xd91, 0xdb1, 0xdca, 0xda9, 0xdbb, 0xdca, 0xdb6, 0xdbb,
0xdd2, 0xd9a, 0xdd2, 0xdbb, 0xdd2, 0xdb8, 0xda7, 0xdd2, 0xd9a, 0xddc, 0xdc3, 0xdca,
0xdbb, 0xdda, 0xd9a, 0xdca, 0xdc0, 0xdcf, 0xda2, 0xdbd, 0xdd9, 0xdba, 0xdd2, 0xdb1,
0xdca, 0xdb8, 0xda2, 0xdd4, 0xdbb, 0xddd, 0xdb8, 0xdcf, 0xd9a, 0xdd4, 0xdc0, 0xdd9,
0xdc3, 0xdcf, 0xdc3, 0xdca, 0xdb8, 0xdd2, 0xda9, 0xdca, 0xdc0, 0xdda, 0xdb1, 0xdc0,
0xdd4, 0xdbb, 0xdd4, 0xdb1, 0xddd, 0xdc6, 0xddd, 0xd9a, 0xdca, 0xdb1, 0xddd, 0xdb8,
0xdd2, 0xdba, 0xdcf, 0xdb4, 0xd9c, 0xddd, 0x20, 0xdb4, 0xd9c, 0xddd, 0xdb4, 0xdbd,
0xdcf, 0xdc0, 0xdd4, 0xdb4, 0xdd2, 0xda7, 0xdca, 0xd9a, 0xdd9, 0xdba, 0xdcf, 0xdb1,
0xdca, 0xdb4, 0xddc, 0xdb1, 0xdca, 0xdb4, 0xdd9, 0xdba, 0xdd2, 0xdb4, 0xddc, 0xdbb,
0xdca, 0xda7, 0xdca, 0x20, 0xdb8, 0xddc, 0xdbb, 0xdd9, 0xdc3, 0xdca, 0xdb6, 0xdd2,
0xdbb, 0xdbb, 0xddd, 0xda7, 0xddd, 0xdb1, 0xdca, 0xd9c, 0xdcf, 0xdc3, 0xdba, 0xdd2,
0xdb4, 0xdcf, 0xdb1, 0xdca, 0xda7, 0xdc4, 0xdd2, 0xdad, 0xdd2, 0xda7, 0xdbb, 0xdcf,
0xdc0, 0xdcf, 0xda7, 0xddc, 0xdb1, 0xdca, 0xd9c, 0xda7, 0xdcf, 0xdb4, 0xdd4, 0xdc0,
0xdda, 0xd9a, 0xdca, 0xdc0, 0xdd0, 0xdbd, 0xdd2, 0xdc3, 0xdca, 0x45, 0x6c, 0x2d,
0x41, 0x61, 0x69, 0xfa, 0x6e, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x161,
0x6f, 0x53, 0x76, 0xe4, 0x74, 0xfd, 0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x161,
0x4b, 0x61, 0x6a, 0x6d, 0x61, 0x6e, 0x69, 0x65, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x6f, 0x76, 0x79, 0x53, 0x61, 0x6c, 0x76, 0xe1, 0x64, 0x6f, 0x72,
0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65,
0x72, 0x6e, 0xe1, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e,
0xe1, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20,
0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72,
0x6e, 0xe1, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x76, 0xe4,
0x74, 0xfd, 0x20, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x65,
0x6a, 0x53, 0x76, 0xe4, 0x74, 0xe1, 0x20, 0x4c, 0x75, 0x63, 0x69, 0x61,
0x53, 0x76, 0x2e, 0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x161, 0x53, 0x76, 0x2e,
0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x74, 0x160, 0xf3, 0x77, 0x61,
0x48, 0x6f, 0x10d, 0x69, 0x6d, 0x69, 0x6e, 0x6f, 0x76, 0x6f, 0x20, 0x4d,
0x65, 0x73, 0x74, 0x6f, 0x43, 0x68, 0x6f, 0x76, 0x64, 0x4b, 0x61, 0x72,
0xe1, 0x10d, 0x69, 0x4b, 0xe1, 0x74, 0x68, 0x6d, 0x61, 0x6e, 0x64, 0x75,
0x4e, 0x69, 0x6b, 0xf3, 0x7a, 0x69, 0x61, 0x55, 0x72, 0x61, 0x13e, 0x73,
0x6b, 0x50, 0x68, 0x6e, 0x6f, 0x6d, 0x20, 0x50, 0xe9, 0x6e, 0x68, 0x55,
0x6c, 0x61, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x55, 0x73, 0x165, 0x2d,
0x4e, 0x65, 0x72, 0x61, 0x4b, 0x61, 0x6e, 0xe1, 0x72, 0x73, 0x6b, 0x65,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x79, 0x4a, 0x75, 0x17e, 0x6e,
0xe1, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x53, 0x76, 0xe4,
0x74, 0xe1, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x73, 0x74,
0x72, 0x61, 0x63, 0x68, 0xe1, 0x6e, 0x41, 0x74, 0xe9, 0x6e, 0x79, 0x42,
0x65, 0x6c, 0x65, 0x68, 0x72, 0x61, 0x64, 0x4b, 0x69, 0x161, 0x69, 0x148,
0x6f, 0x76, 0x13d, 0x75, 0x62, 0x13e, 0x61, 0x6e, 0x61, 0x4c, 0x75, 0x78,
0x65, 0x6d, 0x62, 0x75, 0x72, 0x73, 0x6b, 0x6f, 0x50, 0x61, 0x72, 0xed,
0x17e, 0x52, 0xed, 0x6d, 0x55, 0x13e, 0x6a, 0x61, 0x6e, 0x6f, 0x76, 0x73,
0x6b, 0x56, 0x69, 0x65, 0x64, 0x65, 0x148, 0x5a, 0xe1, 0x68, 0x72, 0x65,
0x62, 0x56, 0x69, 0x61, 0x6e, 0x6f, 0x10d, 0x6e, 0xfd, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x4b, 0x65, 0x72, 0x67, 0x75, 0x65, 0x6c, 0x65,
0x6e, 0x79, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x79, 0x4d, 0x61, 0x75,
0x72, 0xed, 0x63, 0x69, 0x75, 0x73, 0x56, 0x65, 0x13e, 0x6b, 0x6f, 0x6e,
0x6f, 0x10d, 0x6e, 0xfd, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x42,
0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72,
0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e,
0x74, 0x65, 0x72, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53,
0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x53, 0x76, 0x2e, 0x20,
0x4b, 0x72, 0x69, 0x161, 0x74, 0x6f, 0x66, 0x53, 0x76, 0x2e, 0x20, 0x54,
0x6f, 0x6d, 0x61, 0x17e, 0x53, 0x76, 0x2e, 0x20, 0x56, 0x69, 0x6e, 0x63,
0x65, 0x6e, 0x63, 0x69, 0x6a, 0x48, 0x6f, 0x161, 0x69, 0x6d, 0x69, 0x6e,
0x68, 0x4d, 0x75, 0x161, 0x6b, 0x61, 0x74, 0x4b, 0x69, 0x7a, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x69, 0x4b, 0x61,
0x6e, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69,
0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x69, 0x20,
0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x46, 0x65, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x4a, 0x75, 0x17e, 0x6e, 0x61, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x53, 0x76, 0x2e, 0x20, 0x48, 0x65,
0x6c, 0x65, 0x6e, 0x61, 0x42, 0x72, 0x75, 0x73, 0x65, 0x6c, 0x6a, 0x4b,
0xf6, 0x62, 0x65, 0x6e, 0x68, 0x61, 0x76, 0x6e, 0x44, 0x75, 0x6e, 0x61,
0x6a, 0x42, 0x6f, 0x17e, 0x69, 0x10d, 0x6e, 0x69, 0x20, 0x6f, 0x74, 0x6f,
0x6b, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76, 0x69, 0x20, 0x6f, 0x74,
0x6f, 0x6b, 0x69, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d,
0x6e, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x41, 0x62, 0x69, 0x64, 0x6a,
0x61, 0x61, 0x6e, 0x41, 0x6c, 0x6a, 0x65, 0x65, 0x72, 0x69, 0x73, 0x42,
0x61, 0x6d, 0x61, 0x61, 0x6b, 0x6f, 0x42, 0x61, 0x61, 0x67, 0x69, 0x42,
0x69, 0x73, 0x61, 0x61, 0x77, 0x42, 0x61, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x72, 0x65, 0x42, 0x61, 0x72, 0x61, 0x73, 0x61, 0x66, 0x69, 0x6c, 0x51,
0x61, 0x61, 0x68, 0x69, 0x72, 0x61, 0x4b, 0x61, 0x73, 0x61, 0x62, 0x61,
0x6c, 0x61, 0x61, 0x6e, 0x6b, 0x61, 0x43, 0x6f, 0x6e, 0x61, 0x6b, 0x72,
0x69, 0x44, 0x61, 0x72, 0x65, 0x73, 0x61, 0x6c, 0x61, 0x61, 0x6d, 0x4a,
0x69, 0x62, 0x75, 0x75, 0x74, 0x69, 0x44, 0x6f, 0x75, 0x61, 0x61, 0x6c,
0x61, 0x45, 0x6c, 0x20, 0x43, 0x65, 0x79, 0x75, 0x75, 0x6e, 0x46, 0x69,
0x72, 0x69, 0x69, 0x74, 0x61, 0x77, 0x6e, 0x47, 0x61, 0x62, 0x72, 0x6f,
0x6f, 0x6e, 0x48, 0x61, 0x72, 0x61, 0x61, 0x72, 0x65, 0x4a, 0x6f, 0x68,
0x61, 0x6e, 0x73, 0x62, 0x61, 0x61, 0x67, 0x4b, 0x61, 0x6d, 0x62, 0x61,
0x61, 0x6c, 0x61, 0x4b, 0x68, 0x61, 0x72, 0x74, 0x75, 0x75, 0x6d, 0x4c,
0x61, 0x61, 0x67, 0x6f, 0x6f, 0x73, 0x4c, 0x69, 0x62, 0x72, 0x65, 0x66,
0x69, 0x6c, 0x4c, 0x6f, 0x6f, 0x6d, 0x4c, 0x75, 0x77, 0x61, 0x61, 0x6e,
0x64, 0x61, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62, 0x61, 0x61, 0x73, 0x68,
0x69, 0x4c, 0x75, 0x73, 0x61, 0x61, 0x6b, 0x61, 0x4d, 0x61, 0x62, 0x75,
0x75, 0x74, 0x6f, 0x4d, 0x61, 0x73, 0x65, 0x65, 0x72, 0x6f, 0x4d, 0x61,
0x62, 0x61, 0x62, 0x61, 0x61, 0x6e, 0x65, 0x4d, 0x6f, 0x6e, 0x72, 0x6f,
0x66, 0x69, 0x79, 0x61, 0x4e, 0x61, 0x79, 0x72, 0x6f, 0x6f, 0x62, 0x69,
0x49, 0x6e, 0x6a, 0x61, 0x6d, 0x69, 0x6e, 0x61, 0x4e, 0x6f, 0x6f, 0x6b,
0x6a, 0x6f, 0x74, 0x42, 0x6f, 0x6f, 0x72, 0x74, 0x6f, 0x2d, 0x4e, 0x6f,
0x6f, 0x66, 0x6f, 0x53, 0x61, 0x77, 0x20, 0x54, 0x6f, 0x6f, 0x6d, 0x54,
0x69, 0x72, 0x69, 0x62, 0x6f, 0x6f, 0x6c, 0x69, 0x54, 0x75, 0x75, 0x6e,
0x69, 0x73, 0x57, 0x69, 0x6e, 0x64, 0x68, 0x6f, 0x6f, 0x6b, 0x41, 0x6e,
0x6a, 0x6f, 0x72, 0x61, 0x67, 0x65, 0x41, 0x6e, 0x67, 0x75, 0x77, 0x69,
0x6c, 0x61, 0x41, 0x6e, 0x74, 0x69, 0x67, 0x75, 0x77, 0x61, 0x41, 0x72,
0x61, 0x67, 0x75, 0x77, 0x61, 0x79, 0x6e, 0x61, 0x42, 0x75, 0x77, 0x65,
0x6e, 0x6f, 0x73, 0x20, 0x41, 0x79, 0x72, 0x69, 0x73, 0x4b, 0x61, 0x74,
0x61, 0x6d, 0x61, 0x61, 0x72, 0x6b, 0x61, 0x4b, 0x6f, 0x72, 0x64, 0x6f,
0x6f, 0x62, 0x61, 0x4c, 0x61, 0x20, 0x52, 0x69, 0x79, 0x6f, 0x6a, 0x61,
0x4d, 0x65, 0x65, 0x6e, 0x64, 0x6f, 0x6f, 0x73, 0x61, 0x52, 0x69, 0x79,
0x6f, 0x20, 0x4a, 0x61, 0x6c, 0x65, 0x6a, 0x6f, 0x73, 0x53, 0x61, 0x6e,
0x20, 0x4a, 0x75, 0x77, 0x61, 0x61, 0x6e, 0x53, 0x61, 0x6e, 0x20, 0x4c,
0x75, 0x77, 0x69, 0x73, 0x54, 0x75, 0x6b, 0x75, 0x75, 0x6d, 0x61, 0x61,
0x6e, 0x55, 0x73, 0x68, 0x75, 0x61, 0x61, 0x79, 0x41, 0x73, 0x75, 0x6e,
0x6b, 0x69, 0x79, 0x6f, 0x6e, 0x41, 0x74, 0x69, 0x6b, 0x6f, 0x6b, 0x61,
0x61, 0x6e, 0x42, 0x61, 0x61, 0x68, 0x69, 0x79, 0x61, 0x42, 0x61, 0x68,
0x69, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73,
0x42, 0x65, 0x6c, 0x69, 0x69, 0x73, 0x65, 0x42, 0x61, 0x6c, 0x61, 0x6e,
0x6b, 0x2d, 0x53, 0x61, 0x62, 0x6c, 0x6f, 0x6e, 0x42, 0x6f, 0x77, 0x20,
0x46, 0x69, 0x73, 0x74, 0x61, 0x4b, 0x61, 0x6d, 0x62, 0x69, 0x72, 0x69,
0x69, 0x6a, 0x20, 0x42, 0x61, 0x61, 0x79, 0x4b, 0x61, 0x61, 0x6d, 0x62,
0x6f, 0x20, 0x47, 0x61, 0x72, 0x61, 0x6e, 0x64, 0x69, 0x4b, 0x61, 0x61,
0x6e, 0x6b, 0x75, 0x75, 0x6e, 0x4b, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x61,
0x73, 0x4b, 0x61, 0x79, 0x65, 0x65, 0x6e, 0x4b, 0x65, 0x79, 0x6d, 0x61,
0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x61, 0x67, 0x6f, 0x4a, 0x69, 0x77,
0x61, 0x61, 0x68, 0x75, 0x75, 0x4d, 0x61, 0x67, 0x61, 0x61, 0x6c, 0x61,
0x64, 0x61, 0x20, 0x4a, 0x75, 0x61, 0x72, 0x65, 0x7a, 0x4b, 0x6f, 0x73,
0x74, 0x61, 0x20, 0x52, 0x69, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x65,
0x73, 0x74, 0x6f, 0x6f, 0x6e, 0x4b, 0x75, 0x79, 0x61, 0x61, 0x62, 0x61,
0x4b, 0x75, 0x72, 0x61, 0x6b, 0x6f, 0x6f, 0x77, 0x44, 0x61, 0x61, 0x6e,
0x6d, 0x61, 0x61, 0x6b, 0x73, 0x68, 0x61, 0x61, 0x6e, 0x44, 0x6f, 0x6f,
0x73, 0x61, 0x6e, 0x44, 0x6f, 0x6f, 0x73, 0x61, 0x6e, 0x20, 0x4b, 0x69,
0x72, 0x65, 0x65, 0x6b, 0x44, 0x65, 0x6e, 0x66, 0x61, 0x72, 0x44, 0x65,
0x74, 0x6f, 0x72, 0x6f, 0x79, 0x74, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69,
0x69, 0x6b, 0x61, 0x49, 0x69, 0x72, 0x75, 0x6e, 0x65, 0x65, 0x62, 0x45,
0x6c, 0x20, 0x53, 0x61, 0x6c, 0x66, 0x61, 0x64, 0x6f, 0x6f, 0x72, 0x46,
0x6f, 0x6f, 0x74, 0x20, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x46, 0x6f,
0x6f, 0x74, 0x61, 0x6c, 0x65, 0x73, 0x61, 0x47, 0x61, 0x6c, 0x65, 0x79,
0x73, 0x20, 0x42, 0x61, 0x61, 0x79, 0x47, 0x75, 0x75, 0x73, 0x20, 0x42,
0x61, 0x61, 0x79, 0x47, 0x61, 0x72, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x47, 0x61, 0x72, 0x65, 0x6e, 0x61, 0x61, 0x64, 0x61, 0x47,
0x75, 0x77, 0x61, 0x64, 0x65, 0x6c, 0x75, 0x75, 0x62, 0x47, 0x75, 0x77,
0x61, 0x74, 0x65, 0x6d, 0x61, 0x61, 0x6c, 0x61, 0x47, 0x75, 0x77, 0x61,
0x79, 0x61, 0x71, 0x75, 0x77, 0x69, 0x6c, 0x47, 0x75, 0x79, 0x61, 0x61,
0x6e, 0x61, 0x48, 0x61, 0x6c, 0x69, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x48,
0x61, 0x66, 0x61, 0x61, 0x6e, 0x61, 0x48, 0x61, 0x72, 0x6d, 0x6f, 0x73,
0x69, 0x6c, 0x6f, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61,
0x62, 0x6f, 0x6c, 0x69, 0x73, 0x4e, 0x6f, 0x6b, 0x69, 0x73, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x4d, 0x61, 0x72,
0x65, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79,
0x61, 0x61, 0x6e, 0x61, 0x42, 0x65, 0x74, 0x65, 0x73, 0x62, 0x61, 0x61,
0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61,
0x54, 0x65, 0x6c, 0x20, 0x53, 0x69, 0x74, 0x69, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x46, 0x65, 0x65, 0x66, 0x61,
0x61, 0x79, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x61, 0x6e,
0x61, 0x46, 0x69, 0x6e, 0x73, 0x65, 0x65, 0x6e, 0x65, 0x73, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e,
0x61, 0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79,
0x61, 0x61, 0x6e, 0x61, 0x49, 0x6e, 0x75, 0x75, 0x66, 0x69, 0x6b, 0x49,
0x71, 0x61, 0x61, 0x6c, 0x75, 0x75, 0x74, 0x4a, 0x75, 0x6e, 0x69, 0x79,
0x75, 0x75, 0x4c, 0x75, 0x75, 0x73, 0x66, 0x69, 0x6c, 0x65, 0x4d, 0x6f,
0x6e, 0x74, 0x69, 0x73, 0x65, 0x65, 0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x65,
0x6e, 0x74, 0x61, 0x6b, 0x69, 0x4b, 0x69, 0x72, 0x61, 0x61, 0x6c, 0x65,
0x6e, 0x64, 0x69, 0x6b, 0x4c, 0x61, 0x61, 0x20, 0x42, 0x61, 0x61, 0x73,
0x4c, 0x69, 0x69, 0x6d, 0x61, 0x4c, 0x6f, 0x6f, 0x73, 0x20, 0x41, 0x6e,
0x6a, 0x61, 0x6c, 0x69, 0x73, 0x4c, 0x6f, 0x6f, 0x77, 0x61, 0x20, 0x42,
0x69, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x73, 0x20, 0x4b, 0x75, 0x77, 0x61,
0x61, 0x74, 0x61, 0x4d, 0x61, 0x61, 0x73, 0x69, 0x69, 0x79, 0x6f, 0x4d,
0x61, 0x6e, 0x61, 0x61, 0x67, 0x75, 0x77, 0x61, 0x4d, 0x61, 0x6e, 0x61,
0x61, 0x75, 0x75, 0x73, 0x4d, 0x61, 0x61, 0x72, 0x69, 0x67, 0x6f, 0x74,
0x4d, 0x61, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x69, 0x6b, 0x75, 0x79,
0x75, 0x75, 0x4d, 0x61, 0x7a, 0x61, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x4d,
0x65, 0x6e, 0x6f, 0x6f, 0x6d, 0x69, 0x6e, 0x65, 0x65, 0x4d, 0x65, 0x72,
0x69, 0x69, 0x64, 0x61, 0x4d, 0x65, 0x74, 0x6c, 0x61, 0x61, 0x6b, 0x61,
0x74, 0x6c, 0x61, 0x4d, 0x69, 0x69, 0x71, 0x75, 0x75, 0x6c, 0x6f, 0x6e,
0x4d, 0x6f, 0x6f, 0x6e, 0x6b, 0x74, 0x6f, 0x6f, 0x6e, 0x4d, 0x6f, 0x6f,
0x6e, 0x74, 0x65, 0x65, 0x72, 0x65, 0x79, 0x4d, 0x6f, 0x6f, 0x6e, 0x74,
0x61, 0x66, 0x69, 0x69, 0x64, 0x69, 0x79, 0x6f, 0x4d, 0x6f, 0x6f, 0x6e,
0x74, 0x73, 0x65, 0x72, 0x61, 0x61, 0x74, 0x4e, 0x61, 0x73, 0x61, 0x61,
0x77, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x59, 0x6f, 0x6f, 0x6b, 0x42,
0x69, 0x79, 0x75, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x57, 0x61, 0x71,
0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x6f,
0x74, 0x61, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x2c,
0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x6f, 0x74, 0x61, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20,
0x53, 0x61, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x57, 0x61, 0x71, 0x6f,
0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x6f, 0x74,
0x61, 0x4f, 0x6a, 0x69, 0x6e, 0x61, 0x61, 0x67, 0x61, 0x42, 0x61, 0x6e,
0x61, 0x61, 0x6d, 0x61, 0x46, 0x6f, 0x6f, 0x6e, 0x69, 0x6b, 0x69, 0x73,
0x42, 0x6f, 0x6f, 0x72, 0x74, 0x2d, 0x61, 0x77, 0x2d, 0x42, 0x69, 0x72,
0x69, 0x69, 0x6e, 0x73, 0x42, 0x6f, 0x6f, 0x72, 0x74, 0x20, 0x6f, 0x66,
0x20, 0x49, 0x73, 0x62, 0x61, 0x79, 0x6e, 0x42, 0x6f, 0x6f, 0x72, 0x74,
0x61, 0x20, 0x52, 0x69, 0x69, 0x6b, 0x6f, 0x42, 0x75, 0x6e, 0x74, 0x61,
0x20, 0x41, 0x72, 0x69, 0x6e, 0x61, 0x61, 0x73, 0x52, 0x61, 0x61, 0x6e,
0x6b, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x6c, 0x65, 0x65, 0x74, 0x52, 0x65,
0x63, 0x65, 0x79, 0x66, 0x52, 0x65, 0x6a, 0x69, 0x69, 0x6e, 0x61, 0x52,
0x65, 0x73, 0x6f, 0x6c, 0x75, 0x75, 0x74, 0x52, 0x69, 0x79, 0x6f, 0x20,
0x42, 0x61, 0x72, 0x61, 0x61, 0x6e, 0x6b, 0x6f, 0x53, 0x61, 0x6e, 0x74,
0x61, 0x72, 0x65, 0x65, 0x6d, 0x53, 0x61, 0x6e, 0x74, 0x69, 0x79, 0x61,
0x61, 0x67, 0x6f, 0x53, 0x61, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x44, 0x6f,
0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x53, 0x61, 0x61, 0x77, 0x20, 0x42, 0x6f,
0x6f, 0x6c, 0x6f, 0x49, 0x74, 0x6f, 0x71, 0x6f, 0x72, 0x74, 0x6f, 0x6f,
0x6d, 0x69, 0x69, 0x74, 0x53, 0x69, 0x69, 0x74, 0x6b, 0x61, 0x53, 0x74,
0x2e, 0x20, 0x42, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6c, 0x65, 0x6d, 0x69,
0x53, 0x74, 0x2e, 0x20, 0x4a, 0x6f, 0x6f, 0x6e, 0x53, 0x74, 0x2e, 0x20,
0x4b, 0x69, 0x74, 0x69, 0x73, 0x53, 0x74, 0x2e, 0x20, 0x4c, 0x75, 0x73,
0x69, 0x79, 0x61, 0x53, 0x74, 0x2e, 0x20, 0x54, 0x6f, 0x6f, 0x6d, 0x61,
0x73, 0x53, 0x74, 0x2e, 0x20, 0x46, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74,
0x49, 0x73, 0x77, 0x69, 0x66, 0x74, 0x20, 0x4b, 0x61, 0x72, 0x65, 0x6e,
0x74, 0x54, 0x65, 0x67, 0x75, 0x73, 0x69, 0x67, 0x61, 0x6c, 0x62, 0x61,
0x54, 0x75, 0x75, 0x6c, 0x54, 0x69, 0x6a, 0x75, 0x77, 0x61, 0x61, 0x6e,
0x61, 0x54, 0x6f, 0x72, 0x74, 0x6f, 0x6f, 0x6c, 0x61, 0x46, 0x61, 0x6e,
0x6b, 0x75, 0x75, 0x66, 0x61, 0x72, 0x57, 0x61, 0x79, 0x74, 0x68, 0x6f,
0x72, 0x61, 0x73, 0x57, 0x69, 0x6e, 0x69, 0x62, 0x65, 0x67, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x61, 0x61, 0x74, 0x4b, 0x61, 0x79, 0x73, 0x65, 0x65,
0x44, 0x61, 0x66, 0x69, 0x73, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20,
0x64, 0x2019, 0x75, 0x72, 0x66, 0x69, 0x6c, 0x65, 0x4d, 0x61, 0x6b, 0x71,
0x75, 0x77, 0x61, 0x72, 0x69, 0x79, 0x42, 0x61, 0x61, 0x6d, 0x61, 0x72,
0x53, 0x69, 0x79, 0x6f, 0x77, 0x61, 0x54, 0x6f, 0x72, 0x6f, 0x6f, 0x6c,
0x46, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4c, 0x6f, 0x6e, 0x6a, 0x69, 0x72,
0x62, 0x79, 0x65, 0x65, 0x6e, 0x43, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d,
0x6d, 0x61, 0x61, 0x6e, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x79, 0x72, 0x41,
0x6b, 0x74, 0x61, 0x77, 0x41, 0x74, 0x69, 0x79, 0x72, 0x61, 0x77, 0x42,
0x61, 0x71, 0x64, 0x61, 0x61, 0x64, 0x42, 0x61, 0x78, 0x72, 0x65, 0x79,
0x6e, 0x42, 0x61, 0x6e, 0x67, 0x6b, 0x6f, 0x6f, 0x6b, 0x42, 0x61, 0x72,
0x6e, 0x61, 0x61, 0x75, 0x75, 0x6c, 0x42, 0x65, 0x79, 0x72, 0x75, 0x75,
0x64, 0x42, 0x75, 0x72, 0x75, 0x6e, 0x65, 0x79, 0x4a, 0x69, 0x69, 0x74,
0x61, 0x4a, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x44, 0x69,
0x6d, 0x69, 0x73, 0x68, 0x69, 0x71, 0x44, 0x68, 0x61, 0x61, 0x6b, 0x61,
0x51, 0x61, 0x73, 0x61, 0x48, 0x6f, 0x6f, 0x20, 0x4a, 0x69, 0x20, 0x4d,
0x69, 0x68, 0x20, 0x53, 0x69, 0x74, 0x69, 0x48, 0x6f, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x66, 0x75, 0x64, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x69, 0x6b, 0x4a, 0x61, 0x6b, 0x61, 0x61,
0x72, 0x74, 0x61, 0x4a, 0x61, 0x79, 0x61, 0x62, 0x75, 0x72, 0x61, 0x4a,
0x65, 0x65, 0x72, 0x75, 0x73, 0x61, 0x61, 0x6c, 0x65, 0x6d, 0x4b, 0x61,
0x61, 0x62, 0x75, 0x75, 0x6c, 0x4b, 0x61, 0x6d, 0x6b, 0x61, 0x74, 0x6b,
0x61, 0x4b, 0x61, 0x72, 0x61, 0x61, 0x6a, 0x69, 0x4b, 0x68, 0x61, 0x6e,
0x64, 0x69, 0x79, 0x67, 0x61, 0x4b, 0x6f, 0x6c, 0x6b, 0x61, 0x61, 0x74,
0x61, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x61, 0x4b, 0x75, 0x61, 0x6c, 0x61, 0x20, 0x4c, 0x61, 0x6d, 0x62,
0x75, 0x75, 0x72, 0x4b, 0x75, 0x6a, 0x69, 0x6e, 0x67, 0x4d, 0x61, 0x6b,
0x6f, 0x77, 0x4d, 0x61, 0x67, 0x65, 0x64, 0x61, 0x6e, 0x4d, 0x61, 0x6e,
0x69, 0x69, 0x6c, 0x61, 0x4e, 0x6f, 0x66, 0x6f, 0x6b, 0x75, 0x73, 0x6e,
0x65, 0x74, 0x73, 0x69, 0x6b, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x69, 0x6b, 0x42, 0x65, 0x6e, 0x6f, 0x6d, 0x20, 0x42,
0x65, 0x6e, 0x42, 0x6f, 0x74, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x6b,
0x42, 0x6f, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x51, 0x61,
0x64, 0x64, 0x61, 0x72, 0x51, 0x69, 0x79, 0x73, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x52, 0x69, 0x79, 0x61, 0x61, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x6b, 0x61, 0x61, 0x6e, 0x53, 0x68, 0x61, 0x6e, 0x67, 0x68, 0x61, 0x61,
0x79, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x62, 0x75, 0x75, 0x72, 0x53, 0x61,
0x72, 0x65, 0x64, 0x6e, 0x6f, 0x6b, 0x6f, 0x6c, 0x65, 0x79, 0x6d, 0x69,
0x73, 0x6b, 0x54, 0x65, 0x79, 0x62, 0x65, 0x79, 0x54, 0x6f, 0x73, 0x68,
0x6b, 0x65, 0x65, 0x6e, 0x74, 0x54, 0x69, 0x62, 0x69, 0x6c, 0x69, 0x73,
0x69, 0x54, 0x65, 0x68, 0x72, 0x61, 0x61, 0x6e, 0x46, 0x69, 0x79, 0x61,
0x61, 0x6e, 0x74, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x46, 0x61, 0x6c, 0x61,
0x64, 0x69, 0x66, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x61, 0x61, 0x67, 0x59, 0x65, 0x72,
0x65, 0x66, 0x61, 0x6e, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x73, 0x42, 0x61,
0x72, 0x6d, 0x75, 0x75, 0x64, 0x61, 0x4b, 0x65, 0x79, 0x62, 0x20, 0x46,
0x61, 0x61, 0x72, 0x64, 0x65, 0x46, 0x61, 0x72, 0x6f, 0x77, 0x52, 0x65,
0x79, 0x6b, 0x6a, 0x61, 0x66, 0x69, 0x6b, 0x53, 0x6f, 0x77, 0x74, 0x20,
0x4a, 0x6f, 0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x49, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x6c, 0x65, 0x79, 0x41, 0x64, 0x65, 0x6c, 0x61, 0x79, 0x64,
0x65, 0x42, 0x69, 0x72, 0x69, 0x73, 0x62, 0x61, 0x6e, 0x42, 0x6f, 0x72,
0x6f, 0x6b, 0x65, 0x6e, 0x20, 0x48, 0x69, 0x6c, 0x48, 0x75, 0x62, 0x61,
0x61, 0x72, 0x74, 0x4c, 0x6f, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x4d, 0x65,
0x6c, 0x62, 0x6f, 0x6f, 0x6e, 0x42, 0x65, 0x72, 0x74, 0x41, 0x6d, 0x73,
0x74, 0x65, 0x72, 0x64, 0x61, 0x61, 0x6d, 0x41, 0x6e, 0x64, 0x6f, 0x6f,
0x72, 0x61, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x6b, 0x68, 0x61, 0x61,
0x6e, 0x41, 0x74, 0x65, 0x6e, 0x73, 0x42, 0x65, 0x6c, 0x67, 0x61, 0x72,
0x61, 0x79, 0x64, 0x68, 0x42, 0x61, 0x72, 0x6c, 0x69, 0x69, 0x6e, 0x42,
0x61, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c, 0x61, 0x66, 0x61, 0x42, 0x61,
0x72, 0x61, 0x73, 0x61, 0x6c, 0x69, 0x73, 0x42, 0x75, 0x6a, 0x61, 0x72,
0x65, 0x73, 0x74, 0x42, 0x75, 0x64, 0x61, 0x62, 0x65, 0x73, 0x74, 0x42,
0x75, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x65, 0x6e, 0x4a, 0x69, 0x73, 0x69,
0x6e, 0x61, 0x61, 0x77, 0x4b, 0x6f, 0x62, 0x65, 0x6e, 0x68, 0x61, 0x61,
0x67, 0x61, 0x6e, 0x44, 0x68, 0x61, 0x61, 0x62, 0x6c, 0x69, 0x6e, 0x47,
0x65, 0x65, 0x72, 0x6e, 0x69, 0x73, 0x69, 0x48, 0x65, 0x6c, 0x65, 0x6e,
0x69, 0x73, 0x6b, 0x69, 0x41, 0x79, 0x6c, 0x65, 0x20, 0x6f, 0x66, 0x20,
0x4d, 0x61, 0x61, 0x6e, 0x49, 0x73, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x75,
0x6c, 0x4a, 0x61, 0x61, 0x72, 0x73, 0x65, 0x79, 0x4b, 0x69, 0x72, 0x6f,
0x6f, 0x66, 0x4b, 0x69, 0x79, 0x65, 0x65, 0x66, 0x4c, 0x75, 0x62, 0x61,
0x6c, 0x61, 0x61, 0x6e, 0x61, 0x4c, 0x61, 0x6e, 0x64, 0x61, 0x6e, 0x4c,
0x75, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x65, 0x72, 0x67, 0x4d, 0x61, 0x64,
0x72, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x61, 0x72, 0x69, 0x68, 0x61, 0x61,
0x6d, 0x4d, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x4d, 0x6f, 0x73, 0x6b, 0x6f,
0x77, 0x42, 0x61, 0x61, 0x72, 0x69, 0x69, 0x73, 0x42, 0x6f, 0x64, 0x67,
0x6f, 0x72, 0x69, 0x6b, 0x61, 0x42, 0x61, 0x72, 0x61, 0x61, 0x67, 0x53,
0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x69, 0x6e, 0x6f, 0x53, 0x61,
0x72, 0x61, 0x79, 0x65, 0x65, 0x66, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x74,
0x6f, 0x6f, 0x66, 0x53, 0x69, 0x6d, 0x66, 0x65, 0x72, 0x6f, 0x62, 0x6f,
0x6c, 0x49, 0x73, 0x6b, 0x6f, 0x6f, 0x62, 0x6a, 0x65, 0x49, 0x73, 0x74,
0x6f, 0x6b, 0x68, 0x6f, 0x6f, 0x6d, 0x54, 0x61, 0x61, 0x6c, 0x69, 0x6e,
0x54, 0x69, 0x72, 0x61, 0x61, 0x6e, 0x65, 0x55, 0x6c, 0x79, 0x61, 0x6e,
0x6f, 0x66, 0x69, 0x73, 0x6b, 0x46, 0x61, 0x64, 0x75, 0x75, 0x73, 0x46,
0x61, 0x74, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x46, 0x69, 0x79, 0x65, 0x65,
0x6e, 0x61, 0x46, 0x69, 0x6e, 0x69, 0x79, 0x75, 0x75, 0x73, 0x46, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x61, 0x71, 0x72, 0x69,
0x62, 0x53, 0x75, 0x75, 0x72, 0x69, 0x6b, 0x68, 0x41, 0x6e, 0x74, 0x61,
0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x66, 0x6f, 0x4a, 0x61, 0x67, 0x6f,
0x73, 0x4b, 0x69, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x4b, 0x65,
0x72, 0x67, 0x61, 0x6c, 0x65, 0x6e, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x66,
0x69, 0x73, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x68, 0x69, 0x79, 0x61, 0x61,
0x73, 0x4d, 0x61, 0x79, 0x6f, 0x6f, 0x74, 0x65, 0x52, 0x69, 0x79, 0x75,
0x75, 0x6e, 0x69, 0x79, 0x6f, 0x6e, 0x4f, 0x77, 0x6b, 0x6c, 0x61, 0x61,
0x6e, 0x42, 0x6f, 0x6f, 0x67, 0x61, 0x79, 0x6e, 0x66, 0x69, 0x6c, 0x4a,
0x61, 0x74, 0x61, 0x6d, 0x4a, 0x75, 0x75, 0x6b, 0x49, 0x69, 0x73, 0x74,
0x61, 0x72, 0x66, 0x61, 0x6b, 0x6f, 0x66, 0x6f, 0x47, 0x61, 0x6c, 0x61,
0x62, 0x61, 0x67, 0x6f, 0x73, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x72,
0x43, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x47,
0x75, 0x77, 0x61, 0x6d, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61,
0x61, 0x74, 0x69, 0x4b, 0x6f, 0x73, 0x72, 0x69, 0x69, 0x4b, 0x75, 0x77,
0x61, 0x6a, 0x61, 0x6c, 0x65, 0x79, 0x6e, 0x4d, 0x61, 0x6a, 0x72, 0x6f,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x77, 0x65, 0x73, 0x61, 0x73, 0x4e, 0x61,
0x77, 0x72, 0x6f, 0x6f, 0x4e, 0x6f, 0x6f, 0x72, 0x66, 0x65, 0x6b, 0x4e,
0x6f, 0x6f, 0x6d, 0x69, 0x79, 0x61, 0x42, 0x61, 0x67, 0x6f, 0x20, 0x42,
0x61, 0x67, 0x6f, 0x42, 0x61, 0x6c, 0x61, 0x77, 0x42, 0x69, 0x74, 0x6b,
0x61, 0x79, 0x72, 0x6e, 0x42, 0x6f, 0x6e, 0x62, 0x65, 0x79, 0x42, 0x6f,
0x6f, 0x72, 0x74, 0x20, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62, 0x69, 0x53,
0x65, 0x79, 0x62, 0x61, 0x6e, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0x61,
0x62, 0x75, 0x57, 0x61, 0x6c, 0x69, 0x73, 0x46, 0x75, 0x65, 0x72, 0x74,
0x65, 0x20, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x4e, 0x61, 0x73, 0xe1,
0x75, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x54, 0x6f, 0x6d, 0xe1, 0x73,
0x49, 0x73, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x42, 0x75, 0x79,
0x75, 0x6d, 0x62, 0x75, 0x72, 0x61, 0x52, 0x69, 0x6f, 0x20, 0x42, 0x72,
0x61, 0x6e, 0x63, 0x6f, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0xfd, 0x41, 0x6b,
0x74, 0x6f, 0x62, 0xe9, 0x42, 0x69, 0x73, 0xe1, 0x75, 0x43, 0x6f, 0x6e,
0x61, 0x6b, 0x72, 0x79, 0x44, 0x61, 0x72, 0x20, 0x65, 0x73, 0x2d, 0x53,
0x61, 0x6c, 0x61, 0x6d, 0x59, 0x69, 0x62, 0x75, 0x74, 0xed, 0x53, 0x61,
0x6e, 0x74, 0x6f, 0x20, 0x54, 0x6f, 0x6d, 0xe9, 0x4c, 0x6f, 0x73, 0x20,
0xc1, 0x6e, 0x67, 0x65, 0x6c, 0x65, 0x73, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0xf3, 0x6e, 0x4e, 0x61, 0x73, 0x73, 0x61, 0x75, 0x4e, 0x65, 0x77,
0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65,
0x53, 0x61, 0x6e, 0x20, 0x43, 0x72, 0x69, 0x73, 0x74, 0xf3, 0x62, 0x61,
0x6c, 0x44, 0x75, 0x62, 0xe1, 0x69, 0x43, 0x69, 0x75, 0x64, 0x61, 0x64,
0x20, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68,
0x4a, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0xe9, 0x6e, 0x4b, 0x68, 0x61,
0x6e, 0x64, 0x79, 0x67, 0x61, 0x53, 0x68, 0x61, 0x6e, 0x67, 0x68, 0xe1,
0x69, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x55, 0x6c, 0xe1, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x59, 0x61, 0x6e, 0x67, 0xf3, 0x6e, 0x20,
0x28, 0x52, 0x61, 0x6e, 0x67, 0xfa, 0x6e, 0x29, 0x45, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x49, 0x73, 0x6c,
0x61, 0x73, 0x20, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x61, 0x73, 0x49,
0x73, 0x6c, 0x61, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6f, 0x65, 0x41, 0x73,
0x74, 0x72, 0x61, 0x6b, 0x68, 0x61, 0x6e, 0x47, 0x75, 0x65, 0x72, 0x6e,
0x73, 0x65, 0x79, 0x55, 0x6c, 0x79, 0x61, 0x6e, 0x6f, 0x76, 0x73, 0x6b,
0x4e, 0x61, 0x76, 0x69, 0x64, 0x61, 0x64, 0x49, 0x73, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x53, 0x61, 0x69,
0x70, 0xe1, 0x6e, 0x4a, 0x69, 0x6a, 0x69, 0x20, 0x6c, 0x61, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x6f, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x20, 0x4b, 0x75, 0x73, 0x69, 0x6e, 0x69, 0x42, 0x61, 0x62, 0x61, 0x64,
0x6f, 0x73, 0x6b, 0x75, 0x72, 0x61, 0x6b, 0x61, 0x6f, 0x45, 0x6c, 0x73,
0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x61, 0x73,
0x6b, 0x61, 0x7a, 0x69, 0x6e, 0x69, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72,
0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4b, 0x61, 0x73,
0x6b, 0x61, 0x7a, 0x69, 0x6e, 0x69, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20,
0x4b, 0x61, 0x73, 0x6b, 0x61, 0x7a, 0x69, 0x6e, 0x69, 0x42, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x69, 0x20, 0x79, 0x61, 0x20, 0x55, 0x68, 0x69, 0x73,
0x70, 0x61, 0x6e, 0x69, 0x61, 0x50, 0x77, 0x65, 0x74, 0x6f, 0x72, 0x69,
0x6b, 0x6f, 0x4a, 0x69, 0x6a, 0x69, 0x20, 0x6c, 0x61, 0x20, 0x48, 0x6f,
0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x6f, 0x6f, 0x4b, 0x72, 0x69, 0x73, 0x6d, 0x61, 0x73,
0x69, 0x43, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0xf6, 0x61, 0x72, 0x6e, 0x61,
0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x20, 0x43, 0x69, 0x74, 0x79, 0x54,
0x6a, 0x69, 0x74, 0x61, 0x54, 0x6a, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e,
0x68, 0x2d, 0x73, 0x74, 0x61, 0x64, 0x65, 0x6e, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x72, 0x6e, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x65, 0xf6,
0x61, 0x72, 0x6e, 0x61, 0x54, 0x6f, 0x72, 0x73, 0x68, 0x61, 0x6d, 0x6e,
0x53, 0x79, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x6e, 0x53,
0x61, 0x6e, 0x6b, 0x74, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x4b,
0xf6, 0x70, 0x65, 0x6e, 0x68, 0x61, 0x6d, 0x6e, 0x56, 0x61, 0x74, 0x69,
0x6b, 0x61, 0x6e, 0x65, 0x6e, 0x4a, 0x75, 0x6c, 0xf6, 0x6e, 0x4b, 0x6f,
0x6b, 0x6f, 0x73, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x4b, 0x6f, 0x6d, 0x6f,
0x72, 0x65, 0x72, 0x6e, 0x61, 0x4b, 0x65, 0x72, 0x67, 0x75, 0x65, 0x6c,
0x65, 0x6e, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x65, 0x72, 0x6e, 0x61, 0x50, 0xe5, 0x73, 0x6b, 0xf6, 0x6e, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x4d,
0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0xf6, 0x61, 0x72, 0x6e,
0x61, 0x4d, 0x69, 0x64, 0x77, 0x61, 0x79, 0xf6, 0x61, 0x72, 0x6e, 0x61,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0xf6, 0x61, 0x72, 0x6e,
0x61, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0xf6, 0x6e, 0x4b, 0x61, 0x69,
0x6d, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6c, 0x65, 0x4d, 0x65, 0x78, 0x69,
0x6b, 0x6f, 0x2d, 0x53, 0x63, 0x68, 0x74, 0x61, 0x64, 0x74, 0x57, 0x6f,
0x73, 0x63, 0x68, 0x74, 0x6f, 0x6b, 0x4d, 0x75, 0x73, 0x63, 0x68, 0x6b,
0x61, 0x74, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x61, 0x74, 0x6f,
0x72, 0x53, 0xfc, 0xfc, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x65, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x65, 0x73, 0x63, 0x68, 0x74, 0x4b,
0x6f, 0x70, 0x65, 0x68, 0x61, 0x67, 0x65, 0x57, 0x69, 0x65, 0x6e, 0x61,
0x63, 0x68, 0x74, 0x73, 0x2d, 0x49, 0x6e, 0x73, 0x6c, 0x65, 0x4f, 0x73,
0x63, 0x68, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x6c, 0x65, 0x710, 0x712,
0x715, 0x713, 0x722, 0x710, 0x71f, 0x72a, 0x710, 0x710, 0x715, 0x71d, 0x723, 0x20,
0x710, 0x712, 0x712, 0x710, 0x713, 0x719, 0x710, 0x710, 0x72a, 0x710, 0x723, 0x721,
0x710, 0x72a, 0x710, 0x712, 0x710, 0x721, 0x710, 0x71f, 0x718, 0x712, 0x710, 0x722,
0x713, 0x710, 0x718, 0x71d, 0x712, 0x710, 0x722, 0x713, 0x718, 0x720, 0x712, 0x71d,
0x723, 0x710, 0x718, 0x712, 0x720, 0x722, 0x72c, 0x71d, 0x72a, 0x712, 0x72a, 0x710,
0x719, 0x710, 0x712, 0x71d, 0x720, 0x712, 0x718, 0x713, 0x718, 0x721, 0x712, 0x718,
0x72a, 0x710, 0x729, 0x710, 0x717, 0x716, 0x308, 0x717, 0x71f, 0x710, 0x723, 0x710,
0x712, 0x720, 0x722, 0x71f, 0x710, 0x723, 0x712, 0x72c, 0x710, 0x71f, 0x718, 0x722,
0x710, 0x71f, 0x72a, 0x71d, 0x715, 0x710, 0x71f, 0x710, 0x72a, 0x715, 0x710, 0x72a,
0x20, 0x72b, 0x720, 0x721, 0x710, 0x713, 0x71d, 0x712, 0x718, 0x72c, 0x71d, 0x715,
0x718, 0x710, 0x720, 0x710, 0x710, 0x720, 0x20, 0x725, 0x71d, 0x718, 0x722, 0x726,
0x72a, 0x71d, 0x72c, 0x710, 0x718, 0x722, 0x713, 0x712, 0x718, 0x72a, 0x718, 0x722,
0x717, 0x72a, 0x710, 0x72a, 0x71d, 0x71d, 0x718, 0x717, 0x722, 0x71d, 0x723, 0x712,
0x718, 0x72a, 0x713, 0x713, 0x718, 0x712, 0x710, 0x71f, 0x710, 0x721, 0x726, 0x710,
0x720, 0x710, 0x71a, 0x72a, 0x71b, 0x718, 0x721, 0x71f, 0x71d, 0x713, 0x710, 0x720,
0x71d, 0x71f, 0x71d, 0x722, 0x72b, 0x710, 0x723, 0x710, 0x720, 0x710, 0x713, 0x718,
0x723, 0x720, 0x71d, 0x712, 0x72a, 0x71d, 0x712, 0x71d, 0x720, 0x720, 0x718, 0x721,
0x71d, 0x720, 0x718, 0x710, 0x722, 0x715, 0x710, 0x720, 0x718, 0x712, 0x718, 0x721,
0x712, 0x72b, 0x71d, 0x720, 0x718, 0x723, 0x710, 0x71f, 0x710, 0x721, 0x710, 0x720,
0x710, 0x712, 0x718, 0x721, 0x710, 0x726, 0x718, 0x72c, 0x718, 0x721, 0x710, 0x723,
0x71d, 0x72a, 0x718, 0x721, 0x712, 0x710, 0x712, 0x710, 0x722, 0x71d, 0x721, 0x718,
0x729, 0x715, 0x71d, 0x72b, 0x718, 0x721, 0x718, 0x722, 0x72a, 0x718, 0x712, 0x71d,
0x710, 0x722, 0x71d, 0x72a, 0x718, 0x712, 0x71d, 0x722, 0x713, 0x721, 0x71d, 0x722,
0x710, 0x722, 0x71d, 0x710, 0x721, 0x71d, 0x722, 0x718, 0x710, 0x71f, 0x72b, 0x718,
0x72c, 0x710, 0x718, 0x710, 0x713, 0x710, 0x715, 0x710, 0x718, 0x713, 0x710, 0x718,
0x726, 0x718, 0x72a, 0x72c, 0x718, 0x2d, 0x722, 0x718, 0x712, 0x718, 0x723, 0x710,
0x718, 0x20, 0x72c, 0x718, 0x721, 0x71d, 0x71b, 0x72a, 0x71d, 0x726, 0x718, 0x720,
0x71d, 0x723, 0x72c, 0x718, 0x722, 0x723, 0x718, 0x71d, 0x722, 0x715, 0x717, 0x718,
0x71f, 0x710, 0x715, 0x710, 0x71f, 0x710, 0x722, 0x71f, 0x718, 0x72a, 0x713, 0x710,
0x722, 0x713, 0x718, 0x71d, 0x710, 0x710, 0x722, 0x72c, 0x71d, 0x713, 0x718, 0x710,
0x710, 0x72a, 0x710, 0x713, 0x710, 0x718, 0x71d, 0x722, 0x710, 0x712, 0x718, 0x710,
0x71d, 0x722, 0x718, 0x723, 0x20, 0x725, 0x71d, 0x72a, 0x723, 0x71f, 0x710, 0x72c,
0x710, 0x721, 0x72a, 0x71f, 0x710, 0x71f, 0x718, 0x72a, 0x715, 0x718, 0x712, 0x710,
0x713, 0x718, 0x713, 0x718, 0x71d, 0x720, 0x710, 0x20, 0x72a, 0x71d, 0x718, 0x717,
0x710, 0x721, 0x722, 0x715, 0x718, 0x719, 0x710, 0x72a, 0x71d, 0x718, 0x20, 0x713,
0x71d, 0x713, 0x718, 0x723, 0x723, 0x720, 0x72c, 0x710, 0x723, 0x710, 0x722, 0x20,
0x718, 0x710, 0x722, 0x723, 0x710, 0x722, 0x20, 0x720, 0x718, 0x71d, 0x723, 0x72c,
0x718, 0x71f, 0x718, 0x721, 0x710, 0x722, 0x710, 0x718, 0x72b, 0x718, 0x710, 0x71d,
0x710, 0x710, 0x72a, 0x718, 0x712, 0x710, 0x710, 0x723, 0x718, 0x722, 0x71f, 0x723,
0x71d, 0x718, 0x722, 0x710, 0x72c, 0x71d, 0x71f, 0x718, 0x71f, 0x710, 0x722, 0x712,
0x710, 0x717, 0x71d, 0x710, 0x712, 0x710, 0x717, 0x71d, 0x710, 0x20, 0x712, 0x710,
0x722, 0x71d, 0x72a, 0x723, 0x712, 0x710, 0x72a, 0x712, 0x715, 0x718, 0x723, 0x712,
0x71d, 0x720, 0x71d, 0x721, 0x712, 0x71d, 0x720, 0x71d, 0x719, 0x712, 0x720, 0x710,
0x722, 0x71f, 0x2d, 0x723, 0x710, 0x712, 0x720, 0x718, 0x722, 0x712, 0x718, 0x725,
0x20, 0x712, 0x71d, 0x723, 0x72c, 0x710, 0x712, 0x718, 0x713, 0x718, 0x72c, 0x710,
0x712, 0x718, 0x71d, 0x719, 0x71d, 0x71f, 0x721, 0x712, 0x72a, 0x713, 0x20, 0x712,
0x710, 0x71d, 0x71f, 0x710, 0x721, 0x726, 0x718, 0x20, 0x713, 0x72a, 0x722, 0x715,
0x71d, 0x71f, 0x710, 0x722, 0x71f, 0x718, 0x722, 0x71f, 0x710, 0x72a, 0x710, 0x71f,
0x723, 0x71f, 0x710, 0x71d, 0x710, 0x722, 0x71f, 0x710, 0x71d, 0x721, 0x71d, 0x722,
0x72b, 0x71f, 0x713, 0x718, 0x71f, 0x71d, 0x718, 0x710, 0x718, 0x710, 0x723, 0x71d,
0x718, 0x715, 0x710, 0x715, 0x20, 0x710, 0x718, 0x72a, 0x71d, 0x719, 0x71f, 0x718,
0x723, 0x72c, 0x710, 0x20, 0x72a, 0x71d, 0x71f, 0x710, 0x71f, 0x72a, 0x71d, 0x723,
0x72c, 0x718, 0x722, 0x71f, 0x718, 0x71d, 0x710, 0x712, 0x710, 0x71f, 0x718, 0x72a,
0x710, 0x71f, 0x710, 0x718, 0x715, 0x710, 0x722, 0x721, 0x72a, 0x71f, 0x72b, 0x712,
0x71d, 0x722, 0x715, 0x710, 0x723, 0x718, 0x722, 0x715, 0x710, 0x723, 0x718, 0x722,
0x20, 0x71f, 0x72a, 0x71d, 0x71f, 0x715, 0x722, 0x712, 0x72a, 0x715, 0x71d, 0x72c,
0x72a, 0x718, 0x71d, 0x72c, 0x715, 0x718, 0x721, 0x71d, 0x722, 0x71d, 0x71f, 0x710,
0x710, 0x715, 0x721, 0x718, 0x722, 0x72c, 0x718, 0x722, 0x710, 0x71d, 0x72a, 0x718,
0x722, 0x71d, 0x726, 0x71d, 0x710, 0x720, 0x20, 0x723, 0x720, 0x712, 0x710, 0x715,
0x718, 0x72a, 0x726, 0x718, 0x72a, 0x72c, 0x20, 0x722, 0x71d, 0x720, 0x723, 0x718,
0x722, 0x726, 0x718, 0x72a, 0x72c, 0x710, 0x720, 0x71d, 0x719, 0x710, 0x713, 0x720,
0x71d, 0x723, 0x20, 0x712, 0x710, 0x71d, 0x713, 0x718, 0x723, 0x20, 0x712, 0x710,
0x71d, 0x713, 0x72a, 0x710, 0x722, 0x715, 0x20, 0x72c, 0x718, 0x72a, 0x71f, 0x713,
0x72a, 0x71d, 0x722, 0x715, 0x710, 0x713, 0x718, 0x710, 0x715, 0x710, 0x720, 0x718,
0x726, 0x71d, 0x713, 0x718, 0x710, 0x72c, 0x721, 0x710, 0x720, 0x710, 0x713, 0x718,
0x710, 0x71d, 0x710, 0x729, 0x718, 0x71d, 0x720, 0x713, 0x718, 0x71d, 0x710, 0x722,
0x710, 0x717, 0x720, 0x71d, 0x726, 0x710, 0x71f, 0x723, 0x717, 0x710, 0x712, 0x710,
0x722, 0x710, 0x717, 0x71d, 0x72a, 0x721, 0x718, 0x723, 0x71d, 0x710, 0x710, 0x722,
0x715, 0x71d, 0x710, 0x722, 0x710, 0x726, 0x718, 0x720, 0x71d, 0x723, 0x722, 0x718,
0x71f, 0x723, 0x60c, 0x20, 0x710, 0x722, 0x715, 0x71d, 0x710, 0x722, 0x710, 0x721,
0x72a, 0x71d, 0x722, 0x713, 0x718, 0x60c, 0x20, 0x710, 0x722, 0x715, 0x71d, 0x710,
0x722, 0x710, 0x726, 0x71d, 0x72c, 0x71d, 0x72a, 0x723, 0x712, 0x71d, 0x72a, 0x713,
0x60c, 0x20, 0x710, 0x722, 0x715, 0x71d, 0x710, 0x722, 0x710, 0x721, 0x722, 0x715,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x72c, 0x71d, 0x720, 0x60c, 0x20, 0x710, 0x722,
0x715, 0x71d, 0x710, 0x722, 0x710, 0x712, 0x71d, 0x712, 0x710, 0x71d, 0x60c, 0x20,
0x710, 0x722, 0x715, 0x71d, 0x710, 0x722, 0x710, 0x712, 0x71d, 0x722, 0x723, 0x71d,
0x722, 0x71d, 0x723, 0x60c, 0x20, 0x710, 0x722, 0x715, 0x71d, 0x710, 0x722, 0x710,
0x718, 0x71d, 0x722, 0x710, 0x721, 0x710, 0x71f, 0x60c, 0x20, 0x710, 0x722, 0x715,
0x71d, 0x710, 0x722, 0x710, 0x710, 0x722, 0x718, 0x712, 0x71d, 0x71f, 0x710, 0x71d,
0x729, 0x720, 0x718, 0x71d, 0x72c, 0x713, 0x721, 0x710, 0x71d, 0x71f, 0x710, 0x713,
0x718, 0x722, 0x718, 0x720, 0x718, 0x71d, 0x723, 0x712, 0x71d, 0x720, 0x721, 0x718,
0x722, 0x72c, 0x710, 0x71f, 0x71d, 0x720, 0x718, 0x60c, 0x20, 0x71f, 0x71d, 0x722,
0x72c, 0x710, 0x71f, 0x71d, 0x71f, 0x72a, 0x720, 0x71d, 0x722, 0x713, 0x729, 0x720,
0x710, 0x20, 0x726, 0x710, 0x719, 0x720, 0x71d, 0x721, 0x710, 0x720, 0x718, 0x723,
0x20, 0x710, 0x722, 0x713, 0x720, 0x718, 0x723, 0x72a, 0x718, 0x712, 0x742, 0x725,
0x710, 0x20, 0x715, 0x72b, 0x720, 0x71d, 0x71b, 0x710, 0x20, 0x72c, 0x71a, 0x72c,
0x71d, 0x710, 0x721, 0x710, 0x723, 0x71d, 0x710, 0x718, 0x721, 0x710, 0x722, 0x710,
0x713, 0x718, 0x710, 0x721, 0x710, 0x722, 0x710, 0x718, 0x723, 0x721, 0x72a, 0x71d,
0x713, 0x718, 0x72c, 0x721, 0x710, 0x72a, 0x72c, 0x71d, 0x722, 0x71d, 0x729, 0x721,
0x710, 0x72c, 0x710, 0x721, 0x718, 0x72a, 0x718, 0x723, 0x721, 0x719, 0x71b, 0x720,
0x710, 0x722, 0x721, 0x722, 0x718, 0x721, 0x71d, 0x722, 0x71d, 0x721, 0x72a, 0x71d,
0x715, 0x710, 0x721, 0x71b, 0x720, 0x71f, 0x710, 0x72c, 0x720, 0x710, 0x721, 0x715,
0x71d, 0x722, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x71f, 0x723, 0x71d, 0x71f, 0x718,
0x721, 0x729, 0x718, 0x720, 0x718, 0x722, 0x721, 0x718, 0x722, 0x71f, 0x72c, 0x718,
0x722, 0x721, 0x718, 0x722, 0x71b, 0x72a, 0x710, 0x71d, 0x721, 0x718, 0x722, 0x72c,
0x71d, 0x712, 0x71d, 0x715, 0x71d, 0x718, 0x721, 0x718, 0x722, 0x72c, 0x723, 0x71d,
0x72a, 0x710, 0x72c, 0x722, 0x710, 0x723, 0x710, 0x718, 0x722, 0x71d, 0x718, 0x20,
0x71d, 0x718, 0x72a, 0x71f, 0x722, 0x718, 0x72a, 0x718, 0x722, 0x717, 0x710, 0x712,
0x71d, 0x718, 0x720, 0x710, 0x60c, 0x20, 0x715, 0x710, 0x71f, 0x718, 0x72c, 0x710,
0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x723, 0x71d, 0x722, 0x72c,
0x72a, 0x60c, 0x20, 0x715, 0x710, 0x71f, 0x718, 0x72c, 0x710, 0x20, 0x713, 0x72a,
0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x722, 0x71d, 0x718, 0x20, 0x723, 0x710, 0x720,
0x71d, 0x721, 0x60c, 0x200c, 0x20, 0x715, 0x710, 0x71f, 0x718, 0x72c, 0x710, 0x20,
0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x718, 0x713, 0x71d, 0x722, 0x710,
0x713, 0x710, 0x726, 0x722, 0x721, 0x710, 0x726, 0x710, 0x72a, 0x710, 0x721, 0x710,
0x72a, 0x71d, 0x712, 0x718, 0x726, 0x71d, 0x722, 0x71d, 0x71f, 0x723, 0x726, 0x718,
0x72a, 0x72c, 0x20, 0x710, 0x718, 0x20, 0x726, 0x72a, 0x71d, 0x722, 0x723, 0x726,
0x718, 0x72a, 0x72c, 0x20, 0x715, 0x710, 0x723, 0x726, 0x722, 0x71d, 0x710, 0x726,
0x718, 0x72a, 0x72c, 0x718, 0x20, 0x712, 0x71d, 0x720, 0x717, 0x718, 0x726, 0x718,
0x72a, 0x72c, 0x718, 0x20, 0x72a, 0x71d, 0x71f, 0x718, 0x726, 0x718, 0x722, 0x72c,
0x710, 0x20, 0x725, 0x72a, 0x71d, 0x722, 0x723, 0x72a, 0x710, 0x722, 0x71f, 0x722,
0x20, 0x710, 0x722, 0x720, 0x71d, 0x72c, 0x72a, 0x71d, 0x723, 0x71d, 0x726, 0x71d,
0x72a, 0x71d, 0x713, 0x71d, 0x722, 0x710, 0x72a, 0x71d, 0x723, 0x718, 0x720, 0x718,
0x72c, 0x72a, 0x71d, 0x718, 0x20, 0x712, 0x72a, 0x722, 0x71f, 0x718, 0x723, 0x710,
0x722, 0x72c, 0x710, 0x72a, 0x721, 0x723, 0x710, 0x722, 0x72c, 0x71d, 0x710, 0x713,
0x718, 0x723, 0x722, 0x72c, 0x718, 0x20, 0x715, 0x718, 0x721, 0x71d, 0x722, 0x713,
0x718, 0x723, 0x710, 0x718, 0x20, 0x726, 0x710, 0x718, 0x720, 0x718, 0x710, 0x71d,
0x71b, 0x718, 0x729, 0x718, 0x72a, 0x721, 0x71d, 0x72c, 0x723, 0x71d, 0x71b, 0x71f,
0x710, 0x721, 0x72a, 0x20, 0x712, 0x72a, 0x20, 0x72c, 0x718, 0x720, 0x721, 0x71d,
0x721, 0x72a, 0x20, 0x71d, 0x718, 0x71a, 0x722, 0x722, 0x723, 0x710, 0x722, 0x72c,
0x20, 0x71f, 0x72c, 0x723, 0x721, 0x72a, 0x72c, 0x71d, 0x20, 0x720, 0x718, 0x72b,
0x710, 0x721, 0x72a, 0x20, 0x72c, 0x710, 0x718, 0x721, 0x710, 0x721, 0x72a, 0x20,
0x712, 0x722, 0x723, 0x722, 0x72c, 0x722, 0x715, 0x718, 0x72a, 0x72c, 0x710, 0x72c,
0x71d, 0x713, 0x718, 0x723, 0x71d, 0x713, 0x710, 0x720, 0x726, 0x710, 0x72c, 0x71d,
0x710, 0x718, 0x710, 0x722, 0x710, 0x72c, 0x718, 0x72a, 0x718, 0x722, 0x72c, 0x718,
0x72c, 0x718, 0x72a, 0x72c, 0x718, 0x720, 0x710, 0x712, 0x722, 0x71f, 0x718, 0x712,
0x71d, 0x72a, 0x723, 0x718, 0x723, 0x72c, 0x710, 0x20, 0x71a, 0x718, 0x72a, 0x72c,
0x710, 0x718, 0x71d, 0x722, 0x71d, 0x726, 0x713, 0x71d, 0x729, 0x718, 0x72c, 0x710,
0x72c, 0x71f, 0x710, 0x71d, 0x723, 0x71d, 0x715, 0x712, 0x71d, 0x723, 0x715, 0x718,
0x721, 0x718, 0x722, 0x72c, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x712, 0x71d, 0x720,
0x721, 0x710, 0x71f, 0x710, 0x718, 0x72a, 0x71d, 0x721, 0x710, 0x718, 0x723, 0x718,
0x722, 0x721, 0x71f, 0x721, 0x718, 0x72a, 0x715, 0x718, 0x726, 0x710, 0x720, 0x721,
0x71d, 0x72a, 0x72a, 0x718, 0x72c, 0x71d, 0x72a, 0x710, 0x723, 0x71d, 0x718, 0x710,
0x72c, 0x72a, 0x718, 0x720, 0x712, 0x718, 0x723, 0x72c, 0x718, 0x71f, 0x720, 0x718,
0x722, 0x713, 0x71d, 0x725, 0x72a, 0x712, 0x71d, 0x71d, 0x722, 0x725, 0x715, 0x722,
0x710, 0x720, 0x721, 0x710, 0x72c, 0x71d, 0x725, 0x721, 0x710, 0x722, 0x710, 0x722,
0x710, 0x715, 0x71d, 0x72a, 0x710, 0x71f, 0x72c, 0x710, 0x718, 0x710, 0x71f, 0x72c,
0x718, 0x712, 0x725, 0x72b, 0x729, 0x710, 0x712, 0x710, 0x715, 0x710, 0x72c, 0x71d,
0x72a, 0x718, 0x712, 0x713, 0x715, 0x715, 0x712, 0x71a, 0x72a, 0x71d, 0x722, 0x712,
0x710, 0x722, 0x71f, 0x718, 0x71f, 0x712, 0x710, 0x72a, 0x722, 0x710, 0x718, 0x720,
0x712, 0x71d, 0x72a, 0x718, 0x72c, 0x712, 0x71d, 0x72b, 0x71f, 0x71d, 0x71f, 0x712,
0x72a, 0x718, 0x722, 0x710, 0x71d, 0x72c, 0x72b, 0x71d, 0x72c, 0x710, 0x71f, 0x718,
0x71d, 0x712, 0x720, 0x723, 0x722, 0x71f, 0x718, 0x720, 0x718, 0x721, 0x712, 0x718,
0x715, 0x72a, 0x721, 0x723, 0x718, 0x729, 0x715, 0x71f, 0x710, 0x715, 0x71d, 0x720,
0x71d, 0x715, 0x718, 0x712, 0x710, 0x71d, 0x715, 0x718, 0x72b, 0x710, 0x722, 0x712,
0x71d, 0x726, 0x710, 0x721, 0x710, 0x713, 0x718, 0x723, 0x72c, 0x710, 0x725, 0x719,
0x710, 0x71a, 0x712, 0x72a, 0x718, 0x722, 0x721, 0x715, 0x71d, 0x722, 0x72c, 0x710,
0x20, 0x715, 0x717, 0x718, 0x20, 0x71f, 0x71d, 0x20, 0x721, 0x71d, 0x722, 0x717,
0x718, 0x722, 0x713, 0x20, 0x71f, 0x718, 0x722, 0x713, 0x717, 0x718, 0x712, 0x715,
0x710, 0x71d, 0x72a, 0x71f, 0x718, 0x72c, 0x723, 0x71f, 0x713, 0x710, 0x71f, 0x710,
0x72a, 0x72c, 0x710, 0x713, 0x710, 0x71d, 0x726, 0x718, 0x72a, 0x710, 0x710, 0x718,
0x72a, 0x72b, 0x720, 0x721, 0x71f, 0x710, 0x712, 0x718, 0x720, 0x71f, 0x710, 0x721,
0x72c, 0x72b, 0x710, 0x72c, 0x71f, 0x710, 0x71f, 0x72a, 0x710, 0x71f, 0x71d, 0x71f,
0x710, 0x72c, 0x721, 0x710, 0x722, 0x715, 0x718, 0x71a, 0x710, 0x722, 0x715, 0x71d,
0x713, 0x710, 0x71f, 0x718, 0x720, 0x71f, 0x72c, 0x710, 0x71f, 0x72a, 0x710, 0x723,
0x722, 0x718, 0x71d, 0x72a, 0x723, 0x71f, 0x71f, 0x718, 0x710, 0x720, 0x710, 0x20,
0x720, 0x718, 0x721, 0x726, 0x718, 0x72a, 0x71f, 0x718, 0x72b, 0x71d, 0x722, 0x713,
0x71f, 0x718, 0x71d, 0x72c, 0x721, 0x710, 0x713, 0x710, 0x715, 0x710, 0x722, 0x721,
0x710, 0x71f, 0x710, 0x723, 0x710, 0x72a, 0x721, 0x710, 0x722, 0x71d, 0x720, 0x710,
0x721, 0x723, 0x729, 0x71b, 0x722, 0x71d, 0x729, 0x718, 0x723, 0x71d, 0x710, 0x722,
0x718, 0x712, 0x718, 0x71f, 0x718, 0x719, 0x722, 0x71d, 0x72c, 0x723, 0x71f, 0x722,
0x718, 0x712, 0x718, 0x723, 0x71d, 0x712, 0x72a, 0x723, 0x71f, 0x710, 0x718, 0x721,
0x723, 0x71f, 0x710, 0x718, 0x72a, 0x710, 0x720, 0x726, 0x722, 0x718, 0x721, 0x20,
0x726, 0x722, 0x726, 0x718, 0x722, 0x72c, 0x71d, 0x710, 0x722, 0x710, 0x71f, 0x726,
0x71d, 0x718, 0x722, 0x713, 0x71d, 0x722, 0x713, 0x729, 0x71b, 0x72a, 0x71f, 0x718,
0x723, 0x72c, 0x710, 0x722, 0x710, 0x71d, 0x71f, 0x71d, 0x719, 0x71d, 0x720, 0x718,
0x72a, 0x715, 0x710, 0x72a, 0x71d, 0x710, 0x715, 0x723, 0x710, 0x71a, 0x710, 0x720,
0x71d, 0x722, 0x723, 0x721, 0x72a, 0x71f, 0x722, 0x715, 0x723, 0x710, 0x718, 0x720,
0x72b, 0x722, 0x713, 0x717, 0x710, 0x71d, 0x71d, 0x723, 0x71d, 0x722, 0x713, 0x710,
0x726, 0x718, 0x72a, 0x723, 0x72a, 0x71d, 0x715, 0x722, 0x71d, 0x71f, 0x718, 0x720,
0x71d, 0x721, 0x723, 0x71f, 0x72c, 0x710, 0x71d, 0x726, 0x710, 0x71d, 0x72c, 0x72b,
0x71f, 0x71d, 0x722, 0x72c, 0x72c, 0x726, 0x720, 0x71d, 0x723, 0x72c, 0x717, 0x72a,
0x722, 0x72c, 0x71d, 0x721, 0x726, 0x718, 0x71b, 0x718, 0x71f, 0x71d, 0x718, 0x72c,
0x718, 0x721, 0x723, 0x71f, 0x710, 0x718, 0x720, 0x710, 0x722, 0x712, 0x710, 0x72c,
0x718, 0x72a, 0x710, 0x718, 0x72a, 0x718, 0x721, 0x71f, 0x71d, 0x710, 0x718, 0x723,
0x72c, 0x2d, 0x722, 0x71d, 0x72a, 0x710, 0x712, 0x71d, 0x710, 0x722, 0x72c, 0x71d,
0x710, 0x722, 0x712, 0x720, 0x710, 0x715, 0x71d, 0x712, 0x718, 0x723, 0x72c, 0x718,
0x71f, 0x71d, 0x710, 0x71f, 0x718, 0x72c, 0x723, 0x71f, 0x71d, 0x722, 0x713, 0x718,
0x722, 0x71d, 0x71f, 0x710, 0x72c, 0x71d, 0x72a, 0x722, 0x712, 0x71d, 0x72a, 0x713,
0x71d, 0x72a, 0x712, 0x710, 0x722, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x715,
0x710, 0x719, 0x718, 0x72a, 0x712, 0x71d, 0x72a, 0x721, 0x71d, 0x718, 0x715, 0x710,
0x71f, 0x710, 0x722, 0x72a, 0x71d, 0x71f, 0x710, 0x726, 0x20, 0x712, 0x71d, 0x72a,
0x715, 0x71d, 0x726, 0x710, 0x72a, 0x718, 0x721, 0x715, 0x710, 0x71d, 0x72a, 0x710,
0x72a, 0x710, 0x71d, 0x71f, 0x712, 0x71d, 0x71f, 0x72c, 0x721, 0x71d, 0x722, 0x20,
0x713, 0x718, 0x72a, 0x713, 0x71d, 0x710, 0x721, 0x72a, 0x72c, 0x71d, 0x20, 0x717,
0x71d, 0x720, 0x71d, 0x722, 0x710, 0x723, 0x72c, 0x710, 0x722, 0x720, 0x71d, 0x712,
0x72a, 0x71d, 0x723, 0x712, 0x710, 0x722, 0x712, 0x72a, 0x718, 0x71f, 0x71d, 0x722,
0x20, 0x717, 0x71d, 0x720, 0x715, 0x72a, 0x718, 0x71d, 0x722, 0x710, 0x718, 0x71f,
0x720, 0x710, 0x717, 0x718, 0x712, 0x72a, 0x72c, 0x720, 0x71d, 0x722, 0x715, 0x721,
0x710, 0x722, 0x720, 0x718, 0x72a, 0x715, 0x20, 0x717, 0x710, 0x718, 0x721, 0x71d,
0x720, 0x712, 0x718, 0x72a, 0x722, 0x726, 0x71d, 0x72a, 0x72c, 0x723, 0x71d, 0x715,
0x722, 0x71d, 0x710, 0x721, 0x723, 0x72c, 0x72a, 0x715, 0x721, 0x710, 0x722, 0x715,
0x718, 0x72a, 0x710, 0x710, 0x723, 0x72c, 0x72a, 0x710, 0x71a, 0x722, 0x710, 0x72c,
0x722, 0x718, 0x723, 0x712, 0x720, 0x713, 0x72a, 0x715, 0x712, 0x72a, 0x720, 0x71d,
0x722, 0x712, 0x72a, 0x72c, 0x71d, 0x723, 0x720, 0x712, 0x710, 0x200f, 0x712, 0x72a,
0x718, 0x71f, 0x723, 0x720, 0x712, 0x718, 0x729, 0x718, 0x72a, 0x72b, 0x71b, 0x712,
0x718, 0x715, 0x726, 0x72b, 0x71b, 0x712, 0x718, 0x723, 0x71d, 0x722, 0x713, 0x722,
0x71f, 0x71d, 0x723, 0x71d, 0x722, 0x710, 0x718, 0x71f, 0x718, 0x726, 0x722, 0x717,
0x710, 0x713, 0x722, 0x715, 0x712, 0x720, 0x722, 0x713, 0x712, 0x72a, 0x720, 0x71b,
0x72a, 0x713, 0x718, 0x72a, 0x722, 0x719, 0x71d, 0x717, 0x720, 0x723, 0x722, 0x71f,
0x71d, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x710, 0x722, 0x710,
0x723, 0x71b, 0x722, 0x712, 0x718, 0x720, 0x713, 0x71d, 0x72a, 0x719, 0x71d, 0x71f,
0x710, 0x720, 0x71d, 0x722, 0x71d, 0x722, 0x713, 0x72a, 0x710, 0x715, 0x71f, 0x71d,
0x72a, 0x718, 0x712, 0x71f, 0x71d, 0x71d, 0x712, 0x720, 0x72b, 0x712, 0x718, 0x722,
0x710, 0x720, 0x71d, 0x718, 0x712, 0x720, 0x71d, 0x710, 0x722, 0x710, 0x720, 0x718,
0x722, 0x715, 0x718, 0x722, 0x720, 0x718, 0x71f, 0x723, 0x721, 0x712, 0x718, 0x72a,
0x713, 0x721, 0x715, 0x72a, 0x71d, 0x715, 0x721, 0x71d, 0x720, 0x71b, 0x710, 0x721,
0x710, 0x72a, 0x71d, 0x717, 0x710, 0x721, 0x721, 0x71d, 0x722, 0x723, 0x71f, 0x721,
0x718, 0x722, 0x710, 0x71f, 0x718, 0x721, 0x718, 0x723, 0x71f, 0x718, 0x710, 0x718,
0x723, 0x720, 0x718, 0x726, 0x710, 0x72a, 0x71d, 0x723, 0x726, 0x718, 0x715, 0x713,
0x718, 0x72a, 0x71d, 0x71f, 0x710, 0x726, 0x72a, 0x710, 0x713, 0x72a, 0x71d, 0x713,
0x710, 0x72a, 0x718, 0x721, 0x71d, 0x723, 0x722, 0x20, 0x721, 0x72a, 0x71d, 0x722,
0x718, 0x723, 0x72a, 0x710, 0x71d, 0x71d, 0x712, 0x718, 0x723, 0x72a, 0x710, 0x72c,
0x718, 0x712, 0x723, 0x721, 0x726, 0x72a, 0x718, 0x726, 0x720, 0x723, 0x729, 0x718,
0x726, 0x71d, 0x710, 0x723, 0x718, 0x726, 0x71d, 0x710, 0x723, 0x72c, 0x718, 0x71f,
0x717, 0x718, 0x720, 0x721, 0x72c, 0x710, 0x720, 0x71d, 0x722, 0x72c, 0x71d, 0x72a,
0x710, 0x722, 0x710, 0x718, 0x720, 0x71d, 0x722, 0x718, 0x712, 0x723, 0x71f, 0x712,
0x715, 0x718, 0x719, 0x718, 0x710, 0x72c, 0x71d, 0x729, 0x710, 0x722, 0x712, 0x71d,
0x71d, 0x722, 0x710, 0x712, 0x720, 0x722, 0x718, 0x723, 0x712, 0x718, 0x720, 0x713,
0x718, 0x713, 0x72a, 0x710, 0x715, 0x718, 0x710, 0x72a, 0x723, 0x718, 0x719, 0x710,
0x713, 0x72a, 0x712, 0x719, 0x71d, 0x718, 0x72a, 0x71a, 0x710, 0x722, 0x72c, 0x710,
0x722, 0x710, 0x722, 0x72a, 0x71d, 0x712, 0x718, 0x72c, 0x72b, 0x713, 0x718, 0x723,
0x71f, 0x72a, 0x71d, 0x723, 0x72c, 0x721, 0x723, 0x71f, 0x718, 0x71f, 0x718, 0x723,
0x71f, 0x718, 0x721, 0x718, 0x72a, 0x718, 0x71f, 0x71d, 0x72a, 0x713, 0x718, 0x720,
0x71d, 0x722, 0x721, 0x710, 0x717, 0x71d, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20,
0x721, 0x710, 0x720, 0x715, 0x71d, 0x712, 0x71d, 0x72c, 0x710, 0x721, 0x718, 0x72a,
0x71d, 0x72b, 0x718, 0x723, 0x721, 0x710, 0x71d, 0x718, 0x72c, 0x72a, 0x71d, 0x718,
0x722, 0x71d, 0x718, 0x722, 0x710, 0x726, 0x71d, 0x710, 0x710, 0x71f, 0x720, 0x710,
0x722, 0x715, 0x712, 0x718, 0x713, 0x710, 0x71d, 0x722, 0x712, 0x71d, 0x720, 0x72c,
0x72b, 0x710, 0x72c, 0x721, 0x72c, 0x72b, 0x718, 0x71f, 0x726, 0x728, 0x71a, 0x710,
0x71d, 0x726, 0x710, 0x72c, 0x726, 0x710, 0x71f, 0x710, 0x718, 0x726, 0x718, 0x726,
0x71d, 0x713, 0x71d, 0x726, 0x718, 0x722, 0x710, 0x726, 0x718, 0x72c, 0x71d, 0x713,
0x710, 0x720, 0x710, 0x726, 0x713, 0x718, 0x723, 0x713, 0x721, 0x712, 0x71d, 0x72a,
0x713, 0x718, 0x710, 0x715, 0x710, 0x720, 0x71f, 0x710, 0x722, 0x710, 0x720, 0x71f,
0x710, 0x722, 0x72c, 0x718, 0x722, 0x71f, 0x71d, 0x72a, 0x71d, 0x721, 0x710, 0x72c,
0x71d, 0x71f, 0x718, 0x723, 0x72a, 0x710, 0x71d, 0x71f, 0x718, 0x710, 0x713, 0x710,
0x720, 0x71d, 0x722, 0x721, 0x710, 0x713, 0x718, 0x72a, 0x718, 0x721, 0x710, 0x72a,
0x71f, 0x710, 0x718, 0x723, 0x710, 0x723, 0x721, 0x71d, 0x715, 0x718, 0x710, 0x71d,
0x722, 0x710, 0x718, 0x72a, 0x718, 0x722, 0x71d, 0x718, 0x71d, 0x722, 0x718, 0x72a,
0x726, 0x720, 0x71f, 0x722, 0x718, 0x721, 0x71d, 0x710, 0x726, 0x710, 0x713, 0x718,
0x20, 0x726, 0x710, 0x713, 0x718, 0x726, 0x720, 0x710, 0x718, 0x726, 0x71d, 0x72c,
0x71f, 0x710, 0x71d, 0x72a, 0x722, 0x726, 0x718, 0x717, 0x722, 0x726, 0x710, 0x71d,
0x726, 0x718, 0x72a, 0x72c, 0x20, 0x721, 0x718, 0x72a, 0x71d, 0x723, 0x712, 0x710,
0x71d, 0x72a, 0x710, 0x72a, 0x718, 0x72c, 0x718, 0x722, 0x713, 0x710, 0x723, 0x710,
0x71d, 0x726, 0x710, 0x722, 0x72c, 0x710, 0x717, 0x71d, 0x72c, 0x71d, 0x72c, 0x710,
0x72a, 0x710, 0x718, 0x710, 0x72c, 0x718, 0x722, 0x713, 0x710, 0x72c, 0x710, 0x726,
0x718, 0x718, 0x710, 0x71d, 0x71f, 0x718, 0x71d, 0x720, 0x71d, 0x723, 0xb85, 0xbaa,
0xbbf, 0xb9f, 0xbcd, 0xb9c, 0xbbe, 0xba9, 0xbcd, 0xb85, 0xb95, 0xbcd, 0xbb0, 0xbbe,
0xb85, 0xb9f, 0xbbf, 0xbb8, 0xbcd, 0x20, 0xb85, 0xbaa, 0xbbe, 0xbaa, 0xbbe, 0xb85,
0xbb2, 0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb85, 0xbb8, 0xbcd,
0xbae, 0xbbe, 0xbb0, 0xbbe, 0xbaa, 0xbae, 0xbbe, 0xb95, 0xbcb, 0xbaa, 0xbbe, 0xb99,
0xbcd, 0xb95, 0xbc1, 0xbaf, 0xbc0, 0xbaa, 0xb9e, 0xbcd, 0xb9a, 0xbc1, 0xbb2, 0xbcd,
0xbaa, 0xbbf, 0xbb8, 0xbbe, 0xbb5, 0xbcd, 0xbaa, 0xbbf, 0xbb3, 0xbbe, 0xba9, 0xbcd,
0xb9f, 0xbc8, 0xbaf, 0xbb0, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbe, 0xbb8, 0xbbe, 0xbb5,
0xbbf, 0xbb2, 0xbcd, 0xbb2, 0xbbf, 0xbaa, 0xbc1, 0xb9c, 0xbc1, 0xbae, 0xbcd, 0xbaa,
0xbc1, 0xbb0, 0xbbe, 0xb95, 0xbc6, 0xbaf, 0xbcd, 0xbb0, 0xbcb, 0xb95, 0xbbe, 0xbb8,
0xbbe, 0xbaa, 0xbbf, 0xbb3, 0xbbe, 0xba9, 0xbcd, 0xb95, 0xbbe, 0xb9a, 0xbbf, 0xbaf,
0xbc2, 0xb9f, 0xbcd, 0xb9f, 0xbbe, 0xb95, 0xbcb, 0xba9, 0xb95, 0xbcd, 0xbb0, 0xbc7,
0xb9f, 0xbbe, 0xb95, 0xbb0, 0xbcd, 0xba4, 0xbbe, 0xbb0, 0xbcd, 0x20, 0xb8e, 0xbb8,
0xbcd, 0x20, 0xb9a, 0xbb2, 0xbbe, 0xbae, 0xbcd, 0xb9c, 0xbbf, 0xbaa, 0xbcc, 0xb9f,
0xbcd, 0xb9f, 0xbbf, 0xba4, 0xbb5, 0xbc1, 0xbb2, 0xbbe, 0xb8e, 0xbb2, 0xbcd, 0x20,
0xb86, 0xbaf, 0xbc1, 0xba9, 0xbcd, 0xb83, 0xbaa, 0xbcd, 0xbb0, 0xbc0, 0xb9f, 0xbb5,
0xbc1, 0xba9, 0xbcd, 0xb95, 0xbaa, 0xbcb, 0xbb0, 0xbcb, 0xba9, 0xbcd, 0xbb9, 0xbb0,
0xbbe, 0xbb0, 0xbc7, 0xb9c, 0xbcb, 0xbb9, 0xba9, 0xbcd, 0xba9, 0xbb8, 0xbcd, 0xbaa,
0xbc6, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xb9c, 0xbc1, 0xbaa, 0xbbe, 0xb95, 0xbae, 0xbcd,
0xbaa, 0xbbe, 0xbb2, 0xbbe, 0xb95, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcb, 0xbae, 0xbcd,
0xb95, 0xbbf, 0xb95, 0xbb2, 0xbbf, 0xb95, 0xbbf, 0xba9, 0xbcd, 0xbb7, 0xb9a, 0xbbe,
0xbb2, 0xbbe, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xbb2, 0xbbf, 0xbaa, 0xbcd, 0xbb0, 0xbc7,
0xbb5, 0xbbf, 0xbb2, 0xbcd, 0xbb2, 0xbc7, 0xbb2, 0xbcb, 0xbae, 0xbcd, 0xbb2, 0xbc1,
0xbb5, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbbe, 0xbb2, 0xbc1, 0xbaa, 0xbc1, 0xbae, 0xbcd,
0xbaa, 0xbbe, 0xbb7, 0xbbf, 0xbb2, 0xbc1, 0xb9a, 0xbbe, 0xb95, 0xbbe, 0xbae, 0xbbe,
0xbb2, 0xbaa, 0xbcb, 0xbae, 0xbaa, 0xbc1, 0xb9f, 0xbcb, 0xbae, 0xb9a, 0xbc7, 0xbb0,
0xbc1, 0xbaa, 0xbaa, 0xbbe, 0xba9, 0xbcd, 0xbae, 0xbca, 0xb95, 0xbbe, 0xba4, 0xbbf,
0xbb7, 0xbc1, 0xbae, 0xbbe, 0xba9, 0xbcd, 0xbb0, 0xbcb, 0xbb5, 0xbbf, 0xbaf, 0xbbe,
0xba8, 0xbc8, 0xbb0, 0xbcb, 0xbaa, 0xbbf, 0xb9c, 0xbae, 0xbc7, 0xba9, 0xbbe, 0xba8,
0xbbf, 0xbaf, 0xbbe, 0xbae, 0xbc7, 0xba8, 0xbcb, 0xbb5, 0xbbe, 0xb95, 0xbcd, 0xb9a,
0xbcb, 0xb9f, 0xbcd, 0xb85, 0xbb5, 0xbc1, 0xb95, 0xb9f, 0xbb5, 0xbcd, 0xb95, 0xbc1,
0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0xb9f, 0xbcb, 0x2d, 0xba8, 0xbcb, 0xbb5,
0xbcb, 0xb9a, 0xbbe, 0xbb5, 0xbcb, 0x20, 0xb9f, 0xbcb, 0xbae, 0xbc7, 0xba4, 0xbcd,
0xbb0, 0xbbf, 0xbaa, 0xbcb, 0xbb2, 0xbbf, 0xb9f, 0xbc1, 0xba9, 0xbbf, 0xbb8, 0xbcd,
0xbb5, 0xbc8, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xbb9, 0xbcb, 0xb95, 0xbcd, 0xb85, 0xb9f,
0xb95, 0xbcd, 0xb85, 0xb99, 0xbcd, 0xb95, 0xbcb, 0xbb0, 0xbc7, 0xb9c, 0xbcd, 0xb85,
0xb99, 0xbcd, 0xb95, 0xbc1, 0xbaf, 0xbc1, 0xbb2, 0xbbe, 0xb86, 0xba9, 0xbcd, 0xb9f,
0xbbf, 0xb95, 0xbc1, 0xbb5, 0xbbe, 0xb85, 0xbb0, 0xb95, 0xbc1, 0xbb5, 0xbbe, 0xbaf,
0xbcd, 0xba9, 0xbbe, 0xbaa, 0xbcd, 0xbaf, 0xbc2, 0xba9, 0xbcb, 0xbb8, 0xbcd, 0x20,
0xb8f, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xb9f, 0xbae, 0xbbe, 0xbb0, 0xbcd, 0xb95,
0xbbe, 0xb95, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcb, 0xbaa, 0xbbe, 0xb9c, 0xbc2, 0xb9c,
0xbc1, 0xbaf, 0xbcd, 0xbb2, 0xbbe, 0x20, 0xbb0, 0xbbf, 0xbaf, 0xbcb, 0xb9c, 0xbbe,
0xbae, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbcb, 0xbb8, 0xbbe, 0xbb0, 0xbbf, 0xbaf, 0xbcb,
0x20, 0xb95, 0xbc7, 0xbb2, 0xbc6, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xb9a, 0xbbe, 0xbb2,
0xbcd, 0xb9f, 0xbbe, 0xb9a, 0xbbe, 0xba9, 0xbcd, 0x20, 0xb9c, 0xbc1, 0xbb5, 0xbbe,
0xba9, 0xbcd, 0xb9a, 0xbbe, 0xba9, 0xbcd, 0x20, 0xbb2, 0xbc2, 0xbaf, 0xbbf, 0xbb8,
0xbcd, 0xb9f, 0xbc1, 0xb95, 0xbc1, 0xbae, 0xba9, 0xbcd, 0xb89, 0xbb7, 0xbc1, 0xbb5,
0xbbe, 0xbaf, 0xbbe, 0xb85, 0xbb0, 0xbc2, 0xbaa, 0xbbe, 0xb85, 0xbb8, 0xba9, 0xbcd,
0xb9a, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0xb85, 0xb9f, 0xbbf, 0xb95, 0xbcb, 0xb95, 0xba9,
0xbcd, 0xbaa, 0xbbe, 0xbb9, 0xbbf, 0xbaf, 0xbbe, 0xbaa, 0xbb9, 0xbbf, 0xbaf, 0xbbe,
0x20, 0xbaa, 0xba8, 0xbcd, 0xba4, 0xbc7, 0xbb0, 0xbbe, 0xbb8, 0xbcd, 0xbaa, 0xbbe,
0xbb0, 0xbcd, 0xbaa, 0xb9f, 0xbbe, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xbb2, 0xbc6, 0xbae,
0xbcd, 0xbaa, 0xbc6, 0xbb2, 0xbbf, 0xbb8, 0xbcd, 0xbaa, 0xbcd, 0xbb2, 0xbbe, 0xb99,
0xbcd, 0xb95, 0xbcd, 0x2d, 0xb9a, 0xbaa, 0xbcd, 0xbb2, 0xbbe, 0xba9, 0xbcd, 0xbaa,
0xbcb, 0xbb5, 0xbbe, 0x20, 0xbb5, 0xbbf, 0xbb8, 0xbcd, 0xb9f, 0xbbe, 0xbaa, 0xbcb,
0xb95, 0xbcb, 0xb9f, 0xbbe, 0xbaa, 0xbcb, 0xbaf, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbc7,
0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb9f, 0xbcd, 0xb9c, 0xbcd, 0x20, 0xbb5,
0xbb3, 0xbc8, 0xb95, 0xbc1, 0xb9f, 0xbbe, 0xb95, 0xbc7, 0xbae, 0xbcd, 0xbaa, 0xbcb,
0x20, 0xb95, 0xbbf, 0xbb0, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbc7, 0xb95, 0xba9, 0xbcd,
0xb95, 0xbc1, 0xba9, 0xbcd, 0xb95, 0xbb0, 0xb95, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbc6,
0xbaf, 0xbcd, 0xba9, 0xbcd, 0xb95, 0xbc7, 0xbae, 0xba9, 0xbcd, 0xb9a, 0xbbf, 0xb95,
0xbbe, 0xb95, 0xbcb, 0xb9a, 0xbc1, 0xbb5, 0xbbe, 0xbb5, 0xbbe, 0xb9a, 0xbbf, 0xbaf,
0xbc1, 0xb9f, 0xbbe, 0xb9f, 0xbcd, 0x20, 0xbb5, 0xbbe, 0xbb0, 0xbb8, 0xbcd, 0xb95,
0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbbe, 0x20, 0xbb0, 0xbbf, 0xb95, 0xbbe, 0xb95, 0xbcd,
0xbb0, 0xbc6, 0xbb8, 0xbcd, 0xb9f, 0xbbe, 0xba9, 0xbcd, 0xb95, 0xbc1, 0xbaf, 0xbbe,
0xbaa, 0xbc7, 0xb95, 0xbcd, 0xbaf, 0xbc2, 0xbb0, 0xbbe, 0xb95, 0xbcb, 0xb9f, 0xbc6,
0xba9, 0xbcd, 0xbae, 0xbbe, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbb7, 0xbbe, 0xbb5, 0xbcd,
0xba9, 0xbcd, 0xb9f, 0xbbe, 0xbb5, 0xbcd, 0xb9a, 0xba9, 0xbcd, 0xb9f, 0xbbe, 0xbb5,
0xbcd, 0xb9a, 0xba9, 0xbcd, 0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xb95, 0xbcd, 0xb9f,
0xbc6, 0xba9, 0xbcd, 0xbb5, 0xbb0, 0xbcd, 0xb9f, 0xbc6, 0xb9f, 0xbcd, 0xbb0, 0xbbe,
0xbaf, 0xbcd, 0xb9f, 0xbcd, 0xb9f, 0xbca, 0xbae, 0xbbf, 0xba9, 0xbbf, 0xb95, 0xbbe,
0xb8e, 0xb9f, 0xbcd, 0xbae, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbbe, 0xba9, 0xbcd, 0xb88,
0xbb0, 0xbc1, 0xba9, 0xbc6, 0xbaa, 0xbc7, 0xb8e, 0xbb2, 0xbcd, 0x20, 0xb9a, 0xbbe,
0xbb2, 0xbcd, 0xbb5, 0xb9f, 0xbcb, 0xbb0, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xbb0, 0xbcd,
0xb9f, 0xbcd, 0x20, 0xba8, 0xbc6, 0xbb2, 0xbcd, 0xb9a, 0xba9, 0xbcd, 0xb83, 0xbaa,
0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbbe, 0xbb2, 0xbc6, 0xb9a, 0xbbe, 0xb95, 0xbbf, 0xbb2,
0xbc7, 0xbb8, 0xbcd, 0x20, 0xbb5, 0xbb3, 0xbc8, 0xb95, 0xbc1, 0xb9f, 0xbbe, 0xb95,
0xbc2, 0xbb8, 0xbcd, 0x20, 0xbaa, 0xbc7, 0xb95, 0xbbf, 0xbb0, 0xbbe, 0xba3, 0xbcd,
0xb9f, 0xbcd, 0x20, 0xb9f, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xb95, 0xbbf, 0xbb0, 0xba9,
0xb9f, 0xbbe, 0xb95, 0xbb5, 0xbc1, 0xb9f, 0xbc7, 0xbb2, 0xbc2, 0xbaa, 0xbcd, 0xb95,
0xbb5, 0xbc1, 0xba4, 0xbae, 0xbbe, 0xbb2, 0xbbe, 0xb95, 0xbc1, 0xbb5, 0xbbe, 0xbaf,
0xb95, 0xbbf, 0xbb2, 0xbcd, 0xb95, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xbb9, 0xbb2, 0xbbf,
0xb83, 0xbaa, 0xbc7, 0xb95, 0xbcd, 0xbb8, 0xbcd, 0xbb9, 0xbb5, 0xbbe, 0xba9, 0xbbe,
0xbb9, 0xbc6, 0xbb0, 0xbcd, 0xbae, 0xbcb, 0xb9a, 0xbbf, 0xbb2, 0xbcb, 0xb87, 0xba3,
0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xbaa, 0xbca, 0xbb2, 0xbbf, 0xbb8,
0xbcd, 0xba8, 0xbbe, 0xb95, 0xbcd, 0xbb8, 0xbcd, 0x2c, 0x20, 0xb87, 0xba3, 0xbcd,
0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xbae, 0xbb0, 0xbc6, 0xba9, 0xbcd, 0xb95,
0xbcb, 0x2c, 0x20, 0xb87, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe,
0xbaa, 0xbc0, 0xb9f, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xbb0,
0xbcd, 0xb95, 0xbcd, 0x2c, 0x20, 0xb87, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe,
0xba9, 0xbbe, 0xb9f, 0xbc6, 0xbb2, 0xbcd, 0x20, 0xb9a, 0xbbf, 0xb9f, 0xbcd, 0xb9f,
0xbbf, 0x2c, 0x20, 0xb87, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe,
0xbb5, 0xbc7, 0xbb5, 0xbc7, 0x2c, 0x20, 0xb87, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf,
0xbbe, 0xba9, 0xbbe, 0xbb5, 0xbbf, 0xba9, 0xbcd, 0xb9a, 0xbc6, 0xba9, 0xbc7, 0xbb8,
0xbcd, 0x2c, 0x20, 0xb87, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe,
0xbb5, 0xbbf, 0xba9, 0xbbe, 0xbae, 0xbc7, 0xb95, 0xbcd, 0x2c, 0x20, 0xb87, 0xba3,
0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xb87, 0xba9, 0xbc1, 0xbb5, 0xbbf,
0xb95, 0xbcd, 0xb87, 0xb95, 0xbbe, 0xbb2, 0xbc2, 0xbaf, 0xbbf, 0xba4, 0xbcd, 0xb9c,
0xbae, 0xbc8, 0xb95, 0xbcd, 0xb95, 0xbbe, 0xb9c, 0xbc1, 0xba9, 0xbbf, 0xbaf, 0xbc2,
0xbb2, 0xbc2, 0xbaf, 0xbbf, 0xbb8, 0xbcd, 0xbb5, 0xbbf, 0xbb2, 0xbcd, 0xbb2, 0xbc7,
0xbae, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xb9a, 0xbc6, 0xbb2, 0xbcd, 0xbb2, 0xbcb,
0x2c, 0x20, 0xb95, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xb95, 0xbbf, 0xb95, 0xbbf, 0xbb0,
0xbc6, 0xbb2, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xb9c, 0xbcd, 0xbb2, 0xbbe, 0x20, 0xbaa,
0xbbe, 0xbb8, 0xbcd, 0xbb2, 0xbbf, 0xbae, 0xbbe, 0xbb2, 0xbbe, 0xbb8, 0xbcd, 0x20,
0xb8f, 0xb9e, 0xbcd, 0xb9a, 0xbb2, 0xbcd, 0xbb8, 0xbcd, 0xbb2, 0xbcb, 0xbaf, 0xbb0,
0xbcd, 0x20, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xba9, 0xbcd, 0xbb8, 0xbb8, 0xbcd, 0x20,
0xb95, 0xbc1, 0xbb5, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0xbae,
0xbc7, 0xb9a, 0xbbf, 0xbaf, 0xbcb, 0xbae, 0xbbe, 0xba9, 0xbbe, 0xb95, 0xbc1, 0xbb5,
0xbbe, 0xbae, 0xba9, 0xbbe, 0xbb8, 0xbcd, 0xbae, 0xbbe, 0xbb0, 0xbbf, 0xb95, 0xbbe,
0xb9f, 0xbcd, 0xbae, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbf,
0xb95, 0xbcd, 0xbae, 0xb9f, 0xbae, 0xbcb, 0xbb0, 0xbbe, 0xbb8, 0xbcd, 0xbae, 0xbb8,
0xb9f, 0xbcd, 0xbb2, 0xba9, 0xbcd, 0xbae, 0xbc6, 0xba9, 0xbcb, 0xbae, 0xbbf, 0xba9,
0xbbf, 0xbae, 0xbc6, 0xbb0, 0xbbf, 0xb9f, 0xbbe, 0xbae, 0xbc6, 0xb9f, 0xbcd, 0xbb2,
0xb95, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xbae, 0xbc6, 0xb95, 0xbcd, 0xbb8, 0xbbf, 0xb95,
0xbcb, 0x20, 0xba8, 0xb95, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbf, 0xb95, 0xbcd, 0xbaf,
0xbc2, 0xbb2, 0xba9, 0xbcd, 0xbae, 0xbbe, 0xb99, 0xbcd, 0xb9f, 0xbbe, 0xba9, 0xbcd,
0xbae, 0xbbe, 0xba9, 0xbcd, 0xb9f, 0xbc6, 0xbb0, 0xbcd, 0xbb0, 0xbc7, 0xbae, 0xbbe,
0xba9, 0xbcd, 0xb9f, 0xbc7, 0xbb5, 0xbc0, 0xb9f, 0xbbf, 0xbaf, 0xbcb, 0xbae, 0xbbe,
0xba9, 0xbcd, 0xb9a, 0xbc6, 0xbb0, 0xbc7, 0xb9f, 0xbcd, 0xba8, 0xb9a, 0xbb5, 0xbcd,
0xba8, 0xbbf, 0xbaf, 0xbc2, 0xbaf, 0xbbe, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xba8, 0xbcb,
0xbae, 0xbcd, 0xba8, 0xbcb, 0xbb0, 0xba9, 0xbcd, 0xbb9, 0xbbe, 0xbaa, 0xbc6, 0xbb5,
0xbc1, 0xbb2, 0xbbe, 0x2c, 0x20, 0xbb5, 0xb9f, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20,
0xb9f, 0xb95, 0xbcb, 0xb9f, 0xbcd, 0xb9f, 0xbbe, 0xb9a, 0xbc6, 0xba9, 0xbcd, 0xb9f,
0xbb0, 0xbcd, 0x2c, 0x20, 0xbb5, 0xb9f, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb9f,
0xb95, 0xbcb, 0xb9f, 0xbbe, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0x20, 0xb9a, 0xbb2, 0xbc7,
0xbae, 0xbcd, 0x2c, 0x20, 0xbb5, 0xb9f, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb9f,
0xb95, 0xbcb, 0xb9f, 0xbbe, 0xba8, 0xbc2, 0xb95, 0xbcd, 0xb92, 0xb9c, 0xbbf, 0xba9,
0xb95, 0xbbe, 0xbaa, 0xba9, 0xbbe, 0xbae, 0xbbe, 0xbaa, 0xbb0, 0xbae, 0xbb0, 0xbbf,
0xbaa, 0xbcb, 0xb83, 0xbaa, 0xbcb, 0xba9, 0xbbf, 0xb95, 0xbcd, 0xbb8, 0xbcd, 0xbaa,
0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x2d, 0xb85, 0xbb5, 0xbcd, 0x2d, 0xbaa, 0xbbf,
0xbb0, 0xbbf, 0xba9, 0xbcd, 0xbb8, 0xbcd, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcd,
0x20, 0xb86, 0xb83, 0xbaa, 0xbcd, 0x20, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xbaf, 0xbbf,
0xba9, 0xbcd, 0xbaa, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0xb9f, 0xbcb, 0x20, 0xbb5,
0xbc6, 0xbb2, 0xbcd, 0xbb9, 0xbcb, 0xbaa, 0xbbf, 0xbaf, 0xbc2, 0xbb0, 0xbcd, 0xb9f,
0xbcb, 0x20, 0xbb0, 0xbbf, 0xb95, 0xbcb, 0xbaa, 0xbc1, 0xba9, 0xbcd, 0xb9f, 0xbbe,
0x20, 0xb85, 0xbb0, 0xbc0, 0xba9, 0xbbe, 0xbb8, 0xbcd, 0xbb0, 0xbbe, 0xba9, 0xbcd,
0xb95, 0xbbf, 0xba9, 0xbcd, 0x20, 0xb87, 0xba9, 0xbcd, 0xbb2, 0xbc6, 0xb9f, 0xbcd,
0xbb0, 0xbc6, 0xbb8, 0xbbf, 0xb83, 0xbaa, 0xbbf, 0xbb0, 0xbc6, 0xb9c, 0xbbf, 0xba9,
0xbbe, 0xbb0, 0xbc6, 0xb9a, 0xbb2, 0xbc2, 0xb9f, 0xbcd, 0xbb0, 0xbbf, 0xbaf, 0xbcb,
0x20, 0xbaa, 0xbbf, 0xbb0, 0xbbe, 0xba9, 0xbcd, 0xb95, 0xbcb, 0xb9a, 0xbbe, 0xba9,
0xbcd, 0xb9f, 0xbb0, 0xbc6, 0xbae, 0xbcd, 0xb9a, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbbf,
0xbaf, 0xbbe, 0xb95, 0xbcb, 0xb9a, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbcb, 0x20, 0xb9f,
0xbcb, 0xbae, 0xbbf, 0xb99, 0xbcd, 0xb95, 0xbcb, 0xb9a, 0xbbe, 0xbb5, 0xbcb, 0x20,
0xbaa, 0xbb5, 0xbc1, 0xbb2, 0xbcb, 0xb87, 0xb9f, 0xbcb, 0xb95, 0xbcb, 0xbb0, 0xbcd,
0xb9f, 0xbcb, 0xbb0, 0xbcd, 0xbae, 0xbbf, 0xb9f, 0xbcd, 0xb9a, 0xbbf, 0xb9f, 0xbcd,
0xb95, 0xbbe, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbaa,
0xbbe, 0xbb0, 0xbcd, 0xba4, 0xbc7, 0xbb2, 0xbc6, 0xbae, 0xbbf, 0xb9a, 0xbc6, 0xbaf,
0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xb9c, 0xbbe, 0xba9, 0xbcd, 0xbb8, 0xbcd,
0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xb95, 0xbc0, 0xb9f,
0xbcd, 0xbb8, 0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20,
0xbb2, 0xbc2, 0xb9a, 0xbbf, 0xbaf, 0xbbe, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd,
0xb9f, 0xbcd, 0x20, 0xba4, 0xbbe, 0xbae, 0xbb8, 0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf,
0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb5, 0xbbf, 0xba9, 0xbcd, 0xb9a, 0xbc6, 0xba9,
0xbcd, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xbb5, 0xbbf, 0xb83, 0xbaa, 0xbcd, 0xb9f, 0xbcd,
0x20, 0xb95, 0xbb0, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xba4, 0xbc6, 0xb95, 0xbc1, 0xb9a,
0xbbf, 0xb95, 0xbb2, 0xbcd, 0xbaa, 0xbbe, 0xba4, 0xbc1, 0xbb2, 0xbc7, 0xb9f, 0xbbf,
0xb9c, 0xbc1, 0xbb5, 0xbbe, 0xba9, 0xbbe, 0xb9f, 0xbca, 0xbb0, 0xbca, 0xba9, 0xbcd,
0xb9f, 0xbcb, 0xb9f, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbcb, 0xbb2, 0xbbe, 0xbb5, 0xbbe,
0xba9, 0xbcd, 0xb95, 0xbc2, 0xbb5, 0xbb0, 0xbcd, 0xbb5, 0xbca, 0xbaf, 0xbbf, 0xb9f,
0xbcd, 0xbb9, 0xbbe, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xbb5, 0xbbf, 0xba9, 0xbcd, 0xba9,
0xbbf, 0xbaa, 0xbc6, 0xb95, 0xbcd, 0xbaf, 0xb95, 0xbc1, 0xb9f, 0xb9f, 0xbcd, 0xb95,
0xbc7, 0xbb8, 0xbbf, 0xb9f, 0xbc7, 0xbb5, 0xbbf, 0xbb8, 0xbcd, 0xb9f, 0xbc1, 0xbae,
0xbcb, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0x2d, 0xb9f, 0xbbf, 0x20, 0xb89, 0xbb0, 0xbcd,
0xbb5, 0xbbf, 0xbb2, 0xbcd, 0xbb2, 0xbc7, 0xbae, 0xbbe, 0xb95, 0xbcd, 0xb95, 0xbbf,
0xbaf, 0xbc2, 0xbb0, 0xbbf, 0xbae, 0xbbe, 0xb9a, 0xba9, 0xbcd, 0xbae, 0xbc6, 0xb95,
0xbcd, 0xbae, 0xbb0, 0xbcd, 0xb9f, 0xbcb, 0xbaa, 0xbbe, 0xbb2, 0xbcd, 0xbae, 0xbb0,
0xbcd, 0xbb0, 0xbcb, 0xba4, 0xbc7, 0xbb0, 0xbbe, 0xbb8, 0xbcd, 0xbaf, 0xbcb, 0xbb5,
0xbbe, 0xb9f, 0xbcd, 0xbb0, 0xbcb, 0xbb2, 0xbcd, 0xbb5, 0xbcb, 0xbb8, 0xbcd, 0xb9f,
0xbcb, 0xb95, 0xbcd, 0xbb2, 0xbbe, 0xb99, 0xbcd, 0xb87, 0xbaf, 0xbb0, 0xbcd, 0xbaa,
0xbbf, 0xbaf, 0xba9, 0xbcd, 0xb8f, 0xb9f, 0xba9, 0xbcd, 0xb85, 0xbb2, 0xbcd, 0xbae,
0xbbe, 0xba4, 0xbbf, 0xb85, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xba9, 0xbcd, 0xb85, 0xba9,
0xb9f, 0xbc0, 0xbb0, 0xbcd, 0xb85, 0xb95, 0xbcd, 0xba4, 0xbb5, 0xbcd, 0xb85, 0xb95,
0xbcd, 0xb9f, 0xbcb, 0xbaa, 0xbcd, 0xb85, 0xbb7, 0xbcd, 0xb95, 0xbbe, 0xbaa, 0xbbe,
0xba4, 0xbcd, 0xb85, 0xb9f, 0xbbf, 0xbb0, 0xbbe, 0xbaa, 0xbbe, 0xb95, 0xbcd, 0xba4,
0xbbe, 0xba4, 0xbcd, 0xbaa, 0xbb9, 0xbcd, 0xbb0, 0xbc8, 0xba9, 0xbcd, 0xbaa, 0xbbe,
0xb95, 0xbcd, 0xb95, 0xbc2, 0xbaa, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb95, 0xbcd,
0xbaa, 0xbbe, 0xbb0, 0xbcd, 0xba9, 0xbbe, 0xbb2, 0xbcd, 0xbaa, 0xbc6, 0xbaf, 0xbcd,
0xbb0, 0xbc2, 0xb9f, 0xbcd, 0xbaa, 0xbbf, 0xbb7, 0xbcd, 0xb95, 0xbc6, 0xb95, 0xbcd,
0xbaa, 0xbc1, 0xbb0, 0xbc1, 0xba9, 0xbc7, 0xb9a, 0xbbf, 0xb9f, 0xbbe, 0xb9a, 0xbcb,
0xbaf, 0xbcd, 0xbaa, 0xbbe, 0xbb2, 0xbcd, 0xb9a, 0xbbe, 0xba9, 0xbcd, 0xb95, 0xbca,
0xbb4, 0xbc1, 0xbae, 0xbcd, 0xbaa, 0xbc1, 0xb9f, 0xbae, 0xbbe, 0xbb8, 0xbcd, 0xb95,
0xbb8, 0xbcd, 0xb9f, 0xbbe, 0xb95, 0xbcd, 0xb95, 0xbbe, 0xb9f, 0xbbf, 0xbb2, 0xbbf,
0xba4, 0xbc1, 0xbaa, 0xbbe, 0xbaf, 0xbcd, 0xba4, 0xbc1, 0xbb7, 0xba9, 0xbcd, 0xbaa,
0xbc7, 0xb83, 0xbaa, 0xbbe, 0xbae, 0xb95, 0xbc1, 0xbb8, 0xbcd, 0xb9f, 0xbbe, 0xbb9,
0xbc6, 0xbaa, 0xbcd, 0xbb0, 0xbbe, 0xba9, 0xbcd, 0xbb9, 0xbcb, 0x20, 0xb9a, 0xbbf,
0x20, 0xbae, 0xbbf, 0xba9, 0xbcd, 0x20, 0xb9a, 0xbbf, 0xb9f, 0xbcd, 0xb9f, 0xbbf,
0xbb9, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb99, 0xbcd, 0xbb9, 0xbcb, 0xbb5, 0xbcd,
0xba4, 0xbcd, 0xb87, 0xbb0, 0xbcd, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95,
0xbcd, 0xb9c, 0xb95, 0xbbe, 0xbb0, 0xbcd, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xb9c, 0xbc6,
0xbaf, 0xbaa, 0xbc2, 0xbb0, 0xbbe, 0xb9c, 0xbc6, 0xbb0, 0xbc1, 0xb9a, 0xbb2, 0xbc7,
0xbae, 0xbcd, 0xb95, 0xbbe, 0xbaa, 0xbc2, 0xbb2, 0xbcd, 0xb95, 0xbbe, 0xbae, 0xbcd,
0xb9a, 0xb9f, 0xbcd, 0xb95, 0xbbe, 0xb95, 0xbb0, 0xbbe, 0xb9a, 0xbcd, 0xb9a, 0xbbf,
0xb95, 0xbbe, 0xba4, 0xbcd, 0xbae, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbc1, 0xb95, 0xbbe,
0xba9, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbbe, 0xb95, 0xbca, 0xbb2, 0xbcd, 0xb95, 0xba4,
0xbcd, 0xba4, 0xbbe, 0xb95, 0xbbf, 0xbb0, 0xbbe, 0xbb8, 0xbcd, 0xba9, 0xbcb, 0xbaf,
0xbbe, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcb, 0xbb2, 0xbbe, 0xbb2,
0xbae, 0xbcd, 0xbaa, 0xbc2, 0xbb0, 0xbcd, 0xb95, 0xbc1, 0xb9a, 0xbbf, 0xb99, 0xbcd,
0xb95, 0xbc1, 0xbb5, 0xbc8, 0xba4, 0xbcd, 0xbae, 0xb95, 0xbbe, 0xbb5, 0xbc1, 0xbae,
0xb95, 0xba4, 0xba9, 0xbcd, 0xbae, 0xb95, 0xbcd, 0xb95, 0xbb8, 0xbb0, 0xbcd, 0xbae,
0xba3, 0xbbf, 0xbb2, 0xbbe, 0xbae, 0xbb8, 0xbcd, 0xb95, 0xb9f, 0xbcd, 0xba8, 0xbbf,
0xb95, 0xbcb, 0xb9a, 0xbbf, 0xbaf, 0xbbe, 0xba8, 0xbcb, 0xbb5, 0xbcb, 0xb95, 0xbc1,
0xbb8, 0xbcd, 0xba8, 0xbc6, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xba8, 0xbcb,
0xbb5, 0xbcb, 0xb9a, 0xbc0, 0xbaa, 0xbbf, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd,
0xb93, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xb93, 0xbb0, 0xbb2, 0xbcd, 0xb83,
0xbaa, 0xbcd, 0xba9, 0xbcb, 0xbae, 0xbcd, 0x20, 0xbaa, 0xbc6, 0xba9, 0xbcd, 0xbaa,
0xbcb, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0xb95, 0xbcd, 0xbaa,
0xbbf, 0xbaf, 0xbbe, 0xb99, 0xbcd, 0xbaf, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xba4, 0xbcd,
0xba4, 0xbbe, 0xbb0, 0xbcd, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbbe, 0xba9, 0xbc7,
0xb95, 0xbbf, 0xbb8, 0xbbf, 0xbb2, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbbe, 0xbb0, 0xbbf,
0xbaf, 0xbbe, 0xba4, 0xbcd, 0xb9a, 0xb95, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0xb9a, 0xbae,
0xbbe, 0xbb0, 0xbcd, 0xb95, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xb9a, 0xbbf, 0xbaf, 0xbcb,
0xbb2, 0xbcd, 0xbb7, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xbaf, 0xbcd, 0xb9a, 0xbbf,
0xb99, 0xbcd, 0xb95, 0xbaa, 0xbcd, 0xbaa, 0xbc2, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xbb0,
0xbc6, 0xb9f, 0xbcd, 0xba8, 0xbbf, 0xb95, 0xbcb, 0xbb2, 0xbbf, 0xbae, 0xbcd, 0xbb8,
0xbcd, 0xb95, 0xbcd, 0xba4, 0xbbe, 0xbaf, 0xbcd, 0xbaa, 0xbc7, 0xba4, 0xbbe, 0xbb7,
0xbcd, 0xb95, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xba4, 0xbcd, 0xbaa, 0xbbf, 0xbb2, 0xbbf,
0xb9a, 0xbbf, 0xb9f, 0xbc6, 0xbb9, 0xbcd, 0xbb0, 0xbbe, 0xba9, 0xbcd, 0xba4, 0xbbf,
0xbae, 0xbcd, 0xbaa, 0xbc1, 0xb9f, 0xbcb, 0xb95, 0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbcb,
0xb9f, 0xbbe, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xb89, 0xbb2, 0xbbe, 0xba9,
0xbcd, 0xbaa, 0xbbe, 0xb9f, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0xb89, 0xbb0, 0xbc1, 0xbae,
0xbcd, 0xb95, 0xbbf, 0xb89, 0xbb8, 0xbcd, 0xb9f, 0xbcd, 0x2d, 0xba8, 0xbc7, 0xbb0,
0xbbe, 0xbb5, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0xbb5,
0xbbf, 0xbb3, 0xbbe, 0xb9f, 0xbbf, 0xbb5, 0xbca, 0xbb8, 0xbcd, 0xba4, 0xbcb, 0xb95,
0xbcd, 0xbaf, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xbb0, 0xb99,
0xbcd, 0xb95, 0xbc2, 0xba9, 0xbcd, 0xbaf, 0xbc6, 0xb95, 0xbbe, 0xb9f, 0xbbf, 0xbb0,
0xbbf, 0xba9, 0xbcd, 0xbaa, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xb8f, 0xbb0, 0xbc7, 0xbb5,
0xba9, 0xbcd, 0xb85, 0xb9a, 0xbcb, 0xbb0, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd,
0xbae, 0xbc1, 0xb9f, 0xbbe, 0xb95, 0xbc7, 0xba9, 0xbb0, 0xbbf, 0xb95, 0xbc7, 0xbaa,
0xbcd, 0x20, 0xbb5, 0xbc6, 0xbb0, 0xbcd, 0xb9f, 0xbc7, 0xb83, 0xbaa, 0xbb0, 0xbcb,
0xbae, 0xb9f, 0xbbf, 0xbb0, 0xbbe, 0xbb0, 0xbc7, 0xb95, 0xbcd, 0xb9c, 0xbbe, 0xbb5,
0xbbf, 0xb95, 0xbcd, 0xba4, 0xbc6, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb9c, 0xbbe,
0xbb0, 0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbbe, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd,
0xb9f, 0xbcd, 0x20, 0xbb9, 0xbc6, 0xbb2, 0xbc6, 0xba9, 0xbbe, 0xbb8, 0xbcd, 0xb9f,
0xbc7, 0xba9, 0xbcd, 0xbb2, 0xbbf, 0xb85, 0xb9f, 0xbbf, 0xbb2, 0xbc6, 0xbaf, 0xbcd,
0xb9f, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xbb8, 0xbcd, 0xbaa, 0xbc7, 0xba9, 0xbcd,
0xbaa, 0xbc1, 0xbb0, 0xbcb, 0xb95, 0xbcd, 0xb95, 0xba9, 0xbcd, 0x20, 0xbb9, 0xbbf,
0xbb2, 0xbcd, 0xb9f, 0xbbe, 0xbb0, 0xbcd, 0xbb5, 0xbbf, 0xba9, 0xbcd, 0xbaf, 0xbc2,
0xb95, 0xbcd, 0xbb2, 0xbbe, 0xbb9, 0xbcb, 0xbaa, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0xbb2,
0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbc6, 0xbae, 0xba9, 0xbcd, 0xbb2, 0xbbe, 0xbb0, 0xbcd,
0xb9f, 0xbcd, 0x20, 0xbb9, 0xbcb, 0xbb5, 0xbc7, 0xbae, 0xbc6, 0xbb2, 0xbcd, 0xbaa,
0xbcb, 0xbb0, 0xbcd, 0xba9, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xba4, 0xbcd, 0xb9a,
0xbbf, 0xb9f, 0xbcd, 0xba9, 0xbbf, 0xb86, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb9f, 0xbcd,
0xbb0, 0xb9f, 0xbbe, 0xbae, 0xbcd, 0xb85, 0xba3, 0xbcd, 0xb9f, 0xbcb, 0xbb0, 0xbbe,
0xb85, 0xbb8, 0xbcd, 0xb9f, 0xbcd, 0xbb0, 0xbbe, 0xb95, 0xbbe, 0xba9, 0xbcd, 0xb8f,
0xba4, 0xba9, 0xbcd, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xbb2, 0xbcd, 0xb95, 0xbbf, 0xbb0,
0xbc7, 0xb9f, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0xbaa,
0xbbf, 0xbb0, 0xb9f, 0xbbf, 0xbb8, 0xbcd, 0xbb2, 0xbbe, 0xbb5, 0xbbe, 0xbaa, 0xbc1,
0xbb0, 0xbc2, 0xb9a, 0xbb2, 0xbcd, 0xbb8, 0xbcd, 0xbaa, 0xbc1, 0xb9a, 0xbbe, 0xbb0,
0xbc6, 0xbb8, 0xbcd, 0xb9f, 0xbcd, 0xbaa, 0xbc1, 0xb9f, 0xbbe, 0xbaa, 0xbc6, 0xbb8,
0xbcd, 0xb9f, 0xbcd, 0xbaa, 0xb9a, 0xbbf, 0xb99, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd,
0xb9a, 0xbbf, 0xb9a, 0xbbf, 0xba9, 0xbb5, 0xbcd, 0xb95, 0xbcb, 0xbaa, 0xba9, 0xbcd,
0xbb9, 0xbc7, 0xb95, 0xba9, 0xbcd, 0xb9f, 0xbaa, 0xbcd, 0xbb3, 0xbbf, 0xba9, 0xbcd,
0xb9c, 0xbbf, 0xbaa, 0xbcd, 0xbb0, 0xbbe, 0xbb2, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0xb95,
0xbb0, 0xbcd, 0xba9, 0xbb8, 0xbc7, 0xbb9, 0xbc6, 0xbb2, 0xbcd, 0xb9a, 0xbbf, 0xb99,
0xbcd, 0xb95, 0xbbf, 0xb90, 0xbb2, 0xbcd, 0x20, 0xb86, 0xb83, 0xbaa, 0xbcd, 0x20,
0xbae, 0xbc7, 0xba9, 0xbcd, 0xb87, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0xbaa,
0xbc1, 0xbb2, 0xbcd, 0xb9c, 0xbc6, 0xbb0, 0xbcd, 0xb9a, 0xbbf, 0xb95, 0xbb2, 0xbbf,
0xba9, 0xbbf, 0xba9, 0xbcd, 0xb95, 0xbbf, 0xbb0, 0xbbe, 0xb9f, 0xbcd, 0xb95, 0xbbf,
0xbb0, 0xbcb, 0xbb5, 0xbcd, 0xb95, 0xbc0, 0xbb5, 0xbcd, 0xbb2, 0xbbf, 0xbb8, 0xbcd,
0xbaa, 0xba9, 0xbcd, 0xb9c, 0xbc1, 0xbaa, 0xbcd, 0xbb2, 0xbbe, 0xba9, 0xbbe, 0xbb2,
0xba3, 0xbcd, 0xb9f, 0xba9, 0xbcd, 0xbb2, 0xb95, 0xbcd, 0xb9a, 0xbae, 0xbcd, 0xbaa,
0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbae, 0xbc7, 0xb9f, 0xbcd, 0xbb0, 0xbbf, 0xb9f, 0xbcd,
0xbae, 0xbbe, 0xbb2, 0xbcd, 0xb9f, 0xbbe, 0xbae, 0xbb0, 0xbbf, 0xbb9, 0xbae, 0xbcd,
0xbae, 0xbbf, 0xba9, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xbae, 0xbca, 0xba9, 0xbbe,
0xb95, 0xbcd, 0xb95, 0xbcb, 0xbae, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbcb, 0xb93, 0xbb8,
0xbcd, 0xbb2, 0xbcb, 0xbaa, 0xbbe, 0xbb0, 0xbc0, 0xbb8, 0xbcd, 0xbaa, 0xbcb, 0xb9f,
0xbcd, 0xb95, 0xbcb, 0xbb0, 0xbbf, 0xb95, 0xbbe, 0xbaa, 0xbcd, 0xbb0, 0xbbe, 0xb95,
0xbcd, 0xbb0, 0xbcb, 0xbae, 0xbcd, 0xb9a, 0xbae, 0xbbe, 0xbb0, 0xbbe, 0xb9a, 0xbbe,
0xba9, 0xbcd, 0x20, 0xbae, 0xbb0, 0xbbf, 0xba9, 0xbcb, 0xb9a, 0xbb0, 0xbaf, 0xbc7,
0xbb5, 0xbcb, 0xb9a, 0xbb0, 0xb9f, 0xbcb, 0xbb5, 0xbcd, 0xb9a, 0xbbf, 0xbae, 0xbcd,
0xb83, 0xbaa, 0xbc6, 0xbb0, 0xbcb, 0xbaa, 0xbcb, 0xbb2, 0xbcd, 0xbb8, 0xbcd, 0xb95,
0xbcb, 0xbaa, 0xbcd, 0xb9c, 0xbc7, 0xb9a, 0xbcb, 0xb83, 0xbaa, 0xbbf, 0xbaf, 0xbbe,
0xbb8, 0xbcd, 0xb9f, 0xbbe, 0xb95, 0xbcd, 0xbb9, 0xbcb, 0xbae, 0xbcd, 0xb9f, 0xbb2,
0xbbf, 0xba9, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbbe, 0xba9, 0xbc7, 0xb89, 0xbb2, 0xbcd,
0xbaf, 0xbbe, 0xba9, 0xbcb, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0xbb5, 0xba4, 0xbc1, 0xbb8,
0xbcd, 0xbb5, 0xbbe, 0xb9f, 0xbbf, 0xb95, 0xba9, 0xbcd, 0xbb5, 0xbbf, 0xbaf, 0xba9,
0xbcd, 0xba9, 0xbbe, 0xbb5, 0xbbf, 0xbb2, 0xbcd, 0xba9, 0xbbf, 0xbaf, 0xbb8, 0xbcd,
0xbb5, 0xbcb, 0xbb2, 0xbcd, 0xb95, 0xbcb, 0xb95, 0xbbf, 0xbb0, 0xbbe, 0xb9f, 0xbcd,
0xbb5, 0xbbe, 0xbb0, 0xbcd, 0xbb8, 0xbbe, 0xbb8, 0xb95, 0xbcd, 0xbb0, 0xbc6, 0xbaa,
0xbcd, 0xb9c, 0xbc2, 0xbb0, 0xbbf, 0xb9a, 0xbcd, 0xb86, 0xba3, 0xbcd, 0xb9f, 0xba9,
0xba8, 0xbb0, 0xbbf, 0xbb5, 0xbcb, 0xb9a, 0xbbe, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xb95,
0xbbf, 0xbb1, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbc1, 0xbae, 0xbb8, 0xbcd, 0xb95, 0xbcb,
0xb95, 0xbcb, 0xbb8, 0xbcd, 0xb95, 0xbca, 0xbae, 0xbb0, 0xbcb, 0xb95, 0xbc6, 0xbb0,
0xbcd, 0xbaf, 0xbc2, 0xbb2, 0xbc6, 0xba9, 0xbcd, 0xbae, 0xbbe, 0xbb9, 0xbc7, 0xbae,
0xbbe, 0xbb2, 0xba4, 0xbcd, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0xbae,
0xbca, 0xbb0, 0xbbf, 0xbb7, 0xbbf, 0xbaf, 0xbb8, 0xbcd, 0xbae, 0xbaf, 0xbcb, 0xb9f,
0xbcd, 0xb9f, 0xbbf, 0xbb0, 0xbc0, 0xbaf, 0xbc2, 0xba9, 0xbbf, 0xbaf, 0xba9, 0xbcd,
0xb85, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0xb86, 0xb95, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd,
0xba4, 0xbc1, 0xbaa, 0xbcb, 0xb95, 0xbc6, 0xbaf, 0xbcd, 0xba9, 0xbcd, 0xbb5, 0xbbf,
0xbb2, 0xbcd, 0xbb2, 0xbc7, 0xb9a, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xbae, 0xbcd, 0xb9a,
0xbc1, 0xb95, 0xbcd, 0xb88, 0xbb8, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0xb88, 0xb83, 0xbaa,
0xbc7, 0xb9f, 0xbcd, 0xb83, 0xbaa, 0xb95, 0xbbe, 0xb83, 0xbaa, 0xbcb, 0xb83, 0xbaa,
0xbbf, 0xb9c, 0xbbf, 0xb83, 0xbaa, 0xbc1, 0xba9, 0xbbe, 0xb83, 0xbaa, 0xbc1, 0xb9f,
0xbbf, 0xb95, 0xbb2, 0xbaa, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xb95, 0xbc7, 0xbae, 0xbcd,
0xbaa, 0xbbf, 0xbaf, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbb5, 0xbbe, 0xb9f, 0xbbe, 0xbb2,
0xbcd, 0xb95, 0xbc7, 0xba9, 0xbb2, 0xbcd, 0xb95, 0xbc1, 0xbb5, 0xbbe, 0xbae, 0xbcd,
0xbb9, 0xbcb, 0xba9, 0xbcb, 0xbb2, 0xbc2, 0xbb2, 0xbc2, 0xb8e, 0xba3, 0xbcd, 0xb9f,
0xbb0, 0xbcd, 0xbaa, 0xbb0, 0xbbf, 0xb95, 0xbbf, 0xbb0, 0xbbf, 0xb9f, 0xbbf, 0xbae,
0xbbe, 0xb9f, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0xbb0, 0xbc7, 0xb95,
0xbcd, 0xbb5, 0xbbe, 0xb9c, 0xbbe, 0xbb2, 0xbc0, 0xbaf, 0xba9, 0xbcd, 0xbae, 0xb9c,
0xbc1, 0xbb0, 0xbcb, 0xbae, 0xbbe, 0xbb0, 0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbc2, 0xb9a,
0xbbe, 0xbb8, 0xbcd, 0xbae, 0xbbf, 0xb9f, 0xbcd, 0xbb5, 0xbc7, 0xba8, 0xbb5, 0xbcd,
0xbb0, 0xbc2, 0xba8, 0xbbf, 0xbaf, 0xbc1, 0xba8, 0xbbe, 0xbb0, 0xbcd, 0xb83, 0xbaa,
0xbcb, 0xb95, 0xbcd, 0xba8, 0xbcb, 0xbb5, 0xbc1, 0xbae, 0xbbf, 0xbaf, 0xbbe, 0xbaa,
0xbc7, 0xb95, 0xbcb, 0x20, 0xbaa, 0xbc7, 0xb95, 0xbcb, 0xbaa, 0xbbe, 0xbb2, 0xbb5,
0xbcd, 0xbaa, 0xbbf, 0xb9f, 0xbcd, 0xb95, 0xbc6, 0xbaf, 0xbcd, 0xbb0, 0xbcd, 0xba9,
0xbcd, 0xb83, 0xbaa, 0xbcb, 0xba9, 0xbcd, 0xbaa, 0xbc6, 0xbaf, 0xbcd, 0xbaa, 0xbcb,
0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbae, 0xbcb, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xbaa,
0xbc8, 0xbb0, 0xbb0, 0xbcb, 0xb9f, 0xbcb, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb9a, 0xbc8,
0xbaa, 0xba9, 0xbcd, 0xba4, 0xbb9, 0xbbf, 0xba4, 0xbbf, 0xba4, 0xbb0, 0xbbe, 0xbb5,
0xbbe, 0xb9f, 0xbcb, 0xba9, 0xbcd, 0xb95, 0xb9f, 0xbaa, 0xbcd, 0xbaa, 0xbc1, 0xbb5,
0xbc7, 0xb95, 0xbcd, 0xbb5, 0xbbe, 0xbb2, 0xbbf, 0xbb8, 0xbcd, 0x4a, 0x69, 0x6b,
0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6e, 0x6b,
0x72, 0x69, 0x6a, 0x69, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c,
0x61, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x61, 0x64, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x4a, 0x69, 0x63, 0x69,
0x61, 0x6b, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e,
0x67, 0x20, 0x54, 0x61, 0x6e, 0x62, 0x6f, 0x4a, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x49, 0x6e, 0x74, 0x69, 0x61,
0x6e, 0x6e, 0x61, 0x70, 0x6f, 0x72, 0x69, 0x73, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x52, 0x6f, 0x73, 0x61,
0x6e, 0x63, 0x69, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61,
0x6e, 0x67, 0x20, 0x4e, 0x69, 0x75, 0x79, 0x75, 0x65, 0x4a, 0x69, 0x6b,
0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x42, 0x6f, 0x6e,
0x68, 0x75, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c,
0x61, 0x6e, 0x67, 0x20, 0x50, 0x75, 0x72, 0x61, 0x6e, 0x6b, 0x4a, 0x69,
0x6b, 0x61, 0x6e, 0x20, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x53, 0x6e,
0x70, 0x61, 0x75, 0x72, 0x6f, 0x77, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20,
0x61, 0x6c, 0x61, 0x6e, 0x67, 0x20, 0x48, 0x6f, 0x6e, 0x6f, 0x72, 0x75,
0x72, 0x75, 0xc05, 0xc2c, 0xc3f, 0xc21, 0xc4d, 0xc1c, 0xc3e, 0xc28, 0xc4d, 0xc05,
0xc15, 0xc4d, 0xc30, 0xc3e, 0xc2f, 0xc3e, 0xc21, 0xc3f, 0xc38, 0xc4d, 0x20, 0xc05,
0xc2c, 0xc3e, 0xc2c, 0xc3e, 0xc05, 0xc32, 0xc4d, 0xc1c, 0xc3f, 0xc2f, 0xc30, 0xc4d,
0xc38, 0xc4d, 0xc05, 0xc38, 0xc4d, 0xc2e, 0xc3e, 0xc30, 0xc3e, 0xc2c, 0xc3e, 0xc2e,
0xc3e, 0xc15, 0xc4b, 0xc2c, 0xc3e, 0xc02, 0xc17, 0xc41, 0xc2f, 0xc4d, 0xc2c, 0xc02,
0xc1c, 0xc42, 0xc32, 0xc4d, 0xc2c, 0xc3f, 0xc38, 0xc4d, 0xc38, 0xc3e, 0xc35, 0xc4d,
0xc2c, 0xc4d, 0xc32, 0xc3e, 0xc28, 0xc4d, 0xc1f, 0xc48, 0xc30, 0xc4d, 0xc2c, 0xc4d,
0xc30, 0xc3e, 0xc1c, 0xc3e, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc47, 0xc2c, 0xc41,
0xc1c, 0xc2e, 0xc4d, 0xc2c, 0xc41, 0xc30, 0xc3e, 0xc15, 0xc48, 0xc30, 0xc4b, 0xc15,
0xc3e, 0xc38, 0xc3e, 0xc2c, 0xc4d, 0xc32, 0xc3e, 0xc02, 0xc15, 0xc3e, 0xc38, 0xc4d,
0xc2f, 0xc42, 0xc1f, 0xc3e, 0xc15, 0xc4b, 0xc28, 0xc3e, 0xc15, 0xc4d, 0xc30, 0xc40,
0xc21, 0xc15, 0xc3e, 0xc30, 0xc4d, 0xc26, 0xc3e, 0xc30, 0xc4d, 0x20, 0xc0e, 0xc38,
0xc4d, 0x20, 0xc38, 0xc32, 0xc3e, 0xc2e, 0xc4d, 0xc21, 0xc3f, 0xc1c, 0xc4d, 0xc2c,
0xc4c, 0xc1f, 0xc3f, 0xc21, 0xc4c, 0xc32, 0xc3e, 0xc0e, 0xc32, 0xc4d, 0x20, 0xc0e,
0xc2f, 0xc41, 0xc28, 0xc4d, 0xc2b, 0xc4d, 0xc30, 0xc40, 0xc1f, 0xc4c, 0xc28, 0xc4d,
0xc17, 0xc2c, 0xc4b, 0xc30, 0xc4b, 0xc28, 0xc4d, 0xc39, 0xc30, 0xc3e, 0xc30, 0xc47,
0xc1c, 0xc4a, 0xc39, 0xc46, 0xc28, 0xc4d, 0xc38, 0xc4d, 0x200c, 0xc2c, 0xc30, 0xc4d,
0xc17, 0xc4d, 0xc1c, 0xc41, 0xc2c, 0xc3e, 0xc15, 0xc02, 0xc2a, 0xc3e, 0xc32, 0xc3e,
0xc16, 0xc3e, 0xc30, 0xc4d, 0xc1f, 0xc4b, 0xc2e, 0xc4d, 0xc15, 0xc40, 0xc17, 0xc32,
0xc3f, 0xc15, 0xc3f, 0xc28, 0xc4d, 0xc37, 0xc3e, 0xc38, 0xc3e, 0xc32, 0xc3e, 0xc17,
0xc4b, 0xc38, 0xc4d, 0xc32, 0xc46, 0xc2c, 0xc30, 0xc4d, 0xc35, 0xc3f, 0xc32, 0xc4d,
0xc32, 0xc47, 0xc32, 0xc4b, 0xc2e, 0xc4d, 0xc32, 0xc41, 0xc35, 0xc3e, 0xc02, 0xc21,
0xc3e, 0xc32, 0xc41, 0xc2c, 0xc02, 0xc2c, 0xc3e, 0xc37, 0xc3f, 0xc32, 0xc41, 0xc38,
0xc3e, 0xc15, 0xc3e, 0xc2e, 0xc32, 0xc3e, 0xc2c, 0xc4b, 0xc2e, 0xc3e, 0xc2a, 0xc41,
0xc1f, 0xc4b, 0xc2e, 0xc38, 0xc47, 0xc30, 0xc41, 0xc2c, 0xc3e, 0xc2c, 0xc46, 0xc28,
0xc4d, 0xc2e, 0xc4b, 0xc17, 0xc3e, 0xc26, 0xc3f, 0xc37, 0xc41, 0xc2e, 0xc4b, 0xc28,
0xc4d, 0xc30, 0xc4b, 0xc35, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc48, 0xc30, 0xc4b, 0xc2c,
0xc40, 0xc21, 0xc4d, 0xc1c, 0xc3e, 0xc2e, 0xc46, 0xc28, 0xc3e, 0xc28, 0xc3f, 0xc2f,
0xc3e, 0xc2e, 0xc47, 0xc28, 0xc4d, 0xc35, 0xc3e, 0xc15, 0xc4d, 0xc37, 0xc4b, 0xc1f,
0xc4d, 0xc14, 0xc17, 0xc3e, 0xc21, 0xc4c, 0xc17, 0xc4b, 0xc35, 0xc4d, 0xc2a, 0xc4b,
0xc30, 0xc4d, 0xc1f, 0xc4b, 0x2d, 0xc28, 0xc4b, 0xc35, 0xc4b, 0xc38, 0xc3e, 0xc35,
0xc4b, 0x20, 0xc1f, 0xc4b, 0xc2e, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc3f, 0xc2a, 0xc4b,
0xc32, 0xc3f, 0xc1f, 0xc4d, 0xc2f, 0xc41, 0xc28, 0xc3f, 0xc38, 0xc4d, 0xc35, 0xc3f,
0xc02, 0xc21, 0xc4d, 0xc39, 0xc4b, 0xc15, 0xc4d, 0xc05, 0xc21, 0xc3e, 0xc15, 0xc4d,
0xc2f, 0xc3e, 0xc02, 0xc15, 0xc30, 0xc47, 0xc1c, 0xc4d, 0xc06, 0xc02, 0xc17, 0xc4d,
0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc3e, 0xc06, 0xc02, 0xc1f, 0xc3f, 0xc17, 0xc4d,
0xc35, 0xc3e, 0xc05, 0xc30, 0xc3e, 0xc17, 0xc4d, 0xc35, 0xc47, 0xc2f, 0xc40, 0xc28,
0xc3e, 0xc2c, 0xc4d, 0xc2f, 0xc42, 0xc28, 0xc4b, 0xc38, 0xc4d, 0x20, 0xc0e, 0xc2f,
0xc3f, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc1f, 0xc2e, 0xc3e, 0xc30, 0xc4d, 0xc15,
0xc3e, 0xc15, 0xc4b, 0xc30, 0xc4d, 0xc21, 0xc4b, 0xc2c, 0xc3e, 0xc1c, 0xc41, 0xc1c,
0xc41, 0xc2f, 0xc4d, 0xc32, 0xc3e, 0x20, 0xc30, 0xc3f, 0xc2f, 0xc4b, 0xc1c, 0xc2e,
0xc46, 0xc02, 0xc21, 0xc4b, 0xc1c, 0xc3e, 0xc30, 0xc3f, 0xc2f, 0xc4b, 0x20, 0xc17,
0xc32, 0xc4d, 0xc32, 0xc47, 0xc17, 0xc4b, 0xc38, 0xc4d, 0xc38, 0xc3e, 0xc32, 0xc4d,
0xc1f, 0xc3e, 0xc36, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc1c, 0xc4d, 0xc2f, 0xc42, 0xc28,
0xc4d, 0xc36, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc32, 0xc42, 0xc2f, 0xc3f, 0xc38, 0xc4d,
0xc1f, 0xc41, 0xc15, 0xc41, 0xc2e, 0xc28, 0xc4d, 0xc09, 0xc37, 0xc4d, 0xc2f, 0xc42,
0xc2f, 0xc05, 0xc30, 0xc41, 0xc2c, 0xc3e, 0xc05, 0xc38, 0xc41, 0xc28, 0xc4d, 0xc38,
0xc3f, 0xc2f, 0xc4b, 0xc28, 0xc4d, 0xc05, 0xc1f, 0xc3f, 0xc15, 0xc4b, 0xc15, 0xc28,
0xc4d, 0xc2c, 0xc39, 0xc3f, 0xc2f, 0xc2c, 0xc39, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2c,
0xc02, 0xc21, 0xc30, 0xc3e, 0xc38, 0xc4d, 0xc2c, 0xc3e, 0xc30, 0xc4d, 0xc2c, 0xc21,
0xc4b, 0xc38, 0xc4d, 0xc2c, 0xc46, 0xc32, 0xc46, 0xc2e, 0xc4d, 0xc2c, 0xc46, 0xc32,
0xc40, 0xc1c, 0xc4d, 0xc2c, 0xc4d, 0xc32, 0xc3e, 0xc02, 0xc15, 0xc4d, 0x2d, 0xc38,
0xc2c, 0xc4d, 0xc32, 0xc4b, 0xc28, 0xc4d, 0xc2c, 0xc4b, 0xc35, 0xc3e, 0x20, 0xc35,
0xc3f, 0xc38, 0xc4d, 0xc1f, 0xc3e, 0xc2c, 0xc17, 0xc4b, 0xc1f, 0xc3e, 0xc2c, 0xc4a,
0xc2f, 0xc3f, 0xc38, 0xc40, 0xc15, 0xc47, 0xc02, 0xc2c, 0xc4d, 0xc30, 0xc3f, 0xc21,
0xc4d, 0xc1c, 0xc4d, 0x20, 0xc2c, 0xc47, 0xc15, 0xc3e, 0xc02, 0xc2a, 0xc4b, 0x20,
0xc17, 0xc4d, 0xc30, 0xc3e, 0xc02, 0xc21, 0xc4d, 0xc15, 0xc28, 0xc4d, 0x200c, 0xc15,
0xc42, 0xc28, 0xc4d, 0xc15, 0xc3e, 0xc30, 0xc3e, 0xc15, 0xc38, 0xc4d, 0xc15, 0xc2f,
0xc47, 0xc28, 0xc4d, 0xc15, 0xc47, 0xc2e, 0xc3e, 0xc28, 0xc4d, 0xc1a, 0xc3f, 0xc15,
0xc3e, 0xc17, 0xc4b, 0xc1a, 0xc41, 0xc35, 0xc3e, 0xc35, 0xc3e, 0xc38, 0xc3f, 0xc2f,
0xc41, 0xc26, 0xc3e, 0xc26, 0xc4d, 0x20, 0xc39, 0xc4d, 0xc35, 0xc3e, 0xc30, 0xc46,
0xc1c, 0xc4d, 0xc15, 0xc4b, 0xc38, 0xc4d, 0xc1f, 0xc3e, 0x20, 0xc30, 0xc3f, 0xc15,
0xc3e, 0xc15, 0xc4d, 0xc30, 0xc46, 0xc38, 0xc4d, 0xc1f, 0xc28, 0xc4d, 0xc15, 0xc41,
0xc2f, 0xc2c, 0xc3e, 0xc15, 0xc41, 0xc30, 0xc3e, 0xc15, 0xc35, 0xc4b, 0xc21, 0xc46,
0xc28, 0xc4d, 0xc2e, 0xc3e, 0xc30, 0xc4d, 0xc15, 0xc4d, 0x200c, 0xc37, 0xc3e, 0xc28,
0xc4d, 0xc21, 0xc3e, 0xc38, 0xc28, 0xc4d, 0xc21, 0xc3e, 0xc38, 0xc28, 0xc4d, 0x20,
0xc15, 0xc4d, 0xc30, 0xc40, 0xc15, 0xc4d, 0xc21, 0xc46, 0xc28, 0xc4d, 0xc35, 0xc46,
0xc30, 0xc4d, 0xc21, 0xc3f, 0xc1f, 0xc4d, 0xc30, 0xc4b, 0xc2f, 0xc3f, 0xc1f, 0xc4d,
0xc21, 0xc4a, 0xc2e, 0xc3f, 0xc28, 0xc3f, 0xc15, 0xc3e, 0xc0e, 0xc21, 0xc4d, 0xc2e,
0xc4b, 0xc02, 0xc1f, 0xc28, 0xc4d, 0xc07, 0xc30, 0xc41, 0xc28, 0xc46, 0xc2a, 0xc4d,
0xc0e, 0xc32, 0xc4d, 0x20, 0xc38, 0xc3e, 0xc32, 0xc4d, 0xc35, 0xc21, 0xc4b, 0xc30,
0xc4d, 0xc2b, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc4d, 0x20, 0xc28, 0xc46, 0xc32, 0xc4d,
0xc38, 0xc28, 0xc4d, 0xc2b, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc32, 0xc47, 0xc1c, 0xc3e,
0xc17, 0xc4d, 0xc32, 0xc47, 0xc38, 0xc4d, 0x20, 0xc2c, 0xc47, 0xc17, 0xc42, 0xc38,
0xc4d, 0x20, 0xc2c, 0xc47, 0xc17, 0xc4d, 0xc30, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20,
0xc1f, 0xc30, 0xc4d, 0xc15, 0xc4d, 0xc17, 0xc4d, 0xc30, 0xc46, 0xc28, 0xc21, 0xc3e,
0xc17, 0xc4d, 0xc35, 0xc3e, 0xc21, 0xc46, 0xc32, 0xc4b, 0xc2a, 0xc4d, 0xc17, 0xc4d,
0xc35, 0xc3e, 0xc1f, 0xc3f, 0xc2e, 0xc3e, 0xc32, 0xc3e, 0xc17, 0xc2f, 0xc3e, 0xc15,
0xc4d, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc17, 0xc2f, 0xc3e, 0xc28, 0xc3e, 0xc39, 0xc3e,
0xc32, 0xc3f, 0xc2b, 0xc3e, 0xc15, 0xc4d, 0xc38, 0xc4d, 0xc39, 0xc35, 0xc3e, 0xc28,
0xc3e, 0xc39, 0xc46, 0xc30, 0xc4d, 0xc2e, 0xc4b, 0xc38, 0xc3f, 0xc32, 0xc4d, 0xc32,
0xc4b, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc2a, 0xc4b, 0xc32, 0xc3f,
0xc38, 0xc4d, 0xc28, 0xc4b, 0xc15, 0xc4d, 0xc38, 0xc4d, 0x2c, 0x20, 0xc07, 0xc02,
0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc2e, 0xc30, 0xc46, 0xc02, 0xc17, 0xc4b, 0x2c,
0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc2a, 0xc40, 0xc1f, 0xc30,
0xc4d, 0xc38, 0xc4d, 0x200c, 0xc2c, 0xc30, 0xc4d, 0xc17, 0xc4d, 0x2c, 0x20, 0xc07,
0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc1f, 0xc46, 0xc32, 0xc4d, 0x20, 0xc28,
0xc17, 0xc30, 0xc02, 0x2c, 0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28,
0xc35, 0xc46, 0xc35, 0xc2f, 0xc4d, 0x2c, 0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f,
0xc3e, 0xc28, 0xc35, 0xc3f, 0xc28, 0xc4d, 0x200c, 0xc38, 0xc46, 0xc28, 0xc4d, 0xc28,
0xc46, 0xc38, 0xc4d, 0x2c, 0x20, 0xc07, 0xc02, 0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28,
0xc35, 0xc3f, 0xc28, 0xc3e, 0xc2e, 0xc3e, 0xc15, 0xc4d, 0x2c, 0x20, 0xc07, 0xc02,
0xc21, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc07, 0xc28, 0xc41, 0xc35, 0xc3f, 0xc15, 0xc4d,
0xc07, 0xc15, 0xc4d, 0xc35, 0xc3e, 0xc32, 0xc3f, 0xc1f, 0xc4d, 0xc1c, 0xc2e, 0xc48,
0xc15, 0xc3e, 0xc1c, 0xc42, 0xc28, 0xc4b, 0xc32, 0xc42, 0xc2f, 0xc3f, 0xc35, 0xc3f,
0xc32, 0xc4d, 0xc2e, 0xc4b, 0xc02, 0xc1f, 0xc3f, 0xc38, 0xc46, 0xc32, 0xc4d, 0xc32,
0xc4b, 0x2c, 0x20, 0xc15, 0xc46, 0xc28, 0xc4d, 0x200c, 0xc1f, 0xc41, 0xc15, 0xc4d,
0xc15, 0xc40, 0xc15, 0xc4d, 0xc30, 0xc32, 0xc46, 0xc02, 0xc21, 0xc4d, 0xc1c, 0xc3f,
0xc15, 0xc4d, 0xc32, 0xc3e, 0x20, 0xc2a, 0xc3e, 0xc1c, 0xc4d, 0xc32, 0xc3f, 0xc2e,
0xc3e, 0xc32, 0xc3e, 0xc38, 0xc4d, 0x20, 0xc0f, 0xc02, 0xc1c, 0xc32, 0xc4d, 0xc38,
0xc4d, 0xc32, 0xc4b, 0xc2f, 0xc30, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3f, 0xc28,
0xc4d, 0xc38, 0xc4d, 0x20, 0xc15, 0xc4d, 0xc35, 0xc3e, 0xc30, 0xc4d, 0xc1f, 0xc30,
0xc4d, 0xc2e, 0xc3e, 0xc38, 0xc3f, 0xc2f, 0xc4b, 0xc2e, 0xc28, 0xc3e, 0xc17, 0xc41,
0xc35, 0xc3e, 0xc2e, 0xc28, 0xc3e, 0xc38, 0xc4d, 0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc17,
0xc4b, 0xc1f, 0xc4d, 0xc2e, 0xc3e, 0xc30, 0xc4d, 0xc1f, 0xc3f, 0xc28, 0xc40, 0xc15,
0xc4d, 0xc2e, 0xc3e, 0xc1f, 0xc2e, 0xc4a, 0xc30, 0xc4b, 0xc38, 0xc4d, 0xc2e, 0xc3e,
0xc38, 0xc1f, 0xc4d, 0x200c, 0xc32, 0xc3e, 0xc28, 0xc4d, 0xc2e, 0xc46, 0xc28, 0xc4b,
0xc2e, 0xc3f, 0xc28, 0xc40, 0xc2e, 0xc46, 0xc30, 0xc3f, 0xc21, 0xc3e, 0xc2e, 0xc46,
0xc1f, 0xc4d, 0xc32, 0xc15, 0xc1f, 0xc4d, 0xc32, 0xc2e, 0xc46, 0xc15, 0xc4d, 0xc38,
0xc3f, 0xc15, 0xc4b, 0x20, 0xc28, 0xc17, 0xc30, 0xc02, 0xc2e, 0xc3f, 0xc15, 0xc46,
0xc32, 0xc3e, 0xc28, 0xc4d, 0xc2e, 0xc4b, 0xc28, 0xc4d, 0x200c, 0xc15, 0xc4d, 0xc1f,
0xc4b, 0xc28, 0xc4d, 0xc2e, 0xc4b, 0xc02, 0xc1f, 0xc46, 0xc30, 0xc4d, 0xc30, 0xc47,
0xc2e, 0xc4b, 0xc02, 0xc1f, 0xc46, 0xc35, 0xc40, 0xc21, 0xc3f, 0xc2f, 0xc4b, 0xc2e,
0xc3e, 0xc02, 0xc1f, 0xc4d, 0xc38, 0xc46, 0xc30, 0xc3e, 0xc1f, 0xc4d, 0xc28, 0xc3e,
0xc38, 0xc4d, 0xc38, 0xc3e, 0xc35, 0xc4d, 0xc28, 0xc4d, 0xc2f, 0xc42, 0xc2f, 0xc3e,
0xc30, 0xc4d, 0xc15, 0xc4d, 0xc28, 0xc4b, 0xc2e, 0xc4d, 0xc28, 0xc30, 0xc4b, 0xc28,
0xc4d, 0xc39, 0xc3e, 0xc2c, 0xc4d, 0xc2f, 0xc41, 0xc32, 0xc3e, 0x2c, 0x20, 0xc09,
0xc24, 0xc4d, 0xc24, 0xc30, 0x20, 0xc21, 0xc15, 0xc4b, 0xc1f, 0xc38, 0xc46, 0xc02,
0xc1f, 0xc30, 0xc4d, 0x2c, 0x20, 0xc09, 0xc24, 0xc4d, 0xc24, 0xc30, 0x20, 0xc21,
0xc15, 0xc4b, 0xc1f, 0xc3e, 0xc28, 0xc4d, 0xc2f, 0xc42, 0x20, 0xc38, 0xc32, 0xc47,
0xc2e, 0xc4d, 0x2c, 0x20, 0xc09, 0xc24, 0xc4d, 0xc24, 0xc30, 0x20, 0xc21, 0xc15,
0xc4b, 0xc1f, 0xc28, 0xc42, 0xc15, 0xc4d, 0xc12, 0xc1c, 0xc3f, 0xc28, 0xc17, 0xc2a,
0xc28, 0xc3e, 0xc2e, 0xc3e, 0xc2a, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc2c,
0xc4b, 0xc2b, 0xc3f, 0xc28, 0xc3f, 0xc15, 0xc4d, 0xc38, 0xc4d, 0xc2a, 0xc4b, 0xc30,
0xc4d, 0xc1f, 0xc4d, 0x2d, 0xc05, 0xc35, 0xc4d, 0x2d, 0xc2a, 0xc4d, 0xc30, 0xc3f,
0xc28, 0xc4d, 0xc38, 0xc4d, 0xc2a, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc4d, 0x20, 0xc06,
0xc2b, 0xc4d, 0x20, 0xc38, 0xc4d, 0xc2a, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d, 0xc2a,
0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc4b, 0x20, 0xc35, 0xc46, 0xc32, 0xc4d, 0xc39, 0xc4b,
0xc2a, 0xc4d, 0xc2f, 0xc42, 0xc30, 0xc4d, 0xc1f, 0xc4b, 0x20, 0xc30, 0xc3f, 0xc15,
0xc4b, 0xc2a, 0xc41, 0xc02, 0xc1f, 0xc3e, 0x20, 0xc05, 0xc30, 0xc40, 0xc28, 0xc38,
0xc4d, 0xc30, 0xc28, 0xc4d, 0x200c, 0xc15, 0xc3f, 0xc28, 0xc4d, 0x20, 0xc07, 0xc28,
0xc4d, 0x200c, 0xc32, 0xc46, 0xc1f, 0xc4d, 0xc30, 0xc46, 0xc38, 0xc3f, 0xc2b, 0xc40,
0xc30, 0xc46, 0xc1c, 0xc40, 0xc28, 0xc3e, 0xc30, 0xc3f, 0xc1c, 0xc32, 0xc4d, 0xc2f,
0xc42, 0xc1f, 0xc4d, 0xc30, 0xc3f, 0xc2f, 0xc4b, 0x20, 0xc2c, 0xc4d, 0xc30, 0xc3e,
0xc02, 0xc15, 0xc4b, 0xc38, 0xc3e, 0xc02, 0xc1f, 0xc30, 0xc46, 0xc2e, 0xc4d, 0xc36,
0xc3e, 0xc02, 0xc1f, 0xc3f, 0xc2f, 0xc3e, 0xc17, 0xc4b, 0xc36, 0xc3e, 0xc02, 0xc1f,
0xc4b, 0x20, 0xc21, 0xc4b, 0xc2e, 0xc3f, 0xc02, 0xc17, 0xc4b, 0xc38, 0xc3e, 0xc35,
0xc4b, 0x20, 0xc2a, 0xc3e, 0xc32, 0xc4b, 0xc07, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0xc15,
0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc42, 0xc30, 0xc4d, 0xc2e, 0xc3f, 0xc1f, 0xc4d, 0xc38,
0xc3f, 0xc1f, 0xc4d, 0xc15, 0xc3e, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d,
0x20, 0xc2c, 0xc30, 0xc4d, 0xc24, 0xc46, 0xc32, 0xc46, 0xc2e, 0xc40, 0xc38, 0xc46,
0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc1c, 0xc3e, 0xc28, 0xc4d, 0xc38, 0xc4d,
0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc15, 0xc3f, 0xc1f, 0xc4d,
0xc38, 0xc4d, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc32, 0xc42,
0xc38, 0xc3f, 0xc2f, 0xc3e, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20,
0xc25, 0xc3e, 0xc2e, 0xc38, 0xc4d, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d,
0x20, 0xc35, 0xc3f, 0xc28, 0xc4d, 0xc38, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc38, 0xc4d,
0xc35, 0xc3f, 0xc2b, 0xc4d, 0xc1f, 0xc4d, 0x20, 0xc15, 0xc30, 0xc46, 0xc02, 0xc1f,
0xc4d, 0xc24, 0xc46, 0xc17, 0xc41, 0xc38, 0xc3f, 0xc17, 0xc32, 0xc4d, 0xc2a, 0xc3e,
0xc25, 0xc41, 0xc32, 0xc47, 0xc1f, 0xc3f, 0xc1c, 0xc41, 0xc35, 0xc3e, 0xc28, 0xc3e,
0xc1f, 0xc4a, 0xc30, 0xc02, 0xc1f, 0xc4b, 0xc1f, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc4b,
0xc32, 0xc3e, 0xc35, 0xc3e, 0xc28, 0xc4d, 0xc15, 0xc42, 0xc35, 0xc30, 0xc4d, 0xc35,
0xc48, 0xc1f, 0xc4d, 0x200c, 0xc39, 0xc3e, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc35, 0xc3f,
0xc28, 0xc4d, 0xc28, 0xc3f, 0xc2a, 0xc46, 0xc17, 0xc4d, 0xc2f, 0xc15, 0xc41, 0xc1f,
0xc3e, 0xc1f, 0xc4d, 0xc15, 0xc47, 0xc38, 0xc40, 0xc21, 0xc46, 0xc35, 0xc3f, 0xc38,
0xc4d, 0xc21, 0xc4d, 0xc2f, 0xc42, 0xc2e, 0xc3e, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc21,
0xc3f, 0x2019, 0xc09, 0xc30, 0xc4d, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc47, 0xc2e,
0xc15, 0xc3e, 0xc30, 0xc40, 0xc2e, 0xc3e, 0xc38, 0xc28, 0xc4d, 0xc2e, 0xc46, 0xc15,
0xc4d, 0x200c, 0xc2e, 0xc41, 0xc30, 0xc4d, 0xc21, 0xc4b, 0xc2a, 0xc3e, 0xc32, 0xc4d,
0xc2e, 0xc30, 0xc4d, 0xc30, 0xc4a, 0xc24, 0xc47, 0xc30, 0xc3e, 0xc38, 0xc4d, 0xc2f,
0xc4b, 0xc35, 0xc3e, 0xc1f, 0xc4d, 0xc30, 0xc4b, 0xc32, 0xc4d, 0xc35, 0xc4b, 0xc38,
0xc4d, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0xc32, 0xc3e, 0xc02, 0xc17, 0xc4d, 0x200c, 0xc2f,
0xc3f, 0xc2f, 0xc30, 0xc4d, 0x200c, 0xc2c, 0xc48, 0xc2f, 0xc28, 0xc4d, 0xc0e, 0xc21,
0xc46, 0xc28, 0xc4d, 0xc06, 0xc32, 0xc4d, 0xc2e, 0xc3e, 0xc1f, 0xc3f, 0xc05, 0xc2e,
0xc4d, 0xc2e, 0xc28, 0xc4d, 0xc05, 0xc28, 0xc21, 0xc48, 0xc30, 0xc4d, 0xc05, 0xc15,
0xc4d, 0xc1f, 0xc3e, 0xc35, 0xc4d, 0xc05, 0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc2c, 0xc4d,
0xc2f, 0xc3e, 0xc37, 0xc4d, 0xc17, 0xc3e, 0xc2c, 0xc3e, 0xc1f, 0xc4d, 0xc06, 0xc1f,
0xc3f, 0xc30, 0xc3e, 0xc2c, 0xc3e, 0xc17, 0xc4d, 0xc26, 0xc3e, 0xc26, 0xc4d, 0xc2c,
0xc39, 0xc4d, 0xc30, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d, 0xc2c, 0xc4d, 0xc2f, 0xc3e,
0xc02, 0xc15, 0xc3e, 0xc15, 0xc4d, 0xc2c, 0xc3e, 0xc30, 0xc4d, 0xc28, 0xc3e, 0xc32,
0xc4d, 0xc2c, 0xc40, 0xc30, 0xc1f, 0xc4d, 0xc2c, 0xc3f, 0xc37, 0xc4d, 0xc15, 0xc46,
0xc15, 0xc4d, 0xc2c, 0xc4d, 0xc30, 0xc42, 0xc28, 0xc48, 0xc1a, 0xc3f, 0xc24, 0xc3e,
0xc1a, 0xc4b, 0xc2f, 0xc3f, 0xc2c, 0xc3e, 0xc32, 0xc4d, 0xc38, 0xc28, 0xc4d, 0xc15,
0xc4a, 0xc32, 0xc02, 0xc2c, 0xc4b, 0xc21, 0xc2e, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc38,
0xc4d, 0xc22, 0xc3e, 0xc15, 0xc3e, 0xc21, 0xc3f, 0xc32, 0xc3f, 0xc26, 0xc41, 0xc2c,
0xc3e, 0xc2f, 0xc3f, 0xc21, 0xc41, 0xc37, 0xc28, 0xc4d, 0xc2c, 0xc40, 0xc2b, 0xc3e,
0xc2e, 0xc17, 0xc41, 0xc38, 0xc4d, 0xc1f, 0xc3e, 0xc17, 0xc3e, 0xc1c, 0xc3e, 0xc39,
0xc46, 0xc2c, 0xc4d, 0xc30, 0xc3e, 0xc28, 0xc4d, 0xc39, 0xc4b, 0x20, 0xc1a, 0xc3f,
0x20, 0xc2e, 0xc3f, 0xc28, 0xc4d, 0xc39, 0xc4d, 0x20, 0xc28, 0xc17, 0xc30, 0xc02,
0xc39, 0xc3e, 0xc02, 0xc15, 0xc3e, 0xc02, 0xc17, 0xc4d, 0xc39, 0xc4b, 0xc35, 0xc4d,
0xc21, 0xc4d, 0xc07, 0xc30, 0xc4d, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15,
0xc4d, 0xc1c, 0xc15, 0xc3e, 0xc30, 0xc4d, 0xc24, 0xc3e, 0xc1c, 0xc2f, 0xc2a, 0xc41,
0xc30, 0xc1c, 0xc30, 0xc42, 0xc38, 0xc32, 0xc47, 0xc02, 0xc15, 0xc3e, 0xc2c, 0xc41,
0xc32, 0xc4d, 0xc15, 0xc2e, 0xc4d, 0x200c, 0xc1a, 0xc24, 0xc4d, 0xc15, 0xc3e, 0xc15,
0xc30, 0xc3e, 0xc1a, 0xc40, 0xc16, 0xc3e, 0xc1f, 0xc4d, 0xc2e, 0xc02, 0xc21, 0xc41,
0xc15, 0xc02, 0xc26, 0xc4d, 0xc2f, 0xc3e, 0xc17, 0xc15, 0xc4b, 0xc32, 0xc4d, 0x200c,
0xc15, 0xc24, 0xc3e, 0xc15, 0xc4d, 0xc30, 0xc38, 0xc28, 0xc4b, 0xc2f, 0xc3e, 0xc30,
0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc15, 0xc4c, 0xc32, 0xc3e, 0xc32, 0xc02, 0xc2a,
0xc42, 0xc30, 0xc4d, 0xc15, 0xc41, 0xc1a, 0xc3f, 0xc02, 0xc17, 0xc4d, 0xc15, 0xc41,
0xc35, 0xc48, 0xc1f, 0xc4d, 0xc2e, 0xc15, 0xc3e, 0xc35, 0xc4d, 0xc2e, 0xc17, 0xc21,
0xc3e, 0xc28, 0xc4d, 0xc2e, 0xc15, 0xc3e, 0xc38, 0xc4d, 0xc38, 0xc30, 0xc4d, 0xc2e,
0xc28, 0xc40, 0xc32, 0xc3e, 0xc2e, 0xc38, 0xc4d, 0xc15, 0xc1f, 0xc4d, 0xc28, 0xc3f,
0xc15, 0xc4b, 0xc38, 0xc3f, 0xc2f, 0xc3e, 0xc28, 0xc4a, 0xc35, 0xc4a, 0xc15, 0xc41,
0xc1c, 0xc4d, 0x200c, 0xc28, 0xc46, 0xc1f, 0xc4d, 0x200c, 0xc38, 0xc4d, 0xc15, 0xc4d,
0xc28, 0xc35, 0xc4b, 0xc38, 0xc3f, 0xc2c, 0xc3f, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15,
0xc4d, 0xc13, 0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc13, 0xc30, 0xc32, 0xc4d,
0xc28, 0xc4b, 0xc2e, 0xc4d, 0x200c, 0xc2a, 0xc46, 0xc28, 0xc4d, 0xc39, 0xc4d, 0xc2a,
0xc4a, 0xc28, 0xc4d, 0xc1f, 0xc3f, 0xc2f, 0xc28, 0xc3e, 0xc15, 0xc4d, 0xc2a, 0xc4d,
0xc2f, 0xc4b, 0xc02, 0xc17, 0xc3e, 0xc02, 0xc17, 0xc4d, 0xc16, 0xc24, 0xc3e, 0xc30,
0xc4d, 0xc15, 0xc4b, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc47, 0xc15, 0xc4d, 0xc35,
0xc3f, 0xc1c, 0xc3f, 0xc32, 0xc4b, 0xc30, 0xc4d, 0xc21, 0xc3e, 0xc30, 0xc3f, 0xc2f,
0xc3e, 0xc27, 0xc4d, 0xc38, 0xc16, 0xc3e, 0xc32, 0xc3f, 0xc28, 0xc4d, 0xc38, 0xc2e,
0xc30, 0xc4d, 0xc15, 0xc3e, 0xc02, 0xc21, 0xc4d, 0xc38, 0xc3f, 0xc2f, 0xc4b, 0xc32,
0xc4d, 0xc37, 0xc3e, 0xc02, 0xc18, 0xc48, 0xc38, 0xc3f, 0xc02, 0xc17, 0xc2a, 0xc42,
0xc30, 0xc4d, 0xc38, 0xc4d, 0xc30, 0xc46, 0xc21, 0xc4d, 0xc28, 0xc46, 0xc15, 0xc4a,
0xc32, 0xc3f, 0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc24, 0xc48, 0xc2a, 0xc40,
0xc24, 0xc3e, 0xc37, 0xc4d, 0xc15, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc1f, 0xc3f, 0xc2c,
0xc3f, 0xc32, 0xc3f, 0xc38, 0xc3f, 0xc1f, 0xc46, 0xc39, 0xc4d, 0xc30, 0xc3e, 0xc28,
0xc4d, 0xc25, 0xc3f, 0xc02, 0xc2b, 0xc41, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0xc2f, 0xc4b,
0xc1f, 0xc3e, 0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc09, 0xc32, 0xc3e, 0xc28,
0xc4d, 0xc2c, 0xc3e, 0xc1f, 0xc30, 0xc4d, 0xc09, 0xc30, 0xc41, 0xc2e, 0xc4d, 0x200c,
0xc15, 0xc40, 0xc05, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0x2d, 0xc28, 0xc46, 0xc30, 0xc3e,
0xc35, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0xc1f, 0xc48, 0xc28, 0xc4d, 0xc35, 0xc4d, 0xc32,
0xc3e, 0xc21, 0xc3f, 0xc35, 0xc4b, 0xc38, 0xc4d, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0xc2f,
0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc17,
0xc28, 0xc4d, 0xc2f, 0xc46, 0xc15, 0xc1f, 0xc46, 0xc30, 0xc3f, 0xc02, 0xc2c, 0xc30,
0xc4d, 0xc17, 0xc4d, 0xc2f, 0xc46, 0xc30, 0xc46, 0xc35, 0xc28, 0xc4d, 0xc05, 0xc1c,
0xc4b, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc2c, 0xc46, 0xc30, 0xc4d, 0xc2e, 0xc41, 0xc21,
0xc3e, 0xc15, 0xc46, 0xc28, 0xc30, 0xc40, 0xc15, 0xc47, 0xc2a, 0xc4d, 0x20, 0xc35,
0xc46, 0xc30, 0xc4d, 0xc21, 0xc46, 0xc2b, 0xc3e, 0xc30, 0xc4b, 0xc2e, 0xc26, 0xc48,
0xc30, 0xc3e, 0xc30, 0xc46, 0xc15, 0xc4d, 0xc1c, 0xc3e, 0xc35, 0xc3f, 0xc15, 0xc4d,
0xc26, 0xc15, 0xc4d, 0xc37, 0xc3f, 0xc23, 0x20, 0xc1c, 0xc3e, 0xc30, 0xc4d, 0xc1c,
0xc3f, 0xc2f, 0xc3e, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc39,
0xc46, 0xc32, 0xc46, 0xc28, 0xc3e, 0xc38, 0xc4d, 0xc1f, 0xc3e, 0xc28, 0xc4d, 0xc32,
0xc40, 0xc05, 0xc21, 0xc46, 0xc32, 0xc48, 0xc21, 0xc4d, 0xc2c, 0xc4d, 0xc30, 0xc3f,
0xc38, 0xc4d, 0x200c, 0xc2c, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d, 0xc2c, 0xc4d, 0xc30,
0xc4b, 0xc15, 0xc46, 0xc28, 0xc4d, 0x20, 0xc39, 0xc3f, 0xc32, 0xc4d, 0xc21, 0xc3e,
0xc30, 0xc4d, 0xc35, 0xc3f, 0xc28, 0xc4d, 0xc2f, 0xc41, 0xc15, 0xc4d, 0xc32, 0xc3e,
0xc39, 0xc4b, 0xc2c, 0xc30, 0xc4d, 0xc1f, 0xc4d, 0xc32, 0xc3f, 0xc02, 0xc21, 0xc46,
0xc2e, 0xc3e, 0xc28, 0xc4d, 0xc32, 0xc3e, 0xc30, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc39,
0xc4c, 0xc2e, 0xc46, 0xc32, 0xc4d, 0xc2c, 0xc4b, 0xc30, 0xc4d, 0xc28, 0xc4d, 0xc2a,
0xc46, 0xc30, 0xc4d, 0xc24, 0xc4d, 0xc38, 0xc3f, 0xc21, 0xc4d, 0xc28, 0xc40, 0xc06,
0xc2e, 0xc4d, 0x200c, 0xc38, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0x200c, 0xc21, 0xc3e, 0xc2e,
0xc4d, 0xc05, 0xc02, 0xc21, 0xc4b, 0xc30, 0xc3e, 0xc05, 0xc38, 0xc4d, 0xc1f, 0xc4d,
0xc30, 0xc16, 0xc3e, 0xc28, 0xc4d, 0xc0f, 0xc25, 0xc46, 0xc28, 0xc4d, 0xc38, 0xc4d,
0xc2c, 0xc46, 0xc32, 0xc4d, 0x200c, 0xc17, 0xc4d, 0xc30, 0xc47, 0xc21, 0xc4d, 0xc2c,
0xc46, 0xc30, 0xc4d, 0xc32, 0xc3f, 0xc28, 0xc4d, 0xc2c, 0xc4d, 0xc30, 0xc3e, 0xc1f,
0xc3f, 0xc38, 0xc4d, 0xc32, 0xc3e, 0xc35, 0xc3e, 0xc2c, 0xc4d, 0xc30, 0xc38, 0xc4d,
0xc38, 0xc46, 0xc32, 0xc4d, 0xc38, 0xc4d, 0xc2c, 0xc41, 0xc15, 0xc3e, 0xc30, 0xc46,
0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc2c, 0xc41, 0xc21, 0xc3e, 0xc2a, 0xc46, 0xc38, 0xc4d,
0xc1f, 0xc4d, 0xc2c, 0xc38, 0xc3f, 0xc02, 0xc1c, 0xc28, 0xc4d, 0xc1a, 0xc3f, 0xc38,
0xc3f, 0xc28, 0xc3e, 0xc35, 0xc4d, 0xc15, 0xc4b, 0xc2a, 0xc46, 0xc28, 0xc4d, 0xc39,
0xc3e, 0xc17, 0xc28, 0xc4d, 0xc21, 0xc2c, 0xc4d, 0xc32, 0xc3f, 0xc28, 0xc4d, 0xc1c,
0xc3f, 0xc2c, 0xc4d, 0xc30, 0xc3e, 0xc32, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0xc17, 0xc4d,
0xc35, 0xc46, 0xc30, 0xc4d, 0xc28, 0xc4d, 0xc38, 0xc47, 0xc39, 0xc46, 0xc32, 0xc4d,
0xc38, 0xc3f, 0xc02, 0xc15, 0xc3f, 0xc10, 0xc32, 0xc4d, 0x20, 0xc06, 0xc2b, 0xc4d,
0x20, 0xc2e, 0xc47, 0xc28, 0xc4d, 0xc07, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc02, 0xc2c,
0xc41, 0xc32, 0xc4d, 0xc1c, 0xc46, 0xc30, 0xc4d, 0xc38, 0xc40, 0xc15, 0xc32, 0xc3f,
0xc28, 0xc3f, 0xc28, 0xc4d, 0x200c, 0xc17, 0xc4d, 0xc30, 0xc26, 0xc4d, 0xc15, 0xc3f,
0xc30, 0xc4b, 0xc35, 0xc4d, 0xc15, 0xc40, 0xc35, 0xc4d, 0xc32, 0xc3f, 0xc38, 0xc4d,
0xc2c, 0xc28, 0xc4d, 0xc32, 0xc4d, 0xc2f, 0xc42, 0xc2c, 0xc4d, 0xc32, 0xc4d, 0xc2f,
0xc3e, 0xc28, 0xc3e, 0xc32, 0xc02, 0xc21, 0xc28, 0xc4d, 0xc32, 0xc15, 0xc4d, 0xc38,
0xc46, 0xc02, 0xc2c, 0xc30, 0xc4d, 0xc17, 0xc4d, 0xc2e, 0xc3e, 0xc21, 0xc4d, 0xc30,
0xc3f, 0xc21, 0xc4d, 0xc2e, 0xc3e, 0xc32, 0xc4d, 0xc1f, 0xc3e, 0xc2e, 0xc3e, 0xc30,
0xc40, 0xc2f, 0xc41, 0xc39, 0xc2e, 0xc4d, 0xc2e, 0xc3f, 0xc28, 0xc4d, 0xc38, 0xc4d,
0xc15, 0xc4d, 0xc2e, 0xc4a, 0xc28, 0xc3e, 0xc15, 0xc4b, 0xc2e, 0xc3e, 0xc38, 0xc4d,
0xc15, 0xc4b, 0xc13, 0xc38, 0xc4d, 0xc32, 0xc4b, 0xc2a, 0xc4d, 0xc2f, 0xc3e, 0xc30,
0xc3f, 0xc38, 0xc4d, 0xc2a, 0xc4b, 0xc21, 0xc4d, 0xc17, 0xc4b, 0xc30, 0xc3f, 0xc15,
0xc3e, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc17, 0xc4d, 0xc30, 0xc40, 0xc17, 0xc3e, 0xc30,
0xc4b, 0xc2e, 0xc4d, 0xc38, 0xc2e, 0xc3e, 0xc30, 0xc36, 0xc3e, 0xc28, 0xc4d, 0x20,
0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc28, 0xc4b, 0xc38, 0xc30, 0xc3e, 0xc1c, 0xc4b, 0xc35,
0xc4b, 0xc38, 0xc30, 0xc3e, 0xc1f, 0xc35, 0xc4d, 0xc38, 0xc3f, 0xc2e, 0xc4d, 0x200c,
0xc2b, 0xc46, 0xc30, 0xc4b, 0xc2a, 0xc4b, 0xc32, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4b,
0xc2a, 0xc4d, 0x200c, 0xc2f, 0xc47, 0xc38, 0xc4b, 0xc2b, 0xc3f, 0xc2f, 0xc3e, 0xc38,
0xc4d, 0xc1f, 0xc3e, 0xc15, 0xc4d, 0xc39, 0xc4b, 0xc2e, 0xc4d, 0xc24, 0xc3e, 0xc32,
0xc4d, 0xc32, 0xc3f, 0xc28, 0xc4d, 0xc1f, 0xc3f, 0xc30, 0xc47, 0xc28, 0xc4d, 0xc09,
0xc32, 0xc4d, 0xc2f, 0xc28, 0xc4b, 0xc35, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc35,
0xc3e, 0xc21, 0xc41, 0xc1c, 0xc4d, 0xc35, 0xc3e, 0xc1f, 0xc3f, 0xc15, 0xc28, 0xc4d,
0xc35, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0xc28, 0xc3e, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc28,
0xc3f, 0xc2f, 0xc38, 0xc4d, 0xc35, 0xc4b, 0xc32, 0xc4d, 0xc17, 0xc4b, 0xc17, 0xc4d,
0xc30, 0xc3e, 0xc21, 0xc4d, 0xc35, 0xc3e, 0xc30, 0xc4d, 0xc37, 0xc3e, 0xc1c, 0xc3e,
0xc17, 0xc4d, 0xc30, 0xc46, 0xc2c, 0xc4d, 0xc1c, 0xc4d, 0xc2f, 0xc42, 0xc30, 0xc3f,
0xc1a, 0xc4d, 0xc05, 0xc02, 0xc1f, 0xc3e, 0xc28, 0xc3e, 0xc28, 0xc3e, 0xc30, 0xc3f,
0xc35, 0xc4b, 0xc1a, 0xc3e, 0xc17, 0xc4b, 0xc38, 0xc4d, 0xc15, 0xc4d, 0xc30, 0xc3f,
0xc38, 0xc4d, 0xc2e, 0xc38, 0xc4d, 0xc15, 0xc4b, 0xc15, 0xc4b, 0xc38, 0xc4d, 0xc15,
0xc4a, 0xc2e, 0xc4b, 0xc30, 0xc4b, 0xc15, 0xc46, 0xc30, 0xc4d, 0xc17, 0xc4d, 0xc2f,
0xc42, 0xc32, 0xc46, 0xc28, 0xc4d, 0xc2e, 0xc3e, 0xc39, 0xc46, 0xc2e, 0xc3e, 0xc32,
0xc4d, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0xc41, 0xc2e, 0xc3e, 0xc30, 0xc3f, 0xc37,
0xc38, 0xc4d, 0xc2e, 0xc2f, 0xc4b, 0xc1f, 0xc4d, 0xc30, 0xc40, 0xc2f, 0xc42, 0xc28,
0xc3f, 0xc2f, 0xc28, 0xc4d, 0xc0f, 0xc2a, 0xc3f, 0xc2f, 0xc3e, 0xc06, 0xc15, 0xc4d,
0xc32, 0xc3e, 0xc02, 0xc21, 0xc4d, 0xc2c, 0xc4a, 0xc17, 0xc46, 0xc2f, 0xc3f, 0xc28,
0xc4d, 0x200c, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc47, 0xc1a, 0xc3e, 0xc25, 0xc2e,
0xc4d, 0xc1a, 0xc41, 0xc15, 0xc4d, 0xc08, 0xc38, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0xc07,
0xc2b, 0xc47, 0xc1f, 0xc4d, 0xc2b, 0xc3e, 0xc15, 0xc4b, 0xc2b, 0xc4b, 0xc2b, 0xc40,
0xc1c, 0xc40, 0xc2b, 0xc41, 0xc28, 0xc3e, 0xc2b, 0xc41, 0xc1f, 0xc3f, 0xc17, 0xc3e,
0xc32, 0xc3e, 0xc2a, 0xc3e, 0xc17, 0xc4b, 0xc38, 0xc4d, 0xc17, 0xc3e, 0xc02, 0xc2c,
0xc3f, 0xc2f, 0xc47, 0xc30, 0xc4d, 0xc17, 0xc4d, 0xc35, 0xc3e, 0xc21, 0xc32, 0xc4d,
0xc15, 0xc46, 0xc28, 0xc3e, 0xc32, 0xc4d, 0xc17, 0xc4d, 0xc35, 0xc3e, 0xc2e, 0xc4d,
0xc39, 0xc4b, 0xc28, 0xc4b, 0xc32, 0xc41, 0xc32, 0xc41, 0xc0e, 0xc02, 0xc21, 0xc30,
0xc4d, 0xc2c, 0xc46, 0xc30, 0xc40, 0xc15, 0xc3f, 0xc30, 0xc40, 0xc1f, 0xc3f, 0xc2e,
0xc3e, 0xc1f, 0xc3f, 0xc15, 0xc4b, 0xc38, 0xc4d, 0xc30, 0xc47, 0xc15, 0xc4d, 0xc35,
0xc3e, 0xc1c, 0xc3e, 0xc32, 0xc48, 0xc28, 0xc4d, 0xc2e, 0xc1c, 0xc41, 0xc30, 0xc4b,
0xc2e, 0xc3e, 0xc30, 0xc4d, 0xc15, 0xc4d, 0xc35, 0xc47, 0xc38, 0xc3e, 0xc38, 0xc4d,
0xc2e, 0xc3f, 0xc21, 0xc4d, 0xc35, 0xc47, 0xc28, 0xc4c, 0xc30, 0xc41, 0xc28, 0xc3f,
0xc2f, 0xc42, 0xc28, 0xc4b, 0xc30, 0xc4d, 0xc2b, 0xc4b, 0xc15, 0xc4d, 0xc28, 0xc4c,
0xc2e, 0xc3f, 0xc2f, 0xc3e, 0xc2a, 0xc3e, 0xc17, 0xc4b, 0x20, 0xc2a, 0xc3e, 0xc17,
0xc4b, 0xc2a, 0xc3e, 0xc32, 0xc3e, 0xc35, 0xc4d, 0xc2a, 0xc3f, 0xc1f, 0xc4d, 0x200c,
0xc15, 0xc48, 0xc30, 0xc4d, 0xc28, 0xc4d, 0xc2a, 0xc4b, 0xc28, 0xc4d, 0x200c, 0xc2a,
0xc48, 0xc2a, 0xc4b, 0xc30, 0xc4d, 0xc1f, 0xc4d, 0x20, 0xc2e, 0xc4b, 0xc30, 0xc46,
0xc38, 0xc4d, 0xc2c, 0xc47, 0xc30, 0xc30, 0xc4b, 0xc1f, 0xc4b, 0xc02, 0xc17, 0xc3e,
0xc38, 0xc3e, 0xc2f, 0xc4d, 0xc2a, 0xc3e, 0xc28, 0xc4d, 0xc24, 0xc39, 0xc3f, 0xc24,
0xc3f, 0xc1f, 0xc30, 0xc3e, 0xc35, 0xc3e, 0xc1f, 0xc4b, 0xc02, 0xc17, 0xc3e, 0xc1f,
0xc3e, 0xc2a, 0xc41, 0xc35, 0xc47, 0xc15, 0xc4d, 0xc35, 0xc3e, 0xc32, 0xc4d, 0xc32,
0xc3f, 0xc38, 0xc4d, 0xe2d, 0xe32, 0xe1a, 0xe35, 0xe08, 0xe32, 0xe19, 0xe2d, 0xe31,
0xe01, 0xe01, 0xe23, 0xe32, 0xe41, 0xe2d, 0xe14, 0xe14, 0xe34, 0xe2a, 0xe2d, 0xe32,
0xe1a, 0xe32, 0xe1a, 0xe32, 0xe41, 0xe2d, 0xe25, 0xe40, 0xe08, 0xe35, 0xe22, 0xe23,
0xe4c, 0xe41, 0xe2d, 0xe2a, 0xe21, 0xe32, 0xe23, 0xe32, 0xe1a, 0xe32, 0xe21, 0xe32,
0xe42, 0xe01, 0xe1a, 0xe31, 0xe07, 0xe01, 0xe35, 0xe1a, 0xe31, 0xe19, 0xe08, 0xe39,
0xe25, 0xe1a, 0xe34, 0xe2a, 0xe40, 0xe0b, 0xe32, 0xe41, 0xe1a, 0xe25, 0xe19, 0xe44,
0xe17, 0xe23, 0xe4c, 0xe1a, 0xe23, 0xe32, 0xe0b, 0xe0b, 0xe32, 0xe27, 0xe34, 0xe25,
0xe1a, 0xe39, 0xe08, 0xe38, 0xe21, 0xe1a, 0xe39, 0xe23, 0xe32, 0xe44, 0xe04, 0xe42,
0xe23, 0xe04, 0xe32, 0xe2a, 0xe0b, 0xe32, 0xe1a, 0xe25, 0xe32, 0xe07, 0xe01, 0xe49,
0xe32, 0xe40, 0xe0b, 0xe27, 0xe15, 0xe32, 0xe42, 0xe01, 0xe19, 0xe32, 0xe01, 0xe23,
0xe35, 0xe14, 0xe32, 0xe01, 0xe32, 0xe23, 0xe4c, 0xe14, 0xe32, 0xe23, 0xe4c, 0xe40,
0xe2d, 0xe2a, 0xe0b, 0xe32, 0xe25, 0xe32, 0xe21, 0xe08, 0xe34, 0xe1a, 0xe39, 0xe15,
0xe35, 0xe14, 0xe39, 0xe2d, 0xe32, 0xe25, 0xe32, 0xe40, 0xe2d, 0xe25, 0xe44, 0xe2d,
0xe22, 0xe4c, 0xe2d, 0xe38, 0xe07, 0xe1f, 0xe23, 0xe35, 0xe17, 0xe32, 0xe27, 0xe19,
0xe4c, 0xe01, 0xe32, 0xe42, 0xe1a, 0xe42, 0xe23, 0xe40, 0xe19, 0xe2e, 0xe32, 0xe23,
0xe32, 0xe40, 0xe23, 0xe42, 0xe08, 0xe2e, 0xe31, 0xe19, 0xe40, 0xe19, 0xe2a, 0xe40,
0xe1a, 0xe2d, 0xe23, 0xe4c, 0xe01, 0xe08, 0xe39, 0xe1a, 0xe32, 0xe04, 0xe31, 0xe21,
0xe1e, 0xe32, 0xe25, 0xe32, 0xe04, 0xe32, 0xe23, 0xe4c, 0xe17, 0xe39, 0xe21, 0xe04,
0xe34, 0xe01, 0xe32, 0xe25, 0xe35, 0xe01, 0xe34, 0xe19, 0xe0a, 0xe32, 0xe0b, 0xe32,
0xe25, 0xe32, 0xe01, 0xe2d, 0xe2a, 0xe25, 0xe35, 0xe40, 0xe1a, 0xe23, 0xe2d, 0xe27,
0xe34, 0xe25, 0xe42, 0xe25, 0xe40, 0xe21, 0xe25, 0xe39, 0xe2d, 0xe31, 0xe19, 0xe14,
0xe32, 0xe25, 0xe39, 0xe1a, 0xe31, 0xe21, 0xe1a, 0xe32, 0xe0a, 0xe34, 0xe25, 0xe39,
0xe0b, 0xe32, 0xe01, 0xe32, 0xe21, 0xe32, 0xe25, 0xe32, 0xe42, 0xe1a, 0xe21, 0xe32,
0xe1b, 0xe39, 0xe42, 0xe15, 0xe21, 0xe32, 0xe40, 0xe0b, 0xe23, 0xe39, 0xe2d, 0xe31,
0xe21, 0xe1a, 0xe32, 0xe1a, 0xe32, 0xe40, 0xe19, 0xe42, 0xe21, 0xe01, 0xe32, 0xe14,
0xe34, 0xe0a, 0xe39, 0xe21, 0xe31, 0xe19, 0xe42, 0xe23, 0xe40, 0xe27, 0xe35, 0xe22,
0xe44, 0xe19, 0xe42, 0xe23, 0xe40, 0xe1a, 0xe35, 0xe22, 0xe40, 0xe2d, 0xe47, 0xe19,
0xe08, 0xe32, 0xe40, 0xe21, 0xe19, 0xe32, 0xe19, 0xe35, 0xe2d, 0xe32, 0xe40, 0xe21,
0xe22, 0xe4c, 0xe19, 0xe39, 0xe41, 0xe2d, 0xe01, 0xe0a, 0xe2d, 0xe15, 0xe27, 0xe32,
0xe01, 0xe32, 0xe14, 0xe39, 0xe01, 0xe39, 0xe1b, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe15,
0x2d, 0xe42, 0xe19, 0xe42, 0xe27, 0xe40, 0xe0b, 0xe32, 0xe15, 0xe39, 0xe40, 0xe21,
0xe15, 0xe23, 0xe35, 0xe42, 0xe1b, 0xe25, 0xe35, 0xe15, 0xe39, 0xe19, 0xe34, 0xe2a,
0xe27, 0xe34, 0xe19, 0xe14, 0xe4c, 0xe2e, 0xe38, 0xe01, 0xe40, 0xe2d, 0xe14, 0xe31,
0xe01, 0xe41, 0xe2d, 0xe07, 0xe40, 0xe04, 0xe2d, 0xe40, 0xe23, 0xe08, 0xe41, 0xe2d,
0xe07, 0xe01, 0xe34, 0xe25, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe19, 0xe15, 0xe34, 0xe01,
0xe32, 0xe2d, 0xe32, 0xe23, 0xe32, 0xe01, 0xe31, 0xe27, 0xe19, 0xe32, 0xe1a, 0xe31,
0xe27, 0xe42, 0xe19, 0xe2a, 0xe44, 0xe2d, 0xe40, 0xe23, 0xe2a, 0xe01, 0xe32, 0xe15,
0xe32, 0xe21, 0xe32, 0xe23, 0xe4c, 0xe01, 0xe32, 0xe04, 0xe2d, 0xe23, 0xe4c, 0xe42,
0xe14, 0xe1a, 0xe32, 0xe08, 0xe39, 0xe08, 0xe34, 0xe27, 0xe25, 0xe32, 0xe23, 0xe34,
0xe42, 0xe2d, 0xe08, 0xe32, 0xe40, 0xe21, 0xe19, 0xe14, 0xe39, 0xe0b, 0xe32, 0xe23,
0xe34, 0xe42, 0xe2d, 0xe01, 0xe32, 0xe25, 0xe40, 0xe25, 0xe01, 0xe2d, 0xe2a, 0xe0b,
0xe31, 0xe25, 0xe15, 0xe32, 0xe0b, 0xe32, 0xe19, 0xe2e, 0xe27, 0xe19, 0xe0b, 0xe31,
0xe19, 0xe25, 0xe39, 0xe2d, 0xe34, 0xe2a, 0xe17, 0xe39, 0xe04, 0xe39, 0xe41, 0xe21,
0xe19, 0xe2d, 0xe39, 0xe0a, 0xe39, 0xe40, 0xe2d, 0xe35, 0xe22, 0xe2d, 0xe32, 0xe23,
0xe39, 0xe1a, 0xe32, 0xe2d, 0xe30, 0xe0b, 0xe38, 0xe19, 0xe0b, 0xe34, 0xe2d, 0xe2d,
0xe07, 0xe04, 0xe2d, 0xe23, 0xe31, 0xe25, 0xe2e, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe1a,
0xe2d, 0xe23, 0xe4c, 0xe1a, 0xe32, 0xe40, 0xe22, 0xe35, 0xe22, 0xe1a, 0xe32, 0xe40,
0xe2d, 0xe35, 0xe22, 0xe1a, 0xe31, 0xe19, 0xe40, 0xe14, 0xe23, 0xe31, 0xe2a, 0xe1a,
0xe32, 0xe23, 0xe4c, 0xe40, 0xe1a, 0xe42, 0xe14, 0xe2a, 0xe40, 0xe1a, 0xe40, 0xe25,
0xe07, 0xe40, 0xe1a, 0xe25, 0xe35, 0xe0b, 0xe1a, 0xe25, 0xe31, 0xe07, 0xe04, 0xe4c,
0x2d, 0xe0b, 0xe32, 0xe1a, 0xe25, 0xe2d, 0xe19, 0xe1a, 0xe31, 0xe27, 0xe27, 0xe35,
0xe0a, 0xe15, 0xe32, 0xe42, 0xe1a, 0xe42, 0xe01, 0xe15, 0xe32, 0xe1a, 0xe2d, 0xe22,
0xe0b, 0xe35, 0xe2d, 0xe48, 0xe32, 0xe27, 0xe41, 0xe04, 0xe21, 0xe1a, 0xe23, 0xe34,
0xe14, 0xe08, 0xe4c, 0xe01, 0xe31, 0xe21, 0xe1b, 0xe39, 0xe01, 0xe23, 0xe31, 0xe19,
0xe14, 0xe35, 0xe41, 0xe04, 0xe19, 0xe04, 0xe38, 0xe19, 0xe04, 0xe32, 0xe23, 0xe32,
0xe04, 0xe31, 0xe2a, 0xe01, 0xe32, 0xe41, 0xe22, 0xe19, 0xe40, 0xe04, 0xe22, 0xe4c,
0xe41, 0xe21, 0xe19, 0xe0a, 0xe34, 0xe04, 0xe32, 0xe42, 0xe01, 0xe0a, 0xe35, 0xe27,
0xe32, 0xe27, 0xe32, 0xe0b, 0xe34, 0xe27, 0xe14, 0xe31, 0xe14, 0xe2e, 0xe31, 0xe27,
0xe40, 0xe23, 0xe0b, 0xe04, 0xe2d, 0xe2a, 0xe15, 0xe32, 0xe23, 0xe34, 0xe01, 0xe32,
0xe40, 0xe04, 0xe23, 0xe2a, 0xe15, 0xe31, 0xe19, 0xe01, 0xe38, 0xe22, 0xe32, 0xe1a,
0xe32, 0xe04, 0xe39, 0xe23, 0xe32, 0xe40, 0xe0b, 0xe32, 0xe14, 0xe32, 0xe19, 0xe21,
0xe32, 0xe23, 0xe4c, 0xe01, 0xe2a, 0xe2e, 0xe32, 0xe27, 0xe19, 0xe4c, 0xe14, 0xe2d,
0xe27, 0xe4c, 0xe2a, 0xe31, 0xe19, 0xe14, 0xe2d, 0xe27, 0xe4c, 0xe2a, 0xe31, 0xe19,
0x20, 0xe04, 0xe23, 0xe35, 0xe01, 0xe40, 0xe14, 0xe19, 0xe40, 0xe27, 0xe2d, 0xe23,
0xe4c, 0xe14, 0xe35, 0xe17, 0xe23, 0xe2d, 0xe22, 0xe15, 0xe4c, 0xe42, 0xe14, 0xe21,
0xe34, 0xe19, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2d, 0xe14, 0xe21, 0xe31, 0xe19, 0xe15,
0xe31, 0xe19, 0xe40, 0xe2d, 0xe23, 0xe39, 0xe40, 0xe19, 0xe40, 0xe1b, 0xe40, 0xe2d,
0xe25, 0xe0b, 0xe31, 0xe25, 0xe27, 0xe32, 0xe14, 0xe2d, 0xe23, 0xe4c, 0xe1f, 0xe2d,
0xe23, 0xe4c, 0xe15, 0xe40, 0xe19, 0xe25, 0xe2a, 0xe31, 0xe19, 0xe1f, 0xe2d, 0xe23,
0xe4c, 0xe15, 0xe32, 0xe40, 0xe25, 0xe0b, 0xe32, 0xe40, 0xe01, 0xe25, 0xe0b, 0xe40,
0xe1a, 0xe22, 0xe4c, 0xe01, 0xe39, 0xe2a, 0xe40, 0xe1a, 0xe22, 0xe4c, 0xe41, 0xe01,
0xe23, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe15, 0xe34, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe01,
0xe23, 0xe19, 0xe32, 0xe14, 0xe32, 0xe01, 0xe27, 0xe32, 0xe40, 0xe14, 0xe2d, 0xe25,
0xe39, 0xe1b, 0xe01, 0xe31, 0xe27, 0xe40, 0xe15, 0xe21, 0xe32, 0xe25, 0xe32, 0xe01,
0xe31, 0xe27, 0xe22, 0xe32, 0xe01, 0xe34, 0xe25, 0xe01, 0xe32, 0xe22, 0xe2d, 0xe32,
0xe19, 0xe32, 0xe41, 0xe2e, 0xe25, 0xe34, 0xe41, 0xe1f, 0xe01, 0xe0b, 0xe4c, 0xe2e,
0xe32, 0xe27, 0xe32, 0xe19, 0xe32, 0xe40, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe21, 0xe0b,
0xe35, 0xe42, 0xe22, 0xe2d, 0xe34, 0xe19, 0xe40, 0xe14, 0xe35, 0xe22, 0xe41, 0xe19,
0xe42, 0xe1e, 0xe25, 0xe34, 0xe2a, 0xe19, 0xe2d, 0xe01, 0xe0b, 0xe4c, 0x2c, 0x20,
0xe2d, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe2d, 0xe19, 0xe32, 0xe21, 0xe32, 0xe40,
0xe23, 0xe07, 0xe42, 0xe01, 0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41,
0xe2d, 0xe19, 0xe32, 0xe1b, 0xe35, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe2a, 0xe40,
0xe1a, 0xe34, 0xe23, 0xe4c, 0xe01, 0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35,
0xe41, 0xe2d, 0xe19, 0xe32, 0xe40, 0xe17, 0xe25, 0xe25, 0xe4c, 0xe0b, 0xe34, 0xe15,
0xe35, 0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe2d, 0xe19, 0xe32,
0xe27, 0xe35, 0xe40, 0xe27, 0xe22, 0xe4c, 0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14,
0xe35, 0xe41, 0xe2d, 0xe19, 0xe32, 0xe27, 0xe34, 0xe19, 0xe40, 0xe0b, 0xe19, 0xe40,
0xe19, 0xe2a, 0x2c, 0x20, 0xe2d, 0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe2d, 0xe19,
0xe32, 0xe27, 0xe34, 0xe19, 0xe32, 0xe41, 0xe21, 0xe04, 0x2c, 0x20, 0xe2d, 0xe34,
0xe19, 0xe14, 0xe35, 0xe41, 0xe2d, 0xe19, 0xe32, 0xe2d, 0xe34, 0xe19, 0xe39, 0xe27,
0xe34, 0xe01, 0xe2d, 0xe35, 0xe01, 0xe27, 0xe32, 0xe25, 0xe34, 0xe15, 0xe08, 0xe32,
0xe40, 0xe21, 0xe01, 0xe32, 0xe08, 0xe39, 0xe42, 0xe19, 0xe25, 0xe39, 0xe2a, 0xe4c,
0xe27, 0xe34, 0xe25, 0xe25, 0xe4c, 0xe21, 0xe2d, 0xe19, 0xe15, 0xe34, 0xe40, 0xe0b,
0xe25, 0xe42, 0xe25, 0x2c, 0x20, 0xe40, 0xe04, 0xe19, 0xe15, 0xe31, 0xe01, 0xe01,
0xe35, 0xe04, 0xe23, 0xe32, 0xe40, 0xe25, 0xe19, 0xe14, 0xe34, 0xe08, 0xe04, 0xe4c,
0xe25, 0xe32, 0xe1b, 0xe32, 0xe0b, 0xe25, 0xe34, 0xe21, 0xe32, 0xe25, 0xe2d, 0xe2a,
0xe41, 0xe2d, 0xe07, 0xe40, 0xe08, 0xe25, 0xe34, 0xe2a, 0xe42, 0xe25, 0xe40, 0xe27,
0xe2d, 0xe23, 0xe4c, 0xe1e, 0xe23, 0xe34, 0xe19, 0xe0b, 0xe4c, 0x20, 0xe04, 0xe27,
0xe2d, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe21, 0xe32, 0xe40, 0xe0b, 0xe42, 0xe2d,
0xe21, 0xe32, 0xe19, 0xe32, 0xe01, 0xe31, 0xe27, 0xe21, 0xe32, 0xe40, 0xe19, 0xe32,
0xe2a, 0xe4c, 0xe21, 0xe32, 0xe23, 0xe34, 0xe42, 0xe01, 0xe15, 0xe4c, 0xe21, 0xe32,
0xe23, 0xe4c, 0xe15, 0xe34, 0xe19, 0xe35, 0xe01, 0xe21, 0xe32, 0xe15, 0xe32, 0xe42,
0xe21, 0xe23, 0xe2d, 0xe2a, 0xe21, 0xe32, 0xe0b, 0xe32, 0xe17, 0xe25, 0xe32, 0xe19,
0xe40, 0xe21, 0xe42, 0xe19, 0xe21, 0xe34, 0xe19, 0xe35, 0xe40, 0xe21, 0xe23, 0xe35,
0xe14, 0xe32, 0xe40, 0xe21, 0xe17, 0xe25, 0xe32, 0xe01, 0xe32, 0xe15, 0xe25, 0xe30,
0xe40, 0xe21, 0xe47, 0xe01, 0xe0b, 0xe34, 0xe42, 0xe01, 0xe0b, 0xe34, 0xe15, 0xe35,
0xe21, 0xe35, 0xe40, 0xe01, 0xe2d, 0xe25, 0xe07, 0xe21, 0xe2d, 0xe07, 0xe15, 0xe31,
0xe19, 0xe21, 0xe2d, 0xe19, 0xe40, 0xe15, 0xe23, 0xe4c, 0xe40, 0xe23, 0xe22, 0xe4c,
0xe21, 0xe2d, 0xe19, 0xe40, 0xe15, 0xe27, 0xe34, 0xe40, 0xe14, 0xe42, 0xe2d, 0xe21,
0xe2d, 0xe19, 0xe40, 0xe0b, 0xe2d, 0xe23, 0xe4c, 0xe23, 0xe31, 0xe15, 0xe41, 0xe19,
0xe2a, 0xe0b, 0xe2d, 0xe19, 0xe34, 0xe27, 0xe22, 0xe2d, 0xe23, 0xe4c, 0xe01, 0xe19,
0xe2d, 0xe21, 0xe42, 0xe19, 0xe23, 0xe2d, 0xe19, 0xe2e, 0xe32, 0xe42, 0xe1a, 0xe25,
0xe32, 0xe2b, 0xe4c, 0x2c, 0x20, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe17, 0xe14, 0xe32,
0xe42, 0xe04, 0xe15, 0xe32, 0xe40, 0xe0b, 0xe19, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c,
0x2c, 0x20, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe17, 0xe14, 0xe32, 0xe42, 0xe04, 0xe15,
0xe32, 0xe19, 0xe34, 0xe27, 0xe40, 0xe0b, 0xe40, 0xe25, 0xe21, 0x2c, 0x20, 0xe19,
0xe2d, 0xe23, 0xe4c, 0xe17, 0xe14, 0xe32, 0xe42, 0xe04, 0xe15, 0xe32, 0xe01, 0xe2d,
0xe14, 0xe41, 0xe18, 0xe1a, 0xe42, 0xe2d, 0xe08, 0xe34, 0xe19, 0xe32, 0xe01, 0xe32,
0xe1b, 0xe32, 0xe19, 0xe32, 0xe21, 0xe32, 0xe1b, 0xe32, 0xe23, 0xe32, 0xe21, 0xe32,
0xe23, 0xe34, 0xe42, 0xe1a, 0xe1f, 0xe34, 0xe19, 0xe34, 0xe01, 0xe0b, 0xe4c, 0xe1b,
0xe2d, 0xe23, 0xe4c, 0xe42, 0xe15, 0xe41, 0xe1b, 0xe23, 0xe07, 0xe0b, 0xe4c, 0xe1e,
0xe2d, 0xe23, 0xe4c, 0xe17, 0xe2d, 0xe2d, 0xe1f, 0xe2a, 0xe40, 0xe1b, 0xe19, 0xe1b,
0xe2d, 0xe23, 0xe4c, 0xe15, 0xe39, 0xe40, 0xe27, 0xe25, 0xe42, 0xe22, 0xe40, 0xe1b,
0xe2d, 0xe42, 0xe15, 0xe23, 0xe34, 0xe42, 0xe01, 0xe1b, 0xe38, 0xe19, 0xe15, 0xe32,
0xe2d, 0xe32, 0xe40, 0xe23, 0xe19, 0xe31, 0xe2a, 0xe41, 0xe23, 0xe07, 0xe01, 0xe34,
0xe19, 0xe2d, 0xe34, 0xe19, 0xe40, 0xe25, 0xe47, 0xe15, 0xe40, 0xe23, 0xe0b, 0xe35,
0xe40, 0xe1f, 0xe23, 0xe34, 0xe44, 0xe08, 0xe19, 0xe32, 0xe40, 0xe23, 0xe42, 0xe0b,
0xe25, 0xe39, 0xe17, 0xe23, 0xe35, 0xe42, 0xe2d, 0xe1a, 0xe23, 0xe31, 0xe07, 0xe42,
0xe01, 0xe0b, 0xe31, 0xe19, 0xe15, 0xe32, 0xe40, 0xe23, 0xe21, 0xe0b, 0xe31, 0xe19,
0xe15, 0xe34, 0xe2d, 0xe32, 0xe42, 0xe01, 0xe0b, 0xe32, 0xe19, 0xe42, 0xe15, 0x20,
0xe42, 0xe14, 0xe21, 0xe34, 0xe07, 0xe42, 0xe01, 0xe40, 0xe0b, 0xe32, 0xe40, 0xe1b,
0xe32, 0xe25, 0xe39, 0xe2a, 0xe01, 0xe2d, 0xe40, 0xe23, 0xe2a, 0xe44, 0xe1a, 0xe0b,
0xe31, 0xe19, 0xe14, 0xe4c, 0xe0b, 0xe34, 0xe15, 0xe01, 0xe32, 0xe40, 0xe0b, 0xe19,
0xe15, 0xe4c, 0xe1a, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe18, 0xe40, 0xe25, 0xe21, 0xe35,
0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c, 0xe08, 0xe2d, 0xe19, 0xe2a, 0xe4c, 0xe40, 0xe0b,
0xe19, 0xe15, 0xe4c, 0xe04, 0xe34, 0xe15, 0xe2a, 0xe4c, 0xe40, 0xe0b, 0xe19, 0xe15,
0xe4c, 0xe25, 0xe39, 0xe40, 0xe0b, 0xe35, 0xe22, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c,
0xe42, 0xe18, 0xe21, 0xe31, 0xe2a, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c, 0xe27, 0xe34,
0xe19, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c, 0xe2a, 0xe27, 0xe34, 0xe1f, 0xe15, 0xe4c,
0xe40, 0xe04, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe23, 0xe19, 0xe15, 0xe4c, 0xe40, 0xe15,
0xe01, 0xe39, 0xe0b, 0xe34, 0xe01, 0xe31, 0xe25, 0xe1b, 0xe32, 0xe17, 0xe39, 0xe40,
0xe25, 0xe17, 0xe34, 0xe2e, 0xe31, 0xe27, 0xe19, 0xe32, 0xe42, 0xe17, 0xe23, 0xe2d,
0xe19, 0xe42, 0xe15, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe15, 0xe25, 0xe32, 0xe41,
0xe27, 0xe19, 0xe04, 0xe39, 0xe40, 0xe27, 0xe2d, 0xe23, 0xe4c, 0xe44, 0xe27, 0xe15,
0xe4c, 0xe2e, 0xe2d, 0xe23, 0xe4c, 0xe2a, 0xe27, 0xe34, 0xe19, 0xe19, 0xe34, 0xe40,
0xe1e, 0xe01, 0xe22, 0xe32, 0xe01, 0xe39, 0xe17, 0xe31, 0xe15, 0xe40, 0xe04, 0xe0b,
0xe35, 0xe22, 0xe4c, 0xe40, 0xe14, 0xe27, 0xe34, 0xe2a, 0xe14, 0xe39, 0xe21, 0xe2d,
0xe07, 0xe15, 0xe4c, 0xe14, 0xe39, 0xe23, 0xe4c, 0xe27, 0xe34, 0xe25, 0xe25, 0xe4c,
0xe41, 0xe21, 0xe04, 0xe04, 0xe27, 0xe2d, 0xe23, 0xe35, 0xe21, 0xe2d, 0xe27, 0xe4c,
0xe2a, 0xe31, 0xe19, 0xe41, 0xe21, 0xe04, 0xe21, 0xe31, 0xe27, 0xe42, 0xe14, 0xe1e,
0xe32, 0xe25, 0xe4c, 0xe40, 0xe21, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe23, 0xe18, 0xe35,
0xe23, 0xe32, 0xe44, 0xe0b, 0xe42, 0xe22, 0xe27, 0xe32, 0xe42, 0xe17, 0xe23, 0xe25,
0xe27, 0xe2d, 0xe2a, 0xe15, 0xe2d, 0xe04, 0xe25, 0xe2d, 0xe07, 0xe40, 0xe22, 0xe35,
0xe22, 0xe23, 0xe4c, 0xe40, 0xe1a, 0xe35, 0xe22, 0xe19, 0xe40, 0xe2d, 0xe40, 0xe14,
0xe19, 0xe2d, 0xe31, 0xe25, 0xe21, 0xe32, 0xe15, 0xe35, 0xe2d, 0xe31, 0xe21, 0xe21,
0xe32, 0xe19, 0xe2d, 0xe32, 0xe19, 0xe32, 0xe14, 0xe35, 0xe23, 0xe4c, 0xe2d, 0xe31,
0xe04, 0xe15, 0xe32, 0xe2d, 0xe39, 0xe2d, 0xe31, 0xe04, 0xe42, 0xe17, 0xe1a, 0xe35,
0xe2d, 0xe32, 0xe0a, 0xe01, 0xe32, 0xe1a, 0xe31, 0xe15, 0xe2d, 0xe17, 0xe35, 0xe23,
0xe32, 0xe27, 0xe41, 0xe1a, 0xe01, 0xe41, 0xe14, 0xe14, 0xe1a, 0xe32, 0xe2b, 0xe4c,
0xe40, 0xe23, 0xe19, 0xe1a, 0xe32, 0xe01, 0xe39, 0xe01, 0xe23, 0xe38, 0xe07, 0xe40,
0xe17, 0xe1e, 0xe1a, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe19, 0xe32, 0xe27, 0xe4c, 0xe40,
0xe1a, 0xe23, 0xe38, 0xe15, 0xe1a, 0xe34, 0xe0a, 0xe40, 0xe04, 0xe01, 0xe1a, 0xe23,
0xe39, 0xe44, 0xe19, 0xe0a, 0xe34, 0xe15, 0xe32, 0xe0a, 0xe2d, 0xe22, 0xe1a, 0xe32,
0xe25, 0xe0b, 0xe32, 0xe19, 0xe42, 0xe04, 0xe25, 0xe31, 0xe21, 0xe42, 0xe1a, 0xe14,
0xe32, 0xe21, 0xe31, 0xe2a, 0xe01, 0xe31, 0xe2a, 0xe14, 0xe34, 0xe25, 0xe35, 0xe14,
0xe39, 0xe44, 0xe1a, 0xe14, 0xe39, 0xe0a, 0xe32, 0xe19, 0xe40, 0xe1a, 0xe41, 0xe1f,
0xe21, 0xe32, 0xe01, 0xe38, 0xe2a, 0xe15, 0xe32, 0xe01, 0xe32, 0xe0b, 0xe32, 0xe40,
0xe2e, 0xe1a, 0xe23, 0xe2d, 0xe19, 0xe19, 0xe04, 0xe23, 0xe42, 0xe2e, 0xe08, 0xe34,
0xe21, 0xe34, 0xe19, 0xe2b, 0xe4c, 0xe2e, 0xe48, 0xe2d, 0xe07, 0xe01, 0xe07, 0xe2e,
0xe2d, 0xe1f, 0xe14, 0xe4c, 0xe2d, 0xe35, 0xe23, 0xe4c, 0xe04, 0xe38, 0xe15, 0xe2a,
0xe04, 0xe4c, 0xe08, 0xe32, 0xe01, 0xe32, 0xe23, 0xe4c, 0xe15, 0xe32, 0xe08, 0xe32,
0xe22, 0xe32, 0xe1b, 0xe38, 0xe23, 0xe30, 0xe40, 0xe22, 0xe23, 0xe39, 0xe0b, 0xe32,
0xe40, 0xe25, 0xe21, 0xe04, 0xe32, 0xe1a, 0xe39, 0xe25, 0xe04, 0xe32, 0xe21, 0xe0a,
0xe31, 0xe15, 0xe01, 0xe32, 0xe01, 0xe32, 0xe23, 0xe32, 0xe08, 0xe35, 0xe01, 0xe32,
0xe15, 0xe21, 0xe31, 0xe19, 0xe14, 0xe38, 0xe2e, 0xe31, 0xe19, 0xe14, 0xe37, 0xe22,
0xe01, 0xe32, 0xe42, 0xe01, 0xe25, 0xe01, 0xe32, 0xe15, 0xe32, 0xe04, 0xe23, 0xe31,
0xe2a, 0xe42, 0xe19, 0xe22, 0xe32, 0xe23, 0xe4c, 0xe2a, 0xe01, 0xe4c, 0xe01, 0xe31,
0xe27, 0xe25, 0xe32, 0xe25, 0xe31, 0xe21, 0xe40, 0xe1b, 0xe2d, 0xe23, 0xe4c, 0xe01,
0xe39, 0xe0a, 0xe34, 0xe07, 0xe04, 0xe39, 0xe40, 0xe27, 0xe15, 0xe21, 0xe32, 0xe40,
0xe01, 0xe4a, 0xe32, 0xe21, 0xe32, 0xe01, 0xe32, 0xe14, 0xe32, 0xe19, 0xe21, 0xe32,
0xe01, 0xe31, 0xe2a, 0xe0b, 0xe32, 0xe23, 0xe4c, 0xe21, 0xe30, 0xe19, 0xe34, 0xe25,
0xe32, 0xe21, 0xe31, 0xe2a, 0xe01, 0xe31, 0xe15, 0xe19, 0xe34, 0xe42, 0xe04, 0xe40,
0xe0b, 0xe35, 0xe22, 0xe42, 0xe19, 0xe42, 0xe27, 0xe04, 0xe38, 0xe0b, 0xe40, 0xe19,
0xe15, 0xe2a, 0xe04, 0xe4c, 0xe42, 0xe19, 0xe42, 0xe27, 0xe0b, 0xe34, 0xe1a, 0xe34,
0xe23, 0xe4c, 0xe2a, 0xe01, 0xe4c, 0xe42, 0xe2d, 0xe21, 0xe2a, 0xe01, 0xe4c, 0xe2d,
0xe2d, 0xe23, 0xe31, 0xe25, 0xe1e, 0xe19, 0xe21, 0xe40, 0xe1b, 0xe0d, 0xe1e, 0xe2d,
0xe19, 0xe40, 0xe17, 0xe35, 0xe22, 0xe19, 0xe31, 0xe01, 0xe40, 0xe1b, 0xe35, 0xe22,
0xe07, 0xe22, 0xe32, 0xe07, 0xe01, 0xe32, 0xe15, 0xe32, 0xe23, 0xe4c, 0xe04, 0xe2d,
0xe2a, 0xe15, 0xe32, 0xe40, 0xe19, 0xe22, 0xe4c, 0xe44, 0xe04, 0xe0b, 0xe35, 0xe25,
0xe2d, 0xe23, 0xe4c, 0xe14, 0xe32, 0xe23, 0xe34, 0xe22, 0xe32, 0xe23, 0xe4c, 0xe14,
0xe0b, 0xe32, 0xe04, 0xe32, 0xe25, 0xe34, 0xe19, 0xe0b, 0xe32, 0xe21, 0xe32, 0xe23,
0xe4c, 0xe01, 0xe32, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe0b, 0xe35, 0xe48, 0xe22, 0xe07,
0xe44, 0xe2e, 0xe49, 0xe2a, 0xe34, 0xe07, 0xe04, 0xe42, 0xe1b, 0xe23, 0xe4c, 0xe0b,
0xe40, 0xe23, 0xe14, 0xe40, 0xe19, 0xe04, 0xe42, 0xe04, 0xe25, 0xe34, 0xe21, 0xe2a,
0xe01, 0xe4c, 0xe44, 0xe17, 0xe40, 0xe1b, 0xe17, 0xe32, 0xe0a, 0xe40, 0xe04, 0xe19,
0xe15, 0xe4c, 0xe17, 0xe1a, 0xe34, 0xe25, 0xe34, 0xe0b, 0xe34, 0xe40, 0xe15, 0xe2b,
0xe30, 0xe23, 0xe32, 0xe19, 0xe17, 0xe34, 0xe21, 0xe1e, 0xe39, 0xe42, 0xe15, 0xe40,
0xe01, 0xe35, 0xe22, 0xe27, 0xe15, 0xe2d, 0xe21, 0xe2a, 0xe04, 0xe4c, 0xe2d, 0xe39,
0xe25, 0xe32, 0xe19, 0xe1a, 0xe32, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe2d, 0xe38, 0xe23,
0xe38, 0xe21, 0xe0a, 0xe35, 0xe2d, 0xe38, 0xe2a, 0xe15, 0xe4c, 0xe40, 0xe19, 0xe23,
0xe32, 0xe40, 0xe27, 0xe35, 0xe22, 0xe07, 0xe08, 0xe31, 0xe19, 0xe17, 0xe19, 0xe4c,
0xe27, 0xe25, 0xe32, 0xe14, 0xe34, 0xe42, 0xe27, 0xe2a, 0xe15, 0xe47, 0xe2d, 0xe01,
0xe22, 0xe32, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe22, 0xe48, 0xe32, 0xe07,
0xe01, 0xe38, 0xe49, 0xe07, 0xe22, 0xe35, 0xe04, 0xe32, 0xe40, 0xe15, 0xe2d, 0xe23,
0xe34, 0xe19, 0xe40, 0xe1a, 0xe34, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe22, 0xe40, 0xe23,
0xe27, 0xe32, 0xe19, 0xe2d, 0xe30, 0xe42, 0xe0b, 0xe23, 0xe4c, 0xe2a, 0xe40, 0xe1a,
0xe2d, 0xe23, 0xe4c, 0xe21, 0xe34, 0xe27, 0xe14, 0xe32, 0xe04, 0xe30, 0xe40, 0xe19,
0xe23, 0xe35, 0xe40, 0xe04, 0xe1b, 0xe40, 0xe27, 0xe34, 0xe23, 0xe4c, 0xe14, 0xe41,
0xe1f, 0xe42, 0xe23, 0xe21, 0xe32, 0xe40, 0xe14, 0xe23, 0xe32, 0xe40, 0xe23, 0xe04,
0xe22, 0xe32, 0xe27, 0xe34, 0xe01, 0xe40, 0xe0b, 0xe32, 0xe17, 0xe4c, 0x20, 0xe08,
0xe2d, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe35, 0xe22, 0xe40, 0xe0b, 0xe19, 0xe15, 0xe4c,
0xe40, 0xe2e, 0xe40, 0xe25, 0xe19, 0xe32, 0xe2a, 0xe41, 0xe15, 0xe19, 0xe25, 0xe35,
0xe22, 0xe4c, 0xe41, 0xe2d, 0xe14, 0xe34, 0xe40, 0xe25, 0xe14, 0xe1a, 0xe23, 0xe34,
0xe2a, 0xe40, 0xe1a, 0xe19, 0xe42, 0xe1a, 0xe23, 0xe01, 0xe40, 0xe04, 0xe19, 0xe2e,
0xe34, 0xe25, 0xe25, 0xe4c, 0xe14, 0xe32, 0xe23, 0xe4c, 0xe27, 0xe34, 0xe19, 0xe22,
0xe39, 0xe04, 0xe25, 0xe32, 0xe42, 0xe2e, 0xe1a, 0xe32, 0xe23, 0xe4c, 0xe15, 0xe25,
0xe34, 0xe19, 0xe14, 0xe35, 0xe41, 0xe21, 0xe19, 0xe25, 0xe2d, 0xe23, 0xe4c, 0xe14,
0xe42, 0xe2e, 0xe27, 0xe4c, 0xe40, 0xe21, 0xe25, 0xe40, 0xe1a, 0xe34, 0xe23, 0xe4c,
0xe19, 0xe40, 0xe1e, 0xe34, 0xe23, 0xe4c, 0xe17, 0xe0b, 0xe34, 0xe14, 0xe19, 0xe35,
0xe22, 0xe4c, 0xe2d, 0xe31, 0xe21, 0xe2a, 0xe40, 0xe15, 0xe2d, 0xe14, 0xe31, 0xe21,
0xe2d, 0xe31, 0xe19, 0xe14, 0xe2d, 0xe23, 0xe4c, 0xe23, 0xe32, 0xe41, 0xe2d, 0xe2a,
0xe15, 0xe23, 0xe32, 0xe04, 0xe32, 0xe19, 0xe40, 0xe2d, 0xe40, 0xe18, 0xe19, 0xe2a,
0xe4c, 0xe40, 0xe1a, 0xe25, 0xe40, 0xe01, 0xe23, 0xe14, 0xe40, 0xe1a, 0xe2d, 0xe23,
0xe4c, 0xe25, 0xe34, 0xe19, 0xe1a, 0xe23, 0xe32, 0xe15, 0xe34, 0xe2a, 0xe25, 0xe32,
0xe27, 0xe32, 0xe1a, 0xe23, 0xe31, 0xe2a, 0xe40, 0xe0b, 0xe25, 0xe2a, 0xe4c, 0xe1a,
0xe39, 0xe04, 0xe32, 0xe40, 0xe23, 0xe2a, 0xe1a, 0xe39, 0xe14, 0xe32, 0xe40, 0xe1b,
0xe2a, 0xe1a, 0xe38, 0xe2a, 0xe0b, 0xe34, 0xe07, 0xe40, 0xe07, 0xe47, 0xe19, 0xe04,
0xe35, 0xe0a, 0xe35, 0xe40, 0xe19, 0xe32, 0xe42, 0xe04, 0xe40, 0xe1b, 0xe19, 0xe40,
0xe2e, 0xe40, 0xe01, 0xe19, 0xe14, 0xe31, 0xe1a, 0xe1a, 0xe25, 0xe34, 0xe19, 0xe22,
0xe34, 0xe1a, 0xe23, 0xe2d, 0xe25, 0xe15, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe01, 0xe34,
0xe23, 0xe4c, 0xe19, 0xe0b, 0xe35, 0xe22, 0xe4c, 0xe40, 0xe2e, 0xe25, 0xe0b, 0xe34,
0xe07, 0xe01, 0xe34, 0xe40, 0xe01, 0xe32, 0xe30, 0xe41, 0xe21, 0xe19, 0xe2d, 0xe34,
0xe2a, 0xe15, 0xe31, 0xe19, 0xe1a, 0xe39, 0xe25, 0xe40, 0xe08, 0xe2d, 0xe23, 0xe4c,
0xe0b, 0xe35, 0xe22, 0xe4c, 0xe04, 0xe32, 0xe25, 0xe34, 0xe19, 0xe34, 0xe07, 0xe01,
0xe23, 0xe32, 0xe14, 0xe04, 0xe34, 0xe23, 0xe2d, 0xe1f, 0xe40, 0xe04, 0xe35, 0xe22,
0xe1f, 0xe25, 0xe34, 0xe2a, 0xe1a, 0xe2d, 0xe19, 0xe25, 0xe39, 0xe1a, 0xe25, 0xe34,
0xe22, 0xe32, 0xe19, 0xe32, 0xe25, 0xe2d, 0xe19, 0xe14, 0xe2d, 0xe19, 0xe25, 0xe31,
0xe01, 0xe40, 0xe0b, 0xe21, 0xe40, 0xe1a, 0xe34, 0xe23, 0xe4c, 0xe01, 0xe21, 0xe32,
0xe14, 0xe23, 0xe34, 0xe14, 0xe21, 0xe2d, 0xe25, 0xe15, 0xe32, 0xe21, 0xe32, 0xe23,
0xe35, 0xe2e, 0xe32, 0xe21, 0xe19, 0xe4c, 0xe21, 0xe34, 0xe19, 0xe2a, 0xe01, 0xe4c,
0xe42, 0xe21, 0xe19, 0xe32, 0xe42, 0xe01, 0xe21, 0xe2d, 0xe2a, 0xe42, 0xe01, 0xe2d,
0xe2d, 0xe2a, 0xe42, 0xe25, 0xe1b, 0xe32, 0xe23, 0xe35, 0xe2a, 0xe1e, 0xe2d, 0xe14,
0xe01, 0xe2d, 0xe23, 0xe35, 0xe15, 0xe0b, 0xe32, 0xe1b, 0xe23, 0xe32, 0xe01, 0xe0b,
0xe32, 0xe21, 0xe32, 0xe23, 0xe32, 0xe0b, 0xe32, 0xe19, 0xe21, 0xe32, 0xe23, 0xe35,
0xe42, 0xe19, 0xe0b, 0xe32, 0xe23, 0xe32, 0xe40, 0xe22, 0xe42, 0xe27, 0xe0b, 0xe32,
0xe23, 0xe32, 0xe17, 0xe2d, 0xe1f, 0xe0b, 0xe34, 0xe21, 0xe40, 0xe1f, 0xe2d, 0xe42,
0xe23, 0xe42, 0xe1b, 0xe25, 0xe2a, 0xe42, 0xe01, 0xe40, 0xe1b, 0xe35, 0xe22, 0xe42,
0xe0b, 0xe40, 0xe1f, 0xe35, 0xe22, 0xe2a, 0xe15, 0xe2d, 0xe01, 0xe42, 0xe2e, 0xe25,
0xe4c, 0xe21, 0xe17, 0xe32, 0xe25, 0xe25, 0xe34, 0xe19, 0xe19, 0xe4c, 0xe15, 0xe34,
0xe23, 0xe32, 0xe19, 0xe32, 0xe2d, 0xe30, 0xe25, 0xe34, 0xe22, 0xe32, 0xe19, 0xe2d,
0xe1f, 0xe27, 0xe32, 0xe14, 0xe38, 0xe0b, 0xe27, 0xe32, 0xe15, 0xe34, 0xe01, 0xe31,
0xe19, 0xe40, 0xe27, 0xe35, 0xe22, 0xe19, 0xe19, 0xe32, 0xe27, 0xe34, 0xe25, 0xe19,
0xe35, 0xe2d, 0xe38, 0xe2a, 0xe27, 0xe2d, 0xe25, 0xe42, 0xe01, 0xe01, 0xe23, 0xe32,
0xe14, 0xe27, 0xe2d, 0xe23, 0xe4c, 0xe0b, 0xe2d, 0xe0b, 0xe32, 0xe40, 0xe01, 0xe23,
0xe47, 0xe1a, 0xe0b, 0xe39, 0xe23, 0xe34, 0xe04, 0xe2d, 0xe31, 0xe19, 0xe15, 0xe32,
0xe19, 0xe32, 0xe19, 0xe32, 0xe23, 0xe34, 0xe42, 0xe27, 0xe0a, 0xe32, 0xe01, 0xe31,
0xe2a, 0xe04, 0xe23, 0xe34, 0xe2a, 0xe15, 0xe4c, 0xe21, 0xe32, 0xe2a, 0xe42, 0xe04,
0xe42, 0xe04, 0xe2a, 0xe42, 0xe04, 0xe42, 0xe21, 0xe42, 0xe23, 0xe41, 0xe01, 0xe23,
0xe4c, 0xe40, 0xe01, 0xe2d, 0xe25, 0xe2d, 0xe07, 0xe21, 0xe32, 0xe40, 0xe2d, 0xe21,
0xe31, 0xe25, 0xe14, 0xe35, 0xe1f, 0xe2a, 0xe4c, 0xe21, 0xe2d, 0xe23, 0xe34, 0xe40,
0xe0a, 0xe35, 0xe22, 0xe2a, 0xe21, 0xe32, 0xe42, 0xe22, 0xe40, 0xe15, 0xe40, 0xe23,
0xe2d, 0xe39, 0xe19, 0xe35, 0xe22, 0xe07, 0xe2d, 0xe32, 0xe1b, 0xe35, 0xe2d, 0xe32,
0xe42, 0xe2d, 0xe04, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe1a, 0xe39, 0xe40, 0xe01,
0xe19, 0xe27, 0xe34, 0xe25, 0xe25, 0xe4c, 0xe41, 0xe0a, 0xe17, 0xe31, 0xe21, 0xe17,
0xe23, 0xe31, 0xe01, 0xe2d, 0xe35, 0xe2a, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe40,
0xe2d, 0xe1f, 0xe32, 0xe40, 0xe15, 0xe1f, 0xe32, 0xe40, 0xe04, 0xe32, 0xe42, 0xe1f,
0xe1f, 0xe34, 0xe08, 0xe34, 0xe1f, 0xe39, 0xe19, 0xe30, 0xe1f, 0xe39, 0xe15, 0xe35,
0xe01, 0xe32, 0xe25, 0xe32, 0xe1b, 0xe32, 0xe42, 0xe01, 0xe2a, 0xe41, 0xe01, 0xe21,
0xe40, 0xe1a, 0xe35, 0xe22, 0xe23, 0xe4c, 0xe01, 0xe31, 0xe27, 0xe14, 0xe31, 0xe25,
0xe04, 0xe32, 0xe19, 0xe31, 0xe25, 0xe01, 0xe27, 0xe21, 0xe42, 0xe2e, 0xe42, 0xe19,
0xe25, 0xe39, 0xe25, 0xe39, 0xe40, 0xe2d, 0xe19, 0xe40, 0xe14, 0xe2d, 0xe23, 0xe4c,
0xe40, 0xe1a, 0xe2d, 0xe23, 0xe35, 0xe04, 0xe34, 0xe23, 0xe34, 0xe17, 0xe34, 0xe21,
0xe32, 0xe15, 0xe35, 0xe04, 0xe2d, 0xe2a, 0xe44, 0xe23, 0xe04, 0xe27, 0xe32, 0xe08,
0xe32, 0xe40, 0xe25, 0xe19, 0xe21, 0xe32, 0xe08, 0xe39, 0xe42, 0xe23, 0xe21, 0xe32,
0xe23, 0xe4c, 0xe40, 0xe04, 0xe0b, 0xe31, 0xe2a, 0xe21, 0xe34, 0xe14, 0xe40, 0xe27,
0xe22, 0xe4c, 0xe19, 0xe32, 0xe2d, 0xe39, 0xe23, 0xe39, 0xe19, 0xe35, 0xe2d, 0xe39,
0xe40, 0xe2d, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe1f, 0xe2d, 0xe25, 0xe4c, 0xe01, 0xe19,
0xe39, 0xe40, 0xe21, 0xe2d, 0xe32, 0xe1b, 0xe32, 0xe42, 0xe01, 0x20, 0xe1b, 0xe32,
0xe42, 0xe01, 0xe1b, 0xe32, 0xe40, 0xe25, 0xe32, 0xe1e, 0xe34, 0xe15, 0xe41, 0xe04,
0xe23, 0xe4c, 0xe19, 0xe42, 0xe1b, 0xe19, 0xe32, 0xe40, 0xe1b, 0xe1e, 0xe2d, 0xe23,
0xe4c, 0xe15, 0xe21, 0xe2d, 0xe23, 0xe4c, 0xe2a, 0xe1a, 0xe35, 0xe23, 0xe32, 0xe42,
0xe23, 0xe15, 0xe2d, 0xe07, 0xe01, 0xe32, 0xe44, 0xe0b, 0xe1b, 0xe31, 0xe19, 0xe15,
0xe32, 0xe2e, 0xe35, 0xe15, 0xe35, 0xe15, 0xe32, 0xe23, 0xe30, 0xe27, 0xe32, 0xe15,
0xe2d, 0xe07, 0xe01, 0xe32, 0xe15, 0xe32, 0xe1b, 0xe39, 0xe40, 0xe27, 0xe01, 0xe27,
0xe32, 0xe25, 0xe25, 0xe34, 0xe2a, 0x12a3, 0x1262, 0x1303, 0x1295, 0x12a3, 0x12ad, 0x122b,
0x12a3, 0x12f2, 0x1235, 0x20, 0x12a3, 0x1260, 0x1263, 0x12a3, 0x120d, 0x1300, 0x122d, 0x1235,
0x12a3, 0x1235, 0x1218, 0x122b, 0x1263, 0x1295, 0x130a, 0x1262, 0x1233, 0x12cd, 0x1261, 0x1301,
0x121d, 0x1261, 0x122b, 0x1234, 0x12cd, 0x1273, 0x12ae, 0x1293, 0x12ad, 0x122a, 0x12f3, 0x122d,
0x20, 0x12a4, 0x1235, 0x20, 0x1233, 0x120b, 0x121d, 0x12f1, 0x12cb, 0x120b, 0x12a4, 0x120d,
0x20, 0x12a3, 0x12e9, 0x1295, 0x1203, 0x122b, 0x1228, 0x120a, 0x1265, 0x1228, 0x126a, 0x120d,
0x120e, 0x1218, 0x1209, 0x12cb, 0x1295, 0x12f3, 0x121b, 0x120b, 0x1266, 0x121b, 0x1351, 0x1276,
0x121b, 0x1230, 0x1229, 0x121d, 0x1263, 0x1263, 0x1290, 0x121e, 0x1253, 0x12f5, 0x123e, 0x121e,
0x1295, 0x122e, 0x126d, 0x12eb, 0x1295, 0x1303, 0x1218, 0x1293, 0x1295, 0x12eb, 0x1218, 0x12ed,
0x1295, 0x12cb, 0x12ad, 0x123e, 0x1275, 0x12cb, 0x130b, 0x12f1, 0x1309, 0x1356, 0x122d, 0x1276,
0x20, 0x1296, 0x126e, 0x1233, 0x12a6, 0x20, 0x1276, 0x1218, 0x1271, 0x1292, 0x1235, 0x12a3,
0x12f3, 0x12ad, 0x12a3, 0x1295, 0x12ae, 0x1228, 0x1305, 0x12a3, 0x1295, 0x1312, 0x120b, 0x12a3,
0x1295, 0x1272, 0x1313, 0x12a3, 0x122b, 0x1313, 0x12ed, 0x1293, 0x1265, 0x12c8, 0x1296, 0x1235,
0x20, 0x12a3, 0x12ed, 0x1228, 0x1235, 0x1201, 0x1201, 0x12ed, 0x120b, 0x20, 0x122d, 0x12ee,
0x1203, 0x1218, 0x1295, 0x12f6, 0x12db, 0x122d, 0x12ee, 0x20, 0x130b, 0x12e8, 0x130e, 0x1235,
0x1233, 0x1295, 0x20, 0x1205, 0x12cb, 0x1295, 0x1233, 0x1295, 0x20, 0x120d, 0x12ca, 0x1235,
0x12a1, 0x123d, 0x12cb, 0x12eb, 0x12a3, 0x1229, 0x1263, 0x12a3, 0x1231, 0x1295, 0x1235, 0x12ee,
0x1295, 0x12a3, 0x1272, 0x12ae, 0x12ab, 0x1295, 0x1263, 0x1205, 0x12eb, 0x1263, 0x12a5, 0x12eb,
0x20, 0x12f0, 0x20, 0x1263, 0x1295, 0x12f0, 0x122b, 0x1235, 0x1263, 0x122d, 0x1263, 0x12f6,
0x1235, 0x1260, 0x1208, 0x121d, 0x1260, 0x120a, 0x12dd, 0x1265, 0x120b, 0x1295, 0x12ad, 0x2d,
0x1233, 0x1265, 0x120e, 0x1295, 0x1266, 0x12cb, 0x20, 0x126a, 0x1235, 0x1273, 0x1266, 0x12ed,
0x12da, 0x12ab, 0x121d, 0x1265, 0x122a, 0x1305, 0x20, 0x1260, 0x12ed, 0x12ab, 0x121d, 0x1356,
0x20, 0x130d, 0x122b, 0x1295, 0x12f0, 0x12ab, 0x12e8, 0x1295, 0x127a, 0x12cb, 0x12cb, 0x1232,
0x12e9, 0x12f3, 0x12f5, 0x20, 0x1301, 0x12cb, 0x1228, 0x12dd, 0x12ae, 0x1235, 0x1273, 0x20,
0x122a, 0x12ab, 0x12a9, 0x12eb, 0x1263, 0x12a9, 0x122b, 0x1233, 0x12cd, 0x12f3, 0x12cd, 0x1230,
0x1295, 0x20, 0x12ad, 0x122a, 0x12ad, 0x12f0, 0x1295, 0x1268, 0x122d, 0x12f6, 0x121a, 0x1292,
0x12ab, 0x12a4, 0x12ed, 0x1229, 0x1294, 0x1352, 0x12a4, 0x120d, 0x20, 0x1233, 0x120d, 0x126b,
0x12f6, 0x122d, 0x134e, 0x122d, 0x1275, 0x20, 0x1290, 0x120d, 0x1230, 0x1295, 0x134e, 0x122d,
0x1273, 0x1208, 0x12db, 0x130d, 0x120c, 0x1235, 0x20, 0x1260, 0x12ed, 0x1309, 0x12dd, 0x20,
0x1260, 0x12ed, 0x130d, 0x122b, 0x1295, 0x12f5, 0x20, 0x1271, 0x122d, 0x12ad, 0x130d, 0x1228,
0x1293, 0x12f3, 0x1313, 0x12f0, 0x1209, 0x1355, 0x1313, 0x12eb, 0x12aa, 0x120d, 0x12a4, 0x122d,
0x121e, 0x1235, 0x12ee, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x1356, 0x120a, 0x1235, 0x1296,
0x12ad, 0x1235, 0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x121b, 0x1228, 0x1295,
0x130e, 0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x1352, 0x1270, 0x122d, 0x1235,
0x1260, 0x122d, 0x130d, 0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x1270, 0x120d,
0x20, 0x1232, 0x1272, 0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x126a, 0x126a,
0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x126a, 0x1295, 0x1230, 0x1295, 0x1235,
0x1363, 0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x12ca, 0x1293, 0x121b, 0x12ad, 0x1363,
0x20, 0x12a2, 0x1295, 0x12f5, 0x12eb, 0x1293, 0x12a2, 0x1243, 0x120d, 0x12ca, 0x1275, 0x1301,
0x1290, 0x12cd, 0x120d, 0x12ca, 0x126a, 0x120d, 0x121e, 0x1295, 0x1272, 0x1278, 0x120e, 0x1363,
0x20, 0x12a8, 0x1295, 0x1273, 0x12aa, 0x12ad, 0x122b, 0x1208, 0x1295, 0x12f3, 0x12ed, 0x12ad,
0x120e, 0x1235, 0x20, 0x12a3, 0x1295, 0x1300, 0x1208, 0x1235, 0x1208, 0x12c8, 0x122d, 0x20,
0x1355, 0x122a, 0x1295, 0x1230, 0x1235, 0x20, 0x12b3, 0x122d, 0x1270, 0x122d, 0x121b, 0x1230,
0x12ee, 0x121b, 0x1293, 0x1313, 0x121b, 0x1293, 0x12cd, 0x1235, 0x121b, 0x122a, 0x130e, 0x1275,
0x1218, 0x122a, 0x12f3, 0x12a8, 0x1270, 0x121b, 0x20, 0x121c, 0x12ad, 0x1232, 0x12ae, 0x121a,
0x12a8, 0x120e, 0x1295, 0x121e, 0x1295, 0x12ad, 0x1270, 0x1295, 0x121e, 0x1295, 0x1270, 0x1228,
0x12ed, 0x121e, 0x1295, 0x1270, 0x126a, 0x12f0, 0x12ee, 0x121e, 0x1295, 0x1275, 0x1230, 0x122b,
0x1275, 0x1292, 0x12cd, 0x20, 0x12ee, 0x122d, 0x12ad, 0x1290, 0x12cd, 0x121d, 0x1296, 0x122e,
0x1295, 0x12eb, 0x1265, 0x12e9, 0x120b, 0x1363, 0x20, 0x1230, 0x121c, 0x1295, 0x20, 0x12f3,
0x12ae, 0x1273, 0x1230, 0x1295, 0x1270, 0x122d, 0x1363, 0x20, 0x1230, 0x121c, 0x1295, 0x20,
0x12f3, 0x12ae, 0x1273, 0x1292, 0x12cd, 0x20, 0x1233, 0x1208, 0x121d, 0x1363, 0x20, 0x1230,
0x121c, 0x1295, 0x20, 0x12f3, 0x12ae, 0x1273, 0x1291, 0x12ad, 0x12a6, 0x1302, 0x1293, 0x130b,
0x1356, 0x122d, 0x1275, 0x2d, 0x12a6, 0x2d, 0x1355, 0x122a, 0x1295, 0x1235, 0x1356, 0x122d,
0x1275, 0x20, 0x12a6, 0x134d, 0x20, 0x1235, 0x1354, 0x1295, 0x1356, 0x122d, 0x1276, 0x20,
0x1268, 0x120d, 0x12ee, 0x1356, 0x122d, 0x1276, 0x20, 0x122a, 0x12ae, 0x1351, 0x1295, 0x1273,
0x20, 0x12a3, 0x1228, 0x1293, 0x1235, 0x122b, 0x1295, 0x12aa, 0x1295, 0x20, 0x12a2, 0x1295,
0x1208, 0x1275, 0x1228, 0x1232, 0x1348, 0x1228, 0x12de, 0x1209, 0x1275, 0x122d, 0x12ee, 0x20,
0x1265, 0x122b, 0x1295, 0x12ae, 0x1233, 0x1295, 0x1273, 0x1228, 0x121d, 0x1233, 0x1295, 0x1275,
0x12eb, 0x130e, 0x1233, 0x12a6, 0x20, 0x1353, 0x12cd, 0x120e, 0x12a2, 0x1276, 0x1246, 0x122d,
0x1276, 0x121a, 0x1275, 0x1245, 0x12f1, 0x1235, 0x20, 0x1263, 0x122d, 0x1270, 0x1208, 0x121a,
0x1245, 0x12f1, 0x1235, 0x20, 0x12ee, 0x1203, 0x1295, 0x1235, 0x1245, 0x12f5, 0x1235, 0x1272,
0x20, 0x1209, 0x1235, 0x12eb, 0x1230, 0x12ed, 0x1295, 0x1275, 0x20, 0x1276, 0x121b, 0x1235,
0x1235, 0x12ca, 0x134d, 0x1275, 0x20, 0x12ab, 0x1228, 0x1295, 0x1275, 0x1270, 0x1309, 0x1232,
0x130b, 0x120d, 0x1353, 0x12d9, 0x120d, 0x1272, 0x1305, 0x12cb, 0x1293, 0x126b, 0x1295, 0x12a9,
0x1268, 0x122d, 0x12cb, 0x12ed, 0x1275, 0x1206, 0x12dd, 0x12ca, 0x1292, 0x1350, 0x130d, 0x12a8,
0x12ed, 0x12da, 0x12f0, 0x126a, 0x1235, 0x12f1, 0x121e, 0x1295, 0x1275, 0x20, 0x12f2, 0x12a1,
0x122d, 0x126a, 0x120d, 0x121b, 0x12b3, 0x122a, 0x121b, 0x12cd, 0x1230, 0x1295, 0x121b, 0x12ad,
0x1219, 0x122d, 0x12f6, 0x122e, 0x12d8, 0x122b, 0x120e, 0x1295, 0x130d, 0x12e8, 0x122d, 0x1263,
0x12e8, 0x1295, 0x12d3, 0x12f0, 0x1295, 0x12a3, 0x120d, 0x121b, 0x1272, 0x12d3, 0x121b, 0x1295,
0x12a3, 0x1293, 0x12f2, 0x122d, 0x12a3, 0x12ad, 0x1273, 0x12cd, 0x12a3, 0x12ad, 0x1276, 0x1260,
0x12a3, 0x123d, 0x130b, 0x1263, 0x1275, 0x12a3, 0x1272, 0x122b, 0x12cd, 0x1263, 0x1255, 0x12f3,
0x12f5, 0x1263, 0x1215, 0x122c, 0x1295, 0x1263, 0x1295, 0x130d, 0x12ae, 0x12ad, 0x1263, 0x122d,
0x1293, 0x12cd, 0x120d, 0x1260, 0x12ed, 0x1229, 0x1275, 0x1265, 0x1229, 0x1290, 0x12ed, 0x12f1,
0x123b, 0x1295, 0x1260, 0x1253, 0x12db, 0x12a8, 0x1270, 0x121b, 0x20, 0x1206, 0x20, 0x127a,
0x20, 0x121a, 0x1295, 0x12b3, 0x120b, 0x20, 0x1209, 0x121d, 0x1351, 0x122d, 0x121b, 0x12ab,
0x12cd, 0x1292, 0x12ae, 0x1235, 0x12eb, 0x1296, 0x126e, 0x12a9, 0x12dd, 0x1290, 0x1275, 0x1235,
0x12ad, 0x1296, 0x126e, 0x1232, 0x1262, 0x122d, 0x1235, 0x12ad, 0x1355, 0x1296, 0x121d, 0x20,
0x1350, 0x1295, 0x1356, 0x1295, 0x1275, 0x12eb, 0x1293, 0x12ad, 0x1250, 0x1320, 0x122d, 0x12aa,
0x12da, 0x120e, 0x122d, 0x12f3, 0x122d, 0x12eb, 0x12f5, 0x1233, 0x12ab, 0x120a, 0x1295, 0x1236,
0x12cd, 0x120d, 0x123b, 0x1295, 0x130d, 0x1203, 0x12ed, 0x1235, 0x122c, 0x12f5, 0x1290, 0x12ae,
0x120a, 0x121d, 0x1235, 0x12ad, 0x1273, 0x12ed, 0x1350, 0x12ed, 0x1273, 0x123d, 0x12a8, 0x1295,
0x1275, 0x1275, 0x1262, 0x120a, 0x1232, 0x1270, 0x1205, 0x122b, 0x1295, 0x1276, 0x12ad, 0x12ee,
0x12a1, 0x120b, 0x1295, 0x20, 0x1263, 0x1276, 0x122d, 0x126d, 0x12e8, 0x1295, 0x1275, 0x12eb,
0x1295, 0x12e8, 0x1228, 0x126b, 0x1295, 0x12a3, 0x12de, 0x1228, 0x1235, 0x1260, 0x122d, 0x1219,
0x12f3, 0x12ac, 0x1355, 0x20, 0x1268, 0x122d, 0x12f0, 0x12f0, 0x1234, 0x1273, 0x1275, 0x20,
0x134b, 0x122e, 0x121b, 0x12f0, 0x12ed, 0x122b, 0x1228, 0x12ed, 0x12ad, 0x12eb, 0x126a, 0x12ad,
0x12f0, 0x1261, 0x1265, 0x20, 0x1306, 0x122d, 0x1305, 0x12eb, 0x1245, 0x12f5, 0x1235, 0x1272,
0x20, 0x1204, 0x1208, 0x1293, 0x1235, 0x1273, 0x1295, 0x120a, 0x12a3, 0x12f0, 0x1208, 0x12ed,
0x12f5, 0x1265, 0x122a, 0x12dd, 0x1264, 0x1295, 0x12e9, 0x12ad, 0x120b, 0x120e, 0x122d, 0x12f5,
0x20, 0x1203, 0x12cd, 0x1218, 0x120d, 0x1260, 0x122d, 0x1295, 0x12a3, 0x121d, 0x1235, 0x1270,
0x122d, 0x12f3, 0x121d, 0x12a3, 0x1295, 0x12f6, 0x122b, 0x12a3, 0x1235, 0x1275, 0x122b, 0x12ab,
0x1295, 0x12a3, 0x1274, 0x1295, 0x1235, 0x1260, 0x120d, 0x130d, 0x122c, 0x12f5, 0x1265, 0x122b,
0x1235, 0x120d, 0x1235, 0x1261, 0x12ab, 0x1228, 0x1235, 0x1275, 0x1261, 0x12f3, 0x1350, 0x1235,
0x1275, 0x12aa, 0x123a, 0x1293, 0x12cd, 0x1308, 0x122d, 0x1295, 0x12da, 0x12a3, 0x12ed, 0x120d,
0x20, 0x12a6, 0x134d, 0x20, 0x121b, 0x1295, 0x1300, 0x122d, 0x12da, 0x12ad, 0x12e8, 0x126d,
0x120d, 0x12e9, 0x1265, 0x120d, 0x12eb, 0x1293, 0x120e, 0x1295, 0x12f0, 0x1295, 0x121b, 0x122a,
0x1203, 0x121d, 0x1295, 0x1356, 0x12f5, 0x130e, 0x122a, 0x133b, 0x1233, 0x122b, 0x12e8, 0x126e,
0x1236, 0x134d, 0x12eb, 0x1272, 0x122b, 0x1293, 0x126d, 0x12e8, 0x1293, 0x126a, 0x120d, 0x1295,
0x12e8, 0x1235, 0x126e, 0x120d, 0x130e, 0x130d, 0x122b, 0x12f5, 0x12cb, 0x122d, 0x1233, 0x12cd,
0x12db, 0x130d, 0x1228, 0x1265, 0x12a3, 0x1295, 0x1273, 0x1293, 0x1293, 0x122a, 0x126e, 0x12ad,
0x122a, 0x1235, 0x121b, 0x1235, 0x12a8, 0x122d, 0x1309, 0x1208, 0x1295, 0x121b, 0x120d, 0x12f2,
0x126d, 0x1235, 0x121b, 0x12cd, 0x122a, 0x1238, 0x1235, 0x121b, 0x12ee, 0x1275, 0x122d, 0x12e9,
0x1295, 0x12e8, 0x1295, 0x12a3, 0x1355, 0x12eb, 0x1261, 0x1308, 0x1295, 0x126a, 0x120d, 0x12f0,
0x1234, 0x1275, 0x20, 0x134b, 0x1232, 0x12ab, 0x12a4, 0x134b, 0x1274, 0x1349, 0x1293, 0x1349,
0x1272, 0x130b, 0x121d, 0x1265, 0x12e8, 0x122d, 0x1313, 0x12f3, 0x120d, 0x12ab, 0x1293, 0x120d,
0x1313, 0x121d, 0x12a4, 0x1295, 0x12f0, 0x122d, 0x1260, 0x122a, 0x121b, 0x1301, 0x122e, 0x121b,
0x122d, 0x12ac, 0x1233, 0x1235, 0x1291, 0x1218, 0x12eb, 0x1352, 0x1275, 0x12a8, 0x122d, 0x1295,
0x1356, 0x1295, 0x1350, 0x12ed, 0x1356, 0x122d, 0x1275, 0x20, 0x121e, 0x122d, 0x1235, 0x1262,
0x1233, 0x12ed, 0x1353, 0x1295, 0x12cc, 0x12ad, 0x4e, 0x69, 0x75, 0x20, 0x2bb, 0x49,
0x6f, 0x6b, 0x65, 0x48, 0x65, 0x70, 0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x53,
0x65, 0x6c, 0x16b, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x61, 0x41, 0x74, 0x65,
0x6c, 0x61, 0x69, 0x74, 0x65, 0x50, 0x65, 0x6c, 0x69, 0x73, 0x69, 0x70,
0x61, 0x6e, 0x65, 0x4d, 0x65, 0x6c, 0x69, 0x70, 0x6f, 0x61, 0x6e, 0x65,
0x53, 0x65, 0x6e, 0x113, 0x2bb, 0x41, 0x74, 0x65, 0x6e, 0x69, 0x73, 0x69,
0x4c, 0x6f, 0x6e, 0x69, 0x74, 0x6f, 0x6e, 0x69, 0x4d, 0x6f, 0x73, 0x69,
0x6b, 0x6f, 0x75, 0x50, 0x61, 0x6c, 0x65, 0x73, 0x69, 0x4c, 0x6f, 0x6d,
0x61, 0x2bb, 0x41, 0x6f, 0x6b, 0x61, 0x6c, 0x61, 0x6e, 0x69, 0x50, 0x75,
0x6b, 0x61, 0x6e, 0x69, 0x76, 0x69, 0x6c, 0x61, 0x54, 0x16b, 0x6b, 0x65,
0x4c, 0x61, 0x70, 0x61, 0x6e, 0x75, 0x69, 0x46, 0x69, 0x73, 0x69, 0x4b,
0x61, 0x6d, 0x69, 0x70, 0x69, 0x113, 0x4b, 0x75, 0x61, 0x74, 0x101, 0x6b,
0x61, 0x6e, 0x61, 0x6c, 0x69, 0x4b, 0x75, 0x61, 0x6d, 0x69, 0x2bb, 0x45,
0x6e, 0x69, 0x74, 0x69, 0x70, 0x75, 0x6c, 0x12b, 0x4b, 0x69, 0x6c, 0x69,
0x73, 0x69, 0x6d, 0x61, 0x73, 0x69, 0x4b, 0x6f, 0x73, 0x69, 0x6c, 0x61,
0x65, 0x4b, 0x75, 0x61, 0x73, 0x61, 0x6c, 0x65, 0x6e, 0x69, 0x4d, 0x61,
0x73, 0x75, 0x6c, 0x6f, 0x4d, 0x61, 0x6c, 0x69, 0x6b, 0x75, 0x65, 0x73,
0x61, 0x4d, 0x69, 0x74, 0x69, 0x75, 0x61, 0x69, 0x4e, 0x61, 0x75, 0x6c,
0x75, 0x4e, 0x69, 0x75, 0x113, 0x4e, 0x6f, 0x61, 0x66, 0x14d, 0x6b, 0x69,
0x50, 0x61, 0x6e, 0x67, 0x6f, 0x70, 0x61, 0x6e, 0x67, 0x6f, 0x50, 0x69,
0x74, 0x69, 0x6b, 0x65, 0x6e, 0x69, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x113,
0x54, 0x61, 0x75, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x4d, 0x6f, 0x6c,
0x65, 0x73, 0x69, 0x70, 0x69, 0x4c, 0x61, 0x6c, 0x6f, 0x74, 0x6f, 0x6e,
0x67, 0x61, 0x53, 0x61, 0x2bb, 0x69, 0x70, 0x61, 0x6e, 0x69, 0x54, 0x61,
0x68, 0x69, 0x73, 0x69, 0x54, 0x61, 0x6c, 0x61, 0x76, 0x61, 0x55, 0x65,
0x6b, 0x65, 0x2bb, 0x55, 0x76, 0x65, 0x61, 0x43, 0x65, 0x7a, 0x61, 0x79,
0x69, 0x72, 0x4b, 0x61, 0x68, 0x69, 0x72, 0x65, 0x44, 0x61, 0x72, 0xfc,
0x73, 0x73, 0x65, 0x6c, 0x61, 0x6d, 0x4c, 0x61, 0x79, 0x75, 0x6e, 0x4d,
0x6f, 0x67, 0x61, 0x64, 0x69, 0x15f, 0x75, 0x54, 0x75, 0x6e, 0x75, 0x73,
0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x4b, 0x75, 0x7a, 0x65,
0x79, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4d, 0x65, 0x72, 0x6b,
0x65, 0x7a, 0x2c, 0x20, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65,
0x6d, 0x2c, 0x20, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x41, 0x6b, 0x74, 0x61, 0x76, 0x41, 0x15f, 0x6b, 0x61,
0x62, 0x61, 0x74, 0x41, 0x74, 0x131, 0x72, 0x61, 0x76, 0x42, 0x61, 0x11f,
0x64, 0x61, 0x74, 0x42, 0x61, 0x68, 0x72, 0x65, 0x79, 0x6e, 0x42, 0x61,
0x6b, 0xfc, 0x44, 0x75, 0x15f, 0x61, 0x6e, 0x62, 0x65, 0x47, 0x61, 0x7a,
0x69, 0x6d, 0x61, 0x11f, 0x75, 0x73, 0x61, 0x47, 0x61, 0x7a, 0x7a, 0x65,
0x45, 0x6c, 0x20, 0x48, 0x61, 0x6c, 0x69, 0x6c, 0x48, 0x6f, 0x20, 0x43,
0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x20, 0x4b, 0x65, 0x6e, 0x74,
0x69, 0x4b, 0x75, 0x64, 0xfc, 0x73, 0x4b, 0x61, 0x6c, 0x6b, 0xfc, 0x74,
0x61, 0x4b, 0x75, 0x76, 0x65, 0x79, 0x74, 0x4c, 0x65, 0x66, 0x6b, 0x6f,
0x15f, 0x61, 0x4b, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x54,
0x61, 0x68, 0x72, 0x61, 0x6e, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x75, 0x72, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x41, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x131, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x79, 0x61, 0x20, 0x41,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x4d, 0x61, 0x64, 0x65, 0x69, 0x72,
0x61, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x47, 0xfc, 0x6e,
0x65, 0x79, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x42, 0x72,
0xfc, 0x6b, 0x73, 0x65, 0x6c, 0x42, 0xfc, 0x6b, 0x72, 0x65, 0x15f, 0x42,
0x75, 0x64, 0x61, 0x70, 0x65, 0x15f, 0x74, 0x65, 0x4b, 0x69, 0x15f, 0x69,
0x6e, 0x65, 0x76, 0x43, 0x65, 0x62, 0x65, 0x6c, 0x69, 0x74, 0x61, 0x72,
0x131, 0x6b, 0x4d, 0x61, 0x6e, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x4c,
0xfc, 0x6b, 0x73, 0x65, 0x6d, 0x62, 0x75, 0x72, 0x67, 0xdc, 0x73, 0x6b,
0xfc, 0x70, 0x56, 0x61, 0x72, 0x15f, 0x6f, 0x76, 0x61, 0x5a, 0xfc, 0x72,
0x69, 0x68, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x6c, 0x65, 0x72, 0x50,
0x61, 0x73, 0x6b, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x41, 0x64, 0x61, 0x73,
0x131, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x20, 0x41, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x131, 0x41, 0x62, 0x69, 0x6a, 0x61, 0x6e, 0x42, 0x6c, 0x61,
0x6e, 0x74, 0x61, 0xfd, 0x72, 0x42, 0x72, 0x61, 0x7a, 0x7a, 0x61, 0x77,
0x69, 0x6c, 0x45, 0x6c, 0x2d, 0x41, 0xfd, 0x75, 0x6e, 0xdd, 0x6f, 0x68,
0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67, 0x4c, 0x69, 0x62,
0x72, 0x65, 0x77, 0x69, 0x6c, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x15f,
0x6f, 0x4d, 0x6f, 0x6e, 0x72, 0x6f, 0x77, 0x69, 0xfd, 0x61, 0x4e, 0x61,
0xfd, 0x72, 0x6f, 0x62, 0x69, 0x4a, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e,
0x69, 0x61, 0x6d, 0x65, 0xfd, 0x4e, 0x75, 0x61, 0x6b, 0x15f, 0x6f, 0x74,
0x50, 0x6f, 0x72, 0x74, 0x6f, 0x2d, 0x4e, 0x6f, 0x77, 0x6f, 0x53, 0x61,
0x6e, 0x2d, 0x54, 0x6f, 0x6d, 0x65, 0x41, 0x64, 0x61, 0x6b, 0x20, 0x61,
0x64, 0x61, 0x73, 0x79, 0x41, 0x6e, 0x67, 0x69, 0x6c, 0xfd, 0x61, 0x41,
0x72, 0x61, 0x67, 0x75, 0x61, 0xfd, 0x6e, 0x61, 0x42, 0x75, 0x65, 0x6e,
0x6f, 0x73, 0x2d, 0x41, 0xfd, 0x72, 0x65, 0x73, 0x4b, 0x6f, 0x72, 0x64,
0x6f, 0x77, 0x61, 0x17d, 0x75, 0x17e, 0x75, 0xfd, 0x4c, 0x61, 0x2d, 0x52,
0x69, 0x6f, 0x68, 0x61, 0x53, 0x61, 0x6e, 0x2d, 0x4c, 0x75, 0x69, 0x73,
0x55, 0x15f, 0x75, 0x61, 0xfd, 0x61, 0x41, 0x73, 0x75, 0x6e, 0x73, 0xfd,
0x6f, 0x6e, 0x42, 0x61, 0x69, 0xfd, 0x61, 0x42, 0x61, 0x69, 0xfd, 0x61,
0x2d, 0x64, 0x65, 0x2d, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73,
0x42, 0x6f, 0x61, 0x2d, 0x57, 0x69, 0x73, 0x74, 0x61, 0x42, 0x6f, 0xfd,
0x73, 0x65, 0x4b, 0x65, 0x6d, 0x62, 0x72, 0x69, 0x6a, 0x2d, 0x42, 0x65,
0xfd, 0x4b, 0x61, 0x6d, 0x70, 0x75, 0x2d, 0x47, 0x72, 0x61, 0x6e, 0x64,
0x69, 0x4b, 0x61, 0xfd, 0x65, 0x6e, 0x6e, 0x61, 0x4b, 0x61, 0xfd, 0x6d,
0x61, 0x6e, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x53, 0xfd,
0x75, 0x64, 0x61, 0x64, 0x2d, 0x48, 0x75, 0x61, 0x72, 0x65, 0x73, 0x4b,
0x6f, 0x73, 0x74, 0x61, 0x2d, 0x52, 0x69, 0x6b, 0x61, 0x4b, 0x75, 0xfd,
0x61, 0x62, 0x61, 0x4b, 0xfd, 0x75, 0x72, 0x61, 0x73, 0x61, 0x6f, 0x44,
0x65, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x68, 0x61, 0x77, 0x6e, 0x44,
0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x2d, 0x4b, 0x72, 0x69, 0x6b, 0x44, 0x65,
0x6e, 0x77, 0x65, 0x72, 0x44, 0x65, 0x74, 0x72, 0x6f, 0xfd, 0x74, 0x45,
0xfd, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x65, 0x47, 0x6c, 0x65, 0xfd, 0x73,
0x2d, 0x42, 0x65, 0xfd, 0x47, 0x75, 0x73, 0x2d, 0x42, 0x65, 0xfd, 0x47,
0x72, 0x61, 0x6e, 0x64, 0x2d, 0x54, 0x65, 0x72, 0x6b, 0x47, 0x75, 0xfd,
0x61, 0x6b, 0x69, 0x6c, 0x47, 0x61, 0xfd, 0x61, 0x6e, 0x61, 0x47, 0x61,
0x6c, 0x69, 0x66, 0x61, 0x6b, 0x73, 0x47, 0x61, 0x77, 0x61, 0x6e, 0x61,
0x45, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6f, 0x4e, 0x6f, 0x6b, 0x73,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x57, 0x69, 0x77,
0x65, 0xfd, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x57,
0x69, 0x6e, 0x73, 0x65, 0x6e, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x49, 0x6e, 0x75, 0x77, 0x69, 0x6b, 0xdd, 0x61, 0x6d,
0x61, 0xfd, 0x6b, 0x61, 0x4c, 0x75, 0x69, 0x73, 0x77, 0x69, 0x6c, 0x6c,
0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73, 0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20,
0x4b, 0x65, 0x6e, 0x74, 0x75, 0x6b, 0x69, 0x4b, 0x72, 0x61, 0x6c, 0x65,
0x6e, 0x64, 0x65, 0xfd, 0x6b, 0x4c, 0x61, 0x2d, 0x50, 0x61, 0x73, 0x4c,
0x6f, 0x73, 0x2d, 0x41, 0x6e, 0x6a, 0x65, 0x6c, 0x65, 0x73, 0x4c, 0x6f,
0x77, 0x65, 0x72, 0x2d, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x73, 0x2d,
0x4b, 0x75, 0x6f, 0x72, 0x74, 0x65, 0x72, 0x4d, 0x61, 0x73, 0x65, 0xfd,
0x6f, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x72, 0x72, 0x65, 0xfd, 0x4d, 0x6f,
0x6e, 0x74, 0x65, 0x77, 0x69, 0x64, 0x65, 0x6f, 0x4e, 0xfd, 0x75, 0x2d,
0xdd, 0x6f, 0x72, 0x6b, 0x42, 0x6f, 0xfd, 0x6c, 0x61, 0x2c, 0x20, 0x44,
0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x2c, 0x20, 0x44,
0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x4e, 0xfd, 0x75, 0x2d, 0x53, 0x61, 0x6c, 0x65,
0x6d, 0x2c, 0x20, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79,
0x6b, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x50, 0x6f, 0x72, 0x74,
0x2d, 0x6f, 0x66, 0x2d, 0x53, 0x70, 0x65, 0xfd, 0x6e, 0x50, 0x6f, 0x72,
0x74, 0x75, 0x2d, 0x57, 0x65, 0x6c, 0xfd, 0x75, 0x50, 0x75, 0x65, 0x72,
0x74, 0x6f, 0x2d, 0x52, 0x69, 0x6b, 0x6f, 0x52, 0x65, 0x6a, 0x61, 0xfd,
0x6e, 0x61, 0x52, 0x65, 0x7a, 0x6f, 0x6c, 0xfd, 0x75, 0x74, 0x52, 0x69,
0x75, 0x2d, 0x42, 0x72, 0x61, 0x6e, 0x6b, 0x75, 0x53, 0x61, 0x6e, 0x74,
0xfd, 0x61, 0x67, 0x6f, 0x53, 0x61, 0x6e, 0x2d, 0x50, 0x61, 0x75, 0x6c,
0x75, 0x49, 0x6c, 0x6c, 0x6f, 0x6b, 0x6b, 0x6f, 0x72, 0x74, 0x6f, 0x6f,
0x72, 0x6d, 0x69, 0x75, 0x74, 0x53, 0x65, 0x6e, 0x74, 0x2d, 0x4a, 0x6f,
0x6e, 0x73, 0x53, 0x65, 0x6e, 0x74, 0x2d, 0x4b, 0x69, 0x74, 0x73, 0x53,
0x65, 0x6e, 0x74, 0x2d, 0x4c, 0xfd, 0x75, 0x73, 0x69, 0xfd, 0x61, 0x53,
0x65, 0x6e, 0x74, 0x2d, 0x54, 0x6f, 0x6d, 0x61, 0x73, 0x53, 0x65, 0x6e,
0x74, 0x2d, 0x57, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x53, 0x77, 0x69,
0x66, 0x74, 0x2d, 0x4b, 0x61, 0x72, 0x65, 0x6e, 0x74, 0x57, 0x61, 0x6e,
0x6b, 0x75, 0x77, 0x65, 0x72, 0x57, 0x61, 0xfd, 0x74, 0x68, 0x6f, 0x72,
0x73, 0xdd, 0x61, 0x6b, 0x75, 0x74, 0x61, 0x74, 0x4b, 0x65, 0xfd, 0x73,
0x69, 0x44, 0x65, 0xfd, 0x77, 0x69, 0x73, 0x44, 0xfd, 0x75, 0x6d, 0x6f,
0x6e, 0x2d, 0x64, 0x2d, 0xdd, 0x75, 0x72, 0x77, 0x69, 0x6c, 0x4d, 0x61,
0x6b, 0x6b, 0x75, 0x6f, 0x72, 0x69, 0x4d, 0x61, 0x6b, 0x2d, 0x4d, 0x65,
0x72, 0x64, 0x6f, 0x53, 0xfd, 0x6f, 0x77, 0x61, 0x4c, 0x6f, 0x6e, 0x67,
0x69, 0x72, 0x42, 0x61, 0x67, 0x64, 0x61, 0x74, 0x42, 0x61, 0x68, 0x72,
0x65, 0xfd, 0x6e, 0x42, 0x65, 0xfd, 0x72, 0x75, 0x74, 0x42, 0x72, 0x75,
0x6e, 0x65, 0xfd, 0xc7, 0x6f, 0xfd, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x44, 0x75, 0x62, 0x61, 0xfd, 0x48, 0x65, 0x77, 0x72, 0x6f, 0x6e, 0x48,
0x6f, 0x15f, 0x69, 0x6d, 0x69, 0x6e, 0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e,
0x67, 0x48, 0x6f, 0x77, 0x64, 0x4a, 0x61, 0xfd, 0x61, 0x70, 0x75, 0x72,
0x61, 0x49, 0xfd, 0x65, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x69, 0x6d, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0xfd, 0x61, 0x72, 0x73, 0x6b, 0x4b, 0x75,
0x77, 0x65, 0xfd, 0x74, 0x4e, 0x69, 0x6b, 0x6f, 0x73, 0x69, 0xfd, 0x61,
0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x75, 0x7a, 0x6e, 0x65, 0x73, 0x6b, 0x50,
0x68, 0x65, 0x6e, 0xfd, 0x61, 0x6e, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e,
0x61, 0xfd, 0x47, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x45,
0x72, 0x2d, 0x52, 0x69, 0xfd, 0x61, 0x64, 0x15e, 0x61, 0x6e, 0x68, 0x61,
0xfd, 0x54, 0x61, 0xfd, 0x62, 0x65, 0xfd, 0x54, 0xe4, 0x68, 0x72, 0x61,
0x6e, 0x57, 0xfd, 0x65, 0x6e, 0x74, 0xfd, 0x61, 0x6e, 0xdd, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0xdd, 0x61, 0x6e, 0x67, 0x6f, 0x6e, 0xdd, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xdd,
0x65, 0x72, 0x65, 0x77, 0x61, 0x6e, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x61,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x20,
0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x4b, 0x61, 0x62, 0x6f, 0x2d,
0x57, 0x65, 0x72, 0x64, 0x65, 0x46, 0x61, 0x72, 0x65, 0x72, 0x20, 0x61,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x4d, 0x61, 0x64, 0x65, 0xfd, 0x72,
0x61, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x52, 0x65, 0xfd,
0x6b, 0xfd, 0x61, 0x77, 0x69, 0x6b, 0x47, 0xfc, 0x6e, 0x6f, 0x72, 0x74,
0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0xfd, 0x61, 0x4b, 0x65,
0x72, 0x61, 0x6d, 0x61, 0x74, 0x6c, 0x79, 0x20, 0xdd, 0x65, 0x6c, 0x65,
0x6e, 0x61, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0xdd, 0x75, 0x6b, 0x6c,
0x61, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x61, 0x75, 0x53, 0x69, 0x64,
0x6e, 0x65, 0xfd, 0x41, 0x66, 0x69, 0x6e, 0x79, 0x42, 0x72, 0x61, 0x74,
0x69, 0x73, 0x6c, 0x61, 0x77, 0x61, 0x42, 0x72, 0xfd, 0x75, 0x73, 0x73,
0x65, 0x6c, 0x42, 0x75, 0x68, 0x61, 0x72, 0x65, 0x73, 0x74, 0x42, 0xfd,
0x75, 0x7a, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x4b, 0x69, 0x15f, 0x69, 0x6e,
0xfd, 0x6f, 0x77, 0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x67, 0x61, 0x67, 0x65,
0x6e, 0x4d, 0x65, 0x6e, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x4a, 0x65,
0x72, 0x73, 0x69, 0x4b, 0x69, 0xfd, 0x65, 0x77, 0x4c, 0xfd, 0x75, 0x62,
0x6c, 0xfd, 0x61, 0x6e, 0x61, 0x4c, 0xfd, 0x75, 0x6b, 0x73, 0x65, 0x6d,
0x62, 0x75, 0x72, 0x67, 0x4d, 0x61, 0x72, 0x69, 0xfd, 0x65, 0x68, 0x61,
0x6d, 0x6e, 0x50, 0x61, 0x72, 0x69, 0x17e, 0x50, 0x6f, 0x64, 0x67, 0x6f,
0x72, 0x69, 0x73, 0x61, 0x53, 0x61, 0x72, 0x61, 0xfd, 0x65, 0x77, 0x6f,
0x53, 0x6b, 0x6f, 0x70, 0xfd, 0x65, 0x53, 0x6f, 0x66, 0x69, 0xfd, 0x61,
0x53, 0x74, 0x6f, 0x6b, 0x67, 0x6f, 0x6c, 0x6d, 0x55, 0x6c, 0xfd, 0x61,
0x6e, 0x6f, 0x77, 0x73, 0x6b, 0x57, 0x61, 0x64, 0x75, 0x7a, 0x57, 0x61,
0x74, 0x69, 0x6b, 0x61, 0x6e, 0x57, 0x65, 0x6e, 0x61, 0x57, 0x69, 0x6c,
0x6e, 0xfd, 0x75, 0x73, 0x57, 0x61, 0x72, 0x15f, 0x61, 0x77, 0x61, 0x53,
0xfd, 0x75, 0x72, 0x69, 0x68, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e,
0x61, 0x72, 0x69, 0x77, 0x75, 0x52, 0x6f, 0x17e, 0x64, 0x65, 0x73, 0x74,
0x77, 0x6f, 0x4b, 0x6f, 0x6d, 0x6f, 0x72, 0x20, 0x61, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x79, 0x4d, 0x61, 0xfd, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x77, 0x6c, 0x65, 0x72, 0x4d, 0x61, 0x77, 0x72, 0x69, 0x6b, 0x69, 0xfd,
0x4d, 0x61, 0xfd, 0x6f, 0x74, 0x74, 0x61, 0x52, 0x65, 0xfd, 0x75, 0x6e,
0xfd, 0x6f, 0x6e, 0x42, 0x75, 0x67, 0x65, 0x6e, 0x77, 0x69, 0x6c, 0xc7,
0x61, 0x74, 0x65, 0x6d, 0x50, 0x61, 0x73, 0x68, 0x61, 0x20, 0x61, 0x64,
0x61, 0x73, 0x79, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x47, 0x61, 0x6d, 0x62,
0xfd, 0x65, 0x47, 0x77, 0x61, 0x64, 0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61,
0x6c, 0x47, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x4b, 0x6f, 0x73,
0x72, 0x61, 0xfd, 0x65, 0x4b, 0x77, 0x61, 0x6a, 0x61, 0x6c, 0x65, 0xfd,
0x6e, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x20, 0x61, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x79, 0x4d, 0x69, 0x64, 0x75, 0x65, 0xfd, 0x50, 0x6f, 0x72,
0x74, 0x2d, 0x4d, 0x6f, 0x72, 0x73, 0x62, 0x69, 0x53, 0x61, 0xfd, 0x70,
0x61, 0x6e, 0x57, 0x65, 0xfd, 0x6b, 0x411, 0x430, 0x43d, 0x491, 0x456, 0x411,
0x440, 0x430, 0x437, 0x437, 0x430, 0x432, 0x456, 0x43b, 0x44c, 0x41a, 0x430, 0x457,
0x440, 0x41a, 0x43e, 0x43d, 0x430, 0x43a, 0x440, 0x456, 0x414, 0x436, 0x456, 0x431,
0x443, 0x442, 0x456, 0x415, 0x43b, 0x44c, 0x2d, 0x410, 0x44e, 0x43d, 0x424, 0x440,
0x456, 0x442, 0x430, 0x443, 0x43d, 0x490, 0x430, 0x431, 0x43e, 0x440, 0x43e, 0x43d,
0x435, 0x419, 0x43e, 0x433, 0x430, 0x43d, 0x43d, 0x435, 0x441, 0x431, 0x443, 0x440,
0x491, 0x41b, 0x430, 0x491, 0x43e, 0x441, 0x41b, 0x456, 0x431, 0x440, 0x435, 0x432,
0x456, 0x43b, 0x44c, 0x41b, 0x443, 0x431, 0x443, 0x43c, 0x431, 0x430, 0x448, 0x456,
0x41c, 0x43e, 0x491, 0x430, 0x434, 0x456, 0x448, 0x43e, 0x41c, 0x43e, 0x43d, 0x440,
0x43e, 0x432, 0x456, 0x44f, 0x41d, 0x456, 0x430, 0x43c, 0x435, 0x439, 0x41d, 0x443,
0x430, 0x43a, 0x448, 0x43e, 0x442, 0x442, 0x423, 0x430, 0x491, 0x430, 0x434, 0x443,
0x491, 0x443, 0x422, 0x440, 0x438, 0x43f, 0x43e, 0x43b, 0x456, 0x412, 0x456, 0x43d,
0x434, 0x433, 0x443, 0x43a, 0x410, 0x43d, 0x491, 0x456, 0x43b, 0x44c, 0x44f, 0x410,
0x43d, 0x442, 0x438, 0x491, 0x443, 0x430, 0x410, 0x440, 0x430, 0x491, 0x443, 0x430,
0x456, 0x43d, 0x430, 0x411, 0x443, 0x435, 0x43d, 0x43e, 0x441, 0x2d, 0x410, 0x439,
0x440, 0x435, 0x441, 0x41b, 0x430, 0x2d, 0x420, 0x456, 0x43e, 0x445, 0x430, 0x420,
0x456, 0x43e, 0x2d, 0x490, 0x430, 0x43b, 0x44c, 0x454, 0x491, 0x43e, 0x441, 0x421,
0x430, 0x43d, 0x2d, 0x41b, 0x443, 0x457, 0x441, 0x410, 0x441, 0x443, 0x43d, 0x441,
0x44c, 0x439, 0x43e, 0x43d, 0x410, 0x442, 0x456, 0x43a, 0x43e, 0x43a, 0x430, 0x43d,
0x411, 0x430, 0x457, 0x44f, 0x20, 0x411, 0x430, 0x43d, 0x434, 0x435, 0x440, 0x430,
0x441, 0x411, 0x43e, 0x491, 0x43e, 0x442, 0x430, 0x41a, 0x435, 0x439, 0x43c, 0x431,
0x440, 0x456, 0x434, 0x436, 0x2d, 0x411, 0x435, 0x439, 0x41a, 0x430, 0x43c, 0x43f,
0x443, 0x2d, 0x490, 0x440, 0x430, 0x43d, 0x434, 0x456, 0x41a, 0x430, 0x454, 0x43d,
0x43d, 0x430, 0x41a, 0x430, 0x439, 0x43c, 0x430, 0x43d, 0x43e, 0x432, 0x456, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x427, 0x456, 0x43a, 0x430, 0x491,
0x43e, 0x427, 0x456, 0x443, 0x430, 0x443, 0x430, 0x421, 0x2019, 0x44e, 0x434, 0x430,
0x434, 0x2d, 0x425, 0x443, 0x430, 0x440, 0x435, 0x441, 0x41a, 0x43e, 0x441, 0x442,
0x430, 0x2d, 0x420, 0x456, 0x43a, 0x430, 0x414, 0x43e, 0x443, 0x441, 0x43e, 0x43d,
0x2d, 0x41a, 0x440, 0x456, 0x43a, 0x414, 0x43e, 0x43c, 0x456, 0x43d, 0x456, 0x43a,
0x430, 0x490, 0x43b, 0x435, 0x439, 0x441, 0x2d, 0x411, 0x435, 0x439, 0x490, 0x443,
0x441, 0x2d, 0x411, 0x435, 0x439, 0x490, 0x440, 0x430, 0x43d, 0x434, 0x2d, 0x422,
0x435, 0x440, 0x43a, 0x490, 0x440, 0x435, 0x43d, 0x430, 0x434, 0x430, 0x490, 0x432,
0x430, 0x434, 0x435, 0x43b, 0x443, 0x43f, 0x430, 0x490, 0x432, 0x430, 0x442, 0x435,
0x43c, 0x430, 0x43b, 0x430, 0x490, 0x443, 0x430, 0x44f, 0x43a, 0x456, 0x43b, 0x44c,
0x490, 0x430, 0x439, 0x430, 0x43d, 0x430, 0x415, 0x440, 0x43c, 0x43e, 0x441, 0x456,
0x43b, 0x44c, 0x439, 0x43e, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430, 0x43f,
0x43e, 0x43b, 0x456, 0x441, 0x41d, 0x43e, 0x43a, 0x441, 0x2c, 0x20, 0x406, 0x43d,
0x434, 0x456, 0x430, 0x43d, 0x430, 0x41c, 0x430, 0x440, 0x435, 0x43d, 0x433, 0x43e,
0x2c, 0x20, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430, 0x41f, 0x456, 0x442,
0x435, 0x440, 0x441, 0x431, 0x435, 0x440, 0x433, 0x2c, 0x20, 0x406, 0x43d, 0x434,
0x456, 0x430, 0x43d, 0x430, 0x422, 0x435, 0x43b, 0x43b, 0x2d, 0x421, 0x456, 0x442,
0x456, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430, 0x412, 0x456,
0x432, 0x435, 0x439, 0x2c, 0x20, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430,
0x412, 0x456, 0x43d, 0x441, 0x435, 0x43d, 0x441, 0x2c, 0x20, 0x406, 0x43d, 0x434,
0x456, 0x430, 0x43d, 0x430, 0x412, 0x456, 0x43d, 0x430, 0x43c, 0x430, 0x43a, 0x2c,
0x20, 0x406, 0x43d, 0x434, 0x456, 0x430, 0x43d, 0x430, 0x406, 0x43a, 0x430, 0x43b,
0x443, 0x457, 0x442, 0x41b, 0x443, 0x457, 0x441, 0x432, 0x456, 0x43b, 0x43b, 0x41c,
0x43e, 0x43d, 0x442, 0x456, 0x441, 0x435, 0x43b, 0x43b, 0x43e, 0x2c, 0x20, 0x41a,
0x435, 0x43d, 0x442, 0x443, 0x43a, 0x43a, 0x456, 0x41b, 0x43e, 0x443, 0x435, 0x440,
0x2d, 0x41f, 0x440, 0x438, 0x43d, 0x441, 0x435, 0x441, 0x2d, 0x41a, 0x432, 0x43e,
0x442, 0x435, 0x440, 0x41c, 0x430, 0x43d, 0x430, 0x491, 0x443, 0x430, 0x41c, 0x430,
0x440, 0x456, 0x491, 0x43e, 0x41c, 0x430, 0x440, 0x442, 0x456, 0x43d, 0x456, 0x43a,
0x430, 0x41c, 0x435, 0x440, 0x456, 0x434, 0x430, 0x41c, 0x435, 0x445, 0x456, 0x43a,
0x43e, 0x41c, 0x43e, 0x43d, 0x442, 0x435, 0x432, 0x456, 0x434, 0x435, 0x43e, 0x411,
0x2bc, 0x44e, 0x43b, 0x430, 0x2c, 0x20, 0x41f, 0x456, 0x432, 0x43d, 0x456, 0x447,
0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x421, 0x435, 0x43d,
0x442, 0x440, 0x2c, 0x20, 0x41f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x430,
0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41d, 0x44c, 0x44e, 0x2d, 0x421,
0x430, 0x43b, 0x435, 0x43c, 0x2c, 0x20, 0x41f, 0x456, 0x432, 0x43d, 0x456, 0x447,
0x43d, 0x430, 0x20, 0x414, 0x430, 0x43a, 0x43e, 0x442, 0x430, 0x41e, 0x445, 0x456,
0x43d, 0x430, 0x491, 0x430, 0x41f, 0x443, 0x435, 0x440, 0x442, 0x43e, 0x2d, 0x420,
0x456, 0x43a, 0x43e, 0x420, 0x435, 0x43d, 0x43a, 0x456, 0x43d, 0x2d, 0x406, 0x43d,
0x43b, 0x435, 0x442, 0x420, 0x435, 0x441, 0x456, 0x444, 0x456, 0x420, 0x456, 0x43e,
0x2d, 0x411, 0x440, 0x430, 0x43d, 0x43a, 0x43e, 0x421, 0x430, 0x43d, 0x442, 0x44c,
0x44f, 0x491, 0x43e, 0x421, 0x430, 0x43d, 0x442, 0x43e, 0x2d, 0x414, 0x43e, 0x43c,
0x456, 0x43d, 0x491, 0x43e, 0x406, 0x442, 0x442, 0x43e, 0x43a, 0x43a, 0x43e, 0x440,
0x442, 0x43e, 0x43e, 0x440, 0x43c, 0x456, 0x443, 0x442, 0x421, 0x435, 0x43d, 0x2d,
0x411, 0x430, 0x440, 0x442, 0x435, 0x43b, 0x435, 0x43c, 0x456, 0x421, 0x435, 0x43d,
0x442, 0x2d, 0x41a, 0x456, 0x442, 0x442, 0x441, 0x421, 0x432, 0x456, 0x444, 0x442,
0x2d, 0x41a, 0x430, 0x440, 0x440, 0x435, 0x43d, 0x442, 0x422, 0x435, 0x491, 0x443,
0x441, 0x456, 0x491, 0x430, 0x43b, 0x44c, 0x43f, 0x430, 0x422, 0x456, 0x445, 0x443,
0x430, 0x43d, 0x430, 0x412, 0x430, 0x439, 0x442, 0x433, 0x43e, 0x440, 0x441, 0x412,
0x456, 0x43d, 0x43d, 0x456, 0x43f, 0x435, 0x491, 0x41a, 0x435, 0x439, 0x441, 0x456,
0x414, 0x435, 0x432, 0x456, 0x441, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434,
0x2bc, 0x42e, 0x440, 0x432, 0x456, 0x43b, 0x44c, 0x41c, 0x430, 0x43a, 0x43a, 0x443,
0x43e, 0x440, 0x456, 0x421, 0x44c, 0x43e, 0x432, 0x430, 0x41b, 0x43e, 0x43d, 0x433,
0x2bc, 0x457, 0x440, 0x414, 0x456, 0x43b, 0x456, 0x425, 0x43e, 0x448, 0x438, 0x43c,
0x456, 0x43d, 0x406, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x404, 0x440,
0x443, 0x441, 0x430, 0x43b, 0x438, 0x43c, 0x41a, 0x430, 0x440, 0x430, 0x447, 0x456,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x44c, 0x43a, 0x41d,
0x456, 0x43a, 0x43e, 0x441, 0x456, 0x44f, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x443,
0x437, 0x43d, 0x435, 0x446, 0x44c, 0x43a, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x456, 0x440, 0x441, 0x44c, 0x43a, 0x41e, 0x43c, 0x441, 0x44c, 0x43a, 0x41f,
0x43e, 0x43d, 0x442, 0x456, 0x430, 0x43d, 0x430, 0x43a, 0x415, 0x440, 0x2d, 0x420,
0x456, 0x44f, 0x434, 0x421, 0x435, 0x440, 0x435, 0x434, 0x43d, 0x44c, 0x43e, 0x43a,
0x43e, 0x43b, 0x438, 0x43c, 0x441, 0x44c, 0x43a, 0x422, 0x430, 0x439, 0x431, 0x435,
0x439, 0x422, 0x43e, 0x43a, 0x456, 0x43e, 0x422, 0x43e, 0x43c, 0x441, 0x44c, 0x43a,
0x423, 0x440, 0x443, 0x43c, 0x447, 0x456, 0x412, 0x2bc, 0x454, 0x43d, 0x442, 0x44c,
0x44f, 0x43d, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x404, 0x43a, 0x430,
0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x404, 0x440, 0x435,
0x432, 0x430, 0x43d, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x44c, 0x43a, 0x456, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41a, 0x430, 0x43d, 0x430, 0x440,
0x441, 0x44c, 0x43a, 0x456, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x424, 0x430, 0x440, 0x435, 0x440, 0x441, 0x44c, 0x43a, 0x456, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x420, 0x435, 0x439, 0x43a, 0x2bc, 0x44f, 0x432,
0x456, 0x43a, 0x41f, 0x456, 0x432, 0x434, 0x435, 0x43d, 0x43d, 0x430, 0x20, 0x414,
0x436, 0x43e, 0x440, 0x434, 0x436, 0x456, 0x44f, 0x41e, 0x441, 0x442, 0x440, 0x456,
0x432, 0x20, 0x421, 0x432, 0x44f, 0x442, 0x43e, 0x457, 0x20, 0x404, 0x43b, 0x435,
0x43d, 0x438, 0x421, 0x442, 0x435, 0x43d, 0x43b, 0x456, 0x410, 0x434, 0x435, 0x43b,
0x430, 0x457, 0x434, 0x430, 0x411, 0x440, 0x456, 0x441, 0x431, 0x435, 0x43d, 0x411,
0x440, 0x43e, 0x43a, 0x435, 0x43d, 0x2d, 0x425, 0x456, 0x43b, 0x43b, 0x415, 0x432,
0x43a, 0x43b, 0x430, 0x413, 0x43e, 0x431, 0x430, 0x440, 0x442, 0x41b, 0x456, 0x43d,
0x434, 0x435, 0x43c, 0x430, 0x43d, 0x410, 0x444, 0x456, 0x43d, 0x438, 0x41a, 0x438,
0x448, 0x438, 0x43d, 0x456, 0x432, 0x413, 0x435, 0x43b, 0x44c, 0x441, 0x456, 0x43d,
0x43a, 0x456, 0x41e, 0x441, 0x442, 0x440, 0x456, 0x432, 0x20, 0x41c, 0x435, 0x43d,
0x414, 0x436, 0x435, 0x440, 0x441, 0x456, 0x41a, 0x456, 0x440, 0x43e, 0x432, 0x41a,
0x438, 0x457, 0x432, 0x41c, 0x430, 0x440, 0x456, 0x454, 0x433, 0x430, 0x43c, 0x43d,
0x41c, 0x456, 0x43d, 0x441, 0x44c, 0x43a, 0x41f, 0x43e, 0x434, 0x433, 0x43e, 0x440,
0x438, 0x446, 0x44f, 0x421, 0x430, 0x440, 0x430, 0x454, 0x432, 0x43e, 0x421, 0x456,
0x43c, 0x444, 0x435, 0x440, 0x43e, 0x43f, 0x43e, 0x43b, 0x44c, 0x421, 0x43a, 0x43e,
0x43f, 0x2bc, 0x454, 0x421, 0x43e, 0x444, 0x456, 0x44f, 0x422, 0x430, 0x43b, 0x43b,
0x456, 0x43d, 0x43d, 0x423, 0x43b, 0x44c, 0x44f, 0x43d, 0x456, 0x432, 0x441, 0x44c,
0x43a, 0x412, 0x456, 0x434, 0x435, 0x43d, 0x44c, 0x426, 0x44e, 0x440, 0x456, 0x445,
0x410, 0x43d, 0x442, 0x430, 0x43d, 0x430, 0x43d, 0x430, 0x440, 0x456, 0x432, 0x443,
0x427, 0x430, 0x491, 0x43e, 0x441, 0x41e, 0x441, 0x442, 0x440, 0x456, 0x432, 0x20,
0x420, 0x456, 0x437, 0x434, 0x432, 0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e,
0x432, 0x456, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41a, 0x435,
0x440, 0x491, 0x435, 0x43b, 0x435, 0x43d, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x456,
0x432, 0x438, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x456, 0x439, 0x420, 0x435,
0x44e, 0x43d, 0x44c, 0x439, 0x43e, 0x43d, 0x410, 0x43f, 0x456, 0x430, 0x411, 0x443,
0x491, 0x435, 0x43d, 0x432, 0x456, 0x43b, 0x44c, 0x41e, 0x441, 0x442, 0x440, 0x456,
0x432, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x424, 0x456, 0x434, 0x436, 0x456,
0x424, 0x443, 0x43d, 0x430, 0x444, 0x443, 0x442, 0x456, 0x490, 0x430, 0x43b, 0x430,
0x43f, 0x430, 0x491, 0x43e, 0x441, 0x413, 0x430, 0x43c, 0x431, 0x2bc, 0x454, 0x415,
0x43d, 0x434, 0x435, 0x440, 0x431, 0x435, 0x440, 0x456, 0x41a, 0x456, 0x440, 0x456,
0x442, 0x456, 0x43c, 0x430, 0x442, 0x456, 0x41c, 0x430, 0x440, 0x43a, 0x456, 0x437,
0x44c, 0x43a, 0x456, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41c,
0x456, 0x434, 0x432, 0x435, 0x439, 0x41d, 0x456, 0x443, 0x435, 0x41f, 0x456, 0x442,
0x43a, 0x435, 0x440, 0x43d, 0x422, 0x430, 0x457, 0x442, 0x456, 0x412, 0x435, 0x439,
0x43a, 0x423, 0x43e, 0x43b, 0x43b, 0x456, 0x441, 0x44, 0x17a, 0x69, 0x62, 0x75,
0x74, 0x69, 0x4b, 0x61, 0x69, 0x6d, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x6b, 0x75, 0x70, 0x79, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f, 0x20, 0x6d,
0x11b, 0x73, 0x74, 0x6f, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20,
0x53, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x53,
0x65, 0x77, 0x6a, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c,
0x20, 0x53, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69,
0x6b, 0x61, 0x2f, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x20,
0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x6f, 0x77, 0x65, 0x20,
0x6d, 0x11b, 0x73, 0x74, 0x6f, 0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x75, 0x7a,
0x6e, 0x6a, 0x65, 0x63, 0x6b, 0x4a, 0x65, 0x72, 0x6a, 0x65, 0x77, 0x61,
0x6e, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x73, 0x6b, 0x65, 0x20, 0x6b,
0x75, 0x70, 0x79, 0x46, 0xe4, 0x72, 0xf6, 0x73, 0x6b, 0x65, 0x20, 0x6b,
0x75, 0x70, 0x79, 0x4a, 0x75, 0x17e, 0x6e, 0x61, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x73, 0x6b, 0x61, 0x42, 0x11b, 0x142, 0x6f, 0x68, 0x72,
0xf3, 0x64, 0x4b, 0x69, 0x161, 0x69, 0x6e, 0x6a, 0x6f, 0x77, 0x48, 0x6f,
0x64, 0x6f, 0x77, 0x6e, 0x61, 0x20, 0x6b, 0x75, 0x70, 0x61, 0x4a, 0x75,
0x74, 0x72, 0x6f, 0x77, 0x6e, 0x61, 0x20, 0x6b, 0x75, 0x70, 0x61, 0x46,
0x69, 0x64, 0x17a, 0x69, 0x639, 0x627, 0x628, 0x62f, 0x62c, 0x627, 0x646, 0x627,
0x6a9, 0x651, 0x631, 0x627, 0x627, 0x644, 0x62c, 0x6cc, 0x626, 0x631, 0x633, 0x627,
0x633, 0x645, 0x627, 0x631, 0x627, 0x628, 0x646, 0x6af, 0x648, 0x626, 0x6cc, 0x628,
0x646, 0x62c, 0x648, 0x644, 0x628, 0x644, 0x6cc, 0x646, 0x679, 0x627, 0x626, 0x631,
0x628, 0x631, 0x627, 0x632, 0x627, 0x648, 0x6cc, 0x644, 0x6d2, 0x628, 0x62c, 0x645,
0x628, 0x631, 0x627, 0x642, 0x627, 0x6c1, 0x631, 0x6c1, 0x6a9, 0x6cc, 0x633, 0x627,
0x20, 0x628, 0x644, 0x627, 0x646, 0x6a9, 0x627, 0x633, 0x6cc, 0x648, 0x679, 0x627,
0x6a9, 0x648, 0x646, 0x6a9, 0x631, 0x6cc, 0x688, 0x6a9, 0x627, 0x631, 0x62c, 0x628,
0x648, 0x62a, 0x6cc, 0x688, 0x648, 0x622, 0x644, 0x627, 0x641, 0x631, 0x6cc, 0x20,
0x679, 0x627, 0x624, 0x646, 0x6af, 0x628, 0x631, 0x648, 0x646, 0x6c1, 0x631, 0x627,
0x631, 0x6d2, 0x6a9, 0x6cc, 0x645, 0x67e, 0x627, 0x644, 0x627, 0x6a9, 0x6af, 0x627,
0x644, 0x6cc, 0x6a9, 0x646, 0x634, 0x627, 0x633, 0x627, 0x644, 0x628, 0x631, 0x6d2,
0x20, 0x648, 0x6cc, 0x644, 0x6d2, 0x644, 0x648, 0x627, 0x646, 0x688, 0x627, 0x644,
0x6cc, 0x648, 0x633, 0x627, 0x6a9, 0x627, 0x645, 0x67e, 0x648, 0x679, 0x648, 0x645,
0x633, 0x6cc, 0x631, 0x648, 0x645, 0x628, 0x627, 0x628, 0x6cc, 0x646, 0x627, 0x6cc,
0x646, 0x62c, 0x627, 0x645, 0x6cc, 0x646, 0x627, 0x646, 0x648, 0x627, 0x6a9, 0x634,
0x648, 0x637, 0x627, 0x624, 0x6af, 0x627, 0x688, 0x624, 0x6af, 0x648, 0x648, 0x633,
0x627, 0x624, 0x20, 0x679, 0x648, 0x645, 0x679, 0x631, 0x6cc, 0x67e, 0x648, 0x644,
0x6cc, 0x62a, 0x6cc, 0x648, 0x646, 0x633, 0x648, 0x646, 0x688, 0x6c1, 0x648, 0x6a9,
0x627, 0x646, 0x6af, 0x648, 0x6cc, 0x644, 0x627, 0x627, 0x646, 0x679, 0x6cc, 0x6af,
0x648, 0x627, 0x628, 0x6cc, 0x648, 0x646, 0x633, 0x20, 0x622, 0x626, 0x631, 0x633,
0x6a9, 0x648, 0x631, 0x688, 0x648, 0x628, 0x627, 0x62c, 0x648, 0x62c, 0x648, 0x626,
0x6cc, 0x645, 0x6cc, 0x646, 0x688, 0x648, 0x632, 0x627, 0x633, 0x627, 0x646, 0x20,
0x644, 0x648, 0x626, 0x633, 0x679, 0x648, 0x6a9, 0x648, 0x645, 0x6cc, 0x646, 0x627,
0x633, 0x646, 0x633, 0x6cc, 0x626, 0x646, 0x627, 0x679, 0x6cc, 0x6a9, 0x648, 0x6a9,
0x646, 0x628, 0x6c1, 0x6cc, 0x627, 0x20, 0x628, 0x646, 0x62f, 0x631, 0x627, 0x632,
0x628, 0x6cc, 0x644, 0x6cc, 0x645, 0x628, 0x6cc, 0x644, 0x627, 0x626, 0x632, 0x628,
0x644, 0x627, 0x646, 0x6a9, 0x20, 0x633, 0x628, 0x644, 0x648, 0x646, 0x628, 0x648,
0x622, 0x20, 0x648, 0x633, 0x679, 0x627, 0x628, 0x6af, 0x648, 0x679, 0x627, 0x628,
0x648, 0x627, 0x626, 0x633, 0x6a9, 0x6cc, 0x645, 0x628, 0x631, 0x62c, 0x20, 0x6a9,
0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc, 0x6a9, 0x6cc, 0x645, 0x67e, 0x648,
0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x688, 0x6a9, 0x646, 0x6a9, 0x6cc, 0x648, 0x646,
0x6a9, 0x631, 0x627, 0x6a9, 0x627, 0x633, 0x6a9, 0x627, 0x626, 0x6cc, 0x646, 0x6a9,
0x6cc, 0x645, 0x6cc, 0x646, 0x634, 0x6a9, 0x627, 0x6af, 0x648, 0x686, 0x6cc, 0x6c1,
0x648, 0x622, 0x6c1, 0x648, 0x622, 0x6a9, 0x648, 0x633, 0x679, 0x627, 0x20, 0x631,
0x6cc, 0x6a9, 0x627, 0x6a9, 0x631, 0x6cc, 0x633, 0x679, 0x648, 0x646, 0x6a9, 0x648,
0x626, 0x6cc, 0x627, 0x628, 0x627, 0x6a9, 0x6cc, 0x648, 0x631, 0x627, 0x6a9, 0x627,
0x624, 0x688, 0x627, 0x624, 0x633, 0x646, 0x688, 0x627, 0x624, 0x633, 0x646, 0x20,
0x6a9, 0x631, 0x6cc, 0x6a9, 0x688, 0x6cc, 0x646, 0x648, 0x631, 0x688, 0x6cc, 0x679,
0x631, 0x627, 0x626, 0x679, 0x688, 0x648, 0x645, 0x646, 0x6cc, 0x6a9, 0x627, 0x627,
0x6cc, 0x688, 0x645, 0x648, 0x646, 0x679, 0x646, 0x627, 0x6cc, 0x631, 0x648, 0x646,
0x6cc, 0x67e, 0x627, 0x644, 0x20, 0x633, 0x644, 0x648, 0x627, 0x688, 0x648, 0x631,
0x641, 0x648, 0x631, 0x679, 0x627, 0x644, 0x6cc, 0x632, 0x627, 0x6af, 0x644, 0x6cc,
0x633, 0x20, 0x6a9, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc, 0x6af, 0x648,
0x633, 0x20, 0x6a9, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691, 0x6cc, 0x639, 0x638,
0x6cc, 0x645, 0x20, 0x62a, 0x631, 0x6a9, 0x63a, 0x631, 0x646, 0x627, 0x637, 0x6c1,
0x6af, 0x648, 0x627, 0x688, 0x6cc, 0x644, 0x648, 0x67e, 0x6af, 0x648, 0x627, 0x679,
0x6d2, 0x20, 0x645, 0x627, 0x644, 0x627, 0x6af, 0x648, 0x622, 0x6cc, 0x627, 0x6a9,
0x648, 0x626, 0x644, 0x6af, 0x6cc, 0x627, 0x646, 0x627, 0x6c1, 0x6cc, 0x644, 0x6cc,
0x641, 0x6cc, 0x6a9, 0x633, 0x6c1, 0x648, 0x627, 0x646, 0x627, 0x6c1, 0x631, 0x645,
0x648, 0x633, 0x6cc, 0x644, 0x648, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627,
0x67e, 0x648, 0x644, 0x633, 0x6a9, 0x646, 0x648, 0x6a9, 0x633, 0x60c, 0x20, 0x627,
0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x645, 0x631, 0x6cc, 0x646, 0x6af, 0x648,
0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x67e, 0x6cc, 0x679,
0x631, 0x632, 0x628, 0x631, 0x6af, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627,
0x646, 0x627, 0x679, 0x6cc, 0x644, 0x20, 0x633, 0x679, 0x6cc, 0x60c, 0x20, 0x627,
0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x6cc, 0x648, 0x6d2, 0x60c, 0x20,
0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x646, 0x633, 0x6cc, 0x646,
0x6cc, 0x632, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc, 0x627, 0x646, 0x627, 0x648,
0x6cc, 0x646, 0x627, 0x645, 0x6cc, 0x6a9, 0x60c, 0x20, 0x627, 0x646, 0x688, 0x6cc,
0x627, 0x646, 0x627, 0x627, 0x646, 0x648, 0x648, 0x650, 0x6a9, 0x627, 0x6cc, 0x6a9,
0x627, 0x644, 0x648, 0x626, 0x679, 0x62c, 0x645, 0x627, 0x626, 0x6cc, 0x6a9, 0x627,
0x62c, 0x648, 0x646, 0x6cc, 0x626, 0x648, 0x644, 0x648, 0x626, 0x633, 0x20, 0x648,
0x6cc, 0x644, 0x6d2, 0x645, 0x648, 0x646, 0x679, 0x6cc, 0x633, 0x6cc, 0x644, 0x648,
0x60c, 0x20, 0x6a9, 0x6cc, 0x646, 0x679, 0x648, 0x6a9, 0x6cc, 0x6a9, 0x631, 0x627,
0x644, 0x6cc, 0x646, 0x688, 0x6cc, 0x62c, 0x6a9, 0x644, 0x648, 0x648, 0x631, 0x20,
0x67e, 0x631, 0x646, 0x633, 0x633, 0x20, 0x6a9, 0x648, 0x627, 0x631, 0x679, 0x631,
0x645, 0x646, 0x627, 0x6af, 0x648, 0x622, 0x645, 0x646, 0x627, 0x624, 0x633, 0x645,
0x627, 0x631, 0x679, 0x6cc, 0x646, 0x6a9, 0x645, 0x6cc, 0x632, 0x679, 0x644, 0x627,
0x646, 0x645, 0x6cc, 0x646, 0x648, 0x645, 0x6cc, 0x646, 0x6cc, 0x645, 0x6cc, 0x631,
0x6cc, 0x688, 0x627, 0x645, 0x6cc, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x648, 0x20, 0x633,
0x679, 0x6cc, 0x645, 0x6cc, 0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646, 0x645, 0x648,
0x646, 0x6a9, 0x679, 0x646, 0x645, 0x648, 0x646, 0x679, 0x6cc, 0x631, 0x6cc, 0x645,
0x648, 0x646, 0x679, 0x6cc, 0x20, 0x648, 0x6cc, 0x688, 0x6cc, 0x648, 0x645, 0x648,
0x646, 0x679, 0x633, 0x6cc, 0x631, 0x627, 0x679, 0x646, 0x6cc, 0x648, 0x20, 0x6cc,
0x627, 0x631, 0x6a9, 0x628, 0x6cc, 0x648, 0x644, 0x627, 0x6c1, 0x60c, 0x20, 0x634,
0x645, 0x627, 0x644, 0x6cc, 0x20, 0x688, 0x6a9, 0x648, 0x679, 0x627, 0x648, 0x633,
0x637, 0x60c, 0x20, 0x634, 0x645, 0x627, 0x644, 0x6cc, 0x20, 0x688, 0x6a9, 0x648,
0x679, 0x627, 0x646, 0x6cc, 0x648, 0x20, 0x633, 0x644, 0x6cc, 0x645, 0x60c, 0x20,
0x634, 0x645, 0x627, 0x644, 0x6cc, 0x20, 0x688, 0x6a9, 0x648, 0x679, 0x627, 0x627,
0x648, 0x62c, 0x6cc, 0x646, 0x627, 0x6af, 0x627, 0x67e, 0x646, 0x627, 0x645, 0x6c1,
0x67e, 0x631, 0x627, 0x645, 0x627, 0x631, 0x6cc, 0x628, 0x648, 0x67e, 0x648, 0x631,
0x679, 0x20, 0x627, 0x648, 0x20, 0x67e, 0x631, 0x646, 0x633, 0x67e, 0x648, 0x631,
0x679, 0x20, 0x622, 0x641, 0x20, 0x627, 0x633, 0x67e, 0x6cc, 0x646, 0x67e, 0x648,
0x631, 0x679, 0x648, 0x20, 0x648, 0x6cc, 0x644, 0x6c1, 0x648, 0x67e, 0x6cc, 0x648,
0x631, 0x679, 0x648, 0x20, 0x631, 0x6cc, 0x6a9, 0x648, 0x67e, 0x646, 0x679, 0x627,
0x20, 0x627, 0x631, 0x6cc, 0x646, 0x627, 0x633, 0x631, 0x6cc, 0x646, 0x6a9, 0x646,
0x20, 0x627, 0x646, 0x644, 0x6cc, 0x679, 0x631, 0x6cc, 0x633, 0x627, 0x626, 0x641,
0x631, 0x6cc, 0x632, 0x648, 0x644, 0x6cc, 0x648, 0x679, 0x631, 0x6cc, 0x626, 0x648,
0x20, 0x628, 0x631, 0x6cc, 0x646, 0x6a9, 0x648, 0x633, 0x646, 0x679, 0x627, 0x631,
0x6cc, 0x646, 0x633, 0x646, 0x679, 0x6cc, 0x627, 0x6af, 0x648, 0x633, 0x627, 0x646,
0x62a, 0x648, 0x20, 0x688, 0x648, 0x645, 0x646, 0x6af, 0x648, 0x627, 0x633, 0x6a9,
0x648, 0x631, 0x632, 0x20, 0x628, 0x627, 0x626, 0x6cc, 0x20, 0x633, 0x646, 0x688,
0x633, 0x6cc, 0x679, 0x6a9, 0x627, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x628, 0x631,
0x62a, 0x6be, 0x6cc, 0x644, 0x645, 0x6cc, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x62c,
0x627, 0x646, 0x632, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x6a9, 0x679, 0x633, 0x633,
0x6cc, 0x646, 0x679, 0x20, 0x644, 0x648, 0x633, 0x6cc, 0x627, 0x633, 0x6cc, 0x646,
0x679, 0x20, 0x62a, 0x6be, 0x627, 0x645, 0x633, 0x633, 0x6cc, 0x646, 0x679, 0x20,
0x648, 0x646, 0x633, 0x646, 0x679, 0x633, 0x648, 0x650, 0x641, 0x679, 0x20, 0x6a9,
0x631, 0x646, 0x679, 0x679, 0x6cc, 0x6af, 0x648, 0x633, 0x6cc, 0x6af, 0x627, 0x644,
0x67e, 0x6d2, 0x62a, 0x6be, 0x648, 0x644, 0x648, 0x62a, 0x6cc, 0x62c, 0x648, 0x622,
0x646, 0x627, 0x679, 0x648, 0x631, 0x646, 0x679, 0x648, 0x679, 0x648, 0x631, 0x679,
0x648, 0x644, 0x627, 0x648, 0x6c1, 0x627, 0x626, 0x679, 0x20, 0x6c1, 0x627, 0x631,
0x633, 0x648, 0x646, 0x651, 0x6cc, 0x67e, 0x6cc, 0x6af, 0x6cc, 0x6a9, 0x648, 0x679,
0x6cc, 0x679, 0x688, 0x6cc, 0x648, 0x633, 0x688, 0x648, 0x645, 0x648, 0x646, 0x679,
0x20, 0x688, 0x6cc, 0x20, 0x627, 0x631, 0x648, 0x6cc, 0x644, 0x6d2, 0x645, 0x6cc,
0x6a9, 0x648, 0x627, 0x631, 0x6cc, 0x645, 0x627, 0x624, 0x633, 0x646, 0x645, 0x6cc,
0x6a9, 0x20, 0x645, 0x631, 0x688, 0x648, 0x67e, 0x644, 0x645, 0x6cc, 0x631, 0x679,
0x631, 0x648, 0x644, 0x644, 0x627, 0x646, 0x6af, 0x20, 0x627, 0x6cc, 0x626, 0x631,
0x20, 0x628, 0x6cc, 0x646, 0x627, 0x644, 0x645, 0x627, 0x679, 0x6cc, 0x627, 0x645,
0x651, 0x627, 0x646, 0x627, 0x646, 0x6cc, 0x62f, 0x631, 0x627, 0x6a9, 0x62a, 0x627,
0x624, 0x627, 0x6a9, 0x679, 0x648, 0x628, 0x627, 0x634, 0x63a, 0x628, 0x62a, 0x622,
0x62a, 0x6cc, 0x631, 0x627, 0x624, 0x628, 0x631, 0x646, 0x627, 0x644, 0x686, 0x648,
0x626, 0x628, 0x627, 0x644, 0x633, 0x627, 0x646, 0x688, 0x6be, 0x627, 0x6a9, 0x6c1,
0x688, 0x644, 0x6cc, 0x62f, 0x628, 0x626, 0x6cc, 0x62f, 0x648, 0x634, 0x627, 0x646,
0x628, 0x6d2, 0x641, 0x627, 0x645, 0x627, 0x6af, 0x648, 0x633, 0x679, 0x627, 0x6c1,
0x648, 0x20, 0x686, 0x6cc, 0x20, 0x645, 0x646, 0x6c1, 0x20, 0x633, 0x679, 0x6cc,
0x6c1, 0x648, 0x648, 0x627, 0x631, 0x688, 0x627, 0x631, 0x6a9, 0x62a, 0x633, 0x6a9,
0x62c, 0x6a9, 0x627, 0x631, 0x62a, 0x6c1, 0x62c, 0x6d2, 0x20, 0x67e, 0x648, 0x631,
0x6c1, 0x6a9, 0x6cc, 0x645, 0x686, 0x679, 0x6a9, 0x627, 0x6a9, 0x627, 0x679, 0x6be,
0x645, 0x646, 0x688, 0x648, 0x62e, 0x646, 0x62f, 0x6cc, 0x6af, 0x627, 0x6a9, 0x648,
0x644, 0x6a9, 0x627, 0x62a, 0x627, 0x6a9, 0x631, 0x6cc, 0x633, 0x646, 0x648, 0x6cc,
0x627, 0x631, 0x633, 0x6a9, 0x6a9, 0x648, 0x627, 0x644, 0x627, 0x20, 0x644, 0x645,
0x67e, 0x648, 0x631, 0x6a9, 0x6cc, 0x648, 0x686, 0x646, 0x6af, 0x645, 0x6a9, 0x627,
0x624, 0x645, 0x6cc, 0x6af, 0x6cc, 0x62f, 0x646, 0x645, 0x6a9, 0x627, 0x633, 0x631,
0x646, 0x6a9, 0x648, 0x633, 0x6cc, 0x627, 0x646, 0x648, 0x648, 0x648, 0x6a9, 0x6cc,
0x648, 0x632, 0x646, 0x6cc, 0x633, 0x6a9, 0x67e, 0x648, 0x646, 0x679, 0x6cc, 0x627,
0x646, 0x6a9, 0x67e, 0x6cc, 0x648, 0x646, 0x6af, 0x20, 0x6cc, 0x627, 0x646, 0x6af,
0x6a9, 0x648, 0x633, 0x62a, 0x627, 0x646, 0x6d2, 0x6a9, 0x6cc, 0x632, 0x6cc, 0x644,
0x648, 0x631, 0x688, 0x627, 0x633, 0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x633, 0x6cc,
0x626, 0x648, 0x644, 0x633, 0x631, 0x6c1, 0x62f, 0x646, 0x6cc, 0x6a9, 0x648, 0x644,
0x6cc, 0x645, 0x633, 0x6a9, 0x62a, 0x627, 0x626, 0x67e, 0x6d2, 0x637, 0x628, 0x644,
0x6cc, 0x633, 0x6cc, 0x62a, 0x6c1, 0x631, 0x627, 0x646, 0x62a, 0x6be, 0x645, 0x67e,
0x648, 0x679, 0x627, 0x645, 0x633, 0x6a9, 0x627, 0x648, 0x644, 0x627, 0x646, 0x20,
0x628, 0x627, 0x62a, 0x627, 0x631, 0x6cc, 0x648, 0x631, 0x648, 0x645, 0x6a9, 0x6cc,
0x627, 0x648, 0x633, 0x62a, 0x2d, 0x646, 0x6cc, 0x631, 0x627, 0x648, 0x6cc, 0x646,
0x679, 0x6cc, 0x627, 0x646, 0x627, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x20, 0x648,
0x648, 0x633, 0x62a, 0x6a9, 0x6cc, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x6cc, 0x6a9,
0x627, 0x679, 0x6cc, 0x631, 0x650, 0x646, 0x628, 0x631, 0x6af, 0x6cc, 0x631, 0x6cc,
0x648, 0x627, 0x646, 0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633, 0x6a9, 0x6cc, 0x67e,
0x20, 0x648, 0x631, 0x688, 0x6cc, 0x645, 0x688, 0x6cc, 0x626, 0x631, 0x627, 0x631,
0x6cc, 0x6a9, 0x62c, 0x627, 0x648, 0x6a9, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x6c1,
0x6cc, 0x644, 0x6cc, 0x646, 0x627, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x644, 0x6d2,
0x627, 0x6cc, 0x688, 0x6cc, 0x644, 0x6cc, 0x688, 0x628, 0x631, 0x648, 0x6a9, 0x646,
0x20, 0x6c1, 0x650, 0x644, 0x688, 0x627, 0x631, 0x648, 0x646, 0x627, 0x6cc, 0x648,
0x6a9, 0x644, 0x627, 0x6c1, 0x648, 0x628, 0x627, 0x631, 0x679, 0x644, 0x650, 0x646,
0x688, 0x645, 0x6cc, 0x646, 0x644, 0x627, 0x631, 0x688, 0x20, 0x6c1, 0x648, 0x648,
0x6d2, 0x67e, 0x631, 0x62a, 0x6be, 0x633, 0x688, 0x646, 0x6cc, 0x627, 0x6cc, 0x645,
0x633, 0x679, 0x631, 0x688, 0x645, 0x627, 0x646, 0x688, 0x648, 0x631, 0x627, 0x627,
0x633, 0x62a, 0x631, 0x627, 0x62e, 0x627, 0x646, 0x627, 0x6cc, 0x62a, 0x6be, 0x646,
0x632, 0x628, 0x631, 0x6cc, 0x679, 0x650, 0x633, 0x644, 0x627, 0x648, 0x627, 0x628,
0x62e, 0x627, 0x631, 0x633, 0x679, 0x628, 0x688, 0x627, 0x67e, 0x633, 0x679, 0x628,
0x632, 0x646, 0x62c, 0x646, 0x686, 0x6cc, 0x633, 0x6cc, 0x646, 0x627, 0x624, 0x688,
0x628, 0x644, 0x646, 0x6c1, 0x6cc, 0x644, 0x633, 0x646, 0x6a9, 0x6cc, 0x6a9, 0x627,
0x644, 0x64a, 0x646, 0x64a, 0x646, 0x63a, 0x631, 0x627, 0x62f, 0x644, 0x6af, 0x698,
0x645, 0x628, 0x631, 0x6af, 0x645, 0x6cc, 0x688, 0x631, 0x688, 0x645, 0x6cc, 0x631,
0x6cc, 0x6c1, 0x627, 0x645, 0x67e, 0x6cc, 0x631, 0x633, 0x67e, 0x648, 0x688, 0x6af,
0x648, 0x631, 0x633, 0x6cc, 0x627, 0x633, 0x627, 0x646, 0x20, 0x645, 0x627, 0x631,
0x6cc, 0x627, 0x646, 0x648, 0x633, 0x6cc, 0x631, 0x627, 0x679, 0x648, 0x627, 0x633,
0x6a9, 0x648, 0x67e, 0x62c, 0x6d2, 0x635, 0x648, 0x641, 0x6cc, 0x6c1, 0x627, 0x633,
0x679, 0x627, 0x6a9, 0x20, 0x6c1, 0x648, 0x645, 0x679, 0x627, 0x644, 0x646, 0x679,
0x6cc, 0x631, 0x627, 0x646, 0x6cc, 0x627, 0x644, 0x6cc, 0x627, 0x646, 0x648, 0x633,
0x6a9, 0x648, 0x6cc, 0x688, 0x648, 0x632, 0x648, 0x627, 0x679, 0x6cc, 0x6a9, 0x646,
0x648, 0x650, 0x644, 0x646, 0x6cc, 0x626, 0x633, 0x632, 0x6cc, 0x6af, 0x631, 0x6cc,
0x628, 0x632, 0x6cc, 0x648, 0x631, 0x62e, 0x627, 0x646, 0x679, 0x627, 0x646, 0x627,
0x646, 0x627, 0x631, 0x6cc, 0x648, 0x648, 0x6a9, 0x631, 0x633, 0x645, 0x633, 0x6a9,
0x631, 0x6af, 0x6cc, 0x648, 0x644, 0x6cc, 0x646, 0x645, 0x627, 0x644, 0x62f, 0x6cc,
0x67e, 0x645, 0x627, 0x631, 0x6cc, 0x634, 0x633, 0x645, 0x627, 0x6cc, 0x648, 0x679,
0x631, 0x6cc, 0x20, 0x6cc, 0x648, 0x646, 0x6cc, 0x646, 0x628, 0x648, 0x6af, 0x6cc,
0x646, 0x648, 0x644, 0x6d2, 0x686, 0x6cc, 0x62a, 0x6be, 0x645, 0x686, 0x6cc, 0x648,
0x6a9, 0x627, 0x6cc, 0x641, 0x6cc, 0x679, 0x641, 0x6a9, 0x627, 0x624, 0x641, 0x648,
0x641, 0x62c, 0x6cc, 0x641, 0x6cc, 0x648, 0x646, 0x627, 0x641, 0x6cc, 0x648, 0x679,
0x6cc, 0x6af, 0x6cc, 0x644, 0x627, 0x67e, 0x6cc, 0x6af, 0x648, 0x633, 0x6af, 0x627,
0x645, 0x628, 0x6cc, 0x626, 0x631, 0x6af, 0x648, 0x627, 0x688, 0x644, 0x20, 0x6a9,
0x6cc, 0x646, 0x627, 0x644, 0x6c1, 0x648, 0x646, 0x648, 0x644, 0x648, 0x644, 0x648,
0x627, 0x6cc, 0x646, 0x688, 0x631, 0x628, 0x631, 0x6cc, 0x6a9, 0x631, 0x6cc, 0x62a,
0x6cc, 0x645, 0x627, 0x679, 0x6cc, 0x6a9, 0x648, 0x633, 0x631, 0x627, 0x626, 0x6cc,
0x645, 0x627, 0x631, 0x6a9, 0x6cc, 0x633, 0x627, 0x633, 0x645, 0x688, 0x648, 0x6d2,
0x646, 0x627, 0x624, 0x631, 0x648, 0x646, 0x624, 0x645, 0x6cc, 0x627, 0x67e, 0x644,
0x627, 0x624, 0x67e, 0x679, 0x6a9, 0x627, 0x626, 0x631, 0x646, 0x67e, 0x648, 0x646,
0x67e, 0x6cc, 0x626, 0x6cc, 0x67e, 0x648, 0x631, 0x679, 0x20, 0x645, 0x648, 0x631,
0x6cc, 0x633, 0x628, 0x6cc, 0x633, 0x627, 0x626, 0x67e, 0x6cc, 0x646, 0x62a, 0x627,
0x6c1, 0x6cc, 0x62a, 0x6cc, 0x679, 0x631, 0x627, 0x648, 0x627, 0x679, 0x648, 0x646,
0x6af, 0x627, 0x679, 0x627, 0x67e, 0x648, 0x648, 0x644, 0x651, 0x6cc, 0x633, 0x6a9,
0x6cc, 0x645, 0x628, 0x631, 0x62c, 0x20, 0x628, 0x6d2, 0x6af, 0x644, 0x6cc, 0x633,
0x20, 0x628, 0x6d2, 0x6af, 0x648, 0x633, 0x20, 0x628, 0x6d2, 0x6af, 0x631, 0x6cc,
0x646, 0x688, 0x20, 0x62a, 0x631, 0x6a9, 0x627, 0x62a, 0x62a, 0x648, 0x642, 0x642,
0x648, 0x631, 0x62a, 0x648, 0x631, 0x645, 0x6cc, 0x62a, 0x62a, 0x6be, 0x648, 0x644,
0x6d2, 0x628, 0x648, 0x688, 0x627, 0x67e, 0x6cc, 0x633, 0x679, 0x633, 0x627, 0x646,
0x2d, 0x62a, 0x648, 0x645, 0x6d0, 0x626, 0x627, 0x633, 0x6c7, 0x646, 0x633, 0x649,
0x64a, 0x648, 0x646, 0x643, 0x6c7, 0x631, 0x627, 0x633, 0x648, 0x633, 0x627, 0x64a,
0x646, 0x649, 0x62a, 0x2d, 0x628, 0x627, 0x631, 0x62a, 0x6be, 0x6d0, 0x644, 0x6d5,
0x645, 0x649, 0x64a, 0x62f, 0x6c7, 0x645, 0x648, 0x646, 0x62a, 0x20, 0x62f, 0x6c7,
0x631, 0x6cb, 0x649, 0x644, 0x644, 0x6d0, 0x62e, 0x648, 0x686, 0x649, 0x645, 0x649,
0x646, 0x20, 0x634, 0x6d5, 0x6be, 0x649, 0x631, 0x649, 0x631, 0x6d0, 0x626, 0x648,
0x646, 0x649, 0x64a, 0x648, 0x646, 0x4a, 0x61, 0x7a, 0x6f, 0x69, 0x72, 0x42,
0x6c, 0x61, 0x6e, 0x74, 0x61, 0x79, 0x72, 0x51, 0x6f, 0x68, 0x69, 0x72,
0x61, 0x44, 0x6f, 0x72, 0x2d, 0x75, 0x73, 0x2d, 0x53, 0x61, 0x6c, 0x6f,
0x6d, 0x41, 0x6c, 0x2d, 0x41, 0x79, 0x75, 0x6e, 0x58, 0x61, 0x72, 0x61,
0x72, 0x65, 0x59, 0x6f, 0x78, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75,
0x72, 0x67, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x73, 0x68, 0x6f, 0x4d,
0x6f, 0x6e, 0x72, 0x6f, 0x76, 0x69, 0x79, 0x61, 0x4e, 0x61, 0x79, 0x72,
0x6f, 0x62, 0x69, 0x56, 0x69, 0x6e, 0x64, 0x78, 0x75, 0x6b, 0x41, 0x64,
0x61, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x41, 0x6e, 0x6b, 0x6f,
0x72, 0x69, 0x6a, 0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x2d, 0x41, 0x79,
0x72, 0x65, 0x73, 0x4c, 0x61, 0x2d, 0x52, 0x69, 0x6f, 0x78, 0x61, 0x52,
0x69, 0x6f, 0x2d, 0x47, 0x61, 0x6c, 0x79, 0x65, 0x67, 0x6f, 0x73, 0x53,
0x61, 0x6e, 0x2d, 0x58, 0x75, 0x61, 0x6e, 0x55, 0x73, 0x68, 0x75, 0x61,
0x79, 0x61, 0x41, 0x73, 0x75, 0x6e, 0x73, 0x6f, 0x6e, 0x4b, 0x6f, 0x72,
0x61, 0x6c, 0x2d, 0x58, 0x61, 0x72, 0x62, 0x6f, 0x72, 0x42, 0x61, 0x69,
0x79, 0x61, 0x42, 0x61, 0x68, 0x69, 0x79, 0x61, 0x2d, 0x42, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x61, 0x73, 0x4b, 0x65, 0x6d, 0x62, 0x72, 0x69, 0x6a,
0x2d, 0x42, 0x65, 0x79, 0x4b, 0x61, 0x79, 0x65, 0x6e, 0x6e, 0x61, 0x4b,
0x61, 0x79, 0x6d, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61,
0x72, 0x69, 0x43, 0x68, 0x69, 0x6b, 0x61, 0x67, 0x6f, 0x4b, 0x79, 0x75,
0x72, 0x61, 0x73, 0x61, 0x6f, 0x44, 0x65, 0x6e, 0x6d, 0x61, 0x72, 0x6b,
0x73, 0x78, 0x61, 0x76, 0x6e, 0x45, 0x79, 0x72, 0x75, 0x6e, 0x65, 0x70,
0x65, 0x47, 0x6c, 0x65, 0x79, 0x73, 0x2d, 0x42, 0x65, 0x79, 0x47, 0x75,
0x73, 0x2d, 0x42, 0x65, 0x79, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6b, 0x69,
0x6c, 0x47, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x47, 0x61, 0x76, 0x61, 0x6e,
0x61, 0x45, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6c, 0x6f, 0x50, 0x69,
0x74, 0x65, 0x72, 0x73, 0x62, 0x65, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x76, 0x65, 0x79, 0x2c, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56, 0x69, 0x6e, 0x73, 0x65,
0x6e, 0x73, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x56,
0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x4a, 0x75, 0x6e, 0x6f, 0x4c, 0x75, 0x69, 0x73, 0x76,
0x69, 0x6c, 0x6c, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73, 0x65, 0x6c, 0x6c,
0x6f, 0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x6b, 0x6b, 0x69, 0x4b,
0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x65, 0x79, 0x6b, 0x4c, 0x6f, 0x75,
0x65, 0x72, 0x2d, 0x50, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x73, 0x2d, 0x4b,
0x75, 0x6f, 0x72, 0x74, 0x65, 0x72, 0x4e, 0x79, 0x75, 0x2d, 0x59, 0x6f,
0x72, 0x6b, 0x42, 0x6f, 0x79, 0x6c, 0x61, 0x2c, 0x20, 0x53, 0x68, 0x69,
0x6d, 0x6f, 0x6c, 0x69, 0x79, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x2c, 0x20, 0x53, 0x68, 0x69, 0x6d,
0x6f, 0x6c, 0x69, 0x79, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e,
0x79, 0x75, 0x2d, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x53, 0x68,
0x69, 0x6d, 0x6f, 0x6c, 0x69, 0x79, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x47, 0x6f, 0x74, 0x78, 0x6f, 0x62, 0x4f, 0x78, 0x69, 0x6e, 0x61,
0x67, 0x61, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x6f, 0x66, 0x2d, 0x53, 0x70,
0x65, 0x79, 0x6e, 0x50, 0x6f, 0x72, 0x74, 0x75, 0x2d, 0x56, 0x65, 0x6c,
0x79, 0x75, 0x52, 0x65, 0x6a, 0x61, 0x79, 0x6e, 0x61, 0x53, 0x61, 0x6e,
0x74, 0x79, 0x61, 0x67, 0x6f, 0x49, 0x74, 0x74, 0x6f, 0x6b, 0x6b, 0x6f,
0x72, 0x74, 0x6f, 0x6f, 0x72, 0x6d, 0x69, 0x75, 0x74, 0x53, 0x65, 0x6e,
0x74, 0x2d, 0x4c, 0x79, 0x75, 0x73, 0x69, 0x79, 0x61, 0x53, 0x65, 0x6e,
0x74, 0x2d, 0x56, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0x53, 0x76, 0x69,
0x66, 0x74, 0x2d, 0x4b, 0x61, 0x72, 0x72, 0x65, 0x6e, 0x74, 0x55, 0x61,
0x79, 0x74, 0x78, 0x6f, 0x72, 0x73, 0x56, 0x69, 0x6e, 0x6e, 0x69, 0x70,
0x65, 0x67, 0x44, 0x79, 0x75, 0x6d, 0x6f, 0x6e, 0x2d, 0x64, 0x2019, 0x59,
0x75, 0x72, 0x76, 0x69, 0x6c, 0x4c, 0x6f, 0x6e, 0x67, 0x79, 0x69, 0x72,
0x41, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x6d, 0x6f, 0x6e, 0x4f, 0x71, 0x74,
0x6f, 0x76, 0x4f, 0x71, 0x74, 0x6f, 0x2018, 0x62, 0x61, 0x41, 0x73, 0x68,
0x78, 0x6f, 0x62, 0x6f, 0x64, 0x42, 0x61, 0x67, 0x2018, 0x64, 0x6f, 0x64,
0x42, 0x61, 0x68, 0x72, 0x61, 0x79, 0x6e, 0x42, 0x6f, 0x6b, 0x75, 0x44,
0x61, 0x6d, 0x61, 0x73, 0x68, 0x71, 0x47, 0x2018, 0x61, 0x7a, 0x6f, 0x58,
0x65, 0x76, 0x72, 0x6f, 0x6e, 0x58, 0x6f, 0x73, 0x68, 0x69, 0x6d, 0x69,
0x6e, 0x58, 0x6f, 0x76, 0x64, 0x4a, 0x61, 0x79, 0x70, 0x75, 0x72, 0x51,
0x75, 0x64, 0x64, 0x75, 0x73, 0x51, 0x6f, 0x62, 0x75, 0x6c, 0x58, 0x61,
0x6e, 0x64, 0x69, 0x67, 0x61, 0x51, 0x75, 0x76, 0x61, 0x79, 0x74, 0x51,
0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x2018, 0x72, 0x64, 0x61, 0x41, 0x72, 0x2d,
0x52, 0x69, 0x79, 0x6f, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x71, 0x61,
0x6e, 0x64, 0x53, 0x68, 0x61, 0x6e, 0x78, 0x61, 0x79, 0x54, 0x61, 0x79,
0x70, 0x65, 0x79, 0x54, 0x65, 0x68, 0x72, 0x6f, 0x6e, 0x55, 0x72, 0x75,
0x6d, 0x63, 0x68, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x6f, 0x72, 0x6f,
0x6c, 0x6c, 0x61, 0x72, 0x69, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x61,
0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4b, 0x61, 0x6e,
0x61, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4b,
0x61, 0x62, 0x6f, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x46, 0x61, 0x72,
0x65, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4d,
0x61, 0x64, 0x65, 0x79, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69,
0x4a, 0x61, 0x6e, 0x75, 0x62, 0x69, 0x79, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x79, 0x61, 0x4d, 0x75, 0x71, 0x61, 0x64, 0x64, 0x61, 0x73,
0x20, 0x59, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x69, 0x42, 0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x2d, 0x58, 0x69, 0x6c, 0x6c,
0x45, 0x76, 0x6b, 0x6c, 0x61, 0x58, 0x6f, 0x62, 0x61, 0x72, 0x74, 0x4c,
0x6f, 0x72, 0x64, 0x2d, 0x58, 0x61, 0x75, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x69, 0x42, 0x72, 0x79, 0x75, 0x73, 0x73, 0x65, 0x6c, 0x42, 0x75, 0x64,
0x61, 0x70, 0x65, 0x73, 0x68, 0x74, 0x42, 0x79, 0x75, 0x7a, 0x69, 0x6e,
0x67, 0x65, 0x6e, 0x4b, 0x69, 0x73, 0x68, 0x69, 0x6e, 0x79, 0x6f, 0x76,
0x58, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x4d, 0x65, 0x6e, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x4c, 0x79, 0x75, 0x6b, 0x73, 0x65, 0x6d,
0x62, 0x75, 0x72, 0x67, 0x4d, 0x61, 0x72, 0x69, 0x79, 0x65, 0x78, 0x61,
0x6d, 0x6e, 0x50, 0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x74, 0x73, 0x61,
0x56, 0x65, 0x6e, 0x61, 0x56, 0x61, 0x72, 0x73, 0x68, 0x61, 0x76, 0x61,
0x53, 0x79, 0x75, 0x72, 0x69, 0x78, 0x52, 0x6f, 0x6a, 0x64, 0x65, 0x73,
0x74, 0x76, 0x6f, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x4b, 0x6f, 0x6b,
0x6f, 0x73, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4b,
0x6f, 0x6d, 0x6f, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72,
0x69, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x6c, 0x61, 0x72, 0x69, 0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x79,
0x4d, 0x61, 0x79, 0x6f, 0x72, 0x6b, 0x61, 0x52, 0x65, 0x79, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x43, 0x68, 0x61, 0x74, 0x65, 0x6d, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x69, 0x54, 0x72, 0x75, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x6c, 0x61, 0x72, 0x69, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x69, 0x47, 0x61, 0x6d, 0x62, 0x79, 0x65, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x69, 0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0x65, 0x72, 0x69,
0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a,
0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x4d, 0x69, 0x64,
0x76, 0x65, 0x79, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69,
0x50, 0x6f, 0x6e, 0x70, 0x65, 0x69, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69,
0x54, 0x61, 0x69, 0x74, 0x69, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x55,
0x65, 0x79, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x4c, 0x6f, 0x77,
0x65, 0x72, 0x20, 0x50, 0x72, 0x69, 0x6e, 0x63, 0x65, 0x2bc, 0x73, 0x20,
0x51, 0x75, 0x61, 0x72, 0x74, 0x65, 0x72, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x42, 0x1eaf, 0x63, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x42, 0x1eaf, 0x63,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53,
0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x42, 0x1eaf, 0x63, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x54, 0x50, 0x20, 0x48, 0x1ed3, 0x20, 0x43, 0x68,
0xed, 0x20, 0x4d, 0x69, 0x6e, 0x68, 0x48, 0x1ed3, 0x6e, 0x67, 0x20, 0x4b,
0xf4, 0x6e, 0x67, 0x4d, 0x61, 0x20, 0x43, 0x61, 0x6f, 0x42, 0xec, 0x6e,
0x68, 0x20, 0x4e, 0x68, 0x1b0, 0x1ee1, 0x6e, 0x67, 0x54, 0x68, 0x1b0, 0x1ee3,
0x6e, 0x67, 0x20, 0x48, 0x1ea3, 0x69, 0x110, 0xe0, 0x69, 0x20, 0x42, 0x1eaf,
0x63, 0x56, 0x69, 0xea, 0x6e, 0x67, 0x20, 0x43, 0x68, 0x103, 0x6e, 0x4e,
0x61, 0x6d, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x110, 0x1ea3,
0x6f, 0x20, 0x4d, 0x61, 0x6e, 0x4d, 0xe1, 0x74, 0x2d, 0x78, 0x63, 0x1a1,
0x2d, 0x76, 0x61, 0x44, 0x161, 0x69, 0x62, 0x75, 0x74, 0x69, 0x4b, 0x61,
0x69, 0x6d, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6c, 0x61, 0x48, 0x61, 0x77,
0x61, 0x6e, 0x6e, 0x61, 0x52, 0x69, 0x6b, 0x6a, 0x61, 0x77, 0x69, 0x6b,
0x53, 0x69, 0x64, 0x6e, 0x65, 0x69, 0x41, 0x6d, 0x161, 0x74, 0x65, 0x72,
0x64, 0x61, 0x6d, 0x4b, 0x6f, 0x6e, 0x161, 0x74, 0x61, 0x6e, 0x74, 0x69,
0x6e, 0x6f, 0x70, 0x65, 0x6c, 0x4b, 0xf6, 0x6e, 0x69, 0x67, 0x73, 0x62,
0xe4, 0x72, 0x67, 0x4c, 0x61, 0x69, 0x62, 0x61, 0x10d, 0x52, 0x65, 0x76,
0x61, 0x6c, 0x57, 0x61, 0x72, 0x161, 0x61, 0x75, 0x5a, 0xfc, 0x72, 0x69,
0x10d, 0x42, 0x61, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61,
0x73, 0x42, 0x61, 0x65, 0x20, 0x43, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x64,
0x67, 0x65, 0x44, 0x69, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x65, 0x6c, 0x6c,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x44, 0x69, 0x6e,
0x61, 0x73, 0x20, 0x4d, 0x65, 0x63, 0x73, 0x69, 0x63, 0x6f, 0x45, 0x66,
0x72, 0x6f, 0x67, 0x20, 0x4e, 0x65, 0x77, 0x79, 0x64, 0x64, 0x42, 0x65,
0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64,
0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74,
0x65, 0x72, 0x2c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64,
0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x44, 0x69, 0x6e, 0x61, 0x73,
0x20, 0x48, 0xf4, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x68,
0x4a, 0x65, 0x72, 0x77, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x59, 0x72, 0x20,
0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x44, 0x65, 0x64,
0x77, 0x79, 0x64, 0x64, 0x46, 0x66, 0x61, 0x72, 0x6f, 0x44, 0x65, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x42, 0x72, 0x77, 0x73, 0x65,
0x6c, 0x42, 0x77, 0x63, 0x61, 0x72, 0xe9, 0x73, 0x74, 0x44, 0x75, 0x6c,
0x79, 0x6e, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x47, 0x61, 0x72,
0x6e, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4d, 0x61, 0x6e, 0x61, 0x77, 0x43,
0x61, 0x65, 0x72, 0x67, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e, 0x69, 0x6e,
0x4c, 0x6c, 0x75, 0x6e, 0x64, 0x61, 0x69, 0x6e, 0x4c, 0x77, 0x63, 0x73,
0x65, 0x6d, 0x62, 0x77, 0x72, 0x67, 0x52, 0x68, 0x75, 0x66, 0x61, 0x69,
0x6e, 0x59, 0x20, 0x46, 0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x46, 0x69,
0x65, 0x6e, 0x6e, 0x61, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x4e,
0x61, 0x64, 0x6f, 0x6c, 0x69, 0x67, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79,
0x20, 0x50, 0x61, 0x73, 0x67, 0x627, 0x644, 0x645, 0x627, 0x62a, 0x647, 0x627,
0x642, 0x62a, 0x627, 0x648, 0x627, 0x642, 0x62a, 0x648, 0x628, 0x647, 0x639, 0x634,
0x642, 0x20, 0x622, 0x628, 0x627, 0x62a, 0x628, 0x6cc, 0x634, 0x6a9, 0x6cc, 0x6a9,
0x6a9, 0x648, 0x644, 0x648, 0x645, 0x628, 0x648, 0x688, 0x627, 0x6a9, 0x627, 0x6cc,
0x648, 0x631, 0x634, 0x644, 0x6cc, 0x645, 0x6a9, 0x679, 0x645, 0x646, 0x688, 0x648,
0x6a9, 0x648, 0x6cc, 0x679, 0x642, 0x6cc, 0x632, 0x6cc, 0x644, 0x648, 0x631, 0x62f,
0x627, 0x62a, 0x627, 0x634, 0x6a9, 0x6cc, 0x646, 0x62a, 0x62a, 0x6cc, 0x628, 0x644,
0x6cc, 0x633, 0x6cc, 0x679, 0x6cc, 0x645, 0x67e, 0x648, 0x627, 0x648, 0x631, 0x645,
0x686, 0x6cc, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x641, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x6f, 0x2d, 0x73, 0x74, 0x61, 0x64, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x2c, 0x20, 0x4e, 0x6f, 0x61, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e,
0x6f, 0x61, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f,
0x61, 0x72, 0x64, 0x2d, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x50, 0xf4,
0x72, 0x74, 0x6f, 0x20, 0x56, 0x65, 0x6c, 0x68, 0x6f, 0x4b, 0x61, 0x6e,
0x61, 0x72, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x45, 0x69, 0x6c, 0x61, 0x6e,
0x6e, 0x65, 0x6e, 0x53, 0xfb, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x50, 0x72, 0x61, 0x61, 0x63, 0x68, 0x46, 0x61, 0x74, 0x69,
0x6b, 0x61, 0x61, 0x6e, 0x73, 0x74, 0xea, 0x64, 0x4b, 0x72, 0x79, 0x73,
0x74, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x6e, 0x65, 0x6e, 0x50, 0x65, 0x61, 0x73, 0x6b,
0x65, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x6e, 0x65, 0x6e, 0x45, 0x6e, 0x64,
0x65, 0x72, 0x62, 0x75, 0x72, 0x79, 0x2d, 0x65, 0x69, 0x6c, 0xe2, 0x6e,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x6e, 0x65, 0x6e, 0x5d0, 0x5b7, 0x5d1, 0x5d9, 0x5d3, 0x5d6, 0x5e9,
0x5d0, 0x5b7, 0x5df, 0x5d0, 0x5b7, 0x5e1, 0x5de, 0x5d0, 0x5b7, 0x5e8, 0x5d0, 0x5b7,
0x5d8, 0x5d5, 0x5e0, 0x5d9, 0x5e1, 0x5d5, 0x5d5, 0x5d9, 0x5e0, 0x5d8, 0x5d4, 0x5d5,
0x5e7, 0x5d4, 0x5d0, 0x20, 0x5d8, 0x5e9, 0x5d9, 0x20, 0x5de, 0x5d9, 0x5df, 0x20,
0x5e9, 0x5d8, 0x5d0, 0x5b8, 0x5d8, 0xec, 0x6c, 0xfa, 0x20, 0x41, 0x64, 0xe1,
0x6b, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x41, 0x6e, 0x6b, 0x1ecd, 0x301, 0x72,
0xe9, 0xe8, 0x6a, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x41, 0x6e, 0x67, 0xfa,
0xed, 0x6c, 0xe0, 0xec, 0x6c, 0xfa, 0x20, 0x41, 0x6e, 0x74, 0xed, 0x67,
0xf9, 0xe0, 0xec, 0x6c, 0xfa, 0x20, 0x41, 0x72, 0xfa, 0x62, 0xe1, 0xec,
0x6c, 0xfa, 0x20, 0xe0, 0x74, 0x69, 0x6b, 0x1ecd, 0x6b, 0xe0, 0x6e, 0xec,
0x6c, 0xfa, 0x20, 0x42, 0xe1, 0x68, 0xec, 0x20, 0x42, 0xe1, 0x6e, 0x64,
0x1eb9, 0x301, 0x72, 0xe1, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0xe1,
0x62, 0xe1, 0x64, 0x1ecd, 0x301, 0x1ecd, 0x300, 0x73, 0xec, 0xec, 0x6c, 0xfa,
0x20, 0x42, 0x1eb9, 0x300, 0x6c, 0xed, 0xec, 0x73, 0xec, 0xec, 0x6c, 0xfa,
0x20, 0x42, 0x6c, 0x61, 0x6e, 0x6b, 0x20, 0x53, 0x61, 0x62, 0x75, 0x6c,
0x1ecd, 0x301, 0x1ecd, 0x300, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x1ecd,
0x301, 0x69, 0x73, 0xe8, 0xec, 0x6c, 0xfa, 0x20, 0xf2, 0x6b, 0x75, 0x6e,
0x20, 0x6b, 0x61, 0x6d, 0x62, 0xed, 0x72, 0xed, 0xec, 0x6a, 0xec, 0xec,
0x6c, 0xfa, 0x20, 0x4b, 0x61, 0x6e, 0x63, 0x75, 0x6e, 0x69, 0x6c, 0xfa,
0x20, 0x4b, 0x61, 0x79, 0x6d, 0x61, 0x6e, 0xec, 0x6c, 0xfa, 0x20, 0x43,
0x68, 0x69, 0x63, 0x61, 0x67, 0x6f, 0xec, 0x6c, 0xfa, 0x20, 0x1e62, 0x69,
0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4b, 0x6f,
0x73, 0x74, 0x61, 0x20, 0x52, 0xed, 0x6b, 0xe0, 0xec, 0x6c, 0xfa, 0x20,
0x4b, 0xed, 0x72, 0x1eb9, 0x73, 0x74, 0x1ecd, 0x301, 0x1ecd, 0x300, 0x6e, 0xf9,
0xec, 0x6c, 0xfa, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x1e63, 0x61, 0x6f, 0xec,
0x6c, 0xfa, 0x20, 0x42, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x68,
0x61, 0x6e, 0xec, 0x6c, 0xfa, 0x20, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0xec, 0x6c, 0xfa, 0x20, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x43,
0x72, 0x65, 0x65, 0x6b, 0xec, 0x6c, 0xfa, 0x20, 0x44, 0x65, 0x6e, 0x76,
0x65, 0x72, 0xec, 0x6c, 0xfa, 0x20, 0x44, 0x65, 0x74, 0x72, 0x6f, 0x69,
0x74, 0xec, 0x6c, 0xfa, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b,
0x61, 0xec, 0x6c, 0xfa, 0x20, 0x45, 0x64, 0x6d, 0x6f, 0x6e, 0x74, 0x6f,
0x6e, 0xec, 0x6c, 0xfa, 0x20, 0x45, 0x6c, 0x20, 0x53, 0x61, 0x76, 0x61,
0x64, 0x6f, 0x72, 0xec, 0x6c, 0xfa, 0x20, 0x46, 0x6f, 0x72, 0x74, 0x20,
0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0xec, 0x6c, 0xfa, 0x20, 0x6f, 0x6d,
0x69, 0x20, 0x47, 0x6c, 0x61, 0x63, 0x65, 0xec, 0x6c, 0xfa, 0x20, 0x6f,
0x6d, 0x69, 0x20, 0x47, 0x6f, 0x6f, 0x73, 0xf9, 0xec, 0x6c, 0xfa, 0x20,
0x47, 0x72, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x75, 0x72, 0x6b, 0xec, 0x6c,
0xfa, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x61, 0x64, 0x61, 0xec, 0x6c, 0xfa,
0x20, 0x47, 0x75, 0x61, 0x64, 0x65, 0x6c, 0x6f, 0x75, 0x70, 0x65, 0xec,
0x6c, 0xfa, 0x20, 0x47, 0x75, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61,
0xec, 0x6c, 0xfa, 0x20, 0x48, 0x61, 0x6c, 0x69, 0x66, 0xe1, 0x73, 0xec,
0xec, 0x6c, 0xfa, 0x20, 0x48, 0x61, 0x76, 0x61, 0x6e, 0x61, 0xec, 0x6c,
0xfa, 0x20, 0x48, 0x65, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6f, 0xec,
0x6c, 0xfa, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x70, 0x6f,
0x6c, 0x69, 0x73, 0xec, 0x6c, 0xfa, 0x20, 0x6e, 0x1ecd, 0x301, 0x73, 0xec,
0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x72, 0x1eb9, 0x301, 0x6e, 0x67, 0x6f,
0xec, 0x6c, 0xfa, 0x20, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75,
0x72, 0x67, 0xec, 0x6c, 0xfa, 0x20, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43,
0x69, 0x74, 0x79, 0xec, 0x6c, 0xfa, 0x20, 0x56, 0x65, 0x76, 0x61, 0x79,
0xec, 0x6c, 0xfa, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x6e, 0x65,
0x73, 0x20, 0x6e, 0xed, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0xec, 0x6c, 0xfa, 0x20, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x6b, 0x20,
0x6e, 0xed, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0xec, 0x6c,
0xfa, 0x20, 0x49, 0x6e, 0x75, 0x76, 0x69, 0x6b, 0xec, 0x6c, 0xfa, 0x20,
0x49, 0x71, 0x61, 0x6c, 0x75, 0x69, 0x74, 0xec, 0x6c, 0xfa, 0x20, 0x4a,
0x61, 0x6d, 0x61, 0x69, 0x6b, 0xe0, 0xec, 0x6c, 0xfa, 0x20, 0x4a, 0x75,
0x6e, 0x75, 0xec, 0x6c, 0xfa, 0x20, 0x4c, 0xfa, 0xed, 0x66, 0xed, 0xec,
0x6c, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x73,
0x1eb9, 0x301, 0x6c, 0xf2, 0xec, 0x6c, 0xfa, 0x20, 0x4b, 0xed, 0x72, 0x61,
0x6c, 0x1eb9, 0x301, 0x6e, 0x64, 0xe1, 0x69, 0x6b, 0xec, 0xec, 0x6c, 0xfa,
0x20, 0x4c, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x67, 0x65, 0x6c, 0x65, 0x73,
0xec, 0x6c, 0xfa, 0x20, 0x4c, 0x6f, 0x77, 0x65, 0x72, 0x20, 0x50, 0x72,
0x69, 0x6e, 0x63, 0x65, 0x2019, 0x73, 0x20, 0x51, 0x75, 0x61, 0x72, 0x74,
0x65, 0x72, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x6e, 0x61, 0x67, 0x75,
0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x67, 0x1ecd, 0x301,
0x1ecd, 0x300, 0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0xe1, 0x74, 0xed,
0x6e, 0xed, 0x6b, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x74, 0x61,
0x6d, 0x6f, 0x72, 0x6f, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61,
0x73, 0x61, 0x74, 0x69, 0x61, 0x6e, 0x69, 0xec, 0x6c, 0xfa, 0x20, 0x4d,
0x65, 0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d,
0x65, 0x72, 0x69, 0x64, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x65, 0x74,
0x69, 0x6c, 0x61, 0x6b, 0x61, 0x74, 0x69, 0x6c, 0x61, 0xec, 0x6c, 0xfa,
0x20, 0x4d, 0x1eb9, 0x301, 0x73, 0xed, 0x6b, 0xf2, 0xec, 0x6c, 0xfa, 0x20,
0x4d, 0x69, 0x6b, 0x75, 0x6c, 0x1ecd, 0x301, 0x6e, 0xec, 0xec, 0x6c, 0xfa,
0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x6f, 0x6e, 0x69, 0xec, 0x6c, 0xfa, 0x20,
0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x72, 0x69, 0xec, 0x6c, 0xfa, 0x20, 0x4d,
0x6f, 0x6e, 0x73, 0x65, 0x72, 0x61, 0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x4e, 0x61, 0x73, 0x61, 0xf2, 0xec, 0x6c, 0xfa, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x59, 0x6f, 0x72, 0x6b, 0xec, 0x6c, 0xfa, 0x20, 0x4e, 0x6f, 0x6d,
0x69, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x65, 0x75, 0x6c, 0xe0, 0x20, 0x6e,
0xed, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61,
0xec, 0x6c, 0xfa, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65,
0x6d, 0x20, 0x6e, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4e, 0x75, 0x75,
0x6b, 0xec, 0x6c, 0xfa, 0x20, 0x4f, 0x6a, 0x69, 0x6e, 0x61, 0x67, 0x61,
0xec, 0x6c, 0xfa, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x6d, 0x61, 0xec, 0x6c,
0xfa, 0x20, 0x46, 0xed, 0x6e, 0xed, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x50, 0x6f, 0x72, 0x74, 0x2d, 0x61, 0x75, 0x2d, 0x50, 0x72, 0x69, 0x6e,
0x63, 0x65, 0xec, 0x6c, 0xfa, 0x20, 0x65, 0x74, 0xed, 0x20, 0x6f, 0x6d,
0x69, 0x20, 0x53, 0xed, 0x70, 0xe9, 0xe8, 0x6e, 0xec, 0xec, 0x6c, 0xfa,
0x20, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b, 0x6f,
0xec, 0x6c, 0xfa, 0x20, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x49,
0x6e, 0x6c, 0x65, 0x74, 0xec, 0x6c, 0xfa, 0x20, 0x52, 0x65, 0x67, 0x69,
0x6e, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x52, 0x65, 0x73, 0x6f, 0x6c, 0xfa,
0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20,
0x44, 0x6f, 0x6d, 0x69, 0x67, 0x6f, 0xec, 0x6c, 0xfa, 0x20, 0x49, 0x74,
0x6f, 0x6b, 0x6f, 0x74, 0x6f, 0x6d, 0x69, 0x74, 0x69, 0xec, 0x6c, 0xfa,
0x20, 0x53, 0x69, 0x6b, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x61, 0x74,
0x69, 0x6c, 0x65, 0x6d, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x74, 0x20,
0x4a, 0x1ecd, 0x301, 0x1ecd, 0x300, 0x6e, 0xf9, 0xec, 0x6c, 0xfa, 0x20, 0x53,
0x74, 0x20, 0x6b, 0x69, 0x74, 0x69, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x53, 0x74, 0x20, 0x4c, 0x75, 0x73, 0x69, 0x61, 0xec, 0x6c, 0xfa, 0x20,
0x53, 0x74, 0x20, 0x54, 0x1ecd, 0x301, 0x6d, 0xe0, 0x73, 0xec, 0xec, 0x6c,
0xfa, 0x20, 0x46, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x74, 0xec, 0xec, 0x6c,
0xfa, 0x20, 0x53, 0xfa, 0x66, 0xed, 0x74, 0xf9, 0x20, 0x4b, 0x1ecd, 0x300,
0x72, 0x65, 0x6e, 0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x54, 0x65, 0x67,
0x75, 0x73, 0x69, 0x67, 0x61, 0x70, 0xe0, 0xec, 0x6c, 0xfa, 0x20, 0x54,
0x75, 0x6c, 0xe8, 0xec, 0x6c, 0xfa, 0x20, 0x54, 0xed, 0x6a, 0xfa, 0x61,
0x6e, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x54, 0x6f, 0x72, 0x6f, 0x6e, 0x74,
0x6f, 0xec, 0x6c, 0xfa, 0x20, 0x54, 0x6f, 0x74, 0x6f, 0x6c, 0x61, 0xec,
0x6c, 0xfa, 0x20, 0x56, 0x61, 0x6e, 0x6b, 0x75, 0x76, 0x61, 0xec, 0x6c,
0xfa, 0x20, 0x57, 0x68, 0x69, 0x74, 0x65, 0x68, 0x6f, 0x73, 0xec, 0xec,
0x6c, 0xfa, 0x20, 0x57, 0x69, 0x6e, 0x69, 0x70, 0x65, 0x67, 0xec, 0xec,
0x6c, 0xfa, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x61, 0xec, 0x6c, 0xfa,
0x20, 0x42, 0x1eb9, 0x300, 0x6d, 0xfa, 0x64, 0xe0, 0xec, 0x6c, 0xfa, 0x20,
0x41, 0x6e, 0x6b, 0x254, 0x301, 0x72, 0xe9, 0xe8, 0x6a, 0xec, 0xec, 0x6c,
0xfa, 0x20, 0xe0, 0x74, 0x69, 0x6b, 0x254, 0x6b, 0xe0, 0x6e, 0xec, 0x6c,
0xfa, 0x20, 0x42, 0xe1, 0x68, 0xec, 0x20, 0x42, 0xe1, 0x6e, 0x64, 0x25b,
0x301, 0x72, 0xe1, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0xe1, 0x62,
0xe1, 0x64, 0x254, 0x301, 0x254, 0x300, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x42, 0x25b, 0x300, 0x6c, 0xed, 0xec, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20,
0x42, 0x6c, 0x61, 0x6e, 0x6b, 0x20, 0x53, 0x61, 0x62, 0x75, 0x6c, 0x254,
0x301, 0x254, 0x300, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x254, 0x301,
0x69, 0x73, 0xe8, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x68, 0x69, 0x68, 0x75,
0x61, 0x68, 0x75, 0x61, 0xec, 0x6c, 0xfa, 0x20, 0x4b, 0xed, 0x72, 0x25b,
0x73, 0x74, 0x254, 0x301, 0x254, 0x300, 0x6e, 0xf9, 0xec, 0x6c, 0xfa, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0x68, 0x61, 0x6f, 0xec, 0x6c, 0xfa, 0x20,
0x6e, 0x254, 0x301, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x72,
0x25b, 0x301, 0x6e, 0x67, 0x6f, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x69, 0x73, 0x25b, 0x301, 0x6c, 0xf2, 0xec, 0x6c, 0xfa, 0x20, 0x4b,
0xed, 0x72, 0x61, 0x6c, 0x25b, 0x301, 0x6e, 0x64, 0xe1, 0x69, 0x6b, 0xec,
0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x67, 0x254, 0x301, 0x254,
0x300, 0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x25b, 0x301, 0x73, 0xed,
0x6b, 0xf2, 0xec, 0x6c, 0xfa, 0x20, 0x4d, 0x69, 0x6b, 0x75, 0x6c, 0x254,
0x301, 0x6e, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x74, 0x20, 0x4a, 0x254,
0x301, 0x254, 0x300, 0x6e, 0xf9, 0xec, 0x6c, 0xfa, 0x20, 0x53, 0x74, 0x20,
0x54, 0x254, 0x301, 0x6d, 0xe0, 0x73, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x53,
0xfa, 0x66, 0xed, 0x74, 0xf9, 0x20, 0x4b, 0x254, 0x300, 0x72, 0x65, 0x6e,
0x74, 0xec, 0xec, 0x6c, 0xfa, 0x20, 0x42, 0x25b, 0x300, 0x6d, 0xfa, 0x64,
0xe0, 0x69, 0x2d, 0x41, 0x62, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x69, 0x2d,
0x41, 0x63, 0x63, 0x72, 0x61, 0x69, 0x2d, 0x41, 0x64, 0x64, 0x69, 0x73,
0x20, 0x41, 0x62, 0x61, 0x62, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x67, 0x69,
0x65, 0x72, 0x73, 0x69, 0x2d, 0x41, 0x73, 0x6d, 0x61, 0x72, 0x61, 0x69,
0x2d, 0x42, 0x61, 0x6d, 0x61, 0x6b, 0x6f, 0x69, 0x2d, 0x42, 0x61, 0x6e,
0x67, 0x75, 0x69, 0x69, 0x2d, 0x42, 0x61, 0x6e, 0x6a, 0x75, 0x6c, 0x69,
0x2d, 0x42, 0x69, 0x73, 0x73, 0x61, 0x75, 0x69, 0x2d, 0x42, 0x6c, 0x61,
0x6e, 0x74, 0x79, 0x72, 0x65, 0x69, 0x2d, 0x42, 0x72, 0x61, 0x7a, 0x7a,
0x61, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x69, 0x2d, 0x42, 0x75, 0x6a, 0x75,
0x6d, 0x62, 0x75, 0x72, 0x61, 0x69, 0x2d, 0x43, 0x61, 0x69, 0x72, 0x6f,
0x69, 0x2d, 0x43, 0x61, 0x73, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x63, 0x61,
0x69, 0x2d, 0x43, 0x65, 0x75, 0x74, 0x61, 0x69, 0x2d, 0x43, 0x6f, 0x6e,
0x61, 0x6b, 0x72, 0x79, 0x69, 0x2d, 0x44, 0x61, 0x6b, 0x61, 0x72, 0x69,
0x2d, 0x44, 0x61, 0x72, 0x20, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x61,
0x61, 0x6d, 0x69, 0x2d, 0x44, 0x6a, 0x69, 0x62, 0x6f, 0x75, 0x74, 0x69,
0x69, 0x2d, 0x44, 0x6f, 0x75, 0x61, 0x6c, 0x61, 0x69, 0x2d, 0x45, 0x6c,
0x20, 0x41, 0x61, 0x69, 0x75, 0x6e, 0x69, 0x2d, 0x46, 0x72, 0x65, 0x65,
0x74, 0x6f, 0x77, 0x6e, 0x69, 0x2d, 0x47, 0x61, 0x62, 0x6f, 0x72, 0x6f,
0x6e, 0x65, 0x69, 0x2d, 0x48, 0x61, 0x72, 0x61, 0x72, 0x65, 0x69, 0x2d,
0x4a, 0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72, 0x67,
0x69, 0x4a, 0x75, 0x62, 0x61, 0x69, 0x2d, 0x4b, 0x61, 0x6d, 0x70, 0x61,
0x6c, 0x61, 0x69, 0x2d, 0x4b, 0x68, 0x61, 0x72, 0x74, 0x6f, 0x75, 0x6d,
0x69, 0x2d, 0x4b, 0x69, 0x67, 0x61, 0x6c, 0x69, 0x69, 0x2d, 0x4b, 0x69,
0x6e, 0x73, 0x68, 0x61, 0x73, 0x61, 0x69, 0x2d, 0x4c, 0x61, 0x67, 0x6f,
0x73, 0x69, 0x2d, 0x4c, 0x69, 0x62, 0x72, 0x65, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x69, 0x2d, 0x4c, 0x6f, 0x6d, 0x65, 0x69, 0x2d, 0x4c, 0x75, 0x61,
0x6e, 0x64, 0x61, 0x69, 0x2d, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62, 0x61,
0x73, 0x68, 0x69, 0x69, 0x4c, 0x75, 0x73, 0x61, 0x6b, 0x61, 0x69, 0x4d,
0x61, 0x6c, 0x61, 0x62, 0x6f, 0x69, 0x4d, 0x61, 0x70, 0x75, 0x74, 0x6f,
0x69, 0x4d, 0x61, 0x73, 0x65, 0x72, 0x75, 0x69, 0x4d, 0x62, 0x61, 0x62,
0x61, 0x6e, 0x65, 0x69, 0x2d, 0x4d, 0x6f, 0x67, 0x61, 0x64, 0x69, 0x73,
0x68, 0x75, 0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x72, 0x6f, 0x76, 0x69, 0x61,
0x69, 0x2d, 0x4e, 0x61, 0x69, 0x72, 0x6f, 0x62, 0x69, 0x69, 0x2d, 0x4e,
0x64, 0x6a, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x69, 0x2d, 0x4e, 0x69, 0x61,
0x6d, 0x65, 0x79, 0x69, 0x2d, 0x4e, 0x6f, 0x75, 0x61, 0x6b, 0x63, 0x68,
0x6f, 0x74, 0x74, 0x69, 0x2d, 0x4f, 0x75, 0x61, 0x67, 0x61, 0x64, 0x6f,
0x75, 0x67, 0x6f, 0x75, 0x69, 0x2d, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x2d,
0x4e, 0x6f, 0x76, 0x6f, 0x69, 0x2d, 0x53, 0xe3, 0x6f, 0x20, 0x54, 0x6f,
0x6d, 0xe9, 0x69, 0x2d, 0x54, 0x72, 0x69, 0x70, 0x6f, 0x6c, 0x69, 0x69,
0x2d, 0x54, 0x75, 0x6e, 0x69, 0x73, 0x69, 0x2d, 0x57, 0x69, 0x6e, 0x64,
0x68, 0x6f, 0x65, 0x6b, 0x69, 0x2d, 0x41, 0x64, 0x61, 0x6b, 0x69, 0x2d,
0x41, 0x6e, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x67, 0x65, 0x69, 0x2d, 0x41,
0x6e, 0x67, 0x75, 0x69, 0x6c, 0x6c, 0x61, 0x69, 0x2d, 0x41, 0x6e, 0x74,
0x69, 0x67, 0x75, 0x61, 0x69, 0x2d, 0x41, 0x72, 0x61, 0x67, 0x75, 0x61,
0x69, 0x6e, 0x61, 0x69, 0x2d, 0x42, 0x75, 0x65, 0x6e, 0x6f, 0x73, 0x20,
0x41, 0x69, 0x72, 0x65, 0x73, 0x69, 0x2d, 0x43, 0x61, 0x74, 0x61, 0x6d,
0x61, 0x72, 0x63, 0x61, 0x69, 0x2d, 0x43, 0x6f, 0x72, 0x64, 0x6f, 0x62,
0x61, 0x69, 0x2d, 0x4a, 0x75, 0x6a, 0x75, 0x79, 0x69, 0x2d, 0x4c, 0x61,
0x20, 0x52, 0x69, 0x6f, 0x6a, 0x61, 0x69, 0x2d, 0x4d, 0x65, 0x6e, 0x64,
0x6f, 0x7a, 0x61, 0x69, 0x2d, 0x52, 0x69, 0x6f, 0x20, 0x47, 0x61, 0x6c,
0x6c, 0x65, 0x67, 0x6f, 0x73, 0x69, 0x2d, 0x53, 0x61, 0x6c, 0x74, 0x61,
0x69, 0x2d, 0x53, 0x61, 0x6e, 0x20, 0x4a, 0x75, 0x61, 0x6e, 0x69, 0x2d,
0x53, 0x61, 0x6e, 0x20, 0x4c, 0x75, 0x69, 0x73, 0x69, 0x2d, 0x54, 0x75,
0x63, 0x75, 0x6d, 0x61, 0x6e, 0x69, 0x2d, 0x55, 0x73, 0x68, 0x75, 0x61,
0x69, 0x61, 0x69, 0x2d, 0x41, 0x72, 0x75, 0x62, 0x61, 0x69, 0x2d, 0x41,
0x73, 0x75, 0x6e, 0x63, 0x69, 0xf3, 0x6e, 0x69, 0x2d, 0x41, 0x74, 0x69,
0x6b, 0x6f, 0x6b, 0x61, 0x6e, 0x69, 0x2d, 0x42, 0x61, 0x68, 0x69, 0x61,
0x69, 0x2d, 0x42, 0x61, 0x68, 0x69, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x61, 0x73, 0x69, 0x2d, 0x42, 0x61, 0x72, 0x62, 0x61, 0x64,
0x6f, 0x73, 0x69, 0x2d, 0x42, 0x65, 0x6c, 0x65, 0x6d, 0x69, 0x2d, 0x42,
0x65, 0x6c, 0x69, 0x7a, 0x65, 0x69, 0x2d, 0x42, 0x6c, 0x61, 0x6e, 0x63,
0x2d, 0x53, 0x61, 0x62, 0x6c, 0x6f, 0x6e, 0x69, 0x2d, 0x42, 0x6f, 0x61,
0x20, 0x56, 0x69, 0x73, 0x74, 0x61, 0x69, 0x2d, 0x42, 0x6f, 0x67, 0x6f,
0x74, 0x61, 0x69, 0x2d, 0x42, 0x6f, 0x69, 0x73, 0x65, 0x69, 0x2d, 0x43,
0x61, 0x6d, 0x62, 0x72, 0x69, 0x64, 0x67, 0x65, 0x20, 0x42, 0x61, 0x79,
0x69, 0x2d, 0x43, 0x61, 0x6d, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x61, 0x6e,
0x64, 0x65, 0x69, 0x2d, 0x43, 0x61, 0x6e, 0x63, 0x75, 0x6e, 0x69, 0x2d,
0x43, 0x61, 0x72, 0x61, 0x63, 0x61, 0x73, 0x69, 0x2d, 0x43, 0x61, 0x79,
0x65, 0x6e, 0x6e, 0x65, 0x69, 0x2d, 0x43, 0x61, 0x79, 0x6d, 0x61, 0x6e,
0x69, 0x2d, 0x43, 0x68, 0x69, 0x63, 0x61, 0x67, 0x6f, 0x69, 0x2d, 0x43,
0x68, 0x69, 0x68, 0x75, 0x61, 0x68, 0x75, 0x61, 0x69, 0x2d, 0x43, 0x6f,
0x73, 0x74, 0x61, 0x20, 0x52, 0x69, 0x63, 0x61, 0x69, 0x2d, 0x43, 0x72,
0x65, 0x73, 0x74, 0x6f, 0x6e, 0x69, 0x2d, 0x43, 0x75, 0x69, 0x61, 0x62,
0x61, 0x69, 0x2d, 0x43, 0x75, 0x72, 0x61, 0xe7, 0x61, 0x6f, 0x69, 0x2d,
0x44, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x6b, 0x73, 0x68, 0x61, 0x76, 0x6e,
0x69, 0x2d, 0x44, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x69, 0x2d, 0x44, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x20, 0x43, 0x72, 0x65, 0x65, 0x6b, 0x69, 0x2d,
0x44, 0x65, 0x6e, 0x76, 0x65, 0x72, 0x69, 0x2d, 0x44, 0x65, 0x74, 0x72,
0x6f, 0x69, 0x74, 0x69, 0x2d, 0x44, 0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x63,
0x61, 0x69, 0x2d, 0x45, 0x64, 0x6d, 0x6f, 0x6e, 0x74, 0x6f, 0x6e, 0x69,
0x2d, 0x45, 0x69, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x65, 0x69, 0x2d, 0x45,
0x6c, 0x20, 0x53, 0x61, 0x6c, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x69, 0x2d,
0x46, 0x6f, 0x72, 0x74, 0x20, 0x4e, 0x65, 0x6c, 0x73, 0x6f, 0x6e, 0x69,
0x2d, 0x46, 0x6f, 0x72, 0x74, 0x61, 0x6c, 0x65, 0x7a, 0x61, 0x69, 0x2d,
0x47, 0x6c, 0x61, 0x63, 0x65, 0x20, 0x42, 0x61, 0x79, 0x69, 0x2d, 0x47,
0x6f, 0x6f, 0x73, 0x65, 0x20, 0x42, 0x61, 0x79, 0x69, 0x2d, 0x47, 0x72,
0x61, 0x6e, 0x64, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x69, 0x2d, 0x47, 0x72,
0x65, 0x6e, 0x61, 0x64, 0x61, 0x69, 0x2d, 0x47, 0x75, 0x61, 0x64, 0x65,
0x6c, 0x6f, 0x75, 0x70, 0x65, 0x69, 0x2d, 0x47, 0x75, 0x61, 0x74, 0x65,
0x6d, 0x61, 0x6c, 0x61, 0x69, 0x2d, 0x47, 0x75, 0x61, 0x79, 0x61, 0x71,
0x75, 0x69, 0x6c, 0x69, 0x2d, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x69,
0x2d, 0x48, 0x61, 0x6c, 0x69, 0x66, 0x61, 0x78, 0x69, 0x2d, 0x48, 0x61,
0x76, 0x61, 0x6e, 0x61, 0x69, 0x2d, 0x48, 0x65, 0x72, 0x6d, 0x6f, 0x73,
0x69, 0x6c, 0x6c, 0x6f, 0x69, 0x2d, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x69, 0x2d, 0x4b, 0x6e, 0x6f, 0x78,
0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x69, 0x2d, 0x4d,
0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x69, 0x2d, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73, 0x62,
0x75, 0x72, 0x67, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x69, 0x2d, 0x54, 0x65, 0x6c, 0x6c, 0x20, 0x43, 0x69, 0x74, 0x79, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x69, 0x2d, 0x56, 0x65,
0x76, 0x61, 0x79, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x69, 0x2d, 0x56, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x2c,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x69, 0x2d, 0x57, 0x69,
0x6e, 0x61, 0x6d, 0x61, 0x63, 0x2c, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x61, 0x69, 0x2d, 0x49, 0x6e, 0x75, 0x76, 0x69, 0x6b, 0x69, 0x2d,
0x49, 0x71, 0x61, 0x6c, 0x75, 0x69, 0x74, 0x69, 0x2d, 0x4a, 0x61, 0x6d,
0x61, 0x69, 0x63, 0x61, 0x69, 0x2d, 0x4a, 0x75, 0x6e, 0x65, 0x61, 0x75,
0x69, 0x2d, 0x4c, 0x6f, 0x75, 0x69, 0x73, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x63, 0x65, 0x6c, 0x6c, 0x6f,
0x2c, 0x20, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x63, 0x6b, 0x79, 0x69, 0x2d,
0x4b, 0x72, 0x61, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x6a, 0x6b, 0x69, 0x2d,
0x4c, 0x61, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x2d, 0x4c, 0x69, 0x6d, 0x61,
0x69, 0x2d, 0x4c, 0x6f, 0x73, 0x20, 0x41, 0x6e, 0x67, 0x65, 0x6c, 0x65,
0x73, 0x69, 0x2d, 0x4c, 0x6f, 0x77, 0x65, 0x72, 0x20, 0x50, 0x72, 0x69,
0x6e, 0x63, 0x65, 0x2019, 0x73, 0x20, 0x51, 0x75, 0x61, 0x72, 0x74, 0x65,
0x72, 0x69, 0x2d, 0x4d, 0x61, 0x63, 0x65, 0x69, 0x6f, 0x69, 0x2d, 0x4d,
0x61, 0x6e, 0x61, 0x67, 0x75, 0x61, 0x69, 0x2d, 0x4d, 0x61, 0x6e, 0x61,
0x75, 0x73, 0x69, 0x2d, 0x4d, 0x61, 0x72, 0x69, 0x67, 0x6f, 0x74, 0x69,
0x2d, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x71, 0x75, 0x65, 0x69,
0x2d, 0x4d, 0x61, 0x74, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x73, 0x69, 0x2d,
0x4d, 0x61, 0x7a, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x69, 0x2d, 0x4d, 0x65,
0x6e, 0x6f, 0x6d, 0x69, 0x6e, 0x65, 0x65, 0x69, 0x2d, 0x4d, 0x65, 0x72,
0x69, 0x64, 0x61, 0x69, 0x2d, 0x4d, 0x65, 0x74, 0x6c, 0x61, 0x6b, 0x61,
0x74, 0x6c, 0x61, 0x69, 0x2d, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x20,
0x43, 0x69, 0x74, 0x79, 0x69, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x63, 0x74, 0x6f, 0x6e, 0x69,
0x2d, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x72, 0x72, 0x65, 0x79, 0x69, 0x2d,
0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x76, 0x69, 0x64, 0x65, 0x6f, 0x69, 0x2d,
0x4d, 0x6f, 0x6e, 0x74, 0x73, 0x65, 0x72, 0x72, 0x61, 0x74, 0x69, 0x2d,
0x4e, 0x61, 0x73, 0x73, 0x61, 0x75, 0x69, 0x2d, 0x4e, 0x65, 0x77, 0x20,
0x59, 0x6f, 0x72, 0x6b, 0x69, 0x2d, 0x4e, 0x6f, 0x6d, 0x65, 0x69, 0x2d,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x69, 0x2d, 0x42, 0x65, 0x75,
0x6c, 0x61, 0x68, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x69, 0x2d, 0x43, 0x65, 0x6e, 0x74, 0x65,
0x72, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61, 0x6b,
0x6f, 0x74, 0x61, 0x69, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61, 0x6c,
0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x44, 0x61,
0x6b, 0x6f, 0x74, 0x61, 0x69, 0x2d, 0x4e, 0x75, 0x75, 0x6b, 0x69, 0x2d,
0x4f, 0x6a, 0x69, 0x6e, 0x61, 0x67, 0x61, 0x69, 0x2d, 0x50, 0x61, 0x6e,
0x61, 0x6d, 0x61, 0x69, 0x2d, 0x50, 0x61, 0x72, 0x61, 0x6d, 0x61, 0x72,
0x69, 0x62, 0x6f, 0x69, 0x2d, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78,
0x69, 0x2d, 0x50, 0x6f, 0x72, 0x74, 0x2d, 0x61, 0x75, 0x2d, 0x50, 0x72,
0x69, 0x6e, 0x63, 0x65, 0x69, 0x2d, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x6f,
0x66, 0x20, 0x53, 0x70, 0x61, 0x69, 0x6e, 0x69, 0x2d, 0x50, 0x6f, 0x72,
0x74, 0x6f, 0x20, 0x56, 0x65, 0x6c, 0x68, 0x6f, 0x69, 0x2d, 0x50, 0x75,
0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x63, 0x6f, 0x69, 0x2d, 0x50,
0x75, 0x6e, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x6e, 0x61, 0x73, 0x69,
0x2d, 0x52, 0x61, 0x6e, 0x6b, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x6c, 0x65,
0x74, 0x69, 0x2d, 0x52, 0x65, 0x63, 0x69, 0x66, 0x65, 0x69, 0x2d, 0x52,
0x65, 0x67, 0x69, 0x6e, 0x61, 0x69, 0x2d, 0x52, 0x65, 0x73, 0x6f, 0x6c,
0x75, 0x74, 0x65, 0x69, 0x2d, 0x52, 0x69, 0x6f, 0x20, 0x42, 0x72, 0x61,
0x6e, 0x63, 0x6f, 0x69, 0x2d, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x65,
0x6d, 0x69, 0x2d, 0x53, 0x61, 0x6e, 0x74, 0x69, 0x61, 0x67, 0x6f, 0x69,
0x2d, 0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x44, 0x6f, 0x6d, 0x69, 0x6e,
0x67, 0x6f, 0x69, 0x2d, 0x53, 0x61, 0x6f, 0x20, 0x50, 0x61, 0x75, 0x6c,
0x6f, 0x69, 0x2d, 0x49, 0x74, 0x74, 0x6f, 0x71, 0x71, 0x6f, 0x72, 0x74,
0x6f, 0x6f, 0x72, 0x6d, 0x69, 0x69, 0x74, 0x69, 0x2d, 0x53, 0x69, 0x74,
0x6b, 0x61, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20, 0x42, 0x61, 0x72, 0x74,
0x68, 0xe9, 0x6c, 0x65, 0x6d, 0x79, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20,
0x4a, 0x6f, 0x68, 0x6e, 0x2019, 0x73, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20,
0x4b, 0x69, 0x74, 0x74, 0x73, 0x49, 0x2d, 0x53, 0x74, 0x2e, 0x20, 0x4c,
0x75, 0x63, 0x69, 0x61, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20, 0x54, 0x68,
0x6f, 0x6d, 0x61, 0x73, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20, 0x56, 0x69,
0x6e, 0x63, 0x65, 0x6e, 0x74, 0x69, 0x2d, 0x53, 0x77, 0x69, 0x66, 0x74,
0x20, 0x43, 0x75, 0x72, 0x72, 0x65, 0x6e, 0x74, 0x69, 0x2d, 0x54, 0x65,
0x67, 0x75, 0x63, 0x69, 0x67, 0x61, 0x6c, 0x70, 0x61, 0x69, 0x2d, 0x54,
0x68, 0x75, 0x6c, 0x65, 0x69, 0x2d, 0x54, 0x69, 0x6a, 0x75, 0x61, 0x6e,
0x61, 0x69, 0x2d, 0x54, 0x6f, 0x72, 0x6f, 0x6e, 0x74, 0x6f, 0x69, 0x2d,
0x54, 0x6f, 0x72, 0x74, 0x6f, 0x6c, 0x61, 0x69, 0x2d, 0x56, 0x61, 0x6e,
0x63, 0x6f, 0x75, 0x76, 0x65, 0x72, 0x69, 0x2d, 0x57, 0x68, 0x69, 0x74,
0x65, 0x68, 0x6f, 0x72, 0x73, 0x65, 0x69, 0x2d, 0x57, 0x69, 0x6e, 0x6e,
0x69, 0x70, 0x65, 0x67, 0x69, 0x2d, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x61,
0x74, 0x69, 0x2d, 0x43, 0x61, 0x73, 0x65, 0x79, 0x69, 0x2d, 0x44, 0x61,
0x76, 0x69, 0x73, 0x69, 0x2d, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20,
0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x69, 0x2d, 0x4d,
0x61, 0x63, 0x71, 0x75, 0x61, 0x72, 0x69, 0x65, 0x69, 0x2d, 0x4d, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x69, 0x2d, 0x4d, 0x63, 0x4d, 0x75, 0x72, 0x64,
0x6f, 0x69, 0x2d, 0x50, 0x61, 0x6c, 0x6d, 0x65, 0x72, 0x69, 0x2d, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x69, 0x2d, 0x53, 0x79, 0x6f, 0x77,
0x61, 0x69, 0x2d, 0x54, 0x72, 0x6f, 0x6c, 0x6c, 0x69, 0x2d, 0x56, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x69, 0x2d, 0x4c, 0x6f, 0x6e, 0x67, 0x79, 0x65,
0x61, 0x72, 0x62, 0x79, 0x65, 0x6e, 0x69, 0x2d, 0x41, 0x64, 0x65, 0x6e,
0x69, 0x2d, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x69, 0x2d, 0x41, 0x6d,
0x6d, 0x61, 0x6e, 0x69, 0x2d, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x69,
0x2d, 0x41, 0x71, 0x74, 0x61, 0x75, 0x69, 0x2d, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0x69, 0x2d, 0x41, 0x73, 0x68, 0x67, 0x61, 0x62, 0x61, 0x74,
0x69, 0x2d, 0x42, 0x61, 0x67, 0x68, 0x64, 0x61, 0x64, 0x69, 0x2d, 0x42,
0x61, 0x68, 0x72, 0x61, 0x69, 0x6e, 0x69, 0x2d, 0x42, 0x61, 0x6b, 0x75,
0x69, 0x2d, 0x42, 0x61, 0x6e, 0x67, 0x6b, 0x6f, 0x6b, 0x69, 0x2d, 0x42,
0x61, 0x72, 0x6e, 0x61, 0x75, 0x6c, 0x69, 0x2d, 0x42, 0x65, 0x69, 0x72,
0x75, 0x74, 0x69, 0x2d, 0x42, 0x69, 0x73, 0x68, 0x6b, 0x65, 0x6b, 0x69,
0x2d, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x69, 0x2d, 0x43, 0x68, 0x69,
0x74, 0x61, 0x69, 0x2d, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x69, 0x2d, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x6f, 0x69,
0x2d, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x63, 0x75, 0x73, 0x69, 0x2d, 0x44,
0x68, 0x61, 0x6b, 0x61, 0x69, 0x2d, 0x44, 0x69, 0x6c, 0x69, 0x69, 0x2d,
0x44, 0x75, 0x62, 0x61, 0x69, 0x69, 0x2d, 0x44, 0x75, 0x73, 0x68, 0x61,
0x6e, 0x62, 0x65, 0x69, 0x47, 0x61, 0x7a, 0x61, 0x69, 0x2d, 0x48, 0x65,
0x62, 0x72, 0x6f, 0x6e, 0x69, 0x2d, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69,
0x20, 0x4d, 0x69, 0x6e, 0x68, 0x20, 0x43, 0x69, 0x74, 0x79, 0x69, 0x2d,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x69, 0x2d, 0x48,
0x6f, 0x76, 0x64, 0x69, 0x2d, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x2d, 0x4a, 0x61, 0x6b, 0x61, 0x72, 0x74, 0x61, 0x69, 0x2d, 0x4a,
0x61, 0x79, 0x61, 0x70, 0x75, 0x72, 0x61, 0x69, 0x2d, 0x4a, 0x65, 0x72,
0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x69, 0x2d, 0x4b, 0x61, 0x62, 0x75,
0x6c, 0x69, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61,
0x69, 0x2d, 0x4b, 0x61, 0x72, 0x61, 0x63, 0x68, 0x69, 0x69, 0x2d, 0x4b,
0x61, 0x74, 0x68, 0x6d, 0x61, 0x6e, 0x64, 0x75, 0x69, 0x2d, 0x4b, 0x68,
0x61, 0x6e, 0x64, 0x79, 0x67, 0x61, 0x69, 0x2d, 0x4b, 0x6f, 0x6c, 0x6b,
0x61, 0x74, 0x61, 0x69, 0x2d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x69, 0x2d, 0x4b, 0x75, 0x61, 0x6c, 0x61, 0x20,
0x4c, 0x75, 0x6d, 0x70, 0x75, 0x72, 0x69, 0x2d, 0x4b, 0x75, 0x63, 0x68,
0x69, 0x6e, 0x67, 0x69, 0x2d, 0x4b, 0x75, 0x77, 0x61, 0x69, 0x74, 0x69,
0x2d, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x69, 0x2d, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x69, 0x2d, 0x4d, 0x61, 0x6b, 0x61, 0x73, 0x73, 0x61,
0x72, 0x69, 0x2d, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0x61, 0x69, 0x2d, 0x4d,
0x75, 0x73, 0x63, 0x61, 0x74, 0x69, 0x2d, 0x4e, 0x69, 0x63, 0x6f, 0x73,
0x69, 0x61, 0x69, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x75, 0x7a, 0x6e,
0x65, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x2d, 0x4f, 0x6d, 0x73, 0x6b, 0x69,
0x2d, 0x4f, 0x72, 0x61, 0x6c, 0x69, 0x2d, 0x50, 0x68, 0x6e, 0x6f, 0x6d,
0x20, 0x50, 0x65, 0x6e, 0x68, 0x69, 0x2d, 0x50, 0x6f, 0x6e, 0x74, 0x69,
0x61, 0x6e, 0x61, 0x6b, 0x69, 0x2d, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79,
0x61, 0x6e, 0x67, 0x69, 0x2d, 0x51, 0x61, 0x74, 0x61, 0x72, 0x49, 0x2d,
0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x79, 0x69, 0x2d, 0x51, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x69, 0x2d, 0x52, 0x69, 0x79,
0x61, 0x64, 0x68, 0x69, 0x2d, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x69, 0x2d, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x64,
0x69, 0x2d, 0x53, 0x65, 0x6f, 0x75, 0x6c, 0x69, 0x2d, 0x53, 0x68, 0x61,
0x6e, 0x67, 0x68, 0x61, 0x69, 0x69, 0x2d, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x72, 0x65, 0x69, 0x2d, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x65,
0x6b, 0x6f, 0x6c, 0x79, 0x6d, 0x73, 0x6b, 0x69, 0x2d, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x69, 0x2d, 0x54, 0x61, 0x73, 0x68, 0x6b, 0x65, 0x6e,
0x74, 0x69, 0x2d, 0x54, 0x62, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x69, 0x2d,
0x54, 0x65, 0x68, 0x72, 0x61, 0x6e, 0x69, 0x2d, 0x54, 0x68, 0x69, 0x6d,
0x70, 0x68, 0x75, 0x69, 0x2d, 0x54, 0x6f, 0x6b, 0x79, 0x6f, 0x69, 0x2d,
0x54, 0x6f, 0x6d, 0x73, 0x6b, 0x69, 0x2d, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x69, 0x2d, 0x55, 0x72, 0x75, 0x6d,
0x71, 0x69, 0x69, 0x2d, 0x55, 0x73, 0x74, 0x2d, 0x4e, 0x65, 0x72, 0x61,
0x69, 0x2d, 0x56, 0x69, 0x65, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x69,
0x2d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x69, 0x2d, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x52,
0x61, 0x6e, 0x67, 0x6f, 0x6f, 0x6e, 0x69, 0x2d, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x2d, 0x59,
0x65, 0x72, 0x65, 0x76, 0x61, 0x6e, 0x69, 0x2d, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x69, 0x2d, 0x42, 0x65, 0x72, 0x6d, 0x75, 0x64, 0x61, 0x69,
0x2d, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x79, 0x69, 0x2d, 0x43, 0x61, 0x70,
0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x69, 0x2d, 0x46, 0x61, 0x72,
0x6f, 0x65, 0x69, 0x2d, 0x4d, 0x61, 0x64, 0x65, 0x69, 0x72, 0x61, 0x69,
0x2d, 0x52, 0x65, 0x79, 0x6b, 0x6a, 0x61, 0x76, 0x69, 0x6b, 0x69, 0x2d,
0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x69, 0x2d, 0x53, 0x74, 0x2e, 0x20, 0x48, 0x65, 0x6c, 0x65, 0x6e,
0x61, 0x69, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x6c, 0x65, 0x79, 0x69, 0x2d,
0x41, 0x64, 0x65, 0x6c, 0x61, 0x69, 0x64, 0x65, 0x69, 0x2d, 0x42, 0x72,
0x69, 0x73, 0x62, 0x61, 0x6e, 0x65, 0x69, 0x2d, 0x42, 0x72, 0x6f, 0x6b,
0x65, 0x6e, 0x20, 0x48, 0x69, 0x6c, 0x6c, 0x69, 0x2d, 0x44, 0x61, 0x72,
0x77, 0x69, 0x6e, 0x69, 0x2d, 0x45, 0x75, 0x63, 0x6c, 0x61, 0x69, 0x2d,
0x48, 0x6f, 0x62, 0x61, 0x72, 0x74, 0x69, 0x2d, 0x4c, 0x69, 0x6e, 0x64,
0x65, 0x6d, 0x61, 0x6e, 0x69, 0x2d, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x69, 0x2d, 0x4d, 0x65, 0x6c, 0x62, 0x6f, 0x75, 0x72,
0x6e, 0x65, 0x69, 0x2d, 0x50, 0x65, 0x72, 0x74, 0x68, 0x69, 0x2d, 0x53,
0x79, 0x64, 0x6e, 0x65, 0x79, 0x69, 0x2d, 0x41, 0x6d, 0x73, 0x74, 0x65,
0x72, 0x64, 0x61, 0x6d, 0x69, 0x2d, 0x41, 0x6e, 0x64, 0x6f, 0x72, 0x72,
0x61, 0x69, 0x2d, 0x41, 0x73, 0x74, 0x72, 0x61, 0x6b, 0x68, 0x61, 0x6e,
0x69, 0x2d, 0x41, 0x74, 0x68, 0x65, 0x6e, 0x73, 0x69, 0x2d, 0x42, 0x65,
0x6c, 0x67, 0x72, 0x61, 0x64, 0x65, 0x69, 0x2d, 0x42, 0x65, 0x72, 0x6c,
0x69, 0x6e, 0x69, 0x2d, 0x42, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c, 0x61,
0x76, 0x61, 0x69, 0x2d, 0x42, 0x72, 0x75, 0x73, 0x73, 0x65, 0x6c, 0x73,
0x69, 0x2d, 0x42, 0x75, 0x63, 0x68, 0x61, 0x72, 0x65, 0x73, 0x74, 0x69,
0x2d, 0x42, 0x75, 0x64, 0x61, 0x70, 0x65, 0x73, 0x74, 0x69, 0x2d, 0x42,
0x75, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x69, 0x2d, 0x43, 0x68, 0x69,
0x73, 0x69, 0x6e, 0x61, 0x75, 0x69, 0x2d, 0x43, 0x6f, 0x70, 0x65, 0x6e,
0x68, 0x61, 0x67, 0x65, 0x6e, 0x69, 0x2d, 0x44, 0x75, 0x62, 0x6c, 0x69,
0x6e, 0x69, 0x2d, 0x47, 0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x72,
0x69, 0x2d, 0x47, 0x75, 0x65, 0x72, 0x6e, 0x73, 0x65, 0x79, 0x69, 0x2d,
0x48, 0x65, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x69, 0x2d, 0x49, 0x73,
0x6c, 0x65, 0x20, 0x6f, 0x66, 0x20, 0x4d, 0x61, 0x6e, 0x69, 0x2d, 0x49,
0x73, 0x74, 0x61, 0x6e, 0x62, 0x75, 0x6c, 0x69, 0x2d, 0x4a, 0x65, 0x72,
0x73, 0x65, 0x79, 0x69, 0x2d, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e,
0x67, 0x72, 0x61, 0x64, 0x69, 0x2d, 0x4b, 0x69, 0x72, 0x6f, 0x76, 0x69,
0x2d, 0x4b, 0x69, 0x65, 0x76, 0x69, 0x2d, 0x4c, 0x69, 0x73, 0x62, 0x6f,
0x6e, 0x69, 0x2d, 0x4c, 0x6a, 0x75, 0x62, 0x6c, 0x6a, 0x61, 0x6e, 0x61,
0x69, 0x2d, 0x4c, 0x6f, 0x6e, 0x64, 0x6f, 0x6e, 0x69, 0x2d, 0x4c, 0x75,
0x78, 0x65, 0x6d, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x69, 0x2d, 0x4d, 0x61,
0x64, 0x72, 0x69, 0x64, 0x69, 0x2d, 0x4d, 0x61, 0x6c, 0x74, 0x61, 0x69,
0x2d, 0x4d, 0x61, 0x72, 0x69, 0x65, 0x68, 0x61, 0x6d, 0x6e, 0x69, 0x2d,
0x4d, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x2d, 0x4d, 0x6f, 0x6e, 0x61, 0x63,
0x6f, 0x69, 0x2d, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x69, 0x2d, 0x4f,
0x73, 0x6c, 0x6f, 0x69, 0x2d, 0x50, 0x61, 0x72, 0x69, 0x73, 0x69, 0x2d,
0x50, 0x6f, 0x64, 0x67, 0x6f, 0x72, 0x69, 0x63, 0x61, 0x69, 0x2d, 0x50,
0x72, 0x61, 0x67, 0x75, 0x65, 0x69, 0x2d, 0x52, 0x69, 0x67, 0x61, 0x69,
0x2d, 0x52, 0x6f, 0x6d, 0x65, 0x69, 0x2d, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x69, 0x2d, 0x53, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e,
0x6f, 0x69, 0x2d, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0x6f, 0x69,
0x2d, 0x53, 0x61, 0x72, 0x61, 0x74, 0x6f, 0x76, 0x69, 0x2d, 0x53, 0x69,
0x6d, 0x66, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c, 0x69, 0x2d, 0x53, 0x6b,
0x6f, 0x70, 0x6a, 0x65, 0x69, 0x2d, 0x53, 0x6f, 0x66, 0x69, 0x61, 0x69,
0x2d, 0x53, 0x74, 0x6f, 0x63, 0x6b, 0x68, 0x6f, 0x6c, 0x6d, 0x69, 0x2d,
0x54, 0x61, 0x6c, 0x6c, 0x69, 0x6e, 0x6e, 0x69, 0x2d, 0x54, 0x69, 0x72,
0x61, 0x6e, 0x65, 0x69, 0x2d, 0x55, 0x6c, 0x79, 0x61, 0x6e, 0x6f, 0x76,
0x73, 0x6b, 0x69, 0x2d, 0x56, 0x61, 0x64, 0x75, 0x7a, 0x69, 0x2d, 0x56,
0x61, 0x74, 0x69, 0x63, 0x61, 0x6e, 0x69, 0x2d, 0x56, 0x69, 0x65, 0x6e,
0x6e, 0x61, 0x69, 0x2d, 0x56, 0x69, 0x6c, 0x6e, 0x69, 0x75, 0x73, 0x69,
0x2d, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x2d,
0x57, 0x61, 0x72, 0x73, 0x61, 0x77, 0x69, 0x2d, 0x5a, 0x61, 0x67, 0x72,
0x65, 0x62, 0x69, 0x2d, 0x5a, 0x75, 0x72, 0x69, 0x63, 0x68, 0x69, 0x2d,
0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x76, 0x6f,
0x69, 0x2d, 0x43, 0x68, 0x61, 0x67, 0x6f, 0x73, 0x55, 0x6b, 0x68, 0x69,
0x73, 0x69, 0x6d, 0x75, 0x73, 0x69, 0x69, 0x2d, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x69, 0x2d, 0x43, 0x6f, 0x6d, 0x6f, 0x72, 0x6f, 0x69, 0x2d, 0x4b,
0x65, 0x72, 0x67, 0x75, 0x65, 0x6c, 0x65, 0x6e, 0x69, 0x4d, 0x61, 0x68,
0x65, 0x69, 0x2d, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x69,
0x2d, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x2d,
0x4d, 0x61, 0x79, 0x6f, 0x74, 0x74, 0x65, 0x69, 0x2d, 0x52, 0xe9, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x69, 0x2d, 0x41, 0x70, 0x69, 0x61, 0x69, 0x2d,
0x41, 0x75, 0x63, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x2d, 0x42, 0x6f,
0x75, 0x67, 0x61, 0x69, 0x6e, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x69, 0x2d,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x69, 0x2d, 0x43, 0x68, 0x75,
0x75, 0x6b, 0x69, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x69, 0x2d,
0x45, 0x66, 0x61, 0x74, 0x65, 0x69, 0x2d, 0x46, 0x61, 0x6b, 0x61, 0x6f,
0x66, 0x6f, 0x69, 0x2d, 0x46, 0x69, 0x6a, 0x69, 0x69, 0x2d, 0x46, 0x75,
0x6e, 0x61, 0x66, 0x75, 0x74, 0x69, 0x69, 0x2d, 0x47, 0x61, 0x6c, 0x61,
0x70, 0x61, 0x67, 0x6f, 0x73, 0x69, 0x2d, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x69, 0x2d, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x63, 0x61,
0x6e, 0x61, 0x6c, 0x69, 0x2d, 0x47, 0x75, 0x61, 0x6d, 0x69, 0x2d, 0x48,
0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x69, 0x2d, 0x45, 0x6e, 0x64,
0x65, 0x72, 0x62, 0x75, 0x72, 0x79, 0x69, 0x2d, 0x4b, 0x69, 0x72, 0x69,
0x74, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x69, 0x2d, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x69, 0x2d, 0x4b, 0x77, 0x61, 0x6a, 0x61, 0x6c, 0x65, 0x69,
0x6e, 0x69, 0x2d, 0x4d, 0x61, 0x6a, 0x75, 0x72, 0x6f, 0x69, 0x2d, 0x4d,
0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x69, 0x2d, 0x4d, 0x69,
0x64, 0x77, 0x61, 0x79, 0x69, 0x2d, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x69,
0x2d, 0x4e, 0x69, 0x75, 0x65, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x69, 0x2d, 0x4e, 0x6f, 0x75, 0x6d, 0x65, 0x61, 0x69, 0x2d,
0x50, 0x61, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x67, 0x6f, 0x69, 0x2d, 0x50,
0x61, 0x6c, 0x61, 0x75, 0x69, 0x2d, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69,
0x72, 0x6e, 0x69, 0x2d, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x69,
0x2d, 0x50, 0x6f, 0x72, 0x74, 0x20, 0x4d, 0x6f, 0x72, 0x65, 0x73, 0x62,
0x79, 0x69, 0x2d, 0x52, 0x61, 0x72, 0x6f, 0x74, 0x6f, 0x6e, 0x67, 0x61,
0x69, 0x2d, 0x53, 0x61, 0x69, 0x70, 0x61, 0x6e, 0x69, 0x2d, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x69, 0x2d, 0x54, 0x61, 0x72, 0x61, 0x77, 0x61,
0x69, 0x2d, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0x61, 0x70, 0x75, 0x69,
0x2d, 0x57, 0x61, 0x6b, 0x65, 0x69, 0x2d, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x41, 0x6d, 0x69, 0x6e, 0x6a, 0x61, 0x6e, 0x41, 0x6e, 0x69, 0x73,
0x69, 0x20, 0x41, 0x6d, 0x65, 0x6d, 0x61, 0x41, 0x72, 0x6a, 0xe9, 0x72,
0x41, 0x73, 0x69, 0x6d, 0x1ef9, 0x72, 0x61, 0x4d, 0x61, 0x6d, 0x1ef9, 0x6b,
0x6f, 0x4d, 0x61, 0x67, 0x69, 0x4d, 0x61, 0x6e, 0x6a, 0x75, 0x72, 0x4d,
0x69, 0x73, 0x61, 0x76, 0x4d, 0x72, 0x61, 0x6e, 0x74, 0x79, 0x72, 0x65,
0x4d, 0x72, 0x61, 0x6a, 0x61, 0x76, 0x69, 0x72, 0x65, 0x4d, 0x75, 0x6a,
0x169, 0x6d, 0x75, 0x72, 0x61, 0x128, 0x6e, 0x6b, 0x75, 0x70, 0x72, 0x69,
0x53, 0x65, 0x76, 0x75, 0x74, 0x61, 0x4b, 0x6f, 0x6e, 0x1ef9, 0x6b, 0x72,
0x69, 0x4e, 0x61, 0x6b, 0x61, 0x72, 0x4e, 0x61, 0x72, 0x20, 0x45, 0x6e,
0x68, 0x20, 0x53, 0x61, 0x72, 0x1ef9, 0x6d, 0x4e, 0x68, 0x69, 0x6d, 0x75,
0x74, 0x69, 0x4e, 0x6f, 0x76, 0x61, 0x72, 0x61, 0x45, 0x72, 0x20, 0x41,
0x2019, 0x61, 0x6a, 0x75, 0x6e, 0x1ebc, 0x6d, 0xe3, 0x2d, 0x66, 0x72, 0x69,
0x47, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x6e, 0x129, 0x4a, 0x75, 0x76, 0x1ef9,
0x6e, 0x1ebd, 0x6e, 0x68, 0x6d, 0x75, 0x72, 0x67, 0x75, 0x4a, 0x75, 0x6d,
0x61, 0x4b, 0xe3, 0x6d, 0x70, 0x61, 0x72, 0x61, 0x4b, 0x61, 0x72, 0x74,
0x169, 0x6d, 0x4b, 0x69, 0x67, 0x61, 0x72, 0x69, 0x4b, 0x129, 0x73, 0x61,
0x6a, 0x61, 0x52, 0x61, 0x67, 0x6f, 0x52, 0x69, 0x6d, 0x72, 0x65, 0x76,
0x69, 0x72, 0x69, 0x52, 0xf3, 0x6d, 0x1ebd, 0x52, 0x75, 0x76, 0x1ef9, 0x6e,
0x61, 0x52, 0x75, 0x6d, 0x75, 0x6d, 0x61, 0x73, 0x69, 0x52, 0x75, 0x73,
0x61, 0x6b, 0x61, 0x4d, 0x1ef9, 0x72, 0x61, 0x6d, 0x6f, 0x4d, 0x1ef9, 0x70,
0x75, 0x74, 0x75, 0x4d, 0x1ef9, 0x73, 0x65, 0x72, 0x75, 0x4d, 0x129, 0x6d,
0x61, 0x6d, 0x61, 0x6e, 0x1ebd, 0x4d, 0x169, 0x67, 0x61, 0x6e, 0x69, 0x6e,
0x68, 0x73, 0x75, 0x4d, 0xe3, 0x6e, 0x72, 0xf3, 0x76, 0x69, 0x6a, 0x61,
0x4e, 0x1ef9, 0x6a, 0x72, 0xf3, 0x6d, 0x69, 0x4e, 0x69, 0x2019, 0x6e, 0x69,
0x6a, 0x61, 0x6d, 0x1ebd, 0x6e, 0x1ef9, 0x4e, 0x129, 0x61, 0x6d, 0x1ebd, 0x6a,
0x4e, 0xe3, 0x76, 0x61, 0x67, 0x73, 0xf3, 0x74, 0x69, 0x4f, 0x76, 0x61,
0x67, 0x61, 0x6e, 0x6f, 0x67, 0x6f, 0x76, 0x50, 0x6f, 0x72, 0x74, 0x6f,
0x20, 0x54, 0xe3, 0x67, 0x53, 0x1ef9, 0x20, 0x54, 0x75, 0x6d, 0x1ebd, 0x54,
0x72, 0x69, 0x70, 0x6f, 0x72, 0x69, 0x54, 0x75, 0x6e, 0x129, 0x6e, 0x68,
0x56, 0x129, 0x6e, 0x6e, 0x6f, 0xe9, 0x6b, 0x69, 0x41, 0x6e, 0x61, 0x6b,
0x69, 0x1ef8, 0x6b, 0x6f, 0x72, 0x61, 0x67, 0x69, 0x1ef8, 0x67, 0x69, 0x72,
0x61, 0x1ef8, 0x74, 0x69, 0x67, 0x75, 0x76, 0x61, 0x41, 0x72, 0x61, 0x67,
0x75, 0x61, 0x69, 0x6e, 0x1ef9, 0x4d, 0x75, 0x65, 0x6e, 0x169, 0x73, 0x61,
0x69, 0x72, 0x69, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x1ef9, 0x72, 0x6b, 0x61,
0x4b, 0xf3, 0x72, 0x6e, 0x6f, 0x6d, 0x61, 0x4a, 0x75, 0x6a, 0x75, 0x6a,
0x52, 0x61, 0x20, 0x52, 0x69, 0x6f, 0x68, 0x61, 0x4d, 0x1ebd, 0x6e, 0x6f,
0x73, 0x61, 0x47, 0x61, 0x72, 0x65, 0x67, 0x6f, 0x20, 0x47, 0x6f, 0x6a,
0x53, 0x61, 0x72, 0x74, 0x61, 0x53, 0x1ef9, 0x20, 0x4a, 0x75, 0x76, 0x1ef9,
0x53, 0x1ef9, 0x20, 0x52, 0x75, 0x6a, 0x54, 0x75, 0x6b, 0x75, 0x6d, 0x1ef9,
0x55, 0x73, 0x75, 0x61, 0x6a, 0x61, 0x41, 0x72, 0x75, 0x6d, 0x61, 0x41,
0x73, 0x169, 0x67, 0x73, 0x1ef9, 0x41, 0x74, 0x69, 0x6b, 0x6f, 0x6b, 0x1ef9,
0x6e, 0x4d, 0x61, 0x6a, 0x69, 0x61, 0x4d, 0x61, 0x6a, 0x69, 0x61, 0x20,
0x4d, 0xe1, 0x6e, 0x6e, 0x65, 0x72, 0x61, 0x20, 0x41, 0x67, 0x4d, 0x61,
0x72, 0x6d, 0x61, 0x6e, 0x75, 0x4d, 0x65, 0x72, 0x1ebd, 0x6a, 0x4d, 0x65,
0x72, 0x69, 0x73, 0x65, 0x53, 0x61, 0x6d, 0x72, 0xe3, 0x6e, 0x20, 0x4b,
0x75, 0x70, 0x72, 0x69, 0x56, 0x65, 0x6a, 0x61, 0x20, 0x48, 0xe1, 0x4d,
0x6f, 0x67, 0x6f, 0x74, 0x61, 0x4d, 0x6f, 0x6a, 0x73, 0x65, 0x4b, 0x1ebd,
0x6d, 0x72, 0x69, 0x6e, 0x6a, 0x65, 0x20, 0x4d, 0x65, 0x6a, 0x52, 0x65,
0x20, 0x4d, 0xe1, 0x67, 0x4b, 0x1ef9, 0x6b, 0x169, 0x6e, 0x4b, 0x61, 0x72,
0x61, 0x6b, 0x61, 0x6e, 0x68, 0x4b, 0x61, 0x6a, 0x65, 0x6e, 0x1ef9, 0x4b,
0x61, 0x6a, 0x6d, 0x1ef9, 0x6e, 0x53, 0x69, 0x6b, 0x61, 0x67, 0x75, 0x53,
0x69, 0x68, 0x75, 0x76, 0x61, 0x68, 0x75, 0x76, 0x61, 0x50, 0xe3, 0x6e,
0x129, 0x20, 0x54, 0x169, 0x20, 0x4d, 0xe1, 0x67, 0x4b, 0x72, 0xe9, 0x73,
0x69, 0x74, 0xe3, 0x6e, 0x4b, 0x75, 0x6a, 0x61, 0x6d, 0x61, 0x4b, 0x75,
0x72, 0x61, 0x73, 0x61, 0x76, 0x6f, 0x4e, 0xe1, 0x6e, 0x6d, 0x1ef9, 0x72,
0x6b, 0x69, 0x73, 0x61, 0x76, 0x79, 0x6e, 0x4e, 0x61, 0x76, 0x73, 0xe3,
0x6e, 0x4e, 0x61, 0x75, 0x73, 0xe3, 0x6e, 0x20, 0x4b, 0x72, 0x69, 0x6b,
0x69, 0x4e, 0x65, 0x6e, 0x76, 0x65, 0x72, 0x4e, 0x65, 0x74, 0x6f, 0x72,
0xf3, 0x6a, 0x74, 0x69, 0x4e, 0x6f, 0x6d, 0x129, 0x6e, 0x129, 0x6b, 0x61,
0x45, 0x6e, 0x69, 0x6d, 0xe3, 0x6e, 0x74, 0xe3, 0x45, 0x6a, 0x72, 0x75,
0x6e, 0x1ebd, 0x70, 0xe9, 0xc9, 0x72, 0x20, 0x53, 0x61, 0x72, 0x76, 0x61,
0x6e, 0x6f, 0x72, 0x46, 0xf3, 0x72, 0x74, 0x69, 0x20, 0x4e, 0x1ebd, 0x72,
0x73, 0x75, 0x46, 0x6f, 0x72, 0x74, 0x61, 0x72, 0x65, 0x6a, 0x61, 0x47,
0x72, 0x61, 0x73, 0x65, 0x20, 0x4d, 0x65, 0x6a, 0x47, 0x1ef9, 0x73, 0x6f,
0x20, 0x4d, 0x65, 0x6a, 0x54, 0x75, 0x72, 0x6b, 0x69, 0x20, 0x4d, 0xe1,
0x67, 0x47, 0x72, 0x61, 0x6e, 0x1ef9, 0x6e, 0x61, 0x47, 0x75, 0x76, 0x61,
0x6e, 0x61, 0x72, 0x75, 0x70, 0x65, 0x47, 0x75, 0x72, 0x61, 0x74, 0x65,
0x6d, 0x1ef9, 0x72, 0x61, 0x47, 0x75, 0x76, 0x61, 0x6a, 0x61, 0x6b, 0x69,
0x72, 0x47, 0x75, 0x6a, 0xe1, 0x6e, 0x1ef9, 0x48, 0x61, 0x72, 0x69, 0x66,
0x61, 0x67, 0x48, 0x61, 0x76, 0xe1, 0x6e, 0x1ef9, 0x48, 0xe9, 0x72, 0x6d,
0x6f, 0x73, 0x69, 0x72, 0x6f, 0x128, 0x6e, 0x69, 0x6a, 0x61, 0x6e, 0x1ef9,
0x70, 0x6f, 0x72, 0x69, 0x6e, 0x68, 0x4b, 0x69, 0x6e, 0xe3, 0x67, 0x2c,
0x20, 0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x4d, 0x1ef9, 0x72, 0x1ebd,
0x67, 0x67, 0x75, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9,
0x50, 0xe9, 0x74, 0x65, 0x72, 0x6d, 0x75, 0x72, 0x67, 0x69, 0x2c, 0x20,
0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x54, 0x65, 0x72, 0x20, 0x73,
0x69, 0x74, 0x69, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9,
0x56, 0x65, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a, 0x1ef9,
0x6e, 0x1ef9, 0x56, 0x129, 0x73, 0x65, 0x6e, 0x1ebd, 0x6e, 0x68, 0x2c, 0x20,
0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x56, 0x69, 0x6e, 0x1ef9, 0x6d,
0x1ef9, 0x6b, 0x69, 0x2c, 0x20, 0x128, 0x6e, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9,
0x49, 0x6e, 0x169, 0x76, 0x69, 0x6b, 0x69, 0x49, 0x6b, 0x61, 0x72, 0x75,
0x6a, 0x69, 0x6e, 0x4a, 0x61, 0x6d, 0x1ef9, 0x6a, 0x6b, 0x1ef9, 0x4a, 0x75,
0x6e, 0x1ef9, 0x76, 0x6f, 0x52, 0x75, 0x69, 0x6e, 0x68, 0x76, 0x69, 0x72,
0x69, 0x4d, 0xe3, 0x6e, 0x74, 0x69, 0x73, 0xe9, 0x72, 0x75, 0x2c, 0x20,
0x4b, 0x1ebd, 0x74, 0xe1, 0x6b, 0x69, 0x4b, 0x72, 0x61, 0x72, 0x1ebd, 0x6e,
0x69, 0x6b, 0x65, 0x52, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x52, 0x69,
0x6d, 0x1ef9, 0x52, 0x6f, 0x6e, 0x68, 0x20, 0x1ef8, 0x6e, 0x6a, 0x69, 0x72,
0x69, 0x52, 0x6f, 0x76, 0x65, 0x72, 0x20, 0x50, 0x72, 0x129, 0x6e, 0x73,
0x69, 0x20, 0x4b, 0x75, 0x76, 0x61, 0x72, 0x74, 0xe9, 0x72, 0x4d, 0x1ef9,
0x73, 0x65, 0x6a, 0xf3, 0x4d, 0x1ef9, 0x6e, 0x1ef9, 0x67, 0x75, 0x76, 0x61,
0x4d, 0x1ef9, 0x6e, 0x1ef9, 0x76, 0x6f, 0x4d, 0x1ef9, 0x72, 0x69, 0x67, 0xf3,
0x74, 0x69, 0x4d, 0x1ef9, 0x72, 0x74, 0x69, 0x6e, 0x129, 0x6b, 0x61, 0x4d,
0xe3, 0x72, 0x6f, 0x2d, 0x61, 0x67, 0x20, 0x54, 0xe3, 0x6e, 0x68, 0x4d,
0x1ef9, 0x6a, 0x61, 0x74, 0x72, 0x1ef9, 0x6e, 0x4d, 0x1ebd, 0x6e, 0xe3, 0x6d,
0x129, 0x6e, 0x129, 0x4d, 0x1ebd, 0x72, 0x69, 0x6e, 0x61, 0x4d, 0x1ebd, 0x74,
0x61, 0x72, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x72, 0x61, 0x1ebc, 0x6d, 0xe3,
0x20, 0x74, 0x1ef9, 0x20, 0x4d, 0x1ebd, 0x73, 0x69, 0x6b, 0x6f, 0x4d, 0x129,
0x6b, 0x65, 0x72, 0x6f, 0x6e, 0x4d, 0xe3, 0x67, 0x74, 0xe3, 0x6e, 0x4d,
0xe3, 0x74, 0x65, 0x68, 0x65, 0x6a, 0x4d, 0xe3, 0x74, 0x69, 0x76, 0x69,
0x6e, 0xe9, 0x76, 0x4d, 0xe3, 0x6e, 0x73, 0x65, 0x72, 0xe1, 0x74, 0x69,
0x4e, 0x1ef9, 0x73, 0x61, 0x76, 0x59, 0x6a, 0xf3, 0x72, 0x6b, 0x69, 0x20,
0x54, 0xe3, 0x67, 0x4a, 0x79, 0x6a, 0x79, 0x46, 0x65, 0x72, 0x6e, 0x1ef9,
0x6e, 0x75, 0x20, 0x4e, 0xe3, 0x72, 0xe3, 0x6a, 0x61, 0x2d, 0x74, 0xe1,
0x4d, 0x65, 0x75, 0x72, 0x61, 0x2c, 0x20, 0x4e, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x20, 0x4e, 0xe3, 0x72, 0x74, 0x69, 0x53, 0x1ebd, 0x6e, 0x74, 0x65,
0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4e, 0xe3,
0x72, 0x74, 0x69, 0x4e, 0x129, 0x75, 0x20, 0x53, 0x61, 0x72, 0x65, 0x6e,
0x2c, 0x20, 0x4e, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4e, 0xe3, 0x72,
0x74, 0x69, 0x4e, 0x169, 0x67, 0x4f, 0x6a, 0x69, 0x6e, 0x1ef9, 0x67, 0x61,
0x50, 0x61, 0x6e, 0x1ef9, 0x6d, 0x1ef9, 0x50, 0x61, 0x72, 0x61, 0x6d, 0x1ef9,
0x72, 0x69, 0x6d, 0x75, 0x46, 0x75, 0x65, 0x6e, 0x129, 0x6e, 0x68, 0x50,
0x6f, 0x72, 0x74, 0x75, 0x20, 0x50, 0x72, 0x129, 0x73, 0x69, 0x70, 0x69,
0x50, 0xf3, 0x72, 0x74, 0x6f, 0x66, 0x69, 0x20, 0x49, 0x6e, 0x68, 0x70,
0x61, 0x6a, 0x69, 0x6e, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x4b, 0xf3,
0x66, 0x61, 0x50, 0x169, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x6e, 0x1ef9,
0x52, 0x1ef9, 0x67, 0x6b, 0x129, 0x67, 0x20, 0x128, 0x72, 0xe9, 0x6e, 0x52,
0x69, 0x73, 0x69, 0x66, 0x69, 0x52, 0x69, 0x6a, 0x69, 0x6e, 0x1ef9, 0x52,
0x69, 0x6a, 0x6f, 0x72, 0x75, 0x74, 0x65, 0x47, 0x6f, 0x6a, 0x20, 0x4b,
0x75, 0x70, 0x72, 0x69, 0x53, 0x1ef9, 0x74, 0x61, 0x72, 0x1ebd, 0x6a, 0x53,
0x1ef9, 0x74, 0x69, 0x6a, 0x61, 0x67, 0x75, 0x53, 0x1ef9, 0x74, 0x75, 0x20,
0x4e, 0x75, 0x6d, 0x129, 0x67, 0x67, 0x75, 0x53, 0x1ef9, 0x20, 0x50, 0x61,
0x76, 0x6f, 0x72, 0x75, 0x49, 0x74, 0x6f, 0x67, 0x6b, 0x6f, 0x72, 0x74,
0x75, 0x72, 0x6d, 0x129, 0x6e, 0x74, 0x69, 0x53, 0x69, 0x6e, 0x6b, 0x61,
0x53, 0x1ef9, 0x20, 0x4d, 0x61, 0x72, 0x74, 0x6f, 0x72, 0x6f, 0x6d, 0x1ebd,
0x53, 0x1ef9, 0x6e, 0x20, 0x4a, 0x6f, 0x6e, 0x68, 0x53, 0x1ef9, 0x20, 0x4b,
0x72, 0x69, 0x74, 0xf3, 0x76, 0x1ef9, 0x53, 0x1ef9, 0x74, 0x61, 0x20, 0x52,
0x75, 0x73, 0x61, 0x53, 0x1ef9, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x6a, 0x53,
0x1ef9, 0x20, 0x56, 0x69, 0x73, 0x1ebd, 0x74, 0x69, 0x53, 0x75, 0x76, 0x69,
0x66, 0x69, 0x74, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x1ebd, 0x74, 0x69, 0x54,
0x65, 0x67, 0x75, 0x73, 0x69, 0x67, 0x61, 0x72, 0x70, 0x61, 0x54, 0x75,
0x72, 0x65, 0x54, 0x69, 0x76, 0x1ef9, 0x6e, 0x1ef9, 0x54, 0x6f, 0x72, 0xe3,
0x74, 0x6f, 0x54, 0x6f, 0x72, 0x74, 0x6f, 0x72, 0x61, 0x56, 0x1ef9, 0x67,
0x6b, 0x75, 0x76, 0x65, 0x72, 0x4b, 0xe3, 0x76, 0xe3, 0x72, 0x75, 0x20,
0x4b, 0x75, 0x70, 0x72, 0x69, 0x56, 0x129, 0x6e, 0x129, 0x70, 0xe9, 0x67,
0x59, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x61, 0x74, 0x69, 0x4b, 0x61, 0x73,
0x65, 0x6a, 0x4e, 0x61, 0x76, 0x69, 0x73, 0x69, 0x4d, 0x1ef9, 0x6b, 0x69,
0x6b, 0x75, 0x61, 0x72, 0x69, 0x4d, 0x1ef9, 0x75, 0x73, 0xe3, 0x6e, 0x4d,
0xe9, 0x67, 0x2d, 0x4d, 0x169, 0x72, 0x6e, 0x6f, 0x50, 0x61, 0x72, 0x6d,
0x1ebd, 0x72, 0x54, 0x6f, 0x72, 0xf3, 0x72, 0x56, 0x6f, 0x6e, 0x68, 0x74,
0xf3, 0x6b, 0x69, 0x52, 0x169, 0x67, 0x69, 0x6a, 0xe9, 0x20, 0x41, 0x72,
0x6d, 0x79, 0x6a, 0x1ebd, 0x6e, 0x41, 0x6e, 0x65, 0x6e, 0x41, 0x72, 0x6d,
0x1ef9, 0x74, 0x69, 0x41, 0x6d, 0x1ef9, 0x41, 0x6e, 0x1ef9, 0x6e, 0x68, 0x69,
0x72, 0x41, 0x6b, 0x69, 0x74, 0x61, 0x75, 0x41, 0x6b, 0x69, 0x74, 0x6f,
0x6d, 0x65, 0x41, 0x6e, 0x68, 0x67, 0x61, 0x6d, 0x61, 0x74, 0x69, 0x41,
0x74, 0x69, 0x72, 0x61, 0x76, 0x4d, 0x61, 0x67, 0x69, 0x6e, 0x61, 0x4d,
0x61, 0x72, 0x1ebd, 0x6a, 0x4d, 0x79, 0x67, 0x6b, 0xf3, 0x6b, 0x69, 0x4d,
0x61, 0x72, 0x6e, 0x1ef9, 0x75, 0x72, 0x4d, 0x65, 0x6a, 0x72, 0x75, 0x74,
0x69, 0x4d, 0x69, 0x73, 0x69, 0x6b, 0xe9, 0x6b, 0x69, 0x4d, 0x72, 0x75,
0x6e, 0x1ebd, 0x6a, 0x53, 0x69, 0x74, 0x61, 0x53, 0x6f, 0x6a, 0x6d, 0x61,
0x72, 0x73, 0x1ef9, 0x4b, 0x6f, 0x72, 0xe3, 0x6d, 0x6d, 0x75, 0x4e, 0x61,
0x6d, 0x1ef9, 0x6e, 0x68, 0x6b, 0x6f, 0x4e, 0x69, 0x72, 0x69, 0x4e, 0x75,
0x6d, 0x61, 0x6a, 0x4e, 0x75, 0x73, 0xe1, 0x67, 0x6d, 0x65, 0x46, 0x61,
0x6d, 0x1ef9, 0x67, 0x75, 0x6a, 0x74, 0x61, 0x47, 0x61, 0x6a, 0x61, 0x48,
0x65, 0x6d, 0x72, 0x6f, 0x6e, 0x48, 0xe3, 0x67, 0x20, 0x4b, 0xe3, 0x67,
0x48, 0x6f, 0x76, 0x69, 0x6e, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x6e,
0x68, 0x6b, 0x69, 0x4a, 0x61, 0x69, 0x61, 0x70, 0x75, 0x72, 0x61, 0x4a,
0x65, 0x72, 0x75, 0x6a, 0x61, 0x72, 0x1ebd, 0x6a, 0x4b, 0x61, 0x6d, 0x75,
0x72, 0x4b, 0x1ef9, 0x6d, 0x73, 0x61, 0x6e, 0x6b, 0x61, 0x4b, 0x61, 0x72,
0x61, 0x73, 0x69, 0x4b, 0x61, 0x74, 0x69, 0x6d, 0x1ef9, 0x6e, 0x6e, 0x75,
0x4b, 0x1ef9, 0x6e, 0x79, 0x67, 0x61, 0x4b, 0x61, 0x72, 0x6b, 0x75, 0x74,
0x61, 0x4b, 0x72, 0x61, 0x6e, 0x68, 0x6e, 0xe3, 0x6a, 0x61, 0x72, 0x6b,
0x69, 0x4b, 0x75, 0x76, 0x61, 0x72, 0x61, 0x20, 0x52, 0x169, 0x70, 0x75,
0x72, 0x4b, 0x75, 0x73, 0x129, 0x67, 0x4b, 0x75, 0x76, 0x61, 0x6a, 0x74,
0x69, 0x4d, 0x1ef9, 0x6b, 0x61, 0x75, 0x4d, 0x1ef9, 0x67, 0x61, 0x6e, 0x61,
0x6e, 0x4d, 0x1ef9, 0x6e, 0x129, 0x72, 0x61, 0x4e, 0x129, 0x6b, 0xf3, 0x6a,
0x69, 0x61, 0x4b, 0x75, 0x6e, 0x68, 0x6e, 0x1ebd, 0x74, 0x69, 0x6e, 0x68,
0x6b, 0x69, 0x20, 0x54, 0xe3, 0x67, 0x53, 0x69, 0x6d, 0x69, 0x72, 0x73,
0x69, 0x6b, 0x69, 0x20, 0x54, 0xe3, 0x67, 0x4f, 0x6d, 0x129, 0x6e, 0x68,
0x6b, 0x69, 0x4f, 0x72, 0x61, 0x72, 0x46, 0x79, 0x6e, 0xe3, 0x67, 0x20,
0x50, 0x1ebd, 0x6a, 0x50, 0xe3, 0x6e, 0x74, 0x69, 0x1ef9, 0x6e, 0x1ef9, 0x6b,
0x69, 0x50, 0x69, 0xe3, 0x67, 0x2d, 0x6a, 0x1ef9, 0x67, 0x4b, 0x6f, 0x6a,
0x74, 0x61, 0x6e, 0x1ef9, 0x6a, 0x4b, 0x79, 0x6a, 0x79, 0x72, 0x6f, 0x72,
0x6e, 0x61, 0x52, 0x69, 0x6a, 0x61, 0x6e, 0x65, 0x53, 0x61, 0x6b, 0x61,
0x72, 0x129, 0x6e, 0x1ef9, 0x53, 0x61, 0x6d, 0x1ef9, 0x72, 0x6b, 0x1ef9, 0x6e,
0x6e, 0x61, 0x53, 0x65, 0x2019, 0x75, 0x72, 0x53, 0x1ef9, 0x67, 0x67, 0x61,
0x6a, 0x53, 0x129, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x53, 0x69, 0x72,
0x65, 0x6e, 0x1ebd, 0x6b, 0x6f, 0x72, 0x79, 0x6d, 0x73, 0x69, 0x6b, 0x69,
0x54, 0x61, 0x6e, 0x68, 0x6b, 0x1ebd, 0x6e, 0x74, 0x69, 0x54, 0x69, 0x6d,
0x69, 0x72, 0x69, 0x6a, 0x69, 0x54, 0x65, 0x2019, 0x65, 0x72, 0x1ef9, 0x54,
0x129, 0x6d, 0x66, 0x75, 0x54, 0xf3, 0x6b, 0x69, 0x6a, 0x6f, 0x54, 0x6f,
0x6d, 0x129, 0x6e, 0x68, 0x6b, 0x69, 0x55, 0x72, 0x61, 0x6e, 0x20, 0x4d,
0x61, 0x74, 0x6f, 0x72, 0x55, 0x72, 0x169, 0x6d, 0x6b, 0x69, 0x55, 0x6e,
0x68, 0x74, 0x69, 0x2d, 0x4e, 0x1ebd, 0x72, 0x61, 0x56, 0x69, 0x6a, 0x1ebd,
0x74, 0x69, 0x6a, 0x1ef9, 0x6e, 0x129, 0x55, 0x72, 0x61, 0x6e, 0x69, 0x76,
0xf3, 0x6e, 0x68, 0x74, 0xf3, 0x6b, 0x69, 0x59, 0x6a, 0x61, 0x6b, 0x75,
0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x52, 0x61, 0x6e, 0x67, 0x169, 0x6d,
0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129, 0x6e, 0x6d, 0x75, 0x72, 0x67,
0x75, 0x4a, 0x65, 0x72, 0x65, 0x76, 0x1ef9, 0x41, 0x73, 0x6f, 0x72, 0x65,
0x6e, 0x68, 0x4d, 0x65, 0x72, 0x6d, 0x169, 0x6e, 0x61, 0x4b, 0x61, 0x6e,
0x1ef9, 0x72, 0x69, 0x6a, 0x61, 0x20, 0x41, 0x67, 0x50, 0x75, 0x20, 0x54,
0xe1, 0x6e, 0x68, 0x52, 0x65, 0x6a, 0x6b, 0x69, 0x6a, 0x61, 0x76, 0x69,
0x6b, 0x69, 0x4a, 0x69, 0xf3, 0x72, 0x6a, 0x61, 0x20, 0x74, 0x1ef9, 0x20,
0x53, 0x75, 0x72, 0x53, 0x1ef9, 0x74, 0x61, 0x20, 0x45, 0x72, 0x65, 0x6e,
0x1ef9, 0x53, 0x69, 0x74, 0x1ef9, 0x72, 0x65, 0x6a, 0x41, 0x6e, 0x65, 0x72,
0x61, 0x6a, 0x6e, 0x69, 0x4d, 0x72, 0x69, 0x6e, 0x68, 0x6d, 0x61, 0x6e,
0x1ebd, 0x4d, 0x72, 0x75, 0x6b, 0x65, 0x6e, 0x20, 0x48, 0x69, 0x72, 0x4e,
0x61, 0x72, 0x76, 0x129, 0x6e, 0x45, 0x75, 0x6b, 0x72, 0x61, 0x48, 0x6f,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x52, 0x129, 0x6e, 0x65, 0x72, 0x6d, 0x1ef9,
0x6e, 0x52, 0xf3, 0x72, 0x6e, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x65, 0x4d,
0x1ebd, 0x72, 0x6d, 0x75, 0x72, 0x6e, 0x129, 0x50, 0xe9, 0x72, 0x74, 0x69,
0x53, 0x69, 0x6e, 0x69, 0x6e, 0x65, 0x6a, 0x41, 0x6d, 0x129, 0x6e, 0x68,
0x74, 0x65, 0x72, 0x6e, 0xe1, 0x1ef8, 0x6e, 0x6f, 0x68, 0x61, 0x41, 0x6e,
0x68, 0x74, 0x61, 0x6b, 0x1ef9, 0x41, 0x74, 0x65, 0x6e, 0x1ef9, 0x4d, 0x65,
0x72, 0x67, 0x72, 0x61, 0x6e, 0x6f, 0x4d, 0x65, 0x72, 0x2d, 0x72, 0x129,
0x67, 0x4d, 0x72, 0x61, 0x74, 0x69, 0x6e, 0x68, 0x72, 0x61, 0x76, 0x61,
0x4d, 0x72, 0x75, 0x73, 0xe9, 0x72, 0x61, 0x4d, 0x75, 0x6b, 0x61, 0x72,
0xe9, 0x6e, 0x68, 0x74, 0x69, 0x4d, 0x75, 0x6e, 0x61, 0x70, 0x65, 0x6e,
0x68, 0x74, 0x65, 0x4d, 0x79, 0x6a, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x53,
0x69, 0x6a, 0x69, 0x6e, 0x1ef9, 0x76, 0x6f, 0x4b, 0x6f, 0x70, 0x1ebd, 0x6e,
0x68, 0x1ef9, 0x67, 0x65, 0x4e, 0x75, 0x6d, 0x72, 0x69, 0x6e, 0x4a, 0x69,
0x6d, 0x72, 0x61, 0x74, 0x61, 0x72, 0x47, 0xe9, 0x72, 0x6e, 0x129, 0x73,
0x65, 0x6a, 0x48, 0xe9, 0x72, 0x73, 0x129, 0x67, 0x6b, 0x65, 0x168, 0x6e,
0x2d, 0x67, 0x72, 0xe9, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73,
0x6f, 0x49, 0x6e, 0x68, 0x74, 0x61, 0x6d, 0x75, 0x72, 0x4a, 0xe9, 0x72,
0x73, 0x65, 0x6a, 0x4b, 0x61, 0x72, 0x69, 0x6e, 0x129, 0x6e, 0x67, 0x72,
0x61, 0x6e, 0x6f, 0x4b, 0x69, 0x72, 0xf3, 0x76, 0x69, 0x4b, 0x69, 0x6a,
0xe9, 0x76, 0x69, 0x52, 0x69, 0x6e, 0x68, 0x6d, 0x6f, 0x76, 0x61, 0x52,
0x69, 0x6d, 0x72, 0x69, 0x1ef9, 0x6e, 0x1ef9, 0x52, 0xe3, 0x6e, 0x65, 0x72,
0x65, 0x52, 0x75, 0x73, 0x1ebd, 0x67, 0x6d, 0x75, 0x72, 0x67, 0x6f, 0x4d,
0x1ef9, 0x6e, 0x69, 0x72, 0x69, 0x4d, 0x61, 0x72, 0x74, 0x61, 0x4d, 0x1ef9,
0x72, 0x69, 0x65, 0x68, 0x1ef9, 0x6d, 0x4d, 0x129, 0x67, 0x73, 0x69, 0x6b,
0x69, 0x4d, 0xe3, 0x6e, 0x1ef9, 0x6b, 0x6f, 0x4d, 0xe3, 0x6e, 0x68, 0x6b,
0x6f, 0x76, 0xd3, 0x73, 0x69, 0x72, 0x6f, 0x50, 0x61, 0x72, 0x69, 0x6e,
0x68, 0x50, 0x6f, 0x6e, 0x67, 0xf3, 0x72, 0x69, 0x6b, 0x61, 0x52, 0x6f,
0x6d, 0x1ef9, 0x53, 0x61, 0x6d, 0x1ef9, 0x72, 0x61, 0x53, 0x1ef9, 0x20, 0x4d,
0x1ef9, 0x72, 0x129, 0x6e, 0x169, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76,
0x6f, 0x2f, 0x53, 0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0x75, 0x53, 0x61,
0x72, 0x61, 0x74, 0xf3, 0x76, 0x69, 0x53, 0x129, 0x67, 0x66, 0x69, 0x72,
0x69, 0x70, 0x6f, 0x72, 0x53, 0x69, 0x6b, 0x6f, 0x70, 0x69, 0x6a, 0x65,
0x53, 0xf3, 0x66, 0x69, 0x6a, 0x61, 0x45, 0x6e, 0x68, 0x74, 0x75, 0x6b,
0x6f, 0x72, 0x6d, 0x169, 0x54, 0x61, 0x72, 0x129, 0x6e, 0x54, 0x69, 0x72,
0x1ef9, 0x6e, 0x1ef9, 0x55, 0x72, 0x69, 0x6a, 0x61, 0x6e, 0xe3, 0x76, 0x69,
0x6e, 0x68, 0x6b, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x6a, 0x56, 0x61, 0x74,
0x69, 0x6b, 0x1ef9, 0x6e, 0x169, 0x56, 0x69, 0x65, 0x6e, 0x1ef9, 0x56, 0x69,
0x72, 0x6e, 0x129, 0x6a, 0x75, 0x73, 0x69, 0x56, 0x6f, 0x72, 0x67, 0x75,
0x67, 0x72, 0x61, 0x6e, 0x6f, 0x56, 0x61, 0x72, 0x73, 0xf3, 0x76, 0x69,
0x6a, 0x61, 0x4a, 0x61, 0x67, 0x72, 0xe9, 0x6d, 0x65, 0x4a, 0x75, 0x72,
0x69, 0x6b, 0x65, 0x1ef8, 0x74, 0x61, 0x6e, 0x1ef9, 0x6e, 0x1ef9, 0x72, 0x69,
0x76, 0x6f, 0x53, 0x61, 0x67, 0x6f, 0x4b, 0x72, 0x69, 0x6e, 0x68, 0x74,
0x69, 0x6d, 0x1ef9, 0x4b, 0xf3, 0x6b, 0x6f, 0x20, 0x41, 0x67, 0x4b, 0x6f,
0x6d, 0xe3, 0x72, 0x65, 0x4b, 0x69, 0x72, 0x67, 0xe9, 0x72, 0x65, 0x6e,
0x4d, 0x1ef9, 0x68, 0xe9, 0x4d, 0x1ef9, 0x72, 0x6e, 0x69, 0x76, 0x61, 0x4d,
0xe3, 0x72, 0x69, 0x73, 0x69, 0x76, 0x4d, 0x1ef9, 0x6a, 0xf3, 0x74, 0x69,
0x52, 0x169, 0x6e, 0x6a, 0x169, 0x76, 0xd3, 0x67, 0x2d, 0x72, 0x1ef9, 0x6e,
0x4d, 0x75, 0x67, 0x1ef9, 0x6e, 0x76, 0x69, 0x72, 0x65, 0x53, 0x61, 0x74,
0x69, 0x6e, 0x1ef9, 0x6d, 0x50, 0x61, 0x73, 0x69, 0x6b, 0x75, 0x76, 0x61,
0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x46, 0x61, 0x6b,
0x61, 0x6f, 0x66, 0x75, 0x46, 0x75, 0x6e, 0x1ef9, 0x66, 0x75, 0x74, 0x69,
0x47, 0x61, 0x72, 0x61, 0x20, 0x50, 0x61, 0x67, 0x6f, 0x47, 0x1ef9, 0x6d,
0x6d, 0x69, 0xe9, 0x72, 0x47, 0x75, 0x76, 0x61, 0x6e, 0x61, 0x72, 0x6b,
0x61, 0x6e, 0x1ef9, 0x72, 0x47, 0x75, 0x76, 0x1ef9, 0x6d, 0x48, 0x6f, 0x6e,
0xe3, 0x72, 0x75, 0x72, 0x75, 0x1ebc, 0x6e, 0x6e, 0x65, 0x72, 0x6d, 0x75,
0x72, 0x69, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x1ef9, 0x74, 0x69,
0x4b, 0x6f, 0x73, 0x69, 0x72, 0x61, 0x6a, 0x65, 0x4b, 0x75, 0x76, 0x61,
0x6a, 0x61, 0x72, 0x1ebd, 0x6a, 0x4d, 0x1ef9, 0x6a, 0x75, 0x72, 0x6f, 0x4d,
0x1ef9, 0x72, 0x6b, 0x65, 0x6a, 0x61, 0x20, 0x46, 0x61, 0x67, 0x4d, 0x129,
0x6e, 0x6e, 0x75, 0x76, 0x65, 0x6a, 0x4e, 0x1ef9, 0x76, 0x75, 0x72, 0x75,
0x4e, 0x129, 0x76, 0x65, 0x4e, 0xe3, 0x72, 0x66, 0x6f, 0x6b, 0x69, 0x4e,
0xe3, 0x75, 0x6d, 0x1ebd, 0x61, 0x50, 0x61, 0x72, 0x61, 0x76, 0x50, 0x69,
0x74, 0x69, 0x6b, 0x61, 0x69, 0x72, 0x50, 0xf3, 0x72, 0x20, 0x4d, 0xe3,
0x72, 0x65, 0x6e, 0x68, 0x6d, 0x69, 0x52, 0x61, 0x72, 0xe3, 0x74, 0xe3,
0x67, 0x61, 0x53, 0x61, 0x6a, 0x70, 0x1ef9, 0x54, 0x61, 0x6a, 0x74, 0x69,
0x54, 0x61, 0x72, 0x61, 0x75, 0x76, 0x61, 0x54, 0xe3, 0x6e, 0x67, 0x61,
0x74, 0x61, 0x70, 0x75, 0x56, 0x61, 0x6b, 0x69, 0x56, 0x61, 0x72, 0x69,
0x6e, 0x68, 0x41, 0x62, 0x69, 0x64, 0x69, 0x79, 0xe3, 0x41, 0x6b, 0x61,
0x72, 0x61, 0x41, 0x64, 0x69, 0x73, 0x69, 0x20, 0x41, 0x62, 0x65, 0x62,
0x61, 0x41, 0x67, 0x65, 0x75, 0x41, 0x73, 0x69, 0x6d, 0x61, 0x72, 0x61,
0x42, 0xe3, 0x6b, 0x69, 0x42, 0xe3, 0x6a, 0xfa, 0x42, 0x61, 0x72, 0xe3,
0x74, 0x69, 0x72, 0x65, 0x42, 0x61, 0x72, 0x61, 0x7a, 0x61, 0x77, 0x69,
0x72, 0x69, 0x42, 0x75, 0x79, 0x169, 0x62, 0x75, 0x72, 0x61, 0x4b, 0x61,
0x69, 0x72, 0x75, 0x55, 0x6b, 0x61, 0x20, 0x4d, 0x75, 0x72, 0x75, 0x74,
0x129, 0x67, 0x61, 0x4b, 0x75, 0x6e, 0x61, 0x6b, 0x69, 0x72, 0x69, 0x4b,
0x61, 0x74, 0x75, 0x61, 0x77, 0x61, 0x20, 0x72, 0x75, 0x6b, 0x61, 0x44,
0x69, 0x6a, 0x69, 0x62, 0x75, 0x74, 0x69, 0x44, 0x75, 0x77, 0x61, 0x6c,
0x61, 0x45, 0x75, 0x20, 0x41, 0x79, 0x169, 0x46, 0x69, 0x72, 0x65, 0x74,
0x169, 0x47, 0x61, 0x62, 0x75, 0x72, 0x75, 0x6e, 0x69, 0x4a, 0x75, 0x61,
0x6e, 0x65, 0x73, 0x69, 0x62, 0x75, 0x67, 0x75, 0x59, 0x75, 0x62, 0x61,
0x4b, 0xe3, 0x70, 0x61, 0x72, 0x61, 0x4b, 0x61, 0x74, 0x169, 0x4b, 0x129,
0x78, 0x61, 0x73, 0x61, 0x49, 0x70, 0x61, 0x77, 0x61, 0x69, 0x74, 0x61,
0x52, 0x69, 0x62, 0x65, 0x72, 0x65, 0x76, 0x69, 0x6c, 0x69, 0x52, 0x75,
0x6d, 0xe9, 0x52, 0x75, 0xe3, 0x64, 0x61, 0x4c, 0x75, 0x62, 0x169, 0x62,
0x61, 0x78, 0x69, 0x168, 0x62, 0x61, 0x62, 0x61, 0x6e, 0x69, 0x4d, 0x75,
0x67, 0x61, 0x64, 0x69, 0x73, 0x69, 0x6b, 0x75, 0x4d, 0x169, 0x72, 0x75,
0x77, 0x69, 0x79, 0x61, 0x4e, 0x61, 0x69, 0x72, 0x75, 0x62, 0x69, 0x4e,
0x2019, 0x20, 0x44, 0x69, 0x79, 0x61, 0x6d, 0x65, 0x6e, 0x61, 0x4e, 0x6f,
0x77, 0x61, 0x6b, 0x69, 0x78, 0x75, 0x74, 0x69, 0x55, 0x77, 0x61, 0x67,
0x61, 0x64, 0x75, 0x67, 0x75, 0x49, 0x67, 0x61, 0x72, 0x61, 0x70, 0x61,
0x77, 0x61, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x53, 0xe3, 0x20,
0x54, 0x75, 0x6d, 0xe9, 0x54, 0x69, 0x72, 0x69, 0x70, 0x75, 0x72, 0x69,
0x54, 0x75, 0x6e, 0x69, 0x2d, 0x69, 0x74, 0x61, 0x57, 0x129, 0x64, 0x75,
0x65, 0x6b, 0x69, 0xc3, 0x6b, 0x75, 0x72, 0x61, 0x67, 0x65, 0xc3, 0x67,
0x69, 0x72, 0x61, 0xc3, 0x74, 0x69, 0x67, 0x75, 0x77, 0x61, 0x41, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x69, 0x6e, 0x61, 0x42, 0x75, 0x77, 0x65,
0x6e, 0x75, 0x73, 0x61, 0x69, 0x72, 0x69, 0x4b, 0x61, 0x74, 0x61, 0x6d,
0x61, 0x6b, 0x61, 0x4b, 0xf3, 0x64, 0x75, 0x62, 0x61, 0x59, 0x75, 0x79,
0x75, 0x69, 0x52, 0x61, 0x20, 0x52, 0x69, 0x6f, 0x79, 0x61, 0x4d, 0x1ebd,
0x64, 0x75, 0x73, 0x61, 0x50, 0x61, 0x72, 0x61, 0x6e, 0xe3, 0x20, 0x47,
0x61, 0x72, 0x65, 0x67, 0x75, 0x2d, 0x69, 0x74, 0x61, 0x53, 0x61, 0x75,
0x74, 0x61, 0x53, 0xe3, 0x20, 0x59, 0x75, 0x77, 0xe3, 0x53, 0xe3, 0x20,
0x52, 0x75, 0x69, 0x54, 0x75, 0x6b, 0x75, 0x6d, 0xe3, 0x55, 0x78, 0x75,
0x77, 0x61, 0x79, 0x61, 0x41, 0x73, 0x169, 0x73, 0x69, 0x79, 0xf5, 0x41,
0x74, 0x69, 0x6b, 0x6f, 0x6b, 0xe3, 0x42, 0x61, 0xed, 0x79, 0x61, 0x42,
0xe3, 0x64, 0x65, 0x72, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b, 0x75,
0x61, 0x72, 0x61, 0x42, 0x61, 0x72, 0x61, 0x62, 0x61, 0x64, 0x75, 0x42,
0x65, 0x6c, 0x1ebd, 0x42, 0x65, 0x72, 0x69, 0x73, 0x65, 0x49, 0x77, 0x69,
0x6b, 0x75, 0xed, 0x20, 0x4d, 0x75, 0x72, 0x75, 0x74, 0x129, 0x67, 0x61,
0x42, 0x75, 0x77, 0x61, 0x20, 0x56, 0x69, 0x73, 0x74, 0x61, 0x42, 0x75,
0x67, 0x75, 0x74, 0xe1, 0x42, 0x75, 0x77, 0x69, 0x73, 0xe9, 0x4b, 0x1ebd,
0x62, 0x69, 0x72, 0x69, 0x79, 0x69, 0x20, 0x4b, 0x75, 0x61, 0x72, 0x61,
0x4b, 0xe3, 0x70, 0x75, 0x20, 0x57, 0x61, 0x73, 0x75, 0x4b, 0xe3, 0x6b,
0x169, 0x4b, 0x61, 0x79, 0x65, 0x6e, 0x61, 0x4b, 0x61, 0x69, 0x6d, 0xe3,
0x53, 0x69, 0x76, 0x61, 0x76, 0x61, 0x4b, 0x75, 0x70, 0xe9, 0x20, 0x52,
0x69, 0x6b, 0x61, 0x4b, 0x65, 0x72, 0x65, 0x73, 0x74, 0x169, 0x44, 0x69,
0x6e, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x73, 0xe3, 0x76, 0x44,
0x61, 0x75, 0x73, 0x169, 0x44, 0x61, 0x75, 0x73, 0x169, 0x20, 0x4b, 0x69,
0x72, 0x69, 0x6b, 0x69, 0x44, 0x1ebd, 0x77, 0x65, 0x72, 0x44, 0x75, 0x6d,
0x69, 0x6e, 0x69, 0x6b, 0x61, 0x45, 0x64, 0x69, 0x6d, 0x169, 0x74, 0x169,
0x45, 0x75, 0x20, 0x53, 0x61, 0x77, 0x61, 0x64, 0x75, 0x46, 0x75, 0x74,
0x69, 0x20, 0x4e, 0x65, 0x75, 0x73, 0x169, 0x46, 0x75, 0x74, 0x61, 0x72,
0x65, 0x73, 0x61, 0x59, 0x72, 0x75, 0x73, 0xe3, 0x67, 0x75, 0x73, 0x75,
0x2d, 0x61, 0x74, 0xe3, 0x20, 0x4b, 0x75, 0x61, 0x72, 0x61, 0x49, 0x70,
0x65, 0x6b, 0x61, 0x77, 0x61, 0x73, 0x75, 0x20, 0x4b, 0x75, 0x61, 0x72,
0x61, 0x57, 0x61, 0x73, 0xfa, 0x20, 0x54, 0x75, 0x72, 0x69, 0x6b, 0x75,
0x47, 0x61, 0x72, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x47, 0x75, 0x61, 0x64,
0x61, 0x72, 0x75, 0x70, 0x69, 0x47, 0x75, 0x61, 0x74, 0x65, 0x6d, 0x61,
0x72, 0x61, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6b, 0x69, 0x75, 0x47, 0x69,
0x79, 0x61, 0x6e, 0x61, 0x48, 0x65, 0x6d, 0x75, 0x73, 0x69, 0x72, 0x75,
0x128, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x70, 0x75, 0x72, 0x69, 0x4b, 0x6e,
0x6f, 0x78, 0x2c, 0x20, 0x128, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x4d, 0x61,
0x72, 0x1ebd, 0x67, 0x75, 0x2c, 0x20, 0x128, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x50, 0x69, 0x74, 0x69, 0x72, 0x69, 0x73, 0x69, 0x62, 0x75, 0x67, 0x69,
0x2c, 0x20, 0x128, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x54, 0x65, 0x6c, 0x6c,
0x20, 0x54, 0x61, 0x77, 0x61, 0x2d, 0x77, 0x61, 0x73, 0xfa, 0x2c, 0x20,
0x128, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x57, 0x65, 0x77, 0x61, 0x69, 0x2c,
0x20, 0x128, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x57, 0x129, 0x73, 0x65, 0x6e,
0x65, 0x2d, 0x69, 0x74, 0x61, 0x2c, 0x20, 0x128, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x57, 0x69, 0x6e, 0xe3, 0x6d, 0x61, 0x6b, 0x69, 0x2c, 0x20, 0x128,
0x64, 0x69, 0x61, 0x6e, 0x61, 0x49, 0x6e, 0x75, 0x77, 0x69, 0x6b, 0x69,
0x49, 0x6b, 0x61, 0x72, 0x75, 0x69, 0x74, 0x69, 0x59, 0x61, 0x6d, 0x61,
0x69, 0x6b, 0x61, 0x59, 0x75, 0x6e, 0x65, 0x61, 0x75, 0x4c, 0x75, 0x77,
0x69, 0x73, 0x69, 0x76, 0x69, 0x72, 0x69, 0x4d, 0x169, 0x74, 0x69, 0x73,
0x65, 0x72, 0x75, 0x2c, 0x20, 0x4b, 0x1ebd, 0x74, 0x75, 0x6b, 0x69, 0x4b,
0x61, 0x72, 0x61, 0x72, 0x1ebd, 0x64, 0x69, 0x6b, 0x69, 0x52, 0x69, 0x6d,
0x61, 0x50, 0x69, 0x72, 0x129, 0x73, 0x69, 0x70, 0x69, 0x20, 0x51, 0x75,
0x61, 0x72, 0x74, 0x65, 0x72, 0x20, 0x55, 0x69, 0x72, 0x70, 0x65, 0x77,
0x61, 0x72, 0x61, 0x4d, 0x61, 0x73, 0x65, 0x79, 0xf3, 0x4d, 0x61, 0x6e,
0xe1, 0x67, 0x75, 0x77, 0x61, 0x4d, 0x61, 0x72, 0x69, 0x67, 0x75, 0x74,
0x69, 0x4d, 0x61, 0x74, 0x69, 0x6e, 0x69, 0x6b, 0x61, 0x4d, 0x6f, 0x72,
0x6f, 0x69, 0x74, 0x61, 0x79, 0x75, 0x6b, 0xe1, 0x4d, 0x61, 0x73, 0x61,
0x74, 0x61, 0x72, 0xe3, 0x4d, 0x65, 0x74, 0x69, 0x72, 0x61, 0x6b, 0x61,
0x74, 0x69, 0x72, 0x61, 0x1ebc, 0x6d, 0xe3, 0x20, 0x74, 0x1ef9, 0x20, 0x4d,
0x1ebd, 0x73, 0x69, 0x6b, 0x75, 0x4d, 0x69, 0x6b, 0x69, 0x72, 0x169, 0x4d,
0x169, 0x6b, 0x69, 0x74, 0x169, 0x4d, 0xf5, 0x74, 0x65, 0x72, 0x65, 0x69,
0x4d, 0x169, 0x74, 0x69, 0x77, 0x69, 0x64, 0x65, 0x77, 0x75, 0x4d, 0xf5,
0x74, 0x69, 0x73, 0x65, 0x72, 0x61, 0x74, 0x69, 0x59, 0x75, 0x72, 0x6b,
0x69, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x52, 0xe9, 0x72, 0x61,
0x46, 0x65, 0x6e, 0xe3, 0x64, 0x75, 0x20, 0x4e, 0x75, 0x72, 0x75, 0x79,
0xe3, 0x42, 0x65, 0x75, 0x72, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74,
0x61, 0x20, 0x4e, 0x75, 0x74, 0x69, 0x20, 0x73, 0x75, 0xed, 0x53, 0x1ebd,
0x74, 0x65, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x20, 0x4e,
0x75, 0x74, 0x69, 0x20, 0x73, 0x75, 0xed, 0x50, 0x69, 0x73, 0x61, 0x73,
0xfa, 0x20, 0x53, 0x61, 0x72, 0x1ebd, 0x2c, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x20, 0x4e, 0x75, 0x74, 0x69, 0x20, 0x73, 0x75, 0xed, 0x4e,
0x75, 0x6b, 0x69, 0x4f, 0x79, 0x69, 0x6e, 0x61, 0x67, 0x61, 0x50, 0x61,
0x6e, 0x61, 0x6d, 0xe3, 0x50, 0x75, 0x77, 0x65, 0x6e, 0x69, 0x6b, 0x69,
0x73, 0x69, 0x49, 0x67, 0x61, 0x72, 0x61, 0x70, 0x61, 0x77, 0x61, 0x20,
0x50, 0x69, 0x72, 0x129, 0x73, 0x69, 0x70, 0x69, 0x45, 0x73, 0x69, 0x70,
0xe3, 0x79, 0x61, 0x20, 0x49, 0x67, 0x61, 0x72, 0x61, 0x70, 0x61, 0x77,
0x61, 0x49, 0x67, 0x61, 0x72, 0x61, 0x70, 0x61, 0x77, 0x61, 0x20, 0x4b,
0x75, 0x78, 0x69, 0x6d, 0x61, 0x77, 0x61, 0x72, 0x61, 0x49, 0x67, 0x61,
0x72, 0x61, 0x70, 0x61, 0x77, 0x61, 0x20, 0x52, 0x69, 0x6b, 0x75, 0x50,
0x169, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x6e, 0x61, 0x2d, 0x69, 0x74,
0x61, 0x52, 0xe3, 0x6b, 0x129, 0x20, 0x55, 0x69, 0x6b, 0x65, 0x73, 0x61,
0x77, 0x61, 0x52, 0x65, 0x6a, 0x129, 0x6e, 0x1ef9, 0x52, 0x65, 0x73, 0x6f,
0x72, 0x75, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x6e, 0xe3, 0x20, 0x4d,
0x75, 0x72, 0x75, 0x74, 0x129, 0x67, 0x61, 0x53, 0xe3, 0x74, 0x61, 0x72,
0x1ebd, 0x53, 0xe3, 0x74, 0x69, 0x79, 0x61, 0x67, 0x75, 0x53, 0xe3, 0x74,
0x75, 0x20, 0x44, 0x75, 0x6d, 0x129, 0x67, 0x75, 0x53, 0xe3, 0x20, 0x50,
0x61, 0x75, 0x72, 0x75, 0x49, 0x74, 0x75, 0x6b, 0x75, 0x74, 0x75, 0x6d,
0x69, 0x74, 0x69, 0x53, 0x69, 0x74, 0x69, 0x6b, 0x61, 0x53, 0xe3, 0x20,
0x42, 0x61, 0x74, 0x75, 0x6c, 0x75, 0x6d, 0x65, 0x75, 0x53, 0xe3, 0x74,
0x69, 0x20, 0x4a, 0x75, 0x6e, 0x69, 0x53, 0xe3, 0x20, 0x4b, 0x69, 0x72,
0x69, 0x74, 0x75, 0x77, 0xe3, 0x75, 0x53, 0xe3, 0x74, 0x61, 0x20, 0x4c,
0x75, 0x73, 0x69, 0x79, 0x61, 0x53, 0xe3, 0x74, 0x69, 0x20, 0x54, 0x6f,
0x6d, 0xe1, 0x53, 0xe3, 0x20, 0x57, 0x69, 0x73, 0x1ebd, 0x74, 0x69, 0x50,
0x61, 0x72, 0x61, 0x6e, 0xe3, 0x20, 0x50, 0x69, 0x72, 0xe3, 0x74, 0xe3,
0x54, 0x65, 0x67, 0x75, 0x73, 0x69, 0x67, 0x61, 0x75, 0x70, 0x61, 0x54,
0x69, 0x78, 0x75, 0x72, 0x69, 0x54, 0x69, 0x6a, 0x75, 0x76, 0x1ef9, 0x6e,
0x1ef9, 0x54, 0x75, 0x72, 0x169, 0x74, 0x75, 0x54, 0x75, 0x74, 0x75, 0x72,
0x61, 0x56, 0xe3, 0x6b, 0x75, 0x77, 0x65, 0x72, 0x69, 0x4b, 0x61, 0x77,
0x61, 0x72, 0x75, 0x20, 0x4d, 0x75, 0x72, 0x75, 0x74, 0x129, 0x67, 0x61,
0x57, 0x69, 0x6e, 0x129, 0x70, 0x65, 0x67, 0x69, 0x4b, 0x61, 0x73, 0x65,
0x69, 0x44, 0x61, 0x77, 0x69, 0x4d, 0x61, 0x6b, 0x69, 0x6b, 0x75, 0x61,
0x72, 0x69, 0x4d, 0x61, 0x75, 0x73, 0x169, 0x4d, 0x61, 0x6b, 0x69, 0x4d,
0x75, 0x64, 0x75, 0x50, 0x61, 0x75, 0x6d, 0x65, 0x72, 0x65, 0x57, 0x6f,
0x73, 0x69, 0x74, 0x75, 0x6b, 0x69, 0x4c, 0x6f, 0x6e, 0x67, 0x79, 0x65,
0x61, 0x72, 0x20, 0x54, 0x61, 0x77, 0x61, 0x2d, 0x77, 0x61, 0x73, 0xfa,
0x41, 0x64, 0x1ebd, 0x41, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x41, 0x6b, 0x69,
0x74, 0x75, 0x62, 0x69, 0x41, 0x73, 0x69, 0x67, 0x61, 0x62, 0x61, 0x74,
0x69, 0x4d, 0x61, 0x67, 0x6e, 0x61, 0x42, 0x61, 0x72, 0x1ebd, 0x69, 0x4d,
0xe1, 0x67, 0x2d, 0x6b, 0xf3, 0x6b, 0x69, 0x42, 0x69, 0x73, 0x69, 0x6b,
0x65, 0x6b, 0x69, 0x42, 0x75, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x58, 0x69,
0x74, 0x61, 0x58, 0x75, 0x69, 0x62, 0x61, 0x75, 0x73, 0xe3, 0x4b, 0x75,
0x72, 0x169, 0x62, 0x75, 0x44, 0x61, 0x6d, 0x61, 0x73, 0x69, 0x6b, 0x75,
0x44, 0x69, 0x72, 0x69, 0x44, 0x75, 0x78, 0xe3, 0x62, 0x69, 0x46, 0x61,
0x6d, 0x61, 0x67, 0x75, 0x69, 0x74, 0x61, 0x48, 0x65, 0x62, 0x75, 0x72,
0x169, 0x48, 0x169, 0x67, 0x20, 0x4b, 0x169, 0x67, 0x48, 0x6f, 0x77, 0x69,
0x64, 0x69, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69,
0x4a, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x59, 0x61, 0x79, 0x61, 0x70, 0x75,
0x72, 0x61, 0x59, 0x65, 0x72, 0x75, 0x73, 0x61, 0x72, 0x1ebd, 0x69, 0x4b,
0xe3, 0x78, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x72, 0x61, 0x78,
0x69, 0x4b, 0x61, 0x74, 0x69, 0x6d, 0xe3, 0x64, 0x75, 0x4b, 0xe3, 0x64,
0x69, 0x67, 0x61, 0x4b, 0x61, 0x75, 0x6b, 0x75, 0x74, 0xe1, 0x4b, 0x61,
0x72, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x69, 0x73, 0x69,
0x6b, 0x69, 0x4b, 0x75, 0x61, 0x72, 0x61, 0x20, 0x52, 0x169, 0x70, 0x75,
0x72, 0x75, 0x4b, 0x75, 0x78, 0x129, 0x67, 0x69, 0x4d, 0x61, 0x67, 0x61,
0x64, 0xe3, 0x4d, 0x61, 0x73, 0x69, 0x6b, 0x61, 0x74, 0x69, 0x4b, 0x75,
0x73, 0x69, 0x6e, 0x65, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x50,
0x69, 0x73, 0x61, 0x73, 0xfa, 0x53, 0x69, 0x62, 0x69, 0x72, 0x69, 0x73,
0x69, 0x6b, 0x69, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x4f, 0x6d,
0x69, 0x73, 0x69, 0x6b, 0x69, 0x52, 0x61, 0x75, 0x72, 0x65, 0x50, 0x69,
0x6e, 0xf5, 0x20, 0x50, 0x1ebd, 0x69, 0x50, 0x169, 0x74, 0x69, 0x61, 0x6e,
0x61, 0x6b, 0x61, 0x50, 0x69, 0x169, 0x67, 0x69, 0xe3, 0x67, 0x69, 0x4b,
0x61, 0x74, 0x61, 0x72, 0x69, 0x4b, 0x6f, 0x73, 0x69, 0x74, 0x61, 0x6e,
0x61, 0x69, 0x4b, 0x69, 0x73, 0x69, 0x72, 0x6f, 0x64, 0x61, 0x53, 0x61,
0x6b, 0x61, 0x72, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x6b, 0xe3,
0x64, 0x61, 0x58, 0xe3, 0x67, 0x61, 0x69, 0x58, 0x65, 0x72, 0x65, 0x64,
0x69, 0x6e, 0x65, 0x6b, 0x6f, 0x72, 0x69, 0x6d, 0x69, 0x7a, 0x69, 0x6b,
0x69, 0x54, 0x61, 0x73, 0x69, 0x6b, 0x1ebd, 0x74, 0x69, 0x54, 0x65, 0x79,
0x65, 0x72, 0xe3, 0x54, 0x129, 0x70, 0x75, 0x54, 0x6f, 0x6b, 0x69, 0x79,
0x75, 0x54, 0x6f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x55, 0x72, 0xe3,
0x20, 0x42, 0x61, 0x74, 0x75, 0x72, 0x75, 0x55, 0x72, 0x169, 0x6b, 0x69,
0x55, 0x73, 0x69, 0x74, 0x69, 0x2d, 0x4e, 0x65, 0x72, 0x61, 0x56, 0x69,
0x1ebd, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x57, 0x61, 0x72, 0x61, 0x64, 0x69,
0x77, 0x6f, 0x73, 0x69, 0x74, 0x75, 0x6b, 0x69, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x52, 0xe3, 0x67, 0x169, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x129, 0x62, 0x75, 0x67, 0x75, 0x59, 0x65, 0x72,
0x65, 0x77, 0xe3, 0x41, 0x73, 0x6f, 0x72, 0x69, 0x2d, 0x69, 0x74, 0x61,
0x42, 0x65, 0x6d, 0x75, 0x64, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x72, 0x69,
0x79, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x4b, 0x61, 0x62, 0x75, 0x20, 0x53,
0x75, 0x69, 0x6b, 0x69, 0x72, 0x69, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d,
0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x46, 0x61, 0x72, 0x75, 0x77, 0x65,
0x52, 0x65, 0x69, 0x6b, 0x69, 0x79, 0x61, 0x77, 0x69, 0x6b, 0x69, 0x47,
0x65, 0x75, 0x67, 0x69, 0x79, 0x61, 0x20, 0x53, 0x75, 0x20, 0x73, 0x75,
0xed, 0x53, 0xe3, 0x74, 0x61, 0x20, 0x45, 0x72, 0x1ebd, 0x6e, 0x61, 0x49,
0x73, 0x69, 0x74, 0xe3, 0x72, 0x65, 0x69, 0x41, 0x64, 0x65, 0x72, 0x61,
0x69, 0x64, 0x69, 0x42, 0x69, 0x72, 0x69, 0x62, 0x61, 0x6e, 0x69, 0x49,
0x77, 0x69, 0x74, 0x65, 0x72, 0x61, 0x20, 0x4d, 0x75, 0x70, 0x65, 0x6e,
0x61, 0xed, 0x72, 0x61, 0x44, 0x61, 0x72, 0x69, 0x77, 0x129, 0x45, 0x75,
0x6b, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x62, 0x61, 0x74, 0x69, 0x52, 0x129,
0x64, 0x65, 0x6d, 0xe3, 0x4c, 0x75, 0x72, 0x75, 0x64, 0x69, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x4d, 0x65, 0x75, 0x62, 0xfa, 0x6e, 0x69, 0x50, 0x65,
0x72, 0x69, 0x74, 0x69, 0x53, 0x69, 0x64, 0x69, 0x6e, 0x65, 0x69, 0x41,
0x6d, 0x69, 0x74, 0x65, 0x72, 0x69, 0x64, 0xe3, 0xc3, 0x64, 0x75, 0x72,
0x61, 0x41, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6b, 0xe3, 0x41, 0x74,
0x65, 0x6e, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x42, 0x65, 0x75, 0x67, 0x61,
0x72, 0x61, 0x64, 0x75, 0x42, 0x61, 0x72, 0x61, 0x74, 0x69, 0x73, 0x69,
0x6c, 0x61, 0x77, 0x61, 0x42, 0x75, 0x72, 0x75, 0x78, 0x65, 0x72, 0x61,
0x42, 0x75, 0x6b, 0x61, 0x72, 0x65, 0x74, 0x69, 0x42, 0x75, 0x64, 0x61,
0x70, 0x65, 0x74, 0x69, 0x42, 0x75, 0x73, 0x129, 0x67, 0x65, 0x6e, 0x69,
0x58, 0x69, 0x73, 0x69, 0x6e, 0x61, 0x75, 0x4b, 0x6f, 0x70, 0x1ebd, 0x79,
0x61, 0x67, 0x69, 0x44, 0x75, 0x62, 0x69, 0x72, 0x129, 0x47, 0x69, 0x62,
0x61, 0x72, 0x61, 0x75, 0x74, 0xe1, 0x47, 0x75, 0x77, 0x1ebd, 0x6e, 0x69,
0x73, 0x65, 0x69, 0x48, 0x65, 0x75, 0x73, 0x129, 0x6b, 0x65, 0x4d, 0xe3,
0x20, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x4a, 0x65, 0x72, 0x73,
0x65, 0x69, 0x4b, 0x61, 0x72, 0x69, 0x6e, 0x129, 0x67, 0x61, 0x72, 0x61,
0x64, 0x75, 0x4b, 0x69, 0x72, 0x6f, 0x77, 0x69, 0x4b, 0x69, 0x79, 0x65,
0x77, 0x65, 0x52, 0x69, 0x75, 0x62, 0x69, 0x72, 0x69, 0x61, 0x6e, 0x61,
0x52, 0x75, 0x78, 0x1ebd, 0x62, 0x75, 0x67, 0x75, 0x4d, 0x61, 0x75, 0x74,
0x61, 0x4d, 0x61, 0x72, 0x69, 0x1ebd, 0x79, 0xe3, 0x4d, 0x129, 0x73, 0x69,
0x6b, 0x69, 0x55, 0x73, 0x69, 0x72, 0x75, 0x50, 0x75, 0x64, 0x69, 0x67,
0x75, 0x72, 0x69, 0x6b, 0x61, 0x53, 0xe3, 0x20, 0x4d, 0x61, 0x72, 0x69,
0x6e, 0x75, 0x53, 0x61, 0x72, 0x61, 0x79, 0x65, 0x77, 0x6f, 0x53, 0x61,
0x72, 0x61, 0x74, 0x75, 0x77, 0x6f, 0x53, 0x129, 0x77, 0x65, 0x72, 0x75,
0x70, 0x75, 0x49, 0x73, 0x69, 0x6b, 0x75, 0x70, 0x69, 0x79, 0x65, 0x53,
0x75, 0x66, 0x69, 0x79, 0x61, 0x55, 0x6c, 0x69, 0x61, 0x6e, 0x75, 0x77,
0x69, 0x6b, 0x69, 0x56, 0x61, 0x64, 0x75, 0x73, 0x69, 0x57, 0x61, 0x74,
0x69, 0x6b, 0x61, 0x6e, 0x75, 0x57, 0x69, 0x79, 0x65, 0x6e, 0x61, 0x56,
0x69, 0x77, 0x69, 0x75, 0x2d, 0x69, 0x74, 0x61, 0x57, 0x75, 0x67, 0x6f,
0x67, 0x61, 0x72, 0x61, 0x64, 0x75, 0x57, 0x61, 0x72, 0x69, 0x73, 0xf3,
0x77, 0x69, 0x79, 0x61, 0x53, 0x61, 0x67, 0x61, 0x72, 0x65, 0x62, 0x69,
0x5a, 0x75, 0x72, 0x69, 0x6b, 0x69, 0xc3, 0x74, 0x61, 0x6e, 0x61, 0x6e,
0x61, 0x72, 0x69, 0x75, 0x58, 0x61, 0x67, 0x75, 0x2d, 0x69, 0x74, 0x61,
0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x2d, 0x69, 0x74, 0x61,
0x4b, 0x75, 0x6b, 0x75, 0x2d, 0x69, 0x74, 0x61, 0x4b, 0x75, 0x6d, 0x75,
0x72, 0x65, 0x2d, 0x69, 0x74, 0x61, 0x4b, 0x65, 0x72, 0x67, 0x65, 0x6c,
0x1ebd, 0x4d, 0x61, 0xe9, 0x4d, 0x61, 0x75, 0x64, 0x69, 0x77, 0x61, 0x2d,
0x69, 0x74, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x79, 0x75,
0x4d, 0x61, 0x79, 0x75, 0x74, 0x69, 0x59, 0x75, 0x6d, 0x75, 0x61, 0x74,
0x69, 0x72, 0x69, 0x73, 0x61, 0x77, 0x61, 0x41, 0x70, 0x69, 0x79, 0x61,
0xd3, 0x6b, 0x69, 0x72, 0xe3, 0x64, 0x69, 0x42, 0x75, 0x67, 0x61, 0x129,
0x77, 0x69, 0x72, 0x69, 0x58, 0x61, 0x74, 0x69, 0x6e, 0xe3, 0x50, 0x61,
0x73, 0x69, 0x6b, 0x75, 0x77, 0x61, 0x20, 0x4b, 0x61, 0x70, 0x75, 0xe3,
0x6d, 0x61, 0x45, 0x66, 0x61, 0x74, 0xe9, 0x46, 0x61, 0x6b, 0x61, 0x75,
0x66, 0x75, 0x47, 0x61, 0x72, 0x61, 0x70, 0x61, 0x67, 0x75, 0x2d, 0x69,
0x74, 0x61, 0x47, 0xe3, 0x62, 0x69, 0x65, 0x72, 0x65, 0x47, 0x75, 0x61,
0x64, 0x61, 0x75, 0x6b, 0x61, 0x6e, 0x61, 0x75, 0x47, 0x75, 0x77, 0xe3,
0x48, 0x75, 0x6e, 0x75, 0x72, 0x75, 0x72, 0x75, 0x1ebc, 0x64, 0x65, 0x62,
0x75, 0x72, 0x69, 0x4b, 0x75, 0x73, 0x69, 0x72, 0x61, 0x69, 0x4b, 0x75,
0x61, 0x79, 0x61, 0x72, 0x1ebd, 0x69, 0x4d, 0x61, 0x79, 0x75, 0x72, 0x75,
0x4d, 0x61, 0x6b, 0x65, 0x73, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x4d, 0x69,
0x64, 0x69, 0x77, 0x65, 0x69, 0x4e, 0x69, 0x77, 0x65, 0x4e, 0x75, 0x72,
0x75, 0x66, 0x75, 0x6b, 0x69, 0x4e, 0x75, 0x6d, 0x65, 0x79, 0x61, 0x50,
0x61, 0x67, 0x75, 0x20, 0x50, 0x61, 0x67, 0x75, 0x50, 0x61, 0x72, 0x61,
0x75, 0x49, 0x67, 0x61, 0x72, 0x61, 0x70, 0x61, 0x77, 0x61, 0x20, 0x4d,
0x6f, 0x72, 0x65, 0x73, 0x62, 0x79, 0x52, 0x61, 0x72, 0x75, 0x74, 0x169,
0x67, 0x61, 0x53, 0x61, 0x69, 0x70, 0xe3, 0x54, 0x169, 0x67, 0x61, 0x74,
0x61, 0x70, 0x75, 0x57, 0x61, 0x72, 0x69, 0x2d, 0x69, 0x74, 0x61, 0x53,
0xe3, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x75, 0x77, 0x61, 0x6e, 0x41,
0x6e, 0x74, 0x69, 0x69, 0x67, 0x75, 0x61, 0x41, 0x72, 0x75, 0x75, 0x62,
0x61, 0x42, 0x72, 0x61, 0x62, 0x61, 0x61, 0x64, 0x6f, 0x73, 0x44, 0x6f,
0x6d, 0x69, 0x69, 0x6e, 0x69, 0x6b, 0x61, 0x47, 0x72, 0x65, 0x6e, 0x61,
0x61, 0x64, 0x61, 0x47, 0x75, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x61, 0x6c,
0x61, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x53, 0x74, 0x65,
0x65, 0x64, 0x42, 0x65, 0x75, 0x6c, 0x61, 0x68, 0x2c, 0x20, 0x4e, 0x75,
0x75, 0x72, 0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x43, 0x65,
0x6e, 0x74, 0x65, 0x72, 0x2c, 0x20, 0x4e, 0x75, 0x75, 0x72, 0x64, 0x20,
0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x4e, 0x65, 0x77, 0x20, 0x53, 0x61,
0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x4e, 0x75, 0x75, 0x72, 0x64, 0x20, 0x44,
0x61, 0x6b, 0x6f, 0x74, 0x61, 0x41, 0x73, 0x63, 0x68, 0x67, 0x61, 0x62,
0x61, 0x74, 0x48, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x20, 0x4d, 0x69, 0x6e,
0x68, 0x20, 0x53, 0x74, 0x65, 0x65, 0x64, 0x4b, 0x75, 0x74, 0x73, 0x63,
0x68, 0x69, 0x6e, 0x67, 0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x75, 0x73, 0x6e,
0x65, 0x74, 0x73, 0x6b, 0x4b, 0x61, 0x74, 0x61, 0x61, 0x72, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x75, 0x75, 0x72, 0x54, 0x61, 0x69, 0x70, 0x65,
0x65, 0x55, 0x72, 0x75, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x69, 0x57, 0x6a,
0x65, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x52, 0x61, 0x6e, 0x67, 0x75,
0x75, 0x6e, 0x41, 0x74, 0x73, 0x6f, 0x6f, 0x72, 0x65, 0x6e, 0x42, 0x65,
0x72, 0x6d, 0x75, 0x75, 0x64, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x61, 0x72,
0x65, 0x6e, 0x4b, 0x61, 0x70, 0x77, 0x65, 0x72, 0x64, 0x65, 0x6e, 0x46,
0xe4, 0x72, 0xf6, 0x65, 0x72, 0x6e, 0x53, 0xfc, 0xfc, 0x64, 0x67, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x65, 0x6e, 0x41, 0x74, 0x68, 0x65, 0x65, 0x6e,
0x4b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x75, 0x75, 0x77, 0x65, 0x6e, 0x47,
0x69, 0x62, 0x72, 0x61, 0x6c, 0x74, 0x61, 0x61, 0x72, 0x45, 0x69, 0x6c,
0x75, 0x6e, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x75, 0x6b, 0x73, 0x65, 0x6d,
0x62, 0x6f, 0x72, 0x69, 0x67, 0x52, 0x69, 0x69, 0x67, 0x61, 0x57, 0x61,
0x74, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x74, 0xe4, 0xe4, 0x64, 0x57,
0x69, 0x69, 0x6e, 0x4a, 0x75, 0x6c, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e,
0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65,
0x6e, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x69, 0x77, 0x65, 0x6e, 0x50,
0x75, 0x61, 0x73, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75,
0x6e, 0x41, 0x72, 0x67, 0xea, 0xc7, 0xe9, 0x75, 0x74, 0x61, 0x4e, 0x61,
0x69, 0x72, 0xf6, 0x62, 0x69, 0x54, 0x75, 0x6e, 0x65, 0x78, 0x69, 0x43,
0xf2, 0x72, 0x64, 0x6f, 0x76, 0x61, 0x43, 0x61, 0x69, 0x65, 0xf1, 0x61,
0x47, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x75, 0x70, 0x70, 0x61, 0xc7, 0x69,
0x74, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x4d, 0x65, 0x73, 0x63, 0x69,
0x63, 0x6f, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x6f, 0x75, 0x6c, 0x6f, 0x53,
0x61, 0x6e, 0x20, 0x42, 0x65, 0x72, 0x74, 0x6f, 0x6d, 0xea, 0x53, 0x61,
0x6e, 0x20, 0x43, 0x72, 0x69, 0x73, 0x74, 0x6f, 0x66, 0x66, 0x61, 0x53,
0x61, 0x6e, 0x74, 0x61, 0x20, 0x4c, 0x75, 0xe7, 0x69, 0x61, 0x53, 0x61,
0x6e, 0x20, 0x56, 0x69, 0xe7, 0x65, 0x6e, 0x73, 0x6f, 0x53, 0x63, 0x69,
0x6e, 0x67, 0x61, 0x70, 0xf4, 0x43, 0x61, 0x6e, 0xe4, 0x69, 0x65, 0x43,
0x61, 0x70, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x47, 0x65,
0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x73, 0x75, 0x64,
0x42, 0x65, 0x6c, 0x67, 0x72, 0x61, 0x64, 0x64, 0x6f, 0x44, 0x6f, 0x62,
0x6c, 0x69, 0x6e, 0x47, 0x69, 0x62, 0x65, 0x72, 0x74, 0xe2, 0x49, 0x73,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x69, 0x73,
0x62, 0x6f, 0xf1, 0x61, 0x4c, 0x75, 0x62, 0x69, 0x61, 0xf1, 0x61, 0x4d,
0x6f, 0x6e, 0x65, 0x67, 0x6f, 0xd2, 0x73, 0x6c, 0x6f, 0x50, 0x61, 0x72,
0x69, 0x67, 0x67, 0x69, 0x52, 0x6f, 0x6d, 0x6d, 0x61, 0x54, 0x69, 0x72,
0x61, 0xf1, 0x61, 0x43, 0x6f, 0x6d, 0xf6, 0x72, 0x65, 0x4d, 0x61, 0x69,
0xf2, 0x74, 0x74, 0x61, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x69, 0x78,
0x69, 0x6a9, 0x6cc, 0x679, 0x627, 0x645, 0x627, 0x631, 0x633, 0x6cc, 0x6a9, 0x6cc,
0x645, 0x628, 0x631, 0x62c, 0x20, 0x633, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691,
0x6cc, 0x6a9, 0x631, 0x627, 0x633, 0x6cc, 0x633, 0x633, 0x6cc, 0x626, 0x6cc, 0x646,
0x6af, 0x644, 0x6cc, 0x633, 0x20, 0x633, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691,
0x6cc, 0x6af, 0x648, 0x633, 0x20, 0x633, 0x6cc, 0x20, 0x6a9, 0x6be, 0x627, 0x691,
0x6cc, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x633, 0x6cc, 0x646, 0x6af, 0x633, 0x6cc,
0x628, 0x644, 0x633, 0x6cc, 0x679, 0x6be, 0x645, 0x646, 0x688, 0x648, 0x6a9, 0x648,
0x644, 0x633, 0x6cc, 0x62a, 0x627, 0x645, 0x633, 0x6cc, 0x624, 0x627, 0x688, 0x6cc,
0x644, 0x627, 0x626, 0x688, 0x41, 0x62, 0x69, 0x6a, 0x61, 0x14b, 0x41, 0x6b,
0x72, 0x61, 0x61, 0x41, 0x256, 0x69, 0x73, 0x20, 0x41, 0x62, 0x65, 0x62,
0x61, 0x41, 0x6c, 0x6a, 0x25b, 0x25b, 0x72, 0x41, 0x73, 0x6d, 0x61, 0x72,
0x61, 0x61, 0x42, 0x61, 0x6d, 0x61, 0x6b, 0x6f, 0x6f, 0x42, 0x61, 0x14b,
0x67, 0x69, 0x69, 0x42, 0x69, 0x73, 0x6f, 0x6f, 0x42, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x69, 0x72, 0x42, 0x72, 0x61, 0x73, 0x61, 0x66, 0x69, 0x6c,
0x42, 0x75, 0x6a, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x61, 0x61, 0x4b, 0x61,
0x69, 0x72, 0x6f, 0x6f, 0x4b, 0x61, 0x73, 0x61, 0x62, 0x6c, 0x61, 0x14b,
0x6b, 0x61, 0x61, 0x53, 0x65, 0x79, 0x75, 0x74, 0x61, 0x4b, 0x6f, 0x6e,
0x61, 0x6b, 0x72, 0x69, 0x69, 0x189, 0x61, 0x6b, 0x61, 0x61, 0x72, 0x189,
0x61, 0x72, 0x25b, 0x73, 0x61, 0x6c, 0x61, 0x61, 0x6d, 0x4a, 0x69, 0x62,
0x75, 0x74, 0x69, 0x69, 0x189, 0x75, 0x77, 0x61, 0x6c, 0x61, 0x61, 0x190,
0x6c, 0x20, 0x41, 0x79, 0x75, 0x6e, 0x46, 0x72, 0x69, 0x69, 0x74, 0x61,
0x28a, 0x6e, 0x48, 0x61, 0x72, 0x61, 0x72, 0x65, 0x65, 0x59, 0x6f, 0x68,
0x61, 0x6e, 0x25b, 0x73, 0x62, 0x75, 0x75, 0x72, 0x4a, 0x75, 0x62, 0x61,
0x61, 0x4b, 0x61, 0x6d, 0x70, 0x61, 0x6c, 0x61, 0x61, 0x4b, 0x61, 0x72,
0x74, 0x75, 0x75, 0x6d, 0x4b, 0x69, 0x6e, 0x73, 0x68, 0x61, 0x73, 0x61,
0x61, 0x4c, 0x65, 0x67, 0x254, 0x73, 0x4c, 0x69, 0x62, 0x72, 0x1dd, 0x66,
0x69, 0x6c, 0x4c, 0x28a, 0x6d, 0x25b, 0x4c, 0x75, 0x77, 0x61, 0x6e, 0x256,
0x61, 0x61, 0x4c, 0x75, 0x73, 0x61, 0x6b, 0x61, 0x61, 0x4d, 0x61, 0x6c,
0x61, 0x62, 0x6f, 0x6f, 0x4d, 0x61, 0x70, 0x75, 0x74, 0x6f, 0x6f, 0x4d,
0x61, 0x73, 0x65, 0x72, 0x75, 0x75, 0x4d, 0x62, 0x61, 0x62, 0x61, 0x6e,
0x65, 0x65, 0x4d, 0x6f, 0x67, 0x61, 0x256, 0x69, 0x73, 0x68, 0x75, 0x75,
0x4d, 0x6f, 0x6e, 0x72, 0x6f, 0x66, 0x69, 0x79, 0x61, 0x61, 0x4e, 0x6a,
0x61, 0x6d, 0x65, 0x6e, 0x61, 0x61, 0x4e, 0x79, 0x25b, 0x6d, 0x25b, 0x4e,
0x75, 0x77, 0x61, 0x6b, 0x63, 0x254, 0x74, 0x57, 0x61, 0x67, 0x61, 0x256,
0x75, 0x67, 0x75, 0x50, 0x254, 0x72, 0x74, 0x6f, 0x20, 0x4e, 0x6f, 0x66,
0x6f, 0x6f, 0x53, 0x61, 0x6f, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x6b, 0x61,
0x6d, 0x70, 0xe1, 0x54, 0x69, 0x6e, 0x69, 0x69, 0x73, 0x57, 0x69, 0x6e,
0x256, 0x68, 0x75, 0x6b, 0x41, 0x256, 0x61, 0x6b, 0x41, 0x14b, 0x6b, 0x254,
0x72, 0x61, 0x61, 0x6a, 0x269, 0x41, 0x14b, 0x67, 0x75, 0x77, 0x69, 0x6c,
0x61, 0x61, 0x41, 0x6e, 0x74, 0x69, 0x67, 0x75, 0x77, 0x61, 0x61, 0x41,
0x72, 0x61, 0x67, 0x75, 0x77, 0x65, 0x65, 0x6e, 0x61, 0x42, 0x75, 0x77,
0x65, 0x6e, 0x254, 0x73, 0x20, 0x41, 0x69, 0x72, 0x25b, 0x73, 0x4b, 0x61,
0x74, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x61, 0x4b, 0x254, 0x72, 0x256,
0x6f, 0x62, 0x61, 0x61, 0x48, 0x75, 0x68, 0x75, 0x79, 0x69, 0x4c, 0x61,
0x20, 0x52, 0x69, 0x79, 0x254, 0x68, 0x61, 0x4d, 0x25b, 0x6e, 0x256, 0x254,
0x73, 0x61, 0x61, 0x52, 0x69, 0x79, 0x6f, 0x20, 0x47, 0x61, 0x6c, 0x65,
0x67, 0x254, 0x73, 0x53, 0x61, 0x6e, 0x20, 0x48, 0x75, 0x77, 0x61, 0x6e,
0x55, 0x73, 0x68, 0x75, 0x77, 0x65, 0x79, 0x61, 0x61, 0x41, 0x72, 0x75,
0x62, 0x61, 0x61, 0x41, 0x73, 0x75, 0x6e, 0x73, 0x69, 0x79, 0x254, 0x6e,
0x42, 0x61, 0x68, 0x69, 0x79, 0x61, 0x61, 0x42, 0x61, 0x68, 0x69, 0x69,
0x79, 0x61, 0x20, 0x256, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x256, 0x65, 0x72,
0x61, 0x61, 0x73, 0x269, 0x42, 0x61, 0x72, 0x62, 0x61, 0x256, 0x254, 0x254,
0x73, 0x269, 0x42, 0x65, 0x6c, 0x25b, 0x6d, 0x42, 0x6c, 0x61, 0x14b, 0x2d,
0x53, 0x61, 0x62, 0x6c, 0x254, 0x14b, 0x42, 0x6f, 0x77, 0x61, 0x20, 0x46,
0x69, 0x73, 0x74, 0x61, 0x61, 0x42, 0x254, 0x77, 0x61, 0x61, 0x73, 0x269,
0x4b, 0x61, 0x6d, 0x62, 0x72, 0x69, 0x69, 0x6a, 0x269, 0x20, 0x42, 0x61,
0x269, 0x43, 0x61, 0x6d, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x61, 0x6e, 0x256,
0x65, 0x65, 0x4b, 0x61, 0x14b, 0x6b, 0x75, 0x6e, 0x4b, 0x61, 0x79, 0x25b,
0x25b, 0x6e, 0x4b, 0x61, 0x79, 0x69, 0x6d, 0x61, 0x61, 0x6e, 0x53, 0x68,
0x69, 0x6b, 0x61, 0x67, 0x6f, 0x6f, 0x53, 0x68, 0x69, 0x77, 0x61, 0x77,
0x61, 0x61, 0x53, 0x69, 0x77, 0x75, 0x256, 0x61, 0x61, 0x256, 0x20, 0x48,
0x75, 0x77, 0x61, 0x72, 0x25b, 0x73, 0x4b, 0x254, 0x73, 0x74, 0x61, 0x20,
0x52, 0x69, 0x6b, 0x61, 0x61, 0x4b, 0x72, 0x25b, 0x73, 0x74, 0x254, 0x6e,
0x4b, 0x75, 0x79, 0x61, 0x62, 0x61, 0x61, 0x4b, 0x75, 0x72, 0x61, 0x73,
0x61, 0x61, 0x77, 0x75, 0x189, 0x61, 0x6e, 0x1dd, 0x6d, 0x61, 0x72, 0x6b,
0x269, 0x68, 0x61, 0x28a, 0x6e, 0x189, 0x254, 0x254, 0x73, 0x1dd, 0x6e, 0x189,
0x254, 0x254, 0x73, 0x1dd, 0x6e, 0x20, 0x4b, 0x72, 0x69, 0x69, 0x6b, 0x189,
0x25b, 0x6e, 0x66, 0x61, 0x189, 0x69, 0x74, 0x72, 0x254, 0x254, 0x269, 0x189,
0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0x61, 0x61, 0x190, 0x256, 0x6d, 0x254,
0x6e, 0x74, 0x254, 0x6e, 0x190, 0x269, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x65,
0x65, 0x190, 0x6c, 0x20, 0x53, 0x61, 0x6c, 0x66, 0x61, 0x256, 0x254, 0x254,
0x72, 0x46, 0x254, 0x254, 0x72, 0x20, 0x4e, 0x25b, 0x6c, 0x73, 0x254, 0x6e,
0x46, 0x254, 0x72, 0x74, 0x61, 0x6c, 0x25b, 0x73, 0x61, 0x61, 0x47, 0x6c,
0x61, 0x61, 0x73, 0x20, 0x42, 0x61, 0x269, 0x47, 0x75, 0x75, 0x73, 0x20,
0x42, 0x61, 0x269, 0x47, 0x72, 0x61, 0x6e, 0x256, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x47, 0x72, 0x65, 0x6e, 0x61, 0x61, 0x256, 0x61, 0x47, 0x75, 0x77,
0x61, 0x256, 0x65, 0x6c, 0x75, 0x75, 0x70, 0x75, 0x47, 0x75, 0x77, 0x61,
0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61, 0x61, 0x47, 0x75, 0x77, 0x61, 0x79,
0x61, 0x6b, 0x69, 0x6c, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x61, 0x48,
0x61, 0x6c, 0x69, 0x66, 0x61, 0x61, 0x73, 0x48, 0x61, 0x66, 0x61, 0x6e,
0x61, 0x61, 0x190, 0x72, 0x6d, 0x6f, 0x73, 0x69, 0x6c, 0x6f, 0x6f, 0x196,
0x6e, 0x256, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x69,
0x4e, 0x254, 0x6b, 0x2c, 0x20, 0x196, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x61,
0x6e, 0x61, 0x4d, 0x61, 0x72, 0x25b, 0x14b, 0x67, 0x6f, 0x2c, 0x20, 0x196,
0x6e, 0x256, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x50, 0x65, 0x74, 0x25b,
0x72, 0x73, 0x62, 0x75, 0x75, 0x72, 0x2c, 0x20, 0x196, 0x6e, 0x256, 0x69,
0x79, 0x61, 0x61, 0x6e, 0x61, 0x54, 0x25b, 0x6c, 0x20, 0x53, 0x69, 0x74,
0x69, 0x2c, 0x20, 0x196, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61,
0x46, 0x65, 0x66, 0x25b, 0x269, 0x2c, 0x20, 0x196, 0x6e, 0x256, 0x69, 0x79,
0x61, 0x61, 0x6e, 0x61, 0x46, 0x25b, 0x14b, 0x73, 0x25b, 0x6e, 0x2c, 0x20,
0x196, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x57, 0x69, 0x6e,
0x61, 0x6d, 0x61, 0x61, 0x6b, 0x2c, 0x20, 0x196, 0x6e, 0x256, 0x69, 0x79,
0x61, 0x61, 0x6e, 0x61, 0x49, 0x6e, 0x75, 0x66, 0x69, 0x6b, 0x49, 0x6b,
0x61, 0x6c, 0x75, 0x77, 0x69, 0x69, 0x74, 0x4a, 0x61, 0x6d, 0x61, 0x269,
0x6b, 0x61, 0x4a, 0x69, 0x6e, 0x6f, 0x6f, 0x4c, 0x75, 0x77, 0x69, 0x66,
0x69, 0x69, 0x6c, 0x4d, 0x254, 0x6e, 0x74, 0x69, 0x73, 0x68, 0x25b, 0x25b,
0x6c, 0x6f, 0x2c, 0x20, 0x4b, 0x25b, 0x6e, 0x74, 0x61, 0x6b, 0x69, 0x4b,
0x72, 0x61, 0x6c, 0x25b, 0x6e, 0x256, 0x61, 0x269, 0x6b, 0x4c, 0x69, 0x6d,
0x61, 0x61, 0x4c, 0x254, 0x73, 0x20, 0x41, 0x6e, 0x6a, 0x25b, 0x6c, 0x25b,
0x25b, 0x73, 0x4c, 0x6f, 0x77, 0x25b, 0x25b, 0x20, 0x50, 0x72, 0x25b, 0x14b,
0x73, 0x20, 0x4b, 0x77, 0x61, 0x74, 0x61, 0x4d, 0x61, 0x73, 0x25b, 0x269,
0x79, 0x6f, 0x6f, 0x4d, 0x61, 0x6e, 0x61, 0x28a, 0x73, 0x4d, 0x61, 0x72,
0x69, 0x67, 0x6f, 0x6f, 0x4d, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x69, 0x69,
0x6b, 0x69, 0x4d, 0x61, 0x74, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x6f, 0x73,
0x4d, 0x61, 0x73, 0x61, 0x74, 0x1dd, 0x6c, 0x61, 0x6e, 0x4d, 0x65, 0x72,
0x69, 0x69, 0x256, 0x61, 0x4d, 0x65, 0x74, 0x65, 0x6c, 0x61, 0x6b, 0x61,
0x74, 0x65, 0x6c, 0x61, 0x61, 0x4d, 0x25b, 0x73, 0x69, 0x6b, 0x6f, 0x20,
0x53, 0x69, 0x74, 0x69, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x254, 0x14b, 0x4d,
0x254, 0x14b, 0x74, 0x254, 0x6e, 0x4d, 0x254, 0x6e, 0x74, 0x25b, 0x72, 0x25b,
0x269, 0x4d, 0x254, 0x6e, 0x74, 0x65, 0x66, 0x69, 0x256, 0x65, 0x79, 0x6f,
0x6f, 0x4d, 0x254, 0x6e, 0x73, 0x25b, 0x72, 0x61, 0x61, 0x74, 0x269, 0x4e,
0x69, 0x75, 0x20, 0x59, 0x254, 0x72, 0x6b, 0x4e, 0x6f, 0x72, 0x254, 0x6e,
0x79, 0x61, 0x42, 0x69, 0x75, 0x6c, 0x61, 0x2c, 0x20, 0x4e, 0x254, 0x254,
0x72, 0x20, 0x189, 0x61, 0x6b, 0x6f, 0x6f, 0x74, 0x61, 0x53, 0x25b, 0x6e,
0x74, 0x61, 0x2c, 0x20, 0x4e, 0x254, 0x254, 0x72, 0x20, 0x189, 0x61, 0x6b,
0x6f, 0x6f, 0x74, 0x61, 0x4e, 0x69, 0x75, 0x20, 0x53, 0x61, 0x6c, 0x25b,
0x6d, 0x2c, 0x20, 0x4e, 0x254, 0x254, 0x72, 0x20, 0x189, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x50, 0x61, 0x6e, 0x61, 0x6d, 0x61, 0x61, 0x50, 0x61, 0x72,
0x61, 0x6d, 0x61, 0x72, 0x69, 0x62, 0x6f, 0x6f, 0x46, 0x69, 0x6e, 0x69,
0x69, 0x73, 0x50, 0x254, 0x254, 0x72, 0x20, 0x6f, 0x20, 0x50, 0x72, 0x25b,
0x14b, 0x73, 0x50, 0x254, 0x254, 0x72, 0x20, 0x254, 0x66, 0x20, 0x53, 0x70,
0x25b, 0x25b, 0x6e, 0x50, 0x254, 0x72, 0x74, 0x6f, 0x20, 0x46, 0x65, 0x6c,
0x6f, 0x6f, 0x50, 0x254, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x6b, 0x6f,
0x6f, 0x52, 0x61, 0x14b, 0x6b, 0x269, 0x6e, 0x20, 0x196, 0x6e, 0x6c, 0x25b,
0x25b, 0x74, 0x269, 0x52, 0x65, 0x67, 0x69, 0x69, 0x6e, 0x61, 0x52, 0x25b,
0x73, 0x6f, 0x6c, 0x75, 0x75, 0x74, 0x52, 0x69, 0x79, 0x6f, 0x20, 0x42,
0x72, 0x61, 0x14b, 0x6b, 0x6f, 0x6f, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x72,
0x25b, 0x6d, 0x53, 0x61, 0x6e, 0x74, 0x69, 0x79, 0x61, 0x67, 0x6f, 0x6f,
0x53, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x189, 0x6f, 0x6d, 0x269, 0x14b, 0x67,
0x6f, 0x53, 0x61, 0x6f, 0x20, 0x50, 0x61, 0x75, 0x6c, 0x6f, 0x6f, 0x49,
0x74, 0x6f, 0x6b, 0x254, 0x72, 0x74, 0x6f, 0x6f, 0x6d, 0x69, 0x69, 0x74,
0x69, 0x53, 0x69, 0x74, 0x6b, 0x61, 0x61, 0x53, 0x25b, 0x14b, 0x2d, 0x42,
0x61, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0x61, 0x79, 0x254, 0x53, 0x25b, 0x14b,
0x20, 0x4a, 0x254, 0x254, 0x6e, 0x73, 0x53, 0x25b, 0x14b, 0x20, 0x4b, 0x69,
0x74, 0x73, 0x53, 0x25b, 0x14b, 0x20, 0x4c, 0x75, 0x73, 0x69, 0x79, 0x61,
0x53, 0x25b, 0x14b, 0x20, 0x54, 0x6f, 0x6d, 0x61, 0x61, 0x73, 0x269, 0x53,
0x25b, 0x14b, 0x20, 0x46, 0x269, 0x6e, 0x73, 0x61, 0x14b, 0x53, 0x1dd, 0x77,
0x269, 0x66, 0x74, 0x269, 0x20, 0x4b, 0x1dd, 0x72, 0x25b, 0x25b, 0x6e, 0x74,
0x269, 0x54, 0x75, 0x75, 0x6c, 0x65, 0x54, 0x69, 0x68, 0x75, 0x77, 0x61,
0x6e, 0x61, 0x54, 0x6f, 0x72, 0x254, 0x6e, 0x74, 0x6f, 0x6f, 0x54, 0x254,
0x72, 0x74, 0x254, 0x6c, 0x61, 0x61, 0x46, 0x25b, 0x14b, 0x6b, 0x75, 0x75,
0x66, 0x61, 0x57, 0x61, 0x269, 0x74, 0x68, 0x254, 0x254, 0x73, 0x57, 0x69,
0x6e, 0x69, 0x70, 0x25b, 0x25b, 0x67, 0x4b, 0x65, 0x65, 0x73, 0x69, 0x189,
0x65, 0x66, 0x69, 0x73, 0x189, 0x69, 0x6d, 0x254, 0x6e, 0x20, 0x189, 0x69,
0x72, 0x66, 0x69, 0x6c, 0x4d, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x69, 0x4d,
0x254, 0x73, 0x254, 0x6e, 0x4d, 0x25b, 0x6b, 0x4d, 0x254, 0x72, 0x256, 0x6f,
0x6f, 0x50, 0x61, 0x6c, 0x6d, 0x25b, 0x25b, 0x72, 0x52, 0x6f, 0x74, 0x65,
0x72, 0x61, 0x61, 0x53, 0x69, 0x79, 0x6f, 0x77, 0x61, 0x61, 0x54, 0x72,
0x254, 0x6c, 0x46, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x4c, 0x254, 0x14b, 0x79,
0x69, 0x69, 0x72, 0x62, 0x69, 0x79, 0x25b, 0x6e, 0x41, 0x256, 0x25b, 0x6e,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x69, 0x41, 0x6d, 0x61, 0x61, 0x6e,
0x41, 0x6e, 0x61, 0x256, 0x69, 0x69, 0x72, 0x41, 0x6b, 0x74, 0x61, 0x28a,
0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x65, 0x41, 0x73, 0x68, 0x67, 0x61,
0x62, 0x61, 0x61, 0x74, 0x41, 0x74, 0x69, 0x72, 0x61, 0x28a, 0x42, 0x61,
0x67, 0x256, 0x61, 0x61, 0x256, 0x42, 0x61, 0x72, 0x25b, 0x25b, 0x6e, 0x42,
0x61, 0x14b, 0x6b, 0x254, 0x6b, 0x42, 0x61, 0x72, 0x6e, 0x61, 0x77, 0x75,
0x6c, 0x42, 0x65, 0x69, 0x72, 0x75, 0x75, 0x74, 0x42, 0x69, 0x73, 0x68,
0x6b, 0x25b, 0x6b, 0x42, 0x72, 0x75, 0x6e, 0x25b, 0x269, 0x53, 0x68, 0x69,
0x74, 0x61, 0x61, 0x4b, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x61,
0x6e, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x6f, 0x6f, 0x189, 0x61, 0x6d,
0x61, 0x73, 0x6b, 0x254, 0x254, 0x73, 0x269, 0x189, 0x69, 0x6c, 0x69, 0x69,
0x189, 0x75, 0x62, 0x61, 0x269, 0x189, 0x75, 0x73, 0x68, 0x61, 0x6e, 0x62,
0x65, 0x65, 0x46, 0x61, 0x6d, 0x61, 0x67, 0x75, 0x73, 0x74, 0x61, 0x61,
0x47, 0x61, 0x73, 0x61, 0x61, 0x48, 0x65, 0x62, 0x72, 0x254, 0x6e, 0x48,
0x6f, 0x20, 0x43, 0x69, 0x20, 0x4d, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x4d,
0x70, 0xe1, 0x48, 0x254, 0x14b, 0x20, 0x4b, 0x254, 0x14b, 0x48, 0x254, 0x66,
0x256, 0x1dd, 0x4a, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x61, 0x4a, 0x61, 0x79,
0x61, 0x70, 0x75, 0x72, 0x61, 0x61, 0x59, 0x65, 0x72, 0x75, 0x73, 0x61,
0x6c, 0x25b, 0x6d, 0x4b, 0x61, 0x6d, 0x73, 0x68, 0x61, 0x74, 0x6b, 0x61,
0x61, 0x4b, 0x61, 0x72, 0x61, 0x63, 0x269, 0x4b, 0x61, 0x74, 0x6d, 0x61,
0x6e, 0x256, 0x75, 0x75, 0x4b, 0x61, 0x6e, 0x256, 0x69, 0x67, 0x61, 0x61,
0x4b, 0x254, 0x6c, 0x6b, 0x61, 0x74, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x6b, 0x4b, 0x75, 0x77, 0x61, 0x6c, 0x61, 0x20,
0x4c, 0x75, 0x6d, 0x70, 0x75, 0x75, 0x72, 0x4b, 0x75, 0x63, 0x69, 0x14b,
0x4b, 0x6f, 0x77, 0x65, 0x65, 0x74, 0x69, 0x4d, 0x61, 0x6b, 0x61, 0x77,
0x6f, 0x6f, 0x4d, 0x61, 0x67, 0x61, 0x256, 0x61, 0x6e, 0x4d, 0x61, 0x6b,
0x61, 0x73, 0x61, 0x61, 0x72, 0x4d, 0x61, 0x6e, 0x69, 0x6c, 0x61, 0x61,
0x4d, 0x75, 0x73, 0x6b, 0x61, 0x61, 0x74, 0x4e, 0x6f, 0x66, 0x6f, 0x6b,
0x75, 0x73, 0x6e, 0x25b, 0x6b, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x6b, 0x186, 0x6d, 0x73, 0x1dd, 0x6b, 0x1dd, 0x4f, 0x72, 0x61,
0x61, 0x6c, 0x4e, 0x254, 0x6d, 0x20, 0x50, 0x25b, 0x6e, 0x50, 0x254, 0x6e,
0x74, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x6b, 0x50, 0x269, 0x79, 0x254, 0x14b,
0x79, 0x61, 0x14b, 0x4b, 0x254, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x269, 0x4b,
0x69, 0x73, 0x69, 0x6c, 0x254, 0x72, 0x256, 0x61, 0x61, 0x52, 0x69, 0x79,
0x61, 0x61, 0x256, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6e, 0x256,
0x53, 0x65, 0x77, 0x75, 0x6c, 0x53, 0x68, 0x61, 0x14b, 0x67, 0x61, 0x269,
0x53, 0x69, 0x14b, 0x67, 0x61, 0x70, 0x75, 0x75, 0x72, 0x53, 0x72, 0x25b,
0x256, 0x6e, 0x25b, 0x6b, 0x254, 0x6c, 0x69, 0x6d, 0x54, 0x61, 0x269, 0x70,
0x65, 0x69, 0x54, 0x61, 0x73, 0x68, 0x6b, 0x25b, 0x6e, 0x74, 0x54, 0x69,
0x62, 0x6c, 0x69, 0x73, 0x69, 0x69, 0x54, 0x65, 0x65, 0x72, 0x61, 0x61,
0x6e, 0x54, 0x69, 0x6d, 0x66, 0x75, 0x54, 0x6f, 0x6b, 0x69, 0x79, 0x6f,
0x6f, 0x54, 0x254, 0x6d, 0x73, 0x1dd, 0x6b, 0x1dd, 0x55, 0x6c, 0x61, 0x6e,
0x62, 0x61, 0x74, 0x254, 0x254, 0x72, 0x55, 0x72, 0x75, 0x6d, 0x6b, 0x69,
0x69, 0x55, 0x73, 0x74, 0x2d, 0x4e, 0x65, 0x72, 0x61, 0x61, 0x46, 0x69,
0x79, 0x65, 0x6e, 0x74, 0x69, 0x79, 0x61, 0x6e, 0x46, 0x6c, 0x61, 0x256,
0x69, 0x66, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x59, 0x61, 0x14b, 0x67, 0x254,
0x6e, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x75, 0x59, 0x65, 0x72, 0x65, 0x66, 0x61, 0x61, 0x6e, 0x41, 0x73, 0x254,
0x254, 0x72, 0x42, 0x25b, 0x72, 0x6d, 0x75, 0x256, 0x61, 0x61, 0x4b, 0x61,
0x6e, 0x61, 0x72, 0x69, 0x69, 0x4b, 0x61, 0x70, 0x66, 0x25b, 0x25b, 0x72,
0x46, 0x61, 0x72, 0x6f, 0x69, 0x4d, 0x61, 0x256, 0x65, 0x69, 0x72, 0x61,
0x61, 0x52, 0x25b, 0x269, 0x6b, 0x79, 0x61, 0x66, 0x69, 0x6b, 0x4a, 0x254,
0x72, 0x6a, 0x69, 0x79, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x73,
0x6f, 0x6e, 0x6f, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x6a, 0x61, 0x53,
0x25b, 0x14b, 0x20, 0x45, 0x6c, 0x65, 0x6e, 0x61, 0x61, 0x53, 0x74, 0x61,
0x6e, 0x6c, 0x25b, 0x269, 0x41, 0x256, 0x65, 0x6c, 0x25b, 0x25b, 0x256, 0x42,
0x72, 0x69, 0x73, 0x62, 0x25b, 0x6e, 0x42, 0x72, 0x6f, 0x6b, 0x1dd, 0x6e,
0x20, 0x48, 0x69, 0x6c, 0x189, 0x61, 0x72, 0x66, 0x69, 0x6e, 0x55, 0x6b,
0x6c, 0x61, 0x61, 0x48, 0x6f, 0x62, 0x61, 0x61, 0x4c, 0x69, 0x6e, 0x256,
0x65, 0x6d, 0x61, 0x6e, 0x4c, 0x254, 0x72, 0x256, 0x20, 0x48, 0x6f, 0x6f,
0x4d, 0x25b, 0x6c, 0x62, 0x254, 0x72, 0x6e, 0x50, 0x25b, 0x72, 0x74, 0x53,
0x69, 0x256, 0x6e, 0x65, 0x65, 0x41, 0x6d, 0x73, 0x74, 0x25b, 0x72, 0x256,
0x61, 0x6d, 0x41, 0x6e, 0x256, 0x254, 0x72, 0x61, 0x61, 0x41, 0x73, 0x74,
0x72, 0x61, 0x6b, 0x61, 0x61, 0x6e, 0x41, 0x74, 0x25b, 0x6e, 0x42, 0x25b,
0x6c, 0x67, 0x72, 0x61, 0x61, 0x256, 0x42, 0x25b, 0x72, 0x6c, 0x25b, 0x14b,
0x42, 0x72, 0x61, 0x74, 0x69, 0x73, 0x6c, 0x61, 0x66, 0x61, 0x42, 0x72,
0x69, 0x73, 0x25b, 0x6c, 0x42, 0x75, 0x6b, 0x61, 0x72, 0x25b, 0x73, 0x42,
0x75, 0x256, 0x61, 0x70, 0x25b, 0x73, 0x42, 0x75, 0x73, 0x69, 0x14b, 0x67,
0x25b, 0x6e, 0x53, 0x68, 0x69, 0x73, 0x69, 0x6e, 0x6f, 0x6f, 0x4b, 0x6f,
0x70, 0x65, 0x6e, 0x61, 0x61, 0x67, 0x189, 0x254, 0x62, 0x6c, 0x25b, 0x14b,
0x47, 0x1dd, 0x72, 0x1dd, 0x6e, 0x73, 0x25b, 0x269, 0x48, 0x25b, 0x6c, 0x73,
0x69, 0x14b, 0x6b, 0x69, 0x69, 0x4d, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x41,
0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x49, 0x73, 0x74,
0x61, 0x6d, 0x62, 0x75, 0x75, 0x6c, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69,
0x14b, 0x67, 0x72, 0x61, 0x61, 0x256, 0x4b, 0x69, 0x72, 0x254, 0x254, 0x66,
0x4b, 0x69, 0x79, 0x25b, 0x66, 0x4c, 0x69, 0x73, 0x62, 0x254, 0x6e, 0x59,
0x75, 0x62, 0x1dd, 0x6c, 0x79, 0x61, 0x6e, 0x61, 0x4c, 0x254, 0x6e, 0x256,
0x254, 0x6e, 0x4c, 0x75, 0x73, 0x25b, 0x6d, 0x62, 0x75, 0x75, 0x72, 0x4d,
0x61, 0x256, 0x72, 0x69, 0x69, 0x256, 0x4d, 0x61, 0x72, 0x69, 0x79, 0x65,
0x68, 0x61, 0x6d, 0x4d, 0x269, 0x6e, 0x73, 0x4d, 0x6f, 0x6e, 0x61, 0x6b,
0x6f, 0x6f, 0x4d, 0x6f, 0x73, 0x6b, 0x75, 0x75, 0x186, 0x73, 0x6c, 0x6f,
0x6f, 0x50, 0x6f, 0x256, 0x67, 0x6f, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x52,
0x69, 0x67, 0x61, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x61, 0x53,
0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6e, 0x6f, 0x6f, 0x53, 0x61,
0x72, 0x61, 0x79, 0x65, 0x66, 0x6f, 0x6f, 0x53, 0x61, 0x72, 0x61, 0x74,
0x254, 0x254, 0x66, 0x53, 0x6b, 0x254, 0x70, 0x79, 0x25b, 0x53, 0x74, 0x254,
0x6b, 0x68, 0x254, 0x6c, 0x6d, 0x54, 0x69, 0x72, 0x61, 0x6e, 0x61, 0x61,
0x55, 0x6c, 0x79, 0x61, 0x6e, 0x254, 0x254, 0x66, 0x46, 0x61, 0x256, 0x75,
0x73, 0x46, 0x61, 0x74, 0x69, 0x6b, 0x61, 0x14b, 0x46, 0x69, 0x79, 0x25b,
0x6e, 0x61, 0x46, 0x69, 0x6c, 0x6e, 0x69, 0x79, 0x75, 0x73, 0x46, 0x254,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x61, 0x256, 0x57, 0x61, 0x72, 0x73,
0x61, 0x28a, 0x53, 0x61, 0x67, 0x72, 0x25b, 0x62, 0x53, 0x75, 0x72, 0x69,
0x69, 0x6b, 0x41, 0x6e, 0x74, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x72, 0x69,
0x66, 0x6f, 0x6f, 0x53, 0x68, 0x61, 0x67, 0x254, 0x73, 0x4e, 0x6f, 0x77,
0x25b, 0x6c, 0x4b, 0x6f, 0x6b, 0x6f, 0x6f, 0x73, 0x4b, 0x6f, 0x6d, 0x6f,
0x72, 0x6f, 0x6f, 0x4b, 0x25b, 0x72, 0x67, 0x65, 0x6c, 0x25b, 0x25b, 0x6e,
0x4d, 0x61, 0x68, 0x65, 0x65, 0x4d, 0x61, 0x6c, 0x256, 0x69, 0x69, 0x66,
0x75, 0x49, 0x6d, 0x6f, 0x72, 0x69, 0x73, 0x4d, 0x61, 0x79, 0x254, 0x254,
0x74, 0x269, 0x52, 0x65, 0x65, 0x6e, 0x69, 0x79, 0x254, 0x14b, 0x186, 0x254,
0x6b, 0x6c, 0x61, 0x6e, 0x256, 0x42, 0x75, 0x67, 0x25b, 0x25b, 0x14b, 0x66,
0x69, 0x6c, 0x53, 0x68, 0x61, 0x74, 0x61, 0x6d, 0x43, 0x75, 0x75, 0x6b,
0x45, 0x66, 0x61, 0x74, 0x65, 0x65, 0x46, 0x61, 0x6b, 0x61, 0x77, 0x6f,
0x66, 0x6f, 0x6f, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x254, 0x73,
0x47, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x65, 0x65, 0x47, 0x75, 0x77, 0x61,
0x256, 0x61, 0x6c, 0x6b, 0x61, 0x6e, 0x61, 0x6c, 0x4b, 0x61, 0x6e, 0x74,
0x254, 0x6e, 0x4b, 0x69, 0x72, 0x69, 0x6d, 0x61, 0x74, 0x69, 0x69, 0x4b,
0x254, 0x73, 0x72, 0x25b, 0x25b, 0x4b, 0x77, 0x61, 0x6a, 0x61, 0x6c, 0x25b,
0x25b, 0x6e, 0x4d, 0x61, 0x6a, 0x75, 0x72, 0x6f, 0x6f, 0x4d, 0x69, 0x256,
0x77, 0x65, 0x65, 0x4e, 0x61, 0x77, 0x75, 0x72, 0x75, 0x4e, 0x69, 0x77,
0x75, 0x79, 0x65, 0x4e, 0x254, 0x72, 0x66, 0x6f, 0x6f, 0x6b, 0x50, 0x61,
0x67, 0x6f, 0x20, 0x50, 0x61, 0x67, 0x6f, 0x6f, 0x50, 0x61, 0x6c, 0x61,
0x77, 0x6f, 0x6f, 0x50, 0x269, 0x74, 0x6b, 0x25b, 0x25b, 0x6e, 0x50, 0x6f,
0x6f, 0x6e, 0x70, 0x65, 0x65, 0x50, 0x254, 0x254, 0x72, 0x20, 0x4d, 0x254,
0x72, 0x25b, 0x73, 0x62, 0x69, 0x69, 0x52, 0x61, 0x72, 0x6f, 0x74, 0x254,
0x14b, 0x67, 0x61, 0x53, 0x61, 0x269, 0x70, 0x61, 0x6e, 0x54, 0x61, 0x72,
0x61, 0x77, 0x61, 0x61, 0x54, 0x254, 0x14b, 0x67, 0x61, 0x74, 0x61, 0x70,
0x75, 0x75, 0x57, 0x65, 0x65, 0x6b, 0x41, 0x6c, 0x6a, 0x65, 0x72, 0x69,
0x41, 0x7a, 0x6d, 0x61, 0x72, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x68, 0xec,
0x42, 0x69, 0x73, 0xe0, 0x6f, 0x42, 0x75, 0x6a, 0x75, 0x6e, 0x62, 0x75,
0x72, 0x61, 0x43, 0x61, 0x7a, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x63, 0x61,
0x43, 0xe8, 0x75, 0x74, 0x61, 0x4b, 0x61, 0x6e, 0x70, 0x61, 0x6c, 0x61,
0x4c, 0x75, 0x62, 0x75, 0x6e, 0x62, 0x61, 0x73, 0x68, 0x69, 0x4d, 0x61,
0x7a, 0x65, 0x72, 0x75, 0x4e, 0x62, 0x61, 0x62, 0x61, 0x6e, 0x65, 0x4d,
0x6f, 0x67, 0x61, 0x64, 0x69, 0x73, 0x6f, 0x55, 0x61, 0x67, 0x61, 0x64,
0x75, 0x67, 0xf9, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x53, 0x2e, 0x20,
0x54, 0x6f, 0x6d, 0x61, 0x7a, 0x6f, 0x54, 0xf9, 0x6e, 0x65, 0x7a, 0x69,
0x43, 0xf2, 0x72, 0x64, 0x6f, 0x62, 0x61, 0x52, 0x69, 0x6f, 0x20, 0x47,
0xe0, 0x6c, 0x65, 0x67, 0x6f, 0x73, 0x42, 0x61, 0x69, 0x61, 0x20, 0x64,
0x65, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x61, 0x73, 0x43, 0x61,
0x6e, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x61, 0x6e, 0x64, 0x65, 0x43, 0x75,
0x69, 0x61, 0x62, 0xe0, 0x44, 0xe8, 0x6e, 0x76, 0x65, 0x72, 0x44, 0x6f,
0x6d, 0xe9, 0x6e, 0x65, 0x67, 0x61, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0xe0, 0x70, 0x6f, 0x6c, 0x69, 0x73, 0x4b, 0x6e, 0x6f, 0x78, 0x20, 0x28,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x29, 0x4d, 0x61, 0x72, 0x65,
0x6e, 0x67, 0x6f, 0x20, 0x28, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61,
0x29, 0x50, 0x65, 0x74, 0x65, 0x72, 0x73, 0x62, 0x75, 0x72, 0x67, 0x20,
0x28, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x29, 0x54, 0x65, 0x6c,
0x6c, 0x20, 0x43, 0x69, 0x74, 0x79, 0x20, 0x28, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x6e, 0x61, 0x29, 0x56, 0x65, 0x76, 0x61, 0x79, 0x20, 0x28, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x29, 0x56, 0x69, 0x6e, 0x63, 0x65,
0x6e, 0x6e, 0x65, 0x73, 0x20, 0x28, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e,
0x61, 0x29, 0x57, 0x69, 0x6e, 0x61, 0x6d, 0x61, 0x63, 0x20, 0x28, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x29, 0x4a, 0x61, 0x6d, 0xe0, 0x65,
0x67, 0x61, 0x4d, 0x6f, 0x6e, 0x74, 0x69, 0x63, 0x65, 0x6c, 0x6c, 0x6f,
0x20, 0x28, 0x4b, 0x65, 0x6e, 0x74, 0x75, 0x63, 0x6b, 0x79, 0x29, 0x4c,
0x6f, 0x73, 0x20, 0xc0, 0x6e, 0x67, 0x65, 0x6c, 0x65, 0x73, 0x4d, 0x61,
0x72, 0x74, 0x69, 0x6e, 0x69, 0x67, 0x61, 0x53, 0x69, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4d, 0xe8, 0x73, 0x65, 0x67, 0x6f, 0x4d,
0x69, 0x63, 0x68, 0x65, 0x6c, 0x6f, 0x6e, 0x42, 0x65, 0x75, 0x6c, 0x61,
0x68, 0x20, 0x28, 0x4e, 0x6f, 0x72, 0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f,
0x74, 0x61, 0x29, 0x43, 0x65, 0x6e, 0x74, 0x65, 0x72, 0x20, 0x28, 0x4e,
0x6f, 0x72, 0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x29, 0x4e,
0x65, 0x77, 0x20, 0x53, 0x61, 0x6c, 0x65, 0x6d, 0x20, 0x28, 0x4e, 0x6f,
0x72, 0x64, 0x20, 0x44, 0x61, 0x6b, 0x6f, 0x74, 0x61, 0x29, 0x50, 0x6f,
0x72, 0x74, 0x6f, 0x20, 0x50, 0x72, 0xec, 0x6e, 0x73, 0x69, 0x70, 0x65,
0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x70, 0x61,
0x67, 0x6e, 0x61, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x6f, 0x6c, 0x6f, 0x53,
0x2e, 0x20, 0x42, 0x61, 0x72, 0x74, 0x6f, 0x6c, 0x6f, 0x6d, 0xe8, 0x6f,
0x53, 0x2e, 0x20, 0x4a, 0x6f, 0x61, 0x6e, 0x69, 0x53, 0x2e, 0x20, 0x43,
0x72, 0x69, 0x73, 0x74, 0x6f, 0x66, 0x65, 0x72, 0x53, 0x2e, 0x20, 0x4c,
0x75, 0x73, 0xec, 0x61, 0x53, 0x2e, 0x20, 0x56, 0x69, 0x6e, 0x63, 0x65,
0x6e, 0x73, 0x6f, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4d, 0x61, 0x63,
0x71, 0x75, 0x61, 0x72, 0x69, 0x65, 0x41, 0x6b, 0x74, 0xe0, 0x75, 0x41,
0x74, 0x79, 0x72, 0xe0, 0x75, 0x43, 0x6f, 0x6c, 0x6f, 0x6e, 0x62, 0x6f,
0x44, 0x75, 0x62, 0xe0, 0x69, 0x4b, 0x61, 0x74, 0x6d, 0x61, 0x6e, 0x64,
0xf9, 0x4b, 0x75, 0x61, 0x6c, 0x61, 0x20, 0x4c, 0x75, 0x6e, 0x70, 0x75,
0x72, 0x4d, 0x61, 0x63, 0xe0, 0x6f, 0x4b, 0x6f, 0x73, 0x74, 0x61, 0x6e,
0xe0, 0x69, 0x53, 0x68, 0x61, 0x6e, 0x67, 0x68, 0xe0, 0x69, 0x54, 0x69,
0x6e, 0x70, 0x75, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0xe0, 0x74, 0x6f,
0x72, 0x4a, 0xe8, 0x72, 0x65, 0x76, 0x61, 0x6e, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x43, 0x61, 0x6e, 0x61, 0x72, 0x69, 0x65, 0x43, 0x61, 0x6f, 0x20,
0x56, 0x65, 0x72, 0x64, 0x6f, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46,
0xe0, 0x72, 0x6f, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4d, 0x61,
0x64, 0xe8, 0x69, 0x72, 0x61, 0x52, 0x65, 0x6b, 0x69, 0x61, 0x76, 0x69,
0x6b, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x2019,
0x6c, 0x20, 0x73, 0x75, 0x64, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x53,
0x2e, 0x20, 0x45, 0x6c, 0x65, 0x6e, 0x61, 0x41, 0x64, 0x65, 0x6c, 0xe0,
0x69, 0x64, 0x65, 0x42, 0x72, 0x69, 0x7a, 0x62, 0x61, 0x6e, 0x65, 0xcc,
0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0xc0, 0x6d, 0x73, 0x74, 0x65, 0x72, 0x64, 0x61, 0x6d, 0x42,
0x72, 0x61, 0x74, 0x69, 0x7a, 0x6c, 0x61, 0x76, 0x61, 0x4a, 0x69, 0x62,
0x69, 0x6c, 0x74, 0x65, 0x72, 0x61, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x6e, 0x4c, 0x75, 0x62, 0x6c, 0x69, 0x61,
0x6e, 0x61, 0x4c, 0x75, 0x73, 0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f,
0x4f, 0x7a, 0x6c, 0x6f, 0x50, 0x61, 0x72, 0x69, 0x6a, 0x69, 0x53, 0x69,
0x6e, 0x66, 0x65, 0x72, 0xf2, 0x70, 0x6f, 0x6c, 0x69, 0x53, 0x74, 0x6f,
0x63, 0x6f, 0x6c, 0x6d, 0x61, 0x56, 0x61, 0x74, 0x65, 0x67, 0x61, 0x6e,
0x43, 0x69, 0x61, 0x67, 0x6f, 0x73, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x61, 0x64, 0x61, 0x6c, 0x65, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x43, 0x6f, 0x6d, 0x6f, 0x72, 0x65, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x6f, 0xcc,
0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x69, 0x6f, 0x74, 0x65, 0xcc,
0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x42, 0x6f, 0x75, 0x67, 0x61, 0x69,
0x6e, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20,
0x43, 0x69, 0x61, 0x74, 0x65, 0x6d, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20,
0x43, 0x68, 0x75, 0x75, 0x6b, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0xcc, 0x7a, 0x6f, 0x6c,
0x61, 0x20, 0x45, 0x66, 0x61, 0x74, 0x65, 0x41, 0x74, 0x6f, 0x6c, 0x6f,
0x20, 0x46, 0x61, 0x6b, 0x61, 0x6f, 0x66, 0x6f, 0x41, 0x74, 0x6f, 0x6c,
0x6f, 0x20, 0x46, 0x75, 0x6e, 0x61, 0x66, 0x75, 0x74, 0x69, 0x47, 0x61,
0x6c, 0xe0, 0x70, 0x61, 0x67, 0x6f, 0x73, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0xcc, 0x7a, 0x6f, 0x6c,
0x61, 0x20, 0x47, 0x75, 0x61, 0x64, 0x61, 0x6c, 0x63, 0x61, 0x6e, 0x61,
0x6c, 0x41, 0x74, 0x6f, 0x6c, 0x6f, 0x20, 0x43, 0x61, 0x6e, 0x74, 0x6f,
0x6e, 0x41, 0x74, 0x6f, 0x6c, 0x6f, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x74,
0x69, 0x6d, 0x61, 0x74, 0x69, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x41, 0x74, 0x6f, 0x6c, 0x6f, 0x20, 0x4b,
0x77, 0x61, 0x6a, 0x61, 0x6c, 0x65, 0x69, 0x6e, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x7a, 0x69, 0x41, 0x74,
0x6f, 0x6c, 0x6f, 0x20, 0x4d, 0x69, 0x64, 0x77, 0x61, 0x79, 0xcc, 0x7a,
0x6f, 0x6c, 0x61, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0xcc, 0x7a, 0x6f,
0x6c, 0x61, 0x20, 0x4e, 0x69, 0x75, 0x65, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x50, 0x61, 0x6c, 0xe0,
0x75, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x50, 0x6f, 0x6e,
0x70, 0xe8, 0x69, 0x50, 0x6f, 0x72, 0x74, 0x6f, 0x20, 0x4d, 0x6f, 0x72,
0x65, 0x73, 0x62, 0x79, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x52, 0x61,
0x72, 0x6f, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x53, 0x61, 0x69, 0x70, 0xe0, 0x6e, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x54, 0x61, 0x69, 0x74, 0x69, 0x41, 0x74, 0x6f, 0x6c, 0x6c, 0x6f,
0x20, 0x54, 0x61, 0x72, 0x61, 0x77, 0x61, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0x61, 0x70, 0x75, 0x41, 0x74,
0x6f, 0x6c, 0x6f, 0x20, 0x57, 0x61, 0x6b, 0x65, 0xcc, 0x7a, 0x6f, 0x6c,
0x61, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x61, 0x62, 0x69, 0x64,
0x6a, 0x61, 0x6e, 0x65, 0x63, 0x72, 0x61, 0x61, 0x1e0d, 0x69, 0x73, 0x20,
0x61, 0x62, 0x61, 0x62, 0x61, 0x61, 0x6c, 0x6a, 0x12b, 0x79, 0x72, 0x73,
0x61, 0x73, 0x6d, 0x61, 0x72, 0x61, 0x62, 0x61, 0x6d, 0x61, 0x6b, 0x6f,
0x62, 0x61, 0x6e, 0x67, 0x75, 0x69, 0x62, 0x61, 0x6e, 0x6a, 0x75, 0x6c,
0x62, 0x69, 0x73, 0x73, 0x61, 0x75, 0x62, 0x6c, 0x61, 0x6e, 0x1e6d, 0x61,
0x65, 0x72, 0x62, 0x72, 0x61, 0x6a, 0x61, 0x76, 0x69, 0x6c, 0x6c, 0x62,
0x75, 0x6a, 0x75, 0x6d, 0x62, 0x75, 0x72, 0x61, 0x6b, 0x61, 0x69, 0x72,
0x6f, 0x6b, 0x61, 0x73, 0x61, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0x61, 0x73,
0x65, 0x75, 0x1e6d, 0x61, 0x6b, 0x6f, 0x6e, 0x61, 0x6b, 0x72, 0x12b, 0x1e0d,
0x6b, 0x61, 0x72, 0x64, 0x61, 0x72, 0x20, 0x65, 0x73, 0x20, 0x73, 0x61,
0x6c, 0x61, 0x61, 0x6d, 0x64, 0x75, 0x61, 0x6c, 0x61, 0x65, 0x6c, 0x20,
0x61, 0x61, 0x69, 0x79, 0x75, 0x6e, 0x70, 0x72, 0x12b, 0x1e6d, 0x61, 0x75,
0x6e, 0x67, 0x61, 0x62, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x72, 0x61, 0x72,
0x65, 0x6a, 0x6f, 0x68, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x62, 0x75, 0x72,
0x67, 0x6a, 0x75, 0x62, 0x61, 0x6b, 0x6d, 0x70, 0x61, 0x6c, 0x61, 0x6b,
0x61, 0x72, 0x1e6d, 0x6f, 0x75, 0x6d, 0x6b, 0x69, 0x67, 0x61, 0x6c, 0x69,
0x6b, 0x69, 0x6e, 0x73, 0x61, 0x73, 0x61, 0x6c, 0x61, 0x67, 0x6f, 0x73,
0x6c, 0x69, 0x62, 0x72, 0x65, 0x76, 0x69, 0x6c, 0x6c, 0x6c, 0x75, 0x61,
0x6e, 0x1e0d, 0x61, 0x6c, 0x75, 0x62, 0x75, 0x6d, 0x62, 0x61, 0x73, 0x69,
0x6c, 0x75, 0x73, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6c, 0x61, 0x62, 0x6f,
0x6d, 0x61, 0x70, 0x75, 0x1e6d, 0x75, 0x6d, 0x61, 0x73, 0x65, 0x72, 0x75,
0x6d, 0x62, 0x61, 0x62, 0x61, 0x6e, 0x65, 0x6d, 0x6f, 0x67, 0x61, 0x1e0d,
0x69, 0x73, 0x75, 0x6d, 0x6f, 0x6e, 0x72, 0x6f, 0x76, 0x69, 0x61, 0x6e,
0x61, 0x69, 0x72, 0x6f, 0x62, 0x69, 0x6e, 0x6a, 0x61, 0x6d, 0x65, 0x6e,
0x61, 0x6e, 0x69, 0x79, 0x61, 0x6d, 0x65, 0x6e, 0x75, 0x65, 0x6b, 0x73,
0x61, 0x76, 0x61, 0x67, 0x64, 0x75, 0x67, 0x75, 0x70, 0x6f, 0x72, 0x1e6d,
0x6f, 0x2d, 0x1e47, 0x6f, 0x76, 0x6f, 0x73, 0x61, 0x6f, 0x20, 0x1e6d, 0x6f,
0x6d, 0x74, 0x72, 0x69, 0x70, 0x6f, 0x6c, 0x69, 0x1e6d, 0x75, 0x6e, 0x69,
0x73, 0x76, 0x69, 0x6e, 0x64, 0x68, 0x75, 0x6b, 0x61, 0x1e0d, 0x61, 0x6b,
0x61, 0x6e, 0x6b, 0x6f, 0x72, 0x61, 0x6a, 0x65, 0x6e, 0x67, 0x75, 0x69,
0x6c, 0x6c, 0x61, 0x65, 0x6e, 0x1e6d, 0x69, 0x67, 0x75, 0x61, 0x61, 0x72,
0x61, 0x67, 0x76, 0x61, 0x69, 0x6e, 0x61, 0x62, 0x75, 0x65, 0x6e, 0x6f,
0x75, 0x73, 0x20, 0x61, 0x69, 0x72, 0x65, 0x73, 0x6b, 0x61, 0x74, 0x61,
0x6d, 0x61, 0x72, 0x6b, 0x61, 0x6b, 0x6f, 0x72, 0x1e0d, 0x61, 0x62, 0x61,
0x68, 0x75, 0x68, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x72, 0x69, 0x6f, 0x6a,
0x61, 0x6d, 0x65, 0x6e, 0x1e0d, 0x6f, 0x6a, 0x61, 0x72, 0x69, 0x79, 0x6f,
0x20, 0x67, 0x61, 0x6c, 0x6c, 0x65, 0x67, 0x6f, 0x73, 0x73, 0x61, 0x6c,
0x1e6d, 0x61, 0x73, 0x61, 0x6e, 0x20, 0x68, 0x75, 0x61, 0x6e, 0x73, 0x61,
0x6e, 0x20, 0x6c, 0x75, 0x69, 0x73, 0x1e6d, 0x75, 0x6b, 0x16b, 0x6d, 0x6e,
0x75, 0x73, 0x76, 0x61, 0x69, 0x79, 0x61, 0x61, 0x72, 0x75, 0x62, 0x61,
0x65, 0x73, 0x75, 0x6e, 0x73, 0x69, 0x6f, 0x6e, 0x65, 0x1e6d, 0x69, 0x6b,
0x6f, 0x6b, 0x65, 0x6e, 0x62, 0x61, 0x68, 0x69, 0x61, 0x62, 0x61, 0x69,
0x61, 0x20, 0x62, 0x61, 0x6e, 0x1e0d, 0x65, 0x72, 0x61, 0x73, 0x62, 0x61,
0x72, 0x62, 0x61, 0x1e0d, 0x6f, 0x73, 0x62, 0x65, 0x6c, 0x65, 0x6d, 0x62,
0x65, 0x6c, 0x69, 0x6a, 0x62, 0x6c, 0x61, 0x6e, 0x6b, 0x2d, 0x73, 0x61,
0x62, 0x6c, 0x6f, 0x6e, 0x62, 0x6f, 0x61, 0x20, 0x76, 0x69, 0x73, 0x74,
0x61, 0x62, 0x6f, 0x67, 0x6f, 0x1e6d, 0x61, 0x62, 0x6f, 0x69, 0x73, 0x12b,
0x6b, 0x65, 0x6d, 0x62, 0x72, 0x69, 0x6a, 0x20, 0x62, 0x65, 0x6b, 0x61,
0x6d, 0x70, 0x6f, 0x20, 0x67, 0x72, 0x61, 0x6e, 0x1e0d, 0x65, 0x6b, 0x61,
0x6e, 0x6b, 0x75, 0x6e, 0x6b, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x73, 0x6b,
0x65, 0x79, 0x65, 0x6e, 0x6b, 0x65, 0x69, 0x6d, 0x61, 0x6e, 0x63, 0x69,
0x6b, 0x61, 0x67, 0x6f, 0x63, 0x68, 0x69, 0x68, 0x75, 0x61, 0x68, 0x75,
0x61, 0x6b, 0x6f, 0x73, 0x1e6d, 0x61, 0x20, 0x72, 0x69, 0x6b, 0x61, 0x6b,
0x72, 0x65, 0x73, 0x1e6d, 0x6f, 0x6e, 0x6b, 0x75, 0x61, 0x61, 0x62, 0x61,
0x6b, 0x79, 0x75, 0x72, 0x61, 0x73, 0x6f, 0x1e0d, 0x61, 0x6e, 0x6d, 0x61,
0x72, 0x6b, 0x73, 0x61, 0x76, 0x6e, 0x1e0d, 0x61, 0x6f, 0x73, 0x6e, 0x1e0d,
0x61, 0x6f, 0x73, 0x6f, 0x6e, 0x20, 0x6b, 0x72, 0x12b, 0x6b, 0x1e0d, 0x65,
0x6e, 0x76, 0x65, 0x72, 0x1e0d, 0x65, 0x1e6d, 0x72, 0x6f, 0x69, 0x1e6d, 0x1e0d,
0x6f, 0x6d, 0x69, 0x6e, 0x69, 0x6b, 0x61, 0x65, 0x1e0d, 0x6d, 0x6f, 0x6e,
0x1e6d, 0x6f, 0x6e, 0x65, 0x69, 0x72, 0x75, 0x6e, 0x65, 0x70, 0x65, 0x65,
0x6c, 0x20, 0x73, 0x61, 0x6c, 0x76, 0x61, 0x1e0d, 0x6f, 0x72, 0x70, 0x68,
0x6f, 0x72, 0x74, 0x20, 0x6e, 0x65, 0x6c, 0x73, 0x6e, 0x70, 0x6f, 0x72,
0x74, 0x61, 0x6c, 0x65, 0x6a, 0x61, 0x67, 0x6c, 0x61, 0x73, 0x20, 0x62,
0x61, 0x79, 0x67, 0x6f, 0x6f, 0x73, 0x20, 0x62, 0x65, 0x67, 0x72, 0x61,
0x6e, 0x1e0d, 0x20, 0x1e6d, 0x75, 0x72, 0x6b, 0x67, 0x72, 0x65, 0x6e, 0x61,
0x1e0d, 0x61, 0x67, 0x75, 0x61, 0x1e0d, 0x65, 0x6c, 0x6f, 0x75, 0x70, 0x65,
0x67, 0x75, 0x61, 0x74, 0x65, 0x6d, 0x61, 0x6c, 0x61, 0x67, 0x75, 0x61,
0x6a, 0x61, 0x6b, 0x69, 0x6c, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x68,
0x65, 0x6c, 0x69, 0x70, 0x61, 0x6b, 0x73, 0x68, 0x61, 0x76, 0x61, 0x6e,
0x61, 0x69, 0x6e, 0x1e0d, 0x69, 0x61, 0x6e, 0x61, 0x70, 0x6f, 0x6c, 0x69,
0x73, 0x6e, 0x6f, 0x6b, 0x73, 0x2c, 0x20, 0x69, 0x6e, 0x1e0d, 0x69, 0x79,
0x61, 0x6e, 0x61, 0x6d, 0x61, 0x72, 0x65, 0x6e, 0x67, 0x6f, 0x2c, 0x20,
0x69, 0x6e, 0x1e0d, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x70, 0x69, 0x1e6d, 0x74,
0x72, 0x73, 0x62, 0x65, 0x72, 0x67, 0x2c, 0x20, 0x69, 0x6e, 0x1e0d, 0x69,
0x79, 0x61, 0x6e, 0x61, 0x1e6d, 0x65, 0x6c, 0x6c, 0x20, 0x73, 0x69, 0x74,
0x79, 0x2c, 0x20, 0x69, 0x6e, 0x1e0d, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x76,
0x69, 0x76, 0x69, 0x2c, 0x20, 0x69, 0x6e, 0x1e0d, 0x69, 0x79, 0x61, 0x6e,
0x61, 0x76, 0x69, 0x6e, 0x63, 0x65, 0x6e, 0x73, 0x2c, 0x20, 0x69, 0x6e,
0x1e0d, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x76, 0x69, 0x6e, 0x61, 0x6d, 0x61,
0x6b, 0x2c, 0x20, 0x69, 0x6e, 0x1e0d, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x69,
0x6e, 0x16b, 0x76, 0x69, 0x6b, 0x69, 0x6b, 0x61, 0x6c, 0x75, 0x69, 0x1e6d,
0x6a, 0x61, 0x6d, 0x61, 0x69, 0x6b, 0x61, 0x6a, 0x75, 0x6e, 0x6f, 0x76,
0x6c, 0x6f, 0x75, 0x69, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x6d, 0x6f, 0x6e,
0x1e6d, 0x69, 0x73, 0x65, 0x6c, 0x6c, 0x6f, 0x2c, 0x20, 0x6b, 0x65, 0x6e,
0x1e6d, 0x75, 0x6b, 0x12b, 0x6b, 0x72, 0x65, 0x6c, 0x65, 0x6e, 0x1e0d, 0x65,
0x69, 0x6b, 0x6c, 0x61, 0x20, 0x70, 0x61, 0x6a, 0x6c, 0x6f, 0x73, 0x20,
0x61, 0x6e, 0x6a, 0x65, 0x6c, 0x65, 0x73, 0x6c, 0x6f, 0x76, 0x65, 0x72,
0x20, 0x70, 0x72, 0x69, 0x6e, 0x63, 0x2019, 0x73, 0x20, 0x6b, 0x75, 0x76,
0x61, 0x1e6d, 0x61, 0x72, 0x6d, 0x61, 0x73, 0x12b, 0x6f, 0x6d, 0x61, 0x6e,
0x61, 0x67, 0x75, 0x61, 0x6d, 0x61, 0x6e, 0x61, 0x75, 0x73, 0x6d, 0x61,
0x72, 0x69, 0x67, 0x6f, 0x1e6d, 0x6d, 0x61, 0x72, 0x1e6d, 0x69, 0x6e, 0x69,
0x6b, 0x6d, 0x61, 0x1e6d, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x73, 0x6d, 0x61,
0x73, 0x61, 0x1e6d, 0x6c, 0x61, 0x6e, 0x6d, 0x65, 0x6e, 0x6f, 0x6d, 0x69,
0x6e, 0x12b, 0x6d, 0x65, 0x72, 0x69, 0x1e0d, 0x61, 0x6d, 0x65, 0x1e6d, 0x6c,
0x61, 0x6b, 0x61, 0x1e6d, 0x6c, 0x61, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x63,
0x6f, 0x20, 0x73, 0x69, 0x1e6d, 0x79, 0x6d, 0x69, 0x6b, 0x76, 0x69, 0x6c,
0xf5, 0x6d, 0x6f, 0x6e, 0x6b, 0x1e6d, 0x6f, 0x6e, 0x6d, 0x6f, 0x6e, 0x1e6d,
0x65, 0x72, 0x72, 0x65, 0x6d, 0x6f, 0x6e, 0x1e6d, 0x65, 0x76, 0x69, 0x64,
0x69, 0x6f, 0x6d, 0x6f, 0x6e, 0x1e6d, 0x73, 0x65, 0x72, 0x72, 0x65, 0x1e6d,
0x6e, 0x61, 0x73, 0x61, 0x75, 0x6e, 0x69, 0x79, 0x75, 0x20, 0x79, 0x6f,
0x72, 0x6b, 0x6e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x62, 0x69, 0x79,
0x75, 0x6c, 0x61, 0x2c, 0x20, 0x75, 0x74, 0x74, 0x61, 0x72, 0x20, 0x1e0d,
0x61, 0x6b, 0x6f, 0x1e6d, 0x61, 0x6d, 0x61, 0x1e0d, 0x69, 0x6e, 0x12b, 0x20,
0x75, 0x74, 0x74, 0x61, 0x72, 0x20, 0x1e0d, 0x61, 0x6b, 0x6f, 0x1e6d, 0x61,
0x6e, 0x65, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x2c, 0x20, 0x75,
0x74, 0x74, 0x61, 0x72, 0x20, 0x1e0d, 0x61, 0x6b, 0x6f, 0x1e6d, 0x61, 0x6e,
0x16b, 0x6b, 0x6f, 0x6a, 0x69, 0x6e, 0x61, 0x67, 0x61, 0x70, 0x61, 0x6e,
0x61, 0x6d, 0x61, 0x70, 0x61, 0x72, 0x61, 0x6d, 0x61, 0x72, 0x69, 0x62,
0x6f, 0x70, 0x69, 0x6e, 0x69, 0x6b, 0x73, 0x70, 0x6f, 0x72, 0x74, 0x2d,
0x61, 0x75, 0x2d, 0x70, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x70, 0x6f, 0x72,
0x1e6d, 0x20, 0x61, 0x70, 0x20, 0x73, 0x70, 0x61, 0x69, 0x6e, 0x70, 0x6f,
0x72, 0x1e6d, 0x70, 0x20, 0x76, 0x65, 0x6c, 0x68, 0x6f, 0x70, 0x75, 0x65,
0x72, 0x74, 0x6f, 0x20, 0x72, 0x69, 0x6b, 0x6f, 0x70, 0x75, 0x1e47, 0x1e6d,
0x61, 0x20, 0x65, 0x72, 0x65, 0x6e, 0x61, 0x73, 0x72, 0x61, 0x6e, 0x6b,
0x69, 0x6e, 0x20, 0x69, 0x6e, 0x6c, 0x65, 0x64, 0x1e6d, 0x72, 0x65, 0x73,
0x69, 0x70, 0x69, 0x72, 0x65, 0x6a, 0x69, 0x6e, 0x61, 0x72, 0x65, 0x6a,
0x61, 0x6c, 0x79, 0x75, 0x1e6d, 0x72, 0x69, 0x6f, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x6b, 0x6f, 0x73, 0x61, 0x74, 0x61, 0x72, 0x69, 0x73, 0x61, 0x6e,
0x1e6d, 0x69, 0x61, 0x67, 0x6f, 0x73, 0x65, 0x6e, 0x74, 0x6f, 0x20, 0x1e0d,
0x6f, 0x6d, 0x69, 0x6e, 0x67, 0x6f, 0x73, 0x61, 0x6f, 0x20, 0x70, 0x61,
0x75, 0x6c, 0x6f, 0x69, 0x1e6d, 0x6f, 0x6b, 0x6f, 0x1e6d, 0x6f, 0x72, 0x6d,
0x69, 0x1e6d, 0x73, 0x69, 0x74, 0x6b, 0x61, 0x73, 0x65, 0x6e, 0x1e6d, 0x20,
0x62, 0x61, 0x72, 0x74, 0x65, 0x6c, 0x65, 0x6d, 0x69, 0x73, 0x65, 0x6e,
0x1e6d, 0x20, 0x6a, 0x6f, 0x6e, 0x73, 0x73, 0x65, 0x6e, 0x74, 0x20, 0x6b,
0x69, 0x1e6d, 0x1e6d, 0x73, 0x73, 0x65, 0x6e, 0x74, 0x20, 0x6c, 0x75, 0x73,
0x69, 0x61, 0x73, 0x65, 0x6e, 0x1e6d, 0x20, 0x74, 0x6f, 0x6d, 0x61, 0x73,
0x73, 0x65, 0x6e, 0x1e6d, 0x20, 0x76, 0x69, 0x6e, 0x73, 0x65, 0x6e, 0x1e6d,
0x73, 0x76, 0x69, 0x70, 0x1e6d, 0x20, 0x6b, 0x75, 0x72, 0x72, 0x65, 0x6e,
0x1e6d, 0x74, 0x65, 0x67, 0x75, 0x74, 0x75, 0x6c, 0x65, 0x74, 0x69, 0x68,
0x76, 0x61, 0x6e, 0x61, 0x1e6d, 0x6f, 0x72, 0x6f, 0x6e, 0x1e6d, 0x6f, 0x1e6d,
0x6f, 0x72, 0x1e6d, 0x6f, 0x6c, 0x61, 0x76, 0x61, 0x6e, 0x6b, 0x16b, 0x76,
0x65, 0x72, 0x76, 0x68, 0x61, 0x69, 0x1e6d, 0x68, 0x6f, 0x72, 0x73, 0x76,
0x69, 0x6e, 0x6e, 0x69, 0x70, 0x65, 0x67, 0x79, 0x61, 0x6b, 0x75, 0x1e6d,
0x61, 0x1e6d, 0x6b, 0x65, 0x73, 0x65, 0x65, 0x1e0d, 0x65, 0x76, 0x69, 0x73,
0x1e0d, 0x79, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64, 0x65, 0x20, 0x75,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x6d, 0x65, 0x6b, 0x76, 0x61, 0x72, 0x69,
0x6d, 0x61, 0x76, 0x73, 0x6f, 0x6e, 0x6d, 0x65, 0x6b, 0x20, 0x6d, 0x75,
0x72, 0x1e0d, 0x6f, 0x70, 0x61, 0x6c, 0x6d, 0x65, 0x72, 0x72, 0x6f, 0x74,
0x65, 0x72, 0x61, 0x73, 0x79, 0x6f, 0x76, 0x61, 0x1e6d, 0x72, 0x6f, 0x6c,
0x6c, 0x76, 0x6f, 0x73, 0x1e6d, 0x6f, 0x6b, 0x6c, 0x6e, 0x67, 0x79, 0x61,
0x72, 0x62, 0x79, 0x65, 0x6e, 0x65, 0x1e0d, 0x65, 0x6e, 0x61, 0x6c, 0x6d,
0x61, 0x1e6d, 0x79, 0x61, 0x6d, 0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x61, 0x1e0d,
0x69, 0x72, 0x61, 0x6b, 0x74, 0x61, 0x75, 0x61, 0x61, 0x6b, 0x74, 0x6f,
0x62, 0x65, 0x61, 0x73, 0x67, 0x61, 0x62, 0x61, 0x1e6d, 0x61, 0x74, 0x61,
0x72, 0x61, 0x75, 0x62, 0x61, 0x67, 0x64, 0x61, 0x64, 0x62, 0x61, 0x68,
0x72, 0x61, 0x69, 0x6e, 0x62, 0x61, 0x6b, 0x75, 0x62, 0x61, 0x6e, 0x67,
0x6b, 0x6f, 0x6b, 0x62, 0x61, 0x72, 0x6e, 0x61, 0x75, 0x6c, 0x62, 0x65,
0x69, 0x72, 0x75, 0x74, 0x62, 0x69, 0x73, 0x6b, 0x65, 0x6b, 0x62, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x63, 0x69, 0x74, 0x61, 0x6b, 0x6f, 0x69, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x6f,
0x1e0d, 0x61, 0x6d, 0x61, 0x73, 0x6b, 0x75, 0x73, 0x1e0d, 0x69, 0x6c, 0x69,
0x64, 0x75, 0x62, 0x61, 0x69, 0x64, 0x75, 0x73, 0x61, 0x6d, 0x62, 0x65,
0x70, 0x61, 0x6d, 0x61, 0x67, 0x75, 0x73, 0x74, 0x61, 0x67, 0x61, 0x6a,
0x61, 0x68, 0x65, 0x62, 0x72, 0x6f, 0x6e, 0x68, 0x6f, 0x20, 0x63, 0x69,
0x20, 0x6d, 0x69, 0x6e, 0x68, 0x20, 0x73, 0x69, 0x74, 0x69, 0x68, 0x6f,
0x6e, 0x67, 0x20, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x72, 0x6b, 0x75, 0x1e6d,
0x73, 0x6b, 0x6a, 0x61, 0x79, 0x61, 0x70, 0x75, 0x72, 0x61, 0x6a, 0x65,
0x72, 0x75, 0x73, 0x61, 0x6c, 0x65, 0x6d, 0x6b, 0x61, 0x62, 0x75, 0x6c,
0x6b, 0x61, 0x6d, 0x63, 0x61, 0x1e6d, 0x6b, 0x61, 0x6b, 0x61, 0x72, 0x61,
0x63, 0x12b, 0x6b, 0x61, 0x1e6d, 0x6d, 0x61, 0x6e, 0x1e0d, 0x75, 0x6b, 0x61,
0x6e, 0x64, 0x69, 0x67, 0x61, 0x6b, 0x6f, 0x6c, 0x6b, 0x61, 0x74, 0x61,
0x6b, 0x72, 0x61, 0x73, 0x6e, 0x65, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x6b,
0x75, 0x61, 0x6c, 0x61, 0x20, 0x6c, 0x75, 0x6d, 0x70, 0x75, 0x72, 0x6b,
0x75, 0x63, 0x69, 0x6e, 0x67, 0x6b, 0x75, 0x76, 0x61, 0x69, 0x74, 0x6d,
0x6b, 0x61, 0x6f, 0x6d, 0x65, 0x67, 0x61, 0x1e0d, 0x61, 0x6e, 0x6d, 0x61,
0x6b, 0x61, 0x73, 0x61, 0x72, 0x6d, 0x61, 0x6e, 0x12b, 0x6c, 0x61, 0x6d,
0x61, 0x73, 0x6b, 0x61, 0x74, 0x6e, 0x69, 0x6b, 0x6f, 0x73, 0x69, 0x79,
0x61, 0x6e, 0x65, 0x76, 0x6f, 0x6b, 0x75, 0x6a, 0x6e, 0x65, 0x1e6d, 0x73,
0x6b, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x70, 0x6e, 0x6f, 0x6d, 0x20, 0x70, 0x65, 0x6e, 0x68, 0x70, 0x6f, 0x6e,
0x1e6d, 0x69, 0x61, 0x6e, 0x61, 0x6b, 0x70, 0x79, 0x6f, 0x6e, 0x67, 0x79,
0x61, 0x6e, 0x67, 0x6b, 0x6f, 0x73, 0x1e6d, 0x61, 0x6e, 0x65, 0x6b, 0x69,
0x6a, 0x75, 0x6f, 0x72, 0x64, 0x61, 0x72, 0x69, 0x79, 0x61, 0x64, 0x73,
0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x6b,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x6f, 0x6c, 0x73, 0x65, 0x6e, 0x67, 0x61,
0x69, 0x73, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x73, 0x72, 0x65,
0x1e0d, 0x6e, 0x65, 0x6b, 0x6f, 0x6c, 0x79, 0x6d, 0x73, 0x6b, 0x74, 0x61,
0x69, 0x70, 0x65, 0x69, 0x74, 0x61, 0x73, 0x6b, 0x65, 0x6e, 0x1e6d, 0x74,
0x62, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x74, 0x65, 0x68, 0x72, 0x61, 0x6e,
0x74, 0x69, 0x6d, 0x70, 0x68, 0x75, 0x1e6d, 0x6f, 0x6b, 0x79, 0x6f, 0x1e6d,
0x6f, 0x6d, 0x73, 0x6b, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x1e6d, 0x61,
0x72, 0x75, 0x72, 0x75, 0x6d, 0x63, 0x69, 0x79, 0x75, 0x73, 0x74, 0x2d,
0x6e, 0x65, 0x72, 0x61, 0x76, 0x69, 0x65, 0x6e, 0x1e6d, 0x69, 0x61, 0x61,
0x6e, 0x76, 0x6c, 0x61, 0x1e0d, 0x69, 0x76, 0x6f, 0x73, 0x1e6d, 0x6f, 0x6b,
0x79, 0x61, 0x6b, 0x75, 0x1e6d, 0x73, 0x6b, 0x79, 0x61, 0x6e, 0x67, 0x6f,
0x6e, 0x79, 0x69, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x79, 0x65, 0x72, 0x65, 0x76, 0x61, 0x6e, 0x61, 0x6a, 0x6f,
0x72, 0x65, 0x73, 0x62, 0x65, 0x72, 0x6d, 0x16b, 0x64, 0x61, 0x6b, 0x65,
0x6e, 0x65, 0x72, 0x69, 0x6b, 0x65, 0x70, 0x20, 0x76, 0x65, 0x72, 0x1e0d,
0x65, 0x70, 0x65, 0x72, 0x6f, 0x6d, 0x61, 0x1e0d, 0x69, 0x65, 0x72, 0x61,
0x72, 0x65, 0x79, 0x6b, 0x79, 0x61, 0x76, 0x69, 0x6b, 0x64, 0x6b, 0x69,
0x1e47, 0x20, 0x6a, 0x6f, 0x72, 0x6a, 0x69, 0x61, 0x73, 0x65, 0x6e, 0x74,
0x20, 0x68, 0x65, 0x6c, 0x65, 0x6e, 0x61, 0x73, 0x1e6d, 0x61, 0x6e, 0x6c,
0x69, 0x61, 0x65, 0x64, 0x69, 0x6c, 0x65, 0x69, 0x64, 0x62, 0x72, 0x69,
0x73, 0x62, 0x61, 0x6e, 0x65, 0x62, 0x72, 0x6f, 0x6b, 0x65, 0x6e, 0x20,
0x68, 0x69, 0x6c, 0x6c, 0x1e0d, 0x61, 0x72, 0x76, 0x69, 0x6e, 0x79, 0x75,
0x6b, 0x6c, 0x61, 0x68, 0x6f, 0x62, 0x61, 0x72, 0x1e6d, 0x6c, 0x69, 0x6e,
0x1e0d, 0x65, 0x72, 0x6d, 0x61, 0x6e, 0x6c, 0x6f, 0x72, 0x1e0d, 0x20, 0x68,
0x6f, 0x76, 0x65, 0x6d, 0x65, 0x6c, 0x62, 0x6f, 0x72, 0x6e, 0x65, 0x70,
0x65, 0x72, 0x74, 0x73, 0x79, 0x1e0d, 0x6e, 0x12b, 0x61, 0x6d, 0x73, 0x1e6d,
0x65, 0x72, 0x1e0d, 0x61, 0x6d, 0x61, 0x61, 0x1e47, 0x1e0d, 0x6f, 0x72, 0x61,
0x61, 0x61, 0x73, 0x1e6d, 0x72, 0x61, 0x68, 0x61, 0x6e, 0x65, 0x74, 0x65,
0x6e, 0x73, 0x62, 0x65, 0x6c, 0x67, 0x72, 0x61, 0x1e0d, 0x65, 0x62, 0x65,
0x72, 0x6c, 0x69, 0x6e, 0x62, 0x72, 0x61, 0x1e6d, 0x69, 0x73, 0x6c, 0x61,
0x76, 0x61, 0x62, 0x72, 0x75, 0x73, 0x73, 0x65, 0x6c, 0x73, 0x62, 0x75,
0x6b, 0x61, 0x72, 0x65, 0x73, 0x74, 0x62, 0x75, 0x1e0d, 0x61, 0x70, 0x65,
0x73, 0x74, 0x62, 0x75, 0x73, 0x69, 0x6e, 0x67, 0x65, 0x6e, 0x6b, 0x69,
0x73, 0x69, 0x6e, 0x61, 0x75, 0x6b, 0x6f, 0x70, 0x65, 0x6e, 0x68, 0x61,
0x67, 0x65, 0x6e, 0x1e0d, 0x62, 0x6c, 0x69, 0x6e, 0x6a, 0x69, 0x62, 0x72,
0x61, 0x6c, 0x1e6d, 0x61, 0x72, 0x67, 0x65, 0x72, 0x6e, 0x73, 0x69, 0x68,
0x65, 0x6c, 0x73, 0x69, 0x6e, 0x6b, 0x69, 0x61, 0x61, 0x69, 0x6c, 0x20,
0x61, 0x70, 0x20, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x62,
0x75, 0x6c, 0x6a, 0x65, 0x72, 0x73, 0x69, 0x6b, 0x61, 0x6c, 0x69, 0x6e,
0x69, 0x6e, 0x67, 0x72, 0x61, 0x1e0d, 0x6b, 0x69, 0x72, 0x6f, 0x76, 0x6b,
0x69, 0x79, 0x76, 0x6c, 0x69, 0x73, 0x62, 0x6f, 0x6e, 0x6c, 0x79, 0x75,
0x62, 0x6c, 0x79, 0x61, 0x6e, 0x61, 0x6c, 0x6e, 0x1e0d, 0x6e, 0x6c, 0x6b,
0x73, 0x65, 0x6d, 0x62, 0x72, 0x67, 0x6d, 0x65, 0x1e0d, 0x72, 0x69, 0x1e0d,
0x6d, 0x61, 0x6c, 0x1e6d, 0x61, 0x6d, 0x61, 0x72, 0x69, 0x65, 0x68, 0x61,
0x6d, 0x6d, 0x69, 0x6e, 0x73, 0x6b, 0x6d, 0x6f, 0x6e, 0x61, 0x6b, 0x6f,
0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x6f, 0x73, 0x6c, 0x6f, 0x70, 0x61, 0x72,
0x69, 0x73, 0x70, 0x1e0d, 0x67, 0x6f, 0x72, 0x69, 0x74, 0x73, 0x61, 0x70,
0x72, 0x61, 0x67, 0x72, 0x69, 0x67, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x73, 0x61, 0x6e, 0x20, 0x6d, 0x61, 0x72, 0x69, 0x6e, 0x6f, 0x73,
0x61, 0x72, 0x61, 0x6a, 0x65, 0x76, 0x6f, 0x73, 0x61, 0x72, 0x61, 0x1e6d,
0x6f, 0x76, 0x73, 0x69, 0x6d, 0x70, 0x65, 0x72, 0x6f, 0x70, 0x6f, 0x6c,
0x73, 0x6b, 0x6f, 0x70, 0x69, 0x73, 0x6f, 0x70, 0x69, 0x79, 0x61, 0x73,
0x1e6d, 0x6f, 0x6b, 0x68, 0x6f, 0x6d, 0x74, 0x61, 0x6c, 0x69, 0x6e, 0x1e6d,
0x69, 0x72, 0x61, 0x6e, 0x65, 0x75, 0x6c, 0x79, 0x61, 0x6e, 0x6f, 0x76,
0x73, 0x6b, 0x76, 0x61, 0x1e0d, 0x75, 0x6a, 0x76, 0x61, 0x1e6d, 0x69, 0x6b,
0x61, 0x6e, 0x76, 0x69, 0x65, 0x6e, 0x6e, 0x61, 0x76, 0x69, 0x6c, 0x6e,
0x69, 0x75, 0x73, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x1e0d,
0x76, 0x61, 0x72, 0x73, 0x61, 0x74, 0x65, 0x67, 0x75, 0x73, 0x69, 0x67,
0x61, 0x6c, 0x70, 0x61, 0x6a, 0x75, 0x72, 0x69, 0x63, 0x1e6d, 0x61, 0x6e,
0x61, 0x6e, 0x61, 0x72, 0x69, 0x76, 0x63, 0x68, 0x61, 0x67, 0x6f, 0x73,
0x6b, 0x72, 0x69, 0x73, 0x1e6d, 0x6d, 0x61, 0x73, 0x6b, 0x6f, 0x6b, 0x6f,
0x73, 0x6b, 0x6f, 0x6d, 0x6f, 0x72, 0x6f, 0x6b, 0x65, 0x72, 0x67, 0x75,
0x65, 0x6c, 0x65, 0x6e, 0x6d, 0x61, 0x68, 0x65, 0x6d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x65, 0x73, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x75,
0x73, 0x6d, 0x61, 0x79, 0x6f, 0x1e6d, 0x72, 0x65, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x61, 0x70, 0x69, 0x61, 0x101, 0x6b, 0x6c, 0x61, 0x6e, 0x1e0d, 0x62,
0x6f, 0x75, 0x67, 0x61, 0x6e, 0x76, 0x69, 0x6c, 0x6c, 0x63, 0x65, 0x1e6d,
0x61, 0x6d, 0x63, 0x16b, 0x6b, 0x12b, 0x73, 0x74, 0x65, 0x72, 0x69, 0x70,
0x65, 0x1e6d, 0x65, 0x70, 0x61, 0x6b, 0x61, 0x6f, 0x70, 0x6f, 0x70, 0x69,
0x6a, 0x69, 0x70, 0x75, 0x6e, 0x61, 0x70, 0x75, 0x74, 0x69, 0x67, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x67, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x67, 0x75, 0x61, 0x1e0d, 0x61, 0x6c, 0x6b, 0x65, 0x6e, 0x61,
0x6c, 0x6b, 0x61, 0x6e, 0x1e6d, 0x6f, 0x6e, 0x6b, 0x69, 0x72, 0x69, 0x74,
0x69, 0x6d, 0x61, 0x74, 0x69, 0x6b, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6b,
0x76, 0x61, 0x6a, 0x61, 0x6c, 0x65, 0x69, 0x6e, 0x6d, 0x61, 0x6a, 0x75,
0x72, 0x6f, 0x6d, 0x61, 0x72, 0x6b, 0x69, 0x73, 0x61, 0x73, 0x6d, 0x69,
0x1e0d, 0x76, 0x61, 0x79, 0x6e, 0x61, 0x75, 0x72, 0x75, 0x6e, 0x69, 0x75,
0x65, 0x6e, 0x6f, 0x72, 0x70, 0x6f, 0x6b, 0x6e, 0x75, 0x6d, 0x69, 0x65,
0x70, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x70, 0x61, 0x6e, 0x67, 0x6f, 0x70,
0x61, 0x6c, 0x61, 0x75, 0x70, 0x69, 0x1e6d, 0x6b, 0x65, 0x72, 0x6e, 0x70,
0x6f, 0x6e, 0x70, 0x65, 0x69, 0x70, 0x6f, 0x72, 0x1e6d, 0x20, 0x6d, 0x6f,
0x72, 0x65, 0x73, 0x62, 0x69, 0x72, 0x61, 0x72, 0x6f, 0x74, 0x6f, 0x6e,
0x67, 0x61, 0x73, 0x61, 0x69, 0x70, 0x61, 0x6e, 0x1e6d, 0x61, 0x68, 0x69,
0x1e6d, 0x69, 0x74, 0x61, 0x72, 0x61, 0x76, 0x61, 0x1e6d, 0x6f, 0x6e, 0x67,
0x61, 0x1e6d, 0x61, 0x70, 0x75, 0x76, 0x65, 0x6b, 0x76, 0x61, 0x6c, 0x6c,
0x69, 0x73
};

static inline constexpr char16_t shortZoneNameTable[] = {
0x55, 0x54, 0x43, 0x48, 0x53, 0x54, 0x48, 0x44, 0x54, 0x2205, 0x2205, 0x2205,
0x49, 0x53, 0x54, 0x42, 0x53, 0x54, 0x54, 0x55, 0x48, 0x54, 0xd83a, 0xdd11,
0xd83a, 0xdd16, 0xd83a, 0xdd0a, 0xd83a, 0xdd11, 0xd83a, 0xdd16, 0xd83a, 0xdd16, 0xd83a, 0xdd11,
0xd83a, 0xdd15, 0xd83a, 0xdd16, 0x54, 0x53, 0xc8, 0x20, 0x28, 0xc8, 0x69, 0x72,
0x69, 0x6e, 0x6e, 0x29, 0x54, 0x53, 0x42, 0x90f, 0x91a, 0x90f, 0x938, 0x91f,
0x940, 0x41, 0x43, 0xc9, 0x41, 0x53, 0x42, 0x53, 0x47, 0x48, 0x54, 0x43,
0x55, 0x4f, 0x49, 0x48, 0x4f, 0x4c, 0x48, 0x645, 0x20, 0x639, 0x20, 0x648,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x55, 0x54, 0x43, 0x48,
0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x74, 0x69, 0x64, 0x48, 0x6f,
0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x54, 0x4d, 0x54, 0x48,
0x54, 0x54, 0x48, 0x54, 0x4c, 0x47, 0x69, 0x1edd, 0x20, 0x48, 0x53, 0x54,
0x189, 0x41, 0x4b
};

static inline constexpr char16_t longZoneNameTable[] = {
0x43, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x65, 0x64, 0x20,
0x55, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x49, 0x72, 0x69, 0x73, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x72, 0x69,
0x74, 0x69, 0x73, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x410, 0x438, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x20,
0x410, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x410,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43d, 0x433, 0x43b, 0x438, 0x430, 0x20, 0x410,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x430, 0x43c, 0x4ad,
0x430, 0x47, 0x65, 0x6b, 0x6f, 0xf6, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x65,
0x72, 0x64, 0x65, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x65,
0x6c, 0x65, 0x20, 0x74, 0x79, 0x64, 0x49, 0x65, 0x72, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x42, 0x72, 0x69, 0x74, 0x73, 0x65, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x79, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x65, 0x20, 0x65, 0x20, 0x6b, 0x6f, 0x6f, 0x72,
0x64, 0x69, 0x6e, 0x75, 0x61, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x72, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49,
0x72, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e,
0x69, 0x6b, 0x65, 0x12e8, 0x1270, 0x1240, 0x1290, 0x1263, 0x1260, 0x1228, 0x20, 0x1201,
0x1208, 0x1308, 0x1265, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x12ed, 0x122a, 0x123d,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1265, 0x122a, 0x1275, 0x123d, 0x20, 0x12e8, 0x1260, 0x130b,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x627, 0x644,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x639, 0x627, 0x644, 0x645,
0x64a, 0x20, 0x627, 0x644, 0x645, 0x646, 0x633, 0x642, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x623, 0x64a, 0x631, 0x644, 0x646, 0x62f, 0x627, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x631,
0x64a, 0x637, 0x627, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x74, 0x69, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x75, 0x6e, 0x69, 0x76,
0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x65,
0x6e, 0x61, 0x64, 0x6f, 0x540, 0x561, 0x574, 0x561, 0x577, 0x56d, 0x561, 0x580,
0x570, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56f, 0x578, 0x578, 0x580, 0x564, 0x56b,
0x576, 0x561, 0x581, 0x57e, 0x561, 0x56e, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x53b, 0x57c, 0x56c, 0x561, 0x576, 0x564, 0x561, 0x56f, 0x561, 0x576,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x580, 0x56b, 0x57f, 0x561, 0x576, 0x561,
0x56f, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x9b8, 0x9ae, 0x9a8, 0x9cd,
0x9ac, 0x9bf, 0x9a4, 0x20, 0x9b8, 0x9be, 0x9f0, 0x9cd, 0x9ac, 0x99c, 0x9a8, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x987, 0x9f0, 0x9bf, 0x99a, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9f0, 0x9bf,
0x99f, 0x9bf, 0x99b, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x64, 0x61,
0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x62,
0x72, 0x69, 0x74, 0xe1, 0x6e, 0x69, 0x63, 0x61, 0x4b, 0x6f, 0x6f, 0x72,
0x64, 0x69, 0x6e, 0x61, 0x73, 0x69, 0x79, 0x61, 0x20, 0x65, 0x64, 0x69,
0x6c, 0x6d, 0x69, 0x15f, 0x20, 0xfc, 0x6d, 0x75, 0x6d, 0x64, 0xfc, 0x6e,
0x79, 0x61, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x130, 0x72, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x42, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x79, 0x61,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x9b8, 0x9cd,
0x9a5, 0x9be, 0x9a8, 0x9be, 0x982, 0x995, 0x9bf, 0x9a4, 0x20, 0x986, 0x9a8, 0x9cd,
0x9a4, 0x9b0, 0x9cd, 0x99c, 0x9be, 0x9a4, 0x9bf, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x986, 0x987, 0x9b0, 0x9bf, 0x9b6, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9b0, 0x9bf, 0x99f, 0x9bf, 0x9b6, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x75, 0x6e,
0x69, 0x62, 0x65, 0x72, 0x74, 0x73, 0x61, 0x6c, 0x20, 0x6b, 0x6f, 0x6f,
0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x75, 0x61, 0x49, 0x72, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4c, 0x6f, 0x6e,
0x64, 0x72, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x423, 0x43d, 0x456, 0x432, 0x435, 0x440,
0x441, 0x430, 0x43b, 0x44c, 0x43d, 0x44b, 0x20, 0x43a, 0x430, 0x430, 0x440, 0x434,
0x44b, 0x43d, 0x430, 0x432, 0x430, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x406,
0x440, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x411,
0x440, 0x44b, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x917, 0x94b, 0x930, 0x94b, 0x92c, 0x939,
0x94b, 0x928, 0x93e, 0x92f, 0x20, 0x92e, 0x941, 0x932, 0x941, 0x917, 0x928, 0x93e,
0x902, 0x20, 0x938, 0x92e, 0x906, 0x908, 0x930, 0x940, 0x936, 0x20, 0x925, 0x93e,
0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92c, 0x94d, 0x930, 0x940,
0x91f, 0x940, 0x936, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20,
0x938, 0x92e, 0x4b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x69, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x73, 0x76, 0x6a, 0x65, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x69, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41a, 0x43e, 0x43e, 0x440, 0x434,
0x438, 0x43d, 0x438, 0x441, 0x430, 0x43d, 0x43e, 0x20, 0x443, 0x43d, 0x438, 0x432,
0x435, 0x440, 0x437, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x418, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x61, 0x6d, 0x7a, 0x65, 0x72, 0x20, 0x68, 0x6f, 0x6c, 0x6c,
0x76, 0x65, 0x64, 0x65, 0x6c, 0x20, 0x6b, 0x65, 0x6e, 0x75, 0x72, 0x7a,
0x68, 0x69, 0x65, 0x74, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x49, 0x77, 0x65, 0x72, 0x7a, 0x68, 0x6f, 0x6e,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x42, 0x72, 0x65,
0x69, 0x7a, 0x68, 0x2d, 0x56, 0x65, 0x75, 0x72, 0x41a, 0x43e, 0x43e, 0x440,
0x434, 0x438, 0x43d, 0x438, 0x440, 0x430, 0x43d, 0x43e, 0x20, 0x443, 0x43d, 0x438,
0x432, 0x435, 0x440, 0x441, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x100a, 0x103e, 0x102d, 0x1011,
0x102c, 0x1038, 0x101e, 0x100a, 0x1037, 0x103a, 0x20, 0x1000, 0x1019, 0x1039, 0x1018, 0x102c,
0x1037, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x101b, 0x1005, 0x103a, 0x101b, 0x103e,
0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1017, 0x103c, 0x102d, 0x1010, 0x102d, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x5354,
0x8abf, 0x4e16, 0x754c, 0x6642, 0x9593, 0x611b, 0x723e, 0x862d, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x82f1, 0x570b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x534f, 0x8c03, 0x4e16, 0x754c, 0x65f6, 0x95f4,
0x7231, 0x5c14, 0x5170, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x82f1, 0x56fd, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x54, 0x65, 0x6d, 0x70, 0x73, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x61, 0x74, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x62, 0x72, 0x69, 0x74, 0xe0, 0x6e, 0x69, 0x63, 0x61,
0x47, 0x69, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x72, 0x20,
0x6e, 0x67, 0x61, 0x20, 0x4b, 0x69, 0x6e, 0x61, 0x74, 0x69, 0x62, 0x75,
0x6b, 0x2d, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x54, 0x61,
0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x69, 0x73, 0x68, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73, 0x68,
0xd804, 0xdd18, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd20,
0x20, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd17, 0xd804,
0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd2d, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd0c,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x43d, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x13a2,
0x13a9, 0x13e0, 0x13f1, 0x20, 0x13c2, 0x13a6, 0x13d3, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a8, 0x13b5, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c8, 0x13d7, 0x13cf, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x4e16, 0x754c, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x432, 0x441,
0x435, 0x43c, 0x456, 0x301, 0x440, 0x43d, 0x43e, 0x435, 0x20, 0x441, 0x477, 0x433,
0x445, 0x440, 0x43e, 0x43d, 0x457, 0x437, 0x438, 0x301, 0x440, 0x43e, 0x432, 0x430,
0x43d, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x41f,
0x4d7, 0x442, 0x4d7, 0x43c, 0x20, 0x442, 0x4d7, 0x43d, 0x447, 0x435, 0x440, 0x438,
0x20, 0x43a, 0x43e, 0x43e, 0x440, 0x434, 0x438, 0x43d, 0x430, 0x446, 0x438, 0x43b,
0x435, 0x43d, 0x435, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x440,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x411,
0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x69, 0x63, 0x6b, 0x4a, 0x72, 0x75, 0xdf, 0x62,
0x72, 0x65, 0x74, 0x61, 0x6e, 0x6e, 0x69, 0x6a, 0x65, 0x20, 0x73, 0x69,
0x6e, 0x67, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x63,
0x6b, 0x54, 0x65, 0x6d, 0x70, 0x75, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x65, 0x20, 0x63, 0x75, 0x75, 0x72, 0x64, 0x69,
0x6e, 0x61, 0x74, 0x75, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x69,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x73, 0x76, 0x6a, 0x65, 0x74, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x72, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x72, 0x69,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x6f, 0x6f,
0x72, 0x64, 0x69, 0x6e, 0x6f, 0x76, 0x61, 0x6e, 0xfd, 0x20, 0x73, 0x76,
0x11b, 0x74, 0x6f, 0x76, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x62, 0x72, 0x69, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x65, 0x72, 0x65, 0x74, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x72, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x42, 0x72, 0x69, 0x74, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x924, 0x93e, 0x932, 0x92e, 0x947, 0x932, 0x940, 0x20, 0x906, 0x932, 0x92e, 0x940,
0x20, 0x938, 0x92e, 0x93e, 0x902, 0x906, 0x92f, 0x930, 0x93f, 0x936, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x92c, 0x94d, 0x930, 0x93f,
0x91f, 0x93f, 0x936, 0x20, 0x917, 0x930, 0x94d, 0x92e, 0x93f, 0x92f, 0x947, 0x902,
0x20, 0x926, 0x93e, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x67, 0x65, 0x63, 0x6f,
0xf6, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x65, 0x72, 0x64, 0x65, 0x20, 0x77,
0x65, 0x72, 0x65, 0x6c, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x65, 0x72,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x42, 0x72, 0x69, 0x74, 0x73, 0x65, 0x20, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0xf68, 0xf71, 0xf0b, 0xf61,
0xf62, 0xf72, 0xf64, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb2, 0xf71, 0xf72, 0xf0b,
0xf4a, 0xf72, 0xf64, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0x4b, 0x6f, 0x6f, 0x72, 0x64,
0x69, 0x6e, 0x65, 0x65, 0x72, 0x69, 0x74, 0x75, 0x64, 0x20, 0x6d, 0x61,
0x61, 0x69, 0x6c, 0x6d, 0x61, 0x61, 0x65, 0x67, 0x49, 0x69, 0x72, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x72, 0x69, 0x74,
0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x58, 0x65, 0x78,
0x65, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f,
0x61, 0x6e, 0x79, 0x69, 0x20, 0x6d, 0x65, 0x49, 0x72, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x72, 0x69, 0x74, 0x69,
0x73, 0x68, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61,
0x6d, 0x73, 0x6b, 0x69, 0x70, 0x61, 0xf0, 0x20, 0x68, 0x65, 0x69, 0x6d,
0x73, 0x74, 0xed, 0xf0, 0xcd, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x42, 0x72, 0x65, 0x74, 0x73,
0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x42, 0x72,
0x69, 0x74, 0x61, 0x69, 0x6e, 0x55, 0x54, 0x43, 0x2d, 0x79, 0x6c, 0x65,
0x69, 0x73, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x6e,
0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x42,
0x72, 0x69, 0x74, 0x61, 0x6e, 0x6e, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x74, 0x65, 0x6d, 0x70, 0x73, 0x20,
0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x65, 0x6c, 0x20, 0x63, 0x6f,
0x6f, 0x72, 0x64, 0x6f, 0x6e, 0x6e, 0xe9, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x69, 0x73, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x6e,
0x69, 0x71, 0x75, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x69, 0x73, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e,
0x6e, 0x69, 0x71, 0x75, 0x65, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d,
0xd83a, 0xdd24, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd46, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xc0,
0x6d, 0x20, 0x55, 0x69, 0x6c, 0x65, 0x2d, 0x63, 0x68, 0x6f, 0x69, 0x74,
0x63, 0x68, 0x65, 0x61, 0x6e, 0x6e, 0x20, 0x43, 0x6f, 0x2d, 0xf2, 0x72,
0x64, 0x61, 0x6e, 0x61, 0x69, 0x63, 0x68, 0x74, 0x65, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x2d, 0xc8, 0x69, 0x72,
0x65, 0x61, 0x6e, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x42, 0x68, 0x72, 0x65, 0x61,
0x74, 0x61, 0x69, 0x6e, 0x6e, 0x42, 0x65, 0x20, 0x4e, 0x69, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x20, 0x4e, 0x69, 0x20, 0x59, 0x254, 0x254, 0x20, 0x4a,
0x65, 0x14b, 0x20, 0x46, 0x25b, 0x25b, 0x20, 0x4b, 0x25b, 0x74, 0x73, 0x75,
0x254, 0x20, 0x4e, 0x69, 0x69, 0x42, 0x65, 0x20, 0x4e, 0x69, 0x20, 0x49,
0x72, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x62, 0x69, 0x69, 0x20, 0x4b, 0x25b,
0x74, 0x73, 0x75, 0x254, 0x20, 0x4e, 0x69, 0x69, 0x42, 0x65, 0x20, 0x4e,
0x69, 0x20, 0x42, 0x72, 0x69, 0x74, 0x61, 0x69, 0x6e, 0x62, 0x69, 0x69,
0x20, 0x4b, 0x25b, 0x74, 0x73, 0x75, 0x254, 0x20, 0x4e, 0x69, 0x69, 0x20,
0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69,
0x61, 0x14b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x61, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65,
0x73, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x62, 0x72, 0x69, 0x74, 0xe1, 0x6e, 0x69, 0x63,
0x61, 0x10db, 0x10e1, 0x10dd, 0x10e4, 0x10da, 0x10d8, 0x10dd, 0x20, 0x10d9, 0x10dd, 0x10dd,
0x10e0, 0x10d3, 0x10d8, 0x10dc, 0x10d8, 0x10e0, 0x10d4, 0x10d1, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e0, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e0, 0x10d8, 0x10e2, 0x10d0, 0x10dc, 0x10d4, 0x10d7,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x4b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x69, 0x65,
0x72, 0x74, 0x65, 0x20, 0x57, 0x65, 0x6c, 0x74, 0x7a, 0x65, 0x69, 0x74,
0x49, 0x72, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x3a3, 0x3c5, 0x3bd, 0x3c4, 0x3bf, 0x3bd, 0x3b9, 0x3c3, 0x3bc, 0x3ad,
0x3bd, 0x3b7, 0x20, 0x3a0, 0x3b1, 0x3b3, 0x3ba, 0x3cc, 0x3c3, 0x3bc, 0x3b9, 0x3b1,
0x20, 0x38f, 0x3c1, 0x3b1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c1, 0x3bb, 0x3b1, 0x3bd, 0x3b4,
0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x392, 0x3c1, 0x3b5, 0x3c4, 0x3b1, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0xab8,
0xa82, 0xa95, 0xab2, 0xabf, 0xaa4, 0x20, 0xaaf, 0xac1, 0xaa8, 0xabf, 0xab5, 0xab0,
0xacd, 0xab8, 0xab2, 0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xa88, 0xab0, 0xabf, 0xab6,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xacd, 0xab0,
0xabf, 0xa9f, 0xabf, 0xab6, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0x48, 0x61, 0x64, 0x65, 0x77, 0x61, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x6e, 0x61, 0x20, 0x44, 0x75, 0x6e,
0x69, 0x79, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x72, 0x69,
0x73, 0x68, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61,
0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x42, 0x69, 0x72, 0x74,
0x61, 0x6e, 0x69, 0x79, 0x61, 0x5d6, 0x5de, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5e0,
0x5d9, 0x5d1, 0x5e8, 0x5e1, 0x5dc, 0x5d9, 0x20, 0x5de, 0x5ea, 0x5d5, 0x5d0, 0x5dd,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5e5, 0x20, 0x5d0, 0x5d9, 0x5e8,
0x5dc, 0x5e0, 0x5d3, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5e5, 0x20,
0x5d1, 0x5e8, 0x5d9, 0x5d8, 0x5e0, 0x5d9, 0x5d4, 0x938, 0x92e, 0x928, 0x94d, 0x935,
0x93f, 0x924, 0x20, 0x935, 0x948, 0x936, 0x94d, 0x935, 0x93f, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x906, 0x907, 0x930, 0x93f, 0x936, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x93f, 0x91f, 0x93f, 0x936, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0xe1,
0x6c, 0x74, 0x20, 0x76, 0x69, 0x6c, 0xe1, 0x67, 0x69, 0x64, 0x151, 0xed,
0x72, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62,
0x72, 0x69, 0x74, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x53, 0x61, 0x6d, 0x72, 0xe6, 0x6d, 0x64, 0x75, 0x72, 0x20, 0x61,
0x6c, 0xfe, 0x6a, 0xf3, 0xf0, 0x6c, 0x65, 0x67, 0x75, 0x72, 0x20, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0xcd, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x42, 0x72,
0x65, 0x74, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x4e, 0x68, 0x61, 0x7a, 0x69,
0x20, 0x4f, 0x67, 0x65, 0x20, 0x1ee4, 0x77, 0x61, 0x20, 0x4e, 0x69, 0x69,
0x6c, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x49, 0x72, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x42,
0x72, 0x69, 0x74, 0x61, 0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x55, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x54, 0x65,
0x72, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x73, 0x69, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x49, 0x6e, 0x67, 0x67, 0x72, 0x69, 0x73, 0x55, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x54, 0x65, 0x6d, 0x70,
0x6f, 0x72, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61,
0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x62,
0x72, 0x69, 0x74, 0x61, 0x6e, 0x6e, 0x69, 0x63, 0x75, 0x6e, 0x69, 0x76,
0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x69, 0x72,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x62,
0x72, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x41, 0x6d,
0x20, 0x55, 0x69, 0x6c, 0xed, 0x6f, 0x63, 0x68, 0x20, 0x4c, 0xe1, 0x72,
0x6e, 0x61, 0x63, 0x68, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0xe9, 0x61, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68,
0xc9, 0x69, 0x72, 0x65, 0x61, 0x6e, 0x6e, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x42,
0x72, 0x65, 0x61, 0x74, 0x61, 0x69, 0x6e, 0x65, 0x54, 0x65, 0x6d, 0x70,
0x6f, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x74, 0x6f,
0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x52, 0x65, 0x67, 0x6e, 0x6f, 0x20, 0x55, 0x6e, 0x69, 0x74, 0x6f,
0x5354, 0x5b9a, 0x4e16, 0x754c, 0x6642, 0x30a2, 0x30a4, 0x30eb, 0x30e9, 0x30f3, 0x30c9, 0x6a19,
0x6e96, 0x6642, 0x82f1, 0x56fd, 0x590f, 0x6642, 0x9593, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x55, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x4b,
0x61, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x68, 0x69, 0x6e, 0x61, 0x73, 0x69,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x49,
0x6e, 0x67, 0x67, 0x72, 0x69, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x55, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x4b, 0x6f, 0x72, 0x64,
0x65, 0x6e, 0x61, 0x64, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x49, 0x72, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72,
0x6f, 0x6e, 0x20, 0x42, 0x72, 0x69, 0x74, 0xe1, 0x6e, 0x69, 0x6b, 0x75,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x67, 0x72, 0x61, 0x263, 0x6c, 0x61,
0x6e, 0x20, 0x69, 0x6d, 0x79, 0x75, 0x64, 0x64, 0x73, 0x65, 0x6e, 0x41,
0x67, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x42, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x79, 0x61, 0xcb8, 0xc82, 0xc98, 0xc9f,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcbe, 0xcb0, 0xccd, 0xcb5, 0xca4, 0xccd, 0xcb0, 0xcbf,
0xc95, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc90, 0xcb0, 0xcbf, 0xcb7, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac,
0xccd, 0xcb0, 0xcbf, 0xc9f, 0xcbf, 0xcb7, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0x6a9, 0x648, 0x622, 0x631, 0x688, 0x646,
0x6cc, 0x679, 0x688, 0x20, 0x6cc, 0x648, 0x646, 0x6cc, 0x648, 0x631, 0x633, 0x644,
0x20, 0x648, 0x64e, 0x6a9, 0x6be, 0x627, 0x64e, 0x6cc, 0x631, 0x650, 0x634, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628,
0x631, 0x637, 0x672, 0x646, 0x648, 0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x915, 0x94b, 0x911, 0x930, 0x94d, 0x921,
0x928, 0x948, 0x91f, 0x93f, 0x921, 0x20, 0x92f, 0x942, 0x928, 0x935, 0x930, 0x94d,
0x938, 0x932, 0x20, 0x935, 0x916, 0x414, 0x4af, 0x43d, 0x438, 0x435, 0x436, 0x4af,
0x437, 0x456, 0x43b, 0x456, 0x43a, 0x20, 0x4af, 0x439, 0x43b, 0x435, 0x441, 0x442,
0x456, 0x440, 0x456, 0x43b, 0x433, 0x435, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x4b0, 0x43b, 0x44b, 0x431, 0x440, 0x438, 0x442, 0x430, 0x43d,
0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f, 0x1780, 0x179b, 0x178a, 0x17c2,
0x179b, 0x1798, 0x17b6, 0x1793, 0x1780, 0x17b6, 0x179a, 0x179f, 0x1798, 0x17d2, 0x179a, 0x17bd,
0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780,
0x17d2, 0x178a, 0x17c5, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17c0, 0x179a, 0x17a1, 0x1784,
0x17cb, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b,
0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x1780, 0x17d2,
0x179a, 0x1797, 0x1796, 0x200b, 0x17a2, 0x1784, 0x17cb, 0x1782, 0x17d2, 0x179b, 0x17c1, 0x179f,
0x938, 0x92e, 0x928, 0x94d, 0x935, 0x93f, 0x924, 0x20, 0x935, 0x948, 0x936, 0x94d,
0x935, 0x93f, 0x915, 0x20, 0x935, 0x947, 0x933, 0x906, 0x92f, 0x930, 0x93f, 0x936,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x92c, 0x94d, 0x930, 0x93f, 0x91f, 0x93f, 0x936, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0xd611, 0xc815, 0x20, 0xc138,
0xacc4, 0xc2dc, 0xc544, 0xc77c, 0xb79c, 0xb4dc, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc601, 0xad6d,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x47, 0x65, 0x72, 0x64, 0xfb, 0x6e, 0xee, 0x20, 0x79, 0x61, 0x20,
0x48, 0x65, 0x76, 0x64, 0x65, 0x6d, 0xee, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0xce, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x42, 0x69, 0x72, 0xee, 0x74, 0x61, 0x6e, 0x79, 0x61, 0x79, 0xea,
0x411, 0x438, 0x440, 0x434, 0x438, 0x43a, 0x442, 0x4af, 0x4af, 0x20, 0x434, 0x4af,
0x439, 0x43d, 0x4e9, 0x43b, 0x4af, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b,
0x442, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411,
0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0xec0, 0xea7,
0xea5, 0xeb2, 0xeaa, 0xeb2, 0xe81, 0xebb, 0xe99, 0xec0, 0xe8a, 0xeb5, 0xe87, 0xe9e,
0xeb4, 0xe81, 0xeb1, 0xe94, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2,
0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xec4, 0xead, 0x200b,
0xea3, 0xeb4, 0x200b, 0xe8a, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0,
0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xead, 0xeb1, 0xe87,
0x200b, 0xe81, 0xeb4, 0xe94, 0x55, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x101,
0x6c, 0x61, 0x69, 0x73, 0x20, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x113, 0x74, 0x61, 0x69, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x12a,
0x72, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x69, 0x65, 0x6c, 0x62, 0x72,
0x69, 0x74, 0x101, 0x6e, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x70, 0x61,
0x73, 0x61, 0x75, 0x6c, 0x69, 0x6f, 0x20, 0x73, 0x75, 0x64, 0x65, 0x72,
0x69, 0x6e, 0x74, 0x61, 0x73, 0x69, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x41, 0x69, 0x72, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x42, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x11b, 0x72, 0x6f, 0x77,
0x61, 0x6e, 0x79, 0x20, 0x73, 0x77, 0x11b, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x61, 0x73, 0x49, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x42, 0x72,
0x69, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x72, 0x65, 0x73, 0x63,
0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x42, 0x72, 0x69, 0x74, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41a, 0x43e, 0x43e, 0x440, 0x434,
0x438, 0x43d, 0x438, 0x440, 0x430, 0x43d, 0x43e, 0x20, 0x443, 0x43d, 0x438, 0x432,
0x435, 0x440, 0x437, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411,
0x440, 0x438, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0xd15, 0xd4b, 0xd7c, 0xd21,
0xd3f, 0xd28, 0xd47, 0xd31, 0xd4d, 0xd31, 0xd21, 0xd4d, 0x20, 0xd2f, 0xd42, 0xd23,
0xd3f, 0xd35, 0xd47, 0xd34, 0xd4d, 0x200c, 0xd38, 0xd7d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd10, 0xd31, 0xd3f, 0xd37, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2c, 0xd4d, 0xd30, 0xd3f, 0xd1f, 0xd4d, 0xd1f, 0xd40, 0xd37, 0xd4d, 0x20, 0xd17,
0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x6e, 0x69,
0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x72,
0x61, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x49, 0x72, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73, 0x68, 0x995,
0x9cb, 0x993, 0x9b0, 0x9cd, 0x9a6, 0x9bf, 0x9a8, 0x9c7, 0x99f, 0x9c7, 0x9a6, 0x20,
0x9af, 0x9bc, 0x9c1, 0x9a8, 0x9bf, 0x9ad, 0x9b0, 0x9cd, 0x9b8, 0x9c7, 0x9b2, 0x20,
0x99f, 0x9be, 0x987, 0x9ae, 0x57, 0x101, 0x20, 0x41, 0x6f, 0x6e, 0x75, 0x69,
0x20, 0x4b, 0x14d, 0x74, 0x75, 0x69, 0x74, 0x75, 0x69, 0x57, 0x101, 0x20,
0x41, 0x69, 0x72, 0x69, 0x68, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x50, 0x69, 0x72, 0x69, 0x74,
0x61, 0x6e, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x906,
0x92f, 0x930, 0x93f, 0x936, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930, 0x93f, 0x91f, 0x93f, 0x936, 0x20, 0x909,
0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x41e, 0x43b,
0x43e, 0x43d, 0x20, 0x443, 0x43b, 0x441, 0x44b, 0x43d, 0x20, 0x437, 0x43e, 0x445,
0x438, 0x446, 0x443, 0x443, 0x43b, 0x430, 0x43b, 0x442, 0x442, 0x430, 0x439, 0x20,
0x446, 0x430, 0x433, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x1823, 0x182f, 0x1820, 0x1828, 0x20, 0x1823,
0x182f, 0x1823, 0x1830, 0x202f, 0x1824, 0x1828, 0x20, 0x1835, 0x1823, 0x182c, 0x1822, 0x1834,
0x1821, 0x182d, 0x1823, 0x182f, 0x1823, 0x182f, 0x1833, 0x1820, 0x1833, 0x1820, 0x1822, 0x20,
0x1834, 0x1820, 0x182d, 0x938, 0x92e, 0x928, 0x94d, 0x935, 0x93f, 0x924, 0x20, 0x935,
0x93f, 0x936, 0x94d, 0x935, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x947, 0x932, 0x93e,
0x92f, 0x924, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x41, 0x72, 0xe9, 0x6e,
0x6a, 0x6d, 0x1eb9, 0x6e, 0x74, 0x20, 0x1ecd, 0x66, 0x20, 0x44, 0x69, 0x20,
0x48, 0xf3, 0x6c, 0x20, 0x57, 0x1ecd, 0x6c, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x41, 0xed, 0x72, 0xed, 0x73, 0x68, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42, 0x72, 0xed, 0x74, 0xed, 0x73,
0x68, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x65, 0x72, 0x65, 0x6a, 0x75, 0x76, 0x76, 0x6f, 0x6e, 0x20, 0x6f, 0x6b,
0x74, 0x61, 0x73, 0x61, 0x161, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x72,
0x6c, 0xe1, 0x6e, 0x64, 0x64, 0x61, 0x6c, 0x61, 0x161, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x72, 0x69, 0x68, 0x74,
0x61, 0x6c, 0x61, 0x161, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x65, 0x72,
0x74, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x62, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x69, 0x72, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x62,
0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x6f, 0x6f, 0x72,
0x64, 0x6f, 0x6e, 0x61, 0x64, 0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x69, 0x72, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x73, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69,
0x63, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72,
0x73, 0x61, 0x75, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61,
0x64, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0xb38,
0xb2e, 0xb28, 0xb4d, 0xb71, 0xb3f, 0xb24, 0x20, 0xb38, 0xb3e, 0xb30, 0xb4d, 0xb2c,
0xb1c, 0xb28, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb07, 0xb30, 0xb3f,
0xb36, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb3f, 0xb1f, 0xb3f, 0xb36, 0xb4d, 0x200c,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40,
0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0x647, 0x645, 0x63a, 0x696, 0x649, 0x20, 0x646,
0x693, 0x6cc, 0x648, 0x627, 0x644, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x64a, 0x631,
0x634, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x628, 0x631, 0x64a, 0x62a, 0x627, 0x646, 0x648, 0x64a, 0x20, 0x62f, 0x20, 0x627,
0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x632, 0x645, 0x627, 0x646, 0x20,
0x647, 0x645, 0x627, 0x647, 0x646, 0x6af, 0x20, 0x62c, 0x647, 0x627, 0x646, 0x6cc,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x6cc, 0x631,
0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x628, 0x631, 0x6cc, 0x62a, 0x627, 0x646, 0x6cc, 0x627,
0x75, 0x6e, 0x69, 0x77, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x6e, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x79, 0x6e,
0x6f, 0x77, 0x61, 0x6e, 0x79, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x42, 0x72, 0x79, 0x74, 0x79, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x55, 0x6e, 0x69, 0x76, 0x65, 0x72,
0x73, 0x61, 0x6c, 0x20, 0x43, 0x6f, 0x6f, 0x72, 0x64, 0x65, 0x6e, 0x61,
0x64, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0xea,
0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x42, 0x72, 0x69, 0x74, 0xe2, 0x6e,
0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x72,
0x64, 0x65, 0x6e, 0x61, 0x64, 0x61, 0x20, 0x55, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x42, 0x72, 0x69, 0x74, 0xe2, 0x6e,
0x69, 0x63, 0x61, 0xa15, 0xa4b, 0xa14, 0xa30, 0xa21, 0xa40, 0xa28, 0xa47, 0xa1f,
0xa47, 0xa21, 0x20, 0xa35, 0xa3f, 0xa06, 0xa2a, 0xa15, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa06, 0xa07, 0xa30, 0xa3f, 0xa38, 0xa3c, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2c, 0xa4d, 0xa30, 0xa3f, 0xa1f, 0xa3f,
0xa38, 0xa3c, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0x54, 0x69, 0x71, 0x73, 0x69, 0x6d, 0x75,
0x79, 0x75, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x54, 0x75, 0x70, 0x61, 0x63,
0x68, 0x69, 0x73, 0x71, 0x61, 0x20, 0x48, 0x6f, 0x72, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x42, 0x72, 0x69, 0x74, 0xe1, 0x6e, 0x69, 0x63, 0x6f, 0x54, 0x69,
0x6d, 0x70, 0x75, 0x6c, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73,
0x61, 0x6c, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x6f, 0x6e, 0x61, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x61, 0x20, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x62, 0x72,
0x69, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x103, 0x54, 0x65, 0x6d, 0x70, 0x20,
0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f,
0x6f, 0x72, 0x64, 0x69, 0x6e, 0xe0, 0x74, 0x65, 0x6d, 0x70, 0x20, 0x64,
0x61, 0x20, 0x73, 0x74, 0x61, 0x64, 0x20, 0x62, 0x72, 0x69, 0x74, 0x61,
0x6e, 0x6e, 0x69, 0x63, 0x412, 0x441, 0x435, 0x43c, 0x438, 0x440, 0x43d, 0x43e,
0x435, 0x20, 0x43a, 0x43e, 0x43e, 0x440, 0x434, 0x438, 0x43d, 0x438, 0x440, 0x43e,
0x432, 0x430, 0x43d, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x412, 0x435, 0x43b, 0x438, 0x43a, 0x43e, 0x431, 0x440, 0x438,
0x442, 0x430, 0x43d, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x938, 0x92e, 0x928, 0x94d, 0x935,
0x93f, 0x924, 0x903, 0x20, 0x935, 0x948, 0x936, 0x94d, 0x935, 0x93f, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x3a, 0x1c60, 0x1c5a, 0x1c68, 0x1c70, 0x1c64, 0x1c71, 0x1c6e, 0x1c74,
0x1c6e, 0x1c70, 0x20, 0x1c6d, 0x1c69, 0x1c71, 0x1c64, 0x1c63, 0x1c5f, 0x1c68, 0x1c65, 0x1c5f,
0x1c5e, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x54, 0x65, 0x6d, 0x70, 0x75, 0x73,
0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x61, 0x64, 0x75, 0x20,
0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x69, 0x72, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x69, 0x61, 0x6c, 0x65, 0x20, 0x62, 0x72, 0x69,
0x74, 0xe0, 0x6e, 0x6e, 0x69, 0x63, 0x61, 0x41a, 0x43e, 0x43e, 0x440, 0x434,
0x438, 0x43d, 0x438, 0x441, 0x430, 0x43d, 0x43e, 0x20, 0x443, 0x43d, 0x438, 0x432,
0x435, 0x440, 0x437, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x440, 0x441, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440,
0x441, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411,
0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x4b,
0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x69, 0x73, 0x61, 0x6e, 0x6f, 0x20,
0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x7a, 0x61, 0x6c, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x73, 0x6b, 0x61,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x69, 0x74,
0x61, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f,
0x6f, 0x72, 0x64, 0x69, 0x6e, 0x69, 0x73, 0x61, 0x6e, 0x6f, 0x20, 0x75,
0x6e, 0x69, 0x76, 0x65, 0x72, 0x7a, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x42, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x6a, 0x61,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x75, 0x6e, 0x69, 0x77, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x6e,
0x79, 0x20, 0x6b, 0x6f, 0x6f, 0x72, 0x64, 0x79, 0x6e, 0x6f, 0x77, 0x61,
0x6e, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x49, 0x72, 0x6c, 0x61, 0x6e,
0x64, 0x79, 0x6a, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x42, 0x72, 0x79, 0x74, 0x79, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x6af, 0x68f, 0x64a, 0x644, 0x20, 0x62f, 0x646, 0x64a, 0x627,
0x648, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x626, 0x631, 0x634, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x628, 0x631, 0x637, 0x627, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x917, 0x926, 0x93f, 0x952, 0x92f, 0x932, 0x20, 0x906, 0x932, 0x92e, 0x940,
0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0xdc3, 0xdb8, 0xd9a, 0xd9a, 0xdca, 0xdc2,
0x20, 0xdc3, 0xdcf, 0xdbb, 0xdca, 0xdc0, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd85, 0xdba, 0xdbb, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xdca, 0x200d, 0xdbb,
0xdd2, 0xdad, 0xdcf, 0xdb1, 0xdca, 0x200d, 0xdba, 0x20, 0xd9c, 0xdd2, 0xdb8, 0xdca,
0xdc4, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0x6b, 0x6f, 0x6f,
0x72, 0x64, 0x69, 0x6e, 0x6f, 0x76, 0x61, 0x6e, 0xfd, 0x20, 0x73, 0x76,
0x65, 0x74, 0x6f, 0x76, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0xed, 0x72, 0x73,
0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x69, 0x74, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x6e,
0x69, 0x76, 0x65, 0x72, 0x7a, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x6b, 0x6f,
0x6f, 0x72, 0x64, 0x69, 0x6e, 0x69, 0x72, 0x61, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72,
0x69, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x49, 0x73, 0x6b, 0x75, 0x2d, 0x78, 0x69, 0x72, 0x61,
0x6e, 0x20, 0x65, 0x65, 0x20, 0x43, 0x61, 0x61, 0x6c, 0x61, 0x6d, 0x6b,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41,
0x79, 0x72, 0x69, 0x73, 0x68, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x42, 0x69, 0x72, 0x69, 0x74, 0x69, 0x73, 0x68, 0x6b, 0x61,
0x74, 0x69, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x75, 0x6e, 0x69, 0x76, 0x65,
0x72, 0x73, 0x61, 0x6c, 0x20, 0x63, 0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e,
0x61, 0x64, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x62, 0x72, 0x69, 0x74, 0xe1,
0x6e, 0x69, 0x63, 0x61, 0x4d, 0x66, 0x75, 0x6d, 0x6f, 0x20, 0x77, 0x61,
0x20, 0x6b, 0x75, 0x72, 0x61, 0x74, 0x69, 0x62, 0x75, 0x20, 0x73, 0x61,
0x61, 0x20, 0x75, 0x6c, 0x69, 0x6d, 0x77, 0x65, 0x6e, 0x67, 0x75, 0x6e,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x79, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55,
0x69, 0x6e, 0x67, 0x65, 0x72, 0x65, 0x7a, 0x61, 0x53, 0x61, 0x61, 0x20,
0x79, 0x61, 0x20, 0x44, 0x75, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x6f, 0x72,
0x64, 0x69, 0x6e, 0x65, 0x72, 0x61, 0x64, 0x20, 0x75, 0x6e, 0x69, 0x76,
0x65, 0x72, 0x73, 0x65, 0x6c, 0x6c, 0x20, 0x74, 0x69, 0x64, 0x69, 0x72,
0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x62, 0x72, 0x69, 0x74, 0x74, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x725, 0x715,
0x722, 0x718, 0x72c, 0x710, 0x20, 0x72c, 0x712, 0x720, 0x71d, 0x72c, 0x710, 0x20,
0x721, 0x71b, 0x71f, 0x718, 0x723, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x71d, 0x72a, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x712, 0x72a, 0x71d,
0x71b, 0x722, 0x71d, 0x710, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x4b7, 0x430,
0x4b3, 0x43e, 0x43d, 0x438, 0x438, 0x20, 0x4b3, 0x430, 0x43c, 0x43e, 0x4b3, 0x430,
0x43d, 0x433, 0x441, 0x43e, 0x437, 0x4e3, 0xb92, 0xbb0, 0xbc1, 0xb99, 0xbcd, 0xb95,
0xbbf, 0xba3, 0xbc8, 0xba8, 0xbcd, 0xba4, 0x20, 0xb9a, 0xbb0, 0xbcd, 0xbb5, 0xba4,
0xbc7, 0xb9a, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb90, 0xbb0, 0xbbf, 0xbb7,
0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb9f, 0xbcd, 0xb9f, 0xbbf, 0xbb7,
0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0x411, 0x4e9, 0x442, 0x435, 0x43d, 0x434, 0x4e9, 0x43d, 0x44c, 0x44f, 0x20, 0x43a,
0x438, 0x43b, 0x435, 0x448, 0x442, 0x435, 0x440, 0x435, 0x43b, 0x433, 0x4d9, 0x43d,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0xc38, 0xc2e, 0xc28, 0xc4d, 0xc35,
0xc2f, 0x20, 0xc38, 0xc3e, 0xc30, 0xc4d, 0xc35, 0xc1c, 0xc28, 0xc40, 0xc28, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc10, 0xc30, 0xc3f, 0xc37, 0xc4d, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2c, 0xc4d, 0xc30, 0xc3f, 0xc1f, 0xc40, 0xc37, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2a,
0xe32, 0xe01, 0xe25, 0xe40, 0xe0a, 0xe34, 0xe07, 0xe1e, 0xe34, 0xe01, 0xe31, 0xe14,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe44,
0xe2d, 0xe23, 0xe4c, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe31, 0xe07, 0xe01, 0xe24,
0xe29, 0x12dd, 0x1270, 0x1233, 0x1290, 0x12e8, 0x20, 0x12a3, 0x12f5, 0x121b, 0x1233, 0x12ca,
0x20, 0x130d, 0x12dc, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1265,
0x122a, 0x1323, 0x1295, 0x12eb, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x65, 0x6d, 0x101, 0x6d, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x65, 0x61, 0x6c, 0x61,
0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x70, 0x69, 0x6c, 0x69, 0x74, 0x101, 0x6e, 0x69, 0x61, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x45, 0x15f, 0x20,
0x47, 0xfc, 0x64, 0xfc, 0x6d, 0x6c, 0xfc, 0x20, 0x45, 0x76, 0x72, 0x65,
0x6e, 0x73, 0x65, 0x6c, 0x20, 0x5a, 0x61, 0x6d, 0x61, 0x6e, 0x130, 0x72,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x6e, 0x67, 0x69,
0x6c, 0x74, 0x65, 0x72, 0x65, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x55, 0x74, 0x67, 0x61, 0x15f, 0x64, 0x79, 0x72, 0x79,
0x6c, 0xfd, 0x61, 0x6e, 0x20, 0xe4, 0x68, 0x6c, 0x69, 0x75, 0x6d, 0x75,
0x6d, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x49, 0x72, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x65, 0xfd, 0x69, 0x6b,
0x20, 0x42, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x69, 0xfd, 0x61, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x437, 0x430, 0x20, 0x432, 0x441, 0x435, 0x441, 0x432, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x43a, 0x43e, 0x43e, 0x440, 0x434, 0x438, 0x43d, 0x43e, 0x432, 0x430,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x432, 0x20, 0x406, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x456, 0x457, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x412, 0x435, 0x43b, 0x438, 0x43a, 0x456, 0x439,
0x20, 0x411, 0x440, 0x438, 0x442, 0x430, 0x43d, 0x456, 0x457, 0x6b, 0x6f, 0x6f,
0x72, 0x64, 0x69, 0x6e, 0x6f, 0x77, 0x61, 0x6e, 0x79, 0x20, 0x73, 0x77,
0x11b, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x42, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6a9, 0x648, 0x622, 0x631, 0x688, 0x6cc,
0x646, 0x6cc, 0x679, 0x688, 0x20, 0x6cc, 0x648, 0x646, 0x6cc, 0x648, 0x631, 0x633,
0x644, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x626, 0x631, 0x634, 0x20, 0x627,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x628, 0x631, 0x679, 0x634, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x626, 0x649, 0x631, 0x6d0, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d5,
0x646, 0x6af, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x4b, 0x6f, 0x6f, 0x72, 0x64,
0x69, 0x6e, 0x61, 0x74, 0x61, 0x6c, 0x69, 0x20, 0x75, 0x6e, 0x69, 0x76,
0x65, 0x72, 0x73, 0x61, 0x6c, 0x20, 0x76, 0x61, 0x71, 0x74, 0x49, 0x72,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x72, 0x69, 0x74, 0x61,
0x6e, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x418, 0x440, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f,
0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x411,
0x440, 0x438, 0x442, 0x430, 0x43d, 0x438, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x68,
0x1ed1, 0x69, 0x20, 0x68, 0x1ee3, 0x70, 0x20, 0x51, 0x75, 0x1ed1, 0x63, 0x20,
0x74, 0x1ebf, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x41, 0x69, 0x2d, 0x6c, 0x65, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x6e, 0x68, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x43, 0x79, 0x66, 0x66, 0x72, 0x65, 0x64, 0x6e, 0x69, 0x6f,
0x6c, 0x20, 0x43, 0x79, 0x64, 0x6c, 0x79, 0x6e, 0x6f, 0x6c, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x49, 0x77, 0x65, 0x72, 0x64, 0x64, 0x6f, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x50, 0x72, 0x79, 0x64, 0x61, 0x69,
0x6e, 0x49, 0x65, 0x72, 0x73, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x42, 0x72, 0x69, 0x74, 0x73, 0x65, 0x20,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x43, 0x55,
0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x69, 0x6e, 0x69,
0x77, 0x65, 0x6c, 0x73, 0x65, 0x6c, 0x20, 0x79, 0x75, 0xf1, 0x20, 0x62,
0x6f, 0x6f, 0x6c, 0x65, 0x29, 0xc0, 0x70, 0x61, 0x70, 0x1ecd, 0x300, 0x20,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x67, 0x62, 0xe1, 0x79, 0xe9,
0xc0, 0x70, 0x61, 0x70, 0x254, 0x300, 0x20, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x67, 0x62, 0xe1, 0x79, 0xe9, 0x69, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x6f, 0x6d, 0x68, 0x6c, 0x61, 0x62,
0x61, 0x20, 0x65, 0x73, 0x69, 0x64, 0x69, 0x64, 0x69, 0x79, 0x65, 0x6c,
0x77, 0x65, 0x69, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x72, 0x69, 0x73, 0x68, 0x20, 0x65,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x69, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42,
0x72, 0x69, 0x74, 0x69, 0x73, 0x68, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x47, 0x61, 0x2d, 0x6b, 0x61, 0x72, 0x20, 0xd3,
0x72, 0x61, 0x20, 0x56, 0x1ebd, 0x6e, 0x68, 0x2d, 0x6b, 0x72, 0xe9, 0x6e,
0x2d, 0x6a, 0x61, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x49, 0x72,
0x1ef9, 0x6e, 0x61, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x72, 0x69, 0x74, 0x1ef9,
0x6e, 0x129, 0x6b, 0x75, 0x20, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x4d, 0x169, 0x64, 0x69, 0x20,
0x74, 0x75, 0x72, 0x75, 0x73, 0x75, 0x73, 0x61, 0x77, 0x61, 0x20, 0x4b,
0x75, 0x72, 0x64, 0x65, 0x6e, 0x61, 0x64, 0x75, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x20,
0x49, 0x72, 0xe3, 0x64, 0x65, 0x69, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x42, 0x69, 0x72, 0x69, 0x74, 0xe3, 0x6e, 0x69, 0x6b, 0x75,
0x55, 0x66, 0x73, 0x74, 0x65, 0x6d, 0x65, 0x74, 0x20, 0x57, 0xe4, 0xe4,
0x6c, 0x74, 0x74, 0x69, 0x64, 0x6a, 0x49, 0x69, 0x72, 0x73, 0x6b, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x42, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x20,
0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x65, 0x20, 0x63,
0x6f, 0x6f, 0x72, 0x64, 0x69, 0x6e, 0x6f, 0x75, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x62, 0x72, 0x69, 0x74, 0x61, 0x6e, 0x6e, 0x69, 0x63, 0x61, 0x189,
0x75, 0x6c, 0x69, 0x6e, 0x79, 0x61, 0x20, 0x61, 0x14b, 0x75, 0x6e, 0x69,
0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x49, 0x72, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x67, 0x25b, 0x25b,
0x73, 0x68, 0x269, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a,
0x62, 0x61, 0x6b, 0x61, 0x938, 0x92e, 0x928, 0x94d, 0x935, 0x93f, 0x924, 0x20,
0x926, 0x941, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x906, 0x907, 0x930, 0x93f, 0x936, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x94d, 0x930, 0x93f, 0x91f, 0x93f,
0x936, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x4f, 0x72, 0x61, 0x20, 0x75,
0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x65, 0x20, 0x63, 0x6f,
0x6f, 0x72, 0x64, 0x65, 0x6e, 0x61, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x69, 0x72, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x7a, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x62, 0x72, 0x69, 0x74, 0xe0, 0x6e, 0x65, 0x67, 0x61,
0x73, 0x6d, 0x61, 0x6e, 0x69, 0x20, 0x6a, 0x61, 0x67, 0x61, 0x20, 0x70,
0x72, 0x75, 0x74, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x61, 0x69,
0x72, 0x69, 0x73, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x62, 0x72, 0x69, 0x1e6d, 0x69, 0x73, 0x20, 0x6b, 0x61,
0x72, 0xe3, 0x20, 0x6d, 0x61, 0x73, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101
};

static inline constexpr char16_t shortMetaZoneNameTable[] = {
0x41, 0x4b, 0x54, 0x41, 0x4b, 0x53, 0x54, 0x41, 0x4b, 0x44, 0x54, 0x43,
0x54, 0x43, 0x53, 0x54, 0x43, 0x44, 0x54, 0x45, 0x54, 0x45, 0x53, 0x54,
0x45, 0x44, 0x54, 0x4d, 0x54, 0x4d, 0x53, 0x54, 0x4d, 0x44, 0x54, 0x50,
0x54, 0x50, 0x53, 0x54, 0x50, 0x44, 0x54, 0x41, 0x54, 0x41, 0x53, 0x54,
0x41, 0x44, 0x54, 0x47, 0x4d, 0x54, 0x48, 0x41, 0x54, 0x48, 0x41, 0x53,
0x54, 0x48, 0x41, 0x44, 0x54, 0x43, 0x41, 0x54, 0x45, 0x41, 0x54, 0x53,
0x41, 0x53, 0x54, 0x57, 0x41, 0x54, 0x57, 0x41, 0x53, 0x54, 0x47, 0x53,
0x54, 0x9ad, 0x9be, 0x2e, 0x20, 0x9b8, 0x2e, 0x43, 0x45, 0x54, 0x43, 0x45,
0x53, 0x54, 0x45, 0x45, 0x54, 0x45, 0x45, 0x53, 0x54, 0x57, 0x45, 0x54,
0x57, 0x45, 0x53, 0x54, 0x49, 0x53, 0x54, 0x4d, 0x59, 0x54, 0x53, 0x47,
0x54, 0x4d, 0x45, 0x5a, 0x4d, 0x45, 0x53, 0x5a, 0x4f, 0x45, 0x5a, 0x4f,
0x45, 0x53, 0x5a, 0x57, 0x45, 0x5a, 0x57, 0x45, 0x53, 0x5a, 0x53, 0x45,
0x10c, 0x53, 0x45, 0x4c, 0x10c, 0x53, 0x52, 0x54, 0x42, 0x54, 0x2205, 0x2205,
0x2205, 0x41, 0x43, 0x54, 0x41, 0x43, 0x53, 0x54, 0x41, 0x43, 0x44, 0x54,
0x41, 0x43, 0x57, 0x54, 0x41, 0x43, 0x57, 0x53, 0x54, 0x41, 0x43, 0x57,
0x44, 0x54, 0x41, 0x45, 0x54, 0x41, 0x45, 0x53, 0x54, 0x41, 0x45, 0x44,
0x54, 0x41, 0x57, 0x54, 0x41, 0x57, 0x53, 0x54, 0x41, 0x57, 0x44, 0x54,
0x47, 0x75, 0x6c, 0x66, 0x20, 0x53, 0x54, 0x4c, 0x48, 0x54, 0x4c, 0x48,
0x53, 0x54, 0x4c, 0x48, 0x44, 0x54, 0x4e, 0x5a, 0x54, 0x4e, 0x5a, 0x53,
0x54, 0x4e, 0x5a, 0x44, 0x54, 0x41, 0x46, 0x54, 0x41, 0x52, 0x54, 0x42,
0x53, 0x54, 0x42, 0x54, 0x54, 0x42, 0x52, 0x54, 0x42, 0x52, 0x53, 0x54,
0x42, 0x4e, 0x54, 0x43, 0x48, 0x41, 0x53, 0x54, 0x43, 0x48, 0x41, 0x44,
0x54, 0x43, 0x58, 0x54, 0x43, 0x43, 0x54, 0x43, 0x4f, 0x53, 0x54, 0x54,
0x4c, 0x54, 0x45, 0x41, 0x53, 0x54, 0x45, 0x41, 0x53, 0x53, 0x54, 0x45,
0x43, 0x54, 0x46, 0x4b, 0x54, 0x46, 0x4b, 0x53, 0x54, 0x47, 0x41, 0x4c,
0x54, 0x45, 0x47, 0x54, 0x47, 0x59, 0x54, 0x49, 0x43, 0x54, 0x57, 0x49,
0x54, 0x41, 0x57, 0x49, 0x42, 0x49, 0x52, 0x53, 0x54, 0x49, 0x52, 0x44,
0x54, 0x4d, 0x56, 0x54, 0x4e, 0x50, 0x54, 0x4e, 0x53, 0x54, 0x4e, 0x44,
0x54, 0x46, 0x4e, 0x54, 0x50, 0x4b, 0x54, 0x50, 0x59, 0x54, 0x50, 0x59,
0x53, 0x54, 0x50, 0x45, 0x54, 0x50, 0x4d, 0x54, 0x50, 0x4d, 0x53, 0x54,
0x50, 0x4d, 0x44, 0x54, 0x55, 0x59, 0x54, 0x55, 0x59, 0x53, 0x54, 0x56,
0x45, 0x54, 0x43, 0x68, 0x53, 0x54, 0x48, 0x4b, 0x54, 0x48, 0x4b, 0x53,
0x54, 0x43, 0x48, 0x41, 0x54, 0x48, 0x41, 0x4b, 0x48, 0x4e, 0x41, 0x4b,
0x48, 0x45, 0x41, 0x4b, 0x48, 0x43, 0x48, 0x4e, 0x43, 0x48, 0x45, 0x43,
0x48, 0x4e, 0x45, 0x48, 0x45, 0x45, 0x48, 0x52, 0x48, 0x4e, 0x52, 0x48,
0x45, 0x52, 0x48, 0x50, 0x48, 0x4e, 0x50, 0x48, 0x45, 0x50, 0x48, 0x43,
0x55, 0x48, 0x4e, 0x43, 0x55, 0x48, 0x45, 0x43, 0x55, 0x45, 0x45, 0x44,
0x54, 0x57, 0x45, 0x44, 0x54, 0x48, 0x45, 0x47, 0x48, 0x4e, 0x45, 0x47,
0x48, 0x45, 0x45, 0x47, 0x48, 0x4f, 0x47, 0x48, 0x4e, 0x4f, 0x47, 0x48,
0x45, 0x4f, 0x47, 0x48, 0x48, 0x41, 0x48, 0x4e, 0x48, 0x41, 0x48, 0x45,
0x48, 0x41, 0x48, 0x50, 0x4d, 0x58, 0x48, 0x4e, 0x50, 0x4d, 0x58, 0x48,
0x45, 0x50, 0x4d, 0x58, 0x48, 0x54, 0x4e, 0x48, 0x4e, 0x54, 0x4e, 0x48,
0x45, 0x54, 0x4e, 0x48, 0x41, 0x43, 0x48, 0x41, 0x45, 0x48, 0x41, 0x52,
0x48, 0x41, 0x50, 0x47, 0x46, 0x54, 0x43, 0x54, 0x20, 0x28, 0x4b, 0x75,
0x62, 0x61, 0x29, 0x43, 0x53, 0x54, 0x20, 0x28, 0x4b, 0x75, 0x62, 0x61,
0x29, 0x43, 0x44, 0x54, 0x20, 0x28, 0x4b, 0x75, 0x62, 0x61, 0x29, 0x45,
0x47, 0x53, 0x54, 0x45, 0x47, 0x44, 0x54, 0x41, 0x41, 0x43, 0x41, 0x43,
0x41, 0x43, 0x41, 0x53, 0x41, 0x43, 0x4d, 0x41, 0x47, 0x4a, 0x53, 0x54,
0x4a, 0x44, 0x54, 0x53, 0x41, 0x4b, 0x53, 0x53, 0x41, 0x4b, 0x53, 0x48,
0x41, 0x4b, 0x53, 0x4e, 0x53, 0x53, 0x4e, 0x53, 0x48, 0x4e, 0x53, 0x53,
0x52, 0x53, 0x48, 0x52, 0x53, 0xc7, 0x53, 0x53, 0xc7, 0x53, 0x48, 0xc7,
0x53, 0x50, 0x53, 0x53, 0x50, 0x53, 0x48, 0x50, 0x53, 0x4b, 0x53, 0x53,
0x4b, 0x53, 0x48, 0x4b, 0x53, 0x47, 0x52, 0x53, 0x53, 0x47, 0x52, 0x53,
0x48, 0x47, 0x52, 0x53, 0x47, 0x52, 0x4f, 0x53, 0x53, 0x47, 0x52, 0x4f,
0x53, 0x48, 0x47, 0x52, 0x4f, 0x53, 0x48, 0x41, 0x4c, 0x53, 0x53, 0x48,
0x41, 0x53, 0x48, 0x48, 0x41, 0x53, 0x4e, 0x46, 0x53, 0x53, 0x4e, 0x46,
0x53, 0x48, 0x4e, 0x46, 0x53, 0x59, 0x41, 0x4d, 0x54, 0x41, 0x4d, 0x53,
0x54, 0x41, 0x5a, 0x4f, 0x54, 0x41, 0x5a, 0x4f, 0x53, 0x54, 0x50, 0x45,
0x53, 0x54, 0x42, 0x4f, 0x54, 0x4f, 0x41, 0x4b, 0x4f, 0x49, 0x41, 0x4b,
0x4f, 0x4c, 0x41, 0x4b, 0x4f, 0x49, 0x54, 0x4f, 0x4c, 0x54, 0x4f, 0x4f,
0x4f, 0x49, 0x4f, 0x4f, 0x4c, 0x4f, 0x4f, 0x4d, 0x50, 0x4f, 0x49, 0x4d,
0x50, 0x4f, 0x4c, 0x4d, 0x50, 0x4f, 0x50, 0x4f, 0x49, 0x50, 0x4f, 0x4c,
0x50, 0x4f, 0x4d, 0x47, 0x41, 0x52, 0x53, 0x54, 0x57, 0x41, 0x52, 0x54,
0x57, 0x41, 0x52, 0x53, 0x54, 0x43, 0x4c, 0x54, 0x43, 0x4c, 0x53, 0x54,
0x43, 0x4f, 0x54, 0x4a, 0x43, 0x41, 0x53, 0x4a, 0x43, 0x41, 0x4a, 0x43,
0x41, 0x4b, 0x52, 0x4a, 0x54, 0x41, 0x53, 0x4a, 0x54, 0x41, 0x4a, 0x54,
0x41, 0x4b, 0x52, 0x4a, 0x59, 0x41, 0x53, 0x4a, 0x59, 0x41, 0x4a, 0x48,
0x59, 0x41, 0x53, 0x4a, 0x41, 0x44, 0x4a, 0x41, 0x44, 0x4b, 0x52, 0x4a,
0x59, 0x54, 0x53, 0x4a, 0x59, 0x54, 0x4a, 0x59, 0x54, 0x4b, 0x52, 0x4a,
0x43, 0x4b, 0x52, 0x4a, 0x54, 0x4b, 0x52, 0x4a, 0x53, 0x4b, 0x52, 0x4a,
0x51, 0x47, 0x47, 0x6b
};

static inline constexpr char16_t longMetaZoneNameTable[] = {
0x41, 0x63, 0x72, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x63, 0x72,
0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x63, 0x72, 0x65, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x66, 0x67, 0x68, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x6f,
0x75, 0x74, 0x68, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x75,
0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f,
0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x75, 0x6e,
0x74, 0x61, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x41, 0x70, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x70,
0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x61, 0x20, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x71,
0x74, 0x61, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x71, 0x74, 0x61,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x71, 0x74, 0x61, 0x75, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x71, 0x74, 0x6f, 0x62,
0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57,
0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42,
0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x61, 0x73, 0x65, 0x79, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74,
0x6d, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6f,
0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43,
0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20,
0x48, 0x61, 0x6c, 0x66, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x44, 0x61, 0x76, 0x69, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x63, 0x75,
0x61, 0x64, 0x6f, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61,
0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46,
0x75, 0x72, 0x74, 0x68, 0x65, 0x72, 0x2d, 0x65, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x61, 0x6e, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x69, 0x6a,
0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61,
0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x72, 0x65, 0x6e, 0x63,
0x68, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x65, 0x72, 0x6e, 0x20, 0x26,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69,
0x63, 0x68, 0x20, 0x4d, 0x65, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61,
0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73,
0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57,
0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65,
0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x47, 0x75, 0x61, 0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x47, 0x75, 0x6c, 0x66, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x48, 0x6f, 0x76, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x20,
0x4f, 0x63, 0x65, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x6e,
0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x72, 0x61,
0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x7a, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x63,
0x61, 0x6f, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x63, 0x61, 0x6f,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d,
0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x63, 0x6f,
0x77, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x61, 0x75, 0x72,
0x75, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65,
0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65,
0x61, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x5a,
0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72,
0x74, 0x68, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x65,
0x72, 0x75, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x65, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x52,
0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x65, 0x79, 0x63, 0x68,
0x65, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x6f, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x61, 0x6c,
0x6c, 0x69, 0x73, 0x20, 0x26, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x410, 0x43a, 0x440, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x440, 0x438, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x410, 0x43a, 0x440, 0x438, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x444, 0x433, 0x430, 0x43d,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x41c, 0x440, 0x430,
0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x410, 0x43b, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438,
0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430,
0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x2c,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9,
0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43b, 0x438, 0x430, 0x441, 0x43a, 0x430,
0x410, 0x43b, 0x438, 0x430, 0x441, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x410, 0x43b, 0x438, 0x430, 0x441, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442, 0x430, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442, 0x430, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442, 0x430, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41c, 0x440, 0x430,
0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c,
0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20,
0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410,
0x448, 0x44c, 0x445, 0x430, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x20, 0x28, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x410, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x448, 0x44c, 0x445, 0x430, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x20, 0x28, 0x410, 0x4a9, 0x430,
0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x29, 0x410, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x448, 0x44c, 0x445, 0x430, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x20, 0x28, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x410, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x4ad, 0x44b, 0x43d, 0x447, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x4ad, 0x44b, 0x43d, 0x447,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43e, 0x43a, 0x435,
0x430, 0x43d, 0x4ad, 0x44b, 0x43d, 0x447, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x20, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440,
0x20, 0x430, 0x4bf, 0x44b, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x410, 0x43f, 0x438, 0x430, 0x410, 0x43f, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x43f, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a,
0x442, 0x430, 0x443, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x442,
0x430, 0x443, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x442, 0x430,
0x443, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x421, 0x430, 0x443, 0x434, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x440, 0x430, 0x431,
0x441, 0x4ad, 0x430, 0x43d, 0x421, 0x430, 0x443, 0x434, 0x442, 0x4d9, 0x438, 0x20,
0x410, 0x440, 0x430, 0x431, 0x441, 0x4ad, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x421, 0x430, 0x443, 0x434, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x440,
0x430, 0x431, 0x441, 0x4ad, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c,
0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x41c, 0x440,
0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x415, 0x440, 0x43c, 0x430, 0x43d, 0x442,
0x4d9, 0x44b, 0x43b, 0x430, 0x415, 0x440, 0x43c, 0x430, 0x43d, 0x442, 0x4d9, 0x44b,
0x43b, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x415, 0x440, 0x43c, 0x430,
0x43d, 0x442, 0x4d9, 0x44b, 0x43b, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x442, 0x43b,
0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x433, 0x4d9, 0x4ad,
0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x430, 0x2c, 0x20, 0x43c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430,
0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410,
0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x43c, 0x440, 0x430, 0x4ad,
0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x43c,
0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430,
0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430,
0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430,
0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448,
0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c,
0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x430, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x438, 0x45f, 0x44c, 0x430,
0x43d, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x438, 0x45f, 0x44c, 0x430, 0x43d,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430,
0x438, 0x45f, 0x44c, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x437, 0x43e,
0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9,
0x430, 0x410, 0x437, 0x43e, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d,
0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410,
0x437, 0x43e, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430,
0x49b, 0x4d9, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x411, 0x430, 0x43d, 0x433, 0x43b,
0x430, 0x434, 0x435, 0x448, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435,
0x448, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x411, 0x430, 0x43d, 0x433, 0x43b,
0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x411, 0x443, 0x442, 0x430,
0x43d, 0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438, 0x430, 0x411, 0x440, 0x430, 0x437,
0x438, 0x43b, 0x438, 0x430, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x430,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x438,
0x2d, 0x414, 0x430, 0x440, 0x443, 0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x41a,
0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x41a, 0x430, 0x431,
0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41a, 0x435, 0x438, 0x441, 0x438, 0x427, 0x430, 0x43c, 0x43e,
0x440, 0x440, 0x43e, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x427, 0x430, 0x442, 0x435,
0x43c, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x427, 0x430, 0x442, 0x435, 0x43c,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x427, 0x438, 0x43b, 0x438, 0x427, 0x438, 0x43b, 0x438,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x427, 0x438, 0x43b, 0x438, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x41a, 0x438, 0x442, 0x430, 0x438, 0x41a, 0x438, 0x442, 0x430, 0x438,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x438, 0x442, 0x430, 0x438, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x427, 0x43e, 0x438, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d,
0x427, 0x43e, 0x438, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x427, 0x43e, 0x438, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20, 0x49a, 0x44c,
0x44b, 0x440, 0x441, 0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x438, 0x430, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438,
0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x43e, 0x43b, 0x443, 0x43c,
0x431, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x443, 0x43a, 0x20, 0x438,
0x434, 0x433, 0x44c, 0x44b, 0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b,
0x4d9, 0x430, 0x41a, 0x443, 0x43a, 0x20, 0x438, 0x434, 0x433, 0x44c, 0x44b, 0x43b,
0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41a, 0x443, 0x43a, 0x20, 0x438, 0x434, 0x433, 0x44c, 0x44b,
0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x2c, 0x20,
0x43f, 0x43e, 0x43b, 0x443, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x443, 0x431, 0x430, 0x41a, 0x443,
0x431, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x443, 0x431, 0x430,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x414, 0x435, 0x438, 0x432, 0x438, 0x441, 0x414, 0x438,
0x443, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x418, 0x443, 0x440, 0x432, 0x438,
0x43b, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x430, 0x434, 0x2d, 0x445, 0x430,
0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x20,
0x41f, 0x430, 0x441, 0x445, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x430,
0x434, 0x2d, 0x445, 0x430, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x415, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x410, 0x433, 0x4d9,
0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x410, 0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410,
0x433, 0x4d9, 0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b,
0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442,
0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430,
0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x430, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448,
0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430,
0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430,
0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430,
0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x424, 0x438, 0x45f, 0x438, 0x424, 0x438, 0x45f, 0x438, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x438, 0x45f, 0x438, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x424, 0x440, 0x430, 0x43d, 0x446, 0x44b, 0x437, 0x442, 0x4d9, 0x438, 0x20, 0x413,
0x432, 0x438, 0x430, 0x43d, 0x430, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x44b, 0x437,
0x442, 0x4d9, 0x438, 0x20, 0x410, 0x43b, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x435,
0x438, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x43a, 0x430,
0x442, 0x4d9, 0x438, 0x20, 0x440, 0x4b5, 0x430, 0x43a, 0x44b, 0x440, 0x430, 0x434,
0x433, 0x44c, 0x44b, 0x43b, 0x49b, 0x4d9, 0x430, 0x413, 0x430, 0x43b, 0x430, 0x43f,
0x430, 0x433, 0x43e, 0x441, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x2d, 0x445,
0x430, 0x49b, 0x4d9, 0x430, 0x413, 0x430, 0x43c, 0x431, 0x435, 0x49a, 0x44b, 0x440,
0x4ad, 0x442, 0x4d9, 0x44b, 0x43b, 0x430, 0x49a, 0x44b, 0x440, 0x4ad, 0x442, 0x4d9,
0x44b, 0x43b, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x49a, 0x44b, 0x440,
0x4ad, 0x442, 0x4d9, 0x44b, 0x43b, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x430, 0x434,
0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x20, 0x413, 0x438, 0x43b, 0x431, 0x435,
0x440, 0x442, 0x430, 0x410, 0x433, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x20,
0x418, 0x431, 0x436, 0x44c, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x443, 0x20, 0x410,
0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430,
0x442, 0x4d9, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x438, 0x430, 0x41c, 0x440, 0x430,
0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430,
0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9,
0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448,
0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x413, 0x443,
0x430, 0x43c, 0x410, 0x45f, 0x44c, 0x430, 0x43c, 0x442, 0x4d9, 0x20, 0x430, 0x4e1,
0x44b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x43b, 0x430, 0x413, 0x430, 0x438,
0x430, 0x43d, 0x430, 0x4b2, 0x430, 0x432, 0x430, 0x438, 0x2d, 0x430, 0x43b, 0x435,
0x443, 0x442, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x4b2,
0x430, 0x432, 0x430, 0x438, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x4b2, 0x430, 0x432, 0x430, 0x438, 0x2d,
0x430, 0x43b, 0x435, 0x443, 0x442, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x4b2,
0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x4b2, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d,
0x433, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x4b2, 0x43e, 0x43d, 0x43a, 0x43e,
0x43d, 0x433, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x425, 0x43e, 0x432, 0x434, 0x425, 0x43e,
0x432, 0x434, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x425, 0x43e, 0x432, 0x434,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x43d, 0x434, 0x438, 0x430, 0x418, 0x43d, 0x434,
0x438, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x418, 0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x442, 0x430, 0x438, 0x410, 0x433, 0x4d9,
0x4ad, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d,
0x435, 0x437, 0x438, 0x430, 0x41c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440,
0x430, 0x442, 0x4d9, 0x438, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437,
0x438, 0x430, 0x41c, 0x440, 0x430, 0x4ad, 0x430, 0x448, 0x4d9, 0x430, 0x440, 0x430,
0x442, 0x4d9, 0x438, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438,
0x430, 0x418, 0x440, 0x430, 0x43d, 0x418, 0x440, 0x430, 0x43d, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x418, 0x440, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418,
0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x440, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b,
0x44c, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x418, 0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x430, 0x418, 0x430, 0x43f, 0x43e,
0x43d, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x418, 0x430, 0x43f,
0x43e, 0x43d, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f, 0x435, 0x442, 0x440,
0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430,
0x43c, 0x447, 0x430, 0x442, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x41f, 0x435, 0x442,
0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a,
0x430, 0x43c, 0x447, 0x430, 0x442, 0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432,
0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442,
0x43a, 0x430, 0x442, 0x4d9, 0x438, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x440, 0x430,
0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x49e, 0x430,
0x437, 0x430, 0x445, 0x441, 0x4ad, 0x430, 0x43d, 0x41c, 0x440, 0x430, 0x4ad, 0x430,
0x448, 0x4d9, 0x430, 0x440, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x49e, 0x430, 0x437,
0x430, 0x445, 0x441, 0x4ad, 0x430, 0x43d, 0x41a, 0x43e, 0x440, 0x435, 0x438, 0x430,
0x41a, 0x43e, 0x440, 0x435, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41a, 0x43e, 0x440, 0x435, 0x438, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x43e,
0x441, 0x440, 0x430, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x438, 0x430,
0x440, 0x441, 0x43a, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x438, 0x430, 0x440,
0x441, 0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x438, 0x430, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a,
0x438, 0x440, 0x433, 0x438, 0x437, 0x438, 0x430, 0x428, 0x440, 0x438, 0x2d, 0x41b,
0x430, 0x43d, 0x43a, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430,
0x20, 0x41b, 0x430, 0x438, 0x43d, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x4b2, 0x430,
0x443, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x4b2, 0x430, 0x443, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x4b2, 0x430, 0x443, 0x2c,
0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x41c, 0x430, 0x43a, 0x430,
0x43e, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x430, 0x43a, 0x430, 0x43e,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c,
0x430, 0x43b, 0x430, 0x438, 0x437, 0x438, 0x430, 0x41c, 0x430, 0x43b, 0x44c, 0x434,
0x438, 0x432, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41c, 0x430, 0x440, 0x448,
0x430, 0x43b, 0x43b, 0x442, 0x4d9, 0x438, 0x20, 0x410, 0x434, 0x433, 0x44c, 0x44b,
0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41c, 0x430,
0x432, 0x440, 0x438, 0x43a, 0x438, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a,
0x438, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x410,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x4ad, 0x44b, 0x43d, 0x447, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x43c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x4ad,
0x44b, 0x43d, 0x447, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x43c, 0x435, 0x43a, 0x441,
0x438, 0x43a, 0x430, 0x442, 0x4d9, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x4ad, 0x44b, 0x43d, 0x447, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x43c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x442, 0x4d9, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x423,
0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e,
0x440, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x41c,
0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c,
0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41c, 0x438, 0x430,
0x43d, 0x43c, 0x430, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x41d, 0x435, 0x43f, 0x430,
0x43b, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x430, 0x20, 0x4bf,
0x44b, 0x446, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x430, 0x20,
0x4bf, 0x44b, 0x446, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x430, 0x20, 0x4bf, 0x44b, 0x446, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430, 0x20, 0x4bf,
0x44b, 0x446, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x430, 0x20, 0x4bf,
0x44b, 0x446, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x417, 0x435, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x430, 0x20, 0x4bf, 0x44b, 0x446, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41d, 0x438, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x41d, 0x438, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41d, 0x438, 0x443, 0x444, 0x430, 0x443,
0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41d, 0x438,
0x443, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41d, 0x43e,
0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x435, 0x440,
0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440,
0x43e, 0x43d, 0x438, 0x430, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443,
0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x438, 0x430, 0x2c,
0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20,
0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x438, 0x430,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x410, 0x4a9, 0x430, 0x434, 0x430, 0x442, 0x4d9, 0x438,
0x20, 0x41c, 0x430, 0x440, 0x438, 0x430, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x41e, 0x43c, 0x441, 0x43a, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x41f, 0x430, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f, 0x430,
0x43b, 0x430, 0x443, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20, 0x2014, 0x20, 0x413,
0x432, 0x438, 0x43d, 0x435, 0x438, 0x430, 0x20, 0x4bf, 0x44b, 0x446, 0x41f, 0x430,
0x440, 0x430, 0x433, 0x432, 0x430, 0x438, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432,
0x430, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f, 0x430, 0x440, 0x430,
0x433, 0x432, 0x430, 0x438, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442,
0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f, 0x435, 0x440, 0x443,
0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f, 0x435,
0x440, 0x443, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f,
0x438, 0x43d, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x2c, 0x20,
0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x430, 0x434, 0x2d, 0x445, 0x430, 0x49b, 0x4d9, 0x430,
0x20, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x421, 0x435, 0x43d, 0x2d, 0x41f,
0x438, 0x435, 0x440, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d,
0x438, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x438, 0x435, 0x440, 0x438, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x438, 0x435, 0x440, 0x438, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x438, 0x2c, 0x20, 0x430, 0x525, 0x445,
0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x41f,
0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x41f, 0x43e, 0x43d, 0x43f, 0x435, 0x438,
0x41f, 0x445, 0x435, 0x43d, 0x438, 0x430, 0x43d, 0x41a, 0x44b, 0x437, 0x44b, 0x43b,
0x43e, 0x440, 0x434, 0x430, 0x2a, 0x41a, 0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440,
0x434, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x2a, 0x41a, 0x44b, 0x437,
0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x2a, 0x420,
0x435, 0x438, 0x443, 0x43d, 0x43e, 0x43d, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430,
0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x20, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x4bf, 0x44b, 0x421, 0x430, 0x43c,
0x430, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x421, 0x430,
0x43c, 0x430, 0x440, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x421, 0x430, 0x43c,
0x43e, 0x430, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x421, 0x435,
0x438, 0x448, 0x435, 0x43b, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x433, 0x44c,
0x44b, 0x43b, 0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x421,
0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x421, 0x43e, 0x43b, 0x43e, 0x43c,
0x43e, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x434, 0x433, 0x44c, 0x44b, 0x43b,
0x431, 0x436, 0x44c, 0x430, 0x445, 0x430, 0x49b, 0x4d9, 0x430, 0x410, 0x43b, 0x430,
0x434, 0x430, 0x442, 0x4d9, 0x438, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x438,
0x430, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x421, 0x438, 0x43e, 0x432,
0x430, 0x422, 0x430, 0x438, 0x442, 0x438, 0x422, 0x430, 0x438, 0x432, 0x430, 0x43d,
0x422, 0x430, 0x438, 0x432, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x422, 0x430, 0x438, 0x432, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b,
0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x422, 0x430,
0x45f, 0x44c, 0x44b, 0x49b, 0x441, 0x4ad, 0x430, 0x43d, 0x422, 0x43e, 0x43a, 0x435,
0x43b, 0x430, 0x443, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x422, 0x43e, 0x43d, 0x433,
0x430, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x422, 0x43e, 0x43d, 0x433, 0x430,
0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430,
0x430, 0x43c, 0x4ad, 0x430, 0x422, 0x440, 0x443, 0x43a, 0x422, 0x443, 0x440, 0x43a,
0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x422, 0x443, 0x440, 0x43a,
0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438,
0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x438, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x438, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x423, 0x440, 0x443, 0x433,
0x432, 0x430, 0x438, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9,
0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x423, 0x437, 0x431, 0x435, 0x49b,
0x44c, 0x438, 0x441, 0x4ad, 0x430, 0x43d, 0x423, 0x437, 0x431, 0x435, 0x49b, 0x44c,
0x438, 0x441, 0x4ad, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x423,
0x437, 0x431, 0x435, 0x49b, 0x44c, 0x438, 0x441, 0x4ad, 0x430, 0x43d, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x412, 0x430, 0x43d,
0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x430, 0x525, 0x445, 0x44b, 0x43d,
0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x412, 0x435, 0x43d,
0x435, 0x441, 0x443, 0x435, 0x43b, 0x430, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e,
0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x412, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x430,
0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x412, 0x43e,
0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c, 0x20, 0x430, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad,
0x430, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x410, 0x43c, 0x440, 0x430, 0x433, 0x44b, 0x43b, 0x430, 0x440, 0x430,
0x423, 0x435, 0x438, 0x43a, 0x423, 0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x438, 0x20,
0x424, 0x443, 0x442, 0x443, 0x43d, 0x435, 0x438, 0x418, 0x430, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x418, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x430,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x4d9, 0x20, 0x430, 0x430,
0x43c, 0x4ad, 0x430, 0x418, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20,
0x430, 0x525, 0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c,
0x4ad, 0x430, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443,
0x440, 0x433, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443,
0x440, 0x433, 0x2c, 0x20, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x4d9, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430, 0x415, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x430, 0x525,
0x445, 0x44b, 0x43d, 0x442, 0x4d9, 0x438, 0x20, 0x430, 0x430, 0x43c, 0x4ad, 0x430,
0x418, 0x443, 0x43a, 0x43e, 0x43d, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x74,
0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x2d, 0x74, 0x79, 0x64, 0x53, 0x75, 0x69, 0x64, 0x2d, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d,
0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x41, 0x6d, 0x61,
0x73, 0x6f, 0x6e, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x41, 0x6d, 0x61, 0x73,
0x6f, 0x6e, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x65, 0x2d,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72,
0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x74,
0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x61, 0x67, 0x6c,
0x69, 0x67, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x6f,
0x6f, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x74, 0x79, 0x64,
0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x6f, 0x6f, 0x73, 0x74, 0x65, 0x6c,
0x69, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x6f, 0x6f,
0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x64, 0x61, 0x67, 0x6c,
0x69, 0x67, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x62,
0x65, 0x72, 0x67, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x62, 0x65, 0x72, 0x67, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x2d, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x62,
0x65, 0x72, 0x67, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79,
0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74,
0x79, 0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x65, 0x73, 0x65, 0x20, 0x64, 0x61,
0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x2d, 0x74, 0x79, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x79, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x41,
0x70, 0x69, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x64, 0x61, 0x67,
0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69, 0x65,
0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69, 0x65,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x79, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69, 0x65, 0x73, 0x65, 0x20,
0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64, 0x57,
0x65, 0x73, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69,
0xeb, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79,
0x64, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74,
0x79, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x65, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x65, 0x73, 0x65, 0x20,
0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65,
0x73, 0x65, 0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64,
0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x77, 0x65, 0x73,
0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x53, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x77, 0x65, 0x73, 0x74, 0x65,
0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x2d, 0x77, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65,
0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4f, 0x6f,
0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x4f,
0x6f, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73,
0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x65, 0x73, 0x65, 0x20, 0x64, 0x61, 0x67, 0x6c, 0x69,
0x67, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b,
0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73,
0x65, 0x20, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69,
0x6b, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6b, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x73, 0x65,
0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x41, 0x73,
0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x2d, 0x74, 0x79,
0x64, 0x41, 0x73, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x6a, 0x61, 0x6e,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x41, 0x73, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x6a, 0x61, 0x6e,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x41, 0x73, 0x6f,
0x72, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x41, 0x73, 0x6f, 0x72, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x6a,
0x2d, 0x74, 0x79, 0x64, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x6a, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x79, 0x64, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x6a, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x42, 0x68,
0x6f, 0x65, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x42, 0x6f, 0x6c,
0x69, 0x76, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x42, 0x72,
0x6f, 0x65, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x2d, 0x74, 0x79, 0x64, 0x4b, 0x61, 0x61, 0x70,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x4b, 0x61,
0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4b, 0x61, 0x61,
0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x74, 0x79, 0x64,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79,
0x64, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x2d, 0x74, 0x79, 0x64, 0x43, 0x68,
0x69, 0x6c, 0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x2d, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x2d,
0x74, 0x79, 0x64, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x43, 0x68, 0x69,
0x6e, 0x61, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64,
0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x2d, 0x74,
0x79, 0x64, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x2d,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x43, 0x68, 0x72, 0x69,
0x73, 0x74, 0x6d, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d,
0x74, 0x79, 0x64, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0xeb, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x43, 0x6f, 0x6f,
0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64,
0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d,
0x68, 0x61, 0x6c, 0x66, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x4b, 0x75, 0x62, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x4b, 0x75, 0x62, 0x61,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x4b, 0x75, 0x62, 0x61, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67,
0x74, 0x79, 0x64, 0x44, 0x61, 0x76, 0x69, 0x73, 0x2d, 0x74, 0x79, 0x64,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d,
0x54, 0x69, 0x6d, 0x6f, 0x72, 0x2d, 0x74, 0x79, 0x64, 0x50, 0x61, 0x61,
0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x50,
0x61, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x50, 0x61,
0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72,
0x2d, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79,
0x64, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x53, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79,
0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x73, 0x65, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x56,
0x65, 0x72, 0x64, 0x65, 0x72, 0x2d, 0x6f, 0x6f, 0x73, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x57, 0x65,
0x73, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74,
0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x73, 0x65, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x46,
0x69, 0x64, 0x6a, 0x69, 0x2d, 0x74, 0x79, 0x64, 0x46, 0x69, 0x64, 0x6a,
0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x46, 0x69, 0x64, 0x6a, 0x69, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x2d, 0x47, 0x75,
0x69, 0x61, 0x6e, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x46, 0x72, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x53, 0x75, 0x69, 0x64, 0x65, 0x72, 0x2d, 0x20, 0x65,
0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x65, 0x73,
0x65, 0x20, 0x74, 0x79, 0x64, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x2d, 0x74, 0x79, 0x64, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x2d, 0x74, 0x79, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0xeb,
0x2d, 0x74, 0x79, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0xeb, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0xeb, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x2d, 0x74, 0x79, 0x64, 0x4f,
0x6f, 0x73, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73,
0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x57,
0x65, 0x73, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x50, 0x65, 0x72,
0x73, 0x69, 0x65, 0x73, 0x65, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74,
0x65, 0x2d, 0x74, 0x79, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x65, 0x2d, 0x64,
0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x48, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x74, 0x79, 0x64, 0x48, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x67, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x48, 0x6f,
0x76, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x48,
0x6f, 0x76, 0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x49, 0x6e, 0x64, 0x69, 0xeb, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x49, 0x6e, 0x64, 0x69, 0x65, 0x73,
0x65, 0x20, 0x4f, 0x73, 0x65, 0x61, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64,
0x49, 0x6e, 0x64, 0x6f, 0x73, 0x6a, 0x69, 0x6e, 0x61, 0x2d, 0x74, 0x79,
0x64, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79,
0x64, 0x4f, 0x6f, 0x73, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64, 0x49,
0x72, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x49, 0x72, 0x61, 0x6e, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x49, 0x72, 0x61, 0x6e, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74,
0x79, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x74,
0x79, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x49,
0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x2d, 0x74,
0x79, 0x64, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79,
0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x2d, 0x64, 0x61,
0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x74, 0x79, 0x64, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4f, 0x6f, 0x73,
0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74,
0x79, 0x64, 0x57, 0x65, 0x73, 0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73,
0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79,
0x64, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x74,
0x79, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4b, 0x69,
0x72, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x4c,
0x69, 0x6e, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d,
0x74, 0x79, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x2d, 0x74, 0x79, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d,
0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x6c, 0x65,
0x69, 0x73, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x6c, 0x65,
0x64, 0x69, 0x76, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x61, 0x73, 0x2d, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x6c, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65,
0x2d, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x2d, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x79, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66,
0x69, 0x65, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x50, 0x61, 0x73, 0x69,
0x66, 0x69, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69,
0x65, 0x73, 0x65, 0x20, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79,
0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72,
0x2d, 0x74, 0x79, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d, 0x74, 0x79, 0x64, 0x4d, 0x6f,
0x73, 0x6b, 0x6f, 0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4d, 0x69, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x2d, 0x74, 0x79, 0x64, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x2d, 0x74, 0x79, 0x64, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x2d, 0x74, 0x79,
0x64, 0x4e, 0x69, 0x65, 0x75, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x2d,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0xeb, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x69,
0x65, 0x75, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0xeb,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x4e, 0x69, 0x65,
0x75, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x79,
0x64, 0x4e, 0x69, 0x65, 0x75, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x74, 0x79, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67,
0x74, 0x79, 0x64, 0x4e, 0x69, 0x75, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x64, 0x61, 0x67, 0x6c,
0x69, 0x67, 0x74, 0x79, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x2d, 0x74, 0x79, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x2d, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x79, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x74, 0x79, 0x64, 0x4f,
0x6d, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2d, 0x74, 0x79, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x79, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x2d, 0x74, 0x79, 0x64, 0x50, 0x61, 0x70, 0x6f, 0x65, 0x61, 0x2d, 0x4e,
0x69, 0x65, 0x75, 0x2d, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x65, 0x2d, 0x74,
0x79, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x2d, 0x74,
0x79, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x50, 0x65, 0x72, 0x75, 0x2d, 0x74, 0x79, 0x64,
0x50, 0x65, 0x72, 0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x50, 0x65, 0x72, 0x75, 0x2d, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x79, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x79, 0x64, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x79, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x70, 0x79, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x53, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x64, 0x61,
0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x50, 0x6f, 0x6e, 0x61, 0x70,
0x65, 0x2d, 0x74, 0x79, 0x64, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61,
0x6e, 0x67, 0x2d, 0x74, 0x79, 0x64, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x2d, 0x74, 0x79, 0x64, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61,
0x2d, 0x74, 0x79, 0x64, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x2d, 0x74, 0x79, 0x64, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79,
0x64, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x2d, 0x74, 0x79, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67,
0x74, 0x79, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x74, 0x79, 0x64,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d,
0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x53, 0x65, 0x79,
0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x6f, 0x65, 0x72, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x53, 0x61, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x2d,
0x74, 0x79, 0x64, 0x53, 0x75, 0x69, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0xeb, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x75, 0x72, 0x69, 0x6e,
0x61, 0x6d, 0x65, 0x2d, 0x74, 0x79, 0x64, 0x53, 0x79, 0x6f, 0x77, 0x61,
0x2d, 0x74, 0x79, 0x64, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x2d, 0x74,
0x79, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x74, 0x79, 0x64,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x2d, 0x64, 0x61, 0x67, 0x6c, 0x69, 0x67, 0x74, 0x79, 0x64, 0x54,
0x61, 0x64, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x74,
0x79, 0x64, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x2d, 0x74, 0x79,
0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2d, 0x74, 0x79, 0x64, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2d, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x2d,
0x74, 0x79, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x2d, 0x74, 0x79, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x2d,
0x74, 0x79, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x2d, 0x74,
0x79, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x79, 0x64, 0x4f, 0x65, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2d, 0x74, 0x79, 0x64, 0x4f, 0x65, 0x73, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x79, 0x64, 0x4f, 0x65, 0x73, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79,
0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x74, 0x79, 0x64,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x2d, 0x74, 0x79,
0x64, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x2d, 0x74, 0x79, 0x64, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x79, 0x64, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64,
0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x74, 0x79,
0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x57,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x79, 0x64, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d,
0x74, 0x79, 0x64, 0x57, 0x61, 0x6b, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x74, 0x79, 0x64, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73,
0x20, 0x65, 0x6e, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x2d, 0x74,
0x79, 0x64, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x74,
0x79, 0x64, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4a,
0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x79, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x74, 0x79, 0x64, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x79, 0x64, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x2d, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x79, 0x64, 0x59, 0x75, 0x6b,
0x6f, 0x6e, 0x74, 0x79, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x45,
0x6a, 0x6b, 0x72, 0x69, 0x74, 0x20, 0x5b, 0x41, 0x6b, 0x6f, 0x5d, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x45, 0x6a, 0x6b, 0x72, 0x69, 0x74, 0x20, 0x5b, 0x41,
0x6b, 0x6f, 0x5d, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x45, 0x6a, 0x6b, 0x72, 0x69, 0x74, 0x20, 0x5b,
0x41, 0x6b, 0x6f, 0x5d, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x66,
0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0xeb, 0x73, 0x20,
0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0xeb, 0x73, 0x20, 0x4c, 0x69,
0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0xeb, 0x73, 0x20, 0x4a, 0x75, 0x67, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0xeb, 0x73,
0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0xeb, 0x73, 0x20,
0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb,
0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0xeb,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0xeb,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0xeb, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x53, 0x48, 0x42, 0x41, 0x2d, 0x73, 0xeb, 0x20,
0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x53, 0x48, 0x42, 0x41, 0x2d, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64,
0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x53, 0x48, 0x42, 0x41, 0x2d, 0x73, 0xeb,
0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x53, 0x48, 0x42, 0x41, 0x2d, 0x73, 0xeb, 0x20, 0x4c,
0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x53, 0x48,
0x42, 0x41, 0x2d, 0x73, 0xeb, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x53, 0x48, 0x42, 0x41, 0x2d, 0x73, 0xeb, 0x20, 0x4c, 0x69,
0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54,
0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20,
0x42, 0x72, 0x65, 0x7a, 0x69, 0x74, 0x20, 0x4d, 0x61, 0x6c, 0x6f, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72,
0x65, 0x76, 0x65, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x7a, 0x69, 0x74, 0x20,
0x4d, 0x61, 0x6c, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74,
0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x7a, 0x69,
0x74, 0x20, 0x4d, 0x61, 0x6c, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x65, 0x76, 0x65,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x20, 0x74,
0xeb, 0x20, 0x42, 0x72, 0x65, 0x67, 0x75, 0x74, 0x20, 0x74, 0xeb, 0x20,
0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x65, 0x76, 0x65,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x20, 0x74,
0xeb, 0x20, 0x42, 0x72, 0x65, 0x67, 0x75, 0x74, 0x20, 0x74, 0xeb, 0x20,
0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x54, 0x65,
0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42,
0x72, 0x65, 0x67, 0x75, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x71,
0xeb, 0x73, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x69, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41,
0x70, 0x69, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41,
0x6b, 0x74, 0x61, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x61, 0x72, 0x61, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x61, 0x72, 0x61, 0x62, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x61,
0x72, 0x61, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x72,
0x67, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x41, 0x72, 0x67, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xeb, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x41, 0x72, 0x67, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xeb, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x72, 0x67, 0x6a, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64,
0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x72, 0x67,
0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72,
0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x72, 0x67,
0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xeb, 0x73, 0x20, 0x50, 0x65, 0x72,
0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0xeb, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0xeb, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x75, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x75,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x75, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20,
0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb,
0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x6f, 0x2d, 0x50, 0x65,
0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20,
0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x6f, 0x2d, 0x50, 0x65, 0x72,
0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64,
0x72, 0x6f, 0x72, 0x6f, 0x2d, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69,
0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x4c, 0x69, 0x6e,
0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0xeb, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0xeb, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0xeb, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x73, 0xeb, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d,
0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x73, 0xeb, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x6a, 0x78, 0x68, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x78, 0x68, 0x61, 0x6e,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x78, 0x68,
0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x76, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69,
0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x42, 0x72, 0x61, 0x7a,
0x69, 0x6c, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x42, 0x72, 0x61,
0x7a, 0x69, 0x6c, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x42, 0x72, 0x61, 0x7a, 0x69,
0x6c, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x2d, 0x44, 0x75, 0x72, 0x61, 0x73, 0x61, 0x6c,
0x61, 0x6d, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x65,
0x70, 0x69, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x47, 0x6a, 0x65, 0x6c, 0x62,
0xeb, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x65, 0x70, 0x69, 0x74, 0x20,
0x74, 0xeb, 0x20, 0x47, 0x6a, 0x65, 0x6c, 0x62, 0xeb, 0x72, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b,
0x65, 0x70, 0x69, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x47, 0x6a, 0x65, 0x6c,
0x62, 0xeb, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x65, 0x6a,
0x73, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b,
0x61, 0x74, 0x61, 0x6d, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x61,
0x74, 0x61, 0x6d, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x61, 0x74, 0x61, 0x6d, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x6c, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x6c, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x69,
0x6c, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x6e,
0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x6e, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x4b, 0x69, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0xc7,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0xc7, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0xc7, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c,
0x6c, 0x69, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x4b, 0x72, 0x69, 0x73, 0x68,
0x74, 0x6c, 0x69, 0x6e, 0x64, 0x6a, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4b,
0x6f, 0x6b, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4b, 0x75, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4b,
0x75, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4b,
0x75, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x75, 0x62, 0xeb,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x75, 0x62, 0xeb, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b,
0x75, 0x62, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x44, 0x65,
0x6a, 0x76, 0x69, 0x73, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x69,
0x6d, 0x6f, 0x72, 0x69, 0x74, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c,
0x69, 0x74, 0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x73, 0x68, 0x6b, 0xeb,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69,
0x74, 0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x73, 0x68, 0x6b, 0xeb, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x74, 0x20, 0x74, 0xeb,
0x20, 0x50, 0x61, 0x73, 0x68, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x45, 0x6b, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73,
0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x51, 0x65, 0x6e,
0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb,
0x73, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72, 0x6f, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20,
0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x45,
0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x4c,
0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x4c,
0x61, 0x72, 0x67, 0xeb, 0x74, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70,
0xeb, 0x73, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb,
0x73, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x45, 0x76, 0x72, 0x6f, 0x70, 0xeb, 0x73, 0x20, 0x50, 0x65,
0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49,
0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x46, 0x69, 0x78, 0x68, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x46, 0x69, 0x78, 0x68, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x46, 0x69, 0x78, 0x68, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6a, 0x61,
0x6e, 0xeb, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x7a, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74,
0x6f, 0x72, 0x65, 0x76, 0x65, 0x20, 0x4a, 0x75, 0x67, 0x6f, 0x72, 0x65,
0x20, 0x64, 0x68, 0x65, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74,
0x69, 0x6b, 0x65, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x7a, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x47, 0x6a, 0x65, 0x6f, 0x72, 0x67, 0x6a, 0x69, 0x73, 0xeb,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x47, 0x6a, 0x65, 0x6f, 0x72, 0x67, 0x6a, 0x69,
0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x47, 0x6a, 0x65, 0x6f, 0x72, 0x67, 0x6a, 0x69, 0x73,
0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a,
0x76, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x75, 0x69, 0xe7, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0xeb, 0x73, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x4c,
0x69, 0x6e, 0x64, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x47, 0x72, 0xeb, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x50,
0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x47, 0x72, 0xeb, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73,
0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x47, 0x72, 0xeb, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20,
0x50, 0x65, 0x72, 0xeb, 0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6d, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x47, 0x6a, 0x69, 0x72, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65,
0x20, 0x48, 0x61, 0x75, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a,
0x76, 0x65, 0x20, 0x48, 0x61, 0x75, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76,
0x65, 0x20, 0x48, 0x61, 0x75, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x75, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x75, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x75, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4f, 0x71, 0x65,
0x61, 0x6e, 0x69, 0x74, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e,
0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x7a, 0x69, 0x73, 0xeb, 0x20, 0x51, 0x65, 0x6e, 0x64, 0x72,
0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x73, 0xeb, 0x20, 0x4c, 0x69, 0x6e, 0x64,
0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x73, 0xeb, 0x20, 0x50, 0x65, 0x72, 0xeb,
0x6e, 0x64, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61,
0x65, 0x6c, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61,
0x65, 0x6c, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e,
0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e,
0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x73, 0xeb,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0xe7, 0x61, 0x74, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x6b, 0xeb, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x6b, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x74, 0x20, 0x4c, 0x69, 0x6e, 0x64, 0x6f, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x20, 0x50, 0x65, 0x72, 0xeb, 0x6e,
0x64, 0x69, 0x6d, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6b, 0x6f, 0x72,
0x65, 0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x6e,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x6b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x4b, 0x6f, 0x73, 0x72, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x7a, 0x69, 0x73, 0xeb,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x73,
0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a,
0x76, 0x65, 0x20, 0x53, 0x70, 0x6f, 0x72, 0x61, 0x64, 0x69, 0x6b, 0xeb,
0x20, 0x45, 0x6b, 0x75, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c, 0xeb,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48,
0x6f, 0x75, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x2d, 0x48, 0x6f, 0x75, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65,
0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d,
0x48, 0x6f, 0x75, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d,
0x61, 0x6b, 0x61, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x6b,
0x61, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x6a, 0x7a,
0x69, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x6b,
0x65, 0x7a, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68,
0x75, 0x6a, 0x76, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x73, 0x6f, 0x6e, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f,
0x72, 0x65, 0x76, 0x65, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x67, 0x75, 0x74,
0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f, 0x72, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x65, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f,
0x72, 0x65, 0x76, 0x65, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x6e, 0x65, 0x20, 0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x67, 0x75, 0x74,
0x20, 0x74, 0xeb, 0x20, 0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f, 0x72, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x65, 0x76,
0x65, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x20,
0x74, 0xeb, 0x20, 0x42, 0x72, 0x65, 0x67, 0x75, 0x74, 0x20, 0x74, 0xeb,
0x20, 0x50, 0x61, 0x71, 0xeb, 0x73, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x55, 0x6c, 0x61,
0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x4d, 0x6f, 0x73, 0x6b, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x6b,
0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0xeb, 0x20,
0x73, 0xeb, 0x20, 0x52, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0xeb, 0x20, 0x73, 0xeb, 0x20, 0x52,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0xeb,
0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20,
0x52, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e,
0x64, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x74, 0x20, 0x5b, 0x54, 0x6f, 0x6b,
0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x5d, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e,
0x64, 0x69, 0x74, 0x20, 0x5b, 0x54, 0x6f, 0x6b, 0xeb, 0x73, 0x20, 0x73,
0xeb, 0x20, 0x52, 0x65, 0x5d, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75,
0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x69, 0x74, 0x20, 0x5b, 0x54, 0x6f,
0x6b, 0xeb, 0x73, 0x20, 0x73, 0xeb, 0x20, 0x52, 0x65, 0x5d, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x73, 0xeb, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x74,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x74, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f,
0x72, 0x65, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69,
0x74, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d,
0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d,
0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x74, 0xeb,
0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0xeb, 0x73, 0x20, 0x73, 0xeb,
0x20, 0x56, 0x65, 0x72, 0x69, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x73, 0xeb, 0x20, 0x73, 0xeb, 0x20,
0x52, 0x65, 0x2d, 0x50, 0x61, 0x70, 0x75, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65,
0x20, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x50, 0x65, 0x72, 0x75, 0x73, 0xeb, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20,
0x65, 0x20, 0x50, 0x65, 0x72, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x50, 0x65, 0x72,
0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65, 0x20, 0x46, 0x65,
0x6e, 0x69, 0x6b, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x68,
0xeb, 0x6e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x20, 0x64, 0x68, 0x65, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x53,
0x68, 0xeb, 0x6e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x20, 0x64, 0x68, 0x65,
0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x53, 0x68, 0xeb,
0x6e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x20, 0x64, 0x68, 0x65, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x50, 0x65, 0x6e, 0x69, 0x61, 0x6e, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x7a, 0x69, 0x6c,
0x6f, 0x72, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x7a,
0x69, 0x6c, 0x6f, 0x72, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4b, 0x69, 0x7a, 0x69,
0x6c, 0x6f, 0x72, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x52, 0x6f, 0x64, 0x68, 0x65, 0x72, 0xeb, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69,
0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72,
0x65, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0xeb,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0xeb, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x53, 0x65, 0x6a, 0x73,
0x68, 0x65, 0x6c, 0x6c, 0x65, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x49, 0x73, 0x68, 0x75, 0x6a, 0x76, 0x65,
0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x58, 0x68, 0x6f, 0x72, 0x78, 0x68, 0x61, 0x73, 0x20, 0x74,
0xeb, 0x20, 0x4a, 0x75, 0x67, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x53, 0x6a, 0x6f, 0x75, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x65, 0x20, 0x65, 0x20, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x69, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20,
0x54, 0x61, 0x6a, 0x70, 0x65, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x54, 0x61, 0x78, 0x68, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65,
0x20, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0xeb, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0xc7, 0x75, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c,
0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x73, 0xeb, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x56, 0x65, 0x6e,
0x65, 0x7a, 0x75, 0x65, 0x6c, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76,
0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x76, 0x65, 0x72,
0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x69, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65,
0x20, 0x49, 0x73, 0x68, 0x75, 0x6c, 0x6c, 0x69, 0x74, 0x20, 0x55, 0x65,
0x6a, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x55, 0x6f, 0x6c, 0x6c,
0x69, 0x73, 0x69, 0x74, 0x20, 0x64, 0x68, 0x65, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0xeb, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20, 0x4a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x65, 0x20,
0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x75, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x65, 0x20, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x75, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x76, 0x65, 0x72, 0x6f, 0x72, 0x65, 0x20, 0x65, 0x20, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x75, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x65, 0x20, 0x4a, 0x75, 0x6b, 0x6f, 0x6e, 0x69, 0x74,
0x12e8, 0x12a0, 0x134d, 0x130b, 0x1292, 0x1235, 0x1273, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd, 0x20, 0x12a0, 0x134d, 0x122a, 0x12ab,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x12a0, 0x134d,
0x122a, 0x12ab, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12f0, 0x1261, 0x1265, 0x20, 0x12a0,
0x134d, 0x122a, 0x12ab, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x12a0, 0x134d, 0x122a, 0x12ab, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x12a0, 0x134d, 0x122a, 0x12ab,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5,
0x122b, 0x1265, 0x20, 0x12a0, 0x134d, 0x122a, 0x12ab, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x120b, 0x1235, 0x12ab, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x120b, 0x1235, 0x12ab,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x120b, 0x1235, 0x12ab, 0x20, 0x12e8, 0x1240,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12a0, 0x121b, 0x12de, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x12a0, 0x121b, 0x12de, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0,
0x121b, 0x12de, 0x1295, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1230, 0x121c, 0x1295, 0x20, 0x12a0, 0x121c,
0x122a, 0x12ab, 0x20, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1230, 0x121c, 0x1295, 0x20,
0x12a0, 0x121c, 0x122a, 0x12ab, 0x20, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x20, 0x12e8, 0x1240, 0x1295,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x121d, 0x1235,
0x122b, 0x1243, 0x12ca, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x121d, 0x1235, 0x122b,
0x1243, 0x12ca, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1270, 0x122b, 0x122b, 0x20, 0x12e8, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1270, 0x122b, 0x122b, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1270, 0x122b, 0x122b, 0x20, 0x12e8, 0x1240, 0x1295, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1353, 0x1235, 0x134a,
0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x1353, 0x1235, 0x134a, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1353, 0x1235, 0x134a, 0x12ad,
0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x12a0, 0x1293, 0x12f5, 0x12ed, 0x122d, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x1293, 0x12f2, 0x12ed, 0x122d,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0,
0x1293, 0x12f5, 0x12ed, 0x122d, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x12e8, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x1352, 0x12eb, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x1352, 0x12eb, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x1352, 0x12eb, 0x20, 0x12e8, 0x1240, 0x1295,
0x20, 0x130a, 0x12dc, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12d3, 0x1228, 0x1262, 0x12eb,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12d3, 0x1228, 0x1262, 0x12eb, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12d3, 0x1228, 0x1262, 0x12eb, 0x20,
0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x12a0, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x122d, 0x1300, 0x1295, 0x1272,
0x1293, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x20,
0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x12a0, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x20, 0x121d, 0x1235, 0x122b,
0x1243, 0x12ca, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a0, 0x122d, 0x1300, 0x1295, 0x1272,
0x1293, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a0,
0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x122d, 0x1218, 0x1292,
0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x122d, 0x1218, 0x1292, 0x12eb, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x122d, 0x1218,
0x1292, 0x12eb, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12a0, 0x1275, 0x120b, 0x1295, 0x1272, 0x12ad, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x1275, 0x120b, 0x1295, 0x1272, 0x12ad,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x1275, 0x120b, 0x1295, 0x1272, 0x12ad, 0x20,
0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd, 0x20, 0x12a0, 0x12cd, 0x1235,
0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1218,
0x12ab, 0x12a8, 0x1208, 0x129b, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235,
0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x20, 0x12e8,
0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1218, 0x12ab, 0x12a8,
0x1208, 0x129b, 0x20, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b,
0x120a, 0x12eb, 0x20, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x20, 0x121d, 0x1235, 0x122b,
0x1243, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a,
0x12eb, 0x20, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd, 0x20, 0x121d, 0x1235, 0x122b,
0x1245, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a0, 0x12cd,
0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb,
0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0,
0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca,
0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12a0, 0x12cd, 0x1235,
0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275, 0x122b, 0x120a, 0x12eb, 0x20, 0x121d,
0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a0, 0x12cd, 0x1235, 0x1275,
0x122b, 0x120a, 0x12eb, 0x20, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12e8, 0x1240,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12a0, 0x12d8, 0x122d, 0x1263, 0x1303, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0,
0x12d8, 0x122d, 0x1263, 0x1303, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12a0, 0x12d8, 0x122d, 0x1263, 0x1303, 0x1295, 0x20, 0x12ad, 0x1228,
0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x12de, 0x1228, 0x1235, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x12de, 0x1228, 0x1235, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a0, 0x12de, 0x1228, 0x1235, 0x20, 0x12ad,
0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1263, 0x1295, 0x130d, 0x120b,
0x12f4, 0x123d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1263, 0x1295, 0x130d, 0x120b, 0x12f4,
0x123d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1263,
0x1295, 0x130d, 0x120b, 0x12f4, 0x123d, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1261, 0x1273, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1266,
0x120a, 0x126a, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1265, 0x122b, 0x12da, 0x120b,
0x12ca, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x1265, 0x122b, 0x1232, 0x120a, 0x12eb, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1265, 0x122b, 0x12da,
0x120b, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1265, 0x1229, 0x1294, 0x12ed, 0x20, 0x12f3, 0x1229, 0x1233,
0x120b, 0x121d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ac, 0x1355, 0x20, 0x1268, 0x122d,
0x12f4, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ac, 0x1355, 0x20, 0x1268, 0x122d, 0x12f4,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ac, 0x1355,
0x20, 0x1268, 0x122d, 0x12f4, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x127b, 0x121e, 0x122e, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x127b, 0x1273, 0x121d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x127b,
0x1273, 0x121d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x127b, 0x1273, 0x121d, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x127a, 0x120a, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x127a, 0x120a, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x127a, 0x120a, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x127b, 0x12ed, 0x1293, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x127b, 0x12ed, 0x1293, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x127b, 0x12ed, 0x1293,
0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x127e, 0x12ed, 0x1263, 0x120d, 0x1233, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x127e, 0x12ed, 0x1263, 0x120d, 0x1233, 0x1295,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x127e, 0x12ed, 0x1263, 0x120d, 0x1233, 0x1295, 0x20,
0x12e8, 0x1260, 0x130b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x1308, 0x1293, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x12ae, 0x12ae, 0x1235, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12ae, 0x120e, 0x121d, 0x1262, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x12ae, 0x120e, 0x121d, 0x1262, 0x12eb, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ae, 0x120e, 0x121d, 0x1262, 0x12eb, 0x20, 0x12ad, 0x1228,
0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a9, 0x12ad, 0x20, 0x12f0, 0x1234,
0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a9, 0x12ad, 0x20, 0x12f0, 0x1234,
0x1276, 0x127d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12a9, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x130d, 0x121b, 0x123d, 0x20,
0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12a9, 0x1263, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12a9, 0x1263, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12a9, 0x1263, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d,
0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12f4, 0x126a, 0x1235, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12f1, 0x121e, 0x1295, 0x1275, 0x2d, 0x12f1, 0x122d, 0x126a, 0x120d,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x1272, 0x121e,
0x122d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x1235, 0x1270, 0x122d, 0x20, 0x12f0,
0x1234, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x1235, 0x1270, 0x122d, 0x20,
0x12f0, 0x1234, 0x1275, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x12a2, 0x1235, 0x1270, 0x122d, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x12ad, 0x1228,
0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x12b3, 0x12f6, 0x122d, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd, 0x20, 0x12a0,
0x12cd, 0x122e, 0x1353, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208,
0x129b, 0x12cd, 0x20, 0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b, 0x12cd, 0x20,
0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12a0, 0x12cd, 0x122e, 0x1353,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12a0,
0x12cd, 0x122e, 0x1353, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12a0, 0x12cd, 0x122e, 0x1353, 0x20,
0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1229, 0x1245, 0x20,
0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x12e8, 0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a0, 0x12cd, 0x122e,
0x1353, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20,
0x12a0, 0x12cd, 0x122e, 0x1353, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20, 0x12a0, 0x12cd, 0x122e, 0x1353,
0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x134b, 0x120d,
0x12ad, 0x120b, 0x1295, 0x12f5, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x134b, 0x120d, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x20, 0x12f0, 0x1234, 0x1276,
0x127d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x134b,
0x120d, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x12ad,
0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x134a, 0x1302, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x134a, 0x1302, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x134a, 0x1302, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1348, 0x1228, 0x1295, 0x1233, 0x12ed, 0x20, 0x1309, 0x12eb, 0x1293,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1348, 0x1228, 0x1295, 0x1233, 0x12ed, 0x20, 0x12f0,
0x1261, 0x1263, 0x12ca, 0x20, 0x12a5, 0x1293, 0x20, 0x12a0, 0x1295, 0x1273, 0x122d, 0x12ad,
0x1272, 0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x130b, 0x120b, 0x1353, 0x130e, 0x1235,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x130b, 0x121d, 0x1262, 0x12e8, 0x122d, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1302, 0x12ee, 0x122d, 0x1302, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1302, 0x12ee, 0x122d, 0x1302, 0x12eb, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1302, 0x12ee, 0x122d, 0x1302, 0x12eb, 0x20, 0x12ad, 0x1228,
0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1302, 0x120d, 0x1260, 0x122d, 0x1275,
0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x130d, 0x122a, 0x1295,
0x12ca, 0x127d, 0x20, 0x121b, 0x12d5, 0x12a8, 0x120b, 0x12ca, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x130d, 0x122a,
0x1295, 0x120b, 0x1295, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295,
0x12f5, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d,
0x12d5, 0x122b, 0x1265, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x130d, 0x122a, 0x1295, 0x120b,
0x1295, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x130d, 0x122a, 0x1295, 0x120b, 0x1295, 0x12f5, 0x20,
0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1263, 0x1205, 0x1228,
0x1230, 0x120b, 0x1324, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1309, 0x12eb, 0x1293, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1203, 0x12cb, 0x12ed,
0x20, 0x12a0, 0x120c, 0x12a1, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1203, 0x12cb, 0x12ed, 0x20, 0x12a0, 0x120c, 0x12a1, 0x1275,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1203, 0x12cb, 0x12ed, 0x20, 0x12a0, 0x120c, 0x12a1, 0x1275,
0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x1206, 0x1295, 0x130d, 0x20, 0x12ae, 0x1295, 0x130d, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1206, 0x1295, 0x130d, 0x20, 0x12ae, 0x1295, 0x130d, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1206, 0x1295, 0x130d, 0x20,
0x12ae, 0x1295, 0x130d, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1206, 0x126d, 0x12f5, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x1206, 0x126d, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1206,
0x126d, 0x12f5, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1205, 0x1295, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1205, 0x1295, 0x12f5, 0x20, 0x12cd, 0x1245,
0x12eb, 0x1296, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x1295, 0x12f6, 0x127b,
0x12ed, 0x1293, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1218, 0x12ab, 0x12a8, 0x1208, 0x129b,
0x12cd, 0x20, 0x12a2, 0x1295, 0x12f6, 0x1294, 0x12e2, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x121d, 0x1235, 0x122b, 0x1243, 0x12ca, 0x20, 0x12a2, 0x1295, 0x12f6, 0x1294, 0x12e2,
0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca, 0x20,
0x12a2, 0x1295, 0x12f6, 0x1294, 0x12e2, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2,
0x122b, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x122b, 0x1295, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a2, 0x122b, 0x1295, 0x20,
0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12a2, 0x122d, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x12e8,
0x1260, 0x130b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x12a5, 0x1235, 0x122b, 0x12a4, 0x120d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12a5, 0x1235, 0x122b, 0x12a4, 0x120d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12a5, 0x1235, 0x122b, 0x12a4, 0x120d, 0x20, 0x12e8, 0x1240, 0x1295,
0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1303, 0x1353,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1303, 0x1353, 0x1295, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1303, 0x1353, 0x1295, 0x20, 0x12e8,
0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12ab, 0x121d, 0x127b, 0x1275, 0x12ab, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x1354, 0x1275, 0x122e, 0x1353, 0x126d, 0x120e, 0x1235, 0x12aa,
0x20, 0x2d, 0x20, 0x12ab, 0x121d, 0x127b, 0x1275, 0x1235, 0x12aa, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1354, 0x1275, 0x122e, 0x1353,
0x126d, 0x120e, 0x1235, 0x12aa, 0x20, 0x2d, 0x20, 0x12ab, 0x121d, 0x127b, 0x1275, 0x1235,
0x12aa, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x121d, 0x1235, 0x122b, 0x1245, 0x20, 0x12ab, 0x12db, 0x12aa,
0x1235, 0x1273, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121d, 0x12d5, 0x122b, 0x1265,
0x20, 0x12ab, 0x12db, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x12ae, 0x122a, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ae, 0x122a, 0x12eb, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ae, 0x122a, 0x12eb,
0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x12ae, 0x1235, 0x122b, 0x12a4, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12ad,
0x122b, 0x1235, 0x1296, 0x12eb, 0x122d, 0x1235, 0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12ad, 0x122b, 0x1235, 0x1296, 0x12ed, 0x12a0,
0x122d, 0x1235, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12ad, 0x122b, 0x1235, 0x1296, 0x12eb,
0x122d, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12aa, 0x122d, 0x130a, 0x1235, 0x1273, 0x1295,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x120b, 0x12ed, 0x1295, 0x20, 0x12f0, 0x1234, 0x1276,
0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x120e, 0x122d, 0x12f5, 0x20, 0x1206, 0x12cc,
0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x120e, 0x122d, 0x12f5, 0x20, 0x1206, 0x12cc, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x120e,
0x122d, 0x12f5, 0x20, 0x1206, 0x12cc, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121b, 0x130b, 0x12f3, 0x1295,
0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x121b, 0x130b, 0x12f3, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121b, 0x130b, 0x12f3, 0x1295,
0x20, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x121b, 0x120c, 0x12ed, 0x12e2, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x121b, 0x120d, 0x12f2, 0x126d, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b, 0x122d,
0x12b4, 0x1233, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b, 0x122d, 0x123b, 0x120d,
0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b, 0x12cd,
0x122a, 0x123a, 0x12e8, 0x1235, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b, 0x12cd, 0x122a,
0x123a, 0x12e8, 0x1235, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x121b, 0x12cd, 0x122a, 0x123a, 0x12e8, 0x1235, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x121b, 0x12cd, 0x1230, 0x1295, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x121c, 0x12ad, 0x1232, 0x12ae, 0x20, 0x1353, 0x1232, 0x134a, 0x12ad, 0x20,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121c, 0x12ad,
0x1232, 0x12ae, 0x20, 0x1353, 0x1232, 0x134a, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121c,
0x12ad, 0x1232, 0x12ae, 0x20, 0x1353, 0x1232, 0x134a, 0x12ad, 0x20, 0x12e8, 0x1240, 0x1295,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a1,
0x120b, 0x1295, 0x20, 0x1263, 0x1276, 0x122d, 0x20, 0x130a, 0x12dc, 0x12e8, 0x12a1, 0x120b,
0x1295, 0x20, 0x1263, 0x1276, 0x122d, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a1, 0x120b, 0x1295,
0x20, 0x1263, 0x1276, 0x122d, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121e, 0x1235, 0x12ae, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121e, 0x1235, 0x12ae,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x121e, 0x1235, 0x12ae, 0x20, 0x12e8, 0x1260, 0x130b, 0x20,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x121a, 0x12eb,
0x1295, 0x121b, 0x122d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1293, 0x12cd, 0x1229, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1294, 0x1353, 0x120d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x1292, 0x12cd, 0x20, 0x12ab, 0x120c, 0x12f6, 0x1292, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1292, 0x12cd, 0x20, 0x12ab, 0x120c, 0x12f6, 0x1292, 0x12eb, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1292, 0x12cd, 0x20, 0x12ab, 0x120c,
0x12f6, 0x1292, 0x12eb, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1292, 0x12cd, 0x20, 0x12da, 0x120b, 0x1295, 0x12f5, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x1292, 0x12cd, 0x20, 0x12da, 0x120b, 0x1295, 0x12f5, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1292, 0x12cd, 0x20, 0x12da, 0x120b, 0x1295,
0x12f5, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1292, 0x12cd, 0x134b, 0x12cd, 0x1295, 0x12f5, 0x120b, 0x1295, 0x12f5,
0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x1292, 0x12cd, 0x134b, 0x12cd, 0x1295, 0x12f5, 0x120b, 0x1295, 0x12f5, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x1292, 0x12cd, 0x134b, 0x12cd, 0x1295, 0x12f5, 0x120b, 0x1295, 0x12f5, 0x20,
0x12e8, 0x1240, 0x1295, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x1292, 0x12e9, 0x12cc, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1296,
0x122d, 0x134e, 0x120d, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x1296, 0x122d, 0x134e, 0x120d, 0x12ad, 0x20, 0x12f0, 0x1234, 0x1276, 0x127d,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1296, 0x122d, 0x134e, 0x120d, 0x12ad, 0x20, 0x12f0,
0x1234, 0x1276, 0x127d, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275,
0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1296, 0x122e, 0x1295, 0x1203, 0x20,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1348, 0x122d,
0x1293, 0x1295, 0x12f6, 0x20, 0x12f2, 0x20, 0x1296, 0x122e, 0x1295, 0x127b, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x1348, 0x122d, 0x1293, 0x1295, 0x12f6, 0x20, 0x12f2, 0x20, 0x1296, 0x122e,
0x1295, 0x1203, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1296, 0x126e, 0x1232, 0x1265, 0x122a, 0x1235,
0x12ad, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x1296, 0x126e, 0x1232, 0x1262, 0x122d, 0x1235, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x1296, 0x126e, 0x1232, 0x1265, 0x122a, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1260, 0x130b,
0x20, 0x1230, 0x12a0, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a6,
0x121d, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x12a6, 0x121d, 0x1235, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b,
0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12a6,
0x121d, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1353, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1218, 0x12f0,
0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x12aa, 0x1235, 0x1273, 0x1295,
0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x120b,
0x12cd, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x1357, 0x20, 0x1292, 0x12cd, 0x20,
0x130a, 0x1292, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x122b, 0x1313, 0x12ed, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x122b, 0x1313, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1353, 0x122b, 0x1313, 0x12ed, 0x20, 0x12ad,
0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1354, 0x1229, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1354, 0x1229, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1354, 0x1229, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x134a, 0x120a, 0x1352, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x134a, 0x120a, 0x1352, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x134a, 0x120a, 0x1352, 0x1295, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x134a, 0x1292, 0x12ad, 0x1235, 0x20, 0x12f0, 0x1234, 0x1276,
0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x1245, 0x12f1, 0x1235, 0x20, 0x12e8, 0x1352, 0x12ec,
0x122d, 0x20, 0x12a5, 0x1293, 0x20, 0x121a, 0x12b4, 0x120e, 0x1295, 0x20, 0x1230, 0x12d3,
0x1275, 0x1245, 0x12f1, 0x1235, 0x20, 0x12e8, 0x1352, 0x12ec, 0x122d, 0x20, 0x12a5, 0x1293,
0x20, 0x121a, 0x12b4, 0x120e, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230,
0x12d3, 0x1275, 0x1245, 0x12f1, 0x1235, 0x20, 0x12e8, 0x1352, 0x12ec, 0x122d, 0x20, 0x12a5,
0x1293, 0x20, 0x121a, 0x12b4, 0x120e, 0x1295, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265,
0x122d, 0x1203, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1352, 0x1275, 0x12ab, 0x12ed,
0x122d, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1356, 0x1293, 0x1354, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1355, 0x12ee, 0x1295, 0x130d, 0x12eb, 0x1295, 0x130d, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x122c, 0x12e9, 0x1292, 0x12e8, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x122e, 0x1274, 0x122b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1233, 0x12ad, 0x1203,
0x120a, 0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x1233, 0x12ad, 0x1203, 0x120a, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20,
0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1233, 0x12ad,
0x1203, 0x120a, 0x1295, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20,
0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1233, 0x121b, 0x122b, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1233, 0x121b, 0x122b, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x1233, 0x121b, 0x122b, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230,
0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x1233, 0x121e, 0x12cb,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1233, 0x121e, 0x12cb, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1233, 0x121e, 0x12cb, 0x20, 0x12e8, 0x1260,
0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1234, 0x1238, 0x120d, 0x1235, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1232, 0x1295, 0x130b, 0x1352, 0x122d, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1230, 0x1208, 0x121e, 0x1295, 0x20, 0x12f0,
0x1234, 0x1276, 0x127d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12f0, 0x1261, 0x1265, 0x20,
0x1302, 0x12ee, 0x122d, 0x1302, 0x12eb, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1231, 0x122a,
0x1293, 0x121d, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1232, 0x12ee, 0x12cb, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1273, 0x1202, 0x1272, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1273,
0x12ed, 0x1354, 0x12ed, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1273, 0x12ed, 0x1354, 0x12ed,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1273, 0x12ed,
0x1354, 0x12ed, 0x20, 0x12e8, 0x1240, 0x1295, 0x20, 0x1265, 0x122d, 0x1203, 0x1295, 0x20,
0x1230, 0x12d3, 0x1275, 0x12e8, 0x1273, 0x1302, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x1276, 0x12ac, 0x120b, 0x12cd, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x1276, 0x1295, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1276, 0x1295, 0x130b, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1276, 0x1295, 0x130b,
0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1279, 0x12ad,
0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1271, 0x122d, 0x12ad, 0x1218, 0x1292, 0x1235, 0x1273,
0x1295, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1271, 0x122d, 0x12ad, 0x1218, 0x1292, 0x1235,
0x1273, 0x1295, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8,
0x1271, 0x122d, 0x12ad, 0x1218, 0x1292, 0x1235, 0x1273, 0x1295, 0x20, 0x12ad, 0x1228, 0x121d,
0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x1271, 0x126b, 0x1209, 0x20, 0x1230, 0x12d3,
0x1275, 0x12e8, 0x12a1, 0x122b, 0x1313, 0x12ed, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a1,
0x122b, 0x1313, 0x12ed, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x12a1, 0x122b, 0x1313, 0x12ed, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12a1, 0x12dd, 0x1264, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1230,
0x12d3, 0x1275, 0x12e8, 0x12a1, 0x12dd, 0x1264, 0x12aa, 0x1235, 0x1273, 0x1295, 0x20, 0x1218,
0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12a1, 0x12dd, 0x1264, 0x12aa,
0x1235, 0x1273, 0x1295, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275,
0x12e8, 0x126b, 0x1297, 0x1271, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x126b, 0x1297, 0x1271,
0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x126b, 0x1297,
0x1271, 0x20, 0x12ad, 0x1228, 0x121d, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x126c,
0x1294, 0x12dd, 0x12cc, 0x120b, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x126d, 0x120b, 0x12f2,
0x126e, 0x1235, 0x1276, 0x12ad, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x126a, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276, 0x12ad, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12e8, 0x126d, 0x120b, 0x12f2, 0x126e, 0x1235, 0x1276, 0x12ad, 0x20,
0x12e8, 0x1260, 0x130b, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x126e, 0x120d, 0x130e, 0x122b, 0x12f5, 0x20, 0x12e8, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x126e, 0x120d, 0x130e, 0x122b,
0x12f5, 0x20, 0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0,
0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x126b, 0x120d, 0x130e, 0x122b, 0x12f5, 0x20, 0x12e8,
0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d,
0x12e8, 0x126e, 0x1235, 0x1276, 0x12ad, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12cc, 0x12ad,
0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x1230, 0x12d3, 0x1275, 0x12e8, 0x12cb, 0x120a, 0x1235,
0x20, 0x12a5, 0x1293, 0x20, 0x1349, 0x1271, 0x1293, 0x20, 0x1230, 0x12d3, 0x1275, 0x12eb,
0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246,
0x1323, 0x1320, 0x122d, 0x12eb, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x1218, 0x12f0, 0x1260,
0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8,
0x12eb, 0x12a9, 0x1275, 0x1235, 0x12ad, 0x20, 0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3,
0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320, 0x122d, 0x12e8, 0x12e8, 0x12ab, 0x1270, 0x122a,
0x1295, 0x1260, 0x122d, 0x130d, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x12e8, 0x12ab, 0x1270, 0x122a, 0x1295, 0x1260, 0x122d, 0x130d, 0x20,
0x1218, 0x12f0, 0x1260, 0x129b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323,
0x1320, 0x122d, 0x12e8, 0x12e8, 0x12ab, 0x1270, 0x122a, 0x1295, 0x1260, 0x122d, 0x130d, 0x20,
0x12e8, 0x1260, 0x130b, 0x20, 0x1230, 0x12d3, 0x1275, 0x20, 0x12a0, 0x1246, 0x1323, 0x1320,
0x122d, 0x12e8, 0x12e9, 0x12ae, 0x1295, 0x20, 0x130a, 0x12dc, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x623, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x641, 0x631,
0x64a, 0x642, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631,
0x642, 0x20, 0x623, 0x641, 0x631, 0x64a, 0x642, 0x64a, 0x627, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x20, 0x623, 0x641, 0x631, 0x64a,
0x642, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x623, 0x641, 0x631, 0x64a, 0x642, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x641, 0x631, 0x64a, 0x642, 0x64a,
0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x641, 0x631, 0x64a, 0x642, 0x64a,
0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x623, 0x644, 0x627, 0x633, 0x643, 0x627, 0x627, 0x644, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x20, 0x644,
0x623, 0x644, 0x627, 0x633, 0x643, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x623, 0x644, 0x627, 0x633, 0x643, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x645, 0x627,
0x632, 0x648, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623,
0x645, 0x627, 0x632, 0x648, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x645, 0x627, 0x632,
0x648, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x627, 0x644, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x631, 0x643, 0x632, 0x64a,
0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627, 0x644, 0x634,
0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x20, 0x627, 0x644, 0x645, 0x631,
0x643, 0x632, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20,
0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20, 0x627,
0x644, 0x645, 0x631, 0x643, 0x632, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a,
0x643, 0x627, 0x20, 0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627,
0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x634, 0x631, 0x642,
0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627, 0x644,
0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x20, 0x627, 0x644, 0x634,
0x631, 0x642, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20,
0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20, 0x627,
0x644, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643,
0x627, 0x20, 0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x62c, 0x628, 0x644, 0x64a,
0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627, 0x644, 0x634,
0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x62c, 0x628, 0x644, 0x64a, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627, 0x20, 0x627,
0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x627, 0x644, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x62c, 0x628, 0x644, 0x64a, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x20, 0x644, 0x623, 0x645, 0x631, 0x64a, 0x643, 0x627,
0x20, 0x627, 0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627, 0x644,
0x647, 0x627, 0x62f, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644,
0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627, 0x644, 0x647, 0x627, 0x62f, 0x64a, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627, 0x644, 0x647, 0x627, 0x62f,
0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x623, 0x646, 0x627, 0x62f, 0x64a, 0x631, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x623, 0x646, 0x627, 0x62f, 0x64a, 0x631, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20, 0x644, 0x623, 0x646, 0x627, 0x62f, 0x64a,
0x631, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x622, 0x628, 0x64a, 0x627, 0x627,
0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x20, 0x644, 0x622, 0x628, 0x64a, 0x627, 0x627, 0x644, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20, 0x644, 0x623,
0x628, 0x64a, 0x627, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627,
0x644, 0x639, 0x631, 0x628, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x639, 0x631, 0x628, 0x64a, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644,
0x639, 0x631, 0x628, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x631, 0x62c, 0x646, 0x62a,
0x64a, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x631,
0x62c, 0x646, 0x62a, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x631, 0x62c, 0x646,
0x62a, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x644, 0x623, 0x631,
0x62c, 0x646, 0x62a, 0x64a, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a,
0x631, 0x628, 0x20, 0x627, 0x644, 0x623, 0x631, 0x62c, 0x646, 0x62a, 0x64a, 0x646,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x644, 0x623, 0x631, 0x62c, 0x646, 0x62a,
0x64a, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x623, 0x631, 0x645, 0x64a, 0x646, 0x64a, 0x627, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x623, 0x631, 0x645, 0x64a, 0x646, 0x64a, 0x627, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x623, 0x631, 0x645, 0x64a, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x623, 0x637,
0x644, 0x633, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x20, 0x627, 0x644, 0x623, 0x637, 0x644, 0x633,
0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x20, 0x627, 0x644, 0x623, 0x637, 0x644, 0x633, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x648,
0x633, 0x637, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x648, 0x633, 0x637, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x63a, 0x631, 0x628, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a,
0x631, 0x628, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x648, 0x633, 0x637, 0x20,
0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642,
0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x623, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a,
0x631, 0x628, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x623, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x623, 0x630, 0x631, 0x628, 0x64a, 0x62c, 0x627, 0x646, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x623, 0x630, 0x631, 0x628, 0x64a, 0x62c, 0x627, 0x646, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x623, 0x630, 0x631, 0x628, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x632, 0x648,
0x631, 0x633, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x632, 0x648, 0x631,
0x633, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x623, 0x632, 0x648, 0x631, 0x633, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x646, 0x63a, 0x644,
0x627, 0x62f, 0x64a, 0x634, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x646,
0x63a, 0x644, 0x627, 0x62f, 0x64a, 0x634, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x646, 0x63a, 0x644, 0x627,
0x62f, 0x64a, 0x634, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x628, 0x648, 0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x628, 0x648, 0x644, 0x64a, 0x641, 0x64a, 0x627, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x628, 0x631, 0x627, 0x632, 0x64a, 0x644, 0x64a, 0x627,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x631, 0x627, 0x632, 0x64a, 0x644,
0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x628, 0x631, 0x627, 0x632, 0x64a, 0x644, 0x64a, 0x627, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x628, 0x631, 0x648, 0x646, 0x627, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x627, 0x644, 0x631, 0x623, 0x633, 0x20, 0x627, 0x644, 0x623, 0x62e, 0x636, 0x631,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x631, 0x623, 0x633, 0x20,
0x627, 0x644, 0x623, 0x62e, 0x636, 0x631, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x631, 0x623, 0x633,
0x20, 0x627, 0x644, 0x623, 0x62e, 0x636, 0x631, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634, 0x627, 0x645,
0x648, 0x631, 0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634, 0x627,
0x62a, 0x627, 0x645, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634, 0x627,
0x62a, 0x627, 0x645, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634, 0x627, 0x62a, 0x627, 0x645, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a,
0x634, 0x64a, 0x644, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634,
0x64a, 0x644, 0x64a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x634, 0x64a, 0x644, 0x64a, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x648, 0x64a,
0x628, 0x627, 0x644, 0x633, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x634, 0x648, 0x64a, 0x628, 0x627, 0x644, 0x633, 0x627, 0x646, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20, 0x644, 0x634, 0x648, 0x64a, 0x628,
0x627, 0x644, 0x633, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c,
0x632, 0x631, 0x20, 0x627, 0x644, 0x643, 0x631, 0x64a, 0x633, 0x645, 0x627, 0x633,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x643, 0x648,
0x643, 0x648, 0x633, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x644,
0x648, 0x645, 0x628, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643,
0x648, 0x644, 0x648, 0x645, 0x628, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x644, 0x648,
0x645, 0x628, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x643, 0x648, 0x648,
0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x643,
0x648, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x643, 0x648, 0x643, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643,
0x648, 0x628, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x628,
0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x643, 0x648, 0x628, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62f, 0x627, 0x641, 0x64a, 0x632,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62f, 0x64a, 0x20, 0x645, 0x648, 0x646,
0x62a, 0x20, 0x62f, 0x648, 0x20, 0x631, 0x648, 0x641, 0x64a, 0x644, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x64a, 0x645, 0x648, 0x631, 0x20, 0x627, 0x644,
0x634, 0x631, 0x642, 0x64a, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c,
0x632, 0x64a, 0x631, 0x629, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x64a, 0x631, 0x629, 0x20, 0x627, 0x633, 0x62a,
0x631, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62c, 0x632, 0x64a, 0x631, 0x629, 0x20, 0x627, 0x633, 0x62a, 0x631,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x625, 0x643, 0x648, 0x627, 0x62f, 0x648, 0x631, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623, 0x648, 0x631, 0x648,
0x628, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20,
0x623, 0x648, 0x631, 0x648, 0x628, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x623,
0x648, 0x631, 0x648, 0x628, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x623, 0x648,
0x631, 0x648, 0x628, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631,
0x642, 0x20, 0x623, 0x648, 0x631, 0x648, 0x628, 0x627, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642,
0x20, 0x623, 0x648, 0x631, 0x648, 0x628, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644,
0x623, 0x648, 0x631, 0x648, 0x628, 0x64a, 0x20, 0x28, 0x623, 0x643, 0x62b, 0x631,
0x20, 0x634, 0x631, 0x642, 0x64b, 0x627, 0x29, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x648, 0x631, 0x648, 0x628, 0x627, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x648, 0x631,
0x648, 0x628, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x623, 0x648, 0x631, 0x648,
0x628, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x641, 0x648, 0x643, 0x644, 0x627,
0x646, 0x62f, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20,
0x641, 0x648, 0x643, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20,
0x641, 0x648, 0x643, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x64a, 0x62c, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x64a, 0x62c, 0x64a, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641,
0x64a, 0x62c, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x648, 0x64a, 0x627, 0x646, 0x627, 0x20, 0x627,
0x644, 0x641, 0x631, 0x646, 0x633, 0x64a, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x645, 0x642, 0x627, 0x637, 0x639, 0x627, 0x62a, 0x20, 0x627,
0x644, 0x641, 0x631, 0x646, 0x633, 0x64a, 0x629, 0x20, 0x627, 0x644, 0x62c, 0x646,
0x648, 0x628, 0x64a, 0x629, 0x20, 0x648, 0x627, 0x644, 0x623, 0x646, 0x62a, 0x627,
0x631, 0x62a, 0x64a, 0x643, 0x64a, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x63a, 0x644, 0x627, 0x628, 0x627, 0x63a, 0x648, 0x633, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62c, 0x627, 0x645, 0x628, 0x64a, 0x631, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x648, 0x631,
0x62c, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x62c, 0x64a, 0x644, 0x628,
0x631, 0x62a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x64a, 0x646,
0x62a, 0x634, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20,
0x63a, 0x631, 0x64a, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x63a, 0x631, 0x64a, 0x646, 0x644, 0x627,
0x646, 0x62f, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x63a, 0x631, 0x64a, 0x646, 0x644,
0x627, 0x646, 0x62f, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x63a, 0x631, 0x64a, 0x646,
0x644, 0x627, 0x646, 0x62f, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631,
0x628, 0x20, 0x63a, 0x631, 0x64a, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a,
0x631, 0x628, 0x20, 0x63a, 0x631, 0x64a, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x63a, 0x648, 0x627, 0x645, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644,
0x62e, 0x644, 0x64a, 0x62c, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x64a,
0x627, 0x646, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x627, 0x648,
0x627, 0x64a, 0x20, 0x623, 0x644, 0x648, 0x62a, 0x64a, 0x627, 0x646, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x647, 0x627, 0x648, 0x627, 0x64a, 0x20, 0x623, 0x644,
0x648, 0x62a, 0x64a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x627, 0x648, 0x627, 0x64a, 0x20,
0x623, 0x644, 0x648, 0x62a, 0x64a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648, 0x646, 0x63a,
0x20, 0x643, 0x648, 0x646, 0x63a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647,
0x648, 0x646, 0x63a, 0x20, 0x643, 0x648, 0x646, 0x63a, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648, 0x646,
0x63a, 0x20, 0x643, 0x648, 0x646, 0x63a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648, 0x641, 0x62f, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648, 0x641, 0x62f, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x647, 0x648,
0x641, 0x62f, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x647, 0x646, 0x62f, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627, 0x644, 0x647,
0x646, 0x62f, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x647,
0x646, 0x62f, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x646, 0x64a, 0x629, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x648, 0x633, 0x637, 0x20, 0x625, 0x646, 0x62f, 0x648,
0x646, 0x64a, 0x633, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634,
0x631, 0x642, 0x20, 0x625, 0x646, 0x62f, 0x648, 0x646, 0x64a, 0x633, 0x64a, 0x627,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x625, 0x646,
0x62f, 0x648, 0x646, 0x64a, 0x633, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x625, 0x64a, 0x631, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x625, 0x64a, 0x631, 0x627, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625, 0x64a, 0x631, 0x627, 0x646, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x625, 0x631, 0x643, 0x648, 0x62a, 0x633, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x625, 0x631, 0x643, 0x648, 0x62a, 0x633, 0x643, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625, 0x631, 0x643,
0x648, 0x62a, 0x633, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x644,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x625, 0x633, 0x631, 0x627, 0x626, 0x64a,
0x644, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x625, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x644, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644,
0x64a, 0x627, 0x628, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627,
0x644, 0x64a, 0x627, 0x628, 0x627, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x64a, 0x627, 0x628,
0x627, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x643, 0x627, 0x645, 0x634, 0x627, 0x62a, 0x643, 0x627, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x62a, 0x631, 0x648, 0x628, 0x627,
0x641, 0x644, 0x648, 0x641, 0x633, 0x643, 0x2d, 0x643, 0x627, 0x645, 0x62a, 0x634,
0x627, 0x62a, 0x633, 0x643, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628,
0x64a, 0x62a, 0x631, 0x648, 0x628, 0x627, 0x641, 0x644, 0x648, 0x641, 0x633, 0x643,
0x2d, 0x643, 0x627, 0x645, 0x62a, 0x634, 0x627, 0x62a, 0x633, 0x643, 0x64a, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x634, 0x631, 0x642, 0x20, 0x643, 0x627, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627,
0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x643,
0x627, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x643, 0x648, 0x631, 0x64a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x643, 0x648, 0x631, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x648, 0x631, 0x64a, 0x627,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x643, 0x648, 0x633, 0x631, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x643, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x643, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a,
0x627, 0x631, 0x633, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x627,
0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x20, 0x644, 0x643, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631,
0x633, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x642, 0x64a, 0x631, 0x63a,
0x64a, 0x632, 0x633, 0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x62c, 0x632, 0x631, 0x20, 0x644, 0x627, 0x64a, 0x646, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x644, 0x648, 0x631, 0x62f, 0x20, 0x647, 0x627, 0x648, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x644, 0x648, 0x631, 0x62f, 0x20, 0x647, 0x627, 0x648,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x627, 0x644, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x20, 0x644, 0x644,
0x648, 0x631, 0x62f, 0x20, 0x647, 0x627, 0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x645, 0x627, 0x63a, 0x627, 0x62f, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x645, 0x627, 0x63a, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x627,
0x63a, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x627, 0x644, 0x64a, 0x632, 0x64a,
0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x627,
0x644, 0x645, 0x627, 0x644, 0x62f, 0x64a, 0x641, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x645, 0x627, 0x631, 0x643, 0x64a, 0x633, 0x627, 0x633, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x645, 0x627, 0x631, 0x634, 0x627,
0x644, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x631, 0x64a, 0x634,
0x64a, 0x648, 0x633, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x631,
0x64a, 0x634, 0x64a, 0x648, 0x633, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x631, 0x64a, 0x634, 0x64a,
0x648, 0x633, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x645, 0x627, 0x648, 0x633, 0x648, 0x646, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627, 0x644,
0x647, 0x627, 0x62f, 0x64a, 0x20, 0x644, 0x644, 0x645, 0x643, 0x633, 0x64a, 0x643,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637,
0x20, 0x627, 0x644, 0x647, 0x627, 0x62f, 0x64a, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x20, 0x644, 0x644, 0x645, 0x643, 0x633, 0x64a, 0x643, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x645, 0x62d, 0x64a, 0x637, 0x20, 0x627,
0x644, 0x647, 0x627, 0x62f, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x20, 0x644, 0x644, 0x645, 0x643, 0x633, 0x64a, 0x643, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x623, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x62a, 0x648,
0x631, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x644, 0x627, 0x646,
0x20, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x644, 0x627, 0x646,
0x20, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x633, 0x643, 0x648,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x648, 0x633, 0x643, 0x648, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x645, 0x648, 0x633, 0x643, 0x648, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x645, 0x64a, 0x627, 0x646, 0x645, 0x627,
0x631, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x627, 0x648, 0x631, 0x648,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x628, 0x627, 0x644, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x627, 0x644, 0x64a, 0x62f, 0x648, 0x646,
0x64a, 0x627, 0x20, 0x627, 0x644, 0x62c, 0x62f, 0x64a, 0x62f, 0x629, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x643, 0x627, 0x644, 0x64a, 0x62f, 0x648, 0x646, 0x64a,
0x627, 0x20, 0x627, 0x644, 0x62c, 0x62f, 0x64a, 0x62f, 0x629, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x643, 0x627,
0x644, 0x64a, 0x62f, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x62c, 0x62f,
0x64a, 0x62f, 0x629, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x644, 0x646, 0x62f,
0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648, 0x632, 0x64a,
0x644, 0x646, 0x62f, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x644, 0x646,
0x62f, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648, 0x641, 0x627, 0x648, 0x646, 0x62f, 0x644,
0x627, 0x646, 0x62f, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a, 0x648,
0x641, 0x627, 0x648, 0x646, 0x62f, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627, 0x644,
0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x64a,
0x648, 0x641, 0x627, 0x648, 0x646, 0x62f, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646,
0x64a, 0x648, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x64a,
0x631, 0x629, 0x20, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x64a, 0x631, 0x629, 0x20, 0x646, 0x648,
0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x64a, 0x631, 0x629, 0x20,
0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x64a, 0x631, 0x646,
0x627, 0x646, 0x62f, 0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648,
0x646, 0x647, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x631, 0x646,
0x627, 0x646, 0x62f, 0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648,
0x646, 0x647, 0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x641, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x648, 0x20,
0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627, 0x20, 0x627,
0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c,
0x632, 0x631, 0x20, 0x645, 0x627, 0x631, 0x64a, 0x627, 0x646, 0x627, 0x20, 0x627,
0x644, 0x634, 0x645, 0x627, 0x644, 0x64a, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x646, 0x648, 0x641, 0x648, 0x633, 0x64a, 0x628, 0x64a, 0x631, 0x633, 0x643,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x648, 0x641, 0x648, 0x633, 0x64a,
0x628, 0x64a, 0x631, 0x633, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x646, 0x648, 0x641, 0x648, 0x633, 0x64a,
0x628, 0x64a, 0x631, 0x633, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x645, 0x633, 0x643, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x645, 0x633, 0x643, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623,
0x648, 0x645, 0x633, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627, 0x643, 0x633, 0x62a, 0x627, 0x646,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627, 0x643, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x628, 0x627, 0x643, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627,
0x644, 0x627, 0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627, 0x628,
0x648, 0x627, 0x20, 0x63a, 0x64a, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x644, 0x62c,
0x62f, 0x64a, 0x62f, 0x629, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627,
0x631, 0x627, 0x63a, 0x648, 0x627, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x628, 0x627, 0x631, 0x627, 0x63a, 0x648, 0x627, 0x64a, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x627, 0x631,
0x627, 0x63a, 0x648, 0x627, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x631, 0x648, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x631, 0x648, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x631,
0x648, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x627, 0x644, 0x641, 0x64a, 0x644, 0x628, 0x64a, 0x646, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x627, 0x644, 0x641, 0x64a, 0x644, 0x628, 0x64a, 0x646,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x627, 0x644, 0x641, 0x64a, 0x644, 0x628, 0x64a, 0x646, 0x20, 0x627, 0x644,
0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632,
0x631, 0x20, 0x641, 0x64a, 0x646, 0x643, 0x633, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x628, 0x64a, 0x64a, 0x631, 0x20, 0x648,
0x645, 0x64a, 0x643, 0x648, 0x644, 0x648, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x633, 0x627, 0x646, 0x62a, 0x20, 0x628, 0x64a, 0x64a, 0x631, 0x20, 0x648,
0x645, 0x64a, 0x643, 0x648, 0x644, 0x648, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x646, 0x62a,
0x20, 0x628, 0x64a, 0x64a, 0x631, 0x20, 0x648, 0x645, 0x64a, 0x643, 0x648, 0x644,
0x648, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x62a, 0x643, 0x64a, 0x631, 0x646, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x628, 0x648, 0x646, 0x627, 0x628, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x628, 0x64a, 0x648, 0x646, 0x63a, 0x20, 0x64a, 0x627,
0x646, 0x63a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x631, 0x648, 0x64a, 0x646,
0x64a, 0x648, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x631, 0x648, 0x62b,
0x64a, 0x631, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x62e,
0x627, 0x644, 0x64a, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627,
0x62e, 0x627, 0x644, 0x64a, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x62e, 0x627, 0x644, 0x64a,
0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x633, 0x627, 0x645, 0x627, 0x631, 0x627, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x633, 0x645, 0x627, 0x631, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x633, 0x645, 0x627, 0x631, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x645, 0x648, 0x627,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x645, 0x648, 0x627, 0x20,
0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x633, 0x627, 0x645, 0x648, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x633, 0x64a, 0x634, 0x644, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x633, 0x646, 0x63a, 0x627, 0x641, 0x648, 0x631, 0x629,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x632, 0x631, 0x20, 0x633, 0x644,
0x64a, 0x645, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62c, 0x646,
0x648, 0x628, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x64a, 0x627, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x633, 0x648, 0x631, 0x64a, 0x646, 0x627, 0x645, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x633, 0x627, 0x64a, 0x648, 0x648, 0x627, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x627, 0x647, 0x64a, 0x62a, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x627, 0x64a, 0x628, 0x64a, 0x647, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x627, 0x64a, 0x628, 0x64a, 0x647, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a,
0x627, 0x64a, 0x628, 0x64a, 0x647, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x637, 0x627, 0x62c, 0x643, 0x633, 0x62a,
0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x648, 0x643, 0x64a,
0x644, 0x627, 0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x648, 0x646,
0x63a, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x648, 0x646, 0x63a,
0x627, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x62a, 0x648, 0x646, 0x63a, 0x627, 0x20, 0x627, 0x644, 0x635, 0x64a,
0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x634, 0x648, 0x643, 0x62a,
0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x631, 0x643, 0x645, 0x627, 0x646, 0x633,
0x62a, 0x627, 0x646, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x631, 0x643,
0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x631, 0x643, 0x645,
0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x62a, 0x648, 0x641, 0x627, 0x644,
0x648, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x631, 0x648, 0x63a,
0x648, 0x627, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x631,
0x648, 0x63a, 0x648, 0x627, 0x64a, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x631, 0x648, 0x63a, 0x648,
0x627, 0x64a, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x623, 0x648, 0x632, 0x628, 0x643, 0x633, 0x62a, 0x627, 0x646,
0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x632, 0x628, 0x643, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x623, 0x648, 0x632, 0x628, 0x643, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x641, 0x627, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x641, 0x627, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x20, 0x627,
0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641,
0x627, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641,
0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x646, 0x632, 0x648, 0x64a,
0x644, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x641, 0x644, 0x627, 0x62f,
0x64a, 0x641, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x641, 0x644, 0x627, 0x62f, 0x64a, 0x641, 0x648, 0x633, 0x62a, 0x648, 0x643,
0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x641, 0x644, 0x627, 0x62f, 0x64a, 0x641, 0x648, 0x633, 0x62a, 0x648, 0x643,
0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a,
0x20, 0x641, 0x648, 0x644, 0x63a, 0x648, 0x63a, 0x631, 0x627, 0x62f, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x641, 0x648, 0x644, 0x63a, 0x648, 0x63a, 0x631, 0x627,
0x62f, 0x20, 0x627, 0x644, 0x631, 0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a,
0x62a, 0x20, 0x641, 0x648, 0x644, 0x63a, 0x648, 0x63a, 0x631, 0x627, 0x62f, 0x20,
0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x641, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x62c, 0x632, 0x64a, 0x631, 0x629, 0x20, 0x648, 0x64a, 0x643, 0x62a, 0x648, 0x642,
0x64a, 0x62a, 0x20, 0x648, 0x627, 0x644, 0x64a, 0x633, 0x20, 0x648, 0x20, 0x641,
0x648, 0x62a, 0x648, 0x646, 0x627, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a,
0x627, 0x643, 0x648, 0x62a, 0x633, 0x643, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20,
0x64a, 0x627, 0x643, 0x648, 0x62a, 0x633, 0x643, 0x20, 0x627, 0x644, 0x631, 0x633,
0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a, 0x627, 0x643, 0x648,
0x62a, 0x633, 0x643, 0x20, 0x627, 0x644, 0x635, 0x64a, 0x641, 0x64a, 0x62a, 0x648,
0x642, 0x64a, 0x62a, 0x20, 0x64a, 0x643, 0x627, 0x62a, 0x631, 0x64a, 0x646, 0x628,
0x648, 0x631, 0x63a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a, 0x643, 0x627,
0x62a, 0x631, 0x64a, 0x646, 0x628, 0x648, 0x631, 0x63a, 0x20, 0x627, 0x644, 0x631,
0x633, 0x645, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a, 0x643, 0x627,
0x62a, 0x631, 0x64a, 0x646, 0x628, 0x648, 0x631, 0x63a, 0x20, 0x627, 0x644, 0x635,
0x64a, 0x641, 0x64a, 0x62a, 0x648, 0x642, 0x64a, 0x62a, 0x20, 0x64a, 0x648, 0x643,
0x648, 0x646, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64,
0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f,
0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20,
0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019,
0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x79, 0x61, 0x20,
0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019,
0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x79, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e,
0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x6f,
0x6e, 0x74, 0x61, 0x6e, 0x79, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72,
0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x2019, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x6f,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x6f, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x6f, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x65, 0x73, 0x74,
0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x75, 0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x75, 0x65,
0x73, 0x74, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x75, 0x65, 0x73, 0x74,
0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x6f, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64,
0x69, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x531, 0x586, 0x572, 0x561, 0x576, 0x57d, 0x57f,
0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531,
0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20,
0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x540, 0x561, 0x580, 0x561, 0x57e, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575,
0x561, 0x576, 0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f,
0x575, 0x561, 0x576, 0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561,
0x576, 0x20, 0x531, 0x586, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x56c, 0x575, 0x561, 0x57d, 0x56f, 0x561, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x56c, 0x575, 0x561,
0x57d, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x56c,
0x575, 0x561, 0x57d, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x531, 0x574, 0x561, 0x566, 0x578, 0x576, 0x575, 0x561, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x574, 0x561, 0x566, 0x578, 0x576, 0x575,
0x561, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x574, 0x561, 0x566, 0x578,
0x576, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576,
0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x574, 0x565,
0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561,
0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561,
0x56f, 0x561, 0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f, 0x561, 0x575,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575,
0x561, 0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f, 0x561, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565,
0x56c, 0x575, 0x561, 0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f, 0x561,
0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c,
0x575, 0x561, 0x576, 0x20, 0x531, 0x574, 0x565, 0x580, 0x56b, 0x56f, 0x561, 0x575,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53c, 0x565, 0x57c, 0x576, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x20, 0x28,
0x531, 0x544, 0x546, 0x29, 0x53c, 0x565, 0x57c, 0x576, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x20, 0x28, 0x531, 0x544, 0x546, 0x29, 0x53c,
0x565, 0x57c, 0x576, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x20, 0x28, 0x531, 0x544, 0x546, 0x29, 0x53d, 0x561, 0x572, 0x561, 0x572, 0x585,
0x57e, 0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53d, 0x561, 0x572, 0x561, 0x572, 0x585,
0x57e, 0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x53d, 0x561, 0x572, 0x561, 0x572, 0x585, 0x57e, 0x56f, 0x56b,
0x561, 0x576, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x531, 0x57a, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x57a, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x531, 0x57a, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54d, 0x561, 0x578, 0x582, 0x564, 0x575, 0x561, 0x576, 0x20, 0x531, 0x580, 0x561,
0x562, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54d, 0x561, 0x578, 0x582, 0x564, 0x575, 0x561, 0x576, 0x20, 0x531, 0x580,
0x561, 0x562, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d,
0x561, 0x578, 0x582, 0x564, 0x575, 0x561, 0x576, 0x20, 0x531, 0x580, 0x561, 0x562,
0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x563,
0x565, 0x576, 0x57f, 0x56b, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x563, 0x565, 0x576, 0x57f, 0x56b, 0x576,
0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x576, 0x561, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x563, 0x565,
0x576, 0x57f, 0x56b, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x531, 0x580, 0x563,
0x565, 0x576, 0x57f, 0x56b, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576,
0x20, 0x531, 0x580, 0x563, 0x565, 0x576, 0x57f, 0x56b, 0x576, 0x561, 0x575, 0x56b,
0x20, 0x57d, 0x57f, 0x576, 0x561, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561,
0x576, 0x20, 0x531, 0x580, 0x563, 0x565, 0x576, 0x57f, 0x56b, 0x576, 0x561, 0x575,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x561, 0x575, 0x561, 0x57d, 0x57f,
0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540,
0x561, 0x575, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x540, 0x561, 0x575, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x57f, 0x56c, 0x561, 0x576, 0x57f, 0x575, 0x561, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x57f, 0x56c, 0x561,
0x576, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x57f,
0x56c, 0x561, 0x576, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20,
0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580,
0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580,
0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531,
0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f,
0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561,
0x575, 0x56b, 0x20, 0x561, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580,
0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580,
0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x580, 0x587, 0x574, 0x57f,
0x575, 0x561, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f,
0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f,
0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x580, 0x587, 0x574,
0x57f, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587,
0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561,
0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x531, 0x57e,
0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x531,
0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20,
0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f,
0x575, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c, 0x56b,
0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574,
0x57f, 0x575, 0x561, 0x576, 0x20, 0x531, 0x57e, 0x57d, 0x57f, 0x580, 0x561, 0x56c,
0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x564, 0x580,
0x562, 0x565, 0x57b, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x564, 0x580, 0x562, 0x565, 0x57b, 0x561, 0x576, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x531, 0x564, 0x580, 0x562, 0x565, 0x57b, 0x561, 0x576,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x566, 0x578, 0x580, 0x575, 0x561,
0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x566, 0x578, 0x580, 0x575, 0x561,
0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x566, 0x578, 0x580, 0x575, 0x561, 0x576, 0x20, 0x56f,
0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x532, 0x561, 0x576, 0x563, 0x56c, 0x561, 0x564, 0x565, 0x577, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x561, 0x576, 0x563, 0x56c, 0x561,
0x564, 0x565, 0x577, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x561, 0x576,
0x563, 0x56c, 0x561, 0x564, 0x565, 0x577, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x532, 0x578, 0x582, 0x569, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x532, 0x578, 0x56c, 0x56b, 0x57e, 0x56b, 0x561, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x580, 0x561, 0x566,
0x56b, 0x56c, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x532, 0x580, 0x561, 0x566, 0x56b, 0x56c, 0x56b, 0x561, 0x575, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x580, 0x561, 0x566, 0x56b, 0x56c, 0x56b,
0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x532, 0x580, 0x578, 0x582,
0x576, 0x565, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x53f, 0x561, 0x562, 0x578, 0x20, 0x54e, 0x565, 0x580, 0x564, 0x565, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x561, 0x562, 0x578, 0x20,
0x54e, 0x565, 0x580, 0x564, 0x565, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x561, 0x562, 0x578, 0x20, 0x54e, 0x565, 0x580, 0x564, 0x565, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x549, 0x561, 0x574, 0x578, 0x57c, 0x578, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x561, 0x569, 0x565, 0x574,
0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x549, 0x561, 0x569, 0x565, 0x574, 0x20, 0x56f, 0x572, 0x566, 0x578,
0x582, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x561, 0x569, 0x565, 0x574, 0x20,
0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x56b,
0x56c, 0x56b, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549,
0x56b, 0x56c, 0x56b, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x56b, 0x56c,
0x56b, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x56b, 0x576, 0x561, 0x57d,
0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x549, 0x56b, 0x576, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x549, 0x56b, 0x576, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20,
0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x549, 0x578, 0x575, 0x562, 0x561, 0x56c, 0x57d, 0x561,
0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x549, 0x578,
0x575, 0x562, 0x561, 0x56c, 0x57d, 0x561, 0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x549, 0x578, 0x575, 0x562, 0x561, 0x56c, 0x57d, 0x561, 0x576, 0x56b, 0x20,
0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54d, 0x578, 0x582, 0x580, 0x562, 0x20, 0x53e, 0x576,
0x576, 0x564, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x56f, 0x578, 0x57d,
0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x56c, 0x578,
0x582, 0x574, 0x562, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x53f, 0x578, 0x56c, 0x578, 0x582, 0x574, 0x562, 0x56b, 0x561,
0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x56c, 0x578, 0x582,
0x574, 0x562, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x578, 0x582, 0x56f, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x582,
0x56f, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x582, 0x56f, 0x56b, 0x20, 0x56f, 0x572,
0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56f, 0x56b, 0x57d, 0x561, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x53f, 0x578, 0x582, 0x562, 0x561, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x582, 0x562, 0x561, 0x575,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x582, 0x562, 0x561, 0x575,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x534, 0x565, 0x575, 0x57e, 0x56b, 0x57d,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x534, 0x575, 0x578,
0x582, 0x574, 0x578, 0x576, 0x20, 0x564, 0x2019, 0x545, 0x578, 0x582, 0x580, 0x57e,
0x56b, 0x56c, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531,
0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x539, 0x56b, 0x574, 0x578,
0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x536, 0x561,
0x57f, 0x56f, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x536, 0x561, 0x57f, 0x56f, 0x56b, 0x20, 0x56f,
0x572, 0x566, 0x578, 0x582, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x536, 0x561, 0x57f,
0x56f, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x537, 0x56f, 0x57e, 0x561, 0x564, 0x578, 0x580, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576,
0x561, 0x56f, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578, 0x57a, 0x561, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565, 0x576,
0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580,
0x578, 0x57a, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x565,
0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x535, 0x57e,
0x580, 0x578, 0x57a, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531,
0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578,
0x57a, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580,
0x578, 0x57a, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580,
0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578, 0x57a,
0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x56b, 0x576, 0x57d,
0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580,
0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578, 0x57a,
0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531,
0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578,
0x57a, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587,
0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x535, 0x57e, 0x580, 0x578, 0x57a, 0x561,
0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x578, 0x56c, 0x584, 0x56c,
0x565, 0x576, 0x564, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576,
0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556,
0x578, 0x56c, 0x584, 0x56c, 0x565, 0x576, 0x564, 0x575, 0x561, 0x576, 0x20, 0x56f,
0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x556, 0x578, 0x56c, 0x584, 0x56c, 0x565, 0x576, 0x564, 0x575, 0x561, 0x576, 0x20,
0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x556, 0x56b, 0x57b, 0x56b, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x556, 0x56b, 0x57b, 0x56b, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x556, 0x56b, 0x57b, 0x56b, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x580,
0x561, 0x576, 0x57d, 0x56b, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x533, 0x57e, 0x56b,
0x561, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x556, 0x580, 0x561, 0x576, 0x57d, 0x56b, 0x561, 0x56f, 0x561, 0x576, 0x20,
0x570, 0x561, 0x580, 0x561, 0x57e, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x587, 0x20,
0x561, 0x576, 0x57f, 0x561, 0x580, 0x56f, 0x57f, 0x56b, 0x564, 0x575, 0x561, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x533, 0x561, 0x56c, 0x561,
0x57a, 0x561, 0x563, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566,
0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x533, 0x561, 0x574, 0x562, 0x575, 0x565, 0x20, 0x56f, 0x572, 0x566, 0x56b,
0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54e, 0x580, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54e, 0x580, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b,
0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x580, 0x561, 0x57d, 0x57f, 0x561, 0x576,
0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54b, 0x56b, 0x56c, 0x562, 0x565, 0x580,
0x569, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x533, 0x580, 0x56b, 0x576, 0x57e,
0x56b, 0x579, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531,
0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x533, 0x580, 0x565, 0x576,
0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576,
0x20, 0x533, 0x580, 0x565, 0x576, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561, 0x575,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575,
0x561, 0x576, 0x20, 0x533, 0x580, 0x565, 0x576, 0x56c, 0x561, 0x576, 0x564, 0x56b,
0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574,
0x57f, 0x575, 0x561, 0x576, 0x20, 0x533, 0x580, 0x565, 0x576, 0x56c, 0x561, 0x576,
0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x533, 0x580,
0x565, 0x576, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20,
0x533, 0x580, 0x565, 0x576, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x580, 0x57d, 0x56b, 0x581, 0x20,
0x56e, 0x578, 0x581, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x533, 0x561, 0x575,
0x561, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x540, 0x561, 0x57e, 0x561, 0x575, 0x561, 0x576, 0x2d, 0x561, 0x56c, 0x565,
0x578, 0x582, 0x569, 0x575, 0x561, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x540, 0x561, 0x57e, 0x561, 0x575, 0x561, 0x576, 0x2d, 0x561, 0x56c,
0x565, 0x578, 0x582, 0x569, 0x575, 0x561, 0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x540, 0x561, 0x57e, 0x561, 0x575, 0x561, 0x576, 0x2d, 0x561, 0x56c, 0x565, 0x578,
0x582, 0x569, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x578,
0x576, 0x56f, 0x578, 0x576, 0x563, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x540, 0x578, 0x576, 0x56f, 0x578, 0x576, 0x563, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x540, 0x578, 0x576, 0x56f, 0x578, 0x576, 0x563, 0x56b, 0x20,
0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x540, 0x578, 0x57e, 0x564, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x578, 0x57e, 0x564, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x540, 0x578, 0x57e, 0x564, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x540, 0x576, 0x564, 0x56f, 0x561, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x540, 0x576, 0x564, 0x56f, 0x561, 0x56f, 0x561, 0x576,
0x20, 0x585, 0x57e, 0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x576, 0x564, 0x56f, 0x561, 0x579,
0x56b, 0x576, 0x561, 0x56f, 0x561, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x53f, 0x565, 0x576, 0x57f, 0x580, 0x578, 0x576, 0x561, 0x56f, 0x561,
0x576, 0x20, 0x53b, 0x576, 0x564, 0x578, 0x576, 0x565, 0x566, 0x56b, 0x561, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587,
0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x53b, 0x576, 0x564, 0x578, 0x576, 0x565,
0x566, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561, 0x576, 0x20, 0x53b, 0x576,
0x564, 0x578, 0x576, 0x565, 0x566, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x561, 0x576, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x561, 0x576, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x561, 0x576, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x53b, 0x580, 0x56f, 0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x56f, 0x578, 0x582,
0x57f, 0x57d, 0x56f, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b, 0x580, 0x56f,
0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53b,
0x57d, 0x580, 0x561, 0x575, 0x565, 0x56c, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x53b, 0x57d, 0x580, 0x561, 0x575, 0x565, 0x56c, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53b, 0x57d, 0x580, 0x561, 0x575, 0x565, 0x56c, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x543, 0x561, 0x57a, 0x578, 0x576, 0x56b, 0x561,
0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x543, 0x561,
0x57a, 0x578, 0x576, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x543, 0x561, 0x57a, 0x578, 0x576, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x531, 0x580, 0x587, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x542,
0x561, 0x566, 0x561, 0x56d, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x531, 0x580, 0x587, 0x574, 0x57f, 0x575, 0x561,
0x576, 0x20, 0x542, 0x561, 0x566, 0x561, 0x56d, 0x57d, 0x57f, 0x561, 0x576, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f, 0x578, 0x580, 0x565,
0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x578, 0x580, 0x565, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x578, 0x580, 0x565, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x578, 0x57d, 0x580, 0x561, 0x565, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x53f, 0x580, 0x561, 0x57d, 0x576, 0x578, 0x575, 0x561, 0x580,
0x57d, 0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53f,
0x580, 0x561, 0x57d, 0x576, 0x578, 0x575, 0x561, 0x580, 0x57d, 0x56f, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x53f, 0x580, 0x561, 0x57d, 0x576, 0x578, 0x575, 0x561,
0x580, 0x57d, 0x56f, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x542, 0x580, 0x572,
0x566, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x53c, 0x561, 0x575, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576,
0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53c,
0x578, 0x580, 0x564, 0x20, 0x540, 0x561, 0x578, 0x582, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x53c, 0x578, 0x580, 0x564, 0x20, 0x540, 0x561,
0x578, 0x582, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x53c, 0x578, 0x580, 0x564,
0x20, 0x540, 0x561, 0x578, 0x582, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544,
0x561, 0x563, 0x561, 0x564, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x544, 0x561, 0x563, 0x561, 0x564, 0x561, 0x576, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x563, 0x561, 0x564, 0x561, 0x576, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x56c, 0x561, 0x575, 0x566, 0x56b,
0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544,
0x561, 0x56c, 0x564, 0x56b, 0x57e, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x580, 0x56f, 0x56b, 0x566, 0x575,
0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x580, 0x577, 0x561,
0x56c, 0x575, 0x561, 0x576, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565, 0x580,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x57e,
0x580, 0x56b, 0x56f, 0x56b, 0x578, 0x57d, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x544, 0x561, 0x57e, 0x580, 0x56b, 0x56f, 0x56b, 0x578, 0x57d,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x561, 0x57e, 0x580, 0x56b, 0x56f,
0x56b, 0x578, 0x57d, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x578, 0x578,
0x582, 0x57d, 0x578, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x544, 0x565, 0x584, 0x57d, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56d,
0x561, 0x572, 0x561, 0x572, 0x585, 0x57e, 0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d,
0x575, 0x561, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544,
0x565, 0x584, 0x57d, 0x56b, 0x56f, 0x561, 0x575, 0x56b, 0x20, 0x56d, 0x561, 0x572,
0x561, 0x572, 0x585, 0x57e, 0x56f, 0x56b, 0x561, 0x576, 0x578, 0x57d, 0x575, 0x561,
0x576, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x565, 0x584, 0x57d, 0x56b, 0x56f,
0x561, 0x575, 0x56b, 0x20, 0x56d, 0x561, 0x572, 0x561, 0x572, 0x585, 0x57e, 0x56f,
0x56b, 0x561, 0x576, 0x578, 0x57d, 0x575, 0x561, 0x576, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x548, 0x582, 0x56c, 0x561, 0x576, 0x20, 0x532, 0x561, 0x57f, 0x578, 0x580,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x56c,
0x561, 0x576, 0x20, 0x532, 0x561, 0x57f, 0x578, 0x580, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x548, 0x582, 0x56c, 0x561, 0x576, 0x20, 0x532, 0x561, 0x57f, 0x578,
0x580, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544, 0x578, 0x57d, 0x56f, 0x57e,
0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x544,
0x578, 0x57d, 0x56f, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x544, 0x578, 0x57d, 0x56f, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x544, 0x575, 0x561, 0x576, 0x574, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x561, 0x578, 0x582, 0x580, 0x578, 0x582,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x565, 0x57a,
0x561, 0x56c, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546,
0x578, 0x580, 0x20, 0x53f, 0x561, 0x56c, 0x565, 0x564, 0x578, 0x576, 0x56b, 0x561,
0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578,
0x580, 0x20, 0x53f, 0x561, 0x56c, 0x565, 0x564, 0x578, 0x576, 0x56b, 0x561, 0x575,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x20, 0x53f, 0x561,
0x56c, 0x565, 0x564, 0x578, 0x576, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x546, 0x578, 0x580, 0x20, 0x536, 0x565, 0x56c, 0x561, 0x576, 0x564,
0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x546, 0x578, 0x580, 0x20, 0x536, 0x565, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561,
0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x20, 0x536,
0x565, 0x56c, 0x561, 0x576, 0x564, 0x56b, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x546, 0x575, 0x578, 0x582, 0x586, 0x561, 0x578, 0x582, 0x576, 0x564,
0x56c, 0x565, 0x576, 0x564, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x546, 0x575, 0x578, 0x582, 0x586, 0x561, 0x578, 0x582, 0x576, 0x564, 0x56c,
0x565, 0x576, 0x564, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x575, 0x578,
0x582, 0x586, 0x561, 0x578, 0x582, 0x576, 0x564, 0x56c, 0x565, 0x576, 0x564, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x56b, 0x578, 0x582, 0x565, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x586,
0x578, 0x56c, 0x56f, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x580, 0x586, 0x578, 0x56c, 0x56f,
0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546,
0x578, 0x580, 0x586, 0x578, 0x56c, 0x56f, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x565, 0x580, 0x576, 0x561, 0x576, 0x564,
0x578, 0x582, 0x20, 0x564, 0x56b, 0x20, 0x546, 0x578, 0x580, 0x578, 0x576, 0x575,
0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556,
0x565, 0x580, 0x576, 0x561, 0x576, 0x564, 0x578, 0x582, 0x20, 0x564, 0x56b, 0x20,
0x546, 0x578, 0x580, 0x578, 0x576, 0x575, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x556, 0x565, 0x580, 0x576, 0x561, 0x576, 0x564, 0x578, 0x582, 0x20,
0x564, 0x56b, 0x20, 0x546, 0x578, 0x580, 0x578, 0x576, 0x575, 0x561, 0x575, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x57e, 0x578, 0x57d, 0x56b, 0x562,
0x56b, 0x580, 0x57d, 0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x546, 0x578, 0x57e, 0x578, 0x57d, 0x56b, 0x562, 0x56b, 0x580, 0x57d, 0x56f,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x546, 0x578, 0x57e, 0x578, 0x57d, 0x56b,
0x562, 0x56b, 0x580, 0x57d, 0x56f, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561,
0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x555,
0x574, 0x57d, 0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x555, 0x574, 0x57d, 0x56f, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x555, 0x574,
0x57d, 0x56f, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x56f, 0x56b,
0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54a, 0x561, 0x56f, 0x56b, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x57d,
0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x54a, 0x561, 0x56f, 0x56b, 0x57d, 0x57f, 0x561, 0x576, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x56c, 0x561, 0x578, 0x582, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x57a,
0x578, 0x582, 0x561, 0x20, 0x546, 0x578, 0x580, 0x20, 0x533, 0x57e, 0x56b, 0x576,
0x565, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54a, 0x561, 0x580, 0x561, 0x563, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x580, 0x561, 0x563, 0x57e, 0x561,
0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x561, 0x580, 0x561, 0x563,
0x57e, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x565, 0x580,
0x578, 0x582, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a,
0x565, 0x580, 0x578, 0x582, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561,
0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x565,
0x580, 0x578, 0x582, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x56b, 0x56c,
0x56b, 0x57a, 0x56b, 0x576, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x556, 0x56b, 0x56c, 0x56b, 0x57a, 0x56b, 0x576, 0x576,
0x565, 0x580, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x556, 0x56b, 0x56c, 0x56b,
0x57a, 0x56b, 0x576, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x556, 0x56b, 0x576, 0x56b, 0x584, 0x57d, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576,
0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d,
0x565, 0x576, 0x20, 0x54a, 0x56b, 0x565, 0x57c, 0x20, 0x587, 0x20, 0x544, 0x56b,
0x584, 0x565, 0x56c, 0x578, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x54d, 0x565, 0x576, 0x20, 0x54a, 0x56b, 0x565, 0x57c, 0x20, 0x587,
0x20, 0x544, 0x56b, 0x584, 0x565, 0x56c, 0x578, 0x576, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x54d, 0x565, 0x576, 0x20, 0x54a, 0x56b, 0x565, 0x57c, 0x20, 0x587,
0x20, 0x544, 0x56b, 0x584, 0x565, 0x56c, 0x578, 0x576, 0x56b, 0x20, 0x561, 0x574,
0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x54a, 0x56b, 0x57f, 0x56f, 0x565, 0x57c, 0x576, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54a, 0x578, 0x576, 0x561, 0x57a, 0x565,
0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x553, 0x56d, 0x565, 0x576, 0x575, 0x561, 0x576, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54c, 0x565, 0x575, 0x578, 0x582, 0x576,
0x56b, 0x578, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54c, 0x578, 0x57f, 0x565, 0x580, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54d, 0x561, 0x56d, 0x561, 0x56c, 0x56b, 0x576, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x561, 0x56d, 0x561,
0x56c, 0x56b, 0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x561, 0x56d,
0x561, 0x56c, 0x56b, 0x576, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575,
0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x561,
0x574, 0x578, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54d, 0x561, 0x574, 0x578, 0x561, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54d, 0x561, 0x574, 0x578, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54d, 0x565, 0x575, 0x577, 0x565, 0x56c, 0x575, 0x561, 0x576, 0x20, 0x56f,
0x572, 0x566, 0x56b, 0x576, 0x565, 0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x54d, 0x56b, 0x576, 0x563, 0x561, 0x57a, 0x578, 0x582, 0x580,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x578, 0x572,
0x578, 0x574, 0x578, 0x576, 0x56b, 0x20, 0x56f, 0x572, 0x566, 0x56b, 0x576, 0x565,
0x580, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x540, 0x561,
0x580, 0x561, 0x57e, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x54b, 0x578, 0x580, 0x57b,
0x56b, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54d, 0x578, 0x582, 0x580, 0x56b, 0x576, 0x561, 0x574, 0x56b, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54d, 0x575, 0x578, 0x57e, 0x561, 0x575, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x539, 0x561, 0x56b, 0x569,
0x56b, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x539, 0x561,
0x575, 0x57a, 0x565, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x539, 0x561, 0x575, 0x57a, 0x565, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x539, 0x561, 0x575, 0x57a, 0x565, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561,
0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54f, 0x561, 0x57b, 0x56b, 0x56f, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54f, 0x578, 0x56f, 0x565, 0x56c,
0x561, 0x578, 0x582, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x54f, 0x578, 0x576, 0x563, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54f, 0x578, 0x576, 0x563, 0x561, 0x575, 0x56b, 0x20,
0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54f, 0x578, 0x576, 0x563, 0x561, 0x575, 0x56b, 0x20,
0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x54f, 0x580, 0x578, 0x582, 0x56f, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x539, 0x578, 0x582, 0x580, 0x584, 0x574,
0x565, 0x576, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x539, 0x578, 0x582, 0x580, 0x584, 0x574, 0x565, 0x576, 0x57d,
0x57f, 0x561, 0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580,
0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x539, 0x578, 0x582,
0x580, 0x584, 0x574, 0x565, 0x576, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x54f, 0x578, 0x582, 0x57e, 0x561, 0x56c, 0x578, 0x582, 0x575,
0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x580,
0x578, 0x582, 0x563, 0x57e, 0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x548, 0x582, 0x580, 0x578, 0x582, 0x563, 0x57e, 0x561, 0x575,
0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x580, 0x578, 0x582, 0x563,
0x57e, 0x561, 0x575, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b,
0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x566,
0x562, 0x565, 0x56f, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x56a, 0x561, 0x574,
0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x566, 0x562, 0x565, 0x56f, 0x57d, 0x57f,
0x561, 0x576, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x566, 0x562,
0x565, 0x56f, 0x57d, 0x57f, 0x561, 0x576, 0x56b, 0x20, 0x561, 0x574, 0x561, 0x57c,
0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54e, 0x561, 0x576, 0x578, 0x582, 0x561, 0x57f, 0x578, 0x582, 0x575, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x561, 0x576, 0x578, 0x582,
0x561, 0x57f, 0x578, 0x582, 0x575, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564,
0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e,
0x561, 0x576, 0x578, 0x582, 0x561, 0x57f, 0x578, 0x582, 0x575, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x54e, 0x565, 0x576, 0x565, 0x57d, 0x578, 0x582, 0x565, 0x56c,
0x561, 0x575, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e,
0x56c, 0x561, 0x564, 0x56b, 0x57e, 0x578, 0x57d, 0x57f, 0x578, 0x56f, 0x56b, 0x20,
0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x56c, 0x561, 0x564, 0x56b,
0x57e, 0x578, 0x57d, 0x57f, 0x578, 0x56f, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576,
0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f,
0x54e, 0x56c, 0x561, 0x564, 0x56b, 0x57e, 0x578, 0x57d, 0x57f, 0x578, 0x56f, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x578, 0x56c, 0x563, 0x578, 0x563, 0x580,
0x561, 0x564, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e,
0x578, 0x56c, 0x563, 0x578, 0x563, 0x580, 0x561, 0x564, 0x56b, 0x20, 0x57d, 0x57f,
0x561, 0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576,
0x561, 0x56f, 0x54e, 0x578, 0x56c, 0x563, 0x578, 0x563, 0x580, 0x561, 0x564, 0x56b,
0x20, 0x561, 0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561,
0x574, 0x561, 0x576, 0x561, 0x56f, 0x54e, 0x578, 0x57d, 0x57f, 0x578, 0x56f, 0x56b,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x548, 0x582, 0x565, 0x575,
0x584, 0x20, 0x56f, 0x572, 0x566, 0x578, 0x582, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x548, 0x582, 0x578, 0x56c, 0x56b, 0x57d, 0x20, 0x587, 0x20,
0x556, 0x578, 0x582, 0x57f, 0x578, 0x582, 0x576, 0x561, 0x575, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x545, 0x561, 0x56f, 0x578, 0x582, 0x57f,
0x57d, 0x56f, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x545,
0x561, 0x56f, 0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x56b, 0x20, 0x57d, 0x57f, 0x561,
0x576, 0x564, 0x561, 0x580, 0x57f, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x545, 0x561, 0x56f, 0x578, 0x582, 0x57f, 0x57d, 0x56f, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x535, 0x56f, 0x561, 0x57f, 0x565, 0x580, 0x56b, 0x576, 0x562,
0x578, 0x582, 0x580, 0x563, 0x56b, 0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561,
0x56f, 0x535, 0x56f, 0x561, 0x57f, 0x565, 0x580, 0x56b, 0x576, 0x562, 0x578, 0x582,
0x580, 0x563, 0x56b, 0x20, 0x57d, 0x57f, 0x561, 0x576, 0x564, 0x561, 0x580, 0x57f,
0x20, 0x56a, 0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x535, 0x56f, 0x561, 0x57f,
0x565, 0x580, 0x56b, 0x576, 0x562, 0x578, 0x582, 0x580, 0x563, 0x56b, 0x20, 0x561,
0x574, 0x561, 0x57c, 0x561, 0x575, 0x56b, 0x576, 0x20, 0x56a, 0x561, 0x574, 0x561,
0x576, 0x561, 0x56f, 0x545, 0x578, 0x582, 0x56f, 0x578, 0x576, 0x56b, 0x20, 0x56a,
0x561, 0x574, 0x561, 0x576, 0x561, 0x56f, 0x986, 0x9ab, 0x997, 0x9be, 0x9a8, 0x9bf,
0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9a7, 0x9cd, 0x9af, 0x20, 0x986, 0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9ac, 0x20, 0x986, 0x9ab, 0x9cd,
0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a6, 0x995,
0x9cd, 0x9b7, 0x9bf, 0x9a3, 0x20, 0x986, 0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x995, 0x9be,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6,
0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x986, 0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x995, 0x9be,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae,
0x20, 0x986, 0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae,
0x20, 0x986, 0x9ab, 0x9cd, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9be,
0x9b8, 0x9cd, 0x995, 0x9be, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9ae, 0x9be, 0x99c, 0x9a8,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0,
0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7,
0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf,
0x995, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989,
0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be,
0x9f0, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc,
0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995,
0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7,
0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd,
0x9af, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4,
0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0,
0x20, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x9a1, 0x9c7, 0x9b2,
0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4,
0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa,
0x9f0, 0x9cd, 0x9ac, 0x9cd, 0x9ac, 0x9a4, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0,
0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9f0, 0x9cd, 0x9ac, 0x9cd, 0x9ac, 0x9a4,
0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995,
0x9be, 0x9f0, 0x20, 0x9aa, 0x9f0, 0x9cd, 0x9ac, 0x9cd, 0x9ac, 0x9a4, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf,
0x995, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986,
0x9ae, 0x9c7, 0x9f0, 0x9bf, 0x995, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9b6,
0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9f0, 0x20, 0x986, 0x9ae, 0x9c7, 0x9f0, 0x9bf,
0x995, 0x9be, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9aa, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x9f0, 0x9ac, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x986, 0x9f0, 0x9ac, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9ac, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9a1,
0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0,
0x9cd, 0x99c, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9cd, 0x99c, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf,
0x9a8, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x9f0, 0x9cd, 0x99c, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9f0,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be,
0x9a4, 0x9cd, 0x9af, 0x20, 0x986, 0x9f0, 0x9cd, 0x99c, 0x9c7, 0x9a3, 0x9cd, 0x99f,
0x9bf, 0x9a8, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6,
0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x986, 0x9f0, 0x9cd, 0x99c, 0x9c7,
0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd,
0x9af, 0x20, 0x986, 0x9f0, 0x9cd, 0x99c, 0x9c7, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x9a8,
0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9cd, 0x9ae,
0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x986, 0x9f0, 0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9f0, 0x9cd, 0x9ae,
0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x986, 0x99f, 0x9b2, 0x9be, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x99f, 0x9b2, 0x9be, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x995,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99f, 0x9b2,
0x9be, 0x9a3, 0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987,
0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x985,
0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7,
0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6,
0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7,
0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6,
0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be,
0x9a4, 0x9cd, 0x9af, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9f0, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9f0, 0x9c0, 0x9af, 0x9bc,
0x20, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x9a1,
0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd,
0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0,
0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0,
0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9be,
0x99a, 0x9cd, 0x9af, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af,
0x20, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd,
0x9f0, 0x9c7, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9be, 0x9b6,
0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x985, 0x9b7, 0x9cd, 0x99f, 0x9cd, 0x9f0, 0x9c7, 0x9b2, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4,
0x9cd, 0x9af, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x99c, 0x9c7, 0x9f0, 0x9ac, 0x9be, 0x987, 0x99c, 0x9be, 0x9a8,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9c7, 0x9f0, 0x9ac, 0x9be,
0x987, 0x99c, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x99c, 0x9c7, 0x9f0, 0x9ac, 0x9be, 0x987, 0x99c, 0x9be, 0x9a8,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9cb, 0x9f0, 0x9c7,
0x99b, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9cb, 0x9f0, 0x9c7, 0x99b,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c,
0x9cb, 0x9f0, 0x9c7, 0x99b, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac,
0x9be, 0x982, 0x9b2, 0x9be, 0x9a6, 0x9c7, 0x9b6, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ac, 0x9be, 0x982, 0x9b2, 0x9be, 0x9a6, 0x9c7, 0x9b6, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9be, 0x982, 0x9b2, 0x9be,
0x9a6, 0x9c7, 0x9b6, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9c1,
0x99f, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9b2, 0x9bf,
0x9ad, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac,
0x9cd, 0x9f0, 0x9be, 0x99c, 0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x99c, 0x9bf, 0x9b2, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ac, 0x9cd, 0x9f0, 0x9be, 0x99c, 0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9f0, 0x9c1, 0x9a8,
0x9c7, 0x987, 0x20, 0x9a1, 0x9be, 0x9f0, 0x9c1, 0x99b, 0x9be, 0x9b2, 0x9c7, 0x9ae,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be,
0x9f0, 0x9cd, 0x9a6, 0x9c7, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7,
0x9aa, 0x20, 0x9ad, 0x9be, 0x9f0, 0x9cd, 0x9a6, 0x9c7, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be,
0x9f0, 0x9cd, 0x9a6, 0x9c7, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a,
0x9be, 0x9ae, 0x9cb, 0x9f0, 0x9cb, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x99a, 0x9be, 0x9a5, 0x9be, 0x9ae, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99a, 0x9be, 0x9a5, 0x9be, 0x9ae, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9be, 0x9a5, 0x9be, 0x9ae, 0x9f0, 0x20,
0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a,
0x9bf, 0x9b2, 0x9bf, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2,
0x9bf, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a,
0x9bf, 0x9b2, 0x9bf, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9c0,
0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9c0, 0x9a8, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9c0, 0x9a8, 0x9f0,
0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9cb, 0x987, 0x9ac, 0x9be, 0x9b2, 0x9cd, 0x99b, 0x9a8, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x987, 0x9ac, 0x9be, 0x9b2, 0x9cd, 0x99b, 0x9a8,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb,
0x987, 0x9ac, 0x9be, 0x9b2, 0x9cd, 0x99b, 0x9a8, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x996, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x99f, 0x9ae, 0x9be, 0x99b,
0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9cb, 0x995, 0x9cb, 0x99b, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa,
0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9b2,
0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9b2, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9b2, 0x9ae, 0x9cd,
0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9c1, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e,
0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c1, 0x995, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c1, 0x995, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20,
0x985, 0x9f0, 0x9cd, 0x9a7, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf,
0x989, 0x9ac, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x989,
0x9ac, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9bf, 0x989, 0x9ac, 0x9be, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987,
0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a1, 0x9c7, 0x9ad, 0x9bf, 0x99b, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a1, 0x9c1, 0x9ae, 0x9cb, 0x9a3, 0x9cd, 0x99f,
0x2d, 0x9a1, 0x9bf, 0x20, 0x986, 0x9f0, 0x9cd, 0x9ad, 0x9bf, 0x9b2, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9ac, 0x20, 0x9a4, 0x9bf, 0x9ae, 0x9cb,
0x9f0, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9be,
0x9f0, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987,
0x9b7, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x995, 0x9c1, 0x9f1, 0x9c7, 0x9a1,
0x9f0, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20,
0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9a7, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd,
0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x987,
0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20,
0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x997, 0x9cd, 0x9f0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x985, 0x997, 0x9cd, 0x9f0, 0x2d, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a,
0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd,
0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4, 0x9cd, 0x9af,
0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be,
0x9a4, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9f0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x995, 0x9b2, 0x9c7, 0x9a3, 0x9cd,
0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x995, 0x9b2, 0x9c7, 0x9a3, 0x9cd,
0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x995,
0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa,
0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab,
0x9bf, 0x99c, 0x9bf, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x99c,
0x9bf, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab,
0x9bf, 0x99c, 0x9bf, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9cd,
0x9f0, 0x9be, 0x9a8, 0x9cd, 0x9b8, 0x20, 0x997, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3,
0x20, 0x9ab, 0x9cd, 0x9f0, 0x9be, 0x9a8, 0x9cd, 0x9b8, 0x20, 0x986, 0x9f0, 0x9c1,
0x20, 0x98f, 0x9a3, 0x9cd, 0x99f, 0x9be, 0x9f0, 0x9cd, 0x995, 0x99f, 0x9bf, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9be, 0x9b2, 0x9be, 0x9aa, 0x9be, 0x997,
0x9cb, 0x99b, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9c7, 0x9ae, 0x9cd,
0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c,
0x9f0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99c, 0x9f0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9f0, 0x9cd, 0x99c, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9bf,
0x9b2, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x99f, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa,
0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9a3, 0x989, 0x987, 0x99a, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9ac, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0,
0x9a3, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9c2, 0x9ac, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9a3, 0x9b2, 0x9c7, 0x9a3,
0x9cd, 0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9c2, 0x9ac, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9a3, 0x9b2, 0x9c7, 0x9a3,
0x9cd, 0x9a1, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd,
0x99a, 0x9bf, 0x9ae, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9a3, 0x9b2, 0x9c7, 0x9a3,
0x9cd, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a,
0x9bf, 0x9ae, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9a3, 0x9b2, 0x9c7, 0x9a3, 0x9cd,
0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9a3, 0x9b2,
0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989,
0x9aa, 0x9b8, 0x9be, 0x997, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9be, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9be, 0x9f1, 0x9be, 0x987, 0x2d,
0x98f, 0x9b2, 0x9bf, 0x989, 0x9b6, 0x9cd, 0x9ac, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b9, 0x9be, 0x9f1, 0x9be, 0x987, 0x2d, 0x98f, 0x9b2, 0x9bf, 0x989,
0x9b6, 0x9cd, 0x9ac, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b9, 0x9be, 0x9f1, 0x9be, 0x987, 0x2d, 0x98f, 0x9b2, 0x9bf, 0x989,
0x9b6, 0x9cd, 0x9ac, 0x9a8, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x982, 0x20, 0x995, 0x999, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x982, 0x20, 0x995, 0x999, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x982, 0x20, 0x995, 0x999,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9cb, 0x9ad, 0x9cd, 0x200c,
0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9cb, 0x9ad, 0x9cd, 0x200c,
0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9,
0x9cb, 0x9ad, 0x9cd, 0x200c, 0x9a1, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ad, 0x9be, 0x9f0, 0x9a4, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9f0, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be,
0x9b8, 0x9be, 0x997, 0x9f0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x987, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x99a, 0x9be, 0x987, 0x9a8, 0x9be, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x987, 0x9a3, 0x9cd,
0x9a1, 0x9cb, 0x9a8, 0x9c7, 0x99a, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9f0, 0x9be, 0x99a, 0x9cd, 0x9af, 0x20, 0x987,
0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x9a8, 0x9c7, 0x99a, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b6, 0x9cd, 0x99a, 0x9be, 0x9a4,
0x9cd, 0x9af, 0x20, 0x987, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x9a8, 0x9c7, 0x99a, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9f0, 0x9be,
0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9f0, 0x9be, 0x9a8, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9f0, 0x9be,
0x9a8, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x987, 0x9f0, 0x9cd, 0x995, 0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9f0, 0x9cd, 0x995, 0x9c1, 0x99f, 0x9b8,
0x9cd, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x987, 0x9f0, 0x9cd, 0x995, 0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x99c, 0x9f0, 0x9be, 0x987, 0x9b2, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x99c, 0x9f0, 0x9be, 0x987, 0x9b2, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x99c, 0x9f0, 0x9be,
0x987, 0x9b2, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x99c, 0x9be, 0x9aa, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99c, 0x9be, 0x9aa, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9be, 0x9aa, 0x9be, 0x9a8, 0x9f0, 0x20,
0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9c2, 0x9ac, 0x20, 0x995, 0x9be, 0x99c, 0x9be, 0x996, 0x9b8, 0x9cd, 0x9a4, 0x9be,
0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf,
0x9ae, 0x20, 0x995, 0x9be, 0x99c, 0x9be, 0x996, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9f0, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9f0, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9cb, 0x9f0, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2,
0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x99b, 0x9f0,
0x9be, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cd, 0x9f0,
0x9be, 0x99b, 0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cd, 0x9f0, 0x9be, 0x99b, 0x9a8,
0x9cb, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cd, 0x9f0, 0x9be, 0x99b,
0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x9f0, 0x9cd, 0x997, 0x9bf, 0x99c, 0x9b8,
0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9be,
0x987, 0x9a8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd,
0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9f0, 0x9cd, 0x9a1, 0x20,
0x9b9, 0x9be, 0x993, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9f0, 0x9cd,
0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9f0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9f0,
0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ae, 0x9be, 0x997, 0x9be, 0x9a6, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9be, 0x997, 0x9be, 0x9a6, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x997, 0x9be, 0x9a6, 0x9be,
0x9a8, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9b2, 0x9af,
0x9bc, 0x9c7, 0x99a, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9be, 0x9b2, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9f0, 0x9cd, 0x995, 0x99b, 0x9c7, 0x99b, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9f0, 0x9cd, 0x9b6, 0x9cd, 0x9ac,
0x9be, 0x9b2, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd,
0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9f0, 0x9bf, 0x99b, 0x9be,
0x99b, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9f0, 0x9bf, 0x99b, 0x9be,
0x99b, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9f0, 0x9bf, 0x99b, 0x9be, 0x99b, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ae, 0x9be, 0x989, 0x99b, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9cb, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0,
0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9c7,
0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9cb, 0x9f0, 0x20, 0x9aa, 0x9cd, 0x9f0, 0x9b6,
0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9cb, 0x9f0, 0x20, 0x9aa,
0x9cd, 0x9f0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be,
0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8, 0x9ac,
0x9be, 0x99f, 0x9be, 0x9f0, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2,
0x9be, 0x9a8, 0x9ac, 0x9be, 0x99f, 0x9be, 0x9f0, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8, 0x9ac, 0x9be, 0x99f,
0x9be, 0x9f0, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd,
0x995, 0x9cb, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd, 0x995,
0x9cb, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9b8, 0x9cd, 0x995, 0x9cb, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9cd, 0x9af, 0x9be, 0x9a8, 0x9ae, 0x9be, 0x9f0, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9a8, 0x9be, 0x989, 0x9f0, 0x9c1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a8, 0x9c7, 0x9aa, 0x9be, 0x9b2, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8,
0x9bf, 0x989, 0x20, 0x995, 0x9c7, 0x9b2, 0x9bf, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20,
0x995, 0x9c7, 0x9b2, 0x9bf, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989,
0x20, 0x995, 0x9c7, 0x9b2, 0x9bf, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x99c, 0x9bf,
0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8,
0x9bf, 0x989, 0x99c, 0x9bf, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x99c, 0x9bf,
0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987,
0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x9ab, 0x9be, 0x989,
0x9a3, 0x9cd, 0x9a1, 0x9b2, 0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x9ab, 0x9be, 0x989, 0x9a3, 0x9cd, 0x9a1, 0x9b2,
0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x9ab, 0x9be, 0x989, 0x9a3, 0x9cd, 0x9a1, 0x9b2,
0x9c7, 0x9a3, 0x9cd, 0x9a1, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9c1, 0x9f0, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x2019, 0x9f0, 0x9ab, 0x2019, 0x995, 0x20, 0x9a6,
0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x2019,
0x9f0, 0x9ab, 0x2019, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x9cd, 0x9af, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x2019,
0x9f0, 0x9ab, 0x2019, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9f0, 0x20,
0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9be, 0x9f0, 0x9cd, 0x9a8, 0x9be, 0x9a3,
0x9cd, 0x9a1, 0x9cb, 0x20, 0x9a1, 0x9c7, 0x20, 0x9a8, 0x9cb, 0x9f0, 0x9cb, 0x9a8,
0x9cd, 0x200c, 0x9b9, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9be,
0x9f0, 0x9cd, 0x9a8, 0x9be, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x20, 0x9a1, 0x9c7, 0x20,
0x9a8, 0x9cb, 0x9f0, 0x9cb, 0x9a8, 0x9cd, 0x200c, 0x9b9, 0x9be, 0x9f0, 0x20, 0x9ae,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9be, 0x9f0, 0x9cd, 0x9a8,
0x9be, 0x9a3, 0x9cd, 0x9a1, 0x9cb, 0x20, 0x9a1, 0x9c7, 0x20, 0x9a8, 0x9cb, 0x9f0,
0x9cb, 0x9a8, 0x9cd, 0x200c, 0x9b9, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9a8, 0x9ad, 0x9cb, 0x99b, 0x9bf, 0x9ac, 0x9bf, 0x9f0, 0x9cd, 0x9b8, 0x9cd,
0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9ad, 0x9cb, 0x99b, 0x9bf,
0x9ac, 0x9bf, 0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9ad, 0x9cb, 0x99b, 0x9bf, 0x9ac, 0x9bf,
0x9f0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993,
0x9ae, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4,
0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x997,
0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b2, 0x9be, 0x989, 0x9f0, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9aa, 0x9c1, 0x9f1, 0x9be, 0x20, 0x9a8, 0x9bf,
0x989, 0x20, 0x997, 0x9bf, 0x9a8, 0x9bf, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9be, 0x9f0, 0x9be, 0x997, 0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9be, 0x9f0, 0x9be, 0x997, 0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9f0, 0x9be,
0x997, 0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9c7, 0x9f0, 0x9c1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9f0,
0x9c1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9c7, 0x9f0, 0x9c1, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf,
0x9b2, 0x9bf, 0x9aa, 0x9be, 0x987, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ab, 0x9bf, 0x9b2, 0x9bf, 0x9aa, 0x9be, 0x987, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9b2, 0x9bf, 0x9aa, 0x9be,
0x987, 0x9a8, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9a8,
0x9bf, 0x995, 0x9cd, 0x9b8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1,
0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9c7, 0x987,
0x9a3, 0x9cd, 0x99f, 0x20, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9f0, 0x9c7, 0x20,
0x986, 0x9f0, 0x9c1, 0x20, 0x9ae, 0x9bf, 0x995, 0x9bf, 0x989, 0x9f1, 0x9c7, 0x9b2,
0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9c7, 0x987, 0x9a3, 0x9cd,
0x99f, 0x20, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9f0, 0x9c7, 0x20, 0x986, 0x9f0,
0x9c1, 0x20, 0x9ae, 0x9bf, 0x995, 0x9bf, 0x989, 0x9f1, 0x9c7, 0x9b2, 0x9a8, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9c7, 0x987,
0x9a3, 0x9cd, 0x99f, 0x20, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9f0, 0x9c7, 0x20,
0x986, 0x9f0, 0x9c1, 0x20, 0x9ae, 0x9bf, 0x995, 0x9bf, 0x989, 0x9f1, 0x9c7, 0x9b2,
0x9a8, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9bf, 0x99f, 0x995, 0x9c7, 0x987, 0x9f0, 0x9cd, 0x9a3, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cb, 0x9a8, 0x9be, 0x9aa, 0x9c7, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9af, 0x982, 0x9af, 0x9bc, 0x9be,
0x982, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9f0, 0x9bf, 0x987, 0x989, 0x9a8,
0x9bf, 0x9af, 0x9bc, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9f0, 0x9cb,
0x9a5, 0x9c7, 0x9f0, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9be,
0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b,
0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x9f0,
0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9be, 0x9ae, 0x9cb, 0x9f1, 0x9be,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9be, 0x9ae, 0x9cb, 0x9f1, 0x9be,
0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9be,
0x9ae, 0x9cb, 0x9f1, 0x9be, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9bf, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x99b,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9bf, 0x982, 0x997, 0x9be, 0x9aa,
0x9c1, 0x9f0, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x99a, 0x9cb, 0x9b2, 0x9cb, 0x9ae, 0x9cb, 0x9a8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3, 0x20, 0x99c, 0x9f0, 0x9cd, 0x99c, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99b, 0x9c1, 0x9f0,
0x9bf, 0x9a8, 0x9be, 0x9ae, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9cd,
0x9ac, 0x9cb, 0x9f1, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x9b9,
0x9bf, 0x9a4, 0x9bf, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9be, 0x987,
0x9aa, 0x9c7, 0x987, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9be, 0x987,
0x9aa, 0x9c7, 0x987, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99f, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987, 0x9f0, 0x20, 0x9a1, 0x9c7, 0x9b2,
0x9be, 0x987, 0x99f, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x99c, 0x9bf,
0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99f, 0x9cb, 0x995, 0x9c7, 0x9b2, 0x9be, 0x989, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99f, 0x982, 0x997, 0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x99f, 0x982, 0x997, 0x9be, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99f, 0x982, 0x997, 0x9be, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99a, 0x9cd, 0x99a, 0x9c1, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a4, 0x9c1, 0x9f0, 0x9cd, 0x995, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4,
0x9be, 0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9f0, 0x9cd,
0x995, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9f0, 0x9cd,
0x995, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20,
0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9c1, 0x9ad, 0x9be, 0x9b2, 0x9be, 0x989,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9f0, 0x9c1, 0x997, 0x9c1, 0x9f1,
0x9c7, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9f0, 0x9c1, 0x997, 0x9c1,
0x9f1, 0x9c7, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x989, 0x9f0, 0x9c1, 0x997, 0x9c1, 0x9f1, 0x9c7, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x989, 0x99c, 0x9ac, 0x9c7, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be,
0x9a8, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x99c, 0x9ac, 0x9c7, 0x995,
0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x99c, 0x9ac, 0x9c7, 0x995, 0x9bf, 0x9b8, 0x9cd,
0x9a4, 0x9be, 0x9a8, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be,
0x9a8, 0x9be, 0x99f, 0x9c1, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be,
0x9a8, 0x9be, 0x99f, 0x9c1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ad, 0x9be, 0x9a8, 0x9be, 0x99f, 0x9c1, 0x9f0, 0x20, 0x997, 0x9cd,
0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9c7, 0x9a8, 0x9bf, 0x99c, 0x9c1, 0x9f1, 0x9c7, 0x9b2,
0x9be, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a1,
0x9bf, 0x9ad, 0x9cb, 0x9b7, 0x9cd, 0x99f, 0x9cb, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a1, 0x9bf, 0x9ad, 0x9cb, 0x9b7, 0x9cd,
0x99f, 0x9cb, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a1, 0x9bf, 0x9ad, 0x9cb, 0x9b7, 0x9cd, 0x99f,
0x9cb, 0x995, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cb, 0x9b2,
0x9cd, 0x997, 0x9cb, 0x997, 0x9cd, 0x9f0, 0x9be, 0x9a1, 0x9f0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ad, 0x9cb, 0x9b2, 0x9cd, 0x997, 0x9cb, 0x997, 0x9cd, 0x9f0, 0x9be,
0x9a1, 0x9f0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad,
0x9cb, 0x9b2, 0x9cd, 0x997, 0x9cb, 0x997, 0x9cd, 0x9f0, 0x9be, 0x9a1, 0x9f0, 0x20,
0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cb, 0x9b7, 0x9cd, 0x99f, 0x995, 0x9f0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9f1, 0x9c7, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac,
0x9c0, 0x9aa, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9f1, 0x9be, 0x9b2, 0x9bf,
0x99b, 0x20, 0x986, 0x9f0, 0x9c1, 0x20, 0x9ab, 0x9c1, 0x99f, 0x9c1, 0x9a8, 0x9be,
0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x9a4,
0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc,
0x9be, 0x995, 0x9c1, 0x9a4, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x9a4,
0x9cd, 0x9b8, 0x9cd, 0x995, 0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af,
0x9bc, 0x9c7, 0x995, 0x9be, 0x99f, 0x9c7, 0x9f0, 0x9bf, 0x9a3, 0x9ac, 0x9be, 0x9f0,
0x9cd, 0x997, 0x9f0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9c7, 0x995,
0x9be, 0x99f, 0x9c7, 0x9f0, 0x9bf, 0x9a3, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997, 0x9f0,
0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9c7,
0x995, 0x9be, 0x99f, 0x9c7, 0x9f0, 0x9bf, 0x9a3, 0x9ac, 0x9be, 0x9f0, 0x9cd, 0x997,
0x9f0, 0x20, 0x997, 0x9cd, 0x9f0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9af, 0x9bc, 0x9c1, 0x995, 0x9cb,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x63, 0x72, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x63, 0x72,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0xc1, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x75, 0x64, 0xe1, 0x66, 0x72, 0x69,
0x63, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0xc1, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x2019, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x2019, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x6f, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72,
0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72,
0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72,
0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73,
0x74, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x6e, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x73, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x65,
0x73, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x73, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x65, 0x73, 0x20, 0x6e,
0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x6d,
0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x65, 0x73, 0x20, 0x6e, 0x6f, 0x72, 0x74,
0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x65, 0x73, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed,
0x66, 0x69, 0x63, 0x75, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x75, 0x20, 0x6e, 0x6f,
0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x75,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x75, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x6e, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74,
0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0x61, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x2019, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x72, 0x61, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x2019, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x2019, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41,
0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69,
0x63, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x6f, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x6f, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x6f, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x6f, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x78, 0xe1, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x78, 0xe1, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x78,
0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x65, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x78, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x78, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x78, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x75, 0x74, 0xe1, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65,
0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x75, 0x6e, 0xe9, 0x69, 0x20,
0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x75, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62,
0x75, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x61, 0x62, 0x75, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x43,
0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c,
0x65, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x6d, 0x65, 0x64, 0x69, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x73, 0x63, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72,
0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x63, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45,
0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45,
0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74,
0x72, 0x65, 0x6d, 0x75, 0x20, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62,
0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x46, 0x69, 0x78, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46,
0x69, 0x78, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x78, 0x69,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x61, 0x20, 0x47,
0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65,
0x73, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x73,
0x75, 0x72, 0x20, 0x79, 0x20, 0x6c, 0x2019, 0x61, 0x6e, 0x74, 0xe1, 0x72,
0x74, 0x69, 0x63, 0x75, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x63, 0xe9, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6c, 0xe1,
0x70, 0x61, 0x67, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c,
0x65, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x6d, 0x65, 0x64, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6d,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x1e24, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x1e24, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x1e24, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x63, 0xe9,
0x61, 0x6e, 0x75, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65,
0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x6f, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x49, 0x72, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x49, 0x72, 0xe1, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x2019, 0x49, 0x72, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x2019, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x2019, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x58, 0x61, 0x70, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x58,
0x61, 0x70, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x58, 0x61, 0x70,
0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74,
0xe1, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x6f, 0x72, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f,
0x72, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x6f, 0x73,
0x72, 0x61, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73, 0x74, 0xe1,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x61, 0x6e,
0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4c, 0x69, 0x6e,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0xe1, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0xe1, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x63, 0xe1, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x73, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x65, 0x73,
0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69,
0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x69, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x63, 0x69, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x77, 0x73, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x75, 0x20, 0x64,
0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x75, 0x20, 0x64,
0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x75, 0x20, 0x64,
0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0xe1, 0x74, 0x6f,
0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x2019, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0xe1,
0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x2019, 0x55, 0x6c, 0xe1, 0x6e, 0x20,
0x42, 0xe1, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x79,
0x61, 0x6e, 0x6d, 0x61, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76,
0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x69,
0x75, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73,
0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49,
0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e,
0x65, 0x73, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x4f, 0x6d,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73,
0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e,
0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x71,
0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0xfa, 0x61, 0x20, 0x4e, 0x75, 0x65,
0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0xe1, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0xe1, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0xe1, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x65, 0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72,
0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x65, 0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x75, 0x20,
0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e,
0x69, 0x78, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x79,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67,
0x79, 0x61, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x52, 0x65, 0x75,
0x6e, 0x69, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xed, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xed, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x78, 0x61, 0x6c, 0xed, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4c, 0x65, 0x73, 0x20, 0x53, 0x65, 0x69, 0x78, 0x65,
0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x75, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x53,
0x61, 0x6c, 0x6f, 0x6d, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x53, 0x75, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0xe9,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x61, 0x78, 0x69,
0x71, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0xe1, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0xe1, 0x69,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0xe1,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x55, 0x7a,
0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x71, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75,
0x65, 0x6c, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1,
0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65,
0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0xe1, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x6c, 0x61, 0x20, 0x57, 0x61,
0x6b, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x79, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x62, 0x72, 0x61,
0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x62, 0x72, 0x61, 0x6e, 0x69, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62,
0x75, 0x72, 0x67, 0x75, 0x18f, 0x66, 0x71, 0x61, 0x6e, 0x131, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b,
0x259, 0x7a, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x43, 0x259,
0x6e, 0x75, 0x62, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51,
0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x41, 0x6c, 0x79, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x41, 0x6c, 0x79, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41,
0x6c, 0x79, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20,
0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d,
0x61, 0x6c, 0x69, 0x20, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69,
0x6d, 0x61, 0x6c, 0x69, 0x20, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69,
0x20, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61,
0x6c, 0x69, 0x20, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c,
0x69, 0x20, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x44, 0x61, 0x11f, 0x6c,
0x131, 0x71, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x44,
0x61, 0x11f, 0x6c, 0x131, 0x71, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x44,
0x61, 0x11f, 0x6c, 0x131, 0x71, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e,
0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x53, 0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65, 0x61,
0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x69, 0x6d, 0x61, 0x6c,
0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x61,
0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x15e, 0x69, 0x6d, 0x61, 0x6c, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65,
0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x41, 0x70, 0x69, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x70,
0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x70, 0x69, 0x61, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x18f, 0x72, 0x259, 0x62, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x18f, 0x72,
0x259, 0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x18f, 0x72,
0x259, 0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x45, 0x72, 0x6d, 0x259, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x45,
0x72, 0x6d, 0x259, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x45, 0x72, 0x6d, 0x259, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x76, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x76, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259,
0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x51, 0x259,
0x72, 0x62, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b,
0x259, 0x7a, 0x69, 0x20, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x76,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d,
0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x51, 0x259, 0x72, 0x62, 0x69,
0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72,
0x71, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69,
0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x76, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x41, 0x7a, 0x259, 0x72, 0x62, 0x61, 0x79, 0x63,
0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x7a, 0x259, 0x72,
0x62, 0x61, 0x79, 0x63, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x7a, 0x259,
0x72, 0x62, 0x61, 0x79, 0x63, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x41, 0x7a,
0x6f, 0x72, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x42, 0x61, 0x6e, 0x71, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x42, 0x61, 0x6e, 0x71, 0x6c, 0x61, 0x64, 0x65, 0x15f,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x42, 0x61, 0x6e, 0x71, 0x6c, 0x61, 0x64, 0x65, 0x15f,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x75,
0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x6f, 0x6c,
0x69, 0x76, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42,
0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c,
0x61, 0x6d, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x61, 0x70, 0x65,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x4b, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4b, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20,
0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7,
0x69, 0x6c, 0x69, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x69, 0x6c,
0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0xc7, 0x69, 0x6c, 0x69, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x69, 0x6e, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0xc7, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x69, 0x6e,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x6f,
0x79, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0xc7, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0xc7, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x69, 0x6c, 0x61,
0x64, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61,
0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x75, 0x6b, 0x20, 0x41, 0x64,
0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b,
0x75, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4b, 0x75, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x20, 0x59, 0x61, 0x72, 0x131, 0x6d, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x75, 0x62,
0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x44,
0x65, 0x76, 0x69, 0x73, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x44, 0xfc,
0x6d, 0x6f, 0x6e, 0x2d, 0x64, 0x2019, 0xdc, 0x72, 0x76, 0x69, 0x6c, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x54,
0x69, 0x6d, 0x6f, 0x72, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61,
0x73, 0x78, 0x61, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20, 0x41, 0x64, 0x61,
0x73, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20, 0x41,
0x64, 0x61, 0x73, 0x131, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41,
0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d,
0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4d, 0x259, 0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20,
0x41, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x76,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259,
0x72, 0x71, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b,
0x259, 0x6e, 0x61, 0x72, 0x20, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x41,
0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51,
0x259, 0x72, 0x62, 0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x41,
0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62,
0x69, 0x20, 0x41, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65,
0x6e, 0x64, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64,
0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46,
0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x41, 0x64, 0x61, 0x6c,
0x61, 0x72, 0x131, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x46, 0x69, 0x63, 0x69, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46,
0x69, 0x63, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x69, 0x63, 0x69, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x72, 0x61, 0x6e,
0x73, 0x131, 0x7a, 0x20, 0x51, 0x76, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x131,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x131,
0x7a, 0x20, 0x43, 0x259, 0x6e, 0x75, 0x62, 0x69, 0x20, 0x76, 0x259, 0x20,
0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x51, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x71, 0x6f, 0x73,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x47, 0x75, 0x72, 0x63, 0xfc,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x47, 0x75,
0x72, 0x63, 0xfc, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x47, 0x75,
0x72, 0x63, 0xfc, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74,
0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x51, 0x72, 0x69, 0x6e, 0x76, 0x69, 0xe7, 0x20, 0x4f, 0x72,
0x74, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71,
0x69, 0x20, 0x51, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79,
0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69,
0x20, 0x51, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x51, 0x72, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20,
0x51, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x51,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x51, 0x259, 0x72, 0x62, 0x69, 0x20, 0x51, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4b, 0xf6, 0x72, 0x66, 0x259, 0x7a, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x51, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x48, 0x61, 0x76, 0x61, 0x79, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x61, 0x76,
0x61, 0x79, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48,
0x61, 0x76, 0x61, 0x79, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x6f, 0x6e, 0x71,
0x20, 0x4b, 0x6f, 0x6e, 0x71, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48,
0x6f, 0x6e, 0x71, 0x20, 0x4b, 0x6f, 0x6e, 0x71, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48,
0x6f, 0x6e, 0x71, 0x20, 0x4b, 0x6f, 0x6e, 0x71, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48,
0x6f, 0x76, 0x64, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x48, 0x69, 0x6e, 0x64, 0x20, 0x4f, 0x6b, 0x65,
0x61, 0x6e, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x48, 0x69, 0x6e,
0x64, 0xe7, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x259,
0x72, 0x6b, 0x259, 0x7a, 0x69, 0x20, 0x130, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x7a, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x15e, 0x259,
0x72, 0x71, 0x69, 0x20, 0x130, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51, 0x259, 0x72, 0x62,
0x69, 0x20, 0x130, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x130, 0x72, 0x61, 0x6e, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x130, 0x72, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x130,
0x72, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x61, 0x70, 0x6f, 0x6e, 0x69,
0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x61, 0x70, 0x6f,
0x6e, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x61, 0x70, 0x6f, 0x6e,
0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x15e, 0x259, 0x72, 0x71, 0x69, 0x20, 0x51, 0x61, 0x7a, 0x61, 0x78,
0x131, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x51,
0x259, 0x72, 0x62, 0x69, 0x20, 0x51, 0x61, 0x7a, 0x61, 0x78, 0x131, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f, 0x72,
0x65, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f, 0x72,
0x65, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f, 0x72, 0x65, 0x79, 0x61,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x6f,
0x72, 0x73, 0x65, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x51, 0x131, 0x72, 0x11f, 0x131, 0x7a, 0x131, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4c, 0x61, 0x79, 0x6e, 0x20, 0x41,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x61, 0x75, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x61, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x61, 0x75, 0x20, 0x59, 0x61,
0x79, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x61, 0x71, 0x61, 0x64,
0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x61, 0x71, 0x61,
0x64, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x61, 0x71, 0x61, 0x64, 0x61,
0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d,
0x61, 0x6c, 0x61, 0x79, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4d, 0x61, 0x72, 0x6b, 0x65, 0x73, 0x61, 0x73, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4d, 0x61, 0x72, 0x15f, 0x61, 0x6c, 0x20, 0x41,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x4d, 0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d,
0x61, 0x76, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4d, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20,
0x53, 0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65, 0x61, 0x6e, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x20, 0x53, 0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65, 0x61, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x53,
0x61, 0x6b, 0x69, 0x74, 0x20, 0x4f, 0x6b, 0x65, 0x61, 0x6e, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x55, 0x6c, 0x61, 0x6e,
0x62, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x55,
0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x55,
0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x59, 0x61,
0x79, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x4d, 0x79, 0x61, 0x6e, 0x6d,
0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x20,
0x76, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6e, 0x69,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20,
0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6e,
0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65,
0x6e, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x79,
0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x4e, 0x79, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64,
0x6c, 0x65, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x79, 0x75, 0x66, 0x61,
0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x41,
0x64, 0x61, 0x73, 0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x41, 0x64,
0x61, 0x73, 0x131, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20,
0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4f, 0x6d, 0x73,
0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4f, 0x6d, 0x73, 0x6b, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50,
0x61, 0x6c, 0x61, 0x75, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61,
0x70, 0x75, 0x61, 0x20, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x51, 0x76, 0x69,
0x6e, 0x65, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x61,
0x72, 0x61, 0x71, 0x76, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x50, 0x61, 0x72, 0x61, 0x71, 0x76, 0x61, 0x79, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50,
0x61, 0x72, 0x61, 0x71, 0x76, 0x61, 0x79, 0x20, 0x59, 0x61, 0x79, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x65, 0x72, 0x75, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x50, 0x65, 0x72, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x65,
0x72, 0x75, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x20, 0x59, 0x61,
0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x46, 0x65, 0x6e, 0x69, 0x6b,
0x73, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x4d, 0xfc, 0x71, 0x259, 0x64, 0x64, 0x259, 0x73, 0x20,
0x50, 0x79, 0x65, 0x72, 0x20, 0x76, 0x259, 0x20, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x4d, 0xfc, 0x71,
0x259, 0x64, 0x64, 0x259, 0x73, 0x20, 0x50, 0x79, 0x65, 0x72, 0x20, 0x76,
0x259, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x4d, 0xfc, 0x71, 0x259, 0x64, 0x64, 0x259, 0x73, 0x20, 0x50, 0x79, 0x65,
0x72, 0x20, 0x76, 0x259, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e,
0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50, 0x69,
0x74, 0x6b, 0x65, 0x72, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50,
0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x50,
0x78, 0x65, 0x6e, 0x79, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x52, 0x65, 0x79, 0x75, 0x6e, 0x79, 0x6f, 0x6e, 0x52, 0x6f, 0x74, 0x65,
0x72, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x61, 0x78, 0x61,
0x6c, 0x69, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x61, 0x78,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x61, 0x78, 0x61, 0x6c,
0x69, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x78, 0x74, 0x131, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x20, 0x79, 0x61, 0x79, 0x20, 0x76, 0x61, 0x78, 0x74,
0x131, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53,
0x65, 0x79, 0x15f, 0x65, 0x6c, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72,
0x131, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x69, 0x6e, 0x71, 0x61,
0x70, 0x75, 0x72, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x53, 0x6f, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x43, 0x259, 0x6e, 0x75, 0x62, 0x69,
0x20, 0x43, 0x6f, 0x72, 0x63, 0x69, 0x79, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x53, 0x79, 0x6f, 0x76, 0x61, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x54, 0x61, 0x79, 0x62, 0x65, 0x79, 0x20, 0x56, 0x61, 0x78,
0x74, 0x131, 0x54, 0x61, 0x79, 0x62, 0x65, 0x79, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54,
0x61, 0x79, 0x62, 0x65, 0x79, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x54, 0x61, 0x63, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0x6f, 0x6e, 0x71,
0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0x6f, 0x6e, 0x71, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x54, 0x6f, 0x6e, 0x71, 0x61, 0x20, 0x59, 0x61, 0x79,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xc7, 0x75, 0x75, 0x6b, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x259, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54, 0xfc,
0x72, 0x6b, 0x6d, 0x259, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x259, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x55,
0x72, 0x75, 0x71, 0x76, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x55, 0x72, 0x75, 0x71, 0x76, 0x61, 0x79, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x55, 0x72,
0x75, 0x71, 0x76, 0x61, 0x79, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0xd6, 0x7a, 0x62, 0x259, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xd6, 0x7a, 0x62, 0x259, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0xd6, 0x7a, 0x62, 0x259,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20,
0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x56, 0x61, 0x75, 0x6e, 0x61, 0x74, 0x75, 0x20, 0x59,
0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x65, 0x6e, 0x65,
0x73, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x56,
0x61, 0x78, 0x74, 0x131, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61,
0x78, 0x74, 0x131, 0x56, 0x6f, 0x6c, 0x71, 0x6f, 0x71, 0x72, 0x61, 0x64,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x6f, 0x6c, 0x71, 0x6f, 0x71,
0x72, 0x61, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x56, 0x6f, 0x6c, 0x71, 0x6f, 0x71,
0x72, 0x61, 0x64, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x55, 0x65, 0x79, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x55,
0x6f, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x76, 0x259, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74,
0x131, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x71, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x71, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x71, 0x20, 0x59, 0x61, 0x79, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x59,
0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x56, 0x61, 0x78, 0x74, 0x131, 0x98f, 0x995,
0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f, 0x995, 0x9b0, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f, 0x995, 0x9b0, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x9ab, 0x997, 0x9be, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x986,
0x9ab, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x986, 0x9ab, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9be,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a6, 0x995, 0x9cd, 0x9b7, 0x9bf, 0x9a3, 0x20,
0x986, 0x9ab, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20,
0x986, 0x9ab, 0x9cd, 0x9b0, 0x9bf, 0x995, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x986, 0x9ab, 0x9cd, 0x9b0, 0x9bf,
0x995, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x986, 0x9ab, 0x9cd, 0x9b0, 0x9bf,
0x995, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9be, 0x9b8,
0x9cd, 0x995, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9be, 0x9b8,
0x9cd, 0x995, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x986, 0x9b2, 0x9be, 0x9b8, 0x9cd, 0x995, 0x9be, 0x20, 0x9a6, 0x9bf, 0x9ac,
0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9cd,
0x9ae, 0x9be, 0x99f, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b2, 0x9cd,
0x9ae, 0x9be, 0x99f, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x9b2, 0x9cd, 0x9ae, 0x9be, 0x99f, 0x9bf, 0x20, 0x997, 0x9cd,
0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x9ae, 0x9be, 0x99c, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x9ae, 0x9be, 0x99c, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9a6,
0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x9be, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c0, 0x9af, 0x9bc,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x9be, 0x99e,
0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x9be, 0x99e, 0x9cd, 0x99a,
0x9b2, 0x9c7, 0x9b0, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b0, 0x9cd, 0x9ac, 0x9a4, 0x9cd, 0x9af,
0x20, 0x985, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9be, 0x9b0, 0x9cd, 0x9ac, 0x9a4, 0x9cd, 0x9af, 0x20, 0x985, 0x99e,
0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b0, 0x9cd, 0x9ac, 0x9a4, 0x9cd, 0x9af, 0x20,
0x985, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9a6, 0x9bf, 0x9a8, 0x9c7,
0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9b0, 0x9b6, 0x9be, 0x9a8,
0x9cd, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0, 0x9af,
0x9bc, 0x20, 0x985, 0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9b0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae,
0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x985, 0x99e,
0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9b0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20,
0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x985,
0x99e, 0x9cd, 0x99a, 0x9b2, 0x9c7, 0x9b0, 0x20, 0x9a6, 0x9bf, 0x9a8, 0x9c7, 0x9b0,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9a8, 0x9a6, 0x9cd, 0x9af, 0x9cd, 0x9b0,
0x9cd, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9a8, 0x9a6, 0x9cd, 0x9af, 0x9cd,
0x9b0, 0x9cd, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x985, 0x9a8, 0x9a6, 0x9cd, 0x9af, 0x9cd, 0x9b0, 0x9cd, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x985, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x985, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20,
0x9a6, 0x9bf, 0x9a8, 0x9c7, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x995,
0x99f, 0x9be, 0x993, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x995, 0x99f, 0x9be,
0x993, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x995, 0x99f, 0x9be, 0x993, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x995, 0x99f, 0x9cb,
0x9ac, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x995, 0x99f, 0x9cb, 0x9ac, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x995, 0x99f,
0x9cb, 0x9ac, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9ac, 0x9bf, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9ac, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0, 0x9ac, 0x9bf, 0x20, 0x9a6, 0x9bf,
0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0,
0x9cd, 0x99c, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x9a8,
0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20,
0x986, 0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20,
0x986, 0x9b0, 0x9cd, 0x99c, 0x9c7, 0x9a8, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x9b0, 0x20,
0x9aa, 0x9cd, 0x9b0, 0x9ae, 0x9be, 0x9a3, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x986, 0x9b0, 0x9cd, 0x99c, 0x9c7,
0x9a8, 0x99f, 0x9bf, 0x9a8, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986,
0x9b0, 0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x9b0, 0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x9b0,
0x9cd, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x985, 0x9a4, 0x9b2, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x9bf, 0x995, 0x9c7,
0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9a4, 0x9b2, 0x9be, 0x9a8, 0x9cd,
0x9a4, 0x9bf, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x985, 0x9a4, 0x9b2, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x9bf, 0x995, 0x20, 0x9a6,
0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x985, 0x9b8,
0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af,
0x9bc, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8,
0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x995, 0x9c7,
0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9a6, 0x9bf, 0x9ac,
0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd,
0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x995, 0x9c7, 0x9a8,
0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9b6, 0x9cd, 0x99a,
0x9bf, 0x9ae, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f,
0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x995, 0x9c7, 0x9a8, 0x9cd,
0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf,
0x9ae, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20,
0x995, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa,
0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2,
0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac,
0x20, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7,
0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f,
0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9c2, 0x9b0, 0x9cd,
0x9ac, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x985, 0x9b8,
0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9bf, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x985, 0x9b8, 0x9cd, 0x99f, 0x9cd,
0x9b0, 0x9c7, 0x9b2, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf,
0x9ae, 0x9bf, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9be, 0x9b0, 0x9ac, 0x9be, 0x987, 0x99c, 0x9be,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x986, 0x99c, 0x9be, 0x9b0, 0x9ac, 0x9be,
0x987, 0x99c, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x986, 0x99c, 0x9be, 0x9b0, 0x9ac, 0x9be, 0x987, 0x99c, 0x9be, 0x9a8,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f, 0x99c, 0x9cb, 0x9b0, 0x9c7, 0x9b8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f, 0x99c, 0x9cb, 0x9b0, 0x9c7, 0x9b8, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x98f, 0x99c, 0x9cb,
0x9b0, 0x9c7, 0x9b8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9be, 0x982,
0x9b2, 0x9be, 0x9a6, 0x9c7, 0x9b6, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9be,
0x982, 0x9b2, 0x9be, 0x9a6, 0x9c7, 0x9b6, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9be, 0x982, 0x9b2, 0x9be, 0x9a6, 0x9c7, 0x9b6,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9c1, 0x99f, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cb, 0x9b2, 0x9bf, 0x9ad, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9bf,
0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd,
0x9b0, 0x9be, 0x9b8, 0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ac, 0x9cd, 0x9b0, 0x9be, 0x9b8,
0x9bf, 0x9b2, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ac, 0x9cd, 0x9b0, 0x9c1, 0x9a8, 0x9c7, 0x987, 0x20, 0x9a6, 0x9be, 0x9b0, 0x9c1,
0x9b8, 0x9b8, 0x9be, 0x9b2, 0x9be, 0x9ae, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9c7, 0x9aa, 0x20, 0x9ad, 0x9be, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9aa, 0x20,
0x9ad, 0x9be, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a,
0x9be, 0x9ae, 0x9c7, 0x9b0, 0x9cb, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x99a, 0x9cd, 0x9af, 0x9be, 0x9a5, 0x9be, 0x9ae, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x99a, 0x9cd, 0x9af, 0x9be, 0x9a5, 0x9be, 0x9ae, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9cd, 0x9af, 0x9be,
0x9a5, 0x9be, 0x9ae, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x99a, 0x9bf, 0x9b2, 0x9bf, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x99a, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9c0, 0x9a8,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9c0,
0x9a8, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99a, 0x9af, 0x9bc, 0x9ac, 0x9be, 0x9b2, 0x9b8, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x99a, 0x9af, 0x9bc, 0x9ac, 0x9be, 0x9b2, 0x9b8, 0x9a8, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99a, 0x9af, 0x9bc,
0x9ac, 0x9be, 0x9b2, 0x9b8, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995,
0x9cd, 0x9b0, 0x9bf, 0x9b8, 0x9ae, 0x9be, 0x9b8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x995, 0x9cb, 0x9b8, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9b2, 0x9cb, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9b2, 0x9cb, 0x9ae,
0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9b2, 0x9cb, 0x9ae, 0x9cd, 0x9ac, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c1, 0x995,
0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c1, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c1, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x985, 0x9b0, 0x9cd, 0x9a7, 0x9c7,
0x995, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x989, 0x9ac, 0x9be,
0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x989, 0x9ac, 0x9be, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x989,
0x9ac, 0x9be, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9a1, 0x9c7, 0x9ad, 0x9bf, 0x9b8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9a1, 0x9c1, 0x9ae, 0x9a8, 0x9cd, 0x99f, 0x2d, 0x9a6, 0x9cd, 0x9af, 0x2019,
0x989, 0x9b0, 0x9ad, 0x9bf, 0x9b2, 0x9c7, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x99f, 0x9bf, 0x9ae, 0x9b0, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x987, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x20, 0x9a6, 0x9cd, 0x9ac,
0x9c0, 0x9aa, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b8, 0x9cd, 0x99f, 0x9be,
0x9b0, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b8, 0x9cd, 0x99f, 0x9be, 0x9b0, 0x20,
0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987,
0x995, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x9a1, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x987,
0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9a7, 0x9cd, 0x9af, 0x20, 0x987, 0x989,
0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac,
0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac,
0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x997, 0x9cd,
0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9b0, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x9c0, 0x9af,
0x9bc, 0x20, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x987, 0x989, 0x9b0, 0x9cb,
0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd,
0x99a, 0x9bf, 0x9ae, 0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20,
0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae,
0x20, 0x987, 0x989, 0x9b0, 0x9cb, 0x9aa, 0x9c0, 0x9af, 0x9bc, 0x20, 0x997, 0x9cd,
0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ab, 0x995, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1,
0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x995, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd,
0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x995,
0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ab, 0x9bf, 0x99c, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x99c,
0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab,
0x9bf, 0x99c, 0x9bf, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995,
0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9b0, 0x9be,
0x9b8, 0x9bf, 0x20, 0x997, 0x9be, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9b0, 0x9be, 0x9b8, 0x9bf, 0x20, 0x9a6, 0x995, 0x9cd,
0x9b7, 0x9bf, 0x9a3, 0x20, 0x98f, 0x9ac, 0x982, 0x20, 0x986, 0x9a8, 0x9cd, 0x99f,
0x9be, 0x9b0, 0x9cd, 0x995, 0x99f, 0x9bf, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x997, 0x9be, 0x9b2, 0x9be, 0x9aa, 0x9be, 0x997, 0x9cb, 0x9b8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x997, 0x9cd, 0x9af, 0x9be, 0x9ae, 0x9cd, 0x9ac, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9b0, 0x9cd, 0x99c, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9b0, 0x9cd, 0x99c,
0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x99c, 0x9b0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9bf, 0x9b2, 0x9ac, 0x9be, 0x9b0, 0x9cd, 0x99f,
0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9bf, 0x99a, 0x20,
0x9ae, 0x9bf, 0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9aa, 0x9c2, 0x9b0, 0x9cd,
0x9ac, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8,
0x9cd, 0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd,
0x9a1, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9b2, 0x9cd,
0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9b2,
0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa,
0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9b2,
0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x997,
0x9cd, 0x9b0, 0x9c0, 0x9a8, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9ae, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9aa, 0x9b8, 0x9be,
0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x9be, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9be, 0x993, 0x9af, 0x9bc, 0x9be, 0x987, 0x2d, 0x986,
0x9b2, 0x9c7, 0x989, 0x9a4, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9be, 0x993,
0x9af, 0x9bc, 0x9be, 0x987, 0x2d, 0x986, 0x9b2, 0x9c7, 0x989, 0x9a4, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9be, 0x993, 0x9af,
0x9bc, 0x9be, 0x987, 0x2d, 0x986, 0x9b2, 0x9c7, 0x989, 0x9a4, 0x20, 0x9a6, 0x9bf,
0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x982,
0x20, 0x995, 0x982, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x982, 0x20, 0x995,
0x982, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9,
0x982, 0x20, 0x995, 0x982, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9cb,
0x9ad, 0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9cb, 0x9ad, 0x9a1, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b9, 0x9cb, 0x9ad,
0x9a1, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9b0, 0x9a4, 0x9c0,
0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ad, 0x9be, 0x9b0, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8, 0x9be, 0x997, 0x9b0,
0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9a8, 0x9cd, 0x9a6,
0x9cb, 0x99a, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9c7, 0x9a8,
0x9cd, 0x9a6, 0x9cd, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a6,
0x9cb, 0x9a8, 0x9c7, 0x9b6, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a6, 0x9cb,
0x9a8, 0x9c7, 0x9b6, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x9c0, 0x20, 0x987, 0x9a8, 0x9cd, 0x9a6,
0x9cb, 0x9a8, 0x9c7, 0x9b6, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x987, 0x9b0, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b0,
0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x987, 0x9b0, 0x9be, 0x9a8, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b0, 0x995, 0x9c1, 0x99f, 0x9b8, 0x9cd,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9b0, 0x995, 0x9c1, 0x99f, 0x9b8,
0x9cd, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x987, 0x9b0, 0x995, 0x9c1, 0x99f, 0x9b8, 0x9cd, 0x995, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x987, 0x99c, 0x9b0, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9b2, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x987, 0x99c, 0x9b0, 0x9be, 0x9af, 0x9bc, 0x9c7, 0x9b2, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x99c, 0x9b0,
0x9be, 0x9af, 0x9bc, 0x9c7, 0x9b2, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9be, 0x9aa, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9be, 0x9aa, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be,
0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99c, 0x9be, 0x9aa, 0x9be, 0x9a8,
0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9bf, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x9ad,
0x9b2, 0x9b8, 0x9cd, 0x995, 0x2d, 0x20, 0x995, 0x9cd, 0x9af, 0x9be, 0x9ae, 0x99a,
0x9cd, 0x9af, 0x9be, 0x99f, 0x9b8, 0x9cd, 0x995, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9bf, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x9ad,
0x9b2, 0x9b8, 0x9cd, 0x995, 0x2d, 0x20, 0x995, 0x9cd, 0x9af, 0x9be, 0x9ae, 0x99a,
0x9cd, 0x9af, 0x9be, 0x99f, 0x9b8, 0x9cd, 0x995, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9bf, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9aa,
0x9cd, 0x9af, 0x9be, 0x9ad, 0x9b2, 0x9b8, 0x9cd, 0x995, 0x2d, 0x20, 0x995, 0x9cd,
0x9af, 0x9be, 0x9ae, 0x99a, 0x9cd, 0x9af, 0x9be, 0x99f, 0x9b8, 0x9cd, 0x995, 0x9bf,
0x20, 0x997, 0x9c3, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c2, 0x9b0, 0x9cd, 0x9ac, 0x20, 0x995, 0x9be,
0x99c, 0x9be, 0x996, 0x9be, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9b6, 0x9cd, 0x99a, 0x9bf, 0x9ae, 0x20, 0x995, 0x9be, 0x99c,
0x9be, 0x996, 0x9be, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9cb, 0x9b0, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x995, 0x9cb, 0x9b0, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9b0, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cb, 0x9b8, 0x9b0, 0x9c7, 0x987, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9a8, 0x9cb, 0x9af,
0x9bc, 0x9be, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9a8, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x9b0,
0x9cd, 0x9b8, 0x9cd, 0x995, 0x9bf, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9cd, 0x9b0, 0x9be, 0x9b8, 0x9a8, 0x9cb, 0x9af, 0x9bc,
0x9be, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x9bf, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x995, 0x9bf, 0x9b0, 0x997, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x999, 0x9cd, 0x995, 0x9be, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b2, 0x9be, 0x987, 0x9a8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa,
0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b2, 0x9b0,
0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b2, 0x9b0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9af, 0x9bc,
0x9c7, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9ae, 0x9b8, 0x9af, 0x9bc, 0x9b2,
0x9b0, 0x9cd, 0x9a1, 0x20, 0x9b9, 0x9be, 0x993, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9a6,
0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9ae, 0x9b8, 0x9af, 0x9bc, 0x9ae,
0x9be, 0x995, 0x9be, 0x993, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x995,
0x9be, 0x993, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9be, 0x995, 0x9be, 0x993, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9cd, 0x9af, 0x9be,
0x997, 0x9be, 0x9a1, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9cd,
0x9af, 0x9be, 0x997, 0x9be, 0x9a1, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9cd, 0x9af, 0x9be, 0x997, 0x9be, 0x9a1,
0x9be, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9b2, 0x9af,
0x9bc, 0x9c7, 0x9b6, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ae, 0x9be, 0x9b2, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9be, 0x9b0, 0x9cd, 0x995, 0x9c7, 0x9b8, 0x9be, 0x9b8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be, 0x9b0, 0x9cd, 0x9b6, 0x9be, 0x9b2, 0x20, 0x9a6,
0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ae, 0x9b0, 0x9bf, 0x9b6, 0x9be, 0x9b8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9b0, 0x9bf, 0x9b6, 0x9be, 0x9b8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b0, 0x9bf, 0x9b6, 0x9be, 0x9b8, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa,
0x9cd, 0x9b0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8,
0x9be, 0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa, 0x9cd, 0x9b0,
0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9b8, 0x9be, 0x997, 0x9b0,
0x9c0, 0x9af, 0x9bc, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ae, 0x9c7, 0x995, 0x9cd, 0x9b8, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa,
0x9cd, 0x9b0, 0x9b6, 0x9be, 0x9a8, 0x9cd, 0x9a4, 0x20, 0x9ae, 0x9b9, 0x9be, 0x9b8,
0x9be, 0x997, 0x9b0, 0x9c0, 0x9af, 0x9bc, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2,
0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8, 0x20,
0x9ac, 0x9be, 0x9a4, 0x9cb, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2,
0x9be, 0x9a8, 0x20, 0x9ac, 0x9be, 0x9a4, 0x9cb, 0x9b0, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b2, 0x9be, 0x9a8, 0x20, 0x9ac,
0x9be, 0x9a4, 0x9cb, 0x9b0, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8,
0x9cd, 0x995, 0x9cb, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9b8, 0x9cd, 0x995,
0x9cb, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae,
0x9b8, 0x9cd, 0x995, 0x9cb, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae,
0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ae, 0x9be,
0x9af, 0x9bc, 0x9be, 0x9a8, 0x9ae, 0x9be, 0x9b0, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a8, 0x9be, 0x989, 0x9b0, 0x9c1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9c7,
0x9aa, 0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20,
0x995, 0x9cd, 0x9af, 0x9be, 0x9b2, 0x9c7, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x20, 0x995, 0x9cd,
0x9af, 0x9be, 0x9b2, 0x9c7, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x20,
0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989,
0x20, 0x995, 0x9cd, 0x9af, 0x9be, 0x9b2, 0x9c7, 0x9a1, 0x9cb, 0x9a8, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be,
0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x99c,
0x9bf, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9a8, 0x9bf, 0x989, 0x99c, 0x9bf, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd,
0x9a1, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8,
0x9bf, 0x989, 0x99c, 0x9bf, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20,
0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9a8, 0x9bf, 0x989, 0x9ab, 0x9be, 0x989, 0x9a8, 0x9cd, 0x9a1, 0x9b2, 0x9cd, 0x9af,
0x9be, 0x9a8, 0x9cd, 0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989,
0x9ab, 0x9be, 0x989, 0x9a8, 0x9cd, 0x9a1, 0x9b2, 0x9cd, 0x9af, 0x9be, 0x9a8, 0x9cd,
0x9a1, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8,
0x9bf, 0x989, 0x9ab, 0x9be, 0x989, 0x9a8, 0x9cd, 0x9a1, 0x9b2, 0x9cd, 0x9af, 0x9be,
0x9a8, 0x9cd, 0x9a1, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9bf, 0x989, 0x987, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9a8, 0x9b0, 0x9ab, 0x9cb, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9b0, 0x9ab, 0x9cb, 0x995, 0x20, 0x9a6,
0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9a8, 0x9b0, 0x9ab, 0x9cb, 0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9ab, 0x9be, 0x9b0, 0x9cd, 0x9a8, 0x9be, 0x9a8, 0x9cd, 0x9a6, 0x9cb,
0x20, 0x9a1, 0x9bf, 0x20, 0x9a8, 0x9cb, 0x9b0, 0x9cb, 0x9a8, 0x9b9, 0x9be, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9be, 0x9b0, 0x9cd, 0x9a8, 0x9be, 0x9a8, 0x9cd,
0x9a6, 0x9cb, 0x20, 0x9a1, 0x9bf, 0x20, 0x9a8, 0x9cb, 0x9b0, 0x9cb, 0x9a8, 0x9b9,
0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab,
0x9be, 0x9b0, 0x9cd, 0x9a8, 0x9be, 0x9a8, 0x9cd, 0x9a6, 0x9cb, 0x20, 0x9a1, 0x9bf,
0x20, 0x9a8, 0x9cb, 0x9b0, 0x9cb, 0x9a8, 0x9b9, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x989, 0x9a4, 0x9cd, 0x9a4, 0x9b0, 0x20, 0x9ae, 0x9c7, 0x9b0, 0x9bf,
0x9a8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9cb, 0x9ad, 0x9cb, 0x9b8, 0x9bf, 0x9ac,
0x9bf, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8,
0x9cb, 0x9ad, 0x9cb, 0x9b8, 0x9bf, 0x9ac, 0x9bf, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a8, 0x9cb,
0x9ad, 0x9cb, 0x9b8, 0x9bf, 0x9ac, 0x9bf, 0x9b0, 0x9cd, 0x9b8, 0x9cd, 0x995, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993, 0x9ae, 0x9b8, 0x9cd, 0x995, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4,
0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8,
0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9aa, 0x9be, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9b2, 0x9be, 0x989, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9be, 0x9aa, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x20, 0x9a8,
0x9bf, 0x989, 0x20, 0x997, 0x9bf, 0x9a8, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9aa, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9be, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9be, 0x997, 0x9c1,
0x9af, 0x9bc, 0x9c7, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9aa, 0x9cd, 0x9af, 0x9be, 0x9b0, 0x9be, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9c7,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9b0, 0x9c1, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9b0, 0x9c1, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9c7, 0x9b0, 0x9c1, 0x20, 0x997, 0x9cd,
0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9b2, 0x9bf, 0x9aa, 0x9be, 0x987, 0x9a8, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf, 0x9b2, 0x9bf, 0x9aa, 0x9be, 0x987, 0x9a8,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ab, 0x9bf,
0x9b2, 0x9bf, 0x9aa, 0x9be, 0x987, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9ab, 0x9cb, 0x9a8, 0x9bf, 0x995, 0x9cd, 0x9b8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0,
0x9aa, 0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8,
0x9c7, 0x9a8, 0x9cd, 0x99f, 0x20, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x20,
0x993, 0x20, 0x9ae, 0x9bf, 0x995, 0x9c7, 0x9b2, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x99f, 0x20, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7,
0x9b0, 0x20, 0x993, 0x20, 0x9ae, 0x9bf, 0x995, 0x9c7, 0x9b2, 0x9a8, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9c7, 0x9a8, 0x9cd,
0x99f, 0x20, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9c7, 0x9b0, 0x20, 0x993, 0x20, 0x9ae,
0x9bf, 0x995, 0x9c7, 0x9b2, 0x9a8, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9bf, 0x99f, 0x995, 0x9c7, 0x9af,
0x9bc, 0x9be, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9cb,
0x9a8, 0x9be, 0x9aa, 0x9c7, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9aa, 0x9bf, 0x9af,
0x9bc, 0x982, 0x987, 0x9af, 0x9bc, 0x9be, 0x982, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x995, 0x9bf, 0x99c, 0x9bf, 0x9b2, 0x9cb, 0x9b0, 0x9cd, 0x9a1, 0x9be, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf, 0x99c, 0x9bf, 0x9b2, 0x9cb, 0x9b0, 0x9cd, 0x9a1,
0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x995, 0x9bf,
0x99c, 0x9bf, 0x9b2, 0x9cb, 0x9b0, 0x9cd, 0x9a1, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9b0, 0x9bf, 0x987, 0x989, 0x9a8, 0x9bf, 0x9af, 0x9bc, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x9b0, 0x9a5, 0x9c7, 0x9b0, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9b8, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x9b8, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x996, 0x9be, 0x9b2, 0x9bf, 0x9a8,
0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9be, 0x9b0, 0x9be,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9be, 0x9b0, 0x9be, 0x20,
0x9ae, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9be,
0x9b0, 0x9be, 0x20, 0x997, 0x9c3, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0,
0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9cb, 0x9af, 0x9bc,
0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9ae, 0x9cb, 0x9af, 0x9bc,
0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8,
0x9be, 0x9ae, 0x9cb, 0x9af, 0x9bc, 0x9be, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2,
0x9cb, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9c7, 0x9b6, 0x9c7, 0x9b2,
0x9b8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9bf, 0x999, 0x9cd, 0x997, 0x9be,
0x9aa, 0x9c1, 0x9b0, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9b8, 0x9b2, 0x9cb, 0x9ae, 0x9a8, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa,
0x9aa, 0x9c1, 0x99e, 0x9cd, 0x99c, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a6, 0x995,
0x9cd, 0x9b7, 0x9bf, 0x9a3, 0x20, 0x99c, 0x9b0, 0x9cd, 0x99c, 0x9bf, 0x9af, 0x9bc,
0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9c1, 0x9b0, 0x9bf, 0x9a8, 0x9be,
0x9ae, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9b8, 0x9be, 0x9af, 0x9bc, 0x9cb, 0x993,
0x9af, 0x9bc, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x9b9, 0x9bf,
0x9a4, 0x9bf, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x987, 0x9aa, 0x9c7,
0x987, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x987, 0x9aa, 0x9c7, 0x987,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be,
0x987, 0x9aa, 0x9c7, 0x987, 0x20, 0x9a6, 0x9bf, 0x9ac, 0x9be, 0x9b2, 0x9cb, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9be, 0x99c, 0x9be, 0x996, 0x9be, 0x9b8,
0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9cb, 0x995,
0x9c7, 0x9b2, 0x9be, 0x989, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9cb, 0x999,
0x9cd, 0x997, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f, 0x9cb, 0x999, 0x9cd,
0x997, 0x9be, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x99f, 0x9cb, 0x999, 0x9cd, 0x997, 0x9be, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7,
0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x99a, 0x9c1, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9b0, 0x9cd,
0x995, 0x9ae, 0x9c7, 0x9a8, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9b0, 0x9cd, 0x995, 0x9ae, 0x9c7, 0x9a8, 0x9bf,
0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9a4, 0x9c1, 0x9b0, 0x9cd, 0x995, 0x9ae, 0x9c7, 0x9a8, 0x9bf,
0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x99f,
0x9c1, 0x9ad, 0x9be, 0x9b2, 0x9c1, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b0,
0x9c1, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989,
0x9b0, 0x9c1, 0x997, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x9b0, 0x9c1, 0x997, 0x9c1, 0x9af, 0x9bc,
0x9c7, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2,
0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x99c, 0x9ac, 0x9c7, 0x995,
0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989,
0x99c, 0x9ac, 0x9c7, 0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x989, 0x99c, 0x9ac, 0x9c7,
0x995, 0x9bf, 0x9b8, 0x9cd, 0x9a4, 0x9be, 0x9a8, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ad, 0x9be, 0x9a8, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a4, 0x9c1, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be, 0x9a8, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a4, 0x9c1,
0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9be,
0x9a8, 0x9c1, 0x9af, 0x9bc, 0x9be, 0x9a4, 0x9c1, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0,
0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ad, 0x9c7, 0x9a8, 0x9c7, 0x99c, 0x9c1, 0x9af, 0x9bc, 0x9c7, 0x9b2, 0x9be,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a6, 0x9bf, 0x9ad,
0x9b8, 0x9cd, 0x9a4, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2,
0x9be, 0x9a6, 0x9bf, 0x9ad, 0x9b8, 0x9cd, 0x9a4, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8,
0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9cd, 0x9b2, 0x9be, 0x9a6, 0x9bf,
0x9ad, 0x9b8, 0x9cd, 0x9a4, 0x995, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad,
0x9b2, 0x997, 0x9cb, 0x997, 0x9cd, 0x9b0, 0x9be, 0x9a1, 0x20, 0x9b8, 0x9ae, 0x9af,
0x9bc, 0x9ad, 0x9b2, 0x997, 0x9cb, 0x997, 0x9cd, 0x9b0, 0x9be, 0x9a1, 0x20, 0x9ae,
0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad, 0x9b2, 0x997, 0x9cb,
0x997, 0x9cd, 0x9b0, 0x9be, 0x9a1, 0x20, 0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd,
0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x9ad,
0x9b8, 0x99f, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x993, 0x9af, 0x9bc, 0x9c7,
0x995, 0x20, 0x9a6, 0x9cd, 0x9ac, 0x9c0, 0x9aa, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x993, 0x9af, 0x9bc, 0x9be, 0x9b2, 0x9bf, 0x9b8, 0x20, 0x98f, 0x9ac, 0x982, 0x20,
0x9ab, 0x9c1, 0x99f, 0x9c1, 0x9a8, 0x9be, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987,
0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x99f, 0x9be, 0x9b8, 0x9cd, 0x995, 0x20, 0x9b8,
0x9ae, 0x9af, 0x9bc, 0x987, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x99f, 0x9be, 0x9b8,
0x9cd, 0x995, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x987, 0x9af, 0x9bc, 0x9be, 0x995, 0x9c1, 0x99f, 0x9be, 0x9b8, 0x9cd, 0x995, 0x20,
0x997, 0x9cd, 0x9b0, 0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8,
0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x9a4,
0x9c7, 0x9b0, 0x9bf, 0x9a8, 0x9ac, 0x9c1, 0x9b0, 0x9cd, 0x997, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x987, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x9a4, 0x9c7, 0x9b0, 0x9bf,
0x9a8, 0x9ac, 0x9c1, 0x9b0, 0x9cd, 0x997, 0x20, 0x9ae, 0x9be, 0x9a8, 0x995, 0x20,
0x9b8, 0x9ae, 0x9af, 0x9bc, 0x987, 0x9af, 0x9bc, 0x9c7, 0x995, 0x9be, 0x9a4, 0x9c7,
0x9b0, 0x9bf, 0x9a8, 0x9ac, 0x9c1, 0x9b0, 0x9cd, 0x997, 0x20, 0x997, 0x9cd, 0x9b0,
0x9c0, 0x9b7, 0x9cd, 0x9ae, 0x995, 0x9be, 0x9b2, 0x9c0, 0x9a8, 0x20, 0x9b8, 0x9ae,
0x9af, 0x9bc, 0x987, 0x989, 0x995, 0x9cb, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9af, 0x9bc,
0x41, 0x63, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x41, 0x63, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x63,
0x72, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x68, 0x65, 0x67, 0x6f,
0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e,
0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20,
0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f,
0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x65,
0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x65,
0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6d,
0x65, 0x6e, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x69, 0x61, 0x6c,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66,
0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49,
0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b,
0x6f, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x6f, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66,
0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x72,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x70, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x70, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x70, 0x69, 0x61, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6b, 0x74, 0x6f,
0x62, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x6b,
0x74, 0x6f, 0x62, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x6b,
0x74, 0x6f, 0x62, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x72, 0x61, 0x62,
0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x72, 0x61, 0x62,
0x69, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6d, 0x65, 0x6e,
0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64,
0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x49, 0x70, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x6b, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x6f,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x70, 0x61, 0x72, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6b, 0x6f, 0x20, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x6f, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69,
0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b,
0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x65, 0x72, 0x64, 0x69, 0x2d, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65,
0x72, 0x64, 0x69, 0x2d, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69,
0x2d, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b,
0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64,
0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x67, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x65, 0x74, 0x61,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x43, 0x61, 0x62, 0x6f, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x43,
0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x43,
0x61, 0x73, 0x65, 0x79, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x67, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x54, 0x78, 0x69, 0x6c, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x54, 0x78, 0x69, 0x6c, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x54, 0x78, 0x69, 0x6c, 0x65, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x78,
0x69, 0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54,
0x78, 0x69, 0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x54, 0x78,
0x69, 0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x78, 0x6f, 0x69, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54,
0x78, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x67, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x54, 0x78, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73,
0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x75, 0x68, 0x61,
0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4b, 0x6f, 0x6c, 0x6f, 0x6e, 0x62, 0x69, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x6f, 0x6c, 0x6f, 0x6e, 0x62,
0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4b, 0x6f, 0x6c, 0x6f,
0x6e, 0x62, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x75,
0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x75, 0x68, 0x61,
0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65,
0x74, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x20, 0x65, 0x72, 0x64,
0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4b, 0x75, 0x62, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x4b, 0x75, 0x62, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4b,
0x75, 0x62, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x44, 0x61, 0x76, 0x69, 0x73, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x45, 0x6b, 0x69, 0x61, 0x6c,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x72, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x7a, 0x6b,
0x6f, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x7a, 0x6b, 0x6f, 0x20, 0x75, 0x68,
0x61, 0x72, 0x74, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x50, 0x61,
0x7a, 0x6b, 0x6f, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x45, 0x6b, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b,
0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x6b, 0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b,
0x6f, 0x20, 0x65, 0x72, 0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69,
0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69,
0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f,
0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x20, 0x75, 0x72, 0x72,
0x75, 0x6e, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64,
0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20, 0x6d,
0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6b, 0x6f, 0x20,
0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x75, 0x68, 0x61,
0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x75,
0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x75,
0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46, 0x69,
0x6a, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46, 0x69,
0x6a, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x46, 0x69, 0x6a, 0x69,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61,
0x6e, 0x74, 0x73, 0x65, 0x73, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x46, 0x72, 0x61, 0x6e, 0x74, 0x7a, 0x69, 0x61, 0x72, 0x65,
0x6e, 0x20, 0x6c, 0x75, 0x72, 0x72, 0x61, 0x6c, 0x64, 0x65, 0x20, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x65, 0x74, 0x61, 0x20, 0x61,
0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x6b, 0x6f, 0x65, 0x74, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x74, 0x65, 0x67, 0x69, 0x61, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x65, 0x74, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x72, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x20, 0x75, 0x68,
0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x6f,
0x61, 0x72, 0x65, 0x6e, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6b, 0x6f, 0x20,
0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65,
0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e,
0x64, 0x65, 0x62, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x47, 0x75, 0x61,
0x6d, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x47, 0x6f, 0x6c, 0x6b,
0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x72,
0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x72, 0x20, 0x75, 0x68,
0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x72, 0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65,
0x74, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x68, 0x6f, 0x76,
0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x68,
0x6f, 0x76, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4b, 0x68,
0x6f, 0x76, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x7a, 0x65, 0x61, 0x6e, 0x6f, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x6f, 0x74, 0x78, 0x69,
0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x65, 0x72,
0x64, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x6b,
0x6f, 0x20, 0x65, 0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61,
0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49,
0x72, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49,
0x72, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x72, 0x61,
0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4a, 0x61, 0x70, 0x6f,
0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4a, 0x61, 0x70,
0x6f, 0x6e, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x61,
0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x65,
0x6b, 0x69, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x67, 0x6f, 0x20, 0x6d, 0x65, 0x6e, 0x64, 0x65, 0x62, 0x61, 0x6c, 0x64,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x69, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73,
0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x69,
0x72, 0x67, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x75,
0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d,
0x61, 0x63, 0x61, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x4d, 0x61, 0x63, 0x61, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4d,
0x61, 0x63, 0x61, 0x6f, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x6c, 0x61, 0x79,
0x73, 0x69, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x72, 0x6b, 0x65, 0x73, 0x65, 0x74,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x55, 0x68, 0x61, 0x72, 0x74, 0x65,
0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x7a, 0x69, 0x6f, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x7a, 0x69, 0x6f, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x7a, 0x69,
0x6f, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f,
0x6b, 0x6f, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x6f, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x65, 0x78, 0x69, 0x6b,
0x6f, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x6f,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x6f,
0x6b, 0x6f, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x6f, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x72,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x75, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x75, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x75, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x72, 0x65, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x67,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x72, 0x69, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x20, 0x42, 0x65, 0x72, 0x72, 0x69, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x20, 0x42, 0x65, 0x72, 0x72, 0x69, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x5a, 0x65, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x20, 0x42, 0x65, 0x72, 0x72, 0x69, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x20, 0x42, 0x65, 0x72, 0x72, 0x69, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x42,
0x65, 0x72, 0x72, 0x69, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x65, 0x72, 0x6e, 0x75, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x65, 0x72, 0x6e,
0x75, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x54, 0x65, 0x72, 0x6e,
0x75, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x4e, 0x69, 0x75, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x72, 0x61, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20,
0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6b, 0x6f,
0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x49, 0x70, 0x61, 0x72, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61,
0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4f, 0x6d,
0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4f,
0x6d, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4f, 0x6d,
0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x61,
0x70, 0x75, 0x61, 0x20, 0x47, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x42, 0x65,
0x72, 0x72, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x69, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x50, 0x65, 0x72, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x50, 0x65, 0x72, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x50, 0x65, 0x72, 0x75, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x74, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78,
0x20, 0x75, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x74, 0x61, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x75, 0x6e, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x75, 0x6e, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65,
0x74, 0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x75, 0x6e, 0x65, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x50, 0x69, 0x6f, 0x6e,
0x67, 0x69, 0x61, 0x6e, 0x67, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64,
0x75, 0x61, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4b, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4b, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x52, 0x65, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x75,
0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72,
0x72, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x53, 0x61, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x20, 0x55, 0x68, 0x61, 0x72, 0x74, 0x65, 0x65, 0x74,
0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x48, 0x65, 0x67,
0x6f, 0x61, 0x6c, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x65, 0x74, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x67, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x61, 0x64, 0x6a, 0x69, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x72, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6b, 0x6f, 0x20, 0x75,
0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x43, 0x68,
0x75, 0x75, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x69, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x69, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x67,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x72, 0x61, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x67, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20,
0x6f, 0x72, 0x64, 0x75, 0x61, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75,
0x65, 0x6c, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x6b,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x61, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72,
0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72,
0x61, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x6b,
0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75,
0x61, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x61, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x75, 0x68, 0x61,
0x72, 0x74, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x74, 0x61, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x74,
0x65, 0x67, 0x69, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x72, 0x61, 0x4a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61,
0x6b, 0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x65, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x65, 0x6b, 0x6f, 0x20, 0x6f,
0x72, 0x64, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x72, 0x61, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x65, 0x6b, 0x6f, 0x20, 0x75, 0x64, 0x61, 0x6b, 0x6f,
0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x67,
0x6f, 0x20, 0x6f, 0x72, 0x64, 0x75, 0x61, 0x410, 0x444, 0x433, 0x430, 0x43d,
0x456, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441,
0x426, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x430, 0x444,
0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441,
0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x430, 0x444, 0x440, 0x44b, 0x43a,
0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e,
0x434, 0x43d, 0x451, 0x432, 0x430, 0x430, 0x444, 0x440, 0x44b, 0x43a, 0x430, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445, 0x43e, 0x434,
0x43d, 0x435, 0x430, 0x444, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x430,
0x444, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x430, 0x444, 0x440, 0x44b, 0x43a,
0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x410, 0x43b, 0x44f, 0x441, 0x43a,
0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x456, 0x41b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x43b, 0x44f, 0x441,
0x43a, 0x456, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x447, 0x430, 0x441, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430,
0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a,
0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x44c, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d,
0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x44b, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d,
0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x44b, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e,
0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430,
0x43d, 0x441, 0x43a, 0x456, 0x20, 0x45e, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430,
0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x45e, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430,
0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a,
0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x45e, 0x441, 0x445, 0x43e, 0x434, 0x43d,
0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41f,
0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b,
0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x433, 0x43e, 0x440, 0x43d, 0x44b,
0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d, 0x43e, 0x447, 0x43d, 0x430,
0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x433, 0x43e, 0x440, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x45e, 0x43d,
0x43e, 0x447, 0x43d, 0x430, 0x430, 0x43c, 0x435, 0x440, 0x44b, 0x43a, 0x430, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x433, 0x43e, 0x440, 0x43d, 0x44b, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x426, 0x456, 0x445, 0x430, 0x430,
0x43a, 0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x426,
0x456, 0x445, 0x430, 0x430, 0x43a, 0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x426, 0x456, 0x445, 0x430, 0x430, 0x43a, 0x456, 0x44f, 0x43d, 0x441,
0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x427, 0x430, 0x441, 0x20, 0x410, 0x43f, 0x456, 0x456, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410,
0x43f, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x410, 0x43f, 0x456, 0x456, 0x427, 0x430, 0x441, 0x20, 0x421, 0x430, 0x443,
0x434, 0x430, 0x45e, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x410, 0x440, 0x430, 0x432,
0x456, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b,
0x20, 0x447, 0x430, 0x441, 0x20, 0x421, 0x430, 0x443, 0x434, 0x430, 0x45e, 0x441,
0x43a, 0x430, 0x439, 0x20, 0x410, 0x440, 0x430, 0x432, 0x456, 0x456, 0x41b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x421, 0x430, 0x443, 0x434,
0x430, 0x45e, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x410, 0x440, 0x430, 0x432, 0x456,
0x456, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x446, 0x456, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x446, 0x456, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x446,
0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x417, 0x430, 0x445, 0x43e, 0x434,
0x43d, 0x44f, 0x439, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x446, 0x456, 0x43d,
0x44b, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x446, 0x456, 0x43d, 0x44b, 0x41b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x417, 0x430, 0x445, 0x43e,
0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x446, 0x456,
0x43d, 0x44b, 0x427, 0x430, 0x441, 0x20, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x456,
0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x456, 0x456, 0x41b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x440, 0x43c,
0x435, 0x43d, 0x456, 0x456, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x44b, 0x447,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442,
0x44b, 0x447, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x410, 0x442, 0x43b, 0x430, 0x43d,
0x442, 0x44b, 0x447, 0x43d, 0x44b, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x439, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x456, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x439, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430,
0x439, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x426,
0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x2d, 0x437, 0x430,
0x445, 0x43e, 0x434, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x45e,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x446, 0x44d, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x44c, 0x43d, 0x430, 0x2d, 0x437, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x446, 0x44d, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x2d, 0x437, 0x430, 0x445, 0x43e, 0x434,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x456, 0x456, 0x427, 0x430, 0x441, 0x20, 0x443, 0x441, 0x445, 0x43e,
0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x456, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b,
0x20, 0x447, 0x430, 0x441, 0x20, 0x443, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x44f,
0x439, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x41b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x443, 0x441, 0x445,
0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x456, 0x456, 0x427, 0x430, 0x441, 0x20, 0x437, 0x430, 0x445, 0x43e, 0x434,
0x43d, 0x44f, 0x439, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456,
0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x437, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439,
0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x456, 0x41b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x437, 0x430, 0x445, 0x43e,
0x434, 0x43d, 0x44f, 0x439, 0x20, 0x410, 0x45e, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x456, 0x456, 0x427, 0x430, 0x441, 0x20, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430,
0x439, 0x434, 0x436, 0x430, 0x43d, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x437, 0x435,
0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x430, 0x41b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x437, 0x435, 0x440, 0x431,
0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x430, 0x427, 0x430, 0x441, 0x20, 0x410,
0x437, 0x43e, 0x440, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440,
0x430, 0x432, 0x43e, 0x45e, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441,
0x43a, 0x456, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e,
0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x410, 0x437,
0x43e, 0x440, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430,
0x432, 0x43e, 0x45e, 0x427, 0x430, 0x441, 0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b,
0x430, 0x434, 0x44d, 0x448, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b,
0x430, 0x434, 0x44d, 0x448, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x44d, 0x448, 0x427,
0x430, 0x441, 0x20, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x430, 0x411, 0x430, 0x43b,
0x456, 0x432, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x411,
0x440, 0x430, 0x437, 0x456, 0x43b, 0x44c, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x411, 0x440, 0x430, 0x437, 0x456, 0x43b, 0x44c, 0x441, 0x43a, 0x456, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x411, 0x440, 0x430, 0x437, 0x456, 0x43b, 0x44c, 0x441, 0x43a, 0x456,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430,
0x441, 0x20, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x44f, 0x427, 0x430, 0x441, 0x20,
0x41a, 0x430, 0x431, 0x430, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x44d, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x20, 0x41a, 0x430, 0x431, 0x430, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x44d, 0x41b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41a, 0x430, 0x431,
0x430, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x44d, 0x427, 0x430, 0x441, 0x20, 0x427,
0x430, 0x43c, 0x43e, 0x440, 0x430, 0x427, 0x430, 0x441, 0x20, 0x427, 0x430, 0x442,
0x44d, 0x43c, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x427, 0x430, 0x442, 0x44d, 0x43c, 0x430,
0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x427, 0x430,
0x442, 0x44d, 0x43c, 0x430, 0x427, 0x44b, 0x43b, 0x456, 0x439, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x427, 0x44b, 0x43b, 0x456, 0x439, 0x441, 0x43a, 0x456,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x427, 0x44b, 0x43b, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441,
0x20, 0x41a, 0x456, 0x442, 0x430, 0x44f, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41a, 0x456, 0x442,
0x430, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20,
0x41a, 0x456, 0x442, 0x430, 0x44f, 0x427, 0x430, 0x441, 0x20, 0x427, 0x430, 0x439,
0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x427, 0x430,
0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x430, 0x41b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x427, 0x430, 0x439, 0x431, 0x430, 0x43b,
0x441, 0x430, 0x43d, 0x430, 0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442,
0x440, 0x430, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x44f, 0x434, 0x427, 0x430,
0x441, 0x20, 0x41a, 0x430, 0x43a, 0x43e, 0x441, 0x430, 0x432, 0x44b, 0x445, 0x20,
0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x41a, 0x430, 0x43b, 0x443,
0x43c, 0x431, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41a,
0x430, 0x43b, 0x443, 0x43c, 0x431, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x41a, 0x430, 0x43b, 0x443, 0x43c, 0x431, 0x456, 0x439, 0x441, 0x43a, 0x456,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430,
0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x41a,
0x443, 0x43a, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432,
0x43e, 0x45e, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x41f, 0x430, 0x45e, 0x43b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x430, 0x441, 0x442, 0x440,
0x430, 0x432, 0x43e, 0x45e, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x427, 0x430, 0x441,
0x20, 0x41a, 0x443, 0x431, 0x44b, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41a, 0x443, 0x431, 0x44b,
0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41a, 0x443,
0x431, 0x44b, 0x427, 0x430, 0x441, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b,
0x456, 0x20, 0x414, 0x44d, 0x439, 0x432, 0x456, 0x441, 0x427, 0x430, 0x441, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20, 0x414, 0x437, 0x44e, 0x43c,
0x43e, 0x43d, 0x2d, 0x414, 0x437, 0x44e, 0x440, 0x432, 0x456, 0x43b, 0x44c, 0x427,
0x430, 0x441, 0x20, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x433, 0x430,
0x20, 0x422, 0x44b, 0x43c, 0x43e, 0x440, 0x430, 0x427, 0x430, 0x441, 0x20, 0x432,
0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430, 0x20, 0x412, 0x44f, 0x43b, 0x456,
0x43a, 0x430, 0x434, 0x43d, 0x44f, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442,
0x440, 0x430, 0x432, 0x430, 0x20, 0x412, 0x44f, 0x43b, 0x456, 0x43a, 0x430, 0x434,
0x43d, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20,
0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430, 0x20, 0x412, 0x44f, 0x43b,
0x456, 0x43a, 0x430, 0x434, 0x43d, 0x44f, 0x42d, 0x43a, 0x432, 0x430, 0x434, 0x43e,
0x440, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x426, 0x44d, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435,
0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x426, 0x44d, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435,
0x439, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x426, 0x44d, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439,
0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x435, 0x45e, 0x440, 0x430,
0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x423, 0x441,
0x445, 0x43e, 0x434, 0x43d, 0x435, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439,
0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d,
0x435, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x414, 0x430, 0x43b,
0x451, 0x43a, 0x430, 0x45e, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x435, 0x45e,
0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441,
0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x435, 0x45e, 0x440, 0x430, 0x43f,
0x435, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445,
0x43e, 0x434, 0x43d, 0x435, 0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441,
0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x44b, 0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435,
0x435, 0x45e, 0x440, 0x430, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20,
0x424, 0x430, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x445,
0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20,
0x424, 0x430, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x445,
0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x41b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x424, 0x430, 0x43b, 0x43a, 0x43b,
0x435, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440,
0x430, 0x432, 0x43e, 0x45e, 0x427, 0x430, 0x441, 0x20, 0x424, 0x456, 0x434, 0x436,
0x44b, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x424, 0x456, 0x434, 0x436, 0x44b, 0x41b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x424, 0x456, 0x434, 0x436, 0x44b,
0x427, 0x430, 0x441, 0x20, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x441,
0x43a, 0x430, 0x439, 0x20, 0x413, 0x432, 0x456, 0x44f, 0x43d, 0x44b, 0x427, 0x430,
0x441, 0x20, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x441, 0x43a, 0x430,
0x439, 0x20, 0x43f, 0x430, 0x45e, 0x434, 0x43d, 0x451, 0x432, 0x430, 0x2d, 0x430,
0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x44b, 0x447, 0x43d, 0x430, 0x439, 0x20,
0x442, 0x44d, 0x440, 0x44b, 0x442, 0x43e, 0x440, 0x44b, 0x456, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20,
0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x43a, 0x456, 0x445,
0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x427, 0x430, 0x441,
0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x413, 0x430,
0x43c, 0x431, 0x2019, 0x435, 0x413, 0x440, 0x443, 0x437, 0x456, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x413, 0x440, 0x443, 0x437, 0x456, 0x43d, 0x441,
0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x44b, 0x20, 0x447, 0x430, 0x441, 0x413, 0x440, 0x443, 0x437, 0x456, 0x43d, 0x441,
0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x427, 0x430, 0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e,
0x20, 0x413, 0x456, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x430, 0x427, 0x430, 0x441,
0x20, 0x43f, 0x430, 0x20, 0x413, 0x440, 0x44b, 0x43d, 0x432, 0x456, 0x447, 0x44b,
0x427, 0x430, 0x441, 0x20, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439,
0x20, 0x413, 0x440, 0x44d, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x413,
0x440, 0x44d, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456, 0x41b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x423, 0x441, 0x445, 0x43e, 0x434,
0x43d, 0x44f, 0x439, 0x20, 0x413, 0x440, 0x44d, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x44b, 0x456, 0x427, 0x430, 0x441, 0x20, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d,
0x44f, 0x439, 0x20, 0x413, 0x440, 0x44d, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b,
0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x44f, 0x439,
0x20, 0x413, 0x440, 0x44d, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456, 0x41b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x417, 0x430, 0x445,
0x43e, 0x434, 0x43d, 0x44f, 0x439, 0x20, 0x413, 0x440, 0x44d, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x44b, 0x456, 0x427, 0x430, 0x441, 0x20, 0x41f, 0x435, 0x440, 0x441,
0x456, 0x434, 0x441, 0x43a, 0x430, 0x433, 0x430, 0x20, 0x437, 0x430, 0x43b, 0x456,
0x432, 0x430, 0x427, 0x430, 0x441, 0x20, 0x413, 0x430, 0x44f, 0x43d, 0x44b, 0x413,
0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x430, 0x2d, 0x410, 0x43b, 0x435, 0x443,
0x446, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x413, 0x430, 0x432, 0x430, 0x439,
0x441, 0x43a, 0x430, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x446, 0x43a, 0x456, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x413, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x430, 0x2d, 0x410,
0x43b, 0x435, 0x443, 0x446, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x413, 0x430, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x413, 0x430, 0x43d, 0x43a, 0x43e,
0x43d, 0x433, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x413, 0x430, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x430, 0x427, 0x430, 0x441,
0x20, 0x425, 0x43e, 0x45e, 0x434, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x425, 0x43e, 0x45e,
0x434, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20,
0x425, 0x43e, 0x45e, 0x434, 0x430, 0x427, 0x430, 0x441, 0x20, 0x406, 0x43d, 0x434,
0x44b, 0x456, 0x427, 0x430, 0x441, 0x20, 0x406, 0x43d, 0x434, 0x44b, 0x439, 0x441,
0x43a, 0x430, 0x433, 0x430, 0x20, 0x430, 0x43a, 0x456, 0x44f, 0x43d, 0x430, 0x406,
0x43d, 0x434, 0x430, 0x43a, 0x456, 0x442, 0x430, 0x439, 0x441, 0x43a, 0x456, 0x20,
0x447, 0x430, 0x441, 0x426, 0x44d, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d,
0x430, 0x456, 0x43d, 0x434, 0x430, 0x43d, 0x435, 0x437, 0x456, 0x439, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x435,
0x456, 0x43d, 0x434, 0x430, 0x43d, 0x435, 0x437, 0x456, 0x439, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x456,
0x43d, 0x434, 0x430, 0x43d, 0x435, 0x437, 0x456, 0x439, 0x441, 0x43a, 0x456, 0x20,
0x447, 0x430, 0x441, 0x406, 0x440, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447,
0x430, 0x441, 0x406, 0x440, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x406, 0x440, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x406, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x406, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x456, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x406, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x456, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x406, 0x437, 0x440, 0x430, 0x456,
0x43b, 0x44c, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x406, 0x437, 0x440,
0x430, 0x456, 0x43b, 0x44c, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x406, 0x437,
0x440, 0x430, 0x456, 0x43b, 0x44c, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x42f, 0x43f,
0x43e, 0x43d, 0x456, 0x456, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x42f, 0x43f, 0x43e, 0x43d, 0x456,
0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x42f,
0x43f, 0x43e, 0x43d, 0x456, 0x456, 0x423, 0x441, 0x445, 0x43e, 0x434, 0x43d, 0x435,
0x43a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x417, 0x430, 0x445, 0x43e, 0x434, 0x43d, 0x435, 0x43a,
0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x41a, 0x430, 0x440, 0x44d, 0x456,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x20, 0x41a, 0x430, 0x440, 0x44d, 0x456, 0x41b, 0x435, 0x442, 0x43d,
0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41a, 0x430, 0x440, 0x44d, 0x456, 0x427,
0x430, 0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20,
0x41a, 0x443, 0x441, 0x430, 0x456, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x430,
0x44f, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x430, 0x44f, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41a,
0x440, 0x430, 0x441, 0x43d, 0x430, 0x44f, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20,
0x41a, 0x44b, 0x440, 0x433, 0x44b, 0x437, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x427,
0x430, 0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20,
0x41b, 0x430, 0x439, 0x43d, 0x427, 0x430, 0x441, 0x20, 0x41b, 0x43e, 0x440, 0x434,
0x2d, 0x425, 0x430, 0x443, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x2d,
0x425, 0x430, 0x443, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20,
0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x456, 0x456, 0x427, 0x430, 0x441, 0x20,
0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x44b, 0x432, 0x430, 0x45e, 0x427, 0x430, 0x441,
0x20, 0x41c, 0x430, 0x440, 0x43a, 0x456, 0x437, 0x441, 0x43a, 0x456, 0x445, 0x20,
0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x427, 0x430, 0x441, 0x20,
0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x430, 0x432, 0x44b, 0x445, 0x20, 0x430,
0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x427, 0x430, 0x441, 0x20, 0x41c,
0x430, 0x45e, 0x440, 0x44b, 0x43a, 0x456, 0x44f, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41c, 0x430,
0x45e, 0x440, 0x44b, 0x43a, 0x456, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x41c, 0x430, 0x45e, 0x440, 0x44b, 0x43a, 0x456, 0x44f,
0x427, 0x430, 0x441, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20,
0x41c, 0x43e, 0x45e, 0x441, 0x430, 0x43d, 0x41c, 0x435, 0x43a, 0x441, 0x456, 0x43a,
0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x446, 0x456, 0x445, 0x430, 0x430, 0x43a,
0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41c, 0x435,
0x43a, 0x441, 0x456, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x446, 0x456,
0x445, 0x430, 0x430, 0x43a, 0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x41c, 0x435, 0x43a, 0x441, 0x456, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20,
0x446, 0x456, 0x445, 0x430, 0x430, 0x43a, 0x456, 0x44f, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430,
0x441, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x430, 0x440,
0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442,
0x430, 0x440, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x430, 0x440, 0x430,
0x41c, 0x430, 0x441, 0x43a, 0x43e, 0x45e, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x41c, 0x430, 0x441, 0x43a, 0x43e, 0x45e, 0x441, 0x43a, 0x456, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x41c, 0x430, 0x441, 0x43a, 0x43e, 0x45e, 0x441, 0x43a, 0x456, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20,
0x41c, 0x2019, 0x44f, 0x43d, 0x43c, 0x44b, 0x427, 0x430, 0x441, 0x20, 0x41d, 0x430,
0x443, 0x440, 0x443, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x44c, 0x441, 0x43a, 0x456,
0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x41d, 0x43e, 0x432, 0x430,
0x439, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x456, 0x456, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x439, 0x20, 0x41a, 0x430, 0x43b, 0x435,
0x434, 0x43e, 0x43d, 0x456, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x439, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x456, 0x456, 0x427, 0x430, 0x441, 0x20, 0x41d, 0x43e,
0x432, 0x430, 0x439, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x456,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x439, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x44b, 0x456, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x439, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x44b, 0x456, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x45e, 0x43d,
0x434, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441,
0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x45e, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x45e,
0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x41d,
0x456, 0x443, 0x44d, 0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440,
0x430, 0x432, 0x430, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x430, 0x43b, 0x43a, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430, 0x20, 0x41d,
0x43e, 0x440, 0x444, 0x430, 0x43b, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430,
0x20, 0x41d, 0x43e, 0x440, 0x444, 0x430, 0x43b, 0x43a, 0x427, 0x430, 0x441, 0x20,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x437, 0x456,
0x2d, 0x41d, 0x430, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x437, 0x456, 0x2d,
0x41d, 0x430, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x443, 0x2d, 0x434, 0x437, 0x456, 0x2d, 0x41d, 0x430, 0x440, 0x43e, 0x43d, 0x44c,
0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x441, 0x456, 0x431, 0x456, 0x440, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x41d, 0x43e, 0x432, 0x430, 0x441, 0x456, 0x431,
0x456, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41d, 0x43e, 0x432, 0x430,
0x441, 0x456, 0x431, 0x456, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x41e, 0x43c, 0x441, 0x43a, 0x456, 0x20,
0x447, 0x430, 0x441, 0x41e, 0x43c, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41e,
0x43c, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x41f, 0x430, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x43a, 0x456, 0x441, 0x442, 0x430,
0x43d, 0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x430, 0x43a, 0x456, 0x441,
0x442, 0x430, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x41f, 0x430, 0x43b, 0x430,
0x443, 0x427, 0x430, 0x441, 0x20, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x2d, 0x41d,
0x43e, 0x432, 0x430, 0x439, 0x20, 0x413, 0x432, 0x456, 0x43d, 0x435, 0x456, 0x427,
0x430, 0x441, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x44f, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x44f, 0x41b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x41f, 0x430, 0x440, 0x430,
0x433, 0x432, 0x430, 0x44f, 0x41f, 0x435, 0x440, 0x443, 0x430, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x435, 0x440, 0x443, 0x430, 0x43d, 0x441,
0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x44b, 0x20, 0x447, 0x430, 0x441, 0x41f, 0x435, 0x440, 0x443, 0x430, 0x43d, 0x441,
0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x424, 0x456, 0x43b, 0x456, 0x43f, 0x456, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447,
0x430, 0x441, 0x424, 0x456, 0x43b, 0x456, 0x43f, 0x456, 0x43d, 0x441, 0x43a, 0x456,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x424, 0x456, 0x43b, 0x456, 0x43f, 0x456, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427,
0x430, 0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20,
0x424, 0x435, 0x43d, 0x456, 0x43a, 0x441, 0x427, 0x430, 0x441, 0x20, 0x421, 0x435,
0x43d, 0x2d, 0x41f, 0x2019, 0x435, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20, 0x421, 0x435, 0x43d, 0x2d, 0x41f,
0x2019, 0x435, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x421, 0x435,
0x43d, 0x2d, 0x41f, 0x2019, 0x435, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442,
0x440, 0x430, 0x432, 0x430, 0x20, 0x41f, 0x456, 0x442, 0x43a, 0x44d, 0x440, 0x43d,
0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441, 0x442, 0x440, 0x430, 0x432, 0x430,
0x20, 0x41f, 0x430, 0x43d, 0x43f, 0x435, 0x456, 0x41f, 0x445, 0x435, 0x43d, 0x44c,
0x44f, 0x43d, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427, 0x430, 0x441,
0x20, 0x420, 0x44d, 0x44e, 0x43d, 0x44c, 0x451, 0x43d, 0x430, 0x427, 0x430, 0x441,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20, 0x420, 0x43e, 0x442,
0x44d, 0x440, 0x430, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x456, 0x43d, 0x441, 0x43a,
0x456, 0x20, 0x447, 0x430, 0x441, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x456, 0x43d,
0x441, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x456,
0x43d, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456,
0x20, 0x447, 0x430, 0x441, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x427, 0x430,
0x441, 0x20, 0x421, 0x435, 0x439, 0x448, 0x44d, 0x43b, 0x44c, 0x441, 0x43a, 0x456,
0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x421, 0x456,
0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x427, 0x430, 0x441, 0x20, 0x421, 0x430, 0x43b, 0x430, 0x43c, 0x43e, 0x43d,
0x430, 0x432, 0x44b, 0x445, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e,
0x45e, 0x427, 0x430, 0x441, 0x20, 0x41f, 0x430, 0x45e, 0x434, 0x43d, 0x451, 0x432,
0x430, 0x439, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x456, 0x456, 0x427, 0x430,
0x441, 0x20, 0x421, 0x443, 0x440, 0x44b, 0x43d, 0x430, 0x43c, 0x430, 0x427, 0x430,
0x441, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20, 0x421, 0x451,
0x432, 0x430, 0x427, 0x430, 0x441, 0x20, 0x422, 0x430, 0x456, 0x446, 0x456, 0x427,
0x430, 0x441, 0x20, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x44f, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x20,
0x422, 0x430, 0x439, 0x431, 0x44d, 0x44f, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x44f, 0x427, 0x430,
0x441, 0x20, 0x422, 0x430, 0x434, 0x436, 0x44b, 0x43a, 0x456, 0x441, 0x442, 0x430,
0x43d, 0x430, 0x427, 0x430, 0x441, 0x20, 0x422, 0x430, 0x43a, 0x435, 0x43b, 0x430,
0x443, 0x427, 0x430, 0x441, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441,
0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20,
0x447, 0x430, 0x441, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x427, 0x430, 0x441,
0x20, 0x422, 0x440, 0x443, 0x43a, 0x430, 0x427, 0x430, 0x441, 0x20, 0x422, 0x443,
0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441, 0x442,
0x430, 0x43d, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441, 0x442, 0x430,
0x43d, 0x430, 0x427, 0x430, 0x441, 0x20, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x456, 0x20, 0x447,
0x430, 0x441, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x456,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20,
0x447, 0x430, 0x441, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a,
0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427,
0x430, 0x441, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x456, 0x441, 0x442, 0x430,
0x43d, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b,
0x20, 0x447, 0x430, 0x441, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x456, 0x441,
0x442, 0x430, 0x43d, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430,
0x441, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d,
0x430, 0x427, 0x430, 0x441, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447,
0x430, 0x441, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x41b, 0x435,
0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x20, 0x412, 0x430, 0x43d, 0x443,
0x430, 0x442, 0x443, 0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x44c,
0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x423, 0x43b, 0x430, 0x434, 0x437,
0x456, 0x432, 0x430, 0x441, 0x442, 0x43e, 0x446, 0x43a, 0x456, 0x20, 0x447, 0x430,
0x441, 0x423, 0x43b, 0x430, 0x434, 0x437, 0x456, 0x432, 0x430, 0x441, 0x442, 0x43e,
0x446, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x423, 0x43b, 0x430, 0x434, 0x437, 0x456,
0x432, 0x430, 0x441, 0x442, 0x43e, 0x446, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x412, 0x430, 0x43b, 0x433, 0x430, 0x433,
0x440, 0x430, 0x434, 0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x412, 0x430,
0x43b, 0x433, 0x430, 0x433, 0x440, 0x430, 0x434, 0x441, 0x43a, 0x456, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x412, 0x430, 0x43b, 0x433, 0x430, 0x433, 0x440, 0x430, 0x434, 0x441, 0x43a,
0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441, 0x427,
0x430, 0x441, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x44b, 0x456, 0x20, 0x412,
0x430, 0x441, 0x442, 0x43e, 0x43a, 0x427, 0x430, 0x441, 0x20, 0x432, 0x43e, 0x441,
0x442, 0x440, 0x430, 0x432, 0x430, 0x20, 0x423, 0x44d, 0x439, 0x43a, 0x427, 0x430,
0x441, 0x20, 0x430, 0x441, 0x442, 0x440, 0x430, 0x432, 0x43e, 0x45e, 0x20, 0x423,
0x43e, 0x43b, 0x456, 0x441, 0x20, 0x456, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d,
0x430, 0x42f, 0x43a, 0x443, 0x446, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x42f,
0x43a, 0x443, 0x446, 0x43a, 0x456, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430, 0x441, 0x42f, 0x43a, 0x443, 0x446,
0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447, 0x430, 0x441,
0x415, 0x43a, 0x430, 0x446, 0x44f, 0x440, 0x44b, 0x43d, 0x431, 0x443, 0x440, 0x433,
0x441, 0x43a, 0x456, 0x20, 0x447, 0x430, 0x441, 0x415, 0x43a, 0x430, 0x446, 0x44f,
0x440, 0x44b, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x441, 0x43a, 0x456, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x44b, 0x20, 0x447, 0x430,
0x441, 0x415, 0x43a, 0x430, 0x446, 0x44f, 0x440, 0x44b, 0x43d, 0x431, 0x443, 0x440,
0x433, 0x441, 0x43a, 0x456, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x456, 0x20, 0x447,
0x430, 0x441, 0x427, 0x430, 0x441, 0x20, 0x42e, 0x43a, 0x430, 0x43d, 0x430, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x940, 0x928, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x90f, 0x915, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x906,
0x915, 0x930, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x906, 0x915, 0x930, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x906, 0x92b, 0x917, 0x93e, 0x928, 0x940, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92e, 0x93f, 0x930, 0x941, 0x20,
0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x938, 0x93e,
0x928, 0x91c, 0x93e, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20,
0x938, 0x92e, 0x916, 0x94b, 0x932, 0x93e, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f,
0x915, 0x93e, 0x928, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x906, 0x92b, 0x94d,
0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928,
0x93e, 0x92c, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938,
0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e,
0x928, 0x93f, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938,
0x92e, 0x906, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x938,
0x92e, 0x906, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x938, 0x94d, 0x91f,
0x947, 0x928, 0x94d, 0x921, 0x93e, 0x930, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x906,
0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x938, 0x93e, 0x928,
0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905, 0x932, 0x92e, 0x93e, 0x91f, 0x940,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x905, 0x932, 0x92e, 0x93e, 0x91f, 0x940, 0x20,
0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x905, 0x932, 0x92e, 0x93e, 0x91f, 0x940, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x906, 0x92e, 0x93e, 0x91c, 0x928, 0x20, 0x938,
0x92e, 0x906, 0x92e, 0x93e, 0x91c, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x931, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x92e, 0x93e, 0x91c, 0x928, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939,
0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20,
0x92e, 0x93f, 0x930, 0x941, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20,
0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x92e, 0x93f,
0x930, 0x941, 0x20, 0x92e, 0x93e, 0x928, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947,
0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20, 0x92e, 0x93f, 0x930, 0x941, 0x20,
0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939,
0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20,
0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e,
0x928, 0x93f, 0x20, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938,
0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928,
0x93f, 0x20, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x939,
0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f, 0x20,
0x939, 0x93e, 0x91c, 0x94b, 0x92e, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x93e, 0x928, 0x93f, 0x20, 0x939, 0x93e, 0x91c, 0x94b, 0x92e, 0x93e, 0x92f, 0x93e,
0x930, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x94b, 0x930, 0x93f, 0x915,
0x93e, 0x928, 0x93f, 0x20, 0x939, 0x93e, 0x91c, 0x94b, 0x92e, 0x93e, 0x92f, 0x93e,
0x930, 0x93f, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e,
0x928, 0x93f, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938,
0x92e, 0x938, 0x93e, 0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x93e, 0x928, 0x93f, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e,
0x939, 0x93e, 0x20, 0x906, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x928, 0x93f,
0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938, 0x93e, 0x928,
0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x928, 0x93e, 0x926, 0x940, 0x930,
0x20, 0x938, 0x92e, 0x905, 0x928, 0x93e, 0x926, 0x940, 0x930, 0x20, 0x938, 0x94d,
0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e,
0x905, 0x928, 0x93e, 0x926, 0x940, 0x930, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x908, 0x92e, 0x906, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x906,
0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x906, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x93e, 0x928,
0x20, 0x938, 0x92e, 0x905, 0x915, 0x94d, 0x91f, 0x93e, 0x90a, 0x20, 0x938, 0x92e,
0x905, 0x915, 0x93c, 0x94d, 0x91f, 0x93e, 0x90a, 0x20, 0x938, 0x94d, 0x91f, 0x948,
0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x905, 0x915,
0x93c, 0x94d, 0x91f, 0x93e, 0x90a, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x906, 0x915, 0x94d, 0x924, 0x2019, 0x92c, 0x20, 0x938, 0x92e, 0x905,
0x915, 0x93c, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x20, 0x938, 0x94d, 0x91f, 0x948,
0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x905, 0x915,
0x93c, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x908, 0x92e, 0x906, 0x930, 0x93e, 0x92c, 0x940, 0x92f, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x906, 0x930, 0x93e, 0x92c, 0x940, 0x92f, 0x93e, 0x928, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x930, 0x93e,
0x92c, 0x940, 0x92f, 0x93e, 0x928, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d, 0x91f, 0x93f,
0x928, 0x93e, 0x20, 0x938, 0x92e, 0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d,
0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d, 0x91f, 0x93f,
0x928, 0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938,
0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x906, 0x930, 0x94d, 0x91c, 0x947,
0x928, 0x94d, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928,
0x93e, 0x92c, 0x20, 0x906, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x94d, 0x91f, 0x93f,
0x928, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x906, 0x930, 0x94d, 0x91c, 0x947,
0x928, 0x94d, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x906, 0x91f, 0x932,
0x93e, 0x928, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x906, 0x91f, 0x932,
0x93e, 0x928, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x91f, 0x932, 0x93e, 0x928, 0x94d, 0x91f,
0x93f, 0x915, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x92e, 0x93f, 0x930, 0x941, 0x20, 0x905, 0x2019, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x905, 0x2019, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f, 0x20, 0x92e,
0x93f, 0x930, 0x941, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x905, 0x2019, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x20, 0x92e, 0x93f, 0x930, 0x941, 0x20, 0x938, 0x93e, 0x928, 0x93e,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f, 0x20, 0x92e, 0x93f, 0x930, 0x941, 0x20,
0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f,
0x20, 0x92e, 0x93f, 0x930, 0x941, 0x20, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x93e,
0x930, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e,
0x928, 0x93f, 0x20, 0x92e, 0x93f, 0x930, 0x941, 0x20, 0x938, 0x94b, 0x928, 0x93e,
0x92c, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20,
0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x928, 0x93f, 0x20, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930,
0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x928,
0x93f, 0x20, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928,
0x93e, 0x92c, 0x93e, 0x930, 0x93f, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x905, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f, 0x20, 0x938, 0x94b,
0x928, 0x93e, 0x92c, 0x93e, 0x930, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x928, 0x93f, 0x20, 0x938, 0x94b, 0x928, 0x93e, 0x92c,
0x93e, 0x930, 0x93f, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x906, 0x91c, 0x93e, 0x930, 0x92c, 0x93e, 0x908, 0x91c, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x906, 0x91c, 0x93e, 0x930, 0x92c, 0x93e, 0x908, 0x91c, 0x93e, 0x928,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x906,
0x91c, 0x93e, 0x930, 0x92c, 0x93e, 0x908, 0x91c, 0x93e, 0x928, 0x20, 0x926, 0x948,
0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x906, 0x91c, 0x2019, 0x930,
0x94d, 0x938, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x906, 0x91c, 0x2019, 0x930, 0x94d,
0x938, 0x928, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x906, 0x91c, 0x2019, 0x930, 0x94d, 0x938, 0x928, 0x93f, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92c, 0x93e, 0x902,
0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x938, 0x92e, 0x92c, 0x93e, 0x902, 0x932,
0x93e, 0x926, 0x947, 0x936, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x92c, 0x93e, 0x902, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20,
0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92d, 0x941,
0x91f, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92c, 0x932, 0x93f, 0x92d, 0x93f, 0x92f,
0x93e, 0x20, 0x938, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932,
0x93f, 0x92f, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92c,
0x94d, 0x930, 0x941, 0x928, 0x947, 0x908, 0x20, 0x926, 0x930, 0x20, 0x909, 0x938,
0x20, 0x938, 0x932, 0x93e, 0x92e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921,
0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x915, 0x947, 0x92a, 0x20,
0x92d, 0x947, 0x930, 0x94d, 0x926, 0x947, 0x20, 0x938, 0x92e, 0x915, 0x947, 0x92a,
0x20, 0x92d, 0x947, 0x930, 0x94d, 0x926, 0x947, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x92d, 0x947,
0x930, 0x94d, 0x926, 0x947, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902,
0x20, 0x938, 0x92e, 0x91a, 0x93e, 0x92e, 0x2019, 0x930, 0x94d, 0x930, 0x2019, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91a, 0x947,
0x925, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x91a, 0x947, 0x925, 0x93e, 0x92e, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91a, 0x947,
0x925, 0x93e, 0x92e, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x91a, 0x940, 0x932, 0x947, 0x20, 0x938, 0x92e, 0x91a, 0x940, 0x932, 0x947,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91a,
0x940, 0x932, 0x947, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20,
0x938, 0x92e, 0x91a, 0x93e, 0x908, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x91a, 0x93e,
0x908, 0x928, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x91a, 0x93e, 0x908, 0x928, 0x93e, 0x20, 0x938, 0x93e, 0x928, 0x93e,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91a, 0x2019, 0x907, 0x92c, 0x93e, 0x932, 0x938,
0x93e, 0x928, 0x20, 0x938, 0x92e, 0x91a, 0x2019, 0x907, 0x92c, 0x93e, 0x932, 0x938,
0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x91a, 0x2019, 0x907, 0x92c, 0x93e, 0x932, 0x938, 0x93e, 0x928, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x916, 0x94d, 0x930,
0x940, 0x938, 0x94d, 0x91f, 0x92e, 0x93e, 0x938, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x20, 0x938, 0x92e, 0x915, 0x915, 0x2019, 0x938, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x932,
0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x915, 0x932, 0x92e,
0x94d, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x915, 0x932, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x915,
0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928,
0x93f, 0x20, 0x938, 0x92e, 0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x916, 0x93e, 0x935, 0x938,
0x947, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x915, 0x93f, 0x909, 0x92c, 0x93e, 0x20, 0x938, 0x92e, 0x915, 0x93f, 0x909, 0x92c,
0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x915, 0x93f, 0x909, 0x92c, 0x93e, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x921, 0x947, 0x92d, 0x93f, 0x938, 0x20, 0x938, 0x92e, 0x921,
0x941, 0x92e, 0x928, 0x94d, 0x924, 0x2d, 0x921, 0x940, 0x906, 0x930, 0x94d, 0x92d,
0x940, 0x932, 0x20, 0x938, 0x92e, 0x908, 0x938, 0x94d, 0x91f, 0x20, 0x91f, 0x940,
0x92e, 0x94b, 0x930, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d,
0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f,
0x93e, 0x930, 0x93f, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e,
0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x926, 0x94d,
0x935, 0x940, 0x92a, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20,
0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x20, 0x938, 0x92e, 0x90f, 0x915, 0x941, 0x935, 0x93e, 0x921, 0x930, 0x20,
0x938, 0x92e, 0x92e, 0x93f, 0x930, 0x941, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a,
0x928, 0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x93f, 0x930, 0x941, 0x20, 0x92f, 0x941,
0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x93f, 0x930, 0x941, 0x20, 0x92f, 0x941, 0x930,
0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902,
0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f,
0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x938,
0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x92f, 0x941, 0x930,
0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930, 0x93f,
0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x926, 0x948, 0x91c,
0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x917, 0x94b, 0x91c, 0x93e, 0x928,
0x938, 0x93f, 0x928, 0x2d, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x92f, 0x93e, 0x930,
0x93f, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x938, 0x92e,
0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x92f, 0x941, 0x930, 0x2019, 0x92a, 0x928,
0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x92f, 0x941,
0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x92f, 0x941,
0x930, 0x2019, 0x92a, 0x928, 0x93f, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x20, 0x938, 0x92e, 0x92b, 0x915, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20,
0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938,
0x92e, 0x92b, 0x915, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92b, 0x915, 0x932, 0x947, 0x923, 0x94d,
0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92b,
0x940, 0x91c, 0x940, 0x20, 0x938, 0x92e, 0x92b, 0x940, 0x91c, 0x940, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92b, 0x940, 0x91c,
0x940, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x92b, 0x94d, 0x930, 0x947, 0x928, 0x94d, 0x91a, 0x20, 0x917, 0x941, 0x908, 0x92f,
0x93e, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92b, 0x94d, 0x930, 0x947, 0x928, 0x94d,
0x91a, 0x20, 0x916, 0x94b, 0x932, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x906,
0x930, 0x94b, 0x20, 0x90f, 0x928, 0x94d, 0x91f, 0x93e, 0x930, 0x94d, 0x915, 0x91f,
0x93f, 0x915, 0x20, 0x938, 0x92e, 0x917, 0x93e, 0x932, 0x93e, 0x92a, 0x93e, 0x917,
0x2019, 0x938, 0x20, 0x938, 0x92e, 0x917, 0x93e, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f,
0x947, 0x930, 0x20, 0x938, 0x92e, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e,
0x20, 0x938, 0x92e, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91c, 0x930, 0x94d,
0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902,
0x20, 0x938, 0x92e, 0x917, 0x940, 0x932, 0x92c, 0x93e, 0x930, 0x94d, 0x91f, 0x20,
0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938,
0x92e, 0x917, 0x94d, 0x930, 0x93f, 0x928, 0x935, 0x940, 0x91a, 0x20, 0x917, 0x947,
0x91c, 0x947, 0x930, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928,
0x91c, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x923, 0x94d,
0x921, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x91c, 0x93e, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x928, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x928, 0x921, 0x91c,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x923, 0x94d, 0x921,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x938,
0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947,
0x923, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x947, 0x928,
0x93e, 0x92c, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x947, 0x923, 0x94d,
0x921, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x917, 0x941, 0x906, 0x92e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930,
0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x917, 0x93e, 0x932, 0x94d, 0x92b,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x917,
0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x939, 0x93e, 0x935, 0x93e,
0x908, 0x2d, 0x90f, 0x932, 0x94d, 0x92f, 0x941, 0x924, 0x93f, 0x92f, 0x93e, 0x928,
0x20, 0x938, 0x92e, 0x939, 0x93e, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x94d,
0x92f, 0x941, 0x924, 0x93f, 0x92f, 0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x939, 0x93e, 0x935, 0x93e, 0x908, 0x2d,
0x90f, 0x932, 0x94d, 0x92f, 0x941, 0x924, 0x93f, 0x92f, 0x93e, 0x928, 0x20, 0x938,
0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x939, 0x902, 0x915, 0x902,
0x20, 0x938, 0x92e, 0x939, 0x902, 0x915, 0x902, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x939, 0x902, 0x915, 0x902, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x939, 0x2019, 0x92d,
0x94d, 0x926, 0x20, 0x938, 0x92e, 0x939, 0x2019, 0x92d, 0x94d, 0x926, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x939, 0x2019, 0x92d,
0x94d, 0x926, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938,
0x92e, 0x92d, 0x93e, 0x930, 0x924, 0x93e, 0x930, 0x93f, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92d, 0x93e, 0x930, 0x924, 0x20,
0x932, 0x948, 0x925, 0x94b, 0x92e, 0x93e, 0x92f, 0x93e, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x908, 0x902, 0x921, 0x94b, 0x20, 0x91a, 0x907, 0x928, 0x93e, 0x20, 0x938,
0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908,
0x92e, 0x908, 0x902, 0x921, 0x94b, 0x928, 0x940, 0x936, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x94d, 0x928, 0x20, 0x908, 0x902,
0x921, 0x94b, 0x928, 0x940, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x94d, 0x91f,
0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x935,
0x947, 0x938, 0x94d, 0x91f, 0x930, 0x94d, 0x928, 0x20, 0x908, 0x902, 0x921, 0x94b,
0x928, 0x940, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902,
0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x907, 0x930, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x907, 0x930, 0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x907, 0x930, 0x93e, 0x928, 0x20,
0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x908, 0x930, 0x94d,
0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x908, 0x930,
0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x908, 0x930, 0x94d, 0x915, 0x941,
0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x902, 0x20, 0x938, 0x92e, 0x907, 0x91c, 0x94d, 0x930, 0x93e, 0x908, 0x932,
0x20, 0x938, 0x92e, 0x907, 0x91c, 0x94d, 0x930, 0x93e, 0x908, 0x932, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x907, 0x91c, 0x94d,
0x930, 0x93e, 0x908, 0x932, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x91c, 0x93e,
0x92a, 0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x938, 0x93e, 0x928, 0x93e,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92a, 0x947, 0x924, 0x94d, 0x930, 0x2019, 0x92a,
0x93e, 0x92d, 0x94d, 0x932, 0x2019, 0x92d, 0x94d, 0x938, 0x94d, 0x915, 0x2d, 0x915,
0x93e, 0x92e, 0x91a, 0x93e, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x93f, 0x20, 0x938,
0x92e, 0x92a, 0x947, 0x924, 0x94d, 0x930, 0x94b, 0x92a, 0x93e, 0x935, 0x932, 0x94b,
0x938, 0x94d, 0x915, 0x20, 0x915, 0x93e, 0x92e, 0x91a, 0x91f, 0x915, 0x93e, 0x20,
0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e,
0x908, 0x92e, 0x92a, 0x947, 0x924, 0x94d, 0x930, 0x94b, 0x92a, 0x93e, 0x935, 0x932,
0x94b, 0x938, 0x94d, 0x915, 0x20, 0x915, 0x93e, 0x92e, 0x91a, 0x91f, 0x915, 0x93e,
0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x938, 0x93e, 0x928,
0x91c, 0x93e, 0x20, 0x915, 0x93e, 0x91c, 0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x938, 0x94b, 0x928, 0x93e, 0x92c, 0x20, 0x915, 0x93e,
0x91c, 0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x915,
0x2019, 0x930, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x915, 0x2019, 0x930, 0x93f,
0x92f, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x915, 0x2019, 0x930, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x93e, 0x928, 0x93e,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x938, 0x94d, 0x930, 0x947, 0x20, 0x938,
0x92e, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x2019, 0x92f, 0x93e, 0x930,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x915, 0x94d, 0x930, 0x93e, 0x938,
0x94d, 0x928, 0x2019, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x915, 0x94d, 0x930,
0x93e, 0x938, 0x94d, 0x928, 0x2019, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x902, 0x20, 0x938, 0x92e,
0x915, 0x940, 0x930, 0x94d, 0x917, 0x940, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x932, 0x902, 0x915, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902,
0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x932, 0x93e, 0x908,
0x928, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f,
0x20, 0x938, 0x92e, 0x932, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935, 0x93f,
0x20, 0x938, 0x92e, 0x932, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935, 0x93f,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x932,
0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935, 0x93f, 0x20, 0x938, 0x93e, 0x928,
0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x93e, 0x915, 0x93e, 0x90a, 0x20,
0x938, 0x92e, 0x92e, 0x93e, 0x915, 0x93e, 0x90a, 0x20, 0x938, 0x94d, 0x91f, 0x948,
0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x92e, 0x93e,
0x915, 0x93e, 0x90a, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x908, 0x92e,
0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92e, 0x93e,
0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20,
0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92e, 0x932,
0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921,
0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x92e, 0x93e, 0x932, 0x926,
0x940, 0x92d, 0x94d, 0x938, 0x20, 0x938, 0x92e, 0x92e, 0x93e, 0x930, 0x94d, 0x915,
0x947, 0x938, 0x93e, 0x938, 0x20, 0x938, 0x92e, 0x92e, 0x93e, 0x930, 0x94d, 0x936,
0x947, 0x932, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928,
0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x2019, 0x930, 0x93f, 0x936, 0x93f, 0x92f, 0x93e,
0x938, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x2019, 0x930, 0x93f, 0x936, 0x93f,
0x92f, 0x93e, 0x938, 0x928, 0x93f, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x2019, 0x930, 0x93f, 0x936, 0x93f, 0x92f, 0x93e,
0x938, 0x928, 0x93f, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20,
0x938, 0x92e, 0x92e, 0x94c, 0x938, 0x928, 0x20, 0x938, 0x92e, 0x92e, 0x947, 0x915,
0x94d, 0x938, 0x93f, 0x915, 0x2019, 0x928, 0x93f, 0x20, 0x92a, 0x947, 0x938, 0x93f,
0x92b, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f,
0x915, 0x2019, 0x928, 0x93f, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92e,
0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x2019, 0x928, 0x93f, 0x20, 0x92a, 0x947,
0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x93e, 0x924, 0x93e, 0x930,
0x20, 0x938, 0x92e, 0x909, 0x932, 0x93e, 0x928, 0x92c, 0x93e, 0x924, 0x93e, 0x930,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x909,
0x932, 0x93e, 0x928, 0x92c, 0x93e, 0x924, 0x93e, 0x930, 0x20, 0x926, 0x948, 0x91c,
0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92e, 0x938, 0x94d, 0x915, 0x94c,
0x20, 0x938, 0x92e, 0x92e, 0x938, 0x94d, 0x915, 0x94c, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92e, 0x938, 0x94d, 0x915, 0x94c,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92e,
0x94d, 0x92f, 0x93e, 0x928, 0x92e, 0x93e, 0x930, 0x20, 0x938, 0x94d, 0x91f, 0x948,
0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x928, 0x93e,
0x90a, 0x930, 0x941, 0x20, 0x938, 0x92e, 0x928, 0x947, 0x92a, 0x93e, 0x932, 0x20,
0x938, 0x92e, 0x917, 0x94b, 0x926, 0x93e, 0x928, 0x20, 0x915, 0x947, 0x932, 0x947,
0x921, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x917, 0x94b, 0x926, 0x93e,
0x928, 0x20, 0x915, 0x947, 0x932, 0x947, 0x921, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x917, 0x94b,
0x926, 0x93e, 0x928, 0x20, 0x915, 0x947, 0x932, 0x947, 0x921, 0x928, 0x93f, 0x92f,
0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x928, 0x93f, 0x909, 0x91c, 0x93f, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x938,
0x92e, 0x928, 0x93f, 0x909, 0x91c, 0x93f, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x928, 0x93f,
0x909, 0x91c, 0x93f, 0x932, 0x947, 0x923, 0x94d, 0x921, 0x20, 0x938, 0x93e, 0x928,
0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x928, 0x93f, 0x909, 0x92b, 0x93e, 0x909,
0x928, 0x94d, 0x926, 0x932, 0x947, 0x928, 0x94d, 0x926, 0x20, 0x938, 0x92e, 0x928,
0x93f, 0x909, 0x92b, 0x93e, 0x909, 0x928, 0x94d, 0x926, 0x932, 0x947, 0x928, 0x94d,
0x926, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x928, 0x93f, 0x909, 0x92b, 0x93e, 0x909, 0x928, 0x94d, 0x926, 0x932, 0x947, 0x928,
0x94d, 0x926, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x928, 0x940, 0x90a, 0x90f, 0x20, 0x938, 0x92e, 0x928, 0x930, 0x92b, 0x2019, 0x915,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x928, 0x930, 0x92b,
0x2019, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x928, 0x930, 0x92b, 0x2019, 0x915,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x92b, 0x947, 0x930, 0x928, 0x93e, 0x928, 0x94d, 0x926,
0x2019, 0x20, 0x926, 0x947, 0x20, 0x928, 0x930, 0x2019, 0x928, 0x94d, 0x939, 0x93e,
0x20, 0x938, 0x92e, 0x92b, 0x947, 0x930, 0x928, 0x93e, 0x928, 0x94d, 0x926, 0x2019,
0x20, 0x926, 0x947, 0x20, 0x928, 0x930, 0x2019, 0x928, 0x94d, 0x939, 0x93e, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92b, 0x947,
0x930, 0x928, 0x93e, 0x928, 0x94d, 0x926, 0x2019, 0x20, 0x926, 0x947, 0x20, 0x928,
0x930, 0x2019, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x928, 0x949, 0x930, 0x94d, 0x925, 0x20, 0x92e,
0x93e, 0x930, 0x93f, 0x906, 0x928, 0x93e, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902,
0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x928, 0x92d, 0x2019,
0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e,
0x928, 0x92d, 0x2019, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x928,
0x92d, 0x2019, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x905, 0x92e,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x905, 0x92e, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x905, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938,
0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926,
0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92a, 0x93e, 0x932,
0x93e, 0x909, 0x20, 0x938, 0x92e, 0x92a, 0x93e, 0x92a, 0x941, 0x906, 0x20, 0x928,
0x93f, 0x909, 0x20, 0x917, 0x93f, 0x928, 0x940, 0x20, 0x938, 0x92e, 0x92a, 0x93e,
0x930, 0x93e, 0x917, 0x941, 0x935, 0x947, 0x20, 0x938, 0x92e, 0x92a, 0x93e, 0x930,
0x93e, 0x917, 0x941, 0x935, 0x947, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x92a, 0x93e, 0x930, 0x93e, 0x917, 0x941, 0x935, 0x947,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92a,
0x947, 0x930, 0x941, 0x20, 0x938, 0x92e, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x925,
0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92a, 0x947, 0x930,
0x941, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e,
0x92b, 0x93f, 0x932, 0x940, 0x92a, 0x940, 0x928, 0x94d, 0x938, 0x20, 0x938, 0x92e,
0x92b, 0x940, 0x932, 0x940, 0x92a, 0x940, 0x928, 0x94d, 0x938, 0x20, 0x938, 0x94d,
0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e,
0x92b, 0x940, 0x932, 0x940, 0x92a, 0x940, 0x928, 0x94d, 0x938, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x92b, 0x928, 0x940, 0x915, 0x94d, 0x938,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20,
0x938, 0x92e, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x947,
0x930, 0x20, 0x906, 0x930, 0x94b, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947,
0x932, 0x928, 0x20, 0x938, 0x92e, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x92a,
0x93f, 0x92f, 0x947, 0x930, 0x20, 0x906, 0x930, 0x94b, 0x20, 0x92e, 0x93f, 0x915,
0x94d, 0x935, 0x947, 0x932, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930,
0x93f, 0x20, 0x938, 0x92e, 0x938, 0x948, 0x928, 0x94d, 0x91f, 0x20, 0x92a, 0x93f,
0x92f, 0x947, 0x930, 0x20, 0x906, 0x930, 0x94b, 0x20, 0x92e, 0x93f, 0x915, 0x94d,
0x935, 0x947, 0x932, 0x928, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x92a, 0x940, 0x91f, 0x915, 0x948, 0x930, 0x94d, 0x928, 0x20, 0x938,
0x92e, 0x92a, 0x928, 0x93e, 0x92a, 0x947, 0x20, 0x938, 0x92e, 0x92a, 0x94d, 0x92f,
0x902, 0x92f, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x915, 0x93c, 0x940, 0x91c, 0x93c,
0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20, 0x91f, 0x93e, 0x908, 0x92e,
0x915, 0x93c, 0x940, 0x91c, 0x93c, 0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e,
0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f,
0x93e, 0x908, 0x92e, 0x915, 0x93c, 0x940, 0x91c, 0x93c, 0x940, 0x932, 0x94b, 0x930,
0x94d, 0x921, 0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x908, 0x92e,
0x930, 0x93f, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x928, 0x20, 0x938, 0x92e, 0x930,
0x925, 0x947, 0x930, 0x93e, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x916, 0x93e, 0x932,
0x93f, 0x928, 0x20, 0x938, 0x92e, 0x938, 0x93e, 0x916, 0x93e, 0x932, 0x93f, 0x928,
0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938,
0x93e, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932,
0x93e, 0x902, 0x20, 0x938, 0x92e, 0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x20, 0x91f,
0x93e, 0x908, 0x92e, 0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x20, 0x938, 0x94d, 0x91f,
0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x908, 0x92e, 0x938,
0x92e, 0x93e, 0x930, 0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x908,
0x92e, 0x938, 0x93e, 0x92e, 0x2019, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x938, 0x93e,
0x92e, 0x2019, 0x935, 0x93e, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f,
0x20, 0x938, 0x92e, 0x938, 0x93e, 0x92e, 0x2019, 0x935, 0x93e, 0x20, 0x938, 0x93e,
0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x938, 0x947, 0x936, 0x947, 0x932,
0x94d, 0x938, 0x20, 0x938, 0x92e, 0x938, 0x940, 0x902, 0x917, 0x93e, 0x92a, 0x941,
0x930, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x902, 0x921, 0x930, 0x94d, 0x921, 0x20,
0x91f, 0x93e, 0x908, 0x92e, 0x938, 0x932, 0x2019, 0x92e, 0x928, 0x20, 0x926, 0x94d,
0x935, 0x940, 0x92a, 0x92b, 0x94b, 0x930, 0x928, 0x93f, 0x20, 0x938, 0x92e, 0x916,
0x94b, 0x932, 0x93e, 0x20, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x938, 0x941, 0x930, 0x940, 0x928, 0x93e, 0x92e, 0x20, 0x938, 0x92e,
0x938, 0x940, 0x905, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x91f, 0x93e, 0x939, 0x93f,
0x91f, 0x940, 0x20, 0x938, 0x92e, 0x924, 0x93e, 0x908, 0x92a, 0x947, 0x907, 0x20,
0x938, 0x92e, 0x924, 0x93e, 0x908, 0x92a, 0x947, 0x907, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x924, 0x93e, 0x908, 0x92a, 0x947,
0x907, 0x20, 0x938, 0x93e, 0x928, 0x93e, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x924,
0x93e, 0x91c, 0x93f, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938,
0x92e, 0x91f, 0x915, 0x947, 0x932, 0x94c, 0x20, 0x938, 0x92e, 0x91f, 0x902, 0x917,
0x93e, 0x20, 0x938, 0x92e, 0x91f, 0x902, 0x917, 0x93e, 0x20, 0x925, 0x93e, 0x916,
0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x91f, 0x902, 0x917, 0x93e, 0x20,
0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x91a, 0x942,
0x915, 0x20, 0x938, 0x92e, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928,
0x940, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x924, 0x941, 0x930,
0x94d, 0x915, 0x92e, 0x947, 0x928, 0x940, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x924, 0x941,
0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x940, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x924,
0x941, 0x92d, 0x93e, 0x932, 0x941, 0x20, 0x938, 0x92e, 0x909, 0x930, 0x941, 0x917,
0x941, 0x935, 0x947, 0x20, 0x938, 0x92e, 0x909, 0x930, 0x941, 0x917, 0x941, 0x935,
0x947, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e,
0x909, 0x930, 0x941, 0x917, 0x941, 0x935, 0x947, 0x20, 0x926, 0x948, 0x91c, 0x94d,
0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x909, 0x91c, 0x92c, 0x947, 0x915, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x909, 0x91c, 0x92c, 0x947,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x909, 0x91c, 0x92c, 0x947, 0x915, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x20, 0x938, 0x92e, 0x935, 0x93e, 0x928, 0x941, 0x906, 0x91f, 0x941, 0x20,
0x938, 0x92e, 0x935, 0x93e, 0x928, 0x941, 0x906, 0x91f, 0x941, 0x20, 0x925, 0x93e,
0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x935, 0x93e, 0x928, 0x941,
0x906, 0x91f, 0x941, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20,
0x938, 0x92e, 0x92d, 0x947, 0x928, 0x947, 0x91c, 0x941, 0x935, 0x947, 0x932, 0x93e,
0x20, 0x938, 0x92e, 0x92d, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92d, 0x938, 0x94d,
0x924, 0x2019, 0x915, 0x20, 0x938, 0x92e, 0x92d, 0x94d, 0x932, 0x93e, 0x926, 0x93f,
0x92d, 0x938, 0x94d, 0x924, 0x2019, 0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906,
0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92d, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92d,
0x938, 0x94d, 0x924, 0x2019, 0x915, 0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e,
0x902, 0x20, 0x938, 0x92e, 0x92d, 0x932, 0x94d, 0x917, 0x2019, 0x917, 0x94d, 0x930,
0x947, 0x926, 0x20, 0x938, 0x92e, 0x92d, 0x932, 0x94d, 0x917, 0x2019, 0x917, 0x94d,
0x930, 0x947, 0x926, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x92d, 0x932, 0x94d, 0x917, 0x2019, 0x917, 0x94d, 0x930, 0x947, 0x926,
0x20, 0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x935,
0x948, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x935,
0x93e, 0x932, 0x940, 0x938, 0x20, 0x906, 0x930, 0x94b, 0x20, 0x92b, 0x941, 0x924,
0x941, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x925, 0x93e, 0x916, 0x94b, 0x906, 0x930, 0x93f, 0x20,
0x938, 0x92e, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x926, 0x948, 0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92f, 0x947,
0x915, 0x93e, 0x924, 0x947, 0x930, 0x940, 0x928, 0x93e, 0x92c, 0x941, 0x930, 0x94d,
0x917, 0x20, 0x938, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x940,
0x928, 0x93e, 0x92c, 0x941, 0x930, 0x94d, 0x917, 0x20, 0x925, 0x93e, 0x916, 0x94b,
0x906, 0x930, 0x93f, 0x20, 0x938, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947,
0x930, 0x940, 0x928, 0x93e, 0x92c, 0x941, 0x930, 0x94d, 0x917, 0x20, 0x926, 0x948,
0x91c, 0x94d, 0x932, 0x93e, 0x902, 0x20, 0x938, 0x92e, 0x92f, 0x942, 0x915, 0x928,
0x20, 0x938, 0x92e, 0x41, 0x63, 0x72, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x63, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x63,
0x72, 0x65, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61,
0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x6e, 0x61, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x61, 0x66, 0x72,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x75,
0x17e, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x66,
0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x75, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d,
0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x6e, 0x61, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65,
0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72,
0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d,
0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c,
0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c, 0x61, 0x6e,
0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c, 0x61, 0x6e, 0x69, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69,
0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41,
0x70, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x70, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x6b, 0x76, 0x74, 0x61, 0x75, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x41, 0x6b, 0x76, 0x74, 0x61, 0x75, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x6b, 0x76, 0x74, 0x61, 0x75, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x6b, 0x76, 0x74,
0x6f, 0x62, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b, 0x76,
0x74, 0x6f, 0x62, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b, 0x76,
0x74, 0x6f, 0x62, 0x65, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72,
0x61, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53,
0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65,
0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72,
0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x61,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74,
0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x161, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x6f,
0x6c, 0x69, 0x76, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42,
0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65, 0x6e,
0x6f, 0x72, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x10c, 0x61, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x61,
0x74, 0x61, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x10c, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x69, 0x6c,
0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e,
0x65, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x10c, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x6f, 0x6a, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x10c, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x6e, 0x61, 0x20, 0x42, 0x6f, 0x17e, 0x69, 0x107, 0x6e, 0x6f,
0x6d, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x56, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x69, 0x6d, 0x61, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4b,
0x75, 0x6b, 0x6f, 0x76, 0x69, 0x6d, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76,
0x69, 0x6d, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61,
0x20, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x69, 0x6d, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x69, 0x6d, 0x61, 0x50, 0x6f, 0x6c, 0x75, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x6e, 0x61, 0x20, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x69, 0x6d, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x4b, 0x75, 0x62, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b,
0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x63, 0x65, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x56, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63,
0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e,
0x6f, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a,
0x65, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a,
0x65, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x65,
0x6f, 0x73, 0x74, 0x72, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x45,
0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x44, 0x61, 0x6c, 0x65, 0x6b, 0x6f, 0x69,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x6e, 0x61, 0x20, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x75, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x69, 0x64, 0x17e,
0x69, 0x6a, 0x75, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x6f,
0x67, 0x76, 0x61, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b,
0x6f, 0x6a, 0x20, 0x4a, 0x75, 0x17e, 0x6e, 0x6f, 0x6a, 0x20, 0x54, 0x65,
0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x6a, 0x69, 0x20, 0x69, 0x20, 0x41,
0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x75, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x6a, 0x65, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47,
0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x6f, 0x76, 0x69, 0x6d, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76,
0x69, 0x6d, 0x61, 0x47, 0x72, 0x69, 0x6e, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d,
0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74,
0x6f, 0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x75, 0x61, 0x6d, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x6c, 0x69, 0x76, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x76, 0x61, 0x6a, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65,
0x75, 0x107, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d,
0x61, 0x6c, 0x65, 0x75, 0x107, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b,
0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x107, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x6d, 0x20, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x75,
0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x72, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x7a, 0x72, 0x61,
0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61,
0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x6e, 0x61, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x6b, 0x61,
0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x72,
0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x4b, 0x6f, 0x73,
0x72, 0x61, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x72,
0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x20,
0x4c, 0x61, 0x6a, 0x6e, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x61, 0x75, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x61, 0x75, 0x4c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x61, 0x75, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x65, 0x7a,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d,
0x61, 0x20, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x56, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x161, 0x61,
0x6c, 0x6f, 0x76, 0x69, 0x6d, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x69,
0x6d, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65, 0x20, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55,
0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61,
0x74, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73,
0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x69, 0x6a, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x7a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61,
0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75,
0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64,
0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74,
0x72, 0x76, 0x75, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x161, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x161,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x6e, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x69, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x6e, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x69, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x4c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x6a, 0x61, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x20, 0x4d, 0x61,
0x72, 0x69, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74,
0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61,
0x20, 0x50, 0x61, 0x70, 0x75, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x6a,
0x20, 0x47, 0x76, 0x69, 0x6e, 0x65, 0x6a, 0x69, 0x50, 0x61, 0x72, 0x61,
0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x72,
0x61, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50,
0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x69, 0x6d, 0x61, 0x20, 0x46, 0x69, 0x6e, 0x69, 0x6b, 0x73, 0x56, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74,
0x72, 0x76, 0x69, 0x6d, 0x61, 0x20, 0x53, 0x76, 0x65, 0x74, 0x69, 0x20,
0x50, 0x65, 0x74, 0x61, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x20, 0x53, 0x76,
0x65, 0x74, 0x69, 0x20, 0x50, 0x65, 0x74, 0x61, 0x72, 0x20, 0x69, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x20, 0x53, 0x76,
0x65, 0x74, 0x69, 0x20, 0x50, 0x65, 0x74, 0x61, 0x72, 0x20, 0x69, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x56, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x69,
0x6d, 0x61, 0x20, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x56, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74,
0x72, 0x76, 0x75, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x50, 0x6a,
0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x7a, 0x69,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x69,
0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x52, 0x65, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x63, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72,
0x61, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x6a, 0x161, 0x65, 0x6c, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x75, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e,
0x61, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x6b, 0x69,
0x6d, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x69, 0x6d, 0x61, 0x4a, 0x75,
0x17e, 0x6e, 0x6f, 0x64, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77,
0x61, 0x54, 0x61, 0x68, 0x69, 0x107, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x10c, 0x75, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55,
0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x72,
0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x56, 0x65, 0x6e, 0x65, 0x63, 0x75, 0x65, 0x6c, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x10d, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x63, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x56, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x73, 0x74, 0x72, 0x76, 0x75, 0x20, 0x56, 0x65, 0x6a, 0x6b, 0x56, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x73, 0x74,
0x72, 0x76, 0x69, 0x6d, 0x61, 0x20, 0x56, 0x61, 0x6c, 0x69, 0x73, 0x20,
0x69, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x75, 0x6b, 0x6f, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x410, 0x43a, 0x440,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43a, 0x440, 0x435, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x43a, 0x440, 0x435, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x432, 0x433, 0x430, 0x43d,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x2d, 0x430,
0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x2d, 0x430, 0x444,
0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x408, 0x443, 0x436, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x459, 0x430,
0x441, 0x43a, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410,
0x459, 0x430, 0x441, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x410, 0x459, 0x430, 0x441, 0x43a, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43b, 0x43c, 0x430,
0x442, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43b, 0x43c, 0x430,
0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43b, 0x43c, 0x430, 0x442,
0x438, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x426, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x41f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x43b, 0x430,
0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430,
0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x43d,
0x430, 0x434, 0x438, 0x440, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d,
0x430, 0x434, 0x438, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430,
0x434, 0x438, 0x440, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x410, 0x43f, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x458, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x458, 0x430, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x43a, 0x432, 0x430, 0x442,
0x430, 0x443, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43a, 0x432, 0x430,
0x442, 0x430, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43a, 0x432, 0x430,
0x442, 0x430, 0x443, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x410, 0x43a, 0x443, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x43a, 0x443, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x43a, 0x443, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x440, 0x430,
0x431, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x43c, 0x435, 0x43d,
0x438, 0x458, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x442, 0x43b,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x437,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441,
0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e,
0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x443, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441,
0x43a, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x410, 0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435,
0x458, 0x45f, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410,
0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x410, 0x437, 0x43e, 0x440,
0x438, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e,
0x440, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e,
0x440, 0x438, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x430,
0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x43e, 0x43b, 0x438, 0x432,
0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411,
0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437,
0x438, 0x43b, 0x438, 0x458, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x458, 0x20, 0x414,
0x430, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417,
0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440,
0x442, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x427, 0x438,
0x43b, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x438,
0x43b, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b,
0x435, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x41a, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41a, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a,
0x438, 0x43d, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x427, 0x43e, 0x458, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x43e, 0x458, 0x431, 0x430,
0x43b, 0x441, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427,
0x43e, 0x458, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x411, 0x43e, 0x436, 0x438, 0x45b,
0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x28,
0x41a, 0x435, 0x43b, 0x438, 0x43d, 0x433, 0x29, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e,
0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c,
0x431, 0x438, 0x458, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x43a, 0x43e,
0x432, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x43f, 0x43e,
0x43b, 0x443, 0x2d, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41a, 0x443, 0x431, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x414, 0x435, 0x458, 0x432, 0x438, 0x441, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x414, 0x438, 0x43c, 0x43e, 0x43d, 0x20, 0x434,
0x2019, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x2d, 0x41b, 0x435, 0x441, 0x442,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a,
0x440, 0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440, 0x448,
0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440, 0x448, 0x45a, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x415, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434,
0x45a, 0x435, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x435,
0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x435, 0x435, 0x432, 0x440,
0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435,
0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432,
0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e,
0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x414, 0x430, 0x459, 0x435, 0x20, 0x438, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435,
0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430,
0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a,
0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b,
0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x432, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x424, 0x438, 0x45f, 0x438, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x424, 0x438, 0x45f, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x424, 0x438, 0x45f, 0x438, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x441, 0x43a,
0x430, 0x20, 0x413, 0x432, 0x430, 0x458, 0x430, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x441,
0x43a, 0x43e, 0x20, 0x458, 0x443, 0x436, 0x43d, 0x43e, 0x20, 0x438, 0x20, 0x430,
0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430,
0x433, 0x43e, 0x441, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413,
0x430, 0x43c, 0x431, 0x438, 0x458, 0x435, 0x440, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x458, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x458,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437,
0x438, 0x458, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x413, 0x440, 0x438, 0x43d, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441, 0x440, 0x435,
0x434, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x413, 0x443, 0x430, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x417, 0x430, 0x43b, 0x438, 0x432, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458, 0x441, 0x43a, 0x43e,
0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458, 0x441, 0x43a,
0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458, 0x441, 0x43a,
0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x425, 0x43e, 0x43d,
0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x433, 0x43a, 0x43e,
0x43d, 0x448, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x43d, 0x434, 0x438,
0x458, 0x441, 0x43a, 0x43e, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x43d, 0x434,
0x43e, 0x43a, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x2d, 0x438,
0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x2d, 0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x438, 0x43d, 0x434, 0x43e, 0x43d,
0x435, 0x437, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443, 0x446,
0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443,
0x446, 0x43a, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x437,
0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x408, 0x430, 0x43f, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430,
0x43f, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x408, 0x430, 0x43f, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41f, 0x435, 0x442, 0x440,
0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x43e, 0x2d, 0x43a,
0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e,
0x432, 0x441, 0x43a, 0x43e, 0x2d, 0x43a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x442, 0x440, 0x43e,
0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x43e, 0x2d, 0x43a, 0x430,
0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e,
0x2d, 0x43a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x43a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41a, 0x43e, 0x440, 0x435, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435,
0x458, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x41a, 0x43e, 0x448, 0x440, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430,
0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x458, 0x430, 0x440, 0x441, 0x43a, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x428, 0x440, 0x438,
0x20, 0x41b, 0x430, 0x43d, 0x43a, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41b, 0x438, 0x43d, 0x435, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20,
0x425, 0x43e, 0x432, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41c, 0x430, 0x43a,
0x430, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43a, 0x430,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41c, 0x430, 0x43b, 0x435, 0x437,
0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c,
0x430, 0x43b, 0x434, 0x438, 0x432, 0x438, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x441, 0x43a,
0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440, 0x438, 0x446, 0x438, 0x458,
0x443, 0x441, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430,
0x443, 0x440, 0x438, 0x446, 0x438, 0x458, 0x443, 0x441, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440, 0x438, 0x446, 0x438, 0x458, 0x443,
0x441, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x41c, 0x43e, 0x441, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x430,
0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x423, 0x43b, 0x430,
0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e,
0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d,
0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x432,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x441, 0x43a,
0x432, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x41c, 0x438, 0x458, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x430, 0x443, 0x440, 0x443, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430, 0x20,
0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417,
0x435, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x438, 0x20,
0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d,
0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x41d, 0x438, 0x443, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x432, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432,
0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20,
0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434,
0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20,
0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x430, 0x20, 0x41c, 0x430, 0x440, 0x438, 0x458, 0x430, 0x43d, 0x441,
0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431,
0x438, 0x440, 0x441, 0x43a, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41f, 0x430, 0x43b, 0x430, 0x443,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43f, 0x443,
0x430, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435,
0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430,
0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430,
0x458, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443,
0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430,
0x41f, 0x435, 0x440, 0x443, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41f, 0x435, 0x440, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f,
0x435, 0x440, 0x443, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438,
0x43d, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b,
0x438, 0x43f, 0x438, 0x43d, 0x438, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458, 0x435, 0x440, 0x20, 0x438, 0x20,
0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458, 0x435, 0x440, 0x20,
0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458, 0x435, 0x440,
0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a,
0x435, 0x20, 0x432, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41f, 0x438, 0x442, 0x43a,
0x435, 0x440, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f,
0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458, 0x430, 0x43d, 0x433, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x437, 0x438, 0x43b, 0x43e,
0x440, 0x434, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x437,
0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a,
0x438, 0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x420, 0x435, 0x438, 0x43d, 0x438,
0x43e, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x420, 0x43e,
0x442, 0x435, 0x440, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x421,
0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421,
0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430,
0x43c, 0x430, 0x440, 0x430, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x421, 0x435, 0x458, 0x448,
0x435, 0x43b, 0x438, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421,
0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x441, 0x43a, 0x430,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x408, 0x443, 0x436, 0x43d, 0x430, 0x20, 0x40f, 0x43e, 0x440,
0x45f, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x428, 0x43e, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x422, 0x430, 0x445, 0x438, 0x442, 0x438, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435,
0x458, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f,
0x435, 0x458, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x422, 0x430, 0x45f, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43a, 0x435, 0x43b,
0x430, 0x443, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x43e,
0x43d, 0x433, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422,
0x43e, 0x43d, 0x433, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422,
0x43e, 0x43d, 0x433, 0x430, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440,
0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x43d, 0x430, 0x422, 0x440, 0x443, 0x43a, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422,
0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435,
0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x458, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x440,
0x443, 0x433, 0x432, 0x430, 0x458, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x458, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430,
0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x43d, 0x430, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430,
0x442, 0x443, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447,
0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d,
0x430, 0x412, 0x435, 0x43d, 0x435, 0x446, 0x443, 0x435, 0x43b, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x43d, 0x430, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x43e,
0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x412, 0x43e,
0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x412, 0x435, 0x458, 0x43a, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43b, 0x438, 0x441,
0x20, 0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d,
0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x408,
0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a, 0x430,
0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438,
0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x43d, 0x430, 0x65, 0x75, 0x72, 0x20, 0x41, 0x66, 0x67, 0x68,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x4b,
0x72, 0x65, 0x69, 0x7a, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x65, 0x75,
0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61, 0x72, 0x20,
0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x75, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x65, 0x75, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68,
0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61, 0x72, 0x20,
0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x65, 0x75, 0x72, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x65, 0x75,
0x72, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61,
0x6e, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68,
0x72, 0x65, 0x69, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x72, 0x65,
0x69, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61,
0x72, 0x20, 0x43, 0x2bc, 0x68, 0x72, 0x65, 0x69, 0x7a, 0x65, 0x75, 0x72,
0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20,
0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75,
0x72, 0x20, 0x61, 0x72, 0x20, 0x4d, 0x65, 0x6e, 0x65, 0x7a, 0x69, 0x6f,
0xf9, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x61, 0x72, 0x20, 0x4d, 0x65, 0x6e, 0x65, 0x7a, 0x69, 0x6f, 0xf9,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20,
0x4d, 0x65, 0x6e, 0x65, 0x7a, 0x69, 0x6f, 0xf9, 0x65, 0x75, 0x72, 0x20,
0x61, 0x6e, 0x20, 0x48, 0x61, 0x62, 0x61, 0x73, 0x6b, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20,
0x48, 0x61, 0x62, 0x61, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x62, 0x61, 0x73, 0x6b,
0x65, 0x75, 0x72, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2bc, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x2bc, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2bc, 0x65, 0x75,
0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x70, 0x69, 0x61, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x70, 0x69, 0x61, 0x65,
0x75, 0x72, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x41, 0x72,
0x63, 0x2bc, 0x68, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72, 0x63,
0x2bc, 0x68, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72, 0x63, 0x2bc, 0x68, 0x61, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x65, 0x75, 0x72, 0x20, 0x41, 0x72, 0x63, 0x2bc,
0x68, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43,
0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72, 0x63, 0x2bc, 0x68,
0x61, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc,
0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x41, 0x72, 0x63, 0x2bc, 0x68, 0x61, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e,
0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x65, 0x75, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x65, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61,
0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x65, 0x6c, 0x65, 0x75,
0x72, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x61, 0x6f, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x61, 0x6f, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x61, 0x6f, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x4b, 0x72, 0x65,
0x69, 0x7a, 0x61, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4b,
0x72, 0x65, 0x69, 0x7a, 0x61, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4,
0x67, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x72,
0x65, 0x69, 0x7a, 0x61, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67,
0x65, 0x75, 0x72, 0x20, 0x41, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x6f,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x52,
0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x41, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61,
0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x41,
0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20,
0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x6f, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68,
0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x41, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65,
0x75, 0x72, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a,
0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a, 0x61,
0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x65, 0x75, 0x72,
0x20, 0x61, 0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x7a, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x7a, 0x65, 0x75, 0x72, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x20,
0x42, 0x68, 0x6f, 0x75, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x42,
0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x42, 0x72,
0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c,
0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x42,
0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x42,
0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x65, 0x75, 0x72, 0x20, 0x61, 0x72, 0x20, 0x43,
0x2bc, 0x68, 0x61, 0x62, 0x2d, 0x47, 0x6c, 0x61, 0x73, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20,
0x43, 0x2bc, 0x68, 0x61, 0x62, 0x2d, 0x47, 0x6c, 0x61, 0x73, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc,
0x68, 0x61, 0x62, 0x2d, 0x47, 0x6c, 0x61, 0x73, 0x65, 0x75, 0x72, 0x20,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x65, 0x75, 0x72, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x65, 0x75, 0x72, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x65, 0x75,
0x72, 0x20, 0x53, 0x69, 0x6e, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x69, 0x6e, 0x61, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x69, 0x6e, 0x61, 0x65,
0x75, 0x72, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x45, 0x6e,
0x65, 0x7a, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73,
0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x4b, 0x6f,
0x6b, 0x6f, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a,
0x69, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x65,
0x75, 0x72, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x75, 0x62, 0x61,
0x65, 0x75, 0x72, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x65, 0x75, 0x72,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x65, 0x75, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65,
0x75, 0x72, 0x20, 0x45, 0x6e, 0x65, 0x7a, 0x20, 0x50, 0x61, 0x73, 0x6b,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x45, 0x6e, 0x65, 0x7a, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x6e, 0x65, 0x7a, 0x20, 0x50,
0x61, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64,
0x6f, 0x72, 0x65, 0x75, 0x72, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x65, 0x75, 0x72, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x61,
0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75,
0x72, 0x20, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61,
0x64, 0x65, 0x75, 0x72, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68,
0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x61, 0x72, 0x20,
0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20,
0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x75, 0x72, 0x20, 0x46, 0x69, 0x64, 0x6a, 0x69,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x46, 0x69, 0x64, 0x6a, 0x69, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x46, 0x69, 0x64, 0x6a, 0x69, 0x65, 0x75, 0x72, 0x20, 0x47,
0x77, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x63, 0x2bc, 0x68, 0x61, 0x6c, 0x6c,
0x65, 0x75, 0x72, 0x20, 0x44, 0x6f, 0x75, 0x61, 0x72, 0x6f, 0xf9, 0x20,
0x61, 0x6f, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x46, 0x72, 0x61, 0xf1,
0x73, 0x20, 0x68, 0x61, 0x67, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b,
0x74, 0x69, 0x6b, 0x61, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a,
0x69, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x65,
0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x4a, 0x6f, 0x72, 0x6a,
0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x4a, 0x6f, 0x72, 0x6a, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x4a, 0x6f, 0x72, 0x6a, 0x69, 0x61, 0x65,
0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x41, 0x6d, 0x7a, 0x65, 0x72, 0x20, 0x6b, 0x65,
0x69, 0x74, 0x61, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69,
0x63, 0x68, 0x20, 0x28, 0x41, 0x4b, 0x47, 0x29, 0x65, 0x75, 0x72, 0x20,
0x47, 0x72, 0x65, 0x75, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x61, 0x72,
0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x47, 0x72, 0x65, 0x75, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x47, 0x72, 0x65,
0x75, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65,
0x74, 0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x47, 0x72, 0x65, 0x75, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f,
0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x47, 0x72, 0x65, 0x75, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4,
0x67, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x47, 0x72,
0x65, 0x75, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x61, 0x72, 0x20, 0x43,
0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x47, 0x75, 0x61, 0x6d, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61,
0x72, 0x20, 0x50, 0x6c, 0x65, 0x67, 0x2d, 0x6d, 0x6f, 0x72, 0x20, 0x41,
0x72, 0x61, 0x62, 0x2d, 0x68, 0x61, 0x2d, 0x50, 0x65, 0x72, 0x73, 0x65,
0x75, 0x72, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x65, 0x75, 0x72,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x20, 0x68, 0x61, 0x67, 0x20,
0x61, 0x6e, 0x20, 0x41, 0x6c, 0x65, 0x6f, 0x75, 0x74, 0x65, 0x64, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x20, 0x68, 0x61, 0x67, 0x20, 0x61, 0x6e,
0x20, 0x41, 0x6c, 0x65, 0x6f, 0x75, 0x74, 0x65, 0x64, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x20, 0x68, 0x61, 0x67, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6c, 0x65, 0x6f,
0x75, 0x74, 0x65, 0x64, 0x65, 0x75, 0x72, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x65, 0x75, 0x72, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f,
0x61, 0xf1, 0x76, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x65, 0x75, 0x72, 0x20,
0x68, 0x61, 0xf1, 0x76, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x4d, 0x65, 0x75, 0x72, 0x76, 0x6f,
0x72, 0x20, 0x49, 0x6e, 0x64, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x49,
0x6e, 0x64, 0x65, 0x7a, 0x2d, 0x53, 0x69, 0x6e, 0x61, 0x65, 0x75, 0x72,
0x20, 0x4b, 0x72, 0x65, 0x69, 0x7a, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x7a, 0x69, 0x61, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74,
0x65, 0x72, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x7a, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72,
0x6e, 0xf4, 0x67, 0x65, 0x75, 0x72, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x49, 0x72, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x75, 0x72,
0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65,
0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x65, 0x75, 0x72, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x61, 0x72, 0x20, 0x52, 0x65, 0x74, 0x65, 0x72, 0x65,
0x75, 0x72, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x61, 0x72, 0x20, 0x43, 0x2bc, 0x68, 0x6f, 0x72, 0x6e, 0xf4, 0x67,
0x65, 0x75, 0x72, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x65, 0x75, 0x72, 0x20, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x65, 0x75, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x65, 0x75, 0x72, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x7a, 0x73, 0x74,
0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x53, 0x72, 0x69, 0x20, 0x4c, 0x61,
0x6e, 0x6b, 0x61, 0x65, 0x75, 0x72, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x75, 0x72, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x65,
0x75, 0x72, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x61, 0x63, 0x61,
0x75, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x61,
0x63, 0x61, 0x75, 0x65, 0x75, 0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x65, 0x75, 0x72, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69,
0x61, 0x65, 0x75, 0x72, 0x20, 0x61, 0x72, 0x20, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a,
0x69, 0x20, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x65, 0x75, 0x72, 0x20,
0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x65, 0x75, 0x72, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4d,
0x6f, 0x72, 0x69, 0x73, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x65, 0x75, 0x72, 0x20, 0x4d, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x48,
0x61, 0x62, 0x61, 0x73, 0x6b, 0x20, 0x6d, 0x65, 0x63, 0x2bc, 0x68, 0x69,
0x6b, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x48, 0x61, 0x62, 0x61, 0x73, 0x6b,
0x20, 0x6d, 0x65, 0x63, 0x2bc, 0x68, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x6e, 0x20, 0x48, 0x61,
0x62, 0x61, 0x73, 0x6b, 0x20, 0x6d, 0x65, 0x63, 0x2bc, 0x68, 0x69, 0x6b,
0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72,
0x65, 0x75, 0x72, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4d, 0x6f,
0x73, 0x6b, 0x6f, 0x76, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76,
0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x65, 0x75, 0x72, 0x20, 0x4d,
0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x65, 0x75, 0x72, 0x20, 0x4e, 0x61,
0x75, 0x72, 0x75, 0x65, 0x75, 0x72, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c,
0x65, 0x75, 0x72, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x20, 0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75,
0x72, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x65, 0x76,
0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x65, 0x76,
0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x5a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65,
0x75, 0x72, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61,
0xf1, 0x76, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x75, 0x72, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x65, 0x75, 0x72, 0x20,
0x45, 0x6e, 0x65, 0x7a, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20,
0x45, 0x6e, 0x65, 0x7a, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x45, 0x6e, 0x65,
0x7a, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x75, 0x72,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x65, 0x75, 0x72, 0x20, 0x4d,
0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x4e, 0x6f,
0x72, 0x7a, 0x68, 0x65, 0x75, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x65, 0x75, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75,
0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x65, 0x75, 0x72, 0x20, 0x50, 0x61, 0x70, 0x6f, 0x75, 0x61, 0x2d, 0x47,
0x69, 0x6e, 0x65, 0x61, 0x2d, 0x4e, 0x65, 0x76, 0x65, 0x7a, 0x65, 0x75,
0x72, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x65,
0x75, 0x72, 0x20, 0x50, 0x65, 0x72, 0x6f, 0x75, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x50, 0x65, 0x72, 0x6f,
0x75, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x50, 0x65,
0x72, 0x6f, 0x75, 0x65, 0x75, 0x72, 0x20, 0x61, 0x72, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x61, 0x72, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a,
0x69, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x65, 0x75, 0x72,
0x20, 0x53, 0x61, 0x6e, 0x74, 0x2d, 0x50, 0xea, 0x72, 0x2d, 0x68, 0x61,
0x2d, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20,
0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x61, 0x6e, 0x74,
0x2d, 0x50, 0xea, 0x72, 0x2d, 0x68, 0x61, 0x2d, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x53, 0x61, 0x6e, 0x74, 0x2d, 0x50, 0xea, 0x72, 0x2d, 0x68, 0x61, 0x2d,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x50,
0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x50,
0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x65, 0x75, 0x72, 0x20, 0x50, 0x2bc,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x65, 0x75, 0x72, 0x20,
0x61, 0x72, 0x20, 0x52, 0x65, 0xfc, 0x6e, 0x69, 0x6f, 0x6e, 0x65, 0x75,
0x72, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x65, 0x75, 0x72,
0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x65, 0x75,
0x72, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x65, 0x75, 0x72, 0x20, 0x53, 0x65, 0x63, 0x68, 0x65, 0x6c,
0x65, 0x7a, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1,
0x76, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x75, 0x72, 0x65,
0x75, 0x72, 0x20, 0x49, 0x6e, 0x69, 0x7a, 0x69, 0x20, 0x53, 0x61, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x20, 0x61, 0x72, 0x20, 0x53, 0x75, 0x65, 0x75, 0x72,
0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x75, 0x72, 0x20,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x65, 0x75, 0x72, 0x20, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x65, 0x75, 0x72, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x65, 0x75, 0x72, 0x20, 0x68,
0x61, 0xf1, 0x76, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x65, 0x75,
0x72, 0x20, 0x54, 0x61, 0x64, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x65, 0x75, 0x72, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x65, 0x75, 0x72, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x65, 0x75, 0x72,
0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x65, 0x75, 0x72, 0x20, 0x43, 0x68, 0x75, 0x75,
0x6b, 0x65, 0x75, 0x72, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1,
0x76, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x65, 0x75, 0x72, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75,
0x65, 0x75, 0x72, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x65,
0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61,
0xf1, 0x76, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x65, 0x75,
0x72, 0x20, 0x4f, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76,
0x20, 0x4f, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x4f, 0x75, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x75, 0x72, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x65, 0x75, 0x72, 0x20, 0x63,
0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x65, 0x75, 0x72, 0x20, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x65, 0x75, 0x72, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x75, 0x72,
0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc,
0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x75, 0x72,
0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x57,
0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x75,
0x72, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x68, 0x61, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x65, 0x75, 0x72, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x63, 0x2bc, 0x68,
0x6f, 0x61, 0xf1, 0x76, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x65, 0x75, 0x72, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x65, 0x75,
0x72, 0x20, 0x63, 0x2bc, 0x68, 0x6f, 0x61, 0xf1, 0x76, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67,
0x65, 0x75, 0x72, 0x20, 0x68, 0x61, 0xf1, 0x76, 0x20, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x410,
0x444, 0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x42e, 0x436, 0x43d, 0x43e, 0x430, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438,
0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20,
0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430,
0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43c, 0x430,
0x437, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x437, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x438, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x437,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430,
0x434, 0x438, 0x440, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d,
0x430, 0x434, 0x438, 0x440, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x43f, 0x438, 0x44f, 0x410, 0x43f, 0x438, 0x44f, 0x20, 0x2013, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x44f, 0x20, 0x2013, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x436, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x440, 0x436, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x436, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430, 0x440, 0x436, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430, 0x440, 0x436, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430, 0x440, 0x436, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x440, 0x43c, 0x435, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e,
0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x430, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x20, 0x2013, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x2013, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20,
0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x2013, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x446, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440,
0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434,
0x436, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e, 0x440,
0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x410,
0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x438, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x43e,
0x440, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x411, 0x443, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438, 0x439, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430,
0x437, 0x438, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411,
0x440, 0x430, 0x437, 0x438, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x20, 0x414, 0x430,
0x440, 0x443, 0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x41a, 0x430, 0x431, 0x43e,
0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x41a, 0x430, 0x431, 0x43e, 0x20, 0x412,
0x435, 0x440, 0x434, 0x435, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a,
0x430, 0x431, 0x43e, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x2013, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x44a,
0x43c, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430,
0x442, 0x44a, 0x43c, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427,
0x430, 0x442, 0x44a, 0x43c, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x427, 0x438, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b, 0x438, 0x439, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b, 0x438, 0x439, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x442, 0x430,
0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438,
0x442, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a,
0x438, 0x442, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x43e, 0x439, 0x431,
0x430, 0x43b, 0x441, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x20, 0x420, 0x43e, 0x436, 0x434, 0x435, 0x441, 0x442, 0x432, 0x43e,
0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x438, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x439, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x438, 0x20, 0x41a, 0x443, 0x43a, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x438, 0x20, 0x41a, 0x443, 0x43a, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x41a, 0x443, 0x43a, 0x20,
0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x438,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443,
0x431, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a,
0x443, 0x431, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x414, 0x435, 0x439, 0x432, 0x438, 0x441, 0x414, 0x44e, 0x43c, 0x43e,
0x43d, 0x20, 0x414, 0x44e, 0x440, 0x432, 0x438, 0x43b, 0x418, 0x437, 0x442, 0x43e,
0x447, 0x43d, 0x43e, 0x442, 0x438, 0x43c, 0x43e, 0x440, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x435, 0x43b, 0x438, 0x43a, 0x434, 0x435,
0x43d, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x412,
0x435, 0x43b, 0x438, 0x43a, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x412, 0x435, 0x43b, 0x438, 0x43a, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x415, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x414, 0x430, 0x43b, 0x435, 0x447,
0x43d, 0x43e, 0x20, 0x438, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432,
0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440,
0x43e, 0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e,
0x43f, 0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x435,
0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x424,
0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x438,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x2013, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x434, 0x436, 0x438, 0x439, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x434, 0x436,
0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438,
0x434, 0x436, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x424, 0x440, 0x435, 0x43d, 0x441, 0x43a, 0x430, 0x20, 0x413, 0x432,
0x438, 0x430, 0x43d, 0x430, 0x424, 0x440, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20,
0x44e, 0x436, 0x43d, 0x438, 0x20, 0x438, 0x20, 0x430, 0x43d, 0x442, 0x430, 0x440,
0x43a, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x438, 0x20, 0x442, 0x435, 0x440,
0x438, 0x442, 0x43e, 0x440, 0x438, 0x438, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430,
0x433, 0x43e, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x413,
0x430, 0x43c, 0x431, 0x438, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437,
0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x440,
0x443, 0x437, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20, 0x413, 0x438,
0x43b, 0x431, 0x44a, 0x440, 0x442, 0x421, 0x440, 0x435, 0x434, 0x43d, 0x43e, 0x20,
0x433, 0x440, 0x438, 0x43d, 0x443, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x433, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x433, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x433, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x433, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x433, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x433, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x441, 0x438, 0x439, 0x441, 0x43a, 0x438,
0x20, 0x437, 0x430, 0x43b, 0x438, 0x432, 0x413, 0x430, 0x44f, 0x43d, 0x430, 0x425,
0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443,
0x442, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x430,
0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430,
0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x425, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432, 0x434,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432,
0x434, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432,
0x434, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418,
0x43d, 0x434, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x43d, 0x434, 0x438, 0x439, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x418, 0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x442, 0x430, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435,
0x437, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x438, 0x43d, 0x434, 0x43e, 0x43d,
0x435, 0x437, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x438, 0x43d, 0x434, 0x43e,
0x43d, 0x435, 0x437, 0x438, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x440, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418,
0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x42f, 0x43f, 0x43e, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x42f, 0x43f, 0x43e,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x42f, 0x43f, 0x43e,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f,
0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a,
0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432,
0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442,
0x441, 0x43a, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x442, 0x440,
0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430,
0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x20, 0x2013, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x43a,
0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x43a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435,
0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440,
0x435, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41a, 0x43e, 0x448, 0x440, 0x430, 0x439, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x44f, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a,
0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441,
0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x440,
0x433, 0x438, 0x437, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x415, 0x43a, 0x432, 0x430, 0x442, 0x43e, 0x440, 0x438,
0x430, 0x43b, 0x43d, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x41b, 0x43e, 0x440, 0x434,
0x20, 0x425, 0x430, 0x443, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x20, 0x2013, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x438, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43b,
0x434, 0x438, 0x432, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43e, 0x432,
0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41c, 0x430, 0x432,
0x440, 0x438, 0x446, 0x438, 0x439, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438,
0x439, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x432, 0x440,
0x438, 0x446, 0x438, 0x439, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41c, 0x43e, 0x443, 0x441, 0x44a, 0x43d, 0x41c, 0x435, 0x43a, 0x441, 0x438,
0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x423, 0x43b, 0x430, 0x43d, 0x431, 0x430, 0x442, 0x43e, 0x440, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x431,
0x430, 0x442, 0x43e, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x423, 0x43b, 0x430, 0x43d, 0x431, 0x430, 0x442, 0x43e, 0x440, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x43e,
0x432, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x43e,
0x441, 0x43a, 0x43e, 0x432, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41c, 0x438, 0x430, 0x43d, 0x43c, 0x430, 0x440, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x435, 0x43f, 0x430,
0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e,
0x432, 0x43e, 0x43a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x430,
0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x43a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e,
0x432, 0x43e, 0x437, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x437, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x432, 0x43e, 0x437, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x44e, 0x444, 0x430, 0x443,
0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435,
0x43d, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x44e,
0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e,
0x43b, 0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430,
0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d,
0x44f, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435,
0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44f, 0x20, 0x2013, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434,
0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44f, 0x20, 0x2013, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431,
0x438, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e,
0x43c, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x43c,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f,
0x430, 0x43f, 0x443, 0x430, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x413, 0x432,
0x438, 0x43d, 0x435, 0x44f, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440,
0x430, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435,
0x440, 0x443, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41f, 0x435, 0x440, 0x443, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b, 0x438,
0x43f, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424,
0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x20,
0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x438,
0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d,
0x421, 0x435, 0x43d, 0x20, 0x41f, 0x438, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x438, 0x435, 0x440, 0x20, 0x438, 0x20,
0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x2013, 0x20, 0x43b, 0x44f,
0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41f, 0x445, 0x435, 0x43d, 0x44f, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x420, 0x435, 0x44e, 0x43d, 0x438,
0x43e, 0x43d, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x441, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445,
0x430, 0x43b, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x2013, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x2013, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c,
0x43e, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421,
0x430, 0x43c, 0x43e, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x438, 0x421, 0x438,
0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x43e, 0x432,
0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x42e, 0x436, 0x43d,
0x430, 0x20, 0x414, 0x436, 0x43e, 0x440, 0x434, 0x436, 0x438, 0x44f, 0x421, 0x443,
0x440, 0x438, 0x43d, 0x430, 0x43c, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x422, 0x430, 0x438, 0x442, 0x44f, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x422, 0x430,
0x439, 0x43f, 0x435, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430,
0x439, 0x43f, 0x435, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x422, 0x430, 0x434, 0x436, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433,
0x430, 0x20, 0x2013, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433,
0x430, 0x20, 0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x443,
0x44e, 0x43a, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422,
0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c,
0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d,
0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x2013, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20,
0x2013, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x435, 0x43d, 0x435,
0x446, 0x443, 0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x441, 0x43a, 0x43e,
0x20, 0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x20, 0x423, 0x435, 0x439, 0x43a, 0x423, 0x43e, 0x43b, 0x438, 0x441, 0x20,
0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x42f, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x42f, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x42f,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x44f, 0x442,
0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431,
0x443, 0x440, 0x433, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433,
0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x415, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x441, 0x43a, 0x43e, 0x20,
0x43b, 0x44f, 0x442, 0x43d, 0x43e, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x432, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x42e, 0x43a, 0x43e, 0x43d, 0x1021, 0x102c,
0x1016, 0x1002, 0x1014, 0x103a, 0x1014, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x101a, 0x103a, 0x1021, 0x102c,
0x1016, 0x101b, 0x102d, 0x1000, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x101b, 0x103e, 0x1031, 0x1037, 0x1021, 0x102c, 0x1016, 0x101b, 0x102d, 0x1000, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1021, 0x102c,
0x1016, 0x101b, 0x102d, 0x1000, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1021, 0x102c, 0x1016, 0x101b, 0x102d, 0x1000, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a,
0x1021, 0x102c, 0x1016, 0x101b, 0x102d, 0x1000, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a,
0x1021, 0x102c, 0x1016, 0x101b, 0x102d, 0x1000, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x102c,
0x1005, 0x1000, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c,
0x102c, 0x1005, 0x1000, 0x102c, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x102c, 0x1005, 0x1000, 0x102c, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1019, 0x1031, 0x1007, 0x102f, 0x1036,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1019, 0x1031, 0x1007, 0x102f,
0x1036, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x1019, 0x1031, 0x1007, 0x102f, 0x1036, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c, 0x101a,
0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c,
0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1015, 0x102d, 0x102f, 0x1004,
0x103a, 0x1038, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031,
0x1037, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1015,
0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1010, 0x1014, 0x103a, 0x1038, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1010, 0x1014, 0x103a, 0x1038,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010,
0x1031, 0x102c, 0x1004, 0x103a, 0x1010, 0x1014, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1015, 0x1005, 0x102d, 0x1016, 0x102d, 0x1010, 0x103a, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x1005, 0x102d, 0x1016, 0x102d, 0x1010, 0x103a, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x1005,
0x102d, 0x1016, 0x102d, 0x1010, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x1015, 0x102e, 0x101a, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x1015, 0x102e, 0x101a, 0x102c, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1015, 0x102e, 0x101a, 0x102c, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x102c, 0x101b, 0x1031, 0x1017, 0x103b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x102c, 0x101b, 0x1031, 0x1017, 0x103b, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x101b, 0x1031,
0x1017, 0x103b, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x1002, 0x103b, 0x1004, 0x103a, 0x1010,
0x102e, 0x1038, 0x1014, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x102c, 0x1002, 0x103b, 0x1004, 0x103a, 0x1010, 0x102e, 0x1038, 0x1014, 0x102c, 0x1038,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x102c, 0x1002, 0x103b, 0x1004, 0x103a, 0x1010, 0x102e, 0x1038, 0x1014, 0x102c, 0x1038,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x20, 0x1021, 0x102c, 0x1002,
0x103b, 0x1004, 0x103a, 0x1010, 0x102e, 0x1038, 0x1014, 0x102c, 0x1038, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x20, 0x1021,
0x102c, 0x1002, 0x103b, 0x1004, 0x103a, 0x1010, 0x102e, 0x1038, 0x1014, 0x102c, 0x1038, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x20, 0x1021, 0x102c, 0x1002, 0x103b, 0x1004, 0x103a,
0x1010, 0x102e, 0x1038, 0x1014, 0x102c, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x1019,
0x1031, 0x1038, 0x1014, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x102c, 0x1019, 0x1031, 0x1038, 0x1014, 0x102e, 0x1038, 0x101a,
0x102c, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x102c, 0x1019, 0x1031, 0x1038, 0x1014, 0x102e, 0x1038, 0x101a, 0x102c,
0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1010, 0x1039, 0x1010, 0x101c, 0x1014, 0x103a, 0x1010,
0x1005, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1010, 0x1039,
0x1010, 0x101c, 0x1014, 0x103a, 0x1010, 0x1005, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1010, 0x1039, 0x1010, 0x101c,
0x1014, 0x103a, 0x1010, 0x1005, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1029, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1021, 0x101c,
0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1029, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x20,
0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1029, 0x1005,
0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015,
0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101e, 0x103c, 0x1005, 0x1010,
0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x102c, 0x1038, 0x20, 0x1021, 0x1014, 0x1031, 0x102c,
0x1000, 0x103a, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101e, 0x103c, 0x1005, 0x1010, 0x103c,
0x1031, 0x1038, 0x101c, 0x103b, 0x102c, 0x1038, 0x20, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000,
0x103a, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101e,
0x103c, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x102c, 0x1038, 0x20, 0x1021,
0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015, 0x102d, 0x102f,
0x1004, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1029,
0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1029, 0x1005, 0x1010, 0x103c, 0x1031,
0x1038, 0x101c, 0x103b, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1029, 0x1005, 0x1010, 0x103c,
0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000,
0x103a, 0x1029, 0x1005, 0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1029, 0x1005,
0x1010, 0x103c, 0x1031, 0x1038, 0x101c, 0x103b, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1007, 0x102c, 0x1018, 0x102d, 0x102f,
0x1004, 0x103a, 0x1002, 0x103b, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x1007, 0x102c, 0x1018, 0x102d, 0x102f, 0x1004, 0x103a, 0x1002, 0x103b, 0x1014,
0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x1007, 0x102c, 0x1018, 0x102d, 0x102f, 0x1004, 0x103a, 0x1002, 0x103b, 0x1014,
0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1031, 0x1007, 0x102d, 0x102f, 0x1038, 0x101b, 0x102e,
0x1038, 0x1005, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1031,
0x1007, 0x102d, 0x102f, 0x1038, 0x101b, 0x102e, 0x1038, 0x1005, 0x103a, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1031, 0x1007,
0x102d, 0x102f, 0x1038, 0x101b, 0x102e, 0x1038, 0x1005, 0x103a, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018,
0x1004, 0x103a, 0x1039, 0x1002, 0x101c, 0x102c, 0x1038, 0x1012, 0x1031, 0x1037, 0x101b, 0x103e,
0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x1004, 0x103a, 0x1039,
0x1002, 0x101c, 0x102c, 0x1038, 0x1012, 0x1031, 0x1037, 0x101b, 0x103e, 0x103a, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x1004,
0x103a, 0x1039, 0x1002, 0x101c, 0x102c, 0x1038, 0x1012, 0x1031, 0x1037, 0x101b, 0x103e, 0x103a,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1018, 0x1030, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1018, 0x102d, 0x102f, 0x101c, 0x102e, 0x1038, 0x1018, 0x102e, 0x1038,
0x101a, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x101b,
0x102c, 0x1007, 0x102e, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018,
0x101b, 0x102c, 0x1007, 0x102e, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1018, 0x101b, 0x102c, 0x1007, 0x102e, 0x1038, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1018, 0x101b, 0x1030, 0x1014, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000,
0x102d, 0x1010, 0x103a, 0x1017, 0x102c, 0x1012, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1000, 0x102d, 0x1010, 0x103a, 0x1017, 0x102c, 0x1012, 0x102e, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102d,
0x1010, 0x103a, 0x1017, 0x102c, 0x1012, 0x102e, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102c,
0x1019, 0x102d, 0x102f, 0x101b, 0x102d, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1001, 0x103b, 0x102c, 0x1038, 0x101e, 0x1019, 0x103a, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1001, 0x103b, 0x102c, 0x1038, 0x101e, 0x1019, 0x103a, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102c, 0x1038,
0x101e, 0x1019, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102e, 0x101c, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x102e, 0x101c, 0x102e, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001,
0x103b, 0x102e, 0x101c, 0x102e, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x101b, 0x102f, 0x1010, 0x103a,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x101b, 0x102f, 0x1010, 0x103a,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1010, 0x101b, 0x102f, 0x1010, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x103d, 0x1032,
0x1018, 0x1031, 0x102c, 0x103a, 0x1006, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x103d, 0x1032, 0x1018, 0x1031, 0x102c, 0x103a, 0x1006,
0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1001, 0x103b, 0x103d, 0x1032, 0x1018, 0x1031, 0x102c, 0x103a, 0x1006,
0x1014, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x101b, 0x1005, 0x103a, 0x1005, 0x1019,
0x1010, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x1000, 0x102d, 0x102f, 0x1038, 0x1000, 0x103b,
0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000,
0x102d, 0x102f, 0x101c, 0x1036, 0x1018, 0x102e, 0x101a, 0x102c, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x101c, 0x1036, 0x1018, 0x102e, 0x101a, 0x102c,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1000, 0x102d, 0x102f, 0x101c, 0x1036, 0x1018, 0x102e, 0x101a, 0x102c, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1000, 0x103d, 0x1010, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103d, 0x1010, 0x103a, 0x1000,
0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103d, 0x1010, 0x103a, 0x1000,
0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103b,
0x1030, 0x1038, 0x1018, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1000, 0x103b, 0x1030, 0x1038, 0x1018, 0x102c, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x103b, 0x1030, 0x1038, 0x1018,
0x102c, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1012, 0x1031,
0x1038, 0x1017, 0x1005, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1012,
0x1030, 0x1038, 0x1019, 0x1031, 0x102c, 0x1037, 0x1010, 0x103a, 0x20, 0x1012, 0x102b, 0x101b,
0x103a, 0x1017, 0x102e, 0x1038, 0x101c, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1010, 0x102e, 0x1019, 0x1031, 0x102c, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e, 0x1005, 0x1010, 0x102c, 0x1000,
0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1021, 0x102e, 0x1005, 0x1010, 0x102c, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x102e, 0x1005, 0x1010, 0x102c, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x102e, 0x1000, 0x103d, 0x1031, 0x1012, 0x1031, 0x102b, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x1021, 0x101c, 0x101a,
0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x1021, 0x101c, 0x101a, 0x103a, 0x1015,
0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x1021, 0x101c,
0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x101b, 0x103e, 0x1031, 0x1037, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1025, 0x101b, 0x1031,
0x102c, 0x1015, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1016, 0x103b, 0x102c, 0x1038,
0x20, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x1012, 0x1031, 0x101e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1025, 0x101b,
0x1031, 0x102c, 0x1015, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014,
0x1031, 0x102c, 0x1000, 0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031,
0x102c, 0x1000, 0x103a, 0x1025, 0x101b, 0x1031, 0x102c, 0x1015, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016,
0x1031, 0x102c, 0x1037, 0x1000, 0x101c, 0x1014, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a,
0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x1031,
0x102c, 0x1037, 0x1000, 0x101c, 0x1014, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038,
0x1005, 0x102f, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1016, 0x1031, 0x102c, 0x1037, 0x1000, 0x101c, 0x1014, 0x103a, 0x1000, 0x103b,
0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102e, 0x1002,
0x103b, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102e, 0x1002,
0x103b, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1016, 0x102e, 0x1002, 0x103b, 0x102e, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x103c,
0x1004, 0x103a, 0x101e, 0x1005, 0x103a, 0x20, 0x1002, 0x102e, 0x1021, 0x102c, 0x1038, 0x1014,
0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x103c, 0x1004,
0x103a, 0x101e, 0x1005, 0x103a, 0x1010, 0x1031, 0x102c, 0x1004, 0x103a, 0x1015, 0x102d, 0x102f,
0x1004, 0x103a, 0x1038, 0x1014, 0x103e, 0x1004, 0x1037, 0x103a, 0x20, 0x1021, 0x1014, 0x1039,
0x1010, 0x102c, 0x1010, 0x102d, 0x1010, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1002, 0x102b, 0x101c, 0x102c, 0x1038, 0x1015, 0x102b, 0x1002, 0x102d, 0x102f, 0x1038,
0x1005, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x1019, 0x103a,
0x1018, 0x102e, 0x101a, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1002,
0x103b, 0x1031, 0x102c, 0x103a, 0x1002, 0x103b, 0x102e, 0x101a, 0x102c, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x103b, 0x1031, 0x102c, 0x103a, 0x1002, 0x103b, 0x102e,
0x101a, 0x102c, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1002, 0x103b, 0x1031, 0x102c, 0x103a, 0x1002, 0x103b, 0x102e, 0x101a, 0x102c,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1002, 0x102e, 0x101c, 0x103a, 0x1018, 0x1010, 0x103a, 0x1000, 0x103b,
0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1002, 0x101b, 0x1004, 0x103a, 0x1038, 0x1014, 0x1005, 0x103a, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e,
0x1031, 0x1037, 0x1002, 0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x1038, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1002,
0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031,
0x1037, 0x1002, 0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x20,
0x1002, 0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x20, 0x1002,
0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c,
0x1000, 0x103a, 0x20, 0x1002, 0x101b, 0x1004, 0x103a, 0x1038, 0x101c, 0x1014, 0x103a, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x1004, 0x103a, 0x101c, 0x101a,
0x103a, 0x1000, 0x103d, 0x1031, 0x1037, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1002, 0x102d, 0x102f, 0x1004, 0x103a, 0x101a, 0x102c, 0x1014, 0x102c, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x102c, 0x101d, 0x102d, 0x102f, 0x1004, 0x103a, 0x101a,
0x102e, 0x20, 0x1021, 0x101a, 0x103a, 0x101c, 0x1030, 0x1038, 0x101b, 0x103e, 0x1014, 0x103a,
0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x102c, 0x101d, 0x102d,
0x102f, 0x1004, 0x103a, 0x101a, 0x102e, 0x20, 0x1021, 0x101a, 0x103a, 0x101c, 0x1030, 0x1038,
0x101b, 0x103e, 0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x102c, 0x101d, 0x102d, 0x102f, 0x1004, 0x103a,
0x101a, 0x102e, 0x20, 0x1021, 0x101a, 0x103a, 0x101c, 0x1030, 0x1038, 0x101b, 0x103e, 0x1014,
0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x1031,
0x102c, 0x1004, 0x103a, 0x1000, 0x1031, 0x102c, 0x1004, 0x103a, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x1000, 0x1031, 0x102c, 0x1004,
0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x1000, 0x1031, 0x102c, 0x1004, 0x103a, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x101f, 0x102d, 0x102f, 0x1038, 0x1017, 0x103a, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x101f, 0x102d, 0x102f, 0x1038, 0x1017, 0x103a, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101f, 0x102d, 0x102f,
0x1038, 0x1017, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102d, 0x1014, 0x1039, 0x1012, 0x102d,
0x101a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x102d, 0x1014, 0x1039, 0x1012, 0x102d, 0x101a, 0x101e, 0x1019, 0x102f, 0x1012,
0x1039, 0x1012, 0x101b, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x1004, 0x103a, 0x1012, 0x102d, 0x102f, 0x1001, 0x103b, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038,
0x1014, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101c,
0x101a, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1021, 0x1004, 0x103a,
0x1012, 0x102d, 0x102f, 0x1014, 0x102e, 0x1038, 0x101b, 0x103e, 0x102c, 0x1038, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1015, 0x102d,
0x102f, 0x1004, 0x103a, 0x1038, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102d, 0x102f, 0x1014,
0x102e, 0x1038, 0x101b, 0x103e, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a,
0x1038, 0x20, 0x1021, 0x1004, 0x103a, 0x1012, 0x102d, 0x102f, 0x1014, 0x102e, 0x1038, 0x101b,
0x103e, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e,
0x101b, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e,
0x101b, 0x1014, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x102e, 0x101b, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x102e, 0x101b, 0x1030, 0x1001, 0x1030, 0x1010, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x102e, 0x101b, 0x1030, 0x1001, 0x1030, 0x1010, 0x103a, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x102e,
0x101b, 0x1030, 0x1001, 0x1030, 0x1010, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1005, 0x1039,
0x1005, 0x101b, 0x1031, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021,
0x1005, 0x1039, 0x1005, 0x101b, 0x1031, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x1005, 0x1039, 0x1005, 0x101b, 0x1031,
0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x103b, 0x1015, 0x1014, 0x103a, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x103b, 0x1015, 0x1014, 0x103a, 0x20, 0x1005, 0x1036,
0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1002, 0x103b, 0x1015,
0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x101b, 0x103e, 0x1031, 0x1037, 0x1000, 0x102c,
0x1007, 0x1000, 0x103a, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x1014, 0x1031, 0x102c, 0x1000, 0x103a, 0x1000, 0x102c, 0x1007, 0x1000,
0x103a, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1000, 0x102d, 0x102f, 0x101b, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102d, 0x102f, 0x101b, 0x102e, 0x1038, 0x101a, 0x102c,
0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1000, 0x102d, 0x102f, 0x101b, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1001, 0x102d, 0x102f, 0x1005, 0x103a, 0x101b, 0x102d, 0x102f, 0x1004, 0x103a, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001, 0x101b, 0x102c, 0x1037, 0x1005, 0x103a,
0x1014, 0x102d, 0x102f, 0x101b, 0x102c, 0x1005, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1001, 0x101b, 0x102c, 0x1037, 0x1005, 0x103a, 0x1014, 0x102d, 0x102f, 0x101b,
0x102c, 0x1005, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1001, 0x101b, 0x102c, 0x1037, 0x1005, 0x103a, 0x1014, 0x102d, 0x102f,
0x101b, 0x102c, 0x1005, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1000, 0x102c, 0x1002, 0x103b, 0x1005,
0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x101e, 0x102e, 0x101b, 0x102d, 0x101c, 0x1004, 0x103a, 0x1039, 0x1000, 0x102c, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101c, 0x102d, 0x102f, 0x1004, 0x103a, 0x1038, 0x1000,
0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x101c, 0x1031, 0x102c, 0x1037, 0x1012, 0x103a, 0x101f, 0x1031, 0x102c, 0x1004,
0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101c, 0x1031, 0x102c, 0x1037,
0x1012, 0x103a, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101c, 0x1031, 0x102c, 0x1037, 0x1012,
0x103a, 0x101f, 0x1031, 0x102c, 0x1004, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1000, 0x102c,
0x1021, 0x102d, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1000,
0x102c, 0x1021, 0x102d, 0x102f, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1000, 0x102c, 0x1021, 0x102d, 0x102f, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1019, 0x102c, 0x1002, 0x102b, 0x1012, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x102c, 0x1002, 0x102b, 0x1012, 0x1014, 0x103a, 0x1038,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1019, 0x102c, 0x1002, 0x102b, 0x1012, 0x1014, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019,
0x101c, 0x1031, 0x1038, 0x101b, 0x103e, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a, 0x101c, 0x1012, 0x102d, 0x102f, 0x1000, 0x103a,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x102c, 0x1001, 0x1031, 0x1038,
0x1021, 0x1015, 0x103a, 0x1005, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1019, 0x102c, 0x101b, 0x103e, 0x101a, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038,
0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c,
0x101b, 0x1005, 0x103a, 0x101b, 0x103e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1019, 0x1031, 0x102c, 0x101b, 0x1005, 0x103a, 0x101b, 0x103e, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c, 0x101b,
0x1005, 0x103a, 0x101b, 0x103e, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a, 0x1005,
0x103d, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1000,
0x1039, 0x1000, 0x1006, 0x102e, 0x1000, 0x1014, 0x103a, 0x20, 0x1015, 0x1005, 0x102d, 0x1016,
0x102d, 0x1010, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1000,
0x1039, 0x1000, 0x1006, 0x102e, 0x1000, 0x1014, 0x103a, 0x20, 0x1015, 0x1005, 0x102d, 0x1016,
0x102d, 0x1010, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1019, 0x1000, 0x1039, 0x1000, 0x1005, 0x102e, 0x1000, 0x1014, 0x103a,
0x20, 0x1015, 0x1005, 0x102d, 0x1016, 0x102d, 0x1010, 0x103a, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101c, 0x1014, 0x103a, 0x1018, 0x102c, 0x1010, 0x1031,
0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101c, 0x1014, 0x103a,
0x1018, 0x102c, 0x1010, 0x1031, 0x102c, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101c, 0x1014, 0x103a, 0x1018, 0x102c, 0x1010,
0x1031, 0x102c, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a, 0x1005, 0x1000, 0x102d,
0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a,
0x1005, 0x1000, 0x102d, 0x102f, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1019, 0x1031, 0x102c, 0x103a, 0x1005, 0x1000, 0x102d, 0x102f,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1019, 0x103c, 0x1014, 0x103a, 0x1019, 0x102c, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102c, 0x1025, 0x1030, 0x101b, 0x1030, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102e, 0x1015, 0x1031, 0x102b, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1000, 0x101a, 0x103a,
0x101c, 0x102e, 0x1012, 0x102d, 0x102f, 0x1038, 0x1014, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1000,
0x102c, 0x101c, 0x102e, 0x1012, 0x102d, 0x102f, 0x1038, 0x1014, 0x102e, 0x1038, 0x101a, 0x102c,
0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1000, 0x102c, 0x101c, 0x102e, 0x1012, 0x102d, 0x102f,
0x1038, 0x1014, 0x102e, 0x1038, 0x101a, 0x102c, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a,
0x1030, 0x1038, 0x1007, 0x102e, 0x101c, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1007, 0x102e, 0x101c, 0x1014, 0x103a, 0x20,
0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014,
0x101a, 0x1030, 0x1038, 0x1007, 0x102e, 0x101c, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014,
0x101a, 0x1030, 0x1038, 0x1016, 0x1031, 0x102c, 0x1004, 0x103a, 0x101c, 0x1014, 0x103a, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1016, 0x1031,
0x102c, 0x1004, 0x103a, 0x101c, 0x1014, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c,
0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x101a, 0x1030, 0x1038, 0x1016, 0x1031,
0x102c, 0x1004, 0x103a, 0x101c, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1014, 0x102e, 0x1025, 0x1030, 0x1021, 0x1031, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1014, 0x1031, 0x102c, 0x1016, 0x1031, 0x102c, 0x1037, 0x1001, 0x103a,
0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1014, 0x1031, 0x102c, 0x1016, 0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x1000, 0x103b,
0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x1031, 0x102c, 0x1016, 0x1031, 0x102c, 0x1037, 0x1001,
0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b,
0x102c, 0x101e, 0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1016, 0x102c, 0x1014, 0x1014, 0x103a, 0x1012, 0x102d, 0x102f, 0x1038,
0x1012, 0x102e, 0x1014, 0x102d, 0x102f, 0x1038, 0x101b, 0x102d, 0x102f, 0x1038, 0x100a, 0x102c,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102c, 0x1014, 0x1014, 0x103a,
0x1012, 0x102d, 0x102f, 0x1038, 0x1012, 0x102e, 0x1014, 0x102d, 0x102f, 0x1038, 0x101b, 0x102d,
0x102f, 0x1038, 0x100a, 0x102c, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102c, 0x1014, 0x1014, 0x103a, 0x1012, 0x102d, 0x102f,
0x1038, 0x1012, 0x102e, 0x1014, 0x102d, 0x102f, 0x1038, 0x101b, 0x102d, 0x102f, 0x1038, 0x100a,
0x102c, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102d, 0x102f, 0x1017, 0x102d, 0x102f, 0x1005, 0x1032,
0x1018, 0x102e, 0x1021, 0x1032, 0x101a, 0x103a, 0x1005, 0x103a, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1014, 0x102d, 0x102f, 0x1017, 0x102d, 0x102f, 0x1005, 0x1032, 0x1018,
0x102e, 0x1021, 0x1032, 0x101a, 0x103a, 0x1005, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1014, 0x102d, 0x102f, 0x1017, 0x102d,
0x102f, 0x1005, 0x1032, 0x1018, 0x102e, 0x1021, 0x1032, 0x101a, 0x103a, 0x1005, 0x103a, 0x20,
0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1021, 0x103d, 0x1019, 0x103a, 0x1038, 0x1005, 0x103a, 0x1001, 0x103a, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1021, 0x103d, 0x1019, 0x103a, 0x1038, 0x1005,
0x103a, 0x1001, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1021, 0x103d, 0x1019, 0x103a, 0x1038, 0x1005, 0x103a, 0x1001, 0x103a,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x1000, 0x1005, 0x1039,
0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014,
0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x101c, 0x102c, 0x1021, 0x102d, 0x102f, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x1015, 0x1030, 0x1021, 0x102c, 0x1014,
0x101a, 0x1030, 0x1038, 0x1002, 0x102e, 0x1014, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1015, 0x102b, 0x101b, 0x102c, 0x1002, 0x103d, 0x1031, 0x1038, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102b, 0x101b, 0x102c, 0x1002, 0x103d, 0x1031,
0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1015, 0x102b, 0x101b, 0x102c, 0x1002, 0x103d, 0x1031, 0x1038, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1015, 0x102e, 0x101b, 0x1030, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1015, 0x102e, 0x101b, 0x1030, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x102e, 0x101b, 0x1030, 0x1038, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1016, 0x102d, 0x101c, 0x1005, 0x103a, 0x1015, 0x102d, 0x102f, 0x1004, 0x103a, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102d, 0x101c, 0x1005, 0x103a, 0x1015,
0x102d, 0x102f, 0x1004, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102d, 0x101c, 0x1005, 0x103a, 0x1015, 0x102d, 0x102f,
0x1004, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102e, 0x1038, 0x1014, 0x1005, 0x103a, 0x1000,
0x103b, 0x103d, 0x1014, 0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1015, 0x102e, 0x1021, 0x1032, 0x1014,
0x103e, 0x1004, 0x1037, 0x103a, 0x1019, 0x102e, 0x1001, 0x103d, 0x102e, 0x101c, 0x103d, 0x1014,
0x103a, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a,
0x1015, 0x102e, 0x1021, 0x1032, 0x1014, 0x103e, 0x1004, 0x1037, 0x103a, 0x1019, 0x102e, 0x1001,
0x103d, 0x102e, 0x101c, 0x103d, 0x1014, 0x103a, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1005, 0x102d, 0x1014, 0x1037, 0x103a, 0x1015, 0x102e,
0x1021, 0x1032, 0x1014, 0x103e, 0x1004, 0x1037, 0x103a, 0x20, 0x1019, 0x102e, 0x1001, 0x103d,
0x102e, 0x101c, 0x103d, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1015, 0x102b, 0x1010, 0x103a, 0x1000, 0x101a, 0x103a, 0x101b, 0x1004, 0x103a, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1016, 0x102d, 0x102f, 0x1014, 0x102c, 0x1016,
0x1032, 0x1021, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1015, 0x103c,
0x102f, 0x1036, 0x101a, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x101b, 0x102e, 0x101a, 0x1030, 0x1014, 0x102e, 0x101a, 0x1036, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x101b, 0x102d, 0x102f, 0x101e, 0x102e, 0x101b, 0x102c, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006, 0x102c, 0x1001, 0x102b, 0x101c, 0x1004,
0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006, 0x102c, 0x1001, 0x102b,
0x101c, 0x1004, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1006, 0x102c, 0x1001, 0x102b, 0x101c, 0x1004, 0x103a, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1006, 0x1019, 0x102d, 0x102f, 0x1021, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1006, 0x1019, 0x102d, 0x102f, 0x1021, 0x102c, 0x20, 0x1005, 0x1036, 0x1010,
0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006, 0x1019, 0x102d, 0x102f,
0x1038, 0x1021, 0x102c, 0x1038, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006, 0x1031, 0x1038, 0x101b, 0x103e,
0x1032, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1005, 0x1004, 0x103a, 0x1039,
0x1000, 0x102c, 0x1015, 0x1030, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1006,
0x1031, 0x102c, 0x103a, 0x101c, 0x1019, 0x103d, 0x1014, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014,
0x103a, 0x1038, 0x1005, 0x102f, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010,
0x1031, 0x102c, 0x1004, 0x103a, 0x1002, 0x103b, 0x1031, 0x102c, 0x103a, 0x1002, 0x103b, 0x102e,
0x101a, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1005, 0x1030, 0x1038,
0x101b, 0x102e, 0x1014, 0x102c, 0x1019, 0x103a, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x101b, 0x103e, 0x102d, 0x102f, 0x101d, 0x102b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1010, 0x101f, 0x102e, 0x1010, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1011, 0x102d, 0x102f, 0x1004, 0x103a, 0x1015, 0x1031, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1011, 0x102d, 0x102f, 0x1004, 0x103a, 0x1015, 0x1031, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1011, 0x102d,
0x102f, 0x1004, 0x103a, 0x1015, 0x1031, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x102c, 0x1002, 0x103b,
0x1005, 0x103a, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001,
0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x102d, 0x102f, 0x1000, 0x102e, 0x101c, 0x102c, 0x1025,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x103d, 0x1014, 0x103a, 0x1002,
0x102b, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x103d, 0x1014, 0x103a,
0x1002, 0x102b, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1010, 0x103d, 0x1014, 0x103a, 0x1002, 0x102b, 0x20, 0x1014, 0x103d, 0x1031,
0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1001,
0x103b, 0x102f, 0x1001, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010,
0x102c, 0x1037, 0x1001, 0x103a, 0x1019, 0x1004, 0x103a, 0x1014, 0x1005, 0x1039, 0x1005, 0x1010,
0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x102c, 0x1037,
0x1001, 0x103a, 0x1019, 0x1004, 0x103a, 0x1014, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1010, 0x102c, 0x1037, 0x1001, 0x103a, 0x1019, 0x1004, 0x103a, 0x1014, 0x1005, 0x1039, 0x1005,
0x1010, 0x1014, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20,
0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1010, 0x1030, 0x1017, 0x102c, 0x1038, 0x101c,
0x1030, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101b, 0x102f, 0x1002,
0x103d, 0x1031, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101b,
0x102f, 0x1002, 0x103d, 0x1031, 0x1038, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x101b, 0x102f, 0x1002, 0x103d, 0x1031, 0x1038,
0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1025, 0x1007, 0x1018, 0x1000, 0x103a, 0x1000, 0x1005, 0x1039, 0x1005,
0x1010, 0x1014, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x1007,
0x1018, 0x1000, 0x103a, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1025, 0x1007,
0x1018, 0x1000, 0x103a, 0x1000, 0x1005, 0x1039, 0x1005, 0x1010, 0x1014, 0x103a, 0x20, 0x1014,
0x103d, 0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1017, 0x1014, 0x103d, 0x102c, 0x1038, 0x1010, 0x1030, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x1017, 0x1014, 0x103d, 0x102c, 0x1038, 0x1010, 0x1030, 0x20, 0x1005,
0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x1014,
0x103d, 0x102c, 0x1038, 0x1010, 0x1030, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x1004, 0x103a, 0x1014,
0x102e, 0x1007, 0x103d, 0x1032, 0x101c, 0x102c, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d,
0x1014, 0x103a, 0x1017, 0x101c, 0x102c, 0x1012, 0x102e, 0x1017, 0x1031, 0x102c, 0x1037, 0x1005,
0x1010, 0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x1017, 0x101c, 0x102c, 0x1012, 0x102e, 0x1017, 0x1031, 0x102c, 0x1037, 0x1005, 0x1010,
0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a,
0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x101c, 0x102c, 0x1012, 0x102e, 0x1017, 0x1031,
0x102c, 0x1037, 0x1005, 0x1010, 0x1031, 0x102c, 0x1037, 0x1001, 0x103a, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x1017, 0x102d, 0x102f, 0x101c, 0x103a, 0x1002, 0x102d, 0x102f, 0x1002, 0x101b, 0x1000, 0x103a,
0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x102d, 0x102f, 0x101c, 0x103a,
0x1002, 0x102d, 0x102f, 0x1002, 0x101b, 0x1000, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031,
0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x102d, 0x102f, 0x1002, 0x102d,
0x102f, 0x1002, 0x101b, 0x1000, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c, 0x101e,
0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x1017, 0x102d, 0x102f, 0x1005,
0x103a, 0x1010, 0x102d, 0x102f, 0x1001, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014,
0x103a, 0x101d, 0x102d, 0x1010, 0x103a, 0x1001, 0x103a, 0x1000, 0x103b, 0x103d, 0x1014, 0x103a,
0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101d, 0x1031, 0x102b, 0x101c,
0x102e, 0x1005, 0x103a, 0x1014, 0x103e, 0x1004, 0x1037, 0x103a, 0x20, 0x1016, 0x1030, 0x1000,
0x103b, 0x1030, 0x1014, 0x102c, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101a,
0x1030, 0x1001, 0x1030, 0x1038, 0x1010, 0x103a, 0x1005, 0x103a, 0x20, 0x1021, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x101a, 0x1030, 0x1001, 0x1030, 0x1038, 0x1010, 0x103a, 0x1005, 0x103a,
0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x101a, 0x1030, 0x1001, 0x1030, 0x1038, 0x1010, 0x103a, 0x1005, 0x103a, 0x20, 0x1014, 0x103d,
0x1031, 0x101b, 0x102c, 0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a,
0x101b, 0x101a, 0x103a, 0x1001, 0x102b, 0x1038, 0x1010, 0x102e, 0x101b, 0x1004, 0x103a, 0x1018,
0x102c, 0x1038, 0x1001, 0x103a, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101b,
0x101a, 0x103a, 0x1001, 0x102b, 0x1038, 0x1010, 0x102e, 0x101b, 0x1004, 0x103a, 0x1018, 0x102c,
0x1038, 0x1001, 0x103a, 0x20, 0x1005, 0x1036, 0x1010, 0x1031, 0x102c, 0x103a, 0x1001, 0x103b,
0x102d, 0x1014, 0x103a, 0x101b, 0x101a, 0x103a, 0x1001, 0x102b, 0x1038, 0x1010, 0x102e, 0x101b,
0x1004, 0x103a, 0x1018, 0x102c, 0x1001, 0x103a, 0x20, 0x1014, 0x103d, 0x1031, 0x101b, 0x102c,
0x101e, 0x102e, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x101a, 0x1030, 0x1000,
0x103d, 0x1014, 0x103a, 0x1038, 0x20, 0x1021, 0x1001, 0x103b, 0x102d, 0x1014, 0x103a, 0x827e,
0x514b, 0x6642, 0x9593, 0x827e, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x827e, 0x514b, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x963f, 0x5bcc, 0x6c57, 0x6642, 0x9593, 0x4e2d, 0x975e, 0x6642, 0x9593,
0x6771, 0x975e, 0x6642, 0x9593, 0x5357, 0x975e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x897f, 0x975e,
0x6642, 0x9593, 0x897f, 0x975e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x897f, 0x975e, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x963f, 0x62c9, 0x65af, 0x52a0, 0x6642, 0x9593, 0x963f, 0x62c9, 0x65af, 0x52a0,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x62c9, 0x65af, 0x52a0, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x963f, 0x62c9, 0x6728, 0x5716, 0x6642, 0x9593, 0x963f, 0x62c9, 0x6728, 0x5716, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x963f, 0x62c9, 0x6728, 0x5716, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e9e, 0x99ac,
0x905c, 0x6642, 0x9593, 0x4e9e, 0x99ac, 0x905c, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e9e, 0x99ac,
0x905c, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e2d, 0x90e8, 0x6642, 0x9593, 0x4e2d, 0x90e8, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x4e2d, 0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6771, 0x90e8, 0x6642,
0x9593, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6771, 0x90e8, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x5c71, 0x5340, 0x6642, 0x9593, 0x5c71, 0x5340, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5c71,
0x5340, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x592a, 0x5e73, 0x6d0b, 0x6642, 0x9593, 0x592a, 0x5e73,
0x6d0b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x592a, 0x5e73, 0x6d0b, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x963f, 0x7d0d, 0x5fb7, 0x723e, 0x6642, 0x9593, 0x963f, 0x90a3, 0x5e95, 0x6cb3, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x963f, 0x90a3, 0x5e95, 0x6cb3, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x76ae,
0x4e9e, 0x6642, 0x9593, 0x963f, 0x76ae, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x76ae,
0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x514b, 0x9676, 0x6642, 0x9593, 0x963f, 0x514b,
0x9676, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x514b, 0x9676, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x963f, 0x514b, 0x6258, 0x6bd4, 0x6642, 0x9593, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x62c9,
0x4f2f, 0x6642, 0x9593, 0x963f, 0x62c9, 0x4f2f, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x62c9,
0x4f2f, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x963f, 0x6839, 0x5ef7, 0x6642, 0x9593, 0x963f, 0x6839,
0x5ef7, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x6839, 0x5ef7, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x963f, 0x6839, 0x5ef7, 0x897f, 0x90e8, 0x6642, 0x9593, 0x963f, 0x6839, 0x5ef7, 0x897f, 0x90e8,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x963f, 0x6839, 0x5ef7, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x4e9e, 0x7f8e, 0x5c3c, 0x4e9e, 0x6642, 0x9593, 0x4e9e, 0x7f8e, 0x5c3c, 0x4e9e, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x4e9e, 0x7f8e, 0x5c3c, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5927,
0x897f, 0x6d0b, 0x6642, 0x9593, 0x5927, 0x897f, 0x6d0b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5927,
0x897f, 0x6d0b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x4e2d, 0x90e8, 0x6642, 0x9593,
0x6fb3, 0x6d32, 0x4e2d, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x4e2d, 0x90e8,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x4e2d, 0x897f, 0x90e8, 0x6642, 0x9593, 0x6fb3,
0x6d32, 0x4e2d, 0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x4e2d, 0x897f,
0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x6771, 0x90e8, 0x6642, 0x9593, 0x6fb3,
0x6d32, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x6771, 0x90e8, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x897f, 0x90e8, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x897f,
0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6fb3, 0x6d32, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7136, 0x6642, 0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7136, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7136, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e9e,
0x901f, 0x723e, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x4e9e, 0x901f, 0x723e, 0x7fa4, 0x5cf6, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x4e9e, 0x901f, 0x723e, 0x7fa4, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x5b5f, 0x52a0, 0x62c9, 0x6642, 0x9593, 0x5b5f, 0x52a0, 0x62c9, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x5b5f, 0x52a0, 0x62c9, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e0d, 0x4e39, 0x6642, 0x9593, 0x73bb,
0x5229, 0x7dad, 0x4e9e, 0x6642, 0x9593, 0x5df4, 0x897f, 0x5229, 0x4e9e, 0x6642, 0x9593, 0x5df4,
0x897f, 0x5229, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5df4, 0x897f, 0x5229, 0x4e9e, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x6c76, 0x840a, 0x6642, 0x9593, 0x7dad, 0x5fb7, 0x89d2, 0x6642, 0x9593,
0x7dad, 0x5fb7, 0x89d2, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x7dad, 0x5fb7, 0x89d2, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x51f1, 0x897f, 0x7ad9, 0x6642, 0x9593, 0x67e5, 0x83ab, 0x6d1b, 0x6642, 0x9593,
0x67e5, 0x5766, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x67e5, 0x5766, 0x7fa4, 0x5cf6, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x67e5, 0x5766, 0x7fa4, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x667a, 0x5229,
0x6642, 0x9593, 0x667a, 0x5229, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x667a, 0x5229, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x4e2d, 0x570b, 0x6642, 0x9593, 0x4e2d, 0x570b, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x4e2d, 0x570b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x55ac, 0x5df4, 0x5c71, 0x6642, 0x9593, 0x55ac,
0x5df4, 0x5c71, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x55ac, 0x5df4, 0x5c71, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x8056, 0x8a95, 0x5cf6, 0x6642, 0x9593, 0x79d1, 0x79d1, 0x65af, 0x7fa4, 0x5cf6, 0x6642,
0x9593, 0x54e5, 0x502b, 0x6bd4, 0x4e9e, 0x6642, 0x9593, 0x54e5, 0x502b, 0x6bd4, 0x4e9e, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x54e5, 0x502b, 0x6bd4, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5eab,
0x514b, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x5eab, 0x514b, 0x7fa4, 0x5cf6, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x5eab, 0x514b, 0x7fa4, 0x5cf6, 0x534a, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x53e4, 0x5df4,
0x6642, 0x9593, 0x53e4, 0x5df4, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x53e4, 0x5df4, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x6234, 0x7dad, 0x65af, 0x6642, 0x9593, 0x675c, 0x8499, 0x675c, 0x6bd4, 0x723e,
0x6642, 0x9593, 0x6771, 0x5e1d, 0x6c76, 0x6642, 0x9593, 0x5fa9, 0x6d3b, 0x7bc0, 0x5cf6, 0x6642,
0x9593, 0x5fa9, 0x6d3b, 0x7bc0, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5fa9, 0x6d3b, 0x7bc0,
0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5384, 0x74dc, 0x591a, 0x6642, 0x9593, 0x4e2d, 0x6b50,
0x6642, 0x9593, 0x4e2d, 0x6b50, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4e2d, 0x6b50, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x6771, 0x6b50, 0x6642, 0x9593, 0x6771, 0x6b50, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x6771, 0x6b50, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6b50, 0x6d32, 0x9060, 0x6771, 0x6642, 0x9593,
0x897f, 0x6b50, 0x6642, 0x9593, 0x897f, 0x6b50, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x897f, 0x6b50,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x798f, 0x514b, 0x862d, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x798f,
0x514b, 0x862d, 0x7fa4, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x798f, 0x514b, 0x862d, 0x7fa4,
0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6590, 0x6fdf, 0x6642, 0x9593, 0x6590, 0x6fdf, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x6590, 0x6fdf, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6cd5, 0x5c6c, 0x572d,
0x4e9e, 0x90a3, 0x6642, 0x9593, 0x6cd5, 0x570b, 0x5357, 0x65b9, 0x53ca, 0x5357, 0x6975, 0x6642,
0x9593, 0x52a0, 0x62c9, 0x5df4, 0x54e5, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x7518, 0x6bd4, 0x723e,
0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x55ac, 0x6cbb, 0x4e9e, 0x6642, 0x9593, 0x55ac, 0x6cbb, 0x4e9e,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x55ac, 0x6cbb, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5409,
0x723e, 0x4f2f, 0x7279, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x683c, 0x6797, 0x5a01, 0x6cbb, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x683c, 0x9675, 0x862d, 0x6771, 0x90e8, 0x6642, 0x9593, 0x683c, 0x9675,
0x862d, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x683c, 0x9675, 0x862d, 0x6771, 0x90e8,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x683c, 0x9675, 0x862d, 0x897f, 0x90e8, 0x6642, 0x9593, 0x683c,
0x9675, 0x862d, 0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x683c, 0x9675, 0x862d, 0x897f,
0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x95dc, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6ce2,
0x65af, 0x7063, 0x6d77, 0x57df, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x84cb, 0x4e9e, 0x90a3, 0x6642,
0x9593, 0x590f, 0x5a01, 0x5937, 0x2d, 0x963f, 0x7559, 0x7533, 0x6642, 0x9593, 0x590f, 0x5a01,
0x5937, 0x2d, 0x963f, 0x7559, 0x7533, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x590f, 0x5a01, 0x5937,
0x2d, 0x963f, 0x7559, 0x7533, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x9999, 0x6e2f, 0x6642, 0x9593,
0x9999, 0x6e2f, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x9999, 0x6e2f, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x79d1, 0x5e03, 0x591a, 0x6642, 0x9593, 0x79d1, 0x5e03, 0x591a, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x79d1, 0x5e03, 0x591a, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5370, 0x5ea6, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x5370, 0x5ea6, 0x6d0b, 0x6642, 0x9593, 0x5370, 0x5ea6, 0x652f, 0x90a3, 0x6642, 0x9593,
0x5370, 0x5c3c, 0x4e2d, 0x90e8, 0x6642, 0x9593, 0x5370, 0x5c3c, 0x6771, 0x90e8, 0x6642, 0x9593,
0x5370, 0x5c3c, 0x897f, 0x90e8, 0x6642, 0x9593, 0x4f0a, 0x6717, 0x6642, 0x9593, 0x4f0a, 0x6717,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x4f0a, 0x6717, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4f0a, 0x723e,
0x5eab, 0x6b21, 0x514b, 0x6642, 0x9593, 0x4f0a, 0x723e, 0x5eab, 0x6b21, 0x514b, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x4f0a, 0x723e, 0x5eab, 0x6b21, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4ee5,
0x8272, 0x5217, 0x6642, 0x9593, 0x4ee5, 0x8272, 0x5217, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4ee5,
0x8272, 0x5217, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x65e5, 0x672c, 0x6642, 0x9593, 0x65e5, 0x672c,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x65e5, 0x672c, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5f7c, 0x5f97,
0x7f85, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b, 0x6642, 0x9593, 0x5f7c, 0x5f97, 0x7f85,
0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5f7c, 0x5f97,
0x7f85, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b, 0x65e5, 0x5149, 0x7bc0, 0x7d04, 0x6642,
0x9593, 0x6771, 0x54c8, 0x85a9, 0x514b, 0x6642, 0x9593, 0x897f, 0x54c8, 0x85a9, 0x514b, 0x6642,
0x9593, 0x97d3, 0x570b, 0x6642, 0x9593, 0x97d3, 0x570b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x97d3,
0x570b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x79d1, 0x65af, 0x745e, 0x6642, 0x9593, 0x514b, 0x62c9,
0x65af, 0x8afe, 0x4e9e, 0x723e, 0x65af, 0x514b, 0x6642, 0x9593, 0x514b, 0x62c9, 0x65af, 0x8afe,
0x4e9e, 0x723e, 0x65af, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x514b, 0x62c9, 0x65af, 0x8afe,
0x4e9e, 0x723e, 0x65af, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5409, 0x723e, 0x5409, 0x65af,
0x6642, 0x9593, 0x862d, 0x5361, 0x6642, 0x9593, 0x840a, 0x6069, 0x7fa4, 0x5cf6, 0x6642, 0x9593,
0x8c6a, 0x52f3, 0x7235, 0x5cf6, 0x6642, 0x9593, 0x8c6a, 0x52f3, 0x7235, 0x5cf6, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x8c6a, 0x52f3, 0x7235, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6fb3, 0x9580,
0x6642, 0x9593, 0x6fb3, 0x9580, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6fb3, 0x9580, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x99ac, 0x52a0, 0x4e39, 0x6642, 0x9593, 0x99ac, 0x52a0, 0x4e39, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x99ac, 0x52a0, 0x4e39, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x99ac, 0x4f86, 0x897f,
0x4e9e, 0x6642, 0x9593, 0x99ac, 0x723e, 0x5730, 0x592b, 0x6642, 0x9593, 0x99ac, 0x53ef, 0x85a9,
0x65af, 0x6642, 0x9593, 0x99ac, 0x7d39, 0x723e, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x6a21, 0x91cc,
0x897f, 0x65af, 0x6642, 0x9593, 0x6a21, 0x91cc, 0x897f, 0x65af, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x6a21, 0x91cc, 0x897f, 0x65af, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x83ab, 0x68ee, 0x6642, 0x9593,
0x58a8, 0x897f, 0x54e5, 0x592a, 0x5e73, 0x6d0b, 0x6642, 0x9593, 0x58a8, 0x897f, 0x54e5, 0x592a,
0x5e73, 0x6d0b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x58a8, 0x897f, 0x54e5, 0x592a, 0x5e73, 0x6d0b,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x70cf, 0x862d, 0x5df4, 0x6258, 0x6642, 0x9593, 0x70cf, 0x862d,
0x5df4, 0x6258, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x70cf, 0x862d, 0x5df4, 0x6258, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x83ab, 0x65af, 0x79d1, 0x6642, 0x9593, 0x83ab, 0x65af, 0x79d1, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x83ab, 0x65af, 0x79d1, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x7dec, 0x7538, 0x6642,
0x9593, 0x8afe, 0x9b6f, 0x6642, 0x9593, 0x5c3c, 0x6cca, 0x723e, 0x6642, 0x9593, 0x65b0, 0x5580,
0x91cc, 0x591a, 0x5c3c, 0x4e9e, 0x6642, 0x9593, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9e,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9e, 0x7fa4, 0x5cf6,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x7d10, 0x897f, 0x862d, 0x6642, 0x9593, 0x7d10, 0x897f, 0x862d,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x7d10, 0x897f, 0x862d, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x7d10,
0x82ac, 0x862d, 0x6642, 0x9593, 0x7d10, 0x82ac, 0x862d, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x7d10,
0x82ac, 0x862d, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x7d10, 0x57c3, 0x5cf6, 0x6642, 0x9593, 0x8afe,
0x798f, 0x514b, 0x5cf6, 0x6642, 0x9593, 0x8afe, 0x798f, 0x514b, 0x5cf6, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x8afe, 0x798f, 0x514b, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x8cbb, 0x723e, 0x5357,
0x591a, 0x20, 0x2d, 0x20, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x6642, 0x9593, 0x8cbb,
0x723e, 0x5357, 0x591a, 0x20, 0x2d, 0x20, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x8cbb, 0x723e, 0x5357, 0x591a, 0x20, 0x2d, 0x20, 0x8fea, 0x8afe,
0x7f85, 0x5c3c, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5317, 0x99ac, 0x91cc, 0x4e9e, 0x7d0d,
0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9e, 0x6642, 0x9593, 0x65b0,
0x897f, 0x4f2f, 0x5229, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x65b0, 0x897f, 0x4f2f, 0x5229,
0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x9102, 0x6728, 0x65af, 0x514b, 0x6642, 0x9593, 0x9102,
0x6728, 0x65af, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x9102, 0x6728, 0x65af, 0x514b, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x5df4, 0x57fa, 0x65af, 0x5766, 0x6642, 0x9593, 0x5df4, 0x57fa, 0x65af,
0x5766, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5df4, 0x57fa, 0x65af, 0x5766, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x5e1b, 0x7409, 0x6642, 0x9593, 0x5df4, 0x5e03, 0x4e9e, 0x7d10, 0x5e7e, 0x5167, 0x4e9e,
0x6642, 0x9593, 0x5df4, 0x62c9, 0x572d, 0x6642, 0x9593, 0x5df4, 0x62c9, 0x572d, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x5df4, 0x62c9, 0x572d, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x79d8, 0x9b6f, 0x6642,
0x9593, 0x79d8, 0x9b6f, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x79d8, 0x9b6f, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x83f2, 0x5f8b, 0x8cd3, 0x6642, 0x9593, 0x83f2, 0x5f8b, 0x8cd3, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x83f2, 0x5f8b, 0x8cd3, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x9cf3, 0x51f0, 0x7fa4, 0x5cf6,
0x6642, 0x9593, 0x8056, 0x76ae, 0x57c3, 0x723e, 0x548c, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5cf6,
0x6642, 0x9593, 0x8056, 0x76ae, 0x57c3, 0x723e, 0x548c, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5cf6,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x8056, 0x76ae, 0x57c3, 0x723e, 0x548c, 0x5bc6, 0x514b, 0x9686,
0x7fa4, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x76ae, 0x7279, 0x80af, 0x6642, 0x9593, 0x6ce2,
0x7d0d, 0x4f69, 0x6642, 0x9593, 0x5e73, 0x58e4, 0x6642, 0x9593, 0x514b, 0x5b5c, 0x52d2, 0x5967,
0x723e, 0x9054, 0x6642, 0x9593, 0x514b, 0x5b5c, 0x52d2, 0x5967, 0x723e, 0x9054, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x514b, 0x5b5c, 0x52d2, 0x5967, 0x723e, 0x9054, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x7559, 0x5c3c, 0x65fa, 0x6642, 0x9593, 0x7f85, 0x745f, 0x62c9, 0x6642, 0x9593, 0x5eab, 0x9801,
0x5cf6, 0x6642, 0x9593, 0x5eab, 0x9801, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5eab, 0x9801,
0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x85a9, 0x99ac, 0x62c9, 0x6642, 0x9593, 0x85a9, 0x99ac,
0x62c9, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x85a9, 0x99ac, 0x62c9, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x85a9, 0x6469, 0x4e9e, 0x6642, 0x9593, 0x85a9, 0x6469, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593,
0x85a9, 0x6469, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x585e, 0x5e2d, 0x723e, 0x6642, 0x9593,
0x65b0, 0x52a0, 0x5761, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x7d22, 0x7f85, 0x9580, 0x7fa4, 0x5cf6,
0x6642, 0x9593, 0x5357, 0x55ac, 0x6cbb, 0x4e9e, 0x6642, 0x9593, 0x8607, 0x5229, 0x5357, 0x6642,
0x9593, 0x662d, 0x548c, 0x57fa, 0x5730, 0x6642, 0x9593, 0x5927, 0x6eaa, 0x5730, 0x6642, 0x9593,
0x53f0, 0x5317, 0x6642, 0x9593, 0x53f0, 0x5317, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x53f0, 0x5317,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x5854, 0x5409, 0x514b, 0x6642, 0x9593, 0x6258, 0x514b, 0x52de,
0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x6771, 0x52a0, 0x6642, 0x9593, 0x6771, 0x52a0, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x6771, 0x52a0, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x695a, 0x514b, 0x5cf6, 0x6642,
0x9593, 0x571f, 0x5eab, 0x66fc, 0x6642, 0x9593, 0x571f, 0x5eab, 0x66fc, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x571f, 0x5eab, 0x66fc, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5410, 0x74e6, 0x9b6f, 0x6642,
0x9593, 0x70cf, 0x62c9, 0x572d, 0x6642, 0x9593, 0x70cf, 0x62c9, 0x572d, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x70cf, 0x62c9, 0x572d, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x70cf, 0x8332, 0x5225, 0x514b,
0x6642, 0x9593, 0x70cf, 0x8332, 0x5225, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x70cf, 0x8332,
0x5225, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x842c, 0x90a3, 0x675c, 0x6642, 0x9593, 0x842c,
0x90a3, 0x675c, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x842c, 0x90a3, 0x675c, 0x590f, 0x4ee4, 0x6642,
0x9593, 0x59d4, 0x5167, 0x745e, 0x62c9, 0x6642, 0x9593, 0x6d77, 0x53c3, 0x5d34, 0x6642, 0x9593,
0x6d77, 0x53c3, 0x5d34, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6d77, 0x53c3, 0x5d34, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x4f0f, 0x723e, 0x52a0, 0x683c, 0x52d2, 0x6642, 0x9593, 0x4f0f, 0x723e, 0x52a0,
0x683c, 0x52d2, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4f0f, 0x723e, 0x52a0, 0x683c, 0x52d2, 0x590f,
0x4ee4, 0x6642, 0x9593, 0x6c83, 0x65af, 0x6258, 0x514b, 0x6642, 0x9593, 0x5a01, 0x514b, 0x5cf6,
0x6642, 0x9593, 0x74e6, 0x5229, 0x65af, 0x548c, 0x5bcc, 0x5716, 0x7d0d, 0x7fa4, 0x5cf6, 0x6642,
0x9593, 0x96c5, 0x5eab, 0x6b21, 0x514b, 0x6642, 0x9593, 0x96c5, 0x5eab, 0x6b21, 0x514b, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x96c5, 0x5eab, 0x6b21, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x8449,
0x5361, 0x6377, 0x7433, 0x5821, 0x6642, 0x9593, 0x8449, 0x5361, 0x6377, 0x7433, 0x5821, 0x6a19,
0x6e96, 0x6642, 0x9593, 0x8449, 0x5361, 0x6377, 0x7433, 0x5821, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x80b2, 0x7a7a, 0x6642, 0x9593, 0x827e, 0x514b, 0x65f6, 0x95f4, 0x827e, 0x514b, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x827e, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x5bcc, 0x6c57, 0x65f6,
0x95f4, 0x4e2d, 0x975e, 0x65f6, 0x95f4, 0x4e1c, 0x975e, 0x65f6, 0x95f4, 0x5357, 0x975e, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x897f, 0x975e, 0x65f6, 0x95f4, 0x897f, 0x975e, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x897f, 0x975e, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x65af, 0x52a0, 0x65f6,
0x95f4, 0x963f, 0x62c9, 0x65af, 0x52a0, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x65af,
0x52a0, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x6728, 0x56fe, 0x65f6, 0x95f4, 0x963f,
0x62c9, 0x6728, 0x56fe, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x6728, 0x56fe, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x4e9a, 0x9a6c, 0x900a, 0x65f6, 0x95f4, 0x4e9a, 0x9a6c, 0x900a, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x4e9a, 0x9a6c, 0x900a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e2d, 0x90e8,
0x65f6, 0x95f4, 0x4e2d, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e2d, 0x90e8, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x4e1c, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x4e1c, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5c71, 0x533a, 0x65f6, 0x95f4, 0x5c71, 0x533a,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5c71, 0x533a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x592a, 0x5e73,
0x6d0b, 0x65f6, 0x95f4, 0x592a, 0x5e73, 0x6d0b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x592a, 0x5e73,
0x6d0b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x7eb3, 0x5fb7, 0x5c14, 0x65f6, 0x95f4, 0x963f,
0x90a3, 0x5e95, 0x6cb3, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x90a3, 0x5e95, 0x6cb3, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x76ae, 0x4e9a, 0x65f6, 0x95f4, 0x963f, 0x76ae, 0x4e9a, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x963f, 0x76ae, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x514b,
0x9676, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x9676, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x514b,
0x9676, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x65f6, 0x95f4, 0x963f,
0x514b, 0x6258, 0x6bd4, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x6258, 0x6bd4, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x4f2f, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x4f2f, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x963f, 0x62c9, 0x4f2f, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x6839,
0x5ef7, 0x65f6, 0x95f4, 0x963f, 0x6839, 0x5ef7, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x6839,
0x5ef7, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x6839, 0x5ef7, 0x897f, 0x90e8, 0x65f6, 0x95f4,
0x963f, 0x6839, 0x5ef7, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x6839, 0x5ef7,
0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e9a, 0x7f8e, 0x5c3c, 0x4e9a, 0x65f6, 0x95f4,
0x4e9a, 0x7f8e, 0x5c3c, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e9a, 0x7f8e, 0x5c3c, 0x4e9a,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5927, 0x897f, 0x6d0b, 0x65f6, 0x95f4, 0x5927, 0x897f, 0x6d0b,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5927, 0x897f, 0x6d0b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3,
0x6d32, 0x4e2d, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e2d, 0x90e8, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x6fb3, 0x6d32, 0x4e2d, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e2d,
0x897f, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e2d, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x6fb3, 0x6d32, 0x4e2d, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x6d32,
0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x4e1c, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x6fb3, 0x6d32, 0x4e1c, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x897f, 0x90e8,
0x65f6, 0x95f4, 0x6fb3, 0x6d32, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6fb3, 0x6d32,
0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e9a, 0x585e, 0x62dc, 0x7136, 0x65f6, 0x95f4,
0x4e9a, 0x585e, 0x62dc, 0x7136, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e9a, 0x585e, 0x62dc, 0x7136,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e9a, 0x901f, 0x5c14, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x4e9a,
0x901f, 0x5c14, 0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e9a, 0x901f, 0x5c14, 0x7fa4,
0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5b5f, 0x52a0, 0x62c9, 0x65f6, 0x95f4, 0x5b5f, 0x52a0,
0x62c9, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5b5f, 0x52a0, 0x62c9, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x4e0d, 0x4e39, 0x65f6, 0x95f4, 0x73bb, 0x5229, 0x7ef4, 0x4e9a, 0x65f6, 0x95f4, 0x5df4, 0x897f,
0x5229, 0x4e9a, 0x65f6, 0x95f4, 0x5df4, 0x897f, 0x5229, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x5df4, 0x897f, 0x5229, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6c76, 0x83b1, 0x65f6, 0x95f4,
0x7ef4, 0x5fb7, 0x89d2, 0x65f6, 0x95f4, 0x7ef4, 0x5fb7, 0x89d2, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x7ef4, 0x5fb7, 0x89d2, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x51ef, 0x897f, 0x7ad9, 0x65f6, 0x95f4,
0x67e5, 0x83ab, 0x6d1b, 0x65f6, 0x95f4, 0x67e5, 0x5766, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x67e5,
0x5766, 0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x67e5, 0x5766, 0x7fa4, 0x5c9b, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x667a, 0x5229, 0x65f6, 0x95f4, 0x667a, 0x5229, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x667a, 0x5229, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e2d, 0x56fd, 0x65f6, 0x95f4, 0x4e2d,
0x56fd, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e2d, 0x56fd, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e54,
0x5df4, 0x5c71, 0x65f6, 0x95f4, 0x4e54, 0x5df4, 0x5c71, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e54,
0x5df4, 0x5c71, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5723, 0x8bde, 0x5c9b, 0x65f6, 0x95f4, 0x79d1,
0x79d1, 0x65af, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x54e5, 0x4f26, 0x6bd4, 0x4e9a, 0x65f6, 0x95f4,
0x54e5, 0x4f26, 0x6bd4, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x54e5, 0x4f26, 0x6bd4, 0x4e9a,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5e93, 0x514b, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x5e93, 0x514b,
0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5e93, 0x514b, 0x7fa4, 0x5c9b, 0x534a, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x53e4, 0x5df4, 0x65f6, 0x95f4, 0x53e4, 0x5df4, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x53e4, 0x5df4, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6234, 0x7ef4, 0x65af, 0x65f6, 0x95f4,
0x675c, 0x8499, 0x675c, 0x6bd4, 0x5c14, 0x65f6, 0x95f4, 0x4e1c, 0x5e1d, 0x6c76, 0x65f6, 0x95f4,
0x590d, 0x6d3b, 0x8282, 0x5c9b, 0x65f6, 0x95f4, 0x590d, 0x6d3b, 0x8282, 0x5c9b, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x590d, 0x6d3b, 0x8282, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5384, 0x74dc,
0x591a, 0x65f6, 0x95f4, 0x4e2d, 0x6b27, 0x65f6, 0x95f4, 0x4e2d, 0x6b27, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x4e2d, 0x6b27, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e1c, 0x6b27, 0x65f6, 0x95f4, 0x4e1c,
0x6b27, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e1c, 0x6b27, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6b27,
0x6d32, 0x8fdc, 0x4e1c, 0x65f6, 0x95f4, 0x897f, 0x6b27, 0x65f6, 0x95f4, 0x897f, 0x6b27, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x897f, 0x6b27, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x798f, 0x514b, 0x5170,
0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x798f, 0x514b, 0x5170, 0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x798f, 0x514b, 0x5170, 0x7fa4, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6590, 0x6d4e,
0x65f6, 0x95f4, 0x6590, 0x6d4e, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6590, 0x6d4e, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x6cd5, 0x5c5e, 0x572d, 0x4e9a, 0x90a3, 0x65f6, 0x95f4, 0x6cd5, 0x56fd, 0x5357,
0x65b9, 0x53ca, 0x5357, 0x6781, 0x65f6, 0x95f4, 0x52a0, 0x62c9, 0x5df4, 0x54e5, 0x7fa4, 0x5c9b,
0x65f6, 0x95f4, 0x7518, 0x6bd4, 0x5c14, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x4e54, 0x6cbb, 0x4e9a,
0x65f6, 0x95f4, 0x4e54, 0x6cbb, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e54, 0x6cbb, 0x4e9a,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5409, 0x5c14, 0x4f2f, 0x7279, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4,
0x683c, 0x6797, 0x5a01, 0x6cbb, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x4e1c,
0x90e8, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x4e1c, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x683c, 0x9675, 0x5170, 0x4e1c, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170,
0x897f, 0x90e8, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x683c, 0x9675, 0x5170, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5173, 0x5c9b,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6ce2, 0x65af, 0x6e7e, 0x6d77, 0x57df, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x76d6, 0x4e9a, 0x90a3, 0x65f6, 0x95f4, 0x590f, 0x5a01, 0x5937, 0x2d, 0x963f, 0x7559,
0x7533, 0x65f6, 0x95f4, 0x590f, 0x5a01, 0x5937, 0x2d, 0x963f, 0x7559, 0x7533, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x590f, 0x5a01, 0x5937, 0x2d, 0x963f, 0x7559, 0x7533, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x9999, 0x6e2f, 0x65f6, 0x95f4, 0x9999, 0x6e2f, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x9999,
0x6e2f, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x79d1, 0x5e03, 0x591a, 0x65f6, 0x95f4, 0x79d1, 0x5e03,
0x591a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x79d1, 0x5e03, 0x591a, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x5370, 0x5ea6, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5370, 0x5ea6, 0x6d0b, 0x65f6, 0x95f4, 0x5370,
0x5ea6, 0x652f, 0x90a3, 0x65f6, 0x95f4, 0x5370, 0x5c3c, 0x4e2d, 0x90e8, 0x65f6, 0x95f4, 0x5370,
0x5c3c, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x5370, 0x5c3c, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x4f0a,
0x6717, 0x65f6, 0x95f4, 0x4f0a, 0x6717, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4f0a, 0x6717, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x4f0a, 0x5c14, 0x5e93, 0x6b21, 0x514b, 0x65f6, 0x95f4, 0x4f0a, 0x5c14,
0x5e93, 0x6b21, 0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4f0a, 0x5c14, 0x5e93, 0x6b21, 0x514b,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4ee5, 0x8272, 0x5217, 0x65f6, 0x95f4, 0x4ee5, 0x8272, 0x5217,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4ee5, 0x8272, 0x5217, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x65e5,
0x672c, 0x65f6, 0x95f4, 0x65e5, 0x672c, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x65e5, 0x672c, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x5f7c, 0x5f97, 0x7f57, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b,
0x65f6, 0x95f4, 0x5f7c, 0x5f97, 0x7f57, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x5f7c, 0x5f97, 0x7f57, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b,
0x65e5, 0x5149, 0x8282, 0x7ea6, 0x65f6, 0x95f4, 0x4e1c, 0x54c8, 0x8428, 0x514b, 0x65f6, 0x95f4,
0x897f, 0x54c8, 0x8428, 0x514b, 0x65f6, 0x95f4, 0x97e9, 0x56fd, 0x65f6, 0x95f4, 0x97e9, 0x56fd,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x97e9, 0x56fd, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x79d1, 0x65af,
0x745e, 0x65f6, 0x95f4, 0x514b, 0x62c9, 0x65af, 0x8bfa, 0x4e9a, 0x5c14, 0x65af, 0x514b, 0x65f6,
0x95f4, 0x514b, 0x62c9, 0x65af, 0x8bfa, 0x4e9a, 0x5c14, 0x65af, 0x514b, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x514b, 0x62c9, 0x65af, 0x8bfa, 0x4e9a, 0x5c14, 0x65af, 0x514b, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x5409, 0x5c14, 0x5409, 0x65af, 0x65f6, 0x95f4, 0x5170, 0x5361, 0x65f6, 0x95f4, 0x83b1,
0x6069, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x8c6a, 0x52cb, 0x7235, 0x5c9b, 0x65f6, 0x95f4, 0x8c6a,
0x52cb, 0x7235, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8c6a, 0x52cb, 0x7235, 0x5c9b, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x95e8, 0x65f6, 0x95f4, 0x6fb3, 0x95e8, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x6fb3, 0x95e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x9a6c, 0x52a0, 0x4e39, 0x65f6, 0x95f4,
0x9a6c, 0x52a0, 0x4e39, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x9a6c, 0x52a0, 0x4e39, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x9a6c, 0x6765, 0x897f, 0x4e9a, 0x65f6, 0x95f4, 0x9a6c, 0x5c14, 0x5730, 0x592b,
0x65f6, 0x95f4, 0x9a6c, 0x53ef, 0x8428, 0x65af, 0x65f6, 0x95f4, 0x9a6c, 0x7ecd, 0x5c14, 0x7fa4,
0x5c9b, 0x65f6, 0x95f4, 0x6a21, 0x91cc, 0x897f, 0x65af, 0x65f6, 0x95f4, 0x6a21, 0x91cc, 0x897f,
0x65af, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6a21, 0x91cc, 0x897f, 0x65af, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x83ab, 0x68ee, 0x65f6, 0x95f4, 0x58a8, 0x897f, 0x54e5, 0x592a, 0x5e73, 0x6d0b, 0x65f6,
0x95f4, 0x58a8, 0x897f, 0x54e5, 0x592a, 0x5e73, 0x6d0b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x58a8,
0x897f, 0x54e5, 0x592a, 0x5e73, 0x6d0b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e4c, 0x5170, 0x5df4,
0x6258, 0x65f6, 0x95f4, 0x4e4c, 0x5170, 0x5df4, 0x6258, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e4c,
0x5170, 0x5df4, 0x6258, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x83ab, 0x65af, 0x79d1, 0x65f6, 0x95f4,
0x83ab, 0x65af, 0x79d1, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x83ab, 0x65af, 0x79d1, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x7f05, 0x7538, 0x65f6, 0x95f4, 0x8bfa, 0x9c81, 0x65f6, 0x95f4, 0x5c3c, 0x6cca,
0x5c14, 0x65f6, 0x95f4, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9a, 0x65f6, 0x95f4, 0x65b0,
0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x65b0, 0x5580, 0x91cc,
0x591a, 0x5c3c, 0x4e9a, 0x7fa4, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7ebd, 0x897f, 0x5170,
0x65f6, 0x95f4, 0x7ebd, 0x897f, 0x5170, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x7ebd, 0x897f, 0x5170,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7ebd, 0x82ac, 0x5170, 0x65f6, 0x95f4, 0x7ebd, 0x82ac, 0x5170,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x7ebd, 0x82ac, 0x5170, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7ebd,
0x57c3, 0x5c9b, 0x65f6, 0x95f4, 0x8bfa, 0x798f, 0x514b, 0x5c9b, 0x65f6, 0x95f4, 0x8bfa, 0x798f,
0x514b, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8bfa, 0x798f, 0x514b, 0x5c9b, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a, 0x20, 0x2d, 0x20, 0x8fea, 0x8bfa, 0x7f57,
0x5c3c, 0x4e9a, 0x65f6, 0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a, 0x20, 0x2d, 0x20, 0x8fea,
0x8bfa, 0x7f57, 0x5c3c, 0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a,
0x20, 0x2d, 0x20, 0x8fea, 0x8bfa, 0x7f57, 0x5c3c, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x5317, 0x9a6c, 0x91cc, 0x4e9a, 0x7eb3, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x65b0, 0x897f, 0x4f2f,
0x5229, 0x4e9a, 0x65f6, 0x95f4, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9a, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x65b0, 0x897f, 0x4f2f, 0x5229, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x9102, 0x6728,
0x65af, 0x514b, 0x65f6, 0x95f4, 0x9102, 0x6728, 0x65af, 0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x9102, 0x6728, 0x65af, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5df4, 0x57fa, 0x65af, 0x5766,
0x65f6, 0x95f4, 0x5df4, 0x57fa, 0x65af, 0x5766, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5df4, 0x57fa,
0x65af, 0x5766, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5e1b, 0x7409, 0x65f6, 0x95f4, 0x5df4, 0x5e03,
0x4e9a, 0x7ebd, 0x51e0, 0x5185, 0x4e9a, 0x65f6, 0x95f4, 0x5df4, 0x62c9, 0x572d, 0x65f6, 0x95f4,
0x5df4, 0x62c9, 0x572d, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5df4, 0x62c9, 0x572d, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x79d8, 0x9c81, 0x65f6, 0x95f4, 0x79d8, 0x9c81, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x79d8, 0x9c81, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x83f2, 0x5f8b, 0x5bbe, 0x65f6, 0x95f4, 0x83f2,
0x5f8b, 0x5bbe, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x83f2, 0x5f8b, 0x5bbe, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x51e4, 0x51f0, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x5723, 0x76ae, 0x57c3, 0x5c14, 0x548c,
0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x5723, 0x76ae, 0x57c3, 0x5c14, 0x548c,
0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5c9b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5723, 0x76ae, 0x57c3,
0x5c14, 0x548c, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x76ae,
0x7279, 0x80af, 0x65f6, 0x95f4, 0x6ce2, 0x7eb3, 0x4f69, 0x65f6, 0x95f4, 0x5e73, 0x58e4, 0x65f6,
0x95f4, 0x514b, 0x5b5c, 0x52d2, 0x5965, 0x5c14, 0x8fbe, 0x65f6, 0x95f4, 0x514b, 0x5b5c, 0x52d2,
0x5965, 0x5c14, 0x8fbe, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x514b, 0x5b5c, 0x52d2, 0x5965, 0x5c14,
0x8fbe, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7559, 0x5c3c, 0x65fa, 0x65f6, 0x95f4, 0x7f57, 0x745f,
0x62c9, 0x65f6, 0x95f4, 0x5e93, 0x9875, 0x5c9b, 0x65f6, 0x95f4, 0x5e93, 0x9875, 0x5c9b, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x5e93, 0x9875, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x8428, 0x9a6c,
0x62c9, 0x65f6, 0x95f4, 0x8428, 0x9a6c, 0x62c9, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8428, 0x9a6c,
0x62c9, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x8428, 0x6469, 0x4e9a, 0x65f6, 0x95f4, 0x8428, 0x6469,
0x4e9a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x8428, 0x6469, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x585e, 0x5e2d, 0x5c14, 0x65f6, 0x95f4, 0x65b0, 0x52a0, 0x5761, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x7d22, 0x7f57, 0x95e8, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x5357, 0x4e54, 0x6cbb, 0x4e9a, 0x65f6,
0x95f4, 0x82cf, 0x5229, 0x5357, 0x65f6, 0x95f4, 0x662d, 0x548c, 0x57fa, 0x5730, 0x65f6, 0x95f4,
0x5927, 0x6eaa, 0x5730, 0x65f6, 0x95f4, 0x53f0, 0x5317, 0x65f6, 0x95f4, 0x53f0, 0x5317, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x53f0, 0x5317, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5854, 0x5409, 0x514b,
0x65f6, 0x95f4, 0x6258, 0x514b, 0x52b3, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x4e1c, 0x52a0, 0x65f6,
0x95f4, 0x4e1c, 0x52a0, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e1c, 0x52a0, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x695a, 0x514b, 0x5c9b, 0x65f6, 0x95f4, 0x571f, 0x5e93, 0x66fc, 0x65f6, 0x95f4, 0x571f,
0x5e93, 0x66fc, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x571f, 0x5e93, 0x66fc, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x5410, 0x74e6, 0x9c81, 0x65f6, 0x95f4, 0x4e4c, 0x62c9, 0x572d, 0x65f6, 0x95f4, 0x4e4c,
0x62c9, 0x572d, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e4c, 0x62c9, 0x572d, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x4e4c, 0x5179, 0x522b, 0x514b, 0x65f6, 0x95f4, 0x4e4c, 0x5179, 0x522b, 0x514b, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x4e4c, 0x5179, 0x522b, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4e07,
0x90a3, 0x675c, 0x65f6, 0x95f4, 0x4e07, 0x90a3, 0x675c, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e07,
0x90a3, 0x675c, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x59d4, 0x5185, 0x745e, 0x62c9, 0x65f6, 0x95f4,
0x6d77, 0x53c2, 0x5d34, 0x65f6, 0x95f4, 0x6d77, 0x53c2, 0x5d34, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x6d77, 0x53c2, 0x5d34, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x4f0f, 0x5c14, 0x52a0, 0x683c, 0x52d2,
0x65f6, 0x95f4, 0x4f0f, 0x5c14, 0x52a0, 0x683c, 0x52d2, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4f0f,
0x5c14, 0x52a0, 0x683c, 0x52d2, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6c83, 0x65af, 0x6258, 0x514b,
0x65f6, 0x95f4, 0x5a01, 0x514b, 0x5c9b, 0x65f6, 0x95f4, 0x74e6, 0x5229, 0x65af, 0x548c, 0x5bcc,
0x56fe, 0x7eb3, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x6b21, 0x514b, 0x65f6, 0x95f4,
0x96c5, 0x5e93, 0x6b21, 0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x6b21, 0x514b,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x53f6, 0x5361, 0x6377, 0x7433, 0x5821, 0x65f6, 0x95f4, 0x53f6,
0x5361, 0x6377, 0x7433, 0x5821, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x53f6, 0x5361, 0x6377, 0x7433,
0x5821, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x80b2, 0x7a7a, 0x65f6, 0x95f4, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x67, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x73, 0x75, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66,
0x72, 0x69, 0x63, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41,
0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e,
0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d,
0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f,
0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72,
0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x75, 0x6e, 0x74,
0x61, 0x6e, 0x79, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x61, 0x6e, 0x79,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x6d, 0x75, 0x6e, 0x74, 0x61, 0x6e, 0x79, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e,
0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x20, 0x64, 0x2019, 0x41, 0x6d,
0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f,
0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63,
0xed, 0x66, 0x69, 0x63, 0x20, 0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x20,
0x64, 0x2019, 0x41, 0x6d, 0xe8, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x70,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0xe0, 0x72, 0x61, 0x62, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0xe0, 0x72, 0x61,
0x62, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0xe0, 0x72, 0x61, 0x62, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0x65, 0x73, 0x74, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0x65,
0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f,
0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x72, 0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x2019, 0x41, 0x72, 0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41,
0x72, 0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x69, 0x63,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0,
0x6e, 0x74, 0x69, 0x63, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74,
0x6c, 0xe0, 0x6e, 0x74, 0x69, 0x63, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x2d,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x2d, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x65, 0x2d, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69,
0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69,
0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x64, 0x6a, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f,
0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x6f, 0x6c, 0xed, 0x76,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72,
0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44,
0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72,
0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x20, 0x56,
0x65, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x20, 0x56,
0x65, 0x72, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x58, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x58,
0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x58, 0x69, 0x6c, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x58, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x58,
0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x58, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61,
0x74, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c,
0xf2, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43,
0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43,
0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x69, 0x67,
0x20, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x71, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x71,
0x75, 0x61, 0x64, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x73, 0x74, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x78, 0x74,
0x72, 0x65, 0x6d, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x65, 0x73, 0x74, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x4f, 0x65, 0x73, 0x74, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x65, 0x73, 0x74,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c,
0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x65, 0x73, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c,
0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x65, 0x73, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73,
0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x47, 0x75, 0x61, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x6e, 0x74, 0xe0, 0x72, 0x74, 0x69, 0x64, 0x61, 0x20, 0x69,
0x20, 0x46, 0x72, 0x61, 0x6e, 0xe7, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x53, 0x75, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47,
0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x47,
0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4d, 0x65, 0x72, 0x69, 0x64, 0x69, 0xe0, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73,
0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0,
0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x65, 0x73, 0x74, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x65, 0x73, 0x74,
0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e,
0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x6f,
0x6c, 0x66, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0xcd, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0x63, 0x65, 0xe0, 0x20,
0xcd, 0x6e, 0x64, 0x69, 0x63, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x78, 0x69, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe8, 0x73, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x65, 0x73, 0x74,
0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe8, 0x73, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0x65,
0x73, 0x74, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe8, 0x73,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x49, 0x72, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x49, 0x72, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72,
0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x4a, 0x61, 0x70, 0xf3, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x61, 0x6d, 0x74, 0x78, 0x61, 0x74, 0x6b, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6d,
0x74, 0x78, 0x61, 0x74, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6d, 0x74, 0x78, 0x61, 0x74,
0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x6f, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x75,
0x69, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20,
0x4c, 0x69, 0x6e, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x72,
0x69, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0xe0, 0x69, 0x73,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c,
0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63,
0xed, 0x66, 0x69, 0x63, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe8, 0x78, 0x69,
0x63, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed,
0x66, 0x69, 0x63, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe8, 0x78, 0x69, 0x63,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63,
0x20, 0x64, 0x65, 0x20, 0x4d, 0xe8, 0x78, 0x69, 0x63, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2,
0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54,
0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c,
0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x69, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x4f,
0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x65, 0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x65, 0x72,
0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c,
0x6c, 0x65, 0x73, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0xed, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0xed, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0xed, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x53, 0x65,
0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xf3,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0xf2, 0x72,
0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x64, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e,
0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61,
0x68, 0x69, 0x74, 0xed, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x54, 0x61, 0x64, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x61, 0x74, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x61, 0x74, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x56, 0x61, 0x6e, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x6e, 0x65, 0xe7, 0x75, 0x65, 0x6c,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x69, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x49, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x49,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x49, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x49, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59,
0x75, 0x6b, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x20, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e,
0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x6f,
0x75, 0x74, 0x68, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61,
0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57,
0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61,
0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x54, 0x61, 0x6d, 0x64,
0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x54, 0x61,
0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61,
0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61,
0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x73, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x54,
0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61,
0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57,
0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e,
0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x54, 0x61, 0x6d,
0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x54, 0x61,
0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e,
0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e,
0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61,
0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c,
0x61, 0x6d, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x61,
0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x54, 0x61, 0x6d, 0x64,
0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e,
0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x54,
0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e,
0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x73, 0x69, 0x6e, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61,
0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x73, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x54, 0x73, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x54,
0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x43,
0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d,
0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x54, 0x61,
0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4b, 0x61, 0x74, 0x75, 0x6e, 0x67,
0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x75, 0x62, 0x61,
0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x20,
0x54, 0x69, 0x6d, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x54, 0x61, 0x6d, 0x64, 0x61,
0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x54,
0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x46, 0x75, 0x72, 0x74, 0x68, 0x65, 0x72, 0x2d, 0x65, 0x61, 0x73,
0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x54, 0x61, 0x6d,
0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61,
0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46, 0x72, 0x65, 0x6e, 0x63,
0x68, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x53,
0x6f, 0x75, 0x74, 0x68, 0x65, 0x72, 0x6e, 0x20, 0x75, 0x67, 0x20, 0x41,
0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f,
0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61,
0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x20, 0x4d, 0x65, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e,
0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61,
0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x54, 0x61,
0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73,
0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e,
0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x75,
0x6c, 0x66, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e,
0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x73, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x54, 0x61,
0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x20,
0x4f, 0x63, 0x65, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x54, 0x61, 0x6d, 0x64,
0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x54,
0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x64, 0x6c, 0x61, 0x77, 0x61,
0x6e, 0x20, 0x73, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x65, 0x73, 0x74, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x54, 0x61, 0x6d,
0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e,
0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x7a, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4c, 0x69, 0x6e, 0x65,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x54, 0x61,
0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x54, 0x61, 0x6d, 0x64, 0x61,
0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x54, 0x61,
0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77,
0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x77, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x79,
0x61, 0x6e, 0x6d, 0x61, 0x72, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e,
0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c,
0x61, 0x6e, 0x64, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x54,
0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61,
0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x69, 0x75,
0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x54,
0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x73, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x54, 0x61, 0x6d, 0x64,
0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x54, 0x61, 0x6d, 0x64, 0x61,
0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x54, 0x61, 0x6d, 0x64, 0x61,
0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x54, 0x61, 0x6d,
0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50,
0x61, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x54, 0x61, 0x6d, 0x64,
0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x65,
0x72, 0x75, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x61, 0x73, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61,
0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x50, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61,
0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x74, 0x2e, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61,
0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x53, 0x74, 0x2e, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69,
0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x52, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x54, 0x61, 0x6d,
0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e,
0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x65, 0x79,
0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x54, 0x61, 0x6d, 0x64, 0x61,
0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43,
0x68, 0x75, 0x75, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73,
0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x75,
0x76, 0x61, 0x6c, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x54, 0x61, 0x6d, 0x64, 0x61,
0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x73, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61,
0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x73, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e,
0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x73, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75,
0x65, 0x6c, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x54, 0x61,
0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x54, 0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73,
0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x57, 0x61, 0x6b,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x26,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x54, 0x61,
0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x73, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x54,
0x61, 0x6d, 0x64, 0x61, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x73,
0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x59,
0x75, 0x6b, 0x6f, 0x6e, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0xd804,
0xdd09, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd34,
0xd804, 0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd1c, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a,
0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd18,
0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804,
0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd07, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b,
0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd1c, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd28, 0xd804, 0xdd07, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804,
0xdd07, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd07, 0x20, 0xd804,
0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804,
0xdd2e, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd11,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804,
0xdd11, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804,
0xdd11, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f,
0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0f,
0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd22, 0xd804,
0xdd33, 0xd804, 0xdd26, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804,
0xdd26, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd26, 0x20, 0xd804,
0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804,
0xdd2e, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0x20, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1e,
0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd27, 0xd804, 0xdd31, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804,
0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd28,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd27, 0xd804, 0xdd31, 0x20, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd27, 0xd804,
0xdd31, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28,
0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27,
0xd804, 0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804,
0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd20,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd18, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804,
0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804,
0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804,
0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd1b, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c,
0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd0e, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804,
0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd28,
0xd804, 0xdd20, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd28, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd26, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20,
0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd33, 0xd804, 0xdd26,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd0f, 0xd804, 0xdd27, 0xd804,
0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd33,
0xd804, 0xdd26, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28,
0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804,
0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804,
0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09,
0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f,
0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804,
0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c,
0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd1b, 0xd804,
0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd27, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f,
0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d,
0xd804, 0xdd2d, 0xd804, 0xdd0e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2d, 0xd804, 0xdd0e, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd1d, 0xd804, 0xdd2d, 0xd804, 0xdd0e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd2c, 0xd804, 0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c, 0xd804, 0xdd0e,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d,
0xd804, 0xdd01, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd01, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd01, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804,
0xdd2c, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd23, 0xd804, 0xdd28,
0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd23,
0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25,
0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd2d, 0x20, 0xd804,
0xdd18, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd23, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd1b,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd18, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd17, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd17, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd17, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd28,
0xd804, 0xdd23, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03,
0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804,
0xdd27, 0xd804, 0xdd20, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd27,
0xd804, 0xdd20, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd27, 0xd804, 0xdd20,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd07,
0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804,
0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804,
0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c,
0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28,
0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804,
0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804,
0xdd1d, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1d, 0x20,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd13, 0xd804, 0xdd2c, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd13, 0xd804, 0xdd2a,
0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd34,
0x2d, 0xd804, 0xdd18, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd27, 0x20, 0xd804, 0xdd05,
0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd23,
0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804,
0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2a,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18,
0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd16,
0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd16,
0xd804, 0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804,
0xdd34, 0xd804, 0xdd19, 0xd804, 0xdd33, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c,
0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804,
0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e,
0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2c,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd19, 0xd804, 0xdd33,
0xd804, 0xdd06, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804,
0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e, 0xd804, 0xdd1d,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804,
0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e,
0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804,
0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2e,
0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27,
0xd804, 0xdd0f, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804,
0xdd2e, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18,
0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20,
0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804,
0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c,
0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd0e,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd28,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd27, 0xd804,
0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd33,
0xd804, 0xdd03, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd28, 0xd804,
0xdd07, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd23, 0xd804, 0xdd1b, 0xd804, 0xdd09, 0xd804, 0xdd2e,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e,
0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804,
0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd11, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd11, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804,
0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a,
0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd0f, 0xd804, 0xdd2c, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804,
0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f,
0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804,
0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804,
0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804,
0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26,
0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804,
0xdd11, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd2d, 0x2d, 0xd804,
0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804,
0xdd2d, 0x2d, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd16, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd27, 0xd804,
0xdd01, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd07, 0xd804,
0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd07,
0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd27,
0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804,
0xdd27, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804,
0xdd27, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804,
0xdd0c, 0xd804, 0xdd29, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd0f,
0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd22, 0xd804,
0xdd33, 0xd804, 0xdd26, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804,
0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804,
0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd2c,
0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804,
0xdd22, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2c, 0xd804,
0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd11,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804,
0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804,
0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd22, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804,
0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd34,
0xd804, 0xdd22, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd20, 0xd804, 0xdd2c,
0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd0e, 0xd804, 0xdd1b, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804,
0xdd1b, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0e, 0xd804, 0xdd1b, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03,
0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804,
0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x2d, 0xd804, 0xdd07, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804,
0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804,
0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x2d, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd11, 0xd804, 0xdd34,
0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd28,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28,
0xd804, 0xdd16, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x2d, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804,
0xdd20, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd0c, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804,
0xdd11, 0xd804, 0xdd34, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2c, 0xd804,
0xdd18, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd0e, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b,
0xd804, 0xdd27, 0xd804, 0xdd0f, 0xd804, 0xdd28, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd18,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd0e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd20,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd2e, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd22, 0xd804, 0xdd33,
0xd804, 0xdd06, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd01, 0xd804,
0xdd07, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd2d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804,
0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26, 0xd804, 0xdd24, 0xd804,
0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd26, 0xd804, 0xdd24, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd26,
0xd804, 0xdd24, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd07, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd13, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd13, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd09, 0xd804, 0xdd13,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd20,
0xd804, 0xdd2c, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd25,
0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd25, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c,
0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804,
0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28,
0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27,
0xd804, 0xdd25, 0xd804, 0xdd33, 0xd804, 0xdd26, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28,
0xd804, 0xdd07, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd07,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd26, 0xd804, 0xdd25, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18, 0xd804,
0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1d, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd1d, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd23, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1d,
0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd27,
0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1a, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1a, 0xd804, 0xdd2c, 0xd804, 0xdd1b, 0xd804, 0xdd23, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804,
0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804,
0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0x20, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804,
0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1f,
0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd07, 0xd804, 0xdd33, 0xd804, 0xdd20,
0xd804, 0xdd23, 0xd804, 0xdd2c, 0xd804, 0xdd13, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd28,
0xd804, 0xdd20, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0e,
0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804,
0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd18,
0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd1c, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804,
0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1c,
0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1c, 0xd804, 0xdd03, 0xd804, 0xdd2a,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd23, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd13, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd03,
0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18,
0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18,
0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1c,
0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804,
0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03,
0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804,
0xdd2e, 0xd804, 0xdd13, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd26, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804,
0xdd2e, 0xd804, 0xdd13, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd26, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd18, 0xd804, 0xdd2e,
0xd804, 0xdd13, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd26, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0xd804,
0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804,
0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804,
0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd2e,
0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd28,
0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804,
0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd1e, 0xd804,
0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd28, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07,
0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd25,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0xd804, 0xdd1f,
0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b,
0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd07, 0xd804,
0xdd28, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd23, 0xd804, 0xdd05, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd1b,
0xd804, 0xdd2a, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804,
0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd20, 0xd804, 0xdd22,
0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd33,
0xd804, 0xdd20, 0xd804, 0xdd22, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804,
0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804,
0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804,
0xdd2a, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f,
0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804,
0xdd2d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd28, 0xd804, 0xdd23,
0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd2d, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1c,
0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd1b, 0xd804, 0xdd2d, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804,
0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd1c, 0xd804, 0xdd2e, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18, 0xd804,
0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd28, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804, 0xdd11, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd2e, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd28,
0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804,
0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28,
0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd20, 0xd804, 0xdd22,
0xd804, 0xdd34, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd2e, 0xd804,
0xdd1a, 0xd804, 0xdd1b, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1b, 0xd804, 0xdd28, 0xd804, 0xdd20,
0xd804, 0xdd27, 0xd804, 0xdd01, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd01, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd23,
0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22, 0xd804,
0xdd34, 0xd804, 0xdd13, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd28, 0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804,
0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd13, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd03, 0xd804,
0xdd28, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd17, 0xd804, 0xdd2c,
0xd804, 0xdd22, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd08, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd08, 0xd804, 0xdd23, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd08, 0xd804, 0xdd23, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd22, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23,
0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1f, 0xd804,
0xdd2e, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd20,
0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27,
0xd804, 0xdd25, 0xd804, 0xdd1f, 0xd804, 0xdd2e, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2c,
0xd804, 0xdd25, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd28, 0xd804, 0xdd01, 0xd804, 0xdd09, 0xd804, 0xdd1b, 0xd804,
0xdd2a, 0xd804, 0xdd22, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd27, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804,
0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d,
0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804, 0xdd18,
0xd804, 0xdd2c, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd18, 0xd804, 0xdd27, 0xd804, 0xdd09, 0xd804, 0xdd28, 0xd804,
0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd0e, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd0e, 0xd804, 0xdd28, 0xd804, 0xdd20, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804, 0xdd2a, 0xd804,
0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd25,
0xd804, 0xdd20, 0xd804, 0xdd2e, 0xd804, 0xdd24, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd26, 0xd804,
0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd28, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2d, 0xd804, 0xdd1b,
0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2d, 0xd804,
0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2d, 0xd804,
0xdd1b, 0xd804, 0xdd33, 0xd804, 0xdd06, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd18, 0xd804, 0xdd28,
0xd804, 0xdd1a, 0xd804, 0xdd2e, 0xd804, 0xdd03, 0xd804, 0xdd23, 0xd804, 0xdd2e, 0xd804, 0xdd22,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd0e, 0xd804, 0xdd08, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd2e,
0xd804, 0xdd07, 0xd804, 0xdd2c, 0xd804, 0xdd23, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804,
0xdd11, 0xd804, 0xdd2e, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd11,
0xd804, 0xdd2e, 0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0x20, 0xd804, 0xdd1f, 0xd804,
0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd11, 0xd804, 0xdd2e,
0xd804, 0xdd0b, 0xd804, 0xdd34, 0xd804, 0xdd09, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd0c, 0xd804, 0xdd2a, 0xd804, 0xdd07,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804,
0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16,
0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f,
0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd28, 0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd1f, 0xd804, 0xdd2c, 0xd804, 0xdd1a, 0xd804, 0xdd28,
0xd804, 0xdd0c, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd11,
0xd804, 0xdd2a, 0xd804, 0xdd1e, 0xd804, 0xdd23, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804,
0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd2a, 0xd804, 0xdd09,
0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804, 0xdd22,
0xd804, 0xdd2a, 0xd804, 0xdd09, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0x20, 0xd804,
0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03,
0xd804, 0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd07,
0xd804, 0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804,
0xdd2a, 0xd804, 0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804,
0xdd28, 0xd804, 0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2a, 0xd804,
0xdd0e, 0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd28, 0xd804,
0xdd16, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd1a, 0xd804, 0xdd2a, 0xd804,
0xdd20, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd1a, 0xd804, 0xdd2a,
0xd804, 0xdd20, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd1a, 0xd804, 0xdd2a,
0xd804, 0xdd20, 0xd804, 0xdd16, 0xd804, 0xdd2a, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd2c, 0xd804, 0xdd1a,
0xd804, 0xdd2c, 0xd804, 0xdd0e, 0xd804, 0xdd2a, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd23,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804,
0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23, 0xd804, 0xdd18,
0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd16,
0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07,
0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1d, 0xd804, 0xdd33, 0xd804, 0xdd23,
0xd804, 0xdd18, 0xd804, 0xdd28, 0xd804, 0xdd1d, 0xd804, 0xdd2e, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd2e, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804,
0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd27,
0xd804, 0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd33,
0xd804, 0xdd22, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804,
0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd27, 0xd804, 0xdd1a,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804,
0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804,
0xdd23, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd2e, 0xd804, 0xdd09, 0xd804, 0xdd33, 0xd804,
0xdd22, 0xd804, 0xdd13, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22,
0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27,
0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804,
0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd1e, 0xd804, 0xdd27, 0xd804, 0xdd25, 0xd804,
0xdd34, 0xd804, 0xdd11, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27,
0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd2e,
0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804,
0xdd2d, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd2d, 0x20, 0xd804, 0xdd1e, 0xd804, 0xdd28, 0xd804,
0xdd18, 0xd804, 0xdd2c, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd24, 0xd804, 0xdd23, 0xd804, 0xdd28, 0xd804, 0xdd25,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd33, 0xd804, 0xdd03, 0x20, 0xd804, 0xdd1c,
0xd804, 0xdd2a, 0xd804, 0xdd11, 0xd804, 0xdd2a, 0xd804, 0xdd1a, 0x20, 0xd804, 0xdd03, 0xd804,
0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd20, 0xd804,
0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804,
0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd20, 0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd16,
0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03,
0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd20,
0xd804, 0xdd07, 0xd804, 0xdd2a, 0xd804, 0xdd16, 0xd804, 0xdd34, 0xd804, 0xdd25, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804,
0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34, 0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804,
0xdd22, 0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0xd804, 0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c,
0xd804, 0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a,
0xd804, 0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09,
0xd804, 0xdd34, 0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804,
0xdd16, 0xd804, 0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804,
0xdd07, 0xd804, 0xdd16, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804,
0xdd34, 0xd804, 0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804,
0xdd34, 0x20, 0xd804, 0xdd1f, 0xd804, 0xdd1a, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34,
0x20, 0xd804, 0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804,
0xdd27, 0xd804, 0xdd03, 0xd804, 0xdd28, 0xd804, 0xdd20, 0xd804, 0xdd2c, 0xd804, 0xdd07, 0xd804,
0xdd16, 0xd804, 0xdd2c, 0xd804, 0xdd22, 0xd804, 0xdd28, 0xd804, 0xdd1a, 0xd804, 0xdd34, 0xd804,
0xdd1d, 0xd804, 0xdd2a, 0xd804, 0xdd22, 0xd804, 0xdd34, 0xd804, 0xdd09, 0xd804, 0xdd34, 0x20,
0xd804, 0xdd09, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd27, 0xd804, 0xdd1f, 0xd804, 0xdd34,
0xd804, 0xdd07, 0xd804, 0xdd23, 0xd804, 0xdd27, 0xd804, 0xdd22, 0xd804, 0xdd34, 0x20, 0xd804,
0xdd03, 0xd804, 0xdd27, 0xd804, 0xdd07, 0xd804, 0xdd34, 0xd804, 0xdd16, 0xd804, 0xdd27, 0x41e,
0x432, 0x445, 0x4c0, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430,
0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x41c, 0x430, 0x43b, 0x445, 0x431,
0x430, 0x43b, 0x435, 0x43d, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x41a,
0x44a, 0x438, 0x43b, 0x431, 0x430, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430,
0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435,
0x43d, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a,
0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a,
0x435, 0x440, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430,
0x43b, 0x435, 0x43d, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41b, 0x430,
0x44c, 0x43c, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x20, 0x28,
0x410, 0x426, 0x428, 0x29, 0x41b, 0x430, 0x44c, 0x43c, 0x43d, 0x438, 0x439, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x20, 0x28, 0x410, 0x426, 0x428, 0x29, 0x41b, 0x430, 0x44c,
0x43c, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x20, 0x28, 0x410, 0x426, 0x428, 0x29,
0x422, 0x438, 0x439, 0x43d, 0x430, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422, 0x438, 0x439, 0x43d, 0x430, 0x20, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422, 0x438, 0x439,
0x43d, 0x430, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x445,
0x430, 0x43d, 0x20, 0x410, 0x43f, 0x438, 0x430, 0x2c, 0x20, 0x421, 0x430, 0x43c,
0x43e, 0x430, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x20, 0x410, 0x43f, 0x438, 0x430, 0x2c, 0x20, 0x421,
0x430, 0x43c, 0x43e, 0x430, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x20, 0x410, 0x43f, 0x438, 0x430, 0x2c, 0x20, 0x421,
0x430, 0x43c, 0x43e, 0x430, 0x421, 0x430, 0x4c0, 0x443, 0x434, 0x438, 0x439, 0x43d,
0x20, 0x4c0, 0x430, 0x44c, 0x440, 0x431, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x421,
0x430, 0x4c0, 0x443, 0x434, 0x438, 0x439, 0x43d, 0x20, 0x4c0, 0x430, 0x44c, 0x440,
0x431, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x43d, 0x430, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x421, 0x430, 0x4c0, 0x443, 0x434, 0x438, 0x439, 0x43d, 0x20, 0x4c0, 0x430, 0x44c,
0x440, 0x431, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x43d, 0x430, 0x43d, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x41c, 0x430, 0x43b, 0x445, 0x431,
0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443,
0x437, 0x435, 0x43d, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x42d, 0x440, 0x43c, 0x430, 0x43b, 0x43e, 0x439, 0x447, 0x43e,
0x44c, 0x42d, 0x440, 0x43c, 0x430, 0x43b, 0x43e, 0x439, 0x447, 0x43e, 0x44c, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x42d, 0x440, 0x43c, 0x430, 0x43b, 0x43e, 0x439, 0x447, 0x43e,
0x44c, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438,
0x43a, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442,
0x438, 0x43a, 0x430, 0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x42e, 0x43a, 0x43a,
0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e,
0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x2c, 0x20, 0x43c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437,
0x435, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440,
0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20,
0x43c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x43c, 0x430, 0x43b, 0x445, 0x431, 0x443,
0x437, 0x435, 0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435,
0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x41c, 0x430,
0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435,
0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x41c, 0x430,
0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434,
0x436, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x437, 0x435, 0x440, 0x431,
0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x437, 0x43e, 0x440,
0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448,
0x410, 0x437, 0x43e, 0x440, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440,
0x435, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x410, 0x437, 0x43e, 0x440,
0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448,
0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x411, 0x440,
0x443, 0x43d, 0x435, 0x439, 0x2d, 0x414, 0x430, 0x440, 0x443, 0x441, 0x441, 0x430,
0x43b, 0x430, 0x43c, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434,
0x435, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435,
0x440, 0x434, 0x435, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x427, 0x438, 0x43b,
0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x427, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x426,
0x438, 0x439, 0x447, 0x43e, 0x44c, 0x426, 0x438, 0x439, 0x447, 0x43e, 0x44c, 0x43d,
0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x426, 0x438, 0x439, 0x447, 0x43e, 0x44c,
0x43d, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441,
0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b,
0x441, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x4c0, 0x438, 0x439, 0x441, 0x430, 0x20, 0x43f,
0x430, 0x439, 0x445, 0x430, 0x43c, 0x430, 0x440, 0x20, 0x432, 0x438, 0x43d, 0x447,
0x443, 0x20, 0x434, 0x435, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435,
0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x438, 0x439, 0x43d, 0x2c, 0x20, 0x433, 0x4c0,
0x2d, 0x43d, 0x430, 0x448, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x41a, 0x443, 0x43a, 0x430, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430,
0x448, 0x41a, 0x443, 0x43a, 0x430, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d,
0x430, 0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x443, 0x43a, 0x430, 0x43d, 0x2c,
0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x443, 0x431,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x443, 0x431, 0x430, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x414,
0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b,
0x44c, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x422,
0x438, 0x43c, 0x43e, 0x440, 0x41c, 0x430, 0x440, 0x445, 0x438, 0x43d, 0x20, 0x433,
0x4c0, 0x430, 0x439, 0x440, 0x435, 0x41c, 0x430, 0x440, 0x445, 0x438, 0x43d, 0x20,
0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430,
0x440, 0x445, 0x438, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42e, 0x43a, 0x43a,
0x44a, 0x435, 0x440, 0x430, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b,
0x435, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x430, 0x43b, 0x438, 0x43d,
0x438, 0x43d, 0x433, 0x440, 0x430, 0x434, 0x435, 0x440, 0x430, 0x20, 0x430, 0x2c,
0x20, 0x41c, 0x438, 0x43d, 0x441, 0x43a, 0x435, 0x440, 0x430, 0x20, 0x430, 0x20,
0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x41c, 0x430, 0x43b, 0x445, 0x431,
0x443, 0x437, 0x435, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424, 0x43e, 0x43b,
0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439,
0x440, 0x435, 0x43d, 0x430, 0x448, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d,
0x434, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430,
0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d,
0x434, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430,
0x448, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x424, 0x438, 0x434, 0x436, 0x438, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x424, 0x438, 0x434, 0x436, 0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424, 0x440, 0x430, 0x43d, 0x446,
0x443, 0x437, 0x438, 0x439, 0x43d, 0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430,
0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x438, 0x439, 0x43d, 0x20, 0x43a,
0x44a, 0x438, 0x43b, 0x431, 0x430, 0x20, 0x430, 0x2c, 0x20, 0x410, 0x43d, 0x442,
0x430, 0x440, 0x43a, 0x442, 0x438, 0x434, 0x430, 0x43d, 0x20, 0x430, 0x20, 0x445,
0x430, 0x43d, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x430,
0x43d, 0x20, 0x433, 0x4c0, 0x430, 0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x413,
0x430, 0x43c, 0x431, 0x44c, 0x435, 0x413, 0x443, 0x44c, 0x440, 0x436, 0x438, 0x439,
0x447, 0x43e, 0x44c, 0x413, 0x443, 0x44c, 0x440, 0x436, 0x438, 0x439, 0x447, 0x43e,
0x44c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x443, 0x44c, 0x440, 0x436, 0x438, 0x439,
0x447, 0x43e, 0x44c, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442,
0x430, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x413, 0x440,
0x438, 0x43d, 0x432, 0x438, 0x447, 0x438, 0x446, 0x430, 0x20, 0x44e, 0x43a, 0x44a,
0x430, 0x440, 0x430, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431,
0x430, 0x43b, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x2c, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430,
0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437,
0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x443, 0x437, 0x435,
0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x413, 0x4c0, 0x430, 0x436, 0x430, 0x440, 0x438, 0x439, 0x43d, 0x20, 0x430,
0x439, 0x43c, 0x430, 0x413, 0x430, 0x439, 0x430, 0x43d, 0x430, 0x413, 0x430, 0x432,
0x430, 0x439, 0x43d, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x438, 0x439, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x413, 0x430, 0x432, 0x430, 0x439, 0x43d, 0x2d, 0x430,
0x43b, 0x435, 0x443, 0x442, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x430,
0x432, 0x430, 0x439, 0x43d, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x438, 0x439,
0x43d, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x413, 0x43e, 0x43d,
0x43a, 0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x43e, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x425, 0x4c0, 0x438, 0x43d, 0x434,
0x438, 0x418, 0x43d, 0x434, 0x438, 0x43d, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x42e, 0x43a, 0x43a, 0x44a, 0x435, 0x440, 0x430, 0x20, 0x418, 0x43d, 0x434, 0x43e,
0x43d, 0x435, 0x437, 0x438, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435,
0x43d, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x41c, 0x430,
0x43b, 0x445, 0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x418, 0x43d, 0x434, 0x43e,
0x43d, 0x435, 0x437, 0x438, 0x413, 0x4c0, 0x430, 0x436, 0x430, 0x440, 0x438, 0x439,
0x447, 0x43e, 0x44c, 0x413, 0x4c0, 0x430, 0x436, 0x430, 0x440, 0x438, 0x439, 0x447,
0x43e, 0x44c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x413, 0x4c0, 0x430, 0x436, 0x430, 0x440,
0x438, 0x439, 0x447, 0x43e, 0x44c, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x418, 0x440, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42f,
0x43f, 0x43e, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42f, 0x43f, 0x43e, 0x43d, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41c, 0x430, 0x43b, 0x445, 0x431, 0x430, 0x43b, 0x435, 0x43d, 0x20, 0x41a,
0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x41c, 0x430, 0x43b, 0x445,
0x431, 0x443, 0x437, 0x435, 0x43d, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441,
0x442, 0x430, 0x43d, 0x41a, 0x43e, 0x440, 0x435, 0x439, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x41a, 0x43e, 0x440, 0x435, 0x439, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x43e, 0x441, 0x440, 0x430,
0x44d, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441,
0x43a, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41b, 0x430, 0x439, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d,
0x43d, 0x430, 0x448, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x430, 0x43b,
0x44c, 0x434, 0x438, 0x432, 0x430, 0x448, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437,
0x430, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41c, 0x430,
0x440, 0x448, 0x430, 0x43b, 0x430, 0x43d, 0x20, 0x2c, 0x20, 0x433, 0x4c0, 0x2d,
0x43d, 0x430, 0x448, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422,
0x438, 0x439, 0x43d, 0x430, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x430, 0x43d,
0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x422, 0x438, 0x439, 0x43d, 0x430, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x430, 0x43d, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445,
0x430, 0x43d, 0x422, 0x438, 0x439, 0x43d, 0x430, 0x20, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x430, 0x43d, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e,
0x440, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41c, 0x44c, 0x44f,
0x43d, 0x43c, 0x430, 0x41a, 0x435, 0x440, 0x43b, 0x430, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x41a, 0x435, 0x440, 0x43b, 0x430, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a,
0x435, 0x440, 0x43b, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d,
0x438, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41a, 0x435, 0x440, 0x43b, 0x430, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x41a, 0x435, 0x440, 0x43b, 0x430, 0x20, 0x417, 0x435,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41a, 0x435, 0x440,
0x43b, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41d, 0x438, 0x443, 0x44d, 0x41d, 0x43e,
0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d,
0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c,
0x44f, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438,
0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d,
0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41d, 0x43e, 0x432,
0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41e, 0x43c, 0x441,
0x43a, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c,
0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20, 0x2013, 0x20, 0x41a, 0x435, 0x440,
0x43b, 0x430, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435, 0x439, 0x41f, 0x430, 0x440,
0x430, 0x433, 0x432, 0x430, 0x439, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41f, 0x430, 0x440,
0x430, 0x433, 0x432, 0x430, 0x439, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41f, 0x435, 0x440, 0x443, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20,
0x445, 0x430, 0x43d, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424, 0x438, 0x43b,
0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x430, 0x448, 0x424, 0x438, 0x43b, 0x438, 0x43f,
0x43f, 0x438, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424, 0x438, 0x43b,
0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x430, 0x448, 0x2c, 0x20, 0x430, 0x44c, 0x445,
0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x424, 0x435, 0x43d,
0x438, 0x43a, 0x441, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x421,
0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x430, 0x2c, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x430, 0x421, 0x435, 0x43d, 0x2d,
0x41f, 0x44c, 0x435, 0x440, 0x20, 0x430, 0x2c, 0x20, 0x41c, 0x438, 0x43a, 0x435,
0x43b, 0x43e, 0x43d, 0x20, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x421, 0x435, 0x43d,
0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x430, 0x2c, 0x20, 0x41c, 0x438, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x41f, 0x438, 0x442, 0x43a,
0x44d, 0x440, 0x43d, 0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x2c, 0x20, 0x433,
0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x420, 0x435, 0x44e, 0x43d, 0x44c, 0x43e, 0x43d,
0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x421, 0x430, 0x43c, 0x43e,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20,
0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x430, 0x43d, 0x20, 0x433, 0x4c0, 0x430,
0x439, 0x440, 0x435, 0x43d, 0x430, 0x448, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e,
0x43d, 0x430, 0x43d, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430, 0x448, 0x41a,
0x44a, 0x438, 0x43b, 0x431, 0x430, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x438,
0x422, 0x430, 0x438, 0x442, 0x438, 0x2c, 0x20, 0x433, 0x4c0, 0x2d, 0x43d, 0x430,
0x448, 0x422, 0x430, 0x439, 0x432, 0x430, 0x43d, 0x44c, 0x422, 0x430, 0x439, 0x432,
0x430, 0x43d, 0x44c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x422, 0x430, 0x439, 0x432, 0x430,
0x43d, 0x44c, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d,
0x20, 0x445, 0x430, 0x43d, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430,
0x43d, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a,
0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x427, 0x443, 0x443, 0x43a,
0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d,
0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x43d, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x423, 0x440,
0x443, 0x433, 0x432, 0x430, 0x439, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x439, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x423, 0x437, 0x431,
0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x430, 0x43d, 0x20, 0x430, 0x44c,
0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x412, 0x430, 0x43d,
0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x412, 0x435, 0x43d, 0x435, 0x441, 0x443,
0x44d, 0x43b, 0x430, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x412, 0x43e, 0x43b, 0x433,
0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435,
0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x423, 0x44d, 0x439, 0x43a, 0x2c,
0x20, 0x433, 0x4c0, 0x2d, 0x435, 0x423, 0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x20,
0x430, 0x2c, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x20, 0x430, 0x42f,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x42f, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x2c, 0x20, 0x430, 0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430,
0x43d, 0x20, 0x445, 0x430, 0x43d, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438,
0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x415, 0x43a, 0x430,
0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x430,
0x44c, 0x445, 0x43a, 0x435, 0x43d, 0x430, 0x43d, 0x20, 0x445, 0x430, 0x43d, 0x13a0,
0x13eb, 0x13a8, 0x13c2, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a0, 0x13f0, 0x13df, 0x20, 0x13ac, 0x13bf, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13ac, 0x13bf, 0x13a8, 0x13cd,
0x13db, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e7, 0x13a6, 0x13be, 0x13ee, 0x20,
0x13ac, 0x13bf, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13ac, 0x13bf,
0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5,
0x13ac, 0x20, 0x13ac, 0x13bf, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20,
0x13ac, 0x13bf, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13b3, 0x13cd, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a0, 0x13b3, 0x13cd, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b3, 0x13cd, 0x13a6, 0x20, 0x13aa, 0x13af, 0x20,
0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13ba, 0x13cc, 0x13c2,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13ba, 0x13cc, 0x13c2, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13ba,
0x13cc, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13f0, 0x13df, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13f0, 0x13df, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7,
0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3, 0x13d3, 0x13b8, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a3, 0x13d3, 0x13b8, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3, 0x13d3, 0x13b8, 0x20, 0x13aa, 0x13af, 0x20,
0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5,
0x13b5, 0x13ac, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13c8, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13c8, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13c8, 0x13a0, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b4, 0x13c8, 0x13ef, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b4, 0x13c8, 0x13ef, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b4, 0x13c8, 0x13ef, 0x20,
0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0, 0x20,
0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac,
0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20,
0x13a0, 0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7,
0x13dc, 0x20, 0x13a0, 0x13e5, 0x13c2, 0x13d8, 0x13c2, 0x13a0, 0x20, 0x13aa, 0x13a9, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13b5, 0x13bb, 0x13c2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b5,
0x13bb, 0x13c2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13b5, 0x13bb, 0x13c2, 0x13a0, 0x20, 0x13aa, 0x13a9, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20,
0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13f0, 0x13df, 0x20, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df,
0x20, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a1, 0x13b3, 0x13d7,
0x13dc, 0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20,
0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a1,
0x13b3, 0x13d7, 0x13dc, 0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20,
0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a1,
0x13b3, 0x13d7, 0x13dc, 0x20, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13d7, 0x13a7,
0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a1, 0x13b3, 0x13d7, 0x13dc, 0x20, 0x13d7, 0x13a7,
0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13cf, 0x13b5, 0x13c6, 0x13cc, 0x13c2, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13cf, 0x13b5, 0x13c6, 0x13cc, 0x13c2, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13cf, 0x13b5, 0x13c6, 0x13cc, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13d0, 0x13b4, 0x13cf, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a0, 0x13d0, 0x13b4, 0x13cf, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d0, 0x13b4, 0x13cf, 0x20, 0x13aa, 0x13a9, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13c2, 0x13a6, 0x13b5, 0x13d5, 0x13cd, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13c2, 0x13a6, 0x13b5, 0x13d5, 0x13cd, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6,
0x13c2, 0x13a6, 0x13b5, 0x13d5, 0x13cd, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13ca, 0x13d4, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c9,
0x13b5, 0x13eb, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13cf, 0x13b5,
0x13ef, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13cf, 0x13b5, 0x13ef, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6,
0x13cf, 0x13b5, 0x13ef, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13ca, 0x13be, 0x13a2, 0x20, 0x13d3, 0x13b7, 0x13cc, 0x13b3, 0x13bb, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13ac, 0x13be, 0x13d5, 0x13be, 0x20, 0x13a2, 0x13e4, 0x13f3,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13ac, 0x13be, 0x13d5,
0x13be, 0x20, 0x13a2, 0x13e4, 0x13f3, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13ac, 0x13be, 0x13d5, 0x13be,
0x20, 0x13a2, 0x13e4, 0x13f3, 0x13cd, 0x13d7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13bc, 0x13b6, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13dd, 0x13bb, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13dd, 0x13bb, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13dd, 0x13bb, 0x20, 0x13aa, 0x13af,
0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e5, 0x13b5, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e5, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e5, 0x13b5, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d3, 0x13b6, 0x13c2, 0x13a8, 0x13cd, 0x13db,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d3, 0x13b6, 0x13c2, 0x13a8, 0x13cd, 0x13db,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13d3, 0x13b6, 0x13c2, 0x13a8, 0x13cd, 0x13db, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e6, 0x13f1, 0x13c6, 0x13b5, 0x13cc, 0x13c2,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e6, 0x13f1, 0x13c6, 0x13b5, 0x13cc, 0x13c2,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13e6, 0x13f1, 0x13c6, 0x13b5, 0x13cc, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13d3, 0x13c2, 0x13cd, 0x13d3, 0x13f2, 0x13af, 0x13b2, 0x20, 0x13a4,
0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13aa,
0x13cd, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13aa, 0x13b8, 0x13bb, 0x13c8, 0x13a2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13aa, 0x13b8, 0x13bb, 0x13c8, 0x13a2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13b8, 0x13bb, 0x13c8, 0x13a2,
0x13a0, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d3,
0x13cd, 0x13d3, 0x13f4, 0x13b2, 0x13cd, 0x13a9, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d3, 0x13cd, 0x13d3, 0x13f4, 0x13b2,
0x13cd, 0x13a9, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13d3, 0x13cd, 0x13d3,
0x13f4, 0x13b2, 0x13cd, 0x13a9, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0,
0x13f0, 0x13df, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ab,
0x13c6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ab, 0x13c6, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ab, 0x13c6, 0x20,
0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d5,
0x13eb, 0x13cf, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13da, 0x13bc, 0x13be, 0x13d8,
0x2d, 0x13d7, 0x2019, 0x13a4, 0x13b5, 0x13eb, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d8, 0x13bc, 0x13b5, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13e5, 0x13cc, 0x13d5, 0x13b4, 0x13af, 0x13cc, 0x13c5, 0x20, 0x13a4,
0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e5, 0x13cc,
0x13d5, 0x13b4, 0x13af, 0x13cc, 0x13c5, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e5,
0x13cc, 0x13d5, 0x13b4, 0x13af, 0x13cc, 0x13c5, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a1, 0x13c6, 0x13d9,
0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13f3,
0x13b3, 0x13c8, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20,
0x13f3, 0x13b3, 0x13c8, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20,
0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7,
0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20,
0x13f3, 0x13b3, 0x13c8, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5,
0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3,
0x13c8, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13f3, 0x13b3, 0x13c8,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9, 0x13a9, 0x20,
0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9,
0x13a9, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9, 0x13a9, 0x20, 0x13da, 0x13a6,
0x13da, 0x13db, 0x13a2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13eb, 0x13e5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13eb, 0x13e5, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13eb, 0x13e5,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13a6,
0x13b8, 0x20, 0x13c8, 0x13ef, 0x13be, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0,
0x13c2, 0x13a6, 0x13b8, 0x13e5, 0x20, 0x13a4, 0x13a6, 0x13c3, 0x13ee, 0x20, 0x26, 0x20,
0x13a4, 0x13c1, 0x13cd, 0x13d3, 0x13b6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a1,
0x13c6, 0x20, 0x13d3, 0x13a6, 0x13cf, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a6, 0x13bb, 0x13c7, 0x13b5, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13a0, 0x13e5, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13e3, 0x13a0, 0x13e5, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13a0, 0x13e5, 0x13a2, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a9, 0x13b5, 0x13c7, 0x13d8, 0x20, 0x13da,
0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4,
0x20, 0x13a2, 0x13f3, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7,
0x13a7, 0x13b8, 0x13ac, 0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20, 0x13a0, 0x13b5,
0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db,
0x13f1, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13b5, 0x13a2, 0x13b5,
0x13d2, 0x13d7, 0x13a7, 0x13b8, 0x13ac, 0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20,
0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac,
0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20, 0x13a0, 0x13b5, 0x13a2, 0x13b5, 0x13d2,
0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13b5, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5,
0x13b5, 0x13ac, 0x20, 0x13a2, 0x13e4, 0x13cd, 0x13db, 0x13f1, 0x20, 0x13aa, 0x13a9, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a1, 0x13c9, 0x13c4, 0x13b8, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a6, 0x13ef,
0x13be, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ad, 0x13e9, 0x13f1, 0x2d, 0x13a0,
0x13b5, 0x13f3, 0x13cf, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ad,
0x13e9, 0x13f1, 0x2d, 0x13a0, 0x13b5, 0x13f3, 0x13cf, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df,
0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ad, 0x13e9, 0x13f1,
0x2d, 0x13a0, 0x13b5, 0x13f3, 0x13cf, 0x13a0, 0x13c2, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2,
0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b0, 0x13c2, 0x13a9, 0x20, 0x13aa,
0x13c2, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b0, 0x13c2, 0x13a9, 0x20,
0x13aa, 0x13c2, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13b0, 0x13c2, 0x13a9, 0x20, 0x13aa, 0x13c2, 0x13a9, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b0, 0x13e9, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b0, 0x13e9, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b0, 0x13e9, 0x13d7, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13c2, 0x13d7, 0x13a2, 0x13a0,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a0, 0x13c2, 0x13f4, 0x13eb, 0x13ef, 0x20, 0x13a0, 0x13ba, 0x13c9, 0x13af, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13c2, 0x13d9, 0x13d3, 0x13b6, 0x13c2, 0x13a8, 0x13cd,
0x13db, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13f0, 0x13df, 0x20, 0x13a2,
0x13c2, 0x13d9, 0x13c2, 0x13cd, 0x13ef, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7,
0x13a7, 0x13b8, 0x13ac, 0x20, 0x13d7, 0x13dc, 0x20, 0x13a2, 0x13c2, 0x13d9, 0x13c2, 0x13cd,
0x13ef, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20,
0x13d7, 0x13dc, 0x20, 0x13a2, 0x13c2, 0x13d9, 0x13c2, 0x13cd, 0x13ef, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13b3, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a2, 0x13b3, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13b3, 0x13c2, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13ab, 0x13e5, 0x13a7,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13cf, 0x13b5,
0x13f1, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13cf, 0x13b5, 0x13f1, 0x20,
0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2,
0x13cf, 0x13b5, 0x13f1, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a9, 0x13e3, 0x13e9, 0x13c2, 0x13cf, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13e3, 0x13e9, 0x13c2, 0x13cf, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e3, 0x13e9, 0x13c2, 0x13cf, 0x20, 0x13aa,
0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d7, 0x13a7,
0x13b8, 0x13ac, 0x20, 0x13a7, 0x13ce, 0x13a7, 0x13cd, 0x13d5, 0x13c2, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a7, 0x13ce, 0x13a7, 0x13cd,
0x13d5, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13b5, 0x13a0, 0x13c2,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13b5, 0x13a0, 0x13c2, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13aa, 0x13b5,
0x13a0, 0x13c2, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13aa, 0x13cd, 0x13b4, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13dd,
0x13cd, 0x13c3, 0x13ef, 0x13cd, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13dd,
0x13cd, 0x13c3, 0x13ef, 0x13cd, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13dd, 0x13cd, 0x13c3, 0x13ef, 0x13cd, 0x13a7, 0x20,
0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a9, 0x13b5, 0x13e3, 0x13a2,
0x13cd, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13cd, 0x13d3, 0x13c5, 0x13c5,
0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a4, 0x13ac, 0x13eb, 0x13f3, 0x13af, 0x20, 0x13ad, 0x13eb, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a4, 0x13ac, 0x13eb, 0x13f3, 0x13af, 0x20, 0x13ad, 0x13eb, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13ac,
0x13eb, 0x13f3, 0x13af, 0x20, 0x13ad, 0x13eb, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13a6, 0x13d3, 0x13c2, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13a6, 0x13d3, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13a6, 0x13d3, 0x13c2,
0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13b4, 0x13cf,
0x13a2, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13b5, 0x13d7, 0x13eb,
0x13cd, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13b5, 0x13a8, 0x13cc, 0x13cf,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13cc, 0x13b5, 0x20, 0x13da, 0x13a6,
0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13bc, 0x13b5, 0x13cf,
0x13a5, 0x13cd, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13bc, 0x13b5, 0x13cf, 0x13a5,
0x13cd, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13bc, 0x13b5, 0x13cf, 0x13a5, 0x13cd, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13cc, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a0, 0x13c2, 0x13cd, 0x13c6, 0x13c2, 0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13cd, 0x13c6, 0x13c2, 0x20, 0x13ed, 0x13d5,
0x13b5, 0x13ac, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13cd, 0x13c6, 0x13c2, 0x20, 0x13ed, 0x13d5, 0x13b5, 0x13ac,
0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a4, 0x13b3, 0x13c2, 0x20, 0x13c6, 0x13d9, 0x13b8, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a4, 0x13b3, 0x13c2, 0x20, 0x13c6, 0x13d9, 0x13b8, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13b3, 0x13c2, 0x20,
0x13c6, 0x13d9, 0x13b8, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13b9, 0x13cd, 0x13a6, 0x13eb, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13cd,
0x13a6, 0x13eb, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13b9, 0x13cd, 0x13a6, 0x13eb, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13b9, 0x13ef, 0x13b9, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13be, 0x13b7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c1, 0x13c6, 0x13b5,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20, 0x13a7, 0x13b5, 0x13d9,
0x13c2, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20,
0x13a7, 0x13b5, 0x13d9, 0x13c2, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20, 0x13a7, 0x13b5, 0x13d9,
0x13c2, 0x13a0, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a2, 0x13e4, 0x20, 0x13cf, 0x13a2, 0x13b4, 0x13c2, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x20, 0x13cf, 0x13a2, 0x13b4, 0x13c2, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4,
0x20, 0x13cf, 0x13a2, 0x13b4, 0x13c2, 0x13d7, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x13a4, 0x13c2, 0x13e9, 0x13db,
0x13d3, 0x13a6, 0x13d9, 0x13af, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4,
0x13a4, 0x13c2, 0x13e9, 0x13db, 0x13d3, 0x13a6, 0x13d9, 0x13af, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a2, 0x13e4, 0x13a4, 0x13c2,
0x13e9, 0x13db, 0x13d3, 0x13a6, 0x13d9, 0x13af, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c2, 0x13f3, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13c3, 0x13b5, 0x13ec, 0x13b5, 0x13a9, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db,
0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c3, 0x13b5, 0x13ec, 0x13b5, 0x13a9,
0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c3, 0x13b5, 0x13ec, 0x13b5, 0x13a9, 0x20,
0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13ea, 0x13be, 0x13c5, 0x13d9, 0x20, 0x13d5, 0x20, 0x13c3, 0x13b6, 0x13be,
0x13ad, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ea, 0x13be, 0x13c5, 0x13d9, 0x20,
0x13d5, 0x20, 0x13c3, 0x13b6, 0x13be, 0x13ad, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ea, 0x13be, 0x13c5, 0x13d9, 0x20, 0x13d5,
0x20, 0x13c3, 0x13b6, 0x13be, 0x13ad, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13c3, 0x13ec, 0x13cf, 0x13c8, 0x13cd, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13c3, 0x13ec, 0x13cf, 0x13c8, 0x13cd, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c3, 0x13ec, 0x13cf, 0x13c8,
0x13cd, 0x13a7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3,
0x13bb, 0x13cd, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a3, 0x13bb, 0x13cd,
0x13a7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13a3, 0x13bb, 0x13cd, 0x13a7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13c6, 0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13c6, 0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20,
0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13b7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13c7, 0x20, 0x13a2, 0x13e4, 0x20, 0x13a9, 0x13a2,
0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13b5, 0x13c7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13b5, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c6, 0x13b5, 0x13c7, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c7, 0x13b7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13c7, 0x13b7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c7, 0x13b7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13c8, 0x13b5, 0x13a9, 0x13c3, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2, 0x13c8, 0x13b5, 0x13a9, 0x13c3, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a0, 0x13c2,
0x13c8, 0x13b5, 0x13a9, 0x13c3, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13e7, 0x13b4, 0x13af, 0x13cc, 0x13c5, 0x13af, 0x20, 0x13da, 0x13a6, 0x13da, 0x13db,
0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20,
0x13c8, 0x13f0, 0x20, 0x26, 0x20, 0x13bb, 0x13c7, 0x13b6, 0x13c2, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13c8, 0x13f0, 0x20, 0x26,
0x20, 0x13bb, 0x13c7, 0x13b6, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13d3, 0x13c5, 0x13d8, 0x20, 0x13c8, 0x13f0,
0x20, 0x26, 0x20, 0x13bb, 0x13c7, 0x13b6, 0x13c2, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2,
0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c8, 0x13a7, 0x13b5, 0x13c2, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13c9, 0x13be, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13c8, 0x13ef, 0x13c2, 0x13a9, 0x13f0, 0x13c2, 0x13a9, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13b4, 0x13f3, 0x13c2, 0x13a0, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13b3, 0x13de, 0x13b3, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cc,
0x13a7, 0x13b5, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cc, 0x13a7, 0x13b5,
0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13cc, 0x13a7, 0x13b5, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13cc, 0x13bc, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13cc,
0x13bc, 0x13a0, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13cc, 0x13bc, 0x13a0, 0x20, 0x13aa, 0x13af, 0x20, 0x13a2, 0x13a6, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ce, 0x13e4, 0x13b4, 0x13cf, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13cf, 0x13c2, 0x13a6, 0x13c9, 0x13b5, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d0, 0x13b6, 0x13b9, 0x13c2,
0x20, 0x13da, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13e7, 0x13a6, 0x13be, 0x13ee, 0x20, 0x13e3, 0x13a0, 0x13e5, 0x13a2, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13d2, 0x13b5, 0x13be, 0x13bb, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13cf, 0x13f2, 0x13e9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d4, 0x13af,
0x13d8, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d4, 0x13f1, 0x13c7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d4, 0x13f1, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d4, 0x13f1, 0x13c7, 0x20, 0x13aa,
0x13af, 0x20, 0x13a2, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d4, 0x13e5,
0x13a9, 0x13cd, 0x13d5, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d9, 0x13a8,
0x13b3, 0x13a4, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d9, 0x13be, 0x13a6, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d9, 0x13be, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13b6,
0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13d9, 0x13be, 0x13a6, 0x20,
0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e7, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13db, 0x13b5, 0x13a9, 0x13ba, 0x13c2, 0x13cd, 0x13d4, 0x13c2,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13db, 0x13b5, 0x13a9, 0x13ba, 0x13c2, 0x13cd,
0x13d4, 0x13c2, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13db, 0x13b5, 0x13a9, 0x13ba, 0x13c2, 0x13cd, 0x13d4, 0x13c2, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13da, 0x13e9, 0x13b7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f3, 0x13b7, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13f3, 0x13b7, 0x13c7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f3, 0x13b7, 0x13c7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13cd, 0x13c7, 0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13cd, 0x13c7, 0x13a9, 0x13cd, 0x13d6, 0x13c2,
0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13a4, 0x13cd, 0x13c7, 0x13a9, 0x13cd, 0x13d6, 0x13c2, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9, 0x13c4, 0x13e9, 0x13da, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13e9, 0x13c4, 0x13e9, 0x13da, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13e9, 0x13c4, 0x13e9, 0x13da, 0x20, 0x13aa,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ea, 0x13c1, 0x13d1, 0x13ea, 0x13b3,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13b3, 0x13d7, 0x13ec, 0x13cd, 0x13d9,
0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ed, 0x13b3, 0x13d7, 0x13ec, 0x13cd,
0x13d9, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x20, 0x13a0, 0x13df, 0x13a2,
0x13b5, 0x13d2, 0x13ed, 0x13b3, 0x13d7, 0x13ec, 0x13cd, 0x13d9, 0x13a9, 0x20, 0x13aa, 0x13a9,
0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ec, 0x13b6, 0x13aa, 0x13dd, 0x13d7, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ec, 0x13b6, 0x13aa, 0x13dd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ec, 0x13b6,
0x13aa, 0x13dd, 0x13d7, 0x20, 0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2,
0x13ec, 0x13cd, 0x13d9, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13a4, 0x13f0,
0x13e8, 0x20, 0x13a4, 0x13a6, 0x13da, 0x13db, 0x13a2, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5,
0x13d2, 0x13e9, 0x13b5, 0x13cd, 0x20, 0x13a0, 0x13b4, 0x20, 0x13ca, 0x13da, 0x13be, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ef, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13a0, 0x13df,
0x13a2, 0x13b5, 0x13d2, 0x13ef, 0x13ab, 0x13e5, 0x13a7, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd,
0x13d7, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13ef, 0x13ab, 0x13e5, 0x13a7, 0x20,
0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f0, 0x13a7, 0x13d6, 0x13b5,
0x13c2, 0x13ca, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f0, 0x13a7, 0x13d6,
0x13b5, 0x13c2, 0x13ca, 0x13a6, 0x20, 0x13a0, 0x13df, 0x13b6, 0x13cd, 0x13d7, 0x20, 0x13a0,
0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f0, 0x13a7, 0x13d6, 0x13b5, 0x13c2, 0x13ca, 0x13a6, 0x20,
0x13aa, 0x13a9, 0x20, 0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x13f3, 0x13a7, 0x13c2, 0x20,
0x13a0, 0x13df, 0x13a2, 0x13b5, 0x13d2, 0x963f, 0x514b, 0x91cc, 0x65f6, 0x95f4, 0x963f, 0x514b,
0x91cc, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x91cc, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x4e2d, 0x90e8, 0x975e, 0x6d32, 0x65f6, 0x95f4, 0x4e1c, 0x90e8, 0x975e, 0x6d32, 0x65f6, 0x95f4,
0x897f, 0x90e8, 0x975e, 0x6d32, 0x65f6, 0x95f4, 0x897f, 0x90e8, 0x975e, 0x6d32, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x897f, 0x90e8, 0x975e, 0x6d32, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5317, 0x7f8e,
0x4e2d, 0x90e8, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x4e2d, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x5317, 0x7f8e, 0x4e2d, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x4e1c, 0x90e8,
0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x4e1c, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5317, 0x7f8e,
0x4e1c, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x5c71, 0x533a, 0x65f6, 0x95f4,
0x5317, 0x7f8e, 0x5c71, 0x533a, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x5c71, 0x533a,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x65f6, 0x95f4, 0x5317,
0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5317, 0x7f8e, 0x592a, 0x5e73,
0x6d0b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x7eb3, 0x5fb7, 0x5c14, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x963f, 0x7eb3, 0x5fb7, 0x5c14, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x5957,
0x65f6, 0x95f4, 0x963f, 0x514b, 0x5957, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x5957,
0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x6258, 0x522b, 0x65f6, 0x95f4, 0x963f, 0x514b,
0x6258, 0x522b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f, 0x514b, 0x6258, 0x522b, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e2d, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x5927,
0x5229, 0x4e9a, 0x4e2d, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a,
0x4e2d, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e2d, 0x897f,
0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e2d, 0x897f, 0x90e8, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e2d, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6,
0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229,
0x4e9a, 0x4e1c, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x4e1c,
0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x897f, 0x90e8, 0x65f6,
0x95f4, 0x6fb3, 0x5927, 0x5229, 0x4e9a, 0x897f, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6fb3,
0x5927, 0x5229, 0x4e9a, 0x897f, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x963f, 0x585e, 0x62dc,
0x7586, 0x65f6, 0x95f4, 0x963f, 0x585e, 0x62dc, 0x7586, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x963f,
0x585e, 0x62dc, 0x7586, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x73bb, 0x5229, 0x7ef4, 0x4e9a, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x6587, 0x83b1, 0x8fbe, 0x9c81, 0x8428, 0x5170, 0x65f6, 0x95f4, 0x4f5b,
0x5f97, 0x89d2, 0x65f6, 0x95f4, 0x4f5b, 0x5f97, 0x89d2, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4f5b,
0x5f97, 0x89d2, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x51ef, 0x897f, 0x65f6, 0x95f4, 0x67e5, 0x83ab,
0x7f57, 0x65f6, 0x95f4, 0x67e5, 0x5854, 0x59c6, 0x65f6, 0x95f4, 0x67e5, 0x5854, 0x59c6, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x67e5, 0x5854, 0x59c6, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5e93, 0x514b,
0x7fa4, 0x5c9b, 0x4ef2, 0x590f, 0x65f6, 0x95f4, 0x8fea, 0x8499, 0xb7, 0x8fea, 0x7ef4, 0x5c14,
0x65f6, 0x95f4, 0x5384, 0x74dc, 0x591a, 0x5c14, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6b27, 0x6d32,
0x6781, 0x4e1c, 0x65f6, 0x95f4, 0x6cd5, 0x5c5e, 0x572d, 0x4e9a, 0x90a3, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x6cd5, 0x5c5e, 0x5357, 0x65b9, 0x548c, 0x5357, 0x6781, 0x9886, 0x5730, 0x65f6, 0x95f4,
0x52a0, 0x62c9, 0x5e15, 0x6208, 0x65af, 0x65f6, 0x95f4, 0x7518, 0x6bd4, 0x5c14, 0x65f6, 0x95f4,
0x683c, 0x9c81, 0x5409, 0x4e9a, 0x65f6, 0x95f4, 0x683c, 0x9c81, 0x5409, 0x4e9a, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x683c, 0x9c81, 0x5409, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x683c, 0x6797,
0x5c3c, 0x6cbb, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x5c9b, 0x4e1c, 0x90e8,
0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x5c9b, 0x4e1c, 0x90e8, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x683c, 0x9675, 0x5170, 0x5c9b, 0x4e1c, 0x90e8, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x683c, 0x9675,
0x5170, 0x5c9b, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x5c9b, 0x897f, 0x90e8,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x683c, 0x9675, 0x5170, 0x5c9b, 0x897f, 0x90e8, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x5173, 0x5c9b, 0x65f6, 0x95f4, 0x6d77, 0x6e7e, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x5370, 0x5ea6, 0x65f6, 0x95f4, 0x4e2d, 0x5357, 0x534a, 0x5c9b, 0x65f6, 0x95f4, 0x5370, 0x5ea6,
0x5c3c, 0x897f, 0x4e9a, 0x4e2d, 0x90e8, 0x65f6, 0x95f4, 0x5370, 0x5ea6, 0x5c3c, 0x897f, 0x4e9a,
0x4e1c, 0x90e8, 0x65f6, 0x95f4, 0x5370, 0x5ea6, 0x5c3c, 0x897f, 0x4e9a, 0x897f, 0x90e8, 0x65f6,
0x95f4, 0x4f0a, 0x5c14, 0x5e93, 0x8328, 0x514b, 0x65f6, 0x95f4, 0x4f0a, 0x5c14, 0x5e93, 0x8328,
0x514b, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4f0a, 0x5c14, 0x5e93, 0x8328, 0x514b, 0x590f, 0x4ee4,
0x65f6, 0x95f4, 0x5f7c, 0x5f97, 0x7f57, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b, 0x2d,
0x582a, 0x5bdf, 0x52a0, 0x65f6, 0x95f4, 0x5f7c, 0x5f97, 0x7f57, 0x5df4, 0x752b, 0x6d1b, 0x592b,
0x65af, 0x514b, 0x2d, 0x582a, 0x5bdf, 0x52a0, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x5f7c, 0x5f97,
0x7f57, 0x5df4, 0x752b, 0x6d1b, 0x592b, 0x65af, 0x514b, 0x2d, 0x582a, 0x5bdf, 0x52a0, 0x590f,
0x4ee4, 0x65f6, 0x95f4, 0x54c8, 0x8428, 0x514b, 0x65af, 0x5766, 0x4e1c, 0x90e8, 0x65f6, 0x95f4,
0x54c8, 0x8428, 0x514b, 0x65af, 0x5766, 0x897f, 0x90e8, 0x65f6, 0x95f4, 0x79d1, 0x65af, 0x96f7,
0x65f6, 0x95f4, 0x5409, 0x5c14, 0x5409, 0x65af, 0x65af, 0x5766, 0x65f6, 0x95f4, 0x9a6c, 0x5c14,
0x4ee3, 0x592b, 0x65f6, 0x95f4, 0x9a6c, 0x514b, 0x8428, 0x65af, 0x7fa4, 0x5c9b, 0x65f6, 0x95f4,
0x6bdb, 0x91cc, 0x6c42, 0x65af, 0x65f6, 0x95f4, 0x6bdb, 0x91cc, 0x6c42, 0x65af, 0x6807, 0x51c6,
0x65f6, 0x95f4, 0x6bdb, 0x91cc, 0x6c42, 0x65af, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7459, 0x9c81,
0x65f6, 0x95f4, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9a, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x65b0, 0x897f, 0x5170, 0x65f6, 0x95f4, 0x65b0, 0x897f, 0x5170, 0x6807, 0x51c6, 0x65f6, 0x95f4,
0x65b0, 0x897f, 0x5170, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7ebd, 0x57c3, 0x65f6, 0x95f4, 0x8d39,
0x5c14, 0x5357, 0x591a, 0x2d, 0x8fea, 0x8bfa, 0x7f57, 0x5c3c, 0x4e9a, 0x5c9b, 0x65f6, 0x95f4,
0x8d39, 0x5c14, 0x5357, 0x591a, 0x2d, 0x8fea, 0x8bfa, 0x7f57, 0x5c3c, 0x4e9a, 0x5c9b, 0x6807,
0x51c6, 0x65f6, 0x95f4, 0x8d39, 0x5c14, 0x5357, 0x591a, 0x2d, 0x8fea, 0x8bfa, 0x7f57, 0x5c3c,
0x4e9a, 0x5c9b, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x5e15, 0x52b3, 0x65f6, 0x95f4, 0x5df4, 0x5e03,
0x4e9a, 0x65b0, 0x51e0, 0x5185, 0x4e9a, 0x65f6, 0x95f4, 0x83f2, 0x5c3c, 0x514b, 0x65af, 0x7fa4,
0x5c9b, 0x65f6, 0x95f4, 0x76ae, 0x7279, 0x51ef, 0x6069, 0x65f6, 0x95f4, 0x514b, 0x5b5c, 0x6d1b,
0x5c14, 0x8fbe, 0x65f6, 0x95f4, 0x514b, 0x5b5c, 0x6d1b, 0x5c14, 0x8fbe, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x514b, 0x5b5c, 0x6d1b, 0x5c14, 0x8fbe, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x7559, 0x5c3c,
0x6c6a, 0x65f6, 0x95f4, 0x585e, 0x820c, 0x5c14, 0x65f6, 0x95f4, 0x6240, 0x7f57, 0x95e8, 0x7fa4,
0x5c9b, 0x65f6, 0x95f4, 0x5357, 0x4e54, 0x6cbb, 0x4e9a, 0x5c9b, 0x65f6, 0x95f4, 0x82cf, 0x91cc,
0x5357, 0x65f6, 0x95f4, 0x662d, 0x548c, 0x65f6, 0x95f4, 0x5854, 0x5e0c, 0x63d0, 0x5c9b, 0x65f6,
0x95f4, 0x5854, 0x5409, 0x514b, 0x65af, 0x5766, 0x65f6, 0x95f4, 0x6258, 0x514b, 0x52b3, 0x65f6,
0x95f4, 0x6c64, 0x52a0, 0x65f6, 0x95f4, 0x6c64, 0x52a0, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x6c64,
0x52a0, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x695a, 0x514b, 0x65f6, 0x95f4, 0x571f, 0x5e93, 0x66fc,
0x65af, 0x5766, 0x65f6, 0x95f4, 0x571f, 0x5e93, 0x66fc, 0x65af, 0x5766, 0x6807, 0x51c6, 0x65f6,
0x95f4, 0x571f, 0x5e93, 0x66fc, 0x65af, 0x5766, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x56fe, 0x74e6,
0x5362, 0x65f6, 0x95f4, 0x4e4c, 0x5179, 0x522b, 0x514b, 0x65af, 0x5766, 0x65f6, 0x95f4, 0x4e4c,
0x5179, 0x522b, 0x514b, 0x65af, 0x5766, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x4e4c, 0x5179, 0x522b,
0x514b, 0x65af, 0x5766, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x74e6, 0x52aa, 0x963f, 0x56fe, 0x65f6,
0x95f4, 0x74e6, 0x52aa, 0x963f, 0x56fe, 0x6807, 0x51c6, 0x65f6, 0x95f4, 0x74e6, 0x52aa, 0x963f,
0x56fe, 0x590f, 0x4ee4, 0x65f6, 0x95f4, 0x74e6, 0x5229, 0x65af, 0x548c, 0x5bcc, 0x56fe, 0x7eb3,
0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x8328, 0x514b, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x8328, 0x514b,
0x6807, 0x51c6, 0x65f6, 0x95f4, 0x96c5, 0x5e93, 0x8328, 0x514b, 0x590f, 0x4ee4, 0x65f6, 0x95f4,
0x5357, 0x975e, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x4e2d, 0x90e8, 0x6642, 0x9593, 0x5317, 0x7f8e,
0x4e2d, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x4e2d, 0x90e8, 0x590f, 0x4ee4,
0x6642, 0x9593, 0x5317, 0x7f8e, 0x6771, 0x90e8, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x6771, 0x90e8,
0x6a19, 0x6e96, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x6771, 0x90e8, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x5317, 0x7f8e, 0x5c71, 0x5340, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x5c71, 0x5340, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x5317, 0x7f8e, 0x5c71, 0x5340, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x5317, 0x7f8e,
0x592a, 0x5e73, 0x6d0b, 0x6642, 0x9593, 0x5317, 0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x6a19, 0x6e96,
0x6642, 0x9593, 0x5317, 0x7f8e, 0x592a, 0x5e73, 0x6d0b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e9e,
0x585e, 0x62dc, 0x7586, 0x6642, 0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7586, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x4e9e, 0x585e, 0x62dc, 0x7586, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4f5b, 0x5f97, 0x89d2,
0x6642, 0x9593, 0x4f5b, 0x5f97, 0x89d2, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4f5b, 0x5f97, 0x89d2,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x53ef, 0x53ef, 0x65af, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x8fea,
0x8499, 0x8fea, 0x7dad, 0x723e, 0x6642, 0x9593, 0x5384, 0x74dc, 0x591a, 0x723e, 0x6642, 0x9593,
0x52a0, 0x62c9, 0x5e15, 0x6208, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x683c, 0x9b6f, 0x5409, 0x4e9e,
0x6642, 0x9593, 0x683c, 0x9b6f, 0x5409, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x683c, 0x9b6f,
0x5409, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x6ce2, 0x65af, 0x7063, 0x6d77, 0x57df, 0x6642,
0x9593, 0x5370, 0x5ea6, 0x6642, 0x9593, 0x4f0a, 0x723e, 0x5eab, 0x8328, 0x514b, 0x6642, 0x9593,
0x4f0a, 0x723e, 0x5eab, 0x8328, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x4f0a, 0x723e, 0x5eab,
0x8328, 0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x79d1, 0x65af, 0x96f7, 0x6642, 0x9593, 0x99ac,
0x723e, 0x4ee3, 0x592b, 0x6642, 0x9593, 0x99ac, 0x514b, 0x85a9, 0x65af, 0x6642, 0x9593, 0x6bdb,
0x91cc, 0x88d8, 0x65af, 0x6642, 0x9593, 0x6bdb, 0x91cc, 0x88d8, 0x65af, 0x6a19, 0x6e96, 0x6642,
0x9593, 0x6bdb, 0x91cc, 0x88d8, 0x65af, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x7459, 0x9b6f, 0x6642,
0x9593, 0x65b0, 0x5580, 0x91cc, 0x591a, 0x5c3c, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x8cbb,
0x723e, 0x5357, 0x591a, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x6642, 0x9593, 0x8cbb, 0x723e,
0x5357, 0x591a, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x8cbb,
0x723e, 0x5357, 0x591a, 0x8fea, 0x8afe, 0x7f85, 0x5c3c, 0x4e9e, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x5df4, 0x5e03, 0x4e9e, 0x65b0, 0x757f, 0x5167, 0x4e9e, 0x6642, 0x9593, 0x76ae, 0x7279, 0x5eb7,
0x6642, 0x9593, 0x585e, 0x820c, 0x723e, 0x6642, 0x9593, 0x65b0, 0x52a0, 0x5761, 0x6642, 0x9593,
0x6240, 0x7f85, 0x9580, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x8607, 0x91cc, 0x5357, 0x6642, 0x9593,
0x6e6f, 0x52a0, 0x6642, 0x9593, 0x6e6f, 0x52a0, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x6e6f, 0x52a0,
0x590f, 0x4ee4, 0x6642, 0x9593, 0x5716, 0x74e6, 0x76e7, 0x6642, 0x9593, 0x74e6, 0x52aa, 0x963f,
0x5716, 0x6642, 0x9593, 0x74e6, 0x52aa, 0x963f, 0x5716, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x74e6,
0x52aa, 0x963f, 0x5716, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x96c5, 0x5eab, 0x8328, 0x514b, 0x6642,
0x9593, 0x96c5, 0x5eab, 0x8328, 0x514b, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x96c5, 0x5eab, 0x8328,
0x514b, 0x590f, 0x4ee4, 0x6642, 0x9593, 0x4e2d, 0x5357, 0x534a, 0x5cf6, 0x6642, 0x9593, 0x8056,
0x76ae, 0x57c3, 0x8207, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5cf6, 0x6642, 0x9593, 0x8056, 0x76ae,
0x57c3, 0x8207, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x9593, 0x8056,
0x76ae, 0x57c3, 0x8207, 0x5bc6, 0x514b, 0x9686, 0x7fa4, 0x5cf6, 0x590f, 0x4ee4, 0x6642, 0x9593,
0x80b2, 0x7a7a, 0x5730, 0x5340, 0x6642, 0x9593, 0x441, 0x440, 0x435, 0x434, 0x43d, 0x435,
0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x441, 0x440, 0x435, 0x434,
0x43d, 0x435, 0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x441, 0x440, 0x435, 0x434, 0x43d, 0x435,
0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e,
0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301,
0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301,
0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x430, 0x486, 0x43c, 0x435,
0x440, 0x457, 0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43d,
0x430, 0x433, 0x43e, 0x301, 0x440, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x430, 0x486, 0x43c, 0x435, 0x440, 0x457, 0x43a, 0x430, 0x301,
0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43d, 0x430, 0x433, 0x43e, 0x301, 0x440,
0x43d, 0x435, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x430, 0x486, 0x43c, 0x435, 0x440, 0x457,
0x43a, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43d, 0x430, 0x433,
0x43e, 0x301, 0x440, 0x43d, 0x435, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x442, 0x438, 0x445,
0x43e, 0x47b, 0x43a, 0x435, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x442, 0x438, 0x445, 0x43e, 0x47b, 0x43a,
0x435, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301,
0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x442,
0x438, 0x445, 0x43e, 0x47b, 0x43a, 0x435, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x430, 0x486, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x456,
0x301, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x430, 0x486, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x456, 0x301, 0x447,
0x435, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x430, 0x486, 0x442, 0x43b,
0x430, 0x43d, 0x442, 0x456, 0x301, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x435, 0x20,
0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x441, 0x440, 0x435, 0x434, 0x43d, 0x435, 0x454, 0x475, 0x440, 0x461,
0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x441, 0x440, 0x435, 0x434, 0x43d, 0x435, 0x454, 0x475, 0x440,
0x461, 0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438,
0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x441, 0x440, 0x435, 0x434, 0x43d, 0x435, 0x454, 0x475, 0x440, 0x461, 0x43f, 0x435,
0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x454, 0x475, 0x440, 0x461, 0x43f, 0x435, 0x301,
0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x432, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x454, 0x475, 0x440, 0x461,
0x43f, 0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301,
0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x454, 0x475, 0x440, 0x461, 0x43f,
0x435, 0x301, 0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x20, 0x432, 0x44a, 0x20, 0x43a, 0x430, 0x43b, 0x438,
0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x301, 0x434, 0x463, 0x20, 0x438, 0x486,
0x20, 0x43c, 0x438, 0x301, 0x43d, 0x441, 0x43a, 0x463, 0x437, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x43e, 0x454, 0x475, 0x440, 0x461, 0x43f, 0x435, 0x301, 0x439, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x437, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x454, 0x475, 0x440, 0x461, 0x43f, 0x435, 0x301,
0x439, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x437, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x43e, 0x454, 0x475, 0x440, 0x461, 0x43f, 0x435, 0x301, 0x439, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x441, 0x440, 0x435, 0x301, 0x434, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x20, 0x43f, 0x43e,
0x20, 0x433, 0x440, 0x456, 0x301, 0x43d, 0xa64b, 0x438, 0x447, 0xa64b, 0x438, 0x486,
0x440, 0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x438, 0x486, 0x440, 0x43a, 0xa64b, 0x301, 0x442, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x438, 0x486, 0x440, 0x43a, 0xa64b, 0x301,
0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x301, 0x447, 0x43d, 0x44b, 0x439, 0x20, 0x43a, 0x430, 0x437, 0x430, 0x445,
0x441, 0x442, 0x430, 0x301, 0x43d, 0x44a, 0x437, 0x430, 0x301, 0x43f, 0x430, 0x434,
0x43d, 0x44b, 0x439, 0x20, 0x43a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430,
0x301, 0x43d, 0x44a, 0x43a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x467, 0x301, 0x440,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x43a,
0x440, 0x430, 0x441, 0x43d, 0x43e, 0x467, 0x301, 0x440, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x43a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x467, 0x301, 0x440,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x43a, 0x438, 0x440, 0x433, 0x438,
0x301, 0x437, 0x457, 0x430, 0x43c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x301, 0x43d,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x43c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x301, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20,
0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301,
0x43c, 0x467, 0x43c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x301, 0x43d, 0x441, 0x43a,
0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x301, 0x43c, 0x467, 0x43c, 0x43e, 0x441, 0x43a, 0x43e, 0x301, 0x432,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x43c,
0x43e, 0x441, 0x43a, 0x43e, 0x301, 0x432, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437,
0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x43c, 0x43e, 0x441, 0x43a, 0x43e, 0x301, 0x432, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x301, 0x43c, 0x467, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x301,
0x440, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x301, 0x440, 0x441, 0x43a,
0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x301, 0x43c, 0x467, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431,
0x438, 0x301, 0x440, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x47b, 0x486,
0x301, 0x43c, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x47b, 0x486, 0x301, 0x43c, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438,
0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x47b, 0x486, 0x301, 0x43c, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432,
0x440, 0x435, 0x301, 0x43c, 0x467, 0x20, 0x43d, 0x430, 0x20, 0x441, 0x430, 0x445,
0x430, 0x43b, 0x438, 0x301, 0x43d, 0x463, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x20, 0x43d, 0x430, 0x20,
0x441, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x301, 0x43d, 0x463, 0x43b, 0x463, 0x301,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x20,
0x43d, 0x430, 0x20, 0x441, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x301, 0x43d, 0x463,
0x432, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x301, 0x446,
0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x301, 0x446, 0x43a, 0x43e,
0x435, 0x20, 0x437, 0x438, 0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x301, 0x43c, 0x467, 0x432, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x301, 0x446, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e,
0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x301, 0x434, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x432, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x301, 0x434, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438,
0x301, 0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x432, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x301, 0x434, 0x441, 0x43a,
0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x301, 0x43c, 0x467, 0xa657, 0x486, 0x43a, 0xa64b, 0x301, 0x442, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0xa657, 0x486,
0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301,
0x43c, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0xa657,
0x486, 0x43a, 0xa64b, 0x301, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463,
0x301, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467,
0x454, 0x486, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x457, 0x43d, 0x431, 0xa64b, 0x301,
0x440, 0x436, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c,
0x467, 0x454, 0x486, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x457, 0x43d, 0x431, 0xa64b,
0x301, 0x440, 0x436, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x437, 0x438, 0x301, 0x43c,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x454, 0x486,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x457, 0x43d, 0x431, 0xa64b, 0x301, 0x440, 0x436,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x463, 0x301, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x301, 0x43c, 0x467, 0x410, 0x444, 0x433, 0x430, 0x43d,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443,
0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x4d1, 0x43d, 0x442, 0x4d1, 0x440, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a,
0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7,
0x43f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443,
0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442,
0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442,
0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x422, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x20, 0x28, 0x4aa, 0x443, 0x440, 0x4ab, 0x4d7, 0x440, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x29, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x442, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x20, 0x28, 0x4aa, 0x443, 0x440, 0x4ab, 0x4d7, 0x440, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x4aa, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x442, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x20,
0x28, 0x4aa, 0x443, 0x440, 0x4ab, 0x4d7, 0x440, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x29, 0x41b, 0x4d1, 0x43f, 0x43a, 0x4d1, 0x20, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41b, 0x4d1,
0x43f, 0x43a, 0x4d1, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41b, 0x4d1, 0x43f, 0x43a, 0x4d1, 0x20, 0x43e, 0x43a, 0x435,
0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43f, 0x438, 0x430, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43f, 0x438, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x410, 0x43f, 0x438, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x430,
0x43f, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x430, 0x43f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x430, 0x43f, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x43c, 0x435, 0x43d,
0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x440, 0x43c, 0x435, 0x43d,
0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a,
0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422,
0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x20, 0x430, 0x43d, 0x4d1, 0x4ab, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x20, 0x430, 0x43d, 0x4d1, 0x4ab, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x20, 0x430, 0x43d, 0x4d1, 0x4ab, 0x20, 0x4ab, 0x443, 0x43b, 0x43b,
0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7,
0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430,
0x439, 0x434, 0x436, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439,
0x434, 0x436, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x437, 0x43e, 0x440, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x443, 0x442, 0x440,
0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x437, 0x43e, 0x440, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441,
0x435, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434,
0x435, 0x448, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x411, 0x443, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x2d, 0x414, 0x430, 0x440,
0x443, 0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x430, 0x431, 0x43e, 0x2d,
0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a,
0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x427, 0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x427, 0x438, 0x43b,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x427, 0x438, 0x43b, 0x438,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41a, 0x438, 0x442, 0x430, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41a, 0x438, 0x442, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41a, 0x438, 0x442, 0x430, 0x439, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x427, 0x43e, 0x439, 0x431,
0x430, 0x43b, 0x441, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x420, 0x430, 0x448, 0x442, 0x430, 0x432, 0x20, 0x443, 0x442,
0x440, 0x430, 0x432, 0x4d7, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7,
0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x43e,
0x43b, 0x443, 0x43c, 0x431, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a,
0x443, 0x43a, 0x4d1, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441,
0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x443, 0x43a,
0x4d1, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x443, 0x43a, 0x4d1, 0x43d, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41a, 0x443, 0x431, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a,
0x443, 0x431, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x443, 0x431,
0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x414, 0x435, 0x439, 0x432, 0x438, 0x441, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x2d, 0x434,
0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b, 0x44c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab,
0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41c, 0x4d1, 0x43d, 0x43a, 0x443, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430,
0x432, 0x4d7, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x4d1,
0x43d, 0x43a, 0x443, 0x43d, 0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x4d1, 0x43d, 0x43a, 0x443, 0x43d,
0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x43d, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x42d,
0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7,
0x43f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425,
0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425,
0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7,
0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x43d, 0x4ab, 0x435, 0x442, 0x2d,
0x445, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410,
0x43d, 0x4d1, 0x4ab, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x443, 0x442,
0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x443,
0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x443,
0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424,
0x438, 0x434, 0x436, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424,
0x438, 0x434, 0x436, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x438,
0x434, 0x436, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x438,
0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x438, 0x20, 0x41a, 0x4d1, 0x43d,
0x442, 0x4d1, 0x440, 0x43f, 0x430, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a,
0x442, 0x438, 0x43a, 0x430, 0x20, 0x442, 0x435, 0x440, 0x440, 0x438, 0x442, 0x43e,
0x440, 0x438, 0x439, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440, 0x443, 0x437, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440, 0x443, 0x437, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x440, 0x443, 0x437, 0x438, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x4d1, 0x43d, 0x20, 0x443, 0x442,
0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x43f, 0x430, 0x20,
0x432, 0x4d1, 0x442, 0x430, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x413,
0x440, 0x438, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1,
0x4ab, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443,
0x445, 0x4d1, 0x4ab, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x413, 0x440, 0x438,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x410, 0x43d, 0x4d1,
0x4ab, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x435, 0x440, 0x441, 0x438, 0x20, 0x437, 0x430,
0x43b, 0x438, 0x432, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430,
0x439, 0x430, 0x43d, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413,
0x430, 0x432, 0x430, 0x439, 0x438, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x430, 0x432, 0x430, 0x439, 0x438,
0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x413, 0x430, 0x432, 0x430, 0x439, 0x438, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d,
0x433, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x43d, 0x434, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x43d, 0x434, 0x438, 0x20,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x4d7, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x418, 0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x442, 0x430, 0x439, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x4d7, 0x43f, 0x20, 0x418, 0x43d, 0x434,
0x43e, 0x43d, 0x435, 0x437, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x425, 0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x418,
0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d,
0x435, 0x437, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x440,
0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x440, 0x430,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x440, 0x430, 0x43d, 0x20,
0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b,
0x44c, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x425,
0x4d7, 0x432, 0x435, 0x43b, 0x442, 0x443, 0x445, 0x4d1, 0x4ab, 0x20, 0x41a, 0x430,
0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x410, 0x43d, 0x4d1, 0x4ab, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a,
0x43e, 0x440, 0x435, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a,
0x43e, 0x440, 0x435, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x43e,
0x440, 0x435, 0x439, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x43e, 0x441, 0x440, 0x430, 0x435,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440,
0x441, 0x43a, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x437, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41b, 0x430, 0x439, 0x43d, 0x20,
0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41b, 0x43e, 0x440, 0x434, 0x2d,
0x425, 0x430, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41b, 0x43e, 0x440,
0x434, 0x2d, 0x425, 0x430, 0x443, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430,
0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x43b,
0x430, 0x439, 0x437, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c,
0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x441, 0x435, 0x43c, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437, 0x430, 0x441,
0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b,
0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438,
0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430, 0x432, 0x440,
0x438, 0x43a, 0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x430,
0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x43e, 0x443,
0x441, 0x43e, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x435,
0x43a, 0x441, 0x438, 0x43a, 0x430, 0x20, 0x41b, 0x4d1, 0x43f, 0x43a, 0x4d1, 0x20,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x20, 0x41b, 0x4d1, 0x43f, 0x43a,
0x4d1, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x20, 0x41b, 0x4d1, 0x43f,
0x43a, 0x4d1, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423,
0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442,
0x43e, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423, 0x43b, 0x430, 0x43d,
0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x443, 0x441,
0x43a, 0x430, 0x432, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x443,
0x441, 0x43a, 0x430, 0x432, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x443,
0x441, 0x43a, 0x430, 0x432, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c,
0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x430, 0x443, 0x440,
0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x435, 0x43f, 0x430,
0x43b, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x4aa, 0x4d7, 0x43d, 0x4d7,
0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x4aa,
0x4d7, 0x43d, 0x4d7, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x417, 0x435, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x4aa, 0x4d7,
0x43d, 0x4d7, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x417, 0x435, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x44c, 0x44e, 0x444, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435,
0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x44c, 0x44e, 0x444,
0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d,
0x438, 0x443, 0x44d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x43e,
0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438,
0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434,
0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41e, 0x43c,
0x441, 0x43a, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430,
0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x43b,
0x430, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x43f,
0x443, 0x430, 0x20, 0x2014, 0x20, 0x4aa, 0x4d7, 0x43d, 0x4d7, 0x20, 0x413, 0x432,
0x438, 0x43d, 0x435, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x41f, 0x435, 0x440, 0x443, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x438, 0x43b, 0x438, 0x43f,
0x43f, 0x438, 0x43d, 0x441, 0x435, 0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x441, 0x435, 0x43c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f,
0x438, 0x43d, 0x441, 0x435, 0x43c, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x424, 0x435, 0x43d, 0x438,
0x43a, 0x441, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x435, 0x43d,
0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x442, 0x430, 0x442, 0x430, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x442, 0x430,
0x442, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440,
0x20, 0x442, 0x430, 0x442, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41f, 0x43e, 0x43d, 0x43f, 0x435, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x420, 0x435, 0x44e, 0x43d, 0x44c,
0x43e, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x420, 0x43e, 0x442,
0x435, 0x440, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x430,
0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421,
0x430, 0x43c, 0x43e, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421,
0x430, 0x43c, 0x43e, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x430,
0x43c, 0x43e, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x435, 0x439, 0x448, 0x435, 0x43b,
0x20, 0x443, 0x442, 0x440, 0x430, 0x432, 0x4d7, 0x441, 0x435, 0x43d, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443,
0x440, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x43e, 0x43b, 0x43e,
0x43c, 0x43e, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x41a, 0x4d1,
0x43d, 0x442, 0x4d1, 0x440, 0x20, 0x413, 0x435, 0x43e, 0x440, 0x433, 0x438, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430,
0x43c, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x421, 0x451, 0x432, 0x430,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430, 0x438, 0x442, 0x438,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430, 0x439, 0x43f, 0x435,
0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430, 0x439, 0x43f,
0x435, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430, 0x439, 0x43f,
0x435, 0x439, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x430, 0x434, 0x436, 0x438, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422,
0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445,
0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x440, 0x443, 0x43a,
0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x443, 0x440, 0x43a, 0x43c,
0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b,
0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422, 0x443, 0x440, 0x43a,
0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x422,
0x443, 0x432, 0x430, 0x43b, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x4ab,
0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x423, 0x437,
0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412,
0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x430, 0x20, 0x432, 0x4d1,
0x445, 0x4d1, 0x447, 0x4d7, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432,
0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x412, 0x43e, 0x43b,
0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x4ab, 0x443,
0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x423, 0x44d, 0x439, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x423, 0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x20, 0x442, 0x430, 0x442, 0x430, 0x20,
0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447,
0x4d7, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x4ab, 0x443, 0x43b,
0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x415,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20,
0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43b, 0x4d1, 0x20, 0x432, 0x4d1, 0x445, 0x4d1, 0x447, 0x4d7,
0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433,
0x20, 0x4ab, 0x443, 0x43b, 0x43b, 0x430, 0x445, 0x438, 0x20, 0x432, 0x4d1, 0x445,
0x4d1, 0x447, 0x4d7, 0x42e, 0x43a, 0x43e, 0x43d, 0x20, 0x432, 0x4d1, 0x445, 0x4d1,
0x447, 0x4d7, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41,
0x66, 0x66, 0x72, 0x65, 0x6b, 0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x69, 0x67, 0x67, 0x4f, 0xdf, 0x2d, 0x41, 0x66, 0x66,
0x72, 0x65, 0x6b, 0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x69, 0x67, 0x67, 0x53, 0xf6, 0x64, 0x2d, 0x41, 0x66, 0x66, 0x72,
0x65, 0x6b, 0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a,
0x69, 0x67, 0x67, 0x57, 0xe4, 0xdf, 0x2d, 0x41, 0x66, 0x66, 0x72, 0x65,
0x6b, 0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x69,
0x67, 0x67, 0x4a, 0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a,
0x65, 0x20, 0x57, 0xe4, 0xdf, 0x2d, 0x41, 0x66, 0x66, 0x72, 0x65, 0x6b,
0x61, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x69, 0x67,
0x67, 0x57, 0xe4, 0xdf, 0x2d, 0x41, 0x66, 0x66, 0x72, 0x65, 0x6b, 0x61,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x69, 0x67, 0x67, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x20, 0x69, 0x65, 0x72, 0x20, 0x5a, 0x69, 0x63, 0x6b, 0x64,
0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x20, 0x69, 0x65, 0x72, 0x20,
0x6a, 0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a, 0x65, 0x20,
0x5a, 0x69, 0x63, 0x6b, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x20, 0x69, 0x65, 0x72, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x69, 0x63, 0x6b, 0x4b, 0x61, 0x70, 0x76, 0xe4, 0x72, 0x64, 0x65, 0x6a,
0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x69, 0x67, 0x67,
0x4a, 0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a, 0x65, 0x20,
0x4b, 0x61, 0x70, 0x76, 0xe4, 0x72, 0x64, 0x65, 0x6a, 0x61, 0x61, 0x6e,
0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x69, 0x67, 0x67, 0x4b, 0x61, 0x70,
0x76, 0xe4, 0x72, 0x64, 0x65, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68,
0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x67, 0x67,
0x4d, 0x65, 0x64, 0x64, 0x65, 0x6c, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x5a, 0x69, 0x63, 0x6b,
0x4d, 0x65, 0x64, 0x64, 0x65, 0x6c, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x6a, 0x65, 0x77, 0xf6,
0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a, 0x65, 0x20, 0x5a, 0x69, 0x63, 0x6b,
0x4d, 0x65, 0x64, 0x64, 0x65, 0x6c, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x69, 0x63, 0x6b, 0x4f, 0xdf, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x5a, 0x69,
0x63, 0x6b, 0x4f, 0xdf, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x70, 0x61,
0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x6a, 0x65, 0x77, 0xf6, 0x68, 0x6e,
0x6c, 0x69, 0x6a, 0x6a, 0x65, 0x20, 0x5a, 0x69, 0x63, 0x6b, 0x4f, 0xdf,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x70, 0x61, 0x20, 0x73, 0x69, 0x6e,
0x67, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x63, 0x6b,
0x57, 0x65, 0xdf, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x70, 0x61, 0x20,
0x73, 0x69, 0x6e, 0x67, 0x20, 0x5a, 0x69, 0x63, 0x6b, 0x57, 0x65, 0xdf,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x70, 0x61, 0x20, 0x73, 0x69, 0x6e,
0x67, 0x20, 0x6a, 0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a,
0x65, 0x20, 0x5a, 0x69, 0x63, 0x6b, 0x57, 0x65, 0xdf, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x70, 0x61, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x63, 0x6b, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x73, 0x69, 0x6e, 0x67, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x74, 0x2d, 0x5a, 0x69, 0x63, 0x6b,
0x64, 0x65, 0x6d, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x4f, 0x7a, 0x65, 0x6a, 0x61, 0x6e, 0x20, 0x73, 0x69, 0x6e, 0x67,
0x20, 0x5a, 0x69, 0x63, 0x6b, 0x5a, 0x69, 0x67, 0x67, 0x20, 0x76, 0x75,
0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4a,
0x65, 0x77, 0xf6, 0x68, 0x6e, 0x6c, 0x69, 0x6a, 0x6a, 0x65, 0x20, 0x5a,
0x69, 0x67, 0x67, 0x20, 0x76, 0x75, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x69, 0x67, 0x67, 0x20, 0x76, 0x75, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x5a, 0x69, 0x67, 0x67, 0x20, 0x76, 0x75,
0x6e, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x5a, 0x69, 0x67,
0x67, 0x20, 0x76, 0x75, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x53, 0x65, 0x69,
0x73, 0x63, 0x68, 0xe4, 0x6c, 0x6c, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x6d,
0x65, 0x64, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x41, 0x63, 0x72, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x63, 0x72, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x63, 0x72, 0x65, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x61, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72,
0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73, 0x74, 0x6f,
0x10d, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x75, 0x17e, 0x6e, 0x6f, 0x61,
0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x61, 0x6c, 0x6a, 0x61, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6c, 0x6a, 0x61, 0x161, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6c, 0x6a, 0x61, 0x161,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6c, 0x74, 0x6d, 0x61, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6c,
0x74, 0x6d, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x61, 0x6c, 0x74, 0x6d, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73,
0x72, 0x65, 0x64, 0x69, 0x161, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64, 0x69, 0x161, 0x6e, 0x6a,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64,
0x69, 0x161, 0x6e, 0x6a, 0x65, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6e, 0x61, 0x64,
0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x61, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x41, 0x70, 0x69, 0x6a, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x41, 0x70, 0x69, 0x6a, 0x65, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x41, 0x70, 0x69,
0x6a, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72,
0x61, 0x64, 0x61, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x41, 0x6b,
0x74, 0x61, 0x75, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20,
0x41, 0x6b, 0x74, 0x61, 0x75, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64,
0x61, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67,
0x72, 0x61, 0x64, 0x61, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x61,
0x72, 0x61, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x61, 0x72, 0x61, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x72, 0x61, 0x70, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x72, 0x6d,
0x65, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64,
0x6e, 0x6a, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65,
0x64, 0x6e, 0x6a, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65,
0x64, 0x6e, 0x6a, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x72, 0x65, 0x64, 0x69, 0x161, 0x6e, 0x6a,
0x65, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x72, 0x65, 0x64, 0x69, 0x161, 0x6e, 0x6a,
0x65, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x72, 0x65, 0x64, 0x69, 0x161, 0x6e, 0x6a, 0x65,
0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a,
0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x7a, 0x6f, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x7a,
0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62,
0x75, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x62, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x72, 0x61,
0x7a, 0x69, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x62, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x62, 0x72, 0x61, 0x7a, 0x69, 0x6c,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a, 0x20,
0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f,
0x72, 0x74, 0x73, 0x6b, 0x6f, 0x67, 0x20, 0x6f, 0x74, 0x6f, 0x10d, 0x6a,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x65,
0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x6f, 0x67, 0x20, 0x6f, 0x74, 0x6f,
0x10d, 0x6a, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f,
0x72, 0x74, 0x73, 0x6b, 0x6f, 0x67, 0x20, 0x6f, 0x74, 0x6f, 0x10d, 0x6a,
0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x61, 0x73,
0x65, 0x79, 0x6a, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x61, 0x10d, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10d, 0x69, 0x6c, 0x65, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10d, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x69,
0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x63, 0x68,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x63, 0x68, 0x6f, 0x69, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x63, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x42, 0x6f, 0x17e, 0x69, 0x107, 0x6e, 0x6f, 0x67, 0x20,
0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76, 0x69, 0x68, 0x20, 0x6f,
0x74, 0x6f, 0x6b, 0x61, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0x69, 0x68, 0x20,
0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0x69, 0x68, 0x20, 0x6f, 0x74, 0x6f,
0x6b, 0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0x69, 0x20, 0x6f, 0x74,
0x6f, 0x63, 0x69, 0x2c, 0x20, 0x70, 0x6f, 0x6c, 0x75, 0x73, 0x61, 0x74,
0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6d, 0x61, 0x6b, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x75, 0x62, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x61, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x61, 0x69, 0x73, 0x74,
0x6f, 0x10d, 0x6e, 0x6f, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x65,
0x67, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x65, 0x67, 0x20,
0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x55, 0x73, 0x6b, 0x72,
0x161, 0x6e, 0x6a, 0x65, 0x67, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x65,
0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x73, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x73, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73, 0x74, 0x6f,
0x10d, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d,
0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x64,
0x61, 0x6c, 0x65, 0x6b, 0x6f, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f,
0x20, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x66, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x66, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x46, 0x69, 0x64, 0x17e, 0x69,
0x6a, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x46, 0x69, 0x64,
0x17e, 0x69, 0x6a, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x46, 0x69, 0x64, 0x17e, 0x69,
0x6a, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x65, 0x20, 0x47, 0x69, 0x6a, 0x61,
0x6e, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x69, 0x68, 0x20, 0x6a, 0x75, 0x17e,
0x6e, 0x69, 0x68, 0x20, 0x69, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b,
0x74, 0x69, 0x10d, 0x6b, 0x69, 0x68, 0x20, 0x74, 0x65, 0x72, 0x69, 0x74,
0x6f, 0x72, 0x69, 0x6a, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x61, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0x61, 0x67, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x67, 0x72, 0x75, 0x7a,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x67, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x6f, 0x76, 0x69, 0x68, 0x20, 0x6f, 0x74, 0x6f, 0x6b,
0x61, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x7a, 0x61, 0x6c, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73, 0x74, 0x6f,
0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x67, 0x75, 0x61, 0x6d,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61,
0x6c, 0x6a, 0x65, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d,
0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d,
0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x68,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x68, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x68,
0x6f, 0x76, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x69, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x67, 0x20,
0x6f, 0x63, 0x65, 0x61, 0x6e, 0x61, 0x69, 0x6e, 0x64, 0x6f, 0x6b, 0x69,
0x6e, 0x65, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x73, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x69, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x72,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x69, 0x7a, 0x72, 0x61, 0x65,
0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x61, 0x70, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a,
0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x6b, 0x61, 0x7a, 0x61,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x6b,
0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x6f, 0x72, 0x65, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b,
0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4b, 0x6f, 0x73,
0x72, 0x61, 0x65, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6b, 0x69, 0x72,
0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6c, 0x61, 0x6e, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x45, 0x6b, 0x76, 0x61, 0x74, 0x6f,
0x72, 0x73, 0x6b, 0x69, 0x68, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x6f,
0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x6d, 0x61, 0x6b, 0x61, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x6d, 0x61, 0x6b, 0x61, 0x6f, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x6d, 0x61, 0x6b, 0x61, 0x6f, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x61, 0x6c, 0x65,
0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x6d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x61, 0x72, 0x6b, 0x69,
0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4d, 0x61,
0x72, 0x161, 0x61, 0x6c, 0x6f, 0x76, 0x69, 0x68, 0x20, 0x4f, 0x74, 0x6f,
0x6b, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75, 0x73, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a,
0x75, 0x73, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x69, 0x6a, 0x75, 0x73, 0x61, 0x6d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x65,
0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63,
0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75, 0x6c,
0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61,
0x74, 0x6f, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6d, 0x6f, 0x73,
0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6d, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x61, 0x6e, 0x65, 0x70,
0x61, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4e, 0x6f, 0x76,
0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x65, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x6a, 0x65, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6e, 0x6f, 0x76,
0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x4e, 0x69, 0x75, 0x65, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4f, 0x74, 0x6f,
0x6b, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72,
0x61, 0x64, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72,
0x61, 0x64, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x53, 0x6a, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x6f, 0x6d, 0x61, 0x72, 0x69, 0x6a, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x68, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x6e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x6f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x6f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x6f, 0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x50, 0x61,
0x6c, 0x61, 0x75, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x50, 0x61, 0x70, 0x75, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x47,
0x76, 0x69, 0x6e, 0x65, 0x6a, 0x65, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76,
0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x61, 0x72, 0x61, 0x67,
0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x65, 0x72,
0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x70, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x65, 0x72, 0x75, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x50, 0x68, 0x6f, 0x65,
0x6e, 0x69, 0x78, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x7a,
0x61, 0x20, 0x53, 0x76, 0x65, 0x74, 0x69, 0x20, 0x50, 0x65, 0x74, 0x61,
0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x76, 0x65,
0x74, 0x69, 0x20, 0x50, 0x65, 0x74, 0x61, 0x72, 0x20, 0x69, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x7a, 0x61, 0x20,
0x53, 0x76, 0x65, 0x74, 0x69, 0x20, 0x50, 0x65, 0x74, 0x61, 0x72, 0x20,
0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x70, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x70, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x70, 0x6a,
0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x4b, 0x69, 0x7a, 0x69, 0x6c,
0x6f, 0x72, 0x64, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67,
0x72, 0x61, 0x64, 0x61, 0x20, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72,
0x64, 0x65, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x4b,
0x69, 0x7a, 0x69, 0x6c, 0x6f, 0x72, 0x64, 0x65, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x61,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68,
0x65, 0x72, 0x65, 0x73, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x73, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x6d, 0x61, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61,
0x6d, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x6d,
0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73, 0x65, 0x6a, 0x161, 0x65, 0x6c,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x73,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x6b, 0x69,
0x68, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x20, 0x4a, 0x75, 0x17e, 0x6e, 0x65, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x6a, 0x65, 0x73, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x65,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x6a, 0x61, 0x74, 0x61, 0x6a, 0x70, 0x65, 0x161, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74, 0x61, 0x6a, 0x70,
0x65, 0x161, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74,
0x61, 0x6a, 0x70, 0x65, 0x161, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74, 0x61,
0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x61,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x65, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x65, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x61, 0x74,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x74, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x54, 0x75,
0x76, 0x61, 0x6c, 0x75, 0x61, 0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75,
0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x75, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x76, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x10d, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x20, 0x4f, 0x74, 0x6f, 0x6b,
0x61, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x69, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x6a, 0x75, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x61, 0x63, 0x72, 0x65, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x63, 0x72, 0x65, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x63, 0x72, 0x65, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61,
0x66, 0x67, 0x68, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f,
0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a,
0x69, 0x68, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64,
0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c,
0x6a, 0x61, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c,
0x6a, 0x61, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6a,
0x61, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0xe1, 0x6c, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65,
0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0xe1, 0x6c, 0x6e, 0xed, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe1, 0x6c, 0x6e, 0xed, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76,
0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20,
0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0xed, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0xed, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x70, 0x69, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x70, 0x69, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x70, 0x69, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6b,
0x74, 0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6b,
0x74, 0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6b, 0x74,
0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70,
0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0xe9,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0xe9,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0xe9, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65,
0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6e, 0xed, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159,
0x65, 0x64, 0x6f, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6e, 0xed, 0x20, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6e,
0xed, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd,
0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f,
0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70,
0x61, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0xe1,
0x7a, 0x65, 0x72, 0x62, 0xe1, 0x6a, 0x64, 0x17e, 0xe1, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0xe1, 0x7a, 0x65, 0x72, 0x62, 0xe1, 0x6a,
0x64, 0x17e, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0xe1, 0x7a,
0x65, 0x72, 0x62, 0xe1, 0x6a, 0x64, 0x17e, 0xe1, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a,
0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x6f,
0x72, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x6f, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9, 0x161, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9,
0x161, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0xe9, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x68, 0xfa, 0x74, 0xe1, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x6f, 0x6c, 0x69, 0x76, 0x69,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x62, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72,
0x75, 0x6e, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b,
0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61,
0x73, 0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x68, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x63, 0x65, 0x63, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6c, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x6f, 0x6a, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d,
0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x56, 0xe1, 0x6e, 0x6f, 0x10d, 0x6e, 0xed, 0x68,
0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x10d, 0x61, 0x73,
0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x6b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x16f, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x16f, 0x6b, 0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x6b, 0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61,
0x73, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x6f, 0x76, 0x79, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x63, 0x65, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x63, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74,
0x61, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x61, 0x76,
0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x56, 0x65,
0x6c, 0x69, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xed, 0x68, 0x6f, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x56, 0x65,
0x6c, 0x69, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xed, 0x68, 0x6f, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f, 0x6e,
0x6f, 0x10d, 0x6e, 0xed, 0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x61, 0x65, 0x6b, 0x76, 0xe1, 0x64, 0x6f, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x65, 0x76,
0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74,
0x159, 0x65, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6f, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x76,
0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd,
0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x64, 0xe1, 0x6c, 0x6e, 0x11b, 0x76, 0xfd,
0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x76,
0x72, 0x6f, 0x70, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17e, 0x69, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17e, 0x69,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17e, 0x69,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x75, 0x7a, 0x73, 0x6b,
0x6f, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x10d, 0x61, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x75,
0x7a, 0x73, 0x6b, 0xfd, 0x63, 0x68, 0x20, 0x6a, 0x69, 0x17e, 0x6e, 0xed,
0x63, 0x68, 0x20, 0x61, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74,
0x69, 0x63, 0x6b, 0xfd, 0x63, 0x68, 0x20, 0xfa, 0x7a, 0x65, 0x6d, 0xed,
0x67, 0x61, 0x6c, 0x61, 0x70, 0xe1, 0x17e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x67, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f,
0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f,
0x67, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x159, 0x65, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64,
0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76,
0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72,
0xf3, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61,
0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x47, 0x75, 0x61, 0x6d, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x50, 0x65, 0x72,
0x73, 0x6b, 0xe9, 0x68, 0x6f, 0x20, 0x7a, 0xe1, 0x6c, 0x69, 0x76, 0x75,
0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65,
0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x61, 0x76,
0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b,
0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f,
0x76, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x76,
0x64, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x76, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x69, 0x6e, 0x64, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69,
0x6e, 0x64, 0x69, 0x63, 0x6b, 0x6f, 0x6f, 0x63, 0x65, 0xe1, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64, 0x6f, 0x10d, 0xed,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x159, 0x65,
0x64, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0xed, 0x72, 0xe1, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0xed, 0x72, 0xe1, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0xed, 0x72, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70,
0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70,
0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x6f,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x70, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x6f, 0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x2d, 0x6b, 0x61, 0x6d,
0x10d, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x2d,
0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f,
0x64, 0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0xe1, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64,
0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0xe1, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6b,
0x6f, 0x73, 0x72, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x79, 0x72, 0x67, 0x79, 0x7a, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x72, 0xed, 0x6c, 0x61, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x52,
0x6f, 0x76, 0x6e, 0xed, 0x6b, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0x61, 0x20, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20,
0x48, 0x6f, 0x77, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x61, 0x20, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x48, 0x6f, 0x77,
0x61, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x20, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x20, 0x48, 0x6f, 0x77, 0x61, 0x4d, 0x61, 0x63, 0x61, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x63, 0x61, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x63, 0x61, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x6c,
0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x6c,
0x65, 0x64, 0x69, 0x76, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d,
0x61, 0x72, 0x6b, 0xe9, 0x7a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x16f, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x6f, 0x76, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65,
0x6d, 0x65, 0x78, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x70, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78,
0x69, 0x63, 0x6b, 0xfd, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x6c, 0xe1, 0x6e,
0x62, 0xe1, 0x74, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x75, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x75, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x65, 0x76, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x65, 0x76, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x65, 0x76, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x79, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e,
0x61, 0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x65,
0x70, 0xe1, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f,
0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76,
0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76,
0x6f, 0x7a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0xe9, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0xe9,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x69, 0x75, 0x65, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x6f,
0x75, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0xed, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x6f, 0x75, 0x6f, 0x73,
0x74, 0x72, 0x6f, 0x76, 0xed, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x73,
0x6f, 0x75, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0xed, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x6d,
0x61, 0x72, 0x69, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x6f, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6f, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6f, 0x6d,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x70, 0xe1, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x70, 0xe1, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x70, 0xe1, 0x6b, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x50, 0x61, 0x70, 0x75, 0x79,
0x2d, 0x4e, 0x6f, 0x76, 0xe9, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x79,
0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x73,
0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xed, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69,
0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x66, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x66, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73,
0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x70, 0x69, 0x65, 0x72, 0x72,
0x65, 0x2d, 0x6d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d,
0x6d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x70, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x6d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x50, 0x69,
0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x70, 0x6f, 0x6e, 0x61, 0x70,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x63, 0x68, 0x6a, 0x6f,
0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f,
0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x72, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x74,
0x68, 0x65, 0x72, 0x6f, 0x76, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x63, 0x65, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d,
0x6f, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d,
0x6f, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f,
0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x160,
0x61, 0x6c, 0x61, 0x6d, 0x6f, 0x75, 0x6e, 0x6f, 0x76, 0xfd, 0x63, 0x68,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x10d, 0x61, 0x73, 0x20,
0x4a, 0x69, 0x17e, 0x6e, 0xed, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x65, 0x73, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x63, 0x65, 0x20, 0x160, 0xf3, 0x77, 0x61, 0x74, 0x61, 0x68, 0x69, 0x74,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x63, 0x68, 0x61, 0x6a,
0x70, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x63,
0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x74, 0x63, 0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x74, 0xe1, 0x64,
0x17e, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74,
0x6f, 0x6e, 0x17e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f,
0x6e, 0x17e, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6e, 0x17e,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x63, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed,
0x20, 0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73,
0x74, 0x75, 0x76, 0x61, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x75,
0x7a, 0x62, 0x65, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a,
0x62, 0x65, 0x63, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a, 0x62, 0x65,
0x63, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d,
0x61, 0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x65, 0x6e,
0x65, 0x7a, 0x75, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x63, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x63, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x63, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0xfd,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20,
0x10d, 0x61, 0x73, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65,
0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x10d, 0x61, 0x73, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x10d,
0x61, 0x73, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x16f, 0x20, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61,
0x73, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x65, 0x6b, 0x61, 0x74,
0x11b, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x11b, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x65, 0x6b, 0x61,
0x74, 0x11b, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xed, 0x20, 0x10d, 0x61, 0x73, 0x79, 0x75, 0x6b,
0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x63, 0x72,
0x65, 0x2d, 0x74, 0x69, 0x64, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x63, 0x72, 0x65, 0x2d,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x66, 0x67,
0x68, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x53, 0x79, 0x64,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x74,
0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x2d, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x79, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73,
0x2d, 0x74, 0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d,
0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x74, 0x69, 0x64,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4d, 0x6f,
0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x6f,
0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x74, 0x69, 0x64, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x2d, 0x74, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x41, 0x70, 0x69,
0x61, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41,
0x70, 0x69, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x74, 0x69, 0x64, 0x41, 0x71,
0x74, 0x61, 0x75, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x2d, 0x74,
0x69, 0x64, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x2d, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x72,
0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x41, 0x72, 0x61,
0x62, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x56, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d,
0x74, 0x69, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x6c, 0x69, 0x67, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x6c,
0x69, 0x67, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x6c, 0x69, 0x67,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61,
0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x72,
0x6e, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x2d, 0x74,
0x69, 0x64, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x2d, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e,
0x2d, 0x74, 0x69, 0x64, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61,
0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x2d, 0x74, 0x69, 0x64,
0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x74, 0x69,
0x64, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x70, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x2d,
0x74, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x74,
0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x4b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x4b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x69, 0x6e, 0x65, 0x73, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x54, 0x73, 0x6a, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x2d,
0x74, 0x69, 0x64, 0x54, 0x73, 0x6a, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x54, 0x73, 0x6a, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x2d,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4a, 0x75, 0x6c,
0x65, 0xf8, 0x65, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0xf8, 0x65, 0x72, 0x6e, 0x65,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0xf8,
0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x43, 0x6f, 0x6f, 0x6b,
0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x65, 0x72, 0x6e, 0x65,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x43, 0x75,
0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x43, 0x75, 0x62,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x43, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x44, 0x61, 0x76, 0x69, 0x73,
0x2d, 0x74, 0x69, 0x64, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x74, 0x69, 0x64,
0xd8, 0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x2d, 0x74, 0x69, 0x64,
0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x65, 0x6e, 0x2d, 0x74, 0x69, 0x64,
0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x65, 0x6e, 0x2d, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8,
0x65, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x69, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe6, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72,
0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe6, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0xd8, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x46, 0x6a, 0x65, 0x72, 0x6e, 0xf8, 0x73, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x56, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe6, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0xe6, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x65, 0x72, 0x6e, 0x65,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x65, 0x72, 0x6e, 0x65,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x46, 0x69,
0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x46, 0x69,
0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x46, 0x69, 0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x46, 0x72,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x2d,
0x74, 0x69, 0x64, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x53,
0x79, 0x64, 0x6c, 0x69, 0x67, 0x65, 0x20, 0x6f, 0x67, 0x20, 0x41, 0x6e,
0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x6b, 0x65, 0x20, 0x54, 0x65,
0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x65, 0x72, 0x2d, 0x74, 0x69,
0x64, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x2d, 0x74,
0x69, 0x64, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x2d, 0x74, 0x69,
0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0xf8, 0x65, 0x72, 0x6e,
0x65, 0x2d, 0x74, 0x69, 0x64, 0x47, 0x4d, 0x54, 0xd8, 0x73, 0x74, 0x67,
0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0xd8, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0xd8, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x56,
0x65, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf8,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf8,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x47, 0x75, 0x61, 0x6d, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x47, 0x6f, 0x6c, 0x66, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x6e, 0x65, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x74, 0x69,
0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x67, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x76,
0x64, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x48,
0x6f, 0x76, 0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x6b, 0x65,
0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x61,
0x2d, 0x74, 0x69, 0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0xd8, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x49, 0x72,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x2d, 0x74, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x74, 0x69, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0xd8, 0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x65, 0x73, 0x74, 0x6b,
0x61, 0x73, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x2d, 0x74, 0x69,
0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x2d, 0x74, 0x69, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4b, 0x69,
0x72, 0x67, 0x69, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4c,
0x61, 0x6e, 0x67, 0x61, 0x20, 0x74, 0x69, 0x64, 0x4c, 0x69, 0x6e, 0x6a,
0x65, 0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61,
0x63, 0x61, 0x6f, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d,
0x74, 0x69, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x2d, 0x74, 0x69,
0x64, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x72, 0x6e, 0x65, 0x2d,
0x74, 0x69, 0x64, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73,
0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x65, 0x78, 0x69, 0x63,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x2d,
0x74, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76,
0x61, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2d,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x6f, 0x73,
0x6b, 0x76, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x2d, 0x74, 0x69, 0x64,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x2d, 0x74, 0x69, 0x64, 0x4e, 0x65, 0x70,
0x61, 0x6c, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4e,
0x79, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x6e,
0x2d, 0x74, 0x69, 0x64, 0x4e, 0x79, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x4e, 0x79, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x65, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x4e, 0x65, 0x77, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4e, 0x65, 0x77, 0x7a, 0x65, 0x61,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x4e, 0x65, 0x77, 0x7a, 0x65, 0x61, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4e, 0x69, 0x75, 0x65,
0x2d, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x69, 0x64, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x64, 0x6d,
0x61, 0x72, 0x69, 0x61, 0x6e, 0x65, 0x72, 0x6e, 0x65, 0x2d, 0x74, 0x69,
0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x2d, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4f, 0x6d,
0x73, 0x6b, 0x2d, 0x74, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b,
0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x2d, 0x74, 0x69,
0x64, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x79, 0x20, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75,
0x76, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x50, 0x65,
0x72, 0x75, 0x76, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x76, 0x69,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x50, 0x68, 0x6f, 0x65,
0x6e, 0x69, 0x78, 0xf8, 0x65, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x20,
0x6f, 0x67, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d,
0x74, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x2d, 0x20, 0x6f, 0x67, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x2d, 0x20, 0x6f, 0x67, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x2d, 0x74, 0x69,
0x64, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x2d, 0x74, 0x69, 0x64,
0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x74, 0x69,
0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x51, 0x79, 0x7a, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x2d, 0x74,
0x69, 0x64, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x2d, 0x74, 0x69,
0x64, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x74, 0x69,
0x64, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x53, 0x65, 0x79, 0x63, 0x68,
0x65, 0x6c, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x2d, 0x74, 0x69, 0x64, 0x53,
0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0xf8, 0x65, 0x72, 0x6e, 0x65, 0x2d,
0x74, 0x69, 0x64, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x2d, 0x74, 0x69, 0x64, 0x53, 0x79, 0x6f, 0x77, 0x61,
0x2d, 0x74, 0x69, 0x64, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x2d, 0x74,
0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x74, 0x69, 0x64,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x54, 0x61, 0x64, 0x73,
0x6a, 0x69, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x54, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x2d, 0x74, 0x69, 0x64, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x43, 0x68, 0x75,
0x75, 0x6b, 0x2d, 0x74, 0x69, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x54, 0x75, 0x76,
0x61, 0x6c, 0x75, 0x2d, 0x74, 0x69, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x55, 0x73,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d,
0x74, 0x69, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74, 0x69, 0x64, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d,
0x74, 0x69, 0x64, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d,
0x74, 0x69, 0x64, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x74, 0x69, 0x64, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73,
0x20, 0x6f, 0x67, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x2d, 0x74,
0x69, 0x64, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x74, 0x69,
0x64, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x2d, 0x74, 0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x2d, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x64, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940,
0x20, 0x915, 0x947, 0x902, 0x926, 0x930, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915,
0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x915, 0x947, 0x902, 0x926, 0x930,
0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940,
0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x915, 0x947, 0x902, 0x926, 0x930, 0x940, 0x20, 0x938, 0x92e,
0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930,
0x940, 0x915, 0x940, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x938,
0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e,
0x930, 0x940, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x92a, 0x942,
0x930, 0x94d, 0x935, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d,
0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940,
0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20,
0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x909, 0x902, 0x91f,
0x947, 0x928, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930,
0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x92e, 0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x938, 0x92e,
0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930,
0x940, 0x915, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92e,
0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940,
0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x938, 0x92e, 0x93e,
0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940,
0x915, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x936,
0x93e, 0x902, 0x924, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x940, 0x915, 0x940, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924,
0x20, 0x938, 0x92e, 0x93e, 0x902, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f,
0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f,
0x93f, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902,
0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x915, 0x947, 0x902, 0x926,
0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x938, 0x92e,
0x93e, 0x902, 0x915, 0x947, 0x902, 0x926, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930,
0x94b, 0x92a, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e,
0x902, 0x915, 0x947, 0x902, 0x926, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x20, 0x917, 0x930, 0x94d, 0x92e, 0x93f, 0x92f, 0x947, 0x902, 0x20,
0x926, 0x93e, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x909, 0x924, 0x94d, 0x924, 0x930,
0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x938, 0x92e, 0x93e,
0x902, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a,
0x940, 0x20, 0x917, 0x930, 0x94d, 0x92e, 0x93f, 0x92f, 0x947, 0x902, 0x20, 0x926,
0x93e, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x92a, 0x91a, 0x94d, 0x91b, 0x92e, 0x940,
0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x938, 0x92e, 0x93e, 0x902,
0x92a, 0x91a, 0x94d, 0x91b, 0x92e, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a,
0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x93e, 0x902, 0x92a,
0x91a, 0x94d, 0x91b, 0x92e, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940,
0x20, 0x917, 0x930, 0x94d, 0x92e, 0x93f, 0x92f, 0x947, 0x902, 0x20, 0x926, 0x93e,
0x20, 0x938, 0x92e, 0x93e, 0x902, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x74, 0x69,
0x6a, 0x64, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x63, 0x72, 0x65,
0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x66,
0x67, 0x68, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x5a, 0x75, 0x69, 0x64,
0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x2d, 0x41, 0x74,
0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x2d, 0x41,
0x74, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61,
0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x65, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x74, 0x69, 0x6a, 0x64,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x74, 0x69,
0x6a, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e,
0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69,
0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x70,
0x69, 0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x70, 0x69, 0x61, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x71, 0x74, 0x61,
0x75, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41,
0x71, 0x74, 0xf6, 0x62, 0x65, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x71,
0x74, 0xf6, 0x62, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65,
0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72,
0x61, 0x62, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41,
0x72, 0x61, 0x62, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74,
0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6a, 0x6e, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x57,
0x65, 0x73, 0x74, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6a,
0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x63, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x63, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d,
0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d,
0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64,
0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x77, 0x65, 0x73,
0x74, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x77, 0x65, 0x73, 0x74,
0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64,
0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x77, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6a,
0x6b, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f,
0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64,
0x7a, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x6a, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x69,
0x64, 0x7a, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e,
0x2d, 0x74, 0x69, 0x6a, 0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x65, 0x6e, 0x67, 0x61,
0x6c, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c,
0x65, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x42, 0x68, 0x75, 0x74, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x72, 0x61, 0x7a, 0x69,
0x6c, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x4b, 0x61, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x61, 0x61, 0x70, 0x76,
0x65, 0x72, 0x64, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x61,
0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x61, 0x73,
0x65, 0x79, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x6f, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65, 0x20, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x73, 0x6a, 0x6f,
0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64,
0x54, 0x73, 0x6a, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x54, 0x73, 0x6a, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68,
0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x65, 0x20, 0x68, 0x61, 0x6c, 0x76, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x75, 0x62, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x75, 0x62, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x75, 0x62, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x44,
0x61, 0x76, 0x69, 0x73, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x54,
0x69, 0x6d, 0x6f, 0x72, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x50, 0x61, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x61, 0x73, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x61, 0x73, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69,
0x64, 0x64, 0x65, 0x6e, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d,
0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73,
0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x56, 0x65, 0x72, 0x64,
0x65, 0x72, 0x2d, 0x6f, 0x6f, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6a, 0x6b,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x57,
0x65, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x73, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x46, 0x69, 0x6a, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x46, 0x69, 0x6a, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x46, 0x69, 0x6a, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x72, 0x61, 0x6e,
0x73, 0x2d, 0x47, 0x75, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x75,
0x69, 0x64, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65, 0x20, 0x65, 0x6e, 0x20,
0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x65, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73,
0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x57,
0x65, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73, 0x74,
0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x47, 0x75, 0x61, 0x6d, 0x65, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x47, 0x6f, 0x6c, 0x66, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x6f, 0x65, 0x74, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f,
0x6e, 0x67, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x74, 0x69,
0x6a, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x48, 0x6f, 0x76, 0x64,
0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x6e,
0x64, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x61,
0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68,
0x69, 0x6e, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65,
0x73, 0x74, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x72, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x72, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x72,
0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x49,
0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x72, 0x6b,
0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x73, 0x72, 0x61, 0xeb,
0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x49, 0x73, 0x72, 0x61,
0xeb, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x4f, 0x6f, 0x73, 0x74, 0x2d, 0x4b, 0x61, 0x7a, 0x61,
0x63, 0x68, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x65, 0x73,
0x74, 0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x6b, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x7a, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x4c, 0x69, 0x6e, 0x65, 0x2d, 0x65, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72,
0x64, 0x74, 0x69, 0x6a, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x63,
0x61, 0x75, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x63,
0x61, 0x75, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x73,
0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x4d, 0x61, 0x6c, 0x65, 0x69, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x65, 0x78, 0x69, 0x63,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x2d, 0x7a, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x6f, 0x73,
0x6b, 0x6f, 0x75, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x6f, 0x73, 0x6b,
0x6f, 0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x77, 0x2d, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x77, 0x2d, 0x43, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x69,
0x65, 0x75, 0x77, 0x2d, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x6a, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x77, 0x2d, 0x5a, 0x65, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x69,
0x65, 0x75, 0x77, 0x2d, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74,
0x69, 0x6a, 0x64, 0x4e, 0x69, 0x65, 0x75, 0x77, 0x2d, 0x5a, 0x65, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x69, 0x75,
0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64,
0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x20, 0x4d,
0x61, 0x72, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x2d, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x6a, 0x64,
0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x42, 0x65,
0x6c, 0x61, 0x75, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61,
0x70, 0x6f, 0x65, 0x61, 0x2d, 0x4e, 0x69, 0x65, 0x75, 0x77, 0x2d, 0x47,
0x75, 0x69, 0x6e, 0x65, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x65, 0x72, 0x75,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x65,
0x72, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x50, 0x65, 0x72,
0x75, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6a, 0x6e,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6a, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x6a, 0x64, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x65, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d,
0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x6a, 0x64, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e,
0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x2d, 0x74,
0x69, 0x6a, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x2d, 0x74, 0x69, 0x6a, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x52,
0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x74, 0x69,
0x6a, 0x64, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x2d, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a,
0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x61, 0x6e,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x61,
0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73,
0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x5a, 0x75, 0x69, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x53, 0x75, 0x72,
0x69, 0x6e, 0x61, 0x61, 0x6d, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x74, 0x69, 0x6a,
0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x54, 0x61, 0x64, 0x7a, 0x6a, 0x69, 0x65, 0x6b, 0x73, 0x65, 0x20,
0x74, 0x69, 0x6a, 0x64, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x2d,
0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74,
0x69, 0x6a, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x43, 0x68, 0x75,
0x75, 0x6b, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x65,
0x20, 0x74, 0x69, 0x6a, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b,
0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x65, 0x7a, 0x62, 0x65,
0x65, 0x6b, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61,
0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4f, 0x65, 0x7a, 0x62, 0x65, 0x65,
0x6b, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x61, 0x6e, 0x73,
0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x6f,
0x6c, 0x61, 0x61, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a, 0x64, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74,
0x69, 0x6a, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64,
0x74, 0x69, 0x6a, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x2d, 0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a,
0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x74,
0x69, 0x6a, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69,
0x6a, 0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d,
0x7a, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x56, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0x57, 0x61, 0x6b, 0x65,
0x2d, 0x65, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x20, 0x74, 0x69,
0x6a, 0x64, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x6e, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x73, 0x65, 0x20, 0x74, 0x69, 0x6a,
0x64, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x74, 0x69,
0x6a, 0x64, 0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64,
0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x7a, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x74, 0x69,
0x6a, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x65,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x64, 0x74, 0x69, 0x6a, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x7a,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x6a, 0x64, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x2d, 0x74, 0x69, 0x6a, 0x64, 0xf68, 0xf55, 0xf0b, 0xf42, 0xf0b, 0xf53,
0xf72, 0xf0b, 0xf66, 0xf84, 0xf4f, 0xf71, 0xf53, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b,
0xf68, 0xf55, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf71, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b,
0xf68, 0xf55, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf71, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf63, 0xfb7, 0xf7c, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66,
0xf0b, 0xf68, 0xf55, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf71, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42,
0xf66, 0xf0b, 0xf68, 0xf55, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf71, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c,
0xf42, 0xf66, 0xf0b, 0xf68, 0xf55, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf71, 0xf0b,
0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68,
0xf55, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf71, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62,
0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68,
0xf0b, 0xf63, 0xf66, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf68, 0xf0b, 0xf63, 0xf66, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b,
0xf63, 0xf66, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2,
0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b,
0xf58, 0xf0b, 0xf5b, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf68, 0xf7a, 0xf0b, 0xf58, 0xf0b, 0xf5b, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf68, 0xf7a, 0xf0b, 0xf58, 0xf0b, 0xf5b, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf56, 0xfb1,
0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72,
0xf0b, 0xf40, 0xf0b, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42,
0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b,
0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf51, 0xf56,
0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1,
0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b,
0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf49,
0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62,
0xf72, 0xf0b, 0xf40, 0xf0b, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68,
0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf64, 0xf62, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68,
0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf64, 0xf62, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2,
0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44,
0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf58,
0xf60, 0xf74, 0xf0b, 0xf4a, 0xf7a, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72,
0xf0b, 0xf40, 0xf0b, 0xf58, 0xf60, 0xf74, 0xf0b, 0xf4a, 0xf7a, 0xf53, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b,
0xf40, 0xf0b, 0xf58, 0xf60, 0xf74, 0xf0b, 0xf4a, 0xf7a, 0xf53, 0xf0b, 0xf49, 0xf72,
0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72,
0xf0b, 0xf40, 0xf0b, 0xf54, 0xf7a, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf55, 0xf72, 0xf42,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b, 0xf68,
0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf54, 0xf7a, 0xf0b,
0xf66, 0xf72, 0xf0b, 0xf55, 0xf72, 0xf42, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb1, 0xf44, 0xf0b,
0xf68, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf40, 0xf0b, 0xf54, 0xf7a,
0xf0b, 0xf66, 0xf72, 0xf0b, 0xf55, 0xf72, 0xf42, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b,
0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68,
0xf0b, 0xf62, 0xf7a, 0xf0b, 0xf56, 0xfb7, 0xf72, 0xf0b, 0xf61, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf62, 0xf7a, 0xf0b, 0xf56, 0xfb7,
0xf72, 0xf0b, 0xf61, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf62, 0xf7a, 0xf0b, 0xf56,
0xfb7, 0xf72, 0xf0b, 0xf61, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf47, 0xf7a, 0xf53, 0xf0b,
0xf4a, 0xf72, 0xf0b, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68,
0xf62, 0xf0b, 0xf47, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf53, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf68, 0xf62, 0xf0b, 0xf47, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf53, 0xf0b,
0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66,
0xf0b, 0xf68, 0xf62, 0xf0b, 0xf47, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf0b, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55,
0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf62, 0xf0b, 0xf47, 0xf7a, 0xf53, 0xf0b,
0xf4a, 0xf72, 0xf0b, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1,
0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf62, 0xf0b, 0xf47, 0xf7a, 0xf53, 0xf0b, 0xf4a,
0xf72, 0xf0b, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf58, 0xf72,
0xf0b, 0xf53, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf68, 0xf62, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf53, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf68, 0xf62, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf53, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf56,
0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf68, 0xf7a, 0xf4a, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72,
0xf40, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf4a, 0xf0b,
0xf63, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf40, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf4a,
0xf0b, 0xf63, 0xf7a, 0xf53, 0xf0b, 0xf4a, 0xf72, 0xf40, 0xf0b, 0xf49, 0xf72, 0xf53,
0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68,
0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b,
0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66,
0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b,
0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51,
0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51,
0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71,
0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61,
0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf53, 0xf74, 0xf56, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a,
0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf53, 0xf74, 0xf56, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a,
0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56,
0xf74, 0xf66, 0xf0b, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66,
0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63,
0xf72, 0xf0b, 0xf61, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1,
0xf7c, 0xf42, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71,
0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1,
0xf7c, 0xf42, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71,
0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2, 0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf55,
0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf71, 0xf7c, 0xf66, 0xf0b, 0xf4a, 0xfb2,
0xf7a, 0xf63, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b,
0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68,
0xf0b, 0xf5b, 0xf62, 0xf0b, 0xf56, 0xfb7, 0xf60, 0xf72, 0xf0b, 0xf47, 0xf71, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b, 0xf5b, 0xf62, 0xf0b,
0xf56, 0xfb7, 0xf60, 0xf72, 0xf0b, 0xf47, 0xf71, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf0b,
0xf5b, 0xf62, 0xf0b, 0xf56, 0xfb7, 0xf60, 0xf72, 0xf0b, 0xf47, 0xf71, 0xf53, 0xf0b,
0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b, 0xf5b, 0xf7c, 0xf62, 0xf66, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b, 0xf5b, 0xf7c, 0xf62, 0xf66,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf68, 0xf7a, 0xf0b, 0xf5b, 0xf7c, 0xf62, 0xf66, 0xf0b, 0xf56, 0xfb1,
0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf56, 0xf44, 0xf92, 0xf0b, 0xf63, 0xf0b, 0xf51, 0xf7a, 0xf64, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf44, 0xf92, 0xf0b, 0xf63, 0xf0b, 0xf51,
0xf7a, 0xf64, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf44, 0xf92, 0xf0b, 0xf63, 0xf0b, 0xf51, 0xf7a,
0xf64, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf60, 0xf56, 0xfb2, 0xf74, 0xf42, 0xf0b, 0xf61,
0xf74, 0xf63, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb7, 0xf7c,
0xf0b, 0xf63, 0xf72, 0xf0b, 0xf56, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb2, 0xf0b, 0xf5b, 0xf72, 0xf0b, 0xf63, 0xf72, 0xf0b,
0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb2, 0xf0b, 0xf5b,
0xf72, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xfb2, 0xf0b, 0xf5b,
0xf72, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b,
0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7a,
0xf54, 0xf0b, 0xf56, 0xf71, 0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf40, 0xf7a, 0xf54, 0xf0b, 0xf56, 0xf71, 0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7a, 0xf54,
0xf0b, 0xf56, 0xf71, 0xf4c, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74,
0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf45, 0xf72, 0xf0b, 0xf63,
0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf45, 0xf72, 0xf0b, 0xf63,
0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf45, 0xf72, 0xf0b, 0xf63, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71,
0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf62, 0xf92, 0xfb1, 0xf0b, 0xf53, 0xf42, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf62, 0xf92, 0xfb1, 0xf0b, 0xf53, 0xf42, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf62, 0xf92, 0xfb1,
0xf0b, 0xf53, 0xf42, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf41, 0xf72, 0xf0b, 0xf62, 0xf72,
0xf66, 0xf9f, 0xf0b, 0xf58, 0xf7a, 0xf66, 0xf0b, 0xf58, 0xf5a, 0xf7c, 0xf0b, 0xf42,
0xfb3, 0xf72, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7c,
0xf0b, 0xf63, 0xf7c, 0xf58, 0xf0b, 0xf56, 0xfb7, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7c, 0xf0b, 0xf63, 0xf7c, 0xf58, 0xf0b,
0xf56, 0xfb7, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7c, 0xf0b, 0xf63, 0xf7c,
0xf58, 0xf0b, 0xf56, 0xfb7, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62,
0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40,
0xf72, 0xf60, 0xf74, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf40, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf72,
0xf60, 0xf74, 0xf0b, 0xf56, 0xfb7, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2,
0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf66,
0xf72, 0xf0b, 0xf4a, 0xf62, 0xf0b, 0xf68, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a,
0xf53, 0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf66,
0xf72, 0xf0b, 0xf4a, 0xf62, 0xf0b, 0xf68, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a,
0xf53, 0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf66, 0xf72, 0xf0b, 0xf4a, 0xf62, 0xf0b,
0xf68, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf56, 0xfb1,
0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf68, 0xf7a, 0xf0b, 0xf40, 0xf74, 0xf0b, 0xf4c, 0xf7d, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c,
0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b,
0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b,
0xf54, 0xf7a, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74, 0xf66, 0xf0b, 0xf55, 0xfb1,
0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a,
0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42,
0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c,
0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61,
0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71,
0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74,
0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b,
0xf61, 0xf74, 0xf0b, 0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf5a, 0xf51,
0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53,
0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf61, 0xf74, 0xf0b,
0xf62, 0xf7c, 0xf0b, 0xf54, 0xf7a, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b,
0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55, 0xf63,
0xf40, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf68, 0xf71, 0xf60, 0xf72, 0xf0b,
0xf63, 0xf7a, 0xf53, 0xf4c, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf55, 0xf63, 0xf40, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf68, 0xf71, 0xf60,
0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf66, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55, 0xf63, 0xf40,
0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf68, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf63,
0xf7a, 0xf53, 0xf4c, 0xf66, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74,
0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55, 0xf72, 0xf62, 0xf7a,
0xf53, 0xf45, 0xf0b, 0xf42, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf42, 0xf0b, 0xf63, 0xf0b, 0xf54, 0xf0b, 0xf42, 0xf7c,
0xf66, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf47, 0xf7d, 0xf0b,
0xf47, 0xf71, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf47, 0xf7d, 0xf0b,
0xf47, 0xf71, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf47, 0xf7d, 0xf0b, 0xf47, 0xf71, 0xf0b, 0xf56, 0xfb1,
0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf42, 0xf72, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf5d, 0xf72, 0xf46, 0xf0b, 0xf63,
0xf74, 0xf0b, 0xf61, 0xf7c, 0xf51, 0xf0b, 0xf54, 0xf60, 0xf72, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66,
0xf0b, 0xf42, 0xf72, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c,
0xf42, 0xf66, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53,
0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b,
0xf42, 0xf72, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf56,
0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b,
0xf42, 0xf72, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c,
0xf42, 0xf66, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53,
0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf53, 0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66,
0xf0b, 0xf42, 0xf72, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b,
0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf42, 0xf71, 0xf63, 0xf55, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf42, 0xf74, 0xf0b, 0xf61, 0xf0b, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf67, 0xf0b, 0xf5d, 0xf60, 0xf72, 0xf0b, 0x2d, 0xf68,
0xf7a, 0xf0b, 0xf63, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf64, 0xf7a, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf67, 0xf0b, 0xf5d, 0xf60, 0xf72, 0xf0b, 0x2d,
0xf68, 0xf7a, 0xf0b, 0xf63, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf64, 0xf7a, 0xf53, 0xf0b,
0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf67, 0xf0b, 0xf5d, 0xf60, 0xf72, 0xf0b, 0x2d, 0xf68, 0xf7a, 0xf0b, 0xf63,
0xf72, 0xf60, 0xf74, 0xf0b, 0xf64, 0xf7a, 0xf53, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b,
0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf62,
0xf92, 0xfb1, 0xf0b, 0xf42, 0xf62, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf62, 0xf92, 0xfb1, 0xf0b, 0xf42, 0xf62, 0xf0b, 0xf42, 0xfb1, 0xf72, 0xf0b, 0xf62,
0xf92, 0xfb1, 0xf0b, 0xf58, 0xf5a, 0xf7c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf68, 0xf72, 0xf53, 0xf0b, 0xf4c, 0xf7c, 0xf0b, 0xf45, 0xf71, 0xf60, 0xf72,
0xf0b, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf51, 0xf56, 0xf74,
0xf66, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf72, 0xf53, 0xf0b,
0xf4c, 0xf7c, 0xf0b, 0xf53, 0xf7a, 0xf0b, 0xf64, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf64, 0xf62, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42,
0xf66, 0xf0b, 0xf68, 0xf72, 0xf53, 0xf0b, 0xf4c, 0xf7c, 0xf0b, 0xf53, 0xf7a, 0xf0b,
0xf64, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53,
0xf74, 0xf56, 0xf0b, 0xf55, 0xfb1, 0xf7c, 0xf42, 0xf66, 0xf0b, 0xf68, 0xf72, 0xf53,
0xf0b, 0xf4c, 0xf7c, 0xf0b, 0xf53, 0xf7a, 0xf0b, 0xf64, 0xf72, 0xf0b, 0xf61, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf0b, 0xf62, 0xf71, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf0b, 0xf62, 0xf71,
0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72, 0xf0b, 0xf62, 0xf71, 0xf53, 0xf0b, 0xf49, 0xf72,
0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf68, 0xf62, 0xf0b, 0xf40, 0xf74, 0xf59, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf68, 0xf62, 0xf0b, 0xf40, 0xf74, 0xf59, 0xf72, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf68, 0xf62, 0xf0b, 0xf40, 0xf74, 0xf59, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62,
0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68,
0xf72, 0xf66, 0xf0b, 0xf62, 0xf7a, 0xf63, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf68, 0xf72, 0xf66, 0xf0b, 0xf62, 0xf7a, 0xf63, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf72,
0xf66, 0xf0b, 0xf62, 0xf7a, 0xf63, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2,
0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf47, 0xf0b, 0xf54,
0xf71, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf47, 0xf0b, 0xf54,
0xf71, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf47, 0xf0b, 0xf54, 0xf71, 0xf53, 0xf0b, 0xf49, 0xf72,
0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf40, 0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf40, 0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf40, 0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf61, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b,
0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40,
0xf62, 0xf66, 0xf72, 0xf0b, 0xf53, 0xf7c, 0xf0b, 0xf61, 0xf62, 0xf66, 0xf40, 0xf72,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf62, 0xf66, 0xf72, 0xf0b,
0xf53, 0xf7c, 0xf0b, 0xf61, 0xf62, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf40, 0xf62,
0xf66, 0xf72, 0xf0b, 0xf53, 0xf7c, 0xf0b, 0xf61, 0xf62, 0xf66, 0xf40, 0xf72, 0xf0b,
0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf58, 0xf0b, 0xf42, 0xfb7, 0xf0b, 0xf51, 0xf71, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf0b, 0xf42, 0xfb7, 0xf0b, 0xf51,
0xf71, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf0b, 0xf42, 0xfb7, 0xf0b, 0xf51, 0xf71, 0xf53,
0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf71, 0xf63, 0xf0b, 0xf51, 0xf72, 0xf56, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf7c, 0xf0b, 0xf62, 0xf72,
0xf0b, 0xf64, 0xf71, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58,
0xf7c, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf64, 0xf71, 0xf66, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf7c,
0xf0b, 0xf62, 0xf72, 0xf0b, 0xf64, 0xf71, 0xf66, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62,
0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58,
0xf7d, 0xf66, 0xf0b, 0xf40, 0xf7c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf58, 0xf7d, 0xf66, 0xf0b, 0xf40, 0xf7c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1,
0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf58, 0xf7d, 0xf66, 0xf0b,
0xf40, 0xf7c, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf7a, 0xf0b, 0xf54, 0xf71, 0xf63,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf72, 0xf60, 0xf74, 0xf0b,
0xf5b, 0xf72, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf53, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf5b, 0xf72, 0xf0b, 0xf63, 0xf7a,
0xf53, 0xf4c, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf5b, 0xf72, 0xf0b,
0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74,
0xf44, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf72, 0xf60, 0xf74,
0xf0b, 0xf55, 0xf60, 0xf74, 0xf53, 0xf4c, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf72, 0xf60, 0xf74, 0xf0b, 0xf55,
0xf60, 0xf74, 0xf53, 0xf4c, 0xf0b, 0xf63, 0xf7a, 0xf53, 0xf4c, 0xf0b, 0xf5a, 0xf51,
0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53,
0xf72, 0xf60, 0xf74, 0xf0b, 0xf55, 0xf60, 0xf74, 0xf53, 0xf4c, 0xf0b, 0xf63, 0xf7a,
0xf53, 0xf4c, 0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf55, 0xf62, 0xf0b, 0xf53, 0xf7a, 0xf53,
0xf0b, 0xf4c, 0xf7c, 0xf0b, 0x20, 0xf4c, 0xf72, 0xf0b, 0x20, 0xf53, 0xf7c, 0xf0b,
0xf62, 0xf71, 0xf7c, 0xf53, 0xf0b, 0xf67, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf55, 0xf62, 0xf0b, 0xf53, 0xf7a, 0xf53, 0xf0b, 0xf4c, 0xf7c, 0xf0b, 0x20,
0xf4c, 0xf72, 0xf0b, 0x20, 0xf53, 0xf7c, 0xf0b, 0xf62, 0xf71, 0xf7c, 0xf53, 0xf0b,
0xf67, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf55, 0xf62, 0xf0b, 0xf53, 0xf7a, 0xf53, 0xf0b, 0xf4c, 0xf7c,
0xf0b, 0x20, 0xf4c, 0xf72, 0xf0b, 0x20, 0xf53, 0xf7c, 0xf0b, 0xf62, 0xf71, 0xf7c,
0xf53, 0xf0b, 0xf67, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf7c, 0xf0b, 0xf56, 0xf7c,
0xf0b, 0xf66, 0xf72, 0xf0b, 0xf56, 0xf72, 0xf62, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf7c, 0xf0b, 0xf56, 0xf7c, 0xf0b, 0xf66,
0xf72, 0xf0b, 0xf56, 0xf72, 0xf62, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf53, 0xf7c,
0xf0b, 0xf56, 0xf7c, 0xf0b, 0xf66, 0xf72, 0xf0b, 0xf56, 0xf72, 0xf62, 0xf66, 0xf40,
0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7c, 0xf58, 0xf66, 0xf40, 0xf72, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf68, 0xf7c, 0xf58, 0xf66, 0xf40, 0xf72,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf68, 0xf7c, 0xf58, 0xf66, 0xf40, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71,
0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf54, 0xf0b, 0xf40, 0xf72, 0xf0b, 0xf66, 0xf84, 0xf4f, 0xf71, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf40, 0xf72, 0xf0b, 0xf66, 0xf84,
0xf4f, 0xf71, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf40, 0xf72, 0xf0b, 0xf66, 0xf84,
0xf4f, 0xf71, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf62, 0xf0b, 0xf42,
0xf74, 0xf0b, 0xf5d, 0xf60, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf54, 0xf0b, 0xf62, 0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d, 0xf60, 0xf72, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf54, 0xf0b, 0xf62, 0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d, 0xf60, 0xf72, 0xf0b, 0xf56,
0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf54, 0xf0b, 0xf62, 0xf74, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf54, 0xf0b, 0xf62, 0xf74, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf0b, 0xf62, 0xf74, 0xf0b,
0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf54, 0xf60, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf51, 0xf44,
0xf0b, 0xf58, 0xf72, 0xf0b, 0xf40, 0xf72, 0xf0b, 0xf63, 0xf71, 0xf7c, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf60, 0xf72, 0xf0b, 0xf62, 0xf72,
0xf0b, 0xf51, 0xf44, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf40, 0xf72, 0xf0b, 0xf63, 0xf71,
0xf7c, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf54, 0xf60, 0xf72, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf51,
0xf44, 0xf0b, 0xf58, 0xf72, 0xf0b, 0xf40, 0xf72, 0xf0b, 0xf63, 0xf71, 0xf7c, 0xf53,
0xf0b, 0xf49, 0xf72, 0xf53, 0xf0b, 0xf66, 0xfb2, 0xf74, 0xf44, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf62, 0xf72, 0xf0b, 0xf61, 0xf74, 0xf0b, 0xf53, 0xf72,
0xf0b, 0xf61, 0xf71, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf66,
0xf0b, 0xf41, 0xf0b, 0xf63, 0xf72, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf66, 0xf0b, 0xf41, 0xf0b, 0xf63, 0xf72, 0xf53, 0xf0b, 0xf5a, 0xf51, 0xf0b,
0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf66, 0xf0b,
0xf41, 0xf0b, 0xf63, 0xf72, 0xf53, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51,
0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf66, 0xf7a, 0xf0b,
0xf64, 0xf71, 0xf63, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf66,
0xf74, 0xf0b, 0xf62, 0xf72, 0xf0b, 0xf53, 0xf71, 0xf58, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf74, 0xf0b, 0xf42, 0xf74, 0xf0b,
0xf5d, 0xf71, 0xf60, 0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61,
0xf74, 0xf0b, 0xf62, 0xf74, 0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d, 0xf71, 0xf60, 0xf72,
0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a,
0xf7c, 0xf51, 0xf61, 0xf74, 0xf0b, 0xf62, 0xf74, 0xf0b, 0xf42, 0xf74, 0xf0b, 0xf5d,
0xf71, 0xf60, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66,
0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf7a, 0xf0b, 0xf53, 0xf72,
0xf0b, 0xf5b, 0xf74, 0xf0b, 0xf5d, 0xf7a, 0xf0b, 0xf63, 0xf0b, 0xf46, 0xf74, 0xf0b,
0xf5a, 0xf7c, 0xf51, 0xf56, 0xf0b, 0xf63, 0xf0b, 0xf4c, 0xf72, 0xf0b, 0xf56, 0xf7c,
0xf66, 0xf72, 0xf0b, 0xf4f, 0xf7c, 0xf40, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf56, 0xf0b, 0xf63, 0xf0b, 0xf4c, 0xf72, 0xf0b, 0xf56, 0xf7c, 0xf66, 0xf72,
0xf0b, 0xf4f, 0xf7c, 0xf40, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf0b, 0xf63, 0xf0b, 0xf4c, 0xf72,
0xf0b, 0xf56, 0xf7c, 0xf66, 0xf72, 0xf0b, 0xf4f, 0xf7c, 0xf40, 0xf0b, 0xf56, 0xfb1,
0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c,
0xf51, 0xf56, 0xf71, 0xf7c, 0xf63, 0xf0b, 0xf42, 0xf7c, 0xf0b, 0xf42, 0xf72, 0xf62,
0xf7a, 0xf4c, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf56, 0xf71, 0xf7c,
0xf63, 0xf0b, 0xf42, 0xf7c, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf7a, 0xf4c, 0xf0b, 0xf5a,
0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51,
0xf56, 0xf71, 0xf7c, 0xf63, 0xf0b, 0xf42, 0xf7c, 0xf0b, 0xf42, 0xf72, 0xf62, 0xf7a,
0xf4c, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46,
0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf0b, 0xf40, 0xf74, 0xf59, 0xf72, 0xf40,
0xf72, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf0b, 0xf40, 0xf74,
0xf59, 0xf72, 0xf40, 0xf72, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63, 0xfa1, 0xf53, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf0b, 0xf40, 0xf74, 0xf59, 0xf72,
0xf40, 0xf72, 0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b,
0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf7a, 0xf0b, 0xf40, 0xf0b, 0xf4f,
0xf7a, 0xf0b, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf56, 0xf62, 0xf42, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf7a, 0xf0b, 0xf40, 0xf0b, 0xf4f, 0xf7a, 0xf0b,
0xf62, 0xf72, 0xf53, 0xf0b, 0xf56, 0xf62, 0xf42, 0xf0b, 0xf5a, 0xf51, 0xf0b, 0xf63,
0xfa1, 0xf53, 0xf0b, 0xf46, 0xf74, 0xf0b, 0xf5a, 0xf7c, 0xf51, 0xf61, 0xf7a, 0xf0b,
0xf40, 0xf0b, 0xf4f, 0xf7a, 0xf0b, 0xf62, 0xf72, 0xf53, 0xf0b, 0xf56, 0xf62, 0xf42,
0xf0b, 0xf56, 0xfb1, 0xf71, 0xf62, 0xf0b, 0xf51, 0xf74, 0xf66, 0xf0b, 0xf46, 0xf74,
0xf0b, 0xf5a, 0xf7c, 0xf51, 0xd801, 0xdc0a, 0xd801, 0xdc4a, 0xd801, 0xdc30, 0xd801, 0xdc45,
0xd801, 0xdc3f, 0xd801, 0xdc32, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801,
0xdc0a, 0xd801, 0xdc4a, 0xd801, 0xdc30, 0xd801, 0xdc45, 0xd801, 0xdc3f, 0xd801, 0xdc32, 0x20,
0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32,
0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801,
0xdc0a, 0xd801, 0xdc4a, 0xd801, 0xdc30, 0xd801, 0xdc45, 0xd801, 0xdc3f, 0xd801, 0xdc32, 0x20,
0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801,
0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc1d, 0xd801, 0xdc2f, 0xd801, 0xdc4c, 0xd801,
0xdc3b, 0xd801, 0xdc49, 0xd801, 0xdc32, 0xd801, 0xdc4a, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34,
0xd801, 0xdc4b, 0xd801, 0xdc1d, 0xd801, 0xdc2f, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc49,
0xd801, 0xdc32, 0xd801, 0xdc4a, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801,
0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13,
0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc1d, 0xd801, 0xdc2f, 0xd801, 0xdc4c, 0xd801, 0xdc3b,
0xd801, 0xdc49, 0xd801, 0xdc32, 0xd801, 0xdc4a, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801,
0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b,
0xd801, 0xdc00, 0xd801, 0xdc45, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc4c,
0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc00, 0xd801, 0xdc45, 0xd801,
0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc4c, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b,
0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c,
0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc00, 0xd801, 0xdc45, 0xd801,
0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc4c, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29,
0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801,
0xdc4b, 0xd801, 0xdc23, 0xd801, 0xdc35, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801,
0xdc4c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc23, 0xd801, 0xdc35,
0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0x20, 0xd801, 0xdc1d, 0xd801,
0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801,
0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc23, 0xd801, 0xdc35,
0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0x20, 0xd801, 0xdc14, 0xd801,
0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34,
0xd801, 0xdc4b, 0xd801, 0xdc11, 0xd801, 0xdc32, 0xd801, 0xdc45, 0xd801, 0xdc2e, 0xd801, 0xdc41,
0xd801, 0xdc2e, 0xd801, 0xdc3f, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801,
0xdc11, 0xd801, 0xdc32, 0xd801, 0xdc45, 0xd801, 0xdc2e, 0xd801, 0xdc41, 0xd801, 0xdc2e, 0xd801,
0xdc3f, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c,
0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801,
0xdc4b, 0xd801, 0xdc11, 0xd801, 0xdc32, 0xd801, 0xdc45, 0xd801, 0xdc2e, 0xd801, 0xdc41, 0xd801,
0xdc2e, 0xd801, 0xdc3f, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34,
0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc08, 0xd801,
0xdc3b, 0xd801, 0xdc4a, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc2e, 0xd801,
0xdc3f, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc08, 0xd801, 0xdc3b,
0xd801, 0xdc4a, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3b, 0xd801, 0xdc2e, 0xd801, 0xdc3f,
0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801,
0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b,
0xd801, 0xdc08, 0xd801, 0xdc3b, 0xd801, 0xdc4a, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3b,
0xd801, 0xdc2e, 0xd801, 0xdc3f, 0x20, 0xd801, 0xdc14, 0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801,
0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc10,
0xd801, 0xdc31, 0xd801, 0xdc4d, 0x20, 0xd801, 0xdc17, 0xd801, 0xdc31, 0xd801, 0xdc4d, 0x20,
0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc10, 0xd801, 0xdc31, 0xd801, 0xdc4d,
0x20, 0xd801, 0xdc17, 0xd801, 0xdc31, 0xd801, 0xdc4d, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b,
0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c,
0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc10, 0xd801, 0xdc31, 0xd801,
0xdc4d, 0x20, 0xd801, 0xdc17, 0xd801, 0xdc31, 0xd801, 0xdc4d, 0x20, 0xd801, 0xdc14, 0xd801,
0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34,
0xd801, 0xdc4b, 0xd801, 0xdc24, 0xd801, 0xdc2d, 0xd801, 0xdc41, 0xd801, 0xdc32, 0xd801, 0xdc4c,
0xd801, 0xdc3c, 0xd801, 0xdc4a, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0x20, 0xd801,
0xdc13, 0xd801, 0xdc34, 0xd801, 0xdc4b, 0xd801, 0xdc24, 0xd801, 0xdc2d, 0xd801, 0xdc41, 0xd801,
0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0xd801, 0xdc4a, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801,
0xdc3c, 0x20, 0xd801, 0xdc1d, 0xd801, 0xdc3b, 0xd801, 0xdc30, 0xd801, 0xdc4c, 0xd801, 0xdc3c,
0xd801, 0xdc32, 0xd801, 0xdc49, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc13, 0xd801, 0xdc34, 0xd801,
0xdc4b, 0xd801, 0xdc24, 0xd801, 0xdc2d, 0xd801, 0xdc41, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801,
0xdc3c, 0xd801, 0xdc4a, 0xd801, 0xdc32, 0xd801, 0xdc4c, 0xd801, 0xdc3c, 0x20, 0xd801, 0xdc14,
0xd801, 0xdc29, 0xd801, 0xdc4a, 0xd801, 0xdc34, 0xd801, 0xdc3b, 0x20, 0xd801, 0xdc13, 0xd801,
0xdc34, 0xd801, 0xdc4b, 0x53, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x53, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x53, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x20, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6e, 0x61,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69,
0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x6f, 0x75, 0x6e, 0x74,
0x61, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x61, 0x20, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e,
0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61,
0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69,
0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53,
0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20,
0x53, 0x6f, 0x75, 0x74, 0x68, 0x65, 0x72, 0x6e, 0x20, 0x61, 0x6e, 0x64,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69,
0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69,
0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61,
0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e,
0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61,
0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e,
0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65,
0x61, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53,
0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65,
0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69, 0x6e, 0x67, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x53, 0x61, 0x76, 0x69,
0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x20, 0x61, 0x6e, 0x64, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x410, 0x444, 0x433, 0x430, 0x43d, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x410, 0x43b,
0x44f, 0x441, 0x43a, 0x430, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x410, 0x43b,
0x44f, 0x441, 0x43a, 0x430, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430,
0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43e, 0x43d, 0x44c, 0x20,
0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437,
0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x411, 0x430, 0x43d, 0x433, 0x43b,
0x430, 0x434, 0x435, 0x448, 0x435, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x435, 0x43d, 0x44c, 0x20,
0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x435, 0x43d, 0x44c, 0x20,
0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x411, 0x430,
0x437, 0x438, 0x43b, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x411,
0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430,
0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x411, 0x430, 0x437, 0x438,
0x43b, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x427, 0x438, 0x43b, 0x438, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x427, 0x438, 0x43b, 0x438, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430,
0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x427, 0x438, 0x43b, 0x438,
0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x41a, 0x438, 0x442, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430,
0x41a, 0x438, 0x442, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b,
0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x438, 0x442, 0x430, 0x435,
0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x41a, 0x443, 0x431, 0x430, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a,
0x443, 0x431, 0x430, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d,
0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x443, 0x431, 0x430, 0x43d, 0x44c, 0x20,
0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x418, 0x43d,
0x434, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d,
0x44c, 0x20, 0x448, 0x43a, 0x430, 0x418, 0x440, 0x430, 0x43d, 0x43e, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x418, 0x440, 0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20,
0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x418,
0x440, 0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d,
0x44c, 0x20, 0x448, 0x43a, 0x430, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x43d,
0x44c, 0x20, 0x448, 0x43a, 0x430, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x43d,
0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x43a, 0x438,
0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x430, 0x437, 0x430,
0x445, 0x441, 0x442, 0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x447, 0x438, 0x43b,
0x438, 0x441, 0x435, 0x43c, 0x430, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a,
0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20,
0x447, 0x438, 0x432, 0x430, 0x43b, 0x433, 0x43e, 0x43c, 0x430, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x41a, 0x43e, 0x440, 0x435, 0x430, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x41a, 0x43e, 0x440, 0x435, 0x430, 0x43d, 0x44c, 0x20, 0x441, 0x432,
0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41a, 0x43e, 0x440,
0x435, 0x430, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x43e, 0x43d, 0x44c,
0x20, 0x448, 0x43a, 0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x43e, 0x43d,
0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x41c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x43e, 0x43d, 0x44c, 0x20, 0x43a,
0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41e, 0x434, 0x20,
0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x448,
0x43a, 0x430, 0x41e, 0x434, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x41e, 0x434, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x41e, 0x43c, 0x441, 0x43a, 0x43e, 0x435, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x41e, 0x43c, 0x441, 0x43a, 0x43e, 0x435, 0x43d, 0x44c, 0x20,
0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41e,
0x43c, 0x441, 0x43a, 0x43e, 0x435, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432,
0x430, 0x435, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41f, 0x430, 0x440, 0x430,
0x433, 0x432, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432,
0x430, 0x435, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x44d, 0x43d,
0x44c, 0x20, 0x448, 0x43a, 0x430, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d,
0x44d, 0x43d, 0x44c, 0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20,
0x448, 0x43a, 0x430, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x44d, 0x43d,
0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x448, 0x43a,
0x430, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x441,
0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x423, 0x440,
0x443, 0x433, 0x432, 0x430, 0x435, 0x43d, 0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d,
0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430, 0x423,
0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x43e, 0x43d, 0x44c,
0x20, 0x441, 0x432, 0x430, 0x43b, 0x43e, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430,
0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x43e, 0x43d,
0x44c, 0x20, 0x43a, 0x438, 0x437, 0x44d, 0x43d, 0x44c, 0x20, 0x448, 0x43a, 0x430,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x73,
0x75, 0x64, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x63, 0x69,
0x66, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69,
0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x61, 0x72, 0x61, 0x62, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x61, 0x72, 0x61, 0x62, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x61, 0x62, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f,
0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x2d, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f,
0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x2d, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x2d, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x61, 0x72, 0x6d, 0x65, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x61, 0x72, 0x6d, 0x65, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x20, 0x61, 0x16d, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x20, 0x61, 0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x20, 0x61, 0x16d, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f,
0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x61, 0x16d, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x20, 0x61, 0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x20, 0x61, 0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x61, 0x16d, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x61, 0x16d, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x20, 0x61, 0x16d, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x61,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x11d, 0x61, 0x6e, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x11d,
0x61, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x11d, 0x61,
0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x68, 0x6f, 0x72, 0x7a, 0x6f, 0x6e, 0x6f, 0x20, 0x41,
0x63, 0x6f, 0x72, 0x6f, 0x6a, 0x41, 0x63, 0x6f, 0x72, 0x6f, 0x6a, 0x20,
0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x29, 0x41, 0x63, 0x6f, 0x72, 0x6f, 0x6a, 0x20, 0x28, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x62, 0x6f,
0x6c, 0x69, 0x76, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x62,
0x72, 0x61, 0x7a, 0x69, 0x6c, 0x6a, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x62, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x6a, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x62, 0x72, 0x61,
0x7a, 0x69, 0x6c, 0x6a, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x62, 0x72, 0x75, 0x6e, 0x65, 0x6a,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x61, 0x62, 0x6f, 0x76,
0x65, 0x72, 0x64, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x61,
0x62, 0x6f, 0x76, 0x65, 0x72, 0x64, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x61, 0x62, 0x6f, 0x76,
0x65, 0x72, 0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x109, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x69, 0x6c,
0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x69, 0x6c, 0x69,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x109, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x69, 0x6e, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x69, 0x6e, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x109, 0x69, 0x6e,
0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x68, 0x6f, 0x72, 0x7a, 0x6f, 0x6e, 0x6f, 0x20, 0x108, 0x6f,
0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x108, 0x6f, 0x69, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x108, 0x6f, 0x69, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x20, 0x28, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x6b, 0x72, 0x69, 0x73, 0x74, 0x6e,
0x61, 0x73, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x6b, 0x6f, 0x73, 0x69, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x68, 0x6f, 0x72, 0x7a, 0x6f, 0x6e, 0x6f, 0x20, 0x4b,
0x75, 0x62, 0x6f, 0x4b, 0x75, 0x62, 0x6f, 0x20, 0x28, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x4b, 0x75, 0x62,
0x6f, 0x20, 0x28, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x29, 0x68, 0x6f, 0x72, 0x7a, 0x6f, 0x6e, 0x6f, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x68, 0x6f, 0x72, 0x7a, 0x6f, 0x6e, 0x6f,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x74,
0x69, 0x6d, 0x6f, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70,
0x61, 0x73, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x70, 0x61, 0x73, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x70, 0x61, 0x73, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x65, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x16d, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65,
0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x65, 0x6b, 0x73, 0x74, 0x72, 0x65, 0x6d, 0x2d, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x65,
0x16d, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f,
0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x65, 0x16d, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x68, 0x6f, 0x72, 0x7a, 0x6f, 0x6e, 0x6f, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x63, 0x61, 0x20, 0x47, 0x76, 0x69, 0x61, 0x6e, 0x6f, 0x68,
0x6f, 0x72, 0x7a, 0x6f, 0x6e, 0x6f, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63,
0x61, 0x6a, 0x20, 0x53, 0x75, 0x64, 0x61, 0x6a, 0x20, 0x54, 0x65, 0x72,
0x69, 0x74, 0x6f, 0x72, 0x69, 0x6f, 0x6a, 0x67, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x61, 0x72,
0x74, 0x76, 0x65, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b,
0x61, 0x72, 0x74, 0x76, 0x65, 0x6c, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x61, 0x72, 0x74, 0x76,
0x65, 0x6c, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x75, 0x6e, 0x69, 0x76, 0x65, 0x72, 0x73, 0x61,
0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x20, 0x6b, 0x75, 0x6e,
0x6f, 0x72, 0x64, 0x69, 0x67, 0x69, 0x74, 0x61, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x20, 0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x20,
0x67, 0x72, 0x6f, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x67, 0x6f,
0x6c, 0x66, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x67, 0x75, 0x6a, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x68, 0x6f, 0x72, 0x7a, 0x6f, 0x6e, 0x6f, 0x20, 0x48,
0x61, 0x76, 0x61, 0x6a, 0x6f, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x6f,
0x6a, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x6f, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x6f, 0x6a, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x29, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x6f, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x6f, 0x6a, 0x20, 0x28, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x68, 0x6f,
0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x68, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x68, 0x6f, 0x6e, 0x6b,
0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x125, 0x6f, 0x76, 0x64, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x125, 0x6f, 0x76, 0x64, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x125, 0x6f, 0x76,
0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x62, 0x61, 0x72, 0x61, 0x74, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x68, 0x69, 0x6e, 0x64, 0x6f, 0x63, 0x65, 0x61, 0x6e,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x68, 0x69, 0x6e, 0x64, 0x6f,
0x109, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x20, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a,
0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x20, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x20, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a,
0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x69, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x69, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x69,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x61, 0x70, 0x61, 0x6e,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x61, 0x70, 0x61, 0x6e,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x20, 0x6b, 0x61, 0x7a, 0x61, 0x125, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6b, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x20, 0x6b, 0x61, 0x7a, 0x61, 0x125, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6b, 0x69, 0x72, 0x67, 0x69,
0x7a, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x68, 0x6f, 0x72, 0x7a,
0x6f, 0x6e, 0x6f, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x28,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x28, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29,
0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61, 0x6c, 0x61, 0x6a,
0x7a, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61, 0x16d,
0x72, 0x69, 0x63, 0x69, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d,
0x61, 0x16d, 0x72, 0x69, 0x63, 0x69, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x61, 0x16d, 0x72, 0x69,
0x63, 0x69, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x61,
0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69,
0x6b, 0x61, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x75,
0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6d, 0x6f, 0x73, 0x6b, 0x76, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x62, 0x69, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6e, 0x6f, 0x76, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x7a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x68, 0x6f, 0x72, 0x7a, 0x6f, 0x6e, 0x6f, 0x20, 0x4e, 0x6f, 0x76,
0x6c, 0x61, 0x6e, 0x64, 0x6f, 0x4e, 0x6f, 0x76, 0x6c, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x29, 0x4e, 0x6f, 0x76, 0x6c, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x28, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x29, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73,
0x75, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20,
0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x68, 0x6f, 0x72, 0x7a, 0x6f, 0x6e, 0x6f, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x28,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x28, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x6e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6f, 0x6d, 0x73,
0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76,
0x61, 0x6a, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x70, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61,
0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x70, 0x65, 0x72, 0x75, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x70, 0x65, 0x72, 0x75, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x70, 0x65, 0x72, 0x75, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x66, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x68, 0x6f, 0x72, 0x7a, 0x6f, 0x6e,
0x6f, 0x20, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x50, 0x69, 0x65,
0x72, 0x6f, 0x20, 0x6b, 0x61, 0x6a, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x6f, 0x6e, 0x6f, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61, 0x20, 0x50, 0x69,
0x65, 0x72, 0x6f, 0x20, 0x6b, 0x61, 0x6a, 0x20, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x6f, 0x20, 0x28, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x61,
0x20, 0x50, 0x69, 0x65, 0x72, 0x6f, 0x20, 0x6b, 0x61, 0x6a, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x20, 0x28, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x29, 0x70, 0x6a,
0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x68, 0x6f, 0x72, 0x7a, 0x6f, 0x6e, 0x6f, 0x20, 0x52, 0x65,
0x75, 0x6e, 0x69, 0x6f, 0x73, 0x61, 0x125, 0x61, 0x6c, 0x65, 0x6e, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x73, 0x61, 0x125, 0x61, 0x6c, 0x65,
0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x73, 0x61, 0x125, 0x61, 0x6c, 0x65, 0x6e, 0x61, 0x20, 0x73,
0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x73,
0x65, 0x6a, 0x15d, 0x65, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x73, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x68, 0x6f, 0x72,
0x7a, 0x6f, 0x6e, 0x6f, 0x20, 0x53, 0x75, 0x64, 0x2d, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x6f, 0x73, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x61, 0x6a, 0x70, 0x65, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x61, 0x6a, 0x70, 0x65, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x74, 0x61, 0x6a, 0x70, 0x65, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x61, 0x11d, 0x69, 0x6b,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x75, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x75, 0x7a, 0x62,
0x65, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x75, 0x7a, 0x62,
0x65, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65,
0x6d, 0x70, 0x6f, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x61, 0x20, 0x73, 0x6f,
0x6d, 0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70,
0x6f, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65,
0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x76, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x73, 0x6f, 0x6d,
0x65, 0x72, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20, 0x74,
0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f,
0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x20, 0x74, 0x65, 0x6d,
0x70, 0x6f, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x61, 0x20, 0x73, 0x6f, 0x6d, 0x65, 0x72, 0x61, 0x20,
0x74, 0x65, 0x6d, 0x70, 0x6f, 0x6a, 0x75, 0x6b, 0x6f, 0x6e, 0x69, 0x61,
0x20, 0x74, 0x65, 0x6d, 0x70, 0x6f, 0x41, 0x63, 0x72, 0x65, 0x20, 0x61,
0x65, 0x67, 0x41, 0x63, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x63, 0x72, 0x65, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73,
0x6b, 0x2d, 0x41, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61, 0x65,
0x67, 0x49, 0x64, 0x61, 0x2d, 0x41, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x61, 0x65, 0x67, 0x4c, 0xf5, 0x75, 0x6e, 0x61, 0x2d, 0x41, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4,
0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4c,
0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x61, 0x65, 0x67, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0xf5, 0x20, 0x61, 0x65, 0x67,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0xf5, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0xf5,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x61, 0x73, 0x65, 0x20, 0x61, 0x65, 0x67, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x61, 0x73, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b,
0x65, 0x73, 0x6b, 0x2d, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x41, 0x6d, 0x65,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x41, 0x6d,
0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x49, 0x64, 0x61, 0x72, 0x61, 0x6e, 0x6e, 0x69, 0x6b, 0x75,
0x20, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x72, 0x61, 0x6e, 0x6e, 0x69,
0x6b, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x49, 0x64, 0x61, 0x72, 0x61, 0x6e, 0x6e, 0x69, 0x6b, 0x75,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4d, 0xe4, 0x65, 0x73,
0x74, 0x69, 0x6b, 0x75, 0x76, 0xf6, 0xf6, 0x6e, 0x64, 0x69, 0x20, 0x61,
0x65, 0x67, 0x4d, 0xe4, 0x65, 0x73, 0x74, 0x69, 0x6b, 0x75, 0x76, 0xf6,
0xf6, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x4d, 0xe4, 0x65, 0x73, 0x74, 0x69, 0x6b, 0x75,
0x76, 0xf6, 0xf6, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x56, 0x61, 0x69, 0x6b, 0x73, 0x65, 0x20, 0x6f, 0x6f, 0x6b,
0x65, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x56, 0x61, 0x69, 0x6b,
0x73, 0x65, 0x20, 0x6f, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x56, 0x61,
0x69, 0x6b, 0x73, 0x65, 0x20, 0x6f, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x6e, 0x61, 0x64,
0xf5, 0x72, 0x69, 0x20, 0x61, 0x65, 0x67, 0x41, 0x6e, 0x61, 0x64, 0xf5,
0x72, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x41, 0x6e, 0x61, 0x64, 0xf5, 0x72, 0x69, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x70, 0x69, 0x61, 0x20, 0x61, 0x65,
0x67, 0x41, 0x70, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x70, 0x69, 0x61, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x61,
0x65, 0x67, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x6b, 0x74, 0x61, 0x75,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x6b, 0x74, 0xf6,
0x62, 0x65, 0x20, 0x61, 0x65, 0x67, 0x41, 0x6b, 0x74, 0xf6, 0x62, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x41, 0x6b, 0x74, 0xf6, 0x62, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x41, 0x72, 0x61, 0x61, 0x62, 0x69, 0x61, 0x20, 0x61, 0x65,
0x67, 0x41, 0x72, 0x61, 0x61, 0x62, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x72, 0x61, 0x61,
0x62, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x65, 0x67,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4,
0x6e, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41,
0x72, 0x6d, 0x65, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x41,
0x72, 0x6d, 0x65, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x72, 0x6d, 0x65, 0x65,
0x6e, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x61, 0x65, 0x67, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x4c, 0xe4, 0xe4,
0x6e, 0x65, 0x20, 0x61, 0x65, 0x67, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x4c, 0xe4,
0xe4, 0x6e, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x4c, 0xe4, 0xe4, 0x6e, 0x65,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61,
0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4,
0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x61,
0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x41, 0x73, 0x65, 0x72, 0x62,
0x61, 0x69, 0x64, 0x17e, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x41, 0x73, 0x65, 0x72,
0x62, 0x61, 0x69, 0x64, 0x17e, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x41, 0x73, 0x73, 0x6f, 0x6f, 0x72, 0x69,
0x64, 0x65, 0x20, 0x61, 0x65, 0x67, 0x41, 0x73, 0x73, 0x6f, 0x6f, 0x72,
0x69, 0x64, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x41, 0x73, 0x73, 0x6f, 0x6f, 0x72, 0x69, 0x64, 0x65,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x20, 0x61, 0x65, 0x67, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x61, 0x65, 0x67, 0x42, 0x6f, 0x6c, 0x69, 0x69, 0x76, 0x69, 0x61,
0x20, 0x61, 0x65, 0x67, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69, 0x6c, 0x69,
0x61, 0x20, 0x61, 0x65, 0x67, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69, 0x6c,
0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69, 0x6c, 0x69, 0x61, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x69, 0x20, 0x61, 0x65, 0x67, 0x52, 0x6f, 0x68, 0x65, 0x6e, 0x65, 0x65,
0x6d, 0x65, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x61, 0x65, 0x67,
0x52, 0x6f, 0x68, 0x65, 0x6e, 0x65, 0x65, 0x6d, 0x65, 0x73, 0x61, 0x61,
0x72, 0x74, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x52, 0x6f, 0x68, 0x65, 0x6e, 0x65, 0x65, 0x6d, 0x65,
0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x43, 0x61, 0x73, 0x65, 0x79, 0x20, 0x61, 0x65, 0x67, 0x54,
0x161, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x69, 0x20, 0x61, 0x65, 0x67, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x69, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x54, 0x161, 0x69, 0x69, 0x6c,
0x69, 0x20, 0x61, 0x65, 0x67, 0x54, 0x161, 0x69, 0x69, 0x6c, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x54,
0x161, 0x69, 0x69, 0x6c, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x48, 0x69, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x65, 0x67, 0x48, 0x69,
0x69, 0x6e, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x48, 0x69, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x54, 0x161, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x54, 0x161, 0x6f, 0x6a, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x54, 0x161, 0x6f, 0x6a, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x4a, 0xf5, 0x75, 0x6c, 0x75, 0x73, 0x61, 0x61, 0x72, 0x65, 0x20,
0x61, 0x65, 0x67, 0x4b, 0x6f, 0x6f, 0x6b, 0x6f, 0x73, 0x73, 0x61, 0x61,
0x72, 0x74, 0x65, 0x20, 0x61, 0x65, 0x67, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x43, 0x6f, 0x6f, 0x6b, 0x69,
0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x61, 0x65, 0x67, 0x43,
0x6f, 0x6f, 0x6b, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x43,
0x6f, 0x6f, 0x6b, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20,
0x6f, 0x73, 0x61, 0x6c, 0x69, 0x6e, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x20, 0x61, 0x65, 0x67,
0x4b, 0x75, 0x75, 0x62, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x44, 0x61, 0x76, 0x69, 0x73, 0x65,
0x20, 0x61, 0x65, 0x67, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2019, 0x69, 0x20, 0x61,
0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x69,
0x20, 0x61, 0x65, 0x67, 0x4c, 0x69, 0x68, 0x61, 0x76, 0xf5, 0x74, 0x74,
0x65, 0x73, 0x61, 0x61, 0x72, 0x65, 0x20, 0x61, 0x65, 0x67, 0x4c, 0x69,
0x68, 0x61, 0x76, 0xf5, 0x74, 0x74, 0x65, 0x73, 0x61, 0x61, 0x72, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x4c, 0x69, 0x68, 0x61, 0x76, 0xf5, 0x74, 0x74, 0x65, 0x73, 0x61, 0x61,
0x72, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x45, 0x63,
0x75, 0x61, 0x64, 0x6f, 0x72, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x65,
0x73, 0x6b, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20, 0x61,
0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f,
0x70, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x4b, 0x65, 0x73, 0x6b, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f,
0x70, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x49, 0x64,
0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20, 0x61, 0x65,
0x67, 0x49, 0x64, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x49, 0x64, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b, 0x61, 0x6c, 0x69, 0x6e,
0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x69, 0x20, 0x6a, 0x61, 0x20, 0x56,
0x61, 0x6c, 0x67, 0x65, 0x76, 0x65, 0x6e, 0x65, 0x20, 0x61, 0x65, 0x67,
0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70,
0x61, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x61, 0x65, 0x67, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x73, 0x61, 0x61,
0x72, 0x74, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20, 0x61, 0x65, 0x67,
0x46, 0x69, 0x64, 0x17e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x50, 0x72, 0x61, 0x6e, 0x74, 0x73,
0x75, 0x73, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6a, 0x61, 0x61, 0x6e, 0x61,
0x20, 0x61, 0x65, 0x67, 0x50, 0x72, 0x61, 0x6e, 0x74, 0x73, 0x75, 0x73,
0x65, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6c, 0x69,
0x73, 0x74, 0x65, 0x20, 0x6a, 0x61, 0x20, 0x4c, 0xf5, 0x75, 0x6e, 0x61,
0x61, 0x6c, 0x61, 0x64, 0x65, 0x20, 0x61, 0x65, 0x67, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x65, 0x20, 0x61, 0x65, 0x67, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x2019, 0x20, 0x61, 0x65, 0x67, 0x47,
0x72, 0x75, 0x75, 0x73, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x47, 0x72,
0x75, 0x75, 0x73, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x47, 0x72, 0x75, 0x75, 0x73, 0x69, 0x61,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20,
0x61, 0x65, 0x67, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x69, 0x20, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x47, 0x72, 0xf6,
0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x61, 0x65, 0x67, 0x49, 0x64,
0x61, 0x2d, 0x47, 0x72, 0xf6, 0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x49,
0x64, 0x61, 0x2d, 0x47, 0x72, 0xf6, 0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e,
0x65, 0x2d, 0x47, 0x72, 0xf6, 0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20,
0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x47, 0x72, 0xf6,
0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4c, 0xe4, 0xe4, 0x6e, 0x65, 0x2d,
0x47, 0x72, 0xf6, 0xf6, 0x6e, 0x69, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x75,
0x76, 0x65, 0x61, 0x65, 0x67, 0x47, 0x75, 0x61, 0x6d, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x50, 0xe4,
0x72, 0x73, 0x69, 0x61, 0x20, 0x6c, 0x61, 0x68, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x47, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x20, 0x61, 0x65, 0x67, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x75, 0x64, 0x69, 0x20, 0x61, 0x65,
0x67, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x75, 0x64, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x75, 0x64, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x61,
0x65, 0x67, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x48,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x20, 0x61, 0x65,
0x67, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x61, 0x65, 0x67, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x6f,
0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x49, 0x6e, 0x64,
0x6f, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x65,
0x73, 0x6b, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x69,
0x61, 0x20, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x65, 0x73, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c,
0xe4, 0xe4, 0x6e, 0x65, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65,
0x73, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x49, 0x72, 0x61, 0x61, 0x6e,
0x69, 0x20, 0x61, 0x65, 0x67, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x49,
0x72, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x65,
0x67, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x49, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x20, 0x61, 0x65,
0x67, 0x49, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x49, 0x69, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x4a, 0x61, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67,
0x4a, 0x61, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4a, 0x61, 0x61, 0x70, 0x61,
0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x74, 0x161, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x61,
0x65, 0x67, 0x4b, 0x61, 0x6d, 0x74, 0x161, 0x61, 0x74, 0x6b, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b,
0x61, 0x6d, 0x74, 0x161, 0x61, 0x74, 0x6b, 0x61, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x49, 0x64, 0x61, 0x2d, 0x4b, 0x61, 0x73, 0x61,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xe4,
0xe4, 0x6e, 0x65, 0x2d, 0x4b, 0x61, 0x73, 0x61, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20,
0x61, 0x65, 0x67, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b, 0x6f, 0x73,
0x72, 0x61, 0x65, 0x20, 0x61, 0x65, 0x67, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4b, 0xf5, 0x72, 0x67, 0xf5,
0x7a, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x53, 0x72,
0x69, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4c,
0x69, 0x6e, 0x65, 0x2019, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65,
0x20, 0x61, 0x65, 0x67, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x2019, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x2019, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x2019, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x61,
0x63, 0x61, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69,
0x20, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x6c, 0x61, 0x69, 0x73, 0x69, 0x61,
0x20, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x69, 0x76, 0x69,
0x20, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x69, 0x73, 0x61,
0x61, 0x72, 0x74, 0x65, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x69, 0x20, 0x53, 0x61, 0x61, 0x72, 0x74, 0x65,
0x20, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x65, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4d, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4d, 0x65, 0x68,
0x68, 0x69, 0x6b, 0x6f, 0x20, 0x56, 0x61, 0x69, 0x6b, 0x73, 0x65, 0x20,
0x6f, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4d,
0x65, 0x68, 0x68, 0x69, 0x6b, 0x6f, 0x20, 0x56, 0x61, 0x69, 0x6b, 0x73,
0x65, 0x20, 0x6f, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4d, 0x65, 0x68,
0x68, 0x69, 0x6b, 0x6f, 0x20, 0x56, 0x61, 0x69, 0x6b, 0x73, 0x65, 0x20,
0x6f, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x69, 0x20, 0x61, 0x65, 0x67, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x61, 0x65,
0x67, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4d, 0x6f, 0x73, 0x6b, 0x76,
0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x69, 0x72,
0x6d, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20,
0x61, 0x65, 0x67, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x69, 0x20, 0x61, 0x65,
0x67, 0x55, 0x75, 0x73, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6f,
0x6e, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73, 0x2d, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73,
0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73, 0x2d, 0x4d,
0x65, 0x72, 0x65, 0x6d, 0x61, 0x61, 0x20, 0x61, 0x65, 0x67, 0x55, 0x75,
0x73, 0x2d, 0x4d, 0x65, 0x72, 0x65, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x55, 0x75, 0x73,
0x2d, 0x4d, 0x65, 0x72, 0x65, 0x6d, 0x61, 0x61, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4e, 0x69, 0x75, 0x65, 0x20,
0x61, 0x65, 0x67, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x20,
0x73, 0x61, 0x61, 0x72, 0x65, 0x20, 0x61, 0x65, 0x67, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x20, 0x73, 0x61, 0x61, 0x72,
0x65, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x20, 0x61, 0x65, 0x67, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x65, 0x67, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x50, 0xf5, 0x68, 0x6a, 0x61, 0x2d,
0x4d, 0x61, 0x72, 0x69, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x61, 0x65, 0x67, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4f, 0x6d, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x61, 0x65, 0x67, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x42, 0x65, 0x6c, 0x61, 0x75, 0x20,
0x61, 0x65, 0x67, 0x50, 0x61, 0x61, 0x70, 0x75, 0x61, 0x20, 0x55, 0x75,
0x73, 0x2d, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x61, 0x65, 0x67,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x61, 0x65, 0x67,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x50, 0x65, 0x72, 0x75, 0x75, 0x20, 0x61, 0x65, 0x67, 0x50, 0x65, 0x72,
0x75, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x65, 0x67, 0x50, 0x65, 0x72, 0x75, 0x75, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x69,
0x20, 0x61, 0x65, 0x67, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x69, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x46, 0xf6, 0xf6, 0x6e, 0x69, 0x6b,
0x73, 0x69, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x20, 0x61, 0x65, 0x67,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x2019, 0x69, 0x20, 0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2019, 0x69, 0x20, 0x6a, 0x61,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2019, 0x69,
0x20, 0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x50, 0x6f,
0x68, 0x6e, 0x70, 0x65, 0x69, 0x20, 0x61, 0x65, 0x67, 0x50, 0x79, 0x6f,
0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4b,
0xf5, 0x7a, 0xf5, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x61, 0x65, 0x67,
0x4b, 0xf5, 0x7a, 0xf5, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x4b, 0xf5, 0x7a,
0xf5, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x69, 0x20, 0x61,
0x65, 0x67, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x61, 0x65,
0x67, 0x53, 0x61, 0x68, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20, 0x61,
0x65, 0x67, 0x53, 0x61, 0x68, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x53,
0x61, 0x68, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76,
0x65, 0x61, 0x65, 0x67, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x61,
0x65, 0x67, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x20, 0x61, 0x65, 0x67, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65,
0x67, 0x53, 0x65, 0x69, 0x161, 0x65, 0x6c, 0x6c, 0x69, 0x20, 0x61, 0x65,
0x67, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x53, 0x61,
0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x69, 0x20, 0x53, 0x61, 0x61, 0x72,
0x74, 0x65, 0x20, 0x61, 0x65, 0x67, 0x4c, 0xf5, 0x75, 0x6e, 0x61, 0x2d,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x61, 0x65, 0x67, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x20, 0x61, 0x65, 0x67, 0x53,
0x79, 0x6f, 0x77, 0x61, 0x20, 0x61, 0x65, 0x67, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x20, 0x61, 0x65, 0x67, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x20, 0x61, 0x65, 0x67, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67,
0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x61, 0x65, 0x67, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20,
0x61, 0x65, 0x67, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x61, 0x65, 0x67,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73,
0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x69,
0x20, 0x61, 0x65, 0x67, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x61, 0x65, 0x67, 0x54, 0xfc, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x54, 0xfc,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x54, 0x75, 0x76, 0x61, 0x6c,
0x75, 0x20, 0x61, 0x65, 0x67, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x20, 0x61, 0x65, 0x67, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61,
0x65, 0x67, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x61, 0x65, 0x67, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x65, 0x67, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x61, 0x65, 0x67, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x56, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x61, 0x65, 0x67, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x61, 0x65,
0x67, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65,
0x67, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x20, 0x61, 0x65, 0x67, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x65, 0x67, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x61,
0x65, 0x67, 0x57, 0x61, 0x6b, 0x65, 0x2019, 0x69, 0x20, 0x61, 0x65, 0x67,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x65, 0x20, 0x6a, 0x61, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x61, 0x65, 0x67, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x65, 0x67, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x75, 0x76, 0x65, 0x61, 0x65, 0x67, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x20,
0x61, 0x65, 0x67, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x65, 0x67, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x20, 0x73, 0x75, 0x76, 0x65,
0x61, 0x65, 0x67, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x69, 0x20, 0x61, 0x65,
0x67, 0x45, 0x6b, 0x65, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x6d, 0x65, 0x45, 0x6b, 0x65, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x45, 0x6b, 0x65,
0x72, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x66, 0x67, 0x68,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x41, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20,
0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d,
0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20,
0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69,
0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x64, 0x7a, 0x6f,
0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x69, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x54, 0x69, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x54, 0x69, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x6f, 0x75,
0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e,
0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x6e, 0x61, 0x64,
0x69, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61,
0x6e, 0x79, 0x69, 0x6d, 0x65, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x20,
0x14b, 0x6b, 0x65, 0x6b, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x61, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x41, 0x70, 0x69, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x6b,
0x74, 0x61, 0x75, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65,
0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x41, 0x6b, 0x74, 0x61,
0x75, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41, 0x6b, 0x74, 0x6f,
0x62, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x41,
0x6b, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x41, 0x6b, 0x74, 0x74,
0x6f, 0x62, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65,
0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x194, 0x65, 0x74, 0x6f, 0x256,
0x6f, 0x192, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x194,
0x65, 0x74, 0x6f, 0x256, 0x6f, 0x192, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x194, 0x65,
0x74, 0x6f, 0x256, 0x6f, 0x192, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254,
0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x63, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d,
0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d,
0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x6b, 0x65,
0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x64, 0x7a, 0x6f, 0x6d,
0x65, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x263, 0x65, 0x74, 0x6f, 0x256, 0x6f,
0x66, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x6b, 0x65,
0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e,
0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x6b, 0x65, 0x6c,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57,
0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e,
0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x6b, 0x65, 0x6c, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20, 0x64, 0x7a, 0x6f, 0x6d,
0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b,
0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c,
0x61, 0x6d, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x61, 0x70, 0x65,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43,
0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20,
0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x64, 0x7a, 0x6f,
0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6e, 0x61,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x20, 0x6b,
0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x6f,
0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74,
0x6d, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x6f, 0x6f,
0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x6d, 0x65, 0x43, 0x75, 0x62, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x44, 0x61, 0x76, 0x69,
0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73,
0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45,
0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x194, 0x65, 0x64,
0x7a, 0x65, 0x192, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x194, 0x65, 0x64, 0x7a,
0x65, 0x192, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79, 0x69, 0x6d,
0x65, 0x194, 0x65, 0x64, 0x7a, 0x65, 0x192, 0x65, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x20, 0x14b, 0x6b, 0x65, 0x6b, 0x65, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b,
0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x6e, 0x75, 0x74, 0x6f,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254,
0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x46, 0x69, 0x6a, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46,
0x69, 0x6a, 0x69, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46,
0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46, 0x72,
0x65, 0x6e, 0x63, 0x68, 0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x65, 0x72,
0x6e, 0x20, 0x26, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69,
0x63, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x6d, 0x65, 0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73,
0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x47, 0x75, 0x6c, 0x66, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x6f,
0x76, 0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x48, 0x6f, 0x76, 0x64, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x48, 0x6f, 0x76,
0x64, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57,
0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x64, 0x7a, 0x6f,
0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74,
0x73, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f,
0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74,
0x73, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x14b, 0x6b, 0x65, 0x6b, 0x65,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x45,
0x61, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x57, 0x65, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d,
0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x64, 0x7a, 0x6f, 0x6d,
0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4c, 0x69,
0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x4d, 0x61, 0x6b, 0x61, 0x75,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61, 0x6e, 0x79,
0x69, 0x6d, 0x65, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x14b, 0x6b, 0x65,
0x6b, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d,
0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x6c,
0x61, 0x79, 0x73, 0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x61, 0x77, 0x73,
0x6f, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x63, 0x6f,
0x77, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d,
0x6f, 0x73, 0x63, 0x6f, 0x77, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x77, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254,
0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x65, 0x70, 0x61,
0x6c, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e,
0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x65,
0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b,
0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x65,
0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d,
0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x4e, 0x69, 0x75, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65,
0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d,
0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x7a, 0x6f,
0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x70, 0x75, 0x61,
0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x65,
0x72, 0x75, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x20,
0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70,
0x70, 0x69, 0x6e, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x65, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c,
0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x50,
0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x53, 0x74, 0x2e, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69,
0x72, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e,
0x67, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x4b,
0x69, 0x7a, 0x69, 0x6c, 0x254, 0x72, 0x64, 0x61, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x254, 0x72,
0x64, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f, 0x61,
0x6e, 0x79, 0x69, 0x6d, 0x65, 0x4b, 0x69, 0x7a, 0x69, 0x6c, 0x254, 0x72,
0x64, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x52, 0x65, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x256, 0x6f,
0x61, 0x6e, 0x79, 0x69, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x20, 0x14b, 0x6b, 0x65, 0x6b, 0x65, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x6b, 0x65, 0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x20, 0x6e, 0x75, 0x74, 0x6f,
0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x6d, 0x65, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x6b, 0x65,
0x6c, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x64, 0x7a, 0x6f, 0x6d,
0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x64, 0x7a,
0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x6e,
0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f,
0x20, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x64,
0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75,
0x65, 0x6c, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x6e, 0x75,
0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20,
0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192,
0x6f, 0x20, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192,
0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x6f, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x56, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20,
0x6d, 0x65, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x26, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x6e, 0x75, 0x74, 0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f,
0x192, 0x6f, 0x20, 0x6d, 0x65, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69, 0x20, 0x67,
0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x67, 0x61,
0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x6e, 0x75, 0x74,
0x6f, 0x6d, 0x65, 0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d,
0x65, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x20, 0x64, 0x7a, 0x6f, 0x6d, 0x65, 0x14b, 0x254, 0x6c, 0x69,
0x20, 0x67, 0x61, 0x192, 0x6f, 0x192, 0x6f, 0x20, 0x6d, 0x65, 0x41, 0x66,
0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0,
0x4d, 0x69, 0xf0, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0xed,
0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x75, 0xf0, 0x75, 0x72, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75,
0x72, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x74,
0xed, 0xf0, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x41,
0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x6d,
0x61, 0x73, 0x6f, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x61, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69,
0x6e, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69,
0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x41, 0x70, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0,
0x41, 0x70, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x41, 0x70, 0x69, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b,
0x20, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x72,
0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75,
0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x63, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x63, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x6d, 0x69, 0xf0, 0x20, 0x41,
0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0,
0x6d, 0x69, 0xf0, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x6d, 0x69, 0xf0, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x6d,
0x69, 0xf0, 0x76, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x41, 0x76, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x6d, 0x69,
0xf0, 0x76, 0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x41, 0x76, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x6d, 0x69, 0xf0, 0x76, 0x65, 0x73, 0x74, 0x75,
0x72, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x65, 0x79, 0x73,
0x74, 0x75, 0x72, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x74, 0xed, 0xf0, 0x65, 0x79, 0x73, 0x74, 0x75, 0x72, 0x20,
0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x65, 0x79, 0x73, 0x74,
0x75, 0x72, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x73,
0x65, 0x72, 0x62, 0x61, 0x64, 0x6a, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0,
0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x64, 0x6a, 0x61, 0x6e, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x73, 0x65,
0x72, 0x62, 0x61, 0x64, 0x6a, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x72, 0x6e,
0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x72,
0x6e, 0x61, 0x72, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x41, 0x7a, 0x6f, 0x72, 0x75, 0x72, 0x6e, 0x61, 0x72, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x6a, 0x20, 0x74, 0xed, 0xf0, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x6a, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x6a, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed,
0xf0, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed,
0xf0, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75,
0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x72,
0xf8, 0x6e, 0x68, 0xf8, 0x76, 0x64, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a,
0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x72, 0xf8, 0x6e, 0x68, 0xf8,
0x76, 0x64, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x72, 0xf8,
0x6e, 0x68, 0xf8, 0x76, 0x64, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61,
0x72, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x4b, 0x69, 0x6c, 0x69, 0x20, 0x74, 0xed, 0xf0,
0x4b, 0x69, 0x6c, 0x69, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x4b, 0x69, 0x6c, 0x69, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x4b, 0x69, 0x6e, 0x61, 0x20, 0x74, 0xed,
0xf0, 0x4b, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x4b, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x68, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed,
0xf0, 0x4a, 0xf3, 0x6c, 0x61, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x20, 0x74,
0xed, 0xf0, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a,
0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x43, 0x6f, 0x6f, 0x6b, 0x73,
0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x43,
0x6f, 0x6f, 0x6b, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x6f,
0x6f, 0x6b, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x43, 0x75, 0x62, 0x61,
0x20, 0x74, 0xed, 0xf0, 0x43, 0x75, 0x62, 0x61, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x43, 0x75, 0x62, 0x61, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x44, 0x61, 0x76,
0x69, 0x73, 0x20, 0x74, 0xed, 0xf0, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74,
0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x20, 0x74,
0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x74, 0x69, 0x6d, 0x6f,
0x72, 0x20, 0x74, 0xed, 0xf0, 0x50, 0xe1, 0x73, 0x6b, 0x61, 0x6f, 0x79,
0x67, 0x67, 0x69, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x50, 0xe1, 0x73, 0x6b,
0x61, 0x6f, 0x79, 0x67, 0x67, 0x69, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x50, 0xe1, 0x73, 0x6b, 0x61, 0x6f,
0x79, 0x67, 0x67, 0x69, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x74,
0xed, 0xf0, 0x4d, 0x69, 0xf0, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x74, 0xed, 0xf0, 0x4d, 0x69, 0xf0, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4d,
0x69, 0xf0, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x79,
0x73, 0x74, 0x75, 0x72, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x79, 0x73,
0x74, 0x75, 0x72, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x6c, 0x6f, 0x6e, 0x67, 0x72,
0x69, 0x20, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65,
0x73, 0x74, 0x75, 0x72, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73,
0x74, 0x75, 0x72, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20,
0x74, 0xed, 0xf0, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x46, 0x69, 0x6a, 0x69,
0x20, 0x74, 0xed, 0xf0, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x69, 0x6a, 0x69, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x46, 0x72, 0x61,
0x6e, 0x73, 0x6b, 0x61, 0x20, 0x47, 0x75, 0x6a, 0x61, 0x6e, 0x61, 0x20,
0x74, 0xed, 0xf0, 0x46, 0x72, 0x6f, 0x6e, 0x73, 0x6b, 0x75, 0x20, 0x73,
0x75, 0x6e, 0x6e, 0x61, 0x72, 0x75, 0x20, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0xf8, 0x6b, 0x69, 0x20, 0x6f, 0x67, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72,
0x6b, 0x74, 0x69, 0x73, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x61, 0x6c, 0x61,
0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x4d, 0x65,
0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72,
0x20, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b, 0x20,
0x74, 0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x20, 0x67, 0x72,
0xf8, 0x6e, 0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x79, 0x73, 0x74, 0x75,
0x72, 0x20, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x56, 0x65,
0x73, 0x74, 0x75, 0x72, 0x20, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x65, 0x6e,
0x64, 0x73, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75,
0x72, 0x20, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x65, 0x6e, 0x64, 0x73, 0x6b,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x56,
0x65, 0x73, 0x74, 0x75, 0x72, 0x20, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x65,
0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x47, 0x75, 0x6c, 0x66, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x48, 0x6f, 0x76, 0x64, 0x20,
0x74, 0xed, 0xf0, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x74, 0xed, 0xf0, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x61,
0x76, 0x20, 0x74, 0xed, 0xf0, 0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e,
0x61, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x69, 0xf0, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x45, 0x79,
0x73, 0x74, 0x75, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x74,
0xed, 0xf0, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x49, 0x72,
0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x74,
0xed, 0xf0, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed,
0xf0, 0xcd, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x74, 0xed, 0xf0, 0xcd,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0xcd, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x20, 0x74, 0xed, 0xf0, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x45, 0x79, 0x73, 0x74, 0x75, 0x72, 0x20, 0x4b, 0x61, 0x73, 0x61, 0x6b,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x73, 0x74,
0x75, 0x72, 0x20, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x74, 0xed,
0xf0, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e,
0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x69, 0x61,
0x20, 0x74, 0xed, 0xf0, 0x4c, 0x69, 0x6e, 0x65, 0x6f, 0x79, 0x67, 0x67,
0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x20, 0x74, 0xed, 0xf0, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67,
0x20, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x6c,
0x61, 0x69, 0x73, 0x69, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74,
0xed, 0xf0, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x20,
0x74, 0xed, 0xf0, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f,
0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0xf3,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0xf3,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0xf3, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4d,
0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x74, 0xed, 0xf0, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61,
0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x55, 0x6c, 0x61,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x74, 0xed, 0xf0,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4d, 0x79, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x20, 0x28, 0x42, 0x75, 0x72, 0x6d, 0x61, 0x29,
0x20, 0x74, 0xed, 0xf0, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x74, 0xed,
0xf0, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0xfd,
0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x20, 0x74, 0xed,
0xf0, 0x4e, 0xfd, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69, 0x61,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4e,
0xfd, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4e, 0xfd, 0x73, 0xe6,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0xfd, 0x73, 0xe6,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x4e, 0xfd, 0x73, 0xe6, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x74, 0xed,
0xf0, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x4e, 0x69,
0x75, 0x65, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x20, 0x76,
0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x4f,
0x6d, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x74,
0xed, 0xf0, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0xfd, 0x67, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x69, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x69, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x50, 0x65, 0x72,
0x75, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x65, 0x72, 0x75, 0x20, 0x76, 0x61,
0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x65, 0x72, 0x75,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x73, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20,
0x74, 0xed, 0xf0, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x73, 0x6f, 0x79, 0x67,
0x67, 0x6a, 0x61, 0x72, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x73, 0x6f, 0x79, 0x67,
0x67, 0x6a, 0x61, 0x72, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0xed, 0xf0, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x6f, 0x79, 0x67,
0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x74, 0x2e, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x74, 0x2e,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x79, 0x67, 0x67,
0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x6f, 0x6e, 0x61, 0x70,
0x65, 0x20, 0x74, 0xed, 0xf0, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61,
0x6e, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x20, 0x74, 0xed, 0xf0, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61,
0x20, 0x74, 0xed, 0xf0, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x20, 0x74, 0xed, 0xf0, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x74,
0xed, 0xf0, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x53, 0x65, 0x79,
0x73, 0x6b, 0x65, 0x6c, 0x6c, 0x6f, 0x79, 0x67, 0x67, 0x6a, 0x61, 0x72,
0x20, 0x74, 0xed, 0xf0, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72,
0x20, 0x74, 0xed, 0xf0, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x6f,
0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x75,
0xf0, 0x75, 0x72, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6f, 0x79,
0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x75, 0x72,
0x69, 0x6e, 0x61, 0x6d, 0x20, 0x74, 0xed, 0xf0, 0x53, 0x79, 0x6f, 0x77,
0x61, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20,
0x74, 0xed, 0xf0, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x74, 0xed,
0xf0, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x54, 0x61,
0x64, 0x73, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74,
0xed, 0xf0, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x74, 0xed,
0xf0, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74,
0xed, 0xf0, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73, 0x75, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0xed, 0xf0, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x20, 0x74,
0xed, 0xf0, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69,
0x67, 0x20, 0x74, 0xed, 0xf0, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x74, 0xed,
0xf0, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x20, 0x74, 0xed, 0xf0,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x69, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x55,
0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xed,
0xf0, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x55, 0x73,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x73, 0x75, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0xed, 0xf0, 0x56, 0x65, 0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c,
0x61, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c,
0x69, 0x67, 0x20, 0x74, 0xed, 0xf0, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0xed, 0xf0, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x20, 0x74, 0xed, 0xf0, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed, 0xf0,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x73, 0x75,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x56, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x20, 0x74, 0xed, 0xf0, 0x57, 0x61, 0x6b, 0x65, 0x6f, 0x79, 0x67,
0x67, 0x6a, 0x20, 0x74, 0xed, 0xf0, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73,
0x2d, 0x20, 0x6f, 0x67, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x6f,
0x79, 0x67, 0x67, 0x6a, 0x61, 0x72, 0x20, 0x74, 0xed, 0xf0, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x74, 0xed, 0xf0, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20,
0x74, 0xed, 0xf0, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x73,
0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x20, 0x76, 0x61, 0x6e, 0x6c, 0x69, 0x67, 0x20, 0x74, 0xed,
0xf0, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x20, 0x73, 0x75, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0xed, 0xf0,
0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x74, 0xed, 0xf0, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x67, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b,
0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b,
0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x4b,
0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x6e, 0x67, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x53, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x6e, 0x61, 0x20,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x74,
0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61,
0x6e, 0x20, 0x6e, 0x67, 0x20, 0x48, 0x69, 0x6c, 0x61, 0x67, 0x61, 0x6e,
0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61,
0x6e, 0x20, 0x6e, 0x67, 0x20, 0x48, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61,
0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x44, 0x61,
0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67,
0x61, 0x6e, 0x20, 0x6e, 0x67, 0x20, 0x48, 0x69, 0x6c, 0x61, 0x67, 0x61,
0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x42, 0x75, 0x6e, 0x64, 0x6f, 0x6b,
0x20, 0x73, 0x61, 0x20, 0x48, 0x69, 0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x42, 0x75, 0x6e, 0x64, 0x6f, 0x6b, 0x20, 0x73,
0x61, 0x20, 0x48, 0x69, 0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x42, 0x75, 0x6e, 0x64, 0x6f, 0x6b, 0x20, 0x73, 0x61, 0x20,
0x48, 0x69, 0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f, 0x20, 0x73, 0x61, 0x20,
0x48, 0x49, 0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f, 0x20, 0x73, 0x61, 0x20,
0x48, 0x69, 0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x6f, 0x20, 0x73, 0x61, 0x20,
0x48, 0x69, 0x6c, 0x61, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x70,
0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72,
0x61, 0x62, 0x69, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e,
0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x6e, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x6f, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6b, 0x6f, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x6f, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73,
0x61, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x6e, 0x67, 0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4b,
0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x20, 0x6e, 0x67, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x6e, 0x67,
0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4b, 0x61, 0x6e,
0x6c, 0x75, 0x72, 0x61, 0x6e, 0x20, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67,
0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47,
0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75,
0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69,
0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61,
0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e,
0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75,
0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75,
0x72, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a,
0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x61,
0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73,
0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x43,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b,
0x61, 0x6c, 0x61, 0x68, 0x61, 0x74, 0x69, 0x20, 0x6e, 0x67, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x75, 0x62, 0x61, 0x44,
0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x43, 0x75, 0x62, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x47,
0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67,
0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x47, 0x69, 0x74,
0x6e, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e,
0x67, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e,
0x67, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e,
0x67, 0x61, 0x6e, 0x67, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x69, 0x6e, 0x61, 0x6b,
0x61, 0x2d, 0x73, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e,
0x67, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e,
0x67, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e,
0x67, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x6d, 0x75, 0x67, 0x61, 0x6e, 0x67,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x20, 0x61, 0x74, 0x20, 0x41,
0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x6b, 0x6f, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69,
0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e,
0x67, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75,
0x72, 0x61, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61,
0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61, 0x6e, 0x67, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20,
0x6e, 0x67, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x47, 0x69, 0x74, 0x6e, 0x61, 0x6e, 0x67, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75, 0x72, 0x61,
0x6e, 0x67, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68,
0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x6e, 0x67,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x6c, 0x75,
0x72, 0x61, 0x6e, 0x67, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69,
0x74, 0x20, 0x6e, 0x67, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x4b, 0x79, 0x72, 0x67, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70, 0x69, 0x6b,
0x6f, 0x20, 0x6e, 0x67, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x70,
0x69, 0x6b, 0x6f, 0x20, 0x6e, 0x67, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63,
0x6f, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x73,
0x69, 0x70, 0x69, 0x6b, 0x6f, 0x20, 0x6e, 0x67, 0x20, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x6f, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f,
0x77, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e,
0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61,
0x6e, 0x64, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a,
0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x44, 0x61, 0x79, 0x6c,
0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e,
0x67, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61,
0x73, 0x20, 0x73, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20,
0x73, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50,
0x65, 0x72, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x50,
0x65, 0x72, 0x75, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x50,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x50, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61,
0x73, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61,
0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73,
0x61, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x20, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x6f, 0x67, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61,
0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20,
0x54, 0x69, 0x6d, 0x65, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d,
0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74,
0x20, 0x6e, 0x67, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61,
0x20, 0x54, 0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72,
0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54,
0x61, 0x67, 0x2d, 0x69, 0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x72, 0x61, 0x73,
0x20, 0x73, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6e,
0x61, 0x20, 0x4f, 0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f,
0x72, 0x61, 0x73, 0x20, 0x73, 0x61, 0x20, 0x54, 0x61, 0x67, 0x2d, 0x69,
0x6e, 0x69, 0x74, 0x20, 0x6e, 0x67, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x63, 0x72, 0x65,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x63, 0x72, 0x65, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x41, 0x63, 0x72, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69,
0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x49, 0x74, 0xe4, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x45, 0x74, 0x65, 0x6c, 0xe4, 0x2d, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c,
0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69,
0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79,
0x73, 0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0x6b, 0x69, 0x6e, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x68,
0x64, 0x79, 0x73, 0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0x6b, 0x69, 0x6e, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79,
0x73, 0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0x6b, 0x69, 0x6e, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c, 0x74, 0x61,
0x69, 0x6e, 0x20, 0x69, 0x74, 0xe4, 0x69, 0x6e, 0x65, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c, 0x74,
0x61, 0x69, 0x6e, 0x20, 0x69, 0x74, 0xe4, 0x69, 0x6e, 0x65, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e,
0x20, 0x69, 0x74, 0xe4, 0x69, 0x6e, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6c, 0x6c, 0x69, 0x6f, 0x76,
0x75, 0x6f, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b,
0x61, 0x6c, 0x6c, 0x69, 0x6f, 0x76, 0x75, 0x6f, 0x72, 0x74, 0x65, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x4b, 0x61, 0x6c, 0x6c, 0x69, 0x6f, 0x76, 0x75, 0x6f, 0x72, 0x74,
0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x59,
0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20,
0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e, 0x6d, 0x65, 0x72, 0x65, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61, 0x6c,
0x74, 0x61, 0x69, 0x6e, 0x20, 0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e, 0x6d,
0x65, 0x72, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x59, 0x68, 0x64, 0x79, 0x73, 0x76, 0x61,
0x6c, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e,
0x6d, 0x65, 0x72, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x69, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x69, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x70, 0x69, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x41, 0x70, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x70, 0x69,
0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x71, 0x74, 0x61, 0x77, 0x2019, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x71, 0x74, 0x61, 0x77, 0x2019, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x71, 0x74, 0x61, 0x77,
0x2019, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x71, 0x74, 0xf6, 0x62, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x71, 0x74, 0xf6, 0x62, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x71, 0x74, 0xf6, 0x62,
0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x53,
0x61, 0x75, 0x64, 0x69, 0x2d, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x75, 0x64, 0x69, 0x2d, 0x41,
0x72, 0x61, 0x62, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x75, 0x64, 0x69,
0x2d, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x69, 0x6e, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e,
0x73, 0x69, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e,
0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b,
0x61, 0x6e, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x6e, 0x20,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x74,
0x69, 0x73, 0x65, 0x6e, 0x20, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x74, 0x69, 0x73, 0x65, 0x6e, 0x20, 0x4b,
0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x74, 0x69, 0x73, 0x65, 0x6e,
0x20, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x49, 0x74, 0xe4, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4c,
0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73,
0x69, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x61,
0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x64, 0x17e, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x61, 0x6e, 0x69, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x7a, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x7a, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x42,
0x68, 0x75, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x70, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61,
0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x70,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x43, 0x61, 0x73, 0x65, 0x79, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x54, 0x161, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x69, 0x69, 0x6e,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x69, 0x69, 0x6e, 0x61,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x4b, 0x69, 0x69, 0x6e, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x161, 0x6f, 0x69, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x161, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x161, 0x6f, 0x69, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x4a, 0x6f, 0x75, 0x6c, 0x75, 0x73, 0x61, 0x61, 0x72, 0x65,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x6f, 0x6f, 0x6b, 0x6f, 0x73,
0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e,
0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72,
0x74, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x4b, 0x75, 0x75, 0x62, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b,
0x75, 0x75, 0x62, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x44, 0x61, 0x76,
0x69, 0x73, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x54,
0x69, 0x6d, 0x6f, 0x72, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50,
0xe4, 0xe4, 0x73, 0x69, 0xe4, 0x69, 0x73, 0x73, 0x61, 0x61, 0x72, 0x65,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0xe4, 0xe4, 0x73, 0x69, 0xe4,
0x69, 0x73, 0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x50, 0xe4, 0xe4,
0x73, 0x69, 0xe4, 0x69, 0x73, 0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x45, 0x63, 0x75, 0x61,
0x64, 0x6f, 0x72, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x65,
0x73, 0x6b, 0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b,
0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x49, 0x74, 0xe4, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x49, 0x74, 0xe4, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74,
0xe4, 0x69, 0x73, 0x65, 0x6d, 0x6d, 0xe4, 0x6e, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4,
0x6e, 0x73, 0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73,
0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x73, 0x61, 0x61,
0x72, 0x74, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x46, 0x69, 0x64, 0x17e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x69, 0x64, 0x17e, 0x69,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x52, 0x61,
0x6e, 0x73, 0x6b, 0x61, 0x6e, 0x20, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x52, 0x61, 0x6e, 0x73, 0x6b,
0x61, 0x6e, 0x20, 0x65, 0x74, 0x65, 0x6c, 0xe4, 0x69, 0x73, 0x74, 0x65,
0x6e, 0x20, 0x6a, 0x61, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74,
0x69, 0x73, 0x74, 0x65, 0x6e, 0x20, 0x61, 0x6c, 0x75, 0x65, 0x69, 0x64,
0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x47, 0x61, 0x6c, 0xe1, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x73,
0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x73, 0x61, 0x61,
0x72, 0x74, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4,
0x2d, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x47, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x47,
0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d,
0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d, 0x47, 0x72, 0xf6,
0x6e, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73,
0x69, 0x2d, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x47, 0x75, 0x61,
0x6d, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x41, 0x72, 0x61, 0x62,
0x69, 0x65, 0x6d, 0x69, 0x69, 0x72, 0x69, 0x6b, 0x75, 0x6e, 0x74, 0x69,
0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x48, 0x61, 0x76, 0x61, 0x69, 0x6a, 0x69, 0x6e, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x74, 0x69, 0x65, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x48, 0x61, 0x76, 0x61, 0x69, 0x6a, 0x69, 0x6e, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x74, 0x69, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x48, 0x61, 0x76,
0x61, 0x69, 0x6a, 0x69, 0x6e, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x74,
0x69, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x48, 0x6f, 0x76,
0x64, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x48, 0x6f, 0x76, 0x64,
0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x48, 0x6f, 0x76, 0x64, 0x69, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x6e, 0x74, 0x69, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x6e, 0x74, 0x69, 0x61, 0x6e, 0x20,
0x76, 0x61, 0x6c, 0x74, 0x61, 0x6d, 0x65, 0x72, 0x65, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x69, 0x6e, 0x61,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x65, 0x73, 0x6b, 0x69, 0x2d,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x49, 0x74, 0xe4, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4,
0x6e, 0x73, 0x69, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x72, 0x61, 0x6e, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x4b, 0x61, 0x6d, 0x74, 0x161, 0x61, 0x74, 0x6b, 0x61, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6d, 0x74, 0x161, 0x61, 0x74, 0x6b,
0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x4b, 0x61, 0x6d, 0x74, 0x161, 0x61, 0x74, 0x6b, 0x61,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x49, 0x74,
0xe4, 0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0xe4, 0x6e, 0x73, 0x69, 0x2d,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x73, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x72, 0x69,
0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x4c, 0x69, 0x6e, 0x65, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x4d, 0x61, 0x63, 0x61, 0x6f, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x63, 0x61, 0x6f,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x6c, 0x65, 0x73, 0x69, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x69,
0x76, 0x69, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x69, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x6b, 0x73, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x6b, 0x73, 0x65, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x6b, 0x73, 0x65, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x6f, 0x6e, 0x20, 0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e, 0x6d,
0x65, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x6f, 0x6e, 0x20, 0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e,
0x6d, 0x65, 0x72, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x6f, 0x6e, 0x20, 0x54, 0x79, 0x79, 0x6e, 0x65, 0x6e, 0x6d, 0x65, 0x72,
0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64, 0x65, 0x6e, 0x2d,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x55, 0x75, 0x64, 0x65, 0x6e, 0x2d, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64, 0x65,
0x6e, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64,
0x65, 0x6e, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64, 0x65, 0x6e, 0x2d, 0x53,
0x65, 0x65, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x75, 0x64,
0x65, 0x6e, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x6e, 0x69, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x69, 0x75, 0x65, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69,
0x6e, 0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x61, 0x61,
0x72, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69,
0x61, 0x69, 0x6b, 0x61, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69,
0x6e, 0x73, 0x61, 0x61, 0x72, 0x65, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x6f, 0x68, 0x6a, 0x6f,
0x69, 0x73, 0x2d, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x61, 0x6e, 0x69, 0x65,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4f,
0x6d, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x6e,
0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65,
0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x55,
0x75, 0x64, 0x65, 0x6e, 0x2d, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50,
0x65, 0x72, 0x75, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x65, 0x72,
0x75, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x50, 0x65, 0x72, 0x75, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x68, 0x6f, 0x65,
0x6e, 0x69, 0x78, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20, 0x61,
0x69, 0x6b, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x6e, 0x20, 0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x6e,
0x20, 0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61,
0x69, 0x6b, 0x61, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x6e, 0x20, 0x6a, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61,
0x69, 0x6b, 0x61, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x50, 0x6a, 0x6f, 0x6e, 0x67,
0x6a, 0x61, 0x6e, 0x67, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x51,
0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6e, 0x20,
0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x52, 0xe9, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x52, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53,
0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x53,
0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6e,
0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73,
0xe4, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69,
0x6b, 0x61, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x69, 0x65,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x6f, 0x72, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53, 0x61, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x61, 0x61, 0x72, 0x74, 0x65, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x45, 0x74, 0x65, 0x6c, 0xe4, 0x2d, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b,
0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x69, 0x6e, 0x20,
0x61, 0x69, 0x6b, 0x61, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69,
0x6b, 0x61, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b,
0x61, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6e, 0x20, 0x61, 0x69,
0x6b, 0x61, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4,
0x61, 0x69, 0x6b, 0x61, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c,
0x69, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x6e, 0x20, 0x61, 0x69, 0x6b,
0x61, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x69, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x69, 0x6e, 0x20, 0x6b,
0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61, 0x56, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x6b, 0x65,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73,
0x69, 0x6e, 0x20, 0x6a, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61,
0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x61, 0x6c, 0x69, 0x61, 0x69, 0x6b, 0x61,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x69, 0x6e, 0x20, 0x6b, 0x65, 0x73, 0xe4, 0x61, 0x69, 0x6b, 0x61,
0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x69, 0x6e, 0x20, 0x61, 0x69, 0x6b, 0x61,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x63, 0x72, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x63,
0x72, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x63, 0x72, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66,
0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72,
0x69, 0x71, 0x75, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73,
0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65,
0x20, 0x6d, 0xe9, 0x72, 0x69, 0x64, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69,
0x71, 0x75, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65,
0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d,
0x61, 0x20, 0x41, 0x74, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d,
0x61, 0x20, 0x41, 0x74, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x41, 0x6c, 0x6d, 0x61, 0x20,
0x41, 0x74, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61,
0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63,
0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73,
0x74, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69,
0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63,
0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73,
0x20, 0x52, 0x6f, 0x63, 0x68, 0x65, 0x75, 0x73, 0x65, 0x73, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x68, 0x65, 0x75, 0x73, 0x65,
0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x68, 0x65, 0x75, 0x73,
0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x71, 0x75,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0xe9, 0x72, 0x69,
0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x71, 0x75, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d,
0xe9, 0x72, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41,
0x70, 0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x2019, 0x41, 0x70, 0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0x41, 0x6b, 0x74, 0x61, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41,
0x6b, 0x74, 0x61, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x41, 0x6b, 0x74, 0x61, 0x6f,
0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74,
0xf6, 0x62, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0xf6, 0x62,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62,
0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61,
0x62, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62,
0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x4f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65,
0x73, 0x74, 0x20, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0xe9, 0x6e,
0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x71, 0x75, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x71, 0x75,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x71, 0x75, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x65, 0x2d, 0x6f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x2d,
0x6f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x65, 0x2d, 0x6f, 0x75, 0x65, 0x73, 0x74, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65,
0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75,
0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0xef,
0x64, 0x6a, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0xef, 0x64, 0x6a, 0x61, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0xef, 0x64, 0x6a, 0x61, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f,
0x72, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f,
0x72, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65,
0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x75, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x42, 0x68, 0x6f,
0x75, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75,
0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x74, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x75, 0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x74, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75,
0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x74, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x73, 0x65, 0x79, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65,
0x73, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x73,
0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x75, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x68, 0x69,
0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x68,
0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65,
0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73,
0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0xee, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x73,
0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x43,
0x75, 0x62, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50,
0xe2, 0x71, 0x75, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xee, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe2, 0x71, 0x75, 0x65,
0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x50, 0xe2, 0x71, 0x75, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc9, 0x71, 0x75, 0x61, 0x74, 0x65,
0x75, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6c,
0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73,
0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee,
0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x6f, 0x75, 0x69, 0x6e, 0x65,
0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20,
0x4d, 0x61, 0x6c, 0x6f, 0x75, 0x69, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x73,
0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x6f, 0x75, 0x69,
0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73,
0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x69, 0x64, 0x6a, 0x69, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x46, 0x69,
0x64, 0x6a, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20,
0x46, 0x69, 0x64, 0x6a, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x65, 0x20,
0x66, 0x72, 0x61, 0x6e, 0xe7, 0x61, 0x69, 0x73, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x54, 0x65, 0x72, 0x72, 0x65,
0x73, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x20,
0x65, 0x74, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x71,
0x75, 0x65, 0x73, 0x20, 0x66, 0x72, 0x61, 0x6e, 0xe7, 0x61, 0x69, 0x73,
0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0xee, 0x6c, 0x65, 0x73, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0xee, 0x6c, 0x65, 0x73, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x47, 0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x47, 0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x20, 0x47, 0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x47,
0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6d, 0x6f, 0x79, 0x65, 0x6e, 0x6e, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64,
0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x75,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x75, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20,
0x64, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74,
0x20, 0x64, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20,
0x64, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0x61,
0x6d, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x47, 0x6f,
0x6c, 0x66, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20,
0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0x48, 0x61, 0x77, 0x61, 0xef, 0x20, 0x2d, 0x20, 0x41, 0x6c,
0xe9, 0x6f, 0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x2019, 0x48, 0x61, 0x77, 0x61, 0xef, 0x20, 0x2d, 0x20, 0x41, 0x6c,
0xe9, 0x6f, 0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019,
0x48, 0x61, 0x77, 0x61, 0xef, 0x20, 0x2d, 0x20, 0x41, 0x6c, 0xe9, 0x6f,
0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f,
0x63, 0xe9, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x65, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x63,
0x68, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x69, 0x6e, 0x64, 0x6f,
0x6e, 0xe9, 0x73, 0x69, 0x65, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x20, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x65, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x65, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x49,
0x72, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x49,
0x72, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x49,
0x72, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b,
0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x49,
0x73, 0x72, 0x61, 0xeb, 0x6c, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0xeb,
0x6c, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x4a, 0x61, 0x70, 0x6f,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x75, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64,
0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x43, 0x6f, 0x72, 0xe9, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x43, 0x6f, 0x72, 0xe9, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72,
0xe9, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0xef, 0x61, 0x72, 0x73,
0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0xef, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0xef, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x75, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x68, 0x69, 0x7a, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x4c, 0x69, 0x67, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x63, 0x61, 0x6f, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x69, 0x73, 0x69, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75,
0x69, 0x73, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x63, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x71,
0x75, 0x65, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x71, 0x75,
0x65, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65, 0x20, 0x6d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0x4f, 0x75, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x4f, 0x75, 0x6c, 0x61, 0x6e, 0x2d,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x2019, 0x4f, 0x75, 0x6c, 0x61, 0x6e,
0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x75, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x4e, 0xe9, 0x70, 0x61,
0x6c, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x43, 0x61,
0x6c, 0xe9, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d,
0x43, 0x61, 0x6c, 0xe9, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x43, 0x61, 0x6c,
0xe9, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c,
0x6c, 0x65, 0x2d, 0x5a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c,
0x6c, 0x65, 0x2d, 0x5a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65,
0x2d, 0x5a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x4e,
0x65, 0x75, 0x76, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x65, 0x2d, 0x4e, 0x65, 0x75, 0x76, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x54,
0x65, 0x72, 0x72, 0x65, 0x2d, 0x4e, 0x65, 0x75, 0x76, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64,
0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20,
0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x6e, 0x65, 0x73, 0x20, 0x64, 0x75,
0x20, 0x4e, 0x6f, 0x72, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x6f, 0x73,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x50, 0x61, 0x70, 0x6f, 0x75, 0x61, 0x73, 0x69, 0x65, 0x2d, 0x4e, 0x6f,
0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x47, 0x75, 0x69, 0x6e, 0xe9,
0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x75, 0x20, 0x50, 0xe9, 0x72, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75,
0x20, 0x50, 0xe9, 0x72, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x50, 0xe9, 0x72,
0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x73, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x73, 0x20, 0x50, 0x68, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x50, 0x68,
0x6f, 0x65, 0x6e, 0x69, 0x78, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65,
0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73,
0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67,
0x79, 0x61, 0x6e, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x61, 0x20,
0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x73, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x73, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65,
0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x6f, 0x75, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x53, 0x61,
0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x47, 0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x20, 0x64, 0x75,
0x20, 0x53, 0x75, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75,
0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019,
0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x54, 0x61,
0x64, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x75, 0x75,
0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9,
0x74, 0xe9, 0x20, 0x64, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe9,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x73, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x7a, 0x62, 0xe9, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f,
0x75, 0x7a, 0x62, 0xe9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x7a, 0x62, 0xe9, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x75,
0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c,
0x65, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x65, 0x74,
0x2d, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x6f, 0x75, 0x74, 0x73,
0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74, 0xe9,
0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75, 0x72, 0x67, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x75,
0x72, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0xe9, 0x74,
0xe9, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x6f, 0x75, 0x72, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x75, 0x20, 0x59, 0x75,
0x6b, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x63,
0x72, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66,
0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66,
0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x71,
0x75, 0x65, 0x20, 0x64, 0x75, 0x20, 0x53, 0x75, 0x64, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x2019, 0x41, 0x66, 0x72, 0x69, 0x71, 0x75, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019,
0x41, 0x6c, 0x6d, 0x61, 0x20, 0x41, 0x74, 0x61, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x65, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x75, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x75, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x73,
0x20, 0x52, 0x6f, 0x63, 0x68, 0x65, 0x75, 0x73, 0x65, 0x73, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x75, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x71, 0x75, 0x65,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x2019, 0x41, 0x70, 0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x6b,
0x74, 0x61, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x71, 0x74, 0xf6,
0x62, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61,
0x62, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0xe9,
0x6e, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x71, 0x75, 0x65, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x75, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x2d, 0x6f, 0x75,
0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x4f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0xef, 0x64, 0x6a, 0x61,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65,
0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x75, 0x20, 0x42, 0x72, 0x75, 0x6e, 0xe9, 0x69, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x75,
0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x74, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x75, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65,
0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x65, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50,
0xe2, 0x71, 0x75, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x73,
0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x4f, 0x75, 0x65, 0x73, 0x74, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee,
0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x6f, 0x75, 0x69, 0x6e, 0x65,
0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20, 0xee, 0x6c, 0x65, 0x73, 0x20,
0x46, 0x69, 0x64, 0x6a, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x64,
0x65, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x65, 0x20, 0x66, 0x72, 0x61,
0x6e, 0xe7, 0x61, 0x69, 0x73, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47,
0xe9, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x73, 0x74, 0x20, 0x64, 0x75, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x4f, 0x75, 0x65, 0x73, 0x74, 0x20, 0x64, 0x75, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x2019, 0x48, 0x61, 0x77, 0x61, 0xef, 0x2d, 0x41, 0x6c, 0xe9, 0x6f,
0x75, 0x74, 0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019,
0x48, 0x61, 0x77, 0x61, 0xef, 0x2d, 0x41, 0x6c, 0xe9, 0x6f, 0x75, 0x74,
0x69, 0x65, 0x6e, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x48, 0x61,
0x77, 0x61, 0xef, 0x2d, 0x41, 0x6c, 0xe9, 0x6f, 0x75, 0x74, 0x69, 0x65,
0x6e, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x75, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0xe9, 0x65, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0xef, 0x61, 0x72,
0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63,
0x61, 0x6f, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x63, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x71, 0x75, 0x65, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x69,
0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x2019, 0x4f, 0x75, 0x6c, 0x61, 0x6e, 0x2d, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0x6f, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x75, 0x76,
0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x43, 0x61, 0x6c, 0xe9, 0x64, 0x6f, 0x6e,
0x69, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f,
0x75, 0x76, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x5a, 0xe9, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x65,
0x2d, 0x4e, 0x65, 0x75, 0x76, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x69, 0x6f, 0x75, 0xe9, 0x68, 0x65, 0x75, 0x72,
0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0xee, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x2019, 0x4f,
0x6d, 0x73, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9,
0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63,
0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x50, 0xe9, 0x72, 0x6f, 0x75, 0x68,
0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65,
0x20, 0x64, 0x65, 0x73, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x65, 0x73, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74,
0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x52, 0xe9, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x65, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20,
0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x73, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61,
0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76,
0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e,
0x63, 0xe9, 0x65, 0x20, 0x64, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0xe9, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x75, 0x7a, 0x62, 0xe9, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x68, 0x65, 0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61,
0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x65, 0x75, 0x72, 0x65,
0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x68, 0x65, 0x75,
0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20, 0x64,
0x65, 0x20, 0x49, 0x61, 0x6b, 0x6f, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x65,
0x75, 0x72, 0x65, 0x20, 0x61, 0x76, 0x61, 0x6e, 0x63, 0xe9, 0x65, 0x20,
0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f,
0x75, 0x72, 0x67, 0x4f, 0x72, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe2, 0x6c,
0x4f, 0x72, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe2, 0x6c, 0x4f, 0x72,
0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x6f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x20, 0x6f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0xe2, 0x6c, 0x4f, 0x72, 0x65, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x20,
0x6f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0xe2, 0x6c, 0x4f, 0x72, 0x65,
0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x68, 0x65, 0x4f, 0x72,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x68, 0x65, 0x4f, 0x72, 0x65, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x68, 0x65, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a,
0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a,
0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a,
0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2c, 0xd83a, 0xdd3a, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a,
0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd2c, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd2c, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2c, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2c, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2c, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2c, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0x20, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0x20, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a,
0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a,
0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd41,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd41,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25,
0xd83a, 0xdd22, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x20,
0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a,
0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a,
0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a,
0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a,
0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a,
0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35,
0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd10, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a,
0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a,
0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd46, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a,
0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a,
0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd46, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd33, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd10, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd10,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b,
0xd83a, 0xdd30, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd46, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd00, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd28, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd28, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd33,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a,
0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd33, 0xd83a, 0xdd3c, 0xd83a,
0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd33, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd26,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd22,
0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00,
0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a,
0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a,
0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c,
0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a,
0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a,
0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a,
0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a,
0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35,
0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a,
0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a,
0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20,
0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1a,
0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a,
0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20,
0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1a,
0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20,
0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20,
0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f,
0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0xd83a,
0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd00, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a,
0xdd34, 0xd83a, 0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd36, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a,
0xdd36, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd41,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd41, 0xd83a, 0xdd2e,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd41,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd04, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd35,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd04, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2a,
0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd04, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd04, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd26, 0xd83a, 0xdd2e,
0x2d, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0x2d, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a,
0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2e, 0xd83a, 0xdd25,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a,
0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd22, 0xd83a,
0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd2e,
0xd83a, 0xdd34, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd27, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd26,
0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd05, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd24,
0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a,
0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd25,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd26,
0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd24,
0xd83a, 0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd2e,
0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45,
0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd37, 0xd83a, 0xdd46, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2d,
0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0x2d, 0xd83a,
0xdd01, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3e, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd1a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd25, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x2d, 0xd83a, 0xdd00,
0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x2d, 0xd83a,
0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22,
0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a,
0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a,
0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26, 0xd83a,
0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd26,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a,
0xdd33, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0x2d, 0xd83a, 0xdd00,
0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e,
0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd34, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a,
0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd34,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd22,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0x2d, 0xd83a,
0xdd0a, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd02, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0x20, 0x26, 0x20, 0xd83a, 0xdd00, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd28, 0xd83a,
0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd18, 0xd83a, 0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a,
0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2d, 0xd83a,
0xdd24, 0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd32,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd31, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35,
0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd2c, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18,
0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a,
0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd18, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a, 0xdd2d,
0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a,
0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd18, 0xd83a,
0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd25, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd3b, 0xd83a, 0xdd35, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd34, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd2e,
0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0x2d, 0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0x2d,
0xd83a, 0xdd00, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd3a, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd2c, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd2e, 0xd83a, 0xdd2c, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd2e, 0xd83a, 0xdd2c, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a,
0xdd2e, 0xd83a, 0xdd25, 0xd83a, 0xdd26, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd2f, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd0b, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0b, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b,
0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27,
0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c,
0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0b,
0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27,
0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a,
0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd0b, 0xd83a, 0xdd27, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0b, 0xd83a, 0xdd27, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a,
0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd46,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd46, 0xd83a, 0xdd2e,
0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd46,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a,
0xdd41, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2d,
0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd3a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd41, 0xd83a,
0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a,
0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd32,
0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd11, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27, 0xd83a, 0xdd32, 0xd83a, 0xdd2e,
0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd27,
0xd83a, 0xdd32, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd02, 0xd83a,
0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd02,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0x2d, 0xd83a, 0xdd16, 0xd83a,
0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a,
0xdd35, 0x2d, 0xd83a, 0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd2b, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd02, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd35, 0x2d, 0xd83a,
0xdd16, 0xd83a, 0xdd2e, 0xd83a, 0xdd31, 0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03,
0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03,
0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a,
0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a,
0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd34, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03,
0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd22,
0xd83a, 0xdd27, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22,
0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0xd83a,
0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd03,
0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a, 0xdd46, 0xd83a, 0xdd2e, 0x20, 0xd83a,
0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd01, 0xd83a, 0xdd2b, 0xd83a, 0xdd30, 0xd83a,
0xdd46, 0xd83a, 0xdd2e, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd13, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd13, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd26, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd13, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd22, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e,
0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd33,
0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd33,
0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd22, 0xd83a, 0xdd25, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd28, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd23,
0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a,
0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2b, 0xd83a, 0xdd27,
0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d,
0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2b, 0xd83a, 0xdd27, 0xd83a, 0xdd2e, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd1f, 0xd83a,
0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a,
0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd1f, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd22,
0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2b,
0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd1f, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2d, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2b,
0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2b, 0xd83a,
0xdd31, 0x2d, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd35,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd10,
0xd83a, 0xdd2b, 0xd83a, 0xdd31, 0x2d, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd31, 0xd83a, 0xdd2d,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a,
0xd83a, 0xdd2c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e,
0xd83a, 0xdd2a, 0xd83a, 0xdd2c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd33, 0xd83a, 0xdd35,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a,
0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x20, 0xd83a, 0xdd23,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd3d, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a,
0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x20,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3d, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a,
0xdd0a, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a,
0xdd23, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0x20, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3d, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a,
0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27,
0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd10, 0xd83a, 0xdd2e, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27,
0xd83a, 0xdd26, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd27, 0xd83a, 0xdd2d, 0xd83a, 0xdd33,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a, 0xdd33, 0xd83a,
0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd0c, 0xd83a, 0xdd25, 0xd83a, 0xdd27, 0xd83a, 0xdd35, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd33,
0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06,
0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd33,
0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd31, 0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd22,
0xd83a, 0xdd28, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0x20, 0xd83a, 0xdd18, 0xd83a,
0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd2b, 0xd83a, 0xdd27,
0xd83a, 0xdd2e, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d,
0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24,
0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd0a, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd28, 0xd83a, 0xdd2d,
0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a,
0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0a,
0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd27, 0xd83a, 0xdd2d,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd3c, 0x2e, 0x20, 0xd83a, 0xdd06, 0xd83a,
0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd34, 0x20, 0x26, 0x20,
0xd83a, 0xdd03, 0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2e,
0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd05, 0xd83a, 0xdd3c, 0x2e, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2d, 0xd83a, 0xdd34,
0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0x20, 0x26, 0x20, 0xd83a, 0xdd03, 0xd83a,
0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd3c, 0x2e, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2d, 0xd83a,
0xdd34, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0x20, 0x26, 0x20, 0xd83a, 0xdd03,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2b, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd33, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd06, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd28,
0xd83a, 0xdd2b, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd06, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a,
0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a,
0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd24,
0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd24, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a,
0xd83a, 0xdd23, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd08, 0xd83a,
0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd08, 0xd83a, 0xdd2e, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a,
0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a,
0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd3f, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd25,
0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd05, 0xd83a, 0xdd2b, 0xd83a, 0xdd34, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd2b, 0xd83a,
0xdd24, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a,
0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd28, 0xd83a,
0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35,
0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd36, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd25, 0xd83a,
0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a,
0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22,
0xd83a, 0xdd31, 0xd83a, 0xdd2c, 0x2d, 0xd83a, 0xdd14, 0xd83a, 0xdd2e, 0xd83a, 0xdd2a, 0xd83a,
0xdd36, 0xd83a, 0xdd2d, 0xd83a, 0xdd34, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd05, 0xd83a, 0xdd2d, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd22,
0xd83a, 0xdd25, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd34, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a,
0xd83a, 0xdd22, 0xd83a, 0xdd38, 0xd83a, 0xdd2d, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd45,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd28, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd28, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f,
0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd34, 0xd83a, 0xdd28, 0xd83a,
0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd22, 0xd83a, 0xdd36,
0xd83a, 0xdd2d, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a,
0xdd31, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32,
0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a,
0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a,
0xdd35, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd2e, 0xd83a, 0xdd32, 0xd83a, 0xdd3a, 0xd83a, 0xdd22,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a,
0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a,
0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd33,
0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd27,
0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a,
0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd2a,
0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd25, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d,
0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20,
0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd25,
0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22,
0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1a, 0xd83a, 0xdd35, 0xd83a,
0xdd45, 0xd83a, 0xdd3e, 0xd83a, 0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a,
0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd12, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e,
0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a,
0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20,
0xd83a, 0xdd12, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e,
0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a,
0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd35, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd34, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd13, 0xd83a,
0xdd41, 0xd83a, 0xdd26, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd3c, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a,
0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16,
0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b,
0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd13, 0xd83a, 0xdd41, 0xd83a, 0xdd26, 0xd83a,
0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd13, 0xd83a, 0xdd41, 0xd83a, 0xdd26, 0xd83a,
0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd32, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3c, 0xd83a, 0xdd35,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a,
0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c,
0xd83a, 0xdd22, 0xd83a, 0xdd32, 0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44,
0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd22, 0xd83a, 0xdd32,
0xd83a, 0xdd35, 0xd83a, 0xdd31, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd3c, 0xd83a, 0xdd35,
0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a,
0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2b, 0xd83a, 0xdd32, 0xd83a, 0xdd2d, 0xd83a,
0xdd45, 0xd83a, 0xdd41, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2d, 0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd3e,
0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd11,
0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a,
0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2d,
0xd83a, 0xdd24, 0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c,
0xd83a, 0xdd2e, 0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e,
0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a,
0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2d, 0xd83a, 0xdd24,
0xd83a, 0xdd22, 0xd83a, 0xdd3e, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e,
0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a,
0xdd3a, 0xd83a, 0xdd2e, 0xd83a, 0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a,
0xdd44, 0xd83a, 0xdd23, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31,
0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b,
0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a,
0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd23, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35,
0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd24, 0xd83a, 0xdd3a, 0xd83a, 0xdd2e, 0xd83a,
0xdd3a, 0xd83a, 0xdd22, 0xd83a, 0xdd2a, 0xd83a, 0xdd22, 0xd83a, 0xdd44, 0xd83a, 0xdd23, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd1c, 0xd83a, 0xdd2e, 0xd83a, 0xdd27, 0xd83a, 0xdd3c, 0xd83a, 0xdd2e,
0xd83a, 0xdd33, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd05, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a,
0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0f, 0xd83a, 0xdd2b,
0xd83a, 0xdd45, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd0f, 0xd83a,
0xdd22, 0xd83a, 0xdd24, 0xd83a, 0xdd46, 0xd83a, 0xdd2d, 0xd83a, 0xdd45, 0xd83a, 0xdd27, 0x20,
0x26, 0x20, 0xd83a, 0xdd0a, 0xd83a, 0xdd35, 0xd83a, 0xdd3c, 0xd83a, 0xdd35, 0xd83a, 0xdd32,
0xd83a, 0xdd22, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45,
0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a, 0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35,
0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a, 0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a,
0xdd22, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a,
0xdd33, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a,
0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45,
0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd22, 0xd83a, 0xdd33, 0xd83a,
0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2d, 0xd83a, 0xdd27, 0xd83a, 0xdd33, 0xd83a, 0xdd35, 0xd83a,
0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a,
0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c,
0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd35,
0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d, 0xd83a, 0xdd36,
0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd16, 0xd83a,
0xdd22, 0xd83a, 0xdd31, 0xd83a, 0xdd2a, 0xd83a, 0xdd35, 0xd83a, 0xdd32, 0xd83a, 0xdd4b, 0xd83a,
0xdd23, 0xd83a, 0xdd2b, 0x20, 0xd83a, 0xdd12, 0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd22,
0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a, 0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd26,
0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a, 0xd83a, 0xdd35, 0xd83a, 0xdd11, 0xd83a, 0xdd2d,
0xd83a, 0xdd36, 0xd83a, 0xdd2e, 0xd83a, 0xdd45, 0xd83a, 0xdd2a, 0xd83a, 0xdd2b, 0x20, 0xd83a,
0xdd15, 0xd83a, 0xdd2b, 0xd83a, 0xdd45, 0xd83a, 0xdd2f, 0xd83a, 0xdd35, 0x20, 0xd83a, 0xdd12,
0xd83a, 0xdd2b, 0xd83a, 0xdd33, 0xd83a, 0xdd22, 0xd83a, 0xdd3c, 0xd83a, 0xdd2b, 0xd83a, 0xdd2a,
0xd83a, 0xdd2d, 0xd83a, 0xdd32, 0xd83a, 0xdd26, 0xd83a, 0xdd35, 0xd83a, 0xdd2a, 0xd83a, 0xdd3a,
0xd83a, 0xdd35, 0xc0, 0x6d, 0x20, 0x41, 0x63, 0x72, 0x65, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x63, 0x72, 0x65, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41,
0x63, 0x72, 0x65, 0xc0, 0x6d, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e,
0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0xc0, 0x6d, 0x20, 0x4d, 0x65, 0x61,
0x64, 0x68, 0x61, 0x6e, 0x20, 0x41, 0x66, 0x72, 0x61, 0x67, 0x61, 0xc0,
0x6d, 0x20, 0x41, 0x66, 0x72, 0x61, 0x67, 0x61, 0x20, 0x61, 0x6e, 0x20,
0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x41, 0x66, 0x72, 0x61, 0x67, 0x61,
0x20, 0x61, 0x20, 0x44, 0x65, 0x61, 0x73, 0xc0, 0x6d, 0x20, 0x41, 0x66,
0x72, 0x61, 0x67, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x66, 0x72, 0x61, 0x67, 0x61,
0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x66,
0x72, 0x61, 0x67, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0,
0x6d, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0xc0, 0x6d, 0x20, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0xc0, 0x6d, 0x20, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x69, 0x6e, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x69,
0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x69, 0x6e, 0xc0,
0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x41, 0x69,
0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x20, 0x61, 0x20, 0x54,
0x75, 0x61, 0x74, 0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d,
0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x41, 0x69, 0x6d, 0x65, 0x69,
0x72, 0x65, 0x61, 0x67, 0x61, 0x20, 0x61, 0x20, 0x54, 0x75, 0x61, 0x74,
0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20,
0x41, 0x69, 0x6d, 0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x20, 0x61,
0x20, 0x54, 0x75, 0x61, 0x74, 0x68, 0xc0, 0x6d, 0x20, 0x41, 0x69, 0x6d,
0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x20, 0x61, 0x20, 0x54, 0x75,
0x61, 0x74, 0x68, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x69, 0x6d, 0x65, 0x69, 0x72, 0x65,
0x61, 0x67, 0x61, 0x20, 0x61, 0x20, 0x54, 0x75, 0x61, 0x74, 0x68, 0x20,
0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x69, 0x6d,
0x65, 0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x20, 0x61, 0x20, 0x54, 0x75,
0x61, 0x74, 0x68, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d,
0x20, 0x4d, 0x6f, 0x6e, 0x61, 0x64, 0x68, 0x20, 0x41, 0x69, 0x6d, 0x65,
0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x20, 0x61, 0x20, 0x54, 0x75, 0x61,
0x74, 0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x6f, 0x6e,
0x61, 0x64, 0x68, 0x20, 0x41, 0x69, 0x6d, 0x65, 0x69, 0x72, 0x65, 0x61,
0x67, 0x61, 0x20, 0x61, 0x20, 0x54, 0x75, 0x61, 0x74, 0x68, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x4d, 0x6f, 0x6e, 0x61, 0x64, 0x68, 0x20, 0x41, 0x69, 0x6d, 0x65,
0x69, 0x72, 0x65, 0x61, 0x67, 0x61, 0x20, 0x61, 0x20, 0x54, 0x75, 0x61,
0x74, 0x68, 0xc0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61,
0x69, 0x6e, 0x20, 0x53, 0xe8, 0x69, 0x6d, 0x68, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e,
0x20, 0x53, 0xe8, 0x69, 0x6d, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x61, 0x2019, 0x20,
0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20, 0x53, 0xe8, 0x69, 0x6d, 0x68,
0xc0, 0x6d, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0xc0, 0x6d, 0x20, 0x41, 0x70,
0x69, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x70, 0x69,
0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x41, 0x70, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x41,
0x71, 0x74, 0x61, 0x75, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41,
0x71, 0x74, 0x61, 0x75, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75,
0xc0, 0x6d, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0xc0, 0x6d, 0x20, 0x41, 0x72,
0x61, 0x62, 0x61, 0x63, 0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x41, 0x72, 0x61, 0x62, 0x61, 0x63, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x72,
0x61, 0x62, 0x61, 0x63, 0x68, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x2d, 0x41, 0x72, 0x67, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x65, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x2d, 0x41,
0x72, 0x67, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x65, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x2d, 0x41, 0x72, 0x67, 0x61, 0x6e, 0x74, 0x61,
0x69, 0x6e, 0x65, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x2d, 0x41,
0x72, 0x67, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x65, 0x20, 0x53, 0x69,
0x61, 0x72, 0x61, 0x69, 0x63, 0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x2d, 0x41, 0x72, 0x67, 0x61, 0x6e, 0x74,
0x61, 0x69, 0x6e, 0x65, 0x20, 0x53, 0x69, 0x61, 0x72, 0x61, 0x69, 0x63,
0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x2d, 0x41, 0x72, 0x67,
0x61, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x65, 0x20, 0x53, 0x69, 0x61, 0x72,
0x61, 0x69, 0x63, 0x68, 0xc0, 0x6d, 0x20, 0x41, 0x69, 0x72, 0x6d, 0x65,
0x69, 0x6e, 0x69, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41,
0x69, 0x72, 0x6d, 0x65, 0x69, 0x6e, 0x69, 0x61, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41,
0x69, 0x72, 0x6d, 0x65, 0x69, 0x6e, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x61,
0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20, 0x53, 0x69, 0x61,
0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43,
0x68, 0x75, 0x61, 0x69, 0x6e, 0x20, 0x53, 0x69, 0x61, 0x72, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20, 0x53,
0x69, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61,
0x6e, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61,
0x6e, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x41, 0x73,
0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x4d, 0x65,
0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69,
0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e,
0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61,
0x6e, 0x20, 0x49, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x65, 0x61, 0x64,
0x68, 0x61, 0x6e, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69,
0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x41,
0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20,
0x45, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x73,
0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45,
0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c,
0x69, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20,
0x41, 0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e,
0x20, 0x49, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41,
0x73, 0x74, 0x72, 0xe0, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20,
0x49, 0x61, 0x72, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x73, 0x74, 0x72, 0xe0, 0x69,
0x6c, 0x69, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d,
0x20, 0x41, 0x73, 0x61, 0x72, 0x62, 0x61, 0x69, 0x64, 0x65, 0xe0, 0x69,
0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x41, 0x73, 0x61, 0x72,
0x62, 0x61, 0x69, 0x64, 0x65, 0xe0, 0x69, 0x6e, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41,
0x73, 0x61, 0x72, 0x62, 0x61, 0x69, 0x64, 0x65, 0xe0, 0x69, 0x6e, 0xc0,
0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e,
0x61, 0x6e, 0x20, 0x41, 0x73, 0x6f, 0x72, 0x61, 0x63, 0x68, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c,
0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x41, 0x73, 0x6f, 0x72, 0x61, 0x63,
0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65,
0x61, 0x6e, 0x61, 0x6e, 0x20, 0x41, 0x73, 0x6f, 0x72, 0x61, 0x63, 0x68,
0xc0, 0x6d, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x61, 0x69,
0x73, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x61, 0x69, 0x73, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x61, 0x69, 0x73, 0xc0, 0x6d, 0x20, 0x42, 0x75,
0x74, 0xe0, 0x69, 0x6e, 0xc0, 0x6d, 0x20, 0x42, 0x6f, 0x69, 0x6c, 0x69,
0x62, 0x68, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x42, 0x68, 0x72, 0x61, 0x73,
0xed, 0x6c, 0x69, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x42,
0x68, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x42,
0x68, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x42,
0x68, 0x72, 0xf9, 0x6e, 0x61, 0x69, 0x67, 0x68, 0x20, 0x44, 0xe0, 0x72,
0x20, 0x61, 0x73, 0x2d, 0x53, 0x61, 0x6c, 0xe0, 0x6d, 0xc0, 0x6d, 0x20,
0x61, 0x2019, 0x20, 0x43, 0x68, 0x69, 0x70, 0x20, 0x55, 0x61, 0x69, 0x6e,
0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43,
0x68, 0x69, 0x70, 0x20, 0x55, 0x61, 0x69, 0x6e, 0x65, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x61, 0x2019, 0x20, 0x43, 0x68, 0x69, 0x70, 0x20, 0x55, 0x61, 0x69, 0x6e,
0x65, 0xc0, 0x6d, 0x20, 0x43, 0x68, 0x61, 0x73, 0x65, 0x79, 0xc0, 0x6d,
0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0xc0, 0x6d, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0xc0, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x53, 0x69, 0x6c, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x53, 0x69, 0x6c, 0x65, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53,
0xec, 0x6e, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x53, 0xec, 0x6e, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53,
0xec, 0x6e, 0x65, 0xc0, 0x6d, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x43,
0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0xc0, 0x6d,
0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x4e,
0x6f, 0x6c, 0x6c, 0x61, 0x69, 0x67, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c,
0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x43, 0x68, 0x6f, 0x63, 0x6f, 0x73,
0xc0, 0x6d, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x69, 0x6d, 0x62, 0x69, 0x61,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x69,
0x6d, 0x62, 0x69, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x69,
0x6d, 0x62, 0x69, 0x61, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61,
0x6e, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x4c, 0x65, 0x74, 0x68, 0x2d, 0x74, 0x68, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0xc0, 0x6d, 0x20, 0x43, 0xf9, 0x62, 0x61, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x43, 0xf9, 0x62, 0x61, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43,
0xf9, 0x62, 0x61, 0xc0, 0x6d, 0x20, 0x44, 0x68, 0x61, 0x76, 0x69, 0x73,
0xc0, 0x6d, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019,
0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0xc0, 0x6d, 0x20, 0x54, 0x68,
0xec, 0x6f, 0x6d, 0x6f, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72,
0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x6e, 0x61,
0x20, 0x43, 0xe0, 0x69, 0x73, 0x67, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x20,
0x43, 0xe0, 0x69, 0x73, 0x67, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x45, 0x69, 0x6c,
0x65, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x43, 0xe0, 0x69, 0x73, 0x67,
0x65, 0xc0, 0x6d, 0x20, 0x45, 0x61, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x69,
0x72, 0xc0, 0x6d, 0x20, 0x4d, 0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20,
0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72,
0x70, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x65, 0x61,
0x64, 0x68, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e,
0x6e, 0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x65,
0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69,
0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72, 0x70, 0x61, 0xc0, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72, 0x70,
0x61, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d,
0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d,
0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72,
0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d,
0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x6e, 0x61, 0x73, 0x20, 0x66, 0x68,
0x61, 0x69, 0x64, 0x65, 0x20, 0x65, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d, 0x45, 0xf2, 0x72, 0x70,
0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d,
0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x69, 0x6e, 0x6e, 0x2d,
0x45, 0xf2, 0x72, 0x70, 0x61, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72,
0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61,
0x6e, 0x61, 0x6e, 0x20, 0x46, 0xe0, 0x63, 0x6c, 0x61, 0x6e, 0x6e, 0x61,
0x63, 0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x6e,
0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x46, 0xe0,
0x63, 0x6c, 0x61, 0x6e, 0x6e, 0x61, 0x63, 0x68, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e,
0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20,
0x46, 0xe0, 0x63, 0x6c, 0x61, 0x6e, 0x6e, 0x61, 0x63, 0x68, 0xc0, 0x6d,
0x20, 0x46, 0xec, 0x64, 0x69, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x46, 0xec, 0x64, 0x69, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x46, 0xec, 0x64, 0x69, 0xc0,
0x6d, 0x20, 0x47, 0x75, 0x69, 0x64, 0x68, 0x65, 0xe0, 0x6e, 0x61, 0x20,
0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x69, 0x6e, 0x67, 0x65, 0xc0, 0x6d,
0x20, 0x44, 0x65, 0x61, 0x73, 0x61, 0x63, 0x68, 0x20, 0x61, 0x67, 0x75,
0x73, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x61, 0x69, 0x67, 0x65,
0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x69, 0x6e,
0x67, 0x65, 0xc0, 0x6d, 0x20, 0x47, 0x68, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x6f, 0x73, 0xc0, 0x6d, 0x20, 0x47, 0x68, 0x61, 0x6d, 0x62, 0x69,
0x65, 0x72, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x61, 0x69, 0x72,
0x74, 0x62, 0x68, 0x65, 0x69, 0x6c, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x61, 0x69, 0x72, 0x74, 0x62, 0x68,
0x65, 0x69, 0x6c, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x61,
0x69, 0x72, 0x74, 0x62, 0x68, 0x65, 0x69, 0x6c, 0x65, 0xc0, 0x6d, 0x20,
0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x47, 0x68, 0x69,
0x6c, 0x65, 0x61, 0x62, 0x61, 0x69, 0x72, 0x74, 0xc0, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e,
0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61,
0x69, 0x6e, 0x6e, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69,
0x6e, 0x6e, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e,
0x6e, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c,
0x61, 0x69, 0x6e, 0x6e, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61,
0x69, 0x6e, 0x6e, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d,
0x20, 0x47, 0x75, 0x61, 0x6d, 0xc0, 0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43,
0x68, 0x61, 0x6d, 0x61, 0x69, 0x73, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e,
0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x2019, 0x69, 0x20, 0x2019, 0x73, 0x20, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x61, 0x63, 0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2019, 0x69, 0x20, 0x2019, 0x73, 0x20,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x63, 0x68, 0x54, 0xec, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e,
0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x2019, 0x69, 0x20, 0x2019, 0x73, 0x20, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x61, 0x63, 0x68, 0xc0, 0x6d, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0xc0,
0x6d, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x48, 0x6f, 0x76, 0x64,
0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x6e, 0x73, 0x65,
0x61, 0x63, 0x68, 0x61, 0x6e, 0xc0, 0x6d, 0x20, 0x43, 0x75, 0x61, 0x6e,
0x20, 0x6e, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x6e, 0x73, 0x65, 0x61, 0x63,
0x68, 0x61, 0x6e, 0xc0, 0x6d, 0x20, 0x53, 0xec, 0x6e, 0x2d, 0x49, 0x6e,
0x6e, 0x73, 0x65, 0x61, 0x6e, 0x61, 0x63, 0x68, 0xc0, 0x6d, 0x20, 0x4d,
0x65, 0x61, 0x64, 0x68, 0x61, 0x6e, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x49,
0x6e, 0x6e, 0x64, 0x2d, 0x49, 0x6e, 0x6e, 0x73, 0x65, 0xc0, 0x6d, 0x20,
0x6e, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x6e, 0x64, 0x2d, 0x49, 0x6e, 0x6e,
0x73, 0x65, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20,
0x6e, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x6e, 0x64, 0x2d, 0x49, 0x6e, 0x6e,
0x73, 0x65, 0x20, 0x61, 0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20,
0x49, 0x6f, 0x72, 0xe0, 0x69, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x49, 0x6f, 0x72, 0xe0, 0x69, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x6f,
0x72, 0xe0, 0x69, 0x6e, 0xc0, 0x6d, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x54, 0xec, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0xc0, 0x6d, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c,
0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x65, 0x61, 0x70, 0x61, 0x69,
0x6e, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x20,
0x53, 0x65, 0x61, 0x70, 0x61, 0x69, 0x6e, 0x65, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e,
0x61, 0x20, 0x53, 0x65, 0x61, 0x70, 0x61, 0x69, 0x6e, 0x65, 0xc0, 0x6d,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x79, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x79, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x79, 0xc0,
0x6d, 0x20, 0x43, 0x61, 0x73, 0x61, 0x63, 0x68, 0x73, 0x74, 0xe0, 0x69,
0x6e, 0x20, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x43,
0x61, 0x73, 0x61, 0x63, 0x68, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0x20, 0x61,
0x6e, 0x20, 0x49, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x72, 0xe8, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x43, 0x68,
0x6f, 0x69, 0x72, 0xe8, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x72, 0xe8, 0x61, 0xc0, 0x6d, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65,
0xc0, 0x6d, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0xc0, 0x6d,
0x20, 0x43, 0xec, 0x6f, 0x72, 0x67, 0x61, 0x73, 0x74, 0x61, 0x69, 0x6e,
0xc0, 0x6d, 0x20, 0x4c, 0x61, 0x6e, 0x63, 0x61, 0xc0, 0x6d, 0x20, 0x45,
0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x54, 0x65, 0x72, 0x61,
0x69, 0x6e, 0x61, 0xc0, 0x6d, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0xc0, 0x6d, 0x20, 0x4d, 0x61,
0x63, 0xe0, 0x74, 0x68, 0x75, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x4d, 0x61, 0x63, 0xe0, 0x74, 0x68, 0x75, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x61,
0x63, 0xe0, 0x74, 0x68, 0x75, 0xc0, 0x6d, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0xc0, 0x6d, 0x20, 0x4d, 0x68, 0x61, 0x6c, 0x61, 0x69,
0x64, 0x68, 0x73, 0x65, 0x61, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20,
0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x61,
0x6c, 0x61, 0x64, 0x61, 0x69, 0x62, 0x68, 0xc0, 0x6d, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x61, 0x2019, 0x20, 0x4d, 0x68,
0x61, 0x72, 0x63, 0x61, 0x69, 0x73, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c,
0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x4d, 0x68, 0x6f, 0x69, 0x72,
0x69, 0x73, 0x65, 0x61, 0x73, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e,
0x20, 0x4d, 0x68, 0x6f, 0x69, 0x72, 0x69, 0x73, 0x65, 0x61, 0x73, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e,
0x61, 0x6e, 0x20, 0x4d, 0x68, 0x6f, 0x69, 0x72, 0x69, 0x73, 0x65, 0x61,
0x73, 0xc0, 0x6d, 0x20, 0x4d, 0x68, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0xc0,
0x6d, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20,
0x53, 0xe8, 0x69, 0x6d, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x61, 0x67, 0x73,
0x61, 0x67, 0x6f, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x61, 0x2019,
0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20, 0x53, 0xe8, 0x69, 0x6d,
0x68, 0x20, 0x4d, 0x68, 0x65, 0x61, 0x67, 0x73, 0x61, 0x67, 0x6f, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0x75, 0x61, 0x69, 0x6e, 0x20,
0x53, 0xe8, 0x69, 0x6d, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x61, 0x67, 0x73,
0x61, 0x67, 0x6f, 0xc0, 0x6d, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0xc0, 0x6d,
0x20, 0x4d, 0x68, 0x6f, 0x73, 0x67, 0x6f, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x4d, 0x68, 0x6f, 0x73, 0x67, 0x6f, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d,
0x68, 0x6f, 0x73, 0x67, 0x6f, 0xc0, 0x6d, 0x20, 0x4d, 0x69, 0xe0, 0x6e,
0x6d, 0x61, 0x72, 0xc0, 0x6d, 0x20, 0x4e, 0x61, 0x62, 0x68, 0x72, 0x75,
0xc0, 0x6d, 0x20, 0x4e, 0x65, 0x61, 0x70, 0xe0, 0x6c, 0xc0, 0x6d, 0x20,
0x43, 0x68, 0x61, 0x69, 0x6c, 0x6c, 0x65, 0x61, 0x6e, 0x6e, 0x20, 0x4e,
0x75, 0x61, 0x69, 0x64, 0x68, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20,
0x43, 0x68, 0x61, 0x69, 0x6c, 0x6c, 0x65, 0x61, 0x6e, 0x6e, 0x20, 0x4e,
0x75, 0x61, 0x69, 0x64, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x68, 0x61, 0x69,
0x6c, 0x6c, 0x65, 0x61, 0x6e, 0x6e, 0x20, 0x4e, 0x75, 0x61, 0x69, 0x64,
0x68, 0xc0, 0x6d, 0x20, 0x53, 0x68, 0x65, 0x61, 0x6c, 0x61, 0x69, 0x6e,
0x6e, 0x20, 0x4e, 0x75, 0x61, 0x69, 0x64, 0x68, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x53, 0x68, 0x65, 0x61, 0x6c, 0x61, 0x69, 0x6e, 0x6e,
0x20, 0x4e, 0x75, 0x61, 0x69, 0x64, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53, 0x68,
0x65, 0x61, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x20, 0x4e, 0x75, 0x61, 0x69,
0x64, 0x68, 0xc0, 0x6d, 0x20, 0x54, 0x61, 0x6c, 0x61, 0x6d, 0x68, 0x20,
0x61, 0x6e, 0x20, 0xc8, 0x69, 0x73, 0x67, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x54, 0x61, 0x6c, 0x61, 0x6d, 0x68, 0x20, 0x61, 0x6e, 0x20,
0xc8, 0x69, 0x73, 0x67, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x54, 0x61, 0x6c, 0x61, 0x6d,
0x68, 0x20, 0x61, 0x6e, 0x20, 0xc8, 0x69, 0x73, 0x67, 0xc0, 0x6d, 0x20,
0x4e, 0x69, 0x75, 0x65, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x69,
0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x69, 0x6e, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x45, 0x69, 0x6c,
0x65, 0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xc0,
0x6d, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x54,
0xec, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0xc0, 0x6d, 0x20,
0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e,
0x20, 0x4d, 0x61, 0x69, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x61, 0x20, 0x54, 0x75, 0x61, 0x74, 0x68, 0xc0, 0x6d, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0xc0, 0x6d, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0xc0, 0x6d, 0x20, 0x50, 0x61, 0x67,
0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x50, 0x61, 0x67, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0x54, 0xec,
0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x50, 0x61, 0x67, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0xc0, 0x6d,
0x20, 0x50, 0x61, 0x6c, 0x61, 0x62, 0x68, 0xc0, 0x6d, 0x20, 0x47, 0x69,
0x6e, 0x69, 0x20, 0x4e, 0x75, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50, 0x61,
0x70, 0x75, 0x74, 0x68, 0x61, 0x69, 0x63, 0x68, 0x65, 0xc0, 0x6d, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x64, 0x68, 0x42, 0x75,
0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x69, 0x64, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x69, 0x64, 0x68, 0xc0, 0x6d, 0x20, 0x50, 0x65, 0x61, 0x72, 0xf9,
0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x50, 0x65, 0x61, 0x72, 0xf9,
0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x50, 0x65, 0x61, 0x72, 0xf9, 0xc0, 0x6d, 0x20, 0x6e,
0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x61, 0x63, 0x68, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x65,
0x61, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x61,
0x63, 0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69, 0x6c,
0x65, 0x61, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65,
0x61, 0x63, 0x68, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e,
0x61, 0x6e, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0xc0, 0x6d,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x61, 0x67, 0x75, 0x73, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x61,
0x67, 0x75, 0x73, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x54, 0xec, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x61, 0x67, 0x75, 0x73, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xc0, 0x6d, 0x20, 0x50, 0x65, 0x69, 0x74,
0x20, 0x61, 0x2019, 0x20, 0x43, 0x68, 0xe0, 0x69, 0x72, 0x6e, 0xc0, 0x6d,
0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0xc0, 0x6d, 0x20, 0x50,
0x79, 0x65, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0xc0, 0x6d, 0x20,
0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x51, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0xc0, 0x6d, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0xc0,
0x6d, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0xc0, 0x6d, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x42, 0x75, 0x6e, 0x2d,
0xe0, 0x6d, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x54,
0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0xc0, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0xc0, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x74, 0x68, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x74, 0x68, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x74, 0x68, 0x61, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x6e, 0x20, 0x45, 0x69,
0x6c, 0x65, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x53, 0x68, 0x65, 0x69, 0x73,
0x65, 0x61, 0x6c, 0x6c, 0xc0, 0x6d, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x65,
0x61, 0x70, 0xf2, 0x72, 0xc0, 0x6d, 0x20, 0x45, 0x69, 0x6c, 0x65, 0x61,
0x6e, 0x61, 0x6e, 0x20, 0x53, 0x68, 0x6f, 0x6c, 0x61, 0x69, 0x6d, 0x68,
0xc0, 0x6d, 0x20, 0x53, 0x65, 0xf2, 0x72, 0x73, 0x65, 0x61, 0x20, 0x61,
0x20, 0x44, 0x65, 0x61, 0x73, 0xc0, 0x6d, 0x20, 0x53, 0x75, 0x72, 0x61,
0x6e, 0x61, 0x69, 0x6d, 0xc0, 0x6d, 0x20, 0x53, 0x68, 0x79, 0x6f, 0x77,
0x61, 0xc0, 0x6d, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0xc0, 0x6d,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0xc0, 0x6d, 0x20, 0x54, 0x61, 0x69, 0x64,
0x69, 0x67, 0x65, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0xc0, 0x6d, 0x20,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0xc0, 0x6d, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0xc0,
0x6d, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0xc0, 0x6d, 0x20, 0x54, 0x75,
0x72, 0x63, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x54, 0x75, 0x72, 0x63, 0x6d, 0x61,
0x6e, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x54, 0x75,
0x72, 0x63, 0x6d, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe0, 0x69, 0x6e, 0xc0,
0x6d, 0x20, 0x54, 0x75, 0x62, 0x68, 0x61, 0x6c, 0x75, 0xc0, 0x6d, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x64, 0x68, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x64,
0x68, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x64,
0x68, 0xc0, 0x6d, 0x20, 0x55, 0x73, 0x62, 0x61, 0x67, 0x61, 0x73, 0x74,
0xe0, 0x6e, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x55, 0x73, 0x62,
0x61, 0x67, 0x61, 0x73, 0x74, 0xe0, 0x6e, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x55, 0x73,
0x62, 0x61, 0x67, 0x61, 0x73, 0x74, 0xe0, 0x6e, 0xc0, 0x6d, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x54, 0xec, 0x64, 0x65,
0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0xc0, 0x6d, 0x20, 0x6e, 0x61, 0x20,
0x42, 0x68, 0x65, 0x69, 0x6e, 0x69, 0x73, 0x65, 0x61, 0x6c, 0x61, 0xc0,
0x6d, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x42, 0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x54, 0xec, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0xc0, 0x6d, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x42, 0x75, 0x6e,
0x2d, 0xe0, 0x6d, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x54, 0xec, 0x64, 0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0xc0, 0x6d, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0xc0, 0x6d,
0x20, 0x45, 0x69, 0x6c, 0x65, 0x61, 0x6e, 0x20, 0x57, 0x61, 0x6b, 0x65,
0xc0, 0x6d, 0x20, 0x55, 0x61, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x61, 0x67,
0x75, 0x73, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0xc0, 0x6d, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x42, 0x75, 0x6e, 0x2d, 0xe0,
0x6d, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x54, 0xec, 0x64,
0x65, 0x20, 0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xc0, 0x6d, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x42,
0x75, 0x6e, 0x2d, 0xe0, 0x6d, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x54, 0xec, 0x64, 0x65, 0x20,
0x73, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xc0,
0x6d, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x54, 0x65, 0x14b, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x6f, 0x6b, 0x61, 0x67, 0x62,
0x25b, 0x20, 0x42, 0x65, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x65, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x41, 0x6e, 0x61, 0x69, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x69, 0x67,
0x62, 0x25b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x41, 0x6e, 0x61, 0x69, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x20,
0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69,
0x61, 0x14b, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x42, 0x65, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20,
0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61,
0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x54, 0x65, 0x14b, 0x67, 0x62, 0x25b, 0x62,
0x69, 0x69, 0x20, 0x41, 0x62, 0x65, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x54, 0x65, 0x14b, 0x67, 0x62, 0x25b, 0x62, 0x69, 0x69, 0x20,
0x41, 0x62, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42,
0x65, 0x69, 0x61, 0x14b, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x54, 0x65, 0x14b, 0x67, 0x62, 0x25b, 0x62, 0x69, 0x69, 0x20, 0x41, 0x62,
0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42,
0x65, 0x69, 0x61, 0x14b, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x42, 0x6f, 0x6b, 0xe3, 0x67, 0x62, 0x25b, 0x62, 0x69, 0x69, 0x20, 0x41,
0x62, 0x65, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x6f,
0x6b, 0xe3, 0x67, 0x62, 0x25b, 0x62, 0x69, 0x69, 0x20, 0x41, 0x62, 0x65,
0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61,
0x14b, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x6f, 0x6b,
0xe3, 0x67, 0x62, 0x25b, 0x62, 0x69, 0x69, 0x20, 0x41, 0x62, 0x65, 0x20,
0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69,
0x61, 0x14b, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x47, 0x254,
0x6a, 0x69, 0x61, 0x6e, 0x254, 0x62, 0x69, 0x69, 0x20, 0x41, 0x62, 0x65,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x47, 0x254, 0x6a, 0x69,
0x61, 0x6e, 0x254, 0x62, 0x69, 0x69, 0x20, 0x41, 0x62, 0x65, 0x20, 0x59,
0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x47, 0x254, 0x6a, 0x69, 0x61,
0x6e, 0x254, 0x62, 0x69, 0x69, 0x20, 0x41, 0x62, 0x65, 0x20, 0x59, 0x25b,
0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b,
0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x50, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20,
0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c,
0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20,
0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20,
0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x42, 0x65, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x4b, 0x61, 0x70, 0x65, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x20, 0x42, 0x65, 0x4b, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20,
0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x4b, 0x61, 0x70,
0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x42, 0x65, 0x20, 0x59,
0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61,
0x14b, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x42, 0x65, 0x4b, 0x75, 0x62, 0x61,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42,
0x65, 0x69, 0x61, 0x14b, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x42, 0x65, 0x20,
0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69,
0x61, 0x14b, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x20,
0x4b, 0x25b, 0x20, 0x57, 0x75, 0x6f, 0x79, 0x69, 0x67, 0x62, 0x25b, 0x62,
0x69, 0x69, 0x20, 0x4e, 0x69, 0x20, 0x57, 0x69, 0x65, 0x254, 0x20, 0x46,
0x72, 0x25b, 0x6e, 0x74, 0x73, 0x69, 0x20, 0x42, 0x65, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x6f, 0x6b, 0xe3, 0x67,
0x62, 0x25b, 0x20, 0x42, 0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x42, 0x6f, 0x6b, 0xe3, 0x67, 0x62, 0x25b, 0x20, 0x42,
0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69,
0x61, 0x14b, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x42, 0x6f, 0x6b, 0xe3, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x20, 0x59,
0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61,
0x14b, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41,
0x6e, 0x61, 0x69, 0x67, 0x62, 0x25b, 0x20, 0x42, 0x65, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x6e, 0x61, 0x69, 0x67,
0x62, 0x25b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x41, 0x6e, 0x61, 0x69, 0x67, 0x62, 0x25b, 0x20,
0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20,
0x42, 0x65, 0x69, 0x61, 0x14b, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20, 0x42, 0x65, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42,
0x65, 0x69, 0x61, 0x14b, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b,
0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x20, 0x14a, 0x73, 0x68, 0x254, 0x20,
0x4c, 0x25b, 0x20, 0x42, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x20, 0x42, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61,
0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x4d, 0x65, 0x7a,
0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20,
0x42, 0x65, 0x4d, 0x65, 0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46,
0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x4d, 0x65, 0x7a, 0x69,
0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x42,
0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20, 0x42,
0x65, 0x69, 0x61, 0x14b, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x65, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42, 0x65, 0x20, 0x59,
0x25b, 0x20, 0x46, 0x25b, 0x69, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61, 0x74, 0x73, 0x61, 0x20,
0x42, 0x65, 0x69, 0x61, 0x14b, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x4b, 0x25b, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x6f, 0x6e, 0x20, 0x42, 0x65, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x4b, 0x25b, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x6f, 0x6e, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x46, 0x25b, 0x69,
0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x4b, 0x25b, 0x20, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x20, 0x42, 0x65, 0x20, 0x59, 0x25b, 0x20, 0x4c, 0x61,
0x74, 0x73, 0x61, 0x20, 0x42, 0x65, 0x69, 0x61, 0x14b, 0x52, 0xe9, 0x75,
0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x42, 0x65, 0x53, 0x65, 0x79, 0x73, 0x68,
0x65, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x42, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x4d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x6f, 0x6e, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d,
0xe9, 0x72, 0x69, 0x63, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0xe9,
0x72, 0x69, 0x63, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x2c, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x61, 0x6d, 0xe9, 0x72,
0x69, 0x63, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6c,
0x65, 0x73, 0x74, 0x65, 0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x6f, 0x20, 0x6c, 0x65, 0x73, 0x74, 0x65, 0x2c, 0x20, 0x41, 0x6d,
0xe9, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72,
0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20, 0x6c, 0x65, 0x73, 0x74, 0x65,
0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x2c, 0x20, 0x41,
0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74,
0x61, 0xf1, 0x61, 0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x2c, 0x20, 0x41,
0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x2c, 0x20, 0x41, 0x6d, 0xe9,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x6f, 0x2c, 0x20, 0x41, 0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f,
0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x2c, 0x20, 0x41,
0x6d, 0xe9, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x48, 0x6f, 0x72, 0x61,
0x72, 0x69, 0x6f, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x48, 0x6f,
0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0xe1,
0x72, 0x61, 0x62, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0xe1, 0x72, 0x61, 0x62, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0xe1, 0x72, 0x61, 0x62, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x41,
0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61, 0x20,
0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x61,
0x20, 0x41, 0x72, 0x78, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe1,
0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x74,
0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x63, 0x65, 0x72,
0x62, 0x61, 0x69, 0x78, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41,
0x63, 0x65, 0x72, 0x62, 0x61, 0x69, 0x78, 0xe1, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x65, 0x20, 0x41, 0x63, 0x65, 0x72, 0x62, 0x61, 0x69, 0x78, 0xe1, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42,
0x75, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c,
0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61,
0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x63, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x43, 0x68,
0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x63, 0x6f, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x6d, 0x65,
0x64, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43,
0x75, 0x62, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x61, 0x76, 0x69,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61, 0x20, 0x49,
0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x65, 0x78,
0x74, 0x72, 0x65, 0x6d, 0x6f, 0x20, 0x6c, 0x65, 0x73, 0x74, 0x65, 0x20,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69,
0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x46, 0x69, 0x78, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x46, 0x69, 0x78, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x78,
0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0xfc, 0x69,
0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x54, 0x65, 0x72,
0x72, 0x61, 0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x69, 0x73,
0x20, 0x65, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69, 0x63, 0x61,
0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x47, 0x61, 0x6c, 0xe1,
0x70, 0x61, 0x67, 0x6f, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61,
0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d,
0x69, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x2d, 0x69, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x69, 0x6c, 0x6c, 0x61, 0x73, 0x20,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4f, 0x63,
0xe9, 0x61, 0x6e, 0x6f, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63,
0x68, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x4f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0xe1, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x49, 0x72, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x58, 0x61, 0x70, 0xf3, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x6f, 0x20, 0x58, 0x61, 0x70, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20,
0x58, 0x61, 0x70, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0xe1, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0xe1, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0xe1, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x6f, 0x72, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f,
0x72, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x6f, 0x73,
0x72, 0x61, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72,
0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x75, 0x69, 0x7a, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49,
0x6c, 0x6c, 0x61, 0x73, 0x20, 0x64, 0x61, 0x20, 0x4c, 0x69, 0xf1, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x61,
0x69, 0x73, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65,
0x73, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20,
0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x6f, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f,
0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d,
0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x79, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54,
0x65, 0x72, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x69, 0x75, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49,
0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x71,
0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0xfa, 0x61, 0x2d, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65,
0x72, 0xfa, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69,
0x78, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x74,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x65, 0x74, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x74, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70,
0x65, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x52, 0x65,
0x75, 0x6e, 0x69, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0xed, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0xed,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0xed, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x72,
0x69, 0x6f, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x73, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c,
0x6c, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x73, 0x20, 0x53, 0x61,
0x6c, 0x6f, 0x6d, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x58, 0x65, 0x6f, 0x72, 0x78, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x53, 0x75, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0xed, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x78, 0x69, 0x71, 0x75,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe1,
0x6e, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x20, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62,
0x75, 0x72, 0x67, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x10d0,
0x10d5, 0x10e6, 0x10d0, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d0,
0x10e4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6,
0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e4, 0x10e0,
0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10db, 0x10ee,
0x10e0, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20,
0x10d0, 0x10e4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3,
0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e4, 0x10e0, 0x10d8,
0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5,
0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d0, 0x10da, 0x10d0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0,
0x10da, 0x10d0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10da,
0x10d0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10db, 0x10d0, 0x10d6, 0x10dd, 0x10dc,
0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10db, 0x10d0, 0x10d6, 0x10dd,
0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10db, 0x10d0, 0x10d6,
0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd,
0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0,
0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ea, 0x10d4, 0x10dc, 0x10e2,
0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0,
0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8,
0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3,
0x10e0, 0x10d8, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20,
0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d0, 0x10e6, 0x10db,
0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db,
0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1,
0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9,
0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0,
0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5,
0x10da, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd,
0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10db, 0x10d0, 0x10e3, 0x10dc, 0x10d7, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db,
0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10db, 0x10d0, 0x10e3, 0x10dc, 0x10d7,
0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e0, 0x10d3, 0x10d8,
0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4, 0x10e0, 0x10d8, 0x10d9, 0x10d8,
0x10e1, 0x20, 0x10db, 0x10d0, 0x10e3, 0x10dc, 0x10d7, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4,
0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0, 0x10d8,
0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4,
0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0, 0x10d8,
0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e9, 0x10e0, 0x10d3, 0x10d8, 0x10da, 0x10dd, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10db, 0x10d4,
0x10e0, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0, 0x10d8,
0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10de, 0x10d8,
0x10d0, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10de, 0x10d8, 0x10d0, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10de, 0x10d8, 0x10d0, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0, 0x10d0,
0x10d1, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0, 0x10d0,
0x10d1, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0, 0x10d0,
0x10d1, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0, 0x10d2, 0x10d4, 0x10dc, 0x10e2,
0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0, 0x10d2, 0x10d4,
0x10dc, 0x10e2, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e0,
0x10d2, 0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1,
0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e0, 0x10d2, 0x10d4, 0x10dc, 0x10e2,
0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0,
0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e0, 0x10d2, 0x10d4, 0x10dc, 0x10e2, 0x10d8,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5,
0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10e0, 0x10d2, 0x10d4, 0x10dc, 0x10e2, 0x10d8, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10dd, 0x10db, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10dd, 0x10db, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10dd, 0x10db, 0x10ee, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d0, 0x10e2, 0x10da, 0x10d0, 0x10dc, 0x10e2, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10dd,
0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e2,
0x10da, 0x10d0, 0x10dc, 0x10e2, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10dd, 0x10d9, 0x10d4,
0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e2, 0x10da, 0x10d0,
0x10dc, 0x10e2, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0,
0x10d8, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0,
0x10d8, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da,
0x10d8, 0x10d8, 0x10e1, 0x20, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3,
0x10e0, 0x10d8, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0,
0x10d8, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4,
0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3,
0x10e0, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da,
0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5,
0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4,
0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5,
0x10da, 0x10d4, 0x10d7, 0x20, 0x10d0, 0x10d5, 0x10e1, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d6, 0x10d4, 0x10e0, 0x10d1, 0x10d0, 0x10d8, 0x10ef, 0x10d0,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d6, 0x10d4, 0x10e0, 0x10d1,
0x10d0, 0x10d8, 0x10ef, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0,
0x10d6, 0x10d4, 0x10e0, 0x10d1, 0x10d0, 0x10d8, 0x10ef, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d0, 0x10d6, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3,
0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d6, 0x10dd,
0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10d6, 0x10dd, 0x10e0, 0x10d8, 0x10e1,
0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d1, 0x10d0, 0x10dc, 0x10d2, 0x10da, 0x10d0, 0x10d3, 0x10d4, 0x10e8, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10d0, 0x10dc, 0x10d2, 0x10da, 0x10d0, 0x10d3, 0x10d4, 0x10e8,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10d0, 0x10dc, 0x10d2, 0x10da, 0x10d0,
0x10d3, 0x10d4, 0x10e8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e3, 0x10e2, 0x10d0, 0x10dc, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10dd, 0x10da, 0x10d8, 0x10d5, 0x10d8, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e0, 0x10d0, 0x10d6, 0x10d8, 0x10da, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e0, 0x10d0, 0x10d6, 0x10d8, 0x10da,
0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e0, 0x10d0, 0x10d6, 0x10d8,
0x10da, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d1, 0x10e0, 0x10e3, 0x10dc, 0x10d4, 0x10d8, 0x2d,
0x10d3, 0x10d0, 0x10e0, 0x10e3, 0x10e1, 0x10d0, 0x10da, 0x10d0, 0x10db, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10d0, 0x10d1, 0x10dd, 0x2d, 0x10d5, 0x10d4, 0x10e0, 0x10d3,
0x10d4, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10d0, 0x10d1, 0x10dd, 0x2d, 0x10d5,
0x10d4, 0x10e0, 0x10d3, 0x10d4, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10d0, 0x10d1,
0x10dd, 0x2d, 0x10d5, 0x10d4, 0x10e0, 0x10d3, 0x10d4, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d0, 0x10db,
0x10dd, 0x10e0, 0x10dd, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d0, 0x10e2, 0x10d4,
0x10db, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d0, 0x10e2, 0x10d4, 0x10db,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d0, 0x10e2, 0x10d4, 0x10db, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e9, 0x10d8, 0x10da, 0x10d4, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9,
0x10d8, 0x10da, 0x10d4, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10d8, 0x10da, 0x10d4,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e9, 0x10d8, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e9, 0x10d8, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e9, 0x10d8, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10d0, 0x10e0, 0x10e2, 0x10e7, 0x10d4, 0x10da, 0x10d8, 0x10e9, 0x10dd,
0x10d8, 0x10d1, 0x10d0, 0x10da, 0x10e1, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e9, 0x10dd, 0x10d8, 0x10d1, 0x10d0, 0x10da, 0x10e1, 0x10d0, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10dd, 0x10d8, 0x10d1, 0x10d0, 0x10da, 0x10e1, 0x10d0,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e8, 0x10dd, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3,
0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e5, 0x10dd,
0x10e5, 0x10dd, 0x10e1, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da,
0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10da, 0x10e3,
0x10db, 0x10d1, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10da,
0x10e3, 0x10db, 0x10d1, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd,
0x10da, 0x10e3, 0x10db, 0x10d1, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e3, 0x10d9, 0x10d8,
0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e3, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3,
0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d9, 0x10e3, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da,
0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10dc, 0x10d0, 0x10ee, 0x10d4, 0x10d5, 0x10e0, 0x10d0,
0x10d3, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d9, 0x10e3, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9,
0x10e3, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e3, 0x10d1, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d3, 0x10d4, 0x10d5, 0x10d8, 0x10e1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d3, 0x10e3, 0x10db, 0x10dd, 0x10dc, 0x2d, 0x10d3, 0x10e3, 0x10e0, 0x10d5, 0x10d8,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1,
0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10e2, 0x10d8, 0x10db, 0x10dd, 0x10e0, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10d3, 0x10d2, 0x10dd, 0x10db, 0x10d8,
0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10d3, 0x10d2, 0x10dd, 0x10db, 0x10d8, 0x10e1, 0x20, 0x10d9,
0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0,
0x10e6, 0x10d3, 0x10d2, 0x10dd, 0x10db, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d4, 0x10d9, 0x10d5, 0x10d0, 0x10d3, 0x10dd, 0x10e0,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0,
0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3,
0x10e0, 0x10d8, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2, 0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8,
0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db,
0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd,
0x10de, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1,
0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5,
0x10da, 0x10d4, 0x10d7, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e8, 0x10dd, 0x10e0, 0x10d4, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d0, 0x10e6, 0x10db, 0x10dd,
0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da,
0x10d4, 0x10d7, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d4,
0x10d5, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0,
0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d4, 0x10d5, 0x10e0, 0x10dd, 0x10de,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10dd, 0x10da, 0x10d9, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8,
0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10dd, 0x10da, 0x10d9, 0x10da, 0x10d4, 0x10dc, 0x10d3,
0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10dd, 0x10da, 0x10d9, 0x10da, 0x10d4, 0x10dc,
0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10ef, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e4, 0x10d8, 0x10ef, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10ef,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10e4, 0x10e0, 0x10d0, 0x10dc, 0x10d2, 0x10d4, 0x10d7,
0x10d8, 0x10e1, 0x20, 0x10d2, 0x10d5, 0x10d8, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e4, 0x10e0, 0x10d0, 0x10dc, 0x10d2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10e1,
0x10d0, 0x10db, 0x10ee, 0x10e0, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10d0, 0x20,
0x10d0, 0x10dc, 0x10e2, 0x10d0, 0x10e0, 0x10e5, 0x10e2, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d2, 0x10d0, 0x10da, 0x10d0, 0x10de, 0x10d0, 0x10d2, 0x10dd, 0x10e1,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d2, 0x10d0, 0x10db, 0x10d1, 0x10d8, 0x10d4,
0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10e5, 0x10d0, 0x10e0,
0x10d7, 0x10d5, 0x10d4, 0x10da, 0x10dd, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0,
0x10e5, 0x10d0, 0x10e0, 0x10d7, 0x10d5, 0x10d4, 0x10da, 0x10dd, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e1, 0x10d0, 0x10e5, 0x10d0, 0x10e0, 0x10d7, 0x10d5, 0x10d4, 0x10da, 0x10dd, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d2, 0x10d8, 0x10da, 0x10d1, 0x10d4, 0x10e0, 0x10e2, 0x10d8, 0x10e1, 0x20, 0x10d9,
0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d2, 0x10e0, 0x10d8, 0x10dc, 0x10d5, 0x10d8, 0x10e9, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10d0, 0x10e8, 0x10e3, 0x10d0, 0x10da, 0x10dd, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6,
0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0, 0x10d4,
0x10dc, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d2,
0x10e0, 0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7,
0x20, 0x10d2, 0x10e0, 0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0,
0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0,
0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10e0,
0x10d4, 0x10dc, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10de,
0x10d0, 0x10e0, 0x10e1, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e7, 0x10e3, 0x10e0, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d2, 0x10d0, 0x10d8, 0x10d0, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10d0, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x10d0, 0x20,
0x10d3, 0x10d0, 0x20, 0x10d0, 0x10da, 0x10d4, 0x10e3, 0x10e2, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10f0, 0x10d0, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x10d0, 0x20, 0x10d3, 0x10d0,
0x20, 0x10d0, 0x10da, 0x10d4, 0x10e3, 0x10e2, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10f0, 0x10d0, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x10d0, 0x20, 0x10d3, 0x10d0, 0x20, 0x10d0,
0x10da, 0x10d4, 0x10e3, 0x10e2, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10dd, 0x10dc, 0x10d9, 0x10dd,
0x10dc, 0x10d2, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10dd, 0x10dc, 0x10d9,
0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10dd, 0x10dc,
0x10d9, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10dd, 0x10d5, 0x10d3, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10dd, 0x10d5, 0x10d3, 0x10d8, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10f0, 0x10dd, 0x10d5, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10dc,
0x10d3, 0x10dd, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10dc,
0x10d3, 0x10dd, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc,
0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10e9, 0x10d8, 0x10dc,
0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10ea, 0x10d4, 0x10dc, 0x10e2,
0x10e0, 0x10d0, 0x10da, 0x10e3, 0x10e0, 0x10d8, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10dc,
0x10d4, 0x10d6, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db,
0x10dd, 0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10dd,
0x10dc, 0x10d4, 0x10d6, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0,
0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10d8, 0x10dc, 0x10d3, 0x10dd, 0x10dc,
0x10d4, 0x10d6, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e0, 0x10d0,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e0, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10d0, 0x10e0, 0x10e2, 0x10e7, 0x10d4,
0x10da, 0x10d8, 0x10d8, 0x10e0, 0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e0, 0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e0, 0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d8, 0x10e1, 0x10e0, 0x10d0, 0x10d4, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d8, 0x10e1, 0x10e0, 0x10d0, 0x10d4, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d8, 0x10e1, 0x10e0, 0x10d0, 0x10d4, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8,
0x10d0, 0x10de, 0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8,
0x10d0, 0x10de, 0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8,
0x10d0, 0x10de, 0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10e6, 0x10db, 0x10dd,
0x10e1, 0x10d0, 0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10e7, 0x10d0, 0x10d6, 0x10d0, 0x10ee,
0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d3, 0x10d0, 0x10e1, 0x10d0,
0x10d5, 0x10da, 0x10d4, 0x10d7, 0x20, 0x10e7, 0x10d0, 0x10d6, 0x10d0, 0x10ee, 0x10d4, 0x10d7,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10e0, 0x10d4, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10e0, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10e0, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10dd, 0x10e1,
0x10e0, 0x10d4, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e0, 0x10d0, 0x10e1, 0x10dc,
0x10dd, 0x10d8, 0x10d0, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10d9, 0x10e0, 0x10d0, 0x10e1, 0x10dc, 0x10dd, 0x10d8, 0x10d0, 0x10e0, 0x10e1, 0x10d9, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d9, 0x10e0, 0x10d0, 0x10e1, 0x10dc, 0x10dd, 0x10d8,
0x10d0, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e7, 0x10d8, 0x10e0, 0x10d2, 0x10d8,
0x10d6, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10da, 0x10d0, 0x10d8,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10da, 0x10dd, 0x10e0, 0x10d3, 0x2d, 0x10f0,
0x10d0, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10da, 0x10dd, 0x10e0, 0x10d3, 0x2d,
0x10f0, 0x10d0, 0x10e3, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10da, 0x10dd, 0x10e0, 0x10d3,
0x2d, 0x10f0, 0x10d0, 0x10e3, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10d2, 0x10d0, 0x10d3, 0x10d0,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10d2, 0x10d0, 0x10d3,
0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10d2, 0x10d0,
0x10d3, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10da, 0x10d0, 0x10d8, 0x10d6,
0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10da, 0x10d3, 0x10d8,
0x10d5, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10e0,
0x10d9, 0x10d8, 0x10d6, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da,
0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10e0, 0x10e8,
0x10d0, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4,
0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10d5, 0x10e0, 0x10d8,
0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10d5, 0x10e0, 0x10d8,
0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d0, 0x10d5, 0x10e0, 0x10d8,
0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10dd, 0x10e3, 0x10e1, 0x10dd, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d4, 0x10e5, 0x10e1, 0x10d8, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0, 0x10d8, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d4, 0x10e5, 0x10e1, 0x10d8,
0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0, 0x10d8, 0x20, 0x10dd,
0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d4,
0x10e5, 0x10e1, 0x10d8, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10ec, 0x10e7, 0x10dc, 0x10d0, 0x10e0,
0x10d8, 0x20, 0x10dd, 0x10d9, 0x10d4, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0,
0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10da,
0x10d0, 0x10dc, 0x2d, 0x10d1, 0x10d0, 0x10e2, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e3, 0x10da, 0x10d0, 0x10dc, 0x2d, 0x10d1, 0x10d0, 0x10e2, 0x10dd, 0x10e0,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10da, 0x10d0, 0x10dc, 0x2d, 0x10d1,
0x10d0, 0x10e2, 0x10dd, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10dd, 0x10e1, 0x10d9, 0x10dd,
0x10d5, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10dd, 0x10e1, 0x10d9, 0x10dd,
0x10d5, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10dd, 0x10e1, 0x10d9, 0x10dd,
0x10d5, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10db, 0x10d8, 0x10d0, 0x10dc, 0x10db, 0x10d0, 0x10e0, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10d0, 0x10e3, 0x10e0, 0x10e3, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10d4, 0x10de, 0x10d0, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20, 0x10d9, 0x10d0, 0x10da, 0x10d4,
0x10d3, 0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10ee,
0x10d0, 0x10da, 0x10d8, 0x20, 0x10d9, 0x10d0, 0x10da, 0x10d4, 0x10d3, 0x10dd, 0x10dc, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20,
0x10d9, 0x10d0, 0x10da, 0x10d4, 0x10d3, 0x10dd, 0x10dc, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0,
0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20, 0x10d6, 0x10d4, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20,
0x10d6, 0x10d4, 0x10da, 0x10d0, 0x10dc, 0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20, 0x10d6, 0x10d4, 0x10da, 0x10d0, 0x10dc,
0x10d3, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10e3, 0x10e4, 0x10d0, 0x10e3, 0x10dc,
0x10d3, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc,
0x10d8, 0x10e3, 0x10e4, 0x10d0, 0x10e3, 0x10dc, 0x10d3, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10e3, 0x10e4, 0x10d0, 0x10e3, 0x10dc,
0x10d3, 0x10da, 0x10d4, 0x10dc, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10d8, 0x10e3, 0x10d4,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10dd, 0x10e0, 0x10e4, 0x10dd, 0x10da, 0x10d9,
0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10dd, 0x10e0, 0x10e4, 0x10dd, 0x10da, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10dc, 0x10dd, 0x10e0, 0x10e4, 0x10dd, 0x10da, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9,
0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d4, 0x10e0, 0x10dc,
0x10d0, 0x10dc, 0x10d3, 0x10dd, 0x2d, 0x10d3, 0x10d4, 0x2d, 0x10dc, 0x10dd, 0x10e0, 0x10dd,
0x10dc, 0x10d8, 0x10d0, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d4, 0x10e0, 0x10dc,
0x10d0, 0x10dc, 0x10d3, 0x10dd, 0x2d, 0x10d3, 0x10d4, 0x2d, 0x10dc, 0x10dd, 0x10e0, 0x10dd,
0x10dc, 0x10d8, 0x10d0, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d4, 0x10e0, 0x10dc,
0x10d0, 0x10dc, 0x10d3, 0x10dd, 0x2d, 0x10d3, 0x10d4, 0x2d, 0x10dc, 0x10dd, 0x10e0, 0x10dd,
0x10dc, 0x10d8, 0x10d0, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10dd, 0x10d5, 0x10dd, 0x10e1, 0x10d8, 0x10d1,
0x10d8, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10dd,
0x10d5, 0x10dd, 0x10e1, 0x10d8, 0x10d1, 0x10d8, 0x10e0, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10dc, 0x10dd, 0x10d5, 0x10dd, 0x10e1, 0x10d8, 0x10d1, 0x10d8, 0x10e0,
0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10dd, 0x10db, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10dd, 0x10db, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10dd, 0x10db, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10d9, 0x10d8,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0,
0x10d9, 0x10d8, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0,
0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10de, 0x10d0, 0x10d9, 0x10d8, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6,
0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de,
0x10d0, 0x10da, 0x10d0, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10de,
0x10e3, 0x10d0, 0x2d, 0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x20, 0x10d2, 0x10d5, 0x10d8,
0x10dc, 0x10d4, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10e0, 0x10d0,
0x10d2, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0, 0x10e0,
0x10d0, 0x10d2, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3,
0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d0,
0x10e0, 0x10d0, 0x10d2, 0x10d5, 0x10d0, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d4, 0x10e0, 0x10e3,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d4, 0x10e0, 0x10e3, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10de, 0x10d4, 0x10e0, 0x10e3, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10da, 0x10d8,
0x10de, 0x10d8, 0x10dc, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4,
0x10d8, 0x10da, 0x10d8, 0x10de, 0x10d8, 0x10dc, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10e1,
0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e4, 0x10d8, 0x10da, 0x10d8, 0x10de, 0x10d8, 0x10dc, 0x10d4, 0x10d1, 0x10d8,
0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10e4, 0x10d4, 0x10dc, 0x10d8, 0x10e5, 0x10e1, 0x10d8, 0x10e1, 0x20, 0x10d9,
0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10de, 0x10d8, 0x10d4, 0x10e0, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10d0, 0x20, 0x10db, 0x10d8, 0x10d9, 0x10d4, 0x10da, 0x10dd, 0x10dc, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10de, 0x10d8, 0x10d4, 0x10e0,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10d0, 0x20, 0x10db, 0x10d8, 0x10d9, 0x10d4, 0x10da, 0x10dd,
0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d4, 0x10dc, 0x2d, 0x10de,
0x10d8, 0x10d4, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10d0, 0x20, 0x10db, 0x10d8, 0x10d9,
0x10d4, 0x10da, 0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3,
0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10d8, 0x10e2, 0x10d9, 0x10d4,
0x10e0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10de, 0x10dd, 0x10dc, 0x10d0,
0x10de, 0x10d4, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e4, 0x10ee, 0x10d4, 0x10dc, 0x10d8,
0x10d0, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e0, 0x10d4, 0x10d8, 0x10e3,
0x10dc, 0x10d8, 0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e0, 0x10dd,
0x10d7, 0x10d4, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10ee,
0x10d0, 0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0,
0x10ee, 0x10d0, 0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc,
0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1,
0x10d0, 0x10ee, 0x10d0, 0x10da, 0x10d8, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10db,
0x10dd, 0x10d0, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10db, 0x10dd, 0x10d0,
0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da,
0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d0, 0x10db, 0x10dd, 0x10d0, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e1, 0x10d4, 0x10d8, 0x10e8, 0x10d4, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc,
0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1,
0x10d8, 0x10dc, 0x10d2, 0x10d0, 0x10de, 0x10e3, 0x10e0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e1, 0x10dd, 0x10da, 0x10dd, 0x10db, 0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20, 0x10d9,
0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d4, 0x10d1, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e1, 0x10d0, 0x10db, 0x10ee, 0x10e0, 0x10d4, 0x10d7, 0x20, 0x10d2, 0x10d4, 0x10dd,
0x10e0, 0x10d2, 0x10d8, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10e3, 0x10e0,
0x10d8, 0x10dc, 0x10d0, 0x10db, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e1, 0x10d8,
0x10dd, 0x10d5, 0x10d0, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10d0, 0x10d8, 0x10e2,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10d0, 0x10d8, 0x10d1, 0x10d4, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10d0, 0x10d8, 0x10d1, 0x10d4, 0x10d8, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10d0, 0x10d8, 0x10d1, 0x10d4, 0x10d8, 0x10e1, 0x20,
0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd,
0x10e2, 0x10d0, 0x10ef, 0x10d8, 0x10d9, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e2, 0x10dd, 0x10d9, 0x10d4, 0x10da, 0x10d0, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10e2, 0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2,
0x10dd, 0x10dc, 0x10d2, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10dd, 0x10dc,
0x10d2, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e9, 0x10e3, 0x10e3, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d7, 0x10e3, 0x10e0, 0x10e5, 0x10db, 0x10d4, 0x10dc, 0x10d4, 0x10d7, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d7, 0x10e3, 0x10e0, 0x10e5, 0x10db, 0x10d4, 0x10dc,
0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0,
0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d7, 0x10e3, 0x10e0, 0x10e5,
0x10db, 0x10d4, 0x10dc, 0x10d4, 0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee,
0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e2, 0x10e3, 0x10d5, 0x10d0,
0x10da, 0x10e3, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10e0, 0x10e3, 0x10d2, 0x10d5,
0x10d0, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10e0, 0x10e3, 0x10d2, 0x10d5,
0x10d0, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2,
0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10e0, 0x10e3, 0x10d2, 0x10d5,
0x10d0, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10d6, 0x10d1, 0x10d4, 0x10d9, 0x10d4, 0x10d7, 0x10d8,
0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10d6, 0x10d1, 0x10d4, 0x10d9, 0x10d4, 0x10d7,
0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3,
0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10e3, 0x10d6, 0x10d1, 0x10d4, 0x10d9, 0x10d4,
0x10d7, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10d0, 0x10dc, 0x10e3, 0x10d0, 0x10e2, 0x10e3, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10d0, 0x10dc, 0x10e3, 0x10d0, 0x10e2, 0x10e3, 0x10e1,
0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10d0, 0x10dc, 0x10e3, 0x10d0, 0x10e2, 0x10e3, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d5, 0x10d4, 0x10dc, 0x10d4, 0x10e1, 0x10e3, 0x10d4, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10da, 0x10d0, 0x10d3, 0x10d8, 0x10d5, 0x10dd, 0x10e1, 0x10e2,
0x10dd, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10da, 0x10d0, 0x10d3,
0x10d8, 0x10d5, 0x10dd, 0x10e1, 0x10e2, 0x10dd, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d5, 0x10da, 0x10d0, 0x10d3, 0x10d8, 0x10d5, 0x10dd, 0x10e1, 0x10e2, 0x10dd, 0x10d9,
0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10dd, 0x10da, 0x10d2, 0x10dd, 0x10d2, 0x10e0, 0x10d0, 0x10d3,
0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10dd, 0x10da, 0x10d2, 0x10dd, 0x10d2,
0x10e0, 0x10d0, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0,
0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10dd, 0x10da,
0x10d2, 0x10dd, 0x10d2, 0x10e0, 0x10d0, 0x10d3, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4,
0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10d4, 0x10d8,
0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d9, 0x10e3, 0x10dc, 0x10eb, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d5, 0x10dd, 0x10da, 0x10d8, 0x10e1, 0x10d8, 0x20, 0x10d3,
0x10d0, 0x20, 0x10e4, 0x10e3, 0x10e2, 0x10e3, 0x10dc, 0x10d0, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d8, 0x10d0, 0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20, 0x10d3,
0x10e0, 0x10dd, 0x10d8, 0x10d0, 0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1, 0x20,
0x10e1, 0x10e2, 0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10d0, 0x10d9, 0x10e3, 0x10e2, 0x10e1, 0x10d9, 0x10d8, 0x10e1,
0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d4, 0x10d9, 0x10d0, 0x10e2, 0x10d4, 0x10e0, 0x10d8, 0x10dc, 0x10d1, 0x10e3, 0x10e0,
0x10d2, 0x10d8, 0x10e1, 0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d4, 0x10d9, 0x10d0, 0x10e2, 0x10d4,
0x10e0, 0x10d8, 0x10dc, 0x10d1, 0x10e3, 0x10e0, 0x10d2, 0x10d8, 0x10e1, 0x20, 0x10e1, 0x10e2,
0x10d0, 0x10dc, 0x10d3, 0x10d0, 0x10e0, 0x10e2, 0x10e3, 0x10da, 0x10d8, 0x20, 0x10d3, 0x10e0,
0x10dd, 0x10d4, 0x10d9, 0x10d0, 0x10e2, 0x10d4, 0x10e0, 0x10d8, 0x10dc, 0x10d1, 0x10e3, 0x10e0,
0x10d2, 0x10d8, 0x10e1, 0x20, 0x10d6, 0x10d0, 0x10e4, 0x10ee, 0x10e3, 0x10da, 0x10d8, 0x10e1,
0x20, 0x10d3, 0x10e0, 0x10dd, 0x10d8, 0x10e3, 0x10d9, 0x10dd, 0x10dc, 0x10d8, 0x10e1, 0x20,
0x10d3, 0x10e0, 0x10dd, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x41, 0x63, 0x72, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x5a, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73,
0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x53, 0xfc, 0x64, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65,
0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57,
0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x61, 0x73, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x4f, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x7a, 0x65,
0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4f, 0x73, 0x74, 0x6b,
0xfc, 0x73, 0x74, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4f, 0x73,
0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x2d,
0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x2d, 0x4d, 0x6f, 0x75, 0x6e, 0x74,
0x61, 0x69, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x2d, 0x4d, 0x6f, 0x75, 0x6e,
0x74, 0x61, 0x69, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x57, 0x65, 0x73,
0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x7a, 0x65, 0x69, 0x74, 0x4e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x57, 0x65, 0x73, 0x74, 0x6b, 0xfc, 0x73,
0x74, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x57, 0x65, 0x73, 0x74,
0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41,
0x70, 0x69, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x41, 0x70, 0x69, 0x61, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x71, 0x74, 0x61, 0x75,
0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41,
0x71, 0x74, 0xf6, 0x62, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x71,
0x74, 0xf6, 0x62, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x2d, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x61, 0x62,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x72,
0x61, 0x62, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x65,
0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65,
0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x2f, 0x57,
0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x73, 0x65, 0x72,
0x62, 0x61, 0x69, 0x64, 0x73, 0x63, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x41, 0x73, 0x65, 0x72, 0x62,
0x65, 0x69, 0x64, 0x73, 0x63, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x73, 0x63, 0x68, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63,
0x68, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x63, 0x68, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x63, 0x68, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x42, 0x72, 0x61, 0x73, 0xed,
0x6c, 0x69, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x42, 0x72, 0x61, 0x73,
0xed, 0x6c, 0x69, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x2d,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x2d, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61,
0x6c, 0x61, 0x6d, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x61, 0x62, 0x6f,
0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x43,
0x61, 0x62, 0x6f, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x61, 0x62, 0x6f,
0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x61, 0x73, 0x65, 0x79, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e,
0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x73,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x54, 0x73, 0x63, 0x68, 0x6f, 0x69, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x54, 0x73, 0x63,
0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x54, 0x73, 0x63, 0x68,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x2d, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x69, 0x68, 0x6e,
0x61, 0x63, 0x68, 0x74, 0x73, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x69, 0x6e, 0x73, 0x65,
0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65,
0x69, 0x74, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65,
0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x6f, 0x6f, 0x6b, 0x69,
0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65,
0x6c, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x44, 0x61,
0x76, 0x69, 0x73, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f,
0x72, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x69, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4d,
0x69, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x69, 0x74,
0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x4f, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x4f, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61,
0x64, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x46,
0x69, 0x64, 0x73, 0x63, 0x68, 0x69, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x46, 0x72, 0x61, 0x6e, 0x7a, 0xf6, 0x73,
0x69, 0x73, 0x63, 0x68, 0x2d, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x61,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x46, 0x72, 0x61, 0x6e, 0x7a, 0xf6, 0x73,
0x69, 0x73, 0x63, 0x68, 0x65, 0x2d, 0x53, 0xfc, 0x64, 0x2d, 0x75, 0x6e,
0x64, 0x2d, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x67,
0x65, 0x62, 0x69, 0x65, 0x74, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x2d, 0x49, 0x6e, 0x73, 0x65, 0x6c, 0x6e,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x6c, 0x65, 0x72,
0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x67,
0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73,
0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74,
0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x47, 0x75, 0x61, 0x6d, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x47, 0x6f, 0x6c, 0x66, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x65, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x48, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x67, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x43, 0x68, 0x6f, 0x77, 0x64, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x43, 0x68,
0x6f, 0x77, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x2d, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x63, 0x68, 0x65, 0x72, 0x2d, 0x4f,
0x7a, 0x65, 0x61, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x49, 0x6e, 0x64,
0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x5a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4f,
0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x63, 0x68,
0x61, 0x74, 0x6b, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x61, 0x74,
0x6b, 0x61, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69,
0x74, 0x4f, 0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x63, 0x68, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74,
0x6b, 0x61, 0x73, 0x61, 0x63, 0x68, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x5a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e,
0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4b, 0x69, 0x72,
0x67, 0x69, 0x73, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x53, 0x72, 0x69, 0x2d, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x4c, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x69, 0x6e, 0x73,
0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64,
0x2d, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4c, 0x6f,
0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48,
0x6f, 0x77, 0x65, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x4d, 0x61, 0x63, 0x61, 0x75, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x2d, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x6c, 0x61,
0x79, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x76, 0x65, 0x6e, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b,
0x7a, 0x65, 0x69, 0x74, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4d,
0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75,
0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61,
0x75, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4d, 0x79, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x65, 0x70, 0x61, 0x6c,
0x65, 0x73, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x4e, 0x65, 0x75, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x65, 0x75, 0x6b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x65,
0x75, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x4e, 0x65, 0x75, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x4e, 0x65, 0x75, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74,
0x4e, 0x65, 0x75, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x65, 0x75,
0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x4e, 0x65, 0x75, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e,
0x65, 0x75, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x69, 0x75,
0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d,
0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x53,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x4e, 0xf6, 0x72,
0x64, 0x6c, 0x69, 0x63, 0x68, 0x65, 0x2d, 0x4d, 0x61, 0x72, 0x69, 0x61,
0x6e, 0x65, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4e, 0x6f, 0x77, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65,
0x69, 0x74, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x65, 0x72, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4f, 0x6d, 0x73, 0x6b, 0x65, 0x72,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x4e, 0x65, 0x75,
0x67, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x50, 0x65, 0x72,
0x75, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74,
0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x69, 0x73, 0x63,
0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65,
0x69, 0x74, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x69, 0x6e, 0x73,
0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x74, 0x2e, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x75, 0x6e, 0x64, 0x2d, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x53, 0x74, 0x2e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x75,
0x6e, 0x64, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x74,
0x2e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x75, 0x6e, 0x64,
0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x53, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x6e, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x50, 0x6a, 0xf6, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x51, 0x75, 0x79, 0x73, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x51, 0x75, 0x79, 0x73, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0x65, 0x69, 0x74, 0x51, 0x79, 0x73, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74,
0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x61, 0x63, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65,
0x69, 0x74, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65,
0x6c, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x75, 0x72, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x53, 0x61, 0x6c,
0x6f, 0x6d, 0x6f, 0x6e, 0x65, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53,
0xfc, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x63, 0x68, 0x65,
0x20, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x65, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x2d,
0x5a, 0x65, 0x69, 0x74, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x54, 0x61, 0x69, 0x70, 0x65, 0x68, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x54, 0x61, 0x69, 0x70, 0x65, 0x68, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x54, 0x61, 0x69, 0x70, 0x65,
0x68, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74,
0x54, 0x61, 0x64, 0x73, 0x63, 0x68, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x69,
0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0x65, 0x69, 0x74, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x65, 0x69, 0x74, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x69, 0x73,
0x63, 0x68, 0x65, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x69, 0x73, 0x63, 0x68, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x55, 0x73, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x2d, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0x65, 0x69, 0x74, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61,
0x2d, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69, 0x74, 0x57,
0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x72,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x57,
0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x72,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x57,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x72, 0x20, 0x5a,
0x65, 0x69, 0x74, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69,
0x74, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x65, 0x72,
0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x65, 0x69, 0x74, 0x57,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x57, 0x61,
0x6b, 0x65, 0x2d, 0x49, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a, 0x65, 0x69,
0x74, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x75, 0x6e, 0x64, 0x2d,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0x65, 0x69, 0x74, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x65, 0x72, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x65, 0x72, 0x20, 0x5a, 0x65, 0x69,
0x74, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0x65, 0x69, 0x74, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x65, 0x72, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x65, 0x69, 0x74, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x2d, 0x5a,
0x65, 0x69, 0x74, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x2d, 0x5a, 0x65,
0x69, 0x74, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x69, 0x6e, 0x73,
0x65, 0x6c, 0x6e, 0x2d, 0x5a, 0x65, 0x69, 0x74, 0x38f, 0x3c1, 0x3b1, 0x20,
0x391, 0x3c6, 0x3b3, 0x3b1, 0x3bd, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x391, 0x3c6, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391,
0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c6,
0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3cc, 0x3c4, 0x3b9, 0x3b1,
0x3c2, 0x20, 0x391, 0x3c6, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c6, 0x3c1,
0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x20, 0x391, 0x3c6, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c6, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x391, 0x3bb, 0x3ac, 0x3c3, 0x3ba, 0x3b1, 0x3c2, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x391, 0x3bb, 0x3ac, 0x3c3, 0x3ba, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bb, 0x3ac, 0x3c3, 0x3ba, 0x3b1,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bc, 0x3b1, 0x3b6, 0x3bf, 0x3bd, 0x3af,
0x3bf, 0x3c5, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bc, 0x3b1, 0x3b6, 0x3bf, 0x3bd, 0x3af, 0x3bf,
0x3c5, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x391, 0x3bc, 0x3b1, 0x3b6, 0x3bf, 0x3bd, 0x3af, 0x3bf, 0x3c5, 0x39a, 0x3b5, 0x3bd,
0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc,
0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3ba,
0x3ae, 0x3c2, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x20, 0x3c7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba,
0x3ae, 0x20, 0x3b8, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9,
0x3ba, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9,
0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x391,
0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x20, 0x3c7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392,
0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae,
0x20, 0x3b8, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x39f, 0x3c1, 0x3b5, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x39f, 0x3c1, 0x3b5, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3c7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20,
0x391, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x39f, 0x3c1, 0x3b5, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3b8, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x391, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9,
0x3c1, 0x3b7, 0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9, 0x3c1,
0x3b7, 0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9, 0x3c1, 0x3b7, 0x3bd, 0x3b9, 0x3ba,
0x3bf, 0x3cd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3bd, 0x3c4, 0x3af,
0x3c1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3bd, 0x3c4, 0x3af, 0x3c1, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1,
0x3bd, 0x3c4, 0x3af, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c0, 0x3af, 0x3b1,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x391, 0x3c0, 0x3af, 0x3b1, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c0, 0x3af, 0x3b1, 0x391, 0x3c1, 0x3b1,
0x3b2, 0x3b9, 0x3ba, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x391, 0x3c1, 0x3b1, 0x3b2,
0x3b9, 0x3ba, 0x3ae, 0x20, 0x3c7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x391, 0x3c1, 0x3b1, 0x3b2, 0x3b9, 0x3ba, 0x3ae,
0x20, 0x3b8, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x38f,
0x3c1, 0x3b1, 0x20, 0x391, 0x3c1, 0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae,
0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x391, 0x3c1, 0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x391, 0x3c1, 0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c1,
0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5,
0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c1, 0x3b3, 0x3b5, 0x3bd, 0x3c4,
0x3b9, 0x3bd, 0x3ae, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391,
0x3c1, 0x3b3, 0x3b5, 0x3bd, 0x3c4, 0x3b9, 0x3bd, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x391, 0x3c1, 0x3bc, 0x3b5, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391,
0x3c1, 0x3bc, 0x3b5, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c1, 0x3bc, 0x3b5, 0x3bd, 0x3af,
0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c4, 0x3bb, 0x3b1, 0x3bd, 0x3c4,
0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3c4, 0x3bb, 0x3b1, 0x3bd, 0x3c4,
0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x391, 0x3c4, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x3b9, 0x3ba, 0x3bf,
0x3cd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba,
0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1,
0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2,
0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5,
0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4,
0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5,
0x3bd, 0x3c4, 0x3c1, 0x3bf, 0x3b4, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3bf, 0x3b4, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a,
0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3bf, 0x3b4, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2,
0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2,
0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd,
0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3,
0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x394,
0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1,
0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba,
0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4, 0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x391, 0x3c5, 0x3c3, 0x3c4,
0x3c1, 0x3b1, 0x3bb, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3b6,
0x3b5, 0x3c1, 0x3bc, 0x3c0, 0x3b1, 0x3ca, 0x3c4, 0x3b6, 0x3ac, 0x3bd, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x391, 0x3b6, 0x3b5, 0x3c1, 0x3bc, 0x3c0, 0x3b1, 0x3ca, 0x3c4, 0x3b6, 0x3ac, 0x3bd,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391,
0x3b6, 0x3b5, 0x3c1, 0x3bc, 0x3c0, 0x3b1, 0x3ca, 0x3c4, 0x3b6, 0x3ac, 0x3bd, 0x38f,
0x3c1, 0x3b1, 0x20, 0x391, 0x3b6, 0x3bf, 0x3c1, 0x3ce, 0x3bd, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391,
0x3b6, 0x3bf, 0x3c1, 0x3ce, 0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3b6, 0x3bf, 0x3c1, 0x3ce, 0x3bd, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4,
0x3ad, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3bb, 0x3b1,
0x3bd, 0x3c4, 0x3ad, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3bd, 0x3b3, 0x3ba, 0x3bb, 0x3b1, 0x3bd,
0x3c4, 0x3ad, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3bf, 0x3c5, 0x3c4,
0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x392, 0x3bf, 0x3bb, 0x3b9, 0x3b2, 0x3af,
0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3c1, 0x3b1, 0x3b6, 0x3af,
0x3bb, 0x3b9, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3c1, 0x3b1, 0x3b6, 0x3af,
0x3bb, 0x3b9, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3c1, 0x3b1, 0x3b6, 0x3af, 0x3bb, 0x3b9, 0x3b1,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3c0, 0x3c1, 0x3bf, 0x3c5, 0x3bd, 0x3ad,
0x3b9, 0x20, 0x39d, 0x3c4, 0x3b1, 0x3c1, 0x3bf, 0x3c5, 0x3c3, 0x3b1, 0x3bb, 0x3ac,
0x3bc, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3c1, 0x3ac, 0x3c3, 0x3b9, 0x3bd, 0x3bf,
0x3c5, 0x20, 0x391, 0x3ba, 0x3c1, 0x3c9, 0x3c4, 0x3b7, 0x3c1, 0x3af, 0x3bf, 0x3c5,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a0, 0x3c1, 0x3ac, 0x3c3, 0x3b9, 0x3bd, 0x3bf, 0x3c5, 0x20, 0x391,
0x3ba, 0x3c1, 0x3c9, 0x3c4, 0x3b7, 0x3c1, 0x3af, 0x3bf, 0x3c5, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3c1, 0x3ac, 0x3c3,
0x3b9, 0x3bd, 0x3bf, 0x3c5, 0x20, 0x391, 0x3ba, 0x3c1, 0x3c9, 0x3c4, 0x3b7, 0x3c1,
0x3af, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3c3, 0x3b1, 0x3bc, 0x3cc,
0x3c1, 0x3bf, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3c3, 0x3ac, 0x3c4, 0x3b1, 0x3bc,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a4, 0x3c3, 0x3ac, 0x3c4, 0x3b1, 0x3bc, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3c3, 0x3ac, 0x3c4, 0x3b1,
0x3bc, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3b9, 0x3bb, 0x3ae, 0x3c2, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a7, 0x3b9, 0x3bb, 0x3ae, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3b9, 0x3bb, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39a, 0x3af, 0x3bd, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3af, 0x3bd, 0x3b1,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39a, 0x3af, 0x3bd, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3c3, 0x3bf,
0x3ca, 0x3bc, 0x3c0, 0x3b1, 0x3bb, 0x3c3, 0x3ac, 0x3bd, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3c3,
0x3bf, 0x3ca, 0x3bc, 0x3c0, 0x3b1, 0x3bb, 0x3c3, 0x3ac, 0x3bd, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3c3, 0x3bf, 0x3ca,
0x3bc, 0x3c0, 0x3b1, 0x3bb, 0x3c3, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3ae, 0x3c3, 0x3bf, 0x3c5, 0x20, 0x3a7, 0x3c1, 0x3b9, 0x3c3, 0x3c4, 0x3bf, 0x3c5,
0x3b3, 0x3ad, 0x3bd, 0x3bd, 0x3c9, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae,
0x3c3, 0x3c9, 0x3bd, 0x20, 0x39a, 0x3cc, 0x3ba, 0x3bf, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39a, 0x3bf, 0x3bb, 0x3bf, 0x3bc, 0x3b2, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39a, 0x3bf, 0x3bb, 0x3bf, 0x3bc, 0x3b2, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3bb, 0x3bf,
0x3bc, 0x3b2, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3,
0x3c9, 0x3bd, 0x20, 0x39a, 0x3bf, 0x3c5, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3,
0x3c9, 0x3bd, 0x20, 0x39a, 0x3bf, 0x3c5, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20,
0x39a, 0x3bf, 0x3c5, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3cd, 0x3b2,
0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3cd, 0x3b2, 0x3b1, 0x3c2, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3cd,
0x3b2, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3c4, 0x3ad, 0x3b9, 0x3b2,
0x3b9, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3c4, 0x3b9, 0x3bc, 0x3cc, 0x3bd,
0x20, 0x3bd, 0x3c4, 0x2019, 0x20, 0x39f, 0x3c5, 0x3c1, 0x3b2, 0x3af, 0x3bb, 0x38f,
0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3bf,
0x3cd, 0x20, 0x3a4, 0x3b9, 0x3bc, 0x3cc, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3ae, 0x3c3, 0x3bf, 0x3c5, 0x20, 0x3a0, 0x3ac, 0x3c3, 0x3c7, 0x3b1, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c5, 0x20, 0x3a0, 0x3ac, 0x3c3, 0x3c7, 0x3b1, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae,
0x3c3, 0x3bf, 0x3c5, 0x20, 0x3a0, 0x3ac, 0x3c3, 0x3c7, 0x3b1, 0x38f, 0x3c1, 0x3b1,
0x20, 0x399, 0x3c3, 0x3b7, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3bf, 0x3cd, 0x38f,
0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2,
0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5,
0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce,
0x3c0, 0x3b7, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391,
0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5,
0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf,
0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395,
0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3c0, 0x3b5,
0x3c1, 0x3b1, 0x3b9, 0x3c4, 0x3ad, 0x3c1, 0x3c9, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4,
0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0,
0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394,
0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1, 0x3ce, 0x3c0,
0x3b7, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x395, 0x3c5, 0x3c1,
0x3ce, 0x3c0, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9,
0x3bd, 0x20, 0x3a6, 0x3cc, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x3a6, 0x3cc, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x3a6, 0x3cc, 0x3ba, 0x3bb, 0x3b1, 0x3bd, 0x3c4,
0x38f, 0x3c1, 0x3b1, 0x20, 0x3a6, 0x3af, 0x3c4, 0x3b6, 0x3b9, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a6,
0x3af, 0x3c4, 0x3b6, 0x3b9, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x3a6, 0x3af, 0x3c4, 0x3b6, 0x3b9, 0x38f, 0x3c1, 0x3b1, 0x20,
0x393, 0x3b1, 0x3bb, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x393, 0x3bf, 0x3c5,
0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b1, 0x3bb,
0x3bb, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x20, 0x39d, 0x3cc, 0x3c4, 0x3bf, 0x3c5, 0x20,
0x3ba, 0x3b1, 0x3b9, 0x20, 0x391, 0x3bd, 0x3c4, 0x3b1, 0x3c1, 0x3ba, 0x3c4, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3ba, 0x3b1, 0x3bb, 0x3ac,
0x3c0, 0x3b1, 0x3b3, 0x3ba, 0x3bf, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3ba,
0x3ac, 0x3bc, 0x3c0, 0x3b9, 0x3b5, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b5,
0x3c9, 0x3c1, 0x3b3, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b5, 0x3c9, 0x3c1,
0x3b3, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x393, 0x3b5, 0x3c9, 0x3c1, 0x3b3, 0x3af, 0x3b1, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x393, 0x3ba, 0x3af,
0x3bb, 0x3bc, 0x3c0, 0x3b5, 0x3c1, 0x3c4, 0x39c, 0x3ad, 0x3c3, 0x3b7, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x393, 0x3ba, 0x3c1, 0x3af, 0x3bd, 0x3bf, 0x3c5, 0x3b9, 0x3c4,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x20, 0x393, 0x3c1, 0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4,
0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x20, 0x393, 0x3c1, 0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4,
0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3ae, 0x3c2,
0x20, 0x393, 0x3c1, 0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20,
0x393, 0x3c1, 0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae, 0x3c2, 0x20, 0x393, 0x3c1, 0x3bf,
0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba,
0x3ae, 0x3c2, 0x20, 0x393, 0x3c1, 0x3bf, 0x3b9, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af,
0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3ba, 0x3bf, 0x3c5, 0x3ac, 0x3bc,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3cc, 0x3bb, 0x3c0, 0x3bf, 0x3c5, 0x38f, 0x3c1,
0x3b1, 0x20, 0x393, 0x3bf, 0x3c5, 0x3b9, 0x3ac, 0x3bd, 0x3b1, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x3a7, 0x3b1, 0x3b2, 0x3ac, 0x3b7, 0x3c2, 0x2d, 0x391, 0x3bb, 0x3b5,
0x3bf, 0x3cd, 0x3c4, 0x3b9, 0x3c9, 0x3bd, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a7, 0x3b1, 0x3b2, 0x3ac, 0x3b7, 0x3c2, 0x2d, 0x391, 0x3bb, 0x3b5,
0x3bf, 0x3cd, 0x3c4, 0x3b9, 0x3c9, 0x3bd, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd,
0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7,
0x3b1, 0x3b2, 0x3ac, 0x3b7, 0x3c2, 0x2d, 0x391, 0x3bb, 0x3b5, 0x3bf, 0x3cd, 0x3c4,
0x3b9, 0x3c9, 0x3bd, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x38f, 0x3c1, 0x3b1,
0x20, 0x3a7, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x20, 0x39a, 0x3bf, 0x3bd, 0x3b3, 0x3ba,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a7, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x20, 0x39a, 0x3bf, 0x3bd, 0x3b3,
0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a7, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x20, 0x39a, 0x3bf, 0x3bd, 0x3b3, 0x3ba, 0x38f,
0x3c1, 0x3b1, 0x20, 0x3a7, 0x3bf, 0x3b2, 0x3bd, 0x3c4, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a7, 0x3bf,
0x3b2, 0x3bd, 0x3c4, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a7, 0x3bf, 0x3b2, 0x3bd, 0x3c4, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399,
0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399, 0x3bd, 0x3b4,
0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x20, 0x3a9, 0x3ba, 0x3b5, 0x3b1, 0x3bd, 0x3bf, 0x3cd,
0x38f, 0x3c1, 0x3b1, 0x20, 0x399, 0x3bd, 0x3b4, 0x3bf, 0x3ba, 0x3af, 0x3bd, 0x3b1,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b5, 0x3bd, 0x3c4, 0x3c1, 0x3b9, 0x3ba,
0x3ae, 0x3c2, 0x20, 0x399, 0x3bd, 0x3b4, 0x3bf, 0x3bd, 0x3b7, 0x3c3, 0x3af, 0x3b1,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4, 0x3bf, 0x3bb, 0x3b9,
0x3ba, 0x3ae, 0x3c2, 0x20, 0x399, 0x3bd, 0x3b4, 0x3bf, 0x3bd, 0x3b7, 0x3c3, 0x3af,
0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9, 0x3ba, 0x3ae,
0x3c2, 0x20, 0x399, 0x3bd, 0x3b4, 0x3bf, 0x3bd, 0x3b7, 0x3c3, 0x3af, 0x3b1, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c1, 0x3ac, 0x3bd, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c1,
0x3ac, 0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x399, 0x3c1, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c1, 0x3ba,
0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c1, 0x3ba, 0x3bf, 0x3cd,
0x3c4, 0x3c3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x399, 0x3c1, 0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x38f, 0x3c1,
0x3b1, 0x20, 0x399, 0x3c3, 0x3c1, 0x3b1, 0x3ae, 0x3bb, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3c3,
0x3c1, 0x3b1, 0x3ae, 0x3bb, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x399, 0x3c3, 0x3c1, 0x3b1, 0x3ae, 0x3bb, 0x38f, 0x3c1, 0x3b1,
0x20, 0x399, 0x3b1, 0x3c0, 0x3c9, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399,
0x3b1, 0x3c0, 0x3c9, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x399, 0x3b1, 0x3c0, 0x3c9, 0x3bd, 0x3af,
0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b1, 0x3bc, 0x3c4, 0x3c3, 0x3ac,
0x3c4, 0x3ba, 0x3b1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b5, 0x3c4, 0x3c1, 0x3bf, 0x3c0, 0x3b1,
0x3b2, 0x3bb, 0x3cc, 0x3b2, 0x3c3, 0x3ba, 0x2d, 0x39a, 0x3b1, 0x3bc, 0x3c4, 0x3c3,
0x3ac, 0x3c4, 0x3c3, 0x3ba, 0x3b9, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b5, 0x3c4, 0x3c1, 0x3bf, 0x3c0, 0x3b1, 0x3b2,
0x3bb, 0x3cc, 0x3b2, 0x3c3, 0x3ba, 0x2d, 0x39a, 0x3b1, 0x3bc, 0x3c4, 0x3c3, 0x3ac,
0x3c4, 0x3c3, 0x3ba, 0x3b9, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3bd, 0x3b1, 0x3c4,
0x3bf, 0x3bb, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x20, 0x39a, 0x3b1, 0x3b6, 0x3b1, 0x3ba,
0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x394, 0x3c5, 0x3c4, 0x3b9,
0x3ba, 0x3bf, 0x3cd, 0x20, 0x39a, 0x3b1, 0x3b6, 0x3b1, 0x3ba, 0x3c3, 0x3c4, 0x3ac,
0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3c1, 0x3ad, 0x3b1, 0x3c2, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x39a, 0x3bf, 0x3c1, 0x3ad, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3bf, 0x3c1, 0x3ad, 0x3b1, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3cc, 0x3c3, 0x3c1, 0x3b1, 0x3ca, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39a, 0x3c1, 0x3b1, 0x3c3, 0x3bd, 0x3bf, 0x3b3, 0x3b9, 0x3ac, 0x3c1,
0x3c3, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3c1, 0x3b1, 0x3c3, 0x3bd, 0x3bf, 0x3b3, 0x3b9,
0x3ac, 0x3c1, 0x3c3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39a, 0x3c1, 0x3b1, 0x3c3, 0x3bd, 0x3bf, 0x3b3, 0x3b9, 0x3ac,
0x3c1, 0x3c3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39a, 0x3b9, 0x3c1, 0x3b3, 0x3b9,
0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9,
0x3bd, 0x20, 0x39b, 0x3ac, 0x3b9, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39b, 0x3bf,
0x3c1, 0x3bd, 0x3c4, 0x20, 0x3a7, 0x3ac, 0x3bf, 0x3c5, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39b, 0x3bf,
0x3c1, 0x3bd, 0x3c4, 0x20, 0x3a7, 0x3ac, 0x3bf, 0x3c5, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39b, 0x3bf, 0x3c1, 0x3bd, 0x3c4,
0x20, 0x3a7, 0x3ac, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3ba,
0x3ac, 0x3bf, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3ba, 0x3ac, 0x3bf, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3ba, 0x3ac,
0x3bf, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3b3, 0x3ba, 0x3b1, 0x3bd, 0x3c4,
0x3ac, 0x3bd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3b3, 0x3ba, 0x3b1, 0x3bd, 0x3c4, 0x3ac,
0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39c, 0x3b1, 0x3b3, 0x3ba, 0x3b1, 0x3bd, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39c, 0x3b1, 0x3bb, 0x3b1, 0x3b9, 0x3c3, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39c, 0x3b1, 0x3bb, 0x3b4, 0x3af, 0x3b2, 0x3c9, 0x3bd, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39c, 0x3b1, 0x3c1, 0x3ba, 0x3ad, 0x3b6, 0x3b1, 0x3c2, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x39c, 0x3ac, 0x3c1, 0x3c3,
0x3b1, 0x3bb, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3c5, 0x3c1, 0x3af, 0x3ba,
0x3b9, 0x3bf, 0x3c5, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3b1, 0x3c5, 0x3c1, 0x3af, 0x3ba, 0x3b9,
0x3bf, 0x3c5, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x39c, 0x3b1, 0x3c5, 0x3c1, 0x3af, 0x3ba, 0x3b9, 0x3bf, 0x3c5, 0x38f, 0x3c1,
0x3b1, 0x20, 0x39c, 0x3cc, 0x3c3, 0x3bf, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x395,
0x3b9, 0x3c1, 0x3b7, 0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x20, 0x39c, 0x3b5, 0x3be,
0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9, 0x3c1, 0x3b7, 0x3bd, 0x3b9,
0x3ba, 0x3bf, 0x3cd, 0x20, 0x39c, 0x3b5, 0x3be, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x395, 0x3b9,
0x3c1, 0x3b7, 0x3bd, 0x3b9, 0x3ba, 0x3bf, 0x3cd, 0x20, 0x39c, 0x3b5, 0x3be, 0x3b9,
0x3ba, 0x3bf, 0x3cd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3bb, 0x3ac, 0x3bd,
0x20, 0x39c, 0x3c0, 0x3b1, 0x3c4, 0x3cc, 0x3c1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3bb,
0x3ac, 0x3bd, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3c4, 0x3cc, 0x3c1, 0x398, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3bb, 0x3ac,
0x3bd, 0x20, 0x39c, 0x3c0, 0x3b1, 0x3c4, 0x3cc, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20,
0x39c, 0x3cc, 0x3c3, 0x3c7, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39c, 0x3cc, 0x3c3, 0x3c7,
0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x39c, 0x3cc, 0x3c3, 0x3c7, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39c,
0x3b9, 0x3b1, 0x3bd, 0x3bc, 0x3ac, 0x3c1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3b1,
0x3bf, 0x3cd, 0x3c1, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3b5, 0x3c0,
0x3ac, 0x3bb, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x39a,
0x3b1, 0x3bb, 0x3b7, 0x3b4, 0x3bf, 0x3bd, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3ad, 0x3b1, 0x3c2, 0x20, 0x39a, 0x3b1, 0x3bb, 0x3b7, 0x3b4, 0x3bf, 0x3bd, 0x3af,
0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x39a, 0x3b1, 0x3bb, 0x3b7, 0x3b4, 0x3bf,
0x3bd, 0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2,
0x20, 0x396, 0x3b7, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af, 0x3b1, 0x3c2, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x396, 0x3b7, 0x3bb, 0x3b1, 0x3bd, 0x3b4, 0x3af,
0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x396, 0x3b7, 0x3bb, 0x3b1, 0x3bd, 0x3b4,
0x3af, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20,
0x393, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x393, 0x3b7,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x393, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20,
0x39d, 0x3b9, 0x3bf, 0x3cd, 0x3b5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3,
0x3bf, 0x3c5, 0x20, 0x39d, 0x3cc, 0x3c1, 0x3c6, 0x3bf, 0x3bb, 0x3ba, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c5, 0x20, 0x39d, 0x3cc, 0x3c1, 0x3c6, 0x3bf, 0x3bb,
0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c5, 0x20, 0x39d, 0x3cc, 0x3c1, 0x3c6, 0x3bf, 0x3bb,
0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a6, 0x3b5, 0x3c1, 0x3bd, 0x3ac, 0x3c1, 0x3bd,
0x3c4, 0x3bf, 0x20, 0x3bd, 0x3c4, 0x3b5, 0x20, 0x39d, 0x3bf, 0x3c1, 0x3cc, 0x3bd,
0x3b9, 0x3b1, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a6, 0x3b5, 0x3c1, 0x3bd, 0x3ac, 0x3c1, 0x3bd, 0x3c4,
0x3bf, 0x20, 0x3bd, 0x3c4, 0x3b5, 0x20, 0x39d, 0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3b9,
0x3b1, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a6, 0x3b5, 0x3c1, 0x3bd, 0x3ac, 0x3c1, 0x3bd, 0x3c4, 0x3bf, 0x20, 0x3bd, 0x3c4,
0x3b5, 0x20, 0x39d, 0x3bf, 0x3c1, 0x3cc, 0x3bd, 0x3b9, 0x3b1, 0x38f, 0x3c1, 0x3b1,
0x20, 0x39d, 0x3b7, 0x3c3, 0x3b9, 0x3ce, 0x3bd, 0x20, 0x392, 0x3cc, 0x3c1, 0x3b5,
0x3b9, 0x3b5, 0x3c2, 0x20, 0x39c, 0x3b1, 0x3c1, 0x3b9, 0x3ac, 0x3bd, 0x3b5, 0x3c2,
0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3bf, 0x3b2, 0x3bf, 0x3c3, 0x3b9, 0x3bc, 0x3c0,
0x3af, 0x3c1, 0x3c3, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3bf, 0x3b2, 0x3bf, 0x3c3, 0x3b9,
0x3bc, 0x3c0, 0x3af, 0x3c1, 0x3c3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3bf, 0x3b2, 0x3bf, 0x3c3, 0x3b9, 0x3bc,
0x3c0, 0x3af, 0x3c1, 0x3c3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3bc, 0x3c3,
0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x39f, 0x3bc, 0x3c3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3bc, 0x3c3, 0x3ba, 0x38f, 0x3c1,
0x3b1, 0x20, 0x3a0, 0x3b1, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a0, 0x3b1, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9,
0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3ba, 0x3b9, 0x3c3,
0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3bb, 0x3ac, 0x3bf,
0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3b1, 0x3c2,
0x20, 0x39d, 0x3ad, 0x3b1, 0x3c2, 0x20, 0x393, 0x3bf, 0x3c5, 0x3b9, 0x3bd, 0x3ad,
0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3c1, 0x3b1, 0x3b3, 0x3bf,
0x3c5, 0x3ac, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3c1, 0x3b1, 0x3b3, 0x3bf,
0x3c5, 0x3ac, 0x3b7, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce,
0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b1, 0x3c1, 0x3b1, 0x3b3, 0x3bf, 0x3c5, 0x3ac, 0x3b7,
0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b5, 0x3c1, 0x3bf, 0x3cd, 0x3a7, 0x3b5,
0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a0, 0x3b5, 0x3c1, 0x3bf, 0x3cd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b5, 0x3c1, 0x3bf, 0x3cd, 0x38f, 0x3c1, 0x3b1,
0x20, 0x3a6, 0x3b9, 0x3bb, 0x3b9, 0x3c0, 0x3c0, 0x3b9, 0x3bd, 0x3ce, 0x3bd, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x3a6, 0x3b9, 0x3bb, 0x3b9, 0x3c0, 0x3c0, 0x3b9, 0x3bd, 0x3ce, 0x3bd, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a6, 0x3b9,
0x3bb, 0x3b9, 0x3c0, 0x3c0, 0x3b9, 0x3bd, 0x3ce, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20,
0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x3a6, 0x3bf, 0x3af, 0x3bd, 0x3b9, 0x3be,
0x38f, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b5, 0x3bd, 0x20, 0x3a0, 0x3b9, 0x3b5, 0x3c1,
0x20, 0x3ba, 0x3b1, 0x3b9, 0x20, 0x39c, 0x3b9, 0x3ba, 0x3b5, 0x3bb, 0x3cc, 0x3bd,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a3, 0x3b5, 0x3bd, 0x20, 0x3a0, 0x3b9, 0x3b5, 0x3c1, 0x20, 0x3ba,
0x3b1, 0x3b9, 0x20, 0x39c, 0x3b9, 0x3ba, 0x3b5, 0x3bb, 0x3cc, 0x3bd, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b5, 0x3bd,
0x20, 0x3a0, 0x3b9, 0x3b5, 0x3c1, 0x20, 0x3ba, 0x3b1, 0x3b9, 0x20, 0x39c, 0x3b9,
0x3ba, 0x3b5, 0x3bb, 0x3cc, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3af, 0x3c4,
0x3ba, 0x3b5, 0x3c1, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3bf, 0x3bd, 0x3ac,
0x3c0, 0x3b5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a0, 0x3b9, 0x3bf, 0x3bd, 0x3b3, 0x3b9,
0x3ac, 0x3bd, 0x3b3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a1, 0x3b5, 0x3ca, 0x3bd,
0x3b9, 0x3cc, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a1, 0x3cc, 0x3b8, 0x3b5, 0x3c1,
0x3b1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b1, 0x3c7, 0x3b1, 0x3bb, 0x3af, 0x3bd,
0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b1, 0x3c7, 0x3b1, 0x3bb, 0x3af, 0x3bd, 0x3b7,
0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x3a3, 0x3b1, 0x3c7, 0x3b1, 0x3bb, 0x3af, 0x3bd, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x3a3, 0x3ac, 0x3bc, 0x3b1, 0x3c1, 0x3b1, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3ac,
0x3bc, 0x3b1, 0x3c1, 0x3b1, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b1, 0x3bc, 0x3ac, 0x3c1, 0x3b1, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b1, 0x3bc, 0x3cc, 0x3b1, 0x3a7, 0x3b5, 0x3b9, 0x3bc,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3b1,
0x3bc, 0x3cc, 0x3b1, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a3, 0x3b1, 0x3bc, 0x3cc, 0x3b1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a3,
0x3b5, 0x3cb, 0x3c7, 0x3b5, 0x3bb, 0x3bb, 0x3ce, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20,
0x3a3, 0x3b9, 0x3b3, 0x3ba, 0x3b1, 0x3c0, 0x3bf, 0x3cd, 0x3c1, 0x3b7, 0x3c2, 0x38f,
0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3c9, 0x3bd, 0x20, 0x3a3, 0x3bf, 0x3bb,
0x3bf, 0x3bc, 0x3ce, 0x3bd, 0x3c4, 0x3bf, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d,
0x3cc, 0x3c4, 0x3b9, 0x3b1, 0x3c2, 0x20, 0x393, 0x3b5, 0x3c9, 0x3c1, 0x3b3, 0x3af,
0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3bf, 0x3c5, 0x3c1, 0x3b9, 0x3bd,
0x3ac, 0x3bc, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a3, 0x3af, 0x3bf, 0x3b2, 0x3b1, 0x38f,
0x3c1, 0x3b1, 0x20, 0x3a4, 0x3b1, 0x3ca, 0x3c4, 0x3ae, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x3a4, 0x3b1, 0x3ca, 0x3c0, 0x3ad, 0x3b9, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3b1, 0x3ca,
0x3c0, 0x3ad, 0x3b9, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a4, 0x3b1, 0x3ca, 0x3c0, 0x3ad, 0x3b9, 0x38f, 0x3c1, 0x3b1, 0x20,
0x3a4, 0x3b1, 0x3c4, 0x3b6, 0x3b9, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x38f,
0x3c1, 0x3b1, 0x20, 0x3a4, 0x3bf, 0x3ba, 0x3b5, 0x3bb, 0x3ac, 0x3bf, 0x3c5, 0x38f,
0x3c1, 0x3b1, 0x20, 0x3a4, 0x3cc, 0x3bd, 0x3b3, 0x3ba, 0x3b1, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4,
0x3cc, 0x3bd, 0x3b3, 0x3ba, 0x3b1, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3cc, 0x3bd, 0x3b3, 0x3ba, 0x3b1, 0x38f, 0x3c1,
0x3b1, 0x20, 0x3a4, 0x3c3, 0x3bf, 0x3c5, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4,
0x3bf, 0x3c5, 0x3c1, 0x3ba, 0x3bc, 0x3b5, 0x3bd, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a4, 0x3bf, 0x3c5, 0x3c1, 0x3ba, 0x3bc, 0x3b5, 0x3bd, 0x3b9, 0x3c3,
0x3c4, 0x3ac, 0x3bd, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x3a4, 0x3bf, 0x3c5, 0x3c1, 0x3ba, 0x3bc, 0x3b5, 0x3bd, 0x3b9, 0x3c3,
0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x3a4, 0x3bf, 0x3c5, 0x3b2, 0x3b1,
0x3bb, 0x3bf, 0x3cd, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3c1, 0x3bf, 0x3c5,
0x3b3, 0x3bf, 0x3c5, 0x3ac, 0x3b7, 0x3c2, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3c1, 0x3bf,
0x3c5, 0x3b3, 0x3bf, 0x3c5, 0x3ac, 0x3b7, 0x3c2, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd,
0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3c1, 0x3bf, 0x3c5, 0x3b3,
0x3bf, 0x3c5, 0x3ac, 0x3b7, 0x3c2, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5, 0x3b6,
0x3bc, 0x3c0, 0x3b5, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f,
0x3c5, 0x3b6, 0x3bc, 0x3c0, 0x3b5, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x398,
0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5,
0x3b6, 0x3bc, 0x3c0, 0x3b5, 0x3ba, 0x3b9, 0x3c3, 0x3c4, 0x3ac, 0x3bd, 0x38f, 0x3c1,
0x3b1, 0x20, 0x392, 0x3b1, 0x3bd, 0x3bf, 0x3c5, 0x3ac, 0x3c4, 0x3bf, 0x3c5, 0x3a7,
0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1,
0x20, 0x392, 0x3b1, 0x3bd, 0x3bf, 0x3c5, 0x3ac, 0x3c4, 0x3bf, 0x3c5, 0x398, 0x3b5,
0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3b1, 0x3bd,
0x3bf, 0x3c5, 0x3ac, 0x3c4, 0x3bf, 0x3c5, 0x38f, 0x3c1, 0x3b1, 0x20, 0x392, 0x3b5,
0x3bd, 0x3b5, 0x3b6, 0x3bf, 0x3c5, 0x3ad, 0x3bb, 0x3b1, 0x3c2, 0x38f, 0x3c1, 0x3b1,
0x20, 0x392, 0x3bb, 0x3b1, 0x3b4, 0x3b9, 0x3b2, 0x3bf, 0x3c3, 0x3c4, 0x3cc, 0x3ba,
0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x392, 0x3bb, 0x3b1, 0x3b4, 0x3b9, 0x3b2, 0x3bf, 0x3c3, 0x3c4, 0x3cc,
0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20,
0x392, 0x3bb, 0x3b1, 0x3b4, 0x3b9, 0x3b2, 0x3bf, 0x3c3, 0x3c4, 0x3cc, 0x3ba, 0x38f,
0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3bb, 0x3b3, 0x3ba, 0x3bf, 0x3b3, 0x3ba, 0x3c1,
0x3b1, 0x3bd, 0x3c4, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae,
0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3bb, 0x3b3, 0x3ba, 0x3bf, 0x3b3,
0x3ba, 0x3c1, 0x3b1, 0x3bd, 0x3c4, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3bb, 0x3b3, 0x3ba, 0x3bf, 0x3b3, 0x3ba,
0x3c1, 0x3b1, 0x3bd, 0x3c4, 0x38f, 0x3c1, 0x3b1, 0x20, 0x392, 0x3cc, 0x3c3, 0x3c4,
0x3bf, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39d, 0x3ae, 0x3c3, 0x3bf, 0x3c5, 0x20,
0x393, 0x3bf, 0x3c5, 0x3ad, 0x3b9, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x39f, 0x3c5,
0x3ac, 0x3bb, 0x3b9, 0x3c2, 0x20, 0x3ba, 0x3b1, 0x3b9, 0x20, 0x3a6, 0x3bf, 0x3c5,
0x3c4, 0x3bf, 0x3cd, 0x3bd, 0x3b1, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b9, 0x3b1,
0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x3a7, 0x3b5, 0x3b9, 0x3bc, 0x3b5, 0x3c1,
0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b9, 0x3b1, 0x3ba,
0x3bf, 0x3cd, 0x3c4, 0x3c3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20,
0x3ce, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b9, 0x3b1, 0x3ba, 0x3bf, 0x3cd, 0x3c4, 0x3c3,
0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x391, 0x3b9, 0x3ba, 0x3b1, 0x3c4, 0x3b5, 0x3c1,
0x3af, 0x3bd, 0x3bc, 0x3c0, 0x3bf, 0x3c5, 0x3c1, 0x3b3, 0x3ba, 0x3a7, 0x3b5, 0x3b9,
0x3bc, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1, 0x3b1, 0x20, 0x391,
0x3b9, 0x3ba, 0x3b1, 0x3c4, 0x3b5, 0x3c1, 0x3af, 0x3bd, 0x3bc, 0x3c0, 0x3bf, 0x3c5,
0x3c1, 0x3b3, 0x3ba, 0x398, 0x3b5, 0x3c1, 0x3b9, 0x3bd, 0x3ae, 0x20, 0x3ce, 0x3c1,
0x3b1, 0x20, 0x391, 0x3b9, 0x3ba, 0x3b1, 0x3c4, 0x3b5, 0x3c1, 0x3af, 0x3bd, 0x3bc,
0x3c0, 0x3bf, 0x3c5, 0x3c1, 0x3b3, 0x3ba, 0x38f, 0x3c1, 0x3b1, 0x20, 0x393, 0x3b9,
0x3bf, 0x3cd, 0x3ba, 0x3bf, 0x3bd, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61,
0x20, 0xf3, 0x72, 0x61, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x20,
0xf3, 0x72, 0x61, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x20, 0xf3, 0x72, 0x61, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c,
0x61, 0x20, 0xf3, 0x72, 0x61, 0xa8f, 0xa95, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf,
0xa8f, 0xa95, 0xab0, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2,
0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa95, 0xab0, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xaab, 0xa98, 0xabe,
0xaa8, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae,
0xaa7, 0xacd, 0xaaf, 0x20, 0xa86, 0xaab, 0xacd, 0xab0, 0xabf, 0xa95, 0xabe, 0x20,
0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20, 0xa86, 0xaab, 0xacd,
0xab0, 0xabf, 0xa95, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa6, 0xa95, 0xacd, 0xab7,
0xabf, 0xaa3, 0x20, 0xa86, 0xaab, 0xacd, 0xab0, 0xabf, 0xa95, 0xabe, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf,
0xaae, 0x20, 0xa86, 0xaab, 0xacd, 0xab0, 0xabf, 0xa95, 0xabe, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0x20, 0xa86, 0xaab, 0xacd, 0xab0,
0xabf, 0xa95, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0x20, 0xa86, 0xaab, 0xacd, 0xab0, 0xabf,
0xa95, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xa85, 0xab2, 0xabe, 0xab8, 0xacd, 0xa95, 0xabe, 0x20, 0xab8, 0xaae,
0xaaf, 0xa85, 0xab2, 0xabe, 0xab8, 0xacd, 0xa95, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xab2, 0xabe, 0xab8, 0xacd, 0xa95, 0xabe,
0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xab2, 0xacd,
0xaae, 0xabe, 0xa9f, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xab2, 0xacd, 0xaae,
0xabe, 0xa9f, 0xac0, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2,
0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xab2, 0xacd, 0xaae, 0xabe, 0xa9f, 0xac0,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0xa8f, 0xaae, 0xac7, 0xa9d, 0xacb, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xaae,
0xac7, 0xa9d, 0xacb, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa8f, 0xaae, 0xac7, 0xa9d, 0xacb, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0,
0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8,
0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4,
0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20,
0xa95, 0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20,
0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8, 0xacd,
0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8,
0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0,
0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0x20, 0xab8,
0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0,
0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0,
0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac2, 0xab0,
0xacd, 0xab5, 0xac0, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95,
0xaa8, 0x20, 0xaae, 0xabe, 0xa89, 0xaa8, 0xacd, 0xa9f, 0xaa8, 0x20, 0xab8, 0xaae,
0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf,
0xa95, 0xaa8, 0x20, 0xaae, 0xabe, 0xa89, 0xaa8, 0xacd, 0xa9f, 0xaa8, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0,
0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xaae, 0xabe, 0xa89,
0xaa8, 0xacd, 0xa9f, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae,
0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf,
0xa95, 0xaa8, 0x20, 0xaaa, 0xac7, 0xab8, 0xabf, 0xaab, 0xabf, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0,
0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac7, 0xab8, 0xabf, 0xaab, 0xabf, 0xa95, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4,
0xab0, 0x20, 0xa85, 0xaae, 0xac7, 0xab0, 0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac7,
0xab8, 0xabf, 0xaab, 0xabf, 0xa95, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8,
0xaae, 0xaaf, 0xa85, 0xaa8, 0xabe, 0xaa6, 0xabf, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf,
0xa85, 0xaa8, 0xabe, 0xaa1, 0xabf, 0xab0, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe,
0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xaa8, 0xabe, 0xaa1,
0xabf, 0xab0, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xa8f, 0xaaa, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f,
0xaaa, 0xabf, 0xaaf, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa8f, 0xaaa, 0xabf, 0xaaf, 0xabe, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20,
0xab8, 0xaae, 0xaaf, 0xa85, 0xa95, 0xacd, 0xaa4, 0xabe, 0xa89, 0x20, 0xab8, 0xaae,
0xaaf, 0xa85, 0xa95, 0xacd, 0xaa4, 0xabe, 0xa89, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae,
0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xa95, 0xacd,
0xaa4, 0xabe, 0xa89, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xa8d, 0xa95, 0xacd, 0xa9f, 0xacb, 0xaac, 0x20, 0xab8, 0xaae,
0xaaf, 0xa8d, 0xa95, 0xacd, 0xa9f, 0xacb, 0xaac, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae,
0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8d, 0xa95, 0xacd,
0xa9f, 0xacb, 0xaac, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xa85, 0xab0, 0xac7, 0xaac, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8,
0xaae, 0xaaf, 0xa85, 0xab0, 0xac7, 0xaac, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xab0, 0xac7, 0xaac, 0xabf, 0xaaf,
0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xab0,
0xacd, 0xa9c, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xac0, 0xaa8, 0xabe, 0x20, 0xab8, 0xaae,
0xaaf, 0xa86, 0xab0, 0xacd, 0xa9c, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xac0, 0xaa8, 0xabe,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xab0, 0xacd,
0xa9c, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xac0, 0xaa8, 0xabe, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a,
0xabf, 0xaae, 0xac0, 0x20, 0xa86, 0xab0, 0xacd, 0xa9c, 0xac7, 0xaa8, 0xacd, 0xa9f,
0xac0, 0xaa8, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf,
0xaae, 0xac0, 0x20, 0xa86, 0xab0, 0xacd, 0xa9c, 0xac7, 0xaa8, 0xacd, 0xa9f, 0xac0,
0xaa8, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa,
0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xa86, 0xab0, 0xacd, 0xa9c, 0xac7,
0xaa8, 0xacd, 0xa9f, 0xac0, 0xaa8, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xab0, 0xacd, 0xaae, 0xac7, 0xaa8,
0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa86, 0xab0, 0xacd, 0xaae, 0xac7,
0xaa8, 0xabf, 0xaaf, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa86, 0xab0, 0xacd, 0xaae, 0xac7, 0xaa8, 0xabf, 0xaaf, 0xabe, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa9f,
0xab2, 0xabe, 0xaa8, 0xacd, 0xa9f, 0xabf, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f,
0xa9f, 0xab2, 0xabe, 0xaa8, 0xacd, 0xa9f, 0xabf, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa9f, 0xab2, 0xabe, 0xaa8, 0xacd, 0xa9f,
0xabf, 0xa95, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95,
0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xa91, 0xab8, 0xacd,
0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa91, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20,
0xa95, 0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa93, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0,
0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd,
0xab0, 0xac0, 0xaaf, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa91, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20,
0xa95, 0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaaa, 0xab6,
0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa91, 0xab8, 0xacd,
0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa95, 0xac7, 0xaa8,
0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf,
0xaae, 0xac0, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa93,
0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa95,
0xac7, 0xaa8, 0xacd, 0xaa6, 0xacd, 0xab0, 0xac0, 0xaaf, 0x20, 0xaaa, 0xab6, 0xacd,
0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0xaaf, 0x20, 0xa91, 0xab8, 0xacd,
0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf,
0xa91, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20,
0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0xaaf, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xa93, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2,
0xabf, 0xaaf, 0xaa8, 0x20, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0xaaf, 0x20,
0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a,
0xabf, 0xaae, 0xac0, 0x20, 0xa91, 0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2,
0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa91, 0xab8, 0xacd, 0xa9f, 0xacd,
0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf,
0xaae, 0xac0, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa93,
0xab8, 0xacd, 0xa9f, 0xacd, 0xab0, 0xac7, 0xab2, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaaa,
0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20,
0xab8, 0xaae, 0xaaf, 0xa85, 0xa9d, 0xab0, 0xaac, 0xac8, 0xa9c, 0xabe, 0xaa8, 0x20,
0xab8, 0xaae, 0xaaf, 0xa85, 0xa9d, 0xab0, 0xaac, 0xac8, 0xa9c, 0xabe, 0xaa8, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa85, 0xa9d, 0xab0, 0xaac,
0xac8, 0xa9c, 0xabe, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa9d, 0xacb, 0xab0, 0xacd, 0xab8, 0x20, 0xab8,
0xaae, 0xaaf, 0xa8f, 0xa9d, 0xacb, 0xab0, 0xacd, 0xab8, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa8f, 0xa9d, 0xacb, 0xab0, 0xacd, 0xab8, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac,
0xabe, 0xa82, 0xa97, 0xacd, 0xab2, 0xabe, 0xaa6, 0xac7, 0xab6, 0x20, 0xab8, 0xaae,
0xaaf, 0xaac, 0xabe, 0xa82, 0xa97, 0xacd, 0xab2, 0xabe, 0xaa6, 0xac7, 0xab6, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xabe, 0xa82, 0xa97,
0xacd, 0xab2, 0xabe, 0xaa6, 0xac7, 0xab6, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaad, 0xac2, 0xa9f, 0xabe, 0xaa8, 0x20,
0xab8, 0xaae, 0xaaf, 0xaac, 0xacb, 0xab2, 0xabf, 0xab5, 0xabf, 0xaaf, 0xabe, 0x20,
0xab8, 0xaae, 0xaaf, 0xaac, 0xacd, 0xab0, 0xabe, 0xa9d, 0xabf, 0xab2, 0xabf, 0xaaf,
0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xacd, 0xab0, 0xabe, 0xa9d, 0xabf, 0xab2,
0xabf, 0xaaf, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xaac, 0xacd, 0xab0, 0xabe, 0xa9d, 0xabf, 0xab2, 0xabf, 0xaaf, 0xabe, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaac, 0xacd,
0xab0, 0xac1, 0xaa8, 0xac7, 0xa87, 0x20, 0xaa6, 0xab0, 0xac1, 0xab8, 0xab2, 0xabe,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac5, 0xaaa, 0x20, 0xab5, 0xab0, 0xacd,
0xaa1, 0xac7, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac5, 0xaaa, 0x20, 0xab5, 0xab0,
0xacd, 0xaa1, 0xac7, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xa95, 0xac5, 0xaaa, 0x20, 0xab5, 0xab0, 0xacd, 0xaa1, 0xac7, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac7, 0xaae,
0xacb, 0xab0, 0xacb, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xa9a, 0xac7, 0xaa4, 0xab9, 0xabe, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xac7,
0xaa4, 0xab9, 0xabe, 0xaae, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa9a, 0xac7, 0xaa4, 0xab9, 0xabe, 0xaae, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8,
0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xabf, 0xab2, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf,
0xa9a, 0xabf, 0xab2, 0xac0, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa9a, 0xabf, 0xab2, 0xac0, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xac0, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa9a, 0xac0, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xa9a, 0xac0, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xa9a, 0xacb, 0xa87, 0xaac, 0xabe, 0xab2, 0xacd, 0xab8, 0xaa8, 0x20, 0xab8, 0xaae,
0xaaf, 0xa9a, 0xacb, 0xa87, 0xaac, 0xabe, 0xab2, 0xacd, 0xab8, 0xaa8, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xacb, 0xa87, 0xaac, 0xabe,
0xab2, 0xacd, 0xab8, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacd, 0xab0, 0xabf, 0xab8, 0xaae, 0xab8, 0x20,
0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95,
0xacb, 0xa95, 0xacb, 0xa9d, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1,
0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xab2, 0xaae, 0xacd, 0xaac,
0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xab2, 0xaae, 0xacd,
0xaac, 0xabf, 0xaaf, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa95, 0xacb, 0xab2, 0xaae, 0xacd, 0xaac, 0xabf, 0xaaf, 0xabe, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xac1,
0xa95, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20,
0xab8, 0xaae, 0xaaf, 0xa95, 0xac1, 0xa95, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8,
0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa95, 0xac1, 0xa95, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1,
0xacd, 0xab8, 0x20, 0xa85, 0xab0, 0xacd, 0xaa7, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacd, 0xaaf, 0xac1, 0xaac,
0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacd, 0xaaf, 0xac1, 0xaac, 0xabe, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacd, 0xaaf, 0xac1,
0xaac, 0xabe, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa1,
0xac7, 0xab5, 0xabf, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa1, 0xacd, 0xaaf, 0xac1,
0xaae, 0xacb, 0xaa8, 0xacd, 0xa9f, 0x2d, 0xaa1, 0xac0, 0x2018, 0xa89, 0xab0, 0xacd,
0xab5, 0xabf, 0xab2, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5,
0x20, 0xaa4, 0xabf, 0xaae, 0xacb, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xab8,
0xacd, 0xa9f, 0xab0, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20,
0xab8, 0xaae, 0xaaf, 0xa87, 0xab8, 0xacd, 0xa9f, 0xab0, 0x20, 0xa86, 0xa87, 0xab2,
0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa87, 0xab8, 0xacd, 0xa9f, 0xab0, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8,
0xacd, 0xaa1, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xa8f, 0xa95, 0xacd, 0xab5, 0xabe, 0xaa1, 0xacb, 0xab0, 0x20, 0xab8,
0xaae, 0xaaf, 0xaae, 0xaa7, 0xacd, 0xaaf, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa,
0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xaa7, 0xacd, 0xaaf, 0x20,
0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xaa7, 0xacd, 0xaaf, 0x20, 0xaaf, 0xac1,
0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0,
0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb,
0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0xac0, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb,
0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xac1, 0xaa6, 0xac2, 0xab0, 0x2d, 0xaaa, 0xac2,
0xab0, 0xacd, 0xab5, 0xac0, 0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf,
0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0xac0,
0x20, 0xaaf, 0xac1, 0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xaaf, 0xac1, 0xab0,
0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0xac0, 0x20, 0xaaf, 0xac1,
0xab0, 0xacb, 0xaaa, 0xabf, 0xaaf, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xac9, 0xa95, 0xab2, 0xac7, 0xaa8,
0xacd, 0xaa1, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8,
0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xac9, 0xa95, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1,
0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xac9, 0xa95, 0xab2, 0xac7,
0xaa8, 0xacd, 0xaa1, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd,
0xab8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xaab, 0xac0, 0xa9c, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xac0, 0xa9c,
0xac0, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xac0,
0xa9c, 0xac0, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xaab, 0xacd, 0xab0, 0xac7, 0xa82, 0xa9a, 0x20, 0xa97, 0xaaf, 0xabe,
0xaa8, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xacd, 0xab0, 0xac7, 0xa82, 0xa9a,
0x20, 0xab8, 0xaa7, 0xab0, 0xacd, 0xaa8, 0x20, 0xa85, 0xaa8, 0xac7, 0x20, 0xa8f,
0xaa8, 0xacd, 0xa9f, 0xabe, 0xab0, 0xacd, 0xa95, 0xa9f, 0xabf, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xa97, 0xabe, 0xab2, 0xabe, 0xaaa, 0xabe, 0xa97, 0xacb, 0xab8, 0x20,
0xab8, 0xaae, 0xaaf, 0xa97, 0xac7, 0xaae, 0xacd, 0xaac, 0xabf, 0xaaf, 0xab0, 0x20,
0xab8, 0xaae, 0xaaf, 0xa9c, 0xacd, 0xaaf, 0xacb, 0xab0, 0xacd, 0xa9c, 0xabf, 0xaaf,
0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9c, 0xacd, 0xaaf, 0xacb, 0xab0, 0xacd, 0xa9c,
0xabf, 0xaaf, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xa9c, 0xacd, 0xaaf, 0xacb, 0xab0, 0xacd, 0xa9c, 0xabf, 0xaaf, 0xabe, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa97, 0xabf,
0xab2, 0xaac, 0xab0, 0xacd, 0xa9f, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd,
0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa97, 0xacd, 0xab0, 0xac0, 0xaa8,
0xab5, 0xabf, 0xa9a, 0x20, 0xaae, 0xaa7, 0xacd, 0xaaf, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xaa8,
0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0,
0xacd, 0xab5, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd,
0xaa1, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2,
0xab0, 0xacd, 0xab5, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8,
0xacd, 0xaa1, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa,
0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xaa8, 0xab2,
0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xaa8, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa97, 0xacd, 0xab5, 0xabe, 0xaae, 0x20,
0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae,
0xaaf, 0xa97, 0xab2, 0xacd, 0xaab, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xab9, 0xab5, 0xabe, 0xa88, 0x2d, 0xa8f, 0xab2, 0xacd, 0xaaf, 0xac1,
0xab6, 0xabf, 0xa85, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab9, 0xab5, 0xabe, 0xa87,
0x2d, 0xa8f, 0xab2, 0xacd, 0xaaf, 0xac1, 0xab6, 0xabf, 0xa85, 0xaa8, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab9, 0xab5, 0xabe, 0xa87, 0x2d,
0xa8f, 0xab2, 0xacd, 0xaaf, 0xac1, 0xab6, 0xabf, 0xa85, 0xaa8, 0x20, 0xaa6, 0xabf,
0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab9, 0xacb, 0xa82, 0xa97, 0x20, 0xa95,
0xacb, 0xa82, 0xa97, 0x20, 0xab8, 0xaae, 0xaaf, 0xab9, 0xacb, 0xa82, 0xa97, 0x20,
0xa95, 0xacb, 0xa82, 0xa97, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xab9, 0xacb, 0xa82, 0xa97, 0x20, 0xa95, 0xacb, 0xa82, 0xa97, 0x20, 0xa97,
0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab9, 0xacb,
0xab5, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xab9, 0xacb, 0xab5, 0xacd, 0xaa1,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab9, 0xacb, 0xab5,
0xacd, 0xaa1, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xaad, 0xabe, 0xab0, 0xaa4, 0xac0, 0xaaf, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaad, 0xabe, 0xab0, 0xaa4, 0xac0, 0xaaf, 0x20,
0xaae, 0xab9, 0xabe, 0xab8, 0xabe, 0xa97, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87,
0xaa8, 0xacd, 0xaa1, 0xacb, 0xa9a, 0xabe, 0xa87, 0xaa8, 0xabe, 0x20, 0xab8, 0xaae,
0xaaf, 0xaae, 0xaa7, 0xacd, 0xaaf, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xacb, 0xaa8,
0xac7, 0xab6, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0,
0xacd, 0xab5, 0xac0, 0xaaf, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xacb, 0xaa8, 0xac7,
0xab6, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a,
0xabf, 0xaae, 0xac0, 0x20, 0xa87, 0xaa8, 0xacd, 0xaa1, 0xacb, 0xaa8, 0xac7, 0xab6,
0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa88, 0xab0, 0xabe, 0xaa8, 0x20,
0xab8, 0xaae, 0xaaf, 0xa88, 0xab0, 0xabe, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xa88, 0xab0, 0xabe, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5,
0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xab0, 0xacd, 0xa95, 0xac1, 0xaa4, 0xacd,
0xab8, 0xacd, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xab0, 0xacd, 0xa95, 0xac1,
0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xa87, 0xab0, 0xacd, 0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95,
0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0xa87, 0xa9d, 0xab0, 0xabe, 0xa87, 0xab2, 0x20, 0xab8, 0xaae, 0xaaf, 0xa87, 0xa9d,
0xab0, 0xabe, 0xa87, 0xab2, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xa87, 0xa9d, 0xab0, 0xabe, 0xa87, 0xab2, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8,
0x20, 0xab8, 0xaae, 0xaaf, 0xa9c, 0xabe, 0xaaa, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae,
0xaaf, 0xa9c, 0xabe, 0xaaa, 0xabe, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xa9c, 0xabe, 0xaaa, 0xabe, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5,
0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7, 0xa9f, 0xacd, 0xab0, 0xacb, 0xaaa,
0xac7, 0xab5, 0xab2, 0xacb, 0xab5, 0xacd, 0xab8, 0xacd, 0xa95, 0x2d, 0xa95, 0xabe,
0xaae, 0xa9a, 0xaa4, 0xab8, 0xacd, 0xa95, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa,
0xac7, 0xa9f, 0xacd, 0xab0, 0xacb, 0xaaa, 0xac7, 0xab5, 0xab2, 0xacb, 0xab5, 0xacd,
0xab8, 0xacd, 0xa95, 0x2d, 0xa95, 0xabe, 0xaae, 0xa9a, 0xaa4, 0xab8, 0xacd, 0xa95,
0xac0, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4, 0x20,
0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7, 0xa9f, 0xacd, 0xab0, 0xacb, 0xaaa, 0xac7, 0xab5,
0xab2, 0xacb, 0xab5, 0xacd, 0xab8, 0xacd, 0xa95, 0x2d, 0xa95, 0xabe, 0xaae, 0xa9a,
0xaa4, 0xab8, 0xacd, 0xa95, 0xac0, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac2, 0xab0, 0xacd, 0xab5, 0x20, 0xa95,
0xa9d, 0xabe, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xab6, 0xacd, 0xa9a, 0xabf, 0xaae, 0x20, 0xa95, 0xa9d, 0xabe, 0xa95,
0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb,
0xab0, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacb, 0xab0, 0xabf,
0xaaf, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95,
0xacb, 0xab0, 0xabf, 0xaaf, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8,
0xaae, 0xaaf, 0xa95, 0xacb, 0xab8, 0xab0, 0xac7, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95,
0xacd, 0xab0, 0xac7, 0xab8, 0xacd, 0xaa8, 0xacb, 0xaaf, 0xabe, 0xab0, 0xacd, 0xab8,
0xacd, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacd, 0xab0, 0xac7, 0xab8, 0xacd,
0xaa8, 0xacb, 0xaaf, 0xabe, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xacd, 0xab0, 0xac7, 0xab8, 0xacd,
0xaa8, 0xacb, 0xaaf, 0xabe, 0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xabf, 0xab0,
0xacd, 0xa97, 0xabf, 0xa9d, 0xacd, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8,
0xaae, 0xaaf, 0xab2, 0xa82, 0xa95, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xab2, 0xabe,
0xa87, 0xaa8, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8,
0x20, 0xab8, 0xaae, 0xaaf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0x20, 0xab9, 0xacb,
0xab5, 0x20, 0xab8, 0xaae, 0xaaf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0x20, 0xab9,
0xacb, 0xab5, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab2,
0xacb, 0xab0, 0xacd, 0xaa1, 0x20, 0xab9, 0xacb, 0xab5, 0x20, 0xaa6, 0xabf, 0xab5,
0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xa95, 0xabe, 0xa89, 0x20, 0xab8, 0xaae,
0xaaf, 0xaae, 0xa95, 0xabe, 0xa89, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3,
0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xa95, 0xabe, 0xa89, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae,
0xa97, 0xabe, 0xaa1, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xa97, 0xabe, 0xaa1,
0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xa97,
0xabe, 0xaa1, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xaae, 0xab2, 0xac7, 0xab6, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8,
0xaae, 0xaaf, 0xaae, 0xabe, 0xab2, 0xaa6, 0xac0, 0xab5, 0x20, 0xab8, 0xaae, 0xaaf,
0xaae, 0xabe, 0xab0, 0xacd, 0xa95, 0xacd, 0xaaf, 0xac2, 0xab8, 0xab8, 0x20, 0xab8,
0xaae, 0xaaf, 0xaae, 0xabe, 0xab0, 0xacd, 0xab6, 0xab2, 0x20, 0xa86, 0xa87, 0xab2,
0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb,
0xab0, 0xabf, 0xab6, 0xabf, 0xaaf, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb,
0xab0, 0xabf, 0xab6, 0xabf, 0xaaf, 0xab8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab0, 0xabf, 0xab6, 0xabf, 0xaaf, 0xab8, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae,
0xacb, 0xab8, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xac7, 0xa95, 0xacd, 0xab8,
0xabf, 0xa95, 0xaa8, 0x20, 0xaaa, 0xac7, 0xab8, 0xabf, 0xaab, 0xabf, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xaae, 0xac7, 0xa95, 0xacd, 0xab8, 0xabf, 0xa95, 0xaa8, 0x20,
0xaaa, 0xac7, 0xab8, 0xabf, 0xaab, 0xabf, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xac7, 0xa95, 0xacd, 0xab8, 0xabf, 0xa95, 0xaa8,
0x20, 0xaaa, 0xac7, 0xab8, 0xabf, 0xaab, 0xabf, 0xa95, 0x20, 0xaa6, 0xabf, 0xab5,
0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xab2, 0xabe, 0xaa8, 0x20, 0xaac, 0xabe,
0xa9f, 0xacb, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xab2, 0xabe, 0xaa8, 0x20,
0xaac, 0xabe, 0xa9f, 0xacb, 0xab0, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xa89, 0xab2, 0xabe, 0xaa8, 0x20, 0xaac, 0xa9f, 0xacb, 0xab0, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae,
0xacb, 0xab8, 0xacd, 0xa95, 0xacb, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacb, 0xab8,
0xacd, 0xa95, 0xacb, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xaae, 0xacb, 0xab8, 0xacd, 0xa95, 0xacb, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaae, 0xacd, 0xaaf, 0xabe, 0xaa8, 0xaae,
0xabe, 0xab0, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacc, 0xab0, 0xac1, 0x20, 0xab8,
0xaae, 0xaaf, 0xaa8, 0xac7, 0xaaa, 0xabe, 0xab3, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8,
0xacd, 0xaaf, 0xac1, 0x20, 0xab8, 0xac7, 0xab2, 0xac7, 0xaa1, 0xacb, 0xaa8, 0xabf,
0xaaf, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1, 0x20, 0xab8,
0xac7, 0xab2, 0xac7, 0xaa1, 0xacb, 0xaa8, 0xabf, 0xaaf, 0xabe, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1, 0x20, 0xab8,
0xac7, 0xab2, 0xac7, 0xaa1, 0xacb, 0xaa8, 0xabf, 0xaaf, 0xabe, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf,
0xac1, 0xa9d, 0xac0, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf,
0xaa8, 0xacd, 0xaaf, 0xac1, 0xa9d, 0xac0, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac1,
0xa9d, 0xac0, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8,
0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacd, 0xaaf, 0xac2, 0xaab, 0xabe, 0xa89, 0xaa8,
0xacd, 0xaa1, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8,
0xacd, 0xaaf, 0xac2, 0xaab, 0xabe, 0xa89, 0xaa8, 0xacd, 0xaa1, 0xab2, 0xac7, 0xaa8,
0xacd, 0xaa1, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8,
0xacd, 0xaaf, 0xac2, 0xaab, 0xabe, 0xa89, 0xaa8, 0xacd, 0xaa1, 0xab2, 0xac7, 0xaa8,
0xacd, 0xaa1, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8,
0xac0, 0xaaf, 0xac1, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacb, 0xab0, 0xaab, 0xac9,
0xa95, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8, 0xaae,
0xaaf, 0xaa8, 0xacb, 0xab0, 0xaab, 0xac9, 0xa95, 0x20, 0xa86, 0xa87, 0xab2, 0xac7,
0xaa8, 0xacd, 0xaa1, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xaa8, 0xacb, 0xab0, 0xaab, 0xac9, 0xa95, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8,
0xacd, 0xaa1, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab,
0xab0, 0xacd, 0xaa8, 0xabe, 0xaa8, 0xacd, 0xaa1, 0xacb, 0x20, 0xaa1, 0xac0, 0x20,
0xaa8, 0xacb, 0xab0, 0xacb, 0xaa8, 0xacd, 0xab9, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf,
0xaab, 0xab0, 0xacd, 0xaa8, 0xabe, 0xaa8, 0xacd, 0xaa1, 0xacb, 0x20, 0xaa1, 0xac0,
0x20, 0xaa8, 0xacb, 0xab0, 0xacb, 0xaa8, 0xacd, 0xab9, 0xabe, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xab0, 0xacd, 0xaa8, 0xabe, 0xaa8,
0xacd, 0xaa1, 0xacb, 0x20, 0xaa6, 0xac7, 0x20, 0xaa8, 0xacb, 0xab0, 0xacb, 0xab9,
0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xa89, 0xaa4, 0xacd, 0xaa4, 0xab0, 0x20, 0xaae, 0xabe, 0xab0, 0xabf, 0xaaf,
0xabe, 0xaa8, 0xabe, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd,
0xab8, 0xaa8, 0xacb, 0xab5, 0xab8, 0xabf, 0xaac, 0xabf, 0xab0, 0xacd, 0xab8, 0xacd,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa8, 0xacb, 0xab5, 0xab8, 0xabf, 0xaac, 0xabf,
0xab0, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xaa8, 0xacb, 0xab5, 0xab8, 0xabf, 0xaac, 0xabf, 0xab0, 0xacd, 0xab8,
0xacd, 0xa95, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xa93, 0xaae, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xa93, 0xaae, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xa93, 0xaae, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xa97, 0xacd,
0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xabe, 0xa95,
0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xabe,
0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xabe, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe,
0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xab2, 0xabe, 0xa89, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xabe, 0xaaa,
0xac1, 0xa86, 0x20, 0xaa8, 0xacd, 0xaaf, 0xac1, 0x20, 0xa97, 0xabf, 0xaa8, 0xac0,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7, 0xab0, 0xabe, 0xa97, 0xacd, 0xab5, 0xac7,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7, 0xab0, 0xabe, 0xa97, 0xacd, 0xab5, 0xac7,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7, 0xab0,
0xabe, 0xa97, 0xacd, 0xab5, 0xac7, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaa, 0xac7, 0xab0, 0xac1, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaa, 0xac7, 0xab0, 0xac1, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xaaa, 0xac7, 0xab0, 0xac1, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7,
0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xabf, 0xab2, 0xabf, 0xaaa, 0xabe,
0xa87, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xabf, 0xab2, 0xabf, 0xaaa, 0xabe,
0xa87, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab,
0xabf, 0xab2, 0xabf, 0xaaa, 0xabe, 0xa87, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaab, 0xacb, 0xaa8, 0xabf, 0xa95,
0xacd, 0xab8, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8,
0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xaaa, 0xac0, 0xa8f,
0xab0, 0xac0, 0x20, 0xa85, 0xaa8, 0xac7, 0x20, 0xaae, 0xabf, 0xa95, 0xacd, 0xaaf,
0xac1, 0xab2, 0xacb, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xac7, 0xa82, 0xa9f,
0x20, 0xaaa, 0xac0, 0xa8f, 0xab0, 0xac0, 0x20, 0xa85, 0xaa8, 0xac7, 0x20, 0xaae,
0xabf, 0xa95, 0xacd, 0xaaf, 0xac1, 0xab2, 0xacb, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8,
0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xac7, 0xa82, 0xa9f, 0x20, 0xaaa, 0xac0,
0xa8f, 0xab0, 0xac0, 0x20, 0xa85, 0xaa8, 0xac7, 0x20, 0xaae, 0xabf, 0xa95, 0xacd,
0xaaf, 0xac1, 0xab2, 0xacb, 0xaa8, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8,
0xaae, 0xaaf, 0xaaa, 0xabf, 0xa9f, 0xa95, 0xac7, 0xaaf, 0xab0, 0xacd, 0xaa8, 0x20,
0xab8, 0xaae, 0xaaf, 0xaaa, 0xacb, 0xaa8, 0xaaa, 0xac7, 0x20, 0xab8, 0xaae, 0xaaf,
0xaaa, 0xacd, 0xaaf, 0xacb, 0xa82, 0xa97, 0xaaf, 0xabe, 0xa82, 0xa97, 0x20, 0xab8,
0xaae, 0xaaf, 0xa95, 0xabf, 0xa9d, 0xabf, 0xab2, 0xacb, 0xab0, 0xacd, 0xaa1, 0xabe,
0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xabf, 0xa9d, 0xabf, 0xab2, 0xacb, 0xab0, 0xacd,
0xaa1, 0xabe, 0x20, 0xaaa, 0xacd, 0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4,
0x20, 0xab8, 0xaae, 0xaaf, 0xa95, 0xabf, 0xa9d, 0xabf, 0xab2, 0xacb, 0xab0, 0xacd,
0xaa1, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8,
0xaae, 0xaaf, 0xab0, 0xac0, 0xaaf, 0xac1, 0xaa8, 0xabf, 0xaaf, 0xaa8, 0x20, 0xab8,
0xaae, 0xaaf, 0xab0, 0xacb, 0xaa5, 0xac7, 0xab0, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf,
0xab8, 0xa96, 0xabe, 0xab2, 0xabf, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xa96,
0xabe, 0xab2, 0xabf, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xab8, 0xa96, 0xabe, 0xab2, 0xabf, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0,
0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xaae, 0xabe, 0xab0, 0xabe,
0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xaae, 0xabe, 0xab0, 0xabe, 0x20, 0xaaa, 0xacd,
0xab0, 0xaae, 0xabe, 0xaa3, 0xaad, 0xac2, 0xaa4, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8,
0xaae, 0xabe, 0xab0, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xaae, 0xacb, 0xa86, 0x20, 0xab8, 0xaae, 0xaaf,
0xab8, 0xaae, 0xacb, 0xa86, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xab8, 0xaae, 0xacb, 0xa86, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xac7, 0xab6, 0xab2, 0xacd, 0xab8, 0x20,
0xab8, 0xaae, 0xaaf, 0xab8, 0xabf, 0xa82, 0xa97, 0xabe, 0xaaa, 0xac1, 0xab0, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xab8, 0xacb, 0xab2, 0xacb,
0xaae, 0xaa8, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0xacd, 0xab8,
0x20, 0xab8, 0xaae, 0xaaf, 0xaa6, 0xa95, 0xacd, 0xab7, 0xabf, 0xaa3, 0x20, 0xa9c,
0xacd, 0xaaf, 0xacb, 0xab0, 0xacd, 0xa9c, 0xabf, 0xaaf, 0xabe, 0x20, 0xab8, 0xaae,
0xaaf, 0xab8, 0xac1, 0xab0, 0xac0, 0xaa8, 0xabe, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf,
0xab8, 0xacd, 0xaaf, 0xacb, 0xab5, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xabe,
0xab9, 0xabf, 0xaa4, 0xac0, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xabe, 0xa87, 0xaaa,
0xac7, 0xa87, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xabe, 0xa87, 0xaaa, 0xac7, 0xa87,
0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xabe, 0xa87,
0xaaa, 0xac7, 0xa87, 0x20, 0xaa6, 0xabf, 0xab5, 0xab8, 0x20, 0xab8, 0xaae, 0xaaf,
0xaa4, 0xabe, 0xa9c, 0xac0, 0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20,
0xab8, 0xaae, 0xaaf, 0xa9f, 0xacb, 0xa95, 0xab2, 0xabe, 0xa89, 0x20, 0xab8, 0xaae,
0xaaf, 0xa9f, 0xacb, 0xa82, 0xa97, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9f, 0xacb,
0xa82, 0xa97, 0xabe, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xa9f, 0xacb, 0xa82, 0xa97, 0xabe, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd,
0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9a, 0xac1, 0xa89, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xaa4, 0xac1, 0xab0, 0xacd, 0xa95, 0xaae, 0xac7, 0xaa8, 0xabf, 0xab8, 0xacd,
0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xac1, 0xab0, 0xacd, 0xa95,
0xaae, 0xac7, 0xaa8, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xaae, 0xabe,
0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaa4, 0xac1, 0xab0, 0xacd, 0xa95, 0xaae,
0xac7, 0xaa8, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xa9f, 0xab5, 0xabe, 0xab2,
0xac2, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xab0, 0xac1, 0xa97, 0xacd, 0xab5, 0xac7,
0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xab0, 0xac1, 0xa97, 0xacd, 0xab5, 0xac7, 0x20,
0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xab0, 0xac1, 0xa97,
0xacd, 0xab5, 0xac7, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20,
0xab8, 0xaae, 0xaaf, 0xa89, 0xa9d, 0xacd, 0xaac, 0xac7, 0xa95, 0xabf, 0xab8, 0xacd,
0xaa4, 0xabe, 0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xa9d, 0xacd, 0xaac, 0xac7,
0xa95, 0xabf, 0xab8, 0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95,
0x20, 0xab8, 0xaae, 0xaaf, 0xa89, 0xa9d, 0xacd, 0xaac, 0xac7, 0xa95, 0xabf, 0xab8,
0xacd, 0xaa4, 0xabe, 0xaa8, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xaa8, 0xabe, 0xaa4, 0xac2, 0x20, 0xab8, 0xaae,
0xaaf, 0xab5, 0xaa8, 0xabe, 0xaa4, 0xac2, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20,
0xab8, 0xaae, 0xaaf, 0xab5, 0xaa8, 0xabe, 0xaa4, 0xac2, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xac7, 0xaa8, 0xac7,
0xa9d, 0xac1, 0xa8f, 0xab2, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xacd, 0xab2,
0xabe, 0xaa1, 0xabf, 0xab5, 0xacb, 0xab8, 0xacd, 0xa9f, 0xacb, 0xa95, 0x20, 0xab8,
0xaae, 0xaaf, 0xab5, 0xacd, 0xab2, 0xabe, 0xaa1, 0xabf, 0xab5, 0xacb, 0xab8, 0xacd,
0xa9f, 0xacb, 0xa95, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xab5, 0xacd, 0xab2, 0xabe, 0xaa1, 0xabf, 0xab5, 0xacb, 0xab8, 0xacd, 0xa9f, 0xacb,
0xa95, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae,
0xaaf, 0xab5, 0xacb, 0xab2, 0xacd, 0xa97, 0xacb, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa1,
0x20, 0xab8, 0xaae, 0xaaf, 0xab5, 0xacb, 0xab2, 0xacd, 0xa97, 0xacb, 0xa97, 0xacd,
0xab0, 0xac7, 0xaa1, 0x20, 0xaae, 0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf,
0xab5, 0xacb, 0xab2, 0xacd, 0xa97, 0xacb, 0xa97, 0xacd, 0xab0, 0xac7, 0xaa1, 0x20,
0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xab5,
0xac5, 0xa95, 0x20, 0xa86, 0xa87, 0xab2, 0xac7, 0xaa8, 0xacd, 0xaa1, 0x20, 0xab8,
0xaae, 0xaaf, 0xab5, 0xac9, 0xab2, 0xabf, 0xab8, 0x20, 0xa85, 0xaa8, 0xac7, 0x20,
0xaab, 0xacd, 0xaaf, 0xac1, 0xa9a, 0xac1, 0xaa8, 0xabe, 0x20, 0xab8, 0xaae, 0xaaf,
0xaaf, 0xabe, 0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xab8, 0xaae,
0xaaf, 0xaaf, 0xabe, 0xa95, 0xac1, 0xaa4, 0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xabe, 0xa95, 0xac1, 0xaa4,
0xacd, 0xab8, 0xacd, 0xa95, 0x20, 0xa97, 0xacd, 0xab0, 0xac0, 0xab7, 0xacd, 0xaae,
0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xac7, 0xa95, 0xabe, 0xa9f, 0xac7, 0xab0, 0xabf,
0xaa8, 0xaac, 0xab0, 0xacd, 0xa97, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xac7, 0xa95,
0xabe, 0xa9f, 0xac7, 0xab0, 0xabf, 0xaa8, 0xaac, 0xab0, 0xacd, 0xa97, 0x20, 0xaae,
0xabe, 0xaa8, 0xa95, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xac7, 0xa95, 0xabe, 0xa9f,
0xac7, 0xab0, 0xabf, 0xaa8, 0xaac, 0xab0, 0xacd, 0xa97, 0x20, 0xa97, 0xacd, 0xab0,
0xac0, 0xab7, 0xacd, 0xaae, 0x20, 0xab8, 0xaae, 0xaaf, 0xaaf, 0xac1, 0xa95, 0xacb,
0xaa8, 0x20, 0xab8, 0xaae, 0xaaf, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x69, 0x72,
0x6b, 0x61, 0x20, 0x74, 0x61, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62,
0x61, 0x73, 0x68, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x69, 0x72, 0x6b, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x69, 0x72,
0x6b, 0x61, 0x20, 0x74, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x69, 0x72, 0x6b, 0x61, 0x20,
0x74, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x41, 0x66, 0x69, 0x72, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x20,
0x59, 0x61, 0x6d, 0x6d, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x20, 0x64, 0x61, 0x6b, 0x65, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x61, 0x72, 0x65, 0x77, 0x61, 0x20, 0x74, 0x61, 0x20,
0x74, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x20, 0x64, 0x61, 0x6b, 0x65, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74,
0x61, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x64, 0x61,
0x6b, 0x65, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x20, 0x54,
0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x20, 0x64, 0x61, 0x6b, 0x65,
0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d,
0x75, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x47, 0x61, 0x62, 0x61, 0x73, 0x20, 0x64, 0x61, 0x6b, 0x65, 0x20, 0x41,
0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72,
0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52,
0x61, 0x6e, 0x61, 0x20, 0x74, 0x61, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73,
0x20, 0x64, 0x61, 0x6b, 0x65, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x73, 0x61, 0x75, 0x6e, 0x69,
0x20, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x6d, 0x75, 0x72, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x20, 0x74, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x20, 0x6e,
0x61, 0x20, 0x74, 0x73, 0x61, 0x75, 0x6e, 0x69, 0x20, 0x61, 0x20, 0x41,
0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72,
0x69, 0x63, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52,
0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x73, 0x61, 0x75, 0x6e,
0x69, 0x6e, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x41, 0x6d, 0x75, 0x72, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41,
0x6d, 0x75, 0x72, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x20, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x20, 0x6e, 0x61,
0x20, 0x41, 0x72, 0x65, 0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d,
0x75, 0x72, 0x69, 0x6b, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x65,
0x77, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x6d, 0x75, 0x72, 0x6b, 0x61,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x70, 0x69, 0x61,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x41, 0x72, 0x65, 0x62, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x41, 0x72, 0x65, 0x62, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20,
0x6e, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a,
0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x61,
0x6e, 0x61, 0x64, 0x61, 0x2c, 0x20, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f,
0x20, 0x52, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x56, 0x69, 0x72,
0x67, 0x69, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x2c,
0x20, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f, 0x20, 0x52, 0x69, 0x63, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x56, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4b, 0x61,
0x6e, 0x61, 0x64, 0x61, 0x2c, 0x20, 0x50, 0x75, 0x65, 0x72, 0x74, 0x6f,
0x20, 0x52, 0x69, 0x63, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x56, 0x69, 0x72,
0x67, 0x69, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61,
0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61,
0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d,
0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79,
0x61, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61,
0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x41,
0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68,
0x69, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61,
0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68,
0x69, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61,
0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59,
0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74,
0x69, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x59, 0x61,
0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x69,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42,
0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20,
0x44, 0x61, 0x69, 0x64, 0x61, 0x69, 0x74, 0x61, 0x63, 0x63, 0x65, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x41, 0x7a, 0x75, 0x72, 0x65,
0x20, 0x6e, 0x61, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x6c, 0x69, 0x61, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x6c, 0x69, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x6c,
0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61,
0x6c, 0x61, 0x6d, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43,
0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20,
0x6e, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x6f, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x53, 0x69, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x69,
0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x69, 0x6e, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74,
0x6d, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a,
0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x52, 0x61, 0x62, 0x69, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20,
0x6e, 0x61, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x4b, 0x75, 0x62, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4b, 0x75, 0x62,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x76,
0x69, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x45, 0x61,
0x73, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x54, 0x73,
0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x61, 0x20, 0x6c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x49, 0x64, 0x61,
0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x61, 0x20,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x54, 0x75, 0x72, 0x61, 0x69,
0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x72, 0x20, 0x62, 0x61, 0x7a,
0x61, 0x72, 0x61, 0x20, 0x61, 0x20, 0x6c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x20, 0x61, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x20, 0x67, 0x61, 0x62,
0x61, 0x73, 0x49, 0x64, 0x61, 0x20, 0x6c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x20, 0x61, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x20, 0x67, 0x61, 0x62,
0x61, 0x73, 0x47, 0x61, 0x62, 0x61, 0x73, 0x20, 0x61, 0x20, 0x6c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x20, 0x64,
0x61, 0x20, 0x62, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73, 0x68, 0x69, 0x6e,
0x20, 0x54, 0x75, 0x72, 0x61, 0x69, 0x20, 0x6d, 0x61, 0x69, 0x20, 0x4e,
0x69, 0x73, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x74, 0x61,
0x20, 0x79, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x74, 0x75,
0x72, 0x61, 0x69, 0x49, 0x64, 0x61, 0x20, 0x6c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x20, 0x74, 0x61, 0x20, 0x79, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x49, 0x64, 0x61, 0x20, 0x6c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x74, 0x61, 0x20, 0x79, 0x61, 0x6d,
0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x74, 0x75, 0x72, 0x61, 0x69, 0x20,
0x64, 0x61, 0x20, 0x62, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x46, 0x69, 0x6a, 0x69, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42,
0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x46, 0x72, 0x65,
0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x75, 0x64, 0x61, 0x6e, 0x63,
0x69, 0x6e, 0x20, 0x46, 0x61, 0x72, 0x61, 0x6e, 0x73, 0x61, 0x20, 0x64,
0x61, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x68, 0x69, 0x63, 0x68, 0x20, 0x61, 0x20, 0x4c, 0x6f, 0x6e, 0x64, 0x6f,
0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62,
0x61, 0x73, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x62,
0x61, 0x73, 0x68, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52,
0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x61, 0x62, 0x61, 0x73,
0x68, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d,
0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d,
0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52,
0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x6f, 0x6c,
0x66, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x6e, 0x61,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54,
0x65, 0x6b, 0x75, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63,
0x68, 0x69, 0x6e, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x74, 0x61,
0x20, 0x54, 0x73, 0x61, 0x6b, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x49, 0x72, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x49, 0x73, 0x72, 0x61, 0x2bc, 0x69, 0x6c,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x61,
0x62, 0x61, 0x73, 0x68, 0x69, 0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x59, 0x61, 0x6d, 0x6d, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4b, 0x6f, 0x72, 0x69, 0x79, 0x61, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4b, 0x6f, 0x72, 0x69, 0x79, 0x61, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x4b, 0x6f, 0x72, 0x69, 0x79, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x56, 0x6f, 0x74, 0x65, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x6c, 0x61,
0x79, 0x73, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61,
0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20,
0x6e, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x77, 0x73,
0x6f, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x65,
0x6b, 0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x65, 0x6b, 0x7a, 0x69,
0x6b, 0x6f, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x4d, 0x65, 0x6b, 0x7a, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20,
0x6e, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61,
0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x20, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x20, 0x74, 0x61, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x20, 0x72, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x61,
0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x69, 0x75,
0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a,
0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e,
0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61,
0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e,
0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x50, 0x65, 0x72, 0x75, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x50, 0x65, 0x72, 0x75, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x50,
0x65, 0x72, 0x75, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50,
0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x54, 0x73, 0x61,
0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a,
0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64,
0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x54, 0x73,
0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61,
0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79,
0x61, 0x6e, 0x67, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52,
0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42,
0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x56, 0x6f, 0x74,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65,
0x73, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x72, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x52, 0x61, 0x6e, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x6f, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x4b,
0x75, 0x64, 0x61, 0x6e, 0x63, 0x69, 0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x52, 0x61, 0x6e, 0x61,
0x20, 0x6e, 0x61, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x43,
0x68, 0x75, 0x75, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f,
0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61,
0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x54, 0x73, 0x61, 0x79, 0x61,
0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79,
0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4c, 0x6f, 0x6b, 0x61,
0x63, 0x69, 0x6e, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c,
0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x54, 0x73, 0x61, 0x79,
0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a,
0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x54,
0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b,
0x61, 0x63, 0x69, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61,
0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e,
0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x64, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79,
0x65, 0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72, 0x61, 0x20, 0x6e, 0x61, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4c, 0x6f, 0x6b, 0x61, 0x63,
0x69, 0x6e, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x54, 0x73, 0x61, 0x79, 0x61, 0x79, 0x79, 0x65,
0x6e, 0x20, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4c,
0x6f, 0x6b, 0x61, 0x63, 0x69, 0x6e, 0x20, 0x42, 0x61, 0x7a, 0x61, 0x72,
0x61, 0x20, 0x6e, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4c, 0x6f, 0x6b, 0x61, 0x63, 0x69,
0x6e, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5e4, 0x5d2, 0x5e0, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5e4, 0x5e8, 0x5d9, 0x5e7, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0, 0x5e4,
0x5e8, 0x5d9, 0x5e7, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d3, 0x5e8, 0x5d5,
0x5dd, 0x20, 0x5d0, 0x5e4, 0x5e8, 0x5d9, 0x5e7, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e4, 0x5e8, 0x5d9, 0x5e7, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e4,
0x5e8, 0x5d9, 0x5e7, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e4, 0x5e8,
0x5d9, 0x5e7, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5dc, 0x5e1, 0x5e7, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5dc, 0x5e1, 0x5e7, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5dc, 0x5e1, 0x5e7, 0x5d4, 0x20, 0x28,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5dc, 0x5de,
0x5d8, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5dc, 0x5de, 0x5d8, 0x5d9,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5dc, 0x5de, 0x5d8, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5de, 0x5d6, 0x5d5, 0x5e0, 0x5e1, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5de, 0x5d6, 0x5d5, 0x5e0, 0x5e1, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5de, 0x5d6,
0x5d5, 0x5e0, 0x5e1, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5e8,
0x5d4, 0x5f4, 0x5d1, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5e8, 0x5d4, 0x5f4,
0x5d1, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d4, 0x5d7, 0x5d5, 0x5e3, 0x20, 0x5d4, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x5d9, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d7, 0x5d5, 0x5e3, 0x20, 0x5d4, 0x5de, 0x5d6,
0x5e8, 0x5d7, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d4, 0x5d7, 0x5d5, 0x5e3, 0x20, 0x5d4, 0x5de, 0x5d6, 0x5e8,
0x5d7, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x20, 0x5d4, 0x5d4, 0x5e8, 0x5d9, 0x5dd, 0x20,
0x5d1, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5d6, 0x5d5, 0x5e8, 0x20, 0x5d4, 0x5d4, 0x5e8, 0x5d9, 0x5dd, 0x20, 0x5d1, 0x5d0,
0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x20, 0x5d4, 0x5d4, 0x5e8,
0x5d9, 0x5dd, 0x20, 0x5d1, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1,
0x20, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de,
0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8,
0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d4, 0x5f4, 0x5d1, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e0, 0x5d3, 0x5d9, 0x5e8, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e8, 0x5d2, 0x5d9, 0x5dc, 0x20, 0x5d0, 0x5e0, 0x5d3,
0x5d9, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5e5, 0x20, 0x5d0,
0x5e0, 0x5d3, 0x5d9, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e4, 0x5d9,
0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e4, 0x5d9, 0x5d4, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e4,
0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5e7, 0x5d8, 0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5e7, 0x5d8, 0x5d0, 0x5d5, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e7, 0x5d8, 0x5d0, 0x5d5, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e7, 0x5d8, 0x5d5,
0x5d1, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5e7, 0x5d8, 0x5d5,
0x5d1, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5d5, 0x5e7, 0x5d8, 0x5d5, 0x5d1, 0x5d4, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7, 0x5e6, 0x5d9, 0x20,
0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e2, 0x5e8, 0x5d1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d7, 0x5e6, 0x5d9, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e2, 0x5e8, 0x5d1, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7,
0x5e6, 0x5d9, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x28,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e8, 0x5d2,
0x5e0, 0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e8,
0x5d2, 0x5e0, 0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e8, 0x5d2, 0x5e0, 0x5d8, 0x5d9,
0x5e0, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5e8, 0x5d2, 0x5e0, 0x5d8, 0x5d9,
0x5e0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20,
0x5d0, 0x5e8, 0x5d2, 0x5e0, 0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1,
0x20, 0x5d0, 0x5e8, 0x5d2, 0x5e0, 0x5d8, 0x5d9, 0x5e0, 0x5d4, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e8, 0x5de, 0x5e0,
0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e8, 0x5de, 0x5e0, 0x5d9,
0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5e8, 0x5de, 0x5e0, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x5e7, 0x5d9, 0x5d9,
0x5e0, 0x5d5, 0x5e1, 0x20, 0x5d4, 0x5d0, 0x5d8, 0x5dc, 0x5e0, 0x5d8, 0x5d9, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x5e7, 0x5d9, 0x5d9, 0x5e0, 0x5d5,
0x5e1, 0x20, 0x5d4, 0x5d0, 0x5d8, 0x5dc, 0x5e0, 0x5d8, 0x5d9, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d5,
0x5e7, 0x5d9, 0x5d9, 0x5e0, 0x5d5, 0x5e1, 0x20, 0x5d4, 0x5d0, 0x5d8, 0x5dc, 0x5e0,
0x5d8, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc,
0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20,
0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6,
0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6,
0x2d, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc,
0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x2d,
0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9,
0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x2d, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0,
0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0,
0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9,
0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc,
0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc,
0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20,
0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1,
0x20, 0x5d0, 0x5d5, 0x5e1, 0x5d8, 0x5e8, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d6, 0x5e8, 0x5d1,
0x5d9, 0x5d9, 0x5d2, 0x5f3, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d6,
0x5e8, 0x5d1, 0x5d9, 0x5d9, 0x5d2, 0x5f3, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d6, 0x5e8, 0x5d1, 0x5d9,
0x5d9, 0x5d2, 0x5f3, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5d0, 0x5d6,
0x5d5, 0x5e8, 0x5d9, 0x5d9, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0,
0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x5d9, 0x5d9, 0x5dd,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5d0, 0x5d6, 0x5d5, 0x5e8, 0x5d9,
0x5d9, 0x5dd, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d1, 0x5e0, 0x5d2, 0x5dc, 0x5d3, 0x5e9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d1, 0x5e0, 0x5d2, 0x5dc, 0x5d3, 0x5e9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1, 0x5e0, 0x5d2, 0x5dc, 0x5d3, 0x5e9,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1,
0x5d4, 0x5d5, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1, 0x5d5, 0x5dc,
0x5d9, 0x5d1, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1, 0x5e8, 0x5d6,
0x5d9, 0x5dc, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1, 0x5e8, 0x5d6,
0x5d9, 0x5dc, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1, 0x5e8, 0x5d6, 0x5d9, 0x5dc, 0x5d9, 0x5d4, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d1, 0x5e8,
0x5d5, 0x5e0, 0x5d9, 0x5d9, 0x20, 0x5d3, 0x5d0, 0x5e8, 0x5d5, 0x5e1, 0x5dc, 0x5d0,
0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5db, 0x5e3, 0x20, 0x5d5, 0x5e8, 0x5d3,
0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5db, 0x5e3, 0x20, 0x5d5, 0x5e8, 0x5d3,
0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5db, 0x5e3, 0x20, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9,
0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d0, 0x5de, 0x5d5,
0x5e8, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d8, 0x5d4, 0x5d0,
0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d8, 0x5d4, 0x5d0, 0x5dd,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e6, 0x5f3, 0x5d8, 0x5d4, 0x5d0, 0x5dd, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d9, 0x5dc, 0x5d4, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d9, 0x5dc, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3, 0x5d9, 0x5dc,
0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e1, 0x5d9, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d9, 0x5df, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1,
0x5d9, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e6, 0x5f3, 0x5d5, 0x5d9, 0x5d1, 0x5dc, 0x5e1, 0x5df, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e6, 0x5f3, 0x5d5, 0x5d9, 0x5d1, 0x5dc, 0x5e1, 0x5df, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3,
0x5d5, 0x5d9, 0x5d1, 0x5dc, 0x5e1, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5db, 0x5e8, 0x5d9,
0x5e1, 0x5d8, 0x5de, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9,
0x20, 0x5e7, 0x5d5, 0x5e7, 0x5d5, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7,
0x5d5, 0x5dc, 0x5d5, 0x5de, 0x5d1, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e7, 0x5d5, 0x5dc, 0x5d5, 0x5de, 0x5d1, 0x5d9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5dc, 0x5d5,
0x5de, 0x5d1, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e7, 0x5d5, 0x5e7, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9,
0x5d9, 0x20, 0x5e7, 0x5d5, 0x5e7, 0x20, 0x28, 0x5de, 0x5d7, 0x5e6, 0x5d9, 0x5ea,
0x20, 0x5d4, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7,
0x5d5, 0x5d1, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5d1, 0x5d4,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e7, 0x5d5, 0x5d1, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5d9, 0x5e1, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d3, 0x5d5, 0x5de, 0x5d5, 0x5df, 0x20, 0x5d3, 0x5f3, 0x5d0, 0x5d5,
0x5e8, 0x5d5, 0x5d5, 0x5d9, 0x5dc, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6,
0x5e8, 0x5d7, 0x20, 0x5d8, 0x5d9, 0x5de, 0x5d5, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5d9, 0x20, 0x5d4, 0x5e4, 0x5e1, 0x5d7, 0x5d0, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5d9, 0x20, 0x5d4, 0x5e4, 0x5e1, 0x5d7, 0x5d0, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9,
0x20, 0x5d4, 0x5e4, 0x5e1, 0x5d7, 0x5d0, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5e7, 0x5d5, 0x5d5, 0x5d3, 0x5d5, 0x5e8,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6, 0x20, 0x5d0, 0x5d9,
0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db,
0x5d6, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6,
0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0,
0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6,
0x5e8, 0x5d7, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8,
0x5d7, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9,
0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d9, 0x5e0, 0x5e1, 0x5e7,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d9,
0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8,
0x5d1, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1,
0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d5, 0x5e4, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e4, 0x5d5,
0x5e7, 0x5dc, 0x5e0, 0x5d3, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9,
0x20, 0x5e4, 0x5d5, 0x5e7, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e4,
0x5d5, 0x5e7, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5d9, 0x5d2, 0x5f3, 0x5d9, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e4, 0x5d9, 0x5d2, 0x5f3, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5d9, 0x5d2, 0x5f3, 0x5d9,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2,
0x5d9, 0x5d0, 0x5e0, 0x5d4, 0x20, 0x5d4, 0x5e6, 0x5e8, 0x5e4, 0x5ea, 0x5d9, 0x5ea,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5e8, 0x5e6, 0x5d5, 0x5ea, 0x20,
0x5d4, 0x5d3, 0x5e8, 0x5d5, 0x5de, 0x5d9, 0x5d5, 0x5ea, 0x20, 0x5d5, 0x5d4, 0x5d0,
0x5e0, 0x5d8, 0x5d0, 0x5e8, 0x5e7, 0x5d8, 0x5d9, 0x5d5, 0x5ea, 0x20, 0x5e9, 0x5dc,
0x20, 0x5e6, 0x5e8, 0x5e4, 0x5ea, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9,
0x5d9, 0x20, 0x5d2, 0x5dc, 0x5d0, 0x5e4, 0x5d2, 0x5d5, 0x5e1, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5d2, 0x5de, 0x5d1, 0x5d9, 0x5d9, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2, 0x5d0, 0x5d5, 0x5e8, 0x5d2, 0x5d9, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2, 0x5d0, 0x5d5, 0x5e8, 0x5d2, 0x5d9, 0x5d4,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d2, 0x5d0, 0x5d5, 0x5e8, 0x5d2, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5d2, 0x5d9,
0x5dc, 0x5d1, 0x5e8, 0x5d8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2, 0x5e8, 0x5d9,
0x5e0, 0x5d9, 0x5e5, 0x5f3, 0x200f, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d6,
0x5e8, 0x5d7, 0x20, 0x5d2, 0x5e8, 0x5d9, 0x5e0, 0x5dc, 0x5e0, 0x5d3, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d2, 0x5e8, 0x5d9, 0x5e0,
0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d2, 0x5e8, 0x5d9, 0x5e0,
0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5d2, 0x5e8, 0x5d9, 0x5e0, 0x5dc,
0x5e0, 0x5d3, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20,
0x5d2, 0x5e8, 0x5d9, 0x5e0, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20,
0x5d2, 0x5e8, 0x5d9, 0x5e0, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d2, 0x5d5, 0x5d0, 0x5dd, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5d3, 0x5d9, 0x5e0, 0x5d5, 0x5ea, 0x20, 0x5d4, 0x5de,
0x5e4, 0x5e8, 0x5e5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x5d9,
0x5dd, 0x20, 0x5d4, 0x5d0, 0x5dc, 0x5d0, 0x5d5, 0x5d8, 0x5d9, 0x5d9, 0x5dd, 0x20,
0x5d4, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0,
0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5d0, 0x5dc, 0x5d0, 0x5d5, 0x5d8, 0x5d9, 0x5d9,
0x5dd, 0x20, 0x5d4, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd,
0x20, 0x5d4, 0x5d0, 0x5dc, 0x5d0, 0x5d5, 0x5d8, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4,
0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d4, 0x5d5, 0x5e0, 0x5d2, 0x20, 0x5e7, 0x5d5, 0x5e0, 0x5d2,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d5, 0x5e0, 0x5d2, 0x20, 0x5e7, 0x5d5,
0x5e0, 0x5d2, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d4, 0x5d5, 0x5e0, 0x5d2, 0x20, 0x5e7, 0x5d5, 0x5e0, 0x5d2, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7, 0x5d5,
0x5d1, 0x5d3, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7, 0x5d5, 0x5d1, 0x5d3, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7,
0x5d5, 0x5d1, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d4, 0x5d5, 0x5d3, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4,
0x5d0, 0x5d5, 0x5e7, 0x5d9, 0x5d9, 0x5e0, 0x5d5, 0x5e1, 0x20, 0x5d4, 0x5d4, 0x5d5,
0x5d3, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d5, 0x5d3, 0x5d5, 0x2d,
0x5e1, 0x5d9, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e8, 0x5db, 0x5d6,
0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d5, 0x5e0, 0x5d6, 0x5d9, 0x5d4, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3,
0x5d5, 0x5e0, 0x5d6, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2,
0x5e8, 0x5d1, 0x20, 0x5d0, 0x5d9, 0x5e0, 0x5d3, 0x5d5, 0x5e0, 0x5d6, 0x5d9, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d0, 0x5df, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d0, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5e8, 0x5d0,
0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5d9, 0x5e8, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5d9, 0x5e8, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5e8,
0x5e7, 0x5d5, 0x5e1, 0x5d8, 0x5e7, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e9, 0x5e8, 0x5d0, 0x5dc, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d9, 0x5e9, 0x5e8, 0x5d0, 0x5dc, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e9, 0x5e8, 0x5d0, 0x5dc,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9,
0x5e4, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e4, 0x5df, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e4,
0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e4, 0x5d8, 0x5e8, 0x5d5, 0x5e4, 0x5d1, 0x5dc, 0x5d5, 0x5d1, 0x5e1, 0x5e7, 0x2d,
0x5e7, 0x5de, 0x5e6, 0x5f3, 0x5d8, 0x5e1, 0x5e7, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e8, 0x5d2, 0x5d9, 0x5dc, 0x20, 0x5e4, 0x5d8, 0x5e8, 0x5d5, 0x5e4, 0x5d1,
0x5dc, 0x5d5, 0x5d1, 0x5e1, 0x5e7, 0x2d, 0x5e7, 0x5de, 0x5e6, 0x5f3, 0x5d8, 0x5e1,
0x5e7, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5e5, 0x20, 0x5e4,
0x5d8, 0x5e8, 0x5d5, 0x5e4, 0x5d1, 0x5dc, 0x5d5, 0x5d1, 0x5e1, 0x5e7, 0x2d, 0x5e7,
0x5de, 0x5e6, 0x5f3, 0x5d8, 0x5e1, 0x5e7, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5de, 0x5d6, 0x5e8, 0x5d7, 0x20, 0x5e7, 0x5d6, 0x5d7, 0x5e1, 0x5d8, 0x5df, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5e7, 0x5d6, 0x5d7,
0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5e8, 0x5d9,
0x5d0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5e8, 0x5d9, 0x5d0,
0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e7, 0x5d5, 0x5e8, 0x5d9, 0x5d0, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d5, 0x5e1, 0x5e8, 0x5d0, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5e8, 0x5e1, 0x5e0, 0x5d5, 0x5d9, 0x5d0,
0x5e8, 0x5e1, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5e8, 0x5e1, 0x5e0,
0x5d5, 0x5d9, 0x5d0, 0x5e8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5e8, 0x5e1, 0x5e0, 0x5d5, 0x5d9,
0x5d0, 0x5e8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5e8, 0x5d2, 0x5d9, 0x5d6, 0x5e1, 0x5d8, 0x5df,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5dc, 0x5d9, 0x5d9,
0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x20, 0x5d4, 0x5dc, 0x5d5,
0x5e8, 0x5d3, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5d9, 0x20, 0x5d4, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5d9, 0x20, 0x5d4, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x20, 0x5d4, 0x5d0, 0x5d5, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e7,
0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x20,
0x5de, 0x5e7, 0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5e5,
0x20, 0x5de, 0x5e7, 0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d2,
0x5d3, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d2, 0x5d3, 0x5df, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de,
0x5d2, 0x5d3, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5de, 0x5dc, 0x5d6, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d4, 0x5d0, 0x5d9, 0x5d9, 0x5dd, 0x20, 0x5d4, 0x5de, 0x5dc, 0x5d3, 0x5d9, 0x5d1,
0x5d9, 0x5d9, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20,
0x5de, 0x5e8, 0x5e7, 0x5d9, 0x5d6, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9,
0x5d9, 0x20, 0x5de, 0x5e8, 0x5e9, 0x5dc, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de,
0x5d0, 0x5d5, 0x5e8, 0x5d9, 0x5e6, 0x5d9, 0x5d5, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5de, 0x5d0, 0x5d5, 0x5e8, 0x5d9, 0x5e6, 0x5d9, 0x5d5, 0x5e1, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d0,
0x5d5, 0x5e8, 0x5d9, 0x5e6, 0x5d9, 0x5d5, 0x5e1, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d0, 0x5d5, 0x5e1, 0x5d5, 0x5df,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1, 0x20, 0x5de, 0x5e7,
0x5e1, 0x5d9, 0x5e7, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8,
0x5d1, 0x20, 0x5de, 0x5e7, 0x5e1, 0x5d9, 0x5e7, 0x5d5, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5e2, 0x5e8, 0x5d1,
0x20, 0x5de, 0x5e7, 0x5e1, 0x5d9, 0x5e7, 0x5d5, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5dc, 0x5d0, 0x5df, 0x20,
0x5d1, 0x5d0, 0x5d8, 0x5d5, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5,
0x5dc, 0x5d0, 0x5df, 0x20, 0x5d1, 0x5d0, 0x5d8, 0x5d5, 0x5e8, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5dc,
0x5d0, 0x5df, 0x20, 0x5d1, 0x5d0, 0x5d8, 0x5d5, 0x5e8, 0x20, 0x28, 0x5e7, 0x5d9,
0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d5, 0x5e1, 0x5e7, 0x5d1,
0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d5, 0x5e1, 0x5e7, 0x5d1, 0x5d4,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5de, 0x5d5, 0x5e1, 0x5e7, 0x5d1, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5de, 0x5d9, 0x5d0, 0x5e0, 0x5de, 0x5e8, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e0, 0x5e4, 0x5d0, 0x5dc, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7,
0x5dc, 0x5d3, 0x5d5, 0x5e0, 0x5d9, 0x5d4, 0x20, 0x5d4, 0x5d7, 0x5d3, 0x5e9, 0x5d4,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5dc, 0x5d3, 0x5d5, 0x5e0, 0x5d9, 0x5d4,
0x20, 0x5d4, 0x5d7, 0x5d3, 0x5e9, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5dc, 0x5d3, 0x5d5, 0x5e0, 0x5d9,
0x5d4, 0x20, 0x5d4, 0x5d7, 0x5d3, 0x5e9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x20, 0x5d6, 0x5d9,
0x5dc, 0x5e0, 0x5d3, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x20,
0x5d6, 0x5d9, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x20, 0x5d6, 0x5d9, 0x5dc,
0x5e0, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e0, 0x5d9, 0x5d5, 0x5e4, 0x5d0, 0x5d5, 0x5e0, 0x5d3, 0x5dc, 0x5e0, 0x5d3,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x5e4, 0x5d0, 0x5d5, 0x5e0,
0x5d3, 0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x5e4, 0x5d0, 0x5d5, 0x5e0, 0x5d3,
0x5dc, 0x5e0, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e0, 0x5d9, 0x5d5, 0x5d0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5e4, 0x5d5, 0x5e7, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5e4, 0x5d5,
0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5e4, 0x5d5, 0x5e7, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8,
0x5e0, 0x5e0, 0x5d3, 0x5d5, 0x20, 0x5d3, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5d5,
0x5e0, 0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5e0, 0x5e0,
0x5d3, 0x5d5, 0x20, 0x5d3, 0x5d9, 0x20, 0x5e0, 0x5d5, 0x5e8, 0x5d5, 0x5e0, 0x5d9,
0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e4, 0x5e8, 0x5e0, 0x5e0, 0x5d3, 0x5d5, 0x20, 0x5d3, 0x5d9, 0x20, 0x5e0,
0x5d5, 0x5e8, 0x5d5, 0x5e0, 0x5d9, 0x5d4, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5de, 0x5e8, 0x5d9,
0x5d0, 0x5e0, 0x5d4, 0x20, 0x5d4, 0x5e6, 0x5e4, 0x5d5, 0x5e0, 0x5d9, 0x5d9, 0x5dd,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d5, 0x5d1, 0x5d5, 0x5e1, 0x5d9, 0x5d1,
0x5d9, 0x5e8, 0x5e1, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d5, 0x5d1,
0x5d5, 0x5e1, 0x5d9, 0x5d1, 0x5d9, 0x5e8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e0, 0x5d5, 0x5d1, 0x5d5,
0x5e1, 0x5d9, 0x5d1, 0x5d9, 0x5e8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5de, 0x5e1, 0x5e7, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5de, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5de,
0x5e1, 0x5e7, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e4, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e4, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4,
0x5dc, 0x5d0, 0x5d5, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e4, 0x5d5, 0x5d0,
0x5d4, 0x20, 0x5d2, 0x5d9, 0x5e0, 0x5d9, 0x5d0, 0x5d4, 0x20, 0x5d4, 0x5d7, 0x5d3,
0x5e9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d2, 0x5d5, 0x5d5,
0x5d0, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d2, 0x5d5, 0x5d5,
0x5d0, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e4, 0x5e8, 0x5d2, 0x5d5, 0x5d5, 0x5d0, 0x5d9, 0x20, 0x28, 0x5e7,
0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d5, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d5, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8,
0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e4, 0x5e8, 0x5d5, 0x20, 0x28,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5e4, 0x5d9,
0x5dc, 0x5d9, 0x5e4, 0x5d9, 0x5e0, 0x5d9, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d4, 0x5e4, 0x5d9, 0x5dc, 0x5d9, 0x5e4, 0x5d9, 0x5e0, 0x5d9, 0x5dd, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5e4,
0x5d9, 0x5dc, 0x5d9, 0x5e4, 0x5d9, 0x5e0, 0x5d9, 0x5dd, 0x20, 0x28, 0x5e7, 0x5d9,
0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e4,
0x5d9, 0x5e0, 0x5d9, 0x5e7, 0x5e1, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5e0,
0x5d8, 0x20, 0x5e4, 0x5d9, 0x5d9, 0x5e8, 0x20, 0x5d5, 0x5de, 0x5d9, 0x5e7, 0x5dc,
0x5d5, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5e0, 0x5d8, 0x20, 0x5e4,
0x5d9, 0x5d9, 0x5e8, 0x20, 0x5d5, 0x5de, 0x5d9, 0x5e7, 0x5dc, 0x5d5, 0x5df, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1,
0x5e0, 0x5d8, 0x20, 0x5e4, 0x5d9, 0x5d9, 0x5e8, 0x20, 0x5d5, 0x5de, 0x5d9, 0x5e7,
0x5dc, 0x5d5, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e4, 0x5d9, 0x5d8, 0x5e7, 0x5e8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e4, 0x5d5, 0x5e0, 0x5d0, 0x5e4, 0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e4, 0x5d9, 0x5d5, 0x5e0, 0x5d2, 0x5d9, 0x5d0, 0x5e0, 0x5d2, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e7, 0x5d9, 0x5d6, 0x5d9, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5e7, 0x5d9, 0x5d6, 0x5d9, 0x5dc, 0x5d5, 0x5e8, 0x5d3,
0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e7, 0x5d9, 0x5d6, 0x5d9, 0x5dc, 0x5d5, 0x5e8, 0x5d3, 0x5d4, 0x20, 0x28,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e8, 0x5d0, 0x5d5,
0x5e0, 0x5d9, 0x5d5, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e8, 0x5d5, 0x5ea,
0x5f3, 0x5e8, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d7, 0x5dc, 0x5d9,
0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d7, 0x5dc, 0x5d9, 0x5df, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1,
0x5d7, 0x5dc, 0x5d9, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e1, 0x5de, 0x5e8, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e8, 0x5d2, 0x5d9, 0x5dc, 0x20, 0x5e1, 0x5de, 0x5e8, 0x5d4, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5e7, 0x5d9, 0x5e5, 0x20, 0x5e1, 0x5de, 0x5e8, 0x5d4, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5e1, 0x5de, 0x5d5, 0x5d0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5e1, 0x5de, 0x5d5, 0x5d0, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5de, 0x5d5, 0x5d0, 0x5d4, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d9,
0x5d9, 0x20, 0x5e1, 0x5d9, 0x5d9, 0x5e9, 0x5dc, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5e1, 0x5d9, 0x5e0, 0x5d2, 0x5e4, 0x5d5, 0x5e8, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d0, 0x5d9, 0x5d9, 0x20, 0x5e9, 0x5dc, 0x5de, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d3, 0x5e8, 0x5d5, 0x5dd, 0x20, 0x5d2, 0x5f3, 0x5d5, 0x5e8, 0x5d2, 0x5f3,
0x5d9, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d5, 0x5e8, 0x5d9, 0x5e0,
0x5d0, 0x5dd, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e1, 0x5d9, 0x5d9, 0x5d5, 0x5d5,
0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d4, 0x5d9, 0x5d8, 0x5d9, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d0, 0x5d9, 0x5e4, 0x5d9, 0x5d9, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d8, 0x5d0, 0x5d9, 0x5e4, 0x5d9, 0x5d9, 0x20, 0x28, 0x5d7,
0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d0, 0x5d9,
0x5e4, 0x5d9, 0x5d9, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d8, 0x5d2, 0x5f3, 0x5d9, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e7, 0x5dc, 0x5d0, 0x5d5, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e0, 0x5d2, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d8, 0x5d5, 0x5e0, 0x5d2, 0x5d4, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e0, 0x5d2, 0x5d4, 0x20,
0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5e6, 0x5f3,
0x5d5, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e8, 0x5e7, 0x5de,
0x5e0, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5,
0x5e8, 0x5e7, 0x5de, 0x5e0, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5d7, 0x5d5,
0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5e8, 0x5e7,
0x5de, 0x5e0, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d8, 0x5d5, 0x5d1, 0x5d0, 0x5dc, 0x5d5, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5d2, 0x5d5, 0x5d5, 0x5d0,
0x5d9, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5d2, 0x5d5,
0x5d5, 0x5d0, 0x5d9, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5, 0x5e8, 0x5d5, 0x5d2, 0x5d5, 0x5d5, 0x5d0, 0x5d9,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0,
0x5d5, 0x5d6, 0x5d1, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d0, 0x5d5, 0x5d6, 0x5d1, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28,
0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d0, 0x5d5,
0x5d6, 0x5d1, 0x5e7, 0x5d9, 0x5e1, 0x5d8, 0x5df, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5e0, 0x5d5, 0x5d0, 0x5d8, 0x5d5,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5e0, 0x5d5, 0x5d0, 0x5d8, 0x5d5, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5,
0x5e0, 0x5d5, 0x5d0, 0x5d8, 0x5d5, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5e0, 0x5e6, 0x5d5, 0x5d0, 0x5dc, 0x5d4, 0x5e9,
0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5dc, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5e1, 0x5d8,
0x5d5, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5dc, 0x5d3, 0x5d9, 0x5d5,
0x5d5, 0x5e1, 0x5d8, 0x5d5, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5dc, 0x5d3, 0x5d9, 0x5d5, 0x5d5, 0x5e1,
0x5d8, 0x5d5, 0x5e7, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d5, 0x5d5, 0x5dc, 0x5d2, 0x5d5, 0x5d2, 0x5e8, 0x5d3, 0x5e9, 0x5e2,
0x5d5, 0x5df, 0x20, 0x5d5, 0x5d5, 0x5dc, 0x5d2, 0x5d5, 0x5d2, 0x5e8, 0x5d3, 0x20,
0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5,
0x5d5, 0x5dc, 0x5d2, 0x5d5, 0x5d2, 0x5e8, 0x5d3, 0x20, 0x28, 0x5e7, 0x5d9, 0x5e5,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5d5, 0x5e1, 0x5d8, 0x5d5, 0x5e7,
0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d4, 0x5d0, 0x5d9, 0x20, 0x5d5, 0x5d9, 0x5d9,
0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d5, 0x5d5, 0x5d0, 0x5dc, 0x5d9, 0x5e1,
0x20, 0x5d5, 0x5e4, 0x5d5, 0x5d8, 0x5d5, 0x5e0, 0x5d4, 0x5e9, 0x5e2, 0x5d5, 0x5df,
0x20, 0x5d9, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d9, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7, 0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3,
0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5e7, 0x5d5, 0x5d8, 0x5e1, 0x5e7,
0x20, 0x28, 0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9,
0x5e7, 0x5d8, 0x5e8, 0x5d9, 0x5e0, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x5e9, 0x5e2, 0x5d5,
0x5df, 0x20, 0x5d9, 0x5e7, 0x5d8, 0x5e8, 0x5d9, 0x5e0, 0x5d1, 0x5d5, 0x5e8, 0x5d2,
0x20, 0x28, 0x5d7, 0x5d5, 0x5e8, 0x5e3, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20,
0x5d9, 0x5e7, 0x5d8, 0x5e8, 0x5d9, 0x5e0, 0x5d1, 0x5d5, 0x5e8, 0x5d2, 0x20, 0x28,
0x5e7, 0x5d9, 0x5e5, 0x29, 0x5e9, 0x5e2, 0x5d5, 0x5df, 0x20, 0x5d9, 0x5d5, 0x5e7,
0x5d5, 0x5df, 0x905, 0x92b, 0x93c, 0x917, 0x93e, 0x928, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905,
0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940,
0x915, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923,
0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x93c, 0x94d,
0x930, 0x940, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x93c, 0x94d,
0x930, 0x940, 0x915, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x93e,
0x938, 0x94d, 0x915, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x93e, 0x938,
0x94d, 0x200d, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x200d, 0x915, 0x93e, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x92e, 0x947, 0x91c,
0x93c, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x92e, 0x947, 0x91c, 0x93c, 0x928,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x92e, 0x947,
0x91c, 0x93c, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x915,
0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x940, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930,
0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x915, 0x947,
0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20,
0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x942, 0x930, 0x94d,
0x935, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940,
0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x942, 0x930,
0x94d, 0x935, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f,
0x915, 0x940, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92e,
0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924,
0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940,
0x20, 0x92e, 0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20,
0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x909, 0x902,
0x91f, 0x947, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938,
0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924,
0x20, 0x938, 0x92e, 0x92f, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905,
0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e,
0x902, 0x924, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x940, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x928, 0x93e, 0x921,
0x940, 0x92f, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x928, 0x93e, 0x921, 0x940,
0x92f, 0x930, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x90f,
0x928, 0x93e, 0x921, 0x940, 0x92f, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x90f,
0x92a, 0x93f, 0x906, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x92a, 0x93f, 0x906, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x92a, 0x93f, 0x906,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x905,
0x930, 0x92c, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x92c, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x92c, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x91c,
0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930,
0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f,
0x940, 0x928, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940,
0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f,
0x940, 0x928, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x906, 0x930, 0x94d, 0x92e,
0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x906, 0x930, 0x94d,
0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f,
0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91f,
0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x911, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x938,
0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d,
0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915,
0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926,
0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940,
0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930,
0x940, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947, 0x902,
0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938,
0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x911, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x908, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c,
0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c,
0x93c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c, 0x930, 0x92c, 0x948, 0x91c, 0x93e,
0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947,
0x938, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x93c,
0x94b, 0x930, 0x947, 0x938, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x93e, 0x902,
0x917, 0x94d, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x92c,
0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x93e, 0x902, 0x917, 0x94d, 0x932,
0x93e, 0x926, 0x947, 0x936, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x942, 0x91f,
0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94b, 0x932, 0x940, 0x935, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c,
0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930,
0x93e, 0x91c, 0x93c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c, 0x940,
0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930,
0x942, 0x928, 0x947, 0x908, 0x20, 0x926, 0x93e, 0x930, 0x942, 0x938, 0x94d, 0x938,
0x932, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930,
0x94d, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930,
0x94d, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x91a, 0x93e, 0x92e, 0x94b, 0x930, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x91a, 0x948, 0x925, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x91a,
0x948, 0x925, 0x92e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x91a, 0x948, 0x925, 0x92e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x938, 0x92e, 0x92f, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x91a,
0x93f, 0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x91a, 0x93f, 0x932, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x940, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x940, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x949, 0x907, 0x92c, 0x93e, 0x932,
0x94d, 0x938, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x949, 0x907, 0x92c, 0x93e,
0x932, 0x94d, 0x938, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x949, 0x907, 0x92c, 0x93e, 0x932, 0x94d, 0x938, 0x928, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x915, 0x94b,
0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x902,
0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x941, 0x915,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938,
0x92e, 0x942, 0x939, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x915, 0x941, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942,
0x939, 0x20, 0x905, 0x930, 0x94d, 0x926, 0x94d, 0x927, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x921, 0x947, 0x935, 0x93f, 0x938,
0x20, 0x938, 0x92e, 0x92f, 0x921, 0x94d, 0x92f, 0x942, 0x92e, 0x94b, 0x902, 0x91f,
0x20, 0x921, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x935, 0x93f, 0x932, 0x947, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x924, 0x93f,
0x92e, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x938, 0x94d, 0x91f, 0x930,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x938,
0x94d, 0x91f, 0x930, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20,
0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x915,
0x94d, 0x935, 0x93e, 0x921, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x927,
0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x938,
0x92e, 0x92f, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a,
0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e,
0x927, 0x94d, 0x200d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x200d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940,
0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a,
0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940,
0x92f, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x917, 0x94d, 0x930, 0x20, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940,
0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940,
0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x92f, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x200d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x949,
0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938,
0x92e, 0x942, 0x939, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x949, 0x915, 0x932,
0x948, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942,
0x939, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c,
0x949, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a,
0x938, 0x92e, 0x942, 0x939, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x93f,
0x91c, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x93f, 0x91c, 0x940, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x93f, 0x91c,
0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x94d, 0x930, 0x947, 0x902,
0x91a, 0x20, 0x917, 0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x940, 0x20, 0x92b, 0x93c, 0x94d, 0x930,
0x93e, 0x902, 0x938, 0x20, 0x914, 0x930, 0x20, 0x905, 0x902, 0x91f, 0x93e, 0x930,
0x94d, 0x915, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x917, 0x948, 0x932,
0x93e, 0x92a, 0x947, 0x917, 0x94b, 0x938, 0x20, 0x915, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x917, 0x948, 0x902, 0x92c, 0x93f, 0x92f, 0x930, 0x20, 0x938, 0x92e, 0x92f,
0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x917, 0x93f, 0x932, 0x94d, 0x92c, 0x930,
0x94d, 0x91f, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939,
0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932,
0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948,
0x902, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x916, 0x93e, 0x921, 0x93c, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x935, 0x93e,
0x908, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x939, 0x935, 0x93e, 0x908, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936,
0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x935,
0x93e, 0x908, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936, 0x928, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93e, 0x901,
0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93e,
0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901,
0x917, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20,
0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92d, 0x93e, 0x930, 0x924, 0x940, 0x92f, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93f, 0x902, 0x926, 0x20, 0x92e,
0x939, 0x93e, 0x938, 0x93e, 0x917, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x902,
0x921, 0x94b, 0x91a, 0x93e, 0x907, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92e,
0x927, 0x94d, 0x92f, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940,
0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x907,
0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x908, 0x930, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x930, 0x93e,
0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x930,
0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x92f, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x930,
0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x938, 0x92e, 0x92f,
0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x93e,
0x92a, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x93e, 0x92a, 0x93e, 0x928,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x93e, 0x92a,
0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x947, 0x935, 0x932, 0x93e,
0x938, 0x94d, 0x915, 0x2d, 0x915, 0x948, 0x92e, 0x91a, 0x93e, 0x924, 0x94d, 0x938,
0x915, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b,
0x92a, 0x947, 0x935, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x2d, 0x915, 0x948, 0x92e,
0x91a, 0x93e, 0x924, 0x94d, 0x938, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x947,
0x935, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x2d, 0x915, 0x948, 0x92e, 0x91a, 0x93e,
0x924, 0x94d, 0x938, 0x915, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942,
0x930, 0x94d, 0x935, 0x20, 0x915, 0x91c, 0x93c, 0x93e, 0x916, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x20, 0x915, 0x91c, 0x93c, 0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x938,
0x92e, 0x92f, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e,
0x908, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f,
0x915, 0x94b, 0x938, 0x930, 0x93e, 0x90f, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d,
0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928,
0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928,
0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x93f, 0x930, 0x94d, 0x917, 0x93f, 0x938, 0x94d, 0x200d, 0x924, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x932, 0x93e, 0x907, 0x928, 0x20, 0x926, 0x94d,
0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x938, 0x92e, 0x92f, 0x932,
0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x938, 0x92e,
0x92f, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x932, 0x949, 0x930, 0x94d,
0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907,
0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x917, 0x93e,
0x926, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x932, 0x947, 0x936,
0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x932, 0x926, 0x940,
0x935, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x947, 0x938,
0x938, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x930, 0x94d, 0x936, 0x932, 0x20,
0x926, 0x94d, 0x935, 0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x938, 0x92e,
0x92f, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x938, 0x92e, 0x92f, 0x92e,
0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x92e, 0x93e, 0x935, 0x94d, 0x938, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e,
0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936,
0x93e, 0x902, 0x924, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x947, 0x915, 0x94d, 0x938,
0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x947, 0x915, 0x94d,
0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x909,
0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x92f,
0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c,
0x91f, 0x94b, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x949, 0x938, 0x94d,
0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x949, 0x938,
0x94d, 0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x94d, 0x92f, 0x93e,
0x902, 0x92e, 0x93e, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94c, 0x930, 0x942,
0x20, 0x938, 0x92e, 0x92f, 0x928, 0x947, 0x92a, 0x93e, 0x932, 0x20, 0x938, 0x92e,
0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x948, 0x932, 0x947, 0x921, 0x94b,
0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942,
0x20, 0x915, 0x948, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942,
0x20, 0x915, 0x948, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93c, 0x940, 0x932,
0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c,
0x93c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93c, 0x940, 0x932, 0x948,
0x902, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932,
0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x92b,
0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93c,
0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x940, 0x92f, 0x942, 0x20,
0x938, 0x92e, 0x92f, 0x928, 0x949, 0x930, 0x92b, 0x93c, 0x949, 0x915, 0x20, 0x926,
0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x949, 0x930, 0x92b,
0x93c, 0x949, 0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x949, 0x930, 0x92b, 0x93c, 0x949,
0x915, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x930, 0x94d, 0x928, 0x93e,
0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930,
0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x930,
0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20,
0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x930, 0x94d, 0x928, 0x93e, 0x902,
0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b,
0x928, 0x94d, 0x939, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x935,
0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x913, 0x92e, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x915, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x932, 0x93e, 0x909, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x93e, 0x92a, 0x941, 0x906, 0x20, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x917, 0x93f,
0x928, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d,
0x935, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d,
0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x948, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x947, 0x930, 0x942, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x930, 0x942,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x930,
0x942, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x93f, 0x932, 0x93f, 0x92a,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93c, 0x93f, 0x932, 0x93f, 0x92a,
0x940, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b,
0x93c, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x92b, 0x93c, 0x940, 0x928, 0x93f, 0x915, 0x94d, 0x938, 0x20, 0x926, 0x94d, 0x935,
0x940, 0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947,
0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x914, 0x930, 0x20,
0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20,
0x914, 0x930, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947, 0x902,
0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x914, 0x930, 0x20, 0x92e,
0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93f, 0x91f, 0x915, 0x948,
0x930, 0x94d, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x94b, 0x928, 0x93e, 0x92a,
0x947, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x94d, 0x92f, 0x94b, 0x902, 0x917, 0x92f,
0x93e, 0x902, 0x917, 0x20, 0x938, 0x92e, 0x92f, 0x930, 0x940, 0x92f, 0x942, 0x928,
0x93f, 0x92f, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x930, 0x94b, 0x925, 0x947, 0x930,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x938, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x916, 0x93e, 0x932, 0x93f, 0x928,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x938, 0x92e, 0x93e, 0x930, 0x93e, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x938, 0x92e, 0x94b, 0x906, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x92e,
0x94b, 0x906, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938,
0x92e, 0x94b, 0x906, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938,
0x92e, 0x92f, 0x938, 0x947, 0x936, 0x947, 0x932, 0x94d, 0x938, 0x20, 0x938, 0x92e,
0x92f, 0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x941, 0x930, 0x20, 0x938, 0x92e,
0x92f, 0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x928, 0x20, 0x926, 0x94d, 0x935, 0x940,
0x92a, 0x938, 0x92e, 0x942, 0x939, 0x20, 0x938, 0x92e, 0x92f, 0x926, 0x915, 0x94d,
0x937, 0x93f, 0x923, 0x940, 0x20, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x942, 0x930, 0x940, 0x928, 0x93e, 0x92e,
0x20, 0x938, 0x92e, 0x92f, 0x938, 0x94d, 0x92f, 0x94b, 0x935, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x924, 0x93e, 0x939, 0x93f, 0x924, 0x940, 0x20, 0x938, 0x92e, 0x92f,
0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x907,
0x92a, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x924,
0x93e, 0x907, 0x92a, 0x947, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x91c, 0x93f, 0x915, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91f, 0x94b, 0x915, 0x947, 0x932, 0x93e,
0x90a, 0x20, 0x938, 0x92e, 0x92f, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x91a, 0x941, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x941, 0x930,
0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x941, 0x935,
0x93e, 0x932, 0x942, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x930, 0x941, 0x917, 0x94d,
0x935, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935,
0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x930,
0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x91c,
0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x928, 0x941, 0x906, 0x924,
0x942, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x928, 0x941, 0x906, 0x924, 0x942, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x928, 0x941, 0x906,
0x924, 0x942, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x947, 0x928, 0x947, 0x91c,
0x93c, 0x941, 0x90f, 0x932, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x94d, 0x932,
0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d,
0x924, 0x94b, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b,
0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x94b, 0x932, 0x94d, 0x917, 0x94b,
0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x94b, 0x932,
0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917,
0x94d, 0x930, 0x93e, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x947, 0x915,
0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x938, 0x92e, 0x92f, 0x935, 0x93e,
0x932, 0x93f, 0x938, 0x20, 0x914, 0x930, 0x20, 0x92b, 0x93c, 0x94d, 0x92f, 0x942,
0x91a, 0x942, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x93e, 0x915, 0x941,
0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f,
0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x947, 0x915,
0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x947, 0x915, 0x93e, 0x924,
0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x92f, 0x941, 0x915, 0x949, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x4e, 0x6f,
0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e,
0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f,
0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x65,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e,
0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x44, 0x61, 0x79,
0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x4e, 0x6f,
0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x20,
0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20,
0x44, 0x61, 0x79, 0x6c, 0x69, 0x67, 0x68, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x41, 0x63, 0x72, 0x65, 0x20, 0x69, 0x64, 0x151, 0x41, 0x63, 0x72,
0x65, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x41, 0x63, 0x72,
0x65, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x64,
0xe9, 0x6c, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61,
0x74, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x69, 0x20, 0x69, 0x64,
0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x61, 0x6c, 0x61, 0x73, 0x7a, 0x6b, 0x61, 0x69,
0x20, 0x69, 0x64, 0x151, 0x61, 0x6c, 0x61, 0x73, 0x7a, 0x6b, 0x61, 0x69,
0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x61, 0x6c, 0x61, 0x73,
0x7a, 0x6b, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x69, 0x64, 0x151,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69,
0x64, 0x151, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x6d, 0x61, 0x7a, 0xf3, 0x6e,
0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x6d, 0x61, 0x7a, 0xf3,
0x6e, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x61, 0x6d, 0x61, 0x7a, 0xf3, 0x6e, 0x69, 0x61, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9,
0x70, 0x73, 0x151, 0x20, 0xe1, 0x6c, 0x6c, 0x61, 0x6d, 0x6f, 0x6b, 0x62,
0x65, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70,
0x73, 0x151, 0x20, 0xe1, 0x6c, 0x6c, 0x61, 0x6d, 0x6f, 0x6b, 0x62, 0x65,
0x6c, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x6b, 0xf6,
0x7a, 0xe9, 0x70, 0x73, 0x151, 0x20, 0xe1, 0x6c, 0x6c, 0x61, 0x6d, 0x6f,
0x6b, 0x62, 0x65, 0x6c, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x69, 0x20, 0xe1, 0x6c,
0x6c, 0x61, 0x6d, 0x6f, 0x6b, 0x62, 0x65, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x69, 0x20, 0xe1, 0x6c, 0x6c, 0x61,
0x6d, 0x6f, 0x6b, 0x62, 0x65, 0x6c, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61,
0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x69, 0x20, 0xe1, 0x6c,
0x6c, 0x61, 0x6d, 0x6f, 0x6b, 0x62, 0x65, 0x6c, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0x65, 0x67, 0x79, 0x76,
0x69, 0x64, 0xe9, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0x65, 0x67,
0x79, 0x76, 0x69, 0x64, 0xe9, 0x6b, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61,
0x69, 0x64, 0x151, 0x68, 0x65, 0x67, 0x79, 0x76, 0x69, 0x64, 0xe9, 0x6b,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63,
0x73, 0x65, 0x6e, 0x64, 0x65, 0x73, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e,
0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x73, 0x65, 0x6e, 0x64, 0x65, 0x73,
0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61,
0x69, 0x64, 0x151, 0x63, 0x73, 0x65, 0x6e, 0x64, 0x65, 0x73, 0x2d, 0xf3,
0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x41, 0x6e, 0x61, 0x64, 0xed, 0x72, 0x69, 0x20, 0x7a, 0xf3,
0x6e, 0x61, 0x69, 0x64, 0x151, 0x41, 0x6e, 0x61, 0x64, 0xed, 0x72, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x70,
0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x70, 0x69, 0x61, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x70, 0x69,
0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x41, 0x71, 0x74, 0x61, 0x75, 0x69, 0x20, 0x69, 0x64, 0x151, 0x41, 0x71,
0x74, 0x61, 0x75, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151,
0x41, 0x71, 0x74, 0x61, 0x75, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x69, 0x20,
0x69, 0x64, 0x151, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x69, 0x20, 0x7a,
0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61,
0x72, 0x61, 0x62, 0x20, 0x69, 0x64, 0x151, 0x61, 0x72, 0x61, 0x62, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x72, 0x61, 0x62,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151,
0x7a, 0xf3, 0x6e, 0x61, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74,
0x2d, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0xf6, 0x72, 0x6d,
0xe9, 0x6e, 0x79, 0x6f, 0x72, 0x73, 0x7a, 0xe1, 0x67, 0x69, 0x20, 0x69,
0x64, 0x151, 0xf6, 0x72, 0x6d, 0xe9, 0x6e, 0x79, 0x6f, 0x72, 0x73, 0x7a,
0xe1, 0x67, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0xf6, 0x72, 0x6d, 0xe9, 0x6e, 0x79, 0x6f, 0x72, 0x73, 0x7a, 0xe1, 0x67,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e,
0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61,
0x69, 0x64, 0x151, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x2d, 0xf3,
0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x61, 0x75, 0x73,
0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x61, 0x75, 0x73, 0x7a, 0x74, 0x72,
0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x61, 0x75, 0x73, 0x7a,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x6e,
0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x61, 0x75, 0x73, 0x7a, 0x74, 0x72,
0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a,
0xe9, 0x70, 0x2d, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x61, 0x75,
0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d,
0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x61, 0x75, 0x73, 0x7a, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x61, 0x75,
0x73, 0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x61, 0x75, 0x73, 0x7a, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x61, 0x75, 0x73,
0x7a, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x6a, 0x64, 0x7a, 0x73, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x7a, 0x73, 0xe1, 0x6e, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x6a, 0x64, 0x7a, 0x73, 0xe1, 0x6e, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61, 0x7a, 0x6f, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x61, 0x7a, 0x6f,
0x72, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x61,
0x7a, 0x6f, 0x72, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x69,
0x20, 0x69, 0x64, 0x151, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x75, 0x74, 0xe1, 0x6e,
0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x6f, 0x6c, 0xed, 0x76, 0x69, 0x61,
0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x72,
0x61, 0x7a, 0xed, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x62,
0x72, 0x61, 0x7a, 0xed, 0x6c, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x62, 0x72, 0x61, 0x7a, 0xed, 0x6c, 0x69,
0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x2d, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a,
0xf6, 0x6c, 0x64, 0x2d, 0x66, 0x6f, 0x6b, 0x69, 0x2d, 0x73, 0x7a, 0x69,
0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3,
0x6e, 0x61, 0x7a, 0xf6, 0x6c, 0x64, 0x2d, 0x66, 0x6f, 0x6b, 0x69, 0x2d,
0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf6, 0x6c, 0x64, 0x2d, 0x66,
0x6f, 0x6b, 0x69, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63,
0x61, 0x73, 0x65, 0x79, 0x2d, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0xf3, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x63, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x69,
0x20, 0x69, 0x64, 0x151, 0x63, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x63, 0x68, 0x69, 0x6c, 0x65, 0x69, 0x20, 0x69, 0x64,
0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x63, 0x68, 0x69, 0x6c, 0x65, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x68, 0x69, 0x6c,
0x65, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6b, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xed, 0x6e,
0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b,
0xed, 0x6e, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x63, 0x73, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x7a, 0x61,
0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x73, 0x6f, 0x6a, 0x62, 0x61,
0x6c, 0x73, 0x7a, 0x61, 0x6e, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x63, 0x73, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x7a,
0x61, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0x61, 0x72, 0xe1, 0x63, 0x73, 0x6f, 0x6e, 0x79, 0x2d, 0x73,
0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf3,
0x6b, 0x75, 0x73, 0x7a, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65,
0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x63, 0x6f, 0x6f, 0x6b, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x6f, 0x6f, 0x6b, 0x2d,
0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x63, 0x6f, 0x6f, 0x6b, 0x2d, 0x73,
0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x66, 0xe9, 0x6c,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x75,
0x62, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6b,
0x75, 0x62, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0x75, 0x62, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x64, 0x61, 0x76, 0x69, 0x73, 0x69, 0x20, 0x69,
0x64, 0x151, 0x64, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0xfa, 0x73,
0x76, 0xe9, 0x74, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20,
0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x68, 0xfa, 0x73, 0x76, 0xe9,
0x74, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0xfa, 0x73, 0x76, 0xe9, 0x74,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x65, 0x63, 0x75, 0x61, 0x64, 0x6f,
0x72, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b,
0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61, 0x69,
0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6b, 0xf6, 0x7a, 0xe9,
0x70, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d,
0x65, 0x75, 0x72, 0xf3, 0x70, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x65,
0x75, 0x72, 0xf3, 0x70, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3,
0x6e, 0x61, 0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x65, 0x75, 0x72, 0xf3,
0x70, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d,
0x69, 0x6e, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79,
0x75, 0x67, 0x61, 0x74, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61, 0x69,
0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6e, 0x79, 0x75, 0x67,
0x61, 0x74, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61,
0x74, 0x2d, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x61, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b,
0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65,
0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x66, 0x69, 0x64, 0x7a, 0x73, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0x69,
0x64, 0x7a, 0x73, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x66, 0x69, 0x64, 0x7a, 0x73, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x61,
0x2d, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151,
0x66, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x61, 0x20, 0x64, 0xe9, 0x6c, 0x69,
0x20, 0xe9, 0x73, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69,
0x73, 0x7a, 0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x61, 0x6c, 0xe1, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x67, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x67, 0x72, 0xfa, 0x7a, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64,
0x151, 0x67, 0x72, 0xfa, 0x7a, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x72, 0xfa, 0x7a, 0x69, 0x61, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x69, 0x20, 0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x69,
0x64, 0x151, 0x2c, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6b, 0x65, 0x6c, 0x65, 0x74, 0x2d, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x6b,
0x65, 0x6c, 0x65, 0x74, 0x2d, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b,
0x65, 0x6c, 0x65, 0x74, 0x2d, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x67, 0x72, 0xf6, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61,
0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x67, 0x72, 0xf6, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x67, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x47, 0x75, 0x61, 0x6d, 0x69, 0x20, 0x7a, 0xf3, 0x6e,
0x61, 0x69, 0x64, 0x151, 0xf6, 0x62, 0xf6, 0x6c, 0x62, 0x65, 0x6c, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x67, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x68, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61, 0x68, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x67, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3, 0x6e, 0x61,
0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f,
0x6e, 0x67, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x68, 0x6f, 0x76, 0x64, 0x69, 0x20, 0x69, 0x64, 0x151, 0x68, 0x6f,
0x76, 0x64, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x68, 0x6f, 0x76, 0x64, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x69, 0x6e, 0x64, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x6e, 0x64, 0x69, 0x61, 0x69,
0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69,
0x6e, 0x64, 0x6f, 0x6b, 0xed, 0x6e, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6b, 0xf6, 0x7a, 0xe9, 0x70, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9,
0x7a, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c, 0x65,
0x74, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x7a, 0x69, 0x61, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74, 0x2d, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x7a, 0x69, 0x61, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x72, 0xe1, 0x6e, 0x69, 0x20,
0x69, 0x64, 0x151, 0x69, 0x72, 0xe1, 0x6e, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x72, 0xe1, 0x6e, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x7a,
0x72, 0x61, 0x65, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x69, 0x7a, 0x72,
0x61, 0x65, 0x6c, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x61, 0x70, 0xe1, 0x6e, 0x20,
0x69, 0x64, 0x151, 0x6a, 0x61, 0x70, 0xe1, 0x6e, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x61, 0x70, 0xe1, 0x6e, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x7a, 0x6b, 0x2d, 0x6b,
0x61, 0x6d, 0x63, 0x73, 0x61, 0x74, 0x6b, 0x61, 0x69, 0x20, 0x69, 0x64,
0x151, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x7a, 0x6b, 0x2d, 0x6b, 0x61, 0x6d, 0x63, 0x73, 0x61, 0x74, 0x6b,
0x61, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x7a, 0x6b,
0x2d, 0x6b, 0x61, 0x6d, 0x63, 0x73, 0x61, 0x74, 0x6b, 0x61, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x65, 0x6c,
0x65, 0x74, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x7a, 0x74, 0xe1,
0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x79, 0x75, 0x67, 0x61, 0x74,
0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x6f, 0x73,
0x72, 0x61, 0x65, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6b, 0x72, 0x61, 0x73,
0x7a, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6b, 0x72, 0x61, 0x73, 0x7a, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x7a, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0x72, 0x61, 0x73, 0x7a, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x7a, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6b, 0x69, 0x72, 0x67, 0x69, 0x7a, 0x69, 0x73, 0x7a, 0x74, 0xe1,
0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x69,
0x20, 0x69, 0x64, 0x151, 0x73, 0x6f, 0x72, 0x2d, 0x73, 0x7a, 0x69, 0x67,
0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x69, 0x20, 0x69, 0x64, 0x151, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x4d,
0x61, 0x63, 0x61, 0x75, 0x69, 0x20, 0x69, 0x64, 0x151, 0x4d, 0x61, 0x63,
0x61, 0x75, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x4d,
0x61, 0x63, 0x61, 0x75, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x69, 0x20,
0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x67, 0x61,
0x64, 0xe1, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x6d, 0x61, 0x6c, 0x61, 0x6a, 0x7a, 0x69, 0x61, 0x69, 0x20,
0x69, 0x64, 0x151, 0x6d, 0x61, 0x6c, 0x64, 0xed, 0x76, 0x2d, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d,
0x61, 0x72, 0x71, 0x75, 0x69, 0x73, 0x65, 0x73, 0x2d, 0x73, 0x7a, 0x69,
0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61,
0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x20, 0x69, 0x64, 0x151, 0x7a, 0xf3,
0x6e, 0x61, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6d, 0x65, 0x78, 0x69, 0x6b, 0xf3, 0x69, 0x20,
0x63, 0x73, 0x65, 0x6e, 0x64, 0x65, 0x73, 0x2d, 0xf3, 0x63, 0x65, 0xe1,
0x6e, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x65, 0x78, 0x69, 0x6b, 0xf3,
0x69, 0x20, 0x63, 0x73, 0x65, 0x6e, 0x64, 0x65, 0x73, 0x2d, 0xf3, 0x63,
0x65, 0xe1, 0x6e, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151,
0x6d, 0x65, 0x78, 0x69, 0x6b, 0xf3, 0x69, 0x20, 0x63, 0x73, 0x65, 0x6e,
0x64, 0x65, 0x73, 0x2d, 0xf3, 0x63, 0x65, 0xe1, 0x6e, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x75, 0x6c, 0xe1, 0x6e,
0x62, 0xe1, 0x74, 0x6f, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x75, 0x6c,
0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x6f, 0x72, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x75, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74,
0x6f, 0x72, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6d, 0x6f, 0x73, 0x7a, 0x6b, 0x76, 0x61, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6d, 0x6f, 0x73, 0x7a, 0x6b, 0x76, 0x61, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6d, 0x6f, 0x73, 0x7a, 0x6b, 0x76,
0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6d, 0x69, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x6e, 0x61, 0x75, 0x72, 0x75, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x65,
0x70, 0xe1, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x6b,
0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64,
0x151, 0xfa, 0x6a, 0x2d, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69,
0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfa,
0x6a, 0x2d, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a,
0x2d, 0x7a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x69, 0x64, 0x151,
0xfa, 0x6a, 0x2d, 0x7a, 0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x7a, 0xe9,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x66, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61,
0x69, 0x64, 0x151, 0xfa, 0x6a, 0x2d, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64,
0x151, 0x6e, 0x69, 0x75, 0x65, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74,
0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65,
0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x69, 0x20, 0x69, 0x64,
0x151, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x2d, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0xc9, 0x73, 0x7a, 0x61, 0x6b, 0x2d, 0x6d, 0x61, 0x72, 0x69,
0x61, 0x6e, 0x61, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b,
0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x7a, 0x69,
0x62, 0x69, 0x72, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e,
0x6f, 0x76, 0x6f, 0x73, 0x7a, 0x69, 0x62, 0x69, 0x72, 0x73, 0x7a, 0x6b,
0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6e, 0x6f,
0x76, 0x6f, 0x73, 0x7a, 0x69, 0x62, 0x69, 0x72, 0x73, 0x7a, 0x6b, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6f, 0x6d,
0x73, 0x7a, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6f, 0x6d, 0x73, 0x7a,
0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6f,
0x6d, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20,
0x69, 0x64, 0x151, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e,
0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x7a, 0x74,
0xe1, 0x6e, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151,
0x70, 0x61, 0x6b, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x61, 0x6c, 0x61,
0x75, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0xe1, 0x70, 0x75, 0x61, 0x20,
0xfa, 0x6a, 0x2d, 0x67, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x69, 0x20, 0x69,
0x64, 0x151, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x69, 0x20,
0x69, 0x64, 0x151, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x69,
0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x70, 0x65, 0x72, 0x75, 0x69, 0x20, 0x69, 0x64,
0x151, 0x70, 0x65, 0x72, 0x75, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x70, 0x65, 0x72, 0x75, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x66, 0xfc, 0x6c, 0xf6, 0x70, 0x2d,
0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64,
0x151, 0x66, 0xfc, 0x6c, 0xf6, 0x70, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x65, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x66, 0xfc, 0x6c, 0xf6, 0x70, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65,
0x74, 0x65, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x70, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x2d, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0xe9, 0x73, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x69, 0x20, 0x69, 0x64, 0x151, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0xe9,
0x73, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x69,
0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0xe9, 0x73, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65,
0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x70, 0x6f, 0x6e, 0x61, 0x70, 0x65,
0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20, 0x69, 0x64, 0x151,
0x70, 0x68, 0x65, 0x6e, 0x6a, 0x61, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151,
0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x69, 0x20, 0x69,
0x64, 0x151, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x69,
0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x51, 0x79, 0x7a, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x72, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x69,
0x20, 0x69, 0x64, 0x151, 0x72, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x69,
0x20, 0x69, 0x64, 0x151, 0x73, 0x7a, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x7a, 0x61, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73,
0x7a, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x20, 0x6e, 0x79, 0xe1,
0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x53, 0x7a, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x53, 0x7a, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x69, 0x20, 0x7a, 0xf3, 0x6e, 0x61, 0x69, 0x64, 0x151, 0x53, 0x7a,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69,
0x20, 0x69, 0x64, 0x151, 0x73, 0x7a, 0x61, 0x6d, 0x6f, 0x61, 0x69, 0x20,
0x69, 0x64, 0x151, 0x73, 0x7a, 0x61, 0x6d, 0x6f, 0x61, 0x69, 0x20, 0x74,
0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x7a, 0x61, 0x6d, 0x6f,
0x61, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151,
0x73, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x2d, 0x73, 0x7a,
0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73,
0x7a, 0x69, 0x6e, 0x67, 0x61, 0x70, 0xfa, 0x72, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x6f,
0x6e, 0x2d, 0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x65, 0x6b, 0x69, 0x20,
0x69, 0x64, 0x151, 0x64, 0xe9, 0x6c, 0x69, 0x2d, 0x67, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x7a, 0x75, 0x72,
0x69, 0x6e, 0xe1, 0x6d, 0x69, 0x20, 0x69, 0x64, 0x151, 0x73, 0x79, 0x6f,
0x77, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0x61, 0x68, 0x69, 0x74,
0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20,
0x69, 0x64, 0x151, 0x74, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0x61, 0x69, 0x70, 0x65, 0x69,
0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0xe1,
0x64, 0x7a, 0x73, 0x69, 0x6b, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69,
0x20, 0x69, 0x64, 0x151, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x69,
0x20, 0x69, 0x64, 0x151, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x69, 0x20, 0x69,
0x64, 0x151, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x69, 0x20, 0x74, 0xe9, 0x6c,
0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0x72, 0x75,
0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74, 0xfc, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64, 0x151,
0x74, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x7a, 0x74, 0xe1,
0x6e, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74,
0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x74,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x69, 0x20, 0x69, 0x64, 0x151, 0x75, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x69, 0x20, 0x69, 0x64, 0x151, 0x75, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20,
0x69, 0x64, 0x151, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x69, 0x20,
0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfc, 0x7a, 0x62,
0x65, 0x67, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x69, 0x64,
0x151, 0xfc, 0x7a, 0x62, 0x65, 0x67, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e,
0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0xfc, 0x7a,
0x62, 0x65, 0x67, 0x69, 0x73, 0x7a, 0x74, 0xe1, 0x6e, 0x69, 0x20, 0x6e,
0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64,
0x151, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x65, 0x6e, 0x65, 0x7a,
0x75, 0x65, 0x6c, 0x61, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76, 0x6c, 0x61,
0x67, 0x79, 0x69, 0x76, 0x6f, 0x73, 0x7a, 0x74, 0x6f, 0x6b, 0x69, 0x20,
0x69, 0x64, 0x151, 0x76, 0x6c, 0x61, 0x67, 0x79, 0x69, 0x76, 0x6f, 0x73,
0x7a, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69,
0x64, 0x151, 0x76, 0x6c, 0x61, 0x67, 0x79, 0x69, 0x76, 0x6f, 0x73, 0x7a,
0x74, 0x6f, 0x6b, 0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69,
0x64, 0x151, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1, 0x64, 0x69,
0x20, 0x69, 0x64, 0x151, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1,
0x64, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x76,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0xe1, 0x64, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x77, 0x61, 0x6b, 0x65, 0x2d,
0x73, 0x7a, 0x69, 0x67, 0x65, 0x74, 0x69, 0x20, 0x69, 0x64, 0x151, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0xe9, 0x73, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x2d, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b, 0x69, 0x20, 0x74, 0xe9, 0x6c, 0x69,
0x20, 0x69, 0x64, 0x151, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x7a, 0x6b,
0x69, 0x20, 0x6e, 0x79, 0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6a,
0x65, 0x6b, 0x61, 0x74, 0x79, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x79,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x20, 0x74, 0xe9,
0x6c, 0x69, 0x20, 0x69, 0x64, 0x151, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x79,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x69, 0x20, 0x6e, 0x79,
0xe1, 0x72, 0x69, 0x20, 0x69, 0x64, 0x151, 0x79, 0x75, 0x6b, 0x6f, 0x6e,
0x69, 0x20, 0x69, 0x64, 0x151, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x74, 0xed, 0x6d, 0x69, 0x4d, 0x69, 0xf0, 0x2d, 0x41,
0x66, 0x72, 0xed, 0x6b, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x41, 0x75, 0x73,
0x74, 0x75, 0x72, 0x2d, 0x41, 0x66, 0x72, 0xed, 0x6b, 0x75, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x75, 0xf0, 0x75, 0x72, 0x2d, 0x41, 0x66, 0x72, 0xed,
0x6b, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72,
0x2d, 0x41, 0x66, 0x72, 0xed, 0x6b, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x41, 0x66, 0x72, 0xed, 0x6b,
0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x41, 0x66, 0x72, 0xed,
0x6b, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x73, 0xf3, 0x6e, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x41, 0x6d, 0x61, 0x73, 0xf3, 0x6e, 0x73, 0x76, 0xe6,
0xf0, 0x69, 0x6e, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x41, 0x6d, 0x61, 0x73, 0xf3, 0x6e, 0x73, 0x76,
0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x6d, 0x69, 0xf0, 0x68, 0x6c, 0x75, 0x74, 0x61, 0x20, 0x42, 0x61, 0x6e,
0x64, 0x61, 0x72, 0xed, 0x6b, 0x6a, 0x61, 0x6e, 0x6e, 0x61, 0x20, 0x6f,
0x67, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x6d, 0x69, 0xf0,
0x68, 0x6c, 0x75, 0x74, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x61, 0x72,
0xed, 0x6b, 0x6a, 0x61, 0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67, 0x20, 0x4b,
0x61, 0x6e, 0x61, 0x64, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x6d, 0x69, 0xf0, 0x68, 0x6c, 0x75, 0x74,
0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x61, 0x72, 0xed, 0x6b, 0x6a, 0x61,
0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64,
0x61, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x61, 0x75, 0x73, 0x74,
0x75, 0x72, 0x68, 0x6c, 0x75, 0x74, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64,
0x61, 0x72, 0xed, 0x6b, 0x6a, 0x61, 0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67,
0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x61, 0x75, 0x73, 0x74,
0x75, 0x72, 0x68, 0x6c, 0x75, 0x74, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64,
0x61, 0x72, 0xed, 0x6b, 0x6a, 0x61, 0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67,
0x20, 0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x61, 0x75, 0x73, 0x74, 0x75,
0x72, 0x68, 0x6c, 0x75, 0x74, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x64, 0x61,
0x72, 0xed, 0x6b, 0x6a, 0x61, 0x6e, 0x6e, 0x61, 0x20, 0x6f, 0x67, 0x20,
0x4b, 0x61, 0x6e, 0x61, 0x64, 0x61, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x4b, 0x6c, 0x65, 0x74, 0x74, 0x61, 0x66, 0x6a, 0xf6, 0x6c, 0x6c,
0x75, 0x6d, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x4b, 0x6c, 0x65, 0x74, 0x74, 0x61, 0x66, 0x6a, 0xf6,
0x6c, 0x6c, 0x75, 0x6d, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x4b, 0x6c, 0x65, 0x74, 0x74, 0x61, 0x66, 0x6a,
0xf6, 0x6c, 0x6c, 0x75, 0x6d, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x4b, 0x79, 0x72, 0x72, 0x61, 0x68, 0x61, 0x66, 0x73, 0x73, 0x76, 0xe6,
0xf0, 0x69, 0x6e, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4b, 0x79, 0x72, 0x72, 0x61, 0x68, 0x61,
0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4b, 0x79, 0x72,
0x72, 0x61, 0x68, 0x61, 0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e,
0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x70,
0xed, 0x61, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x41, 0x70, 0xed, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x70, 0xed, 0x61, 0x41,
0x72, 0x61, 0x62, 0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61,
0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x72,
0x61, 0x62, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x41, 0x72, 0x61, 0x62, 0xed, 0x75, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x75, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0xed, 0x6e, 0x75, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72,
0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x75, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75,
0x72, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x75, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0xed,
0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x73, 0x68, 0x61, 0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69,
0x6e, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x68, 0x61,
0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x73, 0x68, 0x61, 0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0,
0x69, 0x6e, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x69,
0xf0, 0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d,
0x69, 0xf0, 0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d,
0x69, 0xf0, 0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x6d, 0x69, 0xf0, 0x76, 0x65, 0x73,
0x74, 0x75, 0x72, 0x68, 0x6c, 0x75, 0x74, 0x61, 0x20, 0xc1, 0x73, 0x74,
0x72, 0x61, 0x6c, 0xed, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x6d, 0x69, 0xf0, 0x76, 0x65, 0x73,
0x74, 0x75, 0x72, 0x68, 0x6c, 0x75, 0x74, 0x61, 0x20, 0xc1, 0x73, 0x74,
0x72, 0x61, 0x6c, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x6d, 0x69, 0xf0, 0x76, 0x65, 0x73, 0x74,
0x75, 0x72, 0x68, 0x6c, 0x75, 0x74, 0x61, 0x20, 0xc1, 0x73, 0x74, 0x72,
0x61, 0x6c, 0xed, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41,
0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c,
0xed, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0xc1, 0x73,
0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72,
0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x54, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0xc1,
0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x65, 0x73, 0x74,
0x75, 0x72, 0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c, 0xed, 0x75, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56,
0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0xc1, 0x73, 0x74, 0x72, 0x61, 0x6c,
0xed, 0x75, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0xed, 0x64, 0x73, 0x6a,
0x61, 0x6e, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x73, 0x65, 0x72, 0x62,
0x61, 0xed, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x73, 0x65, 0x72, 0x62,
0x61, 0xed, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x41, 0x73, 0xf3, 0x72, 0x65,
0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41, 0x73, 0xf3, 0x72,
0x65, 0x79, 0x6a, 0x75, 0x6d, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41, 0x73, 0xf3, 0x72, 0x65, 0x79, 0x6a,
0x75, 0x6d, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x73,
0x2d, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x73, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x73, 0x42, 0xfa, 0x74, 0x61, 0x6e, 0x74, 0xed, 0x6d, 0x69, 0x42,
0xf3, 0x6c, 0x69, 0x76, 0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0xed, 0x75, 0x42, 0x72, 0xfa, 0x6e, 0x65, 0x69, 0x74, 0xed, 0x6d,
0x69, 0x47, 0x72, 0xe6, 0x6e, 0x68, 0xf6, 0x66, 0xf0, 0x61, 0x65, 0x79,
0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x47, 0x72, 0xe6, 0x6e, 0x68,
0xf6, 0x66, 0xf0, 0x61, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x47, 0x72, 0xe6,
0x6e, 0x68, 0xf6, 0x66, 0xf0, 0x61, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x2d, 0x73, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x53, 0xed, 0x6c,
0x65, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x53, 0xed, 0x6c, 0x65, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x53, 0xed,
0x6c, 0x65, 0x4b, 0xed, 0x6e, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b,
0xed, 0x6e, 0x61, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x4b, 0xed, 0x6e, 0x61, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x43,
0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x4a, 0xf3, 0x6c,
0x61, 0x65, 0x79, 0x6a, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x4b, 0xf3,
0x6b, 0x6f, 0x73, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x4b,
0xf3, 0x6c, 0x75, 0x6d, 0x62, 0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x4b, 0xf3, 0x6c, 0x75, 0x6d, 0x62, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61,
0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0xf3, 0x6c, 0x75,
0x6d, 0x62, 0xed, 0x75, 0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x2d, 0x65, 0x79,
0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x73,
0x2d, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x48, 0xe1, 0x6c, 0x66, 0x73, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x73, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x4b, 0xfa, 0x62,
0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4b, 0xfa, 0x62, 0x75, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4b, 0xfa,
0x62, 0x75, 0x44, 0x61, 0x76, 0x69, 0x73, 0x2d, 0x74, 0xed, 0x6d, 0x69,
0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x54, 0xed, 0x6d, 0x6f, 0x72, 0x2d,
0x4c, 0x65, 0x73, 0x74, 0x65, 0x50, 0xe1, 0x73, 0x6b, 0x61, 0x65, 0x79,
0x6a, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x50, 0xe1, 0x73, 0x6b, 0x61,
0x65, 0x79, 0x6a, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x50, 0xe1, 0x73, 0x6b, 0x61, 0x65, 0x79, 0x6a,
0x75, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x4d, 0x69, 0xf0, 0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70, 0x75, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x4d, 0x69, 0xf0, 0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70,
0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x4d, 0x69, 0xf0, 0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70, 0x75, 0x41,
0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70, 0x75,
0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d,
0x45, 0x76, 0x72, 0xf3, 0x70, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72,
0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0x4b, 0x61, 0x6c, 0xed, 0x6e, 0xed,
0x6e, 0x67, 0x72, 0x61, 0x64, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d,
0x45, 0x76, 0x72, 0xf3, 0x70, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56,
0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70, 0x75,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x45, 0x76, 0x72, 0xf3, 0x70,
0x75, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x79,
0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x46,
0xed, 0x64, 0x6a, 0xed, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x46, 0xed, 0x64, 0x6a, 0xed, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x46,
0xed, 0x64, 0x6a, 0xed, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x54, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x46, 0x72, 0xf6, 0x6e, 0x73, 0x6b, 0x75, 0x20,
0x47, 0x76, 0xe6, 0x6a, 0x61, 0x6e, 0x61, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x66, 0x72, 0xf6, 0x6e, 0x73, 0x6b, 0x75, 0x20, 0x73, 0x75,
0xf0, 0x75, 0x72, 0x68, 0x61, 0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x75,
0x6d, 0x20, 0x6f, 0x67, 0x20, 0x53, 0x75, 0xf0, 0x75, 0x72, 0x73, 0x6b,
0x61, 0x75, 0x74, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x73, 0x76, 0xe6,
0xf0, 0x69, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x2d,
0x74, 0xed, 0x6d, 0x69, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x2d,
0x74, 0xed, 0x6d, 0x69, 0x47, 0x65, 0x6f, 0x72, 0x67, 0xed, 0x75, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0xed, 0x75, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0xed, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x2d, 0x65, 0x79, 0x6a,
0x75, 0x6d, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x2d,
0x73, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x41, 0x75,
0x73, 0x74, 0x75, 0x72, 0x2d, 0x47, 0x72, 0xe6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41, 0x75, 0x73, 0x74, 0x75,
0x72, 0x2d, 0x47, 0x72, 0xe6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x41,
0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x47, 0x72, 0xe6, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x47, 0x72,
0xe6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x53,
0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x47, 0x72, 0xe6, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x47,
0x72, 0xe6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0x76, 0x69, 0xf0, 0x20, 0x50,
0x65, 0x72, 0x73, 0x61, 0x66, 0x6c, 0xf3, 0x61, 0x47, 0x76, 0xe6, 0x6a,
0x61, 0x6e, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x48, 0x61, 0x76, 0x61, 0xed, 0x20, 0x6f, 0x67, 0x20, 0x41,
0x6c, 0x65, 0xfa, 0x74, 0x61, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x48, 0x61, 0x76, 0x61, 0xed, 0x20,
0x6f, 0x67, 0x20, 0x41, 0x6c, 0x65, 0xfa, 0x74, 0x61, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x48, 0x61, 0x76,
0x61, 0xed, 0x20, 0x6f, 0x67, 0x20, 0x41, 0x6c, 0x65, 0xfa, 0x74, 0x61,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x2d, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x76,
0x64, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x49, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x74,
0xed, 0x6d, 0x69, 0x49, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x68,
0x61, 0x66, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x49, 0x6e, 0x64, 0xf3, 0x6b,
0xed, 0x6e, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x4d, 0x69, 0xf0, 0x2d, 0x49,
0x6e, 0x64, 0xf3, 0x6e, 0x65, 0x73, 0xed, 0x75, 0x74, 0xed, 0x6d, 0x69,
0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x49, 0x6e, 0x64, 0xf3, 0x6e,
0x65, 0x73, 0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x56, 0x65, 0x73, 0x74,
0x75, 0x72, 0x2d, 0x49, 0x6e, 0x64, 0xf3, 0x6e, 0x65, 0x73, 0xed, 0x75,
0x74, 0xed, 0x6d, 0x69, 0xcd, 0x72, 0x61, 0x6e, 0x73, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0xcd, 0x72, 0x61, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0xcd, 0x72, 0x61, 0x6e, 0x54, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x75, 0x6d, 0x61,
0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0xcd, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0xcd, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0xcd, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x53, 0x75, 0x6d, 0x61,
0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x54, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x41, 0x75, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x4b, 0x61,
0x73, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x56, 0x65, 0x73, 0x74, 0x75, 0x72, 0x2d, 0x4b, 0x61, 0x73,
0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x4b, 0xf3, 0x72, 0x65, 0x75, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x4b, 0xf3, 0x72, 0x65, 0x75, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0xf3, 0x72,
0x65, 0x75, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x2d, 0x74, 0xed, 0x6d,
0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x4c, 0xed, 0x6e, 0x75,
0x65, 0x79, 0x6a, 0x61, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x4d, 0x61, 0x6c, 0x61, 0x73, 0xed, 0x75, 0x74,
0xed, 0x6d, 0x69, 0x4d, 0x61, 0x6c, 0x64, 0xed, 0x76, 0x65, 0x79, 0x6a,
0x61, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x4d, 0x61, 0x72, 0x6b, 0x67, 0x72, 0x65, 0x69, 0x66, 0x61, 0x66,
0x72, 0xfa, 0x72, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x54, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x2d,
0x65, 0x79, 0x6a, 0x75, 0x6d, 0x4d, 0xe1, 0x72, 0x69, 0x74, 0xed, 0x75,
0x73, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4d, 0xe1, 0x72, 0x69, 0x74, 0xed,
0x75, 0x73, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x4d, 0xe1, 0x72, 0x69, 0x74, 0xed, 0x75, 0x73, 0x4d, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x4b, 0x79, 0x72,
0x72, 0x61, 0x68, 0x61, 0x66, 0x73, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x4d, 0x65, 0x78, 0xed, 0x6b, 0xf3, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x65, 0x78, 0xed,
0x6b, 0xf3, 0x20, 0xe1, 0x20, 0x4b, 0x79, 0x72, 0x72, 0x61, 0x68, 0x61,
0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x65, 0x78,
0xed, 0x6b, 0xf3, 0x20, 0xe1, 0x20, 0x4b, 0x79, 0x72, 0x72, 0x61, 0x68,
0x61, 0x66, 0x73, 0x73, 0x76, 0xe6, 0xf0, 0x69, 0x6e, 0x75, 0x54, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0xda, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0xda, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0xda, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d,
0x6f, 0x73, 0x6b, 0x76, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x75, 0x4d,
0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x4e,
0xe1, 0x72, 0xfa, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x4e, 0x65, 0x70, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x4e, 0xfd, 0x6a, 0x75, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e,
0xed, 0x75, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x4e, 0xfd, 0x6a, 0x75, 0x2d, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0xf3, 0x6e, 0xed, 0x75, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x4e, 0xfd, 0x6a, 0x75, 0x2d, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0xf3, 0x6e, 0xed, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x4e, 0xfd, 0x6a, 0x61, 0x2d, 0x53, 0x6a, 0xe1, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x4e, 0xfd, 0x6a, 0x61, 0x2d, 0x53, 0x6a, 0xe1,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e, 0xfd, 0x6a, 0x61, 0x2d, 0x53, 0x6a,
0xe1, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x4e, 0xfd, 0x66, 0x75, 0x6e, 0x64, 0x6e, 0x61, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x4e, 0xfd, 0x66, 0x75, 0x6e, 0x64, 0x6e, 0x61, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x4e, 0xfd, 0x66, 0x75, 0x6e, 0x64, 0x6e, 0x61,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x4e, 0x69, 0x75, 0x65, 0x2d, 0x74, 0xed,
0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x53, 0x74, 0x61,
0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x2d, 0x65, 0x79, 0x6a, 0x75, 0x54, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53, 0x75, 0x6d, 0x61,
0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x74, 0x61,
0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x54, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53, 0x74, 0x61, 0xf0, 0x61,
0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x50, 0x61, 0x6c,
0xe1, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20,
0x50, 0x61, 0x70, 0xfa, 0x61, 0x20, 0x4e, 0xfd, 0x6a, 0x75, 0x2d, 0x47,
0xed, 0x6e, 0x65, 0x75, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0xe6, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xed, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0xe6, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x76, 0xe6, 0x50, 0x65, 0x72, 0xfa, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x73, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x73, 0x65, 0x79,
0x6a, 0x75, 0x6d, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xe1, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x73, 0x65, 0x79,
0x6a, 0x75, 0x6d, 0x46, 0xf6, 0x6e, 0x69, 0x78, 0x65, 0x79, 0x6a, 0x61,
0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x53,
0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x6f, 0x67, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1,
0x20, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x6f, 0x67, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xe1, 0x20, 0x53, 0x61, 0x6e, 0x6b, 0x74, 0x69, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x6f, 0x67, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x2d,
0x74, 0xed, 0x6d, 0x69, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x74,
0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x50, 0x6a,
0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x52, 0xe9, 0x75, 0x6e, 0x69,
0x6f, 0x6e, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x52, 0x6f, 0x74, 0x68, 0x65,
0x72, 0x61, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x74,
0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x53, 0x61, 0x6d, 0xf3, 0x61, 0x2d, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x53, 0x61, 0x6d, 0xf3, 0x61, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x53, 0x61, 0x6d,
0xf3, 0x61, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73,
0x2d, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0xfa, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x61, 0x6c,
0xf3, 0x6d, 0x6f, 0x6e, 0x73, 0x65, 0x79, 0x6a, 0x61, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x75, 0xf0, 0x75, 0x72, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67,
0xed, 0x75, 0x74, 0xed, 0x6d, 0x69, 0x53, 0xfa, 0x72, 0x69, 0x6e, 0x61,
0x6d, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x2d, 0x74,
0xed, 0x6d, 0x69, 0x54, 0x61, 0x68, 0xed, 0x74, 0xed, 0x2d, 0x74, 0xed,
0x6d, 0x69, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d, 0x74, 0xed, 0x6d,
0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x53, 0x75, 0x6d, 0x61,
0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x54, 0x61, 0x64, 0x73, 0x6a, 0xed, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xf3, 0x6b, 0x65, 0x6c,
0xe1, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x2d, 0x74, 0xed, 0x6d, 0x69,
0x54, 0xfa, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x2d, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x54, 0xfa, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x54, 0xfa, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x54, 0xfa, 0x76, 0x61, 0x6c, 0xfa,
0x74, 0xed, 0x6d, 0x69, 0xda, 0x72, 0xfa, 0x67, 0x76, 0xe6, 0x74, 0xed,
0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69,
0x20, 0xed, 0x20, 0xda, 0x72, 0xfa, 0x67, 0x76, 0xe6, 0x53, 0x75, 0x6d,
0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0xda, 0x72, 0xfa,
0x67, 0x76, 0xe6, 0xda, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c,
0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0xda, 0x73, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0xda, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x56, 0x61, 0x6e, 0xfa, 0x61, 0x74, 0xfa, 0x2d, 0x74,
0xed, 0x6d, 0x69, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d,
0x69, 0x20, 0xe1, 0x20, 0x56, 0x61, 0x6e, 0xfa, 0x61, 0x74, 0xfa, 0x53,
0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x56,
0x61, 0x6e, 0xfa, 0x61, 0x74, 0xfa, 0x56, 0x65, 0x6e, 0x65, 0x73, 0xfa,
0x65, 0x6c, 0x61, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x53, 0x74, 0x61, 0xf0, 0x61, 0x6c, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74, 0xed, 0x6d, 0x69, 0x54, 0xed,
0x6d, 0x69, 0x20, 0xe1, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x2d, 0x65, 0x79,
0x6a, 0x75, 0x54, 0xed, 0x6d, 0x69, 0x20, 0xe1, 0x20, 0x57, 0x61, 0x6c,
0x6c, 0x69, 0x73, 0x2d, 0x20, 0x6f, 0x67, 0x20, 0x46, 0xfa, 0x74, 0xfa,
0x6e, 0x61, 0x65, 0x79, 0x6a, 0x75, 0x6d, 0x54, 0xed, 0x6d, 0x69, 0x20,
0xed, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x74, 0x61,
0xf0, 0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x75, 0x6d, 0x61, 0x72, 0x74, 0xed,
0x6d, 0x69, 0x20, 0xed, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x54, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x53, 0x74, 0x61, 0xf0,
0x61, 0x6c, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x6f, 0x72, 0x67, 0x53, 0x75,
0x6d, 0x61, 0x72, 0x74, 0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x54,
0xed, 0x6d, 0x69, 0x20, 0xed, 0x20, 0x4a, 0xfa, 0x6b, 0x6f, 0x6e, 0x4f,
0x67, 0x65, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20,
0x41, 0x66, 0x72, 0x1ecb, 0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70,
0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61,
0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x66, 0x72,
0x1ecb, 0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x4d,
0x67, 0x62, 0x61, 0x64, 0x61, 0x20, 0x55, 0x67, 0x77, 0x75, 0x20, 0x41,
0x66, 0x72, 0x1ecb, 0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x66, 0x72, 0x1ecb,
0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64,
0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20,
0x41, 0x66, 0x72, 0x1ecb, 0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b,
0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e,
0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x66, 0x72, 0x1ecb, 0x6b, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67,
0x68, 0x61, 0x72, 0x61, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61,
0x72, 0x61, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e,
0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77,
0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x4f,
0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x55, 0x67, 0x77, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x55, 0x67, 0x77, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20,
0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x55, 0x67, 0x77,
0x75, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67,
0x68, 0x61, 0x72, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67,
0x68, 0x61, 0x72, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x4f, 0x67, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x4f, 0x67,
0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f,
0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61,
0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e,
0x77, 0x1ee5, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64,
0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61,
0x72, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x4f, 0x67, 0x65, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x68, 0x65, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb,
0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45,
0x74, 0x69, 0x74, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64,
0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20,
0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x74,
0x69, 0x74, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61,
0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5,
0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61,
0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e,
0x77, 0x1ee5, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61,
0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e,
0x77, 0x1ee5, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x68, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77,
0x1ee5, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x67, 0x65,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b,
0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x67, 0x65, 0x20, 0x42, 0x68, 0x75,
0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76,
0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61,
0x6c, 0x61, 0x6d, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd,
0x6b, 0x1ee5, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x4f, 0x67, 0x65, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70,
0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f,
0x67, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c,
0x73, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x43, 0x68, 0x6f,
0x69, 0x62, 0x61, 0x6c, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x45, 0x6b, 0x65,
0x72, 0x65, 0x73, 0x69, 0x6d, 0x65, 0x73, 0x69, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20,
0x43, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d,
0x1ecd, 0x6b, 0x1ee5, 0x20, 0x43, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x1ecc, 0x6b, 0x61, 0x72, 0x61,
0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67,
0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x68, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x44, 0x61,
0x76, 0x69, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x4f,
0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77,
0x1ee5, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x4f, 0x67, 0x65, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20,
0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77,
0x1ee5, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f,
0x72, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x45, 0x74, 0x69, 0x74, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x45, 0x74,
0x69, 0x74, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67,
0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x45, 0x74, 0x69, 0x74,
0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20,
0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5,
0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61,
0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e,
0x77, 0x1ee5, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65,
0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x70,
0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61,
0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68,
0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e,
0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d,
0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61,
0x6e, 0x77, 0x1ee5, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x4f, 0x67,
0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65,
0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b,
0x1ee5, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x46, 0x72,
0x65, 0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x53, 0x6f,
0x75, 0x74, 0x68, 0x65, 0x72, 0x6e, 0x20, 0x26, 0x20, 0x41, 0x6e, 0x74,
0x61, 0x72, 0x63, 0x74, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20, 0x47, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4f, 0x67, 0x65, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x47,
0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61,
0x72, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6d, 0x77, 0x69, 0x63, 0x68,
0x20, 0x4d, 0x65, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61,
0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20,
0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61,
0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e,
0x77, 0x1ee5, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77,
0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64,
0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67,
0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41,
0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72,
0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61,
0x6e, 0x77, 0x1ee5, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x47, 0x75, 0x6c, 0x66, 0x4f, 0x67, 0x65, 0x20, 0x47, 0x75, 0x79, 0x61,
0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x4f, 0x67,
0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x67, 0x65, 0x20, 0x4f,
0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x67, 0x65, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x73, 0x69, 0x6d, 0x69, 0x72, 0x69,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x45,
0x74, 0x69, 0x74, 0x69, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61,
0x72, 0x61, 0x20, 0x1ecc, 0x77, 0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79,
0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61,
0x72, 0x61, 0x20, 0x1ecc, 0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79,
0x61, 0x6e, 0x77, 0x1ee5, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b,
0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65,
0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x4f, 0x67, 0x65, 0x20, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x68, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x4f, 0x67, 0x65,
0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc, 0x77,
0x1ee5, 0x77, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67,
0x65, 0x20, 0x4d, 0x70, 0x61, 0x67, 0x68, 0x61, 0x72, 0x61, 0x20, 0x1ecc,
0x64, 0x1ecb, 0x64, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x6e, 0x77, 0x1ee5,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x67, 0x65, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4b, 0x6f, 0x72, 0x65,
0x61, 0x4f, 0x67, 0x65, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x4f,
0x67, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x4f, 0x67, 0x65, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x7a, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x67, 0x65, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b,
0x1ee5, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x67, 0x65,
0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x4f, 0x67, 0x65,
0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x4f, 0x67, 0x65,
0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x4f, 0x67,
0x65, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d,
0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4f,
0x67, 0x65, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x68, 0x65, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x4f, 0x67, 0x65, 0x20, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x55, 0x6c, 0x61,
0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x55, 0x6c, 0x61,
0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x4f, 0x67, 0x65, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4f,
0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x4f, 0x67, 0x65, 0x20, 0x4d, 0x79,
0x61, 0x6e, 0x6d, 0x61, 0x72, 0x4f, 0x67, 0x65, 0x20, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x4f,
0x67, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x68, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61,
0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65,
0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x67, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x4f, 0x67, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x67, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x67, 0x65,
0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x67, 0x65,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75,
0x67, 0x62, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x4f, 0x67, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20,
0x4e, 0x65, 0x77, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70,
0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x4f, 0x67, 0x65, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x50, 0x65, 0x72,
0x75, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b,
0x1ee5, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x50, 0x68,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70,
0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70,
0x70, 0x69, 0x6e, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x50, 0x68, 0x6f, 0x65,
0x6e, 0x69, 0x78, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f,
0x67, 0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x68, 0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x4f, 0x67, 0x65, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x4f,
0x67, 0x65, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67,
0x4f, 0x67, 0x65, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x4f,
0x67, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x67,
0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70,
0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x53, 0x65, 0x79, 0x63,
0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a,
0x75, 0x67, 0x62, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f,
0x72, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x4f, 0x67, 0x65,
0x20, 0x53, 0x6f, 0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61,
0x6d, 0x65, 0x4f, 0x67, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x4f,
0x67, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x4f, 0x67, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x68, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x4f, 0x67, 0x65, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x67, 0x65, 0x20, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f,
0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x4f, 0x67, 0x65,
0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b,
0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x4f, 0x67, 0x65, 0x20, 0x54, 0x75, 0x76,
0x61, 0x6c, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x4f, 0x67, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67,
0x62, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x4f,
0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x4f, 0x67, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x67, 0x65, 0x20, 0x49,
0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x4f, 0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b,
0x1ee5, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x67, 0x65,
0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x4f, 0x67,
0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f,
0x67, 0x65, 0x20, 0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f,
0x67, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x56, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x57, 0x61, 0x6b, 0x65,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x67, 0x65, 0x20, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x26, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x4f, 0x67, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x4f, 0x67, 0x65, 0x20, 0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x4f,
0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x4f, 0x67, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x67, 0x65, 0x20,
0x49, 0x7a, 0x75, 0x67, 0x62, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x67, 0x65, 0x20,
0x4f, 0x6b, 0x70, 0x6f, 0x6d, 0x1ecd, 0x6b, 0x1ee5, 0x20, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x67,
0x65, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x41, 0x63, 0x72, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x63, 0x72, 0x65, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x63, 0x72, 0x65, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x54, 0x69, 0x6d,
0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53,
0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x61, 0x72,
0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x69,
0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x65, 0x67, 0x75, 0x6e, 0x75, 0x6e, 0x67, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x50, 0x65, 0x67, 0x75, 0x6e, 0x75, 0x6e, 0x67, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x65, 0x67, 0x75, 0x6e,
0x75, 0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x41, 0x70, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x70,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x71, 0x74, 0x61,
0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x71, 0x74,
0x6f, 0x62, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x71, 0x74,
0x6f, 0x62, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x61,
0x62, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x72, 0x61, 0x62, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x41, 0x72, 0x61, 0x62, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x42, 0x61, 0x67, 0x69, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x72,
0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x42, 0x61, 0x67, 0x69, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x72,
0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x42, 0x61, 0x67, 0x69, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61,
0x68, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x20, 0x54, 0x65, 0x6e,
0x67, 0x61, 0x68, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x20, 0x54, 0x65, 0x6e,
0x67, 0x61, 0x68, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54,
0x69, 0x6d, 0x75, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x61,
0x72, 0x61, 0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x42, 0x61, 0x72, 0x61, 0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x68, 0x75,
0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x6f, 0x6c,
0x69, 0x76, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x61, 0x6e, 0x6a,
0x75, 0x6e, 0x67, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x61,
0x73, 0x65, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x43, 0x69, 0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x43, 0x69,
0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x69, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x69, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x43, 0x68,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20,
0x4e, 0x61, 0x74, 0x61, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x63,
0x6f, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x70, 0x2e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x65, 0x70,
0x2e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b, 0x65, 0x70, 0x2e, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x75,
0x62, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x50, 0x61, 0x73, 0x6b,
0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x50, 0x61,
0x73, 0x6b, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x75,
0x6c, 0x61, 0x75, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x45, 0x6b, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54,
0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61,
0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x45, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x20, 0x4a, 0x61, 0x75, 0x68, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x42, 0x61,
0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x42,
0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x45, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x50, 0x72, 0x61, 0x6e,
0x63, 0x69, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x57, 0x69, 0x6c,
0x61, 0x79, 0x61, 0x68, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e,
0x20, 0x64, 0x61, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74,
0x69, 0x6b, 0x61, 0x20, 0x50, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x73, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x2e,
0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x42,
0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x47, 0x75, 0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x65, 0x6c, 0x75, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x48, 0x6f, 0x76, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61,
0x6d, 0x75, 0x64, 0x65, 0x72, 0x61, 0x20, 0x48, 0x69, 0x6e, 0x64, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63,
0x68, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4a,
0x65, 0x70, 0x61, 0x6e, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4a, 0x65, 0x70, 0x61, 0x6e,
0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4a, 0x65, 0x70, 0x61, 0x6e,
0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x79, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x75,
0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x69, 0x72,
0x67, 0x69, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x70, 0x2e, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x6b, 0x61,
0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x64, 0x65, 0x77, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61,
0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x2e, 0x20,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4d, 0x6f,
0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4d, 0x6f,
0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x79,
0x61, 0x6e, 0x6d, 0x61, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x75, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x42,
0x61, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x75, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x42, 0x61, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x75, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x42, 0x61, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4e, 0x69, 0x75, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75,
0x6c, 0x61, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x75, 0x6c, 0x61,
0x75, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x70, 0x2e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61,
0x6e, 0x61, 0x20, 0x55, 0x74, 0x61, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x75,
0x67, 0x69, 0x6e, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x65, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61,
0x6e, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x6e, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x64, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x6f,
0x6e, 0x61, 0x70, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x79,
0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65,
0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69,
0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20,
0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x53, 0x65, 0x6c,
0x61, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54,
0x61, 0x68, 0x69, 0x74, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54,
0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73,
0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c,
0x61, 0x75, 0x61, 0x6e, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x64, 0x61,
0x6e, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45,
0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x64, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x57, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20,
0x64, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x65,
0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6d,
0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x69, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x69, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x69, 0x61, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x70, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20,
0x41, 0x70, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x61, 0x72, 0x61,
0x62, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x72, 0x61, 0x62, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x61, 0x72, 0x61,
0x62, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20,
0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x61, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x2d,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x2d, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x6f, 0x2d, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x68, 0x61, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x7a,
0x68, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65,
0x69, 0x64, 0x7a, 0x68, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x6f, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61,
0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43,
0x61, 0x70, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x45, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x63, 0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x63, 0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x68, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x75, 0x6c,
0x74, 0x72, 0x61, 0x2d, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75,
0x6c, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63,
0x65, 0x73, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x66, 0x72, 0x61, 0x6e,
0x63, 0x65, 0x73, 0x65, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x6f,
0x6e, 0x61, 0x6c, 0x20, 0x65, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x63,
0x74, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c,
0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x6d, 0x65, 0x64, 0x69, 0x65, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x6f, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x49, 0x72, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45, 0x73,
0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x57, 0x65, 0x73, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72,
0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x68, 0x69, 0x7a,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x6f,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x6f,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x6f, 0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x65, 0x20, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0x75, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x50, 0x65, 0x72, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0x75,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x68, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x50, 0x68, 0x65,
0x6e, 0x69, 0x63, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x6f, 0x6e, 0x61, 0x70, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53, 0x75, 0x64,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76,
0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64,
0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69,
0x76, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20,
0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x65, 0x20, 0x64, 0x65, 0x20,
0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6e, 0x20, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6e, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6e, 0x20, 0x65, 0x73, 0x74,
0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x65,
0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x20, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x6d, 0x6f, 0x6e, 0x74,
0x61, 0x6e, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x70, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c,
0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x65, 0x73,
0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c,
0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x63, 0x75, 0x62, 0x61, 0x6e, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x63, 0x75, 0x62, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x63, 0x75, 0x62,
0x61, 0x6e, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x6e, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x6f, 0x73, 0x74, 0x2d, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x6e, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x6f, 0x73,
0x74, 0x2d, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x6f, 0x73, 0x74, 0x2d, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x77, 0x65, 0x73, 0x74, 0x2d, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x6e, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x77, 0x65,
0x73, 0x74, 0x2d, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x6e, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x77, 0x65, 0x73, 0x74, 0x2d, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x6e, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65,
0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x65, 0x20, 0x6d, 0x65, 0x64, 0x69, 0x61, 0x6c, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x73, 0x74,
0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x20,
0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x73, 0x74, 0xe9, 0x6d,
0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20,
0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x65, 0x73, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73,
0x20, 0x4c, 0x69, 0x6e, 0x65, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d,
0x48, 0x6f, 0x77, 0x65, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49,
0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x70,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x70, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x6d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x6e, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20,
0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f,
0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x74, 0xe9,
0x6d, 0x70, 0x6f, 0x72, 0x20, 0x65, 0x73, 0x74, 0x69, 0x76, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73,
0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x73,
0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x74, 0xe9, 0x6d, 0x70,
0x6f, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c,
0x20, 0x57, 0x61, 0x6b, 0x65, 0x74, 0xe9, 0x6d, 0x70, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x41, 0x6d, 0x20, 0x41,
0x63, 0x72, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x41, 0x63, 0x72, 0x65, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x41, 0x63, 0x72, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x66, 0x72, 0x61, 0x69, 0x63,
0x65, 0x20, 0x4c, 0xe1, 0x69, 0x72, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72,
0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x66,
0x72, 0x61, 0x69, 0x63, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x41, 0x66, 0x72, 0x61, 0x69, 0x63, 0x65, 0x20, 0x54, 0x68, 0x65, 0x61,
0x73, 0x41, 0x6d, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x41, 0x66, 0x72, 0x61, 0x69, 0x63, 0x65, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x41, 0x66, 0x72, 0x61, 0x69, 0x63, 0x65, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49,
0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41,
0x66, 0x72, 0x61, 0x69, 0x63, 0x65, 0x41, 0x6d, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x63, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63,
0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x41, 0x6d, 0x20, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x41, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x6d, 0x61, 0x73, 0xf3, 0x69, 0x6e,
0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x6d, 0x61,
0x73, 0xf3, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x6d,
0x61, 0x73, 0xf3, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x4c, 0xe1, 0x72,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63,
0x65, 0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x4c, 0xe1, 0x72, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68,
0x65, 0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20, 0x54, 0x68, 0x75, 0x61,
0x69, 0x64, 0x68, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x4c, 0xe1, 0x72, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20, 0x54, 0x68,
0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74,
0x68, 0x65, 0x61, 0x72, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x69,
0x72, 0x69, 0x63, 0x65, 0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64,
0x68, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61,
0x72, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63,
0x65, 0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f,
0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x61, 0x63, 0x68, 0x20, 0x4d,
0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20, 0x54, 0x68, 0x75,
0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20, 0x53, 0x6c, 0xe9, 0x69, 0x62,
0x68, 0x74, 0x65, 0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65,
0xe1, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x53, 0x6c, 0xe9, 0x69, 0x62, 0x68, 0x74, 0x65, 0x20, 0x4d, 0x68,
0x65, 0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20, 0x54, 0x68, 0x75, 0x61,
0x69, 0x64, 0x68, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x53, 0x6c, 0xe9, 0x69, 0x62, 0x68, 0x74, 0x65,
0x20, 0x4d, 0x68, 0x65, 0x69, 0x72, 0x69, 0x63, 0x65, 0xe1, 0x20, 0x54,
0x68, 0x75, 0x61, 0x69, 0x64, 0x68, 0x41, 0x6d, 0x20, 0x61, 0x6e, 0x20,
0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0xfa, 0x69,
0x6e, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x69, 0x67, 0xe9,
0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0xfa, 0x69, 0x6e, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x61, 0x6e,
0x20, 0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0xfa,
0x69, 0x6e, 0x41, 0x6d, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x41, 0x6d, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x70, 0x69, 0x61,
0x41, 0x6d, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x41, 0x71, 0x74, 0x61, 0x75, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x41,
0x6d, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62,
0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x72, 0x61, 0x69,
0x62, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x72,
0x61, 0x69, 0x62, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x72, 0x61,
0x69, 0x62, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69,
0x72, 0x67, 0x69, 0x6e, 0x74, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x74, 0xed,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x67, 0x69,
0x6e, 0x74, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x49, 0x61, 0x72, 0x74,
0x68, 0x61, 0x72, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41,
0x69, 0x72, 0x67, 0x69, 0x6e, 0x74, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x61, 0x63, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x74, 0xed,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x61, 0x63,
0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x67, 0x69, 0x6e,
0x74, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41,
0x69, 0x72, 0x6d, 0xe9, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x6d, 0xe9, 0x69, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x41, 0x69, 0x72, 0x6d, 0xe9, 0x69, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x61, 0x69, 0x67, 0x68, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x61, 0x6e, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x67, 0x68, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x61, 0x6e,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x61, 0x69, 0x67, 0x68, 0x41,
0x6d, 0x20, 0x4c, 0xe1, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73,
0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4c, 0xe1,
0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69,
0x6c, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x4c, 0xe1, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41,
0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x4d, 0x68,
0x65, 0xe1, 0x6e, 0x69, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x4d, 0x68, 0x65, 0xe1, 0x6e, 0x69, 0x61, 0x72, 0x74,
0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72,
0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x68, 0x65, 0xe1, 0x6e, 0x69, 0x61,
0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73,
0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72,
0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73,
0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69,
0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41,
0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69, 0x72, 0x74,
0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74,
0x72, 0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x49, 0x61, 0x72, 0x74,
0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72,
0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x49, 0x61, 0x72, 0x74,
0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72,
0xe1, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x74, 0x72, 0xe1, 0x69, 0x6c,
0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x61, 0x72,
0x62, 0x61, 0x69, 0x73, 0x65, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x41, 0x73, 0x61, 0x72, 0x62, 0x61, 0x69,
0x73, 0x65, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x41,
0x73, 0x61, 0x72, 0x62, 0x61, 0x69, 0x73, 0x65, 0xe1, 0x69, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x41, 0x73, 0xf3, 0x72, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x41, 0x73, 0xf3, 0x72, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x6e, 0x41, 0x73, 0xf3, 0x72, 0x41, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x69, 0x64, 0xe9, 0x69,
0x73, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x69, 0x64, 0xe9, 0x69, 0x73, 0x65, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x69, 0x64, 0xe9, 0x69, 0x73,
0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x42, 0xfa, 0x74, 0xe1, 0x69,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x61,
0x69, 0x76, 0x65, 0x41, 0x6d, 0x20, 0x42, 0x68, 0x72, 0x61, 0x73, 0xed,
0x6c, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x42, 0x68, 0x72, 0x61, 0x73,
0xed, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x42, 0x68, 0x72, 0x61, 0x73, 0xed, 0x6c,
0x69, 0x61, 0x41, 0x6d, 0x20, 0x42, 0x68, 0x72, 0xfa, 0x69, 0x6e, 0xe9,
0x20, 0x44, 0x61, 0x72, 0x61, 0x73, 0x61, 0x6c, 0xe1, 0x6d, 0x41, 0x6d,
0x20, 0x52, 0x69, 0x6e, 0x6e, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x52, 0x69, 0x6e, 0x6e, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x52, 0x69, 0x6e, 0x6e, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x41, 0x6d, 0x20, 0x53, 0x74, 0xe1, 0x69, 0x73, 0x69, 0xfa, 0x6e, 0x20,
0x43, 0x61, 0x73, 0x65, 0x79, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x53, 0x65, 0x61,
0x6d, 0xf3, 0x72, 0x61, 0x63, 0x68, 0x41, 0x6d, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x41,
0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x6e, 0x61, 0x20, 0x53, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20,
0x53, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0xed,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0xed, 0x6e,
0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x41,
0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x6e, 0x61, 0x20,
0x4e, 0x6f, 0x6c, 0x6c, 0x61, 0x67, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c,
0x65, 0xe1, 0x69, 0x6e, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x41, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf3, 0x69, 0x6d, 0x65, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf3, 0x69, 0x6d,
0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x6e, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf3, 0x69, 0x6d, 0x65,
0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1,
0x69, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x41, 0x6d, 0x20, 0x4c, 0x65,
0x61, 0x74, 0x68, 0x73, 0x68, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x68, 0xfa, 0x62, 0x61, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x43, 0x68, 0xfa, 0x62, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x43, 0x68, 0xfa, 0x62,
0x61, 0x41, 0x6d, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x41, 0x6d, 0x20,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x41, 0x6d, 0x20, 0x54, 0x68, 0xed, 0x6f, 0x6d,
0xf3, 0x72, 0x20, 0x54, 0x68, 0x6f, 0x69, 0x72, 0x41, 0x6d, 0x20, 0x4f,
0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x43, 0xe1, 0x73,
0x63, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e,
0x20, 0x6e, 0x61, 0x20, 0x43, 0xe1, 0x73, 0x63, 0x61, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x43, 0xe1, 0x73, 0x63,
0x61, 0x41, 0x6d, 0x20, 0x45, 0x61, 0x63, 0x75, 0x61, 0x64, 0xf3, 0x72,
0x41, 0x6d, 0x20, 0x4c, 0xe1, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45,
0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4c, 0xe1, 0x72, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4c, 0xe1,
0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41,
0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20,
0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68,
0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70,
0x61, 0x41, 0x6d, 0x20, 0x43, 0x68, 0x69, 0x61, 0x6e, 0x6f, 0x69, 0x72,
0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f,
0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61,
0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e,
0x61, 0x20, 0x68, 0x45, 0x6f, 0x72, 0x70, 0x61, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x61, 0x72,
0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x45, 0x6f, 0x72,
0x70, 0x61, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e,
0x20, 0x46, 0x68, 0xe1, 0x63, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x46,
0x68, 0xe1, 0x63, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x46, 0x68, 0xe1, 0x63, 0x6c, 0x61,
0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x46, 0x68, 0x69, 0x64, 0x73,
0xed, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x46, 0x68, 0x69, 0x64, 0x73, 0xed, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x46, 0x68, 0x69, 0x64, 0x73, 0xed, 0x41, 0x6d, 0x20, 0x47, 0x68, 0x75,
0xe1, 0x69, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x69, 0x6e,
0x63, 0x65, 0x41, 0x6d, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x61, 0x63,
0x68, 0x20, 0x44, 0x68, 0x65, 0x69, 0x73, 0x63, 0x65, 0x61, 0x72, 0x74,
0x20, 0x61, 0x6e, 0x20, 0x44, 0x6f, 0x6d, 0x68, 0x61, 0x69, 0x6e, 0x20,
0x61, 0x67, 0x75, 0x73, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61,
0x72, 0x74, 0x61, 0x69, 0x67, 0x68, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c,
0x65, 0xe1, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x41, 0x6d, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x65, 0x6f, 0x69, 0x72, 0x73,
0x69, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x65, 0x6f,
0x69, 0x72, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x53, 0x65, 0x6f,
0x69, 0x72, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x43, 0x68, 0x69, 0x72,
0x65, 0x61, 0x62, 0x61, 0x69, 0x74, 0xed, 0x4d, 0x65, 0xe1, 0x6e, 0x2d,
0x41, 0x6d, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20,
0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65,
0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c,
0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65,
0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e, 0x6c,
0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x49, 0x61, 0x72, 0x74,
0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f, 0x6e,
0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x49, 0x61,
0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61,
0x6f, 0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53,
0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x61, 0x72,
0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x72, 0x61, 0x6f,
0x6e, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x47,
0x68, 0x75, 0x61, 0x6d, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x4d,
0x75, 0x72, 0x61, 0x73, 0x63, 0x61, 0x69, 0x6c, 0x6c, 0x65, 0x41, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x47, 0x75, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d,
0x20, 0x48, 0x61, 0x76, 0xe1, 0xed, 0x2d, 0x41, 0x69, 0x6c, 0x69, 0xfa,
0x69, 0x74, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x48, 0x61, 0x76, 0xe1, 0xed, 0x2d,
0x41, 0x69, 0x6c, 0x69, 0xfa, 0x69, 0x74, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x48, 0x61, 0x76, 0xe1,
0xed, 0x2d, 0x41, 0x69, 0x6c, 0x69, 0xfa, 0x69, 0x74, 0x41, 0x6d, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x43, 0x6f, 0x6e, 0x67, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x43, 0x6f, 0x6e, 0x67, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x43, 0x6f, 0x6e, 0x67, 0x41, 0x6d, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x49, 0x6e, 0x64, 0x69, 0x61, 0x41, 0x6d, 0x20, 0x61, 0x6e, 0x20, 0x41,
0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x69,
0x67, 0x68, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x49, 0x6e, 0x64,
0x2d, 0x53, 0xed, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x4c, 0xe1, 0x72, 0x20,
0x6e, 0x61, 0x20, 0x68, 0x49, 0x6e, 0x64, 0x69, 0x6e, 0xe9, 0x69, 0x73,
0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x49, 0x6e, 0x64, 0x69, 0x6e, 0xe9, 0x69,
0x73, 0x65, 0x41, 0x6d, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72,
0x20, 0x6e, 0x61, 0x20, 0x68, 0x49, 0x6e, 0x64, 0x69, 0x6e, 0xe9, 0x69,
0x73, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0x49, 0x61, 0x72,
0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x49, 0x61, 0x72, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68,
0x49, 0x61, 0x72, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x41, 0x6d, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x49, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49, 0x6f,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53,
0x65, 0x61, 0x70, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e,
0x61, 0x20, 0x53, 0x65, 0x61, 0x70, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e,
0x61, 0x20, 0x53, 0x65, 0x61, 0x70, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d,
0x20, 0x50, 0x68, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x50, 0x68, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50, 0x68, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x41, 0x6d, 0x20,
0x4f, 0x69, 0x72, 0x74, 0x68, 0x65, 0x61, 0x72, 0x20, 0x6e, 0x61, 0x20,
0x43, 0x61, 0x73, 0x61, 0x63, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x49, 0x61, 0x72, 0x74, 0x68, 0x61, 0x72, 0x20, 0x6e, 0x61,
0x20, 0x43, 0x61, 0x73, 0x61, 0x63, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x43, 0xf3, 0x69, 0x72, 0xe9, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x43, 0xf3, 0x69, 0x72, 0xe9, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x43, 0xf3, 0x69, 0x72, 0xe9, 0x41, 0x6d, 0x20, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x41, 0x6d, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d,
0x20, 0x6e, 0x61, 0x20, 0x43, 0x69, 0x72, 0x67, 0x65, 0x61, 0x73, 0x74,
0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x68, 0x72, 0xed, 0x20,
0x4c, 0x61, 0x6e, 0x63, 0x61, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65,
0xe1, 0x69, 0x6e, 0x20, 0x6e, 0x61, 0x20, 0x4c, 0xed, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d,
0x20, 0x4d, 0x68, 0x61, 0x63, 0x61, 0x6f, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d,
0x68, 0x61, 0x63, 0x61, 0x6f, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4d, 0x68, 0x61, 0x63, 0x61, 0x6f,
0x41, 0x6d, 0x20, 0x4d, 0x68, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x4d, 0x68, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x4d, 0x68, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x20, 0x6e,
0x61, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x65, 0x69, 0x73, 0x69, 0x61, 0x41,
0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x4d, 0x68,
0x61, 0x69, 0x6c, 0x64, 0xed, 0x76, 0x65, 0x41, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x6e, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d, 0x61, 0x72,
0x63, 0x61, 0x73, 0x61, 0x63, 0x68, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c,
0x65, 0xe1, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d,
0x68, 0x75, 0x69, 0x72, 0xed, 0x73, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d, 0x68, 0x75, 0x69, 0x72, 0xed, 0x73,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d, 0x68, 0x75, 0x69,
0x72, 0xed, 0x73, 0x41, 0x6d, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x41, 0x6d, 0x20, 0x4d, 0x65, 0x69, 0x63, 0x73, 0x69, 0x63, 0x65, 0x61,
0x63, 0x68, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e,
0x20, 0x43, 0x68, 0x69, 0xfa, 0x69, 0x6e, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d,
0x65, 0x69, 0x63, 0x73, 0x69, 0x63, 0x65, 0x61, 0x63, 0x68, 0x20, 0x61,
0x6e, 0x20, 0x41, 0x69, 0x67, 0xe9, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69,
0xfa, 0x69, 0x6e, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x4d, 0x65, 0x69, 0x63, 0x73, 0x69, 0x63, 0x65,
0x61, 0x63, 0x68, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x69, 0x67, 0xe9, 0x69,
0x6e, 0x20, 0x43, 0x68, 0x69, 0xfa, 0x69, 0x6e, 0x41, 0x6d, 0x20, 0x55,
0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x55, 0x6c, 0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x41, 0x6d, 0x20,
0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x55, 0x6c,
0xe1, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x41, 0x6d, 0x20, 0x4d, 0x68,
0x6f, 0x73, 0x63, 0xf3, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4d, 0x68, 0x6f, 0x73,
0x63, 0xf3, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x4d, 0x68, 0x6f, 0x73, 0x63, 0xf3, 0x41, 0x6d, 0x20,
0x4d, 0x68, 0x61, 0x65, 0x6e, 0x6d, 0x61, 0x72, 0x41, 0x6d, 0x20, 0x4e,
0xe1, 0x72, 0xfa, 0x41, 0x6d, 0x20, 0x4e, 0x65, 0x69, 0x70, 0x65, 0x61,
0x6c, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x75, 0x61, 0x2d, 0x43,
0x68, 0x61, 0x6c, 0x61, 0x64, 0xf3, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x6e, 0x61, 0x20, 0x4e, 0x75, 0x61, 0x2d, 0x43, 0x68, 0x61, 0x6c,
0x61, 0x64, 0xf3, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x75,
0x61, 0x2d, 0x43, 0x68, 0x61, 0x6c, 0x61, 0x64, 0xf3, 0x69, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x75, 0x61, 0x2d, 0x53, 0x68,
0xe9, 0x61, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x4e, 0x75, 0x61, 0x2d, 0x53, 0x68, 0xe9, 0x61, 0x6c,
0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x4e, 0x75, 0x61,
0x2d, 0x53, 0x68, 0xe9, 0x61, 0x6c, 0x61, 0x69, 0x6e, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x54, 0x68, 0x61, 0x6c, 0x61, 0x6d, 0x68, 0x20, 0x61, 0x6e,
0x20, 0xc9, 0x69, 0x73, 0x63, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x54, 0x68, 0x61,
0x6c, 0x61, 0x6d, 0x68, 0x20, 0x61, 0x6e, 0x20, 0xc9, 0x69, 0x73, 0x63,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x54, 0x68, 0x61, 0x6c, 0x61, 0x6d, 0x68, 0x20, 0x61, 0x6e, 0x20,
0xc9, 0x69, 0x73, 0x63, 0x41, 0x6d, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x41,
0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x69, 0x6c, 0x65,
0xe1, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f,
0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x41, 0x6d, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20,
0x6e, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x4d, 0xe1, 0x69, 0x72,
0x69, 0x61, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x54, 0x68, 0x75, 0x61, 0x69,
0x64, 0x68, 0x41, 0x6d, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65,
0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x50, 0x61, 0x63,
0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e,
0x61, 0x20, 0x50, 0x61, 0x63, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x6e, 0x61, 0x20, 0x50, 0x61, 0x63, 0x61, 0x73, 0x74, 0xe1, 0x69,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e,
0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x41, 0x6d, 0x20, 0x4e, 0x75, 0x61,
0x2d, 0x47, 0x68, 0x75, 0x69, 0x6e, 0x65, 0x20, 0x50, 0x68, 0x61, 0x70,
0x75, 0x61, 0x41, 0x6d, 0x20, 0x50, 0x68, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x50, 0x68, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x50, 0x68, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x41, 0x6d,
0x20, 0x50, 0x68, 0x65, 0x69, 0x72, 0x69, 0xfa, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x50, 0x68, 0x65, 0x69, 0x72, 0x69, 0xfa, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x50, 0x68, 0x65, 0x69,
0x72, 0x69, 0xfa, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x4f, 0x69,
0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e,
0x65, 0x61, 0x63, 0x68, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x6e,
0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0xed, 0x6e, 0x65, 0x61, 0x63, 0x68, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x6e, 0x4f,
0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0xed,
0x6e, 0x65, 0x61, 0x63, 0x68, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65,
0xe1, 0x69, 0x6e, 0x20, 0x61, 0x6e, 0x20, 0x46, 0x68, 0xe9, 0x69, 0x6e,
0x69, 0x63, 0x73, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65,
0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d,
0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x41,
0x6d, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x41, 0x6d,
0x20, 0x50, 0x68, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x41, 0x6d, 0x20,
0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x41, 0x6d, 0x20,
0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x51,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x41, 0x6d, 0x20, 0x52,
0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x41, 0x6d, 0x20, 0x52, 0x6f, 0x74,
0x68, 0x65, 0x72, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x68, 0x61, 0x63, 0x61,
0x69, 0x6c, 0xed, 0x6e, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x53, 0x68, 0x61, 0x63,
0x61, 0x69, 0x6c, 0xed, 0x6e, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68,
0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x53, 0x68, 0x61, 0x63, 0x61, 0x69,
0x6c, 0xed, 0x6e, 0x41, 0x6d, 0x20, 0x53, 0x68, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1,
0x6e, 0x61, 0x63, 0x68, 0x20, 0x53, 0x68, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68,
0x20, 0x53, 0x68, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x41, 0x6d, 0x20, 0x53,
0x68, 0x61, 0x6d, 0xf3, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x53, 0x68, 0x61, 0x6d,
0xf3, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64,
0x68, 0x20, 0x53, 0x68, 0x61, 0x6d, 0xf3, 0x41, 0x6d, 0x20, 0x6e, 0x61,
0x20, 0x53, 0xe9, 0x69, 0x73, 0xe9, 0x61, 0x6c, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x53, 0x68, 0x69, 0x6e, 0x67, 0x65, 0x61, 0x70, 0xf3, 0x72, 0x41, 0x6d,
0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x69, 0x6e, 0x20, 0x53, 0x68, 0x6f,
0x6c, 0x61, 0x69, 0x6d, 0x68, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x53,
0x65, 0x6f, 0x69, 0x72, 0x73, 0x69, 0x61, 0x20, 0x54, 0x68, 0x65, 0x61,
0x73, 0x41, 0x6d, 0x20, 0x53, 0x68, 0x75, 0x72, 0x61, 0x6e, 0x61, 0x6d,
0x41, 0x6d, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x41, 0x6d, 0x20, 0x54,
0x68, 0x61, 0x69, 0x68, 0xed, 0x74, 0xed, 0x41, 0x6d, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68,
0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69,
0x64, 0x68, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x41, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x54, 0xe1, 0x69, 0x64, 0x73, 0xed, 0x63, 0x65, 0x61,
0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c,
0x65, 0xe1, 0x69, 0x6e, 0x20, 0x54, 0xf3, 0x63, 0x61, 0x6c, 0xe1, 0x41,
0x6d, 0x20, 0x54, 0x68, 0x6f, 0x6e, 0x67, 0x61, 0x41, 0x6d, 0x20, 0x43,
0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20,
0x54, 0x68, 0x6f, 0x6e, 0x67, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x54, 0x68, 0x6f, 0x6e, 0x67,
0x61, 0x41, 0x6d, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x41, 0x6d, 0x20,
0x6e, 0x61, 0x20, 0x54, 0x75, 0x69, 0x72, 0x63, 0x6d, 0xe9, 0x61, 0x6e,
0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x43, 0x61,
0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x6e,
0x61, 0x20, 0x54, 0x75, 0x69, 0x72, 0x63, 0x6d, 0xe9, 0x61, 0x6e, 0x61,
0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x54, 0x75,
0x69, 0x72, 0x63, 0x6d, 0xe9, 0x61, 0x6e, 0x61, 0x73, 0x74, 0xe1, 0x69,
0x6e, 0x65, 0x41, 0x6d, 0x20, 0x54, 0x68, 0xfa, 0x76, 0x61, 0x6c, 0xfa,
0x41, 0x6d, 0x20, 0x55, 0x72, 0x61, 0x67, 0x75, 0x61, 0x41, 0x6d, 0x20,
0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68,
0x20, 0x55, 0x72, 0x61, 0x67, 0x75, 0x61, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x55, 0x72, 0x61, 0x67,
0x75, 0x61, 0x41, 0x6d, 0x20, 0x6e, 0x61, 0x20, 0x68, 0xda, 0x69, 0x73,
0x62, 0xe9, 0x69, 0x63, 0x65, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65,
0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e,
0x61, 0x63, 0x68, 0x20, 0x6e, 0x61, 0x20, 0x68, 0xda, 0x69, 0x73, 0x62,
0xe9, 0x69, 0x63, 0x65, 0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41,
0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20,
0x6e, 0x61, 0x20, 0x68, 0xda, 0x69, 0x73, 0x62, 0xe9, 0x69, 0x63, 0x65,
0x61, 0x73, 0x74, 0xe1, 0x69, 0x6e, 0x65, 0x41, 0x6d, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0xfa, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67,
0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0xfa, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72,
0x61, 0x69, 0x64, 0x68, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0xfa,
0x41, 0x6d, 0x20, 0x56, 0x65, 0x69, 0x6e, 0x69, 0x73, 0xe9, 0x61, 0x6c,
0x61, 0x41, 0x6d, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64,
0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d,
0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6d, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x41, 0x6d, 0x20, 0x43, 0x61, 0x69,
0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63, 0x68, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x41, 0x6d, 0x20, 0x53, 0x61,
0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x41, 0x6d, 0x20, 0x56, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x41, 0x6d, 0x20, 0x4f, 0x69, 0x6c, 0x65, 0xe1, 0x6e, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x41, 0x6d, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x20, 0x61, 0x67, 0x75, 0x73, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e,
0x61, 0x41, 0x6d, 0x20, 0x49, 0x61, 0x63, 0xfa, 0x74, 0x73, 0x63, 0x41,
0x6d, 0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61,
0x63, 0x68, 0x20, 0x49, 0x61, 0x63, 0xfa, 0x74, 0x73, 0x63, 0x41, 0x6d,
0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61, 0x69, 0x64, 0x68, 0x20, 0x49,
0x61, 0x63, 0xfa, 0x74, 0x73, 0x63, 0x41, 0x6d, 0x20, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6d,
0x20, 0x43, 0x61, 0x69, 0x67, 0x68, 0x64, 0x65, 0xe1, 0x6e, 0x61, 0x63,
0x68, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x41, 0x6d, 0x20, 0x53, 0x61, 0x6d, 0x68, 0x72, 0x61,
0x69, 0x64, 0x68, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6d, 0x20, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66,
0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x6f, 0x6e, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x41, 0x6d, 0x61, 0x7a, 0x7a, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x7a, 0x6f, 0x6e, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0x7a, 0x6f, 0x6e, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41,
0x4f, 0x72, 0x61, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65,
0x20, 0x52, 0x6f, 0x63, 0x63, 0x69, 0x6f, 0x73, 0x65, 0x20, 0x55, 0x53,
0x41, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x20, 0x52,
0x6f, 0x63, 0x63, 0x69, 0x6f, 0x73, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x4d, 0x6f,
0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x20, 0x52, 0x6f, 0x63, 0x63, 0x69,
0x6f, 0x73, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20,
0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x6f, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x55, 0x53, 0x41, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x41, 0x70, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x69, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x61,
0x72, 0x61, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x72, 0x61, 0x62, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x61, 0x72, 0x61,
0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x67, 0x69, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x67, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x67, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x41, 0x7a, 0x7a, 0x6f, 0x72, 0x72,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x41, 0x7a, 0x7a, 0x6f,
0x72, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x41, 0x7a, 0x7a, 0x6f,
0x72, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x42, 0x68, 0x75,
0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61,
0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69,
0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20,
0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x43, 0x61, 0x70, 0x6f, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x43, 0x61, 0x70, 0x6f, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x43, 0x61, 0x70, 0x6f, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x43,
0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x43, 0x69, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x61, 0x20, 0x43, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x61, 0x20, 0x43, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61,
0x20, 0x43, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x69, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x73, 0x6f,
0x6c, 0x61, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73,
0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x43, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x43,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x69, 0x73,
0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x6d, 0x65, 0x64, 0x69, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x6c, 0x65,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x43, 0x75, 0x62, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x69, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x73, 0x6f, 0x6c, 0x61, 0x20,
0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x2019, 0x49, 0x73, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20,
0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x73,
0x6f, 0x6c, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x63,
0x75, 0x61, 0x64, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x20, 0x28, 0x4b, 0x61, 0x6c, 0x69, 0x6e,
0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73,
0x6f, 0x6c, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69,
0x67, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x67,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x67, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x75, 0x69, 0x61,
0x6e, 0x61, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x65, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x20,
0x65, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x63, 0x68, 0x65,
0x20, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x69,
0x73, 0x6f, 0x6c, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6d, 0x65, 0x72, 0x69,
0x64, 0x69, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x6f,
0x6c, 0x66, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61,
0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f,
0x6c, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49,
0x73, 0x6f, 0x6c, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x69, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x4f, 0x63, 0x65,
0x61, 0x6e, 0x6f, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x6f, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f,
0x63, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019,
0x49, 0x72, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x69, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69,
0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x6f, 0x6e, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x6f, 0x6e,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x6f, 0x6e, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x63, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x63,
0x6f, 0x72, 0x65, 0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x63, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4b, 0x69, 0x72, 0x67,
0x68, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x53, 0x70, 0x6f, 0x72, 0x61, 0x64,
0x69, 0x20, 0x65, 0x71, 0x75, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x4d, 0x61, 0x63, 0x61, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x63,
0x61, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x65, 0x73, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x73, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f,
0x6c, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x28, 0x4d, 0x65, 0x73, 0x73, 0x69,
0x63, 0x6f, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x63, 0x6f, 0x20, 0x28, 0x4d, 0x65, 0x73, 0x73, 0x69, 0x63,
0x6f, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x6f, 0x20, 0x28, 0x4d, 0x65, 0x73, 0x73, 0x69, 0x63, 0x6f, 0x29, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x42, 0x69, 0x72, 0x6d,
0x61, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x61, 0x20, 0x4e, 0x75, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61,
0x20, 0x4e, 0x75, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x75, 0x6f,
0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x75,
0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x75, 0x6f, 0x76, 0x61, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20,
0x4e, 0x75, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x65, 0x72, 0x72,
0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x54, 0x65, 0x72,
0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x54, 0x65, 0x72, 0x72,
0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x4e, 0x69, 0x75, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c,
0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73,
0x6f, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x61, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x75,
0x6f, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x65,
0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x65, 0x20, 0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x61, 0x20, 0x46, 0x65, 0x6e, 0x69, 0x63, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61,
0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x6f,
0x68, 0x6e, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x52, 0x69, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65,
0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20,
0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x65, 0x20,
0x49, 0x73, 0x6f, 0x6c, 0x65, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x61, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x53, 0x75, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54,
0x61, 0x67, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x43,
0x68, 0x75, 0x75, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c,
0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x6c, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x56,
0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x6c, 0x2019, 0x49, 0x73, 0x6f, 0x6c,
0x61, 0x20, 0x64, 0x69, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x69, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x6c, 0x6f, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x30a2, 0x30af, 0x30ec,
0x6642, 0x9593, 0x30a2, 0x30af, 0x30ec, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30af, 0x30ec, 0x590f,
0x6642, 0x9593, 0x30a2, 0x30d5, 0x30ac, 0x30cb, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x4e2d,
0x592e, 0x30a2, 0x30d5, 0x30ea, 0x30ab, 0x6642, 0x9593, 0x6771, 0x30a2, 0x30d5, 0x30ea, 0x30ab,
0x6642, 0x9593, 0x5357, 0x30a2, 0x30d5, 0x30ea, 0x30ab, 0x6a19, 0x6e96, 0x6642, 0x897f, 0x30a2,
0x30d5, 0x30ea, 0x30ab, 0x6642, 0x9593, 0x897f, 0x30a2, 0x30d5, 0x30ea, 0x30ab, 0x6a19, 0x6e96,
0x6642, 0x897f, 0x30a2, 0x30d5, 0x30ea, 0x30ab, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30e9, 0x30b9,
0x30ab, 0x6642, 0x9593, 0x30a2, 0x30e9, 0x30b9, 0x30ab, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30e9,
0x30b9, 0x30ab, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30eb, 0x30c8, 0x30de, 0x30a4, 0x6642, 0x9593,
0x30a2, 0x30eb, 0x30c8, 0x30de, 0x30a4, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30eb, 0x30de, 0x30c8,
0x30a4, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30de, 0x30be, 0x30f3, 0x6642, 0x9593, 0x30a2, 0x30de,
0x30be, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30de, 0x30be, 0x30f3, 0x590f, 0x6642, 0x9593,
0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x4e2d, 0x90e8, 0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab,
0x4e2d, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x4e2d, 0x90e8, 0x590f,
0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x6771, 0x90e8, 0x6642, 0x9593, 0x30a2, 0x30e1,
0x30ea, 0x30ab, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x6771,
0x90e8, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x5c71, 0x5730, 0x6642, 0x9593,
0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x5c71, 0x5730, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30e1, 0x30ea,
0x30ab, 0x5c71, 0x5730, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x592a, 0x5e73,
0x6d0b, 0x6642, 0x9593, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x592a, 0x5e73, 0x6d0b, 0x6a19, 0x6e96,
0x6642, 0x30a2, 0x30e1, 0x30ea, 0x30ab, 0x592a, 0x5e73, 0x6d0b, 0x590f, 0x6642, 0x9593, 0x30a2,
0x30ca, 0x30c7, 0x30a3, 0x30ea, 0x6642, 0x9593, 0x30a2, 0x30ca, 0x30c7, 0x30a3, 0x30ea, 0x6a19,
0x6e96, 0x6642, 0x30a2, 0x30ca, 0x30c7, 0x30a3, 0x30ea, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30d4,
0x30a2, 0x6642, 0x9593, 0x30a2, 0x30d4, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30d4, 0x30a2,
0x590f, 0x6642, 0x9593, 0x30a2, 0x30af, 0x30bf, 0x30a6, 0x6642, 0x9593, 0x30a2, 0x30af, 0x30bf,
0x30a6, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30af, 0x30bf, 0x30a6, 0x590f, 0x6642, 0x9593, 0x30a2,
0x30af, 0x30c8, 0x30d9, 0x6642, 0x9593, 0x30a2, 0x30af, 0x30c8, 0x30d9, 0x6a19, 0x6e96, 0x6642,
0x30a2, 0x30af, 0x30c8, 0x30d9, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30e9, 0x30d3, 0x30a2, 0x6642,
0x9593, 0x30a2, 0x30e9, 0x30d3, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30e9, 0x30d3, 0x30a2,
0x590f, 0x6642, 0x9593, 0x30a2, 0x30eb, 0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x6642, 0x9593, 0x30a2,
0x30eb, 0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30eb, 0x30bc, 0x30f3,
0x30c1, 0x30f3, 0x590f, 0x6642, 0x9593, 0x897f, 0x90e8, 0x30a2, 0x30eb, 0x30bc, 0x30f3, 0x30c1,
0x30f3, 0x6642, 0x9593, 0x897f, 0x90e8, 0x30a2, 0x30eb, 0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x6a19,
0x6e96, 0x6642, 0x897f, 0x90e8, 0x30a2, 0x30eb, 0x30bc, 0x30f3, 0x30c1, 0x30f3, 0x590f, 0x6642,
0x9593, 0x30a2, 0x30eb, 0x30e1, 0x30cb, 0x30a2, 0x6642, 0x9593, 0x30a2, 0x30eb, 0x30e1, 0x30cb,
0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30a2, 0x30eb, 0x30e1, 0x30cb, 0x30a2, 0x590f, 0x6642, 0x9593,
0x5927, 0x897f, 0x6d0b, 0x590f, 0x6642, 0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea,
0x30a2, 0x4e2d, 0x90e8, 0x6642, 0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2,
0x4e2d, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2,
0x4e2d, 0x90e8, 0x590f, 0x6642, 0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2,
0x4e2d, 0x897f, 0x90e8, 0x6642, 0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2,
0x4e2d, 0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea,
0x30a2, 0x4e2d, 0x897f, 0x90e8, 0x590f, 0x6642, 0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9,
0x30ea, 0x30a2, 0x6771, 0x90e8, 0x6642, 0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea,
0x30a2, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea,
0x30a2, 0x6771, 0x90e8, 0x590f, 0x6642, 0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea,
0x30a2, 0x897f, 0x90e8, 0x6642, 0x9593, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2,
0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x30aa, 0x30fc, 0x30b9, 0x30c8, 0x30e9, 0x30ea, 0x30a2,
0x897f, 0x90e8, 0x590f, 0x6642, 0x9593, 0x30a2, 0x30bc, 0x30eb, 0x30d0, 0x30a4, 0x30b8, 0x30e3,
0x30f3, 0x6642, 0x9593, 0x30a2, 0x30bc, 0x30eb, 0x30d0, 0x30a4, 0x30b8, 0x30e3, 0x30f3, 0x6a19,
0x6e96, 0x6642, 0x30a2, 0x30bc, 0x30eb, 0x30d0, 0x30a4, 0x30b8, 0x30e3, 0x30f3, 0x590f, 0x6642,
0x9593, 0x30a2, 0x30be, 0x30ec, 0x30b9, 0x6642, 0x9593, 0x30a2, 0x30be, 0x30ec, 0x30b9, 0x6a19,
0x6e96, 0x6642, 0x30a2, 0x30be, 0x30ec, 0x30b9, 0x590f, 0x6642, 0x9593, 0x30d0, 0x30f3, 0x30b0,
0x30e9, 0x30c7, 0x30b7, 0x30e5, 0x6642, 0x9593, 0x30d0, 0x30f3, 0x30b0, 0x30e9, 0x30c7, 0x30b7,
0x30e5, 0x6a19, 0x6e96, 0x6642, 0x30d0, 0x30f3, 0x30b0, 0x30e9, 0x30c7, 0x30b7, 0x30e5, 0x590f,
0x6642, 0x9593, 0x30d6, 0x30fc, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x30dc, 0x30ea, 0x30d3, 0x30a2,
0x6642, 0x9593, 0x30d6, 0x30e9, 0x30b8, 0x30ea, 0x30a2, 0x6642, 0x9593, 0x30d6, 0x30e9, 0x30b8,
0x30ea, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30d6, 0x30e9, 0x30b8, 0x30ea, 0x30a2, 0x590f, 0x6642,
0x9593, 0x30d6, 0x30eb, 0x30cd, 0x30a4, 0x30fb, 0x30c0, 0x30eb, 0x30b5, 0x30e9, 0x30fc, 0x30e0,
0x6642, 0x9593, 0x30ab, 0x30fc, 0x30dc, 0x30d9, 0x30eb, 0x30c7, 0x6642, 0x9593, 0x30ab, 0x30fc,
0x30dc, 0x30d9, 0x30eb, 0x30c7, 0x6a19, 0x6e96, 0x6642, 0x30ab, 0x30fc, 0x30dc, 0x30d9, 0x30eb,
0x30c7, 0x590f, 0x6642, 0x9593, 0x30b1, 0x30a4, 0x30b7, 0x30fc, 0x57fa, 0x5730, 0x6642, 0x9593,
0x30c1, 0x30e3, 0x30e2, 0x30ed, 0x6642, 0x9593, 0x30c1, 0x30e3, 0x30bf, 0x30e0, 0x6642, 0x9593,
0x30c1, 0x30e3, 0x30bf, 0x30e0, 0x6a19, 0x6e96, 0x6642, 0x30c1, 0x30e3, 0x30bf, 0x30e0, 0x590f,
0x6642, 0x9593, 0x30c1, 0x30ea, 0x6642, 0x9593, 0x30c1, 0x30ea, 0x6a19, 0x6e96, 0x6642, 0x30c1,
0x30ea, 0x590f, 0x6642, 0x9593, 0x4e2d, 0x56fd, 0x6642, 0x9593, 0x4e2d, 0x56fd, 0x6a19, 0x6e96,
0x6642, 0x4e2d, 0x56fd, 0x590f, 0x6642, 0x9593, 0x30c1, 0x30e7, 0x30a4, 0x30d0, 0x30eb, 0x30b5,
0x30f3, 0x6642, 0x9593, 0x30c1, 0x30e7, 0x30a4, 0x30d0, 0x30eb, 0x30b5, 0x30f3, 0x6a19, 0x6e96,
0x6642, 0x30c1, 0x30e7, 0x30a4, 0x30d0, 0x30eb, 0x30b5, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30af,
0x30ea, 0x30b9, 0x30de, 0x30b9, 0x5cf6, 0x6642, 0x9593, 0x30b3, 0x30b3, 0x30b9, 0x8af8, 0x5cf6,
0x6642, 0x9593, 0x30b3, 0x30ed, 0x30f3, 0x30d3, 0x30a2, 0x6642, 0x9593, 0x30b3, 0x30ed, 0x30f3,
0x30d3, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30b3, 0x30ed, 0x30f3, 0x30d3, 0x30a2, 0x590f, 0x6642,
0x9593, 0x30af, 0x30c3, 0x30af, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30af, 0x30c3, 0x30af, 0x8af8,
0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x30af, 0x30c3, 0x30af, 0x8af8, 0x5cf6, 0x590f, 0x6642, 0x9593,
0x30ad, 0x30e5, 0x30fc, 0x30d0, 0x6642, 0x9593, 0x30ad, 0x30e5, 0x30fc, 0x30d0, 0x6a19, 0x6e96,
0x6642, 0x30ad, 0x30e5, 0x30fc, 0x30d0, 0x590f, 0x6642, 0x9593, 0x30c7, 0x30fc, 0x30d3, 0x30b9,
0x57fa, 0x5730, 0x6642, 0x9593, 0x30c7, 0x30e5, 0x30e2, 0x30f3, 0x30fb, 0x30c7, 0x30e5, 0x30eb,
0x30f4, 0x30a3, 0x30eb, 0x57fa, 0x5730, 0x6642, 0x9593, 0x6771, 0x30c6, 0x30a3, 0x30e2, 0x30fc,
0x30eb, 0x6642, 0x9593, 0x30a4, 0x30fc, 0x30b9, 0x30bf, 0x30fc, 0x5cf6, 0x6642, 0x9593, 0x30a4,
0x30fc, 0x30b9, 0x30bf, 0x30fc, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x30a4, 0x30fc, 0x30b9, 0x30bf,
0x30fc, 0x5cf6, 0x590f, 0x6642, 0x9593, 0x30a8, 0x30af, 0x30a2, 0x30c9, 0x30eb, 0x6642, 0x9593,
0x4e2d, 0x592e, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6642, 0x9593, 0x4e2d, 0x592e, 0x30e8,
0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6a19, 0x6e96, 0x6642, 0x4e2d, 0x592e, 0x30e8, 0x30fc, 0x30ed,
0x30c3, 0x30d1, 0x590f, 0x6642, 0x9593, 0x6771, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6642,
0x9593, 0x6771, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6a19, 0x6e96, 0x6642, 0x6771, 0x30e8,
0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x590f, 0x6642, 0x9593, 0x6975, 0x6771, 0x30e8, 0x30fc, 0x30ed,
0x30c3, 0x30d1, 0x6642, 0x9593, 0x897f, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6642, 0x9593,
0x897f, 0x30e8, 0x30fc, 0x30ed, 0x30c3, 0x30d1, 0x6a19, 0x6e96, 0x6642, 0x897f, 0x30e8, 0x30fc,
0x30ed, 0x30c3, 0x30d1, 0x590f, 0x6642, 0x9593, 0x30d5, 0x30a9, 0x30fc, 0x30af, 0x30e9, 0x30f3,
0x30c9, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30d5, 0x30a9, 0x30fc, 0x30af, 0x30e9, 0x30f3, 0x30c9,
0x8af8, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x30d5, 0x30a9, 0x30fc, 0x30af, 0x30e9, 0x30f3, 0x30c9,
0x8af8, 0x5cf6, 0x590f, 0x6642, 0x9593, 0x30d5, 0x30a3, 0x30b8, 0x30fc, 0x6642, 0x9593, 0x30d5,
0x30a3, 0x30b8, 0x30fc, 0x6a19, 0x6e96, 0x6642, 0x30d5, 0x30a3, 0x30b8, 0x30fc, 0x590f, 0x6642,
0x9593, 0x4ecf, 0x9818, 0x30ae, 0x30a2, 0x30ca, 0x6642, 0x9593, 0x4ecf, 0x9818, 0x5357, 0x65b9,
0x5357, 0x6975, 0x6642, 0x9593, 0x30ac, 0x30e9, 0x30d1, 0x30b4, 0x30b9, 0x6642, 0x9593, 0x30ac,
0x30f3, 0x30d3, 0x30a8, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30b8, 0x30e7, 0x30fc, 0x30b8, 0x30a2,
0x6642, 0x9593, 0x30b8, 0x30e7, 0x30fc, 0x30b8, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30b8, 0x30e7,
0x30fc, 0x30b8, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30ae, 0x30eb, 0x30d0, 0x30fc, 0x30c8, 0x8af8,
0x5cf6, 0x6642, 0x9593, 0x30b0, 0x30ea, 0x30cb, 0x30c3, 0x30b8, 0x6a19, 0x6e96, 0x6642, 0x30b0,
0x30ea, 0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x6771, 0x90e8, 0x6642, 0x9593, 0x30b0, 0x30ea,
0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x6771, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x30b0, 0x30ea,
0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x6771, 0x90e8, 0x590f, 0x6642, 0x9593, 0x30b0, 0x30ea,
0x30fc, 0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x897f, 0x90e8, 0x6642, 0x9593, 0x30b0, 0x30ea, 0x30fc,
0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x897f, 0x90e8, 0x6a19, 0x6e96, 0x6642, 0x30b0, 0x30ea, 0x30fc,
0x30f3, 0x30e9, 0x30f3, 0x30c9, 0x897f, 0x90e8, 0x590f, 0x6642, 0x9593, 0x30b0, 0x30a2, 0x30e0,
0x6642, 0x9593, 0x6e7e, 0x5cb8, 0x6a19, 0x6e96, 0x6642, 0x30ac, 0x30a4, 0x30a2, 0x30ca, 0x6642,
0x9593, 0x30cf, 0x30ef, 0x30a4, 0x30fb, 0x30a2, 0x30ea, 0x30e5, 0x30fc, 0x30b7, 0x30e3, 0x30f3,
0x6642, 0x9593, 0x30cf, 0x30ef, 0x30a4, 0x30fb, 0x30a2, 0x30ea, 0x30e5, 0x30fc, 0x30b7, 0x30e3,
0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30cf, 0x30ef, 0x30a4, 0x30fb, 0x30a2, 0x30ea, 0x30e5, 0x30fc,
0x30b7, 0x30e3, 0x30f3, 0x590f, 0x6642, 0x9593, 0x9999, 0x6e2f, 0x590f, 0x6642, 0x9593, 0x30db,
0x30d6, 0x30c9, 0x6642, 0x9593, 0x30db, 0x30d6, 0x30c9, 0x6a19, 0x6e96, 0x6642, 0x30db, 0x30d6,
0x30c9, 0x590f, 0x6642, 0x9593, 0x30a4, 0x30f3, 0x30c9, 0x6a19, 0x6e96, 0x6642, 0x30a4, 0x30f3,
0x30c9, 0x6d0b, 0x6642, 0x9593, 0x30a4, 0x30f3, 0x30c9, 0x30b7, 0x30ca, 0x6642, 0x9593, 0x30a4,
0x30f3, 0x30c9, 0x30cd, 0x30b7, 0x30a2, 0x4e2d, 0x90e8, 0x6642, 0x9593, 0x30a4, 0x30f3, 0x30c9,
0x30cd, 0x30b7, 0x30a2, 0x6771, 0x90e8, 0x6642, 0x9593, 0x30a4, 0x30f3, 0x30c9, 0x30cd, 0x30b7,
0x30a2, 0x897f, 0x90e8, 0x6642, 0x9593, 0x30a4, 0x30e9, 0x30f3, 0x6642, 0x9593, 0x30a4, 0x30e9,
0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30a4, 0x30e9, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30a4, 0x30eb,
0x30af, 0x30fc, 0x30c4, 0x30af, 0x6642, 0x9593, 0x30a4, 0x30eb, 0x30af, 0x30fc, 0x30c4, 0x30af,
0x6a19, 0x6e96, 0x6642, 0x30a4, 0x30eb, 0x30af, 0x30fc, 0x30c4, 0x30af, 0x590f, 0x6642, 0x9593,
0x30a4, 0x30b9, 0x30e9, 0x30a8, 0x30eb, 0x6642, 0x9593, 0x30a4, 0x30b9, 0x30e9, 0x30a8, 0x30eb,
0x6a19, 0x6e96, 0x6642, 0x30a4, 0x30b9, 0x30e9, 0x30a8, 0x30eb, 0x590f, 0x6642, 0x9593, 0x65e5,
0x672c, 0x590f, 0x6642, 0x9593, 0x30da, 0x30c8, 0x30ed, 0x30d1, 0x30d6, 0x30ed, 0x30d5, 0x30b9,
0x30af, 0x30fb, 0x30ab, 0x30e0, 0x30c1, 0x30e3, 0x30c4, 0x30ad, 0x30fc, 0x6642, 0x9593, 0x30da,
0x30c8, 0x30ed, 0x30d1, 0x30d6, 0x30ed, 0x30d5, 0x30b9, 0x30af, 0x30fb, 0x30ab, 0x30e0, 0x30c1,
0x30e3, 0x30c4, 0x30ad, 0x30fc, 0x6a19, 0x6e96, 0x6642, 0x30da, 0x30c8, 0x30ed, 0x30d1, 0x30d6,
0x30ed, 0x30d5, 0x30b9, 0x30af, 0x30fb, 0x30ab, 0x30e0, 0x30c1, 0x30e3, 0x30c4, 0x30ad, 0x30fc,
0x590f, 0x6642, 0x9593, 0x6771, 0x30ab, 0x30b6, 0x30d5, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593,
0x897f, 0x30ab, 0x30b6, 0x30d5, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x97d3, 0x56fd, 0x6642,
0x9593, 0x97d3, 0x56fd, 0x6a19, 0x6e96, 0x6642, 0x97d3, 0x56fd, 0x590f, 0x6642, 0x9593, 0x30b3,
0x30b9, 0x30e9, 0x30a8, 0x6642, 0x9593, 0x30af, 0x30e9, 0x30b9, 0x30ce, 0x30e4, 0x30eb, 0x30b9,
0x30af, 0x6642, 0x9593, 0x30af, 0x30e9, 0x30b9, 0x30ce, 0x30e4, 0x30eb, 0x30b9, 0x30af, 0x6a19,
0x6e96, 0x6642, 0x30af, 0x30e9, 0x30b9, 0x30ce, 0x30e4, 0x30eb, 0x30b9, 0x30af, 0x590f, 0x6642,
0x9593, 0x30ad, 0x30eb, 0x30ae, 0x30b9, 0x6642, 0x9593, 0x30e9, 0x30f3, 0x30ab, 0x6642, 0x9593,
0x30e9, 0x30a4, 0x30f3, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30ed, 0x30fc, 0x30c9, 0x30cf, 0x30a6,
0x6642, 0x9593, 0x30ed, 0x30fc, 0x30c9, 0x30cf, 0x30a6, 0x6a19, 0x6e96, 0x6642, 0x30ed, 0x30fc,
0x30c9, 0x30cf, 0x30a6, 0x590f, 0x6642, 0x9593, 0x30de, 0x30ab, 0x30aa, 0x6642, 0x9593, 0x30de,
0x30ab, 0x30aa, 0x6a19, 0x6e96, 0x6642, 0x30de, 0x30ab, 0x30aa, 0x590f, 0x6642, 0x9593, 0x30de,
0x30ac, 0x30c0, 0x30f3, 0x6642, 0x9593, 0x30de, 0x30ac, 0x30c0, 0x30f3, 0x6a19, 0x6e96, 0x6642,
0x30de, 0x30ac, 0x30c0, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30de, 0x30ec, 0x30fc, 0x30b7, 0x30a2,
0x6642, 0x9593, 0x30e2, 0x30eb, 0x30c7, 0x30a3, 0x30d6, 0x6642, 0x9593, 0x30de, 0x30eb, 0x30ad,
0x30fc, 0x30ba, 0x6642, 0x9593, 0x30de, 0x30fc, 0x30b7, 0x30e3, 0x30eb, 0x8af8, 0x5cf6, 0x6642,
0x9593, 0x30e2, 0x30fc, 0x30ea, 0x30b7, 0x30e3, 0x30b9, 0x6642, 0x9593, 0x30e2, 0x30fc, 0x30ea,
0x30b7, 0x30e3, 0x30b9, 0x6a19, 0x6e96, 0x6642, 0x30e2, 0x30fc, 0x30ea, 0x30b7, 0x30e3, 0x30b9,
0x590f, 0x6642, 0x9593, 0x30e2, 0x30fc, 0x30bd, 0x30f3, 0x57fa, 0x5730, 0x6642, 0x9593, 0x30e1,
0x30ad, 0x30b7, 0x30b3, 0x592a, 0x5e73, 0x6d0b, 0x6642, 0x9593, 0x30e1, 0x30ad, 0x30b7, 0x30b3,
0x592a, 0x5e73, 0x6d0b, 0x6a19, 0x6e96, 0x6642, 0x30e1, 0x30ad, 0x30b7, 0x30b3, 0x592a, 0x5e73,
0x6d0b, 0x590f, 0x6642, 0x9593, 0x30a6, 0x30e9, 0x30f3, 0x30d0, 0x30fc, 0x30c8, 0x30eb, 0x6642,
0x9593, 0x30a6, 0x30e9, 0x30f3, 0x30d0, 0x30fc, 0x30c8, 0x30eb, 0x6a19, 0x6e96, 0x6642, 0x30a6,
0x30e9, 0x30f3, 0x30d0, 0x30fc, 0x30c8, 0x30eb, 0x590f, 0x6642, 0x9593, 0x30e2, 0x30b9, 0x30af,
0x30ef, 0x6642, 0x9593, 0x30e2, 0x30b9, 0x30af, 0x30ef, 0x6a19, 0x6e96, 0x6642, 0x30e2, 0x30b9,
0x30af, 0x30ef, 0x590f, 0x6642, 0x9593, 0x30df, 0x30e3, 0x30f3, 0x30de, 0x30fc, 0x6642, 0x9593,
0x30ca, 0x30a6, 0x30eb, 0x6642, 0x9593, 0x30cd, 0x30d1, 0x30fc, 0x30eb, 0x6642, 0x9593, 0x30cb,
0x30e5, 0x30fc, 0x30ab, 0x30ec, 0x30c9, 0x30cb, 0x30a2, 0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc,
0x30ab, 0x30ec, 0x30c9, 0x30cb, 0x30a2, 0x6a19, 0x6e96, 0x6642, 0x30cb, 0x30e5, 0x30fc, 0x30ab,
0x30ec, 0x30c9, 0x30cb, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc, 0x30b8, 0x30fc,
0x30e9, 0x30f3, 0x30c9, 0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc, 0x30b8, 0x30fc, 0x30e9, 0x30f3,
0x30c9, 0x6a19, 0x6e96, 0x6642, 0x30cb, 0x30e5, 0x30fc, 0x30b8, 0x30fc, 0x30e9, 0x30f3, 0x30c9,
0x590f, 0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc, 0x30d5, 0x30a1, 0x30f3, 0x30c9, 0x30e9, 0x30f3,
0x30c9, 0x6642, 0x9593, 0x30cb, 0x30e5, 0x30fc, 0x30d5, 0x30a1, 0x30f3, 0x30c9, 0x30e9, 0x30f3,
0x30c9, 0x6a19, 0x6e96, 0x6642, 0x30cb, 0x30e5, 0x30fc, 0x30d5, 0x30a1, 0x30f3, 0x30c9, 0x30e9,
0x30f3, 0x30c9, 0x590f, 0x6642, 0x9593, 0x30cb, 0x30a6, 0x30a8, 0x6642, 0x9593, 0x30ce, 0x30fc,
0x30d5, 0x30a9, 0x30fc, 0x30af, 0x5cf6, 0x6642, 0x9593, 0x30ce, 0x30fc, 0x30d5, 0x30a9, 0x30fc,
0x30af, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x30ce, 0x30fc, 0x30d5, 0x30a9, 0x30fc, 0x30af, 0x5cf6,
0x590f, 0x6642, 0x9593, 0x30d5, 0x30a7, 0x30eb, 0x30ca, 0x30f3, 0x30c9, 0x30fb, 0x30c7, 0x30fb,
0x30ce, 0x30ed, 0x30fc, 0x30cb, 0x30e3, 0x6642, 0x9593, 0x30d5, 0x30a7, 0x30eb, 0x30ca, 0x30f3,
0x30c9, 0x30fb, 0x30c7, 0x30fb, 0x30ce, 0x30ed, 0x30fc, 0x30cb, 0x30e3, 0x6a19, 0x6e96, 0x6642,
0x30d5, 0x30a7, 0x30eb, 0x30ca, 0x30f3, 0x30c9, 0x30fb, 0x30c7, 0x30fb, 0x30ce, 0x30ed, 0x30fc,
0x30cb, 0x30e3, 0x590f, 0x6642, 0x9593, 0x5317, 0x30de, 0x30ea, 0x30a2, 0x30ca, 0x8af8, 0x5cf6,
0x6642, 0x9593, 0x30ce, 0x30f4, 0x30a9, 0x30b7, 0x30d3, 0x30eb, 0x30b9, 0x30af, 0x6642, 0x9593,
0x30ce, 0x30f4, 0x30a9, 0x30b7, 0x30d3, 0x30eb, 0x30b9, 0x30af, 0x6a19, 0x6e96, 0x6642, 0x30ce,
0x30f4, 0x30a9, 0x30b7, 0x30d3, 0x30eb, 0x30b9, 0x30af, 0x590f, 0x6642, 0x9593, 0x30aa, 0x30e0,
0x30b9, 0x30af, 0x6642, 0x9593, 0x30aa, 0x30e0, 0x30b9, 0x30af, 0x6a19, 0x6e96, 0x6642, 0x30aa,
0x30e0, 0x30b9, 0x30af, 0x590f, 0x6642, 0x9593, 0x30d1, 0x30ad, 0x30b9, 0x30bf, 0x30f3, 0x6642,
0x9593, 0x30d1, 0x30ad, 0x30b9, 0x30bf, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30d1, 0x30ad, 0x30b9,
0x30bf, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30d1, 0x30e9, 0x30aa, 0x6642, 0x9593, 0x30d1, 0x30d7,
0x30a2, 0x30cb, 0x30e5, 0x30fc, 0x30ae, 0x30cb, 0x30a2, 0x6642, 0x9593, 0x30d1, 0x30e9, 0x30b0,
0x30a2, 0x30a4, 0x6642, 0x9593, 0x30d1, 0x30e9, 0x30b0, 0x30a2, 0x30a4, 0x6a19, 0x6e96, 0x6642,
0x30d1, 0x30e9, 0x30b0, 0x30a2, 0x30a4, 0x590f, 0x6642, 0x9593, 0x30da, 0x30eb, 0x30fc, 0x6642,
0x9593, 0x30da, 0x30eb, 0x30fc, 0x6a19, 0x6e96, 0x6642, 0x30da, 0x30eb, 0x30fc, 0x590f, 0x6642,
0x9593, 0x30d5, 0x30a3, 0x30ea, 0x30d4, 0x30f3, 0x6642, 0x9593, 0x30d5, 0x30a3, 0x30ea, 0x30d4,
0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30d5, 0x30a3, 0x30ea, 0x30d4, 0x30f3, 0x590f, 0x6642, 0x9593,
0x30d5, 0x30a7, 0x30cb, 0x30c3, 0x30af, 0x30b9, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30b5, 0x30f3,
0x30d4, 0x30a8, 0x30fc, 0x30eb, 0x5cf6, 0x30fb, 0x30df, 0x30af, 0x30ed, 0x30f3, 0x5cf6, 0x6642,
0x9593, 0x30b5, 0x30f3, 0x30d4, 0x30a8, 0x30fc, 0x30eb, 0x5cf6, 0x30fb, 0x30df, 0x30af, 0x30ed,
0x30f3, 0x5cf6, 0x6a19, 0x6e96, 0x6642, 0x30b5, 0x30f3, 0x30d4, 0x30a8, 0x30fc, 0x30eb, 0x5cf6,
0x30fb, 0x30df, 0x30af, 0x30ed, 0x30f3, 0x5cf6, 0x590f, 0x6642, 0x9593, 0x30d4, 0x30c8, 0x30b1,
0x30a2, 0x30f3, 0x6642, 0x9593, 0x30dd, 0x30ca, 0x30da, 0x6642, 0x9593, 0x5e73, 0x58cc, 0x6642,
0x9593, 0x30af, 0x30ba, 0x30ed, 0x30eb, 0x30c0, 0x6642, 0x9593, 0x30af, 0x30ba, 0x30ed, 0x30eb,
0x30c0, 0x6a19, 0x6e96, 0x6642, 0x30af, 0x30ba, 0x30ed, 0x30eb, 0x30c0, 0x590f, 0x6642, 0x9593,
0x30ec, 0x30e6, 0x30cb, 0x30aa, 0x30f3, 0x6642, 0x9593, 0x30ed, 0x30bc, 0x30e9, 0x57fa, 0x5730,
0x6642, 0x9593, 0x30b5, 0x30cf, 0x30ea, 0x30f3, 0x6642, 0x9593, 0x30b5, 0x30cf, 0x30ea, 0x30f3,
0x6a19, 0x6e96, 0x6642, 0x30b5, 0x30cf, 0x30ea, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30b5, 0x30de,
0x30e9, 0x6642, 0x9593, 0x30b5, 0x30de, 0x30e9, 0x6a19, 0x6e96, 0x6642, 0x30b5, 0x30de, 0x30e9,
0x590f, 0x6642, 0x9593, 0x30b5, 0x30e2, 0x30a2, 0x6642, 0x9593, 0x30b5, 0x30e2, 0x30a2, 0x6a19,
0x6e96, 0x6642, 0x30b5, 0x30e2, 0x30a2, 0x590f, 0x6642, 0x9593, 0x30bb, 0x30fc, 0x30b7, 0x30a7,
0x30eb, 0x6642, 0x9593, 0x30b7, 0x30f3, 0x30ac, 0x30dd, 0x30fc, 0x30eb, 0x6a19, 0x6e96, 0x6642,
0x30bd, 0x30ed, 0x30e2, 0x30f3, 0x8af8, 0x5cf6, 0x6642, 0x9593, 0x30b5, 0x30a6, 0x30b9, 0x30b8,
0x30e7, 0x30fc, 0x30b8, 0x30a2, 0x6642, 0x9593, 0x30b9, 0x30ea, 0x30ca, 0x30e0, 0x6642, 0x9593,
0x30bf, 0x30d2, 0x30c1, 0x6642, 0x9593, 0x53f0, 0x5317, 0x590f, 0x6642, 0x9593, 0x30bf, 0x30b8,
0x30ad, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x30c8, 0x30b1, 0x30e9, 0x30a6, 0x6642, 0x9593,
0x30c8, 0x30f3, 0x30ac, 0x6642, 0x9593, 0x30c8, 0x30f3, 0x30ac, 0x6a19, 0x6e96, 0x6642, 0x30c8,
0x30f3, 0x30ac, 0x590f, 0x6642, 0x9593, 0x30c1, 0x30e5, 0x30fc, 0x30af, 0x6642, 0x9593, 0x30c8,
0x30eb, 0x30af, 0x30e1, 0x30cb, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x30c8, 0x30eb, 0x30af,
0x30e1, 0x30cb, 0x30b9, 0x30bf, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30c8, 0x30eb, 0x30af, 0x30e1,
0x30cb, 0x30b9, 0x30bf, 0x30f3, 0x590f, 0x6642, 0x9593, 0x30c4, 0x30d0, 0x30eb, 0x6642, 0x9593,
0x30a6, 0x30eb, 0x30b0, 0x30a2, 0x30a4, 0x6642, 0x9593, 0x30a6, 0x30eb, 0x30b0, 0x30a2, 0x30a4,
0x6a19, 0x6e96, 0x6642, 0x30a6, 0x30eb, 0x30b0, 0x30a2, 0x30a4, 0x590f, 0x6642, 0x9593, 0x30a6,
0x30ba, 0x30d9, 0x30ad, 0x30b9, 0x30bf, 0x30f3, 0x6642, 0x9593, 0x30a6, 0x30ba, 0x30d9, 0x30ad,
0x30b9, 0x30bf, 0x30f3, 0x6a19, 0x6e96, 0x6642, 0x30a6, 0x30ba, 0x30d9, 0x30ad, 0x30b9, 0x30bf,
0x30f3, 0x590f, 0x6642, 0x9593, 0x30d0, 0x30cc, 0x30a2, 0x30c4, 0x6642, 0x9593, 0x30d0, 0x30cc,
0x30a2, 0x30c4, 0x6a19, 0x6e96, 0x6642, 0x30d0, 0x30cc, 0x30a2, 0x30c4, 0x590f, 0x6642, 0x9593,
0x30d9, 0x30cd, 0x30ba, 0x30a8, 0x30e9, 0x6642, 0x9593, 0x30a6, 0x30e9, 0x30b8, 0x30aa, 0x30b9,
0x30c8, 0x30af, 0x6642, 0x9593, 0x30a6, 0x30e9, 0x30b8, 0x30aa, 0x30b9, 0x30c8, 0x30af, 0x6a19,
0x6e96, 0x6642, 0x30a6, 0x30e9, 0x30b8, 0x30aa, 0x30b9, 0x30c8, 0x30af, 0x590f, 0x6642, 0x9593,
0x30dc, 0x30eb, 0x30b4, 0x30b0, 0x30e9, 0x30fc, 0x30c9, 0x6642, 0x9593, 0x30dc, 0x30eb, 0x30b4,
0x30b0, 0x30e9, 0x30fc, 0x30c9, 0x6a19, 0x6e96, 0x6642, 0x30dc, 0x30eb, 0x30b4, 0x30b0, 0x30e9,
0x30fc, 0x30c9, 0x590f, 0x6642, 0x9593, 0x30dc, 0x30b9, 0x30c8, 0x30fc, 0x30af, 0x57fa, 0x5730,
0x6642, 0x9593, 0x30a6, 0x30a7, 0x30fc, 0x30af, 0x5cf6, 0x6642, 0x9593, 0x30a6, 0x30a9, 0x30ea,
0x30b9, 0x30fb, 0x30d5, 0x30c4, 0x30ca, 0x6642, 0x9593, 0x30e4, 0x30af, 0x30fc, 0x30c4, 0x30af,
0x6642, 0x9593, 0x30e4, 0x30af, 0x30fc, 0x30c4, 0x30af, 0x6a19, 0x6e96, 0x6642, 0x30e4, 0x30af,
0x30fc, 0x30c4, 0x30af, 0x590f, 0x6642, 0x9593, 0x30a8, 0x30ab, 0x30c6, 0x30ea, 0x30f3, 0x30d6,
0x30eb, 0x30b0, 0x6642, 0x9593, 0x30a8, 0x30ab, 0x30c6, 0x30ea, 0x30f3, 0x30d6, 0x30eb, 0x30b0,
0x6a19, 0x6e96, 0x6642, 0x30a8, 0x30ab, 0x30c6, 0x30ea, 0x30f3, 0x30d6, 0x30eb, 0x30b0, 0x590f,
0x6642, 0x9593, 0x30e6, 0x30fc, 0x30b3, 0x30f3, 0x6642, 0x9593, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x57, 0x65, 0x74,
0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4b,
0x69, 0x64, 0x75, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6f,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x65, 0x6e,
0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65,
0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57,
0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57,
0x65, 0x74, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x69,
0x72, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x47, 0x69, 0x72, 0x69, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x47, 0x69, 0x72,
0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x41, 0x70, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x41, 0x70, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72,
0x61, 0x62, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x41, 0x72, 0x61, 0x62, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x72, 0x61,
0x62, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b,
0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75,
0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61,
0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20,
0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54,
0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20,
0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x20, 0x73,
0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73,
0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73,
0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a,
0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a,
0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x42,
0x72, 0x75, 0x6e, 0x61, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x61,
0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68,
0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x69,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x69, 0x6e, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x43, 0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x43, 0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x57, 0x45, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4e,
0x61, 0x74, 0x61, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f,
0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f,
0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x65, 0x70, 0x75,
0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x75, 0x62,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61,
0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x6b,
0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x6b,
0x61, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x6b, 0x75, 0x61,
0x64, 0x6f, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x45,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68,
0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x20,
0x73, 0x65, 0x6e, 0x67, 0x20, 0x4c, 0x75, 0x77, 0x69, 0x68, 0x20, 0x41,
0x64, 0x6f, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c,
0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x69,
0x73, 0x69, 0x68, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x45, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x75,
0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70,
0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x65,
0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x46, 0x69, 0x6a, 0x69,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61,
0x20, 0x50, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x73, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x6b, 0x61, 0x20,
0x6c, 0x61, 0x6e, 0x20, 0x50, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x73, 0x20,
0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x4b, 0x69, 0x64, 0x75, 0x6c, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x52, 0x65, 0x72, 0x61, 0x74, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47,
0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x57, 0x65, 0x74, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4b,
0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x47, 0x72, 0x69,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x54, 0x65, 0x6c, 0x75, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x65,
0x67, 0x6f, 0x72, 0x6f, 0x20, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x69, 0x6e,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20, 0x57, 0x65, 0x74,
0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x73, 0x69, 0x73, 0x69, 0x68, 0x20,
0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x49, 0x72,
0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x49, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4a, 0x65, 0x70, 0x61, 0x6e, 0x67, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x4a, 0x65, 0x70, 0x61, 0x6e, 0x67, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4a, 0x65, 0x70, 0x61,
0x6e, 0x67, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x4b, 0x75, 0x6c, 0x6f, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x69,
0x72, 0x67, 0x69, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x4c,
0x69, 0x6e, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4c, 0x6f, 0x72,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x64, 0x65, 0x77,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75,
0x65, 0x73, 0x61, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74,
0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0x6f, 0x77, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x41, 0x6e, 0x79,
0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x41, 0x6e, 0x79, 0x61, 0x72, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x53, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x41, 0x6e, 0x79,
0x61, 0x72, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x50, 0x75, 0x6c, 0x6f, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e,
0x75, 0x67, 0x69, 0x6e, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x72, 0x75,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x50,
0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x6c, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x6c, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x53, 0x61, 0x6e, 0x74, 0x61, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6c, 0x61, 0x6e, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x57, 0x65, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65,
0x74, 0x69, 0x67, 0x6f, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x65, 0x69, 0x63, 0x68, 0x65, 0x6c, 0x6c,
0x65, 0x73, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x75, 0x72, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x6c, 0x6f, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x20, 0x4b, 0x69, 0x64, 0x75, 0x6c, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x61, 0x6a, 0x69,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75,
0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69,
0x67, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x75, 0x76,
0x61, 0x6c, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67,
0x6f, 0x20, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x65,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b,
0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x56, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x6c,
0x6f, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x6c, 0x61, 0x6e, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x74,
0x69, 0x67, 0x6f, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x57, 0x65, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x57, 0x65, 0x6b,
0x74, 0x75, 0x20, 0x4b, 0x65, 0x74, 0x69, 0x67, 0x6f, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x57,
0x65, 0x6b, 0x74, 0x75, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x53, 0x75, 0x6c, 0x20, 0x64, 0x69, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0xc1, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64,
0x69, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0xc1, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20,
0x64, 0x69, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20,
0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72,
0x6f, 0x6e, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72,
0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x61, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61,
0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61,
0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61,
0x73, 0xed, 0x66, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61,
0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0xed,
0x66, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0xed,
0x66, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41,
0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x74,
0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41,
0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x6b, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61,
0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20,
0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe1, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe1, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x75, 0x2d,
0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61,
0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x65, 0x6e,
0x74, 0x72, 0x75, 0x2d, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x75, 0x2d, 0x4f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20,
0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20,
0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72,
0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x61, 0x62, 0x75, 0x20,
0x56, 0x65, 0x72, 0x64, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x61, 0x62, 0x75, 0x20,
0x56, 0x65, 0x72, 0x64, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x62,
0x75, 0x20, 0x56, 0x65, 0x72, 0x64, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61,
0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x4b, 0x75, 0x62, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e,
0x20, 0x64, 0x69, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x53, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x53, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20,
0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64,
0x69, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x73, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x4d, 0xe9, 0x64, 0x69, 0x75, 0x20,
0x64, 0x69, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65,
0x6c, 0xe1, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe1,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56, 0x65, 0x72, 0x6f,
0x6e, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe1,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e,
0x65, 0x6c, 0xe1, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64,
0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65,
0x6c, 0xe1, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x56,
0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x47, 0x72, 0x6f, 0x6e,
0x65, 0x6c, 0xe1, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20,
0x41, 0x76, 0x61, 0xed, 0x20, 0x69, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e,
0x20, 0x64, 0x69, 0x20, 0x41, 0x76, 0x61, 0xed, 0x20, 0x69, 0x20, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x41, 0x76,
0x61, 0xed, 0x20, 0x69, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66,
0x69, 0x6b, 0x75, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x75,
0x4f, 0x72, 0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64,
0x69, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x50,
0x61, 0x73, 0xed, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x4d, 0x65, 0x78, 0x69,
0x6b, 0x61, 0x6e, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x20, 0x54,
0x65, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x54, 0x65,
0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x54,
0x65, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x75,
0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x50, 0x61, 0x64, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20,
0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x75, 0x20, 0x69, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x20, 0x56, 0x65, 0x72, 0x6f, 0x6e, 0x20, 0x64, 0x69, 0x20, 0x53,
0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x75, 0x20, 0x69, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x41, 0x66, 0x263, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x66, 0x72, 0x69, 0x71,
0x74, 0x20, 0x74, 0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x66, 0x72, 0x69, 0x71,
0x74, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x54, 0x65, 0x66, 0x72, 0x69, 0x71, 0x74, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x1e93, 0x75, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x54, 0x65, 0x66, 0x72, 0x69, 0x71, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d,
0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x66, 0x72, 0x69, 0x71,
0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x54, 0x65, 0x66, 0x72, 0x69, 0x71, 0x74, 0x20, 0x6e, 0x20, 0x75,
0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x1e93, 0x75,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x41, 0x6d, 0x61, 0x1e93, 0x75, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x41, 0x6d, 0x61, 0x1e93, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x74, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20,
0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x74, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20, 0x4d,
0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x74, 0x6c,
0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d,
0x61, 0x72, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64,
0x75, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x20,
0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x79, 0x69, 0x64, 0x75, 0x72, 0x61, 0x72, 0x20,
0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x79, 0x69, 0x64, 0x75, 0x72, 0x61, 0x72, 0x20, 0x6e, 0x20, 0x4d, 0x61,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x79, 0x69, 0x64,
0x75, 0x72, 0x61, 0x72, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x6c, 0x77,
0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x61, 0x6d, 0x65, 0x6c, 0x77, 0x69, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x75, 0x6d, 0x65, 0x6c, 0x77, 0x69, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41,
0x70, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x70, 0x79, 0x61, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x41, 0x70, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x25b, 0x72, 0x61, 0x62, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x25b, 0x72,
0x61, 0x62, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x25b, 0x72, 0x61, 0x62, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x41, 0x72, 0x6a, 0x75, 0x6e, 0x74, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41,
0x72, 0x6a, 0x75, 0x6e, 0x74, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x41,
0x72, 0x6a, 0x75, 0x6e, 0x74, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x41, 0x72, 0x6a, 0x75, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6e,
0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41,
0x72, 0x6a, 0x75, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x75, 0x73,
0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x72, 0x6a,
0x75, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d,
0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x72,
0x6d, 0x69, 0x6e, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x72, 0x6d, 0x69,
0x6e, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x72, 0x6d, 0x69, 0x6e,
0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x1e6d, 0x6c, 0x61, 0x73,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x61, 0x1e6d, 0x6c, 0x61, 0x73, 0x61, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x1e6d, 0x6c, 0x61, 0x73, 0x61, 0x6e, 0x20, 0x6e,
0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x74,
0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x55,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x74, 0x61, 0x6c, 0x65,
0x6d, 0x6d, 0x61, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x74, 0x61, 0x6c, 0x65, 0x6d, 0x6d,
0x61, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x74, 0x6c,
0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61,
0x6c, 0x75, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x74, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74,
0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x20, 0x6e, 0x20, 0x55,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x74,
0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d,
0x61, 0x6c, 0x75, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d,
0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64,
0x75, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61,
0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x61, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x55, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x6e, 0x20, 0x75,
0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x55, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x65, 0x72, 0x62, 0x69, 0x1e7,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x65, 0x72, 0x62, 0x69, 0x1e7,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x65, 0x72, 0x62, 0x69,
0x1e7, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x41, 0x1e93,
0x75, 0x72, 0x69, 0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x75, 0x72, 0x69,
0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x41, 0x1e93, 0x75, 0x72, 0x69, 0x73, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x61, 0x63, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x61, 0x63, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x61, 0x63, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x42,
0x75, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x42,
0x75, 0x6c, 0x69, 0x76, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x42, 0x72, 0x61, 0x1e93, 0x69, 0x6c, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x42,
0x72, 0x61, 0x1e93, 0x69, 0x6c, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x42,
0x72, 0x61, 0x1e93, 0x69, 0x6c, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x61, 0x79, 0x20, 0x44, 0x61, 0x72,
0x20, 0x53, 0x61, 0x6c, 0x61, 0x6d, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x69, 0x72, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4b,
0x61, 0x70, 0x20, 0x56, 0x69, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4b, 0x61,
0x70, 0x20, 0x56, 0x69, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x43, 0x61, 0x6d, 0x75,
0x72, 0x72, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x43, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x43, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x43, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x43, 0x69, 0x6c, 0x69, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x43,
0x69, 0x6c, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x43, 0x69, 0x6c, 0x69, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x43, 0x69, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x43, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x43, 0x69, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b, 0x77, 0x61, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x4b, 0x77, 0x61, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4b, 0x77, 0x61, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65,
0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x4b, 0x72, 0x69,
0x73, 0x6d, 0x61, 0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54,
0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x4b, 0x75,
0x6b, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x6c,
0x75, 0x6d, 0x62, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x6c, 0x75,
0x6d, 0x62, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x6c, 0x75,
0x6d, 0x62, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69,
0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x6b, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20,
0x4b, 0x75, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b, 0x75,
0x62, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x4b, 0x75, 0x62, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54,
0x69, 0x6d, 0x75, 0x72, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d,
0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61,
0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e,
0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61,
0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x49,
0x6b, 0x77, 0x61, 0x1e6d, 0x75, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x74, 0x61, 0x6c, 0x65,
0x6d, 0x6d, 0x61, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75,
0x66, 0x74, 0x20, 0x74, 0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64,
0x75, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x74,
0x61, 0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x6e, 0x20,
0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x75,
0x72, 0x75, 0x66, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d,
0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74,
0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x6e, 0x69, 0x1e0d, 0x65, 0x6e, 0x20, 0x6e, 0x20,
0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x6e, 0x20, 0x55, 0x73, 0x61,
0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54,
0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c,
0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x75, 0x66, 0x74, 0x20, 0x6e,
0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x54, 0x75, 0x72, 0x75,
0x66, 0x74, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69,
0x6e, 0x20, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20,
0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x47, 0x77, 0x69, 0x79, 0x61, 0x6e, 0x20, 0x74,
0x61, 0x66, 0x72, 0x61, 0x6e, 0x73, 0x69, 0x73, 0x74, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x77, 0x61, 0x6b, 0x61, 0x6c, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x1e93, 0x75, 0x6c, 0x20, 0x64, 0x20, 0x41, 0x6e, 0x74, 0x61,
0x72, 0x6b, 0x74, 0x69, 0x6b, 0x20, 0x6e, 0x20, 0x46, 0x72, 0x61, 0x6e,
0x73, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x75, 0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x47, 0x61,
0x6d, 0x62, 0x79, 0x65, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x4a, 0x79, 0x75, 0x72, 0x6a, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4a, 0x79,
0x75, 0x72, 0x6a, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4a, 0x79, 0x75,
0x72, 0x6a, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54,
0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x4a, 0x69, 0x6c, 0x62,
0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6c, 0x65, 0x6d, 0x6d,
0x61, 0x73, 0x20, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69,
0x63, 0x68, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x47, 0x72, 0x69,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d,
0x6d, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64,
0x75, 0x20, 0x6e, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20,
0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6d, 0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x47, 0x72, 0x69, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x47, 0x75, 0x6c, 0x66, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x79, 0x2d, 0x41, 0x6c, 0x69, 0x77,
0x73, 0x79, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x79,
0x2d, 0x41, 0x6c, 0x69, 0x77, 0x73, 0x79, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x48, 0x61, 0x77, 0x61, 0x79, 0x2d, 0x41, 0x6c, 0x69, 0x77, 0x73,
0x79, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x48, 0x75,
0x6e, 0x67, 0x20, 0x4b, 0x75, 0x6e, 0x67, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x48, 0x75,
0x6e, 0x67, 0x20, 0x4b, 0x75, 0x6e, 0x67, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x48,
0x75, 0x6e, 0x67, 0x20, 0x4b, 0x75, 0x6e, 0x67, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x48, 0x75, 0x76, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x48, 0x75,
0x76, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x48, 0x75, 0x76, 0x64, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e,
0x20, 0x4c, 0x68, 0x65, 0x6e, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x55, 0x67, 0x61, 0x72, 0x61, 0x77, 0x20, 0x41, 0x68, 0x65, 0x6e,
0x64, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x49, 0x6e, 0x64,
0x75, 0x63, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x74,
0x6c, 0x65, 0x6d, 0x6d, 0x61, 0x73, 0x74, 0x20, 0x6e, 0x20, 0x41, 0x6e,
0x64, 0x75, 0x6e, 0x69, 0x73, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d, 0x61, 0x72, 0x20, 0x6e, 0x20,
0x41, 0x6e, 0x64, 0x75, 0x6e, 0x69, 0x73, 0x79, 0x61, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6d, 0x61, 0x6c, 0x75, 0x20, 0x6e, 0x20,
0x41, 0x6e, 0x64, 0x75, 0x6e, 0x69, 0x73, 0x79, 0x61, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x79, 0x69, 0x6c, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e,
0x20, 0x49, 0x7a, 0x72, 0x61, 0x79, 0x69, 0x6c, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x49, 0x7a, 0x72, 0x61, 0x79, 0x69, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x4a, 0x61, 0x70, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4a, 0x61,
0x70, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4a, 0x61, 0x70, 0x75, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x73, 0x61, 0x6d, 0x6d,
0x61, 0x72, 0x20, 0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x78, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x6e, 0x20,
0x75, 0x6d, 0x61, 0x6c, 0x75, 0x20, 0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x78, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x4b, 0x75, 0x72, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x72,
0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4b, 0x75, 0x72, 0x79, 0x61, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x67, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65,
0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x4c, 0x69, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e,
0x75, 0x20, 0x6e, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4d, 0x61, 0x6c, 0x69,
0x7a, 0x79, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54,
0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x6b,
0x69, 0x7a, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67,
0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x63, 0x61, 0x6c,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4d, 0x75, 0x72, 0x69, 0x73,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79,
0x20, 0x6e, 0x20, 0x4d, 0x75, 0x72, 0x69, 0x73, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x4d, 0x75, 0x72, 0x69, 0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x4d, 0x61, 0x77, 0x73, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x6c, 0x77, 0x69, 0x20, 0x6e, 0x20, 0x4d, 0x69, 0x6b, 0x73,
0x69, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64,
0x61, 0x79, 0x20, 0x61, 0x6d, 0x65, 0x6c, 0x77, 0x69, 0x20, 0x6e, 0x20,
0x4d, 0x69, 0x6b, 0x73, 0x69, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61,
0x6d, 0x65, 0x6c, 0x77, 0x69, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x4d, 0x69, 0x6b, 0x73, 0x69, 0x6b, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61,
0x74, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61,
0x74, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x62,
0x61, 0x74, 0x61, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4d,
0x75, 0x1e63, 0x6b, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x4d, 0x75, 0x1e63, 0x6b, 0x75,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64,
0x75, 0x20, 0x6e, 0x20, 0x4d, 0x75, 0x1e63, 0x6b, 0x75, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4e, 0x75, 0x72, 0x75, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x4e, 0x69, 0x70, 0x61, 0x6c, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x4b, 0x61, 0x6c, 0x69, 0x64, 0x75, 0x6e,
0x79, 0x61, 0x20, 0x54, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x4b, 0x61, 0x6c, 0x69, 0x64, 0x75, 0x6e, 0x79, 0x61, 0x20,
0x54, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x4b, 0x61, 0x6c, 0x69, 0x64, 0x75, 0x6e, 0x79, 0x61, 0x20, 0x54, 0x61,
0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x6d, 0x61,
0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65,
0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x5a, 0x69, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x54, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x5a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x6d, 0x61, 0x79,
0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x57, 0x61,
0x6b, 0x61, 0x6c, 0x20, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x57, 0x61, 0x6b, 0x61, 0x6c, 0x20, 0x61, 0x6d, 0x61, 0x79,
0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x57, 0x61, 0x6b, 0x61, 0x6c,
0x20, 0x61, 0x6d, 0x61, 0x79, 0x6e, 0x75, 0x74, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x74, 0x20, 0x6e, 0x20,
0x4e, 0x75, 0x72, 0x66, 0x75, 0x6c, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x65,
0x67, 0x7a, 0x69, 0x72, 0x74, 0x20, 0x6e, 0x20, 0x4e, 0x75, 0x72, 0x66,
0x75, 0x6c, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x54, 0x65, 0x67, 0x7a, 0x69,
0x72, 0x74, 0x20, 0x6e, 0x20, 0x4e, 0x75, 0x72, 0x66, 0x75, 0x6c, 0x6b,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4e, 0x75, 0x72, 0x75, 0x6e, 0x68,
0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61,
0x79, 0x20, 0x6e, 0x20, 0x46, 0x69, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75,
0x20, 0x6e, 0x20, 0x4e, 0x75, 0x72, 0x75, 0x6e, 0x68, 0x61, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x46, 0x69, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x4e, 0x75, 0x72, 0x75, 0x6e, 0x68, 0x61, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e,
0x75, 0x20, 0x6e, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x41, 0x6d,
0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x20, 0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x70, 0x77, 0x61, 0x20, 0x6e, 0x20,
0x194, 0x69, 0x6e, 0x79, 0x61, 0x20, 0x54, 0x61, 0x6d, 0x61, 0x79, 0x6e,
0x75, 0x74, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x77, 0x61, 0x79, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x77, 0x61, 0x79, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x77, 0x61, 0x79, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x50,
0x69, 0x72, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a,
0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x50, 0x69, 0x72, 0x75, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x50, 0x69, 0x72, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20, 0x46,
0x69, 0x6e, 0x69, 0x6b, 0x73, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x53, 0x61, 0x6e, 0x20, 0x50, 0x79, 0x69, 0x72, 0x20, 0x26, 0x20, 0x4d,
0x69, 0x6b, 0x6c, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x6e, 0x20,
0x50, 0x79, 0x69, 0x72, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x6b, 0x6c, 0x75,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62,
0x64, 0x75, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x79, 0x69,
0x72, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x6b, 0x6c, 0x75, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x50, 0x69, 0x74, 0x6b, 0x61, 0x72, 0x6e,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70,
0x65, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x50, 0x79, 0x75, 0x6e,
0x67, 0x79, 0x75, 0x6e, 0x67, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x52, 0x69, 0x79, 0x75, 0x6e, 0x79, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75,
0x20, 0x6e, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x20, 0x6e, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x77, 0x61, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20,
0x6e, 0x20, 0x53, 0x61, 0x6d, 0x77, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x53,
0x61, 0x6d, 0x77, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x53,
0x61, 0x79, 0x63, 0x61, 0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x53, 0x61, 0x6e, 0x67,
0x61, 0x70, 0x75, 0x72, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54,
0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6c, 0x75,
0x6d, 0x75, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x4a, 0x79,
0x75, 0x72, 0x6a, 0x79, 0x61, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x1e93, 0x75,
0x6c, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x53, 0x79,
0x75, 0x77, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54,
0x61, 0x79, 0x70, 0x61, 0x79, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x61, 0x79, 0x70,
0x61, 0x79, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65,
0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x54, 0x61, 0x79, 0x70, 0x61, 0x79,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x1e6c, 0x61, 0x6a, 0x69, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x1e6c, 0x75, 0x6b, 0x69, 0x6c, 0x61, 0x77, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x6e, 0x20, 0x1e6c, 0x75, 0x6e, 0x67, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x1e6c, 0x75,
0x6e, 0x67, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x1e6c, 0x75, 0x6e, 0x67, 0x61,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b,
0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x54, 0x75, 0x76,
0x61, 0x6c, 0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x72,
0x75, 0x67, 0x77, 0x61, 0x79, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x61, 0x6d,
0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x55, 0x72, 0x75, 0x67,
0x77, 0x61, 0x79, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x75, 0x6e,
0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77,
0x61, 0x79, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x7a, 0x62,
0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64, 0x20,
0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x55, 0x7a,
0x62, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x6e, 0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20,
0x55, 0x7a, 0x62, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x79, 0x61, 0x74,
0x75, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75,
0x20, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x79, 0x61, 0x74, 0x75, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75,
0x20, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x79, 0x61, 0x74, 0x75, 0x41,
0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x56, 0x69, 0x6e, 0x69, 0x7a, 0x77,
0x69, 0x6c, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6b, 0x75,
0x64, 0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6b,
0x75, 0x64, 0x20, 0x6e, 0x20, 0x55, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20,
0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x41, 0x6d,
0x61, 0x67, 0x6e, 0x75, 0x20, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x54, 0x65, 0x67, 0x7a, 0x69, 0x72, 0x69, 0x6e, 0x20, 0x6e, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x6b, 0x6b, 0x65, 0x64, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64,
0x20, 0x61, 0x6d, 0x65, 0x7a, 0x64, 0x61, 0x79, 0x20, 0x6e, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e,
0x20, 0x75, 0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x41, 0x6d, 0x61, 0x67, 0x6e, 0x75,
0x20, 0x6e, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x41, 0x6b, 0x75, 0x64, 0x20, 0x6e, 0x20, 0x55,
0x6e, 0x65, 0x62, 0x64, 0x75, 0x20, 0x6e, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xc8f, 0xc95, 0xcb0,
0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc8f, 0xc95, 0xcb0, 0xccd, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc8f, 0xc95,
0xcb0, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc85, 0xcab, 0xc98, 0xcbe, 0xca8, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xca7, 0xccd, 0xcaf, 0x20, 0xc86, 0xcab, 0xccd,
0xcb0, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd,
0xcb5, 0x20, 0xc86, 0xcab, 0xccd, 0xcb0, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcb8, 0xcae,
0xcaf, 0xca6, 0xc95, 0xccd, 0xcb7, 0xcbf, 0xca3, 0x20, 0xc86, 0xcab, 0xccd, 0xcb0,
0xcbf, 0xc95, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc86,
0xcab, 0xccd, 0xcb0, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6,
0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc86, 0xcab, 0xccd, 0xcb0, 0xcbf, 0xc95, 0xcbe,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc86, 0xcab, 0xccd, 0xcb0,
0xcbf, 0xc95, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc85, 0xcb2, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xcbe, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc85, 0xcb2, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0x200c, 0xc85, 0xcb2,
0xcbe, 0xcb8, 0xccd, 0xc95, 0xcbe, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc86, 0xcb2, 0xccd, 0xcae, 0xcc7, 0xc9f, 0xcbf, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc86, 0xcb2, 0xccd, 0xcae, 0xcc7, 0xc9f, 0xcbf, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb2, 0xccd,
0xcae, 0xcc7, 0xc9f, 0xcbf, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc85, 0xcae, 0xcc6, 0xc9c, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc85, 0xcae, 0xcc6, 0xc9c, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcae,
0xcc6, 0xc9c, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae,
0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae,
0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf,
0xc95, 0xca6, 0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0xcc0, 0xcaf, 0x20,
0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4,
0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcc2,
0xcb0, 0xccd, 0xcb5, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4,
0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcc2,
0xcb0, 0xccd, 0xcb5, 0xca6, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85,
0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5,
0xca6, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4,
0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20,
0xcaa, 0xcb0, 0xccd, 0xcb5, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd,
0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa,
0xcb0, 0xccd, 0xcb5, 0xca4, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85,
0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa, 0xcb0, 0xccd, 0xcb5, 0xca4,
0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd,
0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20, 0xcaa,
0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89,
0xca4, 0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6,
0x20, 0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4,
0xccd, 0xca4, 0xcb0, 0x20, 0xc85, 0xcae, 0xcc6, 0xcb0, 0xcbf, 0xc95, 0xca6, 0x20,
0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf, 0xc95, 0xccd, 0x20, 0xca6, 0xcbf, 0xca8,
0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xca8, 0xca1, 0xcc0, 0xcb0, 0xccd, 0x200c,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xca8, 0xca1, 0xcc0, 0xcb0, 0xccd, 0x200c, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc85, 0xca8, 0xca1, 0xcc0, 0xcb0, 0xccd, 0x200c, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcaa, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc85, 0xcaa, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcaa, 0xcbf, 0xcaf, 0xcbe,
0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc95, 0xccd,
0xc9f, 0xccc, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc95, 0xccd, 0xc9f, 0xccc, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc85, 0xc95, 0xccd, 0xc9f, 0xccc, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc95, 0xccd, 0xc9f, 0xccb, 0xcac, 0xcc6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc85, 0xc95, 0xccd, 0xc9f, 0xccb, 0xcac, 0xcc6, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85,
0xc95, 0xccd, 0xc9f, 0xccb, 0xcac, 0xcc6, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xcc7, 0xcac, 0xcbf, 0xcaf, 0xca8,
0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xcc7, 0xcac, 0xcbf, 0xcaf, 0xca8,
0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc85, 0xcb0, 0xcc7, 0xcac, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb9,
0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd, 0xc9c, 0xcc6,
0xc82, 0xc9f, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd,
0xc9c, 0xcc6, 0xc82, 0xc9f, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd, 0xc9c,
0xcc6, 0xc82, 0xc9f, 0xcbf, 0xca8, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20,
0xc85, 0xcb0, 0xccd, 0xc9c, 0xcc6, 0xc82, 0xc9f, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc85, 0xcb0, 0xccd,
0xc9c, 0xcc6, 0xc82, 0xc9f, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a,
0xcbf, 0xcae, 0x20, 0xc85, 0xcb0, 0xccd, 0xc9c, 0xcc6, 0xc82, 0xc9f, 0xcc0, 0xca8,
0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc85, 0xcb0, 0xccd, 0xcae, 0xcc7, 0xca8, 0xcbf, 0xcaf, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc85, 0xcb0, 0xccd, 0xcae, 0xcc7, 0xca8, 0xcbf, 0xcaf, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xcb0, 0xccd,
0xcae, 0xcc7, 0xca8, 0xcbf, 0xcaf, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9f, 0xccd, 0xcb2, 0xcbe, 0xc82, 0xc9f, 0xcbf,
0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9f, 0xccd, 0xcb2, 0xcbe, 0xc82,
0xc9f, 0xcbf, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9f, 0xccd, 0xcb2, 0xcbe, 0xc82, 0xc9f,
0xcbf, 0xc95, 0xccd, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd,
0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8,
0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xc95,
0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0,
0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd,
0xcb0, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8,
0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xc95,
0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2,
0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xc95, 0xcc7, 0xc82, 0xca6, 0xccd, 0xcb0, 0x20,
0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd,
0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xc95, 0xcc7, 0xc82, 0xca6,
0xccd, 0xcb0, 0x20, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xcb9, 0xc97,
0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20,
0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf,
0xcaf, 0xcbe, 0xca6, 0x20, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2,
0xcb0, 0xccd, 0xcb5, 0x20, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2,
0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc86, 0xcb8, 0xccd, 0xc9f,
0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86,
0xcb8, 0xccd, 0xc9f, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20,
0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc86, 0xcb8, 0xccd, 0xc9f, 0xccd,
0xcb0, 0xcc7, 0xcb2, 0xcbf, 0xcaf, 0xcbe, 0xca6, 0x20, 0xcaa, 0xcb6, 0xccd, 0xc9a,
0xcbf, 0xcae, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85,
0xc9c, 0xcb0, 0xccd, 0xcac, 0xcc8, 0xc9c, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc85, 0xc9c, 0xcb0, 0xccd, 0xcac, 0xcc8, 0xc9c, 0xcbe, 0xca8, 0xccd, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc85, 0xc9c, 0xcb0, 0xccd, 0xcac, 0xcc8, 0xc9c, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9c, 0xccb,
0xcb0, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc85, 0xc9c, 0xccb, 0xcb0, 0xcb8,
0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc85, 0xc9c, 0xccb, 0xcb0, 0xcb8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xcbe, 0xc82, 0xc97, 0xccd,
0xcb2, 0xcbe, 0xca6, 0xcc7, 0xcb6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xcbe, 0xc82,
0xc97, 0xccd, 0xcb2, 0xcbe, 0xca6, 0xcc7, 0xcb6, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xcbe, 0xc82, 0xc97,
0xccd, 0xcb2, 0xcbe, 0xca6, 0xcc7, 0xcb6, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcad, 0xcc2, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcac, 0xcca, 0xcb2, 0xcbf, 0xcb5, 0xcbf, 0xcaf, 0xcbe, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcac, 0xccd, 0xcb0, 0xcc6, 0xcb8, 0xcbf, 0xcb2, 0xcbf, 0xcaf,
0xcbe, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xccd, 0xcb0, 0xcc6, 0xcb8, 0xcbf,
0xcb2, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcac, 0xccd, 0xcb0, 0xcc6, 0xcb8, 0xcbf, 0xcb2,
0xcbf, 0xcaf, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcac, 0xccd, 0xcb0, 0xcc2, 0xca8, 0xcbf, 0x20, 0xca6, 0xcbe, 0xcb0,
0xcc1, 0xcb8, 0xcb2, 0xcc6, 0xcae, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc7,
0xcaa, 0xccd, 0x20, 0xcb5, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc95, 0xcc7, 0xcaa, 0xccd, 0x20, 0xcb5, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95,
0xcc7, 0xcaa, 0xccd, 0x20, 0xcb5, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcae, 0xccb, 0xcb0,
0xccb, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc9a, 0xca5, 0xcbe, 0xcae, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a,
0xca5, 0xcbe, 0xcae, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xca5, 0xcbe, 0xcae, 0xccd, 0x20, 0xcb9,
0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcbf, 0xcb2, 0xcbf, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcbf, 0xcb2, 0xcbf, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcbf, 0xcb2, 0xcbf,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a,
0xcc0, 0xca8, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcc0, 0xca8, 0xcbe, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc9a, 0xcc0, 0xca8, 0xcbe, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc9a, 0xcca, 0xcaf, 0xccd, 0x200c, 0xcac, 0xcb2, 0xccd, 0xcb8, 0xcbe, 0xca8,
0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcca, 0xcaf, 0xccd, 0x200c, 0x200c, 0xcac,
0xcb2, 0xccd, 0xcb8, 0xcbe, 0xca8, 0xccd, 0x200c, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a, 0xcca, 0xcaf, 0xccd,
0x200c, 0xcac, 0xcb2, 0xccd, 0xcb8, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcb0, 0xcbf, 0xcb8,
0xccd, 0xcae, 0xcb8, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc95, 0xccb, 0xc95, 0xccb, 0xcb8, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5,
0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb2, 0xc82,
0xcac, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb2, 0xc82,
0xcac, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb2, 0xc82, 0xcac, 0xcbf, 0xcaf,
0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc95, 0xcc1, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc1, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5,
0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcc1, 0xc95, 0xccd, 0x20, 0xca6, 0xccd,
0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcae, 0xca7, 0xccd, 0xcaf, 0xc95, 0xcbe,
0xcb2, 0xcc0, 0xca8, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc95, 0xccd, 0xcaf, 0xcc2, 0xcac, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc95, 0xccd, 0xcaf, 0xcc2, 0xcac, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcaf, 0xcc2, 0xcac,
0xcbe, 0x20, 0xca6, 0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca1, 0xcc7,
0xcb5, 0xcbf, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca1, 0xcc1, 0xcae, 0xc82,
0xc9f, 0xccd, 0x2d, 0xca1, 0xcbf, 0x20, 0xc89, 0xcb0, 0xccd, 0xcb5, 0xcbf, 0xcb2,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc9f,
0xcbf, 0xcae, 0xccb, 0xcb0, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc88, 0xcb8, 0xccd,
0xc9f, 0xcb0, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc88, 0xcb8, 0xccd, 0xc9f, 0xcb0, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0,
0xcaa, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc88, 0xcb8, 0xccd, 0xc9f, 0xcb0, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5,
0xcc0, 0xcaa, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc88, 0xc95, 0xccd, 0xcb5, 0xcc6, 0xca1, 0xcbe, 0xcb0, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcae, 0xca7, 0xccd, 0xcaf, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa,
0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xca7, 0xccd, 0xcaf,
0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae,
0xca7, 0xccd, 0xcaf, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8,
0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf,
0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5,
0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa,
0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf,
0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcae, 0xca4, 0xccd, 0xca4, 0xcb7, 0xccd, 0xc9f, 0xcc1, 0x2d, 0xcaa, 0xcc2,
0xcb0, 0xccd, 0xcb5, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8,
0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20,
0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb,
0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae,
0x20, 0xcaf, 0xcc1, 0xcb0, 0xccb, 0xcaa, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbe, 0xc95,
0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xca6, 0xccd,
0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbe, 0xc95,
0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xca6, 0xccd,
0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbe, 0xc95, 0xccd, 0x200c, 0xcb2,
0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa,
0xc97, 0xcb3, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcab, 0xcbf, 0xc9c, 0xcbf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbf, 0xc9c,
0xcbf, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcab, 0xcbf, 0xc9c, 0xcbf, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xccd, 0xcb0, 0xcc6, 0xc82, 0xc9a, 0xccd,
0x20, 0xc97, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca6, 0xc95,
0xccd, 0xcb7, 0xcbf, 0xca3, 0x20, 0xcab, 0xccd, 0xcb0, 0xcc6, 0xc82, 0xc9a, 0xccd,
0x20, 0xcae, 0xca4, 0xccd, 0xca4, 0xcc1, 0x20, 0xc85, 0xc82, 0xc9f, 0xcbe, 0xcb0,
0xccd, 0xc9f, 0xcbf, 0xc95, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc97, 0xcbe, 0xcb2,
0xcbe, 0xcaa, 0xcbe, 0xc97, 0xccb, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc97,
0xccd, 0xcaf, 0xcbe, 0xc82, 0xcac, 0xcbf, 0xcaf, 0xcb0, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc9c, 0xcbe, 0xcb0, 0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc9c, 0xcbe, 0xcb0, 0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9c, 0xcbe,
0xcb0, 0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc97, 0xcbf, 0xcb2, 0xccd, 0xcac, 0xcb0, 0xccd,
0xc9f, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb5, 0xcbf, 0xc9a,
0xccd, 0x20, 0xcb8, 0xcb0, 0xcbe, 0xcb8, 0xcb0, 0xcbf, 0x20, 0xc95, 0xcbe, 0xcb2,
0xcae, 0xcbe, 0xca8, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc97, 0xccd, 0xcb0,
0xcc0, 0xca8, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc97, 0xccd, 0xcb0,
0xcc0, 0xca8, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcaa, 0xcc2, 0xcb0, 0xccd, 0xcb5, 0x20, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd,
0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf,
0xcae, 0x20, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf,
0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a,
0xcbf, 0xcae, 0x20, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb2, 0xccd,
0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3,
0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae,
0x20, 0xc97, 0xccd, 0xcb0, 0xcc0, 0xca8, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe,
0xc82, 0xca1, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc97, 0xcc1, 0xcb5, 0xcbe, 0xcae, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc97, 0xcb2, 0xccd,
0xcab, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5, 0xcbe, 0xcaf, 0xcbf, 0x2d, 0xc85, 0xcb2, 0xccd,
0xcaf, 0xcc1, 0xc9f, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9,
0xcb5, 0xcbe, 0xcaf, 0xcbf, 0x2d, 0xc85, 0xcb2, 0xccd, 0xcaf, 0xcc1, 0xc9f, 0xcbf,
0xcaf, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5, 0xcbe, 0xcaf, 0xcbf, 0x2d, 0xc85, 0xcb2,
0xccd, 0xcaf, 0xcc1, 0xc9f, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2,
0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcbe, 0xc82, 0xc97, 0xccd, 0x20, 0xc95,
0xcbe, 0xc82, 0xc97, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcbe, 0xc82, 0xc97,
0xccd, 0x20, 0xc95, 0xcbe, 0xc82, 0xc97, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcbe, 0xc82, 0xc97,
0xccd, 0x20, 0xc95, 0xcbe, 0xc82, 0xc97, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5, 0xccd, 0xca1, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5, 0xccd, 0xca1, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcb5, 0xccd,
0xca1, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcad, 0xcbe, 0xcb0, 0xca4, 0xcc0, 0xcaf, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcbf, 0xc82, 0xca6,
0xcc2, 0x20, 0xcae, 0xcb9, 0xcbe, 0xcb8, 0xcbe, 0xc97, 0xcb0, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc87, 0xc82, 0xca1, 0xcca, 0xc9a, 0xcc8, 0xca8, 0xcbe, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcae, 0xca7, 0xccd, 0xcaf, 0x20, 0xc87, 0xc82, 0xca1, 0xccb, 0xca8, 0xcc7,
0xcb7, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd,
0xcb5, 0x20, 0xc87, 0xc82, 0xca1, 0xccb, 0xca8, 0xcc7, 0xcb7, 0xcbf, 0xcaf, 0xcbe,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc87,
0xc82, 0xca1, 0xccb, 0xca8, 0xcc7, 0xcb7, 0xcbf, 0xcaf, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc87, 0xcb0, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0, 0xcbe,
0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2,
0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0, 0xccd, 0x200c, 0xc95, 0xcc1, 0xc9f,
0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0, 0xccd, 0x200c,
0xc95, 0xcc1, 0xc9f, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb0, 0xccd, 0x200c,
0xc95, 0xcc1, 0xc9f, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb8, 0xccd, 0xcb0, 0xcc7, 0xcb2,
0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc87, 0xcb8, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xccd,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc87, 0xcb8, 0xccd, 0xcb0, 0xcc7, 0xcb2, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2,
0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9c, 0xcaa, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xc9c, 0xcaa, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9c, 0xcaa, 0xcbe, 0xca8,
0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc6,
0xca4, 0xccd, 0xcb0, 0xccb, 0xcaa, 0xcbe, 0xcb5, 0xccd, 0xcb2, 0xccb, 0xcb8, 0xccd,
0xc95, 0xccd, 0x200c, 0x200c, 0x2d, 0xc95, 0xcae, 0xccd, 0xc9a, 0xca4, 0xccd, 0xcb8,
0xccd, 0x200c, 0xc95, 0xcc0, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc6, 0xca4, 0xccd,
0xcb0, 0xccb, 0xcaa, 0xcbe, 0xcb5, 0xccd, 0xcb2, 0xccb, 0xcb8, 0xccd, 0xc95, 0xccd,
0x200c, 0x200c, 0x2d, 0xc95, 0xcae, 0xccd, 0xc9a, 0xca4, 0xccd, 0xcb8, 0xccd, 0x200c,
0xc95, 0xcc0, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc6, 0xca4, 0xccd, 0xcb0, 0xccb, 0xcaa, 0xcbe, 0xcb5,
0xccd, 0xcb2, 0xccb, 0xcb8, 0xccd, 0xc95, 0xccd, 0x200c, 0x200c, 0x2d, 0xc95, 0xcae,
0xccd, 0xc9a, 0xca4, 0xccd, 0xcb8, 0xccd, 0x200c, 0xc95, 0xcc0, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc2, 0xcb0, 0xccd,
0xcb5, 0x20, 0xc95, 0xc9c, 0xc95, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb6, 0xccd, 0xc9a, 0xcbf, 0xcae, 0x20, 0xc95,
0xc9c, 0xc95, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc95, 0xcca, 0xcb0, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc95, 0xcca, 0xcb0, 0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcca, 0xcb0, 0xcbf,
0xcaf, 0xca8, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc95, 0xcca, 0xcb8, 0xcb0, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcb0,
0xcbe, 0xcb8, 0xccd, 0x200c, 0xca8, 0xcca, 0xcaf, 0xcbe, 0xcb0, 0xccd, 0xcb8, 0xccd,
0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccd, 0xcb0, 0xcbe, 0xcb8, 0xccd,
0x200c, 0xca8, 0xcca, 0xcaf, 0xcbe, 0xcb0, 0xccd, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc95, 0xccd, 0xcb0, 0xcbe, 0xcb8, 0xccd, 0x200c, 0xca8, 0xcca, 0xcaf, 0xcbe, 0xcb0,
0xccd, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xcbf, 0xcb0, 0xccd, 0xc97, 0xcbf, 0xcb8, 0xccd,
0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb2, 0xc82, 0xc95, 0xcbe,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb2, 0xcc8, 0xca8, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5,
0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb2, 0xcbe, 0xcb0, 0xccd,
0xca1, 0xccd, 0x20, 0xcb9, 0xccb, 0xcb5, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb2,
0xcbe, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcb9, 0xccb, 0xcb5, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb2,
0xcbe, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcb9, 0xccb, 0xcb5, 0xccd, 0x20, 0xcac,
0xcc6, 0xcb3, 0xc97, 0xcbf, 0xca8, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xc95, 0xcbe,
0xcb5, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xc95, 0xcbe, 0xcb5, 0xccd, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcae, 0xc95, 0xcbe, 0xcb5, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xc97, 0xca1, 0xcbe, 0xca8, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcae, 0xc97, 0xca1, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xc97, 0xca1,
0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcae, 0xcb2, 0xcc7, 0xcb7, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcae, 0xcbe, 0xcb2, 0xccd, 0xca1, 0xcc0, 0xcb5, 0xccd, 0xcb8, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb0, 0xccd, 0xc95, 0xccd, 0xcaf, 0xcc1, 0xcb8,
0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb0, 0xccd, 0xcb7, 0xcb2,
0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcae, 0xcbe, 0xcb0, 0xcbf, 0xcb7, 0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcae, 0xcbe, 0xcb0, 0xcbf, 0xcb7, 0xcb8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb0, 0xcbf,
0xcb7, 0xcb8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb5, 0xccd, 0x200c, 0xcb8, 0xca8, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcae, 0xcc6, 0xc95, 0xccd, 0xcb8, 0xcbf, 0xc95, 0xca8, 0xccd, 0x20,
0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcae, 0xcc6, 0xc95, 0xccd, 0xcb8, 0xcbf, 0xc95, 0xca8, 0xccd, 0x20, 0xcaa, 0xcc6,
0xcb8, 0xcbf, 0xcab, 0xcbf, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcc6, 0xc95, 0xccd, 0xcb8,
0xcbf, 0xc95, 0xca8, 0xccd, 0x20, 0xcaa, 0xcc6, 0xcb8, 0xcbf, 0xcab, 0xcbf, 0xc95,
0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xcb2,
0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xca4, 0xcca, 0xcb0, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc89, 0xcb2, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xca4, 0xcca, 0xcb0, 0xccd,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc89, 0xcb2, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xca4, 0xcca, 0xcb0, 0xccd,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae,
0xcbe, 0xcb8, 0xccd, 0xc95, 0xccb, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb8,
0xccd, 0xc95, 0xccb, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xccb, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcae, 0xccd, 0xcaf,
0xcbe, 0xca8, 0xccd, 0xcae, 0xcbe, 0xcb0, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8,
0xccc, 0xcb0, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcc7, 0xcaa, 0xcbe, 0xcb3,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcca, 0xcb8, 0x20, 0xc95, 0xccd, 0xcaf, 0xcbe,
0xcb2, 0xcc6, 0xca1, 0xccb, 0xca8, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcb9, 0xcca, 0xcb8, 0x20, 0xc95, 0xccd, 0xcaf, 0xcbe, 0xcb2, 0xcc6, 0xca1, 0xccb,
0xca8, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb9, 0xcca, 0xcb8, 0x20, 0xc95, 0xccd, 0xcaf,
0xcbe, 0xcb2, 0xcc6, 0xca1, 0xccb, 0xca8, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0xcaf, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xccd, 0xcaf,
0xcc2, 0xc9c, 0xcbf, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcb8,
0xcae, 0xcaf, 0xca8, 0xccd, 0xcaf, 0xcc2, 0xc9c, 0xcbf, 0xcb2, 0xccd, 0xcaf, 0xcbe,
0xc82, 0xca1, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xccd, 0xcaf, 0xcc2, 0xc9c, 0xcbf, 0xcb2, 0xccd,
0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8,
0xcae, 0xcaf, 0xca8, 0xccd, 0xcaf, 0xcc2, 0xcab, 0xccc, 0xc82, 0xca1, 0xccd, 0x200c,
0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8,
0xccd, 0xcaf, 0xcc2, 0xcab, 0xccc, 0xc82, 0xca1, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf,
0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf,
0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xccd, 0xcaf, 0xcc2, 0xcab, 0xccc, 0xc82,
0xca1, 0xccd, 0x200c, 0xcb2, 0xccd, 0xcaf, 0xcbe, 0xc82, 0xca1, 0xccd, 0x20, 0xca6,
0xcbf, 0xca8, 0xca6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcbf, 0xcaf, 0xcc1, 0x20,
0xcb8, 0xcae, 0xcaf, 0xca8, 0xcbe, 0xcb0, 0xccd, 0xcab, 0xccb, 0xc95, 0xccd, 0x20,
0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcbe, 0xcb0,
0xccd, 0xcab, 0xccb, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xca8, 0xcbe, 0xcb0, 0xccd, 0xcab, 0xccb, 0xc95, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5,
0xcc0, 0xcaa, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcab, 0xcc6, 0xcb0, 0xccd, 0xca8, 0xcbe, 0xc82, 0xca1, 0xccb, 0x20, 0xca1,
0xcc6, 0x20, 0xca8, 0xcca, 0xcb0, 0xcca, 0xca8, 0xccd, 0xcb9, 0xcbe, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcab, 0xcc6, 0xcb0, 0xccd, 0xca8, 0xcbe, 0xc82, 0xca1, 0xccb, 0x20,
0xca1, 0xcc6, 0x20, 0xca8, 0xcca, 0xcb0, 0xcca, 0xca8, 0xccd, 0xcb9, 0xcbe, 0x20,
0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcab, 0xcb0, 0xccd, 0xca8, 0xcbe, 0xc82, 0xca1, 0xccb, 0x20, 0xca6, 0xcc6, 0x20,
0xca8, 0xcca, 0xcb0, 0xcca, 0xca8, 0xccd, 0xcb9, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xca4, 0xccd, 0xca4, 0xcb0,
0x20, 0xcae, 0xcb0, 0xcbf, 0xcaf, 0xcbe, 0xca8, 0xcbe, 0x20, 0xca6, 0xccd, 0xcb5,
0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcca, 0xcb5, 0xcca,
0xcb8, 0xcbf, 0xcac, 0xcbf, 0xcb0, 0xccd, 0x200c, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xca8, 0xcca, 0xcb5, 0xcca, 0xcb8, 0xcbf, 0xcac, 0xcbf, 0xcb0,
0xccd, 0x200c, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca8, 0xcca, 0xcb5, 0xcca, 0xcb8,
0xcbf, 0xcac, 0xcbf, 0xcb0, 0xccd, 0x200c, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc92, 0xcae, 0xcbe,
0xcb8, 0xccd, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc92, 0xcae, 0xcbe, 0xcb8,
0xccd, 0xc95, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc92, 0xcae, 0xcbe, 0xcb8, 0xccd, 0xc95, 0xccd, 0x20,
0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcbe,
0xc95, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa,
0xcbe, 0xc95, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcbe, 0xc95,
0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcbe, 0xcb2, 0xcbe, 0xcb5, 0xccd, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcaa, 0xcc1, 0xcb5, 0xcbe, 0x20, 0xca8, 0xccd, 0xcaf,
0xcc2, 0x20, 0xc97, 0xcbf, 0xca8, 0xcbf, 0xcaf, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcaa, 0xcb0, 0xcbe, 0xc97, 0xccd, 0xcb5, 0xcc7, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa,
0xcb0, 0xcbe, 0xc97, 0xccd, 0xcb5, 0xcc7, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcb0, 0xcbe, 0xc97, 0xccd,
0xcb5, 0xcc7, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaa, 0xcc6, 0xcb0, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcc6, 0xcb0,
0xcc1, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcaa, 0xcc6, 0xcb0, 0xcc1, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbf, 0xcb2, 0xcbf, 0xcab, 0xcc8, 0xca8,
0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbf, 0xcb2, 0xcbf, 0xcab, 0xcc8, 0xca8,
0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcab, 0xcbf, 0xcb2, 0xcbf, 0xcab, 0xcc8, 0xca8, 0xccd, 0x20, 0xcac,
0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcab, 0xcbf, 0xca8,
0xcbf, 0xc95, 0xccd, 0xcb8, 0xccd, 0x20, 0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0xc97,
0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcaa,
0xcbf, 0xcaf, 0xcb0, 0xccd, 0x20, 0xcae, 0xca4, 0xccd, 0xca4, 0xcc1, 0x20, 0xcae,
0xcbf, 0xc95, 0xccd, 0xcb5, 0xcc6, 0xcb2, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcaa, 0xcbf, 0xcaf, 0xcb0, 0xccd, 0x20,
0xcae, 0xca4, 0xccd, 0xca4, 0xcc1, 0x20, 0xcae, 0xcbf, 0xc95, 0xccd, 0xcb5, 0xcc6,
0xcb2, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcc7, 0xc82, 0xc9f, 0xccd, 0x20, 0xcaa, 0xcbf,
0xcaf, 0xcb0, 0xccd, 0x20, 0xcae, 0xca4, 0xccd, 0xca4, 0xcc1, 0x20, 0xcae, 0xcbf,
0xc95, 0xccd, 0xcb5, 0xcc6, 0xcb2, 0xca8, 0xccd, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcbf, 0xc9f, 0xccd, 0x200c, 0xc95, 0xcc8, 0xcb0,
0xccd, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xcca, 0xca8, 0xcbe, 0xcaa,
0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaa, 0xccd, 0xcaf, 0xcca, 0xc82, 0xc97, 0xccd,
0xcaf, 0xcbe, 0xc82, 0xc97, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccb, 0xc9c,
0xcbf, 0xcb2, 0xccb, 0xcb0, 0xccd, 0xca1, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95,
0xccb, 0xc9c, 0xcbf, 0xcb2, 0xccb, 0xcb0, 0xccd, 0xca1, 0xcbe, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc95, 0xccb,
0xc9c, 0xcbf, 0xcb2, 0xccb, 0xcb0, 0xccd, 0xca1, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb0, 0xcbf, 0xcaf, 0xcc2, 0xca8,
0xcbf, 0xcaf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb0, 0xcca, 0xca4, 0xcc7,
0xcb0, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xccd, 0xcaf, 0xcbe, 0xc95, 0xccd,
0x200c, 0xcb9, 0xcb2, 0xcbf, 0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xccd,
0xcaf, 0xcbe, 0xc95, 0xccd, 0x200c, 0xcb9, 0xcb2, 0xcbf, 0xca8, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8,
0xccd, 0xcaf, 0xcbe, 0xc95, 0xccd, 0x200c, 0xcb9, 0xcb2, 0xcbf, 0xca8, 0xccd, 0x20,
0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcae,
0xcb0, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcae, 0xcb0, 0x20, 0xcaa, 0xccd, 0xcb0,
0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcae, 0xcb0,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8,
0xcae, 0xccb, 0xcb5, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcae, 0xccb, 0xcb5,
0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcb8, 0xcae, 0xccb, 0xcb5, 0xcbe, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcc0, 0xcb7, 0xcc6, 0xcb2, 0xccd,
0xcb8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcbf, 0xc82, 0xc97, 0xcaa, 0xcc1,
0xcb0, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcbe, 0xcb2, 0xcae, 0xca8, 0xccd, 0x20, 0xca6, 0xccd,
0xcb5, 0xcc0, 0xcaa, 0xc97, 0xcb3, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca6, 0xc95, 0xccd,
0xcb7, 0xcbf, 0xca3, 0x20, 0xc9c, 0xcbe, 0xcb0, 0xccd, 0xc9c, 0xcbf, 0xcaf, 0xcbe,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xcc1, 0xcb0, 0xcbf, 0xca8, 0xcc7, 0xcae, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcb8, 0xccd, 0xcaf, 0xcca, 0xcb5, 0xcbe, 0x20, 0xcb8,
0xcae, 0xcaf, 0xca4, 0xcbe, 0xcb9, 0xcbf, 0xca4, 0xcbf, 0x20, 0xcb8, 0xcae, 0xcaf,
0xca4, 0xcc8, 0xcaa, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc8, 0xcaa, 0xcc6,
0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae,
0xcaf, 0xca4, 0xcc8, 0xcaa, 0xcc6, 0x20, 0xcb9, 0xc97, 0xcb2, 0xcc1, 0x20, 0xcb8,
0xcae, 0xcaf, 0xca4, 0xc9d, 0xc95, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc9f, 0xcca, 0xc95, 0xcc6, 0xcb2, 0xcbe, 0xcb5, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xc9f, 0xcca, 0xc82, 0xc97, 0xcbe, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc9f, 0xcca, 0xc82, 0xc97, 0xcbe, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9f, 0xcca, 0xc82, 0xc97, 0xcbe,
0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc9a,
0xcc1, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc1, 0xcb0, 0xccd, 0xc95,
0xccd, 0x200c, 0xcae, 0xcc7, 0xca8, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc1, 0xcb0, 0xccd, 0xc95, 0xccd, 0x200c, 0xcae,
0xcc7, 0xca8, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd,
0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xca4, 0xcc1,
0xcb0, 0xccd, 0xc95, 0xccd, 0x200c, 0xcae, 0xcc7, 0xca8, 0xcbf, 0xcb8, 0xccd, 0xca4,
0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8,
0xcae, 0xcaf, 0xca4, 0xcc1, 0xcb5, 0xcbe, 0xcb2, 0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf,
0xc89, 0xcb0, 0xcc1, 0xc97, 0xccd, 0xcb5, 0xcc7, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89,
0xcb0, 0xcc1, 0xc97, 0xccd, 0xcb5, 0xcc7, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe,
0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xcb0, 0xcc1, 0xc97, 0xccd,
0xcb5, 0xcc7, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae,
0xcaf, 0xc89, 0xc9c, 0xccd, 0xcac, 0xcc7, 0xc95, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe,
0xca8, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xc9c, 0xccd, 0xcac, 0xcc7, 0xc95,
0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xc89, 0xc9c, 0xccd, 0xcac,
0xcc7, 0xc95, 0xcbf, 0xcb8, 0xccd, 0xca4, 0xcbe, 0xca8, 0xccd, 0x20, 0xcac, 0xcc7,
0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xca8, 0xccc, 0xca4,
0xcc1, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xca8, 0xccc, 0xca4, 0xcc1, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5,
0xca8, 0xccc, 0xca4, 0xcc1, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb5, 0xcc6, 0xca8, 0xcbf, 0xc9c, 0xcc1, 0xcb5, 0xcc6, 0xcb2,
0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccd, 0xcb2, 0xcbe, 0xca1, 0xcbf, 0xcb5,
0xccb, 0xcb8, 0xccd, 0xc9f, 0xccb, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5,
0xccd, 0xcb2, 0xcbe, 0xca1, 0xcbf, 0xcb5, 0xccb, 0xcb8, 0xccd, 0xc9f, 0xccb, 0xc95,
0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8,
0xcae, 0xcaf, 0xcb5, 0xccd, 0xcb2, 0xcbe, 0xca1, 0xcbf, 0xcb5, 0xccb, 0xcb8, 0xccd,
0xc9f, 0xccb, 0xc95, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf, 0xc97, 0xcc6, 0x20,
0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccb, 0xcb2, 0xccd, 0xc97, 0xccb, 0xc97, 0xcbe, 0xcb0,
0xccd, 0xca1, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccb, 0xcb2, 0xccd, 0xc97,
0xccb, 0xc97, 0xcbe, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae,
0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccb, 0xcb2, 0xccd,
0xc97, 0xccb, 0xc97, 0xcbe, 0xcb0, 0xccd, 0xca1, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccb, 0xcb8, 0xccd, 0xc9f,
0xcca, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xcc7, 0xc95, 0xccd, 0x20,
0xca6, 0xccd, 0xcb5, 0xcc0, 0xcaa, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcb5, 0xccd, 0xcaf,
0xcbe, 0xcb2, 0xcc0, 0xcb8, 0xccd, 0x20, 0xcae, 0xca4, 0xccd, 0xca4, 0xcc1, 0x20,
0xcab, 0xccd, 0xcaf, 0xcc1, 0xc9f, 0xcc1, 0xca8, 0xcbe, 0x20, 0xcb8, 0xcae, 0xcaf,
0xcaf, 0xcbe, 0xc95, 0xcc1, 0xc9f, 0xccd, 0xcb8, 0xc95, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaf, 0xcbe, 0xc95, 0xcc1, 0xc9f, 0xccd, 0xcb8, 0xc95, 0xccd, 0x20, 0xcaa,
0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf,
0xcbe, 0xc95, 0xcc1, 0xc9f, 0xccd, 0xcb8, 0xc95, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8,
0xcbf, 0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf, 0xcc7, 0xc95, 0xc9f, 0xcb0,
0xcbf, 0xca8, 0xccd, 0x200c, 0xcac, 0xcb0, 0xccd, 0xc97, 0xccd, 0x20, 0xcb8, 0xcae,
0xcaf, 0xcaf, 0xcc7, 0xc95, 0xc9f, 0xcb0, 0xcbf, 0xca8, 0xccd, 0x200c, 0xcac, 0xcb0,
0xccd, 0xc97, 0xccd, 0x20, 0xcaa, 0xccd, 0xcb0, 0xcae, 0xcbe, 0xca3, 0xcbf, 0xca4,
0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf, 0xcc7, 0xc95, 0xcc7, 0xc9f, 0xcb0, 0xcbf, 0xca8,
0xccd, 0x200d, 0xcac, 0xcb0, 0xccd, 0xc97, 0xccd, 0x20, 0xcac, 0xcc7, 0xcb8, 0xcbf,
0xc97, 0xcc6, 0x20, 0xcb8, 0xcae, 0xcaf, 0xcaf, 0xcc1, 0xc95, 0xcbe, 0xca8, 0xccd,
0x20, 0xcb8, 0xcae, 0xcaf, 0x627, 0x620, 0x6a9, 0x631, 0x6d2, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9, 0x631, 0x6d2, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620,
0x6a9, 0x631, 0x6d2, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x650, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc,
0x20, 0x627, 0x641, 0x631, 0x6cc, 0x656, 0x642, 0x627, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc,
0x656, 0x642, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x646, 0x648,
0x657, 0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x641,
0x631, 0x6cc, 0x656, 0x642, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x656, 0x642, 0x627,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc,
0x656, 0x642, 0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x620, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x620, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x688, 0x6d2, 0x644,
0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x620, 0x644, 0x645, 0x620, 0x679, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x620, 0x644, 0x645, 0x620, 0x679, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x644,
0x645, 0x620, 0x679, 0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x645, 0x64e, 0x632, 0x64e, 0x646, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x645, 0x64e, 0x632, 0x64e, 0x646,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x627, 0x620, 0x645, 0x64e, 0x632, 0x64e, 0x646, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9,
0x632, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632,
0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x688, 0x6d2, 0x644,
0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x634, 0x631, 0x642, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634,
0x631, 0x642, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x688,
0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x627, 0x648, 0x646, 0x679, 0x6cc, 0x646, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x627, 0x648, 0x646, 0x679, 0x6cc, 0x646, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x627, 0x648, 0x646, 0x679, 0x6cc, 0x646, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc,
0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6cc, 0x633,
0x650, 0x641, 0x650, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6cc,
0x633, 0x650, 0x641, 0x650, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6cc, 0x633, 0x650, 0x641,
0x650, 0x6a9, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x646, 0x64e, 0x691, 0x6cc, 0x656,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x646, 0x64e, 0x691,
0x6cc, 0x656, 0x631, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x646, 0x691, 0x6cc, 0x656, 0x631,
0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x620, 0x6a9, 0x679, 0x627, 0x624, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x620, 0x6a9, 0x679, 0x627, 0x624, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691,
0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x6a9, 0x679,
0x627, 0x624, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x627, 0x620, 0x6a9, 0x679, 0x648, 0x628, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x627, 0x620, 0x6a9, 0x679, 0x648, 0x628, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620,
0x6a9, 0x679, 0x648, 0x628, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x627, 0x631, 0x620, 0x628, 0x650, 0x6cc, 0x64e, 0x646,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x631, 0x620, 0x628, 0x650, 0x6cc,
0x64e, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x627, 0x631, 0x620, 0x628, 0x650, 0x6cc, 0x64e, 0x646,
0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x631, 0x62c, 0x620, 0x646, 0x679, 0x6cc, 0x656, 0x646,
0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x631, 0x62c, 0x620, 0x646,
0x679, 0x6cc, 0x656, 0x646, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x631, 0x62c, 0x620, 0x646,
0x679, 0x6cc, 0x656, 0x646, 0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627,
0x631, 0x62c, 0x620, 0x646, 0x679, 0x6cc, 0x656, 0x646, 0x627, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c,
0x620, 0x646, 0x679, 0x6cc, 0x656, 0x646, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631,
0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x620, 0x646, 0x679, 0x6cc, 0x656, 0x646,
0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x631, 0x645, 0x6cc, 0x656, 0x646, 0x650, 0x6cc, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x627, 0x631, 0x645, 0x6cc, 0x656, 0x646, 0x650, 0x6cc,
0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x631, 0x645, 0x6cc, 0x656, 0x646, 0x650, 0x6cc, 0x627,
0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x627, 0x620, 0x679, 0x644, 0x627, 0x646, 0x679, 0x650, 0x6a9, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x620, 0x679, 0x644, 0x627, 0x646, 0x679, 0x650, 0x6a9,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x627, 0x620, 0x679, 0x644, 0x627, 0x646, 0x679, 0x650, 0x6a9, 0x20,
0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x622, 0x633, 0x679, 0x631,
0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645,
0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644,
0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x688,
0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20,
0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650,
0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x645, 0x63a,
0x631, 0x628, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650,
0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x645, 0x63a,
0x631, 0x628, 0x6cc, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x650, 0x642, 0x6cc,
0x20, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc,
0x64e, 0x646, 0x20, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633,
0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x634, 0x631,
0x642, 0x6cc, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20,
0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e,
0x646, 0x20, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x622, 0x633,
0x679, 0x631, 0x6cc, 0x644, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x645, 0x63a, 0x631,
0x650, 0x628, 0x6cc, 0x656, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x654, 0x679,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x632, 0x631, 0x628, 0x627, 0x626,
0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x632, 0x631,
0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x632, 0x631, 0x628,
0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x620, 0x632, 0x648, 0x631, 0x655, 0x633,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x620, 0x632, 0x648, 0x631, 0x655,
0x633, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x620, 0x632, 0x648, 0x631, 0x655, 0x633, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x64e, 0x646,
0x6af, 0x644, 0x627, 0x62f, 0x6cc, 0x634, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x628, 0x64e, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628,
0x64e, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x648, 0x657, 0x679,
0x627, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x648, 0x644, 0x650,
0x648, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x631,
0x620, 0x633, 0x650, 0x644, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x628, 0x631, 0x620, 0x633, 0x650, 0x644, 0x650, 0x6cc, 0x627, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x628, 0x631, 0x620, 0x633, 0x650, 0x644, 0x650, 0x6cc, 0x627, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x628, 0x631, 0x648,
0x657, 0x646, 0x64e, 0x6d2, 0x20, 0x62f, 0x64e, 0x631, 0x648, 0x657, 0x633, 0x64e,
0x644, 0x64e, 0x645, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x6cc, 0x67e,
0x20, 0x624, 0x631, 0x691, 0x648, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9,
0x6cc, 0x67e, 0x20, 0x624, 0x631, 0x691, 0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x6cc, 0x67e,
0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x6a9, 0x620, 0x645, 0x648, 0x631, 0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691,
0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x620, 0x62a, 0x6be,
0x64e, 0x645, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x620, 0x62a, 0x6be,
0x64e, 0x645, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x686, 0x620, 0x62a, 0x6be, 0x64e, 0x645, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x686, 0x650,
0x644, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x686, 0x650, 0x644, 0x6cc,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x686, 0x650, 0x644, 0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x686, 0x64e, 0x6cc, 0x646, 0x627, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x686, 0x64e, 0x6cc, 0x646, 0x627, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x686, 0x64e, 0x6cc, 0x646, 0x627, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650,
0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x6d2, 0x628,
0x620, 0x644, 0x633, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9,
0x648, 0x6d2, 0x628, 0x620, 0x644, 0x633, 0x64e, 0x646, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x648,
0x6d2, 0x628, 0x620, 0x644, 0x633, 0x64e, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x631, 0x633, 0x645, 0x64e,
0x633, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x6a9, 0x648, 0x632,
0x20, 0x627, 0x64e, 0x6cc, 0x644, 0x6cc, 0x646, 0x691, 0x632, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x644, 0x648, 0x645, 0x628, 0x650, 0x6cc, 0x627,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x644, 0x648, 0x645, 0x628,
0x650, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x644, 0x648, 0x645, 0x628, 0x650,
0x6cc, 0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x6a9, 0x64f, 0x6a9, 0x20, 0x627, 0x64e, 0x6cc, 0x644, 0x6cc, 0x646,
0x691, 0x632, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x64f, 0x6a9, 0x20,
0x627, 0x64e, 0x6cc, 0x644, 0x6cc, 0x646, 0x691, 0x632, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x64f,
0x6a9, 0x20, 0x627, 0x64e, 0x6cc, 0x644, 0x6cc, 0x646, 0x691, 0x632, 0x20, 0x62d,
0x627, 0x641, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x6a9, 0x6cc, 0x648, 0x657, 0x628, 0x627, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x6a9, 0x6cc, 0x648, 0x657, 0x628, 0x627, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x6cc,
0x648, 0x657, 0x628, 0x627, 0x20, 0x688, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654,
0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x691, 0x6cc, 0x648, 0x650, 0x633,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x691, 0x645, 0x627, 0x646, 0x679, 0x20,
0x688, 0x6cc, 0x20, 0x627, 0x64f, 0x631, 0x648, 0x6cc, 0x656, 0x644, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6cc, 0x656, 0x633, 0x679, 0x20, 0x679, 0x6cc,
0x656, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6cc,
0x656, 0x633, 0x679, 0x64e, 0x631, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x655, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6cc, 0x656, 0x633, 0x679, 0x64e, 0x631,
0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x655, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6cc, 0x656, 0x633,
0x679, 0x64e, 0x631, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x655, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x6a9,
0x648, 0x6cc, 0x691, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x6cc,
0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc,
0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc,
0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e,
0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x657,
0x631, 0x67e, 0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x632, 0x6cc, 0x62f, 0x20, 0x645, 0x634, 0x631, 0x642,
0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645,
0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e,
0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628,
0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x67e, 0x6cc, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a,
0x631, 0x650, 0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x657, 0x631, 0x650, 0x67e, 0x6cc,
0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x641, 0x627, 0x644, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x62c, 0x632,
0x6cc, 0x631, 0x655, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x627, 0x644, 0x6a9,
0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x655, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x641, 0x627, 0x644, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x62c, 0x632,
0x6cc, 0x631, 0x655, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x641, 0x6cc, 0x656, 0x62c, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x641, 0x6cc, 0x656, 0x62c, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691,
0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x641, 0x6cc, 0x656, 0x62c,
0x6cc, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x641, 0x631, 0x620, 0x646, 0x686, 0x20, 0x6af, 0x6cc, 0x648, 0x657, 0x620,
0x646, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x641, 0x631, 0x6cc, 0x646,
0x686, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20, 0x62a, 0x655, 0x6c1, 0x20,
0x627, 0x646, 0x679, 0x627, 0x631, 0x6a9, 0x679, 0x650, 0x6a9, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6af, 0x620, 0x644, 0x620, 0x67e, 0x6cc, 0x6af, 0x648, 0x632, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6af, 0x620, 0x645, 0x628, 0x650, 0x6cc, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x648, 0x631, 0x62c, 0x650,
0x6cc, 0x627, 0x6c1, 0x64f, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c,
0x648, 0x631, 0x62c, 0x650, 0x6cc, 0x627, 0x6c1, 0x64f, 0x6a9, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c,
0x648, 0x631, 0x62c, 0x650, 0x6cc, 0x627, 0x6c1, 0x64f, 0x6a9, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6af, 0x650, 0x644,
0x628, 0x672, 0x679, 0x20, 0x62c, 0x62c, 0x6cc, 0x656, 0x631, 0x64f, 0x6a9, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6af, 0x631, 0x6cc, 0x656, 0x646, 0x20, 0x648,
0x650, 0x686, 0x20, 0x645, 0x6cc, 0x656, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x634, 0x631, 0x650, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x656,
0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x64f, 0x6a9, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x645, 0x634, 0x631, 0x650, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc,
0x656, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x64f, 0x6a9, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x634, 0x631, 0x650, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x656, 0x646, 0x20,
0x644, 0x6cc, 0x646, 0x688, 0x64f, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc,
0x20, 0x6af, 0x631, 0x6cc, 0x656, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x64f,
0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628,
0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x656, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688,
0x64f, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x6af,
0x631, 0x6cc, 0x656, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x64f, 0x6a9, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6af,
0x64f, 0x648, 0x620, 0x645, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6af, 0x64e,
0x644, 0x641, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x6af, 0x64f, 0x6cc, 0x64e, 0x646, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x62d, 0x64e, 0x648, 0x627, 0x6d2, 0x20, 0x627, 0x620,
0x644, 0x6cc, 0x648, 0x657, 0x679, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x62d, 0x64e, 0x648, 0x627, 0x6d2, 0x20, 0x627, 0x620, 0x644,
0x6cc, 0x648, 0x657, 0x679, 0x650, 0x6cc, 0x64e, 0x646, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62d, 0x64e,
0x648, 0x627, 0x6d2, 0x20, 0x627, 0x620, 0x644, 0x6cc, 0x648, 0x657, 0x679, 0x650,
0x6cc, 0x64e, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627,
0x646, 0x6af, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af,
0x20, 0x633, 0x64e, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62d, 0x648,
0x648, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62d, 0x648, 0x648, 0x691,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x62d, 0x648, 0x648, 0x691, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6c1, 0x650, 0x646, 0x62f, 0x648, 0x633,
0x62a, 0x627, 0x646, 0x6c1, 0x650, 0x646, 0x62f, 0x648, 0x633, 0x62a, 0x672, 0x646,
0x620, 0x20, 0x627, 0x648, 0x634, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x650, 0x646, 0x691, 0x648, 0x686, 0x64e, 0x6cc, 0x646, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x645, 0x631, 0x6a9, 0x632, 0x6cc, 0x20, 0x627, 0x650,
0x646, 0x691, 0x648, 0x646, 0x6cc, 0x634, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x634, 0x631, 0x650, 0x642, 0x6cc, 0x20, 0x627, 0x650,
0x646, 0x691, 0x648, 0x646, 0x6cc, 0x634, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x63a, 0x631, 0x650, 0x628, 0x6cc, 0x20, 0x627, 0x650,
0x646, 0x691, 0x648, 0x646, 0x6cc, 0x634, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x650, 0x6cc, 0x656, 0x631, 0x672, 0x646, 0x620, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x6cc, 0x656, 0x631, 0x672, 0x646,
0x620, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x627, 0x650, 0x6cc, 0x656, 0x631, 0x672, 0x646, 0x6cc, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x650, 0x631, 0x6a9, 0x64f, 0x679, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x627, 0x650, 0x631, 0x6a9, 0x64f, 0x679, 0x633, 0x6a9, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x650, 0x631, 0x6a9, 0x64f, 0x679, 0x633, 0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x633, 0x631, 0x672,
0x6cc, 0x650, 0x644, 0x6cc, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650,
0x633, 0x631, 0x672, 0x6cc, 0x650, 0x644, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x650, 0x633,
0x631, 0x672, 0x6cc, 0x650, 0x644, 0x6cc, 0x20, 0x691, 0x6d2, 0x644, 0x627, 0x6cc,
0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x627, 0x67e,
0x672, 0x646, 0x620, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x627, 0x67e,
0x672, 0x646, 0x620, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x62c, 0x627, 0x67e, 0x672, 0x646, 0x620, 0x20,
0x691, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x6a9, 0x64e, 0x645, 0x686, 0x64e, 0x679, 0x6a9, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x64e, 0x645, 0x686, 0x64e, 0x679, 0x6a9, 0x627,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x6a9, 0x64e, 0x645, 0x686, 0x64e, 0x679, 0x6a9, 0x627, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x634,
0x631, 0x642, 0x6cc, 0x20, 0x642, 0x627, 0x632, 0x642, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x642,
0x627, 0x632, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645,
0x6a9, 0x648, 0x631, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x6a9, 0x648, 0x631, 0x650, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x631, 0x650,
0x6cc, 0x627, 0x20, 0x691, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x648, 0x631, 0x633, 0x64e, 0x6d2, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9, 0x631, 0x620, 0x633, 0x646, 0x648, 0x6d2,
0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6a9,
0x631, 0x620, 0x633, 0x646, 0x648, 0x6d2, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x6a9, 0x631, 0x620, 0x633, 0x646, 0x648, 0x6d2, 0x6cc, 0x627, 0x631, 0x633,
0x6a9, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x6a9, 0x631, 0x63a, 0x632, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627,
0x626, 0x645, 0x644, 0x64e, 0x646, 0x6a9, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x646, 0x20, 0x62c, 0x62c, 0x6cc, 0x656,
0x631, 0x64f, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x644, 0x639, 0x627,
0x691, 0x20, 0x62d, 0x648, 0x648, 0x6d2, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x644, 0x639, 0x627, 0x691, 0x20, 0x62d, 0x648, 0x648, 0x6d2, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x644,
0x639, 0x627, 0x691, 0x20, 0x691, 0x6d2, 0x644, 0x627, 0x6cc, 0x654, 0x679, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x64e, 0x6a9, 0x64e, 0x639, 0x648, 0x657,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x64e, 0x6a9, 0x64e, 0x639, 0x648,
0x657, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x64e, 0x6a9, 0x64e, 0x639, 0x648, 0x657, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x64e,
0x6af, 0x64e, 0x62f, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645,
0x64e, 0x6af, 0x64e, 0x62f, 0x64e, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x64e, 0x6af, 0x64e,
0x62f, 0x64e, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x645, 0x64e, 0x644, 0x6cc, 0x634, 0x650, 0x6cc, 0x627, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x644, 0x62f, 0x6cc, 0x656, 0x648,
0x655, 0x632, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x631, 0x642,
0x6cc, 0x648, 0x657, 0x633, 0x64e, 0x633, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x645, 0x627, 0x631, 0x634, 0x64e, 0x644, 0x20, 0x62c, 0x62c, 0x6cc, 0x656, 0x631,
0x64f, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x648, 0x631, 0x650,
0x634, 0x64e, 0x633, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x648, 0x631,
0x650, 0x634, 0x64e, 0x633, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x648, 0x631, 0x650, 0x634, 0x64e,
0x633, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x645, 0x627, 0x633, 0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x645, 0x6cc, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x6cc,
0x641, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6a9, 0x633, 0x6cc,
0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x6cc, 0x641, 0x6a9, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc,
0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x6cc, 0x641, 0x6a9,
0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626,
0x645, 0x627, 0x648, 0x644, 0x646, 0x20, 0x628, 0x627, 0x679, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x648, 0x644, 0x646, 0x20, 0x628, 0x627, 0x679, 0x631,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x627, 0x648, 0x644, 0x646, 0x20, 0x628, 0x627, 0x679, 0x631, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627,
0x633, 0x6a9, 0x64e, 0x648, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627,
0x633, 0x6a9, 0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x645, 0x650,
0x6cc, 0x627, 0x646, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x646, 0x64e, 0x639, 0x648, 0x657, 0x631, 0x648, 0x657, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x646, 0x620, 0x67e, 0x672, 0x644, 0x620, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x646, 0x650, 0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x620, 0x691, 0x648,
0x646, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x650,
0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x620, 0x691, 0x648, 0x646, 0x650, 0x6cc, 0x627,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x646, 0x650, 0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x620, 0x691, 0x648,
0x646, 0x650, 0x6cc, 0x633, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x646, 0x650, 0x648, 0x632, 0x650, 0x644, 0x6cc, 0x646,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x650, 0x648, 0x632, 0x650,
0x644, 0x6cc, 0x646, 0x691, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x650, 0x648, 0x632, 0x650, 0x644,
0x6cc, 0x646, 0x691, 0x20, 0x691, 0x6d2, 0x644, 0x627, 0x6cc, 0x654, 0x679, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x6cc, 0x648, 0x20, 0x641, 0x627, 0x624,
0x646, 0x688, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x646, 0x6cc, 0x648, 0x20, 0x641, 0x627, 0x624, 0x646, 0x688, 0x20, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x646, 0x6cc, 0x648, 0x20, 0x641, 0x627, 0x624, 0x646, 0x688,
0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626,
0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x650, 0x6cc, 0x648, 0x657, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x648, 0x631, 0x641, 0x639, 0x627, 0x6a9,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x648, 0x631, 0x641, 0x639, 0x627,
0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x646, 0x648, 0x631, 0x641, 0x639, 0x627, 0x6a9, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x641, 0x631,
0x646, 0x6cc, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631,
0x648, 0x646, 0x6c1, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x631, 0x646,
0x6cc, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648,
0x646, 0x6c1, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x641, 0x631, 0x646, 0x6cc, 0x646, 0x688, 0x648, 0x20,
0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6c1, 0x627, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x634, 0x64f, 0x645,
0x672, 0x644, 0x6cc, 0x20, 0x645, 0x64e, 0x631, 0x650, 0x6cc, 0x627, 0x646, 0x627,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646, 0x6c4, 0x648, 0x6c4, 0x633, 0x650,
0x628, 0x654, 0x631, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x646,
0x6c4, 0x648, 0x6c4, 0x633, 0x650, 0x628, 0x654, 0x631, 0x633, 0x6a9, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x646, 0x6c4, 0x648, 0x6c4, 0x633, 0x650, 0x628, 0x654, 0x631, 0x633, 0x6a9, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627,
0x6c4, 0x645, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6c4,
0x645, 0x633, 0x6a9, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x6c4, 0x645, 0x633, 0x6a9, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x627,
0x6a9, 0x650, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x67e, 0x627, 0x6a9, 0x650, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x627,
0x6a9, 0x650, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x64e, 0x644, 0x627, 0x648, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x627, 0x67e, 0x64f, 0x639, 0x627, 0x20,
0x646, 0x6cc, 0x648, 0x657, 0x20, 0x6af, 0x620, 0x646, 0x6cc, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x67e, 0x6cc, 0x631, 0x620, 0x6af, 0x648, 0x6d2, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6cc, 0x631, 0x620, 0x6af, 0x648, 0x6d2, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x67e, 0x6cc, 0x631, 0x620, 0x6af, 0x648, 0x6d2, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x654, 0x631, 0x648,
0x657, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x654, 0x631, 0x648, 0x657,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x67e, 0x654, 0x631, 0x648, 0x657, 0x20, 0x633, 0x64e, 0x645, 0x64e,
0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6be, 0x650, 0x644, 0x650,
0x67e, 0x627, 0x6cc, 0x650, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e,
0x6be, 0x650, 0x644, 0x650, 0x67e, 0x627, 0x6cc, 0x650, 0x646, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e,
0x6be, 0x650, 0x644, 0x650, 0x67e, 0x627, 0x6cc, 0x650, 0x646, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x6be, 0x648,
0x646, 0x650, 0x6a9, 0x633, 0x20, 0x62c, 0x62c, 0x6cc, 0x656, 0x631, 0x64f, 0x6a9,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e,
0x64e, 0x6cc, 0x631, 0x6cc, 0x20, 0x645, 0x650, 0x642, 0x6cc, 0x648, 0x657, 0x644,
0x64e, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x6cc, 0x646, 0x679,
0x20, 0x67e, 0x64e, 0x6cc, 0x631, 0x6cc, 0x20, 0x645, 0x650, 0x642, 0x6cc, 0x648,
0x657, 0x644, 0x64e, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e,
0x64e, 0x6cc, 0x631, 0x6cc, 0x20, 0x645, 0x650, 0x642, 0x6cc, 0x648, 0x657, 0x644,
0x64e, 0x646, 0x20, 0x691, 0x6d2, 0x644, 0x627, 0x6cc, 0x650, 0x654, 0x679, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x650, 0x679, 0x6a9, 0x6cc, 0x631, 0x655,
0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x67e, 0x648, 0x646, 0x6cc, 0x67e,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x648, 0x646, 0x6af, 0x20, 0x6cc,
0x627, 0x646, 0x6af, 0x20, 0x679, 0x627, 0x626, 0x645, 0x642, 0x650, 0x632, 0x644,
0x648, 0x691, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x642, 0x650, 0x632,
0x644, 0x648, 0x691, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x642, 0x650, 0x632, 0x644, 0x648, 0x691,
0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x631, 0x650, 0x6cc, 0x648, 0x657, 0x646, 0x650, 0x6cc, 0x64e, 0x646, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x631, 0x648, 0x62a, 0x6be, 0x620, 0x631, 0x627,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x64e, 0x6a9, 0x6be, 0x620, 0x644,
0x650, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x64e, 0x6a9, 0x6be,
0x620, 0x644, 0x650, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x64e, 0x6a9, 0x6be, 0x620, 0x644,
0x650, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x633, 0x645, 0x620, 0x631, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x633, 0x645, 0x620, 0x631, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691,
0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x645, 0x620, 0x631,
0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x633, 0x620, 0x645, 0x648, 0x622, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x633, 0x620, 0x645, 0x648, 0x622, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627,
0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x633, 0x620, 0x645, 0x648, 0x622,
0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x633, 0x6cc, 0x634, 0x620, 0x644, 0x655, 0x632, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x633, 0x650, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x657, 0x631, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x633, 0x648, 0x644, 0x648, 0x645, 0x64e, 0x646, 0x20,
0x62c, 0x62c, 0x6cc, 0x656, 0x631, 0x64e, 0x646, 0x20, 0x6c1, 0x64f, 0x646, 0x62f,
0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x634, 0x64f, 0x645, 0x672, 0x644, 0x6cc,
0x20, 0x62c, 0x648, 0x631, 0x62c, 0x650, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x633, 0x64f, 0x631, 0x650, 0x646, 0x627, 0x645, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x633, 0x6cc, 0x648, 0x648, 0x627, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x679, 0x627, 0x6c1, 0x650, 0x679, 0x6cc, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x679, 0x6d2, 0x20, 0x67e, 0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645,
0x679, 0x6d2, 0x20, 0x67e, 0x6d2, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x679, 0x6d2, 0x20, 0x67e, 0x6d2, 0x20,
0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645,
0x62a, 0x627, 0x62c, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626,
0x645, 0x679, 0x648, 0x6a9, 0x620, 0x644, 0x648, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x679, 0x639, 0x627, 0x646, 0x6af, 0x627, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x679, 0x639, 0x627, 0x646, 0x6af, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x691, 0x627, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x679, 0x639, 0x627,
0x646, 0x6af, 0x627, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x679, 0x654, 0x631, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x679, 0x627, 0x626, 0x645, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x62a, 0x64f, 0x631, 0x6a9, 0x645, 0x620, 0x646, 0x650, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x679, 0x648, 0x657, 0x648, 0x64e, 0x644, 0x648, 0x657, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x648, 0x657, 0x631, 0x620, 0x6af, 0x648,
0x64e, 0x6d2, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x648, 0x657, 0x631,
0x620, 0x6af, 0x648, 0x64e, 0x6d2, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x648, 0x657, 0x631, 0x620,
0x6af, 0x648, 0x64e, 0x6d2, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x627, 0x64f, 0x632, 0x628, 0x6cc, 0x6a9, 0x650, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x64f, 0x632,
0x628, 0x6cc, 0x6a9, 0x650, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x627, 0x64f,
0x632, 0x628, 0x6cc, 0x6a9, 0x650, 0x633, 0x62a, 0x627, 0x646, 0x64f, 0x6a9, 0x20,
0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648,
0x64e, 0x646, 0x648, 0x657, 0x627, 0x64e, 0x679, 0x648, 0x657, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x648, 0x64e, 0x646, 0x648, 0x657, 0x627, 0x64e, 0x679, 0x648,
0x657, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x691, 0x627, 0x691, 0x20, 0x679, 0x627,
0x6cc, 0x650, 0x645, 0x648, 0x64e, 0x646, 0x648, 0x657, 0x627, 0x64e, 0x679, 0x648,
0x657, 0x20, 0x633, 0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x648, 0x620, 0x646, 0x620, 0x632, 0x6cc, 0x648, 0x657, 0x644, 0x627, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x644, 0x627, 0x691, 0x650, 0x648, 0x648,
0x633, 0x679, 0x648, 0x6a9, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x644,
0x627, 0x691, 0x650, 0x648, 0x648, 0x633, 0x679, 0x648, 0x6a9, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648,
0x644, 0x627, 0x691, 0x650, 0x648, 0x648, 0x633, 0x679, 0x648, 0x6a9, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x648,
0x644, 0x6af, 0x648, 0x6af, 0x631, 0x6cc, 0x691, 0x20, 0x679, 0x627, 0x6cc, 0x650,
0x645, 0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x6cc, 0x691, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645,
0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631, 0x6cc, 0x691, 0x20, 0x633, 0x64e,
0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x648, 0x6cc, 0x6a9,
0x20, 0x62c, 0x62c, 0x6cc, 0x656, 0x631, 0x64f, 0x6a9, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x648, 0x627, 0x644, 0x650, 0x633, 0x20, 0x62a, 0x6c1, 0x655, 0x20,
0x641, 0x6cc, 0x648, 0x657, 0x679, 0x6cc, 0x648, 0x657, 0x646, 0x627, 0x20, 0x679,
0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x64e, 0x6a9, 0x64f, 0x679, 0x633, 0x6a9, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x64e, 0x6a9, 0x64f, 0x679, 0x633, 0x6a9,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc,
0x650, 0x645, 0x6cc, 0x64e, 0x6a9, 0x64f, 0x679, 0x64f, 0x633, 0x6a9, 0x20, 0x633,
0x64e, 0x645, 0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x620,
0x6a9, 0x64e, 0x679, 0x654, 0x631, 0x650, 0x646, 0x628, 0x654, 0x631, 0x6af, 0x20,
0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x620, 0x6a9, 0x64e, 0x679, 0x654, 0x631,
0x650, 0x646, 0x628, 0x654, 0x631, 0x6af, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x620, 0x6a9, 0x64e,
0x679, 0x631, 0x650, 0x646, 0x628, 0x654, 0x631, 0x6af, 0x20, 0x633, 0x64e, 0x645,
0x64e, 0x631, 0x20, 0x679, 0x627, 0x6cc, 0x650, 0x645, 0x6cc, 0x648, 0x6a9, 0x648,
0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x938, 0x947, 0x902, 0x91f, 0x94d, 0x930,
0x932, 0x20, 0x935, 0x916, 0x938, 0x947, 0x902, 0x91f, 0x94d, 0x930, 0x932, 0x20,
0x938, 0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935,
0x916, 0x938, 0x947, 0x902, 0x91f, 0x94d, 0x930, 0x932, 0x20, 0x921, 0x947, 0x20,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x935, 0x916, 0x92e, 0x936, 0x930, 0x93f, 0x915,
0x940, 0x20, 0x935, 0x916, 0x92e, 0x936, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x938,
0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916,
0x92e, 0x936, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x921, 0x947, 0x20, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x935, 0x916, 0x92e, 0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928,
0x20, 0x935, 0x916, 0x92e, 0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x938,
0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935, 0x916,
0x92e, 0x93e, 0x909, 0x902, 0x91f, 0x947, 0x928, 0x20, 0x921, 0x947, 0x20, 0x932,
0x93e, 0x907, 0x91f, 0x20, 0x935, 0x916, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f,
0x915, 0x20, 0x935, 0x916, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20,
0x938, 0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d, 0x921, 0x20, 0x935,
0x916, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x20,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x935, 0x916, 0x905, 0x91f, 0x932, 0x93e, 0x902,
0x91f, 0x93f, 0x915, 0x20, 0x935, 0x916, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f,
0x93f, 0x915, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d,
0x921, 0x20, 0x935, 0x916, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915,
0x20, 0x921, 0x947, 0x20, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x935, 0x916, 0x92e,
0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940, 0x20,
0x935, 0x916, 0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942, 0x930,
0x92a, 0x940, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d,
0x921, 0x20, 0x935, 0x916, 0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x92f,
0x942, 0x930, 0x92a, 0x940, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x935, 0x916, 0x92e,
0x936, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940, 0x20,
0x935, 0x916, 0x92e, 0x936, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92f, 0x942, 0x930,
0x92a, 0x940, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d,
0x921, 0x20, 0x935, 0x916, 0x92e, 0x936, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92f,
0x942, 0x930, 0x92a, 0x940, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x935, 0x916, 0x92e,
0x917, 0x930, 0x940, 0x92c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x92a, 0x940, 0x20,
0x935, 0x916, 0x92e, 0x917, 0x930, 0x940, 0x92c, 0x940, 0x20, 0x92f, 0x942, 0x930,
0x92a, 0x940, 0x20, 0x938, 0x94d, 0x91f, 0x948, 0x928, 0x94d, 0x921, 0x930, 0x94d,
0x921, 0x20, 0x935, 0x916, 0x92e, 0x917, 0x930, 0x940, 0x92c, 0x940, 0x20, 0x92f,
0x942, 0x930, 0x92a, 0x940, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x935, 0x916, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f, 0x91a, 0x20, 0x913, 0x938, 0x924, 0x20,
0x935, 0x916, 0x410, 0x443, 0x493, 0x430, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x440, 0x442, 0x430, 0x43b, 0x44b,
0x49b, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x4a3,
0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441,
0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x410, 0x444, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b,
0x441, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43b, 0x44f,
0x441, 0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43b,
0x44f, 0x441, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43b,
0x44f, 0x441, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a,
0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43c, 0x430, 0x437,
0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af,
0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x43e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x43e, 0x440, 0x442, 0x430,
0x43b, 0x44b, 0x49b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e,
0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x43e, 0x440,
0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x448, 0x44b, 0x493, 0x44b, 0x441, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441,
0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x448,
0x44b, 0x493, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421,
0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x448,
0x44b, 0x493, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421,
0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x442, 0x430, 0x443, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x442, 0x430, 0x443, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441,
0x442, 0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x442, 0x430, 0x443, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442, 0x456,
0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x422, 0x44b,
0x43d, 0x44b, 0x49b, 0x20, 0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442,
0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x422, 0x44b,
0x43d, 0x44b, 0x49b, 0x20, 0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x43e, 0x43b, 0x442, 0x4af, 0x441, 0x442,
0x456, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x49b, 0x20, 0x43c,
0x4b1, 0x445, 0x438, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x410, 0x43f, 0x438, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410,
0x43f, 0x438, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x43f, 0x438,
0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x421, 0x430, 0x443, 0x434, 0x20, 0x410, 0x440, 0x430, 0x431, 0x438,
0x44f, 0x441, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430,
0x443, 0x434, 0x20, 0x410, 0x440, 0x430, 0x431, 0x438, 0x44f, 0x441, 0x44b, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x443, 0x434, 0x20, 0x410, 0x440,
0x430, 0x431, 0x438, 0x44f, 0x441, 0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430,
0x442, 0x44b, 0x441, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410,
0x440, 0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438,
0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x410,
0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x43e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x43e, 0x440, 0x442, 0x430, 0x43b, 0x44b,
0x49b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x443, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x43e, 0x440, 0x442, 0x430, 0x43b, 0x44b,
0x49b, 0x2d, 0x431, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x43e,
0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x2d, 0x431, 0x430, 0x442, 0x44b, 0x441,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x443, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x43e,
0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x2d, 0x431, 0x430, 0x442, 0x44b, 0x441,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441,
0x20, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x448, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x448, 0x44b, 0x493, 0x44b, 0x441, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20,
0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x431, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x431, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4d8, 0x437, 0x456, 0x440, 0x431, 0x430, 0x439,
0x436, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4d8, 0x437,
0x456, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x4d8, 0x437, 0x456, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430, 0x43d,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x437, 0x43e,
0x440, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430,
0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411,
0x443, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411,
0x43e, 0x43b, 0x438, 0x432, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x2d, 0x414, 0x430,
0x440, 0x443, 0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434,
0x435, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x430, 0x431, 0x43e,
0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x427,
0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x427, 0x430, 0x442, 0x435, 0x43c, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x427, 0x430, 0x442, 0x435, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x427,
0x430, 0x442, 0x435, 0x43c, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x49a, 0x44b, 0x442, 0x430, 0x439,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x49a, 0x44b, 0x442, 0x430, 0x439,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x49a, 0x44b, 0x442, 0x430, 0x439, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x427, 0x43e, 0x439, 0x431, 0x430,
0x43b, 0x441, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x420, 0x43e, 0x436, 0x434, 0x435, 0x441, 0x442,
0x432, 0x43e, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x43d, 0x44b, 0x4a3, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x4a3, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431,
0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a,
0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x43a, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x4a3, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x43a, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x4a3, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434,
0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x4a3, 0x20, 0x436, 0x430, 0x440, 0x442, 0x44b,
0x43b, 0x430, 0x439, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x414, 0x435, 0x439, 0x432, 0x438, 0x441,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x414, 0x44e, 0x43c, 0x43e, 0x43d,
0x2d, 0x434, 0x2019, 0x42e, 0x440, 0x432, 0x438, 0x43b, 0x44c, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x422, 0x438,
0x43c, 0x43e, 0x440, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430,
0x441, 0x445, 0x430, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x441, 0x445, 0x430, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x441,
0x445, 0x430, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x42d, 0x43a, 0x432,
0x430, 0x434, 0x43e, 0x440, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e,
0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x440, 0x442, 0x430,
0x43b, 0x44b, 0x49b, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x440, 0x442, 0x430, 0x43b, 0x44b, 0x49b, 0x20,
0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441,
0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x415, 0x443, 0x440, 0x43e,
0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b,
0x441, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x49a, 0x438, 0x44b,
0x440, 0x20, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x415, 0x443, 0x440, 0x43e,
0x43f, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442,
0x44b, 0x441, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x415, 0x443,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430,
0x442, 0x44b, 0x441, 0x20, 0x415, 0x443, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424,
0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424,
0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x434, 0x430, 0x440, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x43e,
0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434,
0x430, 0x440, 0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x434, 0x436, 0x438, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x434, 0x436, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x424, 0x438, 0x434, 0x436, 0x438, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x440, 0x430,
0x43d, 0x446, 0x443, 0x437, 0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x441,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x440, 0x430, 0x43d,
0x446, 0x438, 0x44f, 0x43d, 0x44b, 0x4a3, 0x20, 0x43e, 0x4a3, 0x442, 0x4af, 0x441,
0x442, 0x456, 0x43a, 0x20, 0x430, 0x439, 0x43c, 0x430, 0x493, 0x44b, 0x20, 0x436,
0x4d9, 0x43d, 0x435, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438,
0x43a, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x43b,
0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x438,
0x43b, 0x431, 0x435, 0x440, 0x442, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x43d, 0x44b, 0x4a3, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428,
0x44b, 0x493, 0x44b, 0x441, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x440, 0x441, 0x44b, 0x20,
0x448, 0x44b, 0x493, 0x430, 0x43d, 0x430, 0x493, 0x44b, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x413, 0x430, 0x439, 0x430, 0x43d, 0x430, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439, 0x20, 0x436, 0x4d9, 0x43d,
0x435, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413,
0x430, 0x432, 0x430, 0x439, 0x20, 0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x410, 0x43b,
0x435, 0x443, 0x442, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439, 0x20,
0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x430,
0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x43e, 0x43d, 0x43a,
0x43e, 0x43d, 0x433, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413, 0x43e,
0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x413,
0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x425, 0x43e, 0x432, 0x434, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4ae, 0x43d, 0x434,
0x456, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4ae,
0x43d, 0x434, 0x456, 0x20, 0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4ae, 0x43d, 0x434, 0x456, 0x49b, 0x44b, 0x442,
0x430, 0x439, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x440, 0x442,
0x430, 0x43b, 0x44b, 0x49b, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437,
0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493,
0x44b, 0x441, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441,
0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x440,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b,
0x44c, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x437, 0x440, 0x430,
0x438, 0x43b, 0x44c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x418, 0x437, 0x440,
0x430, 0x438, 0x43b, 0x44c, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x44f,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x43f, 0x43e, 0x43d,
0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x43f, 0x43e,
0x43d, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x428, 0x44b, 0x493, 0x44b, 0x441, 0x20, 0x49a, 0x430,
0x437, 0x430, 0x49b, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x441, 0x20, 0x49a, 0x430, 0x437, 0x430,
0x49b, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a,
0x43e, 0x440, 0x435, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x443, 0x441, 0x430, 0x438, 0x435, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x44f, 0x440, 0x441, 0x43a, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41a,
0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441,
0x43a, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x49a, 0x44b, 0x440, 0x493, 0x44b, 0x437, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41b, 0x430, 0x439, 0x43d, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425,
0x430, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41b, 0x43e, 0x440, 0x434,
0x2d, 0x425, 0x430, 0x443, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434,
0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x438, 0x44f,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x43b, 0x44c, 0x434,
0x438, 0x432, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x440, 0x43a, 0x438, 0x437,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a,
0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x432, 0x440,
0x438, 0x43a, 0x438, 0x439, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a,
0x430, 0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x49b, 0x20, 0x43c, 0x4b1, 0x445, 0x438,
0x442, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x435, 0x43a, 0x441,
0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x422, 0x44b, 0x43d, 0x44b, 0x49b, 0x20, 0x43c, 0x4b1, 0x445,
0x438, 0x442, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x435, 0x43a,
0x441, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x422,
0x44b, 0x43d, 0x44b, 0x49b, 0x20, 0x43c, 0x4b1, 0x445, 0x438, 0x442, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4b0, 0x43b, 0x430, 0x43d, 0x431, 0x430, 0x442,
0x44b, 0x440, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4b0, 0x43b, 0x430,
0x43d, 0x431, 0x430, 0x442, 0x44b, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x4b0, 0x43b, 0x430, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x440, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x4d9,
0x441, 0x43a, 0x435, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c,
0x4d9, 0x441, 0x43a, 0x435, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c,
0x4d9, 0x441, 0x43a, 0x435, 0x443, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c, 0x430,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x430, 0x443, 0x440, 0x443,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x435, 0x43f, 0x430, 0x43b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x4a3, 0x430, 0x20,
0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x4a3, 0x430, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x416, 0x430, 0x4a3, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d,
0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x416, 0x430, 0x4a3, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416,
0x430, 0x4a3, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x4a3, 0x430, 0x20, 0x417,
0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x44c, 0x44e, 0x444,
0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b,
0x435, 0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x44c, 0x44e,
0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x438,
0x443, 0x44d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x44b, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434,
0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443,
0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x456, 0x431, 0x456, 0x440, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x456, 0x431,
0x456, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41d, 0x43e, 0x432, 0x43e,
0x441, 0x456, 0x431, 0x456, 0x440, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x43c, 0x431, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41e, 0x43c, 0x431, 0x44b, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41e, 0x43c, 0x431, 0x44b, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x4d9, 0x43a, 0x456,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f,
0x4d9, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41f, 0x4d9, 0x43a, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430,
0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430,
0x43b, 0x430, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430,
0x43f, 0x443, 0x430, 0x20, 0x2013, 0x20, 0x416, 0x430, 0x4a3, 0x430, 0x20, 0x413,
0x432, 0x438, 0x43d, 0x435, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430,
0x439, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x41f,
0x435, 0x440, 0x443, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x436, 0x430, 0x437,
0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x424, 0x435, 0x43d,
0x438, 0x43a, 0x441, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x435, 0x43d, 0x2d, 0x41f,
0x44c, 0x435, 0x440, 0x20, 0x436, 0x4d9, 0x43d, 0x435, 0x20, 0x41c, 0x438, 0x43a,
0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421,
0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x436, 0x4d9, 0x43d, 0x435,
0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x436,
0x4d9, 0x43d, 0x435, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x41f, 0x43e, 0x43d, 0x43f, 0x435, 0x438, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x41f, 0x445, 0x435, 0x43d, 0x44c, 0x44f, 0x43d, 0x20, 0x443,
0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x420, 0x435, 0x44e, 0x43d, 0x44c, 0x43e, 0x43d,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x420, 0x43e, 0x442, 0x435, 0x440,
0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x445, 0x430,
0x43b, 0x438, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430,
0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x44c, 0x20, 0x430, 0x440, 0x430, 0x43b,
0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421,
0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x41e, 0x4a3, 0x442, 0x4af, 0x441, 0x442, 0x456, 0x43a, 0x20, 0x413, 0x435, 0x43e,
0x440, 0x433, 0x438, 0x44f, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x421,
0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x421, 0x451, 0x432, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x422, 0x430, 0x438, 0x442, 0x438, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x422, 0x430, 0x439, 0x431, 0x44d, 0x439, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x422, 0x430, 0x439, 0x431, 0x44d, 0x439, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x4d9, 0x436, 0x456,
0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x440, 0x443, 0x43a, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x4af, 0x440, 0x456, 0x43a, 0x43c,
0x435, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x422, 0x4af, 0x440, 0x456, 0x43a, 0x43c, 0x435, 0x43d, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x4af, 0x440, 0x456, 0x43a,
0x43c, 0x435, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x422, 0x443, 0x432, 0x430,
0x43b, 0x443, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x439, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x423,
0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x4e8, 0x437, 0x431, 0x435,
0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x4e8, 0x437, 0x431, 0x435, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x4e8, 0x437, 0x431, 0x435, 0x43a, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20,
0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x430, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x43a, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434,
0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e,
0x433, 0x440, 0x430, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x43e,
0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x436, 0x430, 0x437, 0x493,
0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x412, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x423, 0x44d, 0x439,
0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b,
0x442, 0x44b, 0x423, 0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x20, 0x436, 0x4d9, 0x43d,
0x435, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x20, 0x443, 0x430, 0x49b,
0x44b, 0x442, 0x44b, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430,
0x49b, 0x44b, 0x442, 0x44b, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x436,
0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x415,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20,
0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x442, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433,
0x20, 0x436, 0x430, 0x437, 0x493, 0x44b, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442,
0x44b, 0x42e, 0x43a, 0x43e, 0x43d, 0x20, 0x443, 0x430, 0x49b, 0x44b, 0x442, 0x44b,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0, 0x17d2,
0x179c, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1793, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0, 0x17d2,
0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0, 0x17d2,
0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be,
0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0,
0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x178f,
0x17d2, 0x1794, 0x17bc, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b,
0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x17a0, 0x17d2, 0x179c, 0x17d2, 0x179a, 0x17b7, 0x1780, 0x200b, 0x200b, 0x200b, 0x1781, 0x17b6,
0x1784, 0x179b, 0x17b7, 0x1785, 0x200b, 0x200b, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c,
0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a1, 0x17b6, 0x179f, 0x17d2, 0x1780, 0x17b6, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a1, 0x17b6, 0x179f, 0x17d2, 0x1780, 0x17b6, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3,
0x200b, 0x1793, 0x17c5, 0x200b, 0x200b, 0x17a2, 0x17b6, 0x17a1, 0x17b6, 0x179f, 0x17d2, 0x1780,
0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1798,
0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17bc, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f,
0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17b6, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17bc, 0x1793, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2,
0x179f, 0x17bc, 0x1793, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f,
0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1791, 0x17d2,
0x179c, 0x17b8, 0x1794, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1797, 0x17b6, 0x1782, 0x1780, 0x178e, 0x17d2,
0x178f, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x200b, 0x1791, 0x17d2, 0x179c, 0x17b8,
0x1794, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6, 0x1784,
0x200b, 0x1787, 0x17be, 0x1784, 0x1797, 0x17b6, 0x1782, 0x1780, 0x178e, 0x17d2, 0x178f, 0x17b6,
0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1796, 0x17c1, 0x179b, 0x1790, 0x17d2,
0x1784, 0x17c3, 0x1793, 0x17c5, 0x200b, 0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x17a2, 0x17b6,
0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be,
0x1784, 0x1797, 0x17b6, 0x1782, 0x1780, 0x178e, 0x17d2, 0x178f, 0x17b6, 0x179b, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x1793, 0x17c5, 0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x17a2, 0x17b6, 0x1798,
0x179a, 0x17b7, 0x1780, 0x1781, 0x17b6, 0x1784, 0x1787, 0x17be, 0x1784, 0x1797, 0x17b6, 0x1782,
0x1781, 0x17b6, 0x1784, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x1791, 0x17d2, 0x179c, 0x17b8,
0x1794, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x1781, 0x17b6, 0x1784, 0x1787,
0x17be, 0x1784, 0x1797, 0x17b6, 0x1782, 0x1781, 0x17b6, 0x1784, 0x1780, 0x17be, 0x178f, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x1796, 0x17c1, 0x179b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x1793, 0x17c5,
0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780,
0x1781, 0x17b6, 0x1784, 0x1787, 0x17be, 0x1784, 0x1797, 0x17b6, 0x1782, 0x1781, 0x17b6, 0x1784,
0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x178f, 0x17c6,
0x1794, 0x1793, 0x17cb, 0x1797, 0x17d2, 0x1793, 0x17c6, 0x1793, 0x17c3, 0x1791, 0x17d2, 0x179c,
0x17b8, 0x1794, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b, 0x200b,
0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x178f,
0x17c6, 0x1794, 0x1793, 0x17cb, 0x1797, 0x17d2, 0x1793, 0x17c6, 0x1793, 0x17c3, 0x1791, 0x17d2,
0x179c, 0x17b8, 0x1794, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b,
0x200b, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x1793, 0x17c5, 0x178f,
0x17c6, 0x1794, 0x1793, 0x17cb, 0x1797, 0x17d2, 0x1793, 0x17c6, 0x1793, 0x17c3, 0x1791, 0x17d2,
0x179c, 0x17b8, 0x1794, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x200b,
0x200b, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1787, 0x17be, 0x1784, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x1793, 0x17c5, 0x1794, 0x17c9, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c,
0x17b7, 0x1780, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x179f, 0x17d2, 0x178a, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x1794,
0x17c9, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x17a2, 0x17b6,
0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1796, 0x17c1, 0x179b,
0x1790, 0x17d2, 0x1784, 0x17c3, 0x1793, 0x17c5, 0x1794, 0x17c9, 0x17b6, 0x179f, 0x17ca, 0x17b8,
0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x179a, 0x17b7, 0x1780,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1794, 0x17d2,
0x1799, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1794, 0x17d2, 0x1799, 0x17b6,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784,
0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1794, 0x17d2, 0x1799, 0x17b6, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17b6,
0x1794, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17b6,
0x1794, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790,
0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179a, 0x17c9, 0x17b6,
0x1794, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x1784,
0x17cb, 0x1791, 0x17b8, 0x1793, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2,
0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1797, 0x17b6, 0x1782, 0x1781,
0x17b6, 0x1784, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x17a0,
0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1797, 0x17b6, 0x1782, 0x1781, 0x17b6,
0x1784, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1791, 0x17b8, 0x1793, 0x1797, 0x17b6,
0x1782, 0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7, 0x1785, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc,
0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x1793, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178a, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17b6, 0x1798, 0x17c1, 0x1793, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17b6, 0x1798, 0x17c1, 0x1793, 0x17b8, 0x1793, 0x17b6, 0x179a, 0x178a,
0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x178f, 0x17d2, 0x179b, 0x1784, 0x17cb, 0x1791,
0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x178f, 0x17d2, 0x179b,
0x1784, 0x17cb, 0x1791, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1,
0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6,
0x178f, 0x17d2, 0x179b, 0x1784, 0x17cb, 0x1791, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b,
0x17b8, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b,
0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796,
0x17c1, 0x179b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x200b, 0x200b, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b,
0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x200b, 0x200b, 0x1797, 0x17b6, 0x1782, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b,
0x179b, 0x17b7, 0x1785, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f,
0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1797, 0x17b6, 0x1782, 0x200b, 0x1781, 0x17b6, 0x1784,
0x200b, 0x179b, 0x17b7, 0x1785, 0x200b, 0x1793, 0x17c3, 0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2,
0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6,
0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2,
0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x200b, 0x1797, 0x17b6, 0x1782, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x200b, 0x1793, 0x17c3, 0x200b, 0x17a2, 0x17bc,
0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1780, 0x178e, 0x17d2,
0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc,
0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1781, 0x17b6, 0x1784,
0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b,
0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x179f,
0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b,
0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b,
0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b,
0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17d2, 0x179a, 0x17b6, 0x179b, 0x17b8, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179f, 0x17ca, 0x17c2, 0x1794, 0x17c3, 0x17a0, 0x17d2, 0x179f,
0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179f, 0x17ca, 0x17c2,
0x1794, 0x17c3, 0x17a0, 0x17d2, 0x179f, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17b6, 0x179f, 0x17ca, 0x17c2, 0x1794, 0x17c3, 0x17a0,
0x17d2, 0x179f, 0x1784, 0x17cb, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780,
0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17c1, 0x17a0, 0x17d2, 0x179f, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17c1,
0x17a0, 0x17d2, 0x179f, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17c1, 0x17a0, 0x17d2, 0x179f, 0x179f, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc,
0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1794, 0x1784, 0x17cb, 0x1780, 0x17d2, 0x179b, 0x17b6, 0x178a, 0x17c2, 0x179f,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178a, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x1784, 0x17cb, 0x1780, 0x17d2, 0x179b, 0x17b6,
0x178a, 0x17c2, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1794, 0x1784, 0x17cb, 0x1780, 0x17d2, 0x179b, 0x17b6, 0x178a, 0x17c2, 0x179f, 0x1793, 0x17b6,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x1793, 0x17c5, 0x1794, 0x17ca, 0x17bc, 0x178f, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17bc, 0x179b, 0x17b8, 0x179c, 0x17b8, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17d2, 0x179a, 0x17b6, 0x179f,
0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1799, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1794, 0x17d2, 0x179a, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1799, 0x17c9, 0x17b6,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17d2, 0x179a, 0x17b6,
0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1799, 0x17c9, 0x17b6, 0x1793, 0x17b6, 0x200b, 0x200b,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x1793, 0x17c5, 0x1796, 0x17d2, 0x179a, 0x17bb, 0x1799, 0x178e, 0x17c1, 0x178a, 0x17b6,
0x179a, 0x17bc, 0x179f, 0x17b6, 0x17a1, 0x17b9, 0x1798, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x1794, 0x17cb, 0x179c, 0x17c2, 0x179a, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x1794, 0x17cb, 0x179c, 0x17c2, 0x179a, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x1794, 0x17cb, 0x179c,
0x17c2, 0x179a, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a,
0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b6, 0x17c6, 0x1798, 0x17c9, 0x17bc, 0x179a,
0x17c9, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b6,
0x1790, 0x17b6, 0x17c6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b6, 0x1790, 0x17b6,
0x17c6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2,
0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b6, 0x1790, 0x17b6, 0x17c6, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5,
0x179f, 0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x179f,
0x17ca, 0x17b8, 0x179b, 0x17b8, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2,
0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b7,
0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1785, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1786, 0x17bc, 0x1794, 0x17b6, 0x179b, 0x17cb, 0x179f, 0x17b6, 0x1793, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1786, 0x17bc, 0x1794, 0x17b6, 0x179b, 0x17cb, 0x179f, 0x17b6, 0x1793,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1786, 0x17bc, 0x1794, 0x17b6,
0x179b, 0x17cb, 0x179f, 0x17b6, 0x1793, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b,
0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x179f, 0x17d2, 0x1798,
0x17b6, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17d2,
0x179a, 0x1787, 0x17bb, 0x17c6, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1780, 0x17bc, 0x1780, 0x17bc,
0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x17a1,
0x17bb, 0x17c6, 0x1794, 0x17ca, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc,
0x17a1, 0x17bb, 0x17c6, 0x1794, 0x17ca, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1780, 0x17bc, 0x17a1, 0x17bb, 0x17c6, 0x1794, 0x17ca, 0x17b8, 0x1793, 0x17b6,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb, 0x17c6, 0x200b,
0x1780, 0x17c4, 0x17c7, 0x200b, 0x1781, 0x17bc, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb, 0x17c6, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1781, 0x17bc,
0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x1794, 0x17d2, 0x179a, 0x1787,
0x17bb, 0x17c6, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1781, 0x17bc, 0x1780, 0x1793, 0x17b6,
0x1796, 0x17b6, 0x1780, 0x17cb, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x200b, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1782, 0x17bb, 0x1799, 0x1794, 0x17b6, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1782, 0x17bb, 0x1799, 0x1794, 0x17b6, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1782, 0x17bb, 0x1799, 0x1794, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x178a, 0x17b6, 0x179c, 0x17b8, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x178c, 0x17bb, 0x1799, 0x1798, 0x17c9, 0x17bb, 0x1784, 0x178a,
0x17ba, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x179c, 0x17b8, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x200b, 0x1791, 0x17b8, 0x1798, 0x17d0, 0x179a, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5,
0x1780, 0x17c4, 0x17c7, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x1791, 0x17d0, 0x179a, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793,
0x17c5, 0x1780, 0x17c4, 0x17c7, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x1791, 0x17d0, 0x179a,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1780, 0x17c4, 0x17c7, 0x17a2, 0x17ca, 0x17b8,
0x179f, 0x17d2, 0x1791, 0x17d0, 0x179a, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780,
0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17c1, 0x1780, 0x17d2, 0x179c, 0x17b6, 0x1791, 0x17d0, 0x179a, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x1780,
0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba,
0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb,
0x1794, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b, 0x1793, 0x17b6, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x200b,
0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x200b, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc,
0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1785, 0x17bb, 0x1784, 0x1794, 0x17bc, 0x1796, 0x17cc, 0x17b6, 0x1793, 0x17c3,
0x1791, 0x17d2, 0x179c, 0x17b8, 0x1794, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794,
0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a,
0x17c9, 0x17bb, 0x1794, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ba, 0x179a, 0x17c9, 0x17bb, 0x1794, 0x200b, 0x1781, 0x17b6, 0x1784,
0x200b, 0x179b, 0x17b7, 0x1785, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780,
0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb, 0x17c6, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x17a0, 0x17d2,
0x179c, 0x1780, 0x17cb, 0x17a1, 0x17c2, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f,
0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x1794, 0x17d2,
0x179a, 0x1787, 0x17bb, 0x17c6, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x17a0, 0x17d2, 0x179c,
0x1780, 0x17cb, 0x17a1, 0x17c2, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793,
0x17c5, 0x1794, 0x17d2, 0x179a, 0x1787, 0x17bb, 0x17c6, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b,
0x17a0, 0x17d2, 0x179c, 0x1780, 0x17cb, 0x17a1, 0x17c2, 0x1793, 0x1793, 0x17b6, 0x179a, 0x178a,
0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x1787, 0x17b8, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x1787, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x1787, 0x17b8, 0x1793,
0x17b6, 0x200b, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x1782, 0x17b8,
0x1799, 0x17c9, 0x17b6, 0x1793, 0x200b, 0x1794, 0x17b6, 0x179a, 0x17b6, 0x17c6, 0x1784, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1794, 0x17b6, 0x179a, 0x17b6, 0x17c6, 0x1784, 0x1781,
0x17b6, 0x1784, 0x178f, 0x17d2, 0x1794, 0x17bc, 0x1784, 0x20, 0x1793, 0x17b7, 0x1784, 0x1793,
0x17c5, 0x17a2, 0x1784, 0x17cb, 0x178f, 0x17b6, 0x17c6, 0x1784, 0x1791, 0x17b7, 0x1780, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17b6, 0x17a1, 0x17b6, 0x1794,
0x17c9, 0x17b6, 0x1780, 0x17bc, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1780, 0x17b6, 0x17c6, 0x1794, 0x17c0, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179f, 0x1780, 0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17b8, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179f, 0x1780, 0x17a0, 0x17d2, 0x179f, 0x17ca,
0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179f,
0x1780, 0x17a0, 0x17d2, 0x179f, 0x17ca, 0x17b8, 0x1793, 0x17b6, 0x200b, 0x200b, 0x179a, 0x178a,
0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1780, 0x17b8, 0x1794, 0x17ca, 0x17ba,
0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793,
0x179c, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b, 0x17c2, 0x1793, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5,
0x200b, 0x200b, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b, 0x17c2, 0x1793,
0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b,
0x17c2, 0x1793, 0x1781, 0x17b6, 0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1793, 0x17b6, 0x200b,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b, 0x17c2,
0x1793, 0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f,
0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x1782,
0x17d2, 0x179a, 0x17b8, 0x1793, 0x179b, 0x17c2, 0x1793, 0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7,
0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x1782, 0x17d2, 0x179a,
0x17b8, 0x1793, 0x179b, 0x17c2, 0x1793, 0x1781, 0x17b6, 0x1784, 0x179b, 0x17b7, 0x1785, 0x1793,
0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1780, 0x17b6, 0x179b, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a0, 0x17b6, 0x179c, 0x17c3, 0x2d, 0x17a2, 0x17b6, 0x179b, 0x17cb, 0x178a,
0x17d2, 0x1799, 0x17bc, 0x179f, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f,
0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a0, 0x17b6, 0x179c, 0x17c3, 0x2d, 0x17a2, 0x17b6, 0x179b, 0x17cb, 0x178a, 0x17d2, 0x1799,
0x17bc, 0x179f, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b,
0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17b6, 0x179c,
0x17c3, 0x2d, 0x17a2, 0x17b6, 0x179b, 0x17cb, 0x178a, 0x17d2, 0x1799, 0x17bc, 0x179f, 0x17b7,
0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17bb, 0x1784,
0x1780, 0x17bb, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17bb, 0x1784, 0x1780,
0x17bb, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17bb, 0x1784,
0x1780, 0x17bb, 0x1784, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780,
0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a0, 0x17bc, 0x179c, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x17a0, 0x17bc, 0x179c, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17bc, 0x179c, 0x1793, 0x17b6, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5,
0x200b, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1798, 0x17a0, 0x17b6, 0x179f, 0x1798, 0x17bb, 0x1791, 0x17d2, 0x179a, 0x200b,
0x17a5, 0x178e, 0x17d2, 0x178c, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17bc, 0x1785, 0x17b7, 0x1793, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a5, 0x178e, 0x17d2, 0x178c, 0x17bc, 0x178e, 0x17c1,
0x179f, 0x17ca, 0x17b8, 0x200b, 0x200b, 0x200b, 0x1780, 0x178e, 0x17d2, 0x178a, 0x17b6, 0x179b,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a5, 0x178e, 0x17d2, 0x178c,
0x17bc, 0x178e, 0x17c1, 0x179f, 0x17ca, 0x17b8, 0x200b, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b,
0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a5,
0x178e, 0x17d2, 0x178c, 0x17bc, 0x178e, 0x17c1, 0x179f, 0x17ca, 0x17b8, 0x200b, 0x200b, 0x1781,
0x17b6, 0x1784, 0x200b, 0x179b, 0x17b7, 0x1785, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179a, 0x17c9, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x1793, 0x17c5, 0x17a2, 0x17c0, 0x179a, 0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780,
0x17b7, 0x17cd, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x1793, 0x17c5, 0x17a2, 0x17c0, 0x179a, 0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2,
0x1780, 0x17b7, 0x17cd, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x17a2, 0x17c0, 0x179a,
0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1793, 0x17b6, 0x179a, 0x178a,
0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x179a, 0x17b6, 0x17a2, 0x17c2, 0x179b,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x179a, 0x17b6,
0x17a2, 0x17c2, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b,
0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b8, 0x179f,
0x17d2, 0x179a, 0x17b6, 0x17a2, 0x17c2, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1787, 0x1794, 0x17c9, 0x17bb, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1787, 0x1794, 0x17c9, 0x17bb, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1,
0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x1787, 0x1794, 0x17c9,
0x17bb, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1780, 0x17b6, 0x17a0, 0x17d2, 0x179f,
0x17b6, 0x1780, 0x17cb, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x200b, 0x200b, 0x1781, 0x17b6,
0x1784, 0x200b, 0x1780, 0x17be, 0x178f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1780, 0x17b6, 0x17a0, 0x17d2, 0x179f, 0x17b6, 0x1780, 0x17cb, 0x179f, 0x17d2, 0x1790,
0x17b6, 0x1793, 0x200b, 0x1781, 0x17b6, 0x1784, 0x200b, 0x200b, 0x200b, 0x179b, 0x17b7, 0x1785,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x179a, 0x17c9,
0x17c1, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x179a, 0x17c9, 0x17c1, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x179a, 0x17c9, 0x17c1, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17bc, 0x179f, 0x17d2, 0x179a, 0x17c3, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x178e,
0x17bc, 0x1799, 0x17c9, 0x17b6, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17d2,
0x179a, 0x17b6, 0x178e, 0x17bc, 0x1799, 0x17c9, 0x17b6, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x178e, 0x17bc, 0x1799, 0x17c9,
0x17b6, 0x179f, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2,
0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1780, 0x17c0, 0x17a0, 0x17d2,
0x179f, 0x17ca, 0x17b8, 0x179f, 0x17ca, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x17a1,
0x17b6, 0x1789, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a1, 0x178f,
0x17a0, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a1, 0x178f, 0x200b, 0x17a0, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784,
0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a1, 0x178f, 0x17a0, 0x17c5, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1780, 0x17b6,
0x178a, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x17a0,
0x17d2, 0x1780, 0x17b6, 0x178a, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1780, 0x17b6, 0x178a, 0x17b6, 0x1793,
0x200b, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x17a1,
0x17c1, 0x179f, 0x17ca, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1798, 0x17c9, 0x17b6, 0x179b, 0x17cb, 0x178c, 0x17b8, 0x179c, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x1798, 0x17c9, 0x17b6, 0x1782,
0x17ba, 0x179f, 0x17b6, 0x179f, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1798, 0x17c9, 0x17b6, 0x179f, 0x17b6, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17bc, 0x179a, 0x17b8, 0x179f, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1798, 0x17c9, 0x17bc, 0x179a, 0x17b8, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1793,
0x17c5, 0x200b, 0x1798, 0x17c9, 0x17bc, 0x179a, 0x17b8, 0x179f, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c5, 0x200b, 0x179f, 0x17b6, 0x1793, 0x17cb,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x179f,
0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x200b, 0x1798, 0x17c9, 0x17b7, 0x1780,
0x179f, 0x17ca, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x179f,
0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x200b, 0x1798, 0x17c9, 0x17b7, 0x1780,
0x179f, 0x17ca, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b,
0x200b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6,
0x179f, 0x17ca, 0x17b8, 0x17a0, 0x17d2, 0x179c, 0x17b7, 0x1780, 0x200b, 0x1798, 0x17c9, 0x17b7,
0x1780, 0x179f, 0x17ca, 0x17b7, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x17a2, 0x17ca, 0x17bc, 0x17a1, 0x17b6, 0x1793, 0x200b, 0x1794, 0x17b6, 0x1791, 0x17bc,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bc, 0x17a1, 0x17b6, 0x1793, 0x200b,
0x1794, 0x17b6, 0x1791, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17ca, 0x17bc, 0x17a1, 0x17b6, 0x1793, 0x200b, 0x1794, 0x17b6, 0x1791, 0x17bc, 0x1793, 0x17b6,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17bc, 0x179f, 0x17d2, 0x1782,
0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17bc, 0x179f, 0x17d2, 0x1782, 0x17bc,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17bc, 0x179f, 0x17d2,
0x1782, 0x17bc, 0x200b, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2,
0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1798, 0x17b8,
0x1799, 0x17c9, 0x17b6, 0x1793, 0x17cb, 0x1798, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x178e, 0x17bc, 0x179a, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x1793, 0x17c5, 0x1793, 0x17c1, 0x1794, 0x17c9, 0x17b6, 0x179b, 0x17cb, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17c2, 0x179b, 0x1780, 0x17b6,
0x17a1, 0x17c1, 0x178a, 0x17bc, 0x1793, 0x17c0, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x179f, 0x17d2,
0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x178e, 0x17bc, 0x179c,
0x17c9, 0x17c2, 0x179b, 0x1780, 0x17b6, 0x17a1, 0x17c1, 0x178a, 0x17bc, 0x1793, 0x17c0, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17c2, 0x179b,
0x1780, 0x17b6, 0x17a1, 0x17c1, 0x178a, 0x17bc, 0x1793, 0x17c0, 0x1793, 0x17b6, 0x179a, 0x178a,
0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1793, 0x17bc, 0x179c, 0x17c2, 0x179b, 0x179f, 0x17c1, 0x17a1, 0x1784, 0x17cb,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1793, 0x17bc, 0x179c, 0x17c2, 0x179b, 0x179f, 0x17c1,
0x17a1, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b,
0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1793, 0x17bc, 0x179c, 0x17c2,
0x179b, 0x179f, 0x17c1, 0x17a1, 0x1784, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b,
0x1793, 0x17c5, 0x1789, 0x17bc, 0x179c, 0x17a0, 0x17d2, 0x179c, 0x17c4, 0x1793, 0x17a1, 0x17c2,
0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1789, 0x17bc, 0x179c, 0x17a0,
0x17d2, 0x179c, 0x17c4, 0x1793, 0x17a1, 0x17c2, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1796, 0x17c1, 0x179b, 0x1790, 0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1789,
0x17bc, 0x179c, 0x17a0, 0x17d2, 0x179c, 0x17c4, 0x1793, 0x17a1, 0x17c2, 0x1793, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x1793, 0x17c5, 0x200b, 0x1793, 0x17b8, 0x179c, 0x17c9, 0x17c1, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x178e, 0x17d0,
0x179a, 0x17a0, 0x17d2, 0x179c, 0x1780, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f,
0x17d2, 0x178a, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178e,
0x17d0, 0x179a, 0x17a0, 0x17d2, 0x179c, 0x1780, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793,
0x17c5, 0x200b, 0x178e, 0x17d0, 0x179a, 0x17a0, 0x17d2, 0x179c, 0x1780, 0x17cb, 0x1793, 0x17b6,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x179c, 0x17ca, 0x17c2, 0x178e, 0x17b6,
0x1793, 0x17cb, 0x178a, 0x17bc, 0x200b, 0x178a, 0x17ba, 0x178e, 0x17bc, 0x179a, 0x17c9, 0x17bc,
0x1789, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x17a0, 0x17d2, 0x179c, 0x17ca, 0x17c2, 0x178e,
0x17b6, 0x1793, 0x17cb, 0x178a, 0x17bc, 0x200b, 0x178a, 0x17ba, 0x178e, 0x17bc, 0x179a, 0x17c9,
0x17bc, 0x1789, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x17a0,
0x17d2, 0x179c, 0x17ca, 0x17c2, 0x178e, 0x17b6, 0x1793, 0x17cb, 0x178a, 0x17bc, 0x200b, 0x178a,
0x17ba, 0x178e, 0x17bc, 0x179a, 0x17c9, 0x17bc, 0x1789, 0x17c9, 0x17b6, 0x1793, 0x17b6, 0x179a,
0x178a, 0x17bc, 0x179c, 0x1780, 0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17ca, 0x17b8, 0x1794,
0x17ca, 0x17b8, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178e, 0x17bc, 0x179c, 0x17c9,
0x17bc, 0x179f, 0x17ca, 0x17b8, 0x1794, 0x17ca, 0x17b8, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x178e, 0x17bc, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17ca, 0x17b8,
0x1794, 0x17ca, 0x17b8, 0x1780, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b,
0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17bc, 0x1798, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784,
0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x1798, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17bc, 0x1798, 0x1793, 0x17b6,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x1782, 0x17b8, 0x179f,
0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178a,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6,
0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793,
0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793,
0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1795, 0x17b6, 0x17a1,
0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x1794, 0x17c9, 0x17b6, 0x1796,
0x17bc, 0x17a2, 0x17b6, 0x179f, 0x17ca, 0x17b8, 0x20, 0x1793, 0x17bc, 0x179c, 0x17c2, 0x179b,
0x17a0, 0x17d2, 0x1782, 0x17b8, 0x178e, 0x17c1, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17b6,
0x1799, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17b6,
0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1799, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x1794, 0x17c9, 0x17b6, 0x179a, 0x17c9, 0x17b6, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1799, 0x1793,
0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17c1, 0x179a, 0x17bc,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17c1, 0x179a, 0x17bc, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1794, 0x17c9, 0x17c1, 0x179a, 0x17bc,
0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x179b,
0x17b8, 0x1796, 0x17b8, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c,
0x17b8, 0x179b, 0x17b8, 0x1796, 0x17b8, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a0, 0x17d2, 0x179c, 0x17b8, 0x179b, 0x17b8, 0x1796, 0x17b8, 0x1793,
0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b,
0x1795, 0x17bc, 0x1793, 0x17b8, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x200b,
0x1793, 0x17c5, 0x179f, 0x1784, 0x17cb, 0x200b, 0x1796, 0x17d2, 0x1799, 0x17c2, 0x179a, 0x20,
0x1793, 0x17b7, 0x1784, 0x200b, 0x1798, 0x17b8, 0x1782, 0x17bb, 0x1799, 0x17a1, 0x17bb, 0x1784,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6,
0x179a, 0x200b, 0x1793, 0x17c5, 0x179f, 0x1784, 0x17cb, 0x200b, 0x1796, 0x17d2, 0x1799, 0x17c2,
0x179a, 0x20, 0x1793, 0x17b7, 0x1784, 0x200b, 0x1798, 0x17b8, 0x1782, 0x17bb, 0x1799, 0x17a1,
0x17bb, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790,
0x17d2, 0x1784, 0x17c3, 0x200b, 0x1793, 0x17c5, 0x179f, 0x1784, 0x17cb, 0x200b, 0x1796, 0x17d2,
0x1799, 0x17c2, 0x179a, 0x20, 0x1793, 0x17b7, 0x1784, 0x200b, 0x1798, 0x17b8, 0x1782, 0x17bb,
0x1799, 0x17a1, 0x17bb, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x1797, 0x17b8, 0x1783, 0x17ba, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x1794, 0x17c9, 0x17bc, 0x178e, 0x17b6, 0x1794, 0x17cb, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x1793, 0x17c5, 0x1796, 0x17d2, 0x1799, 0x17bb, 0x1784, 0x1799, 0x17c9, 0x17b6, 0x1784, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x179a, 0x17c1, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x17c9,
0x17bb, 0x1784, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179a, 0x17c9,
0x17bc, 0x1792, 0x17ba, 0x179a, 0x17c9, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x179f, 0x17b6, 0x1780, 0x17cb, 0x1781, 0x17b6, 0x179b, 0x17b8, 0x1793, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a,
0x200b, 0x1793, 0x17c5, 0x200b, 0x179f, 0x17b6, 0x1780, 0x17cb, 0x1781, 0x17b6, 0x179b, 0x17b8,
0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179f, 0x17b6, 0x1780,
0x17cb, 0x1781, 0x17b6, 0x179b, 0x17b8, 0x1793, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc,
0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x179f, 0x17b6, 0x1798, 0x17d0, 0x179a, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x1793, 0x17c5, 0x200b, 0x179f,
0x17b6, 0x1798, 0x17d0, 0x179a, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x179f, 0x17b6, 0x1798, 0x17d0, 0x179a, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x1780,
0x17d2, 0x178f, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179f,
0x17b8, 0x179f, 0x17d2, 0x17a0, 0x17c2, 0x179b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793,
0x17c5, 0x200b, 0x179f, 0x17b7, 0x1784, 0x17d2, 0x17a0, 0x1794, 0x17bc, 0x179a, 0x17b8, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x179f,
0x17bc, 0x17a1, 0x17bc, 0x1798, 0x17c9, 0x17bb, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x200b, 0x17a0, 0x17d2, 0x179f, 0x1780, 0x17a0,
0x17d2, 0x179f, 0x17ca, 0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b,
0x179f, 0x17bc, 0x179a, 0x17b8, 0x178e, 0x17b6, 0x1798, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x1793, 0x17c5, 0x200b, 0x179f, 0x17ca, 0x17b8, 0x17a2, 0x17bc, 0x179c, 0x17c9, 0x17b6, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17b6, 0x17a0, 0x17b7, 0x1791,
0x17b8, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17c3, 0x1794,
0x17c9, 0x17b7, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17c3, 0x1794, 0x17c9, 0x17b7,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1796, 0x17c1, 0x179b, 0x200b, 0x1790, 0x17d2, 0x1784,
0x17c3, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17c3, 0x1794, 0x17c9, 0x17b7, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x1793, 0x17c5, 0x178f, 0x17b6, 0x1787, 0x17b8, 0x1782, 0x17b8, 0x179f, 0x17d2,
0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f,
0x17bc, 0x1781, 0x17c1, 0x17a1, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x178f, 0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x200b, 0x1793,
0x17c5, 0x200b, 0x178f, 0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17bb, 0x1784, 0x17a0, 0x17d2, 0x1782,
0x17b6, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1785, 0x17bc, 0x17a2, 0x17bb,
0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17bd, 0x1780,
0x1798, 0x17c9, 0x17c1, 0x1793, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x178f, 0x17bd, 0x1780, 0x1798, 0x17c9, 0x17c1, 0x1793, 0x17b8, 0x179f, 0x17d2,
0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c,
0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1793, 0x17c5, 0x200b, 0x178f, 0x17bd, 0x1780,
0x1798, 0x17c9, 0x17c1, 0x1793, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x200b, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1791, 0x17bb, 0x1799, 0x179c, 0x17c9,
0x17b6, 0x179b, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17ca, 0x17bb, 0x1799, 0x179a, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1799, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b,
0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bb, 0x1799, 0x179a, 0x17bc, 0x17a0, 0x17d2, 0x1782,
0x17b6, 0x1799, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca,
0x17bb, 0x1799, 0x179a, 0x17bc, 0x17a0, 0x17d2, 0x1782, 0x17b6, 0x1799, 0x1793, 0x17b6, 0x200b,
0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bc, 0x179f, 0x1794, 0x17c1,
0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b,
0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b,
0x17a2, 0x17ca, 0x17bc, 0x179f, 0x1794, 0x17c1, 0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6,
0x1793, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17bc,
0x179f, 0x1794, 0x17c1, 0x1782, 0x17b8, 0x179f, 0x17d2, 0x1790, 0x17b6, 0x1793, 0x1793, 0x17b6,
0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9,
0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17b6, 0x1793, 0x17bc, 0x1791,
0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb,
0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17b6, 0x1793, 0x17bc,
0x1791, 0x17bc, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9,
0x17b6, 0x1793, 0x17bc, 0x1791, 0x17bc, 0x1793, 0x17b6, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b,
0x1780, 0x17d2, 0x178a, 0x17c5, 0x200b, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x179c, 0x17c9, 0x17c1, 0x178e, 0x17c1, 0x179f, 0x17ca, 0x17bb, 0x1799, 0x17a2, 0x17c1,
0x17a1, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9,
0x17d2, 0x179b, 0x17b6, 0x178c, 0x17b8, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17bb,
0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a,
0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17d2, 0x179b, 0x17b6, 0x178c,
0x17b8, 0x179c, 0x17c9, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17d2, 0x179b, 0x17b6, 0x178c, 0x17b8,
0x179c, 0x17c9, 0x17bc, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1793, 0x17b6, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17bc, 0x17a0, 0x17d2, 0x1780, 0x17c4, 0x1780,
0x17d2, 0x179a, 0x17b6, 0x178a, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f, 0x17d2, 0x178f,
0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17bc,
0x17a0, 0x17d2, 0x1780, 0x17c4, 0x1780, 0x17d2, 0x179a, 0x17b6, 0x178a, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17bc, 0x17a0, 0x17d2, 0x1780, 0x17c4,
0x1780, 0x17d2, 0x179a, 0x17b6, 0x178a, 0x1793, 0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c,
0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5,
0x200b, 0x179c, 0x17d0, 0x179a, 0x179f, 0x17d2, 0x178f, 0x17bb, 0x1780, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1780, 0x17c4, 0x17c7, 0x179c, 0x17c1, 0x1780, 0x1798,
0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x179c, 0x17c9, 0x17b6, 0x179b, 0x17b8,
0x179f, 0x20, 0x1793, 0x17b7, 0x1784, 0x17a0, 0x17d2, 0x179c, 0x17ca, 0x17bb, 0x1791, 0x17bc,
0x178e, 0x17b6, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1799, 0x17c9,
0x17b6, 0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1798, 0x17c9, 0x17c4,
0x1784, 0x200b, 0x179f, 0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793,
0x17c5, 0x200b, 0x1799, 0x17c9, 0x17b6, 0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7,
0x17cd, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x1799, 0x17c9, 0x17b6,
0x1782, 0x17bb, 0x178f, 0x179f, 0x17d2, 0x1780, 0x17b7, 0x17cd, 0x1793, 0x17b6, 0x200b, 0x179a,
0x178a, 0x17bc, 0x179c, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5, 0x1798, 0x17c9, 0x17c4, 0x1784,
0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b7, 0x1781, 0x17b6, 0x1792, 0x17ba, 0x179a,
0x17b8, 0x1793, 0x1794, 0x17ca, 0x17d0, 0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x179f,
0x17d2, 0x178f, 0x1784, 0x17cb, 0x178a, 0x17b6, 0x179a, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2,
0x17ca, 0x17b7, 0x1781, 0x17b6, 0x1792, 0x17ba, 0x179a, 0x17b8, 0x1793, 0x1794, 0x17ca, 0x17d0,
0x1780, 0x1798, 0x17c9, 0x17c4, 0x1784, 0x200b, 0x1793, 0x17c5, 0x200b, 0x17a2, 0x17ca, 0x17b7,
0x1781, 0x17b6, 0x1792, 0x17ba, 0x179a, 0x17b8, 0x1793, 0x1794, 0x17ca, 0x17d0, 0x1780, 0x1793,
0x17b6, 0x200b, 0x179a, 0x178a, 0x17bc, 0x179c, 0x200b, 0x200b, 0x1780, 0x17d2, 0x178a, 0x17c5,
0x1798, 0x17c9, 0x17c4, 0x1784, 0x1793, 0x17c5, 0x1799, 0x17bc, 0x1781, 0x1793, 0x17cb, 0x905,
0x92b, 0x917, 0x93e, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f,
0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20,
0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x926,
0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915,
0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x906, 0x92b, 0x94d,
0x930, 0x93f, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924,
0x902, 0x924, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905,
0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933,
0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x932, 0x93e, 0x938,
0x94d, 0x915, 0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935,
0x947, 0x933, 0x905, 0x92e, 0x947, 0x91d, 0x94b, 0x928, 0x20, 0x935, 0x947, 0x933,
0x905, 0x92e, 0x947, 0x91d, 0x94b, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x92e, 0x947, 0x91d, 0x94b,
0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d,
0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f,
0x91f, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x935,
0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902,
0x924, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x930, 0x94d, 0x935, 0x924, 0x940, 0x92f, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x930, 0x94d, 0x935, 0x924, 0x940, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x930, 0x94d, 0x935, 0x924,
0x940, 0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94d, 0x930, 0x936,
0x93e, 0x902, 0x924, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935,
0x947, 0x933, 0x905, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x905,
0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930, 0x92c,
0x940, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930, 0x92c, 0x940, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930,
0x92c, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947,
0x933, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20,
0x935, 0x947, 0x933, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928,
0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947,
0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d,
0x924, 0x902, 0x924, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f,
0x928, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x905, 0x930,
0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x906, 0x930, 0x94d, 0x92e,
0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x906, 0x930, 0x94d,
0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x906, 0x930, 0x94d, 0x92e, 0x947,
0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x20, 0x935, 0x947, 0x933, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x940, 0x915,
0x20, 0x935, 0x947, 0x933, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x940, 0x915,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x940, 0x915, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f,
0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x940, 0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d,
0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933,
0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x928, 0x20,
0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20,
0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940,
0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x938, 0x94d, 0x924,
0x902, 0x924, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940,
0x92f, 0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x938, 0x94d, 0x924,
0x902, 0x924, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947,
0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x940, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x928, 0x20, 0x909, 0x926,
0x947, 0x902, 0x924, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x940, 0x92f, 0x928, 0x20, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924,
0x902, 0x924, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940,
0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x940, 0x92f, 0x928, 0x20, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x940, 0x92f, 0x928,
0x20, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c,
0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c,
0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x905,
0x91d, 0x94b, 0x930, 0x947, 0x938, 0x905, 0x91d, 0x94b, 0x930, 0x947, 0x938, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x905, 0x91d, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x93e, 0x902, 0x917, 0x932, 0x93e,
0x926, 0x947, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x93e, 0x902, 0x917, 0x932,
0x93e, 0x926, 0x947, 0x936, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x93e, 0x902, 0x917, 0x932, 0x93e, 0x926,
0x947, 0x936, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935,
0x947, 0x933, 0x92d, 0x942, 0x924, 0x93e, 0x928, 0x92c, 0x94b, 0x932, 0x93f, 0x935,
0x94d, 0x939, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930,
0x93e, 0x91d, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92c,
0x94d, 0x930, 0x93e, 0x91d, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d,
0x930, 0x93e, 0x91d, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930, 0x941,
0x928, 0x947, 0x907, 0x20, 0x926, 0x93e, 0x930, 0x942, 0x938, 0x94d, 0x938, 0x932,
0x93e, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930,
0x94d, 0x926, 0x947, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935,
0x930, 0x94d, 0x926, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d,
0x926, 0x947, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935,
0x947, 0x933, 0x915, 0x945, 0x92e, 0x94b, 0x930, 0x93e, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x945, 0x925,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x945, 0x925, 0x92e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x945,
0x925, 0x92e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947,
0x933, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x93f, 0x932,
0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x940, 0x928, 0x20, 0x935, 0x947,
0x933, 0x91a, 0x940, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x940, 0x928, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x94b, 0x908, 0x92c, 0x93e,
0x932, 0x94d, 0x938, 0x928, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x94b, 0x908, 0x92c,
0x93e, 0x932, 0x94d, 0x938, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x94b, 0x908, 0x92c, 0x93e, 0x932,
0x94d, 0x938, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938, 0x20, 0x906,
0x92f, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x915, 0x94b,
0x938, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x915,
0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x915,
0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x932, 0x902,
0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x20, 0x935, 0x947, 0x933, 0x915, 0x942, 0x915, 0x20, 0x906, 0x92f, 0x932, 0x901,
0x921, 0x20, 0x935, 0x947, 0x933, 0x915, 0x942, 0x915, 0x20, 0x906, 0x92f, 0x932,
0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x915, 0x942, 0x915, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921,
0x20, 0x905, 0x930, 0x94d, 0x926, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x92f, 0x941, 0x92c, 0x93e, 0x20,
0x935, 0x947, 0x933, 0x915, 0x94d, 0x92f, 0x941, 0x92c, 0x93e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d,
0x92f, 0x941, 0x92c, 0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20,
0x935, 0x947, 0x933, 0x921, 0x947, 0x935, 0x94d, 0x939, 0x940, 0x938, 0x20, 0x935,
0x947, 0x933, 0x926, 0x94d, 0x92f, 0x941, 0x92e, 0x93e, 0x901, 0x20, 0x926, 0x942,
0x930, 0x935, 0x940, 0x92f, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902,
0x924, 0x20, 0x924, 0x93f, 0x92e, 0x94b, 0x930, 0x20, 0x935, 0x947, 0x933, 0x908,
0x938, 0x94d, 0x91f, 0x930, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935,
0x947, 0x933, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x906, 0x92f, 0x932, 0x901,
0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x906, 0x92f, 0x932, 0x901,
0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x907, 0x915, 0x94d, 0x935, 0x947, 0x921, 0x94b, 0x930, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f,
0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92f,
0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f,
0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947,
0x902, 0x924, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20,
0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x92f, 0x941, 0x930,
0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20,
0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x906, 0x928, 0x940, 0x915,
0x2d, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a,
0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902,
0x924, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x935,
0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x92f, 0x941, 0x930,
0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924,
0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x949, 0x915,
0x932, 0x901, 0x921, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x94d, 0x938, 0x20,
0x935, 0x947, 0x933, 0x92b, 0x949, 0x915, 0x932, 0x901, 0x921, 0x20, 0x906, 0x92f,
0x932, 0x901, 0x921, 0x94d, 0x938, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x949, 0x915, 0x932, 0x901, 0x921,
0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x94d, 0x938, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x91c, 0x940,
0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x91c, 0x940, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x91c,
0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x92b, 0x94d, 0x930, 0x947, 0x902, 0x91a, 0x20, 0x917, 0x92f, 0x93e, 0x928,
0x93e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x94d, 0x930, 0x947, 0x928, 0x94d, 0x91a,
0x20, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x906, 0x928, 0x940, 0x20,
0x905, 0x902, 0x91f, 0x93e, 0x930, 0x94d, 0x915, 0x94d, 0x91f, 0x93f, 0x915, 0x20,
0x935, 0x947, 0x933, 0x917, 0x93e, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938,
0x20, 0x935, 0x947, 0x933, 0x917, 0x93e, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930,
0x20, 0x935, 0x947, 0x933, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x917, 0x93f, 0x932,
0x94d, 0x92c, 0x930, 0x94d, 0x91f, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20,
0x935, 0x947, 0x933, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f, 0x91a, 0x20,
0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902,
0x924, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x935,
0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x940,
0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902,
0x924, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x935,
0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x917, 0x932, 0x94d,
0x92b, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x917, 0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933,
0x939, 0x935, 0x93e, 0x908, 0x2d, 0x905, 0x932, 0x947, 0x92f, 0x941, 0x936, 0x93f,
0x928, 0x20, 0x935, 0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x905, 0x932,
0x947, 0x92f, 0x941, 0x936, 0x93f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d,
0x905, 0x932, 0x947, 0x92f, 0x941, 0x936, 0x93f, 0x928, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x939, 0x93e, 0x901, 0x917, 0x20,
0x915, 0x93e, 0x901, 0x917, 0x20, 0x935, 0x947, 0x933, 0x939, 0x93e, 0x901, 0x917,
0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915,
0x93e, 0x901, 0x917, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x939, 0x94b, 0x935, 0x94d, 0x939, 0x921, 0x939, 0x94b, 0x935,
0x94d, 0x939, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x939, 0x94b, 0x935, 0x94d, 0x939, 0x921, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92d, 0x93e,
0x930, 0x924, 0x940, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x939, 0x93f, 0x902, 0x926, 0x20, 0x92e, 0x94d,
0x939, 0x93e, 0x938, 0x93e, 0x917, 0x930, 0x20, 0x935, 0x947, 0x933, 0x907, 0x902,
0x921, 0x94b, 0x91a, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d,
0x92f, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20, 0x907, 0x902,
0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933,
0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928,
0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930, 0x93e,
0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x907, 0x930, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f,
0x91f, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x91f, 0x938,
0x94d, 0x915, 0x20, 0x935, 0x947, 0x933, 0x908, 0x930, 0x94d, 0x915, 0x941, 0x91f,
0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x908, 0x930, 0x94d, 0x915, 0x941, 0x91f, 0x938, 0x94d,
0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x935, 0x947, 0x933,
0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x907, 0x91c, 0x93c, 0x930,
0x93e, 0x907, 0x932, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935,
0x947, 0x933, 0x91c, 0x92a, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x92a,
0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x91c, 0x92a, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x909, 0x926, 0x947, 0x902, 0x924, 0x20,
0x915, 0x91d, 0x93e, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947,
0x933, 0x905, 0x938, 0x94d, 0x924, 0x902, 0x924, 0x20, 0x915, 0x91d, 0x93e, 0x915,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x930,
0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x930, 0x93f, 0x92f,
0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20, 0x921, 0x947, 0x932,
0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x938, 0x930, 0x947,
0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b,
0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x935, 0x947, 0x933, 0x915,
0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f,
0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x915, 0x93f, 0x930, 0x94d, 0x917, 0x93f,
0x91c, 0x93c, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x932,
0x93e, 0x92f, 0x928, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947,
0x933, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20,
0x935, 0x947, 0x933, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935,
0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935,
0x947, 0x933, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x92e,
0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x917, 0x93e,
0x926, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x92e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x932,
0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x93e, 0x932,
0x926, 0x940, 0x935, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x93e, 0x930, 0x94d, 0x915,
0x93f, 0x938, 0x938, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x93e, 0x930, 0x94d, 0x936,
0x932, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x92e,
0x949, 0x930, 0x93f, 0x936, 0x938, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x930,
0x93f, 0x936, 0x938, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x930, 0x93f, 0x936, 0x938, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949,
0x938, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f,
0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a,
0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x947, 0x915, 0x94d, 0x938,
0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x909, 0x932,
0x93e, 0x928, 0x92c, 0x924, 0x93e, 0x930, 0x20, 0x935, 0x947, 0x933, 0x909, 0x932,
0x93e, 0x928, 0x92c, 0x924, 0x93e, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x909, 0x932, 0x93e, 0x928, 0x92c,
0x924, 0x93e, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x938, 0x94d,
0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x92e, 0x93e, 0x930, 0x20, 0x935,
0x947, 0x933, 0x928, 0x930, 0x942, 0x20, 0x935, 0x947, 0x933, 0x928, 0x947, 0x92a,
0x93e, 0x933, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x20, 0x915,
0x945, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947,
0x933, 0x928, 0x94d, 0x92f, 0x941, 0x20, 0x915, 0x945, 0x932, 0x947, 0x921, 0x94b,
0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f,
0x924, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x20, 0x915, 0x945,
0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941,
0x91d, 0x940, 0x932, 0x945, 0x928, 0x94d, 0x921, 0x20, 0x935, 0x947, 0x933, 0x928,
0x94d, 0x92f, 0x941, 0x91d, 0x940, 0x932, 0x945, 0x928, 0x94d, 0x921, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x928,
0x94d, 0x92f, 0x941, 0x91d, 0x940, 0x932, 0x945, 0x928, 0x94d, 0x921, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f,
0x941, 0x92b, 0x93e, 0x935, 0x902, 0x921, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947,
0x933, 0x928, 0x94d, 0x92f, 0x941, 0x92b, 0x93e, 0x935, 0x902, 0x921, 0x932, 0x901,
0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x940, 0x924, 0x20, 0x935,
0x947, 0x933, 0x928, 0x94d, 0x92f, 0x941, 0x92b, 0x93e, 0x935, 0x902, 0x921, 0x932,
0x901, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947,
0x933, 0x928, 0x94d, 0x92f, 0x941, 0x90f, 0x20, 0x935, 0x947, 0x933, 0x928, 0x949,
0x930, 0x92b, 0x949, 0x915, 0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935,
0x947, 0x933, 0x928, 0x949, 0x930, 0x92b, 0x949, 0x915, 0x20, 0x906, 0x92f, 0x932,
0x901, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x928, 0x949, 0x930, 0x92b, 0x949, 0x915, 0x20, 0x906, 0x92f,
0x932, 0x901, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20,
0x935, 0x947, 0x933, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x921, 0x94b, 0x20,
0x926, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20,
0x935, 0x947, 0x933, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x921, 0x94b, 0x20,
0x926, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933,
0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x921, 0x94b, 0x20, 0x926, 0x940, 0x20,
0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94b, 0x935, 0x94b,
0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94b,
0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x913,
0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x935, 0x947, 0x933, 0x913, 0x92e, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x93e,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x93e, 0x915, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x932, 0x93e, 0x90a, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x93e, 0x92a, 0x941, 0x906, 0x20, 0x928, 0x94d, 0x92f, 0x941, 0x20,
0x917, 0x93f, 0x928, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x930, 0x93e, 0x917,
0x941, 0x906, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x930, 0x93e, 0x917, 0x941, 0x906,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x930, 0x93e, 0x917, 0x941, 0x906, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x947, 0x930, 0x942, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x947, 0x930, 0x942,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93e, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933,
0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93e, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x932,
0x93f, 0x92a, 0x93e, 0x92f, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x928, 0x93f, 0x915, 0x94d, 0x938,
0x20, 0x906, 0x92f, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x938, 0x947,
0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930, 0x20, 0x906, 0x928, 0x940, 0x20,
0x92e, 0x93f, 0x915, 0x932, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x938, 0x947,
0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930, 0x20, 0x906, 0x928, 0x940, 0x20,
0x92e, 0x93f, 0x915, 0x932, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x940, 0x924, 0x20, 0x935, 0x947, 0x933, 0x938, 0x947, 0x902, 0x91f, 0x20,
0x92a, 0x93f, 0x92f, 0x930, 0x20, 0x906, 0x928, 0x940, 0x20, 0x92e, 0x93f, 0x915,
0x932, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x93f, 0x91f, 0x915, 0x945, 0x930, 0x928, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x94b, 0x928, 0x947, 0x92a, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94d,
0x92f, 0x94b, 0x902, 0x917, 0x92f, 0x93e, 0x902, 0x917, 0x20, 0x935, 0x947, 0x933,
0x930, 0x93f, 0x92f, 0x941, 0x928, 0x93f, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933,
0x930, 0x94b, 0x925, 0x947, 0x930, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x938, 0x916,
0x932, 0x93f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x938, 0x916, 0x932, 0x93f, 0x928,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x938, 0x916, 0x932, 0x93f, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e, 0x94b, 0x906, 0x20,
0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e, 0x94b, 0x906, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e,
0x94b, 0x906, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947,
0x933, 0x938, 0x947, 0x936, 0x947, 0x932, 0x94d, 0x938, 0x20, 0x935, 0x947, 0x933,
0x938, 0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x942, 0x930, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x938, 0x94b, 0x932,
0x94b, 0x92e, 0x928, 0x20, 0x906, 0x907, 0x932, 0x901, 0x921, 0x94d, 0x938, 0x20,
0x935, 0x947, 0x933, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x91c, 0x949,
0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x938, 0x941,
0x930, 0x93f, 0x928, 0x93e, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x938, 0x94d, 0x92f,
0x94b, 0x935, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x924, 0x93e, 0x939, 0x93f, 0x924,
0x940, 0x20, 0x935, 0x947, 0x933, 0x924, 0x948, 0x92a, 0x947, 0x908, 0x20, 0x935,
0x947, 0x933, 0x924, 0x948, 0x92a, 0x947, 0x908, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x924, 0x948, 0x92a, 0x947,
0x908, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x92f, 0x91f, 0x20, 0x935, 0x947, 0x933,
0x924, 0x91c, 0x940, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935,
0x947, 0x933, 0x91f, 0x94b, 0x915, 0x932, 0x93e, 0x90a, 0x20, 0x935, 0x947, 0x933,
0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x91f, 0x94b, 0x902,
0x917, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x941, 0x915, 0x20,
0x935, 0x947, 0x933, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e,
0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x91f, 0x941, 0x935, 0x93e,
0x932, 0x942, 0x20, 0x935, 0x947, 0x933, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935,
0x947, 0x20, 0x935, 0x947, 0x933, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x909, 0x91c, 0x93c, 0x94d,
0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947,
0x933, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20,
0x935, 0x947, 0x933, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x20, 0x935, 0x947, 0x933, 0x935, 0x928, 0x93e, 0x924, 0x942, 0x20, 0x935, 0x947,
0x933, 0x935, 0x928, 0x93e, 0x924, 0x942, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x935, 0x928, 0x93e, 0x924, 0x942,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x935, 0x947, 0x928, 0x947, 0x91d, 0x941, 0x90f, 0x932, 0x93e, 0x20, 0x935, 0x947,
0x933, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924,
0x94b, 0x915, 0x20, 0x935, 0x947, 0x933, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f,
0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947, 0x933, 0x935, 0x94d, 0x932, 0x93e,
0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x935, 0x94d, 0x939,
0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x935,
0x947, 0x933, 0x935, 0x94d, 0x939, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d,
0x930, 0x93e, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x935, 0x94d, 0x939, 0x94b, 0x932, 0x94d, 0x917, 0x94b,
0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x935, 0x948, 0x915, 0x20, 0x906, 0x92f, 0x932,
0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x935, 0x93e, 0x932, 0x93f, 0x938, 0x20,
0x906, 0x928, 0x940, 0x20, 0x92b, 0x94d, 0x92f, 0x942, 0x91a, 0x942, 0x928, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x92f, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x935, 0x947, 0x933, 0x92f, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924, 0x20, 0x935, 0x947,
0x933, 0x92f, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x947, 0x915,
0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x935,
0x947, 0x933, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c,
0x930, 0x94d, 0x917, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x93f, 0x924,
0x20, 0x935, 0x947, 0x933, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f,
0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x941, 0x915, 0x949, 0x928, 0x20, 0x935,
0x947, 0x933, 0xc544, 0xd06c, 0xb808, 0x20, 0xc2dc, 0xac04, 0xc544, 0xd06c, 0xb808, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc544, 0xd06c, 0xb808, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc544, 0xd504, 0xac00, 0xb2c8, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xc911, 0xc559,
0xc544, 0xd504, 0xb9ac, 0xce74, 0x20, 0xc2dc, 0xac04, 0xb3d9, 0xc544, 0xd504, 0xb9ac, 0xce74,
0x20, 0xc2dc, 0xac04, 0xb0a8, 0xc544, 0xd504, 0xb9ac, 0xce74, 0x20, 0xc2dc, 0xac04, 0xc11c,
0xc544, 0xd504, 0xb9ac, 0xce74, 0x20, 0xc2dc, 0xac04, 0xc11c, 0xc544, 0xd504, 0xb9ac, 0xce74,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc11c, 0xc544, 0xd504, 0xb9ac, 0xce74, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc54c, 0xb798, 0xc2a4, 0xce74, 0x20, 0xc2dc, 0xac04, 0xc54c,
0xb798, 0xc2a4, 0xce74, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc54c, 0xb798, 0xc2a4, 0xce74, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc54c, 0xb9c8, 0xd2f0, 0x20, 0xd45c, 0xc900,
0x20, 0xc2dc, 0xac04, 0xc54c, 0xb9c8, 0xd2f0, 0x20, 0xd45c, 0xc900, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc54c, 0xb9c8, 0xd2f0, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544,
0xb9c8, 0xc874, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb9c8, 0xc874, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc544, 0xb9c8, 0xc874, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20,
0xc911, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xbbf8, 0x20, 0xc911, 0xbd80, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xbbf8, 0x20, 0xc911, 0xbd80, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xbbf8, 0x20, 0xb3d9, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xbbf8, 0x20, 0xb3d9, 0xbd80, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xb3d9, 0xbd80, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xbbf8, 0x20, 0xc0b0, 0xc9c0, 0x20, 0xc2dc, 0xac04, 0xbbf8, 0x20, 0xc0b0,
0xc545, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xc0b0, 0xc9c0, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xc2dc, 0xac04,
0xbbf8, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0x20, 0xd0dc,
0xd3c9, 0xc591, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb098, 0xb514,
0xb9ac, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb098, 0xb514, 0xb9ac, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc544, 0xb098, 0xb514, 0xb9ac, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544,
0xd53c, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc544, 0xd53c, 0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc544, 0xd53c, 0xc544, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc545, 0xd0c0,
0xc6b0, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xc545, 0xd0c0, 0xc6b0, 0x20, 0xd45c,
0xc900, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc545, 0xd0c0, 0xc6b0, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc545, 0xd1f4, 0xbca0, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04,
0xc545, 0xd1f4, 0xbca0, 0x20, 0xd45c, 0xc900, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc545, 0xd1f4,
0xbca0, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb77c, 0xbe44, 0xc544,
0x20, 0xc2dc, 0xac04, 0xc544, 0xb77c, 0xbe44, 0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544,
0xb77c, 0xbe44, 0xc544, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974,
0xd5e8, 0xd2f0, 0xb098, 0x20, 0xc2dc, 0xac04, 0xc544, 0xb974, 0xd5e8, 0xd2f0, 0xb098, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974, 0xd5e8, 0xd2f0, 0xb098, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974, 0xd5e8, 0xd2f0, 0xb098, 0x20, 0xc11c, 0xbd80, 0x20,
0xc2dc, 0xac04, 0xc544, 0xb974, 0xd5e8, 0xd2f0, 0xb098, 0x20, 0xc11c, 0xbd80, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc544, 0xb974, 0xd5e8, 0xd2f0, 0xb098, 0x20, 0xc11c, 0xbd80, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974, 0xba54, 0xb2c8, 0xc544, 0x20, 0xc2dc,
0xac04, 0xc544, 0xb974, 0xba54, 0xb2c8, 0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xb974,
0xba54, 0xb2c8, 0xc544, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb300, 0xc11c,
0xc591, 0x20, 0xc2dc, 0xac04, 0xb300, 0xc11c, 0xc591, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb300,
0xc11c, 0xc591, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8,
0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xc624, 0xc2a4,
0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911, 0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911, 0xbd80, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544,
0x20, 0xc911, 0xc11c, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c,
0xb9ac, 0xc544, 0x20, 0xc911, 0xc11c, 0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4,
0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc911, 0xc11c, 0xbd80, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20,
0xb3d9, 0xbd80, 0x20, 0xc2dc, 0xac04, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544,
0x20, 0xb3d9, 0xbd80, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c,
0xb9ac, 0xc544, 0x20, 0xb3d9, 0xbd80, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc11c, 0xbd80, 0x20, 0xc2dc,
0xac04, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc11c, 0xbd80, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc624, 0xc2a4, 0xd2b8, 0xb808, 0xc77c, 0xb9ac, 0xc544, 0x20, 0xc11c,
0xbd80, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xc81c, 0xb974, 0xbc14,
0xc774, 0xc794, 0x20, 0xc2dc, 0xac04, 0xc544, 0xc81c, 0xb974, 0xbc14, 0xc774, 0xc794, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc544, 0xc81c, 0xb974, 0xbc14, 0xc774, 0xc794, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xc870, 0xb808, 0xc2a4, 0x20, 0xc2dc, 0xac04, 0xc544,
0xc870, 0xb808, 0xc2a4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc544, 0xc870, 0xb808, 0xc2a4, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbc29, 0xae00, 0xb77c, 0xb370, 0xc2dc, 0x20,
0xc2dc, 0xac04, 0xbc29, 0xae00, 0xb77c, 0xb370, 0xc2dc, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbc29,
0xae00, 0xb77c, 0xb370, 0xc2dc, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbd80,
0xd0c4, 0x20, 0xc2dc, 0xac04, 0xbcfc, 0xb9ac, 0xbe44, 0xc544, 0x20, 0xc2dc, 0xac04, 0xbe0c,
0xb77c, 0xc9c8, 0xb9ac, 0xc544, 0x20, 0xc2dc, 0xac04, 0xbe0c, 0xb77c, 0xc9c8, 0xb9ac, 0xc544,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xbe0c, 0xb77c, 0xc9c8, 0xb9ac, 0xc544, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xbe0c, 0xb8e8, 0xb098, 0xc774, 0x20, 0xc2dc, 0xac04, 0xce74,
0xbcf4, 0x20, 0xbca0, 0xb974, 0xb370, 0x20, 0xc2dc, 0xac04, 0xce74, 0xbcf4, 0x20, 0xbca0,
0xb974, 0xb370, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xce74, 0xbcf4, 0x20, 0xbca0, 0xb974, 0xb370,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcf00, 0xc774, 0xc2dc, 0x20, 0xc2dc,
0xac04, 0xcc28, 0xbaa8, 0xb85c, 0x20, 0xc2dc, 0xac04, 0xcc44, 0xd140, 0x20, 0xc2dc, 0xac04,
0xcc44, 0xd140, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcc44, 0xd140, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xce60, 0xb808, 0x20, 0xc2dc, 0xac04, 0xce60, 0xb808, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xce60, 0xb808, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc911,
0xad6d, 0x20, 0xc2dc, 0xac04, 0xc911, 0xad6d, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc911, 0xad6d,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcd08, 0xc774, 0xbc1c, 0xc0b0, 0x20,
0xc2dc, 0xac04, 0xcd08, 0xc774, 0xbc1c, 0xc0b0, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcd08, 0xc774,
0xbc1c, 0xc0b0, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd06c, 0xb9ac, 0xc2a4,
0xb9c8, 0xc2a4, 0xc12c, 0x20, 0xc2dc, 0xac04, 0xcf54, 0xcf54, 0xc2a4, 0x20, 0xc81c, 0xb3c4,
0x20, 0xc2dc, 0xac04, 0xcf5c, 0xb86c, 0xbe44, 0xc544, 0x20, 0xc2dc, 0xac04, 0xcf5c, 0xb86c,
0xbe44, 0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcf5c, 0xb86c, 0xbe44, 0xc544, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcfe1, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04,
0xcfe1, 0x20, 0xc81c, 0xb3c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcfe1, 0x20, 0xc81c, 0xb3c4,
0x20, 0xc808, 0xbc18, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcfe0, 0xbc14,
0x20, 0xc2dc, 0xac04, 0xcfe0, 0xbc14, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcfe0, 0xbc14, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb370, 0xc774, 0xbe44, 0xc2a4, 0x20, 0xc2dc,
0xac04, 0xb4a4, 0xbabd, 0xb4a4, 0xb974, 0xbe4c, 0x20, 0xc2dc, 0xac04, 0xb3d9, 0xd2f0, 0xbaa8,
0xb974, 0x20, 0xc2dc, 0xac04, 0xc774, 0xc2a4, 0xd130, 0xc12c, 0x20, 0xc2dc, 0xac04, 0xc774,
0xc2a4, 0xd130, 0xc12c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774, 0xc2a4, 0xd130, 0xc12c, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc5d0, 0xcf70, 0xb3c4, 0xb974, 0x20, 0xc2dc,
0xac04, 0xc911, 0xbd80, 0xc720, 0xb7fd, 0x20, 0xc2dc, 0xac04, 0xc911, 0xbd80, 0xc720, 0xb7fd,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc911, 0xbd80, 0xc720, 0xb7fd, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xb3d9, 0xc720, 0xb7fd, 0x20, 0xc2dc, 0xac04, 0xb3d9, 0xc720, 0xb7fd,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xb3d9, 0xc720, 0xb7fd, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xadf9, 0xb3d9, 0xc720, 0xb7fd, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc11c, 0xc720,
0xb7fd, 0x20, 0xc2dc, 0xac04, 0xc11c, 0xc720, 0xb7fd, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc11c,
0xc720, 0xb7fd, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd3ec, 0xd074, 0xb79c,
0xb4dc, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xd3ec, 0xd074, 0xb79c, 0xb4dc, 0x20,
0xc81c, 0xb3c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd3ec, 0xd074, 0xb79c, 0xb4dc, 0x20, 0xc81c,
0xb3c4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd53c, 0xc9c0, 0x20, 0xc2dc,
0xac04, 0xd53c, 0xc9c0, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd53c, 0xc9c0, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xd504, 0xb791, 0xc2a4, 0xb839, 0x20, 0xac00, 0xc774, 0xc544,
0xb098, 0x20, 0xc2dc, 0xac04, 0xd504, 0xb791, 0xc2a4, 0xb839, 0x20, 0xb0a8, 0xbd80, 0x20,
0xc2dd, 0xbbfc, 0xc9c0, 0x20, 0xbc0f, 0x20, 0xb0a8, 0xadf9, 0x20, 0xc2dc, 0xac04, 0xac08,
0xb77c, 0xd30c, 0xace0, 0xc2a4, 0x20, 0xc2dc, 0xac04, 0xac10, 0xbe44, 0xc5d0, 0x20, 0xc2dc,
0xac04, 0xc870, 0xc9c0, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc870, 0xc9c0, 0xc544, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc870, 0xc9c0, 0xc544, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xae38, 0xbc84, 0xd2b8, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xadf8, 0xb9ac, 0xb2c8,
0xce58, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xb3d9, 0xbd80,
0x20, 0xc2dc, 0xac04, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xb3d9, 0xbd80, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xb3d9, 0xbd80, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xc11c, 0xbd80, 0x20,
0xc2dc, 0xac04, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xc11c, 0xbd80, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xadf8, 0xb9b0, 0xb780, 0xb4dc, 0x20, 0xc11c, 0xbd80, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xad0c, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xac78, 0xd504,
0xb9cc, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd558, 0xc640, 0xc774, 0x20, 0xc54c, 0xb958, 0xc0e8,
0x20, 0xc2dc, 0xac04, 0xd558, 0xc640, 0xc774, 0x20, 0xc54c, 0xb958, 0xc0e8, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xd558, 0xc640, 0xc774, 0x20, 0xc54c, 0xb958, 0xc0e8, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xd64d, 0xcf69, 0x20, 0xc2dc, 0xac04, 0xd64d, 0xcf69, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xd64d, 0xcf69, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xd638, 0xbe0c, 0xb4dc, 0x20, 0xc2dc, 0xac04, 0xd638, 0xbe0c, 0xb4dc, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xd638, 0xbe0c, 0xb4dc, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc778,
0xb3c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc778, 0xb3c4, 0xc591, 0x20, 0xc2dc, 0xac04, 0xc778,
0xb3c4, 0xcc28, 0xc774, 0xb098, 0x20, 0xc2dc, 0xac04, 0xc911, 0xbd80, 0x20, 0xc778, 0xb3c4,
0xb124, 0xc2dc, 0xc544, 0x20, 0xc2dc, 0xac04, 0xb3d9, 0xbd80, 0x20, 0xc778, 0xb3c4, 0xb124,
0xc2dc, 0xc544, 0x20, 0xc2dc, 0xac04, 0xc11c, 0xbd80, 0x20, 0xc778, 0xb3c4, 0xb124, 0xc2dc,
0xc544, 0x20, 0xc2dc, 0xac04, 0xc774, 0xb780, 0x20, 0xc2dc, 0xac04, 0xc774, 0xb780, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc774, 0xb780, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xc774, 0xb974, 0xcfe0, 0xce20, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xc774, 0xb974, 0xcfe0, 0xce20,
0xd06c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774, 0xb974, 0xcfe0, 0xce20, 0xd06c, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774, 0xc2a4, 0xb77c, 0xc5d8, 0x20, 0xc2dc, 0xac04,
0xc774, 0xc2a4, 0xb77c, 0xc5d8, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc774, 0xc2a4, 0xb77c, 0xc5d8,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc77c, 0xbcf8, 0x20, 0xc2dc, 0xac04,
0xc77c, 0xbcf8, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc77c, 0xbcf8, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xd398, 0xd2b8, 0xb85c, 0xd30c, 0xbe14, 0xb86d, 0xc2a4, 0xd06c, 0x2d,
0xce84, 0xcc28, 0xce20, 0xd0a4, 0x20, 0xc2dc, 0xac04, 0xd398, 0xd2b8, 0xb85c, 0xd30c, 0xbe14,
0xb86d, 0xc2a4, 0xd06c, 0x2d, 0xce84, 0xcc28, 0xce20, 0xd0a4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xd398, 0xd2b8, 0xb85c, 0xd30c, 0xbe14, 0xb86d, 0xc2a4, 0xd06c, 0x2d, 0xce84, 0xcc28, 0xce20,
0xd0a4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb3d9, 0xbd80, 0x20, 0xce74,
0xc790, 0xd750, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xc11c, 0xbd80, 0x20, 0xce74, 0xc790,
0xd750, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xb300, 0xd55c, 0xbbfc, 0xad6d, 0x20, 0xc2dc,
0xac04, 0xb300, 0xd55c, 0xbbfc, 0xad6d, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb300, 0xd55c, 0xbbfc,
0xad6d, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xcf54, 0xc2a4, 0xb77c, 0xc5d0,
0xc12c, 0x20, 0xc2dc, 0xac04, 0xd06c, 0xb77c, 0xc2a4, 0xb178, 0xc57c, 0xb974, 0xc2a4, 0xd06c,
0x20, 0xc2dc, 0xac04, 0xd06c, 0xb77c, 0xc2a4, 0xb178, 0xc57c, 0xb974, 0xc2a4, 0xd06c, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xd06c, 0xb77c, 0xc2a4, 0xb178, 0xc57c, 0xb974, 0xc2a4, 0xd06c, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd0a4, 0xb974, 0xae30, 0xc2a4, 0xc2a4, 0xd0c4,
0x20, 0xc2dc, 0xac04, 0xb791, 0xce74, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xb77c,
0xc778, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xb85c, 0xb4dc, 0x20, 0xd558, 0xc6b0,
0x20, 0xc2dc, 0xac04, 0xb85c, 0xb4dc, 0x20, 0xd558, 0xc6b0, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xb85c, 0xb4dc, 0x20, 0xd558, 0xc6b0, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xb9c8, 0xce74, 0xc624, 0x20, 0xc2dc, 0xac04, 0xb9c8, 0xce74, 0xc624, 0x20, 0xd45c, 0xc900,
0x20, 0xc2dc, 0xac04, 0xb9c8, 0xce74, 0xc624, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xb9c8, 0xac00, 0xb2e8, 0x20, 0xc2dc, 0xac04, 0xb9c8, 0xac00, 0xb2e8, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xb9c8, 0xac00, 0xb2e8, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xb9d0, 0xb808, 0xc774, 0xc2dc, 0xc544, 0x20, 0xc2dc, 0xac04, 0xbab0, 0xb514, 0xbe0c, 0x20,
0xc2dc, 0xac04, 0xb9c8, 0xb974, 0xd0a4, 0xc988, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04,
0xb9c8, 0xc15c, 0x20, 0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xbaa8, 0xb9ac, 0xc154, 0xc2a4,
0x20, 0xc2dc, 0xac04, 0xbaa8, 0xb9ac, 0xc154, 0xc2a4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbaa8,
0xb9ac, 0xc154, 0xc2a4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbaa8, 0xc2a8,
0x20, 0xc2dc, 0xac04, 0xba55, 0xc2dc, 0xcf54, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xc2dc,
0xac04, 0xba55, 0xc2dc, 0xcf54, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xba55, 0xc2dc, 0xcf54, 0x20, 0xd0dc, 0xd3c9, 0xc591, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xc6b8, 0xb780, 0xbc14, 0xd1a0, 0xb974, 0x20, 0xc2dc, 0xac04, 0xc6b8, 0xb780,
0xbc14, 0xd1a0, 0xb974, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc6b8, 0xb780, 0xbc14, 0xd1a0, 0xb974,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbaa8, 0xc2a4, 0xd06c, 0xbc14, 0x20,
0xc2dc, 0xac04, 0xbaa8, 0xc2a4, 0xd06c, 0xbc14, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbaa8, 0xc2a4,
0xd06c, 0xbc14, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbbf8, 0xc580, 0xb9c8,
0x20, 0xc2dc, 0xac04, 0xb098, 0xc6b0, 0xb8e8, 0x20, 0xc2dc, 0xac04, 0xb124, 0xd314, 0x20,
0xc2dc, 0xac04, 0xb274, 0xce7c, 0xb808, 0xb3c4, 0xb2c8, 0xc544, 0x20, 0xc2dc, 0xac04, 0xb274,
0xce7c, 0xb808, 0xb3c4, 0xb2c8, 0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb274, 0xce7c, 0xb808,
0xb3c4, 0xb2c8, 0xc544, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb274, 0xc9c8,
0xb79c, 0xb4dc, 0x20, 0xc2dc, 0xac04, 0xb274, 0xc9c8, 0xb79c, 0xb4dc, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xb274, 0xc9c8, 0xb79c, 0xb4dc, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xb274, 0xd380, 0xb4e4, 0xb79c, 0xb4dc, 0x20, 0xc2dc, 0xac04, 0xb274, 0xd380, 0xb4e4, 0xb79c,
0xb4dc, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb274, 0xd380, 0xb4e4, 0xb79c, 0xb4dc, 0x20, 0xd558,
0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb2c8, 0xc6b0, 0xc5d0, 0x20, 0xc2dc, 0xac04, 0xb178,
0xd37d, 0xc12c, 0x20, 0xc2dc, 0xac04, 0xb178, 0xd37d, 0xc12c, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xb178, 0xd37d, 0xc12c, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd398, 0xb974,
0xb09c, 0xb3c4, 0x20, 0xb370, 0x20, 0xb178, 0xb85c, 0xb0d0, 0x20, 0xc2dc, 0xac04, 0xd398,
0xb974, 0xb09c, 0xb3c4, 0x20, 0xb370, 0x20, 0xb178, 0xb85c, 0xb0d0, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xd398, 0xb974, 0xb09c, 0xb3c4, 0x20, 0xb370, 0x20, 0xb178, 0xb85c, 0xb0d0, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbd81, 0xb9c8, 0xb9ac, 0xc544, 0xb098, 0x20,
0xc81c, 0xb3c4, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xb178, 0xbcf4, 0xc2dc, 0xbe44,
0xb974, 0xc2a4, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xb178, 0xbcf4, 0xc2dc, 0xbe44, 0xb974, 0xc2a4,
0xd06c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb178, 0xbcf4, 0xc2dc, 0xbe44, 0xb974, 0xc2a4, 0xd06c,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc634, 0xc2a4, 0xd06c, 0x20, 0xc2dc,
0xac04, 0xc634, 0xc2a4, 0xd06c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc634, 0xc2a4, 0xd06c, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd30c, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xc2dc,
0xac04, 0xd30c, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd30c, 0xd0a4, 0xc2a4,
0xd0c4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd314, 0xb77c, 0xc6b0, 0x20,
0xc2dc, 0xac04, 0xd30c, 0xd478, 0xc544, 0xb274, 0xae30, 0xb2c8, 0x20, 0xc2dc, 0xac04, 0xd30c,
0xb77c, 0xacfc, 0xc774, 0x20, 0xc2dc, 0xac04, 0xd30c, 0xb77c, 0xacfc, 0xc774, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xd30c, 0xb77c, 0xacfc, 0xc774, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xd398, 0xb8e8, 0x20, 0xc2dc, 0xac04, 0xd398, 0xb8e8, 0x20, 0xd45c, 0xc900, 0xc2dc,
0xd398, 0xb8e8, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd544, 0xb9ac, 0xd540,
0x20, 0xc2dc, 0xac04, 0xd544, 0xb9ac, 0xd540, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd544, 0xb9ac,
0xd540, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd53c, 0xb2c9, 0xc2a4, 0x20,
0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xc138, 0xc778, 0xd2b8, 0xd53c, 0xc5d0, 0xb974, 0x20,
0xbbf8, 0xd074, 0xb871, 0x20, 0xc2dc, 0xac04, 0xc138, 0xc778, 0xd2b8, 0xd53c, 0xc5d0, 0xb974,
0x20, 0xbbf8, 0xd074, 0xb871, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc138, 0xc778, 0xd2b8, 0xd53c,
0xc5d0, 0xb974, 0x20, 0xbbf8, 0xd074, 0xb871, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xd54f, 0xcf00, 0xc5b8, 0x20, 0xc2dc, 0xac04, 0xd3ec, 0xb098, 0xd398, 0x20, 0xc2dc,
0xac04, 0xd0a4, 0xc9c8, 0xb85c, 0xb974, 0xb2e4, 0x20, 0xc2dc, 0xac04, 0xd0a4, 0xc9c8, 0xb85c,
0xb974, 0xb2e4, 0x20, 0xd45c, 0xc900, 0x20, 0xc2dc, 0xac04, 0xd0a4, 0xc9c8, 0xb85c, 0xb974,
0xb2e4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb808, 0xc704, 0xb2c8, 0xc639,
0x20, 0xc2dc, 0xac04, 0xb85c, 0xb370, 0xb77c, 0x20, 0xc2dc, 0xac04, 0xc0ac, 0xd560, 0xb9b0,
0x20, 0xc2dc, 0xac04, 0xc0ac, 0xd560, 0xb9b0, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xd560,
0xb9b0, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xb9c8, 0xb77c, 0x20,
0xc2dc, 0xac04, 0xc0ac, 0xb9c8, 0xb77c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xb9c8, 0xb77c,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xbaa8, 0xc544, 0x20, 0xc2dc,
0xac04, 0xc0ac, 0xbaa8, 0xc544, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc0ac, 0xbaa8, 0xc544, 0x20,
0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc138, 0xc774, 0xc178, 0x20, 0xc2dc, 0xac04,
0xc2f1, 0xac00, 0xd3ec, 0xb974, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc194, 0xb85c, 0xbaac, 0x20,
0xc81c, 0xb3c4, 0x20, 0xc2dc, 0xac04, 0xc0ac, 0xc6b0, 0xc2a4, 0x20, 0xc870, 0xc9c0, 0xc544,
0x20, 0xc2dc, 0xac04, 0xc218, 0xb9ac, 0xb0a8, 0x20, 0xc2dc, 0xac04, 0xc1fc, 0xc640, 0x20,
0xc2dc, 0xac04, 0xd0c0, 0xd788, 0xd2f0, 0x20, 0xc2dc, 0xac04, 0xb300, 0xb9cc, 0x20, 0xc2dc,
0xac04, 0xb300, 0xb9cc, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xb300, 0xb9cc, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xd0c0, 0xc9c0, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04,
0xd1a0, 0xcf08, 0xb77c, 0xc6b0, 0x20, 0xc2dc, 0xac04, 0xd1b5, 0xac00, 0x20, 0xc2dc, 0xac04,
0xd1b5, 0xac00, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd1b5, 0xac00, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xcd94, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xd22c, 0xb974, 0xd06c, 0xba54,
0xb2c8, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04, 0xd22c, 0xb974, 0xd06c, 0xba54, 0xb2c8, 0xc2a4,
0xd0c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd22c, 0xb974, 0xd06c, 0xba54, 0xb2c8, 0xc2a4, 0xd0c4,
0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xd22c, 0xbc1c, 0xb8e8, 0x20, 0xc2dc,
0xac04, 0xc6b0, 0xb8e8, 0xacfc, 0xc774, 0x20, 0xc2dc, 0xac04, 0xc6b0, 0xb8e8, 0xacfc, 0xc774,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc6b0, 0xb8e8, 0xacfc, 0xc774, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc6b0, 0xc988, 0xbca0, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xc2dc, 0xac04,
0xc6b0, 0xc988, 0xbca0, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc6b0, 0xc988,
0xbca0, 0xd0a4, 0xc2a4, 0xd0c4, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbc14,
0xb204, 0xc544, 0xd22c, 0x20, 0xc2dc, 0xac04, 0xbc14, 0xb204, 0xc544, 0xd22c, 0x20, 0xd45c,
0xc900, 0xc2dc, 0xbc14, 0xb204, 0xc544, 0xd22c, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xbca0, 0xb124, 0xc218, 0xc5d8, 0xb77c, 0x20, 0xc2dc, 0xac04, 0xbe14, 0xb77c, 0xb514,
0xbcf4, 0xc2a4, 0xd1a0, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xbe14, 0xb77c, 0xb514, 0xbcf4, 0xc2a4,
0xd1a0, 0xd06c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbe14, 0xb77c, 0xb514, 0xbcf4, 0xc2a4, 0xd1a0,
0xd06c, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xbcfc, 0xace0, 0xadf8, 0xb77c,
0xb4dc, 0x20, 0xc2dc, 0xac04, 0xbcfc, 0xace0, 0xadf8, 0xb77c, 0xb4dc, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xbcfc, 0xace0, 0xadf8, 0xb77c, 0xb4dc, 0x20, 0xd558, 0xacc4, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xbcf4, 0xc2a4, 0xd1a1, 0x20, 0xc2dc, 0xac04, 0xc6e8, 0xc774, 0xd06c, 0xc12c, 0x20,
0xc2dc, 0xac04, 0xc6d4, 0xb9ac, 0xc2a4, 0xd478, 0xd22c, 0xb098, 0x20, 0xc81c, 0xb3c4, 0x20,
0xc2dc, 0xac04, 0xc57c, 0xcfe0, 0xce20, 0xd06c, 0x20, 0xc2dc, 0xac04, 0xc57c, 0xcfe0, 0xce20,
0xd06c, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc57c, 0xcfe0, 0xce20, 0xd06c, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0xc608, 0xce74, 0xd14c, 0xb9b0, 0xbd80, 0xb974, 0xd06c, 0x20,
0xc2dc, 0xac04, 0xc608, 0xce74, 0xd14c, 0xb9b0, 0xbd80, 0xb974, 0xd06c, 0x20, 0xd45c, 0xc900,
0xc2dc, 0xc608, 0xce74, 0xd14c, 0xb9b0, 0xbd80, 0xb974, 0xd06c, 0x20, 0xd558, 0xacc4, 0x20,
0xd45c, 0xc900, 0xc2dc, 0xc720, 0xcf58, 0x20, 0xc2dc, 0xac04, 0xc870, 0xc120, 0x20, 0xc2dc,
0xac04, 0xc870, 0xc120, 0x20, 0xd45c, 0xc900, 0xc2dc, 0xc870, 0xc120, 0x20, 0xd558, 0xacc4,
0x20, 0xd45c, 0xc900, 0xc2dc, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x66,
0x78, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x41, 0x66, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20,
0x4e, 0x61, 0x76, 0xee, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41,
0x66, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68,
0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x66, 0x72,
0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x15f, 0xfb, 0x72, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x66, 0x72, 0xee, 0x6b, 0x61, 0x79,
0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x41, 0x66, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f,
0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x66, 0x72, 0xee,
0x6b, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e, 0x61, 0x76, 0x65,
0x6e, 0x64, 0xee, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee,
0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x4e, 0x61, 0x76, 0x65, 0x6e, 0x64, 0xee, 0x20, 0x79,
0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20,
0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76,
0x65, 0x6e, 0x64, 0xee, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72,
0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61,
0x74, 0x20, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79,
0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x20, 0x79, 0x61, 0x20,
0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61,
0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69,
0x6c, 0x61, 0x74, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee,
0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0xc7, 0x69, 0x79, 0x61, 0x79, 0xee, 0x20, 0x79,
0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20,
0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xc7, 0x69,
0x79, 0x61, 0x79, 0xee, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72,
0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0xc7, 0x69, 0x79, 0x61, 0x79, 0xee, 0x20, 0x79, 0x61,
0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42,
0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x61,
0x73, 0xee, 0x66, 0xee, 0x6b, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d,
0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75,
0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x50, 0x61, 0x73, 0xee, 0x66, 0xee,
0x6b, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b,
0x61, 0x79, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x50, 0x61, 0x73, 0xee, 0x66, 0xee, 0x6b, 0xea, 0x20, 0x79, 0x61,
0x20, 0x41, 0x6d, 0x65, 0x72, 0xee, 0x6b, 0x61, 0x79, 0x61, 0x20, 0x42,
0x61, 0x6b, 0x75, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x70,
0x69, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x70, 0x69,
0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x72, 0x65, 0x62, 0x69,
0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x72,
0x65, 0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x45, 0x72, 0x65, 0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0xee, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74,
0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e,
0x74, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x72,
0x6a, 0x61, 0x6e, 0x74, 0xee, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61,
0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e,
0x74, 0xee, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0xee, 0x6e, 0x61,
0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x45, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0xee, 0x6b, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0xee, 0x6b, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0xee, 0x6b,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee,
0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x79, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x79, 0x61,
0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e, 0x20, 0x61, 0x20, 0x41, 0x77, 0x69,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x79, 0x61, 0x20, 0x4e,
0x61, 0x76, 0xee, 0x6e, 0x20, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x52,
0x6f, 0x6a, 0x61, 0x76, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee,
0x6e, 0x20, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x41, 0x77,
0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79, 0x61, 0x20, 0x52,
0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79, 0x61,
0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x79,
0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x41, 0x77, 0x69, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x79, 0x63,
0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x65, 0x79, 0x63, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x65, 0x79, 0x63, 0x61, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x61, 0x6e, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x61, 0x6e, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x61, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x42, 0x65, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x42, 0x65, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x15f, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x42, 0x65, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x15f, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x42, 0xfb,
0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x42, 0x6f,
0x6c, 0xee, 0x76, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x42, 0x72, 0x61, 0x73, 0xee, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0xee, 0x6c, 0x79, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0xee, 0x6c,
0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x42, 0x72,
0xfb, 0x6e, 0x65, 0x79, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x65,
0x6c, 0x61, 0x6d, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x43, 0x61,
0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x15e, 0xee, 0x6c, 0x69, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x15e, 0xee, 0x6c, 0x69, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x15e, 0xee, 0x6c, 0x69, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0xc7, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0xc7, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0xc7, 0xee, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x43,
0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76,
0x61, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea,
0x6e, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61,
0x76, 0xea, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x4e, 0xee, 0x76, 0x68, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4b,
0x75, 0x62, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4b, 0x75,
0x62, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x75, 0x62, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x44, 0x61, 0x76, 0xee,
0x73, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0xee, 0x6d, 0xfb,
0x72, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20,
0x50, 0x61, 0x73, 0x6b, 0x61, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20, 0x50, 0x61,
0x73, 0x6b, 0x61, 0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20, 0x50, 0x61, 0x73, 0x6b, 0x61,
0x6c, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45,
0x6b, 0x77, 0x61, 0x64, 0x6f, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61,
0x76, 0xee, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f,
0x70, 0x61, 0x79, 0x61, 0x20, 0x4e, 0x61, 0x76, 0xee, 0x6e, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x4e,
0x61, 0x76, 0xee, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45, 0x77,
0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69,
0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f,
0x70, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61,
0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79,
0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61,
0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x20, 0x61, 0x20,
0x50, 0xea, 0x15f, 0x74, 0x69, 0x72, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a,
0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f,
0x70, 0x61, 0x79, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x61, 0x20,
0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x69,
0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76,
0xea, 0x6e, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x46, 0xee, 0x6a, 0x69, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x46, 0xee, 0x6a, 0x69, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x46, 0xee, 0x6a, 0x69, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x47, 0x75, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x79, 0x61,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x73, 0xee, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0xee, 0x6b, 0x61, 0x20,
0xfb, 0x20, 0x42, 0x61, 0x15f, 0xfb, 0x72, 0xea, 0x20, 0x46, 0x72, 0x61,
0x6e, 0x73, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x75, 0x72, 0x63, 0x69, 0x73, 0x74,
0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x75, 0x72, 0x63,
0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x47, 0x75,
0x72, 0x63, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x4e, 0x61, 0x76, 0xee, 0x6e, 0xee, 0x20, 0x79, 0x61, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x47, 0x72, 0xee, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20,
0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x47, 0x72, 0xee, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x52,
0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x47,
0x72, 0xee, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x52, 0x6f, 0x6a,
0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47,
0x72, 0xee, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x52, 0x6f, 0x6a,
0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x72, 0xee, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea,
0x20, 0x79, 0x61, 0x20, 0x47, 0x72, 0xee, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x4b, 0x65, 0x6e, 0x64, 0x61, 0x76, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x48,
0x6f, 0x76, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x48, 0x69, 0x6e,
0x64, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x4f, 0x6b, 0x79, 0x61, 0x6e, 0xfb, 0x73, 0x61, 0x20, 0x48, 0x69,
0x6e, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x69, 0x6e,
0x64, 0x69, 0xe7, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x45, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x61, 0x79, 0x61, 0x20,
0x4e, 0x61, 0x76, 0xee, 0x6e, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x45,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x61, 0x79, 0x61, 0x20, 0x52,
0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x45, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x61, 0x79, 0x61,
0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0xce, 0x72, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xce,
0x72, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0xce, 0x72, 0x61, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0xce, 0x73,
0x72, 0x61, 0xee, 0x6c, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0xce, 0x73,
0x72, 0x61, 0xee, 0x6c, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0xce, 0x73, 0x72,
0x61, 0xee, 0x6c, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0x15f, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x51, 0x61, 0x7a, 0x61, 0x78, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x68, 0x69, 0x6c, 0x61, 0x74,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x51, 0x61, 0x7a, 0x61, 0x78, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x52, 0x6f, 0x6a, 0x61, 0x76, 0x61,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x51, 0x69, 0x72, 0x78,
0x69, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x4c, 0xee,
0x6e, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x79, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0xee, 0x76, 0x61, 0x6e,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65,
0x73, 0x61, 0x73, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69,
0x72, 0x61, 0x76, 0xea, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x15f, 0x61, 0x6c,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x61, 0x73, 0xee, 0x66, 0xee, 0x6b,
0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0xee, 0x6b, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x50, 0x61, 0x73, 0xee, 0x66, 0xee, 0x6b, 0x61,
0x20, 0x4d, 0x65, 0x6b, 0x73, 0xee, 0x6b, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x50, 0x61, 0x73, 0xee, 0x66, 0xee, 0x6b, 0x61, 0x20, 0x4d,
0x65, 0x6b, 0x73, 0xee, 0x6b, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0xdb, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0xdb, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74,
0x61, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0xdb, 0x6c, 0x61, 0x6e, 0x62,
0x61, 0x74, 0x61, 0x72, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4d,
0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e, 0x61, 0xfb, 0x72, 0xfb,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e, 0x65, 0x70, 0x61,
0x6c, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x79, 0x61, 0x79, 0x61, 0x20, 0x4e, 0xfb, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79, 0x61,
0x79, 0x61, 0x20, 0x4e, 0xfb, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48,
0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x79, 0x61, 0x79, 0x61, 0x20, 0x4e, 0xfb, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61,
0x79, 0x61, 0x20, 0x4e, 0xfb, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x79, 0x61, 0x20, 0x4e, 0xfb, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x79, 0x61, 0x20,
0x4e, 0xfb, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76,
0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x69,
0x72, 0x61, 0x76, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50,
0x61, 0x6c, 0x61, 0x75, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x47, 0xee, 0x6e, 0x65, 0x79, 0x61, 0x20, 0x4e, 0xfb, 0x20, 0x79, 0x61,
0x20, 0x50, 0x61, 0x70, 0xfb, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0xfb, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0xfb, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0xfb, 0x61,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x65, 0x72, 0xfb,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x50, 0x65, 0x72, 0xfb, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x50, 0x65, 0x72, 0xfb, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x46, 0xee, 0x6c, 0xee, 0x70, 0xee, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x46, 0xee, 0x6c, 0xee, 0x70, 0xee,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee,
0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x46, 0xee, 0x6c, 0xee, 0x70, 0xee,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61,
0x76, 0xea, 0x6e, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0xee, 0x6b, 0x73,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0xfb, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0xfb, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0xfb, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x52, 0x6f, 0x74, 0x68,
0x65, 0x72, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x61, 0x78, 0x61, 0x6c, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x53, 0x61, 0x78, 0x61, 0x6c, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x53, 0x61, 0x78, 0x61, 0x6c, 0xee, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x65, 0x79, 0x15f, 0x65, 0x6c, 0x65, 0x72, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x53, 0xee, 0x6e, 0x67, 0x61, 0x70, 0xfb, 0x72,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76,
0xea, 0x6e, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x79, 0x61, 0x20, 0x42, 0x61, 0x15f, 0xfb, 0x72, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x53, 0x75, 0x72, 0xee, 0x6e, 0x61, 0x6d, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x61, 0x68, 0xee, 0x74, 0x69, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x61, 0xee, 0x70, 0x65,
0x69, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x54, 0x61, 0xee, 0x70,
0x65, 0x69, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x54, 0x61, 0xee, 0x70,
0x65, 0x69, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x61,
0x63, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x79,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x79, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0xea,
0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x54, 0x69, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x54, 0x69, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e,
0xea, 0x20, 0x79, 0x61, 0x20, 0x54, 0x69, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x54, 0xfb, 0x76, 0x61, 0x6c, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0xdb, 0x72, 0xfb, 0x67, 0xfb, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0xdb, 0x72, 0xfb, 0x67, 0xfb, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0xdb, 0x72, 0xfb, 0x67, 0xfb, 0x61, 0x79, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x4f, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4f, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x4f, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x56, 0x61, 0x6e, 0xfb, 0x61, 0x74, 0xfb,
0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x56, 0x61, 0x6e, 0xfb, 0x61,
0x74, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61,
0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20, 0x56, 0x61, 0x6e, 0xfb,
0x61, 0x74, 0xfb, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x56,
0x65, 0x6e, 0x65, 0x7a, 0xfb, 0x65, 0x6c, 0x61, 0x79, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0xea, 0x53, 0x61, 0x65,
0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0xea, 0x53, 0x61,
0x65, 0x74, 0x61, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x47, 0x69, 0x72, 0x61, 0x76, 0x61, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0xfb, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x79, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20, 0x79, 0x61, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0xea, 0x53, 0x61, 0x65, 0x74,
0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0xea, 0x53,
0x61, 0x65, 0x74, 0x61, 0x20, 0x48, 0x61, 0x76, 0xee, 0x6e, 0xea, 0x20,
0x79, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0xea, 0x53, 0x61, 0x65, 0x74, 0x61, 0x20, 0x59,
0x75, 0x6b, 0x6f, 0x6e, 0xea, 0x410, 0x444, 0x433, 0x430, 0x43d, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x411, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x410, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af,
0x448, 0x442, 0x4af, 0x43a, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b,
0x448, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410,
0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448,
0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x43b,
0x44f, 0x441, 0x43a, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x43b,
0x44f, 0x441, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x431, 0x43e, 0x440, 0x431,
0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x2c, 0x20, 0x431, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443,
0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x431, 0x43e, 0x440, 0x431,
0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434,
0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20,
0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x442, 0x43e, 0x43e, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x442, 0x43e,
0x43e, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x442, 0x43e, 0x43e, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x422, 0x44b, 0x43d, 0x447, 0x20, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x2c, 0x20, 0x422, 0x44b, 0x43d, 0x447, 0x20, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x43d, 0x434, 0x4af, 0x43a,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x422, 0x44b,
0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410,
0x43f, 0x438, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x410, 0x43f, 0x438, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x43f, 0x438, 0x430, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x410, 0x440, 0x430, 0x431, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x440, 0x430, 0x431, 0x438, 0x44f, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x410, 0x440, 0x430, 0x431, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442,
0x44b, 0x448, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x440, 0x43c, 0x435, 0x43d,
0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x442, 0x43b, 0x430,
0x43d, 0x442, 0x438, 0x43a, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e, 0x440, 0x431, 0x43e, 0x440,
0x434, 0x443, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e,
0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e, 0x440, 0x431, 0x43e,
0x440, 0x434, 0x443, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434,
0x443, 0x43a, 0x20, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x20, 0x431, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a,
0x20, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x43e, 0x440, 0x431, 0x43e, 0x440,
0x434, 0x443, 0x43a, 0x20, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x447, 0x44b,
0x433, 0x44b, 0x448, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x447, 0x44b,
0x433, 0x44b, 0x448, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x20, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x430, 0x442,
0x44b, 0x448, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x431, 0x430, 0x442,
0x44b, 0x448, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x437,
0x435, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439,
0x436, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439,
0x436, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x410, 0x437, 0x43e, 0x440, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448,
0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x411, 0x443, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x411, 0x43e, 0x43b, 0x438, 0x432, 0x438, 0x44f, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x440, 0x430, 0x437,
0x438, 0x43b, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411,
0x440, 0x443, 0x43d, 0x435, 0x439, 0x20, 0x414, 0x430, 0x440, 0x443, 0x441, 0x441,
0x430, 0x43b, 0x430, 0x43c, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41a, 0x430, 0x43f, 0x435, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x430, 0x43f, 0x435,
0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x430, 0x43f, 0x435,
0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x430, 0x43c,
0x43e, 0x440, 0x440, 0x43e, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x427, 0x430, 0x442,
0x430, 0x43c, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x44b, 0x442, 0x430,
0x439, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x44b,
0x442, 0x430, 0x439, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b,
0x441, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441,
0x430, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x440, 0x438, 0x441, 0x43c, 0x430, 0x441,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20,
0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a,
0x443, 0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d,
0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a,
0x443, 0x43a, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d,
0x44b, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x443, 0x43a, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x436, 0x430, 0x440,
0x44b, 0x43c, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x414, 0x44d, 0x432, 0x438,
0x441, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x414, 0x44e,
0x43c, 0x43e, 0x43d, 0x2d, 0x434, 0x2d, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b,
0x448, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x418, 0x441, 0x442, 0x435, 0x440, 0x20, 0x430, 0x440,
0x430, 0x43b, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x418, 0x441, 0x442, 0x435, 0x440, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x441, 0x442, 0x435, 0x440,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42d,
0x43a, 0x443, 0x430, 0x434, 0x43e, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x411, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x411, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434, 0x443,
0x43a, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x411, 0x43e,
0x440, 0x431, 0x43e, 0x440, 0x434, 0x443, 0x43a, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x430, 0x43b, 0x438, 0x43d,
0x438, 0x43d, 0x433, 0x440, 0x430, 0x434, 0x20, 0x436, 0x430, 0x43d, 0x430, 0x20,
0x41c, 0x438, 0x43d, 0x441, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411,
0x430, 0x442, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x43e, 0x43b,
0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x424, 0x43e, 0x43b, 0x43a, 0x43b,
0x435, 0x43d, 0x434, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b,
0x43d, 0x44b, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x438, 0x436, 0x438, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x438, 0x436, 0x438, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x424, 0x438, 0x436, 0x438, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x440, 0x430, 0x43d,
0x446, 0x443, 0x437, 0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x440, 0x430, 0x43d, 0x446,
0x443, 0x437, 0x20, 0x422, 0x4af, 0x448, 0x442, 0x4af, 0x433, 0x4af, 0x20, 0x436,
0x430, 0x43d, 0x430, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438,
0x43a, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413,
0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x435, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x440, 0x443, 0x437,
0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413,
0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x440, 0x443, 0x437, 0x438,
0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x440, 0x438, 0x43d,
0x432, 0x438, 0x447, 0x20, 0x431, 0x43e, 0x44e, 0x43d, 0x447, 0x430, 0x20, 0x43e,
0x440, 0x442, 0x43e, 0x447, 0x43e, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b,
0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411,
0x430, 0x442, 0x44b, 0x448, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x443, 0x43b, 0x443, 0x4a3, 0x434,
0x443, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x442,
0x44b, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413, 0x430,
0x432, 0x430, 0x439, 0x438, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x430, 0x432, 0x430, 0x439,
0x438, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x430,
0x432, 0x430, 0x439, 0x438, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433,
0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x436, 0x430,
0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x425, 0x43e, 0x432, 0x434, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x425, 0x43e, 0x432, 0x434, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x418, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x418, 0x43d, 0x434, 0x438, 0x20, 0x43e, 0x43a, 0x435,
0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x418,
0x43d, 0x434, 0x43e, 0x43a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x43e, 0x440, 0x431, 0x43e, 0x440, 0x434,
0x443, 0x43a, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x44b, 0x433,
0x44b, 0x448, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x411, 0x430, 0x442,
0x44b, 0x448, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x418, 0x440, 0x430,
0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x418, 0x440,
0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x43a, 0x4af, 0x43d,
0x434, 0x4af, 0x437, 0x433, 0x4af, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x44c, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x43f,
0x43e, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x416,
0x430, 0x43f, 0x43e, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x416, 0x430, 0x43f, 0x43e, 0x43d, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x427, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x41a, 0x430, 0x437, 0x430,
0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x411, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x41a, 0x430, 0x437, 0x430,
0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a,
0x43e, 0x440, 0x435, 0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x43e, 0x441, 0x440, 0x430,
0x435, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x440,
0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f,
0x440, 0x441, 0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f,
0x440, 0x441, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41a, 0x44b, 0x440, 0x433, 0x44b, 0x437,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41b, 0x430, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41b, 0x43e, 0x440, 0x434, 0x20,
0x425, 0x430, 0x443, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x430,
0x443, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x438, 0x44f, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x43b, 0x44c,
0x434, 0x438, 0x432, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41c, 0x430, 0x440, 0x43a, 0x435, 0x437, 0x430, 0x441, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x430, 0x432,
0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c,
0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x2c, 0x20, 0x422,
0x44b, 0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a,
0x430, 0x2c, 0x20, 0x422, 0x44b, 0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x2c,
0x20, 0x422, 0x44b, 0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20,
0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x423, 0x43b, 0x430,
0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x423, 0x43b, 0x430,
0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x436, 0x430, 0x439, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x43e,
0x441, 0x43a, 0x432, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a,
0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41c, 0x43e, 0x441,
0x43a, 0x432, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41c, 0x439, 0x430, 0x43d, 0x43c, 0x430,
0x440, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x430,
0x443, 0x440, 0x443, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434,
0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e,
0x43d, 0x438, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x416,
0x430, 0x4a3, 0x44b, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x416, 0x430, 0x4a3,
0x44b, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x44d,
0x43d, 0x434, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d,
0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x44d, 0x43d, 0x434, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41d, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x44d, 0x43d, 0x434,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41d, 0x438, 0x443, 0x44d, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x43e, 0x440, 0x444,
0x43e, 0x43b, 0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b,
0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e,
0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x435, 0x440, 0x43d,
0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e,
0x43d, 0x44c, 0x44f, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d,
0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c,
0x44f, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x43a, 0x44b, 0x448,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41e, 0x43c,
0x441, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x430, 0x43b, 0x430, 0x443,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x430, 0x43f,
0x443, 0x430, 0x2d, 0x416, 0x430, 0x4a3, 0x44b, 0x20, 0x413, 0x432, 0x438, 0x43d,
0x435, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x430,
0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x435, 0x440,
0x443, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d,
0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x438, 0x43b,
0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430,
0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x438, 0x43b, 0x438,
0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440,
0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x424, 0x435, 0x43d, 0x438, 0x43a,
0x441, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b,
0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x435,
0x43d, 0x20, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x436, 0x430, 0x43d, 0x430, 0x20,
0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x44c, 0x435, 0x440,
0x20, 0x436, 0x430, 0x43d, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x44c, 0x435, 0x440,
0x20, 0x436, 0x430, 0x43d, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x438, 0x442, 0x43a, 0x44d, 0x440, 0x43d, 0x433,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x41f, 0x43e, 0x43d,
0x430, 0x43f, 0x435, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x41f, 0x445, 0x435, 0x43d, 0x44c, 0x44f, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x420, 0x435, 0x44e, 0x43d, 0x438, 0x43e, 0x43d, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x420, 0x43e, 0x442, 0x435, 0x440,
0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x430,
0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x430, 0x43c,
0x43e, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421,
0x430, 0x43c, 0x43e, 0x430, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x430, 0x43c, 0x43e, 0x430,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f,
0x443, 0x440, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421,
0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x20, 0x430, 0x440, 0x430, 0x43b, 0x434,
0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x448, 0x442, 0x4af, 0x43a, 0x20, 0x416, 0x43e,
0x440, 0x436, 0x438, 0x44f, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x435, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x421, 0x430, 0x43e, 0x430, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x430, 0x438, 0x442, 0x438,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x430, 0x439,
0x43f, 0x435, 0x439, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x439, 0x43f,
0x435, 0x439, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x430, 0x436, 0x438, 0x43a, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x43e,
0x43a, 0x435, 0x43b, 0x430, 0x443, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x43a, 0x44b,
0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x427, 0x443, 0x443, 0x43a,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x440,
0x43a, 0x43c, 0x4e9, 0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x4af, 0x440, 0x43a, 0x43c, 0x4e9, 0x43d,
0x441, 0x442, 0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4af, 0x440, 0x43a, 0x43c, 0x4e9,
0x43d, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x422, 0x443, 0x432, 0x430,
0x43b, 0x443, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x423,
0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x43a,
0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x436, 0x430, 0x439,
0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x4e8,
0x437, 0x431, 0x435, 0x43a, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x4e8, 0x437, 0x431, 0x435, 0x43a, 0x441, 0x442,
0x430, 0x43d, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x4e8, 0x437, 0x431, 0x435, 0x43a, 0x441, 0x442, 0x430,
0x43d, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412, 0x430, 0x43d, 0x443,
0x430, 0x442, 0x443, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x430,
0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e,
0x441, 0x442, 0x43e, 0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20,
0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x412, 0x43e, 0x43b, 0x433,
0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b,
0x441, 0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20,
0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x436,
0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441,
0x44b, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a,
0x442, 0x44b, 0x441, 0x44b, 0x423, 0x435, 0x439, 0x43a, 0x20, 0x430, 0x440, 0x430,
0x43b, 0x434, 0x430, 0x440, 0x44b, 0x43d, 0x44b, 0x43d, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x423, 0x43e, 0x43b, 0x438, 0x441, 0x20, 0x436,
0x430, 0x43d, 0x430, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x42f, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x42f, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443,
0x431, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442,
0x44b, 0x441, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431,
0x443, 0x440, 0x433, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b,
0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433,
0x20, 0x43a, 0x44b, 0x448, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443,
0x440, 0x433, 0x20, 0x436, 0x430, 0x439, 0x43a, 0x44b, 0x20, 0x443, 0x431, 0x430,
0x43a, 0x442, 0x44b, 0x441, 0x44b, 0x42e, 0x43a, 0x43e, 0x43d, 0x20, 0x443, 0x431,
0x430, 0x43a, 0x442, 0x44b, 0x441, 0x44b, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe82, 0xead,
0xe87, 0xead, 0xeb2, 0xec0, 0xe81, 0xea3, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2,
0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe82, 0xead, 0xe87, 0xead, 0xeb2, 0xec0,
0xe81, 0xea3, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9,
0xead, 0xe99, 0xe82, 0xead, 0xe87, 0xead, 0xeb2, 0xec0, 0xe81, 0xea3, 0xec0, 0xea7,
0xea5, 0xeb2, 0x20, 0xead, 0xeb1, 0xe9f, 0xe81, 0xeb2, 0xe99, 0xeb4, 0xeaa, 0xe96,
0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xe9f,
0xea3, 0xeb4, 0x200b, 0xe81, 0xeb2, 0x200b, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xe9f, 0xea3, 0xeb4, 0x200b, 0xe81, 0xeb2,
0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xe9f, 0xea3, 0xeb4,
0x200b, 0xe81, 0xeb2, 0x200b, 0xec3, 0xe95, 0xec9, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xead, 0xeb2, 0x200b, 0xe9f, 0xea3, 0xeb4, 0x200b, 0xe81, 0xeb2, 0x200b, 0xe95,
0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96,
0xeb2, 0xe99, 0x200b, 0xead, 0xeb2, 0x200b, 0xe9f, 0xea3, 0xeb4, 0x200b, 0xe81, 0xeb2,
0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b,
0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xead, 0xeb2, 0x200b, 0xe9f, 0xea3, 0xeb4, 0x200b,
0xe81, 0xeb2, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95,
0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xeb0, 0xea5, 0xeb1, 0xeaa, 0xe81,
0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2,
0xe99, 0xead, 0xeb0, 0xea5, 0xeb1, 0xeaa, 0xe81, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2,
0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xeb0, 0xea5, 0xeb1, 0xeaa,
0xe81, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xecd, 0xea1, 0xeb2, 0xe95, 0xeb5,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xead, 0xecd, 0xea1, 0xeb2, 0xe95, 0xeb5, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0,
0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xecd, 0xea1, 0xeb2, 0xe95, 0xeb5,
0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2, 0xea1, 0xec0, 0xe82, 0xe94, 0xead, 0xeb2,
0xec0, 0xea1, 0xe8a, 0xead, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1,
0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb2, 0x200b,
0xec0, 0xea1, 0x200b, 0xe8a, 0xead, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xeb2,
0x200b, 0xec0, 0xea1, 0x200b, 0xe8a, 0xead, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7,
0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe81, 0xeb2, 0xe87,
0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xead,
0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2,
0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xead, 0xe81, 0xec0, 0xea7,
0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe95, 0xeb2, 0xec0,
0xea7, 0xeb1, 0xe99, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec1, 0xe96,
0xe9a, 0xe9e, 0xeb9, 0xec0, 0xe82, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec1, 0xe96, 0xe9a, 0xe9e, 0xeb9,
0xec0, 0xe82, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0,
0xea7, 0xeb1, 0xe99, 0xec1, 0xe96, 0xe9a, 0xe9e, 0xeb9, 0xec0, 0xe82, 0xebb, 0xeb2,
0xec0, 0xea7, 0xea5, 0xeb2, 0xec1, 0xe9b, 0xe8a, 0xeb4, 0xe9f, 0xeb4, 0xe81, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec1,
0xe9b, 0xe8a, 0xeb4, 0xe9f, 0xeb4, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2,
0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xec1, 0xe9b, 0xe8a, 0xeb4, 0xe9f, 0xeb4, 0xe81,
0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xead, 0xec0, 0xe9e, 0xe8d, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec0, 0xead, 0xec0,
0xe9e, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1,
0xe99, 0xead, 0xeb2, 0xec0, 0xe9e, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xeb1,
0xe94, 0xe95, 0xeb2, 0xead, 0xeb9, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb1, 0xe94, 0xe95, 0xeb2, 0xead, 0xeb9,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99,
0xead, 0xeb1, 0xe94, 0xe95, 0xeb2, 0xead, 0xeb9, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead,
0xeb1, 0xe94, 0xec2, 0xe95, 0xec0, 0xe9a, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2,
0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb1, 0xe94, 0xec2, 0xe95, 0xec0,
0xe9a, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead,
0xe99, 0xead, 0xeb1, 0xe94, 0xec2, 0xe95, 0xec0, 0xe9a, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xea3, 0xeb2, 0x200b, 0xe9a, 0xebd, 0xe99, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b,
0xe96, 0xeb2, 0xe99, 0xead, 0xeb2, 0x200b, 0xea3, 0xeb2, 0x200b, 0xe9a, 0xebd, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead,
0xeb2, 0xea3, 0xeb2, 0xe9a, 0xebd, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xead, 0xeb2, 0x200b, 0xec0, 0xe88, 0x200b, 0xe97, 0xeb4, 0x200b, 0xe99, 0xeb2, 0x200b,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0,
0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88, 0xe99, 0x200b, 0xe97,
0xeb4, 0x200b, 0xe99, 0xeb2, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5,
0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xead, 0xeb2,
0x200b, 0xec0, 0xe88, 0xe99, 0x200b, 0xe97, 0xeb4, 0x200b, 0xe99, 0xeb2, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xea7, 0x200b, 0xeaa, 0x200b, 0xec0, 0xe97, 0xeb4,
0xe99, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88, 0xe99, 0x200b, 0xe97, 0xeb4, 0x200b, 0xe99,
0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xeaa, 0x200b, 0xec0, 0xe97,
0xeb4, 0xe99, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88, 0xe99, 0x200b, 0xe97, 0xeb4, 0x200b,
0xe99, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94,
0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xec0, 0xea7, 0x200b, 0xeaa, 0x200b, 0xec0,
0xe97, 0xeb4, 0xe99, 0xead, 0xeb2, 0x200b, 0xec0, 0xe88, 0xe99, 0x200b, 0xe97, 0xeb4,
0x200b, 0xe99, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xeb2, 0xec0, 0xea1, 0xec0,
0xe99, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96,
0xeb2, 0xe99, 0xead, 0xeb2, 0xec0, 0xea1, 0xec0, 0xe99, 0xe8d, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xeb2, 0xec0,
0xea1, 0xec0, 0xe99, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe82, 0xead, 0xe87, 0xead,
0xeb2, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb4, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe82, 0xead, 0xe87, 0xead, 0xeb2,
0xec1, 0xea5, 0xe99, 0xe95, 0xeb4, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2,
0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe82, 0xead, 0xe87, 0xead, 0xeb2, 0xec1, 0xea5,
0xe99, 0xe95, 0xeb4, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xead, 0xead, 0x200b,
0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xec0, 0xea5, 0xe8d, 0x200b, 0xe81, 0xeb2,
0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0,
0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xead, 0x200b, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3,
0x200b, 0xec0, 0xea5, 0xe8d, 0x200b, 0xe81, 0x200b, 0xeb2, 0xe87, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b,
0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xec0, 0xea5, 0xe8d, 0x200b,
0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xead, 0xead, 0xeaa, 0x200b,
0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b, 0xe81, 0xeb2, 0xe87, 0x200b,
0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b,
0xe96, 0xeb2, 0xe99, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5,
0xebd, 0xe99, 0x200b, 0xe81, 0xeb2, 0xe87, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7,
0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xead, 0xead, 0xeaa,
0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b, 0xe81, 0xeb2, 0xe87,
0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3,
0x200b, 0xea5, 0xebd, 0xe99, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99,
0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2,
0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0x200b, 0x200b, 0x200b, 0xead, 0xead, 0xeaa,
0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b, 0xe95, 0xeb2, 0x200b,
0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xead,
0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b, 0xe95,
0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xead, 0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b,
0xec0, 0xea5, 0xe8d, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b,
0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0x200b,
0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xead, 0xeaa, 0x200b,
0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0,
0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xead, 0xead,
0xeaa, 0x200b, 0xec0, 0xe95, 0xea3, 0x200b, 0xea5, 0xebd, 0xe99, 0x200b, 0xe95, 0xeb2,
0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5,
0xeb2, 0xead, 0xeb1, 0xeaa, 0xec0, 0xe8a, 0xeb5, 0xec4, 0xe9a, 0xe88, 0xeb1, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xead, 0xeb1, 0xeaa, 0xec0, 0xe8a, 0xeb5, 0xec4, 0xe9a, 0xe88, 0xeb1, 0xe99, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xead,
0xeb1, 0xeaa, 0xec0, 0xe8a, 0xeb5, 0xec4, 0xe9a, 0xe88, 0xeb1, 0xe99, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb2, 0x200b, 0xec2, 0xe8a, 0x200b, 0xec0, 0xea3,
0x200b, 0xeaa, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b,
0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb2, 0x200b, 0xec2, 0xe8a, 0x200b,
0xec0, 0xea3, 0x200b, 0xeaa, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0,
0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xeb2, 0x200b, 0xec2,
0xe8a, 0x200b, 0xec0, 0xea3, 0x200b, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20, 0xe9a,
0xeb1, 0xe87, 0xe81, 0xeb0, 0xea5, 0xeb2, 0xec0, 0xe97, 0xe94, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0x20, 0xe9a, 0xeb1,
0xe87, 0xe81, 0xeb0, 0xea5, 0xeb2, 0xec0, 0xe97, 0xe94, 0xec0, 0xea7, 0xea5, 0xeb2,
0x20, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0x20, 0xe9a, 0xeb1,
0xe87, 0xe81, 0xeb0, 0xea5, 0xeb2, 0xec0, 0xe97, 0xe94, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xe9e, 0xeb9, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xec2, 0xe9a, 0x200b, 0xea5, 0xeb4, 0x200b, 0xec0, 0xea7, 0xe8d, 0xec0,
0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2, 0xea1, 0xec0, 0xe82, 0xe94, 0xe9a, 0xea3, 0xeb2,
0xe8a, 0xeb4, 0xec0, 0xea5, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0xe95, 0xeb2, 0xe96, 0xeb2, 0xe99, 0xec0, 0xe9a, 0xea3, 0xe8a, 0xeb5, 0xec0, 0xea5,
0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2, 0xea1, 0xec0, 0xe82, 0xe94, 0xea5,
0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe95, 0xeb2, 0xea1, 0xec0, 0xe82,
0xe94, 0xe9a, 0xea3, 0xeb2, 0xe8a, 0xeb5, 0xec0, 0xea5, 0xe8d, 0x200b, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0xe9a, 0xea3, 0xeb9, 0x200b, 0xec4, 0xe99, 0xe94, 0xeb2, 0x200b,
0xea3, 0xeb8, 0xeaa, 0x200b, 0xe8a, 0xeb2, 0x200b, 0xea5, 0xeb2, 0xea1, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe84, 0xe9a, 0x200b, 0xec0, 0xea7, 0xeb5, 0xe94,
0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xec0, 0xe84, 0xe9a, 0x200b, 0xec0, 0xea7,
0xeb5, 0xe94, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94,
0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xec0, 0xe84, 0xe9a, 0x200b, 0xec0,
0xea7, 0xeb5, 0xe94, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xe84, 0xe8a, 0xeb5, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe88, 0xeb2, 0x200b, 0xec2, 0xea1, 0x200b, 0xec2,
0xea3, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe8a, 0xeb2, 0x200b, 0xe97, 0xeb2,
0xea1, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe8a, 0xeb2, 0x200b, 0xe97, 0xeb2, 0xea1,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8,
0xebd, 0xe87, 0x200b, 0xe8a, 0xeb2, 0x200b, 0xe97, 0xeb2, 0xea1, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xe8a, 0xeb4, 0x200b, 0xea5, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99,
0xe8a, 0xeb4, 0x200b, 0xea5, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5,
0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xe8a, 0xeb4, 0x200b,
0xea5, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe88, 0xeb5, 0xe99, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe88,
0xeb5, 0xe99, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99,
0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xe88, 0xeb5, 0xe99, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xec2, 0xe8a, 0xe8d, 0x200b, 0xe9a, 0xeb2, 0x200b, 0xe8a, 0xeb1,
0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2,
0xe99, 0xec2, 0xe8a, 0xe8d, 0xe9a, 0xeb2, 0xe8a, 0xeb1, 0xe99, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xec2,
0xe8a, 0xe8d, 0xe9a, 0xeb2, 0xe8a, 0xeb1, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe84, 0x200b, 0xea3, 0xeb4, 0xeaa, 0x200b,
0xea1, 0xeb2, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeab, 0xea1, 0xeb9, 0xec8, 0xec0,
0xe81, 0xeb2, 0xeb0, 0xec2, 0xe81, 0xec2, 0xe81, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2,
0xec2, 0xe84, 0xea5, 0xecd, 0xeb2, 0xec0, 0xe9a, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec2, 0xe84, 0xea5, 0xeb3,
0xec0, 0xe9a, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae,
0xec9, 0xead, 0xe99, 0xec2, 0xe84, 0xea5, 0xecd, 0xeb2, 0xec0, 0xe9a, 0xe8d, 0xec0,
0xea7, 0xea5, 0xeb2, 0xeab, 0xea1, 0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xe84,
0xeb8, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96,
0xeb2, 0xe99, 0xeab, 0xea1, 0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xe84, 0xeb8,
0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe84, 0xeb4, 0xec8, 0xe87,
0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b,
0xedd, 0xeb9, 0xec8, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe84, 0xeb8, 0xe81,
0xec0, 0xea7, 0xea5, 0xeb2, 0xe84, 0xeb4, 0xea7, 0xe9a, 0xeb2, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe82, 0xead, 0xe87,
0xe84, 0xeb4, 0xea7, 0xe9a, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87,
0xec0, 0xea7, 0xeb1, 0xe99, 0xe84, 0xeb4, 0xea7, 0xe9a, 0xeb2, 0xec0, 0xea7, 0xea5,
0xeb2, 0xec0, 0xe94, 0xea7, 0xeb4, 0xe94, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe94, 0xeb9,
0xea1, 0xead, 0xe87, 0xe94, 0xeb9, 0xea7, 0xeb4, 0xea5, 0xec0, 0xea7, 0xea5, 0xeb2,
0xe95, 0xeb5, 0xea1, 0xecd, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xead,
0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b,
0xead, 0xeb5, 0x200b, 0xeaa, 0x200b, 0xec0, 0xe95, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99,
0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xead, 0xeb5, 0x200b, 0xeaa, 0x200b, 0xec0, 0xe95,
0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9,
0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xead, 0xeb5,
0x200b, 0xeaa, 0x200b, 0xec0, 0xe95, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xec0, 0xead, 0x200b, 0xe81, 0xebb, 0xea7, 0x200b, 0xe94, 0xecd, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0x200b, 0xe81, 0xeb2,
0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a,
0xe81, 0xeb2, 0xe87, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0,
0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xea2, 0xeb9, 0x200b,
0xec2, 0xea3, 0xe9a, 0x200b, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0,
0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea1, 0xeb2, 0x200b, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99,
0x200b, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0,
0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xea2,
0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1,
0xe99, 0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0x200b,
0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0x200b, 0xe9b, 0xebd, 0xe99, 0x200b, 0xe95, 0xeb2,
0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xead, 0xead, 0xe81, 0x200b, 0xec4, 0xe81,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a,
0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0,
0x200b, 0xe96, 0xeb2, 0xe99, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0x200b, 0xe95,
0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9,
0xead, 0xe99, 0xea2, 0xeb9, 0x200b, 0xec2, 0xea3, 0xe9a, 0x200b, 0xe95, 0xeb2, 0x200b,
0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0x200b, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xedd, 0xeb9, 0xec8, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b,
0xe9f, 0xead, 0x200b, 0xea5, 0xecc, 0xe81, 0x200b, 0xec1, 0xea5, 0xe99, 0x200b, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b,
0xe96, 0xeb2, 0xe99, 0xedd, 0xeb9, 0xec8, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b,
0xe9f, 0xead, 0x200b, 0xea5, 0xecc, 0xe81, 0x200b, 0xec1, 0xea5, 0xe99, 0x200b, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae,
0xec9, 0xead, 0xe99, 0xedd, 0xeb9, 0xec8, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b,
0xe9f, 0xead, 0x200b, 0xea5, 0xecc, 0xe81, 0x200b, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7,
0xea5, 0xeb2, 0xe9f, 0xeb4, 0xe88, 0xeb4, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2,
0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe9f, 0xeb4, 0xe88, 0xeb4, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe9f, 0xeb4,
0xe88, 0xeb4, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe9f, 0x200b, 0xea3,
0xe99, 0x200b, 0xe8a, 0xecc, 0x200b, 0xec0, 0xe81, 0xe8d, 0x200b, 0xe99, 0xeb2, 0xec0,
0xea7, 0xea5, 0xeb2, 0xe9d, 0xea3, 0xeb1, 0xec8, 0xe87, 0xe95, 0xead, 0xe99, 0xec3,
0xe95, 0xec9, 0x20, 0xec1, 0xea5, 0xeb0, 0x20, 0xec1, 0xead, 0xe99, 0xe95, 0xeb2,
0xe81, 0xe95, 0xeb4, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe81, 0xeb2,
0x200b, 0xea5, 0xeb2, 0x200b, 0xe9b, 0xeb2, 0x200b, 0xe81, 0xead, 0x200b, 0xeaa, 0xec0,
0xea7, 0xea5, 0xeb2, 0xec1, 0xe81, 0xea1, 0xec0, 0xe9a, 0xe8d, 0xec0, 0xea7, 0xea5,
0xeb2, 0xe88, 0xecd, 0xec0, 0xe88, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2,
0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe88, 0xecd, 0xec0, 0xe88, 0xe8d, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe88,
0xecd, 0xec0, 0xe88, 0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeab, 0xea1, 0xeb9, 0xec8,
0xec0, 0xe81, 0xeb2, 0xeb0, 0xe81, 0xeb4, 0xea5, 0xec0, 0xe9a, 0xeb5, 0xe94, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0xe81, 0xea3, 0xeb5, 0xe99, 0x200b, 0xea7, 0xeb4, 0x200b,
0xe8a, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead,
0xead, 0xe81, 0xe82, 0xead, 0xe87, 0xe81, 0xea3, 0xeb5, 0xe99, 0xec1, 0xea5, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xead, 0xe81, 0xe81, 0xea3, 0xeb5,
0xe99, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9,
0xeae, 0xec9, 0xead, 0xe99, 0xe81, 0xea3, 0xeb5, 0xe99, 0xec1, 0xea5, 0xe99, 0xe95,
0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2,
0xe81, 0xea3, 0xeb5, 0xe99, 0xec1, 0xea5, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1,
0xe99, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95,
0xeb0, 0xe96, 0xeb2, 0xe99, 0xe81, 0xea3, 0xeb5, 0xe99, 0xec1, 0xea5, 0xe99, 0xe95,
0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2,
0xe95, 0xead, 0xe99, 0xe97, 0xec8, 0xebd, 0xe87, 0xe81, 0xea3, 0xeb5, 0xe99, 0xec1,
0xea5, 0xe99, 0xe95, 0xeb2, 0xec0, 0xea7, 0xeb1, 0xe99, 0xe95, 0xebb, 0xe81, 0xec0,
0xea7, 0xea5, 0xeb2, 0xe81, 0xea7, 0xea1, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xe81, 0xeb9, 0x200b, 0xea5, 0x200b, 0xecc, 0xe9f, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81,
0xeb2, 0xe8d, 0xead, 0xeb2, 0xe99, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeae, 0xeb2,
0xea7, 0xeb2, 0xe8d, 0x2d, 0xec0, 0xead, 0xea5, 0xeb9, 0xe97, 0xebd, 0xe99, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xeae,
0xeb2, 0xea7, 0xeb2, 0xe8d, 0x2d, 0xec0, 0xead, 0xea5, 0xeb9, 0xe97, 0xebd, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xead, 0xe99, 0xe97, 0xec8, 0xebd, 0xe87, 0xeae,
0xeb2, 0xea7, 0xeb2, 0xe8d, 0x2d, 0xec0, 0xead, 0xea5, 0xeb9, 0xe97, 0xebd, 0xe99,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xeae, 0xead, 0xe87, 0x200b, 0xe81, 0xebb,
0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xeae, 0xead, 0xe87, 0x200b, 0xe81, 0xebb,
0xe87, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94,
0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xeae, 0xead, 0xe87, 0x200b, 0xe81,
0xebb, 0xe87, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xeae, 0xead, 0xe9a, 0x200b,
0xe94, 0xecc, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xeae, 0xead, 0xe9a, 0x200b,
0xe94, 0xecc, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b,
0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xeae, 0xead, 0xe9a, 0x200b,
0xe94, 0xecc, 0xec0, 0xea7, 0xea5, 0xeb2, 0x20, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94,
0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeab, 0xea1, 0xeb0, 0xeab, 0xeb2, 0xeaa, 0xeb0,
0xeab, 0xea1, 0xeb8, 0xe94, 0xead, 0xeb4, 0xe99, 0xec0, 0xe94, 0xe8d, 0xec0, 0xea7,
0xea5, 0xeb2, 0xead, 0xeb4, 0xe99, 0xe94, 0xeb9, 0xe88, 0xeb5, 0xe99, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb4, 0xe99, 0x200b, 0xec2, 0xe94, 0x200b, 0xec0,
0xe99, 0x200b, 0xec0, 0xe8a, 0xe8d, 0x200b, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xead, 0xeb4, 0xe99, 0x200b, 0xec2, 0xe94, 0x200b, 0xec0, 0xe99,
0x200b, 0xec0, 0xe8a, 0xe8d, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99,
0x200b, 0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb4,
0xe99, 0x200b, 0xec2, 0xe94, 0x200b, 0xec0, 0xe99, 0x200b, 0xec0, 0xe8a, 0xe8d, 0x200b,
0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb5, 0x200b, 0xea3, 0xeb2, 0xe99, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b,
0xe96, 0xeb2, 0xe99, 0xead, 0xeb5, 0x200b, 0xea3, 0xeb2, 0xe99, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b,
0xead, 0xeb5, 0x200b, 0xea3, 0xeb2, 0x200b, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0x200b,
0xeb2, 0xead, 0xeb5, 0x200b, 0xe84, 0xeb8, 0xe94, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b,
0xe96, 0xeb2, 0xe99, 0xead, 0xeb5, 0x200b, 0xe84, 0xeb8, 0xe94, 0x200b, 0xeaa, 0x200b,
0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9,
0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xeb5, 0x200b, 0xe84, 0xeb8, 0xe94, 0x200b,
0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead, 0xeb4,
0x200b, 0xeaa, 0x200b, 0xea3, 0xeb2, 0x200b, 0xec0, 0xead, 0xea7, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2,
0xe99, 0xead, 0xeb4, 0x200b, 0xeaa, 0x200b, 0xea3, 0xeb2, 0x200b, 0xec0, 0xead, 0xea7,
0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xead,
0xeb4, 0xeaa, 0xea3, 0xeb2, 0xec0, 0xead, 0xea7, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xe8d, 0xeb5, 0xec8, 0x200b, 0xe9b, 0xeb8, 0xec8, 0xe99, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2,
0xe99, 0x200b, 0xe8d, 0xeb5, 0xec8, 0x200b, 0xe9b, 0xeb8, 0xec8, 0xe99, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87,
0x200b, 0xe8d, 0xeb5, 0xec8, 0x200b, 0xe9b, 0xeb8, 0xec8, 0xe99, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xe84, 0xeb2, 0x200b, 0xe8a, 0xeb1, 0xe81, 0x200b, 0xeaa, 0x200b,
0xe96, 0xeb2, 0xe99, 0x200b, 0xe95, 0xeb2, 0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b,
0xead, 0xead, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe84, 0xeb2, 0x200b,
0xe8a, 0xeb1, 0xe81, 0x200b, 0xeaa, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe95, 0xeb2,
0x200b, 0xec0, 0xea7, 0xeb1, 0xe99, 0x200b, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5,
0xeb2, 0xec0, 0xe81, 0xebb, 0xeb2, 0xeab, 0xebc, 0xeb5, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99,
0x200b, 0xec0, 0xe81, 0xebb, 0xeb2, 0x200b, 0xeab, 0xea5, 0xeb5, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b,
0xec0, 0xe81, 0xebb, 0xeb2, 0x200b, 0xeab, 0xea5, 0xeb5, 0xec0, 0xea7, 0xea5, 0xeb2,
0xe84, 0xead, 0xeaa, 0xec1, 0xea3, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe84,
0xea3, 0xeb1, 0xeaa, 0x200b, 0xec2, 0xe99, 0x200b, 0xea2, 0xeb2, 0x200b, 0xeaa, 0x200b,
0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b,
0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xe84, 0xea3, 0xeb1, 0xeaa, 0x200b, 0xec2,
0xe99, 0x200b, 0xea2, 0xeb2, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead,
0xe99, 0xe84, 0xea3, 0xeb1, 0xeaa, 0x200b, 0xec2, 0xe99, 0x200b, 0xea2, 0xeb2, 0x200b,
0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xe84, 0xe8d, 0xe81,
0xeb4, 0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb1, 0xe87,
0xe81, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xedd, 0xeb9, 0xec8, 0x200b,
0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xea5, 0xeb2, 0xe8d, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea5, 0xead, 0xe94, 0x200b, 0xec0, 0xeae, 0xebb, 0xeb2, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96,
0xeb2, 0xe99, 0x200b, 0xea5, 0xead, 0xe94, 0x200b, 0xec0, 0xeae, 0xebb, 0xeb2, 0x200b,
0xec0, 0xea7, 0x200b, 0xea5, 0x200b, 0xeaa, 0xeb2, 0x200b, 0xe95, 0xead, 0xe99, 0x200b,
0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xea5, 0xead, 0xe94, 0x200b, 0xec0, 0xeae,
0xebb, 0xeb2, 0x200b, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xec0, 0xe81, 0xebb,
0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2,
0xe99, 0xea1, 0xeb2, 0xec0, 0xe81, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5,
0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xea1, 0xeb2, 0xec0, 0xe81, 0xebb,
0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xea1, 0xeb1, 0xe81, 0xe81, 0xeb2, 0xec0,
0xe94, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96,
0xeb2, 0xe99, 0xec0, 0xea1, 0xeb1, 0xe81, 0xe81, 0xeb2, 0xec0, 0xe94, 0xe99, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xec0,
0xea1, 0xeb1, 0xe81, 0xe81, 0xeb2, 0xec0, 0xe94, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea1, 0xeb2, 0x200b, 0xec0, 0xea5, 0x200b, 0xec0, 0xe8a, 0xe8d, 0xec0,
0xea7, 0xea5, 0xeb2, 0xea1, 0xeb1, 0xea5, 0xe94, 0xeb5, 0xe9f, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea1, 0xeb2, 0xec0, 0xe84, 0xeb5, 0xe8a, 0xeb1, 0xeaa, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xedd, 0xeb9, 0xec8, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b,
0xea1, 0xeb2, 0x200b, 0xec1, 0xe8a, 0xea7, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xec0, 0xea1, 0xebb, 0xeb2, 0x200b, 0xea3, 0xeb4, 0x200b, 0xe97, 0xebd, 0x200b, 0xeaa,
0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0,
0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea1, 0xebb, 0xeb2, 0x200b, 0xea3, 0xeb4, 0x200b,
0xe97, 0xebd, 0x200b, 0xeaa, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5,
0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xec0, 0xea1,
0xebb, 0xeb2, 0x200b, 0xea3, 0xeb4, 0x200b, 0xe97, 0xebd, 0x200b, 0xeaa, 0xec0, 0xea7,
0xea5, 0xeb2, 0xea1, 0xecd, 0xeaa, 0xeb1, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec1,
0xe9b, 0xe8a, 0xeb4, 0xe9f, 0xeb4, 0xe81, 0xec0, 0xea1, 0xeb1, 0xe81, 0xe8a, 0xeb4,
0xe81, 0xeb1, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0,
0xe96, 0xeb2, 0xe99, 0xec1, 0xe9b, 0xe8a, 0xeb4, 0xe9f, 0xeb4, 0xe81, 0xec0, 0xea1,
0xeb1, 0xe81, 0xe8a, 0xeb4, 0xe81, 0xeb1, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81,
0xeb2, 0xe87, 0xec0, 0xea7, 0xeb1, 0xe99, 0xec1, 0xe9b, 0xe8a, 0xeb4, 0xe9f, 0xeb4,
0xe81, 0xec0, 0xea1, 0xeb1, 0xe81, 0xe8a, 0xeb4, 0xe81, 0xeb1, 0xe99, 0xec0, 0xea7,
0xea5, 0xeb2, 0x20, 0xead, 0xeb9, 0xea5, 0xeb2, 0xe99, 0xe9a, 0xeb2, 0xec0, 0xe95,
0xeb5, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2,
0xe99, 0x20, 0xead, 0xeb9, 0xea5, 0xeb2, 0xe99, 0xe9a, 0xeb2, 0xec0, 0xe95, 0xeb5,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99,
0xead, 0xeb9, 0xea5, 0xeb2, 0xe99, 0xe9a, 0xeb2, 0xec0, 0xe95, 0xeb5, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xead, 0x200b, 0xeaa, 0x200b, 0xec2, 0xe84, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b,
0xe96, 0xeb2, 0xe99, 0xea1, 0xead, 0x200b, 0xeaa, 0x200b, 0xec2, 0xe84, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9,
0xead, 0xe99, 0xea1, 0xead, 0x200b, 0xeaa, 0x200b, 0xec2, 0xe84, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea1, 0xebd, 0xe99, 0xea1, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xe99, 0xeb2, 0x200b, 0xead, 0xeb9, 0x200b, 0xea3, 0xeb8, 0x200b, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xec0, 0xe99, 0x200b, 0xe9b, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5,
0xeb2, 0xe99, 0xeb4, 0xea7, 0xec1, 0xe84, 0xea5, 0xeb4, 0xec2, 0xe94, 0xec0, 0xe99,
0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2,
0xe99, 0xe99, 0xeb4, 0xea7, 0xec1, 0xe84, 0xea5, 0xeb4, 0xec2, 0xe94, 0xec0, 0xe99,
0xe8d, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead,
0xe99, 0xe99, 0xeb4, 0xea7, 0xec1, 0xe84, 0xea5, 0xeb4, 0xec2, 0xe94, 0xec0, 0xe99,
0xe8d, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe99, 0xeb4, 0xea7, 0x200b, 0xe8a,
0xeb5, 0x200b, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1,
0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xe99, 0xeb4, 0xea7,
0x200b, 0xe8a, 0xeb5, 0x200b, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xe95, 0xead, 0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0x200b, 0xe99, 0xeb4,
0xea7, 0x200b, 0xe8a, 0xeb5, 0x200b, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xe99, 0xeb4, 0xea7, 0x200b, 0xe9f, 0xeb2, 0xea7, 0x200b, 0xec1, 0xea5,
0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95,
0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe99, 0xeb4, 0xea7, 0x200b, 0xe9f, 0xeb2,
0xea7, 0x200b, 0xec1, 0xea5, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe81, 0xeb2, 0xe87,
0xec0, 0xea7, 0xeb1, 0xe99, 0xe99, 0xeb4, 0xea7, 0xe9f, 0xeb2, 0xea7, 0xec1, 0xea5,
0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe99, 0xeb4, 0xead, 0xeb9, 0xec0, 0xead, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe99, 0xecd,
0x200b, 0xe9f, 0xead, 0x200b, 0xea5, 0xecc, 0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0,
0xe81, 0xeb2, 0xeb0, 0x200b, 0xe99, 0xecd, 0x200b, 0xe9f, 0xead, 0x200b, 0xea5, 0xecc,
0xe81, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9,
0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xec0, 0xe81, 0xeb2, 0xeb0, 0x200b, 0xe99, 0xecd,
0x200b, 0xe9f, 0xead, 0x200b, 0xea5, 0xecc, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0,
0xe9f, 0xe99, 0xeb1, 0xe99, 0xec2, 0xe94, 0xec0, 0xe94, 0xec2, 0xe99, 0xeae, 0xead,
0xe99, 0xeae, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0,
0xe96, 0xeb2, 0xe99, 0xec0, 0xe9f, 0xe99, 0xeb1, 0xe99, 0xec2, 0xe94, 0xec0, 0xe94,
0xec2, 0xe99, 0xeae, 0xead, 0xe99, 0xeae, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5,
0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xec0, 0xe9f, 0xe99, 0xeb1, 0xe99,
0xec2, 0xe94, 0xec0, 0xe94, 0xec2, 0xe99, 0xeae, 0xead, 0xe99, 0xeae, 0xeb2, 0xec0,
0xea7, 0xea5, 0xeb2, 0xeab, 0xea1, 0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2, 0xeb0, 0xea1,
0xeb2, 0xec0, 0xea3, 0xe8d, 0xe99, 0xeb2, 0xec0, 0xeab, 0xe99, 0xeb7, 0xead, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec2, 0xe99, 0x200b, 0xec2, 0xe9a, 0x200b, 0xe8a,
0xeb4, 0x200b, 0xe9a, 0xeb4, 0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2,
0xe99, 0xec2, 0xe99, 0x200b, 0xec2, 0xe9a, 0x200b, 0xe8a, 0xeb4, 0x200b, 0xe9a, 0xeb4,
0x200b, 0xeaa, 0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5,
0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xec2, 0xe99, 0x200b,
0xec2, 0xe9a, 0x200b, 0xe8a, 0xeb4, 0x200b, 0xe9a, 0xeb4, 0x200b, 0xeaa, 0x200b, 0xe84,
0xecc, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xead, 0xead, 0xea1, 0x200b, 0xeaa,
0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94,
0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xead, 0xead, 0xea1, 0x200b, 0xeaa,
0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b,
0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xead, 0xea1, 0x200b, 0xeaa,
0x200b, 0xe84, 0xecc, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe9b, 0xeb2, 0x200b,
0xe81, 0xeb5, 0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe9b,
0xeb2, 0x200b, 0xe81, 0xeb5, 0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99,
0x200b, 0xe9b, 0xeb2, 0x200b, 0xe81, 0xeb5, 0x200b, 0xeaa, 0x200b, 0xe96, 0xeb2, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xe9b, 0xeb2, 0xec0, 0xea5, 0xebb, 0xeb2, 0xec0, 0xea7,
0xea5, 0xeb2, 0xe9b, 0xeb2, 0xe9b, 0xebb, 0xea7, 0xe81, 0xeb4, 0xe99, 0xeb5, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe9b, 0xeb2, 0x200b, 0xea3, 0xeb2, 0x200b, 0xe81,
0xea7, 0xe8d, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b,
0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xe9b, 0xeb2, 0x200b, 0xea3, 0xeb2,
0x200b, 0xe81, 0xea7, 0xe8d, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0,
0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xe9b, 0xeb2, 0x200b,
0xea3, 0xeb2, 0x200b, 0xe81, 0xea7, 0xe8d, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xec0, 0xe9b, 0x200b, 0xea3, 0xeb9, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0x200b,
0xea1, 0xeb2, 0x200b, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0xec0,
0xe9b, 0x200b, 0xea3, 0xeb9, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0,
0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9, 0xead, 0xe99, 0x200b, 0xec0, 0xe9b, 0x200b,
0xea3, 0xeb9, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe9f, 0xeb4, 0x200b,
0xea5, 0xeb4, 0xe9a, 0x200b, 0xe9b, 0xeb4, 0xe99, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b,
0xe9f, 0xeb4, 0x200b, 0xea5, 0xeb4, 0xe9a, 0x200b, 0xe9b, 0xeb4, 0xe99, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9,
0xead, 0xe99, 0x200b, 0xe9f, 0xeb4, 0x200b, 0xea5, 0xeb4, 0xe9a, 0x200b, 0xe9b, 0xeb4,
0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeab, 0xea1, 0xeb9, 0xec8, 0xec0, 0xe81, 0xeb2,
0xeb0, 0xe9f, 0xeb5, 0xe99, 0xeb4, 0xe81, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0xec0, 0xe8a, 0xe99, 0x200b, 0xe9b, 0xeb5, 0x200b, 0xec1, 0xead, 0x20, 0xec1, 0xea5,
0xeb0, 0x200b, 0xea1, 0xeb4, 0x200b, 0xe81, 0xebb, 0xea7, 0x200b, 0xea5, 0xead, 0xe99,
0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0,
0x200b, 0xe96, 0xeb2, 0xe99, 0xec0, 0xe8a, 0xe99, 0x200b, 0xe9b, 0xeb5, 0x200b, 0xec1,
0xead, 0x20, 0xec1, 0xea5, 0xeb0, 0x200b, 0xea1, 0xeb4, 0x200b, 0xe81, 0xebb, 0xea7,
0x200b, 0xea5, 0xead, 0xe99, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0xe95, 0xead,
0xe99, 0x200b, 0xe97, 0xec8, 0xebd, 0xe87, 0xec0, 0xe8a, 0xe99, 0x200b, 0xe9b, 0xeb5,
0x200b, 0xec1, 0xead, 0x20, 0xec1, 0xea5, 0xeb0, 0x200b, 0xea1, 0xeb4, 0x200b, 0xe81,
0xebb, 0xea7, 0x200b, 0xea5, 0xead, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe9e, 0xeb4,
0xe94, 0xec1, 0xe84, 0xea3, 0xecc, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec2, 0xe9b,
0xec0, 0xe99, 0xe9a, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe9b, 0xebd, 0xe87, 0xea2, 0xeb2,
0xe87, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe84, 0xeb5, 0xea7, 0xea5, 0xecd, 0xe94, 0xeb2,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xe84, 0xeb5, 0xea7, 0xea5, 0xecd, 0xe94, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5,
0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe84, 0xeb5, 0xea7, 0xea5, 0xecd,
0xe94, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xea3, 0x200b, 0xead,
0xeb9, 0x200b, 0xe99, 0xeb4, 0x200b, 0xea2, 0xebb, 0xe87, 0xec0, 0xea7, 0xea5, 0xeb2,
0x20, 0xec2, 0xea3, 0xe97, 0xeb5, 0xe95, 0xeb2, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xe8a, 0xeb2, 0x200b, 0xeae, 0xeb2, 0x200b, 0xea5, 0xeb4, 0xe99, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96,
0xeb2, 0xe99, 0xe8a, 0xeb2, 0x200b, 0xeae, 0xeb2, 0x200b, 0xea5, 0xeb4, 0xe99, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae,
0xec9, 0xead, 0xe99, 0xe8a, 0xeb2, 0x200b, 0xeae, 0xeb2, 0x200b, 0xea5, 0xeb4, 0xe99,
0xec0, 0xea7, 0xea5, 0xeb2, 0xe8a, 0xeb2, 0xea1, 0xebb, 0xea7, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe8a, 0xeb2, 0xea1,
0xebb, 0xea7, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9,
0xead, 0xe99, 0xe8a, 0xeb2, 0xea1, 0xebb, 0xea7, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xec0, 0xe8a, 0x200b, 0xec0, 0xe8a, 0x200b, 0xea5, 0x200b, 0xeaa, 0xecc, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xeaa, 0xeb4, 0xe87, 0x200b, 0xe81, 0xeb0, 0x200b,
0xec2, 0xe9b, 0xec0, 0xea7, 0xea5, 0xeb2, 0xeab, 0xea1, 0xeb9, 0xec8, 0xec0, 0xe81,
0xeb2, 0xeb0, 0xec2, 0xe8a, 0xec2, 0xea5, 0xea1, 0xead, 0xe99, 0xec0, 0xea7, 0xea5,
0xeb2, 0xe88, 0xecd, 0xec0, 0xe88, 0xe8d, 0xec3, 0xe95, 0xec9, 0xec0, 0xea7, 0x200b,
0xea5, 0xeb2, 0x200b, 0xe8a, 0xeb8, 0x200b, 0xea3, 0xeb4, 0x200b, 0xe99, 0xeb2, 0xea1,
0xec0, 0xea7, 0xea5, 0xeb2, 0x20, 0xec2, 0xe8a, 0xea7, 0xeb2, 0xec0, 0xea7, 0xea5,
0xeb2, 0xe97, 0xeb2, 0xeae, 0xeb4, 0xe95, 0xeb4, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2,
0x200b, 0xec4, 0xe97, 0x200b, 0xec0, 0xe9b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b,
0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99, 0x200b, 0xec4,
0xe97, 0x200b, 0xec0, 0xe9b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xe95, 0xead,
0xe99, 0x200b, 0xe97, 0xec8, 0x200b, 0xebd, 0xe87, 0x200b, 0xec4, 0xe97, 0x200b, 0xec0,
0xe9b, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe97, 0xeb2, 0xe88, 0xeb4, 0xe81, 0xeb4, 0xeaa,
0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec2, 0xe95, 0xec0, 0xe81, 0xec0,
0xea5, 0xebb, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xead, 0xe87, 0xe81, 0xeb2,
0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99,
0xe95, 0xead, 0xe87, 0xe81, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94,
0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe95, 0xead, 0xe87, 0xe81, 0xeb2, 0xec0, 0xea7,
0xea5, 0xeb2, 0xe8a, 0xeb8, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xea7, 0xe81,
0xec0, 0xea1, 0xe99, 0xeb4, 0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xe95, 0xea7, 0xe81, 0xec0,
0xea1, 0xe99, 0xeb4, 0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5,
0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xe95, 0xea7, 0xe81, 0xec0, 0xea1,
0xe99, 0xeb4, 0xeaa, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xe95, 0xeb9,
0xea7, 0xeb2, 0xea5, 0xeb9, 0x200b, 0xec0, 0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xead,
0xeb9, 0x200b, 0xea3, 0xeb9, 0x200b, 0xe81, 0xea7, 0xe8d, 0xec0, 0xea7, 0x200b, 0xea5,
0xeb2, 0x200b, 0xea1, 0xeb2, 0xe94, 0x200b, 0xe95, 0xeb0, 0x200b, 0xe96, 0xeb2, 0xe99,
0x200b, 0xead, 0xeb9, 0x200b, 0xea3, 0xeb9, 0x200b, 0xe81, 0xea7, 0xe8d, 0xec0, 0xea7,
0x200b, 0xea5, 0xeb2, 0x200b, 0xea5, 0xeb0, 0x200b, 0xe94, 0xeb9, 0x200b, 0xeae, 0xec9,
0xead, 0xe99, 0x200b, 0xead, 0xeb9, 0x200b, 0xea3, 0xeb9, 0x200b, 0xe81, 0xea7, 0xe8d,
0xec0, 0xea7, 0xea5, 0xeb2, 0xead, 0xeb8, 0xeaa, 0xec0, 0xe9a, 0xe81, 0xeb4, 0xe94,
0xeaa, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xead, 0xeb8, 0xeaa, 0xec0, 0xe9a, 0xe81, 0xeb4,
0xe94, 0xeaa, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0,
0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xead, 0xeb8, 0xeaa, 0xec0, 0xe9a, 0xe81,
0xeb4, 0xe94, 0xeaa, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea7,
0xeb2, 0xe99, 0xeb9, 0xead, 0xeb2, 0xe95, 0xeb9, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xea7, 0xeb2, 0xe99, 0xeb9, 0xead,
0xeb2, 0xe95, 0xeb9, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae,
0xec9, 0xead, 0xe99, 0xea7, 0xeb2, 0xe99, 0xeb9, 0xead, 0xeb2, 0xe95, 0xeb9, 0xec0,
0xea7, 0x200b, 0xea5, 0xeb2, 0x200b, 0xec0, 0xea7, 0x200b, 0xec0, 0xe99, 0x200b, 0xe8a,
0xeb9, 0x200b, 0xec0, 0xead, 0x200b, 0xea5, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5,
0xeb2, 0xe94, 0xeb5, 0xec2, 0xea7, 0xeaa, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xea5, 0xeb2, 0xe94,
0xeb5, 0xec2, 0xea7, 0xeaa, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5,
0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xea5, 0xeb2, 0xe94, 0xeb5, 0xec2,
0xea7, 0xeaa, 0xe95, 0xebb, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec2, 0xea7, 0xec2,
0xe81, 0xe81, 0xea3, 0xeb2, 0xe94, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94,
0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec2, 0xea7, 0xec2, 0xe81, 0xe81, 0xea3, 0xeb2,
0xe94, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead,
0xe99, 0xec2, 0xea7, 0xec2, 0xe81, 0xe81, 0xea3, 0xeb2, 0xe94, 0xec0, 0xea7, 0xea5,
0xeb2, 0x20, 0xea7, 0xead, 0xeaa, 0xec2, 0xe95, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2,
0xec0, 0xe81, 0xeb2, 0xeb0, 0xec0, 0xea7, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea7,
0xeb2, 0xea5, 0xea5, 0xeb4, 0xeaa, 0x20, 0xec1, 0xea5, 0xeb0, 0x20, 0xe9f, 0xeb9,
0xe95, 0xeb9, 0xe99, 0xeb2, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea2, 0xeb2, 0xe81, 0xeb9,
0xe94, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1, 0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96,
0xeb2, 0xe99, 0xea2, 0xeb2, 0xe81, 0xeb9, 0xe94, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xea2, 0xeb2, 0xe81, 0xeb9,
0xe94, 0xeaa, 0xec0, 0xea7, 0xea5, 0xeb2, 0xec0, 0xea2, 0xe81, 0xeb2, 0xec0, 0xe95,
0xea5, 0xeb4, 0xe99, 0xec0, 0xe9a, 0xeb5, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2, 0xea1,
0xeb2, 0xe94, 0xe95, 0xeb0, 0xe96, 0xeb2, 0xe99, 0xec0, 0xea2, 0xe81, 0xeb2, 0xec0,
0xe95, 0xea5, 0xeb4, 0xe99, 0xec0, 0xe9a, 0xeb5, 0xe81, 0xec0, 0xea7, 0xea5, 0xeb2,
0xea5, 0xeb0, 0xe94, 0xeb9, 0xeae, 0xec9, 0xead, 0xe99, 0xec0, 0xea2, 0xe81, 0xeb2,
0xec0, 0xe95, 0xea5, 0xeb4, 0xe99, 0xec0, 0xe9a, 0xeb5, 0xe81, 0xec0, 0xea7, 0xea5,
0xeb2, 0xea2, 0xeb9, 0xe84, 0xead, 0xe99, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x101, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74,
0x72, 0x75, 0x6d, 0x101, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x44, 0x69, 0x65, 0x6e, 0x76, 0x69, 0x64, 0x101,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75,
0x6d, 0x101, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x101, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x101, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x13c, 0x61, 0x73, 0x6b,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x13c, 0x61, 0x73,
0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x41, 0x13c, 0x61, 0x73, 0x6b, 0x61, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x73, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x69, 0x73,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x75, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x75, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x75, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x61,
0x6c, 0x6e, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x61, 0x6c,
0x6e, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4b, 0x61, 0x6c, 0x6e, 0x75, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6c,
0x75, 0x73, 0x101, 0x20, 0x6f, 0x6b, 0x65, 0x101, 0x6e, 0x61, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6c, 0x75, 0x73, 0x101, 0x20, 0x6f, 0x6b,
0x65, 0x101, 0x6e, 0x61, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6c, 0x75, 0x73, 0x101, 0x20, 0x6f,
0x6b, 0x65, 0x101, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x69,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x6e, 0x61,
0x64, 0x69, 0x72, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x70, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x72, 0x101, 0x62, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x70, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x72, 0x101, 0x62, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x70, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x72, 0x101, 0x62, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x70, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69,
0x65, 0x74, 0x75, 0x6d, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x12b, 0x6e,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74,
0x75, 0x6d, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x12b, 0x6e, 0x61, 0x73,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x72, 0x6d, 0x113, 0x6e,
0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x72,
0x6d, 0x113, 0x6e, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x72, 0x6d, 0x113,
0x6e, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75,
0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x69, 0x73,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20,
0x72, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20,
0x72, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74,
0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x101, 0x6c, 0x61, 0x69, 0x73, 0x20, 0x72, 0x69, 0x65, 0x74, 0x75,
0x6d, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x75,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101,
0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x75,
0x6d, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x75, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x72, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x75, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x72, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x75, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75,
0x73, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x72, 0x69,
0x65, 0x74, 0x75, 0x6d, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x64, 0x17e, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x101,
0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64,
0x17e, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x75,
0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x7a, 0x6f, 0x72, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x7a,
0x6f, 0x72, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x42, 0x75, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x6f, 0x6c, 0x12b, 0x76, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42, 0x72, 0x61, 0x7a,
0x12b, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x42, 0x72, 0x61, 0x7a, 0x12b, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42,
0x72, 0x61, 0x7a, 0x12b, 0x6c, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x42,
0x72, 0x75, 0x6e, 0x65, 0x6a, 0x61, 0x73, 0x20, 0x44, 0x61, 0x72, 0x75,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4b, 0x61, 0x62, 0x6f, 0x76, 0x65, 0x72, 0x64, 0x65, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x61, 0x62, 0x6f, 0x76, 0x65, 0x72,
0x64, 0x65, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4b, 0x61, 0x62, 0x6f, 0x76, 0x65, 0x72, 0x64,
0x65, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x10c, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x61, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x10c, 0x65, 0x74, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x10c, 0x65, 0x74, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c, 0x65, 0x74,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c, 0x12b, 0x6c, 0x65, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c, 0x12b, 0x6c, 0x65, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c,
0x12b, 0x6c, 0x65, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x136, 0x12b, 0x6e, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x136, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x136,
0x12b, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c, 0x6f, 0x69, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x10c,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x5a, 0x69, 0x65, 0x6d, 0x73, 0x76, 0x113, 0x74, 0x6b, 0x75, 0x20, 0x73,
0x61, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f,
0x6b, 0x6f, 0x73, 0x75, 0x20, 0x28, 0x4b, 0x12b, 0x6c, 0x69, 0x6e, 0x67,
0x61, 0x29, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75, 0x6b, 0x61, 0x20, 0x73, 0x61, 0x6c,
0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75, 0x6b, 0x61, 0x20,
0x73, 0x61, 0x6c, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75, 0x6b, 0x61, 0x20, 0x73, 0x61,
0x6c, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x75, 0x62,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x44, 0x65, 0x69, 0x76, 0x69, 0x73, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x44, 0x69, 0x6d, 0x6f, 0x6e, 0x64,
0x69, 0x72, 0x76, 0x69, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x74, 0x69, 0x6d, 0x6f,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x69, 0x65,
0x6c, 0x64, 0x69, 0x65, 0x6e, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x69, 0x65, 0x6c, 0x64, 0x69,
0x65, 0x6e, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x69,
0x65, 0x6c, 0x64, 0x69, 0x65, 0x6e, 0x75, 0x20, 0x73, 0x61, 0x6c, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101,
0x6c, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x65, 0x69, 0x72,
0x6f, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c,
0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73,
0x74, 0x72, 0x75, 0x6d, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d,
0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74,
0x72, 0x75, 0x6d, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x65, 0x69, 0x72, 0x6f, 0x70,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x20, 0x6a, 0x6f, 0x73,
0x6c, 0x61, 0x20, 0x28, 0x46, 0x45, 0x54, 0x29, 0x52, 0x69, 0x65, 0x74,
0x75, 0x6d, 0x65, 0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x65, 0x69, 0x72,
0x6f, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x65,
0x69, 0x72, 0x6f, 0x70, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x6f, 0x6c, 0x6b,
0x6c, 0x65, 0x6e, 0x64, 0x61, 0x20, 0x28, 0x4d, 0x61, 0x6c, 0x76, 0x69,
0x6e, 0x75, 0x29, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x61, 0x20,
0x28, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x75, 0x29, 0x20, 0x73, 0x61,
0x6c, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x61,
0x20, 0x28, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x75, 0x29, 0x20, 0x73,
0x61, 0x6c, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x69,
0x64, 0x17e, 0x69, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x47, 0x76, 0x69, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x44, 0x69, 0x65, 0x6e, 0x76, 0x69, 0x64, 0x6a, 0x16b, 0x72,
0x75, 0x20, 0x75, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74,
0x69, 0x6b, 0x61, 0x73, 0x20, 0x74, 0x65, 0x72, 0x69, 0x74, 0x6f, 0x72,
0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x47, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x47, 0x61, 0x6d, 0x62, 0x6a, 0x113, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x61, 0x20, 0x73,
0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x47, 0x72, 0x69,
0x6e, 0x69, 0x10d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41,
0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75,
0x73, 0x74, 0x72, 0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x67,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52,
0x69, 0x65, 0x74, 0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69, 0x65,
0x74, 0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65,
0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x67, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x72, 0x73,
0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x12b, 0x10d, 0x61, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x47, 0x61, 0x6a, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x75, 0x2013, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x48,
0x61, 0x76, 0x61, 0x6a, 0x75, 0x2013, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x75,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x75, 0x2013, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f,
0x6e, 0x67, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x48,
0x6f, 0x76, 0x64, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x6e, 0x64, 0x69, 0x6a,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6f,
0x6b, 0x65, 0x101, 0x6e, 0x61, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49,
0x6e, 0x64, 0x6f, 0x137, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x113, 0x7a, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x41, 0x75, 0x73, 0x74, 0x72, 0x75, 0x6d, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x113, 0x7a, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x52, 0x69, 0x65, 0x74, 0x75, 0x6d, 0x69, 0x6e, 0x64, 0x6f,
0x6e, 0x113, 0x7a, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x49, 0x72, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x49, 0x72, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x72, 0x101, 0x6e,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x113, 0x6c, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x113, 0x6c,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x113, 0x6c, 0x61, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4a, 0x61, 0x70, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4a, 0x61, 0x70, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61,
0x70, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x61, 0x73, 0x2d, 0x4b,
0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x61, 0x73, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61,
0x74, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x61, 0x73, 0x2d, 0x4b, 0x61,
0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x41, 0x75,
0x73, 0x74, 0x72, 0x75, 0x6d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74,
0x101, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x69,
0x65, 0x74, 0x75, 0x6d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x101,
0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x72,
0x65, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f,
0x72, 0x65, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x73, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x7a, 0x73, 0x74, 0x101,
0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x61, 0x69,
0x6e, 0x61, 0x73, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x48, 0x61, 0x76, 0x61,
0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x4c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x48, 0x61, 0x76, 0x61, 0x20, 0x73,
0x61, 0x6c, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x48,
0x61, 0x76, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d,
0x61, 0x6c, 0x61, 0x69, 0x7a, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4d, 0x61, 0x6c, 0x64, 0x12b, 0x76, 0x69, 0x6a, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x72, 0x137, 0x12b,
0x7a, 0x61, 0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4d, 0x101, 0x72, 0x161, 0x61, 0x6c, 0x61, 0x20, 0x73, 0x61, 0x6c,
0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x12b,
0x63, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d,
0x61, 0x75, 0x72, 0x12b, 0x63, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61,
0x75, 0x72, 0x12b, 0x63, 0x69, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x6f,
0x73, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x4b, 0x6c, 0x75, 0x73,
0x101, 0x20, 0x6f, 0x6b, 0x65, 0x101, 0x6e, 0x61, 0x20, 0x70, 0x69, 0x65,
0x6b, 0x72, 0x61, 0x73, 0x74, 0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x4b, 0x6c,
0x75, 0x73, 0x101, 0x20, 0x6f, 0x6b, 0x65, 0x101, 0x6e, 0x61, 0x20, 0x70,
0x69, 0x65, 0x6b, 0x72, 0x61, 0x73, 0x74, 0x65, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x4b, 0x6c, 0x75, 0x73, 0x101,
0x20, 0x6f, 0x6b, 0x65, 0x101, 0x6e, 0x61, 0x20, 0x70, 0x69, 0x65, 0x6b,
0x72, 0x61, 0x73, 0x74, 0x65, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x6c, 0x61, 0x6e,
0x62, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x73,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x76, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x76, 0x61, 0x73,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4d, 0x61, 0x73, 0x6b, 0x61, 0x76, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4d,
0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e,
0x65, 0x70, 0x101, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x4a, 0x61, 0x75, 0x6e, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x75,
0x6e, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x73,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4a, 0x61, 0x75, 0x6e, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61, 0x75, 0x6e, 0x7a, 0x113,
0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x4a, 0x61, 0x75, 0x6e, 0x7a, 0x113, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x73,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4a, 0x61, 0x75, 0x6e, 0x7a, 0x113, 0x6c, 0x61, 0x6e, 0x64, 0x65,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x145, 0x16b, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65,
0x6e, 0x64, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x145, 0x16b,
0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x61, 0x73, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x145, 0x16b, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4e, 0x69, 0x75, 0x65, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x61, 0x73, 0x20,
0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x61, 0x73, 0x20, 0x73, 0x61, 0x6c,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x61, 0x73,
0x20, 0x73, 0x61, 0x6c, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x75, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x146, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x75, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x146, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x75, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x146, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4f, 0x6d, 0x73,
0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x101,
0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x61, 0x6c, 0x61,
0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x61, 0x70, 0x75, 0x61,
0x2d, 0x4a, 0x61, 0x75, 0x6e, 0x67, 0x76, 0x69, 0x6e, 0x65, 0x6a, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67,
0x76, 0x61, 0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50,
0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x61,
0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x65,
0x72, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x50, 0x65, 0x72, 0x75,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x50, 0x65, 0x72, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x12b, 0x6e, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x12b, 0x6e, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x12b,
0x6e, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x46, 0x113, 0x6e, 0x69, 0x6b, 0x73, 0x61, 0x20,
0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x65,
0x6e, 0x70, 0x6a, 0x113, 0x72, 0x61, 0x73, 0x20, 0x75, 0x6e, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x53, 0x65, 0x6e, 0x70, 0x6a, 0x113, 0x72, 0x61, 0x73, 0x20,
0x75, 0x6e, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x61, 0x73,
0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x53, 0x65, 0x6e, 0x70, 0x6a, 0x113, 0x72, 0x61, 0x73, 0x20, 0x75,
0x6e, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x61, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x50, 0x69, 0x74, 0x6b, 0x113, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x113, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x50, 0x68, 0x65, 0x6e, 0x6a, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x65, 0x69, 0x6e, 0x6a,
0x6f, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x52, 0x6f,
0x74, 0x65, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53,
0x61, 0x68, 0x61, 0x6c, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x12b, 0x6e, 0x61, 0x73, 0x20,
0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x53, 0x61, 0x68, 0x61, 0x6c, 0x12b, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x65, 0x69, 0x161, 0x65, 0x13c, 0x75,
0x20, 0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x16b, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x5a, 0x101, 0x6c, 0x61, 0x6d, 0x61, 0x6e, 0x61, 0x20,
0x73, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x44, 0x69,
0x65, 0x6e, 0x76, 0x69, 0x64, 0x64, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69,
0x6a, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x53, 0x75, 0x72,
0x69, 0x6e, 0x61, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x160, 0x6f, 0x76, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54,
0x61, 0x69, 0x74, 0x69, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x61,
0x69, 0x62, 0x65, 0x69, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x61,
0x69, 0x62, 0x65, 0x69, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x61, 0x69, 0x62, 0x65, 0x69, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x10c, 0x16b, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69,
0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a,
0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65,
0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x61, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x7a, 0x69, 0x65, 0x6d,
0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x65, 0x6e, 0x65, 0x63, 0x75, 0x113,
0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61,
0x73, 0x20, 0x7a, 0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x73, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x65, 0x69, 0x6b, 0x61, 0x20, 0x73,
0x61, 0x6c, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x56, 0x6f,
0x6c, 0x69, 0x73, 0x61, 0x73, 0x20, 0x75, 0x6e, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x73, 0x20, 0x7a,
0x69, 0x65, 0x6d, 0x61, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x61, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x61, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x73, 0x4a, 0x75, 0x6b, 0x6f, 0x6e, 0x61, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x73, 0x4e, 0x74, 0xe1, 0x6e, 0x67, 0x6f, 0x20,
0x79, 0x61, 0x20, 0x4c, 0x75, 0x62, 0x75, 0x6d, 0x62, 0x61, 0x73, 0x68,
0x69, 0x4e, 0x74, 0xe1, 0x6e, 0x67, 0x6f, 0x20, 0x79, 0x61, 0x20, 0x41,
0x66, 0x72, 0xed, 0x6b, 0x61, 0x20, 0x79, 0x61, 0x20, 0x190, 0x301, 0x73,
0x69, 0x74, 0x61, 0x4e, 0x74, 0xe1, 0x6e, 0x67, 0x6f, 0x20, 0x79, 0x61,
0x20, 0x41, 0x66, 0x72, 0xed, 0x6b, 0x61, 0x20, 0x79, 0x61, 0x20, 0x53,
0x69, 0x64, 0x69, 0x4e, 0x74, 0xe1, 0x6e, 0x67, 0x6f, 0x20, 0x79, 0x61,
0x20, 0x4c, 0x6f, 0x6e, 0x64, 0x6f, 0x6e, 0x69, 0x4e, 0x74, 0xe1, 0x6e,
0x67, 0x6f, 0x20, 0x79, 0x61, 0x20, 0x53, 0x65, 0x79, 0x73, 0x68, 0x65,
0x6c, 0x41, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x69,
0x6e, 0x69, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b,
0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117, 0x73, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79,
0x74, 0x173, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x69, 0x65, 0x74, 0x173, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61,
0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x73,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6c, 0x69, 0x61, 0x73,
0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6c,
0x69, 0x61, 0x73, 0x6b, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6c, 0x69, 0x61,
0x73, 0x6b, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x117, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x117, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x117,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69,
0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x6f, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160,
0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x6f, 0x73, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x72, 0x79,
0x74, 0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x72, 0x79, 0x74,
0x173, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x6b, 0x61, 0x6c,
0x6e, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61,
0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f,
0x73, 0x20, 0x6b, 0x61, 0x6c, 0x6e, 0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61,
0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f,
0x73, 0x20, 0x6b, 0x61, 0x6c, 0x6e, 0x173, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69,
0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x6f, 0x73, 0x20, 0x52, 0x61, 0x6d, 0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x76,
0x61, 0x6e, 0x64, 0x65, 0x6e, 0x79, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x52, 0x61, 0x6d, 0x69,
0x6f, 0x6a, 0x6f, 0x20, 0x76, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x79, 0x6e,
0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72, 0x117, 0x73, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x73, 0x20, 0x52, 0x61, 0x6d, 0x69,
0x6f, 0x6a, 0x6f, 0x20, 0x76, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x79, 0x6e,
0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x117, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x117, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x117,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x70, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x70, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x41, 0x70, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b,
0x74, 0x61, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b,
0x74, 0x61, 0x75, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x117, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x117, 0x73, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x41, 0x6b, 0x74, 0x6f, 0x62, 0x117, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x72,
0x61, 0x62, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41,
0x72, 0x61, 0x62, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x6f,
0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x6f, 0x73, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56,
0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x72, 0x6d, 0x117, 0x6e,
0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41,
0x72, 0x6d, 0x117, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x72,
0x6d, 0x117, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x69, 0x6e, 0x117, 0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117, 0x73, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117, 0x73, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117, 0x73, 0x20, 0x76, 0x61, 0x6b, 0x61,
0x72, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x69, 0x6e, 0x117, 0x73, 0x20, 0x76, 0x61, 0x6b, 0x61, 0x72,
0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69, 0x6e, 0x117, 0x73,
0x20, 0x76, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79,
0x74, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79, 0x74,
0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61,
0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56,
0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72,
0x173, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64,
0x17e, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x61, 0x6e, 0x6f, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x17e, 0x61, 0x6e,
0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x41, 0x7a, 0x6f, 0x72, 0x173, 0x20, 0x53, 0x61,
0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x7a, 0x6f,
0x72, 0x173, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x41, 0x7a, 0x6f,
0x72, 0x173, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x42, 0x72, 0x61,
0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x6f, 0x73,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x6f, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x42, 0x72, 0x75, 0x6e, 0x117, 0x6a, 0x61, 0x75, 0x73,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x17d, 0x61, 0x6c, 0x69, 0x6f, 0x6a,
0x6f, 0x20, 0x4b, 0x79, 0x161, 0x75, 0x6c, 0x69, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x17d, 0x61, 0x6c, 0x69, 0x6f, 0x6a, 0x6f, 0x20,
0x4b, 0x79, 0x161, 0x75, 0x6c, 0x69, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x17d, 0x61, 0x6c,
0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x4b, 0x79, 0x161, 0x75, 0x6c, 0x69, 0x6f,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4b, 0x65, 0x69, 0x73, 0x69, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x10c, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x6f,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x6f, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c,
0x69, 0x6c, 0x117, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c,
0x69, 0x6c, 0x117, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c, 0x69, 0x6c, 0x117, 0x73, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4b, 0x69, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4b, 0x69, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4b, 0x69, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x10c, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x10c, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x61, 0x6c, 0x117, 0x64, 0x173, 0x20,
0x53, 0x61, 0x6c, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x173, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69,
0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4b, 0x75, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4b, 0x75, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c,
0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4b, 0x75, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173,
0x20, 0x70, 0x75, 0x73, 0x117, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x75, 0x62,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x75, 0x62,
0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4b, 0x75, 0x62, 0x6f, 0x73, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x44, 0x65, 0x69, 0x76, 0x69, 0x73, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x44, 0x69, 0x75, 0x6d, 0x6f, 0x6e, 0x6f, 0x20, 0x64, 0x2019,
0x55, 0x72, 0x76, 0x69, 0x6c, 0x69, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72,
0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x65, 0x6c, 0x79,
0x6b, 0x173, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x56, 0x65, 0x6c, 0x79, 0x6b, 0x173, 0x20, 0x73, 0x61,
0x6c, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x65, 0x6c, 0x79, 0x6b, 0x173, 0x20,
0x53, 0x61, 0x6c, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x45, 0x6b, 0x76, 0x61,
0x64, 0x6f, 0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56,
0x69, 0x64, 0x75, 0x72, 0x69, 0x6f, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x69, 0x64,
0x75, 0x72, 0x69, 0x6f, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x6f, 0x73,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x56, 0x69, 0x64, 0x75, 0x72, 0x69, 0x6f, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x6f, 0x6c, 0x69,
0x6d, 0x173, 0x6a, 0x173, 0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b,
0x61, 0x72, 0x173, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x6f, 0x73, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x173,
0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x6f, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x46, 0x69, 0x64, 0x17e, 0x69, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x69, 0x64,
0x17e, 0x69, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x72, 0x61, 0x6e, 0x63, 0x16b,
0x7a, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x47, 0x76, 0x69, 0x61, 0x6e, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x69, 0x65, 0x74,
0x173, 0x20, 0x50, 0x72, 0x61, 0x6e, 0x63, 0x16b, 0x7a, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x69, 0x72, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74,
0x69, 0x6e, 0x69, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x47, 0x61, 0x6d, 0x62, 0x79, 0x72, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f,
0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x47, 0x72, 0x69, 0x6e, 0x76, 0x69, 0x10d, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x72, 0x79, 0x74, 0x173, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x72,
0x79, 0x74, 0x173, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x6b, 0x61, 0x72,
0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x6b,
0x61, 0x72, 0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x6b, 0x61, 0x72, 0x173,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x47, 0x75, 0x61, 0x6d, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x50, 0x65, 0x72, 0x73, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x12f, 0x6c, 0x61, 0x6e, 0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x47, 0x61, 0x6a, 0x61, 0x6e, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x173, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x48, 0x61, 0x76, 0x61, 0x6a, 0x173, 0x2013, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x173, 0x2013, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x173, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x6b, 0x6f,
0x6e, 0x67, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f,
0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x6b,
0x6f, 0x6e, 0x67, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x6f,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x6f,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x6e,
0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x49, 0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x6e, 0x64,
0x65, 0x6e, 0x79, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x69,
0x6e, 0x117, 0x73, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79,
0x74, 0x173, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b,
0x61, 0x72, 0x173, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x6f,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x6f, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x6a, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4b, 0x61, 0x6d, 0x10d, 0x69, 0x61, 0x74, 0x6b,
0x6f, 0x73, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4b, 0x61, 0x6d, 0x10d, 0x69, 0x61, 0x74, 0x6b, 0x6f, 0x73, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4b, 0x61, 0x6d, 0x10d, 0x69, 0x61, 0x74, 0x6b, 0x6f,
0x73, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x79, 0x74, 0x173, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6b, 0x61, 0x72, 0x173,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x6f,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x6f, 0x72, 0x117, 0x6a,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x6f, 0x72,
0x117, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x6f, 0x72, 0x117, 0x6a, 0x6f,
0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x6a, 0x117, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4c,
0x61, 0x6e, 0x6b, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4c, 0x61, 0x69, 0x6e, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4c, 0x6f, 0x72, 0x64, 0x6f, 0x20, 0x48,
0x61, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4c, 0x6f, 0x72,
0x64, 0x6f, 0x20, 0x48, 0x61, 0x75, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4c, 0x6f, 0x72, 0x64,
0x6f, 0x20, 0x48, 0x61, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x6b, 0x61,
0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x6b, 0x61,
0x75, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6f, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d,
0x61, 0x6c, 0x61, 0x69, 0x7a, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x6c, 0x64, 0x79, 0x76, 0x173, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a,
0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4d, 0x61, 0x72, 0x161, 0x61, 0x6c, 0x6f, 0x20, 0x53, 0x61, 0x6c,
0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x63, 0x69, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x61, 0x75,
0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x61,
0x75, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6f, 0x6e, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x6f, 0x73, 0x20, 0x52, 0x61, 0x6d, 0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x76,
0x61, 0x6e, 0x64, 0x65, 0x6e, 0x79, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x73, 0x20,
0x52, 0x61, 0x6d, 0x69, 0x6f, 0x6a, 0x6f, 0x20, 0x76, 0x61, 0x6e, 0x64,
0x65, 0x6e, 0x79, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x6f, 0x73, 0x20, 0x52, 0x61, 0x6d, 0x69, 0x6f, 0x6a, 0x6f, 0x20,
0x76, 0x61, 0x6e, 0x64, 0x65, 0x6e, 0x79, 0x6e, 0x6f, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x73,
0x6b, 0x76, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d,
0x61, 0x73, 0x6b, 0x76, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x61, 0x73, 0x6b,
0x76, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4d, 0x69, 0x61, 0x6e, 0x6d, 0x61,
0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x65, 0x70,
0x61, 0x6c, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61,
0x75, 0x6a, 0x6f, 0x73, 0x69, 0x6f, 0x73, 0x20, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a, 0x6f, 0x73, 0x69, 0x6f, 0x73, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4e, 0x61, 0x75, 0x6a, 0x6f, 0x73, 0x69, 0x6f, 0x73, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4e, 0x61, 0x75, 0x6a, 0x6f, 0x73, 0x69, 0x6f, 0x73, 0x20, 0x5a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a, 0x6f, 0x73, 0x69, 0x6f,
0x73, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x4e, 0x61, 0x75, 0x6a, 0x6f, 0x73, 0x69, 0x6f, 0x73, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x6a, 0x6f, 0x73, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4e, 0x69, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e,
0x64, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x69, 0x75,
0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x6f, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4e, 0x69, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64,
0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4e, 0x69, 0x75, 0x6a, 0x117, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f,
0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c,
0x173, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x20,
0x53, 0x61, 0x6c, 0x173, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x6a, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x6a, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x160, 0x69, 0x61, 0x75, 0x72,
0x117, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x6f, 0x73, 0x20,
0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4f,
0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4f,
0x6d, 0x73, 0x6b, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e,
0x61, 0x75, 0x6a, 0x6f, 0x73, 0x69, 0x6f, 0x73, 0x20, 0x47, 0x76, 0x69,
0x6e, 0x117, 0x6a, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x75, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76,
0x61, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67,
0x76, 0x61, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x65, 0x72,
0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x65, 0x72, 0x75,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x50, 0x65, 0x72, 0x75, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x173, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x173, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x46, 0x65, 0x6e,
0x69, 0x6b, 0x73, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x53, 0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72,
0x6f, 0x20, 0x69, 0x72, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e,
0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x65, 0x6e, 0x20,
0x50, 0x6a, 0x65, 0x72, 0x6f, 0x20, 0x69, 0x72, 0x20, 0x4d, 0x69, 0x6b,
0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x65, 0x6e, 0x20, 0x50,
0x6a, 0x65, 0x72, 0x6f, 0x20, 0x69, 0x72, 0x20, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x69, 0x74, 0x6b, 0x65,
0x72, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x6f,
0x6e, 0x61, 0x70, 0x117, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x50, 0x63, 0x68, 0x65, 0x6e, 0x6a, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4b, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x52, 0x65,
0x75, 0x6e, 0x6a, 0x6f, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x6f,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x63, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x6f, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x65, 0x69, 0x161, 0x65, 0x6c,
0x69, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x16b, 0x72, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x53, 0x61, 0x6c, 0x69, 0x61, 0x6d, 0x6f, 0x6e, 0x6f, 0x20, 0x53,
0x61, 0x6c, 0x173, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x50, 0x69,
0x65, 0x74, 0x173, 0x20, 0x44, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x53, 0x75, 0x72,
0x69, 0x6e, 0x61, 0x6d, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x53, 0x69, 0x6f, 0x76, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x54, 0x61, 0x68, 0x69, 0x10d, 0x69, 0x6f, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x54, 0x61, 0x69, 0x70, 0x117, 0x6a, 0x61, 0x75, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x61, 0x69, 0x70, 0x117,
0x6a, 0x61, 0x75, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x61, 0x69, 0x70, 0x117, 0x6a,
0x61, 0x75, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x6f, 0x73, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x54, 0x6f, 0x6e, 0x67, 0x6f, 0x73, 0x20, 0x17e,
0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x54, 0x6f, 0x6e, 0x67, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72,
0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x10c, 0x75, 0x6b,
0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x117, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x117, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x117, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x61, 0x75, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a,
0x61, 0x75, 0x73, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a,
0x61, 0x75, 0x73, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20,
0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x20,
0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f,
0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69,
0x6b, 0x61, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x76, 0x61,
0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x56, 0x65, 0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73,
0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f,
0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c,
0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x20, 0x17e, 0x69, 0x65,
0x6d, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x20, 0x76, 0x61, 0x73,
0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x56, 0x65, 0x69, 0x6b, 0x6f, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x56, 0x6f, 0x6c, 0x69, 0x73,
0x6f, 0x20, 0x69, 0x72, 0x20, 0x46, 0x75, 0x74, 0x16b, 0x6e, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x61, 0x73, 0x61, 0x72, 0x6f, 0x73,
0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x20, 0x6c, 0x61,
0x69, 0x6b, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x20, 0x17e, 0x69, 0x65, 0x6d, 0x6f,
0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x20, 0x76,
0x61, 0x73, 0x61, 0x72, 0x6f, 0x73, 0x20, 0x6c, 0x61, 0x69, 0x6b, 0x61,
0x73, 0x4a, 0x75, 0x6b, 0x6f, 0x6e, 0x6f, 0x20, 0x6c, 0x61, 0x69, 0x6b,
0x61, 0x73, 0x74, 0x63, 0x69, 0x6b, 0x61, 0x20, 0x66, 0x6f, 0x20, 0x6c,
0x61, 0x20, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x69, 0x61, 0x73,
0x6e, 0x61, 0x6c, 0x63, 0x69, 0x73, 0x74, 0x63, 0x69, 0x6b, 0x61, 0x20,
0x66, 0x6f, 0x20, 0x6c, 0x61, 0x20, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x69, 0x61, 0x73, 0x63, 0x69, 0x73, 0x74, 0x63, 0x69, 0x6b, 0x61,
0x20, 0x66, 0x6f, 0x20, 0x6c, 0x61, 0x20, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x69, 0x61, 0x73, 0x74, 0x63, 0x69, 0x6b, 0x61, 0x20, 0x66,
0x6f, 0x20, 0x6c, 0x61, 0x20, 0x67, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x61,
0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3,
0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f,
0x142, 0x64, 0x6e, 0x6a, 0x6f, 0x77, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61,
0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f,
0x72, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f,
0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x6d,
0x61, 0x63, 0x6f, 0x6e, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x41, 0x6d, 0x61, 0x63, 0x6f, 0x6e, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x41, 0x6d, 0x61, 0x63, 0x6f, 0x6e, 0x61, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64,
0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e,
0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x79, 0x20, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50,
0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0xf3, 0x64, 0x7a, 0x61,
0x6a, 0x74, 0x161, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64,
0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74,
0x161, 0x6e, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142,
0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x70, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x79,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x67, 0xf3, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f,
0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x67, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x67, 0xf3, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63,
0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69,
0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x70,
0x6f, 0x142, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x41, 0x70, 0x69, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x41, 0x70, 0x69, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x41,
0x70, 0x69, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x72, 0x61, 0x62, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x72, 0x61, 0x62, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77,
0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3,
0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50,
0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x72,
0x6d, 0x65, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x72,
0x6d, 0x65, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x41, 0x72, 0x6d,
0x65, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65,
0x6a, 0x17a, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f,
0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x61, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a,
0x17a, 0x6f, 0x70, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e,
0x79, 0x20, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x70,
0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x79, 0x20, 0x61,
0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x70, 0xf3, 0x64, 0x77, 0x6a,
0x61, 0x63, 0x6f, 0x72, 0x6e, 0x79, 0x20, 0x61, 0x77, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a,
0x74, 0x161, 0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a,
0x74, 0x161, 0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74,
0x161, 0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e,
0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72,
0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72,
0x6e, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63,
0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61,
0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x63, 0x6f, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x41, 0x63, 0x6f, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x41, 0x63, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x161, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x42, 0x68, 0x75, 0x74, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x42, 0x6f, 0x6c, 0x69,
0x77, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x42, 0x72, 0x61,
0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x61, 0x73, 0x42, 0x72, 0x61,
0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x42, 0x72, 0x61, 0x73,
0xed, 0x6c, 0x69, 0x61, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x61, 0x70, 0x76, 0x65, 0x72,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x61, 0x70, 0x76,
0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x61, 0x70,
0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x43, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x43,
0x68, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x69,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x69, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63,
0x61, 0x73, 0x43, 0x68, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x6f, 0x69, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73,
0x20, 0x47, 0xf3, 0x64, 0x6f, 0x77, 0x6e, 0x79, 0x63, 0x68, 0x20, 0x6b,
0x75, 0x70, 0x6f, 0x77, 0x63, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77,
0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x77,
0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75,
0x70, 0x6f, 0x77, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x77, 0x79, 0x63,
0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x4b, 0x75, 0x62, 0x61, 0x144,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x144,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x144, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x44, 0x61, 0x76, 0x69, 0x73, 0x20, 0x63, 0x61, 0x73,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x44, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a,
0x74, 0x161, 0x6e, 0x6f, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x4a, 0x61, 0x74, 0x161,
0x6f, 0x77, 0x6e, 0x65, 0x6a, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x20, 0x4a, 0x61, 0x74, 0x161, 0x6f, 0x77, 0x6e, 0x65, 0x6a, 0x65,
0x20, 0x6b, 0x75, 0x70, 0x79, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x20, 0x4a, 0x61, 0x74, 0x161, 0x6f, 0x77,
0x6e, 0x65, 0x6a, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x45, 0x6b, 0x75,
0x61, 0x64, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53,
0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a,
0x65, 0x6a, 0x17a, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a,
0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x61, 0x6c,
0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72,
0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f,
0x72, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f,
0x72, 0x6e, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x46, 0x69, 0x64, 0x17a, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x46, 0x69, 0x64, 0x17a, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x46, 0x69, 0x64, 0x17a, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x6a, 0x73, 0x6b,
0x6f, 0x67, 0x75, 0x79, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x63, 0x61, 0x73, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x6a,
0x73, 0x6b, 0x65, 0x67, 0x6f, 0x20, 0x70, 0xf3, 0x64, 0x70, 0x6f, 0x142,
0x64, 0x6e, 0x6a, 0x6f, 0x77, 0x65, 0x67, 0x6f, 0x20, 0x61, 0x20, 0x61,
0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x6b, 0x65, 0x68, 0x6f,
0x20, 0x74, 0x65, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x75, 0x6d, 0x61,
0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70,
0x6f, 0x77, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a,
0x74, 0x161, 0x6e, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61,
0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61,
0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63,
0x6f, 0x72, 0x6e, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a,
0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x77,
0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x50,
0x65, 0x72, 0x73, 0x69, 0x73, 0x6b, 0x65, 0x67, 0x6f, 0x20, 0x67, 0x6f,
0x6c, 0x66, 0x61, 0x47, 0x75, 0x79, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x73, 0x6b, 0x6f,
0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x73, 0x6b, 0x6f, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x67, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x67,
0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x48, 0x6f,
0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68,
0x6f, 0x77, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68,
0x6f, 0x77, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x6f,
0x77, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x6b, 0x6f,
0x6f, 0x63, 0x65, 0x61, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x53, 0x72, 0x6a, 0x65, 0x6a, 0x17a, 0x6f, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e, 0x6f, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x6b, 0x69, 0x50, 0xf3, 0x64, 0x77, 0x6a,
0x61, 0x63, 0x6f, 0x72, 0x6e, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x72, 0x61, 0x144, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x72, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x63, 0x61, 0x73, 0x49, 0x72, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4a, 0x61, 0x70, 0x61, 0x144,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4a, 0x61, 0x70, 0x61, 0x144,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4a, 0x61, 0x70, 0x61, 0x144, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x50, 0xf3, 0x64, 0x7a, 0x61, 0x6a, 0x74, 0x161, 0x6e,
0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x50, 0xf3, 0x64, 0x77, 0x6a, 0x61, 0x63, 0x6f, 0x72, 0x6e,
0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x4c,
0x69, 0x6e, 0x69, 0x6a, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75,
0x70, 0x6f, 0x77, 0x63, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x20,
0x6b, 0x75, 0x70, 0x79, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f,
0x77, 0x65, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x2d, 0x48, 0x6f, 0x77, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x144,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x6c, 0x61, 0x6a,
0x7a, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x61, 0x6c,
0x65, 0x64, 0x69, 0x77, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d,
0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x63, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63,
0x61, 0x73, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x63, 0x61, 0x73, 0x4d,
0x65, 0x78, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x65, 0x78, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x63, 0x61, 0x73, 0x4d, 0x65, 0x78, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x55, 0x6c,
0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x2d,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b,
0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x6f, 0x73,
0x6b, 0x6f, 0x77, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x6f,
0x73, 0x6b, 0x6f, 0x77, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x6f,
0x73, 0x6b, 0x6f, 0x77, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x61,
0x75, 0x72, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f,
0x77, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x6b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4e,
0x6f, 0x77, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x65, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77,
0x6f, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63,
0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x66, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x4e, 0x6f, 0x77, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x66, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x69,
0x75, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73,
0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x63, 0x61, 0x73, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4e, 0x6f,
0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x63, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x4f,
0x6d, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61,
0x6c, 0x61, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61,
0x70, 0x75, 0x61, 0x2d, 0x4e, 0x6f, 0x77, 0x6f, 0x67, 0x69, 0x6e, 0x65,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x50, 0x65, 0x72, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x50, 0x65, 0x72, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x50, 0x65,
0x72, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x6f, 0x77, 0x79, 0x63, 0x68,
0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x53, 0x74, 0x2e, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x2d, 0x61, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x74,
0x2e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x61, 0x2d, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x53, 0x74, 0x2e, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d,
0x61, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69,
0x72, 0x6e, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f,
0x77, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x50, 0x6a, 0xf6, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x61, 0x63,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x53,
0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x65, 0x79, 0x63, 0x68,
0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x75, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x50, 0xf3, 0x64, 0x70, 0x6f, 0x142, 0x64, 0x6e, 0x6a, 0x6f,
0x77, 0x6f, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x61, 0x73, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0x63,
0x61, 0x73, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x54, 0x63, 0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x63, 0x68, 0x61, 0x6a, 0x70,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x54, 0x63, 0x68, 0x61,
0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x61, 0x64, 0x17a,
0x69, 0x6b, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61, 0x73, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x43, 0x68, 0x75, 0x75,
0x6b, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x63, 0x61, 0x73, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x63, 0x61, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x61, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61,
0x73, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x61, 0x73, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x57, 0x6c,
0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x63, 0x61, 0x73, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x57, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x63, 0x61, 0x73, 0x20, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x63, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x57, 0x61, 0x6b,
0x65, 0x63, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x15b, 0x6f, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x63, 0x61,
0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x15b, 0x6f, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x61, 0x73, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x61, 0x73, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63,
0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x4f, 0x6f, 0x73, 0x74, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69,
0x65, 0x74, 0x53, 0xf6, 0xf6, 0x64, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x57, 0x65,
0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63,
0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x57, 0x65,
0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63,
0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74,
0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20,
0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73,
0x63, 0x68, 0x20, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x63, 0x68, 0x20, 0x6f, 0x6f, 0x73, 0x74, 0x65, 0x6e, 0x20, 0x54,
0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x6f, 0x6f, 0x73,
0x74, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x6f, 0x6f,
0x73, 0x74, 0x65, 0x6e, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x42, 0x61, 0x72,
0x67, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x42,
0x61, 0x72, 0x67, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x42, 0x61,
0x72, 0x67, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65,
0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66,
0x69, 0x6b, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20,
0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63,
0x68, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x41, 0x72, 0x61, 0x61,
0x62, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x41, 0x72, 0x61,
0x61, 0x62, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x41, 0x72, 0x61, 0x61, 0x62, 0x73,
0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65,
0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x4e, 0x6f,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x63, 0x68, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b,
0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x5a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x5a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73,
0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65,
0x74, 0x57, 0x65, 0x73, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63, 0x68,
0x20, 0x54, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x7a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x74, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x7a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x65, 0x74, 0x4f, 0x6f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x4f,
0x6f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x73,
0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74,
0x69, 0x65, 0x74, 0x4f, 0x6f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65,
0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x74, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x61,
0x2d, 0x54, 0x69, 0x65, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x2d, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x43,
0x68, 0x69, 0x6e, 0x61, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x65, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69,
0x65, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x5a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4,
0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x65, 0x74, 0x4f, 0x6f, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4,
0xe4, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x4f, 0x6f, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74,
0x4f, 0x6f, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73,
0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65,
0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4,
0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x57, 0x65, 0x73, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63, 0x68, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x57,
0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0xe4, 0x73, 0x63,
0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74,
0x47, 0x72, 0xf6, 0xf6, 0x6e, 0x77, 0x69, 0x73, 0x63, 0x68, 0x2d, 0x57,
0x65, 0x6c, 0x74, 0x74, 0x69, 0x65, 0x74, 0x49, 0x6e, 0x64, 0x69, 0x65,
0x6e, 0x2d, 0x54, 0x69, 0x65, 0x74, 0x53, 0xf6, 0xf6, 0x64, 0x6f, 0x6f,
0x73, 0x74, 0x61, 0x73, 0x69, 0x61, 0x61, 0x74, 0x73, 0x63, 0x68, 0x20,
0x54, 0x69, 0x65, 0x74, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73,
0x63, 0x68, 0x20, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x74,
0x69, 0x65, 0x74, 0x4f, 0x6f, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x65, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x57, 0x65,
0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73, 0x63, 0x68,
0x20, 0x54, 0x69, 0x65, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x2d,
0x54, 0x69, 0x65, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x2d, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x65, 0x74, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x65, 0x74, 0x4a, 0x61, 0x70, 0x61, 0x61, 0x6e, 0x73, 0x63,
0x68, 0x20, 0x54, 0x69, 0x65, 0x74, 0x4a, 0x61, 0x70, 0x61, 0x61, 0x6e,
0x73, 0x63, 0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x74, 0x69, 0x65, 0x74, 0x4a, 0x61, 0x70, 0x61, 0x61, 0x6e, 0x73, 0x63,
0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68, 0x20, 0x54,
0x69, 0x65, 0x74, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x63,
0x68, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69,
0x65, 0x74, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x63, 0x68,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x4d,
0x6f, 0x73, 0x6b, 0x61, 0x75, 0x2d, 0x54, 0x69, 0x65, 0x74, 0x4d, 0x6f,
0x73, 0x6b, 0x61, 0x75, 0x2d, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x74, 0x69, 0x65, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x65, 0x74, 0x41, 0x63,
0x72, 0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x63,
0x72, 0x65, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x53,
0xfc, 0x64, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63,
0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4,
0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x49, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x49,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x49, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4,
0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4f, 0x73, 0x74, 0x6b, 0xfc,
0x73, 0x74, 0x65, 0x6e, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x4f, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63,
0x68, 0x20, 0x4f, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x52, 0x6f,
0x63, 0x6b, 0x79, 0x2d, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x2d, 0x4d,
0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x2d,
0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x57, 0x65, 0x73, 0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x7a, 0xe4,
0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x57, 0x65, 0x73, 0x74, 0x6b,
0xfc, 0x73, 0x74, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x57, 0x65, 0x73,
0x74, 0x6b, 0xfc, 0x73, 0x74, 0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x41, 0x72, 0x61, 0x62, 0x65, 0x73, 0x63, 0x68, 0x20,
0x5a, 0xe4, 0x69, 0x74, 0x41, 0x72, 0x61, 0x62, 0x65, 0x73, 0x63, 0x68,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41,
0x72, 0x61, 0x62, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73,
0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x73, 0x63,
0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x61, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73,
0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x65, 0x73, 0x63,
0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4,
0x69, 0x74, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4,
0x69, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73,
0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x5a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x5a, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x2f, 0x57, 0x65, 0x73, 0x74, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20,
0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61,
0x69, 0x64, 0x73, 0x63, 0x68, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x5a, 0xe4, 0x69, 0x74, 0x41, 0x73, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64,
0x73, 0x63, 0x68, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x41, 0x73, 0x65, 0x72,
0x62, 0x61, 0x69, 0x64, 0x73, 0x63, 0x68, 0x61, 0x6e, 0x65, 0x73, 0x63,
0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x42, 0x68, 0x75,
0x74, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x42, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x42, 0x72, 0x61,
0x73, 0xed, 0x6c, 0x69, 0x61, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x4b, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72,
0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x4b, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65,
0x73, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x43, 0x68,
0x69, 0x6e, 0x65, 0x73, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x69, 0x6e, 0x65,
0x73, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x6f, 0x69, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x43, 0x68, 0x72, 0xeb, 0x73, 0x63, 0x68, 0x74, 0x64, 0x61, 0x67,
0x73, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4b,
0x6f, 0x6b, 0x6f, 0x73, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x6f,
0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65,
0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x43, 0x6f, 0x6f, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x75,
0x62, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74,
0x4b, 0x75, 0x62, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x75, 0x62, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x44, 0x61, 0x76, 0x69, 0x73, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4f,
0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x4f, 0x75, 0x73, 0x63, 0x68, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x73, 0x65,
0x6c, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x75, 0x73, 0x63, 0x68, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x75, 0x73, 0x63, 0x68, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f,
0x72, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x4d, 0xeb, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4d, 0xeb,
0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73,
0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x4d, 0xeb, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68, 0x20,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68,
0x20, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe4, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x69, 0x64, 0x73, 0x63,
0x68, 0x69, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x46, 0x69, 0x64, 0x73, 0x63,
0x68, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x72, 0x61, 0x6e, 0x73,
0xe9, 0x69, 0x73, 0x63, 0x68, 0x2d, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e,
0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x46, 0x72, 0x61, 0x6e, 0x73, 0xe9,
0x69, 0x73, 0x63, 0x68, 0x20, 0x53, 0xfc, 0x64, 0x2d, 0x20, 0x61, 0x6e,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x67, 0x65,
0x62, 0x69, 0x64, 0x64, 0x65, 0x72, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x65, 0x73, 0x63, 0x68,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x2d, 0x49, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x4d, 0xeb, 0x74, 0x74, 0x6c, 0x65, 0x72, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69,
0x74, 0x57, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x67, 0x72,
0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf6,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x47, 0x75, 0x61, 0x6d, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x47, 0x6f, 0x6c, 0x66, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67, 0x2d, 0x4b,
0x6f, 0x6e, 0x67, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67,
0x2d, 0x4b, 0x6f, 0x6e, 0x67, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x48, 0x6f, 0x6e, 0x67, 0x2d, 0x4b, 0x6f, 0x6e,
0x67, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x48, 0x6f, 0x76, 0x64, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x48, 0x6f, 0x76,
0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x48, 0x6f, 0x76, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x49, 0x6e, 0x64, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x49, 0x6e, 0x64, 0x65, 0x73, 0x63, 0x68, 0x65, 0x6e,
0x20, 0x4f, 0x7a, 0x65, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x49,
0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74,
0x4f, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x65, 0x73,
0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x49, 0x72, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x5a, 0xe4, 0x69, 0x74, 0x49, 0x72, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x49,
0x72, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65, 0x73,
0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69,
0x74, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x61, 0x6d,
0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b,
0x61, 0x6d, 0x74, 0x73, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x73, 0x74,
0x6b, 0x61, 0x73, 0x61, 0x63, 0x68, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a,
0xe4, 0x69, 0x74, 0x57, 0x65, 0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x63,
0x68, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4c, 0x69,
0x6e, 0x6e, 0x65, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77,
0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48,
0x6f, 0x77, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4,
0x69, 0x74, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61,
0x6c, 0x61, 0x79, 0x73, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69,
0x74, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x6e, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c,
0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2d, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b,
0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x65,
0x73, 0x63, 0x68, 0x20, 0x50, 0x61, 0x7a, 0x69, 0x66, 0x69, 0x6b, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x65,
0x78, 0x69, 0x6b, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x50, 0x61,
0x7a, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61,
0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62,
0x61, 0x61, 0x74, 0x61, 0x72, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72,
0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65,
0x72, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4d, 0x79, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x65, 0x73,
0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69,
0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x6b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x73, 0xe9, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69,
0x73, 0xe9, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x73, 0xe9, 0x69,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x66, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x4e, 0x69, 0x75, 0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c,
0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d, 0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a,
0xe4, 0x69, 0x74, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x2d,
0x64, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4e, 0x6f, 0x77,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4e,
0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x6d, 0x73,
0x6b, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4f, 0x6d, 0x73,
0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x5a, 0xe4, 0x69, 0x74, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x2d, 0x5a, 0xe4, 0x69,
0x74, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x4e, 0x65, 0x69, 0x67, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20,
0x5a, 0xe4, 0x69, 0x74, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x50, 0x65, 0x72,
0x75, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x50, 0x65, 0x72, 0x75, 0x61, 0x6e,
0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x50, 0x68,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x50,
0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x6e, 0x65, 0x73, 0x63,
0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x69, 0x6e, 0x73, 0x65, 0x6c,
0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x61, 0x2d, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d,
0x61, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x61, 0x2d,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x50, 0x69, 0x74, 0x63, 0x61,
0x69, 0x72, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4,
0x69, 0x74, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x65, 0x79, 0x63,
0x68, 0x65, 0x6c, 0x6c, 0x65, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x2d, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x7a, 0xe4, 0x69, 0x74, 0x53, 0x61, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x69, 0x6e, 0x73, 0x65, 0x6c, 0x65, 0x6e, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x53, 0xfc, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x65,
0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x65, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x53, 0x79, 0x6f,
0x77, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a,
0xe4, 0x69, 0x74, 0x54, 0x61, 0x64, 0x73, 0x63, 0x68, 0x69, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x4e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x2d,
0x5a, 0xe4, 0x69, 0x74, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x54, 0x75, 0x76, 0x61, 0x6c,
0x75, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68, 0x20, 0x5a, 0xe4, 0x69, 0x74,
0x55, 0x72, 0x75, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x63, 0x68,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x55,
0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x55,
0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4,
0x69, 0x74, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x56, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x61, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x6c,
0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x2d, 0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74,
0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d,
0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x5a, 0xe4, 0x69, 0x74,
0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x57, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0xe4, 0x69, 0x74, 0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x5a,
0xe4, 0x69, 0x74, 0x57, 0x61, 0x6b, 0x65, 0x2d, 0x49, 0x6e, 0x73, 0x65,
0x6c, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73,
0x2d, 0x61, 0x2d, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x5a, 0xe4,
0x69, 0x74, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2d, 0x4e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4,
0x69, 0x74, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x65, 0x72, 0x67, 0x2d, 0x5a, 0xe4, 0x69, 0x74, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x65, 0x72, 0x67, 0x2d,
0x4e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x7a, 0xe4, 0x69, 0x74, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x65, 0x72, 0x67,
0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0xe4, 0x69, 0x74, 0x410,
0x43a, 0x440, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x432, 0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x421,
0x440, 0x435, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x443, 0x436, 0x43d, 0x43e, 0x430, 0x444,
0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x430, 0x20, 0x420, 0x435, 0x43f,
0x443, 0x431, 0x43b, 0x438, 0x43a, 0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x430,
0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x43b, 0x458, 0x430, 0x441,
0x43a, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x43b,
0x458, 0x430, 0x441, 0x43a, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441,
0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x43b, 0x458,
0x430, 0x441, 0x43a, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x41b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x426, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421,
0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a,
0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e,
0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20,
0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x418, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x418, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x41f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f,
0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f, 0x43b, 0x430, 0x43d,
0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f,
0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x41f, 0x430, 0x446, 0x438,
0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43d, 0x430, 0x434, 0x438, 0x440,
0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x410, 0x43f, 0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x43f, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x43f, 0x438,
0x458, 0x430, 0x410, 0x440, 0x430, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x430, 0x440, 0x430, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435,
0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432,
0x43e, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x437, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442,
0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x415, 0x440, 0x43c, 0x435,
0x43d, 0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x415, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x415,
0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x410, 0x442, 0x43b, 0x430, 0x43d,
0x442, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x426, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x430, 0x20, 0x438, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x430, 0x20, 0x438, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a,
0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e,
0x20, 0x432, 0x43e, 0x20, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x430, 0x20, 0x438, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435,
0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x437, 0x435, 0x440, 0x431,
0x435, 0x458, 0x45f, 0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x437, 0x435, 0x440,
0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x410, 0x437, 0x43e, 0x440,
0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x430, 0x43d, 0x433,
0x43b, 0x430, 0x434, 0x435, 0x448, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x443, 0x442, 0x430, 0x43d,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x43e, 0x43b,
0x438, 0x432, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x440, 0x430, 0x437, 0x438,
0x43b, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x411, 0x440, 0x430, 0x437,
0x438, 0x43b, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x458, 0x20, 0x414, 0x430, 0x440,
0x443, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41a, 0x430, 0x431, 0x43e, 0x20, 0x412, 0x435, 0x440, 0x434,
0x435, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x430, 0x431,
0x43e, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41a,
0x430, 0x431, 0x43e, 0x20, 0x412, 0x435, 0x440, 0x434, 0x435, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x43e,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x430, 0x442,
0x430, 0x43c, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x430,
0x442, 0x430, 0x43c, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435,
0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x430, 0x442, 0x430, 0x43c,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x438, 0x43b,
0x435, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x438, 0x43b,
0x435, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430,
0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442,
0x43e, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x438, 0x43b, 0x435, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x438, 0x43d, 0x430, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x438, 0x43d, 0x430, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432,
0x43e, 0x20, 0x41a, 0x438, 0x43d, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x427, 0x43e, 0x458, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x43e, 0x458, 0x431,
0x430, 0x43b, 0x441, 0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x43e, 0x458, 0x431,
0x430, 0x43b, 0x441, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x411, 0x43e, 0x436, 0x438, 0x45c, 0x435, 0x43d, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x438, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x43e, 0x43b, 0x443, 0x43c,
0x431, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x438, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x438, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x438, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41a, 0x443, 0x431, 0x430, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41a, 0x443, 0x431, 0x430, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20,
0x41a, 0x443, 0x431, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x414, 0x435, 0x458, 0x432, 0x438, 0x441, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x414, 0x438, 0x43c, 0x43e, 0x43d, 0x20, 0x414, 0x438,
0x440, 0x432, 0x438, 0x43b, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20, 0x422, 0x438, 0x43c,
0x43e, 0x440, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412,
0x435, 0x43b, 0x438, 0x433, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x412, 0x435, 0x43b, 0x438, 0x433, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x435,
0x43b, 0x438, 0x433, 0x434, 0x435, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x415, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440, 0x421, 0x440, 0x435, 0x434,
0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x43d, 0x43e, 0x435, 0x432,
0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421,
0x440, 0x435, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e,
0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x430, 0x43b,
0x438, 0x43d, 0x438, 0x43d, 0x433, 0x440, 0x430, 0x434, 0x441, 0x43a, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440,
0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x43e,
0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x438,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20,
0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x424, 0x438, 0x45f, 0x438, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x424, 0x438, 0x45f, 0x438, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424,
0x438, 0x45f, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x441, 0x43a, 0x430, 0x20, 0x413, 0x432,
0x430, 0x458, 0x430, 0x43d, 0x430, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x441,
0x43a, 0x43e, 0x20, 0x458, 0x443, 0x436, 0x43d, 0x43e, 0x20, 0x438, 0x20, 0x430,
0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x413, 0x430, 0x43c, 0x431, 0x435,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x413, 0x440, 0x443,
0x437, 0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x413, 0x440, 0x443, 0x437, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x413, 0x440,
0x443, 0x437, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x43e, 0x432, 0x438,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x421, 0x440, 0x435, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x43f, 0x43e, 0x20,
0x413, 0x440, 0x438, 0x43d, 0x438, 0x447, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x418,
0x441, 0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c,
0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x435, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x435, 0x440,
0x441, 0x438, 0x441, 0x43a, 0x438, 0x20, 0x417, 0x430, 0x43b, 0x438, 0x432, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x413, 0x432, 0x430, 0x458,
0x430, 0x43d, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x425, 0x430, 0x432, 0x430, 0x438, 0x20, 0x2d, 0x20, 0x410, 0x43b, 0x435, 0x443,
0x442, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x425, 0x430, 0x432, 0x430,
0x438, 0x20, 0x2d, 0x20, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x438,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20,
0x425, 0x430, 0x432, 0x430, 0x438, 0x20, 0x2d, 0x20, 0x410, 0x43b, 0x435, 0x443,
0x442, 0x441, 0x43a, 0x438, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x425, 0x43e, 0x43d,
0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x41b,
0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x425, 0x43e, 0x432, 0x434,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x425, 0x43e, 0x432, 0x434,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x425, 0x43e, 0x432, 0x434, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x430, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x43d, 0x434, 0x438, 0x441,
0x43a, 0x438, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x43d,
0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x426, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x430, 0x20, 0x418, 0x43d, 0x434, 0x43e,
0x43d, 0x435, 0x437, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x20, 0x418,
0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x430, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458, 0x430,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x430,
0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x430,
0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430,
0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442,
0x43e, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x430, 0x43d, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418, 0x440, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418,
0x437, 0x440, 0x430, 0x435, 0x43b, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x418, 0x437, 0x440, 0x430, 0x435, 0x43b, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x418,
0x437, 0x440, 0x430, 0x435, 0x43b, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x408, 0x430, 0x43f, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x430, 0x43f, 0x43e, 0x43d, 0x438,
0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442,
0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x430, 0x43f, 0x43e, 0x43d, 0x438,
0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x418,
0x441, 0x442, 0x43e, 0x447, 0x435, 0x43d, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445,
0x441, 0x442, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x435, 0x43d, 0x20, 0x41a, 0x430, 0x437,
0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41a, 0x43e, 0x440, 0x435, 0x458, 0x430, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x43e, 0x440, 0x435, 0x458, 0x430, 0x41b,
0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435,
0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20,
0x432, 0x43e, 0x20, 0x41a, 0x43e, 0x440, 0x435, 0x458, 0x430, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x43e, 0x441, 0x440, 0x430, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440,
0x441, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458,
0x430, 0x440, 0x441, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41b, 0x438, 0x43d, 0x438, 0x441,
0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x438, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x20,
0x425, 0x430, 0x443, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x430, 0x443, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x433, 0x430, 0x434,
0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x43b,
0x435, 0x437, 0x438, 0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41c, 0x430, 0x43b, 0x434, 0x438, 0x432, 0x438, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x440, 0x43a, 0x435, 0x441,
0x430, 0x441, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c,
0x430, 0x440, 0x448, 0x430, 0x43b, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x446, 0x438, 0x443, 0x441, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x446,
0x438, 0x443, 0x441, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435,
0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438,
0x446, 0x438, 0x443, 0x441, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41c, 0x43e, 0x441, 0x43e, 0x43d, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438,
0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x43e, 0x421, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444,
0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x43e, 0x41b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x435,
0x43a, 0x441, 0x438, 0x43a, 0x43e, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x43b, 0x430, 0x43d,
0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x43b, 0x430,
0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a,
0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e,
0x20, 0x432, 0x43e, 0x20, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41c, 0x458, 0x430, 0x43d, 0x43c,
0x430, 0x440, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d,
0x430, 0x443, 0x440, 0x443, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435,
0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434,
0x43e, 0x43d, 0x438, 0x458, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x430,
0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x20,
0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a,
0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e,
0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x20, 0x417, 0x435, 0x43b, 0x430,
0x43d, 0x434, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x40a,
0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d,
0x434, 0x43b, 0x435, 0x43d, 0x434, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441,
0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x40a, 0x443, 0x444,
0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x438, 0x443, 0x435, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b,
0x448, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b,
0x448, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x41b, 0x435,
0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20,
0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432,
0x43e, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x448, 0x43a, 0x438, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20,
0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x41b,
0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435,
0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20,
0x432, 0x43e, 0x20, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20,
0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438,
0x431, 0x438, 0x440, 0x441, 0x43a, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440,
0x441, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41e,
0x43c, 0x441, 0x43a, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41e,
0x43c, 0x441, 0x43a, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41e, 0x43c, 0x441, 0x43a, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f,
0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x41f, 0x430, 0x43b, 0x430, 0x443, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20, 0x41d,
0x43e, 0x432, 0x430, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435, 0x458, 0x430, 0x412,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x430, 0x440, 0x430,
0x433, 0x432, 0x430, 0x458, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20,
0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x435, 0x440, 0x443, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x435, 0x440, 0x443, 0x41b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e,
0x20, 0x41f, 0x435, 0x440, 0x443, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x421, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438,
0x43d, 0x438, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d,
0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x438, 0x442, 0x435, 0x20, 0x424, 0x435, 0x43d, 0x438,
0x43a, 0x441, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421,
0x435, 0x43d, 0x442, 0x20, 0x41f, 0x458, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x430, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x41f, 0x458, 0x435, 0x440, 0x20,
0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x430, 0x43d, 0x41b, 0x435, 0x442,
0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e,
0x20, 0x421, 0x435, 0x43d, 0x442, 0x20, 0x41f, 0x458, 0x435, 0x440, 0x20, 0x438,
0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x430, 0x43d, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x41f, 0x43e, 0x43d,
0x430, 0x43f, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458, 0x430, 0x43d, 0x433, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x420, 0x435, 0x458, 0x443, 0x43d, 0x438,
0x43e, 0x43d, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x420,
0x43e, 0x442, 0x435, 0x440, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x421, 0x430, 0x43a, 0x430, 0x43b, 0x438, 0x43d, 0x421, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x430, 0x43a, 0x430, 0x43b, 0x438, 0x43d,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x421, 0x430, 0x43a, 0x430, 0x43b, 0x438, 0x43d, 0x421, 0x430,
0x43c, 0x430, 0x440, 0x430, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441,
0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x442, 0x43e, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421,
0x430, 0x43c, 0x43e, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x421, 0x435, 0x458, 0x448, 0x435, 0x43b, 0x438, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443,
0x440, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421, 0x43e,
0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x441, 0x43a, 0x438, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x438, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x408, 0x443, 0x436, 0x43d, 0x430, 0x20, 0x413, 0x440, 0x443, 0x437, 0x438,
0x458, 0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x421,
0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x421, 0x430, 0x458, 0x43e, 0x432, 0x430, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x430, 0x445, 0x438, 0x442, 0x438,
0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x430, 0x458,
0x43f, 0x435, 0x458, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422,
0x430, 0x458, 0x43f, 0x435, 0x458, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441,
0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x430, 0x458,
0x43f, 0x435, 0x458, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x422, 0x430, 0x45f, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x43e, 0x43a, 0x435, 0x43b,
0x430, 0x443, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422,
0x43e, 0x43d, 0x433, 0x430, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x422, 0x43e, 0x43d, 0x433, 0x430, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x43e, 0x43d, 0x433,
0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x427, 0x443,
0x443, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422,
0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x421,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c,
0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x443,
0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x412, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x422, 0x443, 0x432, 0x430, 0x43b,
0x443, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x440,
0x443, 0x433, 0x432, 0x430, 0x458, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x458, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a, 0x435, 0x20, 0x43d, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e, 0x20, 0x432, 0x43e, 0x20,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x458, 0x412, 0x440, 0x435, 0x43c, 0x435,
0x20, 0x432, 0x43e, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x423, 0x437,
0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x423,
0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x412, 0x440, 0x435,
0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442,
0x443, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x430, 0x43d,
0x443, 0x430, 0x442, 0x443, 0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430,
0x442, 0x443, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412,
0x435, 0x43d, 0x435, 0x446, 0x443, 0x435, 0x43b, 0x430, 0x412, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e,
0x441, 0x442, 0x43e, 0x43a, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20,
0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x41b,
0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e,
0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x43e,
0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x421, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434,
0x41b, 0x435, 0x442, 0x43d, 0x43e, 0x20, 0x441, 0x43c, 0x435, 0x442, 0x430, 0x45a,
0x435, 0x20, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x442, 0x43e,
0x20, 0x432, 0x43e, 0x20, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430,
0x434, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412, 0x43e,
0x441, 0x442, 0x43e, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x43e, 0x442, 0x20, 0x412, 0x435,
0x458, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x412,
0x430, 0x43b, 0x438, 0x441, 0x20, 0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d,
0x430, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x408, 0x430,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e,
0x20, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x41b, 0x435, 0x442, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x408,
0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20,
0x432, 0x43e, 0x20, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431,
0x443, 0x440, 0x433, 0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x415,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x41b,
0x435, 0x442, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432,
0x43e, 0x20, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443,
0x440, 0x433, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x432, 0x43e, 0x20, 0x408,
0x443, 0x43a, 0x43e, 0x43d, 0x905, 0x92b, 0x917, 0x93e, 0x928, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x927, 0x94d, 0x92f,
0x20, 0x905, 0x92b, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x940, 0x915,
0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923,
0x20, 0x905, 0x92b, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x20, 0x905, 0x92b, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x94d, 0x930,
0x940, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x940, 0x915,
0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x932,
0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x92e, 0x947, 0x91c,
0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x92e, 0x947, 0x91c, 0x928, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x92e, 0x947,
0x91c, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905,
0x930, 0x947, 0x92c, 0x93f, 0x92f, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905,
0x930, 0x947, 0x92c, 0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x930, 0x947, 0x92c, 0x93f, 0x92f, 0x928, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905,
0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x930,
0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940,
0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20,
0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940,
0x928, 0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906,
0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x930, 0x94d, 0x92e,
0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x906, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20,
0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x908, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92e, 0x927, 0x94d, 0x92f,
0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e,
0x908, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908,
0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20, 0x906, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20,
0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x906, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905,
0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c,
0x93e, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x90f,
0x91c, 0x94b, 0x930, 0x94d, 0x938, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x90f, 0x91c,
0x94b, 0x930, 0x94d, 0x938, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x90f, 0x91c, 0x94b, 0x930, 0x94d, 0x938, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947,
0x936, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x902, 0x917, 0x932, 0x93e, 0x926,
0x947, 0x936, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92c, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x20, 0x938, 0x92e, 0x930,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92d, 0x942, 0x91f, 0x93e, 0x928, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92c, 0x94b, 0x932, 0x940, 0x92c, 0x93f, 0x92f, 0x93e, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932, 0x93f,
0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c,
0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x940, 0x932, 0x93f, 0x92f,
0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94d,
0x930, 0x942, 0x928, 0x947, 0x908, 0x20, 0x926, 0x93e, 0x930, 0x947, 0x938, 0x932,
0x93e, 0x92e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x92c,
0x930, 0x94d, 0x921, 0x947, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x947, 0x92a,
0x20, 0x92c, 0x930, 0x94d, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x92c, 0x930, 0x94d, 0x921, 0x20,
0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x947, 0x925, 0x92e,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x947, 0x925, 0x92e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x947, 0x925, 0x92e, 0x20,
0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a,
0x93f, 0x932, 0x940, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x93f, 0x932, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x93f,
0x932, 0x940, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a,
0x940, 0x928, 0x940, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x940, 0x928, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x924, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x940,
0x928, 0x940, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x91a, 0x94b, 0x907, 0x92c, 0x932, 0x938, 0x947, 0x928, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x91a, 0x94b, 0x907, 0x92c, 0x932, 0x938, 0x947, 0x928, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91a, 0x94b, 0x907,
0x92c, 0x932, 0x938, 0x947, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x915, 0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938, 0x20, 0x926, 0x94d,
0x935, 0x940, 0x92a, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94b, 0x915, 0x94b,
0x938, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94b, 0x932, 0x902, 0x92c,
0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915,
0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x921, 0x947,
0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930,
0x92c, 0x940, 0x20, 0x924, 0x93f, 0x92e, 0x94b, 0x930, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x906, 0x907, 0x932, 0x948, 0x902,
0x921, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20,
0x906, 0x907, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x906, 0x907,
0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x907, 0x915, 0x94d, 0x935, 0x93e, 0x921, 0x94b, 0x930, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92b, 0x930, 0x926, 0x930, 0x2d, 0x908, 0x938, 0x94d, 0x91f, 0x930,
0x94d, 0x928, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x928, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93e, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20,
0x906, 0x907, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b,
0x93e, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x906, 0x907, 0x932, 0x948, 0x902,
0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b,
0x93e, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x906, 0x907, 0x932, 0x948, 0x902,
0x921, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x94d,
0x930, 0x947, 0x902, 0x91a, 0x20, 0x917, 0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93c, 0x94d, 0x930, 0x93e, 0x902, 0x938, 0x940,
0x938, 0x940, 0x20, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x940, 0x20, 0x915,
0x94d, 0x937, 0x947, 0x924, 0x94d, 0x930, 0x20, 0x906, 0x20, 0x905, 0x902, 0x91f,
0x93e, 0x930, 0x94d, 0x915, 0x91f, 0x93f, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x917, 0x948, 0x932, 0x93e, 0x92a, 0x948, 0x917, 0x94b, 0x938, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x91c, 0x93e, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x91c, 0x93e, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91c, 0x93e, 0x930,
0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942,
0x930, 0x92c, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902,
0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a,
0x942, 0x930, 0x92c, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948,
0x902, 0x921, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948,
0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x917, 0x932, 0x94d, 0x92b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x94d, 0x92f,
0x942, 0x91f, 0x93f, 0x92f, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x935,
0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x94d, 0x92f, 0x942, 0x91f, 0x93f, 0x92f, 0x928,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x935,
0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x94d, 0x92f, 0x942, 0x91f, 0x93f, 0x92f, 0x928,
0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x939, 0x93e, 0x902, 0x917, 0x20, 0x915, 0x93e, 0x902, 0x917, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x939, 0x93e, 0x902, 0x917, 0x20, 0x915, 0x93e, 0x902, 0x917, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x93e, 0x902,
0x917, 0x20, 0x915, 0x93e, 0x902, 0x917, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x939, 0x94b, 0x92c, 0x94d, 0x926, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x939, 0x94b, 0x92c, 0x94d, 0x926, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x94b, 0x92c, 0x94d, 0x926, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x939, 0x93f, 0x902, 0x926, 0x20, 0x92e,
0x939, 0x93e, 0x938, 0x93e, 0x917, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x907,
0x902, 0x921, 0x94b, 0x91a, 0x93e, 0x907, 0x928, 0x93e, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947,
0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x942, 0x930,
0x92c, 0x940, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f,
0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x908, 0x930, 0x93e, 0x928, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x908, 0x930, 0x93e, 0x928, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x908, 0x930, 0x93e, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x930, 0x915, 0x942, 0x924,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x930, 0x915,
0x942, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x930, 0x915, 0x942, 0x924, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x91c,
0x930, 0x93e, 0x92f, 0x932, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x907, 0x91c, 0x930,
0x93e, 0x92f, 0x932, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x907, 0x91c, 0x930, 0x93e, 0x92f, 0x932, 0x20, 0x921, 0x947, 0x932, 0x93e,
0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e,
0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92a, 0x942, 0x930, 0x92c, 0x20, 0x915, 0x91c, 0x93e, 0x916, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x20, 0x915, 0x91c, 0x93e, 0x916, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94b, 0x930, 0x93f,
0x92f, 0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x915, 0x94d, 0x930, 0x947, 0x938, 0x94d, 0x928, 0x94b, 0x92f, 0x93e,
0x930, 0x94d, 0x938, 0x94d, 0x915, 0x915, 0x94d, 0x930, 0x947, 0x938, 0x94d, 0x928,
0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x94d, 0x930, 0x947, 0x938, 0x94d,
0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x915, 0x93f, 0x930, 0x94d, 0x917, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x932, 0x93e, 0x930, 0x94d, 0x921, 0x20, 0x939,
0x94b, 0x92c, 0x947, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x932, 0x93e, 0x930, 0x94d,
0x921, 0x20, 0x939, 0x94b, 0x92c, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x932, 0x93e, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b,
0x92c, 0x947, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x92e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20,
0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x932, 0x947, 0x936,
0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x93e, 0x932, 0x926,
0x940, 0x935, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x93e, 0x930, 0x940, 0x936,
0x938, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x93e, 0x930, 0x940, 0x936, 0x938,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x93e,
0x930, 0x940, 0x936, 0x938, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92e, 0x948, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x947,
0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x948,
0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x947, 0x938, 0x93f, 0x92b,
0x93f, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92e, 0x948, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x947, 0x938,
0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x932, 0x928, 0x92c, 0x91f, 0x94b, 0x930, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x932, 0x928, 0x92c, 0x91f, 0x94b, 0x930, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x932, 0x928,
0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x93e, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x938,
0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92e, 0x92f, 0x928, 0x92e, 0x93e,
0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x947, 0x92a, 0x93e, 0x932, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x947, 0x932,
0x948, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x928,
0x94d, 0x92f, 0x942, 0x20, 0x915, 0x947, 0x932, 0x948, 0x921, 0x94b, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928,
0x94d, 0x92f, 0x942, 0x20, 0x915, 0x947, 0x932, 0x948, 0x921, 0x94b, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x938, 0x92e,
0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x940, 0x932, 0x948,
0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94d,
0x92f, 0x942, 0x92b, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902, 0x921, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93e, 0x909, 0x902,
0x921, 0x932, 0x948, 0x902, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93e, 0x909, 0x902, 0x921,
0x932, 0x948, 0x902, 0x921, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93e, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20,
0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93e,
0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93e, 0x915,
0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x94d, 0x935, 0x940, 0x92a, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x930,
0x94d, 0x928, 0x947, 0x902, 0x921, 0x94b, 0x20, 0x921, 0x940, 0x20, 0x928, 0x94b,
0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b,
0x930, 0x94d, 0x928, 0x947, 0x902, 0x921, 0x94b, 0x20, 0x921, 0x940, 0x20, 0x928,
0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x930, 0x94d, 0x928, 0x947, 0x902, 0x921,
0x94b, 0x20, 0x921, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939,
0x93e, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x928, 0x94b,
0x92c, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92a, 0x928, 0x94b, 0x92c, 0x94b, 0x938, 0x93f, 0x92c, 0x93f,
0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x928, 0x94b, 0x92c, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938,
0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x93e, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x930,
0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x930,
0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92a, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x938,
0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x947, 0x930, 0x942, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x938,
0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93f, 0x932, 0x93f, 0x92a,
0x940, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92b, 0x93f, 0x932, 0x93f, 0x92a,
0x940, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f,
0x930, 0x947, 0x20, 0x906, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932,
0x93e, 0x928, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x947, 0x902, 0x91f, 0x20,
0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x906, 0x20, 0x92e, 0x93f, 0x915, 0x94d,
0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930,
0x947, 0x20, 0x906, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e,
0x928, 0x20, 0x921, 0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92a, 0x94d, 0x92f, 0x94b, 0x902, 0x917, 0x92f, 0x93e, 0x902, 0x917, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x930, 0x93f, 0x92f, 0x942, 0x928, 0x93f, 0x92f, 0x928,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x915, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938, 0x915, 0x93e, 0x932,
0x93f, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938,
0x93e, 0x92e, 0x94b, 0x906, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x92e, 0x94b,
0x906, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e,
0x92e, 0x94b, 0x906, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x938,
0x947, 0x936, 0x947, 0x932, 0x94d, 0x938, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x938,
0x93f, 0x902, 0x917, 0x93e, 0x92a, 0x941, 0x930, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20,
0x91c, 0x93e, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x938, 0x941, 0x930, 0x940, 0x928, 0x93e, 0x92e, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x908, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x924, 0x93e, 0x907, 0x92a, 0x947, 0x908, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x908, 0x20, 0x921,
0x947, 0x932, 0x93e, 0x907, 0x91f, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x924, 0x941,
0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947,
0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947,
0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x930, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x930, 0x941,
0x917, 0x94d, 0x935, 0x947, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x909, 0x91c, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x909, 0x91c, 0x92c, 0x947, 0x915, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x93e,
0x907, 0x92e, 0x909, 0x91c, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x947,
0x928, 0x947, 0x91c, 0x941, 0x90f, 0x932, 0x93e, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92c, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92c, 0x94b, 0x938, 0x94d, 0x924, 0x94b,
0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94d, 0x932, 0x93e, 0x926, 0x93f,
0x92c, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92c,
0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92c, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930,
0x93e, 0x926, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94b, 0x932, 0x94d, 0x917,
0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x93e, 0x907, 0x92e, 0x92c, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d,
0x930, 0x93e, 0x926, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92f, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x92f, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92f, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f, 0x93e, 0x907, 0x92e,
0x92f, 0x947, 0x915, 0x93e, 0x91f, 0x948, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d,
0x917, 0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x91f, 0x948,
0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x93e, 0x907, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x91f, 0x948, 0x930,
0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x938, 0x92e, 0x930, 0x20, 0x91f,
0x93e, 0x907, 0x92e, 0x92f, 0x942, 0x915, 0x94b, 0x928, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0xd0e, 0xd2f, 0xd4d, 0xd15, 0xd4d, 0xd15, 0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd0e, 0xd2f, 0xd4d, 0xd15, 0xd4d, 0xd15, 0xd7c, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd0e, 0xd2f, 0xd4d, 0xd15, 0xd4d, 0xd15, 0xd7c, 0x20, 0xd35, 0xd47,
0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd2b, 0xd4d, 0x200c, 0xd17, 0xd3e, 0xd28, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e,
0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd27, 0xd4d, 0xd2f, 0x20, 0xd06,
0xd2b, 0xd4d, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd06, 0xd2b, 0xd4d, 0xd30,
0xd3f, 0xd15, 0xd4d, 0xd15, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd26, 0xd15, 0xd4d,
0xd37, 0xd3f, 0xd23, 0xd3e, 0xd2b, 0xd4d, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e,
0xd31, 0xd7b, 0x20, 0xd06, 0xd2b, 0xd4d, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31,
0xd7b, 0x20, 0xd06, 0xd2b, 0xd4d, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31,
0xd7b, 0x20, 0xd06, 0xd2b, 0xd4d, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0x20, 0xd17,
0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd05, 0xd32, 0xd3e, 0xd38, 0xd4d, 0x200c, 0xd15, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd05, 0xd32, 0xd3e, 0xd38, 0xd4d, 0xd15, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd05, 0xd32, 0xd3e, 0xd38, 0xd4d, 0x200c, 0xd15, 0x20, 0xd21,
0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd7d, 0xd2e, 0xd24, 0xd3f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7d,
0xd2e, 0xd24, 0xd3f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7d, 0xd2e,
0xd24, 0xd3f, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd06, 0xd2e, 0xd38, 0xd4b, 0xd7a, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd06, 0xd2e, 0xd38, 0xd4b, 0xd7a, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd06, 0xd2e, 0xd38, 0xd4b, 0xd7a, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f,
0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20,
0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd38, 0xd46,
0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35,
0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15,
0xd4d, 0xd15, 0xd7b, 0x20, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20,
0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30,
0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15,
0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46,
0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd15,
0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f,
0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b,
0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47,
0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2e, 0xd4c, 0xd23, 0xd4d, 0xd1f,
0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46,
0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2e,
0xd57, 0xd23, 0xd4d, 0xd1f, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35,
0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15,
0xd4d, 0xd15, 0xd7b, 0x20, 0xd2e, 0xd57, 0xd23, 0xd4d, 0xd1f, 0xd7b, 0x20, 0xd21,
0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f,
0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2a, 0xd38, 0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15, 0xd4d, 0xd15, 0xd46, 0x20, 0xd05,
0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2a, 0xd38, 0xd2b,
0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd1f, 0xd15,
0xd4d, 0xd15, 0xd46, 0x20, 0xd05, 0xd2e, 0xd47, 0xd30, 0xd3f, 0xd15, 0xd4d, 0xd15,
0xd7b, 0x20, 0xd2a, 0xd38, 0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32,
0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd28,
0xd3e, 0xd21, 0xd3f, 0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd28, 0xd3e,
0xd21, 0xd3f, 0xd7c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd28, 0xd3e,
0xd21, 0xd3f, 0xd7c, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd2a, 0xd3f, 0xd2f, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd05, 0xd2a, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd05, 0xd2a, 0xd3f, 0xd2f, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd16, 0xd4d, 0xd24, 0xd4c,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd16, 0xd4d, 0xd24, 0xd4c, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd16, 0xd4d, 0xd24, 0xd4c, 0x20, 0xd35, 0xd47,
0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd16, 0xd4d, 0xd24, 0xd4b, 0xd2c, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd16, 0xd4d, 0xd24, 0xd4b, 0xd2c, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd05, 0xd16, 0xd4d, 0xd24, 0xd4b, 0xd2c, 0xd4d, 0x20, 0xd35, 0xd47, 0xd28,
0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05,
0xd31, 0xd47, 0xd2c, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05,
0xd31, 0xd47, 0xd2c, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd31, 0xd47, 0xd2c, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48,
0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7c, 0xd1c,
0xd28, 0xd4d, 0xd31, 0xd40, 0xd28, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd7c,
0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40, 0xd28, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd7c, 0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40, 0xd28, 0x20, 0xd17, 0xd4d, 0xd30,
0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd05,
0xd7c, 0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40, 0xd28, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd05, 0xd7c,
0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40, 0xd28, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd05, 0xd7c,
0xd1c, 0xd28, 0xd4d, 0xd31, 0xd40, 0xd28, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05,
0xd7c, 0xd2e, 0xd47, 0xd28, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05,
0xd7c, 0xd2e, 0xd47, 0xd28, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd7c, 0xd2e, 0xd47, 0xd28, 0xd3f, 0xd2f, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd05, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd3f,
0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd31, 0xd4d, 0xd31, 0xd4d,
0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd05, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28,
0xd4d, 0xd31, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d,
0xd30, 0xd7d, 0x20, 0xd13, 0xd38, 0xd4d, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f,
0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0xd1f, 0xd4d, 0xd30,
0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30,
0xd7d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0xd1f, 0xd4d,
0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d,
0xd30, 0xd7d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32,
0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20,
0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f,
0xd7b, 0x20, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd2a, 0xd1f,
0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd13, 0xd38, 0xd4d, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b,
0x20, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd2a, 0xd1f, 0xd3f,
0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31,
0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15,
0xd4d, 0xd15, 0xd7b, 0x20, 0xd13, 0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30, 0xd47,
0xd32, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0x200c,
0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd15, 0xd3f, 0xd34,
0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38,
0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd15,
0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31,
0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e,
0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd13, 0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d,
0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd38,
0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd2a,
0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd13, 0xd38, 0xd4d, 0x200c, 0xd1f, 0xd4d, 0xd30, 0xd47, 0xd32, 0xd3f,
0xd2f, 0xd7b, 0x20, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b,
0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd05, 0xd38, 0xd7c, 0xd2c, 0xd48, 0xd1c, 0xd3e, 0xd7b, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd05, 0xd38, 0xd7c, 0xd2c, 0xd48, 0xd1c, 0xd3e, 0xd7b, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd38, 0xd7c, 0xd2c, 0xd48, 0xd1c, 0xd3e,
0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd38, 0xd4b, 0xd7c, 0xd38, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd38, 0xd4b, 0xd7c, 0xd38, 0xd4d, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd05, 0xd38, 0xd4b, 0xd7c, 0xd38, 0xd4d, 0x20,
0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd02, 0xd17, 0xd4d, 0xd32, 0xd3e, 0xd26, 0xd47,
0xd36, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd02, 0xd17, 0xd4d, 0xd32,
0xd3e, 0xd26, 0xd47, 0xd36, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c,
0xd02, 0xd17, 0xd4d, 0xd32, 0xd3e, 0xd26, 0xd47, 0xd36, 0xd4d, 0x20, 0xd17, 0xd4d,
0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2d, 0xd42, 0xd1f, 0xd4d, 0xd1f, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2c, 0xd4a, 0xd33, 0xd40, 0xd35, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2c, 0xd4d, 0xd30, 0xd38, 0xd40, 0xd32, 0xd3f, 0xd2f, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd4d, 0xd30, 0xd38, 0xd40, 0xd32, 0xd3f, 0xd2f, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd4d, 0xd30, 0xd38, 0xd40, 0xd32, 0xd3f,
0xd2f, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2c, 0xd4d, 0xd30, 0xd42, 0xd23, 0xd48,
0x20, 0xd26, 0xd3e, 0xd31, 0xd41, 0xd38, 0xd4d, 0xd38, 0xd32, 0xd3e, 0xd02, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd47, 0xd2a, 0xd4d, 0x20, 0xd35, 0xd46, 0xd7c,
0xd26, 0xd46, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd47, 0xd2a, 0xd4d, 0x20,
0xd35, 0xd46, 0xd7c, 0xd26, 0xd46, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15,
0xd47, 0xd2a, 0xd4d, 0x20, 0xd35, 0xd46, 0xd7c, 0xd26, 0xd46, 0x20, 0xd17, 0xd4d,
0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd1a, 0xd2e, 0xd4b, 0xd31, 0xd4b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd1a, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd02, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd1a, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd02, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd1a, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd02, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a,
0xd3f, 0xd32, 0xd3f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd3f, 0xd32, 0xd3f,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd3f, 0xd32, 0xd3f, 0x20, 0xd17,
0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd48, 0xd28, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a,
0xd48, 0xd28, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd48, 0xd28, 0x20,
0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd1a, 0xd4b, 0xd2f, 0xd3f, 0x200d, 0xd2c, 0xd7d, 0xd38, 0xd3e, 0xd7b, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd4b, 0xd2f, 0xd4d, 0x200c, 0xd2c, 0xd7d, 0xd38,
0xd3e, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd4b, 0xd2f, 0xd3f,
0xd2c, 0xd7d, 0xd38, 0xd3e, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d,
0xd30, 0xd3f, 0xd38, 0xd4d, 0x200c, 0xd2e, 0xd38, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35,
0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd15, 0xd4d,
0xd15, 0xd4b, 0xd38, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15,
0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd33, 0xd02, 0xd2c, 0xd3f,
0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd33, 0xd02, 0xd2c, 0xd3f,
0xd2f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd33, 0xd02, 0xd2c,
0xd3f, 0xd2f, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd41, 0xd15, 0xd4d, 0xd15,
0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd15, 0xd41, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d,
0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd15, 0xd41, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a,
0xd41, 0xd15, 0xd7e, 0x20, 0xd05, 0xd7c, 0xd26, 0xd4d, 0xd27, 0x20, 0xd17, 0xd4d,
0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd15, 0xd4d, 0xd2f, 0xd42, 0xd2c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd15, 0xd4d, 0xd2f, 0xd42, 0xd2c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15,
0xd4d, 0xd2f, 0xd42, 0xd2c, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd21, 0xd47, 0xd35, 0xd3f, 0xd38, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd21, 0xd41, 0xd2e, 0xd4b, 0xd23, 0xd4d, 0xd1f,
0xd4d, 0x20, 0xd21, 0xd3f, 0x20, 0xd09, 0xd7c, 0xd35, 0xd3f, 0xd32, 0xd4d, 0xd32,
0xd46, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15,
0xd7b, 0x20, 0xd24, 0xd3f, 0xd2e, 0xd4b, 0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd08, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd7c, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40,
0xd2a, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd08, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd7c, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd08, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd7c, 0x20, 0xd26,
0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd15,
0xd4d, 0xd35, 0xd21, 0xd4b, 0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46,
0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d,
0xd2f, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d,
0xd30, 0xd7d, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd7b, 0xd1f, 0xd4d, 0xd30, 0xd7d,
0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd17, 0xd4d,
0xd30, 0xd40, 0xd37, 0xd4d, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2f, 0xd42, 0xd31,
0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f,
0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d,
0xd2f, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15,
0xd4d, 0xd15, 0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d, 0xd2f, 0xd7b,
0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd47, 0xd2f,
0xd31, 0xd4d, 0xd31, 0xd24, 0xd4d, 0xd24, 0xd46, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b,
0xd2a, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f,
0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a,
0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e,
0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d,
0xd2f, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e,
0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd2f, 0xd42, 0xd31, 0xd4b, 0xd2a, 0xd4d,
0xd2f, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3e, 0xd15, 0xd4d, 0xd15,
0xd4d, 0x200c, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40,
0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3e, 0xd15,
0xd4d, 0xd15, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd26, 0xd4d,
0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2b, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d,
0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd17, 0xd4d,
0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2b, 0xd3f, 0xd1c, 0xd3f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b,
0xd3f, 0xd1c, 0xd3f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3f, 0xd1c,
0xd3f, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd4d, 0xd30, 0xd1e, 0xd4d, 0xd1a,
0xd4d, 0x20, 0xd17, 0xd2f, 0xd3e, 0xd28, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b,
0xd4d, 0xd30, 0xd1e, 0xd4d, 0xd1a, 0xd4d, 0x20, 0xd38, 0xd24, 0xd47, 0xd7a, 0x2c,
0x20, 0xd05, 0xd28, 0xd4d, 0xd31, 0xd3e, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd3f, 0xd15,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd17, 0xd3e, 0xd32, 0xd2a, 0xd4d, 0xd2a,
0xd17, 0xd4b, 0xd38, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd17, 0xd3e, 0xd2e,
0xd4d, 0xd2a, 0xd3f, 0xd2f, 0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1c, 0xd4b,
0xd7c, 0xd1c, 0xd4d, 0xd1c, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1c,
0xd4b, 0xd7c, 0xd1c, 0xd4d, 0xd1c, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd1c, 0xd4b, 0xd7c, 0xd1c, 0xd4d, 0xd1c, 0xd3f, 0xd2f, 0x20, 0xd17, 0xd4d,
0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd17, 0xd3f, 0xd7d, 0xd2c, 0xd47, 0xd7c, 0xd1f, 0xd4d, 0xd1f, 0xd4d,
0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd35, 0xd3f, 0xd1a, 0xd4d, 0xd1a,
0xd4d, 0x20, 0xd2e, 0xd40, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f,
0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32,
0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34,
0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32, 0xd3e,
0xd7b, 0xd21, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34,
0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32, 0xd3e,
0xd7b, 0xd21, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e,
0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b, 0xd32,
0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f,
0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd7b,
0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a,
0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30,
0xd40, 0xd7b, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd17, 0xd4d, 0xd35, 0xd3e, 0xd02, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd17,
0xd7e, 0xd2b, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd35, 0xd3e,
0xd2f, 0xd4d, 0x2d, 0xd05, 0xd32, 0xd42, 0xd37, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd39, 0xd35, 0xd3e, 0xd2f, 0xd4d, 0x2d, 0xd05, 0xd32, 0xd42,
0xd37, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd35,
0xd3e, 0xd2f, 0xd4d, 0x2d, 0xd05, 0xd32, 0xd42, 0xd37, 0xd4d, 0xd2f, 0xd7b, 0x20,
0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd39, 0xd4b, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd19, 0xd4d, 0xd19, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd4b, 0xd19, 0xd4d, 0xd15, 0xd4b, 0xd19, 0xd4d,
0xd19, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd4b, 0xd19, 0xd4d,
0xd15, 0xd4b, 0xd19, 0xd4d, 0xd19, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39,
0xd4b, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd4b, 0xd21, 0xd4d,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd39, 0xd4b, 0xd21, 0xd4d, 0x20, 0xd17,
0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd07, 0xd28, 0xd4d, 0xd24, 0xd4d, 0xd2f, 0xd7b, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd28, 0xd4d, 0xd24, 0xd4d, 0xd2f, 0xd7b, 0x20,
0xd2e, 0xd39, 0xd3e, 0xd38, 0xd2e, 0xd41, 0xd26, 0xd4d, 0xd30, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd07, 0xd7b, 0xd21, 0xd4b, 0xd1a, 0xd48, 0xd28, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2e, 0xd27, 0xd4d, 0xd2f, 0x20, 0xd07, 0xd28, 0xd4d, 0xd24, 0xd4b,
0xd28, 0xd47, 0xd37, 0xd4d, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f,
0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd07, 0xd28, 0xd4d, 0xd24, 0xd4b, 0xd28,
0xd47, 0xd37, 0xd4d, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f,
0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31, 0xd7b, 0x20, 0xd07, 0xd28, 0xd4d, 0xd24, 0xd4b,
0xd28, 0xd47, 0xd37, 0xd4d, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd31,
0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd31, 0xd3e, 0xd7b, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd31, 0xd3e, 0xd7b, 0x20, 0xd21, 0xd47,
0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07,
0xd7c, 0xd15, 0xd38, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07,
0xd7c, 0xd15, 0xd38, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd07, 0xd7c, 0xd15, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30,
0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd07, 0xd38, 0xd4d, 0xd30, 0xd3e, 0xd2f, 0xd47, 0xd7d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd07, 0xd38, 0xd4d, 0xd30, 0xd3e, 0xd2f, 0xd47, 0xd7d, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd07, 0xd38, 0xd4d, 0xd30, 0xd3e, 0xd2f, 0xd47, 0xd7d,
0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd1c, 0xd2a, 0xd4d, 0xd2a, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd1c, 0xd2a, 0xd4d, 0xd2a, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd1c, 0xd2a, 0xd4d, 0xd2a, 0xd3e, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48,
0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd46, 0xd1f,
0xd4d, 0xd30, 0xd4b, 0xd2a, 0xd3e, 0xd35, 0xd4d, 0x200c, 0xd32, 0xd4b, 0xd38, 0xd4d,
0xd15, 0xd4d, 0x20, 0xd15, 0xd02, 0xd1a, 0xd3e, 0xd38, 0xd4d, 0xd15, 0xd3f, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd46, 0xd1f, 0xd4d, 0xd30, 0xd4b, 0xd2a, 0xd3e,
0xd35, 0xd4d, 0x200c, 0xd32, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd15, 0xd02,
0xd1a, 0xd3e, 0xd38, 0xd4d, 0xd15, 0xd3f, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2a, 0xd46, 0xd1f, 0xd4d, 0xd30, 0xd4b, 0xd2a, 0xd3e, 0xd35, 0xd4d, 0x200c, 0xd32,
0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd15, 0xd02, 0xd1a, 0xd3e, 0xd38, 0xd4d,
0xd15, 0xd3f, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd34, 0xd15, 0xd4d, 0xd15, 0xd7b,
0x20, 0xd15, 0xd38, 0xd3e, 0xd16, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd1f, 0xd3f, 0xd1e, 0xd4d, 0xd1e, 0xd3e, 0xd31,
0xd7b, 0x20, 0xd15, 0xd38, 0xd3e, 0xd16, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd31, 0xd3f, 0xd2f, 0xd7b, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd31, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4a, 0xd31, 0xd3f, 0xd2f, 0xd7b, 0x20, 0xd21,
0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd15, 0xd4a, 0xd38, 0xd4d, 0xd30, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d,
0xd30, 0xd3e, 0xd38, 0xd4d, 0x200c, 0xd28, 0xd4b, 0xd2f, 0xd3e, 0xd7c, 0xd38, 0xd4d,
0x200c, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d, 0xd30, 0xd3e,
0xd38, 0xd4d, 0x200c, 0xd28, 0xd4b, 0xd2f, 0xd3e, 0xd7c, 0xd38, 0xd4d, 0x200c, 0xd15,
0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd4d, 0xd30, 0xd3e, 0xd38,
0xd4d, 0x200c, 0xd28, 0xd4b, 0xd2f, 0xd3e, 0xd7c, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d,
0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd15, 0xd3f, 0xd7c, 0xd17, 0xd3f, 0xd38, 0xd4d,
0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd32, 0xd19, 0xd4d, 0xd15,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd32, 0xd48, 0xd7b, 0x20, 0xd26, 0xd4d, 0xd35,
0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd32, 0xd4b,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd39, 0xd4b, 0xd35, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd32, 0xd4b, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd39, 0xd4b, 0xd35, 0xd4d, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd32, 0xd4b, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd39,
0xd4b, 0xd35, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd15, 0xd4c, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2e, 0xd15, 0xd4c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd15,
0xd4c, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd17, 0xd3e, 0xd26, 0xd7b, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2e, 0xd17, 0xd3e, 0xd26, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2e, 0xd17, 0xd3e, 0xd26, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e,
0xd32, 0xd47, 0xd37, 0xd4d, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd3e,
0xd32, 0xd3f, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2e, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd38, 0xd38, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2e, 0xd3e, 0xd7c, 0xd37, 0xd7d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40,
0xd2a, 0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd57, 0xd31,
0xd40, 0xd37, 0xd4d, 0xd2f, 0xd38, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e,
0xd57, 0xd31, 0xd40, 0xd37, 0xd4d, 0xd2f, 0xd38, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2e, 0xd57, 0xd31, 0xd40, 0xd37, 0xd4d, 0xd2f, 0xd38, 0xd4d, 0x20,
0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd3e, 0xd38, 0xd7a, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2e, 0xd46, 0xd15, 0xd4d, 0xd38, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20,
0xd2a, 0xd38, 0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e,
0xd46, 0xd15, 0xd4d, 0x200c, 0xd38, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2a,
0xd38, 0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e,
0xd46, 0xd15, 0xd4d, 0xd38, 0xd3f, 0xd15, 0xd4d, 0xd15, 0xd7b, 0x20, 0xd2a, 0xd38,
0xd2b, 0xd3f, 0xd15, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd32, 0xd3e, 0xd7b, 0xd2c, 0xd3e,
0xd24, 0xd4d, 0xd24, 0xd7c, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd32, 0xd3e,
0xd7b, 0xd2c, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd7c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd09, 0xd32, 0xd3e, 0xd7b, 0xd2c, 0xd3e, 0xd24, 0xd4d, 0xd24, 0xd7c, 0x20,
0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4b, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd4b, 0xd38, 0xd4d, 0xd15, 0xd4b, 0x20, 0xd38, 0xd4d,
0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd4b, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4b, 0x20, 0xd17,
0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd2e, 0xd4d, 0xd2f, 0xd3e, 0xd7b, 0xd2e, 0xd3e, 0xd7c, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd57, 0xd31, 0xd41, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd28, 0xd47, 0xd2a, 0xd4d, 0xd2a, 0xd3e, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0x20, 0xd15, 0xd3e, 0xd32, 0xd3f, 0xd21, 0xd4b,
0xd23, 0xd3f, 0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42,
0x20, 0xd15, 0xd3e, 0xd32, 0xd3f, 0xd21, 0xd4b, 0xd23, 0xd3f, 0xd2f, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0x20, 0xd15, 0xd3e, 0xd32,
0xd3f, 0xd21, 0xd4b, 0xd23, 0xd3f, 0xd2f, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28,
0xd4d, 0xd2f, 0xd42, 0xd38, 0xd3f, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0xd38, 0xd3f, 0xd32, 0xd3e, 0xd7b,
0xd21, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42,
0xd38, 0xd3f, 0xd32, 0xd3e, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48,
0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f,
0xd42, 0xd2b, 0xd57, 0xd23, 0xd4d, 0xd1f, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0xd2b,
0xd57, 0xd23, 0xd4d, 0xd1f, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd4d,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0xd2b, 0xd57,
0xd23, 0xd4d, 0xd1f, 0xd4d, 0x200c, 0xd32, 0xd3e, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20,
0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd28, 0xd4d, 0xd2f, 0xd42, 0xd2f, 0xd3f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd28, 0xd4b, 0xd7c, 0x200c, 0xd2b, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26,
0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4b,
0xd7c, 0x200c, 0xd2b, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35,
0xd40, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4b, 0xd7c,
0x200c, 0xd2b, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40,
0xd2a, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d, 0xd31, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd46, 0xd7c, 0xd23, 0xd3e, 0xd21, 0xd4b, 0x20,
0xd21, 0xd3f, 0x20, 0xd28, 0xd4a, 0xd31, 0xd4b, 0xd7b, 0xd39, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2b, 0xd46, 0xd7c, 0xd23, 0xd3e, 0xd21, 0xd4b, 0x20, 0xd21, 0xd3f,
0x20, 0xd28, 0xd4a, 0xd31, 0xd4b, 0xd7b, 0xd39, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2b, 0xd46, 0xd7c, 0xd23, 0xd3e, 0xd21, 0xd4b, 0x20, 0xd21, 0xd3f, 0x20,
0xd28, 0xd4a, 0xd31, 0xd4b, 0xd7b, 0xd39, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28,
0xd4b, 0xd7c, 0xd24, 0xd4d, 0xd24, 0xd4d, 0x20, 0xd2e, 0xd31, 0xd3f, 0xd2f, 0xd3e,
0xd28, 0xd3e, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd41, 0xd15, 0xd7e, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4b, 0xd35, 0xd4b, 0xd38, 0xd3f, 0xd2c, 0xd3f,
0xd7c, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28,
0xd4b, 0xd35, 0xd4b, 0xd38, 0xd3f, 0xd2c, 0xd3f, 0xd7c, 0xd38, 0xd4d, 0xd15, 0xd4d,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd28, 0xd4b, 0xd35, 0xd4b, 0xd38, 0xd3f,
0xd2c, 0xd3f, 0xd7c, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30,
0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd13, 0xd02, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd13, 0xd02, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15, 0xd4d,
0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21,
0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd13, 0xd02, 0xd38, 0xd4d, 0x200c, 0xd15,
0xd4d, 0xd15, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd3e, 0xd15, 0xd4d,
0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd2a, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd3e, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd38,
0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c,
0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd32, 0xd3e,
0xd35, 0xd41, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd3e, 0xd2a, 0xd4d, 0xd2a,
0xd41, 0xd35, 0x20, 0xd28, 0xd4d, 0xd2f, 0xd42, 0x20, 0xd17, 0xd3f, 0xd28, 0xd3f,
0xd2f, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd30, 0xd3e, 0xd17, 0xd4d, 0xd35,
0xd47, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd30, 0xd3e, 0xd17, 0xd4d, 0xd35,
0xd47, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd30, 0xd3e, 0xd17, 0xd4d,
0xd35, 0xd47, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15,
0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd46, 0xd31, 0xd41, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd46, 0xd31, 0xd41, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd2a, 0xd46, 0xd31, 0xd41, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37,
0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b,
0xd3f, 0xd32, 0xd3f, 0xd2a, 0xd4d, 0xd2a, 0xd48, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2b, 0xd3f, 0xd32, 0xd3f, 0xd2a, 0xd4d, 0xd2a, 0xd48, 0xd7b, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3f, 0xd32, 0xd3f, 0xd2a, 0xd4d, 0xd2a, 0xd48,
0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2b, 0xd3f, 0xd28, 0xd3f, 0xd15, 0xd4d,
0x200c, 0xd38, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd2a, 0xd3f,
0xd2f, 0xd31, 0xd3f, 0x20, 0xd06, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd2e, 0xd3f, 0xd15,
0xd4d, 0xd35, 0xd32, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd28,
0xd4d, 0xd31, 0xd4d, 0x20, 0xd2a, 0xd3f, 0xd2f, 0xd31, 0xd3f, 0x20, 0xd06, 0xd7b,
0xd21, 0xd4d, 0x20, 0xd2e, 0xd3f, 0xd15, 0xd4d, 0xd35, 0xd32, 0xd7b, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd46, 0xd28, 0xd4d, 0xd31, 0xd4d, 0x20, 0xd2a,
0xd3f, 0xd2f, 0xd31, 0xd3f, 0x20, 0xd06, 0xd7b, 0xd21, 0xd4d, 0x20, 0xd2e, 0xd3f,
0xd15, 0xd4d, 0xd35, 0xd32, 0xd7b, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd3f, 0xd31, 0xd4d, 0xd31,
0xd4d, 0xd15, 0xd47, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd4a, 0xd28,
0xd3e, 0xd2a, 0xd4d, 0xd2a, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2a, 0xd4d,
0xd2f, 0xd4b, 0xd02, 0xd17, 0xd4d, 0x200c, 0xd2f, 0xd3e, 0xd02, 0xd17, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd16, 0xd3f, 0xd38, 0xd3f, 0xd32, 0xd4b, 0xd7c, 0xd21,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd16, 0xd3f, 0xd38, 0xd3f, 0xd32, 0xd4b, 0xd7c,
0xd21, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c,
0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd16, 0xd3f, 0xd38, 0xd3f, 0xd32,
0xd4b, 0xd7c, 0xd21, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd31, 0xd40, 0xd2f, 0xd42, 0xd23, 0xd3f,
0xd2f, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd31, 0xd4b, 0xd25, 0xd46, 0xd31,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd7b, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd7b, 0x20, 0xd38,
0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd16, 0xd3e, 0xd32, 0xd3f, 0xd7b, 0x20, 0xd17,
0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd38, 0xd2e, 0xd3e, 0xd30, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd38, 0xd2e, 0xd3e, 0xd31, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b,
0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd2e,
0xd3e, 0xd31, 0x20, 0xd35, 0xd47, 0xd28, 0xd7d, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd32,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd2e, 0xd4b, 0xd35, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd38, 0xd2e, 0xd4b, 0xd35, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd38, 0xd2e, 0xd4b, 0xd35, 0xd3e, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d,
0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd40,
0xd37, 0xd46, 0xd7d, 0xd38, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd3f,
0xd02, 0xd17, 0xd2a, 0xd4d, 0xd2a, 0xd42, 0xd7c, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd38, 0xd4b, 0xd33, 0xd2e, 0xd7b, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a,
0xd41, 0xd15, 0xd7e, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd26, 0xd15, 0xd4d, 0xd37,
0xd3f, 0xd23, 0x20, 0xd1c, 0xd4b, 0xd7c, 0xd1c, 0xd4d, 0xd1c, 0xd3f, 0xd2f, 0xd7b,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd41, 0xd30, 0xd3f, 0xd28, 0xd46, 0xd2f,
0xd3f, 0xd02, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd38, 0xd2f, 0xd4b, 0xd35, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd39, 0xd3f, 0xd24, 0xd3f, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd2f, 0xd4d, 0x200c, 0xd2a, 0xd46, 0xd2f, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd2f, 0xd4d, 0x200c, 0xd2a, 0xd46,
0xd2f, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47,
0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd2f, 0xd4d,
0x200c, 0xd2a, 0xd46, 0xd2f, 0xd4d, 0x20, 0xd21, 0xd47, 0xd32, 0xd48, 0xd31, 0xd4d,
0xd31, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd3e, 0xd1c, 0xd3f, 0xd15,
0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd1f, 0xd4b, 0xd15, 0xd4d, 0xd15, 0xd46, 0xd32, 0xd42, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd1f, 0xd4b, 0xd02, 0xd17, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1f,
0xd4b, 0xd02, 0xd17, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21,
0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1f, 0xd4b, 0xd02,
0xd17, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e,
0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd1a, 0xd42, 0xd15, 0xd4d, 0xd15, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd24, 0xd41, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd4d,
0x200c, 0xd2e, 0xd46, 0xd28, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd24, 0xd41, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c, 0xd2e,
0xd46, 0xd28, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd24, 0xd41, 0xd7c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x200c, 0xd2e, 0xd46,
0xd28, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd1f, 0xd41, 0xd35, 0xd3e, 0xd32, 0xd41, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09,
0xd31, 0xd41, 0xd17, 0xd4d, 0xd35, 0xd47, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09,
0xd31, 0xd41, 0xd17, 0xd4d, 0xd35, 0xd47, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd09, 0xd31, 0xd41, 0xd17, 0xd4d, 0xd35, 0xd47, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40,
0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd09, 0xd38, 0xd4d, 0x200c, 0xd2c, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d,
0xd25, 0xd3e, 0xd7b, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd38, 0xd4d, 0x200c,
0xd2c, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd09, 0xd38, 0xd4d, 0x200c, 0xd2c, 0xd46, 0xd15,
0xd4d, 0xd15, 0xd3f, 0xd38, 0xd4d, 0xd25, 0xd3e, 0xd7b, 0x20, 0xd17, 0xd4d, 0xd30,
0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd35, 0xd28, 0xd4d, 0xd35, 0xd3e, 0xd24, 0xd41, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd35, 0xd28, 0xd4d, 0xd35, 0xd3e, 0xd24, 0xd41, 0x20, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd35, 0xd28, 0xd4d, 0xd35, 0xd3e, 0xd24, 0xd41, 0x20, 0xd17, 0xd4d,
0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e,
0xd2f, 0xd02, 0xd35, 0xd46, 0xd28, 0xd3f, 0xd38, 0xd4d, 0xd35, 0xd47, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd4d, 0xd32, 0xd3e, 0xd21, 0xd3f, 0xd35, 0xd4b,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd4b, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd35, 0xd4d, 0xd32, 0xd3e, 0xd21, 0xd3f, 0xd35, 0xd4b, 0xd38, 0xd4d, 0xd31,
0xd4d, 0xd31, 0xd4b, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35,
0xd4d, 0xd32, 0xd3e, 0xd21, 0xd3f, 0xd35, 0xd4b, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31,
0xd4b, 0xd15, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd4b, 0xd7e, 0xd17,
0xd4b, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02,
0xd35, 0xd4b, 0xd7e, 0xd17, 0xd4b, 0xd17, 0xd4d, 0xd30, 0xd3e, 0xd21, 0xd4d, 0x20,
0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d,
0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd4b, 0xd7e, 0xd17, 0xd4b, 0xd17, 0xd4d,
0xd30, 0xd3e, 0xd21, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c,
0xd2e, 0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd35, 0xd47, 0xd15,
0xd4d, 0xd15, 0xd4d, 0x20, 0xd26, 0xd4d, 0xd35, 0xd40, 0xd2a, 0xd4d, 0x20, 0xd38,
0xd2e, 0xd2f, 0xd02, 0xd35, 0xd3e, 0xd32, 0xd3f, 0xd38, 0xd4d, 0x20, 0xd06, 0xd28,
0xd4d, 0xd31, 0xd4d, 0x20, 0xd2b, 0xd4d, 0xd2f, 0xd42, 0xd1a, 0xd4d, 0xd2f, 0xd41,
0xd28, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2f, 0xd3e, 0xd15, 0xd38, 0xd4d, 0x200c,
0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2f, 0xd3e, 0xd15,
0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d,
0xd31, 0xd3e, 0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2f, 0xd3e, 0xd15, 0xd38, 0xd4d, 0x200c, 0xd15, 0xd4d, 0xd15, 0xd4d, 0x20,
0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e, 0xd15, 0xd3e, 0xd32, 0x20,
0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2f, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd31, 0xd4d,
0xd31, 0xd30, 0xd3f, 0xd7b, 0xd2c, 0xd7c, 0xd17, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f,
0xd02, 0xd2f, 0xd46, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd30, 0xd3f,
0xd7b, 0xd2c, 0xd7c, 0xd17, 0xd4d, 0x20, 0xd38, 0xd4d, 0xd31, 0xd4d, 0xd31, 0xd3e,
0xd7b, 0xd21, 0xd47, 0xd7c, 0xd21, 0xd4d, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2f,
0xd46, 0xd15, 0xd4d, 0xd15, 0xd3e, 0xd31, 0xd4d, 0xd31, 0xd30, 0xd3f, 0xd7b, 0xd2c,
0xd7c, 0xd17, 0xd4d, 0x20, 0xd17, 0xd4d, 0xd30, 0xd40, 0xd37, 0xd4d, 0x200c, 0xd2e,
0xd15, 0xd3e, 0xd32, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0xd2f, 0xd42, 0xd15, 0xd4d,
0xd15, 0xd4b, 0xd7a, 0x20, 0xd38, 0xd2e, 0xd2f, 0xd02, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x63, 0x72,
0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x69, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x73, 0x61, 0x74,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x50, 0x75, 0x73, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x72, 0x67, 0x75, 0x6e, 0x75,
0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x50, 0x65, 0x72, 0x67, 0x75, 0x6e, 0x75,
0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69,
0x61, 0x6e, 0x67, 0x20, 0x50, 0x65, 0x72, 0x67, 0x75, 0x6e, 0x75, 0x6e,
0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x70, 0x69, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62,
0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x41, 0x72, 0x61, 0x62, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x72, 0x61, 0x62, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54,
0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x20, 0x54, 0x65, 0x6e,
0x67, 0x61, 0x68, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67,
0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61,
0x68, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x54, 0x69, 0x6d,
0x75, 0x72, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x69,
0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x42,
0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69,
0x61, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x54, 0x61, 0x6e, 0x6a, 0x75, 0x6e, 0x67, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x68, 0x69,
0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20,
0x43, 0x68, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75,
0x6c, 0x61, 0x75, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61,
0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75,
0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4b,
0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d,
0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x65, 0x70, 0x61, 0x72,
0x75, 0x68, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x43,
0x75, 0x62, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x43, 0x75, 0x62, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x43, 0x75, 0x62, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x75, 0x6c, 0x61, 0x75, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x50, 0x75, 0x6c, 0x61, 0x75, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20,
0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20, 0x54, 0x65, 0x6e, 0x67,
0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20, 0x54, 0x65,
0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75,
0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x45, 0x72,
0x6f, 0x70, 0x61, 0x68, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20,
0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68,
0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68,
0x20, 0x63, 0x65, 0x72, 0x75, 0x6b, 0x20, 0x74, 0x69, 0x6d, 0x75, 0x72,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68,
0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x45, 0x72, 0x6f, 0x70, 0x61,
0x68, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0x45, 0x72, 0x6f, 0x70, 0x61, 0x68, 0x20, 0x42, 0x61, 0x72, 0x61,
0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61,
0x69, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x20, 0x50, 0x65, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x73, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x65, 0x72, 0x61, 0x6e, 0x63, 0x69, 0x73,
0x20, 0x53, 0x65, 0x6c, 0x61, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x61, 0x6e,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x6b, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d,
0x69, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54,
0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x54, 0x65, 0x6c,
0x75, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4c, 0x61, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x48, 0x69, 0x6e, 0x64, 0x69,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x69, 0x61, 0x6e, 0x67, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20,
0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4a, 0x65, 0x70, 0x75, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4a, 0x65, 0x70, 0x75, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x4a,
0x65, 0x70, 0x75, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53,
0x69, 0x61, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c,
0x61, 0x75, 0x61, 0x6e, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61,
0x63, 0x61, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x4d, 0x61,
0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61,
0x6e, 0x67, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x63, 0x6f, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x6f, 0x77, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4d, 0x75, 0x73, 0x69, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61,
0x73, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4b, 0x65, 0x70, 0x75,
0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67,
0x20, 0x4b, 0x65, 0x70, 0x75, 0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x4b, 0x65, 0x70, 0x75,
0x6c, 0x61, 0x75, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e,
0x61, 0x20, 0x55, 0x74, 0x61, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77,
0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x50, 0x65, 0x72, 0x75, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x46, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x6e, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x64, 0x61, 0x6e,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x51, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50,
0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x75, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61,
0x77, 0x61, 0x69, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x69, 0x61, 0x6e, 0x67, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x50, 0x69, 0x61, 0x77, 0x61, 0x69, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77,
0x61, 0x69, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x75, 0x6c, 0x61, 0x75, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69,
0x61, 0x77, 0x61, 0x69, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x69, 0x61, 0x77, 0x61, 0x69,
0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x4d, 0x61, 0x73, 0x61, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x648, 0x642, 0x62a, 0x648, 0x20, 0x628, 0x631, 0x648, 0x646, 0x64a, 0x20, 0x62f,
0x627, 0x631, 0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x648, 0x642, 0x62a, 0x648,
0x20, 0x6a4, 0x64a, 0x627, 0x648, 0x627, 0x64a, 0x20, 0x627, 0x64a, 0x646, 0x62f,
0x64a, 0x627, 0x648, 0x642, 0x62a, 0x648, 0x20, 0x644, 0x627, 0x674, 0x648, 0x62a,
0x646, 0x20, 0x647, 0x64a, 0x646, 0x62f, 0x64a, 0x648, 0x642, 0x62a, 0x648, 0x20,
0x625, 0x646, 0x62f, 0x648, 0x646, 0x64a, 0x633, 0x64a, 0x627, 0x20, 0x62a, 0x6a0,
0x647, 0x648, 0x642, 0x62a, 0x648, 0x20, 0x625, 0x646, 0x62f, 0x648, 0x646, 0x64a,
0x633, 0x64a, 0x627, 0x20, 0x62a, 0x64a, 0x645, 0x648, 0x631, 0x648, 0x642, 0x62a,
0x648, 0x20, 0x625, 0x646, 0x62f, 0x648, 0x646, 0x64a, 0x633, 0x64a, 0x627, 0x20,
0x628, 0x627, 0x631, 0x627, 0x62a, 0x648, 0x642, 0x62a, 0x648, 0x20, 0x645, 0x644,
0x64a, 0x633, 0x64a, 0x627, 0x126, 0x69, 0x6e, 0x20, 0x10a, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x65, 0x77,
0x126, 0x69, 0x6e, 0x20, 0x10a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x65, 0x77, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x126, 0x69, 0x6e, 0x20, 0x10a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x65,
0x77, 0x20, 0x74, 0x61, 0x73, 0x2d, 0x53, 0x61, 0x6a, 0x66, 0x9a8, 0x9cb,
0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8,
0x20, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x99f,
0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7,
0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x9a4, 0x9cd,
0x9b0, 0x9c7, 0x9b2, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0,
0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5,
0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9c7,
0x9a8, 0x9cd, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x9a6, 0x9c7, 0x9b2, 0x9be,
0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5,
0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x987, 0x9b7,
0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb,
0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8,
0x20, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9b7, 0x9cd, 0x99f,
0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae,
0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995,
0x9be, 0x9a8, 0x20, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9a6,
0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb,
0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8,
0x20, 0x9ae, 0x9be, 0x989, 0x9a8, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x20, 0x99f, 0x9be,
0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0,
0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x989, 0x9a8, 0x9cd, 0x99f, 0x9c7,
0x9a8, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6,
0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985,
0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9ae, 0x9be, 0x989, 0x9a8,
0x9cd, 0x99f, 0x9c7, 0x9a8, 0x20, 0x9a6, 0x9c7, 0x9b2, 0x9be, 0x987, 0x99f, 0x20,
0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5, 0x20, 0x985, 0x9ae,
0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa, 0x9c7, 0x9b8, 0x9bf, 0x9ab,
0x9bf, 0x995, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0, 0x9cd, 0x9a5,
0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20, 0x9aa, 0x9c7,
0x9b8, 0x9bf, 0x9ab, 0x9bf, 0x995, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd,
0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9a8, 0x9cb, 0x9b0,
0x9cd, 0x9a5, 0x20, 0x985, 0x9ae, 0x9c7, 0x9b0, 0x9bf, 0x995, 0x9be, 0x9a8, 0x20,
0x9aa, 0x9c7, 0x9b8, 0x9bf, 0x9ab, 0x9bf, 0x995, 0x20, 0x9a6, 0x9c7, 0x9b2, 0x9be,
0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x985, 0x99f, 0x9b2, 0x9be, 0x9a8,
0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x985, 0x99f, 0x9b2,
0x9be, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8,
0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x985, 0x99f,
0x9b2, 0x9be, 0x9a8, 0x9cd, 0x99f, 0x9bf, 0x995, 0x20, 0x9a6, 0x9c7, 0x9b2, 0x9be,
0x987, 0x99f, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9b8, 0x9c7, 0x9a8, 0x9cd, 0x9a4,
0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9b8, 0x9c7, 0x9a8,
0x9cd, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb,
0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8,
0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9b8, 0x9c7,
0x9a8, 0x9cd, 0x9a4, 0x9cd, 0x9b0, 0x9c7, 0x9b2, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0,
0x9cb, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9b0, 0x20,
0x99f, 0x9be, 0x987, 0x9ae, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20,
0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20,
0x99f, 0x9be, 0x987, 0x9ae, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20,
0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20,
0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6, 0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f,
0x9be, 0x987, 0x9ae, 0x987, 0x9b7, 0x9cd, 0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9af,
0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf, 0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b8,
0x9ae, 0x9b0, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9f1, 0x9c7, 0x9b7, 0x9cd, 0x99f,
0x9b0, 0x9cd, 0x9a8, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf, 0x9af,
0x9bc, 0x9be, 0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9f1, 0x9c7, 0x9b7, 0x9cd,
0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b7, 0x9cd, 0x99f, 0x9c7, 0x9a8, 0x9cd, 0x9a6,
0x9b0, 0x9cd, 0x9a6, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x9f1, 0x9c7, 0x9b7, 0x9cd,
0x99f, 0x9b0, 0x9cd, 0x9a8, 0x20, 0x9af, 0x9bc, 0x9c1, 0x9b0, 0x9cb, 0x9aa, 0x9bf,
0x9af, 0x9bc, 0x9be, 0x9a8, 0x20, 0x9b8, 0x9ae, 0x9b0, 0x20, 0x99f, 0x9be, 0x987,
0x9ae, 0x997, 0x9cd, 0x9b0, 0x9bf, 0x9a8, 0x9f1, 0x9bf, 0x99a, 0x20, 0x9ae, 0x9bf,
0x9a8, 0x20, 0x99f, 0x9be, 0x987, 0x9ae, 0x57, 0x101, 0x20, 0x41, 0x77, 0x68,
0x65, 0x6b, 0x65, 0x6e, 0x65, 0x74, 0x101, 0x6e, 0x61, 0x57, 0x101, 0x20,
0x6f, 0x20, 0x54, 0x65, 0x20, 0x50, 0x75, 0x6b, 0x75, 0x20, 0x6f, 0x20,
0x100, 0x77, 0x68, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x101, 0x20, 0x6f,
0x20, 0x100, 0x77, 0x68, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x69,
0x20, 0x74, 0x65, 0x20, 0x72, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57,
0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20,
0x6f, 0x20, 0x100, 0x77, 0x68, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6b,
0x69, 0x20, 0x74, 0x65, 0x20, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x101,
0x20, 0x6f, 0x20, 0x100, 0x77, 0x68, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x6f, 0x20,
0x100, 0x77, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74,
0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20, 0x52, 0x61, 0x75, 0x6d,
0x61, 0x74, 0x69, 0x20, 0x6f, 0x20, 0x100, 0x77, 0x68, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75,
0x57, 0x101, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x101, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65,
0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x101, 0x20, 0x41,
0x6d, 0x61, 0x68, 0x6f, 0x6e, 0x61, 0x57, 0x101, 0x20, 0x41, 0x6d, 0x61,
0x68, 0x6f, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x41, 0x6d, 0x61, 0x68, 0x6f, 0x6e, 0x61,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x57,
0x61, 0x65, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61,
0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x57, 0x61,
0x65, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69,
0x74, 0x69, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101,
0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x52, 0x101, 0x77, 0x68,
0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x75, 0x6e, 0x67, 0x61,
0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x20, 0x4d, 0x61, 0x75, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x41, 0x77,
0x61, 0x74, 0x65, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x6e, 0x67, 0x61, 0x57,
0x101, 0x20, 0x4b, 0x69, 0x77, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x4b, 0x69, 0x77, 0x61, 0x57,
0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x4b, 0x69, 0x77,
0x61, 0x57, 0x101, 0x20, 0x100, 0x70, 0x69, 0x61, 0x57, 0x101, 0x20, 0x100,
0x70, 0x69, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x100, 0x70, 0x69, 0x61, 0x20, 0x41, 0x77, 0x61,
0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72, 0x101, 0x70, 0x69, 0x61,
0x57, 0x101, 0x20, 0x41, 0x72, 0x101, 0x70, 0x69, 0x61, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x41, 0x72,
0x101, 0x70, 0x69, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57,
0x101, 0x20, 0x100, 0x6b, 0x65, 0x74, 0x69, 0x6e, 0x61, 0x57, 0x101, 0x20,
0x100, 0x6b, 0x65, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x100, 0x6b, 0x65, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57,
0x101, 0x20, 0x100, 0x6b, 0x65, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6b, 0x69,
0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20, 0x100, 0x6b,
0x65, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20,
0x75, 0x72, 0x75, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x100, 0x6b, 0x65, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x100, 0x6d, 0x65, 0x6e,
0x69, 0x61, 0x57, 0x101, 0x20, 0x100, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x100, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x74, 0x69, 0x6b,
0x69, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x20, 0x52, 0x61, 0x6e, 0x61, 0x74, 0x69, 0x6b, 0x69, 0x57, 0x101,
0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x52, 0x61, 0x6e, 0x61,
0x74, 0x69, 0x6b, 0x69, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65,
0x72, 0x65, 0x69, 0x72, 0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67,
0x61, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69,
0x72, 0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x41,
0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72, 0x69, 0x61, 0x20, 0x57,
0x61, 0x65, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61,
0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72,
0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x2d, 0x55, 0x72,
0x75, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69,
0x72, 0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x2d, 0x55,
0x72, 0x75, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72,
0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x2d, 0x55, 0x72,
0x75, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x41,
0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72, 0x69, 0x61, 0x20, 0x6b,
0x69, 0x20, 0x74, 0x65, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69,
0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72,
0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x52, 0x101, 0x77,
0x68, 0x69, 0x74, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65,
0x69, 0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x52,
0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65,
0x61, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69,
0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x55, 0x72,
0x75, 0x57, 0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69,
0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x55, 0x72,
0x75, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x41, 0x68, 0x69, 0x74, 0x65, 0x72, 0x65, 0x69, 0x72, 0x69,
0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x55, 0x72, 0x75, 0x20,
0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x41, 0x74, 0x65,
0x70, 0x61, 0x69, 0x68, 0x101, 0x6e, 0x69, 0x61, 0x57, 0x101, 0x20, 0x41,
0x74, 0x65, 0x70, 0x61, 0x69, 0x68, 0x101, 0x6e, 0x69, 0x61, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x41,
0x74, 0x65, 0x70, 0x61, 0x69, 0x68, 0x101, 0x6e, 0x69, 0x61, 0x20, 0x52,
0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x57, 0x101, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x52, 0x61, 0x75, 0x6d,
0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x50, 0x101, 0x6b, 0x61, 0x72, 0x61,
0x74, 0x113, 0x68, 0x69, 0x57, 0x101, 0x20, 0x50, 0x101, 0x6b, 0x61, 0x72,
0x61, 0x74, 0x113, 0x68, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x50, 0x101, 0x6b, 0x61, 0x72, 0x61,
0x74, 0x113, 0x68, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x57, 0x101, 0x20, 0x50, 0x16b, 0x74, 0x61, 0x6e, 0x61, 0x57, 0x101, 0x20,
0x50, 0x6f, 0x72, 0x69, 0x77, 0x69, 0x61, 0x57, 0x101, 0x20, 0x50, 0x61,
0x72, 0x12b, 0x68, 0x69, 0x61, 0x57, 0x101, 0x20, 0x50, 0x61, 0x72, 0x12b,
0x68, 0x69, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x50, 0x61, 0x72, 0x12b, 0x68, 0x69, 0x61, 0x20,
0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x50, 0x6f,
0x72, 0x6f, 0x6e, 0x61, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20,
0x4b, 0x16b, 0x72, 0x61, 0x65, 0x20, 0x4d, 0x61, 0x74, 0x6f, 0x6d, 0x61,
0x74, 0x6f, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x4b, 0x16b, 0x72, 0x61,
0x65, 0x20, 0x4d, 0x61, 0x74, 0x6f, 0x6d, 0x61, 0x74, 0x6f, 0x57, 0x101,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x6f, 0x20, 0x54,
0x65, 0x20, 0x4b, 0x16b, 0x72, 0x61, 0x65, 0x20, 0x4d, 0x61, 0x74, 0x6f,
0x6d, 0x61, 0x74, 0x6f, 0x57, 0x101, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x6f, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x52, 0x113, 0x6b, 0x6f, 0x68, 0x75, 0x57, 0x101,
0x20, 0x52, 0x113, 0x6b, 0x6f, 0x68, 0x75, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x52, 0x113, 0x6b, 0x6f,
0x68, 0x75, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20,
0x48, 0x69, 0x72, 0x69, 0x57, 0x101, 0x20, 0x48, 0x69, 0x72, 0x69, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x48, 0x69, 0x72, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x57, 0x101, 0x20, 0x48, 0x61, 0x69, 0x6e, 0x61, 0x57, 0x101, 0x20, 0x48,
0x61, 0x69, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x48, 0x61, 0x69, 0x6e, 0x61, 0x20, 0x41,
0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x57, 0x101, 0x20, 0x43, 0x68, 0x6f,
0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x4d, 0x6f,
0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x69, 0x68, 0x69,
0x6d, 0x65, 0x74, 0x65, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x4e, 0x67, 0x101,
0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x4b, 0x6f, 0x6b,
0x6f, 0x68, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x6f, 0x72, 0x6f, 0x6d, 0x14d,
0x70, 0x69, 0x61, 0x57, 0x101, 0x20, 0x4b, 0x6f, 0x72, 0x6f, 0x6d, 0x14d,
0x70, 0x69, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x4b, 0x6f, 0x72, 0x6f, 0x6d, 0x14d, 0x70, 0x69,
0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20,
0x4b, 0x75, 0x6b, 0x69, 0x20, 0x41, 0x69, 0x72, 0x61, 0x6e, 0x69, 0x57,
0x101, 0x20, 0x4b, 0x75, 0x6b, 0x69, 0x20, 0x41, 0x69, 0x72, 0x61, 0x6e,
0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x4b, 0x75, 0x6b, 0x69, 0x20, 0x41, 0x69, 0x72, 0x61, 0x6e,
0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x48, 0x61,
0x75, 0x72, 0x75, 0x61, 0x57, 0x101, 0x20, 0x4b, 0x69, 0x75, 0x70, 0x61,
0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x20, 0x4b, 0x69, 0x75, 0x70, 0x61, 0x57, 0x101, 0x20, 0x41, 0x77, 0x61,
0x74, 0x65, 0x61, 0x20, 0x4b, 0x69, 0x75, 0x70, 0x61, 0x57, 0x101, 0x20,
0x52, 0x113, 0x77, 0x65, 0x74, 0x69, 0x57, 0x101, 0x20, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x61, 0x20,
0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69, 0x74,
0x69, 0x57, 0x101, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x4d, 0x6f,
0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x6f, 0x20, 0x41, 0x72, 0x61, 0x6e,
0x67, 0x61, 0x57, 0x101, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x4d,
0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x41, 0x72, 0x61, 0x6e, 0x67,
0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x4d, 0x6f, 0x75, 0x74,
0x65, 0x72, 0x65, 0x20, 0x41, 0x72, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x52,
0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x45, 0x6b, 0x75,
0x61, 0x74, 0x6f, 0x61, 0x57, 0x101, 0x20, 0x55, 0x72, 0x6f, 0x70, 0x69,
0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x55, 0x72, 0x6f, 0x70,
0x69, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x52,
0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x55, 0x72, 0x6f, 0x70, 0x69,
0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x55, 0x72,
0x6f, 0x70, 0x69, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57,
0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20,
0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69, 0x74,
0x69, 0x57, 0x101, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x20,
0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x52, 0x101, 0x77, 0x68, 0x69, 0x74,
0x69, 0x57, 0x101, 0x20, 0x16a, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x6b, 0x69,
0x20, 0x74, 0x65, 0x20, 0x72, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x20,
0x72, 0x61, 0x77, 0x61, 0x57, 0x101, 0x20, 0x55, 0x72, 0x6f, 0x70, 0x69,
0x20, 0x55, 0x72, 0x75, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x20, 0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x55,
0x72, 0x75, 0x57, 0x101, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x20, 0x55, 0x72, 0x6f, 0x70, 0x69, 0x20, 0x55, 0x72, 0x75, 0x57, 0x101,
0x20, 0x6b, 0x69, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x74, 0x75,
0x20, 0x57, 0x68, 0x101, 0x6b, 0x61, 0x6e, 0x61, 0x57, 0x101, 0x20, 0x6b,
0x69, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x74, 0x75, 0x20, 0x57,
0x68, 0x101, 0x6b, 0x61, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x6b, 0x69, 0x20, 0x4e, 0x67,
0x101, 0x20, 0x4d, 0x6f, 0x74, 0x75, 0x20, 0x57, 0x68, 0x101, 0x6b, 0x61,
0x6e, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101,
0x20, 0x57, 0x68, 0x12b, 0x74, 0x12b, 0x57, 0x101, 0x20, 0x57, 0x68, 0x12b,
0x74, 0x12b, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x57, 0x101, 0x20, 0x57, 0x68, 0x12b, 0x74, 0x12b, 0x20, 0x52, 0x61, 0x75,
0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x69, 0x101, 0x6e, 0x61,
0x20, 0x57, 0x12b, 0x77, 0x12b, 0x57, 0x101, 0x20, 0x57, 0x12b, 0x77, 0x12b,
0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20,
0x6d, 0x65, 0x20, 0x74, 0x65, 0x20, 0x4b, 0x14d, 0x70, 0x61, 0x6b, 0x61,
0x74, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x47, 0x61, 0x6c, 0x61,
0x70, 0x61, 0x67, 0x6f, 0x73, 0x57, 0x101, 0x20, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x57, 0x101, 0x20, 0x48, 0x14d, 0x72, 0x69, 0x61, 0x57,
0x101, 0x20, 0x48, 0x14d, 0x72, 0x69, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x48, 0x14d, 0x72, 0x69,
0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20,
0x4b, 0x69, 0x72, 0x69, 0x70, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x54,
0x6f, 0x68, 0x61, 0x72, 0x69, 0x74, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x69,
0x77, 0x12b, 0x74, 0x69, 0x57, 0x101, 0x20, 0x57, 0x68, 0x65, 0x6e, 0x75,
0x61, 0x6b, 0x101, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6b, 0x69, 0x20, 0x74,
0x65, 0x20, 0x72, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x6f, 0x20,
0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x6b, 0x101, 0x72, 0x69, 0x6b, 0x69,
0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x72, 0x101, 0x77, 0x68, 0x69,
0x74, 0x69, 0x57, 0x101, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x20, 0x6f, 0x20, 0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x6b, 0x101, 0x72,
0x69, 0x6b, 0x69, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x72, 0x101,
0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x57, 0x68, 0x65, 0x6e,
0x75, 0x61, 0x6b, 0x101, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6b, 0x69, 0x20,
0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x6f, 0x20, 0x57, 0x68, 0x65,
0x6e, 0x75, 0x61, 0x6b, 0x101, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6b, 0x69,
0x20, 0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x6f, 0x20, 0x57, 0x68, 0x65, 0x6e,
0x75, 0x61, 0x6b, 0x101, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6b, 0x69, 0x20,
0x74, 0x65, 0x20, 0x75, 0x72, 0x75, 0x57, 0x101, 0x20, 0x57, 0x68, 0x61,
0x6e, 0x67, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x4b, 0x61, 0x69, 0x61, 0x6e, 0x61, 0x57, 0x101,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x101, 0x20, 0x41,
0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x57, 0x101, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x69, 0x70, 0x75, 0x61, 0x57, 0x101, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x69, 0x70, 0x75, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x48, 0x6f, 0x6e, 0x67,
0x69, 0x70, 0x75, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x57, 0x101, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x57, 0x101, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x57, 0x101, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x52, 0x61, 0x75, 0x6d,
0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x12a, 0x6e, 0x69, 0x61, 0x57, 0x101,
0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x4d, 0x6f, 0x61, 0x6e, 0x61, 0x20,
0x12a, 0x6e, 0x69, 0x61, 0x57, 0x101, 0x20, 0x12a, 0x6e, 0x69, 0x61, 0x68,
0x61, 0x69, 0x6e, 0x61, 0x57, 0x101, 0x20, 0x49, 0x6e, 0x69, 0x74, 0x6f,
0x6e, 0x12b, 0x68, 0x69, 0x61, 0x20, 0x57, 0x61, 0x65, 0x6e, 0x67, 0x61,
0x57, 0x101, 0x20, 0x49, 0x6e, 0x69, 0x74, 0x6f, 0x6e, 0x12b, 0x68, 0x69,
0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x72, 0x101, 0x77, 0x68,
0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x49, 0x6e, 0x69, 0x74, 0x6f, 0x6e,
0x12b, 0x68, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x75,
0x72, 0x75, 0x57, 0x101, 0x20, 0x49, 0x72, 0x101, 0x6e, 0x61, 0x57, 0x101,
0x20, 0x49, 0x72, 0x101, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68,
0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x49, 0x72, 0x101, 0x6e, 0x61,
0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x101, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20,
0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x49, 0x68,
0x61, 0x72, 0x61, 0x69, 0x72, 0x61, 0x57, 0x101, 0x20, 0x49, 0x68, 0x61,
0x72, 0x61, 0x69, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x49, 0x68, 0x61, 0x72, 0x61, 0x69,
0x72, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20,
0x48, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x57, 0x101, 0x20, 0x48, 0x61, 0x70,
0x61, 0x6e, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x48, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x20, 0x41,
0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x4b, 0x61, 0x74, 0x61,
0x74, 0x101, 0x6e, 0x67, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20,
0x52, 0x101, 0x77, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x61,
0x74, 0x61, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74,
0x65, 0x20, 0x55, 0x72, 0x75, 0x57, 0x101, 0x20, 0x4b, 0x14d, 0x72, 0x65,
0x61, 0x57, 0x101, 0x20, 0x4b, 0x14d, 0x72, 0x65, 0x61, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x14d,
0x72, 0x65, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101,
0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x57, 0x101, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x57, 0x101, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20,
0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4b, 0x69,
0x6b, 0x69, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x6f, 0x20,
0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x20, 0x6f,
0x20, 0x54, 0x65, 0x20, 0x52, 0x61, 0x69, 0x6e, 0x61, 0x57, 0x101, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x57, 0x101, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x41, 0x77, 0x61, 0x74,
0x65, 0x61, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x57, 0x101, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x72, 0x65, 0x69, 0x61, 0x57,
0x101, 0x20, 0x4d, 0x101, 0x72, 0x61, 0x74, 0x69, 0x72, 0x69, 0x57, 0x101,
0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x57, 0x101,
0x20, 0x6f, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x74, 0x75, 0x20,
0x4d, 0x101, 0x68, 0x61, 0x72, 0x61, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x72,
0x69, 0x68, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x68, 0x69,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x4d, 0x61, 0x72, 0x69, 0x68, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d,
0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x57, 0x101, 0x20, 0x4d, 0x113, 0x68, 0x69, 0x6b, 0x6f, 0x20, 0x4b, 0x69,
0x77, 0x61, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x20, 0x4d, 0x113, 0x68, 0x69, 0x6b, 0x6f, 0x20, 0x4b, 0x69,
0x77, 0x61, 0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20,
0x4d, 0x113, 0x68, 0x69, 0x6b, 0x6f, 0x20, 0x4b, 0x69, 0x77, 0x61, 0x57,
0x101, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61,
0x72, 0x57, 0x101, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57,
0x101, 0x20, 0x4d, 0x6f, 0x68, 0x69, 0x6b, 0x61, 0x75, 0x57, 0x101, 0x20,
0x4d, 0x6f, 0x68, 0x69, 0x6b, 0x61, 0x75, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4d, 0x6f, 0x68, 0x69,
0x6b, 0x61, 0x75, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57,
0x101, 0x20, 0x50, 0x113, 0x6d, 0x61, 0x57, 0x101, 0x20, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x57, 0x101, 0x20, 0x4e, 0x65, 0x70, 0x14d, 0x72, 0x61, 0x57,
0x101, 0x20, 0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x20, 0x4b, 0x61, 0x6e,
0x61, 0x6b, 0x69, 0x57, 0x101, 0x20, 0x57, 0x68, 0x65, 0x6e, 0x75, 0x61,
0x20, 0x4b, 0x61, 0x6e, 0x61, 0x6b, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x57, 0x68, 0x65, 0x6e,
0x75, 0x61, 0x20, 0x4b, 0x61, 0x6e, 0x61, 0x6b, 0x69, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x41, 0x6f, 0x74, 0x65,
0x61, 0x72, 0x6f, 0x61, 0x57, 0x101, 0x20, 0x41, 0x6f, 0x74, 0x65, 0x61,
0x72, 0x6f, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x41, 0x6f, 0x74, 0x65, 0x61, 0x72, 0x6f, 0x61,
0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x101,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57,
0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65, 0x61, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x101, 0x20,
0x4e, 0x69, 0x75, 0x65, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20,
0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x4e, 0x14d, 0x70, 0x6f,
0x6b, 0x65, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x4d, 0x6f,
0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x4e, 0x14d, 0x70, 0x6f, 0x6b, 0x65,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x4d, 0x6f, 0x75, 0x74, 0x65, 0x72,
0x65, 0x20, 0x4e, 0x14d, 0x70, 0x6f, 0x6b, 0x65, 0x20, 0x41, 0x77, 0x61,
0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x57, 0x101, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d,
0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x57, 0x101, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x52, 0x61, 0x75, 0x6d,
0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x101,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x52,
0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x74, 0x101, 0x6e, 0x65, 0x57, 0x101, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x74, 0x101, 0x6e, 0x65, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x74, 0x101, 0x6e,
0x65, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20,
0x50, 0x101, 0x72, 0x61, 0x75, 0x57, 0x101, 0x20, 0x50, 0x61, 0x70, 0x75,
0x61, 0x20, 0x4e, 0x16b, 0x6b, 0x69, 0x6e, 0x69, 0x57, 0x101, 0x20, 0x50,
0x61, 0x72, 0x61, 0x6b, 0x61, 0x69, 0x57, 0x101, 0x20, 0x50, 0x61, 0x72,
0x61, 0x6b, 0x61, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x50, 0x61, 0x72, 0x61, 0x6b, 0x61, 0x69,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x50,
0x65, 0x72, 0x75, 0x57, 0x101, 0x20, 0x50, 0x65, 0x72, 0x75, 0x20, 0x41,
0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x50,
0x65, 0x72, 0x75, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57,
0x101, 0x20, 0x50, 0x69, 0x72, 0x69, 0x70, 0x12b, 0x6e, 0x69, 0x57, 0x101,
0x20, 0x50, 0x69, 0x72, 0x69, 0x70, 0x12b, 0x6e, 0x69, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x50, 0x69,
0x72, 0x69, 0x70, 0x12b, 0x6e, 0x69, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x6f, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d,
0x6f, 0x75, 0x74, 0x65, 0x72, 0x65, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e,
0x69, 0x78, 0x57, 0x101, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x6d, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x57, 0x101, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101,
0x6e, 0x75, 0x69, 0x20, 0x6f, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x6d, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x101, 0x20, 0x41, 0x77, 0x61, 0x74, 0x65,
0x61, 0x20, 0x6f, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x6d, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x57, 0x101, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x57, 0x101, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x57, 0x101,
0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x57, 0x101,
0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x57, 0x101, 0x20, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x57, 0x101, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x101, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e,
0x75, 0x69, 0x57, 0x101, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20,
0x48, 0x101, 0x6d, 0x6f, 0x61, 0x57, 0x101, 0x20, 0x48, 0x101, 0x6d, 0x6f,
0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x48, 0x101, 0x6d, 0x6f, 0x61, 0x20, 0x41, 0x77, 0x61, 0x74,
0x65, 0x61, 0x57, 0x101, 0x20, 0x48, 0x65, 0x69, 0x6b, 0x65, 0x72, 0x65,
0x57, 0x101, 0x20, 0x48, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x61, 0x20,
0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20,
0x6f, 0x20, 0x4e, 0x67, 0x101, 0x20, 0x4d, 0x6f, 0x74, 0x75, 0x20, 0x48,
0x6f, 0x72, 0x6f, 0x6d, 0x6f, 0x6e, 0x61, 0x57, 0x101, 0x20, 0x48, 0x14d,
0x72, 0x69, 0x61, 0x20, 0x6b, 0x69, 0x20, 0x74, 0x65, 0x20, 0x54, 0x6f,
0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x48, 0x75, 0x72, 0x69, 0x6e, 0x61,
0x6d, 0x65, 0x57, 0x101, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x57, 0x101,
0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x57, 0x101, 0x20, 0x54, 0x61,
0x69, 0x70, 0x65, 0x69, 0x57, 0x101, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x41, 0x77, 0x61,
0x74, 0x65, 0x61, 0x57, 0x101, 0x20, 0x54, 0x61, 0x6b, 0x69, 0x72, 0x69,
0x74, 0x101, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20, 0x54, 0x6f, 0x6b, 0x65,
0x72, 0x61, 0x75, 0x57, 0x101, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57,
0x101, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x72, 0x6f, 0x77,
0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20,
0x43, 0x68, 0x75, 0x75, 0x6b, 0x57, 0x101, 0x20, 0x54, 0x75, 0x6b, 0x75,
0x6d, 0x61, 0x6e, 0x61, 0x74, 0x101, 0x6e, 0x67, 0x61, 0x57, 0x101, 0x20,
0x54, 0x75, 0x6b, 0x75, 0x6d, 0x61, 0x6e, 0x61, 0x74, 0x101, 0x6e, 0x67,
0x61, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57,
0x101, 0x20, 0x54, 0x75, 0x6b, 0x75, 0x6d, 0x61, 0x6e, 0x61, 0x74, 0x101,
0x6e, 0x67, 0x61, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57,
0x101, 0x20, 0x54, 0x16b, 0x77, 0x61, 0x72, 0x75, 0x57, 0x101, 0x20, 0x55,
0x72, 0x75, 0x6b, 0x6f, 0x69, 0x57, 0x101, 0x20, 0x55, 0x72, 0x75, 0x6b,
0x6f, 0x69, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69,
0x57, 0x101, 0x20, 0x55, 0x72, 0x75, 0x6b, 0x6f, 0x69, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x55, 0x68, 0x69, 0x70,
0x65, 0x6b, 0x65, 0x74, 0x101, 0x6e, 0x65, 0x57, 0x101, 0x20, 0x55, 0x68,
0x69, 0x70, 0x65, 0x6b, 0x65, 0x74, 0x101, 0x6e, 0x65, 0x20, 0x41, 0x72,
0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x55, 0x68,
0x69, 0x70, 0x65, 0x6b, 0x65, 0x74, 0x101, 0x6e, 0x65, 0x20, 0x52, 0x61,
0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x57, 0x68, 0x65, 0x6e,
0x75, 0x61, 0x74, 0x16b, 0x57, 0x101, 0x20, 0x57, 0x68, 0x65, 0x6e, 0x75,
0x61, 0x74, 0x16b, 0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75,
0x69, 0x57, 0x101, 0x20, 0x57, 0x68, 0x65, 0x6e, 0x75, 0x61, 0x74, 0x16b,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x50,
0x65, 0x6e, 0x65, 0x68, 0x16b, 0x65, 0x72, 0x61, 0x57, 0x101, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57, 0x101,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x41, 0x72, 0x6f, 0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x101, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x57, 0x101, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57,
0x101, 0x20, 0x6f, 0x20, 0x54, 0x65, 0x20, 0x4d, 0x6f, 0x74, 0x75, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x57, 0x101, 0x20, 0x57, 0x101, 0x72, 0x69, 0x68,
0x69, 0x20, 0x6d, 0x65, 0x20, 0x57, 0x68, 0x75, 0x74, 0x75, 0x6e, 0x61,
0x57, 0x101, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x57, 0x101,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x20, 0x52, 0x61, 0x75, 0x6d, 0x61, 0x74, 0x69,
0x57, 0x101, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x57, 0x101, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x41, 0x72, 0x6f,
0x77, 0x68, 0x101, 0x6e, 0x75, 0x69, 0x57, 0x101, 0x20, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x52,
0x61, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x57, 0x101, 0x20, 0x59, 0x75, 0x6b,
0x6f, 0x6e, 0x90f, 0x915, 0x930, 0x20, 0x935, 0x947, 0x933, 0x90f, 0x915, 0x930,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x910, 0x915,
0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x905, 0x92b, 0x917, 0x93e, 0x923, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x200d, 0x92f, 0x20, 0x906, 0x92b,
0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x942, 0x930,
0x94d, 0x935, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x935,
0x947, 0x933, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x906, 0x92b, 0x94d,
0x930, 0x93f, 0x915, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x906, 0x92b,
0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x906, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x93e,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915,
0x93e, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e,
0x936, 0x20, 0x935, 0x947, 0x933, 0x905, 0x932, 0x94d, 0x92e, 0x93e, 0x91f, 0x940,
0x20, 0x935, 0x947, 0x933, 0x905, 0x932, 0x94d, 0x92e, 0x93e, 0x91f, 0x940, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x905, 0x932, 0x94d,
0x92e, 0x93e, 0x91f, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945,
0x92e, 0x947, 0x91d, 0x949, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945,
0x92e, 0x947, 0x91d, 0x949, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x92e, 0x947, 0x91d, 0x949, 0x928,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x935, 0x947, 0x933,
0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915, 0x947, 0x902, 0x926, 0x94d,
0x930, 0x940, 0x92f, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930,
0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94c, 0x930, 0x94d, 0x935,
0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94c, 0x930, 0x94d,
0x935, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x93e, 0x924, 0x94d,
0x92f, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e,
0x936, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x930, 0x94d, 0x935, 0x924, 0x940, 0x92f,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x930, 0x94d, 0x935, 0x924, 0x940, 0x92f, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f,
0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x945,
0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x945, 0x938,
0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x945, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938,
0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935,
0x947, 0x933, 0x90f, 0x928, 0x93e, 0x921, 0x940, 0x92f, 0x930, 0x20, 0x935, 0x947,
0x933, 0x905, 0x928, 0x93e, 0x926, 0x940, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x935, 0x947, 0x933, 0x905, 0x928, 0x93e, 0x926, 0x940, 0x930, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x935, 0x947, 0x933, 0x90f, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947,
0x933, 0x90f, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x90f, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x938,
0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935,
0x947, 0x933, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x924, 0x93e, 0x909, 0x20, 0x935,
0x947, 0x933, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x924, 0x93e, 0x909, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x915,
0x94d, 0x924, 0x93e, 0x909, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945,
0x915, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x20, 0x935, 0x947, 0x933, 0x905, 0x200d,
0x945, 0x915, 0x94d, 0x91f, 0x94b, 0x92c, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x915, 0x94d, 0x91f, 0x94b,
0x92c, 0x947, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930, 0x947, 0x92c, 0x93f,
0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930, 0x947, 0x92c, 0x93f, 0x92f,
0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x905, 0x930, 0x947, 0x92c, 0x93f, 0x92f, 0x928, 0x20, 0x938, 0x942, 0x930, 0x94d,
0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x905,
0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x905, 0x930, 0x94d, 0x91c,
0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e,
0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20,
0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x93f, 0x928, 0x93e, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f,
0x93f, 0x928, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x906,
0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x909, 0x928, 0x94d,
0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x905, 0x91f, 0x932, 0x93e,
0x902, 0x91f, 0x93f, 0x915, 0x20, 0x935, 0x947, 0x933, 0x905, 0x91f, 0x932, 0x93e,
0x902, 0x91f, 0x93f, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20,
0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20,
0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x911, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x911,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x928, 0x20, 0x92e,
0x927, 0x94d, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x928, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f,
0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x927,
0x94d, 0x92f, 0x2d, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x935, 0x947,
0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x928,
0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x2d, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x911,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x928, 0x20, 0x92e,
0x927, 0x94d, 0x92f, 0x2d, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x938,
0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x911, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x911,
0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x928, 0x20, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x928, 0x20, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x938, 0x942, 0x930,
0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x911, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x911, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x911, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f,
0x92f, 0x928, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x938, 0x942,
0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947,
0x933, 0x905, 0x91d, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x935, 0x947,
0x933, 0x905, 0x91d, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x905, 0x91d, 0x930, 0x92c,
0x948, 0x91c, 0x93e, 0x928, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940,
0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x91d, 0x94b, 0x930, 0x947, 0x938,
0x20, 0x935, 0x947, 0x933, 0x905, 0x200d, 0x945, 0x91d, 0x94b, 0x930, 0x947, 0x938,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x905,
0x200d, 0x945, 0x91d, 0x94b, 0x930, 0x947, 0x938, 0x20, 0x909, 0x928, 0x94d, 0x939,
0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x93e, 0x902, 0x917, 0x932,
0x93e, 0x926, 0x947, 0x936, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92c, 0x93e, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947, 0x936,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x92d, 0x942, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930,
0x93e, 0x91d, 0x93f, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930, 0x93e, 0x91d, 0x93f,
0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940,
0x20, 0x935, 0x947, 0x933, 0x92c, 0x94d, 0x930, 0x941, 0x928, 0x947, 0x908, 0x20,
0x926, 0x93e, 0x930, 0x942, 0x938, 0x932, 0x93e, 0x92e, 0x20, 0x935, 0x947, 0x933,
0x915, 0x947, 0x92a, 0x20, 0x935, 0x94d, 0x939, 0x930, 0x94d, 0x921, 0x947, 0x20,
0x935, 0x947, 0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x94d, 0x939, 0x930, 0x94d,
0x921, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x94d, 0x939, 0x930, 0x94d, 0x921, 0x947,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x91a, 0x93e, 0x92e, 0x94b, 0x930, 0x94b, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x945, 0x925, 0x92e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x945, 0x925, 0x92e,
0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936,
0x20, 0x935, 0x947, 0x933, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x93f, 0x932, 0x940, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x91a,
0x940, 0x928, 0x940, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x940, 0x928, 0x940, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x940,
0x928, 0x940, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915,
0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x94b, 0x908, 0x92c, 0x93e, 0x932,
0x94d, 0x938, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x91a, 0x94b, 0x908, 0x92c, 0x93e, 0x932, 0x94d, 0x938, 0x928, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x916,
0x94d, 0x930, 0x93f, 0x938, 0x92e, 0x938, 0x20, 0x92c, 0x947, 0x91f, 0x20, 0x935,
0x947, 0x933, 0x915, 0x949, 0x915, 0x94b, 0x938, 0x20, 0x92c, 0x947, 0x91f, 0x947,
0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915,
0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939,
0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x915, 0x941, 0x915, 0x20, 0x92c,
0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x915, 0x941, 0x915, 0x20, 0x92c,
0x947, 0x91f, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x915, 0x941, 0x915, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x905,
0x930, 0x94d, 0x927, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x935, 0x947,
0x933, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e,
0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936,
0x20, 0x935, 0x947, 0x933, 0x921, 0x947, 0x935, 0x94d, 0x939, 0x93f, 0x938, 0x20,
0x935, 0x947, 0x933, 0x921, 0x94d, 0x92f, 0x941, 0x92e, 0x949, 0x928, 0x94d, 0x91f,
0x2d, 0x921, 0x94d, 0x92f, 0x941, 0x930, 0x94d, 0x935, 0x93f, 0x932, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x924, 0x93f, 0x92e, 0x94b,
0x930, 0x20, 0x935, 0x947, 0x933, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x92c,
0x947, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20,
0x92c, 0x947, 0x91f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x92c, 0x947, 0x91f, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92e,
0x927, 0x94d, 0x200d, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f,
0x928, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x200d, 0x92f, 0x20, 0x92f,
0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x200d, 0x92f, 0x20,
0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x909, 0x928, 0x94d,
0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d,
0x935, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x92f, 0x941, 0x930, 0x94b,
0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x92f, 0x941, 0x930,
0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933,
0x940, 0x20, 0x935, 0x947, 0x933, 0x905, 0x917, 0x94d, 0x930, 0x2d, 0x92a, 0x94c,
0x930, 0x94d, 0x935, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b,
0x92a, 0x940, 0x92f, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x92f, 0x941,
0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20,
0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x93f, 0x92f, 0x928, 0x20, 0x909, 0x928, 0x94d,
0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x949, 0x915, 0x932,
0x902, 0x921, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x92b,
0x949, 0x915, 0x932, 0x902, 0x921, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x949, 0x915,
0x932, 0x902, 0x921, 0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x909, 0x928, 0x94d,
0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x91c, 0x940,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92b,
0x93f, 0x91c, 0x940, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x92b, 0x94d, 0x930, 0x947, 0x902, 0x91a, 0x20, 0x926, 0x915,
0x94d, 0x937, 0x93f, 0x923, 0x20, 0x906, 0x923, 0x93f, 0x20, 0x905, 0x902, 0x91f,
0x93e, 0x930, 0x94d, 0x915, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x935, 0x947, 0x933,
0x917, 0x945, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x20, 0x935, 0x947,
0x933, 0x917, 0x945, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930, 0x20, 0x935, 0x947,
0x933, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x949, 0x930, 0x94d,
0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940,
0x20, 0x935, 0x947, 0x933, 0x917, 0x93f, 0x932, 0x94d, 0x92c, 0x930, 0x94d, 0x91f,
0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x93f, 0x91a, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x942, 0x930,
0x94d, 0x935, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x942,
0x930, 0x94d, 0x935, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x932, 0x901, 0x921, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x901, 0x921,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x917, 0x941, 0x906, 0x92e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935,
0x947, 0x933, 0x916, 0x93e, 0x921, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e,
0x923, 0x20, 0x935, 0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x905, 0x932,
0x942, 0x936, 0x928, 0x20, 0x935, 0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d,
0x905, 0x932, 0x942, 0x936, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x905, 0x932, 0x942,
0x936, 0x928, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915,
0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915,
0x93e, 0x901, 0x917, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x939,
0x94b, 0x935, 0x94d, 0x939, 0x94d, 0x921, 0x20, 0x935, 0x947, 0x933, 0x939, 0x94b,
0x935, 0x94d, 0x939, 0x94d, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x939, 0x94b, 0x935, 0x94d, 0x939, 0x94d, 0x921, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92d,
0x93e, 0x930, 0x924, 0x940, 0x92f, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x939, 0x93f, 0x902, 0x926, 0x92e, 0x939, 0x93e, 0x938,
0x93e, 0x917, 0x930, 0x20, 0x935, 0x947, 0x933, 0x907, 0x902, 0x921, 0x94b, 0x91a,
0x93e, 0x92f, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x927, 0x94d, 0x200d,
0x92f, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x93e, 0x924, 0x94d,
0x92f, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e,
0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20,
0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x935,
0x947, 0x933, 0x907, 0x930, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930,
0x93e, 0x923, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x907, 0x930, 0x93e, 0x923, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a,
0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930, 0x94d,
0x915, 0x941, 0x924, 0x94d, 0x938, 0x915, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930,
0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924,
0x94d, 0x938, 0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x907, 0x938, 0x94d, 0x930, 0x93e, 0x92f, 0x932, 0x907, 0x938,
0x94d, 0x930, 0x93e, 0x92f, 0x932, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x907, 0x938, 0x94d, 0x930, 0x93e, 0x92f, 0x932, 0x20,
0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20,
0x935, 0x947, 0x933, 0x91c, 0x92a, 0x93e, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x91c, 0x92a, 0x93e, 0x928, 0x20, 0x938,
0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x93e, 0x935, 0x94d,
0x939, 0x932, 0x94b, 0x938, 0x94d, 0x915, 0x2d, 0x20, 0x915, 0x93e, 0x92e, 0x91a,
0x93e, 0x91f, 0x94d, 0x938, 0x94d, 0x915, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a,
0x947, 0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x93e, 0x935, 0x94d, 0x939, 0x932, 0x94b,
0x938, 0x94d, 0x915, 0x2d, 0x20, 0x915, 0x93e, 0x92e, 0x91a, 0x93e, 0x91f, 0x94d,
0x938, 0x94d, 0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935,
0x947, 0x933, 0x92a, 0x947, 0x91f, 0x94d, 0x930, 0x94b, 0x92a, 0x93e, 0x935, 0x94d,
0x939, 0x932, 0x94b, 0x938, 0x94d, 0x915, 0x2d, 0x20, 0x915, 0x93e, 0x92e, 0x91a,
0x93e, 0x91f, 0x94d, 0x938, 0x94d, 0x915, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x20, 0x915, 0x91d, 0x93e, 0x915, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x20, 0x915, 0x91d, 0x93e, 0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x935, 0x947, 0x933, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x928, 0x20, 0x92a, 0x94d,
0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x930, 0x93f,
0x92f, 0x928, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d, 0x930, 0x915,
0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94b, 0x938, 0x94d, 0x930, 0x93e,
0x908, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928,
0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x915, 0x94d, 0x930, 0x93e, 0x938,
0x94d, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x915, 0x94d, 0x938, 0x20, 0x909,
0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x915, 0x93f,
0x930, 0x917, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933,
0x932, 0x902, 0x915, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x932, 0x93e, 0x907, 0x928,
0x20, 0x92c, 0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x932, 0x949, 0x930,
0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20,
0x939, 0x94b, 0x935, 0x947, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d,
0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x915, 0x93e, 0x90a,
0x20, 0x935, 0x947, 0x933, 0x92e, 0x915, 0x93e, 0x90a, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x92e, 0x915, 0x93e, 0x90a, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x935, 0x947, 0x933, 0x92e, 0x945, 0x917, 0x93e, 0x921, 0x928, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x945, 0x917, 0x93e, 0x921, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x945, 0x917, 0x93e, 0x921, 0x928,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x93e, 0x932, 0x926, 0x93f, 0x935, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x93e,
0x930, 0x94d, 0x915, 0x94d, 0x935, 0x947, 0x938, 0x93e, 0x938, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x93e, 0x930, 0x94d, 0x936, 0x932, 0x20, 0x92c, 0x947, 0x91f, 0x947,
0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x935,
0x947, 0x933, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x930, 0x940, 0x936,
0x938, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947,
0x933, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x20, 0x92a, 0x945,
0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x947, 0x915,
0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x20, 0x92a, 0x945, 0x938, 0x93f, 0x92b, 0x93f,
0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933,
0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x94b, 0x20, 0x92a, 0x945, 0x938,
0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d,
0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x909, 0x932, 0x93e, 0x928,
0x20, 0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x935, 0x947, 0x933, 0x909, 0x932,
0x93e, 0x928, 0x20, 0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x909, 0x932, 0x93e, 0x928, 0x20,
0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933,
0x940, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92e, 0x949,
0x938, 0x94d, 0x915, 0x94b, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940,
0x20, 0x935, 0x947, 0x933, 0x928, 0x909, 0x930, 0x941, 0x20, 0x935, 0x947, 0x933,
0x928, 0x94d, 0x92f, 0x942, 0x20, 0x915, 0x945, 0x932, 0x947, 0x921, 0x94b, 0x928,
0x93f, 0x92f, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x942, 0x20,
0x915, 0x945, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f,
0x942, 0x20, 0x915, 0x945, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x928, 0x94d, 0x92f, 0x942, 0x91d, 0x940, 0x932, 0x902, 0x921, 0x20, 0x935, 0x947,
0x933, 0x928, 0x94d, 0x92f, 0x942, 0x91d, 0x940, 0x932, 0x902, 0x921, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f,
0x942, 0x91d, 0x940, 0x932, 0x902, 0x921, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f,
0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94d,
0x92f, 0x942, 0x20, 0x92b, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x902, 0x921, 0x20,
0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x92b, 0x93e, 0x909, 0x902,
0x921, 0x932, 0x902, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x92b, 0x93e, 0x909, 0x902,
0x921, 0x932, 0x902, 0x921, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d,
0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x928, 0x949, 0x930, 0x92b,
0x94b, 0x915, 0x20, 0x92c, 0x947, 0x91f, 0x20, 0x935, 0x947, 0x933, 0x928, 0x949,
0x930, 0x92b, 0x94b, 0x915, 0x20, 0x92c, 0x947, 0x91f, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x928, 0x949, 0x930, 0x92b, 0x94b,
0x915, 0x20, 0x92c, 0x947, 0x91f, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a,
0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x930, 0x94d,
0x928, 0x93e, 0x902, 0x921, 0x94b, 0x20, 0x926, 0x940, 0x20, 0x928, 0x94b, 0x930,
0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x921, 0x94b,
0x20, 0x926, 0x940, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e,
0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x92e, 0x930, 0x93f, 0x928, 0x93e, 0x20,
0x92c, 0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x928, 0x94b, 0x935, 0x94b,
0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x935, 0x947,
0x933, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935,
0x947, 0x933, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92a, 0x94d, 0x930,
0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x913, 0x92e, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x93e,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x909, 0x928, 0x94d, 0x939,
0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x93e, 0x92a, 0x941, 0x906,
0x20, 0x928, 0x94d, 0x92f, 0x942, 0x20, 0x917, 0x93f, 0x928, 0x940, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x945, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x935,
0x947, 0x933, 0x92a, 0x945, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x945, 0x930,
0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933,
0x940, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x935, 0x947,
0x933, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x909, 0x928, 0x94d,
0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x932, 0x93f,
0x92a, 0x93e, 0x907, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92b, 0x93f, 0x932, 0x93f,
0x92a, 0x93e, 0x907, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93e, 0x907, 0x928, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x200d,
0x92b, 0x94b, 0x928, 0x93f, 0x915, 0x94d, 0x938, 0x20, 0x92c, 0x947, 0x91f, 0x947,
0x20, 0x935, 0x947, 0x933, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x92f,
0x930, 0x20, 0x906, 0x923, 0x93f, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947,
0x932, 0x94b, 0x928, 0x20, 0x935, 0x947, 0x933, 0x938, 0x947, 0x902, 0x91f, 0x20,
0x92a, 0x93f, 0x92f, 0x930, 0x947, 0x20, 0x906, 0x923, 0x93f, 0x20, 0x92e, 0x93f,
0x915, 0x94d, 0x935, 0x947, 0x932, 0x94b, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a,
0x93f, 0x92f, 0x930, 0x947, 0x20, 0x906, 0x923, 0x93f, 0x20, 0x92e, 0x93f, 0x915,
0x94d, 0x935, 0x947, 0x932, 0x94b, 0x928, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f,
0x92a, 0x94d, 0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x93f,
0x91f, 0x915, 0x948, 0x930, 0x94d, 0x928, 0x20, 0x935, 0x947, 0x933, 0x92a, 0x94b,
0x928, 0x945, 0x92a, 0x947, 0x20, 0x935, 0x947, 0x933, 0x915, 0x93c, 0x93f, 0x91d,
0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x915,
0x93c, 0x93f, 0x91d, 0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x915, 0x93c, 0x93f, 0x91d,
0x940, 0x932, 0x94b, 0x930, 0x94d, 0x921, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933,
0x938, 0x916, 0x932, 0x93f, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923,
0x20, 0x935, 0x947, 0x933, 0x938, 0x916, 0x932, 0x93f, 0x928, 0x20, 0x909, 0x928,
0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x938, 0x92e, 0x93e,
0x930, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e, 0x930, 0x93e, 0x20,
0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e,
0x930, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93e, 0x92e, 0x94b, 0x906,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x938,
0x93e, 0x92e, 0x94b, 0x906, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d,
0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x938, 0x93f, 0x902, 0x917,
0x93e, 0x92a, 0x942, 0x930, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x949, 0x928, 0x20, 0x92c,
0x947, 0x91f, 0x947, 0x20, 0x935, 0x947, 0x933, 0x924, 0x948, 0x92a, 0x947, 0x908,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x924,
0x948, 0x92a, 0x947, 0x908, 0x20, 0x938, 0x942, 0x930, 0x94d, 0x92f, 0x92a, 0x94d,
0x930, 0x915, 0x93e, 0x936, 0x20, 0x935, 0x947, 0x933, 0x924, 0x93e, 0x91c, 0x93f,
0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x91f,
0x94b, 0x915, 0x947, 0x932, 0x93e, 0x90a, 0x20, 0x935, 0x947, 0x933, 0x91f, 0x94b,
0x902, 0x917, 0x93e, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x909, 0x928, 0x94d, 0x939,
0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x91a, 0x942, 0x915, 0x20, 0x935,
0x947, 0x933, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947, 0x933, 0x924, 0x941, 0x930, 0x94d,
0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92a,
0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x924, 0x941, 0x930,
0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20,
0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x924,
0x941, 0x935, 0x93e, 0x932, 0x942, 0x20, 0x935, 0x947, 0x933, 0x909, 0x930, 0x941,
0x917, 0x94d, 0x935, 0x947, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x909,
0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x909, 0x91d,
0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x935, 0x947,
0x933, 0x909, 0x91d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x909,
0x91d, 0x92c, 0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x909,
0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x935, 0x93e,
0x928, 0x941, 0x906, 0x924, 0x941, 0x20, 0x935, 0x947, 0x933, 0x935, 0x93e, 0x928,
0x941, 0x906, 0x924, 0x941, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x935, 0x93e, 0x928, 0x941, 0x906, 0x924, 0x941, 0x20, 0x909,
0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935, 0x947, 0x933, 0x935, 0x94d,
0x939, 0x947, 0x928, 0x947, 0x91d, 0x941, 0x90f, 0x932, 0x93e, 0x20, 0x935, 0x947,
0x933, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924,
0x94b, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935, 0x947,
0x933, 0x935, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924,
0x94b, 0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935,
0x947, 0x933, 0x935, 0x94d, 0x939, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d,
0x930, 0x93e, 0x921, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20, 0x935,
0x947, 0x933, 0x935, 0x94d, 0x939, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d,
0x930, 0x93e, 0x921, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20,
0x935, 0x947, 0x933, 0x935, 0x94d, 0x939, 0x94b, 0x938, 0x94d, 0x91f, 0x949, 0x915,
0x20, 0x935, 0x947, 0x933, 0x935, 0x947, 0x915, 0x20, 0x92c, 0x947, 0x91f, 0x20,
0x935, 0x947, 0x933, 0x935, 0x949, 0x932, 0x93f, 0x938, 0x20, 0x906, 0x923, 0x93f,
0x20, 0x92b, 0x941, 0x91f, 0x941, 0x928, 0x93e, 0x20, 0x935, 0x947, 0x933, 0x92f,
0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x915, 0x20, 0x935, 0x947, 0x933, 0x92f,
0x93e, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x915, 0x20, 0x92a, 0x94d, 0x930, 0x92e,
0x93e, 0x923, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x915, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935,
0x947, 0x933, 0x92f, 0x947, 0x915, 0x924, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d,
0x917, 0x20, 0x935, 0x947, 0x933, 0x92f, 0x947, 0x915, 0x924, 0x930, 0x93f, 0x928,
0x92c, 0x930, 0x94d, 0x917, 0x20, 0x92a, 0x94d, 0x930, 0x92e, 0x93e, 0x923, 0x20,
0x935, 0x947, 0x933, 0x92f, 0x947, 0x915, 0x924, 0x930, 0x93f, 0x928, 0x92c, 0x930,
0x94d, 0x917, 0x20, 0x909, 0x928, 0x94d, 0x939, 0x93e, 0x933, 0x940, 0x20, 0x935,
0x947, 0x933, 0x92f, 0x941, 0x915, 0x94b, 0x928, 0x20, 0x935, 0x947, 0x933, 0x410,
0x444, 0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x410,
0x444, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x4e8,
0x43c, 0x43d, 0x4e9, 0x434, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x438, 0x439,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440,
0x443, 0x443, 0x43d, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x444, 0x440, 0x438,
0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x43b,
0x44f, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x44b, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x422, 0x4e9, 0x432, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x44d, 0x440, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x44d, 0x440, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x423, 0x443, 0x43b, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x423, 0x443, 0x43b,
0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x423, 0x443, 0x43b, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e, 0x43c, 0x445, 0x43e, 0x43d, 0x20,
0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e,
0x43c, 0x445, 0x43e, 0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x41d, 0x43e, 0x43c, 0x445, 0x43e, 0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430, 0x439,
0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410, 0x43f,
0x438, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x410, 0x43f,
0x438, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x43f, 0x438, 0x430, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x410, 0x440, 0x430, 0x431, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x410, 0x440,
0x430, 0x431, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x430, 0x431, 0x44b, 0x43d, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440,
0x443, 0x443, 0x43d, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x44b,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x44b,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x44b, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x422, 0x4e9, 0x432, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439,
0x43d, 0x20, 0x442, 0x4e9, 0x432, 0x20, 0x431, 0x430, 0x440, 0x443, 0x443, 0x43d,
0x20, 0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x442, 0x4e9,
0x432, 0x20, 0x431, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x44d, 0x440, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x439, 0x43d, 0x20, 0x442, 0x4e9, 0x432, 0x20, 0x431, 0x430, 0x440, 0x443, 0x443,
0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x437,
0x4af, 0x4af, 0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x4af,
0x4af, 0x43d, 0x20, 0x44d, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x439,
0x43d, 0x20, 0x431, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x44d, 0x440, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x439, 0x43d, 0x20, 0x431, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x44d, 0x440,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x436, 0x430, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x436,
0x430, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x436,
0x430, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x410, 0x437, 0x43e, 0x440, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x410, 0x437,
0x43e, 0x440, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x410, 0x437, 0x43e, 0x440, 0x44b, 0x43d, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x43d, 0x433,
0x43b, 0x430, 0x434, 0x435, 0x448, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x438, 0x439, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x438, 0x439,
0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x443,
0x442, 0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x43e, 0x43b, 0x438,
0x432, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x440, 0x430, 0x437,
0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x440, 0x430,
0x437, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x411, 0x440, 0x430, 0x437, 0x438,
0x43b, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x439, 0x20, 0x414, 0x430, 0x440, 0x443,
0x441, 0x441, 0x430, 0x43b, 0x430, 0x43c, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440,
0x434, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435,
0x440, 0x434, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x427, 0x430, 0x43c, 0x43e, 0x440, 0x440, 0x43e, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x44b, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x44b, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x427, 0x430, 0x442, 0x435, 0x43c, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x427, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x427, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x427, 0x438, 0x43b,
0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x425, 0x44f, 0x442, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x425,
0x44f, 0x442, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x425, 0x44f, 0x442, 0x430, 0x434,
0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x427,
0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430,
0x43d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b,
0x441, 0x430, 0x43d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x41a, 0x440, 0x438, 0x441, 0x43c, 0x430, 0x441, 0x20,
0x430, 0x440, 0x43b, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x43e, 0x43a,
0x43e, 0x441, 0x20, 0x430, 0x440, 0x43b, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x41a, 0x4af, 0x4af, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x440,
0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x4af,
0x4af, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434,
0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x41a, 0x4af, 0x4af, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x430,
0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x445, 0x430, 0x433, 0x430,
0x441, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x443,
0x431, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x443, 0x431, 0x44b, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x41a, 0x443, 0x431, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x414, 0x44d, 0x432, 0x438, 0x441, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x42e,
0x440, 0x432, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x417,
0x4af, 0x4af, 0x43d, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x44b, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x418, 0x441, 0x43b, 0x430,
0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d,
0x20, 0x418, 0x441, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af,
0x4af, 0x43d, 0x20, 0x418, 0x441, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x42d, 0x43a, 0x432, 0x430,
0x434, 0x43e, 0x440, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x422, 0x4e9, 0x432, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x422, 0x4e9, 0x432, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b,
0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x410, 0x43b,
0x441, 0x20, 0x434, 0x43e, 0x440, 0x43d, 0x43e, 0x434, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443,
0x443, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x438, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434,
0x438, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x438, 0x439, 0x43d,
0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x424, 0x438, 0x436, 0x438, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x424, 0x438, 0x436, 0x438, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x424, 0x438, 0x436, 0x438, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x424, 0x440, 0x430, 0x43d,
0x446, 0x44b, 0x43d, 0x20, 0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x44b,
0x43d, 0x20, 0x4e8, 0x43c, 0x43d, 0x4e9, 0x434, 0x20, 0x431, 0x430, 0x20, 0x410,
0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x434, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441,
0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x413, 0x430, 0x43c, 0x431, 0x44c, 0x435,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x413, 0x4af, 0x440, 0x436,
0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x413, 0x4af, 0x440, 0x436, 0x438,
0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x446, 0x430, 0x433, 0x413, 0x4af, 0x440, 0x436, 0x438, 0x439, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x413, 0x438, 0x43b, 0x44c, 0x431,
0x435, 0x440, 0x442, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443,
0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x413, 0x440, 0x438, 0x43d, 0x432,
0x438, 0x447, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af,
0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x417, 0x4af, 0x4af, 0x43d,
0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443,
0x443, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x411,
0x430, 0x440, 0x443, 0x443, 0x43d, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x41f, 0x435, 0x440, 0x441, 0x438, 0x439, 0x43d, 0x20, 0x431, 0x443, 0x43b,
0x430, 0x43d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x413, 0x430,
0x439, 0x430, 0x43d, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x425, 0x430, 0x432, 0x430, 0x439, 0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x44b,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x425, 0x430, 0x432, 0x430, 0x439, 0x2d, 0x410,
0x43b, 0x435, 0x443, 0x442, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x425, 0x430, 0x432, 0x430, 0x439,
0x2d, 0x410, 0x43b, 0x435, 0x443, 0x442, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e,
0x43d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x425, 0x43e, 0x43d,
0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x425, 0x43e,
0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x425, 0x43e, 0x432, 0x434, 0x44b,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x425, 0x43e, 0x432, 0x434, 0x44b, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x425, 0x43e, 0x432, 0x434, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x42d, 0x43d, 0x44d, 0x442, 0x445, 0x44d, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x42d, 0x43d, 0x44d, 0x442, 0x445, 0x44d, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x42d, 0x43d, 0x44d, 0x442, 0x445, 0x44d, 0x433, 0x2d, 0x425, 0x44f,
0x442, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x445, 0x43e, 0x439, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x422, 0x4e9, 0x432, 0x20, 0x418, 0x43d, 0x434,
0x43e, 0x43d, 0x435, 0x437, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x417,
0x4af, 0x4af, 0x43d, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d,
0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x418, 0x440, 0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433,
0x418, 0x440, 0x430, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x418, 0x440, 0x430, 0x43d, 0x44b, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x42d, 0x440, 0x445, 0x4af,
0x4af, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x42d, 0x440, 0x445,
0x4af, 0x4af, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x42d, 0x440, 0x445, 0x4af, 0x4af,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x418, 0x437, 0x440, 0x430, 0x438, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x42f, 0x43f, 0x43e, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x42f, 0x43f, 0x43e, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x42f, 0x43f, 0x43e,
0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x417,
0x4af, 0x4af, 0x43d, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x440, 0x443, 0x443, 0x43d,
0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x421, 0x43e, 0x43b, 0x43e, 0x43d, 0x433, 0x43e, 0x441, 0x44b,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x421, 0x43e, 0x43b, 0x43e, 0x43d, 0x433, 0x43e,
0x441, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x421, 0x43e, 0x43b, 0x43e, 0x43d, 0x433, 0x43e, 0x441,
0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41a,
0x43e, 0x441, 0x440, 0x44d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f,
0x440, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x437, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41b, 0x430, 0x439, 0x43d, 0x20, 0x430,
0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41b,
0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x443, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x443,
0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x44b, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x430,
0x433, 0x430, 0x434, 0x430, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x44b, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x430, 0x440, 0x43a, 0x435, 0x437, 0x44b,
0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x41c, 0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x44b, 0x43d, 0x20,
0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41c,
0x430, 0x432, 0x440, 0x438, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x2d, 0x41d,
0x43e, 0x43c, 0x445, 0x43e, 0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x2d, 0x41d,
0x43e, 0x43c, 0x445, 0x43e, 0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x2d, 0x41d, 0x43e, 0x43c, 0x445,
0x43e, 0x43d, 0x20, 0x434, 0x430, 0x43b, 0x430, 0x439, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x423, 0x43b, 0x430, 0x430, 0x43d, 0x431,
0x430, 0x430, 0x442, 0x430, 0x440, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x423,
0x43b, 0x430, 0x430, 0x43d, 0x431, 0x430, 0x430, 0x442, 0x430, 0x440, 0x44b, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x423, 0x43b, 0x430, 0x430, 0x43d, 0x431, 0x430, 0x430, 0x442, 0x430, 0x440,
0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41c,
0x43e, 0x441, 0x43a, 0x432, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c,
0x430, 0x440, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x430, 0x443, 0x440,
0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x411, 0x430, 0x43b,
0x431, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x428, 0x438, 0x43d, 0x44d, 0x20,
0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x428, 0x438, 0x43d, 0x44d, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434,
0x43e, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x428, 0x438, 0x43d, 0x44d, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x428, 0x438, 0x43d, 0x44d, 0x20, 0x417,
0x435, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x428,
0x438, 0x43d, 0x44d, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x44b, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x428, 0x438, 0x43d, 0x44d, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434,
0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41d,
0x44c, 0x44e, 0x2d, 0x424, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x44c, 0x44e, 0x2d, 0x424, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x44b, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x44c,
0x44e, 0x2d, 0x424, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x44b,
0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x438,
0x443, 0x44d, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e,
0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x430, 0x440, 0x43b, 0x44b, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x430,
0x440, 0x43b, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a,
0x20, 0x430, 0x440, 0x43b, 0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20,
0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x430, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x430, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e,
0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x430, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431,
0x438, 0x440, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41d, 0x43e, 0x432, 0x43e,
0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41e, 0x43c, 0x441, 0x43a, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41e, 0x43c, 0x441, 0x43a, 0x438, 0x439,
0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x41e, 0x43c, 0x441, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430,
0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41f,
0x430, 0x43b, 0x430, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x41f, 0x430, 0x43f, 0x443, 0x430, 0x20, 0x428, 0x438, 0x43d, 0x44d, 0x20, 0x413,
0x432, 0x438, 0x43d, 0x435, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430,
0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x430, 0x440,
0x430, 0x433, 0x432, 0x430, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x41f, 0x435, 0x440, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x41f, 0x435, 0x440, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x41f, 0x435, 0x440, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438,
0x43d, 0x438, 0x439, 0x20, 0x446, 0x430, 0x433, 0x424, 0x438, 0x43b, 0x438, 0x43f,
0x43f, 0x438, 0x43d, 0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f,
0x438, 0x43d, 0x438, 0x439, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x20, 0x430, 0x440, 0x43b, 0x44b,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c,
0x435, 0x440, 0x20, 0x431, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f,
0x44c, 0x435, 0x440, 0x20, 0x431, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b,
0x43e, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c, 0x435,
0x440, 0x20, 0x431, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d,
0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x41f, 0x438,
0x442, 0x43a, 0x435, 0x440, 0x43d, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x41f, 0x451, 0x43d, 0x44c, 0x44f, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x420, 0x435, 0x44e, 0x43d, 0x438, 0x43e, 0x43d, 0x44b, 0x20, 0x446, 0x430,
0x433, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x44b, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x44b, 0x20, 0x437, 0x443, 0x43d,
0x44b, 0x20, 0x446, 0x430, 0x433, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x433,
0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x446, 0x430, 0x433, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x421, 0x435,
0x439, 0x448, 0x435, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443,
0x443, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x421, 0x438, 0x43d, 0x433,
0x430, 0x43f, 0x443, 0x440, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x421, 0x43e,
0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x44b, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443,
0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x4e8, 0x43c, 0x43d, 0x4e9, 0x434,
0x20, 0x416, 0x43e, 0x440, 0x436, 0x438, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x421, 0x443, 0x440, 0x438, 0x43d, 0x430, 0x43c, 0x44b, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x421, 0x451, 0x432, 0x430, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x422, 0x430, 0x438, 0x442, 0x438, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x446, 0x430, 0x433, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x43d,
0x20, 0x446, 0x430, 0x433, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433,
0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b,
0x20, 0x446, 0x430, 0x433, 0x422, 0x430, 0x436, 0x438, 0x43a, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x422, 0x43e, 0x43a, 0x435, 0x43b,
0x430, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x422, 0x43e,
0x43d, 0x433, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x422,
0x43e, 0x43d, 0x433, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x422, 0x43e, 0x43d,
0x433, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20,
0x446, 0x430, 0x433, 0x427, 0x4af, 0x4af, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446,
0x430, 0x433, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x422, 0x443, 0x440, 0x43a, 0x43c,
0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x422, 0x443, 0x440,
0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x422, 0x443, 0x432, 0x430, 0x43b,
0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430,
0x439, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x423,
0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x44b, 0x20, 0x446,
0x430, 0x433, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x44b, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446,
0x430, 0x433, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x44b, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430,
0x433, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x433, 0x438, 0x439, 0x43d,
0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x412, 0x435, 0x43d,
0x435, 0x441, 0x443, 0x44d, 0x43b, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433,
0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432,
0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x438, 0x439, 0x43d, 0x20,
0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43e, 0x43b, 0x433,
0x43e, 0x433, 0x440, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x412,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x412,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x44b, 0x43d, 0x20, 0x437,
0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x412, 0x43e, 0x441, 0x442, 0x43e,
0x43a, 0x438, 0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x423, 0x44d, 0x43a, 0x438,
0x439, 0x43d, 0x20, 0x430, 0x440, 0x43b, 0x443, 0x443, 0x434, 0x44b, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x423, 0x43e, 0x43b, 0x43b, 0x438, 0x441, 0x20, 0x431, 0x430,
0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20,
0x446, 0x430, 0x433, 0x42f, 0x43a, 0x443, 0x442, 0x44b, 0x43d, 0x20, 0x446, 0x430,
0x433, 0x42f, 0x43a, 0x443, 0x442, 0x44b, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x42f, 0x43a, 0x443, 0x442,
0x44b, 0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x415,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x438,
0x439, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x446, 0x430, 0x433, 0x415, 0x43a,
0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x438, 0x439,
0x43d, 0x20, 0x437, 0x443, 0x43d, 0x44b, 0x20, 0x446, 0x430, 0x433, 0x42e, 0x43a,
0x43e, 0x43d, 0x20, 0x446, 0x430, 0x433, 0x438, 0x439, 0x43d, 0x20, 0x431, 0x4af,
0x441, 0x1832, 0x1825, 0x182a, 0x20, 0x1834, 0x1820, 0x182d, 0x1833, 0x1825, 0x182a, 0x20,
0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20,
0x1834, 0x1820, 0x182d, 0x1832, 0x1825, 0x182a, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e,
0x180e, 0x180e, 0x1824, 0x20, 0x1834, 0x1820, 0x182d, 0x1835, 0x1821, 0x182d, 0x1825, 0x1828,
0x20, 0x1821, 0x1837, 0x182d, 0x1821, 0x20, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1834,
0x1820, 0x182d, 0x1835, 0x1821, 0x182d, 0x1825, 0x1828, 0x20, 0x1821, 0x1837, 0x182d, 0x1821,
0x20, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b,
0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1835, 0x1821, 0x182d,
0x1825, 0x1828, 0x20, 0x1821, 0x1837, 0x182d, 0x1821, 0x20, 0x180e, 0x1822, 0x1822, 0x1828,
0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x180e, 0x1824, 0x20, 0x1834, 0x1820,
0x182d, 0x1820, 0x182d, 0x1823, 0x182f, 0x1820, 0x20, 0x180e, 0x180e, 0x180e, 0x180e, 0x1822,
0x1822, 0x1828, 0x20, 0x1834, 0x1820, 0x182d, 0x1820, 0x182d, 0x1823, 0x182f, 0x1820, 0x20,
0x180e, 0x180e, 0x180e, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820,
0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1820,
0x182d, 0x1823, 0x182f, 0x1820, 0x20, 0x180e, 0x1822, 0x1822, 0x1828, 0x20, 0x1835, 0x1823,
0x1828, 0x20, 0x180e, 0x180e, 0x1824, 0x20, 0x1834, 0x1820, 0x182d, 0x1828, 0x1823, 0x182e,
0x1823, 0x182c, 0x1820, 0x1828, 0x20, 0x1833, 0x1820, 0x182f, 0x1820, 0x1822, 0x20, 0x180e,
0x1822, 0x1822, 0x1828, 0x20, 0x1834, 0x1820, 0x182d, 0x1828, 0x1823, 0x182e, 0x1823, 0x182c,
0x1820, 0x1828, 0x20, 0x1833, 0x1820, 0x182f, 0x1820, 0x1822, 0x20, 0x180e, 0x180e, 0x1822,
0x1822, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837,
0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1828, 0x1823, 0x182e, 0x1823, 0x182c, 0x1820,
0x1828, 0x20, 0x1833, 0x1820, 0x182f, 0x1820, 0x1822, 0x20, 0x180e, 0x1822, 0x1822, 0x1828,
0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x180e, 0x182a, 0x20, 0x1834, 0x1820,
0x182d, 0x1820, 0x1832, 0x180b, 0x182f, 0x1820, 0x1828, 0x180b, 0x1832, 0x180b, 0x20, 0x180e,
0x1824, 0x1828, 0x20, 0x1834, 0x1820, 0x182d, 0x1820, 0x1832, 0x180b, 0x182f, 0x1820, 0x1828,
0x180b, 0x1832, 0x20, 0x180e, 0x1824, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828,
0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x1820, 0x1832,
0x180b, 0x182f, 0x1820, 0x1828, 0x180b, 0x1832, 0x20, 0x180e, 0x1824, 0x1828, 0x20, 0x1835,
0x1823, 0x1828, 0x20, 0x180e, 0x182a, 0x20, 0x1834, 0x1820, 0x182d, 0x1832, 0x1825, 0x182a,
0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822,
0x1828, 0x20, 0x1834, 0x1820, 0x182d, 0x1832, 0x1825, 0x182a, 0x20, 0x1821, 0x1838, 0x1823,
0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20, 0x1830, 0x1832,
0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820,
0x182d, 0x1832, 0x1825, 0x182a, 0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b,
0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e,
0x1824, 0x20, 0x1834, 0x1820, 0x182d, 0x1835, 0x1821, 0x182d, 0x1826, 0x1828, 0x20, 0x1821,
0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20,
0x1834, 0x1820, 0x182d, 0x1835, 0x1821, 0x182d, 0x1826, 0x1828, 0x20, 0x1821, 0x1838, 0x1823,
0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20, 0x1830, 0x1832,
0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820, 0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820,
0x182d, 0x1835, 0x1821, 0x182d, 0x1826, 0x1828, 0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837,
0x1824, 0x182b, 0x1820, 0x20, 0x1822, 0x1822, 0x1828, 0x20, 0x1835, 0x1823, 0x1828, 0x20,
0x180e, 0x180e, 0x1824, 0x20, 0x1834, 0x1820, 0x182d, 0x182a, 0x1820, 0x1837, 0x1820, 0x182d,
0x1823, 0x1828, 0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20,
0x1822, 0x1822, 0x1828, 0x20, 0x1834, 0x1820, 0x182d, 0x182a, 0x1820, 0x1837, 0x1820, 0x182d,
0x1823, 0x1828, 0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20,
0x1822, 0x1822, 0x1828, 0x20, 0x1830, 0x1832, 0x180b, 0x1820, 0x1828, 0x180b, 0x1833, 0x1820,
0x1837, 0x1833, 0x180b, 0x20, 0x1834, 0x1820, 0x182d, 0x182a, 0x1820, 0x1837, 0x1820, 0x182d,
0x1823, 0x1828, 0x20, 0x1821, 0x1838, 0x1823, 0x1822, 0x1837, 0x1824, 0x182b, 0x1820, 0x20,
0x1822, 0x1822, 0x1828, 0x20, 0x1835, 0x1823, 0x1828, 0x20, 0x180e, 0x180e, 0x1824, 0x20,
0x1834, 0x1820, 0x182d, 0x182d, 0x1837, 0x1822, 0x1828, 0x180b, 0x1838, 0x1822, 0x1834, 0x1822,
0x20, 0x1822, 0x1822, 0x1828, 0x20, 0x1834, 0x1820, 0x182d, 0x905, 0x92b, 0x917, 0x93e,
0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x905, 0x92b, 0x94d,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d,
0x935, 0x940, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x938,
0x92e, 0x92f, 0x926, 0x915, 0x94d, 0x937, 0x93f, 0x923, 0x20, 0x905, 0x92b, 0x94d,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x94d, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x94d,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x94d, 0x930,
0x93f, 0x915, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x938, 0x94d,
0x915, 0x93e, 0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x932, 0x938, 0x94d,
0x915, 0x93e, 0x915, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x905, 0x932, 0x938, 0x94d, 0x915, 0x93e, 0x915, 0x94b, 0x20, 0x926, 0x93f,
0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x92e, 0x93e, 0x91c, 0x94b, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x92e, 0x93e, 0x91c, 0x94b, 0x928, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x92e, 0x93e, 0x91c, 0x94b,
0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d,
0x930, 0x940, 0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926,
0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x926,
0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935,
0x940, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93f,
0x92e, 0x93e, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x93f, 0x92e, 0x93e,
0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939,
0x93f, 0x92e, 0x93e, 0x932, 0x940, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x94d, 0x92f,
0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x906, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x906, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x906, 0x92a, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93f, 0x935, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x92c, 0x940, 0x20, 0x938, 0x92e, 0x92f,
0x905, 0x930, 0x92c, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x905, 0x930, 0x92c, 0x940, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x91f, 0x93f, 0x928, 0x940,
0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x928, 0x91f, 0x93f,
0x928, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905,
0x930, 0x94d, 0x91c, 0x947, 0x928, 0x91f, 0x93f, 0x928, 0x940, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930,
0x94d, 0x91c, 0x947, 0x928, 0x91f, 0x93f, 0x928, 0x940, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c,
0x947, 0x928, 0x91f, 0x93f, 0x928, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905,
0x930, 0x94d, 0x91c, 0x947, 0x928, 0x91f, 0x93f, 0x928, 0x940, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x930, 0x94d,
0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x90f,
0x91f, 0x94d, 0x932, 0x93e, 0x928, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x90f, 0x91f, 0x94d, 0x932, 0x93e, 0x928, 0x94d, 0x91f, 0x93f, 0x915, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x91f, 0x94d, 0x932,
0x93e, 0x928, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f,
0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940,
0x92f, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947,
0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x905, 0x938, 0x94d, 0x91f,
0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93f, 0x935, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940,
0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a,
0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d,
0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f,
0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93f, 0x935,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20,
0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x905, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940,
0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x930,
0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x905, 0x91c, 0x930,
0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x905, 0x91c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x90f, 0x91c, 0x94b, 0x930, 0x947, 0x938, 0x94d, 0x20, 0x938,
0x92e, 0x92f, 0x90f, 0x91c, 0x94b, 0x930, 0x947, 0x938, 0x94d, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x90f, 0x91c, 0x94b, 0x930, 0x947, 0x938,
0x94d, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x902, 0x917, 0x932, 0x93e, 0x926,
0x947, 0x936, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x902, 0x917, 0x932, 0x93e,
0x926, 0x947, 0x936, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92c, 0x902, 0x917, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x940, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92d, 0x941, 0x91f, 0x93e, 0x928, 0x940, 0x20, 0x938, 0x92e,
0x92f, 0x92c, 0x94b, 0x932, 0x93f, 0x92d, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93f, 0x932, 0x940, 0x92f, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93f, 0x932, 0x93f, 0x92f,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92c, 0x94d,
0x930, 0x93e, 0x91c, 0x93f, 0x932, 0x940, 0x92f, 0x93e, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x92c, 0x94d, 0x930, 0x941, 0x928, 0x93e, 0x907, 0x20, 0x926, 0x93e, 0x930,
0x942, 0x938, 0x94d, 0x938, 0x932, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947,
0x92a, 0x20, 0x92d, 0x930, 0x94d, 0x926, 0x947, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x947, 0x92a, 0x20, 0x92d, 0x930, 0x94d, 0x926, 0x947, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x92a, 0x20, 0x92d, 0x930, 0x94d,
0x926, 0x947, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x93e, 0x92e, 0x94b, 0x930,
0x94d, 0x930, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x91a, 0x93e, 0x925, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x93e, 0x925,
0x93e, 0x92e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91a,
0x93e, 0x925, 0x93e, 0x92e, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x91a, 0x940, 0x928, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x91a, 0x94b, 0x907, 0x92c, 0x93e, 0x932, 0x94d, 0x938, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x91a, 0x94b, 0x907, 0x92c, 0x93e, 0x932, 0x94d, 0x938, 0x928, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x94b, 0x907, 0x92c,
0x93e, 0x932, 0x94d, 0x938, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d,
0x930, 0x93f, 0x938, 0x94d, 0x92e, 0x938, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20,
0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x915, 0x94b, 0x938, 0x20, 0x91f, 0x93e, 0x92a,
0x941, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x92e, 0x94d, 0x92c, 0x93f,
0x92f, 0x93e, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x92e,
0x94d, 0x92c, 0x93f, 0x92f, 0x93e, 0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x932, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f,
0x93e, 0x932, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x941, 0x915, 0x20,
0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x941, 0x915, 0x20,
0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x915, 0x941, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x906, 0x927,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e,
0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e,
0x915, 0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x915, 0x94b, 0x20, 0x926, 0x93f, 0x935, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x921, 0x947, 0x92d, 0x93f, 0x938, 0x20, 0x938, 0x92e,
0x92f, 0x921, 0x941, 0x92e, 0x94b, 0x928, 0x94d, 0x91f, 0x2d, 0x921, 0x940, 0x2018,
0x20, 0x909, 0x930, 0x94d, 0x92d, 0x93f, 0x932, 0x947, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x91f, 0x93f, 0x92e, 0x94b, 0x930,
0x20, 0x938, 0x92e, 0x92f, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x91f, 0x93e,
0x92a, 0x942, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20,
0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x907, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x908,
0x915, 0x94d, 0x935, 0x94b, 0x921, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92f, 0x941, 0x930,
0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928,
0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a,
0x947, 0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x915, 0x947, 0x928, 0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92f, 0x941,
0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947,
0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940,
0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940,
0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x925, 0x92a, 0x2d, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20,
0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b,
0x92a, 0x947, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x941, 0x930, 0x94b, 0x92a, 0x947, 0x932, 0x940,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x932, 0x94d,
0x915, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x92a,
0x942, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x932, 0x94d, 0x915, 0x932, 0x94d, 0x92f,
0x93e, 0x928, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x932, 0x94d, 0x915, 0x932, 0x94d,
0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92b, 0x93f, 0x91c, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92b,
0x93f, 0x91c, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x92b, 0x93f, 0x91c, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x94d, 0x930,
0x947, 0x928, 0x94d, 0x91a, 0x20, 0x917, 0x94d, 0x935, 0x93e, 0x928, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x92b, 0x94d, 0x930, 0x947, 0x928, 0x94d, 0x91a, 0x20, 0x926,
0x915, 0x94d, 0x937, 0x93f, 0x923, 0x940, 0x20, 0x930, 0x20, 0x905, 0x928, 0x94d,
0x91f, 0x93e, 0x930, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x917,
0x93e, 0x932, 0x93e, 0x92a, 0x93e, 0x917, 0x94b, 0x938, 0x94d, 0x20, 0x938, 0x92e,
0x92f, 0x917, 0x94d, 0x92f, 0x93e, 0x92e, 0x94d, 0x92c, 0x93f, 0x92f, 0x930, 0x20,
0x938, 0x92e, 0x92f, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f,
0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x917, 0x93f, 0x932, 0x94d, 0x92c, 0x930,
0x94d, 0x91f, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x93f, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b,
0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915,
0x94b, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942,
0x930, 0x94d, 0x935, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x94d,
0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x94d,
0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x916, 0x93e, 0x921, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x941, 0x91f, 0x93f, 0x92f,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932,
0x941, 0x91f, 0x93f, 0x92f, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x939, 0x935, 0x93e, 0x908, 0x2d, 0x90f, 0x932, 0x941, 0x91f, 0x93f,
0x92f, 0x928, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x939,
0x919, 0x915, 0x919, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x919, 0x915, 0x919, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x919, 0x915, 0x919,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940,
0x928, 0x20, 0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x92c, 0x94d, 0x921, 0x20, 0x938,
0x92e, 0x92f, 0x939, 0x94b, 0x92c, 0x94d, 0x921, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x939, 0x94b, 0x92c, 0x94d, 0x921, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x939, 0x93f, 0x928, 0x94d, 0x926, 0x20, 0x92e, 0x939, 0x93e, 0x938,
0x93e, 0x917, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x928, 0x94d, 0x921, 0x94b,
0x91a, 0x93e, 0x907, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x947, 0x928,
0x94d, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x94b,
0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x942,
0x930, 0x94d, 0x935, 0x940, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x94b, 0x928, 0x947,
0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x907, 0x928, 0x94d, 0x921, 0x94b, 0x928, 0x947, 0x936,
0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x930, 0x93e, 0x928, 0x940,
0x20, 0x938, 0x92e, 0x92f, 0x907, 0x930, 0x93e, 0x928, 0x940, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x930, 0x93e, 0x928, 0x940, 0x20,
0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x930, 0x94d, 0x915,
0x941, 0x91f, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x930,
0x94d, 0x915, 0x941, 0x91f, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x908, 0x930, 0x94d, 0x915, 0x941, 0x91f, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915,
0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x91c, 0x930, 0x93e,
0x92f, 0x932, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x91c, 0x930, 0x93e, 0x92f, 0x932,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x907, 0x91c, 0x930,
0x93e, 0x92f, 0x932, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f,
0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x92a, 0x942, 0x930, 0x94d, 0x935, 0x940, 0x20, 0x915, 0x93e, 0x91c,
0x915, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x20, 0x915, 0x93e, 0x91c, 0x915, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e,
0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e,
0x932, 0x940, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915,
0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x932, 0x940, 0x20, 0x926, 0x93f, 0x935, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x928, 0x94b, 0x92f,
0x93e, 0x930, 0x94d, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93e,
0x938, 0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x928, 0x94b,
0x92f, 0x93e, 0x930, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x915, 0x93f,
0x930, 0x94d, 0x917, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x932, 0x93e, 0x907, 0x928, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938,
0x92e, 0x92f, 0x932, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935, 0x947, 0x20,
0x938, 0x92e, 0x92f, 0x932, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x93e, 0x935, 0x947,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x932, 0x930, 0x94d,
0x921, 0x20, 0x939, 0x93e, 0x935, 0x947, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x92e, 0x932, 0x947, 0x938, 0x93f, 0x92f, 0x93e, 0x20, 0x938,
0x92e, 0x92f, 0x92e, 0x93e, 0x932, 0x94d, 0x926, 0x93f, 0x92d, 0x94d, 0x938, 0x20,
0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x93f, 0x938, 0x938, 0x20,
0x938, 0x92e, 0x92f, 0x92e, 0x93e, 0x930, 0x94d, 0x936, 0x932, 0x20, 0x91f, 0x93e,
0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x909, 0x930, 0x93f, 0x91f, 0x938,
0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x909, 0x930, 0x93f, 0x91f, 0x938, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x909, 0x930, 0x93f, 0x91f,
0x938, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x94d, 0x935, 0x938, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20,
0x92a, 0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d,
0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928,
0x20, 0x92a, 0x94d, 0x92f, 0x93e, 0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x926,
0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x932, 0x93e, 0x928, 0x20,
0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x932, 0x93e,
0x928, 0x20, 0x92c, 0x93e, 0x91f, 0x94b, 0x930, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x93e, 0x91f,
0x94b, 0x930, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x938, 0x94d, 0x915, 0x94b,
0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x938, 0x94d, 0x915, 0x94b, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x92e, 0x94d, 0x92f, 0x93e, 0x928, 0x92e, 0x93e, 0x930,
0x20, 0x938, 0x92e, 0x92f, 0x928, 0x93e, 0x909, 0x930, 0x941, 0x20, 0x938, 0x92e,
0x92f, 0x928, 0x947, 0x92a, 0x93e, 0x932, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x928,
0x92f, 0x93e, 0x901, 0x20, 0x915, 0x93e, 0x932, 0x947, 0x926, 0x94b, 0x928, 0x93f,
0x92f, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x92f, 0x93e, 0x901, 0x20, 0x915,
0x93e, 0x932, 0x947, 0x926, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x92f, 0x93e, 0x901, 0x20, 0x915,
0x93e, 0x932, 0x947, 0x926, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93f, 0x932, 0x94d, 0x92f, 0x93e,
0x928, 0x94d, 0x921, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c,
0x93f, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93f, 0x932,
0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x20, 0x926, 0x93f, 0x935, 0x93e, 0x20,
0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93e, 0x909, 0x928, 0x94d,
0x921, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20, 0x938,
0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93e, 0x909, 0x928, 0x921, 0x932,
0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93e, 0x909,
0x928, 0x932, 0x94d, 0x92f, 0x93e, 0x928, 0x94d, 0x921, 0x915, 0x94b, 0x20, 0x926,
0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x93f, 0x909, 0x90f, 0x20,
0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x930, 0x94d, 0x92b, 0x94b, 0x915, 0x20, 0x91f,
0x93e, 0x92a, 0x942, 0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x930,
0x94d, 0x92b, 0x94b, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x915, 0x94b, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x930, 0x94d,
0x92b, 0x94b, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x915, 0x94b, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x928, 0x94d, 0x921, 0x94b,
0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x930, 0x94d, 0x928, 0x93e, 0x928, 0x94d, 0x921,
0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x930,
0x94d, 0x928, 0x93e, 0x928, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928,
0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x928, 0x94b, 0x92d, 0x94b, 0x938, 0x93f, 0x935, 0x93f, 0x930, 0x94d, 0x938, 0x94d,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x92d, 0x94b, 0x938, 0x93f, 0x935,
0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x928, 0x94b, 0x92d, 0x94b, 0x938, 0x93f, 0x935, 0x93f, 0x930,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x915,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x940, 0x20, 0x938, 0x92e, 0x92f, 0x92a,
0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x940, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x932,
0x93e, 0x909, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x92a, 0x942, 0x935, 0x93e, 0x20,
0x928, 0x94d, 0x92f, 0x942, 0x20, 0x917, 0x93f, 0x928, 0x940, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x93e, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x938, 0x92e,
0x92f, 0x92a, 0x93e, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93e, 0x930, 0x93e, 0x917, 0x94d,
0x935, 0x947, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x930, 0x941, 0x20,
0x938, 0x92e, 0x92f, 0x92a, 0x947, 0x930, 0x941, 0x20, 0x917, 0x94d, 0x930, 0x940,
0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f,
0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93f, 0x928, 0x940, 0x20, 0x938, 0x92e, 0x92f,
0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93f, 0x928, 0x940, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93f, 0x932, 0x93f, 0x92a, 0x93f, 0x928,
0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932,
0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92b, 0x93f, 0x928, 0x93f, 0x915, 0x94d,
0x938, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947,
0x928, 0x94d, 0x91f, 0x20, 0x92a, 0x93f, 0x92f, 0x930, 0x94d, 0x930, 0x947, 0x20,
0x930, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x92f, 0x941, 0x932, 0x94b, 0x928, 0x915,
0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x92a,
0x93f, 0x92f, 0x930, 0x94d, 0x930, 0x947, 0x20, 0x930, 0x20, 0x92e, 0x93f, 0x915,
0x94d, 0x92f, 0x941, 0x932, 0x94b, 0x928, 0x915, 0x94b, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947, 0x928, 0x94d, 0x91f, 0x20, 0x92a,
0x93f, 0x92f, 0x930, 0x94d, 0x930, 0x947, 0x20, 0x930, 0x20, 0x92e, 0x93f, 0x915,
0x94d, 0x92f, 0x941, 0x932, 0x94b, 0x928, 0x915, 0x94b, 0x20, 0x926, 0x93f, 0x935,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x93f, 0x91f, 0x915, 0x948, 0x930, 0x923,
0x20, 0x938, 0x92e, 0x92f, 0x92a, 0x94d, 0x92f, 0x94b, 0x919, 0x92f, 0x93e, 0x919,
0x20, 0x938, 0x92e, 0x92f, 0x930, 0x93f, 0x92f, 0x941, 0x928, 0x93f, 0x92f, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20,
0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x916, 0x93e, 0x932,
0x93f, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e,
0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x92e, 0x94b, 0x905,
0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x92e, 0x94b, 0x905, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93e, 0x92e, 0x94b, 0x905, 0x20,
0x926, 0x93f, 0x935, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x947, 0x92f, 0x947,
0x91a, 0x947, 0x932, 0x93e, 0x938, 0x94d, 0x20, 0x938, 0x92e, 0x92f, 0x938, 0x93f,
0x902, 0x917, 0x93e, 0x92a, 0x941, 0x930, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x938, 0x92e, 0x92f, 0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x94b, 0x928, 0x20, 0x91f,
0x93e, 0x92a, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x926, 0x915, 0x94d, 0x937, 0x93f,
0x923, 0x20, 0x91c, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x938, 0x941, 0x930, 0x93f, 0x928, 0x93e, 0x92e, 0x93e, 0x20, 0x938, 0x92e,
0x92f, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x907, 0x20, 0x938, 0x92e, 0x92f, 0x924,
0x93e, 0x907, 0x92a, 0x947, 0x907, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x907, 0x20, 0x926, 0x93f, 0x935,
0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x93e, 0x91c, 0x93f, 0x915, 0x938, 0x94d,
0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x924, 0x94b, 0x915, 0x947, 0x932,
0x93e, 0x909, 0x20, 0x938, 0x92e, 0x92f, 0x91a, 0x941, 0x909, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x924, 0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e,
0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x91f, 0x941, 0x92d, 0x93e, 0x932, 0x941, 0x20, 0x938, 0x92e, 0x92f,
0x909, 0x930, 0x942, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x938, 0x92e, 0x92f, 0x909, 0x91c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x91c, 0x94d, 0x92c,
0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x909, 0x91c, 0x94d, 0x92c, 0x947, 0x915, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d,
0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x93e,
0x928, 0x941, 0x906, 0x924, 0x941, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x93e, 0x928,
0x941, 0x906, 0x924, 0x941, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e,
0x92f, 0x92d, 0x93e, 0x928, 0x941, 0x906, 0x924, 0x941, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x92d, 0x947, 0x928, 0x947, 0x91c, 0x94d, 0x92f, 0x941, 0x90f, 0x932, 0x93e,
0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x92d, 0x93e,
0x938, 0x94d, 0x91f, 0x94b, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x94d, 0x932,
0x93e, 0x926, 0x93f, 0x92d, 0x93e, 0x938, 0x94d, 0x91f, 0x94b, 0x915, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x94d, 0x932, 0x93e, 0x926,
0x93f, 0x92d, 0x93e, 0x938, 0x94d, 0x91f, 0x94b, 0x915, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e,
0x92f, 0x92d, 0x94b, 0x932, 0x94d, 0x917, 0x93e, 0x917, 0x94d, 0x930, 0x93e, 0x921,
0x20, 0x938, 0x92e, 0x92f, 0x92d, 0x94b, 0x932, 0x94d, 0x917, 0x93e, 0x917, 0x94d,
0x930, 0x93e, 0x926, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938, 0x92e, 0x92f,
0x92d, 0x94b, 0x932, 0x94d, 0x917, 0x93e, 0x917, 0x94d, 0x930, 0x93e, 0x926, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928,
0x20, 0x938, 0x92e, 0x92f, 0x935, 0x947, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x941,
0x20, 0x938, 0x92e, 0x92f, 0x935, 0x93e, 0x932, 0x93f, 0x938, 0x94d, 0x20, 0x930,
0x20, 0x92b, 0x941, 0x91f, 0x941, 0x928, 0x93e, 0x20, 0x938, 0x92e, 0x92f, 0x92f,
0x93e, 0x915, 0x941, 0x938, 0x94d, 0x91f, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x93e,
0x915, 0x941, 0x938, 0x94d, 0x91f, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x92f, 0x93e, 0x915, 0x941, 0x938, 0x94d, 0x91f, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x937, 0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938,
0x92e, 0x92f, 0x92f, 0x947, 0x915, 0x93e, 0x91f, 0x947, 0x930, 0x93f, 0x928, 0x92c,
0x930, 0x94d, 0x917, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x947, 0x915, 0x93e, 0x91f,
0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x938, 0x92e, 0x92f, 0x92f, 0x947, 0x915, 0x93e, 0x91f, 0x947, 0x930,
0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x915, 0x93e, 0x932, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x92f,
0x941, 0x915, 0x94b, 0x928, 0x915, 0x94b, 0x20, 0x938, 0x92e, 0x92f, 0x41, 0x66,
0x67, 0xe1, 0x6e, 0xed, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0xc1, 0x66, 0x72, 0xed, 0x6b,
0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x20, 0xc1, 0x66,
0x72, 0xed, 0x6b, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0x61, 0xfa,
0x74, 0x20, 0xc1, 0x66, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74,
0x20, 0xc1, 0x66, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x57, 0x1eb9, 0x301, 0x73, 0x74, 0x20, 0xc1, 0x66, 0x72, 0xed, 0x6b, 0xe1,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57,
0x1eb9, 0x301, 0x73, 0x74, 0x20, 0xc1, 0x66, 0x72, 0xed, 0x6b, 0xe1, 0x20,
0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x41, 0x6c, 0xe1, 0x73, 0x6b, 0xe1, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x41, 0x6c, 0xe1, 0x73, 0x6b, 0xe1, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x6c, 0xe1, 0x73, 0x6b,
0xe1, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0xc1, 0x6d, 0xe1, 0x7a, 0x1ecd, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0xc1, 0x6d, 0xe1, 0x7a, 0x1ecd, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0xc1, 0x6d, 0xe1, 0x7a, 0x1ecd, 0x6e, 0x20,
0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301,
0x72, 0xed, 0x6b, 0xe1, 0x20, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0xc9,
0x72, 0x69, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74,
0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x4d, 0xed,
0x64, 0xfa, 0x6c, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20,
0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x4d, 0xed, 0x64,
0xfa, 0x6c, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x44, 0xe9, 0x6c, 0x61,
0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20,
0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0xcd, 0x73, 0x74,
0xe1, 0x6e, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b,
0xe1, 0x20, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0xc9, 0x72, 0x69, 0xe1,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e,
0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1,
0x20, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e,
0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1,
0x20, 0x4d, 0x61, 0xfa, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0xc9, 0x72, 0x69,
0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41,
0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x4d, 0x61, 0xfa, 0x6e,
0x74, 0x69, 0x6e, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20,
0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x4d, 0x61, 0xfa,
0x6e, 0x74, 0x69, 0x6e, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301,
0x74, 0x20, 0x41, 0x6d, 0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x50,
0x61, 0x73, 0xed, 0x66, 0xed, 0x6b, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d, 0x1eb9,
0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66, 0xed,
0x6b, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x74, 0x20, 0x41, 0x6d,
0x1eb9, 0x301, 0x72, 0xed, 0x6b, 0xe1, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66,
0xed, 0x6b, 0x20, 0xc9, 0x72, 0x69, 0xe1, 0x20, 0x44, 0xe9, 0x6c, 0x61,
0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xc1, 0x70, 0x69, 0x61, 0x20,
0x54, 0x61, 0x69, 0x6d, 0xc1, 0x70, 0x69, 0x61, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xc1, 0x70, 0x69, 0x61, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41,
0x72, 0xe9, 0x62, 0x69, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x72,
0xe9, 0x62, 0x69, 0xe1, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x41, 0x72, 0xe9, 0x62, 0x69, 0xe1, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x6a, 0x1eb9,
0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x6a,
0x1eb9, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xed, 0x6e,
0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e,
0x20, 0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x41, 0x6a,
0x1eb9, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e,
0x20, 0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xed, 0x6e, 0x61, 0x20, 0x48, 0x1ecd,
0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x41, 0x72, 0x6d, 0x1eb9, 0x301, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x41, 0x72, 0x6d, 0x1eb9, 0x301, 0x6e, 0x69, 0x61, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x72, 0x6d,
0x1eb9, 0x301, 0x6e, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x74, 0x6c,
0xe1, 0x6e, 0x74, 0xed, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x74,
0x6c, 0xe1, 0x6e, 0x74, 0xed, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0xed,
0x6b, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0x1ecc, 0x73, 0x74, 0x72, 0xe9,
0x6c, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72,
0xe9, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74,
0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc,
0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0xed, 0x64, 0xfa,
0x6c, 0x20, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x4d,
0xed, 0x64, 0xfa, 0x6c, 0x20, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc,
0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20, 0x4d, 0xed, 0x64, 0xfa,
0x6c, 0x20, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74,
0xe1, 0x6e, 0x20, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61,
0x20, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69,
0x61, 0x20, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x44, 0xe9, 0x6c, 0x61,
0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74,
0xe1, 0x6e, 0x20, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0x61,
0x20, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc, 0x73, 0x74, 0x72, 0xe9,
0x6c, 0x69, 0x61, 0x20, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20,
0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41,
0x7a, 0x1eb9, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x41, 0x7a, 0x1eb9, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41,
0x7a, 0x1eb9, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x20, 0x48, 0x1ecd,
0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x41, 0x7a, 0x1ecd, 0x7a, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x41, 0x7a,
0x1ecd, 0x7a, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x41, 0x7a, 0x1ecd, 0x7a, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x1eb9, 0x73, 0x68, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x1eb9, 0x73, 0x68, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x1eb9, 0x73, 0x68, 0x20, 0x44, 0xe9, 0x6c, 0x61,
0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42, 0x75, 0x74, 0x61, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x42, 0x6f, 0x6c, 0xed, 0x76, 0x69, 0x61,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42,
0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42, 0x72, 0x61, 0x73, 0xed,
0x6c, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a,
0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x42, 0x72, 0x75, 0x6e, 0x1eb9,
0x69, 0x20, 0x44, 0x61, 0x72, 0xfa, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4b, 0x65, 0x70, 0x20, 0x56, 0x1eb9, 0x301, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x65, 0x70, 0x20, 0x56, 0x1eb9, 0x301,
0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4b, 0x65, 0x70, 0x20, 0x56, 0x1eb9, 0x301, 0x64, 0x20, 0x48, 0x1ecd, 0x301,
0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x43, 0x68, 0x61, 0x6d, 0xf3, 0x72, 0x6f, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0xe1, 0x74, 0x61, 0x6d,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0xe1, 0x74, 0x61, 0x6d, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68,
0xe1, 0x74, 0x61, 0x6d, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0xed, 0x6c, 0x1eb9, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x43, 0x68, 0xed, 0x6c, 0x1eb9, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0xed, 0x6c, 0x1eb9, 0x20,
0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x43, 0x68, 0x61, 0xed, 0x6e, 0x61, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x43, 0x68, 0x61, 0xed, 0x6e, 0x61, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0x61, 0xed, 0x6e,
0x61, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x43, 0x68, 0x6f, 0x69, 0x62, 0xe1, 0x6c, 0x73, 0x61, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x43, 0x68, 0x6f, 0x69, 0x62, 0xe1, 0x6c, 0x73,
0x61, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x43, 0x68, 0x6f, 0x69, 0x62, 0xe1, 0x6c, 0x73, 0x61, 0x6e, 0x20,
0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4b, 0x72, 0xed, 0x73, 0x6d, 0xe1, 0x73, 0x20, 0x41,
0xed, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0xf3,
0x6b, 0xf3, 0x73, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f, 0x6c, 0xf3, 0x6d, 0x62, 0x69, 0x61,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f, 0x6c, 0xf3, 0x6d, 0x62, 0x69,
0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4b, 0x6f, 0x6c, 0xf3, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301,
0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4b, 0xfa, 0x6b, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4b, 0xfa, 0x6b, 0x20, 0x41, 0xed, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4b, 0xfa, 0x6b, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x20, 0x48, 0x61, 0x66, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0xfa, 0x62,
0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0xfa, 0x62, 0x61, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0xfa, 0x62,
0x61, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x44, 0xe9, 0x76, 0x69, 0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x44,
0x69, 0xfa, 0x6d, 0x1ecd, 0x301, 0x6e, 0x2d, 0x64, 0x2019, 0x55, 0x76, 0x69,
0x6c, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x20, 0x54, 0xed,
0x6d, 0x1ecd, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x61, 0x20,
0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x61, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x61, 0x20,
0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x1eb8, 0x301, 0x6b, 0x77, 0x75, 0xe1, 0x64, 0x1ecd, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20, 0x59, 0xfa,
0x72, 0x6f, 0x70, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64, 0xfa,
0x6c, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64, 0xfa, 0x6c, 0x20,
0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74,
0xe1, 0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x54, 0x61, 0x69,
0x6d, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd,
0x73, 0x74, 0xe1, 0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x48,
0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x46, 0x61, 0xe1, 0x2d, 0xcd, 0x73, 0x74, 0xe1, 0x6e, 0x20,
0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9,
0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20,
0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e, 0x20,
0x59, 0xfa, 0x72, 0x6f, 0x70, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0x1ecd, 0x301,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46,
0x1ecd, 0x301, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0x1ecd, 0x301, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed,
0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6a, 0x69,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6a, 0x69, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6a, 0x69,
0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x46, 0x72, 0x1eb9, 0x301, 0x6e, 0x63, 0x68, 0x20,
0x47, 0x69, 0xe1, 0x6e, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0x72,
0x1eb9, 0x301, 0x6e, 0x63, 0x68, 0x20, 0x53, 0x61, 0xfa, 0x74, 0x61, 0x6e,
0x20, 0x61, 0x6e, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x74, 0xed, 0x6b, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0xe1, 0x67, 0x1ecd,
0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x47, 0xe1, 0x6d, 0x62, 0x69, 0x1eb9,
0x72, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a, 0x1ecd, 0x301, 0x6a, 0x69, 0x61,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a, 0x1ecd, 0x301, 0x6a, 0x69, 0x61, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a, 0x1ecd,
0x301, 0x6a, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed,
0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x47, 0xed, 0x6c, 0x62,
0x61, 0x74, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x47, 0x72, 0xed, 0x6e, 0x77, 0xed, 0x63, 0x68, 0x20,
0x4d, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x69, 0x73, 0x74,
0x20, 0x47, 0x72, 0xed, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0xcd, 0x69, 0x73, 0x74, 0x20, 0x47, 0x72, 0xed, 0x6e, 0x6c,
0xe1, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0xcd, 0x69, 0x73, 0x74, 0x20, 0x47, 0x72, 0xed, 0x6e, 0x6c,
0xe1, 0x6e, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a,
0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74,
0x20, 0x47, 0x72, 0xed, 0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0x20, 0x47, 0x72, 0xed, 0x6e,
0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0x20, 0x47, 0x72, 0xed,
0x6e, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x47, 0x1ecd, 0x301,
0x6c, 0x66, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x47, 0x61, 0x79, 0xe1, 0x6e, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x48, 0x61, 0x77, 0x61, 0x69, 0xed, 0x2d, 0x45, 0x6c, 0xfa, 0x73, 0x68,
0x69, 0xe1, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x48, 0x61, 0x77, 0x61,
0x69, 0xed, 0x2d, 0x45, 0x6c, 0xfa, 0x73, 0x68, 0x69, 0xe1, 0x6e, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x48, 0x61,
0x77, 0x61, 0x69, 0xed, 0x2d, 0x45, 0x6c, 0xfa, 0x73, 0x68, 0x69, 0xe1,
0x6e, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x48, 0x1ecd, 0x6e, 0x67, 0x20, 0x4b, 0x1ecd, 0x6e, 0x67, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x48, 0x1ecd, 0x6e, 0x67, 0x20, 0x4b, 0x1ecd, 0x6e, 0x67,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x48,
0x1ecd, 0x6e, 0x67, 0x20, 0x4b, 0x1ecd, 0x6e, 0x67, 0x20, 0x48, 0x1ecd, 0x301,
0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x48, 0x6f, 0x76, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x48, 0x6f, 0x76,
0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x48, 0x6f, 0x76, 0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed,
0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0xcd, 0x6e, 0x64, 0x69, 0xe1, 0x6e, 0x20, 0xd3, 0x73, 0x68, 0x1eb9, 0x301,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68,
0x61, 0xed, 0x6e, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xed, 0x64,
0xfa, 0x6c, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x1eb9, 0x301, 0x73, 0x68,
0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0xe1, 0x6e,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x1eb9, 0x301, 0x73, 0x68, 0x69, 0x61,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9, 0x301, 0x73, 0x74, 0xe1, 0x6e,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x1eb9, 0x301, 0x73, 0x68, 0x69, 0x61,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x72, 0x6b,
0xfa, 0x74, 0x73, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x49, 0x72, 0x6b,
0xfa, 0x74, 0x73, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x49, 0x72, 0x6b, 0xfa, 0x74, 0x73, 0x6b, 0x20, 0x48,
0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0xcd, 0x7a, 0x72, 0x1eb9, 0x6c, 0x20, 0x54, 0x61, 0x69, 0x6d,
0xcd, 0x7a, 0x72, 0x1eb9, 0x6c, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0xcd, 0x7a, 0x72, 0x1eb9, 0x6c, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20,
0x54, 0x61, 0x69, 0x6d, 0xcd, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0xe9,
0x6b, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0x1eb9,
0x301, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x7a, 0xe9, 0x6b, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f, 0x72, 0x69, 0x61, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f, 0x72, 0x69, 0x61, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4b, 0x6f, 0x72, 0x69,
0x61, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4b, 0x1ecd, 0x301, 0x73, 0x72, 0x61, 0x1eb9, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x73, 0x6b, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x73, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x73, 0x6b, 0x20,
0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4b, 0x1eb9, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4c, 0x61, 0xed, 0x6e, 0x20, 0x41, 0xed, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4c, 0x1ecd, 0x64,
0x20, 0x48, 0x61, 0xfa, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4c, 0x1ecd, 0x64,
0x20, 0x48, 0x61, 0xfa, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4c, 0x1ecd, 0x64, 0x20, 0x48, 0x61, 0xfa, 0x20, 0x44,
0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xe1,
0x67, 0xe1, 0x64, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xe1,
0x67, 0xe1, 0x64, 0x61, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4d, 0xe1, 0x67, 0xe1, 0x64, 0x61, 0x6e, 0x20,
0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4d, 0x61, 0x6c, 0xe9, 0x73, 0x68, 0x69, 0x61, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x301, 0x64, 0x69, 0x76, 0x73, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4d, 0x61, 0x6b, 0x77, 0x1eb9, 0x301, 0x73, 0x61,
0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0xe1, 0x73, 0x68, 0x61, 0x6c,
0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4d, 0x1ecd, 0x72, 0xed, 0x73, 0x68, 0x1ecd, 0x73, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4d, 0x1ecd, 0x72, 0xed, 0x73, 0x68, 0x1ecd, 0x73, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x72,
0xed, 0x73, 0x68, 0x1ecd, 0x73, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x301,
0x73, 0x1ecd, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1eb9, 0x301, 0x6b,
0x73, 0xed, 0x6b, 0xf3, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66, 0xed, 0x6b,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1eb9, 0x301, 0x6b, 0x73, 0xed, 0x6b,
0xf3, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66, 0xed, 0x6b, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1eb9, 0x301, 0x6b,
0x73, 0xed, 0x6b, 0xf3, 0x20, 0x50, 0x61, 0x73, 0xed, 0x66, 0xed, 0x6b,
0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4d, 0x1ecd, 0x6e, 0x67, 0xf3, 0x6c, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x4d, 0x1ecd, 0x6e, 0x67, 0xf3, 0x6c, 0x69, 0x61, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x6e, 0x67,
0xf3, 0x6c, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed,
0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x301, 0x73,
0x6b, 0x6f, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x1ecd, 0x301, 0x73, 0x6b,
0x6f, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4d, 0x1ecd, 0x301, 0x73, 0x6b, 0x6f, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20,
0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4d, 0x69,
0xe1, 0x6e, 0x6d, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x61, 0xfa,
0x72, 0x75, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x1eb9, 0x70, 0x1ecd, 0x6c,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x20, 0x4b, 0x61, 0x6c,
0x1eb9, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e,
0x69, 0xfa, 0x20, 0x4b, 0x61, 0x6c, 0x1eb9, 0x64, 0xf3, 0x6e, 0x69, 0x61,
0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e,
0x69, 0xfa, 0x20, 0x4b, 0x61, 0x6c, 0x1eb9, 0x64, 0xf3, 0x6e, 0x69, 0x61,
0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x20, 0x5a, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x20, 0x5a,
0x69, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x20, 0x5a, 0x69, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x66, 0x61, 0xfa, 0x6e, 0x64, 0x6c, 0xe1,
0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x66, 0x61,
0xfa, 0x6e, 0x64, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x66, 0x61, 0xfa,
0x6e, 0x64, 0x6c, 0xe1, 0x6e, 0x64, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed,
0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x4e, 0x69, 0xfa, 0x1eb9, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x72, 0x66, 0x1ecd, 0x301, 0x6c, 0x6b,
0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4e, 0x1ecd, 0x301, 0x72, 0x66, 0x1ecd, 0x301, 0x6c, 0x6b, 0x20, 0x41, 0xed,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4e, 0x1ecd, 0x301, 0x72, 0x66, 0x1ecd, 0x301, 0x6c, 0x6b,
0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x44, 0xe9, 0x6c, 0x61,
0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0x1eb9, 0x72, 0x6e, 0xe1,
0x6e, 0x64, 0xf3, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x1ecd, 0x72, 0xf3, 0x6e,
0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0x1eb9, 0x72, 0x6e, 0xe1,
0x6e, 0x64, 0xf3, 0x20, 0x64, 0x69, 0x20, 0x4e, 0x1ecd, 0x72, 0xf3, 0x6e,
0x69, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x46, 0x1eb9, 0x72, 0x6e, 0xe1, 0x6e, 0x64, 0xf3, 0x20, 0x64, 0x69,
0x20, 0x4e, 0x1ecd, 0x72, 0xf3, 0x6e, 0x69, 0x61, 0x20, 0x48, 0x1ecd, 0x301,
0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73,
0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x48,
0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x1ecc, 0x6d, 0x73, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x1ecc,
0x6d, 0x73, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x1ecc, 0x6d, 0x73, 0x6b, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20,
0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0xe1,
0x6b, 0xed, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50,
0xe1, 0x6b, 0xed, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73,
0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0xe1, 0x6b, 0xed, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0x61, 0x6c, 0xe1, 0x75, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x50, 0xe1, 0x70, 0x75, 0xe1, 0x20, 0x4e, 0x69,
0xfa, 0x20, 0x47, 0xed, 0x6e, 0x69, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50,
0xe1, 0x72, 0xe1, 0x67, 0x77, 0x75, 0x65, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x50, 0xe1, 0x72, 0xe1, 0x67, 0x77, 0x75, 0x65, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0xe1, 0x72, 0xe1, 0x67,
0x77, 0x75, 0x65, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a,
0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0x1eb9, 0x72, 0x75, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x50, 0x1eb9, 0x72, 0x75, 0x20, 0x46, 0xed, 0x6b,
0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0x1eb9, 0x72, 0x75, 0x20,
0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0xed, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x46, 0xed, 0x6c, 0xed, 0x70, 0x69, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x46, 0xed, 0x6c, 0xed, 0x70, 0x69, 0x6e, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6c,
0xed, 0x70, 0x69, 0x6e, 0x20, 0x48, 0x1ecd, 0x74, 0x20, 0x53, 0xed, 0x7a,
0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x46, 0xed, 0x6e, 0x69, 0x6b,
0x73, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x1eb9, 0x72, 0x20,
0x26, 0x20, 0x4d, 0xed, 0x6b, 0x1eb9, 0x6c, 0x1ecd, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x53, 0x65, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x1eb9, 0x72, 0x20,
0x61, 0x6e, 0x20, 0x4d, 0xed, 0x6b, 0x1eb9, 0x6c, 0x1ecd, 0x6e, 0x20, 0x46,
0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0x65, 0x6e,
0x74, 0x20, 0x50, 0x69, 0x1eb9, 0x72, 0x20, 0x61, 0x6e, 0x20, 0x4d, 0xed,
0x6b, 0x1eb9, 0x6c, 0x1ecd, 0x6e, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed, 0x74,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x50, 0xed, 0x74, 0x6b, 0x61, 0x6e, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x50, 0xf3, 0x6e, 0xe1, 0x70, 0x1eb9, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x50, 0x69, 0x1ecd, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x52, 0x69, 0x79, 0xfa, 0x6e, 0x69, 0x1ecd,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x52, 0x6f, 0x74, 0x1eb9, 0x301, 0x72,
0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0xe1, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0xe1, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x53, 0xe1, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x48,
0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x53, 0xe1, 0x6d, 0x6f, 0xe1, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x53, 0xe1, 0x6d, 0x6f, 0xe1, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x53, 0xe1, 0x6d, 0x6f, 0xe1, 0x20, 0x44, 0xe9,
0x6c, 0x61, 0xed, 0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0x1eb9, 0x301,
0x63, 0x68, 0x1eb9, 0x6c, 0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x1ecd, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0xf3,
0x6c, 0xf3, 0x6d, 0x1ecd, 0x301, 0x6e, 0x20, 0x41, 0xed, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53, 0x61, 0xfa, 0x74, 0x20,
0x4a, 0x1ecd, 0x301, 0x6a, 0x69, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53,
0xfa, 0x72, 0xed, 0x6e, 0x61, 0x6d, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x53,
0x69, 0xf3, 0x77, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x61, 0x68,
0xed, 0x74, 0x69, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x61, 0x69, 0x70,
0x1eb9, 0x69, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x61, 0x69, 0x70, 0x1eb9,
0x69, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x54, 0x61, 0x69, 0x70, 0x1eb9, 0x69, 0x20, 0x44, 0xe9, 0x6c, 0x61, 0xed,
0x74, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x61, 0x6a, 0xed, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x6f, 0x6b,
0x1eb9, 0x6c, 0xe1, 0x75, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x1ecd, 0x301,
0x6e, 0x67, 0x61, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x1ecd, 0x301, 0x6e,
0x67, 0x61, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x54, 0x1ecd, 0x301, 0x6e, 0x67, 0x61, 0x20, 0x48, 0x1ecd, 0x301, 0x74,
0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x43,
0x68, 0x75, 0x6b, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x1ecd, 0x6b, 0x6d,
0x1eb9, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x54, 0x1ecd, 0x6b, 0x6d, 0x1eb9, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x54, 0x1ecd,
0x6b, 0x6d, 0x1eb9, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x48, 0x1ecd,
0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x54, 0x75, 0x76, 0xe1, 0x6c, 0x75, 0x20, 0x54, 0x61, 0x69, 0x6d,
0x59, 0xfa, 0x72, 0x75, 0x67, 0x77, 0x75, 0x65, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x59, 0xfa, 0x72, 0x75, 0x67, 0x77, 0x75, 0x65, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0xfa, 0x72, 0x75,
0x67, 0x77, 0x75, 0x65, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed,
0x7a, 0xed, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x55, 0x7a, 0x62, 0x1eb9,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x55,
0x7a, 0x62, 0x1eb9, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x46, 0xed,
0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x55, 0x7a, 0x62, 0x1eb9,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20,
0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x61,
0x6e, 0x75, 0xe1, 0x74, 0x75, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x61,
0x6e, 0x75, 0xe1, 0x74, 0x75, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x56, 0x61, 0x6e, 0x75, 0xe1, 0x74, 0x75, 0x20,
0x53, 0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x1eb9,
0x6e, 0x1eb9, 0x7a, 0x75, 0x1eb9, 0x301, 0x6c, 0x61, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x1ecd, 0x73, 0x74, 0x1ecd, 0x6b,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x1ecd,
0x73, 0x74, 0x1ecd, 0x6b, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69,
0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x6f, 0x6c, 0x76, 0xf3, 0x67,
0x72, 0x61, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x6f, 0x6c, 0x76,
0xf3, 0x67, 0x72, 0x61, 0x64, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x56, 0x6f, 0x6c, 0x76, 0xf3, 0x67, 0x72, 0x61,
0x64, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x56, 0x1ecd, 0x301, 0x73, 0x74, 0x1ecd, 0x6b,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0xe9, 0x6b, 0x20, 0x41, 0xed, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x57, 0xe1, 0x6c, 0x69,
0x73, 0x20, 0x61, 0x6e, 0x20, 0x46, 0xfa, 0x74, 0xfa, 0x6e, 0x61, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x59, 0xe9, 0x6b, 0xfa, 0x74, 0x73, 0x6b, 0x20,
0x54, 0x61, 0x69, 0x6d, 0x59, 0xe9, 0x6b, 0xfa, 0x74, 0x73, 0x6b, 0x20,
0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0xe9,
0x6b, 0xfa, 0x74, 0x73, 0x6b, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53,
0xed, 0x7a, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0x1eb9, 0x6b,
0x65, 0x74, 0x1eb9, 0x72, 0xed, 0x6e, 0x62, 0x75, 0x67, 0x20, 0x54, 0x61,
0x69, 0x6d, 0x59, 0x1eb9, 0x6b, 0x65, 0x74, 0x1eb9, 0x72, 0xed, 0x6e, 0x62,
0x75, 0x67, 0x20, 0x46, 0xed, 0x6b, 0x73, 0x64, 0x20, 0x54, 0x61, 0x69,
0x6d, 0x59, 0x1eb9, 0x6b, 0x65, 0x74, 0x1eb9, 0x72, 0xed, 0x6e, 0x62, 0x75,
0x67, 0x20, 0x48, 0x1ecd, 0x301, 0x74, 0x20, 0x53, 0xed, 0x7a, 0x69, 0x6e,
0x20, 0x54, 0x61, 0x69, 0x6d, 0x59, 0x75, 0x6b, 0xf3, 0x6e, 0x20, 0x54,
0x61, 0x69, 0x6d, 0x7dd, 0x7d8, 0x7ca, 0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec, 0x7e0,
0x7ca, 0x7eb, 0x20, 0x7d5, 0x7ca, 0x7f2, 0x7d3, 0x7ca, 0x7f2, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2, 0x7ca, 0x7d3, 0x7d8,
0x7cd, 0x7dd, 0x7d8, 0x7ca, 0x7ec, 0x7cc, 0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x7eb, 0x20,
0x7d3, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7d0, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2, 0x7ca, 0x7d3, 0x7d8, 0x7cd, 0x7dd, 0x7d8,
0x7cc, 0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x7eb, 0x20, 0x7e5, 0x7d9,
0x7cf, 0x7ec, 0x7d8, 0x7ce, 0x7ee, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec,
0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2, 0x7ca, 0x7d3, 0x7d8, 0x7cd, 0x7dd, 0x7d8, 0x7ca,
0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb,
0x7ec, 0x7d3, 0x7cb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb,
0x7f2, 0x7dd, 0x7d8, 0x7ca, 0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x7eb,
0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1,
0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2, 0x7ca, 0x7d3, 0x7d8, 0x7cd, 0x7dd,
0x7d8, 0x7ca, 0x7ec, 0x7dd, 0x7cc, 0x7f2, 0x7ec, 0x7e0, 0x7ca, 0x7eb, 0x20, 0x7d5,
0x7df, 0x7cb, 0x7ec, 0x7d3, 0x7cb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7ca, 0x7d9, 0x7cb, 0x7f2, 0x7dc, 0x7d9, 0x7cb,
0x7de, 0x7ce, 0x7f2, 0x7eb, 0x2d, 0x7dd, 0x7d9, 0x7cc, 0x7db, 0x7cc, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x7dc, 0x7d9, 0x7cb,
0x7de, 0x7ce, 0x7f2, 0x7eb, 0x2d, 0x7dd, 0x7d9, 0x7cc, 0x7db, 0x7cc, 0x7eb, 0x20,
0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2, 0x7ca,
0x7d3, 0x7d8, 0x7cd, 0x7dc, 0x7d9, 0x7cb, 0x7de, 0x7ce, 0x7f2, 0x7eb, 0x2d, 0x7dd,
0x7d9, 0x7cc, 0x7db, 0x7cc, 0x7eb, 0x20, 0x7d5, 0x7df, 0x7cb, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7ca, 0x7d9, 0x7cb, 0x7f2, 0x7d0, 0x7db, 0x7d1,
0x7d5, 0x7d9, 0x7ca, 0x7df, 0x7cc, 0x7eb, 0x20, 0x7d8, 0x7ce, 0x7f0, 0x7de, 0x7df,
0x7cf, 0x20, 0x7e3, 0x7cc, 0x7eb, 0x20, 0x7dd, 0x7ca, 0x7ec, 0x7d9, 0x7ca, 0x7f2,
0x7ec, 0x7db, 0x7cc, 0x7eb, 0x20, 0x7ca, 0x7f2, 0x7d5, 0x7ca, 0x7d9, 0x7d1, 0x7d5,
0x7cc, 0x7de, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7dc,
0x7ed, 0x7d9, 0x7cb, 0x7e3, 0x7cd, 0x7d5, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d8, 0x7ca, 0x7cd, 0x7f2, 0x7d8, 0x7ce, 0x7eb, 0x20, 0x7df, 0x7cc, 0x7f2,
0x7d3, 0x7ca, 0x7f2, 0x7d8, 0x7ce, 0x7ef, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca,
0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x7e1, 0x7cf, 0x7d9, 0x7cc, 0x7db, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x7e1, 0x7cf, 0x7d9, 0x7cc, 0x7db,
0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x20, 0x7e2,
0x7ca, 0x7d3, 0x7d8, 0x7cd, 0x7e1, 0x7cf, 0x7d9, 0x7cc, 0x7db, 0x20, 0x7d5, 0x7df,
0x7cb, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x20, 0x7d5, 0x7ce, 0x7e1, 0x7ca, 0x7d9, 0x7cb,
0x7f2, 0x7d9, 0x7cb, 0x7e3, 0x7cc, 0x7e6, 0x7d0, 0x7f2, 0x7eb, 0x20, 0x7d5, 0x7ce,
0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2, 0x7db, 0x7cb, 0x7e6, 0x7cc, 0x7db,
0x7cc, 0x7df, 0x20, 0x7d5, 0x7ce, 0x7ec, 0x7e1, 0x7ca, 0x7ec, 0x7d9, 0x7cb, 0x7f2,
0x6af, 0x627, 0x65b, 0x62a, 0x20, 0x645, 0x6cc, 0x646, 0x62c, 0x627, 0x65b, 0x6cc,
0x6cc, 0x6af, 0x627, 0x65b, 0x62a, 0x20, 0x645, 0x6cc, 0x646, 0x62c, 0x627, 0x65b,
0x6cc, 0x6cc, 0x20, 0x626, 0x633, 0x62a, 0x627, 0x65b, 0x646, 0x62f, 0x627, 0x65b,
0x631, 0x62f, 0x631, 0x648, 0x659, 0x634, 0x646, 0x627, 0x6cc, 0x6cc, 0x20, 0x646,
0x626, 0x647, 0x627, 0x62f, 0x627, 0x631, 0x20, 0x631, 0x648, 0x659, 0x632, 0x67,
0x61, 0x73, 0x6b, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x67, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x64, 0xe1, 0x62, 0xe1, 0x6c,
0x61, 0x161, 0xe1, 0x69, 0x67, 0x69, 0x67, 0x61, 0x73, 0x6b, 0x61, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x69, 0xe1, 0x69, 0x67, 0x69, 0x6e, 0x75, 0x6f, 0x72, 0x74, 0x69,
0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x6e, 0x75, 0x6f, 0x72, 0x74, 0x69, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x68, 0x70, 0xe1, 0x20, 0x64, 0xe1, 0x62, 0xe1, 0x6c, 0x61, 0x161, 0xe1,
0x69, 0x67, 0x69, 0x6e, 0x75, 0x6f, 0x72, 0x74, 0x69, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x69,
0xe1, 0x69, 0x67, 0x69, 0x6f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x6f, 0x61,
0x72, 0x6a, 0x65, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20,
0x64, 0xe1, 0x62, 0xe1, 0x6c, 0x61, 0x161, 0xe1, 0x69, 0x67, 0x69, 0x6f,
0x61, 0x72, 0x6a, 0x65, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x69, 0xe1, 0x69, 0x67, 0x69, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x67, 0x61, 0x73,
0x6b, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x6f, 0x73, 0x6b,
0x76, 0x61, 0x2d, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76,
0x61, 0x2d, 0x64, 0xe1, 0x62, 0xe1, 0x6c, 0x61, 0x161, 0xe1, 0x69, 0x67,
0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2d, 0x67, 0x65, 0x61, 0x73,
0x73, 0x69, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x68, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47,
0x61, 0x73, 0x6b, 0x61, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x68, 0x6b, 0xe1,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d,
0x41, 0x66, 0x72, 0x69, 0x68, 0x6b, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4c, 0x75, 0x6c, 0x6c, 0x69, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x68, 0x6b,
0xe1, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f,
0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x68, 0x6b, 0xe1,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41,
0x66, 0x72, 0x69, 0x68, 0x6b, 0xe1, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x66,
0x72, 0x69, 0x68, 0x6b, 0xe1, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x64,
0xe1, 0x62, 0xe1, 0x6c, 0x61, 0x161, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x64, 0xe1, 0x62, 0xe1, 0x6c, 0x61, 0x161, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0xe1, 0x69,
0x67, 0x69, 0x20, 0x6e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x6e, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x20, 0x6e, 0x75, 0x6f, 0x72,
0x74, 0x61, 0x6e, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x20, 0x6e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x6e, 0x64, 0x75, 0x6f,
0x74, 0x74, 0x61, 0x72, 0xe1, 0x69, 0x67, 0x69, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0x64, 0x75, 0x6f, 0x74, 0x74, 0x61, 0x72, 0xe1, 0x69, 0x67, 0x69,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0x64, 0x75, 0x6f, 0x74, 0x74, 0x61,
0x72, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61, 0x73, 0x6b, 0x65, 0x73, 0xe1,
0x62, 0x69, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61, 0x73, 0x6b, 0x65,
0x73, 0xe1, 0x62, 0x69, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x4a, 0x61, 0x73, 0x6b, 0x65, 0x73, 0xe1, 0x62, 0x69, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x70,
0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x70, 0x69,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x41, 0x72, 0xe1, 0x62, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x41,
0x72, 0xe1, 0x62, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72,
0x6a, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61,
0x72, 0x6a, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x61,
0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x161, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x61, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x161,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x61, 0x74,
0x6c, 0xe1, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x161, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x61,
0x72, 0x6a, 0x6a, 0x61, 0x62, 0x65, 0x61, 0x6c, 0x69, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x61, 0x72, 0x6a, 0x6a, 0x61,
0x62, 0x65, 0x61, 0x6c, 0x69, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x61, 0x72, 0x6a, 0x6a,
0x61, 0x62, 0x65, 0x61, 0x6c, 0x69, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61,
0x69, 0x17e, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x73,
0x65, 0x72, 0x62, 0x61, 0x69, 0x17e, 0x61, 0x6e, 0x61, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x73, 0x65, 0x72, 0x62,
0x61, 0x69, 0x17e, 0x61, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x61, 0x69, 0x64,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x61, 0x69, 0x64,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x7a,
0x6f, 0x72, 0x61, 0x69, 0x64, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20,
0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x10c, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x61, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x69, 0x69,
0x6e, 0x6e, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x69, 0x69, 0x6e,
0x6e, 0xe1, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x4b, 0x69, 0x69, 0x6e, 0x6e, 0xe1, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x6f, 0x6c,
0x73, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x6f,
0x69, 0x62, 0x6f, 0x6c, 0x73, 0x61, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x6f,
0x6c, 0x73, 0x61, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4a, 0x75, 0x6f, 0x76, 0x6c, 0x61, 0x73, 0x75,
0x6c, 0x6c, 0x6f, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x75, 0x6c, 0x6c, 0x6f,
0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43, 0x6f, 0x6f, 0x6b, 0x73,
0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x43, 0x6f, 0x6f, 0x6b, 0x73, 0x75, 0x6c, 0x6c,
0x6f, 0x69, 0x64, 0x20, 0x67, 0x65, 0x61, 0x73, 0x69, 0x20, 0x62, 0x65,
0x61, 0x6c, 0x6c, 0x65, 0x6d, 0x75, 0x74, 0x74, 0x75, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x43, 0x75, 0x62, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x43,
0x75, 0x62, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x43, 0x75, 0x62, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x44, 0x61, 0x76, 0x69, 0x73, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019,
0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x54, 0x69, 0x6d, 0x6f, 0x72,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x65, 0x61, 0x73, 0x73, 0xe1,
0x161, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x42,
0x65, 0x61, 0x73, 0x73, 0xe1, 0x161, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x42, 0x65, 0x61,
0x73, 0x73, 0xe1, 0x161, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x45, 0x63, 0x75, 0x61,
0x64, 0x6f, 0x72, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73,
0x6b, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x68, 0x70, 0xe1, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x47, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x68, 0x70, 0xe1, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x45, 0x75, 0x72,
0x6f, 0x68, 0x70, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f,
0x72, 0x74, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0x61, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f,
0x72, 0x74, 0x61, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0x61, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47, 0xe1,
0x69, 0x64, 0x64, 0x75, 0x73, 0x2d, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x74,
0x69, 0x20, 0x65, 0x75, 0x72, 0x6f, 0x68, 0x70, 0x61, 0x6c, 0x61, 0x161,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x45,
0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f,
0x61, 0x72, 0x6a, 0x65, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61,
0x72, 0x6a, 0x65, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x68, 0x70, 0xe1, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x75, 0x6c, 0x6c, 0x75, 0x69,
0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x75, 0x6c, 0x6c, 0x75, 0x69, 0x64, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x75, 0x6c, 0x6c, 0x75, 0x69, 0x64, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x69, 0x6a,
0x69, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x69, 0x6a, 0x69,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x46,
0x72, 0x61, 0x6e, 0x6b, 0x72, 0x69, 0x69, 0x6b, 0x6b, 0x61, 0x20, 0x47,
0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x72,
0x61, 0x6e, 0x6b, 0x72, 0x69, 0x69, 0x6b, 0x6b, 0x61, 0x20, 0x6c, 0x75,
0x6c, 0x6c, 0x69, 0x20, 0x26, 0x20, 0x61, 0x6e, 0x74, 0xe1, 0x72, 0x6b,
0x74, 0x69, 0x73, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x61, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x73, 0x75, 0x6c, 0x6c, 0x6f,
0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75,
0x6f, 0x72, 0x74, 0x61, 0x2d, 0x52, 0x75, 0x6f, 0x6e, 0xe1, 0x65, 0x61,
0x74, 0x6e, 0x61, 0x6d, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75,
0x6f, 0x72, 0x74, 0x61, 0x2d, 0x52, 0x75, 0x6f, 0x6e, 0xe1, 0x65, 0x61,
0x74, 0x6e, 0x61, 0x6d, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d, 0x52, 0x75,
0x6f, 0x6e, 0xe1, 0x65, 0x61, 0x74, 0x6e, 0x61, 0x6d, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72,
0x6a, 0x65, 0x2d, 0x52, 0x75, 0x6f, 0x6e, 0xe1, 0x65, 0x61, 0x74, 0x6e,
0x61, 0x6d, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a,
0x65, 0x2d, 0x52, 0x75, 0x6f, 0x6e, 0xe1, 0x65, 0x61, 0x74, 0x6e, 0x61,
0x6d, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x52, 0x75, 0x6f, 0x6e, 0xe1, 0x65,
0x61, 0x74, 0x6e, 0x61, 0x6d, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x6f, 0x6c, 0x66, 0x61, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x68, 0x74, 0x61, 0x6c, 0x61,
0x161, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x61, 0x6c, 0x65, 0x75, 0x68, 0x74, 0x61, 0x6c, 0x61, 0x161, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x68, 0x74, 0x61, 0x6c,
0x61, 0x161, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x6f, 0x76,
0x64, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x6f, 0x76, 0x64, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x48, 0x6f,
0x76, 0x64, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x6e, 0x64, 0x69, 0x61, 0xe1, 0x62,
0x69, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69,
0x69, 0x6e, 0x6e, 0xe1, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x47, 0x61, 0x73,
0x6b, 0x61, 0x2d, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f, 0x72, 0x74, 0x61, 0x2d,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65, 0x2d, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x72,
0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x72, 0x61, 0x6e,
0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x49,
0x72, 0x61, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x61, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x72, 0x6b, 0x75,
0x63, 0x6b, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x61, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x4a, 0x61, 0x70, 0xe1, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4a, 0x61, 0x70, 0xe1, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61, 0x70, 0xe1, 0x6e, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x75, 0x6f,
0x72, 0x74, 0x61, 0x2d, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x73, 0x74, 0x61,
0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x61, 0x72, 0x6a, 0x65,
0x2d, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x6f, 0x73,
0x72, 0x61, 0x65, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4b,
0x69, 0x72, 0x67, 0x69, 0x73, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4c, 0x69, 0x6e, 0x65, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d,
0x61, 0x6c, 0x65, 0x73, 0x69, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d,
0x61, 0x6c, 0x65, 0x64, 0x69, 0x69, 0x76, 0x76, 0x61, 0x69, 0x64, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61,
0x73, 0x69, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x6c, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69, 0x64,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x4a, 0xe1, 0x73,
0x6b, 0x65, 0x73, 0xe1, 0x62, 0x69, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x4a, 0xe1, 0x73, 0x6b, 0x65,
0x73, 0xe1, 0x62, 0x69, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x4a, 0xe1,
0x73, 0x6b, 0x65, 0x73, 0xe1, 0x62, 0x69, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x55, 0x6c,
0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x2d,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4d, 0x6f, 0x73, 0x6b,
0x76, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f,
0x111, 0x111, 0x61, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x111, 0x111, 0x61, 0x2d, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x64, 0xe1, 0x6c,
0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x111, 0x111, 0x61, 0x2d, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x67, 0x65, 0x61,
0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x111, 0x111, 0x61, 0x2d,
0x53, 0x65, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4f, 0x111, 0x111, 0x61, 0x2d, 0x53, 0x65, 0x6c, 0xe1, 0x6e, 0x64, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x111,
0x111, 0x61, 0x2d, 0x53, 0x65, 0x6c, 0xe1, 0x6e, 0x64, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x4e, 0x69, 0x75, 0x65, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x73, 0x75, 0x6c, 0x6c, 0x6f,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4f, 0x111, 0x111,
0x61, 0x2d, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0xe1, 0x69, 0x67,
0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61,
0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x65, 0x72, 0x75, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x50, 0x65, 0x72, 0x75, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x65, 0x72, 0x75, 0x20, 0x67, 0x65,
0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x70, 0x69, 0x69, 0x6e, 0x6e, 0x61, 0x69, 0x64, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x69, 0x6e, 0x6e,
0x61, 0x69, 0x64, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x69, 0x6e, 0x6e, 0x61,
0x69, 0x64, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x73, 0x75, 0x6c, 0x6c,
0x6f, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x74, 0x2e, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x74, 0x2e,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x20,
0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x69,
0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x69,
0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61,
0x6e, 0x67, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x52, 0x65, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x52, 0x6f, 0x74,
0x68, 0x65, 0x72, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x61, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x61,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c,
0x61, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x6f, 0x72, 0x65, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x75,
0x6c, 0x6c, 0x6f, 0x69, 0x64, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4c, 0x75,
0x6c, 0x6c, 0x69, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x61, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x54, 0x61, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61,
0x20, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x43,
0x68, 0x75, 0x75, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x20,
0x64, 0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65,
0xe1, 0x69, 0x67, 0x69, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x55, 0x73, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1,
0x69, 0x67, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0xe1,
0x69, 0x67, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73,
0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x61, 0x20, 0x64, 0xe1, 0x6c, 0x76,
0x65, 0xe1, 0x69, 0x67, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69,
0x67, 0x69, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x57, 0x61, 0x6b, 0x65, 0x73, 0x75, 0x6c, 0x6c, 0x6f, 0x20,
0xe1, 0x69, 0x67, 0x69, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x20,
0x6a, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x61, 0x20, 0xe1, 0x69,
0x67, 0x69, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x61, 0x20, 0x64, 0xe1,
0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x61, 0x6b, 0x75, 0x63,
0x6b, 0x61, 0x20, 0x67, 0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67,
0x69, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x61, 0x20, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x20, 0x64,
0xe1, 0x6c, 0x76, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x61, 0x20, 0x67,
0x65, 0x61, 0x73, 0x73, 0x65, 0xe1, 0x69, 0x67, 0x69, 0x41, 0x63, 0x72,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41,
0x63, 0x72, 0x65, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x61, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x73, 0xf8, 0x72, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65,
0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73,
0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x61, 0x6c, 0x61, 0x73,
0x6b, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x6c, 0x61, 0x73,
0x6b, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x79, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x74, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2c, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x61, 0x73, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x73, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x64, 0x65, 0x74, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x74, 0x20, 0x73, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x74, 0x20,
0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65,
0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0xf8, 0x73, 0x74, 0x6b, 0x79, 0x73,
0x74, 0x65, 0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0xf8, 0x73, 0x74, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0xf8, 0x73, 0x74, 0x6b,
0x79, 0x73, 0x74, 0x65, 0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x20,
0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x73, 0x20, 0x28, 0x55,
0x53, 0x41, 0x29, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x20, 0x4d,
0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x73, 0x20, 0x28, 0x55, 0x53,
0x41, 0x29, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x20, 0x4d, 0x6f,
0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x73, 0x20, 0x28, 0x55, 0x53, 0x41,
0x29, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x53, 0x74,
0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74,
0x65, 0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x53, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79,
0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x53, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73,
0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73,
0x6b, 0x20, 0x28, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x29, 0x20, 0x74,
0x69, 0x64, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x29, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20,
0x28, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x29, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x41, 0x70, 0x69, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41,
0x71, 0x74, 0x61, 0x75, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2c, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74,
0x6f, 0x62, 0x65, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x74, 0x69, 0x64, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x2c, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x61, 0x72, 0x61,
0x62, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x72, 0x61, 0x62,
0x69, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x74, 0x69, 0x64, 0x61, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x61, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x65, 0x72, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74,
0x65, 0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x65, 0x72, 0x68, 0x61, 0x76, 0x73,
0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x65, 0x72,
0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x76, 0x65,
0x73, 0x74, 0x2d, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76,
0x65, 0x73, 0x74, 0x2d, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x2d, 0x73,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0xf8, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x61, 0x73, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64,
0x73, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x73,
0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x61, 0x73,
0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x73, 0x6a, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x61, 0x73,
0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x73, 0x6f,
0x72, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x61, 0x73, 0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x62, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x62,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x62, 0x68,
0x75, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x62, 0x6f,
0x6c, 0x69, 0x76, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x6b,
0x61, 0x70, 0x70, 0x76, 0x65, 0x72, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x6b, 0x61, 0x70, 0x70, 0x76, 0x65, 0x72, 0x64, 0x69, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6b,
0x61, 0x70, 0x70, 0x76, 0x65, 0x72, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x43, 0x61, 0x73, 0x65,
0x79, 0x2d, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72,
0x72, 0x6f, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x63, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x63, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x63, 0x68,
0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6b, 0x69,
0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x73, 0x6a, 0x6f, 0x6a, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x73, 0x6a, 0x6f, 0x6a, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x73, 0x6a, 0x6f, 0x6a,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73,
0x74, 0x6d, 0x61, 0x73, 0xf8, 0x79, 0x61, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x63, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x63, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x63, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x79, 0x65, 0x6e,
0x65, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xf8, 0x79, 0x65, 0x6e, 0x65,
0x68, 0x61, 0x6c, 0x76, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0xf8,
0x79, 0x65, 0x6e, 0x65, 0x63, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x63, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x63, 0x75, 0x62, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x44, 0x75, 0x6d, 0x6f,
0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65,
0xf8, 0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x73, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8,
0x79, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x79, 0x61,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x79, 0x61, 0x65, 0x63,
0x75, 0x61, 0x64, 0x6f, 0x72, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0xf8, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x66, 0x6a, 0x65, 0x72, 0x6e, 0x2d, 0xf8, 0x73, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x76, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x79, 0x65,
0x6e, 0x65, 0x66, 0x69, 0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x66, 0x69, 0x6a, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x66, 0x69, 0x6a, 0x69,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x61, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x44, 0x65, 0x20, 0x66, 0x72, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0xf8, 0x72, 0x74, 0x65, 0x72, 0x72, 0x69,
0x74, 0x6f, 0x72, 0x69, 0x65, 0x72, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70,
0x61, 0x67, 0x6f, 0x73, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x67,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x20, 0x6d, 0x69, 0x64, 0x64, 0x65, 0x6c, 0x74, 0x69,
0x64, 0xf8, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x67, 0x72, 0xf8,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65,
0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0x65,
0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x47, 0x75,
0x61, 0x6d, 0x2d, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0x65, 0x72, 0x73, 0x69,
0x61, 0x62, 0x75, 0x6b, 0x74, 0x61, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x20, 0x6f, 0x67, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x65,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x20, 0x6f, 0x67, 0x20,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x20, 0x6f, 0x67, 0x20, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x65, 0x6e, 0x65, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x67, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x68,
0x6f, 0x76, 0x64, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4b, 0x68, 0x6f, 0x76, 0x64, 0x69, 0x6e, 0x64, 0x69, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x68, 0x61, 0x76,
0x65, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74,
0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x69, 0x73,
0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x61,
0x70, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6a, 0x61, 0x70,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x52, 0x75, 0x73, 0x73, 0x69,
0x73, 0x6b, 0x20, 0x28, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x6a, 0x29, 0x20, 0x74, 0x69, 0x64, 0x52,
0x75, 0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x6a, 0x29, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x52, 0x75, 0x73,
0x73, 0x69, 0x73, 0x6b, 0x20, 0x28, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74,
0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x6a, 0x29, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0xf8, 0x73, 0x74, 0x6b, 0x61,
0x73, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x6b, 0x61, 0x73, 0x61, 0x6b, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6b, 0x6f,
0x72, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6b, 0x6f,
0x72, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b,
0x6f, 0x73, 0x72, 0x61, 0x65, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x6b, 0x69, 0x72, 0x67, 0x69,
0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4c, 0x61, 0x6e, 0x6b,
0x61, 0x2d, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4c, 0x69, 0x6e, 0x6a, 0x65, 0xf8,
0x79, 0x65, 0x6e, 0x65, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x2d, 0xf8, 0x79, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0xf8, 0x79, 0x61, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0xf8, 0x79, 0x61,
0x4d, 0x61, 0x63, 0x61, 0x75, 0x2d, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x63,
0x61, 0x75, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x74, 0x69, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x2c, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6d, 0x61, 0x6c,
0x61, 0x79, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0xf8, 0x79, 0x65,
0x6e, 0x65, 0x6d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x65, 0x73,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d,
0x61, 0x77, 0x73, 0x6f, 0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x53, 0x74,
0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74,
0x65, 0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x53, 0x74, 0x69, 0x6c,
0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x53, 0x74, 0x69, 0x6c, 0x6c, 0x65,
0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x6f,
0x73, 0x6b, 0x76, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x6d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6e, 0x61, 0x75,
0x72, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6e, 0x65, 0x70, 0x61,
0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6e, 0x65,
0x77, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x6e, 0x65, 0x77, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x6e, 0x65, 0x77, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0xf8, 0x79, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0xf8, 0x79, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0xf8, 0x79, 0x61, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4e, 0x6f, 0x72,
0x64, 0x2d, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x65, 0x6e, 0x65, 0x2d,
0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x70, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x70, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x70, 0x61, 0x6c, 0x61, 0x75, 0x69, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x70, 0x61, 0x70, 0x75, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x70, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x70, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x70, 0x65, 0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x70, 0x65,
0x72, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69,
0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50,
0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x50, 0x79,
0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x51, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x74, 0x69, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x52, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x61, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x52, 0x75,
0x73, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x28, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x29, 0x20, 0x74, 0x69, 0x64, 0x52, 0x75, 0x73, 0x73, 0x69, 0x73,
0x6b, 0x20, 0x28, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x29, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x52, 0x75, 0x73, 0x73,
0x69, 0x73, 0x6b, 0x20, 0x28, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x29,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x73, 0x61,
0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x61,
0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x73, 0x65,
0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x73, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x73, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0xf8, 0x72, 0x2d, 0x47, 0x65, 0x6f,
0x72, 0x67, 0x69, 0x61, 0x73, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x74,
0x61, 0x68, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x74, 0x61,
0x64, 0x73, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x74, 0x6f, 0x6e, 0x67,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x6f, 0x6e, 0x67,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x68, 0x75,
0x75, 0x6b, 0xf8, 0x79, 0x65, 0x6e, 0x65, 0x74, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x74, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x74, 0x75,
0x76, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x75, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x75, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x75, 0x73, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x75, 0x73, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x75, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x6e,
0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69,
0x73, 0x2d, 0x20, 0x6f, 0x67, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0xf8, 0x79, 0x65, 0x6e, 0x65, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e,
0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x73, 0x6f,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x74, 0x69, 0x64, 0x73, 0x73,
0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x73, 0x6f, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x61, 0x75, 0x73,
0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x64, 0x2d, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x61,
0x75, 0x73, 0x74, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x61, 0x75, 0x73, 0x74, 0x6b,
0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x61, 0x75, 0x73, 0x74, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x52, 0x6f, 0x63, 0x6b, 0x79, 0x20, 0x4d, 0x6f, 0x75, 0x6e,
0x74, 0x61, 0x69, 0x6e, 0x73, 0x20, 0x28, 0x55, 0x53, 0x41, 0x29, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x69, 0x6c,
0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74,
0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74,
0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79,
0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x41, 0x70, 0x69, 0x61, 0x61, 0x72,
0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x61, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x64,
0x65, 0x6e, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x65, 0x72, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65,
0x6e, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x74, 0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x2d,
0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x2d, 0x73, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61,
0x75, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x61, 0x73, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64,
0x73, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x61, 0x73, 0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x62, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6b, 0x61, 0x70, 0x70, 0x76, 0x65,
0x72, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x63, 0x68, 0x69, 0x6c, 0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x54, 0x73, 0x6a, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65,
0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x6f,
0x6f, 0x6b, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0xf8, 0x79, 0x61, 0x6e, 0x65, 0x6b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x74, 0x69, 0x64, 0x6b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6b, 0x75, 0x62,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019,
0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x61, 0x75, 0x73, 0x74, 0x74,
0x69, 0x6d, 0x6f, 0x72, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x50, 0xe5, 0x73, 0x6b, 0x65, 0xf8, 0x79, 0x61, 0x73,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x69, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x74, 0x69, 0x64, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x75,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x61,
0x75, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x66, 0x6a,
0x65, 0x72, 0x6e, 0x2d, 0x61, 0x75, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x74, 0x69, 0x64, 0x76, 0x65,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64,
0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf8,
0x79, 0x61, 0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x66, 0x69, 0x6a, 0x69,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x44, 0x65, 0x69, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0xf8, 0x72, 0x74, 0x65, 0x72, 0x72, 0x69, 0x74,
0x6f, 0x72, 0x69, 0x61, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67,
0x6f, 0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x67, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0xf8, 0x79, 0x61,
0x6e, 0x65, 0x61, 0x75, 0x73, 0x74, 0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74,
0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74,
0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x76, 0x65, 0x73, 0x74,
0x67, 0x72, 0xf8, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x20, 0x6f, 0x67, 0x20, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x65, 0x6e, 0x65, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67,
0x6b, 0x69, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x6b, 0x69, 0x6e, 0x65,
0x73, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x6b, 0x69,
0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x68, 0x6f, 0x76, 0x64, 0x61,
0x75, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x69, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x75, 0x73, 0x74, 0x6b, 0x61, 0x73,
0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x6b, 0x6f, 0x72, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4c, 0x69, 0x6e,
0x65, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0xf8, 0x79, 0x61, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f,
0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75,
0x65, 0x73, 0x61, 0x73, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x6d, 0x61, 0x75,
0x72, 0x69, 0x74, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65,
0x20, 0x66, 0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x69,
0x6c, 0x6c, 0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65,
0x6e, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x64, 0x65, 0x6e, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c,
0x65, 0x68, 0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x64, 0x65, 0x6e, 0x20, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c, 0x65, 0x68,
0x61, 0x76, 0x73, 0x6b, 0x79, 0x73, 0x74, 0x65, 0x6e, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4d,
0x6f, 0x73, 0x6b, 0x76, 0x61, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x6e, 0x79, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x6e, 0x79, 0x7a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xf8, 0x79, 0x61, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x70,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x70, 0x65, 0x72,
0x75, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74,
0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20,
0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0xf8, 0x79, 0x61, 0x6e, 0x65,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20,
0x66, 0x6f, 0x72, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x73, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x74, 0x69, 0x64, 0x73,
0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x43, 0x68, 0x75,
0x75, 0x6b, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x74, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x75,
0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x69,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f,
0x72, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66,
0x6f, 0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x74, 0x69, 0x64, 0x73, 0x73, 0x6f, 0x6e, 0x65, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x20, 0x6f, 0x67, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0xf8, 0x79, 0x61, 0x6e, 0x65, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72,
0x20, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x20, 0x66, 0x6f, 0x72, 0x20, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x75, 0x64, 0x61, 0x66, 0x72, 0x69, 0x63,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a,
0xf2, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e,
0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc8, 0x73, 0x74, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0xc8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61,
0x6e, 0x68, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x6d, 0x6f, 0x6e,
0x74, 0x61, 0x6e, 0x68, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x70,
0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x69, 0x75,
0x20, 0x64, 0x2019, 0x41, 0x70, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x61, 0x62,
0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74, 0x20, 0x64, 0x2019, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74, 0x20, 0x64, 0x2019, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x6f, 0xe8, 0x73, 0x74, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x65, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x41, 0xe7, 0xf2, 0x72,
0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x41, 0xe7, 0xf2, 0x72, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x41, 0xe7, 0xf2, 0x72, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x61, 0x6c, 0x64, 0x65, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x61, 0x6c,
0x64, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x61,
0x6c, 0x64, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x42, 0x75, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61,
0x64, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x69, 0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d,
0x62, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c,
0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x6d, 0x69, 0x65, 0x6a, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x75, 0x62, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54,
0x69, 0x6d, 0xf2, 0x72, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x69,
0x6c, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x6c,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x45, 0x71, 0x75, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0xe8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0xf2, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x65, 0x78, 0x74, 0x72, 0x65, 0x6d, 0x20, 0x64, 0x2019, 0x45,
0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0xf2, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73,
0x74, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x6f, 0xe8, 0x73, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c,
0x76, 0x69, 0x6e, 0x61, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x73, 0x20, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c,
0x76, 0x69, 0x6e, 0x61, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x69, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e,
0x61, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x75, 0x61,
0x79, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20,
0x54, 0x65, 0x72, 0x72, 0x65, 0x73, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x73, 0x20, 0x65, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72,
0x74, 0x69, 0x71, 0x75, 0x65, 0x73, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x63,
0x65, 0x73, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x65, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x61, 0x6c, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e,
0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73,
0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xe8, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8,
0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x6f, 0xe8, 0x73, 0x74, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x47, 0xf2, 0x6c, 0x66, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x63, 0x65,
0x61, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x61,
0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019,
0x49, 0x72, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x73,
0x72, 0x61, 0xe8, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72,
0x61, 0xe8, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0xe8, 0x6c, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x61, 0x74,
0x61, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72,
0xe8, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0xe8,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0xe8, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x69, 0x72, 0x67, 0x75, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x45, 0x73, 0x70, 0xf2,
0x72, 0x61, 0x64, 0x65, 0x73, 0x20, 0x45, 0x71, 0x75, 0x61, 0x74, 0x6f,
0x72, 0x69, 0x61, 0x6c, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x65, 0x73,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49,
0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73,
0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73,
0x20, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x63, 0x69, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61,
0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0xf2, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0xf2, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xf2, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0xf2, 0x76, 0x61, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x4e, 0xf2, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2,
0x6e, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4e, 0xf2, 0x76, 0x61, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4e, 0xf2, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0xf2, 0x76, 0x61,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x4e,
0xf2, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x2d,
0x4e, 0xf2, 0x76, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65,
0x72, 0x72, 0x61, 0x2d, 0x4e, 0xf2, 0x76, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x54,
0x65, 0x72, 0x72, 0x61, 0x2d, 0x4e, 0xf2, 0x76, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x69, 0x6c, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61,
0x6f, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x70,
0x75, 0x61, 0x20, 0x4e, 0xf2, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e,
0xe8, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65,
0x72, 0xfa, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x73, 0x20, 0x49, 0x6c, 0x6c,
0x65, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x52, 0x65, 0xfc, 0x6e, 0x69, 0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0x69, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61,
0x70, 0x6f, 0x72, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x73, 0x20, 0x49, 0x6c, 0x6c, 0x65, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f,
0x6d, 0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x49, 0x6c, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x61,
0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62,
0x75, 0x72, 0x67, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x61, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x64, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e, 0x65, 0x73,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e,
0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0xf2, 0x6e,
0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61,
0x6e, 0x68, 0x65, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x6f, 0x73, 0x65, 0x73,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x6e,
0x68, 0x65, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x6f, 0x73, 0x65, 0x73, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x65, 0x73,
0x20, 0x52, 0x6f, 0x63, 0x6f, 0x73, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x74, 0x68, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20,
0x64, 0x2019, 0x61, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x41, 0x67, 0x65, 0x72, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x72, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x72, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75,
0x20, 0x64, 0x65, 0x72, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69,
0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x65, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x65, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x65, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x6a, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73,
0x20, 0x41, 0xe7, 0xf2, 0x72, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x73,
0x20, 0x41, 0xe7, 0xf2, 0x72, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x73, 0x20, 0x41,
0xe7, 0xf2, 0x72, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x43, 0x61, 0x70, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x61, 0x64, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x72, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x6f, 0x63, 0x6f, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f,
0x6c, 0xf2, 0x6d, 0x62, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x73, 0x20, 0x49, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x65, 0x73, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x6d, 0x69, 0x65, 0x6a, 0x61, 0x20, 0x64, 0x65,
0x73, 0x20, 0x49, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0xf2, 0x72, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x72, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x73, 0x71, 0x75, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20,
0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71,
0x75, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0xf2, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xe8, 0x73, 0x74, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x72, 0x20, 0x65, 0x78, 0x74, 0x72, 0x65, 0x6d,
0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72,
0xf2, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69,
0x75, 0x20, 0x64, 0x2019, 0x45, 0x75, 0x72, 0xf2, 0x70, 0x61, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x73,
0x20, 0x69, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x65, 0x73, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74,
0x69, 0x75, 0x20, 0x64, 0x65, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x65, 0x73,
0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x47, 0x75, 0x61, 0x79, 0x61, 0x6e,
0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69,
0x64, 0x61, 0x20, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f,
0x72, 0x69, 0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x69, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x65, 0x73, 0x20, 0x47,
0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x74, 0x68, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x65, 0x73, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x72, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x72, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x63, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x75, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49,
0x72, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0xe8, 0x6c,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x75,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72,
0xe8, 0x61, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x73, 0x20, 0x45, 0x73, 0x70, 0xf2, 0x72, 0x61, 0x64, 0x65, 0x73, 0x20,
0x45, 0x71, 0x75, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c, 0x73, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x65, 0x73,
0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x65, 0x73, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x65, 0x73, 0x20,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x74, 0x68, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x74, 0x68, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x74, 0x68,
0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x6f, 0x73, 0x63, 0xf2, 0x75, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x74, 0x68, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x2d, 0x4e, 0xf2, 0x76, 0x61,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x72, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x72, 0x61, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x2019, 0x4f, 0x6d, 0x73, 0x6b, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65,
0x73, 0x74, 0x69, 0x75, 0x20, 0x64, 0x65, 0x74, 0x68, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x50, 0x65, 0x72, 0xfa, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73,
0x74, 0x75, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x73,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x65,
0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x52, 0x65, 0xfc, 0x6e, 0x69, 0x6f, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0x69, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x65, 0x73,
0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64,
0x65, 0x74, 0x68, 0x20, 0x53, 0x75, 0x64, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x65, 0x73, 0x74, 0x69, 0x75, 0x20, 0x64,
0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x72,
0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62,
0x75, 0x72, 0x67, 0xb06, 0xb15, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb15,
0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb15,
0xb3e, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb06, 0xb2b, 0xb17, 0xb3e, 0xb28, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb06, 0xb2b, 0xb4d,
0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d,
0xb2c, 0x20, 0xb06, 0xb2b, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb26, 0xb15, 0xb4d, 0xb37, 0xb3f, 0xb23, 0x20, 0xb06, 0xb2b, 0xb4d, 0xb30,
0xb3f, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06, 0xb2b,
0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d,
0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06, 0xb2b, 0xb4d, 0xb30, 0xb3f, 0xb15, 0xb3e, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06, 0xb2b, 0xb4d, 0xb30, 0xb3f, 0xb15,
0xb3e, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26, 0xb3f, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb06, 0xb32, 0xb3e, 0xb38, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06,
0xb32, 0xb3e, 0xb38, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb32, 0xb3e, 0xb38, 0xb4d, 0xb15,
0xb3e, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb05, 0xb32, 0xb2e, 0xb3e, 0xb1f, 0xb40, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05,
0xb32, 0xb2e, 0xb3e, 0xb1f, 0xb40, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb05, 0xb32, 0xb2e, 0xb3e, 0xb1f, 0xb40, 0x20, 0xb17, 0xb4d, 0xb30,
0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb2e, 0xb3e, 0xb1c,
0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb2e, 0xb3e, 0xb1c, 0xb28, 0xb4d,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb06, 0xb2e, 0xb3e, 0xb1c, 0xb28, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15,
0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0xb40, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb15, 0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0xb40, 0xb5f, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28,
0xb4d, 0xb26, 0xb4d, 0xb30, 0xb40, 0xb5f, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32,
0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e,
0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d,
0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e,
0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb30, 0xb4d, 0xb2c, 0xb24, 0xb4d, 0xb5f,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb30, 0xb4d, 0xb2c, 0xb24, 0xb4d, 0xb5f,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb3e, 0xb30, 0xb4d, 0xb2c, 0xb24, 0xb4d, 0xb5f, 0x20, 0xb26, 0xb3f, 0xb2c,
0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb38, 0xb3f,
0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb38, 0xb3f,
0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb38, 0xb3f, 0xb2b, 0xb3f, 0xb15, 0xb4d,
0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb05, 0xb28, 0xb3e, 0xb21, 0xb3f, 0xb30, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05,
0xb28, 0xb3e, 0xb21, 0xb3f, 0xb30, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb05, 0xb28, 0xb3e, 0xb21, 0xb3f, 0xb30, 0xb4d, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb2a,
0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb2a, 0xb3f, 0xb06, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb2a,
0xb3f, 0xb06, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb05, 0xb15, 0xb24, 0xb4c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb15,
0xb24, 0xb4c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05,
0xb15, 0xb24, 0xb4c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb05, 0xb15, 0xb4d, 0xb24, 0xb4b, 0xb2c, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb05, 0xb15, 0xb4d, 0xb24, 0xb4b, 0xb2c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb15, 0xb4d, 0xb24, 0xb4b, 0xb2c, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30,
0xb2c, 0xb40, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30, 0xb2c, 0xb40, 0xb5f,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb06, 0xb30, 0xb2c, 0xb40, 0xb5f, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d,
0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30, 0xb4d, 0xb1c,
0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47,
0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d,
0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f,
0xb3f, 0xb28, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f,
0xb2e, 0x20, 0xb06, 0xb30, 0xb4d, 0xb1c, 0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28,
0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb06, 0xb30, 0xb4d, 0xb1c,
0xb47, 0xb23, 0xb4d, 0xb1f, 0xb3f, 0xb28, 0xb3e, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30,
0xb4d, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30,
0xb4d, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb06, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb30, 0xb4d, 0xb2e, 0xb47, 0xb28,
0xb3f, 0xb06, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e,
0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1f, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f,
0xb3f, 0xb15, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1f, 0xb32, 0xb3e, 0xb23,
0xb4d, 0xb1f, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1f, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb1f,
0xb3f, 0xb15, 0xb4d, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb05, 0xb37, 0xb4d, 0xb1f,
0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37,
0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2e, 0xb27, 0xb4d,
0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20,
0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32,
0xb3f, 0xb5f, 0x20, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb2a, 0xb36, 0xb4d, 0xb1a,
0xb3f, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30,
0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb2a, 0xb36,
0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32,
0xb3f, 0xb5f, 0x20, 0xb2e, 0xb27, 0xb4d, 0xb5f, 0x20, 0xb2a, 0xb36, 0xb4d, 0xb1a,
0xb3f, 0xb2e, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb05, 0xb37, 0xb4d, 0xb1f,
0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37,
0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2a, 0xb42, 0xb30,
0xb4d, 0xb2c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f,
0x20, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32,
0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e,
0x20, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb06, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f, 0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f,
0xb5f, 0x20, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb05, 0xb37, 0xb4d, 0xb1f,
0xb4d, 0xb30, 0xb47, 0xb32, 0xb3f, 0xb5f, 0x20, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f,
0xb2e, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb06, 0xb1c, 0xb47, 0xb30, 0xb2c, 0xb3e, 0xb07, 0xb1c, 0xb3e, 0xb28, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1c, 0xb47, 0xb30, 0xb2c, 0xb3e, 0xb07, 0xb1c, 0xb3e,
0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb06, 0xb1c, 0xb47, 0xb30, 0xb2c, 0xb3e, 0xb07, 0xb1c, 0xb3e, 0xb28, 0x20, 0xb16,
0xb30, 0xb3e, 0xb26, 0xb3f, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1c, 0xb4b,
0xb30, 0xb47, 0xb38, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1c, 0xb4b, 0xb30,
0xb47, 0xb38, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb06, 0xb1c, 0xb4b, 0xb30, 0xb47, 0xb38, 0xb4d, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2c, 0xb3e, 0xb02, 0xb32, 0xb3e, 0xb26, 0xb47, 0xb36, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2c, 0xb3e, 0xb02, 0xb32, 0xb3e, 0xb26, 0xb47, 0xb36, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb3e, 0xb02,
0xb32, 0xb3e, 0xb26, 0xb47, 0xb36, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb41, 0xb1f, 0xb3e,
0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb32, 0xb3f, 0xb2d, 0xb3f, 0xb06, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb3f, 0xb32, 0xb3f, 0xb06,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb3f, 0xb32, 0xb3f,
0xb06, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb3f, 0xb32, 0xb3f, 0xb06, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2c, 0xb4d, 0xb30, 0xb41, 0xb28, 0xb47, 0xb07, 0x20, 0xb21, 0xb3e, 0xb30,
0xb41, 0xb38, 0xb3e, 0xb32, 0xb3e, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47,
0xb2a, 0xb4d, 0x200c, 0x20, 0xb2d, 0xb30, 0xb4d, 0xb26, 0xb47, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb15, 0xb47, 0xb2a, 0xb4d, 0x200c, 0x20, 0xb2d, 0xb30, 0xb4d, 0xb26, 0xb47,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb15, 0xb47, 0xb2a, 0xb4d, 0x200c, 0x20, 0xb2d, 0xb30, 0xb4d, 0xb26, 0xb47, 0x20,
0xb16, 0xb30, 0xb3e, 0xb26, 0xb3f, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3e,
0xb2e, 0xb4b, 0xb30, 0xb4b, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3e, 0xb25, 0xb3e, 0xb2e, 0xb4d, 0x200c, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3e, 0xb25, 0xb3e, 0xb2e, 0xb4d, 0x200c, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3e,
0xb25, 0xb3e, 0xb2e, 0xb4d, 0x200c, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3f, 0xb32, 0xb3f, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb1a, 0xb3f, 0xb32, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb3f, 0xb32, 0xb3f, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb1a, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb40, 0xb28, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a, 0xb40,
0xb28, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb1a, 0xb4b, 0xb07, 0xb2c, 0xb3e, 0xb32, 0xb38, 0xb3e, 0xb28, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb1a, 0xb4b, 0xb07, 0xb2c, 0xb3e, 0xb32, 0xb38, 0xb3e, 0xb28, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1a,
0xb4b, 0xb07, 0xb2c, 0xb3e, 0xb32, 0xb38, 0xb3e, 0xb28, 0x20, 0xb17, 0xb4d, 0xb30,
0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb16,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb1f, 0xb2e, 0xb3e, 0xb38, 0x20, 0xb26, 0xb4d,
0xb71, 0xb40, 0xb2a, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4b, 0xb15, 0xb4b, 0xb38,
0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d,
0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb06,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb06, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15,
0xb32, 0xb2e, 0xb4d, 0xb2c, 0xb3f, 0xb06, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb41, 0xb15,
0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d,
0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb41, 0xb15, 0xb4d, 0x200c, 0x20, 0xb26,
0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb41, 0xb15,
0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d,
0xb1c, 0x20, 0xb05, 0xb30, 0xb4d, 0xb26, 0xb4d, 0xb27, 0x20, 0xb17, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15,
0xb4d, 0xb5f, 0xb41, 0xb2c, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4d, 0xb5f,
0xb41, 0xb2c, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4d, 0xb5f, 0xb41, 0xb2c, 0xb3e, 0x20, 0xb26, 0xb3f,
0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb21, 0xb47, 0xb2d,
0xb3f, 0xb38, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb21, 0xb41, 0xb2e, 0xb4b,
0xb23, 0xb4d, 0xb1f, 0x2d, 0xb21, 0xb3f, 0x2018, 0xb09, 0xb30, 0xb2d, 0xb3f, 0xb32,
0xb4d, 0xb32, 0xb47, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c,
0x20, 0xb24, 0xb3f, 0xb2e, 0xb4b, 0xb30, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb07, 0xb37, 0xb4d, 0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb06, 0xb07, 0xb32, 0xb4d,
0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb37, 0xb4d,
0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb06, 0xb07, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23,
0xb4d, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb07, 0xb37, 0xb4d, 0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb06, 0xb07,
0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb15,
0xb4d, 0xb71, 0xb47, 0xb21, 0xb30, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28,
0xb4d, 0xb26, 0xb4d, 0xb30, 0xb40, 0xb5f, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a,
0xb40, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d,
0xb30, 0xb40, 0xb5f, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15,
0xb47, 0xb28, 0xb4d, 0xb26, 0xb4d, 0xb30, 0xb40, 0xb5f, 0x20, 0xb5f, 0xb41, 0xb30,
0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e,
0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c,
0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40,
0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb1e,
0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f, 0xb41,
0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb30, 0xb2c, 0xb30,
0xb4d, 0xb24, 0xb4d, 0xb24, 0xb40, 0x2d, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0xb3e,
0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0xb3e, 0xb1e,
0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0xb3e, 0xb1e, 0xb4d,
0xb1a, 0xb33, 0x20, 0xb5f, 0xb41, 0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36,
0xb4d, 0xb1a, 0xb3f, 0xb2e, 0xb3e, 0xb1e, 0xb4d, 0xb1a, 0xb33, 0x20, 0xb5f, 0xb41,
0xb30, 0xb4b, 0xb2a, 0xb40, 0xb5f, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb15, 0xb32, 0xb4d,
0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a,
0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb15, 0xb32, 0xb4d,
0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a,
0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb15, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d,
0xb21, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb1c, 0xb3f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b,
0xb3f, 0xb1c, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb1c, 0xb3f, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb4d,
0xb30, 0xb47, 0xb1e, 0xb4d, 0xb1a, 0x20, 0xb17, 0xb41, 0xb06, 0xb28, 0xb3e, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2b, 0xb4d, 0xb30, 0xb47, 0xb1e, 0xb4d, 0xb1a, 0x20, 0xb26,
0xb15, 0xb4d, 0xb37, 0xb3f, 0xb23, 0x20, 0xb0f, 0xb2c, 0xb02, 0x20, 0xb06, 0xb23,
0xb4d, 0xb1f, 0xb3e, 0xb30, 0xb4d, 0xb15, 0xb3e, 0xb1f, 0xb3f, 0xb15, 0xb4d, 0x200c,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb3e, 0xb32, 0xb3e, 0xb2a, 0xb3e, 0xb17, 0xb4b,
0xb38, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb3e, 0xb2e, 0xb4d, 0xb2c, 0xb3f,
0xb5f, 0xb30, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f,
0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f, 0xb06, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1c,
0xb30, 0xb4d, 0xb1c, 0xb3f, 0xb06, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26, 0xb3f, 0xb28,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb3f, 0xb32, 0xb2c, 0xb30, 0xb4d, 0xb1f, 0xb4d,
0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb28, 0xb71, 0xb3f, 0xb1a,
0xb4d, 0x20, 0xb2e, 0xb3f, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42,
0xb30, 0xb4d, 0xb2c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb28, 0xb32, 0xb4d, 0xb5f,
0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30,
0xb4d, 0xb2c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb28, 0xb32, 0xb4d, 0xb5f, 0xb3e,
0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb28, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb17, 0xb4d, 0xb30,
0xb40, 0xb28, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb17, 0xb4d, 0xb30,
0xb40, 0xb28, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36,
0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb28, 0xb32, 0xb4d,
0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb32, 0xb4d, 0xb2b, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb17, 0xb41,
0xb0f, 0xb28, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39, 0xb71, 0xb3e, 0xb07, 0x2d,
0xb06, 0xb32, 0xb47, 0xb09, 0xb1f, 0xb3f, 0xb5f, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39,
0xb71, 0xb3e, 0xb07, 0x2d, 0xb06, 0xb32, 0xb47, 0xb09, 0xb1f, 0xb3f, 0xb5f, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39,
0xb71, 0xb3e, 0xb07, 0x2d, 0xb06, 0xb32, 0xb47, 0xb09, 0xb1f, 0xb3f, 0xb5f, 0x20,
0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39,
0xb02, 0x20, 0xb15, 0xb02, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39, 0xb02, 0x20, 0xb15,
0xb02, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb39, 0xb02, 0x20, 0xb15, 0xb02, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39, 0xb4b, 0xb2d,
0xb21, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb39, 0xb4b, 0xb2d, 0xb21, 0xb4d,
0x200c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb39, 0xb4b, 0xb2d, 0xb21, 0xb4d, 0x200c, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb3e,
0xb30, 0xb24, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2d, 0xb3e, 0xb30, 0xb24, 0x20, 0xb2e, 0xb3e, 0xb39, 0xb3e, 0xb38,
0xb3e, 0xb17, 0xb30, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb4b,
0xb1a, 0xb3e, 0xb07, 0xb28, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb27, 0xb4d,
0xb5f, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb4b, 0xb28, 0xb47, 0xb38, 0xb3f, 0xb06,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb07, 0xb23,
0xb4d, 0xb21, 0xb4b, 0xb28, 0xb47, 0xb38, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb36, 0xb4d, 0xb1a, 0xb3f, 0xb2e, 0x20, 0xb07, 0xb23, 0xb4d, 0xb21, 0xb4b,
0xb28, 0xb47, 0xb38, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb30, 0xb3e,
0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb30, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb30, 0xb3e,
0xb28, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb07, 0xb05, 0xb30, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb07, 0xb05, 0xb30, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07,
0xb30, 0xb4d, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15, 0xb4d, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0xb40, 0xb28, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb07, 0xb38, 0xb4d, 0xb30, 0xb3e, 0xb0f, 0xb32, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb07, 0xb38, 0xb4d, 0xb30, 0xb3e, 0xb0f, 0xb32, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb07, 0xb38, 0xb4d, 0xb30,
0xb3e, 0xb0f, 0xb32, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb1c, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb1c, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1c, 0xb3e, 0xb2a, 0xb3e, 0xb28, 0x20, 0xb26,
0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb47,
0xb1f, 0xb4d, 0xb30, 0xb4b, 0xb2a, 0xb3e, 0xb2d, 0xb32, 0xb4b, 0xb2d, 0xb4d, 0xb38,
0xb15, 0x2d, 0xb15, 0xb2e, 0xb1a, 0xb1f, 0xb38, 0xb4d, 0xb15, 0xb3f, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2a, 0xb47, 0xb1f, 0xb4d, 0xb30, 0xb4b, 0xb2a, 0xb3e, 0xb2d, 0xb32,
0xb4b, 0xb2d, 0xb4d, 0xb38, 0xb15, 0x2d, 0xb15, 0xb2e, 0xb1a, 0xb1f, 0xb38, 0xb4d,
0xb15, 0xb3f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb47, 0xb1f, 0xb4d, 0xb30, 0xb4b, 0xb2a, 0xb3e, 0xb2d, 0xb32, 0xb4b, 0xb2d, 0xb4d,
0xb38, 0xb15, 0x2d, 0xb15, 0xb2e, 0xb1a, 0xb1f, 0xb38, 0xb4d, 0xb15, 0xb3f, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb42, 0xb30, 0xb4d, 0xb2c, 0x20, 0xb15, 0xb3e, 0xb1c, 0xb3e, 0xb15, 0xb38, 0xb4d,
0xb24, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb36, 0xb4d, 0xb1a,
0xb3f, 0xb2e, 0x20, 0xb15, 0xb3e, 0xb1c, 0xb3e, 0xb15, 0xb38, 0xb4d, 0xb24, 0xb3e,
0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4b, 0xb30, 0xb3f, 0xb5f, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb15, 0xb4b, 0xb30, 0xb3f, 0xb5f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4b, 0xb30, 0xb3f, 0xb5f,
0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb15, 0xb4b, 0xb38, 0xb30, 0xb47, 0xb07, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4d,
0xb30, 0xb3e, 0xb38, 0xb28, 0xb4b, 0xb5f, 0xb3e, 0xb30, 0xb38, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb28, 0xb4b, 0xb5f, 0xb3e,
0xb30, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb4d, 0xb30, 0xb3e, 0xb38, 0xb28, 0xb4b, 0xb5f,
0xb3e, 0xb30, 0xb38, 0xb4d, 0xb15, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb3f, 0xb30, 0xb4d,
0xb17, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb32,
0xb19, 0xb4d, 0xb15, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb32, 0xb3e, 0xb07, 0xb28,
0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d,
0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20, 0xb39, 0xb4b,
0xb71, 0xb47, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20, 0xb39,
0xb4b, 0xb71, 0xb47, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb32, 0xb30, 0xb4d, 0xb21, 0x20, 0xb39, 0xb4b, 0xb71, 0xb47,
0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2e, 0xb3e, 0xb15, 0xb3e, 0xb13, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb15, 0xb3e,
0xb13, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb15,
0xb3e, 0xb13, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb17, 0xb3e, 0xb21, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb17, 0xb3e, 0xb21, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e,
0xb17, 0xb3e, 0xb21, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb32,
0xb47, 0xb38, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb33, 0xb26,
0xb4d, 0xb71, 0xb40, 0xb2a, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb30, 0xb4d,
0xb15, 0xb4d, 0xb71, 0xb47, 0xb38, 0xb3e, 0xb38, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0xb36, 0xb3e, 0xb32, 0xb4d, 0x200c, 0x20, 0xb26,
0xb4d, 0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2e, 0xb4c, 0xb30, 0xb3f, 0xb38, 0xb38, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2e, 0xb4c, 0xb30, 0xb3f, 0xb38, 0xb38, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb4c, 0xb30,
0xb3f, 0xb38, 0xb38, 0xb4d, 0x200c, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26, 0xb3f, 0xb28,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3e, 0xb71, 0xb38, 0xb28, 0xb4d, 0x200c, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb47, 0xb15, 0xb4d, 0xb38, 0xb3f, 0xb15, 0xb3e, 0xb28,
0xb4d, 0x20, 0xb2a, 0xb3e, 0xb38, 0xb3f, 0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2e, 0xb47, 0xb15, 0xb4d, 0xb38, 0xb3f, 0xb15, 0xb3e, 0xb28, 0xb4d,
0x20, 0xb2a, 0xb3e, 0xb38, 0xb3f, 0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb47, 0xb15,
0xb4d, 0xb38, 0xb3f, 0xb15, 0xb3e, 0xb28, 0xb4d, 0x20, 0xb2a, 0xb3e, 0xb38, 0xb3f,
0xb2b, 0xb3f, 0xb15, 0xb4d, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb32, 0xb3e, 0xb28, 0xb4d, 0x200c, 0xb2c, 0xb3e,
0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb32, 0xb3e, 0xb28,
0xb4d, 0x200c, 0xb2c, 0xb3e, 0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb32, 0xb3e, 0xb28,
0xb4d, 0x200c, 0x20, 0xb2c, 0xb3e, 0xb1f, 0xb30, 0xb4d, 0x200c, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2e, 0xb38, 0xb4d, 0xb15, 0xb4b, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb38, 0xb4d,
0xb15, 0xb4b, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2e, 0xb38, 0xb4d, 0xb15, 0xb4b, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2e, 0xb3f,
0xb06, 0xb01, 0xb2e, 0xb3e, 0xb30, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28,
0xb3e, 0xb09, 0xb30, 0xb41, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb47, 0xb2a, 0xb3e,
0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb15, 0xb3e,
0xb32, 0xb47, 0xb21, 0xb4b, 0xb28, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28,
0xb4d, 0xb5f, 0xb41, 0x20, 0xb15, 0xb3e, 0xb32, 0xb47, 0xb21, 0xb4b, 0xb28, 0xb3f,
0xb06, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0x20, 0xb15, 0xb3e, 0xb32, 0xb47, 0xb21, 0xb4b,
0xb28, 0xb3f, 0xb06, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15,
0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb1c, 0xb3f,
0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f,
0xb41, 0xb1c, 0xb3f, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41,
0xb1c, 0xb3f, 0xb32, 0xb3e, 0xb23, 0xb4d, 0xb21, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e,
0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb2b,
0xb3e, 0xb09, 0xb23, 0xb4d, 0xb21, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21,
0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4d, 0xb5f, 0xb41, 0xb2b, 0xb3e, 0xb09,
0xb23, 0xb4d, 0xb21, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20,
0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28,
0xb4d, 0xb5f, 0xb41, 0xb2b, 0xb3e, 0xb09, 0xb23, 0xb4d, 0xb21, 0xb32, 0xb4d, 0xb5f,
0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4d, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb3f, 0xb5f, 0xb41, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb28, 0xb30, 0xb2b, 0xb4b, 0xb15, 0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71,
0xb40, 0xb2a, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb30, 0xb2b, 0xb4b, 0xb15, 0xb4d,
0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb30, 0xb2b, 0xb4b, 0xb15,
0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a, 0x20, 0xb17, 0xb4d, 0xb30,
0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2b,
0xb30, 0xb4d, 0xb23, 0xb4d, 0xb23, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4b, 0x20, 0xb21,
0xb3f, 0x20, 0xb28, 0xb4b, 0xb30, 0xb4b, 0xb28, 0xb4d, 0xb28, 0xb3e, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2b, 0xb30, 0xb4d, 0xb23, 0xb4d, 0xb23, 0xb3e, 0xb23, 0xb4d, 0xb21,
0xb4b, 0x20, 0xb21, 0xb3f, 0x20, 0xb28, 0xb4b, 0xb30, 0xb4b, 0xb28, 0xb4d, 0xb28,
0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2b, 0xb30, 0xb4d, 0xb23, 0xb4d, 0xb23, 0xb3e, 0xb23, 0xb4d, 0xb21, 0xb4b,
0x20, 0xb21, 0xb3f, 0x20, 0xb28, 0xb4b, 0xb30, 0xb4b, 0xb28, 0xb4d, 0xb28, 0xb3e,
0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb09, 0xb24, 0xb4d, 0xb24, 0xb30, 0x20, 0xb2e, 0xb47, 0xb30,
0xb3f, 0xb06, 0xb28, 0xb3e, 0x20, 0xb06, 0xb07, 0xb32, 0xb4d, 0xb5f, 0xb3e, 0xb23,
0xb4d, 0xb21, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4b, 0xb2d, 0xb4b, 0xb38, 0xb3f,
0xb2c, 0xb3f, 0xb30, 0xb38, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4b,
0xb2d, 0xb4b, 0xb38, 0xb3f, 0xb2c, 0xb3f, 0xb30, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb28, 0xb4b,
0xb2d, 0xb4b, 0xb38, 0xb3f, 0xb2c, 0xb3f, 0xb30, 0xb38, 0xb4d, 0xb15, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb13, 0xb2e, 0xb38, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb13, 0xb2e,
0xb38, 0xb4d, 0xb15, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb13, 0xb2e, 0xb38, 0xb4d, 0xb15, 0x20, 0xb17, 0xb4d, 0xb30,
0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a,
0xb3e, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb3e, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb15,
0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb32,
0xb3e, 0xb09, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb2a, 0xb41, 0xb06, 0x20, 0xb28,
0xb4d, 0xb5f, 0xb41, 0x20, 0xb17, 0xb41, 0xb28, 0xb3f, 0xb06, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2a, 0xb3e, 0xb30, 0xb3e, 0xb17, 0xb41, 0xb0f, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb3e, 0xb30, 0xb3e, 0xb17, 0xb41, 0xb0f, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e,
0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb3e, 0xb30, 0xb3e, 0xb17,
0xb41, 0xb0f, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e,
0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb47, 0xb30, 0xb41, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2a, 0xb47, 0xb30, 0xb41, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2a, 0xb47, 0xb30, 0xb41, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2b, 0xb3f, 0xb32, 0xb3f, 0xb2a, 0xb3e, 0xb07, 0xb28, 0xb4d, 0x200c, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2b, 0xb3f, 0xb32, 0xb3f, 0xb2a, 0xb3e, 0xb07, 0xb28, 0xb4d, 0x200c,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2b, 0xb3f, 0xb32, 0xb3f, 0xb2a, 0xb3e, 0xb07, 0xb28, 0xb4d, 0x200c, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2b, 0xb4b, 0xb28, 0xb3f, 0xb15, 0xb4d, 0xb38, 0x20, 0xb26, 0xb4d, 0xb71,
0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38,
0xb47, 0xb23, 0xb4d, 0xb1f, 0x2e, 0x20, 0xb2a, 0xb3f, 0xb0f, 0xb30, 0xb47, 0x20,
0xb0f, 0xb2c, 0xb02, 0x20, 0xb2e, 0xb3f, 0xb15, 0xb4d, 0xb5f, 0xb41, 0xb32, 0xb4b,
0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb47, 0xb23, 0xb4d, 0xb1f, 0x2e,
0x20, 0xb2a, 0xb3f, 0xb0f, 0xb30, 0xb47, 0x20, 0xb0f, 0xb2c, 0xb02, 0x20, 0xb2e,
0xb3f, 0xb15, 0xb4d, 0xb5f, 0xb41, 0xb32, 0xb4b, 0xb28, 0xb4d, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb47, 0xb23,
0xb4d, 0xb1f, 0x2e, 0x20, 0xb2a, 0xb3f, 0xb0f, 0xb30, 0xb47, 0x20, 0xb0f, 0xb2c,
0xb02, 0x20, 0xb2e, 0xb3f, 0xb15, 0xb4d, 0xb5f, 0xb41, 0xb32, 0xb4b, 0xb28, 0xb4d,
0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb3f, 0xb1f, 0xb15, 0xb3e, 0xb30, 0xb3f, 0xb28, 0xb4d, 0x200c, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb2a, 0xb4b, 0xb28, 0xb3e, 0xb2a, 0xb47, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2a, 0xb4b, 0xb5f, 0xb19, 0xb4d, 0xb17, 0xb5f, 0xb3e, 0xb19, 0xb4d, 0xb17, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb15, 0xb3e, 0xb1c, 0xb3f, 0xb32, 0xb4b, 0xb30, 0xb4d, 0xb21,
0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb15, 0xb3e, 0xb1c, 0xb3f, 0xb32, 0xb4b, 0xb30,
0xb4d, 0xb21, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb15, 0xb3e, 0xb1c, 0xb3f, 0xb32, 0xb4b, 0xb30, 0xb4d, 0xb21, 0xb3e, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb30, 0xb3f,
0xb5f, 0xb41, 0xb28, 0xb3f, 0xb05, 0xb28, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb30, 0xb4b, 0xb25, 0xb47, 0xb30, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb16,
0xb3e, 0xb32, 0xb3f, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb16, 0xb3e,
0xb32, 0xb3f, 0xb28, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb16, 0xb3e, 0xb32, 0xb3f, 0xb28, 0xb4d, 0x20,
0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb38, 0xb2e, 0xb3e, 0xb30, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38,
0xb2e, 0xb3e, 0xb30, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb38, 0xb2e, 0xb3e, 0xb30, 0xb3e, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb3e, 0xb2e, 0xb4b, 0xb06, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb38, 0xb3e, 0xb2e, 0xb4b, 0xb06, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb3e, 0xb2e, 0xb4b,
0xb06, 0x20, 0xb26, 0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb38, 0xb47, 0xb1a, 0xb47, 0xb32, 0xb4d, 0xb32, 0xb47, 0xb38, 0xb4d, 0x200c,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb3f, 0xb19, 0xb4d, 0xb17, 0xb3e, 0xb2a, 0xb41,
0xb30, 0xb4d, 0x200c, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb38, 0xb4b, 0xb32, 0xb4b, 0xb2e, 0xb28, 0x20, 0xb26, 0xb4d,
0xb71, 0xb40, 0xb2a, 0xb2a, 0xb41, 0xb1e, 0xb4d, 0xb1c, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb26, 0xb15, 0xb4d, 0xb37, 0xb3f, 0xb23, 0x20, 0xb1c, 0xb30, 0xb4d, 0xb1c, 0xb3f,
0xb06, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb41, 0xb30, 0xb3f, 0xb28, 0xb47, 0xb2e,
0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb38, 0xb4b, 0xb71, 0xb3e, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb24, 0xb3e, 0xb39, 0xb3f, 0xb24, 0xb3f, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb24, 0xb3e, 0xb07, 0xb2a, 0xb47, 0xb07, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb3e,
0xb07, 0xb2a, 0xb47, 0xb07, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb3e, 0xb07, 0xb2a, 0xb47, 0xb07, 0x20, 0xb26,
0xb3f, 0xb2c, 0xb3e, 0xb32, 0xb4b, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb3e,
0xb1c, 0xb3f, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb1f, 0xb4b, 0xb15, 0xb47, 0xb32, 0xb3e, 0xb09, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb1f, 0xb4b, 0xb19, 0xb4d, 0xb17, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb1f, 0xb4b,
0xb19, 0xb4d, 0xb17, 0xb3e, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb1f, 0xb4b, 0xb19, 0xb4d, 0xb17, 0xb3e, 0x20, 0xb17,
0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb1a, 0xb42, 0xb15, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb41,
0xb30, 0xb4d, 0xb15, 0xb2e, 0xb47, 0xb28, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb41, 0xb30, 0xb4d, 0xb15, 0xb2e, 0xb47, 0xb28,
0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19,
0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb41, 0xb30, 0xb4d, 0xb15, 0xb2e,
0xb47, 0xb28, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb16, 0xb30, 0xb3e,
0xb26, 0xb3f, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb24, 0xb41, 0xb2d, 0xb3e, 0xb32,
0xb41, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb30, 0xb41, 0xb17, 0xb41, 0xb0f, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb09, 0xb30, 0xb41, 0xb17, 0xb41, 0xb0f, 0x20, 0xb2e, 0xb3e,
0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb30, 0xb41,
0xb17, 0xb41, 0xb0f, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15,
0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb1c, 0xb2c, 0xb47, 0xb15, 0xb3f,
0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb1c, 0xb2c,
0xb47, 0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb2e, 0xb3e, 0xb28,
0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb09, 0xb1c, 0xb2c, 0xb47,
0xb15, 0xb3f, 0xb38, 0xb4d, 0xb24, 0xb3e, 0xb28, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26,
0xb3f, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb3e, 0xb28, 0xb41, 0xb06, 0xb1f,
0xb41, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb3e, 0xb28, 0xb41, 0xb06, 0xb1f, 0xb41,
0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2d, 0xb3e, 0xb28, 0xb41, 0xb06, 0xb1f, 0xb41, 0x20, 0xb16, 0xb30, 0xb3e, 0xb26,
0xb3f, 0xb28, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb47, 0xb28, 0xb3f, 0xb1c, 0xb41,
0xb0f, 0xb32, 0xb3e, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb4d, 0xb32, 0xb3e, 0xb21,
0xb3f, 0xb2d, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb2d, 0xb4d, 0xb32, 0xb3e, 0xb21, 0xb3f, 0xb2d, 0xb4b, 0xb37, 0xb4d, 0xb1f,
0xb4b, 0xb15, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb4d, 0xb32, 0xb3e, 0xb21, 0xb3f, 0xb2d, 0xb4b, 0xb37,
0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37, 0xb4d,
0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb4b, 0xb32, 0xb17,
0xb4b, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d,
0xb4b, 0xb32, 0xb17, 0xb4b, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb2d, 0xb4b,
0xb32, 0xb17, 0xb4b, 0xb17, 0xb4d, 0xb30, 0xb3e, 0xb21, 0xb4d, 0x20, 0xb17, 0xb4d,
0xb30, 0xb40, 0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f,
0xb2d, 0xb4b, 0xb37, 0xb4d, 0xb1f, 0xb4b, 0xb15, 0xb4d, 0x200c, 0x20, 0xb38, 0xb2e,
0xb5f, 0xb71, 0xb47, 0xb15, 0xb4d, 0x200c, 0x20, 0xb26, 0xb4d, 0xb71, 0xb40, 0xb2a,
0x20, 0xb38, 0xb2e, 0xb5f, 0xb71, 0xb3e, 0xb32, 0xb3f, 0xb38, 0xb4d, 0x200c, 0x20,
0xb0f, 0xb2c, 0xb02, 0x20, 0xb2b, 0xb41, 0xb1f, 0xb41, 0xb28, 0xb3e, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb5f, 0xb3e, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15, 0x20, 0xb38,
0xb2e, 0xb5f, 0xb5f, 0xb3e, 0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15, 0x20, 0xb2e,
0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d, 0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb3e,
0xb15, 0xb41, 0xb1f, 0xb38, 0xb4d, 0xb15, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40, 0xb37,
0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb47, 0xb15,
0xb3e, 0xb1f, 0xb47, 0xb30, 0xb3f, 0xb28, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb4d, 0x20,
0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb47, 0xb15, 0xb3e, 0xb1f, 0xb47, 0xb30, 0xb3f, 0xb28,
0xb2c, 0xb30, 0xb4d, 0xb17, 0xb4d, 0x20, 0xb2e, 0xb3e, 0xb28, 0xb3e, 0xb19, 0xb4d,
0xb15, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb47, 0xb15, 0xb3e, 0xb1f, 0xb47, 0xb30,
0xb3f, 0xb28, 0xb2c, 0xb30, 0xb4d, 0xb17, 0xb4d, 0x20, 0xb17, 0xb4d, 0xb30, 0xb40,
0xb37, 0xb4d, 0xb2e, 0xb15, 0xb3e, 0xb33, 0x20, 0xb38, 0xb2e, 0xb5f, 0xb5f, 0xb41,
0xb15, 0xb4b, 0xb28, 0xb4d, 0x20, 0xb38, 0xb2e, 0xb5f, 0x410, 0x441, 0x442, 0x4d5,
0x443, 0x43a, 0x43a, 0x430, 0x433, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5,
0x439, 0x430, 0x433, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x410, 0x441,
0x442, 0x4d5, 0x443, 0x43a, 0x43a, 0x430, 0x433, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x410,
0x441, 0x442, 0x4d5, 0x443, 0x43a, 0x43a, 0x430, 0x433, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20, 0x441, 0x4d5, 0x440, 0x434, 0x44b,
0x433, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x421, 0x43a,
0x4d5, 0x441, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5, 0x439,
0x430, 0x433, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x421, 0x43a, 0x4d5,
0x441, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5, 0x439, 0x430,
0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43e, 0x43d,
0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x421, 0x43a, 0x4d5, 0x441, 0x4d5,
0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20,
0x441, 0x4d5, 0x440, 0x434, 0x44b, 0x433, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441,
0x442, 0x4d5, 0x433, 0x41d, 0x44b, 0x433, 0x44a, 0x443, 0x44b, 0x43b, 0x4d5, 0x43d,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5, 0x439, 0x430, 0x433, 0x20, 0x440,
0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x41d, 0x44b, 0x433, 0x44a, 0x443, 0x44b, 0x43b,
0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5, 0x439, 0x430, 0x433,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43e, 0x43d, 0x20,
0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x41d, 0x44b, 0x433, 0x44a, 0x443, 0x44b,
0x43b, 0x4d5, 0x43d, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x4d5, 0x439, 0x430,
0x433, 0x20, 0x441, 0x4d5, 0x440, 0x434, 0x44b, 0x433, 0x43e, 0x43d, 0x20, 0x440,
0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x413, 0x443, 0x44b, 0x440, 0x434, 0x437, 0x44b,
0x441, 0x442, 0x43e, 0x43d, 0x44b, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433,
0x413, 0x443, 0x44b, 0x440, 0x434, 0x437, 0x44b, 0x441, 0x442, 0x43e, 0x43d, 0x44b,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43e, 0x43d, 0x20,
0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x413, 0x443, 0x44b, 0x440, 0x434, 0x437,
0x44b, 0x441, 0x442, 0x43e, 0x43d, 0x44b, 0x20, 0x441, 0x4d5, 0x440, 0x434, 0x44b,
0x433, 0x43e, 0x43d, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x413, 0x440,
0x438, 0x43d, 0x432, 0x438, 0x447, 0x44b, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5,
0x43c, 0x431, 0x438, 0x441, 0x20, 0x440, 0x4d5, 0x441, 0x442, 0x4d5, 0x433, 0x41c,
0x4d5, 0x441, 0x43a, 0x443, 0x44b, 0x439, 0x44b, 0x20, 0x440, 0x4d5, 0x441, 0x442,
0x4d5, 0x433, 0x41c, 0x4d5, 0x441, 0x43a, 0x443, 0x44b, 0x439, 0x44b, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43e, 0x43d, 0x20, 0x440, 0x4d5,
0x441, 0x442, 0x4d5, 0x433, 0x41c, 0x4d5, 0x441, 0x43a, 0x443, 0x44b, 0x439, 0x44b,
0x20, 0x441, 0x4d5, 0x440, 0x434, 0x44b, 0x433, 0x43e, 0x43d, 0x20, 0x440, 0x4d5,
0x441, 0x442, 0x4d5, 0x433, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x646, 0x681, 0x646, 0x6cc, 0x20, 0x627,
0x641, 0x631, 0x64a, 0x642, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a,
0x681, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x642, 0x627, 0x20, 0x648, 0x62e, 0x62a,
0x62c, 0x646, 0x648, 0x628, 0x64a, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x642, 0x627,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644,
0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627, 0x641, 0x631, 0x64a, 0x642, 0x627,
0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x627,
0x641, 0x631, 0x6cc, 0x642, 0x627, 0x6cc, 0x64a, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x641, 0x631, 0x6cc,
0x642, 0x627, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627, 0x20, 0x644, 0x648,
0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x644, 0x627, 0x633,
0x6a9, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x644, 0x627, 0x633, 0x6a9, 0x627,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0,
0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x6cc, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x6cc,
0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x62e, 0x62a, 0x627, 0x6cc, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9,
0x632, 0x64a, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648,
0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x6cc, 0x681,
0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681, 0x20,
0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648,
0x62e, 0x62a, 0x62f, 0x20, 0x63a, 0x631, 0x647, 0x20, 0x62f, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x63a, 0x631, 0x647, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x63a, 0x631, 0x647, 0x20, 0x62f,
0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e,
0x62a, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20,
0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x62f, 0x20,
0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a,
0x627, 0x67e, 0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x67e, 0x6cc, 0x627,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x67e, 0x64a, 0x627, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631,
0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x639, 0x631, 0x628,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x639, 0x631, 0x628, 0x64a, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x639, 0x631, 0x628, 0x64a,
0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627,
0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67c, 0x627,
0x6cc, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67c, 0x627,
0x6cc, 0x646, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e,
0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67c, 0x627, 0x6cc, 0x646, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681,
0x20, 0x627, 0x631, 0x62c, 0x646, 0x67c, 0x627, 0x64a, 0x646, 0x20, 0x648, 0x62e,
0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627, 0x631, 0x62c, 0x646,
0x67c, 0x627, 0x64a, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627, 0x631,
0x62c, 0x646, 0x67c, 0x627, 0x64a, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x631, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x631, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631,
0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x62a,
0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627,
0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9,
0x632, 0x64a, 0x20, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a, 0x627, 0x20,
0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a, 0x20,
0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a,
0x20, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681,
0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a,
0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a, 0x20, 0x645, 0x631, 0x6a9,
0x632, 0x64a, 0x20, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x648, 0x62e,
0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648, 0x64a, 0x20, 0x645, 0x631,
0x6a9, 0x632, 0x64a, 0x20, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x648, 0x64a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20,
0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681,
0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a,
0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x64a,
0x627, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648,
0x64a, 0x20, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x648,
0x64a, 0x20, 0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681,
0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a,
0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x622, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x64a, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x648, 0x64a, 0x20, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x648, 0x64a, 0x20, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681,
0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627,
0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x622, 0x630, 0x631, 0x628,
0x627, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x622, 0x630, 0x631,
0x628, 0x627, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x630, 0x631, 0x628, 0x627, 0x6cc, 0x6cc,
0x62c, 0x627, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x627, 0x64a, 0x632, 0x648, 0x631, 0x633, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a,
0x632, 0x648, 0x631, 0x633, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x64a, 0x632, 0x648, 0x631, 0x633, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x646, 0x6af, 0x644, 0x647, 0x20,
0x62f, 0x6d0, 0x634, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x646, 0x6ab, 0x644, 0x627,
0x62f, 0x6cc, 0x634, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x628, 0x646, 0x6ab, 0x644, 0x647, 0x20, 0x62f, 0x64a, 0x634, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x647, 0x648, 0x67c,
0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x648, 0x644, 0x6cc, 0x648, 0x6cc,
0x627, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x631, 0x633, 0x644, 0x6cc, 0x627, 0x20,
0x648, 0x62e, 0x62a, 0x628, 0x631, 0x633, 0x644, 0x6cc, 0x627, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x628, 0x631, 0x633, 0x644,
0x6cc, 0x627, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x628,
0x631, 0x648, 0x646, 0x627, 0x64a, 0x64a, 0x20, 0x62f, 0x627, 0x631, 0x627, 0x644,
0x633, 0x644, 0x627, 0x645, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x67e, 0x20,
0x648, 0x631, 0x62f, 0x689, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x67e, 0x20,
0x648, 0x631, 0x62f, 0x689, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x67e, 0x20, 0x648, 0x631, 0x62f, 0x689, 0x20,
0x633, 0x645, 0x631, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x645, 0x627, 0x631, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x686,
0x627, 0x62a, 0x627, 0x645, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x627, 0x62a, 0x627,
0x645, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x686, 0x627, 0x62a, 0x627, 0x645, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0,
0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x686,
0x644, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x644, 0x6cc, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x644, 0x6cc, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x64a, 0x646, 0x20,
0x648, 0x62e, 0x62a, 0x686, 0x6cc, 0x646, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x686, 0x64a, 0x646, 0x20, 0x62f,
0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e,
0x62a, 0x686, 0x648, 0x626, 0x6cc, 0x628, 0x6cc, 0x644, 0x633, 0x646, 0x20, 0x648,
0x62e, 0x62a, 0x686, 0x648, 0x626, 0x6cc, 0x628, 0x6cc, 0x644, 0x633, 0x646, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x686, 0x648,
0x626, 0x6cc, 0x628, 0x6cc, 0x644, 0x633, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x631, 0x64a, 0x633, 0x645, 0x633, 0x20, 0x67c,
0x627, 0x67e, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x6a9, 0x648, 0x632,
0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x648, 0x62e,
0x62a, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a,
0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc,
0x627, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648,
0x6a9, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x648,
0x62e, 0x62a, 0x6a9, 0x648, 0x6a9, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab, 0x627,
0x646, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x6a9, 0x648, 0x6a9, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab, 0x627, 0x646,
0x648, 0x20, 0x646, 0x64a, 0x645, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x6a9, 0x64a, 0x648, 0x628, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x6a9,
0x6cc, 0x648, 0x628, 0x627, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x648, 0x628, 0x627, 0x20, 0x62f, 0x20, 0x631,
0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x689,
0x64a, 0x648, 0x633, 0x20, 0x648, 0x62e, 0x62a, 0x689, 0x648, 0x645, 0x648, 0x646,
0x67c, 0x20, 0x689, 0x64a, 0x20, 0x627, 0x631, 0x648, 0x644, 0x62e, 0x62a, 0x64a,
0x681, 0x20, 0x62a, 0x64a, 0x645, 0x648, 0x631, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x64a, 0x633, 0x62a, 0x631, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x648, 0x62e,
0x62a, 0x627, 0x64a, 0x633, 0x62a, 0x631, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a,
0x633, 0x62a, 0x631, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x627, 0x648, 0x693,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x6a9, 0x648, 0x627, 0x62f,
0x648, 0x631, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20,
0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f,
0x20, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645,
0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x62a, 0x64a, 0x681,
0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62e,
0x62a, 0x64a, 0x681, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20,
0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x631, 0x6d0, 0x20,
0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627,
0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648,
0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a, 0x64a,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644,
0x648, 0x6d0, 0x62f, 0x64a, 0x681, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x64a,
0x64a, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x648,
0x6a9, 0x644, 0x646, 0x689, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x641, 0x648, 0x6a9, 0x644, 0x646, 0x689, 0x20, 0x67c, 0x627,
0x67e, 0x648, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x641, 0x648, 0x6a9, 0x644, 0x646, 0x689, 0x20, 0x67c, 0x627,
0x67e, 0x648, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x641,
0x62c, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x62c, 0x6cc, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x62c, 0x64a, 0x20,
0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20,
0x641, 0x631, 0x627, 0x646, 0x633, 0x648, 0x64a, 0x20, 0x6ab, 0x627, 0x646, 0x627,
0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x641, 0x631, 0x627, 0x646, 0x633, 0x6d0,
0x20, 0x633, 0x648, 0x6cc, 0x644, 0x20, 0x627, 0x648, 0x20, 0x627, 0x646, 0x67c,
0x627, 0x631, 0x67c, 0x64a, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x6ab, 0x627, 0x644,
0x67e, 0x6ab, 0x648, 0x633, 0x20, 0x648, 0x62e, 0x62a, 0x6ab, 0x64a, 0x645, 0x628,
0x64a, 0x631, 0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x648, 0x631, 0x62c, 0x6cc, 0x627,
0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x648, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x62c,
0x648, 0x631, 0x62c, 0x64a, 0x627, 0x20, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x644, 0x628, 0x631, 0x67c, 0x20, 0x67c, 0x627,
0x67e, 0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x6ab, 0x631,
0x64a, 0x646, 0x648, 0x64a, 0x686, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x62e, 0x62a, 0x6cc, 0x681, 0x20, 0x6ab,
0x631, 0x6cc, 0x646, 0x644, 0x6cc, 0x646, 0x689, 0x20, 0x648, 0x62e, 0x62a, 0x62f,
0x20, 0x62e, 0x62a, 0x6cc, 0x681, 0x20, 0x6ab, 0x631, 0x6cc, 0x646, 0x644, 0x6cc,
0x646, 0x689, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x62e, 0x62a, 0x6cc, 0x681, 0x20, 0x6ab, 0x631, 0x6cc, 0x646,
0x644, 0x6cc, 0x646, 0x689, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x6ab, 0x631, 0x6cc, 0x646,
0x644, 0x6cc, 0x646, 0x689, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6cc, 0x62f,
0x6cc, 0x681, 0x20, 0x6ab, 0x631, 0x6cc, 0x646, 0x644, 0x6cc, 0x646, 0x689, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648,
0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x6ab, 0x631, 0x6cc, 0x646, 0x644, 0x6cc, 0x646,
0x689, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62e, 0x644,
0x6cc, 0x62c, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x62f, 0x20, 0x6ab, 0x648, 0x627, 0x646, 0x627, 0x646, 0x627, 0x20, 0x648,
0x62e, 0x62a, 0x647, 0x648, 0x627, 0x6cc, 0x6cc, 0x20, 0x627, 0x644, 0x6cc, 0x648,
0x62a, 0x6cc, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x648, 0x627, 0x6cc, 0x6cc,
0x20, 0x627, 0x644, 0x6cc, 0x648, 0x62a, 0x6cc, 0x646, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x648, 0x627, 0x6cc, 0x6cc,
0x20, 0x627, 0x644, 0x6cc, 0x648, 0x62a, 0x6cc, 0x646, 0x20, 0x631, 0x6bc, 0x627,
0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x627, 0x646,
0x6ab, 0x20, 0x6a9, 0x627, 0x646, 0x6ab, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x627,
0x646, 0x6ab, 0x20, 0x6a9, 0x627, 0x646, 0x6ab, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x627, 0x646, 0x6ab, 0x20, 0x6a9,
0x627, 0x646, 0x6ab, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x647, 0x627, 0x648, 0x689, 0x20, 0x648, 0x62e, 0x62a, 0x647, 0x627, 0x648, 0x689,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x647,
0x627, 0x648, 0x689, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x647, 0x646, 0x62f, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x62f, 0x20, 0x647, 0x646, 0x62f, 0x20, 0x633, 0x645, 0x646, 0x62f,
0x631, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x646, 0x689, 0x648, 0x686, 0x627, 0x6cc,
0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20,
0x627, 0x62f, 0x648, 0x646, 0x64a, 0x632, 0x64a, 0x627, 0x20, 0x648, 0x62e, 0x62a,
0x627, 0x646, 0x62f, 0x648, 0x646, 0x6cc, 0x632, 0x6cc, 0x627, 0x20, 0x648, 0x62e,
0x62a, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc, 0x681, 0x20, 0x627, 0x646, 0x62f, 0x648,
0x646, 0x6cc, 0x632, 0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x6cc, 0x631,
0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x64a,
0x631, 0x627, 0x646, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631,
0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631, 0x6a9,
0x648, 0x67c, 0x633, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x631, 0x6a9, 0x648,
0x67c, 0x633, 0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x627, 0x631, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x633, 0x631, 0x627, 0x6cc, 0x6cc,
0x644, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x633, 0x631, 0x627, 0x6cc, 0x6cc, 0x644,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x633, 0x631, 0x627, 0x64a, 0x64a, 0x644, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681,
0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a,
0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x627, 0x67e,
0x627, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681,
0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a,
0x62e, 0x62a, 0x64a, 0x681, 0x20, 0x642, 0x627, 0x632, 0x642, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc, 0x681, 0x20,
0x642, 0x632, 0x627, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a,
0x6a9, 0x648, 0x631, 0x64a, 0x627, 0x64a, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9,
0x648, 0x631, 0x64a, 0x627, 0x64a, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x631, 0x64a, 0x627, 0x64a, 0x64a,
0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627,
0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x648, 0x633, 0x631, 0x627, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x631, 0x64a, 0x633, 0x646, 0x648, 0x64a, 0x627,
0x631, 0x633, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x631, 0x64a, 0x633, 0x646,
0x648, 0x64a, 0x627, 0x631, 0x633, 0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x631, 0x64a, 0x633, 0x646, 0x648, 0x64a,
0x627, 0x631, 0x633, 0x6a9, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x6a9, 0x631, 0x63a, 0x6cc, 0x632, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648,
0x62e, 0x62a, 0x644, 0x627, 0x64a, 0x646, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x6ab,
0x627, 0x646, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x627, 0x631, 0x689, 0x20,
0x647, 0x648, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x644, 0x627, 0x631, 0x689, 0x20,
0x647, 0x648, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x644, 0x627, 0x631, 0x689, 0x20, 0x647, 0x648, 0x64a, 0x20, 0x62f,
0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647,
0x20, 0x648, 0x62e, 0x62a, 0x645, 0x64a, 0x6ab, 0x689, 0x627, 0x646, 0x20, 0x648,
0x62e, 0x62a, 0x645, 0x64a, 0x6ab, 0x689, 0x627, 0x646, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x64a, 0x6ab, 0x689, 0x627,
0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x644,
0x627, 0x626, 0x6cc, 0x634, 0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627,
0x644, 0x62f, 0x64a, 0x67e, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x631, 0x6a9,
0x633, 0x633, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x631, 0x634, 0x644, 0x20,
0x67c, 0x627, 0x67e, 0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x648, 0x62e, 0x62a,
0x645, 0x627, 0x631, 0x64a, 0x634, 0x64a, 0x633, 0x20, 0x648, 0x62e, 0x62a, 0x645,
0x627, 0x631, 0x6cc, 0x634, 0x6cc, 0x633, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x631, 0x64a, 0x634, 0x64a, 0x633,
0x20, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645,
0x627, 0x648, 0x633, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x6a9, 0x633, 0x6cc,
0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x648, 0x62e, 0x62a,
0x645, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x645,
0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20,
0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x648, 0x62e, 0x62a,
0x627, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x67c, 0x631, 0x20, 0x648,
0x62e, 0x62a, 0x627, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x67c, 0x631,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627,
0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x67c, 0x631, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20,
0x648, 0x62e, 0x62a, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x627, 0x633, 0x6a9, 0x648,
0x20, 0x633, 0x645, 0x631, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x6cc, 0x627, 0x646,
0x645, 0x627, 0x631, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x627, 0x648, 0x631, 0x648,
0x20, 0x648, 0x62e, 0x62a, 0x646, 0x6cc, 0x67e, 0x627, 0x644, 0x20, 0x648, 0x62e,
0x62a, 0x646, 0x6cc, 0x648, 0x20, 0x6a9, 0x627, 0x644, 0x6cc, 0x689, 0x648, 0x646,
0x6cc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x6a9, 0x627,
0x644, 0x6cc, 0x689, 0x648, 0x646, 0x6cc, 0x627, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x6a9, 0x627,
0x64a, 0x689, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x20, 0x644, 0x6d0, 0x646,
0x689, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x20, 0x644,
0x6d0, 0x646, 0x689, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x20, 0x644, 0x6d0, 0x646, 0x689,
0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627,
0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x64a, 0x648, 0x20, 0x641, 0x627,
0x648, 0x646, 0x689, 0x644, 0x6d0, 0x646, 0x689, 0x20, 0x648, 0x62e, 0x62a, 0x62f,
0x20, 0x646, 0x648, 0x64a, 0x20, 0x641, 0x6cc, 0x644, 0x689, 0x644, 0x6cc, 0x646,
0x689, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a,
0x646, 0x64a, 0x648, 0x20, 0x641, 0x627, 0x648, 0x646, 0x689, 0x644, 0x6d0, 0x646,
0x689, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a,
0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x6cc, 0x6cc, 0x648, 0x20,
0x648, 0x62e, 0x62a, 0x646, 0x648, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x67c, 0x627,
0x67e, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x646, 0x648, 0x631, 0x641,
0x6a9, 0x627, 0x633, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x646, 0x648, 0x631,
0x641, 0x6a9, 0x627, 0x633, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688,
0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627,
0x20, 0x648, 0x62e, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20,
0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x631, 0x646,
0x627, 0x646, 0x688, 0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648,
0x646, 0x6be, 0x627, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x646, 0x648, 0x648, 0x633, 0x64a, 0x628, 0x631, 0x633, 0x6a9, 0x20, 0x648, 0x62e,
0x62a, 0x646, 0x648, 0x648, 0x633, 0x64a, 0x628, 0x631, 0x633, 0x6a9, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x646, 0x648, 0x648,
0x633, 0x64a, 0x628, 0x631, 0x633, 0x6a9, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x627, 0x648, 0x645, 0x633, 0x6a9, 0x20, 0x648, 0x62e, 0x62a,
0x627, 0x648, 0x645, 0x633, 0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x627, 0x648, 0x645, 0x633, 0x6a9, 0x20, 0x627, 0x648,
0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x67e,
0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x67e, 0x627, 0x644, 0x627, 0x648, 0x20, 0x648, 0x62e, 0x62a,
0x67e, 0x627, 0x67e, 0x648, 0x627, 0x20, 0x646, 0x6cc, 0x648, 0x20, 0x6ab, 0x646,
0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x64a, 0x631, 0x627, 0x6ab, 0x648, 0x627,
0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x6cc, 0x631, 0x627, 0x6ab, 0x648, 0x627,
0x6cc, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x67e, 0x627, 0x631, 0x627, 0x6ab, 0x648, 0x627, 0x6cc, 0x20, 0x627, 0x648, 0x693,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x648, 0x62e,
0x62a, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x627, 0x648, 0x693,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x644, 0x67e, 0x627, 0x64a, 0x646, 0x20,
0x648, 0x62e, 0x62a, 0x641, 0x644, 0x67e, 0x627, 0x64a, 0x646, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x641, 0x644, 0x67e, 0x627,
0x64a, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f,
0x20, 0x641, 0x6cc, 0x646, 0x6a9, 0x633, 0x20, 0x67c, 0x627, 0x67e, 0x648, 0x20,
0x648, 0x62e, 0x62a, 0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x67e, 0x6cc, 0x6cc, 0x631,
0x627, 0x20, 0x648, 0x20, 0x645, 0x64a, 0x6a9, 0x644, 0x64a, 0x646, 0x20, 0x648,
0x62e, 0x62a, 0x633, 0x6cc, 0x646, 0x67c, 0x20, 0x67e, 0x6cc, 0x6cc, 0x631, 0x627,
0x20, 0x648, 0x20, 0x645, 0x64a, 0x6a9, 0x644, 0x64a, 0x646, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x6cc, 0x646, 0x67c,
0x20, 0x67e, 0x6cc, 0x6cc, 0x631, 0x627, 0x20, 0x648, 0x20, 0x645, 0x64a, 0x6a9,
0x644, 0x64a, 0x646, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d0,
0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x6cc, 0x67c, 0x20, 0x6a9, 0x627, 0x631, 0x6cc,
0x646, 0x20, 0x648, 0x62e, 0x62a, 0x67e, 0x648, 0x646, 0x64a, 0x67e, 0x20, 0x648,
0x62e, 0x62a, 0x67e, 0x64a, 0x627, 0x646, 0x6ab, 0x20, 0x64a, 0x627, 0x646, 0x6ab,
0x20, 0x648, 0x62e, 0x62a, 0x631, 0x64a, 0x20, 0x64a, 0x648, 0x646, 0x64a, 0x646,
0x20, 0x648, 0x62e, 0x62a, 0x631, 0x648, 0x62f, 0x631, 0x627, 0x20, 0x648, 0x62e,
0x62a, 0x633, 0x62e, 0x644, 0x64a, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x62e,
0x644, 0x6cc, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x62e, 0x62a, 0x633, 0x62e, 0x644, 0x64a, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x633, 0x645, 0x648, 0x627, 0x20, 0x648, 0x62e, 0x62a,
0x633, 0x645, 0x648, 0x627, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x633, 0x645, 0x648, 0x627, 0x20, 0x62f, 0x20, 0x648, 0x631,
0x681, 0x6d0, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e,
0x62a, 0x633, 0x6cc, 0x686, 0x6cc, 0x644, 0x633, 0x20, 0x648, 0x62e, 0x62a, 0x633,
0x646, 0x6ab, 0x627, 0x67e, 0x648, 0x631, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x644, 0x6cc, 0x645, 0x627, 0x646, 0x20,
0x67c, 0x627, 0x67e, 0x648, 0x6ab, 0x627, 0x646, 0x648, 0x20, 0x648, 0x62e, 0x62a,
0x62f, 0x20, 0x633, 0x648, 0x6cc, 0x644, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x6cc,
0x627, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x648, 0x631, 0x6cc, 0x646, 0x627, 0x645,
0x20, 0x648, 0x62e, 0x62a, 0x633, 0x6cc, 0x648, 0x627, 0x20, 0x648, 0x62e, 0x62a,
0x67c, 0x647, 0x64a, 0x67c, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x627, 0x64a,
0x67e, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x627, 0x64a, 0x67e, 0x64a, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x627,
0x64a, 0x67e, 0x64a, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d0, 0x20, 0x631,
0x648, 0x69a, 0x627, 0x646, 0x647, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x627, 0x62c,
0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x648, 0x6a9,
0x6cc, 0x644, 0x627, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x67c, 0x648, 0x646, 0x6ab,
0x627, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x67c, 0x648, 0x646, 0x6ab, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x67c, 0x648,
0x646, 0x6ab, 0x627, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a,
0x686, 0x648, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x631, 0x6a9, 0x645, 0x627,
0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x631, 0x6a9,
0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x631, 0x6a9, 0x645, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62a,
0x648, 0x648, 0x627, 0x644, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x6cc, 0x648, 0x631,
0x648, 0x6ab, 0x648, 0x627, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x6cc, 0x648, 0x631,
0x648, 0x6ab, 0x648, 0x627, 0x6cc, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x6cc, 0x648, 0x631, 0x648, 0x6ab, 0x648, 0x627, 0x6cc,
0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x632, 0x628,
0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x632, 0x628,
0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x648,
0x646, 0x627, 0x62a, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x646, 0x648, 0x627,
0x62a, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x648, 0x646, 0x648, 0x627, 0x62a, 0x648, 0x20, 0x627, 0x648, 0x693, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x648, 0x6cc, 0x646, 0x632, 0x648, 0x6cc, 0x644, 0x627,
0x20, 0x648, 0x62e, 0x62a, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x648, 0x633, 0x62a,
0x627, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x644, 0x627, 0x62f, 0x64a, 0x648,
0x633, 0x62a, 0x627, 0x6a9, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x62e, 0x62a, 0x648, 0x644, 0x627, 0x62f, 0x64a, 0x648, 0x633, 0x62a, 0x627,
0x6a9, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x627,
0x644, 0x6ab, 0x648, 0x6ab, 0x631, 0x627, 0x62f, 0x20, 0x648, 0x62e, 0x62a, 0x648,
0x627, 0x644, 0x6ab, 0x648, 0x6ab, 0x631, 0x627, 0x62f, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x648, 0x627, 0x644, 0x6ab, 0x648,
0x6ab, 0x631, 0x627, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e,
0x62a, 0x648, 0x627, 0x633, 0x62a, 0x648, 0x6a9, 0x20, 0x648, 0x62e, 0x62a, 0x648,
0x64a, 0x6a9, 0x20, 0x62a, 0x627, 0x67e, 0x648, 0x20, 0x648, 0x62e, 0x62a, 0x648,
0x627, 0x644, 0x633, 0x20, 0x627, 0x648, 0x20, 0x641, 0x648, 0x62a, 0x648, 0x646,
0x627, 0x20, 0x648, 0x62e, 0x62a, 0x64a, 0x627, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9,
0x20, 0x648, 0x62e, 0x62a, 0x64a, 0x627, 0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x64a, 0x627,
0x6a9, 0x648, 0x67c, 0x633, 0x6a9, 0x20, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x6cc, 0x6a9, 0x627, 0x67c, 0x6cc, 0x631, 0x6cc, 0x646,
0x628, 0x631, 0x6ab, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x64a, 0x627, 0x6a9,
0x64a, 0x67c, 0x631, 0x646, 0x628, 0x631, 0x6ab, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x64a, 0x627, 0x6a9, 0x67c,
0x631, 0x646, 0x628, 0x631, 0x6ab, 0x20, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a,
0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x6cc, 0x648, 0x6a9, 0x648, 0x646, 0x20,
0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20,
0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646,
0x647, 0x20, 0x6a9, 0x648, 0x644, 0x645, 0x631, 0x6a9, 0x632, 0x64a, 0x20, 0x631,
0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62e,
0x62a, 0x64a, 0x681, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631,
0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x63a, 0x631, 0x647, 0x20,
0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648,
0x62e, 0x62a, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x62f, 0x20, 0x631, 0x6bc,
0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20,
0x627, 0x67e, 0x6cc, 0x627, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20,
0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x639, 0x631, 0x628, 0x64a, 0x20, 0x648, 0x631,
0x681, 0x67e, 0x627, 0x6bc, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x627, 0x62a, 0x644,
0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20,
0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x633,
0x67c, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x20, 0x644, 0x648, 0x6cc, 0x62f, 0x6cc,
0x681, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a,
0x62f, 0x20, 0x686, 0x64a, 0x646, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20,
0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x6a9, 0x6cc, 0x648, 0x628,
0x627, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2,
0x20, 0x648, 0x62e, 0x62a, 0x644, 0x648, 0x6d0, 0x62f, 0x64a, 0x632, 0x6d2, 0x20,
0x627, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x644,
0x648, 0x62f, 0x6cc, 0x681, 0x6d2, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20,
0x644, 0x648, 0x62f, 0x6cc, 0x681, 0x6d2, 0x20, 0x627, 0x648, 0x631, 0x67e, 0x627,
0x20, 0x62f, 0x20, 0x627, 0x648, 0x693, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x62f,
0x20, 0x641, 0x631, 0x627, 0x646, 0x633, 0x6d2, 0x20, 0x633, 0x648, 0x6cc, 0x644,
0x20, 0x627, 0x648, 0x20, 0x627, 0x646, 0x67c, 0x627, 0x631, 0x67c, 0x64a, 0x6a9,
0x20, 0x648, 0x62e, 0x62a, 0x647, 0x648, 0x627, 0x6cc, 0x6cc, 0x20, 0x627, 0x644,
0x6cc, 0x648, 0x62a, 0x6cc, 0x646, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631,
0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x627, 0x6cc, 0x631, 0x627,
0x646, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a,
0x62f, 0x20, 0x627, 0x633, 0x631, 0x627, 0x6cc, 0x6cc, 0x644, 0x648, 0x20, 0x62f,
0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62c, 0x627, 0x67e,
0x627, 0x646, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681,
0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x6a9, 0x648, 0x631, 0x6cc, 0x627,
0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x62f, 0x20, 0x648, 0x631,
0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x6a9, 0x631, 0x69a, 0x6d2,
0x20, 0x67c, 0x627, 0x67c, 0x648, 0x628, 0x64a, 0x20, 0x648, 0x62e, 0x62a, 0x631,
0x628, 0x20, 0x647, 0x627, 0x648, 0x20, 0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2,
0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x62e, 0x62a, 0x645, 0x6a9,
0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x631,
0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x62f,
0x20, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20,
0x62f, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20, 0x62f, 0x20, 0x631, 0x6bc, 0x627,
0x20, 0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x646, 0x648, 0x64a, 0x20, 0x641, 0x6cc,
0x644, 0x689, 0x644, 0x6cc, 0x646, 0x689, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648,
0x631, 0x681, 0x6d2, 0x20, 0x648, 0x62e, 0x62a, 0x633, 0x6cc, 0x646, 0x67c, 0x20,
0x67e, 0x6cc, 0x6cc, 0x631, 0x627, 0x20, 0x648, 0x20, 0x645, 0x64a, 0x6a9, 0x644,
0x64a, 0x646, 0x20, 0x631, 0x6bc, 0x627, 0x20, 0x648, 0x631, 0x681, 0x6d2, 0x20,
0x648, 0x62e, 0x62a, 0x62f, 0x20, 0x633, 0x645, 0x648, 0x627, 0x20, 0x62f, 0x20,
0x648, 0x631, 0x681, 0x6d2, 0x20, 0x631, 0x648, 0x69a, 0x627, 0x646, 0x647, 0x20,
0x6a9, 0x648, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x641, 0x63a, 0x627, 0x646,
0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632,
0x20, 0x622, 0x641, 0x631, 0x6cc, 0x642, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x634,
0x631, 0x642, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x20, 0x627,
0x641, 0x631, 0x6cc, 0x642, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x641, 0x631, 0x6cc,
0x642, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x622, 0x644, 0x627, 0x633,
0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x622, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x622, 0x644, 0x645, 0x627, 0x622, 0x62a, 0x627, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x622, 0x644, 0x645, 0x627, 0x622, 0x62a, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x622, 0x644, 0x645, 0x627, 0x622, 0x62a, 0x627, 0x648, 0x642, 0x62a, 0x20,
0x622, 0x645, 0x627, 0x632, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x622, 0x645, 0x627, 0x632, 0x648, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x622, 0x645,
0x627, 0x632, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632,
0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x645, 0x631,
0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x645, 0x631,
0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627,
0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x634, 0x631, 0x642, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x6a9, 0x648, 0x647, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x6a9, 0x648, 0x647, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6a9, 0x648, 0x647, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20,
0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x646,
0x627, 0x62f, 0x6cc, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x622, 0x646, 0x627, 0x62f, 0x6cc, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x622, 0x646, 0x627, 0x62f,
0x6cc, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x67e, 0x6cc, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x622, 0x67e, 0x6cc, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x622, 0x67e, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x631, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x639, 0x631, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x639, 0x631, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x631, 0x698, 0x627, 0x646,
0x62a, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x622, 0x631, 0x698, 0x627, 0x646, 0x62a, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x622, 0x631, 0x698,
0x627, 0x646, 0x62a, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x622, 0x631, 0x698, 0x627, 0x646, 0x62a, 0x6cc, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x622, 0x631,
0x698, 0x627, 0x646, 0x62a, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x622,
0x631, 0x698, 0x627, 0x646, 0x62a, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x627,
0x631, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x631, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x627, 0x631, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x622, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x622, 0x62a, 0x644, 0x627, 0x646, 0x62a,
0x6cc, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x622, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x6cc, 0x6a9, 0x648,
0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644,
0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632,
0x2d, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x645, 0x631,
0x6a9, 0x632, 0x2d, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x2d, 0x63a, 0x631, 0x628,
0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x634, 0x631, 0x642, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x634, 0x631,
0x642, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x634,
0x631, 0x642, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x645, 0x647, 0x648, 0x631, 0x6cc, 0x20,
0x622, 0x630, 0x631, 0x628, 0x627, 0x6cc, 0x62c, 0x627, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62c, 0x645, 0x647, 0x648, 0x631, 0x6cc,
0x20, 0x622, 0x630, 0x631, 0x628, 0x627, 0x6cc, 0x62c, 0x627, 0x646, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x62c,
0x645, 0x647, 0x648, 0x631, 0x6cc, 0x20, 0x622, 0x630, 0x631, 0x628, 0x627, 0x6cc,
0x62c, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x622, 0x632, 0x648, 0x631, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x622, 0x632, 0x648, 0x631,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x622, 0x632, 0x648, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x628, 0x646, 0x6af,
0x644, 0x627, 0x62f, 0x634, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x628, 0x646, 0x6af, 0x644, 0x627, 0x62f, 0x634, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x628, 0x646, 0x6af,
0x644, 0x627, 0x62f, 0x634, 0x648, 0x642, 0x62a, 0x20, 0x628, 0x648, 0x62a, 0x627,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x628, 0x648, 0x644, 0x6cc, 0x648, 0x6cc, 0x648,
0x642, 0x62a, 0x20, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x6cc, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x628, 0x631, 0x627, 0x632,
0x6cc, 0x644, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x6cc,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x628, 0x631, 0x648, 0x646, 0x626, 0x6cc, 0x20,
0x62f, 0x627, 0x631, 0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x648, 0x642, 0x62a,
0x20, 0x6a9, 0x6cc, 0x67e, 0x200c, 0x648, 0x631, 0x62f, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6a9, 0x6cc, 0x67e, 0x200c, 0x648, 0x631, 0x62f,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x6a9, 0x6cc, 0x67e, 0x200c, 0x648, 0x631, 0x62f, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x686, 0x627, 0x645, 0x648, 0x631, 0x648, 0x648,
0x642, 0x62a, 0x20, 0x686, 0x62a, 0x200c, 0x647, 0x627, 0x645, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x686, 0x62a, 0x200c, 0x647, 0x627, 0x645,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x686, 0x62a, 0x200c, 0x647, 0x627, 0x645, 0x648, 0x642, 0x62a, 0x20, 0x634,
0x6cc, 0x644, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x634, 0x6cc, 0x644, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x634, 0x6cc, 0x644, 0x6cc, 0x648, 0x642, 0x62a,
0x20, 0x686, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x686, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x686, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20,
0x686, 0x648, 0x6cc, 0x628, 0x627, 0x644, 0x633, 0x627, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x686, 0x648, 0x6cc, 0x628, 0x627, 0x644,
0x633, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x686, 0x648, 0x6cc, 0x628, 0x627, 0x644, 0x633, 0x627,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20,
0x6a9, 0x631, 0x6cc, 0x633, 0x645, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632,
0x627, 0x6cc, 0x631, 0x20, 0x6a9, 0x648, 0x6a9, 0x648, 0x633, 0x648, 0x642, 0x62a,
0x20, 0x6a9, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x6a9, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6a9,
0x644, 0x645, 0x628, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x6a9, 0x648, 0x6a9, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x6a9, 0x648, 0x6a9, 0x648, 0x642, 0x62a,
0x20, 0x6a9, 0x648, 0x628, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x6a9, 0x648, 0x628, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6a9, 0x648, 0x628, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x62f, 0x6cc, 0x648, 0x6cc, 0x633, 0x648, 0x642, 0x62a, 0x20,
0x62f, 0x648, 0x645, 0x648, 0x646, 0x20, 0x62f, 0x648, 0x631, 0x648, 0x6cc, 0x644,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x6cc, 0x645, 0x648, 0x631, 0x20, 0x634, 0x631,
0x642, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654,
0x20, 0x627, 0x6cc, 0x633, 0x62a, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x627, 0x6cc,
0x633, 0x62a, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x627,
0x6cc, 0x633, 0x62a, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x6a9, 0x648, 0x627,
0x62f, 0x648, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20,
0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x634, 0x631, 0x642,
0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627,
0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645, 0x6a9, 0x627, 0x646, 0x200c, 0x647, 0x627,
0x6cc, 0x20, 0x62f, 0x6cc, 0x6af, 0x631, 0x20, 0x634, 0x631, 0x642, 0x20, 0x627,
0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20,
0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x63a, 0x631, 0x628, 0x20, 0x627, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x641, 0x627, 0x644, 0x6a9, 0x644,
0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62c,
0x632, 0x627, 0x6cc, 0x631, 0x20, 0x641, 0x627, 0x644, 0x6a9, 0x644, 0x646, 0x62f,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x641, 0x627, 0x644, 0x6a9, 0x644,
0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x641, 0x6cc, 0x62c, 0x6cc, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x641, 0x6cc, 0x62c, 0x6cc, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x641, 0x6cc, 0x62c, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x6af, 0x648, 0x6cc, 0x627,
0x646, 0x20, 0x641, 0x631, 0x627, 0x646, 0x633, 0x647, 0x648, 0x642, 0x62a, 0x20,
0x633, 0x631, 0x632, 0x645, 0x6cc, 0x646, 0x200c, 0x647, 0x627, 0x6cc, 0x20, 0x62c,
0x646, 0x648, 0x628, 0x6cc, 0x20, 0x648, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x6af,
0x627, 0x646, 0x20, 0x641, 0x631, 0x627, 0x646, 0x633, 0x647, 0x648, 0x642, 0x62a,
0x20, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x633, 0x648, 0x642,
0x62a, 0x20, 0x6af, 0x627, 0x645, 0x628, 0x6cc, 0x647, 0x648, 0x642, 0x62a, 0x20,
0x6af, 0x631, 0x62c, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x6af, 0x631, 0x62c, 0x633, 0x62a, 0x627, 0x646, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x6af, 0x631, 0x62c, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62c,
0x632, 0x627, 0x6cc, 0x631, 0x20, 0x6af, 0x6cc, 0x644, 0x628, 0x631, 0x62a, 0x648,
0x642, 0x62a, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x648, 0x6cc, 0x686, 0x648, 0x642,
0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x644, 0x646,
0x62f, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x634, 0x631,
0x642, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x634, 0x631,
0x642, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a,
0x20, 0x63a, 0x631, 0x628, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x644, 0x646, 0x62f,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x63a, 0x631, 0x628,
0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6af, 0x648, 0x627, 0x645, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62e, 0x644, 0x6cc, 0x62c, 0x20, 0x641,
0x627, 0x631, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x647, 0x627, 0x648, 0x627, 0x6cc,
0x6cc, 0x2010, 0x627, 0x644, 0x648, 0x634, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x647, 0x627, 0x648, 0x627, 0x6cc, 0x6cc, 0x2010, 0x627,
0x644, 0x648, 0x634, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x647, 0x627, 0x648, 0x627, 0x6cc, 0x6cc, 0x2010,
0x627, 0x644, 0x648, 0x634, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x647, 0x646, 0x6af,
0x200c, 0x6a9, 0x646, 0x6af, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc,
0x20, 0x647, 0x646, 0x6af, 0x200c, 0x6a9, 0x646, 0x6af, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x647, 0x646, 0x6af,
0x200c, 0x6a9, 0x646, 0x6af, 0x648, 0x642, 0x62a, 0x20, 0x62e, 0x648, 0x648, 0x62f,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62e, 0x648, 0x648,
0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x62e, 0x648, 0x648, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x647, 0x646,
0x62f, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x642, 0x6cc, 0x627, 0x646, 0x648, 0x633,
0x20, 0x647, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x647, 0x646, 0x62f, 0x648,
0x686, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x631, 0x6a9, 0x632, 0x20,
0x627, 0x646, 0x62f, 0x648, 0x646, 0x632, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x634,
0x631, 0x642, 0x20, 0x627, 0x646, 0x62f, 0x648, 0x646, 0x632, 0x6cc, 0x648, 0x642,
0x62a, 0x20, 0x63a, 0x631, 0x628, 0x20, 0x627, 0x646, 0x62f, 0x648, 0x646, 0x632,
0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x6cc, 0x631, 0x627, 0x646,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x6cc,
0x631, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x627, 0x6cc, 0x631, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x627, 0x6cc, 0x631, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20,
0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x698,
0x627, 0x67e, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x698, 0x627, 0x67e, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x698, 0x627, 0x67e, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x67e, 0x62a, 0x631, 0x648, 0x67e, 0x627, 0x648, 0x644, 0x648, 0x633, 0x6a9,
0x2010, 0x6a9, 0x627, 0x645, 0x686, 0x627, 0x62a, 0x633, 0x6a9, 0x6cc, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x67e, 0x62a, 0x631, 0x648, 0x67e,
0x627, 0x648, 0x644, 0x648, 0x633, 0x6a9, 0x2010, 0x6a9, 0x627, 0x645, 0x686, 0x627,
0x62a, 0x633, 0x6a9, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x67e, 0x62a, 0x631, 0x648, 0x67e, 0x627, 0x648,
0x644, 0x648, 0x633, 0x6a9, 0x2010, 0x6a9, 0x627, 0x645, 0x686, 0x627, 0x62a, 0x633,
0x6a9, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x634, 0x631, 0x642, 0x20, 0x642, 0x632,
0x627, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x63a, 0x631,
0x628, 0x20, 0x642, 0x632, 0x627, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642,
0x62a, 0x20, 0x6a9, 0x631, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x6a9, 0x631, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6a9, 0x631, 0x647, 0x648, 0x642, 0x62a,
0x20, 0x6a9, 0x648, 0x633, 0x631, 0x627, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x6a9,
0x631, 0x627, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6a9, 0x631, 0x627, 0x633, 0x646,
0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6a9, 0x631, 0x627, 0x633, 0x646,
0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x642, 0x631,
0x642, 0x6cc, 0x632, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x644,
0x627, 0x646, 0x6a9, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc,
0x631, 0x20, 0x644, 0x627, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x644, 0x631,
0x62f, 0x647, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x644, 0x631, 0x62f, 0x647, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x644, 0x631, 0x62f, 0x647, 0x648, 0x648,
0x642, 0x62a, 0x20, 0x645, 0x627, 0x6a9, 0x627, 0x626, 0x648, 0x648, 0x642, 0x62a,
0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x645, 0x627, 0x6a9, 0x627, 0x626, 0x648,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x645, 0x627, 0x6a9, 0x627, 0x626, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x645,
0x627, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x645, 0x627, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645,
0x627, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x627,
0x644, 0x632, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x627, 0x644, 0x62f, 0x6cc,
0x648, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x627, 0x631, 0x6a9, 0x648, 0x626, 0x632,
0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x645, 0x627,
0x631, 0x634, 0x627, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x648, 0x631, 0x6cc,
0x633, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x645, 0x648,
0x631, 0x6cc, 0x633, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x645, 0x648, 0x631, 0x6cc, 0x633, 0x648, 0x642, 0x62a,
0x20, 0x645, 0x627, 0x648, 0x633, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x634,
0x631, 0x642, 0x20, 0x645, 0x6a9, 0x632, 0x6cc, 0x6a9, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20, 0x645, 0x6a9, 0x632,
0x6cc, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x634, 0x631, 0x642, 0x20, 0x645, 0x6a9, 0x632, 0x6cc, 0x6a9,
0x648, 0x642, 0x62a, 0x20, 0x627, 0x648, 0x644, 0x627, 0x646, 0x200c, 0x628, 0x627,
0x62a, 0x648, 0x631, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x627, 0x648, 0x644, 0x627, 0x646, 0x200c, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x627, 0x648, 0x644, 0x627, 0x646, 0x200c, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x648,
0x642, 0x62a, 0x20, 0x645, 0x633, 0x6a9, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x639,
0x627, 0x62f, 0x6cc, 0x20, 0x645, 0x633, 0x6a9, 0x648, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x645, 0x633, 0x6a9,
0x648, 0x648, 0x642, 0x62a, 0x20, 0x645, 0x6cc, 0x627, 0x646, 0x645, 0x627, 0x631,
0x648, 0x642, 0x62a, 0x20, 0x646, 0x627, 0x626, 0x648, 0x631, 0x648, 0x648, 0x642,
0x62a, 0x20, 0x646, 0x67e, 0x627, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x6a9, 0x627,
0x644, 0x62f, 0x648, 0x646, 0x6cc, 0x627, 0x6cc, 0x20, 0x62c, 0x62f, 0x6cc, 0x62f,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6a9, 0x627, 0x644,
0x62f, 0x648, 0x646, 0x6cc, 0x627, 0x6cc, 0x20, 0x62c, 0x62f, 0x6cc, 0x62f, 0x648,
0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20,
0x6a9, 0x627, 0x644, 0x62f, 0x648, 0x646, 0x6cc, 0x627, 0x6cc, 0x20, 0x62c, 0x62f,
0x6cc, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x644,
0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x646,
0x6cc, 0x648, 0x632, 0x6cc, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x646, 0x6cc, 0x648, 0x632,
0x6cc, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x646, 0x6cc, 0x648, 0x641,
0x627, 0x646, 0x62f, 0x644, 0x646, 0x62f, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x646, 0x6cc, 0x648, 0x641, 0x627, 0x646, 0x62f, 0x644, 0x646,
0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x646, 0x6cc, 0x648, 0x641, 0x627, 0x646, 0x62f, 0x644, 0x646, 0x62f,
0x648, 0x642, 0x62a, 0x20, 0x646, 0x6cc, 0x648, 0x626, 0x647, 0x648, 0x642, 0x62a,
0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x646, 0x648, 0x631, 0x641,
0x648, 0x644, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x646, 0x648, 0x631, 0x641, 0x648,
0x644, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x62c, 0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x646, 0x648,
0x631, 0x641, 0x648, 0x644, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x641, 0x631, 0x646,
0x627, 0x646, 0x62f, 0x648, 0x20, 0x62f, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648,
0x646, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x641, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x648, 0x20, 0x62f, 0x6cc, 0x20, 0x646,
0x648, 0x631, 0x648, 0x646, 0x6cc, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x641, 0x631, 0x646, 0x627, 0x646,
0x62f, 0x648, 0x20, 0x62f, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x6cc,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x645,
0x627, 0x631, 0x6cc, 0x627, 0x646, 0x627, 0x6cc, 0x20, 0x634, 0x645, 0x627, 0x644,
0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x646, 0x648, 0x648, 0x633, 0x6cc, 0x628, 0x6cc,
0x631, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x646, 0x648, 0x648, 0x633, 0x6cc, 0x628, 0x6cc, 0x631, 0x633, 0x6a9, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x646,
0x648, 0x648, 0x633, 0x6cc, 0x628, 0x6cc, 0x631, 0x633, 0x6a9, 0x648, 0x642, 0x62a,
0x20, 0x627, 0x648, 0x645, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627,
0x62f, 0x6cc, 0x20, 0x627, 0x648, 0x645, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x648, 0x645,
0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x67e, 0x627,
0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627,
0x646, 0x648, 0x642, 0x62a, 0x20, 0x67e, 0x627, 0x644, 0x627, 0x626, 0x648, 0x648,
0x642, 0x62a, 0x20, 0x67e, 0x627, 0x67e, 0x648, 0x627, 0x20, 0x6af, 0x6cc, 0x646,
0x647, 0x654, 0x20, 0x646, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x67e, 0x627, 0x631,
0x627, 0x6af, 0x648, 0x626, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x67e, 0x627, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x647, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x67e,
0x627, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x67e,
0x631, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x67e,
0x631, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x67e, 0x631, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x641, 0x6cc,
0x644, 0x6cc, 0x67e, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x641, 0x6cc, 0x644, 0x6cc, 0x67e, 0x6cc, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x641, 0x6cc,
0x644, 0x6cc, 0x67e, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627,
0x6cc, 0x631, 0x20, 0x641, 0x648, 0x646, 0x6cc, 0x6a9, 0x633, 0x648, 0x642, 0x62a,
0x20, 0x633, 0x646, 0x62a, 0x200c, 0x67e, 0x6cc, 0x631, 0x20, 0x648, 0x20, 0x645,
0x6cc, 0x6a9, 0x644, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x633, 0x646, 0x62a, 0x200c, 0x67e, 0x6cc, 0x631, 0x20, 0x648, 0x20,
0x645, 0x6cc, 0x6a9, 0x644, 0x648, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x633, 0x646, 0x62a, 0x200c, 0x67e,
0x6cc, 0x631, 0x20, 0x648, 0x20, 0x645, 0x6cc, 0x6a9, 0x644, 0x648, 0x646, 0x648,
0x642, 0x62a, 0x20, 0x67e, 0x6cc, 0x62a, 0x6a9, 0x627, 0x6cc, 0x631, 0x646, 0x648,
0x642, 0x62a, 0x20, 0x67e, 0x648, 0x646, 0x627, 0x67e, 0x647, 0x648, 0x642, 0x62a,
0x20, 0x67e, 0x6cc, 0x648, 0x646, 0x6af, 0x200c, 0x6cc, 0x627, 0x646, 0x6af, 0x648,
0x642, 0x62a, 0x20, 0x642, 0x6cc, 0x632, 0x6cc, 0x644, 0x200c, 0x627, 0x648, 0x631,
0x62f, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x642,
0x6cc, 0x632, 0x6cc, 0x644, 0x200c, 0x627, 0x648, 0x631, 0x62f, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x642,
0x6cc, 0x632, 0x6cc, 0x644, 0x200c, 0x627, 0x648, 0x631, 0x62f, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x631, 0x626, 0x648, 0x646, 0x6cc, 0x648, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x631, 0x648, 0x62a, 0x631, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x627,
0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x633, 0x627, 0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x633, 0x627,
0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x627, 0x645,
0x627, 0x631, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20,
0x633, 0x627, 0x645, 0x627, 0x631, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x633, 0x627, 0x645, 0x627, 0x631,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x627, 0x645, 0x648, 0x627, 0x648, 0x642,
0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x633, 0x627, 0x645, 0x648, 0x627,
0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc,
0x20, 0x633, 0x627, 0x645, 0x648, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x6cc,
0x634, 0x644, 0x648, 0x642, 0x62a, 0x20, 0x633, 0x646, 0x6af, 0x627, 0x67e, 0x648,
0x631, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x632, 0x627, 0x6cc, 0x631, 0x20, 0x633,
0x644, 0x6cc, 0x645, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62c, 0x648, 0x631,
0x62c, 0x6cc, 0x627, 0x6cc, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x648, 0x642,
0x62a, 0x20, 0x633, 0x648, 0x631, 0x6cc, 0x646, 0x627, 0x645, 0x648, 0x642, 0x62a,
0x20, 0x634, 0x648, 0x648, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x647,
0x6cc, 0x62a, 0x6cc, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x6cc, 0x67e, 0x647,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62a, 0x627, 0x6cc,
0x67e, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627,
0x646, 0x6cc, 0x20, 0x62a, 0x627, 0x6cc, 0x67e, 0x647, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x62c, 0x6cc, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a,
0x20, 0x62a, 0x648, 0x6a9, 0x644, 0x627, 0x626, 0x648, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x648, 0x646, 0x6af, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x62a,
0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x62a, 0x648, 0x646, 0x6af,
0x627, 0x648, 0x642, 0x62a, 0x20, 0x686, 0x648, 0x626, 0x648, 0x6a9, 0x648, 0x642,
0x62a, 0x20, 0x62a, 0x631, 0x6a9, 0x645, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x648,
0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x62a, 0x631, 0x6a9, 0x645,
0x646, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x62a, 0x631, 0x6a9, 0x645, 0x646, 0x633,
0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x648, 0x648, 0x627, 0x644,
0x648, 0x648, 0x642, 0x62a, 0x20, 0x627, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x647,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x631, 0x648,
0x6af, 0x648, 0x626, 0x647, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x647,
0x648, 0x642, 0x62a, 0x20, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x627, 0x632, 0x628,
0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628,
0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a,
0x627, 0x646, 0x648, 0x642, 0x62a, 0x20, 0x648, 0x627, 0x646, 0x627, 0x62a, 0x648,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x627, 0x646,
0x627, 0x62a, 0x648, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x648, 0x627, 0x646, 0x627, 0x62a, 0x648, 0x648, 0x642,
0x62a, 0x20, 0x648, 0x646, 0x632, 0x648, 0x626, 0x644, 0x627, 0x648, 0x642, 0x62a,
0x20, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x200c, 0x648, 0x633, 0x62a, 0x648, 0x6a9,
0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x644, 0x627,
0x62f, 0x6cc, 0x200c, 0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20,
0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x648, 0x644, 0x627,
0x62f, 0x6cc, 0x200c, 0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20,
0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x648, 0x642, 0x62a, 0x20,
0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631, 0x627,
0x62f, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x648, 0x642,
0x62a, 0x20, 0x648, 0x633, 0x62a, 0x648, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62c,
0x632, 0x6cc, 0x631, 0x647, 0x654, 0x20, 0x648, 0x6cc, 0x6a9, 0x648, 0x642, 0x62a,
0x20, 0x648, 0x627, 0x644, 0x6cc, 0x633, 0x20, 0x648, 0x20, 0x641, 0x648, 0x62a,
0x648, 0x646, 0x627, 0x648, 0x642, 0x62a, 0x20, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a,
0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f, 0x6cc, 0x20, 0x6cc,
0x627, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627,
0x628, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a,
0x633, 0x6a9, 0x648, 0x642, 0x62a, 0x20, 0x6cc, 0x6a9, 0x627, 0x62a, 0x631, 0x6cc,
0x646, 0x628, 0x648, 0x631, 0x6af, 0x648, 0x642, 0x62a, 0x20, 0x639, 0x627, 0x62f,
0x6cc, 0x20, 0x6cc, 0x6a9, 0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x648, 0x631,
0x6af, 0x648, 0x642, 0x62a, 0x20, 0x62a, 0x627, 0x628, 0x633, 0x62a, 0x627, 0x646,
0x6cc, 0x20, 0x6cc, 0x6a9, 0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x648, 0x631,
0x6af, 0x648, 0x642, 0x62a, 0x20, 0x6cc, 0x648, 0x6a9, 0x627, 0x646, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f,
0x77, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x6f, 0x142, 0x75, 0x64, 0x6e, 0x69,
0x6f, 0x77, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x142, 0x6d, 0x61, 0x63,
0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x142, 0x6d, 0x61, 0x63,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x142, 0x6d, 0x61, 0x63,
0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x6d, 0x61, 0x7a, 0x6f, 0x144, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144,
0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b,
0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x67, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x67, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x67, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63, 0x79, 0x66,
0x69, 0x63, 0x7a, 0x6e, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61,
0x63, 0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x41, 0x70, 0x69, 0x61, 0x41, 0x70, 0x69, 0x61, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x41, 0x70, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x61, 0x75, 0x6b, 0x74, 0x61, 0x75, 0x63, 0x6b, 0x69, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x61, 0x75, 0x6b, 0x74, 0x61, 0x75, 0x63, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x75, 0x6b, 0x74, 0x61, 0x75,
0x63, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x61, 0x6b, 0x74, 0x69, 0x75, 0x62, 0x69, 0x144, 0x73, 0x6b,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x6b, 0x74, 0x69, 0x75, 0x62,
0x69, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x6b,
0x74, 0x69, 0x75, 0x62, 0x69, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0xf3, 0x142, 0x77,
0x79, 0x73, 0x65, 0x70, 0x20, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69,
0x50, 0xf3, 0x142, 0x77, 0x79, 0x73, 0x65, 0x70, 0x20, 0x41, 0x72, 0x61,
0x62, 0x73, 0x6b, 0x69, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x50,
0xf3, 0x142, 0x77, 0x79, 0x73, 0x65, 0x70, 0x20, 0x41, 0x72, 0x61, 0x62,
0x73, 0x6b, 0x69, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x79, 0x6e, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x5a,
0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x79, 0x63, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x79, 0x63, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x79, 0x63, 0x6b,
0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f,
0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x2d, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64,
0x6b, 0x6f, 0x77, 0x6f, 0x2d, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b,
0x6f, 0x77, 0x6f, 0x2d, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17c, 0x61, 0x6e, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17c, 0x61, 0x6e, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a,
0x64, 0x17c, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x79, 0x41, 0x7a, 0x6f, 0x72, 0x79, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x41, 0x7a, 0x6f, 0x72, 0x79, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x7a,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x7a, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x7a, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x42, 0x68, 0x75,
0x74, 0x61, 0x6e, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x42, 0x6f, 0x6c, 0x69,
0x77, 0x69, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x42, 0x72, 0x61, 0x73,
0xed, 0x6c, 0x69, 0x61, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x42, 0x72, 0x61, 0x73, 0xed,
0x6c, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70,
0x79, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x67, 0x6f, 0x20,
0x50, 0x72, 0x7a, 0x79, 0x6c, 0x105, 0x64, 0x6b, 0x61, 0x57, 0x79, 0x73,
0x70, 0x79, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x67, 0x6f,
0x20, 0x50, 0x72, 0x7a, 0x79, 0x6c, 0x105, 0x64, 0x6b, 0x61, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x5a,
0x69, 0x65, 0x6c, 0x6f, 0x6e, 0x65, 0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a,
0x79, 0x6c, 0x105, 0x64, 0x6b, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x43, 0x7a, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x79, 0x43, 0x68, 0x69, 0x6e,
0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x43, 0x68, 0x69, 0x6e,
0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x43, 0x7a, 0x6f, 0x6a, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x43, 0x7a, 0x6f, 0x6a, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x43, 0x7a,
0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x42, 0x6f, 0x17c, 0x65,
0x67, 0x6f, 0x20, 0x4e, 0x61, 0x72, 0x6f, 0x64, 0x7a, 0x65, 0x6e, 0x69,
0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20,
0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x77, 0x65, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x61, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x61,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x57, 0x79, 0x73, 0x70, 0x79,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x4b, 0x75, 0x62, 0x61, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x69, 0x6d, 0x6f,
0x72, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x57, 0x69, 0x65,
0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x61, 0x57, 0x79, 0x73, 0x70,
0x61, 0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x57, 0x79, 0x73, 0x70,
0x61, 0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x45, 0x6b, 0x77, 0x61, 0x64,
0x6f, 0x72, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b,
0x6f, 0x77, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f,
0x77, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x15b, 0x72, 0x6f, 0x64, 0x6b, 0x6f, 0x77,
0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x64, 0x61, 0x6c, 0x73, 0x7a, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x7a,
0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x7a,
0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x7a,
0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x79, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x46, 0x69, 0x64,
0x17c, 0x69, 0x46, 0x69, 0x64, 0x17c, 0x69, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x29, 0x46, 0x69, 0x64, 0x17c, 0x69, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x47, 0x75, 0x6a, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e,
0x63, 0x75, 0x73, 0x6b, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x69, 0x65, 0x20, 0x54, 0x65, 0x72,
0x79, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x20, 0x50, 0x6f, 0x142, 0x75, 0x64,
0x6e, 0x69, 0x6f, 0x77, 0x65, 0x20, 0x69, 0x20, 0x41, 0x6e, 0x74, 0x61,
0x72, 0x6b, 0x74, 0x79, 0x63, 0x7a, 0x6e, 0x65, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x47, 0x72,
0x75, 0x7a, 0x6a, 0x61, 0x47, 0x72, 0x75, 0x7a, 0x6a, 0x61, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x47, 0x72, 0x75, 0x7a, 0x6a, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x47,
0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x75, 0x6e, 0x69, 0x77, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x6e, 0x79, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61,
0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x57,
0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x61, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x5a, 0x61, 0x74, 0x6f, 0x6b, 0x61,
0x20, 0x50, 0x65, 0x72, 0x73, 0x6b, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x48, 0x61, 0x77, 0x61, 0x6a, 0x65, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x79, 0x48, 0x61, 0x77, 0x61, 0x6a, 0x65, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x48, 0x61, 0x77,
0x61, 0x6a, 0x65, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x79, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67,
0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x62, 0x64, 0x6f, 0x4b, 0x6f,
0x62, 0x64, 0x6f, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4b, 0x6f,
0x62, 0x64, 0x6f, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x69, 0x6e, 0x64,
0x79, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4f, 0x63,
0x65, 0x61, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x79, 0x6a, 0x73, 0x6b, 0x69,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x69, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69,
0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x6a, 0x61, 0x20, 0x15a, 0x72, 0x6f, 0x64, 0x6b,
0x6f, 0x77, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x7a, 0x6a, 0x61, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x7a, 0x6a, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x49, 0x72, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x49,
0x72, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x63, 0x6b, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x49, 0x7a,
0x72, 0x61, 0x65, 0x6c, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x49,
0x7a, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4a,
0x61, 0x70, 0x6f, 0x6e, 0x69, 0x61, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4a, 0x61, 0x70, 0x6f,
0x6e, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x69, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x77, 0x142, 0x6f, 0x77, 0x73, 0x6b, 0x20,
0x4b, 0x61, 0x6d, 0x63, 0x7a, 0x61, 0x63, 0x6b, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x50, 0x69, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x77, 0x142,
0x6f, 0x77, 0x73, 0x6b, 0x20, 0x4b, 0x61, 0x6d, 0x63, 0x7a, 0x61, 0x63,
0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x69, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x77, 0x142, 0x6f, 0x77, 0x73, 0x6b, 0x20, 0x4b, 0x61,
0x6d, 0x63, 0x7a, 0x61, 0x63, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x65, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x29, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73,
0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4c, 0x69, 0x6e,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x29, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x6a, 0x61, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x77, 0x79,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x79,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x61, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x29, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4d,
0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x29, 0x4d,
0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4d,
0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x55, 0x142,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x55, 0x142, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x55, 0x142, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x4d,
0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x4d, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e, 0x65, 0x70,
0x61, 0x6c, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e, 0x6f, 0x77, 0x61, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4e, 0x6f, 0x77,
0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x61, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4e,
0x6f, 0x77, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x46,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x4e, 0x6f, 0x77,
0x61, 0x20, 0x46, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4e, 0x6f, 0x77, 0x61, 0x20,
0x46, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4e, 0x6f, 0x77,
0x6f, 0x73, 0x79, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4e, 0x6f, 0x77, 0x6f,
0x73, 0x79, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x79, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f,
0x6d, 0x73, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4f, 0x6d,
0x73, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61,
0x2d, 0x4e, 0x6f, 0x77, 0x61, 0x20, 0x47, 0x77, 0x69, 0x6e, 0x65, 0x61,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61,
0x6a, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x6a, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x6a,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x65, 0x72, 0x75, 0x50, 0x65,
0x72, 0x75, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x50, 0x65, 0x72,
0x75, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x79, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x79, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x79, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x46, 0x65, 0x6e, 0x69,
0x6b, 0x73, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x69, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x69, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x6a, 0x6f, 0x6e, 0x67, 0x6a,
0x61, 0x6e, 0x67, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6b, 0x79, 0x7a, 0x79,
0x142, 0x6f, 0x72, 0x64, 0x7a, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x6b, 0x79, 0x7a, 0x79, 0x142, 0x6f, 0x72, 0x64, 0x7a, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6b, 0x79, 0x7a, 0x79, 0x142, 0x6f, 0x72, 0x64,
0x7a, 0x6b, 0x69, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61,
0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x65,
0x73, 0x7a, 0x65, 0x6c, 0x65, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57,
0x79, 0x73, 0x70, 0x79, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x20, 0x50, 0x6f, 0x142, 0x75, 0x64, 0x6e, 0x69, 0x6f, 0x77, 0x61,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x54, 0x61, 0x6a, 0x70,
0x65, 0x6a, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x54, 0x61, 0x6a,
0x70, 0x65, 0x6a, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x54, 0x61, 0x64,
0x17c, 0x79, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x6a, 0x55, 0x72,
0x75, 0x67, 0x77, 0x61, 0x6a, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x6a, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x28, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x29, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x28,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x29, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x65, 0x6e, 0x65, 0x7a, 0x75,
0x65, 0x6c, 0x61, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x142, 0x61, 0x64,
0x79, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57, 0x142, 0x61, 0x64, 0x79,
0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x29, 0x57, 0x142, 0x61, 0x64, 0x79, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x6f, 0x142, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x57, 0x6f, 0x142, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x57, 0x6f, 0x142, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x61, 0x6b,
0x65, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73,
0x20, 0x69, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x4a, 0x61, 0x6b, 0x75, 0x63, 0x6b, 0x4a, 0x61, 0x6b, 0x75,
0x63, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4a, 0x61, 0x6b,
0x75, 0x63, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x79, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x79, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20,
0x28, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x79, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x28, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x4a, 0x75, 0x6b, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x66, 0x65, 0x67, 0x61, 0x6e,
0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75, 0x6c, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x63, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x63, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c,
0x61, 0x73, 0x63, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x4c, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4c,
0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x4c, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68,
0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x61, 0x6e, 0x68, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x73,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x71, 0x74, 0x61,
0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x71, 0x74, 0x61,
0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0xe1, 0x62, 0x69,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0xe1, 0x62, 0x69,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0xe1,
0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x6d, 0xea, 0x6e, 0x69, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x6d, 0xea, 0x6e, 0x69, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x6d, 0xea,
0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe2, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe2, 0x6e, 0x74, 0x69,
0x63, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x74,
0x6c, 0xe2, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x6f, 0x2d, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x2d, 0x4f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x2d, 0x4f, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x72,
0x7a, 0x65, 0x69, 0x62, 0x61, 0x69, 0x6a, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x41, 0x72, 0x7a, 0x65, 0x69, 0x62, 0x61, 0x69, 0x6a,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x72,
0x7a, 0x65, 0x69, 0x62, 0x61, 0x69, 0x6a, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x41, 0xe7, 0x6f,
0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x41, 0xe7,
0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x73,
0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x42, 0x75, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0xed, 0x76, 0x69, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42,
0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20,
0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68,
0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf4, 0x6d, 0x62,
0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf4,
0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x43, 0x6f, 0x6c, 0xf4, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4d,
0x65, 0x69, 0x6f, 0x20, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75,
0x62, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x44,
0x61, 0x76, 0x69, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019,
0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x2d,
0x4c, 0x65, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0xe1, 0x73, 0x63, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe1, 0x73, 0x63,
0x6f, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c,
0x68, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x45,
0x71, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x45, 0x78,
0x74, 0x72, 0x65, 0x6d, 0x6f, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61,
0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61,
0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x54, 0x65, 0x72,
0x72, 0x69, 0x74, 0xf3, 0x72, 0x69, 0x6f, 0x73, 0x20, 0x46, 0x72, 0x61,
0x6e, 0x63, 0x65, 0x73, 0x65, 0x73, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75,
0x6c, 0x20, 0x65, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69, 0x64,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62,
0x69, 0x65, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf3, 0x72, 0x67, 0x69,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x4d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20,
0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6d,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x47,
0x6f, 0x6c, 0x66, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x48, 0x61, 0x76, 0x61, 0xed, 0x20, 0x65, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x73, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x76, 0x61, 0xed, 0x20,
0x65, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x48,
0x61, 0x76, 0x61, 0xed, 0x20, 0x65, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73,
0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4f,
0x63, 0x65, 0x61, 0x6e, 0x6f, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63, 0x6f,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0xe9, 0x73, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x73, 0x69,
0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x49, 0x72,
0xe3, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x4a, 0x61, 0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x4a, 0x61, 0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x4a, 0x61, 0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x61, 0x73, 0x61, 0x71, 0x75,
0x69, 0x73, 0x74, 0xe3, 0x6f, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x43, 0x61, 0x73, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f,
0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72,
0x65, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72,
0x65, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43,
0x6f, 0x72, 0x65, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x51, 0x75, 0x69, 0x72, 0x67,
0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c,
0x68, 0x61, 0x73, 0x20, 0x64, 0x61, 0x20, 0x4c, 0x69, 0x6e, 0x68, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0xe1, 0x73, 0x69, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20,
0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61,
0x73, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x6f, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x6f, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0xed, 0x63,
0x69, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63,
0x6f, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x75, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f,
0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x69, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0xf4, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f,
0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf4, 0x6e, 0x69, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf4, 0x6e, 0x69, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c,
0xe2, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0xe2, 0x6e, 0x64,
0x69, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x54,
0x65, 0x72, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x69, 0x75, 0x65,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x74, 0x65, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69,
0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61,
0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x6c, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0xe9, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x46, 0xea,
0x6e, 0x69, 0x78, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x53, 0xe3, 0x6f, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20,
0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xe3, 0x6f, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x53, 0xe3, 0x6f, 0x20, 0x50, 0x65, 0x64, 0x72,
0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xe3, 0x6f,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x56, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0xe3, 0x6f, 0x20, 0x50, 0x65, 0x64,
0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70,
0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x51,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x52, 0x65, 0x75, 0x6e, 0x69, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x53, 0x65, 0x69, 0x63, 0x68, 0x65, 0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20,
0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53,
0x75, 0x6c, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77,
0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x54, 0x61, 0x69, 0x74, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54,
0x61, 0x6a, 0x69, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0xe1,
0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54,
0x75, 0x72, 0x63, 0x6f, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe3, 0x6f,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x63, 0x6f, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x63, 0x6f, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75,
0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55,
0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75,
0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69,
0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x48, 0x6f,
0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68,
0x61, 0x73, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73,
0x20, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x50,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x63, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72,
0x69, 0x6f, 0x20, 0x50, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x45, 0x63, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x6f, 0x48, 0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x63,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x48,
0x6f, 0x72, 0xe1, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x59, 0x75,
0x6b, 0x6f, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x41,
0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x63, 0x72, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x41, 0x66, 0x65, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x75, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0xc1, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c, 0x61, 0x73,
0x63, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x63, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x74,
0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61,
0x6e, 0x68, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x6f,
0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65,
0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x68, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x74,
0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6e,
0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66,
0x69, 0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x74, 0x65, 0x2d, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41,
0x71, 0x74, 0x61, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x71, 0x74,
0x6f, 0x62, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72,
0xe1, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0xe1, 0x62, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x4f,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0x41, 0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x41, 0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x61, 0x20, 0x41, 0x72, 0x6d, 0xe9, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe2, 0x6e, 0x74, 0x69,
0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe2, 0x6e, 0x74, 0x69,
0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0xe2, 0x6e,
0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x69,
0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x6a, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x6a, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x6f, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x73, 0x20, 0x41, 0xe7, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x63, 0x68, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x63, 0x68, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x63, 0x68, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x75, 0x74, 0xe3, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0xed,
0x76, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed,
0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x42,
0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4e, 0x61, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf4, 0x6d, 0x62,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf4, 0x6d, 0x62, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf4, 0x6d, 0x62,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x4c, 0x65, 0x73,
0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c,
0x68, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50,
0xe1, 0x73, 0x63, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c,
0x68, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50, 0xe1, 0x73, 0x63, 0x6f, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x45, 0x71, 0x75, 0x61,
0x64, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x45, 0x78,
0x74, 0x72, 0x65, 0x6d, 0x6f, 0x20, 0x4c, 0x65, 0x73, 0x74, 0x65, 0x20,
0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c,
0x68, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20,
0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61,
0x6e, 0x63, 0x65, 0x73, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x73, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x73, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x69, 0x73, 0x20, 0x65, 0x20, 0x41, 0x6e, 0x74, 0xe1,
0x72, 0x63, 0x74, 0x69, 0x63, 0x61, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e,
0x63, 0x65, 0x73, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x73, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf3, 0x72, 0x67,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68,
0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47,
0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72,
0x6f, 0x6e, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x47,
0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe2, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe2, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x6e, 0x65, 0x6c, 0xe2,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0x61, 0x6d, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20,
0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x48, 0x61, 0x76, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x48, 0x61, 0x76, 0x61, 0x69,
0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x61, 0x73, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x6f, 0x20, 0x48, 0x61, 0x76, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x6f, 0x20, 0xcd,
0x6e, 0x64, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe9,
0x73, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0xe9, 0x73, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0xe9, 0x73, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x49, 0x72, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x6f, 0x20, 0x49, 0x72, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x4a, 0x61, 0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4a,
0x61, 0x70, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x4a, 0x61, 0x70,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63,
0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x43, 0x61, 0x7a, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3,
0x6f, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x61, 0x7a, 0x61, 0x71, 0x75,
0x69, 0x73, 0x74, 0xe3, 0x6f, 0x20, 0x4f, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43,
0x6f, 0x72, 0x65, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x51, 0x75, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73, 0x74,
0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x53, 0x72,
0x69, 0x20, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4c, 0x69,
0x6e, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0xe1, 0x73, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64,
0x69, 0x76, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75,
0x65, 0x73, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68,
0x61, 0x6c, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x4d,
0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x61,
0x75, 0x72, 0xed, 0x63, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4d,
0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78, 0x69,
0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x63,
0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61,
0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0x6f, 0x76, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x69, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf3,
0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf3,
0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0xe2,
0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0xe2, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x20,
0x4e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61,
0x20, 0x4e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x54, 0x65,
0x72, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x69, 0x75, 0xea, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x61, 0x20,
0x49, 0x6c, 0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4e, 0x6f, 0x72, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61,
0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x50, 0x61, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20,
0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0xe9, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65,
0x72, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x65, 0x72, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73,
0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0xe3, 0x6f, 0x20,
0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0xe3, 0x6f, 0x20, 0x50,
0x65, 0x64, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0xe3, 0x6f, 0x20,
0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64,
0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x52, 0x65, 0x75, 0x6e, 0x69, 0xe3, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x63, 0x61, 0x6c, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x73, 0x20,
0x53, 0x65, 0x69, 0x63, 0x68, 0x65, 0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x61, 0x73, 0x20, 0x49, 0x6c, 0x68, 0x61, 0x73, 0x20,
0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x61, 0x20, 0x47, 0x65, 0xf3, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x53, 0x75, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x61, 0x69, 0x74, 0x69, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54,
0x61, 0x69, 0x70, 0xe9, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x54, 0x61, 0x6a, 0x69, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x54, 0x75, 0x72, 0x71, 0x75, 0x65, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x71, 0x75, 0x65,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f,
0x20, 0x54, 0x75, 0x72, 0x71, 0x75, 0x65, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70,
0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x6f, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75, 0x69, 0x73, 0x74, 0xe3,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x71, 0x75,
0x69, 0x73, 0x74, 0xe3, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x6f, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x61, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x61, 0x20, 0x49,
0x6c, 0x68, 0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x70, 0x61, 0x64, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x45, 0x63, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72,
0x67, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x70, 0x61, 0x64, 0x72, 0xe3,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x63, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0xe3, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45,
0x63, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x73, 0x20, 0x41,
0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64,
0x101, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x73, 0x20, 0x41,
0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x113, 0x69, 0x6d,
0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x101, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b,
0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65,
0x72, 0x64, 0x101, 0x44, 0x113, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73,
0x20, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x6b, 0x65,
0x72, 0x64, 0x101, 0x44, 0x113, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73,
0x20, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x113,
0x69, 0x6d, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x44, 0x113,
0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113, 0x72,
0x69, 0x6b, 0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20,
0x6b, 0x65, 0x72, 0x64, 0x101, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61,
0x73, 0x20, 0x67, 0x101, 0x72, 0x62, 0x61, 0x6e, 0x20, 0x6b, 0x65, 0x72,
0x64, 0x101, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x67,
0x101, 0x72, 0x62, 0x61, 0x6e, 0x20, 0x7a, 0x113, 0x69, 0x6d, 0x61, 0x73,
0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b,
0x61, 0x73, 0x20, 0x67, 0x101, 0x72, 0x62, 0x61, 0x6e, 0x20, 0x64, 0x61,
0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x50, 0x61,
0x63, 0x12b, 0x66, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113,
0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x50,
0x61, 0x63, 0x12b, 0x66, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x41, 0x6d,
0x113, 0x72, 0x69, 0x6b, 0x61, 0x73, 0x20, 0x7a, 0x113, 0x69, 0x6d, 0x61,
0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x50, 0x61, 0x63, 0x12b, 0x66,
0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x41, 0x6d, 0x113, 0x72, 0x69, 0x6b,
0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65,
0x72, 0x64, 0x101, 0x41, 0x74, 0x6c, 0x101, 0x6e, 0x74, 0x69, 0x73, 0x6b,
0x61, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x41, 0x74, 0x6c, 0x101, 0x6e,
0x74, 0x69, 0x73, 0x6b, 0x61, 0x20, 0x7a, 0x113, 0x69, 0x6d, 0x61, 0x73,
0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x41, 0x74, 0x6c, 0x101, 0x6e, 0x74,
0x69, 0x73, 0x6b, 0x61, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20,
0x6b, 0x65, 0x72, 0x64, 0x101, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c,
0x61, 0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x6b,
0x65, 0x72, 0x64, 0x101, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61,
0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x113,
0x69, 0x6d, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x101, 0x6c, 0x61, 0x73, 0x20, 0x45, 0x75, 0x72, 0x14d,
0x70, 0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b,
0x65, 0x72, 0x64, 0x101, 0x44, 0x113, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61,
0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x6b, 0x65,
0x72, 0x64, 0x101, 0x44, 0x113, 0x69, 0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73,
0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x7a, 0x113, 0x69,
0x6d, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x44, 0x113, 0x69,
0x6e, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70,
0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67, 0x61, 0x73, 0x20, 0x6b, 0x65,
0x72, 0x64, 0x101, 0x57, 0x61, 0x6b, 0x6b, 0x61, 0x72, 0x69, 0x73, 0x6b,
0x61, 0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x6b,
0x65, 0x72, 0x64, 0x101, 0x57, 0x61, 0x6b, 0x6b, 0x61, 0x72, 0x69, 0x73,
0x6b, 0x61, 0x73, 0x20, 0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20,
0x7a, 0x113, 0x69, 0x6d, 0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101,
0x57, 0x61, 0x6b, 0x6b, 0x61, 0x72, 0x69, 0x73, 0x6b, 0x61, 0x73, 0x20,
0x45, 0x75, 0x72, 0x14d, 0x70, 0x61, 0x73, 0x20, 0x64, 0x61, 0x67, 0x67,
0x61, 0x73, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x6b, 0x65, 0x72, 0x64, 0x101, 0xa05,
0xa2b, 0xa3c, 0xa17, 0xa3e, 0xa28, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20, 0xa05, 0xa2b,
0xa30, 0xa40, 0xa15, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30,
0xa2c, 0xa40, 0x20, 0xa05, 0xa2b, 0xa30, 0xa40, 0xa15, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa26, 0xa71, 0xa16, 0xa23, 0xa40, 0x20, 0xa05, 0xa2b, 0xa3c, 0xa30,
0xa40, 0xa15, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa05, 0xa2b, 0xa30, 0xa40,
0xa15, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40,
0x20, 0xa05, 0xa2b, 0xa30, 0xa40, 0xa15, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20,
0xa05, 0xa2b, 0xa30, 0xa40, 0xa15, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06,
0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa32, 0xa3e,
0xa38, 0xa15, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa32, 0xa3e, 0xa38,
0xa15, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa05, 0xa32, 0xa3e, 0xa38, 0xa15, 0xa3e, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15,
0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa32, 0xa2e, 0xa3e,
0xa1f, 0xa40, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa32, 0xa2e, 0xa3e, 0xa1f,
0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02,
0xa05, 0xa32, 0xa2e, 0xa3e, 0xa1f, 0xa40, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0x2d,
0xa30, 0xa41, 0xa71, 0xa24, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa2e, 0xa47,
0xa1c, 0xa3c, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa2e, 0xa47, 0xa1c,
0xa3c, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa05, 0xa2e, 0xa47, 0xa1c, 0xa3c, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71,
0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa15,
0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71,
0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa15,
0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e,
0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20,
0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40,
0x20, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09,
0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20,
0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e,
0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2a,
0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09,
0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20,
0xa2e, 0xa3e, 0xa09, 0xa02, 0xa1f, 0xa47, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40,
0x20, 0xa2e, 0xa3e, 0xa09, 0xa02, 0xa1f, 0xa47, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40,
0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2e, 0xa3e, 0xa09, 0xa02,
0xa1f, 0xa47, 0xa28, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e,
0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2a, 0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f, 0xa15,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05,
0xa2e, 0xa30, 0xa40, 0xa15, 0xa40, 0x20, 0xa2a, 0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f,
0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa05, 0xa2e, 0xa30, 0xa40, 0xa15, 0xa40,
0x20, 0xa2a, 0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f, 0xa15, 0x20, 0xa2a, 0xa4d, 0xa30,
0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa10, 0xa2a, 0xa40,
0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa10, 0xa2a, 0xa40, 0xa06, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa10, 0xa2a, 0xa40,
0xa06, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa05, 0xa15, 0xa24, 0xa3e, 0xa09, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02,
0xa05, 0xa15, 0xa24, 0xa3e, 0xa09, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa15, 0xa24, 0xa3e, 0xa09, 0x20, 0xa17, 0xa30,
0xa2e, 0xa40, 0x2d, 0xa30, 0xa41, 0xa71, 0xa24, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02,
0xa05, 0xa15, 0xa24, 0xa4b, 0xa2c, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa15,
0xa24, 0xa4b, 0xa2c, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa38, 0xa2e,
0xa3e, 0xa02, 0xa05, 0xa15, 0xa24, 0xa4b, 0xa2c, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0x2d, 0xa30, 0xa41, 0xa71, 0xa24, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa05, 0xa30,
0xa2c, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa2c, 0xa40, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa30,
0xa2c, 0xa40, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05,
0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71,
0xa1b, 0xa2e, 0xa40, 0x20, 0xa05, 0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa05,
0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20,
0xa05, 0xa30, 0xa1c, 0xa28, 0xa1f, 0xa40, 0xa28, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e,
0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05,
0xa30, 0xa2e, 0xa40, 0xa28, 0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05,
0xa30, 0xa2e, 0xa40, 0xa28, 0xa40, 0xa06, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa30, 0xa2e, 0xa40, 0xa28, 0xa40, 0xa06,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa05, 0xa1f, 0xa32, 0xa3e, 0xa02, 0xa1f, 0xa3f, 0xa15, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1f, 0xa32, 0xa3e, 0xa02, 0xa1f, 0xa3f, 0xa15,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05,
0xa1f, 0xa32, 0xa3e, 0xa02, 0xa1f, 0xa3f, 0xa15, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15,
0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa47, 0xa02, 0xa26,
0xa30, 0xa40, 0x20, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06,
0xa08, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47,
0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38,
0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02,
0xa26, 0xa30, 0xa40, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40,
0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20, 0xa2a, 0xa71,
0xa1b, 0xa2e, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d,
0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30,
0xa40, 0x20, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47,
0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa15, 0xa47, 0xa02, 0xa26, 0xa30, 0xa40, 0x20,
0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38,
0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20,
0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06,
0xa08, 0x20, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47,
0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2a,
0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a,
0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa06, 0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32,
0xa40, 0xa06, 0xa08, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06, 0xa38, 0xa1f, 0xa4d,
0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa06,
0xa38, 0xa1f, 0xa4d, 0xa30, 0xa47, 0xa32, 0xa40, 0xa06, 0xa08, 0x20, 0xa2a, 0xa71,
0xa1b, 0xa2e, 0xa40, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa3c, 0xa30, 0xa2c, 0xa3e, 0xa08, 0xa1c,
0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa3c, 0xa30, 0xa2c,
0xa3e, 0xa08, 0xa1c, 0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa3c, 0xa30, 0xa2c, 0xa3e, 0xa08, 0xa1c,
0xa3e, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa4b, 0xa30, 0xa47, 0xa38, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa4b, 0xa30, 0xa47, 0xa38, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa05, 0xa1c, 0xa4b,
0xa30, 0xa47, 0xa38, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26,
0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2c, 0xa70, 0xa17, 0xa32, 0xa3e, 0xa26,
0xa47, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2c, 0xa70, 0xa17, 0xa32,
0xa3e, 0xa26, 0xa47, 0xa38, 0xa3c, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2c, 0xa70, 0xa17, 0xa32, 0xa3e, 0xa26, 0xa47, 0xa38,
0xa3c, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2d, 0xa42, 0xa1f, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2c, 0xa4b, 0xa32, 0xa40, 0xa35, 0xa40, 0xa06, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa40, 0xa06,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa1c, 0xa3c, 0xa40,
0xa32, 0xa40, 0xa06, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2c, 0xa4d, 0xa30, 0xa3e, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa40, 0xa06,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2c, 0xa30, 0xa42, 0xa28, 0xa47, 0xa08, 0x20, 0xa26, 0xa3e,
0xa30, 0xa42, 0xa38, 0xa32, 0xa3e, 0xa2e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15,
0xa47, 0xa2a, 0x20, 0xa35, 0xa30, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15,
0xa47, 0xa2a, 0x20, 0xa35, 0xa30, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa47, 0xa2a, 0x20, 0xa35, 0xa30, 0xa21,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa15, 0xa47, 0xa38, 0xa40, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02,
0xa1a, 0xa3e, 0xa2e, 0xa4b, 0xa30, 0xa4b, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa48, 0xa25, 0xa2e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa1a, 0xa48, 0xa25, 0xa2e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa48, 0xa25, 0xa2e, 0x20, 0xa2a, 0xa4d,
0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa3f,
0xa32, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa3f, 0xa32, 0xa40, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa3f,
0xa32, 0xa40, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa40, 0xa28, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa1a, 0xa40, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa1a, 0xa40, 0xa28, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e,
0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa4c, 0xa07, 0xa2c, 0xa3e,
0xa32, 0xa38, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1a, 0xa4c, 0xa07, 0xa2c,
0xa3e, 0xa32, 0xa38, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa1a, 0xa4c, 0xa07, 0xa2c, 0xa3e, 0xa32, 0xa38, 0xa28, 0x20,
0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa15, 0xa4d, 0xa30, 0xa3f, 0xa38, 0xa2e, 0xa38, 0x20, 0xa06, 0xa08,
0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b, 0xa15,
0xa38, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa15, 0xa4b, 0xa32, 0xa70, 0xa2c, 0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa15, 0xa4b, 0xa32, 0xa70, 0xa2c, 0xa40, 0xa06, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b, 0xa32, 0xa70, 0xa2c,
0xa40, 0xa06, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa41, 0xa71, 0xa15, 0x20, 0xa06, 0xa08,
0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa41, 0xa71,
0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa41, 0xa71, 0xa15, 0x20,
0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa05, 0xa71, 0xa27, 0x20, 0xa17,
0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa15, 0xa3f, 0xa0a, 0xa2c, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15,
0xa3f, 0xa0a, 0xa2c, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa15, 0xa3f, 0xa0a, 0xa2c, 0xa3e, 0x20, 0xa2a, 0xa4d, 0xa30,
0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa21, 0xa47, 0xa35,
0xa3f, 0xa38, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa21, 0xa3f, 0xa09, 0xa2e, 0xa4b,
0xa02, 0xa1f, 0x20, 0xa21, 0xa3f, 0xa09, 0xa30, 0xa35, 0xa3f, 0xa32, 0xa47, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa24, 0xa3f,
0xa2e, 0xa42, 0xa30, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa08, 0xa38, 0xa1f, 0xa30,
0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa08, 0xa38, 0xa1f, 0xa30, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa08, 0xa38,
0xa1f, 0xa30, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa17, 0xa30,
0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa07, 0xa15, 0xa35, 0xa47, 0xa21, 0xa4b, 0xa30, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2e, 0xa71, 0xa27, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2e, 0xa71, 0xa27, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa71,
0xa27, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42,
0xa30, 0xa2c, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a,
0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40, 0x20,
0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa39, 0xa4b, 0xa30, 0x2d, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20,
0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71,
0xa1b, 0xa2e, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a,
0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa2f, 0xa42, 0xa30, 0xa2a, 0xa40, 0x20,
0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2b, 0xa3c, 0xa3e, 0xa15, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa06,
0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0xa38, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b,
0xa3c, 0xa3e, 0xa15, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa06, 0xa08, 0xa32, 0xa48,
0xa02, 0xa21, 0xa38, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2b, 0xa3c, 0xa3e, 0xa15, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa06,
0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0xa38, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06,
0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3c, 0xa3f,
0xa1c, 0xa3c, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3c, 0xa3f, 0xa1c,
0xa3c, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2b, 0xa3c, 0xa3f, 0xa1c, 0xa3c, 0xa40, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa4d,
0xa30, 0xa48, 0xa02, 0xa1a, 0x20, 0xa17, 0xa41, 0xa0f, 0xa28, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2b, 0xa4d, 0xa30, 0xa48, 0xa02, 0xa1a, 0x20, 0xa26, 0xa71,
0xa16, 0xa23, 0xa40, 0x20, 0xa05, 0xa24, 0xa47, 0x20, 0xa10, 0xa02, 0xa1f, 0xa3e,
0xa30, 0xa1f, 0xa3f, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa17, 0xa32, 0xa3e,
0xa2a, 0xa3e, 0xa17, 0xa4b, 0xa38, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa17, 0xa48,
0xa02, 0xa2c, 0xa40, 0xa05, 0xa30, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1c, 0xa3e,
0xa30, 0xa1c, 0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1c, 0xa3e, 0xa30,
0xa1c, 0xa40, 0xa06, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa1c, 0xa3e, 0xa30, 0xa1c, 0xa40, 0xa06, 0x20, 0xa17, 0xa30, 0xa2e,
0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa17,
0xa3f, 0xa32, 0xa2c, 0xa30, 0xa1f, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa17, 0xa4d, 0xa30, 0xa40, 0xa28, 0xa35, 0xa3f,
0xa1a, 0x20, 0xa2e, 0xa40, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42,
0xa30, 0xa2c, 0xa40, 0x20, 0xa17, 0xa4d, 0xa30, 0xa40, 0xa28, 0xa32, 0xa48, 0xa02,
0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20,
0xa17, 0xa4d, 0xa30, 0xa40, 0xa28, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c,
0xa40, 0x20, 0xa17, 0xa4d, 0xa30, 0xa40, 0xa28, 0xa32, 0xa48, 0xa02, 0xa21, 0x20,
0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa17, 0xa4d, 0xa30, 0xa40,
0xa28, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71,
0xa1b, 0xa2e, 0xa40, 0x20, 0xa17, 0xa4d, 0xa30, 0xa40, 0xa28, 0xa32, 0xa48, 0xa02,
0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa17, 0xa4d, 0xa30, 0xa40, 0xa28, 0xa32,
0xa48, 0xa02, 0xa21, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26,
0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa17, 0xa41, 0xa06, 0xa2e, 0x20, 0xa38,
0xa2e, 0xa3e, 0xa02, 0xa16, 0xa3e, 0xa5c, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa17, 0xa41, 0xa2f, 0xa3e, 0xa28, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa35, 0xa3e, 0xa08, 0x2d, 0xa05, 0xa32,
0xa47, 0xa2f, 0xa42, 0xa38, 0xa3c, 0xa3f, 0xa05, 0xa28, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa39, 0xa35, 0xa3e, 0xa08, 0x2d, 0xa05, 0xa32, 0xa47, 0xa2f, 0xa42, 0xa38,
0xa3c, 0xa3f, 0xa05, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa39, 0xa35, 0xa3e, 0xa08, 0x2d, 0xa05, 0xa32, 0xa47, 0xa2f,
0xa42, 0xa38, 0xa3c, 0xa3f, 0xa05, 0xa28, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e,
0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa3e, 0xa02, 0xa17, 0x20,
0xa15, 0xa3e, 0xa02, 0xa17, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa3e, 0xa02,
0xa17, 0x20, 0xa15, 0xa3e, 0xa02, 0xa17, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa3e, 0xa02, 0xa17, 0x20, 0xa15, 0xa3e,
0xa02, 0xa17, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa4b, 0xa35, 0xa21, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa39, 0xa4b, 0xa35, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa4b, 0xa35, 0xa21, 0x20, 0xa17, 0xa30,
0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2d, 0xa3e, 0xa30, 0xa24, 0xa40, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa39, 0xa3f, 0xa70, 0xa26, 0x20, 0xa2e, 0xa39, 0xa3e,
0xa02, 0xa38, 0xa3e, 0xa17, 0xa30, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa07, 0xa70,
0xa21, 0xa4b, 0xa1a, 0xa3e, 0xa08, 0xa28, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa2e, 0xa71, 0xa27, 0x20, 0xa07, 0xa70, 0xa21, 0xa4b, 0xa28, 0xa47, 0xa38, 0xa3c,
0xa40, 0xa06, 0xa08, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c,
0xa40, 0x20, 0xa07, 0xa70, 0xa21, 0xa4b, 0xa28, 0xa47, 0xa38, 0xa3c, 0xa40, 0xa06,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa07,
0xa70, 0xa21, 0xa4b, 0xa28, 0xa47, 0xa38, 0xa3c, 0xa40, 0xa06, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa08, 0xa30, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa08,
0xa30, 0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa08, 0xa30, 0xa3e, 0xa28, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e,
0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa07, 0xa30, 0xa15, 0xa41, 0xa24,
0xa38, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa07, 0xa30, 0xa15, 0xa41, 0xa24,
0xa38, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa07, 0xa30, 0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0x20, 0xa17, 0xa30, 0xa2e,
0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa07,
0xa1c, 0xa3c, 0xa30, 0xa3e, 0xa08, 0xa32, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa07,
0xa1c, 0xa3c, 0xa30, 0xa3e, 0xa08, 0xa32, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa07, 0xa1c, 0xa3c, 0xa30, 0xa3e, 0xa08, 0xa32,
0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa1c, 0xa2a, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa1c, 0xa2a,
0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa1c, 0xa2a, 0xa3e, 0xa28, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38,
0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa42, 0xa30, 0xa2c, 0xa40, 0x20,
0xa15, 0xa1c, 0xa3c, 0xa3e, 0xa16, 0xa3c, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2a, 0xa71, 0xa1b, 0xa2e, 0xa40, 0x20, 0xa15, 0xa1c, 0xa3c,
0xa3e, 0xa16, 0xa3c, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa15, 0xa4b, 0xa30, 0xa40, 0xa06, 0xa08, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15,
0xa4b, 0xa30, 0xa40, 0xa06, 0xa08, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4b, 0xa30, 0xa40, 0xa06, 0xa08, 0x20, 0xa2a,
0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15,
0xa4b, 0xa38, 0xa30, 0xa47, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa4d, 0xa30,
0xa3e, 0xa38, 0xa28, 0xa4b, 0xa2f, 0xa3e, 0xa30, 0xa38, 0xa15, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa15, 0xa4d, 0xa30, 0xa3e, 0xa38, 0xa28, 0xa4b, 0xa2f, 0xa3e, 0xa30,
0xa38, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa15, 0xa4d, 0xa30, 0xa3e, 0xa38, 0xa28, 0xa4b, 0xa2f, 0xa3e, 0xa30, 0xa38,
0xa15, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa15, 0xa3f, 0xa30, 0xa17, 0xa3f, 0xa38, 0xa24, 0xa3e,
0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa32, 0xa70, 0xa15, 0xa3e, 0x20, 0xa38,
0xa2e, 0xa3e, 0xa02, 0xa32, 0xa3e, 0xa08, 0xa28, 0x20, 0xa06, 0xa08, 0xa32, 0xa48,
0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa32, 0xa4c, 0xa30, 0xa21, 0x20,
0xa39, 0xa4b, 0xa35, 0xa47, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa32, 0xa4c, 0xa30,
0xa21, 0x20, 0xa39, 0xa4b, 0xa35, 0xa47, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa32, 0xa4c, 0xa30, 0xa21, 0x20, 0xa39, 0xa4b,
0xa35, 0xa47, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2e, 0xa15, 0xa3e, 0xa09, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02,
0xa2e, 0xa15, 0xa3e, 0xa09, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa38,
0xa2e, 0xa3e, 0xa02, 0xa2e, 0xa15, 0xa3e, 0xa09, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0x2d, 0xa30, 0xa41, 0xa71, 0xa24, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa2e, 0xa48,
0xa17, 0xa47, 0xa21, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa48, 0xa17,
0xa47, 0xa21, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2e, 0xa48, 0xa17, 0xa47, 0xa21, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e,
0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e,
0xa32, 0xa47, 0xa38, 0xa3c, 0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e,
0xa3e, 0xa32, 0xa26, 0xa40, 0xa35, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3e,
0xa30, 0xa15, 0xa3f, 0xa38, 0xa3e, 0xa38, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e,
0xa3e, 0xa30, 0xa38, 0xa3c, 0xa32, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa4c, 0xa30, 0xa3f, 0xa38, 0xa3c, 0xa38,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa4c, 0xa30, 0xa3f, 0xa38, 0xa3c, 0xa38,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e,
0xa4c, 0xa30, 0xa3f, 0xa38, 0xa3c, 0xa38, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06,
0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa4c, 0xa38,
0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa48, 0xa15, 0xa38, 0xa40, 0xa15,
0xa28, 0x20, 0xa2a, 0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f, 0xa15, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2e, 0xa48, 0xa15, 0xa38, 0xa40, 0xa15, 0xa28, 0x20, 0xa2a, 0xa48,
0xa38, 0xa3f, 0xa2b, 0xa3f, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa48, 0xa15, 0xa38, 0xa40, 0xa15, 0xa28, 0x20,
0xa2a, 0xa48, 0xa38, 0xa3f, 0xa2b, 0xa3f, 0xa15, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15,
0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa32, 0xa28, 0x20,
0xa2c, 0xa1f, 0xa4b, 0xa30, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa32, 0xa28,
0x20, 0xa2c, 0xa1f, 0xa4b, 0xa30, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa32, 0xa28, 0x20, 0xa2c, 0xa1f, 0xa4b, 0xa30,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3e, 0xa38, 0xa15, 0xa4b, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2e, 0xa3e, 0xa38, 0xa15, 0xa4b, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2e, 0xa3e, 0xa38, 0xa15, 0xa4b, 0x20, 0xa17,
0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2e, 0xa3f, 0xa06, 0xa02, 0xa2e, 0xa3e, 0xa30, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa28, 0xa3e, 0xa09, 0xa30, 0xa42, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28,
0xa47, 0xa2a, 0xa3e, 0xa32, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a,
0x20, 0xa15, 0xa48, 0xa32, 0xa47, 0xa21, 0xa4b, 0xa28, 0xa40, 0xa06, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0x20, 0xa15, 0xa48, 0xa32, 0xa47, 0xa21,
0xa4b, 0xa28, 0xa40, 0xa06, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0x20, 0xa15, 0xa48, 0xa32, 0xa47, 0xa21,
0xa4b, 0xa28, 0xa40, 0xa06, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20,
0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa1c, 0xa3c,
0xa40, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f,
0xa0a, 0xa1c, 0xa3c, 0xa40, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa1c, 0xa3c,
0xa40, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38,
0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f, 0xa0a, 0xa2b, 0xa3e, 0xa09,
0xa02, 0xa21, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28,
0xa3f, 0xa0a, 0xa2b, 0xa3e, 0xa09, 0xa02, 0xa21, 0xa32, 0xa48, 0xa02, 0xa21, 0x20,
0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa3f,
0xa0a, 0xa2b, 0xa3e, 0xa09, 0xa02, 0xa21, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa2a,
0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28,
0xa40, 0xa0a, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa4b, 0xa30, 0xa2b, 0xa4c,
0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa28, 0xa4b, 0xa30, 0xa2b, 0xa4c, 0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48,
0xa02, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa28, 0xa4b, 0xa30, 0xa2b, 0xa4c, 0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48,
0xa02, 0xa21, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2b, 0xa30, 0xa28, 0xa3e, 0xa02, 0xa21, 0xa4b, 0x20, 0xa21,
0xa47, 0x20, 0xa28, 0xa4b, 0xa30, 0xa4b, 0xa28, 0xa39, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2b, 0xa30, 0xa28, 0xa3e, 0xa02, 0xa21, 0xa4b, 0x20, 0xa21, 0xa47,
0x20, 0xa28, 0xa4b, 0xa30, 0xa4b, 0xa28, 0xa39, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa30, 0xa28, 0xa3e, 0xa02,
0xa21, 0xa4b, 0x20, 0xa21, 0xa47, 0x20, 0xa28, 0xa4b, 0xa30, 0xa4b, 0xa28, 0xa39,
0xa3e, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa71, 0xa24, 0xa30, 0xa40, 0x20, 0xa2e, 0xa30,
0xa3f, 0xa06, 0xa28, 0xa3e, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20,
0xa38, 0xa2e, 0xa3e, 0xa02, 0xa28, 0xa4c, 0xa35, 0xa4b, 0xa38, 0xa3f, 0xa2c, 0xa40,
0xa30, 0xa38, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa4c, 0xa35, 0xa4b,
0xa38, 0xa3f, 0xa2c, 0xa40, 0xa30, 0xa38, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa28, 0xa4c, 0xa35, 0xa4b, 0xa38, 0xa3f,
0xa2c, 0xa40, 0xa30, 0xa38, 0xa15, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02,
0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa13, 0xa2e, 0xa38, 0xa15,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa13, 0xa2e, 0xa38, 0xa15, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa13, 0xa2e, 0xa38, 0xa15,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2a, 0xa3e, 0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa3e, 0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a,
0xa3e, 0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40,
0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa32,
0xa3e, 0xa09, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa3e, 0xa2a, 0xa42, 0xa06,
0x20, 0xa28, 0xa3f, 0xa0a, 0x20, 0xa17, 0xa3f, 0xa28, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2a, 0xa48, 0xa30, 0xa3e, 0xa17, 0xa35, 0xa47, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2a, 0xa48, 0xa30, 0xa3e, 0xa17, 0xa35, 0xa47, 0x20, 0xa2e, 0xa3f,
0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa48, 0xa30, 0xa3e,
0xa17, 0xa35, 0xa47, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26,
0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa47, 0xa30, 0xa42, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2a, 0xa47, 0xa30, 0xa42, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa47, 0xa30, 0xa42, 0x20, 0xa17,
0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa2b, 0xa3f, 0xa32, 0xa3f, 0xa2a, 0xa40, 0xa28, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2b, 0xa3f, 0xa32, 0xa3f, 0xa2a, 0xa40, 0xa28, 0xa40, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3f, 0xa32,
0xa3f, 0xa2a, 0xa40, 0xa28, 0xa40, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02,
0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2b, 0xa3f, 0xa28, 0xa3f,
0xa15, 0xa38, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa38, 0xa48, 0xa02, 0xa1f, 0x20, 0xa2a, 0xa40, 0xa05, 0xa30, 0xa47,
0x20, 0xa05, 0xa24, 0xa47, 0x20, 0xa2e, 0xa3f, 0xa15, 0xa47, 0xa32, 0xa28, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa48, 0xa02, 0xa1f, 0x20, 0xa2a, 0xa40, 0xa05,
0xa30, 0xa47, 0x20, 0xa05, 0xa24, 0xa47, 0x20, 0xa2e, 0xa3f, 0xa15, 0xa47, 0xa32,
0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa38, 0xa48, 0xa02, 0xa1f, 0x20, 0xa2a, 0xa40, 0xa05, 0xa30, 0xa47, 0x20, 0xa05,
0xa24, 0xa47, 0x20, 0xa2e, 0xa3f, 0xa15, 0xa47, 0xa32, 0xa28, 0x20, 0xa2a, 0xa4d,
0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa3f,
0xa1f, 0xa15, 0xa48, 0xa30, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa4b,
0xa28, 0xa3e, 0xa2a, 0xa47, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2a, 0xa2f, 0xa4b,
0xa02, 0xa17, 0xa2f, 0xa3e, 0xa02, 0xa17, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa15,
0xa3f, 0xa1c, 0xa3c, 0xa32, 0xa4b, 0xa30, 0xa21, 0xa3e, 0x20, 0xa38, 0xa2e, 0xa3e,
0xa02, 0xa15, 0xa3f, 0xa1c, 0xa3c, 0xa32, 0xa4b, 0xa30, 0xa21, 0xa3e, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa15, 0xa3f, 0xa1c,
0xa3c, 0xa32, 0xa4b, 0xa30, 0xa21, 0xa3e, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0x2d,
0xa30, 0xa41, 0xa71, 0xa24, 0x20, 0xa38, 0xa2e, 0xa3e, 0xa02, 0xa30, 0xa3f, 0xa2f,
0xa42, 0xa28, 0xa40, 0xa05, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa30, 0xa4b,
0xa25, 0xa47, 0xa30, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa16, 0xa32,
0xa40, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa16, 0xa32, 0xa40, 0xa28,
0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38,
0xa16, 0xa32, 0xa40, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20,
0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa3e, 0xa2e, 0xa4b, 0xa06,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa3e, 0xa2e, 0xa4b, 0xa06, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa3e, 0xa2e,
0xa4b, 0xa06, 0x20, 0xa2a, 0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa38, 0xa47, 0xa38, 0xa3c, 0xa32, 0xa38, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa38, 0xa3f, 0xa70, 0xa17, 0xa3e, 0xa2a, 0xa41, 0xa30, 0x20, 0xa2e,
0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa4b, 0xa32,
0xa4b, 0xa2e, 0xa28, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02, 0xa21, 0xa38, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa26, 0xa71, 0xa16, 0xa23, 0xa40, 0x20, 0xa1c, 0xa3e,
0xa30, 0xa1c, 0xa40, 0xa06, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa42, 0xa30,
0xa40, 0xa28, 0xa3e, 0xa2e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa38, 0xa3f, 0xa13,
0xa35, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa24, 0xa3e, 0xa39, 0xa40, 0xa24,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa24, 0xa48, 0xa2a, 0xa08, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa24, 0xa48, 0xa2a, 0xa08, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa24, 0xa48, 0xa2a, 0xa08, 0x20, 0xa2a,
0xa4d, 0xa30, 0xa15, 0xa3e, 0xa38, 0xa3c, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa24,
0xa3e, 0xa1c, 0xa3f, 0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa1f, 0xa4b, 0xa15, 0xa47, 0xa32, 0xa3e, 0xa09, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa1f, 0xa4b, 0xa02, 0xa17, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa1f, 0xa4b, 0xa02, 0xa17, 0xa3e, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa1f, 0xa4b, 0xa02, 0xa17, 0xa3e, 0x20, 0xa17, 0xa30,
0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa1a, 0xa42, 0xa15, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa24, 0xa41, 0xa30, 0xa15,
0xa2e, 0xa47, 0xa28, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa24, 0xa41, 0xa30, 0xa15, 0xa2e, 0xa47, 0xa28, 0xa3f, 0xa38, 0xa24, 0xa3e,
0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e,
0xa24, 0xa41, 0xa30, 0xa15, 0xa2e, 0xa47, 0xa28, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa1f, 0xa41, 0xa35, 0xa3e, 0xa32, 0xa42, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa09, 0xa30, 0xa42, 0xa17, 0xa35, 0xa47, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa09, 0xa30, 0xa42, 0xa17, 0xa35, 0xa47, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30,
0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa30, 0xa42, 0xa17, 0xa35, 0xa47,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa09, 0xa1c, 0xa3c, 0xa2c, 0xa47, 0xa15, 0xa3f, 0xa38, 0xa24,
0xa3e, 0xa28, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa1c, 0xa3c, 0xa2c, 0xa47,
0xa15, 0xa3f, 0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa09, 0xa1c, 0xa3c, 0xa2c, 0xa47, 0xa15, 0xa3f,
0xa38, 0xa24, 0xa3e, 0xa28, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20,
0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa3e, 0xa28, 0xa42, 0xa06,
0xa1f, 0xa42, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa3e, 0xa28, 0xa42, 0xa06,
0xa1f, 0xa42, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32,
0xa3e, 0xa35, 0xa3e, 0xa28, 0xa42, 0xa06, 0xa1f, 0xa42, 0x20, 0xa17, 0xa30, 0xa2e,
0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35,
0xa48, 0xa28, 0xa47, 0xa1c, 0xa3c, 0xa42, 0xa0f, 0xa32, 0xa3e, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa35, 0xa32, 0xa3e, 0xa26, 0xa40, 0xa35, 0xa4b, 0xa38, 0xa24, 0xa15,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa32, 0xa3e, 0xa26, 0xa40, 0xa35, 0xa4b,
0xa38, 0xa24, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa35, 0xa32, 0xa3e, 0xa26, 0xa40, 0xa35, 0xa4b, 0xa38, 0xa24, 0xa15,
0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa35, 0xa4b, 0xa32, 0xa17, 0xa4b, 0xa17, 0xa4d, 0xa30, 0xa47,
0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa4b, 0xa32, 0xa17, 0xa4b, 0xa17,
0xa4d, 0xa30, 0xa47, 0xa21, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa35, 0xa4b, 0xa32, 0xa17, 0xa4b, 0xa17, 0xa4d, 0xa30, 0xa47,
0xa21, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa4b, 0xa38, 0xa1f, 0xa4b, 0xa15, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa35, 0xa47, 0xa15, 0x20, 0xa06, 0xa08, 0xa32, 0xa48, 0xa02,
0xa21, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa35, 0xa3e, 0xa32, 0xa3f, 0xa38, 0x20,
0xa05, 0xa24, 0xa47, 0x20, 0xa2b, 0xa41, 0xa1f, 0xa42, 0xa28, 0xa3e, 0x20, 0xa35,
0xa47, 0xa32, 0xa3e, 0xa2f, 0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0x20, 0xa35, 0xa47,
0xa32, 0xa3e, 0xa2f, 0xa15, 0xa41, 0xa24, 0xa38, 0xa15, 0x20, 0xa2e, 0xa3f, 0xa06,
0xa30, 0xa40, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2f, 0xa15, 0xa41, 0xa24, 0xa38,
0xa15, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2f, 0xa15, 0xa47, 0xa24, 0xa30, 0xa3f, 0xa28, 0xa2c,
0xa30, 0xa17, 0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2f, 0xa15, 0xa47, 0xa24, 0xa30,
0xa3f, 0xa28, 0xa2c, 0xa30, 0xa17, 0x20, 0xa2e, 0xa3f, 0xa06, 0xa30, 0xa40, 0x20,
0xa35, 0xa47, 0xa32, 0xa3e, 0xa2f, 0xa15, 0xa47, 0xa24, 0xa30, 0xa3f, 0xa28, 0xa2c,
0xa30, 0xa17, 0x20, 0xa17, 0xa30, 0xa2e, 0xa40, 0xa06, 0xa02, 0x20, 0xa26, 0xa3e,
0x20, 0xa35, 0xa47, 0xa32, 0xa3e, 0xa2f, 0xa42, 0xa15, 0xa4b, 0xa28, 0x20, 0xa38,
0xa2e, 0xa3e, 0xa02, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x75, 0x64, 0x61, 0x66, 0x72, 0x69, 0x63, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63,
0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61,
0xf1, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4d, 0x6f,
0x6e, 0x74, 0x61, 0xf1, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed,
0x66, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x41, 0x70, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72,
0x61, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61,
0x62, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65,
0x73, 0x74, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65,
0x73, 0x74, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f,
0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74,
0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c,
0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x44,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x4f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x79, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x79, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x79, 0xe1, 0x6e,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x68, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69,
0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68,
0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x79, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x73, 0x63, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x6d, 0xe1, 0x73, 0x20, 0x4f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61,
0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76,
0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x75, 0x61, 0x79, 0x61,
0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0xe9, 0x73, 0x20,
0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x6f, 0x6e, 0x61, 0x6c, 0x20, 0x79,
0x20, 0x61, 0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69, 0x63, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4d, 0x65, 0x72, 0x69, 0x64,
0x69, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x6f, 0x6c,
0x66, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x69, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x6f, 0x20, 0xcd, 0x6e, 0x64, 0x69,
0x63, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72,
0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0xe1, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0xe1, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4a, 0x61, 0x70, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0xe1,
0x6e, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65, 0x73,
0x74, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73, 0x74,
0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73,
0x70, 0xf3, 0x72, 0x61, 0x64, 0x61, 0x73, 0x20, 0x45, 0x63, 0x75, 0x61,
0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x69, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x63, 0x69, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c,
0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x4d, 0x65, 0x78,
0x69, 0x63, 0x61, 0x6e, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe1, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x65, 0x70, 0x61, 0x6c, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65,
0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65,
0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x6e,
0x6f, 0x76, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d,
0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x75, 0x65,
0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x65, 0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72,
0xfa, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x50,
0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x20,
0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xf3, 0x6e, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f,
0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xf3, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64,
0x72, 0x6f, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xf3,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x6f, 0x68,
0x6e, 0x70, 0x65, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x69, 0x6f, 0x6e, 0x79, 0x61, 0x6e, 0x67, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x53, 0x75, 0x72, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54,
0x61, 0x69, 0x70, 0xe9, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54,
0x61, 0x69, 0x70, 0xe9, 0x69, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x79, 0x69, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0xe1,
0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x45, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x48, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x48, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x45, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x6f, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x55,
0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x63, 0x72, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x41,
0x63, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x41, 0x63, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x75, 0x6c, 0x75,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x69, 0x69,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x69, 0x69, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x66,
0x72, 0x69, 0x63, 0x69, 0x69, 0x20, 0x4d, 0x65, 0x72, 0x69, 0x64, 0x69,
0x6f, 0x6e, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x66, 0x72,
0x69, 0x63, 0x69, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63,
0x69, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x63, 0x69, 0x69, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72, 0x61,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x75, 0x6c, 0x75, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x75, 0x6c, 0x75, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x103,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x103,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x103, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x7a,
0x6f, 0x6e, 0x65, 0x69, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x65,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0xee, 0x6e, 0x20, 0x7a, 0x6f, 0x6e, 0x61, 0x20,
0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e, 0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0xee, 0x6e,
0x20, 0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x6e,
0x103, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65,
0x69, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0xee, 0x6e, 0x20, 0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0xee, 0x6e, 0x20, 0x7a, 0x6f,
0x6e, 0x61, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x103, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x70, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x71, 0x74,
0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x7a, 0x6f,
0x6e, 0x65, 0x69, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x41, 0x71, 0x74, 0x6f,
0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65, 0x69, 0x20, 0x41, 0x71,
0x74, 0x6f, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x61, 0x72, 0x61, 0x62,
0x103, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x72, 0x61, 0x62, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x72, 0x61, 0x62, 0x103,
0x4f, 0x72, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x65, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x65, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x65, 0x69, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e,
0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x7a, 0x6f, 0x6e, 0x65, 0x69,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0xee, 0x6e, 0x20, 0x7a, 0x6f, 0x6e, 0x61, 0x20, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0xee, 0x6e, 0x20, 0x7a,
0x6f, 0x6e, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x2d, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x103, 0x4f, 0x72, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69, 0x20,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x65, 0x69, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65,
0x69, 0x20, 0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69, 0x20,
0x4f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69, 0x20,
0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x65, 0x69,
0x20, 0x4f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64,
0x6a, 0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a, 0x61, 0x6e, 0x75, 0x6c, 0x75,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x6a,
0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x41, 0x7a, 0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x4f, 0x72, 0x61, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x75,
0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76,
0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x42,
0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x43, 0x61, 0x70, 0x75, 0x6c, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x43, 0x61, 0x70, 0x75, 0x6c, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x61, 0x70, 0x75, 0x6c,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x43, 0x68, 0x69, 0x6e, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x43, 0x68, 0x69,
0x6e, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x65, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x43, 0x68,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61,
0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20,
0x43, 0x6f, 0x63, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x43, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x43, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x75,
0x6d, 0x62, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65,
0x6c, 0x6f, 0x72, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x43, 0x75, 0x62, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x43, 0x75, 0x62, 0x65,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x61, 0x20, 0x43, 0x75, 0x62, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d,
0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x75, 0x6c, 0x75, 0x69, 0x20, 0x64,
0x65, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x50, 0x61, 0x219, 0x74,
0x65, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x61, 0x20, 0x50, 0x61, 0x219, 0x74, 0x65, 0x6c, 0x75, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x50,
0x61, 0x219, 0x74, 0x65, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x45,
0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73,
0x74, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20,
0x64, 0x65, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x64, 0x65,
0x20, 0x45, 0x73, 0x74, 0x20, 0xee, 0x6e, 0x64, 0x65, 0x70, 0x103, 0x72,
0x74, 0x61, 0x74, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x73, 0x74, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x20, 0x64, 0x65,
0x20, 0x56, 0x65, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x69, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x73, 0x74, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65,
0x6c, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x65,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x65, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x7a, 0x103, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x65, 0x72, 0x69, 0x74, 0x6f,
0x72, 0x69, 0x69, 0x6c, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x20, 0x219, 0x69, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63,
0x74, 0x69, 0x63, 0x65, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x7a,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x4f, 0x72, 0x61,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x68, 0x69, 0x63, 0x68, 0x4f, 0x72, 0x61, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x20,
0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x69, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x65,
0x69, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x69, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x61, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x75, 0x6c, 0x75,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f,
0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x75,
0x6c, 0x75, 0x69, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x65, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69,
0x65, 0x69, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x65,
0x69, 0x20, 0x64, 0x65, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x65, 0x69, 0x20, 0x64,
0x65, 0x20, 0x56, 0x65, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x72,
0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x75,
0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49,
0x72, 0x6b, 0x75, 0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x4a, 0x61, 0x70,
0x6f, 0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x6f,
0x6e, 0x69, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x69,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x63, 0x69, 0x61, 0x21b, 0x6b, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x69, 0x61, 0x21b, 0x6b,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x69,
0x61, 0x21b, 0x6b, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x75, 0x6c, 0x20,
0x64, 0x65, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x75,
0x6c, 0x20, 0x64, 0x65, 0x20, 0x56, 0x65, 0x73, 0x74, 0x4f, 0x72, 0x61,
0x20, 0x43, 0x6f, 0x72, 0x65, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x43, 0x6f,
0x72, 0x65, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x65, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69, 0x61, 0x72, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x69,
0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x4b, 0xe2, 0x72, 0x67, 0xe2, 0x7a, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65,
0x6c, 0x65, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f,
0x72, 0x20, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x69, 0x7a, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72, 0x20,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x7a, 0x6f,
0x6e, 0x65, 0x69, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x20,
0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x7a,
0x6f, 0x6e, 0x65, 0x69, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x7a,
0x6f, 0x6e, 0x65, 0x69, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x6d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x76, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4d,
0x6f, 0x73, 0x63, 0x6f, 0x76, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0x6f, 0x76, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x4d, 0x79, 0x61,
0x6e, 0x6d, 0x61, 0x72, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x4e, 0x6f, 0x69, 0x69, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4e, 0x6f, 0x69, 0x69, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x4e,
0x6f, 0x69, 0x69, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x4e, 0x6f, 0x69, 0x69, 0x20, 0x5a, 0x65,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x4e, 0x6f, 0x69,
0x69, 0x20, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20,
0x4e, 0x6f, 0x69, 0x69, 0x20, 0x5a, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77,
0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e,
0x73, 0x75, 0x6c, 0x65, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x69, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x49, 0x6e, 0x73,
0x75, 0x6c, 0x65, 0x69, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x75, 0x6c, 0x75,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f, 0x75,
0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x72,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x50, 0x65, 0x72, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x6c, 0x6f, 0x72,
0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x219, 0x69, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x219, 0x69,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x219, 0x69, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x52, 0x65, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x52,
0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x53,
0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65,
0x6c, 0x6f, 0x72, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x69, 0x20,
0x64, 0x65, 0x20, 0x53, 0x75, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x53, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x6d, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x64, 0x6a, 0x69,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75,
0x61, 0x79, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x75, 0x6c, 0x75, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x69, 0x6e, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x65, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x61, 0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x69, 0x6e, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x49, 0x6e, 0x73, 0x75, 0x6c, 0x65, 0x69, 0x20, 0x57, 0x61,
0x6b, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x219, 0x69, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x61,
0x6b, 0x75, 0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x61, 0x6b,
0x75, 0x21b, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61,
0x72, 0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x49, 0x61, 0x6b, 0x75, 0x21b,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x61, 0x72,
0x103, 0x20, 0x64, 0x69, 0x6e, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x69,
0x6e, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x54, 0x65, 0x6d, 0x70, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x54, 0x65, 0x6d, 0x70, 0x20,
0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x54, 0x65, 0x6d, 0x70, 0x20,
0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x64, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61,
0x20, 0x73, 0x74, 0x61, 0x64, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x6d, 0x75,
0x6e, 0x74, 0x6f, 0x67, 0x6e, 0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x61, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x6f, 0x67, 0x6e, 0x61, 0x54, 0x65,
0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x64, 0x20, 0x64,
0x61, 0x20, 0x6d, 0x75, 0x6e, 0x74, 0x6f, 0x67, 0x6e, 0x61, 0x54, 0x65,
0x6d, 0x70, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x54, 0x65,
0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x54, 0x65,
0x6d, 0x70, 0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x64, 0x20, 0x70,
0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x61,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x54, 0x65, 0x6d, 0x70, 0x20,
0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x54, 0x65, 0x6d, 0x70,
0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x64, 0x20, 0x61, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61,
0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x54, 0x65, 0x6d, 0x70, 0x20,
0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x64, 0x20, 0x64, 0x61, 0x20, 0x6c,
0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x43, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64, 0x61, 0x20,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x54, 0x65, 0x6d, 0x70,
0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x64, 0x20, 0x64, 0x61, 0x20,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x4f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x61, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x64,
0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64,
0x61, 0x6c, 0x20, 0x56, 0x65, 0x73, 0x74, 0x54, 0x65, 0x6d, 0x70, 0x20,
0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x61, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x64, 0x61, 0x6c, 0x20, 0x56, 0x65, 0x73, 0x74, 0x54, 0x65, 0x6d, 0x70,
0x20, 0x64, 0x61, 0x20, 0x73, 0x74, 0x61, 0x64, 0x20, 0x64, 0x61, 0x20,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x61, 0x6c,
0x20, 0x56, 0x65, 0x73, 0x74, 0x54, 0x65, 0x6d, 0x70, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x410, 0x43a, 0x440, 0x438, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43a, 0x440, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x410, 0x43a, 0x440, 0x438, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x444, 0x440, 0x438,
0x43a, 0x430, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x42e, 0x436, 0x43d, 0x430, 0x44f, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x430, 0x44f, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x417, 0x430, 0x43f,
0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x430, 0x44f, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43b, 0x43c,
0x430, 0x2d, 0x410, 0x442, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410,
0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x410, 0x43b, 0x43c, 0x430, 0x2d, 0x410, 0x442, 0x430, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442,
0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x412, 0x43e,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x413, 0x43e, 0x440, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x20, 0x28, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x430, 0x44f, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29,
0x421, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x433, 0x43e, 0x440, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x20, 0x28, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x44f, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x41b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x433, 0x43e, 0x440, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x20, 0x28, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x430, 0x44f,
0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x29, 0x422, 0x438, 0x445,
0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x422, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422,
0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x412, 0x440, 0x435, 0x43c, 0x44f, 0x20, 0x43f, 0x43e, 0x20, 0x410, 0x43d,
0x430, 0x434, 0x44b, 0x440, 0x44e, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440,
0x44c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x410, 0x43f, 0x438, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x410, 0x43f, 0x438, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43a, 0x442, 0x430, 0x443,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x43a, 0x442, 0x430, 0x443, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x410, 0x43a, 0x442, 0x43e, 0x431, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x443, 0x434, 0x43e,
0x432, 0x441, 0x43a, 0x430, 0x44f, 0x20, 0x410, 0x440, 0x430, 0x432, 0x438, 0x44f,
0x421, 0x430, 0x443, 0x434, 0x43e, 0x432, 0x441, 0x43a, 0x430, 0x44f, 0x20, 0x410,
0x440, 0x430, 0x432, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x421, 0x430, 0x443, 0x434, 0x43e, 0x432, 0x441, 0x43a, 0x430, 0x44f, 0x20, 0x410,
0x440, 0x430, 0x432, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430,
0x44f, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x410, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x44f, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x442, 0x43b, 0x430, 0x43d,
0x442, 0x438, 0x447, 0x435, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447, 0x435, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x2c, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x2c, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430,
0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x412, 0x43e, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412,
0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410,
0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x437, 0x435, 0x440, 0x431, 0x430, 0x439,
0x434, 0x436, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a,
0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x410, 0x437, 0x43e, 0x440, 0x441,
0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x43a, 0x438, 0x435, 0x20,
0x43e, 0x2d, 0x432, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430,
0x434, 0x435, 0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x411, 0x430,
0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x411, 0x440, 0x430,
0x437, 0x438, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x430,
0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434,
0x435, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x41a, 0x435, 0x439, 0x441, 0x438, 0x427, 0x430, 0x442, 0x435,
0x43c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x427, 0x430, 0x442, 0x435,
0x43c, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x427, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x427, 0x438, 0x43b, 0x438, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x438, 0x442, 0x430,
0x439, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x438, 0x442, 0x430,
0x439, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x427, 0x43e, 0x439, 0x431, 0x430,
0x43b, 0x441, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x43e, 0x2d, 0x432, 0x20, 0x420, 0x43e,
0x436, 0x434, 0x435, 0x441, 0x442, 0x432, 0x430, 0x41a, 0x43e, 0x43a, 0x43e, 0x441,
0x43e, 0x432, 0x44b, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x443, 0x43a, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x20, 0x41a, 0x443, 0x43a, 0x430, 0x2c, 0x20, 0x43f, 0x43e, 0x43b, 0x443, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a,
0x443, 0x431, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x443,
0x431, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x44b,
0x439, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x41e, 0x2d, 0x432, 0x20, 0x41f,
0x430, 0x441, 0x445, 0x438, 0x41e, 0x2d, 0x432, 0x20, 0x41f, 0x430, 0x441, 0x445,
0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41e, 0x2d, 0x432, 0x20,
0x41f, 0x430, 0x441, 0x445, 0x438, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430, 0x44f,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x430,
0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20,
0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x43e, 0x441, 0x43a, 0x43e,
0x432, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d,
0x434, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x424, 0x43e,
0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e,
0x2d, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x43e,
0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e,
0x2d, 0x432, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x438, 0x434, 0x436, 0x438, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x438, 0x434, 0x436, 0x438, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x441, 0x43a, 0x430, 0x44f, 0x20,
0x413, 0x432, 0x438, 0x430, 0x43d, 0x430, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443,
0x437, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x42e, 0x436, 0x43d, 0x44b, 0x435, 0x20,
0x438, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x447, 0x435,
0x441, 0x43a, 0x438, 0x435, 0x20, 0x442, 0x435, 0x440, 0x440, 0x438, 0x442, 0x43e,
0x440, 0x438, 0x438, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441,
0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x413, 0x440, 0x443,
0x437, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x413, 0x440,
0x443, 0x437, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x43e, 0x2d, 0x432, 0x430, 0x20, 0x413,
0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x430, 0x421, 0x440, 0x435, 0x434, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x20, 0x43f, 0x43e, 0x20,
0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x443, 0x412, 0x43e, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x438, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430,
0x44f, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x430, 0x44f, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f,
0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x435,
0x440, 0x441, 0x438, 0x434, 0x441, 0x43a, 0x438, 0x439, 0x20, 0x437, 0x430, 0x43b,
0x438, 0x432, 0x413, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430,
0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x413, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430,
0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x413, 0x430, 0x432, 0x430, 0x439, 0x441, 0x43a, 0x43e, 0x2d, 0x430,
0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x413, 0x43e, 0x43d,
0x43a, 0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x413,
0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x425, 0x43e, 0x432, 0x434,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x425, 0x43e, 0x432, 0x434, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x418, 0x43d, 0x434, 0x438, 0x439, 0x441, 0x43a, 0x438, 0x439, 0x20, 0x43e,
0x43a, 0x435, 0x430, 0x43d, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x430, 0x44f, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438,
0x44f, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x430, 0x44f, 0x20, 0x418,
0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x430, 0x44f, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437,
0x438, 0x44f, 0x418, 0x440, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x418, 0x440, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x440, 0x43a, 0x443, 0x442,
0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x440, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x437, 0x440, 0x430, 0x438, 0x43b,
0x44c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x418, 0x437, 0x440, 0x430,
0x438, 0x43b, 0x44c, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e,
0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a,
0x438, 0x439, 0x41f, 0x435, 0x442, 0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e,
0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a,
0x438, 0x439, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x435, 0x442,
0x440, 0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a,
0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x439, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412,
0x43e, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x44b, 0x439, 0x20, 0x41a, 0x430, 0x437,
0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d,
0x44b, 0x439, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d,
0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x41a, 0x438, 0x440, 0x433, 0x438, 0x437, 0x438, 0x44f, 0x43e, 0x2d, 0x432, 0x430,
0x20, 0x41b, 0x430, 0x439, 0x43d, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430,
0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41b, 0x43e, 0x440, 0x434,
0x2d, 0x425, 0x430, 0x443, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x44b, 0x41c, 0x430, 0x440, 0x43a,
0x438, 0x437, 0x441, 0x43a, 0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x41c,
0x430, 0x440, 0x448, 0x430, 0x43b, 0x43b, 0x43e, 0x432, 0x44b, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x438,
0x439, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x430, 0x432, 0x440,
0x438, 0x43a, 0x438, 0x439, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422, 0x438, 0x445, 0x43e, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43c, 0x435, 0x43a, 0x441,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x422, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x43c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d,
0x441, 0x43a, 0x43e, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422, 0x438,
0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435, 0x20,
0x43c, 0x435, 0x43a, 0x441, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x435,
0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411,
0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41c, 0x43e, 0x441, 0x43a, 0x432,
0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20, 0x41a, 0x430, 0x43b,
0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20,
0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20, 0x41a, 0x430,
0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x43e, 0x432,
0x430, 0x44f, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x41d,
0x43e, 0x432, 0x430, 0x44f, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x43e, 0x432, 0x430,
0x44f, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x44c, 0x44e, 0x444, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d, 0x43e, 0x440,
0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41d,
0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x435, 0x440, 0x43d,
0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e, 0x440, 0x43e,
0x43d, 0x44c, 0x44f, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x435,
0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x438, 0x2d, 0x41d, 0x43e,
0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x44b, 0x435, 0x20, 0x41c, 0x430, 0x440, 0x438, 0x430, 0x43d, 0x441, 0x43a,
0x438, 0x435, 0x20, 0x43e, 0x2d, 0x432, 0x430, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x430, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x430, 0x43f, 0x443,
0x430, 0x20, 0x2013, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x44f, 0x20, 0x413, 0x432,
0x438, 0x43d, 0x435, 0x44f, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x439,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x430, 0x440, 0x430, 0x433,
0x432, 0x430, 0x439, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424, 0x438,
0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x44b, 0x424, 0x438, 0x43b, 0x438, 0x43f,
0x43f, 0x438, 0x43d, 0x44b, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x424,
0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x44b, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x43e, 0x2d,
0x432, 0x430, 0x20, 0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x421, 0x435, 0x43d,
0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435,
0x43b, 0x43e, 0x43d, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440, 0x20,
0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x44c, 0x435, 0x440,
0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x41a, 0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x2a, 0x41a, 0x44b, 0x437, 0x44b, 0x43b, 0x43e, 0x440,
0x434, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x2a, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x412, 0x440, 0x435, 0x43c, 0x44f, 0x20, 0x432, 0x20,
0x421, 0x430, 0x43c, 0x430, 0x440, 0x435, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x441,
0x43a, 0x43e, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x43c,
0x430, 0x440, 0x441, 0x43a, 0x43e, 0x435, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x43c, 0x43e, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x421, 0x430, 0x43c, 0x43e, 0x430,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x44f, 0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x44c, 0x441, 0x43a, 0x438,
0x435, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x421, 0x43e, 0x43b,
0x43e, 0x43c, 0x43e, 0x43d, 0x43e, 0x432, 0x44b, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x42e, 0x436, 0x43d, 0x430, 0x44f, 0x20, 0x413, 0x435, 0x43e,
0x440, 0x433, 0x438, 0x44f, 0x422, 0x430, 0x439, 0x432, 0x430, 0x43d, 0x44c, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422, 0x430, 0x439, 0x432, 0x430, 0x43d,
0x44c, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422, 0x443,
0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x439, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x440, 0x443,
0x433, 0x432, 0x430, 0x439, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423,
0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412,
0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x44f, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x43d, 0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x412, 0x43e, 0x43b,
0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x423, 0x43e, 0x43b, 0x43b,
0x438, 0x441, 0x20, 0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x42f,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f,
0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d,
0x435, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x44f, 0x415, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x43e, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x44f, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431,
0x443, 0x440, 0x433, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x43d, 0x435, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x44f, 0x410, 0x440, 0x430, 0x430, 0x431, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x410, 0x440, 0x430, 0x430, 0x431, 0x20, 0x441, 0x4af, 0x440,
0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x410, 0x440, 0x430, 0x430, 0x431,
0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x42d, 0x440, 0x43c, 0x44d, 0x44d, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x42d, 0x440, 0x43c, 0x44d, 0x44d, 0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x42d, 0x440, 0x43c, 0x44d, 0x44d, 0x43d, 0x20,
0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x41a, 0x438, 0x438, 0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x44f, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x438, 0x438, 0x43d, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x4af,
0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x438, 0x438, 0x43d,
0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441,
0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x418,
0x43b, 0x438, 0x43d, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x43b, 0x438, 0x43d, 0x20, 0x410,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x4af, 0x440,
0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x43b, 0x438, 0x43d, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x430,
0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x410, 0x440,
0x495, 0x430, 0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x410, 0x440, 0x495, 0x430, 0x430, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x4af,
0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x410, 0x440, 0x495, 0x430,
0x430, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20,
0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x41a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x44b,
0x442, 0x430, 0x439, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x41a, 0x44b, 0x442, 0x430, 0x439, 0x20, 0x441, 0x430, 0x439, 0x44b,
0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x427, 0x43e, 0x439, 0x431,
0x430, 0x43b, 0x441, 0x430, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x427, 0x43e,
0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af,
0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b,
0x441, 0x430, 0x43d, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x443, 0x440, 0x443, 0x441, 0x443, 0x443, 0x43d,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x443, 0x440, 0x443, 0x441, 0x443, 0x443,
0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x41a, 0x443, 0x440, 0x443, 0x441, 0x443, 0x443, 0x43d, 0x20, 0x441, 0x430, 0x439,
0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x438, 0x43d,
0x434, 0x438, 0x439, 0x44d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x418, 0x440, 0x430, 0x430, 0x43d, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x418, 0x440, 0x430, 0x430, 0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x42b, 0x440, 0x430, 0x430, 0x43d, 0x20, 0x441,
0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x414,
0x44c, 0x43e, 0x43f, 0x43f, 0x443, 0x43e, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x414, 0x44c, 0x43e, 0x43f, 0x43f, 0x443, 0x43e, 0x43d, 0x20, 0x441, 0x4af, 0x440,
0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x414, 0x44c, 0x43e, 0x43f, 0x43f,
0x443, 0x43e, 0x43d, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x418, 0x43b, 0x438, 0x43d, 0x20, 0x41a, 0x430, 0x437,
0x430, 0x445, 0x441, 0x442, 0x430, 0x430, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x410, 0x440, 0x495, 0x430, 0x430, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441,
0x442, 0x430, 0x430, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x44d, 0x440,
0x438, 0x44d, 0x439, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x44d, 0x440, 0x438,
0x44d, 0x439, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x41a, 0x44d, 0x440, 0x438, 0x44d, 0x439, 0x20, 0x441, 0x430, 0x439, 0x44b,
0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x430,
0x439, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x430, 0x439,
0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x41a, 0x44b, 0x440, 0x433, 0x44b, 0x441, 0x442, 0x430, 0x430, 0x43d, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x430, 0x43d,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x430,
0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x41c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x430, 0x43d, 0x20, 0x441, 0x430, 0x439,
0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x423, 0x43b, 0x430,
0x43d, 0x20, 0x411, 0x430, 0x430, 0x442, 0x430, 0x440, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x430, 0x442, 0x430, 0x440,
0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x423,
0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x430, 0x442, 0x430, 0x440, 0x20, 0x441,
0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41c,
0x43e, 0x441, 0x43a, 0x443, 0x431, 0x430, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41c,
0x43e, 0x441, 0x43a, 0x443, 0x431, 0x430, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d,
0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41c, 0x43e, 0x441, 0x43a, 0x443, 0x431, 0x430,
0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x421, 0x430, 0x4a5, 0x430, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x421, 0x430, 0x4a5, 0x430, 0x20,
0x421, 0x44d, 0x439, 0x43b, 0x44d, 0x43d, 0x434, 0x20, 0x441, 0x4af, 0x440, 0x4af,
0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x421, 0x430, 0x4a5, 0x430, 0x20, 0x421,
0x44d, 0x439, 0x43b, 0x44d, 0x43d, 0x434, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5,
0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a,
0x430, 0x439, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a,
0x430, 0x439, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x41e, 0x43c, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x41e, 0x43c, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x441, 0x4af, 0x440,
0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x41e, 0x43c, 0x441, 0x43a, 0x430,
0x439, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d,
0x43c, 0x44d, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x430, 0x43d, 0x20,
0x43a, 0x44d, 0x43c, 0x44d, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x430,
0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d,
0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x430, 0x43d, 0x20, 0x441, 0x430,
0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x421, 0x430,
0x445, 0x430, 0x43b, 0x438, 0x438, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x44b, 0x44b, 0x43d, 0x20, 0x441, 0x4af, 0x440, 0x4af,
0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x44b,
0x44b, 0x43d, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a,
0x44d, 0x43c, 0x44d, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x411, 0x44b, 0x43b, 0x430, 0x434,
0x44c, 0x44b, 0x431, 0x430, 0x441, 0x442, 0x44b, 0x430, 0x43a, 0x20, 0x441, 0x4af,
0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x411, 0x44b, 0x43b, 0x430,
0x434, 0x44c, 0x44b, 0x431, 0x430, 0x441, 0x442, 0x44b, 0x430, 0x43a, 0x20, 0x441,
0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x412,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x43a, 0x44d, 0x43c,
0x44d, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x441,
0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x412, 0x43e, 0x43b,
0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5,
0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x414, 0x44c, 0x43e, 0x43a, 0x443,
0x443, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x414, 0x44c,
0x43e, 0x43a, 0x443, 0x443, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x441, 0x4af, 0x440,
0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x414, 0x44c, 0x43e, 0x43a, 0x443,
0x443, 0x441, 0x43a, 0x430, 0x439, 0x20, 0x441, 0x430, 0x439, 0x44b, 0x4a5, 0x4a5,
0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x415,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x443, 0x440, 0x20,
0x441, 0x4af, 0x440, 0x4af, 0x43d, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x415, 0x43a,
0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x443, 0x440, 0x20, 0x441,
0x430, 0x439, 0x44b, 0x4a5, 0x4a5, 0x44b, 0x20, 0x43a, 0x44d, 0x43c, 0x44d, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e,
0x3a, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e,
0x3a, 0x20, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x906, 0x926, 0x930, 0x94d, 0x936,
0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905,
0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x92e, 0x927, 0x94d, 0x92f,
0x20, 0x905, 0x92f, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924,
0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a,
0x20, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e,
0x3a, 0x20, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x20, 0x906, 0x926, 0x930, 0x94d,
0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20,
0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x92a, 0x94c, 0x930,
0x94d, 0x935, 0x20, 0x905, 0x92f, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x3a,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x93e, 0x3a, 0x20, 0x936, 0x948, 0x932, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e,
0x3a, 0x20, 0x936, 0x948, 0x932, 0x20, 0x906, 0x926, 0x930, 0x94d, 0x936, 0x20,
0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e,
0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x936, 0x948, 0x932, 0x20, 0x905,
0x92f, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x938,
0x928, 0x94d, 0x927, 0x93f, 0x92a, 0x94d, 0x930, 0x93f, 0x92f, 0x20, 0x938, 0x92e,
0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930,
0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x938, 0x928, 0x94d, 0x927, 0x93f, 0x92a, 0x94d,
0x930, 0x93f, 0x92f, 0x20, 0x906, 0x926, 0x930, 0x94d, 0x936, 0x20, 0x938, 0x92e,
0x92f, 0x903, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x20, 0x905, 0x92e, 0x947, 0x930,
0x93f, 0x915, 0x93e, 0x3a, 0x20, 0x938, 0x928, 0x94d, 0x927, 0x93f, 0x92a, 0x94d,
0x930, 0x93f, 0x92f, 0x20, 0x905, 0x92f, 0x93e, 0x92e, 0x20, 0x938, 0x92e, 0x92f,
0x903, 0x905, 0x91f, 0x932, 0x93e, 0x923, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x938,
0x92e, 0x92f, 0x903, 0x905, 0x91f, 0x932, 0x93e, 0x923, 0x94d, 0x91f, 0x93f, 0x915,
0x20, 0x906, 0x926, 0x930, 0x94d, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x905,
0x91f, 0x932, 0x93e, 0x923, 0x94d, 0x91f, 0x93f, 0x915, 0x20, 0x905, 0x92f, 0x93e,
0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x92f,
0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92e,
0x927, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20,
0x906, 0x926, 0x930, 0x94d, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92e, 0x927,
0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x917,
0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a,
0x94c, 0x930, 0x94d, 0x935, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f,
0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x20, 0x92f,
0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x906, 0x926, 0x930, 0x94d, 0x936,
0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x94c, 0x930, 0x94d, 0x935, 0x20, 0x92f,
0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937,
0x94d, 0x92e, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x93e, 0x936, 0x94d, 0x91a,
0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f,
0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x93e, 0x936, 0x94d, 0x91a, 0x93e, 0x924,
0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x92f, 0x20, 0x906,
0x926, 0x930, 0x94d, 0x936, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x92a, 0x93e, 0x936,
0x94d, 0x91a, 0x93e, 0x924, 0x94d, 0x92f, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a,
0x940, 0x92f, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x937, 0x94d, 0x92e, 0x20, 0x938,
0x92e, 0x92f, 0x903, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f, 0x91a, 0x20,
0x92e, 0x940, 0x928, 0x20, 0x938, 0x92e, 0x92f, 0x903, 0x1c5f, 0x1c79, 0x1c5e, 0x1c5f,
0x1c65, 0x1c60, 0x1c5f, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c5f, 0x1c79, 0x1c5e, 0x1c5f,
0x1c65, 0x1c60, 0x1c5f, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68,
0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c5f, 0x1c79, 0x1c5e, 0x1c5f, 0x1c65, 0x1c60,
0x1c5f, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a,
0x1c5f, 0x1c62, 0x1c5f, 0x1c61, 0x1c5a, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c5f,
0x1c62, 0x1c5f, 0x1c61, 0x1c5a, 0x1c71, 0x20, 0x1c6e, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f,
0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c5f, 0x1c62, 0x1c5f, 0x1c61, 0x1c5a,
0x1c71, 0x20, 0x1c65, 0x1c64, 0x1c5b, 0x1c69, 0x1c5d, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a,
0x1c5b, 0x1c5f, 0x1c71, 0x1c5f, 0x1c5e, 0x1c5f, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c5b,
0x1c5f, 0x1c71, 0x1c5f, 0x1c5e, 0x1c5f, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70,
0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c5b, 0x1c5f, 0x1c71, 0x1c5f,
0x1c5e, 0x1c5f, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a,
0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71,
0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c64, 0x1c65, 0x1c74,
0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c75, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c5f, 0x1c69, 0x1c71, 0x1c74, 0x1c6e, 0x1c71, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c5f, 0x1c69, 0x1c71, 0x1c74, 0x1c6e, 0x1c71, 0x20, 0x1c6e,
0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x1c62, 0x1c5f, 0x1c69, 0x1c71, 0x1c74, 0x1c6e, 0x1c71, 0x20, 0x1c65, 0x1c64, 0x1c67,
0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6f, 0x1c6e, 0x1c65, 0x1c64, 0x1c6f,
0x1c77, 0x1c64, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6f, 0x1c6e, 0x1c65, 0x1c64,
0x1c6f, 0x1c77, 0x1c64, 0x1c60, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f,
0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6f, 0x1c6e, 0x1c65, 0x1c64, 0x1c6f,
0x1c77, 0x1c64, 0x1c60, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c5f, 0x1c79, 0x1c68, 0x1c61, 0x1c6e, 0x1c71, 0x1c74, 0x1c64, 0x1c71, 0x1c5f,
0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c5f, 0x1c79, 0x1c68, 0x1c61, 0x1c6e, 0x1c71, 0x1c74,
0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c6e, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70,
0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c5f, 0x1c79, 0x1c68, 0x1c61, 0x1c6e, 0x1c71, 0x1c74,
0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c65, 0x1c64, 0x1c5b, 0x1c69, 0x1c5d, 0x20, 0x1c5a, 0x1c60,
0x1c74, 0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5f, 0x1c79, 0x1c68, 0x1c61, 0x1c6e,
0x1c71, 0x1c74, 0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c6f, 0x1c5f,
0x1c6a, 0x1c6e, 0x20, 0x1c5f, 0x1c79, 0x1c68, 0x1c61, 0x1c6e, 0x1c71, 0x1c74, 0x1c64, 0x1c71,
0x1c5f, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20,
0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5f, 0x1c79, 0x1c68,
0x1c61, 0x1c6e, 0x1c71, 0x1c74, 0x1c64, 0x1c71, 0x1c5f, 0x20, 0x1c65, 0x1c64, 0x1c5b, 0x1c69,
0x1c5d, 0x20, 0x1c5a, 0x1c60, 0x1c74, 0x1c5a, 0x1c6e, 0x1c74, 0x1c5e, 0x1c5f, 0x1c71, 0x1c74,
0x1c64, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c74, 0x1c5e, 0x1c5f, 0x1c71,
0x1c74, 0x1c64, 0x1c60, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68,
0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c74, 0x1c5e, 0x1c5f, 0x1c71, 0x1c74,
0x1c64, 0x1c60, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x1c60, 0x1c6d, 0x1c69, 0x1c75, 0x1c5f, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c60,
0x1c6d, 0x1c69, 0x1c75, 0x1c5f, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c60, 0x1c6d, 0x1c69, 0x1c75, 0x1c5f, 0x20, 0x1c65, 0x1c64, 0x1c67,
0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c65, 0x1c6e, 0x1c71, 0x1c74, 0x1c68,
0x1c5f, 0x1c5e, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c65, 0x1c6e, 0x1c71, 0x1c74, 0x1c68, 0x1c5f, 0x1c5e, 0x20,
0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c6e, 0x1c65, 0x1c74,
0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c65,
0x1c6e, 0x1c71, 0x1c74, 0x1c68, 0x1c5f, 0x1c5e, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64,
0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c68, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c69, 0x1c68, 0x1c5a,
0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c64, 0x1c65,
0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f,
0x1c71, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e, 0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c64, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c69,
0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f,
0x1c68, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c63, 0x1c6e, 0x1c65, 0x1c74, 0x1c5f, 0x1c68,
0x1c71, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c63, 0x1c6e, 0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c69,
0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c6e, 0x1c65, 0x1c74, 0x1c6e,
0x1c71, 0x1c70, 0x1c5f, 0x1c68, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c63, 0x1c6e,
0x1c65, 0x1c74, 0x1c5f, 0x1c68, 0x1c71, 0x20, 0x1c69, 0x1c68, 0x1c5a, 0x1c6f, 0x1c64, 0x1c6d,
0x1c5f, 0x1c71, 0x20, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c68, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x1c5c, 0x1c68, 0x1c64, 0x1c71, 0x1c63, 0x1c64, 0x1c6a, 0x20, 0x1c62, 0x1c64, 0x1c71,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c5d, 0x20, 0x1c5c,
0x1c7d, 0x1c68, 0x1c64, 0x1c71, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c5d, 0x20, 0x1c5c, 0x1c7d, 0x1c68, 0x1c64, 0x1c71,
0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c65, 0x1c5f, 0x1c62, 0x1c5f, 0x1c5d, 0x20, 0x1c5c, 0x1c7d, 0x1c68,
0x1c64, 0x1c71, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c65, 0x1c64, 0x1c5b, 0x1c69, 0x1c5d,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5c, 0x1c7d,
0x1c68, 0x1c64, 0x1c71, 0x1c5e, 0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a,
0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5c, 0x1c7d, 0x1c68, 0x1c64, 0x1c71, 0x1c5e, 0x1c6e,
0x1c71, 0x1c70, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b,
0x1c5a, 0x1c6f, 0x1c5f, 0x1c6a, 0x1c6e, 0x20, 0x1c5c, 0x1c7d, 0x1c68, 0x1c64, 0x1c71, 0x1c5e,
0x1c6e, 0x1c71, 0x1c70, 0x20, 0x1c65, 0x1c64, 0x1c5b, 0x1c69, 0x1c5d, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c66, 0x1c5f, 0x1c63, 0x1c5f, 0x1c6d, 0x1c64, 0x1c7c, 0x1c5f, 0x1c5e, 0x1c6e,
0x1c63, 0x1c74, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c66,
0x1c5f, 0x1c63, 0x1c5f, 0x1c6d, 0x1c64, 0x1c7c, 0x1c5f, 0x1c5e, 0x1c6e, 0x1c63, 0x1c74, 0x1c64,
0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c66, 0x1c5f, 0x1c63, 0x1c5f, 0x1c6d, 0x1c64, 0x1c7c, 0x1c5f, 0x1c5e, 0x1c6e,
0x1c63, 0x1c74, 0x1c64, 0x1c6d, 0x1c5f, 0x1c71, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c6e, 0x1c60, 0x1c65, 0x1c64, 0x1c60, 0x1c5f,
0x1c71, 0x20, 0x1c6f, 0x1c68, 0x1c5a, 0x1c65, 0x1c5f, 0x1c71, 0x1c5b, 0x20, 0x1c5a, 0x1c60,
0x1c5b, 0x1c5a, 0x1c62, 0x1c6e, 0x1c60, 0x1c65, 0x1c64, 0x1c60, 0x1c5f, 0x1c71, 0x20, 0x1c6f,
0x1c68, 0x1c5a, 0x1c65, 0x1c5f, 0x1c71, 0x1c5b, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c62, 0x1c6e, 0x1c60, 0x1c65, 0x1c64, 0x1c60, 0x1c5f,
0x1c71, 0x20, 0x1c6f, 0x1c68, 0x1c5a, 0x1c65, 0x1c5f, 0x1c71, 0x1c5b, 0x20, 0x1c65, 0x1c64,
0x1c67, 0x1c5f, 0x1c5c, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c71, 0x1c64, 0x1c6d, 0x1c69,
0x1c6f, 0x1c77, 0x1c5f, 0x1c69, 0x1c71, 0x1c70, 0x1c5e, 0x1c6e, 0x1c78, 0x1c70, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x1c71, 0x1c64, 0x1c6d, 0x1c69, 0x1c6f, 0x1c77, 0x1c5f, 0x1c69, 0x1c71,
0x1c70, 0x1c5e, 0x1c6e, 0x1c78, 0x1c70, 0x20, 0x1c62, 0x1c5f, 0x1c71, 0x1c5a, 0x1c60, 0x20,
0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c71, 0x1c64, 0x1c6d, 0x1c69, 0x1c6f, 0x1c77, 0x1c5f, 0x1c69,
0x1c71, 0x1c70, 0x1c5e, 0x1c6e, 0x1c78, 0x1c70, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64, 0x20, 0x1c6f,
0x1c64, 0x1c6d, 0x1c6e, 0x1c68, 0x1c6e, 0x20, 0x1c5f, 0x1c68, 0x20, 0x1c62, 0x1c64, 0x1c60,
0x1c6e, 0x1c5e, 0x1c5a, 0x1c71, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c65, 0x20,
0x1c74, 0x1c64, 0x20, 0x1c6f, 0x1c64, 0x1c6d, 0x1c6e, 0x1c68, 0x1c6e, 0x20, 0x1c5f, 0x1c68,
0x20, 0x1c62, 0x1c64, 0x1c60, 0x1c6e, 0x1c5e, 0x1c5a, 0x1c71, 0x20, 0x1c62, 0x1c5f, 0x1c71,
0x1c5a, 0x1c60, 0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6e, 0x1c65, 0x20, 0x1c74, 0x1c64,
0x20, 0x1c6f, 0x1c64, 0x1c6d, 0x1c6e, 0x1c68, 0x1c6e, 0x20, 0x1c5f, 0x1c68, 0x20, 0x1c62,
0x1c64, 0x1c60, 0x1c6e, 0x1c5e, 0x1c5a, 0x1c71, 0x20, 0x1c65, 0x1c64, 0x1c67, 0x1c5f, 0x1c5c,
0x20, 0x1c5a, 0x1c60, 0x1c5b, 0x1c5a, 0x1c6d, 0x1c69, 0x1c60, 0x1c5a, 0x1c71, 0x20, 0x1c5a,
0x1c60, 0x1c5b, 0x1c5a, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x63,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x63, 0x72, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x41, 0x63, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x41, 0x66, 0x67, 0x68, 0xe0, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xc0, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xc0,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xc0, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x6f,
0x6e, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x74, 0x7a,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019,
0xc0, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x41, 0x6d, 0x61, 0x74, 0x7a, 0xf2, 0x6e, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x6d, 0x61, 0x74, 0x7a, 0xf2, 0x6e,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x6d, 0x61, 0x74, 0x7a, 0xf2,
0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x74, 0x7a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x74, 0x7a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x20, 0x55, 0x53, 0x41, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x20, 0x55,
0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x73,
0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x73, 0x6f, 0x73, 0x20, 0x55, 0x53,
0x41, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x65, 0x73, 0x20, 0x50, 0x65,
0x64, 0x72, 0x6f, 0x73, 0x6f, 0x73, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x4d, 0x6f, 0x6e,
0x74, 0x65, 0x73, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f, 0x73, 0x6f, 0x73,
0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x50, 0x61, 0x74, 0x7a, 0xec, 0x66, 0x69, 0x63, 0x75, 0x20,
0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50,
0x61, 0x74, 0x7a, 0xec, 0x66, 0x69, 0x63, 0x75, 0x20, 0x55, 0x53, 0x41,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x61, 0x74, 0x7a, 0xec, 0x66, 0x69,
0x63, 0x75, 0x20, 0x55, 0x53, 0x41, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41,
0x6b, 0x74, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74,
0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41,
0x6b, 0x74, 0x6f, 0x62, 0x65, 0x4f, 0x72, 0x61, 0x20, 0xe0, 0x72, 0x61,
0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0xe0, 0x72, 0x61, 0x62, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0xe0, 0x72, 0x61, 0x62,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f,
0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x72, 0x6d, 0xe8,
0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x72,
0x6d, 0xe8, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x72, 0x6d,
0xe8, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x69, 0x63, 0x75, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x69,
0x63, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74,
0x69, 0x63, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x74, 0x7a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20,
0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x74,
0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x75, 0x2d, 0x6f,
0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x75, 0x2d, 0x6f,
0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20,
0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x75, 0x2d, 0x6f, 0x74, 0x7a, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0,
0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c,
0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20, 0x6f,
0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0xe0, 0x6c, 0x69, 0x61, 0x20,
0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x67, 0x69, 0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x67, 0x69, 0xe0,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x67, 0x69, 0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x72, 0x61, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x72,
0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x72, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x42, 0x68, 0x75, 0x74, 0xe0, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x42, 0x6f,
0x6c, 0xec, 0x76, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x42, 0x72, 0x61, 0x73, 0xec, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x42, 0x72, 0x61, 0x73, 0xec, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x42,
0x72, 0x61, 0x73, 0xec, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x43, 0x61, 0x62,
0x75, 0x20, 0x42, 0x69, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x73, 0x75, 0x20, 0x43, 0x61, 0x62, 0x75, 0x20, 0x42, 0x69, 0x72, 0x64,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x43, 0x61, 0x62, 0x75, 0x20, 0x42,
0x69, 0x72, 0x64, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43,
0x61, 0x73, 0x65, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x73, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x54, 0x7a, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x73, 0x75, 0x20, 0x54, 0x7a, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x54, 0x7a, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x20, 0x54, 0x7a, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x20, 0x54, 0x7a, 0x69, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x20, 0x54, 0x7a, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x4e, 0x61, 0x74, 0x69, 0x76, 0x69, 0x64, 0x61, 0x64, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73,
0x75, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0xf2,
0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20,
0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x43, 0x6f, 0x6c, 0xf2, 0x6d, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c,
0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d,
0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72,
0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc, 0x73,
0x75, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x7a, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x65,
0x73, 0x74, 0x72, 0x65, 0x6d, 0x75, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x65, 0x20, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x75, 0x20, 0x28,
0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x29,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73,
0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73,
0x20, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73,
0x75, 0x6c, 0x61, 0x73, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73,
0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e,
0x74, 0x7a, 0x65, 0x73, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x20, 0x54, 0x65, 0x72, 0x72, 0x61, 0x73, 0x20, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x20, 0x65, 0x20, 0x61, 0x6e,
0x74, 0xe0, 0x72, 0x74, 0x69, 0x63, 0x61, 0x73, 0x20, 0x66, 0x72, 0x61,
0x6e, 0x74, 0x7a, 0x65, 0x73, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x47, 0x61, 0x6c, 0xe0, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20,
0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x75, 0x20, 0x64, 0x65,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65,
0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x74,
0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe0,
0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0xe0, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x74, 0x7a,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x47, 0x75, 0xe0, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x47, 0x75, 0x6c, 0x66, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xec, 0x73, 0x75,
0x6c, 0x61, 0x73, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x73, 0x20, 0xec, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xec, 0x73,
0x75, 0x6c, 0x61, 0x73, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc,
0x6e, 0x64, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x4f, 0x74, 0x7a, 0xe8, 0x61, 0x6e, 0x75, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x6e, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x74, 0x7a, 0x69, 0x6e, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0xe8, 0x73, 0x69, 0x61, 0x20, 0x74, 0x7a, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0xe8, 0x73, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0xe8, 0x73, 0x69, 0x61,
0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x49, 0x72, 0xe0,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x49, 0x72, 0xe0, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x49, 0x72, 0xe0, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x75, 0x20, 0x47, 0x69, 0x61, 0x70, 0x6f, 0x6e, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x47, 0x69, 0x61, 0x70, 0x6f,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x47, 0x69, 0x61, 0x70, 0x6f,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x4b, 0x61, 0x7a, 0xe0, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0xe0,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x74, 0x7a, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x63, 0x6f, 0x72, 0x65,
0x61, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x4b, 0x69, 0x72, 0x67, 0x68, 0xec, 0x7a, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x61, 0x6e, 0x6b,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20,
0xcc, 0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x4c, 0xec, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x63, 0x61, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x4d, 0x61, 0x6c, 0xe8, 0x73, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x73, 0x20, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x73, 0x61, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73,
0x75, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x75, 0x72, 0xec, 0x74, 0x7a, 0x69, 0x75, 0x73, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x75, 0x72, 0xec,
0x74, 0x7a, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x75, 0x72, 0xec, 0x74, 0x7a, 0x69, 0x75, 0x73, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x61, 0x74, 0x7a,
0xec, 0x66, 0x69, 0x63, 0x75, 0x20, 0x28, 0x4d, 0xe8, 0x73, 0x73, 0x69,
0x63, 0x75, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50,
0x61, 0x74, 0x7a, 0xec, 0x66, 0x69, 0x63, 0x75, 0x20, 0x28, 0x4d, 0xe8,
0x73, 0x73, 0x69, 0x63, 0x75, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50,
0x61, 0x74, 0x7a, 0xec, 0x66, 0x69, 0x63, 0x75, 0x20, 0x28, 0x4d, 0xe8,
0x73, 0x73, 0x69, 0x63, 0x75, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x55, 0x6c, 0xe0, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe0, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe0, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73,
0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0xe0, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x20, 0x4e,
0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x20, 0x4e, 0x6f, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69,
0x61, 0x20, 0x4e, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x4e,
0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x4e, 0x6f, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x4e, 0x6f,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x72,
0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65,
0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x69, 0x75, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0xcc, 0x73,
0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0xcc, 0x73, 0x75, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc,
0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e,
0x6e, 0x61, 0x73, 0x20, 0x53, 0x65, 0x74, 0x65, 0x6e, 0x74, 0x72, 0x69,
0x6f, 0x6e, 0x61, 0x6c, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0xe0, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0xe0, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0xe0,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x20, 0x50, 0xe0, 0x70, 0x75, 0x61, 0x20, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x20, 0x4e, 0x6f, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x73, 0x75, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x61, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61,
0x73, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73,
0x75, 0x6c, 0x61, 0x73, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x46,
0x65, 0x6e, 0xec, 0x74, 0x7a, 0x69, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x73, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x6f, 0x68, 0x6e,
0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x79,
0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20, 0x52, 0x65,
0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0x53, 0x65,
0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc,
0x73, 0x75, 0x6c, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x20,
0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x53, 0x75, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x75, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x61, 0x67, 0xec, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x75, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x43, 0x68, 0x75,
0x75, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe8, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0xe8, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0xe8, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72, 0x61,
0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x73, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f,
0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x73, 0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019, 0x55, 0x7a, 0x62, 0xe8, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x2019,
0x55, 0x7a, 0x62, 0xe8, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x73, 0x2019, 0x55, 0x7a, 0x62, 0xe8, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x73, 0x75,
0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x74, 0x7a,
0x75, 0x65, 0x6c, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61,
0x20, 0x6c, 0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x61, 0x73, 0x20, 0xcc, 0x73, 0x75, 0x6c,
0x61, 0x73, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6c,
0x65, 0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x4f, 0x72, 0x61, 0x20, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x6c, 0x65,
0x67, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x75, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x410, 0x432, 0x433, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x2d, 0x430,
0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x408, 0x443, 0x436, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434,
0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d,
0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x430, 0x444, 0x440, 0x438,
0x447, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x459, 0x430, 0x441, 0x43a, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x459, 0x430, 0x441, 0x43a, 0x430, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e,
0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438,
0x447, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x446, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x438,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438,
0x447, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e,
0x447, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a,
0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430,
0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d,
0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e,
0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x430, 0x446,
0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x421, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438,
0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a,
0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x430,
0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x458, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x458, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438, 0x458, 0x430, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x440, 0x430, 0x431, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430,
0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x440,
0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x408, 0x435, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x408, 0x435, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441,
0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d,
0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441,
0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e,
0x20, 0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x438,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20,
0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x443,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435,
0x458, 0x45f, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410, 0x437,
0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x410, 0x437, 0x43e, 0x440, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x410, 0x437, 0x43e, 0x440, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x410,
0x437, 0x43e, 0x440, 0x438, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434,
0x435, 0x448, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433,
0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411,
0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x443, 0x442,
0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x43e, 0x43b, 0x438,
0x432, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x440,
0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458,
0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x411, 0x440, 0x443, 0x43d, 0x435, 0x458, 0x20, 0x414, 0x430, 0x440,
0x443, 0x441, 0x430, 0x43b, 0x443, 0x43c, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x430, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x430, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x417, 0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x430, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x43c, 0x43e, 0x440,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430, 0x43c, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427,
0x438, 0x43b, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b,
0x435, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b, 0x435, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41a, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438,
0x43d, 0x435, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438,
0x43d, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x427, 0x43e, 0x458, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x43e, 0x458, 0x431, 0x430, 0x43b,
0x441, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427, 0x43e, 0x458,
0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x411, 0x43e, 0x436, 0x438, 0x45b,
0x43d, 0x43e, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x28, 0x41a, 0x435,
0x43b, 0x438, 0x43d, 0x433, 0x29, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431,
0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443,
0x43a, 0x43e, 0x432, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432,
0x430, 0x2c, 0x20, 0x43f, 0x43e, 0x43b, 0x443, 0x2d, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x414, 0x435,
0x458, 0x432, 0x438, 0x441, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x414, 0x438,
0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x423, 0x440, 0x432, 0x438, 0x43b, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438,
0x20, 0x442, 0x438, 0x43c, 0x43e, 0x440, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x423, 0x441, 0x43a, 0x440, 0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440,
0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440, 0x448, 0x45a, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x415, 0x43a, 0x432, 0x430, 0x434,
0x43e, 0x440, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434,
0x45a, 0x435, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x435, 0x435, 0x432,
0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421,
0x440, 0x435, 0x434, 0x45a, 0x435, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a,
0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e,
0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x412, 0x440, 0x435, 0x43c, 0x435, 0x20, 0x434, 0x430,
0x459, 0x435, 0x433, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x43a, 0x430, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e,
0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a,
0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a,
0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a,
0x43b, 0x430, 0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x424, 0x438, 0x45f, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x424, 0x438, 0x45f, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424,
0x438, 0x45f, 0x438, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x441, 0x43a,
0x430, 0x20, 0x413, 0x432, 0x430, 0x458, 0x430, 0x43d, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x413, 0x430, 0x43b, 0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x430, 0x43c, 0x431, 0x438, 0x458,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438,
0x458, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437,
0x438, 0x458, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443,
0x437, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x20, 0x43f, 0x43e, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x438, 0x447, 0x443,
0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430,
0x43d, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43b, 0x438, 0x432, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430,
0x458, 0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458,
0x441, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458, 0x441, 0x43a,
0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e,
0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d,
0x433, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x425, 0x43e,
0x432, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x418, 0x43d, 0x434, 0x438, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x418, 0x43d, 0x434, 0x43e, 0x43a, 0x438, 0x43d, 0x430, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x426, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x43d, 0x43e, 0x2d, 0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442,
0x43e, 0x447, 0x43d, 0x43e, 0x2d, 0x438, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437,
0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d, 0x438, 0x43d, 0x434, 0x43e, 0x43d,
0x435, 0x437, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x418, 0x440, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418,
0x440, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x430,
0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443, 0x446, 0x43a, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x440,
0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x418,
0x437, 0x440, 0x430, 0x435, 0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43f, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x430,
0x43f, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408,
0x430, 0x43f, 0x430, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x442, 0x440, 0x43e,
0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x43e, 0x2d, 0x43a, 0x430,
0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x43e, 0x2d, 0x43a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x43e, 0x2d, 0x43a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430,
0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e,
0x440, 0x435, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41a, 0x43e, 0x440, 0x435, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41a, 0x43e, 0x440, 0x435, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x448,
0x440, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e,
0x458, 0x430, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x441, 0x442,
0x440, 0x432, 0x430, 0x20, 0x41b, 0x430, 0x458, 0x43d, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e, 0x432,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425,
0x43e, 0x432, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43a, 0x430, 0x43e, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x440, 0x430, 0x447, 0x443, 0x43d, 0x430, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x43d, 0x430, 0x41c, 0x430, 0x433, 0x430, 0x434,
0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43b, 0x435, 0x437, 0x438, 0x458,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x43b, 0x434, 0x438,
0x432, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x440, 0x43a,
0x438, 0x437, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x440, 0x448,
0x430, 0x43b, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440, 0x438, 0x446,
0x438, 0x458, 0x443, 0x441, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x430,
0x443, 0x440, 0x438, 0x446, 0x438, 0x458, 0x443, 0x441, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440, 0x438, 0x446, 0x438, 0x458, 0x443, 0x441,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41c, 0x43e, 0x441, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x438, 0x20, 0x41f, 0x430, 0x446,
0x438, 0x444, 0x438, 0x43a, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x438,
0x20, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x43a, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x438, 0x20, 0x41f,
0x430, 0x446, 0x438, 0x444, 0x438, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x20,
0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423,
0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e,
0x440, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41c, 0x438, 0x458,
0x430, 0x43d, 0x43c, 0x430, 0x440, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d,
0x430, 0x443, 0x440, 0x443, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x435,
0x43f, 0x430, 0x43b, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435,
0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x438, 0x20,
0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417, 0x435,
0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x40a, 0x443,
0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x438,
0x443, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444,
0x43e, 0x43b, 0x43a, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20,
0x41e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x43e, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20,
0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20,
0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20,
0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e,
0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440,
0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e,
0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x43b, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a,
0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f,
0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430,
0x43b, 0x430, 0x443, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43f,
0x443, 0x430, 0x20, 0x41d, 0x43e, 0x432, 0x430, 0x20, 0x413, 0x432, 0x438, 0x43d,
0x435, 0x458, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440,
0x430, 0x433, 0x432, 0x430, 0x458, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435,
0x440, 0x443, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x424,
0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x424, 0x435, 0x43d, 0x438, 0x43a, 0x441, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f,
0x458, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f,
0x458, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41f, 0x43e, 0x43d, 0x43f, 0x435, 0x458, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458, 0x430, 0x43d, 0x448,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x420, 0x435, 0x438, 0x43d,
0x438, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x420, 0x43e, 0x442,
0x435, 0x440, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x445,
0x430, 0x43b, 0x438, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430,
0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421,
0x435, 0x458, 0x448, 0x435, 0x43b, 0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x421, 0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x441, 0x43a, 0x430,
0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x408, 0x443, 0x436, 0x43d, 0x430, 0x20, 0x40f, 0x43e, 0x440, 0x45f, 0x438,
0x458, 0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x421, 0x443, 0x440, 0x438,
0x43d, 0x430, 0x43c, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x428, 0x43e, 0x432,
0x430, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x445, 0x438, 0x442,
0x438, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435,
0x458, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435,
0x458, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435,
0x458, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x422, 0x430, 0x45f, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430,
0x443, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20,
0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x427,
0x443, 0x443, 0x43a, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440,
0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440,
0x43a, 0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x43b,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x422, 0x443,
0x432, 0x430, 0x43b, 0x443, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x440,
0x443, 0x433, 0x432, 0x430, 0x458, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423,
0x440, 0x443, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x43b, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442,
0x430, 0x43d, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443,
0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c,
0x435, 0x412, 0x435, 0x43d, 0x435, 0x446, 0x443, 0x435, 0x43b, 0x430, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x43a, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430,
0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e,
0x43a, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20,
0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440,
0x430, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433,
0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x441, 0x442, 0x43e, 0x43a,
0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412, 0x435, 0x458, 0x43a, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x412,
0x430, 0x43b, 0x438, 0x441, 0x20, 0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d,
0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x440,
0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441, 0x43a, 0x2c,
0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435,
0x408, 0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440,
0x433, 0x20, 0x432, 0x440, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x435,
0x43c, 0x435, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431,
0x443, 0x440, 0x433, 0x2c, 0x20, 0x43b, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x435, 0x43c, 0x435, 0x417, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x2d,
0x430, 0x444, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x459, 0x430,
0x441, 0x43a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410,
0x459, 0x430, 0x441, 0x43a, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43c, 0x430, 0x437,
0x43e, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435,
0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20,
0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x446, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440,
0x430, 0x43b, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x438,
0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c,
0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a,
0x43e, 0x20, 0x438, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421,
0x458, 0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438,
0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435,
0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a,
0x43e, 0x20, 0x43f, 0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f,
0x43b, 0x430, 0x43d, 0x438, 0x43d, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458,
0x435, 0x432, 0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447,
0x43a, 0x43e, 0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432,
0x435, 0x440, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e,
0x20, 0x43f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x458, 0x435, 0x432, 0x435, 0x440, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43f, 0x430,
0x446, 0x438, 0x444, 0x438, 0x447, 0x43a, 0x43e, 0x20, 0x43b, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43f, 0x438,
0x458, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x43f,
0x438, 0x458, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x430, 0x431, 0x438, 0x458,
0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x430,
0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x430, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d,
0x442, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x440, 0x43c, 0x435,
0x43d, 0x438, 0x458, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x408, 0x435, 0x440, 0x43c, 0x435, 0x43d, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x440, 0x43c, 0x435, 0x43d, 0x438,
0x458, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x441,
0x43a, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x441, 0x43a, 0x43e, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x410, 0x443, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a,
0x43e, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x43d, 0x43e, 0x20,
0x437, 0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x438, 0x441,
0x442, 0x43e, 0x447, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x443, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x438, 0x458, 0x441, 0x43a, 0x43e, 0x20, 0x437, 0x430, 0x43f, 0x430,
0x434, 0x43d, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x410, 0x437, 0x435, 0x440, 0x431, 0x435, 0x458,
0x45f, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x410,
0x437, 0x435, 0x440, 0x431, 0x435, 0x458, 0x45f, 0x430, 0x43d, 0x2c, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x410, 0x437, 0x43e, 0x440, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x410, 0x437, 0x43e, 0x440, 0x438, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x430, 0x43d,
0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x458, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x430, 0x437, 0x438,
0x43b, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x411, 0x440, 0x443, 0x43d, 0x435,
0x458, 0x20, 0x414, 0x430, 0x440, 0x443, 0x441, 0x430, 0x43b, 0x443, 0x43c, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b, 0x435, 0x43d,
0x43e, 0x440, 0x442, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432,
0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417, 0x435, 0x43b,
0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442,
0x440, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417,
0x435, 0x43b, 0x435, 0x43d, 0x43e, 0x440, 0x442, 0x441, 0x43a, 0x430, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442, 0x430,
0x43c, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x430, 0x442,
0x430, 0x43c, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b, 0x435, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x438, 0x43b, 0x435, 0x2c, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41a, 0x438, 0x43d, 0x435, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41a, 0x438, 0x43d, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x43e, 0x458,
0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x427, 0x43e, 0x458, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x411, 0x43e, 0x436, 0x438, 0x45b, 0x43d, 0x43e, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x432, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41a, 0x43e, 0x43a, 0x43e, 0x441, 0x43e, 0x432, 0x430, 0x20, 0x28, 0x41a, 0x438,
0x43b, 0x438, 0x43d, 0x433, 0x29, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x438, 0x458, 0x430, 0x2c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443,
0x43a, 0x43e, 0x432, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x443, 0x43a, 0x43e, 0x432,
0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x43f, 0x43e,
0x43b, 0x443, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41a, 0x443, 0x431, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x414, 0x438,
0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2019, 0x418, 0x440, 0x432, 0x438, 0x43b, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447,
0x43d, 0x438, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440, 0x448, 0x45a, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x423, 0x441, 0x43a, 0x440, 0x448, 0x45a, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x432, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x45a,
0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x43e, 0x435,
0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x43e, 0x435, 0x432, 0x440, 0x43e,
0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d,
0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x43e, 0x435, 0x432, 0x440, 0x43e, 0x43f, 0x441,
0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430, 0x43d, 0x434,
0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x430,
0x43d, 0x434, 0x441, 0x43a, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x432, 0x430,
0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x424, 0x438, 0x45f, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x424, 0x438, 0x45f, 0x438, 0x2c, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x413, 0x430,
0x43c, 0x431, 0x438, 0x458, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x458, 0x430, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x413, 0x440, 0x443, 0x437, 0x438, 0x458, 0x430, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x43e, 0x432, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x421, 0x440, 0x435, 0x434, 0x45a, 0x435, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x20, 0x43f, 0x43e, 0x20, 0x413, 0x440, 0x438,
0x43d, 0x438, 0x447, 0x443, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x418, 0x441, 0x442, 0x43e, 0x447, 0x43d, 0x438, 0x20,
0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x417,
0x430, 0x43f, 0x430, 0x434, 0x43d, 0x438, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x430, 0x432, 0x430, 0x458, 0x441,
0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x43a, 0x43e, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a, 0x43e, 0x43d, 0x433, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x43d, 0x433, 0x20, 0x41a,
0x43e, 0x43d, 0x433, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x425, 0x43e, 0x432, 0x434, 0x2c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x418, 0x440, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x418, 0x440, 0x430, 0x43d, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440, 0x43a, 0x443,
0x446, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x440,
0x43a, 0x443, 0x446, 0x43a, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x418, 0x437, 0x440, 0x430, 0x435,
0x43b, 0x441, 0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43f, 0x430, 0x43d, 0x441,
0x43a, 0x43e, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41a, 0x43e, 0x440, 0x435, 0x458, 0x441, 0x43a, 0x43e,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441,
0x43a, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430,
0x441, 0x43d, 0x43e, 0x458, 0x430, 0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x458, 0x430,
0x440, 0x441, 0x43a, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41b, 0x438, 0x43d, 0x438, 0x458, 0x441,
0x43a, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e,
0x432, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41b, 0x43e, 0x440,
0x434, 0x20, 0x425, 0x43e, 0x432, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c,
0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443,
0x440, 0x438, 0x446, 0x438, 0x458, 0x443, 0x441, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41c, 0x430, 0x443, 0x440, 0x438, 0x446, 0x438, 0x458, 0x443,
0x441, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x438,
0x20, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x43a, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x447, 0x43a, 0x438,
0x20, 0x41f, 0x430, 0x446, 0x438, 0x444, 0x438, 0x43a, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423,
0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x43b, 0x430, 0x43d, 0x20, 0x411, 0x430,
0x442, 0x43e, 0x440, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x441, 0x43a, 0x432,
0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x43e, 0x441,
0x43a, 0x432, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41c, 0x458, 0x430, 0x43d, 0x43c, 0x430,
0x440, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x430, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430,
0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x430,
0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x458, 0x430, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x40a,
0x443, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x40a, 0x443, 0x444, 0x430, 0x443, 0x43d,
0x434, 0x43b, 0x435, 0x43d, 0x434, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x438, 0x458, 0x443,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x43e, 0x441, 0x442,
0x440, 0x432, 0x43e, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x43e, 0x441, 0x442, 0x440, 0x432,
0x43e, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440,
0x438, 0x458, 0x435, 0x43c, 0x435, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x43e, 0x20,
0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x2c, 0x20, 0x459, 0x435, 0x442,
0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x435,
0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e,
0x440, 0x43e, 0x45a, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20,
0x41d, 0x43e, 0x440, 0x43e, 0x45a, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432,
0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438,
0x440, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41d,
0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x2c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41e, 0x43c, 0x441, 0x43a, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f, 0x430, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x41f, 0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f,
0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x41f, 0x430, 0x440, 0x430, 0x433, 0x432, 0x430, 0x458, 0x2c,
0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x41f, 0x435, 0x440, 0x443, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x424, 0x438, 0x43b,
0x438, 0x43f, 0x438, 0x43d, 0x438, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x438, 0x43d, 0x438, 0x2c, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458, 0x435, 0x440, 0x20, 0x438, 0x20, 0x41c,
0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x435, 0x43d, 0x20, 0x41f, 0x458, 0x435, 0x440, 0x20, 0x438,
0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x2c, 0x20, 0x459, 0x435,
0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41f,
0x43e, 0x43d, 0x43f, 0x435, 0x458, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x41f, 0x458, 0x43e, 0x43d, 0x433, 0x458, 0x430, 0x43d, 0x448, 0x43a, 0x43e,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x420, 0x435, 0x443, 0x43d,
0x438, 0x43e, 0x43d, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421,
0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d, 0x2c, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x421, 0x438, 0x43d,
0x433, 0x430, 0x43f, 0x443, 0x440, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x2c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x422, 0x430, 0x458, 0x43f, 0x435, 0x458, 0x2c, 0x20, 0x459,
0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435,
0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a,
0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x427, 0x443, 0x43a,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a,
0x43c, 0x435, 0x43d, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x440, 0x443, 0x433, 0x432,
0x430, 0x458, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434,
0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x440,
0x443, 0x433, 0x432, 0x430, 0x458, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x423, 0x437, 0x431, 0x435,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435,
0x43c, 0x435, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d,
0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458,
0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442,
0x443, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412,
0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x2c, 0x20,
0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x2c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x412, 0x43e, 0x43b, 0x433, 0x43e, 0x433,
0x440, 0x430, 0x434, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x43e,
0x20, 0x412, 0x435, 0x458, 0x43a, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c,
0x435, 0x43e, 0x441, 0x442, 0x440, 0x432, 0x430, 0x20, 0x412, 0x430, 0x43b, 0x438,
0x441, 0x20, 0x438, 0x20, 0x424, 0x443, 0x442, 0x443, 0x43d, 0x430, 0x20, 0x432,
0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a, 0x443, 0x442, 0x441,
0x43a, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d,
0x43e, 0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x430, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435, 0x20,
0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x434, 0x43d, 0x43e, 0x20, 0x432, 0x440, 0x438,
0x458, 0x435, 0x43c, 0x435, 0x408, 0x435, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438,
0x43d, 0x431, 0x443, 0x440, 0x433, 0x2c, 0x20, 0x459, 0x435, 0x442, 0x45a, 0x435,
0x20, 0x432, 0x440, 0x438, 0x458, 0x435, 0x43c, 0x435, 0x41, 0x6b, 0x72, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b, 0x72, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x6b, 0x72, 0x65, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x76, 0x67, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x2d,
0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x2d, 0x61,
0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4a, 0x75, 0x17e, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b, 0x61, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b,
0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x69, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e,
0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65,
0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x53, 0x6a, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c, 0x61, 0x6e, 0x69, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6a, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x70, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x6b, 0x76, 0x61, 0x74, 0x61, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x6b, 0x76, 0x61, 0x74, 0x61, 0x75, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x6b, 0x76, 0x61, 0x74, 0x61, 0x75, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x6b, 0x75,
0x74, 0x6f, 0x62, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b,
0x75, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x6b,
0x75, 0x74, 0x6f, 0x62, 0x65, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4a, 0x65, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x69, 0x73,
0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61,
0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x161, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x6a, 0x61, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c,
0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42,
0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a,
0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x75, 0x6d, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f,
0x72, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65,
0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x65,
0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73,
0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x61, 0x6d,
0x6f, 0x72, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c,
0x61, 0x74, 0x61, 0x6d, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x10c, 0x61, 0x74, 0x61, 0x6d, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x10c, 0x61, 0x74, 0x61, 0x6d, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c,
0x69, 0x6c, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c,
0x69, 0x6c, 0x65, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c,
0x69, 0x6c, 0x65, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x61,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x61,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x10c, 0x6f,
0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x10c, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x42, 0x6f, 0x17e, 0x69, 0x107, 0x6e, 0x6f,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76, 0x61, 0x20,
0x28, 0x4b, 0x69, 0x6c, 0x69, 0x6e, 0x67, 0x29, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a,
0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b,
0x75, 0x6b, 0x6f, 0x76, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x6b, 0x6f,
0x76, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x61, 0x20,
0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x70, 0x6f, 0x6c, 0x75,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4b, 0x75, 0x62, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4b, 0x75, 0x62, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x44, 0x65, 0x6a, 0x76, 0x69, 0x73, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x44, 0x69, 0x6d, 0x6f, 0x6e, 0x20, 0x64, 0x2019, 0x49, 0x72,
0x76, 0x69, 0x6c, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72,
0x161, 0x6e, 0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161,
0x6e, 0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e,
0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x6f, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74,
0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x56, 0x72, 0x65, 0x6d, 0x65, 0x20, 0x64, 0x61,
0x6c, 0x6a, 0x65, 0x67, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x6b, 0x61, 0x20,
0x45, 0x76, 0x72, 0x6f, 0x70, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x61,
0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69, 0x2c,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b,
0x61, 0x20, 0x47, 0x76, 0x61, 0x6a, 0x61, 0x6e, 0x61, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73,
0x6b, 0x6f, 0x20, 0x6a, 0x75, 0x17e, 0x6e, 0x6f, 0x20, 0x69, 0x20, 0x61,
0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47,
0x61, 0x6d, 0x62, 0x69, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x61,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a,
0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x6f, 0x76, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76,
0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65,
0x64, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x20, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x69, 0x10d, 0x75, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x47,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x47,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x69, 0x20, 0x47, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x69, 0x20,
0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x69, 0x20,
0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x5a, 0x61, 0x6c, 0x69, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b,
0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73,
0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73,
0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6f, 0x6b, 0x65,
0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x49, 0x6e, 0x64, 0x6f, 0x6b, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x6f, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x2c,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b, 0x75,
0x63, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x49, 0x7a, 0x72, 0x61, 0x65,
0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x2d, 0x6b,
0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x6f, 0x2d, 0x6b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73,
0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x6f, 0x2d, 0x6b, 0x61,
0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x72, 0x61, 0x10d, 0x75, 0x6e, 0x61, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x6e, 0x61, 0x49, 0x73, 0x74, 0x6f,
0x10d, 0x6e, 0x6f, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x6b, 0x61, 0x7a, 0x61,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x161, 0x72, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a,
0x61, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4b,
0x69, 0x72, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x160, 0x72, 0x69, 0x20, 0x4c, 0x61, 0x6e, 0x6b,
0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4c, 0x69, 0x6e, 0x69, 0x6a,
0x73, 0x6b, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x76, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x76, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x76, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6c,
0x65, 0x7a, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x69, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x72, 0x161, 0x61, 0x6c,
0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x69, 0x6a, 0x75, 0x73, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75, 0x73, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63,
0x69, 0x6a, 0x75, 0x73, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73,
0x6f, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x69, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x6b, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x69, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x69, 0x20, 0x50,
0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x2c, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76,
0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4d, 0x6a, 0x61, 0x6e, 0x6d, 0x61,
0x72, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e,
0x6f, 0x76, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a,
0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x69, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4e, 0x6f, 0x76, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x69, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e,
0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61,
0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4e, 0x69, 0x6a, 0x75, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6f,
0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x2c,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2c,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61,
0x6c, 0x61, 0x75, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50,
0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x76,
0x69, 0x6e, 0x65, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61,
0x6a, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x72,
0x61, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65,
0x72, 0x75, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65,
0x72, 0x75, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x65,
0x72, 0x75, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x69, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69, 0x2c,
0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x53, 0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x20, 0x69, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x65, 0x6e, 0x20, 0x50,
0x6a, 0x65, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f,
0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x6e,
0x20, 0x50, 0x6a, 0x65, 0x72, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50, 0x69, 0x74, 0x6b,
0x65, 0x72, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x50,
0x6f, 0x6e, 0x70, 0x65, 0x6a, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2c, 0x20, 0x6c, 0x6a, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x53, 0x65, 0x6a, 0x161, 0x65, 0x6c, 0x69, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x6f, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x75, 0x17e, 0x6e,
0x61, 0x20, 0x44, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a, 0x61, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x53, 0x75, 0x72, 0x69, 0x6e,
0x61, 0x6d, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x160, 0x6f,
0x76, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x2c, 0x20,
0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x10c, 0x75, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69,
0x6a, 0x65, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x54, 0x75, 0x76,
0x61, 0x6c, 0x75, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55,
0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a,
0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x69, 0x6a, 0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2c, 0x20, 0x6c,
0x6a, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x56, 0x65, 0x6e, 0x65, 0x63, 0x75, 0x65, 0x6c, 0x61, 0x20,
0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65,
0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2c, 0x20, 0x6c, 0x6a,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d,
0x65, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x56, 0x65,
0x6a, 0x6b, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x6f, 0x73,
0x74, 0x72, 0x76, 0x61, 0x20, 0x56, 0x61, 0x6c, 0x69, 0x73, 0x20, 0x69,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x76,
0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x69, 0x6a,
0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x6c, 0x6a, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x69, 0x6a, 0x65, 0x6d, 0x65, 0x41, 0x76, 0x67,
0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x2d, 0x61,
0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69,
0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x75, 0x17e,
0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d,
0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72,
0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x61, 0x66, 0x72, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x41, 0x6c, 0x6a, 0x61, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72,
0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65,
0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d,
0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72,
0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x69,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72,
0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70,
0x6c, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x6c, 0x61, 0x6e, 0x69, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65,
0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x70, 0x6c, 0x61, 0x6e, 0x69, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65,
0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b,
0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63,
0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53,
0x65, 0x76, 0x65, 0x72, 0x6e, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x10d,
0x6b, 0x6f, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x10d, 0x6b, 0x6f,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x41, 0x70, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x70, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x61, 0x62, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x61,
0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41,
0x72, 0x61, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x61, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x65,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4a, 0x65, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6e, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d, 0x6e,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x69, 0x73, 0x74, 0x6f, 0x10d,
0x6e, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x7a, 0x61, 0x70, 0x61, 0x64,
0x6e, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e,
0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17e, 0x61, 0x6e,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x41, 0x7a, 0x6f, 0x72, 0x69, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x41, 0x7a, 0x6f, 0x72, 0x69, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x161, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42,
0x6f, 0x6c, 0x69, 0x76, 0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a, 0x61, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x6a,
0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x61, 0x7a, 0x69,
0x6c, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x6a,
0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x61, 0x6c, 0x75, 0x6d, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74,
0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74,
0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x5a, 0x65, 0x6c, 0x65, 0x6e, 0x6f, 0x72, 0x74,
0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x10c, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x10c, 0x61, 0x74, 0x61, 0x6d, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c,
0x61, 0x74, 0x61, 0x6d, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c, 0x61,
0x74, 0x61, 0x6d, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c, 0x69, 0x6c, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x10c, 0x69, 0x6c, 0x65, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x10c, 0x69, 0x6c, 0x65, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x61, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x65, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x69, 0x6e, 0x61, 0x2c, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c,
0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x10c, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x10c, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x42, 0x6f, 0x17e, 0x69, 0x107, 0x6e, 0x6f, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x20, 0x28, 0x4b, 0x65, 0x6c, 0x69, 0x6e,
0x67, 0x29, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x6b, 0x6f,
0x76, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4b, 0x75, 0x6b, 0x6f, 0x76, 0x61, 0x20, 0x6f, 0x73,
0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x75,
0x6b, 0x6f, 0x76, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c,
0x20, 0x70, 0x6f, 0x6c, 0x75, 0x2d, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x62, 0x61, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x4b, 0x75, 0x62, 0x61, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x44, 0x65,
0x6a, 0x76, 0x69, 0x73, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x44, 0x69,
0x6d, 0x6f, 0x6e, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69,
0x20, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e, 0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74,
0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b,
0x72, 0x161, 0x6e, 0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x73, 0x6b, 0x72, 0x161, 0x6e,
0x6a, 0x61, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x45,
0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65,
0x64, 0x6e, 0x6a, 0x65, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74,
0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e,
0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e,
0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x65, 0x76,
0x72, 0x6f, 0x70, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a,
0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x2c, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46,
0x69, 0x64, 0x17e, 0x69, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x69,
0x64, 0x17e, 0x69, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x64,
0x17e, 0x69, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b,
0x61, 0x20, 0x47, 0x76, 0x61, 0x6a, 0x61, 0x6e, 0x61, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73, 0x6b, 0x6f,
0x20, 0x6a, 0x75, 0x17e, 0x6e, 0x6f, 0x20, 0x69, 0x20, 0x61, 0x6e, 0x74,
0x61, 0x72, 0x6b, 0x74, 0x69, 0x10d, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a,
0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x69,
0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x47, 0x72, 0x75, 0x7a,
0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x20, 0x70, 0x6f, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x69, 0x10d,
0x75, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x47, 0x72, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49,
0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x69,
0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x69, 0x20, 0x47,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x6c,
0x69, 0x76, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48,
0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x61,
0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74,
0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x61, 0x76, 0x61,
0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b,
0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x6e, 0x67,
0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x48, 0x6f, 0x76, 0x64, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x6e, 0x64,
0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x6e,
0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x6f, 0x6b, 0x65, 0x61, 0x6e,
0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x6e, 0x64,
0x6f, 0x6b, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x2d, 0x69, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f, 0x10d, 0x6e, 0x6f, 0x2d, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f,
0x2d, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x49, 0x72, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b,
0x75, 0x63, 0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b,
0x75, 0x63, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x72, 0x6b,
0x75, 0x63, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73,
0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x7a, 0x72, 0x61,
0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x7a,
0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a,
0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x49, 0x73, 0x74, 0x6f,
0x10d, 0x6e, 0x6f, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x5a, 0x61,
0x70, 0x61, 0x64, 0x6e, 0x6f, 0x2d, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x6f, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4b, 0x6f, 0x161, 0x72, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61,
0x72, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4b, 0x69,
0x72, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x4c, 0x61, 0x6a, 0x6e,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x76, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x76, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x76, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6c, 0x65, 0x7a,
0x69, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x69, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61,
0x72, 0x6b, 0x69, 0x7a, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61,
0x72, 0x161, 0x61, 0x6c, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73, 0x74, 0x72,
0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x63, 0x69, 0x6a, 0x75, 0x73, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x75, 0x73, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6a,
0x75, 0x73, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6f, 0x6e, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x69, 0x20,
0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x10d, 0x6b, 0x69, 0x20, 0x50, 0x61,
0x63, 0x69, 0x66, 0x69, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55,
0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4d,
0x69, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4e, 0x65, 0x70, 0x61, 0x6c, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e,
0x6f, 0x76, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x69,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75,
0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4e, 0x6a, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c,
0x65, 0x6e, 0x64, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x4f,
0x73, 0x74, 0x72, 0x76, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76,
0x6f, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x20, 0x4f, 0x73, 0x74, 0x72, 0x76, 0x6f, 0x2c, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x6a, 0x61, 0x2c, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73,
0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x2c,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47,
0x76, 0x69, 0x6e, 0x65, 0x6a, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x50, 0x65, 0x72, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x50, 0x65, 0x72, 0x75, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x65,
0x72, 0x75, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x69,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x69, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x69, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73,
0x20, 0x6f, 0x73, 0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x53, 0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x20, 0x69, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x53, 0x65, 0x6e, 0x20, 0x50, 0x6a, 0x65, 0x72, 0x20, 0x69, 0x20,
0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x69, 0x74,
0x6b, 0x65, 0x72, 0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x6f,
0x6e, 0x70, 0x65, 0x6a, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x50, 0x6a,
0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x161, 0x6b, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x52, 0x65, 0x69, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x65, 0x6a, 0x161,
0x65, 0x6c, 0x69, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x75, 0x72, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53,
0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x6b, 0x61, 0x20, 0x4f, 0x73,
0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x75,
0x17e, 0x6e, 0x61, 0x20, 0x44, 0x17e, 0x6f, 0x72, 0x64, 0x17e, 0x69, 0x6a,
0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x53, 0x75, 0x72, 0x69, 0x6e,
0x61, 0x6d, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x160, 0x6f, 0x76, 0x61,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61,
0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f,
0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2c,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x10c, 0x75, 0x75, 0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x76, 0x72, 0x65, 0x6d, 0x65, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d,
0x65, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a, 0x2c, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a,
0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x65, 0x6e, 0x65, 0x63,
0x75, 0x65, 0x6c, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2c, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72,
0x65, 0x6d, 0x65, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x56, 0x65, 0x6a, 0x6b, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x76,
0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x56, 0x61, 0x6c, 0x69, 0x73,
0x20, 0x69, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x4f, 0x73,
0x74, 0x72, 0x76, 0x61, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65,
0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x76,
0x72, 0x65, 0x6d, 0x65, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x2c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x6f, 0x20, 0x76, 0x72, 0x65, 0x6d, 0x65, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x2c, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0x6a, 0x65, 0x20, 0x76, 0x72, 0x65,
0x6d, 0x65, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f,
0x77, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x142, 0x65, 0x64, 0x6e, 0x69,
0x6f, 0x77, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x6f, 0x61, 0x66, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x144, 0x73, 0x6b, 0x69, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x144,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x61, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x61, 0x142, 0x6d, 0x61, 0x63, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x61, 0x142, 0x6d, 0x61, 0x63, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x61, 0x6d, 0x61, 0x7a, 0x14d, 0x144, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x7a, 0x14d, 0x144, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x61, 0x6d, 0x61, 0x7a,
0x14d, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x6f,
0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d,
0x65, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x63, 0x7a, 0x61,
0x73, 0x20, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f,
0x77, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x6f, 0x73, 0x74, 0x72,
0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79,
0x6b, 0x61, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61, 0x144, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x77, 0x73, 0x63, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x79, 0x6b, 0x61,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x67, 0x14d, 0x72, 0x73, 0x6b, 0x69, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x67, 0x14d, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x67, 0x14d, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61,
0x74, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x70, 0x61, 0x63,
0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x75, 0x6b, 0x74, 0x61, 0x75, 0x63,
0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x75, 0x6b, 0x74,
0x61, 0x75, 0x63, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x6b, 0x74, 0x69, 0x75, 0x62, 0x69,
0x144, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x6b,
0x74, 0x69, 0x75, 0x62, 0x69, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61,
0x74, 0x6f, 0x77, 0x79, 0x50, 0x14d, 0x142, 0x77, 0x79, 0x73, 0x65, 0x70,
0x20, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x50, 0x14d, 0x142, 0x77,
0x79, 0x73, 0x65, 0x70, 0x20, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69,
0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x50, 0x14d, 0x142, 0x77,
0x79, 0x73, 0x65, 0x70, 0x20, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x41, 0x72, 0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x41,
0x72, 0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x28, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x41, 0x72, 0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e,
0x61, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x41, 0x72, 0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61,
0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x41, 0x72,
0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x5a, 0x61, 0x63, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x41, 0x72, 0x67, 0x79, 0x6e, 0x74, 0x79, 0x6e, 0x61, 0x20, 0x5a,
0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x20, 0x28, 0x6c, 0x61,
0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41, 0x72,
0x6d, 0x79, 0x6e, 0x69, 0x6a, 0x6f, 0x41, 0x72, 0x6d, 0x79, 0x6e, 0x69,
0x6a, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41, 0x72,
0x6d, 0x79, 0x6e, 0x69, 0x6a, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x79, 0x63, 0x6b, 0x69, 0x20,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x79,
0x63, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x70, 0x6f,
0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b,
0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73,
0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61,
0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73,
0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x2d, 0x7a, 0x61,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f,
0x2d, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64,
0x6b, 0x6f, 0x77, 0x6f, 0x2d, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64,
0x17c, 0x61, 0x6e, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x65, 0x6a, 0x64, 0x17c, 0x61, 0x6e, 0x20, 0x28,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x41, 0x7a, 0x6f, 0x72, 0x79, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x41, 0x7a, 0x6f, 0x72, 0x79, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x7a, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x7a,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x42, 0x6f, 0x6c, 0x69, 0x77, 0x69, 0x6a, 0x6f, 0x42, 0x72,
0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x28,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x14d, 0x6e,
0x65, 0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a, 0x69, 0x6c, 0x14d, 0x6e, 0x64,
0x6b, 0x61, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x5a, 0x69, 0x65, 0x6c,
0x14d, 0x6e, 0x65, 0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a, 0x69, 0x6c, 0x14d,
0x6e, 0x64, 0x6b, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x5a, 0x69, 0x65, 0x6c, 0x14d, 0x6e,
0x65, 0x67, 0x6f, 0x20, 0x50, 0x72, 0x7a, 0x69, 0x6c, 0x14d, 0x6e, 0x64,
0x6b, 0x61, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x28, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x28,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x43, 0x68, 0x69, 0x6e, 0x79, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x43, 0x68, 0x69, 0x6e, 0x79, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x43, 0x7a, 0x6f, 0x6a,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x43, 0x7a, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x47, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x20, 0x57, 0x79, 0x73,
0x70, 0x61, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x6f, 0x20,
0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x6a, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x61,
0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57, 0x79, 0x73, 0x70,
0x61, 0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e,
0x6f, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x57, 0x69, 0x65, 0x6c, 0x6b,
0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x57, 0x79, 0x73, 0x70, 0x61, 0x20, 0x57, 0x69, 0x65, 0x6c,
0x6b, 0x61, 0x6e, 0x6f, 0x63, 0x6e, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x70, 0x6f, 0x73,
0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x70, 0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64,
0x6b, 0x6f, 0x77, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x77, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x77, 0x73,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x64, 0x61, 0x6c, 0x73, 0x7a, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x7a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x20, 0x28,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x79, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x69, 0x64, 0x17c, 0x69, 0x20, 0x28,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x69, 0x64, 0x17c, 0x69, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x47, 0x75, 0x6a, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e,
0x63, 0x75, 0x73, 0x6b, 0x6f, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x75, 0x73,
0x6b, 0x65, 0x20, 0x54, 0x65, 0x72, 0x79, 0x74, 0x6f, 0x72, 0x69, 0x61,
0x20, 0x50, 0x6f, 0x142, 0x65, 0x64, 0x6e, 0x69, 0x6f, 0x77, 0x65, 0x20,
0x69, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x79, 0x63, 0x7a,
0x6e, 0x65, 0x47, 0x72, 0x75, 0x7a, 0x79, 0x6a, 0x6f, 0x47, 0x72, 0x75,
0x7a, 0x79, 0x6a, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x47, 0x72, 0x75, 0x7a, 0x79, 0x6a, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x6f, 0x77, 0x65, 0x20, 0x57, 0x79, 0x73, 0x70,
0x79, 0x75, 0x6e, 0x69, 0x77, 0x65, 0x72, 0x73, 0x61, 0x6c, 0x6e, 0x65,
0x67, 0x6f, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x75, 0x47, 0x72, 0x79, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x57, 0x73, 0x63, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x61, 0x47, 0x72, 0x79, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x79, 0x6a, 0x6f, 0x20, 0x57, 0x73, 0x63, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x47, 0x72,
0x79, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x57, 0x73,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x61, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x47, 0x72, 0x79,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x5a, 0x61, 0x63,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x47, 0x72, 0x79, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x47,
0x72, 0x79, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x5a,
0x61, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x20, 0x28, 0x6c, 0x61,
0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x48, 0x61,
0x77, 0x61, 0x6a, 0x65, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x79, 0x20,
0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x48, 0x61, 0x77, 0x61, 0x6a,
0x65, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x79, 0x20, 0x28, 0x6c, 0x61,
0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x48, 0x14d,
0x6e, 0x67, 0x6b, 0x14d, 0x6e, 0x67, 0x48, 0x14d, 0x6e, 0x67, 0x6b, 0x14d,
0x6e, 0x67, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x48, 0x14d,
0x6e, 0x67, 0x6b, 0x14d, 0x6e, 0x67, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4b, 0x6f, 0x62, 0x64,
0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4b, 0x6f, 0x62,
0x64, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x69, 0x6e, 0x64, 0x79, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0xd4, 0x63, 0x65, 0x61, 0x6e, 0x20,
0x49, 0x6e, 0x64, 0x79, 0x6a, 0x73, 0x6b, 0x69, 0x69, 0x6e, 0x64, 0x6f,
0x63, 0x68, 0x69, 0x144, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x49, 0x6e, 0x64, 0x14d, 0x6e, 0x65, 0x7a, 0x79, 0x6a, 0x6f, 0x20, 0x50,
0x6f, 0x73, 0x74, 0x72, 0x7a, 0x6f, 0x64, 0x6b, 0x6f, 0x77, 0x6f, 0x49,
0x6e, 0x64, 0x14d, 0x6e, 0x65, 0x7a, 0x79, 0x6a, 0x6f, 0x20, 0x57, 0x73,
0x63, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x6f, 0x49, 0x6e, 0x64, 0x14d, 0x6e,
0x65, 0x7a, 0x79, 0x6a, 0x6f, 0x20, 0x5a, 0x61, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x6f, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x49, 0x72, 0x6b, 0x75, 0x63, 0x6b, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x28, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4a,
0x61, 0x70, 0x14d, 0x6e, 0x69, 0x6a, 0x6f, 0x4a, 0x61, 0x70, 0x14d, 0x6e,
0x69, 0x6a, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4a,
0x61, 0x70, 0x14d, 0x6e, 0x69, 0x6a, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x20, 0x50, 0x69, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x77,
0x142, 0x6f, 0x77, 0x73, 0x6b, 0x20, 0x4b, 0x61, 0x6d, 0x63, 0x7a, 0x61,
0x63, 0x6b, 0x69, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x50, 0x69, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x77, 0x142, 0x6f, 0x77, 0x73, 0x6b, 0x20, 0x4b,
0x61, 0x6d, 0x63, 0x7a, 0x61, 0x63, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72,
0x73, 0x6b, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x28, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x79, 0x6a, 0x6f, 0x4d,
0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73, 0x20,
0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29,
0x4d, 0x65, 0x6b, 0x73, 0x79, 0x6b, 0x20, 0x28, 0x63, 0x7a, 0x61, 0x73,
0x20, 0x70, 0x61, 0x63, 0x79, 0x66, 0x69, 0x63, 0x7a, 0x6e, 0x79, 0x20,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x29, 0x55, 0x142, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x55, 0x142, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x29, 0x4d, 0x6f,
0x73, 0x6b, 0x77, 0x61, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x14d,
0x6e, 0x69, 0x6a, 0x6f, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x14d, 0x6e, 0x69, 0x6a, 0x6f, 0x20, 0x28, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x4b, 0x61, 0x6c, 0x65,
0x64, 0x14d, 0x6e, 0x69, 0x6a, 0x6f, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4e, 0x6f, 0x77, 0x6f,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x4e, 0x6f,
0x77, 0x6f, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f,
0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4e, 0x6f, 0x77, 0x6f,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x28,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x46, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x79, 0x6a, 0x6f, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x46, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x28, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x20, 0x46, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x79, 0x6a, 0x6f, 0x20, 0x28, 0x6c, 0x61,
0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x65,
0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x28, 0x6c, 0x61,
0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4e, 0x6f,
0x77, 0x6f, 0x73, 0x79, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x28, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x79, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0xd4, 0x6d, 0x73, 0x6b, 0xd4, 0x6d,
0x73, 0x6b, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0xd4, 0x6d,
0x73, 0x6b, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x50, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x4e, 0x6f,
0x77, 0x6f, 0x20, 0x47, 0x77, 0x69, 0x6e, 0x65, 0x61, 0x50, 0x61, 0x72,
0x61, 0x67, 0x77, 0x61, 0x6a, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x6a, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x50,
0x65, 0x72, 0x75, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x50,
0x65, 0x72, 0x75, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20,
0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x79, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x79, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77,
0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x46, 0x79, 0x6e, 0x69, 0x6b,
0x73, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x69, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x69, 0x20, 0x4d,
0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x28, 0x6c, 0x61, 0x74,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x50, 0x6a, 0x14d,
0x6e, 0x67, 0x6a, 0x61, 0x6e, 0x67, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6b,
0x79, 0x7a, 0x79, 0x142, 0x6f, 0x72, 0x64, 0x7a, 0x6b, 0x69, 0x20, 0x73,
0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x63,
0x7a, 0x61, 0x73, 0x20, 0x6b, 0x79, 0x7a, 0x79, 0x142, 0x6f, 0x72, 0x64,
0x7a, 0x6b, 0x69, 0x20, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x53, 0x61,
0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x28,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x63, 0x7a, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x57, 0x79, 0x73, 0x70, 0x79, 0x20,
0x53, 0x61, 0x6c, 0x14d, 0x6d, 0x14d, 0x6e, 0x61, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x20, 0x50, 0x6f, 0x142, 0x65, 0x64, 0x6e, 0x69, 0x6f,
0x77, 0x6f, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20, 0x28, 0x73, 0x7a,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20, 0x28,
0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29,
0x54, 0x14d, 0x6e, 0x67, 0x61, 0x54, 0x14d, 0x6e, 0x67, 0x61, 0x20, 0x28,
0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x54, 0x14d, 0x6e, 0x67, 0x61, 0x20,
0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73,
0x29, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x79, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x79, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x79, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x55,
0x72, 0x75, 0x67, 0x77, 0x61, 0x6a, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61,
0x73, 0x29, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x6a, 0x20, 0x28, 0x6c,
0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57,
0x79, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x57, 0x142, 0x61, 0x64,
0x79, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x28, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x57, 0x142, 0x61, 0x64, 0x79, 0x77, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79, 0x20, 0x63,
0x7a, 0x61, 0x73, 0x29, 0x57, 0x6f, 0x142, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x57, 0x6f, 0x142,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f,
0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4a, 0x61, 0x6b, 0x75,
0x63, 0x6b, 0x20, 0x28, 0x73, 0x7a, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4a, 0x61,
0x6b, 0x75, 0x63, 0x6b, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x79, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x28, 0x73, 0x7a, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6f, 0x77, 0x79, 0x20, 0x63, 0x7a,
0x61, 0x73, 0x29, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x79, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x20, 0x28, 0x6c, 0x61, 0x74, 0x6f, 0x77, 0x79,
0x20, 0x63, 0x7a, 0x61, 0x73, 0x29, 0x63, 0x7a, 0x61, 0x73, 0x20, 0x6a,
0x75, 0x6b, 0x14d, 0x144, 0x73, 0x6b, 0x69, 0x627, 0x641, 0x63a, 0x627, 0x646,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x622, 0x641, 0x631, 0x64a, 0x642, 0x627, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x680, 0x631, 0x20, 0x622,
0x641, 0x631, 0x64a, 0x6aa, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x68f, 0x6a9, 0x6bb, 0x20, 0x622, 0x641, 0x631, 0x64a, 0x6aa, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x622, 0x641, 0x631, 0x64a, 0x642, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647,
0x20, 0x622, 0x641, 0x631, 0x64a, 0x642, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644,
0x647, 0x647, 0x20, 0x622, 0x641, 0x631, 0x64a, 0x642, 0x627, 0x20, 0x62c, 0x64a,
0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x644, 0x627, 0x633, 0x6aa, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x644, 0x627, 0x633, 0x6aa, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x644, 0x627, 0x633, 0x6aa, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a,
0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a,
0x645, 0x64a, 0x632, 0x648, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x64a, 0x645, 0x64a, 0x632, 0x648, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x645,
0x64a, 0x632, 0x648, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647,
0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x631,
0x6aa, 0x632, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20,
0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x67e, 0x647, 0x627, 0x699, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x647, 0x627,
0x699, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x67e, 0x647, 0x627, 0x699, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x633, 0x64a, 0x641,
0x6aa, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x633, 0x64a, 0x641, 0x6aa, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a,
0x633, 0x64a, 0x641, 0x6aa, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x67e, 0x64a, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x67e, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x67e,
0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x639, 0x631, 0x628, 0x64a, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x639, 0x631, 0x628, 0x64a, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x639, 0x631, 0x628, 0x64a, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a,
0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x631,
0x62c, 0x646, 0x67d, 0x64a, 0x646, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x631, 0x62c, 0x646, 0x67d, 0x627, 0x626, 0x646, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x631, 0x62c, 0x646,
0x67d, 0x627, 0x626, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647,
0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648,
0x644, 0x647, 0x647, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x67d, 0x64a, 0x646, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647,
0x20, 0x627, 0x631, 0x62c, 0x646, 0x67d, 0x64a, 0x646, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x648, 0x644, 0x647, 0x647, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x67d, 0x64a, 0x646,
0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x631, 0x645, 0x64a, 0x646,
0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x631, 0x645,
0x64a, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x631, 0x645, 0x64a, 0x646, 0x64a,
0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x67d, 0x644, 0x627,
0x646, 0x67d, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a,
0x67d, 0x644, 0x627, 0x646, 0x67d, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x67d, 0x644,
0x627, 0x646, 0x67d, 0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x632,
0x64a, 0x20, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d,
0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x631, 0x6aa,
0x632, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x645, 0x63a, 0x631,
0x628, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644,
0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20,
0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x645, 0x63a,
0x631, 0x628, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x680, 0x631, 0x20, 0x622, 0x633, 0x67d,
0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x68f, 0x64a,
0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a,
0x631, 0x628, 0x64a, 0x20, 0x622, 0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x633, 0x67d, 0x631, 0x64a,
0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x63a, 0x631, 0x628, 0x64a,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622,
0x633, 0x67d, 0x631, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x63a, 0x631, 0x628, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x64a,
0x62c, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x630,
0x631, 0x628, 0x627, 0x626, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x622, 0x630,
0x631, 0x628, 0x627, 0x626, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x62c, 0x64a, 0x20,
0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x632, 0x648, 0x631, 0x632, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x632, 0x648, 0x631, 0x632, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x648,
0x631, 0x632, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631,
0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x646, 0x6af, 0x644,
0x627, 0x62f, 0x64a, 0x634, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x628,
0x646, 0x6af, 0x644, 0x627, 0x62f, 0x64a, 0x634, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x646, 0x6af,
0x644, 0x627, 0x62f, 0x64a, 0x634, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646,
0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x680,
0x648, 0x67d, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x628,
0x648, 0x644, 0x648, 0x64a, 0x627, 0x626, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x628,
0x631, 0x64a, 0x633, 0x64a, 0x644, 0x64a, 0x627, 0x626, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x628, 0x631, 0x627, 0x633, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x628,
0x631, 0x627, 0x633, 0x64a, 0x644, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x628, 0x631, 0x648, 0x646, 0x627, 0x626, 0x64a, 0x20, 0x62f, 0x627, 0x631,
0x627, 0x644, 0x633, 0x644, 0x627, 0x645, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x6aa, 0x64a, 0x67e, 0x20, 0x648, 0x631, 0x68a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x6aa, 0x64a, 0x67e, 0x20, 0x648, 0x631, 0x68a, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x6aa, 0x64a, 0x67e, 0x20, 0x648, 0x631, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x686, 0x645, 0x648, 0x631, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x626, 0x67f, 0x645,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x626, 0x67f, 0x645, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x686, 0x626, 0x67f, 0x645, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646,
0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x644, 0x64a,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x644, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x686, 0x644, 0x64a, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627,
0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x64a, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x627, 0x626, 0x646, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x686, 0x627, 0x626, 0x646, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x68f,
0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x686,
0x648, 0x626, 0x64a, 0x20, 0x628, 0x64a, 0x644, 0x633, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x686, 0x648, 0x626, 0x64a, 0x20, 0x628, 0x64a, 0x644,
0x633, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x686, 0x648, 0x626, 0x64a, 0x20, 0x628, 0x64a, 0x644,
0x633, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631,
0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x631, 0x633, 0x645,
0x633, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x6aa, 0x648, 0x633, 0x20, 0x622,
0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x6aa, 0x648, 0x644, 0x645, 0x628, 0x64a, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x644, 0x645, 0x628, 0x64a, 0x627, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x6aa, 0x648, 0x644, 0x645, 0x628, 0x64a, 0x627, 0x20, 0x62c, 0x64a, 0x20,
0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x6aa, 0x648, 0x6aa, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a,
0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x6aa,
0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6aa,
0x648, 0x6aa, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20,
0x62c, 0x64a, 0x20, 0x627, 0x68c, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631,
0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x64a, 0x648, 0x628,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x64a, 0x648, 0x628,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x6aa, 0x64a, 0x648, 0x628, 0x627, 0x20, 0x62c, 0x64a, 0x20,
0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x68a, 0x64a, 0x648, 0x633, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x68a,
0x648, 0x645, 0x627, 0x646, 0x67d, 0x20, 0x62f, 0x631, 0x648, 0x64a, 0x626, 0x644,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x680, 0x631, 0x20,
0x62a, 0x64a, 0x645, 0x648, 0x631, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x64a, 0x633, 0x67d, 0x631, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a,
0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x633,
0x67d, 0x631, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x64a, 0x633, 0x67d, 0x631, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644,
0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627,
0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x6aa,
0x648, 0x627, 0x68a, 0x648, 0x631, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x64a, 0x648, 0x631,
0x67e, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x645, 0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a,
0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x64a, 0x648, 0x631,
0x67e, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20,
0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x64a, 0x648,
0x631, 0x67e, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x68c, 0x64a, 0x6aa, 0x20, 0x645,
0x634, 0x631, 0x642, 0x64a, 0x20, 0x64a, 0x648, 0x631, 0x67e, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x64a, 0x648, 0x631, 0x67e,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x64a,
0x648, 0x631, 0x67e, 0x64a, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x64a, 0x648, 0x631,
0x67e, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x641, 0x627, 0x6aa, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20,
0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x641, 0x627, 0x6aa, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20,
0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x627,
0x6aa, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644,
0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627,
0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x62c, 0x64a,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x62c, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x641, 0x62c, 0x64a, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627,
0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631, 0x627,
0x646, 0x633, 0x64a, 0x633, 0x64a, 0x20, 0x6af, 0x64a, 0x627, 0x646, 0x627, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631, 0x627, 0x646, 0x633, 0x64a,
0x633, 0x64a, 0x20, 0x68f, 0x627, 0x6a9, 0x6bb, 0x64a, 0x648, 0x20, 0x6fd, 0x20,
0x627, 0x646, 0x67d, 0x627, 0x631, 0x6aa, 0x67d, 0x6aa, 0x20, 0x648, 0x642, 0x62a,
0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x632, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x6af, 0x64a, 0x645, 0x628, 0x64a, 0x626, 0x631, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62c, 0x627, 0x631, 0x62c, 0x64a, 0x627,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62c, 0x627, 0x631, 0x62c, 0x64a,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x62c, 0x627, 0x631, 0x62c, 0x64a, 0x627, 0x20, 0x62c, 0x64a,
0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x6af, 0x644, 0x628, 0x631, 0x67d, 0x20, 0x622, 0x626, 0x64a,
0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x6af, 0x631, 0x64a, 0x646, 0x20, 0x648, 0x686, 0x20, 0x645, 0x64a, 0x646, 0x20,
0x67d, 0x627, 0x626, 0x64a, 0x645, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x6af,
0x631, 0x64a, 0x646, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x64a, 0x20, 0x6af, 0x631, 0x64a,
0x646, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642,
0x64a, 0x20, 0x6af, 0x631, 0x64a, 0x646, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20,
0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x6af,
0x631, 0x64a, 0x646, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x64a, 0x20, 0x6af, 0x631, 0x64a,
0x646, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628,
0x64a, 0x20, 0x6af, 0x631, 0x64a, 0x646, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20,
0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x62e, 0x644, 0x64a, 0x62c, 0x64a, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6af, 0x64a, 0x627,
0x646, 0x627, 0x626, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x647, 0x648, 0x627, 0x626,
0x64a, 0x20, 0x627, 0x644, 0x64a, 0x648, 0x67d, 0x64a, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x647, 0x648, 0x627, 0x626, 0x64a, 0x20, 0x627, 0x644,
0x64a, 0x648, 0x67d, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x647, 0x648, 0x627, 0x626, 0x64a,
0x20, 0x627, 0x644, 0x64a, 0x648, 0x67d, 0x64a, 0x646, 0x20, 0x62c, 0x64a, 0x20,
0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x647, 0x627, 0x646, 0x6af, 0x20, 0x6aa, 0x627, 0x646, 0x6af, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x647, 0x627, 0x646, 0x6af, 0x20, 0x6aa, 0x627, 0x646,
0x6af, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x647, 0x627, 0x646, 0x6af, 0x20, 0x6aa, 0x627, 0x646, 0x6af,
0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x647, 0x648, 0x648, 0x68a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x647, 0x648, 0x648, 0x68a, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x647,
0x648, 0x648, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627,
0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x680, 0x627, 0x631,
0x62a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x647, 0x646, 0x62f, 0x20, 0x633, 0x645, 0x646, 0x68a, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x646, 0x68a, 0x648, 0x20, 0x686,
0x627, 0x626, 0x646, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x631, 0x6aa, 0x632, 0x64a, 0x20, 0x627, 0x646, 0x68a, 0x648, 0x646, 0x64a, 0x634,
0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x680,
0x631, 0x20, 0x627, 0x646, 0x68a, 0x648, 0x646, 0x64a, 0x634, 0x64a, 0x627, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20,
0x627, 0x646, 0x68a, 0x648, 0x646, 0x64a, 0x634, 0x64a, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x631, 0x627, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x627, 0x64a, 0x631, 0x627, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x64a, 0x631, 0x627, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x631, 0x6aa, 0x62a,
0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x631, 0x6aa,
0x62a, 0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x631, 0x6aa, 0x62a, 0x633, 0x6aa, 0x20,
0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x644, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x644,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x644, 0x20, 0x62c, 0x64a,
0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x62c, 0x627, 0x67e, 0x627,
0x646, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x680, 0x631, 0x20, 0x642, 0x632,
0x627, 0x642, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x648, 0x644, 0x647, 0x647, 0x20, 0x642, 0x627, 0x632, 0x642, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648,
0x631, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648,
0x631, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x648, 0x631, 0x64a, 0x627, 0x20, 0x62c,
0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x6aa, 0x648, 0x633, 0x631, 0x627, 0x626, 0x64a, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x631, 0x6cc, 0x633, 0x646, 0x648, 0x6cc, 0x627,
0x631, 0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x6aa, 0x631,
0x6cc, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6aa, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x6aa,
0x631, 0x6cc, 0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6aa, 0x20, 0x62c,
0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x6aa, 0x631, 0x63a, 0x632, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x644, 0x627, 0x626, 0x646, 0x20, 0x622, 0x626,
0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x644, 0x648, 0x631, 0x68a, 0x20, 0x647, 0x648, 0x648, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x644, 0x648, 0x631, 0x68a, 0x20, 0x647, 0x648,
0x648, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x644, 0x648, 0x631, 0x68a, 0x20, 0x647, 0x648, 0x648,
0x64a, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x6af, 0x627, 0x62f, 0x627, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x6af, 0x627, 0x62f, 0x627, 0x646, 0x20, 0x62c, 0x64a, 0x20,
0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x645, 0x644, 0x627, 0x626, 0x64a, 0x634, 0x64a, 0x627, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x627, 0x644, 0x62f, 0x64a, 0x67e, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x645, 0x631, 0x6aa, 0x64a, 0x648, 0x633, 0x633, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x627, 0x631, 0x634, 0x644, 0x20,
0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x648, 0x631, 0x64a, 0x634, 0x64a, 0x633, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x648, 0x631, 0x64a, 0x634, 0x64a, 0x633,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x648, 0x631, 0x64a, 0x634, 0x64a, 0x633, 0x20, 0x62c, 0x64a,
0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x627, 0x626, 0x648, 0x633, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x645, 0x64a, 0x6aa, 0x633, 0x64a, 0x6aa, 0x646, 0x20,
0x67e, 0x64a, 0x633, 0x64a, 0x641, 0x6aa, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x64a,
0x6aa, 0x633, 0x64a, 0x6aa, 0x646, 0x20, 0x67e, 0x64a, 0x633, 0x64a, 0x641, 0x6aa,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x64a, 0x6aa, 0x633, 0x64a, 0x6aa, 0x646, 0x20, 0x67e, 0x64a,
0x633, 0x64a, 0x641, 0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x644, 0x627,
0x646, 0x20, 0x628, 0x627, 0x62a, 0x631, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x62a, 0x631, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x627, 0x62a, 0x631, 0x20,
0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x627, 0x633, 0x6aa, 0x648, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x627, 0x633, 0x6aa, 0x648, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x645, 0x627, 0x633, 0x6aa, 0x648, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646,
0x647, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x64a, 0x627,
0x646, 0x645, 0x627, 0x631, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646,
0x627, 0x626, 0x648, 0x631, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x646, 0x64a, 0x67e, 0x627, 0x644, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x646, 0x64a, 0x648, 0x20, 0x6aa, 0x64a, 0x644, 0x64a, 0x68a, 0x648, 0x646, 0x64a,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x20,
0x6aa, 0x64a, 0x644, 0x64a, 0x68a, 0x648, 0x646, 0x64a, 0x627, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x646,
0x64a, 0x648, 0x20, 0x6aa, 0x64a, 0x644, 0x64a, 0x68a, 0x648, 0x646, 0x64a, 0x627,
0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x632, 0x64a, 0x644,
0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a,
0x648, 0x632, 0x64a, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648,
0x632, 0x64a, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a,
0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a,
0x648, 0x20, 0x641, 0x627, 0x626, 0x648, 0x646, 0x68a, 0x20, 0x644, 0x64a, 0x646,
0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x20,
0x641, 0x627, 0x626, 0x648, 0x646, 0x68a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x646, 0x64a, 0x648, 0x20, 0x641, 0x627, 0x626, 0x648, 0x646, 0x68a, 0x20,
0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x64a, 0x648, 0x648,
0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x627, 0x631, 0x20,
0x641, 0x648, 0x6aa, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x627, 0x631, 0x20, 0x641,
0x648, 0x6aa, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20,
0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642,
0x62a, 0x646, 0x627, 0x631, 0x20, 0x641, 0x648, 0x6aa, 0x20, 0x622, 0x626, 0x64a,
0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646,
0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631, 0x646,
0x627, 0x646, 0x68a, 0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648,
0x646, 0x647, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631,
0x646, 0x627, 0x646, 0x68a, 0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631,
0x648, 0x646, 0x647, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x68a,
0x648, 0x20, 0x62f, 0x64a, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646, 0x647, 0x627,
0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x646, 0x648, 0x648, 0x627, 0x633, 0x628, 0x626, 0x64a, 0x631,
0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x648, 0x648,
0x627, 0x633, 0x628, 0x626, 0x64a, 0x631, 0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x648,
0x648, 0x627, 0x633, 0x628, 0x626, 0x64a, 0x631, 0x633, 0x6aa, 0x20, 0x62c, 0x64a,
0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x648, 0x645, 0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x648, 0x645, 0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x648, 0x645, 0x633,
0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x627, 0x6aa, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x627, 0x6aa, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x67e, 0x627, 0x6aa, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x644, 0x627, 0x626, 0x648, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x627, 0x67e, 0x648, 0x627, 0x20, 0x646,
0x64a, 0x648, 0x20, 0x6af, 0x646, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x67e, 0x64a, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x64a, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x64a,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x67e, 0x64a, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x64a, 0x20, 0x62c,
0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x631, 0x648, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x631, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x64a, 0x631,
0x648, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x644, 0x67e, 0x627, 0x626,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x644, 0x67e, 0x627,
0x626, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a,
0x20, 0x648, 0x642, 0x62a, 0x641, 0x644, 0x67e, 0x627, 0x626, 0x646, 0x20, 0x62c,
0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x641, 0x648, 0x646, 0x64a, 0x6aa, 0x633, 0x20, 0x622,
0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x633, 0x64a, 0x646, 0x67d, 0x20, 0x67e, 0x64a, 0x626, 0x631, 0x20,
0x6fd, 0x20, 0x645, 0x64a, 0x6aa, 0x644, 0x648, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x633, 0x64a, 0x646, 0x67d, 0x20, 0x67e, 0x64a, 0x626, 0x631,
0x20, 0x6fd, 0x20, 0x645, 0x64a, 0x6aa, 0x644, 0x648, 0x646, 0x20, 0x62c, 0x648,
0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x633,
0x64a, 0x646, 0x67d, 0x20, 0x67e, 0x64a, 0x626, 0x631, 0x20, 0x6fd, 0x20, 0x645,
0x64a, 0x6aa, 0x644, 0x648, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646,
0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x67d, 0x6aa,
0x64a, 0x631, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x648,
0x646, 0x627, 0x67e, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67e,
0x64a, 0x627, 0x646, 0x6af, 0x20, 0x64a, 0x627, 0x646, 0x6af, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x631, 0x64a, 0x20, 0x64a, 0x648, 0x646, 0x64a, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x631, 0x648, 0x67f, 0x64a, 0x631,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x62e, 0x627, 0x644,
0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x62e, 0x627,
0x644, 0x64a, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x62e, 0x627, 0x644, 0x64a, 0x646, 0x20,
0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x633, 0x627, 0x645, 0x648, 0x627, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x633, 0x627, 0x645, 0x648, 0x627, 0x20, 0x62c, 0x648, 0x20,
0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x627,
0x645, 0x648, 0x627, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x634, 0x64a, 0x20, 0x634, 0x644,
0x632, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x646, 0x6af, 0x627,
0x67e, 0x648, 0x631, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631,
0x64a, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x648, 0x644, 0x648, 0x645, 0x646, 0x20,
0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c, 0x648, 0x20,
0x648, 0x642, 0x62a, 0x68f, 0x6a9, 0x6bb, 0x20, 0x62c, 0x627, 0x631, 0x62c, 0x64a,
0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x648, 0x631, 0x64a,
0x20, 0x646, 0x627, 0x645, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x633,
0x627, 0x626, 0x64a, 0x648, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x62a, 0x627, 0x647, 0x64a, 0x67d, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x62a, 0x627, 0x626, 0x67e, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x62a, 0x627, 0x626, 0x67e, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x627, 0x626, 0x67e,
0x64a, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x627, 0x62c, 0x6aa, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67d, 0x648, 0x6aa, 0x64a,
0x644, 0x627, 0x626, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67d,
0x648, 0x646, 0x6af, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x67d,
0x648, 0x646, 0x6af, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x67d, 0x648, 0x646, 0x6af, 0x627, 0x20,
0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x64a, 0x648, 0x6aa, 0x20, 0x62c, 0x648,
0x20, 0x648, 0x642, 0x62a, 0x62a, 0x631, 0x6aa, 0x645, 0x627, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x631, 0x6aa,
0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x62a, 0x631, 0x6aa,
0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627,
0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x62a, 0x648, 0x648, 0x627, 0x644, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x64a, 0x648, 0x631, 0x648, 0x6af, 0x627, 0x626, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x648, 0x631, 0x648, 0x6af, 0x627, 0x626,
0x64a, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20,
0x648, 0x642, 0x62a, 0x64a, 0x648, 0x631, 0x648, 0x6af, 0x627, 0x626, 0x64a, 0x20,
0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631, 0x64a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x628, 0x6aa, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x628, 0x6aa,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627,
0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x628, 0x6aa, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x62c, 0x64a, 0x20, 0x627, 0x648, 0x646, 0x647, 0x627, 0x631,
0x64a, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x627, 0x646, 0x648,
0x627, 0x62a, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x627,
0x646, 0x648, 0x627, 0x62a, 0x648, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a,
0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x627, 0x646, 0x648, 0x627,
0x62a, 0x648, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20,
0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x64a, 0x646, 0x632, 0x648, 0x64a,
0x644, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x644, 0x627,
0x62f, 0x648, 0x648, 0x633, 0x62a, 0x648, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x648, 0x644, 0x627, 0x62f, 0x648, 0x648, 0x633, 0x62a, 0x648, 0x6aa,
0x20, 0x62c, 0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x648, 0x644, 0x627, 0x62f, 0x648, 0x648, 0x633, 0x62a, 0x648,
0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x648, 0x644, 0x6af, 0x648, 0x6af, 0x631,
0x627, 0x62f, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x648, 0x644,
0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x62c, 0x648, 0x20, 0x645, 0x639,
0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x648, 0x644, 0x6af,
0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646,
0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x648, 0x633,
0x67d, 0x648, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x64a,
0x6aa, 0x20, 0x622, 0x626, 0x64a, 0x20, 0x644, 0x64a, 0x646, 0x68a, 0x20, 0x62c,
0x648, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x64a, 0x644, 0x633, 0x20, 0x6fd, 0x20,
0x641, 0x62a, 0x648, 0x646, 0x627, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x64a, 0x627, 0x6aa, 0x62a, 0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x64a, 0x627, 0x6aa, 0x62a, 0x633, 0x6aa, 0x20, 0x62c, 0x648, 0x20, 0x645,
0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x627, 0x6aa,
0x62a, 0x633, 0x6aa, 0x20, 0x62c, 0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646,
0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a, 0x64a, 0x6aa, 0x627, 0x67d, 0x64a,
0x631, 0x646, 0x628, 0x631, 0x6af, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642, 0x62a,
0x64a, 0x6aa, 0x627, 0x67d, 0x64a, 0x631, 0x646, 0x628, 0x631, 0x6af, 0x20, 0x62c,
0x648, 0x20, 0x645, 0x639, 0x64a, 0x627, 0x631, 0x64a, 0x20, 0x648, 0x642, 0x62a,
0x64a, 0x6aa, 0x627, 0x67d, 0x64a, 0x631, 0x646, 0x628, 0x631, 0x6af, 0x20, 0x62c,
0x64a, 0x20, 0x68f, 0x64a, 0x646, 0x647, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648,
0x642, 0x62a, 0x64a, 0x6aa, 0x648, 0x646, 0x20, 0x62c, 0x648, 0x20, 0x648, 0x642,
0x62a, 0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924,
0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x92e, 0x905, 0x92f, 0x93e, 0x930,
0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940,
0x20, 0x926, 0x940, 0x902, 0x952, 0x939, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915,
0x94d, 0x924, 0x913, 0x92d, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x913,
0x92d, 0x930, 0x940, 0x20, 0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935,
0x915, 0x94d, 0x924, 0x913, 0x92d, 0x930, 0x940, 0x20, 0x926, 0x940, 0x902, 0x952,
0x939, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92a, 0x939, 0x93e,
0x921, 0x93c, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92a, 0x939, 0x93e, 0x921,
0x93c, 0x940, 0x20, 0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915,
0x94d, 0x924, 0x92a, 0x939, 0x93e, 0x921, 0x93c, 0x940, 0x20, 0x926, 0x940, 0x902,
0x952, 0x939, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92a, 0x947,
0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92a, 0x947,
0x938, 0x93f, 0x92b, 0x93f, 0x915, 0x20, 0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940,
0x20, 0x935, 0x915, 0x94d, 0x924, 0x92a, 0x947, 0x938, 0x93f, 0x92b, 0x93f, 0x915,
0x20, 0x926, 0x940, 0x902, 0x952, 0x939, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915,
0x94d, 0x924, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x935,
0x915, 0x94d, 0x924, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20,
0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x905,
0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x926, 0x940, 0x902, 0x952,
0x939, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x92e, 0x930, 0x915,
0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x935,
0x915, 0x94d, 0x924, 0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20, 0x92f, 0x942,
0x930, 0x94b, 0x92a, 0x940, 0x20, 0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20,
0x935, 0x915, 0x94d, 0x924, 0x941, 0x92e, 0x930, 0x915, 0x91c, 0x93c, 0x940, 0x20,
0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x909, 0x928, 0x939, 0x93e, 0x930,
0x947, 0x20, 0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0x913, 0x92d,
0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x935, 0x915,
0x94d, 0x924, 0x941, 0x913, 0x92d, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x20, 0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915,
0x94d, 0x924, 0x941, 0x913, 0x92d, 0x930, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x20, 0x909, 0x928, 0x939, 0x93e, 0x930, 0x947, 0x20, 0x91c, 0x94b,
0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0x909, 0x932, 0x939, 0x902, 0x926, 0x94b,
0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x940, 0x20, 0x935, 0x915, 0x94d, 0x924,
0x941, 0x909, 0x932, 0x939, 0x902, 0x926, 0x94b, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x940, 0x20, 0x92e, 0x905, 0x92f, 0x93e, 0x930, 0x940, 0x20, 0x935, 0x915,
0x94d, 0x924, 0x941, 0x909, 0x932, 0x939, 0x902, 0x926, 0x94b, 0x20, 0x92f, 0x942,
0x930, 0x94b, 0x92a, 0x940, 0x20, 0x909, 0x928, 0x939, 0x93e, 0x930, 0x947, 0x20,
0x91c, 0x94b, 0x20, 0x935, 0x915, 0x94d, 0x924, 0x941, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x940, 0x928, 0x20, 0x935, 0x915, 0x94d,
0x924, 0x941, 0xd87, 0xdc6, 0xdca, 0xd9c, 0xdb1, 0xdd2, 0xdc3, 0xdca, 0xdae, 0xdcf,
0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba,
0xdb8, 0x20, 0xd85, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4,
0xdd2, 0xdbb, 0x20, 0xd85, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1,
0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdaf, 0xd9a, 0xdd4, 0xdab, 0xdd4,
0x20, 0xd85, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd85,
0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd85, 0xdb4, 0xdca,
0x200d, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb,
0x20, 0xd85, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd,
0xdba, 0xd87, 0xdbd, 0xdc3, 0xdca, 0xd9a, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd87, 0xdbd, 0xdc3, 0xdca, 0xd9a, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdbd, 0xdc3, 0xdca, 0xd9a,
0xdcf, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdb8, 0xdbb, 0xdca, 0xdc3, 0xdb1, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdb8, 0xdbb, 0xdca, 0xdc3, 0xdb1, 0xdca,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd87, 0xdb8, 0xdbb, 0xdca, 0xdc3, 0xdb1, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd8b, 0xdad, 0xdd4,
0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20,
0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4,
0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20,
0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86,
0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4,
0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20,
0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2,
0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2,
0xdbb, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a,
0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb,
0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb,
0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9a, 0xdb3, 0xdd4, 0xd9a, 0xdbb, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87,
0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9a, 0xdb3, 0xdd4, 0xd9a,
0xdbb, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a,
0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9a, 0xdb3, 0xdd4, 0xd9a, 0xdbb, 0x20, 0xdaf, 0xdd2,
0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdb4, 0xdd0, 0xdc3, 0xdd2, 0xdc6, 0xdd2, 0xd9a, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87,
0xdb8, 0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb4, 0xdd0, 0xdc3, 0xdd2,
0xdc6, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdad, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xd87, 0xdb8,
0xdbb, 0xdd2, 0xd9a, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdb4, 0xdd0, 0xdc3, 0xdd2, 0xdc6,
0xdd2, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdb4, 0xdd2, 0xdba, 0xdcf, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdb4, 0xdd2, 0xdba, 0xdcf, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdb4,
0xdd2, 0xdba, 0xdcf, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd85, 0xdbb, 0xdcf, 0xdb6, 0xdd2, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd85, 0xdbb, 0xdcf, 0xdb6, 0xdd2, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdbb, 0xdcf, 0xdb6, 0xdd2, 0x20,
0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd86,
0xdbb, 0xdca, 0xda2, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdb1, 0xdcf, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd86, 0xdbb, 0xdca, 0xda2, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdb1,
0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd86, 0xdbb, 0xdca, 0xda2, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdb1, 0xdcf, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd,
0xdba, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd86, 0xdbb, 0xdca, 0xda2, 0xdb1,
0xdca, 0xda7, 0xdd2, 0xdb1, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6,
0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd86, 0xdbb, 0xdca, 0xda2, 0xdb1, 0xdca, 0xda7,
0xdd2, 0xdb1, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd86, 0xdbb, 0xdca,
0xda2, 0xdb1, 0xdca, 0xda7, 0xdd2, 0xdb1, 0xdcf, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd86, 0xdb8, 0xdda,
0xdb1, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd86, 0xdb8, 0xdda, 0xdb1, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd86, 0xdb8, 0xdda,
0xdb1, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdad, 0xdca,
0xdbd, 0xdcf, 0xdb1, 0xdca, 0xdad, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd85, 0xdad, 0xdca, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0xdad, 0xdd2, 0xd9a,
0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd85, 0xdad, 0xdca, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0xdad, 0xdd2, 0xd9a, 0xdca,
0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xd95, 0xdc3,
0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d,
0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d,
0xdba, 0xdb8, 0x20, 0xd94, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd,
0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20,
0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd94, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d,
0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdb6, 0xda7, 0xdc4,
0xdd2, 0xdbb, 0x20, 0xd94, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd,
0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20,
0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd94, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d,
0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdaf, 0xdc4, 0xdc0,
0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9,
0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb,
0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd95, 0xdc3,
0xdca, 0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb1, 0xdd0, 0xd9f, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd95, 0xdc3, 0xdca,
0xda7, 0xdca, 0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20,
0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6,
0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca, 0x200d, 0xdbb,
0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd95, 0xdc3, 0xdca, 0xda7, 0xdca,
0x200d, 0xdbb, 0xdda, 0xdbd, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd85, 0xdc3, 0xdbb,
0xdca, 0xdb6, 0xdba, 0xdd2, 0xda2, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd85, 0xdc3, 0xdbb, 0xdca, 0xdb6, 0xdba, 0xdd2, 0xda2, 0xdcf, 0xdb1,
0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd85, 0xdc3, 0xdbb, 0xdca, 0xdb6, 0xdba, 0xdd2, 0xda2, 0xdcf, 0xdb1, 0xdca,
0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdc3, 0xddc, 0xdbb, 0xdca, 0xdc3, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87, 0xdc3, 0xddc, 0xdbb, 0xdca, 0xdc3, 0xdca, 0x20,
0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd87,
0xdc3, 0xddc, 0xdbb, 0xdca, 0xdc3, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xd82, 0xd9c,
0xdbd, 0xdcf, 0xdaf, 0xdda, 0xdc1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6,
0xd82, 0xd9c, 0xdbd, 0xdcf, 0xdaf, 0xdda, 0xdc1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xd82, 0xd9c, 0xdbd, 0xdcf,
0xdaf, 0xdda, 0xdc1, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8,
0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb7, 0xdd4, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xddc, 0xdbd, 0xdd2, 0xdc0, 0xdd2, 0xdba, 0xdcf,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdd3,
0xdbd, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdc3,
0xdd3, 0xdbd, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb6, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdd3, 0xdbd, 0x20, 0xd9c, 0xdca,
0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb6,
0xdd8, 0xdb1, 0xdcf, 0xdba, 0xdd2, 0x20, 0xdaf, 0xdbb, 0xdd4, 0xdc3, 0xdca, 0xdc3,
0xdbd, 0xdcf, 0xdb8, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdda,
0xdb4, 0xdca, 0xdc0, 0xdda, 0xda9, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd9a, 0xdda, 0xdb4, 0xdca, 0xdc0, 0xdda, 0xda9, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdda, 0xdb4, 0xdca,
0xdc0, 0xdda, 0xda9, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xda0, 0xdb8, 0xddc, 0xdbb, 0xddd, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd0, 0xdad, 0xdb8, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd0, 0xdad, 0xdb8, 0xdca, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd0, 0xdad,
0xdb8, 0xdca, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xda0, 0xdd2, 0xdbd, 0xdd3, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0,
0xdd2, 0xdbd, 0xdd3, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd2, 0xdbd, 0xdd3, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xda0, 0xdd3, 0xdb1,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd3, 0xdb1, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xdd3, 0xdb1,
0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xda0, 0xddc, 0xdba, 0xdd2, 0xdb6, 0xdbd, 0xdca, 0xdc3, 0xdcf, 0xdb1, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xddc, 0xdba, 0xdd2, 0xdb6, 0xdbd, 0xdca,
0xdc3, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda0, 0xddc, 0xdba, 0xdd2, 0xdb6, 0xdbd, 0xdca, 0xdc3,
0xdcf, 0xdb1, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdc3,
0xdca, 0xdb8, 0xdc3, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc, 0xd9a, 0xddd, 0xdc3, 0xdca, 0x20, 0xdaf,
0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc,
0xdbd, 0xddc, 0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd9a, 0xddc, 0xdbd, 0xddc, 0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdcf,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd9a, 0xddc, 0xdbd, 0xddc, 0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdcf, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba,
0xd9a, 0xdd4, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd4, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4,
0xdad, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd9a, 0xdd4, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca,
0x20, 0xdb7, 0xdcf, 0xd9c, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd2, 0xdba, 0xdd4, 0xdb6,
0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd2, 0xdba,
0xdd4, 0xdb6, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd2, 0xdba, 0xdd4, 0xdb6, 0xdcf, 0xdb1,
0xdd4, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda9, 0xdcf, 0xdc0, 0xdd2, 0xdc3, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdaf, 0xdd4, 0xdb8, 0xddc, 0xdb1, 0xdca, 0xdad, 0xdca,
0x2d, 0xda9, 0xdca, 0xd8b, 0xdbb, 0xdca, 0xdc0, 0xdd2, 0xdbd, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb,
0x20, 0xda7, 0xdd2, 0xdb8, 0xddd, 0xdbb, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd8a, 0xdc3, 0xdca, 0xda7, 0xdbb, 0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc3, 0xdca, 0xda7, 0xdbb,
0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc3, 0xdca, 0xda7, 0xdbb,
0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd89, 0xd9a, 0xdca,
0xdc0, 0xdaf, 0xddd, 0xdbb, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8,
0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3,
0xdba, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba,
0xdb8, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdb0, 0xdca,
0x200d, 0xdba, 0xdb8, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdba,
0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb,
0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb,
0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdad,
0xdc0, 0xdad, 0xdca, 0x2d, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb,
0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd,
0xdb4, 0xdd3, 0xdba, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4,
0xdd2, 0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7,
0xdc4, 0xdd2, 0xdbb, 0x20, 0xdba, 0xdd4, 0xdbb, 0xddd, 0xdb4, 0xdd3, 0xdba, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdc6, 0xddd, 0xd9a, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xda9, 0xdca, 0x20,
0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc6,
0xddd, 0xd9a, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xda9, 0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4,
0xdad, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdc6, 0xddd, 0xd9a, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xda9, 0xdca, 0x20,
0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2,
0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdc6, 0xdd2, 0xda2, 0xdd2, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xdd2, 0xda2, 0xdd2, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xdd2, 0xda2,
0xdd2, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdca, 0x200d, 0xdbb, 0xd82, 0xdc1, 0x20, 0xd9c,
0xdba, 0xdb1, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdca, 0x200d,
0xdbb, 0xd82, 0xdc1, 0x20, 0xdaf, 0xd9a, 0xdd4, 0xdab, 0xdd4, 0xdaf, 0xdd2, 0xd9c,
0x20, 0xdc3, 0xdc4, 0x20, 0xd87, 0xdb1, 0xdca, 0xda7, 0xdcf, 0xdbb, 0xdca, 0xd9a,
0xdca, 0xda7, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9c,
0xdbd, 0xdb4, 0xd9c, 0xddc, 0xdc3, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xd9c, 0xdd0, 0xdb8, 0xdca, 0xdb6, 0xdd2, 0xdba, 0xdbb, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xda2, 0xddd, 0xdbb, 0xdca, 0xda2, 0xdd2, 0xdba, 0xdcf, 0xdb1,
0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda2, 0xddd, 0xdbb, 0xdca, 0xda2,
0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda2, 0xddd, 0xdbb, 0xdca, 0xda2, 0xdd2, 0xdba,
0xdcf, 0xdb1, 0xdd4, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9c, 0xdd2, 0xdbd, 0xdca, 0xdb6, 0xdbb,
0xdca, 0xda7, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd2, 0xdb1, 0xdd2, 0xda0, 0xdca,
0x20, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9c, 0xdca,
0x200d, 0xdbb, 0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20,
0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1,
0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb6, 0xda7, 0xdc4,
0xdd2, 0xdbb, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1,
0xdca, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2,
0xdbb, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca,
0xdad, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd9c, 0xdbd, 0xdca, 0xdc6,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xdc0, 0xdcf, 0xdba, 0xdd2,
0x2d, 0xd85, 0xdbd, 0xdd9, 0xdba, 0xdd4, 0xdad, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdca,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xdc0, 0xdcf, 0xdba, 0xdd2, 0x2d,
0xd85, 0xdbd, 0xdd9, 0xdba, 0xdd4, 0xdad, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdca, 0x20,
0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4,
0xdc0, 0xdcf, 0xdba, 0xdd2, 0x2d, 0xd85, 0xdbd, 0xdd9, 0xdba, 0xdd4, 0xdad, 0xdd2,
0xdba, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd,
0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xddc, 0xd82, 0xd9a, 0xddc,
0xd82, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xddc, 0xd82, 0xd9a, 0xddc,
0xd82, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdc4, 0xddc, 0xd82, 0xd9a, 0xddc, 0xd82, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xddc,
0xdc0, 0xdca, 0xda9, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xddc,
0xdc0, 0xdca, 0xda9, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc4, 0xddc, 0xdc0, 0xdca, 0xda9, 0xdca, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd2, 0xdba, 0xdb1, 0xdca,
0x20, 0xdc3, 0xdcf, 0xd9c, 0xdbb, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89,
0xdb1, 0xdca, 0xdaf, 0xdd4, 0xda0, 0xdd3, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb8, 0xdb0, 0xdca, 0x200d, 0xdba, 0xdb8, 0x20, 0xd89, 0xdb1, 0xdca, 0xdaf,
0xdd4, 0xdb1, 0xdd3, 0xdc3, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4, 0xdd2, 0xdbb, 0x20,
0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd4, 0xdb1, 0xdd3, 0xdc3, 0xdd2, 0xdba, 0xdcf, 0xdb1,
0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb,
0x20, 0xd89, 0xdb1, 0xdca, 0xdaf, 0xdd4, 0xdb1, 0xdd3, 0xdc3, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdbb, 0xdcf, 0xdb1,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdbb, 0xdcf, 0xdb1, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdbb,
0xdcf, 0xdb1, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba,
0xd89, 0xdbb, 0xdca, 0xd9a, 0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdbb, 0xdca, 0xd9a, 0xdd4, 0xda7, 0xdca,
0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd89, 0xdbb, 0xdca, 0xd9a, 0xdd4, 0xda7, 0xdca, 0xdc3,
0xdca, 0xd9a, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc1, 0xdca, 0x200d, 0xdbb, 0xdcf,
0xdba, 0xdbd, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc1, 0xdca, 0x200d,
0xdbb, 0xdcf, 0xdba, 0xdbd, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8a, 0xdc1, 0xdca, 0x200d, 0xdbb, 0xdcf, 0xdba, 0xdbd,
0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xda2, 0xdb4, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda2, 0xdb4,
0xdcf, 0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xda2, 0xdb4, 0xdcf, 0xdb1, 0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd, 0xdca,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd0, 0xd9c, 0xdd9, 0xdb1, 0xdc4,
0xdd2, 0xdbb, 0x20, 0xd9a, 0xdc3, 0xd9a, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb6, 0xda7, 0xdc4, 0xdd2, 0xdbb, 0x20, 0xd9a,
0xdc3, 0xd9a, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd9a, 0xddc, 0xdbb, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc, 0xdbb, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20,
0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a,
0xddc, 0xdbb, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xddc, 0xdc3, 0xdca, 0xdbb,
0xdda, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdc3,
0xdca, 0xdb1, 0xddc, 0xdba, 0xdcf, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdca, 0x200d, 0xdbb, 0xdc3, 0xdca, 0xdb1,
0xddc, 0xdba, 0xdcf, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdca, 0x200d,
0xdbb, 0xdc3, 0xdca, 0xdb1, 0xddc, 0xdba, 0xdcf, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a,
0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd9a, 0xdd2, 0xdbb, 0xdca, 0xd9c, 0xdd2, 0xdc3, 0xdca,
0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc1, 0xdca, 0x200d,
0xdbb, 0xdd3, 0x20, 0xdbd, 0xd82, 0xd9a, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdbd, 0xdba, 0xdd2, 0xdb1, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdbd, 0xddd, 0xdbb, 0xdca, 0xda9, 0xdca,
0x20, 0xdc4, 0xddd, 0xdc0, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdbd,
0xddd, 0xdbb, 0xdca, 0xda9, 0xdca, 0x20, 0xdc4, 0xddd, 0xdc0, 0xdca, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdbd, 0xddd,
0xdbb, 0xdca, 0xda9, 0xdca, 0x20, 0xdc4, 0xddd, 0xdc0, 0xdca, 0x20, 0xdaf, 0xdd2,
0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd9, 0xd9c, 0xda9,
0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd9, 0xd9c, 0xda9,
0xdb1, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb8, 0xdd9, 0xd9c, 0xda9, 0xdb1, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8,
0xdd0, 0xdbd, 0xdda, 0xdc3, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdcf, 0xdbd, 0xdaf, 0xdd2, 0xdc0, 0xdba, 0xdd2, 0xdb1,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdcf, 0xdbb, 0xdca, 0xd9a,
0xdd4, 0xd91, 0xdc3, 0xdcf, 0xdc3, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb8, 0xdcf, 0xdbb, 0xdca, 0xdc2, 0xdbd, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd4, 0xdbb, 0xdd4, 0xdc3,
0xdd2, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd4, 0xdbb, 0xdd4, 0xdc3,
0xdd2, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb8, 0xdd4, 0xdbb, 0xdd4, 0xdc3, 0xdd2, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb8, 0xddc, 0xdc0,
0xdca, 0xdc3, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd9,
0xd9a, 0xdca, 0xdc3, 0xdd2, 0xd9a, 0xddd, 0x20, 0xdb4, 0xdd0, 0xdc3, 0xdd2, 0xdc6,
0xdd2, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd9, 0xd9a,
0xdca, 0xdc3, 0xdd2, 0xd9a, 0xddd, 0x20, 0xdb4, 0xdd0, 0xdc3, 0xdd2, 0xdc6, 0xdd2,
0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb8, 0xdd9, 0xd9a, 0xdca, 0xdc3, 0xdd2, 0xd9a, 0xddd, 0x20, 0xdb4,
0xdd0, 0xdc3, 0xdd2, 0xdc6, 0xdd2, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf,
0xd86, 0xdbd, 0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbd,
0xdcf, 0xdb1, 0xdca, 0x20, 0xdb6, 0xdcf, 0xda7, 0xdbb, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdb6, 0xdcf, 0xda7,
0xdbb, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd8b, 0xdbd, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdb6, 0xdcf, 0xda7, 0xdbb,
0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xddc, 0xdc3, 0xdca, 0xd9a, 0xdc0, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xddc, 0xdc3, 0xdca, 0xd9a, 0xdc0, 0xdca,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb8, 0xddc, 0xdc3, 0xdca, 0xd9a, 0xdc0, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb8, 0xdd2,
0xdba, 0xdb1, 0xdca, 0xdb8, 0xdcf, 0xdbb, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb1, 0xdcf, 0xdc0, 0xdd4, 0xdbb, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb1, 0xdda, 0xdb4, 0xdcf, 0xdbd, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb1, 0xdc0, 0x20, 0xdc3, 0xdd9, 0xdbd, 0xda9, 0xddc, 0xdb1, 0xdd2, 0xdba, 0xdcf,
0xdb1, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdc0, 0x20, 0xdc3,
0xdd9, 0xdbd, 0xda9, 0xddc, 0xdb1, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdc0,
0x20, 0xdc3, 0xdd9, 0xdbd, 0xda9, 0xddc, 0xdb1, 0xdd2, 0xdba, 0xdcf, 0xdb1, 0xdd4,
0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdc0, 0xdc3, 0xdd3, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdc0, 0xdc3, 0xdd3, 0xdbd, 0xdb1, 0xdca,
0xdad, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb1, 0xdc0, 0xdc3, 0xdd3, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdaf, 0xdd2,
0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd2, 0xdc0, 0xdca,
0xdc6, 0xdc0, 0xdd4, 0xdb1, 0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd2, 0xdc0, 0xdca, 0xdc6, 0xdc0, 0xdd4, 0xdb1, 0xdca,
0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd2, 0xdc0, 0xdca, 0xdc6, 0xdc0, 0xdd4, 0xdb1,
0xdca, 0xdbd, 0xdb1, 0xdca, 0xdad, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd,
0xddd, 0xd9a, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xdd2, 0xdba, 0xdd4,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc6, 0xddc, 0xdbd, 0xdca,
0xd9a, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc6, 0xddc, 0xdbd, 0xdca, 0xd9a, 0xdca, 0x20, 0xdaf,
0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc6, 0xddc, 0xdbd, 0xdca, 0xd9a, 0xdca,
0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xdbb, 0xdca,
0xdb1, 0xdd0, 0xdb1, 0xdca, 0xda9, 0xddd, 0x20, 0xda9, 0xdd2, 0x20, 0xdb1, 0xddc,
0xdbb, 0xddc, 0xdb1, 0xdca, 0xdc4, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdc6, 0xdbb, 0xdca, 0xdb1, 0xdd0, 0xdb1, 0xdca, 0xda9, 0xddd, 0x20, 0xda9, 0xdd2,
0x20, 0xdb1, 0xddc, 0xdbb, 0xddc, 0xdb1, 0xdca, 0xdc4, 0xdcf, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xdbb, 0xdca,
0xdb1, 0xdd0, 0xdb1, 0xdca, 0xda9, 0xddd, 0x20, 0xda9, 0xdd2, 0x20, 0xdb1, 0xddc,
0xdbb, 0xddc, 0xdb1, 0xdca, 0xdc4, 0xdcf, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3,
0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb1, 0xddc, 0xdc0, 0xdc3,
0xdd2, 0xdb6, 0xdd2, 0xdbb, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc0, 0xdc3, 0xdd2, 0xdb6, 0xdd2, 0xdbb, 0xdca,
0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb1, 0xddc, 0xdc0, 0xdc3, 0xdd2, 0xdb6, 0xdd2, 0xdbb,
0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2,
0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd94, 0xdb8, 0xdca, 0xdc3,
0xdca, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd94, 0xdb8, 0xdca,
0xdc3, 0xdca, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd94, 0xdb8, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb4, 0xdcf, 0xd9a, 0xdd2, 0xdc3, 0xdca, 0xdae, 0xdcf, 0xdb1, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdcf, 0xd9a, 0xdd2, 0xdc3, 0xdca, 0xdae,
0xdcf, 0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb4, 0xdcf, 0xd9a, 0xdd2, 0xdc3, 0xdca, 0xdae, 0xdcf, 0xdb1, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd,
0xdba, 0xdb4, 0xdbd, 0xdcf, 0xdc0, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb4, 0xdd0, 0xdb4, 0xdd4, 0xdc0, 0xdcf, 0x20, 0xdb1, 0xdd2, 0xdc0, 0xdca, 0x20,
0xd9c, 0xdd2, 0xdb1, 0xdd3, 0xdba, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdb4, 0xdd0, 0xdbb, 0xd9c, 0xdd4, 0xdc0, 0xdda, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdb4, 0xdd0, 0xdbb, 0xd9c, 0xdd4, 0xdc0, 0xdda, 0x20, 0xdc3, 0xdb8, 0xdca,
0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdd0, 0xdbb, 0xd9c,
0xdd4, 0xdc0, 0xdda, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8,
0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xdb4, 0xdda, 0xdbb, 0xdd4, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdda, 0xdbb, 0xdd4, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8,
0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdda, 0xdbb, 0xdd4, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd,
0xdba, 0xdb4, 0xdd2, 0xdbd, 0xdd2, 0xdb4, 0xdd3, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdb4, 0xdd2, 0xdbd, 0xdd2, 0xdb4, 0xdd3, 0xdb1, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdd2, 0xdbd,
0xdd2, 0xdb4, 0xdd3, 0xdb1, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca,
0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc6, 0xdd3, 0xdb1, 0xdd2, 0xd9a,
0xdca, 0xdc3, 0xdca, 0x20, 0xdaf, 0xdd4, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xdb4, 0xdd2, 0xdba,
0xdbb, 0xdda, 0x20, 0xdc3, 0xdc4, 0x20, 0xdb8, 0xdd0, 0xd9a, 0xddd, 0xdbd, 0xdb1,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad,
0x20, 0xdb4, 0xdd2, 0xdba, 0xdbb, 0xdda, 0x20, 0xdc3, 0xdc4, 0x20, 0xdb8, 0xdd0,
0xd9a, 0xddd, 0xdbd, 0xdb1, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc1, 0xdcf, 0xdb1, 0xdca, 0xdad, 0x20, 0xdb4,
0xdd2, 0xdba, 0xdbb, 0xdda, 0x20, 0xdc3, 0xdc4, 0x20, 0xdb8, 0xdd0, 0xd9a, 0xddd,
0xdbd, 0xdb1, 0xdca, 0x20, 0xdaf, 0xdd2, 0xdc0, 0xdcf, 0xd86, 0xdbd, 0xddd, 0xd9a,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdd2, 0xda7, 0xdca, 0xd9a, 0xdd9,
0xdba, 0xdcf, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xddc,
0xdb1, 0xdb4, 0xdda, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdb4, 0xdca, 0xdba,
0xddc, 0xdb1, 0xdca, 0xd9c, 0xdca, 0xdba, 0xdb1, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdbb, 0xdd2, 0xdba, 0xdd4, 0xdb1, 0xdd2, 0xdba, 0xdb1, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdbb, 0xddc, 0xdad, 0xdd9, 0xdbb, 0xdcf, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xd9b, 0xdbd, 0xdd2, 0xdb1, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xd9b, 0xdbd, 0xdd2, 0xdb1, 0xdca, 0x20,
0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3,
0xd9b, 0xdbd, 0xdd2, 0xdb1, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2,
0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xdd0, 0xdb8, 0xddd,
0xdc0, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xdd0, 0xdb8, 0xddd,
0xdc0, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdc3, 0xdd0, 0xdb8, 0xddd, 0xdc0, 0xdcf, 0x20, 0xd9c, 0xdca, 0x200d,
0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3,
0xdd3, 0xdc2, 0xdd9, 0xdbd, 0xdca, 0xdc3, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdc3, 0xdd2, 0xd82, 0xd9c, 0xdb4, 0xdca, 0xdb4, 0xdd4, 0xdbb, 0xdd4, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xddc, 0xdbd, 0xddc, 0xdb8, 0xdb1, 0xdca,
0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdaf, 0xd9a, 0xdd4, 0xdab, 0xdd4, 0x20, 0xda2, 0xddd, 0xda2, 0xdd2, 0xdba, 0xdcf,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xdd4, 0xdbb, 0xdd2, 0xdb1, 0xdcf,
0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc3, 0xdca, 0xdba, 0xddd, 0xdc0,
0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xdcf, 0xdc4, 0xdd2, 0xda7,
0xdd2, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdad, 0xdcf, 0xdba, 0xdd2, 0xdb4,
0xdda, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdad, 0xdcf, 0xdba, 0xdd2, 0xdb4,
0xdda, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xdad, 0xdcf, 0xdba, 0xdd2, 0xdb4, 0xdda, 0x20, 0xdaf, 0xdc4, 0xdc0, 0xdbd,
0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xda2, 0xdd2, 0xd9a, 0xdd2,
0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7,
0xddc, 0xd9a, 0xdd9, 0xdbd, 0xdcf, 0xdc0, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xda7, 0xddc, 0xd82, 0xd9c, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xda7, 0xddc, 0xd82, 0xd9c, 0xdcf, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xddc, 0xd82, 0xd9c, 0xdcf, 0x20, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xda0, 0xdd4, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7,
0xdbb, 0xdca, 0xd9a, 0xdca, 0xdb8, 0xdd9, 0xdb1, 0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf,
0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xdbb, 0xdca, 0xd9a, 0xdca,
0xdb8, 0xdd9, 0xdb1, 0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc3, 0xdb8,
0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xda7, 0xdbb, 0xdca,
0xd9a, 0xdca, 0xdb8, 0xdd9, 0xdb1, 0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xda7, 0xdd4, 0xdc0, 0xdcf, 0xdbd, 0xdd4, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xd8b, 0xdbb, 0xdd4, 0xd9c, 0xdd4, 0xdc0, 0xdda, 0x20, 0xdc0, 0xdda,
0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbb, 0xdd4, 0xd9c, 0xdd4, 0xdc0, 0xdda, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xd8b, 0xdbb,
0xdd4, 0xd9c, 0xdd4, 0xdc0, 0xdda, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2,
0xdca, 0xdb8, 0x20, 0xd9a, 0xdcf, 0xdbd, 0xdba, 0xd8b, 0xdc3, 0xdca, 0xdb6, 0xdd9,
0xd9a, 0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf, 0xdb1, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd8b, 0xdc3, 0xdca, 0xdb6, 0xdd9, 0xd9a, 0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf,
0xdb1, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf,
0xdc0, 0xd8b, 0xdc3, 0xdca, 0xdb6, 0xdd9, 0xd9a, 0xdd2, 0xdc3, 0xdca, 0xdad, 0xdcf,
0xdb1, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdb1, 0xdd4, 0xd86, 0xda7, 0xdd4, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdb1, 0xdd4, 0xd86, 0xda7, 0xdd4, 0x20, 0xdc3,
0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdb1,
0xdd4, 0xd86, 0xda7, 0xdd4, 0x20, 0xd9c, 0xdd2, 0xdb8, 0xdca, 0xdc4, 0xdcf, 0xdb1,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdd9, 0xdb1, 0xdd2, 0xdc3, 0xdd2,
0xdba, 0xdd4, 0xdbd, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdca,
0xdbd, 0xdaf, 0xdd2, 0xdc0, 0xddd, 0xdc3, 0xdca, 0xda7, 0xddd, 0xd9a, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdca, 0xdbd, 0xdaf, 0xdd2, 0xdc0, 0xddd,
0xdc3, 0xdca, 0xda7, 0xddd, 0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdca, 0xdbd, 0xdaf, 0xdd2, 0xdc0,
0xddd, 0xdc3, 0xdca, 0xda7, 0xddd, 0xd9a, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xddc,
0xdbd, 0xdca, 0xd9c, 0xddc, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdcf, 0xda9, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xddc, 0xdbd, 0xdca, 0xd9c, 0xddc, 0xd9c,
0xdca, 0x200d, 0xdbb, 0xdcf, 0xda9, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad,
0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xddc, 0xdbd, 0xdca, 0xd9c, 0xddc,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdcf, 0xda9, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb,
0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xddc,
0xdc3, 0xdca, 0xda7, 0xddc, 0xd9a, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdc0, 0xdda, 0xd9a, 0xdca, 0x20, 0xdaf, 0xdd6, 0xdb4, 0xdad, 0xdca, 0x20, 0xdc0,
0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdc0, 0xdd0, 0xdbd, 0xdd2, 0xdc3, 0xdca, 0x20, 0xdc3,
0xdc4, 0x20, 0xdc6, 0xdd4, 0xda7, 0xdd4, 0xdb1, 0xdcf, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdba, 0xd9a, 0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xd9a, 0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca,
0xd9a, 0xdca, 0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdba, 0xd9a, 0xdd4, 0xda7, 0xdca, 0xdc3, 0xdca, 0xd9a, 0xdca, 0x20,
0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20, 0xdc0, 0xdda, 0xdbd,
0xdcf, 0xdc0, 0xdba, 0xdd9, 0xd9a, 0xda7, 0xdd9, 0xdbb, 0xdd2, 0xdb1, 0xdca, 0xdb6,
0xdbb, 0xdca, 0xd9c, 0xdca, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xdd9,
0xd9a, 0xda7, 0xdd9, 0xdbb, 0xdd2, 0xdb1, 0xdca, 0xdb6, 0xdbb, 0xdca, 0xd9c, 0xdca,
0x20, 0xdc3, 0xdb8, 0xdca, 0xdb8, 0xdad, 0x20, 0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0,
0xdba, 0xdd9, 0xd9a, 0xda7, 0xdd9, 0xdbb, 0xdd2, 0xdb1, 0xdca, 0xdb6, 0xdbb, 0xdca,
0xd9c, 0xdca, 0x20, 0xd9c, 0xdca, 0x200d, 0xdbb, 0xdd3, 0xdc2, 0xdca, 0xdb8, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0xdba, 0xdd4, 0xd9a, 0xddc, 0xdb1, 0xdca, 0x20,
0xdc0, 0xdda, 0xdbd, 0xdcf, 0xdc0, 0x61, 0x63, 0x72, 0x65, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x63, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x66,
0x67, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74,
0x72, 0x65, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6a, 0x75, 0x68, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f,
0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1,
0x70, 0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6a,
0x61, 0x161, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6a, 0x61,
0x161, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x6c, 0x6d, 0x61, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6d, 0x61, 0x61, 0x74, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x6c, 0x6d, 0x61, 0x61, 0x74, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0xe1, 0x6c, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76,
0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0xe1, 0x6c, 0x6e, 0x79, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0xe1, 0x6c, 0x6e, 0x79,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65,
0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd,
0x20, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64,
0x6e, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x6b, 0xfd, 0x20, 0x68, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72,
0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x74, 0x69,
0x63, 0x68, 0x6f, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x6d, 0x6f, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x76, 0x65, 0x72, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x74, 0x69, 0x63,
0x68, 0x6f, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x70, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x61, 0x70, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6b, 0x74, 0x61, 0x75, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6b, 0x74, 0x61, 0x75, 0x73, 0x6b,
0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x6b, 0x74, 0x61, 0x75, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6b,
0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6b,
0x74, 0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6b, 0x74,
0x6f, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74,
0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61,
0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61,
0x72, 0x67, 0x65, 0x6e, 0x74, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0xe9, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0xe9, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x61, 0x72, 0x6d, 0xe9, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f,
0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72,
0x65, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b,
0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x74, 0x72, 0x65, 0x64, 0x6f, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6e, 0xfd,
0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x7a, 0xe1, 0x70,
0x61, 0x64, 0x6e, 0xfd, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c,
0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f,
0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6e, 0xfd, 0x20, 0x61, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61,
0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74,
0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63,
0x68, 0x6f, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73,
0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1,
0x6c, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61,
0x64, 0x6f, 0x61, 0x75, 0x73, 0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x75, 0x73,
0x74, 0x72, 0xe1, 0x6c, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a,
0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x61,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64,
0x17e, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x7a, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0xe9, 0x161, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x62,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9, 0x161, 0x73, 0x6b, 0x79, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9, 0x161, 0x73,
0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x62, 0x68, 0x75, 0x74, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x62, 0x6f, 0x6c, 0xed, 0x76, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x62, 0x72, 0x61, 0x7a, 0xed, 0x6c, 0x73, 0x6b, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x61, 0x7a, 0xed, 0x6c, 0x73, 0x6b,
0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x61, 0x7a, 0xed, 0x6c, 0x73, 0x6b,
0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b,
0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c,
0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x69, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x10d, 0x69, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x69, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d,
0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x10d, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x6f, 0x6a, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x56, 0x69, 0x61, 0x6e,
0x6f, 0x10d, 0x6e, 0xe9, 0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x61, 0x10d, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f,
0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f,
0x76, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0xfd, 0x63,
0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0xfd,
0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x6b,
0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6b,
0x75, 0x62, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x75,
0x62, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x44, 0x61, 0x76, 0x69,
0x73, 0x6f, 0x76, 0x65, 0x6a, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63,
0x65, 0x10d, 0x61, 0x73, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x20, 0x64, 0x2019, 0x55,
0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x61, 0x10d, 0x61, 0x73, 0x20, 0x56,
0x65, 0x13e, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xe9, 0x68, 0x6f, 0x20,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x56, 0x65,
0x13e, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0xe9, 0x68, 0x6f, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x20, 0x56, 0x65, 0x13e, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d,
0x6e, 0xe9, 0x68, 0x6f, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61,
0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x73,
0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f,
0x65, 0x75, 0x72, 0xf3, 0x70, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x74, 0x72, 0x65, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x73, 0x6b,
0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76,
0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x73,
0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64,
0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70,
0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x73, 0x6b,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x65,
0x75, 0x72, 0xf3, 0x70, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1,
0x70, 0x61, 0x64, 0x6f, 0x65, 0x75, 0x72, 0xf3, 0x70, 0x73, 0x6b, 0x79,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x64,
0x17e, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x64,
0x17e, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x66, 0x72, 0x61, 0x6e, 0x63, 0xfa, 0x7a, 0x73,
0x6b, 0x6f, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63, 0xfa,
0x7a, 0x73, 0x6b, 0x79, 0x63, 0x68, 0x20, 0x6a, 0x75, 0x17e, 0x6e, 0xfd,
0x63, 0x68, 0x20, 0x61, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74,
0x69, 0x63, 0x6b, 0xfd, 0x63, 0x68, 0x20, 0xfa, 0x7a, 0x65, 0x6d, 0xed,
0x67, 0x61, 0x6c, 0x61, 0x70, 0xe1, 0x67, 0x73, 0x6b, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x67, 0x72, 0x75, 0x7a, 0xed, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61,
0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x76, 0xfd,
0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x67,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72,
0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63,
0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf3,
0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e,
0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64,
0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0xe1, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72, 0xf3, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x75,
0x61, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x20,
0x50, 0x65, 0x72, 0x7a, 0x73, 0x6b, 0xe9, 0x68, 0x6f, 0x20, 0x7a, 0xe1,
0x6c, 0x69, 0x76, 0x75, 0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f,
0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x68, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65,
0x75, 0x74, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x6f, 0x6e, 0x67, 0x6b, 0x6f,
0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x63, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x63, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x63, 0x68, 0x6f, 0x76, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64, 0x69, 0x63,
0x6b, 0x6f, 0x6f, 0x63, 0x65, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x69, 0x6e, 0x64, 0x6f, 0x10d, 0xed, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x74, 0x72, 0x65, 0x64, 0x6f, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0xe9, 0x7a, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0xe9, 0x7a, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70,
0x61, 0x64, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0xe9, 0x7a, 0x73, 0x6b,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0xe1, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x69, 0x72, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x69, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x6f, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x70, 0x6f, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0xfd, 0x63, 0x68, 0x6f, 0x64,
0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0xe1, 0x70, 0x61, 0x64, 0x6f,
0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0xf3, 0x72, 0x65, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0xf3, 0x72, 0x65, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6b, 0xf3, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b,
0x69, 0x72, 0x67, 0x69, 0x7a, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x72, 0xed, 0x6c, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x10d, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x76, 0x6e, 0xed, 0x6b, 0x6f,
0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f,
0x76, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x61, 0x20,
0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x48, 0x6f, 0x77, 0x61, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x73, 0x74, 0x72,
0x6f, 0x76, 0x61, 0x20, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20, 0x48, 0x6f,
0x77, 0x61, 0x6d, 0x61, 0x63, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6d, 0x61, 0x63, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x61, 0x63, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65,
0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x6c, 0x61, 0x6a, 0x7a, 0x69, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x72, 0x6b,
0xe9, 0x7a, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f, 0x76, 0xfd,
0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x6d,
0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6d, 0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x6a, 0x73, 0x6b,
0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x75, 0x72, 0xed, 0x63, 0x69, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x10d, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x6f,
0x76, 0x65, 0x6a, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65, 0x6d,
0x65, 0x78, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x74, 0x69, 0x63, 0x68, 0x6f,
0x6d, 0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65,
0x78, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x6d,
0x6f, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69,
0x63, 0x6b, 0xfd, 0x20, 0x74, 0x69, 0x63, 0x68, 0x6f, 0x6d, 0x6f, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0xe1, 0x74, 0x61, 0x72, 0x73, 0x6b,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x6c, 0x61, 0x6e, 0x62, 0xe1, 0x74,
0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x6c, 0x61, 0x6e,
0x62, 0xe1, 0x74, 0x61, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x76,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x6f,
0x76, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x6f, 0x73, 0x6b, 0x6f,
0x76, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6d, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6e, 0x61, 0x75, 0x72, 0x75, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6e, 0x65, 0x70, 0xe1, 0x6c, 0x73, 0x6b, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f,
0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0xf3, 0x6e, 0x73, 0x6b, 0x79,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64,
0xf3, 0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x7a, 0xe9, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x7a,
0xe9, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x73, 0x6b, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x6e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73,
0x20, 0x73, 0xfa, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x69, 0x61, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x73, 0xfa,
0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x69, 0x61, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x20, 0x73, 0xfa, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x69, 0x61,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x73, 0x65, 0x76, 0x65,
0x72, 0x6f, 0x6d, 0x61, 0x72, 0x69, 0xe1, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x6f, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6f, 0x6d, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x50, 0x61, 0x70,
0x75, 0x79, 0x2d, 0x4e, 0x6f, 0x76, 0x65, 0x6a, 0x20, 0x47, 0x75, 0x69,
0x6e, 0x65, 0x79, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e, 0x73,
0x6b, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e,
0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0xe1, 0x6e,
0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x66, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x66, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x6c, 0x69, 0x70, 0xed, 0x6e, 0x73,
0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x6f, 0x76, 0xfd,
0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x70,
0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x6d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x69, 0x65, 0x72,
0x72, 0x65, 0x2d, 0x6d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e,
0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f, 0x76,
0x6f, 0x76, 0x70, 0x63, 0x68, 0x6a, 0x6f, 0x6e, 0x67, 0x6a, 0x61, 0x6e,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x79, 0x7a, 0x79, 0x6c,
0x6f, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x73, 0x6b, 0xfd, 0x20,
0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73,
0x20, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x6f, 0x76, 0x65, 0x6a, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x63, 0x65, 0x73, 0x61, 0x63, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x61, 0x6d, 0x6f, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61,
0x6d, 0x6f, 0x6a, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x160, 0x61,
0x6c, 0x61, 0x6d, 0xfa, 0x6e, 0x6f, 0x76, 0xfd, 0x63, 0x68, 0x20, 0x6f,
0x73, 0x74, 0x72, 0x6f, 0x76, 0x6f, 0x76, 0x10d, 0x61, 0x73, 0x20, 0x4a,
0x75, 0x17e, 0x6e, 0x65, 0x6a, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x65, 0x74, 0x63, 0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x74, 0x63, 0x68, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x74, 0x61, 0x64, 0x17e, 0x69, 0x63, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x74, 0x6f, 0x6e, 0x17e, 0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f,
0x6e, 0x17e, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0xe9, 0x6e, 0x73, 0x6b,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0xe9, 0x6e,
0x73, 0x6b, 0x79, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0xe9,
0x6e, 0x73, 0x6b, 0x79, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x6a, 0x73, 0x6b, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x6a, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x6a,
0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x75, 0x7a, 0x62, 0x65, 0x63, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x75,
0x7a, 0x62, 0x65, 0x63, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b,
0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd,
0x20, 0x10d, 0x61, 0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x73, 0x6b,
0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x63, 0x6b, 0xfd,
0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x76, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x63, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73,
0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0xfd, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20,
0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x73, 0x74, 0x72, 0x6f,
0x76, 0x6f, 0x76, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x6a, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0xfd, 0x20, 0x161, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d, 0x61, 0x73,
0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x73, 0x6b, 0xfd, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x73, 0x6b, 0xfd, 0x20, 0x161,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0xfd, 0x20, 0x10d, 0x61,
0x73, 0x6a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x73, 0x6b, 0xfd, 0x20, 0x6c, 0x65, 0x74, 0x6e, 0xfd, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x161, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x61, 0x66,
0x72, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x75, 0x17e,
0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69,
0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64,
0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x161,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x41, 0x6c, 0x6a, 0x61, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x6c, 0x6a, 0x61, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41,
0x6c, 0x6a, 0x61, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56,
0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x47, 0x6f, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x47, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x47, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x63, 0x69, 0x66,
0x69, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x63, 0x69,
0x66, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6e, 0x61, 0x64, 0x69, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x41, 0x70, 0x69, 0x61,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x3a, 0x20, 0x41, 0x70, 0x69, 0x61, 0x50, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x41, 0x70, 0x69,
0x61, 0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x41, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72,
0x61, 0x62, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68,
0x6f, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x74, 0x6c, 0x61,
0x6e, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x76, 0x7a, 0x68, 0x6f, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x76, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x76, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64,
0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x6a, 0x64, 0x17e, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41,
0x7a, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a,
0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x7a, 0x6f, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x75, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x6f, 0x6c, 0x69, 0x76,
0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72,
0x61, 0x73, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x61, 0x70, 0x76,
0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x61,
0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b,
0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x6d,
0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x74, 0x61,
0x6d, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x74, 0x61,
0x6d, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x74, 0x61, 0x6d,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x10c, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x10c, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x69,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4b, 0x69, 0x74, 0x61, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4b, 0x69, 0x74, 0x61, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4b, 0x69, 0x74, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c,
0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x10c, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x6f, 0x6a, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x6f, 0x17e, 0x69, 0x10d, 0x6e,
0x6f, 0x6f, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c,
0x61, 0x73, 0x3a, 0x20, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x6f, 0x76, 0x69,
0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62,
0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x6f, 0x6c,
0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x43, 0x6f,
0x6f, 0x6b, 0x6f, 0x76, 0x6f, 0x6f, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x76, 0x6f, 0x6f, 0x74,
0x6f, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x43, 0x6f, 0x6f, 0x6b, 0x6f,
0x76, 0x6f, 0x6f, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x72, 0x65,
0x64, 0x6e, 0x6a, 0x65, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4b, 0x75, 0x62, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61,
0x73, 0x3a, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x10c, 0x61, 0x73, 0x3a,
0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72,
0x76, 0x69, 0x6c, 0x6c, 0x65, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f,
0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x10c, 0x61, 0x73, 0x3a, 0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f, 0x6e,
0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a,
0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f, 0x6e, 0x6f, 0x10d, 0x6e, 0x69,
0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x50, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x56, 0x65, 0x6c, 0x69, 0x6b, 0x6f,
0x6e, 0x6f, 0x10d, 0x6e, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x45, 0x6b,
0x76, 0x61, 0x64, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x72, 0x65, 0x64, 0x6e,
0x6a, 0x65, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x53, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x65, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56,
0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x44, 0x6f, 0x64, 0x61,
0x74, 0x6e, 0x69, 0x20, 0x76, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a,
0x61, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e,
0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x65, 0x76, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6f, 0x74, 0x6f, 0x10d,
0x6a, 0x65, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x3a, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x6f, 0x20, 0x6f, 0x74, 0x6f, 0x10d, 0x6a, 0x65,
0x50, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a,
0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x6f,
0x20, 0x6f, 0x74, 0x6f, 0x10d, 0x6a, 0x65, 0x46, 0x69, 0x64, 0x17e, 0x69,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46, 0x69, 0x64, 0x17e,
0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46, 0x69, 0x64, 0x17e,
0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x63, 0x6f, 0x73, 0x6b, 0x61, 0x20, 0x47, 0x76, 0x61, 0x6a,
0x61, 0x6e, 0x61, 0x46, 0x72, 0x61, 0x6e, 0x63, 0x6f, 0x73, 0x6b, 0x69,
0x20, 0x6a, 0x75, 0x17e, 0x6e, 0x69, 0x20, 0x69, 0x6e, 0x20, 0x61, 0x6e,
0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x10d, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x161, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x6f, 0x76, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x72,
0x65, 0x64, 0x6e, 0x6a, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68,
0x6f, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64,
0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x67,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61,
0x68, 0x6f, 0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f,
0x64, 0x6e, 0x6f, 0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x6f,
0x67, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a,
0x61, 0x6c, 0x69, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x47, 0x76,
0x61, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48,
0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x61, 0x76, 0x61,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x48, 0x61, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f, 0x6e,
0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f,
0x76, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f, 0x76,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f, 0x76, 0x64, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x49, 0x6e, 0x64, 0x69, 0x6a, 0x73, 0x6b, 0x6f, 0x6f, 0x63, 0x65,
0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x6e, 0x64,
0x6f, 0x6b, 0x69, 0x74, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x6f, 0x73, 0x72, 0x65, 0x64, 0x6e, 0x6a, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x76, 0x7a, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x7a, 0x61, 0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x49, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x49,
0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a,
0x61, 0x70, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a,
0x61, 0x70, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x61,
0x70, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70,
0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x10d,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x10d, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x7a,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x5a, 0x61,
0x68, 0x6f, 0x64, 0x6e, 0x69, 0x20, 0x6b, 0x61, 0x7a, 0x61, 0x68, 0x73,
0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x6f,
0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x6f,
0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x6f, 0x72,
0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x6a, 0x161,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20,
0x45, 0x6b, 0x76, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6f,
0x74, 0x6f, 0x6b, 0x69, 0x10c, 0x61, 0x73, 0x20, 0x6f, 0x74, 0x6f, 0x6b,
0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x50, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x61, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x6c,
0x65, 0x7a, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x10c, 0x61, 0x73, 0x3a, 0x20, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x6e,
0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x10c, 0x61, 0x73, 0x3a, 0x20,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f, 0x76, 0x69, 0x20,
0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x63, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x65,
0x68, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69,
0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d, 0x65, 0x68, 0x69, 0x161,
0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x161, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4d, 0x65, 0x68, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x70,
0x61, 0x63, 0x69, 0x66, 0x69, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e,
0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x6f, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4d, 0x6a, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x4e, 0x6f, 0x76, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x6b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e,
0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x7a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f,
0x7a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76,
0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x66, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f,
0x76, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4e, 0x69, 0x75, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x161, 0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x3a,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x161, 0x6b, 0x69, 0x20, 0x6f,
0x74, 0x6f, 0x6b, 0x69, 0x50, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x3a, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x161,
0x6b, 0x69, 0x20, 0x6f, 0x74, 0x6f, 0x6b, 0x69, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4f, 0x6d,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x50, 0x61, 0x6c, 0x61, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x50, 0x61, 0x70, 0x75, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x76,
0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x72, 0x75, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x72, 0x75, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x50, 0x65, 0x72, 0x75, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x4f, 0x74, 0x6f, 0x10d, 0x6a, 0x65,
0x20, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x69, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x3a, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x69, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x50, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x3a, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x69, 0x6e, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x6f, 0x6e, 0x61,
0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x6a, 0x6f, 0x6e,
0x67, 0x6a, 0x61, 0x6e, 0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x52,
0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x65, 0x6a, 0x161,
0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x69, 0x6e,
0x67, 0x61, 0x70, 0x75, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53,
0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x6f, 0x76, 0x6f, 0x6f, 0x74, 0x6f,
0x161, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x75, 0x17e, 0x6e, 0x6f,
0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x53, 0x79, 0x6f,
0x77, 0x61, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x6a, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x54, 0x61, 0x64, 0x17e, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x6f,
0x6e, 0x67, 0x6f, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54,
0x6f, 0x6e, 0x67, 0x6f, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54,
0x6f, 0x6e, 0x67, 0x6f, 0x76, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a,
0x20, 0x4f, 0x74, 0x6f, 0x6b, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x72, 0x75, 0x67, 0x76,
0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x72, 0x75, 0x67,
0x76, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6a,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c,
0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x65, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x161, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x161, 0x6b, 0x69, 0x20, 0x70, 0x6f,
0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x6a, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x6a, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x161, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x4f, 0x74, 0x6f,
0x6b, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x10c, 0x61, 0x73, 0x3a, 0x20, 0x57,
0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x69, 0x6e, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x70,
0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x161, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x161,
0x6b, 0x69, 0x20, 0x70, 0x6f, 0x6c, 0x65, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4a, 0x75, 0x6b, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41,
0x63, 0x72, 0x65, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65,
0x65, 0x20, 0x41, 0x63, 0x72, 0x65, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69,
0x67, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6b, 0x61, 0x20, 0x65, 0x65, 0x20,
0x41, 0x63, 0x72, 0x65, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x41, 0x66, 0x67, 0x67, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72,
0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72,
0x69, 0x67, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x79, 0x61, 0x64, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x6f,
0x6f, 0x6e, 0x66, 0x75, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62,
0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x47, 0x61,
0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61,
0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x47, 0x61, 0x6c,
0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61,
0x20, 0x65, 0x65, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x6c,
0x6d, 0x61, 0x74, 0x79, 0x53, 0x61, 0x61, 0x63, 0x61, 0x64, 0x61, 0x20,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x61,
0x79, 0x6c, 0x61, 0x68, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41,
0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20,
0x65, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x41, 0x6d, 0x61, 0x73, 0x6f, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d,
0x61, 0x68, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67,
0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x74,
0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79,
0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72,
0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72,
0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f,
0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72,
0x69, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f,
0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42,
0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79,
0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72,
0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72,
0x69, 0x67, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67,
0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x75, 0x75, 0x72, 0x61, 0x6c,
0x65, 0x79, 0x64, 0x61, 0x20, 0x65, 0x65, 0x20, 0x57, 0x61, 0x71, 0x6f,
0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20,
0x42, 0x75, 0x75, 0x72, 0x61, 0x6c, 0x65, 0x79, 0x64, 0x61, 0x20, 0x57,
0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65,
0x65, 0x20, 0x42, 0x75, 0x75, 0x72, 0x6c, 0x65, 0x79, 0x64, 0x61, 0x20,
0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d,
0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x42, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x65,
0x65, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20,
0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61,
0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72,
0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x61, 0x20, 0x57, 0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61,
0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x6b,
0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67,
0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6b, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x41, 0x62, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x41, 0x62, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72,
0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x62, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x71,
0x74, 0x61, 0x75, 0x53, 0x61, 0x61, 0x63, 0x61, 0x64, 0x61, 0x20, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x61, 0x79,
0x6c, 0x61, 0x68, 0x61, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x71,
0x74, 0x6f, 0x62, 0x65, 0x53, 0x61, 0x61, 0x63, 0x61, 0x64, 0x61, 0x20,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x6b, 0x75, 0x6c, 0x61,
0x79, 0x6c, 0x61, 0x68, 0x61, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x72, 0x61,
0x62, 0x74, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x43, 0x61, 0x72, 0x61, 0x62, 0x74, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74,
0x61, 0x20, 0x65, 0x65, 0x20, 0x43, 0x61, 0x72, 0x61, 0x62, 0x74, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e,
0x74, 0x69, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20,
0x65, 0x65, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67,
0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x72, 0x6a, 0x65,
0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20,
0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x62,
0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74,
0x69, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x47,
0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x72, 0x6a,
0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x69, 0x79, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x72,
0x6d, 0x65, 0x65, 0x6e, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x61, 0x20, 0x65, 0x65, 0x20, 0x57, 0x61, 0x71, 0x6f,
0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x57, 0x61,
0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x57,
0x61, 0x71, 0x6f, 0x6f, 0x79, 0x69, 0x67, 0x61, 0x20, 0x41, 0x6d, 0x65,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x41,
0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x74,
0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65,
0x65, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20,
0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61,
0x6d, 0x61, 0x68, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64,
0x6b, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x47, 0x61,
0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61,
0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61,
0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x47,
0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x41, 0x73,
0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x79, 0x61, 0x64, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72,
0x69, 0x67, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44,
0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20,
0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72,
0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20,
0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x47, 0x61, 0x6c,
0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x41, 0x73, 0x74, 0x61, 0x72,
0x61, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20,
0x65, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61,
0x20, 0x41, 0x73, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x41, 0x73, 0x61, 0x72,
0x62, 0x65, 0x79, 0x6a, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x41, 0x73, 0x61, 0x72, 0x62, 0x65, 0x79, 0x6a,
0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61,
0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x73, 0x61,
0x72, 0x62, 0x65, 0x79, 0x6a, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x41, 0x73, 0x6f, 0x72, 0x65, 0x73, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x41, 0x73, 0x6f, 0x72, 0x65,
0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67,
0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x41, 0x73, 0x6f, 0x72,
0x65, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x65, 0x64, 0x65, 0x65, 0x73, 0x68, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x65, 0x64, 0x65, 0x65, 0x73, 0x68, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x65, 0x64, 0x65, 0x65, 0x73, 0x68,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x75, 0x74, 0x61,
0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x6f,
0x6c, 0x69, 0x69, 0x66, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20,
0x42, 0x61, 0x72, 0x61, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x61, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x42, 0x75, 0x72, 0x75, 0x6e, 0x65, 0x79, 0x20, 0x44, 0x61, 0x61,
0x72, 0x75, 0x73, 0x61, 0x6c, 0x61, 0x61, 0x6d, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x4b, 0x65, 0x79, 0x62, 0x20, 0x46, 0x61, 0x61,
0x72, 0x64, 0x65, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x4b, 0x65, 0x79, 0x62, 0x20, 0x46, 0x61, 0x61, 0x72, 0x64, 0x65,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61,
0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x65, 0x79, 0x62, 0x20,
0x46, 0x61, 0x61, 0x72, 0x64, 0x65, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x4a, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x4a, 0x61, 0x61, 0x74, 0x61, 0x6d, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4a, 0x61, 0x61, 0x74,
0x61, 0x6d, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68,
0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4a,
0x61, 0x61, 0x74, 0x61, 0x6d, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4a, 0x69, 0x6c, 0x69, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20,
0x65, 0x65, 0x20, 0x4a, 0x69, 0x6c, 0x69, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x4a, 0x69, 0x6c, 0x69, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x53, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x68, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x53, 0x68, 0x69, 0x69,
0x6e, 0x61, 0x68, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65,
0x20, 0x53, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x68, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x4a, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x4a, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x69, 0x72,
0x69, 0x73, 0x6d, 0x61, 0x73, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x6f, 0x6b, 0x6f,
0x73, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x69, 0x79,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b,
0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x4b, 0x6f, 0x6c, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x75, 0x6b,
0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x75, 0x75, 0x6b, 0x20,
0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x4e, 0x75, 0x73, 0x20, 0x58, 0x61, 0x67, 0x61,
0x61, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x75,
0x75, 0x6b, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x75, 0x62, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x75,
0x75, 0x62, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44,
0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20,
0x4b, 0x75, 0x75, 0x62, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x44, 0x61, 0x66, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x2d, 0x20, 0x64,
0x2019, 0x75, 0x72, 0x66, 0x69, 0x6c, 0x6c, 0x65, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x49, 0x69, 0x73, 0x74, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x6f, 0x72, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x49,
0x69, 0x73, 0x74, 0x61, 0x72, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x49, 0x69,
0x73, 0x74, 0x61, 0x72, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x49, 0x69, 0x73, 0x74, 0x61, 0x72,
0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x45, 0x6b, 0x75, 0x77, 0x61, 0x64, 0x6f, 0x6f, 0x72,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74,
0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75, 0x62, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72,
0x74, 0x61, 0x6d, 0x61, 0x68, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75, 0x62,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61,
0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61,
0x6d, 0x61, 0x68, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75, 0x62, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61,
0x20, 0x59, 0x75, 0x72, 0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x59,
0x75, 0x72, 0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42,
0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75, 0x62, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67,
0x61, 0x20, 0x46, 0x6f, 0x67, 0x20, 0x65, 0x65, 0x20, 0x59, 0x75, 0x72,
0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61,
0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75,
0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x47,
0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x59, 0x75, 0x72,
0x75, 0x62, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61,
0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x47, 0x61, 0x6c,
0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x59, 0x75, 0x72, 0x75, 0x62,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x61, 0x61, 0x6c,
0x6b, 0x6c, 0x61, 0x61, 0x6e, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e,
0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x46, 0x61, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x61, 0x6e, 0x20, 0x41, 0x79,
0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x46, 0x61, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x61, 0x6e, 0x20, 0x41,
0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x46, 0x65, 0x72, 0x65, 0x6e, 0x6a, 0x20, 0x47,
0x75, 0x79, 0x61, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x66, 0x75, 0x72, 0x74, 0x61, 0x20, 0x46,
0x61, 0x72, 0x61, 0x6e, 0x73, 0x69, 0x69, 0x73, 0x6b, 0x61, 0x20, 0x26,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x62,
0x61, 0x67, 0x6f, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x47, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x72, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x4a, 0x6f, 0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4a, 0x6f,
0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65,
0x20, 0x4a, 0x6f, 0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x69, 0x6c, 0x62, 0x65, 0x65, 0x72,
0x74, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x68,
0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x77,
0x69, 0x6a, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61,
0x72, 0x69, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x47, 0x69, 0x72, 0x69,
0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68,
0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x47,
0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x47,
0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64,
0x6b, 0x61, 0x20, 0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x47,
0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20, 0x47, 0x69, 0x72,
0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65, 0x65, 0x64, 0x6b, 0x61, 0x20,
0x47, 0x69, 0x72, 0x69, 0x69, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x63, 0x61, 0x6e, 0x6b,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x75, 0x79,
0x61, 0x61, 0x6e, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x48, 0x61, 0x77, 0x61, 0x61, 0x79, 0x2d, 0x41, 0x6c, 0x75, 0x74, 0x69,
0x79, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x61, 0x79, 0x2d, 0x41, 0x6c, 0x75,
0x74, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20,
0x65, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x61, 0x79, 0x2d, 0x41, 0x6c,
0x75, 0x74, 0x69, 0x79, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x48, 0x6f, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6f,
0x6e, 0x67, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x48, 0x6f, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x67, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x48, 0x6f, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6f, 0x6e, 0x67, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x48, 0x6f, 0x66, 0x75, 0x64, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x48, 0x6f, 0x66, 0x75, 0x64, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x48, 0x6f, 0x66, 0x75, 0x64, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x48, 0x69, 0x6e, 0x64, 0x69,
0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61,
0x64, 0x77, 0x65, 0x79, 0x6e, 0x74, 0x61, 0x20, 0x48, 0x69, 0x6e, 0x64,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x73, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x72, 0x74, 0x61, 0x6d, 0x61,
0x68, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x79, 0x73,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x79, 0x73, 0x69, 0x79, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x62, 0x65,
0x65, 0x64, 0x6b, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65,
0x79, 0x73, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x49, 0x69, 0x72, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x49, 0x69, 0x72, 0x61, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61,
0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x49, 0x69, 0x72, 0x61,
0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x69, 0x6b,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61,
0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x49,
0x73, 0x72, 0x61, 0x61, 0x69, 0x69, 0x6c, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x49, 0x73, 0x72, 0x61, 0x61, 0x69, 0x69,
0x6c, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61,
0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x49, 0x73,
0x72, 0x61, 0x61, 0x69, 0x69, 0x6c, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x4a, 0x61, 0x62, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4a, 0x61, 0x62, 0x61, 0x61, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72,
0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4a, 0x61, 0x62,
0x61, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x57,
0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6b,
0x61, 0x20, 0x65, 0x65, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68,
0x61, 0x74, 0x73, 0x6b, 0x69, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x42, 0x61, 0x72, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x61, 0x73, 0x61,
0x6b, 0x68, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x6e, 0x66, 0x75, 0x72, 0x74,
0x61, 0x20, 0x4b, 0x61, 0x73, 0x61, 0x6b, 0x68, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x75,
0x72, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x4b, 0x75, 0x75, 0x72, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72,
0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x75, 0x75, 0x72, 0x69, 0x79,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x6f, 0x73,
0x72, 0x69, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b,
0x61, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x69, 0x6b,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x61,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x69, 0x6b, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x4b, 0x69, 0x79, 0x72, 0x69, 0x67, 0x69, 0x73, 0x74,
0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4c,
0x65, 0x79, 0x6e, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4c, 0x6f, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4c,
0x6f, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20,
0x65, 0x65, 0x20, 0x4c, 0x6f, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x57, 0x61,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x65, 0x64, 0x61, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4d, 0x61,
0x67, 0x65, 0x64, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20,
0x4d, 0x61, 0x67, 0x65, 0x64, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x65, 0x79, 0x73, 0x68, 0x69, 0x79,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x66, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x77, 0x65, 0x73, 0x61, 0x73, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x61, 0x61, 0x72, 0x73,
0x68, 0x61, 0x61, 0x6c, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69,
0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x6f, 0x72,
0x69, 0x73, 0x68, 0x69, 0x79, 0x61, 0x61, 0x73, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x68,
0x69, 0x79, 0x61, 0x61, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20,
0x4d, 0x6f, 0x72, 0x69, 0x73, 0x68, 0x69, 0x79, 0x61, 0x61, 0x73, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x73, 0x6f,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x64, 0x61, 0x20, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x6f, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x42, 0x61, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x64,
0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72,
0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x61, 0x64, 0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x6f, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x55, 0x6c, 0x61,
0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x61, 0x72, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61,
0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x61, 0x72, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65,
0x65, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61,
0x61, 0x72, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x6f,
0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x4d, 0x61, 0x79, 0x61, 0x6e, 0x6d, 0x61,
0x61, 0x72, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4e, 0x61,
0x77, 0x72, 0x6f, 0x6f, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x4e, 0x65, 0x65, 0x62, 0x61, 0x61, 0x6c, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x4b, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x53, 0x69, 0x2019,
0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65,
0x65, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x53, 0x69, 0x2019, 0x6c,
0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44,
0x68, 0x61, 0x72, 0x61, 0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20,
0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x53, 0x69, 0x2019, 0x6c, 0x61, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4e, 0x69, 0x79,
0x75, 0x75, 0x66, 0x6f, 0x6f, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x4e, 0x69, 0x79, 0x75,
0x75, 0x66, 0x6f, 0x6f, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61, 0x72,
0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x66,
0x6f, 0x6f, 0x6e, 0x6c, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20,
0x65, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x6e, 0x64, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x4d, 0x61, 0x61, 0x6c, 0x69, 0x6e, 0x74, 0x61, 0x20, 0x65, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46,
0x61, 0x72, 0x6e, 0x61, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x46, 0x61, 0x72, 0x6e, 0x61, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61,
0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x46, 0x61, 0x72,
0x6e, 0x61, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73,
0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x69, 0x6b,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61,
0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x4e, 0x6f, 0x66, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x69, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41,
0x68, 0x20, 0x65, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x42, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x6c, 0x61,
0x77, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x61, 0x62,
0x75, 0x77, 0x20, 0x4e, 0x69, 0x79, 0x75, 0x75, 0x20, 0x47, 0x69, 0x6e,
0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x42,
0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x61, 0x79, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61, 0x72, 0x61, 0x67, 0x77,
0x61, 0x61, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58,
0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x61,
0x72, 0x61, 0x67, 0x77, 0x61, 0x61, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x42, 0x65, 0x65, 0x72, 0x75, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x42, 0x65, 0x65, 0x72, 0x75, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x42, 0x65, 0x65, 0x72, 0x75, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x62,
0x69, 0x69, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x62, 0x69, 0x69, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x62, 0x69, 0x69, 0x6e,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x6f, 0x6f, 0x6e,
0x69, 0x6b, 0x69, 0x73, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69,
0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x74, 0x2e,
0x20, 0x42, 0x65, 0x65, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x6b,
0x69, 0x77, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x53, 0x74, 0x2e, 0x20, 0x42, 0x65, 0x65, 0x72, 0x65, 0x20, 0x26,
0x20, 0x4d, 0x69, 0x6b, 0x69, 0x77, 0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61,
0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x42,
0x65, 0x65, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x6b, 0x69, 0x77,
0x65, 0x6c, 0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x42, 0x69, 0x74, 0x6b, 0x65, 0x65, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x42, 0x6f, 0x6e, 0x61, 0x62, 0x65, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x42, 0x6f, 0x79, 0x6f, 0x6e, 0x67, 0x79,
0x61, 0x6e, 0x67, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x51,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x61, 0x68, 0x20, 0x65, 0x65, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x53, 0x61, 0x61, 0x63, 0x61, 0x64, 0x61, 0x20, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x61, 0x79,
0x6c, 0x61, 0x68, 0x61, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x52, 0x69,
0x79, 0x75, 0x75, 0x6e, 0x69, 0x79, 0x6f, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61,
0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x57,
0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x57, 0x61, 0x6b, 0x68, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x61, 0x68, 0x20, 0x65, 0x65,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x57, 0x61, 0x6b, 0x68, 0x74,
0x69, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x6c, 0x6b, 0x61, 0x20, 0x65, 0x65,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20,
0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61,
0x61, 0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x69, 0x69,
0x73, 0x68, 0x61, 0x6c, 0x69, 0x73, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x62, 0x75, 0x75, 0x72, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0x20, 0x41, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x6f, 0x77, 0x74, 0x20,
0x4a, 0x6f, 0x6f, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x65, 0x79, 0x6d,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x53, 0x69, 0x79, 0x6f,
0x77, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x61,
0x68, 0x69, 0x74, 0x69, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x54, 0x65, 0x79, 0x62, 0x65, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68,
0x20, 0x65, 0x65, 0x20, 0x54, 0x65, 0x79, 0x62, 0x65, 0x79, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x44, 0x68, 0x61, 0x72, 0x61, 0x61,
0x72, 0x74, 0x61, 0x20, 0x65, 0x65, 0x20, 0x54, 0x65, 0x79, 0x62, 0x65,
0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x61, 0x6a,
0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x75, 0x75, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61,
0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x4a, 0x75,
0x75, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69,
0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x54, 0x75, 0x66, 0x61, 0x6c, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67,
0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x61, 0x79, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67,
0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x55, 0x72, 0x75, 0x67,
0x77, 0x61, 0x61, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20,
0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x55,
0x72, 0x75, 0x67, 0x77, 0x61, 0x61, 0x79, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x55,
0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x71,
0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61,
0x20, 0x65, 0x65, 0x20, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x61,
0x6e, 0x75, 0x75, 0x74, 0x75, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61,
0x20, 0x43, 0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20,
0x65, 0x65, 0x20, 0x46, 0x61, 0x6e, 0x75, 0x75, 0x74, 0x75, 0x57, 0x61,
0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67,
0x61, 0x20, 0x65, 0x65, 0x20, 0x46, 0x61, 0x6e, 0x75, 0x75, 0x74, 0x75,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x65, 0x6e, 0x65,
0x7a, 0x75, 0x77, 0x65, 0x65, 0x6c, 0x61, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x46, 0x61, 0x6c, 0x61, 0x64, 0x69, 0x66, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43,
0x61, 0x61, 0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65,
0x20, 0x46, 0x61, 0x6c, 0x61, 0x64, 0x69, 0x66, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67,
0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x46, 0x61, 0x6c, 0x61,
0x64, 0x69, 0x66, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x46, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61,
0x64, 0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x46,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x57, 0x61, 0x71, 0x74,
0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20,
0x65, 0x65, 0x20, 0x46, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x46, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x57, 0x61,
0x79, 0x6b, 0x20, 0x49, 0x79, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x73, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x57, 0x61, 0x6c, 0x69, 0x73,
0x20, 0x26, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x57, 0x61, 0x71,
0x74, 0x69, 0x79, 0x61, 0x64, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58,
0x61, 0x67, 0x61, 0x61, 0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x61, 0x61, 0x67,
0x57, 0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x43, 0x61, 0x61, 0x64,
0x69, 0x67, 0x61, 0x20, 0x41, 0x68, 0x20, 0x65, 0x65, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x61, 0x61, 0x67, 0x57,
0x61, 0x71, 0x74, 0x69, 0x67, 0x61, 0x20, 0x58, 0x61, 0x67, 0x61, 0x61,
0x67, 0x61, 0x20, 0x65, 0x65, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x61, 0x61, 0x67, 0x57, 0x61, 0x71, 0x74, 0x69,
0x67, 0x61, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61,
0x73, 0x20, 0x52, 0x6f, 0x63, 0x6f, 0x73, 0x61, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0xf1,
0x61, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x6f, 0x73, 0x61, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x6f, 0x6e, 0x74,
0x61, 0xf1, 0x61, 0x73, 0x20, 0x52, 0x6f, 0x63, 0x6f, 0x73, 0x61, 0x73,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63,
0xed, 0x66, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50,
0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69, 0x63, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x72, 0x69, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1,
0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x41,
0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x41, 0x74, 0x6c, 0xe1, 0x6e, 0x74, 0x69, 0x63, 0x6f,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x41, 0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69, 0x64, 0x61, 0x20,
0x79, 0x20, 0x54, 0x65, 0x72, 0x72, 0x69, 0x74, 0x6f, 0x72, 0x69, 0x6f,
0x73, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x20,
0x46, 0x72, 0x61, 0x6e, 0x63, 0x65, 0x73, 0x65, 0x73, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x77, 0x69, 0x63, 0x68, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x48, 0x61, 0x77, 0xe1, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77,
0xe1, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0xe1, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6f, 0x63, 0xe9,
0x61, 0x6e, 0x6f, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74,
0x61, 0xf1, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x6d,
0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0xf1, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73,
0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x6d, 0x65, 0x64,
0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73,
0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45, 0x73,
0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64,
0x65, 0x6c, 0x20, 0x45, 0x73, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x72,
0x69, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x6c, 0x65, 0x6a, 0x61, 0x6e,
0x6f, 0x20, 0x65, 0x73, 0x74, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f,
0x65, 0x73, 0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x65, 0x73, 0x74, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x69, 0x6f, 0x6e, 0x79,
0x61, 0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0xf3, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x20, 0x57, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x6f,
0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x61, 0x76, 0x69, 0x64, 0x61, 0x64, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73,
0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20,
0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c, 0x76,
0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x20, 0x57, 0x61, 0x6b,
0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72,
0xfa, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x50, 0x65, 0x72, 0xfa, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x41, 0x63, 0x72, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41,
0x63, 0x72, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0xc1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x75, 0x64, 0xe1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0xc1, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0xc1,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0xc1, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x6c, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6e, 0x61, 0x64,
0x79, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61,
0x75, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x61, 0x75, 0x48, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65,
0x48, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x6b, 0x74, 0x6f, 0x62, 0x65, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x63, 0x63,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20,
0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x79, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x79, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x79, 0xe1, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0xe9, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0xe9, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x75, 0x6e,
0xe9, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x62, 0x6f,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x49,
0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x76, 0x69, 0x64,
0x61, 0x64, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61,
0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x6d, 0x65, 0x64,
0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x4f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x73, 0x63, 0x75, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x65, 0x78, 0x74, 0x72, 0x65, 0x6d, 0x6f, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x20, 0x64, 0x65, 0x20, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c,
0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x6c,
0x76, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x46, 0x69, 0x79, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69,
0x79, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x79, 0x69,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47,
0x75, 0x61, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0x61, 0x6e, 0x63,
0x65, 0x73, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x73, 0x20, 0x54, 0x69, 0x65, 0x72, 0x72, 0x61, 0x73, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x73, 0x20, 0x79, 0x20, 0x41,
0x6e, 0x74, 0xe1, 0x72, 0x74, 0x69, 0x63, 0x61, 0x73, 0x20, 0x46, 0x72,
0x61, 0x6e, 0x63, 0x65, 0x73, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73,
0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65,
0x72, 0x74, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e,
0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x6f, 0x63,
0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x6f, 0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4f, 0x63,
0xe9, 0x61, 0x6e, 0x6f, 0x20, 0xcd, 0x6e, 0x64, 0x69, 0x63, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x6f, 0x63, 0x63, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x49, 0x72, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x49,
0x73, 0x72, 0x61, 0x65, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4a, 0x61, 0x70, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4a,
0x61, 0x70, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4a, 0x61,
0x70, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x6b, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x6b,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6a, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6a, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x20, 0x6f,
0x63, 0x63, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4b, 0x69, 0x72, 0x67, 0x75, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x48,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x72, 0x69, 0x20, 0x4c,
0x61, 0x6e, 0x6b, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x73, 0x20, 0x45, 0x73, 0x70, 0xf3, 0x72, 0x61, 0x64, 0x61,
0x73, 0x20, 0x45, 0x63, 0x75, 0x61, 0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c,
0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x63, 0x61, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x63, 0x61, 0x6f, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x63, 0x61, 0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65,
0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x61, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x73, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x71,
0x75, 0x65, 0x73, 0x61, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x63, 0x69, 0x6f, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61,
0x72, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66, 0x69,
0x63, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63, 0x6f,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x50, 0x61, 0x63, 0xed, 0x66,
0x69, 0x63, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0xe9, 0x78, 0x69, 0x63,
0x6f, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe1,
0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x55, 0x6c, 0xe1, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0xfa, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63,
0xfa, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0xfa,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x20, 0x28, 0x42, 0x69, 0x72, 0x6d, 0x61, 0x6e, 0x69,
0x61, 0x29, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75,
0x65, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20,
0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72,
0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x54, 0x65, 0x72, 0x72, 0x61, 0x6e, 0x6f, 0x76, 0x61, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x69, 0x73, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x4d, 0x61, 0x72, 0x69,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x64, 0x65, 0x6c, 0x20, 0x4e, 0x6f, 0x72,
0x74, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64,
0x65, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x6c, 0x61,
0x6f, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x70, 0xfa, 0x61, 0x20, 0x4e, 0x75, 0x65, 0x76, 0x61, 0x20, 0x47, 0x75,
0x69, 0x6e, 0x65, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x72,
0x61, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x73, 0x20, 0x69, 0x73,
0x6c, 0x61, 0x73, 0x20, 0x46, 0xe9, 0x6e, 0x69, 0x78, 0x68, 0x6f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65, 0x64,
0x72, 0x6f, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0xf3,
0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0x65,
0x64, 0x72, 0x6f, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65,
0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20,
0x50, 0x65, 0x64, 0x72, 0x6f, 0x20, 0x79, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0xf3, 0x6e, 0x48, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x51, 0x79,
0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0xed, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0xed, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6a, 0x61, 0x6c, 0xed, 0x6e, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72,
0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64,
0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68,
0x65, 0x6c, 0x6c, 0x65, 0x73, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x73, 0x20, 0x53,
0x61, 0x6c, 0x6f, 0x6d, 0xf3, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65,
0x6c, 0x20, 0x53, 0x75, 0x72, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20,
0x65, 0x73, 0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x54, 0x61, 0x79, 0x69, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e,
0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73,
0x74, 0xe1, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0xe1, 0x6e, 0x68, 0x6f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76, 0x65, 0x72, 0x61, 0x6e,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x68, 0x6f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x69, 0x73, 0x6c, 0x61, 0x20,
0x57, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x79, 0x20, 0x46, 0x75, 0x74,
0x75, 0x6e, 0x61, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x65, 0x73, 0x74, 0xe1,
0x6e, 0x64, 0x61, 0x72, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x76,
0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x68, 0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x76, 0x65, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6d, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x68,
0x6f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x75, 0x6b, 0xf3, 0x6e,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73, 0x75, 0x6d, 0x20, 0x50,
0x61, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x57, 0xe9, 0x74, 0x61, 0x6e, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20,
0x57, 0xe9, 0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55,
0x73, 0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0x57, 0xe9,
0x74, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x50, 0x61, 0x67,
0x75, 0x6e, 0x75, 0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0x50, 0x61, 0x67,
0x75, 0x6e, 0x75, 0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x55, 0x73, 0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0x50, 0x61, 0x67, 0x75, 0x6e, 0x75, 0x6e, 0x67, 0x61, 0x6e, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x55, 0x73, 0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x57, 0x61,
0x6b, 0x74, 0x75, 0x20, 0x55, 0x73, 0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e,
0x61, 0x73, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0x55, 0x73, 0x75, 0x6d, 0x20, 0x50, 0x61,
0x6e, 0x61, 0x73, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0xc9, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61, 0x68, 0x57, 0x61, 0x6b, 0x74,
0x75, 0x20, 0x55, 0x73, 0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73,
0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x65, 0x6e, 0x67, 0x61,
0x68, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0xc9, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57, 0x61, 0x6b, 0x74, 0x75,
0x20, 0x55, 0x73, 0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20,
0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x54, 0x69, 0x6d, 0x75, 0x72, 0x57,
0x61, 0x6b, 0x74, 0x75, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x42,
0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x20, 0xc9, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b, 0x74, 0x75, 0x20, 0x55,
0x73, 0x75, 0x6d, 0x20, 0x50, 0x61, 0x6e, 0x61, 0x73, 0x20, 0xc9, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x42, 0x61, 0x72, 0x61, 0x74, 0x57, 0x61, 0x6b,
0x74, 0x75, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b,
0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69,
0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x4b, 0x75, 0x73, 0x69, 0x6e, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d,
0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d, 0x61, 0x67,
0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x74,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68,
0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74,
0x61, 0x69, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63,
0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x75, 0x6e,
0x74, 0x61, 0x69, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x61, 0x6e, 0x67, 0x61,
0x7a, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x70, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x55, 0x61, 0x72, 0x61, 0x62, 0x75, 0x6e, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x55, 0x61, 0x72, 0x61, 0x62, 0x75, 0x6e, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77,
0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69,
0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69,
0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62,
0x69, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x20, 0x79, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62,
0x69, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x20, 0x79, 0x61,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69,
0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x20, 0x79, 0x61, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69,
0x6b, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73,
0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76,
0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x42,
0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x42,
0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x43,
0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64,
0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72,
0x72, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x55, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x6f,
0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61, 0x20,
0x4b, 0x72, 0x69, 0x73, 0x6d, 0x61, 0x73, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79,
0x61, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76,
0x79, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x6e, 0x75, 0x73, 0x75, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69,
0x73, 0x69, 0x77, 0x61, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x79,
0x61, 0x20, 0x4b, 0x75, 0x62, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b,
0x75, 0x62, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x44, 0x61,
0x76, 0x69, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77,
0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61,
0x20, 0x63, 0x68, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x45, 0x6b, 0x77, 0x61, 0x64, 0x6f, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61,
0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79,
0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c,
0x61, 0x79, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72,
0x69, 0x6b, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61,
0x72, 0x69, 0x6b, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c, 0x61,
0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69,
0x20, 0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69,
0x6b, 0x69, 0x20, 0x7a, 0x61, 0x69, 0x64, 0x69, 0x20, 0x6d, 0x77, 0x61,
0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d,
0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20,
0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20,
0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73,
0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x46, 0x69, 0x6a,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x79, 0x61, 0x20, 0x55,
0x66, 0x61, 0x72, 0x61, 0x6e, 0x73, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x75, 0x73, 0x69, 0x6e, 0x69, 0x20, 0x6d, 0x77, 0x61,
0x20, 0x55, 0x66, 0x61, 0x72, 0x61, 0x6e, 0x73, 0x61, 0x20, 0x6e, 0x61,
0x20, 0x41, 0x6e, 0x74, 0x61, 0x6b, 0x74, 0x69, 0x6b, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a,
0x6f, 0x6a, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x6f,
0x6a, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x4a, 0x6f, 0x6a, 0x69, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76,
0x79, 0x61, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69,
0x63, 0x68, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61,
0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x73, 0x68,
0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72,
0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61,
0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x75, 0x6c, 0x66, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x48,
0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f, 0x6e,
0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x42, 0x61, 0x68, 0x61, 0x72, 0x69, 0x20, 0x48, 0x69,
0x6e, 0x64, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x20,
0x6d, 0x77, 0x61, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68,
0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x49, 0x72, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x61,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63,
0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x61,
0x70, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63,
0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f,
0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x61,
0x6e, 0x67, 0x61, 0x7a, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x4d, 0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x68, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72,
0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68,
0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x73, 0x6b, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x73,
0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69,
0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x4c, 0x69, 0x6e,
0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4c,
0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x65, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x73, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x6f, 0x72, 0x69, 0x73, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x77, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x6d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74,
0x6f, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x6e,
0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x6f, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f,
0x77, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x61,
0x75, 0x72, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65,
0x70, 0x61, 0x6c, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61,
0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20,
0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4e, 0x69, 0x75, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b,
0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4b,
0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61,
0x20, 0x63, 0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x70,
0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75, 0x69, 0x6e, 0x65,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x77, 0x61, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x65,
0x72, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x65, 0x72, 0x75,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x50, 0x65, 0x72, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x55, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x6f, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x55, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x6f,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x55, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x6f, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20,
0x76, 0x79, 0x61, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x79, 0x61, 0x20,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x6e, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e,
0x67, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x52, 0x65, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x52, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x53,
0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4b, 0x69, 0x61, 0x6e, 0x67, 0x61, 0x7a, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x73, 0x68,
0x65, 0x6c, 0x69, 0x73, 0x68, 0x65, 0x6c, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61,
0x20, 0x76, 0x79, 0x61, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x20, 0x4b, 0x75, 0x73, 0x69, 0x6e, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68,
0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x61, 0x6a, 0x69,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x43, 0x68, 0x75, 0x75,
0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x75,
0x76, 0x61, 0x6c, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x55,
0x72, 0x75, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67,
0x77, 0x61, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65,
0x6c, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68, 0x61,
0x20, 0x57, 0x61, 0x6b, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x6a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x59, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x6a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x4d,
0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6a,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x41, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x4a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x6a, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67,
0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x41,
0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d,
0x77, 0x61, 0x20, 0x41, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77,
0x61, 0x20, 0x41, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x20, 0x6d,
0x77, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x69, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69,
0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69,
0x20, 0x6d, 0x77, 0x61, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x61, 0x62, 0x61, 0x6a, 0x61,
0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x61, 0x62,
0x61, 0x6a, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x61, 0x62, 0x61, 0x6a,
0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x76, 0x65, 0x64, 0x65,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x76, 0x65,
0x64, 0x65, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x4b, 0x65, 0x70, 0x75, 0x76, 0x65, 0x64, 0x65, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61,
0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20,
0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x4e, 0x75, 0x73,
0x75, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x56, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x76, 0x79, 0x61, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57,
0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x75,
0x62, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69, 0x77, 0x61, 0x20, 0x63, 0x68,
0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61,
0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61,
0x79, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x74, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x73, 0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x20, 0x6d, 0x77, 0x61,
0x20, 0x55, 0x6c, 0x61, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x67, 0x68, 0x61,
0x72, 0x69, 0x62, 0x69, 0x20, 0x6d, 0x77, 0x61, 0x20, 0x55, 0x6c, 0x61,
0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x69, 0x6f,
0x6a, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x69, 0x6f,
0x6a, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x4a, 0x69, 0x6f, 0x6a, 0x69, 0x61, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4d, 0x61, 0x73,
0x68, 0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20,
0x47, 0x68, 0x75, 0x62, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x49, 0x72, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f,
0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68,
0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x4d, 0x61, 0x73, 0x68,
0x61, 0x72, 0x69, 0x6b, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20,
0x4d, 0x61, 0x67, 0x68, 0x61, 0x72, 0x69, 0x62, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x4d, 0x61, 0x6c, 0x65, 0x73, 0x69, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f,
0x72, 0x69, 0x73, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x55,
0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x61, 0x72, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x74,
0x61, 0x61, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61, 0x61,
0x74, 0x61, 0x72, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61,
0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x20, 0x4d, 0x70, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20,
0x4d, 0x70, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x20, 0x4d, 0x70, 0x79, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4e, 0x79, 0x75, 0x7a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x79, 0x75, 0x7a, 0x69, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x63, 0x68, 0x61, 0x6e, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4e, 0x79, 0x75,
0x7a, 0x69, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x4b, 0x69, 0x73, 0x69,
0x77, 0x61, 0x20, 0x63, 0x68, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x50, 0x65,
0x72, 0x75, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a,
0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20,
0x7a, 0x61, 0x20, 0x55, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x6f,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x69, 0x73, 0x69, 0x77,
0x61, 0x20, 0x76, 0x79, 0x61, 0x20, 0x46, 0x69, 0x6e, 0x69, 0x6b, 0x73,
0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69,
0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a,
0x61, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x63, 0x68, 0x61, 0x6e, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x53, 0x61, 0x61, 0x20,
0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x7a,
0x61, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x6f, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4a, 0x6f, 0x6a, 0x69, 0x61, 0x20, 0x4b,
0x75, 0x73, 0x69, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x7a, 0x61, 0x20, 0x54,
0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x57, 0x61, 0x73, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a,
0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20,
0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61,
0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79,
0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x72,
0x75, 0x67, 0x77, 0x61, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x69, 0x53,
0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x77, 0x61, 0x73, 0x74, 0x61, 0x6e,
0x69, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x69, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d,
0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74,
0x6f, 0x20, 0x7a, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x53, 0x61, 0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72,
0x61, 0x20, 0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61,
0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x53, 0x61,
0x61, 0x20, 0x7a, 0x61, 0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20,
0x79, 0x61, 0x20, 0x4a, 0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x53, 0x61, 0x61, 0x20, 0x7a, 0x61,
0x20, 0x4d, 0x61, 0x6a, 0x69, 0x72, 0x61, 0x20, 0x79, 0x61, 0x20, 0x4a,
0x6f, 0x74, 0x6f, 0x20, 0x7a, 0x61, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x76, 0xe4, 0x73, 0x74,
0x62, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x62, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x62, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0xf6, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x79, 0x64, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73,
0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2c, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x73, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0xf6, 0x73, 0x74, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4b, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x67, 0x61, 0x20, 0x62, 0x65, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x64,
0x4b, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x67, 0x61, 0x20, 0x62, 0x65, 0x72,
0x67, 0x65, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x4b, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x67, 0x61, 0x20, 0x62,
0x65, 0x72, 0x67, 0x65, 0x6e, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x6e, 0x6f,
0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x74, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41, 0x6e,
0x61, 0x64, 0x79, 0x72, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x41, 0x70, 0x69, 0x61, 0x74, 0x69, 0x64, 0x41, 0x70, 0x69, 0x61,
0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x41,
0x70, 0x69, 0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x74, 0x69, 0x64, 0x41, 0x71,
0x74, 0x61, 0x75, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x41, 0x71, 0x74, 0x61, 0x75, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x74, 0x69, 0x64, 0x41,
0x71, 0x74, 0xf6, 0x62, 0x65, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x73, 0x61, 0x75, 0x64, 0x69, 0x61, 0x72,
0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x73, 0x61, 0x75,
0x64, 0x69, 0x61, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x73, 0x61, 0x75, 0x64, 0x69,
0x61, 0x72, 0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf6, 0x73,
0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x6b, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x61, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x74,
0x69, 0x64, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x61, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6e, 0x6f, 0x72, 0x64,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x61,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x76, 0xe4, 0x73, 0x74, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x61,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0xf6, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0xf6, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x76,
0xe4, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x61, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a,
0x64, 0x7a, 0x6a, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x7a, 0x6a, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x61,
0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x7a, 0x6a, 0x61, 0x6e, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x61,
0x7a, 0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x61, 0x7a,
0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x61, 0x7a, 0x6f, 0x72, 0x69, 0x73, 0x6b, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x2c,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x70, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x70, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x4b, 0x61, 0x70, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65,
0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x43,
0x61, 0x73, 0x65, 0x79, 0x74, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x74, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x54, 0x6a, 0x6f, 0x6a, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x6e, 0x74, 0x69, 0x64, 0x54, 0x6a, 0x6f, 0x6a, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x54, 0x6a, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x4a, 0x75, 0x6c, 0xf6, 0x6e, 0x73, 0x20, 0x74, 0x69, 0x64, 0x4b, 0x65,
0x65, 0x6c, 0x69, 0x6e, 0x67, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20,
0x74, 0x69, 0x64, 0x63, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6e,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x43, 0x6f, 0x6f, 0x6b, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74,
0x69, 0x64, 0x43, 0x6f, 0x6f, 0x6b, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x6f,
0x6f, 0x6b, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74,
0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x2d, 0x74,
0x69, 0x64, 0xf6, 0x73, 0x74, 0x74, 0x69, 0x6d, 0x6f, 0x72, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x50, 0xe5, 0x73, 0x6b, 0xf6, 0x74, 0x69,
0x64, 0x50, 0xe5, 0x73, 0x6b, 0xf6, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x50, 0xe5, 0x73, 0x6b, 0xf6, 0x6e,
0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x63,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x63, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0xf6,
0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20,
0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0xf6, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69, 0x73,
0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4b,
0x61, 0x6c, 0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x74, 0x69,
0x64, 0x76, 0xe4, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf6, 0x61, 0x72, 0x6e,
0x61, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0xf6, 0x61, 0x72,
0x6e, 0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x46, 0x69, 0x6a, 0x69, 0x74, 0x69, 0x64, 0x46, 0x69, 0x6a, 0x69,
0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46,
0x69, 0x6a, 0x69, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x6b, 0x61, 0x20, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x61, 0x74, 0x69, 0x64, 0x46, 0x72, 0x61, 0x6e, 0x73,
0x6b, 0x61, 0x20, 0x53, 0x79, 0x64, 0x74, 0x65, 0x72, 0x72, 0x69, 0x74,
0x6f, 0x72, 0x69, 0x65, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64,
0x47, 0x61, 0x6c, 0xe1, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x74, 0x69, 0x64,
0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x74, 0x69, 0x64, 0x4b, 0x69,
0x72, 0x69, 0x62, 0x61, 0x74, 0x69, 0x74, 0x69, 0x64, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74,
0x67, 0x72, 0xf6, 0x6e, 0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0xf6, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0xe4, 0x6e,
0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0xf6, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0xe4, 0x6e, 0x64,
0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x76, 0xe4, 0x73, 0x74, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0xe4, 0x6e, 0x64,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x67, 0x72,
0xf6, 0x6e, 0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x76, 0xe4, 0x73, 0x74, 0x67, 0x72,
0xf6, 0x6e, 0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x47, 0x75, 0x61, 0x6d, 0x74, 0x69,
0x64, 0x50, 0x65, 0x72, 0x73, 0x69, 0x73, 0x6b, 0x61, 0x20, 0x76, 0x69,
0x6b, 0x65, 0x6e, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75,
0x6c, 0x75, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c,
0x75, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x48, 0x6f, 0x6e, 0x6f, 0x6c, 0x75, 0x6c, 0x75, 0x2c, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b,
0x6f, 0x6e, 0x67, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x2c, 0x20,
0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x43, 0x68, 0x6f,
0x76, 0x64, 0x74, 0x69, 0x64, 0x43, 0x68, 0x6f, 0x76, 0x64, 0x2c, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x43, 0x68, 0x6f,
0x76, 0x64, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x42, 0x72, 0x69, 0x74, 0x74, 0x69, 0x73, 0x6b, 0x61, 0x20, 0x49,
0x6e, 0x64, 0x69, 0x73, 0x6b, 0x61, 0x20, 0x6f, 0x63, 0x65, 0x61, 0x6e,
0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x63, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64,
0x76, 0xe4, 0x73, 0x74, 0x69, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x74, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69,
0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x69, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x6d, 0x74, 0x6a,
0x61, 0x74, 0x6b, 0x61, 0x74, 0x69, 0x64, 0x4b, 0x61, 0x6d, 0x74, 0x6a,
0x61, 0x74, 0x6b, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x4b, 0x61, 0x6d, 0x74, 0x6a, 0x61, 0x74, 0x6b, 0x61, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0xf6, 0x73, 0x74, 0x6b, 0x61,
0x7a, 0x61, 0x6b, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69,
0x64, 0x76, 0xe4, 0x73, 0x74, 0x6b, 0x61, 0x7a, 0x61, 0x6b, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x6b, 0x69, 0x72, 0x67, 0x69, 0x7a, 0x69, 0x73,
0x6b, 0x20, 0x74, 0x69, 0x64, 0x53, 0x72, 0x69, 0x20, 0x4c, 0x61, 0x6e,
0x6b, 0x61, 0x74, 0x69, 0x64, 0x4c, 0x69, 0x6e, 0x65, 0xf6, 0x61, 0x72,
0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4d,
0x61, 0x63, 0x61, 0x6f, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x6f,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x63,
0x61, 0x6f, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4d,
0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x72, 0x6e, 0x61, 0x74, 0x69, 0x64,
0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x74, 0x69, 0x64,
0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0xf6, 0x61, 0x72, 0x6e,
0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x74, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x2c,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x6d, 0x65,
0x78, 0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x69, 0x6c,
0x6c, 0x61, 0x68, 0x61, 0x76, 0x73, 0x74, 0x69, 0x64, 0x6d, 0x65, 0x78,
0x69, 0x6b, 0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c,
0x61, 0x68, 0x61, 0x76, 0x73, 0x74, 0x69, 0x64, 0x2c, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x6d, 0x65, 0x78, 0x69, 0x6b,
0x61, 0x6e, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x69, 0x6c, 0x6c, 0x61, 0x68,
0x61, 0x76, 0x73, 0x74, 0x69, 0x64, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x74, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e,
0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x74, 0x69, 0x64,
0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x2c,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x62, 0x75,
0x72, 0x6d, 0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4e,
0x61, 0x75, 0x72, 0x75, 0x74, 0x69, 0x64, 0x6e, 0x65, 0x70, 0x61, 0x6c,
0x65, 0x73, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x4e, 0x79, 0x61,
0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x74,
0x69, 0x64, 0x4e, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x65, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x4e, 0x79, 0x61, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a, 0x65, 0x65, 0x6c, 0xe4, 0x6e,
0x64, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a, 0x65, 0x65,
0x6c, 0xe4, 0x6e, 0x64, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x6e, 0x79, 0x7a, 0x65, 0x65, 0x6c, 0xe4, 0x6e,
0x64, 0x73, 0x6b, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69,
0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e,
0x64, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64,
0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xf6, 0x6e, 0x73, 0x20, 0x74, 0x69,
0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0xf6, 0x6e, 0x73, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0xf6, 0x6e, 0x73, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x74, 0x69, 0x64, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2c, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x46, 0x65, 0x72,
0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72,
0x6f, 0x6e, 0x68, 0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x64, 0x6d, 0x61, 0x72, 0x69, 0x61,
0x6e, 0x65, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d,
0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x74, 0x69,
0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x6f,
0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x50, 0x61, 0x6c, 0x61, 0x75,
0x74, 0x69, 0x64, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x79, 0x61,
0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64,
0x45, 0x6e, 0x64, 0x65, 0x72, 0x62, 0x75, 0x72, 0x79, 0x74, 0x69, 0x64,
0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x2d, 0x74, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x74, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x74, 0x69, 0x64, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x74, 0x69, 0x64, 0x51,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x74, 0x69, 0x64, 0x51,
0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x53,
0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x74, 0x69, 0x64, 0x53, 0x61,
0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x74, 0x69, 0x64, 0x53, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72,
0x74, 0x69, 0x64, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65,
0x72, 0x6e, 0x61, 0x74, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70,
0x6f, 0x72, 0x65, 0x74, 0x69, 0x64, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x73,
0x79, 0x64, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x20, 0x74,
0x69, 0x64, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x74, 0x69, 0x64,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x74, 0x69, 0x64, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x74, 0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x74,
0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x2c, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64,
0x54, 0x61, 0x64, 0x7a, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x74, 0x69, 0x64, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x74, 0x69,
0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x74, 0x69, 0x64, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69,
0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d,
0x61, 0x72, 0x74, 0x69, 0x64, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x74, 0x69,
0x64, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x74, 0x69, 0x64, 0x75, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x69, 0x64, 0x75, 0x7a,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x74, 0x69, 0x64, 0x75, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x6b,
0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x74, 0x69, 0x64, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x2c, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x56, 0x6c, 0x61, 0x64,
0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2c, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61,
0x72, 0x74, 0x69, 0x64, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2c, 0x20, 0x73,
0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74, 0x69, 0x64, 0x57, 0x61, 0x6b, 0x65,
0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74, 0x69, 0x64, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x2d, 0x20, 0x6f, 0x63, 0x68, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0xf6, 0x61, 0x72, 0x6e, 0x61, 0x73, 0x20, 0x74,
0x69, 0x64, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x2c, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74, 0x69, 0x64, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x2c, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x74,
0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x2c, 0x20, 0x73, 0x6f, 0x6d, 0x6d, 0x61, 0x72, 0x74,
0x69, 0x64, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x74, 0x69, 0x64, 0x41, 0x63,
0x72, 0x65, 0x2d, 0x5a, 0x69, 0x69, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d,
0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69,
0x69, 0x74, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65,
0x72, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x2d, 0x5a, 0x69, 0x69, 0x74, 0x5a,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x6e, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x5a, 0x69, 0x69, 0x74, 0x4f,
0x73, 0x63, 0x68, 0x74, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69,
0x73, 0x63, 0x68, 0x69, 0x20, 0x5a, 0x69, 0x69, 0x74, 0x53, 0xfc, 0xfc,
0x64, 0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68,
0x69, 0x20, 0x7a, 0x69, 0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68, 0x74,
0x61, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x69,
0x20, 0x5a, 0x69, 0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68, 0x74, 0x61,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20,
0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69,
0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68, 0x74, 0x61, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x6e, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x2d, 0x5a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x2d, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a,
0x69, 0x69, 0x74, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61,
0x74, 0x79, 0x2d, 0x5a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74,
0x79, 0x2d, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x7a, 0x69, 0x69, 0x74, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x2d, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x5a, 0x69, 0x69, 0x74, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x53, 0x63, 0x68, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x2d, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x69, 0x69, 0x74, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x2d,
0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x5a, 0x69, 0x69,
0x74, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x5a, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x69, 0x74, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x2d, 0x5a, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x4d,
0x69, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69,
0x73, 0x63, 0x68, 0x69, 0x20, 0x5a, 0x69, 0x69, 0x74, 0x4d, 0x69, 0x74,
0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63,
0x68, 0x69, 0x20, 0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x7a, 0x69, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x4f, 0x73, 0x63,
0x68, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68,
0x69, 0x20, 0x5a, 0x69, 0x69, 0x74, 0x4f, 0x73, 0x63, 0x68, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53,
0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x69,
0x74, 0x4f, 0x73, 0x63, 0x68, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4,
0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72,
0x7a, 0x69, 0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68, 0x74, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x5a, 0x69,
0x69, 0x74, 0x57, 0x65, 0x73, 0x63, 0x68, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0xe4, 0x69, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x63, 0x68, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x69, 0x74, 0x57, 0x65,
0x73, 0x63, 0x68, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x73,
0x63, 0x68, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69,
0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20, 0x5a,
0x69, 0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20,
0x53, 0x63, 0x68, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69,
0x69, 0x74, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x65, 0x72, 0x20, 0x53,
0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x69, 0x74, 0x725, 0x715, 0x722,
0x718, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x71f, 0x71d, 0x72a, 0x725, 0x715,
0x722, 0x718, 0x72c, 0x710, 0x20, 0x72b, 0x72a, 0x72b, 0x71d, 0x72c, 0x710, 0x20,
0x725, 0x715, 0x722, 0x718, 0x72c, 0x710, 0x20, 0x72b, 0x72a, 0x72b, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x710, 0x71d, 0x71f, 0x71d, 0x72a, 0x725, 0x715, 0x722, 0x718,
0x72c, 0x710, 0x20, 0x729, 0x71d, 0x72c, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x71d, 0x71f, 0x71d, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x726,
0x713, 0x710, 0x722, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x20, 0x710, 0x726, 0x72a, 0x71d,
0x729, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x715, 0x722, 0x71a,
0x20, 0x710, 0x726, 0x72a, 0x71d, 0x729, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x72c,
0x71d, 0x721, 0x722, 0x20, 0x710, 0x726, 0x72a, 0x71d, 0x729, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x20, 0x710, 0x726, 0x72a,
0x71d, 0x729, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x20,
0x710, 0x726, 0x72a, 0x71d, 0x729, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x729,
0x71d, 0x72c, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x20,
0x710, 0x726, 0x72a, 0x71d, 0x729, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x710, 0x720, 0x710, 0x723, 0x71f, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x720,
0x710, 0x723, 0x71f, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x720, 0x710, 0x723,
0x71f, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x720, 0x721, 0x710,
0x72c, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x720, 0x721, 0x710, 0x72c, 0x71d,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x710, 0x720, 0x721, 0x710, 0x72c, 0x71d, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x710, 0x721, 0x719, 0x722, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x710, 0x721, 0x719, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x721, 0x719, 0x722,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d,
0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712,
0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a,
0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x715,
0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f,
0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a,
0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x71b, 0x718, 0x72a, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712,
0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x71b, 0x718, 0x72a, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a,
0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x71b, 0x718, 0x72a, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713,
0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x72b,
0x71d, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f,
0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x72b, 0x71d, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d,
0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x72b, 0x71d, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a,
0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x722, 0x710, 0x715, 0x71d, 0x72a, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x710, 0x722, 0x710, 0x715, 0x71d, 0x72a, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x710, 0x722, 0x710, 0x715, 0x71d, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x710, 0x726, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x726, 0x71d,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x726, 0x71d, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x710, 0x71f, 0x72c, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x71f, 0x72c, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x71f, 0x72c, 0x710,
0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x71f, 0x72c, 0x718, 0x712,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x710, 0x71f, 0x72c, 0x718, 0x712, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x710, 0x71f, 0x72c, 0x718, 0x712, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x710, 0x72a, 0x712, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x712,
0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x712, 0x71d, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c, 0x71d, 0x722,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c, 0x71d, 0x722,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c, 0x71d, 0x722,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c,
0x71d, 0x722, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x710, 0x72a, 0x713, 0x722, 0x72c, 0x71d, 0x722, 0x710, 0x20,
0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710,
0x72a, 0x713, 0x722, 0x72c, 0x71d, 0x722, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712,
0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x721,
0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x721, 0x722, 0x71d,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x72a, 0x721, 0x722, 0x71d, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x710, 0x71b, 0x720, 0x722, 0x71b, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712,
0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x710, 0x71b, 0x720, 0x722, 0x71b,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d, 0x71f, 0x710, 0x20,
0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x710, 0x71b,
0x720, 0x722, 0x71b, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x721, 0x72a, 0x71d,
0x71f, 0x710, 0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710,
0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710,
0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d,
0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720,
0x71d, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x20, 0x721, 0x728,
0x725, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720,
0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710,
0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723,
0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712,
0x71d, 0x72c, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721,
0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718,
0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d,
0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720,
0x71d, 0x710, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710,
0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x710, 0x718, 0x723, 0x72c, 0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721, 0x725, 0x72a,
0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x723, 0x72c,
0x72a, 0x720, 0x71d, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x719, 0x72a, 0x712, 0x71d, 0x713,
0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x719, 0x72a, 0x712, 0x71d, 0x713,
0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x719, 0x72a, 0x712, 0x71d, 0x713,
0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x719, 0x718, 0x72a,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x710, 0x719, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715,
0x710, 0x719, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x722,
0x713, 0x720, 0x710, 0x715, 0x71d, 0x72b, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x712, 0x722,
0x713, 0x720, 0x710, 0x715, 0x71d, 0x72b, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x712, 0x722,
0x713, 0x720, 0x710, 0x715, 0x71d, 0x72b, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x718, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x718, 0x720, 0x71d, 0x712, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x72a, 0x710, 0x723, 0x71d, 0x720, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x712, 0x72a, 0x710, 0x723, 0x71d, 0x720, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715,
0x712, 0x72a, 0x710, 0x723, 0x71d, 0x720, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x72a, 0x718, 0x722, 0x710, 0x71d, 0x20, 0x715, 0x710, 0x72a,
0x718, 0x723, 0x710, 0x720, 0x721, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x71f,
0x710, 0x726, 0x20, 0x712, 0x71d, 0x72a, 0x715, 0x71d, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x71f, 0x710, 0x726, 0x20, 0x712, 0x71d, 0x72a, 0x715, 0x71d, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x71f, 0x710, 0x726, 0x20, 0x712, 0x71d, 0x72a, 0x715, 0x71d, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x71f, 0x710, 0x71d, 0x723, 0x71d, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x72c, 0x72b, 0x710, 0x721, 0x718, 0x72a, 0x718, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x72c, 0x72b, 0x710, 0x72c, 0x721, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x72c,
0x72b, 0x710, 0x72c, 0x721, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x72c, 0x72b, 0x710, 0x72c,
0x721, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x72b, 0x71d, 0x720, 0x71d,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x72c, 0x72b, 0x71d, 0x720, 0x71d, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x72c, 0x72b, 0x71d, 0x720, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x728, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x728, 0x71d, 0x722, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x715, 0x728, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x71f,
0x718, 0x71d, 0x712, 0x720, 0x723, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x718,
0x71d, 0x712, 0x720, 0x723, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x71d,
0x712, 0x720, 0x723, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719,
0x72a, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x72a, 0x71d, 0x723, 0x72c, 0x721, 0x723,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x716, 0x308, 0x710,
0x20, 0x715, 0x71f, 0x718, 0x71f, 0x718, 0x723, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x71f, 0x718, 0x720, 0x718, 0x721, 0x712, 0x71d, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x71f, 0x718, 0x720, 0x718, 0x721, 0x712, 0x71d, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x71f, 0x718, 0x720, 0x718, 0x721, 0x712, 0x71d, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x71f, 0x718,
0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715,
0x71f, 0x718, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710,
0x20, 0x715, 0x71f, 0x718, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x71f,
0x718, 0x712, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x712, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x71f, 0x718, 0x712, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x715, 0x712, 0x71d, 0x723, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x715,
0x718, 0x721, 0x718, 0x722, 0x72c, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x712, 0x71d,
0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x715, 0x722, 0x71a, 0x20,
0x72c, 0x71d, 0x721, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713,
0x719, 0x72a, 0x72c, 0x710, 0x20, 0x726, 0x728, 0x71a, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x726, 0x728, 0x71a, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x726, 0x728, 0x71a,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x729, 0x718, 0x710, 0x715,
0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718,
0x726, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x710, 0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721, 0x728, 0x725, 0x71d,
0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x726, 0x710,
0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x710, 0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721, 0x715, 0x722, 0x71a,
0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x726,
0x710, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x710, 0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721, 0x715, 0x722, 0x71a,
0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a,
0x718, 0x726, 0x710, 0x20, 0x28, 0x717, 0x721, 0x20, 0x721, 0x715, 0x722, 0x71a,
0x710, 0x29, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718,
0x726, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x726, 0x710, 0x20, 0x721, 0x725, 0x72a,
0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x72a, 0x718,
0x726, 0x710, 0x20, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715,
0x726, 0x720, 0x71f, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x713, 0x719,
0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x726, 0x720, 0x71f, 0x720, 0x722, 0x715,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715,
0x726, 0x720, 0x71f, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x726, 0x71d, 0x713, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x713, 0x71d,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x713, 0x71d, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x713, 0x718, 0x71d, 0x710, 0x722, 0x710, 0x20, 0x726, 0x72a, 0x722,
0x723, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726, 0x72a,
0x722, 0x723, 0x710, 0x20, 0x72c, 0x71d, 0x721, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x718, 0x710, 0x722, 0x72c, 0x72a, 0x72c, 0x71d, 0x71f, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x713, 0x710, 0x720, 0x726, 0x710, 0x713, 0x718, 0x723, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x721, 0x712, 0x71d, 0x72a, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x713, 0x718, 0x72a, 0x713, 0x71d, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x713, 0x718, 0x72a, 0x713, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715,
0x713, 0x718, 0x72a, 0x713, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x713, 0x71d, 0x720, 0x712,
0x72a, 0x72c, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x72a, 0x71d, 0x722,
0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x713, 0x72a, 0x71d, 0x722, 0x720, 0x722,
0x715, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x713, 0x72a, 0x71d, 0x722, 0x720, 0x722, 0x715, 0x20, 0x721, 0x715, 0x722, 0x71a,
0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x713, 0x72a, 0x71d, 0x722, 0x720, 0x722,
0x715, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x713, 0x72a, 0x71d, 0x722, 0x720, 0x722, 0x715, 0x20, 0x715,
0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x713,
0x72a, 0x71d, 0x722, 0x720, 0x722, 0x715, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712,
0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x713, 0x72a, 0x71d, 0x722, 0x720,
0x722, 0x715, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x713, 0x718, 0x710, 0x721, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721,
0x725, 0x720, 0x722, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x717, 0x710,
0x718, 0x710, 0x71d, 0x71d, 0x20, 0x710, 0x720, 0x718, 0x72b, 0x71d, 0x722, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x717, 0x710, 0x718, 0x710, 0x71d, 0x71d, 0x20, 0x710, 0x720,
0x718, 0x72b, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x717, 0x710, 0x718, 0x710,
0x71d, 0x71d, 0x20, 0x710, 0x720, 0x718, 0x72b, 0x71d, 0x722, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x717, 0x718, 0x722, 0x713, 0x20, 0x71f, 0x718, 0x722, 0x713,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x717, 0x718, 0x722, 0x713, 0x20, 0x71f, 0x718, 0x722,
0x713, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x717, 0x718, 0x722, 0x713, 0x20, 0x71f, 0x718,
0x722, 0x713, 0x715, 0x717, 0x718, 0x712, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x717,
0x718, 0x712, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x717, 0x718, 0x712, 0x715, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x717, 0x722, 0x715, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x710, 0x718, 0x729, 0x71d, 0x722, 0x718, 0x723, 0x20, 0x717, 0x722, 0x715,
0x718, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x717, 0x722, 0x715,
0x718, 0x728, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x717, 0x722,
0x715, 0x718, 0x728, 0x71d, 0x722, 0x20, 0x721, 0x728, 0x725, 0x71d, 0x72c, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x717, 0x722, 0x715, 0x718, 0x728, 0x71d,
0x722, 0x20, 0x721, 0x715, 0x722, 0x71a, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x717, 0x722, 0x715, 0x718, 0x728, 0x71d, 0x722, 0x20, 0x721,
0x725, 0x72a, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x710, 0x71d, 0x72a, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a,
0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a, 0x710, 0x722, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a, 0x71f, 0x718, 0x72c, 0x723,
0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a, 0x71f, 0x718, 0x72c, 0x723,
0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x72a, 0x71f, 0x718, 0x72c, 0x723,
0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x723, 0x72a, 0x710,
0x71d, 0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x723, 0x72a, 0x710, 0x71d,
0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x71d, 0x723, 0x72a, 0x710, 0x71d, 0x720,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x71d, 0x726, 0x722, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x71d, 0x726, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717,
0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71d, 0x726, 0x722, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x72c, 0x72a, 0x718, 0x726, 0x712,
0x720, 0x712, 0x723, 0x71f, 0x71d, 0x2d, 0x71f, 0x710, 0x721, 0x71f, 0x72c, 0x723,
0x71f, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x72c, 0x72a, 0x718, 0x726,
0x712, 0x720, 0x712, 0x723, 0x71f, 0x71d, 0x2d, 0x71f, 0x710, 0x721, 0x71f, 0x72c,
0x723, 0x71f, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x72c, 0x72a, 0x718,
0x726, 0x712, 0x720, 0x712, 0x723, 0x71f, 0x71d, 0x2d, 0x71f, 0x710, 0x721, 0x71f,
0x72c, 0x723, 0x71f, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x715,
0x722, 0x71a, 0x20, 0x71f, 0x719, 0x729, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x721, 0x725, 0x72a, 0x712, 0x20, 0x71f, 0x719, 0x729,
0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x71f, 0x718,
0x72a, 0x71d, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x72a, 0x71d,
0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71f, 0x718, 0x72a, 0x71d, 0x71d, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x71f, 0x718, 0x723, 0x72a, 0x710, 0x71d, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x71f, 0x72a, 0x710, 0x723, 0x722, 0x718, 0x71d,
0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x71f, 0x72a, 0x710, 0x723, 0x722,
0x718, 0x71d, 0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71f, 0x72a, 0x710,
0x723, 0x722, 0x718, 0x71d, 0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x729, 0x71d, 0x72a, 0x713, 0x71d, 0x719, 0x723, 0x72c, 0x710, 0x722, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x720, 0x710, 0x722, 0x71f, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715,
0x720, 0x710, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x720, 0x718,
0x72a, 0x715, 0x20, 0x717, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x720, 0x718,
0x72a, 0x715, 0x20, 0x717, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x720, 0x718,
0x72a, 0x715, 0x20, 0x717, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x721, 0x710, 0x71f, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x710, 0x71f,
0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x721, 0x710, 0x71f, 0x710, 0x718, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x710, 0x713, 0x710, 0x715, 0x710, 0x722,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x721, 0x710, 0x713, 0x715, 0x710, 0x722, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x715, 0x721, 0x710, 0x713, 0x715, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x721, 0x720, 0x71d, 0x719, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x721, 0x710, 0x720, 0x715,
0x71d, 0x712, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721,
0x710, 0x72a, 0x71f, 0x710, 0x718, 0x723, 0x710, 0x723, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x721, 0x710,
0x72a, 0x72b, 0x710, 0x720, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x718,
0x72a, 0x71d, 0x71b, 0x71d, 0x718, 0x723, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x718,
0x72a, 0x71d, 0x71b, 0x71d, 0x718, 0x723, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x721, 0x718,
0x72a, 0x71d, 0x71b, 0x71d, 0x718, 0x723, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x721, 0x710, 0x718, 0x723, 0x718, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x721, 0x71f, 0x723, 0x71d, 0x729, 0x718, 0x20, 0x72b, 0x71d, 0x722, 0x71d, 0x72c,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x71f, 0x723, 0x71d, 0x729, 0x718, 0x20,
0x72b, 0x71d, 0x722, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x721, 0x71f,
0x723, 0x71d, 0x729, 0x718, 0x20, 0x72b, 0x71d, 0x722, 0x71d, 0x72c, 0x710, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x720, 0x710, 0x722, 0x712, 0x710,
0x72c, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a,
0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x720, 0x710, 0x722,
0x712, 0x710, 0x72c, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x720,
0x710, 0x722, 0x712, 0x710, 0x72c, 0x718, 0x72a, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x721, 0x718, 0x723, 0x71f, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721,
0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x721, 0x718,
0x723, 0x71f, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x721, 0x718, 0x723, 0x71f, 0x718,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x721, 0x71d, 0x710, 0x722, 0x721, 0x710,
0x72a, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x710, 0x718, 0x72a, 0x718,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x726, 0x710, 0x720, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x20, 0x71f, 0x720, 0x71d,
0x715, 0x718, 0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718,
0x20, 0x71f, 0x720, 0x71d, 0x715, 0x718, 0x722, 0x71d, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x722, 0x71d, 0x718, 0x20, 0x71f, 0x720, 0x71d, 0x715, 0x718, 0x722, 0x71d,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x20, 0x719,
0x71d, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x20,
0x719, 0x71d, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712,
0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718,
0x20, 0x719, 0x71d, 0x720, 0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x722, 0x71d, 0x718, 0x726, 0x710, 0x718, 0x722, 0x720, 0x722, 0x715, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x722, 0x71d, 0x718, 0x726, 0x710, 0x718, 0x722, 0x720, 0x722, 0x715,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x726, 0x710, 0x718, 0x722, 0x720,
0x722, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x722, 0x71d, 0x718, 0x71d,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20,
0x715, 0x722, 0x718, 0x72a, 0x726, 0x720, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x713,
0x719, 0x72a, 0x72c, 0x710, 0x20, 0x715, 0x722, 0x718, 0x72a, 0x726, 0x720, 0x71f,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718,
0x721, 0x710, 0x20, 0x715, 0x713, 0x719, 0x72a, 0x72c, 0x710, 0x20, 0x715, 0x722,
0x718, 0x72a, 0x726, 0x720, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726,
0x72a, 0x722, 0x722, 0x715, 0x718, 0x20, 0x715, 0x722, 0x718, 0x72a, 0x718, 0x722,
0x717, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x72a, 0x722, 0x722, 0x715, 0x718,
0x20, 0x715, 0x722, 0x718, 0x72a, 0x718, 0x722, 0x717, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20,
0x715, 0x726, 0x72a, 0x722, 0x722, 0x715, 0x718, 0x20, 0x715, 0x722, 0x718, 0x72a,
0x718, 0x722, 0x717, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719,
0x71d, 0x716, 0x308, 0x710, 0x20, 0x715, 0x721, 0x72a, 0x71d, 0x710, 0x722, 0x710,
0x20, 0x713, 0x72a, 0x712, 0x71d, 0x71d, 0x72c, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x722, 0x718, 0x712, 0x718, 0x723, 0x71d, 0x712, 0x72a, 0x723, 0x71f,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x722, 0x718, 0x712, 0x718, 0x723, 0x71d, 0x712, 0x72a,
0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x722, 0x718, 0x712, 0x718, 0x723, 0x71d,
0x712, 0x72a, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x718, 0x721,
0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x718, 0x721, 0x723, 0x71f, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x715, 0x718, 0x721, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x726, 0x710, 0x71f, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726,
0x710, 0x71f, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x726, 0x710,
0x71f, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726,
0x720, 0x710, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726, 0x710, 0x726,
0x718, 0x710, 0x20, 0x713, 0x71d, 0x722, 0x71d, 0x710, 0x20, 0x71a, 0x715, 0x72c,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726, 0x72a, 0x713, 0x718, 0x710,
0x71d, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x72a, 0x713, 0x718, 0x710, 0x71d, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x726, 0x72a, 0x713, 0x718, 0x710, 0x71d, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x726, 0x71d, 0x72a, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20,
0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726,
0x71d, 0x72a, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x72a, 0x718, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x720, 0x71d, 0x726, 0x71d, 0x722,
0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c,
0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x720, 0x71d, 0x726, 0x71d,
0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x720, 0x71d, 0x726,
0x71d, 0x722, 0x71d, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713, 0x719,
0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x726, 0x71d, 0x722, 0x71d, 0x71f, 0x723,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723, 0x710, 0x722, 0x72c, 0x20, 0x726,
0x71d, 0x725, 0x72a, 0x20, 0x718, 0x721, 0x729, 0x718, 0x720, 0x718, 0x722, 0x725,
0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c,
0x710, 0x20, 0x715, 0x723, 0x710, 0x722, 0x72c, 0x20, 0x726, 0x71d, 0x725, 0x72a,
0x20, 0x718, 0x721, 0x729, 0x718, 0x720, 0x718, 0x722, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715,
0x723, 0x710, 0x722, 0x72c, 0x20, 0x726, 0x71d, 0x725, 0x72a, 0x20, 0x718, 0x721,
0x729, 0x718, 0x720, 0x718, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x726,
0x71d, 0x72c, 0x71f, 0x710, 0x71d, 0x72a, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x726, 0x718, 0x722, 0x710, 0x726, 0x71d, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x726, 0x71d, 0x718, 0x722, 0x713, 0x71d, 0x722, 0x713, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x71f, 0x71d, 0x719, 0x71d, 0x720, 0x718, 0x72a, 0x715, 0x710,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x71f, 0x71d, 0x719, 0x71d, 0x720, 0x718, 0x72a, 0x715,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x71f, 0x71d, 0x719, 0x71d, 0x720, 0x718, 0x72a,
0x715, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72a, 0x71d, 0x718, 0x722,
0x71d, 0x718, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72a, 0x718, 0x72c,
0x71d, 0x72a, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723, 0x710, 0x71a,
0x710, 0x720, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x723, 0x710, 0x71a, 0x710,
0x720, 0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x723, 0x710, 0x71a, 0x710, 0x720,
0x71d, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723, 0x721, 0x710, 0x72a,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x723, 0x721, 0x710, 0x72a, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x715, 0x723, 0x721, 0x710, 0x72a, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x723, 0x721, 0x718, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x723, 0x721, 0x718,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d,
0x718, 0x721, 0x710, 0x20, 0x715, 0x723, 0x721, 0x718, 0x710, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x723, 0x710, 0x71d, 0x72b, 0x71d, 0x720, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x723, 0x71d, 0x722, 0x713, 0x710, 0x726, 0x718, 0x72a, 0x725, 0x715, 0x722,
0x710, 0x20, 0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x72b,
0x720, 0x71d, 0x721, 0x718, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x713,
0x71d, 0x718, 0x72a, 0x713, 0x71d, 0x710, 0x20, 0x72c, 0x71d, 0x721, 0x722, 0x71d,
0x72c, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723, 0x718, 0x72a, 0x71d,
0x722, 0x710, 0x721, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x723, 0x71d, 0x718,
0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x717, 0x71d, 0x72c, 0x71d,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x710, 0x71d, 0x726, 0x710, 0x71d,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x72c, 0x710, 0x71d, 0x726, 0x710, 0x71d, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x715, 0x72c, 0x710, 0x71d, 0x726, 0x710, 0x71d, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x72c, 0x710, 0x713, 0x71d, 0x71f, 0x723, 0x72c, 0x710, 0x722, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x718, 0x71f, 0x71d, 0x720, 0x710, 0x718,
0x725, 0x715, 0x722, 0x710, 0x20, 0x72c, 0x718, 0x722, 0x713, 0x710, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x72c, 0x718, 0x722, 0x713, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x72c, 0x718, 0x722,
0x713, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x72b, 0x718, 0x71f,
0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x72c, 0x718, 0x72a, 0x71f, 0x721, 0x722,
0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x72c, 0x718, 0x72a, 0x71f,
0x721, 0x722, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x72c, 0x718,
0x72a, 0x71f, 0x721, 0x722, 0x723, 0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x72c, 0x718, 0x712, 0x710, 0x720, 0x718, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x713, 0x718, 0x710, 0x71d, 0x725, 0x715,
0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710,
0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x713, 0x718, 0x710, 0x71d, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710,
0x20, 0x715, 0x710, 0x718, 0x72a, 0x718, 0x713, 0x718, 0x710, 0x71d, 0x725, 0x715,
0x722, 0x710, 0x20, 0x715, 0x710, 0x718, 0x719, 0x712, 0x71f, 0x723, 0x72c, 0x710,
0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722,
0x71d, 0x72c, 0x710, 0x20, 0x715, 0x710, 0x718, 0x719, 0x712, 0x71f, 0x723, 0x72c,
0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20,
0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x710, 0x718, 0x719, 0x712, 0x71f, 0x723,
0x72c, 0x710, 0x722, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x710, 0x722,
0x718, 0x710, 0x71b, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718,
0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x712, 0x710, 0x722, 0x718,
0x710, 0x71b, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x712, 0x710, 0x722, 0x718, 0x710,
0x71b, 0x718, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x722, 0x719, 0x718,
0x71d, 0x720, 0x710, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x720, 0x710,
0x715, 0x71d, 0x712, 0x718, 0x723, 0x72c, 0x718, 0x71f, 0x725, 0x715, 0x722, 0x710,
0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715,
0x712, 0x720, 0x710, 0x715, 0x71d, 0x712, 0x718, 0x723, 0x72c, 0x718, 0x71f, 0x725,
0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721,
0x710, 0x20, 0x715, 0x712, 0x720, 0x710, 0x715, 0x71d, 0x712, 0x718, 0x723, 0x72c,
0x718, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x718, 0x720, 0x713,
0x718, 0x713, 0x72a, 0x710, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b,
0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20, 0x715, 0x712, 0x718, 0x720,
0x713, 0x718, 0x713, 0x72a, 0x710, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x712, 0x718,
0x720, 0x713, 0x718, 0x713, 0x72a, 0x710, 0x715, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x712, 0x718, 0x723, 0x72c, 0x718, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x713, 0x719, 0x71d, 0x72a, 0x308, 0x710, 0x20, 0x715, 0x718, 0x710, 0x71d,
0x71f, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x718, 0x71d, 0x720, 0x71d, 0x723,
0x20, 0x718, 0x726, 0x718, 0x72c, 0x718, 0x722, 0x710, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x71d, 0x710, 0x71f, 0x718, 0x72c, 0x723, 0x71f, 0x725, 0x715, 0x722,
0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d, 0x72c, 0x710, 0x20,
0x715, 0x71d, 0x710, 0x71f, 0x718, 0x72c, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710,
0x20, 0x715, 0x712, 0x717, 0x72a, 0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715,
0x71d, 0x710, 0x71f, 0x718, 0x72c, 0x723, 0x71f, 0x725, 0x715, 0x722, 0x710, 0x20,
0x715, 0x71d, 0x71f, 0x710, 0x72c, 0x71d, 0x72a, 0x722, 0x712, 0x71d, 0x72a, 0x713,
0x725, 0x715, 0x722, 0x710, 0x20, 0x721, 0x72b, 0x718, 0x71a, 0x72c, 0x722, 0x71d,
0x72c, 0x710, 0x20, 0x715, 0x71d, 0x71f, 0x710, 0x72c, 0x71d, 0x72a, 0x722, 0x712,
0x71d, 0x72a, 0x713, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715, 0x712, 0x717, 0x72a,
0x20, 0x71d, 0x718, 0x721, 0x710, 0x20, 0x715, 0x71d, 0x71f, 0x710, 0x72c, 0x71d,
0x72a, 0x722, 0x712, 0x71d, 0x72a, 0x713, 0x725, 0x715, 0x722, 0x710, 0x20, 0x715,
0x71d, 0x718, 0x729, 0x718, 0x722, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x43c,
0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x43c,
0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x43c, 0x430, 0x440, 0x43a,
0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x448, 0x430, 0x440,
0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x4e3,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430,
0x438, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x43a, 0x4ef, 0x4b3, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x43a, 0x4ef,
0x4b3, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x43a, 0x4ef, 0x4b3, 0x4e3, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x423, 0x49b, 0x451, 0x43d, 0x443, 0x441, 0x438, 0x20, 0x41e, 0x440,
0x43e, 0x43c, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x423, 0x49b, 0x451, 0x43d, 0x443,
0x441, 0x438, 0x20, 0x41e, 0x440, 0x43e, 0x43c, 0x412, 0x430, 0x49b, 0x442, 0x438,
0x20, 0x440, 0x4ef, 0x437, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x423, 0x49b, 0x451,
0x43d, 0x443, 0x441, 0x438, 0x20, 0x41e, 0x440, 0x43e, 0x43c, 0x412, 0x430, 0x49b,
0x442, 0x438, 0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x4e3,
0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x438, 0x438, 0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438,
0x43a, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x440, 0x4ef, 0x437, 0x43e,
0x43d, 0x430, 0x438, 0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x430, 0x432, 0x440, 0x443, 0x43f,
0x43e, 0x438, 0x438, 0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x430, 0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438,
0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x430, 0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x43c, 0x430,
0x440, 0x43a, 0x430, 0x437, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x430,
0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x448, 0x430, 0x440, 0x49b,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x438, 0x438, 0x20, 0x430, 0x432, 0x440, 0x443, 0x43f, 0x43e,
0x438, 0x438, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442,
0x438, 0x20, 0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438,
0x20, 0x430, 0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x448, 0x430,
0x440, 0x49b, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x430, 0x432, 0x440,
0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x493, 0x430, 0x440, 0x431, 0x4e3, 0x412,
0x430, 0x49b, 0x442, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x438, 0x438, 0x20, 0x430, 0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438,
0x20, 0x493, 0x430, 0x440, 0x431, 0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20,
0x442, 0x43e, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x430, 0x438, 0x20, 0x430,
0x432, 0x440, 0x443, 0x43f, 0x43e, 0x438, 0x438, 0x20, 0x493, 0x430, 0x440, 0x431,
0x4e3, 0x412, 0x430, 0x49b, 0x442, 0x438, 0x20, 0x43c, 0x438, 0x451, 0x43d, 0x430,
0x438, 0x20, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0xb85, 0xb95, 0xbcd,
0xbb0, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb95, 0xbcd, 0xbb0,
0xbc7, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb95,
0xbcd, 0xbb0, 0xbc7, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb86, 0xb83, 0xbaa, 0xbcd, 0xb95, 0xbbe, 0xba9, 0xbbf, 0xbb8, 0xbcd,
0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4,
0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf,
0xb95, 0xbcd, 0xb95, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4,
0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf,
0xb95, 0xbcd, 0xb95, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc6, 0xba9,
0xbcd, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb95, 0xbcd, 0xb95,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb86, 0xbaa, 0xbcd,
0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb95, 0xbcd, 0xb95, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa,
0xbbf, 0xbb0, 0xbbf, 0xb95, 0xbcd, 0xb95, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd,
0xb95, 0xbc1, 0x20, 0xb86, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xb95, 0xbcd,
0xb95, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xbb2, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb85, 0xbb2, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb2,
0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbbe, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb2, 0xbcd, 0xbae, 0xbbe, 0xb9f,
0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb2, 0xbcd, 0xbae, 0xbbe,
0xb9f, 0xbbf, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85,
0xbb2, 0xbcd, 0xbae, 0xbbe, 0xb9f, 0xbbf, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbae, 0xbc7, 0xb9a, 0xbbe, 0xba9, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbae, 0xbc7, 0xb9a, 0xbbe, 0xba9,
0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb85, 0xbae, 0xbc7, 0xb9a, 0xbbe, 0xba9, 0xbcd, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd,
0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd,
0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20,
0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4,
0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95,
0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbb5, 0xbc1, 0xba9, 0xbcd, 0xb9f,
0xbc8, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbb5, 0xbc1,
0xba9, 0xbcd, 0xb9f, 0xbc8, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbb5, 0xbc1, 0xba9,
0xbcd, 0xb9f, 0xbc8, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xb95,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf,
0xb95, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xb95, 0xbcd, 0x20,
0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xba9, 0xb9f, 0xbc0, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xba9, 0xbbe, 0xb9f, 0xbc8, 0xbaf, 0xbb0, 0xbcd, 0x20, 0xba4, 0xbb0, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xba9, 0xbbe, 0xb9f, 0xbc8, 0xbaf, 0xbb0,
0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb8f, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb8f,
0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb8f, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20,
0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xb9f, 0xbcd, 0xb9f, 0xbcc, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85,
0xb9f, 0xbcd, 0xb9f, 0xbcc, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb85, 0xb9f, 0xbcd, 0xb9f, 0xbcc, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd, 0xb9f, 0xbcb, 0xbaa, 0xbc7,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd, 0xb9f, 0xbcb, 0xbaa,
0xbc7, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f,
0xbcd, 0xb9f, 0xbcb, 0xbaa, 0xbc7, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb0, 0xbc7, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb0, 0xbc7, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xbb0, 0xbc7, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3,
0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6,
0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbe, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbe,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb85, 0xbb0,
0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9, 0xbbe, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf,
0xbaf, 0x20, 0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9,
0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf,
0xbaf, 0x20, 0xb85, 0xbb0, 0xbcd, 0xb9c, 0xbc6, 0xba9, 0xbcd, 0xb9f, 0xbbf, 0xba9,
0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb86, 0xbb0, 0xbcd, 0xbae, 0xbc7, 0xba9, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb86, 0xbb0, 0xbcd, 0xbae, 0xbc7, 0xba9, 0xbbf, 0xbaf, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb86, 0xbb0, 0xbcd, 0xbae, 0xbc7, 0xba9, 0xbbf, 0xbaf, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9f, 0xbcd, 0xbb2, 0xbbe,
0xba3, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb85, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbcd, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb85, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbbf, 0xb95, 0xbcd,
0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb86, 0xbb8, 0xbcd, 0xba4,
0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd,
0x20, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd,
0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xba4,
0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7,
0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0x20, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7,
0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0x20, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9,
0xbcd, 0x20, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbae, 0xbc7, 0xbb1,
0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca,
0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95,
0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb86, 0xbb8, 0xbcd, 0xba4,
0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd,
0x20, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf,
0xba9, 0xbcd, 0x20, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4,
0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf,
0xbaf, 0x20, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf, 0xbb0,
0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95,
0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb86, 0xbb8, 0xbcd, 0xba4, 0xbbf,
0xbb0, 0xbc7, 0xbb2, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xbae, 0xbc7, 0xbb1, 0xbcd,
0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3,
0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a, 0xbb0, 0xbcd, 0xbaa,
0xbc8, 0xb9c, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85,
0xb9a, 0xbb0, 0xbcd, 0xbaa, 0xbc8, 0xb9c, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85,
0xb9a, 0xbb0, 0xbcd, 0xbaa, 0xbc8, 0xb9c, 0xbbe, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb,
0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a, 0xbcb, 0xbb0,
0xbb8, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a, 0xbcb, 0xbb0,
0xbb8, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb85, 0xb9a, 0xbcb, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xb99,
0xbcd, 0xb95, 0xba4, 0xbc7, 0xb9a, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5,
0xb99, 0xbcd, 0xb95, 0xba4, 0xbc7, 0xb9a, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xb99, 0xbcd, 0xb95,
0xba4, 0xbc7, 0xb9a, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbaa, 0xbc2, 0xb9f, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbaa, 0xbca, 0xbb2, 0xbbf, 0xbb5, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbc7, 0xb9a, 0xbbf, 0xbb2, 0xbbf,
0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbc7,
0xb9a, 0xbbf, 0xbb2, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbc7,
0xb9a, 0xbbf, 0xbb2, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc1, 0xbb0, 0xbc1, 0xba9, 0xbc7, 0x20,
0xb9f, 0xbb0, 0xbc1, 0xbb8, 0xbcd, 0xbb8, 0xbb2, 0xbbe, 0xbae, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc7, 0xbaa, 0xbcd, 0x20, 0xbb5, 0xbc6, 0xbb0,
0xbcd, 0xb9f, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc7, 0xbaa,
0xbcd, 0x20, 0xbb5, 0xbc6, 0xbb0, 0xbcd, 0xb9f, 0xbc7, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc7,
0xbaa, 0xbcd, 0x20, 0xbb5, 0xbc6, 0xbb0, 0xbcd, 0xb9f, 0xbc7, 0x20, 0xb95, 0xbcb,
0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbe, 0xbae, 0xbcb,
0xbb0, 0xbcb, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xbae, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xbae, 0xbcd,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb9a, 0xba4, 0xbcd, 0xba4, 0xbbe, 0xbae, 0xbcd, 0x20, 0xbaa, 0xb95,
0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbf,
0xbb2, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbf, 0xbb2, 0xbbf,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb9a, 0xbbf, 0xbb2, 0xbbf, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc0, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb9a, 0xbc0, 0xba9, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc0, 0xba9, 0x20, 0xbaa,
0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a,
0xbcb, 0xbaf, 0xbcd, 0xbaa, 0xbbe, 0xbb2, 0xbcd, 0xb9a, 0xba9, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbcb, 0xbaf, 0xbcd, 0xbaa, 0xbbe, 0xbb2, 0xbcd,
0xb9a, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbcb, 0xbaf, 0xbcd, 0xbaa, 0xbbe, 0xbb2,
0xbcd, 0xb9a, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb1, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbc1, 0xbae,
0xbb8, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb95, 0xbcb, 0xb95, 0xbcb, 0xbb8, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1,
0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca, 0xbb2,
0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb95, 0xbca, 0xbb2, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95,
0xbca, 0xbb2, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc1, 0xb95, 0xbcd, 0x20,
0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb95, 0xbc1, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3,
0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc1, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1,
0xb95, 0xbb3, 0xbcd, 0x20, 0xb85, 0xbb0, 0xbc8, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbc2, 0xbaa, 0xbbe,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbc2, 0xbaa, 0xbbe,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb95, 0xbbf, 0xbaf, 0xbc2, 0xbaa, 0xbbe, 0x20, 0xbaa, 0xb95, 0xbb2,
0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9f, 0xbc7, 0xbb5,
0xbbf, 0xbb8, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9f, 0xbc1, 0xbae,
0xbcb, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0x2d, 0xb9f, 0xbbf, 0x20, 0xb89, 0xbb0, 0xbcd,
0xbb5, 0xbbf, 0xbb2, 0xbcd, 0xbb2, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xba4, 0xbbf, 0xbae, 0xbcb,
0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb88, 0xbb8, 0xbcd, 0xb9f,
0xbb0, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb88, 0xbb8, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb88, 0xbb8, 0xbcd, 0xb9f, 0xbb0, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5,
0xbc1, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb88, 0xb95, 0xbcd, 0xbb5, 0xb9f, 0xbbe, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb,
0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa,
0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90,
0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95,
0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa,
0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95,
0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa,
0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95,
0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa,
0xbbf, 0xbaf, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xba4, 0xbc2, 0xbb0, 0x2d, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1,
0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf,
0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4,
0xbbf, 0xbaf, 0x20, 0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20,
0xb90, 0xbb0, 0xbcb, 0xbaa, 0xbcd, 0xbaa, 0xbbf, 0xbaf, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbe, 0xb95, 0xbcd,
0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95,
0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbe, 0xb95,
0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1,
0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbe, 0xb95, 0xbcd, 0xbb2, 0xbbe,
0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83,
0xbaa, 0xbbf, 0xb9c, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa,
0xbbf, 0xb9c, 0xbbf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbf, 0xb9c, 0xbbf, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbbf,
0xbb0, 0xb9e, 0xbcd, 0xb9a, 0xbc1, 0x20, 0xb95, 0xbaf, 0xbbe, 0xba9, 0xbbe, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb0, 0xbc6, 0xb9e, 0xbcd, 0xb9a,
0xbc1, 0x20, 0xba4, 0xbc6, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0x20, 0x26, 0x20, 0xb85, 0xba3, 0xbcd, 0xb9f, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbbf,
0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbb2, 0xbaa, 0xb95,
0xbcb, 0xbb8, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc7, 0xbae,
0xbcd, 0xbaa, 0xbbf, 0xbaf, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb9c, 0xbbe, 0xbb0, 0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb9c, 0xbbe, 0xbb0, 0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb9c, 0xbbe, 0xbb0, 0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb2, 0xbcd, 0xbaa,
0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd,
0xbb5, 0xbbf, 0xb9a, 0xbcd, 0x20, 0xb9a, 0xbb0, 0xbbe, 0xb9a, 0xbb0, 0xbbf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1,
0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4,
0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd,
0xb95, 0xbc1, 0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8,
0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1,
0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4,
0xbc1, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9,
0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0,
0xba9, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7,
0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95, 0xbbf, 0xbb0, 0xbc0, 0xba9, 0xbcd, 0xbb2,
0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbae, 0xbcd, 0x20, 0xba4, 0xbb0, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbb3, 0xbc8, 0xb95, 0xbc1, 0xb9f, 0xbbe, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbb9, 0xbb5, 0xbbe, 0xbaf, 0xbcd, 0x2d, 0xb85, 0xbb2, 0xbc7, 0xbb7, 0xbbf,
0xbaf, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbb5, 0xbbe,
0xbaf, 0xbcd, 0x2d, 0xb85, 0xbb2, 0xbc7, 0xbb7, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbb9, 0xbb5, 0xbbe, 0xbaf, 0xbcd, 0x2d, 0xb85, 0xbb2, 0xbc7, 0xbb7, 0xbbf,
0xbaf, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb99, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe,
0xb99, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbbe, 0xb99, 0xbcd, 0xb95, 0xbbe, 0xb99, 0xbcd,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9,
0xbcb, 0xbb5, 0xbcd, 0xba4, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9,
0xbcb, 0xbb5, 0xbcd, 0xba4, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb9, 0xbcb, 0xbb5, 0xbcd, 0xba4,
0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb87, 0xba8, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xba8, 0xbcd, 0xba4,
0xbbf, 0xbaf, 0xbaa, 0xbcd, 0x20, 0xbaa, 0xbc6, 0xbb0, 0xbc1, 0xb99, 0xbcd, 0xb95,
0xb9f, 0xbb2, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xba8, 0xbcd,
0xba4, 0xbcb, 0xb9a, 0xbc0, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb87, 0xba8, 0xbcd, 0xba4, 0xbcb, 0xba9,
0xbc7, 0xb9a, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbbf,
0xbb4, 0xb95, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf, 0x20, 0xb87, 0xba8,
0xbcd, 0xba4, 0xbcb, 0xba9, 0xbc7, 0xb9a, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xbc7, 0xbb1, 0xbcd, 0xb95, 0xba4, 0xbcd, 0xba4, 0xbbf, 0xbaf,
0x20, 0xb87, 0xba8, 0xbcd, 0xba4, 0xbcb, 0xba9, 0xbc7, 0xb9a, 0xbbf, 0xbaf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb88, 0xbb0, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb88, 0xbb0, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb88,
0xbb0, 0xbbe, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb0, 0xbcd, 0xb95, 0xbc1, 0xb9f, 0xbcd,
0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb0,
0xbcd, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87,
0xbb0, 0xbcd, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb8, 0xbcd,
0xbb0, 0xbc7, 0xbb2, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb8,
0xbcd, 0xbb0, 0xbc7, 0xbb2, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb87, 0xbb8, 0xbcd, 0xbb0, 0xbc7,
0xbb2, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb9c, 0xbaa, 0xbcd, 0xbaa, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9c, 0xbaa, 0xbcd, 0xbaa, 0xbbe, 0xba9, 0xbcd, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb9c, 0xbaa, 0xbcd, 0xbaa, 0xbbe, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2,
0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xb9f,
0xbcd, 0xbb0, 0xbcb, 0xbaa, 0xbb5, 0xbcd, 0xbb2, 0xbcb, 0xbb5, 0xbcd, 0xbb8, 0xbcd,
0xb95, 0xbcd, 0x20, 0xb95, 0xbae, 0xbcd, 0xb9a, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95,
0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xb9f, 0xbcd, 0xbb0,
0xbcb, 0xbaa, 0xbb5, 0xbcd, 0xbb2, 0xbcb, 0xbb5, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd,
0x20, 0xb95, 0xbae, 0xbcd, 0xb9a, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbbf, 0x20,
0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xb9f, 0xbcd,
0xbb0, 0xbcb, 0xbaa, 0xbb5, 0xbcd, 0xbb2, 0xbcb, 0xbb5, 0xbcd, 0xbb8, 0xbcd, 0xb95,
0xbcd, 0x20, 0xb95, 0xbae, 0xbcd, 0xb9a, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbbf,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95,
0xbbf, 0xbb4, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95, 0xb9c, 0xb95, 0xbb8, 0xbcd,
0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc7,
0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20, 0xb95, 0xb9c, 0xb95, 0xbb8, 0xbcd, 0xba4, 0xbbe,
0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca, 0xbb0, 0xbbf,
0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbca, 0xbb0, 0xbbf, 0xbaf,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb95, 0xbca, 0xbb0, 0xbbf, 0xbaf, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca,
0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbcb, 0xbb8, 0xbcd,
0xbb0, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbcd, 0xbb0, 0xbb8,
0xbcd, 0xba9, 0xbcb, 0xbaf, 0xbbe, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbcd, 0xbb0, 0xbb8, 0xbcd, 0xba9, 0xbcb,
0xbaf, 0xbbe, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbcd,
0xbb0, 0xbb8, 0xbcd, 0xba9, 0xbcb, 0xbaf, 0xbbe, 0xbb0, 0xbcd, 0xbb8, 0xbcd, 0xb95,
0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb95, 0xbbf, 0xbb0, 0xbcd, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb2, 0xb99, 0xbcd, 0xb95, 0xbbe, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb2, 0xbc8, 0xba9, 0xbcd, 0x20, 0xba4, 0xbc0,
0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb2,
0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb9, 0xbcb, 0xbb5, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb2, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbb9,
0xbcb, 0xbb5, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb2, 0xbbe, 0xbb0, 0xbcd, 0xb9f, 0xbcd, 0x20,
0xbb9, 0xbcb, 0xbb5, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95, 0xbcd, 0xb95, 0xbbe, 0xbb5, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95, 0xbcd, 0xb95, 0xbbe, 0xbb5,
0xbcd, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95,
0xbcd, 0xb95, 0xbbe, 0xbb5, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95, 0xba4, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95, 0xba4, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xb95,
0xba4, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xbb2, 0xbc7, 0xbb7, 0xbbf, 0xbaf, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb2, 0xba4, 0xbcd, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95,
0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb0, 0xbcd,
0xb95, 0xbbf, 0xbaf, 0xbc2, 0xb9a, 0xbbe, 0xbb8, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb0, 0xbcd, 0xbb7, 0xbb2, 0xbcd, 0x20, 0xba4, 0xbc0,
0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae,
0xbca, 0xbb0, 0xbbf, 0xbb7, 0xbbf, 0xbaf, 0xbb8, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xbca, 0xbb0, 0xbbf, 0xbb7, 0xbbf, 0xbaf, 0xbb8, 0xbcd, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbae, 0xbca, 0xbb0, 0xbbf, 0xbb7, 0xbbf, 0xbaf, 0xbb8, 0xbcd, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xb9a,
0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc6, 0xb95, 0xbcd,
0xbb8, 0xbbf, 0xb95, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xb95,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbc6, 0xb95, 0xbcd, 0xbb8,
0xbbf, 0xb95, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xb95, 0xbcd,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbae, 0xbc6, 0xb95, 0xbcd, 0xbb8, 0xbbf, 0xb95, 0xba9, 0xbcd, 0x20,
0xbaa, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xb95, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca,
0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb2, 0xba9, 0xbcd,
0x20, 0xbaa, 0xbbe, 0xb9f, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb89, 0xbb2, 0xba9, 0xbcd, 0x20, 0xbaa, 0xbbe, 0xb9f, 0xbb0, 0xbcd, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb89, 0xbb2, 0xba9, 0xbcd, 0x20, 0xbaa, 0xbbe, 0xb9f, 0xbb0, 0xbcd, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb8,
0xbcd, 0xb95, 0xbcb, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb8,
0xbcd, 0xb95, 0xbcb, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbe, 0xbb8, 0xbcd, 0xb95, 0xbcb, 0x20,
0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbae, 0xbbf,
0xbaf, 0xbbe, 0xba9, 0xbcd, 0xbae, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xba8, 0xbb5, 0xbcd, 0xbb0, 0xbc2, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xba8, 0xbc7, 0xbaa, 0xbbe, 0xbb3, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8,
0xbbf, 0xbaf, 0xbc2, 0x20, 0xb95, 0xbb2, 0xbbf, 0xb9f, 0xbcb, 0xba9, 0xbbf, 0xbaf,
0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0x20,
0xb95, 0xbb2, 0xbbf, 0xb9f, 0xbcb, 0xba9, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8,
0xbbf, 0xbaf, 0xbc2, 0x20, 0xb95, 0xbb2, 0xbbf, 0xb9f, 0xbcb, 0xba9, 0xbbf, 0xbaf,
0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xba8, 0xbbf, 0xbaf, 0xbc2, 0xb9a, 0xbbf, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0xb9a, 0xbbf,
0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2,
0xb9a, 0xbbf, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20, 0xbaa, 0xb95, 0xbb2,
0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf,
0xbc2, 0xb83, 0xbaa, 0xbb5, 0xbc1, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba8,
0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf,
0xbc2, 0xb83, 0xbaa, 0xbb5, 0xbc1, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba8,
0xbcd, 0xba4, 0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0xb83, 0xbaa, 0xbb5,
0xbc1, 0xba3, 0xbcd, 0xb9f, 0xbcd, 0xbb2, 0xbbe, 0xba8, 0xbcd, 0xba4, 0xbc1, 0x20,
0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xba8, 0xbbf, 0xbaf, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbe,
0xbb0, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbe, 0xbb0, 0xbcd, 0xb83, 0xbaa,
0xbcb, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8, 0xbbe,
0xbb0, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xb95, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1,
0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xba9, 0xbbe, 0xba3, 0xbcd, 0xb9f, 0xbcb, 0x20,
0xb9f, 0xbbf, 0x20, 0xba8, 0xbcb, 0xbb0, 0xba9, 0xbcd, 0xbb9, 0xbbe, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xba9, 0xbbe, 0xba9, 0xbcd,
0xb9f, 0xbcb, 0x20, 0xb9f, 0xbbf, 0x20, 0xba8, 0xbcb, 0xbb0, 0xbcb, 0xba9, 0xbcd,
0xbb9, 0xbbe, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbcd, 0xba9, 0xbbe, 0xba9, 0xbcd,
0xb9f, 0xbcb, 0x20, 0xb9f, 0xbbf, 0x20, 0xba8, 0xbcb, 0xbb0, 0xbcb, 0xba9, 0xbcd,
0xbb9, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbb5, 0xb9f, 0xb95, 0xbcd, 0xb95, 0xbc1, 0x20, 0xbae, 0xbb0, 0xbbf, 0xba9,
0xbbe, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xba8, 0xbcb, 0xbb5, 0xbcb, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xbb0,
0xbbf, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba8,
0xbcb, 0xbb5, 0xbcb, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xbb0, 0xbbf, 0xbb8, 0xbcd, 0xb95,
0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xba8, 0xbcb, 0xbb5, 0xbcb, 0xb9a, 0xbbf, 0xbaa, 0xbbf, 0xbb0,
0xbbf, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb93, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb93, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd,
0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb93, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbcb,
0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbe, 0xb95, 0xbbf,
0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbaa, 0xbbe, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbaa, 0xbbe, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbe, 0xbb2,
0xbb5, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbaa, 0xbc1, 0xbb5,
0xbbe, 0x20, 0xba8, 0xbbf, 0xbaf, 0xbc2, 0x20, 0xb95, 0xbbf, 0xba9, 0xbbf, 0xbaf,
0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbb0, 0xbbe, 0xb95, 0xbc1,
0xbb5, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbb0, 0xbbe, 0xb95,
0xbc1, 0xbb5, 0xbc7, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbb0, 0xbbe, 0xb95, 0xbc1, 0xbb5, 0xbc7,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa,
0xbc6, 0xbb0, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb0,
0xbc1, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbc6, 0xbb0, 0xbc1, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xbb2, 0xbbf, 0xbaa, 0xbcd,
0xbaa, 0xbc8, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf,
0xbb2, 0xbbf, 0xbaa, 0xbcd, 0xbaa, 0xbc8, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2,
0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf,
0xbb2, 0xbbf, 0xbaa, 0xbcd, 0xbaa, 0xbc8, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb83, 0xbaa, 0xbcb, 0xba9, 0xbbf,
0xb95, 0xbcd, 0xbb8, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd,
0xb9f, 0xbcd, 0x20, 0xbaa, 0xbbf, 0xbaf, 0xbb0, 0xbbf, 0x20, 0x26, 0x20, 0xbae,
0xbbf, 0xb95, 0xbcd, 0xb95, 0xbc1, 0xbaf, 0xbbf, 0xbb2, 0xbbe, 0xba9, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f,
0xbcd, 0x20, 0xbaa, 0xbbf, 0xbaf, 0xbb0, 0xbbf, 0x20, 0x26, 0x20, 0xbae, 0xbbf,
0xb95, 0xbcd, 0xb95, 0xbc1, 0xbaf, 0xbbf, 0xbb2, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb9a, 0xbc6, 0xbaf, 0xbbf, 0xba9, 0xbcd, 0xb9f, 0xbcd, 0x20, 0xbaa, 0xbbf, 0xbaf,
0xbb0, 0xbbf, 0x20, 0x26, 0x20, 0xbae, 0xbbf, 0xb95, 0xbcd, 0xb95, 0xbc1, 0xbaf,
0xbbf, 0xbb2, 0xbbe, 0xba9, 0xbcd, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaa, 0xbbf, 0xb9f, 0xbcd, 0xb95, 0xbc6,
0xbaf, 0xbcd, 0xbb0, 0xbcd, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbaa, 0xbcb, 0xba9, 0xbbe, 0xbaa, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbaa, 0xbbf, 0xbaf, 0xbbe, 0xb99, 0xbcd, 0xbaf, 0xbbe, 0xb99, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc8, 0xb9c, 0xbc8, 0xbb2, 0xbcb, 0xbb0, 0xbcd,
0xb9f, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc8, 0xb9c, 0xbc8,
0xbb2, 0xbcb, 0xbb0, 0xbcd, 0xb9f, 0xbbe, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb95, 0xbc8, 0xb9c, 0xbc8, 0xbb2, 0xbcb, 0xbb0, 0xbcd, 0xb9f,
0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbb0, 0xbc0, 0xbaf, 0xbc2, 0xba9, 0xbbf, 0xbaf, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xbb0, 0xbcb, 0xba4, 0xbc7, 0xbb0, 0xbbe, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb9a, 0xb95, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb9a, 0xb95, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a,
0xb95, 0xbb2, 0xbbf, 0xba9, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbae, 0xbb0, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb9a, 0xbae, 0xbb0, 0xbbe, 0x20, 0xba4, 0xbb0, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbae, 0xbb0, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbae, 0xbcb, 0xbb5, 0xbbe, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbae, 0xbcb, 0xbb5, 0xbbe, 0x20, 0xba8,
0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xb9a, 0xbae, 0xbcb, 0xbb5, 0xbbe, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc0, 0xb9a, 0xbc6, 0xbb2, 0xbcd,
0xbb8, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbf, 0xb99, 0xbcd,
0xb95, 0xbaa, 0xbcd, 0xbaa, 0xbc2, 0xbb0, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8,
0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbbe, 0xbb2,
0xbae, 0xba9, 0xbcd, 0x20, 0xba4, 0xbc0, 0xbb5, 0xbc1, 0xb95, 0xbb3, 0xbcd, 0x20,
0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc6, 0xbb1, 0xbcd, 0xb95, 0xbc1, 0x20,
0xb9c, 0xbbe, 0xbb0, 0xbcd, 0xb9c, 0xbbf, 0xbaf, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xb9a, 0xbc1, 0xbb0, 0xbbf, 0xba9, 0xbbe, 0xbae, 0xbcd, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb8, 0xbcd, 0xbaf, 0xbcb, 0xbb5, 0xbbe, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbb9, 0xbbf, 0xba4, 0xbbf, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xba4, 0xbbe, 0xbaf, 0xbcd, 0xbaa, 0xbc7, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xba4, 0xbbe, 0xbaf, 0xbcd, 0xbaa, 0xbc7, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4,
0xbbe, 0xbaf, 0xbcd, 0xbaa, 0xbc7, 0x20, 0xbaa, 0xb95, 0xbb2, 0xbca, 0xbb3, 0xbbf,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xb9c, 0xbbf, 0xb95, 0xbbf, 0xbb8,
0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9f,
0xbcb, 0xb95, 0xbcd, 0xb95, 0xbc6, 0xbb2, 0xbbe, 0xbb5, 0xbcd, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb9f, 0xbcb, 0xb99, 0xbcd, 0xb95, 0xbbe, 0x20, 0xba8, 0xbc7,
0xbb0, 0xbae, 0xbcd, 0xb9f, 0xbcb, 0xb99, 0xbcd, 0xb95, 0xbbe, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9f,
0xbcb, 0xb99, 0xbcd, 0xb95, 0xbbe, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb9a, 0xbc1, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xba4, 0xbc1, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbae, 0xbc6, 0xba9, 0xbbf,
0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xba4, 0xbc1, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0xbae, 0xbc6, 0xba9, 0xbbf, 0xbb8, 0xbcd,
0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc1, 0xbb0, 0xbcd, 0xb95, 0xbcd,
0xbae, 0xbc6, 0xba9, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xba4, 0xbc1, 0xbb5,
0xbbe, 0xbb2, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb0, 0xbc1,
0xb95, 0xbc1, 0xbb5, 0xbc7, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb0,
0xbc1, 0xb95, 0xbc1, 0xbb5, 0xbc7, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe,
0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb0, 0xbc1, 0xb95, 0xbc1,
0xbb5, 0xbc7, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xb89, 0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe,
0xba9, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89, 0xbb8, 0xbcd, 0xbaa,
0xbc6, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xb89,
0xbb8, 0xbcd, 0xbaa, 0xbc6, 0xb95, 0xbbf, 0xbb8, 0xbcd, 0xba4, 0xbbe, 0xba9, 0xbcd,
0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5,
0xba9, 0xbc1, 0xbb5, 0xbbe, 0xb9f, 0xbcd, 0xb9f, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbb5, 0xba9, 0xbc1, 0xbb5, 0xbbe, 0xb9f, 0xbcd, 0xb9f, 0xbc1, 0x20,
0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae,
0xbcd, 0xbb5, 0xba9, 0xbc1, 0xbb5, 0xbbe, 0xb9f, 0xbcd, 0xb9f, 0xbc1, 0x20, 0xb95,
0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbc6, 0xba9,
0xbbf, 0xb9a, 0xbc1, 0xbb2, 0xbbe, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5,
0xbbf, 0xbb3, 0xbbe, 0xb9f, 0xbbf, 0xbb5, 0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbcb, 0xb95,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbbf, 0xbb3, 0xbbe, 0xb9f,
0xbbf, 0xbb5, 0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbcb, 0xb95, 0xbcd, 0x20, 0xba8, 0xbbf,
0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5,
0xbbf, 0xbb3, 0xbbe, 0xb9f, 0xbbf, 0xbb5, 0xbcb, 0xbb8, 0xbcd, 0xb9f, 0xbcb, 0xb95,
0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd,
0xbb5, 0xbcb, 0xbb2, 0xbcd, 0xb95, 0xbcb, 0xb95, 0xbcd, 0xbb0, 0xbbe, 0xb9f, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbcb, 0xbb2, 0xbcd, 0xb95, 0xbcb,
0xb95, 0xbcd, 0xbb0, 0xbbe, 0xb9f, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf,
0xbbe, 0xba9, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbcb, 0xbb2, 0xbcd,
0xb95, 0xbcb, 0xb95, 0xbcd, 0xbb0, 0xbbe, 0xb9f, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f,
0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbc7, 0xb95, 0xbcd, 0x20,
0xba4, 0xbc0, 0xbb5, 0xbc1, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbb5, 0xbbe,
0xbb2, 0xbbf, 0xbb8, 0xbcd, 0x20, 0xbae, 0xbb1, 0xbcd, 0xbb1, 0xbc1, 0xbae, 0xbcd,
0x20, 0xb83, 0xbaa, 0xbcd, 0xbaf, 0xbc2, 0xb9f, 0xbc1, 0xba9, 0xbbe, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8, 0xbcd, 0xb95,
0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xb95, 0xbc1, 0xb9f, 0xbcd,
0xbb8, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xb95, 0xbc1, 0xb9f, 0xbcd, 0xbb8,
0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8, 0x20, 0xba8, 0xbc7, 0xbb0,
0xbae, 0xbcd, 0xbaf, 0xbc7, 0xb95, 0xbbe, 0xb9f, 0xbc6, 0xbb0, 0xbbf, 0xba9, 0xbcd,
0xbaa, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf,
0xbc7, 0xb95, 0xbbe, 0xb9f, 0xbc6, 0xbb0, 0xbbf, 0xba9, 0xbcd, 0xbaa, 0xbb0, 0xbcd,
0xb95, 0xbcd, 0x20, 0xba8, 0xbbf, 0xbb2, 0xbc8, 0xbaf, 0xbbe, 0xba9, 0x20, 0xba8,
0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xbc7, 0xb95, 0xbbe, 0xb9f, 0xbc6, 0xbb0, 0xbbf,
0xba9, 0xbcd, 0xbaa, 0xbb0, 0xbcd, 0xb95, 0xbcd, 0x20, 0xb95, 0xbcb, 0xb9f, 0xbc8,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0xbaf, 0xbc2, 0xb95, 0xbcb, 0xba9, 0xbcd,
0x20, 0xba8, 0xbc7, 0xbb0, 0xbae, 0xbcd, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20,
0x43, 0x6f, 0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x53,
0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20,
0x43, 0x6f, 0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x4a,
0x69, 0x6b, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6e, 0x2d, 0x41, 0x6d, 0x61,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x74,
0x20, 0x52, 0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x54, 0x6f, 0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61,
0x53, 0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x54, 0x6f, 0x6e, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61,
0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x2d, 0x41, 0x6d,
0x61, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61,
0x74, 0x20, 0x52, 0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x59, 0x61, 0x6d, 0x61, 0x2d, 0x41, 0x6d, 0x61, 0x72, 0x69,
0x6b, 0x61, 0x53, 0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b,
0x61, 0x6e, 0x20, 0x59, 0x61, 0x6d, 0x61, 0x2d, 0x41, 0x6d, 0x61, 0x72,
0x69, 0x6b, 0x61, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x48, 0x69, 0x64,
0x61, 0x77, 0x20, 0x6e, 0x69, 0x71, 0x20, 0x59, 0x61, 0x6d, 0x61, 0x2d,
0x41, 0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x4a, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x44, 0x61, 0x79, 0x62, 0x69, 0x6e, 0x79, 0x61, 0x6e, 0x53, 0x6e,
0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x41,
0x6d, 0x61, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x44, 0x61, 0x79, 0x62, 0x69,
0x6e, 0x79, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x41, 0x6d,
0x61, 0x72, 0x69, 0x6b, 0x61, 0x2d, 0x44, 0x61, 0x79, 0x62, 0x69, 0x6e,
0x79, 0x61, 0x6e, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x20,
0x52, 0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20,
0x59, 0x61, 0x79, 0x75, 0x6e, 0x20, 0x54, 0x61, 0x73, 0x69, 0x79, 0x61,
0x6e, 0x53, 0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x59, 0x61, 0x79, 0x75, 0x6e, 0x20, 0x54, 0x61, 0x73, 0x69,
0x79, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x79,
0x75, 0x6e, 0x20, 0x54, 0x61, 0x73, 0x69, 0x79, 0x61, 0x6e, 0x20, 0x6f,
0x20, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x20, 0x52, 0x62, 0x61, 0x67, 0x61,
0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x43, 0x6f, 0x6e, 0x6f, 0x77,
0x53, 0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x43, 0x6f, 0x6e, 0x6f, 0x77, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20,
0x43, 0x6f, 0x6e, 0x6f, 0x77, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61,
0x74, 0x20, 0x52, 0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x6f, 0x77, 0x53, 0x6e, 0x65, 0x67, 0x75,
0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x6f,
0x77, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x54, 0x6f, 0x6e, 0x6f, 0x77,
0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x74, 0x20, 0x52, 0x62, 0x61,
0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0x53, 0x69, 0x6f,
0x77, 0x53, 0x6e, 0x65, 0x67, 0x75, 0x6e, 0x20, 0x4a, 0x69, 0x6b, 0x61,
0x6e, 0x20, 0x53, 0x69, 0x6f, 0x77, 0x4a, 0x69, 0x6b, 0x61, 0x6e, 0x20,
0x53, 0x69, 0x6f, 0x77, 0x20, 0x6f, 0x20, 0x4b, 0x61, 0x72, 0x61, 0x74,
0x20, 0x52, 0x62, 0x61, 0x67, 0x61, 0x6e, 0x4a, 0x69, 0x6b, 0x61, 0x6e,
0x20, 0x51, 0x75, 0x72, 0x69, 0x20, 0x47, 0x72, 0x69, 0x6e, 0x77, 0x65,
0x79, 0x6a, 0x69, 0x410, 0x43a, 0x440, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x4af, 0x437, 0x4d9, 0x43a, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438,
0x20, 0x4af, 0x437, 0x4d9, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x4af, 0x437, 0x4d9,
0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c,
0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x43a,
0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20,
0x43a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20,
0x43a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c,
0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x442, 0x430, 0x443, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x433, 0x430, 0x434, 0x4d9, 0x442,
0x438, 0x20, 0x442, 0x430, 0x443, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x442, 0x430, 0x443,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f,
0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x422, 0x44b,
0x43d, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20,
0x422, 0x44b, 0x43d, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x422, 0x44b, 0x43d, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x20, 0x432,
0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x410, 0x43d, 0x430, 0x434, 0x44b, 0x440, 0x44c,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f,
0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x430, 0x442,
0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442,
0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x430,
0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x422, 0x4e9, 0x43d, 0x44c, 0x44f, 0x43a, 0x20, 0x410, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x20, 0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x430,
0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x4ae, 0x437, 0x4d9, 0x43a, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f,
0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x433, 0x430, 0x434, 0x4d9,
0x442, 0x438, 0x20, 0x4ae, 0x437, 0x4d9, 0x43a, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x497, 0x4d9, 0x439,
0x433, 0x435, 0x20, 0x4ae, 0x437, 0x4d9, 0x43a, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d,
0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x433, 0x430, 0x434, 0x4d9, 0x442,
0x438, 0x20, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433, 0x44b, 0x448, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b,
0x497, 0x4d9, 0x439, 0x433, 0x435, 0x20, 0x41a, 0x4e9, 0x43d, 0x447, 0x44b, 0x433,
0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448,
0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b,
0x442, 0x44b, 0x433, 0x430, 0x434, 0x4d9, 0x442, 0x438, 0x20, 0x41a, 0x4e9, 0x43d,
0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430,
0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x497, 0x4d9, 0x439, 0x433, 0x435,
0x20, 0x41a, 0x4e9, 0x43d, 0x431, 0x430, 0x442, 0x44b, 0x448, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x413,
0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x20, 0x443, 0x440, 0x442, 0x430, 0x447,
0x430, 0x20, 0x432, 0x430, 0x43a, 0x44b, 0x442, 0x44b, 0x41f, 0x435, 0x442, 0x440,
0x43e, 0x43f, 0x430, 0x432, 0x43b, 0x43e, 0x432, 0x441, 0x43a, 0x2d, 0x41a, 0x430,
0x43c, 0x447, 0x430, 0x442, 0x441, 0x43a, 0x438, 0x439, 0x20, 0x432, 0x430, 0x43a,
0x44b, 0x442, 0x44b, 0x421, 0x430, 0x43c, 0x430, 0x440, 0x430, 0x20, 0x432, 0x430,
0x43a, 0x44b, 0x442, 0x44b, 0xc0f, 0xc15, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc0f, 0xc15, 0xc30, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc0f, 0xc15, 0xc30, 0xc4d,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06,
0xc2b, 0xc4d, 0xc18, 0xc28, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc30, 0xc32, 0xc4d,
0x20, 0xc06, 0xc2b, 0xc4d, 0xc30, 0xc3f, 0xc15, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc06, 0xc2b, 0xc4d, 0xc30,
0xc3f, 0xc15, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc26, 0xc15, 0xc4d, 0xc37,
0xc3f, 0xc23, 0x20, 0xc06, 0xc2b, 0xc4d, 0xc30, 0xc3f, 0xc15, 0xc3e, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc06, 0xc2b, 0xc4d, 0xc30,
0xc3f, 0xc15, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a,
0xc3f, 0xc2e, 0x20, 0xc06, 0xc2b, 0xc4d, 0xc30, 0xc3f, 0xc15, 0xc3e, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc06, 0xc2b, 0xc4d, 0xc30,
0xc3f, 0xc15, 0xc3e, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc05, 0xc32, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc3e, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc05, 0xc32, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc3e, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc05, 0xc32, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc3e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f,
0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc05, 0xc32, 0xc4d, 0xc2e, 0xc3e, 0xc1f, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc05, 0xc32, 0xc4d, 0xc2e, 0xc3e, 0xc1f, 0xc3f, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc05, 0xc32, 0xc4d, 0xc2e, 0xc3e, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc2e, 0xc46, 0xc1c, 0xc3e, 0xc28,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc2e, 0xc46, 0xc1c, 0xc3e, 0xc28,
0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc2e, 0xc46, 0xc1c, 0xc3e, 0xc28, 0xc4d, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc27,
0xc4d, 0xc2f, 0xc2e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc27, 0xc4d, 0xc2f,
0xc2e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20, 0xc2a, 0xc17,
0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20,
0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2e, 0xc4c, 0xc02, 0xc1f, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc4c, 0xc02, 0xc1f, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc4c, 0xc02, 0xc1f, 0xc46, 0xc2f, 0xc3f, 0xc28, 0xc4d,
0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41,
0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc38, 0xc3f, 0xc2b, 0xc3f,
0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc38, 0xc3f, 0xc2b, 0xc3f,
0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc38, 0xc3f, 0xc2b, 0xc3f, 0xc15, 0xc4d,
0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41,
0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc28, 0xc21, 0xc48, 0xc30,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc28, 0xc3e, 0xc28, 0xc4d, 0xc21,
0xc4d, 0xc30, 0xc3f, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc28, 0xc3e, 0xc28, 0xc4d, 0xc21,
0xc4d, 0xc30, 0xc3f, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc0f, 0xc2a, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc0f, 0xc2a, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc0f, 0xc2a, 0xc3f, 0xc2f,
0xc3e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05,
0xc15, 0xc4d, 0xc35, 0xc3e, 0xc1f, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05,
0xc15, 0xc4d, 0xc35, 0xc3e, 0xc1f, 0xc41, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc15, 0xc4d,
0xc35, 0xc3e, 0xc1f, 0xc41, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc05, 0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc2c, 0xc46, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc05, 0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc2c, 0xc46, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc05, 0xc15, 0xc4d, 0xc1f, 0xc4b, 0xc2c, 0xc46, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc30, 0xc47, 0xc2c, 0xc3f,
0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc30, 0xc47, 0xc2c,
0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc30, 0xc47, 0xc2c, 0xc3f,
0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32,
0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc30,
0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40, 0xc28, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc05, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40, 0xc28, 0xc3e, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc06, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40, 0xc28, 0xc3e,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a,
0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc05, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02,
0xc1f, 0xc40, 0xc28, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d,
0xc1a, 0xc3f, 0xc2e, 0x20, 0xc05, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40,
0xc28, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20,
0xc05, 0xc30, 0xc4d, 0xc1c, 0xc46, 0xc02, 0xc1f, 0xc40, 0xc28, 0xc3e, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc30, 0xc4d,
0xc2e, 0xc47, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06,
0xc30, 0xc4d, 0xc2e, 0xc47, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06,
0xc30, 0xc4d, 0xc2e, 0xc47, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1f, 0xc4d, 0xc32, 0xc3e,
0xc02, 0xc1f, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1f,
0xc4d, 0xc32, 0xc3e, 0xc02, 0xc1f, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05,
0xc1f, 0xc4d, 0xc32, 0xc3e, 0xc02, 0xc1f, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc17,
0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32,
0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f,
0xc3e, 0x20, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc38,
0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2e, 0xc27,
0xc4d, 0xc2f, 0xc2e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32,
0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc38,
0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2e, 0xc27,
0xc4d, 0xc2f, 0xc2e, 0x20, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e, 0x20, 0xc06, 0xc38, 0xc4d,
0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc36,
0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d,
0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2e, 0xc27, 0xc4d, 0xc2f, 0xc2e,
0x20, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f,
0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc06, 0xc38, 0xc4d,
0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc28,
0xc4d, 0x20, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc06,
0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20,
0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20,
0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc06, 0xc38, 0xc4d, 0xc1f,
0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc06, 0xc38, 0xc4d, 0xc1f, 0xc4d, 0xc30, 0xc47, 0xc32, 0xc3f, 0xc2f, 0xc28, 0xc4d,
0x20, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f,
0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc05, 0xc1c, 0xc30, 0xc4d, 0xc2c, 0xc48, 0xc1c, 0xc3e, 0xc28, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1c, 0xc30, 0xc4d, 0xc2c, 0xc48, 0xc1c,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1c, 0xc30, 0xc4d, 0xc2c, 0xc48,
0xc1c, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1c, 0xc4b, 0xc30, 0xc4d, 0xc38, 0xc4d, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc05, 0xc1c, 0xc4b, 0xc30, 0xc4d, 0xc38, 0xc4d, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc05, 0xc1c, 0xc4b, 0xc30, 0xc4d, 0xc38, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc02, 0xc17, 0xc4d, 0xc32,
0xc3e, 0xc26, 0xc47, 0xc36, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc02,
0xc17, 0xc4d, 0xc32, 0xc3e, 0xc26, 0xc47, 0xc36, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c,
0xc02, 0xc17, 0xc4d, 0xc32, 0xc3e, 0xc26, 0xc47, 0xc36, 0xc4d, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2d, 0xc42, 0xc1f, 0xc3e,
0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc4a, 0xc32, 0xc40, 0xc35,
0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc4d, 0xc30, 0xc46,
0xc1c, 0xc3f, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c,
0xc4d, 0xc30, 0xc46, 0xc1c, 0xc3f, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2c, 0xc4d, 0xc30, 0xc46, 0xc1c, 0xc3f, 0xc32, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2c, 0xc4d, 0xc30,
0xc42, 0xc28, 0xc47, 0x20, 0xc26, 0xc30, 0xc41, 0xc38, 0xc32, 0xc3e, 0xc2e, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc47, 0xc2a, 0xc4d, 0x20, 0xc35, 0xc46,
0xc30, 0xc4d, 0xc21, 0xc46, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc47, 0xc2a,
0xc4d, 0x20, 0xc35, 0xc46, 0xc30, 0xc4d, 0xc21, 0xc46, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15,
0xc47, 0xc2a, 0xc4d, 0x20, 0xc35, 0xc46, 0xc30, 0xc4d, 0xc21, 0xc46, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3e, 0xc2e,
0xc30, 0xc4d, 0xc30, 0xc4b, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3e, 0xc25, 0xc2e, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3e, 0xc25, 0xc2e, 0xc4d, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc1a, 0xc3e, 0xc25, 0xc2e, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20,
0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc1a, 0xc3f, 0xc32, 0xc40, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3f,
0xc32, 0xc40, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc3f, 0xc32, 0xc40, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc48, 0xc28, 0xc3e,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc48, 0xc28, 0xc3e, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc1a, 0xc48, 0xc28, 0xc3e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46,
0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a,
0xc4b, 0xc2f, 0xc3f, 0xc2c, 0xc32, 0xc4d, 0xc38, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc4b, 0xc2f, 0xc3f, 0xc2c, 0xc32, 0xc4d, 0xc38, 0xc3e,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc4b, 0xc2f, 0xc3f, 0xc2c, 0xc32, 0xc4d,
0xc38, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d, 0xc30, 0xc3f, 0xc38, 0xc4d, 0xc2e, 0xc38, 0xc4d,
0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4b,
0xc15, 0xc4b, 0xc38, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a, 0xc32, 0xc02, 0xc2c, 0xc3f, 0xc2f, 0xc3e, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a, 0xc32, 0xc02, 0xc2c, 0xc3f, 0xc2f, 0xc3e,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a, 0xc32, 0xc02, 0xc2c, 0xc3f, 0xc2f, 0xc3e, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc41,
0xc15, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc15, 0xc41, 0xc15, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc15, 0xc41, 0xc15, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32,
0x20, 0xc05, 0xc30, 0xc4d, 0xc27, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d, 0xc2f, 0xc42, 0xc2c, 0xc3e, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d, 0xc2f, 0xc42, 0xc2c, 0xc3e, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc15, 0xc4d, 0xc2f, 0xc42, 0xc2c, 0xc3e, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20,
0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc21, 0xc47, 0xc35, 0xc3f, 0xc38, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc21, 0xc4d, 0xc2f, 0xc42, 0xc2e, 0xc3e, 0xc02, 0xc1f, 0xc4d, 0x2d, 0xc21, 0xc3f,
0x2019, 0xc09, 0xc30, 0xc4d, 0xc35, 0xc3f, 0xc32, 0xc4d, 0xc32, 0xc47, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc24, 0xc48,
0xc2e, 0xc42, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc08, 0xc38, 0xc4d,
0xc1f, 0xc30, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc08, 0xc38, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc08, 0xc38, 0xc4d, 0xc1f, 0xc30, 0xc4d, 0x20, 0xc26, 0xc40,
0xc35, 0xc3f, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc08, 0xc15, 0xc4d, 0xc35, 0xc21, 0xc3e, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc38, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc30, 0xc32, 0xc4d, 0x20, 0xc2f,
0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc38, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc30, 0xc32, 0xc4d, 0x20, 0xc2f, 0xc42,
0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc46,
0xc02, 0xc1f, 0xc4d, 0xc30, 0xc32, 0xc4d, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a,
0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2f, 0xc42,
0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a,
0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a,
0xc41, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc41,
0xc26, 0xc42, 0xc30, 0x2d, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc2f,
0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b,
0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36,
0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20,
0xc2f, 0xc42, 0xc30, 0xc4b, 0xc2a, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3e, 0xc15, 0xc4d,
0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35,
0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3e, 0xc15, 0xc4d, 0x200c,
0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41,
0xc32, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3e, 0xc15, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f,
0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc1c,
0xc40, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc1c, 0xc40, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2b, 0xc3f, 0xc1c, 0xc40, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc4d, 0xc30, 0xc46, 0xc02, 0xc1a, 0xc4d, 0x20,
0xc17, 0xc2f, 0xc3e, 0xc28, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc4d,
0xc30, 0xc46, 0xc02, 0xc1a, 0xc4d, 0x20, 0xc26, 0xc15, 0xc4d, 0xc37, 0xc3f, 0xc23,
0x20, 0xc2e, 0xc30, 0xc3f, 0xc2f, 0xc41, 0x20, 0xc05, 0xc02, 0xc1f, 0xc3e, 0xc30,
0xc4d, 0xc15, 0xc3f, 0xc1f, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc17, 0xc3e, 0xc32, 0xc3e, 0xc2a, 0xc3e, 0xc17, 0xc4b, 0xc38, 0xc4d, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc17, 0xc3e, 0xc02, 0xc2c, 0xc3f, 0xc2f, 0xc30, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1c, 0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f, 0xc2f, 0xc3e,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1c, 0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f, 0xc2f,
0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1c, 0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f, 0xc2f, 0xc3e,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc17,
0xc3f, 0xc32, 0xc4d, 0xc2c, 0xc30, 0xc4d, 0xc1f, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35,
0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc17, 0xc4d, 0xc30, 0xc40, 0xc28,
0xc4d, 0x200c, 0xc35, 0xc3f, 0xc1a, 0xc4d, 0x20, 0xc38, 0xc17, 0xc1f, 0xc41, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc17,
0xc4d, 0xc30, 0xc40, 0xc28, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41,
0x20, 0xc17, 0xc4d, 0xc30, 0xc40, 0xc28, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e,
0xc02, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41,
0x20, 0xc17, 0xc4d, 0xc30, 0xc40, 0xc28, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e,
0xc02, 0xc21, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc17, 0xc4d, 0xc30,
0xc40, 0xc28, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc17,
0xc4d, 0xc30, 0xc40, 0xc28, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21,
0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc17,
0xc4d, 0xc30, 0xc40, 0xc28, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21,
0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc17, 0xc4d, 0xc35, 0xc3e, 0xc2e, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc17, 0xc32, 0xc4d,
0xc2b, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc35, 0xc3e, 0xc2f, 0xc4d, 0x2d, 0xc05,
0xc32, 0xc4d, 0xc2f, 0xc42, 0xc37, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc39, 0xc35, 0xc3e, 0xc2f, 0xc4d, 0x2d, 0xc05, 0xc32, 0xc4d, 0xc2f,
0xc42, 0xc37, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc35, 0xc3e,
0xc2f, 0xc4d, 0x2d, 0xc05, 0xc32, 0xc4d, 0xc2f, 0xc42, 0xc37, 0xc3f, 0xc2f, 0xc28,
0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24,
0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc3e, 0xc02, 0xc15,
0xc3e, 0xc02, 0xc17, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc3e, 0xc02,
0xc15, 0xc3e, 0xc02, 0xc17, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39, 0xc3e, 0xc02, 0xc15,
0xc3e, 0xc02, 0xc17, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc39, 0xc4b, 0xc35, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc39, 0xc4b, 0xc35, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc39,
0xc4b, 0xc35, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2d, 0xc3e, 0xc30, 0xc24, 0xc26, 0xc47, 0xc36, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc39, 0xc3f, 0xc02, 0xc26, 0xc42, 0x20, 0xc2e, 0xc39, 0xc3e, 0x20,
0xc38, 0xc2e, 0xc41, 0xc26, 0xc4d, 0xc30, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07,
0xc02, 0xc21, 0xc4b, 0xc1a, 0xc48, 0xc28, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc38, 0xc46, 0xc02, 0xc1f, 0xc4d, 0xc30, 0xc32, 0xc4d, 0x20, 0xc07, 0xc02, 0xc21,
0xc4b, 0xc28, 0xc47, 0xc37, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc24, 0xc42, 0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc07, 0xc02, 0xc21, 0xc4b, 0xc28,
0xc47, 0xc37, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36,
0xc4d, 0xc1a, 0xc3f, 0xc2e, 0x20, 0xc07, 0xc02, 0xc21, 0xc4b, 0xc28, 0xc47, 0xc37,
0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc30, 0xc3e, 0xc28,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc30, 0xc3e, 0xc28, 0xc4d, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc07, 0xc30, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f,
0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc07, 0xc30, 0xc4d, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc30, 0xc4d, 0xc15, 0xc41, 0xc1f,
0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc30, 0xc4d, 0xc15,
0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc1c, 0xc4d, 0xc30, 0xc3e, 0xc2f,
0xc3f, 0xc32, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc1c, 0xc4d, 0xc30,
0xc3e, 0xc2f, 0xc3f, 0xc32, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc07, 0xc1c, 0xc4d, 0xc30,
0xc3e, 0xc2f, 0xc3f, 0xc32, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35,
0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc1c, 0xc2a, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1c, 0xc2a,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1c, 0xc2a, 0xc3e, 0xc28, 0xc4d, 0x20,
0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30,
0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc46, 0xc1f, 0xc4d, 0xc30, 0xc4b,
0xc2a, 0xc3e, 0xc35, 0xc4d, 0xc32, 0xc4b, 0xc35, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d,
0x2d, 0xc15, 0xc3e, 0xc2e, 0xc4d, 0xc1b, 0xc3e, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc46, 0xc1f, 0xc4d, 0xc30, 0xc4b,
0xc2a, 0xc3e, 0xc35, 0xc4d, 0xc32, 0xc4b, 0xc35, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d,
0x2d, 0xc15, 0xc3e, 0xc2e, 0xc4d, 0xc1b, 0xc3e, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15,
0xc3f, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc46, 0xc1f, 0xc4d, 0xc30, 0xc4b, 0xc2a, 0xc3e,
0xc35, 0xc4d, 0xc32, 0xc4b, 0xc35, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x2d, 0xc15,
0xc3e, 0xc2e, 0xc4d, 0xc1b, 0xc3e, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc3f, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc42,
0xc30, 0xc4d, 0xc2a, 0xc41, 0x20, 0xc15, 0xc1c, 0xc15, 0xc3f, 0x200c, 0xc38, 0xc4d,
0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc36, 0xc4d,
0xc1a, 0xc3f, 0xc2e, 0x20, 0xc15, 0xc1c, 0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e,
0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a, 0xc30, 0xc3f, 0xc2f,
0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a, 0xc30, 0xc3f, 0xc2f,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4a, 0xc30, 0xc3f, 0xc2f, 0xc28, 0xc4d,
0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41,
0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4b, 0xc38, 0xc4d, 0xc30,
0xc3e, 0xc2f, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d, 0xc30, 0xc3e,
0xc38, 0xc4d, 0xc28, 0xc4b, 0xc2f, 0xc3e, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc4d, 0xc30, 0xc3e, 0xc38, 0xc4d, 0xc28,
0xc4b, 0xc2f, 0xc3e, 0xc30, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc15, 0xc4d, 0xc30, 0xc3e, 0xc38, 0xc4d, 0xc28, 0xc4b, 0xc2f, 0xc3e, 0xc30, 0xc4d,
0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc15, 0xc3f, 0xc30, 0xc4d, 0xc17, 0xc3f, 0xc38, 0xc4d, 0xc24,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc32, 0xc02, 0xc15, 0xc3e,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc32, 0xc48, 0xc28, 0xc4d, 0x20, 0xc26, 0xc40,
0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc32, 0xc3e, 0xc30, 0xc4d,
0xc21, 0xc4d, 0x20, 0xc39, 0xc4b, 0xc35, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc32, 0xc3e, 0xc30, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc39, 0xc4b, 0xc35, 0xc4d, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc32, 0xc3e, 0xc30, 0xc4d, 0xc21, 0xc4d, 0x20, 0xc39, 0xc4b, 0xc35,
0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e,
0xc15, 0xc3e, 0xc35, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc15, 0xc3e,
0xc35, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc15, 0xc3e, 0xc35, 0xc4d, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc17, 0xc21,
0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc17, 0xc21, 0xc3e,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc17, 0xc21, 0xc3e, 0xc28, 0xc4d, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc32,
0xc47, 0xc37, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e,
0xc32, 0xc4d, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2e, 0xc3e, 0xc30, 0xc4d, 0xc15, 0xc4d, 0xc35, 0xc47, 0xc38, 0xc3e, 0xc38, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc30, 0xc4d, 0xc37, 0xc32, 0xc4d,
0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e,
0xc3e, 0xc30, 0xc3f, 0xc37, 0xc38, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e,
0xc3e, 0xc30, 0xc3f, 0xc37, 0xc38, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e,
0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc30,
0xc3f, 0xc37, 0xc38, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc38, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2e, 0xc46, 0xc15, 0xc4d, 0xc38, 0xc3f, 0xc15, 0xc28, 0xc4d, 0x20, 0xc2a,
0xc38, 0xc3f, 0xc2b, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e,
0xc46, 0xc15, 0xc4d, 0xc38, 0xc3f, 0xc15, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc38, 0xc3f,
0xc2b, 0xc3f, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc46, 0xc15, 0xc4d, 0xc38,
0xc3f, 0xc15, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc38, 0xc3f, 0xc2b, 0xc3f, 0xc15, 0xc4d,
0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41,
0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09, 0xc32, 0xc28, 0xc4d, 0x20,
0xc2c, 0xc24, 0xc4b, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09, 0xc32,
0xc28, 0xc4d, 0x20, 0xc2c, 0xc24, 0xc4b, 0xc30, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09,
0xc32, 0xc28, 0xc4d, 0x20, 0xc2c, 0xc24, 0xc4b, 0xc30, 0xc4d, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc38, 0xc4d,
0xc15, 0xc4b, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc38, 0xc4d, 0xc15,
0xc4b, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc3e, 0xc38, 0xc4d, 0xc15, 0xc4b, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2e, 0xc2f, 0xc28,
0xc4d, 0xc2e, 0xc3e, 0xc30, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4c,
0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc47, 0xc2a, 0xc3e, 0xc32,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0x20, 0xc15,
0xc3e, 0xc32, 0xc46, 0xc21, 0xc4b, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0x20, 0xc15, 0xc3e, 0xc32, 0xc46, 0xc21,
0xc4b, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42,
0x20, 0xc15, 0xc3e, 0xc32, 0xc46, 0xc21, 0xc4b, 0xc28, 0xc3f, 0xc2f, 0xc3e, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d,
0xc2f, 0xc42, 0xc1c, 0xc3f, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0xc1c, 0xc3f, 0xc32, 0xc4d,
0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42,
0xc1c, 0xc3f, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc17,
0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0xc2b, 0xc4c, 0xc02, 0xc21,
0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc28, 0xc4d, 0xc2f, 0xc42, 0xc2b, 0xc4c, 0xc02, 0xc21, 0xc4d, 0x200c,
0xc32, 0xc4d, 0xc2f, 0xc3e, 0xc02, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4d,
0xc2f, 0xc42, 0xc2b, 0xc4c, 0xc02, 0xc21, 0xc4d, 0x200c, 0xc32, 0xc4d, 0xc2f, 0xc3e,
0xc02, 0xc21, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32,
0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc3f,
0xc2f, 0xc42, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc3e, 0xc30, 0xc4d, 0xc2b,
0xc4b, 0xc15, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc28, 0xc3e, 0xc30, 0xc4d, 0xc2b, 0xc4b, 0xc15, 0xc4d, 0x20, 0xc26, 0xc40,
0xc35, 0xc3f, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc3e, 0xc30, 0xc4d, 0xc2b, 0xc4b, 0xc15,
0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc46, 0xc30, 0xc4d, 0xc28, 0xc3e, 0xc02, 0xc21,
0xc4b, 0x20, 0xc21, 0xc3f, 0x20, 0xc28, 0xc4a, 0xc30, 0xc4b, 0xc28, 0xc4d, 0xc39,
0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc46, 0xc30, 0xc4d, 0xc28, 0xc3e,
0xc02, 0xc21, 0xc4b, 0x20, 0xc21, 0xc3f, 0x20, 0xc28, 0xc4a, 0xc30, 0xc4b, 0xc28,
0xc4d, 0xc39, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc46, 0xc30, 0xc4d, 0xc28, 0xc3e,
0xc02, 0xc21, 0xc4b, 0x20, 0xc21, 0xc3f, 0x20, 0xc28, 0xc4a, 0xc30, 0xc4b, 0xc28,
0xc4d, 0xc39, 0xc3e, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc09, 0xc24, 0xc4d, 0xc24, 0xc30, 0x20, 0xc2e, 0xc3e, 0xc30, 0xc3f,
0xc2f, 0xc3e, 0xc28, 0xc3e, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4b, 0xc35, 0xc4b, 0xc38, 0xc3f, 0xc2c, 0xc3f, 0xc30,
0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4b,
0xc35, 0xc4b, 0xc38, 0xc3f, 0xc2c, 0xc3f, 0xc30, 0xc4d, 0xc15, 0xc4d, 0xc38, 0xc4d,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc28, 0xc4b, 0xc35, 0xc4b, 0xc38, 0xc3f, 0xc2c, 0xc3f, 0xc30,
0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc13, 0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc13, 0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20,
0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc13, 0xc2e, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc3e, 0xc15, 0xc3f,
0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a,
0xc3e, 0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc2a, 0xc3e, 0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc35,
0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc3e, 0xc32,
0xc3e, 0xc35, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc3e, 0xc2a, 0xc41,
0xc35, 0xc3e, 0x20, 0xc28, 0xc4d, 0xc2f, 0xc42, 0x20, 0xc17, 0xc3f, 0xc28, 0xc3f,
0xc2f, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc30, 0xc3e, 0xc17, 0xc4d,
0xc35, 0xc47, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc30, 0xc3e, 0xc17, 0xc4d,
0xc35, 0xc47, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc30, 0xc3e, 0xc17, 0xc4d, 0xc35, 0xc47,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a,
0xc46, 0xc30, 0xc42, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc46, 0xc30, 0xc42,
0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc46, 0xc30, 0xc42, 0x20, 0xc35, 0xc47, 0xc38, 0xc35,
0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc32, 0xc3f, 0xc2a, 0xc4d,
0xc2a, 0xc48, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc32,
0xc3f, 0xc2a, 0xc4d, 0xc2a, 0xc48, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e,
0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f,
0xc32, 0xc3f, 0xc2a, 0xc4d, 0xc2a, 0xc48, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2b, 0xc3f, 0xc28, 0xc3f, 0xc15,
0xc4d, 0xc38, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc2a, 0xc3f,
0xc2f, 0xc46, 0xc30, 0xc4d, 0x20, 0xc2e, 0xc30, 0xc3f, 0xc2f, 0xc41, 0x20, 0xc2e,
0xc3f, 0xc15, 0xc4d, 0xc35, 0xc46, 0xc32, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc2a, 0xc3f,
0xc2f, 0xc46, 0xc30, 0xc4d, 0x20, 0xc2e, 0xc30, 0xc3f, 0xc2f, 0xc41, 0x20, 0xc2e,
0xc3f, 0xc15, 0xc4d, 0xc35, 0xc46, 0xc32, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc38, 0xc46, 0xc2f, 0xc3f, 0xc02, 0xc1f, 0xc4d, 0x20, 0xc2a, 0xc3f, 0xc2f, 0xc30,
0xc4d, 0x20, 0xc2e, 0xc30, 0xc3f, 0xc2f, 0xc41, 0x20, 0xc2e, 0xc3f, 0xc15, 0xc4d,
0xc35, 0xc46, 0xc32, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20,
0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc2a, 0xc3f, 0xc1f, 0xc4d, 0x200c, 0xc15, 0xc48, 0xc30, 0xc4d, 0xc28, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc4a, 0xc28, 0xc47, 0xc2a, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2a, 0xc4d, 0xc2f, 0xc4b, 0xc02, 0xc17, 0xc3e, 0xc02,
0xc17, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc3f, 0xc1c, 0xc3f, 0xc32,
0xc4b, 0xc30, 0xc4d, 0xc21, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15, 0xc3f,
0xc1c, 0xc3f, 0xc32, 0xc4b, 0xc30, 0xc4d, 0xc21, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc15,
0xc3f, 0xc1c, 0xc3f, 0xc32, 0xc4b, 0xc30, 0xc4d, 0xc21, 0xc3e, 0x20, 0xc35, 0xc47,
0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc30, 0xc40, 0xc2f, 0xc42,
0xc28, 0xc3f, 0xc2f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc30, 0xc4a,
0xc24, 0xc47, 0xc30, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc16, 0xc3e,
0xc32, 0xc3f, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc16, 0xc3e,
0xc32, 0xc3f, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc16, 0xc3e, 0xc32, 0xc3f,
0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc38, 0xc2e, 0xc3e, 0xc30, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38,
0xc2e, 0xc3e, 0xc30, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23,
0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc2e, 0xc3e, 0xc30, 0xc3e,
0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38,
0xc2e, 0xc4b, 0xc35, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc2e, 0xc4b,
0xc35, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc2e, 0xc4b, 0xc35, 0xc3e, 0x20, 0xc2a,
0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41, 0xc30, 0xc41,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc40, 0xc37, 0xc46, 0xc32, 0xc4d, 0xc38,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc3f, 0xc02, 0xc17, 0xc2a, 0xc42,
0xc30, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc4b, 0xc32, 0xc2e, 0xc28, 0xc4d, 0x20,
0xc26, 0xc40, 0xc35, 0xc41, 0xc32, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc26, 0xc15,
0xc4d, 0xc37, 0xc3f, 0xc23, 0x20, 0xc1c, 0xc3e, 0xc30, 0xc4d, 0xc1c, 0xc3f, 0xc2f,
0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc42, 0xc30, 0xc3f, 0xc28, 0xc3e,
0xc2e, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc38, 0xc4d, 0xc2f, 0xc4b, 0xc35,
0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc39, 0xc3f, 0xc24, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc48, 0xc2a, 0xc40, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc24, 0xc48, 0xc2a, 0xc40, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e,
0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc48, 0xc2a, 0xc40,
0x20, 0xc2a, 0xc17, 0xc1f, 0xc3f, 0x20, 0xc35, 0xc46, 0xc32, 0xc41, 0xc24, 0xc41,
0xc30, 0xc41, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc1c, 0xc3f, 0xc15, 0xc3f,
0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1f,
0xc4b, 0xc15, 0xc46, 0xc32, 0xc3e, 0xc35, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc1f, 0xc3e, 0xc02, 0xc17, 0xc3e, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1f, 0xc3e,
0xc02, 0xc17, 0xc3e, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1f, 0xc3e, 0xc02, 0xc17, 0xc3e, 0x20,
0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc1a, 0xc15,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc41, 0xc30, 0xc4d, 0xc15, 0xc4d,
0x200c, 0xc2e, 0xc46, 0xc28, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc41, 0xc30, 0xc4d, 0xc15, 0xc4d, 0x200c, 0xc2e,
0xc46, 0xc28, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d,
0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc24, 0xc41, 0xc30, 0xc4d, 0xc15, 0xc4d, 0x200c, 0xc2e, 0xc46, 0xc28, 0xc3f, 0xc38,
0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc24, 0xc41, 0xc35, 0xc3e, 0xc32, 0xc41, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc09, 0xc30, 0xc41, 0xc17, 0xc4d, 0xc35, 0xc47, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc09, 0xc30, 0xc41, 0xc17, 0xc4d, 0xc35, 0xc47, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc09, 0xc30, 0xc41, 0xc17, 0xc4d, 0xc35, 0xc47, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09, 0xc1c, 0xc4d, 0xc2c, 0xc46,
0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc09, 0xc1c, 0xc4d, 0xc2c, 0xc46, 0xc15, 0xc3f, 0xc38, 0xc4d, 0xc24, 0xc3e,
0xc28, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc09, 0xc1c, 0xc4d, 0xc2c, 0xc46, 0xc15, 0xc3f,
0xc38, 0xc4d, 0xc24, 0xc3e, 0xc28, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc28, 0xc4c, 0xc1f, 0xc41, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc35, 0xc28, 0xc4c, 0xc1f, 0xc41, 0x20, 0xc2a, 0xc4d, 0xc30,
0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35,
0xc28, 0xc4c, 0xc1f, 0xc41, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc35, 0xc46, 0xc28, 0xc3f, 0xc1c, 0xc41, 0xc32, 0xc3e, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc4d, 0xc32, 0xc3e, 0xc21, 0xc3f, 0xc35, 0xc4b,
0xc38, 0xc4d, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35,
0xc4d, 0xc32, 0xc3e, 0xc21, 0xc3f, 0xc35, 0xc4b, 0xc38, 0xc4d, 0xc1f, 0xc4b, 0xc15,
0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20,
0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc4d, 0xc32, 0xc3e, 0xc21, 0xc3f, 0xc35, 0xc4b,
0xc38, 0xc4d, 0xc1f, 0xc4b, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc4b, 0xc32, 0xc4d, 0xc17, 0xc4b, 0xc17,
0xc4d, 0xc30, 0xc3e, 0xc21, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc35, 0xc4b,
0xc32, 0xc4d, 0xc17, 0xc4b, 0xc17, 0xc4d, 0xc30, 0xc3e, 0xc21, 0xc4d, 0x20, 0xc2a,
0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15, 0x20, 0xc38, 0xc2e, 0xc2f,
0xc02, 0xc35, 0xc4b, 0xc32, 0xc4d, 0xc17, 0xc4b, 0xc17, 0xc4d, 0xc30, 0xc3e, 0xc21,
0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02,
0xc35, 0xc47, 0xc15, 0xc4d, 0x20, 0xc26, 0xc40, 0xc35, 0xc3f, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc35, 0xc3e, 0xc32, 0xc40, 0xc38, 0xc4d, 0x20, 0xc2e, 0xc30, 0xc3f,
0xc2f, 0xc41, 0x20, 0xc2b, 0xc41, 0xc1f, 0xc41, 0xc28, 0xc3e, 0x20, 0xc38, 0xc2e,
0xc2f, 0xc02, 0xc2f, 0xc3e, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38, 0xc4d, 0xc15, 0xc4d,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc3e, 0xc15, 0xc41, 0xc1f, 0xc4d, 0xc38,
0xc4d, 0xc15, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f,
0xc15, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc3e, 0xc15, 0xc41, 0xc1f, 0xc4d,
0xc38, 0xc4d, 0xc15, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38, 0xc35, 0xc3f, 0x20, 0xc38,
0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc46, 0xc15, 0xc1f, 0xc46, 0xc30, 0xc3f, 0xc28, 0xc4d,
0x200c, 0xc2c, 0xc30, 0xc4d, 0xc17, 0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2f,
0xc46, 0xc15, 0xc1f, 0xc46, 0xc30, 0xc3f, 0xc28, 0xc4d, 0x200c, 0xc2c, 0xc30, 0xc4d,
0xc17, 0xc4d, 0x20, 0xc2a, 0xc4d, 0xc30, 0xc3e, 0xc2e, 0xc3e, 0xc23, 0xc3f, 0xc15,
0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc46, 0xc15, 0xc1f, 0xc46, 0xc30, 0xc3f,
0xc28, 0xc4d, 0x200c, 0xc2c, 0xc30, 0xc4d, 0xc17, 0xc4d, 0x20, 0xc35, 0xc47, 0xc38,
0xc35, 0xc3f, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xc2f, 0xc41, 0xc15, 0xc4b, 0xc28,
0xc4d, 0x20, 0xc38, 0xc2e, 0xc2f, 0xc02, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe32,
0xe40, 0xe01, 0xe23, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe2d, 0xe32, 0xe40, 0xe01, 0xe23, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe32, 0xe40, 0xe01, 0xe23, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2d, 0xe31, 0xe1f, 0xe01, 0xe32, 0xe19, 0xe34, 0xe2a, 0xe16,
0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe1f, 0xe23, 0xe34, 0xe01,
0xe32, 0xe01, 0xe25, 0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe1f,
0xe23, 0xe34, 0xe01, 0xe32, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01,
0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe1f, 0xe23, 0xe34, 0xe01, 0xe32, 0xe43,
0xe15, 0xe49, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe1f, 0xe23, 0xe34, 0xe01,
0xe32, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe2d, 0xe1f, 0xe23, 0xe34,
0xe01, 0xe32, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe41, 0xe2d, 0xe1f, 0xe23,
0xe34, 0xe01, 0xe32, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe30, 0xe41, 0xe25, 0xe2a, 0xe01, 0xe32, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe30, 0xe41, 0xe25,
0xe2a, 0xe01, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a,
0xe07, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe30, 0xe41, 0xe25, 0xe2a, 0xe01, 0xe32, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2d, 0xe31, 0xe25, 0xe21, 0xe32, 0xe15, 0xe35, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe31, 0xe25,
0xe21, 0xe32, 0xe15, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23,
0xe49, 0xe2d, 0xe19, 0xe2d, 0xe31, 0xe25, 0xe21, 0xe32, 0xe15, 0xe35, 0xe40, 0xe27,
0xe25, 0xe32, 0xe41, 0xe2d, 0xe21, 0xe30, 0xe0b, 0xe2d, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe2d, 0xe21, 0xe30,
0xe0b, 0xe2d, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe41, 0xe2d, 0xe21, 0xe30, 0xe0b, 0xe2d, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe43, 0xe19, 0xe2d, 0xe40,
0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe15, 0xe2d, 0xe19,
0xe01, 0xe25, 0xe32, 0xe07, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01,
0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d,
0xe21, 0xe41, 0xe2a, 0xe07, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe43,
0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37,
0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31,
0xe19, 0xe2d, 0xe2d, 0xe01, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01,
0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31,
0xe19, 0xe2d, 0xe2d, 0xe01, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01,
0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d,
0xe21, 0xe41, 0xe2a, 0xe07, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe2d, 0xe2d, 0xe01, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32,
0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe16, 0xe1a,
0xe20, 0xe39, 0xe40, 0xe02, 0xe32, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34,
0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe16, 0xe1a, 0xe20, 0xe39, 0xe40,
0xe02, 0xe32, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40,
0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41,
0xe2a, 0xe07, 0xe41, 0xe16, 0xe1a, 0xe20, 0xe39, 0xe40, 0xe02, 0xe32, 0xe43, 0xe19,
0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d,
0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe1b, 0xe0b, 0xe34, 0xe1f, 0xe34, 0xe01, 0xe43,
0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37,
0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe41, 0xe1b, 0xe0b, 0xe34, 0xe1f, 0xe34, 0xe01, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21,
0xe23, 0xe34, 0xe01, 0xe32, 0xe40, 0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe41, 0xe1b, 0xe0b, 0xe34, 0xe1f,
0xe34, 0xe01, 0xe43, 0xe19, 0xe2d, 0xe40, 0xe21, 0xe23, 0xe34, 0xe01, 0xe32, 0xe40,
0xe2b, 0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe30, 0xe19, 0xe32,
0xe14, 0xe35, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe2d, 0xe30, 0xe19, 0xe32, 0xe14, 0xe35, 0xe23, 0xe4c, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe30,
0xe19, 0xe32, 0xe14, 0xe35, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe32,
0xe1b, 0xe35, 0xe2d, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe1b, 0xe35, 0xe2d, 0xe32, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe2d, 0xe32, 0xe1b, 0xe35, 0xe2d,
0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe31, 0xe04, 0xe15, 0xe32, 0xe2d, 0xe39,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d,
0xe31, 0xe04, 0xe15, 0xe32, 0xe2d, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe31, 0xe04, 0xe15, 0xe32, 0xe2d, 0xe39,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe31, 0xe04, 0xe42, 0xe17, 0xe40, 0xe1a, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe31,
0xe04, 0xe42, 0xe17, 0xe40, 0xe1a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe31, 0xe04, 0xe42, 0xe17, 0xe40, 0xe1a, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2d, 0xe32, 0xe2b, 0xe23, 0xe31, 0xe1a, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe2b, 0xe23,
0xe31, 0xe1a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07,
0xe2d, 0xe32, 0xe2b, 0xe23, 0xe31, 0xe1a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe32,
0xe23, 0xe4c, 0xe40, 0xe08, 0xe19, 0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe23, 0xe4c,
0xe40, 0xe08, 0xe19, 0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe08,
0xe19, 0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe15, 0xe30, 0xe27,
0xe31, 0xe19, 0xe15, 0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40,
0xe08, 0xe19, 0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31,
0xe19, 0xe15, 0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe08,
0xe19, 0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe15, 0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe19,
0xe15, 0xe34, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe32, 0xe23, 0xe4c,
0xe40, 0xe21, 0xe40, 0xe19, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe21, 0xe40,
0xe19, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe2d, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe21, 0xe40, 0xe19, 0xe35, 0xe22,
0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe2d, 0xe15, 0xe41, 0xe25, 0xe19, 0xe15, 0xe34,
0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe41, 0xe2d, 0xe15, 0xe41, 0xe25, 0xe19, 0xe15, 0xe34, 0xe01, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe41, 0xe2d,
0xe15, 0xe41, 0xe25, 0xe19, 0xe15, 0xe34, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe01, 0xe25, 0xe32,
0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe17, 0xe32, 0xe07, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe02, 0xe2d,
0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe17, 0xe32, 0xe07,
0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d,
0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32,
0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe15, 0xe2d,
0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40,
0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31,
0xe19, 0xe15, 0xe01, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe02, 0xe2d,
0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe17, 0xe32, 0xe07,
0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25,
0xe32, 0xe07, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40,
0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15,
0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d,
0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe02,
0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe17, 0xe32,
0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe02, 0xe2d, 0xe07,
0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25, 0xe35, 0xe22,
0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27,
0xe31, 0xe19, 0xe15, 0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15,
0xe23, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21,
0xe41, 0xe2a, 0xe07, 0xe17, 0xe32, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15,
0xe01, 0xe02, 0xe2d, 0xe07, 0xe2d, 0xe2d, 0xe2a, 0xe40, 0xe15, 0xe23, 0xe40, 0xe25,
0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe32, 0xe40, 0xe0b, 0xe2d, 0xe23,
0xe4c, 0xe44, 0xe1a, 0xe08, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe32, 0xe40, 0xe0b, 0xe2d, 0xe23, 0xe4c,
0xe44, 0xe1a, 0xe08, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe32, 0xe40, 0xe0b, 0xe2d, 0xe23, 0xe4c, 0xe44,
0xe1a, 0xe08, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe30, 0xe42, 0xe0b,
0xe23, 0xe4c, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe2d, 0xe30, 0xe42, 0xe0b, 0xe23, 0xe4c, 0xe2a, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe30, 0xe42, 0xe0b,
0xe23, 0xe4c, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1a, 0xe31, 0xe07, 0xe01, 0xe25,
0xe32, 0xe40, 0xe17, 0xe28, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe1a, 0xe31, 0xe07, 0xe01, 0xe25, 0xe32, 0xe40, 0xe17, 0xe28,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe1a,
0xe31, 0xe07, 0xe01, 0xe25, 0xe32, 0xe40, 0xe17, 0xe28, 0xe40, 0xe27, 0xe25, 0xe32,
0xe20, 0xe39, 0xe0f, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe42, 0xe1a, 0xe25,
0xe34, 0xe40, 0xe27, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1a, 0xe23, 0xe32,
0xe0b, 0xe34, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe1a, 0xe23, 0xe32, 0xe0b, 0xe34, 0xe40, 0xe25,
0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe1a, 0xe23, 0xe32, 0xe0b, 0xe34, 0xe40, 0xe25, 0xe35, 0xe22, 0xe40, 0xe27,
0xe25, 0xe32, 0xe1a, 0xe23, 0xe39, 0xe44, 0xe19, 0xe14, 0xe32, 0xe23, 0xe38, 0xe2a,
0xe0b, 0xe32, 0xe25, 0xe32, 0xe21, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe04, 0xe1b,
0xe40, 0xe27, 0xe34, 0xe23, 0xe4c, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe40, 0xe04, 0xe1b, 0xe40, 0xe27, 0xe34, 0xe23,
0xe4c, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe40, 0xe04, 0xe1b, 0xe40, 0xe27, 0xe34, 0xe23, 0xe4c, 0xe14, 0xe40, 0xe27,
0xe25, 0xe32, 0xe40, 0xe04, 0xe0b, 0xe35, 0xe22, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe0a, 0xe32, 0xe21, 0xe2d, 0xe23, 0xe4c, 0xe42, 0xe23, 0xe40, 0xe27, 0xe25, 0xe32,
0xe41, 0xe0a, 0xe17, 0xe31, 0xe21, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe0a, 0xe17, 0xe31, 0xe21, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe41, 0xe0a, 0xe17, 0xe31, 0xe21,
0xe40, 0xe27, 0xe25, 0xe32, 0xe0a, 0xe34, 0xe25, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32,
0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe0a, 0xe34, 0xe25, 0xe35, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe0a, 0xe34,
0xe25, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe08, 0xe35, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe08, 0xe35, 0xe19, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe08, 0xe35, 0xe19,
0xe40, 0xe27, 0xe25, 0xe32, 0xe0a, 0xe2d, 0xe22, 0xe1b, 0xe32, 0xe25, 0xe0a, 0xe32,
0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe0a, 0xe2d, 0xe22, 0xe1b, 0xe32, 0xe25, 0xe0a, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe0a, 0xe2d, 0xe22, 0xe1b,
0xe32, 0xe25, 0xe0a, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe01, 0xe32,
0xe30, 0xe04, 0xe23, 0xe34, 0xe2a, 0xe15, 0xe4c, 0xe21, 0xe32, 0xe2a, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe42, 0xe04,
0xe42, 0xe04, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe42, 0xe04, 0xe25, 0xe2d, 0xe21,
0xe40, 0xe1a, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe42, 0xe04, 0xe25, 0xe2d, 0xe21, 0xe40, 0xe1a, 0xe35, 0xe22,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe42,
0xe04, 0xe25, 0xe2d, 0xe21, 0xe40, 0xe1a, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32,
0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe04, 0xe38, 0xe01, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2b, 0xe21,
0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe04, 0xe38, 0xe01, 0xe40, 0xe27, 0xe25,
0xe32, 0xe04, 0xe23, 0xe36, 0xe48, 0xe07, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe04, 0xe38, 0xe01,
0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe34, 0xe27, 0xe1a, 0xe32, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe04, 0xe34, 0xe27, 0xe1a,
0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe02,
0xe2d, 0xe07, 0xe04, 0xe34, 0xe27, 0xe1a, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40,
0xe14, 0xe27, 0xe34, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe14, 0xe39, 0xe21, 0xe2d,
0xe07, 0xe15, 0xe4c, 0xe14, 0xe39, 0xe23, 0xe4c, 0xe27, 0xe34, 0xe25, 0xe25, 0xe4c,
0xe40, 0xe27, 0xe25, 0xe32, 0xe15, 0xe34, 0xe21, 0xe2d, 0xe23, 0xe4c, 0xe15, 0xe30,
0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe01,
0xe32, 0xe30, 0xe2d, 0xe35, 0xe2a, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe40, 0xe01, 0xe32,
0xe30, 0xe2d, 0xe35, 0xe2a, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe40, 0xe01, 0xe32, 0xe30,
0xe2d, 0xe35, 0xe2a, 0xe40, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe40, 0xe2d, 0xe01, 0xe27, 0xe32, 0xe14, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe01, 0xe25, 0xe32, 0xe07, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe22, 0xe38, 0xe42,
0xe23, 0xe1b, 0xe01, 0xe25, 0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe01, 0xe25,
0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe15,
0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe15,
0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe15,
0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe22,
0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01,
0xe44, 0xe01, 0xe25, 0xe40, 0xe27, 0xe25, 0xe32, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b,
0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe15,
0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe22, 0xe38, 0xe42, 0xe23, 0xe1b, 0xe15, 0xe30,
0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39,
0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe1f, 0xe2d, 0xe25, 0xe4c, 0xe01, 0xe41, 0xe25,
0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe1f, 0xe2d,
0xe25, 0xe4c, 0xe01, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40,
0xe01, 0xe32, 0xe30, 0xe1f, 0xe2d, 0xe25, 0xe4c, 0xe01, 0xe41, 0xe25, 0xe19, 0xe14,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1f, 0xe34, 0xe08, 0xe34, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe1f, 0xe34, 0xe08, 0xe34,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe1f,
0xe34, 0xe08, 0xe34, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe1f, 0xe23, 0xe19, 0xe0a,
0xe4c, 0xe40, 0xe01, 0xe35, 0xe22, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40,
0xe1f, 0xe23, 0xe19, 0xe0a, 0xe4c, 0xe40, 0xe0b, 0xe32, 0xe40, 0xe17, 0xe34, 0xe23,
0xe4c, 0xe19, 0xe41, 0xe25, 0xe30, 0xe41, 0xe2d, 0xe19, 0xe15, 0xe32, 0xe23, 0xe4c,
0xe01, 0xe15, 0xe34, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe01, 0xe32, 0xe25, 0xe32,
0xe1b, 0xe32, 0xe42, 0xe01, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe01, 0xe21,
0xe40, 0xe1a, 0xe35, 0xe22, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe08, 0xe2d,
0xe23, 0xe4c, 0xe40, 0xe08, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe08, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe35,
0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19,
0xe08, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe08, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32,
0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe01, 0xe34, 0xe25, 0xe40,
0xe1a, 0xe34, 0xe23, 0xe4c, 0xe15, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe01, 0xe23, 0xe35, 0xe19, 0xe34, 0xe0a, 0xe40, 0xe27,
0xe25, 0xe32, 0xe01, 0xe23, 0xe35, 0xe19, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe15,
0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe01, 0xe23, 0xe35, 0xe19, 0xe41, 0xe25,
0xe19, 0xe14, 0xe4c, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe01, 0xe23,
0xe35, 0xe19, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe01, 0xe23, 0xe35, 0xe19, 0xe41,
0xe25, 0xe19, 0xe14, 0xe4c, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe01, 0xe23,
0xe35, 0xe19, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19,
0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe01, 0xe23, 0xe35, 0xe19, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe15, 0xe30,
0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe01, 0xe27, 0xe21,
0xe40, 0xe27, 0xe25, 0xe32, 0xe01, 0xe31, 0xe25, 0xe1f, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe01, 0xe32, 0xe22, 0xe2d, 0xe32, 0xe19, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32,
0xe2e, 0xe32, 0xe27, 0xe32, 0xe22, 0x2d, 0xe2d, 0xe30, 0xe25, 0xe39, 0xe40, 0xe0a,
0xe35, 0xe22, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe2e, 0xe32, 0xe27, 0xe32, 0xe22, 0x2d, 0xe2d, 0xe30, 0xe25, 0xe39,
0xe40, 0xe0a, 0xe35, 0xe22, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21,
0xe41, 0xe2a, 0xe07, 0xe2e, 0xe32, 0xe27, 0xe32, 0xe22, 0x2d, 0xe2d, 0xe30, 0xe25,
0xe39, 0xe40, 0xe0a, 0xe35, 0xe22, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2e, 0xe48,
0xe2d, 0xe07, 0xe01, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe2e, 0xe48, 0xe2d, 0xe07, 0xe01, 0xe07, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe2e, 0xe48, 0xe2d, 0xe07,
0xe01, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2e, 0xe2d, 0xe1f, 0xe14, 0xe4c, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2e, 0xe2d,
0xe1f, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe2e, 0xe2d, 0xe1f, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d,
0xe34, 0xe19, 0xe40, 0xe14, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe2b,
0xe32, 0xe2a, 0xe21, 0xe38, 0xe17, 0xe23, 0xe2d, 0xe34, 0xe19, 0xe40, 0xe14, 0xe35,
0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe34, 0xe19, 0xe42, 0xe14, 0xe08, 0xe35,
0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe34, 0xe19, 0xe42, 0xe14, 0xe19, 0xe35,
0xe40, 0xe0b, 0xe35, 0xe22, 0xe15, 0xe2d, 0xe19, 0xe01, 0xe25, 0xe32, 0xe07, 0xe40,
0xe27, 0xe25, 0xe32, 0xe2d, 0xe34, 0xe19, 0xe42, 0xe14, 0xe19, 0xe35, 0xe40, 0xe0b,
0xe35, 0xe22, 0xe1d, 0xe31, 0xe48, 0xe07, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d,
0xe2d, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe34, 0xe19, 0xe42, 0xe14, 0xe19,
0xe35, 0xe40, 0xe0b, 0xe35, 0xe22, 0xe1d, 0xe31, 0xe48, 0xe07, 0xe15, 0xe30, 0xe27,
0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe34, 0xe2b, 0xe23,
0xe48, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe2d, 0xe34, 0xe2b, 0xe23, 0xe48, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe2d, 0xe34, 0xe2b, 0xe23, 0xe48,
0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe35, 0xe23, 0xe4c, 0xe04, 0xe38,
0xe15, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe2d, 0xe35, 0xe23, 0xe4c, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19,
0xe2d, 0xe35, 0xe23, 0xe4c, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe34, 0xe2a, 0xe23, 0xe32, 0xe40, 0xe2d, 0xe25, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe34, 0xe2a,
0xe23, 0xe32, 0xe40, 0xe2d, 0xe25, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21,
0xe41, 0xe2a, 0xe07, 0xe2d, 0xe34, 0xe2a, 0xe23, 0xe32, 0xe40, 0xe2d, 0xe25, 0xe40,
0xe27, 0xe25, 0xe32, 0xe0d, 0xe35, 0xe48, 0xe1b, 0xe38, 0xe48, 0xe19, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe0d, 0xe35, 0xe48,
0xe1b, 0xe38, 0xe48, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41,
0xe2a, 0xe07, 0xe0d, 0xe35, 0xe48, 0xe1b, 0xe38, 0xe48, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe04, 0xe31, 0xe21, 0xe0a, 0xe31, 0xe15, 0xe04, 0xe32, 0xe40, 0xe27, 0xe25,
0xe32, 0xe40, 0xe1b, 0xe42, 0xe15, 0xe23, 0xe1b, 0xe31, 0xe1f, 0xe25, 0xe2d, 0xe1f,
0xe2a, 0xe04, 0xe4c, 0x2d, 0xe04, 0xe31, 0xe21, 0xe0a, 0xe31, 0xe15, 0xe2a, 0xe01,
0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19,
0xe40, 0xe1b, 0xe42, 0xe15, 0xe23, 0xe1b, 0xe31, 0xe1f, 0xe25, 0xe2d, 0xe1f, 0xe2a,
0xe04, 0xe4c, 0x2d, 0xe04, 0xe31, 0xe21, 0xe0a, 0xe31, 0xe15, 0xe2a, 0xe01, 0xe35,
0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe32, 0xe0b, 0xe31, 0xe04, 0xe2a, 0xe16, 0xe32,
0xe19, 0xe15, 0xe30, 0xe27, 0xe31, 0xe19, 0xe2d, 0xe2d, 0xe01, 0xe40, 0xe27, 0xe25,
0xe32, 0xe04, 0xe32, 0xe0b, 0xe31, 0xe04, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe15, 0xe30,
0xe27, 0xe31, 0xe19, 0xe15, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe01, 0xe32,
0xe2b, 0xe25, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe40, 0xe01, 0xe32, 0xe2b, 0xe25, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32,
0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe40, 0xe01, 0xe32, 0xe2b, 0xe25, 0xe35,
0xe40, 0xe27, 0xe25, 0xe32, 0xe04, 0xe2d, 0xe2a, 0xe44, 0xe23, 0xe40, 0xe27, 0xe25,
0xe32, 0xe04, 0xe23, 0xe31, 0xe2a, 0xe42, 0xe19, 0xe22, 0xe32, 0xe2a, 0xe04, 0xe4c,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe04,
0xe23, 0xe31, 0xe2a, 0xe42, 0xe19, 0xe22, 0xe32, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe04, 0xe23, 0xe31,
0xe2a, 0xe42, 0xe19, 0xe22, 0xe32, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe04, 0xe35, 0xe23, 0xe4c, 0xe01, 0xe35, 0xe0b, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40,
0xe27, 0xe25, 0xe32, 0xe25, 0xe31, 0xe07, 0xe01, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32,
0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe44, 0xe25, 0xe19, 0xe4c,
0xe40, 0xe27, 0xe25, 0xe32, 0xe25, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe42, 0xe2e, 0xe27,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe25, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe42, 0xe2e, 0xe27, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe25, 0xe2d, 0xe23, 0xe4c, 0xe14,
0xe42, 0xe2e, 0xe27, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe40, 0xe01,
0xe4a, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe21, 0xe32, 0xe40, 0xe01, 0xe4a, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24,
0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe21, 0xe32, 0xe40, 0xe01, 0xe4a, 0xe32,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe01, 0xe32, 0xe14, 0xe32, 0xe19, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe21, 0xe32,
0xe01, 0xe32, 0xe14, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe21, 0xe32, 0xe01, 0xe32, 0xe14, 0xe32, 0xe19, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe40, 0xe25, 0xe40, 0xe0b, 0xe35, 0xe22, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe31, 0xe25, 0xe14, 0xe35, 0xe1f, 0xe2a, 0xe4c, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe23, 0xe4c, 0xe40, 0xe04, 0xe0b, 0xe31, 0xe2a,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30,
0xe21, 0xe32, 0xe23, 0xe4c, 0xe41, 0xe0a, 0xe25, 0xe25, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe21, 0xe2d, 0xe23, 0xe34, 0xe40, 0xe0a, 0xe35, 0xe22, 0xe2a, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe21, 0xe2d, 0xe23,
0xe34, 0xe40, 0xe0a, 0xe35, 0xe22, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe02, 0xe2d, 0xe07, 0xe21, 0xe2d, 0xe23, 0xe34,
0xe40, 0xe0a, 0xe35, 0xe22, 0xe2a, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe2d, 0xe27,
0xe4c, 0xe2a, 0xe31, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe41, 0xe1b, 0xe0b, 0xe34,
0xe1f, 0xe34, 0xe01, 0xe40, 0xe21, 0xe47, 0xe01, 0xe0b, 0xe34, 0xe42, 0xe01, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe1b,
0xe0b, 0xe34, 0xe1f, 0xe34, 0xe01, 0xe40, 0xe21, 0xe47, 0xe01, 0xe0b, 0xe34, 0xe42,
0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe41,
0xe1b, 0xe0b, 0xe34, 0xe1f, 0xe34, 0xe01, 0xe40, 0xe21, 0xe47, 0xe01, 0xe0b, 0xe34,
0xe42, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe39, 0xe25, 0xe32, 0xe19, 0xe1a,
0xe32, 0xe15, 0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15,
0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe39, 0xe25, 0xe32, 0xe19, 0xe1a, 0xe32, 0xe15,
0xe2d, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe2d, 0xe39, 0xe25, 0xe32, 0xe19, 0xe1a, 0xe32, 0xe15, 0xe2d, 0xe23,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe2d, 0xe2a, 0xe42, 0xe01, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe21, 0xe2d, 0xe2a,
0xe42, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe21, 0xe2d, 0xe2a, 0xe42, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1e, 0xe21,
0xe48, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe19, 0xe32, 0xe2d, 0xe39, 0xe23, 0xe39,
0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe19, 0xe1b, 0xe32, 0xe25, 0xe40, 0xe27, 0xe25,
0xe32, 0xe19, 0xe34, 0xe27, 0xe41, 0xe04, 0xe25, 0xe34, 0xe42, 0xe14, 0xe40, 0xe19,
0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe19, 0xe34, 0xe27, 0xe41, 0xe04, 0xe25, 0xe34, 0xe42, 0xe14, 0xe40, 0xe19,
0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe19, 0xe34, 0xe27, 0xe41, 0xe04, 0xe25, 0xe34, 0xe42, 0xe14, 0xe40, 0xe19,
0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe19, 0xe34, 0xe27, 0xe0b, 0xe35, 0xe41,
0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe19, 0xe34, 0xe27, 0xe0b, 0xe35, 0xe41, 0xe25, 0xe19, 0xe14,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe19,
0xe34, 0xe27, 0xe0b, 0xe35, 0xe41, 0xe25, 0xe19, 0xe14, 0xe4c, 0xe40, 0xe27, 0xe25,
0xe32, 0xe19, 0xe34, 0xe27, 0xe1f, 0xe31, 0xe19, 0xe14, 0xe4c, 0xe41, 0xe25, 0xe19,
0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32,
0xe19, 0xe19, 0xe34, 0xe27, 0xe1f, 0xe31, 0xe19, 0xe14, 0xe4c, 0xe41, 0xe25, 0xe19,
0xe14, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07,
0xe19, 0xe34, 0xe27, 0xe1f, 0xe31, 0xe19, 0xe14, 0xe4c, 0xe41, 0xe25, 0xe19, 0xe14,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe19, 0xe35, 0xe2d, 0xe39, 0xe40, 0xe2d, 0xe40,
0xe27, 0xe25, 0xe32, 0xe40, 0xe01, 0xe32, 0xe30, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe1f,
0xe2d, 0xe25, 0xe4c, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe40, 0xe01, 0xe32, 0xe30, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe1f,
0xe2d, 0xe25, 0xe4c, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41,
0xe2a, 0xe07, 0xe40, 0xe01, 0xe32, 0xe30, 0xe19, 0xe2d, 0xe23, 0xe4c, 0xe1f, 0xe2d,
0xe25, 0xe4c, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40,
0xe01, 0xe32, 0xe30, 0xe40, 0xe1f, 0xe2d, 0xe23, 0xe4c, 0xe19, 0xe31, 0xe19, 0xe42,
0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe40, 0xe1f, 0xe2d, 0xe23,
0xe4c, 0xe19, 0xe31, 0xe19, 0xe42, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe02, 0xe2d, 0xe07, 0xe2b, 0xe21, 0xe39, 0xe48,
0xe40, 0xe01, 0xe32, 0xe30, 0xe40, 0xe1f, 0xe2d, 0xe23, 0xe4c, 0xe19, 0xe31, 0xe19,
0xe42, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01,
0xe32, 0xe30, 0xe21, 0xe32, 0xe40, 0xe23, 0xe35, 0xe22, 0xe19, 0xe32, 0xe40, 0xe2b,
0xe19, 0xe37, 0xe2d, 0xe40, 0xe27, 0xe25, 0xe32, 0xe42, 0xe19, 0xe42, 0xe27, 0xe0b,
0xe35, 0xe1a, 0xe35, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe42, 0xe19, 0xe42, 0xe27, 0xe0b, 0xe35, 0xe1a,
0xe35, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23,
0xe49, 0xe2d, 0xe19, 0xe42, 0xe19, 0xe42, 0xe27, 0xe0b, 0xe35, 0xe1a, 0xe35, 0xe2a,
0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe2a, 0xe04, 0xe4c,
0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d,
0xe2d, 0xe21, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe2d, 0xe21, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27,
0xe25, 0xe32, 0xe1b, 0xe32, 0xe01, 0xe35, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe1b, 0xe32, 0xe01,
0xe35, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe1b, 0xe32, 0xe01, 0xe35, 0xe2a, 0xe16, 0xe32, 0xe19,
0xe40, 0xe27, 0xe25, 0xe32, 0xe1b, 0xe32, 0xe40, 0xe25, 0xe32, 0xe40, 0xe27, 0xe25,
0xe32, 0xe1b, 0xe32, 0xe1b, 0xe31, 0xe27, 0xe19, 0xe34, 0xe27, 0xe01, 0xe34, 0xe19,
0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1b, 0xe32, 0xe23, 0xe32, 0xe01, 0xe27, 0xe31,
0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe1b, 0xe32, 0xe23, 0xe32, 0xe01, 0xe27, 0xe31, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe1b, 0xe32, 0xe23, 0xe32, 0xe01,
0xe27, 0xe31, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe1b, 0xe23, 0xe39, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe40, 0xe1b,
0xe23, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d,
0xe19, 0xe40, 0xe1b, 0xe23, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe1f, 0xe34, 0xe25,
0xe34, 0xe1b, 0xe1b, 0xe34, 0xe19, 0xe2a, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe1f, 0xe34, 0xe25, 0xe34, 0xe1b, 0xe1b,
0xe34, 0xe19, 0xe2a, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23,
0xe49, 0xe2d, 0xe19, 0xe1f, 0xe34, 0xe25, 0xe34, 0xe1b, 0xe1b, 0xe34, 0xe19, 0xe2a,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21, 0xe39, 0xe48, 0xe40, 0xe01, 0xe32,
0xe30, 0xe1f, 0xe34, 0xe19, 0xe34, 0xe01, 0xe0b, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32,
0xe41, 0xe0b, 0xe07, 0xe1b, 0xe35, 0xe41, 0xe22, 0xe23, 0xe4c, 0xe41, 0xe25, 0xe30,
0xe21, 0xe35, 0xe40, 0xe01, 0xe2d, 0xe25, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe41, 0xe0b, 0xe07, 0xe1b, 0xe35, 0xe41,
0xe22, 0xe23, 0xe4c, 0xe41, 0xe25, 0xe30, 0xe21, 0xe35, 0xe40, 0xe01, 0xe2d, 0xe25,
0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe02,
0xe2d, 0xe07, 0xe41, 0xe0b, 0xe07, 0xe1b, 0xe35, 0xe41, 0xe22, 0xe23, 0xe4c, 0xe41,
0xe25, 0xe30, 0xe21, 0xe35, 0xe40, 0xe01, 0xe2d, 0xe25, 0xe07, 0xe40, 0xe27, 0xe25,
0xe32, 0xe1e, 0xe34, 0xe15, 0xe41, 0xe04, 0xe23, 0xe4c, 0xe19, 0xe40, 0xe27, 0xe25,
0xe32, 0xe42, 0xe1b, 0xe19, 0xe32, 0xe40, 0xe1b, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40,
0xe1b, 0xe35, 0xe22, 0xe07, 0xe22, 0xe32, 0xe07, 0xe40, 0xe27, 0xe25, 0xe32, 0xe04,
0xe37, 0xe22, 0xe0b, 0xe34, 0xe25, 0xe2d, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe32, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe04, 0xe37,
0xe22, 0xe0b, 0xe34, 0xe25, 0xe2d, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe32, 0xe40, 0xe27,
0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe04, 0xe37, 0xe22,
0xe0b, 0xe34, 0xe25, 0xe2d, 0xe2d, 0xe23, 0xe4c, 0xe14, 0xe32, 0xe40, 0xe27, 0xe25,
0xe32, 0xe40, 0xe23, 0xe2d, 0xe39, 0xe19, 0xe35, 0xe22, 0xe07, 0xe40, 0xe27, 0xe25,
0xe32, 0xe42, 0xe23, 0xe18, 0xe35, 0xe23, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0b,
0xe32, 0xe04, 0xe32, 0xe25, 0xe34, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe0b, 0xe32, 0xe04, 0xe32, 0xe25, 0xe34, 0xe19,
0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe0b,
0xe32, 0xe04, 0xe32, 0xe25, 0xe34, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0b, 0xe32,
0xe21, 0xe32, 0xe23, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23,
0xe10, 0xe32, 0xe19, 0xe0b, 0xe32, 0xe21, 0xe32, 0xe23, 0xe32, 0xe40, 0xe27, 0xe25,
0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe0b, 0xe32, 0xe21, 0xe32,
0xe23, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0b, 0xe32, 0xe21, 0xe31, 0xe27, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe0b, 0xe32,
0xe21, 0xe31, 0xe27, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49,
0xe2d, 0xe19, 0xe0b, 0xe32, 0xe21, 0xe31, 0xe27, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40,
0xe0b, 0xe40, 0xe0a, 0xe25, 0xe2a, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2a, 0xe34,
0xe07, 0xe04, 0xe42, 0xe1b, 0xe23, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2b, 0xe21,
0xe39, 0xe48, 0xe40, 0xe01, 0xe32, 0xe30, 0xe42, 0xe0b, 0xe42, 0xe25, 0xe21, 0xe2d,
0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe0b, 0xe32, 0xe17, 0xe4c, 0xe08, 0xe2d,
0xe23, 0xe4c, 0xe40, 0xe08, 0xe35, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe0b, 0xe39,
0xe23, 0xe34, 0xe19, 0xe32, 0xe40, 0xe21, 0xe40, 0xe27, 0xe25, 0xe32, 0xe44, 0xe0b,
0xe42, 0xe22, 0xe27, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe15, 0xe32, 0xe2e, 0xe35,
0xe15, 0xe35, 0xe40, 0xe27, 0xe25, 0xe32, 0xe44, 0xe17, 0xe40, 0xe1b, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe44, 0xe17, 0xe40,
0xe1b, 0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe2d, 0xe21, 0xe41, 0xe2a, 0xe07, 0xe44,
0xe17, 0xe40, 0xe1b, 0xe40, 0xe27, 0xe25, 0xe32, 0xe17, 0xe32, 0xe08, 0xe34, 0xe01,
0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe42, 0xe15, 0xe40,
0xe01, 0xe40, 0xe25, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe15, 0xe2d, 0xe07, 0xe01,
0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19,
0xe15, 0xe2d, 0xe07, 0xe01, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe15, 0xe2d, 0xe07, 0xe01, 0xe32, 0xe40, 0xe27, 0xe25,
0xe32, 0xe0a, 0xe38, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe15, 0xe34, 0xe23,
0xe4c, 0xe01, 0xe40, 0xe21, 0xe19, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe40, 0xe15, 0xe34,
0xe23, 0xe4c, 0xe01, 0xe40, 0xe21, 0xe19, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe40, 0xe15,
0xe34, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe21, 0xe19, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19,
0xe40, 0xe27, 0xe25, 0xe32, 0xe15, 0xe39, 0xe27, 0xe32, 0xe25, 0xe39, 0xe40, 0xe27,
0xe25, 0xe32, 0xe2d, 0xe38, 0xe23, 0xe38, 0xe01, 0xe27, 0xe31, 0xe22, 0xe40, 0xe27,
0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe2d, 0xe38, 0xe23,
0xe38, 0xe01, 0xe27, 0xe31, 0xe22, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39,
0xe23, 0xe49, 0xe2d, 0xe19, 0xe2d, 0xe38, 0xe23, 0xe38, 0xe01, 0xe27, 0xe31, 0xe22,
0xe40, 0xe27, 0xe25, 0xe32, 0xe2d, 0xe38, 0xe0b, 0xe40, 0xe1a, 0xe01, 0xe34, 0xe2a,
0xe16, 0xe32, 0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe2d, 0xe38, 0xe0b, 0xe40, 0xe1a, 0xe01, 0xe34, 0xe2a, 0xe16, 0xe32,
0xe19, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19,
0xe2d, 0xe38, 0xe0b, 0xe40, 0xe1a, 0xe01, 0xe34, 0xe2a, 0xe16, 0xe32, 0xe19, 0xe40,
0xe27, 0xe25, 0xe32, 0xe27, 0xe32, 0xe19, 0xe39, 0xe2d, 0xe32, 0xe15, 0xe39, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe27, 0xe32,
0xe19, 0xe39, 0xe2d, 0xe32, 0xe15, 0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe27, 0xe32, 0xe19, 0xe39, 0xe2d, 0xe32, 0xe15,
0xe39, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe27, 0xe40, 0xe19, 0xe0b, 0xe38, 0xe40,
0xe2d, 0xe25, 0xe32, 0xe40, 0xe27, 0xe25, 0xe32, 0xe27, 0xe25, 0xe32, 0xe14, 0xe35,
0xe27, 0xe2d, 0xe2a, 0xe15, 0xe2d, 0xe04, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32,
0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe27, 0xe25, 0xe32, 0xe14, 0xe35, 0xe27, 0xe2d,
0xe2a, 0xe15, 0xe2d, 0xe04, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23,
0xe49, 0xe2d, 0xe19, 0xe27, 0xe25, 0xe32, 0xe14, 0xe35, 0xe27, 0xe2d, 0xe2a, 0xe15,
0xe2d, 0xe04, 0xe40, 0xe27, 0xe25, 0xe32, 0xe27, 0xe2d, 0xe25, 0xe42, 0xe01, 0xe01,
0xe23, 0xe32, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10,
0xe32, 0xe19, 0xe27, 0xe2d, 0xe25, 0xe42, 0xe01, 0xe01, 0xe23, 0xe32, 0xe14, 0xe40,
0xe27, 0xe25, 0xe32, 0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe27, 0xe2d,
0xe25, 0xe42, 0xe01, 0xe01, 0xe23, 0xe32, 0xe14, 0xe40, 0xe27, 0xe25, 0xe32, 0xe27,
0xe2d, 0xe2a, 0xe15, 0xe2d, 0xe04, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe01, 0xe32,
0xe30, 0xe40, 0xe27, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe27, 0xe32, 0xe25, 0xe25,
0xe34, 0xe2a, 0xe41, 0xe25, 0xe30, 0xe1f, 0xe38, 0xe15, 0xe39, 0xe19, 0xe32, 0xe40,
0xe27, 0xe25, 0xe32, 0xe22, 0xe32, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe40,
0xe27, 0xe25, 0xe32, 0xe21, 0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe22, 0xe32,
0xe04, 0xe38, 0xe15, 0xe2a, 0xe04, 0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe24, 0xe14,
0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe22, 0xe32, 0xe04, 0xe38, 0xe15, 0xe2a, 0xe04,
0xe4c, 0xe40, 0xe27, 0xe25, 0xe32, 0xe40, 0xe22, 0xe04, 0xe32, 0xe40, 0xe15, 0xe23,
0xe34, 0xe19, 0xe1a, 0xe39, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32, 0xe21,
0xe32, 0xe15, 0xe23, 0xe10, 0xe32, 0xe19, 0xe40, 0xe22, 0xe04, 0xe32, 0xe40, 0xe15,
0xe23, 0xe34, 0xe19, 0xe1a, 0xe39, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe27, 0xe25, 0xe32,
0xe24, 0xe14, 0xe39, 0xe23, 0xe49, 0xe2d, 0xe19, 0xe40, 0xe22, 0xe04, 0xe32, 0xe40,
0xe15, 0xe23, 0xe34, 0xe19, 0xe1a, 0xe39, 0xe23, 0xe4c, 0xe01, 0xe40, 0xe27, 0xe25,
0xe32, 0xe22, 0xe39, 0xe04, 0xe2d, 0xe19, 0x130d, 0x12dc, 0x20, 0x12a3, 0x12ad, 0x122a,
0x121d, 0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x12a3, 0x12ad, 0x122a, 0x130d, 0x12dc,
0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a3, 0x12ad, 0x122a, 0x130d, 0x12dc, 0x20,
0x121b, 0x12a5, 0x12a8, 0x120b, 0x12ed, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243, 0x130d, 0x12dc,
0x20, 0x121d, 0x1265, 0x122b, 0x1255, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243, 0x130d, 0x12dc,
0x20, 0x12f0, 0x1261, 0x1265, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243, 0x130d, 0x12dc, 0x20,
0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243, 0x121d, 0x12f1, 0x1265,
0x20, 0x130d, 0x12dc, 0x20, 0x121d, 0x12d5, 0x122b, 0x1265, 0x20, 0x12a3, 0x134d, 0x122a,
0x1243, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x121d, 0x12d5, 0x122b,
0x1265, 0x20, 0x12a3, 0x134d, 0x122a, 0x1243, 0x130d, 0x12dc, 0x20, 0x12a3, 0x120b, 0x1235,
0x12ab, 0x121d, 0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x12a3, 0x120b, 0x1235, 0x12ab,
0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a3, 0x120b, 0x1235, 0x12ab,
0x130d, 0x12dc, 0x20, 0x12a3, 0x121b, 0x12de, 0x1295, 0x121d, 0x12f1, 0x1265, 0x20, 0x130d,
0x12dc, 0x20, 0x12a3, 0x121b, 0x12de, 0x1295, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d,
0x1272, 0x20, 0x12a3, 0x121b, 0x12de, 0x1295, 0x130d, 0x12dc, 0x20, 0x12a3, 0x122d, 0x1300,
0x1295, 0x1272, 0x1293, 0x121d, 0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x12a3, 0x122d,
0x1300, 0x1295, 0x1272, 0x1293, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20,
0x12a3, 0x122d, 0x1300, 0x1295, 0x1272, 0x1293, 0x130d, 0x12dc, 0x20, 0x12a3, 0x12de, 0x1228,
0x1235, 0x121d, 0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x12a3, 0x12de, 0x1228, 0x1235,
0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a3, 0x12de, 0x1228, 0x1235,
0x130d, 0x12dc, 0x20, 0x1266, 0x120a, 0x126d, 0x12eb, 0x130d, 0x12dc, 0x20, 0x1265, 0x122b,
0x12da, 0x120d, 0x12eb, 0x121d, 0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x1265, 0x122b,
0x12da, 0x120d, 0x12eb, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1265,
0x122b, 0x12da, 0x120d, 0x12eb, 0x130d, 0x12dc, 0x20, 0x12ac, 0x1355, 0x20, 0x1268, 0x122d,
0x12f0, 0x121d, 0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x12ac, 0x1355, 0x20, 0x1268,
0x122d, 0x12f0, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12ac, 0x1355,
0x20, 0x1268, 0x122d, 0x12f0, 0x130d, 0x12dc, 0x20, 0x127a, 0x120c, 0x121d, 0x12f1, 0x1265,
0x20, 0x130d, 0x12dc, 0x20, 0x127a, 0x120c, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d,
0x1272, 0x20, 0x127a, 0x120c, 0x130d, 0x12dc, 0x20, 0x12ae, 0x120e, 0x121d, 0x1265, 0x12eb,
0x121d, 0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x12ae, 0x120e, 0x121d, 0x1265, 0x12eb,
0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12ae, 0x120e, 0x121d, 0x1265,
0x12eb, 0x130d, 0x12dc, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x134b, 0x1232, 0x12ab, 0x121d,
0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x12f0, 0x1234, 0x1275, 0x20, 0x134b, 0x1232,
0x12ab, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12f0, 0x1234, 0x1275,
0x20, 0x134b, 0x1232, 0x12ab, 0x130d, 0x12dc, 0x20, 0x12a4, 0x12b3, 0x12f6, 0x122d, 0x130d,
0x12dc, 0x20, 0x121b, 0x12a5, 0x12a8, 0x120b, 0x12ed, 0x20, 0x12a4, 0x12cd, 0x122e, 0x1333,
0x121d, 0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x121b, 0x12a5, 0x12a8, 0x120b, 0x12ed,
0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272,
0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x130d, 0x12dc, 0x20, 0x121d, 0x1265, 0x122b, 0x1255,
0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x121d, 0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20,
0x121d, 0x1265, 0x122b, 0x1255, 0x20, 0x12a4, 0x12cd, 0x122e, 0x1333, 0x130d, 0x12dc, 0x20,
0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x121d, 0x1265, 0x122b, 0x1255, 0x20, 0x12a4, 0x12cd,
0x122e, 0x1333, 0x130d, 0x12dc, 0x20, 0x12f0, 0x1234, 0x1273, 0x1275, 0x20, 0x134e, 0x12ad,
0x120b, 0x1295, 0x12f5, 0x121d, 0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x12f0, 0x1234,
0x1273, 0x1275, 0x20, 0x134e, 0x12ad, 0x120b, 0x1295, 0x12f5, 0x130d, 0x12dc, 0x20, 0x12a8,
0x1228, 0x121d, 0x1272, 0x20, 0x12f0, 0x1234, 0x1273, 0x1275, 0x20, 0x134e, 0x12ad, 0x120b,
0x1295, 0x12f5, 0x130d, 0x12dc, 0x20, 0x1348, 0x1228, 0x1295, 0x1233, 0x12ca, 0x1275, 0x20,
0x130a, 0x12eb, 0x1293, 0x130d, 0x12dc, 0x20, 0x1348, 0x1228, 0x1295, 0x1233, 0x12ca, 0x20,
0x12f0, 0x1261, 0x1263, 0x12ca, 0x20, 0x130d, 0x12dd, 0x12a3, 0x1273, 0x1275, 0x1295, 0x20,
0x12a3, 0x1295, 0x1273, 0x122d, 0x1272, 0x12ad, 0x1295, 0x130d, 0x12dc, 0x20, 0x130b, 0x120b,
0x1353, 0x130e, 0x1235, 0x130d, 0x12dc, 0x20, 0x1309, 0x12eb, 0x1293, 0x130d, 0x12dc, 0x20,
0x1205, 0x1295, 0x12f3, 0x12ca, 0x20, 0x12cd, 0x1245, 0x12eb, 0x1296, 0x1235, 0x130d, 0x12dc,
0x20, 0x121b, 0x12a5, 0x12a8, 0x120b, 0x12ed, 0x20, 0x12a2, 0x1295, 0x12f6, 0x1290, 0x12e5,
0x12eb, 0x130d, 0x12dc, 0x20, 0x121d, 0x1265, 0x122b, 0x1253, 0x12ca, 0x20, 0x12a2, 0x1295,
0x12f6, 0x1290, 0x12e5, 0x12eb, 0x130d, 0x12dc, 0x20, 0x121d, 0x12d5, 0x122b, 0x1263, 0x12ca,
0x20, 0x12a2, 0x1295, 0x12f6, 0x1290, 0x12e5, 0x12eb, 0x130d, 0x12dc, 0x20, 0x121b, 0x1208,
0x12e5, 0x12eb, 0x130d, 0x12dc, 0x20, 0x121b, 0x12cd, 0x122a, 0x1238, 0x1235, 0x121d, 0x12f1,
0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x121b, 0x12cd, 0x122a, 0x1238, 0x1235, 0x130d, 0x12dc,
0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x121b, 0x12cd, 0x122a, 0x1238, 0x1235, 0x130d,
0x12dc, 0x20, 0x1348, 0x122d, 0x1293, 0x1295, 0x12f6, 0x20, 0x12f0, 0x20, 0x1296, 0x122e,
0x1295, 0x12eb, 0x121d, 0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x1348, 0x122d, 0x1293,
0x1295, 0x12f6, 0x20, 0x12f0, 0x20, 0x1296, 0x122e, 0x1295, 0x12eb, 0x130d, 0x12dc, 0x20,
0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1348, 0x122d, 0x1293, 0x1295, 0x12f6, 0x20, 0x12f0,
0x20, 0x1296, 0x122e, 0x1295, 0x12eb, 0x130d, 0x12dc, 0x20, 0x1353, 0x122b, 0x1313, 0x12ed,
0x121d, 0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x1353, 0x122b, 0x1313, 0x12ed, 0x130d,
0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1353, 0x122b, 0x1313, 0x12ed, 0x130d,
0x12dc, 0x20, 0x1354, 0x1229, 0x121d, 0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x1354,
0x1229, 0x130d, 0x12dc, 0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x1354, 0x1229, 0x130d,
0x12dc, 0x20, 0x122d, 0x12e9, 0x1295, 0x12e8, 0x1295, 0x130d, 0x12dc, 0x20, 0x1232, 0x1238,
0x120d, 0x1235, 0x130d, 0x12dc, 0x20, 0x1232, 0x1295, 0x130b, 0x1356, 0x122d, 0x130d, 0x12dc,
0x20, 0x12f0, 0x1261, 0x1265, 0x20, 0x1306, 0x122d, 0x1305, 0x12eb, 0x130d, 0x12dc, 0x20,
0x1231, 0x122a, 0x1293, 0x121d, 0x130d, 0x12dc, 0x20, 0x12a1, 0x122b, 0x1313, 0x12ed, 0x121d,
0x12f1, 0x1265, 0x20, 0x130d, 0x12dc, 0x20, 0x12a1, 0x122b, 0x1313, 0x12ed, 0x130d, 0x12dc,
0x20, 0x12ad, 0x1228, 0x121d, 0x1272, 0x20, 0x12a1, 0x122b, 0x1313, 0x12ed, 0x130d, 0x12dc,
0x20, 0x126c, 0x1294, 0x12dd, 0x12cc, 0x120b, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x65, 0x6c, 0x12b, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x65, 0x6c, 0x12b,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6b, 0x65, 0x6c, 0x12b, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x66, 0x69, 0x6b, 0x101, 0x6e, 0x69, 0x73, 0x69, 0x74,
0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x61, 0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x6c, 0x6f, 0x74,
0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b,
0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6e, 0x67, 0x61,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x66,
0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x66,
0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x66, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66,
0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6c, 0x61, 0x73, 0x69, 0x6b, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x6b, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c,
0x61, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x61, 0x6c, 0x61, 0x6d, 0x61, 0x74, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c, 0x61, 0x6d,
0x61, 0x74, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6c, 0x61, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x61, 0x73, 0x14d,
0x6e, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x6d, 0x61, 0x73, 0x14d, 0x6e, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x61, 0x73, 0x14d, 0x6e,
0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x20, 0x6c, 0x6f, 0x74, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61,
0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6c, 0x6f, 0x74,
0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65,
0x6c, 0x61, 0x75, 0x20, 0x6c, 0x6f, 0x74, 0x6f, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b,
0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x68, 0x61,
0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x20, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69,
0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6d,
0x6f, 0x2bb, 0x75, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d,
0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6d, 0x6f, 0x2bb, 0x75,
0x6e, 0x67, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f,
0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x6d, 0x6f, 0x2bb, 0x75, 0x6e, 0x67,
0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65,
0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c,
0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20,
0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb,
0x61, 0x6e, 0x61, 0x74, 0x69, 0x6c, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x61,
0x6e, 0x61, 0x74, 0x69, 0x6c, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x61,
0x6e, 0x61, 0x74, 0x69, 0x6c, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x61, 0x70, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x61, 0x70, 0x69, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x61, 0x70, 0x69, 0x61, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x69, 0x74,
0x61, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x6b, 0x69, 0x74, 0x61, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x69, 0x74, 0x61, 0x75, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x69,
0x74, 0x14d, 0x70, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6b, 0x69, 0x74, 0x14d, 0x70, 0x65, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6b, 0x69, 0x74,
0x14d, 0x70, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x61, 0x6c, 0x65, 0x70, 0x65, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c, 0x65, 0x70, 0x65, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6c,
0x65, 0x70, 0x65, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69, 0x6e, 0x61, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x65,
0x6e, 0x69, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69, 0x6e, 0x61, 0x2d, 0x68, 0x69,
0x68, 0x69, 0x66, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69, 0x6e, 0x61, 0x2d,
0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x65, 0x6e, 0x69, 0x74, 0x69,
0x6e, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x101,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d, 0x65, 0x6c,
0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20,
0x2bb, 0x61, 0x74, 0x61, 0x6c, 0x61, 0x6e, 0x69, 0x74, 0x69, 0x6b, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6d,
0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61,
0x75, 0x20, 0x2bb, 0x61, 0x74, 0x61, 0x6c, 0x61, 0x6e, 0x69, 0x74, 0x69,
0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x61, 0x6d, 0x65, 0x6c, 0x69, 0x6b, 0x61, 0x2d, 0x74, 0x6f, 0x6b,
0x65, 0x6c, 0x61, 0x75, 0x20, 0x2bb, 0x61, 0x74, 0x61, 0x6c, 0x61, 0x6e,
0x69, 0x74, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c,
0x69, 0x61, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c,
0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69,
0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f, 0x74, 0x6f,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f,
0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f,
0x74, 0x6f, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74,
0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x2d,
0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c,
0x113, 0x6c, 0x69, 0x61, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x2d, 0x68, 0x69,
0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69,
0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65,
0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61,
0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x68,
0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c,
0x69, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6f, 0x73, 0x69, 0x74,
0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66,
0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x6f, 0x73, 0x69, 0x74, 0x65, 0x6c, 0x113, 0x6c, 0x69, 0x61, 0x2d,
0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x61, 0x70, 0x61, 0x69, 0x73, 0x61,
0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x61, 0x73, 0x61, 0x70, 0x61, 0x69, 0x73, 0x61, 0x6e, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x73, 0x61,
0x70, 0x61, 0x69, 0x73, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x73, 0x6f, 0x6c, 0x65, 0x73, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x73,
0x6f, 0x6c, 0x65, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x101, 0x73, 0x6f, 0x6c, 0x65, 0x73, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x101, 0x6e, 0x67,
0x69, 0x6c, 0x101, 0x74, 0x65, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x70, 0x101, 0x6e, 0x67, 0x69, 0x6c, 0x101, 0x74,
0x65, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x70, 0x101, 0x6e, 0x67, 0x69, 0x6c, 0x101, 0x74, 0x65, 0x73, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x16b, 0x74,
0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x70, 0x6f, 0x6c, 0x12b, 0x76, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c, 0x101, 0x73, 0x69, 0x6c, 0x61,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c,
0x101, 0x73, 0x69, 0x6c, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c, 0x101, 0x73, 0x69, 0x6c, 0x61, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x75, 0x6c, 0x75,
0x6e, 0x65, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6d, 0x75, 0x69, 0x2bb, 0x69, 0x2d, 0x76, 0x113, 0x74, 0x65, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x75, 0x69, 0x2bb, 0x69,
0x2d, 0x76, 0x113, 0x74, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6d, 0x75, 0x69, 0x2bb, 0x69, 0x2d, 0x76, 0x113, 0x74,
0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x65,
0x65, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6b, 0x61, 0x6d, 0x6f, 0x6c, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x73, 0x61, 0x74, 0x69, 0x68, 0x61, 0x6d, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x61, 0x74, 0x69,
0x68, 0x61, 0x6d, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x73, 0x61, 0x74, 0x69, 0x68, 0x61, 0x6d, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73,
0x69, 0x61, 0x69, 0x6e, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x73, 0x69, 0x61, 0x69, 0x6e, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x61, 0x69, 0x6e, 0x61,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x6f, 0x69,
0x70, 0x61, 0x6c, 0x69, 0x73, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x6f, 0x69, 0x70, 0x61, 0x6c, 0x69,
0x73, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6b, 0x6f, 0x69, 0x70, 0x61, 0x6c, 0x69, 0x73, 0x61, 0x6e,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f,
0x74, 0x75, 0x6b, 0x69, 0x6c, 0x69, 0x73, 0x69, 0x6d, 0x61, 0x73, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x74,
0x75, 0x6b, 0x6f, 0x6b, 0x6f, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x69, 0x70, 0x69,
0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x6f,
0x6c, 0x6f, 0x6d, 0x69, 0x70, 0x69, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x69, 0x70,
0x69, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x6b, 0x75, 0x6b, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75,
0x6d, 0x6f, 0x74, 0x75, 0x6b, 0x75, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d, 0x6f,
0x74, 0x75, 0x6b, 0x75, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6b, 0x69, 0x75, 0x70, 0x61, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x75, 0x70, 0x61, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x75, 0x70,
0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x61,
0x76, 0x69, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x74, 0x16b, 0x6d, 0x6f, 0x6e, 0x69, 0x2d, 0x74, 0x16b, 0x76, 0x69,
0x6c, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74,
0x69, 0x6d, 0x6f, 0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x6e, 0x75, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x6e, 0x75, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x6e, 0x75, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x6b,
0x75, 0x65, 0x74, 0x6f, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70, 0x65, 0x2d, 0x6c, 0x6f,
0x74, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x65, 0x75, 0x6c, 0x6f, 0x70, 0x65, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75,
0x6c, 0x6f, 0x70, 0x65, 0x2d, 0x6c, 0x6f, 0x74, 0x6f, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70,
0x65, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70, 0x65,
0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70, 0x65,
0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f, 0x70, 0x65, 0x2d,
0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x2d, 0x61, 0x6e, 0x67, 0x65, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c,
0x6f, 0x70, 0x65, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f,
0x70, 0x65, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x65, 0x75, 0x6c, 0x6f,
0x70, 0x65, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d, 0x6f,
0x74, 0x75, 0x2d, 0x66, 0x6f, 0x6b, 0x75, 0x6c, 0x61, 0x6e, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75,
0x6d, 0x6f, 0x74, 0x75, 0x2d, 0x66, 0x6f, 0x6b, 0x75, 0x6c, 0x61, 0x6e,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x2d, 0x66, 0x6f, 0x6b, 0x75,
0x6c, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x66, 0x69, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x66, 0x69, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x66, 0x69, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x2d,
0x66, 0x61, 0x6b, 0x61, 0x66, 0x61, 0x6c, 0x61, 0x6e, 0x69, 0x73, 0x113,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x61, 0x6e,
0x65, 0x74, 0x101, 0x74, 0x69, 0x6b, 0x61, 0x66, 0x61, 0x6c, 0x61, 0x6e,
0x69, 0x73, 0x113, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6b, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x6b, 0x6f, 0x73, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x61, 0x6d, 0x69, 0x70,
0x69, 0x113, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73,
0x65, 0x14d, 0x73, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x73, 0x65, 0x14d, 0x73, 0x69, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x65, 0x14d, 0x73, 0x69, 0x61,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x6c,
0x69, 0x70, 0x61, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6b, 0x69, 0x6c, 0x69, 0x6e, 0x69, 0x75, 0x69, 0x73, 0x69,
0x20, 0x6d, 0x101, 0x6c, 0x69, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x6d, 0x61, 0x74, 0x61,
0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x6d, 0x61, 0x74,
0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x6d,
0x61, 0x74, 0x61, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61,
0x6d, 0x61, 0x74, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75,
0x61, 0x6d, 0x61, 0x74, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f,
0x6e, 0x75, 0x61, 0x6d, 0x61, 0x74, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69,
0x66, 0x6f, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b,
0x75, 0x61, 0x6d, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6b, 0x16b, 0x6c, 0x69, 0x66, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x68, 0x61, 0x75, 0x61, 0x69, 0x2bb, 0x69, 0x2d,
0x61, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x68, 0x61, 0x75, 0x61, 0x69, 0x2bb, 0x69, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x68, 0x61, 0x75, 0x61, 0x69, 0x2bb, 0x69, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x68, 0x6f, 0x6e, 0x67, 0x69, 0x2d, 0x6b, 0x6f, 0x6e, 0x67,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x6f,
0x6e, 0x67, 0x69, 0x2d, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x6f, 0x6e, 0x67, 0x69,
0x2d, 0x6b, 0x6f, 0x6e, 0x67, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x68, 0x6f, 0x76, 0x69, 0x74, 0x65, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x6f, 0x76, 0x69, 0x74, 0x65,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x6f,
0x76, 0x69, 0x74, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x69, 0x6e, 0x69, 0x74, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x61, 0x6e, 0x61, 0x2bb, 0x69,
0x6e, 0x69, 0x74, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x69, 0x6e, 0x69, 0x74, 0x6f, 0x73, 0x69, 0x61, 0x69,
0x6e, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x69, 0x6e, 0x69, 0x74, 0x6f, 0x6e, 0x69, 0x73, 0x69, 0x61, 0x2d, 0x6c,
0x6f, 0x74, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x2bb, 0x69, 0x6e, 0x69, 0x74, 0x6f, 0x6e, 0x69, 0x73, 0x69, 0x61, 0x2d,
0x68, 0x61, 0x68, 0x61, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x6e, 0x69, 0x74, 0x6f, 0x6e, 0x69, 0x73,
0x69, 0x61, 0x2d, 0x68, 0x69, 0x68, 0x69, 0x66, 0x6f, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x6c, 0x61, 0x61, 0x6e,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69,
0x6c, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x6c, 0x61, 0x61, 0x6e, 0x69, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61,
0x2d, 0x2bb, 0x12b, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69,
0x61, 0x2d, 0x2bb, 0x12b, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b,
0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x12b, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69,
0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x69, 0x73, 0x69, 0x6c, 0x65, 0x6c, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69, 0x73, 0x69, 0x6c, 0x65, 0x6c, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x69,
0x73, 0x69, 0x6c, 0x65, 0x6c, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x73, 0x69, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x61, 0x70, 0x61,
0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x73, 0x69, 0x61, 0x70, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x70, 0x65,
0x74, 0x65, 0x6c, 0x6f, 0x70, 0x61, 0x76, 0x69, 0x6c, 0x6f, 0x76, 0x69,
0x73, 0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x70, 0x65, 0x74, 0x65, 0x6c,
0x6f, 0x70, 0x61, 0x76, 0x69, 0x6c, 0x6f, 0x76, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c,
0x16b, 0x73, 0x69, 0x61, 0x2d, 0x70, 0x65, 0x74, 0x65, 0x6c, 0x6f, 0x70,
0x61, 0x76, 0x69, 0x6c, 0x6f, 0x76, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x61, 0x73, 0x61,
0x6b, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x2d, 0x68, 0x61, 0x68, 0x61, 0x6b,
0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x61,
0x73, 0x61, 0x6b, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x2d, 0x68, 0x69, 0x68,
0x69, 0x66, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6b, 0x14d, 0x6c, 0x65, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6b, 0x14d, 0x6c, 0x65, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x14d, 0x6c, 0x65, 0x61, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x6f, 0x73, 0x69, 0x6c,
0x61, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c,
0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6b, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x6e,
0x6f, 0x69, 0x101, 0x73, 0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6b, 0x61,
0x6c, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x69, 0x101, 0x73, 0x69, 0x6b, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b,
0x73, 0x69, 0x61, 0x2d, 0x6b, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x6e, 0x6f,
0x69, 0x101, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6b, 0x12b, 0x6b, 0x69, 0x73, 0x69, 0x74, 0x61, 0x6e,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x61,
0x6e, 0x67, 0x69, 0x6b, 0x101, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x6c, 0x61,
0x69, 0x6e, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6d, 0x6f, 0x74, 0x75, 0x2bb, 0x65, 0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75,
0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f,
0x74, 0x75, 0x2bb, 0x65, 0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x65, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x74,
0x75, 0x2bb, 0x65, 0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x65, 0x20, 0x74,
0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6b, 0x61, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6b,
0x61, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6d, 0x61, 0x6b, 0x61,
0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6d, 0x61, 0x6b, 0x61, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6d, 0x61, 0x6b, 0x61, 0x74,
0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6d, 0x61, 0x6c, 0x65, 0x69, 0x73, 0x69, 0x61, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6c, 0x61, 0x74, 0x69, 0x76,
0x69, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6d, 0x101, 0x6b, 0x65, 0x73, 0x61, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74,
0x75, 0x6d, 0x61, 0x73, 0x6f, 0x6c, 0x6f, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x75, 0x6c, 0x69, 0x74, 0x69, 0x75,
0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d,
0x61, 0x75, 0x6c, 0x69, 0x74, 0x69, 0x75, 0x73, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x75, 0x6c, 0x69,
0x74, 0x69, 0x75, 0x73, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6d, 0x61, 0x75, 0x73, 0x6f, 0x6e, 0x69, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x65, 0x6b, 0x69, 0x73, 0x69,
0x6b, 0x6f, 0x75, 0x2d, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x65, 0x6b,
0x69, 0x73, 0x69, 0x6b, 0x6f, 0x75, 0x2d, 0x70, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6d, 0x65, 0x6b, 0x69, 0x73, 0x69, 0x6b, 0x6f, 0x75, 0x2d, 0x70,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x6c, 0x101, 0x6e, 0x69, 0x70, 0x101,
0x74, 0x101, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb,
0x75, 0x6c, 0x101, 0x6e, 0x69, 0x70, 0x101, 0x74, 0x101, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x6c, 0x101, 0x6e,
0x69, 0x70, 0x101, 0x74, 0x101, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6d, 0x6f, 0x73, 0x69,
0x6b, 0x6f, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6d, 0x6f, 0x73, 0x69, 0x6b, 0x6f,
0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f,
0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c,
0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6d, 0x6f, 0x73, 0x69, 0x6b, 0x6f, 0x75,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x65, 0x6d,
0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x61,
0x75, 0x6c, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6e, 0x65, 0x70, 0x61, 0x6c, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6b, 0x61, 0x6c, 0x65, 0x74, 0x14d, 0x6e, 0x69, 0x61,
0x66, 0x6f, 0x2bb, 0x6f, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6b, 0x61, 0x6c, 0x65, 0x74, 0x14d, 0x6e, 0x69, 0x61, 0x66,
0x6f, 0x2bb, 0x6f, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6b, 0x61, 0x6c, 0x65, 0x74, 0x14d, 0x6e, 0x69, 0x61, 0x66,
0x6f, 0x2bb, 0x6f, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6e, 0x75, 0x2bb, 0x75, 0x73, 0x69, 0x6c, 0x61, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x75, 0x2bb, 0x75, 0x73, 0x69,
0x6c, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6e, 0x75, 0x2bb, 0x75, 0x73, 0x69, 0x6c, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x2bb, 0x69,
0x6c, 0x6f, 0x66, 0x6f, 0x2bb, 0x6f, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x2bb, 0x69, 0x6c,
0x6f, 0x66, 0x6f, 0x2bb, 0x6f, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x66, 0x6f, 0x6e, 0x75, 0x61, 0x2bb, 0x69, 0x6c,
0x6f, 0x66, 0x6f, 0x2bb, 0x6f, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6e, 0x69, 0x75, 0x113, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6e, 0x6f, 0x61, 0x66, 0x14d, 0x6b, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6e, 0x6f, 0x61, 0x66,
0x14d, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6e, 0x6f, 0x61, 0x66, 0x14d, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x113, 0x6e, 0x61, 0x6e, 0x69, 0x74,
0x6f, 0x2d, 0x74, 0x65, 0x2d, 0x6e, 0x6f, 0x6c, 0x14d, 0x6e, 0x69, 0x61,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x113, 0x6e,
0x61, 0x6e, 0x69, 0x74, 0x6f, 0x2d, 0x74, 0x65, 0x2d, 0x6e, 0x6f, 0x6c,
0x14d, 0x6e, 0x69, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x66, 0x113, 0x6e, 0x61, 0x6e, 0x69, 0x74, 0x6f, 0x2d, 0x74,
0x65, 0x2d, 0x6e, 0x6f, 0x6c, 0x14d, 0x6e, 0x69, 0x61, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6d, 0x61, 0x6c, 0x69, 0x61, 0x6e,
0x61, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x70, 0x12b, 0x73, 0x69, 0x6b, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69,
0x61, 0x2d, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x70, 0x12b, 0x73, 0x69,
0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x6e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x70, 0x12b, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x6f, 0x6d,
0x69, 0x73, 0x69, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x6f, 0x6d, 0x69,
0x73, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74,
0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x6f, 0x6d, 0x69,
0x73, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x70, 0x101, 0x6b, 0x69, 0x73, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x101, 0x6b, 0x69,
0x73, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x70, 0x101, 0x6b, 0x69, 0x73, 0x69, 0x74, 0x61,
0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70,
0x61, 0x6c, 0x61, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x70, 0x61, 0x70, 0x75, 0x61, 0x6e, 0x69, 0x75, 0x6b, 0x69, 0x6e,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x61,
0x6c, 0x61, 0x6b, 0x75, 0x61, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c, 0x61, 0x6b, 0x75, 0x61, 0x69, 0x20,
0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x61, 0x6c,
0x61, 0x6b, 0x75, 0x61, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x70, 0x65, 0x6c, 0x16b, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x70, 0x65, 0x6c, 0x16b, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x65, 0x6c, 0x16b, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x61,
0x69, 0x6e, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x66, 0x69, 0x6c, 0x69, 0x70, 0x61, 0x69, 0x6e, 0x65, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x66, 0x69, 0x6c, 0x69, 0x70,
0x61, 0x69, 0x6e, 0x65, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x66, 0x6f, 0x69,
0x6e, 0x69, 0x6b, 0x69, 0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x73, 0x101, 0x2d, 0x70, 0x69, 0x65, 0x61, 0x2d, 0x6d,
0x6f, 0x2d, 0x6d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x101, 0x2d, 0x70, 0x69,
0x65, 0x61, 0x2d, 0x6d, 0x6f, 0x2d, 0x6d, 0x69, 0x6b, 0x65, 0x6c, 0x6f,
0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x73, 0x101, 0x2d, 0x70, 0x69, 0x65, 0x61, 0x2d, 0x6d, 0x6f, 0x2d, 0x6d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d,
0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x70, 0x69, 0x74, 0x69, 0x6b, 0x61, 0x6e, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x70, 0x14d, 0x6e,
0x61, 0x70, 0x113, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x70, 0x69, 0x6f, 0x6e, 0x67, 0x69, 0x61, 0x6e, 0x67, 0x69, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x73, 0x69, 0x6c,
0x14d, 0x74, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6b, 0x69, 0x73, 0x69, 0x6c, 0x14d, 0x74, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6b, 0x69, 0x73, 0x69, 0x6c, 0x14d,
0x74, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c,
0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c,
0x113, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x6f, 0x74, 0x65, 0x6c, 0x61, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61,
0x2d, 0x73, 0x61, 0x6b, 0x101, 0x6c, 0x69, 0x6e, 0x65, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d,
0x73, 0x61, 0x6b, 0x101, 0x6c, 0x69, 0x6e, 0x65, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d,
0x73, 0x61, 0x6b, 0x101, 0x6c, 0x69, 0x6e, 0x65, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x73,
0x61, 0x6d, 0x61, 0x6c, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x73, 0x61, 0x6d, 0x61,
0x6c, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74,
0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x73, 0x61, 0x6d, 0x61, 0x6c, 0x61,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x61, 0x2bb,
0x61, 0x6d, 0x6f, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x68, 0x61, 0x2bb, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x68, 0x61, 0x2bb, 0x61, 0x6d, 0x6f,
0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f,
0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75, 0x2d, 0x73, 0x65, 0x69, 0x73, 0x65,
0x6c, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x6f, 0x74, 0x75, 0x6d, 0x6f, 0x74, 0x75,
0x73, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x65, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x6f, 0x73, 0x69, 0x61, 0x74,
0x6f, 0x6e, 0x67, 0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x73, 0x75, 0x6c, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x73, 0x69, 0x6f, 0x75, 0x61, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x68, 0x69,
0x73, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74,
0x61, 0x69, 0x70, 0x65, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x74, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x74, 0x61, 0x69,
0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x69, 0x70, 0x65, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x61, 0x73,
0x69, 0x6b, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x6f, 0x6e, 0x67,
0x61, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x74, 0x6f,
0x6e, 0x67, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x74, 0x16b, 0x6b, 0x65, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x74, 0x16b, 0x6b, 0x69, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x74, 0x16b, 0x6b, 0x69, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x69, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x74, 0x16b, 0x6b, 0x69, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x69, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x74, 0x16b, 0x76, 0x61, 0x6c, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x6c, 0x75, 0x6b, 0x75, 0x61,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75,
0x6c, 0x75, 0x6b, 0x75, 0x61, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x6c, 0x75, 0x6b, 0x75, 0x61, 0x69,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x73,
0x69, 0x70, 0x65, 0x6b, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x68, 0x6f, 0x75,
0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x73, 0x69, 0x70, 0x65,
0x6b, 0x69, 0x74, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x73, 0x69, 0x70, 0x65, 0x6b, 0x69,
0x74, 0x61, 0x6e, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c,
0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x68, 0x6f, 0x75, 0x61,
0x20, 0x66, 0x61, 0x6b, 0x61, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x76, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20,
0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61,
0x6b, 0x61, 0x76, 0x65, 0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69,
0x61, 0x2d, 0x76, 0x61, 0x6c, 0x61, 0x74, 0x69, 0x76, 0x6f, 0x73, 0x69,
0x74, 0x6f, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x76, 0x61, 0x6c, 0x61, 0x74,
0x69, 0x76, 0x6f, 0x73, 0x69, 0x74, 0x6f, 0x6b, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61,
0x2d, 0x76, 0x61, 0x6c, 0x61, 0x74, 0x69, 0x76, 0x6f, 0x73, 0x69, 0x74,
0x6f, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69,
0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61,
0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x76, 0x6f, 0x6c, 0x69, 0x6b, 0x6f,
0x6b, 0x61, 0x6c, 0x61, 0x74, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x76, 0x6f, 0x6c,
0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x6c, 0x61, 0x74, 0x69, 0x20, 0x74, 0x61,
0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f,
0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61,
0x2d, 0x76, 0x6f, 0x6c, 0x69, 0x6b, 0x6f, 0x6b, 0x61, 0x6c, 0x61, 0x74,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x76, 0x6f,
0x73, 0x69, 0x74, 0x6f, 0x6b, 0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6d, 0x6f, 0x74, 0x75, 0x75, 0x65, 0x6b, 0x65, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x2bb, 0x75, 0x76, 0x65,
0x61, 0x20, 0x6d, 0x6f, 0x20, 0x66, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x68,
0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69,
0x61, 0x2d, 0x2bb, 0x69, 0x101, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b,
0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x69, 0x101, 0x6b, 0x75, 0x74, 0x69, 0x73,
0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x74, 0x6f,
0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b,
0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x69, 0x101, 0x6b, 0x75,
0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x6c, 0x69, 0x6c, 0x69, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66,
0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x69, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x6c, 0x69, 0x6e, 0x65, 0x70, 0x16b, 0x6b, 0x69,
0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73,
0x69, 0x61, 0x2d, 0x2bb, 0x69, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x6c, 0x69,
0x6e, 0x65, 0x70, 0x16b, 0x6b, 0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69,
0x20, 0x74, 0x6f, 0x74, 0x6f, 0x6e, 0x75, 0x68, 0x6f, 0x75, 0x61, 0x20,
0x66, 0x61, 0x6b, 0x61, 0x6c, 0x16b, 0x73, 0x69, 0x61, 0x2d, 0x2bb, 0x69,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x6c, 0x69, 0x6e, 0x65, 0x70, 0x16b, 0x6b,
0x69, 0x20, 0x74, 0x61, 0x69, 0x6d, 0x69, 0x20, 0x6c, 0x69, 0x6c, 0x69,
0x75, 0x68, 0x6f, 0x75, 0x61, 0x20, 0x66, 0x61, 0x6b, 0x61, 0x69, 0x75,
0x6b, 0x6f, 0x6e, 0x69, 0x41, 0x63, 0x72, 0x65, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x41, 0x63, 0x72, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x63, 0x72,
0x65, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41,
0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x47, 0xfc, 0x6e, 0x65, 0x79, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74,
0x131, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42,
0x61, 0x74, 0x131, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6c, 0x6d,
0x61, 0x74, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x131,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6d,
0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b,
0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a,
0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44,
0x6f, 0x11f, 0x75, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a,
0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44,
0x6f, 0x11f, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44, 0x6f, 0x11f, 0x75,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75,
0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x44, 0x61, 0x11f, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a,
0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44,
0x61, 0x11f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65, 0x79, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x44, 0x61, 0x11f, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a, 0x65,
0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x50, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b,
0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b,
0x75, 0x7a, 0x65, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6e, 0x61, 0x64, 0x131, 0x72,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x41, 0x6e, 0x61, 0x64, 0x131, 0x72, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x41, 0x70, 0x69, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x41, 0x6b, 0x74, 0x61, 0x76, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x41, 0x6b, 0x74, 0x61, 0x76, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6b, 0x74,
0x61, 0x76, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x41, 0x6b, 0x74, 0xf6, 0x62, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x41, 0x6b, 0x74, 0xf6, 0x62, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x6b, 0x74,
0xf6, 0x62, 0x65, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41,
0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41,
0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74,
0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x72,
0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x45, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x45, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6b, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4f, 0x72, 0x74, 0x61, 0x20, 0x41, 0x76, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x72, 0x74,
0x61, 0x20, 0x41, 0x76, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x41, 0x76, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x130, 0xe7, 0x20, 0x42, 0x61, 0x74, 0x131, 0x20,
0x41, 0x76, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x130, 0xe7, 0x20, 0x42, 0x61, 0x74, 0x131, 0x20,
0x41, 0x76, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x130, 0xe7, 0x20, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x76, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41, 0x76,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41, 0x76, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75,
0x20, 0x41, 0x76, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x61, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x79, 0x63, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x79, 0x63, 0x61, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x79, 0x63, 0x61, 0x6e,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x7a,
0x6f, 0x72, 0x6c, 0x61, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41,
0x7a, 0x6f, 0x72, 0x6c, 0x61, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x41, 0x7a, 0x6f,
0x72, 0x6c, 0x61, 0x72, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x15f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x15f, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42,
0x6f, 0x6c, 0x69, 0x76, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x69, 0x20, 0x44, 0x61, 0x72, 0xfc, 0x2019, 0x73, 0x2d, 0x53, 0x65, 0x6c,
0x61, 0x6d, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x61, 0x70, 0x65,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x61, 0x73,
0x65, 0x79, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x15e, 0x69, 0x6c, 0x69, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x15e, 0x69, 0x6c, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x15e, 0x69, 0x6c, 0x69, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0xc7, 0x69, 0x6e,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0xc7, 0x69, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0xc7, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0xc7, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0xc7, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0xc7, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43,
0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x20, 0x41, 0x64, 0x61,
0x73, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x79, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62,
0x69, 0x79, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72,
0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x6f, 0x6f, 0x6b, 0x20,
0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x43, 0x6f,
0x6f, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x59,
0x61, 0x72, 0x131, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4b, 0xfc, 0x62, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b,
0xfc, 0x62, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0xfc, 0x62, 0x61, 0x20, 0x59,
0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x61, 0x76, 0x69,
0x73, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x75, 0x6d, 0x6f, 0x6e,
0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x54, 0x69,
0x6d, 0x6f, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x73,
0x6b, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x73, 0x6b, 0x61, 0x6c, 0x79,
0x61, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61,
0x73, 0x6b, 0x61, 0x6c, 0x79, 0x61, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x45, 0x6b,
0x76, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f,
0x72, 0x74, 0x61, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x41, 0x76, 0x72,
0x75, 0x70, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x41,
0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41, 0x76, 0x72, 0x75,
0x70, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75,
0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f,
0x11f, 0x75, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x6c, 0x65, 0x72, 0x69,
0x20, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41,
0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42,
0x61, 0x74, 0x131, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x41, 0x76, 0x72, 0x75, 0x70, 0x61,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61,
0x72, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x69,
0x6a, 0x69, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x46, 0x72, 0x61, 0x6e, 0x73, 0x131, 0x7a, 0x20, 0x47, 0x75, 0x79, 0x61,
0x6e, 0x61, 0x73, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x72,
0x61, 0x6e, 0x73, 0x131, 0x7a, 0x20, 0x47, 0xfc, 0x6e, 0x65, 0x79, 0x20,
0x76, 0x65, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b,
0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47,
0xfc, 0x72, 0x63, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x47, 0xfc, 0x72, 0x63, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x47, 0xfc, 0x72, 0x63, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0x69, 0x6c,
0x62, 0x65, 0x72, 0x74, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x20, 0x4f, 0x72, 0x74, 0x61, 0x6c, 0x61, 0x6d, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x47,
0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x47, 0x72,
0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x47, 0x72, 0xf6,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42,
0x61, 0x74, 0x131, 0x20, 0x47, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x42, 0x61, 0x74, 0x131, 0x20, 0x47, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x47, 0x75, 0x61, 0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0xf6, 0x72,
0x66, 0x65, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x47, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e,
0x67, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x48, 0x6f, 0x76, 0x64, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x48, 0x6f, 0x76, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x6f, 0x76, 0x64, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x69, 0x6e,
0x64, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x48, 0x69, 0x6e,
0x74, 0x20, 0x4f, 0x6b, 0x79, 0x61, 0x6e, 0x75, 0x73, 0x75, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x48, 0x69, 0x6e, 0x64, 0x69, 0xe7, 0x69, 0x6e,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x72, 0x74, 0x61, 0x20, 0x45,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x45, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x7a, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42, 0x61,
0x74, 0x131, 0x20, 0x45, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x72, 0x61, 0x6e, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x130, 0x72, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x130,
0x72, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x130, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x130, 0x73, 0x72, 0x61, 0x69, 0x6c, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x79,
0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4a, 0x61, 0x70, 0x6f, 0x6e,
0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x79, 0x61,
0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0xe7, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61,
0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0xe7, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x44, 0x6f, 0x11f, 0x75, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x42,
0x61, 0x74, 0x131, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x6f, 0x72, 0x65,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x6f, 0x72, 0x65, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4b, 0x6f, 0x72, 0x65, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x131, 0x72, 0x67, 0x131,
0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4c,
0x69, 0x6e, 0x65, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4d, 0x61, 0x6b, 0x61, 0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61,
0x6b, 0x61, 0x6f, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x79, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76,
0x6c, 0x65, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x72,
0x6b, 0x69, 0x7a, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c,
0x6c, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x50, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x20, 0x4b, 0x131, 0x79, 0x131, 0x73, 0x131, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4b, 0x131, 0x79,
0x131, 0x73, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x4b, 0x131,
0x79, 0x131, 0x73, 0x131, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x75, 0x72,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x75, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x20,
0x42, 0x61, 0x74, 0x75, 0x72, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x76, 0x61, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4d, 0x79, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x61, 0x75, 0x72,
0x75, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x65, 0x70, 0x61, 0x6c,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x79, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6e, 0x69, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6e, 0x69, 0x20,
0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6e, 0x69, 0x20, 0x5a, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e,
0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x69, 0x75,
0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x6f, 0x72, 0x66, 0x6f,
0x6c, 0x6b, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x41, 0x64,
0x61, 0x73, 0x131, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e,
0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68,
0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x75, 0x7a,
0x65, 0x79, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x41,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x4f, 0x6d, 0x73,
0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x59,
0x65, 0x6e, 0x69, 0x20, 0x47, 0x69, 0x6e, 0x65, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x65, 0x72,
0x75, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x65, 0x72, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x50, 0x65, 0x72, 0x75, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x6c,
0x65, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x6c, 0x65, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x69, 0x6e, 0x6c, 0x65, 0x72, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78,
0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x76, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x76, 0x65, 0x20,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x76, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50,
0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x50,
0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4b, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x4b, 0x131, 0x7a, 0x131, 0x6c, 0x6f, 0x72,
0x64, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x59, 0x61,
0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x65, 0x79, 0x15f, 0x65, 0x6c, 0x6c,
0x65, 0x72, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x75, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53, 0x6f, 0x6c, 0x6f, 0x6d,
0x6f, 0x6e, 0x20, 0x41, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x131, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x47, 0xfc, 0x6e, 0x65, 0x79, 0x20, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x53,
0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x53, 0x68, 0x6f, 0x77, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54,
0x61, 0x68, 0x69, 0x74, 0x69, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54,
0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x54, 0x61, 0x63, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0xfc, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54,
0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x54, 0x75, 0x76,
0x61, 0x6c, 0x75, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x55, 0x72,
0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x55, 0x72, 0x75, 0x67,
0x75, 0x61, 0x79, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0xd6, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0xd6, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0xd6, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x59, 0x61, 0x7a,
0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75,
0x65, 0x6c, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69, 0x57,
0x61, 0x6b, 0x65, 0x20, 0x41, 0x64, 0x61, 0x73, 0x131, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x76, 0x65,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x53, 0x61, 0x61, 0x74,
0x69, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x59, 0x61, 0x7a, 0x20,
0x53, 0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x53, 0x61, 0x61, 0x74, 0x69,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x53,
0x61, 0x61, 0x74, 0x69, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x59, 0x61, 0x7a, 0x20, 0x53, 0x61,
0x61, 0x74, 0x69, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x53, 0x61, 0x61,
0x74, 0x69, 0x4f, 0x77, 0x67, 0x61, 0x6e, 0x79, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a,
0x69, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0xfc, 0x6e, 0x6f, 0x72, 0x74, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72,
0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61,
0x74, 0x61, 0x72, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x41, 0x6c, 0xfd, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x41, 0x6c, 0xfd, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41,
0x6c, 0xfd, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x6d, 0x61, 0x7a,
0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x6b, 0x61, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65,
0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79,
0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x67, 0xfc,
0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x67, 0xfc, 0x6e, 0x64, 0x6f,
0x67, 0x61, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67,
0x61, 0x7a, 0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x67, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x44,
0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x64, 0x61, 0x67, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79,
0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x64, 0x61,
0x67, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a,
0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x64,
0x61, 0x67, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a,
0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0xdd,
0x75, 0x77, 0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d, 0x61, 0x6e, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a,
0x79, 0x6b, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0xdd,
0x75, 0x77, 0x61, 0x15f, 0x20, 0x75, 0x6d, 0x6d, 0x61, 0x6e, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x44, 0x65, 0x6d, 0x69, 0x72, 0x67, 0x61, 0x7a, 0x79, 0x6b, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0xdd, 0x75, 0x77, 0x61,
0x15f, 0x20, 0x75, 0x6d, 0x6d, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x6e, 0x61,
0x64, 0x79, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x70, 0x69,
0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x70, 0x69, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x41, 0x70, 0x69, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x72, 0x61, 0x70,
0x20, 0xfd, 0x75, 0x72, 0x74, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x41, 0x72, 0x61, 0x70, 0x20, 0xfd, 0x75, 0x72, 0x74,
0x6c, 0x61, 0x72, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x72, 0x61, 0x70, 0x20,
0xfd, 0x75, 0x72, 0x74, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e,
0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x45, 0x72, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x45, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72,
0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72,
0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a,
0x69, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x77,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x67, 0xfc, 0x6e,
0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d,
0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x67, 0xfc, 0x6e, 0x62, 0x61, 0x74,
0x61, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69,
0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20,
0x67, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc,
0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x41, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e,
0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74,
0x61, 0x72, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd,
0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61,
0x74, 0x61, 0x72, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61,
0x72, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0xfd, 0x61,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0xfd, 0x6a, 0x61, 0x6e,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0xfd, 0x6a, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0xfd, 0x6a, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x41, 0x7a, 0x6f, 0x72, 0x20,
0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72,
0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x61, 0x64, 0x61,
0x6c, 0x61, 0x72, 0x79, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x15f, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x15f, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x75, 0x74, 0x61,
0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x6f, 0x6c, 0x69, 0x77,
0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42, 0x72, 0x61,
0x7a, 0x69, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x42,
0x72, 0x75, 0x6e, 0x65, 0xfd, 0x2d, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73,
0x61, 0x6c, 0x61, 0x6d, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x61,
0x62, 0x6f, 0x2d, 0x57, 0x65, 0x72, 0x64, 0x65, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4b, 0x61, 0x62, 0x6f, 0x2d, 0x57, 0x65, 0x72, 0x64, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4b, 0x61, 0x62, 0x6f, 0x2d, 0x57, 0x65, 0x72, 0x64,
0x65, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0xc7, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0xc7, 0x61, 0x74, 0x65, 0x6d, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0xc7, 0x61, 0x74, 0x65, 0x6d, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7,
0x61, 0x74, 0x65, 0x6d, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7, 0x69, 0x6c, 0x69, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0xc7, 0x69, 0x6c, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7,
0x69, 0x6c, 0x69, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x48, 0x79, 0x74, 0x61, 0xfd, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x48, 0x79, 0x74, 0x61, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x48, 0x79, 0x74, 0x61, 0xfd, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7, 0x6f, 0xfd, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7, 0x6f,
0xfd, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xc7, 0x6f,
0xfd, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x52, 0x6f, 0x17e, 0x64,
0x65, 0x73, 0x74, 0x77, 0x6f, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x6f, 0x6b, 0x6f, 0x73, 0x20, 0x61,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0xfd, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0xfd, 0x61,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4b, 0x75, 0x6b, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x75, 0x6b, 0x20, 0x61,
0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x75, 0x6b,
0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x75,
0x62, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x75, 0x62, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0x65, 0xfd,
0x77, 0x69, 0x73, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x44, 0xfd, 0x75,
0x6d, 0x6f, 0x6e, 0x2d, 0x64, 0x2d, 0xdd, 0x75, 0x72, 0x77, 0x69, 0x6c,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67,
0x61, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x50, 0x61, 0x73, 0x68, 0x61, 0x20, 0x61, 0x64, 0x61, 0x73,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x73, 0x68, 0x61,
0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x73,
0x68, 0x61, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x45, 0x6b,
0x77, 0x61, 0x64, 0x6f, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d,
0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b,
0x65, 0x7a, 0x69, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0xdd, 0x65,
0x77, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f,
0x67, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61,
0x72, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77,
0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x7a, 0x61, 0x6b, 0x20, 0x47,
0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e,
0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61,
0x74, 0x61, 0x72, 0x20, 0xdd, 0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0xdd,
0x65, 0x77, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x6f, 0x6c, 0x6b,
0x6c, 0x65, 0x6e, 0x64, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65,
0x6e, 0x64, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x61, 0x64,
0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x69, 0x6a, 0x69, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x46, 0x69, 0x6a, 0x69, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x75,
0x7a, 0x20, 0x47, 0x77, 0x69, 0x61, 0x6e, 0x61, 0x73, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x75, 0x7a, 0x20,
0x47, 0xfc, 0x6e, 0x6f, 0x72, 0x74, 0x61, 0x20, 0x77, 0x65, 0x20, 0x41,
0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74, 0x69, 0x6b, 0x61, 0x20, 0xfd, 0x65,
0x72, 0x6c, 0x65, 0x72, 0x69, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0x61, 0x64, 0x61,
0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x61,
0x6d, 0x62, 0xfd, 0x65, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x72,
0x75, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0x72, 0x75, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x72, 0x75,
0x7a, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x47, 0x72, 0x69, 0x6e, 0x77, 0x69, 0xe7, 0x20, 0x6f,
0x72, 0x74, 0x61, 0xe7, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x47, 0x72,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc,
0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x47, 0x72, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x72,
0x73, 0x20, 0x61, 0xfd, 0x6c, 0x61, 0x67, 0x79, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0x61, 0xfd, 0x61, 0x6e, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47,
0x61, 0x77, 0x61, 0xfd, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x47, 0x61, 0x77, 0x61, 0xfd, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x61, 0x77, 0x61, 0xfd, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x6f, 0x6e, 0x6b, 0x6f,
0x6e, 0x67, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x6f, 0x6e, 0x6b,
0x6f, 0x6e, 0x67, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e,
0x67, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x48, 0x6f, 0x77, 0x64, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x48, 0x6f, 0x77, 0x64, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x48, 0x6f, 0x77, 0x64,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x48, 0x69, 0x6e, 0x64, 0x69, 0x20, 0x75, 0x6d, 0x6d, 0x61,
0x6e, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x48, 0x69, 0x6e, 0x64,
0x69, 0x68, 0x79, 0x74, 0x61, 0xfd, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4d, 0x65, 0x72, 0x6b, 0x65, 0x7a, 0x69, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x45, 0xfd, 0x72, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x45, 0xfd, 0x72, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x45, 0xfd, 0x72,
0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x59, 0x73, 0x72, 0x61, 0xfd, 0x79, 0x6c, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x59, 0x73, 0x72, 0x61, 0xfd, 0x79, 0x6c, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x59,
0x73, 0x72, 0x61, 0xfd, 0x79, 0x6c, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x61, 0x70, 0x6f,
0x6e, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x61,
0x70, 0x6f, 0x6e, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x61, 0x70,
0x6f, 0x6e, 0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x77, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0xe7, 0x61, 0x74, 0x73, 0x6b, 0x69, 0xfd, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x47, 0xfc, 0x6e, 0x64, 0x6f, 0x67, 0x61, 0x72, 0x20, 0x47, 0x61,
0x7a, 0x61, 0x67, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x47, 0xfc, 0x6e, 0x62, 0x61, 0x74, 0x61, 0x72, 0x20, 0x47,
0x61, 0x7a, 0x61, 0x67, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4b, 0x6f, 0x72, 0x65, 0xfd, 0x61, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4b, 0x6f, 0x72, 0x65, 0xfd, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4b, 0x6f, 0x72, 0x65, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0xfd, 0x65, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0xfd, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0xfd, 0x61, 0x72, 0x73,
0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0xfd, 0x61,
0x72, 0x73, 0x6b, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0x79, 0x72, 0x67, 0x79, 0x7a, 0x79,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4c, 0x61,
0xfd, 0x6e, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x61, 0x75,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48,
0x61, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x61,
0x75, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d,
0x61, 0x6c, 0x61, 0xfd, 0x7a, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x77, 0x6c, 0x65, 0x72, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61, 0x72, 0x6b, 0x69, 0x7a, 0x20,
0x61, 0x64, 0x61, 0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4d, 0x61, 0x72, 0x15f, 0x61, 0x6c, 0x6c, 0x20, 0x61, 0x64, 0x61,
0x6c, 0x61, 0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x61,
0x77, 0x72, 0x69, 0x6b, 0x69, 0xfd, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4d, 0x61, 0x77, 0x72, 0x69, 0x6b, 0x69, 0xfd, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d,
0x61, 0x77, 0x72, 0x69, 0x6b, 0x69, 0xfd, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x6f, 0x75,
0x73, 0x6f, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0xdd, 0x75, 0x77, 0x61, 0x15f, 0x20,
0x75, 0x6d, 0x6d, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0xdd, 0x75, 0x77, 0x61,
0x15f, 0x20, 0x75, 0x6d, 0x6d, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0x61, 0x6e, 0x20, 0xdd, 0x75, 0x77, 0x61, 0x15f,
0x20, 0x75, 0x6d, 0x6d, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73,
0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x55, 0x6c, 0x61, 0x6e,
0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4d, 0x6f, 0x73, 0x6b, 0x77, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0x6f,
0x73, 0x6b, 0x77, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4d, 0xfd, 0x61, 0x6e, 0x6d, 0x61,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x54, 0xe4, 0x7a, 0x65, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x54, 0xe4, 0x7a, 0x65, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0xfd, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0xe4, 0x7a, 0x65, 0x20,
0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0xfd, 0x61, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x54, 0xe4, 0x7a, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0xfd, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0xe4, 0x7a, 0x65,
0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0xfd, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x54, 0xe4, 0x7a, 0x65, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0xfd, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0xfd, 0x75, 0x66, 0x61, 0x75, 0x6e,
0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e,
0xfd, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4e, 0xfd, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65,
0x6e, 0x64, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x4e, 0x69, 0x75, 0x65, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x61, 0x64,
0x61, 0x73, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x61, 0x64, 0x61,
0x73, 0x79, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75,
0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0xfd, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x75, 0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0xfd, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x2d,
0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0xfd, 0x61, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4e, 0x6f, 0x77, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x4f, 0x6d, 0x73, 0x6b, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x70, 0x75, 0x61, 0x20, 0x2d, 0x20,
0x54, 0xe4, 0x7a, 0x65, 0x20, 0x47, 0x77, 0x69, 0x6e, 0x65, 0xfd, 0x61,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x72, 0x61, 0x67, 0x77,
0x61, 0xfd, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x72, 0x61,
0x67, 0x77, 0x61, 0xfd, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x61, 0x72, 0x61, 0x67,
0x77, 0x61, 0xfd, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x65, 0x72, 0x75, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x50, 0x65, 0x72, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x65,
0x72, 0x75, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x6c, 0x65, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x6e, 0x6c, 0x65, 0x72, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x6c, 0x65, 0x72, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61,
0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x65, 0x6e, 0x2d,
0x50, 0xfd, 0x65, 0x72, 0x20, 0x77, 0x65, 0x20, 0x4d, 0x69, 0x6b, 0x65,
0x6c, 0x6f, 0x6e, 0x53, 0x65, 0x6e, 0x2d, 0x50, 0xfd, 0x65, 0x72, 0x20,
0x77, 0x65, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x53, 0x65, 0x6e, 0x2d, 0x50, 0xfd, 0x65, 0x72, 0x20, 0x77, 0x65,
0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50, 0x69,
0x74, 0x6b, 0x65, 0x72, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50,
0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x50,
0x68, 0x65, 0x6e, 0xfd, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x52, 0x65, 0xfd, 0x75, 0x6e, 0xfd, 0x6f, 0x6e, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x53, 0x61, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x74,
0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x53, 0x65, 0xfd, 0x15f, 0x65, 0x6c, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61,
0x72, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x69, 0x6e, 0x67,
0x61, 0x70, 0x75, 0x72, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x6f,
0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x61, 0x64, 0x61, 0x6c, 0x61, 0x72,
0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x47, 0xfc, 0x6e, 0x6f, 0x72,
0x74, 0x61, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0xfd, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x53, 0xfd, 0x6f, 0x77, 0x61, 0x20,
0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x61, 0x69, 0x74, 0x69, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x54, 0x61, 0xfd, 0x62, 0x65, 0xfd, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0x54, 0x61, 0xfd, 0x62, 0x65, 0xfd, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x54, 0x61, 0xfd, 0x62, 0x65, 0xfd, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0xe4, 0x6a,
0x69, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0xc7, 0x75, 0x75, 0x6b, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x54, 0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54, 0xfc, 0x72, 0x6b,
0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x54,
0xfc, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x54, 0x75, 0x77, 0x61, 0x6c, 0x75, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0xfd, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0xfd, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x55, 0x72, 0x75, 0x67, 0x77, 0x61, 0xfd, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xd6, 0x7a, 0x62,
0x65, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0xd6, 0x7a, 0x62, 0x65, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0xd6, 0x7a, 0x62, 0x65, 0x67, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x74, 0x6f, 0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0x57, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x57, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x57, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x74, 0x6f, 0x6d,
0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x65,
0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x6c, 0x61, 0x64,
0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x57,
0x65, 0xfd, 0x6b, 0x20, 0x61, 0x64, 0x61, 0x73, 0x79, 0x20, 0x77, 0x61,
0x67, 0x74, 0x79, 0x55, 0x6f, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x77, 0x65,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x77, 0x61, 0x67, 0x74,
0x79, 0xdd, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x77, 0x61, 0x67,
0x74, 0x79, 0xdd, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79,
0xdd, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x74, 0x6f, 0x6d, 0x75,
0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x77,
0x61, 0x67, 0x74, 0x79, 0xdd, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x74, 0x6f,
0x6d, 0x75, 0x73, 0x6b, 0x79, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0xdd,
0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x77, 0x61, 0x67, 0x74, 0x79, 0x447, 0x430,
0x441, 0x3a, 0x20, 0x410, 0x43a, 0x440, 0x456, 0x447, 0x430, 0x441, 0x3a, 0x20,
0x410, 0x43a, 0x440, 0x456, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x439, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x410, 0x43a,
0x440, 0x456, 0x2c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x439, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x410, 0x444, 0x433,
0x430, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x430, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x43e,
0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432,
0x434, 0x435, 0x43d, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d,
0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430,
0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x43b, 0x44f, 0x441, 0x446, 0x456,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x410, 0x43b, 0x44f, 0x441, 0x446, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x410, 0x43b, 0x44f, 0x441, 0x446, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e,
0x43d, 0x446, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x446, 0x456, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d,
0x446, 0x456, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f,
0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x446, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d,
0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447,
0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d,
0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x438,
0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456,
0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x445, 0x456, 0x434,
0x43d, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456,
0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x433, 0x456, 0x440, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456,
0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x433, 0x456, 0x440, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435,
0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x433,
0x456, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f,
0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438,
0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x442, 0x438, 0x445,
0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d,
0x456, 0x447, 0x43d, 0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a,
0x435, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43f, 0x456, 0x432, 0x43d, 0x456, 0x447, 0x43d,
0x43e, 0x430, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x410,
0x43d, 0x430, 0x434, 0x438, 0x440, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x410, 0x43d,
0x430, 0x434, 0x438, 0x440, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x439, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x410, 0x43d,
0x430, 0x434, 0x438, 0x440, 0x2c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x439,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x410,
0x43f, 0x456, 0x430, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x432, 0x20, 0x410, 0x43f, 0x456, 0x430, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20,
0x410, 0x43f, 0x456, 0x430, 0x437, 0x430, 0x20, 0x430, 0x440, 0x430, 0x431, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x430, 0x440, 0x430, 0x431, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x430, 0x440, 0x430, 0x431, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x430, 0x440, 0x433, 0x435,
0x43d, 0x442, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x430, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x430,
0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445,
0x456, 0x434, 0x43d, 0x43e, 0x430, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e,
0x430, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432, 0x456,
0x440, 0x43c, 0x435, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432, 0x456, 0x440, 0x43c, 0x435, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x432, 0x456, 0x440, 0x43c, 0x435, 0x43d, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438,
0x447, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x430, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x447, 0x43d, 0x438, 0x43c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x430, 0x442, 0x43b,
0x430, 0x43d, 0x442, 0x438, 0x447, 0x43d, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x430, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x446, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440,
0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x430,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x446, 0x435,
0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x437, 0x430, 0x445, 0x456,
0x434, 0x43d, 0x438, 0x43c, 0x20, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x43e, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x430,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456,
0x442, 0x43d, 0x456, 0x43c, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b,
0x44c, 0x43d, 0x43e, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20,
0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441,
0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b,
0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445,
0x456, 0x434, 0x43d, 0x43e, 0x430, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456,
0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x430,
0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x430, 0x437,
0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x430,
0x437, 0x435, 0x440, 0x431, 0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x430, 0x437, 0x435, 0x440, 0x431,
0x430, 0x439, 0x434, 0x436, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410, 0x437, 0x43e, 0x440, 0x441, 0x44c, 0x43a,
0x438, 0x445, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x410,
0x437, 0x43e, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x445, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x410, 0x437, 0x43e, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x445, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434,
0x435, 0x448, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443,
0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x437, 0x430,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435,
0x448, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20,
0x411, 0x443, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x431, 0x43e, 0x43b,
0x456, 0x432, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x431, 0x440, 0x430, 0x437, 0x438, 0x43b,
0x44c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x431, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x44c, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x431, 0x440, 0x430, 0x437, 0x438,
0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20,
0x411, 0x440, 0x443, 0x43d, 0x435, 0x457, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x430, 0x445, 0x20, 0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434,
0x435, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41a, 0x430,
0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20,
0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41f, 0x456,
0x432, 0x43d, 0x456, 0x447, 0x43d, 0x438, 0x445, 0x20, 0x41c, 0x430, 0x440, 0x456,
0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x445, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x430, 0x440, 0x445, 0x456, 0x43f, 0x435, 0x43b, 0x430,
0x437, 0x456, 0x20, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x430, 0x440, 0x445, 0x456,
0x43f, 0x435, 0x43b, 0x430, 0x437, 0x456, 0x20, 0x427, 0x430, 0x442, 0x435, 0x43c,
0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x430, 0x440, 0x445, 0x456, 0x43f,
0x435, 0x43b, 0x430, 0x437, 0x456, 0x20, 0x427, 0x430, 0x442, 0x435, 0x43c, 0x437,
0x430, 0x20, 0x447, 0x438, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x438, 0x43b,
0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x438, 0x43b, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x438, 0x442, 0x430, 0x439, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43a, 0x438, 0x442, 0x430, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x438, 0x442, 0x430,
0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b,
0x441, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x443, 0x20, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d,
0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x427, 0x43e, 0x439, 0x431, 0x430,
0x43b, 0x441, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456,
0x20, 0x420, 0x456, 0x437, 0x434, 0x432, 0x430, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41a, 0x43e, 0x43a, 0x43e, 0x441,
0x43e, 0x432, 0x438, 0x445, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x437, 0x430, 0x20, 0x43a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x456, 0x439,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x43a, 0x43e, 0x43b, 0x443, 0x43c, 0x431, 0x456, 0x439, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x43a, 0x43e, 0x43b, 0x443, 0x43c,
0x431, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41a,
0x443, 0x43a, 0x430, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20,
0x41a, 0x443, 0x43a, 0x430, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41a, 0x443, 0x43a, 0x430,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x41a, 0x443, 0x431, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x41a, 0x443, 0x431, 0x456, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x41a, 0x443, 0x431, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446,
0x456, 0x457, 0x20, 0x414, 0x435, 0x432, 0x456, 0x441, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x446, 0x456, 0x457, 0x20, 0x414, 0x44e, 0x43c, 0x43e, 0x43d, 0x20, 0x434, 0x2bc,
0x42e, 0x440, 0x432, 0x456, 0x43b, 0x44c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x421, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x43c,
0x443, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x456, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x456, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x437, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x456, 0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x437, 0x430,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456,
0x20, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x432, 0x20, 0x415, 0x43a, 0x432, 0x430, 0x434, 0x43e, 0x440,
0x456, 0x437, 0x430, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c,
0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x446,
0x435, 0x43d, 0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x454, 0x432, 0x440,
0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x446, 0x435, 0x43d, 0x442, 0x440, 0x430,
0x43b, 0x44c, 0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x445, 0x456, 0x434,
0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441,
0x445, 0x456, 0x434, 0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e,
0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x434, 0x430, 0x43b, 0x435, 0x43a, 0x43e, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x438,
0x43c, 0x20, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437,
0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435,
0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x454, 0x432,
0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434,
0x43d, 0x43e, 0x454, 0x432, 0x440, 0x43e, 0x43f, 0x435, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441,
0x44c, 0x43a, 0x438, 0x445, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x441, 0x44c, 0x43a,
0x438, 0x445, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x424, 0x43e, 0x43b, 0x43a, 0x43b, 0x435,
0x43d, 0x434, 0x441, 0x44c, 0x43a, 0x438, 0x445, 0x20, 0x41e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x443, 0x20, 0x424, 0x456, 0x434, 0x436, 0x456, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x424, 0x456, 0x434, 0x436, 0x456,
0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x424, 0x456, 0x434, 0x436, 0x456, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x424, 0x440, 0x430, 0x43d,
0x446, 0x443, 0x437, 0x44c, 0x43a, 0x43e, 0x457, 0x20, 0x490, 0x432, 0x456, 0x430,
0x43d, 0x438, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x44c, 0x43a, 0x438,
0x445, 0x20, 0x41f, 0x456, 0x432, 0x434, 0x435, 0x43d, 0x43d, 0x438, 0x445, 0x20,
0x456, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442, 0x438, 0x447, 0x43d,
0x438, 0x445, 0x20, 0x442, 0x435, 0x440, 0x438, 0x442, 0x43e, 0x440, 0x456, 0x44f,
0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x490, 0x430,
0x43b, 0x430, 0x43f, 0x430, 0x491, 0x43e, 0x441, 0x443, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x456, 0x20, 0x490, 0x430, 0x43c, 0x431, 0x2bc, 0x454, 0x437, 0x430,
0x20, 0x433, 0x440, 0x443, 0x437, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x433, 0x440, 0x443,
0x437, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x433, 0x440, 0x443, 0x437, 0x438, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x20, 0x413, 0x456, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x430, 0x437, 0x430,
0x20, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x456, 0x447, 0x435, 0x43c, 0x437, 0x430,
0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d,
0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x437,
0x430, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x490, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x456, 0x457, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490, 0x440,
0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x437, 0x430, 0x445, 0x456, 0x434, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490,
0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x456, 0x20, 0x413, 0x443, 0x430, 0x43c, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x41f, 0x435, 0x440, 0x441, 0x44c, 0x43a,
0x43e, 0x457, 0x20, 0x437, 0x430, 0x442, 0x43e, 0x43a, 0x438, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x490, 0x430, 0x44f, 0x43d,
0x456, 0x437, 0x430, 0x20, 0x433, 0x430, 0x432, 0x430, 0x439, 0x441, 0x44c, 0x43a,
0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x433, 0x430, 0x432,
0x430, 0x439, 0x441, 0x44c, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x433, 0x430, 0x432,
0x430, 0x439, 0x441, 0x44c, 0x43a, 0x43e, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x413, 0x43e,
0x43d, 0x43a, 0x43e, 0x43d, 0x437, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x437,
0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x413, 0x43e, 0x43d, 0x43a, 0x43e,
0x43d, 0x437, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x425, 0x43e, 0x432, 0x434, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x425, 0x43e, 0x432, 0x434, 0x456, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x425, 0x43e, 0x432, 0x434, 0x456, 0x437, 0x430,
0x20, 0x456, 0x43d, 0x434, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x432, 0x20, 0x406, 0x43d, 0x434, 0x456, 0x439, 0x441, 0x44c, 0x43a,
0x43e, 0x43c, 0x443, 0x20, 0x41e, 0x43a, 0x435, 0x430, 0x43d, 0x456, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x406, 0x43d, 0x434,
0x43e, 0x43a, 0x438, 0x442, 0x430, 0x457, 0x437, 0x430, 0x20, 0x446, 0x435, 0x43d,
0x442, 0x440, 0x430, 0x43b, 0x44c, 0x43d, 0x43e, 0x456, 0x43d, 0x434, 0x43e, 0x43d,
0x435, 0x437, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x43e,
0x456, 0x43d, 0x434, 0x43e, 0x43d, 0x435, 0x437, 0x456, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x437,
0x430, 0x445, 0x456, 0x434, 0x43d, 0x43e, 0x456, 0x43d, 0x434, 0x43e, 0x43d, 0x435,
0x437, 0x456, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456, 0x440, 0x430, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456,
0x440, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456, 0x440, 0x430, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456, 0x440, 0x43a, 0x443, 0x442, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x456, 0x440, 0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456, 0x440, 0x43a, 0x443,
0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456, 0x437,
0x440, 0x430, 0x457, 0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x456, 0x437, 0x440, 0x430, 0x457,
0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x456, 0x437, 0x440, 0x430, 0x457, 0x43b, 0x44c, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x44f, 0x43f, 0x43e, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x44f, 0x43f, 0x43e, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x44f, 0x43f, 0x43e, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x430, 0x43c,
0x447, 0x430, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430,
0x20, 0x43a, 0x430, 0x43c, 0x447, 0x430, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x441, 0x445, 0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41a, 0x430, 0x437, 0x430,
0x445, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x437, 0x430, 0x445,
0x456, 0x434, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x41a, 0x430, 0x437, 0x430, 0x445, 0x441, 0x442, 0x430, 0x43d, 0x456,
0x437, 0x430, 0x20, 0x43a, 0x43e, 0x440, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x43e,
0x440, 0x435, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x43e, 0x440, 0x435, 0x439, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20,
0x41a, 0x43e, 0x441, 0x440, 0x430, 0x435, 0x437, 0x430, 0x20, 0x43a, 0x440, 0x430,
0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x44f, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f,
0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41a, 0x438, 0x440, 0x433, 0x438, 0x437,
0x441, 0x442, 0x430, 0x43d, 0x456, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x41b, 0x430,
0x43d, 0x43a, 0x430, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41b,
0x430, 0x439, 0x43d, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41b,
0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x456, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x456, 0x20, 0x41b, 0x43e, 0x440, 0x434, 0x2d, 0x425, 0x430, 0x443, 0x437, 0x430,
0x20, 0x43c, 0x430, 0x433, 0x430, 0x434, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43c, 0x430,
0x433, 0x430, 0x434, 0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43c, 0x430, 0x433, 0x430, 0x434,
0x430, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41c, 0x430, 0x43b, 0x430, 0x439,
0x437, 0x456, 0x457, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x456, 0x432, 0x430, 0x445,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x41c, 0x430, 0x440, 0x43a, 0x456, 0x437, 0x44c, 0x43a, 0x438, 0x445, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x41c, 0x430, 0x440, 0x448, 0x430,
0x43b, 0x43b, 0x43e, 0x432, 0x438, 0x445, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41c,
0x430, 0x432, 0x440, 0x438, 0x43a, 0x456, 0x439, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x456, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x456, 0x439, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x456, 0x20, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a, 0x456, 0x439, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x446, 0x456, 0x457, 0x20, 0x41c, 0x43e, 0x443, 0x441, 0x43e, 0x43d,
0x437, 0x430, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x443, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x446, 0x456, 0x437, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41c,
0x435, 0x43a, 0x441, 0x438, 0x446, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x442, 0x438, 0x445, 0x43e, 0x43e, 0x43a, 0x435, 0x430,
0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x443, 0x20, 0x41c, 0x435, 0x43a, 0x441, 0x438, 0x446, 0x456, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423, 0x43b, 0x430,
0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x456, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423, 0x43b, 0x430, 0x43d, 0x2d,
0x411, 0x430, 0x442, 0x43e, 0x440, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20,
0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x456, 0x437,
0x430, 0x20, 0x43c, 0x43e, 0x441, 0x43a, 0x43e, 0x432, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43c, 0x43e,
0x441, 0x43a, 0x43e, 0x432, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43c, 0x43e, 0x441, 0x43a, 0x43e, 0x432,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41c, 0x2bc, 0x44f, 0x43d, 0x43c, 0x456, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x430, 0x443, 0x440, 0x443,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41d,
0x435, 0x43f, 0x430, 0x43b, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x457, 0x20, 0x41a, 0x430, 0x43b, 0x435,
0x434, 0x43e, 0x43d, 0x456, 0x457, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x20, 0x41d, 0x43e, 0x432, 0x43e, 0x457, 0x20, 0x41a, 0x430, 0x43b, 0x435,
0x434, 0x43e, 0x43d, 0x456, 0x457, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41d, 0x43e, 0x432,
0x43e, 0x457, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x456, 0x457,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41d,
0x43e, 0x432, 0x456, 0x439, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x456,
0x457, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20,
0x41d, 0x43e, 0x432, 0x456, 0x439, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434,
0x456, 0x457, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41d, 0x43e, 0x432, 0x456,
0x439, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x456, 0x457, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443,
0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x456, 0x20, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435,
0x43d, 0x434, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41d, 0x44c, 0x44e, 0x444,
0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x456, 0x20, 0x41d, 0x456, 0x443, 0x435, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x456, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x437,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e,
0x43b, 0x43a, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x456, 0x20, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x430, 0x440, 0x445, 0x456, 0x43f, 0x435, 0x43b, 0x430, 0x437, 0x456, 0x20, 0x424,
0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434, 0x443, 0x2d, 0x434, 0x456, 0x2d, 0x41d,
0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x430, 0x440, 0x445, 0x456, 0x43f, 0x435,
0x43b, 0x430, 0x437, 0x456, 0x20, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d, 0x434,
0x443, 0x2d, 0x434, 0x456, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c, 0x44f,
0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x430, 0x440, 0x445, 0x456, 0x43f,
0x435, 0x43b, 0x430, 0x437, 0x456, 0x20, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d,
0x434, 0x443, 0x2d, 0x434, 0x456, 0x2d, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c,
0x44f, 0x437, 0x430, 0x20, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x456,
0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x456, 0x440,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x43d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x456, 0x440, 0x441,
0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43e, 0x43c, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x43e, 0x43c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x43e, 0x43c, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41f,
0x430, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41f, 0x430, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41f, 0x430,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x456, 0x20, 0x41f, 0x430, 0x43b, 0x430, 0x443, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x20, 0x41f, 0x430, 0x43f, 0x443, 0x430, 0x2d, 0x41d,
0x43e, 0x432, 0x430, 0x20, 0x490, 0x432, 0x456, 0x43d, 0x435, 0x44f, 0x437, 0x430,
0x20, 0x43f, 0x430, 0x440, 0x430, 0x491, 0x432, 0x430, 0x439, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x43f,
0x430, 0x440, 0x430, 0x491, 0x432, 0x430, 0x439, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x43f, 0x430, 0x440, 0x430, 0x491, 0x432, 0x430, 0x439,
0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41f, 0x435,
0x440, 0x443, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443,
0x20, 0x41f, 0x435, 0x440, 0x443, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x41f,
0x435, 0x440, 0x443, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x424, 0x456, 0x43b, 0x456, 0x43f, 0x43f, 0x456, 0x43d, 0x430,
0x445, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x424, 0x456, 0x43b, 0x456, 0x43f, 0x43f, 0x456, 0x43d, 0x430, 0x445, 0x437,
0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x424, 0x456, 0x43b, 0x456, 0x43f, 0x43f,
0x456, 0x43d, 0x430, 0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445,
0x20, 0x424, 0x435, 0x43d, 0x456, 0x43a, 0x441, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x430, 0x445, 0x20, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x2019, 0x454, 0x440,
0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x437, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x421, 0x435, 0x43d, 0x2d, 0x41f,
0x2019, 0x454, 0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x20, 0x421, 0x435, 0x43d, 0x2d, 0x41f, 0x2019, 0x454,
0x440, 0x20, 0x456, 0x20, 0x41c, 0x456, 0x43a, 0x435, 0x43b, 0x43e, 0x43d, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x41f, 0x456, 0x442, 0x43a,
0x435, 0x440, 0x43d, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41f,
0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x41f, 0x445, 0x435, 0x43d, 0x44c, 0x44f, 0x43d, 0x456,
0x447, 0x430, 0x441, 0x3a, 0x20, 0x41a, 0x438, 0x437, 0x438, 0x43b, 0x43e, 0x440,
0x434, 0x430, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x41a, 0x438, 0x437, 0x438, 0x43b,
0x43e, 0x440, 0x434, 0x430, 0x2c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x43d, 0x438, 0x439, 0x447, 0x430, 0x441, 0x3a, 0x20, 0x41a, 0x438,
0x437, 0x438, 0x43b, 0x43e, 0x440, 0x434, 0x430, 0x2c, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x439, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x420,
0x435, 0x44e, 0x43d, 0x44c, 0x439, 0x43e, 0x43d, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446,
0x456, 0x457, 0x20, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430, 0x437, 0x430, 0x20,
0x441, 0x430, 0x445, 0x430, 0x43b, 0x456, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x430, 0x445,
0x430, 0x43b, 0x456, 0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x430, 0x445, 0x430, 0x43b, 0x456,
0x43d, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x430,
0x43c, 0x430, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x441, 0x430, 0x43c, 0x430, 0x440, 0x441, 0x44c,
0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20,
0x441, 0x430, 0x43c, 0x430, 0x440, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e,
0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x421, 0x430, 0x43c, 0x43e,
0x430, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x456, 0x20, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x421, 0x435, 0x439,
0x448, 0x435, 0x43b, 0x44c, 0x441, 0x44c, 0x43a, 0x438, 0x445, 0x20, 0x41e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x443, 0x20, 0x421, 0x456, 0x43d, 0x433, 0x430, 0x43f, 0x443,
0x440, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x43e, 0x432, 0x438,
0x445, 0x20, 0x41e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x41f, 0x456, 0x432, 0x434, 0x435, 0x43d,
0x43d, 0x430, 0x20, 0x414, 0x436, 0x43e, 0x440, 0x434, 0x436, 0x456, 0x44f, 0x437,
0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x421, 0x443,
0x440, 0x438, 0x43d, 0x430, 0x43c, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x446, 0x456,
0x457, 0x20, 0x421, 0x435, 0x432, 0x430, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432,
0x456, 0x20, 0x422, 0x430, 0x457, 0x442, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x422, 0x430, 0x439, 0x431, 0x435, 0x457,
0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x422,
0x430, 0x439, 0x431, 0x435, 0x457, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x422,
0x430, 0x439, 0x431, 0x435, 0x457, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x422, 0x430, 0x434, 0x436, 0x438, 0x43a, 0x438, 0x441,
0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445,
0x20, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430, 0x443, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440,
0x43e, 0x432, 0x430, 0x445, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x437, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430,
0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e,
0x432, 0x430, 0x445, 0x20, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442,
0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x427, 0x443, 0x443, 0x43a, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x422, 0x443, 0x440,
0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x443, 0x20, 0x422, 0x443, 0x440,
0x43a, 0x43c, 0x435, 0x43d, 0x456, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x435, 0x43d, 0x456,
0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430,
0x445, 0x20, 0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x437, 0x430, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423, 0x440, 0x443, 0x491, 0x432,
0x430, 0x457, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432,
0x20, 0x423, 0x440, 0x443, 0x491, 0x432, 0x430, 0x457, 0x437, 0x430, 0x20, 0x43b,
0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20,
0x432, 0x20, 0x423, 0x440, 0x443, 0x491, 0x432, 0x430, 0x457, 0x437, 0x430, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423, 0x437, 0x431, 0x435,
0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423, 0x437, 0x431, 0x435, 0x43a, 0x438,
0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x432, 0x20, 0x423,
0x437, 0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x430, 0x43d, 0x456, 0x437, 0x430,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20, 0x43e, 0x441,
0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x412, 0x430, 0x43d, 0x443, 0x430,
0x442, 0x443, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x412,
0x430, 0x43d, 0x443, 0x430, 0x442, 0x443, 0x437, 0x430, 0x20, 0x43b, 0x456, 0x442,
0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x412, 0x430,
0x43d, 0x443, 0x430, 0x442, 0x443, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x20, 0x443, 0x20, 0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x435, 0x43b,
0x456, 0x437, 0x430, 0x20, 0x432, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x446, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x432, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x446, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x432, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441,
0x442, 0x43e, 0x446, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d,
0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x441, 0x44c, 0x43a, 0x438,
0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432, 0x43e,
0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x441, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c,
0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x432, 0x43e, 0x43b,
0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c,
0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x446, 0x456, 0x457, 0x20, 0x412, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x456, 0x20, 0x412, 0x435,
0x439, 0x43a, 0x437, 0x430, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d,
0x430, 0x20, 0x43e, 0x441, 0x442, 0x440, 0x43e, 0x432, 0x430, 0x445, 0x20, 0x423,
0x43e, 0x43b, 0x43b, 0x456, 0x441, 0x20, 0x456, 0x20, 0x424, 0x443, 0x442, 0x443,
0x43d, 0x430, 0x437, 0x430, 0x20, 0x44f, 0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x44f,
0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441,
0x43e, 0x43c, 0x437, 0x430, 0x20, 0x44f, 0x43a, 0x443, 0x442, 0x441, 0x44c, 0x43a,
0x438, 0x43c, 0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430,
0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x454, 0x43a, 0x430, 0x442, 0x435, 0x440,
0x438, 0x43d, 0x431, 0x443, 0x440, 0x437, 0x44c, 0x43a, 0x438, 0x43c, 0x20, 0x447,
0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x454, 0x43a, 0x430, 0x442, 0x435,
0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x437, 0x44c, 0x43a, 0x438, 0x43c, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x43d, 0x438, 0x43c, 0x20,
0x447, 0x430, 0x441, 0x43e, 0x43c, 0x437, 0x430, 0x20, 0x454, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x437, 0x44c, 0x43a, 0x438, 0x43c,
0x20, 0x43b, 0x456, 0x442, 0x43d, 0x456, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e,
0x43c, 0x437, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x43d, 0x438, 0x43c, 0x20, 0x447, 0x430, 0x441, 0x43e, 0x43c, 0x20, 0x43d, 0x430,
0x20, 0x42e, 0x43a, 0x43e, 0x43d, 0x456, 0x61, 0x66, 0x67, 0x68, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61, 0x66,
0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6a, 0x75, 0x17e,
0x6e, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f,
0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61,
0x70, 0x61, 0x64, 0x6f, 0x61, 0x66, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x63, 0x6f, 0x6e, 0x61,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x63, 0x6f,
0x6e, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x6d, 0x61, 0x63,
0x6f, 0x6e, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77,
0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x6e, 0x79, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x63, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x6e, 0x79, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x77, 0x75, 0x63, 0x68,
0x6f, 0x64, 0x6e, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65,
0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x77,
0x75, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x79, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f,
0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x68, 0xf3, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65,
0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x68,
0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a,
0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x68, 0xf3, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69,
0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x77,
0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x65, 0x77, 0x6a, 0x65, 0x72, 0x6f, 0x61, 0x6d, 0x65, 0x72,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x41, 0x70, 0x69, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x41,
0x70, 0x69, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x41, 0x70, 0x69,
0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x61,
0x72, 0x61, 0x62, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f,
0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x72, 0x67,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a,
0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x61, 0x72, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x74, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a,
0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x61, 0x77,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x7a, 0x61, 0x70,
0x61, 0x64, 0x6e, 0x79, 0x20, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64,
0x17a, 0x6f, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x79, 0x20, 0x61, 0x77,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x79,
0x20, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63,
0x68, 0x6f, 0x64, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64,
0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x61, 0x77, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x61,
0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6f, 0x61, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x6a, 0x64, 0x17a, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x6a, 0x64,
0x17a, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x6a, 0x64, 0x17a, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x63, 0x6f,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x63, 0x6f, 0x72,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x61, 0x63, 0x6f, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x62,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x161,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x161, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x62, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x6f, 0x6c, 0x69, 0x77, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x62, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x61, 0x70, 0x76, 0x65, 0x72,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x61, 0x70, 0x76,
0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x61, 0x70,
0x76, 0x65, 0x72, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x63,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6c, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6c, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x63, 0x68, 0x69, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10c,
0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x10c, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x48, 0x6f, 0x64, 0x6f, 0x77, 0x6e, 0x65, 0x6a,
0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x10d, 0x61, 0x73, 0x20, 0x4b, 0x6f,
0x6b, 0x6f, 0x73, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70,
0x6f, 0x77, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x10d, 0x61, 0x73, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x77, 0x79,
0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70,
0x6f, 0x77, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x20,
0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75,
0x70, 0x6f, 0x77, 0x6b, 0x75, 0x62, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x6b, 0x75, 0x62, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0x75, 0x62, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x44, 0x61, 0x76, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x20, 0x64,
0xb4, 0x20, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6e, 0x6f,
0x74, 0x69, 0x6d, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x10d, 0x61, 0x73, 0x20, 0x4a, 0x75, 0x74, 0x72, 0x6f, 0x77, 0x6e, 0x65,
0x6a, 0x65, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x4a, 0x75,
0x74, 0x72, 0x6f, 0x77, 0x6e, 0x65, 0x6a, 0x65, 0x20, 0x6b, 0x75, 0x70,
0x79, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x4a,
0x75, 0x74, 0x72, 0x6f, 0x77, 0x6e, 0x65, 0x6a, 0x65, 0x20, 0x6b, 0x75,
0x70, 0x79, 0x65, 0x6b, 0x77, 0x61, 0x64, 0x6f, 0x72, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73,
0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f,
0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64,
0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f,
0x64, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x61, 0x6c, 0x69,
0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61,
0x64, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x61, 0x6c, 0x6b, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17a, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x66, 0x69, 0x64, 0x17a, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x66, 0x69, 0x64, 0x17a, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x72, 0x61, 0x6e,
0x63, 0x6f, 0x73, 0x6b, 0x6f, 0x67, 0x75, 0x79, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x46, 0x72, 0x61,
0x6e, 0x63, 0x6f, 0x73, 0x6b, 0x65, 0x68, 0x6f, 0x20, 0x6a, 0x75, 0x17e,
0x6e, 0x65, 0x68, 0x6f, 0x20, 0x61, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72,
0x6b, 0x74, 0x69, 0x73, 0x6b, 0x65, 0x68, 0x6f, 0x20, 0x74, 0x65, 0x72,
0x69, 0x74, 0x6f, 0x72, 0x69, 0x6a, 0x61, 0x67, 0x61, 0x6c, 0x61, 0x70,
0x61, 0x67, 0x6f, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x67, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20,
0x47, 0x69, 0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x77, 0x79, 0x63, 0x68,
0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75,
0x63, 0x68, 0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68,
0x6f, 0x64, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f,
0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70,
0x61, 0x64, 0x6f, 0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f,
0x67, 0x72, 0xf6, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x67, 0x72, 0xf6, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x50, 0x65, 0x72,
0x73, 0x69, 0x73, 0x6b, 0x65, 0x68, 0x6f, 0x20, 0x67, 0x6f, 0x6c, 0x66,
0x61, 0x68, 0x61, 0x77, 0x61, 0x69, 0x69, 0x73, 0x6b, 0x6f, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x68,
0x61, 0x77, 0x61, 0x69, 0x69, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65,
0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x68, 0x61, 0x77, 0x61,
0x69, 0x69, 0x73, 0x6b, 0x6f, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e,
0x67, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x6e, 0x64, 0x69, 0x73, 0x6b,
0x6f, 0x6f, 0x63, 0x65, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x69, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x73, 0x72, 0x6a, 0x65, 0x64, 0x17a, 0x6f, 0x69,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x77, 0x75, 0x63, 0x68, 0x6f, 0x64, 0x6f, 0x69, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x6b, 0x69, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6f, 0x69, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x69,
0x72, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72,
0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x72, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x69, 0x73, 0x72, 0x61, 0x65,
0x6c, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x6a, 0x61, 0x70, 0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x77, 0x75, 0x63, 0x68, 0x6f,
0x64, 0x6e, 0x6f, 0x6b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x7a, 0x61, 0x70, 0x61, 0x64, 0x6e, 0x6f, 0x6b,
0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x6b, 0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6b,
0x6f, 0x72, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6b, 0x69, 0x72, 0x67, 0x69,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x4c,
0x69, 0x6e, 0x69, 0x6a, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75,
0x70, 0x6f, 0x77, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f, 0x77, 0x65, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x20,
0x6b, 0x75, 0x70, 0x79, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f,
0x77, 0x65, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x20,
0x6b, 0x75, 0x70, 0x79, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x2d, 0x48, 0x6f,
0x77, 0x65, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6d,
0x61, 0x6c, 0x61, 0x6a, 0x7a, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x6d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x77, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x6d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x4d, 0x61,
0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20,
0x6b, 0x75, 0x70, 0x6f, 0x77, 0x6d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69, 0x73,
0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x6d, 0x65, 0x78, 0x69, 0x73, 0x6b, 0x69, 0x20,
0x70, 0x61, 0x63, 0x69, 0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x6d, 0x65, 0x78, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x70, 0x61, 0x63, 0x69,
0x66, 0x69, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x55, 0x6c, 0x61, 0x6e,
0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4d,
0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x77, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6d, 0x79, 0x61, 0x6e, 0x6d,
0x61, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x61, 0x75,
0x72, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x65, 0x70,
0x61, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77,
0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x6b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f,
0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c,
0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f,
0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x73, 0x65, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x73,
0x65, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x66,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61,
0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x6f, 0x77, 0x6f,
0x66, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x6b, 0x69, 0x20,
0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x6e, 0x69, 0x75,
0x65, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20,
0x6b, 0x75, 0x70, 0x79, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d,
0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70,
0x6f, 0x77, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f,
0x77, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4e, 0x6f, 0x77,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79,
0x20, 0x10d, 0x61, 0x73, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69,
0x72, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4f, 0x6d, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x4f, 0x6d,
0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x70, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x61, 0x6c, 0x61, 0x75, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x61, 0x70, 0x75, 0x61, 0x2d, 0x6e, 0x6f, 0x77,
0x6f, 0x67, 0x69, 0x6e, 0x65, 0x6a, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x70, 0x65, 0x72, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x70, 0x65, 0x72, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x66, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x10d, 0x61, 0x73, 0x20, 0x50, 0x68, 0x6f, 0x65,
0x6e, 0x69, 0x78, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70,
0x6f, 0x77, 0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20,
0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x61,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x20,
0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50,
0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x61, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x10d, 0x61, 0x73, 0x20, 0x50, 0x69, 0x74, 0x63,
0x61, 0x69, 0x72, 0x6e, 0x6f, 0x77, 0x79, 0x63, 0x68, 0x20, 0x6b, 0x75,
0x70, 0x6f, 0x77, 0x70, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x73, 0x6b, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x50, 0x6a, 0xf6, 0x6e, 0x67, 0x6a, 0x61, 0x6e,
0x67, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x72, 0x65, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x52, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x73, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73, 0x6b, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x61, 0x63, 0x68, 0x61, 0x6c, 0x69,
0x6e, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61,
0x73, 0x73, 0x61, 0x6d, 0x6f, 0x61, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b,
0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x65, 0x79, 0x63, 0x68,
0x65, 0x6c, 0x6c, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x69,
0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61,
0x73, 0x10d, 0x61, 0x73, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e,
0x73, 0x6b, 0x69, 0x63, 0x68, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x6a,
0x75, 0x17e, 0x6e, 0x6f, 0x67, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x73, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x53, 0x79, 0x6f, 0x77, 0x61,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x61, 0x68, 0x69, 0x74,
0x69, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x61, 0x69, 0x70,
0x65, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x61, 0x69,
0x70, 0x65, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x54, 0x61, 0x69,
0x70, 0x65, 0x68, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x74, 0x61, 0x64, 0x17a, 0x69, 0x6b, 0x73, 0x6b,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6e, 0x67, 0x61,
0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6e, 0x67, 0x61,
0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x74, 0x6f, 0x6e, 0x67, 0x61, 0x73,
0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x63, 0x68, 0x75, 0x75, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x74,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20,
0x10d, 0x61, 0x73, 0x75, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x75,
0x7a, 0x62, 0x65, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x75,
0x7a, 0x62, 0x65, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x75, 0x7a,
0x62, 0x65, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69,
0x20, 0x10d, 0x61, 0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x76, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20,
0x10d, 0x61, 0x73, 0x76, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x73,
0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73,
0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x73,
0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e,
0x79, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e,
0x69, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x6f, 0x6c,
0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x73, 0x6b, 0x69,
0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x57, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x73, 0x6b, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x10d,
0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x79, 0x20, 0x57, 0x61, 0x6b, 0x65,
0x10d, 0x61, 0x73, 0x20, 0x6b, 0x75, 0x70, 0x6f, 0x77, 0x20, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e,
0x61, 0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73,
0x4a, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x6c, 0x11b, 0x74,
0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x10d,
0x61, 0x73, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x73, 0x6b, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x6e, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x4a, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x73, 0x6b,
0x69, 0x20, 0x6c, 0x11b, 0x74, 0x6e, 0x69, 0x20, 0x10d, 0x61, 0x73, 0x59,
0x75, 0x6b, 0x6f, 0x6e, 0x6f, 0x77, 0x79, 0x20, 0x10d, 0x61, 0x73, 0x627,
0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x648, 0x633, 0x637, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc,
0x642, 0x6c1, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc,
0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x6c1, 0x20, 0x679, 0x627, 0x626, 0x645,
0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x6c1,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642,
0x6c1, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20,
0x627, 0x641, 0x631, 0x6cc, 0x642, 0x6c1, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc,
0x20, 0x627, 0x641, 0x631, 0x6cc, 0x642, 0x6c1, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x627, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x627, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627,
0x644, 0x627, 0x633, 0x6a9, 0x627, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626,
0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x645, 0x6cc, 0x632, 0x648, 0x646,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x645, 0x6cc, 0x632, 0x648, 0x646,
0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x633, 0x646, 0x679, 0x631, 0x644, 0x20, 0x679, 0x627,
0x626, 0x645, 0x633, 0x646, 0x679, 0x631, 0x644, 0x20, 0x627, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x646, 0x679,
0x631, 0x644, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x679, 0x627,
0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x627, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc,
0x633, 0x679, 0x631, 0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679,
0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x624, 0x646, 0x679, 0x6cc, 0x646,
0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x624, 0x646, 0x679, 0x6cc, 0x646,
0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x645, 0x627, 0x624, 0x646, 0x679, 0x6cc, 0x646, 0x20, 0x688, 0x6d2,
0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc,
0x633, 0x641, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x633, 0x641,
0x6a9, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x688, 0x6d2, 0x20,
0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x646, 0x6cc,
0x62f, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x646, 0x6cc, 0x62f, 0x631,
0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x627, 0x646, 0x6cc, 0x62f, 0x631, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x67e, 0x6cc, 0x627, 0x20, 0x679, 0x627,
0x626, 0x645, 0x627, 0x6cc, 0x67e, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x67e, 0x6cc,
0x627, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627,
0x626, 0x645, 0x639, 0x631, 0x628, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a,
0x639, 0x631, 0x628, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x639, 0x631, 0x628, 0x20, 0x688, 0x6d2, 0x20,
0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x631, 0x62c,
0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x631,
0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648,
0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646,
0x679, 0x6cc, 0x646, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x645,
0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646,
0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c,
0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633,
0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a,
0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645,
0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x679, 0x644, 0x627, 0x646, 0x679, 0x6a9, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x679, 0x644, 0x627, 0x646, 0x679, 0x6a9, 0x20, 0x627,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x679, 0x644, 0x627, 0x646, 0x679, 0x6a9, 0x20, 0x688, 0x6d2, 0x20, 0x644,
0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x646, 0x679, 0x631,
0x644, 0x20, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x627, 0x20, 0x679,
0x627, 0x626, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20,
0x633, 0x646, 0x679, 0x631, 0x644, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc,
0x644, 0x6cc, 0x646, 0x20, 0x633, 0x646, 0x679, 0x631, 0x644, 0x20, 0x688, 0x6d2,
0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x633,
0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20, 0x633, 0x646, 0x679, 0x631, 0x644,
0x20, 0x648, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645,
0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20, 0x633, 0x646, 0x679,
0x631, 0x644, 0x20, 0x648, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x627, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622,
0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20, 0x633, 0x646, 0x679, 0x631,
0x644, 0x20, 0x648, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x688, 0x6d2, 0x20,
0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x633,
0x679, 0x631, 0x646, 0x20, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x627,
0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc,
0x646, 0x20, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x627, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x633,
0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20, 0x627, 0x6cc, 0x633, 0x679, 0x631,
0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627,
0x626, 0x645, 0x648, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x622, 0x633, 0x679,
0x631, 0x6cc, 0x644, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x633,
0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x627, 0x20, 0x648, 0x6cc, 0x633, 0x679, 0x631,
0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x622, 0x633, 0x679, 0x631, 0x6cc, 0x644, 0x6cc, 0x646, 0x20,
0x648, 0x6cc, 0x633, 0x679, 0x631, 0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627,
0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x630, 0x631, 0x628, 0x627,
0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a,
0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a,
0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x632, 0x648, 0x631, 0x6cc,
0x633, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633, 0x20, 0x6a9, 0x627,
0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f,
0x6cc, 0x634, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x646, 0x6af,
0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x646, 0x6af, 0x644,
0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633,
0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x628, 0x6be, 0x648, 0x679, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x628, 0x648, 0x644, 0x6cc, 0x648, 0x6cc, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x6cc, 0x627,
0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x6cc,
0x627, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x6cc, 0x627, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x631, 0x648, 0x646,
0x626, 0x6cc, 0x20, 0x62f, 0x627, 0x631, 0x627, 0x644, 0x633, 0x644, 0x627, 0x645,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6cc, 0x67e, 0x20, 0x648, 0x631, 0x688,
0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6cc, 0x67e, 0x20, 0x648, 0x631,
0x688, 0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6a9, 0x6cc, 0x67e, 0x20, 0x648, 0x631, 0x688, 0x6cc, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x627, 0x645, 0x648,
0x631, 0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x686, 0x6cc, 0x62a, 0x6be, 0x645, 0x20, 0x679, 0x627, 0x626,
0x645, 0x686, 0x6cc, 0x62a, 0x6be, 0x645, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x6cc, 0x62a, 0x6be,
0x645, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627,
0x626, 0x645, 0x686, 0x644, 0x6cc, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a,
0x686, 0x644, 0x6cc, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x686, 0x644, 0x6cc, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x686, 0x6cc, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x686, 0x6cc, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x6cc, 0x646, 0x6cc, 0x20, 0x688,
0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9,
0x648, 0x626, 0x628, 0x644, 0x633, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645,
0x6a9, 0x648, 0x626, 0x628, 0x644, 0x633, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648, 0x627,
0x626, 0x628, 0x644, 0x633, 0x627, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6a9, 0x631, 0x633, 0x645, 0x633, 0x20, 0x622, 0x626, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648, 0x6a9, 0x648,
0x633, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x6a9,
0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648,
0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x6a9, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc,
0x646, 0x688, 0x632, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646,
0x688, 0x632, 0x20, 0x646, 0x635, 0x641, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6a9, 0x6cc, 0x648, 0x628, 0x627, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6a9, 0x6cc, 0x648, 0x628, 0x627, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x6cc, 0x648, 0x628,
0x627, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627,
0x626, 0x645, 0x688, 0x6cc, 0x648, 0x633, 0x20, 0x679, 0x627, 0x626, 0x645, 0x688,
0x648, 0x645, 0x648, 0x646, 0x679, 0x2d, 0x688, 0x6cc, 0x2019, 0x627, 0x631, 0x648,
0x6cc, 0x644, 0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x634, 0x631, 0x642,
0x6cc, 0x20, 0x62a, 0x6cc, 0x645, 0x648, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x6cc, 0x633, 0x679, 0x631,
0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x6cc, 0x633,
0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627,
0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x6cc, 0x6a9, 0x648, 0x627, 0x688, 0x648,
0x631, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x633, 0x637, 0x20,
0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x648,
0x633, 0x637, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x633,
0x637, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x645,
0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x631,
0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642,
0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642,
0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648,
0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x628, 0x639, 0x6cc, 0x62f, 0x20, 0x645, 0x634, 0x631, 0x642, 0x6cc,
0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a,
0x631, 0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x6a9, 0x627, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x631,
0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x631,
0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631,
0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x627, 0x6a9,
0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688,
0x632, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x627, 0x6a9, 0x20,
0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632,
0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x642, 0x62a, 0x641, 0x627, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622,
0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648,
0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x641, 0x62c, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x62c,
0x6cc, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x641, 0x62c, 0x6cc, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x641, 0x631, 0x6cc, 0x646, 0x686, 0x20, 0x6af, 0x6cc, 0x627, 0x646,
0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631, 0x6cc, 0x646,
0x686, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20, 0x627, 0x648, 0x631, 0x20,
0x627, 0x646, 0x679, 0x627, 0x631, 0x6a9, 0x679, 0x6a9, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x632, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x6af, 0x6cc, 0x645, 0x628, 0x6cc, 0x626, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627,
0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x642, 0x62a, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x62c, 0x644, 0x628, 0x631, 0x679, 0x20, 0x622, 0x626,
0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6af, 0x631,
0x6cc, 0x646, 0x20, 0x648, 0x686, 0x20, 0x6a9, 0x627, 0x20, 0x627, 0x635, 0x644,
0x20, 0x648, 0x642, 0x62a, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6af, 0x631,
0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6af,
0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6af, 0x631,
0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6af,
0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x62e, 0x644, 0x6cc, 0x62c, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x6c1, 0x648,
0x627, 0x626, 0x6cc, 0x20, 0x627, 0x644, 0x6cc, 0x648, 0x679, 0x6cc, 0x626, 0x646,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x648, 0x627, 0x626, 0x6cc, 0x20, 0x627,
0x644, 0x6cc, 0x648, 0x679, 0x6cc, 0x626, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x648, 0x627,
0x626, 0x6cc, 0x20, 0x627, 0x644, 0x6cc, 0x648, 0x679, 0x6cc, 0x626, 0x646, 0x20,
0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645,
0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x6a9, 0x627, 0x646, 0x6af, 0x20, 0x633, 0x645,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x648, 0x648, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6c1, 0x648, 0x648, 0x688, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x648, 0x648, 0x688,
0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x646, 0x62f,
0x648, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x628, 0x62d, 0x631, 0x20, 0x6c1,
0x646, 0x62f, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x646, 0x62f, 0x20, 0x686,
0x6cc, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x633, 0x637, 0x6cc, 0x20,
0x627, 0x646, 0x688, 0x648, 0x646, 0x6cc, 0x634, 0x6cc, 0x627, 0x20, 0x679, 0x627,
0x626, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x627, 0x646, 0x688, 0x648,
0x646, 0x6cc, 0x634, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x63a,
0x631, 0x628, 0x6cc, 0x20, 0x627, 0x646, 0x688, 0x648, 0x646, 0x6cc, 0x634, 0x6cc,
0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642,
0x62a, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627,
0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x631, 0x6a9, 0x62a, 0x633,
0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x631, 0x6a9, 0x62a, 0x633, 0x6a9,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x627, 0x631, 0x6a9, 0x62a, 0x633, 0x6a9, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20,
0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc,
0x644, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x642, 0x62a, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20, 0x688,
0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62c,
0x627, 0x67e, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62c, 0x627, 0x67e,
0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x62c, 0x627, 0x67e, 0x627, 0x646, 0x20, 0x688, 0x6d2, 0x20,
0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x679,
0x631, 0x648, 0x67e, 0x627, 0x624, 0x644, 0x648, 0x648, 0x633, 0x6a9, 0x2d, 0x6a9,
0x6cc, 0x645, 0x686, 0x633, 0x6a9, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e,
0x6cc, 0x679, 0x631, 0x648, 0x67e, 0x627, 0x624, 0x644, 0x648, 0x648, 0x633, 0x6a9,
0x2d, 0x6a9, 0x6cc, 0x645, 0x686, 0x633, 0x6a9, 0x6cc, 0x20, 0x627, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc,
0x679, 0x631, 0x648, 0x67e, 0x627, 0x624, 0x644, 0x648, 0x648, 0x633, 0x6a9, 0x2d,
0x6a9, 0x6cc, 0x645, 0x686, 0x633, 0x6a9, 0x6cc, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x642, 0x632,
0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x642, 0x632, 0x627, 0x62e, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x6a9, 0x648,
0x631, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648, 0x631, 0x6cc,
0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6a9, 0x648, 0x631, 0x6cc, 0x627, 0x20, 0x688, 0x6d2, 0x20, 0x644,
0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648, 0x633, 0x631,
0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x631, 0x6cc, 0x633, 0x646, 0x648,
0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x631,
0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x631, 0x6cc,
0x633, 0x646, 0x648, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x631, 0x63a, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x644, 0x627, 0x626, 0x646, 0x20,
0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645,
0x644, 0x627, 0x631, 0x688, 0x20, 0x6c1, 0x648, 0x648, 0x6d2, 0x20, 0x679, 0x627,
0x626, 0x645, 0x644, 0x627, 0x631, 0x688, 0x20, 0x6c1, 0x648, 0x648, 0x6d2, 0x20,
0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x644, 0x627, 0x631, 0x688, 0x20, 0x6c1, 0x648, 0x648, 0x6d2, 0x20, 0x688,
0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645,
0x6cc, 0x6af, 0x6cc, 0x62f, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6af,
0x627, 0x62f, 0x627, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6af, 0x6cc, 0x62f, 0x646,
0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x644, 0x6cc,
0x634, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x644, 0x62f,
0x6cc, 0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x645, 0x627, 0x631,
0x6a9, 0x6cc, 0x633, 0x633, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x631,
0x634, 0x644, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x679,
0x627, 0x626, 0x645, 0x645, 0x627, 0x631, 0x6cc, 0x634, 0x633, 0x20, 0x679, 0x627,
0x626, 0x645, 0x645, 0x627, 0x631, 0x6cc, 0x634, 0x633, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x631,
0x6cc, 0x634, 0x633, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645,
0x645, 0x627, 0x624, 0x633, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc,
0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20,
0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x646, 0x20,
0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x6a9, 0x633, 0x6cc,
0x6a9, 0x646, 0x20, 0x67e, 0x6cc, 0x633, 0x641, 0x6a9, 0x20, 0x688, 0x6d2, 0x20,
0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x648, 0x644,
0x627, 0x646, 0x20, 0x628, 0x6cc, 0x62a, 0x648, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6cc, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x6cc, 0x62a, 0x648, 0x631,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6cc, 0x648, 0x644, 0x627, 0x646, 0x20, 0x628, 0x6cc, 0x62a, 0x648, 0x631,
0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x633,
0x6a9, 0x648, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x633, 0x6a9, 0x648,
0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x645, 0x627, 0x633, 0x6a9, 0x648, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x645, 0x6cc, 0x627, 0x646, 0x645, 0x627, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x646, 0x627, 0x624, 0x631, 0x648, 0x20, 0x679, 0x627,
0x626, 0x645, 0x646, 0x6cc, 0x67e, 0x627, 0x644, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x646, 0x6cc, 0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x6cc, 0x688, 0x648,
0x646, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x648, 0x20,
0x6a9, 0x6cc, 0x644, 0x6cc, 0x688, 0x648, 0x646, 0x6cc, 0x627, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc,
0x648, 0x20, 0x6a9, 0x6cc, 0x644, 0x6cc, 0x688, 0x648, 0x646, 0x6cc, 0x627, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x648, 0x632,
0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642,
0x62a, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20,
0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645,
0x646, 0x6cc, 0x648, 0x20, 0x641, 0x627, 0x624, 0x646, 0x688, 0x20, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x626, 0x648, 0x20, 0x679, 0x627, 0x626,
0x645, 0x646, 0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x646, 0x627, 0x631,
0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a,
0x646, 0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646,
0x688, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x648, 0x642,
0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20,
0x646, 0x648, 0x631, 0x646, 0x6c1, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642,
0x62a, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20,
0x646, 0x648, 0x631, 0x646, 0x6c1, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x631, 0x646, 0x627,
0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x648, 0x646,
0x6c1, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646,
0x648, 0x648, 0x648, 0x633, 0x6cc, 0x628, 0x631, 0x633, 0x6a9, 0x20, 0x679, 0x627,
0x626, 0x645, 0x646, 0x648, 0x648, 0x648, 0x633, 0x6cc, 0x628, 0x631, 0x633, 0x6a9,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x646, 0x648, 0x648, 0x648, 0x633, 0x6cc, 0x628, 0x631, 0x633, 0x6a9, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x648, 0x645, 0x633,
0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x648, 0x645, 0x633, 0x6a9, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x648, 0x645, 0x633, 0x6a9, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642,
0x62a, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x67e, 0x644, 0x627, 0x624, 0x20, 0x679, 0x627, 0x626,
0x645, 0x67e, 0x627, 0x67e, 0x648, 0x622, 0x20, 0x646, 0x6cc, 0x648, 0x20, 0x6af,
0x646, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x631, 0x627, 0x6af,
0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x6cc,
0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639,
0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x6cc, 0x631, 0x627,
0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a,
0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x67e,
0x6cc, 0x631, 0x648, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x6a9, 0x627,
0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9,
0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x644, 0x67e, 0x627, 0x626, 0x646, 0x20,
0x679, 0x627, 0x626, 0x645, 0x641, 0x644, 0x67e, 0x627, 0x626, 0x646, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641,
0x644, 0x67e, 0x627, 0x626, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x641, 0x6cc, 0x646, 0x6a9, 0x633, 0x20, 0x622, 0x626, 0x644, 0x6cc,
0x646, 0x688, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x6cc, 0x646, 0x679,
0x20, 0x67e, 0x6cc, 0x626, 0x631, 0x20, 0x627, 0x648, 0x631, 0x20, 0x645, 0x6a9,
0x644, 0x6cc, 0x626, 0x648, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x6cc,
0x646, 0x679, 0x20, 0x67e, 0x6cc, 0x626, 0x631, 0x20, 0x627, 0x648, 0x631, 0x20,
0x645, 0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x6cc, 0x646,
0x679, 0x20, 0x67e, 0x6cc, 0x626, 0x631, 0x20, 0x627, 0x648, 0x631, 0x20, 0x645,
0x6a9, 0x644, 0x6cc, 0x626, 0x648, 0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627,
0x626, 0x679, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x679, 0x6a9, 0x627, 0x626,
0x631, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x648, 0x646, 0x627, 0x67e,
0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x627, 0x646, 0x6af, 0x20,
0x6cc, 0x627, 0x646, 0x6af, 0x20, 0x648, 0x642, 0x62a, 0x631, 0x6cc, 0x20, 0x6cc,
0x648, 0x646, 0x6cc, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x631, 0x648, 0x62a,
0x6be, 0x6cc, 0x631, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x633,
0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x62e,
0x627, 0x644, 0x6cc, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x62e, 0x627, 0x644, 0x6cc, 0x646, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x645, 0x627, 0x631,
0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x645, 0x627, 0x631, 0x627, 0x20,
0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x633, 0x645, 0x627, 0x631, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x633, 0x627, 0x645, 0x648, 0x622, 0x20, 0x679, 0x627, 0x626,
0x645, 0x633, 0x627, 0x645, 0x648, 0x622, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x627, 0x645, 0x648, 0x622,
0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627, 0x626,
0x645, 0x633, 0x6cc, 0x634, 0x644, 0x6cc, 0x632, 0x20, 0x679, 0x627, 0x626, 0x645,
0x633, 0x646, 0x6af, 0x627, 0x67e, 0x648, 0x631, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x648, 0x644, 0x645,
0x646, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x679, 0x627,
0x626, 0x645, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20, 0x62c, 0x627, 0x631, 0x62c,
0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x648, 0x631, 0x6cc, 0x646,
0x627, 0x645, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x633, 0x6cc, 0x648,
0x648, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x627, 0x6c1, 0x6cc, 0x62a,
0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x627, 0x626, 0x6cc, 0x20, 0x67e,
0x6cc, 0x626, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x627, 0x626, 0x6cc,
0x20, 0x67e, 0x6cc, 0x626, 0x6cc, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x626, 0x6cc, 0x20, 0x67e,
0x6cc, 0x626, 0x6cc, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20,
0x679, 0x627, 0x626, 0x645, 0x62a, 0x627, 0x62c, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x679, 0x648, 0x6a9, 0x6cc, 0x644,
0x627, 0x624, 0x20, 0x679, 0x627, 0x626, 0x645, 0x679, 0x648, 0x646, 0x6af, 0x627,
0x20, 0x679, 0x627, 0x626, 0x645, 0x679, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x679,
0x648, 0x646, 0x6af, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x686, 0x648, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x631, 0x6a9,
0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x642, 0x62a, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x679, 0x648, 0x627, 0x644,
0x648, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648,
0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x6cc, 0x648, 0x631,
0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a, 0x6cc, 0x648, 0x631, 0x648, 0x6af,
0x648, 0x626, 0x6d2, 0x20, 0x6a9, 0x627, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x627,
0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20, 0x648,
0x642, 0x62a, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9,
0x627, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x642, 0x62a,
0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6a9, 0x627, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x648, 0x627, 0x646, 0x648, 0x622, 0x679, 0x648, 0x20,
0x679, 0x627, 0x626, 0x645, 0x648, 0x627, 0x646, 0x648, 0x622, 0x679, 0x648, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x648, 0x627, 0x646, 0x648, 0x622, 0x679, 0x648, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x648, 0x6cc, 0x646, 0x632, 0x648, 0x626, 0x6cc, 0x644,
0x627, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x648, 0x644, 0x627, 0x62f,
0x6cc, 0x20, 0x648, 0x648, 0x633, 0x62a, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645,
0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x648, 0x633, 0x62a, 0x6a9, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x20, 0x648, 0x648, 0x633, 0x62a, 0x6a9, 0x20,
0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x648, 0x644, 0x6af,
0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x648,
0x644, 0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x627, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x648, 0x644,
0x6af, 0x648, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x648, 0x648, 0x633, 0x679, 0x627, 0x6a9, 0x20, 0x6a9, 0x627,
0x20, 0x648, 0x642, 0x62a, 0x648, 0x6cc, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x627, 0x644, 0x6cc, 0x632,
0x20, 0x627, 0x648, 0x631, 0x20, 0x641, 0x679, 0x648, 0x646, 0x627, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6cc, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6cc, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x20, 0x627, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x6a9,
0x648, 0x62a, 0x633, 0x6a9, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6cc, 0x6a9, 0x627, 0x679, 0x6cc, 0x631, 0x650, 0x646, 0x628, 0x631, 0x6af,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x6a9, 0x627, 0x679, 0x6cc, 0x631, 0x650,
0x646, 0x628, 0x631, 0x6af, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x6a9, 0x627, 0x679, 0x6cc, 0x631,
0x650, 0x646, 0x628, 0x631, 0x6af, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20,
0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x6cc, 0x645, 0x6cc, 0x632, 0x648, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x639, 0x631, 0x628, 0x20, 0x679, 0x627, 0x626, 0x645,
0x639, 0x631, 0x628, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631,
0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645,
0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646,
0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627,
0x626, 0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646,
0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x679, 0x627, 0x626,
0x645, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x622, 0x631, 0x645,
0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626,
0x645, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20,
0x679, 0x627, 0x626, 0x645, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c,
0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627,
0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x646,
0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x679, 0x627, 0x626, 0x645,
0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x628, 0x646,
0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x645, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x628, 0x6be, 0x648, 0x679, 0x627, 0x646, 0x20, 0x679,
0x627, 0x626, 0x645, 0x628, 0x648, 0x644, 0x6cc, 0x648, 0x6cc, 0x627, 0x20, 0x679,
0x627, 0x626, 0x645, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x6cc, 0x627, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x686, 0x644, 0x6cc, 0x20, 0x679, 0x627, 0x626, 0x645, 0x686, 0x644, 0x6cc, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x686, 0x644, 0x6cc, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645,
0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646,
0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x648, 0x644, 0x645,
0x628, 0x6cc, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622,
0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x20,
0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x6cc, 0x6a9, 0x648, 0x627, 0x688, 0x648, 0x631, 0x20,
0x679, 0x627, 0x626, 0x645, 0x648, 0x633, 0x637, 0x6cc, 0x20, 0x6cc, 0x648, 0x631,
0x67e, 0x20, 0x6a9, 0x627, 0x20, 0x648, 0x642, 0x62a, 0x641, 0x627, 0x6a9, 0x20,
0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632,
0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x627, 0x6a9, 0x20, 0x644, 0x6cc, 0x646,
0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x627,
0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646,
0x688, 0x632, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641,
0x631, 0x6cc, 0x646, 0x686, 0x20, 0x6af, 0x6cc, 0x627, 0x646, 0x627, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x632,
0x20, 0x679, 0x627, 0x626, 0x645, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x648, 0x686, 0x20, 0x645,
0x6cc, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x62e, 0x644, 0x6cc, 0x62c, 0x20,
0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645,
0x627, 0x646, 0x688, 0x6cc, 0x627, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20,
0x679, 0x627, 0x626, 0x645, 0x627, 0x6cc, 0x631, 0x627, 0x646, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x633,
0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20, 0x679, 0x627, 0x626, 0x645, 0x627, 0x633,
0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631,
0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20,
0x642, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626,
0x645, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x642, 0x632, 0x627, 0x62e, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6a9, 0x631, 0x63a, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x645, 0x627, 0x644, 0x62f,
0x6cc, 0x67e, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x67e, 0x627, 0x644,
0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x20, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x646, 0x6cc, 0x648, 0x632,
0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x631, 0x646, 0x627, 0x646,
0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x646, 0x6c1, 0x627,
0x20, 0x679, 0x627, 0x626, 0x645, 0x641, 0x631, 0x646, 0x627, 0x646, 0x688, 0x648,
0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648, 0x631, 0x646, 0x6c1, 0x627, 0x20, 0x633,
0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e,
0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e,
0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688,
0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x627, 0x6a9, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e,
0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x679, 0x627, 0x626, 0x645,
0x67e, 0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x679, 0x6cc,
0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x631,
0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627,
0x626, 0x645, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x679, 0x627, 0x626, 0x645, 0x67e,
0x6cc, 0x631, 0x648, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20,
0x679, 0x627, 0x626, 0x645, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x633, 0x645, 0x631,
0x20, 0x679, 0x627, 0x626, 0x645, 0x631, 0x648, 0x62a, 0x6be, 0x6cc, 0x631, 0x627,
0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x648, 0x631, 0x6cc, 0x646, 0x627, 0x645,
0x20, 0x679, 0x627, 0x626, 0x645, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645,
0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x679, 0x627, 0x626,
0x645, 0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x679,
0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6cc, 0x648,
0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x645, 0x631, 0x20, 0x679,
0x627, 0x626, 0x645, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x679, 0x627, 0x626, 0x645, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679, 0x627, 0x626,
0x645, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x645,
0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x6cc, 0x646, 0x632, 0x648, 0x626,
0x6cc, 0x644, 0x627, 0x20, 0x679, 0x627, 0x626, 0x645, 0x648, 0x648, 0x633, 0x679,
0x627, 0x6a9, 0x20, 0x679, 0x627, 0x626, 0x645, 0x626, 0x627, 0x643, 0x631, 0x6d0,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x643, 0x631, 0x6d0, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x627, 0x643, 0x631, 0x649, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x641,
0x63a, 0x627, 0x646, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x626, 0x627,
0x641, 0x631, 0x649, 0x642, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634,
0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x626, 0x627, 0x641, 0x631, 0x649, 0x642,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62c, 0x6d5, 0x646, 0x6c7, 0x628,
0x649, 0x64a, 0x20, 0x626, 0x627, 0x641, 0x631, 0x649, 0x642, 0x627, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x626, 0x627, 0x641,
0x631, 0x649, 0x642, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5,
0x631, 0x628, 0x649, 0x64a, 0x20, 0x626, 0x627, 0x641, 0x631, 0x649, 0x642, 0x627,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x626,
0x627, 0x641, 0x631, 0x649, 0x642, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x644, 0x64a, 0x627,
0x633, 0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x644,
0x64a, 0x627, 0x633, 0x643, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x644,
0x64a, 0x627, 0x633, 0x643, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x644, 0x645, 0x6c7, 0x62a,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x644, 0x645, 0x6c7,
0x62a, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x644, 0x645, 0x6c7, 0x62a,
0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x627, 0x645, 0x627, 0x632, 0x648, 0x646, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x627, 0x645, 0x627, 0x632, 0x648, 0x646, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x627, 0x645, 0x627, 0x632, 0x648, 0x646, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648,
0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627,
0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648,
0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a,
0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5,
0x631, 0x642, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a,
0x627, 0x63a, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x627, 0x63a, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x62a, 0x627, 0x63a, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x649, 0x646, 0x686, 0x20,
0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x62a, 0x649, 0x646, 0x686, 0x20, 0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x62a, 0x649, 0x646, 0x686, 0x20, 0x626, 0x648, 0x643, 0x64a,
0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x627, 0x646, 0x627, 0x62f, 0x649, 0x631, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x646, 0x627, 0x62f, 0x649, 0x631, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x627, 0x646, 0x627, 0x62f, 0x649, 0x631, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x642, 0x62a, 0x627, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x642, 0x62a, 0x627, 0x64a, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x642,
0x62a, 0x627, 0x64a, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x642, 0x62a, 0x6c6, 0x628, 0x6d5, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x642, 0x62a, 0x6c6, 0x628, 0x6d5,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x642, 0x62a, 0x6c6, 0x628, 0x6d5, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x6d5, 0x631, 0x6d5, 0x628, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x6d5, 0x631, 0x6d5, 0x628, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d5, 0x631, 0x6d5,
0x628, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x627, 0x631, 0x6af, 0x6d0, 0x646, 0x62a, 0x649, 0x646, 0x627,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x631, 0x6af, 0x6d0, 0x646,
0x62a, 0x649, 0x646, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x631, 0x6af,
0x6d0, 0x646, 0x62a, 0x649, 0x646, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649,
0x64a, 0x20, 0x626, 0x627, 0x631, 0x6af, 0x6d0, 0x646, 0x62a, 0x649, 0x646, 0x627,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a,
0x20, 0x626, 0x627, 0x631, 0x6af, 0x6d0, 0x646, 0x62a, 0x649, 0x646, 0x627, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x626, 0x627,
0x631, 0x6af, 0x6d0, 0x646, 0x62a, 0x649, 0x646, 0x627, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d5, 0x631,
0x645, 0x6d0, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x6d5, 0x631, 0x645, 0x6d0, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x6d5, 0x631, 0x645, 0x6d0, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a, 0x649, 0x643, 0x20, 0x626, 0x648, 0x643,
0x64a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x62a,
0x644, 0x627, 0x646, 0x62a, 0x649, 0x643, 0x20, 0x626, 0x648, 0x643, 0x64a, 0x627,
0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x62a, 0x644, 0x627, 0x646, 0x62a,
0x649, 0x643, 0x20, 0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627,
0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x648,
0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627,
0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627,
0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x648,
0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20,
0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x63a, 0x6d5, 0x631, 0x628,
0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x649, 0x631, 0x627, 0x644, 0x649,
0x64a, 0x6d5, 0x20, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x63a,
0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649,
0x64a, 0x6d5, 0x20, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x63a,
0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20,
0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a,
0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627,
0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x634, 0x6d5,
0x631, 0x642, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x63a,
0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20,
0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb,
0x633, 0x62a, 0x631, 0x627, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x63a, 0x6d5, 0x631,
0x628, 0x649, 0x64a, 0x20, 0x642, 0x649, 0x633, 0x649, 0x645, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d5,
0x632, 0x6d5, 0x631, 0x628, 0x6d5, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x6d5, 0x632, 0x6d5, 0x631, 0x628, 0x6d5, 0x64a, 0x62c,
0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d5, 0x632, 0x6d5, 0x631, 0x628,
0x6d5, 0x64a, 0x62c, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x632, 0x648, 0x631, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x627, 0x632, 0x648, 0x631, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x627, 0x632, 0x648, 0x631, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x627, 0x6ad, 0x644,
0x627, 0x62f, 0x649, 0x634, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x627,
0x6ad, 0x644, 0x627, 0x62f, 0x649, 0x634, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x627,
0x6ad, 0x644, 0x627, 0x62f, 0x649, 0x634, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x6c7, 0x62a, 0x627, 0x646,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x648, 0x644, 0x649, 0x6cb, 0x649,
0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x649, 0x631, 0x627,
0x632, 0x649, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x628, 0x649, 0x631, 0x627, 0x632, 0x649, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x628, 0x649, 0x631, 0x627, 0x632, 0x649, 0x644, 0x649, 0x64a, 0x6d5,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x628, 0x649, 0x631, 0x6c7, 0x646, 0x649, 0x64a, 0x20, 0x62f, 0x627, 0x631,
0x6c7, 0x633, 0x633, 0x627, 0x644, 0x627, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x64a, 0x6d0, 0x634, 0x649, 0x644, 0x20, 0x62a, 0x6c7, 0x645, 0x634, 0x6c7,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x634, 0x649, 0x644,
0x20, 0x62a, 0x6c7, 0x645, 0x634, 0x6c7, 0x642, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a,
0x6d0, 0x634, 0x649, 0x644, 0x20, 0x62a, 0x6c7, 0x645, 0x634, 0x6c7, 0x642, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x643, 0x627, 0x633, 0x6d0, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686,
0x627, 0x645, 0x648, 0x631, 0x631, 0x648, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686, 0x627,
0x62a, 0x627, 0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686, 0x627, 0x62a,
0x627, 0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686, 0x627, 0x62a, 0x627, 0x645, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x686, 0x649, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686, 0x649,
0x644, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686, 0x649, 0x644, 0x649, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62c,
0x6c7, 0x6ad, 0x6af, 0x648, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62c, 0x6c7,
0x6ad, 0x6af, 0x648, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62c, 0x6c7, 0x6ad, 0x6af, 0x648,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x686, 0x648, 0x64a, 0x628, 0x627, 0x644, 0x633, 0x627, 0x646, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x686, 0x648, 0x64a, 0x628, 0x627, 0x644, 0x633, 0x627,
0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x686, 0x648, 0x64a, 0x628, 0x627, 0x644, 0x633,
0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x631, 0x648, 0x698, 0x62f, 0x6d0, 0x633, 0x62a, 0x6cb, 0x648,
0x20, 0x626, 0x627, 0x631, 0x649, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x643, 0x648, 0x643, 0x6c7, 0x633, 0x20, 0x626, 0x627, 0x631, 0x649, 0x644,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x648, 0x644, 0x648, 0x645,
0x628, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x648,
0x644, 0x648, 0x645, 0x628, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643,
0x648, 0x644, 0x648, 0x645, 0x628, 0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x6c7, 0x643,
0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x643, 0x6c7, 0x643, 0x20, 0x626, 0x627, 0x631, 0x627,
0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x6c7, 0x643,
0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x64a,
0x6d0, 0x631, 0x649, 0x645, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x6c7, 0x628, 0x627, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x643, 0x6c7, 0x628, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643,
0x6c7, 0x628, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x62f, 0x627, 0x6cb, 0x649, 0x633, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x62f, 0x6c7, 0x645, 0x648, 0x646, 0x62a, 0x2d, 0x62f, 0x6c7,
0x631, 0x6cb, 0x649, 0x644, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5,
0x631, 0x642, 0x649, 0x64a, 0x20, 0x62a, 0x649, 0x645, 0x648, 0x631, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d0, 0x633, 0x62a, 0x6d0, 0x631, 0x20, 0x626,
0x627, 0x631, 0x649, 0x644, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e,
0x627, 0x633, 0x643, 0x627, 0x644, 0x64a, 0x627, 0x20, 0x626, 0x627, 0x631, 0x649,
0x644, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d0, 0x633, 0x62a, 0x6d0, 0x631,
0x20, 0x626, 0x627, 0x631, 0x649, 0x644, 0x649, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6d0, 0x643, 0x6cb,
0x627, 0x62f, 0x648, 0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648,
0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7,
0x631, 0x627, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x64a, 0x627,
0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a,
0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x64a, 0x627, 0x6cb,
0x631, 0x648, 0x67e, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642,
0x649, 0x64a, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a,
0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649,
0x64a, 0x20, 0x64a, 0x627, 0x6cb, 0x631, 0x648, 0x67e, 0x627, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x64a, 0x627, 0x6cb, 0x631,
0x648, 0x67e, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x641, 0x627, 0x644, 0x643, 0x644, 0x627, 0x646, 0x62f,
0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x641, 0x627, 0x644, 0x643, 0x644, 0x627, 0x646, 0x62f,
0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x641, 0x627, 0x644, 0x643, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x626,
0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x649, 0x62c,
0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x649, 0x62c, 0x649, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x641, 0x649, 0x62c, 0x649, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x649, 0x631, 0x627,
0x646, 0x633, 0x649, 0x64a, 0x6d5, 0x6af, 0x6d5, 0x20, 0x642, 0x627, 0x631, 0x627,
0x634, 0x644, 0x649, 0x642, 0x20, 0x6af, 0x649, 0x6cb, 0x649, 0x64a, 0x627, 0x646,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x649, 0x631, 0x627, 0x646,
0x633, 0x649, 0x64a, 0x6d5, 0x6af, 0x6d5, 0x20, 0x642, 0x627, 0x631, 0x627, 0x634,
0x644, 0x649, 0x642, 0x20, 0x62c, 0x6d5, 0x646, 0x6c7, 0x628, 0x649, 0x64a, 0x20,
0x6cb, 0x6d5, 0x20, 0x626, 0x627, 0x646, 0x62a, 0x627, 0x631, 0x643, 0x62a, 0x649,
0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af, 0x627, 0x644, 0x627,
0x67e, 0x627, 0x6af, 0x648, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af,
0x627, 0x645, 0x628, 0x649, 0x64a, 0x6d0, 0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x6af, 0x649, 0x631, 0x6c7, 0x632, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x6af, 0x649, 0x631, 0x6c7, 0x632, 0x649, 0x64a, 0x6d5, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x6af, 0x649, 0x631, 0x6c7, 0x632, 0x649, 0x64a, 0x6d5, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x6af, 0x649, 0x644, 0x628, 0x6d0, 0x631, 0x62a, 0x20, 0x626, 0x627, 0x631, 0x627,
0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af,
0x649, 0x631, 0x649, 0x646, 0x6cb, 0x649, 0x686, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x6af, 0x649, 0x631, 0x6d0,
0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634,
0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x6af, 0x649, 0x631, 0x6d0, 0x646, 0x644,
0x627, 0x646, 0x62f, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649,
0x64a, 0x20, 0x6af, 0x649, 0x631, 0x6d0, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x6af, 0x649, 0x631, 0x6d0, 0x646,
0x644, 0x627, 0x646, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5,
0x631, 0x628, 0x649, 0x64a, 0x20, 0x6af, 0x649, 0x631, 0x6d0, 0x646, 0x644, 0x627,
0x646, 0x62f, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a,
0x20, 0x6af, 0x649, 0x631, 0x6d0, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af,
0x6c7, 0x626, 0x627, 0x645, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6af, 0x6c7, 0x644, 0x641,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x6be, 0x627, 0x6cb, 0x627, 0x64a, 0x2d, 0x626, 0x627,
0x644, 0x6d0, 0x64a, 0x6c7, 0x62a, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6be,
0x627, 0x6cb, 0x627, 0x64a, 0x2d, 0x626, 0x627, 0x644, 0x6d0, 0x64a, 0x6c7, 0x62a,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x6be, 0x627, 0x6cb, 0x627, 0x64a, 0x2d, 0x626, 0x627,
0x644, 0x6d0, 0x64a, 0x6c7, 0x62a, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x64a, 0x627, 0x6ad, 0x6af, 0x627,
0x6ad, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x64a, 0x627, 0x6ad, 0x6af,
0x627, 0x6ad, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x64a, 0x627, 0x6ad, 0x6af, 0x627,
0x6ad, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x62e, 0x648, 0x6cb, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x62e, 0x648, 0x6cb, 0x62f, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62e, 0x648, 0x6cb, 0x62f,
0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x6be, 0x649, 0x646, 0x62f, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x6be, 0x649, 0x646, 0x62f, 0x649, 0x20, 0x626, 0x648, 0x643, 0x64a,
0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6be, 0x649, 0x646, 0x62f,
0x649, 0x20, 0x686, 0x649, 0x646, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x648, 0x62a, 0x62a, 0x6c7, 0x631, 0x627, 0x20, 0x6be, 0x649, 0x646, 0x62f,
0x648, 0x646, 0x6d0, 0x632, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x634, 0x6d5, 0x631, 0x642, 0x649, 0x64a, 0x20, 0x6be, 0x649, 0x646, 0x62f,
0x648, 0x646, 0x6d0, 0x632, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649, 0x64a, 0x20, 0x6be, 0x649, 0x646, 0x62f,
0x648, 0x646, 0x6d0, 0x632, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x649, 0x631, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x649, 0x631, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649, 0x631,
0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x649, 0x631, 0x643, 0x6c7, 0x62a, 0x633, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649, 0x631, 0x643, 0x6c7, 0x62a, 0x633,
0x643, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649, 0x631, 0x643, 0x6c7, 0x62a, 0x633,
0x643, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x649, 0x633, 0x631, 0x627, 0x626, 0x649, 0x644, 0x649, 0x64a,
0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649, 0x633, 0x631, 0x627,
0x626, 0x649, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x649,
0x633, 0x631, 0x627, 0x626, 0x649, 0x644, 0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x627,
0x67e, 0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x64a, 0x627, 0x67e, 0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x64a, 0x627, 0x67e, 0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x6d0, 0x62a,
0x631, 0x648, 0x67e, 0x627, 0x6cb, 0x644, 0x648, 0x6cb, 0x633, 0x643, 0x2d, 0x643,
0x627, 0x645, 0x686, 0x627, 0x62a, 0x643, 0x633, 0x643, 0x649, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x67e, 0x6d0, 0x62a, 0x631, 0x648, 0x67e, 0x627, 0x6cb, 0x644,
0x648, 0x6cb, 0x633, 0x643, 0x2d, 0x643, 0x627, 0x645, 0x686, 0x627, 0x62a, 0x643,
0x633, 0x643, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x6d0, 0x62a, 0x631, 0x648,
0x67e, 0x627, 0x6cb, 0x644, 0x648, 0x6cb, 0x633, 0x643, 0x2d, 0x643, 0x627, 0x645,
0x686, 0x627, 0x62a, 0x643, 0x633, 0x643, 0x649, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x6d5, 0x631, 0x642,
0x649, 0x64a, 0x20, 0x642, 0x627, 0x632, 0x627, 0x642, 0x649, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x63a, 0x6d5, 0x631, 0x628, 0x649,
0x64a, 0x20, 0x642, 0x627, 0x632, 0x627, 0x642, 0x649, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x648, 0x631, 0x649, 0x64a, 0x6d5,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x648, 0x631, 0x649, 0x64a, 0x6d5,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x643, 0x648, 0x631, 0x649, 0x64a, 0x6d5, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643,
0x648, 0x633, 0x631, 0x627, 0x626, 0x6d0, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x643, 0x649, 0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x649, 0x631, 0x627, 0x633, 0x646,
0x648, 0x64a, 0x627, 0x631, 0x633, 0x643, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x643, 0x649,
0x631, 0x627, 0x633, 0x646, 0x648, 0x64a, 0x627, 0x631, 0x633, 0x643, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x642,
0x649, 0x631, 0x63a, 0x649, 0x632, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x633, 0x649, 0x631, 0x649, 0x20, 0x644, 0x627, 0x646,
0x643, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x644, 0x627, 0x64a, 0x646,
0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x2d, 0x62e, 0x627, 0x64a,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x2d, 0x62e,
0x627, 0x64a, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x644, 0x648, 0x631, 0x62f, 0x2d, 0x62e,
0x627, 0x64a, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x645, 0x6d0, 0x646, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x627, 0x6cb, 0x645, 0x6d0, 0x646, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x627, 0x6cb, 0x645, 0x6d0, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x6af, 0x627,
0x62f, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x6af,
0x627, 0x62f, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x6af, 0x627,
0x62f, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x644, 0x627, 0x64a, 0x634, 0x649, 0x64a,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x644, 0x62f, 0x649,
0x6cb, 0x6d0, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x631, 0x643,
0x649, 0x632, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x631, 0x634,
0x627, 0x644, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627, 0x6cb, 0x631, 0x649, 0x62a,
0x649, 0x626, 0x6c7, 0x633, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x627,
0x6cb, 0x631, 0x649, 0x62a, 0x649, 0x626, 0x6c7, 0x633, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x645, 0x627, 0x6cb, 0x631, 0x649, 0x62a, 0x649, 0x626, 0x6c7, 0x633, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645,
0x627, 0x6cb, 0x633, 0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645,
0x6d0, 0x643, 0x633, 0x649, 0x643, 0x627, 0x20, 0x62a, 0x649, 0x646, 0x686, 0x20,
0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x645, 0x6d0, 0x643, 0x633, 0x649, 0x643, 0x627, 0x20, 0x62a, 0x649, 0x646, 0x686,
0x20, 0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645,
0x6d0, 0x643, 0x633, 0x649, 0x643, 0x627, 0x20, 0x62a, 0x649, 0x646, 0x686, 0x20,
0x626, 0x648, 0x643, 0x64a, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6c7, 0x644, 0x627, 0x646,
0x628, 0x627, 0x62a, 0x648, 0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x6c7, 0x644, 0x627, 0x646, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x626, 0x6c7, 0x644, 0x627, 0x646, 0x628, 0x627, 0x62a, 0x648, 0x631, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x645, 0x648, 0x633, 0x643, 0x6cb, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x645, 0x648, 0x633, 0x643, 0x6cb, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x645, 0x648,
0x633, 0x643, 0x6cb, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x628, 0x649, 0x631, 0x645, 0x627, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x646, 0x627, 0x6cb, 0x631, 0x6c7, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x646, 0x6d0, 0x67e, 0x627, 0x644, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x643, 0x627, 0x644, 0x6d0, 0x62f,
0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a,
0x6d0, 0x6ad, 0x649, 0x20, 0x643, 0x627, 0x644, 0x6d0, 0x62f, 0x648, 0x646, 0x649,
0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x643,
0x627, 0x644, 0x6d0, 0x62f, 0x648, 0x646, 0x649, 0x64a, 0x6d5, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0,
0x6ad, 0x649, 0x20, 0x632, 0x6d0, 0x644, 0x627, 0x646, 0x62f, 0x649, 0x64a, 0x6d5,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x632,
0x6d0, 0x644, 0x627, 0x646, 0x62f, 0x649, 0x64a, 0x6d5, 0x20, 0x626, 0x6c6, 0x644,
0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x632, 0x6d0, 0x644, 0x627, 0x646, 0x62f, 0x649,
0x64a, 0x6d5, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x646, 0x649, 0x6cb, 0x641, 0x648, 0x626, 0x6c7, 0x646, 0x644,
0x627, 0x646, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646, 0x649, 0x6cb,
0x641, 0x648, 0x626, 0x6c7, 0x646, 0x644, 0x627, 0x646, 0x62f, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x646, 0x649, 0x6cb, 0x641, 0x648, 0x626, 0x6c7, 0x646, 0x644, 0x627, 0x646,
0x62f, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x646, 0x649, 0x64a, 0x6c7, 0x626, 0x6d0, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x626, 0x627,
0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x646, 0x648, 0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x626, 0x627, 0x631,
0x627, 0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646, 0x648,
0x631, 0x641, 0x648, 0x644, 0x643, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644,
0x649, 0x631, 0x649, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x641, 0x6d0, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x648,
0x2d, 0x646, 0x648, 0x631, 0x648, 0x646, 0x62e, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x641, 0x6d0, 0x631, 0x646, 0x627, 0x646, 0x62f, 0x648, 0x2d, 0x646,
0x648, 0x631, 0x648, 0x646, 0x62e, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x6d0,
0x631, 0x646, 0x627, 0x646, 0x62f, 0x648, 0x2d, 0x646, 0x648, 0x631, 0x648, 0x646,
0x62e, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x634, 0x649, 0x645, 0x627, 0x644, 0x649, 0x64a, 0x20, 0x645,
0x627, 0x631, 0x649, 0x64a, 0x627, 0x646, 0x627, 0x20, 0x626, 0x627, 0x631, 0x627,
0x644, 0x644, 0x649, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646,
0x648, 0x6cb, 0x648, 0x633, 0x649, 0x628, 0x649, 0x631, 0x633, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x646, 0x648, 0x6cb, 0x648, 0x633, 0x649, 0x628, 0x649,
0x631, 0x633, 0x643, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x646, 0x648, 0x6cb, 0x648, 0x633,
0x649, 0x628, 0x649, 0x631, 0x633, 0x643, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648, 0x645, 0x633, 0x643,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x648, 0x645, 0x633, 0x643, 0x20,
0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x626, 0x648, 0x645, 0x633, 0x643, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x627, 0x643,
0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e,
0x627, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e,
0x627, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x627, 0x644, 0x627,
0x6cb, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x627, 0x67e, 0x6c7, 0x626,
0x627, 0x20, 0x64a, 0x6d0, 0x6ad, 0x649, 0x20, 0x6af, 0x649, 0x6cb, 0x649, 0x646,
0x6d0, 0x64a, 0x6d5, 0x633, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e,
0x627, 0x631, 0x627, 0x6af, 0x6cb, 0x627, 0x64a, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x67e, 0x627, 0x631, 0x627, 0x6af, 0x6cb, 0x627, 0x64a, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x67e, 0x627, 0x631, 0x627, 0x6af, 0x6cb, 0x627, 0x64a, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x6d0,
0x631, 0x6c7, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x67e, 0x6d0, 0x631, 0x6c7,
0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x67e, 0x6d0, 0x631, 0x6c7, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x649, 0x644,
0x649, 0x67e, 0x67e, 0x649, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641,
0x649, 0x644, 0x649, 0x67e, 0x67e, 0x649, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641,
0x649, 0x644, 0x649, 0x67e, 0x67e, 0x649, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x641, 0x6d0, 0x646, 0x649,
0x643, 0x633, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644, 0x649, 0x631, 0x649,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x627, 0x64a, 0x646, 0x649, 0x62a,
0x20, 0x67e, 0x649, 0x626, 0x6d0, 0x631, 0x20, 0x6cb, 0x6d5, 0x20, 0x645, 0x649,
0x643, 0x6d0, 0x644, 0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633,
0x627, 0x64a, 0x646, 0x649, 0x62a, 0x20, 0x67e, 0x649, 0x626, 0x6d0, 0x631, 0x20,
0x6cb, 0x6d5, 0x20, 0x645, 0x649, 0x643, 0x6d0, 0x644, 0x648, 0x646, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x633, 0x627, 0x64a, 0x646, 0x649, 0x62a, 0x20, 0x67e, 0x649, 0x626,
0x6d0, 0x631, 0x20, 0x6cb, 0x6d5, 0x20, 0x645, 0x649, 0x643, 0x6d0, 0x644, 0x648,
0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x67e, 0x649, 0x62a, 0x643, 0x627, 0x64a, 0x631, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x67e, 0x648, 0x646, 0x67e, 0x6d0, 0x64a, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x642, 0x649, 0x632, 0x649, 0x644, 0x626, 0x648, 0x631, 0x62f,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x642, 0x649, 0x632, 0x649, 0x644,
0x626, 0x648, 0x631, 0x62f, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x642, 0x649, 0x632,
0x649, 0x644, 0x626, 0x648, 0x631, 0x62f, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x631, 0x6d0, 0x626, 0x648,
0x646, 0x649, 0x64a, 0x648, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x631,
0x648, 0x62a, 0x6d0, 0x631, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633,
0x627, 0x62e, 0x627, 0x631, 0x649, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x633, 0x627, 0x62e, 0x627, 0x631, 0x649, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633,
0x627, 0x62e, 0x627, 0x631, 0x649, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x627, 0x631,
0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x627, 0x631,
0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x627, 0x645, 0x627, 0x631, 0x627, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x633, 0x627, 0x645, 0x648, 0x626, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x633, 0x627, 0x645, 0x648, 0x626, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x6d5,
0x645, 0x6d5, 0x631, 0x642, 0x6d5, 0x646, 0x62a, 0x20, 0x64a, 0x627, 0x632, 0x644,
0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x6d0, 0x64a, 0x634,
0x6d0, 0x644, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x649, 0x646, 0x6af,
0x627, 0x67e, 0x648, 0x631, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x648,
0x644, 0x648, 0x645, 0x648, 0x646, 0x20, 0x626, 0x627, 0x631, 0x627, 0x644, 0x644,
0x649, 0x631, 0x649, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62c, 0x6d5, 0x646,
0x6c7, 0x628, 0x649, 0x64a, 0x20, 0x62c, 0x648, 0x631, 0x62c, 0x649, 0x64a, 0x6d5,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x633, 0x6c7, 0x631, 0x649, 0x646, 0x627,
0x645, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x634, 0x648, 0x6cb, 0x627, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x627, 0x64a, 0x62a, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x62a, 0x6d5, 0x64a, 0x628, 0x6d0, 0x64a, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x62a, 0x6d5, 0x64a, 0x628, 0x6d0, 0x64a, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x62a, 0x6d5, 0x64a, 0x628, 0x6d0, 0x64a, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x627, 0x62c,
0x649, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x62a, 0x648, 0x643, 0x6d0, 0x644, 0x627, 0x6cb, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x62a, 0x648, 0x646, 0x6af, 0x627, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x648,
0x646, 0x6af, 0x627, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x686, 0x6c7, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x62a, 0x6c8, 0x631, 0x643, 0x645, 0x6d5, 0x646, 0x649, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a, 0x6c8, 0x631, 0x643, 0x645,
0x6d5, 0x646, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686,
0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x62a,
0x6c8, 0x631, 0x643, 0x645, 0x6d5, 0x646, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x62a, 0x6c7, 0x6cb, 0x627, 0x644, 0x6c7, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x626, 0x6c7, 0x631, 0x6c7, 0x6af, 0x6cb, 0x627, 0x64a, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x6c7, 0x631, 0x6c7, 0x6af, 0x6cb, 0x627, 0x64a, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x626, 0x6c7, 0x631, 0x6c7, 0x6af, 0x6cb, 0x627, 0x64a, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626,
0x6c6, 0x632, 0x628, 0x6d0, 0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x626, 0x6c6, 0x632, 0x628, 0x6d0, 0x643, 0x649, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649,
0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x626, 0x6c6, 0x632, 0x628, 0x6d0,
0x643, 0x649, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649,
0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x627, 0x646, 0x6c7, 0x626,
0x627, 0x62a, 0x6c7, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x627, 0x646,
0x6c7, 0x626, 0x627, 0x62a, 0x6c7, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645,
0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x627, 0x646,
0x6c7, 0x626, 0x627, 0x62a, 0x6c7, 0x20, 0x64a, 0x627, 0x632, 0x644, 0x649, 0x642,
0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x6d0, 0x646, 0x6d0, 0x632, 0x6c7,
0x626, 0x6d0, 0x644, 0x627, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x649,
0x644, 0x627, 0x62f, 0x649, 0x6cb, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x649, 0x644, 0x627, 0x62f, 0x649, 0x6cb, 0x648,
0x633, 0x62a, 0x648, 0x643, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644,
0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x649, 0x644, 0x627,
0x62f, 0x649, 0x6cb, 0x648, 0x633, 0x62a, 0x648, 0x643, 0x20, 0x64a, 0x627, 0x632,
0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x648, 0x644,
0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x6cb, 0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x626, 0x6c6,
0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a,
0x649, 0x6cb, 0x648, 0x644, 0x6af, 0x627, 0x6af, 0x631, 0x627, 0x62f, 0x20, 0x64a,
0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x6cb,
0x6d0, 0x64a, 0x643, 0x20, 0x626, 0x627, 0x631, 0x649, 0x644, 0x649, 0x20, 0x6cb,
0x627, 0x642, 0x62a, 0x649, 0x6cb, 0x627, 0x644, 0x644, 0x649, 0x633, 0x20, 0x6cb,
0x6d5, 0x20, 0x641, 0x6c7, 0x62a, 0x6c7, 0x646, 0x627, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x64a, 0x627, 0x643, 0x6c7, 0x62a, 0x633, 0x643, 0x20, 0x6cb, 0x627,
0x642, 0x62a, 0x649, 0x64a, 0x627, 0x643, 0x6c7, 0x62a, 0x633, 0x643, 0x20, 0x626,
0x6c6, 0x644, 0x686, 0x6d5, 0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642,
0x62a, 0x649, 0x64a, 0x627, 0x643, 0x6c7, 0x62a, 0x633, 0x643, 0x20, 0x64a, 0x627,
0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0,
0x643, 0x627, 0x62a, 0x6d0, 0x631, 0x649, 0x646, 0x628, 0x6c7, 0x631, 0x6af, 0x20,
0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0, 0x643, 0x627, 0x62a, 0x6d0, 0x631,
0x649, 0x646, 0x628, 0x6c7, 0x631, 0x6af, 0x20, 0x626, 0x6c6, 0x644, 0x686, 0x6d5,
0x645, 0x644, 0x649, 0x643, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649, 0x64a, 0x6d0,
0x643, 0x627, 0x62a, 0x6d0, 0x631, 0x649, 0x646, 0x628, 0x6c7, 0x631, 0x6af, 0x20,
0x64a, 0x627, 0x632, 0x644, 0x649, 0x642, 0x20, 0x6cb, 0x627, 0x642, 0x62a, 0x649,
0x41, 0x66, 0x67, 0x2bb, 0x6f, 0x6e, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69,
0x79, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4a, 0x61,
0x6e, 0x75, 0x62, 0x69, 0x79, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47,
0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x6c, 0x79, 0x61, 0x73, 0x6b,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x6c, 0x79, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x6c, 0x79, 0x61, 0x73, 0x6b, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x6b, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x6b, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72,
0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61,
0x7a, 0x69, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69,
0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x41,
0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61,
0x72, 0x71, 0x69, 0x79, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x54, 0x6f, 0x67, 0x2018, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x20, 0x28,
0x41, 0x51, 0x53, 0x48, 0x29, 0x54, 0x6f, 0x67, 0x2018, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x20, 0x28, 0x41, 0x51, 0x53, 0x48, 0x29, 0x54, 0x6f, 0x67, 0x2018, 0x20,
0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x20,
0x28, 0x41, 0x51, 0x53, 0x48, 0x29, 0x54, 0x69, 0x6e, 0x63, 0x68, 0x20,
0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x54, 0x69, 0x6e, 0x63, 0x68, 0x20, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x54, 0x69, 0x6e, 0x63, 0x68, 0x20, 0x6f, 0x6b, 0x65,
0x61, 0x6e, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x41, 0x70, 0x69, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x70, 0x69, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x61, 0x75, 0x64, 0x69, 0x79, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62,
0x69, 0x73, 0x74, 0x6f, 0x6e, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x61, 0x75, 0x64, 0x69, 0x79, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62,
0x69, 0x73, 0x74, 0x6f, 0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x61, 0x75,
0x64, 0x69, 0x79, 0x61, 0x20, 0x41, 0x72, 0x61, 0x62, 0x69, 0x73, 0x74,
0x6f, 0x6e, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x72,
0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2bb,
0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x41, 0x72, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x6f,
0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x72, 0x6d, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x72, 0x6d, 0x61,
0x6e, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a,
0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61,
0x7a, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69,
0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x67, 0x2018, 0x61, 0x72,
0x62, 0x69, 0x79, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72,
0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x79, 0x61, 0x20, 0x67, 0x2018, 0x61, 0x72, 0x62, 0x69, 0x79,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20,
0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x67,
0x2018, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69,
0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69,
0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x41,
0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61,
0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61,
0x72, 0x62, 0x69, 0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61, 0x72, 0x62, 0x69,
0x79, 0x20, 0x41, 0x76, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4f, 0x7a, 0x61, 0x72, 0x62, 0x61, 0x79, 0x6a, 0x6f, 0x6e, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4f, 0x7a, 0x61, 0x72, 0x62, 0x61, 0x79, 0x6a,
0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4f, 0x7a, 0x61, 0x72, 0x62, 0x61, 0x79,
0x6a, 0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x41, 0x7a,
0x6f, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x41, 0x7a, 0x6f, 0x72, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c,
0x61, 0x72, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73,
0x68, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x79,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x72, 0x61, 0x7a, 0x69,
0x6c, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x72,
0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x72,
0x61, 0x7a, 0x69, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x79, 0x2d, 0x44, 0x6f, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x6f, 0x6d,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x61, 0x62, 0x6f, 0x2d, 0x56,
0x65, 0x72, 0x64, 0x65, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x61,
0x62, 0x6f, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b,
0x61, 0x62, 0x6f, 0x2d, 0x56, 0x65, 0x72, 0x64, 0x65, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x61,
0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x61, 0x74,
0x65, 0x6d, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x61, 0x74,
0x65, 0x6d, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x61, 0x74, 0x65, 0x6d, 0x20,
0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43,
0x68, 0x69, 0x6c, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68,
0x69, 0x6c, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x69, 0x6c, 0x69, 0x20,
0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x58,
0x69, 0x74, 0x6f, 0x79, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x58, 0x69,
0x74, 0x6f, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x58, 0x69, 0x74, 0x6f, 0x79, 0x20,
0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x43,
0x68, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x43, 0x68, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x6f, 0x79, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x52, 0x6f, 0x6a, 0x64, 0x65, 0x73, 0x74, 0x76, 0x6f, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b,
0x6f, 0x6b, 0x6f, 0x73, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f, 0x6c, 0x75, 0x6d,
0x62, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x75, 0x6b, 0x20, 0x6f,
0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4b, 0x75, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4b, 0x75, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x79, 0x61, 0x72, 0x69, 0x6d, 0x20, 0x79,
0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x75,
0x62, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x75, 0x62, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x44, 0x65, 0x79, 0x76, 0x69,
0x73, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x44, 0x79, 0x75, 0x6d, 0x6f,
0x6e, 0x2d, 0x64, 0x2019, 0x59, 0x75, 0x72, 0x76, 0x69, 0x6c, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20,
0x54, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50,
0x61, 0x73, 0x78, 0x61, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x61, 0x73, 0x78, 0x61, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x45, 0x6b, 0x76, 0x61, 0x64, 0x6f, 0x72,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a,
0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79,
0x20, 0x59, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d,
0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x59,
0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69,
0x79, 0x20, 0x59, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x61, 0x6c,
0x69, 0x6e, 0x69, 0x6e, 0x67, 0x72, 0x61, 0x64, 0x20, 0x76, 0x61, 0x20,
0x4d, 0x69, 0x6e, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47,
0x2018, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61, 0x72,
0x62, 0x69, 0x79, 0x20, 0x59, 0x65, 0x76, 0x72, 0x6f, 0x70, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x47, 0x2018, 0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x59, 0x65,
0x76, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e,
0x64, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64,
0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x46, 0x6f, 0x6c, 0x6b, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x6f, 0x72, 0x6f,
0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x69,
0x6a, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x46, 0x72, 0x61, 0x6e, 0x73, 0x75, 0x7a, 0x20, 0x47, 0x76,
0x69, 0x61, 0x6e, 0x61, 0x73, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x46, 0x72, 0x61, 0x6e, 0x73, 0x75, 0x7a, 0x20, 0x4a, 0x61, 0x6e, 0x75,
0x62, 0x69, 0x79, 0x20, 0x68, 0x75, 0x64, 0x75, 0x64, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x76, 0x61, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x6b, 0x74,
0x69, 0x6b, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x61, 0x6c,
0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x47, 0x61, 0x6d, 0x62, 0x79, 0x65, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x47, 0x72, 0x75, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x47, 0x72, 0x75, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47,
0x72, 0x75, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x47, 0x72, 0x69, 0x6e, 0x76, 0x69, 0x63, 0x68,
0x20, 0x6f, 0x2018, 0x72, 0x74, 0x61, 0x63, 0x68, 0x61, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x47,
0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20,
0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79, 0x20, 0x47, 0x72,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f,
0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61,
0x72, 0x62, 0x69, 0x79, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018,
0x61, 0x72, 0x62, 0x69, 0x79, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2018, 0x61, 0x72,
0x62, 0x69, 0x79, 0x20, 0x47, 0x72, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x46, 0x6f, 0x72, 0x73, 0x20, 0x6b, 0x6f, 0x2018, 0x72,
0x66, 0x61, 0x7a, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x61, 0x79, 0x61, 0x6e,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x61, 0x76, 0x61, 0x79,
0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x47, 0x61, 0x76, 0x61, 0x79, 0x69, 0x2d, 0x61, 0x6c, 0x65, 0x75,
0x74, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x47, 0x61, 0x76, 0x61, 0x79, 0x69, 0x2d, 0x61,
0x6c, 0x65, 0x75, 0x74, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x47, 0x6f, 0x6e, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x79,
0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x58, 0x6f,
0x76, 0x64, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x58, 0x6f, 0x76, 0x64,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x58, 0x6f, 0x76, 0x64, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x48, 0x69, 0x6e, 0x64, 0x69,
0x73, 0x74, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x48, 0x69, 0x6e, 0x64, 0x20,
0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x48, 0x69, 0x6e, 0x64, 0x69, 0x78, 0x69, 0x74, 0x6f, 0x79, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x6b, 0x61, 0x7a, 0x69, 0x79,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79, 0x61, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72, 0x71, 0x69, 0x79,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79, 0x61, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2bb, 0x61, 0x72, 0x62, 0x69, 0x79,
0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x79, 0x61, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x45, 0x72, 0x6f, 0x6e, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x45, 0x72, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x45, 0x72,
0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x79, 0x6f, 0x7a,
0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x49, 0x73, 0x72, 0x6f,
0x69, 0x6c, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x49, 0x73, 0x72, 0x6f,
0x69, 0x6c, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x49, 0x73, 0x72, 0x6f, 0x69, 0x6c, 0x20,
0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59,
0x61, 0x70, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x59, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x59, 0x61, 0x70, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a,
0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x68, 0x61, 0x72,
0x71, 0x69, 0x79, 0x20, 0x51, 0x6f, 0x7a, 0x6f, 0x67, 0x2bb, 0x69, 0x73,
0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x47, 0x2bb, 0x61,
0x72, 0x62, 0x69, 0x79, 0x20, 0x51, 0x6f, 0x7a, 0x6f, 0x67, 0x2bb, 0x69,
0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f,
0x72, 0x65, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f,
0x72, 0x65, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x6f, 0x72, 0x65, 0x79,
0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x51, 0x69, 0x72, 0x67, 0x2bb,
0x69, 0x7a, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4c, 0x61, 0x79, 0x6e, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4c, 0x6f, 0x72, 0x64,
0x2d, 0x58, 0x61, 0x75, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4c, 0x6f,
0x72, 0x64, 0x2d, 0x58, 0x61, 0x75, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4c, 0x6f, 0x72,
0x64, 0x2d, 0x58, 0x61, 0x75, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61,
0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20,
0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d,
0x61, 0x6c, 0x61, 0x79, 0x7a, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x20, 0x6f, 0x72, 0x6f,
0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d,
0x61, 0x72, 0x6b, 0x69, 0x7a, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61,
0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x6c, 0x61, 0x72,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x76, 0x72, 0x69,
0x6b, 0x69, 0x79, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x76,
0x72, 0x69, 0x6b, 0x69, 0x79, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x61, 0x76, 0x72,
0x69, 0x6b, 0x69, 0x79, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4d, 0x6f, 0x75, 0x73, 0x6f, 0x6e, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61, 0x20,
0x54, 0x69, 0x6e, 0x63, 0x68, 0x20, 0x6f, 0x6b, 0x65, 0x61, 0x6e, 0x69,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b,
0x61, 0x20, 0x54, 0x69, 0x6e, 0x63, 0x68, 0x20, 0x6f, 0x6b, 0x65, 0x61,
0x6e, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x61,
0x20, 0x54, 0x69, 0x6e, 0x63, 0x68, 0x20, 0x6f, 0x6b, 0x65, 0x61, 0x6e,
0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x2d, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x4d, 0x6f, 0x73, 0x6b, 0x76, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4d, 0x79, 0x61, 0x6e, 0x6d,
0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x4b,
0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x59, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x4b, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61,
0x6e, 0x67, 0x69, 0x20, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69,
0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x59, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59,
0x61, 0x6e, 0x67, 0x69, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x79, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x5a,
0x65, 0x6c, 0x61, 0x6e, 0x64, 0x69, 0x79, 0x61, 0x20, 0x79, 0x6f, 0x7a,
0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x79, 0x75, 0x66,
0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x4e, 0x79, 0x75, 0x66, 0x61, 0x75, 0x6e, 0x64, 0x6c, 0x65,
0x6e, 0x64, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x79, 0x75, 0x66, 0x61, 0x75, 0x6e,
0x64, 0x6c, 0x65, 0x6e, 0x64, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x69, 0x75, 0x79, 0x65, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20,
0x6f, 0x72, 0x6f, 0x6c, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x6f, 0x72, 0x6f, 0x6c, 0x69,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x6f,
0x72, 0x6f, 0x6c, 0x69, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75,
0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64,
0x75, 0x2d, 0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x75, 0x2d,
0x64, 0x69, 0x2d, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x79, 0x61, 0x20, 0x79,
0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4f, 0x6d, 0x73,
0x6b, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x50, 0x6f, 0x6b, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x50, 0x6f, 0x6b, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x50, 0x6f, 0x6b, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x79,
0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x61,
0x6c, 0x61, 0x75, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x61, 0x70,
0x75, 0x61, 0x2d, 0x59, 0x61, 0x6e, 0x67, 0x69, 0x20, 0x47, 0x76, 0x69,
0x6e, 0x65, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x61,
0x72, 0x61, 0x67, 0x76, 0x61, 0x79, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x50, 0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x79, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50,
0x61, 0x72, 0x61, 0x67, 0x76, 0x61, 0x79, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x65, 0x72, 0x75, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x65, 0x72, 0x75, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x50, 0x65, 0x72, 0x75, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70,
0x69, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x46, 0x65, 0x6e, 0x69, 0x6b, 0x73, 0x20, 0x6f, 0x72, 0x6f, 0x6c,
0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x65,
0x6e, 0x2d, 0x50, 0x79, 0x65, 0x72, 0x20, 0x76, 0x61, 0x20, 0x4d, 0x69,
0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53,
0x65, 0x6e, 0x2d, 0x50, 0x79, 0x65, 0x72, 0x20, 0x76, 0x61, 0x20, 0x4d,
0x69, 0x6b, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x65, 0x6e,
0x2d, 0x50, 0x79, 0x65, 0x72, 0x20, 0x76, 0x61, 0x20, 0x4d, 0x69, 0x6b,
0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x50, 0x69, 0x74, 0x6b, 0x65, 0x72, 0x6e, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x50, 0x78, 0x65, 0x6e, 0x79, 0x61, 0x6e,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x52, 0x65, 0x79, 0x75, 0x6e, 0x69,
0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x52, 0x6f, 0x74, 0x65,
0x72, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x61, 0x78, 0x61,
0x6c, 0x69, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x61, 0x78,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x61, 0x78, 0x61, 0x6c,
0x69, 0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x53, 0x65, 0x79, 0x73, 0x68, 0x65, 0x6c, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x72, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20, 0x6f, 0x72,
0x6f, 0x6c, 0x6c, 0x61, 0x72, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4a, 0x61, 0x6e, 0x75, 0x62, 0x69, 0x79, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x79, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x53,
0x79, 0x6f, 0x76, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x61,
0x69, 0x74, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x61, 0x79,
0x76, 0x61, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x61, 0x79,
0x76, 0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x61, 0x79, 0x76, 0x61, 0x6e,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x54, 0x6f, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x54, 0x75,
0x72, 0x6b, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x6f,
0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x79, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x79, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x55, 0x72, 0x75, 0x67, 0x76, 0x61, 0x79, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x4f, 0x2018, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x4f, 0x2018, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x6f, 0x6e, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x4f, 0x2018, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x6f,
0x6e, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74,
0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x76, 0x61, 0x71,
0x74, 0x69, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x79, 0x6f, 0x7a, 0x67,
0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x65, 0x6e, 0x65, 0x73,
0x75, 0x65, 0x6c, 0x61, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x76, 0x61,
0x71, 0x74, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74,
0x6f, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x56, 0x6f, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x61, 0x64, 0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x76,
0x61, 0x71, 0x74, 0x69, 0x55, 0x65, 0x79, 0x6b, 0x20, 0x6f, 0x72, 0x6f,
0x6c, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x55, 0x6f, 0x6c, 0x6c,
0x69, 0x73, 0x20, 0x76, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x74, 0x20,
0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72,
0x67, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x74, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x59,
0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67,
0x20, 0x79, 0x6f, 0x7a, 0x67, 0x69, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69,
0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x20, 0x76, 0x61, 0x71, 0x74, 0x69, 0x627,
0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x648, 0x642, 0x62a,
0x6cc, 0x410, 0x444, 0x493, 0x43e, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438,
0x439, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x410, 0x444, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x416, 0x430, 0x43d,
0x443, 0x431, 0x438, 0x439, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20,
0x410, 0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x410, 0x444, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x410,
0x444, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x410, 0x43b, 0x44f, 0x441, 0x43a, 0x430, 0x20, 0x43a, 0x443, 0x43d, 0x434,
0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x43c,
0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x410, 0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410,
0x43c, 0x430, 0x437, 0x43e, 0x43d, 0x43a, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438,
0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x428, 0x438, 0x43c,
0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440,
0x438, 0x43a, 0x430, 0x20, 0x43c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439,
0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x448, 0x430, 0x440, 0x49b, 0x438,
0x439, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b,
0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x448,
0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e,
0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x448, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443,
0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c,
0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x442, 0x43e, 0x493, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438,
0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a,
0x430, 0x20, 0x442, 0x43e, 0x493, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e,
0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20,
0x442, 0x43e, 0x493, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438,
0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x442, 0x438,
0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c, 0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410,
0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430, 0x20, 0x442, 0x438, 0x43d, 0x447, 0x20,
0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x438, 0x43c,
0x43e, 0x43b, 0x438, 0x439, 0x20, 0x410, 0x43c, 0x435, 0x440, 0x438, 0x43a, 0x430,
0x20, 0x442, 0x438, 0x43d, 0x447, 0x20, 0x43e, 0x43a, 0x435, 0x430, 0x43d, 0x438,
0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x410, 0x440, 0x430, 0x431, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x430, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x430, 0x431, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442,
0x438, 0x43d, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x433,
0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20,
0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x410, 0x440,
0x433, 0x435, 0x43d, 0x442, 0x438, 0x43d, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430,
0x440, 0x431, 0x438, 0x439, 0x20, 0x410, 0x440, 0x433, 0x435, 0x43d, 0x442, 0x438,
0x43d, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x410, 0x440, 0x430, 0x43c, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x43c, 0x430, 0x43d, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x440, 0x43c, 0x430, 0x43d, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442, 0x438, 0x43a, 0x430,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442,
0x438, 0x43a, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x442, 0x43b, 0x430, 0x43d, 0x442,
0x438, 0x43a, 0x430, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437,
0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437,
0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20,
0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x43a, 0x443,
0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441,
0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x492, 0x430, 0x440, 0x431, 0x438,
0x439, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430,
0x437, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438,
0x44f, 0x20, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c,
0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439,
0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b,
0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x410, 0x432,
0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442,
0x440, 0x430, 0x43b, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431,
0x438, 0x439, 0x20, 0x410, 0x432, 0x441, 0x442, 0x440, 0x430, 0x43b, 0x438, 0x44f,
0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41e, 0x437, 0x430, 0x440, 0x431, 0x430, 0x439, 0x436, 0x43e,
0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41e, 0x437, 0x430, 0x440, 0x431,
0x430, 0x439, 0x436, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41e, 0x437, 0x430, 0x440,
0x431, 0x430, 0x439, 0x436, 0x43e, 0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x410, 0x437, 0x43e, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x410, 0x437,
0x43e, 0x440, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435, 0x448, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435,
0x448, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x411, 0x430, 0x43d, 0x433, 0x43b, 0x430, 0x434, 0x435,
0x448, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x411, 0x443, 0x442, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x411,
0x43e, 0x43b, 0x438, 0x432, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x411, 0x440, 0x430, 0x437, 0x438, 0x43b, 0x438, 0x44f, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x411, 0x440, 0x443, 0x43d,
0x435, 0x439, 0x20, 0x414, 0x430, 0x440, 0x443, 0x441, 0x441, 0x430, 0x43b, 0x43e,
0x43c, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x430, 0x431, 0x43e, 0x2d,
0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a,
0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41a, 0x430, 0x431, 0x43e, 0x2d, 0x412, 0x435, 0x440, 0x434, 0x435, 0x20, 0x451,
0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x430, 0x43c,
0x43e, 0x440, 0x440, 0x43e, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x427, 0x430,
0x442, 0x445, 0x430, 0x43c, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x427, 0x430,
0x442, 0x445, 0x430, 0x43c, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x427, 0x430, 0x442, 0x445, 0x430,
0x43c, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x427, 0x438, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x427, 0x438, 0x43b,
0x438, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x425, 0x438, 0x442, 0x43e, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x425,
0x438, 0x442, 0x43e, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x425, 0x438, 0x442, 0x43e, 0x439,
0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b, 0x441, 0x430, 0x43d,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b,
0x441, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x427, 0x43e, 0x439, 0x431, 0x430, 0x43b,
0x441, 0x430, 0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x420, 0x43e, 0x436, 0x434, 0x435, 0x441, 0x442, 0x432, 0x43e, 0x20,
0x43e, 0x440, 0x43e, 0x43b, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a,
0x43e, 0x43a, 0x43e, 0x441, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x43e, 0x43b, 0x443, 0x43c,
0x431, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x43e, 0x43b,
0x443, 0x43c, 0x431, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x43e, 0x43b, 0x443,
0x43c, 0x431, 0x438, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41a, 0x443, 0x43a, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b,
0x430, 0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x443, 0x43a,
0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41a, 0x443, 0x43a, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438,
0x20, 0x44f, 0x440, 0x438, 0x43c, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41a, 0x443, 0x431, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x443, 0x431,
0x430, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x414, 0x44d, 0x432, 0x438, 0x441, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x414, 0x443, 0x43c, 0x43e, 0x43d, 0x442, 0x2d, 0x434, 0x2d,
0x423, 0x440, 0x432, 0x438, 0x43b, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428,
0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x422, 0x438, 0x43c, 0x43e, 0x440, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430, 0x441, 0x445, 0x438, 0x20, 0x41e,
0x440, 0x43e, 0x43b, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430,
0x441, 0x445, 0x438, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x438, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41f, 0x430, 0x441, 0x445, 0x438, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x438, 0x20,
0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42d, 0x43a,
0x432, 0x430, 0x434, 0x43e, 0x440, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c,
0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x415, 0x432, 0x440, 0x43e,
0x43f, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a,
0x430, 0x437, 0x438, 0x439, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x415,
0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428,
0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430,
0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x415, 0x432,
0x440, 0x43e, 0x43f, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430,
0x440, 0x431, 0x438, 0x439, 0x20, 0x415, 0x432, 0x440, 0x43e, 0x43f, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x415, 0x432, 0x440,
0x43e, 0x43f, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x424, 0x43e, 0x43b, 0x44c, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20,
0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x424, 0x43e, 0x43b, 0x44c, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20,
0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424,
0x43e, 0x43b, 0x44c, 0x43a, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x43e, 0x440, 0x43e,
0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x424, 0x438, 0x436, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x424, 0x438, 0x436, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x438, 0x436,
0x438, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x424, 0x440, 0x430, 0x43d, 0x446, 0x443, 0x437, 0x20, 0x413, 0x432, 0x438, 0x430,
0x43d, 0x430, 0x441, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x440,
0x430, 0x43d, 0x446, 0x443, 0x437, 0x20, 0x436, 0x430, 0x43d, 0x443, 0x431, 0x438,
0x439, 0x20, 0x432, 0x430, 0x20, 0x410, 0x43d, 0x442, 0x430, 0x440, 0x43a, 0x442,
0x438, 0x43a, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x430, 0x43b,
0x430, 0x43f, 0x430, 0x433, 0x43e, 0x441, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x413, 0x430, 0x43c, 0x431, 0x438, 0x435, 0x440, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x413, 0x440, 0x443, 0x437, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x440,
0x443, 0x437, 0x438, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x413, 0x438, 0x43b, 0x431, 0x435, 0x440, 0x442, 0x20, 0x43e,
0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x413, 0x440, 0x438, 0x43d, 0x432, 0x438, 0x447, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x413, 0x440, 0x435,
0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x428, 0x430, 0x440,
0x49b, 0x438, 0x439, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434,
0x438, 0x44f, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x413, 0x440, 0x435, 0x43d,
0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b,
0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492, 0x430, 0x440, 0x431,
0x438, 0x439, 0x20, 0x413, 0x440, 0x435, 0x43d, 0x43b, 0x430, 0x43d, 0x434, 0x438,
0x44f, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41a, 0x45e, 0x440, 0x444, 0x430, 0x437, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x413, 0x430, 0x439, 0x430, 0x43d, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x413, 0x430, 0x432, 0x430, 0x439, 0x438, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x430, 0x432, 0x430, 0x439, 0x438,
0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434,
0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413, 0x430, 0x432,
0x430, 0x439, 0x438, 0x2d, 0x430, 0x43b, 0x435, 0x443, 0x442, 0x20, 0x43a, 0x443,
0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x413, 0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x413,
0x43e, 0x43d, 0x43a, 0x43e, 0x43d, 0x433, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x425, 0x43e, 0x432, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x425, 0x43e,
0x432, 0x434, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x4b2, 0x438, 0x43d, 0x434, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x4b2, 0x438, 0x43d, 0x434, 0x20, 0x43e, 0x43a, 0x435,
0x430, 0x43d, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x4b2, 0x438, 0x43d,
0x434, 0x2d, 0x425, 0x438, 0x442, 0x43e, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41c, 0x430, 0x440, 0x43a, 0x430, 0x437, 0x438, 0x439, 0x20, 0x418, 0x43d,
0x434, 0x43e, 0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x418, 0x43d, 0x434, 0x43e,
0x43d, 0x435, 0x437, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x492,
0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x418, 0x43d, 0x434, 0x43e, 0x43d, 0x435,
0x437, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42d, 0x440, 0x43e,
0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42d, 0x440, 0x43e, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x42d, 0x440, 0x43e, 0x43d, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443,
0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x418, 0x440, 0x43a,
0x443, 0x442, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x418, 0x440,
0x43a, 0x443, 0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x418, 0x440, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x418, 0x441, 0x440, 0x43e, 0x438, 0x43b, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x418, 0x441, 0x440, 0x43e, 0x438, 0x43b, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x418,
0x441, 0x440, 0x43e, 0x438, 0x43b, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43f, 0x43e, 0x43d,
0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43f, 0x43e, 0x43d,
0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43f, 0x43e, 0x43d, 0x438, 0x44f, 0x20,
0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x428, 0x430, 0x440, 0x49b, 0x438, 0x439, 0x20, 0x49a, 0x43e, 0x437,
0x43e, 0x493, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x492, 0x430, 0x440, 0x431, 0x438, 0x439, 0x20, 0x49a, 0x43e, 0x437, 0x43e,
0x493, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41a, 0x43e, 0x440, 0x435, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a,
0x43e, 0x440, 0x435, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x43e, 0x440, 0x435, 0x44f,
0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41a, 0x43e, 0x441, 0x440, 0x430, 0x435, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41a, 0x440, 0x430, 0x441, 0x43d, 0x43e, 0x44f, 0x440, 0x441,
0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x440, 0x430, 0x441, 0x43d,
0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41a, 0x440, 0x430, 0x441,
0x43d, 0x43e, 0x44f, 0x440, 0x441, 0x43a, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x49a, 0x438, 0x440, 0x493, 0x438, 0x437, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41b, 0x430,
0x439, 0x43d, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41b, 0x43e, 0x440, 0x434, 0x20, 0x425, 0x43e,
0x432, 0x435, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41b, 0x43e, 0x440, 0x434,
0x20, 0x425, 0x43e, 0x432, 0x435, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41b, 0x43e, 0x440, 0x434,
0x20, 0x425, 0x43e, 0x432, 0x435, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x433, 0x430,
0x434, 0x430, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x433,
0x430, 0x434, 0x430, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x433, 0x430, 0x434,
0x430, 0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41c, 0x430, 0x43b, 0x430, 0x439, 0x437, 0x438, 0x44f, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41c, 0x430, 0x43b, 0x44c, 0x434, 0x438, 0x432, 0x20, 0x43e,
0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x41c, 0x430, 0x440, 0x43a, 0x435, 0x437,
0x430, 0x441, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x440, 0x448,
0x430, 0x43b, 0x43b, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x432, 0x440, 0x438, 0x43a,
0x438, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x432, 0x440,
0x438, 0x43a, 0x438, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x430, 0x432, 0x440, 0x438,
0x43a, 0x438, 0x439, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41c, 0x43e, 0x443, 0x432, 0x441, 0x43e, 0x43d, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x423, 0x43b, 0x430, 0x43d, 0x2d, 0x411, 0x430, 0x442, 0x43e,
0x440, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423, 0x43b, 0x430, 0x43d, 0x2d,
0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423, 0x43b, 0x430, 0x43d,
0x2d, 0x411, 0x430, 0x442, 0x43e, 0x440, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41c, 0x43e, 0x441, 0x43a, 0x432, 0x430, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41c, 0x44c, 0x44f, 0x43d, 0x43c,
0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x430, 0x443, 0x440, 0x443,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x435, 0x43f, 0x430, 0x43b, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43d, 0x433, 0x438, 0x20, 0x41a, 0x430,
0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x42f, 0x43d, 0x433, 0x438, 0x20, 0x41a, 0x430, 0x43b, 0x435, 0x434, 0x43e,
0x43d, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43d, 0x433, 0x438, 0x20, 0x41a,
0x430, 0x43b, 0x435, 0x434, 0x43e, 0x43d, 0x438, 0x44f, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43d, 0x433, 0x438, 0x20,
0x417, 0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x42f, 0x43d, 0x433, 0x438, 0x20, 0x417, 0x435, 0x43b, 0x430, 0x43d,
0x434, 0x438, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43d, 0x433, 0x438, 0x20, 0x417,
0x435, 0x43b, 0x430, 0x43d, 0x434, 0x438, 0x44f, 0x20, 0x43a, 0x443, 0x43d, 0x434,
0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x44c,
0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41d, 0x44c, 0x44e, 0x444, 0x430, 0x443, 0x43d, 0x434,
0x43b, 0x435, 0x43d, 0x434, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x44c, 0x44e, 0x444, 0x430,
0x443, 0x43d, 0x434, 0x43b, 0x435, 0x43d, 0x434, 0x20, 0x43a, 0x443, 0x43d, 0x434,
0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x438,
0x443, 0x435, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x43e, 0x440, 0x444,
0x43e, 0x43b, 0x43a, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41d, 0x43e, 0x440, 0x444, 0x43e, 0x43b, 0x43a, 0x20, 0x43e,
0x440, 0x43e, 0x43b, 0x438, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x43e, 0x440, 0x444, 0x43e,
0x43b, 0x43a, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x438, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x435, 0x440, 0x43d, 0x430,
0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d,
0x44c, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x435, 0x440, 0x43d,
0x430, 0x43d, 0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e,
0x43d, 0x44c, 0x44f, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x435, 0x440, 0x43d, 0x430, 0x43d,
0x434, 0x43e, 0x20, 0x434, 0x435, 0x20, 0x41d, 0x43e, 0x440, 0x43e, 0x43d, 0x44c,
0x44f, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x43e, 0x432, 0x43e, 0x441, 0x438, 0x431,
0x438, 0x440, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440,
0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41d, 0x43e, 0x432, 0x43e, 0x441,
0x438, 0x431, 0x438, 0x440, 0x441, 0x43a, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41e, 0x43c, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41e, 0x43c,
0x441, 0x43a, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41f, 0x43e, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41f, 0x43e, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x41f, 0x43e, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x451,
0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430, 0x43b,
0x430, 0x443, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430, 0x43f, 0x443,
0x430, 0x2d, 0x42f, 0x43d, 0x433, 0x438, 0x20, 0x413, 0x432, 0x438, 0x43d, 0x435,
0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430, 0x440, 0x430, 0x433,
0x432, 0x430, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430, 0x440,
0x430, 0x433, 0x432, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x430, 0x440, 0x430,
0x433, 0x432, 0x430, 0x439, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x41f, 0x435, 0x440, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x435, 0x440, 0x443,
0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424,
0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x441, 0x442,
0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x424, 0x438, 0x43b, 0x438, 0x43f, 0x43f, 0x438, 0x43d, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x424, 0x435, 0x43d, 0x438, 0x43a,
0x441, 0x20, 0x43e, 0x440, 0x43e, 0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c, 0x435,
0x440, 0x20, 0x432, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e, 0x43d,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f,
0x44c, 0x435, 0x440, 0x20, 0x432, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b,
0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x435, 0x43d, 0x442, 0x2d, 0x41f, 0x44c,
0x435, 0x440, 0x20, 0x432, 0x430, 0x20, 0x41c, 0x438, 0x43a, 0x435, 0x43b, 0x43e,
0x43d, 0x20, 0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x41f, 0x438, 0x442, 0x43a, 0x435, 0x440, 0x43d, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x41f, 0x43e, 0x43d, 0x430, 0x43f, 0x435, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x420, 0x435, 0x44e, 0x43d, 0x44c, 0x43e, 0x43d,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x420, 0x43e, 0x442, 0x435, 0x440, 0x430,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438,
0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x430, 0x445, 0x430, 0x43b,
0x438, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x430, 0x445, 0x430, 0x43b, 0x438, 0x43d,
0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421,
0x430, 0x43c, 0x43e, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x430,
0x43c, 0x43e, 0x430, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x430, 0x43c, 0x43e, 0x430, 0x20,
0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x421, 0x435, 0x439, 0x448, 0x435, 0x43b, 0x20, 0x43e, 0x440, 0x43e,
0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421,
0x438, 0x43d, 0x433, 0x430, 0x43f, 0x443, 0x440, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x421, 0x43e, 0x43b, 0x43e, 0x43c, 0x43e, 0x43d, 0x20, 0x43e, 0x440, 0x43e,
0x43b, 0x43b, 0x430, 0x440, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x416,
0x430, 0x43d, 0x443, 0x431, 0x438, 0x439, 0x20, 0x414, 0x436, 0x43e, 0x440, 0x434,
0x436, 0x438, 0x44f, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x443, 0x440,
0x438, 0x43d, 0x430, 0x43c, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x421, 0x44c,
0x43e, 0x432, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x430, 0x438,
0x442, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x430, 0x439, 0x43f,
0x435, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x430, 0x439, 0x43f,
0x435, 0x439, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x430, 0x439, 0x43f, 0x435, 0x439, 0x20,
0x43a, 0x443, 0x43d, 0x434, 0x443, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x422, 0x43e, 0x436, 0x438, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x43e, 0x43a, 0x435, 0x43b, 0x430,
0x443, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x43e, 0x43d, 0x433, 0x430,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20,
0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b,
0x442, 0x438, 0x422, 0x43e, 0x43d, 0x433, 0x430, 0x20, 0x451, 0x437, 0x433, 0x438,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x427, 0x443, 0x443, 0x43a, 0x20, 0x432,
0x430, 0x49b, 0x442, 0x438, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x430, 0x43d, 0x438,
0x441, 0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x422, 0x443,
0x440, 0x43a, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x441,
0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x422, 0x443, 0x440, 0x43a, 0x43c, 0x430, 0x43d, 0x438, 0x441, 0x442, 0x43e,
0x43d, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x422, 0x443, 0x432, 0x430, 0x43b, 0x443, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x432, 0x430, 0x49b, 0x442,
0x438, 0x423, 0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423,
0x440, 0x443, 0x433, 0x432, 0x430, 0x439, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x40e, 0x437, 0x431, 0x435, 0x43a, 0x438, 0x441,
0x442, 0x43e, 0x43d, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x40e, 0x437, 0x431,
0x435, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x40e, 0x437,
0x431, 0x435, 0x43a, 0x438, 0x441, 0x442, 0x43e, 0x43d, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412, 0x430, 0x43d, 0x443, 0x430,
0x442, 0x443, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412, 0x430, 0x43d, 0x443,
0x430, 0x442, 0x443, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412, 0x430, 0x43d, 0x443, 0x430, 0x442,
0x443, 0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438,
0x412, 0x435, 0x43d, 0x435, 0x441, 0x443, 0x44d, 0x43b, 0x430, 0x20, 0x432, 0x430,
0x49b, 0x442, 0x438, 0x412, 0x43b, 0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442,
0x43e, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412, 0x43b, 0x430, 0x434,
0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d,
0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412, 0x43b,
0x430, 0x434, 0x438, 0x432, 0x43e, 0x441, 0x442, 0x43e, 0x43a, 0x20, 0x451, 0x437,
0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412, 0x43e, 0x43b, 0x433,
0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x441, 0x442, 0x430,
0x43d, 0x434, 0x430, 0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412,
0x43e, 0x43b, 0x433, 0x43e, 0x433, 0x440, 0x430, 0x434, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x412, 0x43e, 0x441, 0x442, 0x43e,
0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423, 0x44d, 0x439, 0x43a, 0x20,
0x43e, 0x440, 0x43e, 0x43b, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x423,
0x44d, 0x43b, 0x43b, 0x438, 0x441, 0x20, 0x432, 0x430, 0x20, 0x424, 0x443, 0x442,
0x443, 0x43d, 0x430, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43a, 0x443,
0x442, 0x441, 0x43a, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430, 0x440, 0x442,
0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x42f, 0x43a, 0x443, 0x442, 0x441, 0x43a,
0x20, 0x451, 0x437, 0x433, 0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x415,
0x43a, 0x430, 0x442, 0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20,
0x432, 0x430, 0x49b, 0x442, 0x438, 0x415, 0x43a, 0x430, 0x442, 0x435, 0x440, 0x438,
0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x441, 0x442, 0x430, 0x43d, 0x434, 0x430,
0x440, 0x442, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x415, 0x43a, 0x430, 0x442,
0x435, 0x440, 0x438, 0x43d, 0x431, 0x443, 0x440, 0x433, 0x20, 0x451, 0x437, 0x433,
0x438, 0x20, 0x432, 0x430, 0x49b, 0x442, 0x438, 0x47, 0x69, 0x1edd, 0x20, 0x41,
0x63, 0x72, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x41, 0x63, 0x72, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x41, 0x63, 0x72, 0x65, 0x47, 0x69, 0x1edd, 0x20,
0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47,
0x69, 0x1edd, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x50, 0x68, 0x69,
0x47, 0x69, 0x1edd, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0x50, 0x68, 0x69,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x61,
0x6d, 0x20, 0x50, 0x68, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0xe2, 0x79,
0x20, 0x50, 0x68, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x50, 0x68, 0x69, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x54, 0xe2, 0x79, 0x20,
0x50, 0x68, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41,
0x6c, 0x61, 0x73, 0x6b, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x6c, 0x6d, 0x61, 0x74, 0x79,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41,
0x6c, 0x6d, 0x61, 0x74, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d,
0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75,
0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x47, 0x69,
0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x6d, 0x69, 0x1ec1,
0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d,
0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20,
0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110,
0xf4, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68,
0xe8, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x47,
0x69, 0x1edd, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x6e, 0xfa, 0x69, 0x47,
0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x6d, 0x69, 0x1ec1,
0x6e, 0x20, 0x6e, 0xfa, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61,
0x20, 0x68, 0xe8, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x6e, 0xfa, 0x69,
0x47, 0x69, 0x1edd, 0x20, 0x54, 0x68, 0xe1, 0x69, 0x20, 0x42, 0xec, 0x6e,
0x68, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x63,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0x68, 0xe1, 0x69, 0x20, 0x42, 0xec,
0x6e, 0x68, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20,
0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x54, 0x68, 0xe1, 0x69, 0x20,
0x42, 0xec, 0x6e, 0x68, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69,
0x1edd, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x70, 0x69,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41,
0x70, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x41, 0x70, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x71,
0x74, 0x61, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x41, 0x71, 0x74, 0x61, 0x75, 0x47, 0x69,
0x1edd, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x71, 0x74, 0x6f, 0x62, 0x65,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41,
0x71, 0x74, 0x6f, 0x62, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x1ea2, 0x20, 0x52,
0x1ead, 0x70, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x1ea2, 0x20, 0x52, 0x1ead, 0x70, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x1ea2, 0x20, 0x52, 0x1ead, 0x70, 0x47, 0x69, 0x1edd,
0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x74, 0xe2,
0x79, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x6d, 0x69, 0x1ec1,
0x6e, 0x20, 0x74, 0xe2, 0x79, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68,
0xe8, 0x20, 0x6d, 0x69, 0x1ec1, 0x6e, 0x20, 0x74, 0xe2, 0x79, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0x1ea1,
0x69, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x110, 0x1ea1, 0x69,
0x20, 0x54, 0xe2, 0x79, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69,
0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x110, 0x1ea1, 0x69,
0x20, 0x54, 0xe2, 0x79, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e,
0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20,
0x48, 0xe8, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e,
0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e,
0x67, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20,
0x54, 0xe2, 0x79, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x54,
0xe2, 0x79, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e,
0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x69, 0x1ec1,
0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20,
0x48, 0xe8, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67,
0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54,
0xe2, 0x79, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4d,
0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x47, 0x69,
0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8,
0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c,
0x61, 0x64, 0x65, 0x73, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44,
0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x43, 0x61, 0x70,
0x65, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x4d,
0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56,
0x65, 0x72, 0x64, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x61, 0x73, 0x65,
0x79, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x43, 0x68,
0x69, 0x6c, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67,
0x20, 0x51, 0x75, 0x1ed1, 0x63, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x51, 0x75, 0x1ed1,
0x63, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x51, 0x75, 0x1ed1, 0x63, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x43, 0x68,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0x1ea3, 0x6f,
0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61, 0x73, 0x47, 0x69,
0x1edd, 0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x43,
0x6f, 0x63, 0x6f, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x51, 0x75, 0x1ea7,
0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x51, 0x75, 0x1ea7, 0x6e,
0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x47, 0x69, 0x1edd,
0x20, 0x4e, 0x1eed, 0x61, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x75, 0x62, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x43, 0x75, 0x62, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x43, 0x75,
0x62, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x44, 0x61, 0x76, 0x69, 0x73, 0x47,
0x69, 0x1edd, 0x20, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019,
0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x110,
0xf4, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x47, 0x69, 0x1edd,
0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x50, 0x68, 0x1ee5, 0x63, 0x20, 0x53, 0x69,
0x6e, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x110, 0x1ea3, 0x6f, 0x20, 0x50, 0x68, 0x1ee5, 0x63, 0x20, 0x53, 0x69, 0x6e,
0x68, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x110, 0x1ea3, 0x6f, 0x20, 0x50, 0x68, 0x1ee5, 0x63, 0x20, 0x53, 0x69, 0x6e,
0x68, 0x47, 0x69, 0x1edd, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72,
0x47, 0x69, 0x1edd, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0xc2, 0x75,
0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0x72,
0x75, 0x6e, 0x67, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9,
0x61, 0x20, 0x68, 0xe8, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0xc2,
0x75, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0xc2, 0x75,
0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x110, 0xf4,
0x6e, 0x67, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61,
0x20, 0x68, 0xe8, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0xc2, 0x75, 0x47,
0x69, 0x1edd, 0x20, 0x56, 0x69, 0x1ec5, 0x6e, 0x20, 0x111, 0xf4, 0x6e, 0x67,
0x20, 0x43, 0x68, 0xe2, 0x75, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20,
0x54, 0xe2, 0x79, 0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0xc2, 0x75, 0x47, 0x69,
0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x54, 0xe2, 0x79,
0x20, 0xc2, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20,
0x110, 0x1ea3, 0x6f, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x51, 0x75,
0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x46,
0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x46,
0x69, 0x6a, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x46, 0x69, 0x6a, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x47, 0x69, 0x1edd, 0x20,
0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x20, 0x74, 0x68, 0x75, 0x1ed9, 0x63,
0x20, 0x50, 0x68, 0xe1, 0x70, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x61, 0x6d,
0x20, 0x43, 0x1ef1, 0x63, 0x20, 0x76, 0xe0, 0x20, 0x4e, 0x61, 0x6d, 0x20,
0x4e, 0x1b0, 0x1edb, 0x63, 0x20, 0x50, 0x68, 0xe1, 0x70, 0x47, 0x69, 0x1edd,
0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x47, 0x69,
0x1edd, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x47, 0x69, 0x1edd,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x51,
0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67,
0x20, 0x62, 0xec, 0x6e, 0x68, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77,
0x69, 0x63, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20,
0x110, 0xf4, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0x47, 0x72,
0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d,
0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110,
0xf4, 0x6e, 0x67, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0xe2,
0x79, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x69, 0x1ec1,
0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x47, 0x75, 0x61, 0x6d, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x56, 0xf9, 0x6e, 0x67,
0x20, 0x56, 0x1ecb, 0x6e, 0x68, 0x47, 0x69, 0x1edd, 0x20, 0x47, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x47, 0x69, 0x1edd, 0x20, 0x48, 0x1ed3, 0x6e, 0x67,
0x20, 0x4b, 0xf4, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x48, 0x1ed3, 0x6e, 0x67, 0x20, 0x4b, 0xf4, 0x6e, 0x67,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x48,
0x1ed3, 0x6e, 0x67, 0x20, 0x4b, 0xf4, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20,
0x48, 0x6f, 0x76, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x1ea4, 0x6e, 0x20, 0x110, 0x1ed9,
0x47, 0x69, 0x1edd, 0x20, 0x1ea4, 0x6e, 0x20, 0x110, 0x1ed9, 0x20, 0x44, 0x1b0,
0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20,
0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1,
0x6e, 0x20, 0x54, 0x72, 0x75, 0x6e, 0x67, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1,
0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e,
0x20, 0x54, 0xe2, 0x79, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x49, 0x72, 0x61, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x4d,
0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x47, 0x69, 0x1edd, 0x20,
0x4e, 0x68, 0x1ead, 0x74, 0x20, 0x42, 0x1ea3, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x68, 0x1ead, 0x74, 0x20, 0x42,
0x1ea3, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x4e, 0x68, 0x1ead, 0x74, 0x20, 0x42, 0x1ea3, 0x6e, 0x47, 0x69, 0x1edd,
0x20, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b,
0x69, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x50,
0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b,
0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x47,
0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x50, 0x65,
0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d,
0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x110, 0xf4, 0x6e, 0x67, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0x69, 0x1ec1, 0x6e, 0x20, 0x54, 0xe2, 0x79, 0x20, 0x4b,
0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd,
0x20, 0x48, 0xe0, 0x6e, 0x20, 0x51, 0x75, 0x1ed1, 0x63, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x48, 0xe0, 0x6e, 0x20, 0x51,
0x75, 0x1ed1, 0x63, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x48, 0xe0, 0x6e, 0x20, 0x51, 0x75, 0x1ed1, 0x63, 0x47, 0x69,
0x1edd, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x47, 0x69, 0x1edd, 0x20,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x4b, 0x79,
0x72, 0x67, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4c,
0x61, 0x6e, 0x6b, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x51, 0x75, 0x1ea7, 0x6e,
0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x4c, 0x69, 0x6e, 0x65, 0x47, 0x69, 0x1edd,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4c, 0x6f, 0x72, 0x64,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x61, 0x20, 0x43, 0x61, 0x6f, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x61, 0x20,
0x43, 0x61, 0x6f, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x4d, 0x61, 0x20, 0x43, 0x61, 0x6f, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x61,
0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x61,
0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x51,
0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x4d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x68,
0xe1, 0x69, 0x20, 0x42, 0xec, 0x6e, 0x68, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e,
0x67, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x6f, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0x68, 0xe1, 0x69, 0x20, 0x42,
0xec, 0x6e, 0x68, 0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x20, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x6f, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20,
0x48, 0xe8, 0x20, 0x54, 0x68, 0xe1, 0x69, 0x20, 0x42, 0xec, 0x6e, 0x68,
0x20, 0x44, 0x1b0, 0x1a1, 0x6e, 0x67, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x63,
0x6f, 0x47, 0x69, 0x1edd, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61,
0x74, 0x6f, 0x72, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x47,
0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0x61, 0x74, 0x78, 0x63, 0x1a1, 0x76, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4d, 0x61, 0x74, 0x78, 0x63, 0x1a1,
0x76, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x4d, 0x61, 0x74, 0x78, 0x63, 0x1a1, 0x76, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x47, 0x69, 0x1edd, 0x20,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x65, 0x70,
0x61, 0x6c, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61,
0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x65,
0x77, 0x20, 0x5a, 0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a,
0x65, 0x61, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61,
0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x65, 0x77, 0x66,
0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x69, 0x75,
0x65, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x110, 0x1ea3, 0x6f, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x47,
0x69, 0x1edd, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48,
0xe8, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x42, 0x1eaf,
0x63, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x61, 0x6e, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b,
0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x4e, 0x6f,
0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x47, 0x69, 0x1edd,
0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x4e, 0x6f, 0x76, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x4f,
0x6d, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x63, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61,
0x20, 0x68, 0xe8, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd,
0x20, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x61,
0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75, 0x69, 0x6e,
0x65, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x47, 0x69, 0x1edd, 0x20,
0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x65, 0x72, 0x75, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x50, 0x65, 0x72,
0x75, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20,
0x50, 0x65, 0x72, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x68, 0x69, 0x6c,
0x69, 0x70, 0x70, 0x69, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x50,
0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x47, 0x69, 0x1edd, 0x20,
0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x50, 0x68, 0x6f,
0x65, 0x6e, 0x69, 0x78, 0x47, 0x69, 0x1edd, 0x20, 0x53, 0x74, 0x2e, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x76, 0xe0, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x76, 0xe0, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x76, 0xe0, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72,
0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x47,
0x69, 0x1edd, 0x20, 0x42, 0xec, 0x6e, 0x68, 0x20, 0x4e, 0x68, 0x1b0, 0x1ee1,
0x6e, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f,
0x72, 0x64, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x51, 0x79, 0x7a,
0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x52, 0x65,
0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x52, 0x6f, 0x74,
0x68, 0x65, 0x72, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9,
0x6e, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x53, 0x61, 0x6b,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e,
0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x6d,
0xf9, 0x61, 0x20, 0x68, 0xe8, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61,
0x47, 0x69, 0x1edd, 0x20, 0x62, 0x61, 0x6e, 0x20, 0x6e, 0x67, 0xe0, 0x79,
0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x53, 0x65,
0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x47, 0x69, 0x1edd, 0x20,
0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65, 0x47, 0x69, 0x1edd,
0x20, 0x51, 0x75, 0x1ea7, 0x6e, 0x20, 0x110, 0x1ea3, 0x6f, 0x20, 0x53, 0x6f,
0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4e, 0x61, 0x6d,
0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x47, 0x69, 0x1edd, 0x20,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x47, 0x69, 0x1edd, 0x20,
0x53, 0x79, 0x6f, 0x77, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0xe0, 0x69, 0x20, 0x42,
0x1eaf, 0x63, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x110, 0xe0, 0x69, 0x20, 0x42, 0x1eaf, 0x63, 0x47, 0x69, 0x1edd, 0x20, 0x4d,
0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x110, 0xe0, 0x69, 0x20, 0x42, 0x1eaf,
0x63, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c,
0x61, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x47,
0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x75, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d,
0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43,
0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61,
0x20, 0x48, 0xe8, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9,
0x61, 0x20, 0x48, 0xe8, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x47, 0x69, 0x1edd, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69,
0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x47, 0x69, 0x1edd, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68,
0x75, 0x1ea9, 0x6e, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x47,
0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x47, 0x69, 0x1edd, 0x20, 0x56, 0x65, 0x6e,
0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x56, 0x6c,
0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x47, 0x69, 0x1edd,
0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69,
0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9,
0x61, 0x20, 0x68, 0xe8, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x56, 0x6f, 0x6c, 0x67,
0x6f, 0x67, 0x72, 0x61, 0x64, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x47, 0x69, 0x1edd, 0x20, 0x4d, 0xf9, 0x61, 0x20, 0x48, 0xe8, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x47, 0x69, 0x1edd, 0x20,
0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x110, 0x1ea3,
0x6f, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x47, 0x69, 0x1edd, 0x20, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x76, 0xe0, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x47, 0x69, 0x1edd, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73,
0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75, 0x1ea9, 0x6e, 0x20, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9,
0x61, 0x20, 0x68, 0xe8, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x47, 0x69, 0x1edd, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x43, 0x68, 0x75,
0x1ea9, 0x6e, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x6d, 0xf9, 0x61, 0x20,
0x68, 0xe8, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x47, 0x69, 0x1edd, 0x20, 0x59, 0x75, 0x6b, 0x6f,
0x6e, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x161, 0x69, 0x20, 0x5a,
0x69, 0x74, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x161, 0x69, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x74, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x161, 0x69, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x7a, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x5a, 0x69,
0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70,
0xe4, 0x69, 0x161, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x7a, 0x69, 0x74, 0x4d, 0x69, 0x74, 0x74, 0x65, 0x6c, 0x65, 0x75,
0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x7a, 0x69, 0x74, 0x4f, 0x161, 0x74, 0x65, 0x75, 0x72, 0x6f,
0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x5a, 0x69, 0x74, 0x4f, 0x161, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69, 0x74, 0x4f, 0x161, 0x74,
0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x53, 0x75,
0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x74, 0x57, 0x65, 0x161, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161, 0x69, 0x20, 0x5a, 0x69, 0x74,
0x57, 0x65, 0x161, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69, 0x161,
0x69, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x7a, 0x69,
0x74, 0x57, 0x65, 0x161, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0xe4, 0x69,
0x161, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x7a, 0x69, 0x74,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43,
0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x41, 0x66,
0x66, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44,
0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x66, 0x72, 0x69,
0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x44, 0x65, 0x20, 0x41, 0x66, 0x66, 0x72, 0x69,
0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x6f, 0x72, 0x6c,
0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x66, 0x72, 0x69, 0x63,
0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e,
0x20, 0x41, 0x66, 0x66, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65,
0x77, 0x69, 0x6e, 0x20, 0x41, 0x66, 0x66, 0x72, 0x69, 0x63, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61,
0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x6d, 0x61,
0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43,
0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x47, 0x6f,
0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74,
0x68, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74,
0x68, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44,
0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65,
0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x47, 0x6f, 0x67, 0x6c,
0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x44, 0x77, 0x79,
0x72, 0x61, 0x69, 0x6e, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x4d, 0x79, 0x6e, 0x79, 0x64, 0x64, 0x6f, 0x65, 0x64, 0x64,
0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4d, 0x79, 0x6e, 0x79, 0x64, 0x64,
0x6f, 0x65, 0x64, 0x64, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4d, 0x79, 0x6e, 0x79, 0x64, 0x64,
0x6f, 0x65, 0x64, 0x64, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x43, 0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x61,
0x77, 0x65, 0x6c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x65, 0x66,
0x6e, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x61, 0x77, 0x65, 0x6c, 0x20, 0x47,
0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x43, 0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x54, 0x61, 0x77, 0x65,
0x6c, 0x20, 0x47, 0x6f, 0x67, 0x6c, 0x65, 0x64, 0x64, 0x20, 0x41, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x41,
0x70, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x70, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61, 0x69, 0x64,
0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x41, 0x72, 0x61, 0x62, 0x61, 0x69, 0x64, 0x64, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x72, 0x61,
0x62, 0x61, 0x69, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x79,
0x72, 0x20, 0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e,
0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x6f, 0x72, 0x6c,
0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e,
0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69,
0x6e, 0x20, 0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e, 0x69, 0x6e, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x47, 0x6f, 0x72, 0x6c,
0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x72, 0x69, 0x61, 0x6e, 0x6e,
0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x41, 0x72,
0x6d, 0x65, 0x6e, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43,
0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x79, 0x72, 0x20, 0x49, 0x77,
0x65, 0x72, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x65, 0x66, 0x6e, 0x66,
0x6f, 0x72, 0x20, 0x79, 0x72, 0x20, 0x49, 0x77, 0x65, 0x72, 0x79, 0x64,
0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43,
0x65, 0x66, 0x6e, 0x66, 0x6f, 0x72, 0x20, 0x79, 0x72, 0x20, 0x49, 0x77,
0x65, 0x72, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43,
0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x41, 0x77,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x61, 0x6e,
0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x41, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74,
0x68, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61,
0x72, 0x74, 0x68, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69,
0x6e, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20,
0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x77,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x48, 0x61, 0x66, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61,
0x72, 0x74, 0x68, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69,
0x6e, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e,
0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x41, 0x77, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x41,
0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20,
0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x47,
0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x41, 0x77, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69,
0x6e, 0x20, 0x41, 0x77, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69,
0x6a, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x79, 0x72, 0x20, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x79, 0x72, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x79,
0x72, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x42, 0x68, 0x75, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x66, 0x69, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed,
0x6c, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x42, 0x72, 0x75,
0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c,
0x61, 0x6d, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x61, 0x62, 0x6f,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x61, 0x62, 0x6f,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x43, 0x61, 0x62, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x72, 0x6f, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x68,
0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x68, 0x69, 0x6c,
0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x73,
0x69, 0x65, 0x69, 0x6e, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x54, 0x73, 0x69, 0x65, 0x69,
0x6e, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x54, 0x73, 0x69, 0x65, 0x69, 0x6e, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43, 0x68, 0x6f,
0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x59, 0x20, 0x4e, 0x61, 0x64, 0x6f,
0x6c, 0x69, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79,
0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69,
0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59,
0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x43, 0x6f, 0x6f, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x43,
0x6f, 0x6f, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x6e,
0x6e, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x59, 0x6e, 0x79, 0x73,
0x6f, 0x65, 0x64, 0x64, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x43, 0x69, 0x77, 0x62, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x69,
0x77, 0x62, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x43, 0x69, 0x77, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44,
0x61, 0x76, 0x69, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x75,
0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c,
0x6c, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x77, 0x79, 0x72,
0x61, 0x69, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x50, 0x61,
0x73, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x50,
0x61, 0x73, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66,
0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x79, 0x20, 0x50, 0x61, 0x73, 0x67,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f,
0x72, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c,
0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20,
0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x43, 0x61, 0x6e, 0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74,
0x68, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x45, 0x77, 0x72,
0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20,
0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x45,
0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x77,
0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x50, 0x65, 0x6c, 0x6c, 0x20, 0x45,
0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x6f,
0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x45, 0x77, 0x72, 0x6f,
0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e,
0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69,
0x6e, 0x20, 0x45, 0x77, 0x72, 0x6f, 0x70, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x46, 0x61,
0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x2f, 0x4d, 0x61, 0x6c, 0x76,
0x69, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65,
0x64, 0x64, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x2f, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f,
0x65, 0x64, 0x64, 0x20, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x2f, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x61, 0x73, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x46, 0x69,
0x6a, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x75,
0x79, 0x61, 0x6e, 0x65, 0x20, 0x46, 0x66, 0x72, 0x65, 0x6e, 0x67, 0x69,
0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x69, 0x72, 0x6f, 0x65,
0x64, 0x64, 0x20, 0x46, 0x66, 0x72, 0x61, 0x69, 0x6e, 0x63, 0x20, 0x79,
0x6e, 0x20, 0x79, 0x20, 0x44, 0x65, 0x20, 0x61, 0x2019, 0x72, 0x20, 0x41,
0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x67, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69,
0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79,
0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69,
0x6e, 0x20, 0x79, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4c, 0x61,
0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x79,
0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4c, 0x61, 0x73, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x44, 0x77, 0x79, 0x72,
0x61, 0x69, 0x6e, 0x20, 0x79, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20,
0x4c, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x6f, 0x72,
0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x79, 0x72, 0x20, 0x59, 0x6e,
0x79, 0x73, 0x20, 0x4c, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x47, 0x6f, 0x72, 0x6c,
0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x79, 0x72, 0x20, 0x59, 0x6e, 0x79,
0x73, 0x20, 0x4c, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e,
0x20, 0x79, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x4c, 0x61, 0x73,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x79, 0x20, 0x47, 0x77, 0x6c, 0x66, 0x66, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b,
0x6f, 0x6e, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x48,
0x6f, 0x76, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x49, 0x6e, 0x64,
0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x65, 0x66, 0x6e,
0x66, 0x6f, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x2d, 0x54, 0x73, 0x69, 0x65,
0x69, 0x6e, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x61, 0x6e,
0x6f, 0x6c, 0x62, 0x61, 0x72, 0x74, 0x68, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x44,
0x77, 0x79, 0x72, 0x61, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x6f,
0x72, 0x6c, 0x6c, 0x65, 0x77, 0x69, 0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f,
0x6e, 0x65, 0x73, 0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x49,
0x72, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x49, 0x72, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x44, 0x77, 0x79, 0x72, 0x61, 0x69, 0x6e,
0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x47, 0x6f, 0x72, 0x6c, 0x6c, 0x65, 0x77,
0x69, 0x6e, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61,
0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x48, 0x61, 0x66, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61,
0x66, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73,
0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4b, 0x79, 0x72, 0x67, 0x79,
0x7a, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59,
0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x4c, 0x69, 0x6e, 0x65,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x79, 0x72, 0x20, 0x41, 0x72, 0x67,
0x6c, 0x77, 0x79, 0x64, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x79, 0x72, 0x20, 0x41, 0x72, 0x67, 0x6c, 0x77, 0x79, 0x64, 0x64, 0x20,
0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61,
0x66, 0x20, 0x79, 0x72, 0x20, 0x41, 0x72, 0x67, 0x6c, 0x77, 0x79, 0x64,
0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61,
0x66, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69,
0x76, 0x65, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x69, 0x73, 0x65, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65, 0x64, 0x64, 0x20, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x6c, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4d,
0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x48, 0x61, 0x66, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4d, 0x61, 0x77, 0x73,
0x6f, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x61, 0x73, 0x69,
0x66, 0x66, 0x69, 0x67, 0x20, 0x4d, 0x65, 0x63, 0x73, 0x69, 0x63, 0x6f,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x66, 0x69, 0x67, 0x20, 0x4d,
0x65, 0x63, 0x73, 0x69, 0x63, 0x6f, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x66, 0x69, 0x67,
0x20, 0x4d, 0x65, 0x63, 0x73, 0x69, 0x63, 0x6f, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x66, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x66, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61,
0x66, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x66, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x72, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4e,
0x65, 0x77, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x79, 0x64, 0x64, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x79, 0x64,
0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x4e, 0x65, 0x77, 0x79, 0x64, 0x64, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x53, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x4e, 0x65, 0x77, 0x79, 0x64, 0x64, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x53, 0x65, 0x6c,
0x61, 0x6e, 0x64, 0x20, 0x4e, 0x65, 0x77, 0x79, 0x64, 0x64, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4e, 0x69,
0x75, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x59, 0x6e,
0x79, 0x73, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x59, 0x6e, 0x79, 0x73,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62,
0x69, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x4f, 0x6d, 0x73, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x48, 0x61, 0x66, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x61, 0x6c, 0x61,
0x75, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x61, 0x70, 0x75, 0x61,
0x20, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x79,
0x64, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50, 0x61, 0x72, 0x61,
0x67, 0x75, 0x61, 0x79, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x50, 0x65, 0x72, 0x69, 0x77, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x50, 0x65, 0x72,
0x69, 0x77, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x50, 0x65, 0x72, 0x69, 0x77, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x50,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x50, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x48, 0x61, 0x66, 0x20, 0x50, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x61, 0x73, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73,
0x6f, 0x65, 0x64, 0x64, 0x20, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x53, 0x61, 0x69, 0x6e,
0x74, 0x2d, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d,
0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x2d,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x2d, 0x65, 0x74, 0x2d, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65,
0x72, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61,
0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c,
0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x53, 0x61,
0x6d, 0x61, 0x72, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61,
0x66, 0x20, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x53, 0x61, 0x6d,
0x6f, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x6c, 0x65, 0x73, 0x41, 0x6d, 0x73,
0x65, 0x72, 0x20, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x59, 0x6e, 0x79, 0x73, 0x6f, 0x65,
0x64, 0x64, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x44, 0x65, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67,
0x69, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x61, 0x6d, 0x65, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x79,
0x6f, 0x77, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66,
0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x54, 0x61, 0x69,
0x70, 0x65, 0x69, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x61, 0x6a,
0x69, 0x63, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x43,
0x68, 0x75, 0x75, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x54, 0x79,
0x72, 0x63, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20,
0x54, 0x79, 0x72, 0x63, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x54, 0x79,
0x72, 0x63, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x54, 0x75, 0x76, 0x61, 0x6c, 0x75, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61,
0x79, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f,
0x6e, 0x6f, 0x6c, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x41,
0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x56, 0x65,
0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f,
0x6c, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e,
0x6f, 0x6c, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x56, 0x6f,
0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x59, 0x6e, 0x79, 0x73, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61,
0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x53, 0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x48,
0x61, 0x66, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x41, 0x6d,
0x73, 0x65, 0x72, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72, 0x20, 0x53,
0x61, 0x66, 0x6f, 0x6e, 0x6f, 0x6c, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6d, 0x73, 0x65,
0x72, 0x20, 0x48, 0x61, 0x66, 0x20, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6d, 0x73, 0x65, 0x72,
0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x627, 0x648, 0x6af, 0x627, 0x646, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x631, 0x648, 0x686, 0x200c,
0x62f, 0x631, 0x627, 0x62a, 0x6cc, 0x646, 0x20, 0x642, 0x632, 0x627, 0x642, 0x633,
0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x631, 0x648, 0x686, 0x200c,
0x6a9, 0x67e, 0x62a, 0x6cc, 0x646, 0x20, 0x642, 0x632, 0x627, 0x642, 0x633, 0x62a,
0x627, 0x646, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x642, 0x6cc, 0x631, 0x63a, 0x6cc,
0x632, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x648, 0x62e, 0x62a, 0x62a, 0x627,
0x62c, 0x6cc, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x6cc, 0x20, 0x648, 0x62e, 0x62a,
0x41, 0x63, 0x72, 0x65, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41, 0x63, 0x72,
0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x41, 0x63, 0x72, 0x65, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x66, 0x67, 0x68, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73,
0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0xfb, 0x64, 0x2d, 0x41, 0x66,
0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57,
0x65, 0x73, 0x74, 0x2d, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x66, 0x72,
0x69, 0x6b, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x2d, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x2d, 0x41,
0x74, 0x61, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x2d,
0x41, 0x74, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x41, 0x6c, 0x6d, 0x61, 0x2d, 0x41, 0x74, 0x61,
0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x65, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6d, 0x61, 0x7a, 0x6f,
0x6e, 0x65, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x74,
0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69,
0x6e, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61,
0x69, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x2d,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x50, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x50, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41,
0x6e, 0x61, 0x64, 0x79, 0x72, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72,
0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41,
0x71, 0x74, 0x61, 0x75, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41, 0x71, 0x74,
0x61, 0x75, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x41, 0x71, 0x74, 0x61, 0x75, 0x2d, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62,
0x65, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x41, 0x71, 0x74, 0xf6, 0x62, 0x65, 0x2d, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x61, 0x62, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x61, 0x62, 0x79,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x61, 0x62, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x79, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65,
0x73, 0x74, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x79, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x79, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e,
0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x65,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x74, 0x69, 0x69, 0x64,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x69,
0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x77, 0x65,
0x73, 0x74, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x77, 0x65, 0x73, 0x74,
0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65,
0x6e, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b,
0x65, 0x20, 0x77, 0x65, 0x73, 0x74, 0x65, 0x6c, 0x69, 0x6a, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x45,
0x61, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79,
0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x65,
0x69, 0x64, 0x7a, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41,
0x7a, 0x65, 0x72, 0x62, 0x65, 0x69, 0x64, 0x7a, 0x6a, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x41, 0x7a, 0x6f, 0x72,
0x65, 0x6e, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x42, 0x65, 0x6e, 0x67, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x42, 0x68, 0x75,
0x74, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x42, 0x72, 0x61, 0x7a, 0x79, 0x6c, 0x6a,
0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x42,
0x72, 0x61, 0x7a, 0x79, 0x6c, 0x6a, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x42, 0x72, 0x61, 0x7a, 0x79, 0x6c, 0x6a, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x73, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4b, 0x61, 0x61, 0x70, 0x76, 0x65, 0x72, 0x64, 0x79, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x61, 0x61, 0x70, 0x76,
0x65, 0x72, 0x64, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x61, 0x61, 0x70,
0x76, 0x65, 0x72, 0x64, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x43, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x72, 0x6f, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x20, 0x74, 0x69, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74,
0x68, 0x61, 0x6d, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69,
0x6c, 0x65, 0x65, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x53, 0x69, 0x6c, 0x65, 0x65, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69,
0x6c, 0x65, 0x65, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x65, 0x65, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x65, 0x65,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x65, 0x65, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x54,
0x73, 0x6a, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x74,
0x69, 0x69, 0x64, 0x54, 0x73, 0x6a, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x54, 0x73, 0x6a, 0x6f, 0x6a, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x4b, 0x72, 0x79, 0x73, 0x74, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x6b, 0x6f, 0x73,
0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x6c,
0x6f, 0x6d, 0x62, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x43, 0x6f, 0x6f,
0x6b, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x43, 0x6f, 0x6f, 0x6b, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x65,
0x20, 0x68, 0x61, 0x6c, 0x76, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x75, 0x62, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x75, 0x62, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x75, 0x62, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x44, 0x61, 0x76, 0x69, 0x73, 0x20, 0x74, 0x69, 0x69, 0x64, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74,
0x2d, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x50, 0x65, 0x61, 0x73, 0x6b, 0x65, 0x65, 0x69, 0x6c, 0xe2,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x61,
0x73, 0x6b, 0x65, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x50, 0x65, 0x61, 0x73, 0x6b, 0x65, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65,
0x6e, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d,
0x69, 0x64, 0x64, 0x65, 0x6e, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x65, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x45,
0x61, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x65, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x45, 0x75,
0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0xe2, 0x6e, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0xe2, 0x6e, 0x65, 0x69,
0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x46, 0x61, 0x6c, 0x6b, 0x6c,
0xe2, 0x6e, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x46, 0x69, 0x6a,
0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x46, 0x69, 0x6a,
0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x46, 0x69, 0x6a, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x46,
0x72, 0xe2, 0x6e, 0x73, 0x2d, 0x47, 0x75, 0x79, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x46, 0x72, 0xe2, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x53, 0xfa, 0x64, 0x6c, 0x69, 0x6b, 0x65, 0x20, 0x65,
0x6e, 0x20, 0x41, 0x6e, 0x74, 0x61, 0x72, 0x63, 0x74, 0x79, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x65,
0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x79, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73,
0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x47,
0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x57,
0x65, 0x73, 0x74, 0x2d, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0xe2, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x47, 0x75, 0x61, 0x6d, 0x65, 0x73, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x47, 0x6f,
0x6c, 0x66, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x6f, 0x65, 0x74, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f,
0x65, 0x74, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x48, 0x61, 0x77, 0x61, 0x69,
0x69, 0x2d, 0x41, 0x6c, 0x65, 0x6f, 0x65, 0x74, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x48,
0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x73, 0x65, 0x20,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x48, 0x6f,
0x76, 0x64, 0x20, 0x74, 0x69, 0x69, 0x64, 0x48, 0x6f, 0x76, 0x64, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x48, 0x6f, 0x76, 0x64, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x59, 0x6e, 0x64, 0x69, 0x61, 0x61, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x59, 0x6e, 0x64, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x4f, 0x63, 0x65, 0x61, 0x61, 0x6e, 0x2d, 0x74, 0x69, 0x69, 0x64,
0x59, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x65, 0x73, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x2d, 0x59, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73, 0x74, 0x2d, 0x59, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x59, 0x6e, 0x64, 0x6f, 0x6e,
0x65, 0x7a, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x49,
0x72, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x49, 0x72, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x49, 0x72, 0x61,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b,
0x2d, 0x74, 0x69, 0x69, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73,
0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x49, 0x72, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x49, 0x73, 0x72,
0x61, 0xeb, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x49, 0x73, 0x72, 0x61, 0xeb, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x49,
0x73, 0x72, 0x61, 0xeb, 0x6c, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4a, 0x61, 0x70, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76,
0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73,
0x6b, 0x69, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x74,
0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b,
0x61, 0x6d, 0x74, 0x73, 0x6a, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x2d, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x45, 0x61, 0x73,
0x74, 0x2d, 0x4b, 0x61, 0x7a, 0x61, 0x63, 0x68, 0x73, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x57, 0x65, 0x73, 0x74, 0x2d, 0x4b, 0x61, 0x7a, 0x61,
0x63, 0x68, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x4b, 0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x6f, 0x73, 0x72,
0x61, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b,
0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4b,
0x69, 0x72, 0x67, 0x69, 0x7a, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4c, 0x61, 0x6e, 0x6b, 0x61, 0x2d, 0x74, 0x69, 0x69, 0x64,
0x4c, 0x69, 0x6e, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x2d, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x2d, 0x65, 0x69,
0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x73, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x4d, 0x61, 0x63, 0x61, 0x75, 0x73, 0x65, 0x20, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x2d, 0x73,
0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x6c,
0x65, 0x69, 0x73, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65, 0x73, 0x61, 0x73,
0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x65, 0x69, 0x6c,
0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x61,
0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65,
0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e,
0x20, 0x74, 0x69, 0x69, 0x64, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x20, 0x74, 0x69, 0x69, 0x64, 0x55, 0x6c, 0x61,
0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x55, 0x6c, 0x61,
0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f,
0x75, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75, 0x2d, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4d, 0x79, 0x61, 0x6e, 0x6d, 0x61,
0x72, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x61, 0x75,
0x72, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x4e, 0x69, 0x6a, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x69,
0x6a, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x79, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x4e, 0x69, 0x6a, 0x2d, 0x4b, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x79, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x4e, 0x69, 0x6a, 0x2d, 0x53, 0x65, 0x65, 0x6c,
0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x69,
0x6a, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x4e, 0x69, 0x6a, 0x2d, 0x53, 0x65, 0x65, 0x6c, 0xe2, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0xe2, 0x6e, 0x73,
0x6b, 0x65, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x2d, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65,
0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4e,
0x69, 0x75, 0x65, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x65, 0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x74, 0x69, 0x69, 0x64,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x4e, 0x6f, 0x61, 0x72, 0x64, 0x6c, 0x69, 0x6b, 0x65, 0x20,
0x4d, 0x61, 0x72, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x74, 0x69,
0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4f, 0x6d, 0x73, 0x6b, 0x2d,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x42, 0x65, 0x6c, 0x61, 0x75, 0x73, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x50, 0x61, 0x70, 0x6f, 0x65, 0x61, 0x2d, 0x4e, 0x69,
0x6a, 0x2d, 0x47, 0x75, 0x69, 0x6e, 0x65, 0x65, 0x73, 0x6b, 0x65, 0x20,
0x74, 0x69, 0x69, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72,
0x74, 0x74, 0x69, 0x69, 0x64, 0x50, 0x65, 0x72, 0x75, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6a, 0x6e, 0x73, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6a,
0x6e, 0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6a, 0x6e,
0x73, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x65, 0x69, 0x6c, 0xe2,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x69,
0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x65, 0x6e,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x2d, 0x74, 0x69,
0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72,
0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c,
0x6f, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x53, 0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65,
0x72, 0x72, 0x65, 0x20, 0x65, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x65, 0x69,
0x6c, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x20, 0x74, 0x69, 0x69, 0x64,
0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x74, 0x69,
0x69, 0x64, 0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64,
0x51, 0x79, 0x7a, 0x79, 0x6c, 0x6f, 0x72, 0x64, 0x61, 0x2d, 0x73, 0x69,
0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x52, 0xe9, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x52, 0x6f,
0x74, 0x68, 0x65, 0x72, 0x61, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61,
0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x74, 0x69, 0x69, 0x64, 0x53,
0x61, 0x63, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x63, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x74, 0x69,
0x69, 0x64, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x6d,
0x61, 0x72, 0x61, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x69, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x61, 0x6e,
0x73, 0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69,
0x64, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c, 0x73, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72, 0x65,
0x73, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x73, 0x65,
0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x53, 0xfb, 0x64, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x79, 0x73, 0x6b,
0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61,
0x61, 0x6d, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x53, 0x79,
0x6f, 0x77, 0x61, 0x20, 0x74, 0x69, 0x69, 0x64, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x74, 0x69, 0x69, 0x64,
0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x54, 0x61, 0x69, 0x70, 0x65,
0x69, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64,
0x54, 0x61, 0x64, 0x7a, 0x6a, 0x69, 0x65, 0x6b, 0x73, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x2d, 0x65,
0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x54, 0x6f, 0x6e, 0x67, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x61, 0x6e, 0x73, 0x6b,
0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69,
0x69, 0x64, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x43,
0x68, 0x75, 0x75, 0x6b, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x54,
0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20, 0x74, 0x69,
0x69, 0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x73, 0x65, 0x20,
0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x54, 0x75,
0x76, 0x61, 0x6c, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74,
0x69, 0x69, 0x64, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x55, 0x72, 0x75,
0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65,
0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x4f,
0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x4f, 0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4f,
0x65, 0x7a, 0x62, 0x65, 0x65, 0x6b, 0x73, 0x65, 0x20, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x61, 0x61, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69,
0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x61, 0x6e, 0x73,
0x6b, 0x65, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74,
0x69, 0x69, 0x64, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x69, 0x64, 0x46, 0x65, 0x6e, 0x65, 0x7a, 0x6f, 0x6c, 0x61, 0x61,
0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69,
0x64, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x57,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x74, 0x69, 0x69,
0x64, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x57,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x2d, 0x73, 0x69, 0x6d,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x56, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x20, 0x74, 0x69, 0x69, 0x64, 0x57, 0x61, 0x6b, 0x65, 0x2d, 0x65,
0x69, 0x6c, 0xe2, 0x6e, 0x73, 0x6b, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64,
0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x6e, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x73, 0x65, 0x20, 0x74, 0x69, 0x69, 0x64, 0x4a,
0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x74, 0x69, 0x69, 0x64,
0x4a, 0x61, 0x6b, 0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x65, 0x72, 0x74, 0x74, 0x69, 0x69, 0x64, 0x4a, 0x61, 0x6b,
0x6f, 0x65, 0x74, 0x73, 0x6b, 0x2d, 0x73, 0x69, 0x6d, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x74, 0x69, 0x69, 0x64,
0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x65, 0x6e, 0x62,
0x75, 0x72, 0x67, 0x2d, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x65, 0x72, 0x74,
0x74, 0x69, 0x69, 0x64, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x65, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x2d, 0x73, 0x69, 0x6d, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x69, 0x64, 0x43, 0x54, 0x20, 0x28, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x73, 0xe0, 0x6e, 0x74, 0x61, 0x72, 0x61, 0x6c,
0x29, 0x43, 0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20,
0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x75, 0x20, 0x73,
0xe0, 0x6e, 0x74, 0x61, 0x72, 0x61, 0x61, 0x6c, 0x29, 0x43, 0x44, 0x54,
0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63,
0xeb, 0x67, 0x75, 0x20, 0x73, 0xe0, 0x6e, 0x74, 0x61, 0x72, 0x61, 0x61,
0x6c, 0x45, 0x54, 0x20, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x70, 0x65,
0x6e, 0x6b, 0x75, 0x45, 0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x75,
0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x29, 0x45, 0x44, 0x54, 0x20, 0x28,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67,
0x75, 0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x29, 0x4d, 0x54, 0x20, 0x28,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x74, 0x75, 0x6e, 0x64, 0x75, 0x29,
0x4d, 0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65,
0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x75, 0x20, 0x74, 0x75,
0x6e, 0x64, 0x75, 0x29, 0x4d, 0x44, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x75, 0x20, 0x74,
0x75, 0x6e, 0x64, 0x75, 0x29, 0x50, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x29, 0x50,
0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73,
0x74, 0xe0, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x75, 0x20, 0x70, 0x61, 0x73,
0x69, 0x66, 0x69, 0x6b, 0x29, 0x50, 0x44, 0x54, 0x20, 0x28, 0x77, 0x61,
0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67, 0x75, 0x20,
0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x29, 0x41, 0x54, 0x20, 0x28,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x61, 0x74, 0x6c, 0xe0, 0x6e, 0x74,
0x69, 0x6b, 0x29, 0x41, 0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x75,
0x20, 0x70, 0x65, 0x6e, 0x6b, 0x75, 0x29, 0x41, 0x44, 0x54, 0x20, 0x28,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x62, 0xeb, 0x63, 0x63, 0xeb, 0x67,
0x75, 0x20, 0x61, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x69, 0x6b, 0x29, 0x43,
0x54, 0x45, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0xeb, 0x72,
0x6f, 0x6f, 0x70, 0x20, 0x73, 0xe0, 0x6e, 0x74, 0x61, 0x72, 0x61, 0x61,
0x6c, 0x29, 0x43, 0x45, 0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61, 0x61, 0x72, 0x75,
0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x73, 0xe0, 0x6e, 0x74, 0x61,
0x72, 0x61, 0x61, 0x6c, 0x29, 0x43, 0x45, 0x53, 0x54, 0x20, 0x28, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x77, 0x75, 0x20,
0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x73, 0xe0, 0x6e, 0x74, 0x61, 0x72,
0x61, 0x61, 0x6c, 0x29, 0x45, 0x45, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x75, 0x20, 0x70,
0x65, 0x6e, 0x6b, 0x75, 0x29, 0x45, 0x45, 0x53, 0x54, 0x20, 0x28, 0x77,
0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0, 0x6e, 0x64, 0x61,
0x61, 0x72, 0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x75, 0x20,
0x70, 0x65, 0x6e, 0x6b, 0x75, 0x29, 0x45, 0x45, 0x53, 0x54, 0x20, 0x28,
0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x77, 0x75,
0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x75, 0x20, 0x70, 0x65, 0x6e,
0x6b, 0x75, 0x29, 0x57, 0x45, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70, 0x20, 0x75, 0x20, 0x73, 0x6f,
0x77, 0x77, 0x75, 0x2d, 0x6a, 0x61, 0x6e, 0x74, 0x57, 0x45, 0x53, 0x54,
0x20, 0x28, 0x77, 0x61, 0x78, 0x74, 0x75, 0x20, 0x65, 0x73, 0x74, 0xe0,
0x6e, 0x64, 0x61, 0x61, 0x72, 0x75, 0x20, 0xeb, 0x72, 0x6f, 0x6f, 0x70,
0x20, 0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x2d, 0x6a, 0x61, 0x6e,
0x74, 0x29, 0x57, 0x45, 0x53, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78, 0x74,
0x75, 0x20, 0x65, 0x74, 0x65, 0x20, 0x77, 0x75, 0x20, 0xeb, 0x72, 0x6f,
0x6f, 0x70, 0x20, 0x75, 0x20, 0x73, 0x6f, 0x77, 0x77, 0x75, 0x2d, 0x6a,
0x61, 0x6e, 0x74, 0x29, 0x47, 0x4d, 0x54, 0x20, 0x28, 0x77, 0x61, 0x78,
0x74, 0x75, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68,
0x29, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe1, 0x72, 0xed, 0x6e,
0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20,
0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20,
0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20,
0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x66, 0xed, 0x72,
0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd,
0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x4f, 0x6f, 0x72, 0x75,
0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x41, 0x6c, 0xe1, 0x73, 0xed, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41,
0x6c, 0xe1, 0x73, 0xed, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xcc, 0x79, 0xe1, 0x6c, 0x1eb9, 0x300, 0x74, 0x61, 0x20, 0x41, 0x6c, 0xe1,
0x73, 0xed, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x41, 0x6d,
0xe1, 0x73, 0x1ecd, 0x301, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x41, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x6d, 0xe1, 0x73,
0x1ecd, 0x301, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x72,
0x75, 0x20, 0x41, 0x6d, 0xe1, 0x73, 0x1ecd, 0x301, 0x6e, 0xec, 0xe0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0xe1, 0x72, 0xed, 0x6e, 0x20, 0x67, 0x62,
0xf9, 0x6e, 0x67, 0x62, 0xf9, 0x6e, 0xe0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x61, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0xe0, 0xe1, 0x72, 0xed, 0x6e, 0x20,
0x67, 0x62, 0xf9, 0x6e, 0x67, 0x62, 0xf9, 0x6e, 0x41, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xe0, 0xe1, 0x72, 0xed, 0x6e, 0x20, 0x67, 0x62, 0xf9, 0x6e,
0x67, 0x62, 0xf9, 0x6e, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xec, 0x68, 0xe0, 0x20, 0xec, 0x6c,
0xe0, 0x20, 0x6f, 0xf2, 0x72, 0xf9, 0x6e, 0x41, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x73, 0x69, 0x6b, 0xf2, 0x20, 0xcc, 0x68, 0x61, 0x20, 0xcc,
0x6c, 0x61, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0xcc, 0x68,
0xe0, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2, 0x72, 0x75, 0x6e, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xf2, 0x6b, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x61, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0xf2, 0x6b, 0xe8, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd,
0x20, 0x4f, 0x72, 0x69, 0x2d, 0xf2, 0x6b, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x50, 0xe0, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0x73, 0x69, 0x6b, 0xf2, 0x20, 0x50,
0xe0, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xcc, 0x79, 0xe1, 0x6c, 0x1eb9, 0x74, 0x61, 0x20, 0x50, 0xe0,
0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0x41, 0x61, 0x67, 0x6f, 0x20,
0x41, 0x6a, 0x1eb9, 0x6e, 0x74, 0xec, 0x6e, 0xe0, 0x41, 0x61, 0x67, 0x6f,
0x20, 0xe0, 0x73, 0xec, 0x6b, 0xf2, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0x61, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x53, 0x6f, 0x6d,
0x61, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x77, 0x1ecd, 0x301, 0x2d, 0x6f, 0xf2,
0x72, 0xf9, 0x6e, 0x20, 0x41, 0x6a, 0x1eb9, 0x301, 0x6e, 0x74, 0xed, 0x6e,
0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x77, 0x1ecd, 0x301, 0x2d,
0x6f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x41, 0x6a, 0x1eb9, 0x301, 0x6e, 0x74, 0xed, 0x6e, 0xe0, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x72, 0x75, 0x20, 0x49, 0x77, 0x1ecd,
0x301, 0x2d, 0x6f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x6a, 0x1eb9, 0x301,
0x6e, 0x74, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0,
0x74, 0xec, 0x6c, 0xe1, 0x144, 0x74, 0xed, 0xec, 0x6b, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0x73, 0x69, 0x6b, 0xf2, 0x20, 0xc0, 0x74,
0xec, 0x6c, 0xe1, 0x144, 0x74, 0xed, 0xec, 0x6b, 0xec, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xcc, 0x79, 0xe1, 0x6c, 0x1eb9, 0x74, 0x61, 0x20, 0xc0,
0x74, 0xec, 0x6c, 0xe1, 0x144, 0x74, 0xed, 0xec, 0x6b, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc1, 0x73, 0x1ecd, 0x301, 0x73, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0xc1, 0x73, 0x1ecd, 0x301, 0x73, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x4f, 0x6f, 0x72, 0x75, 0x20, 0xc1, 0x73, 0x1ecd, 0x301, 0x73, 0xec, 0x41,
0x61, 0x67, 0x6f, 0x20, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x41,
0x61, 0x67, 0x6f, 0x20, 0x42, 0xf9, 0x72, 0xe0, 0x73, 0xed, 0x6c, 0xed,
0xe0, 0x41, 0x61, 0x67, 0x6f, 0x20, 0xe0, 0x73, 0xec, 0x6b, 0xf2, 0x20,
0x42, 0xf9, 0x72, 0xe0, 0x73, 0xed, 0x6c, 0xed, 0xe0, 0x41, 0x61, 0x67,
0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4b, 0xe9, 0x70,
0xfa, 0x20, 0x46, 0xe1, 0xe0, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4b, 0xe9, 0x70,
0xfa, 0x20, 0x46, 0xe1, 0xe0, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x53, 0x1ecd, 0x301, 0x6d, 0xe0, 0x20, 0x4b, 0xe9, 0x70, 0xfa, 0x20,
0x46, 0xe1, 0xe0, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x1e62,
0xed, 0x6c, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x1e62, 0xed, 0x6c, 0xe8, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4f, 0x72, 0x75, 0x20, 0x1e62, 0xed, 0x6c, 0xe8, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x1e62, 0xe1, 0xed, 0x6e, 0xe0, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x73,
0xed, 0x20, 0x1e62, 0xe1, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x1e62, 0xe1, 0xed,
0x6e, 0xe0, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d,
0x62, 0x69, 0x61, 0x41, 0x61, 0x67, 0x6f, 0x20, 0xe0, 0x73, 0xec, 0x6b,
0xf2, 0x20, 0x4b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x41, 0x61,
0x67, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4b, 0xfa,
0x62, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x4b, 0xfa, 0x62, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x4b, 0xfa,
0x62, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x20,
0x6f, 0x6f, 0x72, 0xf9, 0x6e, 0x20, 0x54, 0x69, 0x6d, 0x6f, 0x72, 0x41,
0x61, 0x67, 0x6f, 0x20, 0x41, 0x6a, 0x6e, 0x64, 0x65, 0x20, 0x49, 0x62,
0xf9, 0x67, 0x62, 0xe9, 0x20, 0x4f, 0x6d, 0x69, 0x41, 0x61, 0x67, 0x6f,
0x20, 0xe0, 0x73, 0xec, 0x6b, 0xf2, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x61, 0x67, 0x6f,
0x20, 0x53, 0x6f, 0x6d, 0x61, 0x20, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x61, 0x67, 0x6f, 0x20,
0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0xe1, 0x72, 0x69, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe1, 0x72, 0x69, 0x6e,
0x20, 0xe0, 0x73, 0x69, 0x6b, 0xf2, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe1, 0x72, 0x69, 0x6e,
0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x68, 0xe0, 0x20, 0xcc, 0x6c,
0xe0, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0x73, 0x69, 0x6b,
0xf2, 0x20, 0xcc, 0x68, 0xe0, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2,
0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0xcc, 0x68, 0x61,
0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x49,
0x77, 0xe1, 0x6a, 0xfa, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2, 0x72,
0xf9, 0x6e, 0x20, 0x59, 0xfa, 0x72, 0xf3, 0xf2, 0x70, 0xf9, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x20, 0x4f, 0xf2, 0x72, 0xf9,
0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xe0, 0x73, 0x69, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x20,
0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0xcc,
0x68, 0xe0, 0x20, 0xcc, 0x77, 0x1ecd, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x46, 0xf3, 0x6b, 0xed, 0x6c, 0xe1, 0x6e, 0x64, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x45, 0x74, 0xed, 0x6b, 0x75, 0x6e, 0x20, 0x46, 0xf3, 0x6b, 0xed, 0x6c,
0xe1, 0x6e, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f,
0x72, 0x75, 0x20, 0x45, 0x74, 0xed, 0x6b, 0x75, 0x6e, 0x20, 0x46, 0xf3,
0x6b, 0xed, 0x6c, 0xe1, 0x6e, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x47, 0xfa, 0x79, 0xe1, 0x6e, 0xe0, 0x20, 0x46, 0xe1, 0x72, 0xe1,
0x6e, 0x73, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x47, 0xfa, 0x73,
0xf9, 0x20, 0x46, 0xe1, 0x72, 0x61, 0x6e, 0x73, 0xe9, 0x20, 0xe0, 0x74,
0x69, 0x20, 0xc0, 0x6e, 0x74, 0xe1, 0x74, 0xed, 0xec, 0x6b, 0xec, 0x41,
0x61, 0x67, 0x6f, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x6f, 0x6f,
0x72, 0xf9, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x20,
0xcc, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x6f, 0x6f, 0x72, 0xf9,
0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xec, 0x67, 0x62, 0xe0, 0x20, 0x4f, 0x6f,
0x72, 0x75, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x20, 0x6f,
0x6f, 0x72, 0xf9, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x20, 0x4f, 0xf2, 0x72,
0xf9, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0xcc, 0x67, 0x62, 0x61, 0x20, 0x4f, 0xf2, 0x72, 0x75, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x47, 0xfa, 0x79, 0xe0, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x48, 0x61, 0x77, 0x61,
0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd,
0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x69, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x45, 0x74,
0xed, 0x6b, 0x75, 0x6e, 0x20, 0xcd, 0x6e, 0x64, 0xed, 0xe0, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e,
0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe0, 0x72, 0x69, 0x6e,
0x20, 0x47, 0x62, 0xf9, 0x6e, 0x67, 0x62, 0xf9, 0x6e, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xcc, 0x77, 0x1ecd, 0x300, 0x20, 0x6f, 0x6f, 0x72, 0xf9, 0x6e, 0x20,
0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xcd, 0x6b, 0xf3, 0x73, 0xed, 0x6b, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0xcd, 0x6b, 0xf3, 0x73, 0xed, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x53, 0x1ecd, 0x6d, 0xe0, 0x20, 0xcd, 0x6b, 0xfa, 0x73, 0xed, 0x6b,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4d, 0xe1, 0x72, 0xed, 0x1e63,
0xfa, 0x1e63, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4d, 0xe1, 0x72, 0xed, 0x1e63, 0xfa, 0x1e63,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72, 0x75, 0x20,
0x4d, 0xe1, 0x72, 0xed, 0x1e63, 0xfa, 0x1e63, 0xec, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x50, 0xe1, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0x20,
0x4d, 0x1eb9, 0x301, 0x1e63, 0xed, 0x6b, 0xf2, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x73,
0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0x20, 0x4d, 0x1eb9, 0x301, 0x73, 0xed,
0x6b, 0xf2, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d,
0x1ecd, 0x6d, 0x1ecd, 0x20, 0x50, 0xe1, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b,
0xec, 0x20, 0x4d, 0x1eb9, 0x301, 0x73, 0xed, 0x6b, 0xf2, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c,
0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e,
0x64, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x41, 0x61, 0x67, 0x6f, 0x20,
0x46, 0x65, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x61, 0x41, 0x61, 0x67, 0x6f, 0x20, 0xe0, 0x73,
0xec, 0x6b, 0xf2, 0x20, 0x46, 0x65, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20,
0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x61, 0x41, 0x61, 0x67,
0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x72, 0xe1,
0x67, 0xfa, 0x77, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x72, 0xe1, 0x67, 0xfa,
0x77, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72, 0x75,
0x20, 0x50, 0xe1, 0x72, 0xe1, 0x67, 0xfa, 0x77, 0xe8, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x50, 0xe9, 0x72, 0xf9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe9, 0x72,
0xf9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72, 0x75, 0x20,
0x50, 0xe9, 0x72, 0xf9, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0x69,
0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65,
0x6c, 0x6f, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x1eb9,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x1ecd, 0x6d, 0x1ecd, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x52,
0x1eb9, 0x301, 0x79, 0xfa, 0x6e, 0xed, 0x1ecd, 0x301, 0x6e, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0xe8, 0x1e63, 0x1eb9, 0x301, 0x1eb9, 0x300, 0x6c,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x47, 0xfa, 0x73, 0xf9, 0x20,
0x4a, 0x1ecd, 0x301, 0x6a, 0xed, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0x53, 0xfa, 0x72, 0xed, 0x6e, 0xe1, 0x6d, 0xf9, 0x41, 0x61, 0x67, 0x6f,
0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x1eb9, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xda, 0x72,
0xfa, 0x67, 0xfa, 0x77, 0xe8, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x53, 0x6f,
0x6d, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x41, 0x61,
0x67, 0x6f, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c, 0x61,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x59, 0xfa, 0x6b, 0x1ecd, 0x301, 0x6e,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d,
0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b,
0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d,
0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x41, 0x66, 0xed, 0x72, 0x69, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x2d, 0x4f, 0xf2, 0x72, 0xf9,
0x6e, 0x20, 0x4f, 0x6f, 0x72, 0x75, 0x20, 0x41, 0x66, 0xed, 0x72, 0x69,
0x6b, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x41, 0x6c, 0xe1, 0x73, 0xed, 0x6b, 0xe0, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x79, 0xe1, 0x6c, 0x25b, 0x300, 0x74,
0x61, 0x20, 0x41, 0x6c, 0xe1, 0x73, 0xed, 0x6b, 0xe0, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x41, 0x6d, 0xe1, 0x73, 0x254, 0x301, 0x6e, 0xec, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x41, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0x41, 0x6d, 0xe1, 0x73, 0x254, 0x301, 0x6e, 0xec, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x4f, 0x72, 0x75, 0x20, 0x41, 0x6d, 0xe1, 0x73, 0x254,
0x301, 0x6e, 0xec, 0x41, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xe0, 0xe1, 0x72,
0xed, 0x6e, 0x20, 0x67, 0x62, 0xf9, 0x6e, 0x67, 0x62, 0xf9, 0x6e, 0x20,
0x6f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0xcc, 0x68, 0xe0,
0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2, 0x72, 0x75, 0x6e, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x6f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20,
0x4f, 0x72, 0x69, 0x2d, 0xf2, 0x6b, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xcc, 0x79, 0xe1, 0x6c, 0x25b, 0x74, 0x61, 0x20, 0x50, 0xe0, 0x73,
0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0x41, 0x61, 0x67, 0x6f, 0x20, 0x41,
0x6a, 0x25b, 0x6e, 0x74, 0xec, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x49, 0x77, 0x254, 0x301, 0x2d, 0x6f, 0xf2, 0x72, 0xf9, 0x6e, 0x20,
0x41, 0x6a, 0x25b, 0x301, 0x6e, 0x74, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x49, 0x77, 0x254, 0x301, 0x2d, 0x6f, 0xf2, 0x72, 0xf9,
0x6e, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x41, 0x6a,
0x25b, 0x301, 0x6e, 0x74, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x72, 0x75, 0x20, 0x49, 0x77, 0x254, 0x301, 0x2d, 0x6f, 0xf2,
0x72, 0xf9, 0x6e, 0x20, 0x41, 0x6a, 0x25b, 0x301, 0x6e, 0x74, 0xed, 0x6e,
0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x79, 0xe1, 0x6c, 0x25b,
0x74, 0x61, 0x20, 0xc0, 0x74, 0xec, 0x6c, 0xe1, 0x144, 0x74, 0xed, 0xec,
0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc1, 0x73, 0x254, 0x301,
0x73, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0xc1, 0x73, 0x254, 0x301, 0x73, 0xec, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72, 0x75, 0x20, 0xc1, 0x73, 0x254,
0x301, 0x73, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4b, 0xe9, 0x70, 0xfa, 0x20, 0x46, 0xe1,
0xe0, 0x64, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x301,
0x6d, 0xe0, 0x20, 0x4b, 0xe9, 0x70, 0xfa, 0x20, 0x46, 0xe1, 0xe0, 0x64,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x68, 0xed, 0x6c, 0xe8,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x53, 0x68, 0xed, 0x6c, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x72, 0x75, 0x20, 0x53, 0x68, 0xed, 0x6c, 0xe8, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x68, 0xe1, 0xed, 0x6e, 0xe0, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x73,
0xed, 0x20, 0x53, 0x68, 0xe1, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x53, 0x68,
0xe1, 0xed, 0x6e, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4b, 0xfa, 0x62, 0xe0, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20,
0x4b, 0xfa, 0x62, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0xe1,
0x72, 0x69, 0x6e, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d,
0xe0, 0x20, 0xcc, 0x68, 0x61, 0x20, 0xcc, 0x6c, 0xe0, 0x20, 0x4f, 0xf2,
0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x20, 0x4f, 0xf2, 0x72, 0xf9,
0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xe0, 0x73, 0x69, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x20,
0x4f, 0xf2, 0x72, 0xf9, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53, 0x254, 0x6d, 0xe0, 0x20, 0xcc,
0x68, 0xe0, 0x20, 0xcc, 0x77, 0x254, 0x20, 0x4f, 0xf2, 0x72, 0xf9, 0x6e,
0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x45, 0x74, 0xed,
0x6b, 0x75, 0x6e, 0x20, 0x46, 0xf3, 0x6b, 0xed, 0x6c, 0xe1, 0x6e, 0x64,
0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x20,
0xcc, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x6f, 0x6f, 0x72, 0xf9,
0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x20, 0x6f, 0x6f,
0x72, 0xf9, 0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75,
0x6b, 0xf2, 0x20, 0xcc, 0x77, 0x254, 0x300, 0x20, 0x4f, 0xf2, 0x72, 0xf9,
0x6e, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2,
0x20, 0xcc, 0x67, 0x62, 0x61, 0x20, 0x4f, 0xf2, 0x72, 0x75, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61,
0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254,
0x6d, 0x254, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c,
0x65, 0x75, 0x74, 0x69, 0x61, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xcc, 0x77, 0x254, 0x300, 0x20, 0x6f, 0x6f, 0x72, 0xf9, 0x6e, 0x20, 0x49,
0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0xc0, 0x6b, 0xf3, 0x6b,
0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0xcd, 0x6b,
0xf3, 0x73, 0xed, 0x6b, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53,
0x254, 0x6d, 0xe0, 0x20, 0xcd, 0x6b, 0xfa, 0x73, 0xed, 0x6b, 0xec, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4d, 0xe1, 0x72, 0xed, 0x73, 0x68, 0xfa,
0x73, 0x68, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b,
0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x4d, 0xe1, 0x72, 0xed, 0x73, 0x68, 0xfa,
0x73, 0x68, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6f, 0x72,
0x75, 0x20, 0x4d, 0xe1, 0x72, 0xed, 0x73, 0x68, 0xfa, 0x73, 0x68, 0xec,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x73, 0xed, 0x66, 0xed,
0xec, 0x6b, 0xec, 0x20, 0x4d, 0x25b, 0x301, 0x73, 0x68, 0xed, 0x6b, 0xf2,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b,
0xf2, 0x20, 0x50, 0xe1, 0x73, 0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0x20,
0x4d, 0x25b, 0x301, 0x73, 0xed, 0x6b, 0xf2, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2,
0x20, 0x4f, 0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x50, 0xe1, 0x73,
0xed, 0x66, 0xed, 0xec, 0x6b, 0xec, 0x20, 0x4d, 0x25b, 0x301, 0x73, 0xed,
0x6b, 0xf2, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f, 0x6a,
0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75,
0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20,
0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe1, 0x72, 0xe1,
0x67, 0xfa, 0x77, 0xe8, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66,
0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20, 0x50, 0xe9, 0x72, 0xf9, 0xc0, 0x6b,
0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e, 0x75, 0x6b, 0xf2, 0x20,
0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71,
0x75, 0x65, 0x6c, 0x6f, 0x6e, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x4f,
0x6a, 0xfa, 0x6d, 0x254, 0x6d, 0x254, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72,
0x65, 0x20, 0x26, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x52, 0x25b, 0x301, 0x79, 0xfa, 0x6e,
0xed, 0x254, 0x301, 0x6e, 0xec, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x53,
0xe8, 0x73, 0x68, 0x25b, 0x301, 0x25b, 0x300, 0x6c, 0xec, 0xc0, 0x6b, 0xf3,
0x6b, 0xf2, 0x20, 0x47, 0xfa, 0x73, 0xf9, 0x20, 0x4a, 0x254, 0x301, 0x6a,
0xed, 0xe0, 0xc0, 0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0xc0, 0x66, 0x25b, 0x6e,
0x75, 0x6b, 0xf2, 0x20, 0xda, 0x72, 0xfa, 0x67, 0xfa, 0x77, 0xe8, 0xc0,
0x6b, 0xf3, 0x6b, 0xf2, 0x20, 0x59, 0xfa, 0x6b, 0x254, 0x301, 0x6e, 0xec,
0x47, 0x77, 0x7a, 0x6c, 0x69, 0x6e, 0x7a, 0x76, 0x65, 0x69, 0x68, 0x63,
0x69, 0x20, 0x42, 0x79, 0x61, 0x75, 0x68, 0x63, 0x75, 0x6e, 0x6a, 0x20,
0x53, 0x69, 0x7a, 0x67, 0x65, 0x6e, 0x68, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x66,
0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x66, 0x72, 0x69,
0x63, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x4d, 0x70, 0x75, 0x6d, 0x61, 0x6c, 0x61, 0x6e,
0x67, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4e, 0x69, 0x6e,
0x67, 0x69, 0x7a, 0x69, 0x6d, 0x75, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x4e, 0x74, 0x73, 0x68, 0x6f, 0x6e, 0x61, 0x6c, 0x61,
0x6e, 0x67, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76,
0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4e, 0x74,
0x73, 0x68, 0x6f, 0x6e, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x20, 0x73, 0x61, 0x73, 0x65, 0x4e, 0x74, 0x73, 0x68, 0x6f, 0x6e, 0x61,
0x6c, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77,
0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d,
0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x65, 0x73,
0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x20, 0x73, 0x61,
0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f,
0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e,
0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e,
0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x65, 0x73,
0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65,
0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e,
0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77,
0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x20, 0x73, 0x61, 0x73, 0x65,
0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68,
0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x4d, 0x6f,
0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20,
0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x20, 0x65, 0x73, 0x69,
0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x6e, 0x20, 0x4d, 0x6f, 0x75, 0x6e, 0x74, 0x61, 0x69,
0x6e, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20, 0x41, 0x6d, 0x65, 0x72,
0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63,
0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b,
0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x74, 0x68, 0x20,
0x41, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e,
0x69, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6e, 0x61, 0x64, 0x79,
0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x65, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x41, 0x6e, 0x61, 0x64, 0x79, 0x72, 0x20, 0x53, 0x75, 0x6d,
0x6d, 0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41,
0x70, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x70, 0x69, 0x61, 0x20, 0x65,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41,
0x70, 0x69, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76,
0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41,
0x72, 0x61, 0x62, 0x69, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6e, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69,
0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x4e, 0x79, 0x61, 0x6b, 0x61, 0x74, 0x68, 0x6f, 0x20, 0x6e,
0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x4e, 0x79, 0x61, 0x6b, 0x61, 0x74, 0x68, 0x6f, 0x20, 0x6e, 0x65,
0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x4e, 0x79, 0x61, 0x6b, 0x61, 0x74, 0x68, 0x6f, 0x20,
0x6e, 0x65, 0x2d, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69,
0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x65, 0x73, 0x69, 0x6a,
0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x20, 0x73, 0x61,
0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d,
0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c,
0x20, 0x57, 0x65, 0x73, 0x74, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x57, 0x65, 0x73, 0x74, 0x20, 0x65, 0x73, 0x69, 0x76,
0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x6e, 0x20, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20,
0x57, 0x65, 0x73, 0x74, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73,
0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x45, 0x61, 0x73, 0x74, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x6e, 0x20, 0x45, 0x61, 0x73, 0x74, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57, 0x65,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x6e, 0x20, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e,
0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x6e, 0x20,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x73, 0x61, 0x73, 0x65,
0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x7a, 0x65, 0x72, 0x62,
0x61, 0x69, 0x6a, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61,
0x69, 0x6a, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61,
0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x6a,
0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x73, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x65, 0x73, 0x69, 0x76,
0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x61, 0x6e, 0x67,
0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x42, 0x68, 0x75, 0x74, 0x61,
0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20, 0x65, 0x73,
0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x61, 0x20,
0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x42, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73,
0x73, 0x61, 0x6c, 0x61, 0x6d, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x61, 0x70, 0x65,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x65, 0x7a, 0x69, 0x6e, 0x67, 0x65,
0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x61, 0x70, 0x65,
0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x61, 0x70, 0x65, 0x20, 0x56, 0x65,
0x72, 0x64, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76,
0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43,
0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x43, 0x68, 0x69, 0x6c, 0x65, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61,
0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43,
0x68, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f,
0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76,
0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43,
0x68, 0x69, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x43, 0x68, 0x69, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68,
0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x6f,
0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x20, 0x65, 0x2d, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d, 0x61,
0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43,
0x6f, 0x63, 0x6f, 0x73, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x65, 0x73, 0x69, 0x79, 0x69, 0x6e, 0x67, 0x78, 0x65, 0x6e, 0x79, 0x65,
0x20, 0x79, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x75, 0x62, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x43, 0x75, 0x62, 0x61, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77,
0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x43, 0x75, 0x62, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e,
0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x44, 0x61, 0x76, 0x69, 0x73, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x20,
0x54, 0x69, 0x6d, 0x6f, 0x72, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74,
0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x45, 0x61, 0x73, 0x74, 0x65, 0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65,
0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73, 0x61, 0x73,
0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x63, 0x75,
0x61, 0x64, 0x6f, 0x72, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77,
0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65,
0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61,
0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x46, 0x75, 0x72, 0x74, 0x68, 0x65, 0x72, 0x2d, 0x65,
0x61, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65,
0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x65, 0x73, 0x69, 0x6a,
0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x49, 0x73,
0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c,
0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x69, 0x6a, 0x69, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76,
0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46,
0x69, 0x6a, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x46, 0x69, 0x6a, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x72, 0x65,
0x6e, 0x63, 0x68, 0x20, 0x47, 0x75, 0x69, 0x61, 0x6e, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x46, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x53, 0x6f, 0x75, 0x74,
0x68, 0x65, 0x72, 0x6e, 0x20, 0x6e, 0x61, 0x73, 0x65, 0x2d, 0x41, 0x6e,
0x74, 0x61, 0x72, 0x63, 0x74, 0x69, 0x63, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x61,
0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x61,
0x6d, 0x62, 0x69, 0x65, 0x72, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x65,
0x6f, 0x72, 0x67, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20,
0x4d, 0x65, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x45, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61,
0x6e, 0x64, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c,
0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74,
0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73,
0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57, 0x65,
0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61,
0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57,
0x65, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69,
0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x47, 0x75, 0x6c, 0x66, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x47, 0x75, 0x79, 0x61, 0x6e,
0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20,
0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69,
0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65,
0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x6f, 0x6e, 0x67, 0x20,
0x4b, 0x6f, 0x6e, 0x67, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f,
0x6e, 0x67, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x48, 0x6f, 0x76, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x6d, 0x69, 0x73, 0x69, 0x77, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x48, 0x6f, 0x76, 0x64, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c,
0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x48, 0x6f, 0x76, 0x64, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x6e, 0x64, 0x69, 0x61,
0x6e, 0x20, 0x4f, 0x63, 0x65, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x6e,
0x64, 0x6f, 0x63, 0x68, 0x69, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x6c, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x73, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x45, 0x61, 0x73, 0x74, 0x65, 0x72,
0x6e, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x57, 0x65, 0x73, 0x74, 0x65, 0x72, 0x6e, 0x20, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x72,
0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x72, 0x61, 0x6e, 0x20, 0x65, 0x73,
0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x72,
0x61, 0x6e, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x20, 0x65, 0x2d, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69,
0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x49, 0x73, 0x72, 0x61,
0x65, 0x6c, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x6b,
0x77, 0x61, 0x2d, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x4a, 0x61, 0x70, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4a, 0x61, 0x70, 0x61, 0x6e, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x6d,
0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4a, 0x61, 0x70,
0x61, 0x6e, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x65, 0x74, 0x72,
0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61,
0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x54, 0x69, 0x6d,
0x65, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x65, 0x74, 0x72, 0x6f,
0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d,
0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x65, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x53, 0x75, 0x6d, 0x6d, 0x65, 0x72, 0x20, 0x54,
0x69, 0x6d, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x70, 0x75, 0x6d, 0x61, 0x6c,
0x61, 0x6e, 0x67, 0x61, 0x20, 0x6e, 0x65, 0x2d, 0x4b, 0x61, 0x7a, 0x61,
0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x4e, 0x74, 0x73, 0x68,
0x6f, 0x6e, 0x61, 0x6c, 0x61, 0x6e, 0x67, 0x61, 0x20, 0x6e, 0x65, 0x2d,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x68, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c,
0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4b, 0x6f, 0x72, 0x65, 0x61,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65,
0x6d, 0x69, 0x6e, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4b, 0x6f,
0x72, 0x65, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d,
0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4b, 0x79, 0x72, 0x67, 0x79, 0x73, 0x74, 0x61, 0x6e, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4c, 0x69, 0x6e, 0x65, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69,
0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61, 0x67, 0x61,
0x64, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65,
0x2d, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d,
0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61,
0x6c, 0x64, 0x69, 0x76, 0x65, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61, 0x72,
0x71, 0x75, 0x65, 0x73, 0x61, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61, 0x72,
0x73, 0x68, 0x61, 0x6c, 0x6c, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64,
0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75,
0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61,
0x75, 0x72, 0x69, 0x74, 0x69, 0x75, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x61,
0x77, 0x73, 0x6f, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x65, 0x78, 0x69, 0x63,
0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69, 0x63, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x4d, 0x65, 0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63,
0x69, 0x66, 0x69, 0x63, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79,
0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x65,
0x78, 0x69, 0x63, 0x61, 0x6e, 0x20, 0x50, 0x61, 0x63, 0x69, 0x66, 0x69,
0x63, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69,
0x6d, 0x69, 0x73, 0x69, 0x77, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c,
0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42,
0x61, 0x74, 0x6f, 0x72, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x6f, 0x73, 0x63, 0x6f,
0x77, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4d, 0x6f, 0x73, 0x63, 0x6f, 0x77, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x4d,
0x6f, 0x73, 0x63, 0x6f, 0x77, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x79, 0x61, 0x6e,
0x6d, 0x61, 0x72, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65,
0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f,
0x6e, 0x69, 0x61, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c,
0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65,
0x61, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65,
0x61, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x20, 0x5a, 0x65, 0x61,
0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x66, 0x6f,
0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65,
0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64, 0x6c, 0x61, 0x6e, 0x64, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4e, 0x65, 0x77, 0x66, 0x6f, 0x75, 0x6e, 0x64,
0x6c, 0x61, 0x6e, 0x64, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69, 0x6e,
0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x4e, 0x69, 0x75, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e,
0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x20, 0x65, 0x73, 0x69,
0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x72,
0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x65, 0x72, 0x6e, 0x61,
0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e,
0x68, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x4f, 0x6d, 0x73, 0x6b, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x4f, 0x6d,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x4f, 0x6d, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d,
0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x50, 0x61, 0x6c, 0x61, 0x75, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50,
0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x77, 0x20, 0x47, 0x75, 0x69,
0x6e, 0x65, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x61, 0x79, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61,
0x79, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x73,
0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50,
0x65, 0x72, 0x75, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x65, 0x72, 0x75, 0x20, 0x65,
0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50,
0x65, 0x72, 0x75, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62,
0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69,
0x6e, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x50, 0x68, 0x69, 0x6c, 0x69, 0x70, 0x70, 0x69, 0x6e, 0x65, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20, 0x49, 0x73, 0x6c,
0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6e, 0x61, 0x73, 0x65,
0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x49, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53,
0x61, 0x69, 0x6e, 0x74, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20,
0x6e, 0x61, 0x73, 0x65, 0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f,
0x6e, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65,
0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61, 0x69, 0x6e, 0x74,
0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x6e, 0x61, 0x73, 0x65,
0x2d, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x73, 0x61,
0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x6f, 0x6e, 0x61,
0x70, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61,
0x6e, 0x67, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x52, 0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x52, 0x6f, 0x74, 0x68, 0x65, 0x72, 0x61, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61,
0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61,
0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62,
0x6f, 0x20, 0x65, 0x2d, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x20, 0x54, 0x69, 0x6d, 0x65, 0x65, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53,
0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x65, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20, 0x53, 0x75, 0x6d, 0x6d,
0x65, 0x72, 0x20, 0x54, 0x69, 0x6d, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61,
0x6d, 0x6f, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x73, 0x61, 0x73, 0x65, 0x6d, 0x69,
0x6e, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x65, 0x79, 0x63, 0x68, 0x65, 0x6c,
0x6c, 0x65, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x53, 0x69, 0x6e, 0x67, 0x61, 0x70, 0x6f, 0x72,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f, 0x6e, 0x20,
0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x73, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x53, 0x6f,
0x75, 0x74, 0x68, 0x20, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73,
0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x6d, 0x69, 0x6e, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x49,
0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73,
0x65, 0x2d, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x73, 0x61, 0x73,
0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x43, 0x68, 0x75,
0x75, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x54, 0x75, 0x76, 0x61,
0x6c, 0x75, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x73, 0x61, 0x73, 0x65, 0x2d, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x65,
0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c, 0x65, 0x6b, 0x69, 0x6c,
0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20,
0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d,
0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x65, 0x73, 0x69, 0x76,
0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x55,
0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x65, 0x68, 0x6c, 0x6f,
0x62, 0x6f, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x55, 0x7a, 0x62, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61,
0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x56, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x56, 0x61, 0x6e, 0x75, 0x61,
0x74, 0x75, 0x20, 0x65, 0x73, 0x69, 0x6a, 0x77, 0x61, 0x79, 0x65, 0x6c,
0x65, 0x6b, 0x69, 0x6c, 0x65, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62,
0x6f, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x56, 0x65, 0x6e, 0x65, 0x7a, 0x75, 0x65, 0x6c,
0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73,
0x61, 0x73, 0x65, 0x2d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65,
0x2d, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64,
0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73,
0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x2d, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x20, 0x73,
0x61, 0x73, 0x65, 0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x49, 0x73, 0x69, 0x6b,
0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x56,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74,
0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57, 0x61, 0x6b, 0x65,
0x20, 0x49, 0x73, 0x6c, 0x61, 0x6e, 0x64, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x57, 0x61,
0x6c, 0x6c, 0x69, 0x73, 0x20, 0x6e, 0x61, 0x73, 0x65, 0x2d, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68,
0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20,
0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69, 0x6c, 0x65, 0x20, 0x73, 0x61,
0x73, 0x65, 0x2d, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x49, 0x73,
0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65,
0x68, 0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x49, 0x73, 0x69, 0x6b, 0x68, 0x61, 0x74, 0x68, 0x69,
0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x49, 0x73, 0x69, 0x6b, 0x68,
0x61, 0x74, 0x68, 0x69, 0x20, 0x45, 0x73, 0x69, 0x76, 0x61, 0x6d, 0x69,
0x6c, 0x65, 0x20, 0x73, 0x61, 0x73, 0x65, 0x2d, 0x59, 0x65, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x49, 0x73, 0x69,
0x6b, 0x68, 0x61, 0x74, 0x68, 0x69, 0x20, 0x73, 0x61, 0x73, 0x65, 0x68,
0x6c, 0x6f, 0x62, 0x6f, 0x20, 0x65, 0x2d, 0x59, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x41, 0x6b, 0x72, 0x65,
0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x41, 0x6b, 0x72, 0x65, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20,
0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x6b, 0x72, 0x65, 0x20,
0x74, 0xe1, 0x41, 0x66, 0x65, 0x67, 0x61, 0x6e, 0x129, 0x74, 0xe3, 0x20,
0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61,
0x2d, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72,
0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0x53, 0x75, 0x72, 0x2d, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72,
0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x52, 0xe3, 0x70,
0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x52, 0xe3,
0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x41, 0x72, 0x61, 0x6e, 0x68, 0x6b,
0x61, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x41, 0x72, 0x61, 0x6e, 0x68, 0x6b, 0x61, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41,
0x72, 0x61, 0x6e, 0x68, 0x6b, 0x61, 0x20, 0x74, 0xe1, 0x41, 0x72, 0x6d,
0x61, 0x74, 0x79, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x72, 0x6d, 0x61, 0x74, 0x79, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x41, 0x72, 0x6d, 0x61, 0x74, 0x79, 0x20, 0x74, 0xe1, 0x41, 0x6d, 0x1ef9,
0x6a, 0x6f, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x6d, 0x1ef9, 0x6a, 0x6f, 0x6e,
0x1ef9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x41, 0x6d, 0x1ef9, 0x6a, 0x6f, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x52,
0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75,
0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x72, 0x129, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x72, 0x129,
0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61,
0x20, 0x4b, 0x72, 0x129, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x52,
0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75,
0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x6e, 0x1ef9, 0x6e,
0x6e, 0x79, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x41, 0x6e, 0x1ef9, 0x6e, 0x68, 0x79, 0x72, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x6e, 0x1ef9,
0x6e, 0x68, 0x79, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41,
0x70, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x41, 0x70, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x70, 0x69, 0x6a,
0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x67, 0x74, 0x61,
0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41,
0x67, 0x74, 0x61, 0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x67, 0x74, 0x61, 0x76, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x67, 0x74, 0xf3, 0x6d, 0x65, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x67, 0x74,
0xf3, 0x6d, 0x65, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x41, 0x67, 0x74, 0xf3, 0x6d, 0x65, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x61, 0x6d, 0x69, 0x6a, 0x61,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x72,
0x61, 0x6d, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x61, 0x6d, 0x69, 0x6a,
0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x1ebd,
0x74, 0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4f,
0x70, 0xe3, 0x20, 0x41, 0x72, 0x6a, 0x1ebd, 0x74, 0x129, 0x6e, 0x1ef9, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x41, 0x72, 0x6a, 0x1ebd, 0x74, 0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x1ebd, 0x74, 0x129, 0x6e, 0x1ef9, 0x20,
0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x41, 0x72, 0x6a, 0x1ebd, 0x74, 0x129, 0x6e, 0x1ef9, 0x20,
0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6a, 0x1ebd, 0x74, 0x129,
0x6e, 0x1ef9, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x41, 0x72, 0x6d, 0x1ebd, 0x6e, 0x129, 0x6a, 0x61, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x72, 0x6d, 0x1ebd,
0x6e, 0x129, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x72, 0x6d, 0x1ebd, 0x6e, 0x129, 0x6a,
0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x74, 0x72, 0x1ef9,
0x74, 0x69, 0x6b, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x41, 0x74, 0x72, 0x1ef9, 0x74, 0x69, 0x6b, 0x75, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41,
0x74, 0x72, 0x1ef9, 0x74, 0x69, 0x6b, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61,
0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69,
0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20,
0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79,
0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x61, 0x72,
0x69, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x2d, 0x52, 0xe3, 0x70, 0x75,
0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41,
0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x4b, 0x75,
0x6a, 0x75, 0x2d, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f,
0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75,
0x2d, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20,
0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69,
0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f, 0x74,
0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79,
0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x76, 0x6f,
0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75,
0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x41, 0x76, 0x6f, 0x74, 0x79, 0x72, 0x61, 0x72, 0x69, 0x61,
0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x41, 0x6a, 0x65, 0x72, 0x6d, 0x61, 0x6a, 0x6a, 0xe1, 0x76, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x6a, 0x65,
0x72, 0x6d, 0x61, 0x6a, 0x6a, 0xe1, 0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x41, 0x6a, 0x65, 0x72,
0x6d, 0x61, 0x6a, 0x6a, 0xe1, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x41, 0x73, 0x6f, 0x72, 0x2d, 0x61, 0x67, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x41, 0x73, 0x6f, 0x72, 0x2d, 0x61,
0x67, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x41, 0x73, 0x6f, 0x2d, 0x61, 0x67, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x6e, 0x67, 0x72, 0x61, 0x6e, 0xe9, 0x73,
0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d,
0x1ef9, 0x6e, 0x67, 0x72, 0x61, 0x6e, 0xe9, 0x73, 0x69, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9,
0x6e, 0x67, 0x72, 0x61, 0x6e, 0xe9, 0x73, 0x69, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4d, 0x75, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4d, 0x6f, 0x72, 0x69, 0x76, 0x69, 0x61, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x72, 0x61, 0x73, 0x69, 0x72, 0x6a,
0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d,
0x72, 0x61, 0x73, 0x69, 0x72, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4d, 0x72, 0x61, 0x73,
0x69, 0x72, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d,
0x72, 0x75, 0x6e, 0x1ebd, 0x6a, 0x20, 0x4e, 0x61, 0x72, 0x75, 0x73, 0x61,
0x72, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x75, 0x20,
0x54, 0xe1, 0x6e, 0x68, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x50, 0x75, 0x20, 0x54, 0xe1, 0x6e, 0x68, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x50, 0x75,
0x20, 0x74, 0xe1, 0x6e, 0x68, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x53, 0x61, 0x6d, 0xe3, 0x68, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x53, 0x61, 0x6e, 0x2d, 0x68, 0x1ef9, 0x6d, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x61, 0x6e, 0x2d, 0x68, 0x1ef9,
0x6d, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x53, 0x61, 0x6e, 0x2d, 0x68, 0x1ef9, 0x6d, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x53, 0x69, 0x72, 0x65, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x69, 0x72, 0x65, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x53,
0x69, 0x72, 0x65, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x129,
0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20,
0x53, 0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x53, 0x129, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x53, 0x6f, 0x6a, 0x6d, 0x61, 0x72, 0x73, 0x1ef9,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x6f,
0x6a, 0x6d, 0x61, 0x72, 0x73, 0x1ef9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20,
0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x53, 0x6f, 0x6a, 0x6d, 0x61,
0x72, 0x73, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x72,
0x129, 0x74, 0x69, 0x6d, 0x1ef9, 0x6e, 0x68, 0x20, 0x47, 0x6f, 0x6a, 0x2d,
0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b,
0x6f, 0x6b, 0x6f, 0x6e, 0x68, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd,
0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x6f, 0x72,
0xe3, 0x6d, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x4b, 0x6f, 0x72, 0xe3, 0x6d, 0x69, 0x6a, 0x61, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x4b, 0x6f, 0x72, 0xe3, 0x6d, 0x69, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x4b, 0x75, 0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d,
0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x4b, 0x75, 0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76,
0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a,
0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x4b, 0x75, 0x6d, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x4b, 0x75, 0x6d, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20,
0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x6d, 0x61, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4e, 0x61, 0x76, 0x69, 0x6e, 0x68,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4e, 0x75, 0x6d, 0xe3, 0x2d,
0x4e, 0x75, 0x72, 0x76, 0x69, 0x72, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x54, 0x129, 0x6d, 0xe3, 0x72, 0x2d, 0x52, 0xe3, 0x6a, 0x75,
0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x6e, 0x68,
0x6b, 0x75, 0x76, 0x61, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73,
0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x50,
0x61, 0x6e, 0x68, 0x6b, 0x75, 0x76, 0x61, 0x20, 0x47, 0x6f, 0x6a, 0x2d,
0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x6e, 0x68, 0x6b, 0x75, 0x76,
0x61, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x45, 0x6b, 0x75, 0x76, 0x61, 0x6e, 0x6f,
0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4f, 0x72, 0xf3, 0x70,
0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0xe3, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x4b, 0x75,
0x6a, 0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x4b, 0x75, 0x6a,
0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4f, 0x72, 0xf3, 0x70,
0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x52,
0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x52,
0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0xe3, 0x6a, 0x75, 0x72,
0x20, 0x74, 0x1ef9, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x6a, 0xe3,
0x20, 0x68, 0xe1, 0x20, 0x74, 0xe1, 0x20, 0xf3, 0x72, 0x61, 0xd3, 0x72,
0x61, 0x20, 0x4f, 0x72, 0xf3, 0x70, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75,
0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4f,
0x72, 0xf3, 0x70, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4f,
0x72, 0xf3, 0x70, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x72, 0x76, 0x129, 0x6e, 0x1ef9,
0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x20, 0x50, 0xe3, 0x20, 0x4d, 0x1ef9, 0x72, 0x76, 0x129, 0x6e,
0x1ef9, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74,
0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4d,
0x1ef9, 0x72, 0x76, 0x129, 0x6e, 0x1ef9, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76,
0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x46, 0x69,
0x6a, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20,
0x46, 0x69, 0x6a, 0x69, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x46, 0x72, 0x1ef9, 0x73, 0x61, 0x20, 0x47, 0x69,
0x6a, 0x61, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x46,
0x72, 0x1ef9, 0x73, 0x61, 0x20, 0x47, 0x61, 0x20, 0x53, 0x75, 0x72, 0x20,
0x6b, 0x61, 0x72, 0x20, 0x1ef8, 0x74, 0x61, 0x72, 0x74, 0x69, 0x6e, 0x61,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47, 0x61, 0x72, 0x61, 0x20,
0x50, 0x61, 0x67, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47,
0x1ef9, 0x6d, 0x6d, 0x69, 0x6a, 0x65, 0x72, 0xd3, 0x72, 0x61, 0x20, 0x4a,
0x65, 0xf3, 0x72, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x4a, 0x65, 0xf3, 0x72, 0x6a, 0x61, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4a, 0x65,
0xf3, 0x72, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4a,
0x69, 0x72, 0x6d, 0xe9, 0x72, 0x74, 0x75, 0x20, 0x47, 0x6f, 0x6a, 0x2d,
0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d,
0x69, 0x72, 0x69, 0x6e, 0x6a, 0x1ef9, 0x6e, 0x169, 0x20, 0x47, 0x72, 0x69,
0x6e, 0x169, 0x76, 0x69, 0x73, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x47, 0x72, 0x6f, 0x1ebd, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20,
0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x47, 0x72, 0x6f, 0x1ebd, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a,
0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x1ebd,
0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x1ebd, 0x72,
0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x47, 0x72, 0x6f,
0x1ebd, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75,
0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x1ebd, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a, 0x61,
0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0xe3, 0x20, 0x47, 0x75, 0x76, 0x1ef9, 0x6d, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x47, 0x6f, 0x72, 0x66, 0x75, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x47, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x48, 0x61, 0x76, 0x61, 0x2019, 0x69, 0x20,
0x6b, 0x61, 0x72, 0x20, 0x41, 0x72, 0x65, 0x76, 0x74, 0x61, 0x20, 0x47,
0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x48, 0x61, 0x76, 0x61, 0x2019, 0x69, 0x20,
0x6b, 0x61, 0x72, 0x20, 0x41, 0x72, 0x65, 0x76, 0x74, 0x61, 0x20, 0x47,
0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x6b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x48, 0x61, 0x76, 0x61,
0x2019, 0x69, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x41, 0x72, 0x65, 0x76, 0x74,
0x61, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x48, 0xe3, 0x67, 0x20, 0x4b, 0xe3, 0x67,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x48, 0xe3,
0x67, 0x20, 0x4b, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x48, 0xe3, 0x67, 0x20, 0x4b, 0xe3,
0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x48, 0xf3, 0x76, 0x69,
0x6e, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x48,
0xf3, 0x76, 0x69, 0x6e, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x6b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x48, 0xf3, 0x76, 0x69, 0x6e, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x128, 0x6e, 0x69, 0x6a,
0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4f, 0x73, 0x69, 0x1ef9,
0x6e, 0x6f, 0x20, 0x128, 0x6e, 0x69, 0x6b, 0x6f, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x128, 0x6e, 0x6f, 0x73, 0x129, 0x6e, 0x1ef9, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x128, 0x6e, 0x6f, 0x6e, 0x1ebd, 0x6a, 0x61,
0x20, 0x4b, 0x75, 0x6a, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x128, 0x6e, 0x6f, 0x6e, 0x1ebd, 0x6a, 0x61, 0x20, 0x52, 0xe3, 0x6a, 0x75,
0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x128, 0x6e, 0x6f, 0x6e,
0x1ebd, 0x6a, 0x61, 0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x49, 0x72, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x49, 0x72, 0x1ef9, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x49, 0x72, 0x1ef9,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x69, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b,
0x69, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x49, 0x73, 0x69, 0x68, 0x61,
0xe9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x49,
0x73, 0x69, 0x68, 0x61, 0xe9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x49, 0x73, 0x69, 0x68, 0x61, 0xe9,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4a, 0x61, 0x70, 0xe3, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4a, 0x61, 0x70,
0xe3, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x4a, 0x61, 0x70, 0xe3, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0x65, 0x74, 0x72, 0x75, 0x70, 0x61, 0x76, 0x69, 0x72, 0xf3,
0x76, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x2d, 0x4b, 0x1ef9, 0x6d, 0x73, 0x61,
0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x50, 0x65, 0x74, 0x72, 0x75, 0x70, 0x61, 0x76, 0x69, 0x72, 0xf3,
0x76, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x2d, 0x4b, 0x1ef9, 0x6d, 0x73, 0x61,
0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x50, 0x65, 0x74, 0x72, 0x75, 0x70, 0x61, 0x76,
0x69, 0x72, 0xf3, 0x76, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x2d, 0x4b, 0x1ef9,
0x6d, 0x73, 0x61, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0xd3, 0x72, 0x61,
0x20, 0x4b, 0x61, 0x6a, 0x61, 0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76,
0x20, 0x52, 0xe3, 0x6a, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x4b, 0x61, 0x6a, 0x61, 0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76,
0x20, 0x52, 0xe3, 0x70, 0x75, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x4b, 0x6f, 0x72, 0xe9, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x6f, 0x72, 0xe9, 0x6a, 0x61, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x4b, 0x6f, 0x72, 0xe9, 0x6a, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x6f, 0x73, 0x69, 0x72, 0x61, 0x6a, 0x65,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x6e, 0x68,
0x6e, 0xe3, 0x6a, 0x61, 0x72, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x72, 0x61, 0x6e, 0x68, 0x6e, 0xe3,
0x6a, 0x61, 0x72, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4b, 0x72, 0x61, 0x6e, 0x68, 0x6e,
0xe3, 0x6a, 0x61, 0x72, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x52, 0x1ef9, 0x6e, 0x6b, 0x61, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x56, 0xe3, 0x66, 0x65, 0x20, 0x47,
0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0x1ef9, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x52, 0xf3, 0x72, 0x6e, 0x69, 0x20, 0x48,
0x6f, 0x76, 0x65, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x52, 0xf3, 0x72, 0x6e, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x65, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x52, 0xf3, 0x72, 0x6e, 0x69, 0x20, 0x48, 0x6f, 0x76, 0x65, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x6b, 0x61, 0x76, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d, 0x1ef9, 0x6b, 0x61,
0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x6b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x4d, 0x1ef9, 0x6b, 0x61, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x4d, 0x1ef9, 0x67, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d, 0x1ef9, 0x67, 0x61, 0x6e,
0x61, 0x6e, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x67, 0x61, 0x6e, 0x61, 0x6e, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x72, 0x61, 0x6a, 0x61, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x47, 0x6f, 0x6a, 0x20, 0x56, 0x1ebd,
0x73, 0x6f, 0x20, 0x4d, 0x1ef9, 0x72, 0x6e, 0x69, 0x76, 0x61, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x72, 0x6b, 0x65, 0x6a, 0x61,
0x20, 0x46, 0x61, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d,
0x1ef9, 0x72, 0x53, 0x61, 0x72, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd,
0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0xe3, 0x72,
0x69, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3,
0x20, 0x4d, 0xe3, 0x72, 0x69, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4d, 0xe3, 0x72, 0x69,
0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x1ef9, 0x75,
0x73, 0xe3, 0x6e, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4d, 0x1ebd,
0x73, 0x69, 0x6b, 0x75, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d,
0x1ebd, 0x73, 0x69, 0x6b, 0x75, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69,
0x6b, 0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x4d, 0x1ebd, 0x73, 0x69, 0x6b, 0x75, 0x20, 0x50, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x55, 0x72, 0x61, 0x6e, 0x20, 0x4d, 0x61, 0x74, 0x6f, 0x72, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x55, 0x72, 0x61,
0x6e, 0x20, 0x4d, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x55, 0x72, 0x61, 0x6e,
0x20, 0x4d, 0x61, 0x74, 0x6f, 0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x4d, 0xe3, 0x6e, 0x68, 0x6b, 0x6f, 0x76, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4d, 0xe3, 0x6e, 0x68, 0x6b, 0x6f,
0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x4d, 0xe3, 0x6e, 0x68, 0x6b, 0x6f, 0x76, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x4d, 0x129, 0x6a, 0x1ef9, 0x6d, 0x1ef9, 0x72, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4e, 0x1ef9, 0x76, 0x75, 0x72, 0x75,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4e, 0x1ebd, 0x70, 0x61, 0x72,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x61, 0x72, 0x65, 0x6e,
0x6f, 0x6e, 0x129, 0x6a, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x61, 0x72, 0x65, 0x6e,
0x6f, 0x6e, 0x129, 0x6a, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4b, 0x61,
0x72, 0x65, 0x6e, 0x6f, 0x6e, 0x129, 0x6a, 0x61, 0x20, 0x54, 0xe3, 0x67,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4a, 0x65, 0x72, 0x1ef9, 0x6e,
0x69, 0x6a, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x4a, 0x65, 0x72, 0x1ef9, 0x6e, 0x69, 0x6a,
0x61, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4a, 0x65, 0x72, 0x1ef9, 0x6e, 0x69,
0x6a, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x47, 0x61, 0x20, 0x74, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x47, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x47, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x4e, 0x129, 0x76, 0x65, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x4e, 0xe3, 0x72, 0x66, 0x6f, 0x72, 0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a,
0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x4e, 0xe3, 0x72, 0x66, 0x6f, 0x72, 0x6b, 0x69, 0x20,
0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0x52,
0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x4e, 0xe3, 0x72,
0x66, 0x6f, 0x72, 0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd,
0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x46, 0x65, 0x72,
0x6e, 0x1ef9, 0x6e, 0x75, 0x20, 0x4e, 0xe3, 0x72, 0xe3, 0x6a, 0x61, 0x2d,
0x74, 0xe1, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x1ef9, 0x6e, 0x75, 0x20, 0x4e, 0xe3, 0x72, 0xe3,
0x6a, 0x61, 0x2d, 0x74, 0xe1, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x1ef9, 0x6e,
0x75, 0x20, 0x4e, 0xe3, 0x72, 0xe3, 0x6a, 0x61, 0x2d, 0x74, 0xe1, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4e, 0xe3, 0x72, 0x74, 0x69, 0x2d,
0x4d, 0x1ef9, 0x72, 0x69, 0x6a, 0x1ef9, 0x6e, 0x1ef9, 0x20, 0x47, 0x6f, 0x6a,
0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x53, 0x69, 0x6d, 0x69, 0x72, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x54, 0xe3,
0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53,
0x69, 0x6d, 0x69, 0x72, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x54, 0xe3, 0x67,
0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61,
0x20, 0x50, 0xe3, 0x20, 0x53, 0x69, 0x6d, 0x69, 0x72, 0x73, 0x69, 0x6b,
0x69, 0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x4f, 0x6d, 0x129, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x50, 0xe3, 0x20, 0x4f, 0x6d, 0x129, 0x6e, 0x68, 0x6b, 0x69,
0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61,
0x20, 0x4f, 0x6d, 0x129, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x6b, 0x69,
0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0x61, 0x72, 0x61, 0x76, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x50, 0x61, 0x70, 0x75, 0x76, 0x61, 0x2d, 0x47, 0x69, 0x6e, 0x1ebd,
0x20, 0x54, 0xe3, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x76, 0x61, 0x6a, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75,
0x76, 0x61, 0x6a, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x76, 0x61,
0x6a, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x69, 0x72, 0x75,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x50, 0x69,
0x72, 0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x50, 0x69, 0x72, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x46, 0x69, 0x72, 0x69, 0x70, 0x69, 0x6e, 0x1ef9, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x46, 0x69, 0x72, 0x69,
0x70, 0x69, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0x46, 0x69, 0x72, 0x69, 0x70, 0x69, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x46, 0x1ebd, 0x6e, 0x129, 0x67, 0x20, 0x47, 0x6f,
0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x53, 0x1ef9, 0x20, 0x50, 0x65, 0x6e, 0x72, 0x75, 0x20, 0x6b, 0x61,
0x72, 0x20, 0x4d, 0x129, 0x6b, 0x65, 0x72, 0x1ef9, 0x76, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x1ef9, 0x20, 0x50, 0x65,
0x64, 0x72, 0x6f, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x4d, 0x129, 0x6b, 0x65,
0x72, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x53, 0x1ef9, 0x20, 0x50, 0x65, 0x64, 0x72, 0x6f,
0x20, 0x6b, 0x61, 0x72, 0x20, 0x4d, 0x129, 0x6b, 0x65, 0x72, 0x1ef9, 0x76,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x69, 0x6e, 0x6b, 0x61,
0x6a, 0x69, 0x72, 0x20, 0x47, 0x6f, 0x6a, 0x2d, 0x76, 0x1ebd, 0x73, 0x6f,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x6f, 0x6e, 0x1ef9, 0x70,
0x65, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x69, 0xe3, 0x67,
0x69, 0x1ef9, 0x6e, 0x67, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x4b,
0x79, 0x73, 0x79, 0x72, 0x6f, 0x72, 0x6e, 0x61, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x79, 0x73, 0x79, 0x72, 0x6f,
0x72, 0x6e, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x4b, 0x79, 0x73, 0x79, 0x72, 0x6f, 0x72, 0x6e,
0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x48, 0x75, 0x6a, 0xe1,
0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x52, 0x6f, 0x74, 0x65,
0x72, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x61, 0x6b,
0x61, 0x72, 0x69, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x53, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x6e, 0x1ef9, 0x20,
0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20,
0x53, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x53, 0x61, 0x6d, 0x1ef9, 0x72, 0x61, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x61, 0x6d, 0x1ef9, 0x72,
0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x53, 0x61, 0x6d, 0x1ef9, 0x72, 0x61, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x53, 0x61, 0x6d, 0xe3, 0x76, 0x61, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x61, 0x6d, 0xe3, 0x76,
0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72,
0x61, 0x20, 0x53, 0x61, 0x6d, 0xe3, 0x76, 0x61, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x53, 0x65, 0x6a, 0x73, 0x65, 0x72, 0x65, 0x6e, 0x68,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x53, 0x129,
0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61,
0x20, 0x53, 0x61, 0x72, 0x6f, 0x6d, 0x1ef9, 0x76, 0x20, 0x47, 0x6f, 0x6a,
0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x4a, 0x69, 0xf3, 0x72, 0x6a, 0x61, 0x20, 0x74, 0x1ef9, 0x20, 0x53, 0x75,
0x72, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53, 0x75, 0x72, 0x69,
0x6e, 0x1ef9, 0x6d, 0x129, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x53,
0x69, 0x6f, 0x76, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54,
0x61, 0x6a, 0x74, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54,
0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x54, 0x61, 0x6a, 0x70, 0x65, 0x6a, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x54, 0x61,
0x6a, 0x70, 0x65, 0x6a, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54,
0x61, 0x6a, 0x69, 0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54, 0x6f, 0x6b, 0x65, 0x72, 0x61, 0x76,
0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x61,
0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x54, 0xe3,
0x67, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3,
0x72, 0x61, 0x20, 0x54, 0xe3, 0x67, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x53, 0x75, 0x75, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6f, 0x6d, 0x1ebd, 0x6e, 0x129, 0x6e,
0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6f, 0x6d, 0x1ebd, 0x6e, 0x129, 0x6e,
0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6f, 0x6d, 0x1ebd,
0x6e, 0x129, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x54, 0x75, 0x76, 0x61, 0x72, 0x75, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x76, 0x61, 0x6a, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x55, 0x72, 0x75,
0x67, 0x75, 0x76, 0x61, 0x6a, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x55, 0x72, 0x75, 0x67, 0x75, 0x76,
0x61, 0x6a, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x55, 0x6e, 0x68,
0x6d, 0x65, 0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x55, 0x6e, 0x6e, 0x6d, 0x65,
0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x55, 0x6e, 0x68, 0x6d,
0x65, 0x6b, 0x69, 0x6e, 0x68, 0x74, 0x1ef9, 0x76, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x169, 0x76, 0x61, 0x74, 0x75, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x56, 0x61, 0x6e,
0x169, 0x76, 0x61, 0x74, 0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b,
0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x56, 0x61, 0x6e, 0x169, 0x76, 0x61,
0x74, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x56, 0x65, 0x6e,
0x1ebd, 0x6a, 0x75, 0x76, 0xe9, 0x72, 0x61, 0x20, 0x74, 0xe1, 0xd3, 0x72,
0x61, 0x20, 0x55, 0x72, 0x61, 0x6e, 0x69, 0x76, 0xf3, 0x6e, 0x68, 0x74,
0xf3, 0x6b, 0x69, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50,
0xe3, 0x20, 0x55, 0x72, 0x61, 0x6e, 0x69, 0x76, 0xf3, 0x6e, 0x68, 0x74,
0xf3, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xf3, 0x72, 0x61, 0x20, 0x55, 0x72, 0x61, 0x6e, 0x69, 0x76, 0xf3, 0x6e,
0x68, 0x74, 0xf3, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x56, 0x6f, 0x72, 0x67, 0x75, 0x67, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x56, 0x6f, 0x72, 0x67,
0x75, 0x67, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20,
0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x56, 0x6f, 0x72, 0x67, 0x75,
0x67, 0x72, 0x61, 0x6e, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x56, 0x6f, 0x6e, 0x68, 0x74, 0xf3, 0x6b, 0x69, 0x20, 0x74, 0xe1, 0xd3,
0x72, 0x61, 0x20, 0x56, 0x65, 0x6a, 0x6b, 0x69, 0x20, 0x47, 0x6f, 0x6a,
0x2d, 0x76, 0x1ebd, 0x73, 0x6f, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x56, 0x61, 0x72, 0x69, 0x6e, 0x68, 0x20, 0x6b, 0x61, 0x72, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x1ef9, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x59, 0x6a, 0x61, 0x6b, 0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x59, 0x6a, 0x61,
0x6b, 0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x69, 0x20, 0x74, 0xe1,
0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20, 0xf3, 0x72, 0x61, 0x20, 0x59, 0x6a,
0x61, 0x6b, 0x75, 0x74, 0x69, 0x6e, 0x68, 0x6b, 0x69, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129, 0x6e,
0x6d, 0x75, 0x72, 0x67, 0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20,
0x50, 0xe3, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129, 0x6e, 0x6d,
0x75, 0x72, 0x67, 0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3,
0x20, 0xf3, 0x72, 0x61, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129,
0x6e, 0x6d, 0x75, 0x72, 0x67, 0x75, 0x20, 0x74, 0xe1, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x41, 0x63, 0x72, 0x65, 0x20, 0x79,
0x61, 0x72, 0x61, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x20, 0x41, 0x63, 0x72, 0x65, 0x20,
0x79, 0x61, 0x72, 0x61, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x41, 0x63, 0x72, 0x65, 0x20, 0x79, 0x61, 0x72, 0x61, 0x41, 0x66, 0x65,
0x67, 0xe3, 0x6e, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x66, 0x69, 0x72, 0x69,
0x6b, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x66,
0x69, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61,
0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x41, 0x66, 0x69, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x53, 0x75, 0x20,
0x73, 0x75, 0xed, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x41, 0x66, 0x69, 0x72, 0x69, 0x6b, 0x69, 0x61, 0x20, 0x55, 0x73, 0x69,
0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x66, 0x69, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x41, 0x66, 0x69, 0x72, 0x69, 0x6b, 0x61, 0x20,
0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6c, 0x61, 0x73, 0x69,
0x6b, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41,
0x6c, 0x61, 0x73, 0x69, 0x6b, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x45, 0x74, 0xe9, 0x41, 0x6c, 0x61, 0x73, 0x69,
0x6b, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75,
0x6d, 0x61, 0x74, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x41, 0x75, 0x6d, 0x61, 0x74, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x45, 0x74, 0xe9, 0x41, 0x75, 0x6d, 0x61,
0x74, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6d,
0x61, 0x73, 0x75, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x41, 0x6d, 0x61, 0x73, 0x75, 0x6e, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x45, 0x74, 0xe9, 0x41, 0x6d,
0x61, 0x73, 0x75, 0x6e, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x6a, 0x75, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72,
0x61, 0x77, 0x61, 0x72, 0x61, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x4c, 0x65, 0x73, 0x69, 0x74, 0x69, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x20,
0x4c, 0x65, 0x73, 0x69, 0x74, 0x69, 0x20, 0x79, 0x61, 0x72, 0x61, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x4c, 0x65, 0x73, 0x69, 0x74,
0x69, 0xd3, 0x72, 0x61, 0x20, 0x4b, 0x72, 0x129, 0x2d, 0x61, 0x67, 0x20,
0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x4b, 0x72, 0x129,
0x2d, 0x61, 0x67, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x6b, 0xe3, 0x20,
0xd3, 0x72, 0x61, 0x20, 0x4b, 0x72, 0x129, 0x2d, 0x61, 0x67, 0x20, 0x74,
0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x75, 0x20, 0x74, 0xe1, 0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x50,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x74, 0xe1, 0x52, 0x1ef9,
0x20, 0x4b, 0xe3, 0x20, 0xd3, 0x72, 0x61, 0x20, 0x50, 0x61, 0x73, 0x69,
0x66, 0x69, 0x6b, 0x75, 0x20, 0x74, 0xe1, 0x41, 0x6e, 0x61, 0x64, 0xed,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6e, 0x61,
0x64, 0xed, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x6e, 0x61, 0x64, 0xed, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x70, 0x69, 0x79, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x70, 0x69,
0x79, 0x61, 0x20, 0x55, 0x72, 0x61, 0x72, 0x75, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x70, 0x69, 0x79, 0x61, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6b, 0x69, 0x74, 0x61,
0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6b,
0x69, 0x74, 0x61, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x6b, 0x69, 0x74,
0x61, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x6b,
0x69, 0x74, 0x75, 0x62, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x41, 0x6b, 0x69, 0x74, 0x75, 0x62, 0x69, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77,
0x61, 0x41, 0x6b, 0x69, 0x74, 0x75, 0x62, 0x69, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x72, 0xe1, 0x62, 0x69, 0x79, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x72, 0xe1, 0x62,
0x69, 0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x72, 0x61, 0x62, 0x69,
0x79, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x72,
0x67, 0x1ebd, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x41, 0x72, 0x67, 0x1ebd, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x41, 0x72, 0x67, 0x1ebd, 0x74, 0x69, 0x6e, 0x61, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x72, 0x67, 0x1ebd, 0x74,
0x69, 0x6e, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77,
0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x41, 0x72, 0x67, 0x1ebd, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x55, 0x73, 0x69,
0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x41, 0x72, 0x67, 0x1ebd, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x55, 0x73, 0x69,
0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x72, 0x69, 0x6d, 0x1ebd, 0x6e, 0x69, 0x79,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x72,
0x69, 0x6d, 0x1ebd, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41,
0x72, 0x69, 0x6d, 0x1ebd, 0x6e, 0x69, 0x79, 0x61, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x74, 0x61, 0x6c, 0xe3, 0x74, 0x69, 0x6b,
0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x74,
0x61, 0x6c, 0xe3, 0x74, 0x69, 0x6b, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41,
0x74, 0x61, 0x6c, 0xe3, 0x74, 0x6b, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41,
0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72,
0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x73,
0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69,
0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x2d, 0x55, 0x73, 0x69,
0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61,
0x77, 0x61, 0x72, 0x61, 0x2d, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61,
0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x75, 0x73, 0x69,
0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x74,
0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x2d, 0x55, 0x73, 0x69, 0x64,
0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41,
0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72,
0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x73,
0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x55, 0x73,
0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x75, 0x73, 0x69, 0x74, 0x61,
0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd,
0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x75,
0x73, 0x69, 0x74, 0x61, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x55,
0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x73, 0x65, 0x72, 0x69, 0x72,
0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x41, 0x73, 0x65, 0x72, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x73, 0x65, 0x72, 0x69, 0x72, 0x65,
0x74, 0xe3, 0x6d, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x41, 0x73, 0x6f, 0x72, 0x69, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x41, 0x73, 0x6f, 0x72, 0x69, 0x2d,
0x69, 0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x41, 0x73, 0x6f, 0x72, 0x69,
0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x42, 0xe3, 0x6b, 0x61, 0x72, 0x61, 0x64, 0x65, 0x78, 0x69, 0x20, 0x48,
0x6f, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x42, 0xe3, 0x6b, 0x61, 0x72,
0x61, 0x64, 0x65, 0x78, 0x69, 0x20, 0x48, 0x6f, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x42, 0xe3, 0x6b,
0x61, 0x72, 0x61, 0x64, 0x65, 0x78, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x6f, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x42, 0x75, 0x74, 0xe3, 0x75, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x42, 0x75, 0x72, 0xed, 0x77, 0x69, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0xd3, 0x72, 0x61,
0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61, 0x20, 0x74, 0xe1,
0xd3, 0x72, 0x61, 0x20, 0x50, 0xe3, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed,
0x6c, 0x69, 0x61, 0x20, 0x74, 0xe1, 0x52, 0x1ef9, 0x20, 0x4b, 0xe3, 0x20,
0xd3, 0x72, 0x61, 0x20, 0x42, 0x72, 0x61, 0x73, 0xed, 0x6c, 0x69, 0x61,
0x20, 0x74, 0xe1, 0x42, 0x75, 0x72, 0x75, 0x6e, 0x65, 0x69, 0x20, 0x44,
0x61, 0x72, 0x75, 0x73, 0x61, 0x72, 0x61, 0x6d, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x62, 0x75, 0x20, 0x53, 0x75,
0x69, 0x6b, 0x69, 0x72, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x4b, 0x61, 0x62, 0x75, 0x20, 0x53, 0x75, 0x69, 0x6b, 0x69,
0x72, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x61, 0x62, 0x75, 0x20, 0x53,
0x75, 0x69, 0x6b, 0x69, 0x72, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x58, 0x61, 0x6d, 0x6f, 0x72, 0x6f, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x58, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x58, 0x61, 0x74, 0x68,
0x61, 0x6d, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x58, 0x61, 0x74, 0x68, 0x61, 0x6d,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x58, 0x69, 0x72, 0x69,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x58, 0x69, 0x72,
0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x58, 0x69, 0x72, 0x69, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x58, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x58, 0x69, 0x6e, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x58, 0x69, 0x6e, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x58, 0x6f, 0x69, 0x62, 0x61, 0x73, 0xe3, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x58, 0x6f, 0x69, 0x62, 0x61, 0x73,
0xe3, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x58, 0x6f, 0x69, 0x62, 0x61, 0x73, 0xe3,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75,
0xe3, 0x6d, 0x61, 0x4b, 0x69, 0x72, 0x69, 0x74, 0x69, 0x6d, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75,
0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b, 0x75, 0x6b, 0x75, 0x2d, 0x69,
0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b,
0x75, 0x72, 0x169, 0x62, 0x69, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4b, 0x75, 0x72, 0x169, 0x62, 0x69, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x4b, 0x75, 0x72, 0x169, 0x62, 0x69, 0x61, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69,
0x74, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69,
0x74, 0x61, 0x20, 0x4b, 0x6f, 0x6f, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b, 0x6f,
0x6f, 0x6b, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41,
0x72, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x75, 0x62, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x4b, 0x75, 0x62, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x44, 0x61, 0x77, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64,
0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c, 0x65, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x69, 0x6d, 0x75, 0x2d, 0x53, 0x65,
0x6d, 0x75, 0x73, 0x61, 0x77, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x50, 0x61, 0x73, 0x69, 0x6b, 0x75, 0x77, 0x61, 0x20,
0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x73, 0x69, 0x6b, 0x75, 0x77, 0x61,
0x20, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x50, 0x61, 0x73, 0x69, 0x6b, 0x75, 0x77, 0x61, 0x20, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x6d, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x45, 0x6b, 0x75, 0x61, 0x64, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20, 0x50, 0x69,
0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20,
0x50, 0x69, 0x74, 0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20, 0x50, 0x69, 0x74,
0x65, 0x72, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20, 0x55, 0x72,
0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20,
0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20, 0x55, 0x72, 0x69,
0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20, 0x4c, 0x65,
0x73, 0x69, 0x74, 0x69, 0x2d, 0x65, 0x74, 0xe9, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61, 0x20,
0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x75, 0x72, 0x75,
0x70, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x45, 0x75, 0x72, 0x75, 0x70, 0x61,
0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75,
0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x69, 0x6e, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x69, 0x6e,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69,
0x74, 0x61, 0x20, 0x4d, 0x61, 0x77, 0x69, 0x6e, 0x61, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x69, 0x79, 0x69, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x69, 0x79, 0x69, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x46, 0x69, 0x79, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x47, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x46, 0x72, 0xe3,
0x73, 0x61, 0x20, 0x79, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x54, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x53,
0x75, 0x20, 0x73, 0x75, 0xed, 0x20, 0x46, 0x72, 0xe3, 0x73, 0x61, 0x20,
0x79, 0x61, 0x72, 0x61, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0xc3, 0x74,
0x61, 0x72, 0x74, 0x69, 0x64, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x47, 0x61, 0x72, 0x61, 0x70, 0x61, 0x67, 0x75, 0x2d,
0x69, 0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x47, 0xe3, 0x62, 0x69, 0x65, 0x72, 0x65, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x47, 0x65, 0x75, 0x67, 0x69, 0x79, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x65, 0x75, 0x67,
0x69, 0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x47, 0x65, 0x75, 0x67, 0x69,
0x79, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61,
0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x59, 0x75, 0x62, 0x65,
0x74, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x4d, 0x69, 0x72,
0x69, 0x64, 0x69, 0x79, 0x61, 0x6e, 0x75, 0x20, 0x79, 0x61, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x75, 0x72,
0x75, 0x1ebd, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20, 0x55, 0x72, 0x69,
0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x47, 0x75, 0x72, 0x75, 0x1ebd, 0x72, 0xe3, 0x64,
0x69, 0x79, 0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x47, 0x75, 0x72, 0x75, 0x1ebd, 0x72,
0xe3, 0x64, 0x69, 0x79, 0x61, 0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61,
0x77, 0x61, 0x72, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x47, 0x75, 0x72, 0x75, 0x1ebd, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20,
0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x75, 0x72, 0x75,
0x1ebd, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64,
0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x47,
0x75, 0x72, 0x75, 0x1ebd, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20, 0x55,
0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x75, 0x77, 0xe3, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x47, 0x69, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x48, 0x61, 0x77, 0x61, 0xed,
0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d,
0x69, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x75, 0x74, 0x61, 0x2d, 0x69,
0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x48,
0x61, 0x77, 0x61, 0xed, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4b, 0x61,
0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x75,
0x74, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x48, 0x61,
0x77, 0x61, 0xed, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x41, 0x72, 0x65, 0x75, 0x74,
0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x48, 0x169, 0x67, 0x20, 0x4b, 0x169, 0x67, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x48, 0x169, 0x67, 0x20, 0x4b, 0x169, 0x67,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x48, 0x169, 0x67, 0x20, 0x4b, 0x169, 0x67, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x48, 0x6f, 0x77, 0x69, 0x64,
0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x48, 0x6f,
0x77, 0x69, 0x64, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x48, 0x6f, 0x77, 0x69,
0x64, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x128, 0x64,
0x69, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x55, 0x73, 0x65, 0x79, 0x61, 0x6e,
0x75, 0x20, 0x128, 0x64, 0x69, 0x6b, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x128, 0x64, 0x6f, 0x78, 0x69, 0x6e, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x128, 0x64, 0x6f, 0x6e,
0x65, 0x73, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x74, 0x65, 0x72, 0x61,
0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x128, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69, 0x79, 0x61, 0x20, 0x55,
0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x128, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x79, 0x61, 0x20, 0x55, 0x73, 0x69, 0x64, 0x1ebd, 0x74, 0x61, 0x77,
0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x49, 0x72, 0xe3, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x49, 0x72, 0xe3, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x49, 0x72, 0xe3, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69,
0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x49, 0x73, 0x69,
0x72, 0x61, 0x79, 0x65, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x49, 0x73, 0x69, 0x72, 0x61, 0x79, 0x65, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x49, 0x73, 0x69, 0x72, 0x61, 0x79, 0x65, 0x75, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4e, 0x69, 0x70, 0xf5, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4e, 0x69, 0x70, 0xf5, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x4e, 0x69, 0x70, 0xf5, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c,
0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74,
0x73, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76, 0x6c, 0x6f, 0x76, 0x73,
0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61, 0x74, 0x73, 0x6b, 0x69,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x50, 0x65, 0x74, 0x72, 0x6f, 0x70, 0x61, 0x76,
0x6c, 0x6f, 0x76, 0x73, 0x6b, 0x2d, 0x4b, 0x61, 0x6d, 0x63, 0x68, 0x61,
0x74, 0x73, 0x6b, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4b, 0x61, 0x73, 0x61, 0x6b, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61,
0x20, 0x55, 0x72, 0x69, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x73, 0x61,
0x6b, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x55, 0x73, 0x69,
0x64, 0x1ebd, 0x74, 0x61, 0x77, 0x61, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x75, 0x72, 0x65, 0x79, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x75, 0x72, 0x65,
0x79, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x75, 0x72, 0x65, 0x79, 0x61,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x75, 0x73, 0x69,
0x72, 0x61, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4b, 0x61, 0x72, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x69,
0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4b, 0x61, 0x72, 0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x72,
0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x61, 0x72,
0x61, 0x73, 0x69, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x69, 0x73, 0x69, 0x6b,
0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x69, 0x72,
0x69, 0x67, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x52, 0xe3, 0x6b, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3,
0x2d, 0x69, 0x74, 0x61, 0x20, 0x49, 0x6e, 0x129, 0x62, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4c, 0x75, 0x72, 0x75, 0x64,
0x69, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4c, 0x75, 0x72, 0x75, 0x64, 0x69, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4c, 0x75, 0x72, 0x75, 0x64, 0x69,
0x20, 0x48, 0x6f, 0x77, 0x65, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x4d, 0x61, 0x6b, 0x61, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe3, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe3, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x4d, 0x61, 0x67, 0x61, 0x64, 0xe3, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x6c, 0x61, 0x73, 0x69, 0x79, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x75, 0x64, 0x69,
0x77, 0x61, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4d, 0x61, 0x6b, 0x65, 0x73, 0x61, 0x2d, 0x69, 0x74,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61,
0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x79, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x73, 0x69, 0x79, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x73, 0x69, 0x79, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4d, 0x61, 0x75, 0x73, 0x169, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x75, 0x20,
0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x75, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b,
0x75, 0x20, 0x4d, 0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x50, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x75, 0x20, 0x4d,
0x65, 0x78, 0x69, 0x6b, 0x61, 0x6e, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x55, 0x72, 0xe3, 0x20, 0x42, 0x61, 0x74, 0x75, 0x72,
0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x55, 0x72,
0xe3, 0x20, 0x42, 0x61, 0x74, 0x75, 0x72, 0x75, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x55, 0x72, 0xe3, 0x20, 0x42, 0x61, 0x74, 0x75, 0x72, 0x75, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x6f, 0x73, 0x6b, 0x6f, 0x75,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x6f, 0x73,
0x6b, 0x6f, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x6f,
0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4d, 0x69, 0x79,
0x61, 0x6d, 0xe1, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4e, 0x61, 0x75, 0x72, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x4e, 0x65, 0x70, 0x61, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x72, 0x65, 0x64, 0x169, 0x6e, 0x69,
0x79, 0x61, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x72, 0x65, 0x64, 0x169,
0x6e, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x4b, 0x61, 0x72, 0x65, 0x64, 0x169, 0x6e, 0x69, 0x79,
0x61, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x53, 0x65, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61,
0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x53, 0x65, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61,
0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x53,
0x65, 0x72, 0xe3, 0x64, 0x69, 0x79, 0x61, 0x20, 0x50, 0x69, 0x73, 0x61,
0x73, 0xfa, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72,
0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x49, 0x77,
0xed, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x49, 0x77, 0xed, 0x20, 0x50, 0x69, 0x73,
0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x49, 0x77, 0xed, 0x20, 0x50,
0x69, 0x73, 0x61, 0x73, 0xfa, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed,
0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x4e, 0x69, 0x77, 0x65, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4b, 0x61, 0x70,
0x75, 0xe3, 0x6d, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x65, 0x6e, 0xe3,
0x64, 0x75, 0x20, 0x4e, 0x75, 0x72, 0x75, 0x79, 0xe3, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x65, 0x6e, 0xe3, 0x64, 0x75,
0x20, 0x4e, 0x75, 0x72, 0x75, 0x79, 0xe3, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x46,
0x65, 0x6e, 0xe3, 0x64, 0x75, 0x20, 0x4e, 0x75, 0x72, 0x75, 0x79, 0xe3,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75,
0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x4d, 0x61, 0x72, 0x69, 0x79, 0xe3,
0x6e, 0x61, 0x20, 0x4e, 0x75, 0x74, 0x69, 0x20, 0x73, 0x75, 0xed, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x69, 0x62, 0x69,
0x72, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73,
0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x69,
0x62, 0x69, 0x72, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x50, 0x69, 0x73,
0x61, 0x73, 0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x53, 0x69, 0x62, 0x69, 0x72,
0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73, 0xfa,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4f, 0x6d, 0x69, 0x73,
0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x4f, 0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x4f,
0x6d, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73,
0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x50, 0x61, 0x6b, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x6b,
0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x50,
0x61, 0x6b, 0x69, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x72, 0x61, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x70, 0x75, 0x77,
0x61, 0x2d, 0x47, 0x69, 0x6e, 0xe9, 0x20, 0x50, 0x69, 0x73, 0x61, 0x73,
0xfa, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x77, 0x61, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x77, 0x61,
0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52,
0x65, 0x74, 0x65, 0x77, 0x61, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x77,
0x61, 0x69, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x65, 0x72,
0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x65,
0x72, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x50, 0x65, 0x72, 0x75, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46, 0x69, 0x72, 0x69, 0x70, 0x69,
0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x46,
0x69, 0x72, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x46,
0x69, 0x72, 0x69, 0x70, 0x69, 0x6e, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61,
0x20, 0x46, 0x1ebd, 0x6e, 0x69, 0x78, 0x53, 0xe3, 0x20, 0x50, 0x65, 0x64,
0x75, 0x72, 0x75, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4d, 0x69, 0x6b,
0x69, 0x72, 0xe3, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x53, 0xe3, 0x20, 0x50, 0x65, 0x64, 0x75, 0x72, 0x75, 0x20, 0x61,
0x73, 0x75, 0xed, 0x20, 0x4d, 0x69, 0x6b, 0x69, 0x72, 0xe3, 0x75, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x53, 0xe3, 0x20, 0x50, 0x65, 0x64, 0x75, 0x72, 0x75,
0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4d, 0x69, 0x6b, 0x69, 0x72, 0xe3,
0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x69, 0x74,
0x63, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x50, 0x6f, 0x6e, 0x61, 0x70, 0x65, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x50, 0x69, 0xfa, 0x67, 0x69, 0xe3, 0x67,
0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b, 0x69,
0x73, 0x69, 0x72, 0x6f, 0x64, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x4b, 0x69, 0x73, 0x69, 0x72, 0x6f, 0x64, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x4b, 0x69, 0x73, 0x69, 0x72, 0x6f, 0x64, 0x61, 0x20,
0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x59, 0x75, 0x6d, 0x75, 0x61,
0x74, 0x69, 0x72, 0x69, 0x73, 0x61, 0x77, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x52, 0x6f, 0x74, 0x65, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x61, 0x6b, 0x61,
0x72, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x53, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x6e, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77,
0x61, 0x53, 0x61, 0x6b, 0x61, 0x72, 0x69, 0x6e, 0x61, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x61, 0x6d, 0x61,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20,
0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x53, 0x61, 0x6d, 0x61, 0x72, 0x61,
0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x61, 0x6d, 0x6f,
0x77, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53,
0x61, 0x6d, 0x6f, 0x77, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x53, 0x61, 0x6d,
0x6f, 0x77, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53,
0x65, 0x69, 0x78, 0x65, 0x72, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x53, 0x129, 0x67, 0x61, 0x70, 0x75, 0x72, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74,
0x65, 0x77, 0x61, 0x4b, 0x61, 0x70, 0x75, 0xe3, 0x6d, 0x61, 0x2d, 0x69,
0x74, 0x61, 0x20, 0x53, 0x61, 0x72, 0x75, 0x6d, 0x169, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x47, 0x65, 0x75, 0x67, 0x69, 0x79,
0x61, 0x20, 0x53, 0x75, 0x20, 0x73, 0x75, 0xed, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d,
0x65, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0x69,
0x79, 0x6f, 0x77, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x54, 0x61, 0x69, 0x74, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x61, 0x79, 0x69, 0x6b, 0x69,
0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x54, 0x6f, 0x6b, 0x65, 0x72, 0x61, 0x75, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0xf5, 0x67, 0x61, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0xf5, 0x67, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x54, 0xf5, 0x67, 0x61, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x75, 0x72, 0x6b, 0x75, 0x72,
0x61, 0x6e, 0x61, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x54, 0x75, 0x72, 0x6b, 0x75, 0x72,
0x61, 0x6e, 0x61, 0x72, 0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77,
0x61, 0x54, 0x75, 0x72, 0x6b, 0x75, 0x72, 0x61, 0x6e, 0x61, 0x72, 0x65,
0x74, 0xe3, 0x6d, 0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x54, 0x75, 0x76, 0x61, 0x72, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x55, 0x72, 0x75, 0x67, 0x75, 0x77, 0x61, 0x69, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x55, 0x72, 0x75, 0x67,
0x75, 0x77, 0x61, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x55, 0x72, 0x75, 0x67,
0x75, 0x77, 0x61, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x59, 0x169, 0x62, 0x75, 0x65, 0x73, 0x61, 0x72, 0x61, 0x2d, 0x72, 0x65,
0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x59, 0x169, 0x62, 0x75, 0x65, 0x73, 0x61, 0x72, 0x61, 0x2d, 0x72,
0x65, 0x74, 0xe3, 0x6d, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x59, 0x169, 0x62,
0x75, 0x65, 0x73, 0x61, 0x72, 0x61, 0x2d, 0x72, 0x65, 0x74, 0xe3, 0x6d,
0x61, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x57, 0x61, 0x6e,
0x75, 0x61, 0x74, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x57, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61,
0x57, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61,
0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x57, 0x65, 0x6e, 0x1ebd, 0x73, 0x75, 0x65, 0x72, 0x61,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x57, 0x61, 0x72,
0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x69, 0x74, 0x75, 0x6b, 0x69, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x57, 0x61, 0x72, 0x61,
0x64, 0x69, 0x77, 0x6f, 0x73, 0x69, 0x74, 0x75, 0x6b, 0x69, 0x20, 0x48,
0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65,
0x77, 0x61, 0x57, 0x61, 0x72, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x69,
0x74, 0x75, 0x6b, 0x69, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20,
0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75,
0x57, 0x6f, 0x72, 0x77, 0x6f, 0x67, 0x61, 0x72, 0x61, 0x64, 0x75, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x57, 0x6f, 0x72, 0x77,
0x6f, 0x67, 0x61, 0x72, 0x61, 0x64, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x57,
0x6f, 0x72, 0x77, 0x6f, 0x67, 0x61, 0x72, 0x61, 0x64, 0x75, 0x20, 0x4b,
0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75,
0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x57, 0x6f, 0x73, 0x69, 0x74, 0x75,
0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x4b,
0x61, 0x70, 0x75, 0xe3, 0x2d, 0x69, 0x74, 0x61, 0x20, 0x57, 0x61, 0x6b,
0x65, 0x57, 0x61, 0x72, 0x69, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x46,
0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69,
0x79, 0x75, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x59, 0x61, 0x6b,
0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x59,
0x61, 0x6b, 0x75, 0x74, 0x69, 0x73, 0x69, 0x6b, 0x69, 0x20, 0x4b, 0x75,
0x72, 0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72,
0x61, 0x72, 0x69, 0x79, 0x75, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129,
0x62, 0x75, 0x67, 0x75, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129, 0x62, 0x75, 0x67, 0x75,
0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x20, 0x52, 0x65,
0x74, 0x65, 0x77, 0x61, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x129, 0x62,
0x75, 0x67, 0x75, 0x20, 0x4b, 0x75, 0x72, 0x61, 0x73, 0xed, 0x20, 0x41,
0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x42,
0x75, 0x74, 0x61, 0x6e, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79,
0x75, 0x53, 0xe3, 0x20, 0x50, 0x65, 0x64, 0x75, 0x72, 0x75, 0x20, 0x61,
0x73, 0x75, 0xed, 0x20, 0x4d, 0x69, 0x6b, 0x69, 0x72, 0x61, 0x6e, 0x20,
0x48, 0x75, 0x72, 0x61, 0x72, 0x69, 0x79, 0x75, 0x53, 0xe3, 0x20, 0x50,
0x65, 0x64, 0x75, 0x72, 0x75, 0x20, 0x61, 0x73, 0x75, 0xed, 0x20, 0x4d,
0x69, 0x6b, 0x69, 0x72, 0x61, 0x6e, 0x20, 0x48, 0x75, 0x72, 0x61, 0x72,
0x69, 0x79, 0x75, 0x20, 0x52, 0x65, 0x74, 0x65, 0x77, 0x61, 0x53, 0xe3,
0x20, 0x50, 0x65, 0x64, 0x75, 0x72, 0x75, 0x20, 0x61, 0x73, 0x75, 0xed,
0x20, 0x4d, 0x69, 0x6b, 0x69, 0x72, 0x61, 0x6e, 0x20, 0x4b, 0x75, 0x72,
0x61, 0x73, 0xed, 0x20, 0x41, 0x72, 0x61, 0x20, 0x48, 0x75, 0x72, 0x61,
0x72, 0x69, 0x79, 0x75, 0x41, 0x66, 0x67, 0x68, 0x61, 0x61, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x65,
0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b,
0x6f, 0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20,
0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x53, 0xfc, 0xfc, 0x64, 0x65, 0x6c, 0x6b, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41,
0x66, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57,
0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69, 0x6b, 0x6f, 0x6f,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x66, 0x72, 0x69,
0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x41, 0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41,
0x6d, 0x61, 0x7a, 0x6f, 0x6e, 0x61, 0x73, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x41, 0x6d, 0x65,
0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x41, 0x6d, 0x65,
0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55,
0x61, 0x73, 0x74, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f,
0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x41, 0x6d, 0x65, 0x65,
0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x6d, 0x65,
0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57,
0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b,
0x6f, 0x6f, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x6d,
0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b,
0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f,
0x20, 0x50, 0x61, 0x73, 0x69, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41,
0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x50, 0x61, 0x73,
0x69, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x41, 0x70, 0x69, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x41, 0x70, 0x69, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x70, 0x69, 0x61, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x61, 0x61,
0x62, 0x69, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x61,
0x61, 0x62, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x61, 0x61, 0x62,
0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x69, 0x65,
0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74,
0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x67, 0x65,
0x6e, 0x74, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41,
0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0x69, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e, 0x69,
0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x6d, 0x65, 0x65,
0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x72, 0x6d, 0x65, 0x65, 0x6e,
0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x41, 0x6d, 0x65, 0x65, 0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x41, 0x74,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x6d, 0x65, 0x65,
0x72, 0x69, 0x6b, 0x6f, 0x6f, 0x20, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74,
0x69, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x20, 0x57, 0x61, 0x61, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72,
0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53,
0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x57, 0x61, 0x61, 0x73,
0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x57,
0x61, 0x61, 0x73, 0x74, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c,
0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61,
0x73, 0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69,
0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x75,
0x73, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57,
0x61, 0x61, 0x73, 0x74, 0x20, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x61,
0x6c, 0x69, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x73, 0x63,
0x68, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x41, 0x73, 0x65,
0x72, 0x62, 0x61, 0x69, 0x64, 0x73, 0x63, 0x68, 0x61, 0x61, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x41, 0x73, 0x65, 0x72, 0x62, 0x61, 0x69, 0x64, 0x73, 0x63, 0x68,
0x61, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x41, 0x74, 0x73, 0x6f, 0x6f, 0x72, 0x65, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x41, 0x74, 0x73, 0x6f, 0x6f, 0x72, 0x65, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x41, 0x74, 0x73, 0x6f, 0x6f, 0x72, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61,
0x64, 0x65, 0x73, 0x63, 0x68, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x42, 0x61,
0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x42,
0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x63, 0x68, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x42, 0x68, 0x75, 0x75,
0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x42, 0x6f, 0x6c,
0x69, 0x69, 0x77, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x42,
0x72, 0x61, 0x73, 0x69, 0x69, 0x6c, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69, 0x6c, 0x69, 0x65, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x42, 0x72, 0x61, 0x73, 0x69, 0x69, 0x6c, 0x69, 0x65, 0x6e,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x42, 0x72,
0x75, 0x6e, 0x65, 0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61,
0x6c, 0x61, 0x6d, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x61, 0x70, 0x77,
0x65, 0x72, 0x64, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x61,
0x70, 0x77, 0x65, 0x72, 0x64, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x61, 0x70,
0x77, 0x65, 0x72, 0x64, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72,
0x6f, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61,
0x6d, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x69, 0x69,
0x6c, 0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x69, 0x69, 0x6c,
0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x43, 0x68, 0x69, 0x69, 0x6c, 0x65, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x53, 0x63, 0x68, 0x69, 0x69,
0x6e, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x63, 0x68, 0x69, 0x69,
0x6e, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x53, 0x63, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x54, 0x73, 0x63,
0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x54, 0x73, 0x63, 0x68, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73,
0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x54, 0x73, 0x63, 0x68, 0x6f, 0x69, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x4a, 0x75, 0x6c, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x20, 0x45, 0x69,
0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x6f,
0x6c, 0x75, 0x6d, 0x62, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x4b, 0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b,
0x6f, 0x6c, 0x75, 0x6d, 0x62, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x45,
0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43,
0x6f, 0x6f, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x43, 0x6f, 0x6f, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65,
0x6e, 0x20, 0x48, 0x75, 0x61, 0x6c, 0x65, 0x77, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x4b, 0x75, 0x75, 0x62, 0x61, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b,
0x75, 0x75, 0x62, 0x61, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x44, 0x61, 0x76, 0x69, 0x73, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x44, 0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76,
0x69, 0x6c, 0x6c, 0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73,
0x74, 0x20, 0x54, 0x69, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x50, 0x75, 0x61, 0x73, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x75, 0x61, 0x73, 0x6b, 0x20, 0x45,
0x69, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x75, 0x61, 0x73, 0x6b, 0x20,
0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x45, 0x6b, 0x77, 0x61, 0x64, 0x6f, 0x6f, 0x72, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x64, 0x65, 0x6c, 0x65, 0x75, 0x72,
0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d,
0x61, 0x64, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73,
0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4d, 0x61, 0x64, 0x65, 0x6c, 0x65, 0x75, 0x72, 0x6f,
0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70,
0x65, 0x65, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x55, 0x61, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73,
0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x46,
0x61, 0x61, 0x72, 0x64, 0x65, 0x72, 0x20, 0x55, 0x61, 0x73, 0x74, 0x65,
0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65,
0x65, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73,
0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65, 0x73, 0x6b, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x57, 0x61, 0x61, 0x73, 0x74, 0x65, 0x75, 0x72, 0x6f, 0x70, 0x65, 0x65,
0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x75, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x75,
0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x61, 0x6c, 0x6b,
0x6c, 0x75, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x46, 0x61, 0x6c, 0x6b, 0x6c, 0x75, 0x6e, 0x20, 0x45, 0x69, 0x6c,
0x75, 0x6e, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x46, 0x69, 0x64, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46,
0x69, 0x64, 0x73, 0x63, 0x68, 0x69, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x46, 0x72, 0x61, 0x6e, 0x73, 0xf6, 0xf6, 0x73,
0x6b, 0x20, 0x47, 0x75, 0x61, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x46, 0x72, 0x61, 0x6e, 0x73, 0xf6, 0xf6, 0x73, 0x6b,
0x20, 0x53, 0xfc, 0xfc, 0x64, 0x20, 0x61, 0x6e, 0x20, 0x41, 0x6e, 0x74,
0x61, 0x72, 0x6b, 0x74, 0x69, 0x73, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x47,
0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x6f, 0x73, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65, 0x72, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x47, 0x65, 0x6f, 0x72, 0x67, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x47, 0x69, 0x6c, 0x62, 0x65, 0x72,
0x74, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4d, 0x61, 0x64, 0x65, 0x6c, 0x20, 0x47, 0x72, 0x65, 0x65,
0x6e, 0x77, 0x69, 0x63, 0x68, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61,
0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x75, 0x6e, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65,
0x65, 0x6e, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x6c, 0x75, 0x6e, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x47, 0x72, 0x65, 0x65, 0x6e,
0x6c, 0x75, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x47, 0x6f, 0x6c, 0x66, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x47, 0x75, 0x79, 0x61, 0x61, 0x6e, 0x61, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65,
0x75, 0x74, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x48, 0x61, 0x77,
0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x65, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75,
0x74, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x48, 0x6f, 0x6e, 0x67, 0x6b, 0x6f, 0x6e, 0x67, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x6f, 0x77, 0x64, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x43,
0x68, 0x6f, 0x77, 0x64, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x49, 0x6e, 0x64, 0x69, 0x73, 0x6b, 0x20, 0x4f, 0x6f, 0x73, 0x65,
0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x49, 0x6e, 0x64, 0x6f,
0x73, 0x63, 0x68, 0x69, 0x69, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x53, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x61, 0x6c, 0x20, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x65, 0x73, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65,
0x65, 0x73, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x61,
0x61, 0x73, 0x74, 0x20, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x65, 0x73,
0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x49, 0x72, 0x61, 0x61,
0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x49, 0x72, 0x61, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x4a, 0x61, 0x61, 0x70, 0x61, 0x6e, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x4a, 0x61, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x53,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x4a, 0x61, 0x61, 0x70, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x55, 0x61, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x73,
0x61, 0x63, 0x68, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x57, 0x61, 0x61, 0x73, 0x74, 0x20, 0x4b, 0x61, 0x73, 0x61, 0x63,
0x68, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b,
0x6f, 0x72, 0x65, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x6f, 0x72,
0x65, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x4b, 0x6f, 0x72, 0x65, 0x61, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4b, 0x6f, 0x73, 0x72, 0x61,
0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4b,
0x72, 0x61, 0x73, 0x6e, 0x6f, 0x6a, 0x61, 0x72, 0x73, 0x6b, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4b, 0x69, 0x72, 0x67,
0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4c,
0x69, 0x6e, 0x65, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77,
0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f,
0x77, 0x65, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x67,
0x61, 0x64, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69,
0x64, 0x6a, 0x4d, 0x61, 0x6c, 0x61, 0x79, 0x73, 0x69, 0x61, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4d, 0x61, 0x6c, 0x65, 0x64, 0x69, 0x69, 0x77, 0x65,
0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x72, 0x71, 0x75, 0x65,
0x73, 0x61, 0x73, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x72, 0x73,
0x68, 0x61, 0x6c, 0x6c, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74, 0x69,
0x75, 0x73, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x75, 0x72, 0x69,
0x74, 0x69, 0x75, 0x73, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x4d, 0x61, 0x77, 0x73, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69,
0x69, 0x66, 0x69, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0x6f, 0x20, 0x50, 0x61, 0x73, 0x69, 0x69, 0x66, 0x69,
0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x6f, 0x20, 0x50,
0x61, 0x73, 0x69, 0x69, 0x66, 0x69, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65,
0x72, 0x74, 0x69, 0x64, 0x6a, 0x55, 0x6c, 0x61, 0x61, 0x6e, 0x62, 0x61,
0x61, 0x74, 0x61, 0x72, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x6c, 0x61,
0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x6c,
0x61, 0x61, 0x6e, 0x62, 0x61, 0x61, 0x74, 0x61, 0x72, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4d, 0x6f, 0x73, 0x6b, 0x61,
0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4d, 0x6f, 0x73, 0x6b, 0x61, 0x75, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4d, 0x6a, 0x61, 0x6e, 0x6d, 0x61,
0x61, 0x72, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x65, 0x70, 0x61, 0x61, 0x6c,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x6b, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x4e, 0x65, 0x69, 0x6b, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6f, 0x6e, 0x69,
0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x6b, 0x61, 0x6c, 0x65, 0x64,
0x6f, 0x6f, 0x6e, 0x69, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x73, 0x69, 0x61, 0x6c, 0x75,
0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x73, 0x69, 0x61,
0x6c, 0x75, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x73, 0x69, 0x61, 0x6c,
0x75, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x75, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x75,
0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x4e, 0x65, 0x69, 0x66, 0x75, 0x6e, 0x64, 0x6c, 0x75,
0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4e,
0x69, 0x75, 0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x6f, 0x72, 0x66,
0x6f, 0x6c, 0x6b, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x20, 0x45, 0x69,
0x6c, 0x75, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f,
0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4e, 0x6f, 0x77, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e, 0x6f,
0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4e,
0x6f, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x4f, 0x6d, 0x73, 0x6b,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x4f, 0x6d, 0x73, 0x6b, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4f,
0x6d, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x50, 0x61, 0x6c,
0x61, 0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x61, 0x70, 0x75, 0x61,
0x20, 0x4e, 0x65, 0x69, 0x67, 0x75, 0x69, 0x6e, 0x65, 0x61, 0x20, 0x54,
0x69, 0x64, 0x6a, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x50, 0x65, 0x72, 0x75,
0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x65, 0x72, 0x75, 0x75, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x50, 0x65, 0x72, 0x75, 0x75, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e,
0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x69, 0x6e, 0x65, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0x69, 0x6e, 0x65, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74,
0x69, 0x64, 0x6a, 0x50, 0x68, 0x6f, 0x65, 0x6e, 0x69, 0x78, 0x20, 0x45,
0x69, 0x6c, 0x75, 0x6e, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53,
0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65, 0x20, 0x61, 0x6e,
0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x53, 0x74, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x72, 0x65,
0x20, 0x61, 0x6e, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x50, 0x69,
0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e,
0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x6f, 0x6e, 0x61, 0x70,
0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x50, 0x6a, 0xf6, 0x6e, 0x67, 0x6a,
0x61, 0x6e, 0x67, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x52, 0xe9, 0x75, 0x6e,
0x69, 0x6f, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x52, 0x6f, 0x74, 0x68,
0x65, 0x72, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x61, 0x63, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x61, 0x63,
0x68, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x61, 0x63, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53,
0x61, 0x6d, 0x6f, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x53, 0x65, 0x73,
0x63, 0x68, 0x65, 0x6c, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53,
0x69, 0x6e, 0x67, 0x61, 0x70, 0x75, 0x75, 0x72, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f, 0x6f, 0x6e, 0x65, 0x6e, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x53, 0xfc, 0xfc, 0x64, 0x67, 0x65, 0x6f, 0x72,
0x67, 0x69, 0x65, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x75, 0x75,
0x72, 0x69, 0x6e, 0x61, 0x6d, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x53, 0x79,
0x6f, 0x77, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x61, 0x68, 0x69,
0x74, 0x69, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x61, 0x69, 0x70, 0x65,
0x65, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x61, 0x69, 0x70, 0x65, 0x65,
0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x54, 0x61, 0x69, 0x70, 0x65, 0x65, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x54, 0x61, 0x64, 0x6a, 0x69, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54,
0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54,
0x6f, 0x6e, 0x67, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x20, 0x53, 0x6f,
0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x43, 0x68, 0x75, 0x75, 0x6b,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a,
0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x61, 0x6e, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x54, 0x75, 0x77, 0x61, 0x61, 0x6c, 0x75, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20,
0x54, 0x69, 0x64, 0x6a, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20,
0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64,
0x6a, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x20, 0x53, 0x6f, 0x6d,
0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x55, 0x73, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55, 0x73,
0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x53, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x55,
0x73, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x61, 0x6e, 0x20, 0x53,
0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75,
0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x57, 0x65,
0x65, 0x6e, 0x65, 0x73, 0x75, 0x65, 0x65, 0x6c, 0x61, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74, 0x6f,
0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x6c, 0x61, 0x64, 0x69, 0x77,
0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x6c, 0x61, 0x64, 0x69,
0x77, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72,
0x74, 0x69, 0x64, 0x6a, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67,
0x72, 0x61, 0x64, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x54, 0x69, 0x64, 0x6a, 0x57, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x61, 0x64, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a,
0x57, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x57,
0x61, 0x6b, 0x65, 0x20, 0x45, 0x69, 0x6c, 0x75, 0x6e, 0x20, 0x54, 0x69,
0x64, 0x6a, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x6e, 0x20,
0x46, 0x75, 0x74, 0x75, 0x6e, 0x61, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a,
0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x20, 0x53, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x20, 0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64,
0x6a, 0x4a, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a, 0x65, 0x6b, 0x61, 0x74,
0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20, 0x53, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x4a, 0x65,
0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x20,
0x53, 0x6f, 0x6d, 0x65, 0x72, 0x74, 0x69, 0x64, 0x6a, 0x59, 0x75, 0x6b,
0x6f, 0x6e, 0x20, 0x54, 0x69, 0x64, 0x6a, 0x917, 0x94d, 0x930, 0x940, 0x928,
0x20, 0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x940, 0x928, 0x20, 0x91f, 0x93e, 0x907,
0x92e, 0x74, 0x65, 0x6e, 0x70, 0x6f, 0x20, 0x70, 0x69, 0x20, 0x6d, 0x61,
0x20, 0x4b, 0x65, 0x6e, 0x69, 0x47, 0x72, 0x69, 0x6e, 0x69, 0x73, 0x20,
0x4d, 0x69, 0x6e, 0x20, 0x54, 0x61, 0x65, 0x6d, 0x627, 0x64e, 0x644, 0x627,
0x633, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x64e,
0x644, 0x627, 0x633, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651,
0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x64e, 0x644,
0x627, 0x633, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af,
0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x645, 0x627, 0x632, 0x6c6, 0x646,
0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x645, 0x627, 0x632, 0x6c6,
0x646, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x645, 0x627, 0x632, 0x6c6, 0x646, 0x626,
0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9,
0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645,
0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af,
0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627,
0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627,
0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x645, 0x631,
0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af,
0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627,
0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x6a9, 0x6c6, 0x647, 0x633, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x627, 0x645, 0x631,
0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6a9, 0x6c6,
0x647, 0x633, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9,
0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x6a9, 0x6c6, 0x647, 0x633, 0x62a, 0x6af, 0x6ce,
0x646, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x622, 0x631,
0x627, 0x645, 0x632, 0x650, 0x631, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9,
0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x622, 0x631, 0x627, 0x645,
0x632, 0x650, 0x631, 0x6cc, 0x20, 0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626,
0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x622, 0x631, 0x627, 0x645, 0x632, 0x650, 0x631, 0x6cc, 0x20,
0x627, 0x645, 0x631, 0x6cc, 0x6a9, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x631, 0x62c, 0x646,
0x62a, 0x6cc, 0x646, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627,
0x631, 0x62c, 0x646, 0x62a, 0x6cc, 0x646, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc,
0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627,
0x631, 0x62c, 0x646, 0x62a, 0x6cc, 0x646, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631,
0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646,
0x646, 0x62f, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x62a, 0x6cc, 0x646, 0x627,
0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646, 0x62f,
0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x62a, 0x6cc, 0x646, 0x627, 0x626, 0x6d2,
0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x627, 0x631, 0x62c,
0x646, 0x62a, 0x6cc, 0x646, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc,
0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x627,
0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af,
0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x200c, 0x631, 0x6c6,
0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644,
0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627,
0x645, 0x200c, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x627, 0x64f, 0x633,
0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634,
0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc,
0x627, 0x645, 0x200c, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x627, 0x64f,
0x633, 0x62a, 0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631,
0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f,
0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6,
0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a,
0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631,
0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627, 0x644,
0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x626, 0x6d2,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x626,
0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x628, 0x631, 0x627, 0x632, 0x6cc, 0x644, 0x626, 0x6d2, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646,
0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x6cc, 0x648,
0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce,
0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20,
0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af,
0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a,
0x6a9, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af,
0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627,
0x62a, 0x6a9, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x62f, 0x6ce,
0x645, 0x62a, 0x631, 0x6cc, 0x20, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9,
0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634,
0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6,
0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af,
0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646, 0x62f,
0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x647, 0x626, 0x648, 0x627,
0x6cc, 0x6cc, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x647, 0x626, 0x648,
0x627, 0x6cc, 0x6cc, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af,
0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x647, 0x626, 0x648, 0x627, 0x6cc,
0x6cc, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x627, 0x645, 0x6cc, 0x20, 0x627, 0x646, 0x62f,
0x6c6, 0x646, 0x6cc, 0x632, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634,
0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6,
0x62f, 0x631, 0x627, 0x62a, 0x6a9, 0x6cc, 0x20, 0x627, 0x646, 0x62f, 0x6c6, 0x646,
0x6cc, 0x632, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a,
0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646,
0x62f, 0x6cc, 0x20, 0x627, 0x646, 0x62f, 0x6c6, 0x646, 0x6cc, 0x632, 0x6cc, 0x627,
0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x627, 0x6cc, 0x631, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9,
0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x6cc, 0x631, 0x6a9, 0x648,
0x62a, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af,
0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x6cc, 0x631, 0x6a9, 0x648,
0x62a, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x631, 0x6c6, 0x62f, 0x631, 0x627, 0x62a, 0x6a9,
0x6cc, 0x20, 0x6a9, 0x627, 0x632, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x626, 0x6d2,
0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x631, 0x6c6, 0x646, 0x646, 0x62f, 0x6cc, 0x20, 0x6a9, 0x627, 0x632,
0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651,
0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6a9, 0x631, 0x627,
0x646, 0x633, 0x646, 0x64f, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x626, 0x6d2, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x6a9, 0x631, 0x627, 0x646, 0x633, 0x646, 0x64f, 0x6cc,
0x627, 0x631, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a,
0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6a9, 0x631, 0x627, 0x646,
0x633, 0x646, 0x64f, 0x6cc, 0x627, 0x631, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x644, 0x6c6,
0x631, 0x688, 0x20, 0x647, 0x626, 0x648, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631,
0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x644,
0x6c6, 0x631, 0x688, 0x20, 0x647, 0x626, 0x648, 0x20, 0x627, 0x64f, 0x633, 0x62a,
0x631, 0x627, 0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651,
0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x644, 0x6c6, 0x631,
0x688, 0x20, 0x647, 0x626, 0x648, 0x20, 0x627, 0x64f, 0x633, 0x62a, 0x631, 0x627,
0x644, 0x6cc, 0x627, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x645, 0x64e, 0x6af, 0x64e, 0x62f, 0x64e, 0x646,
0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x645, 0x64e, 0x6af, 0x64e, 0x62f,
0x64e, 0x646, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce,
0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x645, 0x64e, 0x6af, 0x64e, 0x62f, 0x64e,
0x646, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x622, 0x631, 0x627, 0x645, 0x632, 0x650, 0x631, 0x6cc, 0x20,
0x645, 0x650, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x6c6, 0x626, 0x6d2, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x622, 0x631, 0x627, 0x645, 0x632, 0x650, 0x631, 0x6cc, 0x20, 0x645,
0x650, 0x6a9, 0x633, 0x6cc, 0x6a9, 0x6c6, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634,
0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x622, 0x631,
0x627, 0x645, 0x632, 0x650, 0x631, 0x6cc, 0x20, 0x645, 0x650, 0x6a9, 0x633, 0x6cc,
0x6a9, 0x6c6, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x645, 0x627, 0x633, 0x6a9, 0x6c6, 0x626, 0x6d2, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x645, 0x627, 0x633, 0x6a9, 0x6c6, 0x626, 0x6d2, 0x20,
0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x645, 0x627, 0x633, 0x6a9, 0x6c6, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x648, 0x641,
0x627, 0x648, 0x648, 0x646, 0x688, 0x644, 0x626, 0x6cc, 0x646, 0x688, 0x626, 0x6d2,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6cc, 0x648, 0x641, 0x627, 0x648, 0x648,
0x646, 0x688, 0x644, 0x626, 0x6cc, 0x646, 0x688, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc,
0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646,
0x6cc, 0x648, 0x641, 0x627, 0x648, 0x648, 0x646, 0x688, 0x644, 0x626, 0x6cc, 0x646,
0x688, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x646, 0x64f, 0x631, 0x64f, 0x646, 0x647, 0x627, 0x626, 0x6d2,
0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x64f, 0x631, 0x64f, 0x646, 0x647, 0x627,
0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20,
0x633, 0x627, 0x647, 0x62a, 0x646, 0x64f, 0x631, 0x64f, 0x646, 0x647, 0x627, 0x626,
0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x646, 0x6c6, 0x648, 0x6c6, 0x633, 0x6cc, 0x628, 0x6cc, 0x631, 0x633, 0x6a9,
0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6c6, 0x648, 0x6c6, 0x633,
0x6cc, 0x628, 0x6cc, 0x631, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634,
0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x646, 0x6c6,
0x648, 0x6c6, 0x633, 0x6cc, 0x628, 0x6cc, 0x631, 0x633, 0x6a9, 0x626, 0x6d2, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627,
0x6c6, 0x645, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627,
0x6c6, 0x645, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a,
0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x627, 0x6c6, 0x645, 0x633,
0x6a9, 0x626, 0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x648, 0x64f, 0x633, 0x62a,
0x64f, 0x6a9, 0x626, 0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x648, 0x644, 0x627,
0x62f, 0x6cc, 0x648, 0x64f, 0x633, 0x62a, 0x64f, 0x6a9, 0x626, 0x6d2, 0x20, 0x6af,
0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x648, 0x644, 0x627, 0x62f, 0x6cc, 0x648, 0x64f, 0x633, 0x62a, 0x64f, 0x6a9, 0x626,
0x6d2, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647,
0x62a, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x626, 0x6d2, 0x20, 0x633,
0x627, 0x647, 0x62a, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x626, 0x6d2,
0x20, 0x6af, 0x6cc, 0x634, 0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627,
0x647, 0x62a, 0x6cc, 0x627, 0x6a9, 0x648, 0x62a, 0x633, 0x6a9, 0x626, 0x6d2, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6cc,
0x6ce, 0x6a9, 0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x648, 0x631, 0x6af, 0x626,
0x6d2, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6cc, 0x6ce, 0x6a9, 0x627, 0x62a, 0x631,
0x6cc, 0x646, 0x628, 0x648, 0x631, 0x6af, 0x626, 0x6d2, 0x20, 0x6af, 0x6cc, 0x634,
0x651, 0x62a, 0x6af, 0x6ce, 0x646, 0x20, 0x633, 0x627, 0x647, 0x62a, 0x6cc, 0x6ce,
0x6a9, 0x627, 0x62a, 0x631, 0x6cc, 0x646, 0x628, 0x648, 0x631, 0x6af, 0x626, 0x6d2,
0x20, 0x6af, 0x631, 0x645, 0x627, 0x6af, 0x6cc, 0x20, 0x633, 0x627, 0x647, 0x62a,
0x41, 0x6c, 0xe1, 0x73, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x41, 0x6c, 0xe1, 0x73, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x6c, 0xe1,
0x73, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x6d, 0xe1, 0x7a, 0xf3, 0x6e,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x6d, 0xe1, 0x7a, 0xf3,
0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x41, 0x6d, 0xe1, 0x7a, 0xf3, 0x6e, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69,
0x6b, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c,
0x67, 0xe1, 0x68, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0x41, 0x6d, 0x72,
0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1,
0x74, 0x6b, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74,
0x6b, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52,
0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x41, 0x6d, 0x72,
0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0xf3, 0x68, 0x65, 0x73, 0x74,
0x61, 0x67, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0xf3, 0x68, 0x65, 0x73, 0x74, 0x61,
0x67, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b,
0xf3, 0x68, 0x65, 0x73, 0x74, 0x61, 0x67, 0x69, 0x20, 0x41, 0x6d, 0x72,
0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72, 0xe1, 0x6d, 0x7a, 0x65,
0x72, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0xc1, 0x72, 0xe1, 0x6d, 0x7a, 0x65, 0x72, 0x69,
0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72, 0xe1,
0x6d, 0x7a, 0x65, 0x72, 0x69, 0x20, 0x41, 0x6d, 0x72, 0x69, 0x6b, 0xe1,
0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xe1,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x72, 0x6a, 0x65, 0x6e,
0x74, 0x69, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64,
0x69, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xe1, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64,
0x69, 0x20, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0xe1, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0x41, 0x72, 0x6a,
0x65, 0x6e, 0x74, 0x69, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x72, 0x61,
0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c,
0x67, 0xe1, 0x68, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69,
0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67,
0xe1, 0x68, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1,
0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61,
0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0xc1, 0x73,
0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72,
0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c,
0x67, 0xe1, 0x68, 0x69, 0x72, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20,
0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77,
0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x72, 0xf3,
0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c,
0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x72,
0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9,
0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1,
0x74, 0x6b, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72,
0xe1, 0x74, 0x6b, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69,
0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69,
0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72,
0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52,
0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9,
0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69,
0x20, 0xc1, 0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x20,
0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x42, 0x72, 0xe1, 0x7a, 0x69, 0x6c, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x42, 0x72, 0xe1, 0x7a, 0x69, 0x6c, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x42, 0x72, 0xe1,
0x7a, 0x69, 0x6c, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68,
0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0x59, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69,
0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69,
0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72,
0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64,
0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61, 0x72,
0xe1, 0x74, 0x6b, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x59, 0x75,
0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44, 0xe9, 0x6d, 0x74, 0x65, 0x72,
0xe9, 0x6e, 0x20, 0x52, 0xf3, 0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69,
0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65,
0x6e, 0x64, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20,
0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e, 0x64, 0x69, 0x20,
0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e,
0x64, 0x69, 0x20, 0x59, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x79, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48,
0x61, 0x77, 0xe1, 0x69, 0x2f, 0x41, 0x6c, 0x75, 0x73, 0x68, 0x69, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0x61, 0x77, 0xe1, 0x69, 0x2f,
0x41, 0x6c, 0x75, 0x73, 0x68, 0x69, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x48, 0x61, 0x77, 0xe1,
0x69, 0x2f, 0x41, 0x6c, 0x75, 0x73, 0x68, 0x69, 0x61, 0x79, 0x20, 0x67,
0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x44,
0x65, 0x6c, 0x67, 0xe1, 0x68, 0x69, 0x20, 0x45, 0x6e, 0x64, 0x68, 0x6f,
0x6e, 0x69, 0x73, 0x68, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x64, 0x61,
0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x45, 0x6e, 0x64, 0x68, 0x6f, 0x6e,
0x69, 0x73, 0x68, 0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65, 0x6e,
0x64, 0x69, 0x20, 0x45, 0x6e, 0x64, 0x68, 0x6f, 0x6e, 0x69, 0x73, 0x68,
0x69, 0xe1, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x45, 0x72, 0x6b, 0x75, 0x73, 0x6b, 0x61, 0x79,
0x20, 0x77, 0x61, 0x68, 0x64, 0x45, 0x72, 0x6b, 0x75, 0x73, 0x6b, 0x61,
0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x45, 0x72, 0x6b, 0x75, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3,
0x64, 0x61, 0x72, 0xe1, 0x74, 0x6b, 0x69, 0x20, 0x4b, 0xe1, 0x7a, 0x65,
0x6b, 0x65, 0x73, 0x74, 0xe1, 0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x52, 0xf3, 0x6e, 0x65,
0x6e, 0x64, 0x69, 0x20, 0x4b, 0xe1, 0x7a, 0x65, 0x6b, 0x65, 0x73, 0x74,
0xe1, 0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0xf3, 0x79, 0xe1,
0x73, 0x6b, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0xf3, 0x79, 0xe1, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x61, 0x6e,
0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4b, 0x72, 0x61,
0x73, 0x6e, 0xf3, 0x79, 0xe1, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x67, 0x61,
0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x73,
0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x2c, 0x20, 0x4c, 0xe1,
0x64, 0x68, 0x61, 0x77, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1,
0x73, 0x74, 0x72, 0xe9, 0x6c, 0x69, 0xe1, 0x61, 0x79, 0x2c, 0x20, 0x4c,
0xe1, 0x64, 0x68, 0x61, 0x77, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1,
0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x73, 0x74, 0x72, 0xe9,
0x6c, 0x69, 0xe1, 0x61, 0x79, 0x2c, 0x20, 0x4c, 0xe1, 0x64, 0x68, 0x61,
0x77, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x4d, 0xe1, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x61,
0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0xe1, 0x67, 0x61, 0x64, 0xe1,
0x6e, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77,
0x61, 0x68, 0x64, 0x4d, 0xe1, 0x67, 0x61, 0x64, 0xe1, 0x6e, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0xc1, 0x72, 0xe1, 0x6d, 0x7a, 0x65, 0x72, 0x69, 0x20, 0x4d, 0x65,
0x6b, 0x73, 0x69, 0x6b, 0xf3, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0xc1, 0x72, 0xe1, 0x6d, 0x7a, 0x65, 0x72, 0x69, 0x20, 0x4d, 0x65, 0x6b,
0x73, 0x69, 0x6b, 0xf3, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xc1, 0x72, 0xe1, 0x6d, 0x7a, 0x65,
0x72, 0x69, 0x20, 0x4d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0xf3, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4d, 0xe1, 0x73, 0x6b, 0xf3, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4d, 0xe1, 0x73, 0x6b, 0xf3, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a,
0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4d, 0xe1, 0x73, 0x6b,
0xf3, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x4e, 0x69, 0x70, 0xe1, 0x6e, 0x64, 0x6c, 0x61,
0x79, 0x6e, 0x64, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x69,
0x70, 0xe1, 0x6e, 0x64, 0x6c, 0x61, 0x79, 0x6e, 0x64, 0x61, 0x79, 0x20,
0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e,
0x69, 0x70, 0xe1, 0x6e, 0x64, 0x6c, 0x61, 0x79, 0x6e, 0x64, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x6f,
0x72, 0x6f, 0x6e, 0xe1, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1,
0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x4e, 0x61, 0x77, 0xe1, 0x73,
0x69, 0x62, 0x69, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0x4e, 0x61, 0x77, 0xe1, 0x73, 0x69, 0x62, 0x69, 0x73, 0x6b, 0x61, 0x79,
0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64,
0x4e, 0x61, 0x77, 0xe1, 0x73, 0x69, 0x62, 0x69, 0x73, 0x6b, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0xd3, 0x6d, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64,
0xd3, 0x6d, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72,
0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0xd3, 0x6d, 0x73, 0x6b, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x57, 0x61, 0x6c, 0xe1, 0x64, 0x69, 0x77, 0xe1, 0x73, 0x74, 0xf3,
0x6b, 0x61, 0x79, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0x61, 0x6c, 0xe1,
0x64, 0x69, 0x77, 0xe1, 0x73, 0x74, 0xf3, 0x6b, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x57, 0x61,
0x6c, 0xe1, 0x64, 0x69, 0x77, 0xe1, 0x73, 0x74, 0xf3, 0x6b, 0x61, 0x79,
0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1, 0x67, 0x69, 0x20, 0x77, 0x61, 0x68,
0x64, 0x59, 0xe1, 0x6b, 0x75, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x59, 0xe1, 0x6b, 0x75, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x61,
0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x59, 0xe1,
0x6b, 0x75, 0x73, 0x6b, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1,
0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x59, 0x61, 0x6b, 0xe1, 0x74,
0x72, 0x69, 0x6e, 0x62, 0x6f, 0x72, 0x67, 0x61, 0x79, 0x20, 0x77, 0x61,
0x68, 0x64, 0x59, 0x61, 0x6b, 0xe1, 0x74, 0x72, 0x69, 0x6e, 0x62, 0x6f,
0x72, 0x67, 0x61, 0x79, 0x20, 0x61, 0x6e, 0x6a, 0xe1, 0x72, 0x69, 0x20,
0x77, 0x61, 0x68, 0x64, 0x59, 0x61, 0x6b, 0xe1, 0x74, 0x72, 0x69, 0x6e,
0x62, 0x6f, 0x72, 0x67, 0x61, 0x79, 0x20, 0x67, 0x61, 0x72, 0x6d, 0xe1,
0x67, 0x69, 0x20, 0x77, 0x61, 0x68, 0x64, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66,
0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69,
0x63, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74,
0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64,
0x69, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x66, 0x72, 0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e,
0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x72,
0x69, 0x63, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e,
0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x66, 0x72, 0x69, 0x63, 0x61, 0x20,
0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x73, 0x73, 0xf2, 0x6e, 0x69,
0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x73, 0x73, 0xf2,
0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x73, 0x73, 0xf2,
0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a,
0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63,
0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f,
0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d,
0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f,
0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x6c,
0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x20, 0x41,
0x6c, 0x6c, 0x69, 0x67, 0x67, 0x69, 0xe6, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x20, 0x41, 0x6c, 0x6c, 0x69, 0x67,
0x67, 0x69, 0xe6, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65,
0x20, 0x41, 0x6c, 0x6c, 0x69, 0x67, 0x67, 0x69, 0xe6, 0x6f, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1,
0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7, 0x69, 0x66, 0x69, 0x63, 0x6f,
0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61,
0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1,
0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69,
0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69,
0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e,
0x74, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0xf1, 0x61, 0x20, 0x64, 0x65,
0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0xf1, 0x61, 0x20, 0x64,
0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x67, 0x65, 0x6e, 0x74, 0x69, 0xf1, 0x61, 0x20, 0x64, 0x65,
0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61, 0x6d, 0x65, 0x72, 0x69,
0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c,
0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x61,
0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x74, 0x6c, 0x61, 0x6e, 0x74, 0x69, 0x63, 0x6f, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x61, 0x6d, 0x65, 0x72, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x2d, 0x70, 0x6f, 0x6e, 0x65, 0x6e,
0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f,
0x2d, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x6d, 0x65, 0x7a, 0x6f, 0x2d, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76,
0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76,
0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65,
0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64,
0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x64, 0x65,
0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x67,
0x69, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72,
0x62, 0x61, 0x69, 0x67, 0x69, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a,
0x65, 0x72, 0x62, 0x61, 0x69, 0x67, 0x69, 0x61, 0x6e, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x41,
0x7a, 0x6f, 0x72, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65, 0x6f, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64,
0x65, 0x73, 0x68, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x6f, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65,
0x73, 0x68, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x68, 0x75, 0x74,
0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x42, 0x6f, 0x6c, 0x69,
0x76, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61,
0x73, 0x69, 0x6c, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73,
0x69, 0x6c, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69,
0x61, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x42, 0x72, 0x75, 0x6e, 0x65,
0x69, 0x20, 0x44, 0x61, 0x72, 0x75, 0x73, 0x73, 0x61, 0x6c, 0x61, 0x6d,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x70, 0x6f, 0x20,
0x56, 0x65, 0x72, 0x64, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x70,
0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x70, 0x70,
0x6f, 0x20, 0x56, 0x65, 0x72, 0x64, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x61, 0x6d, 0x6f, 0x72, 0x72, 0x6f, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x43, 0x68, 0x61, 0x74, 0x68, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61,
0x74, 0x68, 0x61, 0x6d, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x69,
0x6c, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72,
0x64, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x43, 0x69,
0x6c, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x43, 0x69, 0xf1, 0x61,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x61, 0x20, 0x43, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x61, 0x20, 0x43, 0x69, 0xf1, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62,
0x61, 0x6c, 0x73, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x69, 0x73, 0x6f, 0x61, 0x20, 0x43, 0x68, 0x72, 0x69, 0x73, 0x74, 0x6d,
0x61, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65,
0x20, 0x43, 0x6f, 0x63, 0x6f, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20,
0x43, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x43,
0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f,
0x6d, 0x62, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73,
0x6f, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73,
0x6f, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65,
0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43,
0x75, 0x62, 0x62, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x62, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65,
0x20, 0x43, 0x75, 0x62, 0x62, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x44, 0x61, 0x76, 0x69, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44,
0x75, 0x6d, 0x6f, 0x6e, 0x74, 0x2d, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69,
0x6c, 0x6c, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x69, 0x6d,
0x6f, 0x72, 0x20, 0x45, 0x73, 0x74, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x71, 0x75, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x71, 0x75, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61,
0x73, 0x71, 0x75, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x63, 0x75, 0x61, 0x64, 0x6f, 0x72, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75,
0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d, 0x65, 0x7a, 0x6f,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76,
0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x61, 0x6c, 0x69, 0x6e, 0x69,
0x6e, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70,
0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0xf6, 0x70, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e,
0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0xf6, 0x70,
0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x4d,
0x61, 0x6c, 0x76, 0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f,
0x65, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73,
0x6f, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0xf1, 0x65, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x67, 0x69, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46,
0x69, 0x67, 0x69, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x67, 0x69, 0x6f, 0x61, 0x20, 0x64,
0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x66, 0x72, 0x61,
0x6e, 0x73, 0x65, 0x69, 0x73, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0xe6, 0x72, 0x65, 0x20, 0x61, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x65, 0x20, 0x65, 0x20, 0x61, 0x6e, 0x74, 0x61, 0x72, 0x74, 0x69, 0x63,
0x68, 0x65, 0x20, 0x66, 0x72, 0x61, 0x6e, 0x73, 0x65, 0x69, 0x78, 0x69,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61,
0x67, 0x6f, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x47, 0x61, 0x6d,
0x62, 0x69, 0x65, 0x72, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65,
0xf2, 0x72, 0x67, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72,
0x67, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x47, 0x69,
0x6c, 0x62, 0x65, 0x72, 0x74, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x6d,
0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x47,
0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x6f, 0x61, 0x20, 0x64,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64,
0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74,
0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74, 0x65,
0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e,
0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f,
0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c,
0x61, 0x6e, 0x64, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f, 0x6e,
0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x47, 0x6f, 0x72, 0x66, 0x6f,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69,
0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41, 0x6c, 0x65, 0x75, 0x74,
0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x69, 0x2d, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x48, 0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76,
0x64, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x4f, 0xe7, 0x65, 0x61, 0x6e, 0x6f, 0x20, 0x49, 0x6e,
0x64, 0x69, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x49, 0x6e, 0x64, 0x6f, 0x63, 0x69, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73, 0x69,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6d, 0x65, 0x7a, 0x6f, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x73,
0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x76, 0x61, 0x6e, 0x74,
0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64,
0x6f, 0x6e, 0x65, 0x73, 0x69, 0x61, 0x20, 0x64, 0x65, 0x20, 0x70, 0x6f,
0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72,
0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b,
0x75, 0x74, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72,
0x61, 0x65, 0x6c, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c,
0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x2019, 0x49, 0x73, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x6f, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x47, 0x69, 0x61, 0x70, 0x70, 0x6f, 0x6e, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20,
0x47, 0x69, 0x61, 0x70, 0x70, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x47, 0x69, 0x61, 0x70,
0x70, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4b, 0x61, 0x7a,
0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x76, 0x61, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x64,
0x65, 0x20, 0x70, 0x6f, 0x6e, 0x65, 0x6e, 0x74, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x43,
0x6f, 0x72, 0x65, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x61, 0x20, 0x43, 0x6f, 0x72, 0x65, 0x61, 0x6f, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x4b, 0x6f, 0x73, 0x72, 0x61, 0x65, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61,
0x72, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f,
0x79, 0x61, 0x72, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x4b,
0x69, 0x72, 0x67, 0x68, 0x69, 0x7a, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x64, 0x61,
0x20, 0x4c, 0x69, 0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d,
0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x67, 0x61, 0x64,
0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x65,
0x73, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x6c,
0x64, 0x69, 0x76, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x72, 0x63, 0x68, 0x65, 0x69, 0x78, 0x69, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61,
0x6c, 0x6c, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72,
0x69, 0x74, 0x69, 0x75, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x74,
0x69, 0x75, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61, 0x77,
0x73, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7,
0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65, 0x73, 0x63, 0x69, 0x63,
0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0xe7, 0x69, 0x66, 0x69,
0x63, 0x6f, 0x20, 0x6d, 0x65, 0x73, 0x63, 0x69, 0x63, 0x61, 0xf1, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f,
0x20, 0x50, 0x61, 0xe7, 0x69, 0x66, 0x69, 0x63, 0x6f, 0x20, 0x6d, 0x65,
0x73, 0x63, 0x69, 0x63, 0x61, 0xf1, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x2019,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x2019,
0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x2019, 0x55, 0x6c,
0x61, 0x6e, 0x20, 0x42, 0x61, 0x74, 0x6f, 0x72, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x6f, 0x61, 0x20,
0x64, 0x61, 0x20, 0x42, 0x69, 0x72, 0x6d, 0x61, 0x6e, 0x69, 0x61, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75, 0x6f, 0x61,
0x20, 0x64, 0x6f, 0x20, 0x4e, 0x65, 0x70, 0x61, 0x6c, 0x6f, 0x61, 0x20,
0x64, 0x61, 0x20, 0x4e, 0x65, 0x75, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c,
0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x65, 0x75,
0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0xf2, 0x6e, 0x69, 0x61,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x61,
0x20, 0x4e, 0x65, 0x75, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64,
0xf2, 0x6e, 0x69, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x4e, 0x65,
0x75, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x61, 0x20, 0x4e, 0x65, 0x75, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x61, 0x20, 0x4e, 0x65, 0x75, 0x76, 0x61, 0x20, 0x5a, 0x65,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54,
0xe6, 0x72, 0x61, 0x6e, 0x65, 0x75, 0x76, 0x61, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54,
0xe6, 0x72, 0x61, 0x6e, 0x65, 0x75, 0x76, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x54, 0xe6, 0x72,
0x61, 0x6e, 0x65, 0x75, 0x76, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x69, 0x75, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x69, 0x73, 0x6f, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61, 0x20, 0x4e, 0x6f,
0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73,
0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61,
0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64,
0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6f, 0x61, 0x20,
0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20,
0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20,
0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76,
0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x76, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0xd2, 0x6d, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0xd2,
0x6d, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0xd2, 0x6d, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f,
0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x61, 0x6c, 0x61, 0x75, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x50,
0x61, 0x70, 0x75, 0x61, 0x20, 0x4e, 0x65, 0x75, 0x76, 0x61, 0x20, 0x47,
0x75, 0x69, 0x6e, 0x65, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x73, 0x74,
0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x61, 0x72, 0x61, 0x67,
0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72,
0xf9, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x6f, 0x20, 0x50, 0x65, 0x72,
0xf9, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70,
0x69, 0xf1, 0x65, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61,
0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69,
0xf1, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0xf1, 0x65, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x50, 0x68,
0x6f, 0x65, 0x6e, 0x69, 0x78, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x6e, 0x20, 0x50, 0xea, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75,
0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6e, 0x20, 0x50,
0xea, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6e, 0x20, 0x50, 0xea, 0x20, 0x65, 0x20, 0x4d, 0x69,
0x71, 0x75, 0x65, 0x6c, 0x6f, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x50, 0x6f, 0x68, 0x6e, 0x70, 0x65, 0x69, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x50, 0x79, 0x6f, 0x6e, 0x67, 0x79, 0x61, 0x6e, 0x67,
0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x52, 0xe9, 0x75, 0x6e, 0x69, 0x6f,
0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x52, 0x6f, 0x74, 0x68, 0x65,
0x72, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68,
0x61, 0x6c, 0x69, 0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61,
0x6c, 0x69, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x68, 0x61, 0x6c, 0x69, 0x6e,
0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6f,
0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64,
0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6d, 0x6f,
0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x65, 0x79, 0x63, 0x68,
0x65, 0x6c, 0x6c, 0x65, 0x73, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53,
0x63, 0x69, 0x6e, 0x67, 0x61, 0x70, 0xf4, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x69, 0x73, 0x6f, 0x65, 0x20, 0x53, 0x6f, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0x6f, 0x61, 0x20, 0x64, 0x61, 0x20, 0x47, 0x65, 0xf2, 0x72, 0x67,
0x69, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x73, 0x75, 0x64, 0x6f, 0x61, 0x20,
0x64, 0x6f, 0x20, 0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x79, 0x6f, 0x77, 0x61, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x68, 0x69, 0x74, 0x69, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6f, 0x61,
0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65,
0x20, 0x54, 0x61, 0x69, 0x70, 0x65, 0x69, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70,
0x65, 0x69, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x61, 0x67, 0x69,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x75, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61,
0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e,
0x67, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20,
0x64, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x6f, 0x61, 0x20, 0x64,
0x6f, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x6f,
0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74,
0xe6, 0x20, 0x64, 0x6f, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54,
0x75, 0x76, 0x61, 0x6c, 0x75, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x64,
0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55,
0x72, 0x75, 0x67, 0x75, 0x61, 0x79, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61,
0x6e, 0x75, 0x61, 0x74, 0x75, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75,
0x61, 0x74, 0x75, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x6f,
0x61, 0x20, 0x64, 0x6f, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x73, 0x73, 0x75,
0x65, 0x6c, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61,
0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x61, 0x20, 0x73,
0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f, 0x61,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64, 0x61, 0x72, 0x64,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x64, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6, 0x20, 0x64,
0x65, 0x20, 0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6f, 0x73, 0x74, 0x6f, 0x6b, 0x6f,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x69, 0x73, 0x6f, 0x61, 0x20,
0x64, 0x65, 0x20, 0x57, 0x61, 0x6b, 0x65, 0x6f, 0x61, 0x20, 0x64, 0x65,
0x20, 0x57, 0x61, 0x6c, 0x6c, 0x69, 0x73, 0x20, 0x65, 0x20, 0x46, 0x75,
0x74, 0x75, 0x6e, 0x61, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e,
0x64, 0x61, 0x72, 0x64, 0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75,
0x74, 0x73, 0x6b, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20, 0x73, 0x74, 0xe6,
0x20, 0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x6f,
0x61, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e,
0x62, 0x75, 0x72, 0x67, 0x6f, 0x61, 0x20, 0x73, 0x74, 0x61, 0x6e, 0x64,
0x61, 0x72, 0x64, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x61, 0x20, 0x64, 0x65, 0x20,
0x73, 0x74, 0xe6, 0x20, 0x64, 0x2019, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x61, 0x20, 0x64, 0x6f, 0x20,
0x59, 0x75, 0x6b, 0x6f, 0x6e, 0x627, 0x641, 0x63a, 0x627, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x645,
0x6cc, 0x632, 0x648, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x645, 0x6cc, 0x632, 0x648, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x639, 0x631, 0x628, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x639, 0x631, 0x628, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627,
0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633,
0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x627, 0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631,
0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627, 0x631, 0x62c, 0x646, 0x679,
0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x627,
0x631, 0x62c, 0x646, 0x679, 0x6cc, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645,
0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x622, 0x631, 0x645, 0x6cc, 0x646, 0x6cc, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x622, 0x630, 0x631, 0x628, 0x627, 0x626, 0x6cc,
0x62c, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x622, 0x630,
0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x622, 0x630,
0x631, 0x628, 0x627, 0x626, 0x6cc, 0x62c, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633, 0x20,
0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x627, 0x632, 0x648, 0x631, 0x6cc, 0x633, 0x20, 0x633, 0x6cc, 0x20, 0x645,
0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20, 0x62f, 0x6cc, 0x634,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x628, 0x646, 0x6af, 0x644, 0x6c1,
0x20, 0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x628, 0x646, 0x6af, 0x644, 0x6c1, 0x20,
0x62f, 0x6cc, 0x634, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x628,
0x6be, 0x648, 0x679, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x628, 0x648, 0x644, 0x6cc, 0x648, 0x6cc, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x686, 0x644, 0x6cc, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x686, 0x644, 0x6cc, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x686, 0x644, 0x6cc, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x686, 0x6cc, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x6a9, 0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6a9,
0x648, 0x644, 0x645, 0x628, 0x6cc, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648,
0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x627, 0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x633,
0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627,
0x6cc, 0x633, 0x679, 0x631, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x20,
0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x6a9, 0x648, 0x627,
0x688, 0x648, 0x631, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x648, 0x633,
0x637, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x648, 0x633, 0x637, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633,
0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x648, 0x633, 0x637, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x634,
0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x634,
0x631, 0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x628, 0x639, 0x6cc, 0x62f, 0x20, 0x645, 0x634, 0x631,
0x642, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc, 0x648, 0x631, 0x67e, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a, 0x631, 0x628, 0x6cc, 0x20, 0x6cc,
0x648, 0x631, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641,
0x627, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc,
0x646, 0x688, 0x632, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641, 0x627,
0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646,
0x688, 0x632, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x641, 0x627, 0x6a9, 0x20, 0x644, 0x6cc, 0x646, 0x688,
0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688, 0x632, 0x20, 0x633, 0x6cc, 0x20,
0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x641, 0x631, 0x6cc, 0x646, 0x686, 0x20, 0x6af, 0x6cc,
0x627, 0x646, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641, 0x631,
0x6cc, 0x646, 0x686, 0x20, 0x62c, 0x646, 0x648, 0x628, 0x6cc, 0x20, 0x622, 0x6ba,
0x20, 0x627, 0x646, 0x679, 0x627, 0x631, 0x6a9, 0x679, 0x6a9, 0x20, 0x679, 0x627,
0x626, 0x645, 0x6af, 0x627, 0x644, 0x627, 0x67e, 0x627, 0x6af, 0x648, 0x632, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x62c, 0x627, 0x631, 0x62c, 0x6cc,
0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x62c, 0x627, 0x631, 0x62c, 0x6cc, 0x627, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x648, 0x686,
0x20, 0x633, 0x6cc, 0x20, 0x627, 0x635, 0x644, 0x20, 0x648, 0x64e, 0x62e, 0x645,
0x634, 0x631, 0x642, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc,
0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a,
0x631, 0x628, 0x6cc, 0x20, 0x6af, 0x631, 0x6cc, 0x646, 0x20, 0x644, 0x6cc, 0x646,
0x688, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631,
0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x62e, 0x644, 0x6cc,
0x62c, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x633, 0x6cc, 0x646, 0x6af,
0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x633, 0x6cc,
0x646, 0x6af, 0x20, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688, 0x20, 0x679,
0x627, 0x626, 0x645, 0x6c1, 0x627, 0x646, 0x6af, 0x20, 0x633, 0x6cc, 0x646, 0x6af,
0x20, 0x633, 0x645, 0x631, 0x20, 0x679, 0x627, 0x626, 0x645, 0x6c1, 0x646, 0x62f,
0x648, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc,
0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x631, 0x627, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x6cc, 0x631, 0x627, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x627, 0x633, 0x631, 0x627, 0x626, 0x6cc, 0x644, 0x20,
0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x645, 0x634, 0x631, 0x642, 0x6cc, 0x20, 0x642, 0x632, 0x627, 0x62e, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x63a,
0x631, 0x628, 0x6cc, 0x20, 0x642, 0x632, 0x627, 0x62e, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6a9, 0x631, 0x63a, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x645, 0x627, 0x644,
0x62f, 0x6cc, 0x67e, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x646, 0x6cc,
0x67e, 0x627, 0x644, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x646, 0x6cc,
0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x646, 0x6cc, 0x648, 0x632, 0x6cc, 0x20, 0x644, 0x6cc, 0x646,
0x688, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x646, 0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626,
0x644, 0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x646,
0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626, 0x644, 0x6cc, 0x646, 0x688,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648,
0x64e, 0x62e, 0x646, 0x627, 0x631, 0x641, 0x648, 0x6a9, 0x20, 0x622, 0x626, 0x644,
0x6cc, 0x646, 0x688, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20,
0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641,
0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648,
0x631, 0x646, 0x6c1, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x641,
0x631, 0x646, 0x627, 0x646, 0x688, 0x648, 0x20, 0x688, 0x6cc, 0x20, 0x646, 0x648,
0x631, 0x646, 0x6c1, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627,
0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x627, 0x6a9, 0x633,
0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631,
0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x627, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x6cc, 0x631, 0x627,
0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e,
0x6cc, 0x631, 0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x645,
0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x6cc, 0x631,
0x627, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633,
0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627,
0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x67e, 0x6cc, 0x631, 0x648, 0x20, 0x633,
0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e,
0x6cc, 0x626, 0x631, 0x20, 0x622, 0x6ba, 0x20, 0x645, 0x6a9, 0x644, 0x6cc, 0x626,
0x648, 0x646, 0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20,
0x67e, 0x6cc, 0x626, 0x631, 0x20, 0x622, 0x6ba, 0x20, 0x645, 0x6a9, 0x644, 0x6cc,
0x626, 0x648, 0x646, 0x20, 0x627, 0x633, 0x679, 0x6cc, 0x646, 0x688, 0x631, 0x688,
0x20, 0x679, 0x627, 0x626, 0x645, 0x633, 0x6cc, 0x646, 0x679, 0x20, 0x67e, 0x6cc,
0x626, 0x631, 0x20, 0x622, 0x6ba, 0x20, 0x645, 0x6a9, 0x644, 0x6cc, 0x626, 0x648,
0x646, 0x20, 0x688, 0x6d2, 0x20, 0x644, 0x627, 0x626, 0x679, 0x20, 0x679, 0x627,
0x626, 0x645, 0x67e, 0x6cc, 0x627, 0x646, 0x6af, 0x20, 0x6cc, 0x627, 0x646, 0x6af,
0x20, 0x648, 0x64e, 0x62e, 0x631, 0x648, 0x62a, 0x6be, 0x6cc, 0x631, 0x627, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x633, 0x648, 0x631, 0x6cc, 0x646, 0x627,
0x645, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x62a, 0x627, 0x62c, 0x6a9,
0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x62a,
0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a, 0x627, 0x646, 0x20, 0x633, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x62a, 0x631, 0x6a9, 0x645, 0x627, 0x646, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af,
0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6cc, 0x648,
0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x6cc, 0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc,
0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x6cc,
0x648, 0x631, 0x648, 0x6af, 0x648, 0x626, 0x6d2, 0x20, 0x633, 0x6cc, 0x20, 0x645,
0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645, 0x627, 0x20, 0x633, 0x6cc, 0x20,
0x648, 0x64e, 0x62e, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646, 0x20,
0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a,
0x627, 0x646, 0x20, 0x633, 0x6cc, 0x20, 0x645, 0x639, 0x6cc, 0x627, 0x631, 0x6cc,
0x20, 0x648, 0x64e, 0x62e, 0x627, 0x632, 0x628, 0x6a9, 0x633, 0x62a, 0x627, 0x646,
0x20, 0x633, 0x6cc, 0x20, 0x645, 0x648, 0x633, 0x645, 0x20, 0x6af, 0x631, 0x645,
0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e, 0x648, 0x6cc, 0x646, 0x632,
0x648, 0x626, 0x6cc, 0x644, 0x627, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e, 0x62e,
0x648, 0x648, 0x633, 0x679, 0x627, 0x6a9, 0x20, 0x633, 0x6cc, 0x20, 0x648, 0x64e,
0x62e, 0x41, 0x66, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47,
0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67,
0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x256, 0x6f,
0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x73, 0x6f, 0x6e, 0x6f,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c,
0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74,
0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69,
0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x47, 0x61, 0x72, 0x269, 0x256, 0x6f, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20,
0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41,
0x6d, 0x61, 0x73, 0x254, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x6d, 0x61, 0x73, 0x254, 0x6e, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61, 0x73, 0x254, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61,
0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b,
0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x67, 0x269, 0x63, 0x269,
0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67,
0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20,
0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x6c, 0x269,
0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a,
0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20,
0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61, 0x6c,
0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e,
0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x62, 0x28a, 0x6e, 0x28a, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41,
0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269,
0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x62, 0x28a,
0x6e, 0x28a, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61,
0x6c, 0x269, 0x6b, 0x61, 0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x70, 0x25b,
0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x62, 0x28a, 0x6e, 0x28a,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x6d, 0x61,
0x6c, 0x269, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x70, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x6b,
0x61, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20,
0x6b, 0x61, 0x70, 0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x70, 0x69, 0x79, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x41, 0x70, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x41, 0x70, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x47, 0x61, 0x6c, 0x61, 0x61, 0x72, 0x69, 0x62, 0x75, 0x74,
0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x47, 0x61, 0x6c, 0x61, 0x61, 0x72, 0x69, 0x62, 0x75,
0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47,
0x61, 0x6c, 0x61, 0x61, 0x72, 0x69, 0x62, 0x75, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x72, 0x6a,
0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69,
0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x72, 0x6a, 0x61,
0x6e, 0x74, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x67, 0x269,
0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41,
0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e, 0x20, 0x67, 0x269, 0x74, 0x65,
0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x41, 0x72, 0x6a, 0x61, 0x6e, 0x74, 0x69, 0x6e,
0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x72, 0x6d, 0x65,
0x6e, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x41, 0x6d, 0x61, 0x6c, 0x269, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x74, 0x1dd,
0x6c, 0x61, 0x6e, 0x74, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x6d, 0x61, 0x6c, 0x269,
0x6b, 0x61, 0x20, 0x6b, 0x61, 0x74, 0x1dd, 0x6c, 0x61, 0x6e, 0x74, 0x69,
0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x6d, 0x61,
0x6c, 0x269, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x74, 0x1dd, 0x6c, 0x61, 0x6e,
0x74, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b,
0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x186, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67, 0x269, 0x63, 0x269,
0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x186, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67, 0x269,
0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x6b, 0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x67, 0x269,
0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x186,
0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67,
0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b,
0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x6b, 0x61, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x67,
0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x186, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61,
0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x186, 0x73, 0x74, 0x72,
0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b,
0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b,
0x61, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x186, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20,
0x6b, 0x61, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x186, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x67, 0x269,
0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x73, 0x25b, 0x72,
0x62, 0x61, 0x269, 0x6a, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x73, 0x25b, 0x72, 0x62, 0x61,
0x269, 0x6a, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41,
0x73, 0x25b, 0x72, 0x62, 0x61, 0x269, 0x6a, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x41, 0x73, 0x254, 0x254, 0x72, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41,
0x73, 0x254, 0x254, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41,
0x73, 0x254, 0x254, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x42, 0x61, 0x14b, 0x67, 0x6c, 0x61, 0x256, 0x25b, 0x25b, 0x73, 0x68,
0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x42, 0x61, 0x14b, 0x67, 0x6c, 0x61, 0x256, 0x25b, 0x25b, 0x73, 0x68,
0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x42, 0x61, 0x14b, 0x67,
0x6c, 0x61, 0x256, 0x25b, 0x25b, 0x73, 0x68, 0x269, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x42, 0x75, 0x74, 0x61, 0x6e, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x42, 0x6f,
0x6c, 0x69, 0x66, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c,
0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x42, 0x72, 0x61, 0x73, 0x69, 0x6c, 0x69, 0x79, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x42, 0x72, 0x61, 0x73, 0x69,
0x6c, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x42, 0x72, 0x75, 0x6e, 0x25b, 0x269, 0x20, 0x189, 0x61, 0x72, 0x75,
0x73, 0x61, 0x6c, 0x61, 0x61, 0x6d, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x61, 0x70, 0x66, 0x25b, 0x25b,
0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4b, 0x61, 0x70, 0x66, 0x25b, 0x25b, 0x72, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x4b, 0x61, 0x70, 0x66, 0x25b, 0x25b, 0x72, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x53, 0x68, 0x61, 0x6d, 0x6f,
0x72, 0x6f, 0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x53, 0x68, 0x61, 0x74, 0x61, 0x6d, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x68, 0x61,
0x74, 0x61, 0x6d, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x53, 0x68,
0x61, 0x74, 0x61, 0x6d, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x53, 0x68, 0x69, 0x6c, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x68, 0x69, 0x6c, 0x69,
0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x53, 0x68, 0x69, 0x6c,
0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x43,
0x61, 0x269, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x43, 0x61, 0x269, 0x6e, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x43, 0x61, 0x269, 0x6e, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4b, 0x6f, 0x69, 0x62, 0x61, 0x6c,
0x73, 0x61, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x4b, 0x6f, 0x69, 0x62, 0x61, 0x6c, 0x73, 0x61,
0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4b, 0x6f, 0x69,
0x62, 0x61, 0x6c, 0x73, 0x61, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a,
0x62, 0x61, 0x6b, 0x61, 0x4e, 0x6f, 0x77, 0x25b, 0x6c, 0x20, 0x6b, 0x61,
0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x6f,
0x6b, 0x6f, 0x6f, 0x73, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b,
0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x6f, 0x6c, 0x254, 0x6d, 0x62,
0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x4b, 0x6f, 0x6c, 0x254, 0x6d, 0x62, 0x69, 0x69, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x4b, 0x6f, 0x6c, 0x254, 0x6d, 0x62, 0x69,
0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4b, 0x28a,
0x6b, 0x28a, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c,
0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x4b, 0x28a, 0x6b, 0x28a, 0x20, 0x6b, 0x61, 0x42,
0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x4b, 0x28a, 0x6b, 0x28a, 0x20, 0x6b, 0x61,
0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4b, 0x75, 0x62, 0x61,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4b, 0x75, 0x62, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x4b, 0x75, 0x62, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x189, 0x65, 0x66, 0x69, 0x73, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x189, 0x69, 0x6d, 0x254,
0x6e, 0x20, 0x189, 0x69, 0x72, 0x66, 0x69, 0x6c, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x69, 0x6d, 0x254,
0x254, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50,
0x61, 0x61, 0x6b, 0x269, 0x20, 0x6b, 0x61, 0x41, 0x61, 0x74, 0x75, 0x6b,
0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50, 0x61, 0x61, 0x6b, 0x269, 0x20,
0x6b, 0x61, 0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x50, 0x61, 0x61, 0x6b, 0x269,
0x20, 0x6b, 0x61, 0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x45, 0x6b,
0x75, 0x77, 0x61, 0x256, 0x254, 0x254, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x66,
0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269, 0x63, 0x269, 0x269,
0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e,
0x61, 0x20, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e, 0x74,
0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x47, 0x61, 0x72, 0x269,
0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x6a, 0x61,
0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e,
0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c,
0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47, 0x61, 0x72,
0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x61, 0x6a,
0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47,
0x61, 0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67,
0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x47, 0x61, 0x72, 0x269, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20,
0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47, 0x61, 0x72, 0x269,
0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x269, 0x74, 0x65,
0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x254, 0x6b, 0x6c, 0x61, 0x6e,
0x256, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74,
0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x46, 0x254, 0x6b, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x6b,
0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x46, 0x254, 0x6b, 0x6c, 0x61,
0x6e, 0x256, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c,
0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x46, 0x69, 0x6a, 0x69, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x61,
0x20, 0x47, 0x61, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b,
0x61, 0x6a, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x47, 0x61, 0x66, 0x254, 0x6e, 0x74, 0x1dd, 0x6e, 0x61,
0x20, 0x67, 0x28a, 0x6e, 0x79, 0x269, 0x73, 0x6f, 0x6e, 0x6f, 0x6c, 0x61,
0x14b, 0x20, 0x6e, 0x61, 0x20, 0x47, 0x61, 0x74, 0x75, 0x74, 0x61, 0x6c,
0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x47, 0x61, 0x6c, 0x61, 0x70, 0x61, 0x67, 0x254,
0x73, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x65, 0x65, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4a, 0x254, 0x72,
0x6a, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x4a, 0x254, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4a, 0x254, 0x72, 0x6a, 0x69, 0x79,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4a, 0x69,
0x6c, 0x62, 0x25b, 0x25b, 0x72, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75,
0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x72, 0x69, 0x69, 0x6e,
0x77, 0x69, 0x73, 0x68, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x256,
0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x72,
0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x67, 0x61, 0x6a, 0x61, 0x6b,
0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x47,
0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x67, 0x61, 0x6a, 0x61,
0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x67,
0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x67, 0x269, 0x74,
0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x47, 0x72, 0x69, 0x6e, 0x6c, 0x61, 0x6e,
0x256, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c, 0x65,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x47, 0x75, 0x77, 0x61, 0x6d, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x254, 0x6c, 0x66, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x47, 0x75, 0x79,
0x61, 0x6e, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x41, 0x77, 0x61, 0x79, 0x269, 0x269, 0x20, 0x6e,
0x2019, 0x41, 0x6c, 0x65, 0x77, 0x75, 0x74, 0x69, 0x69, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x41, 0x77, 0x61,
0x79, 0x269, 0x269, 0x20, 0x6e, 0x2019, 0x41, 0x6c, 0x65, 0x77, 0x75, 0x74,
0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x41, 0x77, 0x61,
0x79, 0x269, 0x269, 0x20, 0x6e, 0x2019, 0x41, 0x6c, 0x65, 0x77, 0x75, 0x74,
0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x48,
0x254, 0x14b, 0x20, 0x4b, 0x254, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x48, 0x254, 0x14b, 0x20, 0x4b, 0x254,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x48, 0x254, 0x14b, 0x20,
0x4b, 0x254, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x48, 0x254, 0x66, 0x256, 0x1dd, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x48, 0x254, 0x66, 0x256, 0x1dd, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x48, 0x254, 0x66, 0x256, 0x1dd, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x49, 0x6e, 0x256, 0x69, 0x79,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x49, 0x6e, 0x256, 0x69, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x74, 0x65,
0x14b, 0x6b, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x49, 0x6e, 0x256, 0x69, 0x63, 0x61, 0x269, 0x6e, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x190, 0x6e, 0x256, 0x6f, 0x6e, 0x6f, 0x73, 0x69, 0x69, 0x20, 0x6b, 0x61,
0x67, 0x269, 0x63, 0x269, 0x269, 0x63, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x190, 0x6e, 0x256, 0x6f, 0x6e,
0x6f, 0x73, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6a, 0x61, 0x6b, 0x61,
0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x190, 0x6e, 0x256, 0x6f, 0x6e, 0x6f, 0x73, 0x69, 0x69,
0x20, 0x6b, 0x61, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68, 0x69, 0x6c,
0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x49, 0x72, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x49, 0x72, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x49, 0x72, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x49, 0x72, 0x6b, 0x75,
0x74, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x49, 0x72, 0x6b, 0x75, 0x74, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x59, 0x69, 0x73, 0x68, 0x72, 0x61, 0x25b, 0x6c, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x59,
0x69, 0x73, 0x68, 0x72, 0x61, 0x25b, 0x6c, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x59, 0x69, 0x73, 0x68, 0x72, 0x61, 0x25b, 0x6c, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4a, 0x61, 0x70, 0x61, 0x14b,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x4a, 0x61, 0x70, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x4a, 0x61, 0x70, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x4b, 0x61, 0x73, 0x61, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x67,
0x61, 0x6a, 0x61, 0x6b, 0x61, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x61, 0x73, 0x61,
0x73, 0x74, 0x61, 0x6e, 0x20, 0x67, 0x269, 0x74, 0x65, 0x14b, 0x73, 0x68,
0x69, 0x6c, 0x65, 0x6c, 0x61, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x6f, 0x72, 0x65, 0x65, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b,
0x6f, 0x72, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4b,
0x6f, 0x72, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x4b, 0x254, 0x73, 0x72, 0x25b, 0x25b, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x72, 0x61, 0x73, 0x6e, 0x6f, 0x79,
0x61, 0x72, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4b, 0x72,
0x61, 0x73, 0x6e, 0x6f, 0x79, 0x61, 0x72, 0x6b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66,
0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4b, 0x69, 0x72, 0x67, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x53, 0x69, 0x72, 0x69, 0x20, 0x4c, 0x61, 0x14b, 0x6b, 0x61,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4c, 0x61, 0x269, 0x6e, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75,
0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4c, 0x254, 0x72, 0x256, 0x20,
0x48, 0x6f, 0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x4c, 0x254, 0x72, 0x256, 0x20, 0x48, 0x6f, 0x6f, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4c, 0x254, 0x72, 0x256, 0x20, 0x48,
0x6f, 0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d,
0x61, 0x6b, 0x61, 0x77, 0x6f, 0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d, 0x61, 0x6b, 0x61, 0x77, 0x6f,
0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x61, 0x6b, 0x61, 0x77,
0x6f, 0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d,
0x61, 0x67, 0x61, 0x256, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d, 0x61, 0x67, 0x61, 0x256, 0x61,
0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4d, 0x61, 0x67, 0x61,
0x256, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x4d, 0x61, 0x6c, 0x25b, 0x73, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d, 0x61, 0x6c, 0x256, 0x69,
0x69, 0x66, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x4d, 0x61, 0x72, 0x6b, 0x65, 0x73, 0x61, 0x73, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d,
0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74,
0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x49, 0x6d, 0x6f, 0x72,
0x69, 0x73, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x49, 0x6d, 0x6f, 0x72, 0x69, 0x73, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x49, 0x6d, 0x6f, 0x72, 0x69, 0x73, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d, 0x254, 0x73, 0x254, 0x6e, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d,
0x25b, 0x73, 0x69, 0x6b, 0x20, 0x6b, 0x61, 0x70, 0x61, 0x73, 0x69, 0x66,
0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x4d, 0x25b, 0x73, 0x69, 0x6b, 0x20, 0x6b, 0x61, 0x70,
0x61, 0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x4d, 0x25b, 0x73, 0x69, 0x6b, 0x20, 0x6b, 0x61, 0x70, 0x61,
0x73, 0x69, 0x66, 0x69, 0x6b, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x254, 0x254,
0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61, 0x74, 0x254, 0x254, 0x72, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x55, 0x6c, 0x61, 0x6e, 0x62, 0x61,
0x74, 0x254, 0x254, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x4d, 0x6f, 0x73, 0x6b, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4d, 0x6f, 0x73, 0x6b, 0x75,
0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4d, 0x6f, 0x73, 0x6b,
0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x4d,
0x69, 0x79, 0x61, 0x6e, 0x6d, 0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4e, 0x61, 0x77, 0x75, 0x72,
0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x4e, 0x65, 0x65, 0x70, 0x61, 0x6c, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x61, 0x6c, 0x65, 0x256,
0x6f, 0x6e, 0x69, 0x69, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4b, 0x61,
0x6c, 0x65, 0x256, 0x6f, 0x6e, 0x69, 0x69, 0x20, 0x61, 0x66, 0x254, 0x6c,
0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4b, 0x61, 0x6c, 0x65,
0x256, 0x6f, 0x6e, 0x69, 0x69, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x53, 0x65, 0x6c, 0x61,
0x6e, 0x256, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x65, 0x6c, 0x61,
0x6e, 0x256, 0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x53, 0x65, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x61, 0x66,
0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x46, 0x61, 0x28a, 0x6e, 0x256, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x61, 0x66,
0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x46, 0x61, 0x28a, 0x6e, 0x256, 0x6c, 0x61, 0x6e, 0x256,
0x20, 0x61, 0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x46, 0x61, 0x28a, 0x6e, 0x256, 0x6c, 0x61, 0x6e, 0x256, 0x20, 0x61,
0x66, 0x254, 0x6c, 0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x4e, 0x69, 0x77, 0x75, 0x79, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4e, 0x254, 0x72, 0x66, 0x6f,
0x6f, 0x6b, 0x20, 0x6b, 0x61, 0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74,
0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x4e, 0x254, 0x72, 0x66, 0x6f, 0x6f, 0x6b, 0x20, 0x6b,
0x61, 0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x4e, 0x254, 0x72, 0x66, 0x6f, 0x6f,
0x6b, 0x20, 0x6b, 0x61, 0x41, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46,
0x25b, 0x72, 0x6e, 0x61, 0x6e, 0x256, 0x6f, 0x20, 0x256, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x254, 0x6e, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x25b, 0x72, 0x6e, 0x61, 0x6e,
0x256, 0x6f, 0x20, 0x256, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x254, 0x6e, 0x79,
0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x46, 0x25b, 0x72, 0x6e,
0x61, 0x6e, 0x256, 0x6f, 0x20, 0x256, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x254,
0x6e, 0x79, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61,
0x4d, 0x61, 0x72, 0x69, 0x79, 0x61, 0x6e, 0x20, 0x6b, 0x1dd, 0x62, 0x28a,
0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x67, 0x28a,
0x6e, 0x79, 0x269, 0x70, 0x25b, 0x6e, 0x25b, 0x6c, 0x61, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4e, 0x6f,
0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x6b, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x4e, 0x6f, 0x66, 0x6f,
0x73, 0x69, 0x62, 0x69, 0x72, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x4e, 0x6f, 0x66, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x72, 0x6b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x186, 0x6d, 0x73, 0x1dd,
0x6b, 0x1dd, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x186, 0x6d, 0x73, 0x1dd, 0x6b, 0x1dd, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x186, 0x6d, 0x73, 0x1dd, 0x6b, 0x1dd, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61,
0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x50, 0x61,
0x6c, 0x61, 0x77, 0x6f, 0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50, 0x61, 0x70, 0x75, 0x61, 0x73, 0x69,
0x69, 0x20, 0x47, 0x69, 0x6e, 0x65, 0x65, 0x20, 0x61, 0x66, 0x254, 0x6c,
0x269, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x77, 0x65, 0x65, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0x77, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x50, 0x61, 0x72, 0x61, 0x67, 0x75, 0x77, 0x65, 0x65, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x50, 0x65, 0x72, 0x75,
0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x50, 0x65, 0x72, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x50, 0x65, 0x72, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46,
0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256,
0x65, 0x69, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x69, 0x6e, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67,
0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x6f, 0x65, 0x6e, 0x69,
0x69, 0x73, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c,
0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x53, 0x25b, 0x14b, 0x2d, 0x50, 0x65, 0x74, 0x72,
0x254, 0x73, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x254,
0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x53, 0x25b, 0x14b, 0x2d, 0x50, 0x65, 0x74, 0x72, 0x254, 0x73, 0x20,
0x6e, 0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c, 0x254, 0x14b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x53, 0x25b, 0x14b, 0x2d, 0x50, 0x65, 0x74,
0x72, 0x254, 0x73, 0x20, 0x6e, 0x61, 0x20, 0x4d, 0x69, 0x6b, 0x65, 0x6c,
0x254, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x50,
0x269, 0x74, 0x6b, 0x25b, 0x25b, 0x6e, 0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74,
0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x50, 0x6f, 0x6e, 0x61,
0x70, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x50, 0x269, 0x79, 0x254, 0x14b, 0x79, 0x61, 0x14b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x52,
0x65, 0x65, 0x6e, 0x69, 0x79, 0x254, 0x14b, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x52, 0x6f, 0x74, 0x65, 0x72,
0x61, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x61, 0x6b,
0x61, 0x6c, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x53,
0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62,
0x61, 0x6b, 0x61, 0x53, 0x61, 0x6d, 0x6f, 0x77, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x61, 0x6d,
0x6f, 0x77, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254,
0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x53, 0x61,
0x6d, 0x6f, 0x77, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x53, 0x65, 0x73, 0x68, 0x25b, 0x6c, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x69, 0x14b, 0x67, 0x61,
0x70, 0x75, 0x75, 0x72, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x254, 0x254, 0x6e,
0x20, 0x6b, 0x61, 0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd,
0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x4a, 0x254, 0x72, 0x6a, 0x69, 0x79, 0x61, 0x20, 0x67, 0x28a,
0x6e, 0x79, 0x269, 0x73, 0x6f, 0x6e, 0x6f, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x53, 0x75, 0x72, 0x69, 0x6e,
0x61, 0x6d, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x53, 0x69, 0x79, 0x6f, 0x77, 0x61, 0x61, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x61, 0x68,
0x69, 0x74, 0x69, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x54, 0x61, 0x269, 0x70, 0x65, 0x69, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x61,
0x269, 0x70, 0x65, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x54,
0x61, 0x269, 0x70, 0x65, 0x69, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x54, 0x61, 0x6a, 0x69, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x54, 0x6f, 0x6b, 0x65, 0x6c, 0x61, 0x28a, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x254, 0x14b, 0x67, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x54, 0x254, 0x14b, 0x67, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69,
0x54, 0x254, 0x14b, 0x67, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61,
0x6b, 0x61, 0x43, 0x75, 0x75, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x54, 0x75,
0x66, 0x61, 0x6c, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x55, 0x72, 0x75, 0x67, 0x75, 0x77, 0x65, 0x65,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x55, 0x72, 0x75, 0x67, 0x75, 0x77, 0x65, 0x65, 0x20, 0x6b, 0x61, 0x61,
0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69,
0x256, 0x65, 0x69, 0x55, 0x72, 0x75, 0x67, 0x75, 0x77, 0x65, 0x65, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20,
0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x55, 0x73, 0x62, 0x65,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x55, 0x73, 0x62, 0x65, 0x65,
0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65,
0x69, 0x55, 0x73, 0x62, 0x65, 0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46, 0x61, 0x6e,
0x75, 0x77, 0x61, 0x74, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46, 0x61, 0x6e, 0x75, 0x77, 0x61, 0x74,
0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254,
0x300, 0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x46, 0x61, 0x6e, 0x75,
0x77, 0x61, 0x74, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x46, 0x65, 0x6e, 0x65, 0x73, 0x75, 0x77, 0x65, 0x6c, 0x61, 0x61,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x46, 0x6c, 0x61, 0x256, 0x69, 0x66, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46,
0x6c, 0x61, 0x256, 0x69, 0x66, 0x254, 0x73, 0x74, 0x254, 0x6b, 0x20, 0x6b,
0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256,
0x65, 0x69, 0x256, 0x65, 0x69, 0x46, 0x6c, 0x61, 0x256, 0x69, 0x66, 0x254,
0x73, 0x74, 0x254, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x46, 0x254, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x61, 0x256, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x46,
0x254, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x61, 0x256, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x46, 0x254, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61,
0x61, 0x256, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x46,
0x254, 0x73, 0x74, 0x254, 0x6b, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b,
0x6b, 0x254, 0x14b, 0x254, 0x300, 0x57, 0x65, 0x65, 0x6b, 0x20, 0x6b, 0x61,
0x42, 0x28a, 0x74, 0x75, 0x6b, 0x1dd, 0x6c, 0x74, 0x1dd, 0x6e, 0x61, 0x20,
0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x57,
0x61, 0x6c, 0x69, 0x73, 0x20, 0x6e, 0x61, 0x20, 0x46, 0x75, 0x74, 0x75,
0x6e, 0x61, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b,
0x254, 0x300, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x20, 0x6b, 0x61, 0x61, 0x6b,
0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x256, 0x65, 0x69, 0x256, 0x65, 0x69, 0x59, 0x61, 0x6b, 0x75, 0x74,
0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300,
0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b, 0x61, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x75, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x59, 0x65, 0x6b,
0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x75, 0x20, 0x6b, 0x61,
0x61, 0x6b, 0x254, 0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x20, 0x256, 0x65,
0x69, 0x256, 0x65, 0x69, 0x59, 0x65, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69,
0x6e, 0x62, 0x75, 0x75, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254, 0x14b, 0x6b,
0x254, 0x14b, 0x254, 0x300, 0x20, 0x67, 0x61, 0x66, 0x28a, 0x62, 0x61, 0x6b,
0x61, 0x59, 0x75, 0x6b, 0x254, 0x6e, 0x20, 0x6b, 0x61, 0x61, 0x6b, 0x254,
0x14b, 0x6b, 0x254, 0x14b, 0x254, 0x300, 0x905, 0x92b, 0x93c, 0x917, 0x93e, 0x928,
0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92e, 0x927, 0x94d, 0x92f, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930,
0x940, 0x915, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948,
0x932, 0x947, 0x940, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x926, 0x916, 0x923, 0x20, 0x905,
0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f,
0x92e, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x20,
0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x20, 0x905, 0x92b, 0x93c, 0x94d, 0x930, 0x940, 0x915, 0x93e, 0x20,
0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x915, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x932, 0x93e, 0x938, 0x94d,
0x200d, 0x915, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x905, 0x932, 0x93e, 0x938, 0x94d, 0x200d, 0x915, 0x93e, 0x20,
0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926,
0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x90f, 0x92e, 0x947, 0x91c, 0x93c, 0x94b, 0x928, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x90f, 0x92e, 0x947, 0x91c, 0x93c, 0x94b, 0x928, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x90f,
0x92e, 0x947, 0x91c, 0x93c, 0x94b, 0x928, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x93f, 0x915,
0x940, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20,
0x905, 0x92e, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d,
0x930, 0x940, 0x92f, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940,
0x92f, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947,
0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905,
0x92e, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20,
0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924,
0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x93f, 0x915, 0x940, 0x20,
0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x939,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909,
0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915,
0x940, 0x20, 0x92a, 0x939, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a,
0x939, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924,
0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a,
0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947,
0x930, 0x93f, 0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x909, 0x924, 0x94d, 0x924, 0x930, 0x940, 0x20, 0x905, 0x92e, 0x947, 0x930,
0x93f, 0x915, 0x940, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902, 0x924, 0x947,
0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20,
0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x90f, 0x92a, 0x93f, 0x906, 0x20, 0x926,
0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947,
0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x905, 0x930, 0x92c, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905,
0x930, 0x92c, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x905, 0x930, 0x92c, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x930, 0x94d, 0x91c,
0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905,
0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x926, 0x940,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905,
0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20,
0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902, 0x91f, 0x940, 0x928, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x91c, 0x947, 0x902,
0x91f, 0x940, 0x928, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x906, 0x930,
0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x906, 0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x906,
0x930, 0x94d, 0x92e, 0x947, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x940, 0x20,
0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f,
0x915, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x905, 0x91f, 0x932, 0x93e, 0x902, 0x91f, 0x93f, 0x915, 0x20, 0x926, 0x947,
0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20,
0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x92c, 0x93f, 0x91a, 0x932, 0x947, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930,
0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x905, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x947, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x905, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x947, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f,
0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20,
0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940,
0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x915, 0x947, 0x902, 0x926, 0x94d, 0x930, 0x940,
0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d,
0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x915,
0x947, 0x902, 0x926, 0x94d, 0x930, 0x940, 0x92f, 0x20, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20,
0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x905, 0x938, 0x94d,
0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938,
0x94d, 0x200d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x908, 0x20,
0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a,
0x93f, 0x92e, 0x940, 0x20, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938,
0x94d, 0x91f, 0x94d, 0x930, 0x947, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x936,
0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x938, 0x94d, 0x91f, 0x94d, 0x930, 0x947,
0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940,
0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20,
0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x905, 0x91c, 0x93c, 0x930, 0x92c, 0x948, 0x91c, 0x93e, 0x928,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91c, 0x93c, 0x930, 0x92c,
0x948, 0x91c, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91c, 0x93c, 0x930, 0x92c, 0x948, 0x91c, 0x93e,
0x928, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91c, 0x93c, 0x94b, 0x930,
0x947, 0x938, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91c, 0x93c,
0x94b, 0x930, 0x947, 0x938, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x905, 0x91c, 0x93c, 0x94b, 0x930, 0x947, 0x938, 0x20,
0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e,
0x926, 0x947, 0x936, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c,
0x93e, 0x902, 0x917, 0x94d, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93e,
0x902, 0x917, 0x94d, 0x932, 0x93e, 0x926, 0x947, 0x936, 0x947, 0x20, 0x926, 0x940,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92d, 0x942, 0x91f, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x94b, 0x932, 0x940, 0x935, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c,
0x93c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c, 0x940, 0x932, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x92c, 0x94d, 0x930, 0x93e, 0x91c, 0x93c, 0x940, 0x932, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x94d, 0x930, 0x942, 0x928, 0x947, 0x908,
0x20, 0x926, 0x93e, 0x930, 0x942, 0x938, 0x94d, 0x938, 0x932, 0x92e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d,
0x921, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x947, 0x92a, 0x20,
0x935, 0x930, 0x94d, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x915, 0x947, 0x92a, 0x20, 0x935, 0x930, 0x94d, 0x921,
0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x93e, 0x92e, 0x94b, 0x930, 0x94b,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x91a, 0x948, 0x925, 0x92e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a,
0x948, 0x925, 0x92e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x91a, 0x948, 0x925, 0x92e, 0x20, 0x926, 0x947, 0x20, 0x927,
0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c,
0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x93f,
0x932, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x93f, 0x932,
0x940, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x91a, 0x93f, 0x932, 0x940, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902,
0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a,
0x940, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91a, 0x940, 0x928,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x91a, 0x940, 0x928, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921,
0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x949, 0x907, 0x92c, 0x93e, 0x932,
0x94d, 0x938, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x949,
0x907, 0x92c, 0x93e, 0x932, 0x94d, 0x938, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x949, 0x907, 0x92c, 0x93e,
0x932, 0x94d, 0x938, 0x928, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94d,
0x930, 0x93f, 0x938, 0x92e, 0x938, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94b, 0x915, 0x94b, 0x938, 0x20, 0x91f,
0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902,
0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94b, 0x932,
0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x915, 0x94b, 0x932, 0x902, 0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94b, 0x932, 0x902,
0x92c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x941,
0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20,
0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x915, 0x941, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926,
0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x941, 0x915, 0x20, 0x91f,
0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902,
0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x905, 0x926, 0x94d, 0x927, 0x940, 0x20,
0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x92f, 0x942, 0x92c, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x92f,
0x942, 0x92c, 0x93e, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921,
0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x921, 0x947, 0x935, 0x93f, 0x938, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x921, 0x94d, 0x92f, 0x942, 0x92e, 0x94b,
0x902, 0x91f, 0x20, 0x921, 0x940, 0x20, 0x905, 0x930, 0x94d, 0x935, 0x93f, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947,
0x940, 0x20, 0x924, 0x93f, 0x92e, 0x94b, 0x930, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20, 0x91f, 0x93e, 0x92a,
0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x908, 0x938, 0x94d, 0x91f,
0x930, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x908, 0x938, 0x94d, 0x91f, 0x930, 0x20,
0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x907, 0x915,
0x94d, 0x935, 0x93e, 0x921, 0x94b, 0x930, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92c, 0x93f, 0x91a, 0x932, 0x947, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93f, 0x91a, 0x932,
0x947, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93f, 0x91a, 0x932,
0x947, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x940, 0x20,
0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932,
0x947, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932,
0x947, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x940,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x905, 0x917, 0x94d, 0x930, 0x20, 0x92a, 0x948, 0x932, 0x947,
0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f,
0x942, 0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x942, 0x930, 0x94b,
0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x92f, 0x942,
0x930, 0x94b, 0x92a, 0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c,
0x949, 0x915, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906,
0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x949, 0x915, 0x932, 0x948, 0x902,
0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20,
0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x949, 0x915, 0x932, 0x948, 0x902,
0x921, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947, 0x20,
0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902,
0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b,
0x93c, 0x93f, 0x91c, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b,
0x93c, 0x93f, 0x91c, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x93f, 0x91c, 0x940, 0x20, 0x926, 0x940,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x94d, 0x930, 0x947, 0x902, 0x91a, 0x20, 0x917,
0x941, 0x92f, 0x93e, 0x928, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x926, 0x916, 0x923, 0x947, 0x20, 0x92c, 0x916, 0x947, 0x902, 0x20, 0x92b, 0x93c,
0x94d, 0x930, 0x93e, 0x902, 0x938, 0x20, 0x915, 0x928, 0x947, 0x20, 0x905, 0x902,
0x91f, 0x93e, 0x930, 0x94d, 0x915, 0x91f, 0x93f, 0x915, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x917, 0x948, 0x932, 0x93e, 0x92a, 0x947, 0x917, 0x94b, 0x938,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x917, 0x948, 0x902, 0x92c, 0x93f,
0x92f, 0x930, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91c, 0x949, 0x930,
0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x91c, 0x949, 0x930, 0x94d,
0x91c, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x917, 0x93f,
0x932, 0x94d, 0x92c, 0x930, 0x94d, 0x91f, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906,
0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x935, 0x93f,
0x91a, 0x20, 0x92e, 0x940, 0x928, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932,
0x947, 0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921,
0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940, 0x20, 0x917, 0x94d,
0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940,
0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926,
0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20,
0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x91f, 0x948,
0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e, 0x940, 0x20, 0x917, 0x94d, 0x930,
0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x917, 0x94d, 0x930, 0x940, 0x928, 0x932, 0x948, 0x902, 0x921, 0x20,
0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x916, 0x93e, 0x921, 0x93c, 0x940, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x935,
0x93e, 0x908, 0x2013, 0x906, 0x932, 0x94d, 0x92f, 0x942, 0x936, 0x928, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x935, 0x93e, 0x908, 0x2013, 0x906, 0x932,
0x94d, 0x92f, 0x942, 0x936, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x935, 0x93e, 0x908, 0x2013, 0x906, 0x932,
0x94d, 0x92f, 0x942, 0x936, 0x928, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x93e, 0x901, 0x917,
0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x939, 0x93e, 0x901, 0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x93e, 0x901,
0x917, 0x20, 0x915, 0x93e, 0x901, 0x917, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x94b, 0x935, 0x94d, 0x921, 0x20, 0x926,
0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x92d, 0x93e, 0x930, 0x924, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x939, 0x93f, 0x902,
0x926, 0x20, 0x92e, 0x939, 0x93e, 0x938, 0x93e, 0x917, 0x930, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x907, 0x902, 0x921, 0x94b, 0x91a, 0x93e, 0x907, 0x928,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92c, 0x93f, 0x91a, 0x932,
0x947, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x932, 0x947, 0x940,
0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d, 0x91a, 0x93f, 0x92e,
0x940, 0x20, 0x907, 0x902, 0x921, 0x94b, 0x928, 0x947, 0x936, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x908, 0x930, 0x93e, 0x928, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x908, 0x930, 0x93e, 0x928, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x908, 0x930,
0x93e, 0x928, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c,
0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x907, 0x930, 0x94d, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x907, 0x930,
0x94d, 0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x907, 0x930, 0x94d, 0x915,
0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x907, 0x91c, 0x93c, 0x930, 0x93e, 0x907, 0x932, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x907, 0x91c, 0x93c,
0x930, 0x93e, 0x907, 0x932, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e,
0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e,
0x928, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x91c, 0x93e, 0x92a, 0x93e, 0x928, 0x947, 0x20, 0x926, 0x947, 0x20,
0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909,
0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a,
0x948, 0x932, 0x947, 0x20, 0x915, 0x91c, 0x93c, 0x93e, 0x916, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x936, 0x94d,
0x91a, 0x93f, 0x92e, 0x20, 0x915, 0x91c, 0x93c, 0x93e, 0x916, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94b, 0x930,
0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915,
0x94b, 0x930, 0x93f, 0x92f, 0x93e, 0x908, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94b, 0x930, 0x93f, 0x92f, 0x93e,
0x908, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947,
0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94b, 0x938, 0x930, 0x93e, 0x90f, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928,
0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d, 0x928, 0x94b, 0x92f,
0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x915, 0x94d, 0x930, 0x93e, 0x938, 0x94d,
0x928, 0x94b, 0x92f, 0x93e, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x940,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x915, 0x93f, 0x930, 0x94d, 0x917, 0x93f, 0x938, 0x94d, 0x200d,
0x924, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x932, 0x93e,
0x907, 0x928, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926, 0x947,
0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b, 0x935, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20,
0x939, 0x94b, 0x935, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x932, 0x949, 0x930, 0x94d, 0x921, 0x20, 0x939, 0x94b,
0x935, 0x947, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c,
0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x917, 0x93e, 0x926,
0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x92e, 0x93e, 0x917, 0x93e, 0x926, 0x93e, 0x928, 0x20, 0x926, 0x940,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92e, 0x932, 0x947, 0x936, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x932, 0x926, 0x940, 0x935, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x930, 0x94d, 0x915, 0x947,
0x938, 0x938, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x930,
0x94d, 0x936, 0x932, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926,
0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92e, 0x949, 0x930, 0x940, 0x936, 0x938, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x949, 0x930,
0x940, 0x936, 0x938, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f,
0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x93e, 0x935,
0x94d, 0x938, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x947,
0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e,
0x902, 0x924, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x947, 0x915,
0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d, 0x930, 0x936, 0x93e, 0x902,
0x924, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x92e, 0x947, 0x915, 0x94d, 0x938, 0x93f, 0x915, 0x928, 0x20, 0x92a, 0x94d,
0x930, 0x936, 0x93e, 0x902, 0x924, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f,
0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932,
0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x932, 0x93e, 0x928,
0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x909, 0x932, 0x93e, 0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x932, 0x93e,
0x928, 0x20, 0x92c, 0x91f, 0x94b, 0x930, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x92e, 0x949, 0x938, 0x94d, 0x915, 0x94b, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x949, 0x938, 0x94d, 0x915,
0x94b, 0x20, 0x926, 0x93f, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92e, 0x94d, 0x92f, 0x93e, 0x902,
0x92e, 0x93e, 0x930, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94c,
0x930, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x947, 0x92a,
0x93e, 0x932, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f,
0x942, 0x20, 0x915, 0x948, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x20,
0x915, 0x948, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94d,
0x92f, 0x942, 0x20, 0x915, 0x948, 0x932, 0x947, 0x921, 0x94b, 0x928, 0x93f, 0x92f,
0x93e, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x91c,
0x93c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93c, 0x940, 0x932, 0x948, 0x902, 0x921,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x928, 0x94d, 0x92f, 0x942, 0x91c, 0x93c, 0x940, 0x932, 0x948, 0x902, 0x921, 0x20,
0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926,
0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93c, 0x93e, 0x909, 0x902, 0x921,
0x932, 0x948, 0x902, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928,
0x94d, 0x92f, 0x942, 0x92b, 0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932, 0x948, 0x902,
0x921, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948,
0x92e, 0x928, 0x94d, 0x92f, 0x942, 0x92b, 0x93c, 0x93e, 0x909, 0x902, 0x921, 0x932,
0x948, 0x902, 0x921, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921,
0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x940, 0x92f, 0x942, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x949, 0x930, 0x92b, 0x93c, 0x949, 0x915,
0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x928, 0x949, 0x930, 0x92b, 0x93c, 0x949, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x928, 0x949, 0x930, 0x92b, 0x93c, 0x949, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942,
0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20,
0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x930, 0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d,
0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d,
0x939, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x930,
0x94d, 0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20,
0x928, 0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x930, 0x94d,
0x928, 0x93e, 0x902, 0x930, 0x94d, 0x921, 0x94b, 0x20, 0x921, 0x947, 0x20, 0x928,
0x94b, 0x930, 0x94b, 0x928, 0x94d, 0x939, 0x93e, 0x20, 0x926, 0x947, 0x20, 0x924,
0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x928, 0x94b, 0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938,
0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94b, 0x935,
0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926,
0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x928, 0x94b,
0x935, 0x94b, 0x938, 0x93f, 0x92c, 0x93f, 0x930, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915,
0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e,
0x913, 0x92e, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b,
0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924,
0x93e, 0x928, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x92a, 0x93e, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928,
0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x932, 0x93e, 0x909, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x93e, 0x92a, 0x941, 0x906, 0x20,
0x928, 0x94d, 0x92f, 0x942, 0x20, 0x917, 0x93f, 0x928, 0x940, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x930, 0x93e, 0x917,
0x94d, 0x935, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20,
0x91f, 0x948, 0x92e, 0x92a, 0x948, 0x930, 0x93e, 0x917, 0x94d, 0x935, 0x947, 0x20,
0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x947, 0x930, 0x942, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x947, 0x930, 0x942,
0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x93f, 0x932, 0x93f, 0x92a,
0x940, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x93f,
0x932, 0x93f, 0x92a, 0x940, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928,
0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x93f, 0x932, 0x93f, 0x92a, 0x940,
0x928, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92b, 0x93c, 0x940, 0x928, 0x93f,
0x915, 0x94d, 0x938, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906, 0x902, 0x20, 0x926,
0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x938, 0x947, 0x902, 0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947,
0x20, 0x915, 0x928, 0x947, 0x20, 0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932,
0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x947, 0x902,
0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x915, 0x928, 0x947, 0x20,
0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x926, 0x93e,
0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x947, 0x902,
0x91f, 0x20, 0x92a, 0x93f, 0x90f, 0x930, 0x947, 0x20, 0x915, 0x928, 0x947, 0x20,
0x92e, 0x93f, 0x915, 0x94d, 0x935, 0x947, 0x932, 0x93e, 0x928, 0x20, 0x926, 0x947,
0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20,
0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x92a, 0x93f, 0x91f, 0x915, 0x948, 0x930, 0x94d, 0x928, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x92a, 0x94b, 0x928, 0x93e, 0x92a, 0x947, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x92a, 0x94d, 0x92f, 0x94b, 0x902, 0x917, 0x92f, 0x93e,
0x902, 0x917, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x930, 0x940, 0x92f,
0x942, 0x928, 0x93f, 0x92f, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x930, 0x94b, 0x925, 0x947, 0x930, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x938, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x938, 0x916, 0x93e, 0x932, 0x93f, 0x928, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x916, 0x93e, 0x932,
0x93f, 0x928, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x92e, 0x94b, 0x906,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x92e, 0x94b, 0x906, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x938,
0x92e, 0x94b, 0x906, 0x20, 0x926, 0x947, 0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921,
0x93c, 0x947, 0x20, 0x926, 0x947, 0x20, 0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x947, 0x936, 0x947, 0x932, 0x94d,
0x938, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x93f, 0x902, 0x917,
0x93e, 0x92a, 0x941, 0x930, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x938, 0x94b, 0x932, 0x94b, 0x92e, 0x928, 0x20, 0x91f, 0x93e, 0x92a, 0x941, 0x906,
0x902, 0x20, 0x926, 0x947, 0x20, 0x91d, 0x941, 0x902, 0x921, 0x947, 0x20, 0x926,
0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x926, 0x916, 0x923, 0x947, 0x20, 0x92c, 0x916,
0x947, 0x902, 0x20, 0x91c, 0x949, 0x930, 0x94d, 0x91c, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x942, 0x930, 0x940, 0x928, 0x93e,
0x92e, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x938, 0x94d, 0x92f,
0x94b, 0x935, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924, 0x93e,
0x939, 0x93f, 0x924, 0x940, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924,
0x93e, 0x907, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924,
0x93e, 0x907, 0x92a, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x924, 0x93e, 0x907, 0x92a, 0x947, 0x20, 0x926, 0x947,
0x20, 0x927, 0x94d, 0x92f, 0x93e, 0x921, 0x93c, 0x947, 0x20, 0x926, 0x947, 0x20,
0x909, 0x91c, 0x93e, 0x932, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x924, 0x93e, 0x91c, 0x93f, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91f, 0x94b, 0x915, 0x947, 0x932,
0x93e, 0x90a, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91f, 0x94b, 0x902,
0x917, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x91f, 0x94b, 0x902,
0x917, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f,
0x948, 0x92e, 0x91f, 0x94b, 0x902, 0x917, 0x93e, 0x20, 0x926, 0x940, 0x20, 0x924,
0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x91a, 0x941, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924,
0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924, 0x941, 0x930, 0x94d,
0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x947, 0x20,
0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x924,
0x941, 0x930, 0x94d, 0x915, 0x92e, 0x947, 0x928, 0x93f, 0x938, 0x94d, 0x924, 0x93e,
0x928, 0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f,
0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x924, 0x941, 0x935, 0x93e,
0x932, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x930, 0x941,
0x917, 0x94d, 0x935, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909,
0x930, 0x941, 0x917, 0x94d, 0x935, 0x947, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x930, 0x941, 0x917, 0x94d, 0x935,
0x947, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x909, 0x91c, 0x93c, 0x94d, 0x92c,
0x947, 0x915, 0x93f, 0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f, 0x938,
0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e, 0x928, 0x915,
0x20, 0x91f, 0x948, 0x92e, 0x909, 0x91c, 0x93c, 0x94d, 0x92c, 0x947, 0x915, 0x93f,
0x938, 0x94d, 0x924, 0x93e, 0x928, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902,
0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935,
0x928, 0x941, 0x906, 0x924, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e,
0x935, 0x928, 0x941, 0x906, 0x924, 0x942, 0x20, 0x926, 0x93e, 0x20, 0x92e, 0x93e,
0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x928, 0x941, 0x906, 0x924, 0x942,
0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x947, 0x928, 0x947, 0x91c, 0x93c,
0x941, 0x90f, 0x932, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935,
0x94d, 0x932, 0x93e, 0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915,
0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x94d, 0x932, 0x93e, 0x926,
0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x94d, 0x932, 0x93e,
0x926, 0x93f, 0x935, 0x94b, 0x938, 0x94d, 0x924, 0x94b, 0x915, 0x20, 0x926, 0x940,
0x20, 0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20,
0x91f, 0x948, 0x92e, 0x935, 0x94b, 0x932, 0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930,
0x93e, 0x921, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x94b, 0x932,
0x94d, 0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x926, 0x93e, 0x20,
0x92e, 0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x935, 0x94b, 0x932, 0x94d,
0x917, 0x94b, 0x917, 0x94d, 0x930, 0x93e, 0x921, 0x20, 0x926, 0x940, 0x20, 0x924,
0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948,
0x92e, 0x935, 0x947, 0x915, 0x20, 0x91f, 0x93e, 0x92a, 0x942, 0x20, 0x926, 0x93e,
0x20, 0x91f, 0x948, 0x92e, 0x935, 0x93e, 0x932, 0x93f, 0x938, 0x20, 0x915, 0x928,
0x947, 0x20, 0x92b, 0x93c, 0x94d, 0x92f, 0x942, 0x91a, 0x942, 0x928, 0x93e, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x93e, 0x915, 0x941, 0x924, 0x94d,
0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x93e,
0x915, 0x941, 0x924, 0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x93e, 0x915, 0x941, 0x924,
0x94d, 0x938, 0x94d, 0x915, 0x20, 0x926, 0x940, 0x20, 0x924, 0x94b, 0x902, 0x926,
0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x947,
0x915, 0x93e, 0x924, 0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20,
0x926, 0x93e, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x924, 0x947,
0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x926, 0x93e, 0x20, 0x92e,
0x93e, 0x928, 0x915, 0x20, 0x91f, 0x948, 0x92e, 0x92f, 0x947, 0x915, 0x93e, 0x924,
0x947, 0x930, 0x93f, 0x928, 0x92c, 0x930, 0x94d, 0x917, 0x20, 0x926, 0x940, 0x20,
0x924, 0x94b, 0x902, 0x926, 0x93f, 0x92f, 0x93e, 0x20, 0x926, 0x93e, 0x20, 0x91f,
0x948, 0x92e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x66, 0x67, 0x68, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x65, 0x67,
0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x65, 0x67,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x65,
0x67, 0x61, 0x20, 0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x6f, 0x6e, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0,
0x66, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72,
0x65, 0x67, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc0, 0x66, 0x72, 0x65, 0x67, 0x61,
0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73,
0x6b, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61, 0x73,
0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61,
0x73, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6d, 0x61,
0x73, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019,
0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x6d, 0xe8, 0x72, 0x65, 0x67,
0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x6d, 0xe8, 0x72, 0x65,
0x67, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x6d, 0xe8, 0x72, 0x65,
0x67, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64,
0x20, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x20, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x20, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x6d, 0xe8,
0x72, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x6d,
0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x6e, 0x6f, 0x72, 0x64, 0x20, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x67,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x6d,
0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0x6d, 0x6f, 0x6e, 0x74, 0x61, 0x67, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x6d, 0xe8,
0x72, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61,
0x73, 0xec, 0x66, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e,
0x6f, 0x72, 0x64, 0x20, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x73, 0xec, 0x66, 0x65, 0x67, 0x6f,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x6d, 0xe8, 0x72,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x73,
0xec, 0x66, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x41, 0x70, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x41, 0x70, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x61, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69,
0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74,
0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f,
0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x41, 0x72, 0x6a, 0x65, 0x6e, 0x74, 0x69, 0x6e, 0x61, 0x20, 0x6f,
0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72,
0x64, 0x20, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x65, 0x67, 0x6f, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f, 0x72, 0x64, 0x20, 0x6d, 0xe8, 0x72,
0x65, 0x67, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c,
0xe0, 0x6e, 0x74, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x6e, 0x6f,
0x72, 0x64, 0x20, 0x6d, 0xe8, 0x72, 0x65, 0x67, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6c, 0xe0, 0x6e, 0x74, 0x65, 0x67, 0x6f,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61,
0x6c, 0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73,
0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72,
0x6f, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c,
0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x6f, 0x20, 0x6f, 0x73,
0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x73, 0x65,
0x6e, 0x74, 0x72, 0x6f, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72,
0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41,
0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69, 0x61,
0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x41, 0x75, 0x73, 0x74, 0x72, 0x61, 0x6c, 0x69,
0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x7a, 0x65,
0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x41, 0x7a, 0x65, 0x72, 0x62, 0x61, 0x69, 0x6a, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x41, 0x7a,
0x6f, 0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x41, 0x7a, 0x6f,
0x72, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0x41, 0x7a, 0x6f, 0x72, 0x65,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x42, 0x61, 0x6e,
0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019,
0x6c, 0x20, 0x42, 0x61, 0x6e, 0x67, 0x6c, 0x61, 0x64, 0x65, 0x73, 0x68,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x42, 0x75, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x42, 0x6f, 0x6c, 0x69, 0x76, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x42, 0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x42,
0x72, 0x61, 0x7a, 0x69, 0x6c, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x42, 0x72, 0x75, 0x6e, 0xe9, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x6f, 0x20, 0x56, 0x65, 0x72,
0x64, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x43, 0x61, 0x6f, 0x20, 0x56, 0x65, 0x72, 0x64,
0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x61, 0x6d,
0x6f, 0x72, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x69, 0x61, 0x74, 0x65,
0x6d, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x43, 0x69, 0x61, 0x74, 0x65, 0x6d, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x69, 0x61, 0x74, 0x65, 0x6d,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x43, 0x69, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019,
0x6c, 0x20, 0x43, 0x69, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x53, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x53, 0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x53,
0x69, 0x6e, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x43, 0x68, 0x6f, 0x69, 0x62, 0x61,
0x6c, 0x73, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x61,
0x64, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x63, 0x6f,
0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x43,
0x6f, 0x6c, 0x6f, 0x6e, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61,
0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6e, 0x62, 0x69, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x43, 0x6f, 0x6c, 0x6f, 0x6e, 0x62, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x43, 0x6f, 0x6f,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x43, 0x75, 0x62, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x43, 0x75,
0x62, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x44, 0x75, 0x6d,
0x6f, 0x6e, 0x74, 0x20, 0x64, 0x2019, 0x55, 0x72, 0x76, 0x69, 0x6c, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x54, 0x69,
0x6d, 0x6f, 0x72, 0x20, 0x45, 0x73, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x65,
0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73,
0x63, 0x75, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x64, 0x65, 0x20, 0x50, 0x61, 0x73, 0x63, 0x75, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xc8, 0x63, 0x75, 0x61, 0x64,
0x6f, 0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x73, 0x65, 0x6e, 0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x73, 0x65, 0x6e,
0x74, 0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45,
0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70,
0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f,
0x70, 0x61, 0x20, 0x73, 0x74, 0x72, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x2019, 0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x73, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x45, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x45, 0x75, 0x72,
0x6f, 0x70, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x6c, 0x76, 0x69,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x69,
0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x46, 0x69, 0x6a, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x47, 0x75, 0x79, 0x61, 0x6e, 0x61, 0x20, 0x66, 0x72,
0x61, 0x6e, 0x73, 0x65, 0x7a, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x20, 0x54, 0x65, 0x72, 0x65, 0x20, 0x66, 0x72, 0x61,
0x6e, 0x73, 0x65, 0x7a, 0x69, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x73,
0x75, 0x64, 0x20, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x6e,
0x74, 0xe0, 0x72, 0x74, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x20, 0x47, 0x61, 0x6c, 0xe0, 0x70, 0x61, 0x67,
0x6f, 0x73, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x47, 0x61, 0x6d, 0x62, 0x69, 0x65,
0x72, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4a,
0x65, 0x6f, 0x72, 0x6a, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4a,
0x65, 0x6f, 0x72, 0x6a, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4a, 0x65,
0x6f, 0x72, 0x6a, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x47, 0x69, 0x6c, 0x62,
0x65, 0x72, 0x74, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x6d, 0x65, 0x72, 0x69, 0x64, 0x69, 0x61, 0x6e, 0x20, 0x64, 0x65, 0x20,
0x47, 0x72, 0x65, 0x65, 0x6e, 0x77, 0x69, 0x63, 0x68, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74,
0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x6f, 0x72, 0x69, 0x65,
0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x72,
0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x6f, 0x72, 0x69,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f, 0x65, 0x6e,
0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e,
0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47, 0x72, 0x6f,
0x65, 0x6e, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64,
0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x2019, 0x6c, 0x20, 0x47, 0x6f, 0x6c, 0x66, 0x6f, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41,
0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x61,
0x77, 0x61, 0x69, 0x20, 0x65, 0x20, 0x41, 0x6c, 0x65, 0x75, 0x74, 0x69,
0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x48, 0x61, 0x77, 0x61, 0x69, 0x20, 0x65, 0x20,
0x41, 0x6c, 0x65, 0x75, 0x74, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48,
0x6f, 0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x6e, 0x67, 0x20, 0x4b, 0x6f, 0x6e, 0x67, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x48, 0x6f,
0x76, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x48, 0x6f, 0x76, 0x64, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x4f, 0x73, 0xe8, 0x61, 0x6e,
0x6f, 0x20, 0x49, 0x6e, 0x64, 0x69, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x73, 0x69, 0x6e,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e,
0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20, 0x73, 0x65, 0x6e, 0x74,
0x72, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a, 0x69, 0x61, 0x20, 0x6f,
0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x6e, 0x64, 0x6f, 0x6e, 0x65, 0x7a,
0x69, 0x61, 0x20, 0x6f, 0x73, 0x69, 0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x2019, 0x49, 0x72, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72,
0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x49, 0x72, 0x6b, 0x75, 0x74,
0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x49, 0x7a, 0x72,
0x61, 0x65, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65,
0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x49, 0x7a, 0x72, 0x61, 0x65, 0x6c, 0x65, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0x6f,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65,
0x2019, 0x6c, 0x20, 0x4a, 0x61, 0x70, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4b, 0x61, 0x7a, 0x61, 0x6b, 0x69, 0x73,
0x74, 0x61, 0x6e, 0x20, 0x6f, 0x72, 0x69, 0x65, 0x6e, 0x74, 0x61, 0x6c,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4b, 0x61,
0x7a, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20, 0x6f, 0x73, 0x69,
0x64, 0x65, 0x6e, 0x74, 0x61, 0x6c, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x72, 0xe8, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x61, 0x20, 0x43, 0x6f, 0x72, 0xe8, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x43, 0x6f, 0x72, 0xe8, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4b, 0x6f,
0x73, 0x72, 0x61, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73, 0x6e,
0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x4b, 0x72, 0x61, 0x73,
0x6e, 0x6f, 0x79, 0x61, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x4b, 0x69, 0x72, 0x67, 0x68, 0x69, 0x7a, 0x69,
0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x45, 0x63, 0x75, 0x61,
0x74, 0x6f, 0x72, 0x69, 0x61, 0x6c, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4c, 0x6f,
0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48,
0x6f, 0x77, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61,
0x20, 0x4c, 0x6f, 0x72, 0x64, 0x20, 0x48, 0x6f, 0x77, 0x65, 0x4f, 0x72,
0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65,
0x20, 0x4d, 0x61, 0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x61,
0x67, 0x61, 0x64, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x4d, 0x61, 0x6c, 0x65, 0x7a, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c,
0x65, 0x20, 0x4d, 0x61, 0x6c, 0x64, 0x69, 0x76, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x4d, 0x61, 0x72, 0x63, 0x68, 0x65, 0x7a, 0x69, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x4d, 0x61, 0x72, 0x73, 0x68, 0x61, 0x6c, 0x6c, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x6f, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75,
0x72, 0x69, 0x73, 0x69, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a,
0x6f, 0x6c, 0x65, 0x20, 0x4d, 0x61, 0x75, 0x72, 0x69, 0x73, 0x69, 0x6f,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4d, 0xe8, 0x73,
0x65, 0x67, 0x6f, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x73,
0xec, 0x66, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4d, 0xe8,
0x73, 0x65, 0x67, 0x6f, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61,
0x73, 0xec, 0x66, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4d, 0xe8,
0x73, 0x65, 0x67, 0x6f, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61,
0x73, 0xec, 0x66, 0x65, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0xe0, 0x74, 0x6f, 0x72, 0x4f,
0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64,
0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0xe0, 0x74, 0x6f, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64,
0x65, 0x20, 0x55, 0x6c, 0x61, 0x6e, 0x20, 0x42, 0xe0, 0x74, 0x6f, 0x72,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x4d, 0x6f, 0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x6f,
0x73, 0x63, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x4d, 0x79,
0x61, 0x6e, 0x6d, 0x61, 0x72, 0x20, 0x28, 0x42, 0x69, 0x72, 0x6d, 0x61,
0x6e, 0x69, 0x61, 0x29, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e, 0x61, 0x75, 0x72, 0x75,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x4e, 0x65, 0x70,
0x61, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20,
0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e,
0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61,
0x20, 0x43, 0x61, 0x6c, 0x65, 0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x43, 0x61, 0x6c, 0x65,
0x64, 0x6f, 0x6e, 0x69, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61,
0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x76,
0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e, 0x64, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x5a, 0x65, 0x6c, 0x61, 0x6e,
0x64, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72,
0x61, 0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x61,
0x6e, 0x6f, 0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73,
0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x54, 0x65, 0x72, 0x61, 0x6e, 0x6f,
0x76, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc,
0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e, 0x69, 0x75, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20,
0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6e,
0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e, 0x6f, 0x72, 0x66, 0x6f, 0x6c,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x4e,
0x6f, 0x72, 0x66, 0x6f, 0x6c, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x46, 0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65,
0x20, 0x4e, 0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x46,
0x65, 0x72, 0x6e, 0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e,
0x6f, 0x72, 0x6f, 0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x46, 0x65, 0x72, 0x6e,
0x61, 0x6e, 0x64, 0x6f, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x72, 0x6f,
0x6e, 0x68, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73, 0x69,
0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x4e, 0x6f, 0x76, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x72, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73,
0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x20, 0x4f, 0x6d, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61, 0x6b, 0x69, 0x73, 0x74, 0x61,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc,
0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x50, 0x61, 0x6c, 0xe0, 0x75, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x50, 0x61, 0x70, 0x75,
0x61, 0x20, 0x4e, 0x6f, 0x76, 0x61, 0x20, 0x47, 0x75, 0x69, 0x6e, 0xe8,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x61,
0x72, 0x61, 0x67, 0x75, 0xe0, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0xe0, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50,
0x61, 0x72, 0x61, 0x67, 0x75, 0xe0, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019, 0x6c,
0x20, 0x50, 0x65, 0x72, 0xf9, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x50, 0x65, 0x72,
0xf9, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc,
0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e,
0x65, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x46, 0x69, 0x6c, 0x69, 0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x69, 0x6c, 0x69,
0x70, 0x69, 0x6e, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x46, 0x65, 0x6e, 0x69,
0x7a, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x53, 0x2e, 0x20,
0x50, 0x69, 0x65, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x63, 0x68,
0x65, 0x6c, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d,
0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53, 0x2e, 0x20, 0x50, 0x69,
0x65, 0x72, 0x6f, 0x20, 0x65, 0x20, 0x4d, 0x69, 0x63, 0x68, 0x65, 0x6c,
0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x53, 0x2e, 0x20, 0x50, 0x69, 0x65, 0x72, 0x6f,
0x20, 0x65, 0x20, 0x4d, 0x69, 0x63, 0x68, 0x65, 0x6c, 0x6f, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f,
0x6c, 0x65, 0x20, 0x50, 0x69, 0x74, 0x63, 0x61, 0x69, 0x72, 0x6e, 0x4f,
0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c,
0x61, 0x20, 0x50, 0x6f, 0x6e, 0x70, 0xe8, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x52,
0x65, 0x75, 0x6e, 0x69, 0x6f, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x53, 0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x53,
0x61, 0x6b, 0x61, 0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x53, 0x61, 0x6b, 0x61,
0x6c, 0x69, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65,
0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x53, 0x61, 0x6d, 0xf2, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20,
0x53, 0x61, 0x6d, 0xf2, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a,
0x6f, 0x6c, 0x65, 0x20, 0x53, 0x61, 0x6d, 0xf2, 0x61, 0x4f, 0x72, 0x61,
0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x53, 0x65, 0x69, 0x73,
0x65, 0x6c, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x53, 0x61, 0x6c, 0x6f, 0x6d, 0x6f,
0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x61, 0x20, 0x47,
0x65, 0x6f, 0x72, 0x67, 0x69, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x73, 0x75, 0x64, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20,
0x53, 0x75, 0x72, 0x69, 0x6e, 0x61, 0x6d, 0x65, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x2019, 0xcc, 0x7a, 0x6f, 0x6c, 0x61, 0x20, 0x54,
0x61, 0x69, 0x74, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x54,
0x61, 0x69, 0x70, 0xe9, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72,
0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70,
0xe9, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x20, 0x54, 0x61, 0x69, 0x70, 0xe9, 0x69, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x54, 0x61, 0x6a, 0x69, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x54, 0x6f, 0x6b,
0x65, 0x6c, 0xe0, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65,
0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019,
0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc,
0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x54, 0x6f, 0x6e, 0x67, 0x61, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x43, 0x68, 0x75, 0x75, 0x6b,
0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x54, 0x75, 0x72,
0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x2019,
0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0,
0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x54, 0x75, 0x72, 0x6b, 0x6d, 0x65,
0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65,
0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x54, 0x75,
0x76, 0x61, 0x6c, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0xe0, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c,
0x2019, 0x55, 0x72, 0x75, 0x67, 0x75, 0xe0, 0x69, 0x4f, 0x72, 0x61, 0x20,
0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019,
0x55, 0x72, 0x75, 0x67, 0x75, 0xe0, 0x69, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b, 0x69, 0x73, 0x74,
0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c,
0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62, 0x65, 0x6b,
0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69,
0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x55, 0x7a, 0x62,
0x65, 0x6b, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x56,
0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f,
0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x65, 0x20,
0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x56, 0x61, 0x6e, 0x75, 0x61, 0x74,
0x75, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20,
0x64, 0x65, 0x20, 0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20,
0x56, 0x61, 0x6e, 0x75, 0x61, 0x74, 0x75, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x2019, 0x6c, 0x20, 0x56, 0x65, 0x6e, 0x65, 0x73, 0x75, 0x65, 0x6c,
0x61, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65,
0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f, 0x73,
0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74,
0xe0, 0x20, 0x64, 0x65, 0x20, 0x56, 0x6c, 0x61, 0x64, 0x69, 0x76, 0x6f,
0x73, 0x74, 0x6f, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20, 0x56,
0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x4f, 0x72, 0x61,
0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x56, 0x6f, 0x6c, 0x67, 0x6f, 0x67, 0x72, 0x61, 0x64, 0x6f, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x20, 0x6c, 0x2019, 0x41, 0x74, 0x6f, 0x6c, 0x6f,
0x20, 0x57, 0x61, 0x6b, 0x65, 0x4f, 0x72, 0x61, 0x20, 0x64, 0x65, 0x20,
0x6c, 0x65, 0x20, 0xcc, 0x7a, 0x6f, 0x6c, 0x65, 0x20, 0x57, 0x61, 0x6c,
0x6c, 0x69, 0x73, 0x20, 0x65, 0x20, 0x46, 0x75, 0x74, 0x75, 0x6e, 0x61,
0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61, 0x6c, 0x65, 0x20,
0x64, 0x65, 0x20, 0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20,
0x59, 0x61, 0x6b, 0x75, 0x74, 0x73, 0x6b, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75,
0x72, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x6e, 0x6f, 0x72, 0x6d, 0x61,
0x6c, 0x65, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61, 0x74, 0x65, 0x72,
0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4f, 0x72, 0x61, 0x20, 0x64,
0x2019, 0x69, 0x73, 0x74, 0xe0, 0x20, 0x64, 0x65, 0x20, 0x45, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x75, 0x72, 0x67, 0x6f, 0x4f, 0x72,
0x61, 0x20, 0x64, 0x65, 0x2019, 0x6c, 0x20, 0x59, 0x75, 0x6b, 0x6f, 0x6e,
0x61, 0x61, 0x70, 0x67, 0x61, 0x6e, 0x69, 0x73, 0x74, 0x61, 0x6e, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61, 0x64, 0x69, 0x6e, 0x69, 0x20, 0x61,
0x61, 0x70, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x76,
0x65, 0x1e0d, 0x61, 0x20, 0x68, 0x70, 0x75, 0x20, 0x61, 0x61, 0x70, 0x72,
0x69, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x64, 0x6b, 0x69, 0x1e47,
0x20, 0x61, 0x61, 0x70, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x6b, 0x75, 0x1e47, 0x70, 0x75, 0x20, 0x61,
0x61, 0x70, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x76,
0x65, 0x1e0d, 0x61, 0x20, 0x6b, 0x75, 0x1e47, 0x70, 0x75, 0x20, 0x61, 0x61,
0x70, 0x72, 0x69, 0x6b, 0x61, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x20, 0x6b,
0x75, 0x1e47, 0x70, 0x75, 0x20, 0x61, 0x61, 0x70, 0x72, 0x69, 0x6b, 0x61,
0x20, 0x6b, 0x61, 0x72, 0xe3, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6c,
0x61, 0x73, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6c, 0x61,
0x73, 0x6b, 0x61, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x61, 0x6c, 0x61, 0x73, 0x6b, 0x61, 0x20, 0x1e0d,
0x65, 0x6c, 0x61, 0x61, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101,
0x6d, 0x61, 0x6a, 0x6f, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x6d,
0x61, 0x6a, 0x6f, 0x6e, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x6d, 0x61, 0x6a, 0x6f, 0x6e, 0x20,
0x6b, 0x61, 0x72, 0xe3, 0x20, 0x6d, 0x61, 0x73, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6d, 0x61, 0x64, 0x69, 0x6e, 0x12b, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6d, 0x61, 0x64, 0x69, 0x6e, 0x12b, 0x20, 0x6d, 0x61, 0x6e, 0x61,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61, 0x64, 0x69,
0x6e, 0x12b, 0x20, 0x1e0d, 0x65, 0x6c, 0x61, 0x61, 0x69, 0x1e6d, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x68, 0x61, 0x70, 0x75, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x68, 0x61, 0x70, 0x75,
0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x76, 0x65, 0x1e0d, 0x61, 0x68, 0x61, 0x70, 0x75, 0x20, 0x1e0d, 0x65,
0x6c, 0x61, 0x61, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x61,
0x72, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x61, 0x72, 0x6b,
0x61, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x68, 0x61, 0x72, 0x6b, 0x61, 0x20, 0x1e0d, 0x65, 0x6c, 0x61,
0x61, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x65, 0x73, 0x69,
0x70, 0x69, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x65, 0x73, 0x69,
0x70, 0x69, 0x6b, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x70, 0x65, 0x73, 0x69, 0x70, 0x69, 0x6b, 0x20,
0x1e0d, 0x65, 0x6c, 0x61, 0x61, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x65, 0x70, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x65, 0x70, 0x69,
0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x65, 0x70, 0x69, 0x101, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69,
0x74, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x62, 0x69, 0x101, 0x74,
0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x62, 0x69, 0x101, 0x74,
0x69, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x101, 0x72, 0x62, 0x69, 0x101, 0x74, 0x69, 0x20, 0x64, 0x65,
0x6c, 0x101, 0x69, 0x74, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x6a,
0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101,
0x72, 0x6a, 0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x72, 0x6a,
0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65,
0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x75, 0x20, 0x101, 0x72, 0x6a,
0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77,
0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x75, 0x20, 0x101, 0x72,
0x6a, 0x65, 0x6e, 0x1e6d, 0x69, 0x6e, 0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101,
0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x75, 0x20, 0x101, 0x72, 0x6a, 0x65, 0x6e,
0x1e6d, 0x69, 0x6e, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x101, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x101, 0x72, 0x6d, 0x65, 0x6e, 0x69, 0x101, 0x20, 0x6d,
0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101,
0x72, 0x6d, 0x65, 0x6e, 0x69, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x1e6d, 0x6c, 0x101, 0x6e, 0x1e6d, 0x69,
0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x1e6d, 0x6c, 0x101, 0x6e, 0x1e6d,
0x69, 0x6b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x101, 0x1e6d, 0x6c, 0x101, 0x6e, 0x1e6d, 0x69, 0x6b, 0x20,
0x64, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d,
0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c,
0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72, 0x65,
0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20,
0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69,
0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c,
0x69, 0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20,
0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x75, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x74,
0x69, 0x20, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x77, 0x65, 0x1e0d,
0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x75, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72,
0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x64, 0x69, 0x6e,
0x101, 0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x75,
0x20, 0x64, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x61, 0x70, 0x75, 0x20, 0x101, 0x73, 0x74,
0x72, 0x61, 0x6c, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73,
0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x77, 0x65, 0x1e0d,
0x101, 0x68, 0x61, 0x70, 0x16b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c,
0x69, 0x101, 0x74, 0x69, 0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x61, 0x70,
0x16b, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69, 0x74, 0x69, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x16b,
0x20, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x74, 0x69,
0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x16b, 0x20,
0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x101, 0x73, 0x74, 0x72, 0x65, 0x6c, 0x69, 0x101, 0x74, 0x69, 0x20, 0x77,
0x65, 0x1e0d, 0x101, 0x6b, 0x75, 0x1e47, 0x75, 0x70, 0x16b, 0x20, 0x64, 0x65,
0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101, 0x6a, 0x65,
0x72, 0x62, 0x101, 0x69, 0x6a, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x101, 0x6a, 0x65, 0x72, 0x62, 0x101, 0x69, 0x6a, 0x101, 0x6e, 0x20, 0x6d,
0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x101,
0x6a, 0x65, 0x72, 0x62, 0x101, 0x69, 0x6a, 0x101, 0x6e, 0x20, 0x6b, 0x101,
0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6a, 0x6f, 0x72,
0x65, 0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6a, 0x6f, 0x72, 0x65,
0x73, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x61, 0x6a, 0x6f, 0x72, 0x65, 0x73, 0x20, 0x6b, 0x101, 0x72,
0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x101, 0x6e, 0x67, 0x6c,
0x101, 0x64, 0x65, 0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x101, 0x6e,
0x67, 0x6c, 0x101, 0x64, 0x65, 0x73, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x101, 0x6e, 0x67, 0x6c,
0x101, 0x64, 0x65, 0x73, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x62, 0x75, 0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x62, 0x6f, 0x6c, 0x77, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x62, 0x72, 0x101, 0x6a, 0x69, 0x6c, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x62, 0x72, 0x101, 0x6a, 0x69, 0x6c, 0x69, 0x101, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x72,
0x101, 0x6a, 0x69, 0x6c, 0x69, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x72, 0x75, 0x6e, 0x65, 0x74, 0x69,
0x20, 0x64, 0x101, 0x72, 0x75, 0x73, 0x101, 0x6c, 0x101, 0x6d, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x6b, 0x65, 0x70, 0x20, 0x62, 0x1e0d, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6b, 0x65, 0x70, 0x20, 0x62, 0x1e0d, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x65, 0x70,
0x20, 0x62, 0x1e0d, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x63, 0x101, 0x6d, 0x6f, 0x72, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x79, 0x101, 0x74,
0x101, 0x6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x79, 0x101, 0x74, 0x101,
0x6d, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x63, 0x79, 0x101, 0x74, 0x101, 0x6d, 0x20, 0x64, 0x65, 0x6c,
0x101, 0x69, 0x74, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x69, 0x6e, 0x69,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x69, 0x6c, 0x69, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x69,
0x6c, 0x69, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x63, 0x69, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x63, 0x69, 0x6e,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x63, 0x69, 0x6e, 0x20, 0x64, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6b, 0x6f, 0x69, 0x62, 0x101, 0x6c, 0x73, 0x6e,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x6f, 0x69, 0x62, 0x101, 0x6c, 0x73,
0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6b, 0x6f, 0x69, 0x62, 0x101, 0x6c, 0x73, 0x6e, 0x20, 0x6b,
0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x72, 0x69,
0x73, 0x6d, 0x101, 0x73, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6b, 0x6f, 0x6b, 0x6f, 0x73, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x101,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6b, 0x6f, 0x6c, 0x6f, 0x6d, 0x62, 0x69, 0x101, 0x20, 0x6b, 0x101,
0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x75, 0x6b, 0x20,
0x64, 0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x75, 0x6b, 0x20,
0x64, 0x12b, 0x70, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6b, 0x75, 0x6b, 0x20, 0x64, 0x12b, 0x70, 0x20,
0x101, 0x64, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6b, 0x75, 0x62, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b,
0x75, 0x62, 0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6b, 0x75, 0x62, 0x101, 0x20, 0x64, 0x65, 0x6c,
0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x64, 0x65, 0x62, 0x69,
0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x1e0d, 0x75, 0x6d, 0x6f, 0x6e, 0x1e6d,
0x20, 0x1e0d, 0x69, 0x20, 0x61, 0x72, 0x77, 0x69, 0x6c, 0x113, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x61, 0x70, 0x75, 0x20,
0x74, 0x69, 0x6d, 0x6f, 0x72, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x73,
0x1e6d, 0x72, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69,
0x73, 0x1e6d, 0x72, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x73, 0x1e6d, 0x72,
0x20, 0x64, 0x12b, 0x70, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x69, 0x71, 0x75, 0x101, 0x1e0d, 0x6f, 0x72, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x79, 0x75,
0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70,
0x69, 0x101, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x64, 0x69, 0x6e, 0x101, 0x20,
0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6b, 0x101,
0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101,
0x68, 0x70, 0x75, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74,
0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x70,
0x75, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20,
0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x70, 0x75, 0x20, 0x79, 0x75, 0x72, 0x6f,
0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x61, 0x72, 0x20, 0x77, 0x65, 0x1e0d, 0x101, 0x68,
0x70, 0x75, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x61, 0x74, 0x69, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x16b, 0x1e47, 0x16b,
0x70, 0x75, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x16b, 0x1e47,
0x16b, 0x70, 0x75, 0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74,
0x69, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x16b, 0x1e47, 0x16b, 0x70, 0x75,
0x20, 0x79, 0x75, 0x72, 0x6f, 0x70, 0x69, 0x101, 0x74, 0x69, 0x20, 0x6b,
0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x61, 0x6b,
0x6c, 0x79, 0x101, 0x6e, 0x1e0d, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x74, 0x69,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x61, 0x6b, 0x6c, 0x79, 0x101, 0x6e,
0x1e0d, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x74, 0x69, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x61, 0x6b,
0x6c, 0x79, 0x101, 0x6e, 0x1e0d, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x74, 0x69,
0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70,
0x69, 0x6a, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69, 0x6a, 0x69,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x70, 0x69, 0x6a, 0x69, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x70, 0x72, 0x65, 0x6e, 0x63, 0x68, 0x20, 0x67,
0x75, 0x79, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x72, 0x65,
0x6e, 0x63, 0x20, 0x64, 0x61, 0x6b, 0x69, 0x1e47, 0x20, 0x61, 0x1e0d, 0x65,
0x20, 0x61, 0x61, 0x6e, 0x1e6d, 0x101, 0x72, 0x6b, 0x1e6d, 0x69, 0x6b, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x67, 0x101, 0x6c, 0x101, 0x70, 0x6f, 0x67, 0x73,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x67, 0x61, 0x6d, 0x62, 0x69, 0x79, 0x72,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6a, 0x61, 0x72, 0x6a, 0x12b, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6a, 0x61, 0x72, 0x6a, 0x69, 0x101, 0x20, 0x6d,
0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6a,
0x61, 0x72, 0x6a, 0x69, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x67, 0x12b, 0x6c, 0x62, 0x72, 0x1e6d, 0x20, 0x64,
0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x67, 0x72, 0x69, 0x6e, 0x77,
0x69, 0x63, 0x20, 0x6d, 0x12b, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77,
0x65, 0x1e0d, 0x101, 0x68, 0x70, 0x75, 0x20, 0x67, 0x72, 0x69, 0x6e, 0x6c,
0x101, 0x6e, 0x64, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101,
0x68, 0x70, 0x75, 0x20, 0x67, 0x72, 0x69, 0x6e, 0x6c, 0x101, 0x6e, 0x64,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x70, 0x75, 0x20, 0x67, 0x72, 0x69,
0x6e, 0x6c, 0x101, 0x6e, 0x64, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x61, 0x6b, 0x16b, 0x1e47, 0x16b,
0x70, 0x20, 0x67, 0x72, 0x69, 0x6e, 0x6c, 0x101, 0x6e, 0x1e0d, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x61, 0x6b, 0x16b, 0x1e47, 0x16b, 0x70,
0x20, 0x67, 0x72, 0x69, 0x6e, 0x6c, 0x101, 0x6e, 0x1e0d, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65,
0x1e0d, 0x61, 0x6b, 0x16b, 0x1e47, 0x16b, 0x70, 0x20, 0x67, 0x72, 0x69, 0x6e,
0x6c, 0x101, 0x6e, 0x1e0d, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x67, 0x61, 0x6c, 0x70, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x101, 0x77, 0x101,
0x65, 0x2d, 0x20, 0x61, 0x61, 0x6c, 0x16b, 0x73, 0x101, 0x6e, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x68, 0x101, 0x77, 0x101, 0x65, 0x2d, 0x20, 0x61, 0x61,
0x6c, 0x16b, 0x73, 0x101, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x101, 0x77, 0x101, 0x65, 0x2d,
0x20, 0x61, 0x61, 0x6c, 0x16b, 0x73, 0x101, 0x6e, 0x20, 0x1e0d, 0x65, 0x6c,
0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x61, 0x6e, 0x67,
0x20, 0x6b, 0x61, 0x6e, 0x67, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x61,
0x6e, 0x67, 0x20, 0x6b, 0x61, 0x6e, 0x67, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x61, 0x6e, 0x67,
0x20, 0x6b, 0x61, 0x6e, 0x67, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x68, 0x77, 0x6f, 0x1e0d, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x68, 0x77, 0x6f, 0x1e0d, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x68, 0x77, 0x6f, 0x1e0d, 0x20, 0x6b,
0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x62, 0x101, 0x72,
0x61, 0x74, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x62, 0x101, 0x72, 0x61, 0x74, 0x20, 0x6b, 0x101, 0x6a,
0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x6e, 0x1e0d, 0x6f, 0x63, 0x69,
0x6e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x64, 0x69, 0x6e,
0x69, 0x20, 0x69, 0x6e, 0x1e0d, 0x6e, 0x65, 0x73, 0x69, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d, 0x101, 0x68, 0x70, 0x75, 0x20, 0x69,
0x6e, 0x1e0d, 0x6e, 0x65, 0x73, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x77, 0x65, 0x1e0d, 0x101, 0x6b, 0x16b, 0x1e47, 0x70, 0x16b, 0x20, 0x69, 0x6e,
0x1e0d, 0x6e, 0x65, 0x73, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69,
0x72, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x72, 0x101, 0x6e,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x69, 0x72, 0x101, 0x6e, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x12b, 0x72, 0x6b, 0x75, 0x73, 0x74, 0x61,
0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x12b, 0x72, 0x6b, 0x75, 0x73, 0x74,
0x61, 0x6b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x12b, 0x72, 0x6b, 0x75, 0x73, 0x74, 0x61, 0x6b, 0x20,
0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x6a,
0x61, 0x72, 0x101, 0x12b, 0x6c, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x6a,
0x61, 0x72, 0x101, 0x12b, 0x6c, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x69, 0x6a, 0x61, 0x72, 0x101, 0x12b,
0x6c, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6a, 0x101, 0x70, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6a,
0x101, 0x70, 0x101, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6a, 0x101, 0x70, 0x101, 0x6e, 0x20, 0x1e0d,
0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65,
0x1e0d, 0x101, 0x68, 0x61, 0x70, 0x75, 0x20, 0x6b, 0x101, 0x6a, 0x101, 0x6b,
0x73, 0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x65, 0x1e0d,
0x101, 0x6b, 0x16b, 0x1e47, 0x70, 0x16b, 0x20, 0x6b, 0x101, 0x6a, 0x101, 0x6b,
0x73, 0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x6f, 0x72,
0x69, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x6f, 0x72, 0x69,
0x101, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x6b, 0x6f, 0x72, 0x69, 0x101, 0x6e, 0x20, 0x1e0d, 0x65,
0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x61, 0x73,
0x72, 0x101, 0x113, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x72, 0x101, 0x73,
0x6e, 0x14d, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b,
0x72, 0x101, 0x73, 0x6e, 0x14d, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x72,
0x101, 0x73, 0x6e, 0x14d, 0x72, 0x73, 0x6b, 0x61, 0x20, 0x6b, 0x101, 0x72,
0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6b, 0x69, 0x72, 0x67, 0x69,
0x6a, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6c, 0x101,
0x69, 0x6e, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6c,
0x61, 0x1e5b, 0x20, 0x68, 0x61, 0x77, 0x65, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6c, 0x61, 0x1e5b, 0x20, 0x68, 0x61, 0x77, 0x65, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6c, 0x61, 0x1e5b,
0x20, 0x68, 0x61, 0x77, 0x65, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x67, 0x101, 0x64, 0x61, 0x6e,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x67, 0x101, 0x64, 0x61, 0x6e,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6d, 0x101, 0x67, 0x101, 0x64, 0x61, 0x6e, 0x20, 0x1e0d, 0x65, 0x6c,
0x101, 0x69, 0x74, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61, 0x6c, 0x65,
0x73, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x6c, 0x64,
0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x72, 0x6b, 0x73,
0x61, 0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x101, 0x72, 0x73, 0x101,
0x6c, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d,
0x61, 0x72, 0x12b, 0x73, 0x61, 0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d,
0x61, 0x72, 0x12b, 0x73, 0x61, 0x73, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x61, 0x72, 0x12b, 0x73, 0x61,
0x73, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6d, 0x101, 0x77, 0x6f, 0x73, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d,
0x65, 0x6b, 0x73, 0x69, 0x6b, 0x101, 0x6e, 0x20, 0x70, 0x65, 0x73, 0x69,
0x70, 0x69, 0x63, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x65, 0x6b, 0x73,
0x69, 0x6b, 0x101, 0x6e, 0x20, 0x70, 0x65, 0x73, 0x69, 0x70, 0x69, 0x63,
0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6d, 0x65, 0x6b, 0x73, 0x69, 0x6b, 0x101, 0x6e, 0x20, 0x70, 0x65,
0x73, 0x69, 0x70, 0x69, 0x63, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x16b, 0x6c, 0x101, 0x6e, 0x62, 0x61, 0x74,
0x6f, 0x72, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x16b, 0x6c, 0x101, 0x6e, 0x62,
0x61, 0x74, 0x6f, 0x72, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x16b, 0x6c, 0x101, 0x6e, 0x62, 0x61, 0x74,
0x6f, 0x72, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d,
0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x6f, 0x73, 0x6b, 0x6f, 0x20, 0x6b,
0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6d, 0x69, 0xf1,
0x79, 0x101, 0x6d, 0x101, 0x72, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x101,
0x75, 0x72, 0x16b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x65, 0x70, 0x101,
0x6c, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x20, 0x6b, 0x65,
0x6c, 0x65, 0x1e0d, 0x6f, 0x6e, 0x69, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x6e, 0x69, 0x16b, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x1e0d, 0x6f, 0x6e, 0x69,
0x101, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x20, 0x6b, 0x65, 0x6c, 0x65, 0x1e0d, 0x6f,
0x6e, 0x69, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x6a, 0x69, 0x6c, 0x101, 0x6e, 0x1e0d, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x6a, 0x69, 0x6c, 0x101, 0x6e,
0x1e0d, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x6a, 0x69, 0x6c, 0x101, 0x6e, 0x1e0d, 0x20,
0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e,
0x69, 0x16b, 0x70, 0x61, 0x75, 0x6e, 0x1e0d, 0x6c, 0x79, 0x101, 0x6e, 0x1e0d,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x70, 0x61, 0x75, 0x6e,
0x1e0d, 0x6c, 0x79, 0x101, 0x6e, 0x1e0d, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x70, 0x61,
0x75, 0x6e, 0x1e0d, 0x6c, 0x79, 0x101, 0x6e, 0x1e0d, 0x20, 0x1e0d, 0x65, 0x6c,
0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x69, 0x16b, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x6e, 0x6f, 0x72, 0x70, 0x6f, 0x6b, 0x20, 0x64,
0x12b, 0x70, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x6f, 0x72, 0x70, 0x6f,
0x6b, 0x20, 0x64, 0x12b, 0x70, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x6f, 0x72, 0x70, 0x6f, 0x6b,
0x20, 0x64, 0x12b, 0x70, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x70, 0x61, 0x72, 0x6e, 0x7e, 0x1e0d, 0x6f, 0x20,
0x1e0d, 0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6f, 0x68, 0x101, 0x7e, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x70, 0x61, 0x72, 0x6e, 0x7e, 0x1e0d, 0x6f, 0x20, 0x1e0d,
0x65, 0x20, 0x6e, 0x6f, 0x72, 0x6f, 0x68, 0x101, 0x7e, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x61,
0x72, 0x6e, 0x7e, 0x1e0d, 0x6f, 0x20, 0x1e0d, 0x65, 0x20, 0x6e, 0x6f, 0x72,
0x6f, 0x68, 0x101, 0x7e, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x6e, 0x61, 0x77, 0x6f, 0x73, 0x69, 0x62, 0x69, 0x73,
0x72, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x61, 0x77, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x73, 0x72, 0x6b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x6e, 0x61, 0x77, 0x6f, 0x73,
0x69, 0x62, 0x69, 0x73, 0x72, 0x6b, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x61, 0x6d, 0x73, 0x6b, 0x61, 0x20, 0x6d, 0x101, 0x6e,
0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x61, 0x6d, 0x73,
0x6b, 0x61, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x70, 0x101, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x70, 0x101, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x6d,
0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70,
0x101, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x6b, 0x101, 0x72, 0x7e,
0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x101, 0x6c, 0x101, 0x75, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x70, 0x101, 0x70, 0x75, 0x101, 0x20, 0x6e, 0x69,
0x75, 0x20, 0x67, 0x75, 0x6e, 0x12b, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x70, 0x101, 0x72, 0x101, 0x67, 0x75, 0x79, 0x113, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x70, 0x101, 0x72, 0x101, 0x67, 0x75, 0x79, 0x113, 0x20, 0x6d, 0x101,
0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x101,
0x72, 0x101, 0x67, 0x75, 0x79, 0x113, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x65, 0x72, 0x75, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x70, 0x65, 0x72, 0x75, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x65, 0x72, 0x75, 0x20,
0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69,
0x6c, 0x69, 0x70, 0x69, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b,
0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69, 0x6c, 0x69, 0x70, 0x69,
0x6e, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x70, 0x69, 0x6e, 0x69, 0x6b, 0x73, 0x61, 0x20, 0x64, 0x12b, 0x70, 0x61,
0x74, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x113, 0x6e, 0x1e6d, 0x20,
0x70, 0x69, 0x65, 0x72, 0x113, 0x20, 0x61, 0x1e0d, 0x113, 0x20, 0x6d, 0x65,
0x6b, 0x77, 0x113, 0x6c, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73,
0x113, 0x6e, 0x1e6d, 0x20, 0x70, 0x69, 0x65, 0x72, 0x113, 0x20, 0x61, 0x1e0d,
0x113, 0x20, 0x6d, 0x65, 0x6b, 0x77, 0x113, 0x6c, 0x101, 0x6e, 0x20, 0x6d,
0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73,
0x113, 0x6e, 0x1e6d, 0x20, 0x70, 0x69, 0x65, 0x72, 0x113, 0x20, 0x61, 0x1e0d,
0x113, 0x20, 0x6d, 0x65, 0x6b, 0x77, 0x113, 0x6c, 0x101, 0x6e, 0x20, 0x1e0d,
0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x69,
0x1e6d, 0x6b, 0x113, 0x72, 0x61, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70,
0x6f, 0x6e, 0x101, 0x70, 0x65, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x70, 0x79,
0x6f, 0x6e, 0x67, 0x61, 0x79, 0x101, 0x6e, 0x67, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x72, 0x12b, 0x16b, 0x6e, 0x12b, 0x61, 0x6e, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x72, 0x6f, 0x74, 0x113, 0x72, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x73, 0x61, 0x6b, 0x101, 0x6c, 0x69, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x73, 0x61, 0x6b, 0x101, 0x6c, 0x69, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x61, 0x6b, 0x101,
0x6c, 0x69, 0x6e, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x73, 0x61, 0x101, 0x6d, 0x6f, 0x61, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x73, 0x61, 0x101, 0x6d, 0x6f, 0x61, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x61, 0x101, 0x6d,
0x6f, 0x61, 0x20, 0x1e0d, 0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x73, 0x113, 0x73,
0x65, 0x6c, 0x73, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x69, 0x6e, 0x67,
0x101, 0x70, 0x75, 0x72, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x6f, 0x6c, 0x6f, 0x6d, 0x61, 0x6e,
0x20, 0x64, 0x12b, 0x70, 0x61, 0x74, 0x69, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x64, 0x6b, 0x12b, 0x1e47, 0x61, 0x20, 0x6a, 0x61, 0x72, 0x6a, 0x12b, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x75, 0x72, 0x12b, 0x6e, 0x101, 0x6d,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x73, 0x61, 0x77, 0x101, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x74, 0x101, 0x68, 0x69, 0x74, 0x69, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x74, 0x101, 0x69, 0x70, 0x113, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x74,
0x101, 0x69, 0x70, 0x113, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x74, 0x101, 0x69, 0x70, 0x113, 0x20, 0x1e0d,
0x65, 0x6c, 0x101, 0x69, 0x1e6d, 0x74, 0x101, 0x6a, 0x69, 0x6b, 0x69, 0x73,
0x74, 0x101, 0x6e, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x1e6d, 0x6f, 0x6b, 0x65,
0x6c, 0x101, 0x75, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x1e6d, 0x61, 0x6e, 0x67,
0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x1e6d, 0x61, 0x6e, 0x67, 0x101, 0x20,
0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x1e6d, 0x61, 0x6e, 0x67, 0x101, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x63, 0x75, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e, 0x69, 0x73, 0x74, 0x101, 0x6e,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x6d, 0x61, 0x6e, 0x61, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x74, 0x75, 0x72, 0x6b, 0x6d, 0x65, 0x6e,
0x69, 0x73, 0x74, 0x101, 0x6e, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x74, 0x75, 0x77, 0x61, 0x6c, 0x16b, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x75, 0x72, 0x67, 0x75, 0x77, 0x113, 0x20, 0x62, 0x65,
0x6c, 0x101, 0x75, 0x72, 0x67, 0x75, 0x77, 0x113, 0x20, 0x6d, 0x61, 0x6e,
0x61, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x75, 0x72, 0x67, 0x75,
0x77, 0x113, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x75, 0x7a, 0x77, 0x113, 0x6b, 0x69, 0x73, 0x74, 0x101, 0x6e, 0x20,
0x62, 0x65, 0x6c, 0x101, 0x75, 0x7a, 0x77, 0x113, 0x6b, 0x69, 0x73, 0x74,
0x101, 0x6e, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x75, 0x7a, 0x77, 0x113, 0x6b, 0x69, 0x73, 0x74, 0x101,
0x6e, 0x20, 0x6b, 0x101, 0x72, 0xe3, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77,
0x61, 0x6e, 0x75, 0x101, 0x74, 0x16b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77,
0x61, 0x6e, 0x75, 0x101, 0x74, 0x16b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x61, 0x6e, 0x75, 0x101,
0x74, 0x16b, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x77, 0x65, 0x6e, 0x113, 0x7a, 0x75, 0x65, 0x6c, 0x101, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x77, 0x6c, 0x101, 0x64, 0x69, 0x77, 0x6f, 0x73, 0x74,
0x61, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x6c, 0x101, 0x64, 0x69,
0x77, 0x6f, 0x73, 0x74, 0x61, 0x6b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e,
0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x6c, 0x101, 0x64, 0x69,
0x77, 0x6f, 0x73, 0x74, 0x61, 0x6b, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x101, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x101, 0x1e0d, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x101, 0x6c, 0x67, 0x6f,
0x67, 0x72, 0x101, 0x1e0d, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x77, 0x101, 0x6c, 0x67, 0x6f, 0x67, 0x72,
0x101, 0x1e0d, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c,
0x101, 0x77, 0x61, 0x6b, 0x61, 0x20, 0x64, 0x12b, 0x70, 0x61, 0x20, 0x62,
0x65, 0x6c, 0x101, 0x77, 0x61, 0x6c, 0x69, 0x73, 0x20, 0x61, 0x1e0d, 0x113,
0x20, 0x70, 0x75, 0x1e6d, 0x75, 0x6e, 0x101, 0x79, 0x61, 0x6b, 0x75, 0x73,
0x74, 0x75, 0x6b, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x79, 0x61, 0x6b, 0x75,
0x73, 0x74, 0x75, 0x6b, 0x20, 0x6d, 0x101, 0x6e, 0x101, 0x6e, 0x6b, 0x61,
0x20, 0x62, 0x65, 0x6c, 0x101, 0x79, 0x61, 0x6b, 0x75, 0x73, 0x74, 0x75,
0x6b, 0x20, 0x6b, 0x101, 0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101,
0x79, 0x61, 0x6b, 0x61, 0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x61, 0x72,
0x67, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x79, 0x61, 0x6b, 0x61, 0x74, 0x65,
0x72, 0x69, 0x6e, 0x62, 0x61, 0x72, 0x67, 0x20, 0x6d, 0x101, 0x6e, 0x101,
0x6e, 0x6b, 0x61, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x79, 0x61, 0x6b, 0x61,
0x74, 0x65, 0x72, 0x69, 0x6e, 0x62, 0x61, 0x72, 0x67, 0x20, 0x6b, 0x101,
0x72, 0x7e, 0x101, 0x20, 0x62, 0x65, 0x6c, 0x101, 0x917, 0x94d, 0x930, 0x940,
0x928, 0x935, 0x93f, 0x91a, 0x20, 0x92e, 0x940, 0x928, 0x20, 0x92c, 0x947, 0x932,
0x93e, 0xb17, 0xb4d, 0xb30, 0xb3f, 0xb28, 0xb71, 0xb3f, 0xb1a, 0x20, 0xb2e, 0xb3f,
0xb28, 0x20, 0xb2c, 0xb47, 0xb32, 0xb3e
};

// GENERATED PART ENDS HERE

} // QtTimeZoneLocale

#undef LOCALE_TAGS

QT_END_NAMESPACE

#endif // QTIMEZONELOCALE_DATA_P_H
